!
!  SPDX-License-Identifier: GPL-3.0-or-later
!  Copyright (C) 2019-2022, respective authors of MCFM.
!

      subroutine qqb_wh_g(P,msq)
      implicit none
      include 'types.f'
c---Matrix element squared averaged over initial colors and spins
c---for nwz=1
c     q(-p1)+qbar(-p2) -->  H  + W +g(p7)
c                           |    |
c                           |    --> nu(p3)+e^+(p4)
c                           |
c                           ---> b(p5)+b(p6)
c---for nwz=-1
c     q(-p1)+qbar(-p2) -->  H  + W +g(p7)
c                           |    |
c                           |    --> e^-(p3)+nubar(p4)
c                           |
c                           ---> b(p5)+b(p6)
c   for the moment --- radiation only from initial line

      include 'constants.f'
      include 'nf.f'
      include 'mxpart.f'
      include 'ckm.f'
      include 'sprods_com.f'
      include 'hbbparams.f'
      include 'hdecaymode.f'
      integer:: j,k
      real(dp):: P(mxpart,4),msq(-nf:nf,-nf:nf)
      real(dp):: radi
c      real(dp):: radf
      real(dp):: qqbWHg,qbqWHg,qgWHq,gqWHq,gqbWHqb,qbgWHqb

      msq(:,:)=0._dp

      call dotem(7,p,s)

      qqbWHg=aveqq*radi(1,2,7,5,6,3,4)
c    &  +zip*aveqq*radf(1,2,7,5,6,3,4)
      qbqWHg=aveqq*radi(2,1,7,5,6,3,4)
c    &   +zip*aveqq*radf(2,1,7,5,6,3,4)
c---turn off radiation from final legs
      qgWHq=-radi(1,7,2,5,6,3,4)*aveqg
      gqWHq=-radi(2,7,1,5,6,3,4)*aveqg

      gqbWHqb=-radi(7,2,1,5,6,3,4)*aveqg
      qbgWHqb=-radi(7,1,2,5,6,3,4)*aveqg

c      write(6,*) 'qqbWHg',qqbWHg
c      write(6,*) 'qbqWHg',qbqWHg
c      write(6,*) 'qbgWHqb',qbgWHqb
c      write(6,*) 'gqbWHqb',gqbWHqb
c      write(6,*) 'qgWHq',qgWHq
c      write(6,*) 'gqWHq',gqWHq


      do j=-nf,nf
      do k=-nf,nf

      if     ((j > 0) .and. (k < 0)) then
          msq(j,k)=Vsq(j,k)*qqbWHg
      elseif ((j < 0) .and. (k > 0)) then
          msq(j,k)=Vsq(j,k)*qbqWHg
      elseif ((j > 0) .and. (k == 0)) then
          msq(j,k)=
     &   (Vsq(j,-1)+Vsq(j,-2)+Vsq(j,-3)+Vsq(j,-4)+Vsq(j,-5))*qgWHq
      elseif ((j < 0) .and. (k == 0)) then
          msq(j,k)=
     &    (Vsq(j,+1)+Vsq(j,+2)+Vsq(j,+3)+Vsq(j,+4)+Vsq(j,+5))*qbgWHqb
      elseif ((j == 0) .and. (k > 0)) then
          msq(j,k)=
     &    (Vsq(-1,k)+Vsq(-2,k)+Vsq(-3,k)+Vsq(-4,k)+Vsq(-5,k))*gqWHq
      elseif ((j == 0) .and. (k < 0)) then
          msq(j,k)=
     &    (Vsq(+1,k)+Vsq(+2,k)+Vsq(+3,k)+Vsq(+4,k)+Vsq(+5,k))*gqbWHqb
      endif

      enddo
      enddo

c--- adjust for fixed H->bb BR if necessary
      if ((FixBrHbb) .and. (hdecaymode == 'bqba')) then
        msq(:,:)=msq(:,:)*GamHbb/GamHbb0
      endif


      return
      end


      function radi(j1,j2,j3,j4,j5,j6,j7)
      implicit none
      include 'types.f'
      real(dp):: radi
      include 'constants.f'
      include 'mxpart.f'
      include 'qcdcouple.f'
      include 'ewcouple.f'
      include 'masses.f'
      include 'sprods_com.f'
      include 'hdecaymode.f'
      include 'ewinput.f'
      integer:: j1,j2,j3,j4,j5,j6,j7
      real(dp):: s45,s12,s13,s23,s123,prop
      real(dp):: fac,hdecay,msqhbb,msqhgamgam,msqhtautau

      s12=s(j1,j2)
      s13=s(j1,j3)
      s23=s(j2,j3)
      s123=s12+s13+s23
c---calculate the 2 W propagators
      prop=       ((s123-wmass**2)**2+(wmass*wwidth)**2)
      prop=prop*((s(j6,j7)-wmass**2)**2+(wmass*wwidth)**2)

      if (ewscheme < 4) then
        fac=2._dp*cf*xn*gsq*gwsq**3*wmass**2/prop
      else
c-- note: in CMS, wmass**2 -> sqrt(wmass**4+(wmass*wwidth)**2)
        fac=2._dp*cf*xn*gsq*gwsq**3*sqrt(wmass**4+(wmass*wwidth)**2)/prop
      endif
c      hdecay=xn*gwsq*mbsq/(4._dp*wmass**2)*2._dp*(s45-4._dp*mb**2)

c   Deal with Higgs decay
      if (hdecaymode == 'tlta') then
          s45=s(j4,j5)+2._dp*mtau**2
          hdecay=msqhtautau(s45)
      elseif (hdecaymode == 'bqba') then
          s45=s(j4,j5)+2._dp*mb**2
          hdecay=msqhbb(s45)
      elseif (hdecaymode == 'gaga') then
          s45=s(j4,j5)
          hdecay=msqhgamgam(s45)
      elseif (hdecaymode == 'none') then
          s45=s(j4,j4)
          hdecay=one
      else
      write(6,*) 'Unimplemented process in qqb_wh_g'
      stop
      endif
      if (hdecaymode /= 'none') then
         hdecay=hdecay/((s45-hmass**2)**2+(hmass*hwidth)**2)
      endif
      fac=fac*hdecay

      radi=s12/s13/s23
     & *(2._dp*s(j1,j7)*s(j2,j6)+s(j1,j7)*s(j3,j6)+s(j2,j6)*s(j3,j7))
     & +(s(j1,j7)*s(j2,j6)+s(j2,j6)*s(j3,j7)-s(j1,j6)*s(j1,j7))/s13
     & +(s(j1,j7)*s(j2,j6)+s(j1,j7)*s(j3,j6)-s(j2,j6)*s(j2,j7))/s23
      radi=fac*radi
      return
      end





c      function radf(j1,j2,j3,j4,j5,j6,j7)
c      implicit none
c      include 'types.f'
c      real(dp):: radf

c      include 'constants.f'
c      include 'nf.f'
c      include 'mxpart.f'
c      include 'ewcouple.f'
c      include 'qcdcouple.f'
c      include 'masses.f'
c      include 'sprods_com.f'
c      integer:: j1,j2,j3,j4,j5,j6,j7
c      real(dp):: s34,s35,s45,s12,s67,s345,prop
c      real(dp):: fac


c      s12=s(j1,j2)
c      s67=s(j6,j7)
c      s34=s(j3,j4)
c      s35=s(j3,j5)
c      s45=s(j4,j5)
c      s345=s34+s35+s45+2._dp*mb**2

c---calculate the 3 propagators
c      prop=     ((s12-wmass**2)**2+(wmass*wwidth)**2)
c      prop=prop*((s67-wmass**2)**2+(wmass*wwidth)**2)
c      prop=prop*((s345-hmass**2)**2+(hmass*hwidth)**2)

c      fac=two*CF*xnsq*gsq*gw**8*mbsq*s(j1,j7)*s(j2,j6)/prop
c      radf=(s45/(s34*s35)-mb**2*(1._dp/s34**2+1._dp/s35**2))*(s45-2._dp*mb**2)
c     & +0.5_dp*((s34+s35)**2+2*(s45-mb**2)*(s34+s35))/s34/s35
c     & -mb**2*(s35/s34**2+s34/s35**2)
c      radf=fac*radf
c      return
c      end


