!
!  SPDX-License-Identifier: GPL-3.0-or-later
!  Copyright (C) 2019-2022, respective authors of MCFM.
!

c********************************************************
cAUTHOR: FABIO MALTONI                                  *
cDATE  : 10/10/2001                                     *
cNOTES : PROGRAM GENERATED BY BHIGGS.M                  *
c********************************************************

      function  BBGGH(I1,I2,I3,I4)
      implicit none
      include 'types.f'
      real(dp)::  BBGGH
c ---------------------------------------------------------------------
c returns the matrix element squared for the process

c          0 -> bbar1 b2 g3 g4 h

c All momenta outgoing.
c No averaging is performed for initial spins or colors.
c No Bose symmetry factors are included.
c No STRONG coupling is included.
c No Bottom-Higgs coupling is included.
c ---------------------------------------------------------------------

      include 'constants.f'
      include 'mxpart.f'
      include 'sprods_com.f'
      include 'zprods_com.f'

c     ARGUMENTS
      integer:: I1,I2,I3,I4

c     LOCAL
      real(dp)::  T123,T124,T134,T234,MH2
      complex(dp):: AMP(2,4)
      integer:: L



      t123=s(i1,i2)+s(i2,i3)+s(i1,i3)
      t124=s(i1,i2)+s(i2,i4)+s(i1,i4)
      t134=s(i1,i3)+s(i1,i4)+s(i3,i4)
      t234=s(i2,i3)+s(i2,i4)+s(i3,i4)
      MH2 =t123+s(i1,i4)+s(i2,i4)+s(i3,i4)

      amp(1,1)=
     &(2._dp*mH2*za(i1,i2))/(za(i1,i3)*za(i1,i4)*za(i2,i3)*za(i2,i4))

      amp(2,1)=
     &  (-2._dp*mH2*(za(i1,i4)*za(i2,i3) + za(i1,i3)*za(i2,i4)))/
     &  (za(i1,i3)*za(i1,i4)*za(i2,i3)*za(i2,i4)*za(i3,i4))


      amp(1,2)=
     &(-2._dp*zb(i2,i1)**3)/(zb(i3,i1)*zb(i3,i2)*zb(i4,i1)*zb(i4,i2))

      amp(2,2)=
     &  (2._dp*zb(i2,i1)**2*(zb(i3,i2)*zb(i4,i1) +
     &   zb(i3,i1)*zb(i4,i2)))/
     &  (zb(i3,i1)*zb(i3,i2)*zb(i4,i1)*zb(i4,i2)*zb(i4,i3))


      amp(1,3)=
     &  (-2._dp*zb(i3,i1)*(za(i1,i4)*zb(i2,i1) - za(i3,i4)*zb(i3,i2)))/
     &  (T134*za(i1,i3)*zb(i4,i1)) -
     &  (2._dp*(za(i2,i4)*zb(i2,i1) + za(i3,i4)*zb(i3,i1))*zb(i3,i2))/
     &  (T234*za(i2,i3)*zb(i4,i2)) -
     &  (2._dp*T123*zb(i2,i1))/(za(i1,i3)*za(i2,i3)*zb(i4,i1)*zb(i4,i2))

      amp(2,3)=
     &  (2._dp*(((za(i1,i4)*zb(i2,i1) - za(i3,i4)*zb(i3,i2))*
     &  (((-s(i1,i3) + s(i1,i4))*zb(i3,i1))/
     &  (T134*zb(i4,i1)) - zb(i3,i2)/zb(i4,i2)))/za(i1,i3)
     & +((za(i2,i4)*zb(i2,i1) + za(i3,i4)*zb(i3,i1))*
     &  (zb(i3,i1)/zb(i4,i1) +
     &  ((s(i2,i3) - s(i2,i4))*zb(i3,i2))/(T234*zb(i4,i2)))
     &  )/za(i2,i3)))/s(i3,i4)


      amp(1,4)=
     & (-2._dp*T124*zb(i2,i1))/
     & (za(i1,i4)*za(i2,i4)*zb(i3,i1)*zb(i3,i2)) -
     &  (2._dp*(za(i2,i3)*zb(i2,i1) + za(i4,i3)*zb(i4,i1))*zb(i4,i2))/
     & (T234*za(i2,i4)*zb(i3,i2)) -
     &  (2._dp*zb(i4,i1)*(za(i1,i3)*zb(i2,i1) - za(i4,i3)*zb(i4,i2)))/
     & (T134*za(i1,i4)*zb(i3,i1))

      amp(2,4)=
     &  (-2._dp*(((za(i1,i3)*zb(i2,i1) - za(i4,i3)*zb(i4,i2))*
     &  (((s(i1,i3) - s(i1,i4))*zb(i4,i1))/
     &  (T134*zb(i3,i1)) - zb(i4,i2)/zb(i3,i2)))/za(i1,i4)
     & +((za(i2,i3)*zb(i2,i1) + za(i4,i3)*zb(i4,i1))*
     &  (zb(i4,i1)/zb(i3,i1) +
     &  ((-s(i2,i3) + s(i2,i4))*zb(i4,i2))/(T234*zb(i3,i2))
     &  ))/za(i2,i4)))/s(i3,i4)



      BBGGH=0._dp
      do l=1,4
         BBGGH=BBGGH
     &         +(xn**2-2._dp)/xn*abs(amp(1,l))**2
     &         + xn*abs(amp(2,l))**2
      enddo

      BBGGH=2._dp*BBGGH*V/8._dp

      END
