      ! Added for 2107.12478
      module mod_hplog6

      public :: hplog6

      private

      contains


******************************************************************************
**  hplog6: a subroutine for the evaluation of harmonic polylogarithms
**          Version 1.6         05/05/2017
**  upgraded to w=6 from hplog as described in: 
**  T.Gehrmann and E.Remiddi: Numerical Evaluation of the Harmonic 
**                            Polylogarithms up to Weight 4
**                            (hep-ph/0107173; Comp.Phys.Comm. 141 (2001) 296)
**  the harmonic polylogarithms are defined in: 
**  E.Remiddi and J.Vermaseren: Harmonic Polylogarithms
**                            (hep-ph/9905237; Int.J.Mod.Phys. A15 (2000) 725)
**  email:
**  Thomas.Gehrmann@uzh.ch and Ettore.Remiddi@bo.infn.it
**
******************************************************************************
      subroutine hplog6(x,nw,Hc1,Hc2,Hc3,Hc4,Hc5,Hc6, 
     &                       Hr1,Hr2,Hr3,Hr4,Hr5,Hr6,
     &                       Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2) 
****** 
** x is the argument of the 1dHPL's (1 dimensional Harmonic PolyLogarithms) 
**   to be evaluated; 
** nw is the maximum weight of the required 1dHPL's; 
**    the maximum allowed value of nw of this implementation is 6; 
** Hc1,Hc2,Hc3,Hc4,Hc5,Hc6 are the complex*16 values of the 1dHPL; 
**    they must all be supplied in the arguments even if some of them 
**    are not to be evaluated; 
** Hr1,Hr2,Hr3,Hr4,Hr5,Hr6 are the double precision real parts of 
**    Hc1,Hc2,Hc3,Hc4,Hc5,Hc6; 
** Hi1,Hi2,Hi3,Hi4,Hi5,Hi6 are the double precision immaginary parts of 
**    Hc1,Hc2,Hc3,Hc4,Hc5,Hc6 divided by pi=3.114159.... 
** n1,n2 is the required range of indices, the allowed ranges are 
**    (0,1), (-1,0), (-1,1) ; 
** current version: must be (-1,1)
****** 
      implicit double precision (a-h,o-z) 
      complex*16 Hc1,Hc2,Hc3,Hc4,Hc5,Hc6 
      dimension Hc1(n1:n2),Hc2(n1:n2,n1:n2),Hc3(n1:n2,n1:n2,n1:n2), 
     &          Hc4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hc5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hc6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension Hr1(n1:n2),Hr2(n1:n2,n1:n2),Hr3(n1:n2,n1:n2,n1:n2), 
     &          Hr4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hr5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hr6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension Hi1(n1:n2),Hi2(n1:n2,n1:n2),Hi3(n1:n2,n1:n2,n1:n2), 
     &          Hi4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2),
     &          Hi6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      common /fillred/infilldim,infill(3) 
!$omp threadprivate(/fillred/)
      parameter (r2   = 1.4142135623730950488d0) 
** check on the weight nw 
      if ( (nw.lt.1).or.(nw.gt.6) ) then 
        print*, ' illegal call of eval1dhpl with second argument', 
     &          ' (the weight) = ',nw 
        print*, ' the allowed values of the weight are 1,..,6 ' 
        stop
      endif
** check on the range n1:n2 
      if ( (n1.eq.-1).and.(n2.eq.0) ) then 
        infilldim =  2 
        infill(1) =  0 
        infill(2) = -1  
        print*, ' illegal call of hplog6 with index range'
        print*, ' only (-1,1) is supported ' 
        stop
      elseif ( (n1.eq.0).and.(n2.eq.1) ) then 
        infilldim =  2 
        infill(1) =  0 
        infill(2) =  1  
        print*, ' illegal call of hplog6 with index range'
        print*, ' only (-1,1) is supported ' 
        stop
      elseif ( (n1.eq.-1).and.(n2.eq.1) ) then 
        infilldim =  3 
        infill(1) =  0 
        infill(2) = -1  
        infill(3) =  1  
      else 
        print*, ' illegal call of eval1dhpl with the two last ', 
     &          'arguments = (',n1,',',n2,')' 
        print*, ' the allowed values are (-1,0), (0,1), (-1,1) ' 
        stop 
      endif 
** setting the immaginary parts equal to zero 
      call setzero(nw,Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2) 
** looking at the range of the argument 
*      r2 = sqrt(2.d0) 
      r2m1 = r2 - 1 
      r2p1 = r2 + 1 
      if ( ( x.gt.-r2m1 ).and.( x.le.r2m1) ) then 
*        print*, ' eval1dhpl:      x = ',x,', call eval1dhplat0 ' 
        call eval1dhplat0(x,nw,Hc1,Hc2,Hc3,Hc4,Hc5,Hc6, 
     &                         Hr1,Hr2,Hr3,Hr4,Hr5,Hr6,
     &                         Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2) 
        return 
      elseif ( x.eq.1d0 ) then
*        print*, ' eval1dhpl:      x = ',x,', call eval1dhplin1 ' 
        call eval1dhplin1(x,nw,Hc1,Hc2,Hc3,Hc4,Hc5,Hc6, 
     &                         Hr1,Hr2,Hr3,Hr4,Hr5,Hr6,
     &                         Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2)
        return 
      elseif ( ( x.gt.r2m1 ).and.( x.le.r2p1) ) then 
*        print*, ' eval1dhpl:      x = ',x,', call eval1dhplat1 ' 
        call eval1dhplat1(x,nw,Hc1,Hc2,Hc3,Hc4,Hc5,Hc6, 
     &                         Hr1,Hr2,Hr3,Hr4,Hr5,Hr6,
     &                         Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2) 
        return
      elseif ( ( x.gt.r2p1 ) ) then 
*        print*, ' eval1dhpl:      x = ',x,', call eval1dhplatinf ' 
        call eval1dhplatinf(x,nw,Hc1,Hc2,Hc3,Hc4,Hc5,Hc6, 
     &                           Hr1,Hr2,Hr3,Hr4,Hr5,Hr6,
     &                           Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2) 
        return   
      elseif ( ( x.le.-r2p1) ) then 
*        print*, ' eval1dhpl:      x = ',x,', call eval1dhplatminf ' 
        call eval1dhplatminf(x,nw,Hc1,Hc2,Hc3,Hc4,Hc5,Hc6, 
     &                            Hr1,Hr2,Hr3,Hr4,Hr5,Hr6,
     &                            Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2) 
        return 
      elseif ( x.eq.-1d0 ) then
*        print*, ' eval1dhpl:      x = ',x,', call eval1dhplinm1 ' 
        call eval1dhplinm1(x,nw,Hc1,Hc2,Hc3,Hc4,Hc5,Hc6, 
     &                          Hr1,Hr2,Hr3,Hr4,Hr5,Hr6,
     &                          Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2) 
        return 
      elseif ( ( x.gt.-r2p1 ).and.( x.le.-r2m1) ) then 
*        print*, ' eval1dhpl:      x = ',x,', call eval1dhplatm1 ' 
        call eval1dhplatm1(x,nw,Hc1,Hc2,Hc3,Hc4,Hc5,Hc6, 
     &                          Hr1,Hr2,Hr3,Hr4,Hr5,Hr6,
     &                          Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2) 
        return 
      endif 
** 
      end 
************************************************************************ 
      subroutine eval1dhplat0(y,nw,H1,H2,H3,H4,H5,H6, 
     &                          HY1,HY2,HY3,HY4,HY5,HY6,
     &                          Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2) 
** evaluates 1dhpl's in the 0-range  -(r2-1) < y <= (r2-1) 
** by direct series expansion (Bernoulli-accelerated) 
      implicit double precision (a-h,o-z) 
      complex*16 H1,H2,H3,H4,H5,H6 
      dimension H1(n1:n2),H2(n1:n2,n1:n2),H3(n1:n2,n1:n2,n1:n2), 
     &          H4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          H5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          H6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension HY1(n1:n2),HY2(n1:n2,n1:n2),HY3(n1:n2,n1:n2,n1:n2), 
     &          HY4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HY5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2),
     &          HY6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension Hi1(n1:n2),Hi2(n1:n2,n1:n2),Hi3(n1:n2,n1:n2,n1:n2), 
     &          Hi4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2),
     &          Hi6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
** evaluate the irreducible 1dHPL's first 
      call fillh1(y,H1,HY1,Hi1,n1,n2) 
      if ( nw.eq.1 ) return 
      call fillirr1dhplat0(y,nw,HY1,HY2,HY3,HY4,HY5,HY6,n1,n2) 
** then the reducible 1dHPL's 
      call fillred1dhpl(nw,H1,H2,H3,H4,H5,H6, 
     &                     HY1,HY2,HY3,HY4,HY5,HY6,
     &                     Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2) 
      return 
      end 
************************************************************************ 
      subroutine eval1dhplin1(y,nw,H1,H2,H3,H4,H5,H6, 
     &                          HY1,HY2,HY3,HY4,HY5,HY6,
     &                          Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2)
** evaluates 1dhpl's for y=1 (explicit values are tabulated)
      implicit double precision (a-h,o-z) 
      complex*16 H1,H2,H3,H4,H5,H6 
      dimension H1(n1:n2),H2(n1:n2,n1:n2),H3(n1:n2,n1:n2,n1:n2), 
     &          H4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          H5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          H6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension HY1(n1:n2),HY2(n1:n2,n1:n2),HY3(n1:n2,n1:n2,n1:n2), 
     &          HY4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HY5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HY6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension Hi1(n1:n2),Hi2(n1:n2,n1:n2),Hi3(n1:n2,n1:n2,n1:n2), 
     &          Hi4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2),
     &          Hi6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      parameter (pi   = 3.14159265358979324d0) 
** evaluate the irreducible 1dHPL's first 
      call fillh1(y,H1,HY1,Hi1,n1,n2) 
      if ( nw.eq.1 ) return 
      call fillirr1dhplin1(y,nw,HY1,HY2,HY3,HY4,HY5,HY6,n1,n2) 
** then the reducible 1dHPL's 
      call fillred1dhpl(nw,H1,H2,H3,H4,H5,H6, 
     &                     HY1,HY2,HY3,HY4,HY5,HY6,
     &                     Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2) 
      if (n2.eq.0) return
** correct the ill-defined entries
      HY2(1,0) = - HY2(0,1) 
      Hi2(1,0) = 0d0 
      H2(1,0) = dcmplx(HY2(1,0),Hi2(1,0)*pi)
      if ( nw.eq.2 ) return
      HY3(1,0,0) = HY3(0,0,1) 
      Hi3(1,0,0) = 0d0 
      H3(1,0,0) = dcmplx(HY3(1,0,0),Hi3(1,0,0)*pi)
      if ( nw.eq.3 ) return
      HY4(1,0,0,0) = -HY4(0,0,0,1) 
      Hi4(1,0,0,0) = 0d0 
      H4(1,0,0,0) = dcmplx(HY4(1,0,0,0),Hi4(1,0,0,0)*pi)
      if ( nw.eq.4 ) return
      HY5(1,0,0,0,0) = HY5(0,0,0,0,1) 
      Hi5(1,0,0,0,0) = 0d0 
      H5(1,0,0,0,0) = dcmplx(HY5(1,0,0,0,0),Hi5(1,0,0,0,0)*pi)
      if ( nw.eq.5 ) return
      HY6(1,0,0,0,0,0) = -HY6(0,0,0,0,0,1) 
      Hi6(1,0,0,0,0,0) = 0d0 
      H6(1,0,0,0,0,0) = dcmplx(HY6(1,0,0,0,0,0),Hi6(1,0,0,0,0,0)*pi)
      return 
      end 
************************************************************************ 
      subroutine eval1dhplat1(y,nw,H1,H2,H3,H4,H5,H6, 
     &                          HY1,HY2,HY3,HY4,HY5,HY6,
     &                          Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2)
** evaluates 1dhpl's in the 1-range  (r2-1) < y <= (r2+1) 
** evaluating first the H(..,r=(1-y)/(1+y)) by calling eval1dhplat0(r)  
** and then expressing H(..,y=(1-r)/(1+r)) in terms of H(..,r) 
      implicit double precision (a-h,o-z) 
      complex*16 H1,H2,H3,H4,H5,H6 
      dimension H1(n1:n2),H2(n1:n2,n1:n2),H3(n1:n2,n1:n2,n1:n2), 
     &          H4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          H5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          H6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension HY1(n1:n2),HY2(n1:n2,n1:n2),HY3(n1:n2,n1:n2,n1:n2), 
     &          HY4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HY5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2),
     &          HY6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension Hi1(n1:n2),Hi2(n1:n2,n1:n2),Hi3(n1:n2,n1:n2,n1:n2), 
     &          Hi4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2),
     &          Hi6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
** additional arrays required within this routine 
      dimension HR1(-1:1),HR2(-1:1,-1:1),HR3(-1:1,-1:1,-1:1), 
     &          HR4(-1:1,-1:1,-1:1,-1:1), 
     &          HR5(-1:1,-1:1,-1:1,-1:1,-1:1), 
     &          HR6(-1:1,-1:1,-1:1,-1:1,-1:1,-1:1) 
** the nw = 1 case 
      call fillh1(y,H1,HY1,Hi1,n1,n2) 
      if ( nw.eq.1 ) return 
** the nw > 1 case 
      r = (1.d0-y)/(1.d0+y) 
*      print*,' eval1dhplat1: y = ',y,', r = ',r 
** the whole (-1,1) range is in general needed for any pair (n1,n2)
      call fillirr1dhplat0(r,nw,HR1,HR2,HR3,HR4,HR5,HR6,-1,1) 
** fillirr1dhplat1 takes care automatically of all the immaginary 
** parts as well as of the jump across y=1 
      call fillirr1dhplat1(r,nw,HR1,HR2,HR3,HR4,HR5,HR6, 
     &                          HY1,HY2,HY3,HY4,HY5,HY6, 
     &                          Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2) 
** then the reducible 1dHPL's 
      call fillred1dhpl(nw,H1,H2,H3,H4,H5,H6, 
     &                     HY1,HY2,HY3,HY4,HY5,HY6,
     &                     Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2) 
      return 
      end 
************************************************************************ 
      subroutine eval1dhplatinf(y,nw,H1,H2,H3,H4,H5,H6, 
     &                          HY1,HY2,HY3,HY4,HY5,HY6,
     &                          Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2)
** evaluates 1dhpl's in the inf-range  (r2+1) < abs(y) 
** evaluating first the H(..,x=1/y) by calling eval1dhplat0(x)  
** and then expressing H(..,y=1/x) in terms of H(..,x) 
      implicit double precision (a-h,o-z) 
      complex*16 H1,H2,H3,H4,H5,H6 
      dimension H1(n1:n2),H2(n1:n2,n1:n2),H3(n1:n2,n1:n2,n1:n2), 
     &          H4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          H5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          H6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension HY1(n1:n2),HY2(n1:n2,n1:n2),HY3(n1:n2,n1:n2,n1:n2), 
     &          HY4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HY5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2),
     &          HY6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension Hi1(n1:n2),Hi2(n1:n2,n1:n2),Hi3(n1:n2,n1:n2,n1:n2), 
     &          Hi4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
** additional arrays required within this routine 
      dimension HX1(n1:n2),HX2(n1:n2,n1:n2),HX3(n1:n2,n1:n2,n1:n2), 
     &          HX4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HX5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HX6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      parameter (pi   = 3.14159265358979324d0) 
** the nw = 1 case 
      call fillh1(y,H1,HY1,Hi1,n1,n2) 
      if ( nw.eq.1 ) return 
** the nw > 1 case 
      x = 1.d0/y 
*      print*,' eval1dhplatinf: y = ',y,', x = ',x 
      call fillirr1dhplat0(x,nw,HX1,HX2,HX3,HX4,HX5,HX6,n1,n2) 
** fillirr1dhplatinf takes care automatically of all the immaginary 
** parts as well as of the jump across y=1 
      call fillirr1dhplatinf(x,nw,HX1,HX2,HX3,HX4,HX5,HX6, 
     &                            HY1,HY2,HY3,HY4,HY5,HY6, 
     &                            Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2) 
** then the reducible 1dHPL's 
      call fillred1dhpl(nw,H1,H2,H3,H4,H5,H6, 
     &                     HY1,HY2,HY3,HY4,HY5,HY6,
     &                     Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2) 
      return 
      end 
************************************************************************ 
      subroutine eval1dhplinm1(y,nw,H1,H2,H3,H4,H5,H6, 
     &                          HY1,HY2,HY3,HY4,HY5,HY6,
     &                          Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2)
** evaluates 1dhpl's for y=-1 (explicit values are tabulated)
      implicit double precision (a-h,o-z) 
      complex*16 H1,H2,H3,H4,H5,H6 
      complex*16 G1,G2,G3,G4,G5,G6  
      dimension H1(n1:n2),H2(n1:n2,n1:n2),H3(n1:n2,n1:n2,n1:n2), 
     &          H4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          H5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          H6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension HY1(n1:n2),HY2(n1:n2,n1:n2),HY3(n1:n2,n1:n2,n1:n2), 
     &          HY4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HY5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HY6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension Hi1(n1:n2),Hi2(n1:n2,n1:n2),Hi3(n1:n2,n1:n2,n1:n2), 
     &          Hi4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2),
     &          Hi6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
** additional arrays required within this routine 
      dimension G1(-n2:-n1),G2(-n2:-n1,-n2:-n1),
     &          G3(-n2:-n1,-n2:-n1,-n2:-n1), 
     &          G4(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1), 
     &          G5(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1), 
     &          G6(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1) 
      dimension GY1(-n2:-n1),GY2(-n2:-n1,-n2:-n1),
     &          GY3(-n2:-n1,-n2:-n1,-n2:-n1), 
     &          GY4(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1), 
     &          GY5(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1), 
     &          GY6(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1) 
      dimension Gi1(-n2:-n1),Gi2(-n2:-n1,-n2:-n1),
     &          Gi3(-n2:-n1,-n2:-n1,-n2:-n1), 
     &          Gi4(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1), 
     &          Gi5(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1), 
     &          Gi6(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1) 
      common /fillred/infilldim,infill(3) 
!$omp threadprivate(/fillred/)
      dimension istorfill(3)
      dimension nphase(-1:1) 
      data nphase/-1,1,-1/ 
      parameter (pi   = 3.14159265358979324d0) 
*      print*,' eval1dhplatm1: y = ',y 
      if (infilldim.eq.2) then
         do i=1,2
            istorfill(i) = infill(i)
            infill(i) = -istorfill(i)
         enddo
      endif
** evaluate H(...,-y) 
      call setzero(nw,Gi1,Gi2,Gi3,Gi4,Gi5,Gi6,-n2,-n1) 
      Gi1(0) = -1
      call eval1dhplin1(-y,nw,G1,G2,G3,G4,G5,G6, 
     &                  GY1,GY2,GY3,GY4,GY5,GY6,
     &                  Gi1,Gi2,Gi3,Gi4,Gi5,Gi6,-n2,-n1) 
      if (infilldim.eq.2) then
         do i=1,2
            infill(i) = istorfill(i)
         enddo
      endif
** fill the arrays H's 
      do k1=n1,n2 
        nph1 = nphase(k1) 
        HY1(k1) =   nph1*GY1(-k1) 
        Hi1(k1) = - nph1*Gi1(-k1) 
        H1(k1)  = dcmplx(HY1(k1),Hi1(k1)*pi) 
        if ( nw.gt.1 ) then 
          do k2=n1,n2 
            nph2 = nph1*nphase(k2) 
            HY2(k1,k2) =   nph2*GY2(-k1,-k2) 
            Hi2(k1,k2) = - nph2*Gi2(-k1,-k2) 
            H2(k1,k2)  = dcmplx(HY2(k1,k2),Hi2(k1,k2)*pi) 
            if ( nw.gt.2 ) then 
              do k3=n1,n2 
                nph3 = nph2*nphase(k3) 
                HY3(k1,k2,k3) =   nph3*GY3(-k1,-k2,-k3) 
                Hi3(k1,k2,k3) = - nph3*Gi3(-k1,-k2,-k3) 
                H3(k1,k2,k3)  = dcmplx(HY3(k1,k2,k3),Hi3(k1,k2,k3)*pi) 
                if ( nw.gt.3 ) then 
                  do k4=n1,n2 
                    nph4 = nph3*nphase(k4) 
                    HY4(k1,k2,k3,k4) =   nph4*GY4(-k1,-k2,-k3,-k4) 
                    Hi4(k1,k2,k3,k4) = - nph4*Gi4(-k1,-k2,-k3,-k4) 
                    H4(k1,k2,k3,k4)  = 
     &                    dcmplx(HY4(k1,k2,k3,k4),Hi4(k1,k2,k3,k4)*pi) 
                if ( nw.gt.4 ) then 
                  do k5=n1,n2 
                    nph5 = nph4*nphase(k5) 
                    HY5(k1,k2,k3,k4,k5) = nph5*GY5(-k1,-k2,-k3,-k4,-k5) 
                    Hi5(k1,k2,k3,k4,k5) =-nph5*Gi5(-k1,-k2,-k3,-k4,-k5) 
                    H5(k1,k2,k3,k4,k5)  = 
     &                dcmplx(HY5(k1,k2,k3,k4,k5),Hi5(k1,k2,k3,k4,k5)*pi) 
                if ( nw.gt.5 ) then 
                  do k6=n1,n2 
                    nph6 = nph6*nphase(k6) 
              HY6(k1,k2,k3,k4,k5,k6) = nph6*GY6(-k1,-k2,-k3,-k4,-k5,-k6) 
              Hi6(k1,k2,k3,k4,k5,k6) =-nph6*Gi6(-k1,-k2,-k3,-k4,-k5,-k6) 
              H6(k1,k2,k3,k4,k5,k6)  = 
     &        dcmplx(HY6(k1,k2,k3,k4,k5,k6),Hi6(k1,k2,k3,k4,k5,k6)*pi) 
                  enddo 
                endif 
                  enddo 
                endif 
                  enddo 
                endif 
              enddo 
            endif 
          enddo 
        endif 
      enddo 
      if (n1.eq.0) return
** correct the ill-defined entries
      HY2(-1,0) = - HY2(0,-1) 
      Hi2(-1,0) = Hi1(0)*HY1(-1)
      H2(-1,0) = dcmplx(HY2(-1,0),Hi2(-1,0)*pi)
      if ( nw.eq.2 ) return
      HY3(-1,0,0) = HY1(-1)*HY2(0,0)+HY3(0,0,-1) 
      Hi3(-1,0,0) = HY1(-1)*Hi2(0,0)-HY2(0,-1)*Hi1(0)
      H3(-1,0,0) = dcmplx(HY3(-1,0,0),Hi3(-1,0,0)*pi)
      if ( nw.eq.3 ) return
      HY4(-1,0,0,0) = -HY2(0,-1)*HY2(0,0)-HY4(0,0,0,-1) 
      Hi4(-1,0,0,0) = HY1(-1)*Hi3(0,0,0)+HY3(0,0,-1)*Hi1(0)
      H4(-1,0,0,0) = dcmplx(HY4(-1,0,0,0),Hi4(-1,0,0,0)*pi)
      if ( nw.eq.4 ) return
      HY5(-1,0,0,0,0) = HY4(0,0,0,0)*HY1(-1)+HY2(0,0)*HY3(0,0,-1)
     &                  + HY5(0,0,0,0,-1)
      Hi5(-1,0,0,0,0) = -HY2(0,-1)*Hi3(0,0,0)-HY4(0,0,0,-1)*Hi1(0)
      H5(-1,0,0,0,0) = dcmplx(HY5(-1,0,0,0,0),Hi5(-1,0,0,0,0)*pi)
      if ( nw.eq.5 ) return
      HY6(-1,0,0,0,0,0) = -HY4(0,0,0,0)*HY2(0,-1)-HY2(0,0)*HY4(0,0,0,-1)
     &                  - HY6(0,0,0,0,0,-1)
      Hi6(-1,0,0,0,0,0) = +HY1(-1)*Hi5(0,0,0,0,0)
     &                    +HY3(0,0,-1)*Hi3(0,0,0)+HY5(0,0,0,0,-1)*Hi1(0)
      H6(-1,0,0,0,0,0) = dcmplx(HY6(-1,0,0,0,0,0),Hi6(-1,0,0,0,0,0)*pi)
      return 
      end 
************************************************************************ 
      subroutine eval1dhplatm1(y,nw,H1,H2,H3,H4,H5,H6, 
     &                          HY1,HY2,HY3,HY4,HY5,HY6,
     &                          Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2)
** evaluates 1dhpl's in the (-1)-range  -(r2+1) < y <= -(r2-1) 
** evaluating first the H(..,-y) by calling eval1dhplat1(-y), 
** and then expressing H(..,y) in terms of H(..,-y) 
      implicit double precision (a-h,o-z) 
      complex*16 H1,H2,H3,H4,H5,H6 
      complex*16 G1,G2,G3,G4,G5,G6  
      dimension H1(n1:n2),H2(n1:n2,n1:n2),H3(n1:n2,n1:n2,n1:n2), 
     &          H4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          H5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          H6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension HY1(n1:n2),HY2(n1:n2,n1:n2),HY3(n1:n2,n1:n2,n1:n2), 
     &          HY4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HY5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2),
     &          HY6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension Hi1(n1:n2),Hi2(n1:n2,n1:n2),Hi3(n1:n2,n1:n2,n1:n2), 
     &          Hi4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
** additional arrays required within this routine 
      dimension G1(-n2:-n1),G2(-n2:-n1,-n2:-n1),
     &          G3(-n2:-n1,-n2:-n1,-n2:-n1), 
     &          G4(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1), 
     &          G5(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1), 
     &          G6(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1) 
      dimension GY1(-n2:-n1),GY2(-n2:-n1,-n2:-n1),
     &          GY3(-n2:-n1,-n2:-n1,-n2:-n1), 
     &          GY4(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1), 
     &          GY5(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1), 
     &          GY6(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1) 
      dimension Gi1(-n2:-n1),Gi2(-n2:-n1,-n2:-n1),
     &          Gi3(-n2:-n1,-n2:-n1,-n2:-n1), 
     &          Gi4(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1), 
     &          Gi5(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1), 
     &          Gi6(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1) 
      common /fillred/infilldim,infill(3) 
!$omp threadprivate(/fillred/)
      dimension istorfill(3)
      dimension nphase(-1:1) 
      data nphase/-1,1,-1/ 
      parameter (pi   = 3.14159265358979324d0) 
*      print*,' eval1dhplatm1: y = ',y 
      if (infilldim.eq.2) then
         do i=1,2
            istorfill(i) = infill(i)
            infill(i) = -istorfill(i)
         enddo
      endif
** evaluate H(...,-y) 
      call setzero(nw,Gi1,Gi2,Gi3,Gi4,Gi5,Gi6,-n2,-n1) 
      Gi1(0) = -1
      call eval1dhplat1(-y,nw,G1,G2,G3,G4,G5,G6, 
     &                        GY1,GY2,GY3,GY4,GY5,GY6,
     &                        Gi1,Gi2,Gi3,Gi4,Gi5,Gi6,-n2,-n1) 
      if (infilldim.eq.2) then
         do i=1,2
            infill(i) = istorfill(i)
         enddo
      endif
** fill the arrays H's 
      do k1=n1,n2 
        nph1 = nphase(k1) 
        HY1(k1) =   nph1*GY1(-k1) 
        Hi1(k1) = - nph1*Gi1(-k1) 
        H1(k1)  = dcmplx(HY1(k1),Hi1(k1)*pi) 
        if ( nw.gt.1 ) then 
          do k2=n1,n2 
            nph2 = nph1*nphase(k2) 
            HY2(k1,k2) =   nph2*GY2(-k1,-k2) 
            Hi2(k1,k2) = - nph2*Gi2(-k1,-k2) 
            H2(k1,k2)  = dcmplx(HY2(k1,k2),Hi2(k1,k2)*pi) 
            if ( nw.gt.2 ) then 
              do k3=n1,n2 
                nph3 = nph2*nphase(k3) 
                HY3(k1,k2,k3) =   nph3*GY3(-k1,-k2,-k3) 
                Hi3(k1,k2,k3) = - nph3*Gi3(-k1,-k2,-k3) 
                H3(k1,k2,k3)  = dcmplx(HY3(k1,k2,k3),Hi3(k1,k2,k3)*pi) 
                if ( nw.gt.3 ) then 
                  do k4=n1,n2 
                    nph4 = nph3*nphase(k4) 
                    HY4(k1,k2,k3,k4) =   nph4*GY4(-k1,-k2,-k3,-k4) 
                    Hi4(k1,k2,k3,k4) = - nph4*Gi4(-k1,-k2,-k3,-k4) 
                    H4(k1,k2,k3,k4)  = 
     &                    dcmplx(HY4(k1,k2,k3,k4),Hi4(k1,k2,k3,k4)*pi) 
                if ( nw.gt.4 ) then 
                  do k5=n1,n2 
                    nph5 = nph4*nphase(k5) 
                    HY5(k1,k2,k3,k4,k5) = nph5*GY5(-k1,-k2,-k3,-k4,-k5) 
                    Hi5(k1,k2,k3,k4,k5) =-nph5*Gi5(-k1,-k2,-k3,-k4,-k5) 
                    H5(k1,k2,k3,k4,k5)  = 
     &                dcmplx(HY5(k1,k2,k3,k4,k5),Hi5(k1,k2,k3,k4,k5)*pi) 
                if ( nw.gt.5 ) then 
                  do k6=n1,n2 
                    nph6 = nph5*nphase(k6) 
              HY6(k1,k2,k3,k4,k5,k6) = nph6*GY6(-k1,-k2,-k3,-k4,-k5,-k6) 
              Hi6(k1,k2,k3,k4,k5,k6) =-nph6*Gi6(-k1,-k2,-k3,-k4,-k5,-k6) 
              H6(k1,k2,k3,k4,k5,k6)  = 
     &         dcmplx(HY6(k1,k2,k3,k4,k5,k6),Hi6(k1,k2,k3,k4,k5,k6)*pi) 
                  enddo 
                endif 
                  enddo 
                endif 
                  enddo 
                endif 
              enddo 
            endif 
          enddo 
        endif 
      enddo 
      return 
      end 


      subroutine eval1dhplatminf(y,nw,H1,H2,H3,H4,H5,H6, 
     &                          HY1,HY2,HY3,HY4,HY5,HY6,
     &                          Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2)
** evaluates 1dhpl's in the (-1)-range y  <= -(r2+1) 
** evaluating first the H(..,-y) by calling eval1dhplatinf(-y), 
** and then expressing H(..,y) in terms of H(..,-y) 
      implicit double precision (a-h,o-z) 
      complex*16 H1,H2,H3,H4,H5,H6 
      complex*16 G1,G2,G3,G4,G5,G6  
      dimension H1(n1:n2),H2(n1:n2,n1:n2),H3(n1:n2,n1:n2,n1:n2), 
     &          H4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          H5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          H6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension HY1(n1:n2),HY2(n1:n2,n1:n2),HY3(n1:n2,n1:n2,n1:n2), 
     &          HY4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HY5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2),
     &          HY6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension Hi1(n1:n2),Hi2(n1:n2,n1:n2),Hi3(n1:n2,n1:n2,n1:n2), 
     &          Hi4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
** additional arrays required within this routine 
      dimension G1(-n2:-n1),G2(-n2:-n1,-n2:-n1),
     &          G3(-n2:-n1,-n2:-n1,-n2:-n1), 
     &          G4(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1), 
     &          G5(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1), 
     &          G6(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1) 
      dimension GY1(-n2:-n1),GY2(-n2:-n1,-n2:-n1),
     &          GY3(-n2:-n1,-n2:-n1,-n2:-n1), 
     &          GY4(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1), 
     &          GY5(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1), 
     &          GY6(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1) 
      dimension Gi1(-n2:-n1),Gi2(-n2:-n1,-n2:-n1),
     &          Gi3(-n2:-n1,-n2:-n1,-n2:-n1), 
     &          Gi4(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1), 
     &          Gi5(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1), 
     &          Gi6(-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1,-n2:-n1) 
** 
      common /fillred/infilldim,infill(3) 
!$omp threadprivate(/fillred/)
      dimension istorfill(3)
      dimension nphase(-1:1) 
      data nphase/-1,1,-1/ 
      parameter (pi   = 3.14159265358979324d0) 
*      print*,' eval1dhplatm1: y = ',y 
      if (infilldim.eq.2) then
         do i=1,2
            istorfill(i) = infill(i)
            infill(i) = -istorfill(i)
         enddo
      endif
** evaluate H(...,-y) 
      call setzero(nw,Gi1,Gi2,Gi3,Gi4,Gi5,Gi6,-n2,-n1) 
      Gi1(0) = -1
      call eval1dhplatinf(-y,nw,G1,G2,G3,G4,G5,G6, 
     &                  GY1,GY2,GY3,GY4,GY5,GY6,
     &                  Gi1,Gi2,Gi3,Gi4,Gi5,Gi6,-n2,-n1) 
      if (infilldim.eq.2) then
         do i=1,2
            infill(i) = istorfill(i)
         enddo
      endif
** fill the arrays H's 
      do k1=n1,n2 
        nph1 = nphase(k1) 
        HY1(k1) =   nph1*GY1(-k1) 
        Hi1(k1) = - nph1*Gi1(-k1) 
        H1(k1)  = dcmplx(HY1(k1),Hi1(k1)*pi) 
        if ( nw.gt.1 ) then 
          do k2=n1,n2 
            nph2 = nph1*nphase(k2) 
            HY2(k1,k2) =   nph2*GY2(-k1,-k2) 
            Hi2(k1,k2) = - nph2*Gi2(-k1,-k2) 
            H2(k1,k2)  = dcmplx(HY2(k1,k2),Hi2(k1,k2)*pi) 
            if ( nw.gt.2 ) then 
              do k3=n1,n2 
                nph3 = nph2*nphase(k3) 
                HY3(k1,k2,k3) =   nph3*GY3(-k1,-k2,-k3) 
                Hi3(k1,k2,k3) = - nph3*Gi3(-k1,-k2,-k3) 
                H3(k1,k2,k3)  = dcmplx(HY3(k1,k2,k3),Hi3(k1,k2,k3)*pi) 
                if ( nw.gt.3 ) then 
                  do k4=n1,n2 
                    nph4 = nph3*nphase(k4) 
                    HY4(k1,k2,k3,k4) =   nph4*GY4(-k1,-k2,-k3,-k4) 
                    Hi4(k1,k2,k3,k4) = - nph4*Gi4(-k1,-k2,-k3,-k4) 
                    H4(k1,k2,k3,k4)  = 
     &                    dcmplx(HY4(k1,k2,k3,k4),Hi4(k1,k2,k3,k4)*pi) 
                if ( nw.gt.4 ) then 
                  do k5=n1,n2 
                    nph5 = nph4*nphase(k5) 
                    HY5(k1,k2,k3,k4,k5) = nph5*GY5(-k1,-k2,-k3,-k4,-k5) 
                    Hi5(k1,k2,k3,k4,k5) =-nph5*Gi5(-k1,-k2,-k3,-k4,-k5) 
                    H5(k1,k2,k3,k4,k5)  = 
     &                dcmplx(HY5(k1,k2,k3,k4,k5),Hi5(k1,k2,k3,k4,k5)*pi) 
                if ( nw.gt.5 ) then 
                  do k6=n1,n2 
                    nph6 = nph5*nphase(k6) 
              HY6(k1,k2,k3,k4,k5,k6) = nph6*GY6(-k1,-k2,-k3,-k4,-k5,-k6) 
              Hi6(k1,k2,k3,k4,k5,k6) =-nph6*Gi6(-k1,-k2,-k3,-k4,-k5,-k6) 
              H6(k1,k2,k3,k4,k5,k6)  = 
     &         dcmplx(HY6(k1,k2,k3,k4,k5,k6),Hi6(k1,k2,k3,k4,k5,k6)*pi) 
                  enddo 
                endif 
                  enddo 
                endif 
                  enddo 
                endif 
              enddo 
            endif 
          enddo 
        endif 
      enddo 
      return 
      end 
************************************************************************ 
      subroutine setzero(nw,Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2) 
** initializes with 0 the elements of the arrays 
      implicit double precision (a-h,o-z) 
      dimension Hi1(n1:n2),Hi2(n1:n2,n1:n2),Hi3(n1:n2,n1:n2,n1:n2), 
     &          Hi4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      do k1=n1,n2 
        Hi1(k1) = 0.d0 
        if ( nw.gt.1 ) then 
          do k2=n1,n2 
            Hi2(k1,k2) = 0.d0 
            if ( nw.gt.2 ) then 
              do k3=n1,n2 
                Hi3(k1,k2,k3) = 0.d0 
                if ( nw.gt.3 ) then 
                  do k4=n1,n2 
                    Hi4(k1,k2,k3,k4) = 0.d0 
                    if ( nw.gt.4 ) then 
                      do k5=n1,n2 
                        Hi5(k1,k2,k3,k4,k5) = 0.d0 
                        if ( nw.gt.5 ) then 
                           do k6=n1,n2 
                              Hi6(k1,k2,k3,k4,k5,k6) = 0.d0 
                           enddo
                        endif
                      enddo 
                    endif 
                  enddo 
                endif 
              enddo 
            endif 
          enddo 
        endif 
      enddo 
      return 
      end 
************************************************************************ 
      subroutine fillred1dhpl(nw,H1,H2,H3,H4,H5,H6, 
     &                          HY1,HY2,HY3,HY4,HY5,HY6,
     &                          Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2)
* fills the reducible 1dhpl from the irreducible set
      implicit double precision (a-h,o-z) 
      complex*16 H1,H2,H3,H4,H5,H6
      dimension H1(n1:n2),H2(n1:n2,n1:n2),H3(n1:n2,n1:n2,n1:n2), 
     &          H4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          H5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          H6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension HY1(n1:n2),HY2(n1:n2,n1:n2),HY3(n1:n2,n1:n2,n1:n2), 
     &          HY4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HY5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HY6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension Hi1(n1:n2),Hi2(n1:n2,n1:n2),Hi3(n1:n2,n1:n2,n1:n2), 
     &          Hi4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      common /fillred/infilldim,infill(3) 
!$omp threadprivate(/fillred/)
      parameter (pinv = 0.318309886183790672d0) 
      parameter (pi   = 3.14159265358979324d0) 
** combining real and immaginary into the complex value 
      do k1=n1,n2 
      do k2=n1,n2 
        H2(k1,k2) = dcmplx(HY2(k1,k2),Hi2(k1,k2)*pi) 
        if ( nw.gt.2 ) then 
          do k3=n1,n2 
            H3(k1,k2,k3) = dcmplx(HY3(k1,k2,k3),Hi3(k1,k2,k3)*pi) 
            if ( nw.gt.3 ) then 
              do k4=n1,n2 
                H4(k1,k2,k3,k4) = 
     &               dcmplx(HY4(k1,k2,k3,k4),Hi4(k1,k2,k3,k4)*pi) 
            if ( nw.gt.4 ) then 
              do k5=n1,n2 
                H5(k1,k2,k3,k4,k5) = 
     &             dcmplx(HY5(k1,k2,k3,k4,k5),Hi5(k1,k2,k3,k4,k5)*pi) 
            if ( nw.gt.5 ) then 
              do k6=n1,n2 
                H6(k1,k2,k3,k4,k5,k6) = 
     &         dcmplx(HY6(k1,k2,k3,k4,k5,k6),Hi6(k1,k2,k3,k4,k5,k6)*pi) 
              enddo 
            endif 
              enddo 
            endif 
              enddo 
            endif 
          enddo 
        endif 
      enddo 
      enddo 
** evaluating the reduced HPL's 
** iflag = 0 to suppress auxiliary printings of FILLREDHPLx 
      iflag = 0 
      do ia =  1,infilldim 
      do ib = ia,infilldim 
        call FILLREDHPL2(iflag,H1,H2,n1,n2,infill(ia),infill(ib)) 
        if ( nw.gt.2 ) then 
          do ic = ib,infilldim 
            call FILLREDHPL3(iflag,H1,H2,H3,n1,n2, 
     &                          infill(ia),infill(ib),infill(ic)) 
            if ( nw.gt.3 ) then 
              do id = ic,infilldim 
                call FILLREDHPL4(iflag,H1,H2,H3,H4,n1,n2, 
     &               infill(ia),infill(ib),infill(ic),infill(id)) 
              enddo 
            endif 
          enddo 
        endif 
      enddo 
      enddo 
      if (nw.gt.4) call FILLREDHPL5(iflag,H1,H2,H3,H4,H5,n1,n2)
      if (nw.gt.5) call FILLREDHPL6(iflag,H1,H2,H3,H4,H5,H6,n1,n2)
** extractin real and immaginary parts from the complex value 
      do k1=n1,n2 
      do k2=n1,n2 
        HY2(k1,k2) =  dble(H2(k1,k2)) 
        Hi2(k1,k2) = dimag(H2(k1,k2))*pinv 
        if ( nw.gt.2 ) then 
          do k3=n1,n2 
            HY3(k1,k2,k3) =  dble(H3(k1,k2,k3)) 
            Hi3(k1,k2,k3) = dimag(H3(k1,k2,k3))*pinv 
            if ( nw.gt.3 ) then 
              do k4=n1,n2 
                HY4(k1,k2,k3,k4) =  dble(H4(k1,k2,k3,k4)) 
                Hi4(k1,k2,k3,k4) = dimag(H4(k1,k2,k3,k4))*pinv 
            if ( nw.gt.4 ) then 
              do k5=n1,n2 
                HY5(k1,k2,k3,k4,k5) =  dble(H5(k1,k2,k3,k4,k5)) 
                Hi5(k1,k2,k3,k4,k5) = dimag(H5(k1,k2,k3,k4,k5))*pinv 
            if ( nw.gt.5 ) then 
              do k6=n1,n2 
              HY6(k1,k2,k3,k4,k5,k6) =  dble(H6(k1,k2,k3,k4,k5,k6)) 
              Hi6(k1,k2,k3,k4,k5,k6) = dimag(H6(k1,k2,k3,k4,k5,k6))*pinv 
              enddo 
            endif 
              enddo 
            endif 
              enddo 
            endif 
          enddo 
        endif 
      enddo 
      enddo 
      return 
      end 
************************************************************************ 
      subroutine FILLREDHPL2(iflag,H1,H2,i1,i2,na,nb) 
      implicit double precision (a-h,o-z) 
      complex*16 H1,H2 
      dimension H1(i1:i2),H2(i1:i2,i1:i2) 
*23456789012345678901234567890123456789012345678901234567890123456789012 
* must be called with ordered indices na <= nb 
*      print*,' FILLREDHPL2, iflag =',iflag 
      if ( na.eq.nb ) then 
        H2(na,na) = 1.d0/2*( H1(na) )**2 
      else 
        H2(nb,na) = + H1(na)*H1(nb) - H2(na,nb) 
        if ( iflag.eq.1 ) then 
          call printer2(na,nb) 
        endif 
      endif 
      return 
      end 
************************************************************************ 
      subroutine FILLREDHPL3(iflag,H1,H2,H3,i1,i2,ia,ib,ic) 
      implicit double precision (a-h,o-z) 
      complex*16 H1,H2,H3 
      dimension H1(i1:i2),H2(i1:i2,i1:i2),H3(i1:i2,i1:i2,i1:i2) 
* must be called with "properly ordered" indices 
* note in particular the remapping of, say, (ia,ib,ic) into 
* (na,na,nb) of ReducerTest.out 
      na = ia 
      if ( (ia.eq.ib).and.(ib.eq.ic) ) then 
* case (na,na,na) 
        H3(na,na,na) = 1.d0/6*( H1(na) )**3 
* ic cannot be anymore equal to ia 
      else if ( ic.eq.ia ) then 
        print*,' FILLREDHPL3, error 1, called with arguments ' 
        print*,'               ',ia,ib,ic 
        stop 
      else if ( ia.eq.ib ) then 
* case (na,na,nb) 
        nb = ic 
        if ( iflag.eq.1 ) then 
          call printer3(na,na,nb) 
        endif 
        H3(na,nb,na) = + H1(na)*H2(na,nb) - 2*H3(na,na,nb) 
        H3(nb,na,na) = + 1.d0/2*H1(na)*H1(na)*H1(nb) 
     &                 - H1(na)*H2(na,nb) + H3(na,na,nb) 
* ib cannot be anymore equal to ia 
      else if ( ib.eq.ia ) then 
        print*,' FILLREDHPL3, error 2, called with arguments ' 
        print*,'               ',ia,ib,ic 
        stop 
      else if ( ib.eq.ic ) then 
* case (na,nb,nb) 
        nb = ib 
        if ( iflag.eq.1 ) then 
          call printer3(na,nb,nb) 
        endif 
        H3(nb,na,nb) = + H1(nb)*H2(na,nb) - 2*H3(na,nb,nb) 
        H3(nb,nb,na) = + 1.d0/2*H1(na)*H1(nb)*H1(nb) 
     &                 - H1(nb)*H2(na,nb) + H3(na,nb,nb) 
* no need to protect against ic.eq.ib 
* when arriving here all indices are different 
      else 
* case (na,nb,nc)    all indices are different 
        nb = ib 
        nc = ic 
        if ( iflag.eq.1 ) then 
          call printer3(na,nb,nc) 
          call printer3(na,nc,nb) 
        endif 
        H3(nb,na,nc) = + H1(nb)*H2(na,nc) 
     &                 - H3(na,nb,nc) - H3(na,nc,nb) 
        H3(nb,nc,na) = + H1(na)*H2(nb,nc) 
     &                 - H1(nb)*H2(na,nc) + H3(na,nc,nb) 
        H3(nc,na,nb) = + H1(nc)*H2(na,nb) 
     &                 - H3(na,nb,nc) - H3(na,nc,nb) 
        H3(nc,nb,na) = + H1(na)*H1(nb)*H1(nc) - H1(na)*H2(nb,nc) 
     &                 - H1(nc)*H2(na,nb) + H3(na,nb,nc) 
      endif 
*23456789012345678901234567890123456789012345678901234567890123456789012 
      return 
      end 
************************************************************************ 
      subroutine FILLREDHPL4(iflag,H1,H2,H3,H4,i1,i2,ia,ib,ic,id) 
      implicit double precision (a-h,o-z) 
      complex*16 H1,H2,H3,H4 
      dimension H1(i1:i2),H2(i1:i2,i1:i2),H3(i1:i2,i1:i2,i1:i2) 
      dimension H4(i1:i2,i1:i2,i1:i2,i1:i2) 
*23456789012345678901234567890123456789012345678901234567890123456789012 
* must be called with "properly ordered" indices 
* note in particular the remapping of, say, (ia,ib,ic) into 
* (na,na,nb) of ReducerTest.out 
      na = ia 
      if ( (ia.eq.ib).and.(ib.eq.ic).and.(ic.eq.id) ) then 
* case (na,na,na,na) 
        H4(na,na,na,na) = 1.d0/24*( H1(na) )**4 
* id cannot be anymore equal to ia 
      else if ( id.eq.ia ) then 
        print*,' FILLREDHPL4, error 1, called with arguments ' 
        print*,'               ',ia,ib,ic,id 
        stop 
      else if ( (ia.eq.ib).and.(ib.eq.ic) ) then 
* case (na,na,na,nb) 
        nb = id 
        H4(na,na,nb,na) = + H1(na)*H3(na,na,nb) - 3*H4(na,na,na,nb) 
        H4(na,nb,na,na) = + 1.d0/2*H1(na)*H1(na)*H2(na,nb) 
     &                    - 2*H1(na)*H3(na,na,nb) + 3*H4(na,na,na,nb) 
        H4(nb,na,na,na) = + 1.d0/6*H1(na)*H1(na)*H1(na)*H1(nb) 
     &                    - 1.d0/2*H1(na)*H1(na)*H2(na,nb) 
     &                    + H1(na)*H3(na,na,nb) - H4(na,na,na,nb) 
        if ( iflag.eq.1 ) then 
          call printer4(na,na,na,nb) 
        endif 
* ic cannot be anymore equal to ia 
      else if ( ic.eq.ia ) then 
        print*,' FILLREDHPL4, error 2, called with arguments ' 
        print*,'               ',ia,ib,ic,id 
        stop 
      else if ( (ia.eq.ib).and.(ic.eq.id) ) then 
* case (na,na,nb,nb) 
        nb = ic 
        H4(na,nb,na,nb) = + 1.d0/2*H2(na,nb)*H2(na,nb) 
     &                    - 2*H4(na,na,nb,nb) 
        H4(na,nb,nb,na) = + H1(na)*H3(na,nb,nb) 
     &                    - 1.d0/2*H2(na,nb)*H2(na,nb) 
        H4(nb,na,na,nb) = + H1(nb)*H3(na,na,nb) 
     &                    - 1.d0/2*H2(na,nb)*H2(na,nb) 
        H4(nb,na,nb,na) = + H1(na)*H1(nb)*H2(na,nb) 
     &                    - 2*H1(na)*H3(na,nb,nb) 
     &                    - 2*H1(nb)*H3(na,na,nb) 
     &                    + 1.d0/2*H2(na,nb)*H2(na,nb) 
     &                    + 2*H4(na,na,nb,nb) 
        H4(nb,nb,na,na) = + 1.d0/4*H1(na)*H1(na)*H1(nb)*H1(nb) 
     &                    - H1(na)*H1(nb)*H2(na,nb) 
     &                    + H1(na)*H3(na,nb,nb) 
     &                    + H1(nb)*H3(na,na,nb) - H4(na,na,nb,nb) 
        if ( iflag.eq.1 ) then 
          call printer4(na,na,nb,nb) 
        endif 
      else if ( ia.eq.ib ) then 
* case (na,na,nb,nc) 
        nb = ic 
        nc = id 
        H4(na,nb,nc,na) = + H1(na)*H3(na,nb,nc) - 2*H4(na,na,nb,nc) 
     &                    - H4(na,nb,na,nc) 
        H4(na,nc,na,nb) = + H2(na,nb)*H2(na,nc) - 2*H4(na,na,nb,nc) 
     &                    - 2*H4(na,na,nc,nb) - H4(na,nb,na,nc) 
        H4(na,nc,nb,na) = + H1(na)*H3(na,nc,nb) - H2(na,nb)*H2(na,nc) 
     &                    + 2*H4(na,na,nb,nc) + H4(na,nb,na,nc) 
        H4(nb,na,na,nc) = + H1(nb)*H3(na,na,nc) - H4(na,na,nb,nc) 
     &                    - H4(na,na,nc,nb) - H4(na,nb,na,nc) 
        H4(nb,na,nc,na) = + H1(na)*H1(nb)*H2(na,nc) 
     &                    - H1(na)*H3(na,nb,nc) - H1(na)*H3(na,nc,nb) 
     &                    - 2*H1(nb)*H3(na,na,nc) + 2*H4(na,na,nb,nc) 
     &                    + 2*H4(na,na,nc,nb) + H4(na,nb,na,nc) 
        H4(nb,nc,na,na) = + 1.d0/2*H1(na)*H1(na)*H2(nb,nc) 
     &                    - H1(na)*H1(nb)*H2(na,nc) 
     &                    + H1(na)*H3(na,nc,nb) + H1(nb)*H3(na,na,nc) 
     &                    - H4(na,na,nc,nb) 
        H4(nc,na,na,nb) = + H1(nc)*H3(na,na,nb) - H2(na,nb)*H2(na,nc) 
     &                    + H4(na,na,nb,nc) + H4(na,na,nc,nb) 
     &                    + H4(na,nb,na,nc) 
        H4(nc,na,nb,na) = + H1(na)*H1(nc)*H2(na,nb) 
     &                    - H1(na)*H3(na,nb,nc) - H1(na)*H3(na,nc,nb) 
     &                    - 2*H1(nc)*H3(na,na,nb) + H2(na,nb)*H2(na,nc) 
     &                    - H4(na,nb,na,nc) 
        H4(nc,nb,na,na) = + 1.d0/2*H1(na)*H1(na)*H1(nb)*H1(nc) 
     &                    - 1.d0/2*H1(na)*H1(na)*H2(nb,nc) 
     &                    - H1(na)*H1(nc)*H2(na,nb) 
     &                    + H1(na)*H3(na,nb,nc) + H1(nc)*H3(na,na,nb) 
     &                    - H4(na,na,nb,nc)  
        if ( iflag.eq.1 ) then 
          call printer4(na,na,nb,nc) 
          call printer4(na,na,nc,nb) 
          call printer4(na,nb,na,nc) 
        endif 
* ib cannot be anymore equal to ia 
      else if ( ib.eq.ia ) then 
        print*,' FILLREDHPL4, error 3, called with arguments ' 
        print*,'               ',ia,ib,ic,id 
        stop 
      else if ( (ib.eq.ic).and.(ic.eq.id) ) then 
* case (na,nb,nb,nb) 
        nb = ib 
        H4(nb,na,nb,nb) = + H1(nb)*H3(na,nb,nb) - 3*H4(na,nb,nb,nb) 
        H4(nb,nb,na,nb) = + 1.d0/2*H1(nb)*H1(nb)*H2(na,nb) 
     &                    - 2*H1(nb)*H3(na,nb,nb) + 3*H4(na,nb,nb,nb) 
        H4(nb,nb,nb,na) = + 1.d0/6*H1(na)*H1(nb)*H1(nb)*H1(nb) 
     &                    - 1.d0/2*H1(nb)*H1(nb)*H2(na,nb) 
     &                    + H1(nb)*H3(na,nb,nb) - H4(na,nb,nb,nb) 
        if ( iflag.eq.1 ) then 
          call printer4(na,nb,nb,nb) 
        endif 
* id cannot be anymore equal to ib 
      else if ( id.eq.ib ) then 
        print*,' FILLREDHPL4, error 4, called with arguments ' 
        print*,'               ',ia,ib,ic,id 
        stop 
      else if ( ib.eq.ic ) then 
* case (na,nb,nb,nc) 
        nb = ib 
        nc = id 
        H4(nb,na,nb,nc) = + H1(nb)*H3(na,nb,nc) 
     &                    - 2*H4(na,nb,nb,nc) - H4(na,nb,nc,nb) 
        H4(nb,na,nc,nb) = + H1(nb)*H3(na,nc,nb) - H4(na,nb,nc,nb) 
     &                    - 2*H4(na,nc,nb,nb) 
        H4(nb,nb,na,nc) = + 1.d0/2*H1(nb)*H1(nb)*H2(na,nc) 
     &                    - H1(nb)*H3(na,nb,nc) - H1(nb)*H3(na,nc,nb) 
     &                    + H4(na,nb,nb,nc) + H4(na,nb,nc,nb) 
     &                    + H4(na,nc,nb,nb) 
        H4(nb,nb,nc,na) = + H1(na)*H3(nb,nb,nc) 
     &                    - 1.d0/2*H1(nb)*H1(nb)*H2(na,nc) 
     &                    + H1(nb)*H3(na,nc,nb) - H4(na,nc,nb,nb) 
        H4(nb,nc,na,nb) = - H1(nb)*H3(na,nb,nc) - H1(nb)*H3(na,nc,nb) 
     &                    + H2(na,nb)*H2(nb,nc) + H4(na,nb,nc,nb) 
     &                    + 2*H4(na,nc,nb,nb) 
        H4(nb,nc,nb,na) = + H1(na)*H1(nb)*H2(nb,nc) 
     &                    - 2*H1(na)*H3(nb,nb,nc) 
     &                    + H1(nb)*H3(na,nb,nc) 
     &                    - H2(na,nb)*H2(nb,nc) - H4(na,nb,nc,nb) 
        H4(nc,na,nb,nb) = + H1(nc)*H3(na,nb,nb) - H4(na,nb,nb,nc) 
     &                    - H4(na,nb,nc,nb) - H4(na,nc,nb,nb) 
        H4(nc,nb,na,nb) = + H1(nb)*H1(nc)*H2(na,nb) 
     &                    - 2*H1(nc)*H3(na,nb,nb) 
     &                    - H2(na,nb)*H2(nb,nc) + 2*H4(na,nb,nb,nc) 
     &                    + H4(na,nb,nc,nb) 
        H4(nc,nb,nb,na) = + 1.d0/2*H1(na)*H1(nb)*H1(nb)*H1(nc) 
     &                    - H1(na)*H1(nb)*H2(nb,nc) 
     &                    + H1(na)*H3(nb,nb,nc) 
     &                    - H1(nb)*H1(nc)*H2(na,nb) 
     &                    + H1(nc)*H3(na,nb,nb) + H2(na,nb)*H2(nb,nc) 
     &                    - H4(na,nb,nb,nc) 
        if ( iflag.eq.1 ) then 
          call printer4(na,nb,nb,nc) 
          call printer4(na,nb,nc,nb) 
          call printer4(na,nc,nb,nb) 
        endif 
* ic cannot be anymore equal to ib 
      else if ( ic.eq.ib ) then 
        print*,' FILLREDHPL4, error 5, called with arguments ' 
        print*,'               ',ia,ib,ic,id 
        stop 
      else if ( ic.eq.id ) then 
* case (na,nb,nc,nc) 
        nb = ib 
        nc = ic 
        H4(nb,na,nc,nc) = + H1(nb)*H3(na,nc,nc) - H4(na,nb,nc,nc) 
     &                    - H4(na,nc,nb,nc) - H4(na,nc,nc,nb) 
        H4(nb,nc,na,nc) = - 2*H1(nb)*H3(na,nc,nc) + H2(na,nc)*H2(nb,nc) 
     &                    + H4(na,nc,nb,nc) + 2*H4(na,nc,nc,nb) 
        H4(nb,nc,nc,na) = + H1(na)*H3(nb,nc,nc) + H1(nb)*H3(na,nc,nc) 
     &                    - H2(na,nc)*H2(nb,nc) - H4(na,nc,nc,nb) 
        H4(nc,na,nb,nc) = + H1(nc)*H3(na,nb,nc) - 2*H4(na,nb,nc,nc) 
     &                    - H4(na,nc,nb,nc) 
        H4(nc,na,nc,nb) = + H1(nc)*H3(na,nc,nb) - H4(na,nc,nb,nc) 
     &                    - 2*H4(na,nc,nc,nb) 
        H4(nc,nb,na,nc) = + H1(nb)*H1(nc)*H2(na,nc) 
     &                    - H1(nc)*H3(na,nb,nc) - H1(nc)*H3(na,nc,nb) 
     &                    - H2(na,nc)*H2(nb,nc) + 2*H4(na,nb,nc,nc) 
     &                    + H4(na,nc,nb,nc) 
        H4(nc,nb,nc,na) = + H1(na)*H1(nc)*H2(nb,nc) 
     &                    - 2*H1(na)*H3(nb,nc,nc) 
     &                    - H1(nb)*H1(nc)*H2(na,nc) 
     &                    + H1(nc)*H3(na,nc,nb) + H2(na,nc)*H2(nb,nc) 
     &                    - H4(na,nc,nb,nc) 
        H4(nc,nc,na,nb) = + 1.d0/2*H1(nc)*H1(nc)*H2(na,nb) 
     &                    - H1(nc)*H3(na,nb,nc) - H1(nc)*H3(na,nc,nb) 
     &                    + H4(na,nb,nc,nc) + H4(na,nc,nb,nc) 
     &                    + H4(na,nc,nc,nb) 
        H4(nc,nc,nb,na) = + 1.d0/2*H1(na)*H1(nb)*H1(nc)*H1(nc) 
     &                    - H1(na)*H1(nc)*H2(nb,nc) 
     &                    + H1(na)*H3(nb,nc,nc) 
     &                    - 1.d0/2*H1(nc)*H1(nc)*H2(na,nb) 
     &                    + H1(nc)*H3(na,nb,nc) - H4(na,nb,nc,nc) 
        if ( iflag.eq.1 ) then 
          call printer4(na,nb,nc,nc) 
          call printer4(na,nc,nb,nc) 
          call printer4(na,nc,nc,nb) 
        endif 
* no need to protect against id.eq.ic 
* when arriving here all indices are different 
      else 
* case (na,nb,nc,nd) all indices are different 
        nb = ib 
        nc = ic 
        nd = id 
        H4(nb,na,nc,nd) = + H1(nb)*H3(na,nc,nd) - H4(na,nb,nc,nd) 
     &                    - H4(na,nc,nb,nd) - H4(na,nc,nd,nb) 
        H4(nb,na,nd,nc) = + H1(nb)*H3(na,nd,nc) - H4(na,nb,nd,nc) 
     &                    - H4(na,nd,nb,nc) - H4(na,nd,nc,nb) 
        H4(nb,nc,na,nd) = - H1(nb)*H3(na,nc,nd) - H1(nb)*H3(na,nd,nc) 
     &                    + H2(na,nd)*H2(nb,nc) + H4(na,nc,nb,nd) 
     &                    + H4(na,nc,nd,nb) + H4(na,nd,nc,nb) 
        H4(nb,nc,nd,na) = + H1(na)*H3(nb,nc,nd) + H1(nb)*H3(na,nd,nc) 
     &                    - H2(na,nd)*H2(nb,nc) - H4(na,nd,nc,nb) 
        H4(nb,nd,na,nc) = - H1(nb)*H3(na,nc,nd) - H1(nb)*H3(na,nd,nc) 
     &                    + H2(na,nc)*H2(nb,nd) + H4(na,nc,nd,nb) 
     &                    + H4(na,nd,nb,nc) + H4(na,nd,nc,nb) 
        H4(nb,nd,nc,na) = + H1(na)*H3(nb,nd,nc) + H1(nb)*H3(na,nc,nd) 
     &                    - H2(na,nc)*H2(nb,nd) - H4(na,nc,nd,nb) 
        H4(nc,na,nb,nd) = + H1(nc)*H3(na,nb,nd) - H4(na,nb,nc,nd) 
     &                    - H4(na,nb,nd,nc) - H4(na,nc,nb,nd) 
        H4(nc,na,nd,nb) = + H1(nc)*H3(na,nd,nb) - H4(na,nc,nd,nb) 
     &                    - H4(na,nd,nb,nc) - H4(na,nd,nc,nb) 
        H4(nc,nb,na,nd) = + H1(nb)*H1(nc)*H2(na,nd) 
     &                    - H1(nc)*H3(na,nb,nd) - H1(nc)*H3(na,nd,nb) 
     &                    - H2(na,nd)*H2(nb,nc) + H4(na,nb,nc,nd) 
     &                    + H4(na,nb,nd,nc) + H4(na,nd,nb,nc) 
        H4(nc,nb,nd,na) = + H1(na)*H1(nc)*H2(nb,nd) 
     &                    - H1(na)*H3(nb,nc,nd) - H1(na)*H3(nb,nd,nc) 
     &                    - H1(nb)*H1(nc)*H2(na,nd) 
     &                    + H1(nc)*H3(na,nd,nb) + H2(na,nd)*H2(nb,nc) 
     &                    - H4(na,nd,nb,nc) 
        H4(nc,nd,na,nb) = - H1(nc)*H3(na,nb,nd) - H1(nc)*H3(na,nd,nb) 
     &                    + H2(na,nb)*H2(nc,nd) + H4(na,nb,nd,nc) 
     &                    + H4(na,nd,nb,nc) + H4(na,nd,nc,nb) 
        H4(nc,nd,nb,na) = + H1(na)*H1(nb)*H2(nc,nd) 
     &                    - H1(na)*H1(nc)*H2(nb,nd) 
     &                    + H1(na)*H3(nb,nd,nc) + H1(nc)*H3(na,nb,nd) 
     &                    - H2(na,nb)*H2(nc,nd) - H4(na,nb,nd,nc) 
        H4(nd,na,nb,nc) = + H1(nd)*H3(na,nb,nc) - H4(na,nb,nc,nd) 
     &                    - H4(na,nb,nd,nc) - H4(na,nd,nb,nc) 
        H4(nd,na,nc,nb) = + H1(nd)*H3(na,nc,nb) - H4(na,nc,nb,nd) 
     &                    - H4(na,nc,nd,nb) - H4(na,nd,nc,nb) 
        H4(nd,nb,na,nc) = + H1(nb)*H1(nd)*H2(na,nc) 
     &                    - H1(nd)*H3(na,nb,nc) - H1(nd)*H3(na,nc,nb) 
     &                    - H2(na,nc)*H2(nb,nd) + H4(na,nb,nc,nd) 
     &                    + H4(na,nb,nd,nc) + H4(na,nc,nb,nd) 
        H4(nd,nb,nc,na) = + H1(na)*H1(nd)*H2(nb,nc) 
     &                    - H1(na)*H3(nb,nc,nd) - H1(na)*H3(nb,nd,nc) 
     &                    - H1(nb)*H1(nd)*H2(na,nc) 
     &                    + H1(nd)*H3(na,nc,nb) + H2(na,nc)*H2(nb,nd) 
     &                    - H4(na,nc,nb,nd) 
        H4(nd,nc,na,nb) = + H1(nc)*H1(nd)*H2(na,nb) 
     &                    - H1(nd)*H3(na,nb,nc) - H1(nd)*H3(na,nc,nb) 
     &                    - H2(na,nb)*H2(nc,nd) + H4(na,nb,nc,nd) 
     &                    + H4(na,nc,nb,nd) + H4(na,nc,nd,nb) 
        H4(nd,nc,nb,na) = + H1(na)*H1(nb)*H1(nc)*H1(nd) 
     &                    - H1(na)*H1(nb)*H2(nc,nd) 
     &                    - H1(na)*H1(nd)*H2(nb,nc) 
     &                    + H1(na)*H3(nb,nc,nd) 
     &                    - H1(nc)*H1(nd)*H2(na,nb) 
     &                    + H1(nd)*H3(na,nb,nc) 
     &                    + H2(na,nb)*H2(nc,nd) - H4(na,nb,nc,nd) 
        if ( iflag.eq.1 ) then 
          call printer4(na,nb,nc,nd) 
          call printer4(na,nb,nd,nc) 
          call printer4(na,nc,nb,nd) 
          call printer4(na,nc,nb,nd) 
          call printer4(na,nd,nb,nc) 
          call printer4(na,nd,nc,nb) 
        endif 
      endif 
*23456789012345678901234567890123456789012345678901234567890123456789012 
      return 
      end 
************************************************************************ 
      subroutine printer2(na,nb) 

      write(11,'(''g [H('',$)') 
      call subprint(11,na) 
      write(11,'('','',$)') 
      call subprint(11,nb) 
      write(11,'('',y)] = H('',$)') 
      call subprint(11,na) 
      write(11,'('','',$)') 
      call subprint(11,nb) 
      write(11,'('',y) ; '')') 

      write(12,'(''id H('',$)') 
      call subprint(12,na) 
      write(12,'('','',$)') 
      call subprint(12,nb) 
      write(12,'('',y) = H[('',$)') 
      call subprint(12,na) 
      write(12,'('','',$)') 
      call subprint(12,nb) 
      write(12,'('',y)] ; '')') 

      return 
      end 
*** 
      subroutine printer3(na,nb,nc) 

      write(11,'(''g [H('',$)') 
      call subprint(11,na) 
      write(11,'('','',$)') 
      call subprint(11,nb) 
      write(11,'('','',$)') 
      call subprint(11,nc) 
      write(11,'('',y)] = H('',$)') 
      call subprint(11,na) 
      write(11,'('','',$)') 
      call subprint(11,nb) 
      write(11,'('','',$)') 
      call subprint(11,nc) 
      write(11,'('',y) ; '')') 

      write(12,'(''id H('',$)') 
      call subprint(12,na) 
      write(12,'('','',$)') 
      call subprint(12,nb) 
      write(12,'('','',$)') 
      call subprint(12,nc) 
      write(12,'('',y) = H[('',$)') 
      call subprint(12,na) 
      write(12,'('','',$)') 
      call subprint(12,nb) 
      write(12,'('',y)] ; '')') 

      return 
      end 
*** 
      subroutine printer4(na,nb,nc,nd) 

      write(11,'(''g [H('',$)') 
      call subprint(11,na) 
      write(11,'('','',$)') 
      call subprint(11,nb) 
      write(11,'('','',$)') 
      call subprint(11,nc) 
      write(11,'('','',$)') 
      call subprint(11,nd) 
      write(11,'('',y)] = H('',$)') 
      call subprint(11,na) 
      write(11,'('','',$)') 
      call subprint(11,nb) 
      write(11,'('','',$)') 
      call subprint(11,nc) 
      write(11,'('','',$)') 
      call subprint(11,nd) 
      write(11,'('',y) ; '')') 

      write(12,'(''id H('',$)') 
      call subprint(12,na) 
      write(12,'('','',$)') 
      call subprint(12,nb) 
      write(12,'('','',$)') 
      call subprint(12,nc) 
      write(12,'('','',$)') 
      call subprint(12,nd) 
      write(12,'('',y) = H[('',$)') 
      call subprint(12,na) 
      write(12,'('','',$)') 
      call subprint(12,nb) 
      write(12,'('','',$)') 
      call subprint(12,nc) 
      write(12,'('','',$)') 
      call subprint(12,nd) 
      write(12,'('',y)] ; '')') 

      return 
      end 
*** 
      subroutine subprint(n,na) 
      if ( na.lt.0 ) then 
        write (n,102) na 
      else 
        write (n,101) na 
      endif 
      return 
  101 format(i1,$) 
  102 format(i2,$) 
      end 


      subroutine FILLREDHPL5(iflag,HZ1,HZ2,HZ3,HZ4,HZ5,n1,n2)
      implicit double precision (a-h,o-z) 
      complex*16 HZ1,HZ2,HZ3,HZ4,HZ5 
      dimension HZ1(n1:n2),HZ2(n1:n2,n1:n2),HZ3(n1:n2,n1:n2,n1:n2) 
      dimension HZ4(n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension HZ5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 

** evaluating the expansions 

      HZ5(0,0,0,0,0) = 
     &  + 8.3333333333333333d-03*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)

** (n1,n2) = (0,1) or (-1,1) 
      if (    ( (n1.eq.0).and.(n2.eq.1) ) 
     &    .or.( (n1.eq.-1).and.(n2.eq.1) ) ) then

      HZ5(0,0,0,1,0) = 
     &  + HZ1(0) *HZ4(0,0,0,1)
     &  - 4.0000000000000000d+00*HZ5(0,0,0,0,1)
      HZ5(0,0,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(0,0,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ4(0,0,0,1)
     &  + 6.0000000000000000d+00*HZ5(0,0,0,0,1)
      HZ5(0,0,1,1,0) = 
     &  + HZ1(0) *HZ4(0,0,1,1)
     &  - 3.0000000000000000d+00*HZ5(0,0,0,1,1)
     &  - HZ5(0,0,1,0,1) 
      HZ5(0,1,0,0,0) = 
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ2(0,1)
     &  - HZ1(0) *HZ1(0)*HZ3(0,0,1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ4(0,0,0,1)
     &  - 4.0000000000000000d+00*HZ5(0,0,0,0,1)
      HZ5(0,1,0,0,1) = 
     &  + HZ2(0,1) *HZ3(0,0,1)
     &  - 6.0000000000000000d+00*HZ5(0,0,0,1,1)
     &  - 3.0000000000000000d+00*HZ5(0,0,1,0,1)
      HZ5(0,1,0,1,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ2(0,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ4(0,0,1,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ3(0,0,1)
     &  + 1.2000000000000000d+01*HZ5(0,0,0,1,1)
     &  + 4.0000000000000000d+00*HZ5(0,0,1,0,1)
      HZ5(0,1,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(0,1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ2(0,1)*HZ2(0,1)
     &  + HZ2(0,1) *HZ3(0,0,1)
     &  - 3.0000000000000000d+00*HZ5(0,0,0,1,1)
     &  - HZ5(0,0,1,0,1) 
      HZ5(0,1,1,0,1) = 
     &  + HZ2(0,1) *HZ3(0,1,1)
     &  - 6.0000000000000000d+00*HZ5(0,0,1,1,1)
     &  - 3.0000000000000000d+00*HZ5(0,1,0,1,1)
      HZ5(0,1,1,1,0) = 
     &  + HZ1(0) *HZ4(0,1,1,1)
     &  - HZ2(0,1) *HZ3(0,1,1)
     &  + 4.0000000000000000d+00*HZ5(0,0,1,1,1)
     &  + 2.0000000000000000d+00*HZ5(0,1,0,1,1)
      HZ5(1,0,0,0,0) = 
     &  + 4.1666666666666666d-02*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(1)
     &  - 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(0,0,1)
     &  - HZ1(0) *HZ4(0,0,0,1)
     &  + HZ5(0,0,0,0,1) 
      HZ5(1,0,0,0,1) = 
     &  + HZ1(1) *HZ4(0,0,0,1)
     &  - HZ2(0,1) *HZ3(0,0,1)
     &  + 4.0000000000000000d+00*HZ5(0,0,0,1,1)
     &  + 2.0000000000000000d+00*HZ5(0,0,1,0,1)
      HZ5(1,0,0,1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ3(0,0,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ2(0,1)*HZ2(0,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ4(0,0,0,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ3(0,0,1)
     &  - 6.0000000000000000d+00*HZ5(0,0,0,1,1)
     &  - 3.0000000000000000d+00*HZ5(0,0,1,0,1)
      HZ5(1,0,0,1,1) = 
     &  + HZ1(1) *HZ4(0,0,1,1)
     &  - 3.0000000000000000d+00*HZ5(0,0,1,1,1)
     &  - HZ5(0,1,0,1,1) 
      HZ5(1,0,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ2(0,1)
     &  - HZ1(0) *HZ1(0)*HZ3(0,1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ3(0,0,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ2(0,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ4(0,0,1,1)
     &  + 3.0000000000000000d+00*HZ1(1)*HZ4(0,0,0,1)
     &  - HZ2(0,1) *HZ3(0,0,1)
     &  + HZ5(0,0,1,0,1) 
      HZ5(1,0,1,0,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ2(0,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ4(0,0,1,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ3(0,1,1)
     &  + 1.2000000000000000d+01*HZ5(0,0,1,1,1)
     &  + 4.0000000000000000d+00*HZ5(0,1,0,1,1)
      HZ5(1,0,1,1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ3(0,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ4(0,1,1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ2(0,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ3(0,1,1)
     &  - 6.0000000000000000d+00*HZ5(0,0,1,1,1)
     &  - 3.0000000000000000d+00*HZ5(0,1,0,1,1)
      HZ5(1,0,1,1,1) = 
     &  + HZ1(1) *HZ4(0,1,1,1)
     &  - 4.0000000000000000d+00*HZ5(0,1,1,1,1)
      HZ5(1,1,0,0,0) = 
     &  + 8.3333333333333333d-02*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(1)*HZ1(1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(0,1,1)
     &  + HZ1(0) *HZ1(1)*HZ3(0,0,1)
     &  - HZ1(0) *HZ4(0,0,1,1)
     &  - HZ1(1) *HZ4(0,0,0,1)
     &  + HZ5(0,0,0,1,1) 
      HZ5(1,1,0,0,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(0,0,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ2(0,1)*HZ2(0,1)
     &  + HZ2(0,1) *HZ3(0,1,1)
     &  - 3.0000000000000000d+00*HZ5(0,0,1,1,1)
     &  - HZ5(0,1,0,1,1) 
      HZ5(1,1,0,1,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ3(0,1,1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ4(0,1,1,1)
     &  - HZ1(1) *HZ1(1)*HZ3(0,0,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ2(0,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ4(0,0,1,1)
     &  - HZ2(0,1) *HZ3(0,1,1)
     &  + HZ5(0,1,0,1,1) 
      HZ5(1,1,0,1,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(0,1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ4(0,1,1,1)
     &  + 6.0000000000000000d+00*HZ5(0,1,1,1,1)
      HZ5(1,1,1,0,0) = 
     &  + 8.3333333333333333d-02*HZ1(0)*HZ1(0)*HZ1(1)*HZ1(1)*HZ1(1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ2(0,1)
     &  + HZ1(0) *HZ1(1)*HZ3(0,1,1)
     &  - HZ1(0) *HZ4(0,1,1,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(0,0,1)
     &  - HZ1(1) *HZ4(0,0,1,1)
     &  + HZ5(0,0,1,1,1) 
      HZ5(1,1,1,0,1) = 
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(0,1)
     &  - HZ1(1) *HZ1(1)*HZ3(0,1,1)
     &  + 3.0000000000000000d+00*HZ1(1)*HZ4(0,1,1,1)
     &  - 4.0000000000000000d+00*HZ5(0,1,1,1,1)
      HZ5(1,1,1,1,0) = 
     &  + 4.1666666666666666d-02*HZ1(0)*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)
     &  - 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(0,1,1)
     &  - HZ1(1) *HZ4(0,1,1,1)
     &  + HZ5(0,1,1,1,1) 
      HZ5(1,1,1,1,1) = 
     &  + 8.3333333333333333d-03*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)
      endif

      if (    ( (n1.eq.-1).and.(n2.eq.0) ) 
     &    .or.( (n1.eq.-1).and.(n2.eq.1) ) ) then 
      HZ5(-1,-1,-1,-1,-1) = 
     &  + 8.3333333333333333d-03*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)
      HZ5(-1,-1,-1,-1,0) = 
     &  + 4.1666666666666666d-02*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(0)
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ2(0,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ3(0,-1,-1)
     &  - HZ1( -1)*HZ4(0,-1,-1,-1)
     &  + HZ5(0, -1,-1,-1,-1)
      HZ5(-1,-1,-1,0,-1) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ2(0,-1)
     &  - HZ1( -1)*HZ1(-1)*HZ3(0,-1,-1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ4(0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HZ5(0,-1,-1,-1,-1)
      HZ5(-1,-1,-1,0,0) = 
     &  + 8.3333333333333333d-02*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(0)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ2(0,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ3(0,0,-1)
     &  + HZ1( -1)*HZ1(0)*HZ3(0,-1,-1)
     &  - HZ1( -1)*HZ4(0,0,-1,-1)
     &  - HZ1(0) *HZ4(0,-1,-1,-1)
     &  + HZ5(0,0, -1,-1,-1)
      HZ5(-1,-1,0,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ3(0,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ4(0,-1,-1,-1)
     &  + 6.0000000000000000d+00*HZ5(0,-1,-1,-1,-1)
      HZ5(-1,-1,0,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ2(0,-1)
     &  - HZ1( -1)*HZ1(-1)*HZ3(0,0,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ3(0,-1,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ2(0,-1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ4(0,-1,-1,-1)
     &  - HZ2(0, -1)*HZ3(0,-1,-1)
     &  + HZ5(0, -1,0,-1,-1)
      HZ5(-1,-1,0,0,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ3(0,0,-1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ2(0,-1)*HZ2(0,-1)
     &  + HZ2(0, -1)*HZ3(0,-1,-1)
     &  - HZ5(0, -1,0,-1,-1)
     &  - 3.0000000000000000d+00*HZ5(0,0,-1,-1,-1)
      HZ5(-1,-1,0,0,0) = 
     &  + 8.3333333333333333d-02*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(0)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ2(0,-1)
     &  + HZ1( -1)*HZ1(0)*HZ3(0,0,-1)
     &  - HZ1( -1)*HZ4(0,0,0,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(0,-1,-1)
     &  - HZ1(0) *HZ4(0,0,-1,-1)
     &  + HZ5(0,0,0, -1,-1)
      HZ5(-1,0,-1,-1,-1) = 
     &  + HZ1( -1)*HZ4(0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HZ5(0,-1,-1,-1,-1)
      HZ5(-1,0,-1,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ3(0,-1,-1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ2(0,-1)*HZ2(0,-1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ3(0,-1,-1)
     &  - 3.0000000000000000d+00*HZ5(0,-1,0,-1,-1)
     &  - 6.0000000000000000d+00*HZ5(0,0,-1,-1,-1)
      HZ5(-1,0,-1,0,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ2(0,-1)*HZ2(0,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ3(0,-1,-1)
     &  + 4.0000000000000000d+00*HZ5(0,-1,0,-1,-1)
     &  + 1.2000000000000000d+01*HZ5(0,0,-1,-1,-1)
      HZ5(-1,0,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ2(0,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ3(0,0,-1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ4(0,0,0,-1)
     &  - HZ1(0) *HZ1(0)*HZ3(0,-1,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ2(0,-1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ4(0,0,-1,-1)
     &  - HZ2(0, -1)*HZ3(0,0,-1)
     &  + HZ5(0,0, -1,0,-1)
      HZ5(-1,0,0,-1,-1) = 
     &  + HZ1( -1)*HZ4(0,0,-1,-1)
     &  - HZ5(0, -1,0,-1,-1)
     &  - 3.0000000000000000d+00*HZ5(0,0,-1,-1,-1)
      HZ5(-1,0,0,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ3(0,0,-1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ2(0,-1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ3(0,0,-1)
     &  - 3.0000000000000000d+00*HZ5(0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HZ5(0,0,0,-1,-1)
      HZ5(-1,0,0,0,-1) = 
     &  + HZ1( -1)*HZ4(0,0,0,-1)
     &  - HZ2(0, -1)*HZ3(0,0,-1)
     &  + 2.0000000000000000d+00*HZ5(0,0,-1,0,-1)
     &  + 4.0000000000000000d+00*HZ5(0,0,0,-1,-1)
      HZ5(-1,0,0,0,0) = 
     &  + 4.1666666666666666d-02*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)
     &  - 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ2(0,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(0,0,-1)
     &  - HZ1(0) *HZ4(0,0,0,-1)
     &  + HZ5(0,0,0,0, -1)
      HZ5(0,-1,-1,-1,0) = 
     &  + HZ1(0) *HZ4(0,-1,-1,-1)
     &  - HZ2(0, -1)*HZ3(0,-1,-1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,0,-1,-1)
     &  + 4.0000000000000000d+00*HZ5(0,0,-1,-1,-1)
      HZ5(0,-1,-1,0,-1) = 
     &  + HZ2(0, -1)*HZ3(0,-1,-1)
     &  - 3.0000000000000000d+00*HZ5(0,-1,0,-1,-1)
     &  - 6.0000000000000000d+00*HZ5(0,0,-1,-1,-1)
      HZ5(0,-1,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(0,-1,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ2(0,-1)*HZ2(0,-1)
     &  + HZ2(0, -1)*HZ3(0,0,-1)
     &  - HZ5(0,0, -1,0,-1)
     &  - 3.0000000000000000d+00*HZ5(0,0,0,-1,-1)
      HZ5(0,-1,0,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ2(0,-1)*HZ2(0,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ3(0,0,-1)
     &  + 4.0000000000000000d+00*HZ5(0,0,-1,0,-1)
     &  + 1.2000000000000000d+01*HZ5(0,0,0,-1,-1)
      HZ5(0,-1,0,0,-1) = 
     &  + HZ2(0, -1)*HZ3(0,0,-1)
     &  - 3.0000000000000000d+00*HZ5(0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HZ5(0,0,0,-1,-1)
      HZ5(0,-1,0,0,0) = 
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ2(0,-1)
     &  - HZ1(0) *HZ1(0)*HZ3(0,0,-1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ4(0,0,0,-1)
     &  - 4.0000000000000000d+00*HZ5(0,0,0,0,-1)
      HZ5(0,0,-1,-1,0) = 
     &  + HZ1(0) *HZ4(0,0,-1,-1)
     &  - HZ5(0,0, -1,0,-1)
     &  - 3.0000000000000000d+00*HZ5(0,0,0,-1,-1)
      HZ5(0,0,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(0,0,-1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ4(0,0,0,-1)
     &  + 6.0000000000000000d+00*HZ5(0,0,0,0,-1)
      HZ5(0,0,0,-1,0) = 
     &  + HZ1(0) *HZ4(0,0,0,-1)
     &  - 4.0000000000000000d+00*HZ5(0,0,0,0,-1)
      endif

      if ( (n1.eq.-1).and.(n2.eq.1) ) then 
      HZ5(-1,-1,-1,0,1) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ2(0,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ3(-1,0,1)
     &  + HZ1( -1)*HZ4(-1,-1,0,1)
     &  - HZ5(0, -1,-1,-1,1)
     &  - HZ5(0, -1,-1,1,-1)
     &  - HZ5(0, -1,1,-1,-1)
     &  - HZ5(0,1, -1,-1,-1)
      HZ5(-1,-1,-1,1,-1) = 
     &  + HZ1( -1)*HZ4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HZ5(-1,-1,-1,-1,1)
      HZ5(-1,-1,-1,1,0) = 
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ3(-1,0,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ4(-1,-1,0,1)
     &  - HZ1( -1)*HZ4(-1,0,-1,1)
     &  - HZ1( -1)*HZ4(0,-1,-1,1)
     &  + HZ1(0) *HZ4(-1,-1,-1,1)
     &  + HZ5(0,1, -1,-1,-1)
      HZ5(-1,-1,0,-1,1) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ3(0,-1,1)
     &  + HZ1( -1)*HZ4(-1,0,-1,1)
     &  + 3.0000000000000000d+00*HZ5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,-1,1,-1)
     &  + HZ5(0, -1,1,-1,-1)
      HZ5(-1,-1,0,0,1) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ3(0,0,1)
     &  + HZ1( -1)*HZ4(-1,0,0,1)
     &  + HZ5(0, -1,-1,0,1)
     &  + HZ5(0, -1,0,-1,1)
     &  + HZ5(0, -1,0,1,-1)
     &  + HZ5(0,0, -1,-1,1)
     &  + HZ5(0,0, -1,1,-1)
     &  + HZ5(0,0,1, -1,-1)
      HZ5(-1,-1,0,1,-1) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ2(0,1)
     &  + 1.5000000000000000d+00*HZ1(-1)*HZ1(-1)*HZ3(-1,0,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ4(-1,-1,0,1)
     &  - HZ1( -1)*HZ4(-1,0,-1,1)
     &  + HZ5(0, -1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ5(0,1,-1,-1,-1)
      HZ5(-1,-1,0,1,0) = 
     &  + HZ1( -1)*HZ1(-1)*HZ3(0,0,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ4(-1,0,0,1)
     &  - HZ1( -1)*HZ4(0,-1,0,1)
     &  + HZ1(0) *HZ4(-1,-1,0,1)
     &  - HZ5(0, -1,-1,0,1)
     &  - HZ5(0, -1,0,-1,1)
     &  - HZ5(0, -1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ5(0,0,1,-1,-1)
      HZ5(-1,-1,0,1,1) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ3(0,1,1)
     &  + HZ1( -1)*HZ4(-1,0,1,1)
     &  + HZ5(0, -1,-1,1,1)
     &  + HZ5(0, -1,1,-1,1)
     &  + HZ5(0, -1,1,1,-1)
     &  + HZ5(0,1, -1,-1,1)
     &  + HZ5(0,1, -1,1,-1)
     &  + HZ5(0,1,1, -1,-1)
      HZ5(-1,-1,1,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ3(-1,-1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ4(-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HZ5(-1,-1,-1,-1,1)
      HZ5(-1,-1,1,-1,0) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ3(0,-1,1)
     &  + HZ1( -1)*HZ1(0)*HZ3(-1,-1,1)
     &  + HZ1( -1)*HZ4(-1,0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ4(0,-1,-1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ4(-1,-1,-1,1)
     &  - HZ2(0, -1)*HZ3(-1,-1,1)
     &  + HZ5(0, -1,1,-1,-1)
      HZ5(-1,-1,1,0,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ2(0,1)
     &  - 1.5000000000000000d+00*HZ1(-1)*HZ1(-1)*HZ3(-1,0,1)
     &  - HZ1( -1)*HZ1(-1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ4(-1,-1,0,1)
     &  + HZ1( -1)*HZ4(-1,0,-1,1)
     &  + HZ2(0, -1)*HZ3(-1,-1,1)
     &  - HZ5(0, -1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ5(0,1,-1,-1,-1)
      HZ5(-1,-1,1,0,0) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ3(0,0,1)
     &  + HZ1( -1)*HZ4(-1,0,0,1)
     &  + HZ1( -1)*HZ4(0,-1,0,1)
     &  + HZ1( -1)*HZ4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(-1,-1,1)
     &  - HZ1(0) *HZ4(-1,-1,0,1)
     &  - HZ1(0) *HZ4(-1,0,-1,1)
     &  - HZ1(0) *HZ4(0,-1,-1,1)
     &  + HZ5(0,0,1, -1,-1)
      HZ5(-1,-1,1,0,1) = 
     &  + HZ1( -1)*HZ1(-1)*HZ3(0,1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ4(-1,0,1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ4(0,-1,1,1)
     &  - HZ1( -1)*HZ4(0,1,-1,1)
     &  + HZ2(0,1) *HZ3(-1,-1,1)
     &  - HZ5(0,1, -1,-1,1)
     &  - HZ5(0,1, -1,1,-1)
     &  - 2.0000000000000000d+00*HZ5(0,1,1,-1,-1)
      HZ5(-1,-1,1,1,-1) = 
     &  + HZ1( -1)*HZ4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ5(-1,-1,-1,1,1)
     &  - HZ5( -1,-1,1,-1,1)
      HZ5(-1,-1,1,1,0) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ3(0,1,1)
     &  + HZ1( -1)*HZ4(-1,0,1,1)
     &  + HZ1( -1)*HZ4(0,-1,1,1)
     &  + HZ1( -1)*HZ4(0,1,-1,1)
     &  + HZ1(0) *HZ4(-1,-1,1,1)
     &  - HZ2(0,1) *HZ3(-1,-1,1)
     &  + HZ5(0,1,1, -1,-1)
      HZ5(-1,0,-1,-1,1) = 
     &  + HZ1( -1)*HZ4(0,-1,-1,1)
     &  - 3.0000000000000000d+00*HZ5(0,-1,-1,-1,1)
     &  - HZ5(0, -1,-1,1,-1)
      HZ5(-1,0,-1,0,1) = 
     &  + HZ1( -1)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ5(0,-1,-1,0,1)
     &  - HZ5(0, -1,0,-1,1)
     &  - HZ5(0, -1,0,1,-1)
      HZ5(-1,0,-1,1,-1) = 
     &  + HZ1( -1)*HZ1(-1)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ4(-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ5(0,-1,1,-1,-1)
      HZ5(-1,0,-1,1,0) = 
     &  - HZ1( -1)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ4(0,0,-1,1)
     &  + HZ1(0) *HZ4(-1,0,-1,1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,0,-1,1)
     &  + HZ5(0, -1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ5(0,0,-1,1,-1)
      HZ5(-1,0,-1,1,1) = 
     &  + HZ1( -1)*HZ4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ5(0,-1,-1,1,1)
     &  - HZ5(0, -1,1,-1,1)
     &  - HZ5(0, -1,1,1,-1)
      HZ5(-1,0,0,-1,1) = 
     &  + HZ1( -1)*HZ4(0,0,-1,1)
     &  - HZ5(0, -1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,0,-1,-1,1)
     &  - HZ5(0,0, -1,1,-1)
      HZ5(-1,0,0,0,1) = 
     &  + HZ1( -1)*HZ4(0,0,0,1)
     &  - HZ2(0, -1)*HZ3(0,0,1)
     &  + HZ5(0,0, -1,0,1)
     &  + 2.0000000000000000d+00*HZ5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ5(0,0,0,1,-1)
     &  + HZ5(0,0,1,0, -1)
      HZ5(-1,0,0,1,-1) = 
     &  + HZ1( -1)*HZ1(-1)*HZ3(0,0,1)
     &  - HZ1( -1)*HZ4(-1,0,0,1)
     &  - HZ1( -1)*HZ4(0,-1,0,1)
     &  - HZ1( -1)*HZ4(0,0,-1,1)
     &  - HZ5(0, -1,0,1,-1)
     &  - HZ5(0,0, -1,1,-1)
     &  - 2.0000000000000000d+00*HZ5(0,0,1,-1,-1)
      HZ5(-1,0,0,1,0) = 
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ4(0,0,0,1)
     &  + HZ1(0) *HZ4(-1,0,0,1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ3(0,0,1)
     &  - HZ5(0,0, -1,0,1)
     &  - 3.0000000000000000d+00*HZ5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ5(0,0,1,0,-1)
      HZ5(-1,0,0,1,1) = 
     &  + HZ1( -1)*HZ4(0,0,1,1)
     &  - HZ5(0, -1,0,1,1)
     &  - HZ5(0,0, -1,1,1)
     &  - HZ5(0,0,1, -1,1)
     &  - HZ5(0,0,1,1, -1)
      HZ5(-1,0,1,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ2(0,1)
     &  - HZ1( -1)*HZ1(-1)*HZ3(-1,0,1)
     &  - HZ1( -1)*HZ1(-1)*HZ3(0,-1,1)
     &  + HZ1( -1)*HZ4(-1,-1,0,1)
     &  + HZ1( -1)*HZ4(-1,0,-1,1)
     &  + HZ1( -1)*HZ4(0,-1,-1,1)
     &  - HZ5(0, -1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ5(0,1,-1,-1,-1)
      HZ5(-1,0,1,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ3(-1,0,1)
     &  + HZ1( -1)*HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ4(-1,-1,0,1)
     &  - HZ1(0) *HZ4(-1,0,-1,1)
     &  - HZ2(0, -1)*HZ3(-1,0,1)
     &  - 2.0000000000000000d+00*HZ5(0,-1,0,-1,1)
     &  - HZ5(0, -1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,0,-1,1,-1)
      HZ5(-1,0,1,-1,1) = 
     &  + HZ1( -1)*HZ4(0,1,-1,1)
     &  - HZ5(0, -1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,1,-1,-1,1)
     &  - HZ5(0,1, -1,1,-1)
      HZ5(-1,0,1,0,-1) = 
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(-1)*HZ3(0,0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ4(-1,0,0,1)
     &  + HZ1( -1)*HZ4(0,-1,0,1)
     &  + HZ2(0, -1)*HZ3(-1,0,1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ5(0,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HZ5(0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HZ5(0,0,1,-1,-1)
      HZ5(-1,0,1,0,0) = 
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ4(0,0,0,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(-1,0,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ4(-1,0,0,1)
     &  - HZ1(0) *HZ4(0,-1,0,1)
     &  - HZ2(0, -1)*HZ3(0,0,1)
     &  + HZ5(0,0,1,0, -1)
      HZ5(-1,0,1,0,1) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ2(0,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ4(0,0,1,1)
     &  + HZ2(0,1) *HZ3(-1,0,1)
     &  + HZ2(0,1) *HZ3(0,-1,1)
     &  - HZ5(0, -1,1,0,1)
     &  + 2.0000000000000000d+00*HZ5(0,0,1,-1,1)
     &  + 4.0000000000000000d+00*HZ5(0,0,1,1,-1)
     &  + HZ5(0,1,0,1, -1)
      HZ5(-1,0,1,1,-1) = 
     &  + HZ1( -1)*HZ1(-1)*HZ3(0,1,1)
     &  - HZ1( -1)*HZ4(-1,0,1,1)
     &  - HZ1( -1)*HZ4(0,-1,1,1)
     &  - HZ1( -1)*HZ4(0,1,-1,1)
     &  - HZ5(0, -1,1,1,-1)
     &  - HZ5(0,1, -1,1,-1)
     &  - 2.0000000000000000d+00*HZ5(0,1,1,-1,-1)
      HZ5(-1,0,1,1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ2(0,1)*HZ2(0,1)
     &  + HZ1(0) *HZ4(-1,0,1,1)
     &  - HZ2(0,1) *HZ3(-1,0,1)
     &  - HZ2(0,1) *HZ3(0,-1,1)
     &  + HZ5(0, -1,0,1,1)
     &  + HZ5(0, -1,1,0,1)
     &  + 2.0000000000000000d+00*HZ5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ5(0,0,1,1,-1)
     &  - HZ5(0,1,0,1, -1)
      HZ5(-1,0,1,1,1) = 
     &  + HZ1( -1)*HZ4(0,1,1,1)
     &  - HZ5(0, -1,1,1,1)
     &  - HZ5(0,1, -1,1,1)
     &  - HZ5(0,1,1, -1,1)
     &  - HZ5(0,1,1,1, -1)
      HZ5(-1,1,-1,-1,-1) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ2(-1,1)
     &  - HZ1( -1)*HZ1(-1)*HZ3(-1,-1,1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HZ5(-1,-1,-1,-1,1)
      HZ5(-1,1,-1,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ2(-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ3(-1,-1,1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ2(0,-1)
     &  - HZ1( -1)*HZ4(0,-1,-1,1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ4(-1,-1,-1,1)
     &  + HZ2( -1,1)*HZ3(0,-1,-1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ3(-1,-1,1)
     &  + HZ5(0, -1,-1,1,-1)
      HZ5(-1,1,-1,-1,1) = 
     &  + HZ2( -1,1)*HZ3(-1,-1,1)
     &  - 6.0000000000000000d+00*HZ5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ5(-1,-1,1,-1,1)
      HZ5(-1,1,-1,0,-1) = 
     &  + HZ1( -1)*HZ1(-1)*HZ3(0,-1,1)
     &  + HZ1( -1)*HZ2(-1,1)*HZ2(0,-1)
     &  - HZ1( -1)*HZ4(-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ3(0,-1,-1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ3(-1,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ5(0,-1,1,-1,-1)
      HZ5(-1,1,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ2(-1,1)
     &  - HZ1( -1)*HZ4(0,0,-1,1)
     &  - HZ1(0) *HZ1(0)*HZ3(-1,-1,1)
     &  - HZ1(0) *HZ2(-1,1)*HZ2(0,-1)
     &  + HZ1(0) *HZ4(-1,0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ4(0,-1,-1,1)
     &  + HZ2( -1,1)*HZ3(0,0,-1)
     &  + HZ5(0,0, -1,1,-1)
      HZ5(-1,1,-1,0,1) = 
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ4(0,-1,1,1)
     &  + HZ1( -1)*HZ4(0,1,-1,1)
     &  + HZ2( -1,1)*HZ3(-1,0,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ3(-1,-1,1)
     &  - HZ5(0, -1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,-1,1,1,-1)
     &  - HZ5(0,1, -1,1,-1)
      HZ5(-1,1,-1,1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ2(-1,1)*HZ2(-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ3(-1,-1,1)
     &  + 1.2000000000000000d+01*HZ5(-1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HZ5(-1,-1,1,-1,1)
      HZ5(-1,1,-1,1,0) = 
     &  - HZ1( -1)*HZ4(0,1,-1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ2(-1,1)*HZ2(-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ4(-1,-1,1,1)
     &  - HZ2( -1,1)*HZ3(-1,0,1)
     &  - HZ2( -1,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ3(-1,-1,1)
     &  + HZ5(0,1, -1,1,-1)
      HZ5(-1,1,0,-1,-1) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ2(0,1)
     &  + HZ1( -1)*HZ1(-1)*HZ3(-1,0,1)
     &  - HZ1( -1)*HZ4(-1,-1,0,1)
     &  + HZ2( -1,1)*HZ3(0,-1,-1)
     &  + HZ5(0, -1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ5(0,1,-1,-1,-1)
      HZ5(-1,1,0,-1,0) = 
     &  - HZ1( -1)*HZ1(0)*HZ3(-1,0,1)
     &  - HZ1( -1)*HZ1(0)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ4(0,-1,0,1)
     &  + HZ1(0) *HZ2(-1,1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ4(-1,-1,0,1)
     &  + HZ1(0) *HZ4(-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ3(0,0,-1)
     &  + HZ2(0, -1)*HZ3(-1,0,1)
     &  + HZ2(0, -1)*HZ3(0,-1,1)
     &  + HZ5(0, -1,0,1,-1)
      HZ5(-1,1,0,-1,1) = 
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ4(0,-1,1,1)
     &  - HZ1( -1)*HZ4(0,1,-1,1)
     &  + HZ2( -1,1)*HZ3(0,-1,1)
     &  + HZ5(0, -1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ5(0,1,-1,-1,1)
     &  + HZ5(0,1, -1,1,-1)
      HZ5(-1,1,0,0,-1) = 
     &  + HZ1( -1)*HZ1(-1)*HZ3(0,0,1)
     &  - HZ1( -1)*HZ4(-1,0,0,1)
     &  + HZ2( -1,1)*HZ3(0,0,-1)
     &  - HZ2(0, -1)*HZ3(-1,0,1)
     &  - HZ2(0, -1)*HZ3(0,-1,1)
     &  - HZ5(0, -1,0,1,-1)
     &  - HZ5(0,0, -1,1,-1)
     &  - 2.0000000000000000d+00*HZ5(0,0,1,-1,-1)
      HZ5(-1,1,0,0,0) = 
     &  - HZ1( -1)*HZ4(0,0,0,1)
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ2(-1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(-1,0,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(0,-1,1)
     &  + HZ1(0) *HZ4(-1,0,0,1)
     &  + HZ1(0) *HZ4(0,-1,0,1)
     &  + HZ1(0) *HZ4(0,0,-1,1)
     &  + HZ5(0,0,0,1, -1)
      HZ5(-1,1,0,0,1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ2(0,1)*HZ2(0,1)
     &  + HZ2( -1,1)*HZ3(0,0,1)
     &  - HZ2(0,1) *HZ3(-1,0,1)
     &  - HZ2(0,1) *HZ3(0,-1,1)
     &  - HZ5(0,0,1, -1,1)
     &  - 2.0000000000000000d+00*HZ5(0,0,1,1,-1)
     &  - HZ5(0,1,0,1, -1)
      HZ5(-1,1,0,1,-1) = 
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(-1)*HZ3(0,1,1)
     &  + HZ1( -1)*HZ2(-1,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ4(-1,0,1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ4(0,-1,1,1)
     &  + HZ1( -1)*HZ4(0,1,-1,1)
     &  - HZ2( -1,1)*HZ3(-1,0,1)
     &  - HZ2( -1,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ5(0,1,-1,1,-1)
     &  + 4.0000000000000000d+00*HZ5(0,1,1,-1,-1)
      HZ5(-1,1,0,1,0) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ2(0,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ4(0,0,1,1)
     &  + HZ1(0) *HZ2(-1,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ4(-1,0,1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ4(0,-1,1,1)
     &  - HZ1(0) *HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ3(0,0,1)
     &  + HZ2(0,1) *HZ3(-1,0,1)
     &  + HZ2(0,1) *HZ3(0,-1,1)
     &  + HZ5(0,1,0,1, -1)
      HZ5(-1,1,0,1,1) = 
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ4(0,1,1,1)
     &  + HZ2( -1,1)*HZ3(0,1,1)
     &  + HZ5(0,1, -1,1,1)
     &  + 2.0000000000000000d+00*HZ5(0,1,1,-1,1)
     &  + 3.0000000000000000d+00*HZ5(0,1,1,1,-1)
      HZ5(-1,1,1,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ3(-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ2(-1,1)*HZ2(-1,1)
     &  + HZ2( -1,1)*HZ3(-1,-1,1)
     &  - 3.0000000000000000d+00*HZ5(-1,-1,-1,1,1)
     &  - HZ5( -1,-1,1,-1,1)
      HZ5(-1,1,1,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ3(-1,1,1)
     &  - HZ1( -1)*HZ4(0,-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ2(-1,1)*HZ2(-1,1)
     &  + HZ2( -1,1)*HZ3(0,-1,1)
     &  - HZ2(0, -1)*HZ3(-1,1,1)
     &  + HZ5(0, -1,1,1,-1)
      HZ5(-1,1,1,-1,1) = 
     &  + HZ2( -1,1)*HZ3(-1,1,1)
     &  - 6.0000000000000000d+00*HZ5(-1,-1,1,1,1)
     &  - 3.0000000000000000d+00*HZ5(-1,1,-1,1,1)
      HZ5(-1,1,1,0,-1) = 
     &  + HZ1( -1)*HZ1(-1)*HZ3(0,1,1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ2(0,1)
     &  - HZ1( -1)*HZ4(-1,0,1,1)
     &  + HZ2( -1,1)*HZ3(-1,0,1)
     &  + HZ2(0, -1)*HZ3(-1,1,1)
     &  - HZ5(0, -1,1,1,-1)
     &  - HZ5(0,1, -1,1,-1)
     &  - 2.0000000000000000d+00*HZ5(0,1,1,-1,-1)
      HZ5(-1,1,1,0,0) = 
     &  - HZ1( -1)*HZ4(0,0,1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(-1,1,1)
     &  - HZ1(0) *HZ2(-1,1)*HZ2(0,1)
     &  + HZ1(0) *HZ4(-1,0,1,1)
     &  + HZ1(0) *HZ4(0,-1,1,1)
     &  + HZ1(0) *HZ4(0,1,-1,1)
     &  + HZ2( -1,1)*HZ3(0,0,1)
     &  + HZ5(0,0,1,1, -1)
      HZ5(-1,1,1,0,1) = 
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ4(0,1,1,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ3(0,1,1)
     &  + HZ2(0,1) *HZ3(-1,1,1)
     &  - HZ5(0,1,1, -1,1)
     &  - 3.0000000000000000d+00*HZ5(0,1,1,1,-1)
      HZ5(-1,1,1,1,-1) = 
     &  + HZ1( -1)*HZ4(-1,1,1,1)
     &  - HZ2( -1,1)*HZ3(-1,1,1)
     &  + 4.0000000000000000d+00*HZ5(-1,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ5(-1,1,-1,1,1)
      HZ5(-1,1,1,1,0) = 
     &  - HZ1( -1)*HZ4(0,1,1,1)
     &  + HZ1(0) *HZ4(-1,1,1,1)
     &  + HZ2( -1,1)*HZ3(0,1,1)
     &  - HZ2(0,1) *HZ3(-1,1,1)
     &  + HZ5(0,1,1,1, -1)
      HZ5(0,-1,-1,1,0) = 
     &  + HZ1(0) *HZ4(0,-1,-1,1)
     &  - HZ5(0, -1,-1,0,1)
     &  - HZ5(0, -1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,0,-1,-1,1)
      HZ5(0,-1,0,0,1) = 
     &  + HZ2(0, -1)*HZ3(0,0,1)
     &  - 2.0000000000000000d+00*HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ5(0,0,0,1,-1)
     &  - HZ5(0,0,1,0, -1)
      HZ5(0,-1,0,1,0) = 
     &  + HZ1(0) *HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ3(0,0,1)
     &  + 2.0000000000000000d+00*HZ5(0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HZ5(0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HZ5(0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HZ5(0,0,1,0,-1)
      HZ5(0,-1,1,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ3(0,-1,1)
     &  - HZ1(0) *HZ4(-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ4(0,-1,-1,1)
     &  - HZ2(0, -1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,0,-1,1)
     &  + 4.0000000000000000d+00*HZ5(0,0,-1,-1,1)
      HZ5(0,-1,1,0,-1) = 
     &  + HZ2(0, -1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,-1,0,-1,1)
     &  - HZ5(0, -1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,0,-1,1,-1)
      HZ5(0,-1,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(0,-1,1)
     &  - HZ1(0) *HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ4(0,0,-1,1)
     &  + HZ2(0, -1)*HZ3(0,0,1)
     &  - 3.0000000000000000d+00*HZ5(0,0,0,1,-1)
     &  - HZ5(0,0,1,0, -1)
      HZ5(0,-1,1,1,0) = 
     &  + HZ1(0) *HZ4(0,-1,1,1)
     &  - HZ5(0, -1,0,1,1)
     &  - HZ5(0, -1,1,0,1)
     &  - 2.0000000000000000d+00*HZ5(0,0,-1,1,1)
      HZ5(0,0,-1,1,0) = 
     &  + HZ1(0) *HZ4(0,0,-1,1)
     &  - HZ5(0,0, -1,0,1)
     &  - 3.0000000000000000d+00*HZ5(0,0,0,-1,1)
      HZ5(0,0,1,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ3(0,0,1)
     &  - HZ1(0) *HZ4(-1,0,0,1)
     &  - HZ1(0) *HZ4(0,-1,0,1)
     &  - HZ1(0) *HZ4(0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ5(0,0,0,1,-1)
     &  - HZ5(0,0,1,0, -1)
      HZ5(0,1,-1,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ2(0,1)
     &  - HZ1( -1)*HZ1(0)*HZ3(-1,0,1)
     &  - HZ1( -1)*HZ1(0)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ2(0,1)
     &  + HZ1(0) *HZ4(-1,-1,0,1)
     &  + HZ1(0) *HZ4(-1,0,-1,1)
     &  + HZ1(0) *HZ4(0,-1,-1,1)
     &  + HZ2(0, -1)*HZ3(-1,0,1)
     &  + HZ2(0, -1)*HZ3(0,-1,1)
     &  + HZ2(0,1) *HZ3(0,-1,-1)
     &  - HZ5(0, -1,-1,0,1)
     &  - HZ5(0, -1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,0,-1,-1,1)
      HZ5(0,1,-1,0,-1) = 
     &  + HZ1( -1)*HZ2(0,-1)*HZ2(0,1)
     &  - HZ2(0, -1)*HZ3(-1,0,1)
     &  - HZ2(0, -1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ3(0,-1,-1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,0,-1,1)
     &  + HZ5(0, -1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ5(0,0,-1,1,-1)
      HZ5(0,1,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ2(0,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(-1,0,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(0,-1,1)
     &  - HZ1(0) *HZ2(0,-1)*HZ2(0,1)
     &  + HZ1(0) *HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ4(0,0,-1,1)
     &  + HZ2(0,1) *HZ3(0,0,-1)
     &  - HZ5(0,0, -1,0,1)
     &  - 3.0000000000000000d+00*HZ5(0,0,0,-1,1)
      HZ5(0,1,-1,0,1) = 
     &  + HZ1( -1)*HZ2(0,1)*HZ2(0,1)
     &  - HZ2(0,1) *HZ3(-1,0,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,0,1,1)
     &  + HZ5(0, -1,1,0,1)
     &  + 4.0000000000000000d+00*HZ5(0,0,-1,1,1)
     &  - 4.0000000000000000d+00*HZ5(0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ5(0,1,0,1,-1)
      HZ5(0,1,-1,1,0) = 
     &  - HZ1( -1)*HZ2(0,1)*HZ2(0,1)
     &  + HZ1(0) *HZ4(0,1,-1,1)
     &  + HZ2(0,1) *HZ3(-1,0,1)
     &  + HZ2(0,1) *HZ3(0,-1,1)
     &  + 4.0000000000000000d+00*HZ5(0,0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ5(0,1,0,1,-1)
      HZ5(0,1,0,-1,-1) = 
     &  + HZ2(0,1) *HZ3(0,-1,-1)
     &  - HZ5(0, -1,-1,0,1)
     &  - HZ5(0, -1,0,-1,1)
     &  - HZ5(0, -1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ5(0,0,1,-1,-1)
      HZ5(0,1,0,-1,0) = 
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ3(0,0,1)
     &  + HZ1(0) *HZ2(0,-1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ4(-1,0,0,1)
     &  + HZ1(0) *HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ3(0,0,-1)
     &  + 2.0000000000000000d+00*HZ5(0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HZ5(0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HZ5(0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HZ5(0,0,1,0,-1)
      HZ5(0,1,0,-1,1) = 
     &  + HZ2(0,1) *HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,-1,0,1,1)
     &  - HZ5(0, -1,1,0,1)
     &  - 4.0000000000000000d+00*HZ5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ5(0,0,1,-1,1)
      HZ5(0,1,0,0,-1) = 
     &  + HZ2(0,1) *HZ3(0,0,-1)
     &  - HZ5(0,0, -1,0,1)
     &  - 3.0000000000000000d+00*HZ5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ5(0,0,1,0,-1)
      HZ5(0,1,1,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ3(0,1,1)
     &  - HZ1(0) *HZ4(-1,0,1,1)
     &  - HZ1(0) *HZ4(0,-1,1,1)
     &  - HZ1(0) *HZ4(0,1,-1,1)
     &  - HZ2(0, -1)*HZ3(0,1,1)
     &  + HZ2(0,1) *HZ3(0,-1,1)
     &  - HZ5(0, -1,0,1,1)
     &  - HZ5(0, -1,1,0,1)
     &  - 2.0000000000000000d+00*HZ5(0,0,-1,1,1)
      HZ5(0,1,1,0,-1) = 
     &  + HZ2(0, -1)*HZ3(0,1,1)
     &  - HZ2(0,1) *HZ3(0,-1,1)
     &  + HZ5(0, -1,0,1,1)
     &  + HZ5(0, -1,1,0,1)
     &  + 2.0000000000000000d+00*HZ5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ5(0,0,1,1,-1)
     &  - HZ5(0,1,0,1, -1)
      HZ5(1,-1,-1,-1,-1) = 
     &  + 4.1666666666666666d-02*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(1)
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ2(-1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ3(-1,-1,1)
     &  - HZ1( -1)*HZ4(-1,-1,-1,1)
     &  + HZ5( -1,-1,-1,-1,1)
      HZ5(1,-1,-1,-1,0) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ2(-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ2(0,-1)
     &  + HZ1( -1)*HZ1(0)*HZ3(-1,-1,1)
     &  + HZ1( -1)*HZ1(1)*HZ3(0,-1,-1)
     &  + HZ1( -1)*HZ2(-1,1)*HZ2(0,-1)
     &  - HZ1(0) *HZ4(-1,-1,-1,1)
     &  - HZ1(1) *HZ4(0,-1,-1,-1)
     &  - HZ2( -1,1)*HZ3(0,-1,-1)
     &  - HZ2(0, -1)*HZ3(-1,-1,1)
     &  + HZ5(0, -1,-1,-1,1)
      HZ5(1,-1,-1,-1,1) = 
     &  + HZ1(1) *HZ4(-1,-1,-1,1)
     &  - HZ2( -1,1)*HZ3(-1,-1,1)
     &  + 4.0000000000000000d+00*HZ5(-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ5(-1,-1,1,-1,1)
      HZ5(1,-1,-1,0,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ2(0,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ3(0,-1,-1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ2(0,-1)
     &  + 3.0000000000000000d+00*HZ1(1)*HZ4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ3(0,-1,-1)
     &  + HZ2(0, -1)*HZ3(-1,-1,1)
     &  - 3.0000000000000000d+00*HZ5(0,-1,-1,-1,1)
     &  - HZ5(0, -1,-1,1,-1)
      HZ5(1,-1,-1,0,0) = 
     &  + 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ2(-1,1)
     &  - HZ1( -1)*HZ1(0)*HZ1(1)*HZ2(0,-1)
     &  + HZ1( -1)*HZ1(1)*HZ3(0,0,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(-1,-1,1)
     &  + HZ1(0) *HZ1(1)*HZ3(0,-1,-1)
     &  + HZ1(0) *HZ2(-1,1)*HZ2(0,-1)
     &  - HZ1(0) *HZ4(0,-1,-1,1)
     &  - HZ1(1) *HZ4(0,0,-1,-1)
     &  - HZ2( -1,1)*HZ3(0,0,-1)
     &  + HZ5(0,0, -1,-1,1)
      HZ5(1,-1,-1,0,1) = 
     &  + HZ1(1) *HZ4(-1,-1,0,1)
     &  - HZ2( -1,1)*HZ3(-1,0,1)
     &  + HZ2(0,1) *HZ3(-1,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,-1,-1,1,1)
     &  - HZ5(0, -1,1,-1,1)
     &  - HZ5(0,1, -1,-1,1)
      HZ5(1,-1,-1,1,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ3(-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ2(-1,1)*HZ2(-1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ3(-1,-1,1)
     &  - 6.0000000000000000d+00*HZ5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ5(-1,-1,1,-1,1)
      HZ5(1,-1,-1,1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ3(-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ2(-1,1)*HZ2(-1,1)
     &  - HZ1(1) *HZ4(-1,-1,0,1)
     &  - HZ1(1) *HZ4(-1,0,-1,1)
     &  - HZ1(1) *HZ4(0,-1,-1,1)
     &  + HZ2( -1,1)*HZ3(-1,0,1)
     &  + HZ2( -1,1)*HZ3(0,-1,1)
     &  - HZ2(0,1) *HZ3(-1,-1,1)
     &  + HZ5(0,1, -1,-1,1)
      HZ5(1,-1,-1,1,1) = 
     &  + HZ1(1) *HZ4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ5(-1,-1,1,1,1)
     &  - HZ5( -1,1,-1,1,1)
      HZ5(1,-1,0,-1,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ3(0,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ4(0,-1,-1,-1)
     &  - HZ2( -1,1)*HZ3(0,-1,-1)
     &  + 3.0000000000000000d+00*HZ5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,-1,1,-1)
     &  + HZ5(0, -1,1,-1,-1)
      HZ5(1,-1,0,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ2(0,-1)
     &  + HZ1( -1)*HZ1(0)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ3(0,0,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ3(0,-1,-1)
     &  - HZ1(0) *HZ2(-1,1)*HZ2(0,-1)
     &  - HZ1(0) *HZ4(-1,0,-1,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ2(0,-1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ3(0,0,-1)
     &  - HZ2(0, -1)*HZ3(0,-1,1)
     &  + HZ5(0, -1,0,-1,1)
      HZ5(1,-1,0,-1,1) = 
     &  + HZ1(1) *HZ4(-1,0,-1,1)
     &  - HZ2( -1,1)*HZ3(0,-1,1)
     &  + 4.0000000000000000d+00*HZ5(0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,1,-1,1)
      HZ5(1,-1,0,0,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ3(0,0,-1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ2(0,-1)*HZ2(0,-1)
     &  - HZ2( -1,1)*HZ3(0,0,-1)
     &  + HZ2(0, -1)*HZ3(0,-1,1)
     &  - HZ5(0, -1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,0,-1,-1,1)
     &  - HZ5(0,0, -1,1,-1)
      HZ5(1,-1,0,0,0) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(1)
     &  - 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ2(-1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ2(0,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(0,-1,1)
     &  + HZ1(0) *HZ1(1)*HZ3(0,0,-1)
     &  - HZ1(0) *HZ4(0,0,-1,1)
     &  - HZ1(1) *HZ4(0,0,0,-1)
     &  + HZ5(0,0,0, -1,1)
      HZ5(1,-1,0,0,1) = 
     &  + HZ1(1) *HZ4(-1,0,0,1)
     &  - HZ2( -1,1)*HZ3(0,0,1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,0,1,1)
     &  + HZ5(0, -1,1,0,1)
     &  + 2.0000000000000000d+00*HZ5(0,0,-1,1,1)
     &  + HZ5(0,0,1, -1,1)
      HZ5(1,-1,0,1,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ3(-1,0,1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ4(-1,-1,0,1)
     &  - HZ1(1) *HZ4(-1,0,-1,1)
     &  + HZ2( -1,1)*HZ3(-1,0,1)
     &  + HZ2( -1,1)*HZ3(0,-1,1)
     &  + HZ5(0, -1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ5(0,1,-1,-1,1)
     &  + HZ5(0,1, -1,1,-1)
      HZ5(1,-1,0,1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ3(-1,0,1)
     &  - HZ1(0) *HZ2(-1,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ4(0,-1,1,1)
     &  + HZ1(0) *HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ4(-1,0,0,1)
     &  - HZ1(1) *HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ3(0,0,1)
     &  - 2.0000000000000000d+00*HZ5(0,-1,0,1,1)
     &  - HZ5(0, -1,1,0,1)
     &  - 4.0000000000000000d+00*HZ5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ5(0,0,1,-1,1)
      HZ5(1,-1,0,1,1) = 
     &  + HZ1(1) *HZ4(-1,0,1,1)
     &  - HZ2( -1,1)*HZ3(0,1,1)
     &  + 3.0000000000000000d+00*HZ5(0,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ5(0,1,-1,1,1)
     &  + HZ5(0,1,1, -1,1)
      HZ5(1,-1,1,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ2(-1,1)
     &  - HZ1( -1)*HZ1(-1)*HZ3(-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ3(-1,-1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ2(-1,1)*HZ2(-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HZ1(1)*HZ4(-1,-1,-1,1)
     &  - HZ2( -1,1)*HZ3(-1,-1,1)
     &  + HZ5( -1,-1,1,-1,1)
      HZ5(1,-1,1,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ2(-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ3(-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ3(-1,-1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ2(-1,1)*HZ2(-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ4(-1,-1,1,1)
     &  - HZ1(1) *HZ2(-1,1)*HZ2(0,-1)
     &  + HZ1(1) *HZ4(-1,0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ4(0,-1,-1,1)
     &  - HZ2( -1,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ3(-1,1,1)
     &  + HZ5(0, -1,1,-1,1)
      HZ5(1,-1,1,-1,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ2(-1,1)*HZ2(-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ3(-1,1,1)
     &  + 1.2000000000000000d+01*HZ5(-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HZ5(-1,1,-1,1,1)
      HZ5(1,-1,1,0,-1) = 
     &  - HZ1( -1)*HZ1(1)*HZ3(-1,0,1)
     &  - HZ1( -1)*HZ1(1)*HZ3(0,-1,1)
     &  + HZ1( -1)*HZ2(-1,1)*HZ2(0,1)
     &  + HZ1(1) *HZ2(-1,1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ4(-1,-1,0,1)
     &  + HZ1(1) *HZ4(-1,0,-1,1)
     &  - HZ2( -1,1)*HZ3(-1,0,1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ3(-1,1,1)
     &  - HZ5(0, -1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,1,-1,-1,1)
     &  - HZ5(0,1, -1,1,-1)
      HZ5(1,-1,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ2(-1,1)
     &  - HZ1(0) *HZ1(0)*HZ3(-1,1,1)
     &  - HZ1(0) *HZ1(1)*HZ3(-1,0,1)
     &  - HZ1(0) *HZ1(1)*HZ3(0,-1,1)
     &  + HZ1(0) *HZ2(-1,1)*HZ2(0,1)
     &  - HZ1(0) *HZ4(0,1,-1,1)
     &  + HZ1(1) *HZ4(-1,0,0,1)
     &  + HZ1(1) *HZ4(0,-1,0,1)
     &  + HZ1(1) *HZ4(0,0,-1,1)
     &  - HZ2( -1,1)*HZ3(0,0,1)
     &  + HZ5(0,0,1, -1,1)
      HZ5(1,-1,1,0,1) = 
     &  + HZ1(1) *HZ2(-1,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ4(-1,0,1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ4(0,-1,1,1)
     &  - HZ1(1) *HZ4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ3(0,1,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ3(-1,1,1)
     &  - 2.0000000000000000d+00*HZ5(0,1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ5(0,1,1,-1,1)
      HZ5(1,-1,1,1,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ3(-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ4(-1,1,1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ2(-1,1)*HZ2(-1,1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ3(-1,1,1)
     &  - 6.0000000000000000d+00*HZ5(-1,-1,1,1,1)
     &  - 3.0000000000000000d+00*HZ5(-1,1,-1,1,1)
      HZ5(1,-1,1,1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ3(-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ4(-1,1,1,1)
     &  - HZ1(1) *HZ2(-1,1)*HZ2(0,1)
     &  + HZ1(1) *HZ4(-1,0,1,1)
     &  + HZ1(1) *HZ4(0,-1,1,1)
     &  + HZ1(1) *HZ4(0,1,-1,1)
     &  - HZ2( -1,1)*HZ3(0,1,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ3(-1,1,1)
     &  + HZ5(0,1,1, -1,1)
      HZ5(1,-1,1,1,1) = 
     &  + HZ1(1) *HZ4(-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ5(-1,1,1,1,1)
      HZ5(1,0,-1,-1,-1) = 
     &  + HZ1(1) *HZ4(0,-1,-1,-1)
     &  - HZ5(0, -1,-1,-1,1)
     &  - HZ5(0, -1,-1,1,-1)
     &  - HZ5(0, -1,1,-1,-1)
     &  - HZ5(0,1, -1,-1,-1)
      HZ5(1,0,-1,-1,0) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ2(0,1)
     &  + HZ1( -1)*HZ1(0)*HZ3(-1,0,1)
     &  + HZ1( -1)*HZ2(0,-1)*HZ2(0,1)
     &  + HZ1(0) *HZ1(1)*HZ3(0,-1,-1)
     &  - HZ1(0) *HZ4(-1,-1,0,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ2(0,-1)*HZ2(0,-1)
     &  - HZ2(0, -1)*HZ3(-1,0,1)
     &  - HZ2(0,1) *HZ3(0,-1,-1)
     &  + HZ5(0, -1,-1,0,1)
      HZ5(1,0,-1,-1,1) = 
     &  + HZ1(1) *HZ4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,-1,-1,1,1)
     &  - HZ5(0, -1,1,-1,1)
     &  - HZ5(0,1, -1,-1,1)
      HZ5(1,0,-1,0,-1) = 
     &  - HZ1( -1)*HZ2(0,-1)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ2(0,-1)*HZ2(0,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ4(0,0,-1,-1)
     &  + HZ2(0, -1)*HZ3(-1,0,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ3(0,-1,-1)
     &  - 2.0000000000000000d+00*HZ5(0,-1,-1,0,1)
     &  - HZ5(0, -1,0,-1,1)
     &  - HZ5(0, -1,0,1,-1)
      HZ5(1,0,-1,0,0) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ2(0,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(-1,0,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ3(0,0,-1)
     &  + HZ1(0) *HZ2(0,-1)*HZ2(0,1)
     &  - HZ1(0) *HZ4(0,-1,0,1)
     &  + 3.0000000000000000d+00*HZ1(1)*HZ4(0,0,0,-1)
     &  - HZ2(0,1) *HZ3(0,0,-1)
     &  + HZ5(0,0, -1,0,1)
      HZ5(1,0,-1,0,1) = 
     &  - HZ1( -1)*HZ2(0,1)*HZ2(0,1)
     &  + HZ1(1) *HZ4(0,-1,0,1)
     &  + HZ2(0,1) *HZ3(-1,0,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ3(0,-1,1)
     &  - 4.0000000000000000d+00*HZ5(0,-1,0,1,1)
     &  - 2.0000000000000000d+00*HZ5(0,-1,1,0,1)
     &  - 4.0000000000000000d+00*HZ5(0,0,-1,1,1)
     &  + 4.0000000000000000d+00*HZ5(0,0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ5(0,1,0,1,-1)
      HZ5(1,0,-1,1,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ3(0,-1,1)
     &  - HZ1(1) *HZ4(-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ4(0,-1,-1,1)
     &  - HZ5(0, -1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,-1,1,1,-1)
     &  - HZ5(0,1, -1,1,-1)
      HZ5(1,0,-1,1,0) = 
     &  + HZ1( -1)*HZ2(0,1)*HZ2(0,1)
     &  + HZ1(0) *HZ1(1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ4(0,-1,1,1)
     &  - HZ1(0) *HZ4(0,1,-1,1)
     &  - HZ1(1) *HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ4(0,0,-1,1)
     &  - HZ2(0,1) *HZ3(-1,0,1)
     &  - HZ2(0,1) *HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,0,1,1)
     &  + HZ5(0, -1,1,0,1)
     &  + 4.0000000000000000d+00*HZ5(0,0,-1,1,1)
     &  - 4.0000000000000000d+00*HZ5(0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ5(0,1,0,1,-1)
      HZ5(1,0,-1,1,1) = 
     &  + HZ1(1) *HZ4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HZ5(0,-1,1,1,1)
     &  - HZ5(0,1, -1,1,1)
      HZ5(1,0,0,-1,-1) = 
     &  + HZ1(1) *HZ4(0,0,-1,-1)
     &  - HZ2(0,1) *HZ3(0,-1,-1)
     &  + HZ5(0, -1,-1,0,1)
     &  + HZ5(0, -1,0,-1,1)
     &  + HZ5(0, -1,0,1,-1)
     &  + HZ5(0,0, -1,-1,1)
     &  + HZ5(0,0, -1,1,-1)
     &  + HZ5(0,0,1, -1,-1)
      HZ5(1,0,0,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ3(0,0,1)
     &  + HZ1(0) *HZ1(1)*HZ3(0,0,-1)
     &  - HZ1(0) *HZ2(0,-1)*HZ2(0,1)
     &  - HZ1(0) *HZ4(-1,0,0,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ4(0,0,0,-1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ3(0,0,-1)
     &  - 2.0000000000000000d+00*HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ5(0,0,0,1,-1)
     &  - HZ5(0,0,1,0, -1)
      HZ5(1,0,0,-1,1) = 
     &  + HZ1(1) *HZ4(0,0,-1,1)
     &  - HZ2(0,1) *HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ5(0,-1,0,1,1)
     &  + HZ5(0, -1,1,0,1)
     &  + 2.0000000000000000d+00*HZ5(0,0,-1,1,1)
     &  + HZ5(0,0,1, -1,1)
      HZ5(1,0,0,0,-1) = 
     &  + HZ1(1) *HZ4(0,0,0,-1)
     &  - HZ2(0,1) *HZ3(0,0,-1)
     &  + HZ5(0,0, -1,0,1)
     &  + 2.0000000000000000d+00*HZ5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ5(0,0,0,1,-1)
     &  + HZ5(0,0,1,0, -1)
      HZ5(1,0,0,1,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ3(0,0,1)
     &  - HZ1(1) *HZ4(-1,0,0,1)
     &  - HZ1(1) *HZ4(0,-1,0,1)
     &  - HZ1(1) *HZ4(0,0,-1,1)
     &  - HZ5(0,0,1, -1,1)
     &  - 2.0000000000000000d+00*HZ5(0,0,1,1,-1)
     &  - HZ5(0,1,0,1, -1)
      HZ5(1,0,1,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ2(0,1)
     &  - HZ1( -1)*HZ1(1)*HZ3(-1,0,1)
     &  - HZ1( -1)*HZ1(1)*HZ3(0,-1,1)
     &  + HZ1(1) *HZ4(-1,-1,0,1)
     &  + HZ1(1) *HZ4(-1,0,-1,1)
     &  + HZ1(1) *HZ4(0,-1,-1,1)
     &  - HZ5(0,1, -1,-1,1)
     &  - HZ5(0,1, -1,1,-1)
     &  - 2.0000000000000000d+00*HZ5(0,1,1,-1,-1)
      HZ5(1,0,1,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ3(0,1,1)
     &  - HZ1(0) *HZ1(1)*HZ3(-1,0,1)
     &  - HZ1(0) *HZ1(1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ4(-1,0,1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ4(0,-1,1,1)
     &  + HZ1(0) *HZ4(0,1,-1,1)
     &  - HZ1(1) *HZ2(0,-1)*HZ2(0,1)
     &  + HZ1(1) *HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ3(0,1,1)
     &  - HZ2(0,1) *HZ3(0,-1,1)
     &  + HZ5(0, -1,1,0,1)
      HZ5(1,0,1,-1,1) = 
     &  + HZ1(1) *HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ5(0,1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ5(0,1,1,-1,1)
      HZ5(1,0,1,0,-1) = 
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ3(0,0,1)
     &  + HZ1(1) *HZ2(0,-1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ4(-1,0,0,1)
     &  + HZ1(1) *HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ3(0,1,1)
     &  + HZ2(0,1) *HZ3(0,-1,1)
     &  - HZ5(0, -1,1,0,1)
     &  + 2.0000000000000000d+00*HZ5(0,0,1,-1,1)
     &  + 4.0000000000000000d+00*HZ5(0,0,1,1,-1)
     &  + HZ5(0,1,0,1, -1)
      HZ5(1,0,1,1,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ3(0,1,1)
     &  - HZ1(1) *HZ4(-1,0,1,1)
     &  - HZ1(1) *HZ4(0,-1,1,1)
     &  - HZ1(1) *HZ4(0,1,-1,1)
     &  - HZ5(0,1,1, -1,1)
     &  - 3.0000000000000000d+00*HZ5(0,1,1,1,-1)
      HZ5(1,1,-1,-1,-1) = 
     &  + 8.3333333333333333d-02*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ1(1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ2(-1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ3(-1,1,1)
     &  + HZ1( -1)*HZ1(1)*HZ3(-1,-1,1)
     &  - HZ1( -1)*HZ4(-1,-1,1,1)
     &  - HZ1(1) *HZ4(-1,-1,-1,1)
     &  + HZ5( -1,-1,-1,1,1)
      HZ5(1,1,-1,-1,0) = 
     &  + 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(1)*HZ1(1)
     &  - HZ1( -1)*HZ1(0)*HZ1(1)*HZ2(-1,1)
     &  + HZ1( -1)*HZ1(0)*HZ3(-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ2(0,-1)
     &  + HZ1(0) *HZ1(1)*HZ3(-1,-1,1)
     &  - HZ1(0) *HZ4(-1,-1,1,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(0,-1,-1)
     &  + HZ1(1) *HZ2(-1,1)*HZ2(0,-1)
     &  - HZ1(1) *HZ4(0,-1,-1,1)
     &  - HZ2(0, -1)*HZ3(-1,1,1)
     &  + HZ5(0, -1,-1,1,1)
      HZ5(1,1,-1,-1,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ2(-1,1)*HZ2(-1,1)
     &  + HZ2( -1,1)*HZ3(-1,1,1)
     &  - 3.0000000000000000d+00*HZ5(-1,-1,1,1,1)
     &  - HZ5( -1,1,-1,1,1)
      HZ5(1,1,-1,0,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ2(0,-1)
     &  + HZ1( -1)*HZ1(1)*HZ3(0,-1,1)
     &  - HZ1(1) *HZ1(1)*HZ3(0,-1,-1)
     &  - HZ1(1) *HZ2(-1,1)*HZ2(0,-1)
     &  - HZ1(1) *HZ4(-1,0,-1,1)
     &  + HZ2(0, -1)*HZ3(-1,1,1)
     &  - 2.0000000000000000d+00*HZ5(0,-1,-1,1,1)
     &  - HZ5(0, -1,1,-1,1)
     &  - HZ5(0, -1,1,1,-1)
      HZ5(1,1,-1,0,0) = 
     &  + 2.5000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(1)*HZ1(1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ2(-1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ3(-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ2(0,-1)
     &  + HZ1(0) *HZ1(1)*HZ3(0,-1,1)
     &  - HZ1(0) *HZ4(0,-1,1,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(0,0,-1)
     &  - HZ1(1) *HZ4(0,0,-1,1)
     &  + HZ5(0,0, -1,1,1)
      HZ5(1,1,-1,0,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(-1,0,1)
     &  - HZ1(1) *HZ2(-1,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ4(0,-1,1,1)
     &  + HZ1(1) *HZ4(0,1,-1,1)
     &  + HZ2(0,1) *HZ3(-1,1,1)
     &  - 3.0000000000000000d+00*HZ5(0,-1,1,1,1)
     &  - HZ5(0,1, -1,1,1)
      HZ5(1,1,-1,1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ2(-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ3(-1,1,1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ4(-1,1,1,1)
     &  - HZ1(1) *HZ1(1)*HZ3(-1,-1,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ2(-1,1)*HZ2(-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ4(-1,-1,1,1)
     &  - HZ2( -1,1)*HZ3(-1,1,1)
     &  + HZ5( -1,1,-1,1,1)
      HZ5(1,1,-1,1,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ2(-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ3(-1,1,1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ4(-1,1,1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(-1,0,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(0,-1,1)
     &  + HZ1(1) *HZ2(-1,1)*HZ2(0,1)
     &  - HZ1(1) *HZ4(0,1,-1,1)
     &  - HZ2(0,1) *HZ3(-1,1,1)
     &  + HZ5(0,1, -1,1,1)
      HZ5(1,1,-1,1,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ4(-1,1,1,1)
     &  + 6.0000000000000000d+00*HZ5(-1,1,1,1,1)
      HZ5(1,1,0,-1,-1) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ2(0,1)
     &  + HZ1( -1)*HZ1(1)*HZ3(-1,0,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(0,-1,-1)
     &  - HZ1(1) *HZ4(-1,-1,0,1)
     &  + HZ5(0, -1,-1,1,1)
     &  + HZ5(0, -1,1,-1,1)
     &  + HZ5(0, -1,1,1,-1)
     &  + HZ5(0,1, -1,-1,1)
     &  + HZ5(0,1, -1,1,-1)
     &  + HZ5(0,1,1, -1,-1)
      HZ5(1,1,0,-1,0) = 
     &  - HZ1( -1)*HZ1(0)*HZ1(1)*HZ2(0,1)
     &  + HZ1( -1)*HZ1(0)*HZ3(0,1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ2(0,-1)
     &  + HZ1(0) *HZ1(1)*HZ3(-1,0,1)
     &  - HZ1(0) *HZ4(-1,0,1,1)
     &  - HZ1(1) *HZ1(1)*HZ3(0,0,-1)
     &  + HZ1(1) *HZ2(0,-1)*HZ2(0,1)
     &  - HZ1(1) *HZ4(0,-1,0,1)
     &  - HZ2(0, -1)*HZ3(0,1,1)
     &  + HZ5(0, -1,0,1,1)
      HZ5(1,1,0,-1,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ4(0,-1,1,1)
     &  - HZ1(1) *HZ4(0,1,-1,1)
     &  + 3.0000000000000000d+00*HZ5(0,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ5(0,1,-1,1,1)
     &  + HZ5(0,1,1, -1,1)
      HZ5(1,1,0,0,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ3(0,0,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(0,0,-1)
     &  - HZ1(1) *HZ2(0,-1)*HZ2(0,1)
     &  - HZ1(1) *HZ4(-1,0,0,1)
     &  + HZ2(0, -1)*HZ3(0,1,1)
     &  - HZ5(0, -1,0,1,1)
     &  - HZ5(0,0, -1,1,1)
     &  - HZ5(0,0,1, -1,1)
     &  - HZ5(0,0,1,1, -1)
      HZ5(1,1,0,1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ3(0,1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(-1,0,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ4(-1,0,1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ4(0,-1,1,1)
     &  + HZ1(1) *HZ4(0,1,-1,1)
     &  + HZ5(0,1, -1,1,1)
     &  + 2.0000000000000000d+00*HZ5(0,1,1,-1,1)
     &  + 3.0000000000000000d+00*HZ5(0,1,1,1,-1)
      HZ5(1,1,1,-1,-1) = 
     &  + 8.3333333333333333d-02*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ1(1)*HZ1(1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ2(-1,1)
     &  + HZ1( -1)*HZ1(1)*HZ3(-1,1,1)
     &  - HZ1( -1)*HZ4(-1,1,1,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(-1,-1,1)
     &  - HZ1(1) *HZ4(-1,-1,1,1)
     &  + HZ5( -1,-1,1,1,1)
      HZ5(1,1,1,-1,0) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(0)*HZ1(1)*HZ1(1)*HZ1(1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ2(-1,1)
     &  + HZ1(0) *HZ1(1)*HZ3(-1,1,1)
     &  - HZ1(0) *HZ4(-1,1,1,1)
     &  - 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(0,-1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(0,-1,1)
     &  - HZ1(1) *HZ4(0,-1,1,1)
     &  + HZ5(0, -1,1,1,1)
      HZ5(1,1,1,-1,1) = 
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(-1,1)
     &  - HZ1(1) *HZ1(1)*HZ3(-1,1,1)
     &  + 3.0000000000000000d+00*HZ1(1)*HZ4(-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ5(-1,1,1,1,1)
      HZ5(1,1,1,0,-1) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ2(0,1)
     &  + HZ1( -1)*HZ1(1)*HZ3(0,1,1)
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(0,-1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(-1,0,1)
     &  - HZ1(1) *HZ4(-1,0,1,1)
     &  - HZ5(0, -1,1,1,1)
     &  - HZ5(0,1, -1,1,1)
     &  - HZ5(0,1,1, -1,1)
     &  - HZ5(0,1,1,1, -1)
      HZ5(1,1,1,1,-1) = 
     &  + 4.1666666666666666d-02*HZ1(-1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)
     &  - 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(-1,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ3(-1,1,1)
     &  - HZ1(1) *HZ4(-1,1,1,1)
     &  + HZ5( -1,1,1,1,1)
      endif

      return
      end


      subroutine FILLREDHPL6(iflag,HZ1,HZ2,HZ3,HZ4,HZ5,HZ6,n1,n2)
      implicit double precision (a-h,o-z) 
      complex*16 HZ1,HZ2,HZ3,HZ4,HZ5,HZ6 
      dimension HZ1(n1:n2),HZ2(n1:n2,n1:n2),HZ3(n1:n2,n1:n2,n1:n2) 
      dimension HZ4(n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension HZ5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension HZ6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 

** evaluating the expansions 
* 2017-05-03:15:35:21.hpl 
* <- red.out 
* produced by form-to-fortr for gehrt@tintin 

      HZ6(0,0,0,0,0,0) = 
     &  + 1.3888888888888888d-03*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)
      HZ6(0,0,0,0,1,0) = 
     &  + HZ1(0) *HZ5(0,0,0,0,1)
     &  - 5.0000000000000000d+00*HZ6(0,0,0,0,0,1)
      HZ6(0,0,0,0,-1,0) = 
     &  + HZ1(0) *HZ5(0,0,0,0,-1)
     &  - 5.0000000000000000d+00*HZ6(0,0,0,0,0,-1)
      HZ6(0,0,0,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,0,0,1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,0,1)
     &  + 1.0000000000000000d+01*HZ6(0,0,0,0,0,1)
      HZ6(0,0,0,1,1,0) = 
     &  + HZ1(0) *HZ5(0,0,0,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,0,0,1,1)
     &  - HZ6(0,0,0,1,0,1) 
      HZ6(0,0,0,1,-1,0) = 
     &  + HZ1(0) *HZ5(0,0,0,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,0,0,1,-1)
     &  - HZ6(0,0,0,1,0, -1)
      HZ6(0,0,0,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,0,0,-1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,0,-1)
     &  + 1.0000000000000000d+01*HZ6(0,0,0,0,0,-1)
      HZ6(0,0,0,-1,1,0) = 
     &  + HZ1(0) *HZ5(0,0,0,-1,1)
     &  - HZ6(0,0,0, -1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,0,0,-1,1)
      HZ6(0,0,0,-1,-1,0) = 
     &  + HZ1(0) *HZ5(0,0,0,-1,-1)
     &  - HZ6(0,0,0, -1,0,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,0,0,-1,-1)
      HZ6(0,0,1,0,0,0) = 
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,0,1)
     &  - 1.5000000000000000d+00*HZ1(0)*HZ1(0)*HZ4(0,0,0,1)
     &  + 6.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,0,1)
     &  - 1.0000000000000000d+01*HZ6(0,0,0,0,0,1)
      HZ6(0,0,1,0,0,1) = 
     &  + 5.0000000000000000d-01*HZ3(0,0,1)*HZ3(0,0,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,0,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,0,1)
      HZ6(0,0,1,0,0,-1) = 
     &  + HZ3(0,0, -1)*HZ3(0,0,1)
     &  - HZ6(0,0, -1,0,0,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,0,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,0,-1,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,0,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,0,-1)
      HZ6(0,0,1,0,1,0) = 
     &  + HZ1(0) *HZ5(0,0,1,0,1)
     &  - HZ3(0,0,1) *HZ3(0,0,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,0,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,0,1)
      HZ6(0,0,1,0,-1,0) = 
     &  + HZ1(0) *HZ5(0,0,1,0,-1)
     &  - 2.0000000000000000d+00*HZ3(0,0,-1)*HZ3(0,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,0,0,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,-1,0,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,0,-1,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,0,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,0,-1)
      HZ6(0,0,1,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,0,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,1,1)
     &  - HZ1(0) *HZ5(0,0,1,0,1)
     &  + 5.0000000000000000d-01*HZ3(0,0,1)*HZ3(0,0,1)
      HZ6(0,0,1,1,1,0) = 
     &  + HZ1(0) *HZ5(0,0,1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,1,1)
     &  - HZ6(0,0,1,0,1,1) 
     &  - HZ6(0,0,1,1,0,1) 
      HZ6(0,0,1,1,-1,0) = 
     &  + HZ1(0) *HZ5(0,0,1,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,1,-1)
     &  - HZ6(0,0,1,0,1, -1)
     &  - HZ6(0,0,1,1,0, -1)
      HZ6(0,0,1,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,1,-1)
     &  - HZ1(0) *HZ5(0,0,1,0,-1)
     &  + HZ3(0,0, -1)*HZ3(0,0,1)
     &  - HZ6(0,0, -1,0,0,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,0,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,0,-1,1)
      HZ6(0,0,1,-1,1,0) = 
     &  + HZ1(0) *HZ5(0,0,1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  - HZ6(0,0,1, -1,0,1)
     &  - HZ6(0,0,1,0, -1,1)
      HZ6(0,0,1,-1,-1,0) = 
     &  + HZ1(0) *HZ5(0,0,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,-1,-1)
     &  - HZ6(0,0,1, -1,0,-1)
     &  - HZ6(0,0,1,0, -1,-1)
      HZ6(0,0,-1,0,0,0) = 
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,0,-1)
     &  - 1.5000000000000000d+00*HZ1(0)*HZ1(0)*HZ4(0,0,0,-1)
     &  + 6.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,0,-1)
     &  - 1.0000000000000000d+01*HZ6(0,0,0,0,0,-1)
      HZ6(0,0,-1,0,0,-1) = 
     &  + 5.0000000000000000d-01*HZ3(0,0,-1)*HZ3(0,0,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,0,-1,-1)
      HZ6(0,0,-1,0,1,0) = 
     &  + HZ1(0) *HZ5(0,0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,0,0,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,0,1)
      HZ6(0,0,-1,0,-1,0) = 
     &  + HZ1(0) *HZ5(0,0,-1,0,-1)
     &  - HZ3(0,0, -1)*HZ3(0,0,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,0,-1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,0,-1,-1)
      HZ6(0,0,-1,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,0,-1,1)
     &  - HZ1(0) *HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,-1,1)
     &  + HZ6(0,0, -1,0,0,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,0,-1,1)
      HZ6(0,0,-1,1,1,0) = 
     &  + HZ1(0) *HZ5(0,0,-1,1,1)
     &  - HZ6(0,0, -1,0,1,1)
     &  - HZ6(0,0, -1,1,0,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,1,1)
      HZ6(0,0,-1,1,-1,0) = 
     &  + HZ1(0) *HZ5(0,0,-1,1,-1)
     &  - HZ6(0,0, -1,0,1,-1)
     &  - HZ6(0,0, -1,1,0,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
      HZ6(0,0,-1,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,0,-1,-1)
     &  - HZ1(0) *HZ5(0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,-1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,0,-1)*HZ3(0,0,-1)
      HZ6(0,0,-1,-1,1,0) = 
     &  + HZ1(0) *HZ5(0,0,-1,-1,1)
     &  - HZ6(0,0, -1,-1,0,1)
     &  - HZ6(0,0, -1,0,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,-1,1)
      HZ6(0,0,-1,-1,-1,0) = 
     &  + HZ1(0) *HZ5(0,0,-1,-1,-1)
     &  - HZ6(0,0, -1,-1,0,-1)
     &  - HZ6(0,0, -1,0,-1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,-1,-1)
      HZ6(0,1,0,0,0,0) = 
     &  + 4.1666666666666666d-02*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)*HZ2(0,1)
     &  - 3.3333333333333333d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,0,1)
     &  + 1.5000000000000000d+00*HZ1(0)*HZ1(0)*HZ4(0,0,0,1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,0,1)
     &  + 5.0000000000000000d+00*HZ6(0,0,0,0,0,1)
      HZ6(0,1,0,0,0,1) = 
     &  + HZ2(0,1) *HZ4(0,0,0,1)
     &  - HZ3(0,0,1) *HZ3(0,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,0,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,0,1,0,1)
      HZ6(0,1,0,0,0,-1) = 
     &  + HZ2(0,1) *HZ4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HZ3(0,0,-1)*HZ3(0,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,0,0,1)
     &  + 5.0000000000000000d+00*HZ6(0,0,0,-1,0,1)
     &  + 8.0000000000000000d+00*HZ6(0,0,0,0,-1,1)
     &  + 8.0000000000000000d+00*HZ6(0,0,0,0,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,0,-1)
      HZ6(0,1,0,0,1,0) = 
     &  + HZ1(0) *HZ2(0,1)*HZ3(0,0,1)
     &  - 6.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,0,1)
     &  - 3.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,0,1)
     &  + 2.0000000000000000d+00*HZ3(0,0,1)*HZ3(0,0,1)
      HZ6(0,1,0,0,1,1) = 
     &  + HZ2(0,1) *HZ4(0,0,1,1)
     &  - 9.0000000000000000d+00*HZ6(0,0,0,1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,0,1,1)
     &  - HZ6(0,0,1,1,0,1) 
      HZ6(0,1,0,0,1,-1) = 
     &  + HZ2(0,1) *HZ4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,1,1,-1)
     &  - HZ6(0,0,1, -1,0,1)
     &  - HZ6(0,0,1,0, -1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,1,0,1,-1)
      HZ6(0,1,0,0,-1,0) = 
     &  + HZ1(0) *HZ2(0,1)*HZ3(0,0,-1)
     &  - HZ1(0) *HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,0,-1)
     &  - 3.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,0,-1)
     &  + 4.0000000000000000d+00*HZ3(0,0,-1)*HZ3(0,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,0,0,1)
     &  - 9.0000000000000000d+00*HZ6(0,0,0,-1,0,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,0,-1,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,0,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,0,-1)
      HZ6(0,1,0,0,-1,1) = 
     &  + HZ2(0,1) *HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,0,1,1)
     &  - HZ6(0,0, -1,1,0,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,-1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,0,-1,1)
      HZ6(0,1,0,0,-1,-1) = 
     &  + HZ2(0,1) *HZ4(0,0,-1,-1)
     &  - HZ6(0,0, -1,-1,0,1)
     &  - HZ6(0,0, -1,0,-1,1)
     &  - HZ6(0,0, -1,0,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,0,-1,-1)
      HZ6(0,1,0,1,0,0) = 
     &  + 2.5000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(0,1)*HZ2(0,1)
     &  - HZ1(0) *HZ1(0)*HZ4(0,0,1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ2(0,1)*HZ3(0,0,1)
     &  + 1.2000000000000000d+01*HZ1(0)*HZ5(0,0,0,1,1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,0,1)
     &  + 3.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,0,1)
     &  - HZ3(0,0,1) *HZ3(0,0,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,0,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,0,1)
      HZ6(0,1,0,1,0,1) = 
     &  + 1.6666666666666666d-01*HZ2(0,1)*HZ2(0,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,1,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,0,1,1)
      HZ6(0,1,0,1,0,-1) = 
     &  + 5.0000000000000000d-01*HZ2(0,-1)*HZ2(0,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  - HZ2(0,1) *HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,1,-1)
     &  + HZ6(0, -1,0,1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(0,1,0,1,1,0) = 
     &  + HZ1(0) *HZ5(0,1,0,1,1)
     &  - 1.6666666666666666d-01*HZ2(0,1)*HZ2(0,1)*HZ2(0,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,1,0,1)
      HZ6(0,1,0,1,-1,0) = 
     &  + HZ1(0) *HZ5(0,1,0,1,-1)
     &  - 5.0000000000000000d-01*HZ2(0,-1)*HZ2(0,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  + HZ2(0,1) *HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,-1,1)
     &  - HZ6(0, -1,0,1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,0,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,0,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(0,1,0,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(0,-1)*HZ2(0,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,0,1)
     &  - HZ1(0) *HZ1(0)*HZ4(0,0,-1,1)
     &  - HZ1(0) *HZ1(0)*HZ4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ2(0,1)*HZ3(0,0,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,0,-1)
     &  + 3.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HZ3(0,0,-1)*HZ3(0,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,0,0,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,0,1)
      HZ6(0,1,0,-1,0,1) = 
     &  - 4.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  + HZ2(0,1) *HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,0,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,-1,1,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,1,-1,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,0,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(0,1,0,-1,0,-1) = 
     &  + 5.0000000000000000d-01*HZ2(0,-1)*HZ2(0,-1)*HZ2(0,1)
     &  - HZ2(0, -1)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,-1,-1)
     &  + HZ6(0, -1,0,-1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,0,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,0,-1)
      HZ6(0,1,0,-1,1,0) = 
     &  + HZ1(0) *HZ2(0,1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,0,1,1)
     &  - HZ1(0) *HZ5(0,-1,1,0,1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,-1,1)
     &  + 4.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  - HZ2(0,1) *HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,1,0,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,0,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(0,1,0,-1,1,1) = 
     &  + HZ2(0,1) *HZ4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  - HZ6(0, -1,1,1,0,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,1,1)
      HZ6(0,1,0,-1,1,-1) = 
     &  + HZ2(0,1) *HZ4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,1,1,-1)
     &  - HZ6(0, -1,1,-1,0,1)
     &  - HZ6(0, -1,1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,1,-1)
      HZ6(0,1,0,-1,-1,0) = 
     &  + HZ1(0) *HZ2(0,1)*HZ3(0,-1,-1)
     &  - HZ1(0) *HZ5(0,-1,-1,0,1)
     &  - HZ1(0) *HZ5(0,-1,0,-1,1)
     &  - HZ1(0) *HZ5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,-1,-1)
     &  - 5.0000000000000000d-01*HZ2(0,-1)*HZ2(0,-1)*HZ2(0,1)
     &  + HZ2(0, -1)*HZ4(0,-1,0,1)
     &  - HZ6(0, -1,0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,0,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,-1,-1,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,1,-1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,0,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,0,-1,-1)
      HZ6(0,1,0,-1,-1,1) = 
     &  + HZ2(0,1) *HZ4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,0,1,1)
     &  - HZ6(0, -1,-1,1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  - HZ6(0, -1,0,1,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,-1,1)
      HZ6(0,1,0,-1,-1,-1) = 
     &  + HZ2(0,1) *HZ4(0,-1,-1,-1)
     &  - HZ6(0, -1,-1,-1,0,1)
     &  - HZ6(0, -1,-1,0,-1,1)
     &  - HZ6(0, -1,-1,0,1,-1)
     &  - HZ6(0, -1,0,-1,-1,1)
     &  - HZ6(0, -1,0,-1,1,-1)
     &  - HZ6(0, -1,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,-1,-1)
      HZ6(0,1,1,0,0,0) = 
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,1,1)
     &  - 2.5000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(0,1)*HZ2(0,1)
     &  + HZ1(0) *HZ2(0,1)*HZ3(0,0,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,1,1)
     &  - HZ1(0) *HZ5(0,0,1,0,1)
     &  - HZ2(0,1) *HZ4(0,0,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,0,0,1,1)
     &  + HZ6(0,0,0,1,0,1) 
      HZ6(0,1,1,0,0,1) = 
     &  - 1.6666666666666666d-01*HZ2(0,1)*HZ2(0,1)*HZ2(0,1)
     &  + HZ3(0,0,1) *HZ3(0,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,1,1)
     &  - HZ6(0,0,1,0,1,1) 
      HZ6(0,1,1,0,0,-1) = 
     &  - 5.0000000000000000d-01*HZ2(0,-1)*HZ2(0,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  + HZ2(0,1) *HZ4(0,-1,0,1)
     &  + HZ2(0,1) *HZ4(0,0,-1,1)
     &  + HZ2(0,1) *HZ4(0,0,1,-1)
     &  + HZ3(0,0, -1)*HZ3(0,1,1)
     &  - HZ6(0, -1,0,1,0,1)
     &  + HZ6(0,0, -1,0,1,1)
     &  - HZ6(0,0, -1,1,0,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,1,-1)
     &  - HZ6(0,0,1, -1,0,1)
     &  + HZ6(0,0,1,0, -1,1)
     &  + HZ6(0,0,1,0,1, -1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(0,1,1,0,1,0) = 
     &  + HZ1(0) *HZ2(0,1)*HZ3(0,1,1)
     &  - 6.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,1,0,1,1)
     &  + 1.6666666666666666d-01*HZ2(0,1)*HZ2(0,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,1,1)
     &  - 2.0000000000000000d+00*HZ3(0,0,1)*HZ3(0,1,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,1,0,1)
      HZ6(0,1,1,0,1,1) = 
     &  + 5.0000000000000000d-01*HZ3(0,1,1)*HZ3(0,1,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,1,1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,1,0,1,1,1)
      HZ6(0,1,1,0,1,-1) = 
     &  - HZ2(0,1) *HZ4(0,-1,1,1)
     &  + HZ3(0,1, -1)*HZ3(0,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  + HZ6(0, -1,1,1,0,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,1,1,1,-1)
     &  - HZ6(0,1, -1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,0,1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,1,0,1,1,-1)
      HZ6(0,1,1,0,-1,0) = 
     &  + HZ1(0) *HZ2(0,-1)*HZ3(0,1,1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(0,-1,1)
     &  + HZ1(0) *HZ5(0,-1,0,1,1)
     &  + HZ1(0) *HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,1,-1)
     &  - HZ1(0) *HZ5(0,1,0,1,-1)
     &  + 5.0000000000000000d-01*HZ2(0,-1)*HZ2(0,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  - HZ2(0,1) *HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ3(0,0,-1)*HZ3(0,1,1)
     &  + HZ6(0, -1,0,1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,0,1,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,-1,1,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(0,1,1,0,-1,1) = 
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,1,1)
     &  + HZ3(0, -1,1)*HZ3(0,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,1,0,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  - HZ6(0,1,0,1, -1,1)
      HZ6(0,1,1,0,-1,-1) = 
     &  - HZ2(0,1) *HZ4(0,-1,-1,1)
     &  - HZ2(0,1) *HZ4(0,-1,1,-1)
     &  + HZ3(0, -1,-1)*HZ3(0,1,1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  + HZ6(0, -1,-1,0,1,1)
     &  + HZ6(0, -1,-1,1,0,1)
     &  - HZ6(0, -1,0,-1,1,1)
     &  + HZ6(0, -1,0,1,1,-1)
     &  + HZ6(0, -1,1,-1,0,1)
     &  + HZ6(0, -1,1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,1,-1,-1)
     &  - HZ6(0,1,0,1, -1,-1)
      HZ6(0,1,1,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,1,1,1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(0,1,1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,0,1,1)
     &  - HZ2(0,1) *HZ4(0,0,1,1)
     &  + HZ3(0,0,1) *HZ3(0,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,1,1)
     &  + HZ6(0,0,1,0,1,1) 
     &  + HZ6(0,0,1,1,0,1) 
      HZ6(0,1,1,1,0,1) = 
     &  + HZ2(0,1) *HZ4(0,1,1,1)
     &  - HZ3(0,1,1) *HZ3(0,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,0,1,1,1)
      HZ6(0,1,1,1,0,-1) = 
     &  + HZ2(0, -1)*HZ4(0,1,1,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,1,1)
     &  - HZ3(0, -1,1)*HZ3(0,1,1)
     &  - HZ3(0,1, -1)*HZ3(0,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,1,0,1)
     &  - 8.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,1,-1)
     &  + HZ6(0,1, -1,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,0,1,1,-1)
      HZ6(0,1,1,1,1,0) = 
     &  + HZ1(0) *HZ5(0,1,1,1,1)
     &  - HZ2(0,1) *HZ4(0,1,1,1)
     &  + 5.0000000000000000d-01*HZ3(0,1,1)*HZ3(0,1,1)
      HZ6(0,1,1,1,-1,0) = 
     &  + HZ1(0) *HZ5(0,1,1,1,-1)
     &  - HZ2(0, -1)*HZ4(0,1,1,1)
     &  - HZ2(0,1) *HZ4(0,-1,1,1)
     &  + HZ3(0, -1,1)*HZ3(0,1,1)
     &  + HZ6(0, -1,0,1,1,1)
     &  + HZ6(0, -1,1,0,1,1)
     &  + HZ6(0, -1,1,1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
      HZ6(0,1,1,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,1,1,-1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(0,1,1)
     &  + HZ1(0) *HZ2(0,1)*HZ3(0,-1,1)
     &  - HZ1(0) *HZ5(0,-1,0,1,1)
     &  - HZ1(0) *HZ5(0,-1,1,0,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,1,1)
     &  - HZ2(0,1) *HZ4(0,0,-1,1)
     &  + HZ3(0,0, -1)*HZ3(0,1,1)
     &  + HZ6(0,0, -1,0,1,1)
     &  + HZ6(0,0, -1,1,0,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,1,1)
      HZ6(0,1,1,-1,0,1) = 
     &  + 3.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,1,1)
     &  + HZ2(0,1) *HZ4(0,1,1,-1)
     &  - HZ3(0, -1,1)*HZ3(0,1,1)
     &  - HZ3(0,1, -1)*HZ3(0,1,1)
     &  - 6.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  - 5.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,1,1,0,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,-1,1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  + HZ6(0,1, -1,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,0,1,-1,1)
      HZ6(0,1,1,-1,0,-1) = 
     &  + HZ2(0, -1)*HZ4(0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,-1,1)
     &  + HZ2(0,1) *HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,1,1)
     &  - 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,1,0,1)
     &  - HZ6(0, -1,0,1,1,-1)
     &  - HZ6(0, -1,1,-1,0,1)
     &  - HZ6(0, -1,1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
      HZ6(0,1,1,-1,1,0) = 
     &  + HZ1(0) *HZ5(0,1,1,-1,1)
     &  - HZ2(0,1) *HZ4(0,-1,1,1)
     &  - HZ2(0,1) *HZ4(0,1,1,-1)
     &  + HZ3(0,1, -1)*HZ3(0,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  + HZ6(0, -1,1,1,0,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  - HZ6(0,1, -1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,0,1,-1,1)
      HZ6(0,1,1,-1,-1,0) = 
     &  + HZ1(0) *HZ5(0,1,1,-1,-1)
     &  - HZ2(0, -1)*HZ4(0,1,1,-1)
     &  - HZ2(0,1) *HZ4(0,-1,-1,1)
     &  + HZ3(0, -1,-1)*HZ3(0,1,1)
     &  + HZ6(0, -1,-1,0,1,1)
     &  + HZ6(0, -1,-1,1,0,1)
     &  + HZ6(0, -1,0,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
      HZ6(0,1,-1,0,0,0) = 
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,1,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(0,-1)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,0,1)
     &  + HZ1(0) *HZ1(0)*HZ4(0,0,-1,1)
     &  + HZ1(0) *HZ2(0,1)*HZ3(0,0,-1)
     &  - HZ1(0) *HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,-1,1)
     &  - HZ2(0,1) *HZ4(0,0,0,-1)
     &  + HZ6(0,0,0, -1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,0,0,-1,1)
      HZ6(0,1,-1,0,0,1) = 
     &  + 4.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  - HZ2(0,1) *HZ4(0,-1,0,1)
     &  - HZ2(0,1) *HZ4(0,0,-1,1)
     &  - HZ2(0,1) *HZ4(0,0,1,-1)
     &  + HZ3(0,0,1) *HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,0,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,-1,1,0,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,-1,1,1)
     &  - 9.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,1,1,-1)
     &  + HZ6(0,0,1, -1,0,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,1,0,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(0,1,-1,0,0,-1) = 
     &  - 5.0000000000000000d-01*HZ2(0,-1)*HZ2(0,-1)*HZ2(0,1)
     &  + HZ2(0, -1)*HZ4(0,-1,0,1)
     &  + HZ3(0,0, -1)*HZ3(0,1,-1)
     &  - HZ6(0, -1,0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,0,-1,1)
     &  + HZ6(0,0, -1,0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,0,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,-1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
      HZ6(0,1,-1,0,1,0) = 
     &  - HZ1(0) *HZ2(0,1)*HZ3(0,-1,1)
     &  + HZ1(0) *HZ2(0,1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,0,1,1)
     &  + HZ1(0) *HZ5(0,-1,1,0,1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,1,1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  + HZ2(0,1) *HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ3(0,0,1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,1,0,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,0,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(0,1,-1,0,1,-1) = 
     &  - HZ2(0,1) *HZ4(0,-1,1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  + 5.0000000000000000d-01*HZ3(0,1,-1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,1,1,-1)
     &  + HZ6(0, -1,1,-1,0,1)
     &  + HZ6(0, -1,1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,1,0,1,-1,-1)
      HZ6(0,1,-1,0,-1,0) = 
     &  + HZ1(0) *HZ2(0,-1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ2(0,1)*HZ3(0,-1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,0,-1,1)
     &  + HZ1(0) *HZ5(0,-1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,1,-1)
     &  + 5.0000000000000000d-01*HZ2(0,-1)*HZ2(0,-1)*HZ2(0,1)
     &  - HZ2(0, -1)*HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HZ3(0,0,-1)*HZ3(0,1,-1)
     &  + HZ6(0, -1,0,-1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,0,-1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,-1,-1,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
      HZ6(0,1,-1,0,-1,1) = 
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,-1,1)
     &  - HZ2(0,1) *HZ4(0,-1,1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  + HZ3(0, -1,1)*HZ3(0,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,-1,-1,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  + HZ6(0, -1,0,1,-1,1)
     &  + HZ6(0, -1,1,-1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,1)
      HZ6(0,1,-1,0,-1,-1) = 
     &  - 3.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,-1,-1)
     &  + HZ3(0, -1,-1)*HZ3(0,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,-1,-1,0,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,0,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,-1)
     &  + HZ6(0, -1,0,1,-1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,-1)
      HZ6(0,1,-1,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,1,-1,1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(0,1,-1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,0,1,-1)
     &  - HZ2(0,1) *HZ4(0,0,1,-1)
     &  + HZ3(0,0,1) *HZ3(0,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  + HZ6(0,0,1, -1,0,1)
     &  + HZ6(0,0,1,0, -1,1)
      HZ6(0,1,-1,1,0,1) = 
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,1,1)
     &  + HZ2(0,1) *HZ4(0,1,-1,1)
     &  + 6.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,1,0,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,0,1,-1,1)
      HZ6(0,1,-1,1,0,-1) = 
     &  + HZ2(0, -1)*HZ4(0,1,-1,1)
     &  + HZ2(0,1) *HZ4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  - HZ3(0, -1,1)*HZ3(0,1,-1)
     &  - 5.0000000000000000d-01*HZ3(0,1,-1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  - HZ6(0, -1,1,-1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,-1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,1,0,1,-1,-1)
      HZ6(0,1,-1,1,1,0) = 
     &  + HZ1(0) *HZ5(0,1,-1,1,1)
     &  + HZ2(0,1) *HZ4(0,-1,1,1)
     &  - HZ2(0,1) *HZ4(0,1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  - HZ6(0, -1,1,1,0,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  + HZ6(0,1, -1,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,0,1,-1,1)
      HZ6(0,1,-1,1,-1,0) = 
     &  + HZ1(0) *HZ5(0,1,-1,1,-1)
     &  - HZ2(0, -1)*HZ4(0,1,-1,1)
     &  - HZ2(0,1) *HZ4(0,-1,1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  + HZ3(0, -1,1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  + HZ6(0, -1,1,-1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
      HZ6(0,1,-1,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,1,-1,-1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(0,1,-1)
     &  + HZ1(0) *HZ2(0,1)*HZ3(0,-1,-1)
     &  - HZ1(0) *HZ5(0,-1,-1,0,1)
     &  - HZ1(0) *HZ5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,-1,1)
     &  - HZ2(0,1) *HZ4(0,0,-1,-1)
     &  + HZ3(0,0, -1)*HZ3(0,1,-1)
     &  + HZ6(0,0, -1,-1,0,1)
     &  + HZ6(0,0, -1,0,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,-1,1)
      HZ6(0,1,-1,-1,0,1) = 
     &  + HZ2(0,1) *HZ4(0,-1,-1,1)
     &  + HZ2(0,1) *HZ4(0,-1,1,-1)
     &  + HZ2(0,1) *HZ4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  - HZ3(0, -1,1)*HZ3(0,1,-1)
     &  - 5.0000000000000000d-01*HZ3(0,1,-1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,0,1,1)
     &  - HZ6(0, -1,-1,1,0,1)
     &  - HZ6(0, -1,1,-1,0,1)
      HZ6(0,1,-1,-1,0,-1) = 
     &  + HZ2(0, -1)*HZ4(0,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,-1,-1,0,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,-1,0,-1,1)
     &  - HZ6(0, -1,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  - HZ6(0, -1,0,-1,1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
      HZ6(0,1,-1,-1,1,0) = 
     &  + HZ1(0) *HZ5(0,1,-1,-1,1)
     &  - HZ2(0,1) *HZ4(0,1,-1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,1,-1)*HZ3(0,1,-1)
      HZ6(0,1,-1,-1,-1,0) = 
     &  + HZ1(0) *HZ5(0,1,-1,-1,-1)
     &  - HZ2(0, -1)*HZ4(0,1,-1,-1)
     &  - HZ2(0,1) *HZ4(0,-1,-1,-1)
     &  + HZ3(0, -1,-1)*HZ3(0,1,-1)
     &  + HZ6(0, -1,-1,-1,0,1)
     &  + HZ6(0, -1,-1,0,-1,1)
     &  + HZ6(0, -1,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
      HZ6(0,-1,0,0,0,0) = 
     &  + 4.1666666666666666d-02*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)*HZ2(0,-1)
     &  - 3.3333333333333333d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,0,-1)
     &  + 1.5000000000000000d+00*HZ1(0)*HZ1(0)*HZ4(0,0,0,-1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,0,-1)
     &  + 5.0000000000000000d+00*HZ6(0,0,0,0,0,-1)
      HZ6(0,-1,0,0,0,1) = 
     &  + HZ2(0, -1)*HZ4(0,0,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,0,0,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,0,0,1,-1)
     &  - HZ6(0,0,0,1,0, -1)
      HZ6(0,-1,0,0,0,-1) = 
     &  + HZ2(0, -1)*HZ4(0,0,0,-1)
     &  - HZ3(0,0, -1)*HZ3(0,0,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,0,-1,0,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,0,0,-1,-1)
      HZ6(0,-1,0,0,1,0) = 
     &  + HZ1(0) *HZ2(0,-1)*HZ3(0,0,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,1,-1)
     &  - HZ1(0) *HZ5(0,0,1,0,-1)
     &  - 3.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,0,0,1)
     &  + 9.0000000000000000d+00*HZ6(0,0,0,-1,0,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,0,-1,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,0,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,0,-1)
      HZ6(0,-1,0,0,1,1) = 
     &  + HZ2(0, -1)*HZ4(0,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,0,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,1,-1)
     &  - HZ6(0,0,1,0, -1,1)
     &  - HZ6(0,0,1,0,1, -1)
     &  - HZ6(0,0,1,1,0, -1)
      HZ6(0,-1,0,0,1,-1) = 
     &  + HZ2(0, -1)*HZ4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,1,-1,-1)
     &  - HZ6(0,0,1, -1,0,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,0,-1,-1)
      HZ6(0,-1,0,0,-1,0) = 
     &  + HZ1(0) *HZ2(0,-1)*HZ3(0,0,-1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,0,-1)
     &  + 2.0000000000000000d+00*HZ3(0,0,-1)*HZ3(0,0,-1)
      HZ6(0,-1,0,0,-1,1) = 
     &  + HZ2(0, -1)*HZ4(0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,-1,0,-1,1)
     &  - HZ6(0,0, -1,0,1,-1)
     &  - HZ6(0,0, -1,1,0,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
      HZ6(0,-1,0,0,-1,-1) = 
     &  + HZ2(0, -1)*HZ4(0,0,-1,-1)
     &  - HZ6(0,0, -1,-1,0,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,0,-1,-1)
     &  - 9.0000000000000000d+00*HZ6(0,0,0,-1,-1,-1)
      HZ6(0,-1,0,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ2(0,-1)*HZ3(0,0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,0,-1)
     &  + 3.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,0,0,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,-1,0,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,0,-1,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,0,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,0,-1)
      HZ6(0,-1,0,1,0,-1) = 
     &  + HZ2(0, -1)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,0,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,0,-1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,-1,-1,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,-1,1,-1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,1,-1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,0,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,0,-1,-1)
      HZ6(0,-1,0,1,1,0) = 
     &  + HZ1(0) *HZ5(0,-1,0,1,1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  - HZ6(0, -1,0,1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,0,1,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,-1,1,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(0,-1,0,1,-1,0) = 
     &  + HZ1(0) *HZ5(0,-1,0,1,-1)
     &  - HZ2(0, -1)*HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,0,-1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,-1,-1,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
      HZ6(0,-1,0,-1,0,0) = 
     &  + 2.5000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(0,-1)*HZ2(0,-1)
     &  - HZ1(0) *HZ1(0)*HZ4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ2(0,-1)*HZ3(0,0,-1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,0,-1)
     &  + 1.2000000000000000d+01*HZ1(0)*HZ5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,0,-1)
     &  - HZ3(0,0, -1)*HZ3(0,0,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,0,-1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,0,-1,-1)
      HZ6(0,-1,0,-1,0,-1) = 
     &  + 1.6666666666666666d-01*HZ2(0,-1)*HZ2(0,-1)*HZ2(0,-1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,-1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,0,-1,-1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,-1,-1,-1)
      HZ6(0,-1,0,-1,1,0) = 
     &  + HZ1(0) *HZ5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,-1,1)
     &  - HZ6(0, -1,0,-1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,0,-1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,-1,-1,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
      HZ6(0,-1,0,-1,-1,0) = 
     &  + HZ1(0) *HZ5(0,-1,0,-1,-1)
     &  - 1.6666666666666666d-01*HZ2(0,-1)*HZ2(0,-1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,-1,0,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,0,-1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,-1,-1,-1)
      HZ6(0,-1,1,0,0,0) = 
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,0,1)
     &  - HZ1(0) *HZ1(0)*HZ4(0,0,-1,1)
     &  + HZ1(0) *HZ2(0,-1)*HZ3(0,0,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,1,-1)
     &  - HZ1(0) *HZ5(0,0,1,0,-1)
     &  - HZ2(0, -1)*HZ4(0,0,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,0,0,1,-1)
     &  + HZ6(0,0,0,1,0, -1)
      HZ6(0,-1,1,0,0,1) = 
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  + HZ3(0, -1,1)*HZ3(0,0,1)
     &  - HZ6(0, -1,0,1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,0,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,1,1,-1)
     &  + HZ6(0,0,1,0, -1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(0,-1,1,0,0,-1) = 
     &  - HZ2(0, -1)*HZ4(0,-1,0,1)
     &  + HZ2(0, -1)*HZ4(0,0,-1,1)
     &  + HZ2(0, -1)*HZ4(0,0,1,-1)
     &  + HZ3(0, -1,1)*HZ3(0,0,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,0,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,-1,1,0,-1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,-1,-1,1)
     &  - 9.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,1,-1,-1)
     &  - HZ6(0,0,1, -1,0,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,0,-1,-1)
      HZ6(0,-1,1,0,1,0) = 
     &  + HZ1(0) *HZ5(0,-1,1,0,1)
     &  + 4.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,1)*HZ3(0,0,1)
     &  + HZ6(0, -1,0,1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,0,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,0,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(0,-1,1,0,-1,0) = 
     &  + HZ1(0) *HZ2(0,-1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,0,-1,1)
     &  - HZ1(0) *HZ5(0,-1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,1,-1)
     &  + HZ2(0, -1)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,1)*HZ3(0,0,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,0,-1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,-1,-1,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
      HZ6(0,-1,1,0,-1,1) = 
     &  + 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  - HZ6(0, -1,0,1,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,1)
      HZ6(0,-1,1,0,-1,-1) = 
     &  + HZ3(0, -1,-1)*HZ3(0,-1,1)
     &  - HZ6(0, -1,-1,0,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,-1)
     &  - HZ6(0, -1,0,1,-1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,-1)
      HZ6(0,-1,1,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,1,1)
     &  - HZ1(0) *HZ5(0,-1,0,1,1)
     &  - HZ1(0) *HZ5(0,-1,1,0,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,1,1)
     &  - HZ2(0, -1)*HZ4(0,0,1,1)
     &  + HZ3(0, -1,1)*HZ3(0,0,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,1,-1)
     &  + HZ6(0,0,1,0,1, -1)
     &  + HZ6(0,0,1,1,0, -1)
      HZ6(0,-1,1,1,0,-1) = 
     &  + HZ2(0, -1)*HZ4(0,-1,1,1)
     &  - 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  - HZ6(0, -1,0,1,1,-1)
     &  - HZ6(0, -1,1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
      HZ6(0,-1,1,1,1,0) = 
     &  + HZ1(0) *HZ5(0,-1,1,1,1)
     &  - HZ6(0, -1,0,1,1,1)
     &  - HZ6(0, -1,1,0,1,1)
     &  - HZ6(0, -1,1,1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
      HZ6(0,-1,1,1,-1,0) = 
     &  + HZ1(0) *HZ5(0,-1,1,1,-1)
     &  - HZ2(0, -1)*HZ4(0,-1,1,1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
      HZ6(0,-1,1,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,1,-1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,0,-1,1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,-1,1)
     &  - HZ2(0, -1)*HZ4(0,0,-1,1)
     &  + HZ3(0, -1,1)*HZ3(0,0,-1)
     &  + HZ6(0,0, -1,0,1,-1)
     &  + HZ6(0,0, -1,1,0,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
      HZ6(0,-1,1,-1,0,-1) = 
     &  + HZ2(0, -1)*HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,0,-1,1)
     &  + 6.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,-1)
     &  + 1.2000000000000000d+01*HZ6(0,0,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
      HZ6(0,-1,1,-1,1,0) = 
     &  + HZ1(0) *HZ5(0,-1,1,-1,1)
     &  - 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  - HZ6(0, -1,1,-1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
      HZ6(0,-1,1,-1,-1,0) = 
     &  + HZ1(0) *HZ5(0,-1,1,-1,-1)
     &  - HZ2(0, -1)*HZ4(0,-1,1,-1)
     &  + HZ3(0, -1,-1)*HZ3(0,-1,1)
     &  - HZ6(0, -1,-1,0,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
      HZ6(0,-1,-1,0,0,0) = 
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,-1,-1)
     &  - 2.5000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(0,-1)*HZ2(0,-1)
     &  + HZ1(0) *HZ2(0,-1)*HZ3(0,0,-1)
     &  - HZ1(0) *HZ5(0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,-1,-1)
     &  - HZ2(0, -1)*HZ4(0,0,0,-1)
     &  + HZ6(0,0,0, -1,0,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,0,0,-1,-1)
      HZ6(0,-1,-1,0,0,1) = 
     &  - HZ2(0, -1)*HZ4(0,0,-1,1)
     &  - HZ2(0, -1)*HZ4(0,0,1,-1)
     &  + HZ3(0, -1,-1)*HZ3(0,0,1)
     &  - HZ6(0, -1,0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  + HZ6(0,0, -1,0,-1,1)
     &  + HZ6(0,0, -1,0,1,-1)
     &  + HZ6(0,0, -1,1,0,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,-1,-1)
     &  + HZ6(0,0,1, -1,0,-1)
     &  + HZ6(0,0,1,0, -1,-1)
      HZ6(0,-1,-1,0,0,-1) = 
     &  - 1.6666666666666666d-01*HZ2(0,-1)*HZ2(0,-1)*HZ2(0,-1)
     &  + HZ3(0, -1,-1)*HZ3(0,0,-1)
     &  - HZ6(0,0, -1,0,-1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,-1,-1)
      HZ6(0,-1,-1,0,1,0) = 
     &  + HZ1(0) *HZ5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,0,1)
     &  + HZ6(0, -1,0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,0,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,-1,-1,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,0,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,0,-1,-1)
      HZ6(0,-1,-1,0,-1,0) = 
     &  + HZ1(0) *HZ2(0,-1)*HZ3(0,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,-1,0,-1,-1)
     &  - 6.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,-1,-1)
     &  + 1.6666666666666666d-01*HZ2(0,-1)*HZ2(0,-1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,0,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,-1,0,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,0,-1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,-1,-1,-1)
      HZ6(0,-1,-1,0,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ3(0,-1,-1)*HZ3(0,-1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,-1,-1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,-1,-1,-1,-1)
      HZ6(0,-1,-1,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,-1,1)
     &  - HZ1(0) *HZ5(0,-1,-1,0,1)
     &  - HZ1(0) *HZ5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,-1,1)
     &  - HZ2(0, -1)*HZ4(0,0,1,-1)
     &  + HZ3(0, -1,-1)*HZ3(0,0,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,-1,-1)
     &  + HZ6(0,0,1, -1,0,-1)
     &  + HZ6(0,0,1,0, -1,-1)
      HZ6(0,-1,-1,1,0,-1) = 
     &  + HZ2(0, -1)*HZ4(0,-1,-1,1)
     &  - HZ6(0, -1,-1,0,-1,1)
     &  - HZ6(0, -1,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  - HZ6(0, -1,0,-1,1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
      HZ6(0,-1,-1,1,1,0) = 
     &  + HZ1(0) *HZ5(0,-1,-1,1,1)
     &  - HZ6(0, -1,-1,0,1,1)
     &  - HZ6(0, -1,-1,1,0,1)
     &  - HZ6(0, -1,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
      HZ6(0,-1,-1,1,-1,0) = 
     &  + HZ1(0) *HZ5(0,-1,-1,1,-1)
     &  - HZ2(0, -1)*HZ4(0,-1,-1,1)
     &  + HZ6(0, -1,-1,0,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
      HZ6(0,-1,-1,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,-1,-1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(0,-1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,0,-1,-1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,-1,-1)
     &  - HZ2(0, -1)*HZ4(0,0,-1,-1)
     &  + HZ3(0, -1,-1)*HZ3(0,0,-1)
     &  + HZ6(0,0, -1,-1,0,-1)
     &  + HZ6(0,0, -1,0,-1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,-1,-1)
      HZ6(0,-1,-1,-1,0,-1) = 
     &  + HZ2(0, -1)*HZ4(0,-1,-1,-1)
     &  - HZ3(0, -1,-1)*HZ3(0,-1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,-1,-1)
      HZ6(0,-1,-1,-1,1,0) = 
     &  + HZ1(0) *HZ5(0,-1,-1,-1,1)
     &  - HZ6(0, -1,-1,-1,0,1)
     &  - HZ6(0, -1,-1,0,-1,1)
     &  - HZ6(0, -1,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
      HZ6(0,-1,-1,-1,-1,0) = 
     &  + HZ1(0) *HZ5(0,-1,-1,-1,-1)
     &  - HZ2(0, -1)*HZ4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,-1)*HZ3(0,-1,-1)
      HZ6(1,0,0,0,0,0) = 
     &  + 8.3333333333333333d-03*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(1)
     &  - 4.1666666666666666d-02*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)*HZ2(0,1)
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,0,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,0,0,1)
     &  + HZ1(0) *HZ5(0,0,0,0,1)
     &  - HZ6(0,0,0,0,0,1) 
      HZ6(1,0,0,0,0,1) = 
     &  + HZ1(1) *HZ5(0,0,0,0,1)
     &  - HZ2(0,1) *HZ4(0,0,0,1)
     &  + 5.0000000000000000d-01*HZ3(0,0,1)*HZ3(0,0,1)
      HZ6(1,0,0,0,0,-1) = 
     &  + HZ1(1) *HZ5(0,0,0,0,-1)
     &  - HZ2(0,1) *HZ4(0,0,0,-1)
     &  + HZ3(0,0, -1)*HZ3(0,0,1)
     &  - HZ6(0,0, -1,0,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,0,1,-1)
     &  - HZ6(0,0,0,1,0, -1)
      HZ6(1,0,0,0,1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ4(0,0,0,1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(0,0,1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,0,1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,0,1)
     &  - HZ3(0,0,1) *HZ3(0,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,0,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,0,1,0,1)
      HZ6(1,0,0,0,1,1) = 
     &  + HZ1(1) *HZ5(0,0,0,1,1)
     &  - HZ2(0,1) *HZ4(0,0,1,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,1,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,1,0,1,1)
     &  + HZ6(0,0,1,1,0,1) 
      HZ6(1,0,0,0,1,-1) = 
     &  + HZ1(1) *HZ5(0,0,0,1,-1)
     &  - HZ2(0,1) *HZ4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,0,1,1,-1)
     &  + HZ6(0,0,1, -1,0,1)
     &  + HZ6(0,0,1,0, -1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,0,1,-1)
      HZ6(1,0,0,0,-1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ4(0,0,0,-1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(0,0,-1)
     &  + HZ1(0) *HZ5(0,0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,1,-1)
     &  + HZ1(0) *HZ5(0,0,1,0,-1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HZ3(0,0,-1)*HZ3(0,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,0,0,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,0,0,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,0,0,1,-1)
     &  + HZ6(0,0,0,1,0, -1)
      HZ6(1,0,0,0,-1,1) = 
     &  + HZ1(1) *HZ5(0,0,0,-1,1)
     &  - HZ2(0,1) *HZ4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,0,1,1)
     &  + HZ6(0,0, -1,1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,0,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  + HZ6(0,0,1,0, -1,1)
      HZ6(1,0,0,0,-1,-1) = 
     &  + HZ1(1) *HZ5(0,0,0,-1,-1)
     &  - HZ2(0,1) *HZ4(0,0,-1,-1)
     &  + HZ6(0,0, -1,-1,0,1)
     &  + HZ6(0,0, -1,0,-1,1)
     &  + HZ6(0,0, -1,0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,0,1,-1,-1)
     &  + HZ6(0,0,1,0, -1,-1)
      HZ6(1,0,0,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(0,0,1)
     &  - 2.5000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(0,1)*HZ2(0,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,0,0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ2(0,1)*HZ3(0,0,1)
     &  - 6.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,0,1)
     &  + 6.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,0,1)
     &  + 5.0000000000000000d-01*HZ3(0,0,1)*HZ3(0,0,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,0,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,0,1)
      HZ6(1,0,0,1,0,1) = 
     &  + HZ1(1) *HZ5(0,0,1,0,1)
     &  - 1.6666666666666666d-01*HZ2(0,1)*HZ2(0,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,1,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,1,1,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,1,0,1)
      HZ6(1,0,0,1,0,-1) = 
     &  + HZ1(1) *HZ5(0,0,1,0,-1)
     &  - 5.0000000000000000d-01*HZ2(0,-1)*HZ2(0,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  + HZ2(0,1) *HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,1,-1)
     &  - HZ6(0, -1,0,1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,0,1)
     &  - HZ6(0,0,1,0, -1,1)
     &  - HZ6(0,0,1,0,1, -1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(1,0,0,1,1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ4(0,0,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,1,1)
     &  - HZ1(0) *HZ5(0,1,0,1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,1,1)
     &  - HZ1(1) *HZ5(0,0,1,0,1)
     &  + 1.6666666666666666d-01*HZ2(0,1)*HZ2(0,1)*HZ2(0,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,1,1)
     &  + HZ6(0,0,1,0,1,1) 
      HZ6(1,0,0,1,1,1) = 
     &  + HZ1(1) *HZ5(0,0,1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,1,1)
     &  - HZ6(0,1,0,1,1,1) 
      HZ6(1,0,0,1,1,-1) = 
     &  + HZ1(1) *HZ5(0,0,1,1,-1)
     &  - HZ6(0,0,1,1, -1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,1,1,1,-1)
     &  - HZ6(0,1,0,1,1, -1)
      HZ6(1,0,0,1,-1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ4(0,0,1,-1)
     &  - HZ1(0) *HZ5(0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,1,-1)
     &  - HZ1(0) *HZ5(0,1,0,1,-1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,1,-1)
     &  - HZ1(1) *HZ5(0,0,1,0,-1)
     &  + 5.0000000000000000d-01*HZ2(0,-1)*HZ2(0,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  - HZ2(0,1) *HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,-1,1)
     &  + HZ6(0, -1,0,1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,0,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,1,1,-1)
     &  - HZ6(0,0,1,0, -1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(1,0,0,1,-1,1) = 
     &  + HZ1(1) *HZ5(0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  - HZ6(0,1,0,1, -1,1)
      HZ6(1,0,0,1,-1,-1) = 
     &  + HZ1(1) *HZ5(0,0,1,-1,-1)
     &  - HZ6(0,0,1, -1,-1,1)
     &  - HZ6(0,0,1, -1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,1,-1,-1)
     &  - HZ6(0,1,0,1, -1,-1)
      HZ6(1,0,0,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(0,0,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(0,-1)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,0,0,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ2(0,1)*HZ3(0,0,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,1,-1)
     &  - HZ1(0) *HZ5(0,0,1,0,-1)
     &  + 6.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,0,-1)
     &  - 3.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,0,-1)
     &  + HZ3(0,0, -1)*HZ3(0,0,1)
     &  - HZ6(0,0, -1,0,0,1)
      HZ6(1,0,0,-1,0,1) = 
     &  + HZ1(1) *HZ5(0,0,-1,0,1)
     &  + 4.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  - HZ2(0,1) *HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,1,0,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,-1,0,1,1)
     &  + HZ6(0,0, -1,1,0,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,-1,1,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,1,-1,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,1,1,-1)
     &  + HZ6(0,0,1, -1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,0,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(1,0,0,-1,0,-1) = 
     &  + HZ1(1) *HZ5(0,0,-1,0,-1)
     &  - 5.0000000000000000d-01*HZ2(0,-1)*HZ2(0,-1)*HZ2(0,1)
     &  + HZ2(0, -1)*HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,-1,-1)
     &  - HZ6(0, -1,0,-1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  - HZ6(0,0, -1,0,-1,1)
     &  - HZ6(0,0, -1,0,1,-1)
     &  + HZ6(0,0, -1,1,0,-1)
     &  + HZ6(0,0,1, -1,0,-1)
      HZ6(1,0,0,-1,1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ4(0,0,-1,1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,0,1,1)
     &  + HZ1(0) *HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,1,1)
     &  + HZ1(0) *HZ5(0,0,1,-1,1)
     &  - HZ1(1) *HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  + HZ2(0,1) *HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,0,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,-1,1,0,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,-1,1,1)
     &  + 9.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,1,1,-1)
     &  - HZ6(0,0,1, -1,0,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,1,0,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(1,0,0,-1,1,1) = 
     &  + HZ1(1) *HZ5(0,0,-1,1,1)
     &  - HZ2(0,1) *HZ4(0,-1,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  + HZ6(0, -1,1,1,0,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
     &  + HZ6(0,0,1, -1,1,1)
      HZ6(1,0,0,-1,1,-1) = 
     &  + HZ1(1) *HZ5(0,0,-1,1,-1)
     &  - HZ2(0,1) *HZ4(0,-1,1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,1,1,-1)
     &  + HZ6(0, -1,1,-1,0,1)
     &  + HZ6(0, -1,1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  - HZ6(0,0, -1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
     &  + HZ6(0,0,1, -1,1,-1)
      HZ6(1,0,0,-1,-1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ4(0,0,-1,-1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(0,-1,-1)
     &  + HZ1(0) *HZ5(0,-1,-1,0,1)
     &  + HZ1(0) *HZ5(0,-1,0,-1,1)
     &  + HZ1(0) *HZ5(0,-1,0,1,-1)
     &  + HZ1(0) *HZ5(0,0,-1,-1,1)
     &  + HZ1(0) *HZ5(0,0,-1,1,-1)
     &  + HZ1(0) *HZ5(0,0,1,-1,-1)
     &  - HZ1(1) *HZ5(0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,-1,-1)
     &  + 5.0000000000000000d-01*HZ2(0,-1)*HZ2(0,-1)*HZ2(0,1)
     &  - HZ2(0, -1)*HZ4(0,-1,0,1)
     &  + HZ6(0, -1,0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  - HZ6(0,0, -1,0,-1,1)
     &  - HZ6(0,0, -1,0,1,-1)
     &  - HZ6(0,0, -1,1,0,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,-1,-1)
     &  - HZ6(0,0,1, -1,0,-1)
     &  - HZ6(0,0,1,0, -1,-1)
      HZ6(1,0,0,-1,-1,1) = 
     &  + HZ1(1) *HZ5(0,0,-1,-1,1)
     &  - HZ2(0,1) *HZ4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,0,1,1)
     &  + HZ6(0, -1,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  + HZ6(0, -1,0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  + HZ6(0,0, -1,1,-1,1)
     &  + HZ6(0,0,1, -1,-1,1)
      HZ6(1,0,0,-1,-1,-1) = 
     &  + HZ1(1) *HZ5(0,0,-1,-1,-1)
     &  - HZ2(0,1) *HZ4(0,-1,-1,-1)
     &  + HZ6(0, -1,-1,-1,0,1)
     &  + HZ6(0, -1,-1,0,-1,1)
     &  + HZ6(0, -1,-1,0,1,-1)
     &  + HZ6(0, -1,0,-1,-1,1)
     &  + HZ6(0, -1,0,-1,1,-1)
     &  + HZ6(0, -1,0,1,-1,-1)
     &  + HZ6(0,0, -1,-1,-1,1)
     &  + HZ6(0,0, -1,-1,1,-1)
     &  + HZ6(0,0, -1,1,-1,-1)
     &  + HZ6(0,0,1, -1,-1,-1)
      HZ6(1,0,1,0,0,0) = 
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(1)*HZ2(0,1)
     &  - 3.3333333333333333d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,1,1)
     &  - HZ1(0) *HZ1(0)*HZ1(1)*HZ3(0,0,1)
     &  + 2.5000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(0,1)*HZ2(0,1)
     &  + HZ1(0) *HZ1(0)*HZ4(0,0,1,1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,0,0,1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(0,0,1)
     &  + HZ1(0) *HZ5(0,0,1,0,1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,0,1)
     &  + HZ2(0,1) *HZ4(0,0,0,1)
     &  - HZ6(0,0,0,1,0,1) 
      HZ6(1,0,1,0,0,1) = 
     &  + HZ1(1) *HZ2(0,1)*HZ3(0,0,1)
     &  - 6.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,0,1)
     &  + 1.6666666666666666d-01*HZ2(0,1)*HZ2(0,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ3(0,0,1)*HZ3(0,1,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,1,1,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,1,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,1,0,1)
      HZ6(1,0,1,0,0,-1) = 
     &  + HZ1(1) *HZ2(0,1)*HZ3(0,0,-1)
     &  - HZ1(1) *HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,0,-1)
     &  + 5.0000000000000000d-01*HZ2(0,-1)*HZ2(0,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  - HZ2(0,1) *HZ4(0,-1,0,1)
     &  - HZ2(0,1) *HZ4(0,0,-1,1)
     &  - HZ2(0,1) *HZ4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ3(0,0,-1)*HZ3(0,1,1)
     &  + HZ6(0, -1,0,1,0,1)
     &  + HZ6(0,0, -1,1,0,1)
     &  + HZ6(0,0,1, -1,0,1)
     &  + HZ6(0,0,1,0, -1,1)
     &  + HZ6(0,0,1,0,1, -1)
      HZ6(1,0,1,0,1,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ2(0,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,0,1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ2(0,1)*HZ3(0,1,1)
     &  + 1.2000000000000000d+01*HZ1(0)*HZ5(0,0,1,1,1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,1,0,1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ2(0,1)*HZ3(0,0,1)
     &  + 1.2000000000000000d+01*HZ1(1)*HZ5(0,0,0,1,1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,0,1)
     &  - 1.6666666666666666d-01*HZ2(0,1)*HZ2(0,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,1,1)
     &  + 4.0000000000000000d+00*HZ3(0,0,1)*HZ3(0,1,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,0,1,1)
      HZ6(1,0,1,0,1,1) = 
     &  + HZ1(1) *HZ5(0,1,0,1,1)
     &  - HZ3(0,1,1) *HZ3(0,1,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,1,1,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,1,0,1,1,1)
      HZ6(1,0,1,0,1,-1) = 
     &  + HZ1(1) *HZ5(0,1,0,1,-1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ3(0,1,-1)*HZ3(0,1,1)
     &  - 6.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,1,0,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,-1,1,1,1)
     &  + 8.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,0,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,1,0,1,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,1,0,1,1,-1)
      HZ6(1,0,1,0,-1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ2(0,-1)*HZ2(0,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ2(0,-1)*HZ3(0,1,1)
     &  + HZ1(0) *HZ2(0,1)*HZ3(0,-1,1)
     &  - HZ1(0) *HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,-1,1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,1,-1)
     &  + HZ1(0) *HZ5(0,1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ2(0,1)*HZ3(0,0,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,0,-1)
     &  - 5.0000000000000000d-01*HZ2(0,-1)*HZ2(0,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  + HZ2(0,1) *HZ4(0,-1,0,1)
     &  + 4.0000000000000000d+00*HZ3(0,0,-1)*HZ3(0,1,1)
     &  - HZ6(0, -1,0,1,0,1)
      HZ6(1,0,1,0,-1,1) = 
     &  + HZ1(1) *HZ2(0,1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,0,1,1)
     &  - HZ1(1) *HZ5(0,-1,1,0,1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,1)*HZ3(0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,-1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  + HZ6(0,1,0,1, -1,1)
      HZ6(1,0,1,0,-1,-1) = 
     &  + HZ1(1) *HZ2(0,1)*HZ3(0,-1,-1)
     &  - HZ1(1) *HZ5(0,-1,-1,0,1)
     &  - HZ1(1) *HZ5(0,-1,0,-1,1)
     &  - HZ1(1) *HZ5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,-1,-1)
     &  + HZ2(0,1) *HZ4(0,-1,-1,1)
     &  + HZ2(0,1) *HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,1,1)
     &  - 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  - HZ6(0, -1,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  + HZ6(0, -1,0,1,-1,1)
     &  - HZ6(0, -1,1,-1,0,1)
     &  - HZ6(0, -1,1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,-1,-1)
     &  + HZ6(0,1,0,1, -1,-1)
      HZ6(1,0,1,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(0,1,1)
     &  - 1.5000000000000000d+00*HZ1(0)*HZ1(0)*HZ4(0,1,1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ2(0,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ2(0,1)*HZ3(0,1,1)
     &  - 6.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,1,0,1,1)
     &  + HZ1(1) *HZ2(0,1)*HZ3(0,0,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,1,1)
     &  - HZ1(1) *HZ5(0,0,1,0,1)
     &  + HZ2(0,1) *HZ4(0,0,1,1)
     &  - 2.0000000000000000d+00*HZ3(0,0,1)*HZ3(0,1,1)
     &  - HZ6(0,0,1,1,0,1) 
      HZ6(1,0,1,1,0,1) = 
     &  + HZ1(1) *HZ2(0,1)*HZ3(0,1,1)
     &  - 6.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,1,0,1,1)
     &  - 3.0000000000000000d+00*HZ2(0,1)*HZ4(0,1,1,1)
     &  + 2.0000000000000000d+00*HZ3(0,1,1)*HZ3(0,1,1)
      HZ6(1,0,1,1,0,-1) = 
     &  + HZ1(1) *HZ2(0,-1)*HZ3(0,1,1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(0,-1,1)
     &  + HZ1(1) *HZ5(0,-1,0,1,1)
     &  + HZ1(1) *HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,1,-1)
     &  - HZ1(1) *HZ5(0,1,0,1,-1)
     &  - 3.0000000000000000d+00*HZ2(0,-1)*HZ4(0,1,1,1)
     &  - 3.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,1,1)
     &  + 2.0000000000000000d+00*HZ3(0,-1,1)*HZ3(0,1,1)
     &  + 2.0000000000000000d+00*HZ3(0,1,-1)*HZ3(0,1,1)
     &  + 6.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,1,1,0,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,-1,1,1,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,1,1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,0,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,1,0,1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,1,0,1,1,-1)
      HZ6(1,0,1,1,1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ4(0,1,1,1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(0,1,1,1,1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(0,1,1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,0,1,1)
     &  + 3.0000000000000000d+00*HZ2(0,1)*HZ4(0,1,1,1)
     &  - HZ3(0,1,1) *HZ3(0,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,0,1,1,1)
      HZ6(1,0,1,1,1,1) = 
     &  + HZ1(1) *HZ5(0,1,1,1,1)
     &  - 5.0000000000000000d+00*HZ6(0,1,1,1,1,1)
      HZ6(1,0,1,1,1,-1) = 
     &  + HZ1(1) *HZ5(0,1,1,1,-1)
     &  - HZ6(0,1,1,1, -1,1)
     &  - 4.0000000000000000d+00*HZ6(0,1,1,1,1,-1)
      HZ6(1,0,1,1,-1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ4(0,1,1,-1)
     &  - HZ1(0) *HZ5(0,1,1,-1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,1,1,1,-1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(0,1,1)
     &  + HZ1(1) *HZ2(0,1)*HZ3(0,-1,1)
     &  - HZ1(1) *HZ5(0,-1,0,1,1)
     &  - HZ1(1) *HZ5(0,-1,1,0,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,1,1)
     &  + 3.0000000000000000d+00*HZ2(0,-1)*HZ4(0,1,1,1)
     &  + HZ2(0,1) *HZ4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,1)*HZ3(0,1,1)
     &  - HZ6(0, -1,1,1,0,1)
      HZ6(1,0,1,1,-1,1) = 
     &  + HZ1(1) *HZ5(0,1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,1,1,1,-1,1)
      HZ6(1,0,1,1,-1,-1) = 
     &  + HZ1(1) *HZ5(0,1,1,-1,-1)
     &  - HZ6(0,1,1, -1,-1,1)
     &  - HZ6(0,1,1, -1,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,1,1,1,-1,-1)
      HZ6(1,0,1,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(0,1,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,1,-1,1)
     &  - HZ1(0) *HZ1(0)*HZ4(0,1,1,-1)
     &  - HZ1(0) *HZ1(1)*HZ2(0,-1)*HZ2(0,1)
     &  + HZ1(0) *HZ1(1)*HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ2(0,-1)*HZ3(0,1,1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(0,-1,1)
     &  + HZ1(0) *HZ5(0,-1,1,0,1)
     &  + HZ1(1) *HZ2(0,1)*HZ3(0,0,-1)
     &  - HZ1(1) *HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,-1,1)
     &  + HZ2(0,1) *HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ3(0,0,-1)*HZ3(0,1,1)
     &  - HZ6(0,0, -1,1,0,1)
      HZ6(1,0,1,-1,0,1) = 
     &  - HZ1(1) *HZ2(0,1)*HZ3(0,-1,1)
     &  + HZ1(1) *HZ2(0,1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,0,1,1)
     &  + HZ1(1) *HZ5(0,-1,1,0,1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,1,1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,1,1)
     &  - HZ2(0,1) *HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ3(0,-1,1)*HZ3(0,1,1)
     &  + 2.0000000000000000d+00*HZ3(0,1,-1)*HZ3(0,1,1)
     &  + 6.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  + 6.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,-1,1,1,0,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,0,1,-1,1)
      HZ6(1,0,1,-1,0,-1) = 
     &  + HZ1(1) *HZ2(0,-1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ2(0,1)*HZ3(0,-1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,0,-1,1)
     &  + HZ1(1) *HZ5(0,-1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,1,-1)
     &  - HZ2(0, -1)*HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,-1,1)
     &  - HZ2(0,1) *HZ4(0,-1,1,-1)
     &  + 4.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,1,1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  - HZ6(0, -1,0,1,-1,1)
     &  + HZ6(0, -1,1,-1,0,1)
     &  + HZ6(0, -1,1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,1)
      HZ6(1,0,1,-1,1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,1,-1,1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(0,1,-1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,0,1,-1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,1,1)
     &  + HZ2(0,1) *HZ4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ3(0,1,-1)*HZ3(0,1,1)
     &  - 6.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,1,0,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,-1,1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,0,1,-1,1)
      HZ6(1,0,1,-1,1,1) = 
     &  + HZ1(1) *HZ5(0,1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,1,-1,1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,1,-1,1,1)
      HZ6(1,0,1,-1,1,-1) = 
     &  + HZ1(1) *HZ5(0,1,-1,1,-1)
     &  - HZ6(0,1, -1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,1,1,-1,1,-1)
      HZ6(1,0,1,-1,-1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ4(0,1,-1,-1)
     &  - HZ1(0) *HZ5(0,1,-1,-1,1)
     &  - HZ1(0) *HZ5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,1,-1,-1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(0,1,-1)
     &  + HZ1(1) *HZ2(0,1)*HZ3(0,-1,-1)
     &  - HZ1(1) *HZ5(0,-1,-1,0,1)
     &  - HZ1(1) *HZ5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,-1,1)
     &  + HZ2(0, -1)*HZ4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,1,1,-1)
     &  + HZ2(0,1) *HZ4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,1,1)
     &  - HZ6(0, -1,-1,1,0,1)
      HZ6(1,0,1,-1,-1,1) = 
     &  + HZ1(1) *HZ5(0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,-1,1,1)
     &  - HZ6(0,1, -1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,1,-1,-1,1)
      HZ6(1,0,1,-1,-1,-1) = 
     &  + HZ1(1) *HZ5(0,1,-1,-1,-1)
     &  - HZ6(0,1, -1,-1,-1,1)
     &  - HZ6(0,1, -1,-1,1,-1)
     &  - HZ6(0,1, -1,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,1,1,-1,-1,-1)
      HZ6(1,0,-1,0,0,0) = 
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(1)*HZ2(0,-1)
     &  - 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,-1,1)
     &  - 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,1,-1)
     &  - HZ1(0) *HZ1(0)*HZ1(1)*HZ3(0,0,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(0,-1)*HZ2(0,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,0,1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,0,0,-1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(0,0,-1)
     &  + HZ1(0) *HZ5(0,0,-1,0,1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,0,-1)
     &  + HZ2(0,1) *HZ4(0,0,0,-1)
     &  - HZ6(0,0,0, -1,0,1)
      HZ6(1,0,-1,0,0,1) = 
     &  + HZ1(1) *HZ2(0,-1)*HZ3(0,0,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,1,-1)
     &  - HZ1(1) *HZ5(0,0,1,0,-1)
     &  - 4.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  + HZ2(0,1) *HZ4(0,-1,0,1)
     &  + HZ2(0,1) *HZ4(0,0,-1,1)
     &  + HZ2(0,1) *HZ4(0,0,1,-1)
     &  - HZ3(0, -1,1)*HZ3(0,0,1)
     &  - HZ3(0,0,1) *HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,1,0,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,-1,0,1,1)
     &  - HZ6(0,0, -1,1,0,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,-1,1,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,1,-1,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,1,1,-1)
     &  - HZ6(0,0,1, -1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,0,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(1,0,-1,0,0,-1) = 
     &  + HZ1(1) *HZ2(0,-1)*HZ3(0,0,-1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,-1,-1)
     &  + 5.0000000000000000d-01*HZ2(0,-1)*HZ2(0,-1)*HZ2(0,1)
     &  - HZ2(0, -1)*HZ4(0,-1,0,1)
     &  - HZ3(0, -1,1)*HZ3(0,0,-1)
     &  - HZ3(0,0, -1)*HZ3(0,1,-1)
     &  + HZ6(0, -1,0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  + HZ6(0,0, -1,0,-1,1)
     &  + HZ6(0,0, -1,0,1,-1)
      HZ6(1,0,-1,0,1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ4(0,-1,0,1)
     &  + HZ1(0) *HZ2(0,1)*HZ3(0,-1,1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(0,1,-1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(0,-1,0,1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,1,0,1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,1,1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ2(0,-1)*HZ3(0,0,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,0,-1)
     &  + 4.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  - HZ2(0,1) *HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HZ3(0,-1,1)*HZ3(0,0,1)
     &  + 2.0000000000000000d+00*HZ3(0,0,1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,0,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,-1,1,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,1,-1,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,0,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(1,0,-1,0,1,1) = 
     &  + HZ1(1) *HZ5(0,-1,0,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  - HZ6(0, -1,1,0,1,1)
     &  - HZ6(0,1, -1,0,1,1)
      HZ6(1,0,-1,0,1,-1) = 
     &  + HZ1(1) *HZ5(0,-1,0,1,-1)
     &  + HZ2(0,1) *HZ4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ3(0,1,-1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  - HZ6(0, -1,0,1,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,-1,0,1,1,-1)
     &  - HZ6(0, -1,1,-1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,-1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,1,0,1,-1,-1)
      HZ6(1,0,-1,0,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ2(0,-1)*HZ2(0,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,0,-1,-1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(0,-1,1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ2(0,1)*HZ3(0,-1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,0,1)
     &  - HZ1(0) *HZ5(0,-1,0,-1,1)
     &  - HZ1(0) *HZ5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ2(0,-1)*HZ3(0,0,-1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,0,-1)
     &  + 1.2000000000000000d+01*HZ1(1)*HZ5(0,0,0,-1,-1)
     &  - 5.0000000000000000d-01*HZ2(0,-1)*HZ2(0,-1)*HZ2(0,1)
     &  + HZ2(0, -1)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HZ3(0,-1,1)*HZ3(0,0,-1)
     &  + 2.0000000000000000d+00*HZ3(0,0,-1)*HZ3(0,1,-1)
     &  - HZ6(0, -1,0,-1,0,1)
      HZ6(1,0,-1,0,-1,1) = 
     &  + HZ1(1) *HZ5(0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,-1,1)
     &  + HZ2(0,1) *HZ4(0,-1,1,-1)
     &  - HZ3(0, -1,1)*HZ3(0,-1,1)
     &  - HZ3(0, -1,1)*HZ3(0,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,-1,-1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  - HZ6(0, -1,0,1,-1,1)
     &  - HZ6(0, -1,1,-1,0,1)
      HZ6(1,0,-1,0,-1,-1) = 
     &  + HZ1(1) *HZ5(0,-1,0,-1,-1)
     &  + 3.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,-1,-1)
     &  - HZ3(0, -1,-1)*HZ3(0,-1,1)
     &  - HZ3(0, -1,-1)*HZ3(0,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,0,1,-1)
     &  - HZ6(0, -1,0,-1,-1,1)
     &  - HZ6(0, -1,0,-1,1,-1)
     &  - HZ6(0, -1,0,1,-1,-1)
      HZ6(1,0,-1,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(0,-1,1)
     &  - HZ1(0) *HZ1(0)*HZ4(0,-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,1,-1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,0,-1,1)
     &  + HZ1(0) *HZ2(0,1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,0,1,1)
     &  + HZ1(0) *HZ5(0,-1,1,0,1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,1,1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,0,1,-1)
     &  + HZ1(1) *HZ2(0,-1)*HZ3(0,0,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,1,-1)
     &  - HZ1(1) *HZ5(0,0,1,0,-1)
     &  + HZ2(0,1) *HZ4(0,0,1,-1)
     &  - HZ3(0, -1,1)*HZ3(0,0,1)
     &  - HZ3(0,0,1) *HZ3(0,1,-1)
     &  - HZ6(0,0,1, -1,0,1)
      HZ6(1,0,-1,1,0,1) = 
     &  + HZ1(1) *HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,1,1)
     &  - HZ2(0,1) *HZ4(0,1,-1,1)
     &  - 6.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  - 6.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,-1,1,1,0,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,-1,1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,0,1,-1,1)
      HZ6(1,0,-1,1,0,-1) = 
     &  + HZ1(1) *HZ2(0,-1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,0,-1,1)
     &  - HZ1(1) *HZ5(0,-1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,-1,1,1)
     &  - HZ2(0, -1)*HZ4(0,1,-1,1)
     &  - HZ2(0,1) *HZ4(0,-1,1,-1)
     &  + HZ3(0, -1,1)*HZ3(0,-1,1)
     &  + HZ3(0, -1,1)*HZ3(0,1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,1,-1)*HZ3(0,1,-1)
     &  + HZ6(0, -1,0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,1,1,-1)
     &  + HZ6(0, -1,1,-1,0,1)
     &  + HZ6(0, -1,1,0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,1,0,1,-1,-1)
      HZ6(1,0,-1,1,1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,-1,1,1,1)
     &  - HZ1(0) *HZ5(0,1,-1,1,1)
     &  - HZ1(1) *HZ5(0,-1,0,1,1)
     &  - HZ1(1) *HZ5(0,-1,1,0,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,1,1)
     &  - HZ2(0,1) *HZ4(0,-1,1,1)
     &  + HZ2(0,1) *HZ4(0,1,-1,1)
     &  + 6.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  + 5.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,1,1,0,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  - HZ6(0,1, -1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,0,1,-1,1)
      HZ6(1,0,-1,1,1,1) = 
     &  + HZ1(1) *HZ5(0,-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,-1,1,1,1,1)
     &  - HZ6(0,1, -1,1,1,1)
      HZ6(1,0,-1,1,1,-1) = 
     &  + HZ1(1) *HZ5(0,-1,1,1,-1)
     &  - HZ6(0, -1,1,1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,1,1,1,-1)
     &  - HZ6(0,1, -1,1,1,-1)
      HZ6(1,0,-1,1,-1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ4(0,-1,1,-1)
     &  - HZ1(0) *HZ5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,1,1,-1)
     &  - HZ1(0) *HZ5(0,1,-1,1,-1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,0,-1,1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,-1,1,1)
     &  + HZ2(0, -1)*HZ4(0,1,-1,1)
     &  + HZ2(0,1) *HZ4(0,-1,1,-1)
     &  - HZ3(0, -1,1)*HZ3(0,-1,1)
     &  - HZ3(0, -1,1)*HZ3(0,1,-1)
     &  - HZ6(0, -1,1,-1,0,1)
      HZ6(1,0,-1,1,-1,1) = 
     &  + HZ1(1) *HZ5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,1,-1,1)
     &  - HZ6(0,1, -1,1,-1,1)
      HZ6(1,0,-1,1,-1,-1) = 
     &  + HZ1(1) *HZ5(0,-1,1,-1,-1)
     &  - HZ6(0, -1,1,-1,-1,1)
     &  - HZ6(0, -1,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,1,-1,-1)
     &  - HZ6(0,1, -1,1,-1,-1)
      HZ6(1,0,-1,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(0,-1,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ2(0,-1)*HZ2(0,-1)
     &  + HZ1(0) *HZ2(0,-1)*HZ3(0,-1,1)
     &  + HZ1(0) *HZ2(0,-1)*HZ3(0,1,-1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(0,-1,-1)
     &  + HZ1(0) *HZ5(0,-1,-1,0,1)
     &  + HZ1(1) *HZ2(0,-1)*HZ3(0,0,-1)
     &  - HZ1(1) *HZ5(0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,-1,-1)
     &  + HZ2(0,1) *HZ4(0,0,-1,-1)
     &  - HZ3(0, -1,1)*HZ3(0,0,-1)
     &  - HZ3(0,0, -1)*HZ3(0,1,-1)
     &  - HZ6(0,0, -1,-1,0,1)
      HZ6(1,0,-1,-1,0,1) = 
     &  + HZ1(1) *HZ5(0,-1,-1,0,1)
     &  - HZ2(0,1) *HZ4(0,-1,-1,1)
     &  - HZ2(0,1) *HZ4(0,-1,1,-1)
     &  - HZ2(0,1) *HZ4(0,1,-1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  + HZ3(0, -1,1)*HZ3(0,1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,1,-1)*HZ3(0,1,-1)
      HZ6(1,0,-1,-1,0,-1) = 
     &  + HZ1(1) *HZ2(0,-1)*HZ3(0,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,-1,0,-1,-1)
     &  - 6.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,-1,-1)
     &  - HZ2(0, -1)*HZ4(0,-1,-1,1)
     &  - HZ2(0, -1)*HZ4(0,-1,1,-1)
     &  - HZ2(0, -1)*HZ4(0,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,-1,-1,0,1)
     &  + HZ6(0, -1,-1,0,-1,1)
     &  + HZ6(0, -1,-1,0,1,-1)
      HZ6(1,0,-1,-1,1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,1,1)
     &  - HZ1(0) *HZ5(0,-1,1,-1,1)
     &  - HZ1(0) *HZ5(0,1,-1,-1,1)
     &  - HZ1(1) *HZ5(0,-1,-1,0,1)
     &  - HZ1(1) *HZ5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,-1,1)
     &  + HZ2(0,1) *HZ4(0,1,-1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ3(0,1,-1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,0,1,1)
     &  + HZ6(0, -1,-1,1,0,1)
     &  + HZ6(0, -1,1,-1,0,1)
      HZ6(1,0,-1,-1,1,1) = 
     &  + HZ1(1) *HZ5(0,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,-1,1,1,1)
     &  - HZ6(0, -1,1,-1,1,1)
     &  - HZ6(0,1, -1,-1,1,1)
      HZ6(1,0,-1,-1,1,-1) = 
     &  + HZ1(1) *HZ5(0,-1,-1,1,-1)
     &  - HZ6(0, -1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,1,1,-1)
     &  - HZ6(0, -1,1,-1,1,-1)
     &  - HZ6(0,1, -1,-1,1,-1)
      HZ6(1,0,-1,-1,-1,0) = 
     &  + HZ1(0) *HZ1(1)*HZ4(0,-1,-1,-1)
     &  - HZ1(0) *HZ5(0,-1,-1,-1,1)
     &  - HZ1(0) *HZ5(0,-1,-1,1,-1)
     &  - HZ1(0) *HZ5(0,-1,1,-1,-1)
     &  - HZ1(0) *HZ5(0,1,-1,-1,-1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(0,-1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,0,-1,-1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,-1,-1)
     &  + HZ2(0, -1)*HZ4(0,-1,-1,1)
     &  + HZ2(0, -1)*HZ4(0,-1,1,-1)
     &  + HZ2(0, -1)*HZ4(0,1,-1,-1)
     &  + HZ2(0,1) *HZ4(0,-1,-1,-1)
     &  - HZ3(0, -1,-1)*HZ3(0,-1,1)
     &  - HZ3(0, -1,-1)*HZ3(0,1,-1)
     &  - HZ6(0, -1,-1,-1,0,1)
      HZ6(1,0,-1,-1,-1,1) = 
     &  + HZ1(1) *HZ5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,-1,1,1)
     &  - HZ6(0, -1,-1,1,-1,1)
     &  - HZ6(0, -1,1,-1,-1,1)
     &  - HZ6(0,1, -1,-1,-1,1)
      HZ6(1,0,-1,-1,-1,-1) = 
     &  + HZ1(1) *HZ5(0,-1,-1,-1,-1)
     &  - HZ6(0, -1,-1,-1,-1,1)
     &  - HZ6(0, -1,-1,-1,1,-1)
     &  - HZ6(0, -1,-1,1,-1,-1)
     &  - HZ6(0, -1,1,-1,-1,-1)
     &  - HZ6(0,1, -1,-1,-1,-1)
      HZ6(1,1,0,0,0,0) = 
     &  + 2.0833333333333333d-02*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(1)*HZ1(1)
     &  - 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(1)*HZ2(0,1)
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(0,0,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,0,1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,0,0,1)
     &  + HZ1(0) *HZ5(0,0,0,1,1)
     &  + HZ1(1) *HZ5(0,0,0,0,1)
     &  - HZ6(0,0,0,0,1,1) 
      HZ6(1,1,0,0,0,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,0,0,1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(0,0,1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,0,1)
     &  + HZ3(0,0,1) *HZ3(0,1,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,1,0,1,1)
     &  - HZ6(0,0,1,1,0,1) 
      HZ6(1,1,0,0,0,-1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,0,0,-1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(0,0,-1)
     &  + HZ1(1) *HZ5(0,0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,1,-1)
     &  + HZ1(1) *HZ5(0,0,1,0,-1)
     &  + HZ3(0,0, -1)*HZ3(0,1,1)
     &  - HZ6(0,0, -1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,0,1,1,-1)
     &  - HZ6(0,0,1,0, -1,1)
     &  - HZ6(0,0,1,0,1, -1)
     &  - HZ6(0,0,1,1,0, -1)
      HZ6(1,1,0,0,1,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(0,0,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ2(0,1)*HZ2(0,1)
     &  + HZ1(0) *HZ2(0,1)*HZ3(0,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,1,1)
     &  - HZ1(0) *HZ5(0,1,0,1,1)
     &  - 1.5000000000000000d+00*HZ1(1)*HZ1(1)*HZ4(0,0,0,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ2(0,1)*HZ3(0,0,1)
     &  - 6.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,0,1)
     &  - 2.0000000000000000d+00*HZ3(0,0,1)*HZ3(0,1,1)
     &  + 9.0000000000000000d+00*HZ6(0,0,0,1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,0,1,1)
     &  + HZ6(0,0,1,1,0,1) 
      HZ6(1,1,0,0,1,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,0,1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,1,1)
     &  - HZ1(1) *HZ5(0,1,0,1,1)
     &  + 5.0000000000000000d-01*HZ3(0,1,1)*HZ3(0,1,1)
      HZ6(1,1,0,0,1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,0,1,-1)
     &  - HZ1(1) *HZ5(0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,1,-1)
     &  - HZ1(1) *HZ5(0,1,0,1,-1)
     &  - HZ2(0,1) *HZ4(0,-1,1,1)
     &  + HZ3(0,1, -1)*HZ3(0,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  + HZ6(0, -1,1,1,0,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
     &  + HZ6(0,0,1, -1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,1,1,1,-1)
     &  - HZ6(0,1, -1,0,1,1)
     &  - HZ6(0,1,0,1, -1,1)
     &  - HZ6(0,1,0,1,1, -1)
      HZ6(1,1,0,0,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(0,0,-1)
     &  - HZ1(0) *HZ1(1)*HZ2(0,-1)*HZ2(0,1)
     &  + HZ1(0) *HZ1(1)*HZ4(0,-1,0,1)
     &  + HZ1(0) *HZ1(1)*HZ4(0,0,-1,1)
     &  + HZ1(0) *HZ1(1)*HZ4(0,0,1,-1)
     &  + HZ1(0) *HZ2(0,-1)*HZ3(0,1,1)
     &  - HZ1(0) *HZ5(0,-1,0,1,1)
     &  - HZ1(0) *HZ5(0,0,-1,1,1)
     &  - HZ1(0) *HZ5(0,0,1,-1,1)
     &  - HZ1(0) *HZ5(0,0,1,1,-1)
     &  - 1.5000000000000000d+00*HZ1(1)*HZ1(1)*HZ4(0,0,0,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ2(0,1)*HZ3(0,0,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,1,-1)
     &  - HZ1(1) *HZ5(0,0,1,0,-1)
     &  - 2.0000000000000000d+00*HZ3(0,0,-1)*HZ3(0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,0,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,1,-1)
     &  + HZ6(0,0,1,0, -1,1)
     &  + HZ6(0,0,1,0,1, -1)
     &  + HZ6(0,0,1,1,0, -1)
      HZ6(1,1,0,0,-1,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,0,-1,1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,0,1,1)
     &  + HZ1(1) *HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,1,1)
     &  + HZ1(1) *HZ5(0,0,1,-1,1)
     &  + HZ3(0, -1,1)*HZ3(0,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  - HZ6(0, -1,1,0,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,1,1)
     &  - HZ6(0,0,1,1, -1,1)
      HZ6(1,1,0,0,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,0,-1,-1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(0,-1,-1)
     &  + HZ1(1) *HZ5(0,-1,-1,0,1)
     &  + HZ1(1) *HZ5(0,-1,0,-1,1)
     &  + HZ1(1) *HZ5(0,-1,0,1,-1)
     &  + HZ1(1) *HZ5(0,0,-1,-1,1)
     &  + HZ1(1) *HZ5(0,0,-1,1,-1)
     &  + HZ1(1) *HZ5(0,0,1,-1,-1)
     &  + HZ3(0, -1,-1)*HZ3(0,1,1)
     &  - HZ6(0, -1,-1,0,1,1)
     &  - HZ6(0, -1,0,-1,1,1)
     &  - HZ6(0, -1,0,1,-1,1)
     &  - HZ6(0, -1,0,1,1,-1)
     &  - HZ6(0,0, -1,-1,1,1)
     &  - HZ6(0,0, -1,1,-1,1)
     &  - HZ6(0,0, -1,1,1,-1)
     &  - HZ6(0,0,1, -1,-1,1)
     &  - HZ6(0,0,1, -1,1,-1)
     &  - HZ6(0,0,1,1, -1,-1)
      HZ6(1,1,0,1,0,0) = 
     &  + 2.5000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ1(1)*HZ2(0,1)
     &  - HZ1(0) *HZ1(0)*HZ1(1)*HZ3(0,1,1)
     &  + 1.5000000000000000d+00*HZ1(0)*HZ1(0)*HZ4(0,1,1,1)
     &  - HZ1(0) *HZ1(1)*HZ1(1)*HZ3(0,0,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ2(0,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,0,1,1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(0,1,1)
     &  + HZ1(0) *HZ5(0,1,0,1,1)
     &  + 1.5000000000000000d+00*HZ1(1)*HZ1(1)*HZ4(0,0,0,1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(0,0,1)
     &  + HZ1(1) *HZ5(0,0,1,0,1)
     &  + HZ3(0,0,1) *HZ3(0,1,1)
     &  - HZ6(0,0,1,0,1,1) 
      HZ6(1,1,0,1,0,1) = 
     &  + 2.5000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(0,1)*HZ2(0,1)
     &  - HZ1(1) *HZ1(1)*HZ4(0,0,1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ2(0,1)*HZ3(0,1,1)
     &  + 1.2000000000000000d+01*HZ1(1)*HZ5(0,0,1,1,1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,1,0,1,1)
     &  + 3.0000000000000000d+00*HZ2(0,1)*HZ4(0,1,1,1)
     &  - HZ3(0,1,1) *HZ3(0,1,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,1,1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,1,0,1,1,1)
      HZ6(1,1,0,1,0,-1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(0,-1)*HZ2(0,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,0,1)
     &  - HZ1(1) *HZ1(1)*HZ4(0,0,-1,1)
     &  - HZ1(1) *HZ1(1)*HZ4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ2(0,-1)*HZ3(0,1,1)
     &  + HZ1(1) *HZ2(0,1)*HZ3(0,-1,1)
     &  - HZ1(1) *HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,-1,1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,1,-1)
     &  + HZ1(1) *HZ5(0,1,0,1,-1)
     &  + 3.0000000000000000d+00*HZ2(0,-1)*HZ4(0,1,1,1)
     &  + HZ2(0,1) *HZ4(0,-1,1,1)
     &  - HZ3(0, -1,1)*HZ3(0,1,1)
     &  - HZ3(0,1, -1)*HZ3(0,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  - HZ6(0, -1,1,0,1,1)
     &  - HZ6(0, -1,1,1,0,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,1,1)
     &  + HZ6(0,1, -1,0,1,1)
     &  + HZ6(0,1,0,1, -1,1)
     &  + HZ6(0,1,0,1,1, -1)
      HZ6(1,1,0,1,1,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(0,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,1,1,1)
     &  + 6.0000000000000000d+00*HZ1(0)*HZ5(0,1,1,1,1)
     &  - 2.5000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(0,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ2(0,1)*HZ3(0,1,1)
     &  - 6.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,1,0,1,1)
     &  - 3.0000000000000000d+00*HZ2(0,1)*HZ4(0,1,1,1)
     &  + 5.0000000000000000d-01*HZ3(0,1,1)*HZ3(0,1,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,1,1,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,1,0,1,1,1)
      HZ6(1,1,0,1,1,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,1,1,1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(0,1,1,1,1)
     &  + 1.0000000000000000d+01*HZ6(0,1,1,1,1,1)
      HZ6(1,1,0,1,1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,1,1,-1)
     &  - HZ1(1) *HZ5(0,1,1,-1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,1,1,1,-1)
     &  + HZ6(0,1,1, -1,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,1,1,1,-1,1)
     &  + 6.0000000000000000d+00*HZ6(0,1,1,1,1,-1)
      HZ6(1,1,0,1,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(0,1,-1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,1,1,-1)
     &  + HZ1(0) *HZ5(0,1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,1,-1,1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ5(0,1,1,1,-1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(0,-1)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,0,1)
     &  + HZ1(1) *HZ1(1)*HZ4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ2(0,-1)*HZ3(0,1,1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(0,-1,1)
     &  + HZ1(1) *HZ5(0,-1,1,0,1)
     &  - 3.0000000000000000d+00*HZ2(0,-1)*HZ4(0,1,1,1)
     &  + HZ3(0, -1,1)*HZ3(0,1,1)
     &  - HZ6(0, -1,1,0,1,1)
      HZ6(1,1,0,1,-1,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,1,1,-1,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,1,1,1,-1,1)
      HZ6(1,1,0,1,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,1,-1,-1)
     &  - HZ1(1) *HZ5(0,1,-1,-1,1)
     &  - HZ1(1) *HZ5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,1,-1,-1)
     &  + HZ6(0,1, -1,-1,1,1)
     &  + HZ6(0,1, -1,1,-1,1)
     &  + HZ6(0,1, -1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,1,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,1,-1,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,1,1,1,-1,-1)
      HZ6(1,1,0,-1,0,0) = 
     &  + 2.5000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ1(1)*HZ2(0,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(0,1,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,1,-1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,1,1,-1)
     &  - HZ1(0) *HZ1(1)*HZ1(1)*HZ3(0,0,-1)
     &  + HZ1(0) *HZ1(1)*HZ2(0,-1)*HZ2(0,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,-1,0,1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(0,1,1)
     &  + HZ1(0) *HZ5(0,-1,0,1,1)
     &  + 1.5000000000000000d+00*HZ1(1)*HZ1(1)*HZ4(0,0,0,-1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(0,0,-1)
     &  + HZ1(1) *HZ5(0,0,-1,0,1)
     &  + HZ3(0,0, -1)*HZ3(0,1,1)
     &  - HZ6(0,0, -1,0,1,1)
      HZ6(1,1,0,-1,0,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,0,1)
     &  + HZ1(1) *HZ2(0,1)*HZ3(0,-1,1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(0,1,-1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(0,-1,0,1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,1,0,1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,1,1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,0,1,-1)
     &  + HZ2(0,1) *HZ4(0,-1,1,1)
     &  + HZ2(0,1) *HZ4(0,1,-1,1)
     &  + HZ2(0,1) *HZ4(0,1,1,-1)
     &  - HZ3(0, -1,1)*HZ3(0,1,1)
     &  - HZ3(0,1, -1)*HZ3(0,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  + HZ6(0, -1,1,0,1,1)
     &  + HZ6(0,1, -1,0,1,1)
      HZ6(1,1,0,-1,0,-1) = 
     &  + 2.5000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(0,-1)*HZ2(0,-1)
     &  - HZ1(1) *HZ1(1)*HZ4(0,0,-1,-1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(0,-1,1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ2(0,1)*HZ3(0,-1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,0,1)
     &  - HZ1(1) *HZ5(0,-1,0,-1,1)
     &  - HZ1(1) *HZ5(0,-1,0,1,-1)
     &  + HZ2(0, -1)*HZ4(0,-1,1,1)
     &  + HZ2(0, -1)*HZ4(0,1,-1,1)
     &  + HZ2(0, -1)*HZ4(0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,0,1,1)
     &  + HZ6(0, -1,0,-1,1,1)
     &  + HZ6(0, -1,0,1,-1,1)
     &  + HZ6(0, -1,0,1,1,-1)
      HZ6(1,1,0,-1,1,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,-1,1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,1,-1,1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ5(0,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,-1,1,1)
     &  + HZ1(0) *HZ5(0,1,1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,0,1)
     &  - HZ1(1) *HZ1(1)*HZ4(0,0,-1,1)
     &  + HZ1(1) *HZ2(0,1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,0,1,1)
     &  + HZ1(1) *HZ5(0,-1,1,0,1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,1,1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,0,1,-1)
     &  - HZ2(0,1) *HZ4(0,-1,1,1)
     &  - HZ2(0,1) *HZ4(0,1,-1,1)
     &  - HZ2(0,1) *HZ4(0,1,1,-1)
     &  + HZ3(0,1, -1)*HZ3(0,1,1)
     &  - HZ6(0,1, -1,0,1,1)
      HZ6(1,1,0,-1,1,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,-1,1,1,1)
     &  - HZ1(1) *HZ5(0,1,-1,1,1)
     &  + 6.0000000000000000d+00*HZ6(0,-1,1,1,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,1,-1,1,1,1)
     &  + HZ6(0,1,1, -1,1,1)
      HZ6(1,1,0,-1,1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,1,-1)
     &  - HZ1(1) *HZ5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,1,1,-1)
     &  - HZ1(1) *HZ5(0,1,-1,1,-1)
     &  + HZ6(0, -1,1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,1,1,1,-1)
     &  + HZ6(0,1, -1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,1,1,-1)
     &  + HZ6(0,1,1, -1,1,-1)
      HZ6(1,1,0,-1,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(0,-1,-1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,-1,-1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,-1,1,-1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,1,-1,-1)
     &  + HZ1(0) *HZ5(0,-1,-1,1,1)
     &  + HZ1(0) *HZ5(0,-1,1,-1,1)
     &  + HZ1(0) *HZ5(0,-1,1,1,-1)
     &  + HZ1(0) *HZ5(0,1,-1,-1,1)
     &  + HZ1(0) *HZ5(0,1,-1,1,-1)
     &  + HZ1(0) *HZ5(0,1,1,-1,-1)
     &  - 2.5000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(0,-1)*HZ2(0,-1)
     &  + HZ1(1) *HZ2(0,-1)*HZ3(0,-1,1)
     &  + HZ1(1) *HZ2(0,-1)*HZ3(0,1,-1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(0,-1,-1)
     &  + HZ1(1) *HZ5(0,-1,-1,0,1)
     &  - HZ2(0, -1)*HZ4(0,-1,1,1)
     &  - HZ2(0, -1)*HZ4(0,1,-1,1)
     &  - HZ2(0, -1)*HZ4(0,1,1,-1)
     &  + HZ3(0, -1,-1)*HZ3(0,1,1)
     &  - HZ6(0, -1,-1,0,1,1)
      HZ6(1,1,0,-1,-1,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,1,1)
     &  - HZ1(1) *HZ5(0,-1,1,-1,1)
     &  - HZ1(1) *HZ5(0,1,-1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,-1,1,1)
     &  + HZ6(0, -1,1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,-1,1,1)
     &  + HZ6(0,1, -1,1,-1,1)
     &  + HZ6(0,1,1, -1,-1,1)
      HZ6(1,1,0,-1,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,-1,-1)
     &  - HZ1(1) *HZ5(0,-1,-1,-1,1)
     &  - HZ1(1) *HZ5(0,-1,-1,1,-1)
     &  - HZ1(1) *HZ5(0,-1,1,-1,-1)
     &  - HZ1(1) *HZ5(0,1,-1,-1,-1)
     &  + HZ6(0, -1,-1,-1,1,1)
     &  + HZ6(0, -1,-1,1,-1,1)
     &  + HZ6(0, -1,-1,1,1,-1)
     &  + HZ6(0, -1,1,-1,-1,1)
     &  + HZ6(0, -1,1,-1,1,-1)
     &  + HZ6(0, -1,1,1,-1,-1)
     &  + HZ6(0,1, -1,-1,-1,1)
     &  + HZ6(0,1, -1,-1,1,-1)
     &  + HZ6(0,1, -1,1,-1,-1)
     &  + HZ6(0,1,1, -1,-1,-1)
      HZ6(1,1,1,0,0,0) = 
     &  + 2.7777777777777777d-02*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(1)*HZ1(1)*HZ1(1)
     &  - 2.5000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ1(1)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(0,1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,1,1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(0,0,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,0,1,1)
     &  + HZ1(0) *HZ5(0,0,1,1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,0,0,1)
     &  + HZ1(1) *HZ5(0,0,0,1,1)
     &  - HZ6(0,0,0,1,1,1) 
      HZ6(1,1,1,0,0,1) = 
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,0,1)
     &  - 2.5000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(0,1)*HZ2(0,1)
     &  + HZ1(1) *HZ2(0,1)*HZ3(0,1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,1,1)
     &  - HZ1(1) *HZ5(0,1,0,1,1)
     &  - HZ2(0,1) *HZ4(0,1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,1,1)
     &  + HZ6(0,1,0,1,1,1) 
      HZ6(1,1,1,0,0,-1) = 
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,0,-1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(0,-1)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,0,1,-1)
     &  + HZ1(1) *HZ2(0,-1)*HZ3(0,1,1)
     &  - HZ1(1) *HZ5(0,-1,0,1,1)
     &  - HZ1(1) *HZ5(0,0,-1,1,1)
     &  - HZ1(1) *HZ5(0,0,1,-1,1)
     &  - HZ1(1) *HZ5(0,0,1,1,-1)
     &  - HZ2(0, -1)*HZ4(0,1,1,1)
     &  + HZ6(0, -1,0,1,1,1)
     &  + HZ6(0,0, -1,1,1,1)
     &  + HZ6(0,0,1, -1,1,1)
     &  + HZ6(0,0,1,1, -1,1)
     &  + HZ6(0,0,1,1,1, -1)
      HZ6(1,1,1,0,1,0) = 
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(0,1)
     &  - HZ1(0) *HZ1(1)*HZ1(1)*HZ3(0,1,1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,1,1,1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(0,1,1,1,1)
     &  - 3.3333333333333333d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,0,1)
     &  + 2.5000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(0,1)*HZ2(0,1)
     &  + HZ1(1) *HZ1(1)*HZ4(0,0,1,1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(0,1,1)
     &  + HZ1(1) *HZ5(0,1,0,1,1)
     &  + HZ2(0,1) *HZ4(0,1,1,1)
     &  - HZ6(0,1,0,1,1,1) 
      HZ6(1,1,1,0,1,1) = 
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,1,1)
     &  - 1.5000000000000000d+00*HZ1(1)*HZ1(1)*HZ4(0,1,1,1)
     &  + 6.0000000000000000d+00*HZ1(1)*HZ5(0,1,1,1,1)
     &  - 1.0000000000000000d+01*HZ6(0,1,1,1,1,1)
      HZ6(1,1,1,0,1,-1) = 
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,1,-1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,1,-1,1)
     &  - HZ1(1) *HZ1(1)*HZ4(0,1,1,-1)
     &  + HZ1(1) *HZ5(0,1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,1,-1,1)
     &  + 3.0000000000000000d+00*HZ1(1)*HZ5(0,1,1,1,-1)
     &  - HZ6(0,1, -1,1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,1,1,1,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,1,1,1,1,-1)
      HZ6(1,1,1,0,-1,0) = 
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(0,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(0,1,-1)
     &  + HZ1(0) *HZ1(1)*HZ4(0,-1,1,1)
     &  + HZ1(0) *HZ1(1)*HZ4(0,1,-1,1)
     &  + HZ1(0) *HZ1(1)*HZ4(0,1,1,-1)
     &  - HZ1(0) *HZ5(0,-1,1,1,1)
     &  - HZ1(0) *HZ5(0,1,-1,1,1)
     &  - HZ1(0) *HZ5(0,1,1,-1,1)
     &  - HZ1(0) *HZ5(0,1,1,1,-1)
     &  - 3.3333333333333333d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,0,-1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(0,-1)*HZ2(0,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,0,1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(0,1,1)
     &  + HZ1(1) *HZ5(0,-1,0,1,1)
     &  + HZ2(0, -1)*HZ4(0,1,1,1)
     &  - HZ6(0, -1,0,1,1,1)
      HZ6(1,1,1,0,-1,1) = 
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,-1,1)
     &  - HZ1(1) *HZ1(1)*HZ4(0,-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,1,-1,1)
     &  + 3.0000000000000000d+00*HZ1(1)*HZ5(0,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,-1,1,1)
     &  + HZ1(1) *HZ5(0,1,1,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,-1,1,1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,1,-1,1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,1,-1,1,1)
     &  - HZ6(0,1,1,1, -1,1)
      HZ6(1,1,1,0,-1,-1) = 
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,-1,-1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,1,-1,-1)
     &  + HZ1(1) *HZ5(0,-1,-1,1,1)
     &  + HZ1(1) *HZ5(0,-1,1,-1,1)
     &  + HZ1(1) *HZ5(0,-1,1,1,-1)
     &  + HZ1(1) *HZ5(0,1,-1,-1,1)
     &  + HZ1(1) *HZ5(0,1,-1,1,-1)
     &  + HZ1(1) *HZ5(0,1,1,-1,-1)
     &  - HZ6(0, -1,-1,1,1,1)
     &  - HZ6(0, -1,1,-1,1,1)
     &  - HZ6(0, -1,1,1,-1,1)
     &  - HZ6(0, -1,1,1,1,-1)
     &  - HZ6(0,1, -1,-1,1,1)
     &  - HZ6(0,1, -1,1,-1,1)
     &  - HZ6(0,1, -1,1,1,-1)
     &  - HZ6(0,1,1, -1,-1,1)
     &  - HZ6(0,1,1, -1,1,-1)
     &  - HZ6(0,1,1,1, -1,-1)
      HZ6(1,1,1,1,0,0) = 
     &  + 2.0833333333333333d-02*HZ1(0)*HZ1(0)*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)
     &  - 1.6666666666666666d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(0,1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,1,1,1)
     &  + HZ1(0) *HZ5(0,1,1,1,1)
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,0,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,0,1,1)
     &  + HZ1(1) *HZ5(0,0,1,1,1)
     &  - HZ6(0,0,1,1,1,1) 
      HZ6(1,1,1,1,0,1) = 
     &  + 4.1666666666666666d-02*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(0,1)
     &  - 3.3333333333333333d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,1,1)
     &  + 1.5000000000000000d+00*HZ1(1)*HZ1(1)*HZ4(0,1,1,1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(0,1,1,1,1)
     &  + 5.0000000000000000d+00*HZ6(0,1,1,1,1,1)
      HZ6(1,1,1,1,0,-1) = 
     &  + 4.1666666666666666d-02*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(0,-1)
     &  - 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,-1,1)
     &  - 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,1,-1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,1,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,1,-1,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,1,1,-1)
     &  - HZ1(1) *HZ5(0,-1,1,1,1)
     &  - HZ1(1) *HZ5(0,1,-1,1,1)
     &  - HZ1(1) *HZ5(0,1,1,-1,1)
     &  - HZ1(1) *HZ5(0,1,1,1,-1)
     &  + HZ6(0, -1,1,1,1,1)
     &  + HZ6(0,1, -1,1,1,1)
     &  + HZ6(0,1,1, -1,1,1)
     &  + HZ6(0,1,1,1, -1,1)
     &  + HZ6(0,1,1,1,1, -1)
      HZ6(1,1,1,1,1,0) = 
     &  + 8.3333333333333333d-03*HZ1(0)*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)
     &  - 4.1666666666666666d-02*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(0,1)
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,1,1,1)
     &  + HZ1(1) *HZ5(0,1,1,1,1)
     &  - HZ6(0,1,1,1,1,1) 
      HZ6(1,1,1,1,1,1) = 
     &  + 1.3888888888888888d-03*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)
      HZ6(1,1,1,1,1,-1) = 
     &  + 8.3333333333333333d-03*HZ1(-1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)
     &  - 4.1666666666666666d-02*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(-1,1)
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(-1,1,1,1)
     &  + HZ1(1) *HZ5(-1,1,1,1,1)
     &  - HZ6( -1,1,1,1,1,1)
      HZ6(1,1,1,1,-1,0) = 
     &  + 4.1666666666666666d-02*HZ1(-1)*HZ1(0)*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)
     &  - 1.6666666666666666d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(-1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(-1,1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(-1,1,1,1)
     &  + HZ1(0) *HZ5(-1,1,1,1,1)
     &  - 4.1666666666666666d-02*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(0,-1)
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,1,1)
     &  + HZ1(1) *HZ5(0,-1,1,1,1)
     &  - HZ6(0, -1,1,1,1,1)
      HZ6(1,1,1,1,-1,1) = 
     &  + 4.1666666666666666d-02*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(-1,1)
     &  - 3.3333333333333333d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(-1,1,1)
     &  + 1.5000000000000000d+00*HZ1(1)*HZ1(1)*HZ4(-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(-1,1,1,1,1)
     &  + 5.0000000000000000d+00*HZ6(-1,1,1,1,1,1)
      HZ6(1,1,1,1,-1,-1) = 
     &  + 2.0833333333333333d-02*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ1(1)
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(-1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(-1,1,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(-1,1,1,1)
     &  + HZ1( -1)*HZ5(-1,1,1,1,1)
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(-1,-1,1,1)
     &  + HZ1(1) *HZ5(-1,-1,1,1,1)
     &  - HZ6( -1,-1,1,1,1,1)
      HZ6(1,1,1,-1,0,0) = 
     &  + 8.3333333333333333d-02*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(1)*HZ1(1)*HZ1(1)
     &  - 2.5000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ1(1)*HZ2(-1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(-1,1,1,1)
     &  - 1.6666666666666666d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(0,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(0,-1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,-1,1,1)
     &  + HZ1(0) *HZ5(0,-1,1,1,1)
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,0,-1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,0,-1,1)
     &  + HZ1(1) *HZ5(0,0,-1,1,1)
     &  - HZ6(0,0, -1,1,1,1)
      HZ6(1,1,1,-1,0,1) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(0,1)
     &  - 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,-1,1)
     &  - 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,1,-1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(-1,1)*HZ2(0,1)
     &  + HZ1(1) *HZ1(1)*HZ4(0,-1,1,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,1,-1,1)
     &  + HZ1(1) *HZ2(0,1)*HZ3(-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,-1,1,1,1)
     &  - HZ1(1) *HZ5(0,1,-1,1,1)
     &  - HZ2(0,1) *HZ4(-1,1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,-1,1,1,1,1)
     &  + HZ6(0,1, -1,1,1,1)
      HZ6(1,1,1,-1,0,-1) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(0,-1)
     &  - 3.3333333333333333d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,-1,-1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(-1,1)*HZ2(0,-1)
     &  + HZ1(1) *HZ1(1)*HZ4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,1,-1)
     &  + HZ1(1) *HZ2(0,-1)*HZ3(-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,1,1)
     &  - HZ1(1) *HZ5(0,-1,1,-1,1)
     &  - HZ1(1) *HZ5(0,-1,1,1,-1)
     &  - HZ2(0, -1)*HZ4(-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,1,1,1)
     &  + HZ6(0, -1,1,-1,1,1)
     &  + HZ6(0, -1,1,1,-1,1)
     &  + HZ6(0, -1,1,1,1,-1)
      HZ6(1,1,1,-1,1,0) = 
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(0,1)
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(-1,1)
     &  - HZ1(0) *HZ1(1)*HZ1(1)*HZ3(-1,1,1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(-1,1,1,1,1)
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,1,-1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(-1,1)*HZ2(0,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,1,-1,1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(-1,1,1)
     &  + HZ1(1) *HZ5(0,1,-1,1,1)
     &  + HZ2(0,1) *HZ4(-1,1,1,1)
     &  - HZ6(0,1, -1,1,1,1)
      HZ6(1,1,1,-1,1,1) = 
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(-1,1,1)
     &  - 1.5000000000000000d+00*HZ1(1)*HZ1(1)*HZ4(-1,1,1,1)
     &  + 6.0000000000000000d+00*HZ1(1)*HZ5(-1,1,1,1,1)
     &  - 1.0000000000000000d+01*HZ6(-1,1,1,1,1,1)
      HZ6(1,1,1,-1,1,-1) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(-1,1)
     &  - HZ1( -1)*HZ1(1)*HZ1(1)*HZ3(-1,1,1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(-1,1,1,1,1)
     &  - 3.3333333333333333d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(-1,-1,1)
     &  + 2.5000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(-1,1)*HZ2(-1,1)
     &  + HZ1(1) *HZ1(1)*HZ4(-1,-1,1,1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(-1,1,1)
     &  + HZ1(1) *HZ5(-1,1,-1,1,1)
     &  + HZ2( -1,1)*HZ4(-1,1,1,1)
     &  - HZ6( -1,1,-1,1,1,1)
      HZ6(1,1,1,-1,-1,0) = 
     &  + 8.3333333333333333d-02*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(1)*HZ1(1)*HZ1(1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(1)*HZ1(1)*HZ2(-1,1)
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(-1,1,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(-1,1,1,1)
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ1(1)*HZ2(0,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(-1,-1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(-1,-1,1,1)
     &  + HZ1(0) *HZ5(-1,-1,1,1,1)
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(0,-1,-1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(-1,1)*HZ2(0,-1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,-1,1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(-1,1,1)
     &  + HZ1(1) *HZ5(0,-1,-1,1,1)
     &  + HZ2(0, -1)*HZ4(-1,1,1,1)
     &  - HZ6(0, -1,-1,1,1,1)
      HZ6(1,1,1,-1,-1,1) = 
     &  + 1.6666666666666666d-01*HZ1(1)*HZ1(1)*HZ1(1)*HZ3(-1,-1,1)
     &  - 2.5000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(-1,1)*HZ2(-1,1)
     &  + HZ1(1) *HZ2(-1,1)*HZ3(-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(-1,-1,1,1,1)
     &  - HZ1(1) *HZ5(-1,1,-1,1,1)
     &  - HZ2( -1,1)*HZ4(-1,1,1,1)
     &  + 4.0000000000000000d+00*HZ6(-1,-1,1,1,1,1)
     &  + HZ6( -1,1,-1,1,1,1)
      HZ6(1,1,1,-1,-1,-1) = 
     &  + 2.7777777777777777d-02*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ1(1)*HZ1(1)
     &  - 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ1(1)*HZ2(-1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(-1,1,1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(-1,-1,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(-1,-1,1,1)
     &  + HZ1( -1)*HZ5(-1,-1,1,1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(-1,-1,-1,1)
     &  + HZ1(1) *HZ5(-1,-1,-1,1,1)
     &  - HZ6( -1,-1,-1,1,1,1)
      HZ6(1,1,-1,0,0,0) = 
     &  + 8.3333333333333333d-02*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(1)*HZ1(1)
     &  - 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(1)*HZ2(-1,1)
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(-1,1,1)
     &  - 2.5000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ1(1)*HZ2(0,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(0,0,-1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,0,-1,1)
     &  + HZ1(0) *HZ5(0,0,-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,0,0,-1)
     &  + HZ1(1) *HZ5(0,0,0,-1,1)
     &  - HZ6(0,0,0, -1,1,1)
      HZ6(1,1,-1,0,0,1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(0,0,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,0,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,0,1,-1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(0,0,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,0,1,1)
     &  + HZ1(1) *HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,1,1)
     &  + HZ1(1) *HZ5(0,0,1,-1,1)
     &  + HZ3( -1,1,1)*HZ3(0,0,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  - HZ6(0, -1,1,1,0,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
     &  - HZ6(0,0,1, -1,1,1)
      HZ6(1,1,-1,0,0,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(0,0,-1)
     &  - 2.5000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(0,-1)*HZ2(0,-1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(0,0,-1)
     &  + HZ1(1) *HZ2(0,-1)*HZ3(0,-1,1)
     &  - HZ1(1) *HZ5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,-1,1)
     &  - HZ1(1) *HZ5(0,0,-1,1,-1)
     &  - HZ2(0, -1)*HZ4(0,-1,1,1)
     &  + HZ3( -1,1,1)*HZ3(0,0,-1)
     &  + HZ6(0, -1,0,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  + HZ6(0,0, -1,1,-1,1)
     &  + HZ6(0,0, -1,1,1,-1)
      HZ6(1,1,-1,0,1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(1)*HZ1(1)*HZ2(0,1)
     &  - HZ1( -1)*HZ1(1)*HZ1(1)*HZ3(0,0,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(0,1,-1)
     &  - HZ1(0) *HZ1(1)*HZ2(-1,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,-1,1,1)
     &  + HZ1(0) *HZ1(1)*HZ4(0,1,-1,1)
     &  + HZ1(0) *HZ2(0,1)*HZ3(-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,-1,1,1,1)
     &  - HZ1(0) *HZ5(0,1,-1,1,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,0,1)
     &  + HZ1(1) *HZ1(1)*HZ4(0,0,-1,1)
     &  + HZ1(1) *HZ1(1)*HZ4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ2(-1,1)*HZ3(0,0,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,0,1,1)
     &  - HZ1(1) *HZ5(0,-1,1,0,1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,0,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  + HZ6(0, -1,1,1,0,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,1,1)
      HZ6(1,1,-1,0,1,1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(0,1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,1,1,-1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(0,1,1)
     &  + 3.0000000000000000d+00*HZ1(1)*HZ5(0,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,-1,1,1)
     &  + HZ1(1) *HZ5(0,1,1,-1,1)
     &  + HZ3( -1,1,1)*HZ3(0,1,1)
     &  - 6.0000000000000000d+00*HZ6(0,-1,1,1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,1,-1,1,1,1)
     &  - HZ6(0,1,1, -1,1,1)
      HZ6(1,1,-1,0,1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(0,1,-1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,1,-1)
     &  - HZ1(1) *HZ1(1)*HZ4(0,1,-1,-1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(0,1,-1)
     &  + HZ1(1) *HZ5(0,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,-1,-1,1)
     &  + HZ1(1) *HZ5(0,1,-1,1,-1)
     &  + HZ3( -1,1,1)*HZ3(0,1,-1)
     &  - HZ6(0, -1,1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,1,1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,-1,1,1)
     &  - HZ6(0,1, -1,1,-1,1)
     &  - HZ6(0,1, -1,1,1,-1)
      HZ6(1,1,-1,0,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(1)*HZ1(1)*HZ2(0,-1)
     &  - HZ1( -1)*HZ1(1)*HZ1(1)*HZ3(0,0,-1)
     &  - HZ1(0) *HZ1(1)*HZ1(1)*HZ3(0,-1,-1)
     &  - HZ1(0) *HZ1(1)*HZ2(-1,1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,-1,-1,1)
     &  + HZ1(0) *HZ1(1)*HZ4(0,-1,1,-1)
     &  + HZ1(0) *HZ2(0,-1)*HZ3(-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,1,1)
     &  - HZ1(0) *HZ5(0,-1,1,-1,1)
     &  - HZ1(0) *HZ5(0,-1,1,1,-1)
     &  + 2.5000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(0,-1)*HZ2(0,-1)
     &  + HZ1(1) *HZ1(1)*HZ4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ2(-1,1)*HZ3(0,0,-1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(0,-1,1)
     &  + HZ1(1) *HZ5(0,-1,0,-1,1)
     &  + HZ2(0, -1)*HZ4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,0,-1)
     &  - HZ6(0, -1,0,-1,1,1)
      HZ6(1,1,-1,0,-1,1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(0,-1,1)
     &  - HZ1(1) *HZ1(1)*HZ4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,1,-1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(0,-1,1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,1,-1,1)
     &  + HZ3( -1,1,1)*HZ3(0,-1,1)
     &  - 6.0000000000000000d+00*HZ6(0,-1,-1,1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,1,-1,1,1)
     &  - HZ6(0, -1,1,1,-1,1)
      HZ6(1,1,-1,0,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(0,-1,-1)
     &  - 1.5000000000000000d+00*HZ1(1)*HZ1(1)*HZ4(0,-1,-1,-1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(0,-1,-1)
     &  + 3.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,1,-1)
     &  + HZ1(1) *HZ5(0,-1,1,-1,-1)
     &  + HZ3( -1,1,1)*HZ3(0,-1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,1,1,-1)
     &  - HZ6(0, -1,1,-1,-1,1)
     &  - HZ6(0, -1,1,-1,1,-1)
     &  - HZ6(0, -1,1,1,-1,-1)
      HZ6(1,1,-1,1,0,0) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(1)*HZ1(1)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(0,0,1)
     &  + 2.5000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ1(1)*HZ2(-1,1)
     &  - HZ1(0) *HZ1(0)*HZ1(1)*HZ3(-1,1,1)
     &  + 1.5000000000000000d+00*HZ1(0)*HZ1(0)*HZ4(-1,1,1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(0,1,-1)
     &  + HZ1(0) *HZ1(1)*HZ2(-1,1)*HZ2(0,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,1,-1,1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(-1,1,1)
     &  + HZ1(0) *HZ5(0,1,-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,0,1,-1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(0,0,1)
     &  + HZ1(1) *HZ5(0,0,1,-1,1)
     &  + HZ3( -1,1,1)*HZ3(0,0,1)
     &  - HZ6(0,0,1, -1,1,1)
      HZ6(1,1,-1,1,0,1) = 
     &  - HZ1( -1)*HZ1(1)*HZ1(1)*HZ3(0,1,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(-1,1)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,1,-1,1)
     &  + HZ1(1) *HZ1(1)*HZ4(0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ2(-1,1)*HZ3(0,1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ2(0,1)*HZ3(-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,1,-1,1)
     &  + 3.0000000000000000d+00*HZ2(0,1)*HZ4(-1,1,1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,1,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,1,-1,1,1)
      HZ6(1,1,-1,1,0,-1) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(0,1,-1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(-1,1)*HZ2(0,-1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,1,-1)
     &  + HZ1(1) *HZ1(1)*HZ4(0,1,-1,-1)
     &  + HZ1(1) *HZ2(-1,1)*HZ3(0,-1,1)
     &  + HZ1(1) *HZ2(-1,1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ2(0,-1)*HZ3(-1,1,1)
     &  - HZ1(1) *HZ5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,-1,-1,1)
     &  - HZ1(1) *HZ5(0,1,-1,1,-1)
     &  + 3.0000000000000000d+00*HZ2(0,-1)*HZ4(-1,1,1,1)
     &  - HZ3( -1,1,1)*HZ3(0,-1,1)
     &  - HZ3( -1,1,1)*HZ3(0,1,-1)
     &  + HZ6(0, -1,1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,-1,1,1)
     &  + HZ6(0,1, -1,1,-1,1)
     &  + HZ6(0,1, -1,1,1,-1)
      HZ6(1,1,-1,1,1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(0,1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(-1,1,1,1)
     &  + 6.0000000000000000d+00*HZ1(0)*HZ5(-1,1,1,1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(-1,1)*HZ2(0,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,1,1,-1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(0,1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ2(0,1)*HZ3(-1,1,1)
     &  + HZ1(1) *HZ5(0,1,1,-1,1)
     &  - 3.0000000000000000d+00*HZ2(0,1)*HZ4(-1,1,1,1)
     &  + HZ3( -1,1,1)*HZ3(0,1,1)
     &  - HZ6(0,1,1, -1,1,1)
      HZ6(1,1,-1,1,1,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(-1,1,1,1,1)
     &  + 1.0000000000000000d+01*HZ6(-1,1,1,1,1,1)
      HZ6(1,1,-1,1,1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(-1,1,1,1)
     &  + 6.0000000000000000d+00*HZ1(-1)*HZ5(-1,1,1,1,1)
     &  - 2.5000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(-1,1)*HZ2(-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ2(-1,1)*HZ3(-1,1,1)
     &  - 6.0000000000000000d+00*HZ1(1)*HZ5(-1,-1,1,1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(-1,1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ2(-1,1)*HZ4(-1,1,1,1)
     &  + 5.0000000000000000d-01*HZ3(-1,1,1)*HZ3(-1,1,1)
     &  + 6.0000000000000000d+00*HZ6(-1,-1,1,1,1,1)
     &  + 3.0000000000000000d+00*HZ6(-1,1,-1,1,1,1)
      HZ6(1,1,-1,1,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(1)*HZ1(1)*HZ2(-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ1(1)*HZ3(-1,1,1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(-1,1,1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(0,-1,1)
     &  - HZ1(0) *HZ1(1)*HZ1(1)*HZ3(-1,-1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ2(-1,1)*HZ2(-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(-1,-1,1,1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(-1,1,1)
     &  + HZ1(0) *HZ5(-1,1,-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(-1,1)*HZ2(0,-1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,1,-1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ2(0,-1)*HZ3(-1,1,1)
     &  + HZ1(1) *HZ5(0,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HZ2(0,-1)*HZ4(-1,1,1,1)
     &  + HZ3( -1,1,1)*HZ3(0,-1,1)
     &  - HZ6(0, -1,1,-1,1,1)
      HZ6(1,1,-1,1,-1,1) = 
     &  + 2.5000000000000000d-01*HZ1(1)*HZ1(1)*HZ2(-1,1)*HZ2(-1,1)
     &  - HZ1(1) *HZ1(1)*HZ4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ2(-1,1)*HZ3(-1,1,1)
     &  + 1.2000000000000000d+01*HZ1(1)*HZ5(-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(-1,1,-1,1,1)
     &  + 3.0000000000000000d+00*HZ2(-1,1)*HZ4(-1,1,1,1)
     &  - HZ3( -1,1,1)*HZ3(-1,1,1)
     &  - 1.2000000000000000d+01*HZ6(-1,-1,1,1,1,1)
     &  - 3.0000000000000000d+00*HZ6(-1,1,-1,1,1,1)
      HZ6(1,1,-1,1,-1,-1) = 
     &  + 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ1(1)*HZ2(-1,1)
     &  - HZ1( -1)*HZ1(-1)*HZ1(1)*HZ3(-1,1,1)
     &  + 1.5000000000000000d+00*HZ1(-1)*HZ1(-1)*HZ4(-1,1,1,1)
     &  - HZ1( -1)*HZ1(1)*HZ1(1)*HZ3(-1,-1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ2(-1,1)*HZ2(-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(-1,-1,1,1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(-1,1,1)
     &  + HZ1( -1)*HZ5(-1,1,-1,1,1)
     &  + 1.5000000000000000d+00*HZ1(1)*HZ1(1)*HZ4(-1,-1,-1,1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(-1,-1,1)
     &  + HZ1(1) *HZ5(-1,-1,1,-1,1)
     &  + HZ3( -1,-1,1)*HZ3(-1,1,1)
     &  - HZ6( -1,-1,1,-1,1,1)
      HZ6(1,1,-1,-1,0,0) = 
     &  + 1.2500000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(1)*HZ1(1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(1)*HZ2(-1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(1)*HZ1(1)*HZ2(0,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(0,0,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(-1,-1,1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(0,-1,-1)
     &  + HZ1(0) *HZ1(1)*HZ2(-1,1)*HZ2(0,-1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,-1,-1,1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(-1,1,1)
     &  + HZ1(0) *HZ5(0,-1,-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,0,-1,-1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(0,0,-1)
     &  + HZ1(1) *HZ5(0,0,-1,-1,1)
     &  + HZ3( -1,1,1)*HZ3(0,0,-1)
     &  - HZ6(0,0, -1,-1,1,1)
      HZ6(1,1,-1,-1,0,1) = 
     &  + 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ1(1)*HZ2(0,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(0,1,-1)
     &  - HZ1( -1)*HZ1(1)*HZ2(-1,1)*HZ2(0,1)
     &  + HZ1( -1)*HZ2(0,1)*HZ3(-1,1,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,1,-1)
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,1,-1,-1)
     &  + HZ1(1) *HZ2(-1,1)*HZ3(0,-1,1)
     &  + HZ1(1) *HZ2(-1,1)*HZ3(0,1,-1)
     &  + HZ1(1) *HZ2(0,1)*HZ3(-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,1,1)
     &  - HZ1(1) *HZ5(0,-1,1,-1,1)
     &  - HZ1(1) *HZ5(0,1,-1,-1,1)
     &  - HZ2(0,1) *HZ4(-1,-1,1,1)
     &  - HZ3( -1,1,1)*HZ3(0,-1,1)
     &  - HZ3( -1,1,1)*HZ3(0,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,-1,1,1,1)
     &  + HZ6(0, -1,1,-1,1,1)
     &  + HZ6(0,1, -1,-1,1,1)
      HZ6(1,1,-1,-1,0,-1) = 
     &  + 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ1(1)*HZ2(0,-1)
     &  - HZ1( -1)*HZ1(1)*HZ1(1)*HZ3(0,-1,-1)
     &  - HZ1( -1)*HZ1(1)*HZ2(-1,1)*HZ2(0,-1)
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(-1,1,1)
     &  + 1.5000000000000000d+00*HZ1(1)*HZ1(1)*HZ4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ2(-1,1)*HZ3(0,-1,-1)
     &  + HZ1(1) *HZ2(0,-1)*HZ3(-1,-1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,-1,1)
     &  - HZ1(1) *HZ5(0,-1,-1,1,-1)
     &  - HZ2(0, -1)*HZ4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,-1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,-1,-1,1,1)
     &  + HZ6(0, -1,-1,1,-1,1)
     &  + HZ6(0, -1,-1,1,1,-1)
      HZ6(1,1,-1,-1,1,0) = 
     &  - 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ1(1)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(0,1,-1)
     &  + HZ1( -1)*HZ1(1)*HZ2(-1,1)*HZ2(0,1)
     &  - HZ1( -1)*HZ2(0,1)*HZ3(-1,1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ1(1)*HZ3(-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ2(-1,1)*HZ2(-1,1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(-1,-1,1,1,1)
     &  - HZ1(0) *HZ5(-1,1,-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,1,-1,-1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(0,1,-1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(-1,-1,1)
     &  + HZ1(1) *HZ5(0,1,-1,-1,1)
     &  + HZ2(0,1) *HZ4(-1,-1,1,1)
     &  + HZ3( -1,1,1)*HZ3(0,1,-1)
     &  - HZ6(0,1, -1,-1,1,1)
      HZ6(1,1,-1,-1,1,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(-1,-1,1,1,1)
     &  - HZ1(1) *HZ5(-1,1,-1,1,1)
     &  + 5.0000000000000000d-01*HZ3(-1,1,1)*HZ3(-1,1,1)
      HZ6(1,1,-1,-1,1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ2(-1,1)*HZ2(-1,1)
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(-1,-1,1,1,1)
     &  - HZ1( -1)*HZ5(-1,1,-1,1,1)
     &  - 1.5000000000000000d+00*HZ1(1)*HZ1(1)*HZ4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ2(-1,1)*HZ3(-1,-1,1)
     &  - 6.0000000000000000d+00*HZ1(1)*HZ5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(-1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(-1,1,1)
     &  + 9.0000000000000000d+00*HZ6(-1,-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HZ6(-1,-1,1,-1,1,1)
     &  + HZ6( -1,-1,1,1,-1,1)
      HZ6(1,1,-1,-1,-1,0) = 
     &  + 8.3333333333333333d-02*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(1)*HZ1(1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(1)*HZ2(-1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(-1,1,1)
     &  - 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ1(1)*HZ2(0,-1)
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(-1,-1,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(-1,-1,1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ1(1)*HZ3(0,-1,-1)
     &  + HZ1( -1)*HZ1(1)*HZ2(-1,1)*HZ2(0,-1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(-1,1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(-1,-1,-1,1)
     &  + HZ1(0) *HZ5(-1,-1,-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(0,-1,-1,-1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(0,-1,-1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(-1,-1,1)
     &  + HZ1(1) *HZ5(0,-1,-1,-1,1)
     &  + HZ2(0, -1)*HZ4(-1,-1,1,1)
     &  + HZ3( -1,1,1)*HZ3(0,-1,-1)
     &  - HZ6(0, -1,-1,-1,1,1)
      HZ6(1,1,-1,-1,-1,1) = 
     &  + 5.0000000000000000d-01*HZ1(1)*HZ1(1)*HZ4(-1,-1,-1,1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(-1,-1,1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(-1,-1,1,-1,1)
     &  + HZ3( -1,-1,1)*HZ3(-1,1,1)
     &  - 6.0000000000000000d+00*HZ6(-1,-1,-1,1,1,1)
     &  - 3.0000000000000000d+00*HZ6(-1,-1,1,-1,1,1)
     &  - HZ6( -1,-1,1,1,-1,1)
      HZ6(1,1,-1,-1,-1,-1) = 
     &  + 2.0833333333333333d-02*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ1(1)
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ2(-1,1)
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(-1,1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(-1,-1,1,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(-1,-1,-1,1)
     &  + HZ1( -1)*HZ5(-1,-1,-1,1,1)
     &  + HZ1(1) *HZ5(-1,-1,-1,-1,1)
     &  - HZ6( -1,-1,-1,-1,1,1)
      HZ6(1,-1,0,0,0,0) = 
     &  + 4.1666666666666666d-02*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(1)
     &  - 4.1666666666666666d-02*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)*HZ2(-1,1)
     &  - 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(1)*HZ2(0,-1)
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,-1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(0,0,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,0,-1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,0,0,-1)
     &  + HZ1(0) *HZ5(0,0,0,-1,1)
     &  + HZ1(1) *HZ5(0,0,0,0,-1)
     &  - HZ6(0,0,0,0, -1,1)
      HZ6(1,-1,0,0,0,1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(0,0,0,1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(0,0,1)
     &  + HZ1(1) *HZ5(0,0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,1,-1)
     &  + HZ1(1) *HZ5(0,0,1,0,-1)
     &  - HZ2( -1,1)*HZ4(0,0,0,1)
     &  + HZ3(0, -1,1)*HZ3(0,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,0,1,1)
     &  - HZ6(0,0, -1,1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  - HZ6(0,0,1,0, -1,1)
      HZ6(1,-1,0,0,0,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(0,0,0,-1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(0,0,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,0,-1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,-1,-1)
     &  - HZ2( -1,1)*HZ4(0,0,0,-1)
     &  + HZ3(0, -1,1)*HZ3(0,0,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,0,-1,1)
     &  - HZ6(0,0, -1,0,1,-1)
     &  - HZ6(0,0, -1,1,0,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
      HZ6(1,-1,0,0,1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(0,0,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,0,0,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,-1,0,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,0,-1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,0,1,-1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(0,0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,0,1,1)
     &  + HZ1(0) *HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,1,1)
     &  + HZ1(0) *HZ5(0,0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ2(0,-1)*HZ3(0,0,1)
     &  - HZ1(1) *HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,0,-1)
     &  + 3.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,0,1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,1)*HZ3(0,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,0,1,1)
     &  + HZ6(0,0, -1,1,0,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,-1,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,0,-1,1)
      HZ6(1,-1,0,0,1,1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(0,0,1,1)
     &  - HZ1(1) *HZ5(0,-1,0,1,1)
     &  - HZ1(1) *HZ5(0,0,-1,1,1)
     &  - HZ1(1) *HZ5(0,0,1,-1,1)
     &  - HZ1(1) *HZ5(0,0,1,1,-1)
     &  - HZ2( -1,1)*HZ4(0,0,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  + HZ6(0, -1,1,0,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,1,1)
     &  + HZ6(0,0,1,1, -1,1)
      HZ6(1,-1,0,0,1,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(0,0,1,-1)
     &  - HZ1(1) *HZ5(0,-1,0,1,-1)
     &  - HZ1(1) *HZ5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,-1,-1)
     &  - HZ2( -1,1)*HZ4(0,0,1,-1)
     &  + HZ6(0, -1,0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,1,1,-1)
     &  + HZ6(0, -1,1,0,1,-1)
     &  + HZ6(0,0, -1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,-1,1)
     &  + HZ6(0,0,1, -1,1,-1)
      HZ6(1,-1,0,0,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(0,0,-1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ2(0,-1)*HZ2(0,-1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(0,0,-1)
     &  + HZ1(0) *HZ2(0,-1)*HZ3(0,-1,1)
     &  - HZ1(0) *HZ5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,-1,1)
     &  - HZ1(0) *HZ5(0,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ2(0,-1)*HZ3(0,0,-1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HZ1(1)*HZ5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,1)*HZ3(0,0,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,-1,0,-1,1)
     &  + HZ6(0,0, -1,0,1,-1)
     &  + HZ6(0,0, -1,1,0,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,-1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
      HZ6(1,-1,0,0,-1,1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(0,0,-1,1)
     &  - HZ1(1) *HZ5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,-1,1)
     &  - HZ1(1) *HZ5(0,0,-1,1,-1)
     &  - HZ2( -1,1)*HZ4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
      HZ6(1,-1,0,0,-1,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(0,0,-1,-1)
     &  - HZ1(1) *HZ5(0,-1,0,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,-1,-1)
     &  - HZ2( -1,1)*HZ4(0,0,-1,-1)
     &  + HZ3(0, -1,-1)*HZ3(0,-1,1)
     &  - HZ6(0, -1,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  - HZ6(0, -1,0,-1,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
     &  - HZ6(0,0, -1,1,-1,-1)
      HZ6(1,-1,0,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ1(1)*HZ3(0,0,1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,0,0,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(0,1,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(-1,1)*HZ2(0,1)
     &  + HZ1(0) *HZ1(0)*HZ4(0,-1,1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,1,-1,1)
     &  + HZ1(0) *HZ1(1)*HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ2(-1,1)*HZ3(0,0,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,0,1,1)
     &  - HZ1(0) *HZ5(0,-1,1,0,1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,-1,1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(0,0,1)
     &  + HZ1(1) *HZ5(0,0,1,0,-1)
     &  - 3.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,0,1)
     &  + HZ3(0, -1,1)*HZ3(0,0,1)
     &  - HZ6(0,0,1,0, -1,1)
      HZ6(1,-1,0,1,0,1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ2(0,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,0,1,1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(0,1,-1)
     &  - HZ1(1) *HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,-1,1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,1,-1)
     &  + HZ1(1) *HZ5(0,1,0,1,-1)
     &  - 5.0000000000000000d-01*HZ2(-1,1)*HZ2(0,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,1,1)
     &  + HZ2(0,1) *HZ4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,-1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  - HZ6(0,1,0,1, -1,1)
      HZ6(1,-1,0,1,0,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ2(0,-1)*HZ2(0,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,0,1,-1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(0,-1,1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,-1,-1)
     &  - HZ2( -1,1)*HZ2(0,-1)*HZ2(0,1)
     &  + HZ2( -1,1)*HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,-1,1,1)
     &  + HZ2(0, -1)*HZ4(0,1,-1,1)
     &  - 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,1,1,-1)
     &  - HZ6(0, -1,1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,1,-1)
      HZ6(1,-1,0,1,1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(0,1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ2(0,1)*HZ2(0,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,-1,1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,1,-1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,1,1,-1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(0,1,1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ5(0,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,-1,1,1)
     &  + HZ1(0) *HZ5(0,1,1,-1,1)
     &  + HZ1(1) *HZ2(0,1)*HZ3(0,1,-1)
     &  + HZ1(1) *HZ5(0,-1,0,1,1)
     &  + HZ1(1) *HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,1,-1)
     &  - HZ1(1) *HZ5(0,1,0,1,-1)
     &  + 5.0000000000000000d-01*HZ2(-1,1)*HZ2(0,1)*HZ2(0,1)
     &  - HZ2(0,1) *HZ4(0,1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,1,0,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  + HZ6(0,1,0,1, -1,1)
      HZ6(1,-1,0,1,1,1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(0,1,1,1)
     &  - HZ1(1) *HZ5(0,-1,1,1,1)
     &  - HZ1(1) *HZ5(0,1,-1,1,1)
     &  - HZ1(1) *HZ5(0,1,1,-1,1)
     &  - HZ1(1) *HZ5(0,1,1,1,-1)
     &  - HZ2( -1,1)*HZ4(0,1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,-1,1,1,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,1,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,1,-1,1,1)
     &  + HZ6(0,1,1,1, -1,1)
      HZ6(1,-1,0,1,1,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(0,1,1,-1)
     &  - HZ1(1) *HZ5(0,-1,1,1,-1)
     &  - HZ1(1) *HZ5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,1,-1,-1)
     &  - HZ2( -1,1)*HZ4(0,1,1,-1)
     &  + HZ6(0, -1,1,1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,1,1,1,-1)
     &  + HZ6(0,1, -1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,1,1,-1,-1,1)
     &  + HZ6(0,1,1, -1,1,-1)
      HZ6(1,-1,0,1,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(0,1,-1)
     &  - HZ1( -1)*HZ1(1)*HZ2(0,-1)*HZ2(0,1)
     &  + HZ1( -1)*HZ1(1)*HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,0,-1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,1,-1,-1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(0,1,-1)
     &  + HZ1(0) *HZ5(0,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,-1,-1,1)
     &  + HZ1(0) *HZ5(0,1,-1,1,-1)
     &  + HZ1(1) *HZ2(0,-1)*HZ3(0,-1,1)
     &  + HZ1(1) *HZ2(0,-1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,0,-1,1)
     &  - HZ1(1) *HZ5(0,-1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,1,-1)
     &  + HZ2( -1,1)*HZ2(0,-1)*HZ2(0,1)
     &  - HZ2( -1,1)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,-1,1,1)
     &  - HZ2(0, -1)*HZ4(0,1,-1,1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  + HZ6(0, -1,0,1,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,1)
      HZ6(1,-1,0,1,-1,1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(0,1,-1,1)
     &  - HZ1(1) *HZ5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,-1,-1,1)
     &  - HZ1(1) *HZ5(0,1,-1,1,-1)
     &  - HZ2( -1,1)*HZ4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,1,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,1,-1,1)
      HZ6(1,-1,0,1,-1,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(0,1,-1,-1)
     &  - HZ1(1) *HZ5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,1,-1,-1,-1)
     &  - HZ2( -1,1)*HZ4(0,1,-1,-1)
     &  + HZ6(0, -1,1,-1,-1,1)
     &  + HZ6(0, -1,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,-1,1,-1)
     &  + HZ6(0,1, -1,1,-1,-1)
      HZ6(1,-1,0,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(1)*HZ2(0,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ1(1)*HZ3(0,0,-1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,0,0,-1)
     &  - HZ1(0) *HZ1(0)*HZ1(1)*HZ3(0,-1,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(-1,1)*HZ2(0,-1)
     &  + HZ1(0) *HZ1(0)*HZ4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,1,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ2(0,-1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ2(-1,1)*HZ3(0,0,-1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(0,-1,1)
     &  + HZ1(0) *HZ5(0,-1,0,-1,1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(0,0,-1)
     &  + HZ1(1) *HZ5(0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,0,-1)
     &  + HZ3(0, -1,1)*HZ3(0,0,-1)
     &  - HZ6(0,0, -1,0,-1,1)
      HZ6(1,-1,0,-1,0,1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,0,1)
     &  - HZ1(1) *HZ5(0,-1,0,-1,1)
     &  - HZ1(1) *HZ5(0,-1,0,1,-1)
     &  - HZ2( -1,1)*HZ4(0,-1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,-1,-1,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  + HZ6(0, -1,0,1,-1,1)
     &  + HZ6(0, -1,1,-1,0,1)
      HZ6(1,-1,0,-1,0,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ2(0,-1)*HZ2(0,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ2(0,-1)*HZ3(0,-1,-1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,-1,0,-1,-1)
     &  + 1.2000000000000000d+01*HZ1(1)*HZ5(0,0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HZ2(-1,1)*HZ2(0,-1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,-1,-1,1)
     &  + HZ2(0, -1)*HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  + HZ6(0, -1,0,-1,1,-1)
      HZ6(1,-1,0,-1,1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,-1,-1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,-1,1,-1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(0,-1,1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,0,-1,1)
     &  + HZ1(1) *HZ5(0,-1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,1,-1)
     &  + HZ2( -1,1)*HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,-1,1)
     &  - 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,-1,-1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  - HZ6(0, -1,0,1,-1,1)
     &  - HZ6(0, -1,1,-1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,1)
      HZ6(1,-1,0,-1,1,1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,1,1)
     &  - HZ1(1) *HZ5(0,-1,1,-1,1)
     &  - HZ1(1) *HZ5(0,-1,1,1,-1)
     &  - HZ2( -1,1)*HZ4(0,-1,1,1)
     &  + 6.0000000000000000d+00*HZ6(0,-1,-1,1,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,1,-1,1,1)
     &  + HZ6(0, -1,1,1,-1,1)
      HZ6(1,-1,0,-1,1,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,1,-1,-1)
     &  - HZ2( -1,1)*HZ4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,1,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,-1,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,-1,1,-1)
      HZ6(1,-1,0,-1,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(0,-1,-1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ2(0,-1)*HZ2(0,-1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,-1,-1,-1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(0,-1,-1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,1,-1)
     &  + HZ1(0) *HZ5(0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ2(0,-1)*HZ3(0,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,-1,0,-1,-1)
     &  - 6.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HZ2(-1,1)*HZ2(0,-1)*HZ2(0,-1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,-1,-1,1)
     &  - HZ2(0, -1)*HZ4(0,-1,1,-1)
     &  + HZ3(0, -1,-1)*HZ3(0,-1,1)
     &  - HZ6(0, -1,-1,0,-1,1)
      HZ6(1,-1,0,-1,-1,1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(0,-1,-1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,-1,1)
     &  - HZ1(1) *HZ5(0,-1,-1,1,-1)
     &  - HZ2( -1,1)*HZ4(0,-1,-1,1)
     &  + 6.0000000000000000d+00*HZ6(0,-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,-1,1,-1,1)
     &  + HZ6(0, -1,1,-1,-1,1)
      HZ6(1,-1,0,-1,-1,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,-1,-1)
     &  - HZ2( -1,1)*HZ4(0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,1,-1,-1)
     &  + HZ6(0, -1,1,-1,-1,-1)
      HZ6(1,-1,1,0,0,0) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(1)*HZ2(0,1)
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(0,0,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,0,0,1)
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(1)*HZ2(-1,1)
     &  - 3.3333333333333333d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(-1,1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(0,1,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(-1,1)*HZ2(0,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,1,-1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,0,1,-1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(0,0,1)
     &  + HZ1(0) *HZ5(0,0,1,-1,1)
     &  + HZ1(1) *HZ5(0,0,0,1,-1)
     &  + HZ2( -1,1)*HZ4(0,0,0,1)
     &  - HZ6(0,0,0,1, -1,1)
      HZ6(1,-1,1,0,0,1) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ2(0,1)*HZ2(0,1)
     &  + HZ1(1) *HZ2(-1,1)*HZ3(0,0,1)
     &  + HZ1(1) *HZ2(0,1)*HZ3(0,1,-1)
     &  - HZ1(1) *HZ5(0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,1,-1)
     &  - HZ1(1) *HZ5(0,1,0,1,-1)
     &  + 5.0000000000000000d-01*HZ2(-1,1)*HZ2(0,1)*HZ2(0,1)
     &  - HZ2(0,1) *HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  + HZ6(0,1,0,1, -1,1)
      HZ6(1,-1,1,0,0,-1) = 
     &  - HZ1( -1)*HZ1(1)*HZ2(0,-1)*HZ2(0,1)
     &  + HZ1( -1)*HZ1(1)*HZ4(0,-1,0,1)
     &  + HZ1( -1)*HZ1(1)*HZ4(0,0,-1,1)
     &  + HZ1( -1)*HZ1(1)*HZ4(0,0,1,-1)
     &  + HZ1(1) *HZ2(-1,1)*HZ3(0,0,-1)
     &  + HZ1(1) *HZ2(0,-1)*HZ3(0,1,-1)
     &  - HZ1(1) *HZ5(0,-1,0,1,-1)
     &  - HZ1(1) *HZ5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,-1,-1)
     &  + HZ2( -1,1)*HZ2(0,-1)*HZ2(0,1)
     &  - HZ2( -1,1)*HZ4(0,-1,0,1)
     &  - HZ2( -1,1)*HZ4(0,0,-1,1)
     &  - HZ2( -1,1)*HZ4(0,0,1,-1)
     &  - HZ2(0, -1)*HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,0,-1)
     &  + HZ6(0, -1,0,1,-1,1)
     &  + HZ6(0,0, -1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,-1,1)
     &  + HZ6(0,0,1, -1,1,-1)
      HZ6(1,-1,1,0,1,0) = 
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ1(1)*HZ3(0,1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ2(0,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,0,1,1)
     &  + HZ1(0) *HZ1(1)*HZ2(-1,1)*HZ2(0,1)
     &  + HZ1(0) *HZ1(1)*HZ4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ2(-1,1)*HZ3(0,1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ2(0,1)*HZ3(-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ2(-1,1)*HZ3(0,0,1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(0,1,-1)
     &  + HZ1(1) *HZ5(0,1,0,1,-1)
     &  - 5.0000000000000000d-01*HZ2(-1,1)*HZ2(0,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,1,1)
     &  + HZ2(0,1) *HZ4(0,1,-1,1)
     &  + 4.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,0,1)
     &  - HZ6(0,1,0,1, -1,1)
      HZ6(1,-1,1,0,1,1) = 
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,1,1,1)
     &  + HZ1(1) *HZ2(-1,1)*HZ3(0,1,1)
     &  + HZ1(1) *HZ5(0,1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,1,-1,1)
     &  + 3.0000000000000000d+00*HZ1(1)*HZ5(0,1,1,1,-1)
     &  + 3.0000000000000000d+00*HZ2(-1,1)*HZ4(0,1,1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,1,1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,1,1,1,-1,1)
      HZ6(1,-1,1,0,1,-1) = 
     &  - HZ1( -1)*HZ1(1)*HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,1,1,-1)
     &  + HZ1(1) *HZ2(-1,1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,-1,1,-1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(0,1,1,-1,-1)
     &  + HZ2( -1,1)*HZ4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,1,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,1,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,1,-1,1,-1)
      HZ6(1,-1,1,0,-1,0) = 
     &  - HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(0,1,-1)
     &  + HZ1( -1)*HZ1(1)*HZ2(0,-1)*HZ2(0,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,-1,0,1)
     &  + HZ1(0) *HZ1(1)*HZ2(-1,1)*HZ2(0,-1)
     &  + HZ1(0) *HZ1(1)*HZ4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,1,-1,-1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(0,-1,1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ2(0,-1)*HZ3(-1,1,1)
     &  - HZ1(0) *HZ5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,-1,-1,1)
     &  - HZ1(0) *HZ5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ2(-1,1)*HZ3(0,0,-1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(0,1,-1)
     &  + HZ1(1) *HZ5(0,-1,0,1,-1)
     &  - HZ2( -1,1)*HZ2(0,-1)*HZ2(0,1)
     &  + HZ2( -1,1)*HZ4(0,-1,0,1)
     &  + HZ2(0, -1)*HZ4(0,1,-1,1)
     &  + 4.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,0,-1)
     &  - HZ6(0, -1,0,1,-1,1)
      HZ6(1,-1,1,0,-1,1) = 
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,-1,1,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,1,-1,1)
     &  + HZ1(1) *HZ2(-1,1)*HZ3(0,-1,1)
     &  + HZ1(1) *HZ5(0,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,-1,-1,1)
     &  + HZ1(1) *HZ5(0,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,-1,1,1)
     &  + HZ2( -1,1)*HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,1,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,1,-1,1)
      HZ6(1,-1,1,0,-1,-1) = 
     &  - HZ1( -1)*HZ1(1)*HZ4(0,-1,-1,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,-1,1,-1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,1,-1,-1)
     &  + HZ1(1) *HZ2(-1,1)*HZ3(0,-1,-1)
     &  + HZ1(1) *HZ5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ1(1)*HZ5(0,1,-1,-1,-1)
     &  + HZ2( -1,1)*HZ4(0,-1,-1,1)
     &  + HZ2( -1,1)*HZ4(0,-1,1,-1)
     &  + HZ2( -1,1)*HZ4(0,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,-1,-1)
     &  - HZ6(0, -1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,-1,-1,1)
     &  - HZ6(0, -1,1,-1,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,-1,1,-1)
     &  - HZ6(0,1, -1,1,-1,-1)
      HZ6(1,-1,1,1,0,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(0,1,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,0,1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(-1,1,1)
     &  - 1.5000000000000000d+00*HZ1(0)*HZ1(0)*HZ4(-1,1,1,1)
     &  - HZ1(0) *HZ1(1)*HZ2(-1,1)*HZ2(0,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,1,1,-1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(0,1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ2(0,1)*HZ3(-1,1,1)
     &  + HZ1(0) *HZ5(0,1,1,-1,1)
     &  + HZ1(1) *HZ2(-1,1)*HZ3(0,0,1)
     &  + HZ1(1) *HZ5(0,0,1,1,-1)
     &  + HZ2( -1,1)*HZ4(0,0,1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,0,1)
     &  - HZ6(0,0,1,1, -1,1)
      HZ6(1,-1,1,1,0,1) = 
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,1,1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ2(-1,1)*HZ3(0,1,1)
     &  + HZ1(1) *HZ2(0,1)*HZ3(-1,1,1)
     &  - HZ1(1) *HZ5(0,1,1,-1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,1,1,1,-1)
     &  - 3.0000000000000000d+00*HZ2(-1,1)*HZ4(0,1,1,1)
     &  - 3.0000000000000000d+00*HZ2(0,1)*HZ4(-1,1,1,1)
     &  + 4.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,1,-1,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,1,1,1,-1,1)
      HZ6(1,-1,1,1,0,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(0,-1,1,1)
     &  + HZ1( -1)*HZ1(1)*HZ4(0,1,-1,1)
     &  + HZ1( -1)*HZ1(1)*HZ4(0,1,1,-1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(0,-1,1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(0,1,-1)
     &  + HZ1(1) *HZ2(0,-1)*HZ3(-1,1,1)
     &  - HZ1(1) *HZ5(0,-1,1,1,-1)
     &  - HZ1(1) *HZ5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,1,-1,-1)
     &  - HZ2( -1,1)*HZ4(0,-1,1,1)
     &  - HZ2( -1,1)*HZ4(0,1,-1,1)
     &  - HZ2( -1,1)*HZ4(0,1,1,-1)
     &  - 3.0000000000000000d+00*HZ2(0,-1)*HZ4(-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,1,-1)
     &  + HZ6(0, -1,1,1,-1,1)
     &  + HZ6(0,1, -1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,1,-1,-1,1)
     &  + HZ6(0,1,1, -1,1,-1)
      HZ6(1,-1,1,1,1,0) = 
     &  - HZ1( -1)*HZ1(1)*HZ4(0,1,1,1)
     &  + HZ1(0) *HZ1(1)*HZ4(-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(-1,1,1,1,1)
     &  + HZ1(1) *HZ2(-1,1)*HZ3(0,1,1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(-1,1,1)
     &  + HZ1(1) *HZ5(0,1,1,1,-1)
     &  + HZ2( -1,1)*HZ4(0,1,1,1)
     &  + 3.0000000000000000d+00*HZ2(0,1)*HZ4(-1,1,1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,1,1)
     &  - HZ6(0,1,1,1, -1,1)
      HZ6(1,-1,1,1,1,1) = 
     &  + HZ1(1) *HZ5(-1,1,1,1,1)
     &  - 5.0000000000000000d+00*HZ6(-1,1,1,1,1,1)
      HZ6(1,-1,1,1,1,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(-1,1,1,1,1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(-1,1,1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(-1,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(-1,1,-1,1,1)
     &  + 3.0000000000000000d+00*HZ2(-1,1)*HZ4(-1,1,1,1)
     &  - HZ3( -1,1,1)*HZ3(-1,1,1)
     &  - 4.0000000000000000d+00*HZ6(-1,-1,1,1,1,1)
     &  - 2.0000000000000000d+00*HZ6(-1,1,-1,1,1,1)
      HZ6(1,-1,1,1,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(-1,1,1,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ2(-1,1)*HZ2(-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ2(-1,1)*HZ3(-1,1,1)
     &  - 6.0000000000000000d+00*HZ1(0)*HZ5(-1,-1,1,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(-1,1,-1,1,1)
     &  + HZ1(1) *HZ2(-1,1)*HZ3(0,-1,1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(-1,1,1)
     &  + HZ1(1) *HZ5(0,-1,1,1,-1)
     &  + HZ2( -1,1)*HZ4(0,-1,1,1)
     &  + 3.0000000000000000d+00*HZ2(0,-1)*HZ4(-1,1,1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,-1,1)
     &  - HZ6(0, -1,1,1,-1,1)
      HZ6(1,-1,1,1,-1,1) = 
     &  + HZ1(1) *HZ2(-1,1)*HZ3(-1,1,1)
     &  - 6.0000000000000000d+00*HZ1(1)*HZ5(-1,-1,1,1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(-1,1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ2(-1,1)*HZ4(-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(-1,1,1)
      HZ6(1,-1,1,1,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(-1,1,1)
     &  - 1.5000000000000000d+00*HZ1(-1)*HZ1(-1)*HZ4(-1,1,1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ2(-1,1)*HZ2(-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ2(-1,1)*HZ3(-1,1,1)
     &  - 6.0000000000000000d+00*HZ1(-1)*HZ5(-1,-1,1,1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(-1,1,-1,1,1)
     &  + HZ1(1) *HZ2(-1,1)*HZ3(-1,-1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(-1,-1,-1,1,1)
     &  - HZ1(1) *HZ5(-1,-1,1,-1,1)
     &  + HZ2( -1,1)*HZ4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(-1,1,1)
     &  - HZ6( -1,-1,1,1,-1,1)
      HZ6(1,-1,1,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(1)*HZ2(-1,1)
     &  - HZ1( -1)*HZ1(0)*HZ1(0)*HZ3(-1,1,1)
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,0,-1,1)
     &  - HZ1(0) *HZ1(0)*HZ1(1)*HZ3(-1,-1,1)
     &  + 2.5000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(-1,1)*HZ2(-1,1)
     &  + HZ1(0) *HZ1(0)*HZ4(-1,-1,1,1)
     &  - HZ1(0) *HZ1(1)*HZ2(-1,1)*HZ2(0,-1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,-1,1,-1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ2(0,-1)*HZ3(-1,1,1)
     &  + HZ1(0) *HZ5(0,-1,1,-1,1)
     &  + HZ1(1) *HZ2(-1,1)*HZ3(0,0,-1)
     &  + HZ1(1) *HZ5(0,0,-1,1,-1)
     &  + HZ2( -1,1)*HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,0,-1)
     &  - HZ6(0,0, -1,1,-1,1)
      HZ6(1,-1,1,-1,0,1) = 
     &  + HZ1( -1)*HZ1(1)*HZ2(-1,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,-1,1,1)
     &  + HZ1( -1)*HZ1(1)*HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ2(0,1)*HZ3(-1,1,1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(0,-1,1)
     &  - HZ1(1) *HZ2(-1,1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ2(0,1)*HZ3(-1,-1,1)
     &  - HZ1(1) *HZ5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,1,1,-1)
     &  - HZ1(1) *HZ5(0,1,-1,1,-1)
     &  + 5.0000000000000000d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,-1,1,1)
     &  - HZ2( -1,1)*HZ4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,1,-1,1)
     &  + HZ6(0,1, -1,1,-1,1)
      HZ6(1,-1,1,-1,0,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ2(-1,1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,-1,-1,1)
     &  + HZ1( -1)*HZ1(1)*HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ2(0,-1)*HZ3(-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ2(-1,1)*HZ3(0,-1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ2(0,-1)*HZ3(-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,1,-1,-1)
     &  + 5.0000000000000000d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(0,-1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,-1,-1,1)
     &  - HZ2( -1,1)*HZ4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(-1,-1,1,1)
     &  + 4.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,-1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,-1,-1,1)
     &  + HZ6(0, -1,1,-1,1,-1)
      HZ6(1,-1,1,-1,1,0) = 
     &  - HZ1( -1)*HZ1(1)*HZ2(-1,1)*HZ2(0,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ2(0,1)*HZ3(-1,1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(1)*HZ2(-1,1)*HZ2(-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ2(-1,1)*HZ3(-1,1,1)
     &  + 1.2000000000000000d+01*HZ1(0)*HZ5(-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(-1,1,-1,1,1)
     &  + HZ1(1) *HZ2(-1,1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ2(0,1)*HZ3(-1,-1,1)
     &  + HZ1(1) *HZ5(0,1,-1,1,-1)
     &  - 5.0000000000000000d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(0,1)
     &  + HZ2( -1,1)*HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,1,-1)
     &  - HZ6(0,1, -1,1,-1,1)
      HZ6(1,-1,1,-1,1,1) = 
     &  + HZ1(1) *HZ5(-1,1,-1,1,1)
     &  - HZ3( -1,1,1)*HZ3(-1,1,1)
     &  + 1.2000000000000000d+01*HZ6(-1,-1,1,1,1,1)
     &  + 3.0000000000000000d+00*HZ6(-1,1,-1,1,1,1)
      HZ6(1,-1,1,-1,1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ2(-1,1)*HZ2(-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ2(-1,1)*HZ3(-1,1,1)
     &  + 1.2000000000000000d+01*HZ1(-1)*HZ5(-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(-1,1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ2(-1,1)*HZ3(-1,-1,1)
     &  + 1.2000000000000000d+01*HZ1(1)*HZ5(-1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HZ1(1)*HZ5(-1,-1,1,-1,1)
     &  - 1.6666666666666666d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(-1,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(-1,-1,1,1)
     &  + 4.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(-1,1,1)
     &  - 1.2000000000000000d+01*HZ6(-1,-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ6(-1,-1,1,-1,1,1)
      HZ6(1,-1,1,-1,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(1)*HZ2(-1,1)
     &  - HZ1( -1)*HZ1(-1)*HZ1(0)*HZ3(-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ1(1)*HZ3(-1,-1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ2(-1,1)*HZ2(-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(-1,-1,1,1)
     &  - HZ1( -1)*HZ1(1)*HZ2(-1,1)*HZ2(0,-1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ2(0,-1)*HZ3(-1,1,1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(-1,-1,-1,1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(-1,-1,1)
     &  + HZ1(0) *HZ5(-1,-1,1,-1,1)
     &  + HZ1(1) *HZ2(-1,1)*HZ3(0,-1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ2(0,-1)*HZ3(-1,-1,1)
     &  + HZ1(1) *HZ5(0,-1,-1,1,-1)
     &  - 5.0000000000000000d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(0,-1)
     &  + HZ2( -1,1)*HZ4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,-1,-1)
     &  - HZ6(0, -1,-1,1,-1,1)
      HZ6(1,-1,1,-1,-1,1) = 
     &  + HZ1(1) *HZ2(-1,1)*HZ3(-1,-1,1)
     &  - 6.0000000000000000d+00*HZ1(1)*HZ5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(-1,-1,1,-1,1)
     &  + 1.6666666666666666d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(-1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(-1,1,1)
     &  + 1.2000000000000000d+01*HZ6(-1,-1,-1,1,1,1)
     &  + 6.0000000000000000d+00*HZ6(-1,-1,1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(-1,-1,1,1,-1,1)
      HZ6(1,-1,1,-1,-1,-1) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ2(-1,1)
     &  - 3.3333333333333333d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(-1,1,1)
     &  - HZ1( -1)*HZ1(-1)*HZ1(1)*HZ3(-1,-1,1)
     &  + 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(-1,1)*HZ2(-1,1)
     &  + HZ1( -1)*HZ1(-1)*HZ4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(-1,-1,-1,1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(-1,-1,1)
     &  + HZ1( -1)*HZ5(-1,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(-1,-1,-1,-1,1)
     &  + HZ2( -1,1)*HZ4(-1,-1,-1,1)
     &  - HZ6( -1,-1,-1,1,-1,1)
      HZ6(1,-1,-1,0,0,0) = 
     &  + 8.3333333333333333d-02*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(1)
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(0)*HZ2(-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(1)*HZ2(0,-1)
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(0,0,-1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,0,0,-1)
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(-1,-1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(0,-1,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(-1,1)*HZ2(0,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,-1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,0,-1,-1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(0,0,-1)
     &  + HZ1(0) *HZ5(0,0,-1,-1,1)
     &  + HZ1(1) *HZ5(0,0,0,-1,-1)
     &  + HZ2( -1,1)*HZ4(0,0,0,-1)
     &  - HZ6(0,0,0, -1,-1,1)
      HZ6(1,-1,-1,0,0,1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(0,0,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,-1,0,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,0,-1,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,0,1,-1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(0,0,1)
     &  + HZ1(1) *HZ5(0,-1,-1,0,1)
     &  + HZ1(1) *HZ5(0,-1,0,-1,1)
     &  + HZ1(1) *HZ5(0,-1,0,1,-1)
     &  + HZ1(1) *HZ5(0,0,-1,-1,1)
     &  + HZ1(1) *HZ5(0,0,-1,1,-1)
     &  + HZ1(1) *HZ5(0,0,1,-1,-1)
     &  + HZ2( -1,1)*HZ4(0,-1,0,1)
     &  + HZ2( -1,1)*HZ4(0,0,-1,1)
     &  + HZ2( -1,1)*HZ4(0,0,1,-1)
     &  + HZ3( -1,-1,1)*HZ3(0,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,0,1,1)
     &  - HZ6(0, -1,-1,1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  - HZ6(0, -1,0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  - HZ6(0,0, -1,1,-1,1)
     &  - HZ6(0,0,1, -1,-1,1)
      HZ6(1,-1,-1,0,0,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(0,0,-1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ2(0,-1)*HZ2(0,-1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(0,0,-1)
     &  + HZ1(1) *HZ2(0,-1)*HZ3(0,-1,-1)
     &  - HZ1(1) *HZ5(0,-1,0,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HZ2(-1,1)*HZ2(0,-1)*HZ2(0,-1)
     &  - HZ2(0, -1)*HZ4(0,-1,-1,1)
     &  + HZ3( -1,-1,1)*HZ3(0,0,-1)
     &  + HZ6(0, -1,0,-1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
     &  + HZ6(0,0, -1,-1,1,-1)
      HZ6(1,-1,-1,0,1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(1)*HZ2(0,1)
     &  - HZ1( -1)*HZ1(-1)*HZ1(1)*HZ3(0,0,1)
     &  - HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(0,1,-1)
     &  - HZ1( -1)*HZ1(0)*HZ2(-1,1)*HZ2(0,1)
     &  + HZ1( -1)*HZ1(1)*HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ2(-1,1)*HZ3(0,0,1)
     &  + HZ1(0) *HZ1(1)*HZ4(0,-1,-1,1)
     &  + HZ1(0) *HZ1(1)*HZ4(0,-1,1,-1)
     &  + HZ1(0) *HZ1(1)*HZ4(0,1,-1,-1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(0,-1,1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(0,1,-1)
     &  + HZ1(0) *HZ2(0,1)*HZ3(-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,1,1)
     &  - HZ1(0) *HZ5(0,-1,1,-1,1)
     &  - HZ1(0) *HZ5(0,1,-1,-1,1)
     &  - HZ1(1) *HZ5(0,-1,-1,0,1)
     &  - HZ1(1) *HZ5(0,-1,0,-1,1)
     &  - HZ1(1) *HZ5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,0,1,-1,-1)
     &  - HZ2( -1,1)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,0,1,1)
     &  + HZ6(0, -1,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  + HZ6(0, -1,0,1,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,-1,1)
      HZ6(1,-1,-1,0,1,1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(0,1,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,-1,1,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,1,-1,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,1,1,-1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(0,1,1)
     &  + HZ1(1) *HZ5(0,-1,-1,1,1)
     &  + HZ1(1) *HZ5(0,-1,1,-1,1)
     &  + HZ1(1) *HZ5(0,-1,1,1,-1)
     &  + HZ1(1) *HZ5(0,1,-1,-1,1)
     &  + HZ1(1) *HZ5(0,1,-1,1,-1)
     &  + HZ1(1) *HZ5(0,1,1,-1,-1)
     &  + HZ2( -1,1)*HZ4(0,-1,1,1)
     &  + HZ2( -1,1)*HZ4(0,1,-1,1)
     &  + HZ2( -1,1)*HZ4(0,1,1,-1)
     &  + HZ3( -1,-1,1)*HZ3(0,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,-1,1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,-1,1,1)
     &  - HZ6(0, -1,1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,-1,1,1)
     &  - HZ6(0,1, -1,1,-1,1)
     &  - HZ6(0,1,1, -1,-1,1)
      HZ6(1,-1,-1,0,1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(0,1,-1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,1,-1,-1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(0,1,-1)
     &  + HZ1(1) *HZ5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ1(1)*HZ5(0,1,-1,-1,-1)
     &  + HZ2( -1,1)*HZ4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,1,-1,-1)
     &  + HZ3( -1,-1,1)*HZ3(0,1,-1)
     &  - HZ6(0, -1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,-1,-1,1)
     &  - HZ6(0, -1,1,-1,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,1,-1,-1,-1,1)
     &  - HZ6(0,1, -1,-1,1,-1)
      HZ6(1,-1,-1,0,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(1)*HZ2(0,-1)
     &  - HZ1( -1)*HZ1(-1)*HZ1(1)*HZ3(0,0,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ1(1)*HZ3(0,-1,-1)
     &  - HZ1( -1)*HZ1(0)*HZ2(-1,1)*HZ2(0,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(1)*HZ2(0,-1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ2(-1,1)*HZ3(0,0,-1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ2(-1,1)*HZ3(0,-1,-1)
     &  + HZ1(0) *HZ2(0,-1)*HZ3(-1,-1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,-1,1)
     &  - HZ1(0) *HZ5(0,-1,-1,1,-1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(0,-1,-1)
     &  + HZ1(1) *HZ5(0,-1,0,-1,-1)
     &  - 5.0000000000000000d-01*HZ2(-1,1)*HZ2(0,-1)*HZ2(0,-1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,-1,-1)
     &  + HZ2(0, -1)*HZ4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,0,-1)
     &  - HZ6(0, -1,0,-1,-1,1)
      HZ6(1,-1,-1,0,-1,1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,-1,-1,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,-1,1,-1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(0,-1,1)
     &  + 3.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,1,-1)
     &  + HZ1(1) *HZ5(0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,-1,-1,1)
     &  + HZ2( -1,1)*HZ4(0,-1,1,-1)
     &  + HZ3( -1,-1,1)*HZ3(0,-1,1)
     &  - 6.0000000000000000d+00*HZ6(0,-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,-1,1,-1,1)
     &  - HZ6(0, -1,1,-1,-1,1)
      HZ6(1,-1,-1,0,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(0,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,-1,-1,-1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(0,-1,-1)
     &  + 6.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HZ2(-1,1)*HZ4(0,-1,-1,-1)
     &  + HZ3( -1,-1,1)*HZ3(0,-1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,-1,-1,1,-1)
     &  - HZ6(0, -1,-1,1,-1,-1)
      HZ6(1,-1,-1,1,0,0) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(1)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(0,0,1)
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(0,1,-1)
     &  + HZ1( -1)*HZ1(0)*HZ2(-1,1)*HZ2(0,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,0,1,-1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(0,0,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ1(1)*HZ3(-1,-1,1)
     &  - 2.5000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(-1,1)*HZ2(-1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,1,-1,-1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(0,1,-1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(-1,-1,1)
     &  + HZ1(0) *HZ5(0,1,-1,-1,1)
     &  + HZ1(1) *HZ5(0,0,1,-1,-1)
     &  + HZ2( -1,1)*HZ4(0,0,1,-1)
     &  + HZ3( -1,-1,1)*HZ3(0,0,1)
     &  - HZ6(0,0,1, -1,-1,1)
      HZ6(1,-1,-1,1,0,1) = 
     &  - HZ1( -1)*HZ1(-1)*HZ1(1)*HZ3(0,1,1)
     &  + HZ1( -1)*HZ1(1)*HZ4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ2(-1,1)*HZ3(0,1,1)
     &  + HZ1(1) *HZ2(0,1)*HZ3(-1,-1,1)
     &  - HZ1(1) *HZ5(0,1,-1,-1,1)
     &  - HZ1(1) *HZ5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(1)*HZ5(0,1,1,-1,-1)
     &  - 5.0000000000000000d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(0,1)
     &  - HZ2( -1,1)*HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,-1,1,1)
     &  + HZ6(0,1, -1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,1,-1,-1,1)
      HZ6(1,-1,-1,1,0,-1) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(0,1,-1)
     &  + HZ1( -1)*HZ1(1)*HZ4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,1,-1,-1)
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(0,-1,1)
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(0,1,-1)
     &  + HZ1(1) *HZ2(0,-1)*HZ3(-1,-1,1)
     &  - HZ1(1) *HZ5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(0,1,-1,-1,-1)
     &  - 5.0000000000000000d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(0,-1)
     &  - HZ2( -1,1)*HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,1,-1,-1)
     &  - HZ3( -1,-1,1)*HZ3(0,-1,1)
     &  - HZ3( -1,-1,1)*HZ3(0,1,-1)
     &  + HZ6(0, -1,1,-1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,1,-1,-1,-1,1)
     &  + HZ6(0,1, -1,-1,1,-1)
      HZ6(1,-1,-1,1,1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(0,1,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,1,1,-1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(0,1,1)
     &  + HZ1(0) *HZ1(1)*HZ4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(-1,-1,1,1,1)
     &  - HZ1(0) *HZ5(-1,1,-1,1,1)
     &  - HZ1(1) *HZ2(0,1)*HZ3(-1,-1,1)
     &  + HZ1(1) *HZ5(0,1,1,-1,-1)
     &  + 5.0000000000000000d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(0,1)
     &  + HZ2( -1,1)*HZ4(0,1,1,-1)
     &  + HZ3( -1,-1,1)*HZ3(0,1,1)
     &  - HZ6(0,1,1, -1,-1,1)
      HZ6(1,-1,-1,1,1,1) = 
     &  + HZ1(1) *HZ5(-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ6(-1,-1,1,1,1,1)
     &  - HZ6( -1,1,-1,1,1,1)
      HZ6(1,-1,-1,1,1,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(-1,-1,1,1,1)
     &  - HZ1( -1)*HZ5(-1,1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(1)*HZ5(-1,-1,-1,1,1)
     &  - HZ1(1) *HZ5(-1,-1,1,-1,1)
     &  + 1.6666666666666666d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(-1,1)
     &  + 3.0000000000000000d+00*HZ6(-1,-1,-1,1,1,1)
     &  + HZ6( -1,-1,1,-1,1,1)
      HZ6(1,-1,-1,1,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(0,-1,1)
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ2(-1,1)*HZ2(-1,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,-1,1,-1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(0,-1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ1(1)*HZ4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ2(-1,1)*HZ3(-1,-1,1)
     &  - 6.0000000000000000d+00*HZ1(0)*HZ5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(-1,-1,1,-1,1)
     &  - HZ1(1) *HZ2(0,-1)*HZ3(-1,-1,1)
     &  + HZ1(1) *HZ5(0,-1,1,-1,-1)
     &  + 5.0000000000000000d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(0,-1)
     &  + HZ2( -1,1)*HZ4(0,-1,1,-1)
     &  + HZ3( -1,-1,1)*HZ3(0,-1,1)
     &  - HZ6(0, -1,1,-1,-1,1)
      HZ6(1,-1,-1,1,-1,1) = 
     &  + HZ1(1) *HZ5(-1,-1,1,-1,1)
     &  - 1.6666666666666666d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(-1,1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(-1,-1,1,1)
     &  - 1.2000000000000000d+01*HZ6(-1,-1,-1,1,1,1)
     &  - 6.0000000000000000d+00*HZ6(-1,-1,1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(-1,-1,1,1,-1,1)
      HZ6(1,-1,-1,1,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(-1,-1,1)
     &  - 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(-1,1)*HZ2(-1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ2(-1,1)*HZ3(-1,-1,1)
     &  - 6.0000000000000000d+00*HZ1(-1)*HZ5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(-1,-1,1,-1,1)
     &  + 6.0000000000000000d+00*HZ1(1)*HZ5(-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HZ2(-1,1)*HZ4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HZ3(-1,-1,1)*HZ3(-1,-1,1)
     &  + 6.0000000000000000d+00*HZ6(-1,-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HZ6(-1,-1,-1,1,-1,1)
      HZ6(1,-1,-1,-1,0,0) = 
     &  + 8.3333333333333333d-02*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(1)
     &  - 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(0)*HZ2(-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(1)*HZ2(0,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(0,0,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(-1,-1,1)
     &  + HZ1( -1)*HZ1(0)*HZ1(1)*HZ3(0,-1,-1)
     &  + HZ1( -1)*HZ1(0)*HZ2(-1,1)*HZ2(0,-1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,0,-1,-1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(0,0,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(-1,-1,-1,1)
     &  - HZ1(0) *HZ1(1)*HZ4(0,-1,-1,-1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(0,-1,-1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(-1,-1,1)
     &  + HZ1(0) *HZ5(0,-1,-1,-1,1)
     &  + HZ1(1) *HZ5(0,0,-1,-1,-1)
     &  + HZ2( -1,1)*HZ4(0,0,-1,-1)
     &  + HZ3( -1,-1,1)*HZ3(0,0,-1)
     &  - HZ6(0,0, -1,-1,-1,1)
      HZ6(1,-1,-1,-1,0,1) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ2(0,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(0,1,-1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(-1,1)*HZ2(0,1)
     &  + HZ1( -1)*HZ1(1)*HZ4(0,-1,-1,1)
     &  + HZ1( -1)*HZ1(1)*HZ4(0,-1,1,-1)
     &  + HZ1( -1)*HZ1(1)*HZ4(0,1,-1,-1)
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(0,-1,1)
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(0,1,-1)
     &  + HZ1( -1)*HZ2(0,1)*HZ3(-1,-1,1)
     &  - HZ1(1) *HZ5(0,-1,-1,-1,1)
     &  - HZ1(1) *HZ5(0,-1,-1,1,-1)
     &  - HZ1(1) *HZ5(0,-1,1,-1,-1)
     &  - HZ1(1) *HZ5(0,1,-1,-1,-1)
     &  - HZ2( -1,1)*HZ4(0,-1,-1,1)
     &  - HZ2( -1,1)*HZ4(0,-1,1,-1)
     &  - HZ2( -1,1)*HZ4(0,1,-1,-1)
     &  - HZ2(0,1) *HZ4(-1,-1,-1,1)
     &  - HZ3( -1,-1,1)*HZ3(0,-1,1)
     &  - HZ3( -1,-1,1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,-1,1,1)
     &  + HZ6(0, -1,-1,1,-1,1)
     &  + HZ6(0, -1,1,-1,-1,1)
     &  + HZ6(0,1, -1,-1,-1,1)
      HZ6(1,-1,-1,-1,0,-1) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ2(0,-1)
     &  - HZ1( -1)*HZ1(-1)*HZ1(1)*HZ3(0,-1,-1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(-1,1)*HZ2(0,-1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ1(1)*HZ4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ2(-1,1)*HZ3(0,-1,-1)
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(-1,-1,1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HZ2(-1,1)*HZ4(0,-1,-1,-1)
     &  - HZ2(0, -1)*HZ4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,-1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,-1,-1,-1,-1,1)
     &  + HZ6(0, -1,-1,-1,1,-1)
      HZ6(1,-1,-1,-1,1,0) = 
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(0,1,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(-1,1)*HZ2(0,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,1,-1,-1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(0,1,-1)
     &  - HZ1( -1)*HZ2(0,1)*HZ3(-1,-1,1)
     &  + HZ1(0) *HZ1(1)*HZ4(-1,-1,-1,1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(-1,-1,1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(-1,-1,1,-1,1)
     &  + HZ1(1) *HZ5(0,1,-1,-1,-1)
     &  + HZ2( -1,1)*HZ4(0,1,-1,-1)
     &  + HZ2(0,1) *HZ4(-1,-1,-1,1)
     &  + HZ3( -1,-1,1)*HZ3(0,1,-1)
     &  - HZ6(0,1, -1,-1,-1,1)
      HZ6(1,-1,-1,-1,1,1) = 
     &  + HZ1(1) *HZ5(-1,-1,-1,1,1)
     &  - HZ2( -1,1)*HZ4(-1,-1,1,1)
     &  + 6.0000000000000000d+00*HZ6(-1,-1,-1,1,1,1)
     &  + 3.0000000000000000d+00*HZ6(-1,-1,1,-1,1,1)
     &  + HZ6( -1,-1,1,1,-1,1)
      HZ6(1,-1,-1,-1,1,-1) = 
     &  + HZ1( -1)*HZ1(1)*HZ4(-1,-1,-1,1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(-1,-1,1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(-1,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HZ1(1)*HZ5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HZ2(-1,1)*HZ4(-1,-1,-1,1)
     &  - HZ3( -1,-1,1)*HZ3(-1,-1,1)
     &  - 4.0000000000000000d+00*HZ6(-1,-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(-1,-1,-1,1,-1,1)
      HZ6(1,-1,-1,-1,-1,0) = 
     &  + 4.1666666666666666d-02*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(1)
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ2(-1,1)
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ2(0,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(-1,-1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(1)*HZ3(0,-1,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(-1,1)*HZ2(0,-1)
     &  - HZ1( -1)*HZ1(0)*HZ4(-1,-1,-1,1)
     &  - HZ1( -1)*HZ1(1)*HZ4(0,-1,-1,-1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(0,-1,-1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(-1,-1,1)
     &  + HZ1(0) *HZ5(-1,-1,-1,-1,1)
     &  + HZ1(1) *HZ5(0,-1,-1,-1,-1)
     &  + HZ2( -1,1)*HZ4(0,-1,-1,-1)
     &  + HZ2(0, -1)*HZ4(-1,-1,-1,1)
     &  + HZ3( -1,-1,1)*HZ3(0,-1,-1)
     &  - HZ6(0, -1,-1,-1,-1,1)
      HZ6(1,-1,-1,-1,-1,1) = 
     &  + HZ1(1) *HZ5(-1,-1,-1,-1,1)
     &  - HZ2( -1,1)*HZ4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HZ3(-1,-1,1)*HZ3(-1,-1,1)
      HZ6(1,-1,-1,-1,-1,-1) = 
     &  + 8.3333333333333333d-03*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(1)
     &  - 4.1666666666666666d-02*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ2(-1,1)
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(-1,-1,-1,1)
     &  + HZ1( -1)*HZ5(-1,-1,-1,-1,1)
     &  - HZ6( -1,-1,-1,-1,-1,1)
      HZ6(-1,0,0,0,0,0) = 
     &  + 8.3333333333333333d-03*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)
     &  - 4.1666666666666666d-02*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)*HZ2(0,-1)
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,0,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,0,0,-1)
     &  + HZ1(0) *HZ5(0,0,0,0,-1)
     &  - HZ6(0,0,0,0,0, -1)
      HZ6(-1,0,0,0,0,1) = 
     &  + HZ1( -1)*HZ5(0,0,0,0,1)
     &  - HZ2(0, -1)*HZ4(0,0,0,1)
     &  + HZ6(0,0, -1,0,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,0,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,0,1,-1)
     &  + HZ6(0,0,0,1,0, -1)
      HZ6(-1,0,0,0,0,-1) = 
     &  + HZ1( -1)*HZ5(0,0,0,0,-1)
     &  - HZ2(0, -1)*HZ4(0,0,0,-1)
     &  + 5.0000000000000000d-01*HZ3(0,0,-1)*HZ3(0,0,-1)
      HZ6(-1,0,0,0,1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ4(0,0,0,1)
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,0,1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(0,0,1)
     &  + HZ1(0) *HZ5(0,0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,1,-1)
     &  + HZ1(0) *HZ5(0,0,1,0,-1)
     &  + 3.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,0,0,1)
     &  - 5.0000000000000000d+00*HZ6(0,0,0,-1,0,1)
     &  - 8.0000000000000000d+00*HZ6(0,0,0,0,-1,1)
     &  - 8.0000000000000000d+00*HZ6(0,0,0,0,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,0,-1)
      HZ6(-1,0,0,0,1,1) = 
     &  + HZ1( -1)*HZ5(0,0,0,1,1)
     &  - HZ2(0, -1)*HZ4(0,0,1,1)
     &  + HZ6(0,0, -1,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,0,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,0,1,1,-1)
     &  + HZ6(0,0,1,0, -1,1)
     &  + HZ6(0,0,1,0,1, -1)
     &  + HZ6(0,0,1,1,0, -1)
      HZ6(-1,0,0,0,1,-1) = 
     &  + HZ1( -1)*HZ5(0,0,0,1,-1)
     &  - HZ2(0, -1)*HZ4(0,0,1,-1)
     &  + HZ6(0,0, -1,0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,0,1,-1,-1)
     &  + HZ6(0,0,1, -1,0,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,0,-1,-1)
      HZ6(-1,0,0,0,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ4(0,0,0,-1)
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,0,-1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(0,0,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,0,-1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,0,-1)
     &  - HZ3(0,0, -1)*HZ3(0,0,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,0,-1,0,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,0,0,-1,-1)
      HZ6(-1,0,0,0,-1,1) = 
     &  + HZ1( -1)*HZ5(0,0,0,-1,1)
     &  - HZ2(0, -1)*HZ4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,0,-1,1)
     &  + HZ6(0,0, -1,0,1,-1)
     &  + HZ6(0,0, -1,1,0,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
      HZ6(-1,0,0,0,-1,-1) = 
     &  + HZ1( -1)*HZ5(0,0,0,-1,-1)
     &  - HZ2(0, -1)*HZ4(0,0,-1,-1)
     &  + HZ6(0,0, -1,-1,0,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,-1,0,-1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,-1,-1,-1)
      HZ6(-1,0,0,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(0,0,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,0,0,1)
     &  + 6.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,0,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,0,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ2(0,-1)*HZ3(0,0,1)
     &  - HZ1(0) *HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,0,-1)
     &  - 3.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,0,1)
     &  + HZ6(0,0, -1,0,0,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,0,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,0,-1)
      HZ6(-1,0,0,1,0,1) = 
     &  + HZ1( -1)*HZ5(0,0,1,0,1)
     &  - HZ6(0, -1,0,1,0,1)
     &  - HZ6(0,0, -1,1,0,1)
     &  - HZ6(0,0,1, -1,0,1)
     &  - HZ6(0,0,1,0, -1,1)
     &  - HZ6(0,0,1,0,1, -1)
      HZ6(-1,0,0,1,0,-1) = 
     &  + HZ1( -1)*HZ5(0,0,1,0,-1)
     &  - HZ2(0, -1)*HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,0,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,-1,1,0,-1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,-1,-1,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,-1,1,-1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,1,-1,0,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,1,0,-1,-1)
      HZ6(-1,0,0,1,1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ4(0,0,1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,1,1)
     &  - HZ1( -1)*HZ5(0,0,1,0,1)
     &  - HZ1(0) *HZ5(0,-1,0,1,1)
     &  - HZ1(0) *HZ5(0,0,-1,1,1)
     &  - HZ1(0) *HZ5(0,0,1,-1,1)
     &  - HZ1(0) *HZ5(0,0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  + HZ6(0, -1,0,1,0,1)
     &  - HZ6(0,0, -1,0,1,1)
     &  + HZ6(0,0, -1,1,0,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,1,1,-1)
     &  + HZ6(0,0,1, -1,0,1)
     &  - HZ6(0,0,1,0, -1,1)
     &  - HZ6(0,0,1,0,1, -1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(-1,0,0,1,1,1) = 
     &  + HZ1( -1)*HZ5(0,0,1,1,1)
     &  - HZ6(0, -1,0,1,1,1)
     &  - HZ6(0,0, -1,1,1,1)
     &  - HZ6(0,0,1, -1,1,1)
     &  - HZ6(0,0,1,1, -1,1)
     &  - HZ6(0,0,1,1,1, -1)
      HZ6(-1,0,0,1,1,-1) = 
     &  + HZ1( -1)*HZ5(0,0,1,1,-1)
     &  - HZ6(0, -1,0,1,1,-1)
     &  - HZ6(0,0, -1,1,1,-1)
     &  - HZ6(0,0,1, -1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,1,-1,-1)
      HZ6(-1,0,0,1,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,1,-1)
     &  - HZ1( -1)*HZ5(0,0,1,0,-1)
     &  - HZ1(0) *HZ5(0,-1,0,1,-1)
     &  - HZ1(0) *HZ5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,-1,-1)
     &  + HZ2(0, -1)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,0,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,-1,0,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,-1,1,0,-1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,-1,-1,1)
     &  + 9.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,1,-1,-1)
     &  + HZ6(0,0,1, -1,0,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,0,-1,-1)
      HZ6(-1,0,0,1,-1,1) = 
     &  + HZ1( -1)*HZ5(0,0,1,-1,1)
     &  - HZ6(0, -1,0,1,-1,1)
     &  - HZ6(0,0, -1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,-1,1)
     &  - HZ6(0,0,1, -1,1,-1)
      HZ6(-1,0,0,1,-1,-1) = 
     &  + HZ1( -1)*HZ5(0,0,1,-1,-1)
     &  - HZ6(0, -1,0,1,-1,-1)
     &  - HZ6(0,0, -1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,1,-1,-1,-1)
      HZ6(-1,0,0,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(0,0,-1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,0,0,-1)
     &  + 6.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,0,-1)
     &  - 2.5000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(0,-1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ2(0,-1)*HZ3(0,0,-1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HZ1(0)*HZ5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,0,-1)
     &  + 5.0000000000000000d-01*HZ3(0,0,-1)*HZ3(0,0,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,0,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,0,-1,-1)
      HZ6(-1,0,0,-1,0,1) = 
     &  + HZ1( -1)*HZ5(0,0,-1,0,1)
     &  - HZ6(0, -1,0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  - HZ6(0,0, -1,0,-1,1)
     &  - HZ6(0,0, -1,0,1,-1)
      HZ6(-1,0,0,-1,0,-1) = 
     &  + HZ1( -1)*HZ5(0,0,-1,0,-1)
     &  - 1.6666666666666666d-01*HZ2(0,-1)*HZ2(0,-1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,-1,0,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,-1,0,-1,-1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,-1,-1,-1)
      HZ6(-1,0,0,-1,1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ4(0,0,-1,1)
     &  - HZ1( -1)*HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,-1,1)
     &  - HZ1(0) *HZ5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,-1,1)
     &  - HZ1(0) *HZ5(0,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,-1,1)
     &  + HZ6(0, -1,0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,0,-1,1)
     &  - HZ6(0,0, -1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,0,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
      HZ6(-1,0,0,-1,1,1) = 
     &  + HZ1( -1)*HZ5(0,0,-1,1,1)
     &  - HZ6(0, -1,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  - HZ6(0,0, -1,1,-1,1)
     &  - HZ6(0,0, -1,1,1,-1)
      HZ6(-1,0,0,-1,1,-1) = 
     &  + HZ1( -1)*HZ5(0,0,-1,1,-1)
     &  - HZ6(0, -1,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,-1)
      HZ6(-1,0,0,-1,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ4(0,0,-1,-1)
     &  - HZ1( -1)*HZ5(0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,-1,-1)
     &  - HZ1(0) *HZ5(0,-1,0,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,-1,-1)
     &  + 1.6666666666666666d-01*HZ2(0,-1)*HZ2(0,-1)*HZ2(0,-1)
     &  + HZ6(0,0, -1,0,-1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,-1,-1)
      HZ6(-1,0,0,-1,-1,1) = 
     &  + HZ1( -1)*HZ5(0,0,-1,-1,1)
     &  - HZ6(0, -1,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
     &  - HZ6(0,0, -1,-1,1,-1)
      HZ6(-1,0,0,-1,-1,-1) = 
     &  + HZ1( -1)*HZ5(0,0,-1,-1,-1)
     &  - HZ6(0, -1,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,-1,-1)
      HZ6(-1,0,1,0,0,0) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(0)*HZ2(0,1)
     &  - HZ1( -1)*HZ1(0)*HZ1(0)*HZ3(0,0,1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,0,0,1)
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,0,1)
     &  - 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,-1,1)
     &  - 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,1,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,0,1)
     &  + HZ1(0) *HZ1(0)*HZ4(0,0,-1,1)
     &  + HZ1(0) *HZ1(0)*HZ4(0,0,1,-1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(0,0,1)
     &  + HZ1(0) *HZ5(0,0,1,0,-1)
     &  + HZ2(0, -1)*HZ4(0,0,0,1)
     &  - HZ6(0,0,0,1,0, -1)
      HZ6(-1,0,1,0,0,1) = 
     &  + HZ1( -1)*HZ2(0,1)*HZ3(0,0,1)
     &  - 6.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,0,1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  - HZ3(0, -1,1)*HZ3(0,0,1)
     &  - HZ3(0,0,1) *HZ3(0,1,-1)
     &  + HZ6(0, -1,0,1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,0,1)
     &  + HZ6(0,0,1,0, -1,1)
     &  + HZ6(0,0,1,0,1, -1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(-1,0,1,0,0,-1) = 
     &  + HZ1( -1)*HZ2(0,1)*HZ3(0,0,-1)
     &  - HZ1( -1)*HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,0,-1)
     &  + HZ2(0, -1)*HZ4(0,-1,0,1)
     &  - HZ2(0, -1)*HZ4(0,0,-1,1)
     &  - HZ2(0, -1)*HZ4(0,0,1,-1)
     &  - HZ3(0, -1,1)*HZ3(0,0,-1)
     &  - HZ3(0,0, -1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,0,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,0,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,-1,1,0,-1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,-1,-1,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,-1,1,-1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,1,-1,0,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,1,0,-1,-1)
      HZ6(-1,0,1,0,1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ2(0,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,0,1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ2(0,1)*HZ3(0,0,1)
     &  + 1.2000000000000000d+01*HZ1(-1)*HZ5(0,0,0,1,1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,0,1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(0,1,-1)
     &  - HZ1(0) *HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,-1,1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,1,-1)
     &  + HZ1(0) *HZ5(0,1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,1)
     &  + 2.0000000000000000d+00*HZ3(0,-1,1)*HZ3(0,0,1)
     &  + 2.0000000000000000d+00*HZ3(0,0,1)*HZ3(0,1,-1)
     &  - HZ6(0, -1,0,1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,0,-1)
      HZ6(-1,0,1,0,1,1) = 
     &  + HZ1( -1)*HZ5(0,1,0,1,1)
     &  - HZ2(0,1) *HZ4(0,-1,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  + HZ6(0, -1,1,0,1,1)
     &  + HZ6(0, -1,1,1,0,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,1,1)
     &  - HZ6(0,1, -1,0,1,1)
     &  - HZ6(0,1,0,1, -1,1)
     &  - HZ6(0,1,0,1,1, -1)
      HZ6(-1,0,1,0,1,-1) = 
     &  + HZ1( -1)*HZ5(0,1,0,1,-1)
     &  - 5.0000000000000000d-01*HZ3(0,1,-1)*HZ3(0,1,-1)
     &  - HZ6(0, -1,1,0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,-1,-1)
      HZ6(-1,0,1,0,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ2(0,-1)*HZ2(0,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ2(0,1)*HZ3(0,0,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,0,-1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(0,-1,1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,-1,-1)
     &  - HZ2(0, -1)*HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ3(0,-1,1)*HZ3(0,0,-1)
     &  + 2.0000000000000000d+00*HZ3(0,0,-1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,0,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,0,-1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,-1,-1,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,-1,1,-1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,0,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,0,-1,-1)
      HZ6(-1,0,1,0,-1,1) = 
     &  + HZ1( -1)*HZ2(0,1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,0,1,1)
     &  - HZ1( -1)*HZ5(0,-1,1,0,1)
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,-1,1)
     &  - 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  - HZ3(0, -1,1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,1,1,-1)
     &  + HZ6(0, -1,1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,1,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,1,-1)
      HZ6(-1,0,1,0,-1,-1) = 
     &  + HZ1( -1)*HZ2(0,1)*HZ3(0,-1,-1)
     &  - HZ1( -1)*HZ5(0,-1,-1,0,1)
     &  - HZ1( -1)*HZ5(0,-1,0,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,-1,-1)
     &  - HZ3(0, -1,-1)*HZ3(0,-1,1)
     &  - HZ3(0, -1,-1)*HZ3(0,1,-1)
     &  + HZ6(0, -1,-1,0,-1,1)
     &  + HZ6(0, -1,-1,0,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,-1,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,1,-1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,-1,1,-1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,1,-1,-1,-1)
      HZ6(-1,0,1,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(0,1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ2(0,1)*HZ2(0,1)
     &  + HZ1( -1)*HZ2(0,1)*HZ3(0,0,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,1,1)
     &  - HZ1( -1)*HZ5(0,0,1,0,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,1,1,-1)
     &  + HZ1(0) *HZ2(0,1)*HZ3(0,1,-1)
     &  + HZ1(0) *HZ5(0,-1,0,1,1)
     &  + HZ1(0) *HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,1,-1)
     &  - HZ1(0) *HZ5(0,1,0,1,-1)
     &  + HZ2(0, -1)*HZ4(0,0,1,1)
     &  - HZ3(0, -1,1)*HZ3(0,0,1)
     &  - HZ3(0,0,1) *HZ3(0,1,-1)
     &  - HZ6(0,0,1,1,0, -1)
      HZ6(-1,0,1,1,0,1) = 
     &  + HZ1( -1)*HZ2(0,1)*HZ3(0,1,1)
     &  - 6.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,1,0,1,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,1,1)
     &  - HZ2(0,1) *HZ4(0,1,-1,1)
     &  - HZ2(0,1) *HZ4(0,1,1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,1,1,0,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,-1,1,1,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,0,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,1,0,1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,1,0,1,1,-1)
      HZ6(-1,0,1,1,0,-1) = 
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(0,1,1)
     &  - HZ1( -1)*HZ2(0,1)*HZ3(0,-1,1)
     &  + HZ1( -1)*HZ5(0,-1,0,1,1)
     &  + HZ1( -1)*HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,1,-1)
     &  - HZ1( -1)*HZ5(0,1,0,1,-1)
     &  - HZ2(0, -1)*HZ4(0,-1,1,1)
     &  - HZ2(0, -1)*HZ4(0,1,-1,1)
     &  - HZ2(0, -1)*HZ4(0,1,1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  + HZ3(0, -1,1)*HZ3(0,1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,1,-1)*HZ3(0,1,-1)
      HZ6(-1,0,1,1,1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ4(0,1,1,1)
     &  - HZ1( -1)*HZ2(0,1)*HZ3(0,1,1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,0,1,1)
     &  - HZ1(0) *HZ5(0,-1,1,1,1)
     &  - HZ1(0) *HZ5(0,1,-1,1,1)
     &  - HZ1(0) *HZ5(0,1,1,-1,1)
     &  - HZ1(0) *HZ5(0,1,1,1,-1)
     &  - HZ2(0,1) *HZ4(0,-1,1,1)
     &  + HZ2(0,1) *HZ4(0,1,-1,1)
     &  + HZ2(0,1) *HZ4(0,1,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,1,0,1)
     &  + 8.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,1,-1)
     &  - HZ6(0,1, -1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,0,1,1,-1)
      HZ6(-1,0,1,1,1,1) = 
     &  + HZ1( -1)*HZ5(0,1,1,1,1)
     &  - HZ6(0, -1,1,1,1,1)
     &  - HZ6(0,1, -1,1,1,1)
     &  - HZ6(0,1,1, -1,1,1)
     &  - HZ6(0,1,1,1, -1,1)
     &  - HZ6(0,1,1,1,1, -1)
      HZ6(-1,0,1,1,1,-1) = 
     &  + HZ1( -1)*HZ5(0,1,1,1,-1)
     &  - HZ6(0, -1,1,1,1,-1)
     &  - HZ6(0,1, -1,1,1,-1)
     &  - HZ6(0,1,1, -1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,1,1,1,-1,-1)
      HZ6(-1,0,1,1,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ4(0,1,1,-1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(0,1,1)
     &  + HZ1( -1)*HZ2(0,1)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,0,1,1)
     &  - HZ1( -1)*HZ5(0,-1,1,0,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,1,1)
     &  - HZ1(0) *HZ5(0,-1,1,1,-1)
     &  - HZ1(0) *HZ5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,1,-1,-1)
     &  + HZ2(0, -1)*HZ4(0,-1,1,1)
     &  + HZ2(0, -1)*HZ4(0,1,-1,1)
     &  + HZ2(0, -1)*HZ4(0,1,1,-1)
     &  - 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  - HZ3(0, -1,1)*HZ3(0,1,-1)
     &  + HZ6(0, -1,0,1,1,-1)
     &  + HZ6(0, -1,1,0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
      HZ6(-1,0,1,1,-1,1) = 
     &  + HZ1( -1)*HZ5(0,1,1,-1,1)
     &  - HZ6(0, -1,1,1,-1,1)
     &  - HZ6(0,1, -1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,1,-1,-1,1)
     &  - HZ6(0,1,1, -1,1,-1)
      HZ6(-1,0,1,1,-1,-1) = 
     &  + HZ1( -1)*HZ5(0,1,1,-1,-1)
     &  - HZ6(0, -1,1,1,-1,-1)
     &  - HZ6(0,1, -1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,1,1,-1,-1,-1)
      HZ6(-1,0,1,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(0,1,-1)
     &  - HZ1( -1)*HZ1(0)*HZ2(0,-1)*HZ2(0,1)
     &  + HZ1( -1)*HZ1(0)*HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,0,-1,1)
     &  + HZ1( -1)*HZ2(0,1)*HZ3(0,0,-1)
     &  - HZ1( -1)*HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,1,-1)
     &  - HZ1(0) *HZ1(0)*HZ4(0,1,-1,-1)
     &  + HZ1(0) *HZ2(0,-1)*HZ3(0,-1,1)
     &  + HZ1(0) *HZ2(0,-1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,0,-1,1)
     &  - HZ1(0) *HZ5(0,-1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,1,-1)
     &  + HZ2(0, -1)*HZ4(0,0,-1,1)
     &  - HZ3(0, -1,1)*HZ3(0,0,-1)
     &  - HZ3(0,0, -1)*HZ3(0,1,-1)
     &  - HZ6(0,0, -1,1,0,-1)
      HZ6(-1,0,1,-1,0,1) = 
     &  - HZ1( -1)*HZ2(0,1)*HZ3(0,-1,1)
     &  + HZ1( -1)*HZ2(0,1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,0,1,1)
     &  + HZ1( -1)*HZ5(0,-1,1,0,1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,1,1)
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,1,-1,-1)
     &  + HZ3(0, -1,1)*HZ3(0,1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,1,-1)*HZ3(0,1,-1)
     &  - HZ6(0, -1,0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,1,1,-1)
     &  - HZ6(0, -1,1,-1,0,1)
     &  - HZ6(0, -1,1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,-1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,1,0,1,-1,-1)
      HZ6(-1,0,1,-1,0,-1) = 
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ2(0,1)*HZ3(0,-1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,0,-1,1)
     &  + HZ1( -1)*HZ5(0,-1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,1,-1)
     &  - HZ2(0, -1)*HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,1,-1,-1)
     &  + 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,0,1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,-1,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,1,-1,-1)
     &  - 1.2000000000000000d+01*HZ6(0,0,-1,-1,-1,1)
     &  - 8.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,1,-1,-1)
      HZ6(-1,0,1,-1,1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ4(0,1,-1,1)
     &  - HZ1( -1)*HZ2(0,1)*HZ3(0,1,-1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,0,1,-1)
     &  - HZ1(0) *HZ5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,-1,-1,1)
     &  - HZ1(0) *HZ5(0,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,1,-1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ3(0,1,-1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  + HZ6(0, -1,1,-1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,1,0,1,-1,-1)
      HZ6(-1,0,1,-1,1,1) = 
     &  + HZ1( -1)*HZ5(0,1,-1,1,1)
     &  - HZ6(0, -1,1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,-1,1,1)
     &  - HZ6(0,1, -1,1,-1,1)
     &  - HZ6(0,1, -1,1,1,-1)
      HZ6(-1,0,1,-1,1,-1) = 
     &  + HZ1( -1)*HZ5(0,1,-1,1,-1)
     &  - HZ6(0, -1,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,1,-1,-1)
      HZ6(-1,0,1,-1,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ4(0,1,-1,-1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(0,1,-1)
     &  + HZ1( -1)*HZ2(0,1)*HZ3(0,-1,-1)
     &  - HZ1( -1)*HZ5(0,-1,-1,0,1)
     &  - HZ1( -1)*HZ5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,-1,1)
     &  - HZ1(0) *HZ5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,1,-1,-1,-1)
     &  + HZ2(0, -1)*HZ4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,1,-1,-1)
     &  - HZ3(0, -1,-1)*HZ3(0,-1,1)
     &  - HZ3(0, -1,-1)*HZ3(0,1,-1)
     &  + HZ6(0, -1,-1,0,-1,1)
     &  + HZ6(0, -1,-1,0,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  + HZ6(0, -1,0,-1,1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
      HZ6(-1,0,1,-1,-1,1) = 
     &  + HZ1( -1)*HZ5(0,1,-1,-1,1)
     &  - HZ6(0, -1,1,-1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,1,-1,-1,-1,1)
     &  - HZ6(0,1, -1,-1,1,-1)
      HZ6(-1,0,1,-1,-1,-1) = 
     &  + HZ1( -1)*HZ5(0,1,-1,-1,-1)
     &  - HZ6(0, -1,1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,1,-1,-1,-1,-1)
      HZ6(-1,0,-1,0,0,0) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(0)*HZ2(0,-1)
     &  - HZ1( -1)*HZ1(0)*HZ1(0)*HZ3(0,0,-1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,0,0,-1)
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,0,-1)
     &  - 3.3333333333333333d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,-1,-1)
     &  + 2.5000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(0,-1)*HZ2(0,-1)
     &  + HZ1(0) *HZ1(0)*HZ4(0,0,-1,-1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(0,0,-1)
     &  + HZ1(0) *HZ5(0,0,-1,0,-1)
     &  + HZ2(0, -1)*HZ4(0,0,0,-1)
     &  - HZ6(0,0,0, -1,0,-1)
      HZ6(-1,0,-1,0,0,1) = 
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(0,0,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,1,-1)
     &  - HZ1( -1)*HZ5(0,0,1,0,-1)
     &  + HZ2(0, -1)*HZ4(0,0,-1,1)
     &  + HZ2(0, -1)*HZ4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,0,1)
     &  + HZ6(0, -1,0,-1,0,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  + HZ6(0,0, -1,0,-1,1)
     &  + HZ6(0,0, -1,0,1,-1)
     &  - HZ6(0,0, -1,1,0,-1)
     &  - HZ6(0,0,1, -1,0,-1)
      HZ6(-1,0,-1,0,0,-1) = 
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(0,0,-1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,-1,-1)
     &  + 1.6666666666666666d-01*HZ2(0,-1)*HZ2(0,-1)*HZ2(0,-1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,0,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,-1,0,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,-1,0,-1,-1)
     &  + 1.2000000000000000d+01*HZ6(0,0,0,-1,-1,-1)
      HZ6(-1,0,-1,0,1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ2(0,-1)*HZ3(0,0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,0,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,0,1)
     &  - HZ1(0) *HZ5(0,-1,0,-1,1)
     &  - HZ1(0) *HZ5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,1,-1)
     &  + 4.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,0,1)
     &  - HZ6(0, -1,0,-1,0,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,0,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,0,-1)
      HZ6(-1,0,-1,0,1,1) = 
     &  + HZ1( -1)*HZ5(0,-1,0,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,0,1,1)
     &  - HZ6(0, -1,0,-1,1,1)
     &  - HZ6(0, -1,0,1,-1,1)
     &  - HZ6(0, -1,0,1,1,-1)
      HZ6(-1,0,-1,0,1,-1) = 
     &  + HZ1( -1)*HZ5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,0,1,-1)
     &  - HZ6(0, -1,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,1,-1,-1)
      HZ6(-1,0,-1,0,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ2(0,-1)*HZ2(0,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ2(0,-1)*HZ3(0,0,-1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,0,-1)
     &  + 1.2000000000000000d+01*HZ1(-1)*HZ5(0,0,0,-1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ2(0,-1)*HZ3(0,-1,-1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,-1,0,-1,-1)
     &  + 1.2000000000000000d+01*HZ1(0)*HZ5(0,0,-1,-1,-1)
     &  - 1.6666666666666666d-01*HZ2(0,-1)*HZ2(0,-1)*HZ2(0,-1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,0,-1,-1)
     &  + 4.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,0,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,0,-1,-1)
     &  - 1.2000000000000000d+01*HZ6(0,0,0,-1,-1,-1)
      HZ6(-1,0,-1,0,-1,1) = 
     &  + HZ1( -1)*HZ5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  - HZ6(0, -1,0,-1,1,-1)
      HZ6(-1,0,-1,0,-1,-1) = 
     &  + HZ1( -1)*HZ5(0,-1,0,-1,-1)
     &  - HZ3(0, -1,-1)*HZ3(0,-1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,-1,-1,-1)
     &  + 1.2000000000000000d+01*HZ6(0,0,-1,-1,-1,-1)
      HZ6(-1,0,-1,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,0,-1,1)
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(0,0,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,1,-1)
     &  - HZ1( -1)*HZ5(0,0,1,0,-1)
     &  - HZ1(0) *HZ1(0)*HZ4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,0,-1,1)
     &  + HZ1(0) *HZ5(0,-1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,1,-1)
     &  + HZ2(0, -1)*HZ4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,0,1)
     &  - HZ6(0,0,1, -1,0,-1)
      HZ6(-1,0,-1,1,0,1) = 
     &  + HZ1( -1)*HZ5(0,-1,1,0,1)
     &  - 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  + HZ6(0, -1,0,1,-1,1)
     &  - HZ6(0, -1,1,-1,0,1)
     &  - HZ6(0, -1,1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,1)
      HZ6(-1,0,-1,1,0,-1) = 
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,0,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,-1,-1,1)
     &  - HZ2(0, -1)*HZ4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,0,1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,-1,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,1,-1,-1)
     &  + 1.2000000000000000d+01*HZ6(0,0,-1,-1,-1,1)
     &  + 8.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,1,-1,-1)
      HZ6(-1,0,-1,1,1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ4(0,-1,1,1)
     &  - HZ1( -1)*HZ5(0,-1,0,1,1)
     &  - HZ1( -1)*HZ5(0,-1,1,0,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,1,1)
     &  - HZ1(0) *HZ5(0,-1,1,-1,1)
     &  - HZ1(0) *HZ5(0,-1,1,1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,1,0,1)
     &  + HZ6(0, -1,0,1,1,-1)
     &  + HZ6(0, -1,1,-1,0,1)
     &  + HZ6(0, -1,1,0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
      HZ6(-1,0,-1,1,1,1) = 
     &  + HZ1( -1)*HZ5(0,-1,1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,1,1,1)
     &  - HZ6(0, -1,1,-1,1,1)
     &  - HZ6(0, -1,1,1,-1,1)
     &  - HZ6(0, -1,1,1,1,-1)
      HZ6(-1,0,-1,1,1,-1) = 
     &  + HZ1( -1)*HZ5(0,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,1,1,-1)
     &  - HZ6(0, -1,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,1,-1,-1)
      HZ6(-1,0,-1,1,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ4(0,-1,1,-1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,0,-1,1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,-1,-1,1)
     &  + HZ2(0, -1)*HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,0,-1,1)
     &  - 6.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,-1)
     &  - 1.2000000000000000d+01*HZ6(0,0,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
      HZ6(-1,0,-1,1,-1,1) = 
     &  + HZ1( -1)*HZ5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,-1,-1,1)
     &  - HZ6(0, -1,1,-1,1,-1)
      HZ6(-1,0,-1,1,-1,-1) = 
     &  + HZ1( -1)*HZ5(0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,1,-1,-1,-1)
      HZ6(-1,0,-1,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(0,-1,-1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ2(0,-1)*HZ2(0,-1)
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(0,0,-1)
     &  - HZ1( -1)*HZ5(0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,-1,-1)
     &  - 1.5000000000000000d+00*HZ1(0)*HZ1(0)*HZ4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ2(0,-1)*HZ3(0,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,-1,0,-1,-1)
     &  - 6.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,-1,-1)
     &  + HZ2(0, -1)*HZ4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,0,-1)
     &  - HZ6(0,0, -1,-1,0,-1)
      HZ6(-1,0,-1,-1,0,1) = 
     &  + HZ1( -1)*HZ5(0,-1,-1,0,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,-1,-1,0,1)
     &  - HZ6(0, -1,-1,0,-1,1)
     &  - HZ6(0, -1,-1,0,1,-1)
      HZ6(-1,0,-1,-1,0,-1) = 
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(0,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,0,-1,-1)
     &  - 6.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HZ2(0,-1)*HZ4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,-1,-1)
      HZ6(-1,0,-1,-1,1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ4(0,-1,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,-1,0,1)
     &  - HZ1( -1)*HZ5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,-1,1)
     &  - HZ1(0) *HZ5(0,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,-1,-1,0,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,-1,0,-1,1)
     &  + HZ6(0, -1,-1,0,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  + HZ6(0, -1,0,-1,1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
      HZ6(-1,0,-1,-1,1,1) = 
     &  + HZ1( -1)*HZ5(0,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,-1,-1,1,1)
     &  - HZ6(0, -1,-1,1,-1,1)
     &  - HZ6(0, -1,-1,1,1,-1)
      HZ6(-1,0,-1,-1,1,-1) = 
     &  + HZ1( -1)*HZ5(0,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,1,-1,-1)
      HZ6(-1,0,-1,-1,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ4(0,-1,-1,-1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(0,-1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,0,-1,-1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HZ2(0,-1)*HZ4(0,-1,-1,-1)
     &  - HZ3(0, -1,-1)*HZ3(0,-1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,-1,-1)
      HZ6(-1,0,-1,-1,-1,1) = 
     &  + HZ1( -1)*HZ5(0,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,-1,-1,-1,-1,1)
     &  - HZ6(0, -1,-1,-1,1,-1)
      HZ6(-1,0,-1,-1,-1,-1) = 
     &  + HZ1( -1)*HZ5(0,-1,-1,-1,-1)
     &  - 5.0000000000000000d+00*HZ6(0,-1,-1,-1,-1,-1)
      HZ6(-1,1,0,0,0,0) = 
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(0)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(0,0,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,0,0,1)
     &  + HZ1( -1)*HZ5(0,0,0,0,1)
     &  + 4.1666666666666666d-02*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)*HZ2(-1,1)
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,1,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,0,1,-1)
     &  + HZ1(0) *HZ5(0,0,0,1,-1)
     &  - HZ6(0,0,0,0,1, -1)
      HZ6(-1,1,0,0,0,1) = 
     &  - HZ1( -1)*HZ2(0,1)*HZ3(0,0,1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,0,1)
     &  + HZ2( -1,1)*HZ4(0,0,0,1)
     &  + HZ3(0,0,1) *HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,0,1,1,-1)
     &  - HZ6(0,0,1, -1,0,1)
     &  - HZ6(0,0,1,0, -1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,0,1,-1)
      HZ6(-1,1,0,0,0,-1) = 
     &  - HZ1( -1)*HZ2(0,1)*HZ3(0,0,-1)
     &  + HZ1( -1)*HZ5(0,0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,1,-1)
     &  + HZ1( -1)*HZ5(0,0,1,0,-1)
     &  + HZ2( -1,1)*HZ4(0,0,0,-1)
     &  + HZ3(0,0, -1)*HZ3(0,1,-1)
     &  - HZ6(0,0, -1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,0,1,-1,-1)
     &  - HZ6(0,0,1, -1,0,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,0,-1,-1)
      HZ6(-1,1,0,0,1,0) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ2(0,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ2(0,1)*HZ3(0,0,1)
     &  - 6.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,0,1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(0,0,1)
     &  + HZ1(0) *HZ2(0,1)*HZ3(0,1,-1)
     &  - HZ1(0) *HZ5(0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,1,-1)
     &  - HZ1(0) *HZ5(0,1,0,1,-1)
     &  - 3.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,0,1)
     &  - 2.0000000000000000d+00*HZ3(0,0,1)*HZ3(0,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,-1,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,1,1,-1)
     &  + HZ6(0,0,1, -1,0,1)
     &  + HZ6(0,0,1,0, -1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,1,0,1,-1)
      HZ6(-1,1,0,0,1,1) = 
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,1,1)
     &  - HZ1( -1)*HZ5(0,1,0,1,1)
     &  + HZ2( -1,1)*HZ4(0,0,1,1)
     &  + HZ2(0,1) *HZ4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  - HZ6(0, -1,1,1,0,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
     &  - HZ6(0,0,1, -1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,1,1,1,-1)
     &  + HZ6(0,1, -1,0,1,1)
     &  + HZ6(0,1,0,1, -1,1)
     &  + HZ6(0,1,0,1,1, -1)
      HZ6(-1,1,0,0,1,-1) = 
     &  - HZ1( -1)*HZ5(0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,1,-1)
     &  - HZ1( -1)*HZ5(0,1,0,1,-1)
     &  + HZ2( -1,1)*HZ4(0,0,1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,1,-1)*HZ3(0,1,-1)
      HZ6(-1,1,0,0,-1,0) = 
     &  - HZ1( -1)*HZ1(0)*HZ2(0,-1)*HZ2(0,1)
     &  + HZ1( -1)*HZ1(0)*HZ4(0,-1,0,1)
     &  + HZ1( -1)*HZ1(0)*HZ4(0,0,-1,1)
     &  + HZ1( -1)*HZ1(0)*HZ4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ2(0,1)*HZ3(0,0,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,1,-1)
     &  - HZ1( -1)*HZ5(0,0,1,0,-1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(0,0,-1)
     &  + HZ1(0) *HZ2(0,-1)*HZ3(0,1,-1)
     &  - HZ1(0) *HZ5(0,-1,0,1,-1)
     &  - HZ1(0) *HZ5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HZ3(0,0,-1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,0,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,0,1,-1,-1)
     &  + HZ6(0,0,1, -1,0,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,0,-1,-1)
      HZ6(-1,1,0,0,-1,1) = 
     &  - HZ1( -1)*HZ2(0,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,0,1,1)
     &  + HZ1( -1)*HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,1,1)
     &  + HZ1( -1)*HZ5(0,0,1,-1,1)
     &  + HZ2( -1,1)*HZ4(0,0,-1,1)
     &  + HZ3(0, -1,1)*HZ3(0,1,-1)
     &  - HZ6(0, -1,0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,1,1,-1)
     &  - HZ6(0, -1,1,0,1,-1)
     &  - HZ6(0,0, -1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,-1,1)
     &  - HZ6(0,0,1, -1,1,-1)
      HZ6(-1,1,0,0,-1,-1) = 
     &  - HZ1( -1)*HZ2(0,1)*HZ3(0,-1,-1)
     &  + HZ1( -1)*HZ5(0,-1,-1,0,1)
     &  + HZ1( -1)*HZ5(0,-1,0,-1,1)
     &  + HZ1( -1)*HZ5(0,-1,0,1,-1)
     &  + HZ1( -1)*HZ5(0,0,-1,-1,1)
     &  + HZ1( -1)*HZ5(0,0,-1,1,-1)
     &  + HZ1( -1)*HZ5(0,0,1,-1,-1)
     &  + HZ2( -1,1)*HZ4(0,0,-1,-1)
     &  + HZ3(0, -1,-1)*HZ3(0,1,-1)
     &  - HZ6(0, -1,-1,0,1,-1)
     &  - HZ6(0, -1,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,1,-1,-1)
     &  - HZ6(0,0, -1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,1,-1,-1,-1)
      HZ6(-1,1,0,1,0,0) = 
     &  - HZ1( -1)*HZ1(0)*HZ1(0)*HZ3(0,1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ2(0,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,0,1,1)
     &  - HZ1( -1)*HZ2(0,1)*HZ3(0,0,1)
     &  + HZ1( -1)*HZ5(0,0,1,0,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(-1,1)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,1,-1,1)
     &  + HZ1(0) *HZ1(0)*HZ4(0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ2(-1,1)*HZ3(0,0,1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(0,1,-1)
     &  + HZ1(0) *HZ5(0,1,0,1,-1)
     &  + 3.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,0,1)
     &  + HZ3(0,0,1) *HZ3(0,1,-1)
     &  - HZ6(0,0,1,0,1, -1)
      HZ6(-1,1,0,1,0,1) = 
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ2(0,1)*HZ3(0,1,1)
     &  + 1.2000000000000000d+01*HZ1(-1)*HZ5(0,0,1,1,1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,1,0,1,1)
     &  + 5.0000000000000000d-01*HZ2(-1,1)*HZ2(0,1)*HZ2(0,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,1,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,-1,1,1)
     &  + HZ2(0,1) *HZ4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,1,1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,1,0,1)
     &  + 1.2000000000000000d+01*HZ6(0,0,-1,1,1,1)
     &  - 8.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  - 1.2000000000000000d+01*HZ6(0,0,1,1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,0,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,1,0,1,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,1,0,1,1,-1)
      HZ6(-1,1,0,1,0,-1) = 
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ2(0,-1)*HZ3(0,1,1)
     &  + HZ1( -1)*HZ2(0,1)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,-1,1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,1,-1)
     &  + HZ1( -1)*HZ5(0,1,0,1,-1)
     &  + HZ2( -1,1)*HZ2(0,-1)*HZ2(0,1)
     &  - HZ2( -1,1)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,1,-1)
     &  + HZ2(0, -1)*HZ4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,1,1,-1)
     &  - HZ3(0, -1,1)*HZ3(0,1,-1)
     &  - 5.0000000000000000d-01*HZ3(0,1,-1)*HZ3(0,1,-1)
     &  + HZ6(0, -1,1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,-1,-1)
      HZ6(-1,1,0,1,1,0) = 
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,1,1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ2(0,1)*HZ3(0,1,1)
     &  - 6.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,1,0,1,1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(0,1,1)
     &  + HZ1(0) *HZ5(0,1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,1,-1,1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ5(0,1,1,1,-1)
     &  - 5.0000000000000000d-01*HZ2(-1,1)*HZ2(0,1)*HZ2(0,1)
     &  + HZ2(0,1) *HZ4(0,-1,1,1)
     &  - HZ2(0,1) *HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,1,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,0,1,1)
     &  - HZ6(0, -1,1,1,0,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,-1,1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,-1,1)
     &  + 6.0000000000000000d+00*HZ6(0,0,1,1,1,-1)
     &  + HZ6(0,1, -1,0,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,0,1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,1,0,1,1,-1)
      HZ6(-1,1,0,1,1,1) = 
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(0,1,1,1,1)
     &  + HZ2( -1,1)*HZ4(0,1,1,1)
     &  + HZ6(0,1, -1,1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,1,-1,1,1)
     &  + 3.0000000000000000d+00*HZ6(0,1,1,1,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,1,1,1,1,-1)
      HZ6(-1,1,0,1,1,-1) = 
     &  - HZ1( -1)*HZ5(0,1,1,-1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,1,1,1,-1)
     &  + HZ2( -1,1)*HZ4(0,1,1,-1)
     &  + HZ6(0,1, -1,1,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,1,1,-1,1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,1,1,1,-1,-1)
      HZ6(-1,1,0,1,-1,0) = 
     &  - HZ1( -1)*HZ1(0)*HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ2(0,-1)*HZ3(0,1,1)
     &  - HZ1( -1)*HZ2(0,1)*HZ3(0,-1,1)
     &  + HZ1( -1)*HZ5(0,-1,1,0,1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,-1,1,-1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(0,1,1,-1,-1)
     &  - HZ2( -1,1)*HZ2(0,-1)*HZ2(0,1)
     &  + HZ2( -1,1)*HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,-1,1)
     &  - HZ2(0, -1)*HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,1,1,-1)
     &  + HZ3(0, -1,1)*HZ3(0,1,-1)
     &  - HZ6(0, -1,1,0,1,-1)
      HZ6(-1,1,0,1,-1,1) = 
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,1,-1,1)
     &  + HZ2( -1,1)*HZ4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,1,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,1,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,1,-1,1,-1)
      HZ6(-1,1,0,1,-1,-1) = 
     &  - HZ1( -1)*HZ5(0,1,-1,-1,1)
     &  - HZ1( -1)*HZ5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,1,-1,-1)
     &  + HZ2( -1,1)*HZ4(0,1,-1,-1)
     &  + HZ6(0,1, -1,-1,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,1,-1,1,-1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,1,1,-1,-1,-1)
      HZ6(-1,1,0,-1,0,0) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(0,1,-1)
     &  + HZ1( -1)*HZ1(0)*HZ2(0,-1)*HZ2(0,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,-1,0,1)
     &  - HZ1( -1)*HZ2(0,1)*HZ3(0,0,-1)
     &  + HZ1( -1)*HZ5(0,0,-1,0,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(-1,1)*HZ2(0,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,1,-1)
     &  + HZ1(0) *HZ1(0)*HZ4(0,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ2(-1,1)*HZ3(0,0,-1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(0,1,-1)
     &  + HZ1(0) *HZ5(0,-1,0,1,-1)
     &  + 3.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,0,-1)
     &  + HZ3(0,0, -1)*HZ3(0,1,-1)
     &  - HZ6(0,0, -1,0,1,-1)
      HZ6(-1,1,0,-1,0,1) = 
     &  + HZ1( -1)*HZ2(0,1)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ2(0,1)*HZ3(0,1,-1)
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,0,1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,1,0,1)
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,1,1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,0,1,-1)
     &  + HZ2( -1,1)*HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,1,-1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  - HZ3(0, -1,1)*HZ3(0,1,-1)
     &  - 5.0000000000000000d-01*HZ3(0,1,-1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  + HZ6(0, -1,0,1,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,-1,0,1,1,-1)
     &  + HZ6(0, -1,1,-1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,1,0,1,-1,-1)
      HZ6(-1,1,0,-1,0,-1) = 
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ2(0,1)*HZ3(0,-1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,0,1)
     &  - HZ1( -1)*HZ5(0,-1,0,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,0,1,-1)
     &  + 5.0000000000000000d-01*HZ2(-1,1)*HZ2(0,-1)*HZ2(0,-1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,-1,-1)
     &  + HZ2(0, -1)*HZ4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,0,1,-1)
     &  + HZ6(0, -1,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,1,-1,-1)
      HZ6(-1,1,0,-1,1,0) = 
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,-1,1,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,1,-1,1)
     &  + HZ1( -1)*HZ2(0,1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,0,1,1)
     &  + HZ1( -1)*HZ5(0,-1,1,0,1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,1,1)
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,0,1,-1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(0,-1,1)
     &  + HZ1(0) *HZ5(0,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,-1,-1,1)
     &  + HZ1(0) *HZ5(0,1,-1,1,-1)
     &  - HZ2( -1,1)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  + 5.0000000000000000d-01*HZ3(0,1,-1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,1,1,-1)
     &  - HZ6(0, -1,1,-1,0,1)
     &  - HZ6(0, -1,1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,1,1,-1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,1,0,1,-1,-1)
      HZ6(-1,1,0,-1,1,1) = 
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,1,1,1)
     &  - HZ1( -1)*HZ5(0,1,-1,1,1)
     &  + HZ2( -1,1)*HZ4(0,-1,1,1)
     &  + HZ6(0, -1,1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,-1,1,1)
     &  + HZ6(0,1, -1,1,-1,1)
     &  + HZ6(0,1, -1,1,1,-1)
      HZ6(-1,1,0,-1,1,-1) = 
     &  - HZ1( -1)*HZ5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,1,1,-1)
     &  - HZ1( -1)*HZ5(0,1,-1,1,-1)
     &  + HZ2( -1,1)*HZ4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,-1,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,-1,1,1,-1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,1,-1,-1)
      HZ6(-1,1,0,-1,-1,0) = 
     &  - HZ1( -1)*HZ1(0)*HZ4(0,-1,-1,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,-1,1,-1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,1,-1,-1)
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(0,-1,1)
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(0,1,-1)
     &  - HZ1( -1)*HZ2(0,1)*HZ3(0,-1,-1)
     &  + HZ1( -1)*HZ5(0,-1,-1,0,1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(0,-1,-1)
     &  + HZ1(0) *HZ5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ5(0,1,-1,-1,-1)
     &  - 5.0000000000000000d-01*HZ2(-1,1)*HZ2(0,-1)*HZ2(0,-1)
     &  - HZ2(0, -1)*HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(0,1,-1,-1)
     &  + HZ3(0, -1,-1)*HZ3(0,1,-1)
     &  - HZ6(0, -1,-1,0,1,-1)
      HZ6(-1,1,0,-1,-1,1) = 
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,1,1)
     &  - HZ1( -1)*HZ5(0,-1,1,-1,1)
     &  - HZ1( -1)*HZ5(0,1,-1,-1,1)
     &  + HZ2( -1,1)*HZ4(0,-1,-1,1)
     &  + HZ6(0, -1,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,-1,-1,1)
     &  + HZ6(0, -1,1,-1,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,1,-1,-1,-1,1)
     &  + HZ6(0,1, -1,-1,1,-1)
      HZ6(-1,1,0,-1,-1,-1) = 
     &  - HZ1( -1)*HZ5(0,-1,-1,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,-1,1,-1)
     &  - HZ1( -1)*HZ5(0,-1,1,-1,-1)
     &  - HZ1( -1)*HZ5(0,1,-1,-1,-1)
     &  + HZ2( -1,1)*HZ4(0,-1,-1,-1)
     &  + HZ6(0, -1,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,1,-1,-1,-1,-1)
      HZ6(-1,1,1,0,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(0,1,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,0,1,1)
     &  + HZ1( -1)*HZ5(0,0,0,1,1)
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(-1,1)*HZ2(0,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,1,1,-1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(0,0,1)
     &  + HZ1(0) *HZ5(0,0,1,1,-1)
     &  - HZ2( -1,1)*HZ4(0,0,0,1)
     &  - HZ6(0,0,0,1,1, -1)
      HZ6(-1,1,1,0,0,1) = 
     &  + HZ1( -1)*HZ2(0,1)*HZ3(0,1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,1,1)
     &  - HZ1( -1)*HZ5(0,1,0,1,1)
     &  - 5.0000000000000000d-01*HZ2(-1,1)*HZ2(0,1)*HZ2(0,1)
     &  - HZ2(0,1) *HZ4(0,1,1,-1)
     &  + HZ3( -1,1,1)*HZ3(0,0,1)
     &  + HZ6(0,0,1,1, -1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,1,1,1,-1)
     &  + HZ6(0,1,0,1,1, -1)
      HZ6(-1,1,1,0,0,-1) = 
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(0,1,1)
     &  - HZ1( -1)*HZ5(0,-1,0,1,1)
     &  - HZ1( -1)*HZ5(0,0,-1,1,1)
     &  - HZ1( -1)*HZ5(0,0,1,-1,1)
     &  - HZ1( -1)*HZ5(0,0,1,1,-1)
     &  - HZ2( -1,1)*HZ2(0,-1)*HZ2(0,1)
     &  + HZ2( -1,1)*HZ4(0,-1,0,1)
     &  + HZ2( -1,1)*HZ4(0,0,-1,1)
     &  + HZ2( -1,1)*HZ4(0,0,1,-1)
     &  - HZ2(0, -1)*HZ4(0,1,1,-1)
     &  + HZ3( -1,1,1)*HZ3(0,0,-1)
     &  + HZ6(0, -1,0,1,1,-1)
     &  + HZ6(0,0, -1,1,1,-1)
     &  + HZ6(0,0,1, -1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,1,-1,-1)
      HZ6(-1,1,1,0,1,0) = 
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,1,1,1)
     &  - HZ1( -1)*HZ2(0,1)*HZ3(0,1,1)
     &  + HZ1( -1)*HZ5(0,1,0,1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ2(-1,1)*HZ3(0,1,1)
     &  + HZ1(0) *HZ2(0,1)*HZ3(-1,1,1)
     &  - HZ1(0) *HZ5(0,1,1,-1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,1,1,1,-1)
     &  + 5.0000000000000000d-01*HZ2(-1,1)*HZ2(0,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,1,1)
     &  + HZ2(0,1) *HZ4(0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,0,1)
     &  - HZ6(0,1,0,1,1, -1)
      HZ6(-1,1,1,0,1,1) = 
     &  + 6.0000000000000000d+00*HZ1(-1)*HZ5(0,1,1,1,1)
     &  - 3.0000000000000000d+00*HZ2(-1,1)*HZ4(0,1,1,1)
     &  + HZ3( -1,1,1)*HZ3(0,1,1)
     &  - HZ6(0,1,1, -1,1,1)
     &  - 3.0000000000000000d+00*HZ6(0,1,1,1,-1,1)
     &  - 6.0000000000000000d+00*HZ6(0,1,1,1,1,-1)
      HZ6(-1,1,1,0,1,-1) = 
     &  + HZ1( -1)*HZ5(0,1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,1,-1,1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ5(0,1,1,1,-1)
     &  - HZ2( -1,1)*HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,1,1,-1)
     &  + HZ3( -1,1,1)*HZ3(0,1,-1)
     &  - HZ6(0,1, -1,1,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,1,1,-1,1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,1,1,1,-1,-1)
      HZ6(-1,1,1,0,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ4(0,-1,1,1)
     &  + HZ1( -1)*HZ1(0)*HZ4(0,1,-1,1)
     &  + HZ1( -1)*HZ1(0)*HZ4(0,1,1,-1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(0,1,1)
     &  + HZ1( -1)*HZ5(0,-1,0,1,1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(0,-1,1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(0,1,-1)
     &  + HZ1(0) *HZ2(0,-1)*HZ3(-1,1,1)
     &  - HZ1(0) *HZ5(0,-1,1,1,-1)
     &  - HZ1(0) *HZ5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,1,-1,-1)
     &  + HZ2( -1,1)*HZ2(0,-1)*HZ2(0,1)
     &  - HZ2( -1,1)*HZ4(0,-1,0,1)
     &  + HZ2(0, -1)*HZ4(0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,0,-1)
     &  - HZ6(0, -1,0,1,1,-1)
      HZ6(-1,1,1,0,-1,1) = 
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,-1,1,1)
     &  + HZ1( -1)*HZ5(0,1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,-1,1,1)
     &  - HZ2( -1,1)*HZ4(0,1,-1,1)
     &  + HZ3( -1,1,1)*HZ3(0,-1,1)
     &  - HZ6(0, -1,1,1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,1,1,1,-1)
     &  - HZ6(0,1, -1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,1,1,-1,-1,1)
     &  - HZ6(0,1,1, -1,1,-1)
      HZ6(-1,1,1,0,-1,-1) = 
     &  + HZ1( -1)*HZ5(0,-1,-1,1,1)
     &  + HZ1( -1)*HZ5(0,-1,1,-1,1)
     &  + HZ1( -1)*HZ5(0,-1,1,1,-1)
     &  + HZ1( -1)*HZ5(0,1,-1,-1,1)
     &  + HZ1( -1)*HZ5(0,1,-1,1,-1)
     &  + HZ1( -1)*HZ5(0,1,1,-1,-1)
     &  - HZ2( -1,1)*HZ4(0,-1,-1,1)
     &  - HZ2( -1,1)*HZ4(0,-1,1,-1)
     &  - HZ2( -1,1)*HZ4(0,1,-1,-1)
     &  + HZ3( -1,1,1)*HZ3(0,-1,-1)
     &  - HZ6(0, -1,-1,1,1,-1)
     &  - HZ6(0, -1,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,1,-1,-1)
     &  - HZ6(0,1, -1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,1,1,-1,-1,-1)
      HZ6(-1,1,1,1,0,0) = 
     &  - HZ1( -1)*HZ1(0)*HZ4(0,1,1,1)
     &  + HZ1( -1)*HZ5(0,0,1,1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(-1,1,1,1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(0,1,1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(-1,1,1)
     &  + HZ1(0) *HZ5(0,1,1,1,-1)
     &  - HZ2( -1,1)*HZ4(0,0,1,1)
     &  + HZ3( -1,1,1)*HZ3(0,0,1)
     &  - HZ6(0,0,1,1,1, -1)
      HZ6(-1,1,1,1,0,1) = 
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(0,1,1,1,1)
     &  + 3.0000000000000000d+00*HZ2(-1,1)*HZ4(0,1,1,1)
     &  + HZ2(0,1) *HZ4(-1,1,1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,1,1)
     &  + HZ6(0,1,1,1, -1,1)
     &  + 4.0000000000000000d+00*HZ6(0,1,1,1,1,-1)
      HZ6(-1,1,1,1,0,-1) = 
     &  - HZ1( -1)*HZ5(0,-1,1,1,1)
     &  - HZ1( -1)*HZ5(0,1,-1,1,1)
     &  - HZ1( -1)*HZ5(0,1,1,-1,1)
     &  - HZ1( -1)*HZ5(0,1,1,1,-1)
     &  + HZ2( -1,1)*HZ4(0,-1,1,1)
     &  + HZ2( -1,1)*HZ4(0,1,-1,1)
     &  + HZ2( -1,1)*HZ4(0,1,1,-1)
     &  + HZ2(0, -1)*HZ4(-1,1,1,1)
     &  - HZ3( -1,1,1)*HZ3(0,-1,1)
     &  - HZ3( -1,1,1)*HZ3(0,1,-1)
     &  + HZ6(0, -1,1,1,1,-1)
     &  + HZ6(0,1, -1,1,1,-1)
     &  + HZ6(0,1,1, -1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,1,1,1,-1,-1)
      HZ6(-1,1,1,1,1,0) = 
     &  + HZ1( -1)*HZ5(0,1,1,1,1)
     &  + HZ1(0) *HZ5(-1,1,1,1,1)
     &  - HZ2( -1,1)*HZ4(0,1,1,1)
     &  - HZ2(0,1) *HZ4(-1,1,1,1)
     &  + HZ3( -1,1,1)*HZ3(0,1,1)
     &  - HZ6(0,1,1,1,1, -1)
      HZ6(-1,1,1,1,1,-1) = 
     &  + HZ1( -1)*HZ5(-1,1,1,1,1)
     &  - HZ2( -1,1)*HZ4(-1,1,1,1)
     &  + 5.0000000000000000d-01*HZ3(-1,1,1)*HZ3(-1,1,1)
      HZ6(-1,1,1,1,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ4(-1,1,1,1)
     &  + HZ1( -1)*HZ5(0,-1,1,1,1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(-1,1,1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(-1,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(-1,1,-1,1,1)
     &  - HZ2( -1,1)*HZ4(0,-1,1,1)
     &  - HZ2(0, -1)*HZ4(-1,1,1,1)
     &  + HZ3( -1,1,1)*HZ3(0,-1,1)
     &  - HZ6(0, -1,1,1,1,-1)
      HZ6(-1,1,1,1,-1,1) = 
     &  + HZ2( -1,1)*HZ4(-1,1,1,1)
     &  - HZ3( -1,1,1)*HZ3(-1,1,1)
     &  + 4.0000000000000000d+00*HZ6(-1,-1,1,1,1,1)
     &  + 2.0000000000000000d+00*HZ6(-1,1,-1,1,1,1)
      HZ6(-1,1,1,1,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(-1,1,1,1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(-1,1,1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(-1,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(-1,1,-1,1,1)
     &  - HZ2( -1,1)*HZ4(-1,-1,1,1)
     &  + HZ3( -1,-1,1)*HZ3(-1,1,1)
     &  + 3.0000000000000000d+00*HZ6(-1,-1,-1,1,1,1)
     &  + HZ6( -1,-1,1,-1,1,1)
     &  + HZ6( -1,-1,1,1,-1,1)
      HZ6(-1,1,1,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(-1,1,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,-1,1,1)
     &  + HZ1( -1)*HZ5(0,0,-1,1,1)
     &  - 2.5000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(-1,1)*HZ2(-1,1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(0,-1,1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(-1,1,1)
     &  + HZ1(0) *HZ5(0,-1,1,1,-1)
     &  - HZ2( -1,1)*HZ4(0,0,-1,1)
     &  + HZ3( -1,1,1)*HZ3(0,0,-1)
     &  - HZ6(0,0, -1,1,1,-1)
      HZ6(-1,1,1,-1,0,1) = 
     &  + HZ1( -1)*HZ2(0,1)*HZ3(-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,1,1,1)
     &  - HZ1( -1)*HZ5(0,1,-1,1,1)
     &  - 5.0000000000000000d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,-1,1,1)
     &  + HZ2( -1,1)*HZ4(0,1,-1,1)
     &  - HZ3( -1,1,1)*HZ3(0,-1,1)
     &  - HZ3( -1,1,1)*HZ3(0,1,-1)
     &  + HZ6(0, -1,1,1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,1,1,1,-1)
     &  + HZ6(0,1, -1,1,1,-1)
      HZ6(-1,1,1,-1,0,-1) = 
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,1,1)
     &  - HZ1( -1)*HZ5(0,-1,1,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,1,1,-1)
     &  - 5.0000000000000000d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,-1,-1,1)
     &  + HZ2( -1,1)*HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ3(-1,1,1)*HZ3(0,-1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,1,1,-1)
     &  + HZ6(0, -1,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,1,-1,-1)
      HZ6(-1,1,1,-1,1,0) = 
     &  - HZ1( -1)*HZ2(0,1)*HZ3(-1,1,1)
     &  + HZ1( -1)*HZ5(0,1,-1,1,1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(-1,1,1)
     &  - 6.0000000000000000d+00*HZ1(0)*HZ5(-1,-1,1,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(-1,1,-1,1,1)
     &  + 5.0000000000000000d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(0,1)
     &  - HZ2( -1,1)*HZ4(0,1,-1,1)
     &  + HZ3( -1,1,1)*HZ3(0,1,-1)
     &  - HZ6(0,1, -1,1,1,-1)
      HZ6(-1,1,1,-1,1,1) = 
     &  + 5.0000000000000000d-01*HZ3(-1,1,1)*HZ3(-1,1,1)
     &  - 6.0000000000000000d+00*HZ6(-1,-1,1,1,1,1)
     &  - 3.0000000000000000d+00*HZ6(-1,1,-1,1,1,1)
      HZ6(-1,1,1,-1,1,-1) = 
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(-1,1,1)
     &  - 6.0000000000000000d+00*HZ1(-1)*HZ5(-1,-1,1,1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(-1,1,-1,1,1)
     &  + 1.6666666666666666d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(-1,1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(-1,1,1)
     &  - 6.0000000000000000d+00*HZ6(-1,-1,-1,1,1,1)
     &  - 2.0000000000000000d+00*HZ6(-1,-1,1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(-1,-1,1,1,-1,1)
      HZ6(-1,1,1,-1,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ2(-1,1)*HZ2(-1,1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(-1,1,1)
     &  + HZ1( -1)*HZ5(0,-1,-1,1,1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(-1,-1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(-1,-1,-1,1,1)
     &  - HZ1(0) *HZ5(-1,-1,1,-1,1)
     &  + 5.0000000000000000d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(0,-1)
     &  - HZ2( -1,1)*HZ4(0,-1,-1,1)
     &  + HZ3( -1,1,1)*HZ3(0,-1,-1)
     &  - HZ6(0, -1,-1,1,1,-1)
      HZ6(-1,1,1,-1,-1,1) = 
     &  - 1.6666666666666666d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(-1,1)
     &  + HZ3( -1,-1,1)*HZ3(-1,1,1)
     &  - 3.0000000000000000d+00*HZ6(-1,-1,-1,1,1,1)
     &  - HZ6( -1,-1,1,-1,1,1)
      HZ6(-1,1,1,-1,-1,-1) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(-1,1,1)
     &  - 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(-1,1)*HZ2(-1,1)
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(-1,-1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(-1,-1,-1,1,1)
     &  - HZ1( -1)*HZ5(-1,-1,1,-1,1)
     &  - HZ2( -1,1)*HZ4(-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HZ6(-1,-1,-1,-1,1,1)
     &  + HZ6( -1,-1,-1,1,-1,1)
      HZ6(-1,1,-1,0,0,0) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(0)*HZ2(-1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,0,-1,1)
     &  + HZ1( -1)*HZ5(0,0,0,-1,1)
     &  - 3.3333333333333333d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(-1,1)*HZ2(0,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,1,-1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(0,0,-1)
     &  + HZ1(0) *HZ5(0,0,-1,1,-1)
     &  - HZ2( -1,1)*HZ4(0,0,0,-1)
     &  - HZ6(0,0,0, -1,1,-1)
      HZ6(-1,1,-1,0,0,1) = 
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(0,0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,0,1,1)
     &  + HZ1( -1)*HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,1,1)
     &  + HZ1( -1)*HZ5(0,0,1,-1,1)
     &  - HZ2( -1,1)*HZ4(0,-1,0,1)
     &  - HZ2( -1,1)*HZ4(0,0,-1,1)
     &  - HZ2( -1,1)*HZ4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,0,1)
     &  - 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,1,1,-1)
     &  - HZ6(0, -1,1,-1,0,1)
     &  - HZ6(0, -1,1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  + HZ6(0,0, -1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
     &  - HZ6(0,0,1, -1,1,-1)
      HZ6(-1,1,-1,0,0,-1) = 
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(0,0,-1)
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,-1,1)
     &  - HZ1( -1)*HZ5(0,0,-1,1,-1)
     &  - 5.0000000000000000d-01*HZ2(-1,1)*HZ2(0,-1)*HZ2(0,-1)
     &  - HZ2(0, -1)*HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,0,-1)
     &  + HZ6(0, -1,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,-1)
      HZ6(-1,1,-1,0,1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ2(-1,1)*HZ2(0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,-1,1,1)
     &  + HZ1( -1)*HZ1(0)*HZ4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ2(-1,1)*HZ3(0,0,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,0,1,1)
     &  - HZ1( -1)*HZ5(0,-1,1,0,1)
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,-1,1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(0,-1,1)
     &  - HZ1(0) *HZ2(-1,1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ2(0,1)*HZ3(-1,-1,1)
     &  - HZ1(0) *HZ5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,1,1,-1)
     &  - HZ1(0) *HZ5(0,1,-1,1,-1)
     &  + HZ2( -1,1)*HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,1,-1)
     &  + 4.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,0,1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,1,1,-1)
     &  + HZ6(0, -1,1,-1,0,1)
     &  + HZ6(0, -1,1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,1,-1)
      HZ6(-1,1,-1,0,1,1) = 
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(0,1,1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,-1,1,1)
     &  + HZ1( -1)*HZ5(0,1,1,-1,1)
     &  - HZ2( -1,1)*HZ4(0,-1,1,1)
     &  - HZ2( -1,1)*HZ4(0,1,-1,1)
     &  - HZ2( -1,1)*HZ4(0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,1,1)
     &  - HZ6(0, -1,1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,1,1,1,-1)
     &  - HZ6(0,1, -1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,1,1,-1)
     &  - HZ6(0,1,1, -1,1,-1)
      HZ6(-1,1,-1,0,1,-1) = 
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(0,1,-1)
     &  + HZ1( -1)*HZ5(0,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,-1,-1,1)
     &  + HZ1( -1)*HZ5(0,1,-1,1,-1)
     &  - HZ2( -1,1)*HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,-1,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,-1,1,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,1,-1,-1)
      HZ6(-1,1,-1,0,-1,0) = 
     &  + HZ1( -1)*HZ1(0)*HZ2(-1,1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,-1,-1,1)
     &  + HZ1( -1)*HZ1(0)*HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ2(-1,1)*HZ3(0,0,-1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(0,-1,1)
     &  + HZ1( -1)*HZ5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ2(-1,1)*HZ3(0,-1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ2(0,-1)*HZ3(-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,1,-1,-1)
     &  + 5.0000000000000000d-01*HZ2(-1,1)*HZ2(0,-1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,0,-1,-1)
     &  + HZ2(0, -1)*HZ4(0,-1,1,-1)
     &  + 4.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,0,-1)
     &  - HZ6(0, -1,0,-1,1,-1)
      HZ6(-1,1,-1,0,-1,1) = 
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(0,-1,1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,-1,-1,1)
     &  - HZ2( -1,1)*HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,-1,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,-1,1,-1)
      HZ6(-1,1,-1,0,-1,-1) = 
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(0,-1,-1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,1,-1)
     &  + HZ1( -1)*HZ5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ2(-1,1)*HZ4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,-1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,-1,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,1,-1,-1,-1)
      HZ6(-1,1,-1,1,0,0) = 
     &  - HZ1( -1)*HZ1(0)*HZ2(-1,1)*HZ2(0,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,1,-1,1)
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(0,0,1)
     &  + HZ1( -1)*HZ5(0,0,1,-1,1)
     &  + 2.5000000000000000d-01*HZ1(0)*HZ1(0)*HZ2(-1,1)*HZ2(-1,1)
     &  - HZ1(0) *HZ1(0)*HZ4(-1,-1,1,1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ2(0,1)*HZ3(-1,-1,1)
     &  + HZ1(0) *HZ5(0,1,-1,1,-1)
     &  - HZ2( -1,1)*HZ4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,0,1)
     &  - HZ6(0,0,1, -1,1,-1)
      HZ6(-1,1,-1,1,0,1) = 
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ2(-1,1)*HZ3(0,1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,1,-1,1)
     &  + 5.0000000000000000d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(0,1)
     &  + HZ2( -1,1)*HZ4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,1,1,-1)
     &  - 2.0000000000000000d+00*HZ2(0,1)*HZ4(-1,-1,1,1)
     &  + 4.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,1,1)
     &  + HZ6(0,1, -1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,1,1,-1,1,-1)
      HZ6(-1,1,-1,1,0,-1) = 
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(0,1,-1)
     &  - HZ1( -1)*HZ5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,-1,-1,1)
     &  - HZ1( -1)*HZ5(0,1,-1,1,-1)
     &  + 5.0000000000000000d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(0,-1)
     &  + HZ2( -1,1)*HZ4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ2(-1,1)*HZ4(0,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ2(0,-1)*HZ4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,1,-1)
     &  + HZ6(0, -1,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,1,-1,-1)
      HZ6(-1,1,-1,1,1,0) = 
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(0,1,1)
     &  + HZ1( -1)*HZ5(0,1,1,-1,1)
     &  + HZ1(0) *HZ5(-1,1,-1,1,1)
     &  - 5.0000000000000000d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(0,1)
     &  - HZ2( -1,1)*HZ4(0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ2(0,1)*HZ4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,1,1)
     &  - HZ6(0,1,1, -1,1,-1)
      HZ6(-1,1,-1,1,1,-1) = 
     &  + HZ1( -1)*HZ5(-1,1,-1,1,1)
     &  - 1.6666666666666666d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(-1,1)
     &  + 6.0000000000000000d+00*HZ6(-1,-1,-1,1,1,1)
     &  + 2.0000000000000000d+00*HZ6(-1,-1,1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(-1,-1,1,1,-1,1)
      HZ6(-1,1,-1,1,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ2(-1,1)*HZ2(-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(-1,-1,1,1)
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(0,-1,1)
     &  + HZ1( -1)*HZ5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ2(-1,1)*HZ3(-1,-1,1)
     &  + 1.2000000000000000d+01*HZ1(0)*HZ5(-1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HZ1(0)*HZ5(-1,-1,1,-1,1)
     &  - 5.0000000000000000d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(0,-1)
     &  - HZ2( -1,1)*HZ4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ2(0,-1)*HZ4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,-1,1)
     &  - HZ6(0, -1,1,-1,1,-1)
      HZ6(-1,1,-1,1,-1,1) = 
     &  + 1.6666666666666666d-01*HZ2(-1,1)*HZ2(-1,1)*HZ2(-1,1)
     &  - 2.0000000000000000d+00*HZ2(-1,1)*HZ4(-1,-1,1,1)
     &  + 1.2000000000000000d+01*HZ6(-1,-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HZ6(-1,-1,1,-1,1,1)
      HZ6(-1,1,-1,1,-1,-1) = 
     &  + 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(-1,1)*HZ2(-1,1)
     &  - HZ1( -1)*HZ1(-1)*HZ4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ2(-1,1)*HZ3(-1,-1,1)
     &  + 1.2000000000000000d+01*HZ1(-1)*HZ5(-1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HZ2(-1,1)*HZ4(-1,-1,-1,1)
     &  - HZ3( -1,-1,1)*HZ3(-1,-1,1)
     &  - 1.2000000000000000d+01*HZ6(-1,-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ6(-1,-1,-1,1,-1,1)
      HZ6(-1,1,-1,-1,0,0) = 
     &  + 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(0)*HZ2(-1,1)
     &  - HZ1( -1)*HZ1(0)*HZ1(0)*HZ3(-1,-1,1)
     &  - HZ1( -1)*HZ1(0)*HZ2(-1,1)*HZ2(0,-1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,-1,-1,1)
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(0,0,-1)
     &  + HZ1( -1)*HZ5(0,0,-1,-1,1)
     &  + 1.5000000000000000d+00*HZ1(0)*HZ1(0)*HZ4(-1,-1,-1,1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(0,-1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ2(0,-1)*HZ3(-1,-1,1)
     &  + HZ1(0) *HZ5(0,-1,-1,1,-1)
     &  - HZ2( -1,1)*HZ4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,0,-1)
     &  - HZ6(0,0, -1,-1,1,-1)
      HZ6(-1,1,-1,-1,0,1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(-1,1)*HZ2(0,1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ2(-1,1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ2(0,1)*HZ3(-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,1,1)
     &  - HZ1( -1)*HZ5(0,-1,1,-1,1)
     &  - HZ1( -1)*HZ5(0,1,-1,-1,1)
     &  + HZ2( -1,1)*HZ4(0,-1,-1,1)
     &  + HZ2( -1,1)*HZ4(0,-1,1,-1)
     &  + HZ2( -1,1)*HZ4(0,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ2(0,1)*HZ4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,-1,1)
     &  + 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,1,-1)
     &  + HZ6(0, -1,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,1,1,-1)
     &  + HZ6(0, -1,1,-1,1,-1)
     &  + HZ6(0,1, -1,-1,1,-1)
      HZ6(-1,1,-1,-1,0,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(-1,1)*HZ2(0,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ2(-1,1)*HZ3(0,-1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ2(0,-1)*HZ3(-1,-1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HZ2(-1,1)*HZ4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HZ2(0,-1)*HZ4(-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,-1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,1,-1,-1)
      HZ6(-1,1,-1,-1,1,0) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(-1,1)*HZ2(0,1)
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ2(0,1)*HZ3(-1,-1,1)
     &  + HZ1( -1)*HZ5(0,1,-1,-1,1)
     &  + HZ1(0) *HZ2(-1,1)*HZ3(-1,-1,1)
     &  - 6.0000000000000000d+00*HZ1(0)*HZ5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(-1,-1,1,-1,1)
     &  - HZ2( -1,1)*HZ4(0,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ2(0,1)*HZ4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,1,-1)
     &  - HZ6(0,1, -1,-1,1,-1)
      HZ6(-1,1,-1,-1,1,1) = 
     &  + HZ2( -1,1)*HZ4(-1,-1,1,1)
     &  - 9.0000000000000000d+00*HZ6(-1,-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HZ6(-1,-1,1,-1,1,1)
     &  - HZ6( -1,-1,1,1,-1,1)
      HZ6(-1,1,-1,-1,1,-1) = 
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(-1,-1,1)
     &  - 6.0000000000000000d+00*HZ1(-1)*HZ5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(-1,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HZ2(-1,1)*HZ4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(-1,-1,1)
      HZ6(-1,1,-1,-1,-1,0) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ2(-1,1)
     &  - HZ1( -1)*HZ1(-1)*HZ1(0)*HZ3(-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(-1,1)*HZ2(0,-1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(-1,-1,-1,1)
     &  + HZ1( -1)*HZ2(-1,1)*HZ3(0,-1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ2(0,-1)*HZ3(-1,-1,1)
     &  + HZ1( -1)*HZ5(0,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(-1,-1,-1,-1,1)
     &  - HZ2( -1,1)*HZ4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HZ2(0,-1)*HZ4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,-1,-1)
     &  - HZ6(0, -1,-1,-1,1,-1)
      HZ6(-1,1,-1,-1,-1,1) = 
     &  + HZ2( -1,1)*HZ4(-1,-1,-1,1)
     &  - HZ3( -1,-1,1)*HZ3(-1,-1,1)
     &  + 4.0000000000000000d+00*HZ6(-1,-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(-1,-1,-1,1,-1,1)
      HZ6(-1,1,-1,-1,-1,-1) = 
     &  + 4.1666666666666666d-02*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ2(-1,1)
     &  - 3.3333333333333333d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(-1,-1,1)
     &  + 1.5000000000000000d+00*HZ1(-1)*HZ1(-1)*HZ4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(-1,-1,-1,-1,1)
     &  + 5.0000000000000000d+00*HZ6(-1,-1,-1,-1,-1,1)
      HZ6(-1,-1,0,0,0,0) = 
     &  + 2.0833333333333333d-02*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(0)*HZ1(0)
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(0)*HZ2(0,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(0,0,-1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,0,0,-1)
     &  + HZ1( -1)*HZ5(0,0,0,0,-1)
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(0,-1,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,0,-1,-1)
     &  + HZ1(0) *HZ5(0,0,0,-1,-1)
     &  - HZ6(0,0,0,0, -1,-1)
      HZ6(-1,-1,0,0,0,1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,0,0,1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(0,0,1)
     &  + HZ1( -1)*HZ5(0,0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,1,-1)
     &  + HZ1( -1)*HZ5(0,0,1,0,-1)
     &  + HZ3(0, -1,-1)*HZ3(0,0,1)
     &  - HZ6(0,0, -1,-1,0,1)
     &  - HZ6(0,0, -1,0,-1,1)
     &  - HZ6(0,0, -1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,0,1,-1,-1)
     &  - HZ6(0,0,1,0, -1,-1)
      HZ6(-1,-1,0,0,0,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,0,0,-1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(0,0,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,0,-1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,-1,-1)
     &  + HZ3(0, -1,-1)*HZ3(0,0,-1)
     &  - HZ6(0,0, -1,-1,0,-1)
     &  - 3.0000000000000000d+00*HZ6(0,0,-1,0,-1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,0,-1,-1,-1)
      HZ6(-1,-1,0,0,1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(0,0,1)
     &  - 1.5000000000000000d+00*HZ1(-1)*HZ1(-1)*HZ4(0,0,0,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,-1,0,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,0,-1,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ2(0,-1)*HZ3(0,0,1)
     &  - HZ1( -1)*HZ5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,0,-1)
     &  + HZ1(0) *HZ5(0,-1,-1,0,1)
     &  + HZ1(0) *HZ5(0,-1,0,-1,1)
     &  + HZ1(0) *HZ5(0,-1,0,1,-1)
     &  + HZ1(0) *HZ5(0,0,-1,-1,1)
     &  + HZ1(0) *HZ5(0,0,-1,1,-1)
     &  + HZ1(0) *HZ5(0,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,0,1)
     &  + HZ6(0,0, -1,-1,0,1)
     &  + HZ6(0,0, -1,0,-1,1)
     &  + HZ6(0,0, -1,0,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,-1,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,0,1,-1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,0,-1,-1)
      HZ6(-1,-1,0,0,1,1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,0,1,1)
     &  - HZ1( -1)*HZ5(0,-1,0,1,1)
     &  - HZ1( -1)*HZ5(0,0,-1,1,1)
     &  - HZ1( -1)*HZ5(0,0,1,-1,1)
     &  - HZ1( -1)*HZ5(0,0,1,1,-1)
     &  + HZ6(0, -1,-1,0,1,1)
     &  + HZ6(0, -1,0,-1,1,1)
     &  + HZ6(0, -1,0,1,-1,1)
     &  + HZ6(0, -1,0,1,1,-1)
     &  + HZ6(0,0, -1,-1,1,1)
     &  + HZ6(0,0, -1,1,-1,1)
     &  + HZ6(0,0, -1,1,1,-1)
     &  + HZ6(0,0,1, -1,-1,1)
     &  + HZ6(0,0,1, -1,1,-1)
     &  + HZ6(0,0,1,1, -1,-1)
      HZ6(-1,-1,0,0,1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,0,1,-1)
     &  - HZ1( -1)*HZ5(0,-1,0,1,-1)
     &  - HZ1( -1)*HZ5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,-1,-1)
     &  + HZ6(0, -1,-1,0,1,-1)
     &  + HZ6(0, -1,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,1,-1,-1)
     &  + HZ6(0,0, -1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,1,-1,-1,-1)
      HZ6(-1,-1,0,0,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(0,0,-1)
     &  - 1.5000000000000000d+00*HZ1(-1)*HZ1(-1)*HZ4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ2(0,-1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ2(0,-1)*HZ3(0,0,-1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HZ1(-1)*HZ5(0,0,0,-1,-1)
     &  + HZ1(0) *HZ2(0,-1)*HZ3(0,-1,-1)
     &  - HZ1(0) *HZ5(0,-1,0,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HZ3(0,-1,-1)*HZ3(0,0,-1)
     &  + HZ6(0,0, -1,-1,0,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,0,-1,-1)
     &  + 9.0000000000000000d+00*HZ6(0,0,0,-1,-1,-1)
      HZ6(-1,-1,0,0,-1,1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,0,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,-1,1)
     &  - HZ1( -1)*HZ5(0,0,-1,1,-1)
     &  + HZ6(0, -1,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  + HZ6(0, -1,0,-1,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
     &  + HZ6(0,0, -1,1,-1,-1)
      HZ6(-1,-1,0,0,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,0,-1,-1)
     &  - HZ1( -1)*HZ5(0,-1,0,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,-1)*HZ3(0,-1,-1)
      HZ6(-1,-1,0,1,0,0) = 
     &  + 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(0)*HZ2(0,1)
     &  - HZ1( -1)*HZ1(-1)*HZ1(0)*HZ3(0,0,1)
     &  + 1.5000000000000000d+00*HZ1(-1)*HZ1(-1)*HZ4(0,0,0,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(0,1,-1)
     &  + HZ1( -1)*HZ1(0)*HZ4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,0,1,-1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(0,0,1)
     &  + HZ1( -1)*HZ5(0,0,1,0,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,1,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,1,-1,-1)
     &  - HZ1(0) *HZ5(0,-1,-1,0,1)
     &  - HZ1(0) *HZ5(0,-1,0,-1,1)
     &  - HZ1(0) *HZ5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,0,1,-1,-1)
     &  + HZ3(0, -1,-1)*HZ3(0,0,1)
     &  - HZ6(0,0,1,0, -1,-1)
      HZ6(-1,-1,0,1,0,1) = 
     &  + 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(0,1)*HZ2(0,1)
     &  - HZ1( -1)*HZ1(-1)*HZ4(0,0,1,1)
     &  - HZ1( -1)*HZ2(0,1)*HZ3(0,1,-1)
     &  - HZ1( -1)*HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,-1,1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,1,-1)
     &  + HZ1( -1)*HZ5(0,1,0,1,-1)
     &  + HZ2(0,1) *HZ4(0,1,-1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  + HZ6(0, -1,-1,1,0,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,1)
     &  - HZ6(0, -1,0,1,-1,1)
     &  + HZ6(0, -1,1,-1,0,1)
     &  + HZ6(0, -1,1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,1,-1,1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,0,1,1,-1,-1)
     &  - HZ6(0,1,0,1, -1,-1)
      HZ6(-1,-1,0,1,0,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(0,-1)*HZ2(0,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,0,1)
     &  - HZ1( -1)*HZ1(-1)*HZ4(0,0,-1,1)
     &  - HZ1( -1)*HZ1(-1)*HZ4(0,0,1,-1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(0,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,-1,-1)
     &  + HZ2(0, -1)*HZ4(0,-1,-1,1)
     &  + HZ2(0, -1)*HZ4(0,-1,1,-1)
     &  + HZ2(0, -1)*HZ4(0,1,-1,-1)
     &  - HZ6(0, -1,-1,0,-1,1)
     &  - HZ6(0, -1,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,-1,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,1,-1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,-1,1,-1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,1,-1,-1,-1)
      HZ6(-1,-1,0,1,1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(0,1,1)
     &  - 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(0,1)*HZ2(0,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,-1,1,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,1,-1,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,1,1,-1)
     &  + HZ1( -1)*HZ2(0,1)*HZ3(0,1,-1)
     &  + HZ1( -1)*HZ5(0,-1,0,1,1)
     &  + HZ1( -1)*HZ5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,1,-1)
     &  - HZ1( -1)*HZ5(0,1,0,1,-1)
     &  + HZ1(0) *HZ5(0,-1,-1,1,1)
     &  + HZ1(0) *HZ5(0,-1,1,-1,1)
     &  + HZ1(0) *HZ5(0,-1,1,1,-1)
     &  + HZ1(0) *HZ5(0,1,-1,-1,1)
     &  + HZ1(0) *HZ5(0,1,-1,1,-1)
     &  + HZ1(0) *HZ5(0,1,1,-1,-1)
     &  - HZ2(0,1) *HZ4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HZ3(0,-1,1)*HZ3(0,-1,1)
     &  - HZ6(0, -1,-1,0,1,1)
     &  - HZ6(0, -1,-1,1,0,1)
     &  + HZ6(0, -1,0,-1,1,1)
     &  - HZ6(0, -1,0,1,1,-1)
     &  - HZ6(0, -1,1,-1,0,1)
     &  - HZ6(0, -1,1,0,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,1,-1,-1)
     &  + HZ6(0,1,0,1, -1,-1)
      HZ6(-1,-1,0,1,1,1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,1,1,1)
     &  - HZ1( -1)*HZ5(0,-1,1,1,1)
     &  - HZ1( -1)*HZ5(0,1,-1,1,1)
     &  - HZ1( -1)*HZ5(0,1,1,-1,1)
     &  - HZ1( -1)*HZ5(0,1,1,1,-1)
     &  + HZ6(0, -1,-1,1,1,1)
     &  + HZ6(0, -1,1,-1,1,1)
     &  + HZ6(0, -1,1,1,-1,1)
     &  + HZ6(0, -1,1,1,1,-1)
     &  + HZ6(0,1, -1,-1,1,1)
     &  + HZ6(0,1, -1,1,-1,1)
     &  + HZ6(0,1, -1,1,1,-1)
     &  + HZ6(0,1,1, -1,-1,1)
     &  + HZ6(0,1,1, -1,1,-1)
     &  + HZ6(0,1,1,1, -1,-1)
      HZ6(-1,-1,0,1,1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,1,1,-1)
     &  - HZ1( -1)*HZ5(0,-1,1,1,-1)
     &  - HZ1( -1)*HZ5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,1,-1,-1)
     &  + HZ6(0, -1,-1,1,1,-1)
     &  + HZ6(0, -1,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,1,-1,-1)
     &  + HZ6(0,1, -1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,1,1,-1,-1,-1)
      HZ6(-1,-1,0,1,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(0,1,-1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(0,-1)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,0,1)
     &  + HZ1( -1)*HZ1(-1)*HZ4(0,0,-1,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,1,-1,-1)
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(0,-1,1)
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,0,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,0,1,-1)
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,1,-1)
     &  + HZ1(0) *HZ5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ5(0,1,-1,-1,-1)
     &  - HZ2(0, -1)*HZ4(0,-1,-1,1)
     &  - HZ2(0, -1)*HZ4(0,-1,1,-1)
     &  - HZ2(0, -1)*HZ4(0,1,-1,-1)
     &  + HZ6(0, -1,-1,0,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,-1)
     &  + HZ6(0, -1,0,1,-1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,-1)
      HZ6(-1,-1,0,1,-1,1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,1,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,-1,-1,1)
     &  - HZ1( -1)*HZ5(0,1,-1,1,-1)
     &  + HZ6(0, -1,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,-1,-1,1)
     &  + HZ6(0, -1,1,-1,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,1,-1,-1,1,-1)
     &  + HZ6(0,1, -1,1,-1,-1)
      HZ6(-1,-1,0,1,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,1,-1,-1)
     &  - HZ1( -1)*HZ5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,1,-1,-1,-1)
     &  + HZ6(0, -1,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,1,-1,-1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,1,-1,-1,-1,-1)
      HZ6(-1,-1,0,-1,0,0) = 
     &  + 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(0)*HZ2(0,-1)
     &  - HZ1( -1)*HZ1(-1)*HZ1(0)*HZ3(0,0,-1)
     &  + 1.5000000000000000d+00*HZ1(-1)*HZ1(-1)*HZ4(0,0,0,-1)
     &  - HZ1( -1)*HZ1(0)*HZ1(0)*HZ3(0,-1,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ2(0,-1)*HZ2(0,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,0,-1,-1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(0,0,-1)
     &  + HZ1( -1)*HZ5(0,0,-1,0,-1)
     &  + 1.5000000000000000d+00*HZ1(0)*HZ1(0)*HZ4(0,-1,-1,-1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(0,-1,-1)
     &  + HZ1(0) *HZ5(0,-1,0,-1,-1)
     &  + HZ3(0, -1,-1)*HZ3(0,0,-1)
     &  - HZ6(0,0, -1,0,-1,-1)
      HZ6(-1,-1,0,-1,0,1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,0,1)
     &  - HZ1( -1)*HZ5(0,-1,0,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,0,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,0,1,-1)
     &  + HZ6(0, -1,0,-1,-1,1)
     &  + HZ6(0, -1,0,-1,1,-1)
     &  + HZ6(0, -1,0,1,-1,-1)
      HZ6(-1,-1,0,-1,0,-1) = 
     &  + 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(0,-1)*HZ2(0,-1)
     &  - HZ1( -1)*HZ1(-1)*HZ4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ2(0,-1)*HZ3(0,-1,-1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,0,-1,-1)
     &  + 1.2000000000000000d+01*HZ1(-1)*HZ5(0,0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HZ2(0,-1)*HZ4(0,-1,-1,-1)
     &  - HZ3(0, -1,-1)*HZ3(0,-1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,-1,-1,-1)
     &  - 1.2000000000000000d+01*HZ6(0,0,-1,-1,-1,-1)
      HZ6(-1,-1,0,-1,1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,0,1)
     &  - HZ1( -1)*HZ1(-1)*HZ4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,-1,-1,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,0,-1,1)
     &  + HZ1( -1)*HZ5(0,-1,0,1,-1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,1,-1)
     &  + 3.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,1,-1)
     &  + HZ1(0) *HZ5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,-1,-1,0,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,0,-1,1,-1)
     &  - HZ6(0, -1,0,1,-1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,-1)
      HZ6(-1,-1,0,-1,1,1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,1,1)
     &  - HZ1( -1)*HZ5(0,-1,1,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,1,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,1,1,-1)
     &  + HZ6(0, -1,1,-1,-1,1)
     &  + HZ6(0, -1,1,-1,1,-1)
     &  + HZ6(0, -1,1,1,-1,-1)
      HZ6(-1,-1,0,-1,1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,-1,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,1,-1,-1,-1)
      HZ6(-1,-1,0,-1,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(0,-1,-1)
     &  - 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(0,-1)*HZ2(0,-1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ2(0,-1)*HZ3(0,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,0,-1,-1)
     &  - 6.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,-1,-1)
     &  + 6.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HZ2(0,-1)*HZ4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HZ3(0,-1,-1)*HZ3(0,-1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,0,-1,-1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,0,-1,-1,-1,-1)
      HZ6(-1,-1,0,-1,-1,1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,-1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,-1,1,-1)
     &  + 6.0000000000000000d+00*HZ6(0,-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,-1,-1,1,-1)
     &  + HZ6(0, -1,-1,1,-1,-1)
      HZ6(-1,-1,0,-1,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,-1,-1)
     &  + 1.0000000000000000d+01*HZ6(0,-1,-1,-1,-1,-1)
      HZ6(-1,-1,1,0,0,0) = 
     &  - 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(0)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(0,0,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,0,0,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(0,1,-1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,0,1,-1)
     &  + HZ1( -1)*HZ5(0,0,0,1,-1)
     &  + 1.6666666666666666d-01*HZ1(0)*HZ1(0)*HZ1(0)*HZ3(-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,1,-1,-1)
     &  + HZ1(0) *HZ5(0,0,1,-1,-1)
     &  - HZ6(0,0,0,1, -1,-1)
      HZ6(-1,-1,1,0,0,1) = 
     &  - 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(0,1)*HZ2(0,1)
     &  + HZ1( -1)*HZ2(0,1)*HZ3(0,1,-1)
     &  - HZ1( -1)*HZ5(0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,1,-1)
     &  - HZ1( -1)*HZ5(0,1,0,1,-1)
     &  - HZ2(0,1) *HZ4(0,1,-1,-1)
     &  + HZ3( -1,-1,1)*HZ3(0,0,1)
     &  + HZ6(0,0,1, -1,-1,1)
     &  + HZ6(0,0,1, -1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,1,-1,-1)
     &  + HZ6(0,1,0,1, -1,-1)
      HZ6(-1,-1,1,0,0,-1) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(0,-1)*HZ2(0,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,0,1,-1)
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(0,1,-1)
     &  - HZ1( -1)*HZ5(0,-1,0,1,-1)
     &  - HZ1( -1)*HZ5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,-1,-1)
     &  - HZ2(0, -1)*HZ4(0,1,-1,-1)
     &  + HZ3( -1,-1,1)*HZ3(0,0,-1)
     &  + HZ6(0, -1,0,1,-1,-1)
     &  + HZ6(0,0, -1,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,0,1,-1,-1,-1)
      HZ6(-1,-1,1,0,1,0) = 
     &  - HZ1( -1)*HZ1(-1)*HZ1(0)*HZ3(0,1,1)
     &  + 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(0,1)*HZ2(0,1)
     &  + HZ1( -1)*HZ1(-1)*HZ4(0,0,1,1)
     &  + HZ1( -1)*HZ1(0)*HZ4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,1,1,-1)
     &  - HZ1( -1)*HZ2(0,1)*HZ3(0,1,-1)
     &  + HZ1( -1)*HZ5(0,1,0,1,-1)
     &  + HZ1(0) *HZ2(0,1)*HZ3(-1,-1,1)
     &  - HZ1(0) *HZ5(0,1,-1,-1,1)
     &  - HZ1(0) *HZ5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(0)*HZ5(0,1,1,-1,-1)
     &  + HZ2(0,1) *HZ4(0,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,0,1)
     &  - HZ6(0,1,0,1, -1,-1)
      HZ6(-1,-1,1,0,1,1) = 
     &  - 1.5000000000000000d+00*HZ1(-1)*HZ1(-1)*HZ4(0,1,1,1)
     &  + HZ1( -1)*HZ5(0,1,-1,1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,1,-1,1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ5(0,1,1,1,-1)
     &  + HZ3( -1,-1,1)*HZ3(0,1,1)
     &  - HZ6(0,1, -1,-1,1,1)
     &  - HZ6(0,1, -1,1,-1,1)
     &  - HZ6(0,1, -1,1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,1,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,1,-1,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,1,1,1,-1,-1)
      HZ6(-1,-1,1,0,1,-1) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,1,-1,1)
     &  - HZ1( -1)*HZ1(-1)*HZ4(0,1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,-1,1,-1)
     &  + 4.0000000000000000d+00*HZ1(-1)*HZ5(0,1,1,-1,-1)
     &  + HZ3( -1,-1,1)*HZ3(0,1,-1)
     &  - HZ6(0,1, -1,-1,1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,1,-1,1,-1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,1,1,-1,-1,-1)
      HZ6(-1,-1,1,0,-1,0) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(0,1,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(0,-1)*HZ2(0,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,0,1)
     &  + HZ1( -1)*HZ1(0)*HZ4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,1,-1,-1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(0,1,-1)
     &  + HZ1( -1)*HZ5(0,-1,0,1,-1)
     &  + HZ1(0) *HZ2(0,-1)*HZ3(-1,-1,1)
     &  - HZ1(0) *HZ5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(0,1,-1,-1,-1)
     &  + HZ2(0, -1)*HZ4(0,1,-1,-1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,0,-1)
     &  - HZ6(0, -1,0,1,-1,-1)
      HZ6(-1,-1,1,0,-1,1) = 
     &  - HZ1( -1)*HZ1(-1)*HZ4(0,-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,1,-1,1)
     &  + HZ1( -1)*HZ5(0,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,-1,-1,1)
     &  + HZ1( -1)*HZ5(0,1,-1,1,-1)
     &  + HZ3( -1,-1,1)*HZ3(0,-1,1)
     &  - HZ6(0, -1,1,-1,-1,1)
     &  - HZ6(0, -1,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ6(0,1,-1,-1,1,-1)
     &  - HZ6(0,1, -1,1,-1,-1)
      HZ6(-1,-1,1,0,-1,-1) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,1,-1,-1)
     &  + HZ1( -1)*HZ5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ5(0,1,-1,-1,-1)
     &  + HZ3( -1,-1,1)*HZ3(0,-1,-1)
     &  - HZ6(0, -1,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,1,-1,-1,-1)
     &  - 6.0000000000000000d+00*HZ6(0,1,-1,-1,-1,-1)
      HZ6(-1,-1,1,1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(0,1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,0,1,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,1,1,-1)
     &  + HZ1( -1)*HZ5(0,0,1,1,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(-1,-1,1,1)
     &  - HZ1(0) *HZ2(0,1)*HZ3(-1,-1,1)
     &  + HZ1(0) *HZ5(0,1,1,-1,-1)
     &  + HZ3( -1,-1,1)*HZ3(0,0,1)
     &  - HZ6(0,0,1,1, -1,-1)
      HZ6(-1,-1,1,1,0,1) = 
     &  + 1.5000000000000000d+00*HZ1(-1)*HZ1(-1)*HZ4(0,1,1,1)
     &  - HZ1( -1)*HZ5(0,1,1,-1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,1,1,1,-1)
     &  + HZ2(0,1) *HZ4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,1,1)
     &  + HZ6(0,1,1, -1,-1,1)
     &  + HZ6(0,1,1, -1,1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,1,1,1,-1,-1)
      HZ6(-1,-1,1,1,0,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,1,-1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,1,1,-1)
     &  - HZ1( -1)*HZ5(0,-1,1,1,-1)
     &  - HZ1( -1)*HZ5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,1,-1,-1)
     &  + HZ2(0, -1)*HZ4(-1,-1,1,1)
     &  - HZ3( -1,-1,1)*HZ3(0,-1,1)
     &  - HZ3( -1,-1,1)*HZ3(0,1,-1)
     &  + HZ6(0, -1,1,1,-1,-1)
     &  + HZ6(0,1, -1,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,1,1,-1,-1,-1)
      HZ6(-1,-1,1,1,1,0) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,1,1,1)
     &  + HZ1( -1)*HZ5(0,1,1,1,-1)
     &  + HZ1(0) *HZ5(-1,-1,1,1,1)
     &  - HZ2(0,1) *HZ4(-1,-1,1,1)
     &  + HZ3( -1,-1,1)*HZ3(0,1,1)
     &  - HZ6(0,1,1,1, -1,-1)
      HZ6(-1,-1,1,1,1,-1) = 
     &  + HZ1( -1)*HZ5(-1,-1,1,1,1)
     &  - 3.0000000000000000d+00*HZ6(-1,-1,-1,1,1,1)
     &  - HZ6( -1,-1,1,-1,1,1)
     &  - HZ6( -1,-1,1,1,-1,1)
      HZ6(-1,-1,1,1,-1,0) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,1,1)
     &  + HZ1( -1)*HZ1(0)*HZ4(-1,-1,1,1)
     &  + HZ1( -1)*HZ5(0,-1,1,1,-1)
     &  - 3.0000000000000000d+00*HZ1(0)*HZ5(-1,-1,-1,1,1)
     &  - HZ1(0) *HZ5(-1,-1,1,-1,1)
     &  - HZ2(0, -1)*HZ4(-1,-1,1,1)
     &  + HZ3( -1,-1,1)*HZ3(0,-1,1)
     &  - HZ6(0, -1,1,1,-1,-1)
      HZ6(-1,-1,1,1,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(-1,-1,-1,1,1)
     &  - HZ1( -1)*HZ5(-1,-1,1,-1,1)
     &  + 5.0000000000000000d-01*HZ3(-1,-1,1)*HZ3(-1,-1,1)
      HZ6(-1,-1,1,-1,0,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(-1,-1,1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,-1,1,-1)
     &  + HZ1( -1)*HZ5(0,0,-1,1,-1)
     &  - 1.5000000000000000d+00*HZ1(0)*HZ1(0)*HZ4(-1,-1,-1,1)
     &  - HZ1(0) *HZ2(0,-1)*HZ3(-1,-1,1)
     &  + HZ1(0) *HZ5(0,-1,1,-1,-1)
     &  + HZ3( -1,-1,1)*HZ3(0,0,-1)
     &  - HZ6(0,0, -1,1,-1,-1)
      HZ6(-1,-1,1,-1,0,1) = 
     &  + HZ1( -1)*HZ1(-1)*HZ4(0,-1,1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,1,-1,1)
     &  + HZ1( -1)*HZ2(0,1)*HZ3(-1,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,1,1,-1)
     &  - HZ1( -1)*HZ5(0,1,-1,1,-1)
     &  - 3.0000000000000000d+00*HZ2(0,1)*HZ4(-1,-1,-1,1)
     &  - HZ3( -1,-1,1)*HZ3(0,-1,1)
     &  - HZ3( -1,-1,1)*HZ3(0,1,-1)
     &  + HZ6(0, -1,1,-1,-1,1)
     &  + HZ6(0, -1,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,1,1,-1,-1)
     &  + HZ6(0,1, -1,1,-1,-1)
      HZ6(-1,-1,1,-1,0,-1) = 
     &  + HZ1( -1)*HZ1(-1)*HZ4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,1,-1)
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ2(0,-1)*HZ4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ3(-1,-1,1)*HZ3(0,-1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,-1,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ6(0,-1,1,-1,-1,-1)
      HZ6(-1,-1,1,-1,1,0) = 
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,1,-1,1)
     &  - HZ1( -1)*HZ2(0,1)*HZ3(-1,-1,1)
     &  + HZ1( -1)*HZ5(0,1,-1,1,-1)
     &  + HZ1(0) *HZ5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HZ2(0,1)*HZ4(-1,-1,-1,1)
     &  + HZ3( -1,-1,1)*HZ3(0,1,-1)
     &  - HZ6(0,1, -1,1,-1,-1)
      HZ6(-1,-1,1,-1,1,-1) = 
     &  + HZ1( -1)*HZ5(-1,-1,1,-1,1)
     &  - HZ3( -1,-1,1)*HZ3(-1,-1,1)
     &  + 1.2000000000000000d+01*HZ6(-1,-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HZ6(-1,-1,-1,1,-1,1)
      HZ6(-1,-1,1,-1,-1,0) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,-1,1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(-1,-1,-1,1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(-1,-1,1)
     &  + HZ1( -1)*HZ5(0,-1,-1,1,-1)
     &  + 6.0000000000000000d+00*HZ1(0)*HZ5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HZ2(0,-1)*HZ4(-1,-1,-1,1)
     &  + HZ3( -1,-1,1)*HZ3(0,-1,-1)
     &  - HZ6(0, -1,-1,1,-1,-1)
      HZ6(-1,-1,1,-1,-1,1) = 
     &  + 5.0000000000000000d-01*HZ3(-1,-1,1)*HZ3(-1,-1,1)
     &  - 6.0000000000000000d+00*HZ6(-1,-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HZ6(-1,-1,-1,1,-1,1)
      HZ6(-1,-1,1,-1,-1,-1) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(-1,-1,1)
     &  - 1.5000000000000000d+00*HZ1(-1)*HZ1(-1)*HZ4(-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HZ1(-1)*HZ5(-1,-1,-1,-1,1)
     &  - 1.0000000000000000d+01*HZ6(-1,-1,-1,-1,-1,1)
      HZ6(-1,-1,-1,0,0,0) = 
     &  + 2.7777777777777777d-02*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(0)*HZ1(0)
     &  - 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(0)*HZ2(0,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(0,0,-1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,0,0,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(0)*HZ1(0)*HZ3(0,-1,-1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,0,-1,-1)
     &  + HZ1( -1)*HZ5(0,0,0,-1,-1)
     &  - 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(0,-1,-1,-1)
     &  + HZ1(0) *HZ5(0,0,-1,-1,-1)
     &  - HZ6(0,0,0, -1,-1,-1)
      HZ6(-1,-1,-1,0,0,1) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,0,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,0,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,0,1,-1)
     &  + HZ1( -1)*HZ5(0,-1,-1,0,1)
     &  + HZ1( -1)*HZ5(0,-1,0,-1,1)
     &  + HZ1( -1)*HZ5(0,-1,0,1,-1)
     &  + HZ1( -1)*HZ5(0,0,-1,-1,1)
     &  + HZ1( -1)*HZ5(0,0,-1,1,-1)
     &  + HZ1( -1)*HZ5(0,0,1,-1,-1)
     &  - HZ6(0, -1,-1,-1,0,1)
     &  - HZ6(0, -1,-1,0,-1,1)
     &  - HZ6(0, -1,-1,0,1,-1)
     &  - HZ6(0, -1,0,-1,-1,1)
     &  - HZ6(0, -1,0,-1,1,-1)
     &  - HZ6(0, -1,0,1,-1,-1)
     &  - HZ6(0,0, -1,-1,-1,1)
     &  - HZ6(0,0, -1,-1,1,-1)
     &  - HZ6(0,0, -1,1,-1,-1)
     &  - HZ6(0,0,1, -1,-1,-1)
      HZ6(-1,-1,-1,0,0,-1) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,0,-1)
     &  - 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(0,-1)*HZ2(0,-1)
     &  + HZ1( -1)*HZ2(0,-1)*HZ3(0,-1,-1)
     &  - HZ1( -1)*HZ5(0,-1,0,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,-1,-1)
     &  - HZ2(0, -1)*HZ4(0,-1,-1,-1)
     &  + HZ6(0, -1,0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,0,-1,-1,-1,-1)
      HZ6(-1,-1,-1,0,1,0) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ2(0,1)
     &  - 3.3333333333333333d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,0,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(0,1,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,0,1)
     &  + HZ1( -1)*HZ1(-1)*HZ4(0,0,-1,1)
     &  + HZ1( -1)*HZ1(-1)*HZ4(0,0,1,-1)
     &  + HZ1( -1)*HZ1(0)*HZ4(0,-1,-1,1)
     &  + HZ1( -1)*HZ1(0)*HZ4(0,-1,1,-1)
     &  + HZ1( -1)*HZ1(0)*HZ4(0,1,-1,-1)
     &  - HZ1( -1)*HZ5(0,-1,-1,0,1)
     &  - HZ1( -1)*HZ5(0,-1,0,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,0,1,-1,-1)
     &  - HZ1(0) *HZ5(0,-1,-1,-1,1)
     &  - HZ1(0) *HZ5(0,-1,-1,1,-1)
     &  - HZ1(0) *HZ5(0,-1,1,-1,-1)
     &  - HZ1(0) *HZ5(0,1,-1,-1,-1)
     &  + HZ6(0, -1,-1,-1,0,1)
     &  + HZ6(0, -1,-1,0,-1,1)
     &  + HZ6(0, -1,-1,0,1,-1)
     &  + HZ6(0, -1,0,-1,-1,1)
     &  + HZ6(0, -1,0,-1,1,-1)
     &  + HZ6(0, -1,0,1,-1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,0,1,-1,-1,-1)
      HZ6(-1,-1,-1,0,1,1) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,1,1,-1)
     &  + HZ1( -1)*HZ5(0,-1,-1,1,1)
     &  + HZ1( -1)*HZ5(0,-1,1,-1,1)
     &  + HZ1( -1)*HZ5(0,-1,1,1,-1)
     &  + HZ1( -1)*HZ5(0,1,-1,-1,1)
     &  + HZ1( -1)*HZ5(0,1,-1,1,-1)
     &  + HZ1( -1)*HZ5(0,1,1,-1,-1)
     &  - HZ6(0, -1,-1,-1,1,1)
     &  - HZ6(0, -1,-1,1,-1,1)
     &  - HZ6(0, -1,-1,1,1,-1)
     &  - HZ6(0, -1,1,-1,-1,1)
     &  - HZ6(0, -1,1,-1,1,-1)
     &  - HZ6(0, -1,1,1,-1,-1)
     &  - HZ6(0,1, -1,-1,-1,1)
     &  - HZ6(0,1, -1,-1,1,-1)
     &  - HZ6(0,1, -1,1,-1,-1)
     &  - HZ6(0,1,1, -1,-1,-1)
      HZ6(-1,-1,-1,0,1,-1) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,1,-1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,1,-1)
     &  - HZ1( -1)*HZ1(-1)*HZ4(0,1,-1,-1)
     &  + HZ1( -1)*HZ5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ5(0,1,-1,-1,-1)
     &  - HZ6(0, -1,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,1,-1,-1,-1,-1)
      HZ6(-1,-1,-1,0,-1,0) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ2(0,-1)
     &  - 3.3333333333333333d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,0,-1)
     &  - HZ1( -1)*HZ1(-1)*HZ1(0)*HZ3(0,-1,-1)
     &  + 2.5000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ2(0,-1)*HZ2(0,-1)
     &  + HZ1( -1)*HZ1(-1)*HZ4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ1(0)*HZ4(0,-1,-1,-1)
     &  - HZ1( -1)*HZ2(0,-1)*HZ3(0,-1,-1)
     &  + HZ1( -1)*HZ5(0,-1,0,-1,-1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(0,-1,-1,-1,-1)
     &  + HZ2(0, -1)*HZ4(0,-1,-1,-1)
     &  - HZ6(0, -1,0,-1,-1,-1)
      HZ6(-1,-1,-1,0,-1,1) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,-1,1)
     &  - HZ1( -1)*HZ1(-1)*HZ4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,1,-1)
     &  + 3.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,1,-1)
     &  + HZ1( -1)*HZ5(0,-1,1,-1,-1)
     &  - 4.0000000000000000d+00*HZ6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HZ6(0,-1,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ6(0,-1,-1,1,-1,-1)
     &  - HZ6(0, -1,1,-1,-1,-1)
      HZ6(-1,-1,-1,0,-1,-1) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,-1,-1)
     &  - 1.5000000000000000d+00*HZ1(-1)*HZ1(-1)*HZ4(0,-1,-1,-1)
     &  + 6.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,-1,-1)
     &  - 1.0000000000000000d+01*HZ6(0,-1,-1,-1,-1,-1)
      HZ6(-1,-1,-1,1,0,0) = 
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ2(0,1)
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,0,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(0,1,-1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,0,1,-1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,1,-1,-1)
     &  + HZ1( -1)*HZ5(0,0,1,-1,-1)
     &  + 5.0000000000000000d-01*HZ1(0)*HZ1(0)*HZ4(-1,-1,-1,1)
     &  + HZ1(0) *HZ5(0,1,-1,-1,-1)
     &  - HZ6(0,0,1, -1,-1,-1)
      HZ6(-1,-1,-1,1,0,1) = 
     &  - 3.3333333333333333d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,1,-1,1)
     &  + HZ1( -1)*HZ1(-1)*HZ4(0,1,1,-1)
     &  - HZ1( -1)*HZ5(0,1,-1,-1,1)
     &  - HZ1( -1)*HZ5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HZ1(-1)*HZ5(0,1,1,-1,-1)
     &  + HZ2(0,1) *HZ4(-1,-1,-1,1)
     &  + HZ6(0,1, -1,-1,-1,1)
     &  + HZ6(0,1, -1,-1,1,-1)
     &  + HZ6(0,1, -1,1,-1,-1)
     &  + 2.0000000000000000d+00*HZ6(0,1,1,-1,-1,-1)
      HZ6(-1,-1,-1,1,0,-1) = 
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,-1,1)
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,1,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,1,-1)
     &  + HZ1( -1)*HZ1(-1)*HZ4(0,1,-1,-1)
     &  - HZ1( -1)*HZ5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HZ1(-1)*HZ5(0,1,-1,-1,-1)
     &  + HZ2(0, -1)*HZ4(-1,-1,-1,1)
     &  + HZ6(0, -1,1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HZ6(0,1,-1,-1,-1,-1)
      HZ6(-1,-1,-1,1,1,0) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,1,1,-1)
     &  + HZ1( -1)*HZ5(0,1,1,-1,-1)
     &  + HZ1(0) *HZ5(-1,-1,-1,1,1)
     &  - HZ2(0,1) *HZ4(-1,-1,-1,1)
     &  - HZ6(0,1,1, -1,-1,-1)
      HZ6(-1,-1,-1,1,1,-1) = 
     &  + HZ1( -1)*HZ5(-1,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HZ6(-1,-1,-1,-1,1,1)
     &  - HZ6( -1,-1,-1,1,-1,1)
      HZ6(-1,-1,-1,1,-1,0) = 
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,-1,1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,1,-1)
     &  + HZ1( -1)*HZ1(0)*HZ4(-1,-1,-1,1)
     &  + HZ1( -1)*HZ5(0,-1,1,-1,-1)
     &  - 4.0000000000000000d+00*HZ1(0)*HZ5(-1,-1,-1,-1,1)
     &  - HZ2(0, -1)*HZ4(-1,-1,-1,1)
     &  - HZ6(0, -1,1,-1,-1,-1)
      HZ6(-1,-1,-1,1,-1,-1) = 
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(-1,-1,-1,-1,1)
     &  + 1.0000000000000000d+01*HZ6(-1,-1,-1,-1,-1,1)
      HZ6(-1,-1,-1,-1,0,0) = 
     &  + 2.0833333333333333d-02*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ1(0)
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ2(0,-1)
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,0,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ1(0)*HZ3(0,-1,-1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,0,-1,-1)
     &  - HZ1( -1)*HZ1(0)*HZ4(0,-1,-1,-1)
     &  + HZ1( -1)*HZ5(0,0,-1,-1,-1)
     &  + HZ1(0) *HZ5(0,-1,-1,-1,-1)
     &  - HZ6(0,0, -1,-1,-1,-1)
      HZ6(-1,-1,-1,-1,0,1) = 
     &  + 4.1666666666666666d-02*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ2(0,1)
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,-1,1)
     &  - 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,1,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,1,-1)
     &  + 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,1,-1,-1)
     &  - HZ1( -1)*HZ5(0,-1,-1,-1,1)
     &  - HZ1( -1)*HZ5(0,-1,-1,1,-1)
     &  - HZ1( -1)*HZ5(0,-1,1,-1,-1)
     &  - HZ1( -1)*HZ5(0,1,-1,-1,-1)
     &  + HZ6(0, -1,-1,-1,-1,1)
     &  + HZ6(0, -1,-1,-1,1,-1)
     &  + HZ6(0, -1,-1,1,-1,-1)
     &  + HZ6(0, -1,1,-1,-1,-1)
     &  + HZ6(0,1, -1,-1,-1,-1)
      HZ6(-1,-1,-1,-1,0,-1) = 
     &  + 4.1666666666666666d-02*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ2(0,-1)
     &  - 3.3333333333333333d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,-1,-1)
     &  + 1.5000000000000000d+00*HZ1(-1)*HZ1(-1)*HZ4(0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HZ1(-1)*HZ5(0,-1,-1,-1,-1)
     &  + 5.0000000000000000d+00*HZ6(0,-1,-1,-1,-1,-1)
      HZ6(-1,-1,-1,-1,1,0) = 
     &  - 4.1666666666666666d-02*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ2(0,1)
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,1,-1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,1,-1,-1)
     &  + HZ1( -1)*HZ5(0,1,-1,-1,-1)
     &  + HZ1(0) *HZ5(-1,-1,-1,-1,1)
     &  - HZ6(0,1, -1,-1,-1,-1)
      HZ6(-1,-1,-1,-1,1,-1) = 
     &  + HZ1( -1)*HZ5(-1,-1,-1,-1,1)
     &  - 5.0000000000000000d+00*HZ6(-1,-1,-1,-1,-1,1)
      HZ6(-1,-1,-1,-1,-1,0) = 
     &  + 8.3333333333333333d-03*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(0)
     &  - 4.1666666666666666d-02*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ2(0,-1)
     &  + 1.6666666666666666d-01*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ3(0,-1,-1)
     &  - 5.0000000000000000d-01*HZ1(-1)*HZ1(-1)*HZ4(0,-1,-1,-1)
     &  + HZ1( -1)*HZ5(0,-1,-1,-1,-1)
     &  - HZ6(0, -1,-1,-1,-1,-1)
      HZ6(-1,-1,-1,-1,-1,-1) = 
     &  + 1.3888888888888888d-03*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)*HZ1(-1)

      return
      end


************************************************************************
** the following routines contain th set of routines evaluating 
** irreducible 1dhpl's for various values of the arguments 
************************************************************************ 
      subroutine fillh1(y,H1,HY1,Hi1,n1,n2) 
** fillh1 evaluates the 1dhpl's of weight 1 
      implicit double precision (a-h,o-z) 
      complex*16 H1 
      dimension H1(n1:n2) 
      dimension HY1(n1:n2) 
      dimension Hi1(n1:n2) 
      parameter (pi   = 3.14159265358979324d0) 
      if ( n1.eq.-1) then 
        if ( y.ge.-1.d0 ) then 
          HY1(-1) = log(1.d0+y) 
          Hi1(-1) = 0.d0 
        elseif ( y.lt.-1.d0 ) then 
          HY1(-1) = log(-1.d0-y) 
          Hi1(-1) = 1.d0 
        endif 
        H1(-1) = dcmplx(HY1(-1),pi*Hi1(-1)) 
      endif 
      if ( y.ge.0.d0 ) then 
        HY1(0) = log(y) 
*        Hi1(0) = 0.d0 
      elseif ( y.lt.0.d0 ) then 
        HY1(0) = log(-y) 
        Hi1(0) = 1.d0 
      endif 
      H1(0) = dcmplx(HY1(0),pi*Hi1(0)) 
      if ( n2.eq.1 ) then 
        if ( y.ge.1.d0 ) then 
          HY1(1) = - log(-1.d0+y) 
          Hi1(1) = 1.d0 
        elseif ( y.lt.1.d0 ) then 
          HY1(1) = - log(1.d0-y) 
          Hi1(1) = 0.d0 
        endif 
        H1(1) = dcmplx(HY1(1),pi*Hi1(1)) 
      endif 
      return 
      end 
************************************************************************ 
      subroutine fillirr1dhplat0(y,nw,HY1,HY2,HY3,HY4,HY5,HY6,n1,n2) 
** evaluate the HPL from their power series expansions
** fillirr1dhplat0 is called by eval1dhplat0; 
** it is guaranteed that nw is in the range 1:4, and that (n1,n2) 
** take one of the pairs of values (0,1), (-1,0) or (-1,1) 
** 
** for y < 0 DOES NOT evaluates the immaginary part of H(0,y) = log(y) 
      implicit double precision (a-h,o-z) 
      dimension HY1(n1:n2),HY2(n1:n2,n1:n2),HY3(n1:n2,n1:n2,n1:n2), 
     &          HY4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HY5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2),
     &          HY6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
** evaluating the required 1dHPL of weight 1 
      if ( n1.eq.-1) then 
** 1+y = (1+ep)/(1-ep), ep = y/(2+y) 
** log(1+y) = log((1+y)/(1-y)) = 2*ep*(1+ep^2/3+ep^4/5+.....) 
** at y= -(r2-1) = - 0.4142135624, ep = - 0.26120387496 
** ep2 = 0.068227464296, ep2^13 = 6.9 x 10^(-16) 
         ep = y/(2.d0+y) 
         e2 = ep*ep 
*         v = log(1.d0+y) 
         v = 2*ep*(1+e2*(1.d0/ 3+e2*(1.d0/ 5+e2*(1.d0/ 7+e2*(1.d0/ 9 
     &              +e2*(1.d0/11+e2*(1.d0/13+e2*(1.d0/15+e2*(1.d0/17    
     &              +e2*(1.d0/19+e2*(1.d0/21+e2*(1.d0/23+e2*(1.d0/25    
     &              )))))))))))))
         HY1(-1) = v 
      endif 
      if (y.ge.0d0) then 
         HY1(0) = log(y) 
      else 
         HY1(0) = log(-y) 
** the immaginary part is evaluated in the calling routine eval1dhplat0 
**       Hi1(0) = 1d0 
      endif 
      if ( n2.eq.1) then 
** 1-y = (1-ep)/(1+ep), ep = y/(2-y) 
         ep = y/(2.d0-y) 
         e2 = ep*ep 
*         u = - log(1.d0-y) 
         u = 2*ep*(1+e2*(1.d0/ 3+e2*(1.d0/ 5+e2*(1.d0/ 7+e2*(1.d0/ 9 
     &              +e2*(1.d0/11+e2*(1.d0/13+e2*(1.d0/15+e2*(1.d0/17    
     &              +e2*(1.d0/19+e2*(1.d0/21+e2*(1.d0/23+e2*(1.d0/25    
     &              )))))))))))))
         HY1(1) = u 
      endif 
      if ( nw.eq.1 ) return 
** from now on nw > 1 
** evaluating the Cebyshev polynomials for the expansions 
      ep = y 
      if ( n2.eq.1) then 
        tu01 = 20d0/11d0*u 
        tu02 = 2d0*tu01*tu01 - 1d0 
        tu03 = 2d0*tu01*tu02 - tu01 
        tu04 = 2d0*tu01*tu03 - tu02 
        tu05 = 2d0*tu01*tu04 - tu03 
        tu06 = 2d0*tu01*tu05 - tu04 
        tu07 = 2d0*tu01*tu06 - tu05 
        tu08 = 2d0*tu01*tu07 - tu06 
        tu09 = 2d0*tu01*tu08 - tu07 
        tu10 = 2d0*tu01*tu09 - tu08 
        tu11 = 2d0*tu01*tu10 - tu09 
        tu12 = 2d0*tu01*tu11 - tu10 
        tu13 = 2d0*tu01*tu12 - tu11 
      endif 
      if ( n1.eq.-1 ) then 
        tv01 = 20d0/11d0*v 
        tv02 = 2d0*tv01*tv01 - 1d0 
        tv03 = 2d0*tv01*tv02 - tv01 
        tv04 = 2d0*tv01*tv03 - tv02 
        tv05 = 2d0*tv01*tv04 - tv03 
        tv06 = 2d0*tv01*tv05 - tv04 
        tv07 = 2d0*tv01*tv06 - tv05 
        tv08 = 2d0*tv01*tv07 - tv06 
        tv09 = 2d0*tv01*tv08 - tv07 
        tv10 = 2d0*tv01*tv09 - tv08 
        tv11 = 2d0*tv01*tv10 - tv09 
        tv12 = 2d0*tv01*tv11 - tv10 
        tv13 = 2d0*tv01*tv12 - tv11 
      endif 
** evaluating the expansions 
** (n1,n2) = (0,1) or (-1,1) 
      if (    ( (n1.eq.0).and.(n2.eq.1) ) 
     &    .or.( (n1.eq.-1).and.(n2.eq.1) ) ) then 
      HY2(0,1) = 
     &  - 3.781250000000000d-02 
     &  + 5.534574473824441d-01*tu01 
     &  - 3.781250000000000d-02*tu02 
     &  + 1.151036617760703d-03*tu03 
     &  - 8.659502433858922d-07*tu05 
     &  + 1.109042494804544d-09*tu07 
     &  - 1.624415058184216d-12*tu09 
     &  + 2.528376460336939d-15*tu11 
**    it would be wrong to write 
**    if ( nw.eq.2 ) return 
**    because the (n1.eq.-1).and.(n2.eq.1) case is not yet complete 
      if ( nw.gt.2 ) then 
      HY3(0,0,1) = 
     &  - 5.701592410758114d-02 
     &  + 5.598247957892565d-01*tu01 
     &  - 5.711486614505007d-02*tu02 
     &  + 3.275603992203700d-03*tu03 
     &  - 9.887255877938583d-05*tu04 
     &  + 4.021153684652295d-07*tu05 
     &  + 6.939288687864526d-08*tu06 
     &  - 7.995347631322020d-10*tu07 
     &  - 8.567978673919505d-11*tu08 
     &  + 1.526387027481200d-12*tu09 
     &  + 1.226899454816980d-13*tu10 
     &  - 2.848614761014972d-15*tu11 
     &  - 1.880542777479446d-16*tu12 
      HY3(0,1,1) = 
     &  + 3.816894981500984d-02 
     &  - 1.039843750000000d-02*tu01 
     &  + 3.828760080995617d-02*tu02 
     &  - 3.466145833333333d-03*tu03 
     &  + 1.185518160084905d-04*tu04 
     &  - 9.904555648775859d-08*tu06 
     &  + 1.331803984518588d-10*tu08 
     &  - 2.006389465106708d-13*tu10 
     &  + 3.180731062055677d-16*tu12 
      endif 
      if ( nw.gt.3 ) then 
      HY4(0,0,0,1) = 
     &  - 6.685228257646101d-02 
     &  + 5.645990701998083d-01*tu01 
     &  - 6.707912936340146d-02*tu02 
     &  + 4.876429488624746d-03*tu03 
     &  - 2.268732672568699d-04*tu04 
     &  + 6.038494106229146d-06*tu05 
     &  - 2.642577015932576d-08*tu06 
     &  - 3.679843316593900d-09*tu07 
     &  + 5.444046563879984d-11*tu08 
     &  + 4.063821221202881d-12*tu09 
     &  - 1.055985864474070d-13*tu10 
     &  - 5.190408125225683d-15*tu11 
     &  + 1.985464489219049d-16*tu12 
      HY4(0,0,1,1) = 
     &  + 1.953236111099851d-02 
     &  - 8.741612828671381d-03*tu01 
     &  + 1.974116110893196d-02*tu02 
     &  - 2.926558492394004d-03*tu03 
     &  + 2.088576190269387d-04*tu04 
     &  - 7.604351107741397d-06*tu05 
     &  + 5.751031394942524d-08*tu06 
     &  + 5.832253077603139d-09*tu07 
     &  - 1.105713721511985d-10*tu08 
     &  - 7.453416210082473d-12*tu09 
     &  + 2.077758906032370d-13*tu10 
     &  + 1.085601519719514d-14*tu11 
     &  - 3.848312092795918d-16*tu12 
      HY4(0,1,1,1) = 
     &  - 7.148925781250000d-04 
     &  + 7.019393481825299d-03*tu01 
     &  - 9.531901041666666d-04*tu02 
     &  + 2.354287493676137d-03*tu03 
     &  - 2.382975260416666d-04*tu04 
     &  + 8.682904829408987d-06*tu05 
     &  - 7.768198634676578d-09*tu07 
     &  + 1.083130072188330d-11*tu09 
     &  - 1.668810490326842d-14*tu11 
      endif 
** nw > 3 endif 
      if (nw.gt.4) then
      HY5(0,0,0,0,1) = 
     &  - 7.1883935399674661d-02
     &  + 5.6753782109711975d-01*tu01
     &  - 7.2212923207857999d-02*tu02
     &  + 5.8670869471466117d-03*tu03
     &  - 3.2928069579812269d-04*tu04
     &  + 1.2689437803900269d-05*tu05
     &  - 2.9273255639591246d-07*tu06
     &  + 1.0606320330117018d-09*tu07
     &  + 1.5490385851360920d-10*tu08
     &  - 2.2474559413025843d-12*tu09
     &  - 1.5435486840448295d-13*tu10
     &  + 4.3623305832897835d-15*tu11
     &  + 1.7529593826254750d-16*tu12
      HY5(0,0,0,1,1) = 
     &  + 1.0000521958818753d-02
     &  - 5.5972378484058781d-03*tu01
     &  + 1.0183616888857122d-02*tu02
     &  - 1.8839550865568384d-03*tu03
     &  + 1.8347215924120138d-04*tu04
     &  - 1.0930879320715947d-05*tu05
     &  + 3.7698877255509719d-07*tu06
     &  - 3.8459944472218428d-09*tu07
     &  - 2.4017837972601217d-10*tu08
     &  + 7.0263208823587761d-12*tu09
     &  + 2.5160915023993803d-13*tu10
     &  - 1.2783871754086183d-14*tu11
     &  - 2.8797679566416130d-16*tu12
     &  + 2.3134706555649888d-17*tu13
      HY5(0,0,1,0,1) = 
     &  + 1.9758848802070053d-02
     &  - 9.9630794104335436d-03*tu01
     &  + 2.0043658059117399d-02*tu02
     &  - 3.3447469330099638d-03*tu03
     &  + 2.8517465661196517d-04*tu04
     &  - 1.4231602601165430d-05*tu05
     &  + 3.6511844421403422d-07*tu06
     &  + 4.8298298252332493d-10*tu07
     &  - 2.8074265749907893d-10*tu08
     &  + 5.8456103354109943d-13*tu09
     &  + 3.7717268411948233d-13*tu10
     &  - 2.6564037404227836d-15*tu11
     &  - 5.7368628737502723d-16*tu12
      HY5(0,0,1,1,1) = 
     &  - 4.2151303147672207d-04
     &  + 2.4230392899354007d-03*tu01
     &  - 5.6276481104680303d-04*tu02
     &  + 8.2640657870678230d-04*tu03
     &  - 1.4169972575483287d-04*tu04
     &  + 1.1241768997510332d-05*tu05
     &  - 4.4757876435236780d-07*tu06
     &  + 4.0464701571442372d-09*tu07
     &  + 3.6693605986548572d-10*tu08
     &  - 8.1486736073711877d-12*tu09
     &  - 4.8362219356601919d-13*tu10
     &  + 1.5732397614617162d-14*tu11
     &  + 7.1637443194486341d-16*tu12
     &  - 2.9682648694764162d-17*tu13
      HY5(0,1,0,1,1) = 
     &  - 5.4016355841362305d-04
     &  + 3.5885207401063810d-03*tu01
     &  - 7.2088270891443272d-04*tu02
     &  + 1.2165742608270584d-03*tu03
     &  - 1.8111750748505087d-04*tu04
     &  + 1.2242347596998815d-05*tu05
     &  - 3.9805632127123734d-07*tu06
     &  + 1.3818410996670467d-09*tu07
     &  + 3.0027181656102247d-10*tu08
     &  - 2.5190497445473117d-12*tu09
     &  - 3.9056998701282666d-13*tu10
     &  + 4.6313279849969705d-15*tu11
     &  + 5.8249180278623573d-16*tu12
      HY5(0,1,1,1,1) = 
     &  + 3.6243571484950436d-04
     &  - 1.3106363932291666d-04*tu01
     &  + 4.8407754872309633d-04*tu02
     &  - 6.5531819661458333d-05*tu03
     &  + 1.2213913602645317d-04*tu04
     &  - 1.3106363932291666d-05*tu05
     &  + 4.9683501491940088d-07*tu06
     &  - 4.6646796570171982d-10*tu08
     &  + 6.6892469436720110d-13*tu10
     &  - 1.0496975350928139d-15*tu12
      endif 
** nw > 4 endif 
      endif 
** (n1,n2) = (0,1) or (-1,1) endif 
************ 
** (n1,n2) = (-1,0) or (-1,1) 
      if (    ( (n1.eq.-1).and.(n2.eq.0) ) 
     &    .or.( (n1.eq.-1).and.(n2.eq.1) ) ) then 
      HY2(0,-1) = 
     &  + 3.781250000000000d-02 
     &  + 5.534574473824441d-01*tv01 
     &  + 3.781250000000000d-02*tv02 
     &  + 1.151036617760703d-03*tv03 
     &  - 8.659502433858922d-07*tv05 
     &  + 1.109042494804544d-09*tv07 
     &  - 1.624415058184216d-12*tv09 
     &  + 2.528376460336939d-15*tv11 
      if ( nw.gt.2 ) then 
      HY3(0,0,-1) = 
     &  + 5.701592410758114d-02 
     &  + 5.598247957892565d-01*tv01 
     &  + 5.711486614505007d-02*tv02 
     &  + 3.275603992203700d-03*tv03 
     &  + 9.887255877938583d-05*tv04 
     &  + 4.021153684652295d-07*tv05 
     &  - 6.939288687864526d-08*tv06 
     &  - 7.995347631322020d-10*tv07 
     &  + 8.567978673919505d-11*tv08 
     &  + 1.526387027481200d-12*tv09 
     &  - 1.226899454816980d-13*tv10 
     &  - 2.848614761014972d-15*tv11 
     &  + 1.880542777479446d-16*tv12 
      HY3(0,-1,-1) = 
     &  + 3.816894981500984d-02 
     &  + 1.039843750000000d-02*tv01 
     &  + 3.828760080995617d-02*tv02 
     &  + 3.466145833333333d-03*tv03 
     &  + 1.185518160084905d-04*tv04 
     &  - 9.904555648775859d-08*tv06 
     &  + 1.331803984518588d-10*tv08 
     &  - 2.006389465106708d-13*tv10 
     &  + 3.180731062055677d-16*tv12 
      endif 
      if ( nw.gt.3 ) then 
      HY4(0,0,0,-1) = 
     &  + 6.685228257646101d-02 
     &  + 5.645990701998083d-01*tv01 
     &  + 6.707912936340146d-02*tv02 
     &  + 4.876429488624746d-03*tv03 
     &  + 2.268732672568699d-04*tv04 
     &  + 6.038494106229146d-06*tv05 
     &  + 2.642577015932576d-08*tv06 
     &  - 3.679843316593900d-09*tv07 
     &  - 5.444046563879984d-11*tv08 
     &  + 4.063821221202881d-12*tv09 
     &  + 1.055985864474070d-13*tv10 
     &  - 5.190408125225683d-15*tv11 
     &  - 1.985464489219049d-16*tv12 
      HY4(0,0,-1,-1) = 
     &  + 1.953236111099851d-02 
     &  + 8.741612828671381d-03*tv01 
     &  + 1.974116110893196d-02*tv02 
     &  + 2.926558492394004d-03*tv03 
     &  + 2.088576190269387d-04*tv04 
     &  + 7.604351107741397d-06*tv05 
     &  + 5.751031394942524d-08*tv06 
     &  - 5.832253077603139d-09*tv07 
     &  - 1.105713721511985d-10*tv08 
     &  + 7.453416210082473d-12*tv09 
     &  + 2.077758906032370d-13*tv10 
     &  - 1.085601519719514d-14*tv11 
     &  - 3.848312092795918d-16*tv12 
      HY4(0,-1,-1,-1) = 
     &  + 7.148925781250000d-04 
     &  + 7.019393481825299d-03*tv01 
     &  + 9.531901041666666d-04*tv02 
     &  + 2.354287493676137d-03*tv03 
     &  + 2.382975260416666d-04*tv04 
     &  + 8.682904829408987d-06*tv05 
     &  - 7.768198634676578d-09*tv07 
     &  + 1.083130072188330d-11*tv09 
     &  - 1.668810490326842d-14*tv11 
      endif 
** nw > 3 endif 
      if ( nw.gt.4 ) then 
      HY5(0,0,0,0,-1) = 
     &  + 7.1883935399674661d-02
     &  + 5.6753782109711975d-01*tv01
     &  + 7.2212923207857999d-02*tv02
     &  + 5.8670869471466117d-03*tv03
     &  + 3.2928069579812269d-04*tv04
     &  + 1.2689437803900269d-05*tv05
     &  + 2.9273255639591246d-07*tv06
     &  + 1.0606320330117018d-09*tv07
     &  - 1.5490385851360920d-10*tv08
     &  - 2.2474559413025843d-12*tv09
     &  + 1.5435486840448295d-13*tv10
     &  + 4.3623305832897835d-15*tv11
     &  - 1.7529593826254750d-16*tv12
      HY5(0,0,0,-1,-1) = 
     &  + 1.0000521958818753d-02
     &  + 5.5972378484058781d-03*tv01
     &  + 1.0183616888857122d-02*tv02
     &  + 1.8839550865568384d-03*tv03
     &  + 1.8347215924120138d-04*tv04
     &  + 1.0930879320715947d-05*tv05
     &  + 3.7698877255509719d-07*tv06
     &  + 3.8459944472218428d-09*tv07
     &  - 2.4017837972601217d-10*tv08
     &  - 7.0263208823587761d-12*tv09
     &  + 2.5160915023993803d-13*tv10
     &  + 1.2783871754086183d-14*tv11
     &  - 2.8797679566416130d-16*tv12
     &  - 2.3134706555649888d-17*tv13
      HY5(0,0,-1,0,-1) = 
     &  + 1.9758848802070053d-02
     &  + 9.9630794104335436d-03*tv01
     &  + 2.0043658059117399d-02*tv02
     &  + 3.3447469330099638d-03*tv03
     &  + 2.8517465661196517d-04*tv04
     &  + 1.4231602601165430d-05*tv05
     &  + 3.6511844421403422d-07*tv06
     &  - 4.8298298252332493d-10*tv07
     &  - 2.8074265749907893d-10*tv08
     &  - 5.8456103354109943d-13*tv09
     &  + 3.7717268411948233d-13*tv10
     &  + 2.6564037404227836d-15*tv11
     &  - 5.7368628737502723d-16*tv12
      HY5(0,0,-1,-1,-1) = 
     &  + 4.2151303147672207d-04
     &  + 2.4230392899354007d-03*tv01
     &  + 5.6276481104680303d-04*tv02
     &  + 8.2640657870678230d-04*tv03
     &  + 1.4169972575483287d-04*tv04
     &  + 1.1241768997510332d-05*tv05
     &  + 4.4757876435236780d-07*tv06
     &  + 4.0464701571442372d-09*tv07
     &  - 3.6693605986548572d-10*tv08
     &  - 8.1486736073711877d-12*tv09
     &  + 4.8362219356601919d-13*tv10
     &  + 1.5732397614617162d-14*tv11
     &  - 7.1637443194486341d-16*tv12
     &  - 2.9682648694764162d-17*tv13
      HY5(0,-1,0,-1,-1) = 
     &  + 5.4016355841362305d-04
     &  + 3.5885207401063810d-03*tv01
     &  + 7.2088270891443272d-04*tv02
     &  + 1.2165742608270584d-03*tv03
     &  + 1.8111750748505087d-04*tv04
     &  + 1.2242347596998815d-05*tv05
     &  + 3.9805632127123734d-07*tv06
     &  + 1.3818410996670467d-09*tv07
     &  - 3.0027181656102247d-10*tv08
     &  - 2.5190497445473117d-12*tv09
     &  + 3.9056998701282666d-13*tv10
     &  + 4.6313279849969705d-15*tv11
     &  - 5.8249180278623573d-16*tv12
      HY5(0,-1,-1,-1,-1) = 
     &  + 3.6243571484950436d-04
     &  + 1.3106363932291666d-04*tv01
     &  + 4.8407754872309633d-04*tv02
     &  + 6.5531819661458333d-05*tv03
     &  + 1.2213913602645317d-04*tv04
     &  + 1.3106363932291666d-05*tv05
     &  + 4.9683501491940088d-07*tv06
     &  - 4.6646796570171982d-10*tv08
     &  + 6.6892469436720110d-13*tv10
     &  - 1.0496975350928139d-15*tv12
      endif 
** nw > 4 endif 
      endif 
** (n1,n2) = (-1,0) or (-1,1) endif 
** (n1,n2) = (-1,1) -- completion 
      if ( (n1.eq.-1).and.(n2.eq.1) ) then 
      HY2(-1,1) = 
     &  - 2.924454241163343d-02 
     &  + 3.845279287117326d-01*tu01 
     &  - 2.925485694830038d-02*tu02 
     &  + 1.097780471057338d-03*tu03 
     &  - 1.029703135442673d-05*tu04 
     &  - 7.265175511511970d-07*tu05 
     &  + 1.747461299829753d-08*tu06 
     &  + 7.707353556013722d-10*tu07 
     &  - 3.064611747990741d-11*tu08 
     &  - 8.531228176305706d-13*tu09 
     &  + 5.331187822989144d-14*tu10 
     &  + 8.500141365188675d-16*tu11 
     &  - 6.931471805599453d-01*HY1(-1) 
      if ( nw.gt.2 ) then 
      HY3(0,-1,1) = 
     &  - 4.107537580582269d-02 
     &  + 3.887609555197323d-01*tu01 
     &  - 4.116162793629221d-02*tu02 
     &  + 2.511526558054413d-03*tu03 
     &  - 8.620496933228561d-05*tu04 
     &  + 9.128023201466990d-07*tu05 
     &  + 4.711634663963971d-08*tu06 
     &  - 1.347359673414334d-09*tu07 
     &  - 4.474345520888852d-11*tu08 
     &  + 2.138249646727980d-12*tu09 
     &  + 4.709915818801180d-14*tu10 
     &  - 3.454431385666621d-15*tu11 
     &  - 6.931471805599453d-01*HY2(0,-1) 
      HY3(0,1,-1) = 
     &  - 4.107537580582269d-02 
     &  - 3.887609555197323d-01*tv01 
     &  - 4.116162793629221d-02*tv02 
     &  - 2.511526558054413d-03*tv03 
     &  - 8.620496933228561d-05*tv04 
     &  - 9.128023201466990d-07*tv05 
     &  + 4.711634663963971d-08*tv06 
     &  + 1.347359673414334d-09*tv07 
     &  - 4.474345520888852d-11*tv08 
     &  - 2.138249646727980d-12*tv09 
     &  + 4.709915818801180d-14*tv10 
     &  + 3.454431385666621d-15*tv11 
     &  + 6.931471805599453d-01*HY2(0,1) 
      HY3(-1,-1,1) = 
     &  - 3.590863871372201d-02 
     &  + 3.272029419300922d-01*tu01 
     &  - 3.599657175069328d-02*tu02 
     &  + 2.325685169395631d-03*tu03 
     &  - 8.788997314012583d-05*tu04 
     &  + 1.277831858501559d-06*tu05 
     &  + 4.303730428865162d-08*tu06 
     &  - 1.992295216809703d-09*tu07 
     &  - 2.652932076676834d-11*tu08 
     &  + 3.159865930142703d-12*tu09 
     &  - 2.395589527593406d-15*tu10 
     &  - 4.870947810519399d-15*tu11 
     &  - 5.822405264650125d-01*HY1(-1) 
     &  - 3.465735902799726d-01*HY1(-1)*HY1(-1) 
      HY3(-1,1,1) = 
     &  + 3.668493142404161d-02 
     &  - 1.413123104773291d-01*tu01 
     &  + 3.680167312678666d-02*tu02 
     &  - 3.064044728536094d-03*tu03 
     &  + 1.166524199994130d-04*tu04 
     &  - 8.779983417383380d-07*tu05 
     &  - 8.917940330502000d-08*tu06 
     &  + 1.787575622706040d-09*tu07 
     &  + 1.032182649980912d-10*tu08 
     &  - 3.441821872732193d-12*tu09 
     &  - 1.239218730863368d-13*tu10 
     &  + 6.355731482672869d-15*tu11 
     &  + 1.386175839607904d-16*tu12 
     &  + 2.402265069591007d-01*HY1(-1)       
      endif 
      if ( nw.gt.3 ) then 
      HY4(0,0,-1,1) = 
     &  - 4.713463351559199d-02 
     &  + 3.918037828258655d-01*tu01 
     &  - 4.730698763577787d-02*tu02 
     &  + 3.532784273601097d-03*tu03 
     &  - 1.724036773635937d-04*tu04 
     &  + 5.100573466380115d-06*tu05 
     &  - 4.948996960052575d-08*tu06 
     &  - 2.345390965359666d-09*tu07 
     &  + 6.710522628543514d-11*tu08 
     &  + 1.979867116023822d-12*tu09 
     &  - 1.027163441987459d-13*tu10 
     &  - 1.836436639605094d-15*tu11 
     &  + 1.633620651699784d-16*tu12 
     &  - 6.931471805599453d-01*HY3(0,0,-1) 
      HY4(0,0,1,-1) = 
     &  - 4.713463351559199d-02 
     &  - 3.918037828258655d-01*tv01 
     &  - 4.730698763577787d-02*tv02 
     &  - 3.532784273601097d-03*tv03 
     &  - 1.724036773635937d-04*tv04 
     &  - 5.100573466380115d-06*tv05 
     &  - 4.948996960052575d-08*tv06 
     &  + 2.345390965359666d-09*tv07 
     &  + 6.710522628543514d-11*tv08 
     &  - 1.979867116023822d-12*tv09 
     &  - 1.027163441987459d-13*tv10  
     &  + 1.836436639605094d-15*tv11 
     &  + 1.633620651699784d-16*tv12 
     &  + 6.931471805599453d-01*HY3(0,0,1) 
      HY4(0,-1,0,1) = 
     &  - 5.610575179941452d-02 
     &  + 4.649892609082033d-01*tu01 
     &  - 5.631239161843284d-02*tu02 
     &  + 4.220972769653239d-03*tu03 
     &  - 2.066940413626322d-04*tu04 
     &  + 6.100628682175971d-06*tu05 
     &  - 5.412969106099992d-08*tu06 
     &  - 3.230915912784154d-09*tu07 
     &  + 9.249866333323043d-11*tu08 
     &  + 2.685990764581699d-12*tu09 
     &  - 1.543312114608473d-13*tu10 
     &  - 2.036971731594398d-15*tu11 
     &  + 2.517450307574790d-16*tu12 
     &  - 8.224670334241132d-01*HY2(0,-1) 
      HY4(0,-1,-1,1) = 
     &  - 4.031271939759038d-02 
     &  + 3.295217254379970d-01*tu01 
     &  - 4.047097737450547d-02*tu02 
     &  + 3.104955391145708d-03*tu03 
     &  - 1.583251510732719d-04*tu04 
     &  + 5.083334568184305d-06*tu05 
     &  - 6.708598619683341d-08*tu06 
     &  - 1.944278941559733d-09*tu07 
     &  + 8.804863765356287d-11*tu08 
     &  + 9.341312729419985d-13*tu09 
     &  - 1.231746977889946d-13*tu10 
     &  + 3.370647349658755d-16*tu11 
     &  + 1.718647072955689d-16*tu12 
     &  - 5.822405264650125d-01*HY2(0,-1) 
     &  - 6.931471805599453d-01*HY3(0,-1,-1) 
      HY4(0,-1,1,-1) = 
     &  - 4.495764739674318d-02 
     &  - 2.758514579198452d-01*tv01 
     &  - 4.515130668959398d-02*tv02 
     &  - 3.875995092451054d-03*tv03 
     &  - 1.936768370518385d-04*tv04 
     &  - 5.133195476137788d-06*tv05 
     &  - 1.752786900562004d-08*tv06 
     &  + 2.715518363893619d-09*tv07 
     &  + 1.631155670579918d-11*tv08 
     &  - 2.940721244025822d-12*tv09 
     &  - 2.045219059123054d-14*tv10 
     &  + 3.895696592051861d-15*tv11 
     &  + 4.804530139182014d-01*HY2(0,-1) 
     &  + 6.931471805599453d-01*HY3(0,-1,1) 
      HY4(0,1,-1,-1) = 
     &  - 2.782664607935622d-02 
     &  - 1.410831481728889d-01*tv01 
     &  - 2.801876266982354d-02*tv02 
     &  - 2.997894208020603d-03*tv03 
     &  - 1.921960113936824d-04*tv04 
     &  - 7.016503666427137d-06*tv05 
     &  - 7.928257765061337d-08*tv06 
     &  + 4.388745575295455d-09*tv07 
     &  + 1.381107719492586d-10*tv08 
     &  - 4.341921500497716d-12*tv09 
     &  - 2.375364913875066d-13*tv10 
     &  + 4.522044546598701d-15*tv11 
     &  + 4.033357472727688d-16*tv12 
     &  + 2.402265069591007d-01*HY2(0,1) 
      HY4(0,-1,1,1) = 
     &  + 2.782664607935622d-02 
     &  - 1.410831481728889d-01*tu01 
     &  + 2.801876266982354d-02*tu02 
     &  - 2.997894208020603d-03*tu03 
     &  + 1.921960113936824d-04*tu04 
     &  - 7.016503666427137d-06*tu05 
     &  + 7.928257765061337d-08*tu06 
     &  + 4.388745575295455d-09*tu07 
     &  - 1.381107719492586d-10*tu08 
     &  - 4.341921500497716d-12*tu09 
     &  + 2.375364913875066d-13*tu10 
     &  + 4.522044546598701d-15*tu11 
     &  - 4.033357472727688d-16*tu12 
     &  + 2.402265069591007d-01*HY2(0,-1) 
      HY4(0,1,-1,1) = 
     &  + 4.495764739674318d-02 
     &  - 2.758514579198452d-01*tu01 
     &  + 4.515130668959398d-02*tu02 
     &  - 3.875995092451054d-03*tu03 
     &  + 1.936768370518385d-04*tu04 
     &  - 5.133195476137788d-06*tu05 
     &  + 1.752786900562004d-08*tu06 
     &  + 2.715518363893619d-09*tu07 
     &  - 1.631155670579918d-11*tu08 
     &  - 2.940721244025822d-12*tu09 
     &  + 2.045219059123054d-14*tu10 
     &  + 3.895696592051861d-15*tu11 
     &  + 4.804530139182014d-01*HY2(0,1) 
     &  - 6.931471805599453d-01*HY3(0,1,-1) 
      HY4(0,1,1,-1) = 
     &  + 4.031271939759038d-02 
     &  + 3.295217254379970d-01*tv01 
     &  + 4.047097737450547d-02*tv02 
     &  + 3.104955391145708d-03*tv03 
     &  + 1.583251510732719d-04*tv04 
     &  + 5.083334568184305d-06*tv05 
     &  + 6.708598619683341d-08*tv06 
     &  - 1.944278941559733d-09*tv07 
     &  - 8.804863765356287d-11*tv08 
     &  + 9.341312729419985d-13*tv09 
     &  + 1.231746977889946d-13*tv10 
     &  + 3.370647349658755d-16*tv11 
     &  - 1.718647072955689d-16*tv12 
     &  - 5.822405264650125d-01*HY2(0,1) 
     &  + 6.931471805599453d-01*HY3(0,1,1) 
      HY4(-1,-1,-1,1) = 
     &  - 3.768651335815766d-02 
     &  + 3.043162147119780d-01*tu01 
     &  - 3.784162844891144d-02*tu02 
     &  + 2.958351024362477d-03*tu03 
     &  - 1.551924666783514d-04*tu04 
     &  + 5.216293832777793d-06*tu05 
     &  - 7.726843592398867d-08*tu06 
     &  - 1.910379383726989d-09*tu07 
     &  + 1.073377838077624d-10*tu08 
     &  + 4.147979000313175d-13*tu09 
     &  - 1.506593045440627d-13*tu10 
     &  + 1.921276747438603d-15*tu11 
     &  + 1.977332880766160d-16*tu12 
     &  - 5.372131936080402d-01*HY1(-1) 
     &  - 2.911202632325062d-01*HY1(-1)*HY1(-1) 
     &  - 1.155245300933242d-01*HY1(-1)*HY1(-1)*HY1(-1) 
      HY4(-1,-1,1,1) = 
     &  + 2.908893189635991d-02 
     &  - 1.784837106345115d-01*tu01 
     &  + 2.927117884632272d-02*tu02 
     &  - 2.888221776586007d-03*tu03 
     &  + 1.823501630828519d-04*tu04 
     &  - 6.976883920991888d-06*tu05 
     &  + 1.030302948541690d-07*tu06 
     &  + 3.794029548474434d-09*tu07 
     &  - 1.825184393299693d-10*tu08 
     &  - 2.300206200729610d-12*tu09 
     &  + 3.062629564489397d-13*tu10 
     &  - 7.629393984387632d-16*tu11 
     &  - 4.860728618463296d-16*tu12 
     &  + 3.088253750968339d-01*HY1(-1) 
     &  + 1.201132534795503d-01*HY1(-1)*HY1(-1) 
      HY4(-1,1,1,1) = 
     &  - 9.029205146496301d-03 
     &  + 3.753824045412342d-02*tu01 
     &  - 9.240717745810759d-03*tu02 
     &  + 2.351153976182453d-03*tu03 
     &  - 2.115782190216214d-04*tu04 
     &  + 8.486524807740892d-06*tu05 
     &  - 6.547885807612483d-08*tu06 
     &  - 6.934422754020238d-09*tu07 
     &  + 1.405695202725693d-10*tu08 
     &  + 8.329441237576153d-12*tu09 
     &  - 2.790404594803712d-13*tu10 
     &  - 1.024489568815216d-14*tu11 
     &  + 5.256388245544115d-16*tu12 
     &  - 5.550410866482157d-02*HY1(-1) 
      endif 
** nw > 3 endif 
      if ( nw.gt.4 ) then 
      HY5(-1,-1,-1,-1,1) = 
     &  - 3.8228565672333040d-02
     &  + 2.9434499414685087d-01*tu01
     &  - 3.8424518843646597d-02*tu02
     &  + 3.2580449034204883d-03*tu03
     &  - 1.9620389859663782d-04*tu04
     &  + 8.5297225882767261d-06*tu05
     &  - 2.5065346065715455d-07*tu06
     &  + 3.4415061242343063d-09*tu07
     &  + 7.3821799422249131d-11*tu08
     &  - 4.2538624555519857d-12*tu09
     &  - 7.2315211312081768d-16*tu10
     &  + 5.3108417435543907d-15*tu11
     &  - 9.9323734178161058d-17*tu12
     &  - 5.1747906167389938d-01*HY1(-1)
     &  - 2.6860659680402010d-01*HY1(-1)*HY1(-1)
     &  - 9.7040087744168750d-02*HY1(-1)*HY1(-1)*HY1(-1)
     &  - 2.8881132523331054d-02*HY1(-1)*HY1(-1)*HY1(-1)*HY1(-1)
      HY5(-1,-1,-1,1,1) = 
     &  + 2.7357462427685970d-02
     &  - 1.9031941959865697d-01*tu01
     &  + 2.7543492667729091d-02*tu02
     &  - 2.6605966861706822d-03*tu03
     &  + 1.8636146557007899d-04*tu04
     &  - 9.5345987092093788d-06*tu05
     &  + 3.3110554578258299d-07*tu06
     &  - 5.4595158175118823d-09*tu07
     &  - 1.1999359443095388d-10*tu08
     &  + 8.2902112001115834d-12*tu09
     &  - 1.2181518159922712d-14*tu10
     &  - 1.1713353633997384d-14*tu11
     &  + 2.3651675075246772d-16*tu12
     &  + 3.3160957134970784d-01*HY1(-1)
     &  + 1.5441268754841696d-01*HY1(-1)*HY1(-1)
     &  + 4.0037751159850118d-02*HY1(-1)*HY1(-1)*HY1(-1)
      HY5(-1,-1,1,-1,1) = 
     &  + 4.1458814971381822d-02
     &  - 3.0303763301003844d-01*tu01
     &  + 4.1691982869997716d-02*tu02
     &  - 3.7240373018258494d-03*tu03
     &  + 2.3344990525159924d-04*tu04
     &  - 1.0247007230046718d-05*tu05
     &  + 2.8185771897077629d-07*tu06
     &  - 2.2751673303889503d-09*tu07
     &  - 1.4878969225475077d-10*tu08
     &  + 4.1341440052910325d-12*tu09
     &  + 1.2696091792274665d-13*tu10
     &  - 7.5007742483794021d-15*tu11
     &  - 8.1903610410494478d-17*tu12
     &  + 5.3075770941318154d-01*HY1(-1)
     &  + 2.6126533022459244d-01*HY1(-1)*HY1(-1)
     &  - 6.9314718055994530d-01*HY1(-1)*HY3(-1,-1,1)
     &  + 4.8045301391820142d-01*HY3(-1,-1,1)
     &  + 2.0794415416798359d+00*HY4(-1,-1,-1,1)
      HY5(-1,-1,1,1,1) = 
     &  - 9.5924489184207016d-03
     &  + 5.2772883655818822d-02*tu01
     &  - 9.7366715772116844d-03*tu02
     &  + 1.4142029020952310d-03*tu03
     &  - 1.4463004412432978d-04*tu04
     &  + 9.8218990989160592d-06*tu05
     &  - 4.0715654417888680d-07*tu06
     &  + 6.7544293368802512d-09*tu07
     &  + 2.2866755156107150d-10*tu08
     &  - 1.2497892086833029d-11*tu09
     &  - 1.2173408174458276d-13*tu10
     &  + 2.1387606509664189d-14*tu11
     &  - 1.1689492941664195d-16*tu12
     &  - 3.4168851038812753d-17*tu13
     &  - 8.8326067366010207d-02*HY1(-1)
     &  - 2.7752054332410789d-02*HY1(-1)*HY1(-1)
      HY5(-1,1,-1,1,1) = 
     &  - 1.8886299866599669d-02
     &  + 1.1916393884386234d-01*tu01
     &  - 1.9076154105183607d-02*tu02
     &  + 2.2711434786163337d-03*tu03
     &  - 1.9021198135197550d-04*tu04
     &  + 1.0646966480063299d-05*tu05
     &  - 3.5753730456224019d-07*tu06
     &  + 3.8528188610744423d-09*tu07
     &  + 2.0528377803607047d-10*tu08
     &  - 6.6747997821119625d-12*tu09
     &  - 1.7956391666197610d-13*tu10
     &  + 1.1491009542808092d-14*tu11
     &  + 1.3285418003675663d-16*tu12
     &  - 2.0437039310996688d-01*HY1(-1)
     &  + 2.4022650695910071d-01*HY1(-1)*HY2(-1,1)
     &  - 1.6651232599446473d-01*HY2(-1,1)
     &  - 4.8045301391820142d-01*HY3(-1,-1,1)
      HY5(-1,1,1,1,1) = 
     &  + 1.8171632147206520d-03
     &  - 6.6063186750176729d-03*tu01
     &  + 1.9387809591623771d-03*tu02
     &  - 4.5816039839889645d-04*tu03
     &  + 1.2210356950937828d-04*tu04
     &  - 1.1632122874703951d-05*tu05
     &  + 4.8540847374707648d-07*tu06
     &  - 3.8582255619059790d-09*tu07
     &  - 4.1607939221717784d-10*tu08
     &  + 8.5808819391347779d-12*tu09
     &  + 5.1386964924538773d-13*tu10
     &  - 1.7412399897832375d-14*tu11
     &  - 6.4350879750488940d-16*tu12
     &  + 3.3295304169373632d-17*tu13
     &  + 9.6181291076284771d-03*HY1(-1)
      HY5(0,-1,-1,-1,1) = 
     &  - 3.9520972886442092d-02
     &  + 3.0546490933012253d-01*tu01
     &  - 3.9720322777238976d-02*tu02
     &  + 3.3467933739052856d-03*tu03
     &  - 1.9959549584313573d-04*tu04
     &  + 8.5499586955813381d-06*tu05
     &  - 2.4553310777499893d-07*tu06
     &  + 3.2119489955423060d-09*tu07
     &  + 7.1927096749843198d-11*tu08
     &  - 3.7520798598557195d-12*tu09
     &  - 1.1440674588088539d-14*tu10
     &  + 4.6108325614199602d-15*tu11
     &  - 6.0832641844878483d-17*tu12
     &  - 5.3721319360804020d-01*HY2(0,-1)
     &  - 5.8224052646501250d-01*HY3(0,-1,-1)
     &  - 6.9314718055994530d-01*HY4(0,-1,-1,-1)
      HY5(0,-1,-1,0,1) = 
     &  - 5.4931882964030792d-02
     &  + 4.2696670756984732d-01*tu01
     &  - 5.5201538838609055d-02*tu02
     &  + 4.6052217228062647d-03*tu03
     &  - 2.6995693356946050d-04*tu04
     &  + 1.1207715314455869d-05*tu05
     &  - 3.0093286122242289d-07*tu06
     &  + 2.9252018980885367d-09*tu07
     &  + 1.2606407378756556d-10*tu08
     &  - 4.4893551397064275d-12*tu09
     &  - 6.5916974412553446d-14*tu10
     &  + 6.6066633977237873d-15*tu11
     &  - 7.5128556447474642d-01*HY2(0,-1)
     &  - 8.2246703342411321d-01*HY3(0,-1,-1)
      HY5(0,-1,-1,1,-1) = 
     &  - 3.4235225044469286d-02
     &  - 2.3328399380319120d-01*tv01
     &  - 3.4504895043303540d-02*tv02
     &  - 3.7912859711035303d-03*tv03
     &  - 2.6991423023257518d-04*tv04
     &  - 1.1593278610997365d-05*tv05
     &  - 2.4410984754189600d-07*tv06
     &  + 9.0404989403384568d-10*tv07
     &  + 1.2141001239198240d-10*tv08
     &  - 2.0560625998471416d-12*tv09
     &  - 1.4056423603087806d-13*tv10
     &  + 3.5816535397430763d-15*tv11
     &  + 2.0254744342336250d-16*tv12
     &  + 4.0357837932696163d-01*HY2(0,-1)
     &  + 4.8045301391820142d-01*HY3(0,-1,-1)
     &  + 6.9314718055994530d-01*HY4(0,-1,-1,1)
      HY5(0,-1,-1,1,1) = 
     &  + 2.6517569052959506d-02
     &  - 1.7779971519631702d-01*tu01
     &  + 2.6707704581331518d-02*tu02
     &  - 2.6648397927870009d-03*tu03
     &  + 1.9046558373551644d-04*tu04
     &  - 9.7589088614312887d-06*tu05
     &  + 3.2991992381966999d-07*tu06
     &  - 4.8561819501772588d-09*tu07
     &  - 1.3539320891044926d-10*tu08
     &  + 7.2902837287752652d-12*tu09
     &  + 4.6565266727231378d-14*tu10
     &  - 1.0737158384822919d-14*tu11
     &  + 8.8429222559013066d-17*tu12
     &  + 3.0882537509683393d-01*HY2(0,-1)
     &  + 2.4022650695910071d-01*HY3(0,-1,-1)
      HY5(0,-1,0,-1,1) = 
     &  - 4.6155088778537896d-02
     &  + 3.5912469692896398d-01*tu01
     &  - 4.6380859407450647d-02*tu02
     &  + 3.8633681101991486d-03*tu03
     &  - 2.2602267773980855d-04*tu04
     &  + 9.3668504249552797d-06*tu05
     &  - 2.5195369443104781d-07*tu06
     &  + 2.5727032623578792d-09*tu07
     &  + 9.5079191917240018d-11*tu08
     &  - 3.3913895763562877d-12*tu09
     &  - 5.3430708391974222d-14*tu10
     &  + 4.8668530674676388d-15*tu11
     &  - 6.3196619783816790d-01*HY2(0,-1)
     &  - 3.4657359027997265d-01*HY2(0,-1)*HY2(0,-1)
     &  + 1.3862943611198906d+00*HY4(0,0,-1,-1)
      HY5(0,-1,0,1,-1) = 
     &  - 4.8531326139900090d-02
     &  - 3.2657418615138227d-01*tv01
     &  - 4.8786752720619143d-02*tv02
     &  - 4.3579589418437103d-03*tv03
     &  - 2.5565369900267449d-04*tv04
     &  - 9.9174468231600028d-06*tv05
     &  - 2.2703683570409072d-07*tv06
     &  - 1.2319581804863699d-09*tv07
     &  + 8.1382125029078829d-11*tv08
     &  + 1.0513084104970317d-12*tv09
     &  - 6.5723688498545463d-14*tv10
     &  - 1.2162924878946765d-15*tv11
     &  + 6.9118041556917233d-17*tv12
     &  + 5.7009070532142637d-01*HY2(0,-1)
     &  + 6.9314718055994530d-01*HY4(0,-1,0,1)
      HY5(0,-1,0,1,1) = 
     &  + 1.7588255219497996d-02
     &  - 8.9173088210648816d-02*tu01
     &  + 1.7770012282463661d-02*tu02
     &  - 2.1943047496140125d-03*tu03
     &  + 1.8211824241696307d-04*tu04
     &  - 1.0254443539590487d-05*tu05
     &  + 3.6099159031901396d-07*tu06
     &  - 4.8590233971583916d-09*tu07
     &  - 1.8773473287955894d-10*tu08
     &  + 8.2188353537397648d-12*tu09
     &  + 1.2622338876212118d-13*tu10
     &  - 1.3549277713840031d-14*tu11
     &  - 2.2995013881263894d-17*tu12
     &  + 2.1790299145411898d-17*tu13
     &  + 1.5025711289494928d-01*HY2(0,-1)
      HY5(0,-1,1,-1,-1) = 
     &  - 1.5970421220241350d-02
     &  - 9.8098905332736657d-02*tv01
     &  - 1.6167874610049324d-02*tv02
     &  - 2.1909422567422571d-03*tv03
     &  - 1.9780330842002595d-04*tv04
     &  - 1.1143616753061323d-05*tv05
     &  - 3.4970788285258598d-07*tv06
     &  - 2.4737298345971262d-09*tv07
     &  + 2.1049905271699688d-10*tv08
     &  + 3.6829751452188057d-12*tv09
     &  - 2.2984898297836793d-13*tv10
     &  - 5.8041435355729247d-15*tv11
     &  + 2.9753327209204345d-16*tv12
     &  + 1.6651232599446473d-01*HY2(0,-1)
     &  + 2.4022650695910071d-01*HY3(0,-1,1)
      HY5(0,-1,1,-1,1) = 
     &  + 4.1824168833514614d-02
     &  - 2.9878381943835587d-01*tu01
     &  + 4.2063530444775636d-02*tu02
     &  - 3.8145573036522436d-03*tu03
     &  + 2.3963305636044003d-04*tu04
     &  - 1.0345567145401369d-05*tu05
     &  + 2.7130517497620301d-07*tu06
     &  - 1.7273255620176561d-09*tu07
     &  - 1.3978876407646153d-10*tu08
     &  + 2.7492226699257445d-12*tu09
     &  + 1.3553332679630927d-13*tu10
     &  - 4.7656087734784391d-15*tu11
     &  - 1.4423054189885051d-16*tu12
     &  + 5.2253066044918489d-01*HY2(0,-1)
     &  + 4.8045301391820142d-01*HY3(0,-1,1)
     &  - 6.9314718055994530d-01*HY4(0,-1,1,-1)
      HY5(0,-1,1,0,1) = 
     &  + 3.1399803193734681d-02
     &  - 1.5922070722448512d-01*tu01
     &  + 3.1673651121964784d-02*tu02
     &  - 3.6733024485531673d-03*tu03
     &  + 2.7421207386188742d-04*tu04
     &  - 1.3254697744560554d-05*tu05
     &  + 3.6389461745542725d-07*tu06
     &  - 1.3701468350861509d-09*tu07
     &  - 2.5073096543709732d-10*tu08
     &  + 3.7881257959182565d-12*tu09
     &  + 2.8301894299500673d-13*tu10
     &  - 7.9052198960553283d-15*tu11
     &  - 3.4475230007347254d-16*tu12
     &  + 2.6957647953152780d-01*HY2(0,-1)
      HY5(0,-1,1,1,-1) = 
     &  + 4.4239483860061491d-02
     &  + 3.2493754537326350d-01*tv01
     &  + 4.4462186515298750d-02*tv02
     &  + 3.8108642536456408d-03*tv03
     &  + 2.2293231798666157d-04*tv04
     &  + 8.9901447113222695d-06*tv05
     &  + 2.2959225139995976d-07*tv06
     &  + 2.1985163564934731d-09*tv07
     &  - 7.0460025669936385d-11*tv08
     &  - 2.1618428565949139d-12*tv09
     &  + 3.7961815108691857d-14*tv10
     &  + 2.5062265861081495d-15*tv11
     &  - 5.7009070532142637d-01*HY2(0,-1)
     &  - 5.8224052646501250d-01*HY3(0,-1,1)
     &  + 6.9314718055994530d-01*HY4(0,-1,1,1)
      HY5(0,-1,1,1,1) = 
     &  - 6.7365547952682237d-03
     &  + 3.4286529141621258d-02*tu01
     &  - 6.8840170170275732d-03*tu02
     &  + 1.2704332689803235d-03*tu03
     &  - 1.4787767126158972d-04*tu04
     &  + 1.0413585642621615d-05*tu05
     &  - 4.1517140759699248d-07*tu06
     &  + 5.3439659402510695d-09*tu07
     &  + 2.7781700180675136d-10*tu08
     &  - 9.9205586909926692d-12*tu09
     &  - 2.7736473454099143d-13*tu10
     &  + 1.7683378840938849d-14*tu11
     &  + 2.7645451246534933d-16*tu12
     &  - 3.0721858829145982d-17*tu13
     &  - 5.5504108664821579d-02*HY2(0,-1)
      HY5(0,0,-1,-1,1) = 
     &  - 4.2561833953883235d-02
     &  + 3.3089514249958932d-01*tu01
     &  - 4.2771142986440044d-02*tu02
     &  + 3.5688685742597137d-03*tu03
     &  - 2.0954973101068707d-04*tu04
     &  + 8.7544434158133471d-06*tu05
     &  - 2.4062186086108688d-07*tu06
     &  + 2.7728194984089187d-09*tu07
     &  + 7.6559566355403117d-11*tu08
     &  - 3.1501612342558870d-12*tu09
     &  - 3.3459877468585252d-14*tu10
     &  + 4.0333595766267801d-15*tu11
     &  - 5.8224052646501250d-01*HY3(0,0,-1)
     &  - 6.9314718055994530d-01*HY4(0,0,-1,-1)
      HY5(0,0,-1,0,1) = 
     &  - 5.9694681535720023d-02
     &  + 4.6714213068105458d-01*tu01
     &  - 5.9979199873202721d-02*tu02
     &  + 4.9475467229010258d-03*tu03
     &  - 2.8481020227431533d-04*tu04
     &  + 1.1478749310592714d-05*tu05
     &  - 2.9173546427111336d-07*tu06
     &  + 2.2634918139358350d-09*tu07
     &  + 1.2923277934839262d-10*tu08
     &  - 3.4600488918561596d-12*tu09
     &  - 9.4505709251699823d-14*tu10
     &  + 5.3596484866903464d-15*tu11
     &  + 6.0838246089431129d-17*tu12
     &  - 8.2246703342411321d-01*HY3(0,0,-1)
      HY5(0,0,-1,1,-1) = 
     &  - 4.1110148228889138d-02
     &  - 2.7597446804002515d-01*tv01
     &  - 4.1362136148261452d-02*tv02
     &  - 3.9264219532611858d-03*tv03
     &  - 2.5225247282294274d-04*tv04
     &  - 1.0792153931365190d-05*tv05
     &  - 2.6440687189609348d-07*tv06
     &  - 7.5507087052363252d-10*tv07
     &  + 1.4640592504212342d-10*tv08
     &  + 1.0410408318930554d-12*tv09
     &  - 1.7256282627937244d-13*tv10
     &  - 2.0115212902641882d-15*tv11
     &  + 2.4492838656439672d-16*tv12
     &  + 4.8045301391820142d-01*HY3(0,0,-1)
     &  + 6.9314718055994530d-01*HY4(0,0,-1,1)
      HY5(0,0,-1,1,1) = 
     &  + 2.3290431799544722d-02
     &  - 1.3976916667674197d-01*tu01
     &  + 2.3484837128475192d-02*tu02
     &  - 2.5652450331604480d-03*tu03
     &  + 1.9474432423788960d-04*tu04
     &  - 1.0235257714295329d-05*tu05
     &  + 3.3882452171323895d-07*tu06
     &  - 4.1398941829422872d-09*tu07
     &  - 1.7065390810468420d-10*tu08
     &  + 6.5500661689981377d-12*tu09
     &  + 1.3171293231469574d-13*tu10
     &  - 1.0601060206261924d-14*tu11
     &  - 8.5654606012582514d-17*tu12
     &  + 2.4022650695910071d-01*HY3(0,0,-1)
      HY5(0,0,0,-1,1) = 
     &  - 5.0232086741389027d-02
     &  + 3.9364648125763205d-01*tu01
     &  - 5.0470343357276722d-02*tu02
     &  + 4.1544817848152132d-03*tu03
     &  - 2.3850059599743822d-04*tu04
     &  + 9.5855250288941914d-06*tu05
     &  - 2.4388351988986297d-07*tu06
     &  + 2.0282340296972451d-09*tu07
     &  + 9.6515154204163078d-11*tu08
     &  - 2.5201744413354707d-12*tu09
     &  - 7.4636488424158657d-14*tu10
     &  + 3.7370382545755637d-15*tu11
     &  + 6.2133891701574733d-17*tu12
     &  - 6.9314718055994530d-01*HY4(0,0,0,-1)
      HY5(0,0,0,1,-1) = 
     &  - 5.0232086741389027d-02
     &  - 3.9364648125763205d-01*tv01
     &  - 5.0470343357276722d-02*tv02
     &  - 4.1544817848152132d-03*tv03
     &  - 2.3850059599743822d-04*tv04
     &  - 9.5855250288941914d-06*tv05
     &  - 2.4388351988986297d-07*tv06
     &  - 2.0282340296972451d-09*tv07
     &  + 9.6515154204163078d-11*tv08
     &  + 2.5201744413354707d-12*tv09
     &  - 7.4636488424158657d-14*tv10
     &  - 3.7370382545755637d-15*tv11
     &  + 6.2133891701574733d-17*tv12
     &  + 6.9314718055994530d-01*HY4(0,0,0,1)
      HY5(0,0,1,-1,-1) = 
     &  - 2.3290431799544722d-02
     &  - 1.3976916667674197d-01*tv01
     &  - 2.3484837128475192d-02*tv02
     &  - 2.5652450331604480d-03*tv03
     &  - 1.9474432423788960d-04*tv04
     &  - 1.0235257714295329d-05*tv05
     &  - 3.3882452171323895d-07*tv06
     &  - 4.1398941829422872d-09*tv07
     &  + 1.7065390810468420d-10*tv08
     &  + 6.5500661689981377d-12*tv09
     &  - 1.3171293231469574d-13*tv10
     &  - 1.0601060206261924d-14*tv11
     &  + 8.5654606012582514d-17*tv12
     &  + 2.4022650695910071d-01*HY3(0,0,1)
      HY5(0,0,1,-1,1) = 
     &  + 4.1110148228889138d-02
     &  - 2.7597446804002515d-01*tu01
     &  + 4.1362136148261452d-02*tu02
     &  - 3.9264219532611858d-03*tu03
     &  + 2.5225247282294274d-04*tu04
     &  - 1.0792153931365190d-05*tu05
     &  + 2.6440687189609348d-07*tu06
     &  - 7.5507087052363252d-10*tu07
     &  - 1.4640592504212342d-10*tu08
     &  + 1.0410408318930554d-12*tu09
     &  + 1.7256282627937244d-13*tu10
     &  - 2.0115212902641882d-15*tu11
     &  - 2.4492838656439672d-16*tu12
     &  + 4.8045301391820142d-01*HY3(0,0,1)
     &  - 6.9314718055994530d-01*HY4(0,0,1,-1)
      HY5(0,0,1,0,-1) = 
     &  - 5.9694681535720023d-02
     &  - 4.6714213068105458d-01*tv01
     &  - 5.9979199873202721d-02*tv02
     &  - 4.9475467229010258d-03*tv03
     &  - 2.8481020227431533d-04*tv04
     &  - 1.1478749310592714d-05*tv05
     &  - 2.9173546427111336d-07*tv06
     &  - 2.2634918139358350d-09*tv07
     &  + 1.2923277934839262d-10*tv08
     &  + 3.4600488918561596d-12*tv09
     &  - 9.4505709251699823d-14*tv10
     &  - 5.3596484866903464d-15*tv11
     &  + 6.0838246089431129d-17*tv12
     &  + 8.2246703342411321d-01*HY3(0,0,1)
      HY5(0,0,1,1,-1) = 
     &  + 4.2561833953883235d-02
     &  + 3.3089514249958932d-01*tv01
     &  + 4.2771142986440044d-02*tv02
     &  + 3.5688685742597137d-03*tv03
     &  + 2.0954973101068707d-04*tv04
     &  + 8.7544434158133471d-06*tv05
     &  + 2.4062186086108688d-07*tv06
     &  + 2.7728194984089187d-09*tv07
     &  - 7.6559566355403117d-11*tv08
     &  - 3.1501612342558870d-12*tv09
     &  + 3.3459877468585252d-14*tv10
     &  + 4.0333595766267801d-15*tv11
     &  - 5.8224052646501250d-01*HY3(0,0,1)
     &  + 6.9314718055994530d-01*HY4(0,0,1,1)
      HY5(0,1,-1,-1,-1) = 
     &  - 6.7365547952682237d-03
     &  - 3.4286529141621258d-02*tv01
     &  - 6.8840170170275732d-03*tv02
     &  - 1.2704332689803235d-03*tv03
     &  - 1.4787767126158972d-04*tv04
     &  - 1.0413585642621615d-05*tv05
     &  - 4.1517140759699248d-07*tv06
     &  - 5.3439659402510695d-09*tv07
     &  + 2.7781700180675136d-10*tv08
     &  + 9.9205586909926692d-12*tv09
     &  - 2.7736473454099143d-13*tv10
     &  - 1.7683378840938849d-14*tv11
     &  + 2.7645451246534933d-16*tv12
     &  + 3.0721858829145982d-17*tv13
     &  + 5.5504108664821579d-02*HY2(0,1)
      HY5(0,1,-1,-1,1) = 
     &  + 4.4239483860061491d-02
     &  - 3.2493754537326350d-01*tu01
     &  + 4.4462186515298750d-02*tu02
     &  - 3.8108642536456408d-03*tu03
     &  + 2.2293231798666157d-04*tu04
     &  - 8.9901447113222695d-06*tu05
     &  + 2.2959225139995976d-07*tu06
     &  - 2.1985163564934731d-09*tu07
     &  - 7.0460025669936385d-11*tu08
     &  + 2.1618428565949139d-12*tu09
     &  + 3.7961815108691857d-14*tu10
     &  - 2.5062265861081495d-15*tu11
     &  + 5.7009070532142637d-01*HY2(0,1)
     &  - 5.8224052646501250d-01*HY3(0,1,-1)
     &  - 6.9314718055994530d-01*HY4(0,1,-1,-1)
      HY5(0,1,-1,1,-1) = 
     &  + 4.1824168833514614d-02
     &  + 2.9878381943835587d-01*tv01
     &  + 4.2063530444775636d-02*tv02
     &  + 3.8145573036522436d-03*tv03
     &  + 2.3963305636044003d-04*tv04
     &  + 1.0345567145401369d-05*tv05
     &  + 2.7130517497620301d-07*tv06
     &  + 1.7273255620176561d-09*tv07
     &  - 1.3978876407646153d-10*tv08
     &  - 2.7492226699257445d-12*tv09
     &  + 1.3553332679630927d-13*tv10
     &  + 4.7656087734784391d-15*tv11
     &  - 1.4423054189885051d-16*tv12
     &  - 5.2253066044918489d-01*HY2(0,1)
     &  + 4.8045301391820142d-01*HY3(0,1,-1)
     &  + 6.9314718055994530d-01*HY4(0,1,-1,1)
      HY5(0,1,-1,1,1) = 
     &  - 1.5970421220241350d-02
     &  + 9.8098905332736657d-02*tu01
     &  - 1.6167874610049324d-02*tu02
     &  + 2.1909422567422571d-03*tu03
     &  - 1.9780330842002595d-04*tu04
     &  + 1.1143616753061323d-05*tu05
     &  - 3.4970788285258598d-07*tu06
     &  + 2.4737298345971262d-09*tu07
     &  + 2.1049905271699688d-10*tu08
     &  - 3.6829751452188057d-12*tu09
     &  - 2.2984898297836793d-13*tu10
     &  + 5.8041435355729247d-15*tu11
     &  + 2.9753327209204345d-16*tu12
     &  - 1.6651232599446473d-01*HY2(0,1)
     &  + 2.4022650695910071d-01*HY3(0,1,-1)
      HY5(0,1,0,1,-1) = 
     &  + 4.6155088778537896d-02
     &  + 3.5912469692896398d-01*tv01
     &  + 4.6380859407450647d-02*tv02
     &  + 3.8633681101991486d-03*tv03
     &  + 2.2602267773980855d-04*tv04
     &  + 9.3668504249552797d-06*tv05
     &  + 2.5195369443104781d-07*tv06
     &  + 2.5727032623578792d-09*tv07
     &  - 9.5079191917240018d-11*tv08
     &  - 3.3913895763562877d-12*tv09
     &  + 5.3430708391974222d-14*tv10
     &  + 4.8668530674676388d-15*tv11
     &  - 6.3196619783816790d-01*HY2(0,1)
     &  + 3.4657359027997265d-01*HY2(0,1)*HY2(0,1)
     &  - 1.3862943611198906d+00*HY4(0,0,1,1)
      HY5(0,1,1,-1,-1) = 
     &  + 2.6517569052959506d-02
     &  + 1.7779971519631702d-01*tv01
     &  + 2.6707704581331518d-02*tv02
     &  + 2.6648397927870009d-03*tv03
     &  + 1.9046558373551644d-04*tv04
     &  + 9.7589088614312887d-06*tv05
     &  + 3.2991992381966999d-07*tv06
     &  + 4.8561819501772588d-09*tv07
     &  - 1.3539320891044926d-10*tv08
     &  - 7.2902837287752652d-12*tv09
     &  + 4.6565266727231378d-14*tv10
     &  + 1.0737158384822919d-14*tv11
     &  + 8.8429222559013066d-17*tv12
     &  - 3.0882537509683393d-01*HY2(0,1)
     &  + 2.4022650695910071d-01*HY3(0,1,1)
      HY5(0,1,1,-1,1) = 
     &  - 3.4235225044469286d-02
     &  + 2.3328399380319120d-01*tu01
     &  - 3.4504895043303540d-02*tu02
     &  + 3.7912859711035303d-03*tu03
     &  - 2.6991423023257518d-04*tu04
     &  + 1.1593278610997365d-05*tu05
     &  - 2.4410984754189600d-07*tu06
     &  - 9.0404989403384568d-10*tu07
     &  + 1.2141001239198240d-10*tu08
     &  + 2.0560625998471416d-12*tu09
     &  - 1.4056423603087806d-13*tu10
     &  - 3.5816535397430763d-15*tu11
     &  + 2.0254744342336250d-16*tu12
     &  - 4.0357837932696163d-01*HY2(0,1)
     &  + 4.8045301391820142d-01*HY3(0,1,1)
     &  - 6.9314718055994530d-01*HY4(0,1,1,-1)
      HY5(0,1,1,1,-1) = 
     &  - 3.9520972886442092d-02
     &  - 3.0546490933012253d-01*tv01
     &  - 3.9720322777238976d-02*tv02
     &  - 3.3467933739052856d-03*tv03
     &  - 1.9959549584313573d-04*tv04
     &  - 8.5499586955813381d-06*tv05
     &  - 2.4553310777499893d-07*tv06
     &  - 3.2119489955423060d-09*tv07
     &  + 7.1927096749843198d-11*tv08
     &  + 3.7520798598557195d-12*tv09
     &  - 1.1440674588088539d-14*tv10
     &  - 4.6108325614199602d-15*tv11
     &  - 6.0832641844878483d-17*tv12
     &  + 5.3721319360804020d-01*HY2(0,1)
     &  - 5.8224052646501250d-01*HY3(0,1,1)
     &  + 6.9314718055994530d-01*HY4(0,1,1,1)
      endif 
** nw > 4 endif 
      if ( nw.gt.5 ) then 
* 2017-04-19:17:41:57.hpl 
* <- aaa 
* produced by form-to-fortr for gehrt@tintin 
      HY6(-1,-1,-1,-1,-1,1) = 
     &  - 3.8413550594472307d-02
     &  + 2.8976616805468099d-01*tu01
     &  - 3.8631644802770072d-02*tu02
     &  + 3.3997050597419339d-03*tu03
     &  - 2.1848750445853155d-04*tu04
     &  + 1.0667348420397359d-05*tu05
     &  - 3.9342156502493443d-07*tu06
     &  + 1.0144503946942839d-08*tu07
     &  - 1.2526572533861094d-10*tu08
     &  - 2.5314786563254113d-12*tu09
     &  + 1.3837721291684304d-13*tu10
     &  - 1.3629864161736989d-16*tu11
     &  - 1.5630397882536522d-16*tu12
     &  - 5.0840057924226870d-01*HY1(-1)
     &  - 2.5873953083694969d-01*HY1(-1)*HY1(-1)
     &  - 8.9535532268006700d-02*HY1(-1)*HY1(-1)*HY1(-1)
     &  - 2.4260021936042187d-02*HY1(-1)*HY1(-1)*HY1(-1)*HY1(-1)
     &  - 5.7762265046662109d-03*HY1(-1)*HY1(-1)*HY1(-1)*HY1(-1)*HY1(-1)
      HY6(-1,-1,-1,-1,1,1) = 
     &  + 2.6906919847135213d-02
     &  - 1.9467542835151119d-01*tu01
     &  + 2.7086279823157772d-02*tu02
     &  - 2.5459880458799191d-03*tu03
     &  + 1.7978811949728922d-04*tu04
     &  - 9.9461391367666266d-06*tu05
     &  + 4.2836192387555187d-07*tu06
     &  - 1.3326609298639408d-08*tu07
     &  + 2.1816481588148876d-10*tu08
     &  + 3.3761110185447901d-12*tu09
     &  - 2.8399695032252647d-13*tu10
     &  + 2.3835751442118126d-15*tu11
     &  + 3.3294178603857986d-16*tu12
     &  + 3.4015836653260279d-01*HY1(-1)
     &  + 1.6580478567485392d-01*HY1(-1)*HY1(-1)
     &  + 5.1470895849472322d-02*HY1(-1)*HY1(-1)*HY1(-1)
     &  + 1.0009437789962529d-02*HY1(-1)*HY1(-1)*HY1(-1)*HY1(-1)
      HY6(-1,-1,-1,1,-1,1) = 
     &  + 4.1069415272643869d-02
     &  - 3.0375564032957780d-01*tu01
     &  + 4.1317204235538590d-02*tu02
     &  - 3.7370330050076453d-03*tu03
     &  + 2.4826025523464613d-04*tu04
     &  - 1.2530729196553255d-05*tu05
     &  + 4.7138970957006527d-07*tu06
     &  - 1.1773431765782823d-08*tu07
     &  + 9.7195839187799784d-11*tu08
     &  + 5.4161528206042636d-12*tu09
     &  - 1.7351082201233237d-13*tu10
     &  - 3.4902567154702786d-15*tu11
     &  + 2.9460768843458435d-16*tu12
     &  + 5.3201293181405728d-01*HY1(-1)
     &  + 2.6537885470659077d-01*HY1(-1)*HY1(-1)
     &  + 8.7088443408197481d-02*HY1(-1)*HY1(-1)*HY1(-1)
     &  - 6.9314718055994530d-01*HY1(-1)*HY4(-1,-1,-1,1)
     &  + 4.8045301391820142d-01*HY4(-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HY5(-1,-1,-1,-1,1)
      HY6(-1,-1,-1,1,1,1) = 
     &  - 9.4542610212787802d-03
     &  + 6.0841379798218766d-02*tu01
     &  - 9.5554413560231112d-03*tu02
     &  + 1.1020025680101537d-03*tu03
     &  - 1.0159810012187292d-04*tu04
     &  + 7.4686032690299971d-06*tu05
     &  - 4.1807958232610571d-07*tu06
     &  + 1.6185576487430208d-08*tu07
     &  - 3.1371375653145937d-10*tu08
     &  - 5.4721425018470270d-12*tu09
     &  + 4.9034042184310914d-13*tu10
     &  - 3.0526906883839077d-15*tu11
     &  - 6.8643848367506216d-16*tu12
     &  - 1.0467745774960468d-01*HY1(-1)
     &  - 4.4163033683005103d-02*HY1(-1)*HY1(-1)
     &  - 9.2506847774702633d-03*HY1(-1)*HY1(-1)*HY1(-1)
      HY6(-1,-1,1,-1,1,1) = 
     &  - 1.8582210183251431d-02
     &  + 1.2785302058894487d-01*tu01
     &  - 1.8736928625032445d-02*tu02
     &  + 1.9358972729839026d-03*tu03
     &  - 1.5518535703528318d-04*tu04
     &  + 9.7909345444852182d-06*tu05
     &  - 4.6711653377335028d-07*tu06
     &  + 1.5027786612082862d-08*tu07
     &  - 2.0094440139549188d-10*tu08
     &  - 6.9112848058413562d-12*tu09
     &  + 3.3456602865397233d-13*tu10
     &  + 3.1457421219666297d-15*tu11
     &  - 5.3549481640810315d-16*tu12
     &  - 2.2198941488358761d-01*HY1(-1)
     &  - 1.0218519655498344d-01*HY1(-1)*HY1(-1)
     &  + 2.4022650695910071d-01*HY1(-1)*HY3(-1,-1,1)
     &  - 1.6651232599446473d-01*HY3(-1,-1,1)
     &  - 7.2067952087730213d-01*HY4(-1,-1,-1,1)
      HY6(-1,-1,1,1,-1,1) = 
     &  - 3.5978332039058649d-02
     &  + 2.5878715219056551d-01*tu01
     &  - 3.6224328895253732d-02*tu02
     &  + 3.4562505761045046d-03*tu03
     &  - 2.4652045483792073d-04*tu04
     &  + 1.3351711024676714d-05*tu05
     &  - 5.2362952529181397d-07*tu06
     &  + 1.2486824818286007d-08*tu07
     &  - 3.0763948457558645d-11*tu08
     &  - 8.3173774323497666d-12*tu09
     &  + 1.1819979729409430d-13*tu10
     &  + 9.8094743956600695d-15*tu11
     &  - 3.0579477821586108d-16*tu12
     &  - 4.5179103915388582d-01*HY1(-1)
     &  - 2.1877973835097366d-01*HY1(-1)*HY1(-1)
     &  - 6.9314718055994530d-01*HY1(-1)*HY4(-1,-1,1,1)
     &  - 4.0357837932696163d-01*HY3(-1,-1,1)
     &  + 4.8045301391820142d-01*HY4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HY5(-1,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HY5(-1,-1,1,-1,1)
      HY6(-1,-1,1,1,1,1) = 
     &  + 2.0888476943205402d-03
     &  - 1.0702699845086313d-02*tu01
     &  + 2.1435146006608021d-03*tu02
     &  - 3.6020707247573519d-04*tu03
     &  + 5.5111632088610033d-05*tu04
     &  - 6.1128941737241135d-06*tu05
     &  + 4.4507313789001719d-07*tu06
     &  - 1.9555240253889242d-08*tu07
     &  + 3.4672143488260894d-10*tu08
     &  + 1.1388849955643079d-11*tu09
     &  - 6.6693978590725648d-13*tu10
     &  - 5.6147307420125717d-15*tu11
     &  + 1.1652966231770518d-15*tu12
     &  + 1.7550011107790266d-02*HY1(-1)
     &  + 4.8090645538142385d-03*HY1(-1)*HY1(-1)
      HY6(-1,1,-1,1,1,1) = 
     &  + 4.8926787372922233d-03
     &  - 2.9463074422316789d-02*tu01
     &  + 4.9730038573719289d-03*tu02
     &  - 6.6554127350996382d-04*tu03
     &  + 8.0793001630614684d-05*tu04
     &  - 7.4801981937696409d-06*tu05
     &  + 4.6812186790126317d-07*tu06
     &  - 1.7666933603137034d-08*tu07
     &  + 2.3989063071359964d-10*tu08
     &  + 1.0236706600795278d-11*tu09
     &  - 4.2563373374533687d-13*tu10
     &  - 7.8467691063718599d-15*tu11
     &  + 7.2646673106111446d-16*tu12
     &  + 5.0006777876370569d-02*HY1(-1)
     &  - 5.5504108664821579d-02*HY1(-1)*HY2(-1,1)
     &  + 3.8472516430513908d-02*HY2(-1,1)
     &  + 1.1100821732964315d-01*HY3(-1,-1,1)
      HY6(-1,1,1,1,1,1) = 
     &  - 2.5656192943482856d-04
     &  + 9.5295375827554310d-04*tu01
     &  - 2.7574739466164920d-04*tu02
     &  + 8.2154615793801461d-05*tu03
     &  - 1.9718632516963067d-05*tu04
     &  + 5.4031964697592031d-06*tu05
     &  - 5.3335291993333417d-07*tu06
     &  + 2.2858871378644709d-08*tu07
     &  - 1.8520510891673444d-10*tu08
     &  - 2.0311897034739079d-11*tu09
     &  + 4.2380455327850891d-13*tu10
     &  + 2.5656531088433544d-14*tu11
     &  - 8.7568270503996947d-16*tu12
     &  - 3.2622744500862127d-17*tu13
     &  - 1.3333558146428443d-03*HY1(-1)
      HY6(0,-1,-1,-1,-1,-1) = 
     &  + 6.0070834689670138d-06
     &  + 5.3248530359544277d-05*tv01
     &  + 9.0106252034505208d-06*tv02
     &  + 2.6706380095123042d-05*tv03
     &  + 3.6042500813802083d-06*tv04
     &  + 5.4069130961504932d-06*tv05
     &  + 6.0070834689670138d-07*tv06
     &  + 2.3392901289292290d-08*tv07
     &  - 2.2764221454379075d-11*tv09
     &  + 3.3383355497601399d-14*tv11
     &  - 5.3170479809609942d-17*tv13
      HY6(0,-1,-1,-1,-1,1) = 
     &  - 3.9047609723280097d-02
     &  + 2.9490526605059110d-01*tu01
     &  - 3.9268080810198696d-02*tu02
     &  + 3.4484474205677681d-03*tu03
     &  - 2.2086339957610831d-04*tu04
     &  + 1.0726031827335130d-05*tu05
     &  - 3.9243399128859247d-07*tu06
     &  + 1.0000797143348540d-08*tu07
     &  - 1.2120656672236459d-10*tu08
     &  - 2.4221748486512298d-12*tu09
     &  + 1.2707263896767648d-13*tu10
     &  + 2.4294969573951350d-17*tu11
     &  - 1.3968563316726440d-16*tu12
     &  - 5.1747906167389938d-01*HY2(0,-1)
     &  - 5.3721319360804020d-01*HY3(0,-1,-1)
     &  - 5.8224052646501250d-01*HY4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HY5(0,-1,-1,-1,-1)
      HY6(0,-1,-1,-1,0,1) = 
     &  - 5.4210390477114290d-02
     &  + 4.1041438060498855d-01*tu01
     &  - 5.4512669144003579d-02*tu02
     &  + 4.7657001519164058d-03*tu03
     &  - 3.0279479149517346d-04*tu04
     &  + 1.4495579779915431d-05*tu05
     &  - 5.1623965980491050d-07*tu06
     &  + 1.2391364027287551d-08*tu07
     &  - 1.1489328576322265d-10*tu08
     &  - 4.3322430009826430d-12*tu09
     &  + 1.6042303577001692d-13*tu10
     &  + 1.5090589337905225d-15*tu11
     &  - 2.1185535062052255d-16*tu12
     &  - 7.2034485685378902d-01*HY2(0,-1)
     &  - 7.5128556447474642d-01*HY3(0,-1,-1)
     &  - 8.2246703342411321d-01*HY4(0,-1,-1,-1)
      HY6(0,-1,-1,-1,1,-1) = 
     &  - 3.0081658634243567d-02
     &  - 2.1377036357609870d-01*tv01
     &  - 3.0329672786178485d-02*tv02
     &  - 3.0144165710366503d-03*tv03
     &  - 2.4858382360567612d-04*tv04
     &  - 1.5049849884738328d-05*tv05
     &  - 5.6956244333839007d-07*tv06
     &  - 9.6134425140637308d-09*tv07
     &  + 1.0902911234284070d-10*tv08
     &  + 4.2555707467912415d-12*tv09
     &  - 1.9797730260826362d-13*tv10
     &  - 5.0189926172500353d-15*tv11
     &  + 3.2950206131955252d-16*tv12
     &  + 3.7236781050901709d-01*HY2(0,-1)
     &  + 4.0357837932696163d-01*HY3(0,-1,-1)
     &  + 4.8045301391820142d-01*HY4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HY5(0,-1,-1,-1,1)
      HY6(0,-1,-1,-1,1,1) = 
     &  + 2.6478241559869173d-02
     &  - 1.8993300915405635d-01*tu01
     &  + 2.6658542142639876d-02*tu02
     &  - 2.5326606660653166d-03*tu03
     &  + 1.8073315765542287d-04*tu04
     &  - 1.0065956022582163d-05*tu05
     &  + 4.3277909719234731d-07*tu06
     &  - 1.3237218152535902d-08*tu07
     &  + 2.0395065078922920d-10*tu08
     &  + 3.6798400817507221d-12*tu09
     &  - 2.6150191996663759d-13*tu10
     &  + 1.0484356376131264d-15*tu11
     &  + 3.1909849879503528d-16*tu12
     &  + 3.3160957134970784d-01*HY2(0,-1)
     &  + 3.0882537509683393d-01*HY3(0,-1,-1)
     &  + 2.4022650695910071d-01*HY4(0,-1,-1,-1)
      HY6(0,-1,-1,0,-1,1) = 
     &  - 4.5544596606099586d-02
     &  + 3.4494831648814882d-01*tu01
     &  - 4.5798125288184938d-02*tu02
     &  + 4.0011405126577423d-03*tu03
     &  - 2.5396042145666137d-04*tu04
     &  + 1.2141399615075767d-05*tu05
     &  - 4.3183909697476730d-07*tu06
     &  + 1.0377390115334517d-08*tu07
     &  - 9.9602135168043144d-11*tu08
     &  - 3.3423378644967503d-12*tu09
     &  + 1.2336708525660502d-13*tu10
     &  + 1.3227403520085467d-15*tu11
     &  - 1.6300128464629880d-16*tu12
     &  - 6.0546459868440834d-01*HY2(0,-1)
     &  - 6.9314718055994530d-01*HY2(0,-1)*HY3(0,-1,-1)
     &  - 6.3196619783816790d-01*HY3(0,-1,-1)
     &  + 2.0794415416798359d+00*HY5(0,-1,0,-1,-1)
     &  + 4.1588830833596718d+00*HY5(0,0,-1,-1,-1)
      HY6(0,-1,-1,0,1,-1) = 
     &  - 4.2144360800918290d-02
     &  - 2.9909745750253817d-01*tv01
     &  - 4.2444102396722282d-02*tv02
     &  - 4.2524109767437818d-03*tv03
     &  - 3.0020836987190629d-04*tv04
     &  - 1.4628319272059369d-05*tv05
     &  - 4.6678821470807190d-07*tv06
     &  - 8.1726461182421002d-09*tv07
     &  - 1.4167970180936916d-11*tv08
     &  + 1.5619706553671959d-12*tv09
     &  - 2.1121513998847118d-14*tv10
     &  - 4.4967904065779125d-16*tv11
     &  + 5.5465355069057666d-17*tv12
     &  + 5.2075147081105749d-01*HY2(0,-1)
     &  + 5.7009070532142637d-01*HY3(0,-1,-1)
     &  + 6.9314718055994530d-01*HY5(0,-1,-1,0,1)
      HY6(0,-1,-1,0,1,1) = 
     &  + 1.7791896422777663d-02
     &  - 1.1726124919485326d-01*tu01
     &  + 1.7943634572973799d-02*tu02
     &  - 1.8979691403880508d-03*tu03
     &  + 1.5218422126881573d-04*tu04
     &  - 9.4686896528317885d-06*tu05
     &  + 4.4629449646181249d-07*tu06
     &  - 1.4528526501093767d-08*tu07
     &  + 2.2310139461384280d-10*tu08
     &  + 5.0895885520596840d-12*tu09
     &  - 3.2194610267581245d-13*tu10
     &  - 3.9612924564481733d-17*tu11
     &  + 4.4058492944017363d-16*tu12
     &  + 2.0293560632083841d-01*HY2(0,-1)
     &  + 1.5025711289494928d-01*HY3(0,-1,-1)
      HY6(0,-1,-1,1,-1,-1) = 
     &  - 1.1993441817290255d-02
     &  - 8.0916271174440008d-02*tv01
     &  - 1.2130464177541329d-02*tv02
     &  - 1.3471119587946022d-03*tv03
     &  - 1.3756084384765949d-04*tv04
     &  - 1.0690314820441380d-05*tv05
     &  - 5.3852352153979570d-07*tv06
     &  - 1.4636984458437320d-08*tv07
     &  - 3.9890076073876736d-11*tv08
     &  + 8.6289129602006619d-12*tv09
     &  + 3.4840094804318206d-14*tv10
     &  - 1.0182627549621129d-14*tv11
     &  - 3.7863890375265886d-17*tv12
     &  + 1.3986960788271778d-01*HY2(0,-1)
     &  + 1.6651232599446473d-01*HY3(0,-1,-1)
     &  + 2.4022650695910071d-01*HY4(0,-1,-1,1)
      HY6(0,-1,-1,1,-1,1) = 
     &  + 4.1199587016990418d-02
     &  - 3.0316011051549285d-01*tu01
     &  + 4.1450356300128971d-02*tu02
     &  - 3.7688725501205936d-03*tu03
     &  + 2.5124098217655171d-04*tu04
     &  - 1.2665548316901016d-05*tu05
     &  + 4.7178449118911534d-07*tu06
     &  - 1.1483271087255486d-08*tu07
     &  + 8.5312428048588837d-11*tu08
     &  + 5.1695527482245053d-12*tu09
     &  - 1.4052945993362265d-13*tu10
     &  - 3.8290046620783382d-15*tu11
     &  + 2.3291931370625197d-16*tu12
     &  + 5.3075770941318154d-01*HY2(0,-1)
     &  + 5.2253066044918489d-01*HY3(0,-1,-1)
     &  + 4.8045301391820142d-01*HY4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HY5(0,-1,-1,1,-1)
      HY6(0,-1,-1,1,0,1) = 
     &  + 3.0909692357940660d-02
     &  - 2.0530935758598857d-01*tu01
     &  + 3.1154135680202784d-02*tu02
     &  - 3.2111408631810930d-03*tu03
     &  + 2.4503306400209113d-04*tu04
     &  - 1.4091673374984688d-05*tu05
     &  + 5.8985273315854349d-07*tu06
     &  - 1.5696225411667558d-08*tu07
     &  + 1.1074699110966570d-10*tu08
     &  + 9.0373938847945282d-12*tu09
     &  - 2.4575598843353662d-13*tu10
     &  - 7.3687986294613382d-15*tu11
     &  + 4.4396861336048808d-16*tu12
     &  + 3.5590233346968123d-01*HY2(0,-1)
     &  + 2.6957647953152780d-01*HY3(0,-1,-1)
      HY6(0,-1,-1,1,1,-1) = 
     &  + 4.3262006534038570d-02
     &  + 3.1619298448833055d-01*tv01
     &  + 4.3526923137927670d-02*tv02
     &  + 4.0232986545242805d-03*tv03
     &  + 2.6533391147661949d-04*tv04
     &  + 1.2595919806200067d-05*tv05
     &  + 4.1736903153259173d-07*tv06
     &  + 8.6182264641391703d-09*tv07
     &  + 6.1408664560101190d-11*tv08
     &  - 1.7335460593274057d-12*tv09
     &  - 3.5328664104337078d-14*tv10
     &  + 4.9496390392204875d-16*tv11
     &  - 5.5306546869199291d-01*HY2(0,-1)
     &  - 5.7009070532142637d-01*HY3(0,-1,-1)
     &  - 5.8224052646501250d-01*HY4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HY5(0,-1,-1,1,1)
      HY6(0,-1,-1,1,1,1) = 
     &  - 8.2157686982525192d-03
     &  + 5.1586082580938031d-02*tu01
     &  - 8.3148301533015520d-03*tu02
     &  + 1.0149149749056053d-03*tu03
     &  - 9.9492816528928787d-05*tu04
     &  + 7.6227171000532532d-06*tu05
     &  - 4.3164412746927525d-07*tu06
     &  + 1.6304286197847552d-08*tu07
     &  - 2.8220076321709456d-10*tu08
     &  - 6.7113721207816753d-12*tu09
     &  + 4.4614001262544056d-13*tu10
     &  + 9.3171683200104626d-16*tu11
     &  - 6.6900487954988009d-16*tu12
     &  - 8.8326067366010207d-02*HY2(0,-1)
     &  - 5.5504108664821579d-02*HY3(0,-1,-1)
      HY6(0,-1,0,-1,-1,-1) = 
     &  + 1.2449270165673753d-04
     &  + 7.0586950819910646d-05*tv01
     &  + 1.6688135428533209d-04*tv02
     &  + 3.5361978950729430d-05*tv03
     &  + 4.2923474916462123d-05*tv04
     &  + 7.1271613547345740d-06*tv05
     &  + 5.3494416903827874d-07*tv06
     &  + 1.9525895461351290d-08*tv07
     &  + 1.2165477922805900d-10*tv08
     &  - 1.5472009616260443d-11*tv09
     &  - 2.2597665222302593d-13*tv10
     &  + 2.0425466335375329d-14*tv11
     &  + 4.1406363640638342d-16*tv12
     &  - 3.0641479153741016d-17*tv13
      HY6(0,-1,0,-1,-1,1) = 
     &  - 4.2010292609863307d-02
     &  + 3.1805521974328815d-01*tu01
     &  - 4.2244704098291667d-02*tu02
     &  + 3.6936488813135429d-03*tu03
     &  - 2.3481478836938125d-04*tu04
     &  + 1.1262304507319648d-05*tu05
     &  - 4.0340522071781442d-07*tu06
     &  + 9.8770163522146998d-09*tu07
     &  - 1.0516690455963892d-10*tu08
     &  - 2.7217451842460279d-12*tu09
     &  + 1.1265546185826823d-13*tu10
     &  + 8.3798353128983571d-16*tu11
     &  - 1.3653987117165382d-16*tu12
     &  - 5.5823730083320863d-01*HY2(0,-1)
     &  - 2.9112026323250625d-01*HY2(0,-1)*HY2(0,-1)
     &  + 1.1644810529300250d+00*HY4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HY5(0,-1,0,-1,-1)
      HY6(0,-1,0,-1,0,1) = 
     &  - 5.8846788436392714d-02
     &  + 4.4677224584156263d-01*tu01
     &  - 5.9170482246656676d-02*tu02
     &  + 5.1468893437324637d-03*tu03
     &  - 3.2422514793471323d-04*tu04
     &  + 1.5299268990466691d-05*tu05
     &  - 5.3142779823234312d-07*tu06
     &  + 1.2136878864247169d-08*tu07
     &  - 8.9993000009327048d-11*tu08
     &  - 4.6923980332926899d-12*tu09
     &  + 1.3428259176026469d-13*tu10
     &  + 2.7024062223170273d-15*tu11
     &  - 1.9787038293367771d-16*tu12
     &  - 7.8437816209982583d-01*HY2(0,-1)
     &  - 4.1123351671205660d-01*HY2(0,-1)*HY2(0,-1)
     &  + 1.6449340668482264d+00*HY4(0,0,-1,-1)
      HY6(0,-1,0,-1,1,-1) = 
     &  - 3.5525626510720466d-02
     &  - 2.5164912191938246d-01*tv01
     &  - 3.5791447893538771d-02*tv02
     &  - 3.5983664669075185d-03*tv03
     &  - 2.6633457317902309d-04*tv04
     &  - 1.4224431116121248d-05*tv05
     &  - 5.1317896672420810d-07*tv06
     &  - 1.0157187310192017d-08*tv07
     &  + 1.1359955610992378d-11*tv08
     &  + 4.6088222374621577d-12*tv09
     &  - 3.3983701622541189d-14*tv10
     &  - 5.1605817845495944d-15*tv11
     &  + 5.3659527985513966d-17*tv12
     &  + 4.3804558824071468d-01*HY2(0,-1)
     &  + 2.4022650695910071d-01*HY2(0,-1)*HY2(0,-1)
     &  - 9.6090602783640284d-01*HY4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HY5(0,-1,0,-1,1)
      HY6(0,-1,0,-1,1,1) = 
     &  + 2.3317200664406615d-02
     &  - 1.6152032427829515d-01*tu01
     &  + 2.3491632569370747d-02*tu02
     &  - 2.3342580977868037d-03*tu03
     &  + 1.7488207310387252d-04*tu04
     &  - 1.0176430734830099d-05*tu05
     &  + 4.5036007062613579d-07*tu06
     &  - 1.3770215545093324d-08*tu07
     &  + 1.9165954499954399d-10*tu08
     &  + 4.9538066012358553d-12*tu09
     &  - 2.7096018508357615d-13*tu10
     &  - 1.0746901717607132d-15*tu11
     &  + 3.7992015050754655d-16*tu12
     &  + 2.8103333764091645d-01*HY2(0,-1)
     &  + 1.2011325347955035d-01*HY2(0,-1)*HY2(0,-1)
     &  - 4.8045301391820142d-01*HY4(0,0,-1,-1)
      HY6(0,-1,0,1,-1,-1) = 
     &  - 1.7055594369621343d-02
     &  - 1.1445624643551323d-01*tv01
     &  - 1.7223436762964441d-02*tv02
     &  - 1.9473164053705603d-03*tv03
     &  - 1.6832859841004815d-04*tv04
     &  - 1.0781714147562448d-05*tv05
     &  - 4.8632360724242956d-07*tv06
     &  - 1.3620522331003869d-08*tv07
     &  - 1.1838553654452537d-10*tv08
     &  + 6.1950680265215534d-12*tv09
     &  + 1.5453148659283801d-13*tv10
     &  - 5.3281094738247417d-15*tv11
     &  - 2.2413128815883209d-16*tv12
     &  + 1.9757838252848865d-01*HY2(0,-1)
     &  + 2.4022650695910071d-01*HY4(0,-1,0,1)
      HY6(0,-1,0,1,-1,1) = 
     &  + 4.0340055800306674d-02
     &  - 2.9120852413069066d-01*tu01
     &  + 4.0597191719822186d-02*tu02
     &  - 3.7771671267046796d-03*tu03
     &  + 2.5762850774197426d-04*tu04
     &  - 1.3208159364874940d-05*tu05
     &  + 4.9264403292300714d-07*tu06
     &  - 1.1543142123228317d-08*tu07
     &  + 5.5696562535788366d-11*tu08
     &  + 6.0276201496232974d-12*tu09
     &  - 1.0968117919827094d-13*tu10
     &  - 5.9368271918395365d-15*tu11
     &  + 2.1600796929743529d-16*tu12
     &  + 5.0898724125653122d-01*HY2(0,-1)
     &  + 4.8045301391820142d-01*HY4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HY5(0,-1,0,1,-1)
      HY6(0,-1,0,1,0,1) = 
     &  + 1.9114955269900981d-02
     &  - 9.6847710313156972d-02*tu01
     &  + 1.9337275457258233d-02*tu02
     &  - 2.4862521028325666d-03*tu03
     &  + 2.2294910660707224d-04*tu04
     &  - 1.4245799124097780d-05*tu05
     &  + 6.2898628839284835d-07*tu06
     &  - 1.6547030686247233d-08*tu07
     &  + 6.6826518591952627d-11*tu08
     &  + 1.1372314245188055d-11*tu09
     &  - 2.1159912093080772d-13*tu10
     &  - 1.2083587117699945d-14*tu11
     &  + 4.5335119184896729d-16*tu12
     &  + 1.6265466739742008d-01*HY2(0,-1)
      HY6(0,-1,0,1,1,-1) = 
     &  + 4.4809089426849548d-02
     &  + 3.3250709594269789d-01*tv01
     &  + 4.5058936058063097d-02*tv02
     &  + 3.9674856119891713d-03*tv03
     &  + 2.5024932207373212d-04*tv04
     &  + 1.1717771608689355d-05*tv05
     &  + 4.0278007562946889d-07*tv06
     &  + 9.2796428103243699d-09*tv07
     &  + 8.9137299707539164d-11*tv08
     &  - 2.2791679049103219d-12*tv09
     &  - 7.8063027796295966d-14*tv10
     &  + 9.3858857497264539d-16*tv11
     &  + 8.3887731789387379d-17*tv12
     &  - 5.8302393270318421d-01*HY2(0,-1)
     &  - 5.8224052646501250d-01*HY4(0,-1,0,1)
     &  + 6.9314718055994530d-01*HY5(0,-1,0,1,1)
      HY6(0,-1,0,1,1,1) = 
     &  - 2.9193675811928750d-03
     &  + 1.4698944352369860d-02*tu01
     &  - 2.9951046628304493d-03*tu02
     &  + 5.5692449892478336d-04*tu03
     &  - 7.6188021150252938d-05*tu04
     &  + 7.1509360776589500d-06*tu05
     &  - 4.5122160430887968d-07*tu06
     &  + 1.7710560027261798d-08*tu07
     &  - 2.8159486303226207d-10*tu08
     &  - 9.3960900494044862d-12*tu09
     &  + 4.9593291854780854d-13*tu10
     &  + 5.3335470906748790d-15*tu11
     &  - 8.3287940006997419d-16*tu12
     &  - 2.3752366322618485d-02*HY2(0,-1)
      HY6(0,-1,1,-1,-1,-1) = 
     &  - 3.7766595687872900d-03
     &  - 2.2716791050870549d-02*tv01
     &  - 3.8518715319618466d-03*tv02
     &  - 5.3184038338677655d-04*tv03
     &  - 7.5705693686818158d-05*tv04
     &  - 7.7474637298868374d-06*tv05
     &  - 4.9390092551569811d-07*tv06
     &  - 1.7582973746952598d-08*tv07
     &  - 1.7013439776924935d-10*tv08
     &  + 1.1291746485262810d-11*tv09
     &  + 2.7839649356697850d-13*tv10
     &  - 1.2259759439865939d-14*tv11
     &  - 4.5909056143483948d-16*tv12
     &  + 3.8472516430513908d-02*HY2(0,-1)
     &  + 5.5504108664821579d-02*HY3(0,-1,1)
      HY6(0,-1,1,-1,-1,1) = 
     &  + 4.3239107331474553d-02
     &  - 3.2209539106611691d-01*tu01
     &  + 4.3487198580425984d-02*tu02
     &  - 3.8642063613968709d-03*tu03
     &  + 2.4850976497747861d-04*tu04
     &  - 1.1924945945136055d-05*tu05
     &  + 4.1859587520614688d-07*tu06
     &  - 9.6389406791330185d-09*tu07
     &  + 7.9768832777890164d-11*tu08
     &  + 2.9655387645712579d-12*tu09
     &  - 8.0224666132483367d-14*tu10
     &  - 1.7950613355659708d-15*tu11
     &  + 1.0147674898247373d-16*tu12
     &  + 5.6465878038610476d-01*HY2(0,-1)
     &  + 5.7009070532142637d-01*HY3(0,-1,1)
     &  - 5.8224052646501250d-01*HY4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HY5(0,-1,1,-1,-1)
      HY6(0,-1,1,-1,0,1) = 
     &  + 5.1222217940871043d-02
     &  - 3.6256522138921932d-01*tu01
     &  + 5.1547121042322470d-02*tu02
     &  - 4.8303440630012819d-03*tu03
     &  + 3.2546691990252647d-04*tu04
     &  - 1.6095073043816889d-05*tu05
     &  + 5.6385871646186571d-07*tu06
     &  - 1.1993289312965272d-08*tu07
     &  + 4.0197001377788294d-11*tu08
     &  + 5.4054682122783100d-12*tu09
     &  - 6.8241265732622511d-14*tu10
     &  - 4.8082284365046631d-15*tu11
     &  + 1.1894333998514281d-16*tu12
     &  + 6.3300832829764945d-01*HY2(0,-1)
     &  + 5.7009070532142637d-01*HY3(0,-1,1)
     &  - 8.2246703342411321d-01*HY4(0,-1,1,-1)
      HY6(0,-1,1,-1,1,-1) = 
     &  + 3.9201630228664789d-02
     &  + 2.8264279714168323d-01*tv01
     &  + 3.9462848255190595d-02*tv02
     &  + 3.7531885980346676d-03*tv03
     &  + 2.6170417506915686d-04*tv04
     &  + 1.3451490417960483d-05*tv05
     &  + 4.8618370560297407d-07*tv06
     &  + 1.0504174032801197d-08*tv07
     &  + 3.5124380760343985d-11*tv08
     &  - 4.4416302534538277d-12*tv09
     &  - 3.7816461814279352d-14*tv10
     &  + 4.1338128513427904d-15*tv11
     &  + 5.4996851101388859d-17*tv12
     &  - 4.9354620950987381d-01*HY2(0,-1)
     &  - 5.2253066044918489d-01*HY3(0,-1,1)
     &  + 4.8045301391820142d-01*HY4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HY5(0,-1,1,-1,1)
      HY6(0,-1,1,-1,1,1) = 
     &  - 1.7348148275856081d-02
     &  + 1.1794850098256868d-01*tu01
     &  - 1.7502653757017768d-02*tu02
     &  + 1.8649213614630388d-03*tu03
     &  - 1.5498317861700767d-04*tu04
     &  + 1.0016626891378735d-05*tu05
     &  - 4.7786538296942145d-07*tu06
     &  + 1.4822335615769576d-08*tu07
     &  - 1.6766062914827150d-10*tu08
     &  - 7.3050366817289366d-12*tu09
     &  + 2.6658975773028605d-13*tu10
     &  + 5.4545849506407993d-15*tu11
     &  - 4.2910818955819329d-16*tu12
     &  - 2.0437039310996688d-01*HY2(0,-1)
     &  - 1.6651232599446473d-01*HY3(0,-1,1)
     &  + 2.4022650695910071d-01*HY4(0,-1,1,-1)
      HY6(0,-1,1,0,1,-1) = 
     &  + 4.8334961812599160d-02
     &  + 3.5651646353472660d-01*tv01
     &  + 4.8605560552604308d-02*tv02
     &  + 4.2945618240050944d-03*tv03
     &  + 2.7102970449726241d-04*tv04
     &  + 1.2652114062467079d-05*tv05
     &  + 4.3104598975524217d-07*tv06
     &  + 9.6960346296579569d-09*tv07
     &  + 8.1414061742643826d-11*tv08
     &  - 2.8297081040570044d-12*tv09
     &  - 8.3477321195768870d-14*tv10
     &  + 1.4478648909933284d-15*tv11
     &  + 1.0197366651684088d-16*tv12
     &  - 6.2490176497326899d-01*HY2(0,-1)
     &  - 6.3196619783816790d-01*HY3(0,-1,1)
     &  + 6.9314718055994530d-01*HY5(0,-1,1,0,1)
      HY6(0,-1,1,0,1,1) = 
     &  - 4.0279082655711526d-03
     &  + 2.0344365957710814d-02*tu01
     &  - 4.1276056590464465d-03*tu02
     &  + 7.6557463410708961d-04*tu03
     &  - 1.0019880368313007d-04*tu04
     &  + 8.7471428788604834d-06*tu05
     &  - 5.0159637980792294d-07*tu06
     &  + 1.7188238097607764d-08*tu07
     &  - 1.8582595970620240d-10*tu08
     &  - 1.0568187350589593d-11*tu09
     &  + 3.4539710992588237d-13*tu10
     &  + 9.7770262387602311d-15*tu11
     &  - 6.1383518233850513d-16*tu12
     &  - 3.2893195194356041d-02*HY2(0,-1)
      HY6(0,-1,1,1,-1,-1) = 
     &  + 2.2485681498643441d-02
     &  + 1.5634832891844655d-01*tv01
     &  + 2.2663749328350437d-02*tv02
     &  + 2.3091169157254260d-03*tv03
     &  + 1.7852657393075781d-04*tv04
     &  + 1.0546225386746860d-05*tv05
     &  + 4.5889652995429782d-07*tv06
     &  + 1.3182979350378483d-08*tv07
     &  + 1.5211705621409104d-10*tv08
     &  - 4.8434107737345052d-12*tv09
     &  - 1.8888859405025506d-13*tv10
     &  + 2.5440556514740028d-15*tv11
     &  + 2.4730764835364494d-16*tv12
     &  - 2.7177021267950885d-01*HY2(0,-1)
     &  - 3.0882537509683393d-01*HY3(0,-1,1)
     &  + 2.4022650695910071d-01*HY4(0,-1,1,1)
      HY6(0,-1,1,1,-1,1) = 
     &  - 3.5120173371331632d-02
     &  + 2.5089184077607053d-01*tu01
     &  - 3.5369328343640433d-02*tu02
     &  + 3.4340830240244978d-03*tu03
     &  - 2.4968242042590878d-04*tu04
     &  + 1.3640707452714957d-05*tu05
     &  - 5.2745162664686725d-07*tu06
     &  + 1.1855407005189550d-08*tu07
     &  - 3.4710744723900746d-12*tu08
     &  - 7.4708031493368147d-12*tu09
     &  + 3.8339806149995631d-14*tu10
     &  + 9.3774638797398349d-15*tu11
     &  - 1.2515698744117859d-16*tu12
     &  - 4.3755947670194733d-01*HY2(0,-1)
     &  - 4.0357837932696163d-01*HY3(0,-1,1)
     &  + 4.8045301391820142d-01*HY4(0,-1,1,1)
     &  - 6.9314718055994530d-01*HY5(0,-1,1,1,-1)
      HY6(0,-1,1,1,0,1) = 
     &  - 7.3819014381119210d-03
     &  + 3.7418314820067394d-02*tu01
     &  - 7.5547098808744740d-03*tu02
     &  + 1.3976875735991845d-03*tu03
     &  - 1.7349859618743737d-04*tu04
     &  + 1.3813611588539062d-05*tu05
     &  - 6.9014366280454515d-07*tu06
     &  + 1.8395986505863408d-08*tu07
     &  + 9.8831308467840250d-12*tu08
     &  - 1.5613715853135295d-11*tu09
     &  + 1.2069010614645503d-13*tu10
     &  + 2.0596825140991531d-14*tu11
     &  - 3.6013554907428846d-16*tu12
     &  - 2.8896907082706205d-17*tu13
     &  - 6.0534893171921111d-02*HY2(0,-1)
      HY6(0,-1,1,1,1,-1) = 
     &  - 4.1836104891717346d-02
     &  - 3.1390844391224177d-01*tv01
     &  - 4.2069764617256106d-02*tv02
     &  - 3.6911908264528380d-03*tv03
     &  - 2.3405338837531550d-04*tv04
     &  - 1.1131897442771459d-05*tv05
     &  - 3.9376778515342256d-07*tv06
     &  - 9.5354122062912899d-09*tv07
     &  - 1.0485168205297918d-10*tv08
     &  + 2.2188496151827449d-12*tv09
     &  + 9.6813845819915285d-14*tv10
     &  - 4.7264515463858939d-16*tv11
     &  - 1.0161528389800398d-16*tv12
     &  + 5.5070993482224879d-01*HY2(0,-1)
     &  + 5.3721319360804020d-01*HY3(0,-1,1)
     &  - 5.8224052646501250d-01*HY4(0,-1,1,1)
     &  + 6.9314718055994530d-01*HY5(0,-1,1,1,1)
      HY6(0,-1,1,1,1,1) = 
     &  + 1.2332916013766885d-03
     &  - 5.9850373415782699d-03*tu01
     &  + 1.2814903614477569d-03*tu02
     &  - 2.4203363351148856d-04*tu03
     &  + 4.8671392022545211d-05*tu04
     &  - 6.2349948765925717d-06*tu05
     &  + 4.7291128894336421d-07*tu06
     &  - 2.0039856946713978d-08*tu07
     &  + 2.7879291355779837d-10*tu08
     &  + 1.4119534878270229d-11*tu09
     &  - 5.4355361965349181d-13*tu10
     &  - 1.4226008175514292d-14*tu11
     &  + 9.9762619738606041d-16*tu12
     &  + 9.6181291076284771d-03*HY2(0,-1)
      HY6(0,0,-1,-1,-1,-1) = 
     &  + 9.4458440817220657d-05
     &  + 5.9733423813960670d-05*tv01
     &  + 1.2679481544929151d-04*tv02
     &  + 2.9942033962824793d-05*tv03
     &  + 3.2846810008647230d-05*tv04
     &  + 6.0486198258927182d-06*tv05
     &  + 5.1064820050070357d-07*tv06
     &  + 2.1464872630035616d-08*tv07
     &  + 2.1237651997185427d-10*tv08
     &  - 1.8518924274204752d-11*tv09
     &  - 4.4651729322565372d-13*tv10
     &  + 2.5066373701303167d-14*tv11
     &  + 8.8530625088387685d-16*tv12
     &  - 3.7702710011744510d-17*tv13
      HY6(0,0,-1,-1,-1,1) = 
     &  - 4.0452685957824271d-02
     &  + 3.0609430833844765d-01*tu01
     &  - 4.0679087417221787d-02*tu02
     &  + 3.5605238134951700d-03*tu03
     &  - 2.2679525215032772d-04*tu04
     &  + 1.0917652622282148d-05*tu05
     &  - 3.9390526708597590d-07*tu06
     &  + 9.8078523507829841d-09*tu07
     &  - 1.1240165222259890d-10*tu08
     &  - 2.4089457746957181d-12*tu09
     &  + 1.1249644762579880d-13*tu10
     &  + 4.0862505705308266d-16*tu11
     &  - 1.2469056845897424d-16*tu12
     &  - 5.3721319360804020d-01*HY3(0,0,-1)
     &  - 5.8224052646501250d-01*HY4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HY5(0,0,-1,-1,-1)
      HY6(0,0,-1,-1,0,-1) = 
     &  + 4.6911420419152552d-04
     &  + 2.4547685317693898d-03*tv01
     &  + 6.2672063116386910d-04*tv02
     &  + 8.4232036930638332d-04*tv03
     &  + 1.5834726962649909d-04*tv04
     &  + 1.4463842593710629d-05*tv05
     &  + 7.4071943447722856d-07*tv06
     &  + 1.8069631774720521d-08*tv07
     &  - 1.2311785146282673d-10*tv08
     &  - 1.6569064533344789d-11*tv09
     &  + 1.0177780251227766d-13*tv10
     &  + 2.4932258502277246d-14*tv11
     &  - 4.9103652809402367d-17*tv12
     &  - 4.1022828420193651d-17*tv13
      HY6(0,0,-1,-1,0,1) = 
     &  - 5.6401439498409945d-02
     &  + 4.2794977116954855d-01*tu01
     &  - 5.6712716054129060d-02*tu02
     &  + 4.9388313912802353d-03*tu03
     &  - 3.1179396611628488d-04*tu04
     &  + 1.4773599198927616d-05*tu05
     &  - 5.1751267253936611d-07*tu06
     &  + 1.2070357273343092d-08*tu07
     &  - 1.0213868955719863d-10*tu08
     &  - 4.2299122614409848d-12*tu09
     &  + 1.3649762961212977d-13*tu10
     &  + 1.9851508973205468d-15*tu11
     &  - 1.8213737729563691d-16*tu12
     &  - 7.5128556447474642d-01*HY3(0,0,-1)
     &  - 8.2246703342411321d-01*HY4(0,0,-1,-1)
      HY6(0,0,-1,-1,1,-1) = 
     &  - 3.2723797747217469d-02
     &  - 2.3183749689928543d-01*tv01
     &  - 3.2974628551142358d-02*tv02
     &  - 3.3133268538491978d-03*tv03
     &  - 2.5137487400465379d-04*tv04
     &  - 1.4134260356973640d-05*tv05
     &  - 5.4402573327825782d-07*tv06
     &  - 1.1277890690228467d-08*tv07
     &  + 4.4282677058228674d-11*tv08
     &  + 7.1497967605294778d-12*tv09
     &  - 6.3738690773975166d-14*tv10
     &  - 1.0053874056108859d-14*tv11
     &  + 7.0833916334917349d-17*tv12
     &  + 4.0357837932696163d-01*HY3(0,0,-1)
     &  + 4.8045301391820142d-01*HY4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HY5(0,0,-1,-1,1)
      HY6(0,0,-1,-1,1,1) = 
     &  + 2.5190087238800164d-02
     &  - 1.7721048962953744d-01*tu01
     &  + 2.5370232020970914d-02*tu02
     &  - 2.4692284021811593d-03*tu03
     &  + 1.8058773222096158d-04*tu04
     &  - 1.0249002771696678d-05*tu05
     &  + 4.4313546769722054d-07*tu06
     &  - 1.3299167663733221d-08*tu07
     &  + 1.8517460280676596d-10*tu08
     &  + 4.3790140820551570d-12*tu09
     &  - 2.4255947098474984d-13*tu10
     &  - 8.2365860199211587d-16*tu11
     &  + 3.2237103732361896d-16*tu12
     &  + 3.0882537509683393d-01*HY3(0,0,-1)
     &  + 2.4022650695910071d-01*HY4(0,0,-1,-1)
      HY6(0,0,-1,0,-1,-1) = 
     &  + 2.5854340793219615d-04
     &  + 1.2441789444892364d-03*tv01
     &  + 3.4561702094830907d-04*tv02
     &  + 4.2958773683607680d-04*tv03
     &  + 8.7609270364073388d-05*tv04
     &  + 8.9415913336270322d-06*tv05
     &  + 5.3577926484060390d-07*tv06
     &  + 1.7653693463441633d-08*tv07
     &  + 1.2167240946199801d-10*tv08
     &  - 1.2735778153490247d-11*tv09
     &  - 2.4399336672181442d-13*tv10
     &  + 1.5691672965415483d-14*tv11
     &  + 4.7637560589341893d-16*tv12
     &  - 2.1909154334038158d-17*tv13
      HY6(0,0,-1,0,-1,1) = 
     &  - 4.7416988639529789d-02
     &  + 3.5996520802031370d-01*tu01
     &  - 4.7678125380896056d-02*tu02
     &  + 4.1485308852884251d-03*tu03
     &  - 2.6156933491439051d-04*tu04
     &  + 1.2372839070929941d-05*tu05
     &  - 4.3268287838505531d-07*tu06
     &  + 1.0102814418987412d-08*tu07
     &  - 8.9226879623320354d-11*tu08
     &  - 3.2323820216718220d-12*tu09
     &  + 1.0324486184941271d-13*tu10
     &  + 1.6684965010857455d-15*tu11
     &  - 1.3642644392081747d-16*tu12
     &  - 6.3196619783816790d-01*HY3(0,0,-1)
     &  - 6.9314718055994530d-01*HY5(0,0,-1,0,-1)
      HY6(0,0,-1,0,0,1) = 
     &  - 6.7411083258082625d-02
     &  + 5.1335430775790982d-01*tu01
     &  - 6.7776499737718149d-02*tu02
     &  + 5.8635885703955136d-03*tu03
     &  - 3.6599104950363248d-04*tu04
     &  + 1.7003449958734229d-05*tu05
     &  - 5.7463377858630358d-07*tu06
     &  + 1.2384941633203120d-08*tu07
     &  - 6.3795243173622875d-11*tu08
     &  - 5.3943733524647459d-12*tu09
     &  + 1.1504599994432199d-13*tu10
     &  + 3.9533455688532856d-15*tu11
     &  - 1.8869595085964143d-16*tu12
     &  - 9.0154267736969571d-01*HY3(0,0,-1)
      HY6(0,0,-1,0,1,-1) = 
     &  - 4.6305125549362313d-02
     &  - 3.2654154014228348d-01*tv01
     &  - 4.6594400393751786d-02*tv02
     &  - 4.3538158977015855d-03*tv03
     &  - 2.8974388877097068d-04*tv04
     &  - 1.3972110981977872d-05*tv05
     &  - 4.6906768668041542d-07*tv06
     &  - 9.2924994471028445d-09*tv07
     &  - 2.3299186333120360d-11*tv08
     &  + 3.3081423110721818d-12*tv09
     &  + 5.9949995632896400d-15*tv10
     &  - 3.1494749496819744d-15*tv11
     &  + 5.7009070532142637d-01*HY3(0,0,-1)
     &  + 6.9314718055994530d-01*HY5(0,0,-1,0,1)
      HY6(0,0,-1,0,1,1) = 
     &  + 1.4645142281922033d-02
     &  - 8.7745126076699476d-02*tu01
     &  + 1.4792962436426946d-02*tu02
     &  - 1.7173130894490858d-03*tu03
     &  + 1.4828260624690394d-04*tu04
     &  - 9.6656586010184061d-06*tu05
     &  + 4.6264761654475372d-07*tu06
     &  - 1.4707623062718682d-08*tu07
     &  + 1.9558123647253525d-10*tu08
     &  + 6.1935262836963931d-12*tu09
     &  - 2.9287756927912258d-13*tu10
     &  - 2.9137849919172175d-15*tu11
     &  + 4.3898774313788532d-16*tu12
     &  + 1.5025711289494928d-01*HY3(0,0,-1)
      HY6(0,0,-1,1,-1,-1) = 
     &  - 1.4430812987402793d-02
     &  - 9.6510972140131299d-02*tv01
     &  - 1.4579865251223773d-02*tv02
     &  - 1.6601257062483388d-03*tv03
     &  - 1.4955187698818268d-04*tv04
     &  - 1.0257820991032712d-05*tv05
     &  - 4.9973785067414342d-07*tv06
     &  - 1.4964665747875482d-08*tv07
     &  - 1.2448416356232245d-10*tv08
     &  + 8.5140500522944793d-12*tv09
     &  + 1.9896494631553405d-13*tv10
     &  - 9.0007759948836692d-15*tv11
     &  - 3.4150232729805004d-16*tv12
     &  + 1.6651232599446473d-01*HY3(0,0,-1)
     &  + 2.4022650695910071d-01*HY4(0,0,-1,1)
      HY6(0,0,-1,1,-1,1) = 
     &  + 4.1077534699591344d-02
     &  - 2.9874752932091605d-01*tu01
     &  + 4.1333539254414841d-02*tu02
     &  - 3.8068231197344039d-03*tu03
     &  + 2.5648277692327878d-04*tu04
     &  - 1.2976311566079180d-05*tu05
     &  + 4.7828546131993574d-07*tu06
     &  - 1.1175101719330100d-08*tu07
     &  + 6.3262526792445076d-11*tu08
     &  + 5.1918990766204080d-12*tu09
     &  - 9.8840737144910194d-14*tu10
     &  - 4.7596752590342989d-15*tu11
     &  + 1.7028625094355313d-16*tu12
     &  + 5.2253066044918489d-01*HY3(0,0,-1)
     &  + 4.8045301391820142d-01*HY4(0,0,-1,1)
     &  - 6.9314718055994530d-01*HY5(0,0,-1,1,-1)
      HY6(0,0,-1,1,0,-1) = 
     &  - 4.8882388671276517d-02
     &  - 3.2793171320050594d-01*tv01
     &  - 4.9215933604069848d-02*tv02
     &  - 4.8216214790070867d-03*tv03
     &  - 3.3411408642804686d-04*tv04
     &  - 1.6623583060050235d-05*tv05
     &  - 5.6916761487527928d-07*tv06
     &  - 1.1244349212509453d-08*tv07
     &  - 1.3970762352726373d-11*tv08
     &  + 4.6120768853119196d-12*tv09
     &  + 9.3882572004001882d-15*tv10
     &  - 4.2890498723462224d-15*tv11
     &  + 5.7009070532142637d-01*HY3(0,0,-1)
     &  + 8.2246703342411321d-01*HY4(0,0,-1,1)
      HY6(0,0,-1,1,0,1) = 
     &  + 2.6210034636960371d-02
     &  - 1.5715821162767584d-01*tu01
     &  + 2.6453262510399113d-02*tu02
     &  - 2.9879279167606446d-03*tu03
     &  + 2.4383807583104342d-04*tu04
     &  - 1.4548800025992035d-05*tu05
     &  + 6.1026616405116812d-07*tu06
     &  - 1.5435507227858927d-08*tu07
     &  + 6.3606744492977608d-11*tu08
     &  + 9.6136542855871061d-12*tu09
     &  - 1.6467596924812002d-13*tu10
     &  - 9.9952075912097958d-15*tu11
     &  + 3.2835895533228659d-16*tu12
     &  + 2.6957647953152780d-01*HY3(0,0,-1)
      HY6(0,0,-1,1,1,-1) = 
     &  + 4.4107486356711117d-02
     &  + 3.2540475855587874d-01*tv01
     &  + 4.4363075152372398d-02*tv02
     &  + 3.9719370824670728d-03*tv03
     &  + 2.5600832620753337d-04*tv04
     &  + 1.2201094081892413d-05*tv05
     &  + 4.1959855231504334d-07*tv06
     &  + 9.2611997934815624d-09*tv07
     &  + 6.7951987358771098d-11*tv08
     &  - 2.6316689599715227d-12*tv09
     &  - 5.4018870477337953d-14*tv10
     &  + 1.7165818659732139d-15*tv11
     &  + 5.6454314699235996d-17*tv12
     &  - 5.7009070532142637d-01*HY3(0,0,-1)
     &  - 5.8224052646501250d-01*HY4(0,0,-1,1)
     &  + 6.9314718055994530d-01*HY5(0,0,-1,1,1)
      HY6(0,0,-1,1,1,1) = 
     &  - 5.5158458297047641d-03
     &  + 3.2880017385017115d-02*tu01
     &  - 5.6062587248586548d-03*tu02
     &  + 7.9701147766316489d-04*tu03
     &  - 9.0866371823227041d-05*tu04
     &  + 7.6790617833682920d-06*tu05
     &  - 4.5372689416192977d-07*tu06
     &  + 1.6916507767853580d-08*tu07
     &  - 2.4980406131405385d-10*tu08
     &  - 8.8524022293874127d-12*tu09
     &  + 4.2006548759256983d-13*tu10
     &  + 6.0525576527236310d-15*tu11
     &  - 6.9758526583813641d-16*tu12
     &  - 5.5504108664821579d-02*HY3(0,0,-1)
      HY6(0,0,0,-1,-1,-1) = 
     &  + 1.8847669940575295d-04
     &  + 8.4126791588380595d-04*tv01
     &  + 2.5210127116254848d-04*tv02
     &  + 2.9232323577698764d-04*tv03
     &  + 6.4104307755616893d-05*tv04
     &  + 7.1663485706204491d-06*tv05
     &  + 4.7997315257085651d-07*tv06
     &  + 1.8548710179392192d-08*tv07
     &  + 2.3670440986757158d-10*tv08
     &  - 1.2255353357019150d-11*tv09
     &  - 4.4850782754286274d-13*tv10
     &  + 1.2394849039063976d-14*tv11
     &  + 8.3023390394627707d-16*tv12
      HY6(0,0,0,-1,-1,1) = 
     &  - 4.3705380046193808d-02
     &  + 3.3165601345710473d-01*tu01
     &  - 4.3946671590499026d-02*tu02
     &  + 3.8270050959453154d-03*tu03
     &  - 2.4169566618324511d-04*tu04
     &  + 1.1471768622678529d-05*tu05
     &  - 4.0421793304218198d-07*tu06
     &  + 9.6332838329296186d-09*tu07
     &  - 9.5959845576495773d-11*tu08
     &  - 2.6251120926236781d-12*tu09
     &  + 9.5056026536867187d-14*tu10
     &  + 1.1329347737523036d-15*tu11
     &  - 1.1333119951778950d-16*tu12
     &  - 5.8224052646501250d-01*HY4(0,0,0,-1)
     &  - 6.9314718055994530d-01*HY5(0,0,0,-1,-1)
      HY6(0,0,0,-1,0,-1) = 
     &  + 1.0098904839255951d-02
     &  + 6.0213084976727037d-03*tv01
     &  + 1.0315264173626980d-02*tv02
     &  + 2.0318456094850498d-03*tv03
     &  + 2.1701896405374517d-04*tv04
     &  + 1.4868285048755715d-05*tv05
     &  + 6.5962736705981183d-07*tv06
     &  + 1.6140973266987947d-08*tv07
     &  - 2.4004698749062361d-12*tv08
     &  - 1.1942305690452025d-11*tv09
     &  - 8.4566606220964458d-14*tv10
     &  + 1.5220952845506720d-14*tv11
     &  + 2.4619508313967382d-16*tv12
     &  - 2.1668085109382145d-17*tv13
      HY6(0,0,0,-1,0,1) = 
     &  - 6.1520856091010448d-02
     &  + 4.6834432772014678d-01*tu01
     &  - 6.1855084306702402d-02*tu02
     &  + 5.3551322615323719d-03*tu03
     &  - 3.3475965049626334d-04*tu04
     &  + 1.5603924683568053d-05*tu05
     &  - 5.3151179922217337d-07*tu06
     &  + 1.1733667313331215d-08*tu07
     &  - 7.6888894957534870d-11*tu08
     &  - 4.4445616540516046d-12*tu09
     &  + 1.0586246603160339d-13*tu10
     &  + 2.9968964392028113d-15*tu11
     &  - 1.5482309763158132d-16*tu12
     &  - 8.2246703342411321d-01*HY4(0,0,0,-1)
      HY6(0,0,0,-1,1,-1) = 
     &  - 3.9127986670590087d-02
     &  - 2.7547134000945478d-01*tv01
     &  - 3.9389856771184318d-02*tv02
     &  - 3.7633511283218531d-03*tv03
     &  - 2.6237160854135942d-04*tv04
     &  - 1.3589797917156890d-05*tv05
     &  - 5.0153458799695620d-07*tv06
     &  - 1.1129373560864273d-08*tv07
     &  - 2.6590147788068245d-11*tv08
     &  + 5.9304320446387360d-12*tv09
     &  + 5.0604855333808554d-14*tv10
     &  - 6.8443705956786150d-15*tv11
     &  - 1.1491268796237608d-16*tv12
     &  + 4.8045301391820142d-01*HY4(0,0,0,-1)
     &  + 6.9314718055994530d-01*HY5(0,0,0,-1,1)
      HY6(0,0,0,-1,1,1) = 
     &  + 2.0953791384691381d-02
     &  - 1.3869732294064545d-01*tu01
     &  + 2.1126488033291559d-02*tu02
     &  - 2.2082101411605803d-03*tu03
     &  + 1.7316075079805566d-04*tu04
     &  - 1.0396642259343588d-05*tu05
     &  + 4.6426796180610744d-07*tu06
     &  - 1.3835532700962811d-08*tu07
     &  + 1.6552514903251994d-10*tu08
     &  + 5.7986327224472520d-12*tu09
     &  - 2.3841705874490641d-13*tu10
     &  - 3.4934413943568105d-15*tu11
     &  + 3.6145250212512102d-16*tu12
     &  + 2.4022650695910071d-01*HY4(0,0,0,-1)
      HY6(0,0,0,0,-1,-1) = 
     &  + 5.1007139972048853d-03
     &  + 3.2244058635112390d-03*tv01
     &  + 5.2262508342031766d-03*tv02
     &  + 1.0907312804623211d-03*tv03
     &  + 1.2602887789169060d-04*tv04
     &  + 9.5795436969002532d-06*tv05
     &  + 4.9221061916471793d-07*tv06
     &  + 1.5666412139313395d-08*tv07
     &  + 1.6943785245493956d-10*tv08
     &  - 8.4051195817590671d-12*tv09
     &  - 2.8741998867185150d-13*tv10
     &  + 7.0596958474906422d-15*tv11
     &  + 4.9209256765554336d-16*tv12
      HY6(0,0,0,0,-1,1) = 
     &  - 5.1808654530487943d-02
     &  + 3.9467925471066604d-01*tu01
     &  - 5.2089313732600333d-02*tu02
     &  + 4.5045319277492117d-03*tu03
     &  - 2.8110321314130566d-04*tu04
     &  + 1.3071808211252428d-05*tu05
     &  - 4.4407933465329014d-07*tu06
     &  + 9.8043016260102671d-09*tu07
     &  - 6.8228767174150247d-11*tu08
     &  - 3.3740378692890819d-12*tu09
     &  + 7.6859825221552282d-14*tu10
     &  + 2.4546540520830359d-15*tu11
     &  - 1.1057592409144857d-16*tu12
     &  - 6.9314718055994530d-01*HY5(0,0,0,0,-1)
      HY6(0,0,0,0,0,-1) = 
     &  + 7.4446866292830904d-02
     &  + 5.6919859582115312d-01*tv01
     &  + 7.4842898728615918d-02*tv02
     &  + 6.4296167807324956d-03*tv03
     &  + 3.9662243295612648d-04*tv04
     &  + 1.8067483426713898d-05*tv05
     &  + 5.9002869993575420d-07*tv06
     &  + 1.1835276483272331d-08*tv07
     &  + 3.1459026440220561d-11*tv08
     &  - 5.4554143321149829d-12*tv09
     &  - 6.9660157399524824d-14*tv10
     &  + 5.0028186325156076d-15*tv11
     &  + 1.3596721085576870d-16*tv12
      HY6(0,0,0,0,0,1) = 
     &  - 7.4446866292830904d-02
     &  + 5.6919859582115312d-01*tu01
     &  - 7.4842898728615918d-02*tu02
     &  + 6.4296167807324956d-03*tu03
     &  - 3.9662243295612648d-04*tu04
     &  + 1.8067483426713898d-05*tu05
     &  - 5.9002869993575420d-07*tu06
     &  + 1.1835276483272331d-08*tu07
     &  - 3.1459026440220561d-11*tu08
     &  - 5.4554143321149829d-12*tu09
     &  + 6.9660157399524824d-14*tu10
     &  + 5.0028186325156076d-15*tu11
     &  - 1.3596721085576870d-16*tu12
      HY6(0,0,0,0,1,-1) = 
     &  - 5.1808654530487943d-02
     &  - 3.9467925471066604d-01*tv01
     &  - 5.2089313732600333d-02*tv02
     &  - 4.5045319277492117d-03*tv03
     &  - 2.8110321314130566d-04*tv04
     &  - 1.3071808211252428d-05*tv05
     &  - 4.4407933465329014d-07*tv06
     &  - 9.8043016260102671d-09*tv07
     &  - 6.8228767174150247d-11*tv08
     &  + 3.3740378692890819d-12*tv09
     &  + 7.6859825221552282d-14*tv10
     &  - 2.4546540520830359d-15*tv11
     &  - 1.1057592409144857d-16*tv12
     &  + 6.9314718055994530d-01*HY5(0,0,0,0,1)
      HY6(0,0,0,0,1,1) = 
     &  + 5.1007139972048853d-03
     &  - 3.2244058635112390d-03*tu01
     &  + 5.2262508342031766d-03*tu02
     &  - 1.0907312804623211d-03*tu03
     &  + 1.2602887789169060d-04*tu04
     &  - 9.5795436969002532d-06*tu05
     &  + 4.9221061916471793d-07*tu06
     &  - 1.5666412139313395d-08*tu07
     &  + 1.6943785245493956d-10*tu08
     &  + 8.4051195817590671d-12*tu09
     &  - 2.8741998867185150d-13*tu10
     &  - 7.0596958474906422d-15*tu11
     &  + 4.9209256765554336d-16*tu12
      HY6(0,0,0,1,-1,-1) = 
     &  - 2.0953791384691381d-02
     &  - 1.3869732294064545d-01*tv01
     &  - 2.1126488033291559d-02*tv02
     &  - 2.2082101411605803d-03*tv03
     &  - 1.7316075079805566d-04*tv04
     &  - 1.0396642259343588d-05*tv05
     &  - 4.6426796180610744d-07*tv06
     &  - 1.3835532700962811d-08*tv07
     &  - 1.6552514903251994d-10*tv08
     &  + 5.7986327224472520d-12*tv09
     &  + 2.3841705874490641d-13*tv10
     &  - 3.4934413943568105d-15*tv11
     &  - 3.6145250212512102d-16*tv12
     &  + 2.4022650695910071d-01*HY4(0,0,0,1)
      HY6(0,0,0,1,-1,1) = 
     &  + 3.9127986670590087d-02
     &  - 2.7547134000945478d-01*tu01
     &  + 3.9389856771184318d-02*tu02
     &  - 3.7633511283218531d-03*tu03
     &  + 2.6237160854135942d-04*tu04
     &  - 1.3589797917156890d-05*tu05
     &  + 5.0153458799695620d-07*tu06
     &  - 1.1129373560864273d-08*tu07
     &  + 2.6590147788068245d-11*tu08
     &  + 5.9304320446387360d-12*tu09
     &  - 5.0604855333808554d-14*tu10
     &  - 6.8443705956786150d-15*tu11
     &  + 1.1491268796237608d-16*tu12
     &  + 4.8045301391820142d-01*HY4(0,0,0,1)
     &  - 6.9314718055994530d-01*HY5(0,0,0,1,-1)
      HY6(0,0,0,1,0,-1) = 
     &  - 6.1520856091010448d-02
     &  - 4.6834432772014678d-01*tv01
     &  - 6.1855084306702402d-02*tv02
     &  - 5.3551322615323719d-03*tv03
     &  - 3.3475965049626334d-04*tv04
     &  - 1.5603924683568053d-05*tv05
     &  - 5.3151179922217337d-07*tv06
     &  - 1.1733667313331215d-08*tv07
     &  - 7.6888894957534870d-11*tv08
     &  + 4.4445616540516046d-12*tv09
     &  + 1.0586246603160339d-13*tv10
     &  - 2.9968964392028113d-15*tv11
     &  - 1.5482309763158132d-16*tv12
     &  + 8.2246703342411321d-01*HY4(0,0,0,1)
      HY6(0,0,0,1,0,1) = 
     &  + 1.0098904839255951d-02
     &  - 6.0213084976727037d-03*tu01
     &  + 1.0315264173626980d-02*tu02
     &  - 2.0318456094850498d-03*tu03
     &  + 2.1701896405374517d-04*tu04
     &  - 1.4868285048755715d-05*tu05
     &  + 6.5962736705981183d-07*tu06
     &  - 1.6140973266987947d-08*tu07
     &  - 2.4004698749062361d-12*tu08
     &  + 1.1942305690452025d-11*tu09
     &  - 8.4566606220964458d-14*tu10
     &  - 1.5220952845506720d-14*tu11
     &  + 2.4619508313967382d-16*tu12
     &  + 2.1668085109382145d-17*tu13
      HY6(0,0,0,1,1,-1) = 
     &  + 4.3705380046193808d-02
     &  + 3.3165601345710473d-01*tv01
     &  + 4.3946671590499026d-02*tv02
     &  + 3.8270050959453154d-03*tv03
     &  + 2.4169566618324511d-04*tv04
     &  + 1.1471768622678529d-05*tv05
     &  + 4.0421793304218198d-07*tv06
     &  + 9.6332838329296186d-09*tv07
     &  + 9.5959845576495773d-11*tv08
     &  - 2.6251120926236781d-12*tv09
     &  - 9.5056026536867187d-14*tv10
     &  + 1.1329347737523036d-15*tv11
     &  + 1.1333119951778950d-16*tv12
     &  - 5.8224052646501250d-01*HY4(0,0,0,1)
     &  + 6.9314718055994530d-01*HY5(0,0,0,1,1)
      HY6(0,0,0,1,1,1) = 
     &  - 1.8847669940575295d-04
     &  + 8.4126791588380595d-04*tu01
     &  - 2.5210127116254848d-04*tu02
     &  + 2.9232323577698764d-04*tu03
     &  - 6.4104307755616893d-05*tu04
     &  + 7.1663485706204491d-06*tu05
     &  - 4.7997315257085651d-07*tu06
     &  + 1.8548710179392192d-08*tu07
     &  - 2.3670440986757158d-10*tu08
     &  - 1.2255353357019150d-11*tu09
     &  + 4.4850782754286274d-13*tu10
     &  + 1.2394849039063976d-14*tu11
     &  - 8.3023390394627707d-16*tu12
      HY6(0,0,1,-1,-1,-1) = 
     &  - 5.5158458297047641d-03
     &  - 3.2880017385017115d-02*tv01
     &  - 5.6062587248586548d-03*tv02
     &  - 7.9701147766316489d-04*tv03
     &  - 9.0866371823227041d-05*tv04
     &  - 7.6790617833682920d-06*tv05
     &  - 4.5372689416192977d-07*tv06
     &  - 1.6916507767853580d-08*tv07
     &  - 2.4980406131405385d-10*tv08
     &  + 8.8524022293874127d-12*tv09
     &  + 4.2006548759256983d-13*tv10
     &  - 6.0525576527236310d-15*tv11
     &  - 6.9758526583813641d-16*tv12
     &  + 5.5504108664821579d-02*HY3(0,0,1)
      HY6(0,0,1,-1,-1,1) = 
     &  + 4.4107486356711117d-02
     &  - 3.2540475855587874d-01*tu01
     &  + 4.4363075152372398d-02*tu02
     &  - 3.9719370824670728d-03*tu03
     &  + 2.5600832620753337d-04*tu04
     &  - 1.2201094081892413d-05*tu05
     &  + 4.1959855231504334d-07*tu06
     &  - 9.2611997934815624d-09*tu07
     &  + 6.7951987358771098d-11*tu08
     &  + 2.6316689599715227d-12*tu09
     &  - 5.4018870477337953d-14*tu10
     &  - 1.7165818659732139d-15*tu11
     &  + 5.6454314699235996d-17*tu12
     &  + 5.7009070532142637d-01*HY3(0,0,1)
     &  - 5.8224052646501250d-01*HY4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HY5(0,0,1,-1,-1)
      HY6(0,0,1,-1,0,-1) = 
     &  - 2.6210034636960371d-02
     &  - 1.5715821162767584d-01*tv01
     &  - 2.6453262510399113d-02*tv02
     &  - 2.9879279167606446d-03*tv03
     &  - 2.4383807583104342d-04*tv04
     &  - 1.4548800025992035d-05*tv05
     &  - 6.1026616405116812d-07*tv06
     &  - 1.5435507227858927d-08*tv07
     &  - 6.3606744492977608d-11*tv08
     &  + 9.6136542855871061d-12*tv09
     &  + 1.6467596924812002d-13*tv10
     &  - 9.9952075912097958d-15*tv11
     &  - 3.2835895533228659d-16*tv12
     &  + 2.6957647953152780d-01*HY3(0,0,1)
      HY6(0,0,1,-1,0,1) = 
     &  + 4.8882388671276517d-02
     &  - 3.2793171320050594d-01*tu01
     &  + 4.9215933604069848d-02*tu02
     &  - 4.8216214790070867d-03*tu03
     &  + 3.3411408642804686d-04*tu04
     &  - 1.6623583060050235d-05*tu05
     &  + 5.6916761487527928d-07*tu06
     &  - 1.1244349212509453d-08*tu07
     &  + 1.3970762352726373d-11*tu08
     &  + 4.6120768853119196d-12*tu09
     &  - 9.3882572004001882d-15*tu10
     &  - 4.2890498723462224d-15*tu11
     &  + 5.7009070532142637d-01*HY3(0,0,1)
     &  - 8.2246703342411321d-01*HY4(0,0,1,-1)
      HY6(0,0,1,-1,1,-1) = 
     &  + 4.1077534699591344d-02
     &  + 2.9874752932091605d-01*tv01
     &  + 4.1333539254414841d-02*tv02
     &  + 3.8068231197344039d-03*tv03
     &  + 2.5648277692327878d-04*tv04
     &  + 1.2976311566079180d-05*tv05
     &  + 4.7828546131993574d-07*tv06
     &  + 1.1175101719330100d-08*tv07
     &  + 6.3262526792445076d-11*tv08
     &  - 5.1918990766204080d-12*tv09
     &  - 9.8840737144910194d-14*tv10
     &  + 4.7596752590342989d-15*tv11
     &  + 1.7028625094355313d-16*tv12
     &  - 5.2253066044918489d-01*HY3(0,0,1)
     &  + 4.8045301391820142d-01*HY4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HY5(0,0,1,-1,1)
      HY6(0,0,1,-1,1,1) = 
     &  - 1.4430812987402793d-02
     &  + 9.6510972140131299d-02*tu01
     &  - 1.4579865251223773d-02*tu02
     &  + 1.6601257062483388d-03*tu03
     &  - 1.4955187698818268d-04*tu04
     &  + 1.0257820991032712d-05*tu05
     &  - 4.9973785067414342d-07*tu06
     &  + 1.4964665747875482d-08*tu07
     &  - 1.2448416356232245d-10*tu08
     &  - 8.5140500522944793d-12*tu09
     &  + 1.9896494631553405d-13*tu10
     &  + 9.0007759948836692d-15*tu11
     &  - 3.4150232729805004d-16*tu12
     &  - 1.6651232599446473d-01*HY3(0,0,1)
     &  + 2.4022650695910071d-01*HY4(0,0,1,-1)
      HY6(0,0,1,0,-1,-1) = 
     &  - 1.4645142281922033d-02
     &  - 8.7745126076699476d-02*tv01
     &  - 1.4792962436426946d-02*tv02
     &  - 1.7173130894490858d-03*tv03
     &  - 1.4828260624690394d-04*tv04
     &  - 9.6656586010184061d-06*tv05
     &  - 4.6264761654475372d-07*tv06
     &  - 1.4707623062718682d-08*tv07
     &  - 1.9558123647253525d-10*tv08
     &  + 6.1935262836963931d-12*tv09
     &  + 2.9287756927912258d-13*tv10
     &  - 2.9137849919172175d-15*tv11
     &  - 4.3898774313788532d-16*tv12
     &  + 1.5025711289494928d-01*HY3(0,0,1)
      HY6(0,0,1,0,-1,1) = 
     &  + 4.6305125549362313d-02
     &  - 3.2654154014228348d-01*tu01
     &  + 4.6594400393751786d-02*tu02
     &  - 4.3538158977015855d-03*tu03
     &  + 2.8974388877097068d-04*tu04
     &  - 1.3972110981977872d-05*tu05
     &  + 4.6906768668041542d-07*tu06
     &  - 9.2924994471028445d-09*tu07
     &  + 2.3299186333120360d-11*tu08
     &  + 3.3081423110721818d-12*tu09
     &  - 5.9949995632896400d-15*tu10
     &  - 3.1494749496819744d-15*tu11
     &  + 5.7009070532142637d-01*HY3(0,0,1)
     &  - 6.9314718055994530d-01*HY5(0,0,1,0,-1)
      HY6(0,0,1,0,1,-1) = 
     &  + 4.7416988639529789d-02
     &  + 3.5996520802031370d-01*tv01
     &  + 4.7678125380896056d-02*tv02
     &  + 4.1485308852884251d-03*tv03
     &  + 2.6156933491439051d-04*tv04
     &  + 1.2372839070929941d-05*tv05
     &  + 4.3268287838505531d-07*tv06
     &  + 1.0102814418987412d-08*tv07
     &  + 8.9226879623320354d-11*tv08
     &  - 3.2323820216718220d-12*tv09
     &  - 1.0324486184941271d-13*tv10
     &  + 1.6684965010857455d-15*tv11
     &  + 1.3642644392081747d-16*tv12
     &  - 6.3196619783816790d-01*HY3(0,0,1)
     &  + 6.9314718055994530d-01*HY5(0,0,1,0,1)
      HY6(0,0,1,0,1,1) = 
     &  - 2.5854340793219615d-04
     &  + 1.2441789444892364d-03*tu01
     &  - 3.4561702094830907d-04*tu02
     &  + 4.2958773683607680d-04*tu03
     &  - 8.7609270364073388d-05*tu04
     &  + 8.9415913336270322d-06*tu05
     &  - 5.3577926484060390d-07*tu06
     &  + 1.7653693463441633d-08*tu07
     &  - 1.2167240946199801d-10*tu08
     &  - 1.2735778153490247d-11*tu09
     &  + 2.4399336672181442d-13*tu10
     &  + 1.5691672965415483d-14*tu11
     &  - 4.7637560589341893d-16*tu12
     &  - 2.1909154334038158d-17*tu13
      HY6(0,0,1,1,-1,-1) = 
     &  + 2.5190087238800164d-02
     &  + 1.7721048962953744d-01*tv01
     &  + 2.5370232020970914d-02*tv02
     &  + 2.4692284021811593d-03*tv03
     &  + 1.8058773222096158d-04*tv04
     &  + 1.0249002771696678d-05*tv05
     &  + 4.4313546769722054d-07*tv06
     &  + 1.3299167663733221d-08*tv07
     &  + 1.8517460280676596d-10*tv08
     &  - 4.3790140820551570d-12*tv09
     &  - 2.4255947098474984d-13*tv10
     &  + 8.2365860199211587d-16*tv11
     &  + 3.2237103732361896d-16*tv12
     &  - 3.0882537509683393d-01*HY3(0,0,1)
     &  + 2.4022650695910071d-01*HY4(0,0,1,1)
      HY6(0,0,1,1,-1,1) = 
     &  - 3.2723797747217469d-02
     &  + 2.3183749689928543d-01*tu01
     &  - 3.2974628551142358d-02*tu02
     &  + 3.3133268538491978d-03*tu03
     &  - 2.5137487400465379d-04*tu04
     &  + 1.4134260356973640d-05*tu05
     &  - 5.4402573327825782d-07*tu06
     &  + 1.1277890690228467d-08*tu07
     &  + 4.4282677058228674d-11*tu08
     &  - 7.1497967605294778d-12*tu09
     &  - 6.3738690773975166d-14*tu10
     &  + 1.0053874056108859d-14*tu11
     &  + 7.0833916334917349d-17*tu12
     &  - 4.0357837932696163d-01*HY3(0,0,1)
     &  + 4.8045301391820142d-01*HY4(0,0,1,1)
     &  - 6.9314718055994530d-01*HY5(0,0,1,1,-1)
      HY6(0,0,1,1,0,-1) = 
     &  + 5.6401439498409945d-02
     &  + 4.2794977116954855d-01*tv01
     &  + 5.6712716054129060d-02*tv02
     &  + 4.9388313912802353d-03*tv03
     &  + 3.1179396611628488d-04*tv04
     &  + 1.4773599198927616d-05*tv05
     &  + 5.1751267253936611d-07*tv06
     &  + 1.2070357273343092d-08*tv07
     &  + 1.0213868955719863d-10*tv08
     &  - 4.2299122614409848d-12*tv09
     &  - 1.3649762961212977d-13*tv10
     &  + 1.9851508973205468d-15*tv11
     &  + 1.8213737729563691d-16*tv12
     &  - 7.5128556447474642d-01*HY3(0,0,1)
     &  + 8.2246703342411321d-01*HY4(0,0,1,1)
      HY6(0,0,1,1,0,1) = 
     &  - 4.6911420419152552d-04
     &  + 2.4547685317693898d-03*tu01
     &  - 6.2672063116386910d-04*tu02
     &  + 8.4232036930638332d-04*tu03
     &  - 1.5834726962649909d-04*tu04
     &  + 1.4463842593710629d-05*tu05
     &  - 7.4071943447722856d-07*tu06
     &  + 1.8069631774720521d-08*tu07
     &  + 1.2311785146282673d-10*tu08
     &  - 1.6569064533344789d-11*tu09
     &  - 1.0177780251227766d-13*tu10
     &  + 2.4932258502277246d-14*tu11
     &  + 4.9103652809402367d-17*tu12
     &  - 4.1022828420193651d-17*tu13
      HY6(0,0,1,1,1,-1) = 
     &  - 4.0452685957824271d-02
     &  - 3.0609430833844765d-01*tv01
     &  - 4.0679087417221787d-02*tv02
     &  - 3.5605238134951700d-03*tv03
     &  - 2.2679525215032772d-04*tv04
     &  - 1.0917652622282148d-05*tv05
     &  - 3.9390526708597590d-07*tv06
     &  - 9.8078523507829841d-09*tv07
     &  - 1.1240165222259890d-10*tv08
     &  + 2.4089457746957181d-12*tv09
     &  + 1.1249644762579880d-13*tv10
     &  - 4.0862505705308266d-16*tv11
     &  - 1.2469056845897424d-16*tv12
     &  + 5.3721319360804020d-01*HY3(0,0,1)
     &  - 5.8224052646501250d-01*HY4(0,0,1,1)
     &  + 6.9314718055994530d-01*HY5(0,0,1,1,1)
      HY6(0,0,1,1,1,1) = 
     &  + 9.4458440817220657d-05
     &  - 5.9733423813960670d-05*tu01
     &  + 1.2679481544929151d-04*tu02
     &  - 2.9942033962824793d-05*tu03
     &  + 3.2846810008647230d-05*tu04
     &  - 6.0486198258927182d-06*tu05
     &  + 5.1064820050070357d-07*tu06
     &  - 2.1464872630035616d-08*tu07
     &  + 2.1237651997185427d-10*tu08
     &  + 1.8518924274204752d-11*tu09
     &  - 4.4651729322565372d-13*tu10
     &  - 2.5066373701303167d-14*tu11
     &  + 8.8530625088387685d-16*tu12
     &  + 3.7702710011744510d-17*tu13
      HY6(0,1,-1,-1,-1,-1) = 
     &  - 1.2332916013766885d-03
     &  - 5.9850373415782699d-03*tv01
     &  - 1.2814903614477569d-03*tv02
     &  - 2.4203363351148856d-04*tv03
     &  - 4.8671392022545211d-05*tv04
     &  - 6.2349948765925717d-06*tv05
     &  - 4.7291128894336421d-07*tv06
     &  - 2.0039856946713978d-08*tv07
     &  - 2.7879291355779837d-10*tv08
     &  + 1.4119534878270229d-11*tv09
     &  + 5.4355361965349181d-13*tv10
     &  - 1.4226008175514292d-14*tv11
     &  - 9.9762619738606041d-16*tv12
     &  + 9.6181291076284771d-03*HY2(0,1)
      HY6(0,1,-1,-1,-1,1) = 
     &  + 4.1836104891717346d-02
     &  - 3.1390844391224177d-01*tu01
     &  + 4.2069764617256106d-02*tu02
     &  - 3.6911908264528380d-03*tu03
     &  + 2.3405338837531550d-04*tu04
     &  - 1.1131897442771459d-05*tu05
     &  + 3.9376778515342256d-07*tu06
     &  - 9.5354122062912899d-09*tu07
     &  + 1.0485168205297918d-10*tu08
     &  + 2.2188496151827449d-12*tu09
     &  - 9.6813845819915285d-14*tu10
     &  - 4.7264515463858939d-16*tu11
     &  + 1.0161528389800398d-16*tu12
     &  + 5.5070993482224879d-01*HY2(0,1)
     &  - 5.3721319360804020d-01*HY3(0,1,-1)
     &  - 5.8224052646501250d-01*HY4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HY5(0,1,-1,-1,-1)
      HY6(0,1,-1,-1,1,-1) = 
     &  + 3.5120173371331632d-02
     &  + 2.5089184077607053d-01*tv01
     &  + 3.5369328343640433d-02*tv02
     &  + 3.4340830240244978d-03*tv03
     &  + 2.4968242042590878d-04*tv04
     &  + 1.3640707452714957d-05*tv05
     &  + 5.2745162664686725d-07*tv06
     &  + 1.1855407005189550d-08*tv07
     &  + 3.4710744723900746d-12*tv08
     &  - 7.4708031493368147d-12*tv09
     &  - 3.8339806149995631d-14*tv10
     &  + 9.3774638797398349d-15*tv11
     &  + 1.2515698744117859d-16*tv12
     &  - 4.3755947670194733d-01*HY2(0,1)
     &  + 4.0357837932696163d-01*HY3(0,1,-1)
     &  + 4.8045301391820142d-01*HY4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HY5(0,1,-1,-1,1)
      HY6(0,1,-1,-1,1,1) = 
     &  - 2.2485681498643441d-02
     &  + 1.5634832891844655d-01*tu01
     &  - 2.2663749328350437d-02*tu02
     &  + 2.3091169157254260d-03*tu03
     &  - 1.7852657393075781d-04*tu04
     &  + 1.0546225386746860d-05*tu05
     &  - 4.5889652995429782d-07*tu06
     &  + 1.3182979350378483d-08*tu07
     &  - 1.5211705621409104d-10*tu08
     &  - 4.8434107737345052d-12*tu09
     &  + 1.8888859405025506d-13*tu10
     &  + 2.5440556514740028d-15*tu11
     &  - 2.4730764835364494d-16*tu12
     &  - 2.7177021267950885d-01*HY2(0,1)
     &  + 3.0882537509683393d-01*HY3(0,1,-1)
     &  + 2.4022650695910071d-01*HY4(0,1,-1,-1)
      HY6(0,1,-1,0,1,1) = 
     &  - 1.0036739754010005d-02
     &  + 6.1389162474359647d-02*tu01
     &  - 1.0176541064414075d-02*tu02
     &  + 1.3855196750266142d-03*tu03
     &  - 1.4029036224973019d-04*tu04
     &  + 1.0032155513388204d-05*tu05
     &  - 4.8919482217263247d-07*tu06
     &  + 1.4625011063319347d-08*tu07
     &  - 1.4277225998073624d-10*tu08
     &  - 6.9435832324977467d-12*tu09
     &  + 2.0394924266913317d-13*tu10
     &  + 5.5154397564731892d-15*tu11
     &  - 3.0285229577933867d-16*tu12
     &  - 1.0415029416221149d-01*HY2(0,1)
     &  + 1.5025711289494928d-01*HY3(0,1,-1)
      HY6(0,1,-1,1,-1,-1) = 
     &  + 1.7348148275856081d-02
     &  + 1.1794850098256868d-01*tv01
     &  + 1.7502653757017768d-02*tv02
     &  + 1.8649213614630388d-03*tv03
     &  + 1.5498317861700767d-04*tv04
     &  + 1.0016626891378735d-05*tv05
     &  + 4.7786538296942145d-07*tv06
     &  + 1.4822335615769576d-08*tv07
     &  + 1.6766062914827150d-10*tv08
     &  - 7.3050366817289366d-12*tv09
     &  - 2.6658975773028605d-13*tv10
     &  + 5.4545849506407993d-15*tv11
     &  + 4.2910818955819329d-16*tv12
     &  - 2.0437039310996688d-01*HY2(0,1)
     &  + 1.6651232599446473d-01*HY3(0,1,-1)
     &  + 2.4022650695910071d-01*HY4(0,1,-1,1)
      HY6(0,1,-1,1,-1,1) = 
     &  - 3.9201630228664789d-02
     &  + 2.8264279714168323d-01*tu01
     &  - 3.9462848255190595d-02*tu02
     &  + 3.7531885980346676d-03*tu03
     &  - 2.6170417506915686d-04*tu04
     &  + 1.3451490417960483d-05*tu05
     &  - 4.8618370560297407d-07*tu06
     &  + 1.0504174032801197d-08*tu07
     &  - 3.5124380760343985d-11*tu08
     &  - 4.4416302534538277d-12*tu09
     &  + 3.7816461814279352d-14*tu10
     &  + 4.1338128513427904d-15*tu11
     &  - 5.4996851101388859d-17*tu12
     &  - 4.9354620950987381d-01*HY2(0,1)
     &  + 5.2253066044918489d-01*HY3(0,1,-1)
     &  + 4.8045301391820142d-01*HY4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HY5(0,1,-1,1,-1)
      HY6(0,1,-1,1,1,-1) = 
     &  - 4.3239107331474553d-02
     &  - 3.2209539106611691d-01*tv01
     &  - 4.3487198580425984d-02*tv02
     &  - 3.8642063613968709d-03*tv03
     &  - 2.4850976497747861d-04*tv04
     &  - 1.1924945945136055d-05*tv05
     &  - 4.1859587520614688d-07*tv06
     &  - 9.6389406791330185d-09*tv07
     &  - 7.9768832777890164d-11*tv08
     &  + 2.9655387645712579d-12*tv09
     &  + 8.0224666132483367d-14*tv10
     &  - 1.7950613355659708d-15*tv11
     &  - 1.0147674898247373d-16*tv12
     &  + 5.6465878038610476d-01*HY2(0,1)
     &  - 5.7009070532142637d-01*HY3(0,1,-1)
     &  - 5.8224052646501250d-01*HY4(0,1,-1,1)
     &  + 6.9314718055994530d-01*HY5(0,1,-1,1,1)
      HY6(0,1,-1,1,1,1) = 
     &  + 3.7766595687872900d-03
     &  - 2.2716791050870549d-02*tu01
     &  + 3.8518715319618466d-03*tu02
     &  - 5.3184038338677655d-04*tu03
     &  + 7.5705693686818158d-05*tu04
     &  - 7.7474637298868374d-06*tu05
     &  + 4.9390092551569811d-07*tu06
     &  - 1.7582973746952598d-08*tu07
     &  + 1.7013439776924935d-10*tu08
     &  + 1.1291746485262810d-11*tu09
     &  - 2.7839649356697850d-13*tu10
     &  - 1.2259759439865939d-14*tu11
     &  + 4.5909056143483948d-16*tu12
     &  + 3.8472516430513908d-02*HY2(0,1)
     &  - 5.5504108664821579d-02*HY3(0,1,-1)
      HY6(0,1,0,1,-1,-1) = 
     &  + 2.3317200664406615d-02
     &  + 1.6152032427829515d-01*tv01
     &  + 2.3491632569370747d-02*tv02
     &  + 2.3342580977868037d-03*tv03
     &  + 1.7488207310387252d-04*tv04
     &  + 1.0176430734830099d-05*tv05
     &  + 4.5036007062613579d-07*tv06
     &  + 1.3770215545093324d-08*tv07
     &  + 1.9165954499954399d-10*tv08
     &  - 4.9538066012358553d-12*tv09
     &  - 2.7096018508357615d-13*tv10
     &  + 1.0746901717607132d-15*tv11
     &  + 3.7992015050754655d-16*tv12
     &  - 2.8103333764091645d-01*HY2(0,1)
     &  + 1.2011325347955035d-01*HY2(0,1)*HY2(0,1)
     &  - 4.8045301391820142d-01*HY4(0,0,1,1)
      HY6(0,1,0,1,-1,1) = 
     &  - 3.5525626510720466d-02
     &  + 2.5164912191938246d-01*tu01
     &  - 3.5791447893538771d-02*tu02
     &  + 3.5983664669075185d-03*tu03
     &  - 2.6633457317902309d-04*tu04
     &  + 1.4224431116121248d-05*tu05
     &  - 5.1317896672420810d-07*tu06
     &  + 1.0157187310192017d-08*tu07
     &  + 1.1359955610992378d-11*tu08
     &  - 4.6088222374621577d-12*tu09
     &  - 3.3983701622541189d-14*tu10
     &  + 5.1605817845495944d-15*tu11
     &  + 5.3659527985513966d-17*tu12
     &  - 4.3804558824071468d-01*HY2(0,1)
     &  + 2.4022650695910071d-01*HY2(0,1)*HY2(0,1)
     &  - 9.6090602783640284d-01*HY4(0,0,1,1)
     &  - 6.9314718055994530d-01*HY5(0,1,0,1,-1)
      HY6(0,1,0,1,1,-1) = 
     &  - 4.2010292609863307d-02
     &  - 3.1805521974328815d-01*tv01
     &  - 4.2244704098291667d-02*tv02
     &  - 3.6936488813135429d-03*tv03
     &  - 2.3481478836938125d-04*tv04
     &  - 1.1262304507319648d-05*tv05
     &  - 4.0340522071781442d-07*tv06
     &  - 9.8770163522146998d-09*tv07
     &  - 1.0516690455963892d-10*tv08
     &  + 2.7217451842460279d-12*tv09
     &  + 1.1265546185826823d-13*tv10
     &  - 8.3798353128983571d-16*tv11
     &  - 1.3653987117165382d-16*tv12
     &  + 5.5823730083320863d-01*HY2(0,1)
     &  - 2.9112026323250625d-01*HY2(0,1)*HY2(0,1)
     &  + 1.1644810529300250d+00*HY4(0,0,1,1)
     &  + 6.9314718055994530d-01*HY5(0,1,0,1,1)
      HY6(0,1,0,1,1,1) = 
     &  + 1.2449270165673753d-04
     &  - 7.0586950819910646d-05*tu01
     &  + 1.6688135428533209d-04*tu02
     &  - 3.5361978950729430d-05*tu03
     &  + 4.2923474916462123d-05*tu04
     &  - 7.1271613547345740d-06*tu05
     &  + 5.3494416903827874d-07*tu06
     &  - 1.9525895461351290d-08*tu07
     &  + 1.2165477922805900d-10*tu08
     &  + 1.5472009616260443d-11*tu09
     &  - 2.2597665222302593d-13*tu10
     &  - 2.0425466335375329d-14*tu11
     &  + 4.1406363640638342d-16*tu12
     &  + 3.0641479153741016d-17*tu13
      HY6(0,1,1,-1,-1,-1) = 
     &  + 8.2157686982525192d-03
     &  + 5.1586082580938031d-02*tv01
     &  + 8.3148301533015520d-03*tv02
     &  + 1.0149149749056053d-03*tv03
     &  + 9.9492816528928787d-05*tv04
     &  + 7.6227171000532532d-06*tv05
     &  + 4.3164412746927525d-07*tv06
     &  + 1.6304286197847552d-08*tv07
     &  + 2.8220076321709456d-10*tv08
     &  - 6.7113721207816753d-12*tv09
     &  - 4.4614001262544056d-13*tv10
     &  + 9.3171683200104626d-16*tv11
     &  + 6.6900487954988009d-16*tv12
     &  - 8.8326067366010207d-02*HY2(0,1)
     &  + 5.5504108664821579d-02*HY3(0,1,1)
      HY6(0,1,1,-1,-1,1) = 
     &  - 4.3262006534038570d-02
     &  + 3.1619298448833055d-01*tu01
     &  - 4.3526923137927670d-02*tu02
     &  + 4.0232986545242805d-03*tu03
     &  - 2.6533391147661949d-04*tu04
     &  + 1.2595919806200067d-05*tu05
     &  - 4.1736903153259173d-07*tu06
     &  + 8.6182264641391703d-09*tu07
     &  - 6.1408664560101190d-11*tu08
     &  - 1.7335460593274057d-12*tu09
     &  + 3.5328664104337078d-14*tu10
     &  + 4.9496390392204875d-16*tu11
     &  - 5.5306546869199291d-01*HY2(0,1)
     &  + 5.7009070532142637d-01*HY3(0,1,1)
     &  - 5.8224052646501250d-01*HY4(0,1,1,-1)
     &  - 6.9314718055994530d-01*HY5(0,1,1,-1,-1)
      HY6(0,1,1,-1,1,-1) = 
     &  - 4.1199587016990418d-02
     &  - 3.0316011051549285d-01*tv01
     &  - 4.1450356300128971d-02*tv02
     &  - 3.7688725501205936d-03*tv03
     &  - 2.5124098217655171d-04*tv04
     &  - 1.2665548316901016d-05*tv05
     &  - 4.7178449118911534d-07*tv06
     &  - 1.1483271087255486d-08*tv07
     &  - 8.5312428048588837d-11*tv08
     &  + 5.1695527482245053d-12*tv09
     &  + 1.4052945993362265d-13*tv10
     &  - 3.8290046620783382d-15*tv11
     &  - 2.3291931370625197d-16*tv12
     &  + 5.3075770941318154d-01*HY2(0,1)
     &  - 5.2253066044918489d-01*HY3(0,1,1)
     &  + 4.8045301391820142d-01*HY4(0,1,1,-1)
     &  + 6.9314718055994530d-01*HY5(0,1,1,-1,1)
      HY6(0,1,1,-1,1,1) = 
     &  + 1.1993441817290255d-02
     &  - 8.0916271174440008d-02*tu01
     &  + 1.2130464177541329d-02*tu02
     &  - 1.3471119587946022d-03*tu03
     &  + 1.3756084384765949d-04*tu04
     &  - 1.0690314820441380d-05*tu05
     &  + 5.3852352153979570d-07*tu06
     &  - 1.4636984458437320d-08*tu07
     &  + 3.9890076073876736d-11*tu08
     &  + 8.6289129602006619d-12*tu09
     &  - 3.4840094804318206d-14*tu10
     &  - 1.0182627549621129d-14*tu11
     &  + 3.7863890375265886d-17*tu12
     &  + 1.3986960788271778d-01*HY2(0,1)
     &  - 1.6651232599446473d-01*HY3(0,1,1)
     &  + 2.4022650695910071d-01*HY4(0,1,1,-1)
      HY6(0,1,1,1,-1,-1) = 
     &  - 2.6478241559869173d-02
     &  - 1.8993300915405635d-01*tv01
     &  - 2.6658542142639876d-02*tv02
     &  - 2.5326606660653166d-03*tv03
     &  - 1.8073315765542287d-04*tv04
     &  - 1.0065956022582163d-05*tv05
     &  - 4.3277909719234731d-07*tv06
     &  - 1.3237218152535902d-08*tv07
     &  - 2.0395065078922920d-10*tv08
     &  + 3.6798400817507221d-12*tv09
     &  + 2.6150191996663759d-13*tv10
     &  + 1.0484356376131264d-15*tv11
     &  - 3.1909849879503528d-16*tv12
     &  + 3.3160957134970784d-01*HY2(0,1)
     &  - 3.0882537509683393d-01*HY3(0,1,1)
     &  + 2.4022650695910071d-01*HY4(0,1,1,1)
      HY6(0,1,1,1,-1,1) = 
     &  + 3.0081658634243567d-02
     &  - 2.1377036357609870d-01*tu01
     &  + 3.0329672786178485d-02*tu02
     &  - 3.0144165710366503d-03*tu03
     &  + 2.4858382360567612d-04*tu04
     &  - 1.5049849884738328d-05*tu05
     &  + 5.6956244333839007d-07*tu06
     &  - 9.6134425140637308d-09*tu07
     &  - 1.0902911234284070d-10*tu08
     &  + 4.2555707467912415d-12*tu09
     &  + 1.9797730260826362d-13*tu10
     &  - 5.0189926172500353d-15*tu11
     &  - 3.2950206131955252d-16*tu12
     &  + 3.7236781050901709d-01*HY2(0,1)
     &  - 4.0357837932696163d-01*HY3(0,1,1)
     &  + 4.8045301391820142d-01*HY4(0,1,1,1)
     &  - 6.9314718055994530d-01*HY5(0,1,1,1,-1)
      HY6(0,1,1,1,1,-1) = 
     &  + 3.9047609723280097d-02
     &  + 2.9490526605059110d-01*tv01
     &  + 3.9268080810198696d-02*tv02
     &  + 3.4484474205677681d-03*tv03
     &  + 2.2086339957610831d-04*tv04
     &  + 1.0726031827335130d-05*tv05
     &  + 3.9243399128859247d-07*tv06
     &  + 1.0000797143348540d-08*tv07
     &  + 1.2120656672236459d-10*tv08
     &  - 2.4221748486512298d-12*tv09
     &  - 1.2707263896767648d-13*tv10
     &  + 2.4294969573951350d-17*tv11
     &  + 1.3968563316726440d-16*tv12
     &  - 5.1747906167389938d-01*HY2(0,1)
     &  + 5.3721319360804020d-01*HY3(0,1,1)
     &  - 5.8224052646501250d-01*HY4(0,1,1,1)
     &  + 6.9314718055994530d-01*HY5(0,1,1,1,1)
      HY6(0,1,1,1,1,1) = 
     &  - 6.0070834689670138d-06
     &  + 5.3248530359544277d-05*tu01
     &  - 9.0106252034505208d-06*tu02
     &  + 2.6706380095123042d-05*tu03
     &  - 3.6042500813802083d-06*tu04
     &  + 5.4069130961504932d-06*tu05
     &  - 6.0070834689670138d-07*tu06
     &  + 2.3392901289292290d-08*tu07
     &  - 2.2764221454379075d-11*tu09
     &  + 3.3383355497601399d-14*tu11
     &  - 5.3170479809609942d-17*tu13
      endif 
** nw > 5 endif 
      endif 
** (n1,n2) = (-1,1) -- completion endif 
      return 
      end 

************************************************************************ 
      subroutine fillirr1dhplat1(r,nw,HR1,HR2,HR3,HR4,HR5,HR6, 
     &                                HY1,HY2,HY3,HY4,HY5,HY6,
     &                                Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2) 
** evaluates the HPL for r2m1 < y < r2p1
** fillirr1dhplat1 is called by eval1dhplat1 after calling 
** fillirr1dhplat0 with argument r=(1-y)/(1+y) 
** it is guaranteed that nw is in the range 2:4, and that (n1,n2) 
** take one of the pairs of values (0,1), (-1,0) or (-1,1) 
      implicit double precision (a-h,o-z) 
      dimension HR1(-1:1),HR2(-1:1,-1:1),HR3(-1:1,-1:1,-1:1), 
     &          HR4(-1:1,-1:1,-1:1,-1:1),
     &          HR5(-1:1,-1:1,-1:1,-1:1,-1:1), 
     &          HR6(-1:1,-1:1,-1:1,-1:1,-1:1,-1:1) 
      dimension HY1(n1:n2),HY2(n1:n2,n1:n2),HY3(n1:n2,n1:n2,n1:n2), 
     &          HY4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HY5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HY6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension Hi1(n1:n2),Hi2(n1:n2,n1:n2),Hi3(n1:n2,n1:n2,n1:n2), 
     &          Hi4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
** (n1,n2) = (0,1) or (-1,1) 
      if (    ( (n1.eq.0).and.(n2.eq.1) ) 
     &    .or.( (n1.eq.-1).and.(n2.eq.1) ) ) then 
      HY2(0,1) = 
     &  + 1.6449340668482264d+00
     &  + 6.9314718055994530d-01*HR1(-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)
     &  + HR1( -1)*HR1(0)
     &  - HR1( -1)*HR1(1)
     &  + HR1(0) *HR1(1)
     &  + 6.9314718055994530d-01*HR1(1)
     &  + HR2( -1,1)
     &  - HR2(0, -1)
     &  - HR2(0,1) 
      if (r.lt.0d0) then 
      Hi2(0,1) = 
     &  - HR1( -1) 
     &  - HR1(1) 
      endif 
      if ( nw.gt.2 ) then 
      HY3(0,0,1) = 
     &  + 1.2020569031595942d+00
     &  - 1.6449340668482264d+00*HR1(-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + HR1( -1)*HR2(0,-1)
     &  + HR1( -1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)
     &  - 1.6449340668482264d+00*HR1(1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)
     &  - HR1(1) *HR2(-1,1)
     &  + HR1(1) *HR2(0,-1)
     &  + HR1(1) *HR2(0,1)
     &  - HR3( -1,-1,1)
     &  + HR3( -1,1,1)
     &  - HR3(0, -1,-1)
     &  - HR3(0, -1,1)
     &  - HR3(0,1, -1)
     &  - HR3(0,1,1) 
      HY3(0,1,1) = 
     &  + 1.2020569031595942d+00
     &  - 2.4022650695910071d-01*HR1(-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)
     &  + HR1( -1)*HR1(0)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)
     &  + HR1( -1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)
     &  - HR1(0) *HR2(-1,1)
     &  + HR1(0) *HR2(0,-1)
     &  + HR1(0) *HR2(0,1)
     &  - 2.4022650695910071d-01*HR1(1)
     &  - 6.9314718055994530d-01*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(0,1)
     &  - HR3( -1,-1,1)
     &  - HR3(0, -1,-1)
     &  - HR3(0,0, -1)
     &  - HR3(0,0,1) 
     &  - HR3(0,1, -1)
      if (r.lt.0d0) then 
      HY3(0,1,1) = HY3(0,1,1) 
     &  + 4.9348022005446793d+00*HR1(-1)
     &  + 4.9348022005446793d+00*HR1(1)
      Hi3(0,0,1) = 
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1) 
     &  + HR1( -1)*HR1(1) 
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1) 
      Hi3(0,1,1) = 
     &  + 6.9314718055994530d-01*HR1(-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)
     &  + HR1( -1)*HR1(0)
     &  - HR1( -1)*HR1(1)
     &  + HR1(0) *HR1(1)
     &  + 6.9314718055994530d-01*HR1(1)
     &  + HR2( -1,1)
     &  - HR2(0, -1)
     &  - HR2(0,1) 
      endif
      endif
      if ( nw.gt.3 ) then 
      HY4(0,0,0,1) = 
     &  + 1.0823232337111381d+00
     &  - 1.2020569031595942d+00*HR1(-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR3(0,-1,1)
     &  + HR1( -1)*HR3(0,1,-1)
     &  + HR1( -1)*HR3(0,1,1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.2020569031595942d+00*HR1(1)
     &  + 8.2246703342411321d-01*HR1(1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  + HR1(1) *HR3(-1,-1,1)
     &  - HR1(1) *HR3(-1,1,1)
     &  + HR1(1) *HR3(0,-1,-1)
     &  + HR1(1) *HR3(0,-1,1)
     &  + HR1(1) *HR3(0,1,-1)
     &  + HR1(1) *HR3(0,1,1)
     &  + HR4( -1,-1,-1,1)
     &  - HR4( -1,-1,1,1)
     &  + HR4( -1,1,1,1)
     &  - HR4(0, -1,-1,-1)
     &  - HR4(0, -1,-1,1)
     &  - HR4(0, -1,1,-1)
     &  - HR4(0, -1,1,1)
     &  - HR4(0,1, -1,-1)
     &  - HR4(0,1, -1,1)
     &  - HR4(0,1,1, -1)
     &  - HR4(0,1,1,1) 
      HY4(0,0,1,1) = 
     &  + 2.7058080842778454d-01
     &  - 1.2020569031595942d+00*HR1(-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - HR1( -1)*HR1(0)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR2(0,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(0,1)
     &  - HR1( -1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR3(-1,1,1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR3(0,0,-1)
     &  + HR1( -1)*HR3(0,0,1)
     &  + HR1( -1)*HR3(0,1,-1)
     &  + 2.5000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)
     &  + HR1(0) *HR1(1)*HR2(-1,1)
     &  - HR1(0) *HR1(1)*HR2(0,-1)
     &  - HR1(0) *HR1(1)*HR2(0,1)
     &  + HR1(0) *HR3(-1,-1,1)
     &  - HR1(0) *HR3(-1,1,1)
     &  + HR1(0) *HR3(0,-1,-1)
     &  + HR1(0) *HR3(0,-1,1)
     &  + HR1(0) *HR3(0,1,-1)
     &  + HR1(0) *HR3(0,1,1)
     &  - 1.2020569031595942d+00*HR1(1)
     &  + 1.2011325347955035d-01*HR1(1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(0,1)
     &  + HR1(1) *HR3(-1,-1,1)
     &  + HR1(1) *HR3(0,-1,-1)
     &  + HR1(1) *HR3(0,0,-1)
     &  + HR1(1) *HR3(0,0,1)
     &  + HR1(1) *HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR3(0,1,1)
     &  + 2.0000000000000000d+00*HR4(-1,-1,-1,1)
     &  - HR4( -1,-1,1,1)
     &  - 2.0000000000000000d+00*HR4(0,-1,-1,-1)
     &  - HR4(0, -1,-1,1)
     &  - HR4(0, -1,1,-1)
     &  - HR4(0,0, -1,-1)
     &  - HR4(0,0, -1,1)
     &  - HR4(0,0,1, -1)
     &  - HR4(0,0,1,1) 
     &  - 2.0000000000000000d+00*HR4(0,1,-1,-1)
     &  - HR4(0,1, -1,1)
     &  - HR4(0,1,1, -1)
      HY4(0,1,1,1) = 
     &  + 1.0823232337111381d+00
     &  + 5.5504108664821579d-02*HR1(-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  - HR1( -1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(0,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(0,1)
     &  + HR1(0) *HR3(-1,-1,1)
     &  + HR1(0) *HR3(0,-1,-1)
     &  + HR1(0) *HR3(0,0,-1)
     &  + HR1(0) *HR3(0,0,1)
     &  + HR1(0) *HR3(0,1,-1)
     &  + 5.5504108664821579d-02*HR1(1)
     &  + 2.4022650695910071d-01*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR2(0,1)
     &  + 6.9314718055994530d-01*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR3(0,1,-1)
     &  + HR4( -1,-1,-1,1)
     &  - HR4(0, -1,-1,-1)
     &  - HR4(0,0, -1,-1)
     &  - HR4(0,0,0, -1)
     &  - HR4(0,0,0,1) 
     &  - HR4(0,0,1, -1)
     &  - HR4(0,1, -1,-1)
      if (r.lt.0d0) then 
      HY4(0,0,1,1) = HY4(0,0,1,1) 
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(1)
     &  - 2.4674011002723396d+00*HR1(1)*HR1(1)
      HY4(0,1,1,1) = HY4(0,1,1,1) 
     &  - 3.4205442319285582d+00*HR1(-1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(0)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(1)
     &  - 4.9348022005446793d+00*HR1(0)*HR1(1)
     &  - 3.4205442319285582d+00*HR1(1)
     &  - 4.9348022005446793d+00*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR2(0,-1)
     &  + 4.9348022005446793d+00*HR2(0,1)
      Hi4(0,0,0,1) = 
     &  - 1.666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1) 
     &  - 5.000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1) 
     &  - 5.000000000000000d-01*HR1(-1)*HR1(1)*HR1(1) 
     &  - 1.666666666666666d-01*HR1(1)*HR1(1)*HR1(1) 
      Hi4(0,0,1,1) = 
     &  - 3.465735902799726d-01*HR1(-1)*HR1(-1) 
     &  + 1.666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1) 
     &  - 5.000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0) 
     &  + 5.000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1) 
     &  - HR1( -1)*HR1(0)*HR1(1) 
     &  - 6.931471805599453d-01*HR1(-1)*HR1(1) 
     &  + 5.000000000000000d-01*HR1(-1)*HR1(1)*HR1(1) 
     &  + HR1( -1)*HR2(0,-1) 
     &  + HR1( -1)*HR2(0,1) 
     &  - 5.000000000000000d-01*HR1(0)*HR1(1)*HR1(1) 
     &  - 3.465735902799726d-01*HR1(1)*HR1(1) 
     &  - HR1(1) *HR2(-1,1) 
     &  + HR1(1) *HR2(0,-1) 
     &  + HR1(1) *HR2(0,1) 
     &  - HR3( -1,-1,1) 
     &  + HR3( -1,1,1) 
     &  - HR3(0, -1,-1) 
     &  - HR3(0, -1,1) 
     &  - HR3(0,1, -1) 
     &  - HR3(0,1,1) 
      Hi4(0,1,1,1) = 
     &  + 1.404707559889125d+00*HR1(-1) 
     &  + 3.465735902799726d-01*HR1(-1)*HR1(-1) 
     &  - 1.666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1) 
     &  + 5.000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0) 
     &  - 5.000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1) 
     &  - 6.931471805599453d-01*HR1(-1)*HR1(0) 
     &  - 5.000000000000000d-01*HR1(-1)*HR1(0)*HR1(0) 
     &  + HR1( -1)*HR1(0)*HR1(1) 
     &  + 6.931471805599453d-01*HR1(-1)*HR1(1) 
     &  + HR1( -1)*HR2(-1,1) 
     &  - 5.000000000000000d-01*HR1(0)*HR1(0)*HR1(1) 
     &  - 6.931471805599453d-01*HR1(0)*HR1(1) 
     &  - HR1(0) *HR2(-1,1) 
     &  + HR1(0) *HR2(0,-1) 
     &  + HR1(0) *HR2(0,1) 
     &  + 1.404707559889125d+00*HR1(1) 
     &  - 6.931471805599453d-01*HR2(-1,1) 
     &  + 6.931471805599453d-01*HR2(0,-1) 
     &  + 6.931471805599453d-01*HR2(0,1) 
     &  - HR3( -1,-1,1) 
     &  - HR3(0, -1,-1) 
     &  - HR3(0,0, -1) 
     &  - HR3(0,0,1)  
     &  - HR3(0,1, -1) 
      endif
      endif
** nw > 3 endif 
      if ( nw.gt.4 ) then 
      HY5(0,0,0,0,1) = 
     &  + 1.0369277551433699d+00
     &  - 1.0823232337111381d+00*HR1(-1)
     &  + 6.0102845157979714d-01*HR1(-1)*HR1(-1)
     &  - 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.2020569031595942d+00*HR1(-1)*HR1(1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,1,1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR4(0,-1,1,1)
     &  + HR1( -1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR4(0,1,-1,1)
     &  + HR1( -1)*HR4(0,1,1,-1)
     &  + HR1( -1)*HR4(0,1,1,1)
     &  - 4.1666666666666666d-02*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.0823232337111381d+00*HR1(1)
     &  + 6.0102845157979714d-01*HR1(1)*HR1(1)
     &  - 2.7415567780803773d-01*HR1(1)*HR1(1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,1,1)
     &  - HR1(1) *HR4(-1,-1,-1,1)
     &  + HR1(1) *HR4(-1,-1,1,1)
     &  - HR1(1) *HR4(-1,1,1,1)
     &  + HR1(1) *HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,1,-1)
     &  + HR1(1) *HR4(0,-1,1,1)
     &  + HR1(1) *HR4(0,1,-1,-1)
     &  + HR1(1) *HR4(0,1,-1,1)
     &  + HR1(1) *HR4(0,1,1,-1)
     &  + HR1(1) *HR4(0,1,1,1)
     &  - HR5( -1,-1,-1,-1,1)
     &  + HR5( -1,-1,-1,1,1)
     &  - HR5( -1,-1,1,1,1)
     &  + HR5( -1,1,1,1,1)
     &  - HR5(0, -1,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,1)
     &  - HR5(0, -1,-1,1,-1)
     &  - HR5(0, -1,-1,1,1)
     &  - HR5(0, -1,1,-1,-1)
     &  - HR5(0, -1,1,-1,1)
     &  - HR5(0, -1,1,1,-1)
     &  - HR5(0, -1,1,1,1)
     &  - HR5(0,1, -1,-1,-1)
     &  - HR5(0,1, -1,-1,1)
     &  - HR5(0,1, -1,1,-1)
     &  - HR5(0,1, -1,1,1)
     &  - HR5(0,1,1, -1,-1)
     &  - HR5(0,1,1, -1,1)
     &  - HR5(0,1,1,1, -1)
     &  - HR5(0,1,1,1,1) 
      HY5(0,0,0,1,1) = 
     &  + 9.6551159989443734d-02
     &  - 2.7058080842778454d-01*HR1(-1)
     &  + 6.0102845157979714d-01*HR1(-1)*HR1(-1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(0)*HR3(0,1,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,1,1)
     &  + 1.2020569031595942d+00*HR1(-1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,1)
     &  - HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,1,1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR4(-1,1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR4(0,0,-1,1)
     &  + HR1( -1)*HR4(0,0,1,-1)
     &  + HR1( -1)*HR4(0,0,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR4(0,1,-1,1)
     &  + HR1( -1)*HR4(0,1,1,-1)
     &  - 8.3333333333333333d-02*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,1)
     &  - HR1(0) *HR1(1)*HR3(-1,-1,1)
     &  + HR1(0) *HR1(1)*HR3(-1,1,1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,1)
     &  - HR1(0) *HR1(1)*HR3(0,1,-1)
     &  - HR1(0) *HR1(1)*HR3(0,1,1)
     &  - HR1(0) *HR4(-1,-1,-1,1)
     &  + HR1(0) *HR4(-1,-1,1,1)
     &  - HR1(0) *HR4(-1,1,1,1)
     &  + HR1(0) *HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,-1,-1,1)
     &  + HR1(0) *HR4(0,-1,1,-1)
     &  + HR1(0) *HR4(0,-1,1,1)
     &  + HR1(0) *HR4(0,1,-1,-1)
     &  + HR1(0) *HR4(0,1,-1,1)
     &  + HR1(0) *HR4(0,1,1,-1)
     &  + HR1(0) *HR4(0,1,1,1)
     &  - 2.7058080842778454d-01*HR1(1)
     &  + 6.0102845157979714d-01*HR1(1)*HR1(1)
     &  - 4.0037751159850118d-02*HR1(1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,0,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1(1) *HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,1,-1)
     &  + HR1(1) *HR4(0,0,-1,-1)
     &  + HR1(1) *HR4(0,0,-1,1)
     &  + HR1(1) *HR4(0,0,1,-1)
     &  + HR1(1) *HR4(0,0,1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1(1) *HR4(0,1,-1,1)
     &  + HR1(1) *HR4(0,1,1,-1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR4(-1,1,1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,1,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,1,1)
     &  - 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  - HR5( -1,-1,1,1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - HR5(0, -1,-1,1,1)
     &  - 2.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  - HR5(0, -1,1,-1,1)
     &  - HR5(0, -1,1,1,-1)
     &  - HR5(0,0, -1,-1,-1)
     &  - HR5(0,0, -1,-1,1)
     &  - HR5(0,0, -1,1,-1)
     &  - HR5(0,0, -1,1,1)
     &  - HR5(0,0,1, -1,-1)
     &  - HR5(0,0,1, -1,1)
     &  - HR5(0,0,1,1, -1)
     &  - HR5(0,0,1,1,1) 
     &  - 3.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR5(0,1,-1,1,-1)
     &  - HR5(0,1, -1,1,1)
     &  - 2.0000000000000000d+00*HR5(0,1,1,-1,-1)
     &  - HR5(0,1,1, -1,1)
     &  - HR5(0,1,1,1, -1)
      HY5(0,0,1,0,1) = 
     &  + 2.2881039760335375d-01
     &  - 8.1174242528335364d-01*HR1(-1)
     &  - 1.7721476084810206d+00*HR1(-1)*HR1(-1)
     &  + 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,0,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(0)*HR1(1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,1,1)
     &  - 3.5442952169620413d+00*HR1(-1)*HR1(1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR2(0,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(0,1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(-1,1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,1,1,-1)
     &  - 8.2246703342411321d-01*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR4(-1,1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,1,1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,1,1,1)
     &  - 8.1174242528335364d-01*HR1(1)
     &  - 1.7721476084810206d+00*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  + HR1(1) *HR1(1)*HR3(-1,-1,1)
     &  + HR1(1) *HR1(1)*HR3(0,-1,-1)
     &  + HR1(1) *HR1(1)*HR3(0,0,-1)
     &  + HR1(1) *HR1(1)*HR3(0,0,1)
     &  + HR1(1) *HR1(1)*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  + HR1(1) *HR2(-1,1)*HR2(0,1)
     &  + 1.6449340668482264d+00*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(1) *HR2(0,-1)*HR2(0,1)
     &  + 1.6449340668482264d+00*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(0,1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,1,1)
     &  + 4.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,0,1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,1,1,-1)
     &  + HR2( -1,1)*HR3(-1,1,1)
     &  + HR2(0, -1)*HR3(-1,-1,1)
     &  - HR2(0, -1)*HR3(-1,1,1)
     &  + HR2(0, -1)*HR3(0,-1,-1)
     &  + HR2(0, -1)*HR3(0,-1,1)
     &  + HR2(0, -1)*HR3(0,1,-1)
     &  + HR2(0, -1)*HR3(0,1,1)
     &  + HR2(0,1) *HR3(-1,-1,1)
     &  - HR2(0,1) *HR3(-1,1,1)
     &  + HR2(0,1) *HR3(0,1,-1)
     &  + HR2(0,1) *HR3(0,1,1)
     &  - 1.6449340668482264d+00*HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR3(-1,1,1)
     &  - 1.6449340668482264d+00*HR3(0,-1,-1)
     &  - 1.6449340668482264d+00*HR3(0,-1,1)
     &  - 1.6449340668482264d+00*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR3(0,1,1)
     &  + 2.0794415416798359d+00*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR4(-1,1,1,1)
     &  - 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR4(0,-1,-1,1)
     &  - 2.0794415416798359d+00*HR4(0,-1,1,-1)
     &  - 2.0794415416798359d+00*HR4(0,-1,1,1)
     &  - 2.0794415416798359d+00*HR4(0,1,-1,-1)
     &  - 2.0794415416798359d+00*HR4(0,1,-1,1)
     &  - 2.0794415416798359d+00*HR4(0,1,1,-1)
     &  - 2.0794415416798359d+00*HR4(0,1,1,1)
     &  + 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  - HR5( -1,-1,1,-1,1)
     &  - HR5( -1,1,-1,1,1)
     &  + 7.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + HR5(0, -1,-1,0,1)
     &  + 5.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,1)
     &  - HR5(0, -1,0,-1,-1)
     &  + HR5(0, -1,0,1,1)
     &  + 5.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,1,-1,1)
     &  + HR5(0, -1,1,0,1)
     &  + 3.0000000000000000d+00*HR5(0,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HR5(0,0,-1,1,1)
     &  + 2.0000000000000000d+00*HR5(0,0,1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,0,1,-1,1)
     &  + 7.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  + 5.0000000000000000d+00*HR5(0,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,1,-1,1,1)
     &  - HR5(0,1,0,1, -1)
     &  - HR5(0,1,0,1,1) 
     &  + 5.0000000000000000d+00*HR5(0,1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,1,1,-1,1)
     &  + 3.0000000000000000d+00*HR5(0,1,1,1,-1)
      HY5(0,0,1,1,1) = 
     &  + 9.6551159989443734d-02
     &  - 1.0823232337111381d+00*HR1(-1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(-1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,1)
     &  + HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,0,1)
     &  - HR1( -1)*HR1(0)*HR3(0,1,-1)
     &  - 5.5504108664821579d-02*HR1(-1)*HR1(1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR4(0,0,0,-1)
     &  + HR1( -1)*HR4(0,0,0,1)
     &  + HR1( -1)*HR4(0,0,1,-1)
     &  + HR1( -1)*HR4(0,1,-1,-1)
     &  - 8.3333333333333333d-02*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,1,1)
     &  - 1.2011325347955035d-01*HR1(0)*HR1(1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,1)
     &  - HR1(0) *HR1(1)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(1)*HR3(0,0,-1)
     &  - HR1(0) *HR1(1)*HR3(0,0,1)
     &  - HR1(0) *HR1(1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  + HR1(0) *HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,-1,-1,1)
     &  + HR1(0) *HR4(0,-1,1,-1)
     &  + HR1(0) *HR4(0,0,-1,-1)
     &  + HR1(0) *HR4(0,0,-1,1)
     &  + HR1(0) *HR4(0,0,1,-1)
     &  + HR1(0) *HR4(0,0,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR4(0,1,-1,-1)
     &  + HR1(0) *HR4(0,1,-1,1)
     &  + HR1(0) *HR4(0,1,1,-1)
     &  - 1.0823232337111381d+00*HR1(1)
     &  - 2.7752054332410789d-02*HR1(1)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  - HR1(1) *HR4(-1,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,0,-1,-1)
     &  + HR1(1) *HR4(0,0,0,-1)
     &  + HR1(1) *HR4(0,0,0,1)
     &  + HR1(1) *HR4(0,0,1,-1)
     &  + HR1(1) *HR4(0,1,-1,-1)
     &  - 2.4022650695910071d-01*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR3(-1,1,1)
     &  - 2.4022650695910071d-01*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR3(0,-1,1)
     &  - 2.4022650695910071d-01*HR3(0,1,-1)
     &  - 2.4022650695910071d-01*HR3(0,1,1)
     &  - 1.3862943611198906d+00*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  + 1.3862943611198906d+00*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,1,1)
     &  + 1.3862943611198906d+00*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,1,1,-1)
     &  - 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + HR5( -1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,1)
     &  - HR5(0, -1,-1,1,-1)
     &  - HR5(0, -1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  - HR5(0,0, -1,-1,1)
     &  - HR5(0,0, -1,1,-1)
     &  - HR5(0,0,0, -1,-1)
     &  - HR5(0,0,0, -1,1)
     &  - HR5(0,0,0,1, -1)
     &  - HR5(0,0,0,1,1) 
     &  - 2.0000000000000000d+00*HR5(0,0,1,-1,-1)
     &  - HR5(0,0,1, -1,1)
     &  - HR5(0,0,1,1, -1)
     &  - 3.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  - HR5(0,1, -1,-1,1)
     &  - HR5(0,1, -1,1,-1)
     &  - HR5(0,1,1, -1,-1)
      HY5(0,1,0,1,1) = 
     &  + 2.2881039760335375d-01
     &  + 4.0801720544311065d+00*HR1(-1)
     &  - 6.0102845157979714d-01*HR1(-1)*HR1(-1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + HR1( -1)*HR1(-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 1.2020569031595942d+00*HR1(-1)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  - 1.2020569031595942d+00*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,0,1)
     &  + HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,0,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,0,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,0,0,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,1)
     &  + HR1(0) *HR1(0)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(0)*HR3(-1,1,1)
     &  + HR1(0) *HR1(0)*HR3(0,-1,-1)
     &  + HR1(0) *HR1(0)*HR3(0,-1,1)
     &  + HR1(0) *HR1(0)*HR3(0,1,-1)
     &  + HR1(0) *HR1(0)*HR3(0,1,1)
     &  + 1.2020569031595942d+00*HR1(0)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,0,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + HR1(0) *HR2(-1,1)*HR2(0,-1)
     &  + HR1(0) *HR2(-1,1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(0) *HR2(0,-1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR2(0,1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(0,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(0,1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,0,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR4(0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,1,1,-1)
     &  + 4.0801720544311065d+00*HR1(1)
     &  + 2.4022650695910071d-01*HR1(1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(1)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR1(1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,0,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,0,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,0,0,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + 1.2020569031595942d+00*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,1)
     &  - HR2( -1,1)*HR3(-1,-1,1)
     &  - HR2( -1,1)*HR3(0,-1,-1)
     &  - HR2( -1,1)*HR3(0,0,-1)
     &  - HR2( -1,1)*HR3(0,0,1)
     &  - HR2( -1,1)*HR3(0,1,-1)
     &  - 1.2020569031595942d+00*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR2(0,1)
     &  + HR2(0, -1)*HR3(0,0,-1)
     &  + HR2(0, -1)*HR3(0,0,1)
     &  - 1.2020569031595942d+00*HR2(0,1)
     &  - 3.4657359027997265d-01*HR2(0,1)*HR2(0,1)
     &  + HR2(0,1) *HR3(0,-1,-1)
     &  + HR2(0,1) *HR3(0,0,-1)
     &  + HR2(0,1) *HR3(0,0,1)
     &  + 4.8045301391820142d-01*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR3(-1,1,1)
     &  + 4.8045301391820142d-01*HR3(0,-1,-1)
     &  + 4.8045301391820142d-01*HR3(0,-1,1)
     &  + 4.8045301391820142d-01*HR3(0,1,-1)
     &  + 4.8045301391820142d-01*HR3(0,1,1)
     &  + 2.7725887222397812d+00*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR4(-1,-1,1,1)
     &  - 2.7725887222397812d+00*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR4(0,0,-1,-1)
     &  - 1.3862943611198906d+00*HR4(0,0,-1,1)
     &  - 1.3862943611198906d+00*HR4(0,0,1,-1)
     &  - 1.3862943611198906d+00*HR4(0,0,1,1)
     &  - 2.7725887222397812d+00*HR4(0,1,-1,-1)
     &  - 1.3862943611198906d+00*HR4(0,1,-1,1)
     &  - 1.3862943611198906d+00*HR4(0,1,1,-1)
     &  + 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + HR5( -1,-1,1,-1,1)
     &  + 7.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - HR5(0, -1,-1,0,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  + HR5(0, -1,0,-1,-1)
     &  - HR5(0, -1,0,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  + 7.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  + HR5(0,0, -1,-1,1)
     &  - HR5(0,0, -1,0,-1)
     &  - HR5(0,0, -1,0,1)
     &  + 2.0000000000000000d+00*HR5(0,0,-1,1,-1)
     &  + 5.0000000000000000d+00*HR5(0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,0,1,-1,1)
     &  - HR5(0,0,1,0, -1)
     &  - HR5(0,0,1,0,1) 
     &  + 4.0000000000000000d+00*HR5(0,0,1,1,-1)
     &  + 7.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,1,-1,1,-1)
     &  + HR5(0,1,0,1, -1)
     &  + 2.0000000000000000d+00*HR5(0,1,1,-1,-1)
      HY5(0,1,1,1,1) = 
     &  + 1.0369277551433699d+00
     &  - 9.6181291076284771d-03*HR1(-1)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(-1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.5504108664821579d-02*HR1(-1)*HR1(0)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  - 4.1666666666666666d-02*HR1(0)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR2(0,1)
     &  - 1.2011325347955035d-01*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,0,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  - 5.5504108664821579d-02*HR1(0)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(0)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,1,-1)
     &  - HR1(0) *HR4(-1,-1,-1,1)
     &  + HR1(0) *HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,0,-1,-1)
     &  + HR1(0) *HR4(0,0,0,-1)
     &  + HR1(0) *HR4(0,0,0,1)
     &  + HR1(0) *HR4(0,0,1,-1)
     &  + HR1(0) *HR4(0,1,-1,-1)
     &  - 9.6181291076284771d-03*HR1(1)
     &  - 5.5504108664821579d-02*HR2(-1,1)
     &  + 5.5504108664821579d-02*HR2(0,-1)
     &  + 5.5504108664821579d-02*HR2(0,1)
     &  - 2.4022650695910071d-01*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR3(0,0,-1)
     &  - 2.4022650695910071d-01*HR3(0,0,1)
     &  - 2.4022650695910071d-01*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,0,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,0,1)
     &  + 6.9314718055994530d-01*HR4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  - HR5( -1,-1,-1,-1,1)
     &  - HR5(0, -1,-1,-1,-1)
     &  - HR5(0,0, -1,-1,-1)
     &  - HR5(0,0,0, -1,-1)
     &  - HR5(0,0,0,0, -1)
     &  - HR5(0,0,0,0,1) 
     &  - HR5(0,0,0,1, -1)
     &  - HR5(0,0,1, -1,-1)
     &  - HR5(0,1, -1,-1,-1)
      if (r.lt.0d0) then 
      HY5(0,0,0,1,1) = HY5(0,0,0,1,1) 
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(1)*HR1(1)
     &  + 8.2246703342411321d-01*HR1(1)*HR1(1)*HR1(1)
      HY5(0,0,1,1,1) = HY5(0,0,1,1,1) 
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR1(1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR2(0,1)
     &  + 2.4674011002723396d+00*HR1(0)*HR1(1)*HR1(1)
     &  + 1.7102721159642791d+00*HR1(1)*HR1(1)
     &  + 4.9348022005446793d+00*HR1(1)*HR2(-1,1)
     &  - 4.9348022005446793d+00*HR1(1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(1)*HR2(0,1)
     &  + 4.9348022005446793d+00*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR3(-1,1,1)
     &  + 4.9348022005446793d+00*HR3(0,-1,-1)
     &  + 4.9348022005446793d+00*HR3(0,-1,1)
     &  + 4.9348022005446793d+00*HR3(0,1,-1)
     &  + 4.9348022005446793d+00*HR3(0,1,1)
      HY5(0,1,0,1,1) = HY5(0,1,0,1,1) 
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR2(0,-1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR2(0,1)
     &  - 4.9348022005446793d+00*HR1(1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(1)*HR2(0,-1)
     &  + 4.9348022005446793d+00*HR1(1)*HR2(0,1)
     &  - 9.8696044010893586d+00*HR3(-1,-1,1)
     &  + 9.8696044010893586d+00*HR3(-1,1,1)
     &  - 9.8696044010893586d+00*HR3(0,-1,-1)
     &  - 9.8696044010893586d+00*HR3(0,-1,1)
     &  - 9.8696044010893586d+00*HR3(0,1,-1)
     &  - 9.8696044010893586d+00*HR3(0,1,1)
      HY5(0,1,1,1,1) = HY5(0,1,1,1,1) 
     &  - 2.8732418312458363d+00*HR1(-1)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR1(0)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(0)*HR1(0)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(0)*HR1(1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR1(1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR2(-1,1)
     &  + 2.4674011002723396d+00*HR1(0)*HR1(0)*HR1(1)
     &  + 3.4205442319285582d+00*HR1(0)*HR1(1)
     &  + 4.9348022005446793d+00*HR1(0)*HR2(-1,1)
     &  - 4.9348022005446793d+00*HR1(0)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(0)*HR2(0,1)
     &  - 2.8732418312458363d+00*HR1(1)
     &  + 3.4205442319285582d+00*HR2(-1,1)
     &  - 3.4205442319285582d+00*HR2(0,-1)
     &  - 3.4205442319285582d+00*HR2(0,1)
     &  + 4.9348022005446793d+00*HR3(-1,-1,1)
     &  + 4.9348022005446793d+00*HR3(0,-1,-1)
     &  + 4.9348022005446793d+00*HR3(0,0,-1)
     &  + 4.9348022005446793d+00*HR3(0,0,1)
     &  + 4.9348022005446793d+00*HR3(0,1,-1)
      Hi5(0,0,0,0,1) = 
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(1)*HR1(1)*HR1(1)*HR1(1)
      Hi5(0,0,0,1,1) = 
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR3(0,-1,1)
     &  + HR1( -1)*HR3(0,1,-1)
     &  + HR1( -1)*HR3(0,1,1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  + HR1(1) *HR3(-1,-1,1)
     &  - HR1(1) *HR3(-1,1,1)
     &  + HR1(1) *HR3(0,-1,-1)
     &  + HR1(1) *HR3(0,-1,1)
     &  + HR1(1) *HR3(0,1,-1)
     &  + HR1(1) *HR3(0,1,1)
     &  + HR4( -1,-1,-1,1)
     &  - HR4( -1,-1,1,1)
     &  + HR4( -1,1,1,1)
     &  - HR4(0, -1,-1,-1)
     &  - HR4(0, -1,-1,1)
     &  - HR4(0, -1,1,-1)
     &  - HR4(0, -1,1,1)
     &  - HR4(0,1, -1,-1)
     &  - HR4(0,1, -1,1)
     &  - HR4(0,1,1, -1)
     &  - HR4(0,1,1,1) 
      Hi5(0,0,1,0,1) = 
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR1(1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,1,1)
     &  + 8.2246703342411321d-01*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(0,1,1)
     &  - 3.0000000000000000d+00*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR4(-1,1,1,1)
     &  + 3.0000000000000000d+00*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR4(0,-1,-1,1)
     &  + 3.0000000000000000d+00*HR4(0,-1,1,-1)
     &  + 3.0000000000000000d+00*HR4(0,-1,1,1)
     &  + 3.0000000000000000d+00*HR4(0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR4(0,1,-1,1)
     &  + 3.0000000000000000d+00*HR4(0,1,1,-1)
     &  + 3.0000000000000000d+00*HR4(0,1,1,1)
      Hi5(0,0,1,1,1) = 
     &  - 7.0235377994456286d-01*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - HR1( -1)*HR1(0)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR2(0,1)
     &  - 1.4047075598891257d+00*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(0,1)
     &  - HR1( -1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR3(-1,1,1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR3(0,0,-1)
     &  + HR1( -1)*HR3(0,0,1)
     &  + HR1( -1)*HR3(0,1,-1)
     &  + 2.5000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)
     &  + HR1(0) *HR1(1)*HR2(-1,1)
     &  - HR1(0) *HR1(1)*HR2(0,-1)
     &  - HR1(0) *HR1(1)*HR2(0,1)
     &  + HR1(0) *HR3(-1,-1,1)
     &  - HR1(0) *HR3(-1,1,1)
     &  + HR1(0) *HR3(0,-1,-1)
     &  + HR1(0) *HR3(0,-1,1)
     &  + HR1(0) *HR3(0,1,-1)
     &  + HR1(0) *HR3(0,1,1)
     &  - 7.0235377994456286d-01*HR1(1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(0,1)
     &  + HR1(1) *HR3(-1,-1,1)
     &  + HR1(1) *HR3(0,-1,-1)
     &  + HR1(1) *HR3(0,0,-1)
     &  + HR1(1) *HR3(0,0,1)
     &  + HR1(1) *HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR3(0,1,1)
     &  + 2.0000000000000000d+00*HR4(-1,-1,-1,1)
     &  - HR4( -1,-1,1,1)
     &  - 2.0000000000000000d+00*HR4(0,-1,-1,-1)
     &  - HR4(0, -1,-1,1)
     &  - HR4(0, -1,1,-1)
     &  - HR4(0,0, -1,-1)
     &  - HR4(0,0, -1,1)
     &  - HR4(0,0,1, -1)
     &  - HR4(0,0,1,1) 
     &  - 2.0000000000000000d+00*HR4(0,1,-1,-1)
     &  - HR4(0,1, -1,1)
     &  - HR4(0,1,1, -1)
      Hi5(0,1,0,1,1) = 
     &  - 1.2020569031595942d+00*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + HR1( -1)*HR1(0)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR2(0,1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,1,-1)
     &  - HR1(0) *HR1(1)*HR2(-1,1)
     &  + HR1(0) *HR1(1)*HR2(0,-1)
     &  + HR1(0) *HR1(1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR3(0,1,1)
     &  - 1.2020569031595942d+00*HR1(1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(0,0,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)
     &  - HR2( -1,1)*HR2(0,-1)
     &  - HR2( -1,1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR2(0,-1)*HR2(0,-1)
     &  + HR2(0, -1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR2(0,1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR3(0,-1,1)
     &  - 1.3862943611198906d+00*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR3(0,1,1)
     &  - 4.0000000000000000d+00*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR4(-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HR4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HR4(0,0,1,1)
     &  + 4.0000000000000000d+00*HR4(0,1,-1,-1)
     &  + 2.0000000000000000d+00*HR4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HR4(0,1,1,-1)
      Hi5(0,1,1,1,1) = 
     &  - 1.0846773019780311d+00*HR1(-1)
     &  + 7.0235377994456286d-01*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.4047075598891257d+00*HR1(-1)*HR1(0)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)
     &  + 1.4047075598891257d+00*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  - HR1( -1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(0,1)
     &  - 1.4047075598891257d+00*HR1(0)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(0,1)
     &  + HR1(0) *HR3(-1,-1,1)
     &  + HR1(0) *HR3(0,-1,-1)
     &  + HR1(0) *HR3(0,0,-1)
     &  + HR1(0) *HR3(0,0,1)
     &  + HR1(0) *HR3(0,1,-1)
     &  - 1.0846773019780311d+00*HR1(1)
     &  - 1.4047075598891257d+00*HR2(-1,1)
     &  + 1.4047075598891257d+00*HR2(0,-1)
     &  + 1.4047075598891257d+00*HR2(0,1)
     &  + 6.9314718055994530d-01*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR3(0,1,-1)
     &  + HR4( -1,-1,-1,1)
     &  - HR4(0, -1,-1,-1)
     &  - HR4(0,0, -1,-1)
     &  - HR4(0,0,0, -1)
     &  - HR4(0,0,0,1) 
     &  - HR4(0,0,1, -1)
     &  - HR4(0,1, -1,-1)
      endif 
      endif 
** nw > 4 endif 
      endif 
** (n1,n2) = (0,1) or (-1,1) endif 
************ 
** (n1,n2) = (-1,0) or (-1,1) 
      if (    ( (n1.eq.-1).and.(n2.eq.0) ) 
     &    .or.( (n1.eq.-1).and.(n2.eq.1) ) ) then 
       HY2(0,-1) = 
     &  + 8.2246703342411321d-01
     &  - 6.9314718055994530d-01*HR1(-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)
     &  + HR1( -1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(1)
     &  - HR2( -1,1)
      if ( nw.gt.2 ) then 
      HY3(0,0,-1) = 
     &  + 9.0154267736969571d-01
     &  - 8.2246703342411321d-01*HR1(-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 8.2246703342411321d-01*HR1(1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)
     &  + HR1(1) *HR2(-1,1)
     &  + HR3( -1,-1,1)
     &  - HR3( -1,1,1)
      HY3(0,-1,-1) = 
     &  + 1.5025711289494928d-01
     &  - 2.4022650695910071d-01*HR1(-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)
     &  + HR1( -1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(1)
     &  - 6.9314718055994530d-01*HR2(-1,1)
     &  - HR3( -1,-1,1)
      endif 
      if ( nw.gt.3 ) then 
      HY4(0,0,0,-1) = 
     &  + 9.4703282949724591d-01
     &  - 9.0154267736969571d-01*HR1(-1)
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 9.0154267736969571d-01*HR1(1)
     &  + 4.1123351671205660d-01*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - HR1(1) *HR3(-1,-1,1)
     &  + HR1(1) *HR3(-1,1,1)
     &  - HR4( -1,-1,-1,1)
     &  + HR4( -1,-1,1,1)
     &  - HR4( -1,1,1,1)
      HY4(0,0,-1,-1) = 
     &  + 8.7785671568655302d-02
     &  - 1.5025711289494928d-01*HR1(-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR3(-1,1,1)
     &  - 1.5025711289494928d-01*HR1(1)
     &  + 1.2011325347955035d-01*HR1(1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)
     &  + HR1(1) *HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR4(-1,-1,-1,1)
     &  - HR4( -1,-1,1,1)
      HY4(0,-1,-1,-1) = 
     &  + 2.3752366322618485d-02
     &  - 5.5504108664821579d-02*HR1(-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR3(-1,-1,1)
     &  - 5.5504108664821579d-02*HR1(1)
     &  - 2.4022650695910071d-01*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR3(-1,-1,1)
     &  - HR4( -1,-1,-1,1)
      endif 
** nw > 3 endif 
      if ( nw.gt.4 ) then
      HY5(0,0,0,0,-1) = 
     &  + 9.7211977044690930d-01
     &  - 9.4703282949724591d-01*HR1(-1)
     &  + 4.5077133868484785d-01*HR1(-1)*HR1(-1)
     &  - 1.3707783890401886d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 9.0154267736969571d-01*HR1(-1)*HR1(1)
     &  - 4.1123351671205660d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 9.4703282949724591d-01*HR1(1)
     &  + 4.5077133868484785d-01*HR1(1)*HR1(1)
     &  - 1.3707783890401886d-01*HR1(1)*HR1(1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  + HR1(1) *HR4(-1,-1,-1,1)
     &  - HR1(1) *HR4(-1,-1,1,1)
     &  + HR1(1) *HR4(-1,1,1,1)
     &  + HR5( -1,-1,-1,-1,1)
     &  - HR5( -1,-1,-1,1,1)
     &  + HR5( -1,-1,1,1,1)
     &  - HR5( -1,1,1,1,1)
      HY5(0,0,0,-1,-1) = 
     &  + 4.8936397049969063d-02
     &  - 8.7785671568655302d-02*HR1(-1)
     &  + 7.5128556447474642d-02*HR1(-1)*HR1(-1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.5025711289494928d-01*HR1(-1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(-1,1,1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR4(-1,1,1,1)
     &  - 8.7785671568655302d-02*HR1(1)
     &  + 7.5128556447474642d-02*HR1(1)*HR1(1)
     &  - 4.0037751159850118d-02*HR1(1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1(1) *HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR4(-1,1,1,1)
     &  - 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  - HR5( -1,-1,1,1,1)
      HY5(0,0,-1,0,-1) = 
     &  + 9.2748467341632644d-02
     &  - 1.6265466739742008d-01*HR1(-1)
     &  + 1.3478823976576390d-01*HR1(-1)*HR1(-1)
     &  - 1.3707783890401886d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 2.6957647953152780d-01*HR1(-1)*HR1(1)
     &  - 4.1123351671205660d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(-1,1,1,1)
     &  - 1.6265466739742008d-01*HR1(1)
     &  + 1.3478823976576390d-01*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + HR1(1) *HR1(1)*HR3(-1,-1,1)
     &  + 8.2246703342411321d-01*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(-1,1,1)
     &  + 4.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(-1,-1,1,1)
     &  + HR2( -1,1)*HR3(-1,1,1)
     &  + 8.2246703342411321d-01*HR3(-1,-1,1)
     &  - 8.2246703342411321d-01*HR3(-1,1,1)
     &  + 2.0794415416798359d+00*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR4(-1,1,1,1)
     &  + 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  - HR5( -1,-1,1,-1,1)
     &  - HR5( -1,1,-1,1,1)
      HY5(0,0,-1,-1,-1) = 
     &  + 9.6015684431298325d-03
     &  - 2.3752366322618485d-02*HR1(-1)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(-1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,1,1)
     &  - 2.3752366322618485d-02*HR1(1)
     &  + 2.7752054332410789d-02*HR1(1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  + HR1(1) *HR4(-1,-1,-1,1)
     &  + 2.4022650695910071d-01*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - HR5( -1,-1,-1,1,1)
      HY5(0,-1,0,-1,-1) = 
     &  + 1.3531263989594243d-02
     &  - 3.2893195194356041d-02*HR1(-1)
     &  + 7.5128556447474642d-02*HR1(-1)*HR1(-1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - HR1( -1)*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR3(-1,1,1)
     &  + 1.5025711289494928d-01*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(-1,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 3.2893195194356041d-02*HR1(1)
     &  - 2.4022650695910071d-01*HR1(1)*HR2(-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 1.5025711289494928d-01*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
     &  + HR2( -1,1)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR3(-1,-1,1)
     &  + 4.8045301391820142d-01*HR3(-1,1,1)
     &  - 2.7725887222397812d+00*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR4(-1,-1,1,1)
     &  - 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - HR5( -1,-1,1,-1,1)
       HY5(0,-1,-1,-1,-1) = 
     &  + 3.1350096016808622d-03
     &  - 9.6181291076284771d-03*HR1(-1)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(-1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  - 9.6181291076284771d-03*HR1(1)
     &  - 5.5504108664821579d-02*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - HR5( -1,-1,-1,-1,1)
      endif 
** nw > 4 endif 
      endif 
** (n1,n2) = (-1,0) or (-1,1) endif 
** (n1,n2) = (-1,1) -- completion 
      if ( (n1.eq.-1).and.(n2.eq.1) ) then 
      HY2(-1,1) = 
     &  + 5.8224052646501250d-01
     &  + 6.9314718055994530d-01*HR1(-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)
     &  + HR1( -1)*HR1(0)
     &  - HR2(0, -1)
      if (r.lt.0d0) then 
      Hi2(-1,1) = 
     &  - HR1( -1) 
      endif 
      if ( nw.gt.2 ) then 
      HY3(0,-1,1) = 
     &  + 2.4307035167006157d-01
     &  - 5.8224052646501250d-01*HR1(-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR2(-1,1)
     &  + HR1( -1)*HR2(0,-1)
     &  + HR1(0) *HR2(-1,1)
     &  - 5.8224052646501250d-01*HR1(1)
     &  + HR1(1) *HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)
     &  + HR3( -1,-1,1)
     &  - HR3(0, -1,-1)
     &  - HR3(0, -1,1)
      HY3(0,1,-1) = 
     &  + 5.0821521280468485d-01
     &  + 1.0626935403832139d+00*HR1(-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR2(-1,1)
     &  - HR1( -1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)
     &  + 1.0626935403832139d+00*HR1(1)
     &  - HR1(1) *HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(0,1)
     &  + HR3( -1,-1,1)
     &  + 2.0000000000000000d+00*HR3(0,-1,-1)
     &  + HR3(0, -1,1)
     &  + HR3(0,1, -1)
      HY3(-1,-1,1) = 
     &  + 9.4753004230127705d-02
     &  - 5.8224052646501250d-01*HR1(-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + HR1( -1)*HR2(0,-1)
     &  - HR3(0, -1,-1)
      HY3(-1,1,1) = 
     &  + 5.3721319360804020d-01
     &  - 2.4022650695910071d-01*HR1(-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)
     &  + HR1(0) *HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)
     &  - HR3(0, -1,-1)
     &  - HR3(0,0, -1)
      if (r.lt.0d0) then 
      HY3(-1,1,1) = HY3(-1,1,1) 
     &  + 4.9348022005446793d+00*HR1(-1)
      Hi3(0,-1,1) = 
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1) 
     &  + HR1( -1)*HR1(1) 
     &  - HR2( -1,1) 
      Hi3(0,1,-1) = 
     &  - 6.9314718055994530d-01*HR1(-1) 
     &  - 6.9314718055994530d-01*HR1(1) 
      Hi3(-1,-1,1) = 
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1) 
      Hi3(-1,1,1) = 
     &  + 6.9314718055994530d-01*HR1(-1) 
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1) 
     &  + HR1( -1)*HR1(0) 
     &  - HR2(0, -1) 
      endif 
      endif 
      if ( nw.gt.3 ) then 
      HY4(0,0,-1,1) = 
     &  + 1.1787599965050932d-01
     &  - 2.4307035167006157d-01*HR1(-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR3(-1,1,1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR3(0,-1,1)
     &  - HR1(0) *HR1(1)*HR2(-1,1)
     &  - HR1(0) *HR3(-1,-1,1)
     &  + HR1(0) *HR3(-1,1,1)
     &  - 2.4307035167006157d-01*HR1(1)
     &  + 2.9112026323250625d-01*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)
     &  - HR1(1) *HR3(-1,-1,1)
     &  + HR1(1) *HR3(0,-1,-1)
     &  + HR1(1) *HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR4(-1,-1,-1,1)
     &  + HR4( -1,-1,1,1)
     &  - HR4(0, -1,-1,-1)
     &  - HR4(0, -1,-1,1)
     &  - HR4(0, -1,1,-1)
     &  - HR4(0, -1,1,1)
      HY4(0,0,1,-1) = 
     &  + 1.7284527823898438d-01
     &  - 5.0821521280468485d-01*HR1(-1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(0,1)
     &  + HR1( -1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR3(0,-1,1)
     &  - HR1( -1)*HR3(0,1,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)
     &  - 5.0821521280468485d-01*HR1(1)
     &  - 5.3134677019160696d-01*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(0,1)
     &  - HR1(1) *HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(0,-1,-1)
     &  - HR1(1) *HR3(0,-1,1)
     &  - HR1(1) *HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR3(0,1,1)
     &  - 2.0000000000000000d+00*HR4(-1,-1,-1,1)
     &  + HR4( -1,-1,1,1)
     &  + 3.0000000000000000d+00*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR4(0,-1,1,-1)
     &  + HR4(0, -1,1,1)
     &  + 2.0000000000000000d+00*HR4(0,1,-1,-1)
     &  + HR4(0,1, -1,1)
     &  + HR4(0,1,1, -1)
      HY4(0,-1,0,1) = 
     &  + 2.0293560632083841d-01
     &  - 3.8889584616810632d-01*HR1(-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR3(0,1,-1)
     &  + HR1(0) *HR1(1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR3(-1,1,1)
     &  - 3.8889584616810632d-01*HR1(1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR3(-1,-1,1)
     &  + HR1(1) *HR3(0,-1,-1)
     &  + HR1(1) *HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)
     &  + HR2( -1,1)*HR2(0,-1)
     &  + HR2( -1,1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR3(-1,1,1)
     &  + 4.0000000000000000d+00*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR4(-1,-1,1,1)
     &  - HR4(0, -1,-1,-1)
     &  - HR4(0, -1,-1,1)
     &  - HR4(0,1, -1,-1)
     &  - HR4(0,1, -1,1)
      HY4(0,-1,-1,1) = 
     &  + 3.4159126166513913d-02
     &  - 9.4753004230127705d-02*HR1(-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  - HR1( -1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  + HR1(0) *HR3(-1,-1,1)
     &  - 9.4753004230127705d-02*HR1(1)
     &  + HR1(1) *HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR2(-1,1)
     &  + HR2( -1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR3(-1,-1,1)
     &  + HR4( -1,-1,-1,1)
     &  - HR4(0, -1,-1,-1)
     &  - HR4(0, -1,-1,1)
      HY4(0,-1,1,-1) = 
     &  + 5.4653052738263652d-02
     &  - 2.1407237086670622d-01*HR1(-1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(1)
     &  + HR1( -1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)
     &  - HR1( -1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(-1,1)
     &  - 2.1407237086670622d-01*HR1(1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR2(-1,1)
     &  - HR2( -1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR3(0,-1,1)
     &  + HR4( -1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR4(0,-1,-1,1)
     &  + HR4(0, -1,1,-1)
      HY4(0,1,-1,-1) = 
     &  + 1.1412342741606084d-01
     &  + 4.7533770109129867d-01*HR1(-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)
     &  - HR1( -1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR1(1)
     &  + 4.7533770109129867d-01*HR1(1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(0,-1)
     &  + HR1(1) *HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR2(0,1)
     &  + 6.9314718055994530d-01*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR3(0,1,-1)
     &  + HR4( -1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR4(0,-1,-1,-1)
     &  - HR4(0, -1,-1,1)
     &  - HR4(0, -1,1,-1)
     &  - HR4(0,1, -1,-1)
      HY4(0,-1,1,1) = 
     &  + 9.3097125991768577d-02
     &  - 5.3721319360804020d-01*HR1(-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)
     &  + HR1( -1)*HR1(0)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)
     &  + HR1( -1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  - HR1(0) *HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(-1,1)
     &  - HR1(0) *HR3(-1,-1,1)
     &  + HR1(0) *HR3(0,-1,-1)
     &  + HR1(0) *HR3(0,-1,1)
     &  - 5.3721319360804020d-01*HR1(1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(0,-1)
     &  + HR1(1) *HR3(0,-1,-1)
     &  + HR1(1) *HR3(0,0,-1)
     &  - 2.4022650695910071d-01*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR3(0,-1,1)
     &  - HR4( -1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR4(0,-1,-1,-1)
     &  - HR4(0, -1,-1,1)
     &  - HR4(0, -1,1,-1)
     &  - HR4(0,0, -1,-1)
     &  - HR4(0,0, -1,1)
      HY4(0,1,-1,1) = 
     &  + 1.9355535381306524d-01
     &  + 1.4780047665430420d+00*HR1(-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(0)
     &  + HR1( -1)*HR1(0)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR2(0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(1)
     &  + HR1( -1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)
     &  + HR1( -1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(0)*HR1(1)
     &  + HR1(0) *HR1(1)*HR2(0,-1)
     &  - HR1(0) *HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR3(0,-1,-1)
     &  - HR1(0) *HR3(0,-1,1)
     &  - HR1(0) *HR3(0,1,-1)
     &  + 1.4780047665430420d+00*HR1(1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR2(-1,1)
     &  - HR2( -1,1)*HR2(0,-1)
     &  - 5.8224052646501250d-01*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR2(0,-1)*HR2(0,-1)
     &  + HR2(0, -1)*HR2(0,1)
     &  - 5.8224052646501250d-01*HR2(0,1)
     &  - 6.9314718055994530d-01*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR3(0,1,-1)
     &  - HR4( -1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR4(0,-1,-1,1)
     &  - HR4(0, -1,0,1)
     &  + HR4(0, -1,1,-1)
     &  + 2.0000000000000000d+00*HR4(0,0,-1,-1)
     &  + HR4(0,1, -1,-1)
      HY4(0,1,1,-1) = 
     &  + 4.3369237704895519d-01
     &  - 1.1073038989294665d+00*HR1(-1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(0)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR3(0,0,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)
     &  - 1.0626935403832139d+00*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(0,1)
     &  - 1.1073038989294665d+00*HR1(1)
     &  + HR1(1) *HR3(0,-1,-1)
     &  + HR1(1) *HR3(0,0,-1)
     &  - 1.0626935403832139d+00*HR2(-1,1)
     &  + HR2( -1,1)*HR2(0,-1)
     &  + 1.0626935403832139d+00*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR2(0,-1)*HR2(0,-1)
     &  - HR2(0, -1)*HR2(0,1)
     &  + 1.0626935403832139d+00*HR2(0,1)
     &  - 6.9314718055994530d-01*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR3(0,1,-1)
     &  - HR4( -1,-1,-1,1)
     &  - HR4(0, -1,-1,1)
     &  + HR4(0, -1,0,1)
     &  + HR4(0,0, -1,1)
     &  + HR4(0,0,1, -1)
     &  + HR4(0,1, -1,-1)
      HY4(-1,-1,-1,1) = 
     &  + 1.4134237214990008d-02
     &  - 9.4753004230127705d-02*HR1(-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  - HR4(0, -1,-1,-1)
      HY4(-1,-1,1,1) = 
     &  + 4.0758239159309251d-02
     &  - 5.3721319360804020d-01*HR1(-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - HR1( -1)*HR1(0)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR3(0,0,-1)
     &  + HR1(0) *HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR4(0,-1,-1,-1)
     &  - HR4(0,0, -1,-1)
      HY4(-1,1,1,1) = 
     &  + 5.1747906167389938d-01
     &  + 5.5504108664821579d-02*HR1(-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(0,-1)
     &  + HR1(0) *HR3(0,-1,-1)
     &  + HR1(0) *HR3(0,0,-1)
     &  - 2.4022650695910071d-01*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR3(0,0,-1)
     &  - HR4(0, -1,-1,-1)
     &  - HR4(0,0, -1,-1)
     &  - HR4(0,0,0, -1)
      if (r.lt.0d0) then 
      HY4(0,-1,1,1) = HY4(0,-1,1,1) 
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(1)
     &  + 4.9348022005446793d+00*HR2(-1,1)
      HY4(0,1,1,-1) = HY4(0,1,1,-1) 
     &  + 3.4205442319285582d+00*HR1(-1)
     &  + 3.4205442319285582d+00*HR1(1)
      HY4(-1,-1,1,1) = HY4(-1,-1,1,1) 
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1) 
      HY4(-1,1,1,1) = HY4(-1,1,1,1)
     &  - 3.4205442319285582d+00*HR1(-1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(0)
     &  + 4.9348022005446793d+00*HR2(0,-1)
      Hi4(0,0,-1,1) = 
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1) 
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1) 
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1) 
     &  + HR1(1) *HR2(-1,1) 
     &  + HR3( -1,-1,1) 
     &  - HR3( -1,1,1) 
      Hi4(0,0,1,-1) = 
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1) 
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1) 
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1) 
      Hi4(0,-1,0,1) = 
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1) 
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1) 
     &  + HR1( -1)*HR2(-1,1) 
     &  - HR1(1) *HR2(-1,1) 
     &  - 2.0000000000000000d+00*HR3(-1,-1,1) 
     &  + 2.0000000000000000d+00*HR3(-1,1,1) 
      Hi4(0,-1,-1,1) = 
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1) 
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1) 
     &  + HR1( -1)*HR2(-1,1) 
     &  - HR3( -1,-1,1) 
      Hi4(0,-1,1,-1) = 
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1) 
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1) 
     &  - 6.9314718055994530d-01*HR2(-1,1) 
      Hi4(0,1,-1,-1) =  
     &  - 2.4022650695910071d-01*HR1(-1) 
     &  - 2.4022650695910071d-01*HR1(1) 
      Hi4(0,-1,1,1) = 
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR2(-1,1)
     &  + HR1( -1)*HR2(0,-1)
     &  + HR1(0) *HR2(-1,1)
     &  + HR1(1) *HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)
     &  + HR3( -1,-1,1)
     &  - HR3(0, -1,-1)
     &  - HR3(0, -1,1)
      Hi4(0,1,-1,1) = 
     &  - 5.8224052646501250d-01*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR2(-1,1)
     &  - HR1( -1)*HR2(0,-1)
     &  - 5.8224052646501250d-01*HR1(1)
     &  - HR1(1) *HR2(0,-1)
     &  + HR3( -1,-1,1)
     &  + 2.0000000000000000d+00*HR3(0,-1,-1)
     &  + HR3(0, -1,1)
     &  + HR3(0,1, -1)
      Hi4(0,1,1,-1) = 
     &  + 1.0626935403832139d+00*HR1(-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)
     &  + 1.0626935403832139d+00*HR1(1)
     &  + 6.9314718055994530d-01*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(0,1)
      Hi4(-1,-1,-1,1) = 
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1) 
      Hi4(-1,-1,1,1) = 
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + HR1( -1)*HR2(0,-1)
     &  - HR3(0, -1,-1)
      Hi4(-1,1,1,1) = 
     &  + 1.4047075598891257d+00*HR1(-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)
     &  + HR1(0) *HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)
     &  - HR3(0, -1,-1)
     &  - HR3(0,0, -1)
      endif 
      endif 
** nw > 3 endif 
      if ( nw.gt.4 ) then 
      HY5(-1,-1,-1,-1,1) = 
     &  + 1.8016537870380179d-03
     &  - 1.4134237214990008d-02*HR1(-1)
     &  + 4.7376502115063852d-02*HR1(-1)*HR1(-1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,-1)
      HY5(-1,-1,-1,1,1) = 
     &  + 3.8760673146652637d-03
     &  - 4.0758239159309251d-02*HR1(-1)
     &  + 2.6860659680402010d-01*HR1(-1)*HR1(-1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,0,-1,-1)
     &  + HR1(0) *HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - HR5(0,0, -1,-1,-1)
      HY5(-1,-1,1,-1,1) = 
     &  + 6.2154684604081354d-03
     &  - 8.7985537010508960d-02*HR1(-1)
     &  - 7.3900238327152102d-01*HR1(-1)*HR1(-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR2(0, -1)*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR3(0,-1,-1)
     &  - 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - HR5(0, -1,0,-1,-1)
      HY5(-1,-1,1,1,1) = 
     &  + 1.8530786065466613d-02
     &  - 5.1747906167389938d-01*HR1(-1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(-1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,0,-1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,0,-1,-1)
     &  - 2.4022650695910071d-01*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  - HR5(0,0,0, -1,-1)
      HY5(-1,1,-1,1,1) = 
     &  + 3.8880058841843904d-02
     &  + 1.9248049955307152d+00*HR1(-1)
     &  - 2.6860659680402010d-01*HR1(-1)*HR1(-1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 5.3721319360804020d-01*HR1(-1)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,0,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,0,0,-1)
     &  + HR1(0) *HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(0,-1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,0,-1,-1)
     &  - 5.3721319360804020d-01*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR2(0,-1)*HR2(0,-1)
     &  + HR2(0, -1)*HR3(0,0,-1)
     &  + 4.8045301391820142d-01*HR3(0,-1,-1)
     &  - 2.7725887222397812d+00*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR4(0,0,-1,-1)
     &  + 7.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + HR5(0, -1,0,-1,-1)
     &  + 7.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  - HR5(0,0, -1,0,-1)
      HY5(-1,1,1,1,1) = 
     &  + 5.0840057924226870d-01
     &  - 9.6181291076284771d-03*HR1(-1)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(-1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 5.5504108664821579d-02*HR1(-1)*HR1(0)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,0,-1)
     &  + HR1(0) *HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,0,-1,-1)
     &  + HR1(0) *HR4(0,0,0,-1)
     &  + 5.5504108664821579d-02*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,0,-1)
     &  - HR5(0, -1,-1,-1,-1)
     &  - HR5(0,0, -1,-1,-1)
     &  - HR5(0,0,0, -1,-1)
     &  - HR5(0,0,0,0, -1)
      HY5(0,-1,-1,-1,1) = 
     &  + 4.1914400448554060d-03
     &  - 1.4134237214990008d-02*HR1(-1)
     &  + 4.7376502115063852d-02*HR1(-1)*HR1(-1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  + 9.4753004230127705d-02*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(-1,-1,-1,1)
     &  - 1.4134237214990008d-02*HR1(1)
     &  + HR1(1) *HR4(0,-1,-1,-1)
     &  - 9.4753004230127705d-02*HR2(-1,1)
     &  + HR2( -1,1)*HR3(0,-1,-1)
     &  + HR2(0, -1)*HR3(-1,-1,1)
     &  - 5.8224052646501250d-01*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + HR5( -1,-1,-1,-1,1)
     &  - HR5(0, -1,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,1)
      HY5(0,-1,-1,0,1) = 
     &  + 3.0172237496701167d-02
     &  - 7.7340900566758219d-02*HR1(-1)
     &  + 1.9444792308405316d-01*HR1(-1)*HR1(-1)
     &  - 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  + 3.8889584616810632d-01*HR1(-1)*HR1(1)
     &  + HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR2(-1,1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,1,-1,-1)
     &  - HR1(0) *HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - 7.7340900566758219d-02*HR1(1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,1,-1,-1)
     &  - 3.8889584616810632d-01*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR3(-1,-1,1)
     &  + HR2( -1,1)*HR3(0,-1,-1)
     &  + HR2( -1,1)*HR3(0,1,-1)
     &  + HR2(0, -1)*HR3(-1,-1,1)
     &  + HR2(0,1) *HR3(-1,-1,1)
     &  - 1.6449340668482264d+00*HR3(-1,-1,1)
     &  - 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR5(-1,-1,1,-1,1)
     &  - HR5(0, -1,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,1)
     &  - HR5(0,1, -1,-1,-1)
     &  - HR5(0,1, -1,-1,1)
      HY5(0,-1,-1,1,-1) = 
     &  + 5.9459097989450212d-03
     &  - 2.3275066086727564d-02*HR1(-1)
     &  + 1.0703618543335311d-01*HR1(-1)*HR1(-1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 2.1407237086670622d-01*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  - 2.3275066086727564d-02*HR1(1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.1407237086670622d-01*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  + 1.0626935403832139d+00*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  + HR5( -1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + HR5(0, -1,-1,1,-1)
      HY5(0,-1,-1,1,1) = 
     &  + 8.7734377821481916d-03
     &  - 4.0758239159309251d-02*HR1(-1)
     &  + 2.6860659680402010d-01*HR1(-1)*HR1(-1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.3721319360804020d-01*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  - HR1(0) *HR4(-1,-1,-1,1)
     &  + HR1(0) *HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,-1,-1,1)
     &  - 4.0758239159309251d-02*HR1(1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,0,-1,-1)
     &  - 5.3721319360804020d-01*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  + HR2( -1,1)*HR3(0,-1,-1)
     &  + HR2( -1,1)*HR3(0,0,-1)
     &  - 2.4022650695910071d-01*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  - HR5( -1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - HR5(0, -1,-1,1,-1)
     &  - HR5(0,0, -1,-1,-1)
     &  - HR5(0,0, -1,-1,1)
      HY5(0,-1,0,-1,1) = 
     &  + 1.7042475614121991d-02
     &  - 4.5512223866526096d-02*HR1(-1)
     &  + 1.2153517583503078d-01*HR1(-1)*HR1(-1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR1(-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 2.4307035167006157d-01*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(-1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(-1,-1,1,1)
     &  - 4.5512223866526096d-02*HR1(1)
     &  - 5.8224052646501250d-01*HR1(1)*HR2(-1,1)
     &  + HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,1,-1)
     &  - 2.4307035167006157d-01*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
     &  - HR2( -1,1)*HR3(-1,-1,1)
     &  + HR2( -1,1)*HR3(0,-1,-1)
     &  + HR2( -1,1)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR3(-1,1,1)
     &  - 1.1644810529300250d+00*HR3(-1,-1,1)
     &  + 1.1644810529300250d+00*HR3(-1,1,1)
     &  + 2.7725887222397812d+00*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR4(-1,-1,1,1)
     &  + 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + HR5( -1,-1,1,-1,1)
     &  - HR5(0, -1,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,1)
     &  - HR5(0, -1,1,-1,-1)
     &  - HR5(0, -1,1,-1,1)
      HY5(0,-1,0,1,-1) = 
     &  + 2.2495758621687517d-02
     &  - 6.9368034302854577d-02*HR1(-1)
     &  + 2.5410760640234242d-01*HR1(-1)*HR1(-1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + HR1( -1)*HR1(-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 5.0821521280468485d-01*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR3(-1,1,1)
     &  - 6.9368034302854577d-02*HR1(1)
     &  + 1.0626935403832139d+00*HR1(1)*HR2(-1,1)
     &  - HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(1) *HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - 5.0821521280468485d-01*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,1)
     &  - HR2( -1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR2( -1,1)*HR3(0,-1,1)
     &  - HR2( -1,1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR3(-1,1,1)
     &  + 2.1253870807664278d+00*HR3(-1,-1,1)
     &  - 2.1253870807664278d+00*HR3(-1,1,1)
     &  + 2.7725887222397812d+00*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,1)
     &  + 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + HR5( -1,-1,1,-1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + HR5(0, -1,-1,1,-1)
     &  + HR5(0, -1,1,-1,-1)
     &  + HR5(0, -1,1,-1,1)
     &  + 3.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  + HR5(0,1, -1,1,-1)
      HY5(0,-1,0,1,1) = 
     &  + 3.0833054551948363d-02
     &  - 1.1285749644390297d-01*HR1(-1)
     &  + 6.0102845157979714d-01*HR1(-1)*HR1(-1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - HR1( -1)*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,1,-1)
     &  + 1.2020569031595942d+00*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,1)
     &  - HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - HR1(0) *HR1(0)*HR3(-1,-1,1)
     &  + HR1(0) *HR1(0)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - HR1(0) *HR2(-1,1)*HR2(0,-1)
     &  - HR1(0) *HR2(-1,1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(-1,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR4(-1,-1,1,1)
     &  + HR1(0) *HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,-1,-1,1)
     &  + HR1(0) *HR4(0,1,-1,-1)
     &  + HR1(0) *HR4(0,1,-1,1)
     &  - 1.1285749644390297d-01*HR1(1)
     &  - 2.4022650695910071d-01*HR1(1)*HR2(-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,0,-1,-1)
     &  + HR1(1) *HR4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - 1.2020569031595942d+00*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR2(0,1)
     &  + HR2( -1,1)*HR3(-1,-1,1)
     &  + HR2( -1,1)*HR3(0,-1,-1)
     &  + HR2( -1,1)*HR3(0,0,-1)
     &  + HR2( -1,1)*HR3(0,0,1)
     &  + HR2( -1,1)*HR3(0,1,-1)
     &  - 4.8045301391820142d-01*HR3(-1,-1,1)
     &  + 4.8045301391820142d-01*HR3(-1,1,1)
     &  - 2.7725887222397812d+00*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,1)
     &  - 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - HR5( -1,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - HR5(0, -1,-1,1,-1)
     &  - HR5(0,0, -1,-1,-1)
     &  - HR5(0,0, -1,-1,1)
     &  - HR5(0,0,1, -1,-1)
     &  - HR5(0,0,1, -1,1)
     &  - 3.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  - HR5(0,1, -1,1,-1)
      HY5(0,-1,1,-1,-1) = 
     &  + 9.4133341974174110d-03
     &  - 5.0916764064292634d-02*HR1(-1)
     &  - 2.3766885054564933d-01*HR1(-1)*HR1(-1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 4.7533770109129867d-01*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR2(-1,1)
     &  - 5.0916764064292634d-02*HR1(1)
     &  + 2.4022650695910071d-01*HR1(1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 4.7533770109129867d-01*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  + HR2( -1,1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  + HR5( -1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - HR5(0, -1,1,-1,-1)
      HY5(0,-1,1,-1,1) = 
     &  + 1.3833955759762555d-02
     &  - 8.7985537010508960d-02*HR1(-1)
     &  - 7.3900238327152102d-01*HR1(-1)*HR1(-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 1.4780047665430420d+00*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(0)*HR2(-1,1)
     &  + HR1(0) *HR2(-1,1)*HR2(0,-1)
     &  - HR1(0) *HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,1)
     &  - HR1(0) *HR4(0,-1,1,-1)
     &  - 8.7985537010508960d-02*HR1(1)
     &  + 5.8224052646501250d-01*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,-1)
     &  + 1.4780047665430420d+00*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,0,-1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  + HR2(0, -1)*HR3(0,-1,-1)
     &  + HR2(0, -1)*HR3(0,-1,1)
     &  + 5.8224052646501250d-01*HR3(-1,-1,1)
     &  - 5.8224052646501250d-01*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  - HR5( -1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - HR5(0, -1,0,-1,-1)
     &  - HR5(0, -1,0,-1,1)
     &  + HR5(0, -1,1,-1,-1)
      HY5(0,-1,1,0,1) = 
     &  + 7.6026642213084631d-02
     &  - 3.5228267839753708d-01*HR1(-1)
     &  - 1.7721476084810206d+00*HR1(-1)*HR1(-1)
     &  + 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,0,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(0)*HR1(1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(0)*HR3(0,1,-1)
     &  - 3.5442952169620413d+00*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR4(0,-1,0,1)
     &  - HR1( -1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + HR1(0) *HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1(0) *HR1(1)*HR3(0,-1,1)
     &  + HR1(0) *HR1(1)*HR3(0,1,-1)
     &  + 1.6449340668482264d+00*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + HR1(0) *HR2(-1,1)*HR2(0,-1)
     &  + HR1(0) *HR2(-1,1)*HR2(0,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,-1,1,1)
     &  - HR1(0) *HR4(0,1,-1,-1)
     &  - HR1(0) *HR4(0,1,-1,1)
     &  - 3.5228267839753708d-01*HR1(1)
     &  + 1.6449340668482264d+00*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(1) *HR4(0,-1,0,1)
     &  - HR1(1) *HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + 3.5442952169620413d+00*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,0,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,1,-1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  + HR2(0, -1)*HR3(0,-1,-1)
     &  + HR2(0, -1)*HR3(0,-1,1)
     &  - HR2(0,1) *HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR3(-1,-1,1)
     &  - 1.6449340668482264d+00*HR3(0,-1,-1)
     &  - 1.6449340668482264d+00*HR3(0,-1,1)
     &  - 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR4(0,-1,1,1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,1,-1,1)
     &  + 7.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + HR5(0, -1,-1,0,1)
     &  + 3.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - HR5(0, -1,0,-1,-1)
     &  + HR5(0, -1,0,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,0,1,1)
     &  + 3.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  + HR5(0, -1,1,-1,1)
     &  + HR5(0, -1,1,0,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HR5(0,0,-1,1,1)
     &  + 2.0000000000000000d+00*HR5(0,0,1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,0,1,-1,1)
     &  + 4.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  + HR5(0,1, -1,1,-1)
      HY5(0,-1,1,1,-1) = 
     &  + 2.2801059128486651d-02
     &  - 2.0286579517988963d-01*HR1(-1)
     &  + 5.5365194946473328d-01*HR1(-1)*HR1(-1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(0)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 1.1073038989294665d+00*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,0,-1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR1(0)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,-1,1)
     &  - 2.0286579517988963d-01*HR1(1)
     &  - 1.0626935403832139d+00*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,0,-1)
     &  - 1.1073038989294665d+00*HR2(-1,1)
     &  + HR2( -1,1)*HR3(0,-1,-1)
     &  + HR2( -1,1)*HR3(0,0,-1)
     &  + HR2(0, -1)*HR3(-1,-1,1)
     &  - HR2(0, -1)*HR3(0,-1,-1)
     &  - HR2(0, -1)*HR3(0,-1,1)
     &  - 1.0626935403832139d+00*HR3(-1,-1,1)
     &  + 1.0626935403832139d+00*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,0,-1,1)
     &  - HR5( -1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + HR5(0, -1,-1,1,-1)
     &  + HR5(0, -1,0,-1,-1)
     &  + HR5(0, -1,0,-1,1)
     &  + HR5(0, -1,1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,0,-1,-1,1)
     &  + HR5(0,0, -1,1,-1)
      HY5(0,-1,1,1,1) = 
     &  + 3.9984858137537496d-02
     &  - 5.1747906167389938d-01*HR1(-1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(-1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,0,-1)
     &  - 5.5504108664821579d-02*HR1(-1)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,0,-1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR4(0,0,0,-1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(1)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,-1,1)
     &  + HR1(0) *HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,-1,-1,1)
     &  + HR1(0) *HR4(0,-1,1,-1)
     &  + HR1(0) *HR4(0,0,-1,-1)
     &  + HR1(0) *HR4(0,0,-1,1)
     &  - 5.1747906167389938d-01*HR1(1)
     &  + 2.4022650695910071d-01*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,0,-1)
     &  + HR1(1) *HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,0,-1,-1)
     &  + HR1(1) *HR4(0,0,0,-1)
     &  + 5.5504108664821579d-02*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,-1,1)
     &  + HR5( -1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,1)
     &  - HR5(0, -1,-1,1,-1)
     &  - HR5(0, -1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  - HR5(0,0, -1,-1,1)
     &  - HR5(0,0, -1,1,-1)
     &  - HR5(0,0,0, -1,-1)
     &  - HR5(0,0,0, -1,1)
      HY5(0,0,-1,-1,1) = 
     &  + 1.2444228784499648d-02
     &  - 3.4159126166513913d-02*HR1(-1)
     &  + 4.7376502115063852d-02*HR1(-1)*HR1(-1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR3(-1,1,1)
     &  + 9.4753004230127705d-02*HR1(-1)*HR1(1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  + HR1(0) *HR4(-1,-1,1,1)
     &  - 3.4159126166513913d-02*HR1(1)
     &  + 4.7376502115063852d-02*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(1)*HR2(-1,1)
     &  - HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  - HR1(1) *HR4(-1,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,-1,1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  + HR2(0, -1)*HR3(-1,1,1)
     &  + 5.8224052646501250d-01*HR3(-1,-1,1)
     &  - 5.8224052646501250d-01*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + HR5( -1,-1,-1,1,1)
     &  - HR5(0, -1,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,1)
     &  - HR5(0, -1,-1,1,-1)
     &  - HR5(0, -1,-1,1,1)
      HY5(0,0,-1,0,1) = 
     &  + 1.0679981350605469d-01
     &  - 2.0293560632083841d-01*HR1(-1)
     &  + 1.9444792308405316d-01*HR1(-1)*HR1(-1)
     &  - 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR3(-1,1,1)
     &  + 3.8889584616810632d-01*HR1(-1)*HR1(1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(-1,1,1,1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR4(0,1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(-1,1,1,1)
     &  - 2.0293560632083841d-01*HR1(1)
     &  + 1.9444792308405316d-01*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - HR1(1) *HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 1.6449340668482264d+00*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  - HR1(1) *HR2(-1,1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(-1,1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(-1,-1,1,1)
     &  + HR1(1) *HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,-1,1)
     &  + HR1(1) *HR4(0,1,-1,-1)
     &  + HR1(1) *HR4(0,1,-1,1)
     &  - HR2( -1,1)*HR3(-1,1,1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  + HR2(0, -1)*HR3(-1,1,1)
     &  - HR2(0,1) *HR3(-1,-1,1)
     &  + HR2(0,1) *HR3(-1,1,1)
     &  + 1.6449340668482264d+00*HR3(-1,-1,1)
     &  - 1.6449340668482264d+00*HR3(-1,1,1)
     &  - 2.0794415416798359d+00*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR4(-1,1,1,1)
     &  - 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  + HR5( -1,-1,1,-1,1)
     &  + HR5( -1,1,-1,1,1)
     &  - HR5(0, -1,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,1)
     &  - HR5(0, -1,-1,1,-1)
     &  - HR5(0, -1,-1,1,1)
     &  - HR5(0,1, -1,-1,-1)
     &  - HR5(0,1, -1,-1,1)
     &  - HR5(0,1, -1,1,-1)
     &  - HR5(0,1, -1,1,1)
      HY5(0,0,-1,1,-1) = 
     &  + 1.6991592326175436d-02
     &  - 5.4653052738263652d-02*HR1(-1)
     &  + 1.0703618543335311d-01*HR1(-1)*HR1(-1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 2.1407237086670622d-01*HR1(-1)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(-1,1,1)
     &  - 5.4653052738263652d-02*HR1(1)
     &  + 1.0703618543335311d-01*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + HR1(1) *HR1(1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(1)*HR2(-1,1)
     &  + HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,1)
     &  - HR1(1) *HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,1)
     &  - HR1(1) *HR4(0,-1,1,-1)
     &  + HR2(0, -1)*HR3(-1,-1,1)
     &  - HR2(0, -1)*HR3(-1,1,1)
     &  - 1.0626935403832139d+00*HR3(-1,-1,1)
     &  + 1.0626935403832139d+00*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + HR5( -1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  + HR5(0, -1,1,-1,1)
     &  + HR5(0, -1,1,1,-1)
      HY5(0,0,-1,1,1) = 
     &  + 2.4107342184124538d-02
     &  - 9.3097125991768577d-02*HR1(-1)
     &  + 2.6860659680402010d-01*HR1(-1)*HR1(-1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,1)
     &  + 5.3721319360804020d-01*HR1(-1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  + HR1(0) *HR1(1)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR4(-1,-1,1,1)
     &  + HR1(0) *HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,-1,-1,1)
     &  + HR1(0) *HR4(0,-1,1,-1)
     &  + HR1(0) *HR4(0,-1,1,1)
     &  - 9.3097125991768577d-02*HR1(1)
     &  + 2.6860659680402010d-01*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,1)
     &  + HR1(1) *HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,1,-1)
     &  + HR1(1) *HR4(0,0,-1,-1)
     &  + HR1(1) *HR4(0,0,-1,1)
     &  + 2.4022650695910071d-01*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - HR5( -1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - HR5(0, -1,-1,1,1)
     &  - 2.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  - HR5(0, -1,1,-1,1)
     &  - HR5(0, -1,1,1,-1)
     &  - HR5(0,0, -1,-1,-1)
     &  - HR5(0,0, -1,-1,1)
     &  - HR5(0,0, -1,1,-1)
     &  - HR5(0,0, -1,1,1)
      HY5(0,0,0,-1,1) = 
     &  + 5.9142607400864533d-02
     &  - 1.1787599965050932d-01*HR1(-1)
     &  + 1.2153517583503078d-01*HR1(-1)*HR1(-1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  + 2.4307035167006157d-01*HR1(-1)*HR1(1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR4(-1,1,1,1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR4(0,-1,1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + HR1(0) *HR1(1)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(1)*HR3(-1,1,1)
     &  + HR1(0) *HR4(-1,-1,-1,1)
     &  - HR1(0) *HR4(-1,-1,1,1)
     &  + HR1(0) *HR4(-1,1,1,1)
     &  - 1.1787599965050932d-01*HR1(1)
     &  + 1.2153517583503078d-01*HR1(1)*HR1(1)
     &  - 9.7040087744168750d-02*HR1(1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(1) *HR4(-1,-1,1,1)
     &  + HR1(1) *HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,1,-1)
     &  + HR1(1) *HR4(0,-1,1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR4(-1,1,1,1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  + HR5( -1,-1,1,1,1)
     &  - HR5(0, -1,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,1)
     &  - HR5(0, -1,-1,1,-1)
     &  - HR5(0, -1,-1,1,1)
     &  - HR5(0, -1,1,-1,-1)
     &  - HR5(0, -1,1,-1,1)
     &  - HR5(0, -1,1,1,-1)
     &  - HR5(0, -1,1,1,1)
      HY5(0,0,0,1,-1) = 
     &  + 7.4276054639867797d-02
     &  - 1.7284527823898438d-01*HR1(-1)
     &  + 2.5410760640234242d-01*HR1(-1)*HR1(-1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0821521280468485d-01*HR1(-1)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,1,1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR4(-1,1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR4(0,1,-1,1)
     &  - HR1( -1)*HR4(0,1,1,-1)
     &  + 1.1552453009332421d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.7284527823898438d-01*HR1(1)
     &  + 2.5410760640234242d-01*HR1(1)*HR1(1)
     &  + 1.7711559006386898d-01*HR1(1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1(1) *HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(1) *HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,-1,1,-1)
     &  - HR1(1) *HR4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1(1) *HR4(0,1,-1,1)
     &  - HR1(1) *HR4(0,1,1,-1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR4(-1,1,1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,1,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,1,1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  + HR5( -1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,1,1,-1)
     &  + HR5(0, -1,1,1,1)
     &  + 3.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,1,-1,1,-1)
     &  + HR5(0,1, -1,1,1)
     &  + 2.0000000000000000d+00*HR5(0,1,1,-1,-1)
     &  + HR5(0,1,1, -1,1)
     &  + HR5(0,1,1,1, -1)
      HY5(0,0,1,-1,-1) = 
     &  + 2.5535023438634211d-02
     &  - 1.1412342741606084d-01*HR1(-1)
     &  - 2.3766885054564933d-01*HR1(-1)*HR1(-1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 4.7533770109129867d-01*HR1(-1)*HR1(1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR4(0,1,-1,-1)
     &  - 1.2011325347955035d-01*HR1(0)*HR1(1)*HR1(1)
     &  - 1.1412342741606084d-01*HR1(1)
     &  - 2.3766885054564933d-01*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  - HR1(1) *HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,1,-1)
     &  + HR1(1) *HR4(0,1,-1,-1)
     &  - 2.4022650695910071d-01*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR3(-1,1,1)
     &  - 2.4022650695910071d-01*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR3(0,-1,1)
     &  - 2.4022650695910071d-01*HR3(0,1,-1)
     &  - 2.4022650695910071d-01*HR3(0,1,1)
     &  - 1.3862943611198906d+00*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,1)
     &  + 1.3862943611198906d+00*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,1,1,-1)
     &  - 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + HR5( -1,-1,-1,1,1)
     &  - 6.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - HR5(0, -1,-1,1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  - HR5(0, -1,1,-1,1)
     &  - HR5(0, -1,1,1,-1)
     &  - 3.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  - HR5(0,1, -1,-1,1)
     &  - HR5(0,1, -1,1,-1)
     &  - HR5(0,1,1, -1,-1)
      HY5(0,0,1,-1,1) = 
     &  + 3.6321732111088421d-02
     &  - 1.9355535381306524d-01*HR1(-1)
     &  - 7.3900238327152102d-01*HR1(-1)*HR1(-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(0)*HR3(0,1,-1)
     &  - 1.4780047665430420d+00*HR1(-1)*HR1(1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR2(0,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,0,1)
     &  - HR1( -1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR4(0,1,-1,-1)
     &  - 2.9112026323250625d-01*HR1(0)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + HR1(0) *HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1(0) *HR1(1)*HR3(0,-1,1)
     &  + HR1(0) *HR1(1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,-1,1,-1)
     &  - HR1(0) *HR4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,1,-1,-1)
     &  - HR1(0) *HR4(0,1,-1,1)
     &  - HR1(0) *HR4(0,1,1,-1)
     &  - 1.9355535381306524d-01*HR1(1)
     &  - 7.3900238327152102d-01*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + HR1(1) *HR1(1)*HR3(0,-1,-1)
     &  + HR1(1) *HR1(1)*HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR1(1)*HR2(-1,1)
     &  + HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  + 5.8224052646501250d-01*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(1) *HR2(0,-1)*HR2(0,1)
     &  + 5.8224052646501250d-01*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  + HR1(1) *HR4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,0,1)
     &  - HR1(1) *HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - HR1(1) *HR4(0,1,-1,-1)
     &  + HR2(0, -1)*HR3(-1,-1,1)
     &  - HR2(0, -1)*HR3(-1,1,1)
     &  + HR2(0, -1)*HR3(0,-1,-1)
     &  + HR2(0, -1)*HR3(0,-1,1)
     &  + HR2(0, -1)*HR3(0,1,-1)
     &  + HR2(0, -1)*HR3(0,1,1)
     &  - 5.8224052646501250d-01*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR3(-1,1,1)
     &  - 5.8224052646501250d-01*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR3(0,-1,1)
     &  - 5.8224052646501250d-01*HR3(0,1,-1)
     &  - 5.8224052646501250d-01*HR3(0,1,1)
     &  + 1.3862943611198906d+00*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,1)
     &  - 1.3862943611198906d+00*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,1,1,-1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - HR5( -1,-1,-1,1,1)
     &  + 7.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,1)
     &  - HR5(0, -1,0,-1,-1)
     &  - HR5(0, -1,0,-1,1)
     &  - HR5(0, -1,0,1,-1)
     &  - HR5(0, -1,0,1,1)
     &  + 3.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  + HR5(0, -1,1,-1,1)
     &  + HR5(0, -1,1,1,-1)
     &  + 3.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  + HR5(0,1, -1,-1,1)
     &  + HR5(0,1, -1,1,-1)
     &  + HR5(0,1,1, -1,-1)
      HY5(0,0,1,0,-1) = 
     &  + 1.8615775173851248d-01
     &  - 5.6852588003909690d-01*HR1(-1)
     &  - 6.6068813489808640d-01*HR1(-1)*HR1(-1)
     &  + 1.3707783890401886d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.3213762697961728d+00*HR1(-1)*HR1(1)
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR2(0,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(-1,1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,1,1,-1)
     &  - 4.1123351671205660d-01*HR1(0)*HR1(1)*HR1(1)
     &  - 5.6852588003909690d-01*HR1(1)
     &  - 6.6068813489808640d-01*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  - HR1(1) *HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 8.2246703342411321d-01*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + 8.2246703342411321d-01*HR1(1)*HR2(0,-1)
     &  + 8.2246703342411321d-01*HR1(1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1(1) *HR4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,1,1,-1)
     &  - HR2( -1,1)*HR3(-1,1,1)
     &  - 8.2246703342411321d-01*HR3(-1,-1,1)
     &  + 8.2246703342411321d-01*HR3(-1,1,1)
     &  - 8.2246703342411321d-01*HR3(0,-1,-1)
     &  - 8.2246703342411321d-01*HR3(0,-1,1)
     &  - 8.2246703342411321d-01*HR3(0,1,-1)
     &  - 8.2246703342411321d-01*HR3(0,1,1)
     &  - 2.0794415416798359d+00*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR4(-1,1,1,1)
     &  + 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR4(0,-1,-1,1)
     &  + 2.0794415416798359d+00*HR4(0,-1,1,-1)
     &  + 2.0794415416798359d+00*HR4(0,-1,1,1)
     &  + 2.0794415416798359d+00*HR4(0,1,-1,-1)
     &  + 2.0794415416798359d+00*HR4(0,1,-1,1)
     &  + 2.0794415416798359d+00*HR4(0,1,1,-1)
     &  + 2.0794415416798359d+00*HR4(0,1,1,1)
     &  - 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  + HR5( -1,-1,1,-1,1)
     &  + HR5( -1,1,-1,1,1)
     &  - 6.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - HR5(0, -1,-1,1,1)
     &  - 5.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,1,1,-1)
     &  - 6.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR5(0,1,-1,1,-1)
     &  - HR5(0,1, -1,1,1)
     &  - 5.0000000000000000d+00*HR5(0,1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,1,1,-1,1)
     &  - 3.0000000000000000d+00*HR5(0,1,1,1,-1)
      HY5(0,0,1,1,-1) = 
     &  + 5.7353571803049304d-02
     &  - 4.3369237704895519d-01*HR1(-1)
     &  + 5.5365194946473328d-01*HR1(-1)*HR1(-1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 1.1073038989294665d+00*HR1(-1)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1( -1)*HR2(0,-1)*HR2(0,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR4(0,-1,0,1)
     &  - HR1( -1)*HR4(0,0,-1,1)
     &  - HR1( -1)*HR4(0,0,1,-1)
     &  - HR1( -1)*HR4(0,1,-1,-1)
     &  + 1.7328679513998632d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(0)*HR1(1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,1,1)
     &  - 4.3369237704895519d-01*HR1(1)
     &  + 5.5365194946473328d-01*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 1.0626935403832139d+00*HR1(1)*HR2(-1,1)
     &  - HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1(1) *HR2(0,-1)*HR2(0,1)
     &  - 1.0626935403832139d+00*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  + HR1(1) *HR4(-1,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,-1,1)
     &  - HR1(1) *HR4(0,-1,0,1)
     &  - HR1(1) *HR4(0,0,-1,1)
     &  - HR1(1) *HR4(0,0,1,-1)
     &  - HR1(1) *HR4(0,1,-1,-1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  + HR2(0, -1)*HR3(-1,1,1)
     &  - HR2(0, -1)*HR3(0,-1,-1)
     &  - HR2(0, -1)*HR3(0,-1,1)
     &  - HR2(0, -1)*HR3(0,1,-1)
     &  - HR2(0, -1)*HR3(0,1,1)
     &  + 1.0626935403832139d+00*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR3(-1,1,1)
     &  + 1.0626935403832139d+00*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR3(0,-1,1)
     &  + 1.0626935403832139d+00*HR3(0,1,-1)
     &  + 1.0626935403832139d+00*HR3(0,1,1)
     &  + 1.3862943611198906d+00*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  - 1.3862943611198906d+00*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,0,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,0,1,1)
     &  - 1.3862943611198906d+00*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,1,1,-1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - HR5( -1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - HR5(0, -1,-1,1,1)
     &  + HR5(0, -1,0,-1,-1)
     &  + HR5(0, -1,0,-1,1)
     &  + HR5(0, -1,0,1,-1)
     &  + HR5(0, -1,0,1,1)
     &  + HR5(0, -1,1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,0,-1,1,-1)
     &  + HR5(0,0, -1,1,1)
     &  + 2.0000000000000000d+00*HR5(0,0,1,-1,-1)
     &  + HR5(0,0,1, -1,1)
     &  + HR5(0,0,1,1, -1)
     &  + 3.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  + HR5(0,1, -1,-1,1)
     &  + HR5(0,1, -1,1,-1)
     &  + HR5(0,1,1, -1,-1)
      HY5(0,1,-1,-1,-1) = 
     &  + 1.9555438852482933d-02
     &  + 1.2679858379652411d-01*HR1(-1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(-1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR1(0)
     &  - 5.5504108664821579d-02*HR1(-1)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(0,-1,-1,-1)
     &  + 5.5504108664821579d-02*HR1(0)*HR1(1)
     &  + 1.2679858379652411d-01*HR1(1)
     &  - 2.4022650695910071d-01*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  - HR1(1) *HR4(0,-1,-1,-1)
     &  + 5.5504108664821579d-02*HR2(-1,1)
     &  - 5.5504108664821579d-02*HR2(0,-1)
     &  - 5.5504108664821579d-02*HR2(0,1)
     &  + 2.4022650695910071d-01*HR3(-1,-1,1)
     &  + 4.8045301391820142d-01*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR3(0,-1,1)
     &  + 2.4022650695910071d-01*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  + HR5( -1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + HR5(0, -1,-1,-1,1)
     &  + HR5(0, -1,-1,1,-1)
     &  + HR5(0, -1,1,-1,-1)
     &  + HR5(0,1, -1,-1,-1)
      HY5(0,1,-1,-1,1) = 
     &  + 2.8668668263701248d-02
     &  + 2.3517979306082505d-01*HR1(-1)
     &  - 4.7376502115063852d-02*HR1(-1)*HR1(-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 9.4753004230127705d-02*HR1(-1)*HR1(0)
     &  + HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  - 9.4753004230127705d-02*HR1(-1)*HR1(1)
     &  + HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  + 9.4753004230127705d-02*HR1(0)*HR1(1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,-1,-1,1)
     &  + HR1(0) *HR4(0,-1,1,-1)
     &  + HR1(0) *HR4(0,1,-1,-1)
     &  + 2.3517979306082505d-01*HR1(1)
     &  - 5.8224052646501250d-01*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  + 9.4753004230127705d-02*HR2(-1,1)
     &  - HR2( -1,1)*HR3(0,-1,-1)
     &  - 9.4753004230127705d-02*HR2(0,-1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR3(0,-1,-1)
     &  - HR2(0, -1)*HR3(0,-1,1)
     &  - HR2(0, -1)*HR3(0,1,-1)
     &  - 9.4753004230127705d-02*HR2(0,1)
     &  + HR2(0,1) *HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR3(-1,-1,1)
     &  + 1.1644810529300250d+00*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR3(0,-1,1)
     &  + 5.8224052646501250d-01*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  - HR5( -1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - HR5(0, -1,-1,0,1)
     &  - HR5(0, -1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR5(0,-1,0,-1,-1)
     &  - HR5(0, -1,1,-1,-1)
     &  + 6.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  - HR5(0,1, -1,-1,-1)
      HY5(0,1,-1,1,-1) = 
     &  + 4.7069633474401836d-02
     &  + 6.4210078767232862d-01*HR1(-1)
     &  - 1.0703618543335311d-01*HR1(-1)*HR1(-1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + 2.1407237086670622d-01*HR1(-1)*HR1(0)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 2.1407237086670622d-01*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,0,-1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  + 2.1407237086670622d-01*HR1(0)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,1,-1)
     &  + 6.4210078767232862d-01*HR1(1)
     &  + 1.0626935403832139d+00*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,0,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,-1)
     &  + 2.1407237086670622d-01*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.1407237086670622d-01*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR2(0,1)
     &  + HR2(0, -1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR3(0,-1,-1)
     &  + HR2(0, -1)*HR3(0,-1,1)
     &  + HR2(0, -1)*HR3(0,1,-1)
     &  - 2.1407237086670622d-01*HR2(0,1)
     &  - 2.0000000000000000d+00*HR2(0,1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR3(-1,-1,1)
     &  - 2.1253870807664278d+00*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR3(0,-1,1)
     &  - 1.0626935403832139d+00*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,0,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  - HR5( -1,-1,-1,-1,1)
     &  - 8.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR5(0,-1,0,-1,-1)
     &  + HR5(0, -1,0,-1,1)
     &  + HR5(0, -1,0,1,-1)
     &  - HR5(0, -1,1,-1,-1)
     &  - 1.2000000000000000d+01*HR5(0,0,-1,-1,-1)
     &  - HR5(0,1, -1,-1,-1)
      HY5(0,1,-1,1,1) = 
     &  + 8.2208743029471844d-02
     &  + 1.9248049955307152d+00*HR1(-1)
     &  - 2.6860659680402010d-01*HR1(-1)*HR1(-1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 5.3721319360804020d-01*HR1(-1)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  - 5.3721319360804020d-01*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,0,-1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + HR1(0) *HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  + 5.3721319360804020d-01*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + HR1(0) *HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(0) *HR2(0,-1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,1,-1)
     &  + HR1(0) *HR4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,1)
     &  + HR1(0) *HR4(0,-1,0,1)
     &  - HR1(0) *HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,0,-1,-1)
     &  - HR1(0) *HR4(0,1,-1,-1)
     &  + 1.9248049955307152d+00*HR1(1)
     &  - 2.4022650695910071d-01*HR1(1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,0,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,0,0,-1)
     &  + 5.3721319360804020d-01*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  - HR2( -1,1)*HR3(0,-1,-1)
     &  - HR2( -1,1)*HR3(0,0,-1)
     &  - 5.3721319360804020d-01*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR2(0,1)
     &  + HR2(0, -1)*HR3(0,0,-1)
     &  - 5.3721319360804020d-01*HR2(0,1)
     &  + HR2(0,1) *HR3(0,-1,-1)
     &  + HR2(0,1) *HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR3(-1,-1,1)
     &  + 4.8045301391820142d-01*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR3(0,-1,1)
     &  + 2.4022650695910071d-01*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  + HR5( -1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - HR5(0, -1,-1,0,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  + HR5(0, -1,0,-1,-1)
     &  - HR5(0, -1,0,-1,1)
     &  - HR5(0, -1,0,1,-1)
     &  + HR5(0, -1,1,-1,-1)
     &  + 7.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  + HR5(0,0, -1,-1,1)
     &  - HR5(0,0, -1,0,-1)
     &  - HR5(0,0, -1,0,1)
     &  + HR5(0,1, -1,-1,-1)
      HY5(0,1,0,1,-1) = 
     &  + 1.4122347902560834d-01
     &  + 1.4132080497842155d+00*HR1(-1)
     &  - 2.5410760640234242d-01*HR1(-1)*HR1(-1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - HR1( -1)*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 5.0821521280468485d-01*HR1(-1)*HR1(0)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 5.0821521280468485d-01*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR2(0,1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,0,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,0,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,0,1)
     &  + HR1( -1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + 5.0821521280468485d-01*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR3(0,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR3(0,1,1)
     &  + 1.4132080497842155d+00*HR1(1)
     &  - 1.0626935403832139d+00*HR1(1)*HR2(-1,1)
     &  + HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  + 1.0626935403832139d+00*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(1) *HR2(0,-1)*HR2(0,1)
     &  + 1.0626935403832139d+00*HR1(1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,0,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,0,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,0,1)
     &  + HR1(1) *HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + 5.0821521280468485d-01*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR2(0,1)
     &  + HR2( -1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR2( -1,1)*HR3(0,-1,1)
     &  + HR2( -1,1)*HR3(0,1,-1)
     &  - 5.0821521280468485d-01*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR3(0,1,1)
     &  - 5.0821521280468485d-01*HR2(0,1)
     &  + 3.4657359027997265d-01*HR2(0,1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR2(0,1)*HR3(0,-1,-1)
     &  - HR2(0,1) *HR3(0,-1,1)
     &  - 2.1253870807664278d+00*HR3(-1,-1,1)
     &  + 2.1253870807664278d+00*HR3(-1,1,1)
     &  - 2.1253870807664278d+00*HR3(0,-1,-1)
     &  - 2.1253870807664278d+00*HR3(0,-1,1)
     &  - 2.1253870807664278d+00*HR3(0,1,-1)
     &  - 2.1253870807664278d+00*HR3(0,1,1)
     &  - 2.7725887222397812d+00*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR4(-1,-1,1,1)
     &  + 2.7725887222397812d+00*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR4(0,0,-1,-1)
     &  + 1.3862943611198906d+00*HR4(0,0,-1,1)
     &  + 1.3862943611198906d+00*HR4(0,0,1,-1)
     &  + 1.3862943611198906d+00*HR4(0,0,1,1)
     &  + 2.7725887222397812d+00*HR4(0,1,-1,-1)
     &  + 1.3862943611198906d+00*HR4(0,1,-1,1)
     &  + 1.3862943611198906d+00*HR4(0,1,1,-1)
     &  - 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - HR5( -1,-1,1,-1,1)
     &  - 8.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR5(0,-1,0,-1,-1)
     &  - HR5(0, -1,0,-1,1)
     &  - HR5(0, -1,0,1,-1)
     &  - 3.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  - HR5(0, -1,1,-1,1)
     &  + HR5(0, -1,1,0,1)
     &  - 1.2000000000000000d+01*HR5(0,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR5(0,0,-1,-1,1)
     &  - 4.0000000000000000d+00*HR5(0,0,-1,1,-1)
     &  - 4.0000000000000000d+00*HR5(0,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,0,1,-1,1)
     &  - 4.0000000000000000d+00*HR5(0,0,1,1,-1)
     &  - 7.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR5(0,1,-1,1,-1)
     &  - HR5(0,1,0,1, -1)
     &  - 2.0000000000000000d+00*HR5(0,1,1,-1,-1)
      HY5(0,1,1,-1,-1) = 
     &  + 1.0254618242743082d-01
     &  - 6.0337978402921669d-01*HR1(-1)
     &  + 2.3766885054564933d-01*HR1(-1)*HR1(-1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 4.7533770109129867d-01*HR1(-1)*HR1(0)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(0)*HR1(0)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR1(1)
     &  + 4.7533770109129867d-01*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,0,-1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR4(0,0,-1,-1)
     &  - 1.2011325347955035d-01*HR1(0)*HR1(0)*HR1(1)
     &  - 4.7533770109129867d-01*HR1(0)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(0)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR2(0,1)
     &  - 6.0337978402921669d-01*HR1(1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(1) *HR4(0,0,-1,-1)
     &  - 4.7533770109129867d-01*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  - HR2( -1,1)*HR3(0,-1,-1)
     &  + 4.7533770109129867d-01*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR2(0,1)
     &  + 4.7533770109129867d-01*HR2(0,1)
     &  + HR2(0,1) *HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR3(0,0,-1)
     &  - 2.4022650695910071d-01*HR3(0,0,1)
     &  - 2.4022650695910071d-01*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,0,1)
     &  + 6.9314718055994530d-01*HR4(0,0,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  - HR5( -1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - HR5(0, -1,-1,0,1)
     &  + HR5(0, -1,-1,1,-1)
     &  + HR5(0, -1,0,-1,-1)
     &  - HR5(0, -1,0,-1,1)
     &  - HR5(0, -1,0,1,-1)
     &  + 3.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  - HR5(0,0, -1,-1,1)
     &  - HR5(0,0, -1,1,-1)
     &  - HR5(0,0,1, -1,-1)
     &  - HR5(0,1, -1,-1,-1)
      HY5(0,1,1,-1,1) = 
     &  + 1.7692012816527167d-01
     &  - 2.4370424139224501d+00*HR1(-1)
     &  + 7.3900238327152102d-01*HR1(-1)*HR1(-1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR3(0,0,-1)
     &  - 1.4780047665430420d+00*HR1(-1)*HR1(0)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(0)*HR1(0)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR1(1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,0,-1)
     &  + 1.4780047665430420d+00*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,0,-1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,0,0,-1)
     &  - 2.9112026323250625d-01*HR1(0)*HR1(0)*HR1(1)
     &  - 1.4780047665430420d+00*HR1(0)*HR1(1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(1)*HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR1(0)*HR2(-1,1)
     &  - HR1(0) *HR2(-1,1)*HR2(0,-1)
     &  + 5.8224052646501250d-01*HR1(0)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  + HR1(0) *HR2(0,-1)*HR2(0,1)
     &  + 5.8224052646501250d-01*HR1(0)*HR2(0,1)
     &  + HR1(0) *HR4(-1,-1,-1,1)
     &  + HR1(0) *HR4(0,-1,-1,1)
     &  - HR1(0) *HR4(0,-1,0,1)
     &  - HR1(0) *HR4(0,0,-1,1)
     &  - HR1(0) *HR4(0,0,1,-1)
     &  - HR1(0) *HR4(0,1,-1,-1)
     &  - 2.4370424139224501d+00*HR1(1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,0,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,0,0,-1)
     &  - 1.4780047665430420d+00*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR3(0,0,-1)
     &  + 1.4780047665430420d+00*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR2(0,1)
     &  + HR2(0, -1)*HR3(-1,-1,1)
     &  + HR2(0, -1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR3(0,0,-1)
     &  + HR2(0, -1)*HR3(0,1,-1)
     &  + 1.4780047665430420d+00*HR2(0,1)
     &  - 2.0000000000000000d+00*HR2(0,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR2(0,1)*HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR3(-1,-1,1)
     &  - 5.8224052646501250d-01*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR3(0,0,1)
     &  - 5.8224052646501250d-01*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HR4(0,0,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  + HR5( -1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,0,1)
     &  - HR5(0, -1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR5(0,-1,0,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,0,-1,1)
     &  + HR5(0, -1,0,1,-1)
     &  - 9.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  + HR5(0,0, -1,-1,1)
     &  + 3.0000000000000000d+00*HR5(0,0,-1,0,-1)
     &  + 2.0000000000000000d+00*HR5(0,0,-1,0,1)
     &  + HR5(0,0, -1,1,-1)
     &  + 6.0000000000000000d+00*HR5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,0,0,-1,1)
     &  + 3.0000000000000000d+00*HR5(0,0,0,1,-1)
     &  + HR5(0,0,1, -1,-1)
     &  + HR5(0,0,1,0, -1)
     &  + HR5(0,1, -1,-1,-1)
      HY5(0,1,1,1,-1) = 
     &  + 4.0707197178331534d-01
     &  + 1.0681889964961481d+00*HR1(-1)
     &  - 5.5365194946473328d-01*HR1(-1)*HR1(-1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 1.1073038989294665d+00*HR1(-1)*HR1(0)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 1.1073038989294665d+00*HR1(-1)*HR1(1)
     &  + HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR4(0,0,0,-1)
     &  + 1.1552453009332421d-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(0)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR2(0,1)
     &  + 1.1073038989294665d+00*HR1(0)*HR1(1)
     &  + 1.0626935403832139d+00*HR1(0)*HR2(-1,1)
     &  - 1.0626935403832139d+00*HR1(0)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR1(0)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,1,-1)
     &  + 1.0681889964961481d+00*HR1(1)
     &  - HR1(1) *HR4(0,-1,-1,-1)
     &  - HR1(1) *HR4(0,0,-1,-1)
     &  - HR1(1) *HR4(0,0,0,-1)
     &  + 1.1073038989294665d+00*HR2(-1,1)
     &  - HR2( -1,1)*HR3(0,-1,-1)
     &  - HR2( -1,1)*HR3(0,0,-1)
     &  - 1.1073038989294665d+00*HR2(0,-1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  - HR2(0, -1)*HR3(0,-1,-1)
     &  + HR2(0, -1)*HR3(0,0,-1)
     &  - HR2(0, -1)*HR3(0,1,-1)
     &  - 1.1073038989294665d+00*HR2(0,1)
     &  + HR2(0,1) *HR3(0,-1,-1)
     &  + HR2(0,1) *HR3(0,0,-1)
     &  + 1.0626935403832139d+00*HR3(-1,-1,1)
     &  + 1.0626935403832139d+00*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR3(0,0,-1)
     &  + 1.0626935403832139d+00*HR3(0,0,1)
     &  + 1.0626935403832139d+00*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,0,0,-1)
     &  - 6.9314718055994530d-01*HR4(0,0,0,1)
     &  - 6.9314718055994530d-01*HR4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  + HR5( -1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + HR5(0, -1,-1,-1,1)
     &  - HR5(0, -1,-1,0,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,0,-1,-1)
     &  - HR5(0, -1,0,-1,1)
     &  + 6.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  - HR5(0,0, -1,-1,1)
     &  - 2.0000000000000000d+00*HR5(0,0,-1,0,-1)
     &  - HR5(0,0, -1,0,1)
     &  - 4.0000000000000000d+00*HR5(0,0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,0,0,-1,1)
     &  - 2.0000000000000000d+00*HR5(0,0,0,1,-1)
     &  + HR5(0,0,1, -1,-1)
     &  - HR5(0,0,1,0, -1)
     &  + HR5(0,1, -1,-1,-1)
      if (r.lt.0d0) then 
      HY5(-1,-1,-1,1,1) = HY5(-1,-1,-1,1,1) 
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)
      HY5(-1,-1,1,1,1) = HY5(-1,-1,1,1,1) 
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.9348022005446793d+00*HR1(-1)*HR2(0,-1)
     &  + 4.9348022005446793d+00*HR3(0,-1,-1)
      HY5(-1,1,-1,1,1) = HY5(-1,1,-1,1,1) 
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR2(0,-1)
     &  - 9.8696044010893586d+00*HR3(0,-1,-1)
      HY5(-1,1,1,1,1) = HY5(-1,1,1,1,1) 
     &  - 2.8732418312458363d+00*HR1(-1)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(0)
     &  + 3.4205442319285582d+00*HR1(-1)*HR1(0)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(0)*HR1(0)
     &  - 4.9348022005446793d+00*HR1(0)*HR2(0,-1)
     &  - 3.4205442319285582d+00*HR2(0,-1)
     &  + 4.9348022005446793d+00*HR3(0,-1,-1)
     &  + 4.9348022005446793d+00*HR3(0,0,-1)
      HY5(0,-1,-1,1,1) = HY5(0,-1,-1,1,1) 
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR3(-1,-1,1)
      HY5(0,-1,0,1,1) = HY5(0,-1,0,1,1) 
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(1)*HR2(-1,1)
     &  + 9.8696044010893586d+00*HR3(-1,-1,1)
     &  - 9.8696044010893586d+00*HR3(-1,1,1)
      HY5(0,-1,1,1,-1) = HY5(0,-1,1,1,-1) 
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(-1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR1(1)
     &  + 3.4205442319285582d+00*HR2(-1,1)
      HY5(0,-1,1,1,1) = HY5(0,-1,1,1,1) 
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR1(1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR2(-1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(0)*HR2(-1,1)
     &  - 4.9348022005446793d+00*HR1(1)*HR2(0,-1)
     &  - 3.4205442319285582d+00*HR2(-1,1)
     &  - 4.9348022005446793d+00*HR3(-1,-1,1)
     &  + 4.9348022005446793d+00*HR3(0,-1,-1)
     &  + 4.9348022005446793d+00*HR3(0,-1,1)
      HY5(0,0,-1,1,1) = HY5(0,0,-1,1,1) 
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.9348022005446793d+00*HR1(1)*HR2(-1,1)
     &  - 4.9348022005446793d+00*HR3(-1,-1,1)
     &  + 4.9348022005446793d+00*HR3(-1,1,1)
      HY5(0,0,1,1,-1) = HY5(0,0,1,1,-1) 
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(-1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR1(1)
     &  - 1.7102721159642791d+00*HR1(1)*HR1(1)
      HY5(0,1,-1,1,1) = HY5(0,1,-1,1,1) 
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR2(0,-1)
     &  + 4.9348022005446793d+00*HR1(1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR3(-1,-1,1)
     &  - 9.8696044010893586d+00*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR3(0,-1,1)
     &  - 4.9348022005446793d+00*HR3(0,1,-1)
      HY5(0,1,1,-1,-1) = HY5(0,1,1,-1,-1) 
     &  + 1.1854702951709319d+00*HR1(-1)
     &  + 1.1854702951709319d+00*HR1(1)
      HY5(0,1,1,-1,1) = HY5(0,1,1,-1,1) 
     &  + 2.8732418312458363d+00*HR1(-1)
     &  + 2.8732418312458363d+00*HR1(1)
      HY5(0,1,1,1,-1) = HY5(0,1,1,1,-1) 
     &  - 5.2441824215877001d+00*HR1(-1)
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(-1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR1(0)
     &  + 3.4205442319285582d+00*HR1(-1)*HR1(1)
     &  - 3.4205442319285582d+00*HR1(0)*HR1(1)
     &  - 5.2441824215877001d+00*HR1(1)
     &  - 3.4205442319285582d+00*HR2(-1,1)
     &  + 3.4205442319285582d+00*HR2(0,-1)
     &  + 3.4205442319285582d+00*HR2(0,1)
      Hi5(-1,-1,-1,-1,1) = 
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
      Hi5(-1,-1,-1,1,1) = 
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  - HR4(0, -1,-1,-1)
      Hi5(-1,-1,1,-1,1) = 
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 3.0000000000000000d+00*HR4(0,-1,-1,-1)
      Hi5(-1,-1,1,1,1) = 
     &  - 7.0235377994456286d-01*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - HR1( -1)*HR1(0)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR3(0,0,-1)
     &  + HR1(0) *HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR4(0,-1,-1,-1)
     &  - HR4(0,0, -1,-1)
      Hi5(-1,1,-1,1,1) = 
     &  - 5.3721319360804020d-01*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR2(0,-1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR3(0,-1,-1)
     &  + 4.0000000000000000d+00*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR4(0,0,-1,-1)
      Hi5(-1,1,1,1,1) = 
     &  - 1.0846773019780311d+00*HR1(-1)
     &  + 7.0235377994456286d-01*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.4047075598891257d+00*HR1(-1)*HR1(0)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(0,-1)
     &  + HR1(0) *HR3(0,-1,-1)
     &  + HR1(0) *HR3(0,0,-1)
     &  + 1.4047075598891257d+00*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR3(0,0,-1)
     &  - HR4(0, -1,-1,-1)
     &  - HR4(0,0, -1,-1)
     &  - HR4(0,0,0, -1)
      Hi5(0,-1,-1,-1,1) = 
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR3(-1,-1,1)
     &  - HR4( -1,-1,-1,1)
      Hi5(0,-1,-1,0,1) = 
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR3(-1,-1,1)
     &  + HR1(1) *HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)
      Hi5(0,-1,-1,1,-1) = 
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR3(-1,-1,1)
      Hi5(0,-1,-1,1,1) = 
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  - HR1( -1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  + HR1(0) *HR3(-1,-1,1)
     &  + HR1(1) *HR3(0,-1,-1)
     &  + HR2( -1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR3(-1,-1,1)
     &  + HR4( -1,-1,-1,1)
     &  - HR4(0, -1,-1,-1)
     &  - HR4(0, -1,-1,1)
      Hi5(0,-1,0,-1,1) = 
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)
     &  - 4.0000000000000000d+00*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR4(-1,-1,1,1)
      Hi5(0,-1,0,1,-1) = 
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)
     &  - 1.3862943611198906d+00*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR3(-1,1,1)
      Hi5(0,-1,0,1,1) = 
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR3(0,1,-1)
     &  + HR1(0) *HR1(1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR3(-1,-1,1)
     &  + HR1(1) *HR3(0,-1,-1)
     &  + HR1(1) *HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)
     &  + HR2( -1,1)*HR2(0,-1)
     &  + HR2( -1,1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR3(-1,1,1)
     &  + 4.0000000000000000d+00*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR4(-1,-1,1,1)
     &  - HR4(0, -1,-1,-1)
     &  - HR4(0, -1,-1,1)
     &  - HR4(0,1, -1,-1)
     &  - HR4(0,1, -1,1)
      Hi5(0,-1,1,-1,-1) = 
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)
     &  - 2.4022650695910071d-01*HR2(-1,1)
      Hi5(0,-1,1,-1,1) = 
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(1)
     &  + HR1( -1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR2(-1,1)
     &  - HR2( -1,1)*HR2(0,-1)
     &  + HR4( -1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR4(0,-1,-1,1)
     &  + HR4(0, -1,1,-1)
      Hi5(0,-1,1,0,1) = 
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR1(1)
     &  + HR1( -1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR3(0,-1,1)
     &  - HR1( -1)*HR3(0,1,-1)
     &  - HR1(1) *HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(0,-1,-1)
     &  - HR1(1) *HR3(0,-1,1)
     &  - HR1(1) *HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)
     &  - HR2( -1,1)*HR2(0,-1)
     &  - HR2( -1,1)*HR2(0,1)
     &  + 3.0000000000000000d+00*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR4(0,-1,1,1)
     &  + HR4(0,1, -1,-1)
     &  + HR4(0,1, -1,1)
      Hi5(0,-1,1,1,-1) = 
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(0,-1)
     &  + 1.0626935403832139d+00*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR3(0,-1,1)
      Hi5(0,-1,1,1,1) = 
     &  - 7.0235377994456286d-01*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)
     &  + HR1( -1)*HR1(0)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR2(0,-1)
     &  - 1.4047075598891257d+00*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)
     &  + HR1( -1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  - HR1(0) *HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(-1,1)
     &  - HR1(0) *HR3(-1,-1,1)
     &  + HR1(0) *HR3(0,-1,-1)
     &  + HR1(0) *HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(0,-1)
     &  + HR1(1) *HR3(0,-1,-1)
     &  + HR1(1) *HR3(0,0,-1)
     &  + 1.4047075598891257d+00*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR3(0,-1,1)
     &  - HR4( -1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR4(0,-1,-1,-1)
     &  - HR4(0, -1,-1,1)
     &  - HR4(0, -1,1,-1)
     &  - HR4(0,0, -1,-1)
     &  - HR4(0,0, -1,1)
      Hi5(0,0,-1,-1,1) = 
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR3(-1,1,1)
     &  + HR1(1) *HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR4(-1,-1,-1,1)
     &  - HR4( -1,-1,1,1)
      Hi5(0,0,-1,0,1) = 
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(-1,1,1)
     &  + 3.0000000000000000d+00*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR4(-1,1,1,1)
      Hi5(0,0,-1,1,-1) = 
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR3(-1,1,1)
      Hi5(0,0,-1,1,1) = 
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR3(-1,1,1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR3(0,-1,1)
     &  - HR1(0) *HR1(1)*HR2(-1,1)
     &  - HR1(0) *HR3(-1,-1,1)
     &  + HR1(0) *HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)
     &  - HR1(1) *HR3(-1,-1,1)
     &  + HR1(1) *HR3(0,-1,-1)
     &  + HR1(1) *HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR4(-1,-1,-1,1)
     &  + HR4( -1,-1,1,1)
     &  - HR4(0, -1,-1,-1)
     &  - HR4(0, -1,-1,1)
     &  - HR4(0, -1,1,-1)
     &  - HR4(0, -1,1,1)
      Hi5(0,0,0,-1,1) = 
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - HR1(1) *HR3(-1,-1,1)
     &  + HR1(1) *HR3(-1,1,1)
     &  - HR4( -1,-1,-1,1)
     &  + HR4( -1,-1,1,1)
     &  - HR4( -1,1,1,1)
      Hi5(0,0,0,1,-1) = 
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)
      Hi5(0,0,1,-1,-1) = 
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)
     &  + 1.2011325347955035d-01*HR1(1)*HR1(1)
      Hi5(0,0,1,-1,1) = 
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR3(0,-1,1)
     &  - HR1( -1)*HR3(0,1,-1)
     &  + 2.9112026323250625d-01*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - HR1(1) *HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(0,-1,-1)
     &  - HR1(1) *HR3(0,-1,1)
     &  - HR1(1) *HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR4(-1,-1,-1,1)
     &  + HR4( -1,-1,1,1)
     &  + 3.0000000000000000d+00*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR4(0,-1,1,-1)
     &  + HR4(0, -1,1,1)
     &  + 2.0000000000000000d+00*HR4(0,1,-1,-1)
     &  + HR4(0,1, -1,1)
     &  + HR4(0,1,1, -1)
      Hi5(0,0,1,0,-1) = 
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(1)
     &  + 4.1123351671205660d-01*HR1(1)*HR1(1)
      Hi5(0,0,1,1,-1) = 
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR3(0,1,1)
      Hi5(0,1,-1,-1,-1) = 
     &  - 5.5504108664821579d-02*HR1(-1)
     &  - 5.5504108664821579d-02*HR1(1)
      Hi5(0,1,-1,-1,1) = 
     &  - 9.4753004230127705d-02*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - HR1( -1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  - 9.4753004230127705d-02*HR1(1)
     &  + HR1(1) *HR3(0,-1,-1)
     &  + HR4( -1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR4(0,-1,-1,-1)
     &  - HR4(0, -1,-1,1)
     &  - HR4(0, -1,1,-1)
     &  - HR4(0,1, -1,-1)
      Hi5(0,1,-1,1,-1) = 
     &  - 2.1407237086670622d-01*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)
     &  - 2.1407237086670622d-01*HR1(1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR3(0,1,-1)
      Hi5(0,1,-1,1,1) = 
     &  - 5.3721319360804020d-01*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)
     &  + HR1( -1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR3(0,0,-1)
     &  + HR1(0) *HR1(1)*HR2(0,-1)
     &  - HR1(0) *HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR3(0,-1,-1)
     &  - HR1(0) *HR3(0,-1,1)
     &  - HR1(0) *HR3(0,1,-1)
     &  - 5.3721319360804020d-01*HR1(1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR3(0,0,-1)
     &  - HR2( -1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR2(0,-1)*HR2(0,-1)
     &  + HR2(0, -1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR3(0,1,-1)
     &  - HR4( -1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR4(0,-1,-1,1)
     &  - HR4(0, -1,0,1)
     &  + HR4(0, -1,1,-1)
     &  + 2.0000000000000000d+00*HR4(0,0,-1,-1)
     &  + HR4(0,1, -1,-1)
      Hi5(0,1,0,1,-1) = 
     &  - 5.0821521280468485d-01*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(0,1)
     &  - 5.0821521280468485d-01*HR1(1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR3(0,-1,1)
     &  + 1.3862943611198906d+00*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR3(0,1,1)
      Hi5(0,1,1,-1,-1) = 
     &  + 4.7533770109129867d-01*HR1(-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(0)*HR1(1)
     &  + 4.7533770109129867d-01*HR1(1)
     &  + 2.4022650695910071d-01*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR2(0,1)
      Hi5(0,1,1,-1,1) = 
     &  + 1.4780047665430420d+00*HR1(-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(0)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(0)*HR1(1)
     &  + 1.4780047665430420d+00*HR1(1)
     &  + HR1(1) *HR3(0,-1,-1)
     &  + HR1(1) *HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR2(-1,1)
     &  + HR2( -1,1)*HR2(0,-1)
     &  - 5.8224052646501250d-01*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR2(0,-1)*HR2(0,-1)
     &  - HR2(0, -1)*HR2(0,1)
     &  - 5.8224052646501250d-01*HR2(0,1)
     &  - HR4( -1,-1,-1,1)
     &  - HR4(0, -1,-1,1)
     &  + HR4(0, -1,0,1)
     &  + HR4(0,0, -1,1)
     &  + HR4(0,0,1, -1)
     &  + HR4(0,1, -1,-1)
      Hi5(0,1,1,1,-1) = 
     &  + 3.2877511713386177d-02*HR1(-1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(0)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)
     &  - 1.0626935403832139d+00*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(0,1)
     &  + 3.2877511713386177d-02*HR1(1)
     &  - 1.0626935403832139d+00*HR2(-1,1)
     &  + 1.0626935403832139d+00*HR2(0,-1)
     &  + 1.0626935403832139d+00*HR2(0,1)
     &  - 6.9314718055994530d-01*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR3(0,1,-1)
      endif  
      endif 
** nw > 4 endif
      if (nw.gt.5) then
* 2017-04-20:15:56:07.hpl 
* <- r_to_y.real_theta.out 
* produced by form-to-fortr for gehrt@tintin 

      HY6(-1,-1,-1,-1,-1,1) = 
     &  + 1.9723282021162115d-04
     &  - 1.8016537870380179d-03*HR1(-1)
     &  + 7.0671186074950043d-03*HR1(-1)*HR1(-1)
     &  - 1.5792167371687950d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  - HR6(0, -1,-1,-1,-1,-1)
      HY6(-1,-1,-1,-1,1,1) = 
     &  + 3.5960083109555052d-04
     &  - 3.8760673146652637d-03*HR1(-1)
     &  + 2.0379119579654625d-02*HR1(-1)*HR1(-1)
     &  - 8.9535532268006700d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,-1)
     &  + HR1(0) *HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - HR6(0,0, -1,-1,-1,-1)
      HY6(-1,-1,-1,1,-1,1) = 
     &  + 5.0818576926545529d-04
     &  - 6.2154684604081354d-03*HR1(-1)
     &  + 4.3992768505254480d-02*HR1(-1)*HR1(-1)
     &  + 2.4633412775717367d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + HR2(0, -1)*HR4(0,-1,-1,-1)
     &  - 5.8224052646501250d-01*HR4(0,-1,-1,-1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,-1)
     &  + 1.1000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  - HR6(0, -1,0,-1,-1,-1)
      HY6(-1,-1,-1,1,1,1) = 
     &  + 1.1224699796583776d-03
     &  - 1.8530786065466613d-02*HR1(-1)
     &  + 2.5873953083694969d-01*HR1(-1)*HR1(-1)
     &  + 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + HR1(0) *HR5(0,0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  - HR6(0,0,0, -1,-1,-1)
      HY6(-1,-1,1,-1,1,1) = 
     &  + 1.7659263260696900d-03
     &  - 3.8880058841843904d-02*HR1(-1)
     &  - 9.6240249776535762d-01*HR1(-1)*HR1(-1)
     &  + 8.9535532268006700d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 2.6860659680402010d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,-1,-1)
     &  + 5.3721319360804020d-01*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,0,-1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 2.7725887222397812d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,-1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,0,-1)
     &  + 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,-1,-1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + HR1(0) *HR5(0,-1,0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  - 5.3721319360804020d-01*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,0,-1)
     &  + 7.2067952087730213d-01*HR4(0,-1,-1,-1)
     &  - 4.8520302639196171d+00*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  + 1.4000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  - HR6(0,0, -1,0,-1,-1)
      HY6(-1,-1,1,1,-1,1) = 
     &  + 2.8637640787495101d-03
     &  - 8.4962399751964626d-02*HR1(-1)
     &  + 1.2185212069612250d+00*HR1(-1)*HR1(-1)
     &  - 2.4633412775717367d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 7.3900238327152102d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.4556013161625312d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - 1.4780047665430420d+00*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,0,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  + 9.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,-1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(0)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - HR1(0) *HR5(0,-1,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  + 1.4780047665430420d+00*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,0,-1)
     &  - 1.1644810529300250d+00*HR4(0,-1,-1,-1)
     &  - 5.8224052646501250d-01*HR4(0,0,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,0,-1,-1,-1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + HR6(0,0, -1,-1,0,-1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,0,-1,-1)
     &  + 9.0000000000000000d+00*HR6(0,0,0,-1,-1,-1)
      HY6(-1,-1,1,1,1,1) = 
     &  + 8.7230030575968884d-03
     &  - 5.0840057924226870d-01*HR1(-1)
     &  + 4.8090645538142385d-03*HR1(-1)*HR1(-1)
     &  - 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(0)
     &  + 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,0,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,0,-1)
     &  - 5.5504108664821579d-02*HR1(-1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,0,-1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,0,-1,-1)
     &  + HR1( -1)*HR5(0,0,0,0,-1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(0)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,-1)
     &  + HR1(0) *HR5(0,0,0,-1,-1)
     &  + 5.5504108664821579d-02*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,0,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,0,-1,-1,-1)
     &  - HR6(0,0,0,0, -1,-1)
      HY6(-1,1,-1,1,1,1) = 
     &  + 1.7982121549905783d-02
     &  + 2.3922914695671442d+00*HR1(-1)
     &  - 2.5873953083694969d-01*HR1(-1)*HR1(-1)
     &  - 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  + 5.1747906167389938d-01*HR1(-1)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,0,-1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR2(0,-1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,0,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,0,0,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,0,0,-1)
     &  - 3.3333333333333333d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 2.5000000000000000d-01*HR1(0)*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR1(0)*HR4(0,0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,0,-1)
     &  - 4.8045301391820142d-01*HR1(0)*HR3(0,-1,-1)
     &  + 2.7725887222397812d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,0,-1,-1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - HR1(0) *HR5(0,-1,0,-1,-1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,-1)
     &  + HR1(0) *HR5(0,0,-1,0,-1)
     &  - 5.1747906167389938d-01*HR2(0,-1)
     &  + 1.2011325347955035d-01*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,0,-1)
     &  + HR2(0, -1)*HR4(0,0,-1,-1)
     &  + HR2(0, -1)*HR4(0,0,0,-1)
     &  - 1.1100821732964315d-01*HR3(0,-1,-1)
     &  + 9.6090602783640284d-01*HR4(0,-1,-1,-1)
     &  + 4.8045301391820142d-01*HR4(0,0,-1,-1)
     &  - 4.8520302639196171d+00*HR5(0,-1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  - 4.8520302639196171d+00*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,0,-1)
     &  + 1.1000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + HR6(0, -1,0,-1,-1,-1)
     &  + 1.1000000000000000d+01*HR6(0,0,-1,-1,-1,-1)
     &  - HR6(0,0, -1,-1,0,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,0,-1,-1)
     &  + HR6(0,0,0, -1,-1,-1)
     &  - HR6(0,0,0, -1,0,-1)
      HY6(-1,1,1,1,1,1) = 
     &  + 5.0409539780398855d-01
     &  + 1.3333558146428443d-03*HR1(-1)
     &  - 4.8090645538142385d-03*HR1(-1)*HR1(-1)
     &  + 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(0)
     &  - 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(0)
     &  + 9.6181291076284771d-03*HR1(-1)*HR1(0)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(0)*HR1(0)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(0)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(0)*HR1(0)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(0)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,0,-1)
     &  - 1.2011325347955035d-01*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,0,0,-1)
     &  - 5.5504108664821579d-02*HR1(0)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,0,0,-1)
     &  + HR1(0) *HR5(0,-1,-1,-1,-1)
     &  + HR1(0) *HR5(0,0,-1,-1,-1)
     &  + HR1(0) *HR5(0,0,0,-1,-1)
     &  + HR1(0) *HR5(0,0,0,0,-1)
     &  - 9.6181291076284771d-03*HR2(0,-1)
     &  + 5.5504108664821579d-02*HR3(0,-1,-1)
     &  + 5.5504108664821579d-02*HR3(0,0,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,0,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,0,0,-1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - HR6(0,0, -1,-1,-1,-1)
     &  - HR6(0,0,0, -1,-1,-1)
     &  - HR6(0,0,0,0, -1,-1)
     &  - HR6(0,0,0,0,0, -1)
      HY6(0,-1,-1,-1,-1,-1) = 
     &  + 3.5126812414543725d-04
     &  - 1.3333558146428443d-03*HR1(-1)
     &  + 4.8090645538142385d-03*HR1(-1)*HR1(-1)
     &  - 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 9.6181291076284771d-03*HR1(-1)*HR1(1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  - 1.3333558146428443d-03*HR1(1)
     &  - 9.6181291076284771d-03*HR2(-1,1)
     &  - 5.5504108664821579d-02*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  - HR6( -1,-1,-1,-1,-1,1)
      HY6(0,-1,-1,-1,-1,1) = 
     &  + 4.4712505220528870d-04
     &  - 1.8016537870380179d-03*HR1(-1)
     &  + 7.0671186074950043d-03*HR1(-1)*HR1(-1)
     &  - 1.5792167371687950d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 4.7376502115063852d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + 1.4134237214990008d-02*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 9.4753004230127705d-02*HR1(-1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  + HR1(0) *HR5(-1,-1,-1,-1,1)
     &  - 1.8016537870380179d-03*HR1(1)
     &  + HR1(1) *HR5(0,-1,-1,-1,-1)
     &  - 1.4134237214990008d-02*HR2(-1,1)
     &  + HR2( -1,1)*HR4(0,-1,-1,-1)
     &  + HR2(0, -1)*HR4(-1,-1,-1,1)
     &  - 9.4753004230127705d-02*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - HR6(0, -1,-1,-1,-1,1)
      HY6(0,-1,-1,-1,0,1) = 
     &  + 3.8169691380679236d-03
     &  - 1.2380012773057654d-02*HR1(-1)
     &  + 3.8670450283379109d-02*HR1(-1)*HR1(-1)
     &  - 6.4815974361351054d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 6.8538919452009434d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 1.9444792308405316d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + 7.7340900566758219d-02*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  + 3.8889584616810632d-01*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,1)*HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,1,-1,-1,-1)
     &  + HR1(0) *HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR2(-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(-1,-1,1,-1,1)
     &  - 1.2380012773057654d-02*HR1(1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,1,-1,-1,-1)
     &  - 7.7340900566758219d-02*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,-1,1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(-1,-1,-1,1)
     &  + HR2( -1,1)*HR4(0,-1,-1,-1)
     &  + HR2( -1,1)*HR4(0,1,-1,-1)
     &  + HR2(0, -1)*HR4(-1,-1,-1,1)
     &  + HR2(0,1) *HR4(-1,-1,-1,1)
     &  - 3.8889584616810632d-01*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + HR3( -1,-1,1)*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(-1,-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR5(-1,-1,-1,1,1)
     &  + 1.3862943611198906d+00*HR5(-1,-1,1,-1,1)
     &  + 6.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - HR6(0, -1,-1,-1,-1,1)
     &  - HR6(0,1, -1,-1,-1,-1)
     &  - HR6(0,1, -1,-1,-1,1)
      HY6(0,-1,-1,-1,1,-1) = 
     &  + 5.9109948115862196d-04
     &  - 2.5904915267837065d-03*HR1(-1)
     &  + 1.1637533043363782d-02*HR1(-1)*HR1(-1)
     &  - 3.5678728477784371d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 1.0703618543335311d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 2.3275066086727564d-02*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.1407237086670622d-01*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  - 2.5904915267837065d-03*HR1(1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 2.3275066086727564d-02*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR2(0, -1)*HR4(-1,-1,-1,1)
     &  - 2.1407237086670622d-01*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + HR6(0, -1,-1,-1,1,-1)
      HY6(0,-1,-1,-1,1,1) = 
     &  + 8.0303778050832893d-04
     &  - 3.8760673146652637d-03*HR1(-1)
     &  + 2.0379119579654625d-02*HR1(-1)*HR1(-1)
     &  - 8.9535532268006700d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 2.6860659680402010d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + 4.0758239159309251d-02*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 5.3721319360804020d-01*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR5(-1,-1,-1,-1,1)
     &  + HR1(0) *HR5(0,-1,-1,-1,-1)
     &  + HR1(0) *HR5(0,-1,-1,-1,1)
     &  - 3.8760673146652637d-03*HR1(1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,-1)
     &  - 4.0758239159309251d-02*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  + HR2( -1,1)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - 5.3721319360804020d-01*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + HR3( -1,-1,1)*HR3(0,0,-1)
     &  - 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,1,-1)
     &  - HR6(0,0, -1,-1,-1,-1)
     &  - HR6(0,0, -1,-1,-1,1)
      HY6(0,-1,-1,0,-1,1) = 
     &  + 2.1132277544551763d-03
     &  - 7.0342020569501834d-03*HR1(-1)
     &  + 2.2756111933263048d-02*HR1(-1)*HR1(-1)
     &  - 4.0511725278343596d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 1.2153517583503078d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + 4.5512223866526096d-02*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  + 2.4307035167006157d-01*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,1,-1,1)
     &  - 7.0342020569501834d-03*HR1(1)
     &  - HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,1,-1,-1)
     &  - 4.5512223866526096d-02*HR2(-1,1)
     &  - 2.9112026323250625d-01*HR2(-1,1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(-1,-1,1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(-1,-1,-1,1)
     &  + HR2( -1,1)*HR4(0,-1,-1,-1)
     &  + HR2( -1,1)*HR4(0,-1,1,-1)
     &  - 2.4307035167006157d-01*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR3(-1,-1,1)*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + HR3( -1,-1,1)*HR3(0,-1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  - 4.1588830833596718d+00*HR5(-1,-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,1,-1,1)
     &  - 9.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - HR6(0, -1,-1,-1,-1,1)
     &  - HR6(0, -1,1,-1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,1)
      HY6(0,-1,-1,0,1,-1) = 
     &  + 2.6245807975856978d-03
     &  - 9.4343867936923898d-03*HR1(-1)
     &  + 3.4684017151427288d-02*HR1(-1)*HR1(-1)
     &  - 8.4702535467447475d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 2.5410760640234242d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 6.9368034302854577d-02*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  + 5.0821521280468485d-01*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,1,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - 9.4343867936923898d-03*HR1(1)
     &  + HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - HR1(1) *HR5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  - 6.9368034302854577d-02*HR2(-1,1)
     &  + 5.3134677019160696d-01*HR2(-1,1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  - HR2( -1,1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(-1,-1,1)
     &  - 5.0821521280468485d-01*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR3(-1,-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  - HR3( -1,-1,1)*HR3(0,-1,1)
     &  - HR3( -1,-1,1)*HR3(0,1,-1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  - 4.1588830833596718d+00*HR5(-1,-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  - 9.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + HR6(0, -1,-1,-1,1,-1)
     &  + HR6(0, -1,1,-1,-1,-1)
     &  + HR6(0, -1,1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  + HR6(0,1, -1,-1,1,-1)
      HY6(0,-1,-1,0,1,1) = 
     &  + 3.3406073160092585d-03
     &  - 1.3153861288973380d-02*HR1(-1)
     &  + 5.6428748221951486d-02*HR1(-1)*HR1(-1)
     &  - 2.0034281719326571d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  - 6.0102845157979714d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,1,-1,-1)
     &  + 1.1285749644390297d-01*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  + 1.2020569031595942d+00*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,0,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,0,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 2.5000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,1,-1,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(-1,-1,1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,1,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1(0) *HR2(0,1)*HR3(-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,1,-1,1)
     &  + HR1(0) *HR5(0,-1,-1,-1,-1)
     &  + HR1(0) *HR5(0,-1,-1,-1,1)
     &  + HR1(0) *HR5(0,1,-1,-1,-1)
     &  + HR1(0) *HR5(0,1,-1,-1,1)
     &  - 1.3153861288973380d-02*HR1(1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  - 1.1285749644390297d-01*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR2(-1,1)*HR2(-1,1)
     &  - 1.3862943611198906d+00*HR2(-1,1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  + HR2( -1,1)*HR4(0,0,-1,-1)
     &  + HR2( -1,1)*HR4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(-1,-1,1)
     &  - 1.2020569031595942d+00*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR3(-1,-1,1)*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + HR3( -1,-1,1)*HR3(0,0,-1)
     &  + HR3( -1,-1,1)*HR3(0,0,1)
     &  + HR3( -1,-1,1)*HR3(0,1,-1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  + 4.1588830833596718d+00*HR5(-1,-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  + 9.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,1,-1)
     &  - HR6(0,0, -1,-1,-1,-1)
     &  - HR6(0,0, -1,-1,-1,1)
     &  - HR6(0,0,1, -1,-1,-1)
     &  - HR6(0,0,1, -1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  - HR6(0,1, -1,-1,1,-1)
      HY6(0,-1,-1,1,-1,-1) = 
     &  + 8.3026152441714754d-04
     &  - 4.1807859275052455d-03*HR1(-1)
     &  + 2.5458382032146317d-02*HR1(-1)*HR1(-1)
     &  + 7.9222950181883112d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.3766885054564933d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 5.0916764064292634d-02*HR1(-1)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 4.7533770109129867d-01*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(-1,-1,1)
     &  - 4.1807859275052455d-03*HR1(1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 5.0916764064292634d-02*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR2(-1,1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 4.7533770109129867d-01*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  - 1.0000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,1,-1,-1)
      HY6(0,-1,-1,1,-1,1) = 
     &  + 1.1248758327797278d-03
     &  - 6.2154684604081354d-03*HR1(-1)
     &  + 4.3992768505254480d-02*HR1(-1)*HR1(-1)
     &  + 2.4633412775717367d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 7.3900238327152102d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + 8.7985537010508960d-02*HR1(-1)*HR1(1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 1.4780047665430420d+00*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,0,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(0)*HR3(-1,-1,1)
     &  - HR1(0) *HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  - HR1(0) *HR5(0,-1,-1,1,-1)
     &  - 6.2154684604081354d-03*HR1(1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(0,-1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,-1)
     &  - 8.7985537010508960d-02*HR2(-1,1)
     &  + 5.8224052646501250d-01*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(0,-1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  - 4.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR2(0, -1)*HR4(-1,-1,-1,1)
     &  + HR2(0, -1)*HR4(0,-1,-1,-1)
     &  + HR2(0, -1)*HR4(0,-1,-1,1)
     &  + 1.4780047665430420d+00*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR4(-1,-1,-1,1)
     &  - 5.8224052646501250d-01*HR4(0,-1,-1,-1)
     &  - 5.8224052646501250d-01*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  + 1.1000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + HR6(0, -1,-1,1,-1,-1)
     &  - HR6(0, -1,0,-1,-1,-1)
     &  - HR6(0, -1,0,-1,-1,1)
      HY6(0,-1,-1,1,0,1) = 
     &  + 7.6707164631481205d-03
     &  - 3.2887560518636671d-02*HR1(-1)
     &  + 1.7614133919876854d-01*HR1(-1)*HR1(-1)
     &  + 5.9071586949367355d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 6.8538919452009434d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 1.7721476084810206d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(0)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR1(0)*HR4(0,1,-1,-1)
     &  + 3.5228267839753708d-01*HR1(-1)*HR1(1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,0,1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 3.5442952169620413d+00*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,1,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(0,1)*HR3(-1,-1,1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,-1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,-1,0,1)
     &  - HR1( -1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,1)
     &  - HR1( -1)*HR5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR1(1)*HR4(0,-1,-1,1)
     &  + HR1(0) *HR1(1)*HR4(0,-1,1,-1)
     &  + HR1(0) *HR1(1)*HR4(0,1,-1,-1)
     &  + HR1(0) *HR2(-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(0) *HR2(-1,1)*HR3(0,-1,1)
     &  + HR1(0) *HR2(-1,1)*HR3(0,1,-1)
     &  + HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1(0) *HR2(0,1)*HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR1(0)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(-1,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,1)
     &  - HR1(0) *HR5(0,-1,1,-1,-1)
     &  - HR1(0) *HR5(0,-1,1,-1,1)
     &  - HR1(0) *HR5(0,1,-1,-1,-1)
     &  - HR1(0) *HR5(0,1,-1,-1,1)
     &  - 3.2887560518636671d-02*HR1(1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - HR1(1) *HR5(0,-1,-1,0,1)
     &  - HR1(1) *HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,-1)
     &  - HR1(1) *HR5(0,-1,0,-1,1)
     &  - HR1(1) *HR5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  - 3.5228267839753708d-01*HR2(-1,1)
     &  + 1.6449340668482264d+00*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  - HR2( -1,1)*HR4(0,-1,0,1)
     &  - HR2( -1,1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR2(0, -1)*HR4(-1,-1,-1,1)
     &  + HR2(0, -1)*HR4(0,-1,-1,-1)
     &  + HR2(0, -1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(-1,-1,1)
     &  - HR2(0,1) *HR4(-1,-1,-1,1)
     &  + 3.5442952169620413d+00*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,1,-1)
     &  + 1.6449340668482264d+00*HR4(-1,-1,-1,1)
     &  - 1.6449340668482264d+00*HR4(0,-1,-1,-1)
     &  - 1.6449340668482264d+00*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(-1,-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR5(-1,-1,-1,1,1)
     &  - 1.3862943611198906d+00*HR5(-1,-1,1,-1,1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  + 1.1000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + HR6(0, -1,-1,-1,0,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + HR6(0, -1,-1,0,-1,1)
     &  + HR6(0, -1,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + HR6(0, -1,-1,1,-1,1)
     &  + HR6(0, -1,-1,1,0,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  - HR6(0, -1,0,-1,-1,-1)
     &  + HR6(0, -1,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,1,1)
     &  + HR6(0, -1,0,1,-1,-1)
     &  + HR6(0, -1,0,1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  + HR6(0, -1,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,-1,-1,1)
     &  + 5.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  + HR6(0,1, -1,-1,1,-1)
      HY6(0,-1,-1,1,1,-1) = 
     &  + 1.6236379987219099d-03
     &  - 1.0407788153459236d-02*HR1(-1)
     &  + 1.0143289758994481d-01*HR1(-1)*HR1(-1)
     &  - 1.8455064982157776d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.5365194946473328d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 2.0286579517988963d-01*HR1(-1)*HR1(1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 1.1073038989294665d+00*HR1(-1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,1)
     &  - 1.0407788153459236d-02*HR1(1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - HR1(1) *HR5(0,-1,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  - 2.0286579517988963d-01*HR2(-1,1)
     &  - 1.0626935403832139d+00*HR2(-1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR2(-1,1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,0,-1)
     &  + HR2(0, -1)*HR4(-1,-1,-1,1)
     &  - HR2(0, -1)*HR4(0,-1,-1,-1)
     &  - HR2(0, -1)*HR4(0,-1,-1,1)
     &  - 1.1073038989294665d+00*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + HR3( -1,-1,1)*HR3(0,0,-1)
     &  - 1.0626935403832139d+00*HR4(-1,-1,-1,1)
     &  + 1.0626935403832139d+00*HR4(0,-1,-1,-1)
     &  + 1.0626935403832139d+00*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + HR6(0, -1,-1,1,-1,-1)
     &  + HR6(0, -1,0,-1,-1,-1)
     &  + HR6(0, -1,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  + HR6(0,0, -1,-1,1,-1)
      HY6(0,-1,-1,1,1,1) = 
     &  + 2.4398626538751752d-03
     &  - 1.8530786065466613d-02*HR1(-1)
     &  + 2.5873953083694969d-01*HR1(-1)*HR1(-1)
     &  + 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,-1,-1)
     &  + 5.1747906167389938d-01*HR1(-1)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,0,-1)
     &  - 5.5504108664821579d-02*HR1(-1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,1)
     &  + HR1(0) *HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  + HR1(0) *HR5(0,-1,-1,1,-1)
     &  + HR1(0) *HR5(0,0,-1,-1,-1)
     &  + HR1(0) *HR5(0,0,-1,-1,1)
     &  - 1.8530786065466613d-02*HR1(1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,0,-1,-1)
     &  - 5.1747906167389938d-01*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,0,-1)
     &  + HR2( -1,1)*HR4(0,-1,-1,-1)
     &  + HR2( -1,1)*HR4(0,0,-1,-1)
     &  + HR2( -1,1)*HR4(0,0,0,-1)
     &  + 5.5504108664821579d-02*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  - HR6(0,0, -1,-1,1,-1)
     &  - HR6(0,0,0, -1,-1,-1)
     &  - HR6(0,0,0, -1,-1,1)
      HY6(0,-1,0,-1,-1,-1) = 
     &  + 1.2319555588932807d-03
     &  - 3.9238473414265508d-03*HR1(-1)
     &  + 1.1876183161309242d-02*HR1(-1)*HR1(-1)
     &  - 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.3752366322618485d-02*HR1(-1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(-1,-1,1)
     &  + 4.8045301391820142d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(-1,1,1)
     &  + 2.7725887222397812d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,1,-1,1)
     &  - 3.9238473414265508d-03*HR1(1)
     &  - 5.5504108664821579d-02*HR1(1)*HR2(-1,1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 2.3752366322618485d-02*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,-1,1)
     &  + HR2( -1,1)*HR4(-1,-1,-1,1)
     &  - 1.1100821732964315d-01*HR3(-1,-1,1)
     &  + 1.1100821732964315d-01*HR3(-1,1,1)
     &  - 9.6090602783640284d-01*HR4(-1,-1,-1,1)
     &  + 4.8045301391820142d-01*HR4(-1,-1,1,1)
     &  - 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  - 1.1000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  - HR6( -1,-1,-1,1,-1,1)
      HY6(0,-1,0,-1,-1,1) = 
     &  + 1.5215749746543206d-03
     &  - 5.1570720591993326d-03*HR1(-1)
     &  + 1.7079563083256956d-02*HR1(-1)*HR1(-1)
     &  - 1.5792167371687950d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - 4.7376502115063852d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,1,1)
     &  + 3.4159126166513913d-02*HR1(-1)*HR1(1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,1)
     &  + 9.4753004230127705d-02*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(-1,1,1)
     &  + 1.1644810529300250d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 1.1644810529300250d+00*HR1(-1)*HR3(-1,1,1)
     &  - 2.7725887222397812d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR2(-1,1)*HR3(-1,-1,1)
     &  + 7.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + HR1(0) *HR5(-1,-1,1,-1,1)
     &  - 5.1570720591993326d-03*HR1(1)
     &  - 9.4753004230127705d-02*HR1(1)*HR2(-1,1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - 1.1644810529300250d+00*HR1(1)*HR3(-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,1,-1)
     &  - 3.4159126166513913d-02*HR2(-1,1)
     &  + 2.9112026323250625d-01*HR2(-1,1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,-1,1)
     &  - HR2( -1,1)*HR4(-1,-1,-1,1)
     &  + HR2( -1,1)*HR4(0,-1,-1,-1)
     &  + HR2( -1,1)*HR4(0,-1,-1,1)
     &  + 4.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,1,1)
     &  - 1.8950600846025541d-01*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  + 1.8950600846025541d-01*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  - 2.3289621058600500d+00*HR4(-1,-1,-1,1)
     &  + 1.1644810529300250d+00*HR4(-1,-1,1,1)
     &  + 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  + 1.1000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  + HR6( -1,-1,-1,1,-1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - HR6(0, -1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  - HR6(0, -1,-1,1,-1,1)
      HY6(0,-1,0,-1,0,1) = 
     &  + 1.5846750607844291d-02
     &  - 4.0781534127300318d-02*HR1(-1)
     &  + 1.0146780316041920d-01*HR1(-1)*HR1(-1)
     &  - 6.4815974361351054d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 6.8538919452009434d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - 1.9444792308405316d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,1,1)
     &  + 2.0293560632083841d-01*HR1(-1)*HR1(1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,-1,1)
     &  + 3.8889584616810632d-01*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(-1,1,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,1)*HR3(-1,1,1)
     &  + 3.2898681336964528d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 3.2898681336964528d+00*HR1(-1)*HR3(-1,1,1)
     &  - 2.7725887222397812d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 8.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,-1,1)
     &  - 1.2000000000000000d+01*HR1(-1)*HR5(-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(-1,1,-1,1,1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,1,-1,1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(-1,1,1)
     &  + 8.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(-1,-1,1,-1,1)
     &  + 1.2000000000000000d+01*HR1(0)*HR5(-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(-1,1,-1,1,1)
     &  - 4.0781534127300318d-02*HR1(1)
     &  - 3.8889584616810632d-01*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(-1,-1,1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,1)*HR3(-1,-1,1)
     &  - 3.2898681336964528d+00*HR1(1)*HR3(-1,-1,1)
     &  + 2.7725887222397812d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(-1,-1,1,1)
     &  + 8.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 1.2000000000000000d+01*HR1(1)*HR5(-1,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(-1,-1,1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,1,-1,-1,-1)
     &  + HR1(1) *HR5(0,1,-1,1,-1)
     &  - 2.0293560632083841d-01*HR2(-1,1)
     &  + 8.2246703342411321d-01*HR2(-1,1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR2(-1,1)*HR2(-1,1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR2(-1,1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR2(-1,1)*HR3(-1,1,1)
     &  - 4.0000000000000000d+00*HR2(-1,1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR4(-1,-1,1,1)
     &  + HR2( -1,1)*HR4(0,-1,-1,-1)
     &  + HR2( -1,1)*HR4(0,-1,-1,1)
     &  + HR2( -1,1)*HR4(0,1,-1,-1)
     &  + HR2( -1,1)*HR4(0,1,-1,1)
     &  + 4.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR2(0,1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(0,1)*HR4(-1,-1,1,1)
     &  - 7.7779169233621265d-01*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(-1,-1,1)
     &  - 4.0000000000000000d+00*HR3(-1,-1,1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,1,-1)
     &  + 7.7779169233621265d-01*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,1,-1)
     &  - 6.5797362673929057d+00*HR4(-1,-1,-1,1)
     &  + 3.2898681336964528d+00*HR4(-1,-1,1,1)
     &  + 5.5451774444795624d+00*HR5(-1,-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR5(-1,-1,-1,1,1)
     &  + 2.7725887222397812d+00*HR5(-1,-1,1,-1,1)
     &  + 8.3177661667193437d+00*HR5(-1,-1,1,1,1)
     &  + 2.7725887222397812d+00*HR5(-1,1,-1,1,1)
     &  + 1.6000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  - 8.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + 1.2000000000000000d+01*HR6(-1,-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HR6(-1,-1,1,-1,1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - HR6(0, -1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  - HR6(0, -1,-1,1,-1,1)
     &  - HR6(0,1, -1,-1,-1,-1)
     &  - HR6(0,1, -1,-1,-1,1)
     &  - HR6(0,1, -1,1,-1,-1)
     &  - HR6(0,1, -1,1,-1,1)
      HY6(0,-1,0,-1,1,-1) = 
     &  + 1.9408729584885743d-03
     &  - 7.1641214217965844d-03*HR1(-1)
     &  + 2.7326526369131826d-02*HR1(-1)*HR1(-1)
     &  - 3.5678728477784371d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 1.0703618543335311d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 5.4653052738263652d-02*HR1(-1)*HR1(1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  + 2.1407237086670622d-01*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(-1,1,1)
     &  - 2.1253870807664278d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 2.1253870807664278d+00*HR1(-1)*HR3(-1,1,1)
     &  - 2.7725887222397812d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + 2.7725887222397812d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(-1,-1,1,1)
     &  - 7.1641214217965844d-03*HR1(1)
     &  - 2.1407237086670622d-01*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.1253870807664278d+00*HR1(1)*HR3(-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  - 5.4653052738263652d-02*HR2(-1,1)
     &  - 5.3134677019160696d-01*HR2(-1,1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,1)
     &  - HR2( -1,1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,1)
     &  - HR2( -1,1)*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR2(0,-1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR2(0,-1)*HR3(-1,1,1)
     &  - 4.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,1,1)
     &  - 4.2814474173341245d-01*HR3(-1,-1,1)
     &  - 4.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  + 4.2814474173341245d-01*HR3(-1,1,1)
     &  + 4.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  + 4.2507741615328557d+00*HR4(-1,-1,-1,1)
     &  - 2.1253870807664278d+00*HR4(-1,-1,1,1)
     &  + 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  + 1.1000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  + HR6( -1,-1,-1,1,-1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + HR6(0, -1,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  + HR6(0, -1,1,-1,1,-1)
      HY6(0,-1,0,-1,1,1) = 
     &  + 2.5395172104386521d-03
     &  - 1.0348119946882801d-02*HR1(-1)
     &  + 4.6548562995884288d-02*HR1(-1)*HR1(-1)
     &  - 8.9535532268006700d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  - 2.6860659680402010d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,1,-1)
     &  + 9.3097125991768577d-02*HR1(-1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,-1,1)
     &  + 5.3721319360804020d-01*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,0,-1)
     &  + 4.8045301391820142d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(-1,1,1)
     &  + 2.7725887222397812d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  + 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,1,-1)
     &  - HR1(0) *HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 2.5000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(0)*HR4(-1,-1,-1,1)
     &  + HR1(0) *HR1(0)*HR4(-1,-1,1,1)
     &  - HR1(0) *HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,1,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + HR1(0) *HR2(-1,1)*HR3(-1,-1,1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(0,-1)*HR3(-1,1,1)
     &  - 2.7725887222397812d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(-1,-1,1,1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - HR1(0) *HR5(-1,-1,1,-1,1)
     &  + HR1(0) *HR5(0,-1,-1,-1,-1)
     &  + HR1(0) *HR5(0,-1,-1,-1,1)
     &  + HR1(0) *HR5(0,-1,1,-1,-1)
     &  + HR1(0) *HR5(0,-1,1,-1,1)
     &  - 1.0348119946882801d-02*HR1(1)
     &  - 5.3721319360804020d-01*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,0,-1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,1,-1)
     &  - 9.3097125991768577d-02*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,1)
     &  + HR2( -1,1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  + HR2( -1,1)*HR4(0,-1,-1,1)
     &  + HR2( -1,1)*HR4(0,-1,1,-1)
     &  + HR2( -1,1)*HR4(0,0,-1,-1)
     &  + HR2( -1,1)*HR4(0,0,-1,1)
     &  - 1.3862943611198906d+00*HR2(0,-1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR2(0,-1)*HR3(-1,1,1)
     &  - 1.0744263872160804d+00*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,-1)
     &  + 1.0744263872160804d+00*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,0,-1)
     &  - 9.6090602783640284d-01*HR4(-1,-1,-1,1)
     &  + 4.8045301391820142d-01*HR4(-1,-1,1,1)
     &  - 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  - 1.1000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  - HR6( -1,-1,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  - HR6(0, -1,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  - HR6(0, -1,1,-1,1,-1)
     &  - HR6(0,0, -1,-1,-1,-1)
     &  - HR6(0,0, -1,-1,-1,1)
     &  - HR6(0,0, -1,1,-1,-1)
     &  - HR6(0,0, -1,1,-1,1)
      HY6(0,-1,0,1,-1,-1) = 
     &  + 2.6053618567018756d-03
     &  - 1.1024681194413928d-02*HR1(-1)
     &  + 5.7061713708030422d-02*HR1(-1)*HR1(-1)
     &  + 7.9222950181883112d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.3766885054564933d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 1.1412342741606084d-01*HR1(-1)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 4.7533770109129867d-01*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 4.8045301391820142d-01*HR1(-1)*HR3(-1,1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,1,-1)
     &  - 2.7725887222397812d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,1,-1,1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 4.8045301391820142d-01*HR1(0)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(0)*HR3(-1,1,1)
     &  - 1.1024681194413928d-02*HR1(1)
     &  + 4.7533770109129867d-01*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  + 4.8045301391820142d-01*HR1(1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  - 1.1412342741606084d-01*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR2(-1,1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR2(-1,1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR2(-1,1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,1,-1)
     &  - HR2( -1,1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  + HR2( -1,1)*HR4(0,-1,-1,1)
     &  + HR2( -1,1)*HR4(0,-1,1,-1)
     &  + HR2( -1,1)*HR4(0,1,-1,-1)
     &  - 1.3862943611198906d+00*HR2(0,-1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR2(0,-1)*HR3(-1,1,1)
     &  + 9.5067540218259734d-01*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  - 9.5067540218259734d-01*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  + 9.6090602783640284d-01*HR4(-1,-1,-1,1)
     &  - 4.8045301391820142d-01*HR4(-1,-1,1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,1,-1,1)
     &  + 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,1,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  + 1.1000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  + HR6( -1,-1,-1,1,-1,1)
     &  - 1.0000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - HR6(0, -1,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  - HR6(0, -1,1,-1,1,-1)
     &  - 6.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  - HR6(0,1, -1,1,-1,-1)
      HY6(0,-1,0,1,-1,1) = 
     &  + 3.4140232739564528d-03
     &  - 1.5921422016241947d-02*HR1(-1)
     &  + 9.6777676906532623d-02*HR1(-1)*HR1(-1)
     &  + 2.4633412775717367d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 7.3900238327152102d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(0)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,1,-1,-1)
     &  + 1.9355535381306524d-01*HR1(-1)*HR1(1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,0,1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 1.4780047665430420d+00*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(-1,1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,1,-1)
     &  - 1.1644810529300250d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 1.1644810529300250d+00*HR1(-1)*HR3(-1,1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,1,-1)
     &  + 2.7725887222397812d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,1,-1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,-1)
     &  + HR1( -1)*HR5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  + 5.8224052646501250d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  + HR1(0) *HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR1(1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1(0) *HR2(-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(0) *HR2(-1,1)*HR3(0,-1,1)
     &  + HR1(0) *HR2(-1,1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR2(0,-1)*HR3(-1,1,1)
     &  + 1.1644810529300250d+00*HR1(0)*HR3(-1,-1,1)
     &  - 1.1644810529300250d+00*HR1(0)*HR3(-1,1,1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - HR1(0) *HR5(-1,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  - HR1(0) *HR5(0,-1,-1,1,-1)
     &  - HR1(0) *HR5(0,-1,1,-1,-1)
     &  - HR1(0) *HR5(0,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,1)
     &  - HR1(0) *HR5(0,1,-1,1,-1)
     &  - 1.5921422016241947d-02*HR1(1)
     &  + 1.4780047665430420d+00*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,1,-1)
     &  + 1.1644810529300250d+00*HR1(1)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(0,1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,-1)
     &  + HR1(1) *HR5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  - 1.9355535381306524d-01*HR2(-1,1)
     &  - 2.9112026323250625d-01*HR2(-1,1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)*HR2(0,-1)
     &  + 5.8224052646501250d-01*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(0,-1)*HR2(0,-1)
     &  - HR2( -1,1)*HR2(0,-1)*HR2(0,1)
     &  + 5.8224052646501250d-01*HR2(-1,1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,1,-1)
     &  + HR2( -1,1)*HR4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,1)
     &  + HR2( -1,1)*HR4(0,-1,0,1)
     &  - HR2( -1,1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,-1)
     &  - HR2( -1,1)*HR4(0,1,-1,-1)
     &  + 1.3862943611198906d+00*HR2(0,-1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR2(0,-1)*HR3(-1,1,1)
     &  - 4.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,1,1)
     &  + HR2(0, -1)*HR4(0,-1,-1,-1)
     &  + HR2(0, -1)*HR4(0,-1,-1,1)
     &  + HR2(0, -1)*HR4(0,1,-1,-1)
     &  + HR2(0, -1)*HR4(0,1,-1,1)
     &  + 2.9560095330860840d+00*HR3(-1,-1,1)
     &  - 4.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  - 4.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,-1)
     &  - 2.9560095330860840d+00*HR3(-1,1,1)
     &  + 4.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  + 4.0000000000000000d+00*HR3(-1,1,1)*HR3(0,0,-1)
     &  + 2.3289621058600500d+00*HR4(-1,-1,-1,1)
     &  - 1.1644810529300250d+00*HR4(-1,-1,1,1)
     &  - 5.8224052646501250d-01*HR4(0,-1,-1,-1)
     &  - 5.8224052646501250d-01*HR4(0,-1,-1,1)
     &  - 5.8224052646501250d-01*HR4(0,1,-1,-1)
     &  - 5.8224052646501250d-01*HR4(0,1,-1,1)
     &  - 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  - 2.0794415416798359d+00*HR5(0,1,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  - 1.1000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  - HR6( -1,-1,-1,1,-1,1)
     &  + 1.1000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  - HR6(0, -1,0,-1,-1,-1)
     &  - HR6(0, -1,0,-1,-1,1)
     &  - HR6(0, -1,0,1,-1,-1)
     &  - HR6(0, -1,0,1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  + HR6(0, -1,1,-1,1,-1)
     &  + 6.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  + HR6(0,1, -1,1,-1,-1)
      HY6(0,-1,0,1,0,1) = 
     &  + 2.7716833977494091d-02
     &  - 9.3931044839999322d-02*HR1(-1)
     &  + 4.0587121264167682d-01*HR1(-1)*HR1(-1)
     &  + 5.9071586949367355d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 6.8538919452009434d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,1,1)
     &  + 1.7721476084810206d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(0,1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,1,1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(0)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR1(0)*HR4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,1,1,-1)
     &  + 8.1174242528335364d-01*HR1(-1)*HR1(1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,1,-1)
     &  - 3.5442952169620413d+00*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(-1,1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,1)*HR3(-1,1,1)
     &  - HR1( -1)*HR2(0,1)*HR3(0,1,-1)
     &  - 3.2898681336964528d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 3.2898681336964528d+00*HR1(-1)*HR3(-1,1,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(0,1,-1)
     &  + 2.7725887222397812d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,1,1,-1)
     &  + 8.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,-1,1)
     &  + 1.2000000000000000d+01*HR1(-1)*HR5(-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(-1,1,-1,1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,-1,0,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,-1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,1,-1)
     &  + HR1( -1)*HR5(0,1,0,1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,1,1,-1,-1)
     &  + 1.6449340668482264d+00*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1(0) *HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1(0) *HR1(1)*HR2(-1,1)*HR2(0,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR1(1)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1(0) *HR1(1)*HR4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,1,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(0,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR2(0,-1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(0,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR2(0,1)*HR3(-1,1,1)
     &  + 3.2898681336964528d+00*HR1(0)*HR3(-1,-1,1)
     &  - 3.2898681336964528d+00*HR1(0)*HR3(-1,1,1)
     &  - 8.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(-1,-1,1,-1,1)
     &  - 1.2000000000000000d+01*HR1(0)*HR5(-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(-1,1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,1,1,-1,1)
     &  - 9.3931044839999322d-02*HR1(1)
     &  + 3.5442952169620413d+00*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,0,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,1)*HR3(-1,-1,1)
     &  - HR1(1) *HR2(0,1)*HR3(0,1,-1)
     &  + 3.2898681336964528d+00*HR1(1)*HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(0,-1,-1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(0,1,-1)
     &  - 2.7725887222397812d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,-1,1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,1,1,-1)
     &  - 8.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 1.2000000000000000d+01*HR1(1)*HR5(-1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(-1,-1,1,-1,1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - HR1(1) *HR5(0,-1,-1,0,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,-1)
     &  - HR1(1) *HR5(0,-1,0,-1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,-1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,1,-1)
     &  + HR1(1) *HR5(0,1,0,1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,1,1,-1,-1)
     &  - 8.1174242528335364d-01*HR2(-1,1)
     &  - 8.2246703342411321d-01*HR2(-1,1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR2(-1,1)*HR2(-1,1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)*HR2(0,1)
     &  + 1.6449340668482264d+00*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(0,-1)*HR2(0,-1)
     &  - HR2( -1,1)*HR2(0,-1)*HR2(0,1)
     &  + 1.6449340668482264d+00*HR2(-1,1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(0,1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(0,-1,1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(0,1,1)
     &  + 4.0000000000000000d+00*HR2(-1,1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,0,1,1)
     &  - 4.0000000000000000d+00*HR2(-1,1)*HR4(0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,1,1,-1)
     &  + 1.3862943611198906d+00*HR2(0,-1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR2(0,-1)*HR3(-1,1,1)
     &  - 4.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,1,1)
     &  + HR2(0, -1)*HR4(0,-1,-1,-1)
     &  + HR2(0, -1)*HR4(0,-1,-1,1)
     &  + HR2(0, -1)*HR4(0,1,-1,-1)
     &  + HR2(0, -1)*HR4(0,1,-1,1)
     &  + 1.3862943611198906d+00*HR2(0,1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR2(0,1)*HR3(-1,1,1)
     &  - 4.0000000000000000d+00*HR2(0,1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,1)*HR4(-1,-1,1,1)
     &  + HR2(0,1) *HR4(0,1,-1,-1)
     &  + HR2(0,1) *HR4(0,1,-1,1)
     &  + 7.0885904339240826d+00*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(-1,-1,1)
     &  + 4.0000000000000000d+00*HR3(-1,-1,1)*HR3(-1,1,1)
     &  - 4.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  - 4.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,-1)
     &  - 4.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,1)
     &  - 4.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,1,-1)
     &  - 7.0885904339240826d+00*HR3(-1,1,1)
     &  + 4.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  + 4.0000000000000000d+00*HR3(-1,1,1)*HR3(0,0,-1)
     &  + 4.0000000000000000d+00*HR3(-1,1,1)*HR3(0,0,1)
     &  + 4.0000000000000000d+00*HR3(-1,1,1)*HR3(0,1,-1)
     &  + 6.5797362673929057d+00*HR4(-1,-1,-1,1)
     &  - 3.2898681336964528d+00*HR4(-1,-1,1,1)
     &  - 1.6449340668482264d+00*HR4(0,-1,-1,-1)
     &  - 1.6449340668482264d+00*HR4(0,-1,-1,1)
     &  - 1.6449340668482264d+00*HR4(0,1,-1,-1)
     &  - 1.6449340668482264d+00*HR4(0,1,-1,1)
     &  - 5.5451774444795624d+00*HR5(-1,-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR5(-1,-1,-1,1,1)
     &  - 2.7725887222397812d+00*HR5(-1,-1,1,-1,1)
     &  - 8.3177661667193437d+00*HR5(-1,-1,1,1,1)
     &  - 2.7725887222397812d+00*HR5(-1,1,-1,1,1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,1,-1,1)
     &  - 2.7725887222397812d+00*HR5(0,1,-1,-1,-1)
     &  - 2.7725887222397812d+00*HR5(0,1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,1,-1,1,-1)
     &  - 1.3862943611198906d+00*HR5(0,1,-1,1,1)
     &  - 1.3862943611198906d+00*HR5(0,1,1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,1,1,-1,1)
     &  - 1.6000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  + 8.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - 1.2000000000000000d+01*HR6(-1,-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HR6(-1,-1,1,-1,1,1)
     &  + 1.1000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + HR6(0, -1,-1,-1,0,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + HR6(0, -1,-1,0,-1,1)
     &  + HR6(0, -1,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  + HR6(0, -1,-1,1,0,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  - HR6(0, -1,0,-1,-1,-1)
     &  + HR6(0, -1,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,1,1)
     &  + 6.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,-1,-1,1)
     &  + 1.1000000000000000d+01*HR6(0,1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  + 5.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,1,-1,1,1,-1)
     &  - HR6(0,1,0,1, -1,-1)
     &  - HR6(0,1,0,1, -1,1)
     &  + 6.0000000000000000d+00*HR6(0,1,1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,1,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,1,1,-1,1,-1)
      HY6(0,-1,0,1,1,-1) = 
     &  + 4.7285610638330759d-03
     &  - 2.5649590924073888d-02*HR1(-1)
     &  + 2.1684618852447759d-01*HR1(-1)*HR1(-1)
     &  - 1.8455064982157776d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 5.5365194946473328d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 4.3369237704895519d-01*HR1(-1)*HR1(1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,0,1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  + 1.1073038989294665d+00*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,1,-1)
     &  + 2.1253870807664278d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 2.1253870807664278d+00*HR1(-1)*HR3(-1,1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(0,1,-1)
     &  + 2.7725887222397812d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,-1)
     &  - HR1( -1)*HR5(0,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - HR1( -1)*HR5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  - 1.0626935403832139d+00*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,1)
     &  - 2.1253870807664278d+00*HR1(0)*HR3(-1,-1,1)
     &  + 2.1253870807664278d+00*HR1(0)*HR3(-1,1,1)
     &  - 2.7725887222397812d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,1)
     &  - 2.5649590924073888d-02*HR1(1)
     &  - 1.1073038989294665d+00*HR1(1)*HR2(-1,1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,1,-1)
     &  - 2.1253870807664278d+00*HR1(1)*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(0,1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,1,-1)
     &  - HR1(1) *HR5(0,-1,0,-1,-1)
     &  - HR1(1) *HR5(0,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  - HR1(1) *HR5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  - 4.3369237704895519d-01*HR2(-1,1)
     &  + 5.3134677019160696d-01*HR2(-1,1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR2(-1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR2(-1,1)*HR2(0,-1)*HR2(0,-1)
     &  + HR2( -1,1)*HR2(0,-1)*HR2(0,1)
     &  - 1.0626935403832139d+00*HR2(-1,1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,1,-1)
     &  + HR2( -1,1)*HR4(-1,-1,-1,1)
     &  + HR2( -1,1)*HR4(0,-1,-1,1)
     &  - HR2( -1,1)*HR4(0,-1,0,1)
     &  - HR2( -1,1)*HR4(0,0,-1,1)
     &  - HR2( -1,1)*HR4(0,0,1,-1)
     &  - HR2( -1,1)*HR4(0,1,-1,-1)
     &  + 4.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,1,1)
     &  - HR2(0, -1)*HR4(0,-1,-1,-1)
     &  - HR2(0, -1)*HR4(0,-1,-1,1)
     &  - HR2(0, -1)*HR4(0,1,-1,-1)
     &  - HR2(0, -1)*HR4(0,1,-1,1)
     &  - 2.2146077978589331d+00*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,-1)
     &  + 2.2146077978589331d+00*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,0,-1)
     &  - 4.2507741615328557d+00*HR4(-1,-1,-1,1)
     &  + 2.1253870807664278d+00*HR4(-1,-1,1,1)
     &  + 1.0626935403832139d+00*HR4(0,-1,-1,-1)
     &  + 1.0626935403832139d+00*HR4(0,-1,-1,1)
     &  + 1.0626935403832139d+00*HR4(0,1,-1,-1)
     &  + 1.0626935403832139d+00*HR4(0,1,-1,1)
     &  - 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,-1,1)
     &  - 2.0794415416798359d+00*HR5(0,1,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  - 1.1000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  - HR6( -1,-1,-1,1,-1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,1,-1,1)
     &  + HR6(0, -1,0,-1,-1,-1)
     &  + HR6(0, -1,0,-1,-1,1)
     &  + HR6(0, -1,0,1,-1,-1)
     &  + HR6(0, -1,0,1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  + HR6(0,0, -1,-1,1,-1)
     &  + HR6(0,0, -1,1,-1,-1)
     &  + HR6(0,0, -1,1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,-1,-1,1)
     &  + HR6(0,0,1, -1,1,-1)
     &  + 6.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  + HR6(0,1, -1,1,-1,-1)
      HY6(0,-1,0,1,1,1) = 
     &  + 6.8128613470480598d-03
     &  - 4.4023596785801827d-02*HR1(-1)
     &  + 5.4116161685556909d-01*HR1(-1)*HR1(-1)
     &  + 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  + HR1( -1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,0,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,1,-1,-1)
     &  + 1.0823232337111381d+00*HR1(-1)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,0,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,0,1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 5.5504108664821579d-02*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 4.8045301391820142d-01*HR1(-1)*HR3(-1,1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,1,-1)
     &  - 2.7725887222397812d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,0,-1,-1)
     &  + HR1( -1)*HR5(0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 3.3333333333333333d-01*HR1(0)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 3.3333333333333333d-01*HR1(0)*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + HR1(0) *HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 2.5000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR1(0)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,1,-1,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,0,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,1,-1,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,1)
     &  - HR1(0) *HR2(-1,1)*HR3(-1,-1,1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,0,-1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,0,1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,1,-1)
     &  + 4.8045301391820142d-01*HR1(0)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(0)*HR3(-1,1,1)
     &  + 2.7725887222397812d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,1)
     &  + 7.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + HR1(0) *HR5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  + HR1(0) *HR5(0,-1,-1,1,-1)
     &  + HR1(0) *HR5(0,0,-1,-1,-1)
     &  + HR1(0) *HR5(0,0,-1,-1,1)
     &  + HR1(0) *HR5(0,0,1,-1,-1)
     &  + HR1(0) *HR5(0,0,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,1)
     &  + HR1(0) *HR5(0,1,-1,1,-1)
     &  - 4.4023596785801827d-02*HR1(1)
     &  + 5.5504108664821579d-02*HR1(1)*HR2(-1,1)
     &  + 4.8045301391820142d-01*HR1(1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,0,-1,-1)
     &  + HR1(1) *HR5(0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  - 1.0823232337111381d+00*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR2(-1,1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR2(-1,1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR2(-1,1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,1,-1)
     &  - HR2( -1,1)*HR4(-1,-1,-1,1)
     &  + HR2( -1,1)*HR4(0,-1,-1,-1)
     &  + HR2( -1,1)*HR4(0,0,-1,-1)
     &  + HR2( -1,1)*HR4(0,0,0,-1)
     &  + HR2( -1,1)*HR4(0,0,0,1)
     &  + HR2( -1,1)*HR4(0,0,1,-1)
     &  + HR2( -1,1)*HR4(0,1,-1,-1)
     &  + 1.1100821732964315d-01*HR3(-1,-1,1)
     &  - 1.1100821732964315d-01*HR3(-1,1,1)
     &  + 9.6090602783640284d-01*HR4(-1,-1,-1,1)
     &  - 4.8045301391820142d-01*HR4(-1,-1,1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,1,-1,1)
     &  + 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,1,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  + 1.1000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  + HR6( -1,-1,-1,1,-1,1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  - HR6(0,0, -1,-1,1,-1)
     &  - HR6(0,0,0, -1,-1,-1)
     &  - HR6(0,0,0, -1,-1,1)
     &  - HR6(0,0,0,1, -1,-1)
     &  - HR6(0,0,0,1, -1,1)
     &  - 3.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,1,-1,-1,1)
     &  - HR6(0,0,1, -1,1,-1)
     &  - 6.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  - HR6(0,1, -1,1,-1,-1)
      HY6(0,-1,1,-1,-1,-1) = 
     &  + 1.3039042223628135d-03
     &  - 8.9770798664632968d-03*HR1(-1)
     &  - 6.3399291898262058d-02*HR1(-1)*HR1(-1)
     &  + 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.5504108664821579d-02*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.2679858379652411d-01*HR1(-1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.5504108664821579d-02*HR1(-1)*HR2(-1,1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 5.5504108664821579d-02*HR1(0)*HR2(-1,1)
     &  - 8.9770798664632968d-03*HR1(1)
     &  + 5.5504108664821579d-02*HR1(1)*HR2(0,-1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,-1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 1.2679858379652411d-01*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR2( -1,1)*HR4(0,-1,-1,-1)
     &  + 5.5504108664821579d-02*HR3(-1,-1,1)
     &  - 5.5504108664821579d-02*HR3(0,-1,-1)
     &  - 5.5504108664821579d-02*HR3(0,-1,1)
     &  + 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  + 7.2067952087730213d-01*HR4(0,-1,-1,-1)
     &  + 4.8045301391820142d-01*HR4(0,-1,-1,1)
     &  + 2.4022650695910071d-01*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  - 4.1588830833596718d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  + 1.0000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + HR6(0, -1,1,-1,-1,-1)
      HY6(0,-1,1,-1,-1,1) = 
     &  + 1.7625670847439686d-03
     &  - 1.3266229797875123d-02*HR1(-1)
     &  - 1.1758989653041252d-01*HR1(-1)*HR1(-1)
     &  + 1.5792167371687950d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 4.7376502115063852d-02*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 4.7376502115063852d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 9.4753004230127705d-02*HR1(-1)*HR1(0)*HR1(1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - 2.3517979306082505d-01*HR1(-1)*HR1(1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 9.4753004230127705d-02*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + 9.4753004230127705d-02*HR1(-1)*HR2(0,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 1.1644810529300250d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 9.4753004230127705d-02*HR1(0)*HR2(-1,1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(0) *HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  + HR1(0) *HR5(0,-1,1,-1,-1)
     &  - 1.3266229797875123d-02*HR1(1)
     &  + 9.4753004230127705d-02*HR1(1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - 1.1644810529300250d+00*HR1(1)*HR3(0,-1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + 2.3517979306082505d-01*HR2(-1,1)
     &  - 5.8224052646501250d-01*HR2(-1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR2(-1,1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR2(0, -1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,1)
     &  - HR2(0, -1)*HR4(0,-1,1,-1)
     &  + 9.4753004230127705d-02*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - 9.4753004230127705d-02*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,-1,1)
     &  - 9.4753004230127705d-02*HR3(0,-1,1)
     &  + 5.8224052646501250d-01*HR4(-1,-1,-1,1)
     &  + 1.7467215793950375d+00*HR4(0,-1,-1,-1)
     &  + 1.1644810529300250d+00*HR4(0,-1,-1,1)
     &  + 5.8224052646501250d-01*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  + 4.1588830833596718d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  - 9.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
      HY6(0,-1,1,-1,0,1) = 
     &  + 1.2115413251344344d-02
     &  - 7.1422694109800329d-02*HR1(-1)
     &  - 4.2377986529412007d-01*HR1(-1)*HR1(-1)
     &  + 6.4815974361351054d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 6.8538919452009434d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 1.9444792308405316d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  + 1.9444792308405316d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - 3.8889584616810632d-01*HR1(-1)*HR1(0)*HR1(1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,1,-1)
     &  - 8.4755973058824014d-01*HR1(-1)*HR1(1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,0,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 3.8889584616810632d-01*HR1(-1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,1,-1)
     &  + 3.8889584616810632d-01*HR1(-1)*HR2(0,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(0,1)*HR3(-1,-1,1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 3.2898681336964528d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,1)
     &  + HR1( -1)*HR5(0,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,1,-1)
     &  - HR1( -1)*HR5(0,1,-1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,1,-1)
     &  + 3.8889584616810632d-01*HR1(0)*HR2(-1,1)
     &  + HR1(0) *HR2(-1,1)*HR3(-1,-1,1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(-1,-1,1,-1,1)
     &  + 6.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + 6.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,1)
     &  - 7.1422694109800329d-02*HR1(1)
     &  + 3.8889584616810632d-01*HR1(1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - 3.2898681336964528d+00*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,1)
     &  + HR1(1) *HR5(0,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,1,-1)
     &  - HR1(1) *HR5(0,1,-1,-1,-1)
     &  + 8.4755973058824014d-01*HR2(-1,1)
     &  - 1.6449340668482264d+00*HR2(-1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR2(-1,1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(-1,-1,-1,1)
     &  + HR2( -1,1)*HR4(0,-1,0,1)
     &  + HR2( -1,1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,1)
     &  - HR2( -1,1)*HR4(0,1,-1,-1)
     &  - HR2(0, -1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,1)
     &  - HR2(0, -1)*HR4(0,-1,1,-1)
     &  - HR2(0,1) *HR4(-1,-1,-1,1)
     &  + 3.8889584616810632d-01*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - HR3( -1,-1,1)*HR3(0,1,-1)
     &  - 3.8889584616810632d-01*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,-1,1)
     &  - 3.8889584616810632d-01*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR3(0,-1,1)*HR3(0,-1,1)
     &  + 1.6449340668482264d+00*HR4(-1,-1,-1,1)
     &  + 4.9348022005446793d+00*HR4(0,-1,-1,-1)
     &  + 3.2898681336964528d+00*HR4(0,-1,-1,1)
     &  + 1.6449340668482264d+00*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR5(-1,-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR5(-1,-1,-1,1,1)
     &  - 1.3862943611198906d+00*HR5(-1,-1,1,-1,1)
     &  + 4.1588830833596718d+00*HR5(0,-1,-1,-1,-1)
     &  + 4.1588830833596718d+00*HR5(0,-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,1,-1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,1,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,1,-1,1)
     &  - 6.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  - 9.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,0,1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,0,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,0,1,1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,0,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,1,1)
     &  - HR6(0, -1,0,1,-1,-1)
     &  - HR6(0, -1,0,1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  - HR6(0, -1,1,-1,0,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,-1,1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,1,-1,1)
     &  + HR6(0,1, -1,-1,-1,-1)
     &  + HR6(0,1, -1,-1,-1,1)
      HY6(0,-1,1,-1,1,-1) = 
     &  + 2.5362110264309006d-03
     &  - 2.2023530392320487d-02*HR1(-1)
     &  - 3.2105039383616431d-01*HR1(-1)*HR1(-1)
     &  + 3.5678728477784371d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.0703618543335311d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 1.0703618543335311d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 2.1407237086670622d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 6.4210078767232862d-01*HR1(-1)*HR1(1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 2.1407237086670622d-01*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.1407237086670622d-01*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 2.1253870807664278d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + 8.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  + 1.2000000000000000d+01*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 2.1407237086670622d-01*HR1(0)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,-1)
     &  - 2.2023530392320487d-02*HR1(1)
     &  + 2.1407237086670622d-01*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + 2.1253870807664278d+00*HR1(1)*HR3(0,-1,-1)
     &  - 2.7725887222397812d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,-1)
     &  + 8.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  + 1.2000000000000000d+01*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + 6.4210078767232862d-01*HR2(-1,1)
     &  + 1.0626935403832139d+00*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(0,-1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR2(-1,1)*HR3(0,0,-1)
     &  + 4.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  + HR2(0, -1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,1)
     &  + HR2(0, -1)*HR4(0,-1,1,-1)
     &  + 2.1407237086670622d-01*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  - 2.1407237086670622d-01*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,-1,1)
     &  - 2.1407237086670622d-01*HR3(0,-1,1)
     &  - 1.0626935403832139d+00*HR4(-1,-1,-1,1)
     &  - 3.1880806211496417d+00*HR4(0,-1,-1,-1)
     &  - 2.1253870807664278d+00*HR4(0,-1,-1,1)
     &  - 1.0626935403832139d+00*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  + 4.8520302639196171d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  - 1.5000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  - 8.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  + HR6(0, -1,0,-1,1,-1)
     &  - HR6(0, -1,1,-1,-1,-1)
     &  - 1.2000000000000000d+01*HR6(0,0,-1,-1,-1,-1)
      HY6(0,-1,1,-1,1,1) = 
     &  + 3.7985276723318908d-03
     &  - 3.8880058841843904d-02*HR1(-1)
     &  - 9.6240249776535762d-01*HR1(-1)*HR1(-1)
     &  + 8.9535532268006700d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 2.6860659680402010d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 2.6860659680402010d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 5.3721319360804020d-01*HR1(-1)*HR1(0)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,-1,-1)
     &  - 1.9248049955307152d+00*HR1(-1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,0,-1)
     &  - 5.3721319360804020d-01*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,0,-1)
     &  + 5.3721319360804020d-01*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,0,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,-1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,0,-1)
     &  - HR1(0) *HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(-1,-1,-1,1)
     &  + 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR1(0)*HR4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 4.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,-1,-1)
     &  + 5.3721319360804020d-01*HR1(0)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(0,0,-1)
     &  + HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,-1)
     &  + HR1(0) *HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  + HR1(0) *HR5(0,-1,0,-1,-1)
     &  + HR1(0) *HR5(0,-1,0,-1,1)
     &  - HR1(0) *HR5(0,-1,1,-1,-1)
     &  - 3.8880058841843904d-02*HR1(1)
     &  + 5.3721319360804020d-01*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,0,-1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,-1,-1)
     &  + 2.7725887222397812d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - HR1(1) *HR5(0,-1,0,-1,-1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,0,-1)
     &  + 1.9248049955307152d+00*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR2(-1,1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(0,0,-1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(0,0,0,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  + 5.3721319360804020d-01*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - HR3( -1,-1,1)*HR3(0,0,-1)
     &  - 5.3721319360804020d-01*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,-1,1)
     &  + HR3(0, -1,-1)*HR3(0,0,-1)
     &  - 5.3721319360804020d-01*HR3(0,-1,1)
     &  + HR3(0, -1,1)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  + 7.2067952087730213d-01*HR4(0,-1,-1,-1)
     &  + 4.8045301391820142d-01*HR4(0,-1,-1,1)
     &  + 2.4022650695910071d-01*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  - 4.8520302639196171d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  + 1.4000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  - HR6(0, -1,0,-1,1,-1)
     &  + HR6(0, -1,1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + HR6(0,0, -1,-1,-1,1)
     &  - HR6(0,0, -1,0,-1,-1)
     &  - HR6(0,0, -1,0,-1,1)
      HY6(0,-1,1,0,1,-1) = 
     &  + 9.2351089277639264d-03
     &  - 6.4839845177998156d-02*HR1(-1)
     &  - 7.0660402489210777d-01*HR1(-1)*HR1(-1)
     &  + 8.4702535467447475d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 2.5410760640234242d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 2.5410760640234242d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(0,0,-1,1)
     &  - HR1( -1)*HR1(-1)*HR4(0,0,1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - 5.0821521280468485d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  - 1.4132080497842155d+00*HR1(-1)*HR1(1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,0,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 5.0821521280468485d-01*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,1,-1)
     &  + 5.0821521280468485d-01*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,1,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 2.1253870807664278d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR3(0,-1,1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR3(0,1,-1)
     &  - 2.7725887222397812d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,0,1,-1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,-1,1)
     &  + 8.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  + 1.2000000000000000d+01*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,-1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + 5.0821521280468485d-01*HR1(0)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,1)
     &  - 6.4839845177998156d-02*HR1(1)
     &  + 5.0821521280468485d-01*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,1,-1)
     &  + 1.0626935403832139d+00*HR1(1)*HR3(-1,-1,1)
     &  + 2.1253870807664278d+00*HR1(1)*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR1(1)*HR3(0,-1,1)
     &  + 1.0626935403832139d+00*HR1(1)*HR3(0,1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,0,1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 8.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  + 1.2000000000000000d+01*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,-1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  + 1.4132080497842155d+00*HR2(-1,1)
     &  - 5.3134677019160696d-01*HR2(-1,1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)*HR2(0,-1)
     &  + 1.0626935403832139d+00*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(0,-1)*HR2(0,-1)
     &  - HR2( -1,1)*HR2(0,-1)*HR2(0,1)
     &  + 1.0626935403832139d+00*HR2(-1,1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR2(-1,1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR2(-1,1)*HR3(0,0,-1)
     &  - 1.3862943611198906d+00*HR2(-1,1)*HR3(0,0,1)
     &  - 1.3862943611198906d+00*HR2(-1,1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  + HR2( -1,1)*HR4(0,-1,0,1)
     &  + HR2( -1,1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,1,1)
     &  + HR2(0, -1)*HR4(0,1,-1,-1)
     &  + HR2(0, -1)*HR4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(-1,-1,1)
     &  + 5.0821521280468485d-01*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR3(-1,-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  + HR3( -1,-1,1)*HR3(0,-1,1)
     &  + HR3( -1,-1,1)*HR3(0,1,-1)
     &  - 5.0821521280468485d-01*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,-1,1)
     &  - 5.0821521280468485d-01*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR3(0,-1,1)*HR3(0,-1,1)
     &  - 3.1880806211496417d+00*HR4(0,-1,-1,-1)
     &  - 3.1880806211496417d+00*HR4(0,-1,-1,1)
     &  - 2.1253870807664278d+00*HR4(0,-1,1,-1)
     &  - 2.1253870807664278d+00*HR4(0,-1,1,1)
     &  - 1.0626935403832139d+00*HR4(0,1,-1,-1)
     &  - 1.0626935403832139d+00*HR4(0,1,-1,1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  + 4.1588830833596718d+00*HR5(-1,-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,1,-1,1)
     &  + 4.8520302639196171d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,0,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,0,1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,0,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,1,1,-1)
     &  + 1.3862943611198906d+00*HR5(0,0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,0,-1,1,-1)
     &  + 2.7725887222397812d+00*HR5(0,0,-1,1,1)
     &  + 1.3862943611198906d+00*HR5(0,0,1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,0,1,-1,1)
     &  + 2.7725887222397812d+00*HR5(0,1,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  + 9.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  - 1.5000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  - 8.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + HR6(0, -1,-1,0,-1,1)
     &  - HR6(0, -1,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  - HR6(0, -1,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,1,1,-1)
     &  - 5.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  - HR6(0, -1,1,-1,1,-1)
     &  - HR6(0, -1,1,0,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  - 1.2000000000000000d+01*HR6(0,0,-1,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,-1,1,1)
     &  - 6.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,1,1,-1)
     &  - 6.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,1,-1,1,-1)
     &  - 1.0000000000000000d+01*HR6(0,1,-1,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  - HR6(0,1, -1,1,-1,-1)
      HY6(0,-1,1,0,1,1) = 
     &  + 1.2882812075917590d-02
     &  - 1.0652326514231139d-01*HR1(-1)
     &  - 2.0400860272155532d+00*HR1(-1)*HR1(-1)
     &  + 2.0034281719326571d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 6.0102845157979714d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,0,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 6.0102845157979714d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,0,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  - 1.2020569031595942d+00*HR1(-1)*HR1(0)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(0)*HR4(0,-1,0,1)
     &  + HR1( -1)*HR1(0)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,1,-1,-1)
     &  - 4.0801720544311065d+00*HR1(-1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,0,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 1.2020569031595942d+00*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,0,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,0,1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,1,-1)
     &  + 1.2020569031595942d+00*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,0,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(0,1,-1)
     &  + 2.7725887222397812d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,0,1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,-1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,-1)
     &  - HR1( -1)*HR5(0,-1,0,1,-1)
     &  - HR1( -1)*HR5(0,-1,1,-1,-1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,1,-1)
     &  - 5.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,-1)
     &  + HR1( -1)*HR5(0,0,1,0,-1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + 2.5000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)*HR2(0,1)
     &  + 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(0,-1,-1,-1)
     &  + 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(0,-1,-1,1)
     &  + HR1(0) *HR1(0)*HR4(0,-1,1,-1)
     &  + HR1(0) *HR1(0)*HR4(0,-1,1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR1(1)*HR4(0,-1,0,1)
     &  + HR1(0) *HR1(1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,1,-1,-1)
     &  + 1.2020569031595942d+00*HR1(0)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(0,0,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(0,1,-1)
     &  + HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,-1,1)
     &  + HR1(0) *HR2(0,1)*HR3(-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,1,-1,1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  - HR1(0) *HR5(0,-1,-1,0,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  + HR1(0) *HR5(0,-1,0,-1,-1)
     &  - HR1(0) *HR5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,0,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,-1)
     &  - HR1(0) *HR5(0,-1,1,-1,1)
     &  - HR1(0) *HR5(0,-1,1,0,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,0,-1,1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,0,1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,1)
     &  - HR1(0) *HR5(0,1,-1,1,-1)
     &  - 1.0652326514231139d-01*HR1(1)
     &  + 1.2020569031595942d+00*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,0,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,0,1)
     &  - 2.4022650695910071d-01*HR1(1)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(1)*HR3(0,-1,1)
     &  - 2.4022650695910071d-01*HR1(1)*HR3(0,1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,0,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,0,1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - HR1(1) *HR5(0,-1,0,-1,-1)
     &  - HR1(1) *HR5(0,-1,0,1,-1)
     &  - HR1(1) *HR5(0,-1,1,-1,-1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,1,-1)
     &  - 5.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,-1)
     &  + HR1(1) *HR5(0,0,1,0,-1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  + 4.0801720544311065d+00*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR2(-1,1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR2(-1,1)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR2(-1,1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(0,0,-1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(0,0,1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(0,0,0,1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(-1,-1,1)
     &  + 1.2020569031595942d+00*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR3(-1,-1,1)*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - HR3( -1,-1,1)*HR3(0,0,-1)
     &  - HR3( -1,-1,1)*HR3(0,0,1)
     &  - HR3( -1,-1,1)*HR3(0,1,-1)
     &  - 1.2020569031595942d+00*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,-1,1)
     &  + HR3(0, -1,-1)*HR3(0,0,-1)
     &  + HR3(0, -1,-1)*HR3(0,0,1)
     &  - 1.2020569031595942d+00*HR3(0,-1,1)
     &  + HR3(0, -1,1)*HR3(0,0,-1)
     &  + HR3(0, -1,1)*HR3(0,0,1)
     &  + 7.2067952087730213d-01*HR4(0,-1,-1,-1)
     &  + 7.2067952087730213d-01*HR4(0,-1,-1,1)
     &  + 4.8045301391820142d-01*HR4(0,-1,1,-1)
     &  + 4.8045301391820142d-01*HR4(0,-1,1,1)
     &  + 2.4022650695910071d-01*HR4(0,1,-1,-1)
     &  + 2.4022650695910071d-01*HR4(0,1,-1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  - 4.1588830833596718d+00*HR5(-1,-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,1,-1,1)
     &  - 4.8520302639196171d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,0,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,0,1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,0,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,1,1,-1)
     &  - 1.3862943611198906d+00*HR5(0,0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,0,-1,1,-1)
     &  - 2.7725887222397812d+00*HR5(0,0,-1,1,1)
     &  - 1.3862943611198906d+00*HR5(0,0,1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,0,1,-1,1)
     &  - 2.7725887222397812d+00*HR5(0,1,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  - 9.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  + 1.4000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,0,-1,1)
     &  + HR6(0, -1,-1,0,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,1,-1,-1)
     &  + HR6(0, -1,0,1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,1,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + HR6(0, -1,1,-1,-1,1)
     &  + HR6(0, -1,1,-1,1,-1)
     &  + HR6(0, -1,1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + HR6(0,0, -1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  - HR6(0,0, -1,0,-1,-1)
     &  - HR6(0,0, -1,0,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,1,1,-1)
     &  + 7.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  + 5.0000000000000000d+00*HR6(0,0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,-1,1,-1)
     &  - HR6(0,0,1,0, -1,-1)
     &  - HR6(0,0,1,0, -1,1)
     &  + 1.0000000000000000d+01*HR6(0,1,-1,-1,-1,-1)
     &  + 6.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  + HR6(0,1, -1,1,-1,-1)
      HY6(0,-1,1,1,-1,-1) = 
     &  + 4.0996568687826499d-03
     &  - 4.8888373630876441d-02*HR1(-1)
     &  + 3.0168989201460834d-01*HR1(-1)*HR1(-1)
     &  - 7.9222950181883112d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 2.3766885054564933d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.3766885054564933d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 4.7533770109129867d-01*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 6.0337978402921669d-01*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 4.7533770109129867d-01*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - 4.7533770109129867d-01*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - 1.2011325347955035d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 4.7533770109129867d-01*HR1(0)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(0)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,-1,1)
     &  - 4.8888373630876441d-02*HR1(1)
     &  - 4.7533770109129867d-01*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - HR1(1) *HR5(0,-1,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  - 6.0337978402921669d-01*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  - HR2( -1,1)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  - 4.7533770109129867d-01*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + 4.7533770109129867d-01*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,-1,1)
     &  + 4.7533770109129867d-01*HR3(0,-1,1)
     &  - 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  - 4.8045301391820142d-01*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,1,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,1,-1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,0,-1,1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  - HR6(0, -1,0,-1,1,-1)
     &  - HR6(0, -1,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  - HR6(0,0, -1,1,-1,-1)
      HY6(0,-1,1,1,-1,1) = 
     &  + 6.1007809756250693d-03
     &  - 8.4962399751964626d-02*HR1(-1)
     &  + 1.2185212069612250d+00*HR1(-1)*HR1(-1)
     &  - 2.4633412775717367d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 7.3900238327152102d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.4556013161625312d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  - 7.3900238327152102d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.4780047665430420d+00*HR1(-1)*HR1(0)*HR1(1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + 2.4370424139224501d+00*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,0,-1)
     &  + 1.4780047665430420d+00*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,0,-1)
     &  - 1.4780047665430420d+00*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  + 9.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,-1)
     &  - 2.9112026323250625d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 5.8224052646501250d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 1.4780047665430420d+00*HR1(0)*HR2(-1,1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,0,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,-1,1)
     &  - 5.8224052646501250d-01*HR1(0)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(0)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(0)*HR3(0,-1,1)
     &  + HR1(0) *HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - HR1(0) *HR5(0,-1,-1,1,-1)
     &  - HR1(0) *HR5(0,-1,0,-1,-1)
     &  - HR1(0) *HR5(0,-1,0,-1,1)
     &  - HR1(0) *HR5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,1)
     &  - HR1(0) *HR5(0,0,-1,1,-1)
     &  - 8.4962399751964626d-02*HR1(1)
     &  - 1.4780047665430420d+00*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(0,0,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  + 9.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,-1)
     &  - 2.4370424139224501d+00*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,0,-1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(0,0,0,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  + HR2(0, -1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  + HR2(0, -1)*HR4(0,-1,-1,1)
     &  + HR2(0, -1)*HR4(0,-1,1,-1)
     &  - 1.4780047665430420d+00*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,-1)
     &  + 1.4780047665430420d+00*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,0,-1)
     &  + 1.4780047665430420d+00*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR3(0,-1,1)*HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR4(-1,-1,-1,1)
     &  - 1.1644810529300250d+00*HR4(0,-1,-1,-1)
     &  - 5.8224052646501250d-01*HR4(0,-1,-1,1)
     &  - 5.8224052646501250d-01*HR4(0,-1,1,-1)
     &  - 5.8224052646501250d-01*HR4(0,0,-1,-1)
     &  - 5.8224052646501250d-01*HR4(0,0,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,1,-1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,-1,1)
     &  + HR6(0, -1,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  + HR6(0, -1,0,-1,1,-1)
     &  + HR6(0, -1,1,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  + HR6(0,0, -1,-1,0,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,0,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,0,-1,1)
     &  + HR6(0,0, -1,0,1,-1)
     &  + HR6(0,0, -1,1,-1,-1)
     &  + HR6(0,0, -1,1,0,-1)
     &  + 9.0000000000000000d+00*HR6(0,0,0,-1,-1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,0,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,0,0,-1,1,-1)
      HY6(0,-1,1,1,0,1) = 
     &  + 3.2237101006270680d-02
     &  - 3.3639964524208459d-01*HR1(-1)
     &  + 2.6542655863928879d+00*HR1(-1)*HR1(-1)
     &  - 5.9071586949367355d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 6.8538919452009434d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 1.7721476084810206d+00*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  - 1.7721476084810206d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,0,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 3.5442952169620413d+00*HR1(-1)*HR1(0)*HR1(1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,0,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,0,1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,1,-1,-1)
     &  + 5.3085311727857758d+00*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,0,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  + 3.5442952169620413d+00*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,1)
     &  + HR1( -1)*HR2(-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,1,-1)
     &  - 3.5442952169620413d+00*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,0,1)
     &  - HR1( -1)*HR2(0,1)*HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,-1,0,1)
     &  - HR1( -1)*HR5(0,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,1)
     &  + HR1( -1)*HR5(0,-1,0,1,-1)
     &  + 9.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,0,-1)
     &  - HR1( -1)*HR5(0,0,-1,0,1)
     &  + HR1( -1)*HR5(0,0,-1,1,-1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,0,1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,1,0,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  - 8.2246703342411321d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 1.6449340668482264d+00*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1(0) *HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1(0) *HR1(1)*HR4(0,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,0,1)
     &  - HR1(0) *HR1(1)*HR4(0,0,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,0,1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,1,-1,-1)
     &  - 3.5442952169620413d+00*HR1(0)*HR2(-1,1)
     &  - HR1(0) *HR2(-1,1)*HR3(-1,-1,1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,0,-1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,0,1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,1,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,-1,1)
     &  - HR1(0) *HR2(0,1)*HR3(-1,-1,1)
     &  - 1.6449340668482264d+00*HR1(0)*HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR1(0)*HR3(0,-1,-1)
     &  + 1.6449340668482264d+00*HR1(0)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(-1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  + HR1(0) *HR5(0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,1)
     &  - HR1(0) *HR5(0,-1,0,-1,-1)
     &  + HR1(0) *HR5(0,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,-1,0,1,1)
     &  - HR1(0) *HR5(0,-1,1,-1,-1)
     &  - HR1(0) *HR5(0,-1,1,-1,1)
     &  + HR1(0) *HR5(0,-1,1,0,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,-1)
     &  - HR1(0) *HR5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,0,-1,1,1)
     &  + HR1(0) *HR5(0,0,1,-1,-1)
     &  + HR1(0) *HR5(0,0,1,-1,1)
     &  + HR1(0) *HR5(0,1,-1,-1,-1)
     &  + HR1(0) *HR5(0,1,-1,-1,1)
     &  - 3.3639964524208459d-01*HR1(1)
     &  - 3.5442952169620413d+00*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,0,1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(0,-1,-1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - HR1(1) *HR5(0,-1,-1,0,1)
     &  - HR1(1) *HR5(0,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  - HR1(1) *HR5(0,-1,0,-1,1)
     &  + HR1(1) *HR5(0,-1,0,1,-1)
     &  + 9.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,-1,0,-1)
     &  - HR1(1) *HR5(0,0,-1,0,1)
     &  + HR1(1) *HR5(0,0,-1,1,-1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,0,1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,1,0,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  - 5.3085311727857758d+00*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(0,0,0,1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  + HR2(0, -1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  + HR2(0, -1)*HR4(0,-1,-1,1)
     &  + HR2(0, -1)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(-1,-1,1)
     &  + HR2(0,1) *HR4(-1,-1,-1,1)
     &  - 3.5442952169620413d+00*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,1,-1)
     &  + 3.5442952169620413d+00*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,0,1)
     &  + 3.5442952169620413d+00*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR3(0,-1,1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR3(0,-1,1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,1)*HR3(0,0,1)
     &  - 1.6449340668482264d+00*HR4(-1,-1,-1,1)
     &  - 3.2898681336964528d+00*HR4(0,-1,-1,-1)
     &  - 1.6449340668482264d+00*HR4(0,-1,-1,1)
     &  - 1.6449340668482264d+00*HR4(0,-1,1,-1)
     &  - 1.6449340668482264d+00*HR4(0,0,-1,-1)
     &  - 1.6449340668482264d+00*HR4(0,0,-1,1)
     &  + 1.3862943611198906d+00*HR5(-1,-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR5(-1,-1,-1,1,1)
     &  + 1.3862943611198906d+00*HR5(-1,-1,1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,0,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,0,1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,0,1)
     &  - 2.0794415416798359d+00*HR5(0,0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,0,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + HR6(0, -1,-1,1,-1,1)
     &  + HR6(0, -1,-1,1,0,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  + 5.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  + HR6(0, -1,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,1,-1,-1)
     &  - HR6(0, -1,0,1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,1,1,-1)
     &  + HR6(0, -1,1,-1,-1,-1)
     &  + HR6(0, -1,1,-1,0,1)
     &  + HR6(0, -1,1,-1,1,-1)
     &  - HR6(0, -1,1,0,1,-1)
     &  - 6.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  + HR6(0,0, -1,-1,0,-1)
     &  + HR6(0,0, -1,-1,0,1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,0,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,0,1,1)
     &  - HR6(0,0, -1,1,-1,-1)
     &  - HR6(0,0, -1,1,-1,1)
     &  + HR6(0,0, -1,1,0,-1)
     &  + HR6(0,0, -1,1,0,1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,1,1,-1)
     &  + 9.0000000000000000d+00*HR6(0,0,0,-1,-1,-1)
     &  + 9.0000000000000000d+00*HR6(0,0,0,-1,-1,1)
     &  + 6.0000000000000000d+00*HR6(0,0,0,-1,1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,0,-1,1,1)
     &  + 3.0000000000000000d+00*HR6(0,0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,0,1,-1,1)
     &  - 5.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,1,-1,-1,1)
     &  - HR6(0,0,1, -1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,0,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,0,-1,1)
     &  - 5.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  - HR6(0,1, -1,-1,1,-1)
      HY6(0,-1,1,1,1,-1) = 
     &  + 1.0233318613285890d-02
     &  - 1.9778512187332765d-01*HR1(-1)
     &  - 5.3409449824807409d-01*HR1(-1)*HR1(-1)
     &  + 1.8455064982157776d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.5365194946473328d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.6567338509580348d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 5.5365194946473328d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 1.1073038989294665d+00*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  - 1.0681889964961481d+00*HR1(-1)*HR1(1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,0,-1)
     &  - 1.1073038989294665d+00*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,0,-1)
     &  + 1.1073038989294665d+00*HR1(-1)*HR2(0,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,0,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,0,-1)
     &  - HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,0,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,-1)
     &  + 1.1552453009332421d-01*HR1(0)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.3134677019160696d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  + 1.0626935403832139d+00*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + 1.1073038989294665d+00*HR1(0)*HR2(-1,1)
     &  + 1.0626935403832139d+00*HR1(0)*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR1(0)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(0)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,0,-1,1)
     &  - 1.9778512187332765d-01*HR1(1)
     &  + 1.1073038989294665d+00*HR1(1)*HR2(0,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,0,-1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,0,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,-1)
     &  + 1.0681889964961481d+00*HR2(-1,1)
     &  - HR2( -1,1)*HR4(0,-1,-1,-1)
     &  - HR2( -1,1)*HR4(0,0,-1,-1)
     &  - HR2( -1,1)*HR4(0,0,0,-1)
     &  - HR2(0, -1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  - HR2(0, -1)*HR4(0,-1,-1,1)
     &  - HR2(0, -1)*HR4(0,-1,1,-1)
     &  + 1.1073038989294665d+00*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - HR3( -1,-1,1)*HR3(0,0,-1)
     &  - 1.1073038989294665d+00*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,-1,1)
     &  + HR3(0, -1,-1)*HR3(0,0,-1)
     &  - 1.1073038989294665d+00*HR3(0,-1,1)
     &  + HR3(0, -1,1)*HR3(0,0,-1)
     &  + 1.0626935403832139d+00*HR4(-1,-1,-1,1)
     &  + 2.1253870807664278d+00*HR4(0,-1,-1,-1)
     &  + 1.0626935403832139d+00*HR4(0,-1,-1,1)
     &  + 1.0626935403832139d+00*HR4(0,-1,1,-1)
     &  + 1.0626935403832139d+00*HR4(0,0,-1,-1)
     &  + 1.0626935403832139d+00*HR4(0,0,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,0,-1,1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + HR6(0, -1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,0,-1,1)
     &  + HR6(0, -1,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  - HR6(0, -1,0,-1,-1,1)
     &  + HR6(0, -1,1,-1,-1,-1)
     &  + 8.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  - HR6(0,0, -1,-1,0,-1)
     &  + HR6(0,0, -1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,0,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,0,-1,1)
     &  - HR6(0,0, -1,0,1,-1)
     &  + HR6(0,0, -1,1,-1,-1)
     &  - HR6(0,0, -1,1,0,-1)
     &  - 6.0000000000000000d+00*HR6(0,0,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,0,-1,1,-1)
      HY6(0,-1,1,1,1,1) = 
     &  + 1.8254649462891551d-02
     &  - 5.0840057924226870d-01*HR1(-1)
     &  + 4.8090645538142385d-03*HR1(-1)*HR1(-1)
     &  - 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(0)
     &  + 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,0,-1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,0,-1)
     &  + 9.6181291076284771d-03*HR1(-1)*HR1(1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR2(-1,1)
     &  - 5.5504108664821579d-02*HR1(-1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,0,-1)
     &  + HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,0,-1,-1)
     &  + HR1( -1)*HR5(0,0,0,0,-1)
     &  - 4.1666666666666666d-02*HR1(0)*HR1(0)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(0)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  - 1.2011325347955035d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR1(0)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,0,-1,1)
     &  - 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,0,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,0,0,-1)
     &  - 5.5504108664821579d-02*HR1(0)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(0)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,0,-1,1)
     &  - HR1(0) *HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + HR1(0) *HR5(0,-1,-1,-1,1)
     &  + HR1(0) *HR5(0,-1,-1,1,-1)
     &  + HR1(0) *HR5(0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,-1)
     &  + HR1(0) *HR5(0,0,-1,-1,1)
     &  + HR1(0) *HR5(0,0,-1,1,-1)
     &  + HR1(0) *HR5(0,0,0,-1,-1)
     &  + HR1(0) *HR5(0,0,0,-1,1)
     &  - 5.0840057924226870d-01*HR1(1)
     &  - 5.5504108664821579d-02*HR1(1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,0,-1)
     &  + HR1(1) *HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,0,-1,-1)
     &  + HR1(1) *HR5(0,0,0,0,-1)
     &  - 9.6181291076284771d-03*HR2(-1,1)
     &  - 5.5504108664821579d-02*HR3(-1,-1,1)
     &  + 5.5504108664821579d-02*HR3(0,-1,-1)
     &  + 5.5504108664821579d-02*HR3(0,-1,1)
     &  - 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  - 4.8045301391820142d-01*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,1,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,0,-1,1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - HR6(0, -1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  - HR6(0,0, -1,-1,-1,1)
     &  - HR6(0,0, -1,-1,1,-1)
     &  - HR6(0,0, -1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,0,-1,-1,-1)
     &  - HR6(0,0,0, -1,-1,1)
     &  - HR6(0,0,0, -1,1,-1)
     &  - HR6(0,0,0,0, -1,-1)
     &  - HR6(0,0,0,0, -1,1)
      HY6(0,0,-1,-1,-1,-1) = 
     &  + 9.7351609701343225d-04
     &  - 3.1350096016808622d-03*HR1(-1)
     &  + 4.8090645538142385d-03*HR1(-1)*HR1(-1)
     &  - 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  + 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 9.6181291076284771d-03*HR1(-1)*HR1(1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,1,1)
     &  - 3.1350096016808622d-03*HR1(1)
     &  + 4.8090645538142385d-03*HR1(1)*HR1(1)
     &  + 5.5504108664821579d-02*HR1(1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1(1) *HR5(-1,-1,-1,-1,1)
     &  + 5.5504108664821579d-02*HR3(-1,-1,1)
     &  - 5.5504108664821579d-02*HR3(-1,1,1)
     &  + 4.8045301391820142d-01*HR4(-1,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - HR6( -1,-1,-1,-1,1,1)
      HY6(0,0,-1,-1,-1,1) = 
     &  + 1.2190143410042446d-03
     &  - 4.1914400448554060d-03*HR1(-1)
     &  + 7.0671186074950043d-03*HR1(-1)*HR1(-1)
     &  - 1.5792167371687950d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - 4.7376502115063852d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.4556013161625312d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,1,1)
     &  + 1.4134237214990008d-02*HR1(-1)*HR1(1)
     &  - 4.7376502115063852d-02*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,1,1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,1,1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + HR1(0) *HR5(-1,-1,-1,1,1)
     &  - 4.1914400448554060d-03*HR1(1)
     &  + 7.0671186074950043d-03*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 9.4753004230127705d-02*HR1(1)*HR2(-1,1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(1) *HR5(-1,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  + HR2(0, -1)*HR4(-1,-1,1,1)
     &  + 9.4753004230127705d-02*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - 9.4753004230127705d-02*HR3(-1,1,1)
     &  + HR3( -1,1,1)*HR3(0,-1,-1)
     &  + 1.1644810529300250d+00*HR4(-1,-1,-1,1)
     &  - 5.8224052646501250d-01*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + HR6( -1,-1,-1,-1,1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - HR6(0, -1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,-1,1,1)
      HY6(0,0,-1,-1,0,-1) = 
     &  + 1.0103097445914236d-02
     &  - 2.5378319266019282d-02*HR1(-1)
     &  + 3.0267446585960555d-02*HR1(-1)*HR1(-1)
     &  - 4.4929413255254634d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 3.4269459726004717d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.3707783890401886d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 1.3478823976576390d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 6.0534893171921111d-02*HR1(-1)*HR1(1)
     &  - 1.3478823976576390d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR3(-1,1,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,1,1)
     &  - HR1( -1)*HR5(-1,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,1,1)
     &  - HR1( -1)*HR5(-1,1,-1,1,1)
     &  - 2.5378319266019282d-02*HR1(1)
     &  + 3.0267446585960555d-02*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.6957647953152780d-01*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(-1,-1,1)
     &  + 8.2246703342411321d-01*HR1(1)*HR3(-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,1,1)
     &  + 2.6957647953152780d-01*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(-1,1,1)
     &  - 2.6957647953152780d-01*HR3(-1,1,1)
     &  + 1.6449340668482264d+00*HR4(-1,-1,-1,1)
     &  - 8.2246703342411321d-01*HR4(-1,-1,1,1)
     &  + 1.3862943611198906d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,1,-1,1,1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  - 5.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  + 9.0000000000000000d+00*HR6(-1,-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HR6(-1,-1,1,-1,1,1)
     &  + HR6( -1,-1,1,1,-1,1)
      HY6(0,0,-1,-1,0,1) = 
     &  + 1.1458508205698223d-02
     &  - 3.0172237496701167d-02*HR1(-1)
     &  + 3.8670450283379109d-02*HR1(-1)*HR1(-1)
     &  - 6.4815974361351054d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 6.8538919452009434d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - 1.9444792308405316d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,1,1)
     &  + 7.7340900566758219d-02*HR1(-1)*HR1(1)
     &  - 1.9444792308405316d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(-1,1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR2(0,1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,1)*HR3(-1,1,1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,1,1)
     &  + HR1( -1)*HR5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,1,1)
     &  + HR1( -1)*HR5(-1,1,-1,1,1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1(0) *HR2(-1,1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - HR1(0) *HR5(-1,-1,-1,1,1)
     &  - HR1(0) *HR5(-1,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,1,1,1)
     &  - HR1(0) *HR5(-1,1,-1,1,1)
     &  - 3.0172237496701167d-02*HR1(1)
     &  + 3.8670450283379109d-02*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,-1,-1)
     &  + 3.8889584616810632d-01*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(-1,-1,1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,1,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1(1) *HR2(0,1)*HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,1,-1,-1,-1)
     &  + HR1(1) *HR5(0,1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  + HR2(0, -1)*HR4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR2(0,1)*HR4(-1,-1,-1,1)
     &  + HR2(0,1) *HR4(-1,-1,1,1)
     &  + 3.8889584616810632d-01*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(-1,1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - HR3( -1,-1,1)*HR3(0,1,-1)
     &  - 3.8889584616810632d-01*HR3(-1,1,1)
     &  + HR3( -1,1,1)*HR3(0,-1,-1)
     &  + HR3( -1,1,1)*HR3(0,1,-1)
     &  + 3.2898681336964528d+00*HR4(-1,-1,-1,1)
     &  - 1.6449340668482264d+00*HR4(-1,-1,1,1)
     &  - 1.3862943611198906d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,1,-1,1,1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  + 5.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  - 9.0000000000000000d+00*HR6(-1,-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HR6(-1,-1,1,-1,1,1)
     &  - HR6( -1,-1,1,1,-1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - HR6(0, -1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,-1,1,1)
     &  - HR6(0,1, -1,-1,-1,-1)
     &  - HR6(0,1, -1,-1,-1,1)
     &  - HR6(0,1, -1,-1,1,-1)
     &  - HR6(0,1, -1,-1,1,1)
      HY6(0,0,-1,-1,1,-1) = 
     &  + 1.5800771184704528d-03
     &  - 5.9459097989450212d-03*HR1(-1)
     &  + 1.1637533043363782d-02*HR1(-1)*HR1(-1)
     &  - 3.5678728477784371d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.0703618543335311d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.6567338509580348d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 2.3275066086727564d-02*HR1(-1)*HR1(1)
     &  - 1.0703618543335311d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,1,1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  - HR1( -1)*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,1,1)
     &  - 5.9459097989450212d-03*HR1(1)
     &  + 1.1637533043363782d-02*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.1407237086670622d-01*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  - HR1(1) *HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  - HR1(1) *HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  - HR2(0, -1)*HR4(-1,-1,1,1)
     &  + 2.1407237086670622d-01*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  - 2.1407237086670622d-01*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  - 2.1253870807664278d+00*HR4(-1,-1,-1,1)
     &  + 1.0626935403832139d+00*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + HR6( -1,-1,-1,-1,1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + HR6(0, -1,-1,1,-1,1)
     &  + HR6(0, -1,-1,1,1,-1)
      HY6(0,0,-1,-1,1,1) = 
     &  + 2.1023563780719722d-03
     &  - 8.7734377821481916d-03*HR1(-1)
     &  + 2.0379119579654625d-02*HR1(-1)*HR1(-1)
     &  - 8.9535532268006700d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 1.2500000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 2.6860659680402010d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,1)
     &  + 4.0758239159309251d-02*HR1(-1)*HR1(1)
     &  - 2.6860659680402010d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + HR1(0) *HR1(0)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(-1,-1,1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1(0) *HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + HR1(0) *HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,-1,1)
     &  + HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1(0) *HR2(0,-1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - HR1(0) *HR5(-1,-1,-1,1,1)
     &  + HR1(0) *HR5(0,-1,-1,-1,-1)
     &  + HR1(0) *HR5(0,-1,-1,-1,1)
     &  + HR1(0) *HR5(0,-1,-1,1,-1)
     &  + HR1(0) *HR5(0,-1,-1,1,1)
     &  - 8.7734377821481916d-03*HR1(1)
     &  + 2.0379119579654625d-02*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1(1) *HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 5.3721319360804020d-01*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  + HR1(1) *HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,1,1)
     &  + 5.3721319360804020d-01*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - HR3( -1,-1,1)*HR3(0,0,-1)
     &  - 5.3721319360804020d-01*HR3(-1,1,1)
     &  + HR3( -1,1,1)*HR3(0,-1,-1)
     &  + HR3( -1,1,1)*HR3(0,0,-1)
     &  + 4.8045301391820142d-01*HR4(-1,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - HR6( -1,-1,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - HR6(0, -1,-1,1,-1,1)
     &  - HR6(0, -1,-1,1,1,-1)
     &  - HR6(0,0, -1,-1,-1,-1)
     &  - HR6(0,0, -1,-1,-1,1)
     &  - HR6(0,0, -1,-1,1,-1)
     &  - HR6(0,0, -1,-1,1,1)
      HY6(0,0,-1,0,-1,-1) = 
     &  + 5.3096809374395533d-03
     &  - 1.3531263989594243d-02*HR1(-1)
     &  + 1.6446597597178020d-02*HR1(-1)*HR1(-1)
     &  - 2.5042852149158214d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 7.5128556447474642d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  + 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  + 3.2893195194356041d-02*HR1(-1)*HR1(1)
     &  - 7.5128556447474642d-02*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(-1,1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(-1,1,1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  + HR1( -1)*HR5(-1,-1,1,-1,1)
     &  + HR1( -1)*HR5(-1,1,-1,1,1)
     &  - 1.3531263989594243d-02*HR1(1)
     &  + 1.6446597597178020d-02*HR1(1)*HR1(1)
     &  + 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 1.5025711289494928d-01*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1(1) *HR2(-1,1)*HR3(-1,-1,1)
     &  + 4.8045301391820142d-01*HR1(1)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(-1,1,1)
     &  + 2.7725887222397812d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(-1,-1,1,1)
     &  + 7.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + HR1(1) *HR5(-1,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,1,1)
     &  + 1.5025711289494928d-01*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR3(-1,-1,1)*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(-1,1,1)
     &  - 1.5025711289494928d-01*HR3(-1,1,1)
     &  + 7.2067952087730213d-01*HR4(-1,-1,-1,1)
     &  - 7.2067952087730213d-01*HR4(-1,-1,1,1)
     &  + 7.2067952087730213d-01*HR4(-1,1,1,1)
     &  + 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  - 4.8520302639196171d+00*HR5(-1,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,1,-1,1,1)
     &  + 1.4000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - HR6( -1,-1,1,-1,1,1)
      HY6(0,0,-1,0,-1,1) = 
     &  + 6.2840595064476972d-03
     &  - 1.7042475614121991d-02*HR1(-1)
     &  + 2.2756111933263048d-02*HR1(-1)*HR1(-1)
     &  - 4.0511725278343596d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - 1.2153517583503078d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.4556013161625312d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,1,1,1)
     &  + 4.5512223866526096d-02*HR1(-1)*HR1(1)
     &  - 1.2153517583503078d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(-1,1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,1,1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(-1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(-1,1,1,1)
     &  + 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  - HR1( -1)*HR5(-1,-1,1,-1,1)
     &  - HR1( -1)*HR5(-1,1,-1,1,1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,1,-1,1)
     &  - HR1(0) *HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,1,1)
     &  - HR1(0) *HR2(-1,1)*HR3(-1,1,1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  + HR1(0) *HR5(-1,-1,1,-1,1)
     &  + HR1(0) *HR5(-1,1,-1,1,1)
     &  - 1.7042475614121991d-02*HR1(1)
     &  + 2.2756111933263048d-02*HR1(1)*HR1(1)
     &  + 2.9112026323250625d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,-1)
     &  + 2.4307035167006157d-01*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1(1) *HR2(-1,1)*HR3(-1,-1,1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(-1,1,1)
     &  + 1.1644810529300250d+00*HR1(1)*HR3(-1,-1,1)
     &  - 1.1644810529300250d+00*HR1(1)*HR3(-1,1,1)
     &  - 2.7725887222397812d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(-1,-1,1,1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - HR1(1) *HR5(-1,-1,1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,1,-1,-1)
     &  + HR1(1) *HR5(0,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,1,1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(-1,1,1,1)
     &  + 2.4307035167006157d-01*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR3(-1,-1,1)*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(-1,1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - HR3( -1,-1,1)*HR3(0,-1,1)
     &  - 2.4307035167006157d-01*HR3(-1,1,1)
     &  + HR3( -1,1,1)*HR3(0,-1,-1)
     &  + HR3( -1,1,1)*HR3(0,-1,1)
     &  + 1.7467215793950375d+00*HR4(-1,-1,-1,1)
     &  - 1.7467215793950375d+00*HR4(-1,-1,1,1)
     &  + 1.7467215793950375d+00*HR4(-1,1,1,1)
     &  - 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  + 4.8520302639196171d+00*HR5(-1,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,1,-1,1,1)
     &  - 1.4000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + HR6( -1,-1,1,-1,1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - HR6(0, -1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,-1,1,1)
     &  - HR6(0, -1,1,-1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,1)
     &  - HR6(0, -1,1,-1,1,-1)
     &  - HR6(0, -1,1,-1,1,1)
      HY6(0,0,-1,0,0,1) = 
     &  + 1.0228193143276494d-01
     &  - 1.8863881027582918d-01*HR1(-1)
     &  + 1.6977273454367993d-01*HR1(-1)*HR1(-1)
     &  - 2.0034281719326571d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 6.8538919452009434d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - 6.0102845157979714d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,1,1,1)
     &  + 3.3954546908735986d-01*HR1(-1)*HR1(1)
     &  - 6.0102845157979714d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,1,1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,1,1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,1,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR2(0,1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,1)*HR3(-1,1,1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(-1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(-1,1,1,1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,1,1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(-1,1,1,1,1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,1,-1,1)
     &  + HR1( -1)*HR5(0,1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,1,-1,-1,1)
     &  + HR1( -1)*HR5(0,1,1,-1,-1)
     &  + HR1( -1)*HR5(0,1,1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,1,1,1)
     &  - 6.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  - 6.0000000000000000d+00*HR1(0)*HR5(-1,-1,1,1,1)
     &  + 6.0000000000000000d+00*HR1(0)*HR5(-1,1,1,1,1)
     &  - 1.8863881027582918d-01*HR1(1)
     &  + 1.6977273454367993d-01*HR1(1)*HR1(1)
     &  + 8.2246703342411321d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,1,-1)
     &  + 1.2020569031595942d+00*HR1(1)*HR2(-1,1)
     &  - HR1(1) *HR2(-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(-1,1,1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,-1,1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,1,-1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,1)*HR3(-1,1,1)
     &  + 3.2898681336964528d+00*HR1(1)*HR3(-1,-1,1)
     &  - 3.2898681336964528d+00*HR1(1)*HR3(-1,1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(-1,1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 9.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,1,-1,1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(-1,-1,1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(-1,1,-1,1,1)
     &  + HR1(1) *HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,1,-1,-1)
     &  + HR1(1) *HR5(0,-1,1,-1,1)
     &  + HR1(1) *HR5(0,1,-1,-1,-1)
     &  + HR1(1) *HR5(0,1,-1,-1,1)
     &  + HR1(1) *HR5(0,1,1,-1,-1)
     &  + HR1(1) *HR5(0,1,1,-1,1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(-1,1,1,1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(-1,1,1,1)
     &  - 3.0000000000000000d+00*HR2(0,1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR2(0,1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR2(0,1)*HR4(-1,1,1,1)
     &  + 1.2020569031595942d+00*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR3(-1,-1,1)*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(-1,1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - HR3( -1,-1,1)*HR3(0,-1,1)
     &  - HR3( -1,-1,1)*HR3(0,1,-1)
     &  - HR3( -1,-1,1)*HR3(0,1,1)
     &  - 1.2020569031595942d+00*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR3(-1,1,1)*HR3(-1,1,1)
     &  + HR3( -1,1,1)*HR3(0,-1,-1)
     &  + HR3( -1,1,1)*HR3(0,-1,1)
     &  + HR3( -1,1,1)*HR3(0,1,-1)
     &  + HR3( -1,1,1)*HR3(0,1,1)
     &  + 4.9348022005446793d+00*HR4(-1,-1,-1,1)
     &  - 4.9348022005446793d+00*HR4(-1,-1,1,1)
     &  + 4.9348022005446793d+00*HR4(-1,1,1,1)
     &  - 4.1588830833596718d+00*HR5(-1,-1,-1,-1,1)
     &  + 4.1588830833596718d+00*HR5(-1,-1,-1,1,1)
     &  - 4.1588830833596718d+00*HR5(-1,-1,1,1,1)
     &  + 4.1588830833596718d+00*HR5(-1,1,1,1,1)
     &  - 9.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + 1.5000000000000000d+01*HR6(-1,-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  - 1.8000000000000000d+01*HR6(-1,-1,-1,1,1,1)
     &  - 6.0000000000000000d+00*HR6(-1,-1,1,-1,1,1)
     &  - 3.0000000000000000d+00*HR6(-1,-1,1,1,-1,1)
     &  - 6.0000000000000000d+00*HR6(-1,-1,1,1,1,1)
     &  - 3.0000000000000000d+00*HR6(-1,1,-1,1,1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - HR6(0, -1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,-1,1,1)
     &  - HR6(0, -1,1,-1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,1)
     &  - HR6(0, -1,1,-1,1,-1)
     &  - HR6(0, -1,1,-1,1,1)
     &  - HR6(0,1, -1,-1,-1,-1)
     &  - HR6(0,1, -1,-1,-1,1)
     &  - HR6(0,1, -1,-1,1,-1)
     &  - HR6(0,1, -1,-1,1,1)
     &  - HR6(0,1,1, -1,-1,-1)
     &  - HR6(0,1,1, -1,-1,1)
     &  - HR6(0,1,1, -1,1,-1)
     &  - HR6(0,1,1, -1,1,1)
      HY6(0,0,-1,0,1,-1) = 
     &  + 7.6290419248728392d-03
     &  - 2.2495758621687517d-02*HR1(-1)
     &  + 3.4684017151427288d-02*HR1(-1)*HR1(-1)
     &  - 8.4702535467447475d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 2.5410760640234242d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.6567338509580348d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 6.9368034302854577d-02*HR1(-1)*HR1(1)
     &  - 2.5410760640234242d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(-1,1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,1,1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(-1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(-1,1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,1)
     &  + 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  - HR1( -1)*HR5(-1,-1,1,-1,1)
     &  - HR1( -1)*HR5(-1,1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  - HR1( -1)*HR5(0,-1,-1,1,-1)
     &  - HR1( -1)*HR5(0,-1,1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,1)
     &  - HR1( -1)*HR5(0,1,-1,1,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(-1,1,1,1)
     &  - 2.2495758621687517d-02*HR1(1)
     &  + 3.4684017151427288d-02*HR1(1)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,-1)
     &  + HR1(1) *HR1(1)*HR4(0,1,-1,-1)
     &  + 5.0821521280468485d-01*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  + HR1(1) *HR2(-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,-1,1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(-1,1,1)
     &  - 2.1253870807664278d+00*HR1(1)*HR3(-1,-1,1)
     &  + 2.1253870807664278d+00*HR1(1)*HR3(-1,1,1)
     &  - 2.7725887222397812d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - HR1(1) *HR5(-1,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  - HR1(1) *HR5(0,-1,-1,1,-1)
     &  - HR1(1) *HR5(0,-1,1,-1,-1)
     &  - HR1(1) *HR5(0,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,1)
     &  - HR1(1) *HR5(0,1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,1,1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(-1,1,1,1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(-1,1,1)
     &  + 5.0821521280468485d-01*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR3(-1,-1,1)*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  + HR3( -1,-1,1)*HR3(0,-1,1)
     &  + HR3( -1,-1,1)*HR3(0,1,-1)
     &  - 5.0821521280468485d-01*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  - HR3( -1,1,1)*HR3(0,-1,1)
     &  - HR3( -1,1,1)*HR3(0,1,-1)
     &  - 3.1880806211496417d+00*HR4(-1,-1,-1,1)
     &  + 3.1880806211496417d+00*HR4(-1,-1,1,1)
     &  - 3.1880806211496417d+00*HR4(-1,1,1,1)
     &  - 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  + 4.8520302639196171d+00*HR5(-1,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,1,1)
     &  - 1.4000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + HR6( -1,-1,1,-1,1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + HR6(0, -1,-1,1,-1,1)
     &  + HR6(0, -1,-1,1,1,-1)
     &  + HR6(0, -1,1,-1,-1,-1)
     &  + HR6(0, -1,1,-1,-1,1)
     &  + HR6(0, -1,1,-1,1,-1)
     &  + HR6(0, -1,1,-1,1,1)
     &  + 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  + HR6(0,1, -1,1,-1,1)
     &  + HR6(0,1, -1,1,1,-1)
      HY6(0,0,-1,0,1,1) = 
     &  + 9.4732998638164237d-03
     &  - 3.0833054551948363d-02*HR1(-1)
     &  + 5.6428748221951486d-02*HR1(-1)*HR1(-1)
     &  - 2.0034281719326571d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 1.2500000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  - 6.0102845157979714d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  - HR1( -1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,1,1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,1,-1,1)
     &  + 1.1285749644390297d-01*HR1(-1)*HR1(1)
     &  - 6.0102845157979714d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(-1,1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(-1,1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  + HR1( -1)*HR5(-1,-1,1,-1,1)
     &  + HR1( -1)*HR5(-1,1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,1)
     &  + HR1( -1)*HR5(0,0,1,-1,-1)
     &  + HR1( -1)*HR5(0,0,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,1)
     &  + HR1( -1)*HR5(0,1,-1,1,-1)
     &  + 2.5000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + HR1(0) *HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(0)*HR1(1)*HR3(-1,1,1)
     &  + 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(-1,-1,1,1)
     &  + 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(-1,1,1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + HR1(0) *HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1(0) *HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1(0) *HR1(1)*HR2(-1,1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,1,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,1,-1,1)
     &  + HR1(0) *HR2(-1,1)*HR3(-1,1,1)
     &  + HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1(0) *HR2(0,-1)*HR3(-1,1,1)
     &  + HR1(0) *HR2(0,1)*HR3(-1,-1,1)
     &  - HR1(0) *HR2(0,1)*HR3(-1,1,1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(-1,1,1,1)
     &  + 7.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  - HR1(0) *HR5(-1,-1,1,-1,1)
     &  - HR1(0) *HR5(-1,1,-1,1,1)
     &  + HR1(0) *HR5(0,-1,-1,-1,-1)
     &  + HR1(0) *HR5(0,-1,-1,-1,1)
     &  + HR1(0) *HR5(0,-1,-1,1,-1)
     &  + HR1(0) *HR5(0,-1,-1,1,1)
     &  + HR1(0) *HR5(0,1,-1,-1,-1)
     &  + HR1(0) *HR5(0,1,-1,-1,1)
     &  + HR1(0) *HR5(0,1,-1,1,-1)
     &  + HR1(0) *HR5(0,1,-1,1,1)
     &  - 3.0833054551948363d-02*HR1(1)
     &  + 5.6428748221951486d-02*HR1(1)*HR1(1)
     &  + 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(1) *HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,1,-1)
     &  - HR1(1) *HR1(1)*HR4(0,1,-1,-1)
     &  + 1.2020569031595942d+00*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  - HR1(1) *HR2(-1,1)*HR3(-1,-1,1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,0,-1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,0,1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,1,-1)
     &  + 4.8045301391820142d-01*HR1(1)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(-1,1,1)
     &  + 2.7725887222397812d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,1)
     &  + 7.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + HR1(1) *HR5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,1)
     &  + HR1(1) *HR5(0,0,1,-1,-1)
     &  + HR1(1) *HR5(0,0,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,1)
     &  + HR1(1) *HR5(0,1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(-1,1,1)
     &  + 1.2020569031595942d+00*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR3(-1,-1,1)*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(-1,1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - HR3( -1,-1,1)*HR3(0,0,-1)
     &  - HR3( -1,-1,1)*HR3(0,0,1)
     &  - HR3( -1,-1,1)*HR3(0,1,-1)
     &  - 1.2020569031595942d+00*HR3(-1,1,1)
     &  + HR3( -1,1,1)*HR3(0,-1,-1)
     &  + HR3( -1,1,1)*HR3(0,0,-1)
     &  + HR3( -1,1,1)*HR3(0,0,1)
     &  + HR3( -1,1,1)*HR3(0,1,-1)
     &  + 7.2067952087730213d-01*HR4(-1,-1,-1,1)
     &  - 7.2067952087730213d-01*HR4(-1,-1,1,1)
     &  + 7.2067952087730213d-01*HR4(-1,1,1,1)
     &  + 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  - 4.8520302639196171d+00*HR5(-1,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,1)
     &  + 1.4000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - HR6( -1,-1,1,-1,1,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - HR6(0, -1,-1,1,-1,1)
     &  - HR6(0, -1,-1,1,1,-1)
     &  - HR6(0,0, -1,-1,-1,-1)
     &  - HR6(0,0, -1,-1,-1,1)
     &  - HR6(0,0, -1,-1,1,-1)
     &  - HR6(0,0, -1,-1,1,1)
     &  - HR6(0,0,1, -1,-1,-1)
     &  - HR6(0,0,1, -1,-1,1)
     &  - HR6(0,0,1, -1,1,-1)
     &  - HR6(0,0,1, -1,1,1)
     &  - 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  - HR6(0,1, -1,1,-1,1)
     &  - HR6(0,1, -1,1,1,-1)
      HY6(0,0,-1,1,-1,-1) = 
     &  + 2.1636981353547235d-03
     &  - 9.4133341974174110d-03*HR1(-1)
     &  + 2.5458382032146317d-02*HR1(-1)*HR1(-1)
     &  + 7.9222950181883112d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.3766885054564933d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0916764064292634d-02*HR1(-1)*HR1(1)
     &  + 2.3766885054564933d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,1,1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,-1,1,-1,-1)
     &  - 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(0)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(-1,1,1)
     &  - 9.4133341974174110d-03*HR1(1)
     &  + 2.5458382032146317d-02*HR1(1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 4.7533770109129867d-01*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  - HR1(1) *HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,-1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,1,1)
     &  - 4.7533770109129867d-01*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + 4.7533770109129867d-01*HR3(-1,1,1)
     &  + HR3( -1,1,1)*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR4(-1,-1,-1,1)
     &  + 2.4022650695910071d-01*HR4(-1,-1,1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,1,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,1,-1)
     &  - 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + HR6( -1,-1,-1,-1,1,1)
     &  - 1.0000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  - 5.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,1)
     &  - HR6(0, -1,1,-1,1,-1)
     &  - HR6(0, -1,1,1,-1,-1)
      HY6(0,0,-1,1,-1,1) = 
     &  + 2.8776921291024274d-03
     &  - 1.3833955759762555d-02*HR1(-1)
     &  + 4.3992768505254480d-02*HR1(-1)*HR1(-1)
     &  + 2.4633412775717367d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 7.3900238327152102d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.4556013161625312d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(0,-1,1,-1)
     &  + 8.7985537010508960d-02*HR1(-1)*HR1(1)
     &  + 7.3900238327152102d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR3(-1,1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,1)
     &  - HR1( -1)*HR5(0,-1,1,-1,-1)
     &  - HR1(0) *HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  - HR1(0) *HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1(0) *HR1(1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,1)
     &  + HR1(0) *HR1(1)*HR4(0,-1,1,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1(0) *HR2(0,-1)*HR3(-1,1,1)
     &  - 5.8224052646501250d-01*HR1(0)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(0)*HR3(-1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - HR1(0) *HR5(-1,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,-1)
     &  - HR1(0) *HR5(0,-1,1,-1,1)
     &  - HR1(0) *HR5(0,-1,1,1,-1)
     &  - 1.3833955759762555d-02*HR1(1)
     &  + 4.3992768505254480d-02*HR1(1)*HR1(1)
     &  - 2.9112026323250625d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR1(1)*HR4(0,0,-1,-1)
     &  - 1.4780047665430420d+00*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,0,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,1)
     &  - 5.8224052646501250d-01*HR1(1)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  + HR1(1) *HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,1)
     &  - HR1(1) *HR5(0,-1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  - HR2(0, -1)*HR4(-1,-1,1,1)
     &  + HR2(0, -1)*HR4(0,-1,-1,-1)
     &  + HR2(0, -1)*HR4(0,-1,-1,1)
     &  + HR2(0, -1)*HR4(0,-1,1,-1)
     &  + HR2(0, -1)*HR4(0,-1,1,1)
     &  - 1.4780047665430420d+00*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,-1)
     &  + 1.4780047665430420d+00*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,0,-1)
     &  - 1.1644810529300250d+00*HR4(-1,-1,-1,1)
     &  + 5.8224052646501250d-01*HR4(-1,-1,1,1)
     &  - 5.8224052646501250d-01*HR4(0,-1,-1,-1)
     &  - 5.8224052646501250d-01*HR4(0,-1,-1,1)
     &  - 5.8224052646501250d-01*HR4(0,-1,1,-1)
     &  - 5.8224052646501250d-01*HR4(0,-1,1,1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,1,-1)
     &  + 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - HR6( -1,-1,-1,-1,1,1)
     &  + 1.1000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  - HR6(0, -1,0,-1,-1,-1)
     &  - HR6(0, -1,0,-1,-1,1)
     &  - HR6(0, -1,0,-1,1,-1)
     &  - HR6(0, -1,0,-1,1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + HR6(0, -1,1,-1,-1,1)
     &  + HR6(0, -1,1,-1,1,-1)
     &  + HR6(0, -1,1,1,-1,-1)
      HY6(0,0,-1,1,0,-1) = 
     &  + 1.8113105618022087d-02
     &  - 5.9576541411150522d-02*HR1(-1)
     &  + 1.2051103765015028d-01*HR1(-1)*HR1(-1)
     &  + 2.2022937829936213d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 3.4269459726004717d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.3707783890401886d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.3707783890401886d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 6.6068813489808640d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 2.4102207530030056d-01*HR1(-1)*HR1(1)
     &  + 6.6068813489808640d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(-1,1,1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR3(-1,1,1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR3(0,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,1,1)
     &  + HR1( -1)*HR5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,1,1)
     &  + HR1( -1)*HR5(-1,1,-1,1,1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,1)
     &  + HR1( -1)*HR5(0,1,-1,1,-1)
     &  - 8.2246703342411321d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 8.2246703342411321d-01*HR1(0)*HR3(-1,-1,1)
     &  + 8.2246703342411321d-01*HR1(0)*HR3(-1,1,1)
     &  - 5.9576541411150522d-02*HR1(1)
     &  + 1.2051103765015028d-01*HR1(1)*HR1(1)
     &  - 4.1123351671205660d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,-1)
     &  - HR1(1) *HR1(1)*HR4(0,1,-1,-1)
     &  - 1.3213762697961728d+00*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(-1,-1,1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,1,-1)
     &  - 8.2246703342411321d-01*HR1(1)*HR3(-1,-1,1)
     &  + 8.2246703342411321d-01*HR1(1)*HR3(0,-1,-1)
     &  + 8.2246703342411321d-01*HR1(1)*HR3(0,-1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,1,-1,1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  + HR1(1) *HR5(0,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,1)
     &  + HR1(1) *HR5(0,1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(-1,1,1)
     &  - 1.3213762697961728d+00*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(-1,1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - HR3( -1,-1,1)*HR3(0,1,-1)
     &  + 1.3213762697961728d+00*HR3(-1,1,1)
     &  + HR3( -1,1,1)*HR3(0,-1,-1)
     &  + HR3( -1,1,1)*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR4(-1,-1,-1,1)
     &  + 8.2246703342411321d-01*HR4(-1,-1,1,1)
     &  - 8.2246703342411321d-01*HR4(0,-1,-1,-1)
     &  - 8.2246703342411321d-01*HR4(0,-1,-1,1)
     &  - 8.2246703342411321d-01*HR4(0,-1,1,-1)
     &  - 8.2246703342411321d-01*HR4(0,-1,1,1)
     &  - 1.3862943611198906d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,1,-1,1,1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,1,-1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,1,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,1,1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  + 5.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  - 9.0000000000000000d+00*HR6(-1,-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HR6(-1,-1,1,-1,1,1)
     &  - HR6( -1,-1,1,1,-1,1)
     &  - 1.0000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  - 7.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  - 6.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,1,-1,1,-1)
     &  - HR6(0, -1,1,-1,1,1)
     &  - 5.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,1,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  - HR6(0,1, -1,1,-1,1)
     &  - HR6(0,1, -1,1,1,-1)
      HY6(0,0,-1,1,0,1) = 
     &  + 2.1397522179178781d-02
     &  - 7.6026642213084631d-02*HR1(-1)
     &  + 1.7614133919876854d-01*HR1(-1)*HR1(-1)
     &  + 5.9071586949367355d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 6.8538919452009434d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 1.7721476084810206d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,1,1)
     &  + HR1( -1)*HR1(0)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR1(0)*HR4(0,1,-1,1)
     &  + 3.5228267839753708d-01*HR1(-1)*HR1(1)
     &  + 1.7721476084810206d+00*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(0,0,1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,0,1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(-1,1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,1)
     &  - HR1( -1)*HR2(0,1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(0,1)*HR3(-1,1,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR3(-1,1,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(0,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,1,1)
     &  - HR1( -1)*HR5(-1,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,1,1)
     &  - HR1( -1)*HR5(-1,1,-1,1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  - HR1( -1)*HR5(0,-1,-1,0,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,-1)
     &  - HR1( -1)*HR5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,1,-1,1)
     &  - HR1( -1)*HR5(0,-1,1,0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,1)
     &  - HR1( -1)*HR5(0,1,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1(0) *HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1(0) *HR1(1)*HR2(-1,1)*HR2(0,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,1,1)
     &  + HR1(0) *HR1(1)*HR4(0,1,-1,-1)
     &  + HR1(0) *HR1(1)*HR4(0,1,-1,1)
     &  - HR1(0) *HR2(-1,1)*HR3(-1,1,1)
     &  - HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1(0) *HR2(0,-1)*HR3(-1,1,1)
     &  - HR1(0) *HR2(0,1)*HR3(-1,-1,1)
     &  + HR1(0) *HR2(0,1)*HR3(-1,1,1)
     &  - 1.6449340668482264d+00*HR1(0)*HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR1(0)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + HR1(0) *HR5(-1,-1,-1,1,1)
     &  + HR1(0) *HR5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,1,1,1)
     &  + HR1(0) *HR5(-1,1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,1,1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,1,1,1)
     &  - HR1(0) *HR5(0,1,-1,-1,-1)
     &  - HR1(0) *HR5(0,1,-1,-1,1)
     &  - HR1(0) *HR5(0,1,-1,1,-1)
     &  - HR1(0) *HR5(0,1,-1,1,1)
     &  - 7.6026642213084631d-02*HR1(1)
     &  + 1.7614133919876854d-01*HR1(1)*HR1(1)
     &  - 8.2246703342411321d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,-1)
     &  + HR1(1) *HR1(1)*HR4(0,0,-1,-1)
     &  + HR1(1) *HR1(1)*HR4(0,0,-1,1)
     &  + HR1(1) *HR1(1)*HR4(0,0,1,-1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 3.5442952169620413d+00*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,0,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,1,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,1)
     &  + HR1(1) *HR2(0,1)*HR3(-1,-1,1)
     &  - 1.6449340668482264d+00*HR1(1)*HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(0,-1,-1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(0,-1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,1,-1,1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  - HR1(1) *HR5(0,-1,-1,0,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,-1)
     &  - HR1(1) *HR5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,0,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  - HR1(1) *HR5(0,-1,1,-1,1)
     &  - HR1(1) *HR5(0,-1,1,0,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,1)
     &  - HR1(1) *HR5(0,1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  - HR2(0, -1)*HR4(-1,-1,1,1)
     &  + HR2(0, -1)*HR4(0,-1,-1,-1)
     &  + HR2(0, -1)*HR4(0,-1,-1,1)
     &  + HR2(0, -1)*HR4(0,-1,1,-1)
     &  + HR2(0, -1)*HR4(0,-1,1,1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR2(0,1)*HR4(-1,-1,-1,1)
     &  - HR2(0,1) *HR4(-1,-1,1,1)
     &  - 3.5442952169620413d+00*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,1,-1)
     &  + 3.5442952169620413d+00*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,0,1)
     &  - 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,1)*HR3(0,-1,1)
     &  - 3.2898681336964528d+00*HR4(-1,-1,-1,1)
     &  + 1.6449340668482264d+00*HR4(-1,-1,1,1)
     &  - 1.6449340668482264d+00*HR4(0,-1,-1,-1)
     &  - 1.6449340668482264d+00*HR4(0,-1,-1,1)
     &  - 1.6449340668482264d+00*HR4(0,-1,1,-1)
     &  - 1.6449340668482264d+00*HR4(0,-1,1,1)
     &  + 1.3862943611198906d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,1,-1,1,1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,1,-1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,-1,1,-1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,1,1,-1)
     &  - 2.0794415416798359d+00*HR5(0,-1,1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,1,1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  - 5.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  + 9.0000000000000000d+00*HR6(-1,-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HR6(-1,-1,1,-1,1,1)
     &  + HR6( -1,-1,1,1,-1,1)
     &  + 1.1000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + HR6(0, -1,-1,-1,0,1)
     &  + 8.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  + HR6(0, -1,-1,0,-1,1)
     &  + HR6(0, -1,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,1,1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  + HR6(0, -1,-1,1,0,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  - HR6(0, -1,0,-1,-1,-1)
     &  - HR6(0, -1,0,-1,1,1)
     &  + HR6(0, -1,0,1,-1,-1)
     &  + HR6(0, -1,0,1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,0,1,1,1)
     &  + 6.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  + HR6(0, -1,1,-1,0,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,1,-1,1,-1)
     &  + HR6(0, -1,1,-1,1,1)
     &  + HR6(0, -1,1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,0,1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,1,-1,1)
     &  + HR6(0, -1,1,1,0,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,1,1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,-1,1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,-1,1,1)
     &  + 5.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  + HR6(0,1, -1,1,-1,1)
     &  + HR6(0,1, -1,1,1,-1)
      HY6(0,0,-1,1,1,-1) = 
     &  + 4.0581203978064105d-03
     &  - 2.2801059128486651d-02*HR1(-1)
     &  + 1.0143289758994481d-01*HR1(-1)*HR1(-1)
     &  - 1.8455064982157776d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.5365194946473328d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.6567338509580348d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  + 2.0286579517988963d-01*HR1(-1)*HR1(1)
     &  - 5.5365194946473328d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,-1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR3(-1,1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(0,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,-1,1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,1)
     &  - HR1( -1)*HR5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,1)
     &  - HR1( -1)*HR5(0,0,-1,1,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 1.0626935403832139d+00*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  + 1.0626935403832139d+00*HR1(0)*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR1(0)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,1)
     &  - 2.2801059128486651d-02*HR1(1)
     &  + 1.0143289758994481d-01*HR1(1)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 1.1073038989294665d+00*HR1(1)*HR2(-1,1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,0,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,-1,1)
     &  + 1.0626935403832139d+00*HR1(1)*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,1)
     &  + HR1(1) *HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - HR1(1) *HR5(0,-1,-1,1,-1)
     &  - HR1(1) *HR5(0,-1,0,-1,-1)
     &  - HR1(1) *HR5(0,-1,0,-1,1)
     &  - HR1(1) *HR5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,1)
     &  - HR1(1) *HR5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  + HR2(0, -1)*HR4(-1,-1,1,1)
     &  - HR2(0, -1)*HR4(0,-1,-1,-1)
     &  - HR2(0, -1)*HR4(0,-1,-1,1)
     &  - HR2(0, -1)*HR4(0,-1,1,-1)
     &  - HR2(0, -1)*HR4(0,-1,1,1)
     &  + 1.1073038989294665d+00*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - HR3( -1,-1,1)*HR3(0,0,-1)
     &  - 1.1073038989294665d+00*HR3(-1,1,1)
     &  + HR3( -1,1,1)*HR3(0,-1,-1)
     &  + HR3( -1,1,1)*HR3(0,0,-1)
     &  + 2.1253870807664278d+00*HR4(-1,-1,-1,1)
     &  - 1.0626935403832139d+00*HR4(-1,-1,1,1)
     &  + 1.0626935403832139d+00*HR4(0,-1,-1,-1)
     &  + 1.0626935403832139d+00*HR4(0,-1,-1,1)
     &  + 1.0626935403832139d+00*HR4(0,-1,1,-1)
     &  + 1.0626935403832139d+00*HR4(0,-1,1,1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,1,1)
     &  + 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - HR6( -1,-1,-1,-1,1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + HR6(0, -1,-1,1,-1,1)
     &  + HR6(0, -1,-1,1,1,-1)
     &  + HR6(0, -1,0,-1,-1,-1)
     &  + HR6(0, -1,0,-1,-1,1)
     &  + HR6(0, -1,0,-1,1,-1)
     &  + HR6(0, -1,0,-1,1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + HR6(0, -1,1,-1,-1,1)
     &  + HR6(0, -1,1,-1,1,-1)
     &  + HR6(0, -1,1,1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  + HR6(0,0, -1,1,-1,1)
     &  + HR6(0,0, -1,1,1,-1)
      HY6(0,0,-1,1,1,1) = 
     &  + 5.9544669277129711d-03
     &  - 3.9984858137537496d-02*HR1(-1)
     &  + 2.5873953083694969d-01*HR1(-1)*HR1(-1)
     &  + 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.2500000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  - 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,-1,1)
     &  + 5.1747906167389938d-01*HR1(-1)*HR1(1)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,1)
     &  + HR1( -1)*HR5(0,0,-1,1,-1)
     &  + HR1( -1)*HR5(0,0,0,-1,-1)
     &  + HR1( -1)*HR5(0,0,0,-1,1)
     &  - 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  - 2.5000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  - HR1(0) *HR1(0)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  - HR1(0) *HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,0,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,0,-1,1)
     &  - 2.4022650695910071d-01*HR1(0)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + HR1(0) *HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  + HR1(0) *HR5(0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,-1)
     &  + HR1(0) *HR5(0,-1,1,-1,1)
     &  + HR1(0) *HR5(0,-1,1,1,-1)
     &  + HR1(0) *HR5(0,0,-1,-1,-1)
     &  + HR1(0) *HR5(0,0,-1,-1,1)
     &  + HR1(0) *HR5(0,0,-1,1,-1)
     &  + HR1(0) *HR5(0,0,-1,1,1)
     &  - 3.9984858137537496d-02*HR1(1)
     &  + 2.5873953083694969d-01*HR1(1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,0,-1)
     &  - 5.5504108664821579d-02*HR1(1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,1)
     &  - HR1(1) *HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,1)
     &  + HR1(1) *HR5(0,0,-1,1,-1)
     &  + HR1(1) *HR5(0,0,0,-1,-1)
     &  + HR1(1) *HR5(0,0,0,-1,1)
     &  - 5.5504108664821579d-02*HR3(-1,-1,1)
     &  + 5.5504108664821579d-02*HR3(-1,1,1)
     &  - 4.8045301391820142d-01*HR4(-1,-1,-1,1)
     &  + 2.4022650695910071d-01*HR4(-1,-1,1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,1,1)
     &  - 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + HR6( -1,-1,-1,-1,1,1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - HR6(0, -1,-1,1,-1,1)
     &  - HR6(0, -1,-1,1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,1)
     &  - HR6(0, -1,1,-1,1,-1)
     &  - HR6(0, -1,1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  - HR6(0,0, -1,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  - HR6(0,0, -1,1,-1,1)
     &  - HR6(0,0, -1,1,1,-1)
     &  - HR6(0,0,0, -1,-1,-1)
     &  - HR6(0,0,0, -1,-1,1)
     &  - HR6(0,0,0, -1,1,-1)
     &  - HR6(0,0,0, -1,1,1)
      HY6(0,0,0,-1,-1,-1) = 
     &  + 3.7167695485927819d-03
     &  - 9.6015684431298325d-03*HR1(-1)
     &  + 1.1876183161309242d-02*HR1(-1)*HR1(-1)
     &  - 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  + 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  + 2.3752366322618485d-02*HR1(-1)*HR1(1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  + HR1( -1)*HR5(-1,-1,1,1,1)
     &  - 9.6015684431298325d-03*HR1(1)
     &  + 1.1876183161309242d-02*HR1(1)*HR1(1)
     &  - 9.2506847774702633d-03*HR1(1)*HR1(1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.4022650695910071d-01*HR1(1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + HR1(1) *HR5(-1,-1,-1,1,1)
     &  - 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  + 2.4022650695910071d-01*HR4(-1,-1,1,1)
     &  - 2.4022650695910071d-01*HR4(-1,1,1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(-1,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,1,1)
     &  - 6.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - HR6( -1,-1,-1,1,1,1)
      HY6(0,0,0,-1,-1,1) = 
     &  + 4.4943988992188555d-03
     &  - 1.2444228784499648d-02*HR1(-1)
     &  + 1.7079563083256956d-02*HR1(-1)*HR1(-1)
     &  - 1.5792167371687950d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 4.7376502115063852d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.4556013161625312d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR1(0)*HR4(-1,1,1,1)
     &  + 3.4159126166513913d-02*HR1(-1)*HR1(1)
     &  - 4.7376502115063852d-02*HR1(-1)*HR1(1)*HR1(1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  - HR1( -1)*HR5(-1,-1,1,1,1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  + HR1(0) *HR5(-1,-1,1,1,1)
     &  - 1.2444228784499648d-02*HR1(1)
     &  + 1.7079563083256956d-02*HR1(1)*HR1(1)
     &  - 1.5792167371687950d-02*HR1(1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 2.9112026323250625d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,1)
     &  + HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1(1) *HR2(0,-1)*HR3(-1,1,1)
     &  - 5.8224052646501250d-01*HR1(1)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - HR1(1) *HR5(-1,-1,-1,1,1)
     &  + HR1(1) *HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,-1,-1,1,1)
     &  + HR2(0, -1)*HR4(-1,-1,-1,1)
     &  - HR2(0, -1)*HR4(-1,-1,1,1)
     &  + HR2(0, -1)*HR4(-1,1,1,1)
     &  - 5.8224052646501250d-01*HR4(-1,-1,-1,1)
     &  + 5.8224052646501250d-01*HR4(-1,-1,1,1)
     &  - 5.8224052646501250d-01*HR4(-1,1,1,1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(-1,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,1,1)
     &  + 6.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + HR6( -1,-1,-1,1,1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - HR6(0, -1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,-1,1,1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  - HR6(0, -1,-1,1,-1,1)
     &  - HR6(0, -1,-1,1,1,-1)
     &  - HR6(0, -1,-1,1,1,1)
      HY6(0,0,0,-1,0,-1) = 
     &  + 5.0843191948181848d-02
     &  - 9.2748467341632644d-02*HR1(-1)
     &  + 8.1327333698710040d-02*HR1(-1)*HR1(-1)
     &  - 4.4929413255254634d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 3.4269459726004717d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.3707783890401886d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.3478823976576390d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  + 1.6265466739742008d-01*HR1(-1)*HR1(1)
     &  - 1.3478823976576390d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.3707783890401886d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,1,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(-1,1,1,1,1)
     &  - 9.2748467341632644d-02*HR1(1)
     &  + 8.1327333698710040d-02*HR1(1)*HR1(1)
     &  - 4.4929413255254634d-02*HR1(1)*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.3333333333333333d-01*HR1(1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 4.1123351671205660d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1(1) *HR1(1)*HR4(-1,-1,1,1)
     &  - HR1(1) *HR2(-1,1)*HR3(-1,1,1)
     &  - 8.2246703342411321d-01*HR1(1)*HR3(-1,-1,1)
     &  + 8.2246703342411321d-01*HR1(1)*HR3(-1,1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(-1,1,1,1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,1,1)
     &  + HR1(1) *HR5(-1,-1,1,-1,1)
     &  + HR1(1) *HR5(-1,1,-1,1,1)
     &  + HR2( -1,1)*HR4(-1,1,1,1)
     &  - 8.2246703342411321d-01*HR4(-1,-1,-1,1)
     &  + 8.2246703342411321d-01*HR4(-1,-1,1,1)
     &  - 8.2246703342411321d-01*HR4(-1,1,1,1)
     &  - 2.7725887222397812d+00*HR5(-1,-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR5(-1,-1,-1,1,1)
     &  - 2.7725887222397812d+00*HR5(-1,-1,1,1,1)
     &  + 2.7725887222397812d+00*HR5(-1,1,1,1,1)
     &  - 1.1000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  + 1.1000000000000000d+01*HR6(-1,-1,-1,-1,1,1)
     &  + HR6( -1,-1,-1,1,-1,1)
     &  - 1.0000000000000000d+01*HR6(-1,-1,-1,1,1,1)
     &  - 2.0000000000000000d+00*HR6(-1,-1,1,-1,1,1)
     &  - HR6( -1,-1,1,1,-1,1)
     &  - HR6( -1,1,-1,1,1,1)
      HY6(0,0,0,-1,0,1) = 
     &  + 5.5765213278438523d-02
     &  - 1.0679981350605469d-01*HR1(-1)
     &  + 1.0146780316041920d-01*HR1(-1)*HR1(-1)
     &  - 6.4815974361351054d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 6.8538919452009434d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.9444792308405316d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR1(0)*HR4(-1,1,1,1)
     &  + 2.0293560632083841d-01*HR1(-1)*HR1(1)
     &  - 1.9444792308405316d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.7415567780803773d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,1,1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(-1,1,1,1,1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,1,1)
     &  + HR1( -1)*HR5(0,1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,1,-1,-1,1)
     &  + HR1( -1)*HR5(0,1,-1,1,-1)
     &  + HR1( -1)*HR5(0,1,-1,1,1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + HR1(0) *HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(1)*HR1(1)*HR3(-1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,1,1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(-1,1,1,1,1)
     &  - 1.0679981350605469d-01*HR1(1)
     &  + 1.0146780316041920d-01*HR1(1)*HR1(1)
     &  - 6.4815974361351054d-02*HR1(1)*HR1(1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 3.3333333333333333d-01*HR1(1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 8.2246703342411321d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(1) *HR1(1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,-1,1)
     &  + HR1(1) *HR2(-1,1)*HR3(-1,1,1)
     &  + HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1(1) *HR2(0,-1)*HR3(-1,1,1)
     &  + HR1(1) *HR2(0,1)*HR3(-1,-1,1)
     &  - HR1(1) *HR2(0,1)*HR3(-1,1,1)
     &  - 1.6449340668482264d+00*HR1(1)*HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(-1,1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(-1,1,1,1)
     &  + 7.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,1,1)
     &  - HR1(1) *HR5(-1,-1,1,-1,1)
     &  - HR1(1) *HR5(-1,1,-1,1,1)
     &  + HR1(1) *HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,-1,-1,1,1)
     &  + HR1(1) *HR5(0,1,-1,-1,-1)
     &  + HR1(1) *HR5(0,1,-1,-1,1)
     &  + HR1(1) *HR5(0,1,-1,1,-1)
     &  + HR1(1) *HR5(0,1,-1,1,1)
     &  - HR2( -1,1)*HR4(-1,1,1,1)
     &  + HR2(0, -1)*HR4(-1,-1,-1,1)
     &  - HR2(0, -1)*HR4(-1,-1,1,1)
     &  + HR2(0, -1)*HR4(-1,1,1,1)
     &  + HR2(0,1) *HR4(-1,-1,-1,1)
     &  - HR2(0,1) *HR4(-1,-1,1,1)
     &  + HR2(0,1) *HR4(-1,1,1,1)
     &  - 1.6449340668482264d+00*HR4(-1,-1,-1,1)
     &  + 1.6449340668482264d+00*HR4(-1,-1,1,1)
     &  - 1.6449340668482264d+00*HR4(-1,1,1,1)
     &  + 2.7725887222397812d+00*HR5(-1,-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR5(-1,-1,-1,1,1)
     &  + 2.7725887222397812d+00*HR5(-1,-1,1,1,1)
     &  - 2.7725887222397812d+00*HR5(-1,1,1,1,1)
     &  + 1.1000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  - 1.1000000000000000d+01*HR6(-1,-1,-1,-1,1,1)
     &  - HR6( -1,-1,-1,1,-1,1)
     &  + 1.0000000000000000d+01*HR6(-1,-1,-1,1,1,1)
     &  + 2.0000000000000000d+00*HR6(-1,-1,1,-1,1,1)
     &  + HR6( -1,-1,1,1,-1,1)
     &  + HR6( -1,1,-1,1,1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - HR6(0, -1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,-1,1,1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  - HR6(0, -1,-1,1,-1,1)
     &  - HR6(0, -1,-1,1,1,-1)
     &  - HR6(0, -1,-1,1,1,1)
     &  - HR6(0,1, -1,-1,-1,-1)
     &  - HR6(0,1, -1,-1,-1,1)
     &  - HR6(0,1, -1,-1,1,-1)
     &  - HR6(0,1, -1,-1,1,1)
     &  - HR6(0,1, -1,1,-1,-1)
     &  - HR6(0,1, -1,1,-1,1)
     &  - HR6(0,1, -1,1,1,-1)
     &  - HR6(0,1, -1,1,1,1)
      HY6(0,0,0,-1,1,-1) = 
     &  + 5.5920127803503056d-03
     &  - 1.6991592326175436d-02*HR1(-1)
     &  + 2.7326526369131826d-02*HR1(-1)*HR1(-1)
     &  - 3.5678728477784371d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.0703618543335311d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.6567338509580348d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  + 5.4653052738263652d-02*HR1(-1)*HR1(1)
     &  - 1.0703618543335311d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  - HR1( -1)*HR5(-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,1,-1,1)
     &  - HR1( -1)*HR5(0,-1,1,1,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(-1,1,1,1)
     &  - 1.6991592326175436d-02*HR1(1)
     &  + 2.7326526369131826d-02*HR1(1)*HR1(1)
     &  - 3.5678728477784371d-02*HR1(1)*HR1(1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.3333333333333333d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.3134677019160696d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR1(1)*HR4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1(1) *HR2(0,-1)*HR3(-1,1,1)
     &  + 1.0626935403832139d+00*HR1(1)*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - HR1(1) *HR5(-1,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  - HR1(1) *HR5(0,-1,1,-1,1)
     &  - HR1(1) *HR5(0,-1,1,1,-1)
     &  - HR2(0, -1)*HR4(-1,-1,-1,1)
     &  + HR2(0, -1)*HR4(-1,-1,1,1)
     &  - HR2(0, -1)*HR4(-1,1,1,1)
     &  + 1.0626935403832139d+00*HR4(-1,-1,-1,1)
     &  - 1.0626935403832139d+00*HR4(-1,-1,1,1)
     &  + 1.0626935403832139d+00*HR4(-1,1,1,1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(-1,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,1,1)
     &  + 6.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + HR6( -1,-1,-1,1,1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,-1,1,-1)
     &  + HR6(0, -1,1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  + HR6(0, -1,1,1,-1,1)
     &  + HR6(0, -1,1,1,1,-1)
      HY6(0,0,0,-1,1,1) = 
     &  + 7.1257602963746915d-03
     &  - 2.4107342184124538d-02*HR1(-1)
     &  + 4.6548562995884288d-02*HR1(-1)*HR1(-1)
     &  - 8.9535532268006700d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 1.2500000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  - 2.6860659680402010d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  - HR1( -1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR1(0)*HR4(-1,1,1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,1,1)
     &  + 9.3097125991768577d-02*HR1(-1)*HR1(1)
     &  - 2.6860659680402010d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  + HR1( -1)*HR5(-1,-1,1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,1,-1,1)
     &  + HR1( -1)*HR5(0,-1,1,1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,1)
     &  + HR1( -1)*HR5(0,0,-1,1,-1)
     &  + HR1( -1)*HR5(0,0,-1,1,1)
     &  - 2.5000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(-1,1,1,1)
     &  - 1.6666666666666666d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1(0) *HR1(1)*HR4(-1,-1,1,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(-1,1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  - HR1(0) *HR5(-1,-1,1,1,1)
     &  + HR1(0) *HR5(0,-1,-1,-1,-1)
     &  + HR1(0) *HR5(0,-1,-1,-1,1)
     &  + HR1(0) *HR5(0,-1,-1,1,-1)
     &  + HR1(0) *HR5(0,-1,-1,1,1)
     &  + HR1(0) *HR5(0,-1,1,-1,-1)
     &  + HR1(0) *HR5(0,-1,1,-1,1)
     &  + HR1(0) *HR5(0,-1,1,1,-1)
     &  + HR1(0) *HR5(0,-1,1,1,1)
     &  - 2.4107342184124538d-02*HR1(1)
     &  + 4.6548562995884288d-02*HR1(1)*HR1(1)
     &  - 8.9535532268006700d-02*HR1(1)*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(1) *HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,-1,1)
     &  - 2.4022650695910071d-01*HR1(1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + HR1(1) *HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  + HR1(1) *HR5(0,-1,1,-1,1)
     &  + HR1(1) *HR5(0,-1,1,1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,1)
     &  + HR1(1) *HR5(0,0,-1,1,-1)
     &  + HR1(1) *HR5(0,0,-1,1,1)
     &  - 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  + 2.4022650695910071d-01*HR4(-1,-1,1,1)
     &  - 2.4022650695910071d-01*HR4(-1,1,1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(-1,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,1,1)
     &  - 6.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - HR6( -1,-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  - HR6(0, -1,-1,1,1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,-1,1,-1)
     &  - HR6(0, -1,1,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  - HR6(0, -1,1,1,-1,1)
     &  - HR6(0, -1,1,1,1,-1)
     &  - HR6(0,0, -1,-1,-1,-1)
     &  - HR6(0,0, -1,-1,-1,1)
     &  - HR6(0,0, -1,-1,1,-1)
     &  - HR6(0,0, -1,-1,1,1)
     &  - HR6(0,0, -1,1,-1,-1)
     &  - HR6(0,0, -1,1,-1,1)
     &  - HR6(0,0, -1,1,1,-1)
     &  - HR6(0,0, -1,1,1,1)
      HY6(0,0,0,0,-1,-1) = 
     &  + 2.6399148793116946d-02
     &  - 4.8936397049969063d-02*HR1(-1)
     &  + 4.3892835784327651d-02*HR1(-1)*HR1(-1)
     &  - 2.5042852149158214d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 7.5128556447474642d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  + 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 8.7785671568655302d-02*HR1(-1)*HR1(1)
     &  - 7.5128556447474642d-02*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR1(1)*HR4(-1,1,1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,1,1)
     &  - HR1( -1)*HR5(-1,-1,1,1,1)
     &  + HR1( -1)*HR5(-1,1,1,1,1)
     &  - 4.8936397049969063d-02*HR1(1)
     &  + 4.3892835784327651d-02*HR1(1)*HR1(1)
     &  - 2.5042852149158214d-02*HR1(1)*HR1(1)*HR1(1)
     &  + 1.0009437789962529d-02*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  + HR1(1) *HR1(1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,1,1)
     &  + HR1(1) *HR5(-1,-1,1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,1,1,1,1)
     &  + 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(-1,-1,-1,1,1,1)
     &  - HR6( -1,-1,1,1,1,1)
      HY6(0,0,0,0,-1,1) = 
     &  + 2.9901635581149425d-02
     &  - 5.9142607400864533d-02*HR1(-1)
     &  + 5.8937999825254663d-02*HR1(-1)*HR1(-1)
     &  - 4.0511725278343596d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.2153517583503078d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.4556013161625312d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.1787599965050932d-01*HR1(-1)*HR1(1)
     &  - 1.2153517583503078d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR1(1)*HR4(-1,1,1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,1,1)
     &  + HR1( -1)*HR5(-1,-1,1,1,1)
     &  - HR1( -1)*HR5(-1,1,1,1,1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,1,1)
     &  + HR1( -1)*HR5(0,-1,1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,1,-1,1)
     &  + HR1( -1)*HR5(0,-1,1,1,-1)
     &  + HR1( -1)*HR5(0,-1,1,1,1)
     &  - 1.6666666666666666d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  - HR1(0) *HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1(0) *HR1(1)*HR4(-1,-1,1,1)
     &  - HR1(0) *HR1(1)*HR4(-1,1,1,1)
     &  - HR1(0) *HR5(-1,-1,-1,-1,1)
     &  + HR1(0) *HR5(-1,-1,-1,1,1)
     &  - HR1(0) *HR5(-1,-1,1,1,1)
     &  + HR1(0) *HR5(-1,1,1,1,1)
     &  - 5.9142607400864533d-02*HR1(1)
     &  + 5.8937999825254663d-02*HR1(1)*HR1(1)
     &  - 4.0511725278343596d-02*HR1(1)*HR1(1)*HR1(1)
     &  + 2.4260021936042187d-02*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(1)*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  - HR1(1) *HR1(1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,1,1)
     &  - HR1(1) *HR5(-1,-1,1,1,1)
     &  + HR1(1) *HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,-1,-1,1,1)
     &  + HR1(1) *HR5(0,-1,1,-1,-1)
     &  + HR1(1) *HR5(0,-1,1,-1,1)
     &  + HR1(1) *HR5(0,-1,1,1,-1)
     &  + HR1(1) *HR5(0,-1,1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,1,1,1,1)
     &  - 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(-1,-1,-1,1,1,1)
     &  + HR6( -1,-1,1,1,1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - HR6(0, -1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,-1,1,1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  - HR6(0, -1,-1,1,-1,1)
     &  - HR6(0, -1,-1,1,1,-1)
     &  - HR6(0, -1,-1,1,1,1)
     &  - HR6(0, -1,1,-1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,1)
     &  - HR6(0, -1,1,-1,1,-1)
     &  - HR6(0, -1,1,-1,1,1)
     &  - HR6(0, -1,1,1,-1,-1)
     &  - HR6(0, -1,1,1,-1,1)
     &  - HR6(0, -1,1,1,1,-1)
     &  - HR6(0, -1,1,1,1,1)
      HY6(0,0,0,0,0,-1) = 
     &  + 9.8555109129743510d-01
     &  - 9.7211977044690930d-01*HR1(-1)
     &  + 4.7351641474862295d-01*HR1(-1)*HR1(-1)
     &  - 1.5025711289494928d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 3.4269459726004717d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.3707783890401886d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 4.5077133868484785d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 9.4703282949724591d-01*HR1(-1)*HR1(1)
     &  - 4.5077133868484785d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.3707783890401886d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 9.7211977044690930d-01*HR1(1)
     &  + 4.7351641474862295d-01*HR1(1)*HR1(1)
     &  - 1.5025711289494928d-01*HR1(1)*HR1(1)*HR1(1)
     &  + 3.4269459726004717d-02*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 5.7762265046662109d-03*HR1(1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(1)*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(-1,1,1,1)
     &  - HR1(1) *HR5(-1,-1,-1,-1,1)
     &  + HR1(1) *HR5(-1,-1,-1,1,1)
     &  - HR1(1) *HR5(-1,-1,1,1,1)
     &  + HR1(1) *HR5(-1,1,1,1,1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  + HR6( -1,-1,-1,-1,1,1)
     &  - HR6( -1,-1,-1,1,1,1)
     &  + HR6( -1,-1,1,1,1,1)
     &  - HR6( -1,1,1,1,1,1)
      HY6(0,0,0,0,0,1) = 
     &  + 1.0173430619844491d+00
     &  - 1.0369277551433699d+00*HR1(-1)
     &  + 5.4116161685556909d-01*HR1(-1)*HR1(-1)
     &  - 2.0034281719326571d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 6.8538919452009434d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 6.0102845157979714d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.0823232337111381d+00*HR1(-1)*HR1(1)
     &  - 6.0102845157979714d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.7415567780803773d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,1,1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,1,1)
     &  + HR1( -1)*HR5(0,-1,1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,1,-1,1)
     &  + HR1( -1)*HR5(0,-1,1,1,-1)
     &  + HR1( -1)*HR5(0,-1,1,1,1)
     &  + HR1( -1)*HR5(0,1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,1,-1,-1,1)
     &  + HR1( -1)*HR5(0,1,-1,1,-1)
     &  + HR1( -1)*HR5(0,1,-1,1,1)
     &  + HR1( -1)*HR5(0,1,1,-1,-1)
     &  + HR1( -1)*HR5(0,1,1,-1,1)
     &  + HR1( -1)*HR5(0,1,1,1,-1)
     &  + HR1( -1)*HR5(0,1,1,1,1)
     &  + 8.3333333333333333d-03*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.0369277551433699d+00*HR1(1)
     &  + 5.4116161685556909d-01*HR1(1)*HR1(1)
     &  - 2.0034281719326571d-01*HR1(1)*HR1(1)*HR1(1)
     &  + 6.8538919452009434d-02*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 5.7762265046662109d-03*HR1(1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(1)*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 4.1666666666666666d-02*HR1(1)*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 4.1666666666666666d-02*HR1(1)*HR1(1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(-1,-1,1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(-1,1,1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,1,1)
     &  + HR1(1) *HR5(-1,-1,-1,-1,1)
     &  - HR1(1) *HR5(-1,-1,-1,1,1)
     &  + HR1(1) *HR5(-1,-1,1,1,1)
     &  - HR1(1) *HR5(-1,1,1,1,1)
     &  + HR1(1) *HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,-1,-1,1,1)
     &  + HR1(1) *HR5(0,-1,1,-1,-1)
     &  + HR1(1) *HR5(0,-1,1,-1,1)
     &  + HR1(1) *HR5(0,-1,1,1,-1)
     &  + HR1(1) *HR5(0,-1,1,1,1)
     &  + HR1(1) *HR5(0,1,-1,-1,-1)
     &  + HR1(1) *HR5(0,1,-1,-1,1)
     &  + HR1(1) *HR5(0,1,-1,1,-1)
     &  + HR1(1) *HR5(0,1,-1,1,1)
     &  + HR1(1) *HR5(0,1,1,-1,-1)
     &  + HR1(1) *HR5(0,1,1,-1,1)
     &  + HR1(1) *HR5(0,1,1,1,-1)
     &  + HR1(1) *HR5(0,1,1,1,1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  - HR6( -1,-1,-1,-1,1,1)
     &  + HR6( -1,-1,-1,1,1,1)
     &  - HR6( -1,-1,1,1,1,1)
     &  + HR6( -1,1,1,1,1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - HR6(0, -1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,-1,1,1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  - HR6(0, -1,-1,1,-1,1)
     &  - HR6(0, -1,-1,1,1,-1)
     &  - HR6(0, -1,-1,1,1,1)
     &  - HR6(0, -1,1,-1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,1)
     &  - HR6(0, -1,1,-1,1,-1)
     &  - HR6(0, -1,1,-1,1,1)
     &  - HR6(0, -1,1,1,-1,-1)
     &  - HR6(0, -1,1,1,-1,1)
     &  - HR6(0, -1,1,1,1,-1)
     &  - HR6(0, -1,1,1,1,1)
     &  - HR6(0,1, -1,-1,-1,-1)
     &  - HR6(0,1, -1,-1,-1,1)
     &  - HR6(0,1, -1,-1,1,-1)
     &  - HR6(0,1, -1,-1,1,1)
     &  - HR6(0,1, -1,1,-1,-1)
     &  - HR6(0,1, -1,1,-1,1)
     &  - HR6(0,1, -1,1,1,-1)
     &  - HR6(0,1, -1,1,1,1)
     &  - HR6(0,1,1, -1,-1,-1)
     &  - HR6(0,1,1, -1,-1,1)
     &  - HR6(0,1,1, -1,1,-1)
     &  - HR6(0,1,1, -1,1,1)
     &  - HR6(0,1,1,1, -1,-1)
     &  - HR6(0,1,1,1, -1,1)
     &  - HR6(0,1,1,1,1, -1)
     &  - HR6(0,1,1,1,1,1) 
      HY6(0,0,0,0,1,-1) = 
     &  + 3.4500854404017275d-02
     &  - 7.4276054639867797d-02*HR1(-1)
     &  + 8.6422639119492193d-02*HR1(-1)*HR1(-1)
     &  - 8.4702535467447475d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 2.5410760640234242d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.6567338509580348d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(0,-1,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,1)
     &  + HR1( -1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,1,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.7284527823898438d-01*HR1(-1)*HR1(1)
     &  - 2.5410760640234242d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,1)
     &  + HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR1(1)*HR4(-1,1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(0,1,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,1,1)
     &  + HR1( -1)*HR5(-1,-1,1,1,1)
     &  - HR1( -1)*HR5(-1,1,1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,1,-1)
     &  - HR1( -1)*HR5(0,-1,1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,1,-1)
     &  - HR1( -1)*HR5(0,1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,1,1,-1,-1)
     &  - HR1( -1)*HR5(0,1,1,-1,1)
     &  - HR1( -1)*HR5(0,1,1,1,-1)
     &  - 2.8881132523331054d-02*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 7.4276054639867797d-02*HR1(1)
     &  + 8.6422639119492193d-02*HR1(1)*HR1(1)
     &  - 8.4702535467447475d-02*HR1(1)*HR1(1)*HR1(1)
     &  - 4.4278897515967247d-02*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(1)*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 3.3333333333333333d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  - 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,1,1)
     &  - HR1(1) *HR1(1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(-1,-1,1,1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR1(1)*HR4(0,-1,-1,1)
     &  + HR1(1) *HR1(1)*HR4(0,-1,1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,1)
     &  + HR1(1) *HR1(1)*HR4(0,1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,1,1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,1,1)
     &  - HR1(1) *HR5(-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,1,-1)
     &  - HR1(1) *HR5(0,-1,1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,1,-1,1,-1)
     &  - HR1(1) *HR5(0,1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,1,1,-1,-1)
     &  - HR1(1) *HR5(0,1,1,-1,1)
     &  - HR1(1) *HR5(0,1,1,1,-1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,1,1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,1,1,1)
     &  - 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(-1,-1,-1,1,1,1)
     &  + HR6( -1,-1,1,1,1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,-1,1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,1,1,-1)
     &  + HR6(0, -1,1,1,1,1)
     &  + 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,1,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,1,-1,1,1,-1)
     &  + HR6(0,1, -1,1,1,1)
     &  + 3.0000000000000000d+00*HR6(0,1,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,1,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,1,1,-1,1,-1)
     &  + HR6(0,1,1, -1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,1,1,1,-1,-1)
     &  + HR6(0,1,1,1, -1,1)
     &  + HR6(0,1,1,1,1, -1)
      HY6(0,0,0,0,1,1) = 
     &  + 4.0536897271519737d-02
     &  - 9.6551159989443734d-02*HR1(-1)
     &  + 1.3529040421389227d-01*HR1(-1)*HR1(-1)
     &  - 2.0034281719326571d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 1.2500000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,1,1)
     &  - 6.0102845157979714d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,1)
     &  - HR1( -1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,1,1)
     &  - HR1( -1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,1,-1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,1,1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,1,1,1)
     &  + 2.7058080842778454d-01*HR1(-1)*HR1(1)
     &  - 6.0102845157979714d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,1)
     &  - HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR1(1)*HR4(-1,1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,1,1)
     &  - HR1( -1)*HR5(-1,-1,1,1,1)
     &  + HR1( -1)*HR5(-1,1,1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,1,-1,1)
     &  + HR1( -1)*HR5(0,-1,1,1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,1)
     &  + HR1( -1)*HR5(0,0,-1,1,-1)
     &  + HR1( -1)*HR5(0,0,-1,1,1)
     &  + HR1( -1)*HR5(0,0,1,-1,-1)
     &  + HR1( -1)*HR5(0,0,1,-1,1)
     &  + HR1( -1)*HR5(0,0,1,1,-1)
     &  + HR1( -1)*HR5(0,0,1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,1,-1)
     &  + HR1( -1)*HR5(0,1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,1,1,-1,-1)
     &  + HR1( -1)*HR5(0,1,1,-1,1)
     &  + HR1( -1)*HR5(0,1,1,1,-1)
     &  + 2.0833333333333333d-02*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,1,1)
     &  + HR1(0) *HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(-1,-1,1,1)
     &  + HR1(0) *HR1(1)*HR4(-1,1,1,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,1,1)
     &  - HR1(0) *HR1(1)*HR4(0,1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,1,1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,1,1,1)
     &  + HR1(0) *HR5(-1,-1,-1,-1,1)
     &  - HR1(0) *HR5(-1,-1,-1,1,1)
     &  + HR1(0) *HR5(-1,-1,1,1,1)
     &  - HR1(0) *HR5(-1,1,1,1,1)
     &  + HR1(0) *HR5(0,-1,-1,-1,-1)
     &  + HR1(0) *HR5(0,-1,-1,-1,1)
     &  + HR1(0) *HR5(0,-1,-1,1,-1)
     &  + HR1(0) *HR5(0,-1,-1,1,1)
     &  + HR1(0) *HR5(0,-1,1,-1,-1)
     &  + HR1(0) *HR5(0,-1,1,-1,1)
     &  + HR1(0) *HR5(0,-1,1,1,-1)
     &  + HR1(0) *HR5(0,-1,1,1,1)
     &  + HR1(0) *HR5(0,1,-1,-1,-1)
     &  + HR1(0) *HR5(0,1,-1,-1,1)
     &  + HR1(0) *HR5(0,1,-1,1,-1)
     &  + HR1(0) *HR5(0,1,-1,1,1)
     &  + HR1(0) *HR5(0,1,1,-1,-1)
     &  + HR1(0) *HR5(0,1,1,-1,1)
     &  + HR1(0) *HR5(0,1,1,1,-1)
     &  + HR1(0) *HR5(0,1,1,1,1)
     &  - 9.6551159989443734d-02*HR1(1)
     &  + 1.3529040421389227d-01*HR1(1)*HR1(1)
     &  - 2.0034281719326571d-01*HR1(1)*HR1(1)*HR1(1)
     &  + 1.0009437789962529d-02*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,0,1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,1,1)
     &  + HR1(1) *HR1(1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(-1,-1,1,1)
     &  - HR1(1) *HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,1,1)
     &  - HR1(1) *HR1(1)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,1,1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,1,1)
     &  + HR1(1) *HR5(-1,-1,1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  + HR1(1) *HR5(0,-1,1,-1,1)
     &  + HR1(1) *HR5(0,-1,1,1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,1)
     &  + HR1(1) *HR5(0,0,-1,1,-1)
     &  + HR1(1) *HR5(0,0,-1,1,1)
     &  + HR1(1) *HR5(0,0,1,-1,-1)
     &  + HR1(1) *HR5(0,0,1,-1,1)
     &  + HR1(1) *HR5(0,0,1,1,-1)
     &  + HR1(1) *HR5(0,0,1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,1,-1,1,-1)
     &  + HR1(1) *HR5(0,1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,1,1,-1,-1)
     &  + HR1(1) *HR5(0,1,1,-1,1)
     &  + HR1(1) *HR5(0,1,1,1,-1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,1,1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,1,1,1)
     &  + 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(-1,-1,-1,1,1,1)
     &  - HR6( -1,-1,1,1,1,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  - HR6(0, -1,-1,1,1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,-1,1,-1)
     &  - HR6(0, -1,1,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  - HR6(0, -1,1,1,-1,1)
     &  - HR6(0, -1,1,1,1,-1)
     &  - HR6(0,0, -1,-1,-1,-1)
     &  - HR6(0,0, -1,-1,-1,1)
     &  - HR6(0,0, -1,-1,1,-1)
     &  - HR6(0,0, -1,-1,1,1)
     &  - HR6(0,0, -1,1,-1,-1)
     &  - HR6(0,0, -1,1,-1,1)
     &  - HR6(0,0, -1,1,1,-1)
     &  - HR6(0,0, -1,1,1,1)
     &  - HR6(0,0,1, -1,-1,-1)
     &  - HR6(0,0,1, -1,-1,1)
     &  - HR6(0,0,1, -1,1,-1)
     &  - HR6(0,0,1, -1,1,1)
     &  - HR6(0,0,1,1, -1,-1)
     &  - HR6(0,0,1,1, -1,1)
     &  - HR6(0,0,1,1,1, -1)
     &  - HR6(0,0,1,1,1,1) 
     &  - 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,1,-1,1,1,-1)
     &  - HR6(0,1, -1,1,1,1)
     &  - 3.0000000000000000d+00*HR6(0,1,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,1,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,1,1,-1,1,-1)
     &  - HR6(0,1,1, -1,1,1)
     &  - 2.0000000000000000d+00*HR6(0,1,1,1,-1,-1)
     &  - HR6(0,1,1,1, -1,1)
     &  - HR6(0,1,1,1,1, -1)
      HY6(0,0,0,1,-1,-1) = 
     &  + 7.2761575180718402d-03
     &  - 2.5535023438634211d-02*HR1(-1)
     &  + 5.7061713708030422d-02*HR1(-1)*HR1(-1)
     &  + 7.9222950181883112d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.3766885054564933d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.1412342741606084d-01*HR1(-1)*HR1(1)
     &  + 2.3766885054564933d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,1,1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  - HR1( -1)*HR5(-1,-1,1,1,1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,1,-1,1)
     &  + HR1( -1)*HR5(0,-1,1,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,1,-1,-1,1)
     &  + HR1( -1)*HR5(0,1,-1,1,-1)
     &  + HR1( -1)*HR5(0,1,1,-1,-1)
     &  + 4.0037751159850118d-02*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 2.5535023438634211d-02*HR1(1)
     &  + 5.7061713708030422d-02*HR1(1)*HR1(1)
     &  + 7.9222950181883112d-02*HR1(1)*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,-1,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR1(1)*HR3(-1,1,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,1,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - HR1(1) *HR5(-1,-1,-1,1,1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  + HR1(1) *HR5(0,-1,1,-1,1)
     &  + HR1(1) *HR5(0,-1,1,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  + HR1(1) *HR5(0,1,-1,-1,1)
     &  + HR1(1) *HR5(0,1,-1,1,-1)
     &  + HR1(1) *HR5(0,1,1,-1,-1)
     &  + 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(-1,-1,1,1)
     &  + 2.4022650695910071d-01*HR4(-1,1,1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,1,1)
     &  - 2.4022650695910071d-01*HR4(0,1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,1,1,-1)
     &  - 2.4022650695910071d-01*HR4(0,1,1,1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(-1,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,1,1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,1,1)
     &  + 2.0794415416798359d+00*HR5(0,1,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,1)
     &  + 1.3862943611198906d+00*HR5(0,1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,1,1,-1)
     &  + 6.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + HR6( -1,-1,-1,1,1,1)
     &  - 1.0000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  - HR6(0, -1,-1,1,1,1)
     &  - 6.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,-1,1,-1)
     &  - HR6(0, -1,1,-1,1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  - HR6(0, -1,1,1,-1,1)
     &  - HR6(0, -1,1,1,1,-1)
     &  - 6.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  - HR6(0,1, -1,-1,1,1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  - HR6(0,1, -1,1,-1,1)
     &  - HR6(0,1, -1,1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,1,1,-1,-1,-1)
     &  - HR6(0,1,1, -1,-1,1)
     &  - HR6(0,1,1, -1,1,-1)
     &  - HR6(0,1,1,1, -1,-1)
      HY6(0,0,0,1,-1,1) = 
     &  + 9.3074022781308142d-03
     &  - 3.6321732111088421d-02*HR1(-1)
     &  + 9.6777676906532623d-02*HR1(-1)*HR1(-1)
     &  + 2.4633412775717367d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 7.3900238327152102d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.4556013161625312d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR1(0)*HR4(-1,1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR1(0)*HR4(0,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR1(0)*HR4(0,1,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(0,1,1,-1)
     &  + 1.9355535381306524d-01*HR1(-1)*HR1(1)
     &  + 7.3900238327152102d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,0,1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,-1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,1,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  + HR1( -1)*HR5(-1,-1,1,1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,1)
     &  + HR1( -1)*HR5(0,-1,0,-1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,1)
     &  + HR1( -1)*HR5(0,-1,0,1,-1)
     &  + HR1( -1)*HR5(0,-1,0,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,1,-1,1)
     &  - HR1( -1)*HR5(0,-1,1,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,1,-1,-1,1)
     &  - HR1( -1)*HR5(0,1,-1,1,-1)
     &  - HR1( -1)*HR5(0,1,1,-1,-1)
     &  + 9.7040087744168750d-02*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1(0) *HR1(1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,1,-1)
     &  + HR1(0) *HR1(1)*HR4(0,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1(0) *HR1(1)*HR4(0,1,-1,1)
     &  + HR1(0) *HR1(1)*HR4(0,1,1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  - HR1(0) *HR5(-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,1,1,-1)
     &  - HR1(0) *HR5(0,-1,1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,1,-1,1,-1)
     &  - HR1(0) *HR5(0,1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,1,1,-1,-1)
     &  - HR1(0) *HR5(0,1,1,-1,1)
     &  - HR1(0) *HR5(0,1,1,1,-1)
     &  - 3.6321732111088421d-02*HR1(1)
     &  + 9.6777676906532623d-02*HR1(1)*HR1(1)
     &  + 2.4633412775717367d-01*HR1(1)*HR1(1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.3333333333333333d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 3.3333333333333333d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 2.9112026323250625d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 2.9112026323250625d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - 2.9112026323250625d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR1(1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,-1)
     &  + HR1(1) *HR1(1)*HR4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1(1) *HR2(0,-1)*HR3(-1,1,1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,1,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,1,1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(-1,-1,1)
     &  - 5.8224052646501250d-01*HR1(1)*HR3(-1,1,1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(0,-1,1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(0,1,-1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(0,1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + HR1(1) *HR5(-1,-1,-1,1,1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,1)
     &  + HR1(1) *HR5(0,-1,0,-1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,1)
     &  + HR1(1) *HR5(0,-1,0,1,-1)
     &  + HR1(1) *HR5(0,-1,0,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  - HR1(1) *HR5(0,-1,1,-1,1)
     &  - HR1(1) *HR5(0,-1,1,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  - HR1(1) *HR5(0,1,-1,-1,1)
     &  - HR1(1) *HR5(0,1,-1,1,-1)
     &  - HR1(1) *HR5(0,1,1,-1,-1)
     &  - HR2(0, -1)*HR4(-1,-1,-1,1)
     &  + HR2(0, -1)*HR4(-1,-1,1,1)
     &  - HR2(0, -1)*HR4(-1,1,1,1)
     &  + HR2(0, -1)*HR4(0,-1,-1,-1)
     &  + HR2(0, -1)*HR4(0,-1,-1,1)
     &  + HR2(0, -1)*HR4(0,-1,1,-1)
     &  + HR2(0, -1)*HR4(0,-1,1,1)
     &  + HR2(0, -1)*HR4(0,1,-1,-1)
     &  + HR2(0, -1)*HR4(0,1,-1,1)
     &  + HR2(0, -1)*HR4(0,1,1,-1)
     &  + HR2(0, -1)*HR4(0,1,1,1)
     &  + 5.8224052646501250d-01*HR4(-1,-1,-1,1)
     &  - 5.8224052646501250d-01*HR4(-1,-1,1,1)
     &  + 5.8224052646501250d-01*HR4(-1,1,1,1)
     &  - 5.8224052646501250d-01*HR4(0,-1,-1,-1)
     &  - 5.8224052646501250d-01*HR4(0,-1,-1,1)
     &  - 5.8224052646501250d-01*HR4(0,-1,1,-1)
     &  - 5.8224052646501250d-01*HR4(0,-1,1,1)
     &  - 5.8224052646501250d-01*HR4(0,1,-1,-1)
     &  - 5.8224052646501250d-01*HR4(0,1,-1,1)
     &  - 5.8224052646501250d-01*HR4(0,1,1,-1)
     &  - 5.8224052646501250d-01*HR4(0,1,1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(-1,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,1,1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,1,1)
     &  - 2.0794415416798359d+00*HR5(0,1,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,1,1)
     &  - 1.3862943611198906d+00*HR5(0,1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,1,1,-1)
     &  - 6.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - HR6( -1,-1,-1,1,1,1)
     &  + 1.1000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,1,1)
     &  - HR6(0, -1,0,-1,-1,-1)
     &  - HR6(0, -1,0,-1,-1,1)
     &  - HR6(0, -1,0,-1,1,-1)
     &  - HR6(0, -1,0,-1,1,1)
     &  - HR6(0, -1,0,1,-1,-1)
     &  - HR6(0, -1,0,1,-1,1)
     &  - HR6(0, -1,0,1,1,-1)
     &  - HR6(0, -1,0,1,1,1)
     &  + 6.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,-1,1,-1)
     &  + HR6(0, -1,1,-1,1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  + HR6(0, -1,1,1,-1,1)
     &  + HR6(0, -1,1,1,1,-1)
     &  + 6.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  + HR6(0,1, -1,-1,1,1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  + HR6(0,1, -1,1,-1,1)
     &  + HR6(0,1, -1,1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,1,1,-1,-1,-1)
     &  + HR6(0,1,1, -1,-1,1)
     &  + HR6(0,1,1, -1,1,-1)
     &  + HR6(0,1,1,1, -1,-1)
      HY6(0,0,0,1,0,-1) = 
     &  + 7.8031149720713994d-02
     &  - 1.8615775173851248d-01*HR1(-1)
     &  + 2.8426294001954845d-01*HR1(-1)*HR1(-1)
     &  + 2.2022937829936213d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 3.4269459726004717d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.3707783890401886d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.3707783890401886d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 6.6068813489808640d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR4(0,-1,1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR4(0,1,1,-1)
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.6852588003909690d-01*HR1(-1)*HR1(1)
     &  + 6.6068813489808640d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.3707783890401886d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR3(0,-1,1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR3(0,1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR3(0,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,1,1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,1,-1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,1,1,-1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(-1,1,1,1,1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,1,1)
     &  + 5.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,1,-1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,1,-1)
     &  + HR1( -1)*HR5(0,1,-1,1,1)
     &  + 5.0000000000000000d+00*HR1(-1)*HR5(0,1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,1,1,1,-1)
     &  + 1.3707783890401886d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.8615775173851248d-01*HR1(1)
     &  + 2.8426294001954845d-01*HR1(1)*HR1(1)
     &  + 2.2022937829936213d-01*HR1(1)*HR1(1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 3.3333333333333333d-01*HR1(1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 4.1123351671205660d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 4.1123351671205660d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 4.1123351671205660d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(1) *HR1(1)*HR4(-1,-1,1,1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,1)
     &  - HR1(1) *HR1(1)*HR4(0,-1,1,-1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,-1,1)
     &  - HR1(1) *HR1(1)*HR4(0,1,1,-1)
     &  + HR1(1) *HR2(-1,1)*HR3(-1,1,1)
     &  + 8.2246703342411321d-01*HR1(1)*HR3(-1,-1,1)
     &  - 8.2246703342411321d-01*HR1(1)*HR3(-1,1,1)
     &  + 8.2246703342411321d-01*HR1(1)*HR3(0,-1,-1)
     &  + 8.2246703342411321d-01*HR1(1)*HR3(0,-1,1)
     &  + 8.2246703342411321d-01*HR1(1)*HR3(0,1,-1)
     &  + 8.2246703342411321d-01*HR1(1)*HR3(0,1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(-1,1,1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,1,-1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,1,1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,1,1,1)
     &  + 7.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,1,1)
     &  - HR1(1) *HR5(-1,-1,1,-1,1)
     &  - HR1(1) *HR5(-1,1,-1,1,1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,-1,-1,1,1)
     &  + 5.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,1,1,-1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,1,-1,1,-1)
     &  + HR1(1) *HR5(0,1,-1,1,1)
     &  + 5.0000000000000000d+00*HR1(1)*HR5(0,1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,1,1,1,-1)
     &  - HR2( -1,1)*HR4(-1,1,1,1)
     &  + 8.2246703342411321d-01*HR4(-1,-1,-1,1)
     &  - 8.2246703342411321d-01*HR4(-1,-1,1,1)
     &  + 8.2246703342411321d-01*HR4(-1,1,1,1)
     &  - 8.2246703342411321d-01*HR4(0,-1,-1,-1)
     &  - 8.2246703342411321d-01*HR4(0,-1,-1,1)
     &  - 8.2246703342411321d-01*HR4(0,-1,1,-1)
     &  - 8.2246703342411321d-01*HR4(0,-1,1,1)
     &  - 8.2246703342411321d-01*HR4(0,1,-1,-1)
     &  - 8.2246703342411321d-01*HR4(0,1,-1,1)
     &  - 8.2246703342411321d-01*HR4(0,1,1,-1)
     &  - 8.2246703342411321d-01*HR4(0,1,1,1)
     &  + 2.7725887222397812d+00*HR5(-1,-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR5(-1,-1,-1,1,1)
     &  + 2.7725887222397812d+00*HR5(-1,-1,1,1,1)
     &  - 2.7725887222397812d+00*HR5(-1,1,1,1,1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,1,-1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,1,1)
     &  + 2.7725887222397812d+00*HR5(0,-1,1,-1,-1)
     &  + 2.7725887222397812d+00*HR5(0,-1,1,-1,1)
     &  + 2.7725887222397812d+00*HR5(0,-1,1,1,-1)
     &  + 2.7725887222397812d+00*HR5(0,-1,1,1,1)
     &  + 2.7725887222397812d+00*HR5(0,1,-1,-1,-1)
     &  + 2.7725887222397812d+00*HR5(0,1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR5(0,1,-1,1,-1)
     &  + 2.7725887222397812d+00*HR5(0,1,-1,1,1)
     &  + 2.7725887222397812d+00*HR5(0,1,1,-1,-1)
     &  + 2.7725887222397812d+00*HR5(0,1,1,-1,1)
     &  + 2.7725887222397812d+00*HR5(0,1,1,1,-1)
     &  + 2.7725887222397812d+00*HR5(0,1,1,1,1)
     &  + 1.1000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  - 1.1000000000000000d+01*HR6(-1,-1,-1,-1,1,1)
     &  - HR6( -1,-1,-1,1,-1,1)
     &  + 1.0000000000000000d+01*HR6(-1,-1,-1,1,1,1)
     &  + 2.0000000000000000d+00*HR6(-1,-1,1,-1,1,1)
     &  + HR6( -1,-1,1,1,-1,1)
     &  + HR6( -1,1,-1,1,1,1)
     &  - 1.0000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  - 8.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  - 5.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  - HR6(0, -1,-1,1,1,1)
     &  - 9.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - 5.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR6(0,-1,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,-1,1,1)
     &  - 7.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,1,1,1,-1)
     &  - 1.0000000000000000d+01*HR6(0,1,-1,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,1,1)
     &  - 8.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,1,-1,1,-1,1)
     &  - 5.0000000000000000d+00*HR6(0,1,-1,1,1,-1)
     &  - HR6(0,1, -1,1,1,1)
     &  - 9.0000000000000000d+00*HR6(0,1,1,-1,-1,-1)
     &  - 5.0000000000000000d+00*HR6(0,1,1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR6(0,1,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,1,1,-1,1,1)
     &  - 7.0000000000000000d+00*HR6(0,1,1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,1,1,1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,1,1,1,1,-1)
      HY6(0,0,0,1,0,1) = 
     &  + 8.8483382454368714d-02
     &  - 2.2881039760335375d-01*HR1(-1)
     &  + 4.0587121264167682d-01*HR1(-1)*HR1(-1)
     &  + 5.9071586949367355d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 6.8538919452009434d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,1,1)
     &  + 1.7721476084810206d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(0,1,1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,1,1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR1(0)*HR4(-1,1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,1,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,1,1,1)
     &  + 8.1174242528335364d-01*HR1(-1)*HR1(1)
     &  + 1.7721476084810206d+00*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.7415567780803773d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(0,0,1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,1,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,1,1)
     &  - HR1( -1)*HR2(0,1)*HR3(0,1,-1)
     &  - HR1( -1)*HR2(0,1)*HR3(0,1,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(0,-1,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(0,1,-1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(0,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,1,-1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,1,1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,1,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(-1,1,1,1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  - HR1( -1)*HR5(0,-1,-1,0,1)
     &  - 5.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,1)
     &  + HR1( -1)*HR5(0,-1,0,-1,-1)
     &  - HR1( -1)*HR5(0,-1,0,1,1)
     &  - 5.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,1)
     &  - HR1( -1)*HR5(0,-1,1,0,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,1)
     &  - 5.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,1,1)
     &  + HR1( -1)*HR5(0,1,0,1,-1)
     &  + HR1( -1)*HR5(0,1,0,1,1)
     &  - 5.0000000000000000d+00*HR1(-1)*HR5(0,1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,1,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,1,1,1,-1)
     &  + 2.7415567780803773d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - HR1(0) *HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1(0) *HR1(1)*HR1(1)*HR3(-1,1,1)
     &  - HR1(0) *HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(1)*HR1(1)*HR3(0,-1,1)
     &  - HR1(0) *HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - HR1(0) *HR1(1)*HR1(1)*HR3(0,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,1,1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,1,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(-1,1,1,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,1,1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,1,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,1,1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,1,1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,1,1,1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,1,1,1,1)
     &  - 2.2881039760335375d-01*HR1(1)
     &  + 4.0587121264167682d-01*HR1(1)*HR1(1)
     &  + 5.9071586949367355d-01*HR1(1)*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - 3.3333333333333333d-01*HR1(1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 3.3333333333333333d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 3.3333333333333333d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 3.3333333333333333d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,0,1)
     &  - 3.3333333333333333d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 8.2246703342411321d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  - 8.2246703342411321d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - 8.2246703342411321d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  + 2.5000000000000000d-01*HR1(1)*HR1(1)*HR2(0,1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1(1) *HR1(1)*HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR1(1)*HR4(0,-1,-1,1)
     &  + HR1(1) *HR1(1)*HR4(0,-1,1,-1)
     &  + HR1(1) *HR1(1)*HR4(0,0,-1,-1)
     &  + HR1(1) *HR1(1)*HR4(0,0,-1,1)
     &  + HR1(1) *HR1(1)*HR4(0,0,1,-1)
     &  + HR1(1) *HR1(1)*HR4(0,0,1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1(1) *HR1(1)*HR4(0,1,-1,1)
     &  + HR1(1) *HR1(1)*HR4(0,1,1,-1)
     &  - HR1(1) *HR2(-1,1)*HR3(-1,1,1)
     &  - HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1(1) *HR2(0,-1)*HR3(-1,1,1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,1,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,1,1)
     &  - HR1(1) *HR2(0,1)*HR3(-1,-1,1)
     &  + HR1(1) *HR2(0,1)*HR3(-1,1,1)
     &  - HR1(1) *HR2(0,1)*HR3(0,1,-1)
     &  - HR1(1) *HR2(0,1)*HR3(0,1,1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(-1,-1,1)
     &  - 1.6449340668482264d+00*HR1(1)*HR3(-1,1,1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(0,-1,-1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(0,-1,1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(0,1,-1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(0,1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(-1,1,1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,1,-1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,1,1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,1,1,1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,1,1)
     &  + HR1(1) *HR5(-1,-1,1,-1,1)
     &  + HR1(1) *HR5(-1,1,-1,1,1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  - HR1(1) *HR5(0,-1,-1,0,1)
     &  - 5.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,1)
     &  + HR1(1) *HR5(0,-1,0,-1,-1)
     &  - HR1(1) *HR5(0,-1,0,1,1)
     &  - 5.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,1)
     &  - HR1(1) *HR5(0,-1,1,0,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,1)
     &  - 5.0000000000000000d+00*HR1(1)*HR5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,1,-1,1,1)
     &  + HR1(1) *HR5(0,1,0,1,-1)
     &  + HR1(1) *HR5(0,1,0,1,1)
     &  - 5.0000000000000000d+00*HR1(1)*HR5(0,1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,1,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,1,1,1,-1)
     &  + HR2( -1,1)*HR4(-1,1,1,1)
     &  - HR2(0, -1)*HR4(-1,-1,-1,1)
     &  + HR2(0, -1)*HR4(-1,-1,1,1)
     &  - HR2(0, -1)*HR4(-1,1,1,1)
     &  + HR2(0, -1)*HR4(0,-1,-1,-1)
     &  + HR2(0, -1)*HR4(0,-1,-1,1)
     &  + HR2(0, -1)*HR4(0,-1,1,-1)
     &  + HR2(0, -1)*HR4(0,-1,1,1)
     &  + HR2(0, -1)*HR4(0,1,-1,-1)
     &  + HR2(0, -1)*HR4(0,1,-1,1)
     &  + HR2(0, -1)*HR4(0,1,1,-1)
     &  + HR2(0, -1)*HR4(0,1,1,1)
     &  - HR2(0,1) *HR4(-1,-1,-1,1)
     &  + HR2(0,1) *HR4(-1,-1,1,1)
     &  - HR2(0,1) *HR4(-1,1,1,1)
     &  + HR2(0,1) *HR4(0,1,-1,-1)
     &  + HR2(0,1) *HR4(0,1,-1,1)
     &  + HR2(0,1) *HR4(0,1,1,-1)
     &  + HR2(0,1) *HR4(0,1,1,1)
     &  + 5.0000000000000000d-01*HR3(0,-1,1)*HR3(0,-1,1)
     &  + 1.6449340668482264d+00*HR4(-1,-1,-1,1)
     &  - 1.6449340668482264d+00*HR4(-1,-1,1,1)
     &  + 1.6449340668482264d+00*HR4(-1,1,1,1)
     &  - 1.6449340668482264d+00*HR4(0,-1,-1,-1)
     &  - 1.6449340668482264d+00*HR4(0,-1,-1,1)
     &  - 1.6449340668482264d+00*HR4(0,-1,1,-1)
     &  - 1.6449340668482264d+00*HR4(0,-1,1,1)
     &  - 1.6449340668482264d+00*HR4(0,1,-1,-1)
     &  - 1.6449340668482264d+00*HR4(0,1,-1,1)
     &  - 1.6449340668482264d+00*HR4(0,1,1,-1)
     &  - 1.6449340668482264d+00*HR4(0,1,1,1)
     &  - 2.7725887222397812d+00*HR5(-1,-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR5(-1,-1,-1,1,1)
     &  - 2.7725887222397812d+00*HR5(-1,-1,1,1,1)
     &  + 2.7725887222397812d+00*HR5(-1,1,1,1,1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,1,-1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,1,1)
     &  - 2.7725887222397812d+00*HR5(0,-1,1,-1,-1)
     &  - 2.7725887222397812d+00*HR5(0,-1,1,-1,1)
     &  - 2.7725887222397812d+00*HR5(0,-1,1,1,-1)
     &  - 2.7725887222397812d+00*HR5(0,-1,1,1,1)
     &  - 2.7725887222397812d+00*HR5(0,1,-1,-1,-1)
     &  - 2.7725887222397812d+00*HR5(0,1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR5(0,1,-1,1,-1)
     &  - 2.7725887222397812d+00*HR5(0,1,-1,1,1)
     &  - 2.7725887222397812d+00*HR5(0,1,1,-1,-1)
     &  - 2.7725887222397812d+00*HR5(0,1,1,-1,1)
     &  - 2.7725887222397812d+00*HR5(0,1,1,1,-1)
     &  - 2.7725887222397812d+00*HR5(0,1,1,1,1)
     &  - 1.1000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  + 1.1000000000000000d+01*HR6(-1,-1,-1,-1,1,1)
     &  + HR6( -1,-1,-1,1,-1,1)
     &  - 1.0000000000000000d+01*HR6(-1,-1,-1,1,1,1)
     &  - 2.0000000000000000d+00*HR6(-1,-1,1,-1,1,1)
     &  - HR6( -1,-1,1,1,-1,1)
     &  - HR6( -1,1,-1,1,1,1)
     &  + 1.1000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + HR6(0, -1,-1,-1,0,1)
     &  + 8.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  + HR6(0, -1,-1,0,-1,1)
     &  + HR6(0, -1,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,1,1)
     &  + 9.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + 5.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  + HR6(0, -1,-1,1,0,1)
     &  + 6.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,1,1)
     &  - HR6(0, -1,0,-1,-1,-1)
     &  - HR6(0, -1,0,-1,1,1)
     &  + HR6(0, -1,0,1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,1,1,1)
     &  + 9.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + 5.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  + HR6(0, -1,1,-1,0,1)
     &  + 6.0000000000000000d+00*HR6(0,-1,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,-1,1,1)
     &  + HR6(0, -1,1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,0,1,1)
     &  + 7.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,1,-1,1)
     &  + HR6(0, -1,1,1,0,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,1,1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,1,1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,-1,1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,-1,1,1)
     &  + 1.1000000000000000d+01*HR6(0,1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  + 8.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,1,-1,-1,1,1)
     &  + 9.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  + 5.0000000000000000d+00*HR6(0,1,-1,1,-1,1)
     &  + 6.0000000000000000d+00*HR6(0,1,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,1,-1,1,1,1)
     &  - HR6(0,1,0,1, -1,-1)
     &  - HR6(0,1,0,1, -1,1)
     &  - HR6(0,1,0,1,1, -1)
     &  - HR6(0,1,0,1,1,1) 
     &  + 9.0000000000000000d+00*HR6(0,1,1,-1,-1,-1)
     &  + 5.0000000000000000d+00*HR6(0,1,1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR6(0,1,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,1,1,-1,1,1)
     &  + 7.0000000000000000d+00*HR6(0,1,1,1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,1,1,1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,1,1,1,1,-1)
      HY6(0,0,0,1,1,-1) = 
     &  + 1.2516258597427692d-02
     &  - 5.7353571803049304d-02*HR1(-1)
     &  + 2.1684618852447759d-01*HR1(-1)*HR1(-1)
     &  - 1.8455064982157776d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 5.5365194946473328d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.6567338509580348d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,1,1)
     &  + 4.3369237704895519d-01*HR1(-1)*HR1(1)
     &  - 5.5365194946473328d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,0,1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,-1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,1,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(0,-1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(0,1,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(0,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  + HR1( -1)*HR5(-1,-1,1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,1,1)
     &  - HR1( -1)*HR5(0,-1,0,-1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,1)
     &  - HR1( -1)*HR5(0,-1,0,1,-1)
     &  - HR1( -1)*HR5(0,-1,0,1,1)
     &  - HR1( -1)*HR5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,1,-1)
     &  - HR1( -1)*HR5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,-1)
     &  - HR1( -1)*HR5(0,0,1,-1,1)
     &  - HR1( -1)*HR5(0,0,1,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,1,-1,-1,1)
     &  - HR1( -1)*HR5(0,1,-1,1,-1)
     &  - HR1( -1)*HR5(0,1,1,-1,-1)
     &  - 5.7762265046662109d-02*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.7711559006386898d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(-1,1,1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,1,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,1,1,1)
     &  - 5.7353571803049304d-02*HR1(1)
     &  + 2.1684618852447759d-01*HR1(1)*HR1(1)
     &  - 1.8455064982157776d-01*HR1(1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 5.3134677019160696d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 5.3134677019160696d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  + 5.3134677019160696d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,0,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1(1) *HR2(0,-1)*HR3(-1,1,1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,-1,1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,1,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,1,1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(-1,-1,1)
     &  + 1.0626935403832139d+00*HR1(1)*HR3(-1,1,1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(0,-1,1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(0,1,-1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(0,1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + HR1(1) *HR5(-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,1,1)
     &  - HR1(1) *HR5(0,-1,0,-1,-1)
     &  - HR1(1) *HR5(0,-1,0,-1,1)
     &  - HR1(1) *HR5(0,-1,0,1,-1)
     &  - HR1(1) *HR5(0,-1,0,1,1)
     &  - HR1(1) *HR5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,1,-1)
     &  - HR1(1) *HR5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,-1)
     &  - HR1(1) *HR5(0,0,1,-1,1)
     &  - HR1(1) *HR5(0,0,1,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  - HR1(1) *HR5(0,1,-1,-1,1)
     &  - HR1(1) *HR5(0,1,-1,1,-1)
     &  - HR1(1) *HR5(0,1,1,-1,-1)
     &  + HR2(0, -1)*HR4(-1,-1,-1,1)
     &  - HR2(0, -1)*HR4(-1,-1,1,1)
     &  + HR2(0, -1)*HR4(-1,1,1,1)
     &  - HR2(0, -1)*HR4(0,-1,-1,-1)
     &  - HR2(0, -1)*HR4(0,-1,-1,1)
     &  - HR2(0, -1)*HR4(0,-1,1,-1)
     &  - HR2(0, -1)*HR4(0,-1,1,1)
     &  - HR2(0, -1)*HR4(0,1,-1,-1)
     &  - HR2(0, -1)*HR4(0,1,-1,1)
     &  - HR2(0, -1)*HR4(0,1,1,-1)
     &  - HR2(0, -1)*HR4(0,1,1,1)
     &  - 1.0626935403832139d+00*HR4(-1,-1,-1,1)
     &  + 1.0626935403832139d+00*HR4(-1,-1,1,1)
     &  - 1.0626935403832139d+00*HR4(-1,1,1,1)
     &  + 1.0626935403832139d+00*HR4(0,-1,-1,-1)
     &  + 1.0626935403832139d+00*HR4(0,-1,-1,1)
     &  + 1.0626935403832139d+00*HR4(0,-1,1,-1)
     &  + 1.0626935403832139d+00*HR4(0,-1,1,1)
     &  + 1.0626935403832139d+00*HR4(0,1,-1,-1)
     &  + 1.0626935403832139d+00*HR4(0,1,-1,1)
     &  + 1.0626935403832139d+00*HR4(0,1,1,-1)
     &  + 1.0626935403832139d+00*HR4(0,1,1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(-1,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,1,1)
     &  - 2.0794415416798359d+00*HR5(0,1,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,1,1)
     &  - 1.3862943611198906d+00*HR5(0,1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,1,1,-1)
     &  - 6.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - HR6( -1,-1,-1,1,1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - HR6(0, -1,-1,1,1,1)
     &  + HR6(0, -1,0,-1,-1,-1)
     &  + HR6(0, -1,0,-1,-1,1)
     &  + HR6(0, -1,0,-1,1,-1)
     &  + HR6(0, -1,0,-1,1,1)
     &  + HR6(0, -1,0,1,-1,-1)
     &  + HR6(0, -1,0,1,-1,1)
     &  + HR6(0, -1,0,1,1,-1)
     &  + HR6(0, -1,0,1,1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + HR6(0, -1,1,-1,-1,1)
     &  + HR6(0, -1,1,-1,1,-1)
     &  + HR6(0, -1,1,1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,1,1,-1)
     &  + HR6(0,0, -1,1,1,1)
     &  + 3.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,-1,1,-1)
     &  + HR6(0,0,1, -1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,1,-1,-1)
     &  + HR6(0,0,1,1, -1,1)
     &  + HR6(0,0,1,1,1, -1)
     &  + 6.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  + HR6(0,1, -1,-1,1,1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  + HR6(0,1, -1,1,-1,1)
     &  + HR6(0,1, -1,1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,1,1,-1,-1,-1)
     &  + HR6(0,1,1, -1,-1,1)
     &  + HR6(0,1,1, -1,1,-1)
     &  + HR6(0,1,1,1, -1,-1)
      HY6(0,0,0,1,1,1) = 
     &  + 1.7489853169011404d-02
     &  - 9.6551159989443734d-02*HR1(-1)
     &  + 5.4116161685556909d-01*HR1(-1)*HR1(-1)
     &  + 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.2500000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  - 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,1,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,0,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,1,1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR1(0)*HR4(-1,1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,1,1,-1)
     &  + 1.0823232337111381d+00*HR1(-1)*HR1(1)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,0,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,0,1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  - HR1( -1)*HR5(-1,-1,1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,1)
     &  + HR1( -1)*HR5(0,0,-1,1,-1)
     &  + HR1( -1)*HR5(0,0,0,-1,-1)
     &  + HR1( -1)*HR5(0,0,0,-1,1)
     &  + HR1( -1)*HR5(0,0,0,1,-1)
     &  + HR1( -1)*HR5(0,0,0,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,-1)
     &  + HR1( -1)*HR5(0,0,1,-1,1)
     &  + HR1( -1)*HR5(0,0,1,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,1,-1,-1,1)
     &  + HR1( -1)*HR5(0,1,-1,1,-1)
     &  + HR1( -1)*HR5(0,1,1,-1,-1)
     &  + 2.7777777777777777d-02*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(-1,-1,1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(-1,1,1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,1,1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,1,1,1)
     &  + 4.0037751159850118d-02*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,0,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,0,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,0,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,0,1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,0,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,1,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(-1,1,1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,1,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  + HR1(0) *HR5(-1,-1,1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  + HR1(0) *HR5(0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,-1)
     &  + HR1(0) *HR5(0,-1,1,-1,1)
     &  + HR1(0) *HR5(0,-1,1,1,-1)
     &  + HR1(0) *HR5(0,0,-1,-1,-1)
     &  + HR1(0) *HR5(0,0,-1,-1,1)
     &  + HR1(0) *HR5(0,0,-1,1,-1)
     &  + HR1(0) *HR5(0,0,-1,1,1)
     &  + HR1(0) *HR5(0,0,1,-1,-1)
     &  + HR1(0) *HR5(0,0,1,-1,1)
     &  + HR1(0) *HR5(0,0,1,1,-1)
     &  + HR1(0) *HR5(0,0,1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,1,-1,1,-1)
     &  + HR1(0) *HR5(0,1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,1,1,-1,-1)
     &  + HR1(0) *HR5(0,1,1,-1,1)
     &  + HR1(0) *HR5(0,1,1,1,-1)
     &  - 9.6551159989443734d-02*HR1(1)
     &  + 5.4116161685556909d-01*HR1(1)*HR1(1)
     &  + 9.2506847774702633d-03*HR1(1)*HR1(1)*HR1(1)
     &  + 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,0,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,0,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,-1,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR1(1)*HR3(-1,1,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,1,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - HR1(1) *HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,1)
     &  + HR1(1) *HR5(0,0,-1,1,-1)
     &  + HR1(1) *HR5(0,0,0,-1,-1)
     &  + HR1(1) *HR5(0,0,0,-1,1)
     &  + HR1(1) *HR5(0,0,0,1,-1)
     &  + HR1(1) *HR5(0,0,0,1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,-1)
     &  + HR1(1) *HR5(0,0,1,-1,1)
     &  + HR1(1) *HR5(0,0,1,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  + HR1(1) *HR5(0,1,-1,-1,1)
     &  + HR1(1) *HR5(0,1,-1,1,-1)
     &  + HR1(1) *HR5(0,1,1,-1,-1)
     &  + 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(-1,-1,1,1)
     &  + 2.4022650695910071d-01*HR4(-1,1,1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,1,1)
     &  - 2.4022650695910071d-01*HR4(0,1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,1,1,-1)
     &  - 2.4022650695910071d-01*HR4(0,1,1,1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(-1,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,1,1)
     &  + 2.0794415416798359d+00*HR5(0,1,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,1)
     &  + 1.3862943611198906d+00*HR5(0,1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,1,1,-1)
     &  + 6.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + HR6( -1,-1,-1,1,1,1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - HR6(0, -1,-1,1,-1,1)
     &  - HR6(0, -1,-1,1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,1)
     &  - HR6(0, -1,1,-1,1,-1)
     &  - HR6(0, -1,1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  - HR6(0,0, -1,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  - HR6(0,0, -1,1,-1,1)
     &  - HR6(0,0, -1,1,1,-1)
     &  - HR6(0,0,0, -1,-1,-1)
     &  - HR6(0,0,0, -1,-1,1)
     &  - HR6(0,0,0, -1,1,-1)
     &  - HR6(0,0,0, -1,1,1)
     &  - HR6(0,0,0,1, -1,-1)
     &  - HR6(0,0,0,1, -1,1)
     &  - HR6(0,0,0,1,1, -1)
     &  - HR6(0,0,0,1,1,1) 
     &  - 3.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,1,-1,1,-1)
     &  - HR6(0,0,1, -1,1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,1,1,-1,-1)
     &  - HR6(0,0,1,1, -1,1)
     &  - HR6(0,0,1,1,1, -1)
     &  - 6.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  - HR6(0,1, -1,-1,1,1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  - HR6(0,1, -1,1,-1,1)
     &  - HR6(0,1, -1,1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,1,1,-1,-1,-1)
     &  - HR6(0,1,1, -1,-1,1)
     &  - HR6(0,1,1, -1,1,-1)
     &  - HR6(0,1,1,1, -1,-1)
      HY6(0,0,1,-1,-1,-1) = 
     &  + 3.2766998166444148d-03
     &  - 1.9555438852482933d-02*HR1(-1)
     &  - 6.3399291898262058d-02*HR1(-1)*HR1(-1)
     &  + 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  - 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.5504108664821579d-02*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.2679858379652411d-01*HR1(-1)*HR1(1)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR2(0,-1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR2(0,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,1,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,-1,-1,1)
     &  - HR1( -1)*HR5(0,-1,-1,1,-1)
     &  - HR1( -1)*HR5(0,-1,1,-1,-1)
     &  - HR1( -1)*HR5(0,1,-1,-1,-1)
     &  - 2.7752054332410789d-02*HR1(0)*HR1(1)*HR1(1)
     &  - 1.9555438852482933d-02*HR1(1)
     &  - 6.3399291898262058d-02*HR1(1)*HR1(1)
     &  + 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.5504108664821579d-02*HR1(1)*HR2(-1,1)
     &  + 5.5504108664821579d-02*HR1(1)*HR2(0,-1)
     &  + 5.5504108664821579d-02*HR1(1)*HR2(0,1)
     &  - 2.4022650695910071d-01*HR1(1)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(1)*HR3(0,-1,1)
     &  - 2.4022650695910071d-01*HR1(1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1(1) *HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - HR1(1) *HR5(0,-1,-1,-1,1)
     &  - HR1(1) *HR5(0,-1,-1,1,-1)
     &  - HR1(1) *HR5(0,-1,1,-1,-1)
     &  - HR1(1) *HR5(0,1,-1,-1,-1)
     &  - 5.5504108664821579d-02*HR3(-1,-1,1)
     &  + 5.5504108664821579d-02*HR3(-1,1,1)
     &  - 5.5504108664821579d-02*HR3(0,-1,-1)
     &  - 5.5504108664821579d-02*HR3(0,-1,1)
     &  - 5.5504108664821579d-02*HR3(0,1,-1)
     &  - 5.5504108664821579d-02*HR3(0,1,1)
     &  - 4.8045301391820142d-01*HR4(-1,-1,-1,1)
     &  + 2.4022650695910071d-01*HR4(-1,-1,1,1)
     &  + 7.2067952087730213d-01*HR4(0,-1,-1,-1)
     &  + 4.8045301391820142d-01*HR4(0,-1,-1,1)
     &  + 4.8045301391820142d-01*HR4(0,-1,1,-1)
     &  + 2.4022650695910071d-01*HR4(0,-1,1,1)
     &  + 4.8045301391820142d-01*HR4(0,1,-1,-1)
     &  + 2.4022650695910071d-01*HR4(0,1,-1,1)
     &  + 2.4022650695910071d-01*HR4(0,1,1,-1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  - 4.1588830833596718d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,1,-1)
     &  - 2.0794415416798359d+00*HR5(0,1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + HR6( -1,-1,-1,-1,1,1)
     &  + 1.0000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + HR6(0, -1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + HR6(0, -1,-1,1,-1,1)
     &  + HR6(0, -1,-1,1,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + HR6(0, -1,1,-1,-1,1)
     &  + HR6(0, -1,1,-1,1,-1)
     &  + HR6(0, -1,1,1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  + HR6(0,1, -1,-1,-1,1)
     &  + HR6(0,1, -1,-1,1,-1)
     &  + HR6(0,1, -1,1,-1,-1)
     &  + HR6(0,1,1, -1,-1,-1)
      HY6(0,0,1,-1,-1,1) = 
     &  + 4.3608223423903728d-03
     &  - 2.8668668263701248d-02*HR1(-1)
     &  - 1.1758989653041252d-01*HR1(-1)*HR1(-1)
     &  + 1.5792167371687950d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 4.7376502115063852d-02*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 4.7376502115063852d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.4556013161625312d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 9.4753004230127705d-02*HR1(-1)*HR1(0)*HR1(1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,1,-1,-1)
     &  - 2.3517979306082505d-01*HR1(-1)*HR1(1)
     &  + 4.7376502115063852d-02*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 9.4753004230127705d-02*HR1(-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,-1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,1,-1)
     &  + 9.4753004230127705d-02*HR1(-1)*HR2(0,1)
     &  - HR1( -1)*HR2(0,1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR3(-1,1,1)
     &  - 1.1644810529300250d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR3(0,-1,1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,0,1)
     &  + HR1( -1)*HR5(0,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  + HR1( -1)*HR5(0,-1,1,-1,-1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,1,-1,-1,-1)
     &  - 4.7376502115063852d-02*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1(0) *HR1(1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - HR1(0) *HR5(-1,-1,-1,1,1)
     &  + 6.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  + HR1(0) *HR5(0,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,-1)
     &  + HR1(0) *HR5(0,-1,1,-1,1)
     &  + HR1(0) *HR5(0,-1,1,1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,-1)
     &  + HR1(0) *HR5(0,1,-1,-1,1)
     &  + HR1(0) *HR5(0,1,-1,1,-1)
     &  + HR1(0) *HR5(0,1,1,-1,-1)
     &  - 2.8668668263701248d-02*HR1(1)
     &  - 1.1758989653041252d-01*HR1(1)*HR1(1)
     &  + 2.9112026323250625d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 9.4753004230127705d-02*HR1(1)*HR2(-1,1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  + 9.4753004230127705d-02*HR1(1)*HR2(0,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,-1,1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,1,-1)
     &  + 9.4753004230127705d-02*HR1(1)*HR2(0,1)
     &  - HR1(1) *HR2(0,1)*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR1(1)*HR3(-1,-1,1)
     &  - 1.1644810529300250d+00*HR1(1)*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR1(1)*HR3(0,-1,1)
     &  - 5.8224052646501250d-01*HR1(1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1(1) *HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,0,1)
     &  + HR1(1) *HR5(0,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  + HR1(1) *HR5(0,-1,1,-1,-1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  - HR2(0, -1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,1,-1)
     &  - HR2(0, -1)*HR4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,1,-1,-1)
     &  - HR2(0, -1)*HR4(0,1,-1,1)
     &  - HR2(0, -1)*HR4(0,1,1,-1)
     &  - 9.4753004230127705d-02*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + 9.4753004230127705d-02*HR3(-1,1,1)
     &  - HR3( -1,1,1)*HR3(0,-1,-1)
     &  - 9.4753004230127705d-02*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,-1,1)
     &  + HR3(0, -1,-1)*HR3(0,1,-1)
     &  + HR3(0, -1,-1)*HR3(0,1,1)
     &  - 9.4753004230127705d-02*HR3(0,-1,1)
     &  - 9.4753004230127705d-02*HR3(0,1,-1)
     &  - 9.4753004230127705d-02*HR3(0,1,1)
     &  - 1.1644810529300250d+00*HR4(-1,-1,-1,1)
     &  + 5.8224052646501250d-01*HR4(-1,-1,1,1)
     &  + 1.7467215793950375d+00*HR4(0,-1,-1,-1)
     &  + 1.1644810529300250d+00*HR4(0,-1,-1,1)
     &  + 1.1644810529300250d+00*HR4(0,-1,1,-1)
     &  + 5.8224052646501250d-01*HR4(0,-1,1,1)
     &  + 1.1644810529300250d+00*HR4(0,1,-1,-1)
     &  + 5.8224052646501250d-01*HR4(0,1,-1,1)
     &  + 5.8224052646501250d-01*HR4(0,1,1,-1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  + 4.1588830833596718d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,1,-1)
     &  + 2.0794415416798359d+00*HR5(0,1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - HR6( -1,-1,-1,-1,1,1)
     &  - 9.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,0,-1,1)
     &  - HR6(0, -1,-1,0,1,-1)
     &  - HR6(0, -1,-1,0,1,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - HR6(0, -1,-1,1,-1,1)
     &  - HR6(0, -1,-1,1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,1)
     &  - HR6(0, -1,1,-1,1,-1)
     &  - HR6(0, -1,1,1,-1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  - HR6(0,1, -1,-1,-1,1)
     &  - HR6(0,1, -1,-1,1,-1)
     &  - HR6(0,1, -1,1,-1,-1)
     &  - HR6(0,1,1, -1,-1,-1)
      HY6(0,0,1,-1,0,-1) = 
     &  + 2.7233592149955328d-02
     &  - 1.2434945547583281d-01*HR1(-1)
     &  - 2.7137925510111777d-01*HR1(-1)*HR1(-1)
     &  + 4.4929413255254634d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 3.4269459726004717d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.3707783890401886d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 1.3478823976576390d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.3478823976576390d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  - 2.6957647953152780d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.4275851020223554d-01*HR1(-1)*HR1(1)
     &  + 1.3478823976576390d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(-1,1,1)
     &  + 2.6957647953152780d-01*HR1(-1)*HR2(0,-1)
     &  + 2.6957647953152780d-01*HR1(-1)*HR2(0,1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR3(-1,1,1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR3(0,-1,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,1,1)
     &  + HR1( -1)*HR5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,1,1)
     &  + HR1( -1)*HR5(-1,1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,1,-1)
     &  - HR1( -1)*HR5(0,1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,1,-1,1,-1)
     &  - 1.3478823976576390d-01*HR1(0)*HR1(1)*HR1(1)
     &  - 1.2434945547583281d-01*HR1(1)
     &  - 2.7137925510111777d-01*HR1(1)*HR1(1)
     &  + 4.1123351671205660d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,-1)
     &  - 2.6957647953152780d-01*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(-1,-1,1)
     &  + 2.6957647953152780d-01*HR1(1)*HR2(0,-1)
     &  + 2.6957647953152780d-01*HR1(1)*HR2(0,1)
     &  - 8.2246703342411321d-01*HR1(1)*HR3(-1,-1,1)
     &  - 1.6449340668482264d+00*HR1(1)*HR3(0,-1,-1)
     &  - 8.2246703342411321d-01*HR1(1)*HR3(0,-1,1)
     &  - 8.2246703342411321d-01*HR1(1)*HR3(0,1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - HR1(1) *HR5(0,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  - HR1(1) *HR5(0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,1,-1)
     &  - HR1(1) *HR5(0,1,-1,-1,-1)
     &  - HR1(1) *HR5(0,1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,1,1)
     &  - 2.6957647953152780d-01*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(-1,1,1)
     &  + 2.6957647953152780d-01*HR3(-1,1,1)
     &  - 2.6957647953152780d-01*HR3(0,-1,-1)
     &  - 2.6957647953152780d-01*HR3(0,-1,1)
     &  - 2.6957647953152780d-01*HR3(0,1,-1)
     &  - 2.6957647953152780d-01*HR3(0,1,1)
     &  - 1.6449340668482264d+00*HR4(-1,-1,-1,1)
     &  + 8.2246703342411321d-01*HR4(-1,-1,1,1)
     &  + 2.4674011002723396d+00*HR4(0,-1,-1,-1)
     &  + 1.6449340668482264d+00*HR4(0,-1,-1,1)
     &  + 1.6449340668482264d+00*HR4(0,-1,1,-1)
     &  + 8.2246703342411321d-01*HR4(0,-1,1,1)
     &  + 1.6449340668482264d+00*HR4(0,1,-1,-1)
     &  + 8.2246703342411321d-01*HR4(0,1,-1,1)
     &  + 8.2246703342411321d-01*HR4(0,1,1,-1)
     &  - 1.3862943611198906d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,1,-1,1,1)
     &  - 4.1588830833596718d+00*HR5(0,-1,-1,-1,-1)
     &  - 4.1588830833596718d+00*HR5(0,-1,-1,-1,1)
     &  - 3.4657359027997265d+00*HR5(0,-1,-1,1,-1)
     &  - 3.4657359027997265d+00*HR5(0,-1,-1,1,1)
     &  - 2.7725887222397812d+00*HR5(0,-1,1,-1,-1)
     &  - 2.7725887222397812d+00*HR5(0,-1,1,-1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,1,1,-1)
     &  - 2.0794415416798359d+00*HR5(0,-1,1,1,1)
     &  - 2.0794415416798359d+00*HR5(0,1,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,1,-1,1,-1)
     &  - 1.3862943611198906d+00*HR5(0,1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,1,-1,1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  + 5.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  - 9.0000000000000000d+00*HR6(-1,-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HR6(-1,-1,1,-1,1,1)
     &  - HR6( -1,-1,1,1,-1,1)
     &  + 1.0000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + HR6(0, -1,-1,-1,1,1)
     &  + 8.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  + 7.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,1,-1,1,-1)
     &  + HR6(0, -1,1,-1,1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,1,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  + HR6(0,1, -1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  + HR6(0,1, -1,1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,1,-1,1,1,-1)
     &  + HR6(0,1,1, -1,-1,-1)
     &  + HR6(0,1,1, -1,1,-1)
      HY6(0,0,1,-1,0,1) = 
     &  + 3.2211985771117729d-02
     &  - 1.5840668952014971d-01*HR1(-1)
     &  - 4.2377986529412007d-01*HR1(-1)*HR1(-1)
     &  + 6.4815974361351054d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 6.8538919452009434d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 1.9444792308405316d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  + 1.9444792308405316d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - 3.8889584616810632d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,1,-1,1)
     &  - 8.4755973058824014d-01*HR1(-1)*HR1(1)
     &  + 1.9444792308405316d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,0,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(-1,1,1)
     &  + 3.8889584616810632d-01*HR1(-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,-1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,1,-1)
     &  + 3.8889584616810632d-01*HR1(-1)*HR2(0,1)
     &  - HR1( -1)*HR2(0,1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(0,1)*HR3(-1,1,1)
     &  - HR1( -1)*HR2(0,1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(0,1)*HR3(0,1,-1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR3(-1,1,1)
     &  - 3.2898681336964528d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR3(0,-1,1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,1,1)
     &  - HR1( -1)*HR5(-1,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,1,1)
     &  - HR1( -1)*HR5(-1,1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,0,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,1)
     &  + HR1( -1)*HR5(0,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,1,-1,1)
     &  + HR1( -1)*HR5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,1,-1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,1,1,-1)
     &  - HR1( -1)*HR5(0,1,-1,-1,1)
     &  + HR1( -1)*HR5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,1,0,1,-1)
     &  - 1.9444792308405316d-01*HR1(0)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,1,1)
     &  - HR1(0) *HR1(1)*HR4(0,1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,1,-1,1)
     &  - HR1(0) *HR2(-1,1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + HR1(0) *HR5(-1,-1,-1,1,1)
     &  + HR1(0) *HR5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,1,1,1)
     &  + HR1(0) *HR5(-1,1,-1,1,1)
     &  + 6.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + 6.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  + 5.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  + 5.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,1,1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,1,-1,1,1)
     &  + HR1(0) *HR5(0,1,1,-1,-1)
     &  + HR1(0) *HR5(0,1,1,-1,1)
     &  - 1.5840668952014971d-01*HR1(1)
     &  - 4.2377986529412007d-01*HR1(1)*HR1(1)
     &  + 8.2246703342411321d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,0,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,-1)
     &  - HR1(1) *HR1(1)*HR4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 3.8889584616810632d-01*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(-1,-1,1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,1,-1)
     &  + 3.8889584616810632d-01*HR1(1)*HR2(0,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,-1,1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,1,-1)
     &  + 3.8889584616810632d-01*HR1(1)*HR2(0,1)
     &  + HR1(1) *HR2(0,1)*HR3(-1,-1,1)
     &  - HR1(1) *HR2(0,1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(0,1)*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR1(1)*HR3(-1,-1,1)
     &  - 3.2898681336964528d+00*HR1(1)*HR3(0,-1,-1)
     &  - 1.6449340668482264d+00*HR1(1)*HR3(0,-1,1)
     &  - 1.6449340668482264d+00*HR1(1)*HR3(0,1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,0,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,1)
     &  + HR1(1) *HR5(0,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,0,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  + HR1(1) *HR5(0,-1,1,-1,1)
     &  + HR1(1) *HR5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,1,-1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,1,1,-1)
     &  - HR1(1) *HR5(0,1,-1,-1,1)
     &  + HR1(1) *HR5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,1,0,1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  - HR2(0, -1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,1,-1)
     &  - HR2(0, -1)*HR4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,1,-1,-1)
     &  - HR2(0, -1)*HR4(0,1,-1,1)
     &  - HR2(0, -1)*HR4(0,1,1,-1)
     &  + 2.0000000000000000d+00*HR2(0,1)*HR4(-1,-1,-1,1)
     &  - HR2(0,1) *HR4(-1,-1,1,1)
     &  - HR2(0,1) *HR4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HR2(0,1)*HR4(0,1,-1,-1)
     &  - HR2(0,1) *HR4(0,1,-1,1)
     &  - HR2(0,1) *HR4(0,1,1,-1)
     &  - 3.8889584616810632d-01*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(-1,1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + HR3( -1,-1,1)*HR3(0,1,-1)
     &  + 3.8889584616810632d-01*HR3(-1,1,1)
     &  - HR3( -1,1,1)*HR3(0,-1,-1)
     &  - HR3( -1,1,1)*HR3(0,1,-1)
     &  - 3.8889584616810632d-01*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,-1,1)
     &  + HR3(0, -1,-1)*HR3(0,1,-1)
     &  + HR3(0, -1,-1)*HR3(0,1,1)
     &  - 3.8889584616810632d-01*HR3(0,-1,1)
     &  - HR3(0, -1,1)*HR3(0,-1,1)
     &  - 3.8889584616810632d-01*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR3(0,1,-1)*HR3(0,1,-1)
     &  + HR3(0,1, -1)*HR3(0,1,1)
     &  - 3.8889584616810632d-01*HR3(0,1,1)
     &  - 3.2898681336964528d+00*HR4(-1,-1,-1,1)
     &  + 1.6449340668482264d+00*HR4(-1,-1,1,1)
     &  + 4.9348022005446793d+00*HR4(0,-1,-1,-1)
     &  + 3.2898681336964528d+00*HR4(0,-1,-1,1)
     &  + 3.2898681336964528d+00*HR4(0,-1,1,-1)
     &  + 1.6449340668482264d+00*HR4(0,-1,1,1)
     &  + 3.2898681336964528d+00*HR4(0,1,-1,-1)
     &  + 1.6449340668482264d+00*HR4(0,1,-1,1)
     &  + 1.6449340668482264d+00*HR4(0,1,1,-1)
     &  + 1.3862943611198906d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,1,-1,1,1)
     &  + 4.1588830833596718d+00*HR5(0,-1,-1,-1,-1)
     &  + 4.1588830833596718d+00*HR5(0,-1,-1,-1,1)
     &  + 3.4657359027997265d+00*HR5(0,-1,-1,1,-1)
     &  + 3.4657359027997265d+00*HR5(0,-1,-1,1,1)
     &  + 2.7725887222397812d+00*HR5(0,-1,1,-1,-1)
     &  + 2.7725887222397812d+00*HR5(0,-1,1,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,1,1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,1,1,1)
     &  + 2.0794415416798359d+00*HR5(0,1,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,1,-1,1,-1)
     &  + 1.3862943611198906d+00*HR5(0,1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,1,-1,1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  - 5.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  + 9.0000000000000000d+00*HR6(-1,-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HR6(-1,-1,1,-1,1,1)
     &  + HR6( -1,-1,1,1,-1,1)
     &  - 9.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,0,1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,0,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,0,1,-1)
     &  - 5.0000000000000000d+00*HR6(0,-1,-1,0,1,1)
     &  - 8.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,0,1)
     &  - 5.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,1,-1)
     &  - HR6(0, -1,0,1,-1,-1)
     &  - HR6(0, -1,0,1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,1,1,-1)
     &  - 7.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,-1,0,1)
     &  - 5.0000000000000000d+00*HR6(0,-1,1,-1,1,-1)
     &  - HR6(0, -1,1,-1,1,1)
     &  - HR6(0, -1,1,0,1,-1)
     &  - 5.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,1,1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,1,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,0,1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  + HR6(0,1, -1,-1,1,1)
     &  - HR6(0,1, -1,0,1,1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  - HR6(0,1, -1,1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,1,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,1,0,1,-1,-1)
     &  - HR6(0,1,1, -1,-1,-1)
     &  - HR6(0,1,1, -1,1,-1)
      HY6(0,0,1,-1,1,-1) = 
     &  + 6.1528318685728338d-03
     &  - 4.7069633474401836d-02*HR1(-1)
     &  - 3.2105039383616431d-01*HR1(-1)*HR1(-1)
     &  + 3.5678728477784371d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.0703618543335311d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 1.0703618543335311d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.6567338509580348d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 2.1407237086670622d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  - 6.4210078767232862d-01*HR1(-1)*HR1(1)
     &  + 1.0703618543335311d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 2.1407237086670622d-01*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,1,-1)
     &  + 2.1407237086670622d-01*HR1(-1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR3(-1,1,1)
     &  + 2.1253870807664278d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR3(0,-1,1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  - 2.7725887222397812d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,1,1)
     &  + 8.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,1)
     &  - HR1( -1)*HR5(0,-1,0,1,-1)
     &  + HR1( -1)*HR5(0,-1,1,-1,-1)
     &  + 1.2000000000000000d+01*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,1,-1,-1,-1)
     &  - 1.0703618543335311d-01*HR1(0)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,1,1,-1)
     &  - 4.7069633474401836d-02*HR1(1)
     &  - 3.2105039383616431d-01*HR1(1)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(1) *HR1(1)*HR4(0,0,-1,-1)
     &  - 2.1407237086670622d-01*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.1407237086670622d-01*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,1,-1)
     &  + 2.1407237086670622d-01*HR1(1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,1)*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR1(1)*HR3(-1,-1,1)
     &  + 2.1253870807664278d+00*HR1(1)*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR1(1)*HR3(0,-1,1)
     &  + 1.0626935403832139d+00*HR1(1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1(1) *HR5(-1,-1,-1,-1,1)
     &  + 8.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  - HR1(1) *HR5(0,-1,0,-1,1)
     &  - HR1(1) *HR5(0,-1,0,1,-1)
     &  + HR1(1) *HR5(0,-1,1,-1,-1)
     &  + 1.2000000000000000d+01*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  + HR2(0, -1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,1,-1)
     &  + HR2(0, -1)*HR4(0,-1,1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,1,-1,-1)
     &  + HR2(0, -1)*HR4(0,1,-1,1)
     &  + HR2(0, -1)*HR4(0,1,1,-1)
     &  - 2.1407237086670622d-01*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  + 2.1407237086670622d-01*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  - 2.1407237086670622d-01*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,1,1)
     &  - 2.1407237086670622d-01*HR3(0,-1,1)
     &  - 2.1407237086670622d-01*HR3(0,1,-1)
     &  - 2.1407237086670622d-01*HR3(0,1,1)
     &  + 2.1253870807664278d+00*HR4(-1,-1,-1,1)
     &  - 1.0626935403832139d+00*HR4(-1,-1,1,1)
     &  - 3.1880806211496417d+00*HR4(0,-1,-1,-1)
     &  - 2.1253870807664278d+00*HR4(0,-1,-1,1)
     &  - 2.1253870807664278d+00*HR4(0,-1,1,-1)
     &  - 1.0626935403832139d+00*HR4(0,-1,1,1)
     &  - 2.1253870807664278d+00*HR4(0,1,-1,-1)
     &  - 1.0626935403832139d+00*HR4(0,1,-1,1)
     &  - 1.0626935403832139d+00*HR4(0,1,1,-1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  + 4.8520302639196171d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,1,-1)
     &  + 2.0794415416798359d+00*HR5(0,1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - HR6( -1,-1,-1,-1,1,1)
     &  - 1.5000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  - 8.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 8.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,1,1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,1,-1)
     &  + HR6(0, -1,0,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,1,-1,-1)
     &  + HR6(0, -1,0,1,-1,1)
     &  + HR6(0, -1,0,1,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,1)
     &  - HR6(0, -1,1,-1,1,-1)
     &  - HR6(0, -1,1,1,-1,-1)
     &  - 1.2000000000000000d+01*HR6(0,0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  - HR6(0,1, -1,-1,-1,1)
     &  - HR6(0,1, -1,-1,1,-1)
     &  - HR6(0,1, -1,1,-1,-1)
     &  - HR6(0,1,1, -1,-1,-1)
      HY6(0,0,1,-1,1,1) = 
     &  + 9.0307642704087900d-03
     &  - 8.2208743029471844d-02*HR1(-1)
     &  - 9.6240249776535762d-01*HR1(-1)*HR1(-1)
     &  + 8.9535532268006700d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 2.6860659680402010d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.2500000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 2.6860659680402010d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  - 5.3721319360804020d-01*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR2(0,-1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,0,1)
     &  + HR1( -1)*HR1(0)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR4(0,1,-1,-1)
     &  - 1.9248049955307152d+00*HR1(-1)*HR1(1)
     &  + 2.6860659680402010d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,0,-1)
     &  + 5.3721319360804020d-01*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,0,-1)
     &  + 5.3721319360804020d-01*HR1(-1)*HR2(0,1)
     &  - HR1( -1)*HR2(0,1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(0,1)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,1,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.7725887222397812d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,1)
     &  + HR1( -1)*HR5(0,-1,0,1,-1)
     &  - HR1( -1)*HR5(0,-1,1,-1,-1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - HR1( -1)*HR5(0,0,-1,-1,1)
     &  + HR1( -1)*HR5(0,0,-1,0,-1)
     &  + HR1( -1)*HR5(0,0,-1,0,1)
     &  - HR1( -1)*HR5(0,1,-1,-1,-1)
     &  + 2.5000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - HR1(0) *HR1(0)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(-1,-1,1,1)
     &  + 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR1(0)*HR4(0,-1,-1,1)
     &  + HR1(0) *HR1(0)*HR4(0,-1,1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,1,1)
     &  + HR1(0) *HR1(0)*HR4(0,1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,1,1,-1)
     &  - 2.6860659680402010d-01*HR1(0)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - HR1(0) *HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - HR1(0) *HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1(0) *HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - HR1(0) *HR1(1)*HR4(-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,0,1)
     &  + HR1(0) *HR1(1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,-1,-1)
     &  + HR1(0) *HR1(1)*HR4(0,1,-1,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1(0) *HR2(0,-1)*HR3(-1,1,1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,-1,1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,1,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,1,1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + HR1(0) *HR5(-1,-1,-1,1,1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,1)
     &  + HR1(0) *HR5(0,-1,0,-1,-1)
     &  + HR1(0) *HR5(0,-1,0,-1,1)
     &  + HR1(0) *HR5(0,-1,0,1,-1)
     &  + HR1(0) *HR5(0,-1,0,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,-1)
     &  - HR1(0) *HR5(0,-1,1,-1,1)
     &  - HR1(0) *HR5(0,-1,1,1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,-1)
     &  - HR1(0) *HR5(0,1,-1,-1,1)
     &  - HR1(0) *HR5(0,1,-1,1,-1)
     &  - HR1(0) *HR5(0,1,1,-1,-1)
     &  - 8.2208743029471844d-02*HR1(1)
     &  - 9.6240249776535762d-01*HR1(1)*HR1(1)
     &  + 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,0,0,-1)
     &  - 5.3721319360804020d-01*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,0,-1)
     &  + 5.3721319360804020d-01*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,0,-1)
     &  + 5.3721319360804020d-01*HR1(1)*HR2(0,1)
     &  - HR1(1) *HR2(0,1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(0,1)*HR3(0,0,-1)
     &  - 2.4022650695910071d-01*HR1(1)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(1)*HR3(0,-1,1)
     &  - 2.4022650695910071d-01*HR1(1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,0,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1(1) *HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  - HR1(1) *HR5(0,-1,0,-1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,1)
     &  + HR1(1) *HR5(0,-1,0,1,-1)
     &  - HR1(1) *HR5(0,-1,1,-1,-1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  - HR1(1) *HR5(0,0,-1,-1,1)
     &  + HR1(1) *HR5(0,0,-1,0,-1)
     &  + HR1(1) *HR5(0,0,-1,0,1)
     &  - HR1(1) *HR5(0,1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,1)
     &  - 5.3721319360804020d-01*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + HR3( -1,-1,1)*HR3(0,0,-1)
     &  + 5.3721319360804020d-01*HR3(-1,1,1)
     &  - HR3( -1,1,1)*HR3(0,-1,-1)
     &  - HR3( -1,1,1)*HR3(0,0,-1)
     &  - 5.3721319360804020d-01*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,-1,1)
     &  + HR3(0, -1,-1)*HR3(0,0,-1)
     &  + HR3(0, -1,-1)*HR3(0,1,-1)
     &  + HR3(0, -1,-1)*HR3(0,1,1)
     &  - 5.3721319360804020d-01*HR3(0,-1,1)
     &  + HR3(0, -1,1)*HR3(0,0,-1)
     &  + HR3(0,0, -1)*HR3(0,1,-1)
     &  + HR3(0,0, -1)*HR3(0,1,1)
     &  - 5.3721319360804020d-01*HR3(0,1,-1)
     &  - 5.3721319360804020d-01*HR3(0,1,1)
     &  - 4.8045301391820142d-01*HR4(-1,-1,-1,1)
     &  + 2.4022650695910071d-01*HR4(-1,-1,1,1)
     &  + 7.2067952087730213d-01*HR4(0,-1,-1,-1)
     &  + 4.8045301391820142d-01*HR4(0,-1,-1,1)
     &  + 4.8045301391820142d-01*HR4(0,-1,1,-1)
     &  + 2.4022650695910071d-01*HR4(0,-1,1,1)
     &  + 4.8045301391820142d-01*HR4(0,1,-1,-1)
     &  + 2.4022650695910071d-01*HR4(0,1,-1,1)
     &  + 2.4022650695910071d-01*HR4(0,1,1,-1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  - 4.8520302639196171d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,1,-1)
     &  - 2.0794415416798359d+00*HR5(0,1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + HR6( -1,-1,-1,-1,1,1)
     &  + 1.4000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  - HR6(0, -1,-1,0,-1,1)
     &  - HR6(0, -1,-1,0,1,-1)
     &  - HR6(0, -1,-1,0,1,1)
     &  + 6.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,1,-1)
     &  - HR6(0, -1,0,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,1,-1,-1)
     &  - HR6(0, -1,0,1,-1,1)
     &  - HR6(0, -1,0,1,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + HR6(0, -1,1,-1,-1,1)
     &  + HR6(0, -1,1,-1,1,-1)
     &  + HR6(0, -1,1,1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + HR6(0,0, -1,-1,-1,1)
     &  + HR6(0,0, -1,-1,1,-1)
     &  + HR6(0,0, -1,-1,1,1)
     &  - HR6(0,0, -1,0,-1,-1)
     &  - HR6(0,0, -1,0,-1,1)
     &  - HR6(0,0, -1,0,1,-1)
     &  - HR6(0,0, -1,0,1,1)
     &  + 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  + HR6(0,1, -1,-1,-1,1)
     &  + HR6(0,1, -1,-1,1,-1)
     &  + HR6(0,1, -1,1,-1,-1)
     &  + HR6(0,1,1, -1,-1,-1)
      HY6(0,0,1,0,-1,-1) = 
     &  + 1.4496092068016896d-02
     &  - 6.7510882956032745d-02*HR1(-1)
     &  - 1.5354295024152495d-01*HR1(-1)*HR1(-1)
     &  + 2.5042852149158214d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 7.5128556447474642d-02*HR1(-1)*HR1(-1)*HR1(0)
     &  + 7.5128556447474642d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  - 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - 1.5025711289494928d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 3.0708590048304991d-01*HR1(-1)*HR1(1)
     &  + 7.5128556447474642d-02*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(-1,1,1)
     &  + 1.5025711289494928d-01*HR1(-1)*HR2(0,-1)
     &  + 1.5025711289494928d-01*HR1(-1)*HR2(0,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,1,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,-1,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,1,-1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(-1,1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,1,1,-1)
     &  + 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  - HR1( -1)*HR5(-1,-1,1,-1,1)
     &  - HR1( -1)*HR5(-1,1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,-1,-1,1)
     &  - HR1( -1)*HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,1,-1,-1,1)
     &  - HR1( -1)*HR5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,1,1,-1,-1)
     &  - 7.5128556447474642d-02*HR1(0)*HR1(1)*HR1(1)
     &  - 6.7510882956032745d-02*HR1(1)
     &  - 1.5354295024152495d-01*HR1(1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 1.5025711289494928d-01*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1(1) *HR2(-1,1)*HR3(-1,-1,1)
     &  + 1.5025711289494928d-01*HR1(1)*HR2(0,-1)
     &  + 1.5025711289494928d-01*HR1(1)*HR2(0,1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(-1,-1,1)
     &  + 4.8045301391820142d-01*HR1(1)*HR3(-1,1,1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,-1,1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,1,-1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,1,1)
     &  - 2.7725887222397812d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,-1,1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,1,1,-1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - HR1(1) *HR5(-1,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - HR1(1) *HR5(0,-1,-1,-1,1)
     &  - HR1(1) *HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  - HR1(1) *HR5(0,1,-1,-1,1)
     &  - HR1(1) *HR5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,1,1)
     &  - 1.5025711289494928d-01*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR3(-1,-1,1)*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(-1,1,1)
     &  + 1.5025711289494928d-01*HR3(-1,1,1)
     &  - 1.5025711289494928d-01*HR3(0,-1,-1)
     &  - 1.5025711289494928d-01*HR3(0,-1,1)
     &  - 1.5025711289494928d-01*HR3(0,1,-1)
     &  - 1.5025711289494928d-01*HR3(0,1,1)
     &  - 7.2067952087730213d-01*HR4(-1,-1,-1,1)
     &  + 7.2067952087730213d-01*HR4(-1,-1,1,1)
     &  - 7.2067952087730213d-01*HR4(-1,1,1,1)
     &  + 7.2067952087730213d-01*HR4(0,-1,-1,-1)
     &  + 7.2067952087730213d-01*HR4(0,-1,-1,1)
     &  + 7.2067952087730213d-01*HR4(0,-1,1,-1)
     &  + 7.2067952087730213d-01*HR4(0,-1,1,1)
     &  + 7.2067952087730213d-01*HR4(0,1,-1,-1)
     &  + 7.2067952087730213d-01*HR4(0,1,-1,1)
     &  + 7.2067952087730213d-01*HR4(0,1,1,-1)
     &  + 7.2067952087730213d-01*HR4(0,1,1,1)
     &  - 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  + 4.8520302639196171d+00*HR5(-1,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,1,-1,1,1)
     &  - 4.1588830833596718d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  - 3.4657359027997265d+00*HR5(0,-1,1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,1,-1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,1,1,-1)
     &  - 4.1588830833596718d+00*HR5(0,1,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR5(0,1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,1,1)
     &  - 3.4657359027997265d+00*HR5(0,1,1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,1,1,-1,1)
     &  - 2.0794415416798359d+00*HR5(0,1,1,1,-1)
     &  - 1.4000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + HR6( -1,-1,1,-1,1,1)
     &  + 1.0000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + HR6(0, -1,-1,-1,1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + HR6(0, -1,-1,1,-1,1)
     &  + HR6(0, -1,-1,1,1,-1)
     &  + 7.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  + 1.0000000000000000d+01*HR6(0,1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  + HR6(0,1, -1,-1,1,1)
     &  + 5.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  + HR6(0,1, -1,1,-1,1)
     &  + HR6(0,1, -1,1,1,-1)
     &  + 7.0000000000000000d+00*HR6(0,1,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,1,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,1,1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,1,1,1,-1,-1)
      HY6(0,0,1,0,-1,1) = 
     &  + 1.8138069583596615d-02
     &  - 9.3069117827206623d-02*HR1(-1)
     &  - 2.7411656736720998d-01*HR1(-1)*HR1(-1)
     &  + 4.0511725278343596d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 1.2153517583503078d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 1.2153517583503078d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.4556013161625312d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - 2.4307035167006157d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,1,1,-1)
     &  - 5.4823313473441997d-01*HR1(-1)*HR1(1)
     &  + 1.2153517583503078d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,0,1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(-1,1,1)
     &  + 2.4307035167006157d-01*HR1(-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,1,1)
     &  + 2.4307035167006157d-01*HR1(-1)*HR2(0,1)
     &  - HR1( -1)*HR2(0,1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(0,1)*HR3(0,-1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR3(-1,1,1)
     &  - 1.1644810529300250d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 1.1644810529300250d+00*HR1(-1)*HR3(0,-1,1)
     &  - 1.1644810529300250d+00*HR1(-1)*HR3(0,1,-1)
     &  - 1.1644810529300250d+00*HR1(-1)*HR3(0,1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(-1,1,1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,1,1,-1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  + HR1( -1)*HR5(-1,-1,1,-1,1)
     &  + HR1( -1)*HR5(-1,1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,-1,0,1)
     &  + HR1( -1)*HR5(0,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,1)
     &  - HR1( -1)*HR5(0,-1,0,1,-1)
     &  + HR1( -1)*HR5(0,-1,1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,1,-1,1)
     &  + HR1( -1)*HR5(0,-1,1,0,1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,1,-1,-1,1)
     &  + HR1( -1)*HR5(0,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,1,1,-1,-1)
     &  - 1.2153517583503078d-01*HR1(0)*HR1(1)*HR1(1)
     &  + HR1(0) *HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,1,1,-1)
     &  + HR1(0) *HR2(-1,1)*HR3(-1,1,1)
     &  + 7.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  - HR1(0) *HR5(-1,-1,1,-1,1)
     &  - HR1(0) *HR5(-1,1,-1,1,1)
     &  + 6.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  + HR1(0) *HR5(0,-1,-1,1,1)
     &  + 5.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,1,1,-1)
     &  + 6.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(0,1,-1,1,-1)
     &  + HR1(0) *HR5(0,1,-1,1,1)
     &  + 5.0000000000000000d+00*HR1(0)*HR5(0,1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,1,1,1,-1)
     &  - 9.3069117827206623d-02*HR1(1)
     &  - 2.7411656736720998d-01*HR1(1)*HR1(1)
     &  - 2.9112026323250625d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 2.9112026323250625d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  + 2.9112026323250625d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,-1)
     &  + HR1(1) *HR1(1)*HR4(0,0,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 2.4307035167006157d-01*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1(1) *HR2(-1,1)*HR3(-1,-1,1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,-1,1)
     &  + 2.4307035167006157d-01*HR1(1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,1,1)
     &  + 2.4307035167006157d-01*HR1(1)*HR2(0,1)
     &  - HR1(1) *HR2(0,1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(0,1)*HR3(0,-1,1)
     &  - 1.1644810529300250d+00*HR1(1)*HR3(-1,-1,1)
     &  + 1.1644810529300250d+00*HR1(1)*HR3(-1,1,1)
     &  - 1.1644810529300250d+00*HR1(1)*HR3(0,-1,-1)
     &  - 1.1644810529300250d+00*HR1(1)*HR3(0,-1,1)
     &  - 1.1644810529300250d+00*HR1(1)*HR3(0,1,-1)
     &  - 1.1644810529300250d+00*HR1(1)*HR3(0,1,1)
     &  + 2.7725887222397812d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,-1,1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,1,1,-1)
     &  + 7.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + HR1(1) *HR5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,0,1)
     &  + HR1(1) *HR5(0,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,1)
     &  - HR1(1) *HR5(0,-1,0,1,-1)
     &  + HR1(1) *HR5(0,-1,1,-1,-1)
     &  - HR1(1) *HR5(0,-1,1,-1,1)
     &  + HR1(1) *HR5(0,-1,1,0,1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  + HR1(1) *HR5(0,1,-1,-1,1)
     &  + HR1(1) *HR5(0,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,1,1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(-1,1,1,1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,1,-1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(0,1,-1,-1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(0,1,-1,1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(0,1,1,-1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(0,1,1,1)
     &  - 2.4307035167006157d-01*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR3(-1,-1,1)*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(-1,1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + HR3( -1,-1,1)*HR3(0,-1,1)
     &  + 2.4307035167006157d-01*HR3(-1,1,1)
     &  - HR3( -1,1,1)*HR3(0,-1,-1)
     &  - HR3( -1,1,1)*HR3(0,-1,1)
     &  - 2.4307035167006157d-01*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,-1,1)
     &  + HR3(0, -1,-1)*HR3(0,1,-1)
     &  + HR3(0, -1,-1)*HR3(0,1,1)
     &  - 2.4307035167006157d-01*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR3(0,-1,1)*HR3(0,-1,1)
     &  + HR3(0, -1,1)*HR3(0,1,-1)
     &  + HR3(0, -1,1)*HR3(0,1,1)
     &  - 2.4307035167006157d-01*HR3(0,1,-1)
     &  - 2.4307035167006157d-01*HR3(0,1,1)
     &  - 1.7467215793950375d+00*HR4(-1,-1,-1,1)
     &  + 1.7467215793950375d+00*HR4(-1,-1,1,1)
     &  - 1.7467215793950375d+00*HR4(-1,1,1,1)
     &  + 1.7467215793950375d+00*HR4(0,-1,-1,-1)
     &  + 1.7467215793950375d+00*HR4(0,-1,-1,1)
     &  + 1.7467215793950375d+00*HR4(0,-1,1,-1)
     &  + 1.7467215793950375d+00*HR4(0,-1,1,1)
     &  + 1.7467215793950375d+00*HR4(0,1,-1,-1)
     &  + 1.7467215793950375d+00*HR4(0,1,-1,1)
     &  + 1.7467215793950375d+00*HR4(0,1,1,-1)
     &  + 1.7467215793950375d+00*HR4(0,1,1,1)
     &  + 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  - 4.8520302639196171d+00*HR5(-1,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,1,-1,1,1)
     &  + 4.1588830833596718d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  + 3.4657359027997265d+00*HR5(0,-1,1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,1,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,1,1,-1)
     &  + 4.1588830833596718d+00*HR5(0,1,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR5(0,1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,1)
     &  + 3.4657359027997265d+00*HR5(0,1,1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,1,1,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,1,1,1,-1)
     &  + 1.4000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - HR6( -1,-1,1,-1,1,1)
     &  - 9.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,0,1,-1)
     &  - HR6(0, -1,-1,0,1,1)
     &  - 5.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - HR6(0, -1,-1,1,-1,1)
     &  - HR6(0, -1,-1,1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,1,1)
     &  + HR6(0, -1,0,1,-1,-1)
     &  + HR6(0, -1,0,1,-1,1)
     &  - 6.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,1)
     &  - HR6(0, -1,1,-1,1,-1)
     &  + HR6(0, -1,1,-1,1,1)
     &  - HR6(0, -1,1,0,1,-1)
     &  - HR6(0, -1,1,0,1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,1,-1,1)
     &  - 1.0000000000000000d+01*HR6(0,1,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  - HR6(0,1, -1,-1,1,1)
     &  - 5.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  - HR6(0,1, -1,1,-1,1)
     &  - HR6(0,1, -1,1,1,-1)
     &  - 7.0000000000000000d+00*HR6(0,1,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,1,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,1,1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,1,1,1,-1,-1)
      HY6(0,0,1,0,1,-1) = 
     &  + 2.3758543482195986d-02
     &  - 1.4122347902560834d-01*HR1(-1)
     &  - 7.0660402489210777d-01*HR1(-1)*HR1(-1)
     &  + 8.4702535467447475d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 2.5410760640234242d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 2.5410760640234242d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.6567338509580348d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(0,0,-1,1)
     &  - HR1( -1)*HR1(-1)*HR4(0,0,1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  - 5.0821521280468485d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,1,1)
     &  - 1.4132080497842155d+00*HR1(-1)*HR1(1)
     &  + 2.5410760640234242d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,0,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(-1,1,1)
     &  + 5.0821521280468485d-01*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,1,1)
     &  + 5.0821521280468485d-01*HR1(-1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(0,1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(0,1)*HR3(0,-1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR3(-1,1,1)
     &  + 2.1253870807664278d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 2.1253870807664278d+00*HR1(-1)*HR3(0,-1,1)
     &  + 2.1253870807664278d+00*HR1(-1)*HR3(0,1,-1)
     &  + 2.1253870807664278d+00*HR1(-1)*HR3(0,1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(-1,1,1,1)
     &  - 2.7725887222397812d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,0,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,0,1,1)
     &  - 2.7725887222397812d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,1,1,-1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  + HR1( -1)*HR5(-1,-1,1,-1,1)
     &  + HR1( -1)*HR5(-1,1,-1,1,1)
     &  + 8.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,1)
     &  + HR1( -1)*HR5(0,-1,0,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,1,-1,1)
     &  - HR1( -1)*HR5(0,-1,1,0,1)
     &  + 1.2000000000000000d+01*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,0,1,1,-1)
     &  + 7.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,1,-1)
     &  + HR1( -1)*HR5(0,1,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,1,1,-1,-1)
     &  - 2.5410760640234242d-01*HR1(0)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,1,1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(-1,1,1,1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(0,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(0,-1,1,-1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(0,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(0,1,-1,-1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(0,1,-1,1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(0,1,1,-1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(0,1,1,1)
     &  - 1.4122347902560834d-01*HR1(1)
     &  - 7.0660402489210777d-01*HR1(1)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.3134677019160696d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - 5.3134677019160696d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,0,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,1,-1)
     &  - HR1(1) *HR1(1)*HR4(0,0,-1,-1)
     &  - HR1(1) *HR1(1)*HR4(0,0,-1,1)
     &  - HR1(1) *HR1(1)*HR4(0,0,1,-1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 5.0821521280468485d-01*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  - HR1(1) *HR2(-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,-1,1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,1,-1)
     &  + 5.0821521280468485d-01*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,1,1)
     &  + 5.0821521280468485d-01*HR1(1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(0,1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(0,1)*HR3(0,-1,1)
     &  + 2.1253870807664278d+00*HR1(1)*HR3(-1,-1,1)
     &  - 2.1253870807664278d+00*HR1(1)*HR3(-1,1,1)
     &  + 2.1253870807664278d+00*HR1(1)*HR3(0,-1,-1)
     &  + 2.1253870807664278d+00*HR1(1)*HR3(0,-1,1)
     &  + 2.1253870807664278d+00*HR1(1)*HR3(0,1,-1)
     &  + 2.1253870807664278d+00*HR1(1)*HR3(0,1,1)
     &  + 2.7725887222397812d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(-1,-1,1,1)
     &  - 2.7725887222397812d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,0,1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,0,1,1)
     &  - 2.7725887222397812d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,1,1,-1)
     &  + 7.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + HR1(1) *HR5(-1,-1,1,-1,1)
     &  + 8.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,1)
     &  + HR1(1) *HR5(0,-1,0,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  + HR1(1) *HR5(0,-1,1,-1,1)
     &  - HR1(1) *HR5(0,-1,1,0,1)
     &  + 1.2000000000000000d+01*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,0,1,1,-1)
     &  + 7.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,1,-1,1,-1)
     &  + HR1(1) *HR5(0,1,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(-1,1,1,1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,1,-1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,1,1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(0,1,-1,1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(0,1,1,-1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(0,1,1,1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(0,1,1)
     &  + HR2(0,1) *HR4(0,-1,1,1)
     &  - 5.0821521280468485d-01*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR3(-1,-1,1)*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  - HR3( -1,-1,1)*HR3(0,-1,1)
     &  - HR3( -1,-1,1)*HR3(0,1,-1)
     &  + 5.0821521280468485d-01*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  + HR3( -1,1,1)*HR3(0,-1,1)
     &  + HR3( -1,1,1)*HR3(0,1,-1)
     &  - 5.0821521280468485d-01*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,1,1)
     &  - 5.0821521280468485d-01*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR3(0,-1,1)*HR3(0,-1,1)
     &  - HR3(0, -1,1)*HR3(0,1,-1)
     &  - HR3(0, -1,1)*HR3(0,1,1)
     &  - 5.0821521280468485d-01*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR3(0,1,-1)*HR3(0,1,-1)
     &  - HR3(0,1, -1)*HR3(0,1,1)
     &  - 5.0821521280468485d-01*HR3(0,1,1)
     &  + 3.1880806211496417d+00*HR4(-1,-1,-1,1)
     &  - 3.1880806211496417d+00*HR4(-1,-1,1,1)
     &  + 3.1880806211496417d+00*HR4(-1,1,1,1)
     &  - 3.1880806211496417d+00*HR4(0,-1,-1,-1)
     &  - 3.1880806211496417d+00*HR4(0,-1,-1,1)
     &  - 3.1880806211496417d+00*HR4(0,-1,1,-1)
     &  - 3.1880806211496417d+00*HR4(0,-1,1,1)
     &  - 3.1880806211496417d+00*HR4(0,1,-1,-1)
     &  - 3.1880806211496417d+00*HR4(0,1,-1,1)
     &  - 3.1880806211496417d+00*HR4(0,1,1,-1)
     &  - 3.1880806211496417d+00*HR4(0,1,1,1)
     &  + 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  - 4.8520302639196171d+00*HR5(-1,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,1,-1,1,1)
     &  + 4.8520302639196171d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,0,1)
     &  + 3.4657359027997265d+00*HR5(0,-1,-1,1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,1,1)
     &  + 3.4657359027997265d+00*HR5(0,-1,1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,0,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,1,1,-1)
     &  + 1.3862943611198906d+00*HR5(0,0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,0,-1,1,-1)
     &  + 2.7725887222397812d+00*HR5(0,0,-1,1,1)
     &  + 1.3862943611198906d+00*HR5(0,0,1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,0,1,-1,1)
     &  + 4.8520302639196171d+00*HR5(0,1,-1,-1,-1)
     &  + 2.7725887222397812d+00*HR5(0,1,-1,-1,1)
     &  + 3.4657359027997265d+00*HR5(0,1,-1,1,-1)
     &  + 1.3862943611198906d+00*HR5(0,1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,0,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,0,1,1)
     &  + 3.4657359027997265d+00*HR5(0,1,1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,1,1,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,1,1,1,-1)
     &  + 1.4000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - HR6( -1,-1,1,-1,1,1)
     &  - 1.5000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  - 8.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 8.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  + HR6(0, -1,-1,0,-1,1)
     &  + HR6(0, -1,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,1,1)
     &  - 7.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  - HR6(0, -1,0,-1,-1,1)
     &  - HR6(0, -1,0,-1,1,-1)
     &  - HR6(0, -1,0,-1,1,1)
     &  - HR6(0, -1,0,1,-1,-1)
     &  - HR6(0, -1,0,1,-1,1)
     &  - HR6(0, -1,0,1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,0,1,1,1)
     &  - 8.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,-1,1,-1)
     &  - HR6(0, -1,1,-1,1,1)
     &  - HR6(0, -1,1,0,1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  - HR6(0, -1,1,1,0,1)
     &  - 1.2000000000000000d+01*HR6(0,0,-1,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,-1,1,1)
     &  - 6.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,1,1,-1)
     &  - 6.0000000000000000d+00*HR6(0,0,-1,1,1,1)
     &  - 6.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,1,-1,1,1)
     &  - 4.0000000000000000d+00*HR6(0,0,1,1,-1,-1)
     &  - 1.4000000000000000d+01*HR6(0,1,-1,-1,-1,-1)
     &  - 7.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,1,1)
     &  + HR6(0,1, -1,0,1,1)
     &  - 7.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,1,-1,1,1,-1)
     &  + HR6(0,1,0,1, -1,1)
     &  + HR6(0,1,0,1,1, -1)
     &  - 7.0000000000000000d+00*HR6(0,1,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,1,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,1,1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,1,1,1,-1,-1)
      HY6(0,0,1,0,1,1) = 
     &  + 3.2309028991669881d-02
     &  - 2.2881039760335375d-01*HR1(-1)
     &  - 2.0400860272155532d+00*HR1(-1)*HR1(-1)
     &  + 2.0034281719326571d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 6.0102845157979714d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.2500000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,0,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 6.0102845157979714d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,1,1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,0,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(0,1,1)
     &  - 1.2020569031595942d+00*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR2(0,-1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(0,1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,1,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,1,1,-1)
     &  - 4.0801720544311065d+00*HR1(-1)*HR1(1)
     &  + 6.0102845157979714d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,0,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(-1,1,1)
     &  + 1.2020569031595942d+00*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,0,1)
     &  + 1.2020569031595942d+00*HR1(-1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(0,1)*HR2(0,1)
     &  - HR1( -1)*HR2(0,1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(0,1)*HR3(0,0,-1)
     &  - HR1( -1)*HR2(0,1)*HR3(0,0,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,1,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,-1,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,1,-1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(-1,1,1,1)
     &  + 2.7725887222397812d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,0,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,0,1,1)
     &  + 2.7725887222397812d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,1,1,-1)
     &  + 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  - HR1( -1)*HR5(-1,-1,1,-1,1)
     &  - HR1( -1)*HR5(-1,1,-1,1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - HR1( -1)*HR5(0,0,-1,-1,1)
     &  + HR1( -1)*HR5(0,0,-1,0,-1)
     &  + HR1( -1)*HR5(0,0,-1,0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,1,-1)
     &  - 5.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,1)
     &  + HR1( -1)*HR5(0,0,1,0,-1)
     &  + HR1( -1)*HR5(0,0,1,0,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,1,1,-1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,1,-1)
     &  - HR1( -1)*HR5(0,1,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,1,1,-1,-1)
     &  - 2.5000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR2(0,1)
     &  - HR1(0) *HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + HR1(0) *HR1(0)*HR1(1)*HR3(-1,1,1)
     &  - HR1(0) *HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(0)*HR1(1)*HR3(0,-1,1)
     &  - HR1(0) *HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - HR1(0) *HR1(0)*HR1(1)*HR3(0,1,1)
     &  - 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(-1,-1,-1,1)
     &  + 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(-1,-1,1,1)
     &  - 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(-1,1,1,1)
     &  + 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(0,-1,-1,-1)
     &  + 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(0,-1,-1,1)
     &  + 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(0,-1,1,-1)
     &  + 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(0,-1,1,1)
     &  + 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(0,1,-1,-1)
     &  + 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(0,1,-1,1)
     &  + 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(0,1,1,-1)
     &  + 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(0,1,1,1)
     &  - 6.0102845157979714d-01*HR1(0)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,1)
     &  - HR1(0) *HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - HR1(0) *HR1(1)*HR1(1)*HR3(0,0,1)
     &  - HR1(0) *HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1(0) *HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1(0) *HR1(1)*HR2(-1,1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1(0) *HR1(1)*HR2(0,-1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(0,1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,1,1,-1)
     &  - HR1(0) *HR2(-1,1)*HR3(-1,1,1)
     &  - HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1(0) *HR2(0,-1)*HR3(-1,1,1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,-1,1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,1,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,1,1)
     &  - HR1(0) *HR2(0,1)*HR3(-1,-1,1)
     &  + HR1(0) *HR2(0,1)*HR3(-1,1,1)
     &  - HR1(0) *HR2(0,1)*HR3(0,1,-1)
     &  - HR1(0) *HR2(0,1)*HR3(0,1,1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(-1,1,1,1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(0,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(0,-1,1,-1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(0,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(0,1,-1,-1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(0,1,-1,1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(0,1,1,-1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(0,1,1,1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  + HR1(0) *HR5(-1,-1,1,-1,1)
     &  + HR1(0) *HR5(-1,1,-1,1,1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  - HR1(0) *HR5(0,-1,-1,0,1)
     &  - 5.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,1)
     &  + HR1(0) *HR5(0,-1,0,-1,-1)
     &  - HR1(0) *HR5(0,-1,0,1,1)
     &  - 5.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,1)
     &  - HR1(0) *HR5(0,-1,1,0,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,0,-1,1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,0,1,-1,1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,1)
     &  - 5.0000000000000000d+00*HR1(0)*HR5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,1,-1,1,1)
     &  + HR1(0) *HR5(0,1,0,1,-1)
     &  + HR1(0) *HR5(0,1,0,1,1)
     &  - 5.0000000000000000d+00*HR1(0)*HR5(0,1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,1,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,1,1,1,-1)
     &  - 2.2881039760335375d-01*HR1(1)
     &  - 2.0400860272155532d+00*HR1(1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 1.2011325347955035d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,0,0,-1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,0,0,1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,0,1,-1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 1.2020569031595942d+00*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  + HR1(1) *HR2(-1,1)*HR3(-1,-1,1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,0,-1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,0,1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,1,-1)
     &  + 1.2020569031595942d+00*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,0,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,0,1)
     &  + 1.2020569031595942d+00*HR1(1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(0,1)*HR2(0,1)
     &  - HR1(1) *HR2(0,1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(0,1)*HR3(0,0,-1)
     &  - HR1(1) *HR2(0,1)*HR3(0,0,1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(-1,-1,1)
     &  + 4.8045301391820142d-01*HR1(1)*HR3(-1,1,1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,-1,1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,1,-1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,1,1)
     &  - 2.7725887222397812d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(-1,-1,1,1)
     &  + 2.7725887222397812d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,0,1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,0,1,1)
     &  + 2.7725887222397812d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,1,1,-1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - HR1(1) *HR5(-1,-1,1,-1,1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  - HR1(1) *HR5(0,-1,0,-1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  - HR1(1) *HR5(0,0,-1,-1,1)
     &  + HR1(1) *HR5(0,0,-1,0,-1)
     &  + HR1(1) *HR5(0,0,-1,0,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,1,-1)
     &  - 5.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,1)
     &  + HR1(1) *HR5(0,0,1,0,-1)
     &  + HR1(1) *HR5(0,0,1,0,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,1,1,-1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,1,-1,1,-1)
     &  - HR1(1) *HR5(0,1,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(0,1,1)
     &  - HR2(0,1) *HR4(0,-1,1,1)
     &  - 1.2020569031595942d+00*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR3(-1,-1,1)*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(-1,1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + HR3( -1,-1,1)*HR3(0,0,-1)
     &  + HR3( -1,-1,1)*HR3(0,0,1)
     &  + HR3( -1,-1,1)*HR3(0,1,-1)
     &  + 1.2020569031595942d+00*HR3(-1,1,1)
     &  - HR3( -1,1,1)*HR3(0,-1,-1)
     &  - HR3( -1,1,1)*HR3(0,0,-1)
     &  - HR3( -1,1,1)*HR3(0,0,1)
     &  - HR3( -1,1,1)*HR3(0,1,-1)
     &  - 1.2020569031595942d+00*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,-1,1)
     &  + HR3(0, -1,-1)*HR3(0,0,-1)
     &  + HR3(0, -1,-1)*HR3(0,0,1)
     &  + HR3(0, -1,-1)*HR3(0,1,-1)
     &  + HR3(0, -1,-1)*HR3(0,1,1)
     &  - 1.2020569031595942d+00*HR3(0,-1,1)
     &  + HR3(0, -1,1)*HR3(0,0,-1)
     &  + HR3(0, -1,1)*HR3(0,0,1)
     &  + HR3(0,0, -1)*HR3(0,1,-1)
     &  + HR3(0,0, -1)*HR3(0,1,1)
     &  + HR3(0,0,1) *HR3(0,1,-1)
     &  + HR3(0,0,1) *HR3(0,1,1)
     &  - 1.2020569031595942d+00*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR3(0,1,-1)*HR3(0,1,-1)
     &  + HR3(0,1, -1)*HR3(0,1,1)
     &  - 1.2020569031595942d+00*HR3(0,1,1)
     &  - 7.2067952087730213d-01*HR4(-1,-1,-1,1)
     &  + 7.2067952087730213d-01*HR4(-1,-1,1,1)
     &  - 7.2067952087730213d-01*HR4(-1,1,1,1)
     &  + 7.2067952087730213d-01*HR4(0,-1,-1,-1)
     &  + 7.2067952087730213d-01*HR4(0,-1,-1,1)
     &  + 7.2067952087730213d-01*HR4(0,-1,1,-1)
     &  + 7.2067952087730213d-01*HR4(0,-1,1,1)
     &  + 7.2067952087730213d-01*HR4(0,1,-1,-1)
     &  + 7.2067952087730213d-01*HR4(0,1,-1,1)
     &  + 7.2067952087730213d-01*HR4(0,1,1,-1)
     &  + 7.2067952087730213d-01*HR4(0,1,1,1)
     &  - 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  + 4.8520302639196171d+00*HR5(-1,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,1,-1,1,1)
     &  - 4.8520302639196171d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,0,1)
     &  - 3.4657359027997265d+00*HR5(0,-1,-1,1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,1,1)
     &  - 3.4657359027997265d+00*HR5(0,-1,1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,0,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,1,1,-1)
     &  - 1.3862943611198906d+00*HR5(0,0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,0,-1,1,-1)
     &  - 2.7725887222397812d+00*HR5(0,0,-1,1,1)
     &  - 1.3862943611198906d+00*HR5(0,0,1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,0,1,-1,1)
     &  - 4.8520302639196171d+00*HR5(0,1,-1,-1,-1)
     &  - 2.7725887222397812d+00*HR5(0,1,-1,-1,1)
     &  - 3.4657359027997265d+00*HR5(0,1,-1,1,-1)
     &  - 1.3862943611198906d+00*HR5(0,1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,0,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,0,1,1)
     &  - 3.4657359027997265d+00*HR5(0,1,1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,1,1,-1,1)
     &  - 2.0794415416798359d+00*HR5(0,1,1,1,-1)
     &  - 1.4000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + HR6( -1,-1,1,-1,1,1)
     &  + 1.4000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  - HR6(0, -1,-1,0,-1,1)
     &  - HR6(0, -1,-1,0,1,1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  - HR6(0, -1,0,-1,1,-1)
     &  - HR6(0, -1,0,-1,1,1)
     &  + HR6(0, -1,0,1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,0,1,1,1)
     &  + 7.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,-1,1,-1)
     &  + HR6(0, -1,1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,0,1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  + HR6(0, -1,1,1,0,1)
     &  + 7.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + HR6(0,0, -1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  + HR6(0,0, -1,-1,1,1)
     &  - HR6(0,0, -1,0,-1,-1)
     &  - HR6(0,0, -1,0,-1,1)
     &  - HR6(0,0, -1,0,1,-1)
     &  - HR6(0,0, -1,0,1,1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,1,1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,-1,1,1,1)
     &  + 7.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  + 5.0000000000000000d+00*HR6(0,0,1,-1,-1,1)
     &  + 5.0000000000000000d+00*HR6(0,0,1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,1,-1,1,1)
     &  - HR6(0,0,1,0, -1,-1)
     &  - HR6(0,0,1,0, -1,1)
     &  - HR6(0,0,1,0,1, -1)
     &  - HR6(0,0,1,0,1,1) 
      HY6(0,0,1,0,1,1) = HY6(0,0,1,0,1,1) ! to fix continuation lines
     &  + 4.0000000000000000d+00*HR6(0,0,1,1,-1,-1)
     &  + 1.4000000000000000d+01*HR6(0,1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,1,1)
     &  - HR6(0,1, -1,0,1,1)
     &  + 7.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,1,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,1,-1,1,1,-1)
     &  - HR6(0,1,0,1, -1,1)
     &  - HR6(0,1,0,1,1, -1)
     &  + 7.0000000000000000d+00*HR6(0,1,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,1,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,1,1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,1,1,1,-1,-1)
      HY6(0,0,1,1,-1,-1) = 
     &  + 9.6741436993644730d-03
     &  - 1.0254618242743082d-01*HR1(-1)
     &  + 3.0168989201460834d-01*HR1(-1)*HR1(-1)
     &  - 7.9222950181883112d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 2.3766885054564933d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.3766885054564933d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 4.7533770109129867d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 6.0337978402921669d-01*HR1(-1)*HR1(1)
     &  - 2.3766885054564933d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 4.7533770109129867d-01*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  - 4.7533770109129867d-01*HR1(-1)*HR2(0,1)
     &  - HR1( -1)*HR2(0,1)*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,0,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,0,1)
     &  - HR1( -1)*HR5(0,-1,-1,1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,1)
     &  + HR1( -1)*HR5(0,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,1)
     &  + HR1( -1)*HR5(0,0,-1,1,-1)
     &  + HR1( -1)*HR5(0,0,1,-1,-1)
     &  + HR1( -1)*HR5(0,1,-1,-1,-1)
     &  + 6.0056626739775178d-02*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 2.3766885054564933d-01*HR1(0)*HR1(1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(0,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR1(0)*HR3(-1,1,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,-1,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,1,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,1,1)
     &  - 1.0254618242743082d-01*HR1(1)
     &  + 3.0168989201460834d-01*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + HR1(1) *HR1(1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 4.7533770109129867d-01*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  - 4.7533770109129867d-01*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - 4.7533770109129867d-01*HR1(1)*HR2(0,1)
     &  - HR1(1) *HR2(0,1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,0,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1(1) *HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,0,1)
     &  - HR1(1) *HR5(0,-1,-1,1,-1)
     &  - HR1(1) *HR5(0,-1,0,-1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,1)
     &  + HR1(1) *HR5(0,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,1)
     &  + HR1(1) *HR5(0,0,-1,1,-1)
     &  + HR1(1) *HR5(0,0,1,-1,-1)
     &  + HR1(1) *HR5(0,1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,1)
     &  + 4.7533770109129867d-01*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - 4.7533770109129867d-01*HR3(-1,1,1)
     &  - HR3( -1,1,1)*HR3(0,-1,-1)
     &  + 4.7533770109129867d-01*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,-1,1)
     &  + HR3(0, -1,-1)*HR3(0,1,-1)
     &  + HR3(0, -1,-1)*HR3(0,1,1)
     &  + 4.7533770109129867d-01*HR3(0,-1,1)
     &  + 4.7533770109129867d-01*HR3(0,1,-1)
     &  + 4.7533770109129867d-01*HR3(0,1,1)
     &  + 4.8045301391820142d-01*HR4(-1,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(-1,-1,1,1)
     &  - 4.8045301391820142d-01*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,1,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,0,1,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,1,1)
     &  - 4.8045301391820142d-01*HR4(0,1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,1,1,-1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,1,1)
     &  + 1.3862943611198906d+00*HR5(0,0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,1,-1)
     &  + 2.0794415416798359d+00*HR5(0,1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - HR6( -1,-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  - HR6(0, -1,-1,0,-1,1)
     &  - HR6(0, -1,-1,0,1,-1)
     &  - HR6(0, -1,-1,0,1,1)
     &  + HR6(0, -1,-1,1,-1,-1)
     &  + HR6(0, -1,-1,1,-1,1)
     &  + HR6(0, -1,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,1,-1)
     &  - HR6(0, -1,0,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,1,-1,-1)
     &  - HR6(0, -1,0,1,-1,1)
     &  - HR6(0, -1,0,1,1,-1)
     &  - HR6(0, -1,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  - HR6(0,0, -1,-1,1,1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  - HR6(0,0, -1,1,-1,1)
     &  - HR6(0,0, -1,1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  - HR6(0,0,1, -1,-1,1)
     &  - HR6(0,0,1, -1,1,-1)
     &  - HR6(0,0,1,1, -1,-1)
     &  - 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  - HR6(0,1, -1,-1,-1,1)
     &  - HR6(0,1, -1,-1,1,-1)
     &  - HR6(0,1, -1,1,-1,-1)
     &  - HR6(0,1,1, -1,-1,-1)
      HY6(0,0,1,1,-1,1) = 
     &  + 1.4158547826406217d-02
     &  - 1.7692012816527167d-01*HR1(-1)
     &  + 1.2185212069612250d+00*HR1(-1)*HR1(-1)
     &  - 2.4633412775717367d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 7.3900238327152102d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.4556013161625312d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  - 7.3900238327152102d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.4556013161625312d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.4780047665430420d+00*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR2(0,-1)*HR2(0,1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(0,-1,0,1)
     &  + HR1( -1)*HR1(0)*HR4(0,0,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(0,0,1,-1)
     &  + HR1( -1)*HR1(0)*HR4(0,1,-1,-1)
     &  + 2.4370424139224501d+00*HR1(-1)*HR1(1)
     &  - 7.3900238327152102d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,0,-1)
     &  - 1.4780047665430420d+00*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,1,-1)
     &  - 1.4780047665430420d+00*HR1(-1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,1)*HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(-1,1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,0,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,0,1)
     &  + HR1( -1)*HR5(0,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,1)
     &  - HR1( -1)*HR5(0,-1,0,1,-1)
     &  + 9.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - HR1( -1)*HR5(0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,0,1)
     &  - HR1( -1)*HR5(0,0,-1,1,-1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,0,1,-1)
     &  - HR1( -1)*HR5(0,0,1,-1,-1)
     &  - HR1( -1)*HR5(0,0,1,0,-1)
     &  - HR1( -1)*HR5(0,1,-1,-1,-1)
     &  + 1.4556013161625312d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 7.3900238327152102d-01*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  + HR1(0) *HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.8224052646501250d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(0) *HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - 5.8224052646501250d-01*HR1(0)*HR1(1)*HR2(0,1)
     &  - HR1(0) *HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,-1,1)
     &  + HR1(0) *HR1(1)*HR4(0,-1,0,1)
     &  + HR1(0) *HR1(1)*HR4(0,0,-1,1)
     &  + HR1(0) *HR1(1)*HR4(0,0,1,-1)
     &  + HR1(0) *HR1(1)*HR4(0,1,-1,-1)
     &  + HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1(0) *HR2(0,-1)*HR3(-1,1,1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,-1,1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,1,-1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,1,1)
     &  + 5.8224052646501250d-01*HR1(0)*HR3(-1,-1,1)
     &  - 5.8224052646501250d-01*HR1(0)*HR3(-1,1,1)
     &  + 5.8224052646501250d-01*HR1(0)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(0)*HR3(0,-1,1)
     &  + 5.8224052646501250d-01*HR1(0)*HR3(0,1,-1)
     &  + 5.8224052646501250d-01*HR1(0)*HR3(0,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + HR1(0) *HR5(-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + HR1(0) *HR5(0,-1,-1,1,1)
     &  - HR1(0) *HR5(0,-1,0,-1,-1)
     &  - HR1(0) *HR5(0,-1,0,-1,1)
     &  - HR1(0) *HR5(0,-1,0,1,-1)
     &  - HR1(0) *HR5(0,-1,0,1,1)
     &  - HR1(0) *HR5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,0,-1,1,-1)
     &  - HR1(0) *HR5(0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,0,1,-1,-1)
     &  - HR1(0) *HR5(0,0,1,-1,1)
     &  - HR1(0) *HR5(0,0,1,1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,-1)
     &  - HR1(0) *HR5(0,1,-1,-1,1)
     &  - HR1(0) *HR5(0,1,-1,1,-1)
     &  - HR1(0) *HR5(0,1,1,-1,-1)
     &  - 1.7692012816527167d-01*HR1(1)
     &  + 1.2185212069612250d+00*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,0,0,-1)
     &  + 1.4780047665430420d+00*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,0,-1)
     &  - 1.4780047665430420d+00*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,0,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,1,-1)
     &  - 1.4780047665430420d+00*HR1(1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(0,0,1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,0,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1(1) *HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,0,1)
     &  + HR1(1) *HR5(0,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,1)
     &  - HR1(1) *HR5(0,-1,0,1,-1)
     &  + 9.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  - HR1(1) *HR5(0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,-1,0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,0,1)
     &  - HR1(1) *HR5(0,0,-1,1,-1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,0,1,-1)
     &  - HR1(1) *HR5(0,0,1,-1,-1)
     &  - HR1(1) *HR5(0,0,1,0,-1)
     &  - HR1(1) *HR5(0,1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  + HR2(0, -1)*HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  + HR2(0, -1)*HR4(0,-1,-1,1)
     &  + HR2(0, -1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,1,-1,-1)
     &  + HR2(0, -1)*HR4(0,1,-1,1)
     &  + HR2(0, -1)*HR4(0,1,1,-1)
     &  + 1.4780047665430420d+00*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,-1)
     &  - 1.4780047665430420d+00*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,0,-1)
     &  + 1.4780047665430420d+00*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,1,1)
     &  + 1.4780047665430420d+00*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR3(0,-1,1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR3(0,0,-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR3(0,0,-1)*HR3(0,1,1)
     &  + 1.4780047665430420d+00*HR3(0,1,-1)
     &  + 1.4780047665430420d+00*HR3(0,1,1)
     &  + 1.1644810529300250d+00*HR4(-1,-1,-1,1)
     &  - 5.8224052646501250d-01*HR4(-1,-1,1,1)
     &  - 1.1644810529300250d+00*HR4(0,-1,-1,-1)
     &  - 5.8224052646501250d-01*HR4(0,-1,-1,1)
     &  - 5.8224052646501250d-01*HR4(0,-1,1,-1)
     &  - 5.8224052646501250d-01*HR4(0,0,-1,-1)
     &  - 5.8224052646501250d-01*HR4(0,0,-1,1)
     &  - 5.8224052646501250d-01*HR4(0,0,1,-1)
     &  - 5.8224052646501250d-01*HR4(0,0,1,1)
     &  - 1.1644810529300250d+00*HR4(0,1,-1,-1)
     &  - 5.8224052646501250d-01*HR4(0,1,-1,1)
     &  - 5.8224052646501250d-01*HR4(0,1,1,-1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,1,1)
     &  - 1.3862943611198906d+00*HR5(0,0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,1,-1)
     &  - 2.0794415416798359d+00*HR5(0,1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + HR6( -1,-1,-1,-1,1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,1,1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  - HR6(0, -1,-1,1,-1,1)
     &  - HR6(0, -1,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,1,-1,-1)
     &  + HR6(0, -1,0,1,-1,1)
     &  + HR6(0, -1,0,1,1,-1)
     &  + HR6(0, -1,1,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  + HR6(0,0, -1,-1,0,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  + HR6(0,0, -1,-1,1,1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,0,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,0,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,0,1,1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  + HR6(0,0, -1,1,-1,1)
     &  + HR6(0,0, -1,1,0,-1)
     &  + HR6(0,0, -1,1,1,-1)
     &  + 9.0000000000000000d+00*HR6(0,0,0,-1,-1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,0,-1,-1,1)
     &  + 6.0000000000000000d+00*HR6(0,0,0,-1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,0,-1,1,1)
     &  + 6.0000000000000000d+00*HR6(0,0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,0,1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,0,0,1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  + HR6(0,0,1, -1,-1,1)
     &  + HR6(0,0,1, -1,0,-1)
     &  + HR6(0,0,1, -1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,0,-1,-1)
     &  + HR6(0,0,1,0, -1,1)
     &  + HR6(0,0,1,0,1, -1)
     &  + HR6(0,0,1,1, -1,-1)
     &  + HR6(0,0,1,1,0, -1)
     &  + 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  + HR6(0,1, -1,-1,-1,1)
     &  + HR6(0,1, -1,-1,1,-1)
     &  + HR6(0,1, -1,1,-1,-1)
     &  + HR6(0,1,1, -1,-1,-1)
      HY6(0,0,1,1,0,-1) = 
     &  + 6.2411323447021162d-02
     &  - 4.8796123154736019d-01*HR1(-1)
     &  + 7.1933735509666758d-01*HR1(-1)*HR1(-1)
     &  - 2.2022937829936213d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 3.4269459726004717d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.3707783890401886d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.3707783890401886d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 6.6068813489808640d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 6.6068813489808640d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.3213762697961728d+00*HR1(-1)*HR1(0)*HR1(1)
     &  - 4.1123351671205660d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 1.4386747101933351d+00*HR1(-1)*HR1(1)
     &  - 6.6068813489808640d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(-1,1,1)
     &  - 1.3213762697961728d+00*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  - 1.3213762697961728d+00*HR1(-1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(0,1)*HR2(0,1)
     &  - HR1( -1)*HR2(0,1)*HR3(0,-1,-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR3(-1,1,1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR3(0,0,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR3(0,0,1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,1,1)
     &  - HR1( -1)*HR5(-1,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,1,1)
     &  - HR1( -1)*HR5(-1,1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,0,1)
     &  - HR1( -1)*HR5(0,-1,0,-1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,1)
     &  - HR1( -1)*HR5(0,0,1,-1,-1)
     &  - HR1( -1)*HR5(0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,1)
     &  - HR1( -1)*HR5(0,1,0,1,-1)
     &  + 2.0561675835602830d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 6.6068813489808640d-01*HR1(0)*HR1(1)*HR1(1)
     &  + 8.2246703342411321d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 8.2246703342411321d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 8.2246703342411321d-01*HR1(0)*HR1(1)*HR2(0,1)
     &  + 8.2246703342411321d-01*HR1(0)*HR3(-1,-1,1)
     &  - 8.2246703342411321d-01*HR1(0)*HR3(-1,1,1)
     &  + 8.2246703342411321d-01*HR1(0)*HR3(0,-1,-1)
     &  + 8.2246703342411321d-01*HR1(0)*HR3(0,-1,1)
     &  + 8.2246703342411321d-01*HR1(0)*HR3(0,1,-1)
     &  + 8.2246703342411321d-01*HR1(0)*HR3(0,1,1)
     &  - 4.8796123154736019d-01*HR1(1)
     &  + 7.1933735509666758d-01*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,0,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1(1) *HR1(1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,1,-1)
     &  + HR1(1) *HR1(1)*HR4(0,1,-1,-1)
     &  + 1.3213762697961728d+00*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(-1,-1,1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,1,-1)
     &  - 1.3213762697961728d+00*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - 1.3213762697961728d+00*HR1(1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(0,1)*HR2(0,1)
     &  - HR1(1) *HR2(0,1)*HR3(0,-1,-1)
     &  + 8.2246703342411321d-01*HR1(1)*HR3(-1,-1,1)
     &  + 8.2246703342411321d-01*HR1(1)*HR3(0,-1,-1)
     &  + 8.2246703342411321d-01*HR1(1)*HR3(0,0,-1)
     &  + 8.2246703342411321d-01*HR1(1)*HR3(0,0,1)
     &  + 8.2246703342411321d-01*HR1(1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,0,1)
     &  - HR1(1) *HR5(0,-1,0,-1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,1)
     &  - HR1(1) *HR5(0,0,1,-1,-1)
     &  - HR1(1) *HR5(0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,1)
     &  - HR1(1) *HR5(0,1,0,1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(0,1,1)
     &  - HR2(0,1) *HR4(0,-1,1,1)
     &  + 1.3213762697961728d+00*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(-1,1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + HR3( -1,-1,1)*HR3(0,1,-1)
     &  - 1.3213762697961728d+00*HR3(-1,1,1)
     &  - HR3( -1,1,1)*HR3(0,-1,-1)
     &  - HR3( -1,1,1)*HR3(0,1,-1)
     &  + 1.3213762697961728d+00*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,-1,1)
     &  + HR3(0, -1,-1)*HR3(0,1,-1)
     &  + HR3(0, -1,-1)*HR3(0,1,1)
     &  + 1.3213762697961728d+00*HR3(0,-1,1)
     &  + 1.3213762697961728d+00*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR3(0,1,-1)*HR3(0,1,-1)
     &  + HR3(0,1, -1)*HR3(0,1,1)
     &  + 1.3213762697961728d+00*HR3(0,1,1)
     &  + 1.6449340668482264d+00*HR4(-1,-1,-1,1)
     &  - 8.2246703342411321d-01*HR4(-1,-1,1,1)
     &  - 1.6449340668482264d+00*HR4(0,-1,-1,-1)
     &  - 8.2246703342411321d-01*HR4(0,-1,-1,1)
     &  - 8.2246703342411321d-01*HR4(0,-1,1,-1)
     &  - 8.2246703342411321d-01*HR4(0,0,-1,-1)
     &  - 8.2246703342411321d-01*HR4(0,0,-1,1)
     &  - 8.2246703342411321d-01*HR4(0,0,1,-1)
     &  - 8.2246703342411321d-01*HR4(0,0,1,1)
     &  - 1.6449340668482264d+00*HR4(0,1,-1,-1)
     &  - 8.2246703342411321d-01*HR4(0,1,-1,1)
     &  - 8.2246703342411321d-01*HR4(0,1,1,-1)
     &  + 1.3862943611198906d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,1,1,1)
     &  + 6.9314718055994530d-01*HR5(-1,1,-1,1,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,0,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,0,1)
     &  + 2.0794415416798359d+00*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,0,1,1,-1)
     &  + 2.0794415416798359d+00*HR5(0,0,1,1,1)
     &  + 1.3862943611198906d+00*HR5(0,1,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,0,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,0,1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,1,-1,1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  - 5.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  + 9.0000000000000000d+00*HR6(-1,-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HR6(-1,-1,1,-1,1,1)
     &  + HR6( -1,-1,1,1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  - HR6(0, -1,-1,0,-1,1)
     &  - HR6(0, -1,-1,0,1,1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  - HR6(0, -1,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  - HR6(0, -1,0,-1,1,-1)
     &  - HR6(0, -1,0,-1,1,1)
     &  + HR6(0, -1,0,1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,0,1,1,1)
     &  - HR6(0, -1,1,-1,-1,-1)
     &  - HR6(0, -1,1,-1,1,-1)
     &  + HR6(0, -1,1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,0,1,1)
     &  + HR6(0, -1,1,1,0,1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  - HR6(0,0, -1,-1,1,1)
     &  - HR6(0,0, -1,1,-1,-1)
     &  + HR6(0,0, -1,1,1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,-1,1,1,1)
     &  + HR6(0,0,1, -1,-1,1)
     &  + HR6(0,0,1, -1,1,1)
     &  - HR6(0,0,1,1, -1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,1,1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,0,1,1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,1,-1,-1,1,1)
     &  - HR6(0,1, -1,0,1,1)
     &  - HR6(0,1, -1,1,-1,-1)
     &  - HR6(0,1, -1,1,1,-1)
     &  - HR6(0,1,0,1, -1,1)
     &  - HR6(0,1,0,1,1, -1)
     &  - HR6(0,1,1, -1,-1,-1)
     &  - HR6(0,1,1, -1,1,-1)
      HY6(0,0,1,1,0,1) = 
     &  + 7.9221397565207165d-02
     &  - 7.1156619755057243d-01*HR1(-1)
     &  + 2.6542655863928879d+00*HR1(-1)*HR1(-1)
     &  - 5.9071586949367355d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 6.8538919452009434d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 1.7721476084810206d+00*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  - 1.7721476084810206d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,0,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 3.5442952169620413d+00*HR1(-1)*HR1(0)*HR1(1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,0,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR2(0,-1)*HR2(0,1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(0,1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,1,1)
     &  + 5.3085311727857758d+00*HR1(-1)*HR1(1)
     &  - 1.7721476084810206d+00*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(1)*HR1(1)*HR3(0,0,1)
     &  + HR1( -1)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,0,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(-1,1,1)
     &  - 3.5442952169620413d+00*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,0,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,1,-1)
     &  - 3.5442952169620413d+00*HR1(-1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(0,1)*HR2(0,1)
     &  + HR1( -1)*HR2(0,1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,1)*HR3(0,0,1)
     &  - HR1( -1)*HR2(0,1)*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(-1,1,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(0,0,-1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(0,0,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,1,1)
     &  + HR1( -1)*HR5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,1,1)
     &  + HR1( -1)*HR5(-1,1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,0,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,1)
     &  + 9.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,0,1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,-1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,0,0,1,-1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,0,0,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,1,0,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,1,0,1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(0,0,1,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,1,0,1,-1)
     &  + 4.1123351671205660d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.7721476084810206d+00*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,0,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 1.6449340668482264d+00*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1(0) *HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1(0) *HR1(1)*HR2(-1,1)*HR2(0,1)
     &  - 1.6449340668482264d+00*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(0) *HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - 1.6449340668482264d+00*HR1(0)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(0,1)*HR2(0,1)
     &  + HR1(0) *HR2(-1,1)*HR3(-1,1,1)
     &  + HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1(0) *HR2(0,-1)*HR3(-1,1,1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,-1,1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,1,-1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,1,1)
     &  + HR1(0) *HR2(0,1)*HR3(-1,-1,1)
     &  - HR1(0) *HR2(0,1)*HR3(-1,1,1)
     &  + HR1(0) *HR2(0,1)*HR3(0,1,-1)
     &  + HR1(0) *HR2(0,1)*HR3(0,1,1)
     &  + 1.6449340668482264d+00*HR1(0)*HR3(-1,-1,1)
     &  - 1.6449340668482264d+00*HR1(0)*HR3(-1,1,1)
     &  + 1.6449340668482264d+00*HR1(0)*HR3(0,-1,-1)
     &  + 1.6449340668482264d+00*HR1(0)*HR3(0,-1,1)
     &  + 1.6449340668482264d+00*HR1(0)*HR3(0,1,-1)
     &  + 1.6449340668482264d+00*HR1(0)*HR3(0,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - HR1(0) *HR5(-1,-1,-1,1,1)
     &  - HR1(0) *HR5(-1,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,1,1,1)
     &  - HR1(0) *HR5(-1,1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  + HR1(0) *HR5(0,-1,-1,0,1)
     &  - HR1(0) *HR5(0,-1,-1,1,-1)
     &  - HR1(0) *HR5(0,-1,-1,1,1)
     &  - HR1(0) *HR5(0,-1,0,-1,-1)
     &  + HR1(0) *HR5(0,-1,0,1,1)
     &  - HR1(0) *HR5(0,-1,1,-1,-1)
     &  - HR1(0) *HR5(0,-1,1,-1,1)
     &  + HR1(0) *HR5(0,-1,1,0,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,-1)
     &  - HR1(0) *HR5(0,0,-1,-1,1)
     &  - HR1(0) *HR5(0,0,-1,1,-1)
     &  + HR1(0) *HR5(0,0,-1,1,1)
     &  - HR1(0) *HR5(0,0,1,-1,-1)
     &  - HR1(0) *HR5(0,0,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,0,1,1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,0,1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,1)
     &  - HR1(0) *HR5(0,1,-1,1,-1)
     &  - HR1(0) *HR5(0,1,-1,1,1)
     &  - HR1(0) *HR5(0,1,0,1,-1)
     &  - HR1(0) *HR5(0,1,0,1,1)
     &  - HR1(0) *HR5(0,1,1,-1,-1)
     &  - HR1(0) *HR5(0,1,1,-1,1)
     &  - 7.1156619755057243d-01*HR1(1)
     &  + 2.6542655863928879d+00*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,0,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,0,0,-1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,0,0,1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,0,1,-1)
     &  - 1.5000000000000000d+00*HR1(1)*HR1(1)*HR4(0,1,-1,-1)
     &  + 3.5442952169620413d+00*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,0,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,1,-1)
     &  - 3.5442952169620413d+00*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,0,1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,1,-1)
     &  - 3.5442952169620413d+00*HR1(1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(0,1)*HR2(0,1)
     &  - HR1(1) *HR2(0,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,1)*HR3(0,0,1)
     &  - HR1(1) *HR2(0,1)*HR3(0,1,-1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(0,-1,-1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(0,0,-1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(0,0,1)
     &  + 1.6449340668482264d+00*HR1(1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,0,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,1)
     &  + 9.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,-1,0,1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,-1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(0,0,0,1,-1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(0,0,0,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,1,0,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,1,0,1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(0,0,1,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,1,0,1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  + HR2(0, -1)*HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  + HR2(0, -1)*HR4(0,-1,-1,1)
     &  + HR2(0, -1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,1,-1,-1)
     &  + HR2(0, -1)*HR4(0,1,-1,1)
     &  + HR2(0, -1)*HR4(0,1,1,-1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(0,1,1)
     &  - 2.0000000000000000d+00*HR2(0,1)*HR4(-1,-1,-1,1)
     &  + HR2(0,1) *HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR2(0,1)*HR4(0,-1,1,1)
     &  + 2.0000000000000000d+00*HR2(0,1)*HR4(0,1,-1,-1)
     &  + HR2(0,1) *HR4(0,1,-1,1)
     &  + HR2(0,1) *HR4(0,1,1,-1)
     &  + 3.5442952169620413d+00*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,1,-1)
     &  - 3.5442952169620413d+00*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,0,1)
     &  + 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,1,-1)
     &  + 3.5442952169620413d+00*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,0,1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,1,1)
     &  + 3.5442952169620413d+00*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR3(0,-1,1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR3(0,-1,1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,1)*HR3(0,0,1)
     &  - 2.0000000000000000d+00*HR3(0,0,-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR3(0,0,-1)*HR3(0,1,1)
     &  - 2.0000000000000000d+00*HR3(0,0,1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR3(0,0,1)*HR3(0,1,1)
     &  + 3.5442952169620413d+00*HR3(0,1,-1)
     &  - HR3(0,1, -1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR3(0,1,-1)*HR3(0,1,1)
     &  + 3.5442952169620413d+00*HR3(0,1,1)
     &  + 3.2898681336964528d+00*HR4(-1,-1,-1,1)
     &  - 1.6449340668482264d+00*HR4(-1,-1,1,1)
     &  - 3.2898681336964528d+00*HR4(0,-1,-1,-1)
     &  - 1.6449340668482264d+00*HR4(0,-1,-1,1)
     &  - 1.6449340668482264d+00*HR4(0,-1,1,-1)
     &  - 1.6449340668482264d+00*HR4(0,0,-1,-1)
     &  - 1.6449340668482264d+00*HR4(0,0,-1,1)
     &  - 1.6449340668482264d+00*HR4(0,0,1,-1)
     &  - 1.6449340668482264d+00*HR4(0,0,1,1)
     &  - 3.2898681336964528d+00*HR4(0,1,-1,-1)
     &  - 1.6449340668482264d+00*HR4(0,1,-1,1)
     &  - 1.6449340668482264d+00*HR4(0,1,1,-1)
     &  - 1.3862943611198906d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,1,1,1)
     &  - 6.9314718055994530d-01*HR5(-1,1,-1,1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,0,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,0,1)
     &  - 2.0794415416798359d+00*HR5(0,0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,-1,1)
     &  - 2.0794415416798359d+00*HR5(0,0,1,1,-1)
     &  - 2.0794415416798359d+00*HR5(0,0,1,1,1)
     &  - 1.3862943611198906d+00*HR5(0,1,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,0,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,0,1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,1,-1,1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  + 5.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  - 9.0000000000000000d+00*HR6(-1,-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HR6(-1,-1,1,-1,1,1)
     &  - HR6( -1,-1,1,1,-1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,0,1)
     &  - HR6(0, -1,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,0,1,1)
     &  + HR6(0, -1,-1,1,-1,-1)
     &  + HR6(0, -1,-1,1,0,1)
     &  + HR6(0, -1,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  + 5.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,1,1)
     &  - HR6(0, -1,0,1,1,-1)
     &  - 6.0000000000000000d+00*HR6(0,-1,0,1,1,1)
     &  + HR6(0, -1,1,-1,-1,-1)
     &  + HR6(0, -1,1,-1,0,1)
     &  + HR6(0, -1,1,-1,1,-1)
     &  - HR6(0, -1,1,0,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,-1,1,0,1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,1,0,1)
     &  - 6.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  + HR6(0,0, -1,-1,0,-1)
     &  + HR6(0,0, -1,-1,0,1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  + HR6(0,0, -1,-1,1,1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,0,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,0,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,0,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,0,1,1)
     &  + HR6(0,0, -1,1,-1,-1)
     &  + HR6(0,0, -1,1,0,-1)
     &  + HR6(0,0, -1,1,0,1)
     &  - HR6(0,0, -1,1,1,-1)
     &  - 1.2000000000000000d+01*HR6(0,0,-1,1,1,1)
     &  + 9.0000000000000000d+00*HR6(0,0,0,-1,-1,-1)
     &  + 9.0000000000000000d+00*HR6(0,0,0,-1,-1,1)
     &  + 9.0000000000000000d+00*HR6(0,0,0,-1,1,-1)
     &  + 9.0000000000000000d+00*HR6(0,0,0,-1,1,1)
     &  + 9.0000000000000000d+00*HR6(0,0,0,1,-1,-1)
     &  + 9.0000000000000000d+00*HR6(0,0,0,1,-1,1)
     &  + 9.0000000000000000d+00*HR6(0,0,0,1,1,-1)
     &  + 9.0000000000000000d+00*HR6(0,0,0,1,1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,1,-1,-1,1)
     &  + HR6(0,0,1, -1,0,-1)
     &  + HR6(0,0,1, -1,0,1)
     &  - 2.0000000000000000d+00*HR6(0,0,1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,1,-1,1,1)
     &  + 4.0000000000000000d+00*HR6(0,0,1,0,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,0,1,0,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,0,1,0,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,0,1,0,1,1)
     &  - 3.0000000000000000d+00*HR6(0,0,1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,1,-1,1)
     &  + HR6(0,0,1,1,0, -1)
     &  + HR6(0,0,1,1,0,1) 
     &  + 3.0000000000000000d+00*HR6(0,0,1,1,1,-1)
     &  - HR6(0,1, -1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  - HR6(0,1, -1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,1,-1,0,1,1)
     &  + HR6(0,1, -1,1,-1,-1)
     &  + HR6(0,1, -1,1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,1,0,1,-1,-1)
     &  + HR6(0,1,0,1, -1,1)
     &  + HR6(0,1,0,1,1, -1)
     &  + HR6(0,1,1, -1,-1,-1)
     &  + HR6(0,1,1, -1,1,-1)
      HY6(0,0,1,1,1,-1) = 
     &  + 2.3218887429959911d-02
     &  - 4.0707197178331534d-01*HR1(-1)
     &  - 5.3409449824807409d-01*HR1(-1)*HR1(-1)
     &  + 1.8455064982157776d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.5365194946473328d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.6567338509580348d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.5365194946473328d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.6567338509580348d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,1)
     &  - 1.1073038989294665d+00*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  - 1.0681889964961481d+00*HR1(-1)*HR1(1)
     &  + 5.5365194946473328d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,0,-1)
     &  + 1.1073038989294665d+00*HR1(-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,0,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,1,-1)
     &  + 1.1073038989294665d+00*HR1(-1)*HR2(0,1)
     &  - HR1( -1)*HR2(0,1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(0,1)*HR3(0,0,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(-1,1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(0,0,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(0,0,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,0,-1)
     &  + HR1( -1)*HR5(0,0,-1,0,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,0,1,-1)
     &  - HR1( -1)*HR5(0,0,1,-1,-1)
     &  + HR1( -1)*HR5(0,0,1,0,-1)
     &  - HR1( -1)*HR5(0,1,-1,-1,-1)
     &  - 5.7762265046662109d-02*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 2.6567338509580348d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,1,1)
     &  - 5.5365194946473328d-01*HR1(0)*HR1(1)*HR1(1)
     &  - 1.0626935403832139d+00*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 1.0626935403832139d+00*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 1.0626935403832139d+00*HR1(0)*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 1.0626935403832139d+00*HR1(0)*HR3(-1,-1,1)
     &  + 1.0626935403832139d+00*HR1(0)*HR3(-1,1,1)
     &  - 1.0626935403832139d+00*HR1(0)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(0)*HR3(0,-1,1)
     &  - 1.0626935403832139d+00*HR1(0)*HR3(0,1,-1)
     &  - 1.0626935403832139d+00*HR1(0)*HR3(0,1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,0,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,0,1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,1,1,-1)
     &  - 4.0707197178331534d-01*HR1(1)
     &  - 5.3409449824807409d-01*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR4(0,0,0,-1)
     &  - 1.1073038989294665d+00*HR1(1)*HR2(-1,1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(-1,1)*HR3(0,0,-1)
     &  + 1.1073038989294665d+00*HR1(1)*HR2(0,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,0,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,1,-1)
     &  + 1.1073038989294665d+00*HR1(1)*HR2(0,1)
     &  - HR1(1) *HR2(0,1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(0,1)*HR3(0,0,-1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(0,0,-1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(0,0,1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,0,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1(1) *HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - HR1(1) *HR5(0,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,0,-1)
     &  + HR1(1) *HR5(0,0,-1,0,1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,0,1,-1)
     &  - HR1(1) *HR5(0,0,1,-1,-1)
     &  + HR1(1) *HR5(0,0,1,0,-1)
     &  - HR1(1) *HR5(0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  - HR2(0, -1)*HR4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  - HR2(0, -1)*HR4(0,-1,-1,1)
     &  - HR2(0, -1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,1,-1,-1)
     &  - HR2(0, -1)*HR4(0,1,-1,1)
     &  - HR2(0, -1)*HR4(0,1,1,-1)
     &  - 1.1073038989294665d+00*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + HR3( -1,-1,1)*HR3(0,0,-1)
     &  + 1.1073038989294665d+00*HR3(-1,1,1)
     &  - HR3( -1,1,1)*HR3(0,-1,-1)
     &  - HR3( -1,1,1)*HR3(0,0,-1)
     &  - 1.1073038989294665d+00*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,-1,1)
     &  + HR3(0, -1,-1)*HR3(0,0,-1)
     &  + HR3(0, -1,-1)*HR3(0,1,-1)
     &  + HR3(0, -1,-1)*HR3(0,1,1)
     &  - 1.1073038989294665d+00*HR3(0,-1,1)
     &  + HR3(0, -1,1)*HR3(0,0,-1)
     &  + HR3(0,0, -1)*HR3(0,1,-1)
     &  + HR3(0,0, -1)*HR3(0,1,1)
     &  - 1.1073038989294665d+00*HR3(0,1,-1)
     &  - 1.1073038989294665d+00*HR3(0,1,1)
     &  - 2.1253870807664278d+00*HR4(-1,-1,-1,1)
     &  + 1.0626935403832139d+00*HR4(-1,-1,1,1)
     &  + 2.1253870807664278d+00*HR4(0,-1,-1,-1)
     &  + 1.0626935403832139d+00*HR4(0,-1,-1,1)
     &  + 1.0626935403832139d+00*HR4(0,-1,1,-1)
     &  + 1.0626935403832139d+00*HR4(0,0,-1,-1)
     &  + 1.0626935403832139d+00*HR4(0,0,-1,1)
     &  + 1.0626935403832139d+00*HR4(0,0,1,-1)
     &  + 1.0626935403832139d+00*HR4(0,0,1,1)
     &  + 2.1253870807664278d+00*HR4(0,1,-1,-1)
     &  + 1.0626935403832139d+00*HR4(0,1,-1,1)
     &  + 1.0626935403832139d+00*HR4(0,1,1,-1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,0,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,0,0,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,0,1,1)
     &  - 1.3862943611198906d+00*HR5(0,0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,1,-1)
     &  - 2.0794415416798359d+00*HR5(0,1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  + HR6( -1,-1,-1,-1,1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + HR6(0, -1,-1,-1,1,1)
     &  - HR6(0, -1,-1,0,-1,1)
     &  - HR6(0, -1,-1,0,1,-1)
     &  - HR6(0, -1,-1,0,1,1)
     &  + HR6(0, -1,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  - HR6(0, -1,0,-1,-1,1)
     &  - HR6(0, -1,0,-1,1,-1)
     &  - HR6(0, -1,0,-1,1,1)
     &  + HR6(0, -1,1,-1,-1,-1)
     &  + 8.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  - HR6(0,0, -1,-1,0,-1)
     &  - HR6(0,0, -1,-1,1,1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,0,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,0,1,-1)
     &  - HR6(0,0, -1,0,1,1)
     &  + HR6(0,0, -1,1,-1,-1)
     &  - HR6(0,0, -1,1,0,-1)
     &  - 6.0000000000000000d+00*HR6(0,0,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,0,-1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,0,-1,1,1)
     &  - 4.0000000000000000d+00*HR6(0,0,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,0,1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  + HR6(0,0,1, -1,-1,1)
     &  - HR6(0,0,1, -1,0,-1)
     &  + HR6(0,0,1, -1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,1,0,-1,-1)
     &  - HR6(0,0,1,0, -1,1)
     &  - HR6(0,0,1,0,1, -1)
     &  + HR6(0,0,1,1, -1,-1)
     &  - HR6(0,0,1,1,0, -1)
     &  + 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  + HR6(0,1, -1,-1,-1,1)
     &  + HR6(0,1, -1,-1,1,-1)
     &  + HR6(0,1, -1,1,-1,-1)
     &  + HR6(0,1,1, -1,-1,-1)
      HY6(0,0,1,1,1,1) = 
     &  + 4.0536897271519737d-02
     &  - 1.0369277551433699d+00*HR1(-1)
     &  + 4.8090645538142385d-03*HR1(-1)*HR1(-1)
     &  - 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(0)
     &  + 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.2500000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  + 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR2(0,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,0,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,0,1)
     &  - HR1( -1)*HR1(0)*HR4(0,0,1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,1,-1,-1)
     &  + 9.6181291076284771d-03*HR1(-1)*HR1(1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 5.5504108664821579d-02*HR1(-1)*HR2(0,-1)
     &  - 5.5504108664821579d-02*HR1(-1)*HR2(0,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,0,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,1,1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,0,-1,-1)
     &  + HR1( -1)*HR5(0,0,0,0,-1)
     &  + HR1( -1)*HR5(0,0,0,0,1)
     &  + HR1( -1)*HR5(0,0,0,1,-1)
     &  + HR1( -1)*HR5(0,0,1,-1,-1)
     &  + HR1( -1)*HR5(0,1,-1,-1,-1)
     &  + 2.0833333333333333d-02*HR1(0)*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.7762265046662109d-02*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,1,1)
     &  + 6.0056626739775178d-02*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,0,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,1,1)
     &  + HR1(0) *HR1(0)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(-1,-1,1,1)
     &  - HR1(0) *HR1(0)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,0,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,0,1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,0,1,1)
     &  - HR1(0) *HR1(0)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,1,1,-1)
     &  + 2.7752054332410789d-02*HR1(0)*HR1(1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + HR1(0) *HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,0,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,0,0,-1)
     &  - HR1(0) *HR1(1)*HR4(0,0,0,1)
     &  - HR1(0) *HR1(1)*HR4(0,0,1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,1,-1,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR1(0)*HR3(-1,1,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,-1,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,1,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,0,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,0,1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,1,1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - HR1(0) *HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + HR1(0) *HR5(0,-1,-1,-1,1)
     &  + HR1(0) *HR5(0,-1,-1,1,-1)
     &  + HR1(0) *HR5(0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,-1)
     &  + HR1(0) *HR5(0,0,-1,-1,1)
     &  + HR1(0) *HR5(0,0,-1,1,-1)
     &  + HR1(0) *HR5(0,0,0,-1,-1)
     &  + HR1(0) *HR5(0,0,0,-1,1)
     &  + HR1(0) *HR5(0,0,0,1,-1)
     &  + HR1(0) *HR5(0,0,0,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,0,1,-1,-1)
     &  + HR1(0) *HR5(0,0,1,-1,1)
     &  + HR1(0) *HR5(0,0,1,1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,-1)
     &  + HR1(0) *HR5(0,1,-1,-1,1)
     &  + HR1(0) *HR5(0,1,-1,1,-1)
     &  + HR1(0) *HR5(0,1,1,-1,-1)
     &  - 1.0369277551433699d+00*HR1(1)
     &  + 4.8090645538142385d-03*HR1(1)*HR1(1)
     &  + 5.5504108664821579d-02*HR1(1)*HR2(-1,1)
     &  - 5.5504108664821579d-02*HR1(1)*HR2(0,-1)
     &  - 5.5504108664821579d-02*HR1(1)*HR2(0,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,0,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,0,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1(1) *HR5(-1,-1,-1,-1,1)
     &  + HR1(1) *HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,0,-1,-1)
     &  + HR1(1) *HR5(0,0,0,0,-1)
     &  + HR1(1) *HR5(0,0,0,0,1)
     &  + HR1(1) *HR5(0,0,0,1,-1)
     &  + HR1(1) *HR5(0,0,1,-1,-1)
     &  + HR1(1) *HR5(0,1,-1,-1,-1)
     &  + 5.5504108664821579d-02*HR3(-1,-1,1)
     &  - 5.5504108664821579d-02*HR3(-1,1,1)
     &  + 5.5504108664821579d-02*HR3(0,-1,-1)
     &  + 5.5504108664821579d-02*HR3(0,-1,1)
     &  + 5.5504108664821579d-02*HR3(0,1,-1)
     &  + 5.5504108664821579d-02*HR3(0,1,1)
     &  + 4.8045301391820142d-01*HR4(-1,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(-1,-1,1,1)
     &  - 4.8045301391820142d-01*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,1,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,0,1,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,1,1)
     &  - 4.8045301391820142d-01*HR4(0,1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,1,1,-1)
     &  + 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,0,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,0,0,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,0,1,1)
     &  + 1.3862943611198906d+00*HR5(0,0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,1,-1)
     &  + 2.0794415416798359d+00*HR5(0,1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - HR6( -1,-1,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - HR6(0, -1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  - HR6(0,0, -1,-1,-1,1)
     &  - HR6(0,0, -1,-1,1,-1)
     &  - HR6(0,0, -1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,0,-1,-1,-1)
     &  - HR6(0,0,0, -1,-1,1)
     &  - HR6(0,0,0, -1,1,-1)
     &  - HR6(0,0,0,0, -1,-1)
     &  - HR6(0,0,0,0, -1,1)
     &  - HR6(0,0,0,0,1, -1)
     &  - HR6(0,0,0,0,1,1) 
     &  - 2.0000000000000000d+00*HR6(0,0,0,1,-1,-1)
     &  - HR6(0,0,0,1, -1,1)
     &  - HR6(0,0,0,1,1, -1)
     &  - 3.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  - HR6(0,0,1, -1,-1,1)
     &  - HR6(0,0,1, -1,1,-1)
     &  - HR6(0,0,1,1, -1,-1)
     &  - 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  - HR6(0,1, -1,-1,-1,1)
     &  - HR6(0,1, -1,-1,1,-1)
     &  - HR6(0,1, -1,1,-1,-1)
     &  - HR6(0,1,1, -1,-1,-1)
      HY6(0,1,-1,-1,-1,-1) = 
     &  + 2.6959594046650189d-03
     &  + 2.4216790181004488d-02*HR1(-1)
     &  - 4.8090645538142385d-03*HR1(-1)*HR1(-1)
     &  + 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 9.6181291076284771d-03*HR1(-1)*HR1(0)
     &  - 9.6181291076284771d-03*HR1(-1)*HR1(1)
     &  - 5.5504108664821579d-02*HR1(-1)*HR2(-1,1)
     &  - 5.5504108664821579d-02*HR1(-1)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  + 9.6181291076284771d-03*HR1(0)*HR1(1)
     &  + 2.4216790181004488d-02*HR1(1)
     &  - 5.5504108664821579d-02*HR1(1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,-1,-1,-1)
     &  + 9.6181291076284771d-03*HR2(-1,1)
     &  - 9.6181291076284771d-03*HR2(0,-1)
     &  - 9.6181291076284771d-03*HR2(0,1)
     &  + 5.5504108664821579d-02*HR3(-1,-1,1)
     &  + 1.1100821732964315d-01*HR3(0,-1,-1)
     &  + 5.5504108664821579d-02*HR3(0,-1,1)
     &  + 5.5504108664821579d-02*HR3(0,1,-1)
     &  + 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  - 7.2067952087730213d-01*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,1,-1)
     &  - 2.4022650695910071d-01*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  - 5.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - HR6(0, -1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,-1)
     &  - HR6(0,1, -1,-1,-1,-1)
      HY6(0,1,-1,-1,-1,1) = 
     &  + 3.6389303535337737d-03
     &  + 3.7030939562549564d-02*HR1(-1)
     &  - 7.0671186074950043d-03*HR1(-1)*HR1(-1)
     &  + 1.5792167371687950d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 4.7376502115063852d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 1.4134237214990008d-02*HR1(-1)*HR1(0)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - 1.4134237214990008d-02*HR1(-1)*HR1(1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 9.4753004230127705d-02*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - 9.4753004230127705d-02*HR1(-1)*HR2(0,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + 1.4134237214990008d-02*HR1(0)*HR1(1)
     &  + HR1(0) *HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(0) *HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - HR1(0) *HR5(0,-1,-1,-1,1)
     &  - HR1(0) *HR5(0,-1,-1,1,-1)
     &  - HR1(0) *HR5(0,-1,1,-1,-1)
     &  - HR1(0) *HR5(0,1,-1,-1,-1)
     &  + 3.7030939562549564d-02*HR1(1)
     &  - 9.4753004230127705d-02*HR1(1)*HR2(0,-1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + 1.4134237214990008d-02*HR2(-1,1)
     &  - HR2( -1,1)*HR4(0,-1,-1,-1)
     &  - 1.4134237214990008d-02*HR2(0,-1)
     &  - HR2(0, -1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  + HR2(0, -1)*HR4(0,-1,-1,1)
     &  + HR2(0, -1)*HR4(0,-1,1,-1)
     &  + HR2(0, -1)*HR4(0,1,-1,-1)
     &  - 1.4134237214990008d-02*HR2(0,1)
     &  + HR2(0,1) *HR4(0,-1,-1,-1)
     &  + 9.4753004230127705d-02*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + 1.8950600846025541d-01*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,1)
     &  - HR3(0, -1,-1)*HR3(0,1,-1)
     &  + 9.4753004230127705d-02*HR3(0,-1,1)
     &  + 9.4753004230127705d-02*HR3(0,1,-1)
     &  + 5.8224052646501250d-01*HR4(-1,-1,-1,1)
     &  - 1.7467215793950375d+00*HR4(0,-1,-1,-1)
     &  - 5.8224052646501250d-01*HR4(0,-1,-1,1)
     &  - 5.8224052646501250d-01*HR4(0,-1,1,-1)
     &  - 5.8224052646501250d-01*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,0,1)
     &  + HR6(0, -1,-1,-1,1,-1)
     &  + HR6(0, -1,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  + HR6(0, -1,1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + HR6(0,1, -1,-1,-1,-1)
      HY6(0,1,-1,-1,1,-1) = 
     &  + 5.2254539749517106d-03
     &  + 6.5187621595008706d-02*HR1(-1)
     &  - 1.1637533043363782d-02*HR1(-1)*HR1(-1)
     &  + 3.5678728477784371d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 1.0703618543335311d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 2.3275066086727564d-02*HR1(-1)*HR1(0)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 2.3275066086727564d-02*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.1407237086670622d-01*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.1407237086670622d-01*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + 2.3275066086727564d-02*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,-1)
     &  + 6.5187621595008706d-02*HR1(1)
     &  - 2.1407237086670622d-01*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + 2.3275066086727564d-02*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  - 2.3275066086727564d-02*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR2(0,-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,-1)
     &  + HR2(0, -1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  - HR2(0, -1)*HR4(0,-1,-1,1)
     &  - HR2(0, -1)*HR4(0,-1,1,-1)
     &  - HR2(0, -1)*HR4(0,1,-1,-1)
     &  - 2.3275066086727564d-02*HR2(0,1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(0,-1,-1)
     &  - 3.0000000000000000d+00*HR2(0,1)*HR4(0,-1,-1,-1)
     &  + 2.1407237086670622d-01*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  + 4.2814474173341245d-01*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,1,-1)
     &  + 2.1407237086670622d-01*HR3(0,-1,1)
     &  + 2.1407237086670622d-01*HR3(0,1,-1)
     &  - 1.0626935403832139d+00*HR4(-1,-1,-1,1)
     &  + 3.1880806211496417d+00*HR4(0,-1,-1,-1)
     &  + 1.0626935403832139d+00*HR4(0,-1,-1,1)
     &  + 1.0626935403832139d+00*HR4(0,-1,1,-1)
     &  + 1.0626935403832139d+00*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,0,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,0,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  + 4.1588830833596718d+00*HR5(0,0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,-1,0,1)
     &  + HR6(0, -1,-1,0,-1,1)
     &  + HR6(0, -1,-1,0,1,-1)
     &  + HR6(0, -1,-1,1,-1,-1)
     &  + HR6(0, -1,1,-1,-1,-1)
     &  + HR6(0,1, -1,-1,-1,-1)
      HY6(0,1,-1,-1,1,1) = 
     &  + 7.8087314804442795d-03
     &  + 1.2272387559610690d-01*HR1(-1)
     &  - 2.0379119579654625d-02*HR1(-1)*HR1(-1)
     &  + 8.9535532268006700d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 2.6860659680402010d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 4.0758239159309251d-02*HR1(-1)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 4.0758239159309251d-02*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 5.3721319360804020d-01*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,0,-1)
     &  - 5.3721319360804020d-01*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,0,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,-1)
     &  - HR1( -1)*HR5(0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,1,-1,-1)
     &  + 4.0758239159309251d-02*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1(0) *HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,-1,1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,1,-1)
     &  - HR1(0) *HR2(0,1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,-1)
     &  + HR1(0) *HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + HR1(0) *HR5(0,-1,-1,0,1)
     &  + HR1(0) *HR5(0,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,0,-1,-1)
     &  + HR1(0) *HR5(0,-1,1,-1,-1)
     &  - 6.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,-1)
     &  + HR1(0) *HR5(0,1,-1,-1,-1)
     &  + 1.2272387559610690d-01*HR1(1)
     &  - 5.3721319360804020d-01*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,-1)
     &  - HR1(1) *HR5(0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,-1)
     &  + 4.0758239159309251d-02*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  - HR2( -1,1)*HR4(0,0,-1,-1)
     &  - 4.0758239159309251d-02*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR2(0,-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,-1)
     &  + HR2(0, -1)*HR4(0,0,-1,-1)
     &  - 4.0758239159309251d-02*HR2(0,1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR2(0,1)*HR4(0,-1,-1,-1)
     &  + HR2(0,1) *HR4(0,0,-1,-1)
     &  + 5.3721319360804020d-01*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - HR3( -1,-1,1)*HR3(0,0,-1)
     &  + 1.0744263872160804d+00*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,0,-1)
     &  - HR3(0, -1,-1)*HR3(0,1,-1)
     &  + 5.3721319360804020d-01*HR3(0,-1,1)
     &  - HR3(0, -1,1)*HR3(0,0,-1)
     &  - HR3(0,0, -1)*HR3(0,1,-1)
     &  + 5.3721319360804020d-01*HR3(0,1,-1)
     &  + 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  - 7.2067952087730213d-01*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,1,-1)
     &  - 2.4022650695910071d-01*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,0,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  - 2.0794415416798359d+00*HR5(0,-1,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  - 4.1588830833596718d+00*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,-1,0,1)
     &  - HR6(0, -1,-1,0,-1,1)
     &  - HR6(0, -1,-1,0,1,-1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,-1)
     &  + 5.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + HR6(0,0, -1,-1,-1,1)
     &  - HR6(0,0, -1,-1,0,-1)
     &  - HR6(0,0, -1,-1,0,1)
     &  - HR6(0,1, -1,-1,-1,-1)
      HY6(0,1,-1,0,1,1) = 
     &  + 2.6594848155187647d-02
     &  + 3.1682091096486227d-01*HR1(-1)
     &  - 5.6428748221951486d-02*HR1(-1)*HR1(-1)
     &  + 2.0034281719326571d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 6.0102845157979714d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 1.1285749644390297d-01*HR1(-1)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,0,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(0,1,-1,-1)
     &  - 1.1285749644390297d-01*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 1.2020569031595942d+00*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,0,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,0,1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,1,-1)
     &  - 1.2020569031595942d+00*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,0,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,0,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(-1,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,-1)
     &  + HR1( -1)*HR5(0,-1,0,1,-1)
     &  + HR1( -1)*HR5(0,-1,1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,-1)
     &  - HR1( -1)*HR5(0,0,-1,0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,0,1,-1)
     &  - HR1( -1)*HR5(0,0,1,-1,-1)
     &  - HR1( -1)*HR5(0,0,1,0,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  + 2.5000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(0,-1,-1,-1)
     &  - 1.5000000000000000d+00*HR1(0)*HR1(0)*HR4(0,-1,-1,1)
     &  - HR1(0) *HR1(0)*HR4(0,-1,1,-1)
     &  - HR1(0) *HR1(0)*HR4(0,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,1,-1,1)
     &  + 1.1285749644390297d-01*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,0,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,-1,1)
     &  + HR1(0) *HR1(1)*HR4(0,1,-1,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(-1,-1,1)
     &  + HR1(0) *HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(0) *HR2(-1,1)*HR3(0,1,-1)
     &  + HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,-1,1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,1,-1)
     &  + HR1(0) *HR2(0,1)*HR3(-1,-1,1)
     &  - HR1(0) *HR2(0,1)*HR3(0,-1,-1)
     &  + HR1(0) *HR2(0,1)*HR3(0,1,-1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR1(0)*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,-1,1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,0,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,-1,0,-1,1)
     &  + HR1(0) *HR5(0,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,-1,0,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,-1)
     &  + HR1(0) *HR5(0,-1,1,-1,1)
     &  + HR1(0) *HR5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,-1,1,1,-1)
     &  - 6.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(0,0,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,0,1,1,-1)
     &  - HR1(0) *HR5(0,1,-1,-1,1)
     &  + HR1(0) *HR5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,1,0,1,-1)
     &  + 3.1682091096486227d-01*HR1(1)
     &  - 1.2020569031595942d+00*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,0,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,0,1)
     &  - 2.4022650695910071d-01*HR1(1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,1,-1,-1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,-1)
     &  + HR1(1) *HR5(0,-1,0,1,-1)
     &  + HR1(1) *HR5(0,-1,1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,-1)
     &  - HR1(1) *HR5(0,0,-1,0,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,0,1,-1)
     &  - HR1(1) *HR5(0,0,1,-1,-1)
     &  - HR1(1) *HR5(0,0,1,0,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  + 1.1285749644390297d-01*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR2(-1,1)*HR2(-1,1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  - HR2( -1,1)*HR4(0,0,-1,-1)
     &  - HR2( -1,1)*HR4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,1,-1,-1)
     &  - 1.1285749644390297d-01*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR2(0,-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,-1)
     &  + HR2(0, -1)*HR4(0,0,-1,-1)
     &  + HR2(0, -1)*HR4(0,0,1,-1)
     &  - 1.1285749644390297d-01*HR2(0,1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR2(0,1)*HR4(0,-1,-1,-1)
     &  + HR2(0,1) *HR4(0,0,-1,-1)
     &  + HR2(0,1) *HR4(0,0,1,-1)
     &  + 1.2020569031595942d+00*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR3(-1,-1,1)*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - HR3( -1,-1,1)*HR3(0,0,-1)
     &  - HR3( -1,-1,1)*HR3(0,0,1)
     &  - HR3( -1,-1,1)*HR3(0,1,-1)
     &  + 2.4041138063191885d+00*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,0,1)
     &  - HR3(0, -1,-1)*HR3(0,1,-1)
     &  + 1.2020569031595942d+00*HR3(0,-1,1)
     &  - HR3(0, -1,1)*HR3(0,0,-1)
     &  - HR3(0, -1,1)*HR3(0,0,1)
     &  - HR3(0,0, -1)*HR3(0,1,-1)
     &  - HR3(0,0,1) *HR3(0,1,-1)
     &  + 1.2020569031595942d+00*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR3(0,1,-1)*HR3(0,1,-1)
     &  - 7.2067952087730213d-01*HR4(0,-1,-1,-1)
     &  - 7.2067952087730213d-01*HR4(0,-1,-1,1)
     &  - 4.8045301391820142d-01*HR4(0,-1,1,-1)
     &  - 4.8045301391820142d-01*HR4(0,-1,1,1)
     &  - 2.4022650695910071d-01*HR4(0,1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,1,-1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,-1,-1,1)
     &  - 4.1588830833596718d+00*HR5(-1,-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR5(-1,-1,1,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,0,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,1,-1)
     &  - 2.0794415416798359d+00*HR5(0,-1,0,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,0,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,0,1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,0,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,1,1,-1)
     &  - 4.1588830833596718d+00*HR5(0,0,-1,-1,-1)
     &  + 2.7725887222397812d+00*HR5(0,0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,0,-1,1,-1)
     &  + 2.7725887222397812d+00*HR5(0,0,-1,1,1)
     &  - 2.7725887222397812d+00*HR5(0,0,1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,1,-1)
     &  - 1.3862943611198906d+00*HR5(0,1,0,1,-1)
     &  - 9.0000000000000000d+00*HR6(-1,-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR6(-1,-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR6(-1,-1,-1,1,-1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,-1,0,1)
     &  - HR6(0, -1,-1,0,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,1,-1,-1)
     &  - HR6(0, -1,0,1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,1,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,1)
     &  - HR6(0, -1,1,-1,1,-1)
     &  - HR6(0, -1,1,0,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,1,-1,-1)
     &  + 5.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + HR6(0,0, -1,-1,-1,1)
     &  - HR6(0,0, -1,-1,0,-1)
     &  - HR6(0,0, -1,-1,0,1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,1,1,-1)
     &  + HR6(0,0,1, -1,-1,-1)
     &  + HR6(0,0,1, -1,-1,1)
     &  - HR6(0,0,1, -1,0,-1)
     &  - HR6(0,0,1, -1,0,1)
     &  + 4.0000000000000000d+00*HR6(0,0,1,1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  + HR6(0,1, -1,-1,1,-1)
     &  - HR6(0,1, -1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,1,0,1,-1,-1)
      HY6(0,1,-1,1,-1,-1) = 
     &  + 8.4191205208964350d-03
     &  + 1.6835931890634887d-01*HR1(-1)
     &  - 2.5458382032146317d-02*HR1(-1)*HR1(-1)
     &  - 7.9222950181883112d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.3766885054564933d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 5.0916764064292634d-02*HR1(-1)*HR1(0)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.0916764064292634d-02*HR1(-1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 4.7533770109129867d-01*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + 4.7533770109129867d-01*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,-1)
     &  + 5.0916764064292634d-02*HR1(0)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR1(0)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(0)*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(0)*HR3(0,-1,1)
     &  - 2.4022650695910071d-01*HR1(0)*HR3(0,1,-1)
     &  + 1.6835931890634887d-01*HR1(1)
     &  + 4.7533770109129867d-01*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,0,-1)
     &  + 2.7725887222397812d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,-1)
     &  + 5.0916764064292634d-02*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR2(-1,1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  - 5.0916764064292634d-02*HR2(0,-1)
     &  + 1.2011325347955035d-01*HR2(0,-1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR2(0,-1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR2(0,-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,-1)
     &  - 5.0916764064292634d-02*HR2(0,1)
     &  - 1.3862943611198906d+00*HR2(0,1)*HR3(0,-1,-1)
     &  + 3.0000000000000000d+00*HR2(0,1)*HR4(0,-1,-1,-1)
     &  - 4.7533770109129867d-01*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - 9.5067540218259734d-01*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,1)
     &  - HR3(0, -1,-1)*HR3(0,1,-1)
     &  - 4.7533770109129867d-01*HR3(0,-1,1)
     &  - 4.7533770109129867d-01*HR3(0,1,-1)
     &  - 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  + 9.6090602783640284d-01*HR4(0,-1,-1,-1)
     &  + 4.8045301391820142d-01*HR4(0,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,0,1)
     &  + 2.4022650695910071d-01*HR4(0,-1,1,-1)
     &  + 4.8045301391820142d-01*HR4(0,0,-1,-1)
     &  + 2.4022650695910071d-01*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  - 5.5451774444795624d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,0,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  - 2.7725887222397812d+00*HR5(0,-1,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  - 8.3177661667193437d+00*HR5(0,0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  + 1.5000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,0,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  - HR6(0, -1,0,-1,-1,1)
     &  - HR6(0, -1,0,-1,1,-1)
     &  - HR6(0, -1,0,1,-1,-1)
     &  + HR6(0, -1,1,-1,-1,-1)
     &  + 1.2000000000000000d+01*HR6(0,0,-1,-1,-1,-1)
     &  + HR6(0,1, -1,-1,-1,-1)
      HY6(0,1,-1,1,-1,1) = 
     &  + 1.2504107554099770d-02
     &  + 3.0867184409650406d-01*HR1(-1)
     &  - 4.3992768505254480d-02*HR1(-1)*HR1(-1)
     &  - 2.4633412775717367d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 7.3900238327152102d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 8.7985537010508960d-02*HR1(-1)*HR1(0)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,-1,-1)
     &  - 8.7985537010508960d-02*HR1(-1)*HR1(1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 1.4780047665430420d+00*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,0,-1)
     &  + 1.4780047665430420d+00*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 1.1644810529300250d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 1.1644810529300250d+00*HR1(-1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + 8.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,0,-1)
     &  + 1.2000000000000000d+01*HR1(-1)*HR5(0,0,0,-1,-1)
     &  + 8.7985537010508960d-02*HR1(0)*HR1(1)
     &  + 5.8224052646501250d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,-1,1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(0,1)*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR1(0)*HR3(-1,-1,1)
     &  - 1.1644810529300250d+00*HR1(0)*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR1(0)*HR3(0,-1,1)
     &  - 5.8224052646501250d-01*HR1(0)*HR3(0,1,-1)
     &  + HR1(0) *HR5(-1,-1,-1,-1,1)
     &  + 8.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(0,-1,0,-1,-1)
     &  - HR1(0) *HR5(0,-1,0,-1,1)
     &  - HR1(0) *HR5(0,-1,0,1,-1)
     &  + HR1(0) *HR5(0,-1,1,-1,-1)
     &  + 1.2000000000000000d+01*HR1(0)*HR5(0,0,-1,-1,-1)
     &  + HR1(0) *HR5(0,1,-1,-1,-1)
     &  + 3.0867184409650406d-01*HR1(1)
     &  + 1.4780047665430420d+00*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,0,-1)
     &  - 1.1644810529300250d+00*HR1(1)*HR3(0,-1,-1)
     &  - 1.1644810529300250d+00*HR1(1)*HR3(0,0,-1)
     &  - 2.7725887222397812d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,-1)
     &  + 8.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,0,-1)
     &  + 1.2000000000000000d+01*HR1(1)*HR5(0,0,0,-1,-1)
     &  + 8.7985537010508960d-02*HR2(-1,1)
     &  - 5.8224052646501250d-01*HR2(-1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR2(-1,1)*HR2(0,-1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  + 4.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,-1)
     &  - 8.7985537010508960d-02*HR2(0,-1)
     &  + 2.9112026323250625d-01*HR2(0,-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR2(0,-1)*HR2(0,-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR2(0,-1)*HR2(0,-1)*HR2(0,1)
     &  + 5.8224052646501250d-01*HR2(0,-1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR2(0,-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,-1)
     &  + HR2(0, -1)*HR4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,1)
     &  + HR2(0, -1)*HR4(0,-1,0,1)
     &  - HR2(0, -1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,0,-1,-1)
     &  - HR2(0, -1)*HR4(0,1,-1,-1)
     &  - 8.7985537010508960d-02*HR2(0,1)
     &  + 1.3862943611198906d+00*HR2(0,1)*HR3(0,-1,-1)
     &  - 4.0000000000000000d+00*HR2(0,1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR2(0,1)*HR4(0,0,-1,-1)
     &  - 1.4780047665430420d+00*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,-1)
     &  - 2.9560095330860840d+00*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,-1,1)
     &  + 4.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,1,-1)
     &  - 1.4780047665430420d+00*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR3(0,-1,1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR3(0,0,-1)*HR3(0,1,-1)
     &  - 1.4780047665430420d+00*HR3(0,1,-1)
     &  - 5.8224052646501250d-01*HR4(-1,-1,-1,1)
     &  + 2.3289621058600500d+00*HR4(0,-1,-1,-1)
     &  + 1.1644810529300250d+00*HR4(0,-1,-1,1)
     &  - 5.8224052646501250d-01*HR4(0,-1,0,1)
     &  + 5.8224052646501250d-01*HR4(0,-1,1,-1)
     &  + 1.1644810529300250d+00*HR4(0,0,-1,-1)
     &  + 5.8224052646501250d-01*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  + 5.5451774444795624d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,0,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  + 2.7725887222397812d+00*HR5(0,-1,0,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  + 8.3177661667193437d+00*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  - 1.6000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  - 8.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,0,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  - HR6(0, -1,0,-1,0,1)
     &  + HR6(0, -1,0,-1,1,-1)
     &  + HR6(0, -1,0,1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,-1)
     &  - 8.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,0,-1,-1)
     &  - 1.2000000000000000d+01*HR6(0,0,0,-1,-1,-1)
     &  - HR6(0,1, -1,-1,-1,-1)
      HY6(0,1,-1,1,1,-1) = 
     &  + 2.0912204250035171d-02
     &  + 8.1893361933293943d-01*HR1(-1)
     &  - 1.0143289758994481d-01*HR1(-1)*HR1(-1)
     &  + 1.8455064982157776d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 5.5365194946473328d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0286579517988963d-01*HR1(-1)*HR1(0)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  - 2.0286579517988963d-01*HR1(-1)*HR1(1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 1.1073038989294665d+00*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,0,-1)
     &  - 1.1073038989294665d+00*HR1(-1)*HR2(0,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,0,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 2.1253870807664278d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 2.1253870807664278d+00*HR1(-1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,0,0,-1)
     &  - HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  + 9.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  + 2.0286579517988963d-01*HR1(0)*HR1(1)
     &  - 1.0626935403832139d+00*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(0,-1)*HR2(0,1)
     &  + 1.0626935403832139d+00*HR1(0)*HR3(-1,-1,1)
     &  + 2.1253870807664278d+00*HR1(0)*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR1(0)*HR3(0,-1,1)
     &  + 1.0626935403832139d+00*HR1(0)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,-1)
     &  + 8.1893361933293943d-01*HR1(1)
     &  - 1.1073038989294665d+00*HR1(1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,0,-1)
     &  + 2.1253870807664278d+00*HR1(1)*HR3(0,-1,-1)
     &  + 2.1253870807664278d+00*HR1(1)*HR3(0,0,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  + 9.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,-1)
     &  + 2.0286579517988963d-01*HR2(-1,1)
     &  + 1.0626935403832139d+00*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,0,-1)
     &  - 2.0286579517988963d-01*HR2(0,-1)
     &  - 5.3134677019160696d-01*HR2(0,-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR2(0,-1)*HR2(0,-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR2(0,-1)*HR2(0,-1)*HR2(0,1)
     &  - 1.0626935403832139d+00*HR2(0,-1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,0,-1)
     &  - HR2(0, -1)*HR4(-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,1)
     &  - HR2(0, -1)*HR4(0,-1,0,1)
     &  + HR2(0, -1)*HR4(0,-1,1,-1)
     &  + HR2(0, -1)*HR4(0,1,-1,-1)
     &  - 2.0286579517988963d-01*HR2(0,1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(0,0,-1)
     &  + 1.1073038989294665d+00*HR3(-1,-1,1)
     &  - HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - HR3( -1,-1,1)*HR3(0,0,-1)
     &  + 2.2146077978589331d+00*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,0,-1)
     &  - HR3(0, -1,-1)*HR3(0,1,-1)
     &  + 1.1073038989294665d+00*HR3(0,-1,1)
     &  - HR3(0, -1,1)*HR3(0,0,-1)
     &  - HR3(0,0, -1)*HR3(0,1,-1)
     &  + 1.1073038989294665d+00*HR3(0,1,-1)
     &  + 1.0626935403832139d+00*HR4(-1,-1,-1,1)
     &  - 4.2507741615328557d+00*HR4(0,-1,-1,-1)
     &  - 2.1253870807664278d+00*HR4(0,-1,-1,1)
     &  + 1.0626935403832139d+00*HR4(0,-1,0,1)
     &  - 1.0626935403832139d+00*HR4(0,-1,1,-1)
     &  - 2.1253870807664278d+00*HR4(0,0,-1,-1)
     &  - 1.0626935403832139d+00*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  + 4.8520302639196171d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,0,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  + 4.8520302639196171d+00*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,0,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,0,1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  - 1.0000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + HR6(0, -1,-1,0,-1,1)
     &  + HR6(0, -1,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  + HR6(0, -1,0,-1,0,1)
     &  + HR6(0, -1,0,-1,1,-1)
     &  + HR6(0, -1,0,1,-1,-1)
     &  - HR6(0, -1,1,-1,-1,-1)
     &  - 1.6000000000000000d+01*HR6(0,0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,0,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,0,1)
     &  - HR6(0,0, -1,-1,1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,-1,0,-1,-1)
     &  + HR6(0,0, -1,0,-1,1)
     &  + HR6(0,0, -1,0,1,-1)
     &  + 1.2000000000000000d+01*HR6(0,0,0,-1,-1,-1)
     &  - HR6(0,1, -1,-1,-1,-1)
      HY6(0,1,-1,1,1,1) = 
     &  + 3.7192729240182574d-02
     &  + 2.3922914695671442d+00*HR1(-1)
     &  - 2.5873953083694969d-01*HR1(-1)*HR1(-1)
     &  - 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  + 5.1747906167389938d-01*HR1(-1)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,0,-1)
     &  - 5.1747906167389938d-01*HR1(-1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,0,-1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR2(-1,1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,0,0,-1)
     &  + HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,0,0,-1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 3.3333333333333333d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  - 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - HR1(0) *HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(0)*HR1(1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(0)*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(0,-1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR1(0)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,1,-1)
     &  + HR1(0) *HR1(0)*HR4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,1,-1,-1)
     &  + 5.1747906167389938d-01*HR1(0)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + HR1(0) *HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(0) *HR2(-1,1)*HR3(0,0,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(0,-1)*HR2(0,1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,0,-1)
     &  - HR1(0) *HR2(0,1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(0,1)*HR3(0,0,-1)
     &  - 2.4022650695910071d-01*HR1(0)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(0)*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(0)*HR3(0,-1,1)
     &  - 2.4022650695910071d-01*HR1(0)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,0,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,-1)
     &  - HR1(0) *HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  + HR1(0) *HR5(0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  - HR1(0) *HR5(0,-1,0,-1,-1)
     &  + HR1(0) *HR5(0,-1,0,-1,1)
     &  + HR1(0) *HR5(0,-1,0,1,-1)
     &  - HR1(0) *HR5(0,-1,1,-1,-1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,-1)
     &  - HR1(0) *HR5(0,0,-1,-1,1)
     &  + HR1(0) *HR5(0,0,-1,0,-1)
     &  + HR1(0) *HR5(0,0,-1,0,1)
     &  - HR1(0) *HR5(0,1,-1,-1,-1)
     &  + 2.3922914695671442d+00*HR1(1)
     &  + 5.5504108664821579d-02*HR1(1)*HR2(0,-1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,0,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,0,-1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,0,0,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,0,0,-1)
     &  + 5.1747906167389938d-01*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,0,-1)
     &  - HR2( -1,1)*HR4(0,-1,-1,-1)
     &  - HR2( -1,1)*HR4(0,0,-1,-1)
     &  - HR2( -1,1)*HR4(0,0,0,-1)
     &  - 5.1747906167389938d-01*HR2(0,-1)
     &  + 1.2011325347955035d-01*HR2(0,-1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR2(0,-1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,0,-1)
     &  + HR2(0, -1)*HR4(0,0,-1,-1)
     &  + HR2(0, -1)*HR4(0,0,0,-1)
     &  - 5.1747906167389938d-01*HR2(0,1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(0,0,-1)
     &  + HR2(0,1) *HR4(0,-1,-1,-1)
     &  + HR2(0,1) *HR4(0,0,-1,-1)
     &  + HR2(0,1) *HR4(0,0,0,-1)
     &  - 5.5504108664821579d-02*HR3(-1,-1,1)
     &  - 1.1100821732964315d-01*HR3(0,-1,-1)
     &  - 5.5504108664821579d-02*HR3(0,-1,1)
     &  - 5.5504108664821579d-02*HR3(0,1,-1)
     &  - 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  + 9.6090602783640284d-01*HR4(0,-1,-1,-1)
     &  + 4.8045301391820142d-01*HR4(0,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,0,1)
     &  + 2.4022650695910071d-01*HR4(0,-1,1,-1)
     &  + 4.8045301391820142d-01*HR4(0,0,-1,-1)
     &  + 2.4022650695910071d-01*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  - 4.8520302639196171d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,0,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,-1)
     &  - 4.8520302639196171d+00*HR5(0,0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,0,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,0,1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  + 1.1000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,0,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,0,-1,1)
     &  - HR6(0, -1,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + HR6(0, -1,0,-1,-1,-1)
     &  - HR6(0, -1,0,-1,-1,1)
     &  - HR6(0, -1,0,-1,1,-1)
     &  - HR6(0, -1,0,1,-1,-1)
     &  + HR6(0, -1,1,-1,-1,-1)
     &  + 1.1000000000000000d+01*HR6(0,0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  - HR6(0,0, -1,-1,0,-1)
     &  - HR6(0,0, -1,-1,0,1)
     &  + HR6(0,0, -1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,0,-1,-1)
     &  - HR6(0,0, -1,0,-1,1)
     &  - HR6(0,0, -1,0,1,-1)
     &  + HR6(0,0,0, -1,-1,-1)
     &  + HR6(0,0,0, -1,-1,1)
     &  - HR6(0,0,0, -1,0,-1)
     &  - HR6(0,0,0, -1,0,1)
     &  + HR6(0,1, -1,-1,-1,-1)
      HY6(0,1,0,1,-1,-1) = 
     &  + 2.2264644192427555d-02
     &  + 3.5993499854224487d-01*HR1(-1)
     &  - 5.7061713708030422d-02*HR1(-1)*HR1(-1)
     &  - 7.9222950181883112d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.3766885054564933d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 1.1412342741606084d-01*HR1(-1)*HR1(0)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 1.1412342741606084d-01*HR1(-1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  + 4.7533770109129867d-01*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + 4.7533770109129867d-01*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  + 4.7533770109129867d-01*HR1(-1)*HR2(0,1)
     &  + HR1( -1)*HR2(0,1)*HR3(0,-1,-1)
     &  + 4.8045301391820142d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(-1,1,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,0,-1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,0,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,1,-1)
     &  + 2.7725887222397812d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.7725887222397812d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,0,1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,1,-1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,-1,0,1)
     &  - HR1( -1)*HR5(0,-1,-1,1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,1)
     &  - HR1( -1)*HR5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  + 1.1412342741606084d-01*HR1(0)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(0,1)
     &  - 4.8045301391820142d-01*HR1(0)*HR3(-1,-1,1)
     &  + 4.8045301391820142d-01*HR1(0)*HR3(-1,1,1)
     &  - 4.8045301391820142d-01*HR1(0)*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR1(0)*HR3(0,-1,1)
     &  - 4.8045301391820142d-01*HR1(0)*HR3(0,1,-1)
     &  - 4.8045301391820142d-01*HR1(0)*HR3(0,1,1)
     &  + 3.5993499854224487d-01*HR1(1)
     &  - 4.7533770109129867d-01*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  + 4.7533770109129867d-01*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  + 4.7533770109129867d-01*HR1(1)*HR2(0,1)
     &  + HR1(1) *HR2(0,1)*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,0,-1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,0,1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,0,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,0,1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - HR1(1) *HR5(0,-1,-1,0,1)
     &  - HR1(1) *HR5(0,-1,-1,1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,-1)
     &  - HR1(1) *HR5(0,-1,0,-1,1)
     &  - HR1(1) *HR5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  + 1.1412342741606084d-01*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR2(-1,1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR2(-1,1)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR2(-1,1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,1,-1)
     &  + HR2( -1,1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  - HR2( -1,1)*HR4(0,-1,-1,1)
     &  - HR2( -1,1)*HR4(0,-1,1,-1)
     &  - HR2( -1,1)*HR4(0,1,-1,-1)
     &  - 1.1412342741606084d-01*HR2(0,-1)
     &  + 1.2011325347955035d-01*HR2(0,-1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR2(0,-1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR2(0,-1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR2(0,-1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR2(0,-1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR2(0,-1)*HR3(0,-1,1)
     &  + 1.3862943611198906d+00*HR2(0,-1)*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR2(0,-1)*HR3(0,1,1)
     &  - 1.1412342741606084d-01*HR2(0,1)
     &  + 1.2011325347955035d-01*HR2(0,1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR2(0,1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(0,-1,1)
     &  + 3.0000000000000000d+00*HR2(0,1)*HR4(0,-1,-1,-1)
     &  + HR2(0,1) *HR4(0,-1,-1,1)
     &  + HR2(0,1) *HR4(0,-1,1,-1)
     &  - 9.5067540218259734d-01*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  + 9.5067540218259734d-01*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  - 9.5067540218259734d-01*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,1,1)
     &  - 9.5067540218259734d-01*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR3(0,-1,1)*HR3(0,-1,1)
     &  - 9.5067540218259734d-01*HR3(0,1,-1)
     &  - 9.5067540218259734d-01*HR3(0,1,1)
     &  - 9.6090602783640284d-01*HR4(-1,-1,-1,1)
     &  + 4.8045301391820142d-01*HR4(-1,-1,1,1)
     &  + 9.6090602783640284d-01*HR4(0,-1,-1,-1)
     &  + 4.8045301391820142d-01*HR4(0,-1,-1,1)
     &  + 4.8045301391820142d-01*HR4(0,-1,1,-1)
     &  + 4.8045301391820142d-01*HR4(0,0,-1,-1)
     &  + 4.8045301391820142d-01*HR4(0,0,-1,1)
     &  + 4.8045301391820142d-01*HR4(0,0,1,-1)
     &  + 4.8045301391820142d-01*HR4(0,0,1,1)
     &  + 9.6090602783640284d-01*HR4(0,1,-1,-1)
     &  + 4.8045301391820142d-01*HR4(0,1,-1,1)
     &  + 4.8045301391820142d-01*HR4(0,1,1,-1)
     &  - 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  - 5.5451774444795624d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.7725887222397812d+00*HR5(0,-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,0,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  - 2.7725887222397812d+00*HR5(0,-1,0,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,1,-1)
     &  - 2.0794415416798359d+00*HR5(0,-1,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,0,1)
     &  - 8.3177661667193437d+00*HR5(0,0,-1,-1,-1)
     &  - 2.7725887222397812d+00*HR5(0,0,-1,-1,1)
     &  - 2.7725887222397812d+00*HR5(0,0,-1,1,-1)
     &  - 2.7725887222397812d+00*HR5(0,0,1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,0,1,-1,1)
     &  - 2.7725887222397812d+00*HR5(0,0,1,1,-1)
     &  - 4.8520302639196171d+00*HR5(0,1,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,0,1,-1)
     &  - 1.3862943611198906d+00*HR5(0,1,1,-1,-1)
     &  - 1.1000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  - HR6( -1,-1,-1,1,-1,1)
     &  + 1.5000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,0,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,0,-1,1)
     &  - HR6(0, -1,-1,0,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + HR6(0, -1,-1,1,-1,1)
     &  - HR6(0, -1,-1,1,0,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,1,-1,-1)
     &  + HR6(0, -1,0,1,-1,1)
     &  + 5.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  - HR6(0, -1,1,-1,0,1)
     &  + HR6(0, -1,1,-1,1,-1)
     &  - HR6(0, -1,1,0,1,-1)
     &  + 1.2000000000000000d+01*HR6(0,0,-1,-1,-1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,-1,1,1)
     &  + 6.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,1,-1,1)
     &  + 6.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR6(0,0,1,1,-1,-1)
     &  + 1.1000000000000000d+01*HR6(0,1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  + HR6(0,1,0,1, -1,-1)
     &  + 2.0000000000000000d+00*HR6(0,1,1,-1,-1,-1)
      HY6(0,1,0,1,-1,1) = 
     &  + 3.2107323193639778d-02
     &  + 6.5242009016729586d-01*HR1(-1)
     &  - 9.6777676906532623d-02*HR1(-1)*HR1(-1)
     &  - 2.4633412775717367d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  - 7.3900238327152102d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 1.9355535381306524d-01*HR1(-1)*HR1(0)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR2(0,-1)*HR2(0,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,0,1)
     &  - HR1( -1)*HR1(0)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,1,-1,-1)
     &  - 1.9355535381306524d-01*HR1(-1)*HR1(1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,0,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  + 1.4780047665430420d+00*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,0,-1)
     &  + 1.4780047665430420d+00*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,1,-1)
     &  + 1.4780047665430420d+00*HR1(-1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,1)*HR3(0,0,-1)
     &  + 1.1644810529300250d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 1.1644810529300250d+00*HR1(-1)*HR3(-1,1,1)
     &  - 1.1644810529300250d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 1.1644810529300250d+00*HR1(-1)*HR3(0,0,-1)
     &  - 1.1644810529300250d+00*HR1(-1)*HR3(0,0,1)
     &  - 1.1644810529300250d+00*HR1(-1)*HR3(0,1,-1)
     &  - 2.7725887222397812d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 2.7725887222397812d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,0,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,0,1,-1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,1,-1,1)
     &  + 8.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,1,-1)
     &  + 1.2000000000000000d+01*HR1(-1)*HR5(0,0,0,-1,-1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,1,0,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  + 1.9355535381306524d-01*HR1(0)*HR1(1)
     &  - 5.8224052646501250d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  - HR1(0) *HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 5.8224052646501250d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1(0) *HR1(1)*HR2(0,-1)*HR2(0,1)
     &  + 5.8224052646501250d-01*HR1(0)*HR1(1)*HR2(0,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,0,1)
     &  - HR1(0) *HR1(1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1(0) *HR2(-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,-1,1)
     &  - HR1(0) *HR2(-1,1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(0,-1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR2(0,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR2(0,-1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR2(0,-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR2(0,-1)*HR3(0,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(0,1)*HR3(0,-1,-1)
     &  + HR1(0) *HR2(0,1)*HR3(0,-1,1)
     &  - 1.1644810529300250d+00*HR1(0)*HR3(-1,-1,1)
     &  + 1.1644810529300250d+00*HR1(0)*HR3(-1,1,1)
     &  - 1.1644810529300250d+00*HR1(0)*HR3(0,-1,-1)
     &  - 1.1644810529300250d+00*HR1(0)*HR3(0,-1,1)
     &  - 1.1644810529300250d+00*HR1(0)*HR3(0,1,-1)
     &  - 1.1644810529300250d+00*HR1(0)*HR3(0,1,1)
     &  + 7.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  + HR1(0) *HR5(-1,-1,1,-1,1)
     &  + 8.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(0,-1,0,-1,-1)
     &  + HR1(0) *HR5(0,-1,0,-1,1)
     &  + HR1(0) *HR5(0,-1,0,1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,-1)
     &  + HR1(0) *HR5(0,-1,1,-1,1)
     &  - HR1(0) *HR5(0,-1,1,0,1)
     &  + 1.2000000000000000d+01*HR1(0)*HR5(0,0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(0,0,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,0,1,-1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(0,0,1,1,-1)
     &  + 7.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,1,-1,1,-1)
     &  + HR1(0) *HR5(0,1,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,1,1,-1,-1)
     &  + 6.5242009016729586d-01*HR1(1)
     &  - 1.4780047665430420d+00*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(-1,1)*HR3(0,0,-1)
     &  + 1.4780047665430420d+00*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,1,-1)
     &  + 1.4780047665430420d+00*HR1(1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,1)*HR3(0,0,-1)
     &  - 1.1644810529300250d+00*HR1(1)*HR3(-1,-1,1)
     &  - 1.1644810529300250d+00*HR1(1)*HR3(0,-1,-1)
     &  - 1.1644810529300250d+00*HR1(1)*HR3(0,0,-1)
     &  - 1.1644810529300250d+00*HR1(1)*HR3(0,0,1)
     &  - 1.1644810529300250d+00*HR1(1)*HR3(0,1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,0,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,0,1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 8.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,-1,1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,0,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,0,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,1,-1)
     &  + 1.2000000000000000d+01*HR1(1)*HR5(0,0,0,-1,-1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,1,0,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  + 1.9355535381306524d-01*HR2(-1,1)
     &  + 2.9112026323250625d-01*HR2(-1,1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.8224052646501250d-01*HR2(-1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR2(-1,1)*HR2(0,-1)*HR2(0,-1)
     &  + HR2( -1,1)*HR2(0,-1)*HR2(0,1)
     &  - 5.8224052646501250d-01*HR2(-1,1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,1,-1)
     &  - HR2( -1,1)*HR4(-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,1)
     &  - HR2( -1,1)*HR4(0,-1,0,1)
     &  + HR2( -1,1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,-1)
     &  + HR2( -1,1)*HR4(0,1,-1,-1)
     &  - 1.9355535381306524d-01*HR2(0,-1)
     &  + 2.9112026323250625d-01*HR2(0,-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR2(0,-1)*HR2(0,-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR2(0,-1)*HR2(0,-1)*HR2(0,1)
     &  + 5.8224052646501250d-01*HR2(0,-1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR2(0,-1)*HR2(0,1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR2(0,-1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR2(0,-1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR2(0,-1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR2(0,-1)*HR3(0,-1,1)
     &  - 1.3862943611198906d+00*HR2(0,-1)*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR2(0,-1)*HR3(0,1,1)
     &  + 4.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,0,1,1)
     &  - 4.0000000000000000d+00*HR2(0,-1)*HR4(0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR4(0,1,1,-1)
     &  - 1.9355535381306524d-01*HR2(0,1)
     &  + 2.9112026323250625d-01*HR2(0,1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR2(0,1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(0,-1,1)
     &  - 4.0000000000000000d+00*HR2(0,1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR2(0,1)*HR4(0,-1,-1,1)
     &  + HR2(0,1) *HR4(0,-1,0,1)
     &  - HR2(0,1) *HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR2(0,1)*HR4(0,0,-1,-1)
     &  - 2.9560095330860840d+00*HR3(-1,-1,1)
     &  + 4.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  + 4.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,-1)
     &  + 2.9560095330860840d+00*HR3(-1,1,1)
     &  - 4.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  - 4.0000000000000000d+00*HR3(-1,1,1)*HR3(0,0,-1)
     &  - 2.9560095330860840d+00*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + 4.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,-1,1)
     &  + 4.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,0,-1)
     &  + 4.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,1,-1)
     &  + 4.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,1,1)
     &  - 2.9560095330860840d+00*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR3(0,-1,1)*HR3(0,-1,1)
     &  + 4.0000000000000000d+00*HR3(0,-1,1)*HR3(0,0,-1)
     &  + 4.0000000000000000d+00*HR3(0,0,-1)*HR3(0,1,-1)
     &  + 4.0000000000000000d+00*HR3(0,0,-1)*HR3(0,1,1)
     &  - 2.9560095330860840d+00*HR3(0,1,-1)
     &  - 2.9560095330860840d+00*HR3(0,1,1)
     &  - 2.3289621058600500d+00*HR4(-1,-1,-1,1)
     &  + 1.1644810529300250d+00*HR4(-1,-1,1,1)
     &  + 2.3289621058600500d+00*HR4(0,-1,-1,-1)
     &  + 1.1644810529300250d+00*HR4(0,-1,-1,1)
     &  + 1.1644810529300250d+00*HR4(0,-1,1,-1)
     &  + 1.1644810529300250d+00*HR4(0,0,-1,-1)
     &  + 1.1644810529300250d+00*HR4(0,0,-1,1)
     &  + 1.1644810529300250d+00*HR4(0,0,1,-1)
     &  + 1.1644810529300250d+00*HR4(0,0,1,1)
     &  + 2.3289621058600500d+00*HR4(0,1,-1,-1)
     &  + 1.1644810529300250d+00*HR4(0,1,-1,1)
     &  + 1.1644810529300250d+00*HR4(0,1,1,-1)
     &  + 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  + 5.5451774444795624d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.7725887222397812d+00*HR5(0,-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,0,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  + 2.7725887222397812d+00*HR5(0,-1,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,1,0,1)
     &  + 8.3177661667193437d+00*HR5(0,0,-1,-1,-1)
     &  + 2.7725887222397812d+00*HR5(0,0,-1,-1,1)
     &  + 2.7725887222397812d+00*HR5(0,0,-1,1,-1)
     &  + 2.7725887222397812d+00*HR5(0,0,1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,0,1,-1,1)
     &  + 2.7725887222397812d+00*HR5(0,0,1,1,-1)
     &  + 4.8520302639196171d+00*HR5(0,1,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,0,1,-1)
     &  + 1.3862943611198906d+00*HR5(0,1,1,-1,-1)
     &  + 1.1000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  + HR6( -1,-1,-1,1,-1,1)
     &  - 1.6000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  - 8.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,0,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,0,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  + HR6(0, -1,0,-1,0,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,1,1)
     &  - HR6(0, -1,0,1,-1,-1)
     &  - HR6(0, -1,0,1,0,1)
     &  - 5.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,-1,-1,1)
     &  + HR6(0, -1,1,-1,0,1)
     &  - HR6(0, -1,1,-1,1,-1)
     &  + HR6(0, -1,1,0,1,-1)
     &  - 8.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  - 1.2000000000000000d+01*HR6(0,0,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,0,-1,-1,0,1)
     &  - 8.0000000000000000d+00*HR6(0,0,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,0,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,0,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,0,1,-1)
     &  - 6.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,1,0,-1)
     &  - 1.2000000000000000d+01*HR6(0,0,0,-1,-1,-1)
     &  - 1.2000000000000000d+01*HR6(0,0,0,-1,-1,1)
     &  - 1.2000000000000000d+01*HR6(0,0,0,-1,1,-1)
     &  - 1.2000000000000000d+01*HR6(0,0,0,1,-1,-1)
     &  - 6.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,1,-1,0,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,1,0,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,1,1,-1,-1)
     &  - 1.1000000000000000d+01*HR6(0,1,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  - HR6(0,1,0,1, -1,-1)
     &  - 2.0000000000000000d+00*HR6(0,1,1,-1,-1,-1)
      HY6(0,1,0,1,1,-1) = 
     &  + 5.1634546570914239d-02
     &  + 1.6987486918970417d+00*HR1(-1)
     &  - 2.1684618852447759d-01*HR1(-1)*HR1(-1)
     &  + 1.8455064982157776d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 5.5365194946473328d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 4.3369237704895519d-01*HR1(-1)*HR1(0)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,0,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  - 4.3369237704895519d-01*HR1(-1)*HR1(1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR2(-1,1)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR2(0,-1)*HR2(0,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,0,1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,-1,1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  - 1.1073038989294665d+00*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(-1,1)*HR3(0,0,-1)
     &  - 1.1073038989294665d+00*HR1(-1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR2(0,-1)*HR3(0,1,-1)
     &  - 1.1073038989294665d+00*HR1(-1)*HR2(0,1)
     &  + HR1( -1)*HR2(0,1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(0,1)*HR3(0,0,-1)
     &  - 2.1253870807664278d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 2.1253870807664278d+00*HR1(-1)*HR3(-1,1,1)
     &  + 2.1253870807664278d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 2.1253870807664278d+00*HR1(-1)*HR3(0,0,-1)
     &  + 2.1253870807664278d+00*HR1(-1)*HR3(0,0,1)
     &  + 2.1253870807664278d+00*HR1(-1)*HR3(0,1,-1)
     &  - 2.7725887222397812d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,0,0,-1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,0,0,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,0,1,-1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,-1,0,1)
     &  - HR1( -1)*HR5(0,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,0,-1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,1)
     &  + HR1( -1)*HR5(0,-1,0,1,-1)
     &  + 9.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,0,-1)
     &  - HR1( -1)*HR5(0,0,-1,0,1)
     &  + HR1( -1)*HR5(0,0,-1,1,-1)
     &  - 6.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,0,1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,0,1,0,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(0,1,1)
     &  + 4.3369237704895519d-01*HR1(0)*HR1(1)
     &  + 1.0626935403832139d+00*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 1.0626935403832139d+00*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR1(0)*HR1(1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,0,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(0,-1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR2(0,1)*HR2(0,1)
     &  + 2.1253870807664278d+00*HR1(0)*HR3(-1,-1,1)
     &  - 2.1253870807664278d+00*HR1(0)*HR3(-1,1,1)
     &  + 2.1253870807664278d+00*HR1(0)*HR3(0,-1,-1)
     &  + 2.1253870807664278d+00*HR1(0)*HR3(0,-1,1)
     &  + 2.1253870807664278d+00*HR1(0)*HR3(0,1,-1)
     &  + 2.1253870807664278d+00*HR1(0)*HR3(0,1,1)
     &  + 2.7725887222397812d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(-1,-1,1,1)
     &  - 2.7725887222397812d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,0,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,0,1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,0,1,1)
     &  - 2.7725887222397812d+00*HR1(0)*HR4(0,1,-1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR4(0,1,1,-1)
     &  + 1.6987486918970417d+00*HR1(1)
     &  + 1.1073038989294665d+00*HR1(1)*HR2(-1,1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(-1,1)*HR3(0,0,-1)
     &  - 1.1073038989294665d+00*HR1(1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR2(0,-1)*HR3(0,1,-1)
     &  - 1.1073038989294665d+00*HR1(1)*HR2(0,1)
     &  + HR1(1) *HR2(0,1)*HR3(0,-1,-1)
     &  + HR1(1) *HR2(0,1)*HR3(0,0,-1)
     &  + 2.1253870807664278d+00*HR1(1)*HR3(-1,-1,1)
     &  + 2.1253870807664278d+00*HR1(1)*HR3(0,-1,-1)
     &  + 2.1253870807664278d+00*HR1(1)*HR3(0,0,-1)
     &  + 2.1253870807664278d+00*HR1(1)*HR3(0,0,1)
     &  + 2.1253870807664278d+00*HR1(1)*HR3(0,1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,0,0,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,0,0,1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,0,1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - HR1(1) *HR5(0,-1,-1,0,1)
     &  - HR1(1) *HR5(0,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,0,-1,-1)
     &  - HR1(1) *HR5(0,-1,0,-1,1)
     &  + HR1(1) *HR5(0,-1,0,1,-1)
     &  + 9.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,-1,0,-1)
     &  - HR1(1) *HR5(0,0,-1,0,1)
     &  + HR1(1) *HR5(0,0,-1,1,-1)
     &  - 6.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,0,1,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,0,1,0,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  + 4.3369237704895519d-01*HR2(-1,1)
     &  - 5.3134677019160696d-01*HR2(-1,1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR2(-1,1)*HR2(-1,1)*HR2(0,-1)
     &  + 1.0626935403832139d+00*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(0,-1)*HR2(0,-1)
     &  - HR2( -1,1)*HR2(0,-1)*HR2(0,1)
     &  + 1.0626935403832139d+00*HR2(-1,1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,1,-1)
     &  - HR2( -1,1)*HR4(-1,-1,-1,1)
     &  - HR2( -1,1)*HR4(0,-1,-1,1)
     &  + HR2( -1,1)*HR4(0,-1,0,1)
     &  + HR2( -1,1)*HR4(0,0,-1,1)
     &  + HR2( -1,1)*HR4(0,0,1,-1)
     &  + HR2( -1,1)*HR4(0,1,-1,-1)
     &  - 4.3369237704895519d-01*HR2(0,-1)
     &  - 5.3134677019160696d-01*HR2(0,-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR2(0,-1)*HR2(0,-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR2(0,-1)*HR2(0,-1)*HR2(0,1)
     &  - 1.0626935403832139d+00*HR2(0,-1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR2(0,-1)*HR2(0,1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,0,1)
     &  - 4.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,-1,1,-1)
     &  - HR2(0, -1)*HR4(0,0,-1,1)
     &  - HR2(0, -1)*HR4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,0,1,1)
     &  + 4.0000000000000000d+00*HR2(0,-1)*HR4(0,1,-1,-1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,1,1,-1)
     &  - 4.3369237704895519d-01*HR2(0,1)
     &  - 5.3134677019160696d-01*HR2(0,1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(0,0,1)
     &  + HR2(0,1) *HR4(0,-1,-1,1)
     &  - HR2(0,1) *HR4(0,-1,0,1)
     &  - HR2(0,1) *HR4(0,0,-1,1)
     &  - HR2(0,1) *HR4(0,0,1,-1)
     &  + 2.2146077978589331d+00*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,-1)
     &  - 2.2146077978589331d+00*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR3(-1,1,1)*HR3(0,0,-1)
     &  + 2.2146077978589331d+00*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,1,1)
     &  + 2.2146077978589331d+00*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR3(0,-1,1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR3(0,0,-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR3(0,0,-1)*HR3(0,1,1)
     &  + 2.2146077978589331d+00*HR3(0,1,-1)
     &  + 2.2146077978589331d+00*HR3(0,1,1)
     &  + 4.2507741615328557d+00*HR4(-1,-1,-1,1)
     &  - 2.1253870807664278d+00*HR4(-1,-1,1,1)
     &  - 4.2507741615328557d+00*HR4(0,-1,-1,-1)
     &  - 2.1253870807664278d+00*HR4(0,-1,-1,1)
     &  - 2.1253870807664278d+00*HR4(0,-1,1,-1)
     &  - 2.1253870807664278d+00*HR4(0,0,-1,-1)
     &  - 2.1253870807664278d+00*HR4(0,0,-1,1)
     &  - 2.1253870807664278d+00*HR4(0,0,1,-1)
     &  - 2.1253870807664278d+00*HR4(0,0,1,1)
     &  - 4.2507741615328557d+00*HR4(0,1,-1,-1)
     &  - 2.1253870807664278d+00*HR4(0,1,-1,1)
     &  - 2.1253870807664278d+00*HR4(0,1,1,-1)
     &  + 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  + 4.8520302639196171d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,0,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,1,-1,-1)
     &  + 4.8520302639196171d+00*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,0,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,0,1)
     &  + 1.3862943611198906d+00*HR5(0,0,-1,1,-1)
     &  + 3.4657359027997265d+00*HR5(0,0,1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,0,1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,0,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,0,1)
     &  + 2.7725887222397812d+00*HR5(0,0,1,1,-1)
     &  + 4.8520302639196171d+00*HR5(0,1,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,0,1,-1)
     &  + 1.3862943611198906d+00*HR5(0,1,1,-1,-1)
     &  + 1.1000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  + HR6( -1,-1,-1,1,-1,1)
     &  - 1.0000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,1,-1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  + HR6(0, -1,-1,1,-1,1)
     &  - HR6(0, -1,-1,1,0,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  - HR6(0, -1,0,-1,0,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,1,-1)
     &  - HR6(0, -1,0,1,-1,-1)
     &  - HR6(0, -1,0,1,-1,1)
     &  + HR6(0, -1,0,1,0,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  - 1.6000000000000000d+01*HR6(0,0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,0,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,-1,0,1)
     &  + HR6(0,0, -1,-1,1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,-1,0,-1,-1)
     &  + 5.0000000000000000d+00*HR6(0,0,-1,0,-1,1)
     &  + 5.0000000000000000d+00*HR6(0,0,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  - HR6(0,0, -1,1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,1,0,-1)
     &  + HR6(0,0, -1,1,0,1)
     &  + 1.2000000000000000d+01*HR6(0,0,0,-1,-1,-1)
     &  + 1.2000000000000000d+01*HR6(0,0,0,-1,-1,1)
     &  + 1.2000000000000000d+01*HR6(0,0,0,-1,1,-1)
     &  + 1.2000000000000000d+01*HR6(0,0,0,1,-1,-1)
     &  - 9.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,0,1,-1,0,-1)
     &  + HR6(0,0,1, -1,0,1)
     &  - 3.0000000000000000d+00*HR6(0,0,1,-1,1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,1,0,-1,-1)
     &  + HR6(0,0,1,0, -1,1)
     &  + HR6(0,0,1,0,1, -1)
     &  - 4.0000000000000000d+00*HR6(0,0,1,1,-1,-1)
     &  - 1.1000000000000000d+01*HR6(0,1,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  - HR6(0,1,0,1, -1,-1)
     &  - 2.0000000000000000d+00*HR6(0,1,1,-1,-1,-1)
      HY6(0,1,0,1,1,1) = 
     &  + 8.8483382454368714d-02
     &  + 4.8979203184748778d+00*HR1(-1)
     &  - 5.4116161685556909d-01*HR1(-1)*HR1(-1)
     &  - 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  - HR1( -1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR4(-1,-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,1,-1,-1)
     &  + 1.0823232337111381d+00*HR1(-1)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,1)
     &  + HR1( -1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(0,0,1)
     &  - HR1( -1)*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,0,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,0,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,0,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,1,-1,-1)
     &  - 1.0823232337111381d+00*HR1(-1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,0,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,0,1)
     &  + HR1( -1)*HR1(1)*HR4(0,0,1,-1)
     &  + HR1( -1)*HR1(1)*HR4(0,1,-1,-1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(-1,-1,1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR2(0,-1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR2(0,1)
     &  + 4.8045301391820142d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(-1,1,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,0,-1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,0,1)
     &  - 4.8045301391820142d-01*HR1(-1)*HR3(0,1,-1)
     &  + 2.7725887222397812d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,0,0,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,0,0,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,0,1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + 7.0000000000000000d+00*HR1(-1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,0,0,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,0,1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,1,-1,-1,-1)
     &  - 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR2(0,1)
     &  - 3.3333333333333333d-01*HR1(0)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 3.3333333333333333d-01*HR1(0)*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  - 3.3333333333333333d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 3.3333333333333333d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  - 3.3333333333333333d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  - 3.3333333333333333d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,1)
     &  - HR1(0) *HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(0)*HR1(1)*HR3(0,0,-1)
     &  - HR1(0) *HR1(0)*HR1(1)*HR3(0,0,1)
     &  - HR1(0) *HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + 2.5000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)*HR2(0,1)
     &  + 2.5000000000000000d-01*HR1(0)*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(0,-1)*HR2(0,1)
     &  + 2.5000000000000000d-01*HR1(0)*HR1(0)*HR2(0,1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(0)*HR3(0,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(0)*HR4(-1,-1,-1,1)
     &  + HR1(0) *HR1(0)*HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR1(0)*HR4(0,-1,-1,1)
     &  + HR1(0) *HR1(0)*HR4(0,-1,1,-1)
     &  + HR1(0) *HR1(0)*HR4(0,0,-1,-1)
     &  + HR1(0) *HR1(0)*HR4(0,0,-1,1)
     &  + HR1(0) *HR1(0)*HR4(0,0,1,-1)
     &  + HR1(0) *HR1(0)*HR4(0,0,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(0)*HR4(0,1,-1,-1)
     &  + HR1(0) *HR1(0)*HR4(0,1,-1,1)
     &  + HR1(0) *HR1(0)*HR4(0,1,1,-1)
     &  + 1.0823232337111381d+00*HR1(0)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR1(1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,0,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,0,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,1,-1,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,1)
     &  + HR1(0) *HR2(-1,1)*HR3(-1,-1,1)
     &  + HR1(0) *HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(0) *HR2(-1,1)*HR3(0,0,-1)
     &  + HR1(0) *HR2(-1,1)*HR3(0,0,1)
     &  + HR1(0) *HR2(-1,1)*HR3(0,1,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(0,-1)*HR2(0,1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,0,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,0,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR2(0,1)*HR2(0,1)
     &  - HR1(0) *HR2(0,1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(0,1)*HR3(0,0,-1)
     &  - HR1(0) *HR2(0,1)*HR3(0,0,1)
     &  - 4.8045301391820142d-01*HR1(0)*HR3(-1,-1,1)
     &  + 4.8045301391820142d-01*HR1(0)*HR3(-1,1,1)
     &  - 4.8045301391820142d-01*HR1(0)*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR1(0)*HR3(0,-1,1)
     &  - 4.8045301391820142d-01*HR1(0)*HR3(0,1,-1)
     &  - 4.8045301391820142d-01*HR1(0)*HR3(0,1,1)
     &  - 2.7725887222397812d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(-1,-1,1,1)
     &  + 2.7725887222397812d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,0,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,0,1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,0,1,1)
     &  + 2.7725887222397812d+00*HR1(0)*HR4(0,1,-1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR4(0,1,1,-1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(-1,-1,-1,-1,1)
     &  - HR1(0) *HR5(-1,-1,1,-1,1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  + HR1(0) *HR5(0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,1,-1)
     &  - HR1(0) *HR5(0,-1,0,-1,-1)
     &  + HR1(0) *HR5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,1,-1,-1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,-1)
     &  - HR1(0) *HR5(0,0,-1,-1,1)
     &  + HR1(0) *HR5(0,0,-1,0,-1)
     &  + HR1(0) *HR5(0,0,-1,0,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,0,-1,1,-1)
     &  - 5.0000000000000000d+00*HR1(0)*HR5(0,0,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,0,1,-1,1)
     &  + HR1(0) *HR5(0,0,1,0,-1)
     &  + HR1(0) *HR5(0,0,1,0,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR5(0,0,1,1,-1)
     &  - 7.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,1,-1,1,-1)
     &  - HR1(0) *HR5(0,1,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,1,1,-1,-1)
     &  + 4.8979203184748778d+00*HR1(1)
     &  - 5.5504108664821579d-02*HR1(1)*HR2(-1,1)
     &  + 5.5504108664821579d-02*HR1(1)*HR2(0,-1)
     &  + 5.5504108664821579d-02*HR1(1)*HR2(0,1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,-1,-1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,0,-1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,0,1)
     &  - 4.8045301391820142d-01*HR1(1)*HR3(0,1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,0,-1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,0,0,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,0,0,1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,0,1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,0,0,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,0,1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,1,-1,-1,-1)
     &  + 1.0823232337111381d+00*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR2(-1,1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR2(-1,1)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR2(-1,1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,1,-1)
     &  + HR2( -1,1)*HR4(-1,-1,-1,1)
     &  - HR2( -1,1)*HR4(0,-1,-1,-1)
     &  - HR2( -1,1)*HR4(0,0,-1,-1)
     &  - HR2( -1,1)*HR4(0,0,0,-1)
     &  - HR2( -1,1)*HR4(0,0,0,1)
     &  - HR2( -1,1)*HR4(0,0,1,-1)
     &  - HR2( -1,1)*HR4(0,1,-1,-1)
     &  - 1.0823232337111381d+00*HR2(0,-1)
     &  + 1.2011325347955035d-01*HR2(0,-1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR2(0,-1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,0,1)
     &  + HR2(0, -1)*HR4(0,0,-1,-1)
     &  + HR2(0, -1)*HR4(0,0,0,-1)
     &  + HR2(0, -1)*HR4(0,0,0,1)
     &  + HR2(0, -1)*HR4(0,0,1,-1)
     &  - 1.0823232337111381d+00*HR2(0,1)
     &  + 1.2011325347955035d-01*HR2(0,1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(0,0,1)
     &  + HR2(0,1) *HR4(0,-1,-1,-1)
     &  + HR2(0,1) *HR4(0,0,-1,-1)
     &  + HR2(0,1) *HR4(0,0,0,-1)
     &  + HR2(0,1) *HR4(0,0,0,1)
     &  + HR2(0,1) *HR4(0,0,1,-1)
     &  - 1.1100821732964315d-01*HR3(-1,-1,1)
     &  + 1.1100821732964315d-01*HR3(-1,1,1)
     &  - 1.1100821732964315d-01*HR3(0,-1,-1)
     &  - 1.1100821732964315d-01*HR3(0,-1,1)
     &  - 1.1100821732964315d-01*HR3(0,1,-1)
     &  - 1.1100821732964315d-01*HR3(0,1,1)
     &  - 9.6090602783640284d-01*HR4(-1,-1,-1,1)
     &  + 4.8045301391820142d-01*HR4(-1,-1,1,1)
     &  + 9.6090602783640284d-01*HR4(0,-1,-1,-1)
     &  + 4.8045301391820142d-01*HR4(0,-1,-1,1)
     &  + 4.8045301391820142d-01*HR4(0,-1,1,-1)
     &  + 4.8045301391820142d-01*HR4(0,0,-1,-1)
     &  + 4.8045301391820142d-01*HR4(0,0,-1,1)
     &  + 4.8045301391820142d-01*HR4(0,0,1,-1)
     &  + 4.8045301391820142d-01*HR4(0,0,1,1)
     &  + 9.6090602783640284d-01*HR4(0,1,-1,-1)
     &  + 4.8045301391820142d-01*HR4(0,1,-1,1)
     &  + 4.8045301391820142d-01*HR4(0,1,1,-1)
     &  - 4.8520302639196171d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,1,-1,1)
     &  - 4.8520302639196171d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,0,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,1,-1,-1)
     &  - 4.8520302639196171d+00*HR5(0,0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,0,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,0,1)
     &  - 1.3862943611198906d+00*HR5(0,0,-1,1,-1)
     &  - 3.4657359027997265d+00*HR5(0,0,1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,0,1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,0,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,0,1)
     &  - 2.7725887222397812d+00*HR5(0,0,1,1,-1)
     &  - 4.8520302639196171d+00*HR5(0,1,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,0,1,-1)
     &  - 1.3862943611198906d+00*HR5(0,1,1,-1,-1)
     &  - 1.1000000000000000d+01*HR6(-1,-1,-1,-1,-1,1)
     &  - HR6( -1,-1,-1,1,-1,1)
     &  + 1.1000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,0,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,0,-1,1)
     &  - HR6(0, -1,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,1,-1,-1)
     &  + HR6(0, -1,0,-1,-1,-1)
     &  - HR6(0, -1,0,-1,-1,1)
     &  - HR6(0, -1,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,1,-1,-1,-1)
     &  + 1.1000000000000000d+01*HR6(0,0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  - HR6(0,0, -1,-1,0,-1)
     &  - HR6(0,0, -1,-1,0,1)
     &  + HR6(0,0, -1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,0,-1,-1)
     &  - HR6(0,0, -1,0,-1,1)
     &  - HR6(0,0, -1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,1,-1,-1)
     &  + HR6(0,0,0, -1,-1,-1)
     &  + HR6(0,0,0, -1,-1,1)
     &  - HR6(0,0,0, -1,0,-1)
     &  - HR6(0,0,0, -1,0,1)
     &  + HR6(0,0,0,1, -1,-1)
     &  + HR6(0,0,0,1, -1,1)
     &  - HR6(0,0,0,1,0, -1)
     &  - HR6(0,0,0,1,0,1) 
     &  + 9.0000000000000000d+00*HR6(0,0,1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,0,1,-1,-1,1)
     &  - HR6(0,0,1, -1,0,-1)
     &  - HR6(0,0,1, -1,0,1)
     &  + 3.0000000000000000d+00*HR6(0,0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,1,0,-1,-1)
     &  - HR6(0,0,1,0, -1,1)
     &  - HR6(0,0,1,0,1, -1)
     &  + 4.0000000000000000d+00*HR6(0,0,1,1,-1,-1)
     &  + 1.1000000000000000d+01*HR6(0,1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,1,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,1,-1,1,-1,-1)
     &  + HR6(0,1,0,1, -1,-1)
     &  + 2.0000000000000000d+00*HR6(0,1,1,-1,-1,-1)
      HY6(0,1,1,-1,-1,-1) = 
     &  + 1.8032999288881844d-02
     &  - 1.7923398046073090d-01*HR1(-1)
     &  + 6.3399291898262058d-02*HR1(-1)*HR1(-1)
     &  - 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 1.2679858379652411d-01*HR1(-1)*HR1(0)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(0)*HR1(0)
     &  + 5.5504108664821579d-02*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.2679858379652411d-01*HR1(-1)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,-1)
     &  - 2.7752054332410789d-02*HR1(0)*HR1(0)*HR1(1)
     &  - 1.2679858379652411d-01*HR1(0)*HR1(1)
     &  - 5.5504108664821579d-02*HR1(0)*HR2(-1,1)
     &  + 5.5504108664821579d-02*HR1(0)*HR2(0,-1)
     &  + 5.5504108664821579d-02*HR1(0)*HR2(0,1)
     &  - 1.7923398046073090d-01*HR1(1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,0,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,-1)
     &  - 1.2679858379652411d-01*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR2( -1,1)*HR4(0,-1,-1,-1)
     &  + 1.2679858379652411d-01*HR2(0,-1)
     &  - 1.2011325347955035d-01*HR2(0,-1)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR2(0,-1)*HR2(0,1)
     &  + 1.2679858379652411d-01*HR2(0,1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(0,-1,-1)
     &  - HR2(0,1) *HR4(0,-1,-1,-1)
     &  - 5.5504108664821579d-02*HR3(-1,-1,1)
     &  - 5.5504108664821579d-02*HR3(0,-1,-1)
     &  - 5.5504108664821579d-02*HR3(0,0,-1)
     &  - 5.5504108664821579d-02*HR3(0,0,1)
     &  - 5.5504108664821579d-02*HR3(0,1,-1)
     &  - 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,1)
     &  + 2.4022650695910071d-01*HR4(0,-1,0,1)
     &  + 2.4022650695910071d-01*HR4(0,0,-1,1)
     &  + 2.4022650695910071d-01*HR4(0,0,1,-1)
     &  + 2.4022650695910071d-01*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,0,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,1,-1)
     &  + 2.0794415416798359d+00*HR5(0,0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  - 5.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + HR6(0, -1,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + HR6(0, -1,-1,0,-1,1)
     &  + HR6(0, -1,-1,0,1,-1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  - HR6(0, -1,0,-1,-1,-1)
     &  + HR6(0, -1,0,-1,-1,1)
     &  + HR6(0, -1,0,-1,1,-1)
     &  + HR6(0, -1,0,1,-1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + HR6(0,0, -1,-1,-1,1)
     &  + HR6(0,0, -1,-1,1,-1)
     &  + HR6(0,0, -1,1,-1,-1)
     &  + HR6(0,0,1, -1,-1,-1)
     &  + HR6(0,1, -1,-1,-1,-1)
      HY6(0,1,1,-1,-1,1) = 
     &  + 2.6632640235101822d-02
     &  - 3.5856690288675007d-01*HR1(-1)
     &  + 1.1758989653041252d-01*HR1(-1)*HR1(-1)
     &  - 1.5792167371687950d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 4.7376502115063852d-02*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 4.7376502115063852d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.3517979306082505d-01*HR1(-1)*HR1(0)
     &  - 4.7376502115063852d-02*HR1(-1)*HR1(0)*HR1(0)
     &  + 9.4753004230127705d-02*HR1(-1)*HR1(0)*HR1(1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(0)*HR4(0,0,-1,-1)
     &  + 2.3517979306082505d-01*HR1(-1)*HR1(1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 9.4753004230127705d-02*HR1(-1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,-1,0,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,-1)
     &  - HR1( -1)*HR5(0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,-1)
     &  - 4.7376502115063852d-02*HR1(0)*HR1(0)*HR1(1)
     &  - 2.3517979306082505d-01*HR1(0)*HR1(1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR1(1)*HR4(0,0,-1,-1)
     &  - 9.4753004230127705d-02*HR1(0)*HR2(-1,1)
     &  + HR1(0) *HR2(-1,1)*HR3(0,-1,-1)
     &  + 9.4753004230127705d-02*HR1(0)*HR2(0,-1)
     &  + 9.4753004230127705d-02*HR1(0)*HR2(0,1)
     &  - HR1(0) *HR2(0,1)*HR3(0,-1,-1)
     &  + HR1(0) *HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  + HR1(0) *HR5(0,-1,-1,0,1)
     &  - HR1(0) *HR5(0,-1,-1,1,-1)
     &  - HR1(0) *HR5(0,-1,0,-1,-1)
     &  + HR1(0) *HR5(0,-1,0,-1,1)
     &  + HR1(0) *HR5(0,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,-1)
     &  + HR1(0) *HR5(0,0,-1,-1,1)
     &  + HR1(0) *HR5(0,0,-1,1,-1)
     &  + HR1(0) *HR5(0,0,1,-1,-1)
     &  + HR1(0) *HR5(0,1,-1,-1,-1)
     &  - 3.5856690288675007d-01*HR1(1)
     &  - HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(1)*HR3(0,0,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,-1,0,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,-1)
     &  - HR1(1) *HR5(0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,-1)
     &  - 2.3517979306082505d-01*HR2(-1,1)
     &  + 5.8224052646501250d-01*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR2(-1,1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.3517979306082505d-01*HR2(0,-1)
     &  - 2.9112026323250625d-01*HR2(0,-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR2(0,-1)*HR2(0,-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR2(0,-1)*HR2(0,-1)*HR2(0,1)
     &  - 5.8224052646501250d-01*HR2(0,-1)*HR2(0,1)
     &  + HR2(0, -1)*HR4(-1,-1,-1,1)
     &  + HR2(0, -1)*HR4(0,-1,-1,1)
     &  - HR2(0, -1)*HR4(0,-1,0,1)
     &  - HR2(0, -1)*HR4(0,1,-1,-1)
     &  + 2.3517979306082505d-01*HR2(0,1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(0,-1,-1)
     &  - 9.4753004230127705d-02*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  - 9.4753004230127705d-02*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,1,-1)
     &  - 9.4753004230127705d-02*HR3(0,0,-1)
     &  - 9.4753004230127705d-02*HR3(0,0,1)
     &  - 9.4753004230127705d-02*HR3(0,1,-1)
     &  - 5.8224052646501250d-01*HR4(-1,-1,-1,1)
     &  - 5.8224052646501250d-01*HR4(0,-1,-1,1)
     &  + 5.8224052646501250d-01*HR4(0,-1,0,1)
     &  + 5.8224052646501250d-01*HR4(0,0,-1,1)
     &  + 5.8224052646501250d-01*HR4(0,0,1,-1)
     &  + 5.8224052646501250d-01*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,0,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,1,-1)
     &  - 2.0794415416798359d+00*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - HR6(0, -1,-1,0,1,-1)
     &  + HR6(0, -1,-1,1,-1,-1)
     &  - HR6(0, -1,0,-1,-1,1)
     &  + HR6(0, -1,0,-1,0,1)
     &  - HR6(0, -1,0,-1,1,-1)
     &  - HR6(0, -1,0,1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  - HR6(0,0, -1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,0,1)
     &  - HR6(0,0, -1,-1,1,-1)
     &  + HR6(0,0, -1,0,-1,-1)
     &  - HR6(0,0, -1,0,-1,1)
     &  - HR6(0,0, -1,0,1,-1)
     &  - HR6(0,0, -1,1,-1,-1)
     &  - HR6(0,0, -1,1,0,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,0,0,-1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,0,1,-1,-1)
     &  - HR6(0,0,1, -1,-1,-1)
     &  - HR6(0,0,1, -1,0,-1)
     &  - HR6(0,0,1,0, -1,-1)
     &  - HR6(0,1, -1,-1,-1,-1)
      HY6(0,1,1,-1,1,-1) = 
     &  + 4.4179246042660315d-02
     &  - 1.1958047166863888d+00*HR1(-1)
     &  + 3.2105039383616431d-01*HR1(-1)*HR1(-1)
     &  - 3.5678728477784371d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.0703618543335311d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 1.0703618543335311d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 6.4210078767232862d-01*HR1(-1)*HR1(0)
     &  - 1.0703618543335311d-01*HR1(-1)*HR1(0)*HR1(0)
     &  + 2.1407237086670622d-01*HR1(-1)*HR1(0)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 6.4210078767232862d-01*HR1(-1)*HR1(1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 2.1407237086670622d-01*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR2(0,-1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  + 2.0794415416798359d+00*HR1(-1)*HR4(0,0,0,-1)
     &  - HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - HR1( -1)*HR5(0,-1,0,-1,-1)
     &  - 7.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,0,-1)
     &  - 1.0703618543335311d-01*HR1(0)*HR1(0)*HR1(1)
     &  - 6.4210078767232862d-01*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  - 2.1407237086670622d-01*HR1(0)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + 2.1407237086670622d-01*HR1(0)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(0,-1)*HR2(0,1)
     &  + 2.1407237086670622d-01*HR1(0)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,0,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,-1)
     &  - 1.1958047166863888d+00*HR1(1)
     &  + HR1(1) *HR2(0,-1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(1)*HR3(0,0,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,0,-1,-1)
     &  + 2.0794415416798359d+00*HR1(1)*HR4(0,0,0,-1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - HR1(1) *HR5(0,-1,0,-1,-1)
     &  - 7.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,0,-1)
     &  - 6.4210078767232862d-01*HR2(-1,1)
     &  - 1.0626935403832139d+00*HR2(-1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR2(-1,1)*HR2(0,-1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR2(-1,1)*HR3(0,0,-1)
     &  - 4.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,-1)
     &  + 6.4210078767232862d-01*HR2(0,-1)
     &  + 5.3134677019160696d-01*HR2(0,-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR2(0,-1)*HR2(0,-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR2(0,-1)*HR2(0,-1)*HR2(0,1)
     &  + 1.0626935403832139d+00*HR2(0,-1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR2(0,-1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,-1)
     &  - HR2(0, -1)*HR4(-1,-1,-1,1)
     &  - HR2(0, -1)*HR4(0,-1,-1,1)
     &  + HR2(0, -1)*HR4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR4(0,0,-1,-1)
     &  + HR2(0, -1)*HR4(0,1,-1,-1)
     &  + 6.4210078767232862d-01*HR2(0,1)
     &  - 1.3862943611198906d+00*HR2(0,1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR2(0,1)*HR3(0,0,-1)
     &  + 4.0000000000000000d+00*HR2(0,1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR2(0,1)*HR4(0,0,-1,-1)
     &  - 2.1407237086670622d-01*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  - 2.1407237086670622d-01*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,1,-1)
     &  - 2.1407237086670622d-01*HR3(0,0,-1)
     &  - 2.1407237086670622d-01*HR3(0,0,1)
     &  - 2.1407237086670622d-01*HR3(0,1,-1)
     &  + 1.0626935403832139d+00*HR4(-1,-1,-1,1)
     &  + 1.0626935403832139d+00*HR4(0,-1,-1,1)
     &  - 1.0626935403832139d+00*HR4(0,-1,0,1)
     &  - 1.0626935403832139d+00*HR4(0,0,-1,1)
     &  - 1.0626935403832139d+00*HR4(0,0,1,-1)
     &  - 1.0626935403832139d+00*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,0,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  - 2.0794415416798359d+00*HR5(0,-1,0,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,-1,0,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,1,-1)
     &  - 6.2383246250395077d+00*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,0,-1,0,-1)
     &  + 1.3862943611198906d+00*HR5(0,0,-1,0,1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,1,-1)
     &  + 4.1588830833596718d+00*HR5(0,0,0,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,0,0,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,0,0,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,0,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  + 1.0000000000000000d+01*HR6(0,-1,-1,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,-1,0,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,-1,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,-1,0,1,-1)
     &  + HR6(0, -1,-1,1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  - HR6(0, -1,0,-1,0,1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,1,-1)
     &  - HR6(0, -1,0,1,-1,-1)
     &  + 1.6000000000000000d+01*HR6(0,0,-1,-1,-1,-1)
     &  - HR6(0,0, -1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,-1,0,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,-1,0,1)
     &  - HR6(0,0, -1,-1,1,-1)
     &  - 6.0000000000000000d+00*HR6(0,0,-1,0,-1,-1)
     &  - HR6(0,0, -1,0,-1,1)
     &  - HR6(0,0, -1,0,1,-1)
     &  - HR6(0,0, -1,1,-1,-1)
     &  + HR6(0,0, -1,1,0,-1)
     &  - 1.2000000000000000d+01*HR6(0,0,0,-1,-1,-1)
     &  - HR6(0,0,1, -1,-1,-1)
     &  + HR6(0,0,1, -1,0,-1)
     &  - HR6(0,1, -1,-1,-1,-1)
      HY6(0,1,1,-1,1,1) = 
     &  + 7.7939361164851325d-02
     &  - 4.2555237822406230d+00*HR1(-1)
     &  + 9.6240249776535762d-01*HR1(-1)*HR1(-1)
     &  - 8.9535532268006700d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 2.6860659680402010d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  - 2.6860659680402010d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  - 1.9248049955307152d+00*HR1(-1)*HR1(0)
     &  - 2.6860659680402010d-01*HR1(-1)*HR1(0)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(0,0,-1)
     &  + 5.3721319360804020d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(0)*HR4(0,0,0,-1)
     &  + 1.9248049955307152d+00*HR1(-1)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,0,-1)
     &  + 5.3721319360804020d-01*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 2.0794415416798359d+00*HR1(-1)*HR4(0,0,0,-1)
     &  + HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,-1)
     &  + 6.0000000000000000d+00*HR1(-1)*HR5(0,0,0,0,-1)
     &  - 2.6860659680402010d-01*HR1(0)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(0)*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR2(0,-1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,0,1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,1,-1,-1)
     &  - 1.9248049955307152d+00*HR1(0)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR1(1)*HR4(0,0,0,-1)
     &  - 5.3721319360804020d-01*HR1(0)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR2(-1,1)*HR3(0,0,-1)
     &  + 5.3721319360804020d-01*HR1(0)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR2(0,-1)*HR2(0,1)
     &  - HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(0,-1)*HR3(0,0,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,1,-1)
     &  + 5.3721319360804020d-01*HR1(0)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(0,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR2(0,1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,0,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,0,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,-1)
     &  - HR1(0) *HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,0,1)
     &  + HR1(0) *HR5(0,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR5(0,-1,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,0,-1,1)
     &  - HR1(0) *HR5(0,-1,0,1,-1)
     &  + 9.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,-1)
     &  - HR1(0) *HR5(0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,0,-1,0,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,0,-1,0,1)
     &  - HR1(0) *HR5(0,0,-1,1,-1)
     &  - 6.0000000000000000d+00*HR1(0)*HR5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR5(0,0,0,1,-1)
     &  - HR1(0) *HR5(0,0,1,-1,-1)
     &  - HR1(0) *HR5(0,0,1,0,-1)
     &  - HR1(0) *HR5(0,1,-1,-1,-1)
     &  - 4.2555237822406230d+00*HR1(1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR3(0,0,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - 2.0794415416798359d+00*HR1(1)*HR4(0,0,0,-1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,-1)
     &  + 6.0000000000000000d+00*HR1(1)*HR5(0,0,0,0,-1)
     &  - 1.9248049955307152d+00*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR2(-1,1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR2(-1,1)*HR3(0,0,-1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR2(-1,1)*HR4(0,0,0,-1)
     &  + 1.9248049955307152d+00*HR2(0,-1)
     &  - 1.2011325347955035d-01*HR2(0,-1)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR2(0,-1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR2(0,-1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR2(0,-1)*HR4(0,0,0,-1)
     &  + 1.9248049955307152d+00*HR2(0,1)
     &  + 1.3862943611198906d+00*HR2(0,1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR2(0,1)*HR3(0,0,-1)
     &  - 3.0000000000000000d+00*HR2(0,1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR2(0,1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR2(0,1)*HR4(0,0,0,-1)
     &  - 5.3721319360804020d-01*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + HR3( -1,-1,1)*HR3(0,0,-1)
     &  - 5.3721319360804020d-01*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,0,-1)
     &  + HR3(0, -1,-1)*HR3(0,1,-1)
     &  - 5.3721319360804020d-01*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR3(0,0,-1)*HR3(0,0,-1)
     &  + HR3(0,0, -1)*HR3(0,0,1)
     &  + HR3(0,0, -1)*HR3(0,1,-1)
     &  - 5.3721319360804020d-01*HR3(0,0,1)
     &  - 5.3721319360804020d-01*HR3(0,1,-1)
     &  - 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,1)
     &  + 2.4022650695910071d-01*HR4(0,-1,0,1)
     &  + 2.4022650695910071d-01*HR4(0,0,-1,1)
     &  + 2.4022650695910071d-01*HR4(0,0,1,-1)
     &  + 2.4022650695910071d-01*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,0,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,1,-1)
     &  + 2.0794415416798359d+00*HR5(0,-1,0,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,-1,0,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,1,-1)
     &  + 6.2383246250395077d+00*HR5(0,0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  - 2.0794415416798359d+00*HR5(0,0,-1,0,-1)
     &  - 1.3862943611198906d+00*HR5(0,0,-1,0,1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,1,-1)
     &  - 4.1588830833596718d+00*HR5(0,0,0,-1,-1)
     &  - 2.0794415416798359d+00*HR5(0,0,0,-1,1)
     &  - 2.0794415416798359d+00*HR5(0,0,0,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,0,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  - 9.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,-1,0,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,1,-1)
     &  - HR6(0, -1,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,1,-1)
     &  + HR6(0, -1,0,1,-1,-1)
     &  - 1.5000000000000000d+01*HR6(0,0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,-1,0,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,-1,0,1)
     &  + HR6(0,0, -1,-1,1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,-1,0,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,0,-1,1)
     &  - HR6(0,0, -1,0,0,1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,0,1,-1)
     &  + HR6(0,0, -1,1,-1,-1)
     &  + 9.0000000000000000d+00*HR6(0,0,0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,0,-1,-1,1)
     &  + 3.0000000000000000d+00*HR6(0,0,0,-1,0,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,0,-1,1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,0,1,-1,-1)
     &  + HR6(0,0,1, -1,-1,-1)
     &  + HR6(0,0,1,0, -1,-1)
     &  + HR6(0,1, -1,-1,-1,-1)
      HY6(0,1,1,1,-1,-1) = 
     &  + 9.7962010990336989d-02
     &  + 6.5753420522661971d-01*HR1(-1)
     &  - 3.0168989201460834d-01*HR1(-1)*HR1(-1)
     &  + 7.9222950181883112d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 2.3766885054564933d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.3766885054564933d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 6.0337978402921669d-01*HR1(-1)*HR1(0)
     &  + 2.3766885054564933d-01*HR1(-1)*HR1(0)*HR1(0)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 4.7533770109129867d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 6.0337978402921669d-01*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  - 4.7533770109129867d-01*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(0,0,0,-1)
     &  - HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,0,-1,-1)
     &  + 4.0037751159850118d-02*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 2.3766885054564933d-01*HR1(0)*HR1(0)*HR1(1)
     &  + 1.2011325347955035d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 1.2011325347955035d-01*HR1(0)*HR1(0)*HR2(0,1)
     &  + 6.0337978402921669d-01*HR1(0)*HR1(1)
     &  + 4.7533770109129867d-01*HR1(0)*HR2(-1,1)
     &  - 4.7533770109129867d-01*HR1(0)*HR2(0,-1)
     &  - 4.7533770109129867d-01*HR1(0)*HR2(0,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,0,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,1,-1)
     &  + 6.5753420522661971d-01*HR1(1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,0,-1,-1)
     &  + 6.0337978402921669d-01*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  + HR2( -1,1)*HR4(0,0,-1,-1)
     &  - 6.0337978402921669d-01*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,-1)
     &  - HR2(0, -1)*HR4(0,0,-1,-1)
     &  - 6.0337978402921669d-01*HR2(0,1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(0,1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR2(0,1)*HR4(0,-1,-1,-1)
     &  - HR2(0,1) *HR4(0,0,-1,-1)
     &  + 4.7533770109129867d-01*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + 4.7533770109129867d-01*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,1,-1)
     &  + 4.7533770109129867d-01*HR3(0,0,-1)
     &  + 4.7533770109129867d-01*HR3(0,0,1)
     &  + 4.7533770109129867d-01*HR3(0,1,-1)
     &  + 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,0,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,0,1)
     &  - 2.4022650695910071d-01*HR4(0,0,1,-1)
     &  - 2.4022650695910071d-01*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,0,1)
     &  + 1.3862943611198906d+00*HR5(0,-1,0,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  + 4.1588830833596718d+00*HR5(0,0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,0,-1,0,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,0,1)
     &  - 2.7725887222397812d+00*HR5(0,0,0,-1,-1)
     &  - 1.3862943611198906d+00*HR5(0,0,0,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,0,0,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,0,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  - 5.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,-1,0,1)
     &  - HR6(0, -1,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,-1,0,-1,1)
     &  + HR6(0, -1,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  + HR6(0, -1,0,-1,1,-1)
     &  - 8.0000000000000000d+00*HR6(0,0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  + HR6(0,0, -1,-1,0,-1)
     &  + HR6(0,0, -1,-1,0,1)
     &  + HR6(0,0, -1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,-1,0,-1,-1)
     &  + HR6(0,0, -1,0,-1,1)
     &  + HR6(0,0, -1,0,1,-1)
     &  + 6.0000000000000000d+00*HR6(0,0,0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,0,0,1,-1,-1)
     &  - HR6(0,0,1, -1,-1,-1)
     &  + HR6(0,0,1,0, -1,-1)
     &  - HR6(0,1, -1,-1,-1,-1)
      HY6(0,1,1,1,-1,1) = 
     &  + 1.7017542062334508d-01
     &  + 3.4000922141988652d+00*HR1(-1)
     &  - 1.2185212069612250d+00*HR1(-1)*HR1(-1)
     &  + 2.4633412775717367d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.4260021936042187d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.3333333333333333d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 7.3900238327152102d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.4556013161625312d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 7.3900238327152102d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  + 1.5000000000000000d+00*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  + 2.4370424139224501d+00*HR1(-1)*HR1(0)
     &  + 7.3900238327152102d-01*HR1(-1)*HR1(0)*HR1(0)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.4780047665430420d+00*HR1(-1)*HR1(0)*HR1(1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR1(0)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR4(0,0,0,-1)
     &  - 2.4370424139224501d+00*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR1(1)*HR4(0,0,0,-1)
     &  - 1.4780047665430420d+00*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR2(-1,1)*HR3(0,0,-1)
     &  + HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(0,0,0,-1)
     &  + HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,0,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR5(0,0,0,0,-1)
     &  + 9.7040087744168750d-02*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 7.3900238327152102d-01*HR1(0)*HR1(0)*HR1(1)
     &  + 2.9112026323250625d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 2.9112026323250625d-01*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 2.9112026323250625d-01*HR1(0)*HR1(0)*HR2(0,1)
     &  + 2.4370424139224501d+00*HR1(0)*HR1(1)
     &  + HR1(0) *HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR1(1)*HR4(0,0,-1,-1)
     &  + HR1(0) *HR1(1)*HR4(0,0,0,-1)
     &  + 1.4780047665430420d+00*HR1(0)*HR2(-1,1)
     &  + HR1(0) *HR2(-1,1)*HR3(0,-1,-1)
     &  + HR1(0) *HR2(-1,1)*HR3(0,0,-1)
     &  - 1.4780047665430420d+00*HR1(0)*HR2(0,-1)
     &  + HR1(0) *HR2(0,-1)*HR3(-1,-1,1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(0,-1)*HR3(0,0,-1)
     &  + HR1(0) *HR2(0,-1)*HR3(0,1,-1)
     &  - 1.4780047665430420d+00*HR1(0)*HR2(0,1)
     &  - HR1(0) *HR2(0,1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(0,1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(0)*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(0)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(0)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(0)*HR3(0,0,1)
     &  + 5.8224052646501250d-01*HR1(0)*HR3(0,1,-1)
     &  - HR1(0) *HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,-1,-1,-1)
     &  - HR1(0) *HR5(0,-1,-1,-1,1)
     &  + HR1(0) *HR5(0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR5(0,-1,0,-1,-1)
     &  + HR1(0) *HR5(0,-1,0,-1,1)
     &  - 6.0000000000000000d+00*HR1(0)*HR5(0,0,-1,-1,-1)
     &  + HR1(0) *HR5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,0,-1,0,-1)
     &  + HR1(0) *HR5(0,0,-1,0,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR5(0,0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR5(0,0,0,1,-1)
     &  - HR1(0) *HR5(0,0,1,-1,-1)
     &  + HR1(0) *HR5(0,0,1,0,-1)
     &  - HR1(0) *HR5(0,1,-1,-1,-1)
     &  + 3.4000922141988652d+00*HR1(1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR4(0,0,0,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,0,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR5(0,0,0,0,-1)
     &  + 2.4370424139224501d+00*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR3(0,0,-1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR2(-1,1)*HR4(0,0,0,-1)
     &  - 2.4370424139224501d+00*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR3(0,1,-1)
     &  - HR2(0, -1)*HR4(-1,-1,-1,1)
     &  + HR2(0, -1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR2(0,-1)*HR4(0,0,0,-1)
     &  + HR2(0, -1)*HR4(0,1,-1,-1)
     &  - 2.4370424139224501d+00*HR2(0,1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(0,1)*HR3(0,0,-1)
     &  + 3.0000000000000000d+00*HR2(0,1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR2(0,1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR2(0,1)*HR4(0,0,0,-1)
     &  + 1.4780047665430420d+00*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(-1,-1,1)*HR3(0,0,-1)
     &  + 1.4780047665430420d+00*HR3(0,-1,-1)
     &  - HR3(0, -1,-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR3(0,-1,-1)*HR3(0,1,-1)
     &  + 1.4780047665430420d+00*HR3(0,0,-1)
     &  - HR3(0,0, -1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR3(0,0,-1)*HR3(0,0,1)
     &  - 2.0000000000000000d+00*HR3(0,0,-1)*HR3(0,1,-1)
     &  + 1.4780047665430420d+00*HR3(0,0,1)
     &  + 1.4780047665430420d+00*HR3(0,1,-1)
     &  + 5.8224052646501250d-01*HR4(-1,-1,-1,1)
     &  - 5.8224052646501250d-01*HR4(0,-1,-1,-1)
     &  - 5.8224052646501250d-01*HR4(0,0,-1,-1)
     &  - 5.8224052646501250d-01*HR4(0,0,0,-1)
     &  - 5.8224052646501250d-01*HR4(0,0,0,1)
     &  - 5.8224052646501250d-01*HR4(0,0,1,-1)
     &  - 5.8224052646501250d-01*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,0,1)
     &  - 1.3862943611198906d+00*HR5(0,-1,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,-1,0,-1,1)
     &  - 4.1588830833596718d+00*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,0,-1,0,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,0,1)
     &  + 2.7725887222397812d+00*HR5(0,0,0,-1,-1)
     &  + 1.3862943611198906d+00*HR5(0,0,0,-1,1)
     &  + 1.3862943611198906d+00*HR5(0,0,0,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,0,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR6(0,-1,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,-1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,-1,0,1)
     &  + HR6(0, -1,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,-1,0,-1,1)
     &  - HR6(0, -1,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HR6(0,-1,0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,-1,0,-1,-1,1)
     &  - HR6(0, -1,0,-1,1,-1)
     &  + 1.0000000000000000d+01*HR6(0,0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,-1,-1,0,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,-1,0,1)
     &  - HR6(0,0, -1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,-1,0,-1,-1)
     &  - 3.0000000000000000d+00*HR6(0,0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HR6(0,0,-1,0,0,1)
     &  - HR6(0,0, -1,0,1,-1)
     &  - 6.0000000000000000d+00*HR6(0,0,0,-1,-1,-1)
     &  - 5.0000000000000000d+00*HR6(0,0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,0,-1,0,-1)
     &  + 3.0000000000000000d+00*HR6(0,0,0,-1,0,1)
     &  - 2.0000000000000000d+00*HR6(0,0,0,-1,1,-1)
     &  - 4.0000000000000000d+00*HR6(0,0,0,0,-1,-1)
     &  + 4.0000000000000000d+00*HR6(0,0,0,0,-1,1)
     &  + 4.0000000000000000d+00*HR6(0,0,0,0,1,-1)
     &  + HR6(0,0,0,1, -1,-1)
     &  + HR6(0,0,0,1,0, -1)
     &  + HR6(0,0,1, -1,-1,-1)
     &  + HR6(0,0,1, -1,0,-1)
     &  + HR6(0,1, -1,-1,-1,-1)
      HY6(0,1,1,1,1,-1) = 
     &  + 3.9593139179918261d-01
     &  - 1.0351261013563319d+00*HR1(-1)
     &  + 5.3409449824807409d-01*HR1(-1)*HR1(-1)
     &  - 1.8455064982157776d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.4278897515967247d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 5.5365194946473328d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.6567338509580348d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 5.5365194946473328d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR2(-1,1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(0,0,0,-1)
     &  - 1.0681889964961481d+00*HR1(-1)*HR1(0)
     &  - 5.5365194946473328d-01*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 1.1073038989294665d+00*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 1.0681889964961481d+00*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR4(0,0,0,-1)
     &  + 1.1073038989294665d+00*HR1(-1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR3(0,0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR3(-1,-1,1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + HR1( -1)*HR5(0,-1,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,-1,-1,-1)
     &  + HR1( -1)*HR5(0,0,0,-1,-1)
     &  + HR1( -1)*HR5(0,0,0,0,-1)
     &  - 2.8881132523331054d-02*HR1(0)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.7711559006386898d-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(0)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 1.1552453009332421d-01*HR1(0)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 1.1552453009332421d-01*HR1(0)*HR1(0)*HR1(0)*HR2(0,1)
     &  - 5.5365194946473328d-01*HR1(0)*HR1(0)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 5.3134677019160696d-01*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 5.3134677019160696d-01*HR1(0)*HR1(0)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,0,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,0,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  - 1.0681889964961481d+00*HR1(0)*HR1(1)
     &  - 1.1073038989294665d+00*HR1(0)*HR2(-1,1)
     &  + 1.1073038989294665d+00*HR1(0)*HR2(0,-1)
     &  + 1.1073038989294665d+00*HR1(0)*HR2(0,1)
     &  - 1.0626935403832139d+00*HR1(0)*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR1(0)*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR1(0)*HR3(0,0,-1)
     &  - 1.0626935403832139d+00*HR1(0)*HR3(0,0,1)
     &  - 1.0626935403832139d+00*HR1(0)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,0,0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,0,0,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,-1)
     &  - 1.0351261013563319d+00*HR1(1)
     &  + HR1(1) *HR5(0,-1,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,-1,-1,-1)
     &  + HR1(1) *HR5(0,0,0,-1,-1)
     &  + HR1(1) *HR5(0,0,0,0,-1)
     &  - 1.0681889964961481d+00*HR2(-1,1)
     &  + HR2( -1,1)*HR4(0,-1,-1,-1)
     &  + HR2( -1,1)*HR4(0,0,-1,-1)
     &  + HR2( -1,1)*HR4(0,0,0,-1)
     &  + 1.0681889964961481d+00*HR2(0,-1)
     &  + HR2(0, -1)*HR4(-1,-1,-1,1)
     &  - HR2(0, -1)*HR4(0,-1,-1,-1)
     &  - HR2(0, -1)*HR4(0,0,-1,-1)
     &  - HR2(0, -1)*HR4(0,0,0,-1)
     &  - HR2(0, -1)*HR4(0,1,-1,-1)
     &  + 1.0681889964961481d+00*HR2(0,1)
     &  - HR2(0,1) *HR4(0,-1,-1,-1)
     &  - HR2(0,1) *HR4(0,0,-1,-1)
     &  - HR2(0,1) *HR4(0,0,0,-1)
     &  - 1.1073038989294665d+00*HR3(-1,-1,1)
     &  + HR3( -1,-1,1)*HR3(0,-1,-1)
     &  + HR3( -1,-1,1)*HR3(0,0,-1)
     &  - 1.1073038989294665d+00*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR3(0,-1,-1)*HR3(0,-1,-1)
     &  + HR3(0, -1,-1)*HR3(0,0,-1)
     &  + HR3(0, -1,-1)*HR3(0,1,-1)
     &  - 1.1073038989294665d+00*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR3(0,0,-1)*HR3(0,0,-1)
     &  + HR3(0,0, -1)*HR3(0,0,1)
     &  + HR3(0,0, -1)*HR3(0,1,-1)
     &  - 1.1073038989294665d+00*HR3(0,0,1)
     &  - 1.1073038989294665d+00*HR3(0,1,-1)
     &  - 1.0626935403832139d+00*HR4(-1,-1,-1,1)
     &  + 1.0626935403832139d+00*HR4(0,-1,-1,-1)
     &  + 1.0626935403832139d+00*HR4(0,0,-1,-1)
     &  + 1.0626935403832139d+00*HR4(0,0,0,-1)
     &  + 1.0626935403832139d+00*HR4(0,0,0,1)
     &  + 1.0626935403832139d+00*HR4(0,0,1,-1)
     &  + 1.0626935403832139d+00*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR5(0,-1,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,0,0,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,0,0,1)
     &  - 6.9314718055994530d-01*HR5(0,0,0,1,-1)
     &  - 6.9314718055994530d-01*HR5(0,0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  - HR6( -1,-1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,-1,1)
     &  + HR6(0, -1,-1,-1,0,1)
     &  + HR6(0, -1,-1,0,-1,1)
     &  + HR6(0, -1,0,-1,-1,1)
     &  + HR6(0,0, -1,-1,-1,1)
     &  + HR6(0,0, -1,-1,0,1)
     &  + HR6(0,0, -1,0,-1,1)
     &  - HR6(0,0, -1,0,0,1)
     &  + 2.0000000000000000d+00*HR6(0,0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR6(0,0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HR6(0,0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HR6(0,0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HR6(0,0,0,1,-1,-1)
     &  - HR6(0,0,0,1,0, -1)
     &  + HR6(0,0,1, -1,-1,-1)
     &  - HR6(0,0,1, -1,0,-1)
     &  - HR6(0,0,1,0, -1,-1)
     &  + HR6(0,1, -1,-1,-1,-1)
      HY6(0,1,1,1,1,1) = 
     &  + 1.0173430619844491d+00
     &  + 1.3333558146428443d-03*HR1(-1)
     &  - 4.8090645538142385d-03*HR1(-1)*HR1(-1)
     &  + 9.2506847774702633d-03*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.0009437789962529d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 5.7762265046662109d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.3888888888888888d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 2.7777777777777777d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(0)
     &  - 6.0056626739775178d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 5.7762265046662109d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 2.0833333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 9.6181291076284771d-03*HR1(-1)*HR1(0)
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(0)*HR1(0)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(0)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(0)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 5.5504108664821579d-02*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 9.6181291076284771d-03*HR1(-1)*HR1(1)
     &  - 5.5504108664821579d-02*HR1(-1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR5(-1,-1,-1,-1,1)
     &  + 8.3333333333333333d-03*HR1(0)*HR1(0)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(0)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(0)*HR1(0)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 4.1666666666666666d-02*HR1(0)*HR1(0)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 4.1666666666666666d-02*HR1(0)*HR1(0)*HR1(0)*HR1(0)*HR2(0,1)
     &  + 4.0037751159850118d-02*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(0)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 1.1552453009332421d-01*HR1(0)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 1.1552453009332421d-01*HR1(0)*HR1(0)*HR1(0)*HR2(0,1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,0,-1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,0,1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  + 2.7752054332410789d-02*HR1(0)*HR1(0)*HR1(1)
     &  + 1.2011325347955035d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 1.2011325347955035d-01*HR1(0)*HR1(0)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,0,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,0,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,0,0,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,0,1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR4(0,1,-1,-1)
     &  + 9.6181291076284771d-03*HR1(0)*HR1(1)
     &  + 5.5504108664821579d-02*HR1(0)*HR2(-1,1)
     &  - 5.5504108664821579d-02*HR1(0)*HR2(0,-1)
     &  - 5.5504108664821579d-02*HR1(0)*HR2(0,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,0,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,0,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,0,0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,0,0,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR4(0,1,-1,-1)
     &  + HR1(0) *HR5(-1,-1,-1,-1,1)
     &  + HR1(0) *HR5(0,-1,-1,-1,-1)
     &  + HR1(0) *HR5(0,0,-1,-1,-1)
     &  + HR1(0) *HR5(0,0,0,-1,-1)
     &  + HR1(0) *HR5(0,0,0,0,-1)
     &  + HR1(0) *HR5(0,0,0,0,1)
     &  + HR1(0) *HR5(0,0,0,1,-1)
     &  + HR1(0) *HR5(0,0,1,-1,-1)
     &  + HR1(0) *HR5(0,1,-1,-1,-1)
     &  + 1.3333558146428443d-03*HR1(1)
     &  + 9.6181291076284771d-03*HR2(-1,1)
     &  - 9.6181291076284771d-03*HR2(0,-1)
     &  - 9.6181291076284771d-03*HR2(0,1)
     &  + 5.5504108664821579d-02*HR3(-1,-1,1)
     &  + 5.5504108664821579d-02*HR3(0,-1,-1)
     &  + 5.5504108664821579d-02*HR3(0,0,-1)
     &  + 5.5504108664821579d-02*HR3(0,0,1)
     &  + 5.5504108664821579d-02*HR3(0,1,-1)
     &  + 2.4022650695910071d-01*HR4(-1,-1,-1,1)
     &  - 2.4022650695910071d-01*HR4(0,-1,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,-1,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,0,-1)
     &  - 2.4022650695910071d-01*HR4(0,0,0,1)
     &  - 2.4022650695910071d-01*HR4(0,0,1,-1)
     &  - 2.4022650695910071d-01*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(-1,-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR5(0,-1,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,0,0,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,0,0,1)
     &  + 6.9314718055994530d-01*HR5(0,0,0,1,-1)
     &  + 6.9314718055994530d-01*HR5(0,0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR5(0,1,-1,-1,-1)
     &  + HR6( -1,-1,-1,-1,-1,1)
     &  - HR6(0, -1,-1,-1,-1,-1)
     &  - HR6(0,0, -1,-1,-1,-1)
     &  - HR6(0,0,0, -1,-1,-1)
     &  - HR6(0,0,0,0, -1,-1)
     &  - HR6(0,0,0,0,0, -1)
     &  - HR6(0,0,0,0,0,1) 
     &  - HR6(0,0,0,0,1, -1)
     &  - HR6(0,0,0,1, -1,-1)
     &  - HR6(0,0,1, -1,-1,-1)
     &  - HR6(0,1, -1,-1,-1,-1)
      if (r.lt.0d0) then 
* 2017-04-20:16:04:21.hpl 
* <- r_to_y.real_theta.out 
* produced by form-to-fortr for gehrt@tintin 

      HY6(-1,-1,-1,-1,1,1) = HY6(-1,-1,-1,-1,1,1)  
     &  - 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
      HY6(-1,-1,-1,1,1,1) = HY6(-1,-1,-1,1,1,1) 
     &  - 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,-1,-1,-1)
      HY6(-1,-1,1,-1,1,1) = HY6(-1,-1,1,-1,1,1) 
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 1.4804406601634037d+01*HR4(0,-1,-1,-1)
      HY6(-1,-1,1,1,-1,1) = HY6(-1,-1,1,1,-1,1) 
     &  - 1.4366209156229181d+00*HR1(-1)*HR1(-1)
      HY6(-1,-1,1,1,1,1) = HY6(-1,-1,1,1,1,1) 
     &  + 1.4366209156229181d+00*HR1(-1)*HR1(-1)
     &  + 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.2337005501361698d+00*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,0,-1)
     &  - 4.9348022005446793d+00*HR1(0)*HR3(0,-1,-1)
     &  - 3.4205442319285582d+00*HR3(0,-1,-1)
     &  + 9.8696044010893586d+00*HR4(0,-1,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,0,-1,-1)
      HY6(-1,1,-1,1,1,1) = HY6(-1,1,-1,1,1,1) 
     &  + 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR2(0,-1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(0,0,-1)
     &  + 9.8696044010893586d+00*HR1(0)*HR3(0,-1,-1)
     &  - 2.4674011002723396d+00*HR2(0,-1)*HR2(0,-1)
     &  + 6.8410884638571165d+00*HR3(0,-1,-1)
     &  - 1.9739208802178717d+01*HR4(0,-1,-1,-1)
     &  - 9.8696044010893586d+00*HR4(0,0,-1,-1)
      HY6(-1,1,1,1,1,1) = HY6(-1,1,1,1,1,1) 
     &  + 2.5393830695518106d+00*HR1(-1)
     &  - 1.4366209156229181d+00*HR1(-1)*HR1(-1)
     &  - 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.2337005501361698d+00*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 2.8732418312458363d+00*HR1(-1)*HR1(0)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(0)*HR1(0)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 2.4674011002723396d+00*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 3.4205442319285582d+00*HR1(0)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(0)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(0)*HR3(0,0,-1)
     &  - 2.8732418312458363d+00*HR2(0,-1)
     &  - 3.4205442319285582d+00*HR3(0,-1,-1)
     &  - 3.4205442319285582d+00*HR3(0,0,-1)
     &  + 4.9348022005446793d+00*HR4(0,-1,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,0,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,0,0,-1)
      HY6(0,-1,-1,-1,1,1) = HY6(0,-1,-1,-1,1,1) 
     &  - 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 4.9348022005446793d+00*HR4(-1,-1,-1,1)
      HY6(0,-1,-1,0,1,1) = HY6(0,-1,-1,0,1,1) 
     &  - 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(-1,-1,1)
     &  + 2.4674011002723396d+00*HR2(-1,1)*HR2(-1,1)
      HY6(0,-1,-1,1,1,-1) = HY6(0,-1,-1,1,1,-1) 
     &  + 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR2(-1,1)
     &  + 3.4205442319285582d+00*HR3(-1,-1,1)
      HY6(0,-1,-1,1,1,1) = HY6(0,-1,-1,1,1,1) 
     &  - 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(0)*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR2(-1,1)*HR2(0,-1)
     &  - 3.4205442319285582d+00*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR4(-1,-1,-1,1)
     &  + 4.9348022005446793d+00*HR4(0,-1,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,-1,-1,1)
      HY6(0,-1,0,-1,1,1) = HY6(0,-1,0,-1,1,1) 
     &  - 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 9.8696044010893586d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(-1,1,1)
     &  + 9.8696044010893586d+00*HR1(1)*HR3(-1,-1,1)
     &  - 2.4674011002723396d+00*HR2(-1,1)*HR2(-1,1)
     &  + 1.9739208802178717d+01*HR4(-1,-1,-1,1)
     &  - 9.8696044010893586d+00*HR4(-1,-1,1,1)
      HY6(0,-1,0,1,1,-1) = HY6(0,-1,0,1,1,-1) 
     &  + 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR2(-1,1)
     &  + 3.4205442319285582d+00*HR1(1)*HR2(-1,1)
     &  + 6.8410884638571165d+00*HR3(-1,-1,1)
     &  - 6.8410884638571165d+00*HR3(-1,1,1)
      HY6(0,-1,0,1,1,1) = HY6(0,-1,0,1,1,1) 
     &  - 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR2(-1,1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 9.8696044010893586d+00*HR1(-1)*HR3(-1,1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,1,-1)
     &  - 4.9348022005446793d+00*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 9.8696044010893586d+00*HR1(0)*HR3(-1,-1,1)
     &  + 9.8696044010893586d+00*HR1(0)*HR3(-1,1,1)
     &  - 3.4205442319285582d+00*HR1(1)*HR2(-1,1)
     &  - 9.8696044010893586d+00*HR1(1)*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,1,-1)
     &  + 2.4674011002723396d+00*HR2(-1,1)*HR2(-1,1)
     &  - 4.9348022005446793d+00*HR2(-1,1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR2(-1,1)*HR2(0,1)
     &  - 6.8410884638571165d+00*HR3(-1,-1,1)
     &  + 6.8410884638571165d+00*HR3(-1,1,1)
     &  - 1.9739208802178717d+01*HR4(-1,-1,-1,1)
     &  + 9.8696044010893586d+00*HR4(-1,-1,1,1)
     &  + 4.9348022005446793d+00*HR4(0,-1,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,-1,-1,1)
     &  + 4.9348022005446793d+00*HR4(0,1,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,1,-1,1)
      HY6(0,-1,1,-1,1,1) = HY6(0,-1,1,-1,1,1) 
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 9.8696044010893586d+00*HR1(1)*HR3(0,-1,-1)
     &  + 4.9348022005446793d+00*HR2(-1,1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR4(-1,-1,-1,1)
     &  - 1.4804406601634037d+01*HR4(0,-1,-1,-1)
     &  - 9.8696044010893586d+00*HR4(0,-1,-1,1)
     &  - 4.9348022005446793d+00*HR4(0,-1,1,-1)
      HY6(0,-1,1,0,1,1) = HY6(0,-1,1,0,1,1) 
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR3(0,-1,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR3(0,1,-1)
     &  + 4.9348022005446793d+00*HR1(1)*HR3(-1,-1,1)
     &  + 9.8696044010893586d+00*HR1(1)*HR3(0,-1,-1)
     &  + 4.9348022005446793d+00*HR1(1)*HR3(0,-1,1)
     &  + 4.9348022005446793d+00*HR1(1)*HR3(0,1,-1)
     &  - 2.4674011002723396d+00*HR2(-1,1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR2(-1,1)*HR2(0,-1)
     &  + 4.9348022005446793d+00*HR2(-1,1)*HR2(0,1)
     &  - 1.4804406601634037d+01*HR4(0,-1,-1,-1)
     &  - 1.4804406601634037d+01*HR4(0,-1,-1,1)
     &  - 9.8696044010893586d+00*HR4(0,-1,1,-1)
     &  - 9.8696044010893586d+00*HR4(0,-1,1,1)
     &  - 4.9348022005446793d+00*HR4(0,1,-1,-1)
     &  - 4.9348022005446793d+00*HR4(0,1,-1,1)
      HY6(0,-1,1,1,-1,-1) = HY6(0,-1,1,1,-1,-1) 
     &  - 5.9273514758546595d-01*HR1(-1)*HR1(-1)
     &  - 1.1854702951709319d+00*HR1(-1)*HR1(1)
     &  + 1.1854702951709319d+00*HR2(-1,1)
      HY6(0,-1,1,1,-1,1) = HY6(0,-1,1,1,-1,1) 
     &  - 1.4366209156229181d+00*HR1(-1)*HR1(-1)
     &  - 2.8732418312458363d+00*HR1(-1)*HR1(1)
     &  + 2.8732418312458363d+00*HR2(-1,1)
      HY6(0,-1,1,1,0,1) = HY6(0,-1,1,1,0,1) 
     &  - 4.0587121264167682d+00*HR1(-1)*HR1(-1)
     &  - 8.1174242528335364d+00*HR1(-1)*HR1(1)
     &  + 8.1174242528335364d+00*HR2(-1,1)
      HY6(0,-1,1,1,1,-1) = HY6(0,-1,1,1,1,-1) 
     &  + 2.6220912107938500d+00*HR1(-1)*HR1(-1)
     &  - 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.2441824215877001d+00*HR1(-1)*HR1(1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR2(-1,1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR2(0,-1)
     &  - 3.4205442319285582d+00*HR1(0)*HR2(-1,1)
     &  - 3.4205442319285582d+00*HR1(1)*HR2(0,-1)
     &  - 5.2441824215877001d+00*HR2(-1,1)
     &  - 3.4205442319285582d+00*HR3(-1,-1,1)
     &  + 3.4205442319285582d+00*HR3(0,-1,-1)
     &  + 3.4205442319285582d+00*HR3(0,-1,1)
      HY6(0,-1,1,1,1,1) = HY6(0,-1,1,1,1,1) 
     &  + 1.4366209156229181d+00*HR1(-1)*HR1(-1)
     &  + 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.2337005501361698d+00*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR1(0)*HR1(1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 2.8732418312458363d+00*HR1(-1)*HR1(1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR2(-1,1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,0,-1)
     &  + 2.4674011002723396d+00*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 3.4205442319285582d+00*HR1(0)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(0)*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR1(0)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(0)*HR3(0,-1,1)
     &  + 3.4205442319285582d+00*HR1(1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,0,-1)
     &  - 2.8732418312458363d+00*HR2(-1,1)
     &  + 3.4205442319285582d+00*HR3(-1,-1,1)
     &  - 3.4205442319285582d+00*HR3(0,-1,-1)
     &  - 3.4205442319285582d+00*HR3(0,-1,1)
     &  + 4.9348022005446793d+00*HR4(-1,-1,-1,1)
     &  + 9.8696044010893586d+00*HR4(0,-1,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,-1,-1,1)
     &  + 4.9348022005446793d+00*HR4(0,-1,1,-1)
     &  + 4.9348022005446793d+00*HR4(0,0,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,0,-1,1)
      HY6(0,0,-1,-1,1,1) = HY6(0,0,-1,-1,1,1)
     &  - 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.2337005501361698d+00*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(-1,1,1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(-1,-1,1)
     &  - 9.8696044010893586d+00*HR4(-1,-1,-1,1)
     &  + 4.9348022005446793d+00*HR4(-1,-1,1,1)
      HY6(0,0,-1,0,1,1) = HY6(0,0,-1,0,1,1)
     &  - 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.2337005501361698d+00*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(-1,1,1)
     &  - 2.4674011002723396d+00*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 9.8696044010893586d+00*HR1(1)*HR3(-1,-1,1)
     &  + 9.8696044010893586d+00*HR1(1)*HR3(-1,1,1)
     &  - 1.4804406601634037d+01*HR4(-1,-1,-1,1)
     &  + 1.4804406601634037d+01*HR4(-1,-1,1,1)
     &  - 1.4804406601634037d+01*HR4(-1,1,1,1)
      HY6(0,0,-1,1,1,-1) = HY6(0,0,-1,1,1,-1)
     &  + 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4205442319285582d+00*HR1(1)*HR2(-1,1)
     &  - 3.4205442319285582d+00*HR3(-1,-1,1)
     &  + 3.4205442319285582d+00*HR3(-1,1,1)
      HY6(0,0,-1,1,1,1) = HY6(0,0,-1,1,1,1) 
     &  - 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.2337005501361698d+00*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR3(-1,1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,-1,1)
     &  + 4.9348022005446793d+00*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(0)*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR1(0)*HR3(-1,1,1)
     &  + 2.4674011002723396d+00*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4205442319285582d+00*HR1(1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(1)*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,-1,1)
     &  + 3.4205442319285582d+00*HR3(-1,-1,1)
     &  - 3.4205442319285582d+00*HR3(-1,1,1)
     &  + 9.8696044010893586d+00*HR4(-1,-1,-1,1)
     &  - 4.9348022005446793d+00*HR4(-1,-1,1,1)
     &  + 4.9348022005446793d+00*HR4(0,-1,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,-1,-1,1)
     &  + 4.9348022005446793d+00*HR4(0,-1,1,-1)
     &  + 4.9348022005446793d+00*HR4(0,-1,1,1)
      HY6(0,0,0,-1,1,1) = HY6(0,0,0,-1,1,1)
     &  - 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.2337005501361698d+00*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 2.4674011002723396d+00*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(1)*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(-1,1,1)
     &  + 4.9348022005446793d+00*HR4(-1,-1,-1,1)
     &  - 4.9348022005446793d+00*HR4(-1,-1,1,1)
     &  + 4.9348022005446793d+00*HR4(-1,1,1,1)
      HY6(0,0,0,0,1,1) = HY6(0,0,0,0,1,1) 
     &  - 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.2337005501361698d+00*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 2.0561675835602830d-01*HR1(1)*HR1(1)*HR1(1)*HR1(1)
      HY6(0,0,0,1,1,-1) = HY6(0,0,0,1,1,-1)
     &  + 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.7009070532142637d-01*HR1(1)*HR1(1)*HR1(1)
      HY6(0,0,0,1,1,1) = HY6(0,0,0,1,1,1)
     &  - 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.2337005501361698d+00*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(1)*HR1(1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,-1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,1,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,1,1)
     &  - 8.2246703342411321d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 5.7009070532142637d-01*HR1(1)*HR1(1)*HR1(1)
     &  - 2.4674011002723396d+00*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 2.4674011002723396d+00*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 2.4674011002723396d+00*HR1(1)*HR1(1)*HR2(0,1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(-1,-1,1)
     &  + 4.9348022005446793d+00*HR1(1)*HR3(-1,1,1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,-1,1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,1,-1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,1,1)
     &  - 4.9348022005446793d+00*HR4(-1,-1,-1,1)
     &  + 4.9348022005446793d+00*HR4(-1,-1,1,1)
     &  - 4.9348022005446793d+00*HR4(-1,1,1,1)
     &  + 4.9348022005446793d+00*HR4(0,-1,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,-1,-1,1)
     &  + 4.9348022005446793d+00*HR4(0,-1,1,-1)
     &  + 4.9348022005446793d+00*HR4(0,-1,1,1)
     &  + 4.9348022005446793d+00*HR4(0,1,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,1,-1,1)
     &  + 4.9348022005446793d+00*HR4(0,1,1,-1)
     &  + 4.9348022005446793d+00*HR4(0,1,1,1)
      HY6(0,0,1,-1,1,1) = HY6(0,0,1,-1,1,1)
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.2337005501361698d+00*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR3(-1,1,1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR3(0,-1,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR3(0,1,-1)
     &  - 2.4674011002723396d+00*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 4.9348022005446793d+00*HR1(1)*HR3(-1,-1,1)
     &  + 9.8696044010893586d+00*HR1(1)*HR3(0,-1,-1)
     &  + 4.9348022005446793d+00*HR1(1)*HR3(0,-1,1)
     &  + 4.9348022005446793d+00*HR1(1)*HR3(0,1,-1)
     &  + 9.8696044010893586d+00*HR4(-1,-1,-1,1)
     &  - 4.9348022005446793d+00*HR4(-1,-1,1,1)
     &  - 1.4804406601634037d+01*HR4(0,-1,-1,-1)
     &  - 9.8696044010893586d+00*HR4(0,-1,-1,1)
     &  - 9.8696044010893586d+00*HR4(0,-1,1,-1)
     &  - 4.9348022005446793d+00*HR4(0,-1,1,1)
     &  - 9.8696044010893586d+00*HR4(0,1,-1,-1)
     &  - 4.9348022005446793d+00*HR4(0,1,-1,1)
     &  - 4.9348022005446793d+00*HR4(0,1,1,-1)
      HY6(0,0,1,0,1,1) = HY6(0,0,1,0,1,1)
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.2337005501361698d+00*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR3(-1,1,1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(0,-1,1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(0,1,-1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(0,1,1)
     &  + 2.4674011002723396d+00*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 2.4674011002723396d+00*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 2.4674011002723396d+00*HR1(1)*HR1(1)*HR2(0,1)
     &  + 9.8696044010893586d+00*HR1(1)*HR3(-1,-1,1)
     &  - 9.8696044010893586d+00*HR1(1)*HR3(-1,1,1)
     &  + 9.8696044010893586d+00*HR1(1)*HR3(0,-1,-1)
     &  + 9.8696044010893586d+00*HR1(1)*HR3(0,-1,1)
     &  + 9.8696044010893586d+00*HR1(1)*HR3(0,1,-1)
     &  + 9.8696044010893586d+00*HR1(1)*HR3(0,1,1)
     &  + 1.4804406601634037d+01*HR4(-1,-1,-1,1)
     &  - 1.4804406601634037d+01*HR4(-1,-1,1,1)
     &  + 1.4804406601634037d+01*HR4(-1,1,1,1)
     &  - 1.4804406601634037d+01*HR4(0,-1,-1,-1)
     &  - 1.4804406601634037d+01*HR4(0,-1,-1,1)
     &  - 1.4804406601634037d+01*HR4(0,-1,1,-1)
     &  - 1.4804406601634037d+01*HR4(0,-1,1,1)
     &  - 1.4804406601634037d+01*HR4(0,1,-1,-1)
     &  - 1.4804406601634037d+01*HR4(0,1,-1,1)
     &  - 1.4804406601634037d+01*HR4(0,1,1,-1)
     &  - 1.4804406601634037d+01*HR4(0,1,1,1)
      HY6(0,0,1,1,-1,-1) = HY6(0,0,1,1,-1,-1)
     &  - 5.9273514758546595d-01*HR1(-1)*HR1(-1)
     &  - 1.1854702951709319d+00*HR1(-1)*HR1(1)
     &  - 5.9273514758546595d-01*HR1(1)*HR1(1)
      HY6(0,0,1,1,-1,1) = HY6(0,0,1,1,-1,1)
     &  - 1.4366209156229181d+00*HR1(-1)*HR1(-1)
     &  - 2.8732418312458363d+00*HR1(-1)*HR1(1)
     &  - 1.4366209156229181d+00*HR1(1)*HR1(1)
      HY6(0,0,1,1,0,-1) = HY6(0,0,1,1,0,-1) 
     &  - 2.0293560632083841d+00*HR1(-1)*HR1(-1)
     &  - 4.0587121264167682d+00*HR1(-1)*HR1(1)
     &  - 2.0293560632083841d+00*HR1(1)*HR1(1)
      HY6(0,0,1,1,0,1) = HY6(0,0,1,1,0,1)
     &  - 4.0587121264167682d+00*HR1(-1)*HR1(-1)
     &  - 8.1174242528335364d+00*HR1(-1)*HR1(1)
     &  - 4.0587121264167682d+00*HR1(1)*HR1(1)
      HY6(0,0,1,1,1,-1) = HY6(0,0,1,1,1,-1)
     &  + 2.6220912107938500d+00*HR1(-1)*HR1(-1)
     &  - 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.2441824215877001d+00*HR1(-1)*HR1(1)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR2(0,-1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR2(0,1)
     &  + 1.7102721159642791d+00*HR1(0)*HR1(1)*HR1(1)
     &  + 2.6220912107938500d+00*HR1(1)*HR1(1)
     &  + 3.4205442319285582d+00*HR1(1)*HR2(-1,1)
     &  - 3.4205442319285582d+00*HR1(1)*HR2(0,-1)
     &  - 3.4205442319285582d+00*HR1(1)*HR2(0,1)
     &  + 3.4205442319285582d+00*HR3(-1,-1,1)
     &  - 3.4205442319285582d+00*HR3(-1,1,1)
     &  + 3.4205442319285582d+00*HR3(0,-1,-1)
     &  + 3.4205442319285582d+00*HR3(0,-1,1)
     &  + 3.4205442319285582d+00*HR3(0,1,-1)
     &  + 3.4205442319285582d+00*HR3(0,1,1)
      HY6(0,0,1,1,1,1) = HY6(0,0,1,1,1,1) 
     &  + 1.4366209156229181d+00*HR1(-1)*HR1(-1)
     &  + 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.2337005501361698d+00*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.2337005501361698d+00*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 2.8732418312458363d+00*HR1(-1)*HR1(1)
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(1)*HR1(1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR2(0,-1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR2(0,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(-1,1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,0,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,0,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,1,-1)
     &  - 1.2337005501361698d+00*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.7102721159642791d+00*HR1(0)*HR1(1)*HR1(1)
     &  - 4.9348022005446793d+00*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 4.9348022005446793d+00*HR1(0)*HR1(1)*HR2(0,1)
     &  - 4.9348022005446793d+00*HR1(0)*HR3(-1,-1,1)
     &  + 4.9348022005446793d+00*HR1(0)*HR3(-1,1,1)
     &  - 4.9348022005446793d+00*HR1(0)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(0)*HR3(0,-1,1)
     &  - 4.9348022005446793d+00*HR1(0)*HR3(0,1,-1)
     &  - 4.9348022005446793d+00*HR1(0)*HR3(0,1,1)
     &  + 1.4366209156229181d+00*HR1(1)*HR1(1)
     &  - 3.4205442319285582d+00*HR1(1)*HR2(-1,1)
     &  + 3.4205442319285582d+00*HR1(1)*HR2(0,-1)
     &  + 3.4205442319285582d+00*HR1(1)*HR2(0,1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,0,-1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,0,1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,1,-1)
     &  - 3.4205442319285582d+00*HR3(-1,-1,1)
     &  + 3.4205442319285582d+00*HR3(-1,1,1)
     &  - 3.4205442319285582d+00*HR3(0,-1,-1)
     &  - 3.4205442319285582d+00*HR3(0,-1,1)
     &  - 3.4205442319285582d+00*HR3(0,1,-1)
     &  - 3.4205442319285582d+00*HR3(0,1,1)
     &  - 9.8696044010893586d+00*HR4(-1,-1,-1,1)
     &  + 4.9348022005446793d+00*HR4(-1,-1,1,1)
     &  + 9.8696044010893586d+00*HR4(0,-1,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,-1,-1,1)
     &  + 4.9348022005446793d+00*HR4(0,-1,1,-1)
     &  + 4.9348022005446793d+00*HR4(0,0,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,0,-1,1)
     &  + 4.9348022005446793d+00*HR4(0,0,1,-1)
     &  + 4.9348022005446793d+00*HR4(0,0,1,1)
     &  + 9.8696044010893586d+00*HR4(0,1,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,1,-1,1)
     &  + 4.9348022005446793d+00*HR4(0,1,1,-1)
      HY6(0,1,-1,-1,1,1) = HY6(0,1,-1,-1,1,1)
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR4(-1,-1,-1,1)
     &  + 1.4804406601634037d+01*HR4(0,-1,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,-1,-1,1)
     &  + 4.9348022005446793d+00*HR4(0,-1,1,-1)
     &  + 4.9348022005446793d+00*HR4(0,1,-1,-1)
      HY6(0,1,-1,0,1,1) = HY6(0,1,-1,0,1,1)
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,-1,1)
     &  + 4.9348022005446793d+00*HR1(1)*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,-1,1)
     &  - 2.4674011002723396d+00*HR2(-1,1)*HR2(-1,1)
     &  + 1.4804406601634037d+01*HR4(0,-1,-1,-1)
     &  + 1.4804406601634037d+01*HR4(0,-1,-1,1)
     &  + 9.8696044010893586d+00*HR4(0,-1,1,-1)
     &  + 9.8696044010893586d+00*HR4(0,-1,1,1)
     &  + 4.9348022005446793d+00*HR4(0,1,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,1,-1,1)
      HY6(0,1,-1,1,1,-1) = HY6(0,1,-1,1,1,-1)
     &  - 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR2(-1,1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR2(0,-1)
     &  + 3.4205442319285582d+00*HR1(1)*HR2(0,-1)
     &  - 3.4205442319285582d+00*HR3(-1,-1,1)
     &  - 6.8410884638571165d+00*HR3(0,-1,-1)
     &  - 3.4205442319285582d+00*HR3(0,-1,1)
     &  - 3.4205442319285582d+00*HR3(0,1,-1)
      HY6(0,1,-1,1,1,1) = HY6(0,1,-1,1,1,1)
     &  + 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR2(-1,1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(0,0,-1)
     &  - 4.9348022005446793d+00*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 4.9348022005446793d+00*HR1(0)*HR3(-1,-1,1)
     &  + 9.8696044010893586d+00*HR1(0)*HR3(0,-1,-1)
     &  + 4.9348022005446793d+00*HR1(0)*HR3(0,-1,1)
     &  + 4.9348022005446793d+00*HR1(0)*HR3(0,1,-1)
     &  - 3.4205442319285582d+00*HR1(1)*HR2(0,-1)
     &  + 9.8696044010893586d+00*HR1(1)*HR3(0,-1,-1)
     &  + 9.8696044010893586d+00*HR1(1)*HR3(0,0,-1)
     &  + 4.9348022005446793d+00*HR2(-1,1)*HR2(0,-1)
     &  - 2.4674011002723396d+00*HR2(0,-1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR2(0,-1)*HR2(0,1)
     &  + 3.4205442319285582d+00*HR3(-1,-1,1)
     &  + 6.8410884638571165d+00*HR3(0,-1,-1)
     &  + 3.4205442319285582d+00*HR3(0,-1,1)
     &  + 3.4205442319285582d+00*HR3(0,1,-1)
     &  + 4.9348022005446793d+00*HR4(-1,-1,-1,1)
     &  - 1.9739208802178717d+01*HR4(0,-1,-1,-1)
     &  - 9.8696044010893586d+00*HR4(0,-1,-1,1)
     &  + 4.9348022005446793d+00*HR4(0,-1,0,1)
     &  - 4.9348022005446793d+00*HR4(0,-1,1,-1)
     &  - 9.8696044010893586d+00*HR4(0,0,-1,-1)
     &  - 4.9348022005446793d+00*HR4(0,1,-1,-1)
      HY6(0,1,0,1,1,-1) = HY6(0,1,0,1,1,-1) 
     &  - 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR2(-1,1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR2(0,-1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR2(0,1)
     &  - 3.4205442319285582d+00*HR1(1)*HR2(-1,1)
     &  + 3.4205442319285582d+00*HR1(1)*HR2(0,-1)
     &  + 3.4205442319285582d+00*HR1(1)*HR2(0,1)
     &  - 6.8410884638571165d+00*HR3(-1,-1,1)
     &  + 6.8410884638571165d+00*HR3(-1,1,1)
     &  - 6.8410884638571165d+00*HR3(0,-1,-1)
     &  - 6.8410884638571165d+00*HR3(0,-1,1)
     &  - 6.8410884638571165d+00*HR3(0,1,-1)
     &  - 6.8410884638571165d+00*HR3(0,1,1)
      HY6(0,1,0,1,1,1) = HY6(0,1,0,1,1,1)
     &  + 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR2(-1,1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR2(0,-1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR2(0,1)
     &  - 9.8696044010893586d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(-1,1,1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(0,0,-1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(0,0,1)
     &  + 9.8696044010893586d+00*HR1(-1)*HR3(0,1,-1)
     &  + 4.9348022005446793d+00*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 4.9348022005446793d+00*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(0)*HR1(1)*HR2(0,1)
     &  + 9.8696044010893586d+00*HR1(0)*HR3(-1,-1,1)
     &  - 9.8696044010893586d+00*HR1(0)*HR3(-1,1,1)
     &  + 9.8696044010893586d+00*HR1(0)*HR3(0,-1,-1)
     &  + 9.8696044010893586d+00*HR1(0)*HR3(0,-1,1)
     &  + 9.8696044010893586d+00*HR1(0)*HR3(0,1,-1)
     &  + 9.8696044010893586d+00*HR1(0)*HR3(0,1,1)
     &  + 3.4205442319285582d+00*HR1(1)*HR2(-1,1)
     &  - 3.4205442319285582d+00*HR1(1)*HR2(0,-1)
     &  - 3.4205442319285582d+00*HR1(1)*HR2(0,1)
     &  + 9.8696044010893586d+00*HR1(1)*HR3(-1,-1,1)
     &  + 9.8696044010893586d+00*HR1(1)*HR3(0,-1,-1)
     &  + 9.8696044010893586d+00*HR1(1)*HR3(0,0,-1)
     &  + 9.8696044010893586d+00*HR1(1)*HR3(0,0,1)
     &  + 9.8696044010893586d+00*HR1(1)*HR3(0,1,-1)
     &  - 2.4674011002723396d+00*HR2(-1,1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR2(-1,1)*HR2(0,-1)
     &  + 4.9348022005446793d+00*HR2(-1,1)*HR2(0,1)
     &  - 2.4674011002723396d+00*HR2(0,-1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR2(0,-1)*HR2(0,1)
     &  - 2.4674011002723396d+00*HR2(0,1)*HR2(0,1)
     &  + 6.8410884638571165d+00*HR3(-1,-1,1)
     &  - 6.8410884638571165d+00*HR3(-1,1,1)
     &  + 6.8410884638571165d+00*HR3(0,-1,-1)
     &  + 6.8410884638571165d+00*HR3(0,-1,1)
     &  + 6.8410884638571165d+00*HR3(0,1,-1)
     &  + 6.8410884638571165d+00*HR3(0,1,1)
     &  + 1.9739208802178717d+01*HR4(-1,-1,-1,1)
     &  - 9.8696044010893586d+00*HR4(-1,-1,1,1)
     &  - 1.9739208802178717d+01*HR4(0,-1,-1,-1)
     &  - 9.8696044010893586d+00*HR4(0,-1,-1,1)
     &  - 9.8696044010893586d+00*HR4(0,-1,1,-1)
     &  - 9.8696044010893586d+00*HR4(0,0,-1,-1)
     &  - 9.8696044010893586d+00*HR4(0,0,-1,1)
     &  - 9.8696044010893586d+00*HR4(0,0,1,-1)
     &  - 9.8696044010893586d+00*HR4(0,0,1,1)
     &  - 1.9739208802178717d+01*HR4(0,1,-1,-1)
     &  - 9.8696044010893586d+00*HR4(0,1,-1,1)
     &  - 9.8696044010893586d+00*HR4(0,1,1,-1)
      HY6(0,1,1,-1,-1,-1) = HY6(0,1,1,-1,-1,-1)
     &  + 2.7390179757843253d-01*HR1(-1)
     &  + 2.7390179757843253d-01*HR1(1)
      HY6(0,1,1,-1,-1,1) = HY6(0,1,1,-1,-1,1)
     &  + 4.6758733378305350d-01*HR1(-1)
     &  + 4.6758733378305350d-01*HR1(1)
      HY6(0,1,1,-1,1,-1) = HY6(0,1,1,-1,1,-1)
     &  + 1.0564048068288385d+00*HR1(-1)
     &  + 1.0564048068288385d+00*HR1(1)
      HY6(0,1,1,-1,1,1) = HY6(0,1,1,-1,1,1)
     &  + 2.6510408499785916d+00*HR1(-1)
     &  - 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(-1)*HR3(0,0,-1)
     &  + 2.6510408499785916d+00*HR1(1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(1)*HR3(0,0,-1)
     &  - 4.9348022005446793d+00*HR2(-1,1)*HR2(0,-1)
     &  + 2.4674011002723396d+00*HR2(0,-1)*HR2(0,-1)
     &  + 4.9348022005446793d+00*HR2(0,-1)*HR2(0,1)
     &  + 4.9348022005446793d+00*HR4(-1,-1,-1,1)
     &  + 4.9348022005446793d+00*HR4(0,-1,-1,1)
     &  - 4.9348022005446793d+00*HR4(0,-1,0,1)
     &  - 4.9348022005446793d+00*HR4(0,0,-1,1)
     &  - 4.9348022005446793d+00*HR4(0,0,1,-1)
     &  - 4.9348022005446793d+00*HR4(0,1,-1,-1)
      HY6(0,1,1,1,-1,-1) = HY6(0,1,1,1,-1,-1)
     &  - 2.3456975333471897d+00*HR1(-1)
     &  + 5.9273514758546595d-01*HR1(-1)*HR1(-1)
     &  - 1.1854702951709319d+00*HR1(-1)*HR1(0)
     &  + 1.1854702951709319d+00*HR1(-1)*HR1(1)
     &  - 1.1854702951709319d+00*HR1(0)*HR1(1)
     &  - 2.3456975333471897d+00*HR1(1)
     &  - 1.1854702951709319d+00*HR2(-1,1)
     &  + 1.1854702951709319d+00*HR2(0,-1)
     &  + 1.1854702951709319d+00*HR2(0,1)
      HY6(0,1,1,1,-1,1) = HY6(0,1,1,1,-1,1)
     &  - 7.2936611743521288d+00*HR1(-1)
     &  + 1.4366209156229181d+00*HR1(-1)*HR1(-1)
     &  - 2.8732418312458363d+00*HR1(-1)*HR1(0)
     &  + 2.8732418312458363d+00*HR1(-1)*HR1(1)
     &  - 2.8732418312458363d+00*HR1(0)*HR1(1)
     &  - 7.2936611743521288d+00*HR1(1)
     &  - 2.8732418312458363d+00*HR2(-1,1)
     &  + 2.8732418312458363d+00*HR2(0,-1)
     &  + 2.8732418312458363d+00*HR2(0,1)
      HY6(0,1,1,1,1,-1) = HY6(0,1,1,1,1,-1) 
     &  + 2.6510408499785916d+00*HR1(-1)
     &  - 2.6220912107938500d+00*HR1(-1)*HR1(-1)
     &  + 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.2441824215877001d+00*HR1(-1)*HR1(0)
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(0)*HR1(0)
     &  - 3.4205442319285582d+00*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.2441824215877001d+00*HR1(-1)*HR1(1)
     &  - 3.4205442319285582d+00*HR1(-1)*HR2(-1,1)
     &  + 1.7102721159642791d+00*HR1(0)*HR1(0)*HR1(1)
     &  + 5.2441824215877001d+00*HR1(0)*HR1(1)
     &  + 3.4205442319285582d+00*HR1(0)*HR2(-1,1)
     &  - 3.4205442319285582d+00*HR1(0)*HR2(0,-1)
     &  - 3.4205442319285582d+00*HR1(0)*HR2(0,1)
     &  + 2.6510408499785916d+00*HR1(1)
     &  + 5.2441824215877001d+00*HR2(-1,1)
     &  - 5.2441824215877001d+00*HR2(0,-1)
     &  - 5.2441824215877001d+00*HR2(0,1)
     &  + 3.4205442319285582d+00*HR3(-1,-1,1)
     &  + 3.4205442319285582d+00*HR3(0,-1,-1)
     &  + 3.4205442319285582d+00*HR3(0,0,-1)
     &  + 3.4205442319285582d+00*HR3(0,0,1)
     &  + 3.4205442319285582d+00*HR3(0,1,-1)
      HY6(0,1,1,1,1,1) = HY6(0,1,1,1,1,1) 
     &  + 2.5393830695518106d+00*HR1(-1)
     &  - 1.4366209156229181d+00*HR1(-1)*HR1(-1)
     &  - 5.7009070532142637d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.0561675835602830d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.2337005501361698d+00*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.4674011002723396d+00*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.8732418312458363d+00*HR1(-1)*HR1(0)
     &  - 1.7102721159642791d+00*HR1(-1)*HR1(0)*HR1(0)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 2.4674011002723396d+00*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR1(0)*HR1(1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 2.8732418312458363d+00*HR1(-1)*HR1(1)
     &  + 3.4205442319285582d+00*HR1(-1)*HR2(-1,1)
     &  + 4.9348022005446793d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 8.2246703342411321d-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.7102721159642791d+00*HR1(0)*HR1(0)*HR1(1)
     &  - 2.4674011002723396d+00*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 2.4674011002723396d+00*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 2.4674011002723396d+00*HR1(0)*HR1(0)*HR2(0,1)
     &  + 2.8732418312458363d+00*HR1(0)*HR1(1)
     &  - 3.4205442319285582d+00*HR1(0)*HR2(-1,1)
     &  + 3.4205442319285582d+00*HR1(0)*HR2(0,-1)
     &  + 3.4205442319285582d+00*HR1(0)*HR2(0,1)
     &  - 4.9348022005446793d+00*HR1(0)*HR3(-1,-1,1)
     &  - 4.9348022005446793d+00*HR1(0)*HR3(0,-1,-1)
     &  - 4.9348022005446793d+00*HR1(0)*HR3(0,0,-1)
     &  - 4.9348022005446793d+00*HR1(0)*HR3(0,0,1)
     &  - 4.9348022005446793d+00*HR1(0)*HR3(0,1,-1)
     &  + 2.5393830695518106d+00*HR1(1)
     &  + 2.8732418312458363d+00*HR2(-1,1)
     &  - 2.8732418312458363d+00*HR2(0,-1)
     &  - 2.8732418312458363d+00*HR2(0,1)
     &  - 3.4205442319285582d+00*HR3(-1,-1,1)
     &  - 3.4205442319285582d+00*HR3(0,-1,-1)
     &  - 3.4205442319285582d+00*HR3(0,0,-1)
     &  - 3.4205442319285582d+00*HR3(0,0,1)
     &  - 3.4205442319285582d+00*HR3(0,1,-1)
     &  - 4.9348022005446793d+00*HR4(-1,-1,-1,1)
     &  + 4.9348022005446793d+00*HR4(0,-1,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,0,-1,-1)
     &  + 4.9348022005446793d+00*HR4(0,0,0,-1)
     &  + 4.9348022005446793d+00*HR4(0,0,0,1)
     &  + 4.9348022005446793d+00*HR4(0,0,1,-1)
     &  + 4.9348022005446793d+00*HR4(0,1,-1,-1)
* 2017-04-20:16:26:10.hpl 
* <- r_to_y.imag_theta.out 
* produced by form-to-fortr for gehrt@tintin 
      Hi6(-1,-1,-1,-1,-1,1) = 
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
      Hi6(-1,-1,-1,-1,1,1) = 
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,-1)
      Hi6(-1,-1,-1,1,-1,1) = 
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
      Hi6(-1,-1,-1,1,1,1) = 
     &  + 2.3411792664818762d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,0,-1,-1)
     &  + HR1(0) *HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - HR5(0,0, -1,-1,-1)
      Hi6(-1,-1,1,-1,1,1) = 
     &  + 2.6860659680402010d-01*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR2(0, -1)*HR3(0,-1,-1)
     &  - 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  + 7.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - HR5(0, -1,0,-1,-1)
      Hi6(-1,-1,1,1,-1,1) = 
     &  - 7.3900238327152102d-01*HR1(-1)*HR1(-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - HR2(0, -1)*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + HR5(0, -1,0,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,0,-1,-1,-1)
      Hi6(-1,-1,1,1,1,1) = 
     &  + 5.4233865098901558d-01*HR1(-1)*HR1(-1)
     &  - 2.3411792664818762d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 7.0235377994456286d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,0,-1)
     &  - 1.4047075598891257d+00*HR1(-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,0,-1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,0,-1,-1)
     &  + 1.4047075598891257d+00*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  - HR5(0,0,0, -1,-1)
      Hi6(-1,1,-1,1,1,1) = 
     &  - 5.1747906167389938d-01*HR1(-1)
     &  - 2.3411792664818762d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 1.4047075598891257d+00*HR1(-1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,0,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,0,0,-1)
     &  + HR1(0) *HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(0,-1,-1)
     &  - 4.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,0,-1,-1)
     &  - 3.4657359027997265d-01*HR2(0,-1)*HR2(0,-1)
     &  + HR2(0, -1)*HR3(0,0,-1)
     &  - 2.8094151197782514d+00*HR3(0,-1,-1)
     &  - 2.7725887222397812d+00*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR4(0,0,-1,-1)
     &  + 7.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + HR5(0, -1,0,-1,-1)
     &  + 7.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  - HR5(0,0, -1,0,-1)
      Hi6(-1,1,1,1,1,1) = 
     &  - 4.2620378933400481d-01*HR1(-1)
     &  - 5.4233865098901558d-01*HR1(-1)*HR1(-1)
     &  + 2.3411792664818762d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 7.0235377994456286d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 1.0846773019780311d+00*HR1(-1)*HR1(0)
     &  + 7.0235377994456286d-01*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,0,-1)
     &  - 1.4047075598891257d+00*HR1(0)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,0,-1)
     &  + HR1(0) *HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,0,-1,-1)
     &  + HR1(0) *HR4(0,0,0,-1)
     &  - 1.0846773019780311d+00*HR2(0,-1)
     &  + 1.4047075598891257d+00*HR3(0,-1,-1)
     &  + 1.4047075598891257d+00*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,0,-1)
     &  - HR5(0, -1,-1,-1,-1)
     &  - HR5(0,0, -1,-1,-1)
     &  - HR5(0,0,0, -1,-1)
     &  - HR5(0,0,0,0, -1)
      Hi6(0,-1,-1,-1,-1,-1) = 
     &  0.d0 
      Hi6(0,-1,-1,-1,-1,1) = 
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  - HR5( -1,-1,-1,-1,1)
      Hi6(0,-1,-1,-1,0,1) = 
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  - HR1(1) *HR4(-1,-1,-1,1)
     &  + HR2( -1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR5(-1,-1,1,-1,1)
      Hi6(0,-1,-1,-1,1,-1) = 
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
      Hi6(0,-1,-1,-1,1,1) = 
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(-1,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,-1,-1)
     &  + HR2( -1,1)*HR3(0,-1,-1)
     &  + HR2(0, -1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + HR5( -1,-1,-1,-1,1)
     &  - HR5(0, -1,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,1)
      Hi6(0,-1,-1,0,-1,1) = 
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(-1,-1,1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,1,-1,1)
      Hi6(0,-1,-1,0,1,-1) = 
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  - 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
      Hi6(0,-1,-1,0,1,1) = 
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR2(-1,1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,1,-1,-1)
     &  - HR1(0) *HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,1,-1,-1)
     &  + 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR3(-1,-1,1)
     &  + HR2( -1,1)*HR3(0,-1,-1)
     &  + HR2( -1,1)*HR3(0,1,-1)
     &  + HR2(0, -1)*HR3(-1,-1,1)
     &  + HR2(0,1) *HR3(-1,-1,1)
     &  - 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR5(-1,-1,1,-1,1)
     &  - HR5(0, -1,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,1)
     &  - HR5(0,1, -1,-1,-1)
     &  - HR5(0,1, -1,-1,1)
      Hi6(0,-1,-1,1,-1,-1) = 
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR3(-1,-1,1)
      Hi6(0,-1,-1,1,-1,1) = 
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  - 5.8224052646501250d-01*HR3(-1,-1,1)
     &  + HR5( -1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + HR5(0, -1,-1,1,-1)
      Hi6(0,-1,-1,1,0,1) = 
     &  - 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR4(0,1,-1,-1)
     &  + HR1(1) *HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(1) *HR4(0,-1,-1,1)
     &  - HR1(1) *HR4(0,-1,1,-1)
     &  - HR1(1) *HR4(0,1,-1,-1)
     &  - HR2( -1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR2( -1,1)*HR3(0,-1,1)
     &  - HR2( -1,1)*HR3(0,1,-1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  - HR2(0,1) *HR3(-1,-1,1)
     &  - 1.6449340668482264d+00*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR5(-1,-1,1,-1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,1)
     &  + HR5(0, -1,1,-1,-1)
     &  + HR5(0, -1,1,-1,1)
     &  + HR5(0,1, -1,-1,-1)
     &  + HR5(0,1, -1,-1,1)
      Hi6(0,-1,-1,1,1,-1) = 
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  + 1.0626935403832139d+00*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,1)
      Hi6(0,-1,-1,1,1,1) = 
     &  + 2.3411792664818762d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 7.0235377994456286d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  - 1.4047075598891257d+00*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  - HR1(0) *HR4(-1,-1,-1,1)
     &  + HR1(0) *HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  + HR2( -1,1)*HR3(0,-1,-1)
     &  + HR2( -1,1)*HR3(0,0,-1)
     &  + 1.4047075598891257d+00*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  - HR5( -1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - HR5(0, -1,-1,1,-1)
     &  - HR5(0,0, -1,-1,-1)
     &  - HR5(0,0, -1,-1,1)
      Hi6(0,-1,0,-1,-1,-1) = 
     &  0.d0 
      Hi6(0,-1,0,-1,-1,1) = 
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR2( -1,1)*HR3(-1,-1,1)
     &  - 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - HR5( -1,-1,1,-1,1)
      Hi6(0,-1,0,-1,0,1) = 
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR3(-1,1,1)
     &  - 8.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR5(-1,-1,1,-1,1)
     &  - 1.2000000000000000d+01*HR5(-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HR5(-1,1,-1,1,1)
      Hi6(0,-1,0,-1,1,-1) = 
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  + 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
     &  - 2.7725887222397812d+00*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR4(-1,-1,1,1)
      Hi6(0,-1,0,-1,1,1) = 
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR1(-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(-1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(-1,-1,1,1)
     &  + HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,1,-1)
     &  - 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
     &  - HR2( -1,1)*HR3(-1,-1,1)
     &  + HR2( -1,1)*HR3(0,-1,-1)
     &  + HR2( -1,1)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR3(-1,1,1)
     &  + 2.7725887222397812d+00*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR4(-1,-1,1,1)
     &  + 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + HR5( -1,-1,1,-1,1)
     &  - HR5(0, -1,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,1)
     &  - HR5(0, -1,1,-1,-1)
     &  - HR5(0, -1,1,-1,1)
      Hi6(0,-1,0,1,-1,-1) = 
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(1)*HR2(-1,1)
     &  - 4.8045301391820142d-01*HR3(-1,-1,1)
     &  + 4.8045301391820142d-01*HR3(-1,1,1)
      Hi6(0,-1,0,1,-1,1) = 
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - 5.8224052646501250d-01*HR1(1)*HR2(-1,1)
     &  - HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(1) *HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - HR2( -1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  - HR2( -1,1)*HR3(0,-1,1)
     &  - HR2( -1,1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR3(-1,1,1)
     &  - 1.1644810529300250d+00*HR3(-1,-1,1)
     &  + 1.1644810529300250d+00*HR3(-1,1,1)
     &  + 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + HR5( -1,-1,1,-1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + HR5(0, -1,-1,1,-1)
     &  + HR5(0, -1,1,-1,-1)
     &  + HR5(0, -1,1,-1,1)
     &  + 3.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  + HR5(0,1, -1,1,-1)
      Hi6(0,-1,0,1,0,1) = 
     &  - 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR1(-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,1,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,1,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,1,1,-1)
     &  - 1.6449340668482264d+00*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  - HR1(1) *HR2(-1,1)*HR2(0,1)
     &  + 4.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(1) *HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1(1) *HR4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,1,1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR2(0,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,1)*HR3(-1,1,1)
     &  - 3.2898681336964528d+00*HR3(-1,-1,1)
     &  + 3.2898681336964528d+00*HR3(-1,1,1)
     &  + 8.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR5(-1,-1,1,-1,1)
     &  + 1.2000000000000000d+01*HR5(-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HR5(-1,1,-1,1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,1,-1,1)
     &  + 4.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,1,-1,1,1)
     &  + 2.0000000000000000d+00*HR5(0,1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,1,1,-1,1)
      Hi6(0,-1,0,1,1,-1) = 
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR2(-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR3(-1,1,1)
     &  + 1.0626935403832139d+00*HR1(1)*HR2(-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  - 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,1)
     &  + 2.1253870807664278d+00*HR3(-1,-1,1)
     &  - 2.1253870807664278d+00*HR3(-1,1,1)
     &  + 2.7725887222397812d+00*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,1)
      Hi6(0,-1,0,1,1,1) = 
     &  + 2.3411792664818762d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 7.0235377994456286d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - HR1( -1)*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,1)
     &  - HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  - 1.4047075598891257d+00*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - HR1(0) *HR1(0)*HR3(-1,-1,1)
     &  + HR1(0) *HR1(0)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  - HR1(0) *HR2(-1,1)*HR2(0,-1)
     &  - HR1(0) *HR2(-1,1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(-1,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR4(-1,-1,1,1)
     &  + HR1(0) *HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,-1,-1,1)
     &  + HR1(0) *HR4(0,1,-1,-1)
     &  + HR1(0) *HR4(0,1,-1,1)
     &  + 1.4047075598891257d+00*HR1(1)*HR2(-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,0,-1,-1)
     &  + HR1(1) *HR4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR2(0,1)
     &  + HR2( -1,1)*HR3(-1,-1,1)
     &  + HR2( -1,1)*HR3(0,-1,-1)
     &  + HR2( -1,1)*HR3(0,0,-1)
     &  + HR2( -1,1)*HR3(0,0,1)
     &  + HR2( -1,1)*HR3(0,1,-1)
     &  + 2.8094151197782514d+00*HR3(-1,-1,1)
     &  - 2.8094151197782514d+00*HR3(-1,1,1)
     &  - 2.7725887222397812d+00*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,1)
     &  - 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - HR5( -1,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - HR5(0, -1,-1,1,-1)
     &  - HR5(0,0, -1,-1,-1)
     &  - HR5(0,0, -1,-1,1)
     &  - HR5(0,0,1, -1,-1)
     &  - HR5(0,0,1, -1,1)
     &  - 3.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  - HR5(0,1, -1,1,-1)
      Hi6(0,-1,1,-1,-1,-1) = 
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(-1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR1(1)
     &  - 5.5504108664821579d-02*HR2(-1,1)
      Hi6(0,-1,1,-1,-1,1) = 
     &  + 4.7376502115063852d-02*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 9.4753004230127705d-02*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 9.4753004230127705d-02*HR2(-1,1)
     &  + HR2( -1,1)*HR3(0,-1,-1)
     &  + HR5( -1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - HR5(0, -1,1,-1,-1)
      Hi6(0,-1,1,-1,0,1) = 
     &  + 1.9444792308405316d-01*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 3.8889584616810632d-01*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,1,-1)
     &  + HR1(1) *HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,1,-1)
     &  - 3.8889584616810632d-01*HR2(-1,1)
     &  - HR2( -1,1)*HR3(-1,-1,1)
     &  + HR2( -1,1)*HR3(0,-1,-1)
     &  + HR2( -1,1)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR5(-1,-1,1,-1,1)
     &  - 6.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR5(0,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,-1,1,-1,1)
      Hi6(0,-1,1,-1,1,-1) = 
     &  + 1.0703618543335311d-01*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.1407237086670622d-01*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  - 2.1407237086670622d-01*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,-1)
      Hi6(0,-1,1,-1,1,1) = 
     &  + 2.6860659680402010d-01*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 5.3721319360804020d-01*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1(0) *HR2(-1,1)*HR2(0,-1)
     &  - HR1(0) *HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,1)
     &  - HR1(0) *HR4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  - 4.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - 5.3721319360804020d-01*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,0,-1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  + HR2(0, -1)*HR3(0,-1,-1)
     &  + HR2(0, -1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  - HR5( -1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - HR5(0, -1,0,-1,-1)
     &  - HR5(0, -1,0,-1,1)
     &  + HR5(0, -1,1,-1,-1)
      Hi6(0,-1,1,0,1,-1) = 
     &  + 2.5410760640234242d-01*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 5.0821521280468485d-01*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  - 5.0821521280468485d-01*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR2(0,1)
     &  + 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR4(0,-1,1,1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,1)
      Hi6(0,-1,1,0,1,1) = 
     &  + 6.0102845157979714d-01*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,0,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,1,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(0)*HR3(0,1,-1)
     &  + 1.2020569031595942d+00*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR4(0,-1,0,1)
     &  - HR1( -1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + HR1(0) *HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1(0) *HR1(1)*HR3(0,-1,1)
     &  + HR1(0) *HR1(1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + HR1(0) *HR2(-1,1)*HR2(0,-1)
     &  + HR1(0) *HR2(-1,1)*HR2(0,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,-1,1,1)
     &  - HR1(0) *HR4(0,1,-1,-1)
     &  - HR1(0) *HR4(0,1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(1) *HR4(0,-1,0,1)
     &  - HR1(1) *HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - 1.2020569031595942d+00*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,0,-1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,0,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(0,1,-1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  + HR2(0, -1)*HR3(0,-1,-1)
     &  + HR2(0, -1)*HR3(0,-1,1)
     &  - HR2(0,1) *HR3(-1,-1,1)
     &  - 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR4(0,-1,1,1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,1,-1,1)
     &  + 7.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + HR5(0, -1,-1,0,1)
     &  + 3.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - HR5(0, -1,0,-1,-1)
     &  + HR5(0, -1,0,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,0,1,1)
     &  + 3.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  + HR5(0, -1,1,-1,1)
     &  + HR5(0, -1,1,0,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HR5(0,0,-1,1,1)
     &  + 2.0000000000000000d+00*HR5(0,0,1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,0,1,-1,1)
     &  + 4.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  + HR5(0,1, -1,1,-1)
      Hi6(0,-1,1,1,-1,-1) = 
     &  - 2.3766885054564933d-01*HR1(-1)*HR1(-1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 4.7533770109129867d-01*HR1(-1)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR2(0,-1)
     &  + 4.7533770109129867d-01*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR3(0,-1,1)
      Hi6(0,-1,1,1,-1,1) = 
     &  - 7.3900238327152102d-01*HR1(-1)*HR1(-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.4780047665430420d+00*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  + 5.8224052646501250d-01*HR1(0)*HR2(-1,1)
     &  + 5.8224052646501250d-01*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 1.4780047665430420d+00*HR2(-1,1)
     &  + HR2( -1,1)*HR3(0,-1,-1)
     &  + HR2( -1,1)*HR3(0,0,-1)
     &  + HR2(0, -1)*HR3(-1,-1,1)
     &  - HR2(0, -1)*HR3(0,-1,-1)
     &  - HR2(0, -1)*HR3(0,-1,1)
     &  + 5.8224052646501250d-01*HR3(-1,-1,1)
     &  - 5.8224052646501250d-01*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR3(0,-1,1)
     &  - HR5( -1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + HR5(0, -1,-1,1,-1)
     &  + HR5(0, -1,0,-1,-1)
     &  + HR5(0, -1,0,-1,1)
     &  + HR5(0, -1,1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,0,-1,-1,1)
     &  + HR5(0,0, -1,1,-1)
      Hi6(0,-1,1,1,0,1) = 
     &  - 1.7721476084810206d+00*HR1(-1)*HR1(-1)
     &  + 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(0)*HR1(1)
     &  - 3.5442952169620413d+00*HR1(-1)*HR1(1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,1)
     &  - HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR2(-1,1)*HR2(0,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,0,1)
     &  + HR1( -1)*HR4(0,0,-1,1)
     &  + HR1( -1)*HR4(0,0,1,-1)
     &  + HR1( -1)*HR4(0,1,-1,-1)
     &  + 1.6449340668482264d+00*HR1(0)*HR2(-1,1)
     &  + 1.6449340668482264d+00*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(1) *HR4(-1,-1,-1,1)
     &  - HR1(1) *HR4(0,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,0,1)
     &  + HR1(1) *HR4(0,0,-1,1)
     &  + HR1(1) *HR4(0,0,1,-1)
     &  + HR1(1) *HR4(0,1,-1,-1)
     &  + 3.5442952169620413d+00*HR2(-1,1)
     &  + HR2( -1,1)*HR3(-1,-1,1)
     &  + HR2( -1,1)*HR3(0,-1,-1)
     &  + HR2( -1,1)*HR3(0,0,-1)
     &  + HR2( -1,1)*HR3(0,0,1)
     &  + HR2( -1,1)*HR3(0,1,-1)
     &  + HR2(0, -1)*HR3(-1,-1,1)
     &  - HR2(0, -1)*HR3(0,-1,-1)
     &  - HR2(0, -1)*HR3(0,-1,1)
     &  + HR2(0,1) *HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR3(-1,-1,1)
     &  - 1.6449340668482264d+00*HR3(0,-1,-1)
     &  - 1.6449340668482264d+00*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HR5(-1,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - HR5(0, -1,-1,0,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,1)
     &  + HR5(0, -1,0,-1,-1)
     &  - HR5(0, -1,0,1,-1)
     &  - 2.0000000000000000d+00*HR5(0,-1,0,1,1)
     &  + HR5(0, -1,1,-1,-1)
     &  + HR5(0, -1,1,-1,1)
     &  - HR5(0, -1,1,0,1)
     &  + 3.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  + HR5(0,0, -1,-1,1)
     &  - 2.0000000000000000d+00*HR5(0,0,-1,1,1)
     &  - HR5(0,0,1, -1,-1)
     &  - HR5(0,0,1, -1,1)
     &  - HR5(0,1, -1,-1,-1)
     &  - HR5(0,1, -1,-1,1)
      Hi6(0,-1,1,1,1,-1) = 
     &  - 1.6438755856693088d-02*HR1(-1)*HR1(-1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(0)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 3.2877511713386177d-02*HR1(-1)*HR1(1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR2(-1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,0,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR1(0)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,-1,1)
     &  - 1.0626935403832139d+00*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,0,-1)
     &  + 3.2877511713386177d-02*HR2(-1,1)
     &  - 1.0626935403832139d+00*HR3(-1,-1,1)
     &  + 1.0626935403832139d+00*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,0,-1,1)
      Hi6(0,-1,1,1,1,1) = 
     &  + 5.4233865098901558d-01*HR1(-1)*HR1(-1)
     &  - 2.3411792664818762d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 7.0235377994456286d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 7.0235377994456286d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 1.4047075598891257d+00*HR1(-1)*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,0,-1)
     &  + 1.0846773019780311d+00*HR1(-1)*HR1(1)
     &  + 1.4047075598891257d+00*HR1(-1)*HR2(-1,1)
     &  - 1.4047075598891257d+00*HR1(-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,0,-1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR4(0,0,0,-1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(1)*HR3(0,0,-1)
     &  - 1.4047075598891257d+00*HR1(0)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,-1,1)
     &  + HR1(0) *HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,-1,-1,1)
     &  + HR1(0) *HR4(0,-1,1,-1)
     &  + HR1(0) *HR4(0,0,-1,-1)
     &  + HR1(0) *HR4(0,0,-1,1)
     &  - 1.4047075598891257d+00*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,0,-1)
     &  + HR1(1) *HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,0,-1,-1)
     &  + HR1(1) *HR4(0,0,0,-1)
     &  - 1.0846773019780311d+00*HR2(-1,1)
     &  - 1.4047075598891257d+00*HR3(-1,-1,1)
     &  + 1.4047075598891257d+00*HR3(0,-1,-1)
     &  + 1.4047075598891257d+00*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,-1,1)
     &  + HR5( -1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,1)
     &  - HR5(0, -1,-1,1,-1)
     &  - HR5(0, -1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  - HR5(0,0, -1,-1,1)
     &  - HR5(0,0, -1,1,-1)
     &  - HR5(0,0,0, -1,-1)
     &  - HR5(0,0,0, -1,1)
      Hi6(0,0,-1,-1,-1,-1) = 
     &  0.d0 
      Hi6(0,0,-1,-1,-1,1) = 
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,1,1)
     &  + HR1(1) *HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - HR5( -1,-1,-1,1,1)
      Hi6(0,0,-1,-1,0,-1) = 
     &  0.d0 
      Hi6(0,0,-1,-1,0,1) = 
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - HR2( -1,1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + HR5( -1,-1,-1,1,1)
     &  + HR5( -1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,1,1,1)
     &  + HR5( -1,1,-1,1,1)
      Hi6(0,0,-1,-1,1,-1) = 
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,1,1)
      Hi6(0,0,-1,-1,1,1) = 
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  + HR1(0) *HR4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  - HR1(1) *HR4(-1,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,-1,1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  + HR2(0, -1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + HR5( -1,-1,-1,1,1)
     &  - HR5(0, -1,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,1)
     &  - HR5(0, -1,-1,1,-1)
     &  - HR5(0, -1,-1,1,1)
      Hi6(0,0,-1,0,-1,-1) = 
     &  0.d0 
      Hi6(0,0,-1,0,-1,1) = 
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(-1,1,1,1)
     &  + HR1(1) *HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + 4.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(-1,-1,1,1)
     &  + HR2( -1,1)*HR3(-1,1,1)
     &  + 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  - HR5( -1,-1,1,-1,1)
     &  - HR5( -1,1,-1,1,1)
      Hi6(0,0,-1,0,0,1) = 
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(-1,1,1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(-1,1,1,1)
     &  + 6.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - 6.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  + 6.0000000000000000d+00*HR5(-1,-1,1,1,1)
     &  - 6.0000000000000000d+00*HR5(-1,1,1,1,1)
      Hi6(0,0,-1,0,1,-1) = 
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(-1,1,1)
     &  + 2.0794415416798359d+00*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR4(-1,1,1,1)
      Hi6(0,0,-1,0,1,1) = 
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(-1,1,1,1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR4(0,1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(-1,1,1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - HR1(1) *HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  - HR1(1) *HR2(-1,1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(-1,1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(-1,-1,1,1)
     &  + HR1(1) *HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,-1,1)
     &  + HR1(1) *HR4(0,1,-1,-1)
     &  + HR1(1) *HR4(0,1,-1,1)
     &  - HR2( -1,1)*HR3(-1,1,1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  + HR2(0, -1)*HR3(-1,1,1)
     &  - HR2(0,1) *HR3(-1,-1,1)
     &  + HR2(0,1) *HR3(-1,1,1)
     &  - 2.0794415416798359d+00*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR4(-1,1,1,1)
     &  - 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  + HR5( -1,-1,1,-1,1)
     &  + HR5( -1,1,-1,1,1)
     &  - HR5(0, -1,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,1)
     &  - HR5(0, -1,-1,1,-1)
     &  - HR5(0, -1,-1,1,1)
     &  - HR5(0,1, -1,-1,-1)
     &  - HR5(0,1, -1,-1,1)
     &  - HR5(0,1, -1,1,-1)
     &  - HR5(0,1, -1,1,1)
      Hi6(0,0,-1,1,-1,-1) = 
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR3(-1,1,1)
      Hi6(0,0,-1,1,-1,1) = 
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR4(0,-1,1,-1)
     &  + HR1(1) *HR1(1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR1(1)*HR2(-1,1)
     &  + HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  - HR1(1) *HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,1)
     &  - HR1(1) *HR4(0,-1,1,-1)
     &  + HR2(0, -1)*HR3(-1,-1,1)
     &  - HR2(0, -1)*HR3(-1,1,1)
     &  + 5.8224052646501250d-01*HR3(-1,-1,1)
     &  - 5.8224052646501250d-01*HR3(-1,1,1)
     &  - 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + HR5( -1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  + HR5(0, -1,1,-1,1)
     &  + HR5(0, -1,1,1,-1)
      Hi6(0,0,-1,1,0,-1) = 
     &  - 1.3707783890401886d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1123351671205660d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 8.2246703342411321d-01*HR1(1)*HR2(-1,1)
     &  + 8.2246703342411321d-01*HR3(-1,-1,1)
     &  - 8.2246703342411321d-01*HR3(-1,1,1)
      Hi6(0,0,-1,1,0,1) = 
     &  - 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,1,1)
     &  - HR1( -1)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR4(0,1,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1(1) *HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 1.6449340668482264d+00*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  + HR1(1) *HR2(-1,1)*HR2(0,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,-1,1,1)
     &  - HR1(1) *HR4(0,1,-1,-1)
     &  - HR1(1) *HR4(0,1,-1,1)
     &  + HR2( -1,1)*HR3(-1,1,1)
     &  + HR2(0, -1)*HR3(-1,-1,1)
     &  - HR2(0, -1)*HR3(-1,1,1)
     &  + HR2(0,1) *HR3(-1,-1,1)
     &  - HR2(0,1) *HR3(-1,1,1)
     &  + 1.6449340668482264d+00*HR3(-1,-1,1)
     &  - 1.6449340668482264d+00*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - HR5( -1,-1,-1,1,1)
     &  - HR5( -1,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR5(-1,-1,1,1,1)
     &  - HR5( -1,1,-1,1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR5(0,-1,1,1,-1)
     &  + 3.0000000000000000d+00*HR5(0,-1,1,1,1)
     &  + HR5(0,1, -1,-1,-1)
     &  + HR5(0,1, -1,-1,1)
     &  + HR5(0,1, -1,1,-1)
     &  + HR5(0,1, -1,1,1)
      Hi6(0,0,-1,1,1,-1) = 
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(-1,1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,1)
     &  - 1.0626935403832139d+00*HR3(-1,-1,1)
     &  + 1.0626935403832139d+00*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,1)
      Hi6(0,0,-1,1,1,1) = 
     &  + 2.3411792664818762d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 7.0235377994456286d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,1)
     &  + 7.0235377994456286d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  + HR1(0) *HR1(1)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR4(-1,-1,1,1)
     &  + HR1(0) *HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,-1,-1,1)
     &  + HR1(0) *HR4(0,-1,1,-1)
     &  + HR1(0) *HR4(0,-1,1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 1.4047075598891257d+00*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,1)
     &  + HR1(1) *HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,1,-1)
     &  + HR1(1) *HR4(0,0,-1,-1)
     &  + HR1(1) *HR4(0,0,-1,1)
     &  - 1.4047075598891257d+00*HR3(-1,-1,1)
     &  + 1.4047075598891257d+00*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - HR5( -1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - HR5(0, -1,-1,1,1)
     &  - 2.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  - HR5(0, -1,1,-1,1)
     &  - HR5(0, -1,1,1,-1)
     &  - HR5(0,0, -1,-1,-1)
     &  - HR5(0,0, -1,-1,1)
     &  - HR5(0,0, -1,1,-1)
     &  - HR5(0,0, -1,1,1)
      Hi6(0,0,0,-1,-1,-1) = 
     &  0.d0 
      Hi6(0,0,0,-1,-1,1) = 
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(-1,1,1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR4(-1,1,1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1(1) *HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  - HR5( -1,-1,1,1,1)
      Hi6(0,0,0,-1,0,-1) = 
     &  0.d0 
      Hi6(0,0,0,-1,0,1) = 
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(-1,1,1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR4(-1,1,1,1)
     &  - 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - HR1(1) *HR1(1)*HR3(-1,-1,1)
     &  + HR1(1) *HR1(1)*HR3(-1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(-1,1,1,1)
     &  - 4.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR5(-1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HR5(-1,1,1,1,1)
      Hi6(0,0,0,-1,1,-1) = 
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR4(-1,1,1,1)
      Hi6(0,0,0,-1,1,1) = 
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR4(-1,1,1,1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR4(0,-1,1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + HR1(0) *HR1(1)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(1)*HR3(-1,1,1)
     &  + HR1(0) *HR4(-1,-1,-1,1)
     &  - HR1(0) *HR4(-1,-1,1,1)
     &  + HR1(0) *HR4(-1,1,1,1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(1) *HR4(-1,-1,1,1)
     &  + HR1(1) *HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,1,-1)
     &  + HR1(1) *HR4(0,-1,1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR4(-1,1,1,1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  + HR5( -1,-1,1,1,1)
     &  - HR5(0, -1,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,1)
     &  - HR5(0, -1,-1,1,-1)
     &  - HR5(0, -1,-1,1,1)
     &  - HR5(0, -1,1,-1,-1)
     &  - HR5(0, -1,1,-1,1)
     &  - HR5(0, -1,1,1,-1)
     &  - HR5(0, -1,1,1,1)
      Hi6(0,0,0,0,-1,-1) = 
     &  0.d0 
      Hi6(0,0,0,0,-1,1) = 
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  + HR1(1) *HR4(-1,-1,-1,1)
     &  - HR1(1) *HR4(-1,-1,1,1)
     &  + HR1(1) *HR4(-1,1,1,1)
     &  + HR5( -1,-1,-1,-1,1)
     &  - HR5( -1,-1,-1,1,1)
     &  + HR5( -1,-1,1,1,1)
     &  - HR5( -1,1,1,1,1)
      Hi6(0,0,0,0,0,-1) = 
     &  0.d0 
      Hi6(0,0,0,0,0,1) = 
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 8.3333333333333333d-03*HR1(1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
      Hi6(0,0,0,0,1,-1) = 
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 2.8881132523331054d-02*HR1(1)*HR1(1)*HR1(1)*HR1(1)
      Hi6(0,0,0,0,1,1) = 
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,1,1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR4(0,-1,1,1)
     &  + HR1( -1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR4(0,1,-1,1)
     &  + HR1( -1)*HR4(0,1,1,-1)
     &  + HR1( -1)*HR4(0,1,1,1)
     &  - 4.1666666666666666d-02*HR1(0)*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 2.8881132523331054d-02*HR1(1)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,1,1)
     &  - HR1(1) *HR4(-1,-1,-1,1)
     &  + HR1(1) *HR4(-1,-1,1,1)
     &  - HR1(1) *HR4(-1,1,1,1)
     &  + HR1(1) *HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,1,-1)
     &  + HR1(1) *HR4(0,-1,1,1)
     &  + HR1(1) *HR4(0,1,-1,-1)
     &  + HR1(1) *HR4(0,1,-1,1)
     &  + HR1(1) *HR4(0,1,1,-1)
     &  + HR1(1) *HR4(0,1,1,1)
     &  - HR5( -1,-1,-1,-1,1)
     &  + HR5( -1,-1,-1,1,1)
     &  - HR5( -1,-1,1,1,1)
     &  + HR5( -1,1,1,1,1)
     &  - HR5(0, -1,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,1)
     &  - HR5(0, -1,-1,1,-1)
     &  - HR5(0, -1,-1,1,1)
     &  - HR5(0, -1,1,-1,-1)
     &  - HR5(0, -1,1,-1,1)
     &  - HR5(0, -1,1,1,-1)
     &  - HR5(0, -1,1,1,1)
     &  - HR5(0,1, -1,-1,-1)
     &  - HR5(0,1, -1,-1,1)
     &  - HR5(0,1, -1,1,-1)
     &  - HR5(0,1, -1,1,1)
     &  - HR5(0,1,1, -1,-1)
     &  - HR5(0,1,1, -1,1)
     &  - HR5(0,1,1,1, -1)
     &  - HR5(0,1,1,1,1) 
      Hi6(0,0,0,1,-1,-1) = 
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 4.0037751159850118d-02*HR1(1)*HR1(1)*HR1(1)
      Hi6(0,0,0,1,-1,1) = 
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR4(-1,1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  - HR1( -1)*HR4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - HR1( -1)*HR4(0,1,-1,1)
     &  - HR1( -1)*HR4(0,1,1,-1)
     &  - 9.7040087744168750d-02*HR1(1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1(1) *HR1(1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - HR1(1) *HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,-1,1,-1)
     &  - HR1(1) *HR4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - HR1(1) *HR4(0,1,-1,1)
     &  - HR1(1) *HR4(0,1,1,-1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  + HR5( -1,-1,1,1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,1,1,-1)
     &  + HR5(0, -1,1,1,1)
     &  + 3.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,1,-1,1,-1)
     &  + HR5(0,1, -1,1,1)
     &  + 2.0000000000000000d+00*HR5(0,1,1,-1,-1)
     &  + HR5(0,1,1, -1,1)
     &  + HR5(0,1,1,1, -1)
      Hi6(0,0,0,1,0,-1) = 
     &  - 1.3707783890401886d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.1123351671205660d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.3707783890401886d-01*HR1(1)*HR1(1)*HR1(1)
      Hi6(0,0,0,1,0,1) = 
     &  - 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,1,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,1,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,1,1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,1,1)
     &  - HR1( -1)*HR4(-1,1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,1,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,1,1,1)
     &  - 2.7415567780803773d-01*HR1(1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(1)*HR1(1)*HR1(1)*HR2(0,1)
     &  + HR1(1) *HR1(1)*HR3(-1,-1,1)
     &  - HR1(1) *HR1(1)*HR3(-1,1,1)
     &  + HR1(1) *HR1(1)*HR3(0,-1,-1)
     &  + HR1(1) *HR1(1)*HR3(0,-1,1)
     &  + HR1(1) *HR1(1)*HR3(0,1,-1)
     &  + HR1(1) *HR1(1)*HR3(0,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(-1,1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,1,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,1,1,1)
     &  + 4.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR5(-1,-1,1,1,1)
     &  - 4.0000000000000000d+00*HR5(-1,1,1,1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,-1,1,-1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,1,1,-1)
     &  + 4.0000000000000000d+00*HR5(0,-1,1,1,1)
     &  + 4.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR5(0,1,-1,1,-1)
     &  + 4.0000000000000000d+00*HR5(0,1,-1,1,1)
     &  + 4.0000000000000000d+00*HR5(0,1,1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,1,1,-1,1)
     &  + 4.0000000000000000d+00*HR5(0,1,1,1,-1)
     &  + 4.0000000000000000d+00*HR5(0,1,1,1,1)
      Hi6(0,0,0,1,1,-1) = 
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,1,1)
     &  + 1.1552453009332421d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  + 1.7711559006386898d-01*HR1(1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  + 6.9314718055994530d-01*HR4(-1,1,1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,1,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,1,1)
      Hi6(0,0,0,1,1,1) = 
     &  + 2.3411792664818762d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 7.0235377994456286d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(0)*HR3(0,1,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,1,1)
     &  + 7.0235377994456286d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  + HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,1)
     &  - HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,1,1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR4(-1,1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR4(0,0,-1,1)
     &  + HR1( -1)*HR4(0,0,1,-1)
     &  + HR1( -1)*HR4(0,0,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR4(0,1,-1,1)
     &  + HR1( -1)*HR4(0,1,1,-1)
     &  - 8.3333333333333333d-02*HR1(0)*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,1)
     &  - HR1(0) *HR1(1)*HR3(-1,-1,1)
     &  + HR1(0) *HR1(1)*HR3(-1,1,1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,1)
     &  - HR1(0) *HR1(1)*HR3(0,1,-1)
     &  - HR1(0) *HR1(1)*HR3(0,1,1)
     &  - HR1(0) *HR4(-1,-1,-1,1)
     &  + HR1(0) *HR4(-1,-1,1,1)
     &  - HR1(0) *HR4(-1,1,1,1)
     &  + HR1(0) *HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,-1,-1,1)
     &  + HR1(0) *HR4(0,-1,1,-1)
     &  + HR1(0) *HR4(0,-1,1,1)
     &  + HR1(0) *HR4(0,1,-1,-1)
     &  + HR1(0) *HR4(0,1,-1,1)
     &  + HR1(0) *HR4(0,1,1,-1)
     &  + HR1(0) *HR4(0,1,1,1)
     &  + 2.3411792664818762d-01*HR1(1)*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,0,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1(1) *HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,1,-1)
     &  + HR1(1) *HR4(0,0,-1,-1)
     &  + HR1(1) *HR4(0,0,-1,1)
     &  + HR1(1) *HR4(0,0,1,-1)
     &  + HR1(1) *HR4(0,0,1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1(1) *HR4(0,1,-1,1)
     &  + HR1(1) *HR4(0,1,1,-1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HR4(-1,1,1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,1,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,1,1)
     &  - 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  - HR5( -1,-1,1,1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - HR5(0, -1,-1,1,1)
     &  - 2.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  - HR5(0, -1,1,-1,1)
     &  - HR5(0, -1,1,1,-1)
     &  - HR5(0,0, -1,-1,-1)
     &  - HR5(0,0, -1,-1,1)
     &  - HR5(0,0, -1,1,-1)
     &  - HR5(0,0, -1,1,1)
     &  - HR5(0,0,1, -1,-1)
     &  - HR5(0,0,1, -1,1)
     &  - HR5(0,0,1,1, -1)
     &  - HR5(0,0,1,1,1) 
     &  - 3.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR5(0,1,-1,1,-1)
     &  - HR5(0,1, -1,1,1)
     &  - 2.0000000000000000d+00*HR5(0,1,1,-1,-1)
     &  - HR5(0,1,1, -1,1)
     &  - HR5(0,1,1,1, -1)
      Hi6(0,0,1,-1,-1,-1) = 
     &  + 2.7752054332410789d-02*HR1(-1)*HR1(-1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR1(1)
     &  + 2.7752054332410789d-02*HR1(1)*HR1(1)
      Hi6(0,0,1,-1,-1,1) = 
     &  + 4.7376502115063852d-02*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 9.4753004230127705d-02*HR1(-1)*HR1(1)
     &  + HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,1,-1)
     &  + HR1( -1)*HR4(0,1,-1,-1)
     &  + 4.7376502115063852d-02*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1(1) *HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,1,-1)
     &  + HR1(1) *HR4(0,1,-1,-1)
     &  - 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + HR5( -1,-1,-1,1,1)
     &  - 6.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - HR5(0, -1,-1,1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  - HR5(0, -1,1,-1,1)
     &  - HR5(0, -1,1,1,-1)
     &  - 3.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  - HR5(0,1, -1,-1,1)
     &  - HR5(0,1, -1,1,-1)
     &  - HR5(0,1,1, -1,-1)
      Hi6(0,0,1,-1,0,-1) = 
     &  + 1.3478823976576390d-01*HR1(-1)*HR1(-1)
     &  + 2.6957647953152780d-01*HR1(-1)*HR1(1)
     &  + 1.3478823976576390d-01*HR1(1)*HR1(1)
      Hi6(0,0,1,-1,0,1) = 
     &  + 1.9444792308405316d-01*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  + 3.8889584616810632d-01*HR1(-1)*HR1(1)
     &  + HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,1,1)
     &  + HR1( -1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR4(0,1,-1,1)
     &  + 1.9444792308405316d-01*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,-1,1,1)
     &  + HR1(1) *HR4(0,1,-1,-1)
     &  + HR1(1) *HR4(0,1,-1,1)
     &  + HR2( -1,1)*HR3(-1,1,1)
     &  - 2.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - HR5( -1,-1,-1,1,1)
     &  - HR5( -1,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR5(-1,-1,1,1,1)
     &  - HR5( -1,1,-1,1,1)
     &  - 6.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 6.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - 5.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - 5.0000000000000000d+00*HR5(0,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  - 4.0000000000000000d+00*HR5(0,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,1,1,-1)
     &  - 3.0000000000000000d+00*HR5(0,-1,1,1,1)
     &  - 3.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR5(0,1,-1,1,-1)
     &  - 2.0000000000000000d+00*HR5(0,1,-1,1,1)
     &  - HR5(0,1,1, -1,-1)
     &  - HR5(0,1,1, -1,1)
      Hi6(0,0,1,-1,1,-1) = 
     &  + 1.0703618543335311d-01*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.1407237086670622d-01*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  + 1.0703618543335311d-01*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,1)
     &  + 1.3862943611198906d+00*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,1,1,-1)
      Hi6(0,0,1,-1,1,1) = 
     &  + 2.6860659680402010d-01*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR3(0,-1,1)
     &  + HR1( -1)*HR1(0)*HR3(0,1,-1)
     &  + 5.3721319360804020d-01*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  + HR1( -1)*HR4(0,-1,0,1)
     &  - HR1( -1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR4(0,1,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + HR1(0) *HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + HR1(0) *HR1(1)*HR3(0,-1,1)
     &  + HR1(0) *HR1(1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,-1,1,-1)
     &  - HR1(0) *HR4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,1,-1,-1)
     &  - HR1(0) *HR4(0,1,-1,1)
     &  - HR1(0) *HR4(0,1,1,-1)
     &  + 2.6860659680402010d-01*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + HR1(1) *HR1(1)*HR3(0,-1,-1)
     &  + HR1(1) *HR1(1)*HR3(0,0,-1)
     &  + HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(1) *HR2(0,-1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  + HR1(1) *HR4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,0,1)
     &  - HR1(1) *HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - HR1(1) *HR4(0,1,-1,-1)
     &  + HR2(0, -1)*HR3(-1,-1,1)
     &  - HR2(0, -1)*HR3(-1,1,1)
     &  + HR2(0, -1)*HR3(0,-1,-1)
     &  + HR2(0, -1)*HR3(0,-1,1)
     &  + HR2(0, -1)*HR3(0,1,-1)
     &  + HR2(0, -1)*HR3(0,1,1)
     &  + 1.3862943611198906d+00*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,1)
     &  - 1.3862943611198906d+00*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,1,1,-1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - HR5( -1,-1,-1,1,1)
     &  + 7.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,1)
     &  - HR5(0, -1,0,-1,-1)
     &  - HR5(0, -1,0,-1,1)
     &  - HR5(0, -1,0,1,-1)
     &  - HR5(0, -1,0,1,1)
     &  + 3.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  + HR5(0, -1,1,-1,1)
     &  + HR5(0, -1,1,1,-1)
     &  + 3.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  + HR5(0,1, -1,-1,1)
     &  + HR5(0,1, -1,1,-1)
     &  + HR5(0,1,1, -1,-1)
      Hi6(0,0,1,0,-1,-1) = 
     &  + 7.5128556447474642d-02*HR1(-1)*HR1(-1)
     &  + 1.5025711289494928d-01*HR1(-1)*HR1(1)
     &  + 7.5128556447474642d-02*HR1(1)*HR1(1)
      Hi6(0,0,1,0,-1,1) = 
     &  + 1.2153517583503078d-01*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 2.4307035167006157d-01*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(-1,1,1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + HR1( -1)*HR4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,1,1,-1)
     &  + 1.2153517583503078d-01*HR1(1)*HR1(1)
     &  - HR1(1) *HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,-1,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + HR1(1) *HR4(0,1,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,1,1,-1)
     &  - HR2( -1,1)*HR3(-1,1,1)
     &  - 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  + HR5( -1,-1,1,-1,1)
     &  + HR5( -1,1,-1,1,1)
     &  - 6.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - HR5(0, -1,-1,1,1)
     &  - 5.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,1,1,-1)
     &  - 6.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR5(0,1,-1,1,-1)
     &  - HR5(0,1, -1,1,1)
     &  - 5.0000000000000000d+00*HR5(0,1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,1,1,-1,1)
     &  - 3.0000000000000000d+00*HR5(0,1,1,1,-1)
      Hi6(0,0,1,0,1,-1) = 
     &  + 2.5410760640234242d-01*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 5.0821521280468485d-01*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,1,1)
     &  + 2.5410760640234242d-01*HR1(1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,1,1)
     &  - 2.0794415416798359d+00*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR4(-1,-1,1,1)
     &  - 2.0794415416798359d+00*HR4(-1,1,1,1)
     &  + 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR4(0,-1,-1,1)
     &  + 2.0794415416798359d+00*HR4(0,-1,1,-1)
     &  + 2.0794415416798359d+00*HR4(0,-1,1,1)
     &  + 2.0794415416798359d+00*HR4(0,1,-1,-1)
     &  + 2.0794415416798359d+00*HR4(0,1,-1,1)
     &  + 2.0794415416798359d+00*HR4(0,1,1,-1)
     &  + 2.0794415416798359d+00*HR4(0,1,1,1)
      Hi6(0,0,1,0,1,1) = 
     &  + 6.0102845157979714d-01*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(-1)*HR3(0,0,1)
     &  + HR1( -1)*HR1(-1)*HR3(0,1,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(0)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,1,1)
     &  + 1.2020569031595942d+00*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(0,1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(-1,1,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,0,1,1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,1,1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(1)*HR1(1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR4(-1,1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,-1,1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,1,-1,1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,1,1,-1)
     &  - 3.0000000000000000d+00*HR1(0)*HR4(0,1,1,1)
     &  + 6.0102845157979714d-01*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(1)*HR1(1)*HR2(0,1)
     &  + HR1(1) *HR1(1)*HR3(-1,-1,1)
     &  + HR1(1) *HR1(1)*HR3(0,-1,-1)
     &  + HR1(1) *HR1(1)*HR3(0,0,-1)
     &  + HR1(1) *HR1(1)*HR3(0,0,1)
     &  + HR1(1) *HR1(1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  + HR1(1) *HR2(-1,1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(1) *HR2(0,-1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(0,1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,1,1)
     &  + 4.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,0,1,1)
     &  - 4.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,1,1,-1)
     &  + HR2( -1,1)*HR3(-1,1,1)
     &  + HR2(0, -1)*HR3(-1,-1,1)
     &  - HR2(0, -1)*HR3(-1,1,1)
     &  + HR2(0, -1)*HR3(0,-1,-1)
     &  + HR2(0, -1)*HR3(0,-1,1)
     &  + HR2(0, -1)*HR3(0,1,-1)
     &  + HR2(0, -1)*HR3(0,1,1)
     &  + HR2(0,1) *HR3(-1,-1,1)
     &  - HR2(0,1) *HR3(-1,1,1)
     &  + HR2(0,1) *HR3(0,1,-1)
     &  + HR2(0,1) *HR3(0,1,1)
     &  + 2.0794415416798359d+00*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR4(-1,-1,1,1)
     &  + 2.0794415416798359d+00*HR4(-1,1,1,1)
     &  - 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  - 2.0794415416798359d+00*HR4(0,-1,-1,1)
     &  - 2.0794415416798359d+00*HR4(0,-1,1,-1)
     &  - 2.0794415416798359d+00*HR4(0,-1,1,1)
     &  - 2.0794415416798359d+00*HR4(0,1,-1,-1)
     &  - 2.0794415416798359d+00*HR4(0,1,-1,1)
     &  - 2.0794415416798359d+00*HR4(0,1,1,-1)
     &  - 2.0794415416798359d+00*HR4(0,1,1,1)
     &  + 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - 7.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  - HR5( -1,-1,1,-1,1)
     &  - HR5( -1,1,-1,1,1)
     &  + 7.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + HR5(0, -1,-1,0,1)
     &  + 5.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,1)
     &  - HR5(0, -1,0,-1,-1)
     &  + HR5(0, -1,0,1,1)
     &  + 5.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,1,-1,1)
     &  + HR5(0, -1,1,0,1)
     &  + 3.0000000000000000d+00*HR5(0,-1,1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HR5(0,0,-1,1,1)
     &  + 2.0000000000000000d+00*HR5(0,0,1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,0,1,-1,1)
     &  + 7.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  + 4.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  + 5.0000000000000000d+00*HR5(0,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,1,-1,1,1)
     &  - HR5(0,1,0,1, -1)
     &  - HR5(0,1,0,1,1) 
     &  + 5.0000000000000000d+00*HR5(0,1,1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,1,1,-1,1)
     &  + 3.0000000000000000d+00*HR5(0,1,1,1,-1)
      Hi6(0,0,1,1,-1,-1) = 
     &  - 2.3766885054564933d-01*HR1(-1)*HR1(-1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 4.7533770109129867d-01*HR1(-1)*HR1(1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(0,1)
     &  - 1.2011325347955035d-01*HR1(0)*HR1(1)*HR1(1)
     &  - 2.3766885054564933d-01*HR1(1)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(1)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(1)*HR2(0,1)
     &  - 2.4022650695910071d-01*HR3(-1,-1,1)
     &  + 2.4022650695910071d-01*HR3(-1,1,1)
     &  - 2.4022650695910071d-01*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR3(0,-1,1)
     &  - 2.4022650695910071d-01*HR3(0,1,-1)
     &  - 2.4022650695910071d-01*HR3(0,1,1)
      Hi6(0,0,1,1,-1,1) = 
     &  - 7.3900238327152102d-01*HR1(-1)*HR1(-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.4780047665430420d+00*HR1(-1)*HR1(1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1( -1)*HR2(0,-1)*HR2(0,1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR4(0,-1,-1,1)
     &  - HR1( -1)*HR4(0,-1,0,1)
     &  - HR1( -1)*HR4(0,0,-1,1)
     &  - HR1( -1)*HR4(0,0,1,-1)
     &  - HR1( -1)*HR4(0,1,-1,-1)
     &  - 2.9112026323250625d-01*HR1(0)*HR1(1)*HR1(1)
     &  - 7.3900238327152102d-01*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR1(1)*HR2(-1,1)
     &  - HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  + 5.8224052646501250d-01*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1(1) *HR2(0,-1)*HR2(0,1)
     &  + 5.8224052646501250d-01*HR1(1)*HR2(0,1)
     &  + HR1(1) *HR4(-1,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,-1,1)
     &  - HR1(1) *HR4(0,-1,0,1)
     &  - HR1(1) *HR4(0,0,-1,1)
     &  - HR1(1) *HR4(0,0,1,-1)
     &  - HR1(1) *HR4(0,1,-1,-1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  + HR2(0, -1)*HR3(-1,1,1)
     &  - HR2(0, -1)*HR3(0,-1,-1)
     &  - HR2(0, -1)*HR3(0,-1,1)
     &  - HR2(0, -1)*HR3(0,1,-1)
     &  - HR2(0, -1)*HR3(0,1,1)
     &  - 5.8224052646501250d-01*HR3(-1,-1,1)
     &  + 5.8224052646501250d-01*HR3(-1,1,1)
     &  - 5.8224052646501250d-01*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR3(0,-1,1)
     &  - 5.8224052646501250d-01*HR3(0,1,-1)
     &  - 5.8224052646501250d-01*HR3(0,1,1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - HR5( -1,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - HR5(0, -1,-1,1,1)
     &  + HR5(0, -1,0,-1,-1)
     &  + HR5(0, -1,0,-1,1)
     &  + HR5(0, -1,0,1,-1)
     &  + HR5(0, -1,0,1,1)
     &  + HR5(0, -1,1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,0,-1,1,-1)
     &  + HR5(0,0, -1,1,1)
     &  + 2.0000000000000000d+00*HR5(0,0,1,-1,-1)
     &  + HR5(0,0,1, -1,1)
     &  + HR5(0,0,1,1, -1)
     &  + 3.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  + HR5(0,1, -1,-1,1)
     &  + HR5(0,1, -1,1,-1)
     &  + HR5(0,1,1, -1,-1)
      Hi6(0,0,1,1,0,-1) = 
     &  - 6.6068813489808640d-01*HR1(-1)*HR1(-1)
     &  + 1.3707783890401886d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.3213762697961728d+00*HR1(-1)*HR1(1)
     &  + 4.1123351671205660d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR2(0,-1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR2(0,1)
     &  - 4.1123351671205660d-01*HR1(0)*HR1(1)*HR1(1)
     &  - 6.6068813489808640d-01*HR1(1)*HR1(1)
     &  - 8.2246703342411321d-01*HR1(1)*HR2(-1,1)
     &  + 8.2246703342411321d-01*HR1(1)*HR2(0,-1)
     &  + 8.2246703342411321d-01*HR1(1)*HR2(0,1)
     &  - 8.2246703342411321d-01*HR3(-1,-1,1)
     &  + 8.2246703342411321d-01*HR3(-1,1,1)
     &  - 8.2246703342411321d-01*HR3(0,-1,-1)
     &  - 8.2246703342411321d-01*HR3(0,-1,1)
     &  - 8.2246703342411321d-01*HR3(0,1,-1)
     &  - 8.2246703342411321d-01*HR3(0,1,1)
      Hi6(0,0,1,1,0,1) = 
     &  - 1.7721476084810206d+00*HR1(-1)*HR1(-1)
     &  + 2.7415567780803773d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR1(-1)*HR1(0)*HR1(1)
     &  - 3.5442952169620413d+00*HR1(-1)*HR1(1)
     &  + 8.2246703342411321d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(1)*HR1(1)*HR2(0,1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,0,1)
     &  - HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1( -1)*HR2(0,-1)*HR2(0,1)
     &  + 1.6449340668482264d+00*HR1(-1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(0,1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,1,1)
     &  - 8.2246703342411321d-01*HR1(0)*HR1(1)*HR1(1)
     &  - 1.7721476084810206d+00*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,0,1)
     &  - 5.0000000000000000d-01*HR1(1)*HR1(1)*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  - HR1(1) *HR2(-1,1)*HR2(0,1)
     &  + 1.6449340668482264d+00*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1(1) *HR2(0,-1)*HR2(0,1)
     &  + 1.6449340668482264d+00*HR1(1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(1)*HR2(0,1)*HR2(0,1)
     &  - HR2( -1,1)*HR3(-1,1,1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  + HR2(0, -1)*HR3(-1,1,1)
     &  - HR2(0, -1)*HR3(0,-1,-1)
     &  - HR2(0, -1)*HR3(0,-1,1)
     &  - HR2(0, -1)*HR3(0,1,-1)
     &  - HR2(0, -1)*HR3(0,1,1)
     &  - HR2(0,1) *HR3(-1,-1,1)
     &  + HR2(0,1) *HR3(-1,1,1)
     &  - HR2(0,1) *HR3(0,1,-1)
     &  - HR2(0,1) *HR3(0,1,1)
     &  - 1.6449340668482264d+00*HR3(-1,-1,1)
     &  + 1.6449340668482264d+00*HR3(-1,1,1)
     &  - 1.6449340668482264d+00*HR3(0,-1,-1)
     &  - 1.6449340668482264d+00*HR3(0,-1,1)
     &  - 1.6449340668482264d+00*HR3(0,1,-1)
     &  - 1.6449340668482264d+00*HR3(0,1,1)
     &  + 2.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + HR5( -1,-1,-1,1,1)
     &  + HR5( -1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,1,1,1)
     &  + HR5( -1,1,-1,1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - HR5(0, -1,-1,0,1)
     &  + HR5(0, -1,-1,1,-1)
     &  + HR5(0, -1,-1,1,1)
     &  + HR5(0, -1,0,-1,-1)
     &  - HR5(0, -1,0,1,1)
     &  + HR5(0, -1,1,-1,-1)
     &  + HR5(0, -1,1,-1,1)
     &  - HR5(0, -1,1,0,1)
     &  + 3.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  + HR5(0,0, -1,-1,1)
     &  + HR5(0,0, -1,1,-1)
     &  - HR5(0,0, -1,1,1)
     &  + HR5(0,0,1, -1,-1)
     &  + HR5(0,0,1, -1,1)
     &  + 3.0000000000000000d+00*HR5(0,0,1,1,-1)
     &  + 3.0000000000000000d+00*HR5(0,0,1,1,1)
     &  + 2.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  + HR5(0,1, -1,1,-1)
     &  + HR5(0,1, -1,1,1)
     &  + HR5(0,1,0,1, -1)
     &  + HR5(0,1,0,1,1) 
     &  + HR5(0,1,1, -1,-1)
     &  + HR5(0,1,1, -1,1)
      Hi6(0,0,1,1,1,-1) = 
     &  - 1.6438755856693088d-02*HR1(-1)*HR1(-1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 3.2877511713386177d-02*HR1(-1)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  + 1.7328679513998632d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(0)*HR1(1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(-1,1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,1,1)
     &  - 1.6438755856693088d-02*HR1(1)*HR1(1)
     &  + 1.0626935403832139d+00*HR1(1)*HR2(-1,1)
     &  - 1.0626935403832139d+00*HR1(1)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR1(1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  + 1.0626935403832139d+00*HR3(-1,-1,1)
     &  - 1.0626935403832139d+00*HR3(-1,1,1)
     &  + 1.0626935403832139d+00*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR3(0,-1,1)
     &  + 1.0626935403832139d+00*HR3(0,1,-1)
     &  + 1.0626935403832139d+00*HR3(0,1,1)
     &  + 1.3862943611198906d+00*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  - 1.3862943611198906d+00*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,0,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,0,1,1)
     &  - 1.3862943611198906d+00*HR4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,1,1,-1)
      Hi6(0,0,1,1,1,1) = 
     &  + 5.4233865098901558d-01*HR1(-1)*HR1(-1)
     &  - 2.3411792664818762d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  + 7.0235377994456286d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  - 7.0235377994456286d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,1)
     &  + 1.4047075598891257d+00*HR1(-1)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,1)
     &  + HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,0,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,0,1)
     &  - HR1( -1)*HR1(0)*HR3(0,1,-1)
     &  + 1.0846773019780311d+00*HR1(-1)*HR1(1)
     &  - 7.0235377994456286d-01*HR1(-1)*HR1(1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  - 1.4047075598891257d+00*HR1(-1)*HR2(0,-1)
     &  - 1.4047075598891257d+00*HR1(-1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,1,1)
     &  + HR1( -1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,0,-1,-1)
     &  + HR1( -1)*HR4(0,0,0,-1)
     &  + HR1( -1)*HR4(0,0,0,1)
     &  + HR1( -1)*HR4(0,0,1,-1)
     &  + HR1( -1)*HR4(0,1,-1,-1)
     &  - 8.3333333333333333d-02*HR1(0)*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 1.7328679513998632d-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(-1,1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,1,1)
     &  + 7.0235377994456286d-01*HR1(0)*HR1(1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,1)
     &  - HR1(0) *HR1(1)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(1)*HR3(0,0,-1)
     &  - HR1(0) *HR1(1)*HR3(0,0,1)
     &  - HR1(0) *HR1(1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(-1,1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  + HR1(0) *HR4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,-1,-1,1)
     &  + HR1(0) *HR4(0,-1,1,-1)
     &  + HR1(0) *HR4(0,0,-1,-1)
     &  + HR1(0) *HR4(0,0,-1,1)
     &  + HR1(0) *HR4(0,0,1,-1)
     &  + HR1(0) *HR4(0,0,1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR4(0,1,-1,-1)
     &  + HR1(0) *HR4(0,1,-1,1)
     &  + HR1(0) *HR4(0,1,1,-1)
     &  + 5.4233865098901558d-01*HR1(1)*HR1(1)
     &  + 1.4047075598891257d+00*HR1(1)*HR2(-1,1)
     &  - 1.4047075598891257d+00*HR1(1)*HR2(0,-1)
     &  - 1.4047075598891257d+00*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,1,-1)
     &  - HR1(1) *HR4(-1,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,0,-1,-1)
     &  + HR1(1) *HR4(0,0,0,-1)
     &  + HR1(1) *HR4(0,0,0,1)
     &  + HR1(1) *HR4(0,0,1,-1)
     &  + HR1(1) *HR4(0,1,-1,-1)
     &  + 1.4047075598891257d+00*HR3(-1,-1,1)
     &  - 1.4047075598891257d+00*HR3(-1,1,1)
     &  + 1.4047075598891257d+00*HR3(0,-1,-1)
     &  + 1.4047075598891257d+00*HR3(0,-1,1)
     &  + 1.4047075598891257d+00*HR3(0,1,-1)
     &  + 1.4047075598891257d+00*HR3(0,1,1)
     &  - 1.3862943611198906d+00*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,1,1)
     &  + 1.3862943611198906d+00*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,1,1)
     &  + 1.3862943611198906d+00*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,1,1,-1)
     &  - 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + HR5( -1,-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - HR5(0, -1,-1,-1,1)
     &  - HR5(0, -1,-1,1,-1)
     &  - HR5(0, -1,1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  - HR5(0,0, -1,-1,1)
     &  - HR5(0,0, -1,1,-1)
     &  - HR5(0,0,0, -1,-1)
     &  - HR5(0,0,0, -1,1)
     &  - HR5(0,0,0,1, -1)
     &  - HR5(0,0,0,1,1) 
     &  - 2.0000000000000000d+00*HR5(0,0,1,-1,-1)
     &  - HR5(0,0,1, -1,1)
     &  - HR5(0,0,1,1, -1)
     &  - 3.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  - HR5(0,1, -1,-1,1)
     &  - HR5(0,1, -1,1,-1)
     &  - HR5(0,1,1, -1,-1)
      Hi6(0,1,-1,-1,-1,-1) = 
     &  - 9.6181291076284771d-03*HR1(-1)
     &  - 9.6181291076284771d-03*HR1(1)
      Hi6(0,1,-1,-1,-1,1) = 
     &  - 1.4134237214990008d-02*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(0,-1,-1,-1)
     &  - 1.4134237214990008d-02*HR1(1)
     &  - HR1(1) *HR4(0,-1,-1,-1)
     &  + HR5( -1,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + HR5(0, -1,-1,-1,1)
     &  + HR5(0, -1,-1,1,-1)
     &  + HR5(0, -1,1,-1,-1)
     &  + HR5(0,1, -1,-1,-1)
      Hi6(0,1,-1,-1,1,-1) = 
     &  - 2.3275066086727564d-02*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 2.3275066086727564d-02*HR1(1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,-1)
      Hi6(0,1,-1,-1,1,1) = 
     &  - 4.0758239159309251d-02*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,-1,-1,1)
     &  + HR1(0) *HR4(0,-1,1,-1)
     &  + HR1(0) *HR4(0,1,-1,-1)
     &  - 4.0758239159309251d-02*HR1(1)
     &  + 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  - HR2( -1,1)*HR3(0,-1,-1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR3(0,-1,-1)
     &  - HR2(0, -1)*HR3(0,-1,1)
     &  - HR2(0, -1)*HR3(0,1,-1)
     &  + HR2(0,1) *HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  - HR5( -1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - HR5(0, -1,-1,0,1)
     &  - HR5(0, -1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR5(0,-1,0,-1,-1)
     &  - HR5(0, -1,1,-1,-1)
     &  + 6.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  - HR5(0,1, -1,-1,-1)
      Hi6(0,1,-1,0,1,1) = 
     &  - 1.1285749644390297d-01*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,1)
     &  + HR1( -1)*HR4(0,-1,0,1)
     &  + HR1( -1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,1)
     &  - HR1( -1)*HR4(0,1,-1,-1)
     &  + HR1(0) *HR1(1)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + 3.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR4(0,-1,1,1)
     &  + HR1(0) *HR4(0,1,-1,-1)
     &  + HR1(0) *HR4(0,1,-1,1)
     &  - 1.1285749644390297d-01*HR1(1)
     &  + 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + HR1(1) *HR4(0,-1,0,1)
     &  + HR1(1) *HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,1)
     &  - HR1(1) *HR4(0,1,-1,-1)
     &  - 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
     &  - 2.0000000000000000d+00*HR2(-1,1)*HR3(-1,-1,1)
     &  - HR2( -1,1)*HR3(0,-1,-1)
     &  - HR2( -1,1)*HR3(0,1,-1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR3(0,-1,-1)
     &  - HR2(0, -1)*HR3(0,-1,1)
     &  - HR2(0, -1)*HR3(0,1,-1)
     &  - HR2(0,1) *HR3(-1,-1,1)
     &  + HR2(0,1) *HR3(0,-1,-1)
     &  - HR2(0,1) *HR3(0,1,-1)
     &  + 2.0794415416798359d+00*HR4(0,-1,-1,-1)
     &  + 2.0794415416798359d+00*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR4(0,-1,1,1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + 6.0000000000000000d+00*HR5(-1,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HR5(-1,-1,1,-1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,0,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HR5(0,-1,0,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,-1,0,-1,1)
     &  - HR5(0, -1,0,1,-1)
     &  - 2.0000000000000000d+00*HR5(0,-1,0,1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  - HR5(0, -1,1,-1,1)
     &  - HR5(0, -1,1,0,1)
     &  - 2.0000000000000000d+00*HR5(0,-1,1,1,-1)
     &  + 6.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR5(0,0,-1,1,-1)
     &  - 4.0000000000000000d+00*HR5(0,0,-1,1,1)
     &  + 4.0000000000000000d+00*HR5(0,0,1,1,-1)
     &  + HR5(0,1, -1,-1,1)
     &  - HR5(0,1, -1,1,-1)
     &  + 2.0000000000000000d+00*HR5(0,1,0,1,-1)
      Hi6(0,1,-1,1,-1,-1) = 
     &  - 5.0916764064292634d-02*HR1(-1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR2(0,-1)
     &  - 5.0916764064292634d-02*HR1(1)
     &  - 2.4022650695910071d-01*HR1(1)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR3(-1,-1,1)
     &  + 4.8045301391820142d-01*HR3(0,-1,-1)
     &  + 2.4022650695910071d-01*HR3(0,-1,1)
     &  + 2.4022650695910071d-01*HR3(0,1,-1)
      Hi6(0,1,-1,1,-1,1) = 
     &  - 8.7985537010508960d-02*HR1(-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 8.7985537010508960d-02*HR1(1)
     &  - 5.8224052646501250d-01*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  + 4.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR2(0, -1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR3(0,-1,-1)
     &  + HR2(0, -1)*HR3(0,-1,1)
     &  + HR2(0, -1)*HR3(0,1,-1)
     &  - 2.0000000000000000d+00*HR2(0,1)*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR3(-1,-1,1)
     &  + 1.1644810529300250d+00*HR3(0,-1,-1)
     &  + 5.8224052646501250d-01*HR3(0,-1,1)
     &  + 5.8224052646501250d-01*HR3(0,1,-1)
     &  - HR5( -1,-1,-1,-1,1)
     &  - 8.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR5(0,-1,0,-1,-1)
     &  + HR5(0, -1,0,-1,1)
     &  + HR5(0, -1,0,1,-1)
     &  - HR5(0, -1,1,-1,-1)
     &  - 1.2000000000000000d+01*HR5(0,0,-1,-1,-1)
     &  - HR5(0,1, -1,-1,-1)
      Hi6(0,1,-1,1,1,-1) = 
     &  - 2.0286579517988963d-01*HR1(-1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR2(-1,1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,1,-1)
     &  - 2.0286579517988963d-01*HR1(1)
     &  + 1.0626935403832139d+00*HR1(1)*HR2(0,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR2(0,1)
     &  - 1.0626935403832139d+00*HR3(-1,-1,1)
     &  - 2.1253870807664278d+00*HR3(0,-1,-1)
     &  - 1.0626935403832139d+00*HR3(0,-1,1)
     &  - 1.0626935403832139d+00*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + 2.7725887222397812d+00*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,0,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,-1)
      Hi6(0,1,-1,1,1,1) = 
     &  - 5.1747906167389938d-01*HR1(-1)
     &  - 2.3411792664818762d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 7.0235377994456286d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  + 1.4047075598891257d+00*HR1(-1)*HR2(-1,1)
     &  + 1.4047075598891257d+00*HR1(-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,0,-1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  + HR1(0) *HR1(0)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,-1,1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + HR1(0) *HR2(-1,1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(0) *HR2(0,-1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,1,-1)
     &  + HR1(0) *HR4(-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,1)
     &  + HR1(0) *HR4(0,-1,0,1)
     &  - HR1(0) *HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,0,-1,-1)
     &  - HR1(0) *HR4(0,1,-1,-1)
     &  - 5.1747906167389938d-01*HR1(1)
     &  + 1.4047075598891257d+00*HR1(1)*HR2(0,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,0,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,0,0,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  - HR2( -1,1)*HR3(0,-1,-1)
     &  - HR2( -1,1)*HR3(0,0,-1)
     &  - 3.4657359027997265d-01*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR2(0,1)
     &  + HR2(0, -1)*HR3(0,0,-1)
     &  + HR2(0,1) *HR3(0,-1,-1)
     &  + HR2(0,1) *HR3(0,0,-1)
     &  - 1.4047075598891257d+00*HR3(-1,-1,1)
     &  - 2.8094151197782514d+00*HR3(0,-1,-1)
     &  - 1.4047075598891257d+00*HR3(0,-1,1)
     &  - 1.4047075598891257d+00*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - 2.7725887222397812d+00*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  + HR5( -1,-1,-1,-1,1)
     &  + 7.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - HR5(0, -1,-1,0,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  + HR5(0, -1,0,-1,-1)
     &  - HR5(0, -1,0,-1,1)
     &  - HR5(0, -1,0,1,-1)
     &  + HR5(0, -1,1,-1,-1)
     &  + 7.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  + HR5(0,0, -1,-1,1)
     &  - HR5(0,0, -1,0,-1)
     &  - HR5(0,0, -1,0,1)
     &  + HR5(0,1, -1,-1,-1)
      Hi6(0,1,0,1,-1,-1) = 
     &  - 1.1412342741606084d-01*HR1(-1)
     &  + 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR1(-1)*HR2(0,1)
     &  - 1.1412342741606084d-01*HR1(1)
     &  + 2.4022650695910071d-01*HR1(1)*HR2(-1,1)
     &  - 2.4022650695910071d-01*HR1(1)*HR2(0,-1)
     &  - 2.4022650695910071d-01*HR1(1)*HR2(0,1)
     &  + 4.8045301391820142d-01*HR3(-1,-1,1)
     &  - 4.8045301391820142d-01*HR3(-1,1,1)
     &  + 4.8045301391820142d-01*HR3(0,-1,-1)
     &  + 4.8045301391820142d-01*HR3(0,-1,1)
     &  + 4.8045301391820142d-01*HR3(0,1,-1)
     &  + 4.8045301391820142d-01*HR3(0,1,1)
      Hi6(0,1,0,1,-1,1) = 
     &  - 1.9355535381306524d-01*HR1(-1)
     &  + 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(-1)*HR3(-1,1,1)
     &  - HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(1)*HR3(0,-1,1)
     &  - HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  - HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1( -1)*HR2(0,-1)*HR2(0,1)
     &  - 5.8224052646501250d-01*HR1(-1)*HR2(0,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  + 4.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + HR1( -1)*HR4(0,-1,0,1)
     &  + HR1( -1)*HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  - 1.9355535381306524d-01*HR1(1)
     &  + 5.8224052646501250d-01*HR1(1)*HR2(-1,1)
     &  + HR1(1) *HR2(-1,1)*HR2(0,-1)
     &  - 5.8224052646501250d-01*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(1)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(1) *HR2(0,-1)*HR2(0,1)
     &  - 5.8224052646501250d-01*HR1(1)*HR2(0,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + HR1(1) *HR4(0,-1,0,1)
     &  + HR1(1) *HR4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HR1(1)*HR4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  + HR2( -1,1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  + HR2( -1,1)*HR3(0,-1,1)
     &  + HR2( -1,1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR3(-1,-1,1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR3(0,-1,1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR3(0,1,-1)
     &  + 2.0000000000000000d+00*HR2(0,-1)*HR3(0,1,1)
     &  - 2.0000000000000000d+00*HR2(0,1)*HR3(0,-1,-1)
     &  - HR2(0,1) *HR3(0,-1,1)
     &  + 1.1644810529300250d+00*HR3(-1,-1,1)
     &  - 1.1644810529300250d+00*HR3(-1,1,1)
     &  + 1.1644810529300250d+00*HR3(0,-1,-1)
     &  + 1.1644810529300250d+00*HR3(0,-1,1)
     &  + 1.1644810529300250d+00*HR3(0,1,-1)
     &  + 1.1644810529300250d+00*HR3(0,1,1)
     &  - 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  - HR5( -1,-1,1,-1,1)
     &  - 8.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  - 4.0000000000000000d+00*HR5(0,-1,0,-1,-1)
     &  - HR5(0, -1,0,-1,1)
     &  - HR5(0, -1,0,1,-1)
     &  - 3.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  - HR5(0, -1,1,-1,1)
     &  + HR5(0, -1,1,0,1)
     &  - 1.2000000000000000d+01*HR5(0,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HR5(0,0,-1,-1,1)
     &  - 4.0000000000000000d+00*HR5(0,0,-1,1,-1)
     &  - 4.0000000000000000d+00*HR5(0,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,0,1,-1,1)
     &  - 4.0000000000000000d+00*HR5(0,0,1,1,-1)
     &  - 7.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR5(0,1,-1,1,-1)
     &  - HR5(0,1,0,1, -1)
     &  - 2.0000000000000000d+00*HR5(0,1,1,-1,-1)
      Hi6(0,1,0,1,1,-1) = 
     &  - 4.3369237704895519d-01*HR1(-1)
     &  - 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR2(-1,1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR2(0,-1)
     &  + 1.0626935403832139d+00*HR1(-1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,0,-1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,0,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(-1,1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR3(0,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR3(0,1,-1)
     &  - 1.3862943611198906d+00*HR1(0)*HR3(0,1,1)
     &  - 4.3369237704895519d-01*HR1(1)
     &  - 1.0626935403832139d+00*HR1(1)*HR2(-1,1)
     &  + 1.0626935403832139d+00*HR1(1)*HR2(0,-1)
     &  + 1.0626935403832139d+00*HR1(1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,0,-1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,0,1)
     &  - 1.3862943611198906d+00*HR1(1)*HR3(0,1,-1)
     &  + 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR2(0,1)
     &  + 3.4657359027997265d-01*HR2(0,1)*HR2(0,1)
     &  - 2.1253870807664278d+00*HR3(-1,-1,1)
     &  + 2.1253870807664278d+00*HR3(-1,1,1)
     &  - 2.1253870807664278d+00*HR3(0,-1,-1)
     &  - 2.1253870807664278d+00*HR3(0,-1,1)
     &  - 2.1253870807664278d+00*HR3(0,1,-1)
     &  - 2.1253870807664278d+00*HR3(0,1,1)
     &  - 2.7725887222397812d+00*HR4(-1,-1,-1,1)
     &  + 1.3862943611198906d+00*HR4(-1,-1,1,1)
     &  + 2.7725887222397812d+00*HR4(0,-1,-1,-1)
     &  + 1.3862943611198906d+00*HR4(0,-1,-1,1)
     &  + 1.3862943611198906d+00*HR4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HR4(0,0,-1,-1)
     &  + 1.3862943611198906d+00*HR4(0,0,-1,1)
     &  + 1.3862943611198906d+00*HR4(0,0,1,-1)
     &  + 1.3862943611198906d+00*HR4(0,0,1,1)
     &  + 2.7725887222397812d+00*HR4(0,1,-1,-1)
     &  + 1.3862943611198906d+00*HR4(0,1,-1,1)
     &  + 1.3862943611198906d+00*HR4(0,1,1,-1)
      Hi6(0,1,0,1,1,1) = 
     &  - 1.0823232337111381d+00*HR1(-1)
     &  - 2.3411792664818762d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 7.0235377994456286d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,1)
     &  + HR1( -1)*HR1(-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR3(-1,1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(0,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR1(1)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(-1,1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,0,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR1(0)*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,0,1)
     &  + HR1( -1)*HR1(1)*HR3(0,1,-1)
     &  + 1.4047075598891257d+00*HR1(-1)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR2(-1,1)*HR2(-1,1)
     &  + 1.4047075598891257d+00*HR1(-1)*HR2(0,-1)
     &  + 1.4047075598891257d+00*HR1(-1)*HR2(0,1)
     &  - 1.3862943611198906d+00*HR1(-1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,0,-1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,0,1)
     &  + 1.3862943611198906d+00*HR1(-1)*HR3(0,1,-1)
     &  - 4.0000000000000000d+00*HR1(-1)*HR4(-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(-1)*HR4(-1,-1,1,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,0,0,1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(-1)*HR4(0,1,-1,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR1(1)*HR2(0,1)
     &  + HR1(0) *HR1(0)*HR3(-1,-1,1)
     &  - HR1(0) *HR1(0)*HR3(-1,1,1)
     &  + HR1(0) *HR1(0)*HR3(0,-1,-1)
     &  + HR1(0) *HR1(0)*HR3(0,-1,1)
     &  + HR1(0) *HR1(0)*HR3(0,1,-1)
     &  + HR1(0) *HR1(0)*HR3(0,1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR1(1)*HR2(0,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(-1,-1,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,0,-1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,0,1)
     &  + 2.0000000000000000d+00*HR1(0)*HR1(1)*HR3(0,1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR2(-1,1)*HR2(-1,1)
     &  + HR1(0) *HR2(-1,1)*HR2(0,-1)
     &  + HR1(0) *HR2(-1,1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  - HR1(0) *HR2(0,-1)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR2(0,1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(-1,-1,1)
     &  - 1.3862943611198906d+00*HR1(0)*HR3(-1,1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(0,-1,1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(0,1,-1)
     &  + 1.3862943611198906d+00*HR1(0)*HR3(0,1,1)
     &  + 4.0000000000000000d+00*HR1(0)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(-1,-1,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,0,1,1)
     &  - 4.0000000000000000d+00*HR1(0)*HR4(0,1,-1,-1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HR1(0)*HR4(0,1,1,-1)
     &  - 1.0823232337111381d+00*HR1(1)
     &  - 1.4047075598891257d+00*HR1(1)*HR2(-1,1)
     &  + 1.4047075598891257d+00*HR1(1)*HR2(0,-1)
     &  + 1.4047075598891257d+00*HR1(1)*HR2(0,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(-1,-1,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,-1,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,0,-1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,0,1)
     &  + 1.3862943611198906d+00*HR1(1)*HR3(0,1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,0,0,1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HR1(1)*HR4(0,1,-1,-1)
     &  - 3.4657359027997265d-01*HR2(-1,1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,1)
     &  - HR2( -1,1)*HR3(-1,-1,1)
     &  - HR2( -1,1)*HR3(0,-1,-1)
     &  - HR2( -1,1)*HR3(0,0,-1)
     &  - HR2( -1,1)*HR3(0,0,1)
     &  - HR2( -1,1)*HR3(0,1,-1)
     &  - 3.4657359027997265d-01*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR2(0,1)
     &  + HR2(0, -1)*HR3(0,0,-1)
     &  + HR2(0, -1)*HR3(0,0,1)
     &  - 3.4657359027997265d-01*HR2(0,1)*HR2(0,1)
     &  + HR2(0,1) *HR3(0,-1,-1)
     &  + HR2(0,1) *HR3(0,0,-1)
     &  + HR2(0,1) *HR3(0,0,1)
     &  - 2.8094151197782514d+00*HR3(-1,-1,1)
     &  + 2.8094151197782514d+00*HR3(-1,1,1)
     &  - 2.8094151197782514d+00*HR3(0,-1,-1)
     &  - 2.8094151197782514d+00*HR3(0,-1,1)
     &  - 2.8094151197782514d+00*HR3(0,1,-1)
     &  - 2.8094151197782514d+00*HR3(0,1,1)
     &  + 2.7725887222397812d+00*HR4(-1,-1,-1,1)
     &  - 1.3862943611198906d+00*HR4(-1,-1,1,1)
     &  - 2.7725887222397812d+00*HR4(0,-1,-1,-1)
     &  - 1.3862943611198906d+00*HR4(0,-1,-1,1)
     &  - 1.3862943611198906d+00*HR4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HR4(0,0,-1,-1)
     &  - 1.3862943611198906d+00*HR4(0,0,-1,1)
     &  - 1.3862943611198906d+00*HR4(0,0,1,-1)
     &  - 1.3862943611198906d+00*HR4(0,0,1,1)
     &  - 2.7725887222397812d+00*HR4(0,1,-1,-1)
     &  - 1.3862943611198906d+00*HR4(0,1,-1,1)
     &  - 1.3862943611198906d+00*HR4(0,1,1,-1)
     &  + 7.0000000000000000d+00*HR5(-1,-1,-1,-1,1)
     &  + HR5( -1,-1,1,-1,1)
     &  + 7.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - HR5(0, -1,-1,0,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,1,-1)
     &  + HR5(0, -1,0,-1,-1)
     &  - HR5(0, -1,0,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,1,-1,-1)
     &  + 7.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  + HR5(0,0, -1,-1,1)
     &  - HR5(0,0, -1,0,-1)
     &  - HR5(0,0, -1,0,1)
     &  + 2.0000000000000000d+00*HR5(0,0,-1,1,-1)
     &  + 5.0000000000000000d+00*HR5(0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,0,1,-1,1)
     &  - HR5(0,0,1,0, -1)
     &  - HR5(0,0,1,0,1) 
     &  + 4.0000000000000000d+00*HR5(0,0,1,1,-1)
     &  + 7.0000000000000000d+00*HR5(0,1,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,1,-1,1,-1)
     &  + HR5(0,1,0,1, -1)
     &  + 2.0000000000000000d+00*HR5(0,1,1,-1,-1)
      Hi6(0,1,1,-1,-1,-1) = 
     &  + 1.2679858379652411d-01*HR1(-1)
     &  - 2.7752054332410789d-02*HR1(-1)*HR1(-1)
     &  + 5.5504108664821579d-02*HR1(-1)*HR1(0)
     &  - 5.5504108664821579d-02*HR1(-1)*HR1(1)
     &  + 5.5504108664821579d-02*HR1(0)*HR1(1)
     &  + 1.2679858379652411d-01*HR1(1)
     &  + 5.5504108664821579d-02*HR2(-1,1)
     &  - 5.5504108664821579d-02*HR2(0,-1)
     &  - 5.5504108664821579d-02*HR2(0,1)
      Hi6(0,1,1,-1,-1,1) = 
     &  + 2.3517979306082505d-01*HR1(-1)
     &  - 4.7376502115063852d-02*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 9.4753004230127705d-02*HR1(-1)*HR1(0)
     &  - 9.4753004230127705d-02*HR1(-1)*HR1(1)
     &  + HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR4(0,0,-1,-1)
     &  + 9.4753004230127705d-02*HR1(0)*HR1(1)
     &  + 2.3517979306082505d-01*HR1(1)
     &  - 2.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  - HR1(1) *HR4(0,0,-1,-1)
     &  + 9.4753004230127705d-02*HR2(-1,1)
     &  - HR2( -1,1)*HR3(0,-1,-1)
     &  - 9.4753004230127705d-02*HR2(0,-1)
     &  - 9.4753004230127705d-02*HR2(0,1)
     &  + HR2(0,1) *HR3(0,-1,-1)
     &  - HR5( -1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  - HR5(0, -1,-1,0,1)
     &  + HR5(0, -1,-1,1,-1)
     &  + HR5(0, -1,0,-1,-1)
     &  - HR5(0, -1,0,-1,1)
     &  - HR5(0, -1,0,1,-1)
     &  + 3.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  - HR5(0,0, -1,-1,1)
     &  - HR5(0,0, -1,1,-1)
     &  - HR5(0,0,1, -1,-1)
     &  - HR5(0,1, -1,-1,-1)
      Hi6(0,1,1,-1,1,-1) = 
     &  + 6.4210078767232862d-01*HR1(-1)
     &  - 1.0703618543335311d-01*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.1407237086670622d-01*HR1(-1)*HR1(0)
     &  - 2.1407237086670622d-01*HR1(-1)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(0,0,-1)
     &  + 2.1407237086670622d-01*HR1(0)*HR1(1)
     &  + 6.4210078767232862d-01*HR1(1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(1)*HR3(0,0,-1)
     &  + 2.1407237086670622d-01*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  - 2.1407237086670622d-01*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR2(0,-1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR2(0,-1)*HR2(0,1)
     &  - 2.1407237086670622d-01*HR2(0,1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,0,1)
     &  + 6.9314718055994530d-01*HR4(0,0,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,-1)
      Hi6(0,1,1,-1,1,1) = 
     &  + 1.9248049955307152d+00*HR1(-1)
     &  - 2.6860659680402010d-01*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(-1)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(-1)*HR3(0,0,-1)
     &  + 5.3721319360804020d-01*HR1(-1)*HR1(0)
     &  + HR1( -1)*HR1(0)*HR1(1)*HR2(0,-1)
     &  - HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  - HR1( -1)*HR1(0)*HR3(0,-1,-1)
     &  - HR1( -1)*HR1(0)*HR3(0,0,-1)
     &  - 5.3721319360804020d-01*HR1(-1)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(1)*HR2(0,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR1(-1)*HR1(1)*HR3(0,0,-1)
     &  - HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(0,0,-1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(-1)*HR4(0,0,0,-1)
     &  + 5.3721319360804020d-01*HR1(0)*HR1(1)
     &  - HR1(0) *HR1(1)*HR3(0,-1,-1)
     &  - HR1(0) *HR1(1)*HR3(0,0,-1)
     &  - HR1(0) *HR2(-1,1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(0)*HR2(0,-1)*HR2(0,-1)
     &  + HR1(0) *HR2(0,-1)*HR2(0,1)
     &  + HR1(0) *HR4(-1,-1,-1,1)
     &  + HR1(0) *HR4(0,-1,-1,1)
     &  - HR1(0) *HR4(0,-1,0,1)
     &  - HR1(0) *HR4(0,0,-1,1)
     &  - HR1(0) *HR4(0,0,1,-1)
     &  - HR1(0) *HR4(0,1,-1,-1)
     &  + 1.9248049955307152d+00*HR1(1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(1)*HR3(0,0,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR1(1)*HR4(0,0,0,-1)
     &  + 5.3721319360804020d-01*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR2(-1,1)*HR2(0,-1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR3(0,-1,-1)
     &  + 2.0000000000000000d+00*HR2(-1,1)*HR3(0,0,-1)
     &  - 5.3721319360804020d-01*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR2(0,-1)*HR2(0,-1)
     &  + 6.9314718055994530d-01*HR2(0,-1)*HR2(0,1)
     &  + HR2(0, -1)*HR3(-1,-1,1)
     &  + HR2(0, -1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR2(0,-1)*HR3(0,0,-1)
     &  + HR2(0, -1)*HR3(0,1,-1)
     &  - 5.3721319360804020d-01*HR2(0,1)
     &  - 2.0000000000000000d+00*HR2(0,1)*HR3(0,-1,-1)
     &  - 2.0000000000000000d+00*HR2(0,1)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HR4(0,0,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  + HR5( -1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HR5(0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,0,1)
     &  - HR5(0, -1,-1,1,-1)
     &  - 3.0000000000000000d+00*HR5(0,-1,0,-1,-1)
     &  + 2.0000000000000000d+00*HR5(0,-1,0,-1,1)
     &  + HR5(0, -1,0,1,-1)
     &  - 9.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  + HR5(0,0, -1,-1,1)
     &  + 3.0000000000000000d+00*HR5(0,0,-1,0,-1)
     &  + 2.0000000000000000d+00*HR5(0,0,-1,0,1)
     &  + HR5(0,0, -1,1,-1)
     &  + 6.0000000000000000d+00*HR5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HR5(0,0,0,-1,1)
     &  + 3.0000000000000000d+00*HR5(0,0,0,1,-1)
     &  + HR5(0,0,1, -1,-1)
     &  + HR5(0,0,1,0, -1)
     &  + HR5(0,1, -1,-1,-1)
      Hi6(0,1,1,1,-1,-1) = 
     &  - 2.0822301897223939d-01*HR1(-1)
     &  + 2.3766885054564933d-01*HR1(-1)*HR1(-1)
     &  - 4.0037751159850118d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 4.7533770109129867d-01*HR1(-1)*HR1(0)
     &  - 1.2011325347955035d-01*HR1(-1)*HR1(0)*HR1(0)
     &  + 2.4022650695910071d-01*HR1(-1)*HR1(0)*HR1(1)
     &  + 4.7533770109129867d-01*HR1(-1)*HR1(1)
     &  + 2.4022650695910071d-01*HR1(-1)*HR2(-1,1)
     &  - 1.2011325347955035d-01*HR1(0)*HR1(0)*HR1(1)
     &  - 4.7533770109129867d-01*HR1(0)*HR1(1)
     &  - 2.4022650695910071d-01*HR1(0)*HR2(-1,1)
     &  + 2.4022650695910071d-01*HR1(0)*HR2(0,-1)
     &  + 2.4022650695910071d-01*HR1(0)*HR2(0,1)
     &  - 2.0822301897223939d-01*HR1(1)
     &  - 4.7533770109129867d-01*HR2(-1,1)
     &  + 4.7533770109129867d-01*HR2(0,-1)
     &  + 4.7533770109129867d-01*HR2(0,1)
     &  - 2.4022650695910071d-01*HR3(-1,-1,1)
     &  - 2.4022650695910071d-01*HR3(0,-1,-1)
     &  - 2.4022650695910071d-01*HR3(0,0,-1)
     &  - 2.4022650695910071d-01*HR3(0,0,1)
     &  - 2.4022650695910071d-01*HR3(0,1,-1)
      Hi6(0,1,1,1,-1,1) = 
     &  - 1.4792951368405047d+00*HR1(-1)
     &  + 7.3900238327152102d-01*HR1(-1)*HR1(-1)
     &  - 9.7040087744168750d-02*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(0,-1)
     &  + 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(1)*HR2(0,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,-1,-1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(0,0,-1)
     &  - 1.4780047665430420d+00*HR1(-1)*HR1(0)
     &  - 2.9112026323250625d-01*HR1(-1)*HR1(0)*HR1(0)
     &  + 5.8224052646501250d-01*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.4780047665430420d+00*HR1(-1)*HR1(1)
     &  + HR1( -1)*HR1(1)*HR3(0,-1,-1)
     &  + HR1( -1)*HR1(1)*HR3(0,0,-1)
     &  + 5.8224052646501250d-01*HR1(-1)*HR2(-1,1)
     &  + HR1( -1)*HR2(-1,1)*HR2(0,-1)
     &  - HR1( -1)*HR4(-1,-1,-1,1)
     &  - HR1( -1)*HR4(0,-1,-1,-1)
     &  - HR1( -1)*HR4(0,0,-1,-1)
     &  - HR1( -1)*HR4(0,0,0,-1)
     &  - 2.9112026323250625d-01*HR1(0)*HR1(0)*HR1(1)
     &  - 1.4780047665430420d+00*HR1(0)*HR1(1)
     &  - 5.8224052646501250d-01*HR1(0)*HR2(-1,1)
     &  + 5.8224052646501250d-01*HR1(0)*HR2(0,-1)
     &  + 5.8224052646501250d-01*HR1(0)*HR2(0,1)
     &  - 1.4792951368405047d+00*HR1(1)
     &  - HR1(1) *HR4(0,-1,-1,-1)
     &  - HR1(1) *HR4(0,0,-1,-1)
     &  - HR1(1) *HR4(0,0,0,-1)
     &  - 1.4780047665430420d+00*HR2(-1,1)
     &  - HR2( -1,1)*HR3(0,-1,-1)
     &  - HR2( -1,1)*HR3(0,0,-1)
     &  + 1.4780047665430420d+00*HR2(0,-1)
     &  - HR2(0, -1)*HR3(-1,-1,1)
     &  - HR2(0, -1)*HR3(0,-1,-1)
     &  + HR2(0, -1)*HR3(0,0,-1)
     &  - HR2(0, -1)*HR3(0,1,-1)
     &  + 1.4780047665430420d+00*HR2(0,1)
     &  + HR2(0,1) *HR3(0,-1,-1)
     &  + HR2(0,1) *HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR3(-1,-1,1)
     &  - 5.8224052646501250d-01*HR3(0,-1,-1)
     &  - 5.8224052646501250d-01*HR3(0,0,-1)
     &  - 5.8224052646501250d-01*HR3(0,0,1)
     &  - 5.8224052646501250d-01*HR3(0,1,-1)
     &  + HR5( -1,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,-1,-1,-1)
     &  + HR5(0, -1,-1,-1,1)
     &  - HR5(0, -1,-1,0,1)
     &  + 2.0000000000000000d+00*HR5(0,-1,0,-1,-1)
     &  - HR5(0, -1,0,-1,1)
     &  + 6.0000000000000000d+00*HR5(0,0,-1,-1,-1)
     &  - HR5(0,0, -1,-1,1)
     &  - 2.0000000000000000d+00*HR5(0,0,-1,0,-1)
     &  - HR5(0,0, -1,0,1)
     &  - 4.0000000000000000d+00*HR5(0,0,0,-1,-1)
     &  - 2.0000000000000000d+00*HR5(0,0,0,-1,1)
     &  - 2.0000000000000000d+00*HR5(0,0,0,1,-1)
     &  + HR5(0,0,1, -1,-1)
     &  - HR5(0,0,1,0, -1)
     &  + HR5(0,1, -1,-1,-1)
      Hi6(0,1,1,1,1,-1) = 
     &  - 6.7987181069975186d-01*HR1(-1)
     &  + 1.6438755856693088d-02*HR1(-1)*HR1(-1)
     &  + 1.7711559006386898d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  - 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 3.2877511713386177d-02*HR1(-1)*HR1(0)
     &  + 5.3134677019160696d-01*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR1(0)*HR1(1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 3.2877511713386177d-02*HR1(-1)*HR1(1)
     &  - 1.0626935403832139d+00*HR1(-1)*HR2(-1,1)
     &  - 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + 1.1552453009332421d-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 5.3134677019160696d-01*HR1(0)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR2(0,-1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR2(0,1)
     &  - 3.2877511713386177d-02*HR1(0)*HR1(1)
     &  + 1.0626935403832139d+00*HR1(0)*HR2(-1,1)
     &  - 1.0626935403832139d+00*HR1(0)*HR2(0,-1)
     &  - 1.0626935403832139d+00*HR1(0)*HR2(0,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,-1,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,0,-1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,0,1)
     &  + 6.9314718055994530d-01*HR1(0)*HR3(0,1,-1)
     &  - 6.7987181069975186d-01*HR1(1)
     &  - 3.2877511713386177d-02*HR2(-1,1)
     &  + 3.2877511713386177d-02*HR2(0,-1)
     &  + 3.2877511713386177d-02*HR2(0,1)
     &  + 1.0626935403832139d+00*HR3(-1,-1,1)
     &  + 1.0626935403832139d+00*HR3(0,-1,-1)
     &  + 1.0626935403832139d+00*HR3(0,0,-1)
     &  + 1.0626935403832139d+00*HR3(0,0,1)
     &  + 1.0626935403832139d+00*HR3(0,1,-1)
     &  + 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  - 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HR4(0,0,0,-1)
     &  - 6.9314718055994530d-01*HR4(0,0,0,1)
     &  - 6.9314718055994530d-01*HR4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HR4(0,1,-1,-1)
      Hi6(0,1,1,1,1,1) = 
     &  - 4.2620378933400481d-01*HR1(-1)
     &  - 5.4233865098901558d-01*HR1(-1)*HR1(-1)
     &  + 2.3411792664818762d-01*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 2.8881132523331054d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  - 8.3333333333333333d-03*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
     &  + 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
     &  - 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  + 1.1552453009332421d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 7.0235377994456286d-01*HR1(-1)*HR1(-1)*HR1(0)
     &  + 1.7328679513998632d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
     &  + 8.3333333333333333d-02*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 2.5000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + 7.0235377994456286d-01*HR1(-1)*HR1(-1)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(-1)*HR1(-1)*HR2(-1,1)
     &  - 5.0000000000000000d-01*HR1(-1)*HR1(-1)*HR3(-1,-1,1)
     &  + 1.0846773019780311d+00*HR1(-1)*HR1(0)
     &  + 7.0235377994456286d-01*HR1(-1)*HR1(0)*HR1(0)
     &  - 1.1552453009332421d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
     &  - 4.1666666666666666d-02*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(0)
     &  + 1.6666666666666666d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  + 3.4657359027997265d-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
     &  + 5.0000000000000000d-01*HR1(-1)*HR1(0)*HR1(0)*HR2(-1,1)
     &  - 1.4047075598891257d+00*HR1(-1)*HR1(0)*HR1(1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR1(0)*HR2(-1,1)
     &  + HR1( -1)*HR1(0)*HR3(-1,-1,1)
     &  - 1.0846773019780311d+00*HR1(-1)*HR1(1)
     &  - 1.4047075598891257d+00*HR1(-1)*HR2(-1,1)
     &  + 6.9314718055994530d-01*HR1(-1)*HR3(-1,-1,1)
     &  + HR1( -1)*HR4(-1,-1,-1,1)
     &  - 4.1666666666666666d-02*HR1(0)*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.1552453009332421d-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)
     &  - 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 1.6666666666666666d-01*HR1(0)*HR1(0)*HR1(0)*HR2(0,1)
     &  + 7.0235377994456286d-01*HR1(0)*HR1(0)*HR1(1)
     &  - 3.4657359027997265d-01*HR1(0)*HR1(0)*HR2(-1,1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR2(0,-1)
     &  + 3.4657359027997265d-01*HR1(0)*HR1(0)*HR2(0,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(-1,-1,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,-1,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,0,-1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,0,1)
     &  - 5.0000000000000000d-01*HR1(0)*HR1(0)*HR3(0,1,-1)
     &  + 1.0846773019780311d+00*HR1(0)*HR1(1)
     &  + 1.4047075598891257d+00*HR1(0)*HR2(-1,1)
     &  - 1.4047075598891257d+00*HR1(0)*HR2(0,-1)
     &  - 1.4047075598891257d+00*HR1(0)*HR2(0,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(-1,-1,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,-1,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,0,-1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,0,1)
     &  - 6.9314718055994530d-01*HR1(0)*HR3(0,1,-1)
     &  - HR1(0) *HR4(-1,-1,-1,1)
     &  + HR1(0) *HR4(0,-1,-1,-1)
     &  + HR1(0) *HR4(0,0,-1,-1)
     &  + HR1(0) *HR4(0,0,0,-1)
     &  + HR1(0) *HR4(0,0,0,1)
     &  + HR1(0) *HR4(0,0,1,-1)
     &  + HR1(0) *HR4(0,1,-1,-1)
     &  - 4.2620378933400481d-01*HR1(1)
     &  + 1.0846773019780311d+00*HR2(-1,1)
     &  - 1.0846773019780311d+00*HR2(0,-1)
     &  - 1.0846773019780311d+00*HR2(0,1)
     &  + 1.4047075598891257d+00*HR3(-1,-1,1)
     &  + 1.4047075598891257d+00*HR3(0,-1,-1)
     &  + 1.4047075598891257d+00*HR3(0,0,-1)
     &  + 1.4047075598891257d+00*HR3(0,0,1)
     &  + 1.4047075598891257d+00*HR3(0,1,-1)
     &  - 6.9314718055994530d-01*HR4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HR4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,0,-1)
     &  + 6.9314718055994530d-01*HR4(0,0,0,1)
     &  + 6.9314718055994530d-01*HR4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HR4(0,1,-1,-1)
     &  - HR5( -1,-1,-1,-1,1)
     &  - HR5(0, -1,-1,-1,-1)
     &  - HR5(0,0, -1,-1,-1)
     &  - HR5(0,0,0, -1,-1)
     &  - HR5(0,0,0,0, -1)
     &  - HR5(0,0,0,0,1) 
     &  - HR5(0,0,0,1, -1)
     &  - HR5(0,0,1, -1,-1)
     &  - HR5(0,1, -1,-1,-1)
      endif 
      endif
** nw > 5 endif
      endif 
** (n1,n2) = (-1,1) -- completion endif 
      return 
      end 
************************************************************************ 
      subroutine fillirr1dhplatinf(x,nw,HX1,HX2,HX3,HX4,HX5,HX6, 
     &                                  HY1,HY2,HY3,HY4,HY5,HY6, 
     &                                  Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2) 
** evaluates the HPL for y > r2p1
** fillirr1dhplatinf is called by eval1dhplatinf after calling 
** fillirr1dhplat0 with argument r=1/y 
** it is guaranteed that nw is in the range 2:4, and that (n1,n2) 
** take one of the pairs of values (0,1), (-1,0) or (-1,1) 
      implicit double precision (a-h,o-z) 
      dimension HX1(n1:n2),HX2(n1:n2,n1:n2),HX3(n1:n2,n1:n2,n1:n2), 
     &          HX4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HX5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HX6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension HY1(n1:n2),HY2(n1:n2,n1:n2),HY3(n1:n2,n1:n2,n1:n2), 
     &          HY4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HY5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HY6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
      dimension Hi1(n1:n2),Hi2(n1:n2,n1:n2),Hi3(n1:n2,n1:n2,n1:n2), 
     &          Hi4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          Hi6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
** (n1,n2) = (0,1) or (-1,1) 
      if (    ( (n1.eq.0).and.(n2.eq.1) ) 
     &    .or.( (n1.eq.-1).and.(n2.eq.1) ) ) then 
      HY2(0,1) = 
     &  + 3.2898681336964528d+00 
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0) 
     &  - HX2(0,1) 
      Hi2(0,1) = 
     &  - HX1(0) 
      if ( nw.gt.2 ) then 
      HY3(0,0,1) = 
     &  - 3.2898681336964528d+00*HX1(0) 
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0) 
     &  + HX3(0,0,1) 
      HY3(0,1,1) = 
     &  + 1.2020569031595942d+00 
     &  + 4.9348022005446793d+00*HX1(0) 
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0) 
     &  - HX1(0) *HX2(0,1) 
     &  + HX3(0,0,1) 
     &  - HX3(0,1,1) 
      Hi3(0,0,1) = 
     &  + 5.000000000000000d-01*HX1(0)*HX1(0) 
      Hi3(0,1,1) = 
     &  + 1.6449340668482264d+00 
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0) 
     &  - HX2(0,1) 
      endif 
      if ( nw.gt.3 ) then 
      HY4(0,0,0,1) = 
     &  + 2.1646464674222763d+00 
     &  + 1.6449340668482264d+00*HX1(0)*HX1(0) 
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0) 
     &  - HX4(0,0,0,1) 
      HY4(0,0,1,1) = 
     &  + 2.1646464674222763d+00 
     &  - 1.2020569031595942d+00*HX1(0) 
     &  - 2.4674011002723396d+00*HX1(0)*HX1(0) 
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0) 
     &  + HX1(0) *HX3(0,0,1) 
     &  - 2.0000000000000000d+00*HX4(0,0,0,1) 
     &  + HX4(0,0,1,1) 
      HY4(0,1,1,1) = 
     &  - 5.1410353601279064d+00 
     &  + 2.4674011002723396d+00*HX1(0)*HX1(0) 
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0) 
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1) 
     &  + HX1(0) *HX3(0,0,1) 
     &  - HX1(0) *HX3(0,1,1) 
     &  + 4.9348022005446793d+00*HX2(0,1) 
     &  - HX4(0,0,0,1) 
     &  + HX4(0,0,1,1) 
     &  - HX4(0,1,1,1) 
      Hi4(0,0,0,1) = 
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0) 
      Hi4(0,0,1,1) = 
     &  - 1.2020569031595942d+00 
     &  - 1.6449340668482264d+00*HX1(0) 
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0) 
     &  + HX3(0,0,1) 
      Hi4(0,1,1,1) = 
     &  + 1.6449340668482264d+00*HX1(0) 
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0) 
     &  - HX1(0) *HX2(0,1) 
     &  + HX3(0,0,1) 
     &  - HX3(0,1,1) 
      endif 
** nw > 3 endif 
      if ( nw.gt.4 ) then 
      HY5(0,0,0,0,1) = 
     &  - 2.1646464674222763d+00*HX1(0)
     &  - 5.4831135561607547d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX5(0,0,0,0,1) 
      HY5(0,0,0,1,1) = 
     &  - 2.9176809454512223d+00
     &  - 2.1646464674222763d+00*HX1(0)
     &  + 6.0102845157979714d-01*HX1(0)*HX1(0)
     &  + 8.2246703342411321d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1(0) *HX4(0,0,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - HX5(0,0,0,1,1) 
      HY5(0,0,1,0,1) = 
     &  + 3.7615063806157047d+00
     &  - 1.0823232337111381d+00*HX1(0)
     &  - 1.2020569031595942d+00*HX1(0)*HX1(0)
     &  + 5.4831135561607547d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + 3.2898681336964528d+00*HX3(0,0,1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - HX5(0,0,1,0,1) 
      HY5(0,0,1,1,1) = 
     &  + 3.0142321054406660d+00
     &  + 5.1410353601279064d+00*HX1(0)
     &  - 8.2246703342411321d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + HX1(0) *HX4(0,0,1,1)
     &  - 4.9348022005446793d+00*HX3(0,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + HX5(0,0,1,1,1) 
      HY5(0,1,0,1,1) = 
     &  - 1.5553916327150548d+00
     &  + 8.1174242528335364d-01*HX1(0)
     &  - 6.0102845157979714d-01*HX1(0)*HX1(0)
     &  - 8.2246703342411321d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - HX1(0) *HX1(0)*HX3(0,0,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(0,1)*HX2(0,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,1,1)
     &  - 1.2020569031595942d+00*HX2(0,1)
     &  - HX2(0,1) *HX3(0,0,1)
     &  + 9.8696044010893586d+00*HX3(0,0,1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 7.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + HX5(0,0,1,0,1) 
     &  + HX5(0,1,0,1,1) 
      HY5(0,1,1,1,1) = 
     &  + 1.0369277551433699d+00
     &  - 4.0587121264167682d+00*HX1(0)
     &  + 8.2246703342411321d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX2(0,1)
     &  - HX1(0) *HX4(0,0,0,1)
     &  + HX1(0) *HX4(0,0,1,1)
     &  - HX1(0) *HX4(0,1,1,1)
     &  - 4.9348022005446793d+00*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX3(0,1,1)
     &  + HX5(0,0,0,0,1) 
     &  - HX5(0,0,0,1,1) 
     &  + HX5(0,0,1,1,1) 
     &  - HX5(0,1,1,1,1) 
      Hi5(0,0,0,0,1) = 
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
      Hi5(0,0,0,1,1) = 
     &  + 1.0823232337111381d+00
     &  + 1.2020569031595942d+00*HX1(0)
     &  + 8.2246703342411321d-01*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX4(0,0,0,1) 
      Hi5(0,0,1,0,1) = 
     &  - 3.2469697011334145d+00
     &  - 2.4041138063191885d+00*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1(0) *HX3(0,0,1)
     &  + 3.0000000000000000d+00*HX4(0,0,0,1)
      Hi5(0,0,1,1,1) = 
     &  + 1.8940656589944918d+00
     &  - 8.2246703342411321d-01*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1(0) *HX3(0,0,1)
     &  - 2.0000000000000000d+00*HX4(0,0,0,1)
     &  + HX4(0,0,1,1) 
      Hi5(0,1,0,1,1) = 
     &  - 2.4352272758500609d+00
     &  - 1.2020569031595942d+00*HX1(0)
     &  - 8.2246703342411321d-01*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX3(0,0,1)
     &  - 1.6449340668482264d+00*HX2(0,1)
     &  + 5.0000000000000000d-01*HX2(0,1)*HX2(0,1)
     &  + 4.0000000000000000d+00*HX4(0,0,0,1)
     &  - 2.0000000000000000d+00*HX4(0,0,1,1)
      Hi5(0,1,1,1,1) = 
     &  - 8.1174242528335364d-01
     &  + 8.2246703342411321d-01*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  + HX1(0) *HX3(0,0,1)
     &  - HX1(0) *HX3(0,1,1)
     &  + 1.6449340668482264d+00*HX2(0,1)
     &  - HX4(0,0,0,1) 
     &  + HX4(0,0,1,1) 
     &  - HX4(0,1,1,1) 
      endif 
** nw > 4 endif 
      endif 
** (n1,n2) = (0,1) or (-1,1) endif 
************ 
** (n1,n2) = (-1,0) or (-1,1) 
      if (    ( (n1.eq.-1).and.(n2.eq.0) ) 
     &    .or.( (n1.eq.-1).and.(n2.eq.1) ) ) then 
      HY2(0,-1) = 
     &  + 1.6449340668482264d+00 
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0) 
     &  - HX2(0, -1) 
      if ( nw.gt.2 ) then 
      HY3(0,0,-1) = 
     &  - 1.6449340668482264d+00*HX1(0) 
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0) 
     &  + HX3(0,0, -1) 
      HY3(0,-1,-1) = 
     &  + 1.2020569031595942d+00 
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0) 
     &  + HX1(0) *HX2(0,-1) 
     &  - HX3(0, -1,-1) 
     &  - HX3(0,0, -1) 
      endif 
      if ( nw.gt.3 ) then 
      HY4(0,0,0,-1) = 
     &  + 1.8940656589944918d+00 
     &  + 8.2246703342411321d-01*HX1(0)*HX1(0) 
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0) 
     &  - HX4(0,0,0, -1) 
      HY4(0,0,-1,-1) = 
     &  - 1.8940656589944918d+00 
     &  - 1.2020569031595942d+00*HX1(0) 
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0) 
     &  - HX1(0) *HX3(0,0,-1) 
     &  + HX4(0,0, -1,-1) 
     &  + 2.0000000000000000d+00*HX4(0,0,0,-1) 
      HY4(0,-1,-1,-1) = 
     &  + 1.0823232337111381d+00 
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0) 
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1) 
     &  + HX1(0) *HX3(0,-1,-1) 
     &  + HX1(0) *HX3(0,0,-1) 
     &  - HX4(0, -1,-1,-1) 
     &  - HX4(0,0, -1,-1) 
     &  - HX4(0,0,0, -1) 
      endif 
      if ( nw.gt.4 ) then
      HY5(0,0,0,0,-1) = 
     &  - 1.8940656589944918d+00*HX1(0)
     &  - 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX5(0,0,0,0, -1)
      HY5(0,0,0,-1,-1) = 
     &  + 3.0142321054406660d+00
     &  + 1.8940656589944918d+00*HX1(0)
     &  + 6.0102845157979714d-01*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1(0) *HX4(0,0,0,-1)
     &  - HX5(0,0,0, -1,-1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,0,-1)
      HY5(0,0,-1,0,-1) = 
     &  - 8.1023197211680719d+00
     &  - 5.1410353601279064d+00*HX1(0)
     &  - 1.2020569031595942d+00*HX1(0)*HX1(0)
     &  - 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 1.6449340668482264d+00*HX3(0,0,-1)
     &  - HX5(0,0, -1,0,-1)
     &  + 7.0000000000000000d+00*HX5(0,0,0,0,-1)
      HY5(0,0,-1,-1,-1) = 
     &  - 3.0142321054406660d+00
     &  - 1.0823232337111381d+00*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + HX5(0,0, -1,-1,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,-1)
      HY5(0,-1,0,-1,-1) = 
     &  + 7.4873046836069432d+00
     &  + 3.2469697011334145d+00*HX1(0)
     &  + 6.0102845157979714d-01*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - 1.2020569031595942d+00*HX2(0,-1)
     &  + HX2(0, -1)*HX3(0,0,-1)
     &  + HX5(0, -1,0,-1,-1)
     &  - HX5(0,0, -1,0,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,-1)
      HY5(0,-1,-1,-1,-1) = 
     &  + 1.0369277551433699d+00
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX4(0,-1,-1,-1)
     &  + HX1(0) *HX4(0,0,-1,-1)
     &  + HX1(0) *HX4(0,0,0,-1)
     &  - HX5(0, -1,-1,-1,-1)
     &  - HX5(0,0, -1,-1,-1)
     &  - HX5(0,0,0, -1,-1)
     &  - HX5(0,0,0,0, -1)
      endif 
** nw > 4 endif 
      endif 
** (n1,n2) = (-1,0) or (-1,1) endif 
** (n1,n2) = (-1,1) -- completion 
      if ( (n1.eq.-1).and.(n2.eq.1) ) then 
      HY2(-1,1) = 
     &  + 2.4674011002723396d+00 
     &  + HX1( -1)*HX1(0) 
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0) 
     &  + HX2( -1,1) 
     &  - HX2(0, -1) 
     &  - HX2(0,1) 
      Hi2(-1,1) = 
     &  - 6.9314718055994530d-01 
     &  + HX1( -1) 
     &  - HX1(0) 
      if ( nw.gt.2 ) then 
      HY3(0,-1,1) = 
     &  - 2.5190015545588625d+00 
     &  - 2.4674011002723396d+00*HX1(0) 
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0) 
     &  - HX1(0) *HX2(0,-1) 
     &  - HX3(0, -1,1) 
     &  + 2.0000000000000000d+00*HX3(0,0,-1) 
     &  + HX3(0,0,1) 
      HY3(0,1,-1) = 
     &  + 4.3220869092982539d+00 
     &  + 2.4674011002723396d+00*HX1(0) 
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0) 
     &  + HX1(0) *HX2(0,1) 
     &  - HX3(0,0, -1) 
     &  - 2.0000000000000000d+00*HX3(0,0,1) 
     &  - HX3(0,1, -1) 
      HY3(-1,-1,1) = 
     &  - 2.7620719062289241d+00 
     &  + 2.4674011002723396d+00*HX1(-1) 
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0) 
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0) 
     &  - HX1( -1)*HX2(0,-1) 
     &  - HX1( -1)*HX2(0,1) 
     &  - 2.4674011002723396d+00*HX1(0) 
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0) 
     &  + HX3( -1,-1,1) 
     &  + HX3(0, -1,-1) 
     &  + HX3(0,0, -1) 
     &  + HX3(0,0,1) 
     &  + HX3(0,1, -1) 
      HY3(-1,1,1) = 
     &  + 2.7620719062289241d+00 
     &  - 4.9348022005446793d+00*HX1(-1) 
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0) 
     &  + 4.9348022005446793d+00*HX1(0) 
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0) 
     &  + HX1(0) *HX2(-1,1) 
     &  - HX1(0) *HX2(0,-1) 
     &  - HX1(0) *HX2(0,1) 
     &  + HX3( -1,1,1) 
     &  - HX3(0, -1,1) 
     &  + HX3(0,0, -1) 
     &  + HX3(0,0,1) 
     &  - HX3(0,1,1) 
      Hi3(0,-1,1) = 
     &  + 8.2246703342411321d-01 
     &  + 6.9314718055994530d-01*HX1(0) 
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0) 
     &  - HX2(0, -1) 
      Hi3(0,1,-1) = 
     &  - 6.9314718055994530d-01*HX1(0) 
      Hi3(-1,-1,1) = 
     &  + 2.4022650695910071d-01 
     &  - 6.9314718055994530d-01*HX1(-1) 
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1) 
     &  - HX1( -1)*HX1(0) 
     &  + 6.9314718055994530d-01*HX1(0) 
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0) 
      Hi3(-1,1,1) = 
     &  + 1.8851605738073271d+00 
     &  + HX1( -1)*HX1(0) 
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0) 
     &  + HX2( -1,1) 
     &  - HX2(0, -1) 
     &  - HX2(0,1) 
      endif 
      if ( nw.gt.3 ) then 
      HY4(0,0,-1,1) = 
     &  + 3.9234217222028759d+00
     &  + 2.5190015545588625d+00*HX1(0)
     &  + 1.2337005501361698d+00*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1(0) *HX3(0,0,-1)
     &  + HX4(0,0, -1,1)
     &  - 3.0000000000000000d+00*HX4(0,0,0,-1)
     &  - HX4(0,0,0,1) 
      HY4(0,0,1,-1) = 
     &  - 4.1940025306306604d+00
     &  - 4.3220869092982539d+00*HX1(0)
     &  - 1.2337005501361698d+00*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1(0) *HX3(0,0,1)
     &  + HX4(0,0,0, -1)
     &  + 3.0000000000000000d+00*HX4(0,0,0,1)
     &  + HX4(0,0,1, -1)
      HY4(0,-1,0,1) = 
     &  + 9.4703282949724591d-01
     &  + 1.8030853547393914d+00*HX1(0)
     &  + 1.6449340668482264d+00*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX3(0,0,-1)
     &  - 3.2898681336964528d+00*HX2(0,-1)
     &  + HX4(0, -1,0,1)
     &  + 3.0000000000000000d+00*HX4(0,0,0,-1)
     &  - HX4(0,0,0,1) 
      HY4(0,-1,-1,1) = 
     &  + 2.5209599327464717d+00
     &  + 2.7620719062289241d+00*HX1(0)
     &  + 1.2337005501361698d+00*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX3(0,-1,-1)
     &  - HX1(0) *HX3(0,0,-1)
     &  - 2.4674011002723396d+00*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX2(0,-1)*HX2(0,-1)
     &  - HX4(0, -1,-1,1)
     &  + HX4(0, -1,0,1)
     &  + HX4(0,0, -1,1)
     &  - HX4(0,0,0,1) 
      HY4(0,-1,1,-1) = 
     &  - 8.5266539820739622d+00
     &  - 5.5241438124578482d+00*HX1(0)
     &  - 1.2337005501361698d+00*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX3(0,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX2(0,-1)*HX2(0,-1)
     &  - HX4(0, -1,0,1)
     &  - HX4(0, -1,1,-1)
     &  + 2.0000000000000000d+00*HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX4(0,0,-1,1)
     &  + 4.0000000000000000d+00*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX4(0,0,0,1)
     &  + HX4(0,0,1, -1)
      HY4(0,1,-1,-1) = 
     &  + 5.8027584430066521d+00
     &  + 2.7620719062289241d+00*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  + HX1(0) *HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX3(0,0,1)
     &  + HX1(0) *HX3(0,1,-1)
     &  - HX4(0,0, -1,-1)
     &  - 2.0000000000000000d+00*HX4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HX4(0,0,0,1)
     &  - 2.0000000000000000d+00*HX4(0,0,1,-1)
     &  - HX4(0,1, -1,-1)
      HY4(0,-1,1,1) = 
     &  + 6.2689427375197987d-01
     &  - 2.7620719062289241d+00*HX1(0)
     &  - 2.4674011002723396d+00*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX3(0,-1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX2(0,-1)
     &  - HX4(0, -1,1,1)
     &  + 2.0000000000000000d+00*HX4(0,0,-1,1)
     &  - 3.0000000000000000d+00*HX4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HX4(0,0,0,1)
     &  + HX4(0,0,1,1) 
      HY4(0,1,-1,1) = 
     &  - 4.3326514514433017d+00
     &  - 1.3169446513992682d+00*HX1(0)
     &  - 1.2337005501361698d+00*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  - HX1(0) *HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX3(0,0,1)
     &  - HX1(0) *HX3(0,1,-1)
     &  + HX2(0, -1)*HX2(0,1)
     &  - 2.4674011002723396d+00*HX2(0,1)
     &  + 5.0000000000000000d-01*HX2(0,1)*HX2(0,1)
     &  - HX4(0, -1,0,1)
     &  - 3.0000000000000000d+00*HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX4(0,0,0,-1)
     &  + 4.0000000000000000d+00*HX4(0,0,0,1)
     &  - 2.0000000000000000d+00*HX4(0,0,1,1)
     &  - HX4(0,1, -1,1)
      HY4(0,1,1,-1) = 
     &  - 1.5001934240460787d-01
     &  + 4.0790165576281924d+00*HX1(0)
     &  + 1.2337005501361698d+00*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  - HX1(0) *HX3(0,0,1)
     &  + HX1(0) *HX3(0,1,1)
     &  - HX2(0, -1)*HX2(0,1)
     &  + 2.4674011002723396d+00*HX2(0,1)
     &  - 5.0000000000000000d-01*HX2(0,1)*HX2(0,1)
     &  + HX4(0, -1,0,1)
     &  + 2.0000000000000000d+00*HX4(0,0,-1,1)
     &  - HX4(0,0,0, -1)
     &  + HX4(0,0,1, -1)
     &  - HX4(0,1,1, -1)
      HY4(-1,-1,-1,1) = 
     &  + 2.4278628067547031d+00
     &  - 2.7620719062289241d+00*HX1(-1)
     &  + 1.2337005501361698d+00*HX1(-1)*HX1(-1)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)
     &  - 2.5000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX2(0,1)
     &  - 2.4674011002723396d+00*HX1(-1)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1( -1)*HX3(0,-1,-1)
     &  + HX1( -1)*HX3(0,0,-1)
     &  + HX1( -1)*HX3(0,0,1)
     &  + HX1( -1)*HX3(0,1,-1)
     &  + 2.7620719062289241d+00*HX1(0)
     &  + 1.2337005501361698d+00*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX4( -1,-1,-1,1)
     &  - HX4(0, -1,-1,-1)
     &  - HX4(0,0, -1,-1)
     &  - HX4(0,0,0, -1)
     &  - HX4(0,0,0,1) 
     &  - HX4(0,0,1, -1)
     &  - HX4(0,1, -1,-1)
      HY4(-1,-1,1,1) = 
     &  + 2.0293560632083841d+00
     &  + 2.7620719062289241d+00*HX1(-1)
     &  - 2.4674011002723396d+00*HX1(-1)*HX1(-1)
     &  + 2.5000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  + 4.9348022005446793d+00*HX1(-1)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1( -1)*HX1(0)*HX2(0,-1)
     &  - HX1( -1)*HX1(0)*HX2(0,1)
     &  - HX1( -1)*HX3(0,-1,1)
     &  + HX1( -1)*HX3(0,0,-1)
     &  + HX1( -1)*HX3(0,0,1)
     &  - HX1( -1)*HX3(0,1,1)
     &  - 2.7620719062289241d+00*HX1(0)
     &  - 2.4674011002723396d+00*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1(0) *HX3(-1,-1,1)
     &  + HX1(0) *HX3(0,-1,-1)
     &  + HX1(0) *HX3(0,0,-1)
     &  + HX1(0) *HX3(0,0,1)
     &  + HX1(0) *HX3(0,1,-1)
     &  + HX4( -1,-1,1,1)
     &  + HX4(0, -1,-1,1)
     &  + HX4(0, -1,1,-1)
     &  - HX4(0,0, -1,-1)
     &  + HX4(0,0, -1,1)
     &  - 2.0000000000000000d+00*HX4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HX4(0,0,0,1)
     &  - HX4(0,0,1, -1)
     &  + HX4(0,0,1,1) 
     &  + HX4(0,1, -1,1)
     &  + HX4(0,1,1, -1)
      HY4(-1,1,1,1) = 
     &  - 6.4865749331714713d+00
     &  - 4.9348022005446793d+00*HX1(-1)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 2.4674011002723396d+00*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(-1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  + HX1(0) *HX3(-1,1,1)
     &  - HX1(0) *HX3(0,-1,1)
     &  + HX1(0) *HX3(0,0,-1)
     &  + HX1(0) *HX3(0,0,1)
     &  - HX1(0) *HX3(0,1,1)
     &  - 4.9348022005446793d+00*HX2(-1,1)
     &  + 4.9348022005446793d+00*HX2(0,-1)
     &  + 4.9348022005446793d+00*HX2(0,1)
     &  + HX4( -1,1,1,1)
     &  - HX4(0, -1,1,1)
     &  + HX4(0,0, -1,1)
     &  - HX4(0,0,0, -1)
     &  - HX4(0,0,0,1) 
     &  + HX4(0,0,1,1) 
     &  - HX4(0,1,1,1) 
      Hi4(0,0,-1,1) = 
     &  - 9.0154267736969571d-01 
     &  - 8.2246703342411321d-01*HX1(0) 
     &  - 3.4657359027997265d-01*HX1(0)*HX1(0) 
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0) 
     &  + HX3(0,0, -1) 
      Hi4(0,0,1,-1) = 
     &  + 3.4657359027997265d-01*HX1(0)*HX1(0) 
      Hi4(0,-1,0,1) = 
     &  + 1.8030853547393914d+00 
     &  + 8.2246703342411321d-01*HX1(0) 
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0) 
     &  + HX1(0) *HX2(0,-1) 
     &  - 2.0000000000000000d+00*HX3(0,0,-1) 
      Hi4(0,-1,-1,1) = 
     &  + 4.8170908494321862d-01 
     &  - 2.4022650695910071d-01*HX1(0) 
     &  - 3.4657359027997265d-01*HX1(0)*HX1(0) 
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0) 
     &  + HX1(0) *HX2(0,-1) 
     &  + 6.9314718055994530d-01*HX2(0,-1) 
     &  - HX3(0, -1,-1) 
     &  - HX3(0,0, -1) 
      Hi4(0,-1,1,-1) = 
     &  + 5.7009070532142637d-01 
     &  + 4.8045301391820142d-01*HX1(0) 
     &  + 3.4657359027997265d-01*HX1(0)*HX1(0) 
     &  - 6.9314718055994530d-01*HX2(0,-1) 
      Hi4(0,1,-1,-1) = 
     &  - 2.4022650695910071d-01*HX1(0) 
      Hi4(0,-1,1,1) = 
     &  - 2.7620719062289241d+00 
     &  - 1.8851605738073271d+00*HX1(0) 
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0) 
     &  - HX1(0) *HX2(0,-1) 
     &  - HX3(0, -1,1) 
     &  + 2.0000000000000000d+00*HX3(0,0,-1) 
     &  + HX3(0,0,1) 
      Hi4(0,1,-1,1) = 
     &  + 2.6736902858507163d+00 
     &  + 1.3029200473423146d+00*HX1(0) 
     &  + 3.4657359027997265d-01*HX1(0)*HX1(0) 
     &  + 1.6666666666666665d-01*HX1(0)*HX1(0)*HX1(0) 
     &  + HX1(0) *HX2(0,1) 
     &  + 6.9314718055994530d-01*HX2(0,1) 
     &  - HX3(0,0, -1) 
     &  - 2.0000000000000000d+00*HX3(0,0,1) 
     &  - HX3(0,1, -1) 
      Hi4(0,1,1,-1) =  
     &  + 1.1401814106428527d+00 
     &  + 5.8224052646501250d-01*HX1(0) 
     &  - 3.4657359027997265d-01*HX1(0)*HX1(0) 
     &  - 6.9314718055994530d-01*HX2(0,1) 
      Hi4(-1,-1,-1,1) = 
     &  - 5.5504108664821579d-02
     &  + 2.4022650695910071d-01*HX1(-1)
     &  - 3.4657359027997265d-01*HX1(-1)*HX1(-1)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)
     &  + 6.9314718055994530d-01*HX1(-1)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)
     &  - 2.4022650695910071d-01*HX1(0)
     &  - 3.4657359027997265d-01*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)
      Hi4(-1,-1,1,1) = 
     &  - 2.4532465311320902d+00
     &  + 1.8851605738073271d+00*HX1(-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)
     &  - HX1( -1)*HX2(0,-1)
     &  - HX1( -1)*HX2(0,1)
     &  - 1.8851605738073271d+00*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)
     &  + HX3( -1,-1,1)
     &  + HX3(0, -1,-1)
     &  + HX3(0,0, -1)
     &  + HX3(0,0,1) 
     &  + HX3(0,1, -1)
      Hi4(-1,1,1,1) = 
     &  - 5.5504108664821579d-02
     &  - 1.6449340668482264d+00*HX1(-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)
     &  + 1.6449340668482264d+00*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)
     &  + HX1(0) *HX2(-1,1)
     &  - HX1(0) *HX2(0,-1)
     &  - HX1(0) *HX2(0,1)
     &  + HX3( -1,1,1)
     &  - HX3(0, -1,1)
     &  + HX3(0,0, -1)
     &  + HX3(0,0,1) 
     &  - HX3(0,1,1) 
      endif 
      if ( nw.gt.4 ) then 
      HY5(-1,-1,-1,-1,1) = 
     &  - 2.1900870176160439d+00
     &  + 2.4278628067547031d+00*HX1(-1)
     &  - 1.3810359531144620d+00*HX1(-1)*HX1(-1)
     &  + 4.1123351671205660d-01*HX1(-1)*HX1(-1)*HX1(-1)
     &  + 4.1666666666666666d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)
     &  - 8.3333333333333333d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX2(0,1)
     &  - 1.2337005501361698d+00*HX1(-1)*HX1(-1)*HX1(0)
     &  + 8.3333333333333333d-02*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,1,-1)
     &  + 2.7620719062289241d+00*HX1(-1)*HX1(0)
     &  + 1.2337005501361698d+00*HX1(-1)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1( -1)*HX4(0,-1,-1,-1)
     &  - HX1( -1)*HX4(0,0,-1,-1)
     &  - HX1( -1)*HX4(0,0,0,-1)
     &  - HX1( -1)*HX4(0,0,0,1)
     &  - HX1( -1)*HX4(0,0,1,-1)
     &  - HX1( -1)*HX4(0,1,-1,-1)
     &  - 2.4278628067547031d+00*HX1(0)
     &  - 1.3810359531144620d+00*HX1(0)*HX1(0)
     &  - 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX5( -1,-1,-1,-1,1)
     &  + HX5(0, -1,-1,-1,-1)
     &  + HX5(0,0, -1,-1,-1)
     &  + HX5(0,0,0, -1,-1)
     &  + HX5(0,0,0,0, -1)
     &  + HX5(0,0,0,0,1) 
     &  + HX5(0,0,0,1, -1)
     &  + HX5(0,0,1, -1,-1)
     &  + HX5(0,1, -1,-1,-1)
      HY5(-1,-1,-1,1,1) = 
     &  - 5.5581622138319701d+00
     &  + 2.0293560632083841d+00*HX1(-1)
     &  + 1.3810359531144620d+00*HX1(-1)*HX1(-1)
     &  - 8.2246703342411321d-01*HX1(-1)*HX1(-1)*HX1(-1)
     &  + 8.3333333333333333d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  + 2.4674011002723396d+00*HX1(-1)*HX1(-1)*HX1(0)
     &  - 8.3333333333333333d-02*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,-1,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,1,1)
     &  - 2.7620719062289241d+00*HX1(-1)*HX1(0)
     &  - 2.4674011002723396d+00*HX1(-1)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1( -1)*HX1(0)*HX3(0,-1,-1)
     &  + HX1( -1)*HX1(0)*HX3(0,0,-1)
     &  + HX1( -1)*HX1(0)*HX3(0,0,1)
     &  + HX1( -1)*HX1(0)*HX3(0,1,-1)
     &  + HX1( -1)*HX4(0,-1,-1,1)
     &  + HX1( -1)*HX4(0,-1,1,-1)
     &  - HX1( -1)*HX4(0,0,-1,-1)
     &  + HX1( -1)*HX4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX4(0,0,0,1)
     &  - HX1( -1)*HX4(0,0,1,-1)
     &  + HX1( -1)*HX4(0,0,1,1)
     &  + HX1( -1)*HX4(0,1,-1,1)
     &  + HX1( -1)*HX4(0,1,1,-1)
     &  - 2.0293560632083841d+00*HX1(0)
     &  + 1.3810359531144620d+00*HX1(0)*HX1(0)
     &  + 8.2246703342411321d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1(0) *HX4(-1,-1,-1,1)
     &  - HX1(0) *HX4(0,-1,-1,-1)
     &  - HX1(0) *HX4(0,0,-1,-1)
     &  - HX1(0) *HX4(0,0,0,-1)
     &  - HX1(0) *HX4(0,0,0,1)
     &  - HX1(0) *HX4(0,0,1,-1)
     &  - HX1(0) *HX4(0,1,-1,-1)
     &  + HX5( -1,-1,-1,1,1)
     &  - HX5(0, -1,-1,-1,1)
     &  - HX5(0, -1,-1,1,-1)
     &  - HX5(0, -1,1,-1,-1)
     &  + HX5(0,0, -1,-1,-1)
     &  - HX5(0,0, -1,-1,1)
     &  - HX5(0,0, -1,1,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  - HX5(0,0,0, -1,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - HX5(0,0,0,1,1) 
     &  + HX5(0,0,1, -1,-1)
     &  - HX5(0,0,1, -1,1)
     &  - HX5(0,0,1,1, -1)
     &  - HX5(0,1, -1,-1,1)
     &  - HX5(0,1, -1,1,-1)
     &  - HX5(0,1,1, -1,-1)
      HY5(-1,-1,1,-1,1) = 
     &  + 9.2329419831013177d+00
     &  - 3.3856186219460558d+00*HX1(-1)
     &  + 6.5847232569963413d-01*HX1(-1)*HX1(-1)
     &  + 1.2337005501361698d+00*HX1(-1)*HX1(-1)*HX1(0)
     &  - 8.3333333333333333d-02*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX2(0,1)
     &  + HX1( -1)*HX1(-1)*HX3(0,-1,1)
     &  - HX1( -1)*HX1(-1)*HX3(0,0,-1)
     &  - HX1( -1)*HX1(-1)*HX3(0,0,1)
     &  + HX1( -1)*HX1(-1)*HX3(0,1,1)
     &  - 1.3169446513992682d+00*HX1(-1)*HX1(0)
     &  - 1.2337005501361698d+00*HX1(-1)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX2(0,1)
     &  + HX1( -1)*HX1(0)*HX3(-1,-1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX3(0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX3(0,0,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX3(0,1,-1)
     &  - 2.4674011002723396d+00*HX1(-1)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX2(0,-1)*HX2(0,-1)
     &  + HX1( -1)*HX2(0,-1)*HX2(0,1)
     &  - 2.4674011002723396d+00*HX1(-1)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX2(0,1)*HX2(0,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX4(0,0,-1,1)
     &  + 4.0000000000000000d+00*HX1(-1)*HX4(0,0,0,-1)
     &  + 4.0000000000000000d+00*HX1(-1)*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX4(0,0,1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX4(0,1,1,-1)
     &  + 3.3856186219460558d+00*HX1(0)
     &  + 6.5847232569963413d-01*HX1(0)*HX1(0)
     &  + 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(-1,-1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,1,-1,-1)
     &  - HX2(0, -1)*HX3(-1,-1,1)
     &  - HX2(0, -1)*HX3(0,-1,-1)
     &  - HX2(0, -1)*HX3(0,1,-1)
     &  - HX2(0,1) *HX3(-1,-1,1)
     &  - HX2(0,1) *HX3(0,1,-1)
     &  + 2.4674011002723396d+00*HX3(-1,-1,1)
     &  + 2.4674011002723396d+00*HX3(0,-1,-1)
     &  + 2.4674011002723396d+00*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX3(0,1,-1)
     &  + HX5( -1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HX5(0,-1,-1,-1,1)
     &  - HX5(0, -1,-1,0,1)
     &  + 2.0000000000000000d+00*HX5(0,-1,-1,1,-1)
     &  + HX5(0, -1,0,-1,-1)
     &  - HX5(0, -1,0,-1,1)
     &  + 2.0000000000000000d+00*HX5(0,-1,1,-1,-1)
     &  + HX5(0,0, -1,-1,1)
     &  - HX5(0,0, -1,0,-1)
     &  - HX5(0,0, -1,0,1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,1,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,1,-1,1)
     &  - HX5(0,0,1,0, -1)
     &  - HX5(0,0,1,0,1) 
     &  + 4.0000000000000000d+00*HX5(0,0,1,1,-1)
     &  + 3.0000000000000000d+00*HX5(0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HX5(0,1,-1,1,-1)
     &  + HX5(0,1,0,1, -1)
     &  + 2.0000000000000000d+00*HX5(0,1,1,-1,-1)
      HY5(-1,-1,1,1,1) = 
     &  + 5.5581622138319701d+00
     &  - 6.4865749331714713d+00*HX1(-1)
     &  - 2.4674011002723396d+00*HX1(-1)*HX1(-1)*HX1(0)
     &  + 8.3333333333333333d-02*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 2.4674011002723396d+00*HX1(-1)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX2(0,1)
     &  - HX1( -1)*HX1(0)*HX3(0,-1,1)
     &  + HX1( -1)*HX1(0)*HX3(0,0,-1)
     &  + HX1( -1)*HX1(0)*HX3(0,0,1)
     &  - HX1( -1)*HX1(0)*HX3(0,1,1)
     &  + 4.9348022005446793d+00*HX1(-1)*HX2(0,-1)
     &  + 4.9348022005446793d+00*HX1(-1)*HX2(0,1)
     &  - HX1( -1)*HX4(0,-1,1,1)
     &  + HX1( -1)*HX4(0,0,-1,1)
     &  - HX1( -1)*HX4(0,0,0,-1)
     &  - HX1( -1)*HX4(0,0,0,1)
     &  + HX1( -1)*HX4(0,0,1,1)
     &  - HX1( -1)*HX4(0,1,1,1)
     &  + 6.4865749331714713d+00*HX1(0)
     &  - 8.2246703342411321d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(-1,-1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  + HX1(0) *HX4(-1,-1,1,1)
     &  + HX1(0) *HX4(0,-1,-1,1)
     &  + HX1(0) *HX4(0,-1,1,-1)
     &  - HX1(0) *HX4(0,0,-1,-1)
     &  + HX1(0) *HX4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX4(0,0,1,-1)
     &  + HX1(0) *HX4(0,0,1,1)
     &  + HX1(0) *HX4(0,1,-1,1)
     &  + HX1(0) *HX4(0,1,1,-1)
     &  - 4.9348022005446793d+00*HX3(-1,-1,1)
     &  - 4.9348022005446793d+00*HX3(0,-1,-1)
     &  - 4.9348022005446793d+00*HX3(0,0,-1)
     &  - 4.9348022005446793d+00*HX3(0,0,1)
     &  - 4.9348022005446793d+00*HX3(0,1,-1)
     &  + HX5( -1,-1,1,1,1)
     &  + HX5(0, -1,-1,1,1)
     &  + HX5(0, -1,1,-1,1)
     &  + HX5(0, -1,1,1,-1)
     &  - HX5(0,0, -1,-1,1)
     &  - HX5(0,0, -1,1,-1)
     &  + HX5(0,0, -1,1,1)
     &  + HX5(0,0,0, -1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + HX5(0,0,0,1, -1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,1,1)
     &  - HX5(0,0,1, -1,1)
     &  - HX5(0,0,1,1, -1)
     &  + HX5(0,0,1,1,1) 
     &  + HX5(0,1, -1,1,1)
     &  + HX5(0,1,1, -1,1)
     &  + HX5(0,1,1,1, -1)
      HY5(-1,1,-1,1,1) = 
     &  - 7.7439193717015657d-01
     &  + 8.5393570350547734d-01*HX1(-1)
     &  + 2.7620719062289241d+00*HX1(-1)*HX1(0)
     &  + 2.4674011002723396d+00*HX1(-1)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX2(-1,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX3(0,-1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX3(0,1,1)
     &  - 4.9348022005446793d+00*HX1(-1)*HX2(-1,1)
     &  - 4.9348022005446793d+00*HX1(-1)*HX2(0,-1)
     &  - 4.9348022005446793d+00*HX1(-1)*HX2(0,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HX1(-1)*HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX4(0,0,0,1)
     &  - 3.0000000000000000d+00*HX1(-1)*HX4(0,0,1,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX4(0,1,1,1)
     &  - 8.5393570350547734d-01*HX1(0)
     &  - 1.3810359531144620d+00*HX1(0)*HX1(0)
     &  - 8.2246703342411321d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(-1,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - HX1(0) *HX1(0)*HX3(-1,-1,1)
     &  - HX1(0) *HX1(0)*HX3(0,-1,-1)
     &  - HX1(0) *HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX1(0)*HX3(0,0,1)
     &  - HX1(0) *HX1(0)*HX3(0,1,-1)
     &  + 4.9348022005446793d+00*HX1(0)*HX2(-1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(-1,1)*HX2(-1,1)
     &  - HX1(0) *HX2(-1,1)*HX2(0,-1)
     &  - HX1(0) *HX2(-1,1)*HX2(0,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + HX1(0) *HX2(0,-1)*HX2(0,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(0,1)*HX2(0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,1,1,-1)
     &  + 2.7620719062289241d+00*HX2(-1,1)
     &  - HX2( -1,1)*HX3(0,-1,1)
     &  + HX2( -1,1)*HX3(0,0,-1)
     &  + HX2( -1,1)*HX3(0,0,1)
     &  - HX2( -1,1)*HX3(0,1,1)
     &  - 2.7620719062289241d+00*HX2(0,-1)
     &  - HX2(0, -1)*HX3(0,0,-1)
     &  - HX2(0, -1)*HX3(0,0,1)
     &  - 2.7620719062289241d+00*HX2(0,1)
     &  + HX2(0,1) *HX3(0,-1,1)
     &  - HX2(0,1) *HX3(0,0,-1)
     &  - HX2(0,1) *HX3(0,0,1)
     &  + 9.8696044010893586d+00*HX3(-1,-1,1)
     &  + 9.8696044010893586d+00*HX3(0,-1,-1)
     &  + 9.8696044010893586d+00*HX3(0,0,-1)
     &  + 9.8696044010893586d+00*HX3(0,0,1)
     &  + 9.8696044010893586d+00*HX3(0,1,-1)
     &  + HX5( -1,1,-1,1,1)
     &  - 2.0000000000000000d+00*HX5(0,-1,-1,1,1)
     &  + HX5(0, -1,0,-1,1)
     &  - HX5(0, -1,0,1,1)
     &  - 2.0000000000000000d+00*HX5(0,-1,1,-1,1)
     &  - HX5(0, -1,1,0,1)
     &  - 3.0000000000000000d+00*HX5(0,-1,1,1,-1)
     &  + 4.0000000000000000d+00*HX5(0,0,-1,-1,1)
     &  + HX5(0,0, -1,0,-1)
     &  + HX5(0,0, -1,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,-1,1,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,-1,1,1)
     &  + 7.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 7.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + 2.0000000000000000d+00*HX5(0,0,1,-1,1)
     &  + HX5(0,0,1,0, -1)
     &  + HX5(0,0,1,0,1) 
     &  + 3.0000000000000000d+00*HX5(0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,1,-1,1,1)
     &  + HX5(0,1,0,1,1) 
     &  - 2.0000000000000000d+00*HX5(0,1,1,-1,1)
     &  - 3.0000000000000000d+00*HX5(0,1,1,1,-1)
      HY5(-1,1,1,1,1) = 
     &  + 2.1900870176160439d+00
     &  + 4.0587121264167682d+00*HX1(-1)
     &  - 2.4674011002723396d+00*HX1(-1)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.0587121264167682d+00*HX1(0)
     &  + 8.2246703342411321d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(-1,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(-1,1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX2(-1,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX2(0,-1)
     &  + 4.9348022005446793d+00*HX1(0)*HX2(0,1)
     &  + HX1(0) *HX4(-1,1,1,1)
     &  - HX1(0) *HX4(0,-1,1,1)
     &  + HX1(0) *HX4(0,0,-1,1)
     &  - HX1(0) *HX4(0,0,0,-1)
     &  - HX1(0) *HX4(0,0,0,1)
     &  + HX1(0) *HX4(0,0,1,1)
     &  - HX1(0) *HX4(0,1,1,1)
     &  - 4.9348022005446793d+00*HX3(-1,1,1)
     &  + 4.9348022005446793d+00*HX3(0,-1,1)
     &  - 4.9348022005446793d+00*HX3(0,0,-1)
     &  - 4.9348022005446793d+00*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX3(0,1,1)
     &  + HX5( -1,1,1,1,1)
     &  - HX5(0, -1,1,1,1)
     &  + HX5(0,0, -1,1,1)
     &  - HX5(0,0,0, -1,1)
     &  + HX5(0,0,0,0, -1)
     &  + HX5(0,0,0,0,1) 
     &  - HX5(0,0,0,1,1) 
     &  + HX5(0,0,1,1,1) 
     &  - HX5(0,1,1,1,1) 
      HY5(0,-1,-1,-1,1) = 
     &  - 2.1501021594785064d+00
     &  - 2.4278628067547031d+00*HX1(0)
     &  - 1.3810359531144620d+00*HX1(0)*HX1(0)
     &  - 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX4(0,-1,-1,-1)
     &  - HX1(0) *HX4(0,0,-1,-1)
     &  - HX1(0) *HX4(0,0,0,-1)
     &  + 2.7620719062289241d+00*HX2(0,-1)
     &  + HX2(0, -1)*HX3(0,-1,-1)
     &  - HX2(0, -1)*HX3(0,0,-1)
     &  - HX2(0, -1)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX3(0,-1,-1)
     &  - 2.4674011002723396d+00*HX3(0,0,-1)
     &  - HX5(0, -1,-1,-1,1)
     &  + HX5(0, -1,-1,0,1)
     &  - 2.0000000000000000d+00*HX5(0,-1,0,-1,-1)
     &  + HX5(0, -1,0,-1,1)
     &  - 4.0000000000000000d+00*HX5(0,0,-1,-1,-1)
     &  + HX5(0,0, -1,-1,1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,0,-1)
     &  + HX5(0,0, -1,0,1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + HX5(0,0,0,0,1) 
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + HX5(0,0,1,0, -1)
      HY5(0,-1,-1,0,1) = 
     &  + 2.6781232869596824d+00
     &  + 2.0293560632083841d-01*HX1(0)
     &  - 9.0154267736969571d-01*HX1(0)*HX1(0)
     &  - 5.4831135561607547d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 3.2898681336964528d+00*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + 1.8030853547393914d+00*HX2(0,-1)
     &  + HX2(0, -1)*HX3(0,0,-1)
     &  - HX2(0, -1)*HX3(0,0,1)
     &  - 3.2898681336964528d+00*HX3(0,-1,-1)
     &  - 3.2898681336964528d+00*HX3(0,0,-1)
     &  + HX5(0, -1,-1,0,1)
     &  - HX5(0,0, -1,0,-1)
     &  + HX5(0,0, -1,0,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + HX5(0,0,0,0,1) 
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + HX5(0,0,1,0, -1)
      HY5(0,-1,-1,1,-1) = 
     &  + 7.5170363885043517d+00
     &  + 7.2835884202641093d+00*HX1(0)
     &  + 2.7620719062289241d+00*HX1(0)*HX1(0)
     &  + 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 2.4674011002723396d+00*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + HX1(0) *HX4(0,-1,-1,1)
     &  - HX1(0) *HX4(0,-1,0,1)
     &  - HX1(0) *HX4(0,0,-1,1)
     &  + HX1(0) *HX4(0,0,0,1)
     &  - 5.5241438124578482d+00*HX2(0,-1)
     &  - HX2(0, -1)*HX3(0,-1,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX3(0,-1,-1)
     &  + 2.4674011002723396d+00*HX3(0,0,-1)
     &  - HX5(0, -1,-1,0,1)
     &  - HX5(0, -1,-1,1,-1)
     &  + 3.0000000000000000d+00*HX5(0,-1,0,-1,-1)
     &  - HX5(0, -1,0,-1,1)
     &  + HX5(0, -1,0,1,-1)
     &  + 6.0000000000000000d+00*HX5(0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HX5(0,0,-1,0,-1)
     &  - HX5(0,0, -1,0,1)
     &  + HX5(0,0, -1,1,-1)
     &  - 9.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,1,0,-1)
      HY5(0,-1,-1,1,1) = 
     &  - 6.8749092010391177d+00
     &  - 2.0293560632083841d+00*HX1(0)
     &  + 1.3810359531144620d+00*HX1(0)*HX1(0)
     &  + 8.2246703342411321d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 4.9348022005446793d+00*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  - HX1(0) *HX4(0,-1,-1,1)
     &  + HX1(0) *HX4(0,-1,0,1)
     &  + HX1(0) *HX4(0,0,-1,1)
     &  - HX1(0) *HX4(0,0,0,1)
     &  - 2.7620719062289241d+00*HX2(0,-1)
     &  - HX2(0, -1)*HX3(0,0,-1)
     &  - HX2(0, -1)*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX3(0,-1,-1)
     &  + 4.9348022005446793d+00*HX3(0,0,-1)
     &  - HX5(0, -1,-1,1,1)
     &  + HX5(0, -1,0,-1,1)
     &  + HX5(0, -1,0,1,1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,-1,1)
     &  + HX5(0,0, -1,0,-1)
     &  + HX5(0,0, -1,1,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - HX5(0,0,0,1,1) 
     &  + HX5(0,0,1,0, -1)
      HY5(0,-1,0,-1,1) = 
     &  - 5.6642178198849788d+00
     &  - 3.4847341165810188d+00*HX1(0)
     &  - 1.2595007772794312d+00*HX1(0)*HX1(0)
     &  - 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX1(0)*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 2.5190015545588625d+00*HX2(0,-1)
     &  - 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,-1)
     &  - HX2(0, -1)*HX3(0,0,1)
     &  - 4.9348022005446793d+00*HX3(0,0,-1)
     &  + HX5(0, -1,0,-1,1)
     &  + 4.0000000000000000d+00*HX5(0,0,-1,0,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,0,1)
     &  + 1.2000000000000000d+01*HX5(0,0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 8.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + HX5(0,0,0,0,1) 
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + HX5(0,0,1,0, -1)
      HY5(0,-1,0,1,-1) = 
     &  - 3.4903562040510891d-01
     &  + 3.0788629039393420d+00*HX1(0)
     &  + 2.1610434546491269d+00*HX1(0)*HX1(0)
     &  + 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX1(0)*HX3(0,0,-1)
     &  - 2.4674011002723396d+00*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX4(0,-1,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + HX1(0) *HX4(0,0,0,1)
     &  - 4.3220869092982539d+00*HX2(0,-1)
     &  + HX2(0, -1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX3(0,0,-1)
     &  + HX5(0, -1,0,1,-1)
     &  - 3.0000000000000000d+00*HX5(0,0,-1,0,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,0,1)
     &  - 6.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  - 6.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,1,0,-1)
      HY5(0,-1,0,1,1) = 
     &  - 6.6327319875542747d+00
     &  - 3.8557765200959298d+00*HX1(0)
     &  + 6.0102845157979714d-01*HX1(0)*HX1(0)
     &  + 8.2246703342411321d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX1(0)*HX3(0,0,-1)
     &  - 4.9348022005446793d+00*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX4(0,-1,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - HX1(0) *HX4(0,0,0,1)
     &  - 1.2020569031595942d+00*HX2(0,-1)
     &  - HX2(0, -1)*HX3(0,0,1)
     &  + 9.8696044010893586d+00*HX3(0,0,-1)
     &  + HX5(0, -1,0,1,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - HX5(0,0,0,1,1) 
     &  + HX5(0,0,1,0, -1)
      HY5(0,-1,1,-1,-1) = 
     &  - 1.4571792178347941d+01
     &  - 7.2835884202641093d+00*HX1(0)
     &  - 1.3810359531144620d+00*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  + HX1(0) *HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + HX1(0) *HX4(0,-1,0,1)
     &  + HX1(0) *HX4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX4(0,0,1,-1)
     &  + 2.7620719062289241d+00*HX2(0,-1)
     &  - HX2(0, -1)*HX3(0,0,-1)
     &  - HX2(0, -1)*HX3(0,0,1)
     &  - HX5(0, -1,0,-1,-1)
     &  - HX5(0, -1,0,1,-1)
     &  - HX5(0, -1,1,-1,-1)
     &  + HX5(0,0, -1,0,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,1,-1)
     &  + 7.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  + 7.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + HX5(0,0,1, -1,-1)
     &  + HX5(0,0,1,0, -1)
      HY5(0,-1,1,-1,1) = 
     &  + 8.4053642852001231d+00
     &  + 3.3856186219460558d+00*HX1(0)
     &  + 6.5847232569963413d-01*HX1(0)*HX1(0)
     &  + 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  - HX1(0) *HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  - HX1(0) *HX4(0,-1,0,1)
     &  - HX1(0) *HX4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + HX1(0) *HX4(0,0,1,-1)
     &  - 1.3169446513992682d+00*HX2(0,-1)
     &  + HX2(0, -1)*HX3(0,-1,1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX3(0,-1,1)
     &  + 4.9348022005446793d+00*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX3(0,0,1)
     &  - 3.0000000000000000d+00*HX5(0,-1,0,-1,1)
     &  - HX5(0, -1,1,-1,1)
     &  + HX5(0, -1,1,0,1)
     &  - 4.0000000000000000d+00*HX5(0,0,-1,-1,1)
     &  - 4.0000000000000000d+00*HX5(0,0,-1,0,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,0,1)
     &  - 1.2000000000000000d+01*HX5(0,0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 8.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 9.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + HX5(0,0,1, -1,1)
     &  - 3.0000000000000000d+00*HX5(0,0,1,0,-1)
     &  - HX5(0,0,1,0,1) 
      HY5(0,-1,1,0,1) = 
     &  - 4.5667800194983164d-01
     &  - 3.4847341165810188d+00*HX1(0)
     &  - 1.2020569031595942d+00*HX1(0)*HX1(0)
     &  + 5.4831135561607547d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  - HX1(0) *HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 3.2898681336964528d+00*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + 2.4041138063191885d+00*HX2(0,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,1)
     &  - 3.2898681336964528d+00*HX3(0,-1,1)
     &  + 6.5797362673929057d+00*HX3(0,0,-1)
     &  + 3.2898681336964528d+00*HX3(0,0,1)
     &  + HX5(0, -1,1,0,1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 6.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,1,0,-1)
     &  - HX5(0,0,1,0,1) 
      HY5(0,-1,1,1,-1) = 
     &  + 8.0778313134277241d+00
     &  + 6.7309350447071233d-01*HX1(0)
     &  - 2.0395082788140962d+00*HX1(0)*HX1(0)
     &  - 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  - HX1(0) *HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX4(0,0,1,1)
     &  + 4.0790165576281924d+00*HX2(0,-1)
     &  - HX2(0, -1)*HX3(0,-1,1)
     &  - HX2(0, -1)*HX3(0,0,-1)
     &  - HX2(0, -1)*HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX3(0,-1,1)
     &  - 4.9348022005446793d+00*HX3(0,0,-1)
     &  - 2.4674011002723396d+00*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX5(0,-1,0,-1,1)
     &  - HX5(0, -1,0,1,1)
     &  - HX5(0, -1,1,0,1)
     &  - HX5(0, -1,1,1,-1)
     &  + 4.0000000000000000d+00*HX5(0,0,-1,-1,1)
     &  + 3.0000000000000000d+00*HX5(0,0,-1,0,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,0,1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,1,1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 4.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + 2.0000000000000000d+00*HX5(0,0,1,0,-1)
     &  + HX5(0,0,1,0,1) 
     &  + HX5(0,0,1,1, -1)
      HY5(0,-1,1,1,1) = 
     &  + 6.9367501878022481d+00
     &  + 6.4865749331714713d+00*HX1(0)
     &  - 8.2246703342411321d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  + HX1(0) *HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX4(0,-1,1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + HX1(0) *HX4(0,0,1,1)
     &  + 4.9348022005446793d+00*HX3(0,-1,1)
     &  - 9.8696044010893586d+00*HX3(0,0,-1)
     &  - 4.9348022005446793d+00*HX3(0,0,1)
     &  - HX5(0, -1,1,1,1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,1,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 4.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + HX5(0,0,1,1,1) 
      HY5(0,0,-1,-1,1) = 
     &  - 9.1165552893555038d-01
     &  - 2.5209599327464717d+00*HX1(0)
     &  - 1.3810359531144620d+00*HX1(0)*HX1(0)
     &  - 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 2.4674011002723396d+00*HX3(0,0,-1)
     &  + HX5(0,0, -1,-1,1)
     &  - HX5(0,0, -1,0,-1)
     &  - HX5(0,0, -1,0,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  - HX5(0,0,0, -1,1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + HX5(0,0,0,0,1) 
      HY5(0,0,-1,0,-1) = 
     &  - 8.1023197211680719d+00
     &  - 5.1410353601279064d+00*HX1(0)
     &  - 1.2020569031595942d+00*HX1(0)*HX1(0)
     &  - 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 1.6449340668482264d+00*HX3(0,0,-1)
     &  - HX5(0,0, -1,0,-1)
     &  + 7.0000000000000000d+00*HX5(0,0,0,0,-1)
      HY5(0,0,-1,0,1) = 
     &  + 2.0434339691042511d+00
     &  - 9.4703282949724591d-01*HX1(0)
     &  - 9.0154267736969571d-01*HX1(0)*HX1(0)
     &  - 5.4831135561607547d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 3.2898681336964528d+00*HX3(0,0,-1)
     &  - HX5(0,0, -1,0,1)
     &  - 6.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + HX5(0,0,0,0,1) 
      HY5(0,0,-1,1,-1) = 
     &  + 1.2874316759375889d+01
     &  + 8.5266539820739622d+00*HX1(0)
     &  + 2.7620719062289241d+00*HX1(0)*HX1(0)
     &  + 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + HX1(0) *HX4(0,0,0,1)
     &  - 2.4674011002723396d+00*HX3(0,0,-1)
     &  + HX5(0,0, -1,0,-1)
     &  + HX5(0,0, -1,0,1)
     &  + HX5(0,0, -1,1,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - HX5(0,0,0,1, -1)
      HY5(0,0,-1,1,1) = 
     &  - 4.2205881177506120d+00
     &  - 6.2689427375197987d-01*HX1(0)
     &  + 1.3810359531144620d+00*HX1(0)*HX1(0)
     &  + 8.2246703342411321d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX4(0,0,-1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - HX1(0) *HX4(0,0,0,1)
     &  - 4.9348022005446793d+00*HX3(0,0,-1)
     &  + HX5(0,0, -1,1,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - HX5(0,0,0,1,1) 
      HY5(0,0,0,-1,1) = 
     &  - 4.8071216221292780d+00
     &  - 3.9234217222028759d+00*HX1(0)
     &  - 1.2595007772794312d+00*HX1(0)*HX1(0)
     &  - 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1(0) *HX4(0,0,0,-1)
     &  - HX5(0,0,0, -1,1)
     &  + 4.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + HX5(0,0,0,0,1) 
      HY5(0,0,0,1,-1) = 
     &  + 5.2683829003001246d+00
     &  + 4.1940025306306604d+00*HX1(0)
     &  + 2.1610434546491269d+00*HX1(0)*HX1(0)
     &  + 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1(0) *HX4(0,0,0,1)
     &  - HX5(0,0,0,0, -1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - HX5(0,0,0,1, -1)
      HY5(0,0,1,-1,-1) = 
     &  - 9.4096904031705199d+00
     &  - 5.8027584430066521d+00*HX1(0)
     &  - 1.3810359531144620d+00*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - HX1(0) *HX4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX4(0,0,1,-1)
     &  + HX5(0,0,0, -1,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + HX5(0,0,1, -1,-1)
      HY5(0,0,1,-1,1) = 
     &  + 8.7596705482058876d+00
     &  + 4.3326514514433017d+00*HX1(0)
     &  + 6.5847232569963413d-01*HX1(0)*HX1(0)
     &  + 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + HX1(0) *HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + HX1(0) *HX4(0,0,1,-1)
     &  + 2.4674011002723396d+00*HX3(0,0,1)
     &  + HX5(0,0,0, -1,1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + HX5(0,0,1, -1,1)
     &  - HX5(0,0,1,0, -1)
     &  - HX5(0,0,1,0,1) 
      HY5(0,0,1,0,-1) = 
     &  - 6.8544356072335814d+00
     &  - 5.0057449559140141d+00*HX1(0)
     &  - 9.0154267736969571d-01*HX1(0)*HX1(0)
     &  + 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + 1.6449340668482264d+00*HX3(0,0,1)
     &  - HX5(0,0,0,0, -1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - HX5(0,0,1,0, -1)
      HY5(0,0,1,1,-1) = 
     &  + 3.2887945813357094d+00
     &  + 1.5001934240460787d-01*HX1(0)
     &  - 2.0395082788140962d+00*HX1(0)*HX1(0)
     &  - 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX4(0,0,1,1)
     &  - 2.4674011002723396d+00*HX3(0,0,1)
     &  + HX5(0,0,0,0, -1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + HX5(0,0,0,1, -1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + HX5(0,0,1,0, -1)
     &  + HX5(0,0,1,0,1) 
     &  + HX5(0,0,1,1, -1)
      HY5(0,1,-1,-1,-1) = 
     &  + 7.4845537829591132d+00
     &  + 2.4278628067547031d+00*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  + HX1(0) *HX4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,1,-1)
     &  + HX1(0) *HX4(0,1,-1,-1)
     &  - HX5(0,0, -1,-1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,1,-1,-1)
     &  - HX5(0,1, -1,-1,-1)
      HY5(0,1,-1,-1,1) = 
     &  + 7.4122526767078634d-02
     &  + 3.0440340948125761d+00*HX1(0)
     &  + 1.3810359531144620d+00*HX1(0)*HX1(0)
     &  + 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX1(0)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX2(0,1)
     &  - HX1(0) *HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,1,-1)
     &  - HX1(0) *HX4(0,1,-1,-1)
     &  + HX2(0, -1)*HX3(0,1,-1)
     &  + 2.7620719062289241d+00*HX2(0,1)
     &  - HX2(0,1) *HX3(0,-1,-1)
     &  - HX2(0,1) *HX3(0,0,-1)
     &  - HX2(0,1) *HX3(0,0,1)
     &  + HX2(0,1) *HX3(0,1,-1)
     &  - 2.4674011002723396d+00*HX3(0,0,-1)
     &  - 4.9348022005446793d+00*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX3(0,1,-1)
     &  + HX5(0, -1,-1,0,1)
     &  + HX5(0, -1,0,-1,1)
     &  + HX5(0,0, -1,-1,1)
     &  + HX5(0,0, -1,0,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  + 4.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,1,1)
     &  - 2.0000000000000000d+00*HX5(0,0,1,-1,1)
     &  + 2.0000000000000000d+00*HX5(0,0,1,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,1,0,1)
     &  - 4.0000000000000000d+00*HX5(0,0,1,1,-1)
     &  - HX5(0,1, -1,-1,1)
     &  - 2.0000000000000000d+00*HX5(0,1,0,1,-1)
      HY5(0,1,-1,1,-1) = 
     &  - 1.8385793928455058d+01
     &  - 9.4736868115712082d+00*HX1(0)
     &  - 2.7620719062289241d+00*HX1(0)*HX1(0)
     &  - 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX1(0)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  - HX1(0) *HX2(0,-1)*HX2(0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX2(0,1)*HX2(0,1)
     &  + HX1(0) *HX4(0,-1,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,1,1)
     &  + HX1(0) *HX4(0,1,-1,1)
     &  - HX2(0, -1)*HX3(0,1,-1)
     &  - 5.5241438124578482d+00*HX2(0,1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX3(0,-1,-1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX3(0,0,1)
     &  - HX2(0,1) *HX3(0,1,-1)
     &  + 2.4674011002723396d+00*HX3(0,0,-1)
     &  + 4.9348022005446793d+00*HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX3(0,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HX5(0,-1,0,-1,1)
     &  - HX5(0, -1,0,1,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,-1,-1,1)
     &  - HX5(0,0, -1,0,-1)
     &  - 3.0000000000000000d+00*HX5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX5(0,0,-1,1,-1)
     &  - 9.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 7.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 8.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 1.2000000000000000d+01*HX5(0,0,0,1,1)
     &  - 2.0000000000000000d+00*HX5(0,0,1,0,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,1,0,1)
     &  + 4.0000000000000000d+00*HX5(0,0,1,1,-1)
     &  - HX5(0,1, -1,1,-1)
     &  + 3.0000000000000000d+00*HX5(0,1,0,1,-1)
      HY5(0,1,-1,1,1) = 
     &  + 7.0189712804378578d-01
     &  - 8.5393570350547734d-01*HX1(0)
     &  - 1.3810359531144620d+00*HX1(0)*HX1(0)
     &  - 8.2246703342411321d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  + HX1(0) *HX2(0,-1)*HX2(0,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(0,1)*HX2(0,1)
     &  - HX1(0) *HX4(0,-1,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,1,1)
     &  - HX1(0) *HX4(0,1,-1,1)
     &  - 2.7620719062289241d+00*HX2(0,1)
     &  + HX2(0,1) *HX3(0,-1,1)
     &  - HX2(0,1) *HX3(0,0,-1)
     &  - HX2(0,1) *HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX3(0,0,-1)
     &  + 9.8696044010893586d+00*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX3(0,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,-1,0,1,1)
     &  - HX5(0, -1,1,0,1)
     &  + HX5(0,0, -1,0,1)
     &  - 5.0000000000000000d+00*HX5(0,0,-1,1,1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 6.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 7.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + HX5(0,0,1,0,1) 
     &  - HX5(0,1, -1,1,1)
     &  + HX5(0,1,0,1,1) 
      HY5(0,1,0,1,-1) = 
     &  - 1.0648543070067714d+01
     &  - 4.6326901362525175d+00*HX1(0)
     &  - 2.1610434546491269d+00*HX1(0)*HX1(0)
     &  - 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + HX1(0) *HX1(0)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX2(0,1)*HX2(0,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,1,1)
     &  - 4.3220869092982539d+00*HX2(0,1)
     &  + HX2(0,1) *HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX3(0,0,1)
     &  - HX5(0,0, -1,0,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + HX5(0,0,0,0, -1)
     &  + 8.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 1.2000000000000000d+01*HX5(0,0,0,1,1)
     &  - 2.0000000000000000d+00*HX5(0,0,1,0,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,1,0,1)
     &  + HX5(0,1,0,1, -1)
      HY5(0,1,1,-1,-1) = 
     &  + 3.7073850386582300d+00
     &  + 4.4002966535502479d+00*HX1(0)
     &  + 1.3810359531144620d+00*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,1)
     &  + HX1(0) *HX2(0,-1)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(0,1)*HX2(0,1)
     &  - HX1(0) *HX4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  + HX1(0) *HX4(0,0,0,-1)
     &  - HX1(0) *HX4(0,0,1,-1)
     &  + HX1(0) *HX4(0,1,1,-1)
     &  + 2.7620719062289241d+00*HX2(0,1)
     &  - HX2(0,1) *HX3(0,-1,-1)
     &  - HX2(0,1) *HX3(0,0,-1)
     &  - HX2(0,1) *HX3(0,0,1)
     &  + HX5(0, -1,-1,0,1)
     &  + HX5(0, -1,0,-1,1)
     &  + HX5(0, -1,0,1,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,-1,1)
     &  + HX5(0,0, -1,0,1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,1,-1)
     &  - HX5(0,0,0, -1,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + HX5(0,0,1, -1,-1)
     &  + HX5(0,0,1,0,1) 
     &  - 2.0000000000000000d+00*HX5(0,0,1,1,-1)
     &  - HX5(0,1,0,1, -1)
     &  - HX5(0,1,1, -1,-1)
      HY5(0,1,1,-1,1) = 
     &  - 1.0133791970287534d+01
     &  - 5.5757170132531547d+00*HX1(0)
     &  - 6.5847232569963413d-01*HX1(0)*HX1(0)
     &  - 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,1)
     &  - HX1(0) *HX2(0,-1)*HX2(0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX2(0,1)*HX2(0,1)
     &  + HX1(0) *HX4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  - HX1(0) *HX4(0,0,0,-1)
     &  + HX1(0) *HX4(0,0,1,-1)
     &  - HX1(0) *HX4(0,1,1,-1)
     &  + HX2(0, -1)*HX3(0,1,1)
     &  - 1.3169446513992682d+00*HX2(0,1)
     &  - 2.0000000000000000d+00*HX2(0,1)*HX3(0,-1,1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX3(0,0,1)
     &  + HX2(0,1) *HX3(0,1,1)
     &  + 2.4674011002723396d+00*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX3(0,1,1)
     &  + 3.0000000000000000d+00*HX5(0,-1,0,1,1)
     &  + 2.0000000000000000d+00*HX5(0,-1,1,0,1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HX5(0,0,-1,1,1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 4.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 9.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + HX5(0,0,1, -1,1)
     &  - HX5(0,0,1,0, -1)
     &  - 3.0000000000000000d+00*HX5(0,0,1,0,1)
     &  - 6.0000000000000000d+00*HX5(0,0,1,1,1)
     &  - 3.0000000000000000d+00*HX5(0,1,0,1,1)
     &  - HX5(0,1,1, -1,1)
      HY5(0,1,1,1,-1) = 
     &  + 7.7484048807851773d-01
     &  - 5.6922216412839301d-02*HX1(0)
     &  + 2.0395082788140962d+00*HX1(0)*HX1(0)
     &  + 4.1123351671205660d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX2(0,1)
     &  + HX1(0) *HX4(0,0,0,1)
     &  - HX1(0) *HX4(0,0,1,1)
     &  + HX1(0) *HX4(0,1,1,1)
     &  - HX2(0, -1)*HX3(0,1,1)
     &  + 4.0790165576281924d+00*HX2(0,1)
     &  + HX2(0,1) *HX3(0,-1,1)
     &  - HX2(0,1) *HX3(0,0,-1)
     &  - HX2(0,1) *HX3(0,0,1)
     &  - HX2(0,1) *HX3(0,1,1)
     &  - 2.4674011002723396d+00*HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX3(0,1,1)
     &  - HX5(0, -1,0,1,1)
     &  - HX5(0, -1,1,0,1)
     &  + HX5(0,0, -1,0,1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,1,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - HX5(0,0,0,0, -1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + HX5(0,0,1,0, -1)
     &  + 2.0000000000000000d+00*HX5(0,0,1,0,1)
     &  + HX5(0,0,1,1, -1)
     &  + 4.0000000000000000d+00*HX5(0,0,1,1,1)
     &  + 2.0000000000000000d+00*HX5(0,1,0,1,1)
     &  - HX5(0,1,1,1, -1)
      Hi5(-1,-1,-1,-1,1) = 
     &  + 9.6181291076284771d-03
     &  - 5.5504108664821579d-02*HX1(-1)
     &  + 1.2011325347955035d-01*HX1(-1)*HX1(-1)
     &  - 1.1552453009332421d-01*HX1(-1)*HX1(-1)*HX1(-1)
     &  + 4.1666666666666666d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(-1)
     &  - 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)
     &  + 3.4657359027997265d-01*HX1(-1)*HX1(-1)*HX1(0)
     &  + 2.5000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  - 2.4022650695910071d-01*HX1(-1)*HX1(0)
     &  - 3.4657359027997265d-01*HX1(-1)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.5504108664821579d-02*HX1(0)
     &  + 1.2011325347955035d-01*HX1(0)*HX1(0)
     &  + 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
      Hi5(-1,-1,-1,1,1) = 
     &  + 2.3395367393886929d+00
     &  - 2.4532465311320902d+00*HX1(-1)
     &  + 9.4258028690366357d-01*HX1(-1)*HX1(-1)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)
     &  - 2.5000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX2(0,1)
     &  - 1.8851605738073271d+00*HX1(-1)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1( -1)*HX3(0,-1,-1)
     &  + HX1( -1)*HX3(0,0,-1)
     &  + HX1( -1)*HX3(0,0,1)
     &  + HX1( -1)*HX3(0,1,-1)
     &  + 2.4532465311320902d+00*HX1(0)
     &  + 9.4258028690366357d-01*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX4( -1,-1,-1,1)
     &  - HX4(0, -1,-1,-1)
     &  - HX4(0,0, -1,-1)
     &  - HX4(0,0,0, -1)
     &  - HX4(0,0,0,1) 
     &  - HX4(0,0,1, -1)
     &  - HX4(0,1, -1,-1)
      Hi5(-1,-1,1,-1,1) = 
     &  - 5.1782796940106865d+00
     &  + 3.1962209462999012d+00*HX1(-1)
     &  - 6.5146002367115732d-01*HX1(-1)*HX1(-1)
     &  - 3.4657359027997265d-01*HX1(-1)*HX1(-1)*HX1(0)
     &  - 2.5000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX2(0,1)
     &  + 1.3029200473423146d+00*HX1(-1)*HX1(0)
     &  + 3.4657359027997265d-01*HX1(-1)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1( -1)*HX1(0)*HX2(0,-1)
     &  + HX1( -1)*HX1(0)*HX2(0,1)
     &  + 6.9314718055994530d-01*HX1(-1)*HX2(0,-1)
     &  + 6.9314718055994530d-01*HX1(-1)*HX2(0,1)
     &  + HX1( -1)*HX3(-1,-1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX3(0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX3(0,0,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX3(0,1,-1)
     &  - 3.1962209462999012d+00*HX1(0)
     &  - 6.5146002367115732d-01*HX1(0)*HX1(0)
     &  - 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1(0) *HX3(-1,-1,1)
     &  - HX1(0) *HX3(0,-1,-1)
     &  - HX1(0) *HX3(0,0,-1)
     &  - HX1(0) *HX3(0,0,1)
     &  - HX1(0) *HX3(0,1,-1)
     &  - 6.9314718055994530d-01*HX3(-1,-1,1)
     &  - 6.9314718055994530d-01*HX3(0,-1,-1)
     &  - 6.9314718055994530d-01*HX3(0,0,-1)
     &  - 6.9314718055994530d-01*HX3(0,0,1)
     &  - 6.9314718055994530d-01*HX3(0,1,-1)
     &  - 3.0000000000000000d+00*HX4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HX4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HX4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX4(0,0,0,1)
     &  + 3.0000000000000000d+00*HX4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HX4(0,1,-1,-1)
      Hi5(-1,-1,1,1,1) = 
     &  + 3.1512791646720465d+00
     &  - 5.5504108664821579d-02*HX1(-1)
     &  - 8.2246703342411321d-01*HX1(-1)*HX1(-1)
     &  + 2.5000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  + 1.6449340668482264d+00*HX1(-1)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1( -1)*HX1(0)*HX2(0,-1)
     &  - HX1( -1)*HX1(0)*HX2(0,1)
     &  - HX1( -1)*HX3(0,-1,1)
     &  + HX1( -1)*HX3(0,0,-1)
     &  + HX1( -1)*HX3(0,0,1)
     &  - HX1( -1)*HX3(0,1,1)
     &  + 5.5504108664821579d-02*HX1(0)
     &  - 8.2246703342411321d-01*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1(0) *HX3(-1,-1,1)
     &  + HX1(0) *HX3(0,-1,-1)
     &  + HX1(0) *HX3(0,0,-1)
     &  + HX1(0) *HX3(0,0,1)
     &  + HX1(0) *HX3(0,1,-1)
     &  + HX4( -1,-1,1,1)
     &  + HX4(0, -1,-1,1)
     &  + HX4(0, -1,1,-1)
     &  - HX4(0,0, -1,-1)
     &  + HX4(0,0, -1,1)
     &  - 2.0000000000000000d+00*HX4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HX4(0,0,0,1)
     &  - HX4(0,0,1, -1)
     &  + HX4(0,0,1,1) 
     &  + HX4(0,1, -1,1)
     &  + HX4(0,1,1, -1)
      Hi5(-1,1,-1,1,1) = 
     &  - 5.6883244754482793d+00
     &  + 2.9285842322233888d+00*HX1(-1)
     &  + 1.8851605738073271d+00*HX1(-1)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1( -1)*HX1(0)*HX2(-1,1)
     &  + HX1( -1)*HX1(0)*HX2(0,-1)
     &  + HX1( -1)*HX1(0)*HX2(0,1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX3(0,-1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX3(0,1,1)
     &  - 2.9285842322233888d+00*HX1(0)
     &  - 9.4258028690366357d-01*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(-1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX3(-1,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX3(0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX3(0,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX3(0,1,-1)
     &  + 1.8851605738073271d+00*HX2(-1,1)
     &  + 5.0000000000000000d-01*HX2(-1,1)*HX2(-1,1)
     &  - HX2( -1,1)*HX2(0,-1)
     &  - HX2( -1,1)*HX2(0,1)
     &  - 1.8851605738073271d+00*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX2(0,-1)*HX2(0,-1)
     &  + HX2(0, -1)*HX2(0,1)
     &  - 1.8851605738073271d+00*HX2(0,1)
     &  + 5.0000000000000000d-01*HX2(0,1)*HX2(0,1)
     &  - 2.0000000000000000d+00*HX4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HX4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HX4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX4(0,0,-1,1)
     &  + 4.0000000000000000d+00*HX4(0,0,0,-1)
     &  + 4.0000000000000000d+00*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HX4(0,0,1,1)
     &  - 2.0000000000000000d+00*HX4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HX4(0,1,1,-1)
      Hi5(-1,1,1,1,1) = 
     &  - 8.0212429617572516d-01
     &  - 1.6449340668482264d+00*HX1(-1)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 8.2246703342411321d-01*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(-1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  + HX1(0) *HX3(-1,1,1)
     &  - HX1(0) *HX3(0,-1,1)
     &  + HX1(0) *HX3(0,0,-1)
     &  + HX1(0) *HX3(0,0,1)
     &  - HX1(0) *HX3(0,1,1)
     &  - 1.6449340668482264d+00*HX2(-1,1)
     &  + 1.6449340668482264d+00*HX2(0,-1)
     &  + 1.6449340668482264d+00*HX2(0,1)
     &  + HX4( -1,1,1,1)
     &  - HX4(0, -1,1,1)
     &  + HX4(0,0, -1,1)
     &  - HX4(0,0,0, -1)
     &  - HX4(0,0,0,1) 
     &  + HX4(0,0,1,1) 
     &  - HX4(0,1,1,1) 
      Hi5(0,-1,-1,-1,1) = 
     &  + 5.2709719078152786d-01
     &  + 5.5504108664821579d-02*HX1(0)
     &  + 1.2011325347955035d-01*HX1(0)*HX1(0)
     &  + 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 6.9314718055994530d-01*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX3(0,-1,-1)
     &  + HX1(0) *HX3(0,0,-1)
     &  - 2.4022650695910071d-01*HX2(0,-1)
     &  + 6.9314718055994530d-01*HX3(0,-1,-1)
     &  + 6.9314718055994530d-01*HX3(0,0,-1)
     &  - HX4(0, -1,-1,-1)
     &  - HX4(0,0, -1,-1)
     &  - HX4(0,0,0, -1)
      Hi5(0,-1,-1,0,1) = 
     &  - 3.3822601053473068d-01
     &  - 1.0517997902646449d+00*HX1(0)
     &  - 4.1123351671205660d-01*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX3(0,-1,-1)
     &  + HX1(0) *HX3(0,0,-1)
     &  + 8.2246703342411321d-01*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX2(0,-1)*HX2(0,-1)
      Hi5(0,-1,-1,1,-1) = 
     &  + 3.3389529407850318d-01
     &  - 1.6651232599446473d-01*HX1(0)
     &  - 2.4022650695910071d-01*HX1(0)*HX1(0)
     &  - 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 6.9314718055994530d-01*HX1(0)*HX2(0,-1)
     &  + 4.8045301391820142d-01*HX2(0,-1)
     &  - 6.9314718055994530d-01*HX3(0,-1,-1)
     &  - 6.9314718055994530d-01*HX3(0,0,-1)
      Hi5(0,-1,-1,1,1) = 
     &  + 2.0079271680389850d+00
     &  + 2.4532465311320902d+00*HX1(0)
     &  + 9.4258028690366357d-01*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX3(0,-1,-1)
     &  - HX1(0) *HX3(0,0,-1)
     &  - 1.8851605738073271d+00*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX2(0,-1)*HX2(0,-1)
     &  - HX4(0, -1,-1,1)
     &  + HX4(0, -1,0,1)
     &  + HX4(0,0, -1,1)
     &  - HX4(0,0,0,1) 
      Hi5(0,-1,0,-1,1) = 
     &  + 3.0521251417144869d+00
     &  + 1.5335088752078636d+00*HX1(0)
     &  + 4.1123351671205660d-01*HX1(0)*HX1(0)
     &  + 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 6.9314718055994530d-01*HX1(0)*HX2(0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX3(0,0,-1)
     &  - 8.2246703342411321d-01*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX2(0,-1)*HX2(0,-1)
     &  + 1.3862943611198906d+00*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX4(0,0,-1,-1)
     &  - 4.0000000000000000d+00*HX4(0,0,0,-1)
      Hi5(0,-1,0,1,-1) = 
     &  + 1.2498035299465379d+00
     &  + 5.7009070532142637d-01*HX1(0)
     &  - 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 6.9314718055994530d-01*HX1(0)*HX2(0,-1)
     &  - 1.3862943611198906d+00*HX3(0,0,-1)
      Hi5(0,-1,0,1,1) = 
     &  - 6.0880681896251523d-01
     &  + 1.0517997902646449d+00*HX1(0)
     &  + 8.2246703342411321d-01*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX3(0,0,-1)
     &  - 1.6449340668482264d+00*HX2(0,-1)
     &  + HX4(0, -1,0,1)
     &  + 3.0000000000000000d+00*HX4(0,0,0,-1)
     &  - HX4(0,0,0,1) 
      Hi5(0,-1,1,-1,-1) = 
     &  + 1.9757838252848865d-01
     &  + 1.6651232599446473d-01*HX1(0)
     &  + 1.2011325347955035d-01*HX1(0)*HX1(0)
     &  - 2.4022650695910071d-01*HX2(0,-1)
      Hi5(0,-1,1,-1,1) = 
     &  - 5.7090374034238681d+00
     &  - 3.1962209462999012d+00*HX1(0)
     &  - 6.5146002367115732d-01*HX1(0)*HX1(0)
     &  - 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 6.9314718055994530d-01*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX3(0,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX3(0,0,1)
     &  + 1.3029200473423146d+00*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX2(0,-1)*HX2(0,-1)
     &  + 6.9314718055994530d-01*HX3(0,-1,1)
     &  - 1.3862943611198906d+00*HX3(0,0,-1)
     &  - 6.9314718055994530d-01*HX3(0,0,1)
     &  - HX4(0, -1,0,1)
     &  - HX4(0, -1,1,-1)
     &  + 2.0000000000000000d+00*HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX4(0,0,-1,1)
     &  + 4.0000000000000000d+00*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX4(0,0,0,1)
     &  + HX4(0,0,1, -1)
      Hi5(0,-1,1,0,1) = 
     &  - 5.6493805840032952d+00
     &  - 2.6736902858507163d+00*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX3(0,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX3(0,0,1)
     &  - HX4(0, -1,0,1)
     &  - 2.0000000000000000d+00*HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX4(0,0,0,1)
      Hi5(0,-1,1,1,-1) = 
     &  - 2.3933959928473851d+00
     &  - 1.7102721159642791d+00*HX1(0)
     &  - 2.9112026323250625d-01*HX1(0)*HX1(0)
     &  + 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 6.9314718055994530d-01*HX1(0)*HX2(0,-1)
     &  + 5.8224052646501250d-01*HX2(0,-1)
     &  - 6.9314718055994530d-01*HX3(0,-1,1)
     &  + 1.3862943611198906d+00*HX3(0,0,-1)
     &  + 6.9314718055994530d-01*HX3(0,0,1)
      Hi5(0,-1,1,1,1) = 
     &  + 3.2396052320380567d+00
     &  + 5.5504108664821579d-02*HX1(0)
     &  - 8.2246703342411321d-01*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX3(0,-1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX3(0,0,1)
     &  + 1.6449340668482264d+00*HX2(0,-1)
     &  - HX4(0, -1,1,1)
     &  + 2.0000000000000000d+00*HX4(0,0,-1,1)
     &  - 3.0000000000000000d+00*HX4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HX4(0,0,0,1)
     &  + HX4(0,0,1,1) 
      Hi5(0,0,-1,-1,1) = 
     &  - 1.3569495655898781d+00
     &  - 4.8170908494321862d-01*HX1(0)
     &  + 1.2011325347955035d-01*HX1(0)*HX1(0)
     &  + 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1(0) *HX3(0,0,-1)
     &  - 6.9314718055994530d-01*HX3(0,0,-1)
     &  + HX4(0,0, -1,-1)
     &  + 2.0000000000000000d+00*HX4(0,0,0,-1)
      Hi5(0,0,-1,0,1) = 
     &  - 2.8410984884917377d+00
     &  - 1.8030853547393914d+00*HX1(0)
     &  - 4.1123351671205660d-01*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1(0) *HX3(0,0,-1)
     &  + 3.0000000000000000d+00*HX4(0,0,0,-1)
      Hi5(0,0,-1,1,-1) = 
     &  - 6.2490176497326899d-01
     &  - 5.7009070532142637d-01*HX1(0)
     &  - 2.4022650695910071d-01*HX1(0)*HX1(0)
     &  - 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 6.9314718055994530d-01*HX3(0,0,-1)
      Hi5(0,0,-1,1,1) = 
     &  + 3.8055457225523666d+00
     &  + 2.7620719062289241d+00*HX1(0)
     &  + 9.4258028690366357d-01*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1(0) *HX3(0,0,-1)
     &  + HX4(0,0, -1,1)
     &  - 3.0000000000000000d+00*HX4(0,0,0,-1)
     &  - HX4(0,0,0,1) 
      Hi5(0,0,0,-1,1) = 
     &  + 9.4703282949724591d-01
     &  + 9.0154267736969571d-01*HX1(0)
     &  + 4.1123351671205660d-01*HX1(0)*HX1(0)
     &  + 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX4(0,0,0, -1)
      Hi5(0,0,0,1,-1) = 
     &  - 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
      Hi5(0,0,1,-1,-1) = 
     &  + 1.2011325347955035d-01*HX1(0)*HX1(0)
      Hi5(0,0,1,-1,1) = 
     &  - 3.5336454555719528d+00
     &  - 2.6736902858507163d+00*HX1(0)
     &  - 6.5146002367115732d-01*HX1(0)*HX1(0)
     &  - 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1(0) *HX3(0,0,1)
     &  - 6.9314718055994530d-01*HX3(0,0,1)
     &  + HX4(0,0,0, -1)
     &  + 3.0000000000000000d+00*HX4(0,0,0,1)
     &  + HX4(0,0,1, -1)
      Hi5(0,0,1,0,-1) = 
     &  + 4.1123351671205660d-01*HX1(0)*HX1(0)
      Hi5(0,0,1,1,-1) = 
     &  - 8.3320235329769199d-01
     &  - 1.1401814106428527d+00*HX1(0)
     &  - 2.9112026323250625d-01*HX1(0)*HX1(0)
     &  + 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 6.9314718055994530d-01*HX3(0,0,1)
      Hi5(0,1,-1,-1,-1) = 
     &  - 5.5504108664821579d-02*HX1(0)
      Hi5(0,1,-1,-1,1) = 
     &  + 3.4402173672056757d+00
     &  + 1.0517997902646449d+00*HX1(0)
     &  - 1.2011325347955035d-01*HX1(0)*HX1(0)
     &  - 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  - 6.9314718055994530d-01*HX1(0)*HX2(0,1)
     &  + HX1(0) *HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX3(0,0,1)
     &  + HX1(0) *HX3(0,1,-1)
     &  - 2.4022650695910071d-01*HX2(0,1)
     &  + 6.9314718055994530d-01*HX3(0,0,-1)
     &  + 1.3862943611198906d+00*HX3(0,0,1)
     &  + 6.9314718055994530d-01*HX3(0,1,-1)
     &  - HX4(0,0, -1,-1)
     &  - 2.0000000000000000d+00*HX4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HX4(0,0,0,1)
     &  - 2.0000000000000000d+00*HX4(0,0,1,-1)
     &  - HX4(0,1, -1,-1)
      Hi5(0,1,-1,1,-1) = 
     &  + 1.8532608833279382d+00
     &  + 1.0926213657706112d+00*HX1(0)
     &  + 2.4022650695910071d-01*HX1(0)*HX1(0)
     &  + 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 6.9314718055994530d-01*HX1(0)*HX2(0,1)
     &  + 4.8045301391820142d-01*HX2(0,1)
     &  - 6.9314718055994530d-01*HX3(0,0,-1)
     &  - 1.3862943611198906d+00*HX3(0,0,1)
     &  - 6.9314718055994530d-01*HX3(0,1,-1)
      Hi5(0,1,-1,1,1) = 
     &  - 5.4839540823383124d+00
     &  - 2.9285842322233888d+00*HX1(0)
     &  - 9.4258028690366357d-01*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  - HX1(0) *HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX3(0,0,1)
     &  - HX1(0) *HX3(0,1,-1)
     &  + HX2(0, -1)*HX2(0,1)
     &  - 1.8851605738073271d+00*HX2(0,1)
     &  + 5.0000000000000000d-01*HX2(0,1)*HX2(0,1)
     &  - HX4(0, -1,0,1)
     &  - 3.0000000000000000d+00*HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX4(0,0,0,-1)
     &  + 4.0000000000000000d+00*HX4(0,0,0,1)
     &  - 2.0000000000000000d+00*HX4(0,0,1,1)
     &  - HX4(0,1, -1,1)
      Hi5(0,1,0,1,-1) = 
     &  + 1.6664047065953839d+00
     &  + 6.3196619783816790d-01*HX1(0)
     &  + 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 6.9314718055994530d-01*HX1(0)*HX2(0,1)
     &  - 1.3862943611198906d+00*HX3(0,0,1)
      Hi5(0,1,1,-1,-1) = 
     &  + 3.9515676505697730d-01
     &  + 3.0882537509683393d-01*HX1(0)
     &  - 1.2011325347955035d-01*HX1(0)*HX1(0)
     &  - 2.4022650695910071d-01*HX2(0,1)
      Hi5(0,1,1,-1,1) = 
     &  + 1.3317828347103277d+00
     &  + 2.2701119065237547d+00*HX1(0)
     &  + 6.5146002367115732d-01*HX1(0)*HX1(0)
     &  + 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  + 6.9314718055994530d-01*HX1(0)*HX2(0,1)
     &  - HX1(0) *HX3(0,0,1)
     &  + HX1(0) *HX3(0,1,1)
     &  - HX2(0, -1)*HX2(0,1)
     &  + 1.3029200473423146d+00*HX2(0,1)
     &  - 5.0000000000000000d-01*HX2(0,1)*HX2(0,1)
     &  - 6.9314718055994530d-01*HX3(0,0,1)
     &  + 6.9314718055994530d-01*HX3(0,1,1)
     &  + HX4(0, -1,0,1)
     &  + 2.0000000000000000d+00*HX4(0,0,-1,1)
     &  - HX4(0,0,0, -1)
     &  + HX4(0,0,1, -1)
     &  - HX4(0,1,1, -1)
      Hi5(0,1,1,1,-1) = 
     &  - 9.5774727708194543d-01
     &  + 6.0296821703481255d-01*HX1(0)
     &  + 2.9112026323250625d-01*HX1(0)*HX1(0)
     &  - 1.1552453009332421d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 6.9314718055994530d-01*HX1(0)*HX2(0,1)
     &  + 5.8224052646501250d-01*HX2(0,1)
     &  + 6.9314718055994530d-01*HX3(0,0,1)
     &  - 6.9314718055994530d-01*HX3(0,1,1)
      endif 
** nw > 4 endif 
      if ( nw.gt.5 ) then 
* 2017-05-05:11:53:59.hpl 
* <- x_to_y.real.out 
* produced by form-to-fortr for gehrt@tintin  
      HY6(-1,-1,-1,-1,-1,1) = 
     &  + 2.0820845269881513d+00
     &  - 2.1900870176160439d+00*HX1(-1)
     &  + 1.2139314033773515d+00*HX1(-1)*HX1(-1)
     &  - 4.6034531770482068d-01*HX1(-1)*HX1(-1)*HX1(-1)
     &  + 1.0280837917801415d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX1(-1)
     &  + 8.3333333333333333d-03*HX1(-1)*HX1(-1)*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)
     &  - 2.0833333333333333d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(-1)*HX2(0,-1)
     &  - 4.1666666666666666d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(-1)*HX2(0,1)
     &  - 4.1123351671205660d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)
     &  + 2.7777777777777777d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX3(0,-1,-1)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX3(0,0,-1)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX3(0,0,1)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX3(0,1,-1)
     &  + 1.3810359531144620d+00*HX1(-1)*HX1(-1)*HX1(0)
     &  + 6.1685027506808491d-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  - 2.0833333333333333d-02*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,0,0,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,0,1,-1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,1,-1,-1)
     &  - 2.4278628067547031d+00*HX1(-1)*HX1(0)
     &  - 1.3810359531144620d+00*HX1(-1)*HX1(0)*HX1(0)
     &  - 4.1123351671205660d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1( -1)*HX5(0,-1,-1,-1,-1)
     &  + HX1( -1)*HX5(0,0,-1,-1,-1)
     &  + HX1( -1)*HX5(0,0,0,-1,-1)
     &  + HX1( -1)*HX5(0,0,0,0,-1)
     &  + HX1( -1)*HX5(0,0,0,0,1)
     &  + HX1( -1)*HX5(0,0,0,1,-1)
     &  + HX1( -1)*HX5(0,0,1,-1,-1)
     &  + HX1( -1)*HX5(0,1,-1,-1,-1)
     &  + 2.1900870176160439d+00*HX1(0)
     &  + 1.2139314033773515d+00*HX1(0)*HX1(0)
     &  + 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX6( -1,-1,-1,-1,-1,1)
     &  - HX6(0, -1,-1,-1,-1,-1)
     &  - HX6(0,0, -1,-1,-1,-1)
     &  - HX6(0,0,0, -1,-1,-1)
     &  - HX6(0,0,0,0, -1,-1)
     &  - HX6(0,0,0,0,0, -1)
     &  - HX6(0,0,0,0,0,1) 
     &  - HX6(0,0,0,0,1, -1)
     &  - HX6(0,0,0,1, -1,-1)
     &  - HX6(0,0,1, -1,-1,-1)
     &  - HX6(0,1, -1,-1,-1,-1)
      HY6(-1,-1,-1,-1,1,1) = 
     &  + 8.0070610407491173d+00
     &  - 5.5581622138319701d+00*HX1(-1)
     &  + 1.0146780316041920d+00*HX1(-1)*HX1(-1)
     &  + 4.6034531770482068d-01*HX1(-1)*HX1(-1)*HX1(-1)
     &  - 2.0561675835602830d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX1(-1)
     &  + 2.0833333333333333d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  + 8.2246703342411321d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)
     &  - 2.7777777777777777d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)*HX2(0,1)
     &  - 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX3(0,-1,1)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX3(0,0,-1)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX3(0,0,1)
     &  - 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX3(0,1,1)
     &  - 1.3810359531144620d+00*HX1(-1)*HX1(-1)*HX1(0)
     &  - 1.2337005501361698d+00*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  + 2.0833333333333333d-02*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX3(0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX3(0,1,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,0,-1,1)
     &  - HX1( -1)*HX1(-1)*HX4(0,0,0,-1)
     &  - HX1( -1)*HX1(-1)*HX4(0,0,0,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,0,1,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,0,1,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,1,-1,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,1,1,-1)
     &  - 2.0293560632083841d+00*HX1(-1)*HX1(0)
     &  + 1.3810359531144620d+00*HX1(-1)*HX1(0)*HX1(0)
     &  + 8.2246703342411321d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1( -1)*HX1(0)*HX4(0,-1,-1,-1)
     &  - HX1( -1)*HX1(0)*HX4(0,0,-1,-1)
     &  - HX1( -1)*HX1(0)*HX4(0,0,0,-1)
     &  - HX1( -1)*HX1(0)*HX4(0,0,0,1)
     &  - HX1( -1)*HX1(0)*HX4(0,0,1,-1)
     &  - HX1( -1)*HX1(0)*HX4(0,1,-1,-1)
     &  - HX1( -1)*HX5(0,-1,-1,-1,1)
     &  - HX1( -1)*HX5(0,-1,-1,1,-1)
     &  - HX1( -1)*HX5(0,-1,1,-1,-1)
     &  + HX1( -1)*HX5(0,0,-1,-1,-1)
     &  - HX1( -1)*HX5(0,0,-1,-1,1)
     &  - HX1( -1)*HX5(0,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX5(0,0,0,-1,-1)
     &  - HX1( -1)*HX5(0,0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX5(0,0,0,0,1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX5(0,0,0,1,-1)
     &  - HX1( -1)*HX5(0,0,0,1,1)
     &  + HX1( -1)*HX5(0,0,1,-1,-1)
     &  - HX1( -1)*HX5(0,0,1,-1,1)
     &  - HX1( -1)*HX5(0,0,1,1,-1)
     &  - HX1( -1)*HX5(0,1,-1,-1,1)
     &  - HX1( -1)*HX5(0,1,-1,1,-1)
     &  - HX1( -1)*HX5(0,1,1,-1,-1)
     &  + 5.5581622138319701d+00*HX1(0)
     &  + 1.0146780316041920d+00*HX1(0)*HX1(0)
     &  - 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1(0) *HX5(-1,-1,-1,-1,1)
     &  + HX1(0) *HX5(0,-1,-1,-1,-1)
     &  + HX1(0) *HX5(0,0,-1,-1,-1)
     &  + HX1(0) *HX5(0,0,0,-1,-1)
     &  + HX1(0) *HX5(0,0,0,0,-1)
     &  + HX1(0) *HX5(0,0,0,0,1)
     &  + HX1(0) *HX5(0,0,0,1,-1)
     &  + HX1(0) *HX5(0,0,1,-1,-1)
     &  + HX1(0) *HX5(0,1,-1,-1,-1)
     &  + HX6( -1,-1,-1,-1,1,1)
     &  + HX6(0, -1,-1,-1,-1,1)
     &  + HX6(0, -1,-1,-1,1,-1)
     &  + HX6(0, -1,-1,1,-1,-1)
     &  + HX6(0, -1,1,-1,-1,-1)
     &  - HX6(0,0, -1,-1,-1,-1)
     &  + HX6(0,0, -1,-1,-1,1)
     &  + HX6(0,0, -1,-1,1,-1)
     &  + HX6(0,0, -1,1,-1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,-1,-1)
     &  + HX6(0,0,0, -1,-1,1)
     &  + HX6(0,0,0, -1,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + HX6(0,0,0,0, -1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + HX6(0,0,0,0,1,1) 
     &  - 2.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  + HX6(0,0,0,1, -1,1)
     &  + HX6(0,0,0,1,1, -1)
     &  - HX6(0,0,1, -1,-1,-1)
     &  + HX6(0,0,1, -1,-1,1)
     &  + HX6(0,0,1, -1,1,-1)
     &  + HX6(0,0,1,1, -1,-1)
     &  + HX6(0,1, -1,-1,-1,1)
     &  + HX6(0,1, -1,-1,1,-1)
     &  + HX6(0,1, -1,1,-1,-1)
     &  + HX6(0,1,1, -1,-1,-1)
      HY6(-1,-1,-1,1,-1,1) = 
     &  - 1.7379190730740364d+01
     &  + 9.2329419831013177d+00*HX1(-1)
     &  - 1.6928093109730279d+00*HX1(-1)*HX1(-1)
     &  + 2.1949077523321137d-01*HX1(-1)*HX1(-1)*HX1(-1)
     &  + 4.1123351671205660d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)
     &  - 2.7777777777777777d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)*HX2(0,1)
     &  + 3.3333333333333333d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX3(0,-1,1)
     &  - 3.3333333333333333d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX3(0,0,-1)
     &  - 3.3333333333333333d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX3(0,0,1)
     &  + 3.3333333333333333d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX3(0,1,1)
     &  - 6.5847232569963413d-01*HX1(-1)*HX1(-1)*HX1(0)
     &  - 6.1685027506808491d-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  + 2.0833333333333333d-02*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 2.5000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 2.5000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX2(0,1)
     &  - HX1( -1)*HX1(-1)*HX1(0)*HX3(0,-1,-1)
     &  - HX1( -1)*HX1(-1)*HX1(0)*HX3(0,0,-1)
     &  - HX1( -1)*HX1(-1)*HX1(0)*HX3(0,0,1)
     &  - HX1( -1)*HX1(-1)*HX1(0)*HX3(0,1,-1)
     &  - 1.2337005501361698d+00*HX1(-1)*HX1(-1)*HX2(0,-1)
     &  + 2.5000000000000000d-01*HX1(-1)*HX1(-1)*HX2(0,-1)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX2(0,-1)*HX2(0,1)
     &  - 1.2337005501361698d+00*HX1(-1)*HX1(-1)*HX2(0,1)
     &  + 2.5000000000000000d-01*HX1(-1)*HX1(-1)*HX2(0,1)*HX2(0,1)
     &  - HX1( -1)*HX1(-1)*HX4(0,-1,-1,1)
     &  - HX1( -1)*HX1(-1)*HX4(0,-1,1,-1)
     &  + HX1( -1)*HX1(-1)*HX4(0,0,-1,-1)
     &  - HX1( -1)*HX1(-1)*HX4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX1(-1)*HX4(0,0,0,-1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX1(-1)*HX4(0,0,0,1)
     &  + HX1( -1)*HX1(-1)*HX4(0,0,1,-1)
     &  - HX1( -1)*HX1(-1)*HX4(0,0,1,1)
     &  - HX1( -1)*HX1(-1)*HX4(0,1,-1,1)
     &  - HX1( -1)*HX1(-1)*HX4(0,1,1,-1)
     &  + 3.3856186219460558d+00*HX1(-1)*HX1(0)
     &  + 6.5847232569963413d-01*HX1(-1)*HX1(0)*HX1(0)
     &  + 4.1123351671205660d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  + HX1( -1)*HX1(0)*HX4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,0,0,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,1,-1,-1)
     &  - HX1( -1)*HX2(0,-1)*HX3(0,-1,-1)
     &  - HX1( -1)*HX2(0,-1)*HX3(0,1,-1)
     &  - HX1( -1)*HX2(0,1)*HX3(0,1,-1)
     &  + 2.4674011002723396d+00*HX1(-1)*HX3(0,-1,-1)
     &  + 2.4674011002723396d+00*HX1(-1)*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX1(-1)*HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX1(-1)*HX3(0,1,-1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX5(0,-1,-1,-1,1)
     &  - HX1( -1)*HX5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX5(0,-1,-1,1,-1)
     &  + HX1( -1)*HX5(0,-1,0,-1,-1)
     &  - HX1( -1)*HX5(0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX5(0,-1,1,-1,-1)
     &  + HX1( -1)*HX5(0,0,-1,-1,1)
     &  - HX1( -1)*HX5(0,0,-1,0,-1)
     &  - HX1( -1)*HX5(0,0,-1,0,1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX5(0,0,-1,1,-1)
     &  - 7.0000000000000000d+00*HX1(-1)*HX5(0,0,0,-1,-1)
     &  - 7.0000000000000000d+00*HX1(-1)*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX1(-1)*HX5(0,0,0,0,1)
     &  - 7.0000000000000000d+00*HX1(-1)*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX5(0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX5(0,0,1,-1,1)
     &  - HX1( -1)*HX5(0,0,1,0,-1)
     &  - HX1( -1)*HX5(0,0,1,0,1)
     &  + 4.0000000000000000d+00*HX1(-1)*HX5(0,0,1,1,-1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX5(0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX5(0,1,-1,1,-1)
     &  + HX1( -1)*HX5(0,1,0,1,-1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX5(0,1,1,-1,-1)
     &  - 9.2329419831013177d+00*HX1(0)
     &  - 1.6928093109730279d+00*HX1(0)*HX1(0)
     &  - 2.1949077523321137d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(-1,-1,-1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,-1,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,1,-1,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,1,-1,-1,-1)
     &  - HX2(0, -1)*HX4(-1,-1,-1,1)
     &  + HX2(0, -1)*HX4(0,-1,-1,-1)
     &  + HX2(0, -1)*HX4(0,1,-1,-1)
     &  - HX2(0,1) *HX4(-1,-1,-1,1)
     &  + HX2(0,1) *HX4(0,1,-1,-1)
     &  + 2.4674011002723396d+00*HX4(-1,-1,-1,1)
     &  - 2.4674011002723396d+00*HX4(0,-1,-1,-1)
     &  - 2.4674011002723396d+00*HX4(0,0,-1,-1)
     &  - 2.4674011002723396d+00*HX4(0,0,0,-1)
     &  - 2.4674011002723396d+00*HX4(0,0,0,1)
     &  - 2.4674011002723396d+00*HX4(0,0,1,-1)
     &  - 2.4674011002723396d+00*HX4(0,1,-1,-1)
     &  + HX6( -1,-1,-1,1,-1,1)
     &  - 4.0000000000000000d+00*HX6(0,-1,-1,-1,-1,1)
     &  + HX6(0, -1,-1,-1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,-1,-1,-1,1,-1)
     &  + HX6(0, -1,-1,0,-1,1)
     &  + HX6(0, -1,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,-1,-1,1,-1,-1)
     &  - HX6(0, -1,0,-1,-1,-1)
     &  + HX6(0, -1,0,-1,-1,1)
     &  + HX6(0, -1,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,-1,1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,-1,-1,1)
     &  + HX6(0,0, -1,-1,0,-1)
     &  + HX6(0,0, -1,-1,0,1)
     &  - HX6(0,0, -1,-1,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,-1,-1)
     &  + HX6(0,0, -1,0,-1,1)
     &  + HX6(0,0, -1,0,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,1,-1,-1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,-1,-1,-1)
     &  - HX6(0,0,0, -1,-1,1)
     &  + HX6(0,0,0, -1,0,-1)
     &  + HX6(0,0,0, -1,0,1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,-1,-1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,1,-1,-1)
     &  - HX6(0,0,0,1, -1,1)
     &  + HX6(0,0,0,1,0, -1)
     &  + HX6(0,0,0,1,0,1) 
     &  + 2.0000000000000000d+00*HX6(0,0,1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,1,-1,-1,1)
     &  + HX6(0,0,1, -1,0,-1)
     &  + HX6(0,0,1, -1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,1,-1,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,-1,-1)
     &  + HX6(0,0,1,0, -1,1)
     &  + HX6(0,0,1,0,1, -1)
     &  - 4.0000000000000000d+00*HX6(0,0,1,1,-1,-1)
     &  - 4.0000000000000000d+00*HX6(0,1,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,1,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,1,-1,1,-1,-1)
     &  - HX6(0,1,0,1, -1,-1)
     &  - 2.0000000000000000d+00*HX6(0,1,1,-1,-1,-1)
      HY6(-1,-1,-1,1,1,1) = 
     &  + 6.6763138442729474d-01
     &  + 5.5581622138319701d+00*HX1(-1)
     &  - 3.2432874665857356d+00*HX1(-1)*HX1(-1)
     &  - 8.2246703342411321d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)
     &  + 2.7777777777777777d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.2337005501361698d+00*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  - 2.0833333333333333d-02*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 2.5000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 2.5000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX3(0,-1,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX3(0,1,1)
     &  + 2.4674011002723396d+00*HX1(-1)*HX1(-1)*HX2(0,-1)
     &  + 2.4674011002723396d+00*HX1(-1)*HX1(-1)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,-1,1,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,0,-1,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,0,0,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,0,1,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX4(0,1,1,1)
     &  + 6.4865749331714713d+00*HX1(-1)*HX1(0)
     &  - 8.2246703342411321d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  + HX1( -1)*HX1(0)*HX4(0,-1,-1,1)
     &  + HX1( -1)*HX1(0)*HX4(0,-1,1,-1)
     &  - HX1( -1)*HX1(0)*HX4(0,0,-1,-1)
     &  + HX1( -1)*HX1(0)*HX4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,0,0,1)
     &  - HX1( -1)*HX1(0)*HX4(0,0,1,-1)
     &  + HX1( -1)*HX1(0)*HX4(0,0,1,1)
     &  + HX1( -1)*HX1(0)*HX4(0,1,-1,1)
     &  + HX1( -1)*HX1(0)*HX4(0,1,1,-1)
     &  - 4.9348022005446793d+00*HX1(-1)*HX3(0,-1,-1)
     &  - 4.9348022005446793d+00*HX1(-1)*HX3(0,0,-1)
     &  - 4.9348022005446793d+00*HX1(-1)*HX3(0,0,1)
     &  - 4.9348022005446793d+00*HX1(-1)*HX3(0,1,-1)
     &  + HX1( -1)*HX5(0,-1,-1,1,1)
     &  + HX1( -1)*HX5(0,-1,1,-1,1)
     &  + HX1( -1)*HX5(0,-1,1,1,-1)
     &  - HX1( -1)*HX5(0,0,-1,-1,1)
     &  - HX1( -1)*HX5(0,0,-1,1,-1)
     &  + HX1( -1)*HX5(0,0,-1,1,1)
     &  + HX1( -1)*HX5(0,0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX5(0,0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX5(0,0,0,0,1)
     &  + HX1( -1)*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX5(0,0,0,1,1)
     &  - HX1( -1)*HX5(0,0,1,-1,1)
     &  - HX1( -1)*HX5(0,0,1,1,-1)
     &  + HX1( -1)*HX5(0,0,1,1,1)
     &  + HX1( -1)*HX5(0,1,-1,1,1)
     &  + HX1( -1)*HX5(0,1,1,-1,1)
     &  + HX1( -1)*HX5(0,1,1,1,-1)
     &  - 5.5581622138319701d+00*HX1(0)
     &  - 3.2432874665857356d+00*HX1(0)*HX1(0)
     &  + 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(-1,-1,-1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,-1,-1)
     &  + HX1(0) *HX5(-1,-1,-1,1,1)
     &  - HX1(0) *HX5(0,-1,-1,-1,1)
     &  - HX1(0) *HX5(0,-1,-1,1,-1)
     &  - HX1(0) *HX5(0,-1,1,-1,-1)
     &  + HX1(0) *HX5(0,0,-1,-1,-1)
     &  - HX1(0) *HX5(0,0,-1,-1,1)
     &  - HX1(0) *HX5(0,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  - HX1(0) *HX5(0,0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - HX1(0) *HX5(0,0,0,1,1)
     &  + HX1(0) *HX5(0,0,1,-1,-1)
     &  - HX1(0) *HX5(0,0,1,-1,1)
     &  - HX1(0) *HX5(0,0,1,1,-1)
     &  - HX1(0) *HX5(0,1,-1,-1,1)
     &  - HX1(0) *HX5(0,1,-1,1,-1)
     &  - HX1(0) *HX5(0,1,1,-1,-1)
     &  - 4.9348022005446793d+00*HX4(-1,-1,-1,1)
     &  + 4.9348022005446793d+00*HX4(0,-1,-1,-1)
     &  + 4.9348022005446793d+00*HX4(0,0,-1,-1)
     &  + 4.9348022005446793d+00*HX4(0,0,0,-1)
     &  + 4.9348022005446793d+00*HX4(0,0,0,1)
     &  + 4.9348022005446793d+00*HX4(0,0,1,-1)
     &  + 4.9348022005446793d+00*HX4(0,1,-1,-1)
     &  + HX6( -1,-1,-1,1,1,1)
     &  - HX6(0, -1,-1,-1,1,1)
     &  - HX6(0, -1,-1,1,-1,1)
     &  - HX6(0, -1,-1,1,1,-1)
     &  - HX6(0, -1,1,-1,-1,1)
     &  - HX6(0, -1,1,-1,1,-1)
     &  - HX6(0, -1,1,1,-1,-1)
     &  + HX6(0,0, -1,-1,-1,1)
     &  + HX6(0,0, -1,-1,1,-1)
     &  - HX6(0,0, -1,-1,1,1)
     &  + HX6(0,0, -1,1,-1,-1)
     &  - HX6(0,0, -1,1,-1,1)
     &  - HX6(0,0, -1,1,1,-1)
     &  - HX6(0,0,0, -1,-1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  - HX6(0,0,0, -1,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - HX6(0,0,0,1, -1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  - HX6(0,0,0,1,1,1) 
     &  + HX6(0,0,1, -1,-1,1)
     &  + HX6(0,0,1, -1,1,-1)
     &  - HX6(0,0,1, -1,1,1)
     &  + HX6(0,0,1,1, -1,-1)
     &  - HX6(0,0,1,1, -1,1)
     &  - HX6(0,0,1,1,1, -1)
     &  - HX6(0,1, -1,-1,1,1)
     &  - HX6(0,1, -1,1,-1,1)
     &  - HX6(0,1, -1,1,1,-1)
     &  - HX6(0,1,1, -1,-1,1)
     &  - HX6(0,1,1, -1,1,-1)
     &  - HX6(0,1,1,1, -1,-1)
      HY6(-1,-1,1,-1,1,1) = 
     &  - 3.4316706112749336d+00
     &  - 7.7439193717015657d-01*HX1(-1)
     &  + 4.2696785175273867d-01*HX1(-1)*HX1(-1)
     &  + 1.3810359531144620d+00*HX1(-1)*HX1(-1)*HX1(0)
     &  + 1.2337005501361698d+00*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  - 2.0833333333333333d-02*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 2.5000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 2.5000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX2(0,1)
     &  + HX1( -1)*HX1(-1)*HX1(0)*HX3(0,-1,1)
     &  - HX1( -1)*HX1(-1)*HX1(0)*HX3(0,0,-1)
     &  - HX1( -1)*HX1(-1)*HX1(0)*HX3(0,0,1)
     &  + HX1( -1)*HX1(-1)*HX1(0)*HX3(0,1,1)
     &  - 2.4674011002723396d+00*HX1(-1)*HX1(-1)*HX2(0,-1)
     &  - 2.4674011002723396d+00*HX1(-1)*HX1(-1)*HX2(0,1)
     &  + 1.5000000000000000d+00*HX1(-1)*HX1(-1)*HX4(0,-1,1,1)
     &  - 1.5000000000000000d+00*HX1(-1)*HX1(-1)*HX4(0,0,-1,1)
     &  + 1.5000000000000000d+00*HX1(-1)*HX1(-1)*HX4(0,0,0,-1)
     &  + 1.5000000000000000d+00*HX1(-1)*HX1(-1)*HX4(0,0,0,1)
     &  - 1.5000000000000000d+00*HX1(-1)*HX1(-1)*HX4(0,0,1,1)
     &  + 1.5000000000000000d+00*HX1(-1)*HX1(-1)*HX4(0,1,1,1)
     &  - 8.5393570350547734d-01*HX1(-1)*HX1(0)
     &  - 1.3810359531144620d+00*HX1(-1)*HX1(0)*HX1(0)
     &  - 8.2246703342411321d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX3(-1,-1,1)
     &  - HX1( -1)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - HX1( -1)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - HX1( -1)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - HX1( -1)*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  - 4.9348022005446793d+00*HX1(-1)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + HX1( -1)*HX1(0)*HX2(0,-1)*HX2(0,1)
     &  - 4.9348022005446793d+00*HX1(-1)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX2(0,1)*HX2(0,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,0,-1,1)
     &  + 4.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,0,0,-1)
     &  + 4.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,0,1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,1,1,-1)
     &  - 2.7620719062289241d+00*HX1(-1)*HX2(0,-1)
     &  - HX1( -1)*HX2(0,-1)*HX3(0,0,-1)
     &  - HX1( -1)*HX2(0,-1)*HX3(0,0,1)
     &  - 2.7620719062289241d+00*HX1(-1)*HX2(0,1)
     &  + HX1( -1)*HX2(0,1)*HX3(0,-1,1)
     &  - HX1( -1)*HX2(0,1)*HX3(0,0,-1)
     &  - HX1( -1)*HX2(0,1)*HX3(0,0,1)
     &  - 4.9348022005446793d+00*HX1(-1)*HX3(-1,-1,1)
     &  + 9.8696044010893586d+00*HX1(-1)*HX3(0,-1,-1)
     &  + 9.8696044010893586d+00*HX1(-1)*HX3(0,0,-1)
     &  + 9.8696044010893586d+00*HX1(-1)*HX3(0,0,1)
     &  + 9.8696044010893586d+00*HX1(-1)*HX3(0,1,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX5(0,-1,-1,1,1)
     &  + HX1( -1)*HX5(0,-1,0,-1,1)
     &  - HX1( -1)*HX5(0,-1,0,1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX5(0,-1,1,-1,1)
     &  - HX1( -1)*HX5(0,-1,1,0,1)
     &  - 3.0000000000000000d+00*HX1(-1)*HX5(0,-1,1,1,-1)
     &  + 4.0000000000000000d+00*HX1(-1)*HX5(0,0,-1,-1,1)
     &  + HX1( -1)*HX5(0,0,-1,0,-1)
     &  + HX1( -1)*HX5(0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX5(0,0,-1,1,-1)
     &  - 4.0000000000000000d+00*HX1(-1)*HX5(0,0,-1,1,1)
     &  + 7.0000000000000000d+00*HX1(-1)*HX5(0,0,0,-1,1)
     &  - 7.0000000000000000d+00*HX1(-1)*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX1(-1)*HX5(0,0,0,0,1)
     &  + 7.0000000000000000d+00*HX1(-1)*HX5(0,0,0,1,1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX5(0,0,1,-1,1)
     &  + HX1( -1)*HX5(0,0,1,0,-1)
     &  + HX1( -1)*HX5(0,0,1,0,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX5(0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX5(0,1,-1,1,1)
     &  + HX1( -1)*HX5(0,1,0,1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX5(0,1,1,-1,1)
     &  - 3.0000000000000000d+00*HX1(-1)*HX5(0,1,1,1,-1)
     &  + 7.7439193717015657d-01*HX1(0)
     &  + 4.2696785175273867d-01*HX1(0)*HX1(0)
     &  + 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(-1,-1,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(-1,-1,-1,1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,-1,-1,-1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,1,-1,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(-1,-1,1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,-1,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,1,-1)
     &  - HX1(0) *HX2(0,1)*HX3(-1,-1,1)
     &  - HX1(0) *HX2(0,1)*HX3(0,1,-1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(-1,-1,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,-1,-1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,1,-1)
     &  + HX1(0) *HX5(-1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,-1,-1,-1,1)
     &  - HX1(0) *HX5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,-1,-1,1,-1)
     &  + HX1(0) *HX5(0,-1,0,-1,-1)
     &  - HX1(0) *HX5(0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,-1,1,-1,-1)
     &  + HX1(0) *HX5(0,0,-1,-1,1)
     &  - HX1(0) *HX5(0,0,-1,0,-1)
     &  - HX1(0) *HX5(0,0,-1,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,-1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,1,-1,1)
     &  - HX1(0) *HX5(0,0,1,0,-1)
     &  - HX1(0) *HX5(0,0,1,0,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,1,1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,1,-1,1,-1)
     &  + HX1(0) *HX5(0,1,0,1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,1,1,-1,-1)
     &  - HX2(0,1) *HX4(0,-1,1,1)
     &  + 2.7620719062289241d+00*HX3(-1,-1,1)
     &  - HX3( -1,-1,1)*HX3(0,-1,1)
     &  + HX3( -1,-1,1)*HX3(0,0,-1)
     &  + HX3( -1,-1,1)*HX3(0,0,1)
     &  - HX3( -1,-1,1)*HX3(0,1,1)
     &  + 2.7620719062289241d+00*HX3(0,-1,-1)
     &  + HX3(0, -1,-1)*HX3(0,0,-1)
     &  + HX3(0, -1,-1)*HX3(0,0,1)
     &  - HX3(0, -1,1)*HX3(0,1,-1)
     &  + 2.7620719062289241d+00*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  + HX3(0,0, -1)*HX3(0,0,1)
     &  + HX3(0,0, -1)*HX3(0,1,-1)
     &  + 2.7620719062289241d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  + HX3(0,0,1) *HX3(0,1,-1)
     &  + 2.7620719062289241d+00*HX3(0,1,-1)
     &  + 1.4804406601634037d+01*HX4(-1,-1,-1,1)
     &  - 1.4804406601634037d+01*HX4(0,-1,-1,-1)
     &  - 1.4804406601634037d+01*HX4(0,0,-1,-1)
     &  - 1.4804406601634037d+01*HX4(0,0,0,-1)
     &  - 1.4804406601634037d+01*HX4(0,0,0,1)
     &  - 1.4804406601634037d+01*HX4(0,0,1,-1)
     &  - 1.4804406601634037d+01*HX4(0,1,-1,-1)
     &  + HX6( -1,-1,1,-1,1,1)
     &  + 3.0000000000000000d+00*HX6(0,-1,-1,-1,1,1)
     &  - HX6(0, -1,-1,0,-1,1)
     &  - HX6(0, -1,-1,0,1,1)
     &  + 2.0000000000000000d+00*HX6(0,-1,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,-1,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,-1,0,-1,-1,1)
     &  - HX6(0, -1,0,-1,1,-1)
     &  - HX6(0, -1,0,-1,1,1)
     &  + HX6(0, -1,0,1,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,-1,0,1,1,1)
     &  + 2.0000000000000000d+00*HX6(0,-1,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,-1,1,-1,1,-1)
     &  + HX6(0, -1,1,0,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,-1,1,0,1,1)
     &  + 3.0000000000000000d+00*HX6(0,-1,1,1,-1,-1)
     &  + HX6(0, -1,1,1,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,-1,-1,1,-1)
     &  + HX6(0,0, -1,-1,1,1)
     &  - HX6(0,0, -1,0,-1,-1)
     &  - HX6(0,0, -1,0,-1,1)
     &  - HX6(0,0, -1,0,1,-1)
     &  - HX6(0,0, -1,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,1,-1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,1,-1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,1,1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,-1,1,1,1)
     &  - 7.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  - 7.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  + 7.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - 7.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 1.4000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  + 1.4000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 7.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - 7.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - 7.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - 7.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,-1,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,1,-1,1,1)
     &  - HX6(0,0,1,0, -1,-1)
     &  - HX6(0,0,1,0, -1,1)
     &  - HX6(0,0,1,0,1, -1)
     &  - HX6(0,0,1,0,1,1) 
     &  - 3.0000000000000000d+00*HX6(0,0,1,1,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,1,-1,-1,1,1)
     &  - HX6(0,1, -1,0,1,1)
     &  + 2.0000000000000000d+00*HX6(0,1,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,1,-1,1,1,-1)
     &  - HX6(0,1,0,1, -1,1)
     &  - HX6(0,1,0,1,1, -1)
     &  + 2.0000000000000000d+00*HX6(0,1,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,1,1,-1,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,1,1,1,-1,-1)
      HY6(-1,-1,1,1,-1,1) = 
     &  + 1.2222988460506645d+01
     &  - 1.1314107957029216d+01*HX1(-1)
     &  + 2.7878585066265773d+00*HX1(-1)*HX1(-1)
     &  + 6.5847232569963413d-01*HX1(-1)*HX1(-1)*HX1(0)
     &  + 6.1685027506808491d-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  - 2.0833333333333333d-02*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX3(0,-1,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX3(0,1,1)
     &  - 1.5000000000000000d+00*HX1(-1)*HX1(-1)*HX4(0,-1,1,1)
     &  + 1.5000000000000000d+00*HX1(-1)*HX1(-1)*HX4(0,0,-1,1)
     &  - 1.5000000000000000d+00*HX1(-1)*HX1(-1)*HX4(0,0,0,-1)
     &  - 1.5000000000000000d+00*HX1(-1)*HX1(-1)*HX4(0,0,0,1)
     &  + 1.5000000000000000d+00*HX1(-1)*HX1(-1)*HX4(0,0,1,1)
     &  - 1.5000000000000000d+00*HX1(-1)*HX1(-1)*HX4(0,1,1,1)
     &  - 5.5757170132531547d+00*HX1(-1)*HX1(0)
     &  - 6.5847232569963413d-01*HX1(-1)*HX1(0)*HX1(0)
     &  - 4.1123351671205660d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX3(0,1,1)
     &  - 2.4674011002723396d+00*HX1(-1)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  - HX1( -1)*HX1(0)*HX2(0,-1)*HX2(0,1)
     &  - 2.4674011002723396d+00*HX1(-1)*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX2(0,1)*HX2(0,1)
     &  + HX1( -1)*HX1(0)*HX4(-1,-1,1,1)
     &  - 1.3169446513992682d+00*HX1(-1)*HX2(0,-1)
     &  + HX1( -1)*HX2(0,-1)*HX3(0,-1,1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX2(0,-1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX2(0,-1)*HX3(0,0,1)
     &  + HX1( -1)*HX2(0,-1)*HX3(0,1,1)
     &  - 1.3169446513992682d+00*HX1(-1)*HX2(0,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX2(0,1)*HX3(0,-1,1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX2(0,1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX2(0,1)*HX3(0,0,1)
     &  + HX1( -1)*HX2(0,1)*HX3(0,1,1)
     &  - 2.4674011002723396d+00*HX1(-1)*HX3(0,-1,1)
     &  + 2.4674011002723396d+00*HX1(-1)*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX1(-1)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX1(-1)*HX3(0,1,1)
     &  - 3.0000000000000000d+00*HX1(-1)*HX5(0,-1,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX5(0,-1,0,1,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX5(0,-1,1,0,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX5(0,-1,1,1,-1)
     &  - 6.0000000000000000d+00*HX1(-1)*HX5(0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HX1(-1)*HX5(0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HX1(-1)*HX5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX1(-1)*HX5(0,0,-1,1,-1)
     &  + 6.0000000000000000d+00*HX1(-1)*HX5(0,0,-1,1,1)
     &  - 6.0000000000000000d+00*HX1(-1)*HX5(0,0,0,-1,-1)
     &  - 9.0000000000000000d+00*HX1(-1)*HX5(0,0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX5(0,0,0,0,1)
     &  - 6.0000000000000000d+00*HX1(-1)*HX5(0,0,0,1,-1)
     &  - 9.0000000000000000d+00*HX1(-1)*HX5(0,0,0,1,1)
     &  - 3.0000000000000000d+00*HX1(-1)*HX5(0,0,1,0,-1)
     &  - 3.0000000000000000d+00*HX1(-1)*HX5(0,0,1,0,1)
     &  - 3.0000000000000000d+00*HX1(-1)*HX5(0,0,1,1,-1)
     &  - 6.0000000000000000d+00*HX1(-1)*HX5(0,0,1,1,1)
     &  - 3.0000000000000000d+00*HX1(-1)*HX5(0,1,0,1,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX5(0,1,1,1,-1)
     &  + 1.1314107957029216d+01*HX1(0)
     &  + 2.7878585066265773d+00*HX1(0)*HX1(0)
     &  + 2.1949077523321137d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(-1,-1,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(-1,-1,1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,1)
     &  + HX1(0) *HX1(0)*HX4(0,0,0,-1)
     &  + HX1(0) *HX1(0)*HX4(0,0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,-1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,1,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(-1,-1,1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,-1,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,1,-1)
     &  + HX1(0) *HX2(0,1)*HX3(-1,-1,1)
     &  + HX1(0) *HX2(0,1)*HX3(0,1,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(-1,-1,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,-1,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,1,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(-1,-1,-1,1,1)
     &  - HX1(0) *HX5(-1,-1,1,-1,1)
     &  + HX1(0) *HX5(0,-1,-1,0,1)
     &  + HX1(0) *HX5(0,-1,-1,1,-1)
     &  - HX1(0) *HX5(0,-1,0,-1,-1)
     &  + HX1(0) *HX5(0,-1,0,-1,1)
     &  + HX1(0) *HX5(0,-1,1,-1,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,1)
     &  + HX1(0) *HX5(0,0,-1,0,-1)
     &  + HX1(0) *HX5(0,0,-1,0,1)
     &  + HX1(0) *HX5(0,0,-1,1,-1)
     &  + HX1(0) *HX5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + HX1(0) *HX5(0,0,0,1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  - HX1(0) *HX5(0,0,1,-1,-1)
     &  + HX1(0) *HX5(0,0,1,0,-1)
     &  + HX1(0) *HX5(0,0,1,0,1)
     &  - HX1(0) *HX5(0,0,1,1,-1)
     &  + HX1(0) *HX5(0,1,-1,1,-1)
     &  - HX1(0) *HX5(0,1,0,1,-1)
     &  + HX1(0) *HX5(0,1,1,-1,-1)
     &  - HX2(0, -1)*HX4(-1,-1,1,1)
     &  - HX2(0, -1)*HX4(0,-1,-1,1)
     &  - HX2(0, -1)*HX4(0,-1,1,-1)
     &  - HX2(0, -1)*HX4(0,1,-1,1)
     &  - HX2(0, -1)*HX4(0,1,1,-1)
     &  - HX2(0,1) *HX4(-1,-1,1,1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX4(0,-1,1,1)
     &  - HX2(0,1) *HX4(0,1,-1,1)
     &  - HX2(0,1) *HX4(0,1,1,-1)
     &  + 1.3169446513992682d+00*HX3(-1,-1,1)
     &  + 2.0000000000000000d+00*HX3(-1,-1,1)*HX3(0,-1,1)
     &  - 2.0000000000000000d+00*HX3(-1,-1,1)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX3(-1,-1,1)*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX3(-1,-1,1)*HX3(0,1,1)
     &  + 1.3169446513992682d+00*HX3(0,-1,-1)
     &  - 2.0000000000000000d+00*HX3(0,-1,-1)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX3(0,-1,-1)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX3(0,-1,1)*HX3(0,-1,1)
     &  + 2.0000000000000000d+00*HX3(0,-1,1)*HX3(0,1,-1)
     &  + 1.3169446513992682d+00*HX3(0,0,-1)
     &  - HX3(0,0, -1)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,1)
     &  - 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,1,-1)
     &  + 1.3169446513992682d+00*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,0,1)
     &  - 2.0000000000000000d+00*HX3(0,0,1)*HX3(0,1,-1)
     &  + 1.3169446513992682d+00*HX3(0,1,-1)
     &  + 2.4674011002723396d+00*HX4(-1,-1,1,1)
     &  + 2.4674011002723396d+00*HX4(0,-1,-1,1)
     &  + 2.4674011002723396d+00*HX4(0,-1,1,-1)
     &  - 2.4674011002723396d+00*HX4(0,0,-1,-1)
     &  + 2.4674011002723396d+00*HX4(0,0,-1,1)
     &  - 4.9348022005446793d+00*HX4(0,0,0,-1)
     &  - 4.9348022005446793d+00*HX4(0,0,0,1)
     &  - 2.4674011002723396d+00*HX4(0,0,1,-1)
     &  + 2.4674011002723396d+00*HX4(0,0,1,1)
     &  + 2.4674011002723396d+00*HX4(0,1,-1,1)
     &  + 2.4674011002723396d+00*HX4(0,1,1,-1)
     &  + HX6( -1,-1,1,1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,-1,-1,0,-1,1)
     &  + HX6(0, -1,-1,1,-1,1)
     &  - HX6(0, -1,-1,1,0,1)
     &  + 5.0000000000000000d+00*HX6(0,-1,0,-1,-1,1)
     &  + 3.0000000000000000d+00*HX6(0,-1,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,-1,0,-1,1,1)
     &  - 3.0000000000000000d+00*HX6(0,-1,0,1,1,-1)
     &  - 6.0000000000000000d+00*HX6(0,-1,0,1,1,1)
     &  + HX6(0, -1,1,-1,-1,1)
     &  - HX6(0, -1,1,-1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,-1,1,0,1,-1)
     &  - 4.0000000000000000d+00*HX6(0,-1,1,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,-1,1,1,-1,-1)
     &  - 2.0000000000000000d+00*HX6(0,-1,1,1,0,1)
     &  + 9.0000000000000000d+00*HX6(0,0,-1,-1,-1,1)
     &  + HX6(0,0, -1,-1,0,-1)
     &  + HX6(0,0, -1,-1,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,-1,1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,0,-1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,-1,1,-1,-1)
     &  + HX6(0,0, -1,1,-1,1)
     &  - HX6(0,0, -1,1,0,-1)
     &  - HX6(0,0, -1,1,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,-1,1,1,-1)
     &  - 1.2000000000000000d+01*HX6(0,0,-1,1,1,1)
     &  + 9.0000000000000000d+00*HX6(0,0,0,-1,-1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - HX6(0,0,0,0,0, -1)
     &  - HX6(0,0,0,0,0,1) 
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  + 9.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  - HX6(0,0,1, -1,-1,1)
     &  + HX6(0,0,1, -1,0,-1)
     &  + HX6(0,0,1, -1,0,1)
     &  + 4.0000000000000000d+00*HX6(0,0,1,0,-1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,1,1,-1,-1)
     &  + 7.0000000000000000d+00*HX6(0,0,1,1,-1,1)
     &  - HX6(0,0,1,1,0, -1)
     &  - HX6(0,0,1,1,0,1) 
     &  + 6.0000000000000000d+00*HX6(0,0,1,1,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,1,-1,0,1,1)
     &  + HX6(0,1, -1,1,-1,1)
     &  + 3.0000000000000000d+00*HX6(0,1,0,1,-1,1)
     &  + 3.0000000000000000d+00*HX6(0,1,0,1,1,-1)
     &  + HX6(0,1,1, -1,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,1,1,1,-1,-1)
      HY6(-1,-1,1,1,1,1) = 
     &  - 9.3423238096037068d+00
     &  + 2.1900870176160439d+00*HX1(-1)
     &  + 2.0293560632083841d+00*HX1(-1)*HX1(-1)
     &  - 1.2337005501361698d+00*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  + 2.0833333333333333d-02*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.0587121264167682d+00*HX1(-1)*HX1(0)
     &  + 8.2246703342411321d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX3(0,1,1)
     &  + 4.9348022005446793d+00*HX1(-1)*HX1(0)*HX2(0,-1)
     &  + 4.9348022005446793d+00*HX1(-1)*HX1(0)*HX2(0,1)
     &  - HX1( -1)*HX1(0)*HX4(0,-1,1,1)
     &  + HX1( -1)*HX1(0)*HX4(0,0,-1,1)
     &  - HX1( -1)*HX1(0)*HX4(0,0,0,-1)
     &  - HX1( -1)*HX1(0)*HX4(0,0,0,1)
     &  + HX1( -1)*HX1(0)*HX4(0,0,1,1)
     &  - HX1( -1)*HX1(0)*HX4(0,1,1,1)
     &  + 4.9348022005446793d+00*HX1(-1)*HX3(0,-1,1)
     &  - 4.9348022005446793d+00*HX1(-1)*HX3(0,0,-1)
     &  - 4.9348022005446793d+00*HX1(-1)*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX1(-1)*HX3(0,1,1)
     &  - HX1( -1)*HX5(0,-1,1,1,1)
     &  + HX1( -1)*HX5(0,0,-1,1,1)
     &  - HX1( -1)*HX5(0,0,0,-1,1)
     &  + HX1( -1)*HX5(0,0,0,0,-1)
     &  + HX1( -1)*HX5(0,0,0,0,1)
     &  - HX1( -1)*HX5(0,0,0,1,1)
     &  + HX1( -1)*HX5(0,0,1,1,1)
     &  - HX1( -1)*HX5(0,1,1,1,1)
     &  - 2.1900870176160439d+00*HX1(0)
     &  + 2.0293560632083841d+00*HX1(0)*HX1(0)
     &  - 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(-1,-1,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(-1,-1,1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,1)
     &  - HX1(0) *HX1(0)*HX4(0,0,0,-1)
     &  - HX1(0) *HX1(0)*HX4(0,0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,-1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,1,-1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(-1,-1,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,-1,-1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,1,-1)
     &  + HX1(0) *HX5(-1,-1,1,1,1)
     &  + HX1(0) *HX5(0,-1,-1,1,1)
     &  + HX1(0) *HX5(0,-1,1,-1,1)
     &  + HX1(0) *HX5(0,-1,1,1,-1)
     &  - HX1(0) *HX5(0,0,-1,-1,1)
     &  - HX1(0) *HX5(0,0,-1,1,-1)
     &  + HX1(0) *HX5(0,0,-1,1,1)
     &  + HX1(0) *HX5(0,0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + HX1(0) *HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  - HX1(0) *HX5(0,0,1,-1,1)
     &  - HX1(0) *HX5(0,0,1,1,-1)
     &  + HX1(0) *HX5(0,0,1,1,1)
     &  + HX1(0) *HX5(0,1,-1,1,1)
     &  + HX1(0) *HX5(0,1,1,-1,1)
     &  + HX1(0) *HX5(0,1,1,1,-1)
     &  - 4.9348022005446793d+00*HX4(-1,-1,1,1)
     &  - 4.9348022005446793d+00*HX4(0,-1,-1,1)
     &  - 4.9348022005446793d+00*HX4(0,-1,1,-1)
     &  + 4.9348022005446793d+00*HX4(0,0,-1,-1)
     &  - 4.9348022005446793d+00*HX4(0,0,-1,1)
     &  + 9.8696044010893586d+00*HX4(0,0,0,-1)
     &  + 9.8696044010893586d+00*HX4(0,0,0,1)
     &  + 4.9348022005446793d+00*HX4(0,0,1,-1)
     &  - 4.9348022005446793d+00*HX4(0,0,1,1)
     &  - 4.9348022005446793d+00*HX4(0,1,-1,1)
     &  - 4.9348022005446793d+00*HX4(0,1,1,-1)
     &  + HX6( -1,-1,1,1,1,1)
     &  + HX6(0, -1,-1,1,1,1)
     &  + HX6(0, -1,1,-1,1,1)
     &  + HX6(0, -1,1,1,-1,1)
     &  + HX6(0, -1,1,1,1,-1)
     &  - HX6(0,0, -1,-1,1,1)
     &  - HX6(0,0, -1,1,-1,1)
     &  - HX6(0,0, -1,1,1,-1)
     &  + HX6(0,0, -1,1,1,1)
     &  + HX6(0,0,0, -1,-1,1)
     &  + HX6(0,0,0, -1,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  - HX6(0,0,0,0, -1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - HX6(0,0,0,0,1, -1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  + HX6(0,0,0,1, -1,1)
     &  + HX6(0,0,0,1,1, -1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,1,1,1)
     &  - HX6(0,0,1, -1,1,1)
     &  - HX6(0,0,1,1, -1,1)
     &  - HX6(0,0,1,1,1, -1)
     &  + HX6(0,0,1,1,1,1) 
     &  + HX6(0,1, -1,1,1,1)
     &  + HX6(0,1,1, -1,1,1)
     &  + HX6(0,1,1,1, -1,1)
     &  + HX6(0,1,1,1,1, -1)
      HY6(-1,1,-1,1,1,1) = 
     &  + 9.6737080837053471d+00
     &  - 8.2125444753073106d+00*HX1(-1)
     &  - 6.4865749331714713d+00*HX1(-1)*HX1(0)
     &  + 8.2246703342411321d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX2(-1,1)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + HX1( -1)*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  - HX1( -1)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - HX1( -1)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + HX1( -1)*HX1(0)*HX1(0)*HX3(0,1,1)
     &  - 4.9348022005446793d+00*HX1(-1)*HX1(0)*HX2(-1,1)
     &  - 4.9348022005446793d+00*HX1(-1)*HX1(0)*HX2(0,-1)
     &  - 4.9348022005446793d+00*HX1(-1)*HX1(0)*HX2(0,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,0,0,1)
     &  - 3.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,0,1,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX1(0)*HX4(0,1,1,1)
     &  - 9.8696044010893586d+00*HX1(-1)*HX3(0,-1,1)
     &  + 9.8696044010893586d+00*HX1(-1)*HX3(0,0,-1)
     &  + 9.8696044010893586d+00*HX1(-1)*HX3(0,0,1)
     &  - 9.8696044010893586d+00*HX1(-1)*HX3(0,1,1)
     &  + 4.0000000000000000d+00*HX1(-1)*HX5(0,-1,1,1,1)
     &  - 4.0000000000000000d+00*HX1(-1)*HX5(0,0,-1,1,1)
     &  + 4.0000000000000000d+00*HX1(-1)*HX5(0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HX1(-1)*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX1(-1)*HX5(0,0,0,0,1)
     &  + 4.0000000000000000d+00*HX1(-1)*HX5(0,0,0,1,1)
     &  - 4.0000000000000000d+00*HX1(-1)*HX5(0,0,1,1,1)
     &  + 4.0000000000000000d+00*HX1(-1)*HX5(0,1,1,1,1)
     &  + 8.2125444753073106d+00*HX1(0)
     &  + 3.2432874665857356d+00*HX1(0)*HX1(0)
     &  - 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(-1,1)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(-1,-1,1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(-1,1)
     &  + 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(-1,1)*HX2(-1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(-1,1)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(-1,1)*HX2(0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(0,1)
     &  + 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)*HX2(0,1)
     &  - HX1(0) *HX1(0)*HX4(-1,-1,1,1)
     &  - HX1(0) *HX1(0)*HX4(0,-1,-1,1)
     &  - HX1(0) *HX1(0)*HX4(0,-1,1,-1)
     &  + HX1(0) *HX1(0)*HX4(0,0,-1,-1)
     &  - HX1(0) *HX1(0)*HX4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + HX1(0) *HX1(0)*HX4(0,0,1,-1)
     &  - HX1(0) *HX1(0)*HX4(0,0,1,1)
     &  - HX1(0) *HX1(0)*HX4(0,1,-1,1)
     &  - HX1(0) *HX1(0)*HX4(0,1,1,-1)
     &  - HX1(0) *HX2(-1,1)*HX3(0,-1,1)
     &  + HX1(0) *HX2(-1,1)*HX3(0,0,-1)
     &  + HX1(0) *HX2(-1,1)*HX3(0,0,1)
     &  - HX1(0) *HX2(-1,1)*HX3(0,1,1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,0,1)
     &  + HX1(0) *HX2(0,1)*HX3(0,-1,1)
     &  - HX1(0) *HX2(0,1)*HX3(0,0,-1)
     &  - HX1(0) *HX2(0,1)*HX3(0,0,1)
     &  + 9.8696044010893586d+00*HX1(0)*HX3(-1,-1,1)
     &  + 9.8696044010893586d+00*HX1(0)*HX3(0,-1,-1)
     &  + 9.8696044010893586d+00*HX1(0)*HX3(0,0,-1)
     &  + 9.8696044010893586d+00*HX1(0)*HX3(0,0,1)
     &  + 9.8696044010893586d+00*HX1(0)*HX3(0,1,-1)
     &  + HX1(0) *HX5(-1,1,-1,1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,-1,-1,1,1)
     &  + HX1(0) *HX5(0,-1,0,-1,1)
     &  - HX1(0) *HX5(0,-1,0,1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,-1,1,-1,1)
     &  - HX1(0) *HX5(0,-1,1,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,-1,1,1,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,1)
     &  + HX1(0) *HX5(0,0,-1,0,-1)
     &  + HX1(0) *HX5(0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,-1,1)
     &  + HX1(0) *HX5(0,0,1,0,-1)
     &  + HX1(0) *HX5(0,0,1,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,1,-1,1,1)
     &  + HX1(0) *HX5(0,1,0,1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,1,1,-1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,1,1,1,-1)
     &  - 6.4865749331714713d+00*HX2(-1,1)
     &  - 2.4674011002723396d+00*HX2(-1,1)*HX2(-1,1)
     &  + 4.9348022005446793d+00*HX2(-1,1)*HX2(0,-1)
     &  + 4.9348022005446793d+00*HX2(-1,1)*HX2(0,1)
     &  - HX2( -1,1)*HX4(0,-1,1,1)
     &  + HX2( -1,1)*HX4(0,0,-1,1)
     &  - HX2( -1,1)*HX4(0,0,0,-1)
     &  - HX2( -1,1)*HX4(0,0,0,1)
     &  + HX2( -1,1)*HX4(0,0,1,1)
     &  - HX2( -1,1)*HX4(0,1,1,1)
     &  + 6.4865749331714713d+00*HX2(0,-1)
     &  - 2.4674011002723396d+00*HX2(0,-1)*HX2(0,-1)
     &  - 4.9348022005446793d+00*HX2(0,-1)*HX2(0,1)
     &  - HX2(0, -1)*HX4(0,0,-1,1)
     &  + HX2(0, -1)*HX4(0,0,0,-1)
     &  + HX2(0, -1)*HX4(0,0,0,1)
     &  - HX2(0, -1)*HX4(0,0,1,1)
     &  + 6.4865749331714713d+00*HX2(0,1)
     &  - 2.4674011002723396d+00*HX2(0,1)*HX2(0,1)
     &  + HX2(0,1) *HX4(0,-1,1,1)
     &  - HX2(0,1) *HX4(0,0,-1,1)
     &  + HX2(0,1) *HX4(0,0,0,-1)
     &  + HX2(0,1) *HX4(0,0,0,1)
     &  - HX2(0,1) *HX4(0,0,1,1)
     &  + 9.8696044010893586d+00*HX4(-1,-1,1,1)
     &  + 9.8696044010893586d+00*HX4(0,-1,-1,1)
     &  + 9.8696044010893586d+00*HX4(0,-1,1,-1)
     &  - 9.8696044010893586d+00*HX4(0,0,-1,-1)
     &  + 9.8696044010893586d+00*HX4(0,0,-1,1)
     &  - 1.9739208802178717d+01*HX4(0,0,0,-1)
     &  - 1.9739208802178717d+01*HX4(0,0,0,1)
     &  - 9.8696044010893586d+00*HX4(0,0,1,-1)
     &  + 9.8696044010893586d+00*HX4(0,0,1,1)
     &  + 9.8696044010893586d+00*HX4(0,1,-1,1)
     &  + 9.8696044010893586d+00*HX4(0,1,1,-1)
     &  + HX6( -1,1,-1,1,1,1)
     &  - 2.0000000000000000d+00*HX6(0,-1,-1,1,1,1)
     &  + HX6(0, -1,0,-1,1,1)
     &  - 2.0000000000000000d+00*HX6(0,-1,0,1,1,1)
     &  - 2.0000000000000000d+00*HX6(0,-1,1,-1,1,1)
     &  - 2.0000000000000000d+00*HX6(0,-1,1,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,-1,1,1,-1,1)
     &  - HX6(0, -1,1,1,0,1)
     &  - 4.0000000000000000d+00*HX6(0,-1,1,1,1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,-1,1,1)
     &  + HX6(0,0, -1,0,-1,1)
     &  + HX6(0,0, -1,0,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,-1,1,-1,1)
     &  + HX6(0,0, -1,1,0,-1)
     &  + HX6(0,0, -1,1,0,1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,1,1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,-1,1,1,1)
     &  - HX6(0,0,0, -1,0,-1)
     &  - HX6(0,0,0, -1,0,1)
     &  - HX6(0,0,0, -1,1,-1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,-1,1,1)
     &  - 1.1000000000000000d+01*HX6(0,0,0,0,-1,1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,0,1)
     &  - 1.1000000000000000d+01*HX6(0,0,0,0,1,1)
     &  - HX6(0,0,0,1,0, -1)
     &  - HX6(0,0,0,1,0,1) 
     &  - HX6(0,0,0,1,1, -1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,1,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,-1,1,1)
     &  + HX6(0,0,1,0, -1,1)
     &  + HX6(0,0,1,0,1, -1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,1,1,-1,1)
     &  + HX6(0,0,1,1,0, -1)
     &  + HX6(0,0,1,1,0,1) 
     &  + 4.0000000000000000d+00*HX6(0,0,1,1,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,1,-1,1,1,1)
     &  + HX6(0,1,0,1,1,1) 
     &  - 2.0000000000000000d+00*HX6(0,1,1,-1,1,1)
     &  - 3.0000000000000000d+00*HX6(0,1,1,1,-1,1)
     &  - 4.0000000000000000d+00*HX6(0,1,1,1,1,-1)
      HY6(-1,1,1,1,1,1) = 
     &  - 7.4682175813356189d-01
     &  + 4.0587121264167682d+00*HX1(-1)*HX1(0)
     &  - 8.2246703342411321d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 2.0293560632083841d+00*HX1(0)*HX1(0)
     &  + 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(-1,1)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(-1,1,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(-1,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(-1,1,1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,1,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(-1,1,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,-1,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,1,1)
     &  + HX1(0) *HX5(-1,1,1,1,1)
     &  - HX1(0) *HX5(0,-1,1,1,1)
     &  + HX1(0) *HX5(0,0,-1,1,1)
     &  - HX1(0) *HX5(0,0,0,-1,1)
     &  + HX1(0) *HX5(0,0,0,0,-1)
     &  + HX1(0) *HX5(0,0,0,0,1)
     &  - HX1(0) *HX5(0,0,0,1,1)
     &  + HX1(0) *HX5(0,0,1,1,1)
     &  - HX1(0) *HX5(0,1,1,1,1)
     &  + 4.0587121264167682d+00*HX2(-1,1)
     &  - 4.0587121264167682d+00*HX2(0,-1)
     &  - 4.0587121264167682d+00*HX2(0,1)
     &  - 4.9348022005446793d+00*HX4(-1,1,1,1)
     &  + 4.9348022005446793d+00*HX4(0,-1,1,1)
     &  - 4.9348022005446793d+00*HX4(0,0,-1,1)
     &  + 4.9348022005446793d+00*HX4(0,0,0,-1)
     &  + 4.9348022005446793d+00*HX4(0,0,0,1)
     &  - 4.9348022005446793d+00*HX4(0,0,1,1)
     &  + 4.9348022005446793d+00*HX4(0,1,1,1)
     &  + HX6( -1,1,1,1,1,1)
     &  - HX6(0, -1,1,1,1,1)
     &  + HX6(0,0, -1,1,1,1)
     &  - HX6(0,0,0, -1,1,1)
     &  + HX6(0,0,0,0, -1,1)
     &  - HX6(0,0,0,0,0, -1)
     &  - HX6(0,0,0,0,0,1) 
     &  + HX6(0,0,0,0,1,1) 
     &  - HX6(0,0,0,1,1,1) 
     &  + HX6(0,0,1,1,1,1) 
     &  - HX6(0,1,1,1,1,1) 
      HY6(0,-1,-1,-1,-1,-1) = 
     &  + 1.0173430619844491d+00
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + HX1(0) *HX5(0,-1,-1,-1,-1)
     &  + HX1(0) *HX5(0,0,-1,-1,-1)
     &  + HX1(0) *HX5(0,0,0,-1,-1)
     &  + HX1(0) *HX5(0,0,0,0,-1)
     &  - HX6(0, -1,-1,-1,-1,-1)
     &  - HX6(0,0, -1,-1,-1,-1)
     &  - HX6(0,0,0, -1,-1,-1)
     &  - HX6(0,0,0,0, -1,-1)
     &  - HX6(0,0,0,0,0, -1)
      HY6(0,-1,-1,-1,-1,1) = 
     &  + 2.1003391764510429d+00
     &  + 2.1900870176160439d+00*HX1(0)
     &  + 1.2139314033773515d+00*HX1(0)*HX1(0)
     &  + 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 2.7620719062289241d+00*HX1(0)*HX2(0,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,-1,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX5(0,-1,-1,-1,-1)
     &  - HX1(0) *HX5(0,0,-1,-1,-1)
     &  - HX1(0) *HX5(0,0,0,-1,-1)
     &  - HX1(0) *HX5(0,0,0,0,-1)
     &  - 2.4278628067547031d+00*HX2(0,-1)
     &  + HX2(0, -1)*HX4(0,-1,-1,-1)
     &  + HX2(0, -1)*HX4(0,0,-1,-1)
     &  + HX2(0, -1)*HX4(0,0,0,-1)
     &  + HX2(0, -1)*HX4(0,0,0,1)
     &  + HX2(0, -1)*HX4(0,0,1,-1)
     &  + 2.7620719062289241d+00*HX3(0,-1,-1)
     &  - 5.0000000000000000d-01*HX3(0,-1,-1)*HX3(0,-1,-1)
     &  - HX3(0, -1,-1)*HX3(0,0,-1)
     &  - HX3(0, -1,-1)*HX3(0,0,1)
     &  + 2.7620719062289241d+00*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  - 2.4674011002723396d+00*HX4(0,-1,-1,-1)
     &  - 2.4674011002723396d+00*HX4(0,0,-1,-1)
     &  - 2.4674011002723396d+00*HX4(0,0,0,-1)
     &  - HX6(0, -1,-1,-1,-1,1)
     &  + HX6(0, -1,-1,-1,0,1)
     &  + HX6(0, -1,-1,0,-1,1)
     &  + HX6(0, -1,0,-1,-1,1)
     &  + HX6(0,0, -1,-1,-1,1)
     &  + HX6(0,0, -1,-1,0,1)
     &  + HX6(0,0, -1,0,-1,1)
     &  - HX6(0,0, -1,0,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - HX6(0,0,0,0,0,1) 
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  - HX6(0,0,0,1,0, -1)
     &  - HX6(0,0,1, -1,0,-1)
     &  - HX6(0,0,1,0, -1,-1)
      HY6(0,-1,-1,-1,0,1) = 
     &  - 2.0273622389255968d+00
     &  - 1.7203041663629827d+00*HX1(0)
     &  - 1.0146780316041920d-01*HX1(0)*HX1(0)
     &  + 3.0051422578989857d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.6449340668482264d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 1.8030853547393914d+00*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,-1,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,0,-1)
     &  + 3.2898681336964528d+00*HX1(0)*HX3(0,-1,-1)
     &  + 3.2898681336964528d+00*HX1(0)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,-1,0,-1,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 2.0293560632083841d-01*HX2(0,-1)
     &  - HX2(0, -1)*HX4(0,0,-1,-1)
     &  + HX2(0, -1)*HX4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,-1)
     &  + HX2(0, -1)*HX4(0,0,0,1)
     &  + HX2(0, -1)*HX4(0,0,1,-1)
     &  + 1.8030853547393914d+00*HX3(0,-1,-1)
     &  + HX3(0, -1,-1)*HX3(0,0,-1)
     &  - HX3(0, -1,-1)*HX3(0,0,1)
     &  + 1.8030853547393914d+00*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  - 3.2898681336964528d+00*HX4(0,-1,-1,-1)
     &  - 3.2898681336964528d+00*HX4(0,0,-1,-1)
     &  - 3.2898681336964528d+00*HX4(0,0,0,-1)
     &  + HX6(0, -1,-1,-1,0,1)
     &  + HX6(0,0, -1,-1,0,-1)
     &  + HX6(0,0, -1,-1,0,1)
     &  + HX6(0,0, -1,0,-1,-1)
     &  - HX6(0,0, -1,0,-1,1)
     &  - HX6(0,0, -1,0,0,1)
     &  - HX6(0,0, -1,0,1,-1)
     &  - HX6(0,0, -1,1,0,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  + 8.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - HX6(0,0,0,0,0,1) 
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  - HX6(0,0,0,1,0, -1)
     &  - HX6(0,0,1, -1,0,-1)
     &  - HX6(0,0,1,0, -1,-1)
      HY6(0,-1,-1,-1,1,-1) = 
     &  - 1.1650058311842651d+01
     &  - 8.7603480704641757d+00*HX1(0)
     &  - 3.6417942101320546d+00*HX1(0)*HX1(0)
     &  - 9.2069063540964137d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,-1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 5.5241438124578482d+00*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,-1,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,0,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,-1,-1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX5(0,-1,-1,-1,1)
     &  - HX1(0) *HX5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,-1,0,-1,-1)
     &  - HX1(0) *HX5(0,-1,0,-1,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,-1)
     &  - HX1(0) *HX5(0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,-1)
     &  - HX1(0) *HX5(0,0,-1,0,1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - HX1(0) *HX5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - HX1(0) *HX5(0,0,1,0,-1)
     &  + 7.2835884202641093d+00*HX2(0,-1)
     &  - HX2(0, -1)*HX4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  - 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,-1)
     &  - 5.5241438124578482d+00*HX3(0,-1,-1)
     &  + HX3(0, -1,-1)*HX3(0,-1,-1)
     &  + 2.0000000000000000d+00*HX3(0,-1,-1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX3(0,-1,-1)*HX3(0,0,1)
     &  - 5.5241438124578482d+00*HX3(0,0,-1)
     &  + HX3(0,0, -1)*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX4(0,-1,-1,-1)
     &  + 2.4674011002723396d+00*HX4(0,0,-1,-1)
     &  + 2.4674011002723396d+00*HX4(0,0,0,-1)
     &  - HX6(0, -1,-1,-1,0,1)
     &  - HX6(0, -1,-1,-1,1,-1)
     &  - HX6(0, -1,-1,0,-1,1)
     &  + HX6(0, -1,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,-1,0,-1,-1,-1)
     &  - HX6(0, -1,0,-1,-1,1)
     &  + HX6(0, -1,0,-1,1,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,-1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,-1,0,-1)
     &  - HX6(0,0, -1,-1,0,1)
     &  + HX6(0,0, -1,-1,1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,0,-1,-1)
     &  - HX6(0,0, -1,0,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  + HX6(0,0, -1,0,1,-1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,0,-1,-1)
     &  + 8.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  + 1.3000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,1,-1,0,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,1,0,-1,-1)
      HY6(0,-1,-1,-1,1,1) = 
     &  + 6.7326724972609152d+00
     &  + 5.5581622138319701d+00*HX1(0)
     &  + 1.0146780316041920d+00*HX1(0)*HX1(0)
     &  - 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,-1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 2.7620719062289241d+00*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,-1,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,0,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,-1,-1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX5(0,-1,-1,-1,1)
     &  + HX1(0) *HX5(0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,-1,0,-1,-1)
     &  + HX1(0) *HX5(0,-1,0,-1,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,-1)
     &  + HX1(0) *HX5(0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,-1)
     &  + HX1(0) *HX5(0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + HX1(0) *HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + HX1(0) *HX5(0,0,1,0,-1)
     &  - 2.0293560632083841d+00*HX2(0,-1)
     &  + HX2(0, -1)*HX4(0,0,-1,-1)
     &  - HX2(0, -1)*HX4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  + HX2(0, -1)*HX4(0,0,1,-1)
     &  - HX2(0, -1)*HX4(0,0,1,1)
     &  - 2.7620719062289241d+00*HX3(0,-1,-1)
     &  - HX3(0, -1,-1)*HX3(0,0,-1)
     &  - HX3(0, -1,-1)*HX3(0,0,1)
     &  - 2.7620719062289241d+00*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  + 4.9348022005446793d+00*HX4(0,-1,-1,-1)
     &  + 4.9348022005446793d+00*HX4(0,0,-1,-1)
     &  + 4.9348022005446793d+00*HX4(0,0,0,-1)
     &  - HX6(0, -1,-1,-1,1,1)
     &  + HX6(0, -1,-1,0,-1,1)
     &  + HX6(0, -1,-1,0,1,1)
     &  + HX6(0, -1,0,-1,-1,1)
     &  + HX6(0, -1,0,-1,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,-1,-1,1)
     &  - HX6(0,0, -1,-1,0,-1)
     &  + HX6(0,0, -1,-1,1,1)
     &  - HX6(0,0, -1,0,-1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,-1,1)
     &  - HX6(0,0, -1,0,0,1)
     &  + HX6(0,0, -1,0,1,-1)
     &  + HX6(0,0, -1,0,1,1)
     &  + HX6(0,0, -1,1,0,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,-1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  - 8.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - 8.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + HX6(0,0,0,0,1,1) 
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  - HX6(0,0,1, -1,0,-1)
     &  - HX6(0,0,1,0, -1,-1)
     &  + HX6(0,0,1,0, -1,1)
     &  + HX6(0,0,1,0,1, -1)
     &  + HX6(0,0,1,1,0, -1)
      HY6(0,-1,-1,0,-1,1) = 
     &  + 1.0062463457221988d+01
     &  + 5.9880258797747571d+00*HX1(0)
     &  + 1.7423670582905094d+00*HX1(0)*HX1(0)
     &  + 4.1983359242647709d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  - 2.5190015545588625d+00*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,-1,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,-1,0,-1,-1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,-1)
     &  + 9.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 3.4847341165810188d+00*HX2(0,-1)
     &  - 1.2337005501361698d+00*HX2(0,-1)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX2(0,-1)*HX2(0,-1)*HX2(0,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,-1,-1)
     &  + 4.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,-1)
     &  + HX2(0, -1)*HX4(0,0,0,1)
     &  + HX2(0, -1)*HX4(0,0,1,-1)
     &  + 2.5190015545588625d+00*HX3(0,-1,-1)
     &  - 2.0000000000000000d+00*HX3(0,-1,-1)*HX3(0,0,-1)
     &  - HX3(0, -1,-1)*HX3(0,0,1)
     &  + 2.5190015545588625d+00*HX3(0,0,-1)
     &  - HX3(0,0, -1)*HX3(0,0,-1)
     &  + HX6(0, -1,-1,0,-1,1)
     &  + HX6(0, -1,0,-1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,-1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,-1,-1)
     &  + HX6(0,0, -1,0,-1,1)
     &  - HX6(0,0, -1,0,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 1.6000000000000000d+01*HX6(0,0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - HX6(0,0,0,0,0,1) 
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  - HX6(0,0,0,1,0, -1)
     &  - HX6(0,0,1, -1,0,-1)
     &  - HX6(0,0,1,0, -1,-1)
      HY6(0,-1,-1,0,1,-1) = 
     &  + 6.6666541645113502d-01
     &  - 8.2711338068580881d-01*HX1(0)
     &  - 1.5394314519696710d+00*HX1(0)*HX1(0)
     &  - 7.2034781821637566d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + 4.3220869092982539d+00*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,0,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,-1,-1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX5(0,-1,-1,0,1)
     &  + HX1(0) *HX5(0,0,-1,0,-1)
     &  - HX1(0) *HX5(0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - HX1(0) *HX5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - HX1(0) *HX5(0,0,1,0,-1)
     &  + 3.0788629039393420d+00*HX2(0,-1)
     &  + 1.2337005501361698d+00*HX2(0,-1)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX2(0,-1)*HX2(0,-1)*HX2(0,-1)
     &  - 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,-1,1)
     &  - 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  - 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,-1)
     &  - 4.3220869092982539d+00*HX3(0,-1,-1)
     &  + HX3(0, -1,-1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX3(0,-1,-1)*HX3(0,0,1)
     &  - 4.3220869092982539d+00*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  + HX6(0, -1,-1,0,1,-1)
     &  - HX6(0, -1,0,-1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,-1,0,1)
     &  - HX6(0,0, -1,0,-1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,1,0,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,-1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 9.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  + 1.3000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,1,-1,0,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,1,0,-1,-1)
      HY6(0,-1,-1,0,1,1) = 
     &  + 1.7398134612012662d+00
     &  + 4.9959999654410204d+00*HX1(0)
     &  + 1.9278882600479649d+00*HX1(0)*HX1(0)
     &  - 2.0034281719326571d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + 1.2020569031595942d+00*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,0,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,-1,-1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX5(0,-1,-1,0,1)
     &  - HX1(0) *HX5(0,0,-1,0,-1)
     &  + HX1(0) *HX5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + HX1(0) *HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + HX1(0) *HX5(0,0,1,0,-1)
     &  - 3.8557765200959298d+00*HX2(0,-1)
     &  + 2.4674011002723396d+00*HX2(0,-1)*HX2(0,-1)
     &  + HX2(0, -1)*HX4(0,0,-1,1)
     &  - HX2(0, -1)*HX4(0,0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  + HX2(0, -1)*HX4(0,0,1,-1)
     &  - HX2(0, -1)*HX4(0,0,1,1)
     &  - 1.2020569031595942d+00*HX3(0,-1,-1)
     &  - HX3(0, -1,-1)*HX3(0,0,1)
     &  - 1.2020569031595942d+00*HX3(0,0,-1)
     &  + HX6(0, -1,-1,0,1,1)
     &  - HX6(0,0, -1,0,-1,1)
     &  - HX6(0,0, -1,0,0,1)
     &  - HX6(0,0, -1,0,1,-1)
     &  + HX6(0,0, -1,0,1,1)
     &  - HX6(0,0, -1,1,0,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  + HX6(0,0,0, -1,0,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - 8.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - 8.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + HX6(0,0,0,0,1,1) 
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  - HX6(0,0,1, -1,0,-1)
     &  - HX6(0,0,1,0, -1,-1)
     &  + HX6(0,0,1,0, -1,1)
     &  + HX6(0,0,1,0,1, -1)
     &  + HX6(0,0,1,1,0, -1)
      HY6(0,-1,-1,1,-1,-1) = 
     &  + 1.9414963161326077d+01
     &  + 1.3140522105696263d+01*HX1(0)
     &  + 3.6417942101320546d+00*HX1(0)*HX1(0)
     &  + 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 2.7620719062289241d+00*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,1)
     &  + HX1(0) *HX5(0,-1,-1,0,1)
     &  + HX1(0) *HX5(0,-1,-1,1,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,-1,0,-1,-1)
     &  + HX1(0) *HX5(0,-1,0,-1,1)
     &  - HX1(0) *HX5(0,-1,0,1,-1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,-1)
     &  + HX1(0) *HX5(0,0,-1,0,1)
     &  - HX1(0) *HX5(0,0,-1,1,-1)
     &  + 9.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  - 7.2835884202641093d+00*HX2(0,-1)
     &  + 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  + 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,-1)
     &  + 2.7620719062289241d+00*HX3(0,-1,-1)
     &  - 5.0000000000000000d-01*HX3(0,-1,-1)*HX3(0,-1,-1)
     &  - HX3(0, -1,-1)*HX3(0,0,-1)
     &  - HX3(0, -1,-1)*HX3(0,0,1)
     &  + 2.7620719062289241d+00*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  - HX6(0, -1,-1,0,1,-1)
     &  - HX6(0, -1,-1,1,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,-1,0,-1,-1,-1)
     &  - HX6(0, -1,0,-1,1,-1)
     &  + HX6(0, -1,0,1,-1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,-1,-1,-1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,-1,0,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,-1,1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,-1,0,-1,-1)
     &  - HX6(0,0, -1,0,0,1)
     &  - HX6(0,0, -1,0,1,-1)
     &  + HX6(0,0, -1,1,-1,-1)
     &  - 1.8000000000000000d+01*HX6(0,0,0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  - 1.5000000000000000d+01*HX6(0,0,0,0,-1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,0,1)
     &  - 1.6000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  - 1.6000000000000000d+01*HX6(0,0,0,1,-1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,1,-1,0,-1)
     &  - 5.0000000000000000d+00*HX6(0,0,1,0,-1,-1)
      HY6(0,-1,-1,1,-1,1) = 
     &  - 1.8151733630805092d+01
     &  - 9.2329419831013177d+00*HX1(0)
     &  - 1.6928093109730279d+00*HX1(0)*HX1(0)
     &  - 2.1949077523321137d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 1.3169446513992682d+00*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,-1,-1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX5(0,-1,-1,0,1)
     &  - HX1(0) *HX5(0,-1,-1,1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,-1,0,-1,-1)
     &  - HX1(0) *HX5(0,-1,0,-1,1)
     &  + HX1(0) *HX5(0,-1,0,1,-1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,-1)
     &  - HX1(0) *HX5(0,0,-1,0,1)
     &  + HX1(0) *HX5(0,0,-1,1,-1)
     &  - 9.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  + 3.3856186219460558d+00*HX2(0,-1)
     &  + 1.2337005501361698d+00*HX2(0,-1)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX2(0,-1)*HX2(0,-1)*HX2(0,-1)
     &  + HX2(0, -1)*HX4(0,-1,-1,1)
     &  - HX2(0, -1)*HX4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,-1,1)
     &  - 4.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,-1)
     &  - 4.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  - 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,-1)
     &  - 1.3169446513992682d+00*HX3(0,-1,-1)
     &  + 2.0000000000000000d+00*HX3(0,-1,-1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX3(0,-1,-1)*HX3(0,0,1)
     &  - 1.3169446513992682d+00*HX3(0,0,-1)
     &  + HX3(0,0, -1)*HX3(0,0,-1)
     &  - 2.4674011002723396d+00*HX4(0,-1,-1,1)
     &  + 2.4674011002723396d+00*HX4(0,-1,0,1)
     &  + 2.4674011002723396d+00*HX4(0,0,-1,1)
     &  - 2.4674011002723396d+00*HX4(0,0,0,1)
     &  - 2.0000000000000000d+00*HX6(0,-1,-1,0,-1,1)
     &  - HX6(0, -1,-1,1,-1,1)
     &  + HX6(0, -1,-1,1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,-1,0,-1,-1,1)
     &  + HX6(0, -1,0,-1,0,1)
     &  + HX6(0, -1,0,1,-1,1)
     &  - HX6(0, -1,0,1,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,-1,-1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,-1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,-1,-1)
     &  - 5.0000000000000000d+00*HX6(0,0,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,0,1,-1)
     &  + HX6(0,0, -1,1,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,1,0,-1)
     &  - HX6(0,0, -1,1,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,-1,-1,-1)
     &  - 1.5000000000000000d+01*HX6(0,0,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  + 1.6000000000000000d+01*HX6(0,0,0,0,-1,-1)
     &  + 9.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 2.0000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  - HX6(0,0,0,1, -1,1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + HX6(0,0,0,1,0,1) 
     &  + 2.0000000000000000d+00*HX6(0,0,1,-1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,-1,-1)
      HY6(0,-1,-1,1,0,1) = 
     &  - 1.1965283280637384d+01
     &  - 1.8100489770907654d+00*HX1(0)
     &  + 1.7423670582905094d+00*HX1(0)*HX1(0)
     &  + 4.0068563438653142d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.6449340668482264d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 2.4041138063191885d+00*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,1)
     &  - 3.2898681336964528d+00*HX1(0)*HX3(0,-1,-1)
     &  - 3.2898681336964528d+00*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX5(0,-1,-1,0,1)
     &  - HX1(0) *HX5(0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,1)
     &  - HX1(0) *HX5(0,0,-1,0,-1)
     &  - HX1(0) *HX5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  - 3.4847341165810188d+00*HX2(0,-1)
     &  + 1.6449340668482264d+00*HX2(0,-1)*HX2(0,-1)
     &  - HX2(0, -1)*HX4(0,0,-1,1)
     &  - HX2(0, -1)*HX4(0,0,0,-1)
     &  - 4.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  - 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,-1)
     &  + 2.4041138063191885d+00*HX3(0,-1,-1)
     &  + 2.0000000000000000d+00*HX3(0,-1,-1)*HX3(0,0,1)
     &  + 2.4041138063191885d+00*HX3(0,0,-1)
     &  - 3.2898681336964528d+00*HX4(0,-1,-1,1)
     &  + 3.2898681336964528d+00*HX4(0,-1,0,1)
     &  + 3.2898681336964528d+00*HX4(0,0,-1,1)
     &  - 3.2898681336964528d+00*HX4(0,0,0,1)
     &  + HX6(0, -1,-1,1,0,1)
     &  - HX6(0, -1,0,-1,0,1)
     &  - HX6(0, -1,0,1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,-1,0,1)
     &  + HX6(0,0, -1,0,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  + HX6(0,0, -1,0,1,-1)
     &  + HX6(0,0, -1,1,0,-1)
     &  - HX6(0,0, -1,1,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  + HX6(0,0,0, -1,0,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,0,-1,1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 1.6000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + HX6(0,0,0,1,0,1) 
     &  + 2.0000000000000000d+00*HX6(0,0,1,-1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,-1,-1)
      HY6(0,-1,-1,1,1,-1) = 
     &  - 1.3965470197185443d+01
     &  - 7.4415446583945925d+00*HX1(0)
     &  - 3.3654675223535616d-01*HX1(0)*HX1(0)
     &  + 6.7983609293803206d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,-1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 4.0790165576281924d+00*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,0,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,-1,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX5(0,-1,-1,1,1)
     &  - HX1(0) *HX5(0,-1,0,-1,1)
     &  - HX1(0) *HX5(0,-1,0,1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,1)
     &  - HX1(0) *HX5(0,0,-1,0,-1)
     &  - HX1(0) *HX5(0,0,-1,1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + HX1(0) *HX5(0,0,0,1,1)
     &  - HX1(0) *HX5(0,0,1,0,-1)
     &  + 6.7309350447071233d-01*HX2(0,-1)
     &  - 1.2337005501361698d+00*HX2(0,-1)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX2(0,-1)*HX2(0,-1)*HX2(0,-1)
     &  - HX2(0, -1)*HX4(0,-1,-1,1)
     &  + HX2(0, -1)*HX4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,1)
     &  + 4.0790165576281924d+00*HX3(0,-1,-1)
     &  - HX3(0, -1,-1)*HX3(0,0,-1)
     &  - HX3(0, -1,-1)*HX3(0,0,1)
     &  + 4.0790165576281924d+00*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX4(0,-1,-1,1)
     &  - 2.4674011002723396d+00*HX4(0,-1,0,1)
     &  - 2.4674011002723396d+00*HX4(0,0,-1,1)
     &  + 2.4674011002723396d+00*HX4(0,0,0,1)
     &  + HX6(0, -1,-1,0,-1,1)
     &  - HX6(0, -1,-1,0,1,1)
     &  - HX6(0, -1,-1,1,0,1)
     &  - HX6(0, -1,-1,1,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,-1,0,-1,-1,1)
     &  - HX6(0, -1,0,-1,0,1)
     &  + HX6(0, -1,0,-1,1,-1)
     &  - HX6(0, -1,0,-1,1,1)
     &  + HX6(0, -1,0,1,0,1)
     &  + HX6(0, -1,0,1,1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,-1,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,-1,1,1)
     &  + HX6(0,0, -1,0,-1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,-1,1)
     &  - HX6(0,0, -1,0,0,1)
     &  + HX6(0,0, -1,0,1,-1)
     &  - HX6(0,0, -1,0,1,1)
     &  + HX6(0,0, -1,1,0,-1)
     &  + HX6(0,0, -1,1,0,1)
     &  + HX6(0,0, -1,1,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,-1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - HX6(0,0,0,0,1, -1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - HX6(0,0,0,1,0, -1)
     &  - HX6(0,0,0,1,0,1) 
     &  - 7.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  + HX6(0,0,1,0, -1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,0,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,1,0,-1)
      HY6(0,-1,-1,1,1,1) = 
     &  + 2.0516889677812479d+00
     &  - 5.5581622138319701d+00*HX1(0)
     &  - 3.2432874665857356d+00*HX1(0)*HX1(0)
     &  + 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,-1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,-1,-1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX5(0,-1,-1,1,1)
     &  + HX1(0) *HX5(0,-1,0,-1,1)
     &  + HX1(0) *HX5(0,-1,0,1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,1)
     &  + HX1(0) *HX5(0,0,-1,0,-1)
     &  + HX1(0) *HX5(0,0,-1,1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - HX1(0) *HX5(0,0,0,1,1)
     &  + HX1(0) *HX5(0,0,1,0,-1)
     &  + 6.4865749331714713d+00*HX2(0,-1)
     &  - 2.4674011002723396d+00*HX2(0,-1)*HX2(0,-1)
     &  - HX2(0, -1)*HX4(0,0,-1,1)
     &  + HX2(0, -1)*HX4(0,0,0,-1)
     &  + HX2(0, -1)*HX4(0,0,0,1)
     &  - HX2(0, -1)*HX4(0,0,1,1)
     &  + 4.9348022005446793d+00*HX4(0,-1,-1,1)
     &  - 4.9348022005446793d+00*HX4(0,-1,0,1)
     &  - 4.9348022005446793d+00*HX4(0,0,-1,1)
     &  + 4.9348022005446793d+00*HX4(0,0,0,1)
     &  - HX6(0, -1,-1,1,1,1)
     &  + HX6(0, -1,0,-1,1,1)
     &  + HX6(0, -1,0,1,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,-1,1,1)
     &  + HX6(0,0, -1,0,-1,1)
     &  + HX6(0,0, -1,0,1,-1)
     &  + HX6(0,0, -1,1,0,-1)
     &  + HX6(0,0, -1,1,1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  - HX6(0,0,0, -1,0,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - HX6(0,0,0,1,0, -1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  - HX6(0,0,0,1,1,1) 
     &  + HX6(0,0,1,0, -1,1)
     &  + HX6(0,0,1,0,1, -1)
     &  + HX6(0,0,1,1,0, -1)
      HY6(0,-1,0,-1,-1,-1) = 
     &  + 1.1279257064283309d+01
     &  + 4.1477110205734797d+00*HX1(0)
     &  + 5.4116161685556909d-01*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  - HX1(0) *HX1(0)*HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,0,-1)
     &  - HX1(0) *HX5(0,-1,0,-1,-1)
     &  + HX1(0) *HX5(0,0,-1,0,-1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 1.0823232337111381d+00*HX2(0,-1)
     &  + HX2(0, -1)*HX4(0,0,-1,-1)
     &  + HX2(0, -1)*HX4(0,0,0,-1)
     &  + HX6(0, -1,0,-1,-1,-1)
     &  - HX6(0,0, -1,-1,0,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,-1,-1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,-1,-1,-1)
     &  - HX6(0,0,0, -1,0,-1)
     &  - 1.1000000000000000d+01*HX6(0,0,0,0,-1,-1)
     &  - 1.1000000000000000d+01*HX6(0,0,0,0,0,-1)
      HY6(0,-1,0,-1,-1,1) = 
     &  - 2.8122751824323687d+00
     &  + 1.0667299100688324d+00*HX1(0)
     &  + 1.2604799663732358d+00*HX1(0)*HX1(0)
     &  + 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + HX1(0) *HX1(0)*HX4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 2.7620719062289241d+00*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,0,-1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX5(0,-1,0,-1,-1)
     &  - HX1(0) *HX5(0,0,-1,0,-1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 2.5209599327464717d+00*HX2(0,-1)
     &  + 1.2337005501361698d+00*HX2(0,-1)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX2(0,-1)*HX2(0,-1)*HX2(0,-1)
     &  - HX2(0, -1)*HX4(0,0,-1,1)
     &  + HX2(0, -1)*HX4(0,0,0,1)
     &  + 5.5241438124578482d+00*HX3(0,0,-1)
     &  - HX3(0,0, -1)*HX3(0,0,-1)
     &  - 4.9348022005446793d+00*HX4(0,0,-1,-1)
     &  - 9.8696044010893586d+00*HX4(0,0,0,-1)
     &  + HX6(0, -1,0,-1,-1,1)
     &  - HX6(0, -1,0,-1,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,-1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  + HX6(0,0, -1,0,1,-1)
     &  + HX6(0,0, -1,1,0,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,-1,-1,-1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  + 1.6000000000000000d+01*HX6(0,0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - HX6(0,0,0,0,0,1) 
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - HX6(0,0,0,1,0, -1)
      HY6(0,-1,0,-1,0,1) = 
     &  - 3.4629317138014295d+00
     &  - 6.5700686637072277d-01*HX1(0)
     &  + 4.7351641474862295d-01*HX1(0)*HX1(0)
     &  + 3.0051422578989857d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.6449340668482264d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + HX1(0) *HX1(0)*HX4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 1.8030853547393914d+00*HX1(0)*HX2(0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,-1)
     &  + 6.5797362673929057d+00*HX1(0)*HX3(0,0,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,-1)
     &  - 1.2000000000000000d+01*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 8.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 9.4703282949724591d-01*HX2(0,-1)
     &  + 1.6449340668482264d+00*HX2(0,-1)*HX2(0,-1)
     &  - 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,-1)
     &  + HX2(0, -1)*HX4(0,0,0,1)
     &  + 3.6061707094787828d+00*HX3(0,0,-1)
     &  + HX3(0,0, -1)*HX3(0,0,-1)
     &  - 6.5797362673929057d+00*HX4(0,0,-1,-1)
     &  - 1.3159472534785811d+01*HX4(0,0,0,-1)
     &  - HX6(0, -1,0,-1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,0,-1,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 1.5000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - HX6(0,0,0,0,0,1) 
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - HX6(0,0,0,1,0, -1)
      HY6(0,-1,0,-1,1,-1) = 
     &  - 2.4725076630725787d+01
     &  - 1.4109511579687179d+01*HX1(0)
     &  - 4.2633269910369811d+00*HX1(0)*HX1(0)
     &  - 9.2069063540964137d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + HX1(0) *HX1(0)*HX4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 5.5241438124578482d+00*HX1(0)*HX2(0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,0,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX5(0,-1,0,-1,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,1)
     &  - 1.2000000000000000d+01*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 8.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - HX1(0) *HX5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - HX1(0) *HX5(0,0,1,0,-1)
     &  + 8.5266539820739622d+00*HX2(0,-1)
     &  - 1.2337005501361698d+00*HX2(0,-1)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX2(0,-1)*HX2(0,-1)*HX2(0,-1)
     &  - 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,-1,1)
     &  - 4.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  - HX2(0, -1)*HX4(0,0,1,-1)
     &  - 1.1048287624915696d+01*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,-1)
     &  + 4.9348022005446793d+00*HX4(0,0,-1,-1)
     &  + 9.8696044010893586d+00*HX4(0,0,0,-1)
     &  + HX6(0, -1,0,-1,0,1)
     &  + HX6(0, -1,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,0,-1,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,-1,0,-1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,1,0,-1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,-1,-1,-1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,-1,-1,1)
     &  + 8.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  + 8.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + 8.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 1.6000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  + 1.3000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + HX6(0,0,1, -1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,-1,-1)
      HY6(0,-1,0,-1,1,1) = 
     &  + 4.7221716255277494d+00
     &  + 2.7333771965496118d+00*HX1(0)
     &  + 3.1344713687598993d-01*HX1(0)*HX1(0)
     &  - 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  - HX1(0) *HX1(0)*HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 2.7620719062289241d+00*HX1(0)*HX2(0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,0,1)
     &  - 9.8696044010893586d+00*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX5(0,-1,0,-1,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,1)
     &  + 1.2000000000000000d+01*HX1(0)*HX5(0,0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 8.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + HX1(0) *HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + HX1(0) *HX5(0,0,1,0,-1)
     &  - 6.2689427375197987d-01*HX2(0,-1)
     &  - 2.4674011002723396d+00*HX2(0,-1)*HX2(0,-1)
     &  - 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  - HX2(0, -1)*HX4(0,0,1,1)
     &  - 5.5241438124578482d+00*HX3(0,0,-1)
     &  - HX3(0,0, -1)*HX3(0,0,-1)
     &  + 9.8696044010893586d+00*HX4(0,0,-1,-1)
     &  + 1.9739208802178717d+01*HX4(0,0,0,-1)
     &  + HX6(0, -1,0,-1,1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,1,0,-1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,0,-1,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 1.5000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - 8.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + HX6(0,0,0,0,1,1) 
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  + HX6(0,0,1,0, -1,1)
     &  + HX6(0,0,1,0,1, -1)
     &  + HX6(0,0,1,1,0, -1)
      HY6(0,-1,0,1,-1,-1) = 
     &  + 7.4465413633597362d+00
     &  + 7.8818691705293984d+00*HX1(0)
     &  + 2.9013792215033260d+00*HX1(0)*HX1(0)
     &  + 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 2.7620719062289241d+00*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,1)
     &  - HX1(0) *HX5(0,-1,0,1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  - 5.8027584430066521d+00*HX2(0,-1)
     &  + HX2(0, -1)*HX4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,-1)
     &  + 5.5241438124578482d+00*HX3(0,0,-1)
     &  - HX3(0,0, -1)*HX3(0,0,-1)
     &  + HX6(0, -1,0,1,-1,-1)
     &  - HX6(0,0, -1,-1,0,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,-1,0,-1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,1,-1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  - 5.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,0,-1,1)
     &  + HX6(0,0,0,0,0, -1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,0,1)
     &  - 1.6000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  - 1.3000000000000000d+01*HX6(0,0,0,1,-1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,-1,0,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,1,0,-1,-1)
      HY6(0,-1,0,1,-1,1) = 
     &  - 1.5388344493531442d+01
     &  - 9.8321845897207782d+00*HX1(0)
     &  - 2.1663257257216508d+00*HX1(0)*HX1(0)
     &  - 2.1949077523321137d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 1.3169446513992682d+00*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX5(0,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  + 4.3326514514433017d+00*HX2(0,-1)
     &  - HX2(0, -1)*HX4(0,-1,0,1)
     &  + 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,-1,1)
     &  - 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,-1)
     &  - 4.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  - 2.6338893027985365d+00*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX4(0,-1,0,1)
     &  + 7.4022033008170189d+00*HX4(0,0,0,-1)
     &  - 2.4674011002723396d+00*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX6(0,-1,0,-1,0,1)
     &  + HX6(0, -1,0,1,-1,1)
     &  - HX6(0, -1,0,1,0,1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,-1,0,1)
     &  - 7.0000000000000000d+00*HX6(0,0,-1,0,-1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,1,0,-1)
     &  - 1.8000000000000000d+01*HX6(0,0,0,-1,-1,1)
     &  + 9.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  + 1.5000000000000000d+01*HX6(0,0,0,0,-1,1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 2.0000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  - HX6(0,0,0,1, -1,1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + HX6(0,0,0,1,0,1) 
      HY6(0,-1,0,1,0,1) = 
     &  - 1.4887363572306462d+01
     &  - 6.2635583596698136d+00*HX1(0)
     &  + 5.4116161685556909d-01*HX1(0)*HX1(0)
     &  + 4.0068563438653142d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.6449340668482264d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 2.4041138063191885d+00*HX1(0)*HX2(0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,1)
     &  - 6.5797362673929057d+00*HX1(0)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  - 1.0823232337111381d+00*HX2(0,-1)
     &  - 4.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  + 4.8082276126383771d+00*HX3(0,0,-1)
     &  + 3.2898681336964528d+00*HX4(0,-1,0,1)
     &  + 9.8696044010893586d+00*HX4(0,0,0,-1)
     &  - 3.2898681336964528d+00*HX4(0,0,0,1)
     &  - HX6(0, -1,0,1,0,1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  + 9.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 1.6000000000000000d+01*HX6(0,0,0,0,-1,1)
     &  - 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 1.6000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + HX6(0,0,0,1,0,1) 
      HY6(0,-1,0,1,1,-1) = 
     &  - 1.3427826316361445d+00
     &  - 2.8931925377108745d+00*HX1(0)
     &  - 7.5009671202303939d-02*HX1(0)*HX1(0)
     &  + 6.7983609293803206d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 4.0790165576281924d+00*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX5(0,-1,0,1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + HX1(0) *HX5(0,0,0,1,1)
     &  - HX1(0) *HX5(0,0,1,0,-1)
     &  + 1.5001934240460787d-01*HX2(0,-1)
     &  + HX2(0, -1)*HX4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,-1,1)
     &  + HX2(0, -1)*HX4(0,0,0,-1)
     &  - HX2(0, -1)*HX4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,1)
     &  + 8.1580331152563848d+00*HX3(0,0,-1)
     &  - HX3(0,0, -1)*HX3(0,0,-1)
     &  - 2.4674011002723396d+00*HX4(0,-1,0,1)
     &  - 7.4022033008170189d+00*HX4(0,0,0,-1)
     &  + 2.4674011002723396d+00*HX4(0,0,0,1)
     &  - 2.0000000000000000d+00*HX6(0,-1,0,-1,0,1)
     &  + HX6(0, -1,0,1,0,1)
     &  + HX6(0, -1,0,1,1,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,-1,-1,0,1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,1,0,-1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 9.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - HX6(0,0,0,0,1, -1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - HX6(0,0,0,1,0, -1)
     &  - HX6(0,0,0,1,0,1) 
     &  - 7.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  + HX6(0,0,1, -1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,-1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,0,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,1,0,-1)
      HY6(0,-1,0,1,1,1) = 
     &  + 6.0013585748023513d+00
     &  - 1.7203041663629827d+00*HX1(0)
     &  - 2.5705176800639532d+00*HX1(0)*HX1(0)
     &  + 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,0,1)
     &  + 9.8696044010893586d+00*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX5(0,-1,0,1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - HX1(0) *HX5(0,0,0,1,1)
     &  + HX1(0) *HX5(0,0,1,0,-1)
     &  + 5.1410353601279064d+00*HX2(0,-1)
     &  + HX2(0, -1)*HX4(0,0,0,1)
     &  - HX2(0, -1)*HX4(0,0,1,1)
     &  - 4.9348022005446793d+00*HX4(0,-1,0,1)
     &  - 1.4804406601634037d+01*HX4(0,0,0,-1)
     &  + 4.9348022005446793d+00*HX4(0,0,0,1)
     &  + HX6(0, -1,0,1,1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - HX6(0,0,0,1,0, -1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  - HX6(0,0,0,1,1,1) 
     &  + HX6(0,0,1,0, -1,1)
     &  + HX6(0,0,1,0,1, -1)
     &  + HX6(0,0,1,1,0, -1)
      HY6(0,-1,1,-1,-1,-1) = 
     &  - 2.2064269968047850d+01
     &  - 8.7603480704641757d+00*HX1(0)
     &  - 1.2139314033773515d+00*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,1,-1)
     &  + HX1(0) *HX1(0)*HX4(0,0,-1,-1)
     &  - HX1(0) *HX1(0)*HX4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,0,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,0,1)
     &  + HX1(0) *HX5(0,-1,0,-1,-1)
     &  + HX1(0) *HX5(0,-1,0,1,-1)
     &  + HX1(0) *HX5(0,-1,1,-1,-1)
     &  - HX1(0) *HX5(0,0,-1,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,-1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - HX1(0) *HX5(0,0,1,-1,-1)
     &  - HX1(0) *HX5(0,0,1,0,-1)
     &  + 2.4278628067547031d+00*HX2(0,-1)
     &  - HX2(0, -1)*HX4(0,0,-1,-1)
     &  - HX2(0, -1)*HX4(0,0,0,-1)
     &  - HX2(0, -1)*HX4(0,0,0,1)
     &  - HX2(0, -1)*HX4(0,0,1,-1)
     &  - HX6(0, -1,0,-1,-1,-1)
     &  - HX6(0, -1,0,1,-1,-1)
     &  - HX6(0, -1,1,-1,-1,-1)
     &  + HX6(0,0, -1,-1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,-1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,1,-1,-1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,-1,-1,-1)
     &  + HX6(0,0,0, -1,0,-1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,-1,-1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  + 9.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  + HX6(0,0,0,1,0, -1)
     &  + HX6(0,0,1, -1,-1,-1)
     &  + HX6(0,0,1, -1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,-1,-1)
      HY6(0,-1,1,-1,-1,1) = 
     &  + 7.2722860530274283d+00
     &  - 4.7841885869789657d-01*HX1(0)
     &  - 1.5220170474062880d+00*HX1(0)*HX1(0)
     &  - 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  + 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,1,-1)
     &  - HX1(0) *HX1(0)*HX4(0,0,-1,-1)
     &  + HX1(0) *HX1(0)*HX4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  + 2.7620719062289241d+00*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,-1,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,0,1)
     &  - HX1(0) *HX5(0,-1,0,-1,-1)
     &  - HX1(0) *HX5(0,-1,0,1,-1)
     &  - HX1(0) *HX5(0,-1,1,-1,-1)
     &  + HX1(0) *HX5(0,0,-1,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,-1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + HX1(0) *HX5(0,0,1,-1,-1)
     &  + HX1(0) *HX5(0,0,1,0,-1)
     &  + 3.0440340948125761d+00*HX2(0,-1)
     &  - 1.2337005501361698d+00*HX2(0,-1)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX2(0,-1)*HX2(0,-1)*HX2(0,-1)
     &  + HX2(0, -1)*HX4(0,-1,0,1)
     &  + HX2(0, -1)*HX4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,1)
     &  - HX3(0, -1,-1)*HX3(0,-1,1)
     &  + 2.7620719062289241d+00*HX3(0,-1,1)
     &  + 5.0000000000000000d-01*HX3(0,-1,1)*HX3(0,-1,1)
     &  - HX3(0, -1,1)*HX3(0,0,-1)
     &  - HX3(0, -1,1)*HX3(0,0,1)
     &  - 5.5241438124578482d+00*HX3(0,0,-1)
     &  + HX3(0,0, -1)*HX3(0,0,-1)
     &  + HX3(0,0, -1)*HX3(0,0,1)
     &  - 2.7620719062289241d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX4(0,-1,0,1)
     &  - 2.4674011002723396d+00*HX4(0,-1,1,-1)
     &  + 4.9348022005446793d+00*HX4(0,0,-1,-1)
     &  - 4.9348022005446793d+00*HX4(0,0,-1,1)
     &  + 9.8696044010893586d+00*HX4(0,0,0,-1)
     &  + 7.4022033008170189d+00*HX4(0,0,0,1)
     &  + 2.4674011002723396d+00*HX4(0,0,1,-1)
     &  + HX6(0, -1,-1,0,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,-1,0,-1,-1,1)
     &  - HX6(0, -1,0,-1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,-1,0,-1,1,1)
     &  - HX6(0, -1,0,1,-1,1)
     &  + HX6(0, -1,0,1,0,1)
     &  - HX6(0, -1,1,-1,-1,1)
     &  + HX6(0, -1,1,-1,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,-1,-1,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,-1,0,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,-1,-1,0,1)
     &  - 4.0000000000000000d+00*HX6(0,0,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,-1,-1)
     &  + HX6(0,0, -1,0,-1,1)
     &  + HX6(0,0, -1,0,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,1,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,-1,-1,-1)
     &  + 7.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  - HX6(0,0,0, -1,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  - 1.6000000000000000d+01*HX6(0,0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  + HX6(0,0,1, -1,-1,1)
     &  - HX6(0,0,1, -1,0,-1)
     &  - HX6(0,0,1, -1,0,1)
     &  - HX6(0,0,1,0, -1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,0,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,1,0,-1)
      HY6(0,-1,1,-1,0,1) = 
     &  + 3.6846349707958423d+00
     &  - 1.9229671885315937d+00*HX1(0)
     &  - 1.5394314519696710d+00*HX1(0)*HX1(0)
     &  - 3.0051422578989857d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  + 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 1.6449340668482264d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,1,-1)
     &  - HX1(0) *HX1(0)*HX4(0,0,-1,-1)
     &  + HX1(0) *HX1(0)*HX4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  + 1.8030853547393914d+00*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,0,1)
     &  + 3.2898681336964528d+00*HX1(0)*HX3(0,-1,1)
     &  - 6.5797362673929057d+00*HX1(0)*HX3(0,0,-1)
     &  - 3.2898681336964528d+00*HX1(0)*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,-1,0,-1,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,-1)
     &  + 1.2000000000000000d+01*HX1(0)*HX5(0,0,0,-1,-1)
     &  + 8.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  + 3.0788629039393420d+00*HX2(0,-1)
     &  - 1.6449340668482264d+00*HX2(0,-1)*HX2(0,-1)
     &  - HX2(0, -1)*HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,1)
     &  + 1.8030853547393914d+00*HX3(0,-1,1)
     &  + HX3(0, -1,1)*HX3(0,0,-1)
     &  - HX3(0, -1,1)*HX3(0,0,1)
     &  - 3.6061707094787828d+00*HX3(0,0,-1)
     &  - HX3(0,0, -1)*HX3(0,0,-1)
     &  - HX3(0,0, -1)*HX3(0,0,1)
     &  - 1.8030853547393914d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  - 3.2898681336964528d+00*HX4(0,-1,0,1)
     &  - 3.2898681336964528d+00*HX4(0,-1,1,-1)
     &  + 6.5797362673929057d+00*HX4(0,0,-1,-1)
     &  - 6.5797362673929057d+00*HX4(0,0,-1,1)
     &  + 1.3159472534785811d+01*HX4(0,0,0,-1)
     &  + 9.8696044010893586d+00*HX4(0,0,0,1)
     &  + 3.2898681336964528d+00*HX4(0,0,1,-1)
     &  + HX6(0, -1,0,-1,0,1)
     &  + HX6(0, -1,0,1,0,1)
     &  + HX6(0, -1,1,-1,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  + HX6(0,0, -1,0,1,-1)
     &  + HX6(0,0, -1,1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,0,-1,-1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,0,-1,1)
     &  - 1.5000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  - HX6(0,0,1, -1,0,1)
     &  - HX6(0,0,1,0, -1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,1,0,-1)
      HY6(0,-1,1,-1,1,-1) = 
     &  + 3.7958254394905744d+01
     &  + 1.9422721683598428d+01*HX1(0)
     &  + 4.7368434057856041d+00*HX1(0)*HX1(0)
     &  + 9.2069063540964137d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  + 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,1,-1)
     &  - HX1(0) *HX1(0)*HX4(0,0,-1,-1)
     &  + HX1(0) *HX1(0)*HX4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  - 5.5241438124578482d+00*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,-1,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,-1,0,-1,1)
     &  + HX1(0) *HX5(0,-1,1,-1,1)
     &  - HX1(0) *HX5(0,-1,1,0,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,1)
     &  + 1.2000000000000000d+01*HX1(0)*HX5(0,0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 8.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 9.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - HX1(0) *HX5(0,0,1,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  + HX1(0) *HX5(0,0,1,0,1)
     &  - 9.4736868115712082d+00*HX2(0,-1)
     &  + 1.2337005501361698d+00*HX2(0,-1)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX2(0,-1)*HX2(0,-1)*HX2(0,-1)
     &  - HX2(0, -1)*HX4(0,-1,0,1)
     &  - HX2(0, -1)*HX4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,-1,1)
     &  + 4.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,-1)
     &  + 4.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,-1)
     &  - 4.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,1)
     &  + 2.0000000000000000d+00*HX3(0,-1,-1)*HX3(0,-1,1)
     &  - 5.5241438124578482d+00*HX3(0,-1,1)
     &  - 5.0000000000000000d-01*HX3(0,-1,1)*HX3(0,-1,1)
     &  + 2.0000000000000000d+00*HX3(0,-1,1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX3(0,-1,1)*HX3(0,0,1)
     &  + 1.1048287624915696d+01*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,1)
     &  + 5.5241438124578482d+00*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX4(0,-1,0,1)
     &  + 2.4674011002723396d+00*HX4(0,-1,1,-1)
     &  - 4.9348022005446793d+00*HX4(0,0,-1,-1)
     &  + 4.9348022005446793d+00*HX4(0,0,-1,1)
     &  - 9.8696044010893586d+00*HX4(0,0,0,-1)
     &  - 7.4022033008170189d+00*HX4(0,0,0,1)
     &  - 2.4674011002723396d+00*HX4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,-1,-1,0,-1,1)
     &  - 6.0000000000000000d+00*HX6(0,-1,0,-1,-1,1)
     &  + HX6(0, -1,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,-1,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,-1,0,-1,1,1)
     &  - HX6(0, -1,0,1,0,1)
     &  - HX6(0, -1,1,-1,0,1)
     &  - HX6(0, -1,1,-1,1,-1)
     &  + HX6(0, -1,1,0,1,-1)
     &  - 1.2000000000000000d+01*HX6(0,0,-1,-1,-1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,-1,0,1)
     &  - 4.0000000000000000d+00*HX6(0,0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,-1,0,-1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,1,0,1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  - 8.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 1.6000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - 1.5000000000000000d+01*HX6(0,0,0,0,0,1)
     &  - 1.1000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,0,1,1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,1,-1,-1)
     &  + 9.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - HX6(0,0,0,1,0, -1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,1,1,-1)
     &  - HX6(0,0,1, -1,0,-1)
     &  + HX6(0,0,1, -1,0,1)
     &  + HX6(0,0,1, -1,1,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,1,0,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,1,0,-1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,1,1,0,-1)
      HY6(0,-1,1,-1,1,1) = 
     &  - 1.9688720300662216d+00
     &  + 7.7439193717015657d-01*HX1(0)
     &  + 4.2696785175273867d-01*HX1(0)*HX1(0)
     &  + 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,1,-1)
     &  + HX1(0) *HX1(0)*HX4(0,0,-1,-1)
     &  - HX1(0) *HX1(0)*HX4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  - 2.7620719062289241d+00*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,-1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,-1,1)
     &  + 9.8696044010893586d+00*HX1(0)*HX3(0,0,-1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,-1,0,-1,1)
     &  - HX1(0) *HX5(0,-1,1,-1,1)
     &  + HX1(0) *HX5(0,-1,1,0,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,1)
     &  - 1.2000000000000000d+01*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 8.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 9.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + HX1(0) *HX5(0,0,1,-1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  - HX1(0) *HX5(0,0,1,0,1)
     &  - 8.5393570350547734d-01*HX2(0,-1)
     &  + 2.4674011002723396d+00*HX2(0,-1)*HX2(0,-1)
     &  + 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,-1,1)
     &  - 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  + 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,1)
     &  - 2.7620719062289241d+00*HX3(0,-1,1)
     &  + 5.0000000000000000d-01*HX3(0,-1,1)*HX3(0,-1,1)
     &  - HX3(0, -1,1)*HX3(0,0,-1)
     &  - HX3(0, -1,1)*HX3(0,0,1)
     &  + 5.5241438124578482d+00*HX3(0,0,-1)
     &  + HX3(0,0, -1)*HX3(0,0,-1)
     &  + HX3(0,0, -1)*HX3(0,0,1)
     &  + 2.7620719062289241d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX4(0,-1,0,1)
     &  + 4.9348022005446793d+00*HX4(0,-1,1,-1)
     &  - 9.8696044010893586d+00*HX4(0,0,-1,-1)
     &  + 9.8696044010893586d+00*HX4(0,0,-1,1)
     &  - 1.9739208802178717d+01*HX4(0,0,0,-1)
     &  - 1.4804406601634037d+01*HX4(0,0,0,1)
     &  - 4.9348022005446793d+00*HX4(0,0,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,-1,0,-1,1,1)
     &  - HX6(0, -1,1,-1,1,1)
     &  + HX6(0, -1,1,0,1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,-1,-1,1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,-1,0,-1,1)
     &  + HX6(0,0, -1,0,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,1,0,-1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,-1,-1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 1.5000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  + 1.4000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  - 7.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  + HX6(0,0,1, -1,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,1,0,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  - HX6(0,0,1,0,1,1) 
     &  - 3.0000000000000000d+00*HX6(0,0,1,1,0,-1)
      HY6(0,-1,1,0,1,-1) = 
     &  + 2.6046572213237835d+01
     &  + 1.1277725706802009d+01*HX1(0)
     &  + 2.3163450681262587d+00*HX1(0)*HX1(0)
     &  + 7.2034781821637566d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  + 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  + HX1(0) *HX1(0)*HX4(0,0,-1,1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 4.3220869092982539d+00*HX1(0)*HX2(0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,-1,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,0,1)
     &  - HX1(0) *HX5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  + HX1(0) *HX5(0,0,1,0,1)
     &  - 4.6326901362525175d+00*HX2(0,-1)
     &  - HX2(0, -1)*HX4(0,-1,0,1)
     &  + HX2(0, -1)*HX4(0,0,-1,1)
     &  + HX2(0, -1)*HX4(0,0,0,-1)
     &  + 4.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,-1)
     &  - 4.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,1)
     &  - 4.3220869092982539d+00*HX3(0,-1,1)
     &  + HX3(0, -1,1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX3(0,-1,1)*HX3(0,0,1)
     &  + 8.6441738185965079d+00*HX3(0,0,-1)
     &  - HX3(0,0, -1)*HX3(0,0,-1)
     &  - HX3(0,0, -1)*HX3(0,0,1)
     &  + 4.3220869092982539d+00*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX4(0,-1,0,1)
     &  + 4.9348022005446793d+00*HX4(0,0,-1,1)
     &  - 7.4022033008170189d+00*HX4(0,0,0,-1)
     &  - 7.4022033008170189d+00*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX6(0,-1,0,-1,0,1)
     &  - HX6(0, -1,0,1,0,1)
     &  + HX6(0, -1,1,0,1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,-1,0,1)
     &  - 4.0000000000000000d+00*HX6(0,0,-1,0,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  - 5.0000000000000000d+00*HX6(0,0,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,1,0,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,1,0,1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,-1,1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 1.5000000000000000d+01*HX6(0,0,0,0,0,1)
     &  - 1.1000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,0,1,1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,1,-1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - HX6(0,0,0,1,0, -1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,-1,0,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,1,0,-1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,-1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,1,1,0,-1)
      HY6(0,-1,1,0,1,1) = 
     &  + 5.6860223176087638d+00
     &  + 3.0220693543288129d+00*HX1(0)
     &  - 4.0587121264167682d-01*HX1(0)*HX1(0)
     &  + 2.0034281719326571d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  - HX1(0) *HX1(0)*HX4(0,0,-1,1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 1.2020569031595942d+00*HX1(0)*HX2(0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX2(0,-1)*HX3(0,0,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,-1,1)
     &  + 9.8696044010893586d+00*HX1(0)*HX3(0,0,-1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,1)
     &  + HX1(0) *HX5(0,-1,1,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  - HX1(0) *HX5(0,0,1,0,1)
     &  + 8.1174242528335364d-01*HX2(0,-1)
     &  - 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  + 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,1)
     &  - 1.2020569031595942d+00*HX3(0,-1,1)
     &  - HX3(0, -1,1)*HX3(0,0,1)
     &  + 2.4041138063191885d+00*HX3(0,0,-1)
     &  + 1.2020569031595942d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX4(0,-1,0,1)
     &  + 9.8696044010893586d+00*HX4(0,0,-1,1)
     &  - 1.4804406601634037d+01*HX4(0,0,0,-1)
     &  - 1.4804406601634037d+01*HX4(0,0,0,1)
     &  + HX6(0, -1,1,0,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,1,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  + 8.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  + 1.4000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  - 7.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,0,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  - HX6(0,0,1,0,1,1) 
     &  - 3.0000000000000000d+00*HX6(0,0,1,1,0,-1)
      HY6(0,-1,1,1,-1,-1) = 
     &  + 4.9734265649008446d+00
     &  - 2.2698161834046217d+00*HX1(0)
     &  - 2.2001483267751239d+00*HX1(0)*HX1(0)
     &  - 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  + 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,1,1)
     &  + HX1(0) *HX1(0)*HX4(0,0,-1,1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - HX1(0) *HX1(0)*HX4(0,0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,1)
     &  + 2.7620719062289241d+00*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,-1,1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,0,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,-1,0,-1,1)
     &  + HX1(0) *HX5(0,-1,0,1,1)
     &  + HX1(0) *HX5(0,-1,1,0,1)
     &  + HX1(0) *HX5(0,-1,1,1,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  - HX1(0) *HX5(0,0,1,0,1)
     &  - HX1(0) *HX5(0,0,1,1,-1)
     &  + 4.4002966535502479d+00*HX2(0,-1)
     &  - HX2(0, -1)*HX4(0,0,-1,-1)
     &  + HX2(0, -1)*HX4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  - HX2(0, -1)*HX4(0,0,1,-1)
     &  + HX2(0, -1)*HX4(0,0,1,1)
     &  - HX3(0, -1,-1)*HX3(0,-1,1)
     &  + 2.7620719062289241d+00*HX3(0,-1,1)
     &  - HX3(0, -1,1)*HX3(0,0,-1)
     &  - HX3(0, -1,1)*HX3(0,0,1)
     &  - 5.5241438124578482d+00*HX3(0,0,-1)
     &  + HX3(0,0, -1)*HX3(0,0,-1)
     &  + HX3(0,0, -1)*HX3(0,0,1)
     &  - 2.7620719062289241d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  + HX6(0, -1,-1,0,-1,1)
     &  + 3.0000000000000000d+00*HX6(0,-1,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,-1,0,-1,1,-1)
     &  - HX6(0, -1,0,1,1,-1)
     &  - HX6(0, -1,1,0,1,-1)
     &  - HX6(0, -1,1,1,-1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,-1,-1,-1,1)
     &  + HX6(0,0, -1,-1,0,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,-1,1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,0,-1,-1)
     &  + HX6(0,0, -1,0,0,1)
     &  + HX6(0,0, -1,0,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,1,-1,-1)
     &  - HX6(0,0, -1,1,0,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,1,1,-1)
     &  + 9.0000000000000000d+00*HX6(0,0,0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - HX6(0,0,0,0,0, -1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + 7.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + HX6(0,0,1, -1,0,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,1,0,-1,-1)
     &  + HX6(0,0,1,1, -1,-1)
     &  - HX6(0,0,1,1,0, -1)
      HY6(0,-1,1,1,-1,1) = 
     &  + 1.3362984919066264d+01
     &  + 1.1314107957029216d+01*HX1(0)
     &  + 2.7878585066265773d+00*HX1(0)*HX1(0)
     &  + 2.1949077523321137d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,1,1)
     &  - HX1(0) *HX1(0)*HX4(0,0,-1,1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + HX1(0) *HX1(0)*HX4(0,0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,1)
     &  - 1.3169446513992682d+00*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,-1,1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,-1,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,-1,0,-1,1)
     &  - HX1(0) *HX5(0,-1,0,1,1)
     &  - HX1(0) *HX5(0,-1,1,0,1)
     &  - HX1(0) *HX5(0,-1,1,1,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  + HX1(0) *HX5(0,0,1,0,1)
     &  + HX1(0) *HX5(0,0,1,1,-1)
     &  - 5.5757170132531547d+00*HX2(0,-1)
     &  + HX2(0, -1)*HX4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  - 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,1)
     &  - 1.3169446513992682d+00*HX3(0,-1,1)
     &  - HX3(0, -1,1)*HX3(0,-1,1)
     &  + 2.0000000000000000d+00*HX3(0,-1,1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX3(0,-1,1)*HX3(0,0,1)
     &  + 2.6338893027985365d+00*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,1)
     &  + 1.3169446513992682d+00*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX4(0,-1,1,1)
     &  + 4.9348022005446793d+00*HX4(0,0,-1,1)
     &  - 7.4022033008170189d+00*HX4(0,0,0,-1)
     &  - 4.9348022005446793d+00*HX4(0,0,0,1)
     &  + 2.4674011002723396d+00*HX4(0,0,1,1)
     &  + 2.0000000000000000d+00*HX6(0,-1,0,-1,1,1)
     &  - HX6(0, -1,1,1,-1,1)
     &  + HX6(0, -1,1,1,0,1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,-1,1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  + HX6(0,0, -1,0,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,1,-1,1)
     &  + HX6(0,0, -1,1,0,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,1,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - HX6(0,0,0,0,0,1) 
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - HX6(0,0,0,1,0, -1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  + HX6(0,0,1,1, -1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,1,0,-1)
     &  - HX6(0,0,1,1,0,1) 
      HY6(0,-1,1,1,0,1) = 
     &  + 1.8686138570700015d+01
     &  + 1.1542069134601098d+01*HX1(0)
     &  + 4.0587121264167682d-01*HX1(0)*HX1(0)
     &  - 4.0068563438653142d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.6449340668482264d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,1,1)
     &  - HX1(0) *HX1(0)*HX4(0,0,-1,1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + HX1(0) *HX1(0)*HX4(0,0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,1)
     &  + 2.4041138063191885d+00*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,0,1)
     &  - 3.2898681336964528d+00*HX1(0)*HX3(0,-1,1)
     &  + 6.5797362673929057d+00*HX1(0)*HX3(0,0,-1)
     &  + 3.2898681336964528d+00*HX1(0)*HX3(0,0,1)
     &  - HX1(0) *HX5(0,-1,0,1,1)
     &  - HX1(0) *HX5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  + HX1(0) *HX5(0,0,1,0,-1)
     &  + HX1(0) *HX5(0,0,1,0,1)
     &  - 8.1174242528335364d-01*HX2(0,-1)
     &  + 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,0,1)
     &  - 3.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,1)
     &  + 2.4041138063191885d+00*HX3(0,-1,1)
     &  + 2.0000000000000000d+00*HX3(0,-1,1)*HX3(0,0,1)
     &  - 4.8082276126383771d+00*HX3(0,0,-1)
     &  - 2.4041138063191885d+00*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,0,1)
     &  - 3.2898681336964528d+00*HX4(0,-1,1,1)
     &  + 6.5797362673929057d+00*HX4(0,0,-1,1)
     &  - 9.8696044010893586d+00*HX4(0,0,0,-1)
     &  - 6.5797362673929057d+00*HX4(0,0,0,1)
     &  + 3.2898681336964528d+00*HX4(0,0,1,1)
     &  + HX6(0, -1,1,1,0,1)
     &  - 4.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,1,0,1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 1.6000000000000000d+01*HX6(0,0,0,0,-1,1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - HX6(0,0,0,0,0,1) 
     &  - 1.2000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  + 9.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  + HX6(0,0,1,0, -1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,1,1,0,-1)
     &  - HX6(0,0,1,1,0,1) 
      HY6(0,-1,1,1,1,-1) = 
     &  - 1.1047277650877332d+01
     &  - 9.7217546653543322d-01*HX1(0)
     &  + 2.8461108206419650d-02*HX1(0)*HX1(0)
     &  - 6.7983609293803206d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,1,1)
     &  - HX1(0) *HX1(0)*HX4(0,0,-1,1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + HX1(0) *HX1(0)*HX4(0,0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,1)
     &  + 4.0790165576281924d+00*HX1(0)*HX2(0,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,-1,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,0,1)
     &  + HX1(0) *HX5(0,-1,1,1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  - HX1(0) *HX5(0,0,1,1,1)
     &  - 5.6922216412839301d-02*HX2(0,-1)
     &  - HX2(0, -1)*HX4(0,-1,1,1)
     &  + HX2(0, -1)*HX4(0,0,-1,1)
     &  - HX2(0, -1)*HX4(0,0,0,-1)
     &  - HX2(0, -1)*HX4(0,0,0,1)
     &  + HX2(0, -1)*HX4(0,0,1,1)
     &  + 4.0790165576281924d+00*HX3(0,-1,1)
     &  + 5.0000000000000000d-01*HX3(0,-1,1)*HX3(0,-1,1)
     &  - HX3(0, -1,1)*HX3(0,0,-1)
     &  - HX3(0, -1,1)*HX3(0,0,1)
     &  - 8.1580331152563848d+00*HX3(0,0,-1)
     &  + HX3(0,0, -1)*HX3(0,0,-1)
     &  + HX3(0,0, -1)*HX3(0,0,1)
     &  - 4.0790165576281924d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX4(0,-1,1,1)
     &  - 4.9348022005446793d+00*HX4(0,0,-1,1)
     &  + 7.4022033008170189d+00*HX4(0,0,0,-1)
     &  + 4.9348022005446793d+00*HX4(0,0,0,1)
     &  - 2.4674011002723396d+00*HX4(0,0,1,1)
     &  - HX6(0, -1,0,1,1,1)
     &  - HX6(0, -1,1,0,1,1)
     &  - HX6(0, -1,1,1,0,1)
     &  - HX6(0, -1,1,1,1,-1)
     &  + HX6(0,0, -1,0,0,1)
     &  + HX6(0,0, -1,0,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,1,1)
     &  + HX6(0,0, -1,1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,1,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,1,1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - HX6(0,0,0,0,1, -1)
     &  - 8.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - HX6(0,0,0,1,0, -1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,1,1)
     &  + HX6(0,0,1,0,1,1) 
     &  + HX6(0,0,1,1,0,1) 
     &  + HX6(0,0,1,1,1, -1)
      HY6(0,-1,1,1,1,1) = 
     &  - 1.0528637024782133d+01
     &  - 2.1900870176160439d+00*HX1(0)
     &  + 2.0293560632083841d+00*HX1(0)*HX1(0)
     &  - 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  + 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,1,1)
     &  + HX1(0) *HX1(0)*HX4(0,0,-1,1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - HX1(0) *HX1(0)*HX4(0,0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,-1,1)
     &  - 9.8696044010893586d+00*HX1(0)*HX3(0,0,-1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,1)
     &  - HX1(0) *HX5(0,-1,1,1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  + HX1(0) *HX5(0,0,1,1,1)
     &  - 4.0587121264167682d+00*HX2(0,-1)
     &  + 4.9348022005446793d+00*HX4(0,-1,1,1)
     &  - 9.8696044010893586d+00*HX4(0,0,-1,1)
     &  + 1.4804406601634037d+01*HX4(0,0,0,-1)
     &  + 9.8696044010893586d+00*HX4(0,0,0,1)
     &  - 4.9348022005446793d+00*HX4(0,0,1,1)
     &  - HX6(0, -1,1,1,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,1,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,1,1,1)
     &  + HX6(0,0,1,1,1,1) 
      HY6(0,0,-1,-1,-1,-1) = 
     &  - 4.0288353506662768d+00
     &  - 1.0369277551433699d+00*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  + HX1(0) *HX1(0)*HX4(0,0,0,-1)
     &  - HX1(0) *HX5(0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + HX6(0,0, -1,-1,-1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,0,-1)
      HY6(0,0,-1,-1,-1,1) = 
     &  + 3.3125433143231421d+00
     &  + 2.1501021594785064d+00*HX1(0)
     &  + 1.2139314033773515d+00*HX1(0)*HX1(0)
     &  + 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  - HX1(0) *HX1(0)*HX4(0,0,0,-1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX5(0,0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 2.7620719062289241d+00*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX4(0,0,-1,-1)
     &  + 4.9348022005446793d+00*HX4(0,0,0,-1)
     &  + HX6(0,0, -1,-1,-1,1)
     &  - HX6(0,0, -1,-1,0,-1)
     &  - HX6(0,0, -1,-1,0,1)
     &  - HX6(0,0, -1,0,-1,-1)
     &  - HX6(0,0, -1,0,-1,1)
     &  + HX6(0,0, -1,0,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,-1,-1)
     &  - HX6(0,0,0, -1,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  + HX6(0,0,0, -1,0,1)
     &  - 8.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + HX6(0,0,0,0, -1,1)
     &  - 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - HX6(0,0,0,0,0,1) 
      HY6(0,0,-1,-1,0,-1) = 
     &  - 6.3954420769594533d-01
     &  + 2.3992170678795373d+00*HX1(0)
     &  + 1.6234848505667072d+00*HX1(0)*HX1(0)
     &  + 4.0068563438653142d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 6.8538919452009434d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  + HX1(0) *HX1(0)*HX4(0,0,0,-1)
     &  - 1.6449340668482264d+00*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX5(0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 2.4041138063191885d+00*HX3(0,0,-1)
     &  + HX3(0,0, -1)*HX3(0,0,-1)
     &  + 1.6449340668482264d+00*HX4(0,0,-1,-1)
     &  + 3.2898681336964528d+00*HX4(0,0,0,-1)
     &  - HX6(0,0, -1,-1,0,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - HX6(0,0,0,0,0, -1)
      HY6(0,0,-1,-1,0,1) = 
     &  - 3.6058686066616725d+00
     &  - 2.6781232869596824d+00*HX1(0)
     &  - 1.0146780316041920d-01*HX1(0)*HX1(0)
     &  + 3.0051422578989857d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  - HX1(0) *HX1(0)*HX4(0,0,0,-1)
     &  - 3.2898681336964528d+00*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX5(0,0,-1,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 1.8030853547393914d+00*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  + 3.2898681336964528d+00*HX4(0,0,-1,-1)
     &  + 6.5797362673929057d+00*HX4(0,0,0,-1)
     &  - HX6(0,0, -1,-1,0,1)
     &  + HX6(0,0, -1,0,0,1)
     &  + HX6(0,0,0, -1,0,1)
     &  - HX6(0,0,0,0,0,1) 
      HY6(0,0,-1,-1,1,-1) = 
     &  - 5.1251751312572332d+00
     &  - 7.5170363885043517d+00*HX1(0)
     &  - 3.6417942101320546d+00*HX1(0)*HX1(0)
     &  - 9.2069063540964137d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  - HX1(0) *HX1(0)*HX4(0,0,0,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX5(0,0,-1,-1,1)
     &  + HX1(0) *HX5(0,0,-1,0,-1)
     &  + HX1(0) *HX5(0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  + HX1(0) *HX5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - HX1(0) *HX5(0,0,0,0,1)
     &  + 5.5241438124578482d+00*HX3(0,0,-1)
     &  - HX3(0,0, -1)*HX3(0,0,-1)
     &  - 2.4674011002723396d+00*HX4(0,0,-1,-1)
     &  - 4.9348022005446793d+00*HX4(0,0,0,-1)
     &  + HX6(0,0, -1,-1,0,-1)
     &  + HX6(0,0, -1,-1,0,1)
     &  + HX6(0,0, -1,-1,1,-1)
     &  + HX6(0,0, -1,0,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  - HX6(0,0, -1,0,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - HX6(0,0,0, -1,1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + HX6(0,0,0,0,0, -1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  + HX6(0,0,0,0,1, -1)
      HY6(0,0,-1,-1,1,1) = 
     &  + 1.0559546559664610d+01
     &  + 6.8749092010391177d+00*HX1(0)
     &  + 1.0146780316041920d+00*HX1(0)*HX1(0)
     &  - 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  + HX1(0) *HX1(0)*HX4(0,0,0,-1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX5(0,0,-1,-1,1)
     &  - HX1(0) *HX5(0,0,-1,0,-1)
     &  - HX1(0) *HX5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  - HX1(0) *HX5(0,0,0,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + HX1(0) *HX5(0,0,0,0,1)
     &  + 2.7620719062289241d+00*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  - 4.9348022005446793d+00*HX4(0,0,-1,-1)
     &  - 9.8696044010893586d+00*HX4(0,0,0,-1)
     &  + HX6(0,0, -1,-1,1,1)
     &  - HX6(0,0, -1,0,-1,1)
     &  + HX6(0,0, -1,0,0,1)
     &  - HX6(0,0, -1,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - HX6(0,0,0, -1,1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  + HX6(0,0,0,0,1,1) 
      HY6(0,0,-1,0,-1,-1) = 
     &  - 1.4649205957106797d+01
     &  - 7.4873046836069432d+00*HX1(0)
     &  - 1.6234848505667072d+00*HX1(0)*HX1(0)
     &  - 2.0034281719326571d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + HX1(0) *HX5(0,0,-1,0,-1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 1.2020569031595942d+00*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  - HX6(0,0, -1,0,-1,-1)
     &  + 7.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + 1.4000000000000000d+01*HX6(0,0,0,0,0,-1)
      HY6(0,0,-1,0,-1,1) = 
     &  + 1.0598392014996814d+01
     &  + 5.6642178198849788d+00*HX1(0)
     &  + 1.7423670582905094d+00*HX1(0)*HX1(0)
     &  + 4.1983359242647709d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX5(0,0,-1,0,-1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 2.5190015545588625d+00*HX3(0,0,-1)
     &  + HX3(0,0, -1)*HX3(0,0,-1)
     &  + 7.4022033008170189d+00*HX4(0,0,0,-1)
     &  - HX6(0,0, -1,0,-1,1)
     &  + HX6(0,0, -1,0,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,0,-1,-1)
     &  + HX6(0,0,0,0, -1,1)
     &  - 1.5000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - HX6(0,0,0,0,0,1) 
      HY6(0,0,-1,0,0,1) = 
     &  + 1.5260145929766737d+00
     &  - 1.3837838345125395d+00*HX1(0)
     &  - 3.3822601053473068d-01*HX1(0)*HX1(0)
     &  + 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 3.2898681336964528d+00*HX1(0)*HX3(0,0,-1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 9.8696044010893586d+00*HX4(0,0,0,-1)
     &  + HX6(0,0, -1,0,0,1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - HX6(0,0,0,0,0,1) 
      HY6(0,0,-1,0,1,-1) = 
     &  + 7.3449101073107902d+00
     &  + 3.4903562040510891d-01*HX1(0)
     &  - 1.5394314519696710d+00*HX1(0)*HX1(0)
     &  - 7.2034781821637566d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX5(0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - HX1(0) *HX5(0,0,0,0,1)
     &  + 4.3220869092982539d+00*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  - 7.4022033008170189d+00*HX4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  - HX6(0,0, -1,0,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  + HX6(0,0,0,0,1, -1)
      HY6(0,0,-1,0,1,1) = 
     &  + 6.8592469402994421d+00
     &  + 6.6327319875542747d+00*HX1(0)
     &  + 1.9278882600479649d+00*HX1(0)*HX1(0)
     &  - 2.0034281719326571d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX5(0,0,-1,0,1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + HX1(0) *HX5(0,0,0,0,1)
     &  + 1.2020569031595942d+00*HX3(0,0,-1)
     &  - 1.4804406601634037d+01*HX4(0,0,0,-1)
     &  + HX6(0,0, -1,0,0,1)
     &  - HX6(0,0, -1,0,1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  + HX6(0,0,0,0,1,1) 
      HY6(0,0,-1,1,-1,-1) = 
     &  + 2.5906253470605985d+01
     &  + 1.4571792178347941d+01*HX1(0)
     &  + 3.6417942101320546d+00*HX1(0)*HX1(0)
     &  + 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX5(0,0,-1,0,-1)
     &  - HX1(0) *HX5(0,0,-1,0,1)
     &  - HX1(0) *HX5(0,0,-1,1,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + HX1(0) *HX5(0,0,0,1,-1)
     &  - 2.7620719062289241d+00*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  + HX6(0,0, -1,0,-1,-1)
     &  + HX6(0,0, -1,0,0,1)
     &  + HX6(0,0, -1,0,1,-1)
     &  + HX6(0,0, -1,1,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  - 7.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 1.4000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,0,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - HX6(0,0,0,1, -1,-1)
      HY6(0,0,-1,1,-1,1) = 
     &  - 1.7084462264728741d+01
     &  - 8.4053642852001231d+00*HX1(0)
     &  - 1.6928093109730279d+00*HX1(0)*HX1(0)
     &  - 2.1949077523321137d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX5(0,0,-1,0,-1)
     &  + HX1(0) *HX5(0,0,-1,0,1)
     &  + HX1(0) *HX5(0,0,-1,1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - HX1(0) *HX5(0,0,0,1,-1)
     &  + 1.3169446513992682d+00*HX3(0,0,-1)
     &  - HX3(0,0, -1)*HX3(0,0,-1)
     &  + 2.4674011002723396d+00*HX4(0,0,-1,1)
     &  - 7.4022033008170189d+00*HX4(0,0,0,-1)
     &  - 2.4674011002723396d+00*HX4(0,0,0,1)
     &  + HX6(0,0, -1,0,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  - HX6(0,0, -1,0,1,-1)
     &  + HX6(0,0, -1,1,-1,1)
     &  - HX6(0,0, -1,1,0,-1)
     &  - HX6(0,0, -1,1,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,0,-1,-1)
     &  - 7.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 1.5000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - HX6(0,0,0,1, -1,1)
     &  + HX6(0,0,0,1,0, -1)
     &  + HX6(0,0,0,1,0,1) 
      HY6(0,0,-1,1,0,-1) = 
     &  + 2.6533493449450748d+01
     &  + 1.4568131614372025d+01*HX1(0)
     &  + 3.5687875151780551d+00*HX1(0)*HX1(0)
     &  + 3.0051422578989857d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 6.8538919452009434d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 1.6449340668482264d+00*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 1.8030853547393914d+00*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  + 1.6449340668482264d+00*HX4(0,0,-1,1)
     &  - 4.9348022005446793d+00*HX4(0,0,0,-1)
     &  - 1.6449340668482264d+00*HX4(0,0,0,1)
     &  + HX6(0,0, -1,0,0,1)
     &  - HX6(0,0, -1,1,0,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + HX6(0,0,0,1,0, -1)
      HY6(0,0,-1,1,0,1) = 
     &  - 5.8535488151112357d+00
     &  + 4.5667800194983164d-01*HX1(0)
     &  + 1.7423670582905094d+00*HX1(0)*HX1(0)
     &  + 4.0068563438653142d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 3.2898681336964528d+00*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX5(0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 2.4041138063191885d+00*HX3(0,0,-1)
     &  + 3.2898681336964528d+00*HX4(0,0,-1,1)
     &  - 9.8696044010893586d+00*HX4(0,0,0,-1)
     &  - 3.2898681336964528d+00*HX4(0,0,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  - HX6(0,0, -1,1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + HX6(0,0,0,1,0,1) 
      HY6(0,0,-1,1,1,-1) = 
     &  - 1.9389723591276614d+01
     &  - 8.0778313134277241d+00*HX1(0)
     &  - 3.3654675223535616d-01*HX1(0)*HX1(0)
     &  + 6.7983609293803206d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX5(0,0,-1,1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + HX1(0) *HX5(0,0,0,1,1)
     &  - 4.0790165576281924d+00*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  - 2.4674011002723396d+00*HX4(0,0,-1,1)
     &  + 7.4022033008170189d+00*HX4(0,0,0,-1)
     &  + 2.4674011002723396d+00*HX4(0,0,0,1)
     &  + HX6(0,0, -1,0,0,1)
     &  + HX6(0,0, -1,0,1,-1)
     &  + HX6(0,0, -1,0,1,1)
     &  + HX6(0,0, -1,1,0,-1)
     &  + HX6(0,0, -1,1,0,1)
     &  + HX6(0,0, -1,1,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 9.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - HX6(0,0,0,0,1, -1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - HX6(0,0,0,1,0, -1)
     &  - HX6(0,0,0,1,0,1) 
     &  - HX6(0,0,0,1,1, -1)
      HY6(0,0,-1,1,1,1) = 
     &  - 3.0821994922870848d+00
     &  - 6.9367501878022481d+00*HX1(0)
     &  - 3.2432874665857356d+00*HX1(0)*HX1(0)
     &  + 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX5(0,0,-1,1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - HX1(0) *HX5(0,0,0,1,1)
     &  - 4.9348022005446793d+00*HX4(0,0,-1,1)
     &  + 1.4804406601634037d+01*HX4(0,0,0,-1)
     &  + 4.9348022005446793d+00*HX4(0,0,0,1)
     &  + HX6(0,0, -1,1,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - HX6(0,0,0,1,1,1) 
      HY6(0,0,0,-1,-1,-1) = 
     &  + 5.9999375332611470d+00
     &  + 3.0142321054406660d+00*HX1(0)
     &  + 5.4116161685556909d-01*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + HX1(0) *HX5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - HX6(0,0,0, -1,-1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,0,-1)
      HY6(0,0,0,-1,-1,1) = 
     &  - 1.8068203871889946d+00
     &  + 9.1165552893555038d-01*HX1(0)
     &  + 1.2604799663732358d+00*HX1(0)*HX1(0)
     &  + 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - HX1(0) *HX5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 2.4674011002723396d+00*HX4(0,0,0,-1)
     &  - HX6(0,0,0, -1,-1,1)
     &  + HX6(0,0,0, -1,0,-1)
     &  + HX6(0,0,0, -1,0,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + HX6(0,0,0,0, -1,1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - HX6(0,0,0,0,0,1) 
      HY6(0,0,0,-1,0,-1) = 
     &  + 1.2398868567935473d+01
     &  + 8.1023197211680719d+00*HX1(0)
     &  + 2.5705176800639532d+00*HX1(0)*HX1(0)
     &  + 4.0068563438653142d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 6.8538919452009434d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 1.6449340668482264d+00*HX4(0,0,0,-1)
     &  + HX6(0,0,0, -1,0,-1)
     &  - 1.1000000000000000d+01*HX6(0,0,0,0,0,-1)
      HY6(0,0,0,-1,0,1) = 
     &  - 5.7225547236625264d+00
     &  - 2.0434339691042511d+00*HX1(0)
     &  + 4.7351641474862295d-01*HX1(0)*HX1(0)
     &  + 3.0051422578989857d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 3.2898681336964528d+00*HX4(0,0,0,-1)
     &  + HX6(0,0,0, -1,0,1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - HX6(0,0,0,0,0,1) 
      HY6(0,0,0,-1,1,-1) = 
     &  - 1.8061589040981662d+01
     &  - 1.2874316759375889d+01*HX1(0)
     &  - 4.2633269910369811d+00*HX1(0)*HX1(0)
     &  - 9.2069063540964137d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + HX1(0) *HX5(0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - HX1(0) *HX5(0,0,0,0,1)
     &  + 2.4674011002723396d+00*HX4(0,0,0,-1)
     &  - HX6(0,0,0, -1,0,-1)
     &  - HX6(0,0,0, -1,0,1)
     &  - HX6(0,0,0, -1,1,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  + HX6(0,0,0,0,1, -1)
      HY6(0,0,0,-1,1,1) = 
     &  + 9.1055715509177163d+00
     &  + 4.2205881177506120d+00*HX1(0)
     &  + 3.1344713687598993d-01*HX1(0)*HX1(0)
     &  - 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - HX1(0) *HX5(0,0,0,-1,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + HX1(0) *HX5(0,0,0,0,1)
     &  + 4.9348022005446793d+00*HX4(0,0,0,-1)
     &  - HX6(0,0,0, -1,1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  + HX6(0,0,0,0,1,1) 
      HY6(0,0,0,0,-1,-1) = 
     &  - 3.9422043651897404d+00
     &  - 3.0142321054406660d+00*HX1(0)
     &  - 9.4703282949724591d-01*HX1(0)*HX1(0)
     &  - 2.0034281719326571d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1(0) *HX5(0,0,0,0,-1)
     &  + HX6(0,0,0,0, -1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,0,-1)
      HY6(0,0,0,0,-1,1) = 
     &  + 5.9450985184716246d+00
     &  + 4.8071216221292780d+00*HX1(0)
     &  + 1.9617108611014379d+00*HX1(0)*HX1(0)
     &  + 4.1983359242647709d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1(0) *HX5(0,0,0,0,-1)
     &  + HX6(0,0,0,0, -1,1)
     &  - 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - HX6(0,0,0,0,0,1) 
      HY6(0,0,0,0,0,-1) = 
     &  + 1.9711021825948702d+00
     &  + 9.4703282949724591d-01*HX1(0)*HX1(0)
     &  + 6.8538919452009434d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX6(0,0,0,0,0, -1)
      HY6(0,0,0,0,0,1) = 
     &  + 2.0346861239688982d+00
     &  + 1.0823232337111381d+00*HX1(0)*HX1(0)
     &  + 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX6(0,0,0,0,0,1) 
      HY6(0,0,0,0,1,-1) = 
     &  - 6.0722664012196808d+00
     &  - 5.2683829003001246d+00*HX1(0)
     &  - 2.0970012653153302d+00*HX1(0)*HX1(0)
     &  - 7.2034781821637566d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1(0) *HX5(0,0,0,0,1)
     &  + HX6(0,0,0,0,0, -1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  + HX6(0,0,0,0,1, -1)
      HY6(0,0,0,0,1,1) = 
     &  + 4.0693722479377965d+00
     &  + 2.9176809454512223d+00*HX1(0)
     &  + 1.0823232337111381d+00*HX1(0)*HX1(0)
     &  - 2.0034281719326571d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1(0) *HX5(0,0,0,0,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  + HX6(0,0,0,0,1,1) 
      HY6(0,0,0,1,-1,-1) = 
     &  + 1.4294589866479561d+01
     &  + 9.4096904031705199d+00*HX1(0)
     &  + 2.9013792215033260d+00*HX1(0)*HX1(0)
     &  + 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + HX1(0) *HX5(0,0,0,0,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + HX1(0) *HX5(0,0,0,1,-1)
     &  - HX6(0,0,0,0, -1,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,0,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - HX6(0,0,0,1, -1,-1)
      HY6(0,0,0,1,-1,1) = 
     &  - 1.3704005110260677d+01
     &  - 8.7596705482058876d+00*HX1(0)
     &  - 2.1663257257216508d+00*HX1(0)*HX1(0)
     &  - 2.1949077523321137d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - HX1(0) *HX5(0,0,0,1,-1)
     &  - 2.4674011002723396d+00*HX4(0,0,0,1)
     &  - HX6(0,0,0,0, -1,1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - HX6(0,0,0,1, -1,1)
     &  + HX6(0,0,0,1,0, -1)
     &  + HX6(0,0,0,1,0,1) 
      HY6(0,0,0,1,0,-1) = 
     &  + 1.0968229887019842d+01
     &  + 6.8544356072335814d+00*HX1(0)
     &  + 2.5028724779570070d+00*HX1(0)*HX1(0)
     &  + 3.0051422578989857d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 6.8538919452009434d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 1.6449340668482264d+00*HX4(0,0,0,1)
     &  + HX6(0,0,0,0,0, -1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + HX6(0,0,0,1,0, -1)
      HY6(0,0,0,1,0,1) = 
     &  - 7.6300729648833685d+00
     &  - 3.7615063806157047d+00*HX1(0)
     &  + 5.4116161685556909d-01*HX1(0)*HX1(0)
     &  + 4.0068563438653142d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 3.2898681336964528d+00*HX4(0,0,0,1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + HX6(0,0,0,1,0,1) 
      HY6(0,0,0,1,1,-1) = 
     &  - 6.3834942634739552d+00
     &  - 3.2887945813357094d+00*HX1(0)
     &  - 7.5009671202303939d-02*HX1(0)*HX1(0)
     &  + 6.7983609293803206d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + HX1(0) *HX5(0,0,0,1,1)
     &  + 2.4674011002723396d+00*HX4(0,0,0,1)
     &  - HX6(0,0,0,0,0, -1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - HX6(0,0,0,0,1, -1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - HX6(0,0,0,1,0, -1)
     &  - HX6(0,0,0,1,0,1) 
     &  - HX6(0,0,0,1,1, -1)
      HY6(0,0,0,1,1,1) = 
     &  + 6.7637631101180045d-01
     &  - 3.0142321054406660d+00*HX1(0)
     &  - 2.5705176800639532d+00*HX1(0)*HX1(0)
     &  + 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - HX1(0) *HX5(0,0,0,1,1)
     &  + 4.9348022005446793d+00*HX4(0,0,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - HX6(0,0,0,1,1,1) 
      HY6(0,0,1,-1,-1,-1) = 
     &  - 1.6398484982535364d+01
     &  - 7.4845537829591132d+00*HX1(0)
     &  - 1.2139314033773515d+00*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  - HX1(0) *HX5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - HX1(0) *HX5(0,0,1,-1,-1)
     &  + HX6(0,0,0, -1,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  + HX6(0,0,1, -1,-1,-1)
      HY6(0,0,1,-1,-1,1) = 
     &  + 6.7803525968494872d+00
     &  - 7.4122526767078634d-02*HX1(0)
     &  - 1.5220170474062880d+00*HX1(0)*HX1(0)
     &  - 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,0,1)
     &  + HX1(0) *HX5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + HX1(0) *HX5(0,0,1,-1,-1)
     &  + HX3(0,0, -1)*HX3(0,0,1)
     &  - 2.7620719062289241d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX4(0,0,0,-1)
     &  + 7.4022033008170189d+00*HX4(0,0,0,1)
     &  + 2.4674011002723396d+00*HX4(0,0,1,-1)
     &  - HX6(0,0, -1,0,0,1)
     &  + HX6(0,0,0, -1,-1,1)
     &  - HX6(0,0,0, -1,0,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - 7.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  + HX6(0,0,1, -1,-1,1)
     &  - HX6(0,0,1, -1,0,-1)
     &  - HX6(0,0,1, -1,0,1)
     &  - HX6(0,0,1,0, -1,-1)
     &  - HX6(0,0,1,0, -1,1)
      HY6(0,0,1,-1,0,-1) = 
     &  - 2.6030815229106379d+01
     &  - 1.4981117312548150d+01*HX1(0)
     &  - 3.7717231214988935d+00*HX1(0)*HX1(0)
     &  - 4.0068563438653142d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 6.8538919452009434d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  - 1.6449340668482264d+00*HX1(0)*HX3(0,0,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - HX1(0) *HX5(0,0,1,0,-1)
     &  + 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,1)
     &  - 2.4041138063191885d+00*HX3(0,0,1)
     &  + 1.6449340668482264d+00*HX4(0,0,0,-1)
     &  + 4.9348022005446793d+00*HX4(0,0,0,1)
     &  + 1.6449340668482264d+00*HX4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  - HX6(0,0,0, -1,0,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,0,-1,1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,0,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  - HX6(0,0,1, -1,0,-1)
      HY6(0,0,1,-1,0,1) = 
     &  + 7.3537647480487515d+00
     &  - 4.3692303773706989d-02*HX1(0)
     &  - 1.5394314519696710d+00*HX1(0)*HX1(0)
     &  - 3.0051422578989857d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  - 3.2898681336964528d+00*HX1(0)*HX3(0,0,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + HX1(0) *HX5(0,0,1,0,-1)
     &  - HX3(0,0, -1)*HX3(0,0,1)
     &  - 1.8030853547393914d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  + 3.2898681336964528d+00*HX4(0,0,0,-1)
     &  + 9.8696044010893586d+00*HX4(0,0,0,1)
     &  + 3.2898681336964528d+00*HX4(0,0,1,-1)
     &  + HX6(0,0, -1,0,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  - HX6(0,0,1, -1,0,1)
      HY6(0,0,1,-1,1,-1) = 
     &  + 3.1477970138136150d+01
     &  + 1.8385793928455058d+01*HX1(0)
     &  + 4.7368434057856041d+00*HX1(0)*HX1(0)
     &  + 9.2069063540964137d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,0,1)
     &  - HX1(0) *HX5(0,0,0,-1,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - HX1(0) *HX5(0,0,1,-1,1)
     &  + HX1(0) *HX5(0,0,1,0,-1)
     &  + HX1(0) *HX5(0,0,1,0,1)
     &  - 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,1)
     &  + 5.5241438124578482d+00*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX4(0,0,0,-1)
     &  - 7.4022033008170189d+00*HX4(0,0,0,1)
     &  - 2.4674011002723396d+00*HX4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  + HX6(0,0,0, -1,0,-1)
     &  + 7.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + HX6(0,0,0, -1,1,-1)
     &  + 1.6000000000000000d+01*HX6(0,0,0,0,-1,1)
     &  - 1.1000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - 1.5000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,0,1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  + HX6(0,0,1, -1,0,-1)
     &  + HX6(0,0,1, -1,0,1)
     &  + HX6(0,0,1, -1,1,-1)
     &  + HX6(0,0,1,0, -1,1)
     &  - HX6(0,0,1,0,1, -1)
      HY6(0,0,1,-1,1,1) = 
     &  - 6.7515314509572077d+00
     &  - 7.0189712804378578d-01*HX1(0)
     &  + 4.2696785175273867d-01*HX1(0)*HX1(0)
     &  + 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,1)
     &  + HX1(0) *HX5(0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + HX1(0) *HX5(0,0,1,-1,1)
     &  - HX1(0) *HX5(0,0,1,0,-1)
     &  - HX1(0) *HX5(0,0,1,0,1)
     &  + HX3(0,0, -1)*HX3(0,0,1)
     &  + 2.7620719062289241d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  - 4.9348022005446793d+00*HX4(0,0,0,-1)
     &  - 1.4804406601634037d+01*HX4(0,0,0,1)
     &  - 4.9348022005446793d+00*HX4(0,0,1,-1)
     &  - HX6(0,0, -1,0,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + HX6(0,0,0, -1,1,1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,-1,1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  + 1.4000000000000000d+01*HX6(0,0,0,0,0,1)
     &  - 7.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  + HX6(0,0,1, -1,1,1)
     &  - HX6(0,0,1,0, -1,1)
     &  - HX6(0,0,1,0,1,1) 
      HY6(0,0,1,0,-1,-1) = 
     &  - 7.1930626325130895d+00
     &  - 2.1837971993853584d+00*HX1(0)
     &  + 1.0146780316041920d-01*HX1(0)*HX1(0)
     &  + 2.0034281719326571d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + HX1(0) *HX5(0,0,0,0,-1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + HX1(0) *HX5(0,0,1,0,-1)
     &  - HX3(0,0, -1)*HX3(0,0,1)
     &  + 1.2020569031595942d+00*HX3(0,0,1)
     &  + HX6(0,0, -1,0,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - HX6(0,0,0,0, -1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - HX6(0,0,1,0, -1,-1)
      HY6(0,0,1,0,-1,1) = 
     &  - 1.5242151236338379d+00
     &  - 4.7979704085740251d+00*HX1(0)
     &  - 2.5192806744470971d+00*HX1(0)*HX1(0)
     &  - 4.1983359242647709d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,0,1)
     &  - HX1(0) *HX5(0,0,0,0,-1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - HX1(0) *HX5(0,0,1,0,-1)
     &  + 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,1)
     &  - 2.5190015545588625d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  + 7.4022033008170189d+00*HX4(0,0,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 1.3000000000000000d+01*HX6(0,0,0,0,-1,1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  - HX6(0,0,1,0, -1,1)
      HY6(0,0,1,0,1,-1) = 
     &  + 2.0030564692478731d+01
     &  + 1.0648543070067714d+01*HX1(0)
     &  + 2.3163450681262587d+00*HX1(0)*HX1(0)
     &  + 7.2034781821637566d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,0,1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + HX1(0) *HX5(0,0,1,0,1)
     &  - HX3(0,0, -1)*HX3(0,0,1)
     &  + 4.3220869092982539d+00*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,0,1)
     &  - 7.4022033008170189d+00*HX4(0,0,0,1)
     &  + HX6(0,0, -1,0,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - HX6(0,0,0,0,0, -1)
     &  - 1.5000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,0,1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  - HX6(0,0,1,0,1, -1)
      HY6(0,0,1,0,1,1) = 
     &  - 3.8684500293687428d-02
     &  + 1.5553916327150548d+00*HX1(0)
     &  - 4.0587121264167682d-01*HX1(0)*HX1(0)
     &  + 2.0034281719326571d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - HX1(0) *HX5(0,0,1,0,1)
     &  + 1.2020569031595942d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  - 1.4804406601634037d+01*HX4(0,0,0,1)
     &  + 1.4000000000000000d+01*HX6(0,0,0,0,0,1)
     &  - 7.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - HX6(0,0,1,0,1,1) 
      HY6(0,0,1,1,-1,-1) = 
     &  - 2.0197023854167984d+00
     &  - 3.7073850386582300d+00*HX1(0)
     &  - 2.2001483267751239d+00*HX1(0)*HX1(0)
     &  - 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - HX1(0) *HX1(0)*HX4(0,0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,1)
     &  - HX1(0) *HX5(0,0,0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - HX1(0) *HX5(0,0,0,1,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  - HX1(0) *HX5(0,0,1,0,-1)
     &  - HX1(0) *HX5(0,0,1,0,1)
     &  - HX1(0) *HX5(0,0,1,1,-1)
     &  + HX3(0,0, -1)*HX3(0,0,1)
     &  - 2.7620719062289241d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  - HX6(0,0, -1,0,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + HX6(0,0,0,0, -1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + HX6(0,0,0,1, -1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  + HX6(0,0,1,0, -1,-1)
     &  + HX6(0,0,1,0,1, -1)
     &  + HX6(0,0,1,1, -1,-1)
      HY6(0,0,1,1,-1,1) = 
     &  + 1.1043188676375631d+01
     &  + 1.0133791970287534d+01*HX1(0)
     &  + 2.7878585066265773d+00*HX1(0)*HX1(0)
     &  + 2.1949077523321137d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + HX1(0) *HX1(0)*HX4(0,0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,0,1)
     &  + HX1(0) *HX5(0,0,0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + HX1(0) *HX5(0,0,0,1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  + HX1(0) *HX5(0,0,1,0,-1)
     &  + HX1(0) *HX5(0,0,1,0,1)
     &  + HX1(0) *HX5(0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,1)
     &  + 1.3169446513992682d+00*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,0,1)
     &  - 4.9348022005446793d+00*HX4(0,0,0,1)
     &  + 2.4674011002723396d+00*HX4(0,0,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 1.3000000000000000d+01*HX6(0,0,0,0,-1,1)
     &  - 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - HX6(0,0,0,0,0,1) 
     &  + 8.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  + HX6(0,0,0,1, -1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  + HX6(0,0,1,0, -1,1)
     &  - HX6(0,0,1,0,1, -1)
     &  + HX6(0,0,1,1, -1,1)
     &  - HX6(0,0,1,1,0, -1)
     &  - HX6(0,0,1,1,0,1) 
      HY6(0,0,1,1,0,-1) = 
     &  + 5.9425784521572041d+00
     &  + 1.1951450242367103d+00*HX1(0)
     &  - 1.9278882600479649d+00*HX1(0)*HX1(0)
     &  - 3.0051422578989857d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 6.8538919452009434d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - HX1(0) *HX1(0)*HX4(0,0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,1)
     &  + 1.6449340668482264d+00*HX1(0)*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  - HX1(0) *HX5(0,0,1,0,1)
     &  - HX3(0,0, -1)*HX3(0,0,1)
     &  - 1.8030853547393914d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  - 3.2898681336964528d+00*HX4(0,0,0,1)
     &  + 1.6449340668482264d+00*HX4(0,0,1,1)
     &  + HX6(0,0, -1,0,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - HX6(0,0,0,0,0, -1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  - HX6(0,0,1,1,0, -1)
      HY6(0,0,1,1,0,1) = 
     &  + 1.1321646358532661d+01
     &  + 9.4646090339042393d+00*HX1(0)
     &  + 4.0587121264167682d-01*HX1(0)*HX1(0)
     &  - 4.0068563438653142d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + HX1(0) *HX1(0)*HX4(0,0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,1)
     &  + 3.2898681336964528d+00*HX1(0)*HX3(0,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  + HX1(0) *HX5(0,0,1,0,1)
     &  - 2.4041138063191885d+00*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,0,1)
     &  - 6.5797362673929057d+00*HX4(0,0,0,1)
     &  + 3.2898681336964528d+00*HX4(0,0,1,1)
     &  - HX6(0,0,0,0,0,1) 
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  - HX6(0,0,1,1,0,1) 
      HY6(0,0,1,1,1,-1) = 
     &  - 6.6325222913467024d+00
     &  - 7.7484048807851773d-01*HX1(0)
     &  + 2.8461108206419650d-02*HX1(0)*HX1(0)
     &  - 6.7983609293803206d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + HX1(0) *HX1(0)*HX4(0,0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  - HX1(0) *HX5(0,0,1,1,1)
     &  + HX3(0,0, -1)*HX3(0,0,1)
     &  - 4.0790165576281924d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX4(0,0,0,1)
     &  - 2.4674011002723396d+00*HX4(0,0,1,1)
     &  - HX6(0,0, -1,0,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + HX6(0,0,0,0,0, -1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - 5.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - 8.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  + HX6(0,0,0,1,1, -1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,1,1)
     &  + HX6(0,0,1,0,1, -1)
     &  + HX6(0,0,1,0,1,1) 
     &  + HX6(0,0,1,1,0, -1)
     &  + HX6(0,0,1,1,0,1) 
     &  + HX6(0,0,1,1,1, -1)
      HY6(0,0,1,1,1,1) = 
     &  - 5.3640981195208663d+00
     &  - 1.0369277551433699d+00*HX1(0)
     &  + 2.0293560632083841d+00*HX1(0)*HX1(0)
     &  - 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - HX1(0) *HX1(0)*HX4(0,0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  + HX1(0) *HX5(0,0,1,1,1)
     &  + 9.8696044010893586d+00*HX4(0,0,0,1)
     &  - 4.9348022005446793d+00*HX4(0,0,1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,1,1,1)
     &  + HX6(0,0,1,1,1,1) 
      HY6(0,1,-1,-1,-1,-1) = 
     &  + 9.2883013852984394d+00
     &  + 2.1900870176160439d+00*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  - HX1(0) *HX1(0)*HX4(0,0,0,-1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX1(0)*HX4(0,0,1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,-1,-1)
     &  + HX1(0) *HX5(0,0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,-1,-1)
     &  + HX1(0) *HX5(0,1,-1,-1,-1)
     &  - HX6(0,0, -1,-1,-1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 5.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,-1,-1,-1)
     &  - HX6(0,1, -1,-1,-1,-1)
      HY6(0,1,-1,-1,-1,1) = 
     &  - 4.3330771512378121d+00
     &  - 1.9527671810215909d+00*HX1(0)
     &  - 1.2139314033773515d+00*HX1(0)*HX1(0)
     &  - 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  - 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  + HX1(0) *HX1(0)*HX4(0,0,0,-1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + HX1(0) *HX1(0)*HX4(0,0,1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,-1,-1)
     &  - 2.7620719062289241d+00*HX1(0)*HX2(0,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,0,-1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,1,-1)
     &  - HX1(0) *HX5(0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,-1,-1)
     &  - HX1(0) *HX5(0,1,-1,-1,-1)
     &  + HX2(0, -1)*HX4(0,1,-1,-1)
     &  - 2.4278628067547031d+00*HX2(0,1)
     &  + HX2(0,1) *HX4(0,-1,-1,-1)
     &  + HX2(0,1) *HX4(0,0,-1,-1)
     &  + HX2(0,1) *HX4(0,0,0,-1)
     &  + HX2(0,1) *HX4(0,0,0,1)
     &  + HX2(0,1) *HX4(0,0,1,-1)
     &  + HX2(0,1) *HX4(0,1,-1,-1)
     &  - HX3(0, -1,-1)*HX3(0,1,-1)
     &  + 2.7620719062289241d+00*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,1)
     &  - HX3(0,0, -1)*HX3(0,1,-1)
     &  + 5.5241438124578482d+00*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,1,-1)
     &  + 2.7620719062289241d+00*HX3(0,1,-1)
     &  - 5.0000000000000000d-01*HX3(0,1,-1)*HX3(0,1,-1)
     &  - 2.4674011002723396d+00*HX4(0,0,-1,-1)
     &  - 4.9348022005446793d+00*HX4(0,0,0,-1)
     &  - 7.4022033008170189d+00*HX4(0,0,0,1)
     &  - 4.9348022005446793d+00*HX4(0,0,1,-1)
     &  - 2.4674011002723396d+00*HX4(0,1,-1,-1)
     &  - HX6(0, -1,-1,-1,0,1)
     &  - HX6(0, -1,-1,0,-1,1)
     &  - HX6(0, -1,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,-1,-1,1)
     &  + HX6(0,0, -1,-1,0,-1)
     &  + HX6(0,0, -1,0,-1,-1)
     &  + HX6(0,0, -1,0,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 8.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + 7.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,-1,0,-1)
     &  + HX6(0,0,1, -1,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,-1,-1)
     &  + HX6(0,0,1,0, -1,1)
     &  - HX6(0,1, -1,-1,-1,1)
      HY6(0,1,-1,-1,1,-1) = 
     &  - 1.5131600484292956d+00
     &  + 6.3367204017626693d+00*HX1(0)
     &  + 3.6417942101320546d+00*HX1(0)*HX1(0)
     &  + 9.2069063540964137d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  + 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  + HX1(0) *HX1(0)*HX4(0,0,0,-1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + HX1(0) *HX1(0)*HX4(0,0,1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,-1,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,1,-1)
     &  + 5.5241438124578482d+00*HX1(0)*HX2(0,1)
     &  + HX1(0) *HX2(0,1)*HX3(0,-1,-1)
     &  + HX1(0) *HX2(0,1)*HX3(0,0,-1)
     &  + HX1(0) *HX2(0,1)*HX3(0,0,1)
     &  - HX1(0) *HX2(0,1)*HX3(0,1,-1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,0,-1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,1,-1)
     &  - HX1(0) *HX5(0,-1,-1,0,1)
     &  - HX1(0) *HX5(0,-1,0,-1,1)
     &  - HX1(0) *HX5(0,0,-1,-1,1)
     &  - HX1(0) *HX5(0,0,-1,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,1,1,-1)
     &  + HX1(0) *HX5(0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,1,0,1,-1)
     &  - HX2(0, -1)*HX4(0,1,-1,-1)
     &  + 7.2835884202641093d+00*HX2(0,1)
     &  - 3.0000000000000000d+00*HX2(0,1)*HX4(0,-1,-1,-1)
     &  - 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,-1,-1)
     &  - 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,1)
     &  - 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,1,-1)
     &  - HX2(0,1) *HX4(0,1,-1,-1)
     &  + 2.0000000000000000d+00*HX3(0,-1,-1)*HX3(0,1,-1)
     &  - 5.5241438124578482d+00*HX3(0,0,-1)
     &  + HX3(0,0, -1)*HX3(0,0,-1)
     &  + 4.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,1,-1)
     &  - 1.1048287624915696d+01*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX3(0,0,1)*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX3(0,0,1)*HX3(0,1,-1)
     &  - 5.5241438124578482d+00*HX3(0,1,-1)
     &  + HX3(0,1, -1)*HX3(0,1,-1)
     &  + 2.4674011002723396d+00*HX4(0,0,-1,-1)
     &  + 4.9348022005446793d+00*HX4(0,0,0,-1)
     &  + 7.4022033008170189d+00*HX4(0,0,0,1)
     &  + 4.9348022005446793d+00*HX4(0,0,1,-1)
     &  + 2.4674011002723396d+00*HX4(0,1,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,-1,-1,-1,0,1)
     &  + 3.0000000000000000d+00*HX6(0,-1,-1,0,-1,1)
     &  + HX6(0, -1,-1,0,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,-1,0,-1,-1,1)
     &  + HX6(0, -1,0,-1,1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,-1,-1,-1,1)
     &  - HX6(0,0, -1,-1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,-1,0,1)
     &  + HX6(0,0, -1,-1,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  - 5.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - 8.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - HX6(0,0,0,0,0, -1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,-1,0,-1)
     &  + HX6(0,0,1, -1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,-1,1,-1)
     &  + HX6(0,0,1,0, -1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,1,1,-1,-1)
     &  - HX6(0,1, -1,-1,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,1,0,1,-1,-1)
      HY6(0,1,-1,-1,1,1) = 
     &  - 1.4870411022835581d+01
     &  - 8.3124780562927259d+00*HX1(0)
     &  - 1.0146780316041920d+00*HX1(0)*HX1(0)
     &  + 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  + 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,-1,-1)
     &  - HX1(0) *HX1(0)*HX4(0,0,0,-1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX1(0)*HX4(0,0,1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,-1,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,1,-1)
     &  + 2.7620719062289241d+00*HX1(0)*HX2(0,1)
     &  - HX1(0) *HX2(0,1)*HX3(0,-1,-1)
     &  - HX1(0) *HX2(0,1)*HX3(0,0,-1)
     &  - HX1(0) *HX2(0,1)*HX3(0,0,1)
     &  + HX1(0) *HX2(0,1)*HX3(0,1,-1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  - 9.8696044010893586d+00*HX1(0)*HX3(0,0,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,1,-1)
     &  + HX1(0) *HX5(0,-1,-1,0,1)
     &  + HX1(0) *HX5(0,-1,0,-1,1)
     &  + HX1(0) *HX5(0,0,-1,-1,1)
     &  + HX1(0) *HX5(0,0,-1,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,-1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,1,1,-1)
     &  - HX1(0) *HX5(0,1,-1,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,1,0,1,-1)
     &  - 2.0293560632083841d+00*HX2(0,1)
     &  - HX2(0,1) *HX4(0,-1,-1,1)
     &  - HX2(0,1) *HX4(0,-1,1,-1)
     &  + HX2(0,1) *HX4(0,-1,1,1)
     &  + HX2(0,1) *HX4(0,0,-1,-1)
     &  - HX2(0,1) *HX4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,1)
     &  + HX2(0,1) *HX4(0,0,1,-1)
     &  - HX2(0,1) *HX4(0,0,1,1)
     &  + 5.0000000000000000d-01*HX3(0,-1,1)*HX3(0,-1,1)
     &  + HX3(0, -1,1)*HX3(0,1,-1)
     &  - 2.7620719062289241d+00*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,1)
     &  - HX3(0,0, -1)*HX3(0,1,-1)
     &  - 5.5241438124578482d+00*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,1,-1)
     &  - 2.7620719062289241d+00*HX3(0,1,-1)
     &  + 4.9348022005446793d+00*HX4(0,0,-1,-1)
     &  + 9.8696044010893586d+00*HX4(0,0,0,-1)
     &  + 1.4804406601634037d+01*HX4(0,0,0,1)
     &  + 9.8696044010893586d+00*HX4(0,0,1,-1)
     &  + 4.9348022005446793d+00*HX4(0,1,-1,-1)
     &  + 2.0000000000000000d+00*HX6(0,-1,-1,0,1,1)
     &  + HX6(0, -1,-1,1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,-1,0,1,1,1)
     &  + HX6(0, -1,1,-1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,-1,1,0,1,1)
     &  - HX6(0, -1,1,1,0,1)
     &  - HX6(0,0, -1,-1,0,1)
     &  - HX6(0,0, -1,-1,1,1)
     &  + HX6(0,0, -1,0,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,-1,0,1,1)
     &  + HX6(0,0, -1,1,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,-1,1,1,1)
     &  + HX6(0,0,0, -1,0,1)
     &  + 7.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  - HX6(0,0,0,0,0,1) 
     &  - 5.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  + 9.0000000000000000d+00*HX6(0,0,0,1,1,1)
     &  - HX6(0,0,1, -1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,-1,1,1)
     &  + HX6(0,0,1,0, -1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,1,0,1,1)
     &  + HX6(0,0,1,1,0,1) 
     &  - HX6(0,1, -1,-1,1,1)
     &  + HX6(0,1, -1,0,1,1)
      HY6(0,1,-1,0,1,1) = 
     &  - 1.2475337627830403d+01
     &  - 8.2420046943006692d+00*HX1(0)
     &  - 1.9278882600479649d+00*HX1(0)*HX1(0)
     &  + 2.0034281719326571d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  + HX1(0) *HX1(0)*HX4(0,0,-1,1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 1.2020569031595942d+00*HX1(0)*HX2(0,1)
     &  - HX1(0) *HX2(0,1)*HX3(0,-1,1)
     &  + HX1(0) *HX2(0,1)*HX3(0,0,-1)
     &  - HX1(0) *HX2(0,1)*HX3(0,0,1)
     &  + HX1(0) *HX2(0,1)*HX3(0,1,-1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  - 9.8696044010893586d+00*HX1(0)*HX3(0,0,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,-1,0,1,1)
     &  + HX1(0) *HX5(0,-1,1,0,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,1,0,1,-1)
     &  + 4.9348022005446793d+00*HX2(0,-1)*HX2(0,1)
     &  - 3.8557765200959298d+00*HX2(0,1)
     &  + HX2(0,1) *HX4(0,0,-1,1)
     &  - HX2(0,1) *HX4(0,0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,1)
     &  + HX2(0,1) *HX4(0,0,1,-1)
     &  - HX2(0,1) *HX4(0,0,1,1)
     &  - 1.2020569031595942d+00*HX3(0,0,-1)
     &  - 2.4041138063191885d+00*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,1,-1)
     &  - 1.2020569031595942d+00*HX3(0,1,-1)
     &  - 4.9348022005446793d+00*HX4(0,-1,0,1)
     &  - 9.8696044010893586d+00*HX4(0,0,-1,1)
     &  + 1.4804406601634037d+01*HX4(0,0,0,-1)
     &  + 1.4804406601634037d+01*HX4(0,0,0,1)
     &  - HX6(0,0, -1,0,0,1)
     &  - HX6(0,0, -1,0,1,1)
     &  - HX6(0,0, -1,1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - HX6(0,0,0,0,0,1) 
     &  - 5.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  + 9.0000000000000000d+00*HX6(0,0,0,1,1,1)
     &  - HX6(0,0,1, -1,0,1)
     &  - HX6(0,0,1,0, -1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,1,0,1,1)
     &  + HX6(0,0,1,1,0,1) 
     &  + HX6(0,1, -1,0,1,1)
      HY6(0,1,-1,1,-1,-1) = 
     &  - 3.2705967822787463d+01
     &  - 1.6048081243561883d+01*HX1(0)
     &  - 3.6417942101320546d+00*HX1(0)*HX1(0)
     &  - 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,1)
     &  + 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,-1,1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX1(0)*HX4(0,0,1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,-1,1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,1,-1)
     &  - 2.7620719062289241d+00*HX1(0)*HX2(0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX2(0,1)*HX3(0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX2(0,1)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX2(0,1)*HX3(0,0,1)
     &  + HX1(0) *HX2(0,1)*HX3(0,1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,-1,0,-1,1)
     &  + HX1(0) *HX5(0,-1,0,1,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,1)
     &  + HX1(0) *HX5(0,0,-1,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,-1)
     &  + 9.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 8.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + 1.2000000000000000d+01*HX1(0)*HX5(0,0,0,1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,1,1,-1)
     &  + HX1(0) *HX5(0,1,-1,1,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,1,0,1,-1)
     &  - 7.2835884202641093d+00*HX2(0,1)
     &  + 3.0000000000000000d+00*HX2(0,1)*HX4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,1)
     &  + 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,1,-1)
     &  - HX3(0, -1,-1)*HX3(0,1,-1)
     &  + 2.7620719062289241d+00*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,1)
     &  - HX3(0,0, -1)*HX3(0,1,-1)
     &  + 5.5241438124578482d+00*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,1,-1)
     &  + 2.7620719062289241d+00*HX3(0,1,-1)
     &  - 5.0000000000000000d-01*HX3(0,1,-1)*HX3(0,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,-1,-1,-1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,-1,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,-1,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,-1,0,-1,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,-1,0,-1,1,-1)
     &  - HX6(0, -1,0,1,-1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,-1,-1,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,-1,0,1)
     &  - 4.0000000000000000d+00*HX6(0,0,-1,-1,1,-1)
     &  - HX6(0,0, -1,0,-1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,0,-1,1)
     &  + HX6(0,0, -1,0,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,0,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,1,-1,-1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  + 7.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 1.4000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  + 1.5000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 8.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,0,1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,1,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,1,-1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,0,-1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,1,0,-1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,1,1,-1,-1)
     &  - HX6(0,1, -1,1,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,1,0,1,-1,-1)
      HY6(0,1,-1,1,-1,1) = 
     &  + 1.5475283849478767d+01
     &  + 7.3684365300567532d+00*HX1(0)
     &  + 1.6928093109730279d+00*HX1(0)*HX1(0)
     &  + 2.1949077523321137d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,1)
     &  + 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,1)
     &  - 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,-1,1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + HX1(0) *HX1(0)*HX4(0,0,1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,-1,1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,1,-1)
     &  + 1.3169446513992682d+00*HX1(0)*HX2(0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX2(0,1)*HX3(0,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX2(0,1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX2(0,1)*HX3(0,0,1)
     &  - HX1(0) *HX2(0,1)*HX3(0,1,-1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,0,-1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,-1,0,-1,1)
     &  - HX1(0) *HX5(0,-1,0,1,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,1)
     &  - HX1(0) *HX5(0,0,-1,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,-1)
     &  - 9.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 8.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - 1.2000000000000000d+01*HX1(0)*HX5(0,0,0,1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,1,1,-1)
     &  - HX1(0) *HX5(0,1,-1,1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,1,0,1,-1)
     &  - 5.0000000000000000d-01*HX2(0,-1)*HX2(0,-1)*HX2(0,1)
     &  + 2.4674011002723396d+00*HX2(0,-1)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX2(0,-1)*HX2(0,1)*HX2(0,1)
     &  + HX2(0, -1)*HX4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,1)
     &  + HX2(0, -1)*HX4(0,1,-1,1)
     &  + 3.3856186219460558d+00*HX2(0,1)
     &  + 1.2337005501361698d+00*HX2(0,1)*HX2(0,1)
     &  - 1.6666666666666666d-01*HX2(0,1)*HX2(0,1)*HX2(0,1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX4(0,-1,-1,1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HX2(0,1)*HX4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HX2(0,1)*HX4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX4(0,0,-1,1)
     &  - 4.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,-1)
     &  - 4.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,1)
     &  - 2.0000000000000000d+00*HX2(0,1)*HX4(0,0,1,-1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX4(0,0,1,1)
     &  + HX2(0,1) *HX4(0,1,-1,1)
     &  - HX3(0, -1,1)*HX3(0,-1,1)
     &  - 2.0000000000000000d+00*HX3(0,-1,1)*HX3(0,1,-1)
     &  - 1.3169446513992682d+00*HX3(0,0,-1)
     &  + HX3(0,0, -1)*HX3(0,0,-1)
     &  + 4.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,1,-1)
     &  - 2.6338893027985365d+00*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX3(0,0,1)*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX3(0,0,1)*HX3(0,1,-1)
     &  - 1.3169446513992682d+00*HX3(0,1,-1)
     &  - 2.4674011002723396d+00*HX4(0,-1,0,1)
     &  - 7.4022033008170189d+00*HX4(0,0,-1,1)
     &  + 7.4022033008170189d+00*HX4(0,0,0,-1)
     &  + 9.8696044010893586d+00*HX4(0,0,0,1)
     &  - 4.9348022005446793d+00*HX4(0,0,1,1)
     &  - 2.4674011002723396d+00*HX4(0,1,-1,1)
     &  - 4.0000000000000000d+00*HX6(0,-1,-1,0,1,1)
     &  - 2.0000000000000000d+00*HX6(0,-1,-1,1,0,1)
     &  - HX6(0, -1,0,-1,0,1)
     &  - HX6(0, -1,0,1,-1,1)
     &  + HX6(0, -1,0,1,0,1)
     &  + 6.0000000000000000d+00*HX6(0,-1,0,1,1,1)
     &  - 2.0000000000000000d+00*HX6(0,-1,1,-1,0,1)
     &  + 4.0000000000000000d+00*HX6(0,-1,1,0,1,1)
     &  + 2.0000000000000000d+00*HX6(0,-1,1,1,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,-1,0,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,-1,0,1,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,-1,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,-1,1,0,-1)
     &  + HX6(0,0, -1,1,0,1)
     &  + 1.2000000000000000d+01*HX6(0,0,-1,1,1,1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  - 5.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 9.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,-1,1,1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,0,-1,-1)
     &  - HX6(0,0,0,0, -1,1)
     &  - 1.5000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - 1.6000000000000000d+01*HX6(0,0,0,0,0,1)
     &  - 1.6000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  + 8.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,1,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,-1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,0,-1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,1,0,1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,1,1,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,1,-1,0,1,1)
     &  - HX6(0,1, -1,1,-1,1)
     &  - HX6(0,1,0,1, -1,1)
      HY6(0,1,-1,1,1,-1) = 
     &  + 2.4470748919111122d+01
     &  + 8.4821276453585420d+00*HX1(0)
     &  + 3.3654675223535616d-01*HX1(0)*HX1(0)
     &  - 6.7983609293803206d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,1)
     &  - 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,1)
     &  - 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,-1,1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + HX1(0) *HX1(0)*HX4(0,0,1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,-1,1)
     &  - 4.0790165576281924d+00*HX1(0)*HX2(0,1)
     &  - HX1(0) *HX2(0,1)*HX3(0,-1,1)
     &  + HX1(0) *HX2(0,1)*HX3(0,0,-1)
     &  + HX1(0) *HX2(0,1)*HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,0,-1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,-1,0,1,1)
     &  + HX1(0) *HX5(0,-1,1,0,1)
     &  - HX1(0) *HX5(0,0,-1,0,1)
     &  + 5.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  - HX1(0) *HX5(0,0,1,0,1)
     &  + HX1(0) *HX5(0,1,-1,1,1)
     &  - HX1(0) *HX5(0,1,0,1,1)
     &  + 5.0000000000000000d-01*HX2(0,-1)*HX2(0,-1)*HX2(0,1)
     &  - 2.4674011002723396d+00*HX2(0,-1)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX2(0,-1)*HX2(0,1)*HX2(0,1)
     &  - HX2(0, -1)*HX4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,1)
     &  - HX2(0, -1)*HX4(0,1,-1,1)
     &  + 6.7309350447071233d-01*HX2(0,1)
     &  - 1.2337005501361698d+00*HX2(0,1)*HX2(0,1)
     &  + 1.6666666666666666d-01*HX2(0,1)*HX2(0,1)*HX2(0,1)
     &  + HX2(0,1) *HX4(0,-1,1,1)
     &  - HX2(0,1) *HX4(0,1,-1,1)
     &  + HX3(0, -1,1)*HX3(0,1,-1)
     &  + 4.0790165576281924d+00*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX3(0,0,-1)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,1)
     &  - HX3(0,0, -1)*HX3(0,1,-1)
     &  + 8.1580331152563848d+00*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,1,-1)
     &  + 4.0790165576281924d+00*HX3(0,1,-1)
     &  + 2.4674011002723396d+00*HX4(0,-1,0,1)
     &  + 7.4022033008170189d+00*HX4(0,0,-1,1)
     &  - 7.4022033008170189d+00*HX4(0,0,0,-1)
     &  - 9.8696044010893586d+00*HX4(0,0,0,1)
     &  + 4.9348022005446793d+00*HX4(0,0,1,1)
     &  + 2.4674011002723396d+00*HX4(0,1,-1,1)
     &  + HX6(0, -1,0,-1,0,1)
     &  - HX6(0, -1,0,1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,-1,0,1,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,-1,0,1,1,1)
     &  - HX6(0, -1,1,0,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,-1,1,0,1,1)
     &  - HX6(0, -1,1,1,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,-1,1)
     &  + HX6(0,0, -1,0,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,1,-1)
     &  - HX6(0,0, -1,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,1,0,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,1,0,1)
     &  - 5.0000000000000000d+00*HX6(0,0,-1,1,1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,-1,1,1,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,0,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + 1.8000000000000000d+01*HX6(0,0,0,0,-1,1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 9.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + 1.6000000000000000d+01*HX6(0,0,0,0,1,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  + 7.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,1,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,-1,1)
     &  + HX6(0,0,1,0,1, -1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,0,1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,1,1,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,1,0,1)
     &  + HX6(0,1, -1,0,1,1)
     &  - HX6(0,1, -1,1,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,1,0,1,-1,1)
     &  + HX6(0,1,0,1,1, -1)
      HY6(0,1,-1,1,1,1) = 
     &  + 8.3682759603159873d+00
     &  + 8.2125444753073106d+00*HX1(0)
     &  + 3.2432874665857356d+00*HX1(0)*HX1(0)
     &  - 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(0,1)
     &  + 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  - 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,-1,1)
     &  + 1.5000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX1(0)*HX4(0,0,1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,-1,1)
     &  + HX1(0) *HX2(0,1)*HX3(0,-1,1)
     &  - HX1(0) *HX2(0,1)*HX3(0,0,-1)
     &  - HX1(0) *HX2(0,1)*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,-1)
     &  + 9.8696044010893586d+00*HX1(0)*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,-1,0,1,1)
     &  - HX1(0) *HX5(0,-1,1,0,1)
     &  + HX1(0) *HX5(0,0,-1,0,1)
     &  - 5.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  + HX1(0) *HX5(0,0,1,0,1)
     &  - HX1(0) *HX5(0,1,-1,1,1)
     &  + HX1(0) *HX5(0,1,0,1,1)
     &  - 4.9348022005446793d+00*HX2(0,-1)*HX2(0,1)
     &  + 6.4865749331714713d+00*HX2(0,1)
     &  - 2.4674011002723396d+00*HX2(0,1)*HX2(0,1)
     &  + HX2(0,1) *HX4(0,-1,1,1)
     &  - HX2(0,1) *HX4(0,0,-1,1)
     &  + HX2(0,1) *HX4(0,0,0,-1)
     &  + HX2(0,1) *HX4(0,0,0,1)
     &  - HX2(0,1) *HX4(0,0,1,1)
     &  + 4.9348022005446793d+00*HX4(0,-1,0,1)
     &  + 1.4804406601634037d+01*HX4(0,0,-1,1)
     &  - 1.4804406601634037d+01*HX4(0,0,0,-1)
     &  - 1.9739208802178717d+01*HX4(0,0,0,1)
     &  + 9.8696044010893586d+00*HX4(0,0,1,1)
     &  + 4.9348022005446793d+00*HX4(0,1,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,-1,0,1,1,1)
     &  - 2.0000000000000000d+00*HX6(0,-1,1,0,1,1)
     &  - HX6(0, -1,1,1,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,1,1)
     &  + HX6(0,0, -1,1,0,1)
     &  - 7.0000000000000000d+00*HX6(0,0,-1,1,1,1)
     &  - HX6(0,0,0, -1,0,1)
     &  + 9.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,-1,1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,0,1)
     &  - 1.1000000000000000d+01*HX6(0,0,0,0,1,1)
     &  - HX6(0,0,0,1,0,1) 
     &  + 1.0000000000000000d+01*HX6(0,0,0,1,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,1,1)
     &  + HX6(0,0,1,1,0,1) 
     &  - HX6(0,1, -1,1,1,1)
     &  + HX6(0,1,0,1,1,1) 
      HY6(0,1,0,1,-1,-1) = 
     &  - 2.1451677657903149d+01
     &  - 1.0896901324371520d+01*HX1(0)
     &  - 2.9013792215033260d+00*HX1(0)*HX1(0)
     &  - 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)*HX2(0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX1(0)*HX4(0,0,1,1)
     &  - 2.7620719062289241d+00*HX1(0)*HX2(0,1)
     &  - HX1(0) *HX2(0,1)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX2(0,1)*HX3(0,0,1)
     &  + HX1(0) *HX5(0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - HX1(0) *HX5(0,0,0,0,-1)
     &  - 8.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + 1.2000000000000000d+01*HX1(0)*HX5(0,0,0,1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,1)
     &  - HX1(0) *HX5(0,1,0,1,-1)
     &  - 5.8027584430066521d+00*HX2(0,1)
     &  + HX2(0,1) *HX4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,1)
     &  + 5.5241438124578482d+00*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,0,1)
     &  - HX6(0,0, -1,-1,0,1)
     &  - HX6(0,0, -1,0,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  - HX6(0,0, -1,0,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  + HX6(0,0,0,0, -1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  + 1.5000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 8.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,0,1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,-1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,0,-1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,0,-1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  + HX6(0,1,0,1, -1,-1)
      HY6(0,1,0,1,-1,1) = 
     &  + 1.1363854186094887d+01
     &  + 7.5244757278945982d+00*HX1(0)
     &  + 2.1663257257216508d+00*HX1(0)*HX1(0)
     &  + 2.1949077523321137d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,1)
     &  - 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)*HX2(0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + HX1(0) *HX1(0)*HX4(0,0,1,1)
     &  + 1.3169446513992682d+00*HX1(0)*HX2(0,1)
     &  + HX1(0) *HX2(0,1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX2(0,1)*HX3(0,0,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,1)
     &  - HX1(0) *HX5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + HX1(0) *HX5(0,0,0,0,-1)
     &  + 8.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - 1.2000000000000000d+01*HX1(0)*HX5(0,0,0,1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,1)
     &  + HX1(0) *HX5(0,1,0,1,-1)
     &  - 5.0000000000000000d-01*HX2(0,-1)*HX2(0,1)*HX2(0,1)
     &  - 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,1)
     &  + 4.3326514514433017d+00*HX2(0,1)
     &  + 1.2337005501361698d+00*HX2(0,1)*HX2(0,1)
     &  - 1.6666666666666666d-01*HX2(0,1)*HX2(0,1)*HX2(0,1)
     &  + HX2(0,1) *HX4(0,-1,0,1)
     &  + 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,-1,1)
     &  - 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,-1)
     &  - 4.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX4(0,0,1,1)
     &  + 4.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,1)
     &  - 2.6338893027985365d+00*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX3(0,0,1)*HX3(0,0,1)
     &  + 9.8696044010893586d+00*HX4(0,0,0,1)
     &  - 4.9348022005446793d+00*HX4(0,0,1,1)
     &  - HX6(0, -1,0,1,0,1)
     &  - 4.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,1,0,1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  - 1.1000000000000000d+01*HX6(0,0,0,0,-1,1)
     &  - 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 1.6000000000000000d+01*HX6(0,0,0,0,0,1)
     &  - 1.6000000000000000d+01*HX6(0,0,0,0,1,-1)
     &  + 8.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,1,1,-1)
     &  - 1.2000000000000000d+01*HX6(0,0,0,1,1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  - 4.0000000000000000d+00*HX6(0,0,1,0,1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,1,1,0,-1)
     &  + HX6(0,1,0,1, -1,1)
      HY6(0,1,0,1,1,-1) = 
     &  + 1.0887620789386261d+01
     &  + 3.4438689624689915d+00*HX1(0)
     &  + 7.5009671202303939d-02*HX1(0)*HX1(0)
     &  - 6.7983609293803206d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,1)
     &  - 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)*HX2(0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + HX1(0) *HX1(0)*HX4(0,0,1,1)
     &  - 4.0790165576281924d+00*HX1(0)*HX2(0,1)
     &  + HX1(0) *HX2(0,1)*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  - HX1(0) *HX5(0,0,1,0,1)
     &  - HX1(0) *HX5(0,1,0,1,1)
     &  + 5.0000000000000000d-01*HX2(0,-1)*HX2(0,1)*HX2(0,1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,1)
     &  + 1.5001934240460787d-01*HX2(0,1)
     &  - 1.2337005501361698d+00*HX2(0,1)*HX2(0,1)
     &  + 1.6666666666666666d-01*HX2(0,1)*HX2(0,1)*HX2(0,1)
     &  - HX2(0,1) *HX4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HX2(0,1)*HX4(0,0,-1,1)
     &  + HX2(0,1) *HX4(0,0,0,-1)
     &  - HX2(0,1) *HX4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,1)
     &  + 8.1580331152563848d+00*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,0,1)
     &  - 9.8696044010893586d+00*HX4(0,0,0,1)
     &  + 4.9348022005446793d+00*HX4(0,0,1,1)
     &  + HX6(0, -1,0,1,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,1,0,1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 8.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + HX6(0,0,0,0,0, -1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 9.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + 1.6000000000000000d+01*HX6(0,0,0,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,1,0,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  - 5.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,1,1,1)
     &  + HX6(0,0,1, -1,0,1)
     &  - HX6(0,0,1,0, -1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,0,1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,1,1,0,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,1,0,1)
     &  + HX6(0,1,0,1,1, -1)
      HY6(0,1,0,1,1,1) = 
     &  + 4.6029432200103617d+00
     &  + 4.1477110205734797d+00*HX1(0)
     &  + 2.5705176800639532d+00*HX1(0)*HX1(0)
     &  - 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 3.3333333333333333d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(0,1)
     &  + 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)*HX2(0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX1(0)*HX4(0,0,1,1)
     &  - HX1(0) *HX2(0,1)*HX3(0,0,1)
     &  + 9.8696044010893586d+00*HX1(0)*HX3(0,0,1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  + HX1(0) *HX5(0,0,1,0,1)
     &  + HX1(0) *HX5(0,1,0,1,1)
     &  + 5.1410353601279064d+00*HX2(0,1)
     &  - 2.4674011002723396d+00*HX2(0,1)*HX2(0,1)
     &  + HX2(0,1) *HX4(0,0,0,1)
     &  - HX2(0,1) *HX4(0,0,1,1)
     &  - 1.9739208802178717d+01*HX4(0,0,0,1)
     &  + 9.8696044010893586d+00*HX4(0,0,1,1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,0,1)
     &  - 1.1000000000000000d+01*HX6(0,0,0,0,1,1)
     &  - HX6(0,0,0,1,0,1) 
     &  + 1.0000000000000000d+01*HX6(0,0,0,1,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,1,1)
     &  + HX6(0,0,1,1,0,1) 
     &  + HX6(0,1,0,1,1,1) 
      HY6(0,1,1,-1,-1,-1) = 
     &  + 1.0392853179260542d+01
     &  + 6.1059658089888351d+00*HX1(0)
     &  + 1.2139314033773515d+00*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,1)
     &  - 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  + HX1(0) *HX1(0)*HX4(0,0,-1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,1,-1)
     &  + HX1(0) *HX2(0,1)*HX3(0,-1,-1)
     &  + HX1(0) *HX2(0,1)*HX3(0,0,-1)
     &  + HX1(0) *HX2(0,1)*HX3(0,0,1)
     &  - HX1(0) *HX5(0,-1,-1,0,1)
     &  - HX1(0) *HX5(0,-1,0,-1,1)
     &  - HX1(0) *HX5(0,-1,0,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,1)
     &  - HX1(0) *HX5(0,0,-1,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,-1)
     &  + HX1(0) *HX5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  - HX1(0) *HX5(0,0,1,-1,-1)
     &  - HX1(0) *HX5(0,0,1,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,1,-1)
     &  + HX1(0) *HX5(0,1,0,1,-1)
     &  + HX1(0) *HX5(0,1,1,-1,-1)
     &  + 2.4278628067547031d+00*HX2(0,1)
     &  - HX2(0,1) *HX4(0,-1,-1,-1)
     &  - HX2(0,1) *HX4(0,0,-1,-1)
     &  - HX2(0,1) *HX4(0,0,0,-1)
     &  - HX2(0,1) *HX4(0,0,0,1)
     &  - HX2(0,1) *HX4(0,0,1,-1)
     &  + HX6(0, -1,-1,-1,0,1)
     &  + HX6(0, -1,-1,0,-1,1)
     &  + HX6(0, -1,-1,0,1,-1)
     &  + HX6(0, -1,0,-1,-1,1)
     &  + HX6(0, -1,0,-1,1,-1)
     &  + HX6(0, -1,0,1,-1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,-1,-1,1)
     &  + HX6(0,0, -1,-1,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,-1,1,-1)
     &  + HX6(0,0, -1,0,-1,1)
     &  + HX6(0,0, -1,0,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,1,-1,-1)
     &  - HX6(0,0,0, -1,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  + HX6(0,0,0, -1,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 5.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  + HX6(0,0,0,1,0,1) 
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  + HX6(0,0,1, -1,-1,-1)
     &  + HX6(0,0,1, -1,0,1)
     &  + HX6(0,0,1,0, -1,1)
     &  + HX6(0,0,1,0,1, -1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,1,-1,-1)
     &  - HX6(0,1,0,1, -1,-1)
     &  - HX6(0,1,1, -1,-1,-1)
      HY6(0,1,1,-1,-1,1) = 
     &  - 6.7460686070476163d+00
     &  - 5.6216412826605297d-01*HX1(0)
     &  + 1.5220170474062880d+00*HX1(0)*HX1(0)
     &  + 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,1)
     &  + 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,1)
     &  + 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  - HX1(0) *HX1(0)*HX4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,1,-1)
     &  + 2.7620719062289241d+00*HX1(0)*HX2(0,1)
     &  - HX1(0) *HX2(0,1)*HX3(0,-1,-1)
     &  - HX1(0) *HX2(0,1)*HX3(0,0,-1)
     &  - HX1(0) *HX2(0,1)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,1,1)
     &  + HX1(0) *HX5(0,-1,-1,0,1)
     &  + HX1(0) *HX5(0,-1,0,-1,1)
     &  + HX1(0) *HX5(0,-1,0,1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,-1,1)
     &  + HX1(0) *HX5(0,0,-1,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,-1)
     &  - HX1(0) *HX5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  + HX1(0) *HX5(0,0,1,-1,-1)
     &  + HX1(0) *HX5(0,0,1,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,1,-1)
     &  - HX1(0) *HX5(0,1,0,1,-1)
     &  - HX1(0) *HX5(0,1,1,-1,-1)
     &  + 5.0000000000000000d-01*HX2(0,-1)*HX2(0,-1)*HX2(0,1)
     &  - 2.4674011002723396d+00*HX2(0,-1)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX2(0,-1)*HX2(0,1)*HX2(0,1)
     &  - HX2(0, -1)*HX4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,1)
     &  + HX2(0, -1)*HX4(0,1,1,-1)
     &  + 3.0440340948125761d+00*HX2(0,1)
     &  - 1.2337005501361698d+00*HX2(0,1)*HX2(0,1)
     &  + 1.6666666666666666d-01*HX2(0,1)*HX2(0,1)*HX2(0,1)
     &  + HX2(0,1) *HX4(0,-1,1,1)
     &  + HX2(0,1) *HX4(0,1,1,-1)
     &  - HX3(0, -1,-1)*HX3(0,1,1)
     &  + HX3(0,0, -1)*HX3(0,0,1)
     &  - HX3(0,0, -1)*HX3(0,1,1)
     &  - 2.7620719062289241d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,1,1)
     &  - HX3(0,1, -1)*HX3(0,1,1)
     &  + 2.7620719062289241d+00*HX3(0,1,1)
     &  + 2.4674011002723396d+00*HX4(0,-1,0,1)
     &  + 4.9348022005446793d+00*HX4(0,0,-1,1)
     &  - 2.4674011002723396d+00*HX4(0,0,0,-1)
     &  + 2.4674011002723396d+00*HX4(0,0,1,-1)
     &  - 2.4674011002723396d+00*HX4(0,1,1,-1)
     &  + HX6(0, -1,-1,0,1,1)
     &  + HX6(0, -1,0,-1,0,1)
     &  + HX6(0, -1,0,-1,1,1)
     &  + HX6(0, -1,0,1,-1,1)
     &  - HX6(0, -1,0,1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,-1,0,1,1,1)
     &  - 2.0000000000000000d+00*HX6(0,-1,1,0,1,1)
     &  - HX6(0, -1,1,1,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,-1,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,-1,1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,-1,1)
     &  - HX6(0,0, -1,0,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,0,1,-1)
     &  + HX6(0,0, -1,0,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,1,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,1,0,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,1,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,-1,1,1,1)
     &  - 7.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  + HX6(0,0,0, -1,0,-1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,-1,-1)
     &  - 5.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,1,1)
     &  + HX6(0,0,1, -1,-1,1)
     &  - HX6(0,0,1, -1,0,-1)
     &  - HX6(0,0,1, -1,0,1)
     &  - HX6(0,0,1,0, -1,-1)
     &  + HX6(0,0,1,0, -1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  + HX6(0,0,1,0,1,1) 
     &  + 2.0000000000000000d+00*HX6(0,0,1,1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,1,0,-1)
     &  + HX6(0,1, -1,0,1,1)
     &  + HX6(0,1,0,1, -1,1)
     &  - HX6(0,1,1, -1,-1,1)
      HY6(0,1,1,-1,1,-1) = 
     &  - 2.5172330147323355d+01
     &  - 1.7558216230553864d+01*HX1(0)
     &  - 4.7368434057856041d+00*HX1(0)*HX1(0)
     &  - 9.2069063540964137d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,1)
     &  - 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,1)
     &  + 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  - HX1(0) *HX1(0)*HX4(0,0,-1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,1,-1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,1,1)
     &  - 5.5241438124578482d+00*HX1(0)*HX2(0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX2(0,1)*HX3(0,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX2(0,1)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX2(0,1)*HX3(0,0,1)
     &  - HX1(0) *HX2(0,1)*HX3(0,1,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,-1,0,1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,-1,1,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,1)
     &  + 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + 9.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  - HX1(0) *HX5(0,0,1,-1,1)
     &  + HX1(0) *HX5(0,0,1,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,1,1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,1,0,1,1)
     &  + HX1(0) *HX5(0,1,1,-1,1)
     &  - 5.0000000000000000d-01*HX2(0,-1)*HX2(0,-1)*HX2(0,1)
     &  + 2.4674011002723396d+00*HX2(0,-1)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX2(0,-1)*HX2(0,1)*HX2(0,1)
     &  + HX2(0, -1)*HX4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX4(0,0,1,1)
     &  - HX2(0, -1)*HX4(0,1,1,-1)
     &  - 9.4736868115712082d+00*HX2(0,1)
     &  + 1.2337005501361698d+00*HX2(0,1)*HX2(0,1)
     &  - 1.6666666666666666d-01*HX2(0,1)*HX2(0,1)*HX2(0,1)
     &  - 2.0000000000000000d+00*HX2(0,1)*HX4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HX2(0,1)*HX4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HX2(0,1)*HX4(0,-1,1,1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX2(0,1)*HX4(0,0,-1,1)
     &  + 4.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,-1)
     &  + 4.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HX2(0,1)*HX4(0,0,1,1)
     &  - HX2(0,1) *HX4(0,1,1,-1)
     &  + 2.0000000000000000d+00*HX3(0,-1,-1)*HX3(0,1,1)
     &  + HX3(0, -1,1)*HX3(0,-1,1)
     &  - 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,1,1)
     &  + 5.5241438124578482d+00*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX3(0,0,1)*HX3(0,1,1)
     &  + 2.0000000000000000d+00*HX3(0,1,-1)*HX3(0,1,1)
     &  - 5.5241438124578482d+00*HX3(0,1,1)
     &  - 2.4674011002723396d+00*HX4(0,-1,0,1)
     &  - 4.9348022005446793d+00*HX4(0,0,-1,1)
     &  + 2.4674011002723396d+00*HX4(0,0,0,-1)
     &  - 2.4674011002723396d+00*HX4(0,0,1,-1)
     &  + 2.4674011002723396d+00*HX4(0,1,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,-1,-1,0,1,1)
     &  + 2.0000000000000000d+00*HX6(0,-1,-1,1,0,1)
     &  - HX6(0, -1,0,-1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,-1,0,-1,1,1)
     &  + HX6(0, -1,0,1,0,1)
     &  + 3.0000000000000000d+00*HX6(0,-1,0,1,1,-1)
     &  + 6.0000000000000000d+00*HX6(0,-1,0,1,1,1)
     &  + 2.0000000000000000d+00*HX6(0,-1,1,-1,0,1)
     &  + 2.0000000000000000d+00*HX6(0,-1,1,0,1,-1)
     &  + 4.0000000000000000d+00*HX6(0,-1,1,0,1,1)
     &  + 2.0000000000000000d+00*HX6(0,-1,1,1,0,1)
     &  - 4.0000000000000000d+00*HX6(0,0,-1,-1,0,1)
     &  - 4.0000000000000000d+00*HX6(0,0,-1,-1,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,1,0,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,1,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,-1,1,1,-1)
     &  + 1.2000000000000000d+01*HX6(0,0,-1,1,1,1)
     &  - HX6(0,0,0, -1,0,-1)
     &  + HX6(0,0,0, -1,0,1)
     &  - HX6(0,0,0, -1,1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  - 8.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  + 1.1000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - 1.6000000000000000d+01*HX6(0,0,0,0,1,1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  - 1.5000000000000000d+01*HX6(0,0,0,1,1,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,1,1,1)
     &  + HX6(0,0,1, -1,0,-1)
     &  - HX6(0,0,1, -1,0,1)
     &  + HX6(0,0,1, -1,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,1,0,-1,1)
     &  - 5.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,1,1)
     &  - 8.0000000000000000d+00*HX6(0,0,1,1,-1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,1,1,0,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,1,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,1,1,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,1,-1,0,1,1)
     &  - 4.0000000000000000d+00*HX6(0,1,0,1,-1,1)
     &  - 3.0000000000000000d+00*HX6(0,1,0,1,1,-1)
     &  - HX6(0,1,1, -1,1,-1)
      HY6(0,1,1,-1,1,1) = 
     &  + 1.1862364062699201d+01
     &  + 2.1331672006954635d+00*HX1(0)
     &  - 4.2696785175273867d-01*HX1(0)*HX1(0)
     &  - 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX2(0,-1)*HX2(0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(0,1)
     &  - 2.5000000000000000d-01*HX1(0)*HX1(0)*HX2(0,1)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,-1,0,1)
     &  + HX1(0) *HX1(0)*HX4(0,0,-1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,1,-1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,1,1)
     &  - 2.7620719062289241d+00*HX1(0)*HX2(0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX2(0,1)*HX3(0,-1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX2(0,1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX2(0,1)*HX3(0,0,1)
     &  + HX1(0) *HX2(0,1)*HX3(0,1,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,0,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,-1,0,1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,-1,1,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,1)
     &  - 7.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - 9.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  + HX1(0) *HX5(0,0,1,-1,1)
     &  - HX1(0) *HX5(0,0,1,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,1,1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,1,0,1,1)
     &  - HX1(0) *HX5(0,1,1,-1,1)
     &  + 4.9348022005446793d+00*HX2(0,-1)*HX2(0,1)
     &  - 8.5393570350547734d-01*HX2(0,1)
     &  + 2.4674011002723396d+00*HX2(0,1)*HX2(0,1)
     &  - 3.0000000000000000d+00*HX2(0,1)*HX4(0,-1,1,1)
     &  + 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,-1,1)
     &  - 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,1)
     &  + 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,1,1)
     &  + HX3(0, -1,1)*HX3(0,1,1)
     &  + HX3(0,0, -1)*HX3(0,0,1)
     &  - HX3(0,0, -1)*HX3(0,1,1)
     &  + 2.7620719062289241d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,1,1)
     &  - 2.7620719062289241d+00*HX3(0,1,1)
     &  + 5.0000000000000000d-01*HX3(0,1,1)*HX3(0,1,1)
     &  - 4.9348022005446793d+00*HX4(0,-1,0,1)
     &  - 9.8696044010893586d+00*HX4(0,0,-1,1)
     &  + 4.9348022005446793d+00*HX4(0,0,0,-1)
     &  - 4.9348022005446793d+00*HX4(0,0,1,-1)
     &  + 4.9348022005446793d+00*HX4(0,1,1,-1)
     &  + 6.0000000000000000d+00*HX6(0,-1,0,1,1,1)
     &  + 5.0000000000000000d+00*HX6(0,-1,1,0,1,1)
     &  + 3.0000000000000000d+00*HX6(0,-1,1,1,0,1)
     &  - HX6(0,0, -1,0,0,1)
     &  - 5.0000000000000000d+00*HX6(0,0,-1,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,-1,1,0,1)
     &  + 1.2000000000000000d+01*HX6(0,0,-1,1,1,1)
     &  - 1.6000000000000000d+01*HX6(0,0,0,-1,1,1)
     &  + 1.0000000000000000d+01*HX6(0,0,0,0,-1,1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,0,-1)
     &  - 9.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  + 1.5000000000000000d+01*HX6(0,0,0,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  - 1.8000000000000000d+01*HX6(0,0,0,1,1,1)
     &  + HX6(0,0,1, -1,1,1)
     &  - HX6(0,0,1,0, -1,1)
     &  - 6.0000000000000000d+00*HX6(0,0,1,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,0,1,1,0,1)
     &  - 6.0000000000000000d+00*HX6(0,0,1,1,1,1)
     &  - 3.0000000000000000d+00*HX6(0,1,0,1,1,1)
     &  - HX6(0,1,1, -1,1,1)
      HY6(0,1,1,1,-1,-1) = 
     &  + 9.1194317326569256d+00
     &  + 5.0241320258653776d+00*HX1(0)
     &  + 2.2001483267751239d+00*HX1(0)*HX1(0)
     &  + 4.6034531770482068d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,1,1)
     &  + HX1(0) *HX2(0,-1)*HX3(0,1,1)
     &  + 2.7620719062289241d+00*HX1(0)*HX2(0,1)
     &  - HX1(0) *HX2(0,1)*HX3(0,-1,1)
     &  + HX1(0) *HX2(0,1)*HX3(0,0,-1)
     &  + HX1(0) *HX2(0,1)*HX3(0,0,1)
     &  + HX1(0) *HX2(0,1)*HX3(0,1,1)
     &  + HX1(0) *HX5(0,-1,0,1,1)
     &  + HX1(0) *HX5(0,-1,1,0,1)
     &  - HX1(0) *HX5(0,0,-1,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  + HX1(0) *HX5(0,0,0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  - 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  - HX1(0) *HX5(0,0,1,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,1)
     &  - HX1(0) *HX5(0,0,1,1,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX5(0,0,1,1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,1,0,1,1)
     &  + HX1(0) *HX5(0,1,1,1,-1)
     &  + 4.4002966535502479d+00*HX2(0,1)
     &  + HX2(0,1) *HX4(0,-1,-1,1)
     &  + HX2(0,1) *HX4(0,-1,1,-1)
     &  + HX2(0,1) *HX4(0,-1,1,1)
     &  - HX2(0,1) *HX4(0,0,-1,-1)
     &  + HX2(0,1) *HX4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,1)
     &  - HX2(0,1) *HX4(0,0,1,-1)
     &  + HX2(0,1) *HX4(0,0,1,1)
     &  - HX3(0, -1,-1)*HX3(0,1,1)
     &  - 5.0000000000000000d-01*HX3(0,-1,1)*HX3(0,-1,1)
     &  + HX3(0,0, -1)*HX3(0,0,1)
     &  - HX3(0,0, -1)*HX3(0,1,1)
     &  - 2.7620719062289241d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,1,1)
     &  - HX3(0,1, -1)*HX3(0,1,1)
     &  + 2.7620719062289241d+00*HX3(0,1,1)
     &  - HX6(0, -1,-1,0,1,1)
     &  - HX6(0, -1,-1,1,0,1)
     &  + HX6(0, -1,0,-1,1,1)
     &  - HX6(0, -1,0,1,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,-1,0,1,1,1)
     &  - HX6(0, -1,1,-1,0,1)
     &  - HX6(0, -1,1,0,1,-1)
     &  - 2.0000000000000000d+00*HX6(0,-1,1,0,1,1)
     &  - HX6(0, -1,1,1,0,1)
     &  + HX6(0,0, -1,-1,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,-1,1,1)
     &  + HX6(0,0, -1,0,-1,1)
     &  - HX6(0,0, -1,0,0,1)
     &  + HX6(0,0, -1,0,1,-1)
     &  - HX6(0,0, -1,0,1,1)
     &  - HX6(0,0, -1,1,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,-1,1,1,-1)
     &  - 6.0000000000000000d+00*HX6(0,0,-1,1,1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,-1,1)
     &  - HX6(0,0,0, -1,0,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  - HX6(0,0,0,0, -1,-1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  - 5.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  + 8.0000000000000000d+00*HX6(0,0,0,0,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,1,1)
     &  + HX6(0,0,1, -1,0,1)
     &  + HX6(0,0,1,0, -1,-1)
     &  + HX6(0,0,1,0, -1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,0,1,-1)
     &  - HX6(0,0,1,0,1,1) 
     &  + HX6(0,0,1,1, -1,-1)
     &  + 4.0000000000000000d+00*HX6(0,0,1,1,-1,1)
     &  - HX6(0,0,1,1,0,1) 
     &  + 4.0000000000000000d+00*HX6(0,0,1,1,1,-1)
     &  + HX6(0,1, -1,0,1,1)
     &  + 2.0000000000000000d+00*HX6(0,1,0,1,-1,1)
     &  + 2.0000000000000000d+00*HX6(0,1,0,1,1,-1)
     &  - HX6(0,1,1,1, -1,-1)
      HY6(0,1,1,1,-1,1) = 
     &  - 8.1934284772140170d+00
     &  - 8.8904802883277105d+00*HX1(0)
     &  - 2.7878585066265773d+00*HX1(0)*HX1(0)
     &  - 2.1949077523321137d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,1)
     &  - 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,1,1)
     &  - HX1(0) *HX2(0,-1)*HX3(0,1,1)
     &  - 1.3169446513992682d+00*HX1(0)*HX2(0,1)
     &  + HX1(0) *HX2(0,1)*HX3(0,-1,1)
     &  - HX1(0) *HX2(0,1)*HX3(0,0,-1)
     &  - HX1(0) *HX2(0,1)*HX3(0,0,1)
     &  - HX1(0) *HX2(0,1)*HX3(0,1,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,1,1)
     &  - HX1(0) *HX5(0,-1,0,1,1)
     &  - HX1(0) *HX5(0,-1,1,0,1)
     &  + HX1(0) *HX5(0,0,-1,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,-1,1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX5(0,0,0,-1,1)
     &  - HX1(0) *HX5(0,0,0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,-1)
     &  + 6.0000000000000000d+00*HX1(0)*HX5(0,0,0,1,1)
     &  + HX1(0) *HX5(0,0,1,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,0,1,0,1)
     &  + HX1(0) *HX5(0,0,1,1,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX5(0,0,1,1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX5(0,1,0,1,1)
     &  - HX1(0) *HX5(0,1,1,1,-1)
     &  + HX2(0, -1)*HX4(0,1,1,1)
     &  - 5.5757170132531547d+00*HX2(0,1)
     &  + 3.0000000000000000d+00*HX2(0,1)*HX4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,0,1)
     &  - 3.0000000000000000d+00*HX2(0,1)*HX4(0,0,1,1)
     &  + HX2(0,1) *HX4(0,1,1,1)
     &  - 2.0000000000000000d+00*HX3(0,-1,1)*HX3(0,1,1)
     &  - 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX3(0,0,-1)*HX3(0,1,1)
     &  + 1.3169446513992682d+00*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX3(0,0,1)*HX3(0,1,1)
     &  - 1.3169446513992682d+00*HX3(0,1,1)
     &  - HX3(0,1,1) *HX3(0,1,1)
     &  - 2.4674011002723396d+00*HX4(0,0,0,1)
     &  + 2.4674011002723396d+00*HX4(0,0,1,1)
     &  - 2.4674011002723396d+00*HX4(0,1,1,1)
     &  - 4.0000000000000000d+00*HX6(0,-1,0,1,1,1)
     &  - 4.0000000000000000d+00*HX6(0,-1,1,0,1,1)
     &  - 3.0000000000000000d+00*HX6(0,-1,1,1,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,0,0,1)
     &  + 4.0000000000000000d+00*HX6(0,0,-1,0,1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,-1,1,0,1)
     &  - 8.0000000000000000d+00*HX6(0,0,-1,1,1,1)
     &  + 3.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,-1,1,1)
     &  - HX6(0,0,0,0, -1,1)
     &  + 5.0000000000000000d+00*HX6(0,0,0,0,0,-1)
     &  + 6.0000000000000000d+00*HX6(0,0,0,0,0,1)
     &  + 4.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - 1.0000000000000000d+01*HX6(0,0,0,0,1,1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,1,-1,1)
     &  + HX6(0,0,0,1,0, -1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  + 1.2000000000000000d+01*HX6(0,0,0,1,1,1)
     &  + HX6(0,0,1,0, -1,1)
     &  - HX6(0,0,1,0,1, -1)
     &  + 4.0000000000000000d+00*HX6(0,0,1,0,1,1)
     &  + HX6(0,0,1,1, -1,1)
     &  - HX6(0,0,1,1,0, -1)
     &  + 2.0000000000000000d+00*HX6(0,0,1,1,0,1)
     &  + 4.0000000000000000d+00*HX6(0,0,1,1,1,1)
     &  + 2.0000000000000000d+00*HX6(0,1,0,1,1,1)
     &  - HX6(0,1,1,1, -1,1)
      HY6(0,1,1,1,1,-1) = 
     &  + 2.7374128046504926d+00
     &  + 7.3485562994098023d-01*HX1(0)
     &  - 2.8461108206419650d-02*HX1(0)*HX1(0)
     &  + 6.7983609293803206d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 1.0280837917801415d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,1)
     &  + 1.2337005501361698d+00*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,1,1)
     &  + 4.0790165576281924d+00*HX1(0)*HX2(0,1)
     &  - 2.4674011002723396d+00*HX1(0)*HX3(0,0,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX3(0,1,1)
     &  - HX1(0) *HX5(0,0,0,0,1)
     &  + HX1(0) *HX5(0,0,0,1,1)
     &  - HX1(0) *HX5(0,0,1,1,1)
     &  + HX1(0) *HX5(0,1,1,1,1)
     &  - HX2(0, -1)*HX4(0,1,1,1)
     &  - 5.6922216412839301d-02*HX2(0,1)
     &  - HX2(0,1) *HX4(0,-1,1,1)
     &  + HX2(0,1) *HX4(0,0,-1,1)
     &  - HX2(0,1) *HX4(0,0,0,-1)
     &  - HX2(0,1) *HX4(0,0,0,1)
     &  + HX2(0,1) *HX4(0,0,1,1)
     &  - HX2(0,1) *HX4(0,1,1,1)
     &  + HX3(0, -1,1)*HX3(0,1,1)
     &  + HX3(0,0, -1)*HX3(0,0,1)
     &  - HX3(0,0, -1)*HX3(0,1,1)
     &  - 4.0790165576281924d+00*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX3(0,0,1)*HX3(0,0,1)
     &  - HX3(0,0,1) *HX3(0,1,1)
     &  + 4.0790165576281924d+00*HX3(0,1,1)
     &  + 5.0000000000000000d-01*HX3(0,1,1)*HX3(0,1,1)
     &  + 2.4674011002723396d+00*HX4(0,0,0,1)
     &  - 2.4674011002723396d+00*HX4(0,0,1,1)
     &  + 2.4674011002723396d+00*HX4(0,1,1,1)
     &  + HX6(0, -1,0,1,1,1)
     &  + HX6(0, -1,1,0,1,1)
     &  + HX6(0, -1,1,1,0,1)
     &  - HX6(0,0, -1,0,0,1)
     &  - HX6(0,0, -1,0,1,1)
     &  - HX6(0,0, -1,1,0,1)
     &  + 2.0000000000000000d+00*HX6(0,0,-1,1,1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,-1,1,1)
     &  - 2.0000000000000000d+00*HX6(0,0,0,0,-1,1)
     &  - HX6(0,0,0,0,0, -1)
     &  - 3.0000000000000000d+00*HX6(0,0,0,0,1,-1)
     &  - HX6(0,0,0,1,0, -1)
     &  + 2.0000000000000000d+00*HX6(0,0,0,1,1,-1)
     &  + HX6(0,0,1,0,1, -1)
     &  + HX6(0,0,1,1,0, -1)
     &  + HX6(0,0,1,1,1, -1)
     &  - HX6(0,1,1,1,1, -1)
      HY6(0,1,1,1,1,1) = 
     &  + 3.1791970687014035d-01
     &  - 2.0293560632083841d+00*HX1(0)*HX1(0)
     &  + 2.0561675835602830d-01*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.3888888888888888d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX3(0,1,1)
     &  + 2.4674011002723396d+00*HX1(0)*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,0,1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX4(0,1,1,1)
     &  - 4.9348022005446793d+00*HX1(0)*HX3(0,0,1)
     &  + 4.9348022005446793d+00*HX1(0)*HX3(0,1,1)
     &  + HX1(0) *HX5(0,0,0,0,1)
     &  - HX1(0) *HX5(0,0,0,1,1)
     &  + HX1(0) *HX5(0,0,1,1,1)
     &  - HX1(0) *HX5(0,1,1,1,1)
     &  - 4.0587121264167682d+00*HX2(0,1)
     &  + 4.9348022005446793d+00*HX4(0,0,0,1)
     &  - 4.9348022005446793d+00*HX4(0,0,1,1)
     &  + 4.9348022005446793d+00*HX4(0,1,1,1)
     &  - HX6(0,0,0,0,0,1) 
     &  + HX6(0,0,0,0,1,1) 
     &  - HX6(0,0,0,1,1,1) 
     &  + HX6(0,0,1,1,1,1) 
     &  - HX6(0,1,1,1,1,1) 
* 2017-05-05:12:08:24.hpl 
* <- x_to_y.imag.out 
* produced by form-to-fortr for gehrt@tintin  
      Hi6(-1,-1,-1,-1,-1,1) = 
     &  - 1.3333558146428443d-03
     &  + 9.6181291076284771d-03*HX1(-1)
     &  - 2.7752054332410789d-02*HX1(-1)*HX1(-1)
     &  + 4.0037751159850118d-02*HX1(-1)*HX1(-1)*HX1(-1)
     &  - 2.8881132523331054d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(-1)
     &  + 8.3333333333333333d-03*HX1(-1)*HX1(-1)*HX1(-1)*HX1(-1)*HX1(-1)
     &  - 4.1666666666666666d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)
     &  + 1.1552453009332421d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)
     &  + 8.3333333333333333d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  - 1.2011325347955035d-01*HX1(-1)*HX1(-1)*HX1(0)
     &  - 1.7328679513998632d-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-02*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.5504108664821579d-02*HX1(-1)*HX1(0)
     &  + 1.2011325347955035d-01*HX1(-1)*HX1(0)*HX1(0)
     &  + 1.1552453009332421d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 9.6181291076284771d-03*HX1(0)
     &  - 2.7752054332410789d-02*HX1(0)*HX1(0)
     &  - 4.0037751159850118d-02*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
      Hi6(-1,-1,-1,-1,1,1) = 
     &  - 2.1725370065082536d+00
     &  + 2.3395367393886929d+00*HX1(-1)
     &  - 1.2266232655660451d+00*HX1(-1)*HX1(-1)
     &  + 3.1419342896788785d-01*HX1(-1)*HX1(-1)*HX1(-1)
     &  + 4.1666666666666666d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)
     &  - 8.3333333333333333d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX2(0,1)
     &  - 9.4258028690366357d-01*HX1(-1)*HX1(-1)*HX1(0)
     &  + 8.3333333333333333d-02*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,1,-1)
     &  + 2.4532465311320902d+00*HX1(-1)*HX1(0)
     &  + 9.4258028690366357d-01*HX1(-1)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1( -1)*HX4(0,-1,-1,-1)
     &  - HX1( -1)*HX4(0,0,-1,-1)
     &  - HX1( -1)*HX4(0,0,0,-1)
     &  - HX1( -1)*HX4(0,0,0,1)
     &  - HX1( -1)*HX4(0,0,1,-1)
     &  - HX1( -1)*HX4(0,1,-1,-1)
     &  - 2.3395367393886929d+00*HX1(0)
     &  - 1.2266232655660451d+00*HX1(0)*HX1(0)
     &  - 3.1419342896788785d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX5( -1,-1,-1,-1,1)
     &  + HX5(0, -1,-1,-1,-1)
     &  + HX5(0,0, -1,-1,-1)
     &  + HX5(0,0,0, -1,-1)
     &  + HX5(0,0,0,0, -1)
     &  + HX5(0,0,0,0,1) 
     &  + HX5(0,0,0,1, -1)
     &  + HX5(0,0,1, -1,-1)
     &  + HX5(0,1, -1,-1,-1)
      Hi6(-1,-1,-1,1,-1,1) = 
     &  + 7.0361879898593576d+00
     &  - 5.1782796940106865d+00*HX1(-1)
     &  + 1.5981104731499506d+00*HX1(-1)*HX1(-1)
     &  - 2.1715334122371910d-01*HX1(-1)*HX1(-1)*HX1(-1)
     &  - 1.1552453009332421d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)
     &  - 8.3333333333333333d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(-1)*HX1(-1)*HX1(-1)*HX2(0,1)
     &  + 6.5146002367115732d-01*HX1(-1)*HX1(-1)*HX1(0)
     &  + 1.7328679513998632d-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-02*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX2(0,1)
     &  + 3.4657359027997265d-01*HX1(-1)*HX1(-1)*HX2(0,-1)
     &  + 3.4657359027997265d-01*HX1(-1)*HX1(-1)*HX2(0,1)
     &  - HX1( -1)*HX1(-1)*HX3(0,-1,-1)
     &  - HX1( -1)*HX1(-1)*HX3(0,0,-1)
     &  - HX1( -1)*HX1(-1)*HX3(0,0,1)
     &  - HX1( -1)*HX1(-1)*HX3(0,1,-1)
     &  - 3.1962209462999012d+00*HX1(-1)*HX1(0)
     &  - 6.5146002367115732d-01*HX1(-1)*HX1(0)*HX1(0)
     &  - 1.1552453009332421d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1( -1)*HX1(0)*HX3(0,-1,-1)
     &  - HX1( -1)*HX1(0)*HX3(0,0,-1)
     &  - HX1( -1)*HX1(0)*HX3(0,0,1)
     &  - HX1( -1)*HX1(0)*HX3(0,1,-1)
     &  - 6.9314718055994530d-01*HX1(-1)*HX3(0,-1,-1)
     &  - 6.9314718055994530d-01*HX1(-1)*HX3(0,0,-1)
     &  - 6.9314718055994530d-01*HX1(-1)*HX3(0,0,1)
     &  - 6.9314718055994530d-01*HX1(-1)*HX3(0,1,-1)
     &  + HX1( -1)*HX4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX4(0,0,0,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX4(0,1,-1,-1)
     &  + 5.1782796940106865d+00*HX1(0)
     &  + 1.5981104731499506d+00*HX1(0)*HX1(0)
     &  + 2.1715334122371910d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1(0) *HX4(-1,-1,-1,1)
     &  + HX1(0) *HX4(0,-1,-1,-1)
     &  + HX1(0) *HX4(0,0,-1,-1)
     &  + HX1(0) *HX4(0,0,0,-1)
     &  + HX1(0) *HX4(0,0,0,1)
     &  + HX1(0) *HX4(0,0,1,-1)
     &  + HX1(0) *HX4(0,1,-1,-1)
     &  - 6.9314718055994530d-01*HX4(-1,-1,-1,1)
     &  + 6.9314718055994530d-01*HX4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HX4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HX4(0,0,0,-1)
     &  + 6.9314718055994530d-01*HX4(0,0,0,1)
     &  + 6.9314718055994530d-01*HX4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HX4(0,1,-1,-1)
     &  - 4.0000000000000000d+00*HX5(-1,-1,-1,-1,1)
     &  - 4.0000000000000000d+00*HX5(0,-1,-1,-1,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,-1,-1,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,1,-1,-1)
     &  - 4.0000000000000000d+00*HX5(0,1,-1,-1,-1)
      Hi6(-1,-1,-1,1,1,1) = 
     &  - 5.8454408699671964d+00
     &  + 3.1512791646720465d+00*HX1(-1)
     &  - 2.7752054332410789d-02*HX1(-1)*HX1(-1)
     &  - 2.7415567780803773d-01*HX1(-1)*HX1(-1)*HX1(-1)
     &  + 8.3333333333333333d-02*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  + 8.2246703342411321d-01*HX1(-1)*HX1(-1)*HX1(0)
     &  - 8.3333333333333333d-02*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,-1,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,1,1)
     &  + 5.5504108664821579d-02*HX1(-1)*HX1(0)
     &  - 8.2246703342411321d-01*HX1(-1)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1( -1)*HX1(0)*HX3(0,-1,-1)
     &  + HX1( -1)*HX1(0)*HX3(0,0,-1)
     &  + HX1( -1)*HX1(0)*HX3(0,0,1)
     &  + HX1( -1)*HX1(0)*HX3(0,1,-1)
     &  + HX1( -1)*HX4(0,-1,-1,1)
     &  + HX1( -1)*HX4(0,-1,1,-1)
     &  - HX1( -1)*HX4(0,0,-1,-1)
     &  + HX1( -1)*HX4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX4(0,0,0,1)
     &  - HX1( -1)*HX4(0,0,1,-1)
     &  + HX1( -1)*HX4(0,0,1,1)
     &  + HX1( -1)*HX4(0,1,-1,1)
     &  + HX1( -1)*HX4(0,1,1,-1)
     &  - 3.1512791646720465d+00*HX1(0)
     &  - 2.7752054332410789d-02*HX1(0)*HX1(0)
     &  + 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1(0) *HX4(-1,-1,-1,1)
     &  - HX1(0) *HX4(0,-1,-1,-1)
     &  - HX1(0) *HX4(0,0,-1,-1)
     &  - HX1(0) *HX4(0,0,0,-1)
     &  - HX1(0) *HX4(0,0,0,1)
     &  - HX1(0) *HX4(0,0,1,-1)
     &  - HX1(0) *HX4(0,1,-1,-1)
     &  + HX5( -1,-1,-1,1,1)
     &  - HX5(0, -1,-1,-1,1)
     &  - HX5(0, -1,-1,1,-1)
     &  - HX5(0, -1,1,-1,-1)
     &  + HX5(0,0, -1,-1,-1)
     &  - HX5(0,0, -1,-1,1)
     &  - HX5(0,0, -1,1,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  - HX5(0,0,0, -1,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - HX5(0,0,0,1,1) 
     &  + HX5(0,0,1, -1,-1)
     &  - HX5(0,0,1, -1,1)
     &  - HX5(0,0,1,1, -1)
     &  - HX5(0,1, -1,-1,1)
     &  - HX5(0,1, -1,1,-1)
     &  - HX5(0,1,1, -1,-1)
      Hi6(-1,-1,1,-1,1,1) = 
     &  + 1.1103251540363018d+01
     &  - 5.6883244754482793d+00*HX1(-1)
     &  + 1.4642921161116944d+00*HX1(-1)*HX1(-1)
     &  + 9.4258028690366357d-01*HX1(-1)*HX1(-1)*HX1(0)
     &  - 8.3333333333333333d-02*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX1(0)*HX2(0,1)
     &  + HX1( -1)*HX1(-1)*HX3(0,-1,1)
     &  - HX1( -1)*HX1(-1)*HX3(0,0,-1)
     &  - HX1( -1)*HX1(-1)*HX3(0,0,1)
     &  + HX1( -1)*HX1(-1)*HX3(0,1,1)
     &  - 2.9285842322233888d+00*HX1(-1)*HX1(0)
     &  - 9.4258028690366357d-01*HX1(-1)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX2(0,1)
     &  + HX1( -1)*HX1(0)*HX3(-1,-1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX3(0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX3(0,0,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX3(0,1,-1)
     &  - 1.8851605738073271d+00*HX1(-1)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX2(0,-1)*HX2(0,-1)
     &  + HX1( -1)*HX2(0,-1)*HX2(0,1)
     &  - 1.8851605738073271d+00*HX1(-1)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX2(0,1)*HX2(0,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX4(0,0,-1,1)
     &  + 4.0000000000000000d+00*HX1(-1)*HX4(0,0,0,-1)
     &  + 4.0000000000000000d+00*HX1(-1)*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX4(0,0,1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX4(0,1,1,-1)
     &  + 5.6883244754482793d+00*HX1(0)
     &  + 1.4642921161116944d+00*HX1(0)*HX1(0)
     &  + 3.1419342896788785d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(-1,-1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(-1,-1,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,1,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,1,-1,-1)
     &  - HX2(0, -1)*HX3(-1,-1,1)
     &  - HX2(0, -1)*HX3(0,-1,-1)
     &  - HX2(0, -1)*HX3(0,1,-1)
     &  - HX2(0,1) *HX3(-1,-1,1)
     &  - HX2(0,1) *HX3(0,1,-1)
     &  + 1.8851605738073271d+00*HX3(-1,-1,1)
     &  + 1.8851605738073271d+00*HX3(0,-1,-1)
     &  + 1.8851605738073271d+00*HX3(0,0,-1)
     &  + 1.8851605738073271d+00*HX3(0,0,1)
     &  + 1.8851605738073271d+00*HX3(0,1,-1)
     &  + HX5( -1,-1,1,-1,1)
     &  + 3.0000000000000000d+00*HX5(0,-1,-1,-1,1)
     &  - HX5(0, -1,-1,0,1)
     &  + 2.0000000000000000d+00*HX5(0,-1,-1,1,-1)
     &  + HX5(0, -1,0,-1,-1)
     &  - HX5(0, -1,0,-1,1)
     &  + 2.0000000000000000d+00*HX5(0,-1,1,-1,-1)
     &  + HX5(0,0, -1,-1,1)
     &  - HX5(0,0, -1,0,-1)
     &  - HX5(0,0, -1,0,1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,1,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,1,-1,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,1,-1,1)
     &  - HX5(0,0,1,0, -1)
     &  - HX5(0,0,1,0,1) 
     &  + 4.0000000000000000d+00*HX5(0,0,1,1,-1)
     &  + 3.0000000000000000d+00*HX5(0,1,-1,-1,1)
     &  + 2.0000000000000000d+00*HX5(0,1,-1,1,-1)
     &  + HX5(0,1,0,1, -1)
     &  + 2.0000000000000000d+00*HX5(0,1,1,-1,-1)
      Hi6(-1,-1,1,1,-1,1) = 
     &  + 2.6913384005612259d+00
     &  + 8.9422335800838046d-01*HX1(-1)
     &  - 1.1350559532618773d+00*HX1(-1)*HX1(-1)
     &  - 6.5146002367115732d-01*HX1(-1)*HX1(-1)*HX1(0)
     &  - 1.7328679513998632d-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-02*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,-1,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(-1)*HX3(0,1,1)
     &  + 2.2701119065237547d+00*HX1(-1)*HX1(0)
     &  + 6.5146002367115732d-01*HX1(-1)*HX1(0)*HX1(0)
     &  + 1.1552453009332421d-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 6.9314718055994530d-01*HX1(-1)*HX1(0)*HX2(0,-1)
     &  + 6.9314718055994530d-01*HX1(-1)*HX1(0)*HX2(0,1)
     &  + HX1( -1)*HX1(0)*HX3(0,-1,1)
     &  - HX1( -1)*HX1(0)*HX3(0,0,-1)
     &  - HX1( -1)*HX1(0)*HX3(0,0,1)
     &  + HX1( -1)*HX1(0)*HX3(0,1,1)
     &  + 1.3029200473423146d+00*HX1(-1)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX2(0,-1)*HX2(0,-1)
     &  - HX1( -1)*HX2(0,-1)*HX2(0,1)
     &  + 1.3029200473423146d+00*HX1(-1)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX2(0,1)*HX2(0,1)
     &  + 6.9314718055994530d-01*HX1(-1)*HX3(0,-1,1)
     &  - 6.9314718055994530d-01*HX1(-1)*HX3(0,0,-1)
     &  - 6.9314718055994530d-01*HX1(-1)*HX3(0,0,1)
     &  + 6.9314718055994530d-01*HX1(-1)*HX3(0,1,1)
     &  + HX1( -1)*HX4(-1,-1,1,1)
     &  - 8.9422335800838046d-01*HX1(0)
     &  - 1.1350559532618773d+00*HX1(0)*HX1(0)
     &  - 2.1715334122371910d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(-1,-1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(-1,-1,1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,-1,-1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,0,-1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,0,1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,1,-1)
     &  - HX1(0) *HX4(-1,-1,1,1)
     &  - HX1(0) *HX4(0,-1,-1,1)
     &  - HX1(0) *HX4(0,-1,1,-1)
     &  + HX1(0) *HX4(0,0,-1,-1)
     &  - HX1(0) *HX4(0,0,-1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + HX1(0) *HX4(0,0,1,-1)
     &  - HX1(0) *HX4(0,0,1,1)
     &  - HX1(0) *HX4(0,1,-1,1)
     &  - HX1(0) *HX4(0,1,1,-1)
     &  + HX2(0, -1)*HX3(-1,-1,1)
     &  + HX2(0, -1)*HX3(0,-1,-1)
     &  + HX2(0, -1)*HX3(0,1,-1)
     &  + HX2(0,1) *HX3(-1,-1,1)
     &  + HX2(0,1) *HX3(0,1,-1)
     &  - 1.3029200473423146d+00*HX3(-1,-1,1)
     &  - 1.3029200473423146d+00*HX3(0,-1,-1)
     &  - 1.3029200473423146d+00*HX3(0,0,-1)
     &  - 1.3029200473423146d+00*HX3(0,0,1)
     &  - 1.3029200473423146d+00*HX3(0,1,-1)
     &  - 6.9314718055994530d-01*HX4(-1,-1,1,1)
     &  - 6.9314718055994530d-01*HX4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HX4(0,-1,1,-1)
     &  + 6.9314718055994530d-01*HX4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HX4(0,0,-1,1)
     &  + 1.3862943611198906d+00*HX4(0,0,0,-1)
     &  + 1.3862943611198906d+00*HX4(0,0,0,1)
     &  + 6.9314718055994530d-01*HX4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HX4(0,0,1,1)
     &  - 6.9314718055994530d-01*HX4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HX4(0,1,1,-1)
     &  - 3.0000000000000000d+00*HX5(-1,-1,-1,1,1)
     &  - HX5( -1,-1,1,-1,1)
     &  + HX5(0, -1,-1,0,1)
     &  + HX5(0, -1,-1,1,-1)
     &  - HX5(0, -1,0,-1,-1)
     &  + HX5(0, -1,0,-1,1)
     &  + HX5(0, -1,1,-1,-1)
     &  - 3.0000000000000000d+00*HX5(0,0,-1,-1,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,-1,1)
     &  + HX5(0,0, -1,0,-1)
     &  + HX5(0,0, -1,0,1)
     &  + HX5(0,0, -1,1,-1)
     &  + HX5(0,0,0, -1,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + HX5(0,0,0,1, -1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,1)
     &  - HX5(0,0,1, -1,-1)
     &  + HX5(0,0,1,0, -1)
     &  + HX5(0,0,1,0,1) 
     &  - HX5(0,0,1,1, -1)
     &  + HX5(0,1, -1,1,-1)
     &  - HX5(0,1,0,1, -1)
     &  + HX5(0,1,1, -1,-1)
      Hi6(-1,-1,1,1,1,1) = 
     &  - 2.1725370065082536d+00
     &  - 8.0212429617572516d-01*HX1(-1)
     &  - 8.2246703342411321d-01*HX1(-1)*HX1(-1)*HX1(0)
     &  + 8.3333333333333333d-02*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
     &  + 8.2246703342411321d-01*HX1(-1)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX2(0,1)
     &  - HX1( -1)*HX1(0)*HX3(0,-1,1)
     &  + HX1( -1)*HX1(0)*HX3(0,0,-1)
     &  + HX1( -1)*HX1(0)*HX3(0,0,1)
     &  - HX1( -1)*HX1(0)*HX3(0,1,1)
     &  + 1.6449340668482264d+00*HX1(-1)*HX2(0,-1)
     &  + 1.6449340668482264d+00*HX1(-1)*HX2(0,1)
     &  - HX1( -1)*HX4(0,-1,1,1)
     &  + HX1( -1)*HX4(0,0,-1,1)
     &  - HX1( -1)*HX4(0,0,0,-1)
     &  - HX1( -1)*HX4(0,0,0,1)
     &  + HX1( -1)*HX4(0,0,1,1)
     &  - HX1( -1)*HX4(0,1,1,1)
     &  + 8.0212429617572516d-01*HX1(0)
     &  - 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(-1,-1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  + HX1(0) *HX4(-1,-1,1,1)
     &  + HX1(0) *HX4(0,-1,-1,1)
     &  + HX1(0) *HX4(0,-1,1,-1)
     &  - HX1(0) *HX4(0,0,-1,-1)
     &  + HX1(0) *HX4(0,0,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX4(0,0,1,-1)
     &  + HX1(0) *HX4(0,0,1,1)
     &  + HX1(0) *HX4(0,1,-1,1)
     &  + HX1(0) *HX4(0,1,1,-1)
     &  - 1.6449340668482264d+00*HX3(-1,-1,1)
     &  - 1.6449340668482264d+00*HX3(0,-1,-1)
     &  - 1.6449340668482264d+00*HX3(0,0,-1)
     &  - 1.6449340668482264d+00*HX3(0,0,1)
     &  - 1.6449340668482264d+00*HX3(0,1,-1)
     &  + HX5( -1,-1,1,1,1)
     &  + HX5(0, -1,-1,1,1)
     &  + HX5(0, -1,1,-1,1)
     &  + HX5(0, -1,1,1,-1)
     &  - HX5(0,0, -1,-1,1)
     &  - HX5(0,0, -1,1,-1)
     &  + HX5(0,0, -1,1,1)
     &  + HX5(0,0,0, -1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + HX5(0,0,0,1, -1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,1,1)
     &  - HX5(0,0,1, -1,1)
     &  - HX5(0,0,1,1, -1)
     &  + HX5(0,0,1,1,1) 
     &  + HX5(0,1, -1,1,1)
     &  + HX5(0,1,1, -1,1)
     &  + HX5(0,1,1,1, -1)
      Hi6(-1,1,-1,1,1,1) = 
     &  + 8.5821033503585191d+00
     &  - 3.2780777484685706d+00*HX1(-1)
     &  - 5.5504108664821579d-02*HX1(-1)*HX1(0)
     &  + 8.2246703342411321d-01*HX1(-1)*HX1(0)*HX1(0)
     &  - 4.1666666666666666d-02*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX2(-1,1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(-1)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX3(0,-1,1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX3(0,0,-1)
     &  - 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX1(-1)*HX1(0)*HX3(0,1,1)
     &  - 1.6449340668482264d+00*HX1(-1)*HX2(-1,1)
     &  - 1.6449340668482264d+00*HX1(-1)*HX2(0,-1)
     &  - 1.6449340668482264d+00*HX1(-1)*HX2(0,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX4(0,-1,1,1)
     &  - 3.0000000000000000d+00*HX1(-1)*HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX4(0,0,0,1)
     &  - 3.0000000000000000d+00*HX1(-1)*HX4(0,0,1,1)
     &  + 3.0000000000000000d+00*HX1(-1)*HX4(0,1,1,1)
     &  + 3.2780777484685706d+00*HX1(0)
     &  + 2.7752054332410789d-02*HX1(0)*HX1(0)
     &  - 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(-1,1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - HX1(0) *HX1(0)*HX3(-1,-1,1)
     &  - HX1(0) *HX1(0)*HX3(0,-1,-1)
     &  - HX1(0) *HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX1(0)*HX3(0,0,1)
     &  - HX1(0) *HX1(0)*HX3(0,1,-1)
     &  + 1.6449340668482264d+00*HX1(0)*HX2(-1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(-1,1)*HX2(-1,1)
     &  - HX1(0) *HX2(-1,1)*HX2(0,-1)
     &  - HX1(0) *HX2(-1,1)*HX2(0,1)
     &  - 1.6449340668482264d+00*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + HX1(0) *HX2(0,-1)*HX2(0,1)
     &  - 1.6449340668482264d+00*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(0,1)*HX2(0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(-1,-1,1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,-1,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,1,1,-1)
     &  - 5.5504108664821579d-02*HX2(-1,1)
     &  - HX2( -1,1)*HX3(0,-1,1)
     &  + HX2( -1,1)*HX3(0,0,-1)
     &  + HX2( -1,1)*HX3(0,0,1)
     &  - HX2( -1,1)*HX3(0,1,1)
     &  + 5.5504108664821579d-02*HX2(0,-1)
     &  - HX2(0, -1)*HX3(0,0,-1)
     &  - HX2(0, -1)*HX3(0,0,1)
     &  + 5.5504108664821579d-02*HX2(0,1)
     &  + HX2(0,1) *HX3(0,-1,1)
     &  - HX2(0,1) *HX3(0,0,-1)
     &  - HX2(0,1) *HX3(0,0,1)
     &  + 3.2898681336964528d+00*HX3(-1,-1,1)
     &  + 3.2898681336964528d+00*HX3(0,-1,-1)
     &  + 3.2898681336964528d+00*HX3(0,0,-1)
     &  + 3.2898681336964528d+00*HX3(0,0,1)
     &  + 3.2898681336964528d+00*HX3(0,1,-1)
     &  + HX5( -1,1,-1,1,1)
     &  - 2.0000000000000000d+00*HX5(0,-1,-1,1,1)
     &  + HX5(0, -1,0,-1,1)
     &  - HX5(0, -1,0,1,1)
     &  - 2.0000000000000000d+00*HX5(0,-1,1,-1,1)
     &  - HX5(0, -1,1,0,1)
     &  - 3.0000000000000000d+00*HX5(0,-1,1,1,-1)
     &  + 4.0000000000000000d+00*HX5(0,0,-1,-1,1)
     &  + HX5(0,0, -1,0,-1)
     &  + HX5(0,0, -1,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,-1,1,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,-1,1,1)
     &  + 7.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 7.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + 2.0000000000000000d+00*HX5(0,0,1,-1,1)
     &  + HX5(0,0,1,0, -1)
     &  + HX5(0,0,1,0,1) 
     &  + 3.0000000000000000d+00*HX5(0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,1,-1,1,1)
     &  + HX5(0,1,0,1,1) 
     &  - 2.0000000000000000d+00*HX5(0,1,1,-1,1)
     &  - 3.0000000000000000d+00*HX5(0,1,1,1,-1)
      Hi6(-1,1,1,1,1,1) = 
     &  - 1.3333558146428443d-03
     &  + 8.1174242528335364d-01*HX1(-1)
     &  - 8.2246703342411321d-01*HX1(-1)*HX1(0)*HX1(0)
     &  + 4.1666666666666666d-02*HX1(-1)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.1174242528335364d-01*HX1(0)
     &  + 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(-1,1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(-1,1,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,1)
     &  - 1.6449340668482264d+00*HX1(0)*HX2(-1,1)
     &  + 1.6449340668482264d+00*HX1(0)*HX2(0,-1)
     &  + 1.6449340668482264d+00*HX1(0)*HX2(0,1)
     &  + HX1(0) *HX4(-1,1,1,1)
     &  - HX1(0) *HX4(0,-1,1,1)
     &  + HX1(0) *HX4(0,0,-1,1)
     &  - HX1(0) *HX4(0,0,0,-1)
     &  - HX1(0) *HX4(0,0,0,1)
     &  + HX1(0) *HX4(0,0,1,1)
     &  - HX1(0) *HX4(0,1,1,1)
     &  - 1.6449340668482264d+00*HX3(-1,1,1)
     &  + 1.6449340668482264d+00*HX3(0,-1,1)
     &  - 1.6449340668482264d+00*HX3(0,0,-1)
     &  - 1.6449340668482264d+00*HX3(0,0,1)
     &  + 1.6449340668482264d+00*HX3(0,1,1)
     &  + HX5( -1,1,1,1,1)
     &  - HX5(0, -1,1,1,1)
     &  + HX5(0,0, -1,1,1)
     &  - HX5(0,0,0, -1,1)
     &  + HX5(0,0,0,0, -1)
     &  + HX5(0,0,0,0,1) 
     &  - HX5(0,0,0,1,1) 
     &  + HX5(0,0,1,1,1) 
     &  - HX5(0,1,1,1,1) 
      Hi6(0,-1,-1,-1,-1,-1) = 
     &  0.d0 
      Hi6(0,-1,-1,-1,-1,1) = 
     &  + 5.0706722342762586d-01
     &  - 9.6181291076284771d-03*HX1(0)
     &  - 2.7752054332410789d-02*HX1(0)*HX1(0)
     &  - 4.0037751159850118d-02*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 2.4022650695910071d-01*HX1(0)*HX2(0,-1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,-1,-1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX4(0,-1,-1,-1)
     &  + HX1(0) *HX4(0,0,-1,-1)
     &  + HX1(0) *HX4(0,0,0,-1)
     &  + 5.5504108664821579d-02*HX2(0,-1)
     &  - 2.4022650695910071d-01*HX3(0,-1,-1)
     &  - 2.4022650695910071d-01*HX3(0,0,-1)
     &  + 6.9314718055994530d-01*HX4(0,-1,-1,-1)
     &  + 6.9314718055994530d-01*HX4(0,0,-1,-1)
     &  + 6.9314718055994530d-01*HX4(0,0,0,-1)
     &  - HX5(0, -1,-1,-1,-1)
     &  - HX5(0,0, -1,-1,-1)
     &  - HX5(0,0,0, -1,-1)
     &  - HX5(0,0,0,0, -1)
      Hi6(0,-1,-1,-1,0,1) = 
     &  + 1.7399784466572854d+00
     &  + 1.0585708673885197d+00*HX1(0)
     &  + 5.2589989513232249d-01*HX1(0)*HX1(0)
     &  + 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 8.2246703342411321d-01*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX4(0,-1,-1,-1)
     &  + HX1(0) *HX4(0,0,-1,-1)
     &  + HX1(0) *HX4(0,0,0,-1)
     &  - 1.0517997902646449d+00*HX2(0,-1)
     &  - HX2(0, -1)*HX3(0,-1,-1)
     &  + HX2(0, -1)*HX3(0,0,-1)
     &  + 8.2246703342411321d-01*HX3(0,-1,-1)
     &  + 8.2246703342411321d-01*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX5(0,-1,0,-1,-1)
     &  + 4.0000000000000000d+00*HX5(0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,0,-1)
     &  - 6.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,0,-1)
      Hi6(0,-1,-1,-1,1,-1) = 
     &  + 3.6535593167128363d-01
     &  + 3.8472516430513908d-02*HX1(0)
     &  + 8.3256162997232369d-02*HX1(0)*HX1(0)
     &  + 8.0075502319700237d-02*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 4.8045301391820142d-01*HX1(0)*HX2(0,-1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,-1,-1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,0,-1)
     &  - 1.6651232599446473d-01*HX2(0,-1)
     &  + 4.8045301391820142d-01*HX3(0,-1,-1)
     &  + 4.8045301391820142d-01*HX3(0,0,-1)
     &  - 6.9314718055994530d-01*HX4(0,-1,-1,-1)
     &  - 6.9314718055994530d-01*HX4(0,0,-1,-1)
     &  - 6.9314718055994530d-01*HX4(0,0,0,-1)
      Hi6(0,-1,-1,-1,1,1) = 
     &  - 2.5126953730408564d+00
     &  - 2.3395367393886929d+00*HX1(0)
     &  - 1.2266232655660451d+00*HX1(0)*HX1(0)
     &  - 3.1419342896788785d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.8851605738073271d+00*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX4(0,-1,-1,-1)
     &  - HX1(0) *HX4(0,0,-1,-1)
     &  - HX1(0) *HX4(0,0,0,-1)
     &  + 2.4532465311320902d+00*HX2(0,-1)
     &  + HX2(0, -1)*HX3(0,-1,-1)
     &  - HX2(0, -1)*HX3(0,0,-1)
     &  - HX2(0, -1)*HX3(0,0,1)
     &  - 1.8851605738073271d+00*HX3(0,-1,-1)
     &  - 1.8851605738073271d+00*HX3(0,0,-1)
     &  - HX5(0, -1,-1,-1,1)
     &  + HX5(0, -1,-1,0,1)
     &  - 2.0000000000000000d+00*HX5(0,-1,0,-1,-1)
     &  + HX5(0, -1,0,-1,1)
     &  - 4.0000000000000000d+00*HX5(0,0,-1,-1,-1)
     &  + HX5(0,0, -1,-1,1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,0,-1)
     &  + HX5(0,0, -1,0,1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + HX5(0,0,0,0,1) 
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + HX5(0,0,1,0, -1)
      Hi6(0,-1,-1,0,-1,1) = 
     &  - 2.8080625910192747d+00
     &  - 2.4466605430300786d+00*HX1(0)
     &  - 7.6675443760393181d-01*HX1(0)*HX1(0)
     &  - 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 8.2246703342411321d-01*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,-1,-1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,0,-1)
     &  + 1.5335088752078636d+00*HX2(0,-1)
     &  + 3.4657359027997265d-01*HX2(0,-1)*HX2(0,-1)
     &  + HX2(0, -1)*HX3(0,-1,-1)
     &  - 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,-1)
     &  - 8.2246703342411321d-01*HX3(0,-1,-1)
     &  - 8.2246703342411321d-01*HX3(0,0,-1)
     &  - 3.0000000000000000d+00*HX5(0,-1,0,-1,-1)
     &  - 6.0000000000000000d+00*HX5(0,0,-1,-1,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,-1,0,-1)
     &  + 9.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,-1)
      Hi6(0,-1,-1,0,1,-1) = 
     &  - 2.3444040559418693d-01
     &  - 7.2905205913548049d-01*HX1(0)
     &  - 2.8504535266071318d-01*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,-1,-1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,0,-1)
     &  + 5.7009070532142637d-01*HX2(0,-1)
     &  - 3.4657359027997265d-01*HX2(0,-1)*HX2(0,-1)
      Hi6(0,-1,-1,0,1,1) = 
     &  + 1.5357173524207522d+00
     &  + 4.0587121264167682d-01*HX1(0)
     &  - 5.2589989513232249d-01*HX1(0)*HX1(0)
     &  - 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 1.6449340668482264d+00*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + 1.0517997902646449d+00*HX2(0,-1)
     &  + HX2(0, -1)*HX3(0,0,-1)
     &  - HX2(0, -1)*HX3(0,0,1)
     &  - 1.6449340668482264d+00*HX3(0,-1,-1)
     &  - 1.6449340668482264d+00*HX3(0,0,-1)
     &  + HX5(0, -1,-1,0,1)
     &  - HX5(0,0, -1,0,-1)
     &  + HX5(0,0, -1,0,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + HX5(0,0,0,0,1) 
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + HX5(0,0,1,0, -1)
      Hi6(0,-1,-1,1,-1,-1) = 
     &  + 1.1571929084637414d-01
     &  - 5.7708774645770862d-02*HX1(0)
     &  - 8.3256162997232369d-02*HX1(0)*HX1(0)
     &  - 4.0037751159850118d-02*HX1(0)*HX1(0)*HX1(0)
     &  + 2.4022650695910071d-01*HX1(0)*HX2(0,-1)
     &  + 1.6651232599446473d-01*HX2(0,-1)
     &  - 2.4022650695910071d-01*HX3(0,-1,-1)
     &  - 2.4022650695910071d-01*HX3(0,0,-1)
      Hi6(0,-1,-1,1,-1,1) = 
     &  + 6.5041750580453003d+00
     &  + 5.1782796940106865d+00*HX1(0)
     &  + 1.5981104731499506d+00*HX1(0)*HX1(0)
     &  + 2.1715334122371910d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.3029200473423146d+00*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,-1,-1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX4(0,-1,-1,1)
     &  - HX1(0) *HX4(0,-1,0,1)
     &  - HX1(0) *HX4(0,0,-1,1)
     &  + HX1(0) *HX4(0,0,0,1)
     &  - 3.1962209462999012d+00*HX2(0,-1)
     &  - 3.4657359027997265d-01*HX2(0,-1)*HX2(0,-1)
     &  - HX2(0, -1)*HX3(0,-1,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,1)
     &  + 1.3029200473423146d+00*HX3(0,-1,-1)
     &  + 1.3029200473423146d+00*HX3(0,0,-1)
     &  + 6.9314718055994530d-01*HX4(0,-1,-1,1)
     &  - 6.9314718055994530d-01*HX4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HX4(0,0,-1,1)
     &  + 6.9314718055994530d-01*HX4(0,0,0,1)
     &  - HX5(0, -1,-1,0,1)
     &  - HX5(0, -1,-1,1,-1)
     &  + 3.0000000000000000d+00*HX5(0,-1,0,-1,-1)
     &  - HX5(0, -1,0,-1,1)
     &  + HX5(0, -1,0,1,-1)
     &  + 6.0000000000000000d+00*HX5(0,0,-1,-1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,-1,1)
     &  - 3.0000000000000000d+00*HX5(0,0,-1,0,-1)
     &  - HX5(0,0, -1,0,1)
     &  + HX5(0,0, -1,1,-1)
     &  - 9.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,1,0,-1)
      Hi6(0,-1,-1,1,0,1) = 
     &  + 6.9860394937032664d+00
     &  + 5.2934782505336140d+00*HX1(0)
     &  + 1.3368451429253581d+00*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + HX1(0) *HX4(0,-1,-1,1)
     &  - HX1(0) *HX4(0,-1,0,1)
     &  - HX1(0) *HX4(0,0,-1,1)
     &  + HX1(0) *HX4(0,0,0,1)
     &  - 2.6736902858507163d+00*HX2(0,-1)
     &  + HX2(0, -1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,1)
     &  - HX5(0, -1,-1,0,1)
     &  - HX5(0, -1,0,-1,1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,-1,1)
     &  - HX5(0,0, -1,0,-1)
     &  - HX5(0,0, -1,0,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 6.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,1,0,-1)
      Hi6(0,-1,-1,1,1,-1) = 
     &  + 1.1113185040756189d+00
     &  + 1.8403305241553922d+00*HX1(0)
     &  + 8.5513605798213956d-01*HX1(0)*HX1(0)
     &  + 9.7040087744168750d-02*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.8224052646501250d-01*HX1(0)*HX2(0,-1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,-1,-1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,0,-1)
     &  - 1.7102721159642791d+00*HX2(0,-1)
     &  + 3.4657359027997265d-01*HX2(0,-1)*HX2(0,-1)
     &  + 5.8224052646501250d-01*HX3(0,-1,-1)
     &  + 5.8224052646501250d-01*HX3(0,0,-1)
     &  - 6.9314718055994530d-01*HX4(0,-1,-1,1)
     &  + 6.9314718055994530d-01*HX4(0,-1,0,1)
     &  + 6.9314718055994530d-01*HX4(0,0,-1,1)
     &  - 6.9314718055994530d-01*HX4(0,0,0,1)
      Hi6(0,-1,-1,1,1,1) = 
     &  - 5.7407634122175917d+00
     &  - 3.1512791646720465d+00*HX1(0)
     &  - 2.7752054332410789d-02*HX1(0)*HX1(0)
     &  + 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 1.6449340668482264d+00*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  - HX1(0) *HX4(0,-1,-1,1)
     &  + HX1(0) *HX4(0,-1,0,1)
     &  + HX1(0) *HX4(0,0,-1,1)
     &  - HX1(0) *HX4(0,0,0,1)
     &  + 5.5504108664821579d-02*HX2(0,-1)
     &  - HX2(0, -1)*HX3(0,0,-1)
     &  - HX2(0, -1)*HX3(0,0,1)
     &  + 1.6449340668482264d+00*HX3(0,-1,-1)
     &  + 1.6449340668482264d+00*HX3(0,0,-1)
     &  - HX5(0, -1,-1,1,1)
     &  + HX5(0, -1,0,-1,1)
     &  + HX5(0, -1,0,1,1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,-1,1)
     &  + HX5(0,0, -1,0,-1)
     &  + HX5(0,0, -1,1,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - HX5(0,0,0,1,1) 
     &  + HX5(0,0,1,0, -1)
      Hi6(0,-1,0,-1,-1,-1) = 
     &  0.d0 
      Hi6(0,-1,0,-1,-1,1) = 
     &  + 4.8014710645100928d+00
     &  + 1.9151868664230867d+00*HX1(0)
     &  + 2.4085454247160931d-01*HX1(0)*HX1(0)
     &  - 4.0037751159850118d-02*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX1(0)*HX3(0,0,-1)
     &  + 2.4022650695910071d-01*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  - 1.3862943611198906d+00*HX1(0)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - 4.8170908494321862d-01*HX2(0,-1)
     &  - 3.4657359027997265d-01*HX2(0,-1)*HX2(0,-1)
     &  + HX2(0, -1)*HX3(0,0,-1)
     &  - 4.8045301391820142d-01*HX3(0,0,-1)
     &  + 1.3862943611198906d+00*HX4(0,0,-1,-1)
     &  + 2.7725887222397812d+00*HX4(0,0,0,-1)
     &  + HX5(0, -1,0,-1,-1)
     &  - HX5(0,0, -1,0,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,-1)
      Hi6(0,-1,0,-1,0,1) = 
     &  + 7.2522105348184616d+00
     &  + 3.6254766505915635d+00*HX1(0)
     &  + 9.0154267736969571d-01*HX1(0)*HX1(0)
     &  + 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX1(0)*HX3(0,0,-1)
     &  - 8.2246703342411321d-01*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - 1.8030853547393914d+00*HX2(0,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,-1)
     &  + 1.6449340668482264d+00*HX3(0,0,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,-1,0,-1)
     &  - 1.2000000000000000d+01*HX5(0,0,0,-1,-1)
     &  - 8.0000000000000000d+00*HX5(0,0,0,0,-1)
      Hi6(0,-1,0,-1,1,-1) = 
     &  + 2.1155719366955201d+00
     &  + 1.0629473532139836d+00*HX1(0)
     &  + 2.8504535266071318d-01*HX1(0)*HX1(0)
     &  + 8.0075502319700237d-02*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 4.8045301391820142d-01*HX1(0)*HX2(0,-1)
     &  + 1.3862943611198906d+00*HX1(0)*HX3(0,0,-1)
     &  - 5.7009070532142637d-01*HX2(0,-1)
     &  + 3.4657359027997265d-01*HX2(0,-1)*HX2(0,-1)
     &  + 9.6090602783640284d-01*HX3(0,0,-1)
     &  - 1.3862943611198906d+00*HX4(0,0,-1,-1)
     &  - 2.7725887222397812d+00*HX4(0,0,0,-1)
      Hi6(0,-1,0,-1,1,1) = 
     &  - 6.9310070127553493d+00
     &  - 4.0865790601932830d+00*HX1(0)
     &  - 1.3810359531144620d+00*HX1(0)*HX1(0)
     &  - 3.1419342896788785d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX1(0)*HX3(0,0,-1)
     &  + 1.8851605738073271d+00*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 2.7620719062289241d+00*HX2(0,-1)
     &  - 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,-1)
     &  - HX2(0, -1)*HX3(0,0,1)
     &  - 3.7703211476146542d+00*HX3(0,0,-1)
     &  + HX5(0, -1,0,-1,1)
     &  + 4.0000000000000000d+00*HX5(0,0,-1,0,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,0,1)
     &  + 1.2000000000000000d+01*HX5(0,0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 8.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + HX5(0,0,0,0,1) 
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + HX5(0,0,1,0, -1)
      Hi6(0,-1,0,1,-1,-1) = 
     &  + 4.3314889651815499d-01
     &  + 1.9757838252848865d-01*HX1(0)
     &  - 4.0037751159850118d-02*HX1(0)*HX1(0)*HX1(0)
     &  + 2.4022650695910071d-01*HX1(0)*HX2(0,-1)
     &  - 4.8045301391820142d-01*HX3(0,0,-1)
      Hi6(0,-1,0,1,-1,1) = 
     &  + 1.7321298250359006d+00
     &  + 3.0246582143154216d+00*HX1(0)
     &  + 1.3368451429253581d+00*HX1(0)*HX1(0)
     &  + 2.1715334122371910d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX1(0)*HX3(0,0,-1)
     &  - 1.3029200473423146d+00*HX1(0)*HX2(0,-1)
     &  + 1.3862943611198906d+00*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX4(0,-1,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + HX1(0) *HX4(0,0,0,1)
     &  - 2.6736902858507163d+00*HX2(0,-1)
     &  + HX2(0, -1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,1)
     &  + 2.6058400946846292d+00*HX3(0,0,-1)
     &  - 6.9314718055994530d-01*HX4(0,-1,0,1)
     &  - 2.0794415416798359d+00*HX4(0,0,0,-1)
     &  + 6.9314718055994530d-01*HX4(0,0,0,1)
     &  + HX5(0, -1,0,1,-1)
     &  - 3.0000000000000000d+00*HX5(0,0,-1,0,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,0,1)
     &  - 6.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  - 6.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,1,0,-1)
      Hi6(0,-1,0,1,0,1) = 
     &  + 1.6456590415193708d+00
     &  + 3.0843150337359944d+00*HX1(0)
     &  + 1.2020569031595942d+00*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX4(0,-1,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + HX1(0) *HX4(0,0,0,1)
     &  - 2.4041138063191885d+00*HX2(0,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,0,1)
     &  - 6.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 4.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 6.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,1,0,-1)
      Hi6(0,-1,0,1,1,-1) = 
     &  - 1.4718220962743535d+00
     &  + 2.5017842059450777d-01*HX1(0)
     &  + 5.7009070532142637d-01*HX1(0)*HX1(0)
     &  + 9.7040087744168750d-02*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.8224052646501250d-01*HX1(0)*HX2(0,-1)
     &  - 1.3862943611198906d+00*HX1(0)*HX3(0,0,-1)
     &  - 1.1401814106428527d+00*HX2(0,-1)
     &  + 1.1644810529300250d+00*HX3(0,0,-1)
     &  + 6.9314718055994530d-01*HX4(0,-1,0,1)
     &  + 2.0794415416798359d+00*HX4(0,0,0,-1)
     &  - 6.9314718055994530d-01*HX4(0,0,0,1)
      Hi6(0,-1,0,1,1,1) = 
     &  - 1.7203041663629827d+00
     &  - 1.8703132926718733d+00*HX1(0)
     &  + 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX1(0)*HX3(0,0,-1)
     &  - 1.6449340668482264d+00*HX1(0)*HX2(0,-1)
     &  + HX1(0) *HX4(0,-1,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - HX1(0) *HX4(0,0,0,1)
     &  - HX2(0, -1)*HX3(0,0,1)
     &  + 3.2898681336964528d+00*HX3(0,0,-1)
     &  + HX5(0, -1,0,1,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - HX5(0,0,0,1,1) 
     &  + HX5(0,0,1,0, -1)
      Hi6(0,-1,1,-1,-1,-1) = 
     &  + 4.5650299596405422d-02
     &  + 3.8472516430513908d-02*HX1(0)
     &  + 2.7752054332410789d-02*HX1(0)*HX1(0)
     &  - 5.5504108664821579d-02*HX2(0,-1)
      Hi6(0,-1,1,-1,-1,1) = 
     &  - 9.2966196218492791d+00
     &  - 4.0048761475917805d+00*HX1(0)
     &  - 5.2589989513232249d-01*HX1(0)*HX1(0)
     &  + 4.0037751159850118d-02*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  + HX1(0) *HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 2.4022650695910071d-01*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,-1,1)
     &  + 1.3862943611198906d+00*HX1(0)*HX3(0,0,-1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,0,1)
     &  + HX1(0) *HX4(0,-1,0,1)
     &  + HX1(0) *HX4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX4(0,0,1,-1)
     &  + 1.0517997902646449d+00*HX2(0,-1)
     &  + 3.4657359027997265d-01*HX2(0,-1)*HX2(0,-1)
     &  - HX2(0, -1)*HX3(0,0,-1)
     &  - HX2(0, -1)*HX3(0,0,1)
     &  - 2.4022650695910071d-01*HX3(0,-1,1)
     &  + 4.8045301391820142d-01*HX3(0,0,-1)
     &  + 2.4022650695910071d-01*HX3(0,0,1)
     &  + 6.9314718055994530d-01*HX4(0,-1,0,1)
     &  + 6.9314718055994530d-01*HX4(0,-1,1,-1)
     &  - 1.3862943611198906d+00*HX4(0,0,-1,-1)
     &  + 1.3862943611198906d+00*HX4(0,0,-1,1)
     &  - 2.7725887222397812d+00*HX4(0,0,0,-1)
     &  - 2.0794415416798359d+00*HX4(0,0,0,1)
     &  - 6.9314718055994530d-01*HX4(0,0,1,-1)
     &  - HX5(0, -1,0,-1,-1)
     &  - HX5(0, -1,0,1,-1)
     &  - HX5(0, -1,1,-1,-1)
     &  + HX5(0,0, -1,0,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,1,-1)
     &  + 7.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  + 7.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + HX5(0,0,1, -1,-1)
     &  + HX5(0,0,1,0, -1)
      Hi6(0,-1,1,-1,0,1) = 
     &  - 1.2867187224252233d+01
     &  - 6.2823889123009447d+00*HX1(0)
     &  - 1.1865880300304089d+00*HX1(0)*HX1(0)
     &  - 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  + HX1(0) *HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 8.2246703342411321d-01*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  + HX1(0) *HX4(0,-1,0,1)
     &  + HX1(0) *HX4(0,-1,1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX4(0,0,1,-1)
     &  + 2.3731760600608178d+00*HX2(0,-1)
     &  - HX2(0, -1)*HX3(0,-1,1)
     &  - 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,-1)
     &  - HX2(0, -1)*HX3(0,0,1)
     &  + 8.2246703342411321d-01*HX3(0,-1,1)
     &  - 1.6449340668482264d+00*HX3(0,0,-1)
     &  - 8.2246703342411321d-01*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX5(0,-1,0,-1,1)
     &  + 4.0000000000000000d+00*HX5(0,0,-1,-1,1)
     &  + 4.0000000000000000d+00*HX5(0,0,-1,0,-1)
     &  + 1.2000000000000000d+01*HX5(0,0,0,-1,-1)
     &  + 8.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,1,0,-1)
      Hi6(0,-1,1,-1,1,-1) = 
     &  - 4.1130656244888764d+00
     &  - 2.3468070928378121d+00*HX1(0)
     &  - 5.4631068288530563d-01*HX1(0)*HX1(0)
     &  - 8.0075502319700237d-02*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 4.8045301391820142d-01*HX1(0)*HX2(0,-1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,-1,1)
     &  - 1.3862943611198906d+00*HX1(0)*HX3(0,0,-1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,0,1)
     &  + 1.0926213657706112d+00*HX2(0,-1)
     &  - 3.4657359027997265d-01*HX2(0,-1)*HX2(0,-1)
     &  + 4.8045301391820142d-01*HX3(0,-1,1)
     &  - 9.6090602783640284d-01*HX3(0,0,-1)
     &  - 4.8045301391820142d-01*HX3(0,0,1)
     &  - 6.9314718055994530d-01*HX4(0,-1,0,1)
     &  - 6.9314718055994530d-01*HX4(0,-1,1,-1)
     &  + 1.3862943611198906d+00*HX4(0,0,-1,-1)
     &  - 1.3862943611198906d+00*HX4(0,0,-1,1)
     &  + 2.7725887222397812d+00*HX4(0,0,0,-1)
     &  + 2.0794415416798359d+00*HX4(0,0,0,1)
     &  + 6.9314718055994530d-01*HX4(0,0,1,-1)
      Hi6(0,-1,1,-1,1,1) = 
     &  + 1.1325240955246605d+01
     &  + 5.6883244754482793d+00*HX1(0)
     &  + 1.4642921161116944d+00*HX1(0)*HX1(0)
     &  + 3.1419342896788785d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  - HX1(0) *HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.8851605738073271d+00*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX2(0,-1)*HX2(0,-1)
     &  - HX1(0) *HX4(0,-1,0,1)
     &  - HX1(0) *HX4(0,-1,1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + HX1(0) *HX4(0,0,1,-1)
     &  - 2.9285842322233888d+00*HX2(0,-1)
     &  + HX2(0, -1)*HX3(0,-1,1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,1)
     &  - 1.8851605738073271d+00*HX3(0,-1,1)
     &  + 3.7703211476146542d+00*HX3(0,0,-1)
     &  + 1.8851605738073271d+00*HX3(0,0,1)
     &  - 3.0000000000000000d+00*HX5(0,-1,0,-1,1)
     &  - HX5(0, -1,1,-1,1)
     &  + HX5(0, -1,1,0,1)
     &  - 4.0000000000000000d+00*HX5(0,0,-1,-1,1)
     &  - 4.0000000000000000d+00*HX5(0,0,-1,0,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,0,1)
     &  - 1.2000000000000000d+01*HX5(0,0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 8.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 9.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + HX5(0,0,1, -1,1)
     &  - 3.0000000000000000d+00*HX5(0,0,1,0,-1)
     &  - HX5(0,0,1,0,1) 
      Hi6(0,-1,1,0,1,-1) = 
     &  - 4.4356235876722555d+00
     &  - 2.2913064715686529d+00*HX1(0)
     &  - 3.1598309891908395d-01*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,-1,1)
     &  - 1.3862943611198906d+00*HX1(0)*HX3(0,0,-1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,0,1)
     &  + 6.3196619783816790d-01*HX2(0,-1)
     &  - 6.9314718055994530d-01*HX4(0,-1,0,1)
     &  - 1.3862943611198906d+00*HX4(0,0,-1,1)
     &  + 2.0794415416798359d+00*HX4(0,0,0,-1)
     &  + 2.0794415416798359d+00*HX4(0,0,0,1)
      Hi6(0,-1,1,0,1,1) = 
     &  + 6.9766780549234052d+00
     &  + 2.4681204710444169d+00*HX1(0)
     &  + 6.0102845157979714d-01*HX1(0)*HX1(0)
     &  + 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  - HX1(0) *HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 1.6449340668482264d+00*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - 1.2020569031595942d+00*HX2(0,-1)
     &  + 2.0000000000000000d+00*HX2(0,-1)*HX3(0,0,1)
     &  - 1.6449340668482264d+00*HX3(0,-1,1)
     &  + 3.2898681336964528d+00*HX3(0,0,-1)
     &  + 1.6449340668482264d+00*HX3(0,0,1)
     &  + HX5(0, -1,1,0,1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 6.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,1,0,-1)
     &  - HX5(0,0,1,0,1) 
      Hi6(0,-1,1,1,-1,-1) = 
     &  - 9.1752157610522123d-01
     &  - 6.6692697773648615d-01*HX1(0)
     &  - 1.5441268754841696d-01*HX1(0)*HX1(0)
     &  + 4.0037751159850118d-02*HX1(0)*HX1(0)*HX1(0)
     &  - 2.4022650695910071d-01*HX1(0)*HX2(0,-1)
     &  + 3.0882537509683393d-01*HX2(0,-1)
     &  - 2.4022650695910071d-01*HX3(0,-1,1)
     &  + 4.8045301391820142d-01*HX3(0,0,-1)
     &  + 2.4022650695910071d-01*HX3(0,0,1)
      Hi6(0,-1,1,1,-1,1) = 
     &  + 3.1431294397151118d+00
     &  - 8.9422335800838046d-01*HX1(0)
     &  - 1.1350559532618773d+00*HX1(0)*HX1(0)
     &  - 2.1715334122371910d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  - HX1(0) *HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 1.3029200473423146d+00*HX1(0)*HX2(0,-1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,-1,1)
     &  - 1.3862943611198906d+00*HX1(0)*HX3(0,0,-1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,0,1)
     &  + HX1(0) *HX4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX4(0,0,1,1)
     &  + 2.2701119065237547d+00*HX2(0,-1)
     &  - HX2(0, -1)*HX3(0,-1,1)
     &  - HX2(0, -1)*HX3(0,0,-1)
     &  - HX2(0, -1)*HX3(0,0,1)
     &  + 1.3029200473423146d+00*HX3(0,-1,1)
     &  - 2.6058400946846292d+00*HX3(0,0,-1)
     &  - 1.3029200473423146d+00*HX3(0,0,1)
     &  + 6.9314718055994530d-01*HX4(0,-1,1,1)
     &  - 1.3862943611198906d+00*HX4(0,0,-1,1)
     &  + 2.0794415416798359d+00*HX4(0,0,0,-1)
     &  + 1.3862943611198906d+00*HX4(0,0,0,1)
     &  - 6.9314718055994530d-01*HX4(0,0,1,1)
     &  + 2.0000000000000000d+00*HX5(0,-1,0,-1,1)
     &  - HX5(0, -1,0,1,1)
     &  - HX5(0, -1,1,0,1)
     &  - HX5(0, -1,1,1,-1)
     &  + 4.0000000000000000d+00*HX5(0,0,-1,-1,1)
     &  + 3.0000000000000000d+00*HX5(0,0,-1,0,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,0,1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,1,1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 4.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + 2.0000000000000000d+00*HX5(0,0,1,0,-1)
     &  + HX5(0,0,1,0,1) 
     &  + HX5(0,0,1,1, -1)
      Hi6(0,-1,1,1,0,1) = 
     &  + 3.6328517334119136d+00
     &  - 1.2923691489670016d+00*HX1(0)
     &  - 1.2020569031595942d+00*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  - HX1(0) *HX1(0)*HX3(0,0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + HX1(0) *HX4(0,-1,1,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX4(0,0,1,1)
     &  + 2.4041138063191885d+00*HX2(0,-1)
     &  - HX2(0, -1)*HX3(0,0,1)
     &  - HX5(0, -1,0,1,1)
     &  - HX5(0, -1,1,0,1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,0,1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,1,1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + HX5(0,0,1,0, -1)
     &  + HX5(0,0,1,0,1) 
      Hi6(0,-1,1,1,1,-1) = 
     &  + 4.2955535751944745d+00
     &  + 1.5084572119041942d+00*HX1(0)
     &  - 3.0148410851740627d-01*HX1(0)*HX1(0)
     &  - 9.7040087744168750d-02*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,-1)
     &  + 5.8224052646501250d-01*HX1(0)*HX2(0,-1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,-1,1)
     &  + 1.3862943611198906d+00*HX1(0)*HX3(0,0,-1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,0,1)
     &  + 6.0296821703481255d-01*HX2(0,-1)
     &  + 5.8224052646501250d-01*HX3(0,-1,1)
     &  - 1.1644810529300250d+00*HX3(0,0,-1)
     &  - 5.8224052646501250d-01*HX3(0,0,1)
     &  - 6.9314718055994530d-01*HX4(0,-1,1,1)
     &  + 1.3862943611198906d+00*HX4(0,0,-1,1)
     &  - 2.0794415416798359d+00*HX4(0,0,0,-1)
     &  - 1.3862943611198906d+00*HX4(0,0,0,1)
     &  + 6.9314718055994530d-01*HX4(0,0,1,1)
      Hi6(0,-1,1,1,1,1) = 
     &  - 2.1900870176160439d+00
     &  + 8.0212429617572516d-01*HX1(0)
     &  - 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,-1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,-1,1)
     &  + HX1(0) *HX1(0)*HX3(0,0,-1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 1.6449340668482264d+00*HX1(0)*HX2(0,-1)
     &  - HX1(0) *HX4(0,-1,1,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + HX1(0) *HX4(0,0,1,1)
     &  + 1.6449340668482264d+00*HX3(0,-1,1)
     &  - 3.2898681336964528d+00*HX3(0,0,-1)
     &  - 1.6449340668482264d+00*HX3(0,0,1)
     &  - HX5(0, -1,1,1,1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,1,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 4.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + HX5(0,0,1,1,1) 
      Hi6(0,0,-1,-1,-1,-1) = 
     &  0.d0 
      Hi6(0,0,-1,-1,-1,1) = 
     &  - 1.8666934600740517d+00
     &  - 5.2709719078152786d-01*HX1(0)
     &  - 2.7752054332410789d-02*HX1(0)*HX1(0)
     &  - 4.0037751159850118d-02*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 2.4022650695910071d-01*HX3(0,0,-1)
     &  - 6.9314718055994530d-01*HX4(0,0,-1,-1)
     &  - 1.3862943611198906d+00*HX4(0,0,0,-1)
     &  + HX5(0,0, -1,-1,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,-1)
      Hi6(0,0,-1,-1,0,-1) = 
     &  0.d0 
      Hi6(0,0,-1,-1,0,1) = 
     &  - 1.4423080680238724d+00
     &  + 3.3822601053473068d-01*HX1(0)
     &  + 5.2589989513232249d-01*HX1(0)*HX1(0)
     &  + 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - 8.2246703342411321d-01*HX3(0,0,-1)
     &  + HX5(0,0, -1,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,0,-1)
      Hi6(0,0,-1,-1,1,-1) = 
     &  - 9.4056576555066661d-01
     &  - 3.3389529407850318d-01*HX1(0)
     &  + 8.3256162997232369d-02*HX1(0)*HX1(0)
     &  + 8.0075502319700237d-02*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,0,-1)
     &  - 4.8045301391820142d-01*HX3(0,0,-1)
     &  + 6.9314718055994530d-01*HX4(0,0,-1,-1)
     &  + 1.3862943611198906d+00*HX4(0,0,0,-1)
      Hi6(0,0,-1,-1,1,1) = 
     &  - 3.9918507461764148d-01
     &  - 2.0079271680389850d+00*HX1(0)
     &  - 1.2266232655660451d+00*HX1(0)*HX1(0)
     &  - 3.1419342896788785d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 1.8851605738073271d+00*HX3(0,0,-1)
     &  + HX5(0,0, -1,-1,1)
     &  - HX5(0,0, -1,0,-1)
     &  - HX5(0,0, -1,0,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  - HX5(0,0,0, -1,1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + HX5(0,0,0,0,1) 
      Hi6(0,0,-1,0,-1,-1) = 
     &  0.d0 
      Hi6(0,0,-1,0,-1,1) = 
     &  - 5.4842796501570482d+00
     &  - 3.0521251417144869d+00*HX1(0)
     &  - 7.6675443760393181d-01*HX1(0)*HX1(0)
     &  - 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 8.2246703342411321d-01*HX3(0,0,-1)
     &  - 2.0794415416798359d+00*HX4(0,0,0,-1)
     &  - HX5(0,0, -1,0,-1)
     &  + 7.0000000000000000d+00*HX5(0,0,0,0,-1)
      Hi6(0,0,-1,0,0,1) = 
     &  - 5.8327186226814558d+00
     &  - 2.8410984884917377d+00*HX1(0)
     &  - 4.5077133868484785d-01*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,0,-1)
      Hi6(0,0,-1,0,1,-1) = 
     &  - 1.9692994069911702d+00
     &  - 1.2498035299465379d+00*HX1(0)
     &  - 2.8504535266071318d-01*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,0,-1)
     &  + 2.0794415416798359d+00*HX4(0,0,0,-1)
      Hi6(0,0,-1,0,1,1) = 
     &  + 3.4196124183211685d+00
     &  + 6.0880681896251523d-01*HX1(0)
     &  - 5.2589989513232249d-01*HX1(0)*HX1(0)
     &  - 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 1.6449340668482264d+00*HX3(0,0,-1)
     &  - HX5(0,0, -1,0,1)
     &  - 6.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + HX5(0,0,0,0,1) 
      Hi6(0,0,-1,1,-1,-1) = 
     &  - 2.1657444825907749d-01
     &  - 1.9757838252848865d-01*HX1(0)
     &  - 8.3256162997232369d-02*HX1(0)*HX1(0)
     &  - 4.0037751159850118d-02*HX1(0)*HX1(0)*HX1(0)
     &  + 2.4022650695910071d-01*HX3(0,0,-1)
      Hi6(0,0,-1,1,-1,1) = 
     &  + 9.1696925127657645d+00
     &  + 5.7090374034238681d+00*HX1(0)
     &  + 1.5981104731499506d+00*HX1(0)*HX1(0)
     &  + 2.1715334122371910d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + HX1(0) *HX4(0,0,0,1)
     &  - 1.3029200473423146d+00*HX3(0,0,-1)
     &  - 6.9314718055994530d-01*HX4(0,0,-1,1)
     &  + 2.0794415416798359d+00*HX4(0,0,0,-1)
     &  + 6.9314718055994530d-01*HX4(0,0,0,1)
     &  + HX5(0,0, -1,0,-1)
     &  + HX5(0,0, -1,0,1)
     &  + HX5(0,0, -1,1,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - HX5(0,0,0,1, -1)
      Hi6(0,0,-1,1,0,-1) = 
     &  - 7.4148913136148604d-01
     &  - 6.7645202106946136d-01*HX1(0)
     &  - 2.8504535266071318d-01*HX1(0)*HX1(0)
     &  - 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.2246703342411321d-01*HX3(0,0,-1)
      Hi6(0,0,-1,1,0,1) = 
     &  + 9.6962020075462872d+00
     &  + 5.6493805840032952d+00*HX1(0)
     &  + 1.3368451429253581d+00*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + HX1(0) *HX4(0,0,0,1)
     &  + HX5(0,0, -1,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 6.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,1)
      Hi6(0,0,-1,1,1,-1) = 
     &  + 3.1627179711815413d+00
     &  + 2.3933959928473851d+00*HX1(0)
     &  + 8.5513605798213956d-01*HX1(0)*HX1(0)
     &  + 9.7040087744168750d-02*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,0,-1)
     &  - 5.8224052646501250d-01*HX3(0,0,-1)
     &  + 6.9314718055994530d-01*HX4(0,0,-1,1)
     &  - 2.0794415416798359d+00*HX4(0,0,0,-1)
     &  - 6.9314718055994530d-01*HX4(0,0,0,1)
      Hi6(0,0,-1,1,1,1) = 
     &  - 7.2106519853806807d+00
     &  - 3.2396052320380567d+00*HX1(0)
     &  - 2.7752054332410789d-02*HX1(0)*HX1(0)
     &  + 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX4(0,0,-1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - HX1(0) *HX4(0,0,0,1)
     &  - 1.6449340668482264d+00*HX3(0,0,-1)
     &  + HX5(0,0, -1,1,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - HX5(0,0,0,1,1) 
      Hi6(0,0,0,-1,-1,-1) = 
     &  0.d0 
      Hi6(0,0,0,-1,-1,1) = 
     &  + 2.3088625727269735d+00
     &  + 1.3569495655898781d+00*HX1(0)
     &  + 2.4085454247160931d-01*HX1(0)*HX1(0)
     &  - 4.0037751159850118d-02*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1(0) *HX4(0,0,0,-1)
     &  + 6.9314718055994530d-01*HX4(0,0,0,-1)
     &  - HX5(0,0,0, -1,-1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,0,-1)
      Hi6(0,0,0,-1,0,-1) = 
     &  0.d0 
      Hi6(0,0,0,-1,0,1) = 
     &  + 3.8884790817876372d+00
     &  + 2.8410984884917377d+00*HX1(0)
     &  + 9.0154267736969571d-01*HX1(0)*HX1(0)
     &  + 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1(0) *HX4(0,0,0,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,-1)
      Hi6(0,0,0,-1,1,-1) = 
     &  + 6.5643313566372341d-01
     &  + 6.2490176497326899d-01*HX1(0)
     &  + 2.8504535266071318d-01*HX1(0)*HX1(0)
     &  + 8.0075502319700237d-02*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 6.9314718055994530d-01*HX4(0,0,0,-1)
      Hi6(0,0,0,-1,1,1) = 
     &  - 4.8662642295301426d+00
     &  - 3.8055457225523666d+00*HX1(0)
     &  - 1.3810359531144620d+00*HX1(0)*HX1(0)
     &  - 3.1419342896788785d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1(0) *HX4(0,0,0,-1)
     &  - HX5(0,0,0, -1,1)
     &  + 4.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + HX5(0,0,0,0,1) 
      Hi6(0,0,0,0,-1,-1) = 
     &  0.d0 
      Hi6(0,0,0,0,-1,1) = 
     &  - 9.7211977044690930d-01
     &  - 9.4703282949724591d-01*HX1(0)
     &  - 4.5077133868484785d-01*HX1(0)*HX1(0)
     &  - 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX5(0,0,0,0, -1)
      Hi6(0,0,0,0,0,-1) = 
     &  0.d0 
      Hi6(0,0,0,0,0,1) = 
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
      Hi6(0,0,0,0,1,-1) = 
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
      Hi6(0,0,0,0,1,1) = 
     &  - 1.0369277551433699d+00
     &  - 1.0823232337111381d+00*HX1(0)
     &  - 6.0102845157979714d-01*HX1(0)*HX1(0)
     &  - 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX5(0,0,0,0,1) 
      Hi6(0,0,0,1,-1,-1) = 
     &  - 4.0037751159850118d-02*HX1(0)*HX1(0)*HX1(0)
      Hi6(0,0,0,1,-1,1) = 
     &  + 4.4438975477588586d+00
     &  + 3.5336454555719528d+00*HX1(0)
     &  + 1.3368451429253581d+00*HX1(0)*HX1(0)
     &  + 2.1715334122371910d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1(0) *HX4(0,0,0,1)
     &  + 6.9314718055994530d-01*HX4(0,0,0,1)
     &  - HX5(0,0,0,0, -1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - HX5(0,0,0,1, -1)
      Hi6(0,0,0,1,0,-1) = 
     &  - 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)
      Hi6(0,0,0,1,0,1) = 
     &  + 4.1477110205734797d+00
     &  + 3.2469697011334145d+00*HX1(0)
     &  + 1.2020569031595942d+00*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + HX1(0) *HX4(0,0,0,1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,1)
      Hi6(0,0,0,1,1,-1) = 
     &  + 7.5020929790139819d-01
     &  + 8.3320235329769199d-01*HX1(0)
     &  + 5.7009070532142637d-01*HX1(0)*HX1(0)
     &  + 9.7040087744168750d-02*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 6.9314718055994530d-01*HX4(0,0,0,1)
      Hi6(0,0,0,1,1,1) = 
     &  - 3.0142321054406660d+00
     &  - 1.8940656589944918d+00*HX1(0)
     &  + 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - HX1(0) *HX4(0,0,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - HX5(0,0,0,1,1) 
      Hi6(0,0,1,-1,-1,-1) = 
     &  + 2.7752054332410789d-02*HX1(0)*HX1(0)
      Hi6(0,0,1,-1,-1,1) = 
     &  - 6.6971231109688881d+00
     &  - 3.4402173672056757d+00*HX1(0)
     &  - 5.2589989513232249d-01*HX1(0)*HX1(0)
     &  + 4.0037751159850118d-02*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,0,1)
     &  - HX1(0) *HX4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX4(0,0,1,-1)
     &  + 2.4022650695910071d-01*HX3(0,0,1)
     &  - 6.9314718055994530d-01*HX4(0,0,0,-1)
     &  - 2.0794415416798359d+00*HX4(0,0,0,1)
     &  - 6.9314718055994530d-01*HX4(0,0,1,-1)
     &  + HX5(0,0,0, -1,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + HX5(0,0,1, -1,-1)
      Hi6(0,0,1,-1,0,-1) = 
     &  + 1.3478823976576390d-01*HX1(0)*HX1(0)
      Hi6(0,0,1,-1,0,1) = 
     &  - 9.5303793531573246d+00
     &  - 5.6493805840032952d+00*HX1(0)
     &  - 1.1865880300304089d+00*HX1(0)*HX1(0)
     &  - 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - HX1(0) *HX4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX4(0,0,1,-1)
     &  - 8.2246703342411321d-01*HX3(0,0,1)
     &  + 4.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + HX5(0,0,1,0, -1)
      Hi6(0,0,1,-1,1,-1) = 
     &  - 2.4493363846281626d+00
     &  - 1.8532608833279382d+00*HX1(0)
     &  - 5.4631068288530563d-01*HX1(0)*HX1(0)
     &  - 8.0075502319700237d-02*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,0,1)
     &  - 4.8045301391820142d-01*HX3(0,0,1)
     &  + 6.9314718055994530d-01*HX4(0,0,0,-1)
     &  + 2.0794415416798359d+00*HX4(0,0,0,1)
     &  + 6.9314718055994530d-01*HX4(0,0,1,-1)
      Hi6(0,0,1,-1,1,1) = 
     &  + 9.4232350602313439d+00
     &  + 5.4839540823383124d+00*HX1(0)
     &  + 1.4642921161116944d+00*HX1(0)*HX1(0)
     &  + 3.1419342896788785d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + HX1(0) *HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + HX1(0) *HX4(0,0,1,-1)
     &  + 1.8851605738073271d+00*HX3(0,0,1)
     &  + HX5(0,0,0, -1,1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + HX5(0,0,1, -1,1)
     &  - HX5(0,0,1,0, -1)
     &  - HX5(0,0,1,0,1) 
      Hi6(0,0,1,0,-1,-1) = 
     &  + 7.5128556447474642d-02*HX1(0)*HX1(0)
      Hi6(0,0,1,0,-1,1) = 
     &  - 3.8013132901192512d+00
     &  - 1.4179103271406105d+00*HX1(0)
     &  + 1.6572598602413466d-01*HX1(0)*HX1(0)
     &  + 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + 8.2246703342411321d-01*HX3(0,0,1)
     &  - 2.0794415416798359d+00*HX4(0,0,0,1)
     &  - HX5(0,0,0,0, -1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - HX5(0,0,1,0, -1)
      Hi6(0,0,1,0,1,-1) = 
     &  - 2.2506278937041945d+00
     &  - 1.6664047065953839d+00*HX1(0)
     &  - 3.1598309891908395d-01*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,0,1)
     &  + 2.0794415416798359d+00*HX4(0,0,0,1)
      Hi6(0,0,1,0,1,1) = 
     &  + 5.5100003333096471d+00
     &  + 2.4352272758500609d+00*HX1(0)
     &  + 6.0102845157979714d-01*HX1(0)*HX1(0)
     &  + 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + 1.6449340668482264d+00*HX3(0,0,1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - HX5(0,0,1,0,1) 
      Hi6(0,0,1,1,-1,-1) = 
     &  - 2.8876593101210332d-01
     &  - 3.9515676505697730d-01*HX1(0)
     &  - 1.5441268754841696d-01*HX1(0)*HX1(0)
     &  + 4.0037751159850118d-02*HX1(0)*HX1(0)*HX1(0)
     &  + 2.4022650695910071d-01*HX3(0,0,1)
      Hi6(0,0,1,1,-1,1) = 
     &  + 5.1880224993212387d-01
     &  - 1.3317828347103277d+00*HX1(0)
     &  - 1.1350559532618773d+00*HX1(0)*HX1(0)
     &  - 2.1715334122371910d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX4(0,0,1,1)
     &  - 1.3029200473423146d+00*HX3(0,0,1)
     &  + 1.3862943611198906d+00*HX4(0,0,0,1)
     &  - 6.9314718055994530d-01*HX4(0,0,1,1)
     &  + HX5(0,0,0,0, -1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + HX5(0,0,0,1, -1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + HX5(0,0,1,0, -1)
     &  + HX5(0,0,1,0,1) 
     &  + HX5(0,0,1,1, -1)
      Hi6(0,0,1,1,0,-1) = 
     &  - 9.8865217514864805d-01
     &  - 1.3529040421389227d+00*HX1(0)
     &  - 3.7564278223737321d-01*HX1(0)*HX1(0)
     &  + 1.3707783890401886d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.2246703342411321d-01*HX3(0,0,1)
      Hi6(0,0,1,1,0,1) = 
     &  + 1.5553916327150548d+00
     &  - 1.3529040421389227d+00*HX1(0)
     &  - 1.2020569031595942d+00*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - HX1(0) *HX4(0,0,1,1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + HX5(0,0,1,0,1) 
      Hi6(0,0,1,1,1,-1) = 
     &  + 2.0127525154639915d+00
     &  + 9.5774727708194543d-01*HX1(0)
     &  - 3.0148410851740627d-01*HX1(0)*HX1(0)
     &  - 9.7040087744168750d-02*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,0,1)
     &  - 5.8224052646501250d-01*HX3(0,0,1)
     &  - 1.3862943611198906d+00*HX4(0,0,0,1)
     &  + 6.9314718055994530d-01*HX4(0,0,1,1)
      Hi6(0,0,1,1,1,1) = 
     &  - 1.0369277551433699d+00
     &  + 8.1174242528335364d-01*HX1(0)
     &  - 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + HX1(0) *HX4(0,0,1,1)
     &  - 1.6449340668482264d+00*HX3(0,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + HX5(0,0,1,1,1) 
      Hi6(0,1,-1,-1,-1,-1) = 
     &  - 9.6181291076284771d-03*HX1(0)
      Hi6(0,1,-1,-1,-1,1) = 
     &  + 4.3468294982614502d+00
     &  + 1.0778071256037766d+00*HX1(0)
     &  + 2.7752054332410789d-02*HX1(0)*HX1(0)
     &  + 4.0037751159850118d-02*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  + 2.4022650695910071d-01*HX1(0)*HX2(0,1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,0,-1)
     &  - 1.3862943611198906d+00*HX1(0)*HX3(0,0,1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,1,-1)
     &  + HX1(0) *HX4(0,0,-1,-1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,1,-1)
     &  + HX1(0) *HX4(0,1,-1,-1)
     &  + 5.5504108664821579d-02*HX2(0,1)
     &  - 2.4022650695910071d-01*HX3(0,0,-1)
     &  - 4.8045301391820142d-01*HX3(0,0,1)
     &  - 2.4022650695910071d-01*HX3(0,1,-1)
     &  + 6.9314718055994530d-01*HX4(0,0,-1,-1)
     &  + 1.3862943611198906d+00*HX4(0,0,0,-1)
     &  + 2.0794415416798359d+00*HX4(0,0,0,1)
     &  + 1.3862943611198906d+00*HX4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HX4(0,1,-1,-1)
     &  - HX5(0,0, -1,-1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,1,-1,-1)
     &  - HX5(0,1, -1,-1,-1)
      Hi6(0,1,-1,-1,1,-1) = 
     &  + 2.3845769685919721d+00
     &  + 7.7145477078045052d-01*HX1(0)
     &  - 8.3256162997232369d-02*HX1(0)*HX1(0)
     &  - 8.0075502319700237d-02*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  - 4.8045301391820142d-01*HX1(0)*HX2(0,1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,0,-1)
     &  + 1.3862943611198906d+00*HX1(0)*HX3(0,0,1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,1,-1)
     &  - 1.6651232599446473d-01*HX2(0,1)
     &  + 4.8045301391820142d-01*HX3(0,0,-1)
     &  + 9.6090602783640284d-01*HX3(0,0,1)
     &  + 4.8045301391820142d-01*HX3(0,1,-1)
     &  - 6.9314718055994530d-01*HX4(0,0,-1,-1)
     &  - 1.3862943611198906d+00*HX4(0,0,0,-1)
     &  - 2.0794415416798359d+00*HX4(0,0,0,1)
     &  - 1.3862943611198906d+00*HX4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HX4(0,1,-1,-1)
      Hi6(0,1,-1,-1,1,1) = 
     &  - 1.6671394257290846d+00
     &  + 1.7361569553594762d+00*HX1(0)
     &  + 1.2266232655660451d+00*HX1(0)*HX1(0)
     &  + 3.1419342896788785d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX1(0)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  + 1.8851605738073271d+00*HX1(0)*HX2(0,1)
     &  - HX1(0) *HX4(0,0,-1,-1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,1,-1)
     &  - HX1(0) *HX4(0,1,-1,-1)
     &  + HX2(0, -1)*HX3(0,1,-1)
     &  + 2.4532465311320902d+00*HX2(0,1)
     &  - HX2(0,1) *HX3(0,-1,-1)
     &  - HX2(0,1) *HX3(0,0,-1)
     &  - HX2(0,1) *HX3(0,0,1)
     &  + HX2(0,1) *HX3(0,1,-1)
     &  - 1.8851605738073271d+00*HX3(0,0,-1)
     &  - 3.7703211476146542d+00*HX3(0,0,1)
     &  - 1.8851605738073271d+00*HX3(0,1,-1)
     &  + HX5(0, -1,-1,0,1)
     &  + HX5(0, -1,0,-1,1)
     &  + HX5(0,0, -1,-1,1)
     &  + HX5(0,0, -1,0,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,-1,-1)
     &  + 4.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,1,1)
     &  - 2.0000000000000000d+00*HX5(0,0,1,-1,1)
     &  + 2.0000000000000000d+00*HX5(0,0,1,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,1,0,1)
     &  - 4.0000000000000000d+00*HX5(0,0,1,1,-1)
     &  - HX5(0,1, -1,-1,1)
     &  - 2.0000000000000000d+00*HX5(0,1,0,1,-1)
      Hi6(0,1,-1,0,1,1) = 
     &  - 5.1524666469611440d+00
     &  - 7.1295711312472673d-01*HX1(0)
     &  + 5.2589989513232249d-01*HX1(0)*HX1(0)
     &  + 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX1(0)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  - HX1(0) *HX2(0,-1)*HX2(0,1)
     &  + 1.6449340668482264d+00*HX1(0)*HX2(0,1)
     &  + HX1(0) *HX4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + 1.0517997902646449d+00*HX2(0,1)
     &  - HX2(0,1) *HX3(0,-1,1)
     &  + HX2(0,1) *HX3(0,0,-1)
     &  - HX2(0,1) *HX3(0,0,1)
     &  + HX2(0,1) *HX3(0,1,-1)
     &  - 1.6449340668482264d+00*HX3(0,0,-1)
     &  - 3.2898681336964528d+00*HX3(0,0,1)
     &  - 1.6449340668482264d+00*HX3(0,1,-1)
     &  + 2.0000000000000000d+00*HX5(0,-1,0,1,1)
     &  + HX5(0, -1,1,0,1)
     &  + 4.0000000000000000d+00*HX5(0,0,-1,1,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + 3.0000000000000000d+00*HX5(0,0,1,0,1)
     &  - 4.0000000000000000d+00*HX5(0,0,1,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,1,0,1,-1)
      Hi6(0,1,-1,1,-1,-1) = 
     &  + 6.4229127806039709d-01
     &  + 4.0194877563845553d-01*HX1(0)
     &  + 8.3256162997232369d-02*HX1(0)*HX1(0)
     &  + 4.0037751159850118d-02*HX1(0)*HX1(0)*HX1(0)
     &  + 2.4022650695910071d-01*HX1(0)*HX2(0,1)
     &  + 1.6651232599446473d-01*HX2(0,1)
     &  - 2.4022650695910071d-01*HX3(0,0,-1)
     &  - 4.8045301391820142d-01*HX3(0,0,1)
     &  - 2.4022650695910071d-01*HX3(0,1,-1)
      Hi6(0,1,-1,1,-1,1) = 
     &  - 1.1870349507769848d+01
     &  - 6.2025836129337419d+00*HX1(0)
     &  - 1.5981104731499506d+00*HX1(0)*HX1(0)
     &  - 2.1715334122371910d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  + HX1(0) *HX1(0)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  - HX1(0) *HX2(0,-1)*HX2(0,1)
     &  - 1.3029200473423146d+00*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX2(0,1)*HX2(0,1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,0,-1)
     &  + 1.3862943611198906d+00*HX1(0)*HX3(0,0,1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,1,-1)
     &  + HX1(0) *HX4(0,-1,0,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  - 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,1,1)
     &  + HX1(0) *HX4(0,1,-1,1)
     &  - 6.9314718055994530d-01*HX2(0,-1)*HX2(0,1)
     &  - HX2(0, -1)*HX3(0,1,-1)
     &  - 3.1962209462999012d+00*HX2(0,1)
     &  - 3.4657359027997265d-01*HX2(0,1)*HX2(0,1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX3(0,-1,-1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX3(0,0,1)
     &  - HX2(0,1) *HX3(0,1,-1)
     &  + 1.3029200473423146d+00*HX3(0,0,-1)
     &  + 2.6058400946846292d+00*HX3(0,0,1)
     &  + 1.3029200473423146d+00*HX3(0,1,-1)
     &  + 6.9314718055994530d-01*HX4(0,-1,0,1)
     &  + 2.0794415416798359d+00*HX4(0,0,-1,1)
     &  - 2.0794415416798359d+00*HX4(0,0,0,-1)
     &  - 2.7725887222397812d+00*HX4(0,0,0,1)
     &  + 1.3862943611198906d+00*HX4(0,0,1,1)
     &  + 6.9314718055994530d-01*HX4(0,1,-1,1)
     &  - 2.0000000000000000d+00*HX5(0,-1,-1,0,1)
     &  - 2.0000000000000000d+00*HX5(0,-1,0,-1,1)
     &  - HX5(0, -1,0,1,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,-1,-1,1)
     &  - HX5(0,0, -1,0,-1)
     &  - 3.0000000000000000d+00*HX5(0,0,-1,0,1)
     &  - 3.0000000000000000d+00*HX5(0,0,-1,1,-1)
     &  - 9.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 7.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 8.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 1.2000000000000000d+01*HX5(0,0,0,1,1)
     &  - 2.0000000000000000d+00*HX5(0,0,1,0,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,1,0,1)
     &  + 4.0000000000000000d+00*HX5(0,0,1,1,-1)
     &  - HX5(0,1, -1,1,-1)
     &  + 3.0000000000000000d+00*HX5(0,1,0,1,-1)
      Hi6(0,1,-1,1,1,-1) = 
     &  - 5.3579181501311143d+00
     &  - 2.9580547732334898d+00*HX1(0)
     &  - 8.5513605798213956d-01*HX1(0)*HX1(0)
     &  - 9.7040087744168750d-02*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  - 5.8224052646501250d-01*HX1(0)*HX2(0,1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,0,-1)
     &  - 1.3862943611198906d+00*HX1(0)*HX3(0,0,1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,1,-1)
     &  + 6.9314718055994530d-01*HX2(0,-1)*HX2(0,1)
     &  - 1.7102721159642791d+00*HX2(0,1)
     &  + 3.4657359027997265d-01*HX2(0,1)*HX2(0,1)
     &  + 5.8224052646501250d-01*HX3(0,0,-1)
     &  + 1.1644810529300250d+00*HX3(0,0,1)
     &  + 5.8224052646501250d-01*HX3(0,1,-1)
     &  - 6.9314718055994530d-01*HX4(0,-1,0,1)
     &  - 2.0794415416798359d+00*HX4(0,0,-1,1)
     &  + 2.0794415416798359d+00*HX4(0,0,0,-1)
     &  + 2.7725887222397812d+00*HX4(0,0,0,1)
     &  - 1.3862943611198906d+00*HX4(0,0,1,1)
     &  - 6.9314718055994530d-01*HX4(0,1,-1,1)
      Hi6(0,1,-1,1,1,1) = 
     &  + 8.5320965724821486d+00
     &  + 3.2780777484685706d+00*HX1(0)
     &  + 2.7752054332410789d-02*HX1(0)*HX1(0)
     &  - 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,-1)
     &  - HX1(0) *HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,-1)
     &  + HX1(0) *HX2(0,-1)*HX2(0,1)
     &  - 1.6449340668482264d+00*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(0,1)*HX2(0,1)
     &  - HX1(0) *HX4(0,-1,0,1)
     &  - 3.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  + 3.0000000000000000d+00*HX1(0)*HX4(0,0,0,-1)
     &  + 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,1,1)
     &  - HX1(0) *HX4(0,1,-1,1)
     &  + 5.5504108664821579d-02*HX2(0,1)
     &  + HX2(0,1) *HX3(0,-1,1)
     &  - HX2(0,1) *HX3(0,0,-1)
     &  - HX2(0,1) *HX3(0,0,1)
     &  + 1.6449340668482264d+00*HX3(0,0,-1)
     &  + 3.2898681336964528d+00*HX3(0,0,1)
     &  + 1.6449340668482264d+00*HX3(0,1,-1)
     &  - 2.0000000000000000d+00*HX5(0,-1,0,1,1)
     &  - HX5(0, -1,1,0,1)
     &  + HX5(0,0, -1,0,1)
     &  - 5.0000000000000000d+00*HX5(0,0,-1,1,1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 6.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 7.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + HX5(0,0,1,0,1) 
     &  - HX5(0,1, -1,1,1)
     &  + HX5(0,1,0,1,1) 
      Hi6(0,1,0,1,-1,-1) = 
     &  + 5.7753186202420665d-01
     &  + 2.8103333764091645d-01*HX1(0)
     &  + 4.0037751159850118d-02*HX1(0)*HX1(0)*HX1(0)
     &  + 2.4022650695910071d-01*HX1(0)*HX2(0,1)
     &  - 4.8045301391820142d-01*HX3(0,0,1)
      Hi6(0,1,0,1,-1,1) = 
     &  - 7.1382311691019447d+00
     &  - 3.9716910438126675d+00*HX1(0)
     &  - 1.3368451429253581d+00*HX1(0)*HX1(0)
     &  - 2.1715334122371910d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  + HX1(0) *HX1(0)*HX3(0,0,1)
     &  - 1.3029200473423146d+00*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX2(0,1)*HX2(0,1)
     &  + 1.3862943611198906d+00*HX1(0)*HX3(0,0,1)
     &  - 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,1,1)
     &  - 2.6736902858507163d+00*HX2(0,1)
     &  - 3.4657359027997265d-01*HX2(0,1)*HX2(0,1)
     &  + HX2(0,1) *HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX3(0,0,1)
     &  + 2.6058400946846292d+00*HX3(0,0,1)
     &  - 2.7725887222397812d+00*HX4(0,0,0,1)
     &  + 1.3862943611198906d+00*HX4(0,0,1,1)
     &  - HX5(0,0, -1,0,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + HX5(0,0,0,0, -1)
     &  + 8.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 1.2000000000000000d+01*HX5(0,0,0,1,1)
     &  - 2.0000000000000000d+00*HX5(0,0,1,0,-1)
     &  - 4.0000000000000000d+00*HX5(0,0,1,0,1)
     &  + HX5(0,1,0,1, -1)
      Hi6(0,1,0,1,1,-1) = 
     &  - 3.0877434085512353d+00
     &  - 1.3914396541309006d+00*HX1(0)
     &  - 5.7009070532142637d-01*HX1(0)*HX1(0)
     &  - 9.7040087744168750d-02*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  - 5.8224052646501250d-01*HX1(0)*HX2(0,1)
     &  - 1.3862943611198906d+00*HX1(0)*HX3(0,0,1)
     &  - 1.1401814106428527d+00*HX2(0,1)
     &  + 3.4657359027997265d-01*HX2(0,1)*HX2(0,1)
     &  + 1.1644810529300250d+00*HX3(0,0,1)
     &  + 2.7725887222397812d+00*HX4(0,0,0,1)
     &  - 1.3862943611198906d+00*HX4(0,0,1,1)
      Hi6(0,1,0,1,1,1) = 
     &  + 4.1477110205734797d+00
     &  + 1.8940656589944918d+00*HX1(0)
     &  - 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - HX1(0) *HX1(0)*HX3(0,0,1)
     &  - 1.6449340668482264d+00*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(0,1)*HX2(0,1)
     &  + 4.0000000000000000d+00*HX1(0)*HX4(0,0,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,1,1)
     &  - HX2(0,1) *HX3(0,0,1)
     &  + 3.2898681336964528d+00*HX3(0,0,1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 7.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + HX5(0,0,1,0,1) 
     &  + HX5(0,1,0,1,1) 
      Hi6(0,1,1,-1,-1,-1) = 
     &  + 9.1300599192810844d-02
     &  + 8.8326067366010207d-02*HX1(0)
     &  - 2.7752054332410789d-02*HX1(0)*HX1(0)
     &  - 5.5504108664821579d-02*HX2(0,1)
      Hi6(0,1,1,-1,-1,1) = 
     &  + 4.0094369888298342d+00
     &  + 2.8871518985136828d+00*HX1(0)
     &  + 5.2589989513232249d-01*HX1(0)*HX1(0)
     &  - 4.0037751159850118d-02*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,1)
     &  + HX1(0) *HX2(0,-1)*HX2(0,1)
     &  - 2.4022650695910071d-01*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX2(0,1)*HX2(0,1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,0,1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,1,1)
     &  - HX1(0) *HX4(0,-1,0,1)
     &  - 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  + HX1(0) *HX4(0,0,0,-1)
     &  - HX1(0) *HX4(0,0,1,-1)
     &  + HX1(0) *HX4(0,1,1,-1)
     &  + 6.9314718055994530d-01*HX2(0,-1)*HX2(0,1)
     &  + 1.0517997902646449d+00*HX2(0,1)
     &  + 3.4657359027997265d-01*HX2(0,1)*HX2(0,1)
     &  - HX2(0,1) *HX3(0,-1,-1)
     &  - HX2(0,1) *HX3(0,0,-1)
     &  - HX2(0,1) *HX3(0,0,1)
     &  + 2.4022650695910071d-01*HX3(0,0,1)
     &  - 2.4022650695910071d-01*HX3(0,1,1)
     &  - 6.9314718055994530d-01*HX4(0,-1,0,1)
     &  - 1.3862943611198906d+00*HX4(0,0,-1,1)
     &  + 6.9314718055994530d-01*HX4(0,0,0,-1)
     &  - 6.9314718055994530d-01*HX4(0,0,1,-1)
     &  + 6.9314718055994530d-01*HX4(0,1,1,-1)
     &  + HX5(0, -1,-1,0,1)
     &  + HX5(0, -1,0,-1,1)
     &  + HX5(0, -1,0,1,-1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,-1,1)
     &  + HX5(0,0, -1,0,1)
     &  + 2.0000000000000000d+00*HX5(0,0,-1,1,-1)
     &  - HX5(0,0,0, -1,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + HX5(0,0,1, -1,-1)
     &  + HX5(0,0,1,0,1) 
     &  - 2.0000000000000000d+00*HX5(0,0,1,1,-1)
     &  - HX5(0,1,0,1, -1)
     &  - HX5(0,1,1, -1,-1)
      Hi6(0,1,1,-1,1,-1) = 
     &  + 6.1139662780889303d-01
     &  + 1.3225031739147567d+00*HX1(0)
     &  + 5.4631068288530563d-01*HX1(0)*HX1(0)
     &  + 8.0075502319700237d-02*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  + 4.8045301391820142d-01*HX1(0)*HX2(0,1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,0,1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,1,1)
     &  - 6.9314718055994530d-01*HX2(0,-1)*HX2(0,1)
     &  + 1.0926213657706112d+00*HX2(0,1)
     &  - 3.4657359027997265d-01*HX2(0,1)*HX2(0,1)
     &  - 4.8045301391820142d-01*HX3(0,0,1)
     &  + 4.8045301391820142d-01*HX3(0,1,1)
     &  + 6.9314718055994530d-01*HX4(0,-1,0,1)
     &  + 1.3862943611198906d+00*HX4(0,0,-1,1)
     &  - 6.9314718055994530d-01*HX4(0,0,0,-1)
     &  + 6.9314718055994530d-01*HX4(0,0,1,-1)
     &  - 6.9314718055994530d-01*HX4(0,1,1,-1)
      Hi6(0,1,1,-1,1,1) = 
     &  - 7.6596712484742145d+00
     &  - 5.3440844744555946d+00*HX1(0)
     &  - 1.4642921161116944d+00*HX1(0)*HX1(0)
     &  - 3.1419342896788785d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,1)
     &  - HX1(0) *HX2(0,-1)*HX2(0,1)
     &  - 1.8851605738073271d+00*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX2(0,1)*HX2(0,1)
     &  + HX1(0) *HX4(0,-1,0,1)
     &  + 2.0000000000000000d+00*HX1(0)*HX4(0,0,-1,1)
     &  - HX1(0) *HX4(0,0,0,-1)
     &  + HX1(0) *HX4(0,0,1,-1)
     &  - HX1(0) *HX4(0,1,1,-1)
     &  + HX2(0, -1)*HX3(0,1,1)
     &  - 2.9285842322233888d+00*HX2(0,1)
     &  - 2.0000000000000000d+00*HX2(0,1)*HX3(0,-1,1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX3(0,0,-1)
     &  + 2.0000000000000000d+00*HX2(0,1)*HX3(0,0,1)
     &  + HX2(0,1) *HX3(0,1,1)
     &  + 1.8851605738073271d+00*HX3(0,0,1)
     &  - 1.8851605738073271d+00*HX3(0,1,1)
     &  + 3.0000000000000000d+00*HX5(0,-1,0,1,1)
     &  + 2.0000000000000000d+00*HX5(0,-1,1,0,1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,0,1)
     &  + 6.0000000000000000d+00*HX5(0,0,-1,1,1)
     &  - 7.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  + 4.0000000000000000d+00*HX5(0,0,0,0,-1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,0,1)
     &  - 3.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  - 9.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + HX5(0,0,1, -1,1)
     &  - HX5(0,0,1,0, -1)
     &  - 3.0000000000000000d+00*HX5(0,0,1,0,1)
     &  - 6.0000000000000000d+00*HX5(0,0,1,1,1)
     &  - 3.0000000000000000d+00*HX5(0,1,0,1,1)
     &  - HX5(0,1,1, -1,1)
      Hi6(0,1,1,1,-1,-1) = 
     &  - 5.0799738020396403d-01
     &  + 6.3547193707269457d-02*HX1(0)
     &  + 1.5441268754841696d-01*HX1(0)*HX1(0)
     &  - 4.0037751159850118d-02*HX1(0)*HX1(0)*HX1(0)
     &  - 2.4022650695910071d-01*HX1(0)*HX2(0,1)
     &  + 3.0882537509683393d-01*HX2(0,1)
     &  + 2.4022650695910071d-01*HX3(0,0,1)
     &  - 2.4022650695910071d-01*HX3(0,1,1)
      Hi6(0,1,1,1,-1,1) = 
     &  + 2.0304311292496503d+00
     &  + 1.7041506452193448d+00*HX1(0)
     &  + 1.1350559532618773d+00*HX1(0)*HX1(0)
     &  + 2.1715334122371910d-01*HX1(0)*HX1(0)*HX1(0)
     &  + 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  + 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,1)
     &  + 1.3029200473423146d+00*HX1(0)*HX2(0,1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,0,1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,1,1)
     &  + HX1(0) *HX4(0,0,0,1)
     &  - HX1(0) *HX4(0,0,1,1)
     &  + HX1(0) *HX4(0,1,1,1)
     &  - HX2(0, -1)*HX3(0,1,1)
     &  + 2.2701119065237547d+00*HX2(0,1)
     &  + HX2(0,1) *HX3(0,-1,1)
     &  - HX2(0,1) *HX3(0,0,-1)
     &  - HX2(0,1) *HX3(0,0,1)
     &  - HX2(0,1) *HX3(0,1,1)
     &  - 1.3029200473423146d+00*HX3(0,0,1)
     &  + 1.3029200473423146d+00*HX3(0,1,1)
     &  + 6.9314718055994530d-01*HX4(0,0,0,1)
     &  - 6.9314718055994530d-01*HX4(0,0,1,1)
     &  + 6.9314718055994530d-01*HX4(0,1,1,1)
     &  - HX5(0, -1,0,1,1)
     &  - HX5(0, -1,1,0,1)
     &  + HX5(0,0, -1,0,1)
     &  - 2.0000000000000000d+00*HX5(0,0,-1,1,1)
     &  + 3.0000000000000000d+00*HX5(0,0,0,-1,1)
     &  - HX5(0,0,0,0, -1)
     &  - 2.0000000000000000d+00*HX5(0,0,0,0,1)
     &  + 2.0000000000000000d+00*HX5(0,0,0,1,-1)
     &  + 6.0000000000000000d+00*HX5(0,0,0,1,1)
     &  + HX5(0,0,1,0, -1)
     &  + 2.0000000000000000d+00*HX5(0,0,1,0,1)
     &  + HX5(0,0,1,1, -1)
     &  + 4.0000000000000000d+00*HX5(0,0,1,1,1)
     &  + 2.0000000000000000d+00*HX5(0,1,0,1,1)
     &  - HX5(0,1,1,1, -1)
      Hi6(0,1,1,1,1,-1) = 
     &  + 3.2102330990014856d-01
     &  - 4.4026821540804604d-01*HX1(0)
     &  + 3.0148410851740627d-01*HX1(0)*HX1(0)
     &  + 9.7040087744168750d-02*HX1(0)*HX1(0)*HX1(0)
     &  - 2.8881132523331054d-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 3.4657359027997265d-01*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.8224052646501250d-01*HX1(0)*HX2(0,1)
     &  + 6.9314718055994530d-01*HX1(0)*HX3(0,0,1)
     &  - 6.9314718055994530d-01*HX1(0)*HX3(0,1,1)
     &  + 6.0296821703481255d-01*HX2(0,1)
     &  - 5.8224052646501250d-01*HX3(0,0,1)
     &  + 5.8224052646501250d-01*HX3(0,1,1)
     &  - 6.9314718055994530d-01*HX4(0,0,0,1)
     &  + 6.9314718055994530d-01*HX4(0,0,1,1)
     &  - 6.9314718055994530d-01*HX4(0,1,1,1)
      Hi6(0,1,1,1,1,1) = 
     &  - 8.1174242528335364d-01*HX1(0)
     &  + 2.7415567780803773d-01*HX1(0)*HX1(0)*HX1(0)
     &  - 8.3333333333333333d-03*HX1(0)*HX1(0)*HX1(0)*HX1(0)*HX1(0)
     &  - 1.6666666666666666d-01*HX1(0)*HX1(0)*HX1(0)*HX2(0,1)
     &  + 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,0,1)
     &  - 5.0000000000000000d-01*HX1(0)*HX1(0)*HX3(0,1,1)
     &  + 1.6449340668482264d+00*HX1(0)*HX2(0,1)
     &  - HX1(0) *HX4(0,0,0,1)
     &  + HX1(0) *HX4(0,0,1,1)
     &  - HX1(0) *HX4(0,1,1,1)
     &  - 1.6449340668482264d+00*HX3(0,0,1)
     &  + 1.6449340668482264d+00*HX3(0,1,1)
     &  + HX5(0,0,0,0,1) 
     &  - HX5(0,0,0,1,1) 
     &  + HX5(0,0,1,1,1) 
     &  - HX5(0,1,1,1,1) 
      endif 
** nw > 5 endif 
      endif 
** (n1,n2) = (-1,1) -- completion endif 
      return 
      end 
************************************************************************ 
      subroutine fillirr1dhplin1(y,nw,HY1,HY2,HY3,HY4,HY5,HY6,n1,n2) 
** evaluates the irreducible HPL for y =1
** it is guaranteed that nw is in the range 2:4, and that (n1,n2) 
** take one of the pairs of values (0,1), (-1,0) or (-1,1) 
      implicit double precision (a-h,o-z) 
      dimension HY1(n1:n2),HY2(n1:n2,n1:n2),HY3(n1:n2,n1:n2,n1:n2), 
     &          HY4(n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HY5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2), 
     &          HY6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2) 
** (n1,n2) = (0,1) or (-1,1) 
      if (    ( (n1.eq.0).and.(n2.eq.1) ) 
     &    .or.( (n1.eq.-1).and.(n2.eq.1) ) ) then 
      HY2(0,1) = 
     &  + 1.6449340668482264d+00
      if (nw.gt.2) then 
      HY3(0,0,1) = 
     &  + 1.2020569031595942d+00
      HY3(0,1,1) = 
     &  + 1.2020569031595942d+00
      endif
      if (nw.gt.3) then 
      HY4(0,0,0,1) = 
     &  + 1.0823232337111381d+00
      HY4(0,0,1,1) = 
     &  + 2.7058080842778454d-01
      HY4(0,1,1,1) = 
     &  + 1.0823232337111381d+00
      endif
      if (nw.gt.4) then
      HY5(0,0,0,0,1) = 
     &  + 1.0369277551433699d+00
      HY5(0,0,0,1,1) = 
     &  + 9.6551159989443734d-02
      HY5(0,0,1,0,1) = 
     &  + 2.2881039760335375d-01
      HY5(0,0,1,1,1) = 
     &  + 9.6551159989443734d-02
      HY5(0,1,0,1,1) = 
     &  + 2.2881039760335375d-01
      HY5(0,1,1,1,1) = 
     &  + 1.0369277551433699d+00
      endif
      endif
** (n1,n2) = (0,1) or (-1,1) endif 
************ 
** (n1,n2) = (-1,0) or (-1,1) 
      if (    ( (n1.eq.-1).and.(n2.eq.0) ) 
     &    .or.( (n1.eq.-1).and.(n2.eq.1) ) ) then 
      HY2(0,-1) = 
     &  + 8.2246703342411321d-01
      if (nw.gt.2) then
      HY3(0,-1,-1) = 
     &  + 1.5025711289494928d-01
      HY3(0,0,-1) = 
     &  + 9.0154267736969571d-01
      endif
      if (nw.gt.3) then
      HY4(0,-1,-1,-1) = 
     &  + 2.3752366322618485d-02
      HY4(0,0,-1,-1) = 
     &  + 8.7785671568655302d-02
      HY4(0,0,0,-1) = 
     &  + 9.4703282949724591d-01
      endif
      if (nw.gt.4) then
      HY5(0,0,0,0,-1) = 
     &  + 9.7211977044690930d-01
      HY5(0,0,0,-1,-1) = 
     &  + 4.8936397049969063d-02
      HY5(0,0,-1,0,-1) = 
     &  + 9.2748467341632644d-02
      HY5(0,0,-1,-1,-1) = 
     &  + 9.6015684431298325d-03
      HY5(0,-1,0,-1,-1) = 
     &  + 1.3531263989594243d-02
      HY5(0,-1,-1,-1,-1) = 
     &  + 3.1350096016808622d-03
      endif
      endif 
** (n1,n2) = (-1,0) or (-1,1) endif 
** (n1,n2) = (-1,1) -- completion 
      if ( (n1.eq.-1).and.(n2.eq.1) ) then 
      HY2(-1,1) = 
     &  + 5.8224052646501250d-01
      if (nw.gt.2) then
      HY3(0,-1,1) = 
     &  + 2.4307035167006157d-01
      HY3(0,1,-1) = 
     &  + 5.0821521280468485d-01
      HY3(-1,-1,1) = 
     &  + 9.4753004230127705d-02
      HY3(-1,1,1) = 
     &  + 5.3721319360804020d-01
      endif
      if (nw.gt.3) then 
      HY4(0,0,-1,1) = 
     &  + 1.1787599965050932d-01
      HY4(0,0,1,-1) = 
     &  + 1.7284527823898438d-01
      HY4(0,-1,0,1) = 
     &  + 2.0293560632083841d-01
      HY4(0,-1,-1,1) = 
     &  + 3.4159126166513913d-02
      HY4(0,-1,1,-1) = 
     &  + 5.4653052738263652d-02
      HY4(0,1,-1,-1) = 
     &  + 1.1412342741606084d-01
      HY4(0,-1,1,1) = 
     &  + 9.3097125991768577d-02
      HY4(0,1,-1,1) = 
     &  + 1.9355535381306524d-01
      HY4(0,1,1,-1) = 
     &  + 4.3369237704895519d-01
      HY4(-1,-1,-1,1) = 
     &  + 1.4134237214990008d-02
      HY4(-1,-1,1,1) = 
     &  + 4.0758239159309251d-02
      HY4(-1,1,1,1) = 
     &  + 5.1747906167389938d-01
      endif
      if (nw.gt.4) then 
      HY5(-1,-1,-1,-1,1) = 
     &  + 1.8016537870380179d-03
      HY5(-1,-1,-1,1,1) = 
     &  + 3.8760673146652637d-03
      HY5(-1,-1,1,-1,1) = 
     &  + 6.2154684604081354d-03
      HY5(-1,-1,1,1,1) = 
     &  + 1.8530786065466613d-02
      HY5(-1,1,-1,1,1) = 
     &  + 3.8880058841843904d-02
      HY5(-1,1,1,1,1) = 
     &  + 5.0840057924226870d-01
      HY5(0,-1,-1,-1,1) = 
     &  + 4.1914400448554060d-03
      HY5(0,-1,-1,0,1) = 
     &  + 3.0172237496701167d-02
      HY5(0,-1,-1,1,-1) = 
     &  + 5.9459097989450212d-03
      HY5(0,-1,-1,1,1) = 
     &  + 8.7734377821481916d-03
      HY5(0,-1,0,-1,1) = 
     &  + 1.7042475614121991d-02
      HY5(0,-1,0,1,-1) = 
     &  + 2.2495758621687517d-02
      HY5(0,-1,0,1,1) = 
     &  + 3.0833054551948363d-02
      HY5(0,-1,1,-1,-1) = 
     &  + 9.4133341974174110d-03
      HY5(0,-1,1,-1,1) = 
     &  + 1.3833955759762555d-02
      HY5(0,-1,1,0,1) = 
     &  + 7.6026642213084631d-02
      HY5(0,-1,1,1,-1) = 
     &  + 2.2801059128486651d-02
      HY5(0,-1,1,1,1) = 
     &  + 3.9984858137537496d-02
      HY5(0,0,-1,-1,1) = 
     &  + 1.2444228784499648d-02
      HY5(0,0,-1,0,1) = 
     &  + 1.0679981350605469d-01
      HY5(0,0,-1,1,-1) = 
     &  + 1.6991592326175436d-02
      HY5(0,0,-1,1,1) = 
     &  + 2.4107342184124538d-02
      HY5(0,0,0,-1,1) = 
     &  + 5.9142607400864533d-02
      HY5(0,0,0,1,-1) = 
     &  + 7.4276054639867797d-02
      HY5(0,0,1,-1,-1) = 
     &  + 2.5535023438634211d-02
      HY5(0,0,1,-1,1) = 
     &  + 3.6321732111088421d-02
      HY5(0,0,1,0,-1) = 
     &  + 1.8615775173851248d-01
      HY5(0,0,1,1,-1) = 
     &  + 5.7353571803049304d-02
      HY5(0,1,-1,-1,-1) = 
     &  + 1.9555438852482933d-02
      HY5(0,1,-1,-1,1) = 
     &  + 2.8668668263701248d-02
      HY5(0,1,-1,1,-1) = 
     &  + 4.7069633474401836d-02
      HY5(0,1,-1,1,1) = 
     &  + 8.2208743029471844d-02
      HY5(0,1,0,1,-1) = 
     &  + 1.4122347902560834d-01
      HY5(0,1,1,-1,-1) = 
     &  + 1.0254618242743082d-01
      HY5(0,1,1,-1,1) = 
     &  + 1.7692012816527167d-01
      HY5(0,1,1,1,-1) = 
     &  + 4.0707197178331534d-01
      endif
      if (nw.gt.5) then
* 2017-04-27:17:10:16.hpl 
* <- hpl@1.out 
* produced by form-to-fortr for gehrt@tintin 
      HY6(-1,-1,-1,-1,-1,1) = 
     &  + 1.9723282021162115d-04
      HY6(-1,-1,-1,-1,1,1) = 
     &  + 3.5960083109555052d-04
      HY6(-1,-1,-1,1,-1,1) = 
     &  + 5.0818576926545529d-04
      HY6(-1,-1,-1,1,1,1) = 
     &  + 1.1224699796583776d-03
      HY6(-1,-1,1,-1,1,1) = 
     &  + 1.7659263260696900d-03
      HY6(-1,-1,1,1,-1,1) = 
     &  + 2.8637640787495101d-03
      HY6(-1,-1,1,1,1,1) = 
     &  + 8.7230030575968884d-03
      HY6(-1,1,-1,1,1,1) = 
     &  + 1.7982121549905783d-02
      HY6(-1,1,1,1,1,1) = 
     &  + 5.0409539780398855d-01
      HY6(0,-1,-1,-1,-1,-1) = 
     &  + 3.5126812414543725d-04
      HY6(0,-1,-1,-1,-1,1) = 
     &  + 4.4712505220528870d-04
      HY6(0,-1,-1,-1,0,1) = 
     &  + 3.8169691380679236d-03
      HY6(0,-1,-1,-1,1,-1) = 
     &  + 5.9109948115862196d-04
      HY6(0,-1,-1,-1,1,1) = 
     &  + 8.0303778050832893d-04
      HY6(0,-1,-1,0,-1,1) = 
     &  + 2.1132277544551763d-03
      HY6(0,-1,-1,0,1,-1) = 
     &  + 2.6245807975856978d-03
      HY6(0,-1,-1,0,1,1) = 
     &  + 3.3406073160092585d-03
      HY6(0,-1,-1,1,-1,-1) = 
     &  + 8.3026152441714754d-04
      HY6(0,-1,-1,1,-1,1) = 
     &  + 1.1248758327797278d-03
      HY6(0,-1,-1,1,0,1) = 
     &  + 7.6707164631481205d-03
      HY6(0,-1,-1,1,1,-1) = 
     &  + 1.6236379987219099d-03
      HY6(0,-1,-1,1,1,1) = 
     &  + 2.4398626538751752d-03
      HY6(0,-1,0,-1,-1,-1) = 
     &  + 1.2319555588932807d-03
      HY6(0,-1,0,-1,-1,1) = 
     &  + 1.5215749746543206d-03
      HY6(0,-1,0,-1,0,1) = 
     &  + 1.5846750607844291d-02
      HY6(0,-1,0,-1,1,-1) = 
     &  + 1.9408729584885743d-03
      HY6(0,-1,0,-1,1,1) = 
     &  + 2.5395172104386521d-03
      HY6(0,-1,0,1,-1,-1) = 
     &  + 2.6053618567018756d-03
      HY6(0,-1,0,1,-1,1) = 
     &  + 3.4140232739564528d-03
      HY6(0,-1,0,1,0,1) = 
     &  + 2.7716833977494091d-02
      HY6(0,-1,0,1,1,-1) = 
     &  + 4.7285610638330759d-03
      HY6(0,-1,0,1,1,1) = 
     &  + 6.8128613470480598d-03
      HY6(0,-1,1,-1,-1,-1) = 
     &  + 1.3039042223628135d-03
      HY6(0,-1,1,-1,-1,1) = 
     &  + 1.7625670847439686d-03
      HY6(0,-1,1,-1,0,1) = 
     &  + 1.2115413251344344d-02
      HY6(0,-1,1,-1,1,-1) = 
     &  + 2.5362110264309006d-03
      HY6(0,-1,1,-1,1,1) = 
     &  + 3.7985276723318908d-03
      HY6(0,-1,1,0,1,-1) = 
     &  + 9.2351089277639264d-03
      HY6(0,-1,1,0,1,1) = 
     &  + 1.2882812075917590d-02
      HY6(0,-1,1,1,-1,-1) = 
     &  + 4.0996568687826499d-03
      HY6(0,-1,1,1,-1,1) = 
     &  + 6.1007809756250693d-03
      HY6(0,-1,1,1,0,1) = 
     &  + 3.2237101006270680d-02
      HY6(0,-1,1,1,1,-1) = 
     &  + 1.0233318613285890d-02
      HY6(0,-1,1,1,1,1) = 
     &  + 1.8254649462891551d-02
      HY6(0,0,-1,-1,-1,-1) = 
     &  + 9.7351609701343225d-04
      HY6(0,0,-1,-1,-1,1) = 
     &  + 1.2190143410042446d-03
      HY6(0,0,-1,-1,0,-1) = 
     &  + 1.0103097445914236d-02
      HY6(0,0,-1,-1,0,1) = 
     &  + 1.1458508205698223d-02
      HY6(0,0,-1,-1,1,-1) = 
     &  + 1.5800771184704528d-03
      HY6(0,0,-1,-1,1,1) = 
     &  + 2.1023563780719722d-03
      HY6(0,0,-1,0,-1,-1) = 
     &  + 5.3096809374395533d-03
      HY6(0,0,-1,0,-1,1) = 
     &  + 6.2840595064476972d-03
      HY6(0,0,-1,0,0,1) = 
     &  + 1.0228193143276494d-01
      HY6(0,0,-1,0,1,-1) = 
     &  + 7.6290419248728392d-03
      HY6(0,0,-1,0,1,1) = 
     &  + 9.4732998638164237d-03
      HY6(0,0,-1,1,-1,-1) = 
     &  + 2.1636981353547235d-03
      HY6(0,0,-1,1,-1,1) = 
     &  + 2.8776921291024274d-03
      HY6(0,0,-1,1,0,-1) = 
     &  + 1.8113105618022087d-02
      HY6(0,0,-1,1,0,1) = 
     &  + 2.1397522179178781d-02
      HY6(0,0,-1,1,1,-1) = 
     &  + 4.0581203978064105d-03
      HY6(0,0,-1,1,1,1) = 
     &  + 5.9544669277129711d-03
      HY6(0,0,0,-1,-1,-1) = 
     &  + 3.7167695485927819d-03
      HY6(0,0,0,-1,-1,1) = 
     &  + 4.4943988992188555d-03
      HY6(0,0,0,-1,0,-1) = 
     &  + 5.0843191948181848d-02
      HY6(0,0,0,-1,0,1) = 
     &  + 5.5765213278438523d-02
      HY6(0,0,0,-1,1,-1) = 
     &  + 5.5920127803503056d-03
      HY6(0,0,0,-1,1,1) = 
     &  + 7.1257602963746915d-03
      HY6(0,0,0,0,-1,-1) = 
     &  + 2.6399148793116946d-02
      HY6(0,0,0,0,-1,1) = 
     &  + 2.9901635581149425d-02
      HY6(0,0,0,0,0,-1) = 
     &  + 9.8555109129743510d-01
      HY6(0,0,0,0,0,1) = 
     &  + 1.0173430619844491d+00
      HY6(0,0,0,0,1,-1) = 
     &  + 3.4500854404017275d-02
      HY6(0,0,0,0,1,1) = 
     &  + 4.0536897271519737d-02
      HY6(0,0,0,1,-1,-1) = 
     &  + 7.2761575180718402d-03
      HY6(0,0,0,1,-1,1) = 
     &  + 9.3074022781308142d-03
      HY6(0,0,0,1,0,-1) = 
     &  + 7.8031149720713994d-02
      HY6(0,0,0,1,0,1) = 
     &  + 8.8483382454368714d-02
      HY6(0,0,0,1,1,-1) = 
     &  + 1.2516258597427692d-02
      HY6(0,0,0,1,1,1) = 
     &  + 1.7489853169011404d-02
      HY6(0,0,1,-1,-1,-1) = 
     &  + 3.2766998166444148d-03
      HY6(0,0,1,-1,-1,1) = 
     &  + 4.3608223423903728d-03
      HY6(0,0,1,-1,0,-1) = 
     &  + 2.7233592149955328d-02
      HY6(0,0,1,-1,0,1) = 
     &  + 3.2211985771117729d-02
      HY6(0,0,1,-1,1,-1) = 
     &  + 6.1528318685728338d-03
      HY6(0,0,1,-1,1,1) = 
     &  + 9.0307642704087900d-03
      HY6(0,0,1,0,-1,-1) = 
     &  + 1.4496092068016896d-02
      HY6(0,0,1,0,-1,1) = 
     &  + 1.8138069583596615d-02
      HY6(0,0,1,0,1,-1) = 
     &  + 2.3758543482195986d-02
      HY6(0,0,1,0,1,1) = 
     &  + 3.2309028991669881d-02
      HY6(0,0,1,1,-1,-1) = 
     &  + 9.6741436993644730d-03
      HY6(0,0,1,1,-1,1) = 
     &  + 1.4158547826406217d-02
      HY6(0,0,1,1,0,-1) = 
     &  + 6.2411323447021162d-02
      HY6(0,0,1,1,0,1) = 
     &  + 7.9221397565207165d-02
      HY6(0,0,1,1,1,-1) = 
     &  + 2.3218887429959911d-02
      HY6(0,0,1,1,1,1) = 
     &  + 4.0536897271519737d-02
      HY6(0,1,-1,-1,-1,-1) = 
     &  + 2.6959594046650189d-03
      HY6(0,1,-1,-1,-1,1) = 
     &  + 3.6389303535337737d-03
      HY6(0,1,-1,-1,1,-1) = 
     &  + 5.2254539749517106d-03
      HY6(0,1,-1,-1,1,1) = 
     &  + 7.8087314804442795d-03
      HY6(0,1,-1,0,1,1) = 
     &  + 2.6594848155187647d-02
      HY6(0,1,-1,1,-1,-1) = 
     &  + 8.4191205208964350d-03
      HY6(0,1,-1,1,-1,1) = 
     &  + 1.2504107554099770d-02
      HY6(0,1,-1,1,1,-1) = 
     &  + 2.0912204250035171d-02
      HY6(0,1,-1,1,1,1) = 
     &  + 3.7192729240182574d-02
      HY6(0,1,0,1,-1,-1) = 
     &  + 2.2264644192427555d-02
      HY6(0,1,0,1,-1,1) = 
     &  + 3.2107323193639778d-02
      HY6(0,1,0,1,1,-1) = 
     &  + 5.1634546570914239d-02
      HY6(0,1,0,1,1,1) = 
     &  + 8.8483382454368714d-02
      HY6(0,1,1,-1,-1,-1) = 
     &  + 1.8032999288881844d-02
      HY6(0,1,1,-1,-1,1) = 
     &  + 2.6632640235101822d-02
      HY6(0,1,1,-1,1,-1) = 
     &  + 4.4179246042660315d-02
      HY6(0,1,1,-1,1,1) = 
     &  + 7.7939361164851325d-02
      HY6(0,1,1,1,-1,-1) = 
     &  + 9.7962010990336989d-02
      HY6(0,1,1,1,-1,1) = 
     &  + 1.7017542062334508d-01
      HY6(0,1,1,1,1,-1) = 
     &  + 3.9593139179918261d-01
      HY6(0,1,1,1,1,1) = 
     &  + 1.0173430619844491d+00
       endif
      endif
** (n1,n2) = (-1,1) -- completion endif 
      return
      end

      end module
