!
!  SPDX-License-Identifier: GPL-3.0-or-later
!  Copyright (C) 2019-2022, respective authors of MCFM.
!
c "The Diphoton qT spectrum at N3LL'+NNLO", Tobias Neumann, 2107.12478

      ! form factors alpha,beta,gamma from [Caola, von Manteuffel, Tancredi '20]

      ! The generated code might not be optimal numerically, but
      ! we get a relative agreement of 10^(-8) for the most complicated
      ! three loop coefficients when comparing with the Mathematica implementation.
      ! And I am not even sure our Fortran implementation
      ! is the bottleneck in agreement there.
      ! But in any case, it's plenty for pheno.
      module GammaGammaABCD
          use types
          implicit none

          public :: alpha_1l, beta_1l, gamma_1l, delta_1l

          public :: alphaHPL_2l_NfV2
          public :: betaHPL_2l_NfV2
          public :: gammaHPL_2l_NfV2

          public :: alphaHPL_2l_noNfV2
          public :: betaHPL_2l_noNfV2
          public :: gammaHPL_2l_noNfV2

          public :: alphaHPL_3l_no_nfv
          public :: betaHPL_3l_no_nfv
          public :: gammaHPL_3l_no_nfv

          public :: alphaHPL_3l_nfv
          public :: betaHPL_3l_nfv
          public :: gammaHPL_3l_nfv

          public :: alphaHPL_3l_nfv2
          public :: betaHPL_3l_nfv2
          public :: gammaHPL_3l_nfv2

          public :: inithpl

          private

          integer, parameter :: n1=-1, n2=1, nw = 6

          complex(dp), save :: Hc1(n1:n2),Hc2(n1:n2,n1:n2),Hc3(n1:n2,n1:n2,n1:n2),
     &              Hc4(n1:n2,n1:n2,n1:n2,n1:n2),Hc5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2),
     &              Hc6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2)
!$omp threadprivate(Hc1,Hc2,Hc3,Hc4,Hc5,Hc6)
          real(dp), save :: Hr1(n1:n2),Hr2(n1:n2,n1:n2),Hr3(n1:n2,n1:n2,n1:n2),
     &              Hr4(n1:n2,n1:n2,n1:n2,n1:n2),Hr5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2),
     &              Hr6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2)
!$omp threadprivate(Hr1,Hr2,Hr3,Hr4,Hr5,Hr6)
          real(dp), save :: Hi1(n1:n2),Hi2(n1:n2,n1:n2),Hi3(n1:n2,n1:n2,n1:n2),
     &              Hi4(n1:n2,n1:n2,n1:n2,n1:n2),Hi5(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2),
     &              Hi6(n1:n2,n1:n2,n1:n2,n1:n2,n1:n2,n1:n2)
!$omp threadprivate(Hi1,Hi2,Hi3,Hi4,Hi5,Hi6)

          contains

      subroutine inithpl(x)
        use types
        use mod_hplog6
        implicit none
        real(dp), intent(in) :: x

        call hplog6(x,nw,Hc1,Hc2,Hc3,Hc4,Hc5,Hc6,
     &                   Hr1,Hr2,Hr3,Hr4,Hr5,Hr6,
     &                   Hi1,Hi2,Hi3,Hi4,Hi5,Hi6,n1,n2)

      end subroutine

      function alpha_1l(x)
        use types
        use constants
        implicit none
        complex(dp) :: alpha_1l
        real(dp), intent(in) :: x

        alpha_1l = (2*CF*x-CF)/2._dp
      end function

      function beta_1l(x)
        use types
        use constants
        implicit none
        complex(dp) :: beta_1l
        real(dp), intent(in) :: x

        complex(dp) :: T2,T3,T4,T5

        T2=x*x
        T3=Log(x)
        T4=Log(-x+1)
        T5=12*CF*im*pi
        beta_1l = (6*CF*T2*T3*T3+12*CF*im*pi*T2*T3+6*CF*T4*T4+(6*CF*T2+12*CF*x+
     &  T5)*T4+(7*CF*pi*pi-T5-42*CF)*T2+12*CF*im*pi*x)/(12*T2)

      end function

      function gamma_1l(x)
        use types
        use constants
        implicit none
        complex(dp) :: gamma_1l
        real(dp), intent(in) :: x

        complex(dp) :: T2,T3,T4,T5,T6,T7,T8,T9

        T2=Log(x)
        T3=6*CF*x*x
        T4=x*x
        T5=12*CF*im*pi
        T6=Log(-x+1)
        T7=pi*pi
        T8=7*CF*T7
        T9=-42*CF
        gamma_1l=(6*CF*T2*T2+(T3-24*CF*x+T5+18*CF)*T2+(T3-12*CF*x+6*CF)*T6*T6+(
     &  12*CF*im*pi*T4-24*CF*im*pi*x+T5)*T6+(T8-T5+T9)*T4+(-14*CF*T7+T5+84
     &  *CF)*x+T8+T9)/(12*T4-24*x+12)
      end function

      function delta_1l(x)
        use types
        use constants
        implicit none
        complex(dp) :: delta_1l
        real(dp), intent(in) :: x

        delta_1l = (-2*CF*x+CF)/2._dp

      end function

      function alphaHPL_2l_NfV2(x)
        use types
        use constants
        implicit complex(dp) (T)
        include 'nf.f'
        complex(dp) :: alphaHPL_2l_NfV2
        real(dp), intent(in) :: x

      T2=pi*pi
      T3=Hr2(1,1)
      T4=Hr2(1,0)
      T5=Hr2(0,1)
      T6=Hr2(0,0)
      T7=Hr1(1)
      T8=Hr1(0)
      T9=16*T3
      T10=x*x
      alphaHPL_2l_NfV2=((-2*CF*T2+(-4*T3-4*T4-4*T5-4*T6)*CF)*x**5+(5*CF*T2+(10*T3+10*
     &T4+10*T5+10*T6+12*T7+12*T8)*CF)*x**4+(-4*CF*T2+(4*T7-4*T8+12)*CF*i
     &m*pi+(-12*T3-8*T4-8*T5-12*T6-30*T7-18*T8)*CF)*x**3+(CF*T2+(-14*T7-
     &2*T8-18)*CF*im*pi+(T9+2*T4+2*T5+24*T7+6*T8)*CF)*T10+((16*T7+6)*CF*
     &im*pi+(-T9-6*T7)*CF)*x-6*T7*CF*im*pi+6*T3*CF)/(4*T10-4*x)


      end function alphaHPL_2l_NfV2

      function alphaHPL_2l_noNfV2(x)
        use types
        use constants
        implicit complex(dp) (T)
        include 'nf.f'
        complex(dp) :: alphaHPL_2l_noNfV2
        real(dp), intent(in) :: x

      T2=CF*CF
      T3=Hr2(1,1)
      T4=Hr2(1,0)
      T5=Hr2(0,1)
      T6=Hr2(0,0)
      T7=pi*pi
      T8=Hr1(1)
      T9=Hr1(0)
      T10=432*T3
      T11=432*T4
      T12=432*T5
      T13=432*T6
      T14=36*T8
      T15=36*T9
      T16=12*T8
      T17=-216*T3
      T18=-216*T4
      T19=-216*T5
      T20=12*T9
      T21=72*T3
      T22=x*x
      T23=((216*T2-108*CA*CF)*T7+(T10+T11+T12+T13)*T2+(T17+T18+T19-216*T
     &6)*CA*CF)*x**5+((-540*T2+270*CA*CF)*T7+(-1080*T3-1080*T4-1080*T5-1
     &080*T6-432*T8-432*T9)*T2+(540*T3+540*T4+540*T5+540*T6+216*T8+216*T
     &9)*CA*CF)*x**4+((474*T2-216*CA*CF)*T7+(-T14+T15-72)*T2*im*pi+(-T16
     &+T20-76)*CF*Nf+(900*T3+864*T4+864*T5+900*T6+846*T8+882*T9-306)*T2+
     &(-T10-T11-T12-T13-366*T8-498*T9+430)*CA*CF)*x**3+((-171*T2+54*CA*C
     &F)*T7+((-108*T9+108)*T2+(T14+T15)*CA*CF)*im*pi+(T16-24*T9+114)*CF*
     &Nf+(T17+T18+T19-324*T6-450*T8-504*T9+459)*T2+(T21+108*T4+108*T5+14
     &4*T6+150*T8+348*T9-645)*CA*CF)*T22+(21*T2*T7+((72*T8+T15-36)*T2-72
     &*T8*CA*CF)*im*pi+(T20-38)*CF*Nf+(-T21+36*T6+T14+54*T9-153)*T2+(T21
     &-66*T9+215)*CA*CF)*x+(-36*T8*T2+36*T8*CA*CF)*im*pi+36*T3*T2
      alphaHPL_2l_noNfV2=(T23-36*T3*CA*CF)/(72*T22-72*x)


      end function alphaHPL_2l_noNfV2

      function alpha_2l_noNfV2(x)
        use types
        use constants
        implicit complex(dp) (T)
        include 'nf.f'
        complex(dp) :: alpha_2l_noNfV2
        real(dp), intent(in) :: x

      T2=CF*CF
      T3=216*CA*CF
      T4=x**5
      T5=x**4
      T6=x**3
      T7=216*T2-108*CA*CF
      T8=x*x
      T9=-432*T2+T3
      T10=36*T2*im*pi
      T11=12*CF*Nf
      T12=Log(x)
      T13=T7*T4
      T14=(-540*T2+270*CA*CF)*T5
      T15=(450*T2-T3)*T6
      T16=-108*T2+36*CA*CF
      T17=36*CA*CF
      T18=Log(-x+1)
      T19=432*T2
      T20=450*T2
      T21=72*CA*CF
      T22=-36*T2
      T23=36*T2
      T24=-540*T2+270*CA*CF
      T25=pi*pi
      alpha_2l_noNfV2=((T13+T14+T15+(-162*T2+T21)*T8+18*T2*x)*T12*T12+((T9*T4+(1080*T
     &2-540*CA*CF)*T5+(-864*T2+432*CA*CF)*T6+T7*T8)*T18+T9*T5+(T10+T11+8
     &82*T2-498*CA*CF)*T6+(T16*im*pi-24*CF*Nf-504*T2+348*CA*CF)*T8+(T10+
     &T11+54*T2-66*CA*CF)*x)*T12+(T13+T14+T15+T16*T8+(T22+T17)*x+18*T2-1
     &8*CA*CF)*T18*T18+((T19-T3)*T5+(T10+T11-846*T2+366*CA*CF)*T6+(-36*C
     &A*CF*im*pi-T11+T20-150*CA*CF)*T8+((-72*T2+T21)*im*pi+T22)*x+(T23-T
     &17)*im*pi)*T18+T7*T25*T4+T24*T25*T5+((474*T2-T3)*T25-72*T2*im*pi-7
     &6*CF*Nf-306*T2+430*CA*CF)*T6+((-171*T2+54*CA*CF)*T25+108*T2*im*pi+
     &114*CF*Nf+459*T2-645*CA*CF)*T8+(21*T2*T25-T10-38*CF*Nf-153*T2+215*
     &CA*CF)*x)/(72*T8-72*x)


      end function

      function beta_2l_noNfV2(x)
        use types
        use constants
        implicit complex(dp) (T)
        include 'nf.f'
        complex(dp) :: beta_2l_noNfV2
        real(dp), intent(in) :: x
        real(dp) :: Li2, Li3, Li4

      T2=CF*CF
      T3=x**3
      T4=6480*T2*im*pi
      T5=1440*CF*Nf
      T6=7920*CA*CF
      T7=x*x
      T8=Log(x)
      T9=38880*CA*CF
      T10=Log(-x+1)
      T11=6480*T2
      T12=6480*CA*CF
      T13=9720*CA*CF
      T14=2700*T2
      T15=1080*CA*CF
      T16=pi*pi
      T17=2160*CF*Nf
      T18=9720*T2
      T19=11880*CA*CF
      T20=6840*CF*Nf
      T21=17820*T2
      T22=-19440*CA*CF
      T23=-12960*T2
      T24=12960*CA*CF
      T25=12960*T2
      T26=45360*T2
      T27=22680*CA*CF
      T28=T10*T10
      T29=38880*T2
      T30=19440*CA*CF
      T31=x**5
      T32=x**4
      T33=64800*T2
      T34=25920*CA*CF
      T35=(T23+T24)*im*pi
      T36=-32400*CA*CF
      T37=-T29+T30
      T38=25920*T2
      T39=-x+1
      T40=(T25-T24)*im*pi
      T41=181440*T2
      T42=77760*CA*CF
      T43=Li2(x)
      T44=336960*T2
      T45=155520*CA*CF
      T46=7560*T2
      T47=2160*CA*CF
      T48=pi**3
      T49=19440*T2
      T50=13680*CF*Nf
      T51=70920*CA*CF
      T52=T37*im*pi
      T53=2160*T2
      T54=540*CA*CF
      T55=540*T2
      T56=-23760*T2
      T57=T10**3
      T58=-T11+T12
      T59=(T49-T13)*T31
      T60=(-32400*T2+16200*CA*CF)*T32
      T61=8640*CA*CF
      T62=3240*T2*im*pi
      T63=Li3(x)
      T64=T38-T24
      T65=90720*CA*CF
      T66=116640*CA*CF
      T67=103680*T2
      T68=Li3(real(T39))
      T69=-51840*T2
      T70=155520*T2
      T71=259200*T2
      T72=-11160*CF*Nf
      T73=-4320*CA*CF
      T74=81000*T2
      T75=4320*CA*CF
      T76=42120*T2
      T77=90720*T2
      T78=(-T44+T45)*x
      T79=-103680*CA*CF
      T80=194400*T2
      T81=51840*T2
      T82=207360*T2
      T83=97200*CA*CF
      T84=136080*T2
      T85=-71280*CA*CF
      T86=25920*T2*im*pi
      T87=-304560*T2
      T88=149040*CA*CF
      T89=-77760*T2
      T90=220320*T2
      T91=77760*T2
      T92=51840*CA*CF
      T93=21600*CA*CF
      T94=(-T25+T61)*T16
      T95=45360*CA*CF
      T96=720*CF*Nf
      T97=(T91-T9)*im*pi
      T98=(T89+T9)*im*pi
      T99=T49-T13
      T100=-32400*T2+16200*CA*CF
      T101=pi**4
      T102=21060*T2
      T103=34010*CF*Nf
      T104=217085*CA*CF
      T105=3240*T2
      T106=23760*T2
      T107=93960*T2
      T108=-96840*CA*CF
      T109=(1620*T2*T3-1620*T2*T7)*T8**4+((T4+T5-T6)*T3+(-T4-T5+T6)*T7)*
     &T8**3+((3240*CA*CF*T3+(T29-T27)*T7+(-84240*T2+T9)*x+T26+T22)*T28+(
     &(T23+T13)*T3+(T67-48600*CA*CF)*T7+(-T77+T9)*x)*T10+((T11-T12)*T3+T
     &58*T7)*T43+T59+T60+((-T14-T15)*T16+(T17-T18-T19)*im*pi-T20+T21+225
     &00*CA*CF)*T3+((T14+T15)*T16+(-T17+T18+T19)*im*pi+T20-T21-19260*CA*
     &CF)*T7)*T8*T8+((-4320*T2*T3+(-21600*T2+T24)*T7+(49680*T2+T22)*x+T5
     &6+T12)*T57+((113400*T2-58320*CA*CF)*T3+(-262440*T2+129600*CA*CF)*T
     &7+(T35+T80-93960*CA*CF)*x+T40-T26+T27)*T28+(T37*T31+(T33+T36)*T32+
     &((8640*T2+T73)*T16+(T33-T34)*im*pi+T69+T34)*T3+((-60480*T2+30240*C
     &A*CF)*T16+T35+T33+T36)*T7+((95040*T2-47520*CA*CF)*T16+(-T25-T12)*i
     &m*pi-T29+T30)*x+(-43200*T2+T93)*T16+T52)*T10+((-T38+T34)*T3+(T38-T
     &34)*T7)*T63+(12960*CA*CF*T3+(T70-T65)*T7+T78+T41-T42)*T68+((T40-T3
     &8+T30)*T3+(T35+T82-T83)*T7+(-T41+T42)*x)*T43+((T38-T9)*T3+(-T41+T6
     &6)*T7+(T44-T45)*x-T41+T42)*zeta3+T37*T32+((T46-T47)*im*T48+(T5-T11
     &+3960*CA*CF)*T16+(-T50-16200*T2+T51)*im*pi+T33-T9)*T3+((-T46+T47)*
     &im*T48+(-T5+T49-12600*CA*CF)*T16+(T50+29160*T2-T51)*im*pi-T38+T30)
     &*T7+(T94+T52)*x)*T8
      T110=T109+((T53-T54)*T3+(-T53+T54)*T7+(-T55+T54)*x+T55-T54)*T10**4
     &+((-14040*T2+7560*CA*CF)*T3+(37800*T2-18360*CA*CF)*T7+(T4+T5+T56+6
     &120*CA*CF)*x-T4-T5+4680*CA*CF)*T57+((T58*x+T11-T12)*T43+T59+T60+((
     &T11-T47)*T16+T62+T17+T18-T19)*T3+((-T49+T61)*T16+T62+T17+T25-15120
     &*CA*CF)*T7+((T21-T24)*T16+(T17+T105-T19)*im*pi+T72-30780*T2+55980*
     &CA*CF)*x+(-4860*T2+T12)*T16+(-T17-T18+T19)*im*pi+T20+T102-35460*CA
     &*CF)*T28+(((-T38+T24)*T3+T64*T7)*T63+(T64*T3+(-T41+T65)*T7+(T71-T6
     &6)*x-T67+T9)*T68+((T84+T85)*T3+(T87+T88)*T7+(T35+T90+T79)*x+T40+T6
     &9+T34)*T43+((T70-T42)*T7+(-T71+T66)*x+T67-T9)*zeta3+(T29+T22)*T32+
     &((-44820*T2+T27)*T16+(T17-T76+14040*CA*CF)*im*pi-2520*CF*Nf-46980*
     &T2+34380*CA*CF)*T3+((110700*T2-55080*CA*CF)*T16+(T17+T74-50760*CA*
     &CF)*im*pi+T72-85860*T2+81900*CA*CF)*T7+((T18+T73)*im*T48+(T5-85320
     &*T2+37440*CA*CF)*T16+(-18000*CF*Nf-T74+107640*CA*CF)*im*pi+T50+T10
     &7+T108)*x+(-T18+T75)*im*T48+(-T5+T49-5040*CA*CF)*T16+(T50+T76-T51)
     &*im*pi)*T10
      T111=T110+((T33-T9)*T3+(T77-T9)*T7+T78+T41-T42)*Li4(x)+(25920*T2*T
     &3+(129600*T2-T42)*T7+T78+T41-T42)*Li4(x/(x-1))+((T89+T34)*T3+(2332
     &80*T2+T79)*T7+(-T80+T65)*x+T29-T24)*Li4(real(T39))+(((T69+T9)*im*pi+T38+
     &T22)*T3+((T81-T9)*im*pi-T82+T83)*T7+(T41-T42)*x)*T63+((T86+T84+T85
     &)*T3+(-T86+T87+T88)*T7+(T98+T90+T79)*x+T97+T69+T34)*T68+(((T25+T73
     &)*T16+(110160*T2-T92)*im*pi)*T3+((-T29+17280*CA*CF)*T16+(-97200*T2
     &+T92)*im*pi)*T7+((T29-T93)*T16+(T29-T34)*im*pi)*x+T94+(T69+T34)*im
     &*pi)*T43+(((-T81+T95)*im*pi+T96-58320*T2+93240*CA*CF)*T3+((T81-T95
     &)*im*pi-T96+187920*T2-158040*CA*CF)*T7+(T97-T41+T65)*x+T98+T81-T34
     &)*zeta3+T99*T16*T31+T100*T16*T32+((1053*T2+252*CA*CF)*T101+(-T25+T
     &75)*im*T48+(-4740*CF*Nf+10260*T2+9510*CA*CF)*T16+(20400*CF*Nf+T74-
     &145320*CA*CF)*im*pi+T103+78975*T2-T104)*T3+((-2781*T2+612*CA*CF)*T
     &101+(T18-T47)*im*T48+(6180*CF*Nf-T102-6630*CA*CF)*T16+(-34080*CF*N
     &f-174960*T2+242160*CA*CF)*im*pi-T103-82215*T2+T104)*T7
      beta_2l_noNfV2=(T111+((T53-1008*CA*CF)*T101+(T105-T47)*im*T48+(-T5+T106-9360*
     &CA*CF)*T16+(T50+T107+T108)*im*pi+T105)*x+(-432*T2+144*CA*CF)*T101)
     &/(12960*T3-12960*T7)


      end function beta_2l_noNfV2

      function betaHPL_2l_noNfV2(x)
        use types
        use constants
        implicit complex(dp) (T)
        include 'nf.f'
        complex(dp) :: betaHPL_2l_noNfV2
        real(dp), intent(in) :: x

      T2=CF*CF
      T3=Hr1(1)
      T4=25920*Hr1(0)
      T5=25920*T2
      T6=45360*CA*CF
      T7=720*CF*Nf
      T8=90720*T3
      T9=Hr2(1,1)
      T10=Hr2(1,0)
      T11=Hr2(0,1)
      T12=Hr2(0,0)
      T13=pi*pi
      T14=Hr1(0)
      T15=-64800*T10
      T16=-19440*T12
      T17=12960*Hr3(0,1,0)
      T18=Hr3(0,0,1)
      T19=Hr3(0,0,0)
      T20=4320*T9
      T21=Hr3(1,1,0)
      T22=19440*T9
      T23=Hr4(1,1,1,1)
      T24=Hr4(1,1,1,0)
      T25=Hr4(1,1,0,1)
      T26=12960*Hr4(0,1,0,0)
      T27=Hr4(0,0,1,1)
      T28=Hr3(1,1,1)
      T29=Hr3(1,0,1)
      T30=Hr3(1,0,0)
      T31=Hr3(0,1,1)
      T32=25920*T10
      T33=-25920*T11
      T34=-38880*T14
      T35=x**3
      T36=189*T2
      T37=540*CA*CF
      T38=pi**4
      T39=4320*T3
      T40=7560*T14
      T41=2160*T14
      T42=pi**3
      T43=1440*T14
      T44=4320*T11
      T45=5400*T12
      T46=19440*T14
      T47=2160*T11
      T48=2160*T12
      T49=4320*T12
      T50=-2160*T3
      T51=13680*T14
      T52=25920*T21
      T53=25920*T18
      T54=38880*T19
      T55=6480*T9
      T56=19440*T12
      T57=12960*T18
      T58=12960*T10
      T59=38880*T11
      T60=23760*T12
      T61=70920*T14
      T62=8640*T19
      T63=13680*T12
      T64=Hr4(1,1,0,0)
      T65=25920*T27
      T66=38880*Hr4(0,0,0,0)
      T67=-64800*T11
      T68=35640*T12
      T69=12960*T27
      T70=47520*T19
      T71=30240*T9
      T72=32400*T10
      T73=32400*T11
      T74=x*x
      T75=38880*T10
      T76=51840*T11
      T77=81000*T3
      T78=12960*T29
      T79=-23760*T9
      T80=-19440*T11
      T81=-155520*T64
      T82=12960*Hr4(1,0,1,1)
      T83=-19440*T10
      T84=-10800*T3
      T85=23760*T9
      T86=-8640*T10
      T87=-13680*T3
      T88=38880*T28
      T89=51840*T21
      T90=42120*T3
      T91=12960*T21
      T92=8640*T28
      T93=(((-T5+T6)*im*pi+T7+(51840*T3+T4+77760)*T2+(-12960*T3-T4+21960
     &)*CA*CF)*T35+((T5-T6)*im*pi-T7+(-207360*T3-T4-116640)*T2+(T8+T4-90
     &00)*CA*CF)*T74+((194400*T3+38880)*T2+(-T8-12960)*CA*CF)*x-38880*T3
     &*T2+12960*T3*CA*CF)*zeta3+((19440*T2-9720*CA*CF)*T13+(38880*T9+T75
     &+T59+38880*T12)*T2+(-T22+T83+T80+T16)*CA*CF)*x**5+((-32400*T2+1620
     &0*CA*CF)*T13+(-64800*T9+T15+T67-64800*T12-38880*T3+T34)*T2+(32400*
     &T9+T72+T73+32400*T12+19440*T3+T46)*CA*CF)*x**4+((T36+T37)*T38+((-T
     &39+T40-12960)*T2+(-T41+4320)*CA*CF)*im*T42+((T43-4740)*CF*Nf+(8640
     &*T9+T86+T44-T45+22140*T3-6480*T14+10260)*T2+(-T20+2160*T10-T47-T48
     &+T84+3960*T14+9510)*CA*CF)*T13+((T49+T50-T51+20400)*CF*Nf+(-T52+T1
     &7-T53+T54+T55+T15+45360*T11+T16+T90-16200*T14+81000)*T2+(-T17+T57+
     &T32+T33-T60-14040*T3+T61-145320)*CA*CF)*im*pi+(T62+T20-T63+2520*T3
     &+34010)*CF*Nf+(25920*T23+25920*T24+25920*T25+T26+T65+T66+84240*T28
     &+90720*T21+90720*T29+25920*T30-45360*T31+T22+51840*T10+T76+T68+469
     &80*T3+64800*T14+78975)*T2+(-12960*T23-12960*T24-12960*T25-12960*T6
     &4-T26-T69-45360*T28-45360*T21-45360*T29-19440*T30+25920*T31-T70+T7
     &9-T32+T33+45000*T12-34380*T3+T34-217085)*CA*CF)*T35
      T94=T93+((-T36-T37)*T38+((T39-T40+9720)*T2+(T41-2160)*CA*CF)*im*T4
     &2+((-T43+6180)*CF*Nf+(-60480*T9+34560*T10-T44+T45-59940*T3+T46-210
     &60)*T2+(T71-15120*T10+T47+T48+30240*T3-12600*T14-6630)*CA*CF)*T13+
     &((-T49+T50+T51-34080)*CF*Nf+(T52-T17+T53-T54+T55+T58-84240*T11+T56
     &-T77+29160*T14-174960)*T2+(T17-T57-T58+T59+T60+50760*T3-T61+242160
     &)*CA*CF)*im*pi+(-T62+T20+T63+11160*T3-34010)*CF*Nf+(-181440*T23-18
     &1440*T24-181440*T25+T81-T26-T65-T66-226800*T28-220320*T21-220320*T
     &29-207360*T30+84240*T31+25920*T9+T15+T67-T68+85860*T3-T4-82215)*T2
     &+(90720*T23+90720*T24+90720*T25+90720*T64+T26+T69+110160*T28+11016
     &0*T21+110160*T29+97200*T30-38880*T31+T70-T71+T72+T73-38520*T12-819
     &00*T3+T46+217085)*CA*CF)*T74+(((3240*T3+3240)*T2+(T50-2160)*CA*CF)
     &*im*T42+((-1440*T3-1440)*CF*Nf+(89640*T9-T75+48600*T3-12960*T14+23
     &760)*T2+(-49680*T9+21600*T10-20160*T3+8640*T14-9360)*CA*CF)*T13+((
     &T20+18000*T3+13680)*CF*Nf+(-T88+T89-T78+T55+T58+T76+T77+T34+93960)
     &*T2+(-T91+T78+T79+6480*T10+T80-107640*T3+T46-96840)*CA*CF)*im*pi+(
     &-T92-22320*T9+T87)*CF*Nf+(324000*T23+285120*T24+285120*T25+336960*
     &T64+T82+142560*T28+168480*T21+168480*T29+181440*T30-51840*T31-6156
     &0*T9+T75+T59-93960*T3+3240)*T2+(-142560*T23-142560*T24-142560*T25+
     &T81-T82-36720*T28-84240*T21-84240*T29-77760*T30+19440*T31+111960*T
     &9+T83+T80+96840*T3)*CA*CF)*x
      betaHPL_2l_noNfV2=(T94+(-3240*T3*T2+2160*T3*CA*CF)*im*T42+(1440*T3*CF*Nf+(-37800
     &*T9+T58+T84)*T2+(T85+T86+720*T3)*CA*CF)*T13+((-T20+T87)*CF*Nf+(T88
     &-T89+T78-T22+T75-12960*T11-T90)*T2+(T91-T78+T85+T83+6480*T11+70920
     &*T3)*CA*CF)*im*pi+(T92+13680*T9)*CF*Nf+(-168480*T23-129600*T24-129
     &600*T25-181440*T64-T82-38880*T21-38880*T29+12960*T31+42120*T9)*T2+
     &(64800*T23+64800*T24+64800*T25+77760*T64+T82-28080*T28+19440*T21+1
     &9440*T29-6480*T31-70920*T9)*CA*CF)/(12960*T35-12960*T74)


      end function betaHPL_2l_noNfV2

      function betaHPL_2l_NfV2(x)
        use types
        use constants
        implicit complex(dp) (T)
        include 'nf.f'
        complex(dp) :: betaHPL_2l_NfV2
        real(dp), intent(in) :: x

      T2=Hr1(1)
      T3=Hr2(1,1)
      T4=Hr2(1,0)
      T5=Hr2(0,1)
      T6=Hr2(0,0)
      T7=Hr1(0)
      T8=x**4
      T9=pi**3
      T10=-12*T4
      T11=pi*pi
      T12=Hr3(1,1,0)
      T13=12*T5
      T14=12*T6
      T15=24*T2
      T16=Hr3(1,0,1)
      T17=12*Hr3(0,1,0)
      T18=12*Hr3(0,0,1)
      T19=x**3
      T20=48*T4
      T21=72*T3
      T22=Hr4(1,1,0,0)
      T23=72*T12
      T24=Hr3(1,0,0)
      T25=30*T7
      T26=x*x
      T27=60*T2
      T28=-40*T2
      T29=24*T7
      T30=-48*T3
      T31=-96*T4
      T32=-12*T2
      T33=-48*T12
      T34=-48*T16
      T35=-144*T24
      T36=30*T2
      T37=10*T2
      T38=12*T3
      T39=18*T2
      T40=12*T12
      T41=12*T16
      T42=18*T3
      T43=(12*CF*T8+(72*T2+132)*CF*T19+(-288*T2-288)*CF*T26+(360*T2+144)
     &*CF*x-144*T2*CF)*zeta3+(-2*CF*im*T9+(T37+10*T7+9)*CF*T11+(T38+T10+
     &T13-T14)*CF*im*pi+(T40+T41+T17+T18+T42+18*T4+18*T5+18*T6+T15+T29)*
     &CF)*T8+((T32-22)*CF*im*T9+(T10+T28-34*T7-4)*CF*T11+(-T23+T30+T31-T
     &13+T14+T15-96*T7+24)*CF*im*pi+(-72*T22+T33+T34+T35-T17-T18-T21-T20
     &-48*T5-144*T6-T39+6*T7)*CF)*T19+((48*T2+48)*CF*im*T9+(T20+T27+48*T
     &7-19)*CF*T11+(288*T12+T21+216*T4-T36+T25+6)*CF*im*pi+(288*T22+T23+
     &72*T16+288*T24+T21+42*T4+42*T5+72*T6-36*T2-T25)*CF)*T26+((-T27-24)
     &*CF*im*T9+(-60*T4+T28-T29+14)*CF*T11+(-360*T12+T30+T31+T32+12*T7-3
     &0)*CF*im*pi+(-360*T22+T33+T34+T35+T10-T13+T36)*CF)*x+24*T2*CF*im*T
     &9+(24*T4+T37)*CF*T11+(144*T12+T38+T10+T39)*CF*im*pi+(144*T22+T40+T
     &41-T42)*CF
      betaHPL_2l_NfV2=T43/(18*T19-18*T26)

      end function betaHPL_2l_NfV2

      function gammaHPL_2l_noNfV2(x)
        use types
        use constants
        implicit complex(dp) (T)
        include 'nf.f'
        complex(dp) :: gammaHPL_2l_noNfV2
        real(dp), intent(in) :: x

      T2=CF*CF
      T3=Hr1(1)
      T4=Hr1(0)
      T5=720*CF*Nf
      T6=-64800*T3
      T7=25920*T4
      T8=51840*T3
      T9=Hr2(1,1)
      T10=Hr2(1,0)
      T11=Hr2(0,1)
      T12=Hr2(0,0)
      T13=pi*pi
      T14=-19440*T9
      T15=64800*T11
      T16=Hr3(1,1,0)
      T17=12960*Hr3(1,0,1)
      T18=Hr3(1,1,1)
      T19=4320*T12
      T20=Hr3(0,0,1)
      T21=Hr4(1,1,0,0)
      T22=12960*Hr4(1,0,1,1)
      T23=Hr4(0,0,1,0)
      T24=Hr4(0,0,0,1)
      T25=Hr4(0,0,0,0)
      T26=Hr3(1,0,0)
      T27=Hr3(0,1,1)
      T28=Hr3(0,1,0)
      T29=Hr3(0,0,0)
      T30=-38880*T3
      T31=x**3
      T32=pi**4
      T33=2160*T3
      T34=pi**3
      T35=-8640*T11
      T36=4320*T9
      T37=4320*T10
      T38=8640*T11
      T39=2160*T4
      T40=Hr3(1,0,1)
      T41=38880*T9
      T42=181440*T11
      T43=19440*T3
      T44=25920*T16
      T45=25920*T40
      T46=38880*T10
      T47=8640*T4
      T48=-17280*T12
      T49=Hr4(1,1,1,1)
      T50=Hr4(1,0,1,1)
      T51=Hr4(0,0,1,1)
      T52=51840*T20
      T53=-90720*T10
      T54=-90720*T11
      T55=6480*T4
      T56=25920*T21
      T57=25920*T50
      T58=25920*T20
      T59=25920*T29
      T60=45360*T10
      T61=x*x
      T62=-1080*T3
      T63=-1440*T3
      T64=-5400*T9
      T65=6480*T3
      T66=-2160*T9
      T67=-2160*T10
      T68=2160*T11
      T69=13680*T3
      T70=-38880*T18
      T71=19440*T10
      T72=38880*T12
      T73=45360*T4
      T74=-12960*T16
      T75=12960*T28
      T76=-23760*T9
      T77=-8640*T18
      T78=-13680*T9
      T79=38880*T49
      T80=-12960*T21
      T81=12960*Hr4(0,1,0,0)
      T82=-12960*T51
      T83=47520*T18
      T84=19440*T26
      T85=(((-103680*T2+84240*CA*CF)*im*pi+T5+(T6-T7-32400)*T2+(T8+12960
     &*T4+73800)*CA*CF)*T31+((207360*T2-168480*CA*CF)*im*pi-1440*CF*Nf+(
     &129600*T3-103680*T4+116640)*T2+(-103680*T3+51840*T4-160560)*CA*CF)
     &*T61+((-25920*T2+45360*CA*CF)*im*pi+T5+(T6+T7-136080)*T2+(T8-T7+11
     &2680)*CA*CF)*x)*zeta3+((19440*T2-9720*CA*CF)*T13+(T41+T46+38880*T1
     &1+T72)*T2+(T14-T71-19440*T11-19440*T12)*CA*CF)*x**5+((-64800*T2+32
     &400*CA*CF)*T13+(-129600*T9-129600*T10-129600*T11-129600*T12+T30-38
     &880*T4)*T2+(64800*T9+64800*T10+T15+64800*T12+T43+19440*T4)*CA*CF)*
     &x**4+((3429*T2-900*CA*CF)*T32+((T62+5400)*T2+(-T33-4320)*CA*CF)*im
     &*T34+((T63-4740)*CF*Nf+(T64+T37+T35+8640*T12+T65-22140*T4+75060)*T
     &2+(T66+T67+T68-T19-3960*T3+10800*T4-22890)*CA*CF)*T13+((T36+T69+T3
     &9+20400)*CF*Nf+(T70+T44-T17+T58+T14+T60-T15+6480*T12+16200*T3-4212
     &0*T4+81000)*T2+(T74+T17+T76-25920*T10+25920*T11-70920*T3+14040*T4-
     &145320)*CA*CF)*im*pi+(T77+T78+T19-2520*T4+34010)*CF*Nf+(T79+T56+T2
     &2+25920*T23+25920*T24+25920*T25+45360*T26-25920*T27-90720*T28-9072
     &0*T20-84240*T29+165240*T9+181440*T10+T42+149040*T12+90720*T3+10854
     &0*T4+78975)*T2+(T80-T22+T82-12960*T23-12960*T24-12960*T25+T83-2592
     &0*T26+19440*T27+45360*T28+45360*T20+45360*T29-19800*T9+T53+T54-885
     &60*T12+T30-43380*T4-217085)*CA*CF)*T31
      T86=T85+((486*T2-1872*CA*CF)*T32+((T33-9720)*T2+(4320*T3+6480)*CA*
     &CF)*im*T34+((2880*T3+8040)*CF*Nf+(10800*T9-8640*T10+T35+34560*T12+
     &T55-9720)*T2+(T36+T37+T38+T48-720*T3-T39-21900)*CA*CF)*T13+((-8640
     &*T9-27360*T3-T47-27120)*CF*Nf+(77760*T18-51840*T16+T45-T52+T41-518
     &40*T10+T42-25920*T12-T43+T73-68040)*T2+(T44-T45+47520*T9+T46-T15+1
     &41840*T3+T47+193800)*CA*CF)*im*pi+(17280*T18+27360*T9+T48+18720*T4
     &-68020)*CF*Nf+(-77760*T49-51840*T21-T57+155520*T51+103680*T23+1036
     &80*T24+103680*T25-51840*T26-129600*T27+51840*T28+T52+T59-71280*T9+
     &T53+T54-84240*T12+T6-T55-154710)*T2+(T56+T57-51840*T51-51840*T23-5
     &1840*T24-51840*T25-95040*T18+38880*T26+38880*T27-25920*T28-T58-T59
     &-96480*T9+T60+45360*T11+101520*T12+T43-68400*T4+434170)*CA*CF)*T61
     &+((189*T2+540*CA*CF)*T32+((T62+7560*T4+12960)*T2+(-T33-T39-6480)*C
     &A*CF)*im*T34+((T63+1440*T4-3300)*CF*Nf+(T64+T37+4320*T11-5400*T12-
     &T65+4860*T4-19980)*T2+(T66+T67-T68-2160*T12+4680*T3-7920*T4+22110)
     &*CA*CF)*T13+((T36+T19+T69-7200*T4+6720)*CF*Nf+(T70+T44-T17+T75-T58
     &+38880*T29+T14+T71-155520*T11+T72+29160*T3-T73-12960)*T2+(T74+T17-
     &T75+12960*T20+T76-T71+58320*T11-23760*T12-90360*T3+48240*T4-48480)
     &*CA*CF)*im*pi+(T77+8640*T29+T78-720*T12-16200*T4+34010)*CF*Nf+(T79
     &+T56+T22+T81+25920*T51+38880*T25+T84+155520*T27+58320*T29-29160*T9
     &-16200*T12+12960*T3-63180*T4+75735)*T2+(T80-T22-T81+T82+T83-T84-58
     &320*T27-47520*T29+90360*T9+12600*T12+92340*T4-217085)*CA*CF)*x
      gammaHPL_2l_noNfV2=(T86+(-25920*T3*T2+19440*T3*CA*CF)*im*pi+25920*T9*T2-19440*T9*
     &CA*CF)/(12960*T31-25920*T61+12960*x)


      end function gammaHPL_2l_noNfV2

      function gammaHPL_2l_NfV2(x)
        use types
        use constants
        implicit complex(dp) (T)
        include 'nf.f'
        complex(dp) :: gammaHPL_2l_NfV2
        real(dp), intent(in) :: x

      T2=Hr2(1,1)
      T3=Hr2(1,0)
      T4=Hr2(0,1)
      T5=Hr2(0,0)
      T6=Hr1(1)
      T7=Hr1(0)
      T8=x**4
      T9=60*T4
      T10=pi*pi
      T11=Hr3(0,0,1)
      T12=120*T7
      T13=Hr3(1,1,0)
      T14=Hr3(1,0,1)
      T15=x**3
      T16=11*CF*pi**4
      T17=30*T6
      T18=360*T11
      T19=180*T2
      T20=180*T3
      T21=720*T4
      T22=-360*Hr4(0,0,1,1)
      T23=180*T13
      T24=180*T14
      T25=720*Hr3(0,1,1)
      T26=x*x
      T27=60*T2
      T28=60*T3
      T29=60*T13
      T30=60*T14
      gammaHPL_2l_NfV2=((60*CF*T8-360*CF*im*pi*T15-360*CF*im*pi*T26)*zeta3+(-10*CF*im
     &*pi**3+(50*T6+50*T7-45)*CF*T10+(T27-T28+T9-60*T5)*CF*im*pi+(T29+T3
     &0+60*Hr3(0,1,0)+60*T11-90*T2-90*T3-90*T4-90*T5+120*T6+T12)*CF)*T8+
     &(T16+(-T9-T17+160)*CF*T10+(T18-T19+T20-T21+480*T6-T12+120)*CF*im*p
     &i+(T22-T23-T24+T25-360*T2+120*T3+120*T4-510*T6-390*T7)*CF)*T15+(T1
     &6+(-T9+T17-115)*CF*T10+(T18+T19-T20+T21-1290*T6+210*T7-390)*CF*im*
     &pi+(T22+T23+T24-T25+1260*T2-30*T3-30*T4+180*T5+660*T6+270*T7)*CF)*
     &T26+(-50*T6*CF*T10+(-T27+T28+1080*T6+270)*CF*im*pi+(-T29-T30-1080*
     &T2-270*T6)*CF)*x-270*T6*CF*im*pi+270*T2*CF)/(90*T15-180*T26+90*x)



      end function gammaHPL_2l_NfV2

c!!! three loop coefficients

      function alphaHPL_3l_nfv2(x)
        use types
        use constants
        implicit complex(dp) (T)
        include 'nf.f'
        complex(dp) :: alphaHPL_3l_nfv2
        real(dp), intent(in) :: x

      T2=CF*CF
      T3=Hr1(1)
      T4=Hr1(0)
      T5=0.66666666666666666667D0*CF*Nf
      T6=-25.0D0*T3
      T7=Hr2(1,1)
      T8=Hr2(1,0)
      T9=Hr2(0,1)
      T10=Hr2(0,0)
      T11=Hr3(1,1,1)
      T12=Hr3(1,1,0)
      T13=Hr3(1,0,1)
      T14=Hr3(1,0,0)
      T15=Hr3(0,1,1)
      T16=Hr3(0,1,0)
      T17=Hr3(0,0,1)
      T18=Hr3(0,0,0)
      T19=Hr4(1,1,1,1)
      T20=Hr4(1,1,1,0)
      T21=Hr4(1,1,0,1)
      T22=Hr4(1,1,0,0)
      T23=Hr4(1,0,1,1)
      T24=Hr4(1,0,1,0)
      T25=Hr4(1,0,0,1)
      T26=Hr4(0,1,1,0)
      T27=Hr4(0,1,0,1)
      T28=Hr4(0,1,0,0)
      T29=Hr4(0,0,1,1)
      T30=Hr4(0,0,1,0)
      T31=Hr4(0,0,0,1)
      T32=Hr4(0,0,0,0)
      T33=x**5
      T34=pi**4
      T35=2.0D0*T4
      T36=pi**3
      T37=pi*pi
      T38=-7.5D0*T11
      T39=7.5D0*T18
      T40=1.6666666666666666667D0*T14
      T41=1.6666666666666666667D0*T15
      T42=Hr4(1,0,0,0)
      T43=Hr4(0,1,1,1)
      T44=1.3333333333333333333D0*T16
      T45=1.3333333333333333333D0*T17
      T46=x**4
      T47=0.75D0*T4
      T48=0.33333333333333333333D0*T7
      T49=18.0D0*T13
      T50=6.0D0*T16
      T51=14.5D0*T23
      T52=3.5D0*T24
      T53=3.5D0*T25
      T54=2.0D0*T17
      T55=-1.0D0*T29
      T56=x**3
      T57=-0.055555555555555555556D0*CF*Nf
      T58=-0.16666666666666666667D0*T10
      T59=0.5D0*T10
      T60=-1.0D0*T14
      T61=-4.5D0*T15
      T62=2.0D0*T16
      T63=-1.5D0*T15
      T64=3.0D0*T43
      T65=1.5D0*T26
      T66=1.5D0*T27
      T67=5.0D0*T29
      T68=x*x
      T69=1.3333333333333333333D0*T3
      T70=0.66666666666666666667D0*T4
      T71=-0.16666666666666666667D0*T7
      T72=0.16666666666666666667D0*T8
      T73=1.3333333333333333333D0*T9
      T74=9.5D0*T11
      T75=-1.5D0*T12
      T76=5.5D0*T15
      T77=-0.5D0*T16
      T78=0.83333333333333333333D0*T8
      T79=0.5D0*T17
      T80=2.6666666666666666667D0*T11
      T81=0.5D0*T26
      T82=0.5D0*T27
      T83=4.0D0*T14
      T84=-0.75D0*T9
      T85=2.5D0*T11
      T86=1.5D0*T13
      T87=1.5D0*T17
      T88=-0.75D0*T8
      T89=-1.0D0*T11
      T90=-0.5D0*T15
      T91=6.5D0*T19
      T92=3.0833333333333333333D0*T7
      T93=-1.0D0*T24
      T94=-1.0D0*T25
      T95=(-0.38333333333333333333D0*T2+0.15833333333333333333D0*CA*CF)*
     &T34+((T3+T4+0.58333333333333333333D0)*T2+(-1.0D0*T4+1.166666666666
     &6666667D0)*CA*CF)*im*T36+(0.72222222222222222222D0*CF*Nf+(-0.41666
     &666666666666667D0*T7-0.41666666666666666667D0*T8-0.416666666666666
     &66667D0*T9-0.41666666666666666667D0*T10-0.66666666666666666667D0*T
     &3+T70+2.1666666666666666667D0)*T2+(T71-T72-0.16666666666666666667D
     &0*T9+T58-0.75D0*T3+T47-9.7638888888888888889D0)*CA*CF)*T37+((-4.0D
     &0*T11+6.0D0*T12-6.0D0*T13+T83-4.0D0*T15+T50-6.0D0*T17+4.0D0*T18+1.
     &1666666666666666667D0*T7+1.1666666666666666667D0*T8+1.166666666666
     &6666667D0*T9+1.1666666666666666667D0*T10)*T2+(3.0D0*T11-2.0D0*T12+
     &4.0D0*T13+T60+T15-4.0D0*T16+T54-3.0D0*T18+2.3333333333333333333D0*
     &T7+2.3333333333333333333D0*T8+2.3333333333333333333D0*T9+2.3333333
     &333333333333D0*T10)*CA*CF)*im*pi+(0.66666666666666666667D0*T11-0.6
     &6666666666666666667D0*T14+0.66666666666666666667D0*T15-0.666666666
     &66666666667D0*T18+1.4444444444444444444D0*T7+1.4444444444444444444
     &D0*T8+1.4444444444444444444D0*T9+1.4444444444444444444D0*T10)*CF*N
     &f
      T96=(T95+(3.0D0*T19-1.0D0*T20-1.0D0*T21+5.0D0*T22+5.0D0*T23+T93+T9
     &4+3.0D0*T42+T64-1.0D0*T26-1.0D0*T27+5.0D0*T28+T67-1.0D0*T30-1.0D0*
     &T31+3.0D0*T32+T89-1.3333333333333333333D0*T12-1.333333333333333333
     &3D0*T13-T40+T41+T44+T45+T18+4.3333333333333333333D0*T7+4.333333333
     &3333333333D0*T8+4.3333333333333333333D0*T9+4.3333333333333333333D0
     &*T10)*T2+(-2.0D0*T19+T20+T21-1.0D0*T22-3.0D0*T23+T24+T25+T26+T27-3
     &.0D0*T28+T55+T30+T31-2.0D0*T32+T38+T75-T86+4.5D0*T14+T61+1.5D0*T16
     &+T87+T39-19.527777777777777778D0*T7-19.527777777777777778D0*T8-19.
     &527777777777777778D0*T9-19.527777777777777778D0*T10)*CA*CF)*T33
      T97=(0.99722222222222222222D0*T2-0.62708333333333333333D0*CA*CF)*T
     &34+(T57+(-2.0D0*T3-T35-2.7916666666666666667D0)*T2+(0.25D0*T3+2.58
     &33333333333333333D0*T4-1.3194444444444444444D0)*CA*CF)*im*T36+((0.
     &27777777777777777778D0*T3+0.27777777777777777778D0*T4-1.8055555555
     &555555556D0)*CF*Nf+(0.70833333333333333333D0*T7+1.7083333333333333
     &333D0*T8+0.375D0*T9+1.375D0*T10+3.4166666666666666667D0*T3+0.08333
     &3333333333333333D0*T4-5.4166666666666666667D0)*T2+(0.5416666666666
     &6666667D0*T7-0.95833333333333333333D0*T8+1.7916666666666666667D0*T
     &9+0.29166666666666666667D0*T10-1.0972222222222222222D0*T3-4.847222
     &2222222222222D0*T4+24.409722222222222222D0)*CA*CF)*T37+((T48-0.333
     &33333333333333333D0*T8+0.33333333333333333333D0*T9-0.3333333333333
     &3333333D0*T10-T69-1.3333333333333333333D0*T4)*CF*Nf+(10.0D0*T11-12
     &.0D0*T12+T49-10.0D0*T14+10.0D0*T15-12.0D0*T16+18.0D0*T17-10.0D0*T1
     &8+T92-10.916666666666666667D0*T8+5.0833333333333333333D0*T9-8.9166
     &666666666666667D0*T10-2.1666666666666666667D0*T3-2.166666666666666
     &6667D0*T4)*T2+(T38+3.5D0*T12-12.5D0*T13+3.5D0*T14+T63+7.5D0*T16-6.
     &5D0*T17+T39-14.416666666666666667D0*T7+3.75D0*T8-15.41666666666666
     &6667D0*T9+2.75D0*T10-8.0D0*T3-8.0D0*T4)*CA*CF)*im*pi
      T98=T97+(-1.6666666666666666667D0*T11+0.33333333333333333333D0*T12
     &+0.33333333333333333333D0*T13+T40-T41+0.33333333333333333333D0*T16
     &+0.33333333333333333333D0*T17+1.6666666666666666667D0*T18-4.277777
     &7777777777778D0*T7-3.6111111111111111111D0*T8-3.611111111111111111
     &1D0*T9-2.9444444444444444444D0*T10-6.3333333333333333333D0*T3-6.33
     &33333333333333333D0*T4)*CF*Nf+(-T91+3.5D0*T20+3.5D0*T21-8.5D0*T22-
     &T51+T52+T53-6.5D0*T42-8.5D0*T43+T65+T66-10.5D0*T28-16.5D0*T29+1.5D
     &0*T30+1.5D0*T31-8.5D0*T32-1.5D0*T11+5.3333333333333333333D0*T12+5.
     &3333333333333333333D0*T13-1.8333333333333333333D0*T14-10.166666666
     &666666667D0*T15-T44-T45-6.5D0*T18-13.166666666666666667D0*T7-10.83
     &3333333333333333D0*T8-10.833333333333333333D0*T9-8.5D0*T10-15.3333
     &33333333333333D0*T3-15.333333333333333333D0*T4)*T2+(4.25D0*T19-3.2
     &5D0*T20-3.25D0*T21+0.25D0*T22+7.25D0*T23-5.25D0*T24-5.25D0*T25-1.7
     &5D0*T42+1.75D0*T43+0.25D0*T26+0.25D0*T27+7.75D0*T28+4.75D0*T29-1.7
     &5D0*T30-1.75D0*T31+5.75D0*T32+21.833333333333333333D0*T11-1.75D0*T
     &12-1.75D0*T13-7.1666666666666666667D0*T14+15.333333333333333333D0*
     &T15-9.25D0*T16-9.25D0*T17-15.666666666666666667D0*T18+67.819444444
     &444444444D0*T7+48.819444444444444444D0*T8+48.819444444444444444D0*
     &T9+29.819444444444444444D0*T10+49.25D0*T3+49.25D0*T4)*CA*CF
      T99=(-0.90138888888888888889D0*T2+0.49791666666666666667D0*CA*CF)*
     &T34+(0.027777777777777777778D0*CF*Nf+(0.91666666666666666667D0*T3+
     &T47+4.3333333333333333333D0)*T2+(-0.52083333333333333333D0*T3-1.81
     &25D0*T4+2.1180555555555555556D0)*CA*CF)*im*T36+((-0.19444444444444
     &444444D0*T3-0.91666666666666666667D0*T4+2.9444444444444444444D0)*C
     &F*Nf+(-T48-0.58333333333333333333D0*T8+2.0833333333333333333D0*T9-
     &1.6666666666666666667D0*T10-4.7083333333333333333D0*T3-2.291666666
     &6666666667D0*T4+12.75D0)*T2+(0.45833333333333333333D0*T7+2.9166666
     &666666666667D0*T8-2.5833333333333333333D0*T9+0.9583333333333333333
     &3D0*T10+4.4166666666666666667D0*T3+7.4722222222222222222D0*T4-27.0
     &83333333333333333D0)*CA*CF)*T37+((-0.83333333333333333333D0*T7+0.5
     &D0*T8-0.83333333333333333333D0*T9+T59+1.5555555555555555556D0*T3+3
     &.7777777777777777778D0*T4-6.3333333333333333333D0)*CF*Nf+(-3.5D0*T
     &11+4.0D0*T12-T49+5.5D0*T14-T76+T50-16.0D0*T17+3.5D0*T18-8.0D0*T7+1
     &2.5D0*T8-19.5D0*T9+16.0D0*T10+0.083333333333333333333D0*T3+8.58333
     &33333333333333D0*T4-8.6666666666666666667D0)*T2+(6.375D0*T11+1.125
     &D0*T12+13.125D0*T13-6.125D0*T14+2.125D0*T15-3.125D0*T16+4.875D0*T1
     &7-6.375D0*T18+28.625D0*T7-0.20833333333333333333D0*T8+38.125D0*T9-
     &5.7083333333333333333D0*T10+27.444444444444444444D0*T3+4.555555555
     &5555555556D0*T4+45.25D0)*CA*CF)*im*pi
      T100=T99+(2.0D0*T11-0.83333333333333333333D0*T12-0.833333333333333
     &33333D0*T13-T40+T41+T77-T79-2.0D0*T18+6.3333333333333333333D0*T7+2
     &.8888888888888888889D0*T8+2.8888888888888888889D0*T9+3.66666666666
     &66666667D0*T10+15.833333333333333333D0*T3+9.5D0*T4)*CF*Nf+(-5.0D0*
     &T19-8.5D0*T20-8.5D0*T21-4.5D0*T22+T51-T52-T53+2.0D0*T42+6.0D0*T43+
     &T81+T82+6.5D0*T28+11.5D0*T29-4.5D0*T30-4.5D0*T31-1.0D0*T32+T85-10.
     &0D0*T12-10.0D0*T13-0.5D0*T14+24.5D0*T15+T62+T54+13.5D0*T18+18.3333
     &33333333333333D0*T7+7.1666666666666666667D0*T8+7.16666666666666666
     &67D0*T9+9.0D0*T10+35.0D0*T3+26.333333333333333333D0*T4)*T2+(0.5D0*
     &T19+6.875D0*T20+6.875D0*T21+8.0D0*T22-4.5D0*T23+8.625D0*T24+8.625D
     &0*T25+2.5D0*T42-4.5D0*T43-2.375D0*T26-2.375D0*T27-5.5D0*T28+T55+3.
     &875D0*T30+3.875D0*T31-2.5D0*T32-25.958333333333333333D0*T11+13.666
     &666666666666667D0*T12+13.666666666666666667D0*T13+20.7916666666666
     &66667D0*T14-37.125D0*T15+8.3333333333333333333D0*T16+8.33333333333
     &33333333D0*T17+13.625D0*T18-88.083333333333333333D0*T7-33.13888888
     &8888888889D0*T8-33.138888888888888889D0*T9-12.083333333333333333D0
     &*T10-121.125D0*T3-75.875D0*T4)*CA*CF
      T101=(0.16111111111111111111D0*T2-0.41944444444444444444D0*CA*CF)*
     &T34+(T57+(-0.375D0*T3-0.125D0*T4-2.7083333333333333333D0)*T2+(-1.9
     &583333333333333333D0*T3+0.14583333333333333333D0*T4-2.972222222222
     &2222222D0)*CA*CF)*im*T36+((-1.1944444444444444444D0*T3-0.111111111
     &11111111111D0*T4-2.6111111111111111111D0)*CF*Nf+(1.083333333333333
     &3333D0*T7-1.2083333333333333333D0*T8-3.125D0*T9+T58-2.958333333333
     &3333333D0*T3-3.25D0*T4-13.708333333333333333D0)*T2+(-3.45833333333
     &33333333D0*T7-5.6875D0*T8+4.3541666666666666667D0*T9+T59-4.1805555
     &555555555556D0*T3-5.0138888888888888889D0*T4+16.215277777777777778
     &D0)*CA*CF)*T37+((0.66666666666666666667D0*T7-T78+T73+T58+3.5555555
     &555555555556D0*T3+0.22222222222222222222D0*T4+9.5D0)*CF*Nf+(-T74+5
     &.0D0*T12+8.0D0*T13+T60+T61+T62+5.0D0*T17-7.4166666666666666667D0*T
     &7-1.8333333333333333333D0*T8+18.166666666666666667D0*T9+1.25D0*T10
     &+8.6666666666666666667D0*T3-4.0833333333333333333D0*T4+13.0D0)*T2+
     &(-4.375D0*T11-20.875D0*T12-7.875D0*T13+6.875D0*T14+T90-3.5D0*T16-1
     &6.5D0*T17-0.25D0*T18-28.25D0*T7-10.125D0*T8-35.083333333333333333D
     &0*T9+5.5416666666666666667D0*T10-43.472222222222222222D0*T3-9.1388
     &888888888888889D0*T4-67.875D0)*CA*CF)*im*pi
      T102=T101+(-T80+0.66666666666666666667D0*T12+0.6666666666666666666
     &7D0*T13+0.16666666666666666667D0*T14+T63+0.16666666666666666667D0*
     &T16+0.16666666666666666667D0*T17-8.2777777777777777778D0*T7-0.7222
     &2222222222222222D0*T8-0.72222222222222222222D0*T9+T59-12.666666666
     &666666667D0*T3-3.1666666666666666667D0*T4)*CF*Nf+(24.0D0*T19+14.5D
     &0*T20+14.5D0*T21+19.5D0*T22-7.0D0*T23+T24+T25+T64-T65-T66+0.5D0*T2
     &8-T67+11.5D0*T11+10.083333333333333333D0*T12+10.083333333333333333
     &D0*T13+9.0D0*T14-20.166666666666666667D0*T15-1.25D0*T16-1.25D0*T17
     &-17.583333333333333333D0*T7+0.083333333333333333333D0*T8+0.0833333
     &33333333333333D0*T9-1.75D0*T10-24.0D0*T3-11.0D0*T4)*T2+(-7.0D0*T19
     &-11.375D0*T20-11.375D0*T21-32.25D0*T22+0.25D0*T23-7.625D0*T24-7.62
     &5D0*T25-0.75D0*T42+3.75D0*T43+3.25D0*T26+3.25D0*T27-0.25D0*T28+16.
     &75D0*T29+0.25D0*T30+0.25D0*T31+21.875D0*T11-21.041666666666666667D
     &0*T12-21.041666666666666667D0*T13-33.0D0*T14+31.375D0*T15-5.541666
     &6666666666667D0*T16-5.5416666666666666667D0*T17+66.361111111111111
     &111D0*T7+0.88888888888888888889D0*T8+0.88888888888888888889D0*T9-1
     &3.75D0*T10+94.5D0*T3+26.625D0*T4)*CA*CF
      T103=((T69+0.625D0)*T2+(4.5833333333333333333D0*T3+2.3541666666666
     &666667D0)*CA*CF)*im*T36+((1.8611111111111111111D0*T3+0.75D0)*CF*Nf
     &+(-1.9166666666666666667D0*T7+1.9166666666666666667D0*T8+2.5D0*T9+
     &9.0416666666666666667D0*T3+T70+4.2083333333333333333D0)*T2+(4.2083
     &333333333333333D0*T7+6.6041666666666666667D0*T8-0.6875D0*T9+0.5416
     &6666666666666667D0*T3+2.7083333333333333333D0*T4-3.777777777777777
     &7778D0)*CA*CF)*T37+((T71+T72-T73-6.4444444444444444444D0*T3-3.1666
     &666666666666667D0)*CF*Nf+(T74+T75-3.5D0*T13+T76+T77-2.5D0*T17+20.6
     &66666666666666667D0*T7+T78-3.1666666666666666667D0*T9-9.6666666666
     &666666667D0*T3+T47-4.3333333333333333333D0)*T2+(4.625D0*T11+33.875
     &D0*T12+5.25D0*T13-4.25D0*T14-0.125D0*T15+1.125D0*T16+T79+15.625D0*
     &T7+4.4166666666666666667D0*T8+10.208333333333333333D0*T9-1.0D0*T10
     &+39.569444444444444444D0*T3-2.9583333333333333333D0*T4+22.625D0)*C
     &A*CF)*im*pi+(T80-0.16666666666666666667D0*T12-0.166666666666666666
     &67D0*T13+1.3333333333333333333D0*T15+7.4444444444444444444D0*T7+3.
     &1666666666666666667D0*T3)*CF*Nf
      T104=(T103+(-22.0D0*T19-12.5D0*T20-12.5D0*T21-14.0D0*T22+3.5D0*T23
     &-5.0D0*T43+T81+T82+2.5D0*T29-19.5D0*T11-4.8333333333333333333D0*T1
     &2-4.8333333333333333333D0*T13-T83+3.1666666666666666667D0*T15+11.1
     &66666666666666667D0*T7+T88+T84+4.3333333333333333333D0*T3)*T2+(5.5
     &D0*T19+10.125D0*T20+10.125D0*T21+44.0D0*T22-1.0D0*T23+4.25D0*T24+4
     &.25D0*T25-1.0D0*T43-1.125D0*T26-1.125D0*T27-0.5D0*T29-15.708333333
     &333333333D0*T11+14.583333333333333333D0*T12+14.583333333333333333D
     &0*T13+19.0D0*T14-9.2083333333333333333D0*T15+T16+T17-42.1111111111
     &11111111D0*T7+2.9583333333333333333D0*T8+2.9583333333333333333D0*T
     &9-22.625D0*T3)*CA*CF)*x
      T105=(((-14.0D0*T2+9.0D0*CA*CF)*im*pi-T5+(-6.0D0*T3-6.0D0*T4-3.0D0
     &)*T2+(4.0D0*T3+T35+3.0D0)*CA*CF)*T33+((38.0D0*T2-27.0D0*CA*CF)*im*
     &pi+2.0D0*CF*Nf+(18.0D0*T3+18.0D0*T4+15.5D0)*T2+(-11.0D0*T3-5.0D0*T
     &4-17.083333333333333333D0)*CA*CF)*T46+((-30.5D0*T2+23.0D0*CA*CF)*i
     &m*pi-1.5D0*CF*Nf+(T6-9.0D0*T4-34.5D0)*T2+(12.375D0*T3-0.375D0*T4+2
     &1.041666666666666667D0)*CA*CF)*T56+((9.5D0*T2+5.625D0*CA*CF)*im*pi
     &+T5+(28.0D0*T3+4.0D0*T4+30.25D0)*T2+(-1.125D0*T3-3.0D0*T4-7.791666
     &6666666666667D0)*CA*CF)*T68+((T6-8.5D0)*T2+(-11.5D0*T3-7.25D0)*CA*
     &CF)*x+8.5D0*T3*T2+7.25D0*T3*CA*CF)*zeta3+T96+T98*T46+T100*T56+T102
     &*T68+T104+(-0.625D0*T3*T2-2.3541666666666666667D0*T3*CA*CF)*im*T36
     &+(-0.75D0*T3*CF*Nf+(0.875D0*T7-0.66666666666666666667D0*T8+T84-4.1
     &25D0*T3)*T2+(-1.5833333333333333333D0*T7-2.7083333333333333333D0*T
     &8+1.0694444444444444444D0*T3)*CA*CF)*T37+((0.5D0*T9+2.666666666666
     &6666667D0*T3)*CF*Nf+(-T85+T86+T63+T87-9.5D0*T7+T88-1.0D0*T9+3.0833
     &333333333333333D0*T3)*T2+(-2.125D0*T11-15.625D0*T12-2.0D0*T13+T14-
     &3.9166666666666666667D0*T7+3.9583333333333333333D0*T8-4.125D0*T9-1
     &5.541666666666666667D0*T3)*CA*CF)*im*pi
      alphaHPL_3l_nfv2=(T105+(T89+T90-2.6666666666666666667D0*T7)*CF*Nf+(T91+4.0D0*T2
     &0+4.0D0*T21+4.0D0*T22-1.5D0*T23+1.5D0*T43-1.5D0*T29+8.0D0*T11+0.75
     &D0*T12+0.75D0*T13+T15-T92)*T2+(-1.25D0*T19-3.375D0*T20-3.375D0*T21
     &-19.0D0*T22+T23+T93+T94+5.4583333333333333333D0*T11-3.958333333333
     &3333333D0*T12-3.9583333333333333333D0*T13+4.125D0*T15+15.541666666
     &666666667D0*T7)*CA*CF)/(T68-1.0D0*x)

      end function alphaHPL_3l_nfv2

      function alphaHPL_3l_nfv(x)
        use types
        use constants
        implicit complex(dp) (T)
        include 'nf.f'
        complex(dp) :: alphaHPL_3l_nfv
        real(dp), intent(in) :: x

        real(dp) :: dabc2

        dabc2 = (nc**2-4._dp)*(nc**2-1._dp)/nc

      T2=Hr1(1)
      T3=Hr1(0)
      T4=Hr2(1,1)
      T5=Hr2(1,0)
      T6=Hr2(0,1)
      T7=Hr2(0,0)
      T8=x**5
      T9=pi**4
      T10=pi**3
      T11=pi*pi
      T12=Hr3(1,1,1)
      T13=Hr3(1,0,0)
      T14=Hr3(0,1,1)
      T15=Hr3(0,0,0)
      T16=Hr4(1,1,1,1)
      T17=Hr4(1,1,1,0)
      T18=Hr4(1,1,0,1)
      T19=Hr4(1,1,0,0)
      T20=Hr4(1,0,1,1)
      T21=Hr4(1,0,1,0)
      T22=Hr4(1,0,0,1)
      T23=Hr4(1,0,0,0)
      T24=Hr4(0,1,1,1)
      T25=Hr4(0,1,1,0)
      T26=Hr4(0,1,0,1)
      T27=Hr4(0,1,0,0)
      T28=Hr4(0,0,1,1)
      T29=Hr4(0,0,1,0)
      T30=Hr4(0,0,0,1)
      T31=Hr4(0,0,0,0)
      T32=Hr3(1,1,0)
      T33=Hr3(1,0,1)
      T34=Hr3(0,1,0)
      T35=Hr3(0,0,1)
      T36=x**4
      T37=x**3
      T38=0.25D0*T5
      T39=0.16666666666666666667D0*T6
      T40=0.16666666666666666667D0*T7
      T41=-5.5D0*T16
      T42=0.25D0*T3
      T43=x*x
      T44=0.33333333333333333333D0*T5
      T45=-0.15625D0*T2
      T46=0.16666666666666666667D0*T3
      T47=0.0625D0*T3
      T48=-0.125D0*T23
      T49=-0.25D0*T14
      T50=0.5D0*T4
      T51=0.16666666666666666667D0*T5
      T52=0.0625D0*T5
      T53=0.0625D0*T6
      T54=0.1875D0*T2
      T55=0.0625D0*T32
      T56=0.0625D0*T33
      T57=(-5.0D0*dabc2*T8+(0.25D0*T2+T42+12.5D0)*dabc2*T36+(dabc2*im*pi
     &+(-0.125D0*T2+0.125D0*T3-11.875D0)*dabc2)*T37+(-1.5D0*dabc2*im*pi+
     &(T2+0.625D0*T3+4.8125D0)*dabc2)*T43+(0.5D0*dabc2*im*pi+(-2.125D0*T
     &2-0.4375D0)*dabc2)*x+T2*dabc2)*zeta3+(0.041666666666666666667D0*da
     &bc2*T9-2.5D0*dabc2*im*T10+(0.16666666666666666667D0*T4+T51+T39+T40
     &+3.25D0)*dabc2*T11+(-5.0D0*T4-5.0D0*T5-5.0D0*T6-5.0D0*T7)*dabc2*im
     &*pi+(T16+T17+T18+T19+T20+T21+T22+T23+T24+T25+T26+T27+T28+T29+T30+T
     &31+5.0D0*T12-5.0D0*T13+5.0D0*T14-5.0D0*T15+6.5D0*T4+6.5D0*T5+6.5D0
     &*T6+6.5D0*T7)*dabc2)*T8+(-0.080555555555555555556D0*dabc2*T9+(0.16
     &666666666666666667D0*T2+T46+6.25D0)*dabc2*im*T10+(-0.875D0*T4-T44-
     &0.5D0*T6+0.041666666666666666667D0*T7-0.083333333333333333333D0*T2
     &-0.083333333333333333333D0*T3-8.125D0)*dabc2*T11+(T12+T32+T33+T13+
     &T14+T34+T35+T15+12.5D0*T4+12.5D0*T5+12.5D0*T6+12.5D0*T7+5.0D0*T2+5
     &.0D0*T3)*dabc2*im*pi+(T41-4.5D0*T17-4.5D0*T18-3.5D0*T19-3.75D0*T20
     &-2.75D0*T21-2.75D0*T22-1.75D0*T23-3.25D0*T24-2.25D0*T25-2.25D0*T26
     &-1.25D0*T27-1.5D0*T28-0.5D0*T29-0.5D0*T30+0.5D0*T31-13.0D0*T12-0.5
     &D0*T32-0.5D0*T33+12.0D0*T13-13.0D0*T14-0.5D0*T34-0.5D0*T35+12.0D0*
     &T15-21.25D0*T4-16.25D0*T5-16.25D0*T6-11.25D0*T7-1.5D0*T2-1.5D0*T3)
     &*dabc2)*T36
      T58=T57+(0.071527777777777777778D0*dabc2*T9+(-0.604166666666666666
     &67D0*T2-0.22916666666666666667D0*T3-4.7916666666666666667D0)*dabc2
     &*im*T10+(1.4583333333333333333D0*T4+T38+0.58333333333333333333D0*T
     &6-0.375D0*T7+T45+0.48958333333333333333D0*T3+6.1458333333333333333
     &D0)*dabc2*T11+(-2.625D0*T12-2.875D0*T32-1.875D0*T33-2.125D0*T13-1.
     &875D0*T14-2.125D0*T34-1.125D0*T35-1.375D0*T15-10.25D0*T4-10.25D0*T
     &5-10.25D0*T6-10.25D0*T7-10.0D0*T2-10.0D0*T3-2.5D0)*dabc2*im*pi+(9.
     &5D0*T16+6.875D0*T17+6.875D0*T18+4.0D0*T19+5.0D0*T20+3.125D0*T21+3.
     &125D0*T22+T23+4.0D0*T24+2.125D0*T25+2.125D0*T26-1.125D0*T29-1.125D
     &0*T30-2.5D0*T31+10.9375D0*T12+0.6875D0*T32+0.6875D0*T33-9.5625D0*T
     &13+11.5625D0*T14+1.3125D0*T34+1.3125D0*T35-8.9375D0*T15+23.125D0*T
     &4+13.125D0*T5+13.125D0*T6+3.125D0*T7+4.25D0*T2+1.75D0*T3-0.5D0)*da
     &bc2)*T37+(-0.085763888888888888889D0*dabc2*T9+(0.625D0*T2+T47+1.02
     &08333333333333333D0)*dabc2*im*T10+(-0.625D0*T4-T38-T39-T40+0.39583
     &333333333333333D0*T2-0.57291666666666666667D0*T3-1.09375D0)*dabc2*
     &T11+(2.25D0*T12+2.625D0*T32+1.125D0*T33+1.5D0*T13+1.125D0*T14+1.5D
     &0*T34+0.375D0*T15+3.125D0*T4+3.375D0*T5+2.375D0*T6+2.625D0*T7+5.25
     &D0*T2+5.25D0*T3+3.75D0)*dabc2*im*pi+(T41-3.25D0*T17-3.25D0*T18-0.6
     &25D0*T19-2.75D0*T20-1.625D0*T21-1.625D0*T22+T48-2.375D0*T24-1.25D0
     &*T25-1.25D0*T26+0.25D0*T27-0.375D0*T28-0.375D0*T29-0.375D0*T30-3.0
     &625D0*T12+T55+T56+3.4375D0*T13-3.25D0*T14-0.875D0*T34-0.875D0*T35+
     &1.75D0*T15-8.6875D0*T4-3.4375D0*T5-3.4375D0*T6+1.8125D0*T7-4.0D0*T
     &2-T42+0.75D0)*dabc2)*T43
      alphaHPL_3l_nfv=(T58+(0.0059027777777777777778D0*dabc2*T9+(-T54+0.020833333333
     &333333333D0)*dabc2*im*T10+(-0.45833333333333333333D0*T4+T44+0.0833
     &33333333333333333D0*T6+T45+T46-0.17708333333333333333D0)*dabc2*T11
     &+(-0.625D0*T12-0.75D0*T32-0.25D0*T33-0.375D0*T13+T49-0.375D0*T34+0
     &.125D0*T35-T50-0.75D0*T5+0.25D0*T6-0.4375D0*T2-T47-1.25D0)*dabc2*i
     &m*pi+(-0.5D0*T16-1.125D0*T17-1.125D0*T18-1.875D0*T19+0.5D0*T20+0.2
     &5D0*T21+0.25D0*T22+T48+0.625D0*T24+0.375D0*T25+0.375D0*T26-0.125D0
     &*T28+0.3125D0*T12-0.1875D0*T32-0.1875D0*T33-0.9375D0*T13+T49+T50+T
     &52+T53+1.25D0*T2-0.25D0)*dabc2)*x+(0.33333333333333333333D0*T4-T51
     &)*dabc2*T11+(0.125D0*T4+T52+T53+T54)*dabc2*im*pi+(T16+T17+T18+T19-
     &0.1875D0*T12-T55-T56-0.0625D0*T14-0.1875D0*T4)*dabc2)/(Nc*T43-1.0D
     &0*Nc*x)


      end function alphaHPL_3l_nfv

      function alphaHPL_3l_no_nfv(x)
        use types
        use constants
        implicit complex(dp) (T)
        include 'nf.f'
        complex(dp) :: alphaHPL_3l_no_nfv
        real(dp), intent(in) :: x

      T2=CF*CF
      T3=Hr1(1)
      T4=Hr1(0)
      T5=CF**3
      T6=CA*CA
      T7=-1.0D0*T4
      T8=Hr2(1,1)
      T9=Hr2(1,0)
      T10=Hr2(0,1)
      T11=Hr2(0,0)
      T12=9.0D0*T3
      T13=9.0D0*T4
      T14=Hr3(1,1,1)
      T15=Hr3(1,1,0)
      T16=Hr3(1,0,1)
      T17=Hr3(1,0,0)
      T18=Hr3(0,1,1)
      T19=Hr3(0,1,0)
      T20=Hr3(0,0,1)
      T21=Hr3(0,0,0)
      T22=Hr4(1,1,1,1)
      T23=Hr4(1,1,0,0)
      T24=Hr4(1,0,1,1)
      T25=Hr4(1,0,0,0)
      T26=Hr4(0,1,1,1)
      T27=Hr4(0,1,0,0)
      T28=Hr4(0,0,1,1)
      T29=Hr4(0,0,0,0)
      T30=Hr4(1,1,1,0)
      T31=Hr4(1,1,0,1)
      T32=Hr4(1,0,1,0)
      T33=Hr4(1,0,0,1)
      T34=Hr4(0,1,1,0)
      T35=Hr4(0,1,0,1)
      T36=Hr4(0,0,1,0)
      T37=Hr4(0,0,0,1)
      T38=x**7
      T39=pi**4
      T40=pi**3
      T41=pi*pi
      T42=-16.916666666666666667D0*T9
      T43=-16.916666666666666667D0*T10
      T44=x**6
      T45=-0.83333333333333333334D0*T9
      T46=-0.83333333333333333334D0*T10
      T47=42.583333333333333333D0*T9
      T48=x**5
      T49=5.5D0*T3
      T50=2.6666666666666666667D0*T8
      T51=24.25D0*T21
      T52=Nf*Nf
      T53=x**4
      T54=-4.3333333333333333333D0*T10
      T55=0.11111111111111111111D0*T8
      T56=0.33333333333333333333D0*T11
      T57=0.35185185185185185185D0*T3
      T58=1.0555555555555555556D0*T4
      T59=10.0D0*T36
      T60=10.0D0*T37
      T61=22.125D0*T15
      T62=22.125D0*T16
      T63=1.875D0*T34
      T64=1.875D0*T35
      T65=7.6666666666666666667D0*T19
      T66=7.6666666666666666667D0*T20
      T67=x**3
      T68=-0.5D0*T11
      T69=1.4722222222222222222D0*T4
      T70=5.5D0*T15
      T71=4.0D0*T19
      T72=0.5D0*T11
      T73=13.125D0*T8
      T74=0.625D0*T11
      T75=0.11111111111111111111D0*T11
      T76=0.35185185185185185185D0*T4
      T77=2.0D0*T4
      T78=0.33333333333333333333D0*T10
      T79=-2.0D0*T19
      T80=-2.0D0*T20
      T81=0.5D0*T27
      T82=2.25D0*T19
      T83=2.25D0*T20
      T84=x*x
      T85=4.0D0*T15
      T86=2.5D0*T18
      T87=0.5D0*T19
      T88=-10.416666666666666667D0*T9
      T89=-2.75D0*T15
      T90=-2.5D0*T26
      T91=-0.25D0*T34
      T92=-0.25D0*T35
      T93=((-10.0D0*CA*CF*Nf-214.0D0*T5+254.0D0*CA*T2-40.5D0*T6*CF)*im*p
     &i+(-4.0D0*T2+(-5.0D0*T3-5.0D0*T4+2.0D0)*CA*CF)*Nf+(-117.0D0*T3-93.
     &0D0*T4-26.0D0)*T5+(136.0D0*T3+124.0D0*T4+68.833333333333333333D0)*
     &CA*T2+(-22.25D0*T3-22.25D0*T4-27.916666666666666667D0)*T6*CF)*T38+
     &((35.0D0*CA*CF*Nf+749.0D0*T5-889.0D0*CA*T2+141.75D0*T6*CF)*im*pi+(
     &14.0D0*T2+(16.333333333333333333D0*T3+16.333333333333333333D0*T4-2
     &.0D0)*CA*CF)*Nf+(374.5D0*T3+290.5D0*T4+196.0D0)*T5+(-423.5D0*T3-38
     &1.5D0*T4-370.91666666666666667D0)*CA*T2+(64.291666666666666667D0*T
     &3+64.291666666666666667D0*T4+119.95833333333333333D0)*T6*CF)*T44+(
     &(-45.0D0*CA*CF*Nf-970.0D0*T5+1152.0D0*CA*T2-184.0D0*T6*CF)*im*pi+(
     &-17.111111111111111111D0*T2+(-19.0D0*T3-19.0D0*T4-6.0D0)*CA*CF)*Nf
     &+(-418.0D0*T3-317.5D0*T4-446.25D0)*T5+(452.75D0*T3+402.75D0*T4+706
     &.36111111111111111D0)*CA*T2+(-58.5D0*T3-59.75D0*T4-195.625D0)*T6*C
     &F)*T48+((25.0D0*CA*CF*Nf+555.5D0*T5-658.0D0*CA*T2+104.75D0*T6*CF)*
     &im*pi+(8.8611111111111111111D0*T2+(T12+T13+9.6666666666666666667D0
     &)*CA*CF)*Nf+(188.5D0*T3+149.0D0*T4+428.5D0)*T5+(-186.75D0*T3-172.0
     &D0*T4-601.52777777777777778D0)*CA*T2+(14.5D0*T3+19.75D0*T4+152.104
     &16666666666667D0)*T6*CF)*T53
      T94=(-0.54166666666666666667D0*CA*CF*Nf-11.641666666666666667D0*T5
     &+14.283333333333333333D0*CA*T2-2.44375D0*T6*CF)*T39+((0.8333333333
     &3333333334D0*T3+0.83333333333333333334D0*T4)*CA*CF*Nf+(11.5D0*T3+2
     &3.5D0*T4-25.5D0)*T5+(-18.666666666666666667D0*T3-24.66666666666666
     &6667D0*T4+29.666666666666666667D0)*CA*T2+(3.7083333333333333333D0*
     &T3+3.7083333333333333333D0*T4-8.4583333333333333334D0)*T6*CF)*im*T
     &40+(((-2.0D0*T3+T77-4.8333333333333333333D0)*T2+(-0.83333333333333
     &333334D0*T8+T45+T46-0.83333333333333333334D0*T11+T3+T7+2.416666666
     &6666666667D0)*CA*CF)*Nf+(-13.666666666666666667D0*T8-13.6666666666
     &66666667D0*T9-13.666666666666666667D0*T10-13.666666666666666667D0*
     &T11+4.0D0*T3-4.0D0*T4+6.0833333333333333333D0)*T5+(20.416666666666
     &666667D0*T8+20.416666666666666667D0*T9+20.416666666666666667D0*T10
     &+20.416666666666666667D0*T11+T12-T13+25.541666666666666667D0)*CA*T
     &2+(-4.0416666666666666667D0*T8-4.0416666666666666667D0*T9-4.041666
     &6666666666667D0*T10-4.0416666666666666667D0*T11-T49+5.5D0*T4-14.29
     &1666666666666667D0)*T6*CF)*T41
      T95=T94+((-5.0D0*T14+5.0D0*T15-5.0D0*T16+5.0D0*T17-5.0D0*T18+5.0D0
     &*T19-5.0D0*T20+5.0D0*T21)*CA*CF*Nf+(-137.0D0*T14+93.0D0*T15-117.0D
     &0*T16+113.0D0*T17-113.0D0*T18+117.0D0*T19-93.0D0*T20+137.0D0*T21-5
     &1.0D0*T8-51.0D0*T9-51.0D0*T10-51.0D0*T11)*T5+(154.0D0*T14-124.0D0*
     &T15+136.0D0*T16-142.0D0*T17+142.0D0*T18-136.0D0*T19+124.0D0*T20-15
     &4.0D0*T21+59.333333333333333333D0*T8+59.333333333333333333D0*T9+59
     &.333333333333333333D0*T10+59.333333333333333333D0*T11)*CA*T2+(-26.
     &25D0*T14+22.25D0*T15-22.25D0*T16+26.25D0*T17-26.25D0*T18+22.25D0*T
     &19-22.25D0*T20+26.25D0*T21-16.916666666666666667D0*T8+T42+T43-16.9
     &16666666666666667D0*T11)*T6*CF)*im*pi+((-8.0D0*T14-T85-4.0D0*T16+T
     &71+4.0D0*T20+8.0D0*T21-9.6666666666666666667D0*T8-9.66666666666666
     &66667D0*T9-9.6666666666666666667D0*T10-9.6666666666666666667D0*T11
     &)*T2+(5.0D0*T22+5.0D0*T23+5.0D0*T24+5.0D0*T25+5.0D0*T26+5.0D0*T27+
     &5.0D0*T28+5.0D0*T29+4.0D0*T14+2.0D0*T15+2.0D0*T16+T79+T80-4.0D0*T2
     &1+4.8333333333333333333D0*T8+4.8333333333333333333D0*T9+4.83333333
     &33333333333D0*T10+4.8333333333333333333D0*T11)*CA*CF)*Nf
      T96=T95+(155.0D0*T22+18.0D0*T30+18.0D0*T31+111.0D0*T23+135.0D0*T24
     &+18.0D0*T32+18.0D0*T33+131.0D0*T25+131.0D0*T26+18.0D0*T34+18.0D0*T
     &35+135.0D0*T27+111.0D0*T28+18.0D0*T36+18.0D0*T37+155.0D0*T29+50.0D
     &0*T14+8.0D0*T15+8.0D0*T16-34.0D0*T17+34.0D0*T18-8.0D0*T19-8.0D0*T2
     &0-50.0D0*T21+12.166666666666666667D0*T8+12.166666666666666667D0*T9
     &+12.166666666666666667D0*T10+12.166666666666666667D0*T11)*T5+(-163
     &.0D0*T22-9.0D0*T30-9.0D0*T31-133.0D0*T23-145.0D0*T24-9.0D0*T32-9.0
     &D0*T33-151.0D0*T25-151.0D0*T26-9.0D0*T34-9.0D0*T35-145.0D0*T27-133
     &.0D0*T28-9.0D0*T36-9.0D0*T37-163.0D0*T29-14.833333333333333333D0*T
     &14+18.0D0*T15+18.0D0*T16+50.833333333333333333D0*T17-50.8333333333
     &33333333D0*T18-18.0D0*T19-18.0D0*T20+14.833333333333333333D0*T21+5
     &1.083333333333333333D0*T8+51.083333333333333333D0*T9+51.0833333333
     &33333333D0*T10+51.083333333333333333D0*T11)*CA*T2+(26.25D0*T22+22.
     &25D0*T23+22.25D0*T24+26.25D0*T25+26.25D0*T26+22.25D0*T27+22.25D0*T
     &28+26.25D0*T29-5.0833333333333333333D0*T14-11.0D0*T15-11.0D0*T16-1
     &6.916666666666666667D0*T17+16.916666666666666667D0*T18+11.0D0*T19+
     &11.0D0*T20+5.0833333333333333333D0*T21-28.583333333333333333D0*T8-
     &28.583333333333333333D0*T9-28.583333333333333333D0*T10-28.58333333
     &3333333333D0*T11)*T6*CF
      T97=(1.8407407407407407407D0*CA*CF*Nf+39.305555555555555556D0*T5-4
     &7.795833333333333333D0*CA*T2+8.0002314814814814815D0*T6*CF)*T39+((
     &-2.7222222222222222222D0*T3-2.7222222222222222222D0*T4-0.833333333
     &33333333334D0)*CA*CF*Nf+(-30.916666666666666667D0*T3-72.9166666666
     &66666667D0*T4+71.75D0)*T5+(54.083333333333333333D0*T3+75.083333333
     &333333333D0*T4-82.166666666666666667D0)*CA*T2+(-10.340277777777777
     &778D0*T3-10.340277777777777778D0*T4+25.895833333333333333D0)*T6*CF
     &)*im*T40+(((6.7222222222222222222D0*T3-7.2777777777777777778D0*T4+
     &16.916666666666666667D0)*T2+(3.1111111111111111111D0*T8+2.72222222
     &22222222222D0*T9+3.1111111111111111111D0*T10+2.7222222222222222222
     &D0*T11-2.5277777777777777778D0*T3+4.4722222222222222222D0*T4-8.458
     &3333333333333334D0)*CA*CF)*Nf+(46.416666666666666667D0*T8+42.75D0*
     &T9+52.916666666666666667D0*T10+49.25D0*T11-1.625D0*T3+26.375D0*T4-
     &21.291666666666666667D0)*T5+(-72.708333333333333333D0*T8-63.708333
     &333333333333D0*T9-79.208333333333333333D0*T10-70.20833333333333333
     &3D0*T11-50.020833333333333333D0*T3+12.979166666666666667D0*T4-89.3
     &95833333333333333D0)*CA*T2+(14.472222222222222222D0*T8+12.19444444
     &4444444444D0*T9+16.097222222222222222D0*T10+13.819444444444444444D
     &0*T11+22.666666666666666667D0*T3-15.833333333333333333D0*T4+50.020
     &833333333333333D0)*T6*CF)*T41
      T98=(18.666666666666666667D0*T14-16.333333333333333333D0*T15+18.66
     &6666666666666667D0*T16-16.333333333333333333D0*T17+18.666666666666
     &666667D0*T18-16.333333333333333333D0*T19+18.666666666666666667D0*T
     &20-16.333333333333333333D0*T21+5.0D0*T8-5.0D0*T9+5.0D0*T10-5.0D0*T
     &11)*CA*CF*Nf+(535.5D0*T14-269.5D0*T15+465.5D0*T16-339.5D0*T17+451.
     &5D0*T18-353.5D0*T19+381.5D0*T20-423.5D0*T21+303.5D0*T8+73.5D0*T9+2
     &83.5D0*T10+53.5D0*T11+55.0D0*T3+55.0D0*T4)*T5+(-606.5D0*T14+366.5D
     &0*T15-543.5D0*T16+429.5D0*T17-564.5D0*T18+408.5D0*T19-501.5D0*T20+
     &471.5D0*T21-355.66666666666666667D0*T8-77.666666666666666667D0*T9-
     &337.66666666666666667D0*T10-59.666666666666666667D0*T11-53.3333333
     &33333333333D0*T3-53.333333333333333333D0*T4)*CA*T2+(107.7083333333
     &3333333D0*T14-62.041666666666666667D0*T15+93.708333333333333333D0*
     &T16-76.041666666666666667D0*T17+107.70833333333333333D0*T18-62.041
     &666666666666667D0*T19+93.708333333333333333D0*T20-76.0416666666666
     &66667D0*T21+85.458333333333333333D0*T8+36.958333333333333333D0*T9+
     &81.458333333333333333D0*T10+32.958333333333333333D0*T11+12.9166666
     &66666666667D0*T3+12.916666666666666667D0*T4)*T6*CF
      T99=T97+T98*im*pi+((26.333333333333333333D0*T14+12.333333333333333
     &333D0*T15+12.333333333333333333D0*T16-1.6666666666666666667D0*T17-
     &1.6666666666666666667D0*T18-15.666666666666666667D0*T19-15.6666666
     &66666666667D0*T20-29.666666666666666667D0*T21+37.83333333333333333
     &3D0*T8+33.833333333333333333D0*T9+33.833333333333333333D0*T10+29.8
     &33333333333333333D0*T11+13.666666666666666667D0*T3+13.666666666666
     &666667D0*T4)*T2+(-18.666666666666666667D0*T22-16.33333333333333333
     &3D0*T23-18.666666666666666667D0*T24-16.333333333333333333D0*T25-18
     &.666666666666666667D0*T26-16.333333333333333333D0*T27-18.666666666
     &666666667D0*T28-16.333333333333333333D0*T29-18.166666666666666667D
     &0*T14-6.1666666666666666667D0*T15-6.1666666666666666667D0*T16-4.16
     &66666666666666667D0*T17-4.1666666666666666667D0*T18+7.833333333333
     &3333333D0*T19+7.8333333333333333333D0*T20+9.8333333333333333334D0*
     &T21-18.916666666666666667D0*T8+T42+T43-14.916666666666666667D0*T11
     &-6.8333333333333333333D0*T3-6.8333333333333333333D0*T4)*CA*CF)*Nf+
     &(-633.0D0*T22-97.5D0*T30-97.5D0*T31-367.0D0*T23-533.0D0*T24-67.5D0
     &*T32-67.5D0*T33-407.0D0*T25-510.0D0*T26-58.5D0*T34-58.5D0*T35-412.
     &0D0*T27-410.0D0*T28-28.5D0*T36-28.5D0*T37-452.0D0*T29-325.75D0*T14
     &-53.75D0*T15-53.75D0*T16-11.75D0*T17-249.75D0*T18+T82+T83+T51-88.5
     &83333333333333333D0*T8-T47-42.583333333333333333D0*T10+3.416666666
     &6666666667D0*T11+41.833333333333333333D0*T3+41.833333333333333333D
     &0*T4)*T5
      T100=T99+(671.0D0*T22+64.5D0*T30+64.5D0*T31+431.0D0*T23+581.0D0*T2
     &4+37.5D0*T32+37.5D0*T33+467.0D0*T25+590.0D0*T26+25.5D0*T34+25.5D0*
     &T35+434.0D0*T27+500.0D0*T28-1.5D0*T36-1.5D0*T37+470.0D0*T29+220.29
     &166666666666667D0*T14-42.625D0*T15-42.625D0*T16-27.541666666666666
     &667D0*T17+328.29166666666666667D0*T18+83.375D0*T19+83.375D0*T20+11
     &6.45833333333333333D0*T21-151.95833333333333333D0*T8-178.791666666
     &66666667D0*T9-178.79166666666666667D0*T10-205.625D0*T11-125.916666
     &66666666667D0*T3-125.91666666666666667D0*T4)*CA*T2+(-115.583333333
     &33333333D0*T22-7.875D0*T30-7.875D0*T31-69.916666666666666667D0*T23
     &-95.583333333333333333D0*T24-1.875D0*T32-1.875D0*T33-77.9166666666
     &66666667D0*T25-105.83333333333333333D0*T26+T63+T64-60.166666666666
     &666667D0*T27-85.833333333333333333D0*T28+7.875D0*T36+7.875D0*T37-6
     &8.166666666666666667D0*T29-12.208333333333333333D0*T14+34.75D0*T15
     &+34.75D0*T16+33.208333333333333333D0*T17-85.208333333333333333D0*T
     &18-42.25D0*T19-42.25D0*T20-47.791666666666666667D0*T21+98.125D0*T8
     &+100.04166666666666667D0*T9+100.04166666666666667D0*T10+101.958333
     &33333333333D0*T11+52.5D0*T3+52.5D0*T4)*T6*CF
      T101=(-2.2722222222222222222D0*CA*CF*Nf-47.099305555555555556D0*T5
     &+56.791666666666666667D0*CA*T2-9.0965277777777777778D0*T6*CF)*T39+
     &((-0.30555555555555555556D0*T2+(3.1666666666666666667D0*T3+3.16666
     &66666666666667D0*T4+2.4722222222222222222D0)*CA*CF)*Nf+(21.4583333
     &33333333333D0*T3+80.125D0*T4-65.125D0)*T5+(-48.291666666666666667D
     &0*T3-79.291666666666666667D0*T4+75.847222222222222222D0)*CA*T2+(8.
     &2291666666666666667D0*T3+9.2291666666666666667D0*T4-29.30555555555
     &5555556D0)*T6*CF)*im*T40+(((-8.5138888888888888889D0*T3+10.1805555
     &55555555556D0*T4-22.509259259259259259D0)*T2+(-4.33333333333333333
     &33D0*T8-3.1666666666666666667D0*T9+T54-3.1666666666666666667D0*T11
     &+1.5138888888888888889D0*T3-7.3472222222222222222D0*T4+10.70833333
     &3333333333D0)*CA*CF)*Nf+(-52.375D0*T8-46.75D0*T9-77.25D0*T10-60.87
     &5D0*T11-28.979166666666666667D0*T3-45.270833333333333333D0*T4+13.3
     &95833333333333333D0)*T5+(90.791666666666666667D0*T8+68.29166666666
     &6666667D0*T9+114.79166666666666667D0*T10+83.291666666666666667D0*T
     &11+109.13888888888888889D0*T3+1.9861111111111111111D0*T4+132.83564
     &814814814815D0)*CA*T2+(-18.145833333333333333D0*T8-12.125D0*T9-23.
     &833333333333333333D0*T10-16.1875D0*T11-38.232638888888888889D0*T3+
     &17.732638888888888889D0*T4-67.472222222222222222D0)*T6*CF)*T41
      T102=((-0.5D0*T8+T45+T46+T68+1.1388888888888888889D0*T3-1.13888888
     &88888888889D0*T4+3.7962962962962962963D0)*T2+(-26.0D0*T14+19.0D0*T
     &15-26.0D0*T16+19.0D0*T17-26.0D0*T18+19.0D0*T19-26.0D0*T20+19.0D0*T
     &21-15.5D0*T8+14.5D0*T9-15.5D0*T10+14.5D0*T11-2.5D0*T3+2.5D0*T4+2.5
     &D0)*CA*CF)*Nf+(-802.5D0*T14+243.0D0*T15-702.5D0*T16+342.0D0*T17-67
     &8.0D0*T18+366.5D0*T19-579.0D0*T20+466.5D0*T21-647.1666666666666666
     &7D0*T8+T47-587.41666666666666667D0*T10+102.83333333333333333D0*T11
     &-218.625D0*T3-111.375D0*T4+39.25D0)*T5+(911.75D0*T14-347.75D0*T15+
     &822.75D0*T16-436.75D0*T17+841.75D0*T18-417.75D0*T19+752.75D0*T20-5
     &06.75D0*T21+767.08333333333333334D0*T8-65.083333333333333333D0*T9+
     &714.91666666666666667D0*T10-120.91666666666666667D0*T11+221.569444
     &44444444444D0*T3+98.430555555555555556D0*T4-67.185185185185185185D
     &0)*CA*T2+(-169.875D0*T14+50.125D0*T15-149.625D0*T16+70.375D0*T17-1
     &65.375D0*T18+54.625D0*T19-145.125D0*T20+74.875D0*T21-170.25D0*T8-2
     &4.75D0*T9-158.25D0*T10-12.75D0*T11-50.875D0*T3-26.625D0*T4+5.66666
     &66666666666667D0)*T6*CF
      T103=T101+T102*im*pi+(T55+T75+T57-T76+1.2530864197530864198D0)*CF*
     &T52+((-31.0D0*T14-13.166666666666666667D0*T15-13.16666666666666666
     &7D0*T16+4.0D0*T17+6.0D0*T18+23.166666666666666667D0*T19+23.1666666
     &66666666667D0*T20+41.0D0*T21-54.888888888888888889D0*T8-42.1666666
     &66666666667D0*T9-42.166666666666666667D0*T10-30.888888888888888889
     &D0*T11-40.638888888888888889D0*T3-41.361111111111111111D0*T4+5.443
     &6728395061728395D0)*T2+(26.0D0*T22+19.0D0*T23+26.0D0*T24+19.0D0*T2
     &5+26.0D0*T26+19.0D0*T27+26.0D0*T28+19.0D0*T29+31.08333333333333333
     &3D0*T14+6.5833333333333333333D0*T15+6.5833333333333333333D0*T16+12
     &.083333333333333333D0*T17+12.916666666666666667D0*T18-11.583333333
     &333333333D0*T19-11.583333333333333333D0*T20-6.0833333333333333333D
     &0*T21+28.361111111111111111D0*T8+21.083333333333333333D0*T9+21.083
     &333333333333333D0*T10+16.361111111111111111D0*T11+14.9074074074074
     &07407D0*T3+26.092592592592592593D0*T4-16.580246913580246914D0)*CA*
     &CF)*Nf+(1012.5D0*T22+210.0D0*T30+210.0D0*T31+453.0D0*T23+808.0D0*T
     &24+105.5D0*T32+105.5D0*T33+447.5D0*T25+756.5D0*T26+78.5D0*T34+78.5
     &D0*T35+445.0D0*T27+582.0D0*T28+3.0D0*T36+3.0D0*T37+469.5D0*T29+719
     &.25D0*T14+113.33333333333333333D0*T15+113.33333333333333333D0*T16+
     &196.41666666666666667D0*T17+588.08333333333333334D0*T18+41.1666666
     &66666666667D0*T19+41.166666666666666667D0*T20+185.25D0*T21+227.916
     &66666666666667D0*T8+35.916666666666666667D0*T9+35.9166666666666666
     &67D0*T10-48.083333333333333333D0*T11-142.125D0*T3-108.875D0*T4+20.
     &177083333333333333D0)*T5
      T104=T103+(-1076.0D0*T22-164.25D0*T30-164.25D0*T31-512.0D0*T23-893
     &.0D0*T24-70.25D0*T32-70.25D0*T33-507.0D0*T25-876.0D0*T26-34.25D0*T
     &34-34.25D0*T35-452.0D0*T27-719.0D0*T28+33.75D0*T36+33.75D0*T37-473
     &.0D0*T29-623.875D0*T14+T61+T62-162.20833333333333333D0*T17-740.041
     &66666666666667D0*T18-144.375D0*T19-144.375D0*T20-386.375D0*T21+112
     &.01388888888888889D0*T8+256.375D0*T9+256.375D0*T10+273.01388888888
     &888889D0*T11+396.84722222222222222D0*T3+358.65277777777777778D0*T4
     &-58.486496913580246914D0)*CA*T2+(199.5D0*T22+29.625D0*T30+29.625D0
     &*T31+79.75D0*T23+158.375D0*T24+8.75D0*T32+8.75D0*T33+79.125D0*T25+
     &162.875D0*T26-2.5D0*T34-2.5D0*T35+52.125D0*T27+127.5D0*T28-17.625D
     &0*T36-17.625D0*T37+57.25D0*T29+81.354166666666666667D0*T14-39.3958
     &33333333333333D0*T15-39.395833333333333333D0*T16-14.64583333333333
     &3333D0*T17+170.64583333333333333D0*T18+61.895833333333333333D0*T19
     &+61.895833333333333333D0*T20+98.645833333333333333D0*T21-115.93055
     &555555555556D0*T8-137.16666666666666667D0*T9-137.16666666666666667
     &D0*T10-127.43055555555555556D0*T11-147.96759259259259259D0*T3-167.
     &03240740740740741D0*T4+51.938271604938271605D0)*T6*CF
      T105=(1.1888888888888888889D0*CA*CF*Nf+22.051041666666666667D0*T5-
     &26.588888888888888889D0*CA*T2+3.7604166666666666667D0*T6*CF)*T39+(
     &(0.58333333333333333334D0*T2+(-1.5D0*T3-1.5D0*T4-2.375D0)*CA*CF)*N
     &f+(6.4583333333333333333D0*T3-35.5D0*T4+16.291666666666666667D0)*T
     &5+(T49+31.375D0*T4-23.715277777777777778D0)*CA*T2+(0.8958333333333
     &3333334D0*T3-1.9583333333333333333D0*T4+14.701388888888888889D0)*T
     &6*CF)*im*T40+(((4.9722222222222222222D0*T3-6.7638888888888888889D0
     &*T4+13.981481481481481481D0)*T2+(T50+1.5D0*T9+2.666666666666666666
     &7D0*T10+1.5D0*T11+0.69444444444444444445D0*T3+5.347222222222222222
     &2D0*T4-5.625D0)*CA*CF)*Nf+(20.791666666666666667D0*T8+20.0D0*T9+50
     &.833333333333333333D0*T10+23.166666666666666667D0*T11+52.75D0*T3+2
     &3.479166666666666667D0*T4+19.739583333333333333D0)*T5+(-45.375D0*T
     &8-27.25D0*T9-73.166666666666666667D0*T10-32.541666666666666667D0*T
     &11-115.19444444444444444D0*T3-4.8125D0*T4-108.59953703703703704D0)
     &*CA*T2+(9.2708333333333333334D0*T8+3.7708333333333333333D0*T9+15.3
     &95833333333333333D0*T10+5.8333333333333333333D0*T11+33.90625D0*T3-
     &9.7569444444444444444D0*T4+43.628472222222222222D0)*T6*CF)*T41
      T106=((T50+T9+3.1666666666666666667D0*T10-0.16666666666666666667D0
     &*T11-1.4722222222222222222D0*T3+4.2222222222222222222D0*T4-9.49074
     &07407407407408D0)*T2+(16.0D0*T14-9.0D0*T15+16.0D0*T16-9.0D0*T17+16
     &.0D0*T18-9.0D0*T19+16.0D0*T20-9.0D0*T21+15.75D0*T8-13.416666666666
     &666667D0*T9+15.916666666666666667D0*T10-13.25D0*T11+5.194444444444
     &4444445D0*T3-7.3055555555555555556D0*T4-6.25D0)*CA*CF)*Nf+(569.5D0
     &*T14-35.0D0*T15+489.5D0*T16-114.0D0*T17+458.5D0*T18-135.5D0*T19+39
     &2.5D0*T20-205.5D0*T21+622.41666666666666667D0*T8-93.08333333333333
     &3333D0*T9+562.66666666666666667D0*T10-154.08333333333333333D0*T11+
     &298.70833333333333333D0*T3+30.583333333333333333D0*T4-98.125D0)*T5
     &+(-643.0D0*T14+72.5D0*T15-573.5D0*T16+148.5D0*T17-562.75D0*T18+147
     &.75D0*T19-508.75D0*T20+208.25D0*T21-752.625D0*T8+119.7916666666666
     &6667D0*T9-706.04166666666666667D0*T10+175.54166666666666667D0*T11-
     &314.06944444444444445D0*T3-6.2222222222222222222D0*T4+167.96296296
     &296296296D0)*CA*T2
      T107=T105+(T106+(128.25D0*T14+3.125D0*T15+111.0D0*T16-15.875D0*T17
     &+114.125D0*T18-10.25D0*T19+101.875D0*T20-T51+166.0D0*T8+6.54166666
     &66666666667D0*T9+154.91666666666666667D0*T10-4.5416666666666666667
     &D0*T11+71.680555555555555556D0*T3+11.055555555555555556D0*T4-14.16
     &6666666666666667D0)*T6*CF)*im*pi+(-0.22222222222222222222D0*T8-T56
     &-0.70370370370370370371D0*T3+T58-3.1327160493827160494D0)*CF*T52+(
     &(13.166666666666666667D0*T14+T70+5.5D0*T16-3.5D0*T17-8.5D0*T18-15.
     &333333333333333333D0*T19-15.333333333333333333D0*T20-26.8333333333
     &33333333D0*T21+33.805555555555555556D0*T8+20.833333333333333333D0*
     &T9+20.833333333333333333D0*T10+11.472222222222222222D0*T11+41.9166
     &66666666666667D0*T3+43.722222222222222222D0*T4-13.6091820987654320
     &99D0)*T2+(-16.0D0*T22-9.0D0*T23-16.0D0*T24-9.0D0*T25-16.0D0*T26-9.
     &0D0*T27-16.0D0*T28-9.0D0*T29-23.166666666666666667D0*T14+T89-2.75D
     &0*T16-11.166666666666666667D0*T17-13.25D0*T18+T65+T66-0.25D0*T21-1
     &9.166666666666666667D0*T8+T88-10.416666666666666667D0*T10-8.055555
     &5555555555556D0*T11-8.0648148148148148148D0*T3-36.0277777777777777
     &78D0*T4+41.450617283950617284D0)*CA*CF)*Nf
      T108=T107+(-784.5D0*T22-215.0D0*T30-215.0D0*T31-250.0D0*T23-578.5D
     &0*T24-89.0D0*T32-89.0D0*T33-203.0D0*T25-514.5D0*T26-56.0D0*T34-56.
     &0D0*T35-191.5D0*T27-395.0D0*T28-2.5D0*T36-2.5D0*T37-208.0D0*T29-69
     &3.75D0*T14-94.583333333333333333D0*T15-94.583333333333333333D0*T16
     &-210.16666666666666667D0*T17-594.33333333333333334D0*T18-54.166666
     &666666666667D0*T19-54.166666666666666667D0*T20-233.75D0*T21-255.04
     &166666666666667D0*T8+16.666666666666666667D0*T9+16.666666666666666
     &667D0*T10+18.375D0*T11+178.375D0*T3+95.25D0*T4-50.4427083333333333
     &33D0)*T5+(830.5D0*T22+187.5D0*T30+187.5D0*T31+260.0D0*T23+647.75D0
     &*T24+74.25D0*T32+74.25D0*T33+222.75D0*T25+592.25D0*T26+29.5D0*T34+
     &29.5D0*T35+177.25D0*T27+490.0D0*T28-18.75D0*T36-18.75D0*T37+189.5D
     &0*T29+686.5D0*T14+1.2083333333333333333D0*T15+1.208333333333333333
     &3D0*T16+187.25D0*T17+742.25D0*T18+102.45833333333333333D0*T19+102.
     &45833333333333333D0*T20+352.33333333333333333D0*T21+43.36111111111
     &1111111D0*T8-193.95833333333333333D0*T9-193.95833333333333333D0*T1
     &0-111.97222222222222222D0*T11-447.66666666666666667D0*T3-352.18055
     &555555555556D0*T4+146.21624228395061728D0)*CA*T2
      T109=(T108+(-168.25D0*T22-40.0D0*T30-40.0D0*T31-36.875D0*T23-125.8
     &75D0*T24-14.875D0*T32-14.875D0*T33-30.75D0*T25-114.875D0*T26-0.75D
     &0*T34-0.75D0*T35-11.0D0*T27-91.875D0*T28+T59+T60-14.25D0*T29-117.2
     &9166666666666667D0*T14+23.041666666666666667D0*T15+23.041666666666
     &666667D0*T16+2.0833333333333333333D0*T17-165.10416666666666667D0*T
     &18-37.6875D0*T19-37.6875D0*T20-71.5625D0*T21+47.409722222222222222
     &D0*T8+92.8125D0*T9+92.8125D0*T10+60.784722222222222222D0*T11+141.2
     &6851851851851852D0*T3+188.93055555555555556D0*T4-129.8456790123456
     &7901D0)*T6*CF)*T53
      T110=(-0.21574074074074074074D0*CA*CF*Nf-2.4430555555555555556D0*T
     &5+3.0861111111111111111D0*CA*T2-0.12071759259259259259D0*T6*CF)*T3
     &9+((-0.33333333333333333333D0*T2+(0.22222222222222222222D0*T3+0.22
     &222222222222222222D0*T4+0.76388888888888888889D0)*CA*CF)*Nf+(-10.0
     &83333333333333333D0*T3+4.75D0*T4+3.2916666666666666667D0)*T5+(11.2
     &08333333333333333D0*T3-2.25D0*T4+1.5069444444444444444D0)*CA*T2+(-
     &4.1597222222222222222D0*T3-0.72222222222222222222D0*T4-3.631944444
     &4444444445D0)*T6*CF)*im*T40+(((-1.2083333333333333333D0*T3+2.01388
     &88888888888889D0*T4-3.9351851851851851852D0)*T2+(-0.61111111111111
     &111111D0*T8-0.22222222222222222222D0*T9-0.61111111111111111111D0*T
     &10-0.22222222222222222222D0*T11-0.88888888888888888889D0*T3-T69+0.
     &875D0)*CA*CF)*Nf+(-8.5D0*T8+1.5833333333333333333D0*T9-9.25D0*T10+
     &0.91666666666666666667D0*T11-32.3125D0*T3+3.6041666666666666667D0*
     &T4-24.916666666666666667D0)*T5+(T73-0.125D0*T9+14.2083333333333333
     &33D0*T10-0.041666666666666666667D0*T11+59.173611111111111111D0*T3-
     &4.3680555555555555556D0*T4+48.56712962962962963D0)*CA*T2+(-3.28472
     &22222222222222D0*T8+0.45138888888888888889D0*T9-3.2847222222222222
     &222D0*T10+0.32638888888888888889D0*T11-16.677083333333333333D0*T3+
     &2.5451388888888888889D0*T4-13.465277777777777778D0)*T6*CF)*T41
      T111=((-4.5D0*T8+T54+1.1666666666666666667D0*T11-0.944444444444444
     &44445D0*T3-4.5555555555555555556D0*T4+7.5925925925925925926D0)*T2+
     &(-3.6666666666666666667D0*T14+1.3333333333333333333D0*T15-3.666666
     &6666666666667D0*T16+1.3333333333333333333D0*T17-3.6666666666666666
     &667D0*T18+1.3333333333333333333D0*T19-3.6666666666666666667D0*T20+
     &1.3333333333333333333D0*T21-4.25D0*T8+3.9166666666666666667D0*T9-4
     &.75D0*T10+3.75D0*T11-0.91666666666666666667D0*T3+5.138888888888888
     &8889D0*T4+5.0D0)*CA*CF)*Nf+(-198.0D0*T14-41.0D0*T15-149.5D0*T16-8.
     &5D0*T17-120.5D0*T18+1.5D0*T19-114.0D0*T20+30.0D0*T21-254.75D0*T8+2
     &9.25D0*T9-232.25D0*T10+48.25D0*T11-141.66666666666666667D0*T3+33.0
     &83333333333333333D0*T4+78.5D0)*T5+(216.5D0*T14+55.0D0*T15+175.75D0
     &*T16+9.25D0*T17+144.25D0*T18+0.75D0*T19+142.5D0*T20-22.0D0*T21+321
     &.54166666666666667D0*T8-43.458333333333333333D0*T9+308.20833333333
     &333333D0*T10-54.791666666666666667D0*T11+154.52777777777777778D0*T
     &3-47.277777777777777778D0*T4-134.37037037037037037D0)*CA*T2
      T112=T110+(T111+(-49.583333333333333333D0*T14-22.45833333333333333
     &3D0*T15-37.833333333333333333D0*T16-7.4583333333333333333D0*T17-30
     &.708333333333333333D0*T18-5.3333333333333333333D0*T19-29.208333333
     &333333333D0*T20-0.083333333333333333333D0*T21-77.95833333333333333
     &3D0*T8+0.75D0*T9-73.5D0*T10+T74-43.229166666666666667D0*T3+6.92361
     &11111111111111D0*T4+11.333333333333333333D0)*T6*CF)*im*pi+(T55+T56
     &+T57-T58+2.5061728395061728395D0)*CF*T52+((2.3333333333333333333D0
     &*T14-0.83333333333333333334D0*T15-0.83333333333333333334D0*T16+1.1
     &666666666666666667D0*T17+6.1666666666666666667D0*T18+3.83333333333
     &33333333D0*T19+3.8333333333333333333D0*T20+8.3333333333333333334D0
     &*T21-5.4722222222222222222D0*T8-2.1666666666666666667D0*T9-2.16666
     &66666666666667D0*T10-0.80555555555555555556D0*T11-16.5833333333333
     &33333D0*T3-18.027777777777777778D0*T4+10.887345679012345679D0)*T2+
     &(3.6666666666666666667D0*T22+1.3333333333333333333D0*T23+3.6666666
     &666666666667D0*T24+1.3333333333333333333D0*T25+3.66666666666666666
     &67D0*T26+1.3333333333333333333D0*T27+3.6666666666666666667D0*T28+1
     &.3333333333333333333D0*T29+4.6666666666666666667D0*T14+0.416666666
     &66666666667D0*T15+0.41666666666666666667D0*T16+3.33333333333333333
     &33D0*T17+3.8333333333333333333D0*T18-1.9166666666666666667D0*T19-1
     &.9166666666666666667D0*T20+0.5D0*T21+2.7777777777777777778D0*T8+1.
     &0833333333333333333D0*T9+1.0833333333333333333D0*T10+0.55555555555
     &555555555D0*T11-1.2592592592592592593D0*T3+21.111111111111111111D0
     &*T4-33.160493827160493827D0)*CA*CF)*Nf
      T113=T112+(288.0D0*T22+90.0D0*T30+90.0D0*T31+49.0D0*T23+192.0D0*T2
     &4+42.5D0*T32+42.5D0*T33+34.0D0*T25+142.0D0*T26+21.5D0*T34+21.5D0*T
     &35+23.0D0*T27+124.0D0*T28+T59+T60+40.0D0*T29+271.0D0*T14+19.25D0*T
     &15+19.25D0*T16+53.0D0*T17+248.5D0*T18+20.75D0*T19+20.75D0*T20+76.5
     &D0*T21+99.958333333333333333D0*T8-31.583333333333333333D0*T9-31.58
     &3333333333333333D0*T10+13.125D0*T11-97.708333333333333333D0*T3-31.
     &208333333333333333D0*T4+40.354166666666666667D0)*T5+(-303.5D0*T22-
     &87.0D0*T30-87.0D0*T31-32.0D0*T23-219.75D0*T24-44.0D0*T32-44.0D0*T3
     &3-34.75D0*T25-158.75D0*T26-14.5D0*T34-14.5D0*T35-13.75D0*T27-147.0
     &D0*T28-4.5D0*T36-4.5D0*T37-26.5D0*T29-292.08333333333333333D0*T14+
     &T61+T62-34.583333333333333333D0*T17-320.66666666666666667D0*T18-25
     &.708333333333333333D0*T19-25.708333333333333333D0*T20-101.83333333
     &333333333D0*T21-48.111111111111111111D0*T8+78.541666666666666667D0
     &*T9+78.541666666666666667D0*T10-5.7777777777777777778D0*T11+208.45
     &833333333333333D0*T3+132.06944444444444444D0*T4-116.97299382716049
     &383D0)*CA*T2
      T114=(T113+(70.583333333333333333D0*T22+21.0D0*T30+21.0D0*T31-1.45
     &83333333333333333D0*T23+49.208333333333333333D0*T24+11.375D0*T32+1
     &1.375D0*T33+3.9166666666666666667D0*T25+32.583333333333333333D0*T2
     &6+T63+T64-3.4583333333333333333D0*T27+28.958333333333333333D0*T28-
     &0.25D0*T36-0.25D0*T37-0.33333333333333333333D0*T29+62.083333333333
     &333333D0*T14-15.875D0*T15-15.875D0*T16-9.625D0*T17+75.666666666666
     &666667D0*T18+T65+T66+15.625D0*T21+4.2152777777777777778D0*T8-31.37
     &5D0*T9-31.375D0*T10-3.3680555555555555556D0*T11-48.634259259259259
     &259D0*T3-86.763888888888888889D0*T4+103.87654320987654321D0)*T6*CF
     &)*T67
      T115=(-0.17256944444444444444D0*T5+0.22361111111111111111D0*CA*T2-
     &0.099652777777777777778D0*T6*CF)*T39+((0.055555555555555555556D0*T
     &2-0.027777777777777777778D0*CA*CF)*Nf+(0.70833333333333333334D0*T3
     &+0.041666666666666666667D0*T4-0.70833333333333333334D0)*T5+(-4.375
     &D0*T3-0.25D0*T4-1.1388888888888888889D0)*CA*T2+(2.2083333333333333
     &333D0*T3+0.083333333333333333333D0*T4+0.79861111111111111111D0)*T6
     &*CF)*im*T40+(((-0.055555555555555555556D0*T3-0.1527777777777777777
     &8D0*T4+0.37962962962962962963D0)*T2+(0.27777777777777777778D0*T3+0
     &.083333333333333333333D0)*CA*CF)*Nf+(15.041666666666666667D0*T8-6.
     &9166666666666666667D0*T9-4.5833333333333333333D0*T10+0.20833333333
     &333333333D0*T11+6.4166666666666666667D0*T3-4.1875D0*T4+6.989583333
     &3333333333D0)*T5+(-13.25D0*T8+4.25D0*T9+3.5833333333333333333D0*T1
     &0+0.083333333333333333333D0*T11-13.215277777777777778D0*T3+3.21527
     &77777777777778D0*T4-8.9490740740740740741D0)*CA*T2+(3.479166666666
     &6666667D0*T8-0.4375D0*T9-T78+4.2326388888888888889D0*T3-0.1875D0*T
     &4+1.5798611111111111111D0)*T6*CF)*T41
      T116=T115+(((3.0D0*T8-0.16666666666666666667D0*T9+2.5D0*T10+T68+1.
     &75D0*T3+T69-1.8981481481481481482D0)*T2+(-1.75D0*T8-1.25D0*T10-3.7
     &5D0*T3-0.33333333333333333333D0*T4-1.25D0)*CA*CF)*Nf+(45.0D0*T14+T
     &70+15.0D0*T16+8.0D0*T17-1.0D0*T18+T71+13.0D0*T20-1.5D0*T21+19.4166
     &66666666666667D0*T8-6.1666666666666666667D0*T9+19.5833333333333333
     &33D0*T10+T72-13.291666666666666667D0*T3-7.2916666666666666667D0*T4
     &-19.625D0)*T5+(-45.5D0*T14-25.75D0*T15-20.0D0*T16-10.25D0*T17+1.5D
     &0*T18-2.75D0*T19-9.75D0*T20-31.208333333333333333D0*T8+17.5D0*T9-3
     &6.041666666666666667D0*T10+T72+13.541666666666666667D0*T3+8.402777
     &7777777777778D0*T4+33.592592592592592593D0)*CA*T2+(13.625D0*T14+11
     &.75D0*T15+5.875D0*T16+3.375D0*T17+0.125D0*T18+T87+T20+T73-6.5625D0
     &*T9+13.4375D0*T10+T74+10.333333333333333333D0*T3-4.270833333333333
     &3333D0*T4-2.8333333333333333333D0)*T6*CF)*im*pi+(-T75+T76-0.626543
     &20987654320988D0)*CF*T52+((-3.8333333333333333333D0*T14+0.16666666
     &666666666667D0*T15+0.16666666666666666667D0*T16-T86-0.833333333333
     &33333334D0*T21-2.0833333333333333333D0*T8-0.66666666666666666667D0
     &*T9-0.66666666666666666667D0*T10+0.055555555555555555556D0*T11+1.6
     &388888888888888889D0*T3+T77-2.7218364197530864198D0)*T2+(2.6666666
     &666666666667D0*T14-0.083333333333333333333D0*T15-0.083333333333333
     &333333D0*T16-0.083333333333333333333D0*T17+1.25D0*T18+4.0833333333
     &333333333D0*T8+0.33333333333333333333D0*T9+T78+1.22222222222222222
     &22D0*T11+1.25D0*T3-4.3425925925925925926D0*T4+8.290123456790123456
     &8D0)*CA*CF)*Nf
      T117=T116+(-40.5D0*T22+4.5D0*T30+4.5D0*T31+10.0D0*T23-25.5D0*T24-1
     &0.5D0*T32-10.5D0*T33-2.5D0*T25+T90-3.5D0*T34-3.5D0*T35+T81-13.0D0*
     &T28-1.5D0*T29-9.0D0*T14+12.666666666666666667D0*T15+12.66666666666
     &6666667D0*T16+6.5D0*T17-21.583333333333333333D0*T18+T79+T80-2.25D0
     &*T21+23.458333333333333333D0*T8+9.4166666666666666667D0*T9+9.41666
     &66666666666667D0*T10+T11+19.625D0*T3+3.0D0*T4-10.08854166666666666
     &7D0)*T5+(45.5D0*T22-25.75D0*T23+33.25D0*T24+13.25D0*T32+13.25D0*T3
     &3+3.0D0*T25+0.75D0*T26+2.25D0*T34+2.25D0*T35-T81+9.75D0*T28+7.7083
     &333333333333333D0*T14-31.25D0*T15-31.25D0*T16-13.75D0*T17+38.29166
     &6666666666667D0*T18+T82+T83+4.5833333333333333333D0*T21-28.625D0*T
     &8-13.25D0*T9-13.25D0*T10-0.72222222222222222222D0*T11-31.722222222
     &222222222D0*T3-12.625D0*T4+29.243248456790123457D0)*CA*T2+(-14.75D
     &0*T22-1.125D0*T30-1.125D0*T31+10.625D0*T23-9.875D0*T24-4.0D0*T32-4
     &.0D0*T33-0.625D0*T25-0.375D0*T26+T91+T92+0.25D0*T27-1.0D0*T28-6.16
     &66666666666666667D0*T14+12.458333333333333333D0*T15+12.45833333333
     &3333333D0*T16+5.8958333333333333333D0*T17-14.0625D0*T18-0.625D0*T1
     &9-0.625D0*T20-6.0625D0*T8+4.2708333333333333333D0*T9+4.27083333333
     &33333333D0*T10-3.3611111111111111111D0*T11+2.8333333333333333333D0
     &*T3+12.365740740740740741D0*T4-25.969135802469135803D0)*T6*CF
      T118=(0.875D0*T3*T5+0.54166666666666666667D0*T3*CA*T2-0.5416666666
     &6666666667D0*T3*T6*CF)*im*T40+((0.083333333333333333333D0*T3*T2-0.
     &069444444444444444444D0*T3*CA*CF)*Nf+(-8.7083333333333333334D0*T8+
     &3.0D0*T9+T10-0.25D0*T3)*T5+(8.0D0*T8-1.875D0*T9-0.625D0*T10+1.1180
     &555555555555556D0*T3)*CA*T2+(-2.0D0*T8+0.1875D0*T9-0.3958333333333
     &3333333D0*T3)*T6*CF)*T41+(((-0.66666666666666666667D0*T8-0.5D0*T10
     &-0.47222222222222222222D0*T3)*T2+(0.75D0*T8+0.58333333333333333334
     &D0*T10+1.9722222222222222222D0*T3)*CA*CF)*Nf+(-15.5D0*T14+T85-1.0D
     &0*T16-1.0D0*T17+T86-1.0D0*T20+7.5833333333333333333D0*T8+4.9166666
     &666666666667D0*T9+4.9166666666666666667D0*T10+19.875D0*T3)*T5+(15.
     &75D0*T14+3.5D0*T15+2.5D0*T16+1.75D0*T17-2.25D0*T18-T87+0.75D0*T20-
     &8.4583333333333333334D0*T8+T88-2.7083333333333333333D0*T10-22.2361
     &11111111111111D0*T3)*CA*T2+(-4.625D0*T14+T89-0.875D0*T16-0.625D0*T
     &17+0.375D0*T18+0.25D0*T19+0.54166666666666666667D0*T8+3.9791666666
     &666666667D0*T9-1.1458333333333333333D0*T10-0.82638888888888888889D
     &0*T3)*T6*CF)*im*pi
      T119=(T118+((T14+0.5D0*T18+0.47222222222222222222D0*T8)*T2+(-1.083
     &3333333333333333D0*T14-0.58333333333333333334D0*T18-1.972222222222
     &2222222D0*T8)*CA*CF)*Nf+(5.5D0*T22-10.0D0*T30-10.0D0*T31-6.0D0*T23
     &+2.0D0*T24+T32+T33+T90+T28-11.75D0*T14-4.9166666666666666667D0*T15
     &-4.9166666666666666667D0*T16-4.9166666666666666667D0*T18-19.875D0*
     &T8)*T5+(-7.5D0*T22+8.25D0*T30+8.25D0*T31+11.75D0*T23-4.25D0*T24-1.
     &75D0*T32-1.75D0*T33+2.75D0*T26+0.5D0*T34+0.5D0*T35-0.75D0*T28+16.2
     &91666666666666667D0*T14+10.416666666666666667D0*T15+10.41666666666
     &6666667D0*T16+2.7083333333333333333D0*T18+22.236111111111111111D0*
     &T8)*CA*T2+(3.0D0*T22-1.625D0*T30-1.625D0*T31-4.375D0*T23+1.5D0*T24
     &+0.625D0*T32+0.625D0*T33-0.625D0*T26+T91+T92-2.6875D0*T14-3.979166
     &6666666666667D0*T15-3.9791666666666666667D0*T16+1.1458333333333333
     &333D0*T18+0.82638888888888888889D0*T8)*T6*CF)*x
      alphaHPL_3l_no_nfv=((T93+((-5.0D0*CA*CF*Nf-120.5D0*T5+145.75D0*CA*T2-23.75D0*T6*C
     &F)*im*pi+(-1.8888888888888888889D0*T2+(-1.3333333333333333333D0*T3
     &-1.3333333333333333333D0*T4-3.9166666666666666667D0)*CA*CF)*Nf+(-5
     &6.5D0*T3-28.0D0*T4-172.25D0)*T5+(45.5D0*T3+25.75D0*T4+215.76388888
     &888888889D0)*CA*T2+(-1.9166666666666666667D0*T3-2.0416666666666666
     &667D0*T4-52.458333333333333333D0)*T6*CF)*T67+((-4.75D0*CA*T2+1.75D
     &0*T6*CF)*im*pi+(0.13888888888888888889D0*T2+0.25D0*CA*CF)*Nf+(48.0
     &D0*T3+T7+20.0D0)*T5+(-39.0D0*T3+T4-18.513888888888888889D0)*CA*T2+
     &(6.125D0*T3+3.9375D0)*T6*CF)*T84+(-19.5D0*T3*T5+15.0D0*T3*CA*T2-2.
     &25D0*T3*T6*CF)*x)*zeta3+T96*T38+T100*T44+T104*T48+T109+T114+T117*T
     &84+T119+(T8*T5-1.0D0*T8*CA*T2+0.25D0*T8*T6*CF)*T41+(3.0D0*T14*T5-3
     &.0D0*T14*CA*T2+0.75D0*T14*T6*CF)*im*pi-3.0D0*T22*T5+3.0D0*T22*CA*T
     &2-0.75D0*T22*T6*CF)/(T53-2.0D0*T67+T84)

      end function alphaHPL_3l_no_nfv

      function betaHPL_3l_nfv2(x)
        use types
        use constants
        implicit complex(dp) (T)
        include 'nf.f'
        complex(dp) :: betaHPL_3l_nfv2
        real(dp), intent(in) :: x


      T2=CF*CF
      T3=Hr1(1)
      T4=Hr1(0)
      T5=4.0D0*T3
      T6=Hr2(1,1)
      T7=Hr2(1,0)
      T8=Hr2(0,1)
      T9=Hr2(0,0)
      T10=x**4
      T11=pi*pi
      T12=8.0D0*T7
      T13=Hr3(1,1,1)
      T14=Hr3(1,1,0)
      T15=Hr3(1,0,1)
      T16=Hr3(1,0,0)
      T17=22.0D0*T9
      T18=x**3
      T19=16.0D0*T3
      T20=32.0D0*T7
      T21=32.0D0*T4
      T22=5.3333333333333333333D0*T6
      T23=5.3333333333333333333D0*T4
      T24=-4.0D0*T13
      T25=x*x
      T26=40.0D0*T7
      T27=16.0D0*T4
      T28=-6.6666666666666666667D0*T7
      T29=-2.6666666666666666667D0*T4
      T30=18.0D0*T6
      T31=6.0D0*T7
      T32=8.0D0*T8
      T33=2.0D0*T4
      T34=-16.0D0*T6
      T35=-24.0D0*T6
      T36=16.0D0*T7
      T37=-10.0D0*T6
      T38=-0.33333333333333333333D0*T3
      T39=0.33333333333333333333D0*T4
      T40=-4.0D0*T15
      T41=2.0D0*T13
      T42=Hr3(0,1,1)
      T43=Hr3(0,1,0)
      T44=Hr3(0,0,1)
      T45=Hr3(0,0,0)
      T46=0.66666666666666666667D0*T6
      T47=0.66666666666666666667D0*T7
      T48=0.66666666666666666667D0*T8
      T49=0.66666666666666666667D0*T9
      T50=0.33333333333333333333D0*T6
      T51=0.33333333333333333333D0*T7
      T52=0.33333333333333333333D0*T8
      T53=0.33333333333333333333D0*T9
      T54=Hr4(1,1,1,1)
      T55=Hr4(1,1,0,0)
      T56=Hr4(1,0,1,1)
      T57=Hr4(1,0,0,0)
      T58=Hr4(0,1,1,1)
      T59=Hr4(0,1,0,0)
      T60=Hr4(0,0,1,1)
      T61=Hr4(0,0,0,0)
      T62=x**5
      T63=pi**4
      T64=pi**3
      T65=-0.66666666666666666667D0*T15
      T66=0.66666666666666666667D0*T43
      T67=0.074074074074074074074D0*T7
      T68=0.074074074074074074074D0*T9
      T69=0.88888888888888888889D0*T3
      T70=0.88888888888888888889D0*T4
      T71=-2.0D0*T45
      T72=0.5D0*T6
      T73=0.5D0*T8
      T74=Hr4(1,1,1,0)
      T75=Hr4(1,1,0,1)
      T76=Hr4(1,0,1,0)
      T77=Hr4(0,1,0,1)
      T78=Hr4(0,0,1,0)
      T79=Hr4(0,0,0,1)
      T80=0.22222222222222222222D0*T3
      T81=0.22222222222222222222D0*T4
      T82=Hr4(1,0,0,1)
      T83=Hr4(0,1,1,0)
      T84=-0.66666666666666666667D0*T13
      T85=0.66666666666666666667D0*T16
      T86=0.66666666666666666667D0*T42
      T87=0.66666666666666666667D0*T45
      T88=Hr5(1,1,1,1,0)
      T89=Hr5(1,1,1,0,1)
      T90=Hr5(1,1,0,1,1)
      T91=Hr5(1,1,0,1,0)
      T92=Hr5(1,1,0,0,1)
      T93=Hr5(1,1,0,0,0)
      T94=Hr5(1,0,1,1,1)
      T95=Hr5(1,0,1,1,0)
      T96=Hr5(1,0,1,0,1)
      T97=Hr5(1,0,1,0,0)
      T98=Hr5(0,1,0,1,1)
      T99=Hr5(0,1,0,1,0)
      T100=Hr5(0,1,0,0,1)
      T101=Hr5(0,1,0,0,0)
      T102=Hr5(0,0,1,1,1)
      T103=Hr5(0,0,1,1,0)
      T104=Hr5(0,0,1,0,1)
      T105=Hr5(0,0,1,0,0)
      T106=Hr5(0,0,0,1,0)
      T107=Hr5(0,0,0,0,1)
      T108=pi**5
      T109=-0.22222222222222222222D0*T6
      T110=0.66666666666666666667D0*T13
      T111=0.66666666666666666667D0*T15
      T112=2.6666666666666666667D0*T7
      T113=0.83333333333333333334D0*T9
      T114=0.66666666666666666667D0*T4
      T115=-2.0D0*T14
      T116=-0.44444444444444444445D0*T16
      T117=0.074074074074074074074D0*T8
      T118=1.3333333333333333333D0*T56
      T119=1.3333333333333333333D0*T76
      T120=-0.66666666666666666667D0*T57
      T121=1.3888888888888888889D0*T44
      T122=-1.6666666666666666667D0*T9
      T123=1.3333333333333333333D0*T45
      T124=0.44444444444444444445D0*T42
      T125=0.44444444444444444445D0*T44
      T126=0.44444444444444444445D0*T45
      T127=Hr5(1,1,1,0,0)
      T128=4.0D0*T91
      T129=Hr5(1,0,0,1,0)
      T130=Hr5(1,0,0,0,1)
      T131=0.66666666666666666667D0*T58
      T132=4.6666666666666666667D0*T60
      T133=0.66666666666666666667D0*T61
      T134=6.0D0*T93
      T135=4.0D0*T95
      T136=1.3333333333333333333D0*T61
      T137=-1.3333333333333333333D0*T127
      T138=0.44444444444444444445D0*T83
      T139=0.44444444444444444445D0*T77
      T140=0.44444444444444444445D0*T60
      T141=0.44444444444444444445D0*T78
      T142=0.44444444444444444445D0*T79
      T143=Hr5(1,0,0,1,1)
      T144=0.66666666666666666667D0*Hr5(0,1,1,1,0)
      T145=0.66666666666666666667D0*Hr5(0,1,1,0,1)
      T146=6.0D0*T102
      T147=1.3333333333333333333D0*T103
      T148=1.3333333333333333333D0*T104
      T149=Hr5(0,0,0,1,1)
      T150=0.66666666666666666667D0*T106
      T151=0.66666666666666666667D0*T107
      T152=3.0D0*T58
      T153=Hr6(1,1,1,0,0,0)
      T154=Hr6(1,1,0,1,0,0)
      T155=Hr6(1,1,0,0,1,1)
      T156=Hr6(1,1,0,0,1,0)
      T157=Hr6(1,1,0,0,0,1)
      T158=Hr6(1,1,0,0,0,0)
      T159=Hr6(1,0,1,1,0,0)
      T160=Hr6(1,0,1,0,1,1)
      T161=Hr6(1,0,1,0,1,0)
      T162=Hr6(1,0,1,0,0,1)
      T163=Hr6(1,0,1,0,0,0)
      T164=Hr6(1,0,0,1,0,0)
      T165=Hr6(1,0,0,0,1,1)
      T166=Hr5(1,0,0,0,0)
      T167=Hr5(0,1,1,1,0)
      T168=Hr5(0,1,1,0,1)
      T169=Hr5(0,1,1,0,0)
      T170=4.0D0*T102
      T171=1.3333333333333333333D0*T106
      T172=1.3333333333333333333D0*T107
      T173=-8.0D0*T54
      T174=-2.0D0*T58
      T175=1.1111111111111111111D0*T3
      T176=2.6666666666666666667D0*T13
      T177=4.0D0*T14
      T178=1.7777777777777777778D0*T16
      T179=1.7777777777777777778D0*T9
      T180=5.3333333333333333333D0*T13
      T181=-13.0D0*T15
      T182=-0.5D0*T9
      T183=2.8333333333333333333D0*T3
      T184=0.66666666666666666667D0*T74
      T185=8.0D0*T13
      T186=8.0D0*T15
      T187=4.2962962962962962963D0*T3
      T188=4.0D0*T54
      T189=20.0D0*T55
      T190=24.0D0*T93
      T191=16.0D0*T95
      T192=-12.0D0*T97
      T193=4.0D0*T130
      T194=8.0D0*T54
      T195=-2.6666666666666666667D0*T74
      T196=2.6666666666666666667D0*T75
      T197=2.6666666666666666667D0*T76
      T198=2.6666666666666666667D0*T45
      T199=Hr6(1,1,1,1,0,0)
      T200=4.0D0*T88
      T201=4.0D0*T89
      T202=16.0D0*T91
      T203=16.0D0*T92
      T204=-1.0D0*T14
      T205=Hr6(1,1,1,0,1,0)
      T206=Hr6(1,1,1,0,0,1)
      T207=Hr6(1,1,0,1,1,0)
      T208=Hr6(1,1,0,1,0,1)
      T209=Hr6(1,0,1,1,1,0)
      T210=Hr6(1,0,1,1,0,1)
      T211=Hr6(1,0,0,1,1,0)
      T212=Hr6(1,0,0,1,0,1)
      T213=4.0D0*T165
      T214=-2.0D0*T61
      T215=3.3333333333333333333D0*T6
      T216=0.18518518518518518519D0*T6
      T217=-3.6666666666666666667D0*T13
      T218=7.8888888888888888889D0*T15
      T219=1.6666666666666666667D0*T9
      T220=7.6666666666666666667D0*T44
      T221=1.7777777777777777778D0*T13
      T222=-1.1111111111111111111D0*T15
      T223=-5.3333333333333333333D0*T43
      T224=5.7407407407407407407D0*T6
      T225=-0.11111111111111111111D0*T8
      T226=-2.6666666666666666667D0*T54
      T227=14.666666666666666667D0*T74
      T228=12.666666666666666667D0*T75
      T229=12.666666666666666667D0*T56
      T230=-4.6666666666666666667D0*T82
      T231=-21.333333333333333333D0*T57
      T232=-7.6666666666666666667D0*T43
      T233=20.0D0*T95
      T234=-5.3333333333333333333D0*T54
      T235=-11.5D0*T57
      T236=-10.0D0*T83
      T237=8.0D0*T60
      T238=1.7777777777777777778D0*T74
      T239=1.7777777777777777778D0*T75
      T240=1.1111111111111111111D0*T56
      T241=-1.7777777777777777778D0*T76
      T242=-1.7777777777777777778D0*T82
      T243=-5.3333333333333333333D0*T57
      T244=5.7407407407407407407D0*T14
      T245=5.7407407407407407407D0*T15
      T246=0.44444444444444444445D0*T43
      T247=-2.6666666666666666667D0*T88
      T248=-2.6666666666666666667D0*T89
      T249=12.0D0*T127
      T250=-22.0D0*T90
      T251=-9.3333333333333333334D0*T91
      T252=-9.3333333333333333334D0*T92
      T253=-20.0D0*T94
      T254=-7.3333333333333333333D0*T95
      T255=-7.3333333333333333333D0*T96
      T256=-30.0D0*T143
      T257=-34.666666666666666667D0*T129
      T258=-34.666666666666666667D0*T130
      T259=-56.0D0*T166
      T260=8.0D0*T169
      T261=8.0D0*T105
      T262=8.0D0*T149
      T263=-10.0D0*T59
      T264=0.66666666666666666667D0*T79
      T265=4.0D0*T13
      T266=14.0D0*T7
      T267=-5.3333333333333333333D0*T88
      T268=-5.3333333333333333333D0*T89
      T269=9.6666666666666666667D0*T90
      T270=13.5D0*T94
      T271=15.5D0*T129
      T272=15.5D0*T130
      T273=4.0D0*T166
      T274=3.0D0*T167
      T275=3.0D0*T168
      T276=-7.0D0*T169
      T277=9.8888888888888888889D0*T44
      T278=1.3333333333333333333D0*T13
      T279=2.0D0*T14
      T280=1.3333333333333333333D0*T15
      T281=-0.16666666666666666667D0*T6
      T282=2.6666666666666666667D0*T15
      T283=-0.074074074074074074074D0*T6
      T284=1.8518518518518518519D0*T7
      T285=4.6666666666666666667D0*T55
      T286=-1.3888888888888888889D0*T15
      T287=-0.33333333333333333333D0*T16
      T288=6.6666666666666666667D0*T56
      T289=-0.66666666666666666667D0*T76
      T290=2.6666666666666666667D0*T74
      T291=5.3333333333333333333D0*T76
      T292=-0.44444444444444444445D0*T13
      T293=0.44444444444444444445D0*T15
      T294=-1.4074074074074074074D0*T6
      T295=2.0D0*T3
      T296=8.0D0*T88
      T297=8.0D0*T127
      T298=0.66666666666666666667D0*T54
      T299=-4.6666666666666666667D0*T75
      T300=-4.6666666666666666667D0*T56
      T301=0.66666666666666666667D0*T76
      T302=0.66666666666666666667D0*T82
      T303=9.0D0*T13
      T304=-4.0D0*T127
      T305=12.0D0*T93
      T306=-4.0D0*T94
      T307=8.0D0*T95
      T308=12.0D0*T96
      T309=6.0D0*T97
      T310=1.3333333333333333333D0*T54
      T311=-0.44444444444444444445D0*T74
      T312=-0.44444444444444444445D0*T75
      T313=-0.44444444444444444445D0*T56
      T314=0.44444444444444444445D0*T76
      T315=0.44444444444444444445D0*T82
      T316=-1.4074074074074074074D0*T14
      T317=-1.4074074074074074074D0*T15
      T318=8.0D0*T154
      T319=32.0D0*T156
      T320=32.0D0*T157
      T321=8.0D0*T159
      T322=16.0D0*T161
      T323=16.0D0*T162
      T324=8.0D0*T164
      T325=0.66666666666666666667D0*T88
      T326=0.66666666666666666667D0*T89
      T327=6.0D0*T90
      T328=1.3333333333333333333D0*T91
      T329=1.3333333333333333333D0*T92
      T330=6.0D0*T94
      T331=1.3333333333333333333D0*T95
      T332=1.3333333333333333333D0*T96
      T333=0.66666666666666666667D0*T129
      T334=0.66666666666666666667D0*T130
      T335=10.0D0*T56
      T336=4.0D0*T154
      T337=-16.0D0*T156
      T338=-16.0D0*T157
      T339=4.0D0*T159
      T340=-20.0D0*T160
      T341=-8.0D0*T161
      T342=-8.0D0*T162
      T343=4.0D0*T164
      T344=1.3333333333333333333D0*T88
      T345=1.3333333333333333333D0*T89
      T346=-2.6666666666666666667D0*T90
      T347=-16.0D0*T143
      T348=((-8.0D0*T2+4.0D0*CA*CF)*im*pi+(-T5-4.0D0*T4-3.0D0)*T2+(T295+
     &T33+0.66666666666666666667D0)*CA*CF)*T62+((-2.3888888888888888889D
     &0*T2+1.1111111111111111111D0*CA*CF)*T11+(0.88888888888888888889D0*
     &CF*Nf+(5.3333333333333333333D0*T3+T23+9.0D0)*T2+(-6.66666666666666
     &66667D0*T3+T29-15.222222222222222222D0)*CA*CF)*im*pi+(T69+T70-2.07
     &40740740740740741D0)*CF*Nf+(T37+T31-10.0D0*T8+6.0D0*T9+T5+8.0D0*T4
     &+10.666666666666666667D0)*T2+(T22+T28+T32-4.0D0*T9-7.2777777777777
     &777778D0*T3-7.2777777777777777778D0*T4-1.8703703703703703704D0)*CA
     &*CF)*T10+(((7.0D0*T3+16.388888888888888889D0)*T2+(-1.1666666666666
     &666667D0*T3-0.44444444444444444445D0)*CA*CF)*T11+((-T5-8.888888888
     &8888888889D0)*CF*Nf+(8.0D0*T6-T12-7.3333333333333333333D0*T3-21.33
     &3333333333333333D0*T4+7.6666666666666666667D0)*T2+(T12+51.16666666
     &6666666667D0*T3+18.666666666666666667D0*T4+68.638888888888888889D0
     &)*CA*CF)*im*pi+(1.3333333333333333333D0*T6-1.3333333333333333333D0
     &*T7-4.6666666666666666667D0*T3-3.5555555555555555556D0*T4-4.037037
     &037037037037D0)*CF*Nf+(T24+24.0D0*T14+T40-8.0D0*T16+T30+22.0D0*T7+
     &2.0D0*T8-T17-T19+33.0D0*T4-61.166666666666666667D0)*T2+(T13-17.0D0
     &*T14+T181+9.0D0*T16-49.166666666666666667D0*T6-24.0D0*T7-44.5D0*T8
     &+T17+45.555555555555555556D0*T3-54.638888888888888889D0*T4+139.870
     &37037037037037D0)*CA*CF)*T18
      T349=T348+(((-28.0D0*T3-28.0D0)*T2+(-1.3333333333333333333D0*T3-4.
     &3333333333333333333D0)*CA*CF)*T11+((T19+16.0D0)*CF*Nf+(-32.0D0*T6+
     &T20+24.0D0*T3+T21+6.6666666666666666667D0)*T2+(T35-T20-131.0D0*T3-
     &T21-56.833333333333333333D0)*CA*CF)*im*pi+(-T22+5.3333333333333333
     &333D0*T7+19.111111111111111111D0*T3+T23+16.888888888888888889D0)*C
     &F*Nf+(16.0D0*T13-96.0D0*T14+16.0D0*T15+32.0D0*T16+T34-48.0D0*T7+16
     &.0D0*T8+32.0D0*T9+75.0D0*T3-25.0D0*T4+110.0D0)*T2+(T24+68.0D0*T14+
     &70.0D0*T15-36.0D0*T16+144.33333333333333333D0*T6+76.66666666666666
     &6667D0*T7+82.0D0*T8-36.0D0*T9-213.27777777777777778D0*T3+73.833333
     &333333333333D0*T4-319.0D0)*CA*CF)*T25+(((35.0D0*T3+14.0D0)*T2+(6.1
     &666666666666666667D0*T3+3.6666666666666666667D0)*CA*CF)*T11+((-20.
     &0D0*T3-8.0D0)*CF*Nf+(40.0D0*T6-T26-47.333333333333333333D0*T3-T27-
     &9.3333333333333333334D0)*T2+(48.0D0*T6+T26+123.16666666666666667D0
     &*T3+T27+20.666666666666666667D0)*CA*CF)*im*pi+(6.66666666666666666
     &67D0*T6+T28-27.777777777777777778D0*T3+T29-9.7777777777777777778D0
     &)*CF*Nf+(-20.0D0*T13+120.0D0*T14-20.0D0*T15-40.0D0*T16+T30+T31-T32
     &-16.0D0*T9-117.0D0*T3+T33-57.0D0)*T2+(5.0D0*T13-85.0D0*T14-101.0D0
     &*T15+45.0D0*T16-162.5D0*T6-54.666666666666666667D0*T7-45.5D0*T8+18
     &.0D0*T9+310.5D0*T3-26.666666666666666667D0*T4+164.1666666666666666
     &7D0)*CA*CF)*x
      T350=(-0.43333333333333333333D0*T2+0.21666666666666666667D0*CA*CF)
     &*T63+((0.66666666666666666667D0*T3+T114-0.83333333333333333334D0)*
     &T2+(T38-T39+0.33333333333333333333D0)*CA*CF)*im*T64+(-0.1666666666
     &6666666667D0*CF*Nf+(-T46-T47-T48-T49+T38+T39-0.25D0)*T2+(T50+T51+T
     &52+T53+1.1666666666666666667D0)*CA*CF)*T11+((T24+T177+T40+4.0D0*T1
     &6-4.0D0*T42+4.0D0*T43-4.0D0*T44+4.0D0*T45-1.6666666666666666667D0*
     &T6-1.6666666666666666667D0*T7-1.6666666666666666667D0*T8+T122)*T2+
     &(T41+T115+2.0D0*T15-2.0D0*T16+2.0D0*T42-2.0D0*T43+2.0D0*T44+T71+T4
     &6+T47+T48+T49)*CA*CF)*im*pi+(-T50-T51-T52-T53)*CF*Nf+(T188+4.0D0*T
     &55+4.0D0*T56+4.0D0*T57+4.0D0*T58+4.0D0*T59+4.0D0*T60+4.0D0*T61+T13
     &-0.66666666666666666667D0*T14+T65-2.3333333333333333333D0*T16+2.33
     &33333333333333333D0*T42+T66+0.66666666666666666667D0*T44-1.0D0*T45
     &-T72-0.5D0*T7-T73+T182)*T2+(-2.0D0*T54-2.0D0*T55-2.0D0*T56-2.0D0*T
     &57+T174-2.0D0*T59-2.0D0*T60+T214+T84+T85-T86+T87+2.333333333333333
     &3333D0*T6+2.3333333333333333333D0*T7+2.3333333333333333333D0*T8+2.
     &3333333333333333333D0*T9)*CA*CF
      T351=(0.055555555555555555556D0*T2+0.011111111111111111111D0*CA*CF
     &)*im*T108+(0.0061728395061728395062D0*CF*Nf+(-0.298148148148148148
     &15D0*T3-0.29814814814814814815D0*T4+0.3D0)*T2+(0.15555555555555555
     &556D0*T3+0.3962962962962962963D0*T4-0.54891975308641975309D0)*CA*C
     &F)*T63+((-T80-T81-0.20987654320987654321D0)*CF*Nf+(T281+0.83333333
     &333333333334D0*T7-0.16666666666666666667D0*T8+T113-T183-0.83333333
     &333333333334D0*T4-2.1944444444444444445D0)*T2+(T109-0.444444444444
     &44444445D0*T7+0.22222222222222222222D0*T8+0.65740740740740740741D0
     &*T3+0.65740740740740740741D0*T4+0.58950617283950617284D0)*CA*CF)*i
     &m*T64+((T283+T67-T117+T68-0.95061728395061728395D0*T3-0.2839506172
     &8395061728D0*T4-1.0555555555555555556D0)*CF*Nf+(T13-1.388888888888
     &8888889D0*T14+T286+T16+T42-1.3888888888888888889D0*T43-T121+T45+5.
     &4166666666666666667D0*T6-0.58333333333333333334D0*T7+5.41666666666
     &66666667D0*T8-0.58333333333333333334D0*T9+1.9444444444444444444D0*
     &T3-2.3888888888888888889D0*T4+4.5833333333333333333D0)*T2+(T278-2.
     &2222222222222222222D0*T14+T65-0.22222222222222222222D0*T16-0.22222
     &222222222222222D0*T42-T66-2.2222222222222222222D0*T44+T123+2.84259
     &25925925925926D0*T6-3.3425925925925925926D0*T7+2.84259259259259259
     &26D0*T8-3.3425925925925925926D0*T9+1.4135802469135802469D0*T3-1.25
     &30864197530864198D0*T4+3.2222222222222222222D0)*CA*CF)*T11
      T352=(T292+0.44444444444444444445D0*T14+T293+T116-T124+T246+T125-T
     &126+T294+T67-1.4074074074074074074D0*T8+T68-T69-T70)*CF*Nf+(T298-T
     &184+T299+T285+T300+4.6666666666666666667D0*T76+T302+T120+T131-0.66
     &666666666666666667D0*T83-4.6666666666666666667D0*T77+4.66666666666
     &66666667D0*T59-T132+4.6666666666666666667D0*T78+T264-T133+10.0D0*T
     &13-9.0D0*T14+7.0D0*T15-6.0D0*T16+14.0D0*T42-5.0D0*T43+11.0D0*T44+T
     &71+T72-6.8333333333333333333D0*T7+T73-6.8333333333333333333D0*T9+T
     &38-T39)*T2+(T310+T195-1.3333333333333333333D0*T75+T118-T119+1.3333
     &333333333333333D0*T77-1.3333333333333333333D0*T59+1.33333333333333
     &33333D0*T78+2.6666666666666666667D0*T79-T136-0.6111111111111111111
     &1D0*T13-5.8333333333333333333D0*T14-10.833333333333333333D0*T15+8.
     &3888888888888888889D0*T16-0.61111111111111111111D0*T42-5.833333333
     &3333333333D0*T43-10.833333333333333333D0*T44+8.3888888888888888889
     &D0*T45-3.537037037037037037D0*T6+3.537037037037037037D0*T7-3.53703
     &7037037037037D0*T8+3.537037037037037037D0*T9+T80+T81)*CA*CF
      T353=T351+T352*im*pi+(T311+T312+0.44444444444444444445D0*T55+T313+
     &T314+T315-T138-T139+0.44444444444444444445D0*T59-T140+T141+T142+T8
     &4+T316+T317-T85+T86-0.074074074074074074074D0*T43-0.07407407407407
     &4074074D0*T44+T87-2.1111111111111111111D0*T6-2.1111111111111111111
     &D0*T7-2.1111111111111111111D0*T8-2.1111111111111111111D0*T9-2.4814
     &814814814814815D0*T3-2.4814814814814814815D0*T4)*CF*Nf+(T325+T326+
     &T327+T328+T329+T134+T330+T331+T332+T309+T333+T334+T144+T145+6.0D0*
     &T98+1.3333333333333333333D0*T99+1.3333333333333333333D0*T100+6.0D0
     &*T101+T146+T147+T148+6.0D0*T105+T150+T151-5.0D0*T54+5.0D0*T74+5.0D
     &0*T75-3.0D0*T55-7.0D0*T56+T76+T82-5.0D0*T57-9.0D0*T58+5.0D0*T83+5.
     &0D0*T77+T59-11.0D0*T60+T78+T79-1.0D0*T61-1.0D0*T13+T14+T15-4.33333
     &33333333333333D0*T16-9.6666666666666666667D0*T42+T232-T220-13.0D0*
     &T45+7.5D0*T6+9.1666666666666666667D0*T7+9.1666666666666666667D0*T8
     &+10.833333333333333333D0*T9-5.8333333333333333333D0*T3-5.833333333
     &3333333333D0*T4)*T2
      T354=T344+T345+T137+T346-T128-4.0D0*T92-4.0D0*T93+T306-2.666666666
     &6666666667D0*T95-2.6666666666666666667D0*T96-4.0D0*T97-4.0D0*T98-2
     &.6666666666666666667D0*T99-2.6666666666666666667D0*T100-4.0D0*T101
     &-T170-4.0D0*T103-4.0D0*T104-2.6666666666666666667D0*T105-1.3333333
     &333333333333D0*T149+T171+T172+5.5D0*T54+4.8888888888888888889D0*T7
     &4+4.8888888888888888889D0*T75-3.3888888888888888889D0*T55+8.388888
     &8888888888889D0*T56-4.8888888888888888889D0*T76-4.8888888888888888
     &889D0*T82+3.5D0*T57+5.5D0*T58+4.8888888888888888889D0*T83+4.888888
     &8888888888889D0*T77-3.3888888888888888889D0*T59+8.3888888888888888
     &889D0*T60-4.8888888888888888889D0*T78-4.8888888888888888889D0*T79+
     &3.5D0*T61+10.5D0*T13+3.2962962962962962963D0*T14+3.296296296296296
     &2963D0*T15+3.1666666666666666667D0*T16+5.1666666666666666667D0*T42
     &-2.037037037037037037D0*T43-2.037037037037037037D0*T44-2.166666666
     &6666666667D0*T45+11.111111111111111111D0*T6+6.4444444444444444445D
     &0*T7+6.4444444444444444445D0*T8+T179+17.259259259259259259D0*T3
      T355=(T115+T116+T216-3.3333333333333333333D0*T7+T117-0.96296296296
     &296296297D0*T9+4.8765432098765432099D0*T3-0.36419753086419753086D0
     &*T4+2.0987654320987654321D0)*CF*Nf+(2.0D0*T74+T196-2.3333333333333
     &333333D0*T55+T118+T119+T120+T217-1.7777777777777777778D0*T14+T218-
     &7.3333333333333333333D0*T16+1.6666666666666666667D0*T42+4.05555555
     &55555555556D0*T43+T121-2.3333333333333333333D0*T45-23.333333333333
     &333333D0*T6-4.1111111111111111111D0*T7-4.7777777777777777778D0*T8+
     &T122-3.5555555555555555556D0*T3+3.5D0*T4-18.055555555555555556D0)*
     &T2+(-0.16666666666666666667D0*T74+4.1666666666666666667D0*T75+4.16
     &66666666666666667D0*T55+2.3333333333333333333D0*T56+0.333333333333
     &33333333D0*T76+3.8333333333333333333D0*T82-T180+2.8055555555555555
     &556D0*T14+2.3333333333333333333D0*T15+10.916666666666666667D0*T16+
     &5.7222222222222222222D0*T42+1.8333333333333333333D0*T43+T277-T123-
     &20.037037037037037037D0*T6+10.310185185185185185D0*T7-13.717592592
     &592592593D0*T8+16.037037037037037037D0*T9-2.6219135802469135803D0*
     &T3+0.96141975308641975309D0*T4+9.975308641975308642D0)*CA*CF
      T356=(-1.3333333333333333333D0*T74-T197+T221+1.3333333333333333333
     &D0*T14+T222+T178+T124-5.7777777777777777778D0*T43-T125+T126+T224+1
     &1.481481481481481481D0*T7+1.962962962962962963D0*T8+1.037037037037
     &037037D0*T9-1.8148148148148148148D0*T3+10.037037037037037037D0*T4-
     &2.4814814814814814815D0)*CF*Nf+(T200+T304+T128-T305+T135-4.0D0*T96
     &+T192+4.0D0*T129-T193+T226+T227+T228-8.6666666666666666667D0*T55+T
     &229+T289+T230+T231-T131+8.6666666666666666667D0*T83-3.333333333333
     &3333333D0*T77-28.666666666666666667D0*T59+T132+3.33333333333333333
     &33D0*T78-8.6666666666666666667D0*T79+T133-13.666666666666666667D0*
     &T13+26.666666666666666667D0*T14+10.666666666666666667D0*T15+40.333
     &333333333333333D0*T16-16.333333333333333333D0*T42+1.33333333333333
     &33333D0*T43-6.6666666666666666667D0*T44-17.333333333333333333D0*T4
     &5+15.5D0*T6+46.166666666666666667D0*T7+4.8333333333333333333D0*T8+
     &42.833333333333333333D0*T9+9.6666666666666666667D0*T3+39.333333333
     &333333333D0*T4+0.83333333333333333334D0)*T2+(T89+2.0D0*T127+4.0D0*
     &T90-6.0D0*T91-T203-T134+2.0D0*T94-T135-3.0D0*T96+3.0D0*T97+4.0D0*T
     &143-6.0D0*T129-1.0D0*T130+T234+19.5D0*T74-15.166666666666666667D0*
     &T75+T189-9.3333333333333333334D0*T56+22.5D0*T76-55.5D0*T82+T235+T1
     &52+T236-10.833333333333333333D0*T77+6.3333333333333333333D0*T59+T2
     &37-13.333333333333333333D0*T78-4.6666666666666666667D0*T79+T136-15
     &.555555555555555556D0*T13+50.888888888888888889D0*T14+19.333333333
     &333333333D0*T15+1.2777777777777777778D0*T16-20.972222222222222222D
     &0*T42+74.25D0*T43-65.416666666666666667D0*T44-16.13888888888888888
     &9D0*T45+26.175925925925925926D0*T6-84.203703703703703704D0*T7+88.6
     &20370370370370371D0*T8+1.2407407407407407407D0*T9-11.9074074074074
     &07407D0*T3-87.981481481481481482D0*T4+13.259259259259259259D0)*CA*
     &CF
      T357=((0.21111111111111111111D0*T3+0.36666666666666666667D0)*T2+(-
     &0.28055555555555555556D0*T3-0.63055555555555555556D0)*CA*CF)*im*T1
     &08+((-0.27777777777777777778D0*T3-0.56172839506172839506D0)*CF*Nf+
     &(-0.58888888888888888889D0*T6+0.17777777777777777778D0*T7-1.690740
     &7407407407407D0*T3+0.6537037037037037037D0*T4-3.535185185185185185
     &2D0)*T2+(-0.49444444444444444445D0*T6-0.086111111111111111111D0*T7
     &-1.4333333333333333333D0*T3-0.56851851851851851852D0*T4-0.70246913
     &58024691358D0)*CA*CF)*T63+((T109+1.4074074074074074074D0*T3+T81+2.
     &3024691358024691358D0)*CF*Nf+(T110+T204+T111+T215-T112+1.5D0*T8-T1
     &13+11.166666666666666667D0*T3+T114+11.861111111111111111D0)*T2+(-0
     &.16666666666666666667D0*T13-0.83333333333333333334D0*T15+T287+5.86
     &11111111111111111D0*T6+2.7777777777777777778D0*T7-3.30555555555555
     &55556D0*T8-T49+7.1990740740740740741D0*T3+4.787037037037037037D0*T
     &4-7.2700617283950617284D0)*CA*CF)*im*T64+T355*T11+T356*im*pi+(T137
     &-2.6666666666666666667D0*T93-2.6666666666666666667D0*T97+T238+T239
     &+1.3333333333333333333D0*T55+T240+T241+T242+T243+T138+T139-5.77777
     &77777777777778D0*T59+T140-T141-T142+T176+T244+T245+15.444444444444
     &444444D0*T16-1.2222222222222222222D0*T42-1.037037037037037037D0*T4
     &3-1.037037037037037037D0*T44-5.3333333333333333333D0*T45+9.5555555
     &555555555556D0*T6+7.0740740740740740741D0*T7+7.0740740740740740741
     &D0*T8+17.333333333333333333D0*T9+T175-1.3703703703703703704D0*T4)*
     &CF*Nf
      T358=T205+T206+3.0D0*T153+4.0D0*T207+4.0D0*T208-2.0D0*T154+24.0D0*
     &T155+8.0D0*T156+8.0D0*T157+2.0D0*T158+2.0D0*T209+2.0D0*T210-2.0D0*
     &T159+7.0D0*T160+4.0D0*T161+4.0D0*T162+7.0D0*T163+4.0D0*T211+4.0D0*
     &T212-2.0D0*T164+T165+T267+T268+14.166666666666666667D0*T127+T269-5
     &.5D0*T91-5.5D0*T92+29.166666666666666667D0*T93+T270+4.166666666666
     &6666667D0*T95+4.1666666666666666667D0*T96+26.666666666666666667D0*
     &T97+71.0D0*T143+T271+T272+T273+T274+T275+T276+22.5D0*T98+11.666666
     &666666666667D0*T99+11.666666666666666667D0*T100+18.0D0*T101+T170+1
     &2.0D0*T103+12.0D0*T104-1.3333333333333333333D0*T105+3.333333333333
     &3333333D0*T149-T171-T172+T173-23.555555555555555556D0*T74-23.55555
     &5555555555556D0*T75+37.111111111111111111D0*T55-41.888888888888888
     &889D0*T56-12.777777777777777778D0*T76-12.777777777777777778D0*T82+
     &17.833333333333333333D0*T57+T174-22.972222222222222222D0*T83-22.97
     &2222222222222222D0*T77+53.722222222222222222D0*T59+82.111111111111
     &111111D0*T60+19.138888888888888889D0*T78
      T359=(T358+19.138888888888888889D0*T79+3.0D0*T61-34.33333333333333
     &3333D0*T13+6.5092592592592592593D0*T14+6.5092592592592592593D0*T15
     &-67.916666666666666667D0*T16-105.88888888888888889D0*T42-7.4907407
     &407407407407D0*T43-7.4907407407407407407D0*T44+23.0833333333333333
     &33D0*T45+10.083333333333333333D0*T6+1.8425925925925925926D0*T7+1.8
     &425925925925925926D0*T8-87.361111111111111111D0*T9+1.4722222222222
     &222222D0*T3+14.731481481481481481D0*T4)*CA*CF
      T360=T357+(4.0D0*T199-4.0D0*T153+T336-16.0D0*T155+T337+T338-28.0D0
     &*T158+T339-4.0D0*T160+T341+T342-20.0D0*T163+T343+T213+T247+T248+T2
     &49+T250+T251+T252-T190+T253+T254+T255-8.0D0*T97+T256+T257+T258+T25
     &9-T144-T145+T260-14.0D0*T98-17.333333333333333333D0*T99-17.3333333
     &33333333333D0*T100-46.0D0*T101-T146-T147-T148+2.0D0*T105+T262-T150
     &-T151+T173-21.666666666666666667D0*T74-21.666666666666666667D0*T75
     &+T55+T335+16.666666666666666667D0*T76+16.666666666666666667D0*T82+
     &45.0D0*T57+T152-13.333333333333333333D0*T83-13.333333333333333333D
     &0*T77-13.0D0*T59-19.0D0*T60-26.666666666666666667D0*T78-26.6666666
     &66666666667D0*T79-44.0D0*T61-T303+0.5D0*T14+0.5D0*T15+42.666666666
     &666666667D0*T16+37.333333333333333333D0*T42+38.166666666666666667D
     &0*T43+38.166666666666666667D0*T44+69.0D0*T45-40.5D0*T6-32.33333333
     &3333333333D0*T7-32.333333333333333333D0*T8+7.5D0*T9-4.5D0*T3-3.666
     &6666666666666667D0*T4)*T2+T359
      T361=((-0.84444444444444444445D0*T3-0.84444444444444444445D0)*T2+(
     &1.2388888888888888889D0*T3+1.2972222222222222222D0)*CA*CF)*im*T108
     &+((T175+1.1111111111111111111D0)*CF*Nf+(2.3555555555555555556D0*T6
     &-0.71111111111111111111D0*T7+6.8222222222222222222D0*T3-0.71111111
     &111111111111D0*T4+6.3175925925925925926D0)*T2+(2.71111111111111111
     &11D0*T6+0.34444444444444444445D0*T7+2.45D0*T3+0.344444444444444444
     &45D0*T4-0.23472222222222222222D0)*CA*CF)*T63+((0.88888888888888888
     &889D0*T6-T187-4.4074074074074074074D0)*CF*Nf+(-T176+T177-T282-6.33
     &33333333333333333D0*T6+T51-2.6666666666666666667D0*T8-24.25D0*T3-3
     &.75D0*T4-16.666666666666666667D0)*T2+(T110+4.3333333333333333333D0
     &*T15+1.3333333333333333333D0*T16-20.722222222222222222D0*T6-9.3333
     &333333333333334D0*T7+6.6666666666666666667D0*T8+1.3333333333333333
     &333D0*T9-9.3425925925925925926D0*T3-11.055555555555555556D0*T4+20.
     &763888888888888889D0)*CA*CF)*im*T64+((8.0D0*T14+T178+T109+4.592592
     &5925925925926D0*T7+T179-7.6296296296296296296D0*T3-3.5925925925925
     &925926D0*T4+1.1358024691358024691D0)*CF*Nf+(-8.0D0*T74-10.66666666
     &6666666667D0*T75+9.3333333333333333334D0*T55-5.3333333333333333333
     &D0*T56-T291+2.6666666666666666667D0*T57+T180+23.666666666666666667
     &D0*T14+T181+11.333333333333333333D0*T16-5.3333333333333333333D0*T4
     &2+T223+T198+43.833333333333333333D0*T6-2.8611111111111111111D0*T7+
     &1.4722222222222222222D0*T8+T182+T183-26.194444444444444444D0*T4+31
     &.777777777777777778D0)*T2+(T184-20.666666666666666667D0*T75-16.666
     &666666666666667D0*T55-11.333333333333333333D0*T56-T119-15.33333333
     &3333333333D0*T82+T185+3.1666666666666666667D0*T14+T186-28.27777777
     &7777777778D0*T16-12.0D0*T42-2.3333333333333333333D0*T43-15.3333333
     &33333333333D0*T44+43.388888888888888889D0*T6-13.898148148148148148
     &D0*T7+35.916666666666666667D0*T8-16.972222222222222222D0*T9-27.5D0
     &*T3+6.8194444444444444445D0*T4-41.839506172839506173D0)*CA*CF)*T11
      T362=(5.3333333333333333333D0*T74+10.666666666666666667D0*T76-T176
     &-15.111111111111111111D0*T14+T280-2.6666666666666666667D0*T16+10.6
     &66666666666666667D0*T43-8.6666666666666666667D0*T6-25.111111111111
     &111111D0*T7-0.44444444444444444445D0*T8-1.5555555555555555556D0*T9
     &+T187-1.962962962962962963D0*T4-1.3703703703703703704D0)*CF*Nf+(-1
     &6.0D0*T88+16.0D0*T127-T202+48.0D0*T93-T191+16.0D0*T96+48.0D0*T97-1
     &6.0D0*T129+16.0D0*T130+T188-28.0D0*T74-16.0D0*T75-T189-16.0D0*T56-
     &12.0D0*T76+8.0D0*T82+44.0D0*T57-16.0D0*T83+16.0D0*T77+48.0D0*T59-1
     &6.0D0*T78+16.0D0*T79+22.333333333333333333D0*T13-74.33333333333333
     &3333D0*T14-32.333333333333333333D0*T15-69.666666666666666667D0*T16
     &+5.6666666666666666667D0*T42+10.333333333333333333D0*T43-5.6666666
     &666666666667D0*T44+9.6666666666666666667D0*T45-T30-85.5D0*T7+13.5D
     &0*T8+T122-63.333333333333333333D0*T3-40.166666666666666667D0*T4-17
     &.0D0)*T2+(-T201-T297-16.0D0*T90+24.0D0*T91+88.0D0*T92+T190-8.0D0*T
     &94+T191+18.0D0*T96+T192+T347+24.0D0*T129+T193+T194-58.333333333333
     &333333D0*T74+65.0D0*T75-76.0D0*T55+32.0D0*T56-103.6666666666666666
     &7D0*T76+139.0D0*T82+23.0D0*T57-6.0D0*T58+20.0D0*T83+22.0D0*T77+T26
     &3-16.0D0*T60+24.0D0*T78+4.0D0*T79+33.333333333333333333D0*T13-55.3
     &88888888888888889D0*T14-58.5D0*T15-13.833333333333333333D0*T16+45.
     &666666666666666667D0*T42-144.83333333333333333D0*T43+56.0D0*T44+5.
     &5D0*T45-73.777777777777777778D0*T6+217.16666666666666667D0*T7-168.
     &66666666666666667D0*T8+12.277777777777777778D0*T9+88.8703703703703
     &70371D0*T3+68.490740740740740741D0*T4+22.731481481481481482D0)*CA*
     &CF
      T363=-16.0D0*T199+16.0D0*T153-16.0D0*T154+64.0D0*T155+64.0D0*T156+
     &64.0D0*T157+112.0D0*T158-16.0D0*T159+16.0D0*T160+32.0D0*T161+32.0D
     &0*T162+80.0D0*T163-16.0D0*T164-16.0D0*T165+T200+T201-24.0D0*T127+3
     &2.0D0*T90+T202+T203+20.0D0*T93+28.0D0*T94+12.0D0*T95+T308+60.0D0*T
     &143+68.0D0*T129+68.0D0*T130+112.0D0*T166-16.0D0*T169+16.0D0*T98+32
     &.0D0*T99+32.0D0*T100+80.0D0*T101-16.0D0*T105-16.0D0*T149+12.0D0*T5
     &4+34.333333333333333333D0*T74+34.333333333333333333D0*T75-34.0D0*T
     &55-25.0D0*T56-51.333333333333333333D0*T76-51.333333333333333333D0*
     &T82-97.0D0*T57+5.0D0*T58+10.666666666666666667D0*T83+10.6666666666
     &66666667D0*T77+21.0D0*T59+24.0D0*T60+18.333333333333333333D0*T78+1
     &8.333333333333333333D0*T79+28.0D0*T61+11.0D0*T13+T204-1.0D0*T15-83
     &.0D0*T16-60.333333333333333333D0*T42-43.333333333333333333D0*T43-4
     &3.333333333333333333D0*T44-39.0D0*T45+104.0D0*T6+37.66666666666666
     &6667D0*T7+37.666666666666666667D0*T8-5.0D0*T9
      T364=-4.0D0*T205-4.0D0*T206-12.0D0*T153-16.0D0*T207-16.0D0*T208+T3
     &18-120.0D0*T155-T319-T320-8.0D0*T158-8.0D0*T209-8.0D0*T210+T321-34
     &.0D0*T160-T322-T323-28.0D0*T163-16.0D0*T211-16.0D0*T212+T324-T213+
     &T296+8.0D0*T89-50.333333333333333333D0*T127-14.0D0*T90+51.0D0*T91+
     &51.0D0*T92-83.666666666666666667D0*T93-19.0D0*T94+13.0D0*T95+13.0D
     &0*T96-90.666666666666666667D0*T97-170.0D0*T143-31.0D0*T129-31.0D0*
     &T130-8.0D0*T166-6.0D0*T167-6.0D0*T168+14.0D0*T169-40.0D0*T98-18.0D
     &0*T99-18.0D0*T100-28.0D0*T101-16.0D0*T103-16.0D0*T104+T261-4.0D0*T
     &149+11.0D0*T54+44.333333333333333333D0*T74+44.333333333333333333D0
     &*T75-25.722222222222222222D0*T55+103.0D0*T56+29.833333333333333333
     &D0*T76+29.833333333333333333D0*T82-42.666666666666666667D0*T57-4.5
     &D0*T58+41.166666666666666667D0*T83+41.166666666666666667D0*T77-103
     &.66666666666666667D0*T59-63.5D0*T60-7.5D0*T78-7.5D0*T79+T214
      T365=(T361+T362*im*pi+(5.3333333333333333333D0*T127+10.66666666666
     &6666667D0*T93+10.666666666666666667D0*T97+T195-T196-15.11111111111
     &1111111D0*T55-T118+T197+2.6666666666666666667D0*T82+10.66666666666
     &6666667D0*T57+10.666666666666666667D0*T59-T176-8.66666666666666666
     &67D0*T14-8.6666666666666666667D0*T15-32.222222222222222222D0*T16+T
     &124+1.5555555555555555556D0*T43+1.5555555555555555556D0*T44+T198-9
     &.4444444444444444444D0*T6-6.4814814814814814815D0*T7-6.48148148148
     &14814815D0*T8-9.5555555555555555556D0*T9+5.2222222222222222222D0*T
     &3+3.8518518518518518519D0*T4)*CF*Nf+(T363+26.5D0*T3+9.5D0*T4)*T2+(
     &T364+37.833333333333333333D0*T13-50.611111111111111111D0*T14-50.61
     &1111111111111111D0*T15+158.0D0*T16+168.94444444444444444D0*T42-8.2
     &777777777777777778D0*T43-8.2777777777777777778D0*T44-10.6666666666
     &66666667D0*T45-122.63888888888888889D0*T6-26.435185185185185185D0*
     &T7-26.435185185185185185D0*T8+48.166666666666666667D0*T9-54.722222
     &222222222222D0*T3-31.990740740740740741D0*T4)*CA*CF)*T25
      T366=(-10.0D0*T14-2.2222222222222222222D0*T16+T216+0.5185185185185
     &1851852D0*T7-0.88888888888888888889D0*T9+4.4320987654320987654D0*T
     &3+2.7407407407407407407D0*T4-2.012345679012345679D0)*CF*Nf+(10.0D0
     &*T74+13.333333333333333333D0*T75-11.666666666666666667D0*T55+T288+
     &6.6666666666666666667D0*T76-3.3333333333333333333D0*T57+T217-36.44
     &4444444444444444D0*T14+T218-4.6666666666666666667D0*T16+2.66666666
     &66666666667D0*T42+2.6666666666666666667D0*T43-T123-36.0D0*T6+27.88
     &8888888888888889D0*T7+T225+T219-2.4444444444444444445D0*T3+16.5D0*
     &T4-18.055555555555555556D0)*T2+(-0.83333333333333333334D0*T74+28.8
     &33333333333333333D0*T75+20.833333333333333333D0*T55+15.66666666666
     &6666667D0*T56+1.6666666666666666667D0*T76+19.166666666666666667D0*
     &T82-T180-5.8611111111111111111D0*T14-22.333333333333333333D0*T15+2
     &4.694444444444444444D0*T16+6.5D0*T42+1.1666666666666666667D0*T43+T
     &220-39.703703703703703704D0*T6+9.3194444444444444444D0*T7-19.95833
     &3333333333333D0*T8+7.1111111111111111111D0*T9+51.79475308641975308
     &7D0*T3-4.3888888888888888889D0*T4+27.475308641975308642D0)*CA*CF
      T367=(-6.6666666666666666667D0*T74-13.333333333333333333D0*T76+T22
     &1+24.444444444444444444D0*T14+T222+T178+T223+T224+11.1481481481481
     &48148D0*T7+T225+0.44444444444444444445D0*T9+0.40740740740740740741
     &D0*T3-1.8518518518518518519D0*T4+3.8518518518518518519D0)*CF*Nf+(2
     &0.0D0*T88-20.0D0*T127+20.0D0*T91-60.0D0*T93+T233-20.0D0*T96-60.0D0
     &*T97+20.0D0*T129-20.0D0*T130+T226+T227+T228+47.333333333333333333D
     &0*T55+T229+7.3333333333333333333D0*T76+T230+T231+8.0D0*T83-8.0D0*T
     &77-24.0D0*T59+8.0D0*T78-8.0D0*T79-23.666666666666666667D0*T13+101.
     &66666666666666667D0*T14+22.666666666666666667D0*T15+15.33333333333
     &3333333D0*T16+T86+T232+5.3333333333333333333D0*T44+T87-4.166666666
     &6666666667D0*T6+50.166666666666666667D0*T7-29.0D0*T8-13.6666666666
     &66666667D0*T9+97.5D0*T3+14.0D0*T4+16.166666666666666667D0)*T2+(5.0
     &D0*T89+10.0D0*T127+20.0D0*T90-30.0D0*T91-128.0D0*T92-30.0D0*T93+10
     &.0D0*T94-T233-27.0D0*T96+15.0D0*T97+20.0D0*T143-30.0D0*T129-5.0D0*
     &T130+T234+68.833333333333333333D0*T74-79.166666666666666667D0*T75+
     &92.0D0*T55-41.333333333333333333D0*T56+145.16666666666666667D0*T76
     &-99.5D0*T82+T235+T152+T236-12.5D0*T77+5.0D0*T59+T237-12.0D0*T78-2.
     &0D0*T79-27.555555555555555556D0*T13-10.555555555555555556D0*T14+91
     &.833333333333333333D0*T15-3.7222222222222222222D0*T16-24.083333333
     &333333333D0*T42+74.416666666666666667D0*T43-13.0D0*T44+77.84259259
     &2592592593D0*T6-154.62037037037037037D0*T7+83.0D0*T8-9.88888888888
     &88888889D0*T9-136.99074074074074074D0*T3-15.814814814814814815D0*T
     &4-35.990740740740740741D0)*CA*CF
      T368=((1.0555555555555555556D0*T3+0.42222222222222222222D0)*T2+(-1
     &.6361111111111111111D0*T3-0.67777777777777777778D0)*CA*CF)*im*T108
     &+((-1.3888888888888888889D0*T3-0.55555555555555555555D0)*CF*Nf+(-2
     &.9444444444444444445D0*T6+0.88888888888888888889D0*T7-7.3796296296
     &296296296D0*T3+0.35555555555555555556D0*T4-2.9018518518518518519D0
     &)*T2+(-3.9388888888888888889D0*T6-0.43055555555555555556D0*T7-0.85
     &555555555555555556D0*T3-0.17222222222222222222D0*T4+0.488888888888
     &88888889D0)*CA*CF)*T63+((-1.1111111111111111111D0*T6+5.25925925925
     &92592593D0*T3+2.1481481481481481482D0)*CF*Nf+(3.333333333333333333
     &3D0*T13-5.0D0*T14+3.3333333333333333333D0*T15+T215+4.0D0*T7+1.3333
     &333333333333333D0*T8+26.166666666666666667D0*T3+2.5D0*T4+7.9166666
     &666666666667D0)*T2+(-0.83333333333333333334D0*T13-6.16666666666666
     &66667D0*T15-1.6666666666666666667D0*T16+24.75D0*T6+12.111111111111
     &111111D0*T7-3.5833333333333333333D0*T8-T49-4.125D0*T3+5.7777777777
     &777777778D0*T4-11.722222222222222222D0)*CA*CF)*im*T64+T366*T11+T36
     &7*im*pi+(-6.6666666666666666667D0*T127-13.333333333333333333D0*T93
     &-13.333333333333333333D0*T97+T238+T239+24.444444444444444444D0*T55
     &+T240+T241+T242+T243-5.3333333333333333333D0*T59+T244+T245+16.4444
     &44444444444444D0*T16+0.11111111111111111111D0*T42-T246-T125+T50+T2
     &84+1.8518518518518518519D0*T8-3.8518518518518518519D0*T3)*CF*Nf
      T369=5.0D0*T205+5.0D0*T206+15.0D0*T153+20.0D0*T207+20.0D0*T208-10.
     &0D0*T154+168.0D0*T155+40.0D0*T156+40.0D0*T157+10.0D0*T158+10.0D0*T
     &209+10.0D0*T210-10.0D0*T159+47.0D0*T160+20.0D0*T161+20.0D0*T162+35
     &.0D0*T163+20.0D0*T211+20.0D0*T212-10.0D0*T164+5.0D0*T165+T267+T268
     &+63.5D0*T127+T269-69.5D0*T91-69.5D0*T92+95.833333333333333333D0*T9
     &3+T270-27.833333333333333333D0*T95-27.833333333333333333D0*T96+117
     &.33333333333333333D0*T97+115.0D0*T143+T271+T272+T273+T274+T275+T27
     &6+21.5D0*T98+9.0D0*T99+9.0D0*T100+14.0D0*T101+8.0D0*T103+8.0D0*T10
     &4-4.0D0*T105+2.0D0*T149-10.0D0*T54-37.555555555555555556D0*T74-37.
     &555555555555555556D0*T75-38.333333333333333333D0*T55-103.888888888
     &88888889D0*T56-2.2777777777777777778D0*T76-2.2777777777777777778D0
     &*T82+23.333333333333333333D0*T57+T152-21.083333333333333333D0*T83-
     &21.083333333333333333D0*T77+53.333333333333333333D0*T59+13.0D0*T60
     &-11.333333333333333333D0*T13+66.509259259259259259D0*T14+66.509259
     &259259259259D0*T15
      T370=(T369-85.666666666666666667D0*T16-75.555555555555555556D0*T42
     &+9.8888888888888888889D0*T43+T277+158.91666666666666667D0*T6+15.81
     &4814814814814815D0*T7+15.814814814814814815D0*T8+35.99074074074074
     &0741D0*T3)*CA*CF
      T371=(T368+(20.0D0*T199-20.0D0*T153+20.0D0*T154-80.0D0*T155-80.0D0
     &*T156-80.0D0*T157-140.0D0*T158+20.0D0*T159+T340-40.0D0*T161-40.0D0
     &*T162-100.0D0*T163+20.0D0*T164+20.0D0*T165+T247+T248+T249+T250+T25
     &1+T252+8.0D0*T93+T253+T254+T255+T256+T257+T258+T259+T260-8.0D0*T98
     &-16.0D0*T99-16.0D0*T100-40.0D0*T101+T261+T262-23.66666666666666666
     &7D0*T74-23.666666666666666667D0*T75+74.0D0*T55+28.0D0*T56+46.66666
     &6666666666667D0*T76+46.666666666666666667D0*T82+50.0D0*T57-T152-2.
     &3333333333333333333D0*T83-2.3333333333333333333D0*T77+T263-6.0D0*T
     &60-0.66666666666666666667D0*T78-T264+T265-0.16666666666666666667D0
     &*T14-0.16666666666666666667D0*T15+49.0D0*T16+43.666666666666666667
     &D0*T42+13.666666666666666667D0*T43+13.666666666666666667D0*T44-114
     &.0D0*T6-T266-14.0D0*T8-16.166666666666666667D0*T3)*T2+T370)*x
      T372=((9.3333333333333333334D0*T2-14.0D0*CA*CF)*T10+((-14.0D0*T3-3
     &7.333333333333333333D0)*T2+(31.5D0*T3+99.5D0)*CA*CF)*T18+((56.0D0*
     &T3+56.0D0)*T2+(-171.0D0*T3-193.5D0)*CA*CF)*T25+((-70.0D0*T3-28.0D0
     &)*T2+(247.5D0*T3+108.0D0)*CA*CF)*x+28.0D0*T3*T2-108.0D0*T3*CA*CF)*
     &zeta5+(T349+(-14.0D0*T3*T2-3.6666666666666666667D0*T3*CA*CF)*T11+(
     &8.0D0*T3*CF*Nf+(T34+T36+25.333333333333333333D0*T3)*T2+(T35-T36-36
     &.666666666666666667D0*T3)*CA*CF)*im*pi+(-2.6666666666666666667D0*T
     &6+T112+12.444444444444444444D0*T3)*CF*Nf+(T185-48.0D0*T14+T186+16.
     &0D0*T16+T37+T266+55.0D0*T3)*T2+(-T41+34.0D0*T14+44.0D0*T15-18.0D0*
     &T16+62.0D0*T6+8.6666666666666666667D0*T7-137.5D0*T3)*CA*CF)*zeta3+
     &T350*T62+(T353+(T354+17.259259259259259259D0*T4)*CA*CF)*T10+T360*T
     &18+T365+T371+(-0.42222222222222222222D0*T3*T2+0.677777777777777777
     &78D0*T3*CA*CF)*im*T108+(0.55555555555555555555D0*T3*CF*Nf+(1.17777
     &77777777777778D0*T6-0.35555555555555555556D0*T7+2.5462962962962962
     &963D0*T3)*T2+(1.7222222222222222222D0*T6+0.17222222222222222222D0*
     &T7-0.31666666666666666667D0*T3)*CA*CF)*T63
      T373=T372+((0.44444444444444444445D0*T6-2.1481481481481481482D0*T3
     &)*CF*Nf+(-T278+T279-T280+T281-2.5D0*T7-10.416666666666666667D0*T3)
     &*T2+(0.33333333333333333333D0*T13+T282+T85-9.6666666666666666667D0
     &*T6-5.1111111111111111111D0*T7+5.9444444444444444445D0*T3)*CA*CF)*
     &im*T64+((T177+0.88888888888888888889D0*T16+T283-T284-0.72839506172
     &839506173D0*T3)*CF*Nf+(-4.0D0*T74-5.3333333333333333333D0*T75+T285
     &-2.6666666666666666667D0*T56-T197+1.3333333333333333333D0*T57+T13+
     &15.944444444444444444D0*T14+T286+T287+10.75D0*T6-18.16666666666666
     &6667D0*T7+1.5555555555555555556D0*T3)*T2+(0.33333333333333333333D0
     &*T74-12.333333333333333333D0*T75-8.3333333333333333334D0*T55-T288+
     &T289-7.6666666666666666667D0*T82+T278+2.1111111111111111111D0*T14+
     &12.666666666666666667D0*T15-7.1111111111111111111D0*T16+13.1759259
     &25925925926D0*T6-2.7222222222222222222D0*T7-23.086419753086419753D
     &0*T3)*CA*CF)*T11+((T290+T291+T292-11.111111111111111111D0*T14+T293
     &+T116+T294+1.4074074074074074074D0*T7-T295)*CF*Nf+(-T296+T297-8.0D
     &0*T91+T190-T307+8.0D0*T96+24.0D0*T97-8.0D0*T129+8.0D0*T130+T298-T1
     &84+T299-23.333333333333333333D0*T55+T300+T301+T302+T120+T303-46.0D
     &0*T14+T40+13.0D0*T16+7.8333333333333333333D0*T6-T51+13.33333333333
     &3333333D0*T8-43.5D0*T3)*T2+(-2.0D0*T89+T304-8.0D0*T90+12.0D0*T91+5
     &6.0D0*T92+T305+T306+T307+T308-T309-8.0D0*T143+12.0D0*T129+2.0D0*T1
     &30+T310-27.333333333333333333D0*T74+30.666666666666666667D0*T75-36
     &.0D0*T55+17.333333333333333333D0*T56-62.666666666666666667D0*T76+1
     &6.0D0*T82+8.3888888888888888889D0*T13+22.888888888888888889D0*T14-
     &43.833333333333333333D0*T15+9.8888888888888888889D0*T16-27.3703703
     &7037037037D0*T6+25.703703703703703704D0*T7-T32+59.8055555555555555
     &56D0*T3)*CA*CF)*im*pi
      T374=T373+(2.6666666666666666667D0*T127+5.3333333333333333333D0*T9
     &3+5.3333333333333333333D0*T97+T311+T312-11.111111111111111111D0*T5
     &5+T313+T314+T315+T110+T316+T317+2.0D0*T6)*CF*Nf+(-8.0D0*T199+8.0D0
     &*T153-T318+32.0D0*T155+T319+T320+56.0D0*T158-T321+8.0D0*T160+T322+
     &T323+40.0D0*T163-T324-8.0D0*T165+T325+T326+T327+T328+T329-10.0D0*T
     &93+T330+T331+T332+2.0D0*T97+T333+T334-3.0D0*T54+6.0D0*T74+6.0D0*T7
     &5-39.0D0*T55-T335-13.0D0*T76-13.0D0*T82-6.0D0*T13+0.33333333333333
     &333333D0*T14+0.33333333333333333333D0*T15-13.333333333333333333D0*
     &T42+43.5D0*T6)*T2+(-2.0D0*T205-2.0D0*T206-6.0D0*T153-8.0D0*T207-8.
     &0D0*T208+T336-72.0D0*T155+T337+T338-4.0D0*T158-4.0D0*T209-4.0D0*T2
     &10+T339+T340+T341+T342-14.0D0*T163-8.0D0*T211-8.0D0*T212+T343-2.0D
     &0*T165+T344+T345-26.0D0*T127+T346+28.0D0*T91+28.0D0*T92-37.3333333
     &33333333333D0*T93+T306+13.333333333333333333D0*T95+13.333333333333
     &333333D0*T96-49.333333333333333333D0*T97+T347+3.5D0*T54+11.8888888
     &88888888889D0*T74+11.888888888888888889D0*T75+32.33333333333333333
     &3D0*T55+36.388888888888888889D0*T56-9.8888888888888888889D0*T76-9.
     &8888888888888888889D0*T82-T41-25.703703703703703704D0*T14-25.70370
     &3703703703704D0*T15+8.0D0*T42-59.805555555555555556D0*T6)*CA*CF
      betaHPL_3l_nfv2=T374/(T18-1.0D0*T25)

      end function betaHPL_3l_nfv2

      function betaHPL_3l_nfv(x)
        use types
        use constants
        implicit complex(dp) (T)
        include 'nf.f'
        complex(dp) :: betaHPL_3l_nfv
        real(dp), intent(in) :: x

        real(dp) :: dabc2

        dabc2 = (nc**2-4._dp)*(nc**2-1._dp)/nc


      T2=Hr1(1)
      T3=Hr2(1,1)
      T4=3.0D0*Hr1(0)
      T5=x**3
      T6=pi*pi
      T7=Hr3(1,1,1)
      T8=Hr3(1,1,0)
      T9=Hr3(1,0,1)
      T10=Hr3(1,0,0)
      T11=Hr2(1,0)
      T12=Hr2(0,1)
      T13=Hr2(0,0)
      T14=Hr1(0)
      T15=x*x
      T16=8.0D0*T2
      T17=12.0D0*T3
      T18=6.0D0*T13
      T19=-2.0D0*T2
      T20=x**5
      T21=pi**4
      T22=0.5D0*T2
      T23=pi**3
      T24=3.0D0*T10
      T25=Hr3(0,1,1)
      T26=Hr3(0,0,0)
      T27=2.5D0*T2
      T28=Hr4(1,1,1,1)
      T29=0.5D0*Hr4(1,1,1,0)
      T30=0.5D0*Hr4(1,1,0,1)
      T31=Hr4(1,1,0,0)
      T32=Hr4(1,0,1,1)
      T33=0.5D0*Hr4(1,0,1,0)
      T34=0.5D0*Hr4(1,0,0,1)
      T35=Hr4(1,0,0,0)
      T36=Hr4(0,1,1,1)
      T37=0.5D0*Hr4(0,1,1,0)
      T38=0.5D0*Hr4(0,1,0,1)
      T39=Hr4(0,1,0,0)
      T40=Hr4(0,0,1,1)
      T41=0.5D0*Hr4(0,0,1,0)
      T42=0.5D0*Hr4(0,0,0,1)
      T43=Hr4(0,0,0,0)
      T44=2.5D0*T7
      T45=6.0D0*T10
      T46=Hr3(0,1,0)
      T47=Hr3(0,0,1)
      T48=x**4
      T49=Hr4(1,1,1,0)
      T50=Hr4(1,0,1,0)
      T51=0.083333333333333333333D0*T12
      T52=Hr4(1,0,0,1)
      T53=Hr4(0,1,1,0)
      T54=Hr4(0,0,0,1)
      T55=4.0D0*T9
      T56=2.5D0*T25
      T57=-1.0D0*T11
      T58=-3.0D0*T2
      T59=Hr5(1,1,1,0,0)
      T60=Hr5(1,1,0,0,0)
      T61=2.0D0*Hr5(1,0,1,1,0)
      T62=Hr5(1,0,1,0,0)
      T63=Hr5(1,0,0,1,1)
      T64=Hr4(1,1,0,1)
      T65=Hr4(0,1,0,1)
      T66=Hr4(0,0,1,0)
      T67=pi**5
      T68=2.0D0*T8
      T69=4.0D0*T3
      T70=2.0D0*T52
      T71=2.0D0*T9
      T72=2.0D0*T47
      T73=-0.66666666666666666667D0*T13
      T74=-1.0D0*T2
      T75=Hr5(1,1,1,1,0)
      T76=Hr5(1,1,0,1,1)
      T77=Hr5(1,1,0,1,0)
      T78=Hr5(1,1,0,0,1)
      T79=Hr5(1,0,1,1,0)
      T80=Hr5(1,0,0,0,1)
      T81=4.5D0*T7
      T82=2.5D0*T10
      T83=3.0D0*T46
      T84=2.0D0*T11
      T85=Hr6(1,1,1,1,0,0)
      T86=Hr6(1,1,1,0,0,0)
      T87=Hr6(1,1,0,1,1,0)
      T88=Hr6(1,1,0,1,0,1)
      T89=Hr6(1,1,0,1,0,0)
      T90=Hr6(1,1,0,0,1,1)
      T91=Hr6(1,1,0,0,0,0)
      T92=Hr6(1,0,1,1,0,0)
      T93=Hr6(1,0,1,0,0,0)
      T94=Hr6(1,0,0,1,1,0)
      T95=Hr6(1,0,0,1,0,1)
      T96=Hr6(1,0,0,1,0,0)
      T97=Hr6(1,0,0,0,1,1)
      T98=24.0D0*T59
      T99=Hr5(1,0,1,0,1)
      T100=4.0D0*T63
      T101=Hr5(1,0,0,0,0)
      T102=Hr5(0,1,1,0,0)
      T103=Hr5(0,1,0,0,0)
      T104=Hr5(0,0,1,1,0)
      T105=Hr5(0,0,1,0,1)
      T106=Hr5(0,0,1,0,0)
      T107=Hr5(0,0,0,1,1)
      T108=-0.28333333333333333333D0*T14
      T109=-2.0D0*T3
      T110=-0.66666666666666666667D0*T12
      T111=0.33333333333333333333D0*T13
      T112=2.5D0*T64
      T113=1.3333333333333333333D0*T10
      T114=-12.0D0*T49
      T115=-12.0D0*T31
      T116=2.0D0*T32
      T117=-8.0D0*T50
      T118=-8.0D0*T35
      T119=-4.0D0*T53
      T120=-4.0D0*T39
      T121=2.0D0*T40
      T122=2.0D0*T54
      T123=1.5D0*T7
      T124=4.25D0*T12
      T125=3.25D0*T13
      T126=-12.0D0*T59
      T127=2.0D0*T99
      T128=-6.0D0*T62
      T129=-2.0D0*T63
      T130=-8.0D0*T101
      T131=Hr5(0,1,1,1,1)
      T132=Hr5(0,1,1,1,0)
      T133=Hr5(0,1,1,0,1)
      T134=-4.0D0*T103
      T135=2.0D0*T104
      T136=2.0D0*T105
      T137=2.0D0*T106
      T138=-2.0D0*T107
      T139=3.125D0*T11
      T140=3.125D0*T12
      T141=-0.16666666666666666667D0*T7
      T142=-0.33333333333333333333D0*T25
      T143=12.0D0*T59
      T144=-2.0D0*T76
      T145=-2.0D0*T78
      T146=4.0D0*T79
      T147=3.25D0*T8
      T148=-1.0D0*T131
      T149=-1.0D0*T132
      T150=-1.0D0*T133
      T151=-1.0D0*T64
      T152=-3.25D0*T50
      T153=-3.25D0*T52
      T154=(-3.3333333333333333333D0*T5+3.3333333333333333333D0*T15)*zet
     &a5+(-2.5D0*T20+(-4.5D0*im*pi+T58-T4+3.875D0)*T48+((T22+1.0D0)*T6+(
     &T69+T16+10.0D0)*im*pi+6.0D0*T7+7.0D0*T8+T71+T24+T17+14.0D0*T11+4.0
     &D0*T12+T18+3.75D0*T2+T4+3.5833333333333333333D0)*T5+((T19-2.0D0)*T
     &6+(-16.0D0*T3-16.0D0*T2-5.5D0)*im*pi-24.0D0*T7-28.0D0*T8-8.0D0*T9-
     &12.0D0*T10-24.0D0*T3-27.0D0*T11-9.0D0*T12-12.0D0*T13+2.25D0*T2+8.0
     &D0*T14-13.958333333333333333D0)*T15+((T27+1.0D0)*T6+(20.0D0*T3+T16
     &)*im*pi+30.0D0*T7+35.0D0*T8+10.0D0*T9+15.0D0*T10+T17+12.0D0*T11+6.
     &0D0*T12+T18+T19-7.0D0*T14+9.0D0)*x-1.0D0*T2*T6-8.0D0*T3*im*pi-12.0
     &D0*T7-14.0D0*T8-T55-T45+T11-1.0D0*T12+T74)*zeta3+(0.02083333333333
     &3333333D0*T21-1.25D0*im*T23+(0.083333333333333333333D0*T3+0.083333
     &333333333333333D0*T11+T51+0.083333333333333333333D0*T13+1.625D0)*T
     &6+(-2.5D0*T3-2.5D0*T11-2.5D0*T12-2.5D0*T13)*im*pi+0.5D0*T28+T29+T3
     &0+0.5D0*T31+0.5D0*T32+T33+T34+0.5D0*T35+0.5D0*T36+T37+T38+0.5D0*T3
     &9+0.5D0*T40+T41+T42+0.5D0*T43+T44-T82+T56-2.5D0*T26+3.25D0*T3+3.25
     &D0*T11+3.25D0*T12+T125)*T20
      T155=(-0.019444444444444444444D0*T2-0.038888888888888888889D0)*im*
     &T67+(0.16944444444444444444D0*T3-0.14166666666666666667D0*T11+0.33
     &888888888888888889D0*T2+T108+0.75347222222222222222D0)*T21+(-1.0D0
     &*T7-0.5D0*T8-0.33333333333333333333D0*T9+0.16666666666666666667D0*
     &T10+T109+T57+T110+T111-0.41666666666666666667D0*T2-1.4166666666666
     &666667D0*T14+0.29166666666666666667D0)*im*T23+(T49+T30+0.666666666
     &66666666667D0*T31+0.33333333333333333333D0*T50+T34+T68+T9+T113+0.6
     &6666666666666666667D0*T46+T47-0.375D0*T3+1.1666666666666666667D0*T
     &11-T51+0.83333333333333333333D0*T13+0.375D0*T14+1.625D0)*T6+(-6.0D
     &0*T75-6.0D0*T59+T76-4.0D0*T77+T78-4.0D0*T60-T61-2.0D0*T62+T63+T80+
     &T114+T115+T116+T117+T70+T118+T119+T120+T121+T122+T81-4.5D0*T8+T55-
     &5.0D0*T10+T56-6.5D0*T46+T72-7.0D0*T26+2.25D0*T3+T57+2.25D0*T12-1.0
     &D0*T13+T58-T4-1.25D0)*im*pi-6.0D0*T85-6.0D0*T86+T87+T88-3.0D0*T89-
     &1.0D0*T90-4.0D0*T91-2.0D0*T92-2.0D0*T93
      T156=(T155+T94+T95+T96-1.0D0*T97+T126-12.0D0*T60+T61+T127+T128+T12
     &9+T130-4.0D0*T102+T134+T135+T136+T137+T138-7.0D0*T28-2.5D0*T49-T11
     &2-7.0D0*T31-4.25D0*T32-0.25D0*T50-0.25D0*T52-5.25D0*T35-4.5D0*T36-
     &2.0D0*T53-2.0D0*T65-8.5D0*T39-3.0D0*T40-1.0D0*T66-1.0D0*T54-8.0D0*
     &T43-2.25D0*T7-1.0D0*T10-2.0D0*T25+0.25D0*T46+0.25D0*T47-0.75D0*T26
     &+6.125D0*T3+T139+T140+0.125D0*T13+0.5625D0*T2-0.6875D0*T14)*T5
      T157=(0.077777777777777777778D0*T2+0.077777777777777777778D0)*im*T
     &67+(-0.67777777777777777778D0*T3+0.56666666666666666667D0*T11-0.63
     &333333333333333333D0*T2+0.56666666666666666667D0*T14-0.76041666666
     &666666667D0)*T21+(4.0D0*T7+T68+1.3333333333333333333D0*T9-0.666666
     &66666666666667D0*T10+T69+T84+1.3333333333333333333D0*T12+T73-0.958
     &33333333333333333D0*T2+1.25D0*T14-1.5D0)*im*T23+(-4.0D0*T49-2.0D0*
     &T64-2.6666666666666666667D0*T31-1.3333333333333333333D0*T50-T70+T1
     &41-3.6666666666666666667D0*T8-T71-2.6666666666666666667D0*T10+T142
     &-1.1666666666666666667D0*T46-T72+2.1666666666666666667D0*T3-2.0833
     &333333333333333D0*T11+1.5416666666666666667D0*T12+T73+T74-2.083333
     &3333333333333D0*T14+1.6979166666666666667D0)*T6+(24.0D0*T75+T98-4.
     &0D0*T76+16.0D0*T77-4.0D0*T78+16.0D0*T60+8.0D0*T79+8.0D0*T62-T100-4
     &.0D0*T80+24.0D0*T49+24.0D0*T31-4.0D0*T32+16.0D0*T50-4.0D0*T52+16.0
     &D0*T35+8.0D0*T53+8.0D0*T39-4.0D0*T40-4.0D0*T54-T81+1.5D0*T8-3.5D0*
     &T9+T82+0.25D0*T25+T83+1.25D0*T47+4.0D0*T26-7.875D0*T3+T84-7.625D0*
     &T12+2.25D0*T13-4.6875D0*T2-3.1875D0*T14+0.3125D0)*im*pi
      T158=(T157+24.0D0*T85+24.0D0*T86-4.0D0*T87-4.0D0*T88+12.0D0*T89+4.
     &0D0*T90+16.0D0*T91+8.0D0*T92+8.0D0*T93-4.0D0*T94-4.0D0*T95-4.0D0*T
     &96+4.0D0*T97+T98+T76+T77+T78+25.0D0*T60-T146-4.0D0*T99+12.0D0*T62+
     &T100+16.0D0*T101+T148+T149+T150+7.0D0*T102+8.0D0*T103-4.0D0*T104-4
     &.0D0*T105-4.0D0*T106+4.0D0*T107+8.0D0*T28+3.5D0*T49+3.5D0*T64+5.0D
     &0*T31+0.25D0*T32+T152+T153-0.75D0*T35+4.25D0*T36+4.5D0*T53+4.5D0*T
     &65+7.5D0*T39-1.25D0*T40+4.0D0*T43+T44-5.375D0*T8-5.375D0*T9-3.375D
     &0*T10+1.875D0*T25-5.75D0*T46-5.75D0*T47-3.5D0*T26+9.375D0*T3+4.687
     &5D0*T11+4.6875D0*T12+1.5D0*T13+1.125D0*T2+1.4375D0*T14)*T15
      T159=(-0.097222222222222222222D0*T2-0.038888888888888888889D0)*im*
     &T67+(0.84722222222222222222D0*T3-0.70833333333333333333D0*T11+0.25
     &D0*T2+T108+0.23888888888888888889D0)*T21+(-5.0D0*T7-2.5D0*T8-1.666
     &6666666666666667D0*T9+0.83333333333333333333D0*T10+T109+T57+T110+T
     &111+0.875D0*T2-0.33333333333333333333D0*T14+0.33333333333333333333
     &D0)*im*T23+(5.0D0*T49+T112+3.3333333333333333333D0*T31+1.666666666
     &6666666667D0*T50+2.5D0*T52+0.33333333333333333333D0*T7+1.333333333
     &3333333333D0*T8+T9+T113+0.66666666666666666667D0*T25+0.33333333333
     &333333333D0*T46+T47-2.2916666666666666667D0*T3+2.875D0*T11-1.04166
     &66666666666667D0*T12+0.91666666666666666667D0*T2+1.291666666666666
     &6667D0*T14-0.97916666666666666667D0)*T6+(-30.0D0*T75-30.0D0*T59+5.
     &0D0*T76-20.0D0*T77+5.0D0*T78-20.0D0*T60-10.0D0*T79-10.0D0*T62+5.0D
     &0*T63+5.0D0*T80+T114+T115+T116+T117+T70+T118+T119+T120+T121+T122+T
     &123-T147+T9-3.75D0*T10-1.25D0*T25+0.5D0*T46-1.75D0*T47+4.5D0*T3-3.
     &0D0*T11+T124-T125+8.5D0*T2+3.125D0*T14+0.9375D0)*im*pi
      T160=(T159-30.0D0*T85-30.0D0*T86+5.0D0*T87+5.0D0*T88-15.0D0*T89-5.
     &0D0*T90-20.0D0*T91-10.0D0*T92-10.0D0*T93+5.0D0*T94+5.0D0*T95+5.0D0
     &*T96-5.0D0*T97+T126+T144-2.0D0*T77+T145-14.0D0*T60+T61+T127+T128+T
     &129+T130+2.0D0*T131+2.0D0*T132+2.0D0*T133-2.0D0*T102+T134+T135+T13
     &6+T137+T138-1.5D0*T28-3.25D0*T31+5.75D0*T32+6.75D0*T50+6.75D0*T52+
     &3.0D0*T35-1.25D0*T36-2.5D0*T53-2.5D0*T65-2.0D0*T39+1.75D0*T40-0.75
     &D0*T7+3.75D0*T8+3.75D0*T9+0.75D0*T10-1.0D0*T25+3.25D0*T46+3.25D0*T
     &47-11.625D0*T3-T139-T140-0.9375D0*T2)*x
      T161=T154+(-0.3D0*T21+(T22+0.5D0*T14+2.125D0)*im*T23+(-0.333333333
     &33333333333D0*T3-0.58333333333333333333D0*T11-0.333333333333333333
     &33D0*T12-0.58333333333333333333D0*T13+0.22916666666666666667D0*T2+
     &0.41666666666666666667D0*T14-3.96875D0)*T6+(-T123+3.0D0*T8-1.5D0*T
     &9+T24-1.5D0*T25+T83-1.5D0*T47+3.0D0*T26+4.25D0*T3+4.25D0*T11+T124+
     &4.25D0*T13+T27+2.5D0*T14)*im*pi+T28-T29-T30+2.5D0*T31+T32-T33-T34+
     &2.5D0*T35+T36-T37-T38+2.5D0*T39+T40-T41-T42+2.5D0*T43-T44+1.75D0*T
     &8+1.75D0*T9+T45-2.125D0*T25+2.125D0*T46+2.125D0*T47+6.375D0*T26-10
     &.4375D0*T3-7.9375D0*T11-7.9375D0*T12-5.4375D0*T13-0.75D0*T2-0.75D0
     &*T14)*T48+T156+T158+T160+0.038888888888888888889D0*T2*im*T67+(-0.3
     &3888888888888888889D0*T3+0.28333333333333333333D0*T11+0.0444444444
     &44444444444D0*T2)*T21+(2.0D0*T7+T8+0.66666666666666666667D0*T9-0.3
     &3333333333333333333D0*T10)*im*T23+(-2.0D0*T49+T151-1.3333333333333
     &333333D0*T31-0.66666666666666666667D0*T50-1.0D0*T52+T141+0.3333333
     &3333333333333D0*T8+T142+0.16666666666666666667D0*T46+0.75D0*T3-1.4
     &583333333333333333D0*T11-0.14583333333333333333D0*T2)*T6
      betaHPL_3l_nfv=((T161+(12.0D0*T75+T143+T144+8.0D0*T77+T145+8.0D0*T60+T146+4.0
     &D0*T62+T129-2.0D0*T80+T147+3.25D0*T10-0.625D0*T3+0.125D0*T11-0.75D
     &0*T12-3.3125D0*T2)*im*pi+12.0D0*T85+12.0D0*T86-2.0D0*T87-2.0D0*T88
     &+6.0D0*T89+2.0D0*T90+8.0D0*T91+4.0D0*T92+4.0D0*T93-2.0D0*T94-2.0D0
     &*T95-2.0D0*T96+2.0D0*T97+T76+T77+T78+T60+T148+T149+T150-1.0D0*T102
     &-1.0D0*T28-1.0D0*T49+T151+2.25D0*T31-3.25D0*T32+T152+T153+0.5D0*T7
     &-0.125D0*T8-0.125D0*T9+0.75D0*T25+3.3125D0*T3)*dabc2)/(Nc*T5
     &-1.0D0*Nc*T15)

      end function betaHPL_3l_nfv

      function betaHPL_3l_no_nfv(x)
        use types
        use constants
        implicit complex(dp) (T)
        include 'nf.f'
        complex(dp) :: betaHPL_3l_no_nfv
        real(dp), intent(in) :: x

      T2=CF*CF
      T3=CF**3
      T4=Hr1(1)
      T5=Hr1(0)
      T6=5.25D0*T4
      T7=CA*CA
      T8=(34.0D0*T3-28.0D0*CA*T2-7.0D0*T7*CF)*im*pi
      T9=(-10.777777777777777778D0*T2+3.1666666666666666667D0*CA*CF)*Nf
      T10=-11.0D0*T5
      T11=7.5D0*T5
      T12=3.5D0*T5
      T13=x**4
      T14=x**3
      T15=2.0D0*T3+11.333333333333333333D0*CA*T2-15.777777777777777778D0
     &*T7*CF
      T16=x*x
      T17=7.0D0*T5
      T18=7.0D0*T4
      T19=Hr2(1,1)
      T20=Hr2(1,0)
      T21=Hr2(0,1)
      T22=Hr2(0,0)
      T23=Nf*Nf
      T24=2.0D0*T19
      T25=Hr3(1,1,1)
      T26=Hr3(1,1,0)
      T27=Hr3(1,0,1)
      T28=Hr3(1,0,0)
      T29=Hr3(0,1,1)
      T30=Hr3(0,1,0)
      T31=Hr3(0,0,1)
      T32=6.0D0*Hr3(0,0,0)
      T33=2.0D0*T30
      T34=pi**3
      T35=pi*pi
      T36=12.0D0*T21
      T37=-16.0D0*T19
      T38=Hr3(0,0,0)
      T39=12.0D0*T38
      T40=-4.0D0*T30
      T41=(-10.833333333333333333D0*T3+13.208333333333333333D0*CA*T2-3.9
     &444444444444444444D0*T7*CF)*im*T34
      T42=(-1.3935185185185185185D0*T2+0.63888888888888888889D0*CA*CF)*N
     &f
      T43=-1.1666666666666666667D0*T5
      T44=-1.8333333333333333333D0*T5
      T45=-0.5D0*T5
      T46=-0.074074074074074074074D0*CF*T23
      T47=2.0555555555555555555D0*T5
      T48=-0.66666666666666666667D0*T5
      T49=10.0D0*T22
      T50=-19.0D0*T21
      T51=-8.5D0*T22
      T52=6.0D0*T21
      T53=2.0D0*T22
      T54=-0.21399176954732510288D0*CF*T23
      T55=-1.3333333333333333333D0*T21
      T56=0.72222222222222222222D0*T22
      T57=33.0D0*T19
      T58=1.6666666666666666667D0*T21
      T59=-0.66666666666666666667D0*T22
      T60=22.0D0*T29
      T61=-10.0D0*T31
      T62=-21.0D0*T29
      T63=-6.0D0*T30
      T64=7.0D0*T31
      T65=4.0D0*T29
      T66=1.3333333333333333333D0*T4
      T67=-9.0D0*T5
      T68=12.0D0*T20
      T69=-26.0D0*T19
      T70=32.0D0*T21
      T71=7.0D0*T19
      T72=68.0D0*T27
      T73=1.5D0*T20
      T74=0.5D0*T4
      T75=-2.0D0*T5
      T76=Hr4(1,1,1,1)
      T77=Hr4(1,1,0,0)
      T78=Hr4(1,0,1,1)
      T79=Hr4(1,0,0,0)
      T80=Hr4(0,1,1,1)
      T81=Hr4(0,1,0,0)
      T82=Hr4(0,0,1,1)
      T83=Hr4(0,0,0,0)
      T84=Hr4(1,1,1,0)
      T85=Hr4(1,1,0,1)
      T86=Hr4(1,0,1,0)
      T87=Hr4(1,0,0,1)
      T88=Hr4(0,1,1,0)
      T89=Hr4(0,1,0,1)
      T90=Hr4(0,0,1,0)
      T91=Hr4(0,0,0,1)
      T92=x**6
      T93=pi**4
      T94=23.5D0*T20
      T95=7.0D0*T27
      T96=-4.0D0*T31
      T97=2.5D0*T28
      T98=2.0D0*T31
      T99=x**5
      T100=0.41666666666666666667D0*T22
      T101=-0.11111111111111111111D0*T21
      T102=2.5D0*T76
      T103=2.1666666666666666667D0*T38
      T104=2.3333333333333333333D0*T78
      T105=2.0D0*T91
      T106=0.22222222222222222222D0*T38
      T107=0.11111111111111111111D0*T19
      T108=1.3333333333333333333D0*T90
      T109=-4.5D0*T91
      T110=Hr5(1,1,1,1,1)
      T111=Hr5(1,1,1,1,0)
      T112=Hr5(1,1,1,0,1)
      T113=Hr5(1,1,1,0,0)
      T114=Hr5(1,1,0,1,1)
      T115=Hr5(1,1,0,1,0)
      T116=Hr5(1,1,0,0,1)
      T117=Hr5(1,1,0,0,0)
      T118=Hr5(1,0,1,1,0)
      T119=Hr5(1,0,1,0,1)
      T120=Hr5(1,0,1,0,0)
      T121=Hr5(1,0,0,1,1)
      T122=Hr5(1,0,0,1,0)
      T123=Hr5(1,0,0,0,1)
      T124=Hr5(0,1,1,1,1)
      T125=Hr5(0,1,1,1,0)
      T126=Hr5(0,1,1,0,1)
      T127=Hr5(0,1,1,0,0)
      T128=Hr5(0,1,0,1,1)
      T129=Hr5(0,1,0,1,0)
      T130=Hr5(0,1,0,0,1)
      T131=Hr5(0,0,1,1,1)
      T132=8.0D0*Hr5(0,0,1,1,0)
      T133=Hr5(0,0,1,0,1)
      T134=Hr5(0,0,1,0,0)
      T135=Hr5(0,0,0,1,0)
      T136=Hr5(0,0,0,0,1)
      T137=3.0D0*T87
      T138=Hr5(1,0,1,1,1)
      T139=-1.0D0*T119
      T140=2.0D0*T122
      T141=Hr5(0,1,0,0,0)
      T142=Hr5(0,0,1,1,0)
      T143=-2.0D0*T134
      T144=Hr5(0,0,0,1,1)
      T145=3.0D0*T135
      T146=1.75D0*T82
      T147=Hr5(0,0,0,0,0)
      T148=7.0D0*T80
      T149=7.0D0*T20
      T150=-1.0D0*T141
      T151=1.3333333333333333333D0*T134
      T152=-1.0D0*T144
      T153=-0.5D0*T87
      T154=2.0D0*T88
      T155=2.0D0*T89
      T156=Hr6(1,1,1,1,1,1)
      T157=Hr6(1,1,1,1,1,0)
      T158=Hr6(1,1,1,1,0,1)
      T159=Hr6(1,1,1,1,0,0)
      T160=Hr6(1,1,1,0,1,1)
      T161=Hr6(1,1,1,0,1,0)
      T162=Hr6(1,1,1,0,0,1)
      T163=Hr6(1,1,1,0,0,0)
      T164=Hr6(1,1,0,1,1,1)
      T165=Hr6(1,1,0,1,1,0)
      T166=Hr6(1,1,0,1,0,1)
      T167=Hr6(1,1,0,1,0,0)
      T168=Hr6(1,1,0,0,1,1)
      T169=Hr6(1,1,0,0,1,0)
      T170=Hr6(1,1,0,0,0,1)
      T171=Hr6(1,1,0,0,0,0)
      T172=Hr6(1,0,1,1,1,1)
      T173=Hr6(1,0,1,1,1,0)
      T174=Hr6(1,0,1,1,0,1)
      T175=Hr6(1,0,1,1,0,0)
      T176=Hr6(1,0,1,0,1,1)
      T177=Hr6(1,0,1,0,1,0)
      T178=Hr6(1,0,1,0,0,1)
      T179=Hr6(1,0,1,0,0,0)
      T180=Hr6(1,0,0,1,1,1)
      T181=Hr6(1,0,0,1,1,0)
      T182=Hr6(1,0,0,1,0,1)
      T183=Hr6(1,0,0,1,0,0)
      T184=Hr6(1,0,0,0,1,1)
      T185=Hr6(0,1,1,1,1,0)
      T186=Hr6(0,1,1,1,0,1)
      T187=Hr6(0,1,1,1,0,0)
      T188=Hr6(0,1,1,0,1,1)
      T189=8.0D0*Hr6(0,1,1,0,1,0)
      T190=8.0D0*Hr6(0,1,1,0,0,1)
      T191=Hr6(0,1,1,0,0,0)
      T192=Hr6(0,1,0,1,1,1)
      T193=Hr6(0,1,0,1,1,0)
      T194=Hr6(0,1,0,1,0,1)
      T195=Hr6(0,1,0,1,0,0)
      T196=Hr6(0,1,0,0,1,0)
      T197=Hr6(0,1,0,0,0,1)
      T198=Hr6(0,1,0,0,0,0)
      T199=Hr6(0,0,1,1,1,1)
      T200=Hr6(0,0,1,1,1,0)
      T201=Hr6(0,0,1,1,0,1)
      T202=Hr6(0,0,1,1,0,0)
      T203=Hr6(0,0,1,0,1,1)
      T204=Hr6(0,0,1,0,1,0)
      T205=Hr6(0,0,1,0,0,1)
      T206=Hr6(0,0,0,1,1,1)
      T207=3.0D0*Hr6(0,0,0,1,0,0)
      T208=Hr6(0,0,0,0,1,1)
      T209=Hr5(1,0,0,0,0)
      T210=31.0D0*T142
      T211=31.0D0*T133
      T212=6.0D0*T136
      T213=-1.0D0*T157
      T214=-1.0D0*T158
      T215=Hr6(0,1,1,0,1,0)
      T216=Hr6(0,1,1,0,0,1)
      T217=-4.0D0*T192
      T218=Hr6(0,0,0,1,1,0)
      T219=Hr6(0,0,0,1,0,1)
      T220=2.5D0*T119
      T221=pi**6
      T222=pi**5
      T223=0.11111111111111111111D0*T4
      T224=0.22222222222222222222D0*T21
      T225=2.0D0*T38
      T226=5.3333333333333333333D0*T27
      T227=0.66666666666666666667D0*T30
      T228=2.6666666666666666667D0*T27
      T229=0.22222222222222222222D0*T29
      T230=-0.55555555555555555555D0*T31
      T231=0.40740740740740740741D0*T22
      T232=-1.3333333333333333333D0*T89
      T233=-1.3333333333333333333D0*T79
      T234=-1.0D0*T80
      T235=-3.5D0*T82
      T236=2.0D0*T83
      T237=1.6666666666666666667D0*T86
      T238=1.6666666666666666667D0*T88
      T239=7.4583333333333333333D0*T22
      T240=0.44444444444444444444D0*T38
      T241=-1.3333333333333333333D0*T88
      T242=2.6666666666666666667D0*T90
      T243=4.0D0*T91
      T244=6.3333333333333333333D0*T20
      T245=2.0D0*T25
      T246=8.0D0*T26
      T247=2.0D0*T27
      T248=2.0D0*T29
      T249=8.2222222222222222222D0*T30
      T250=2.0D0*T20
      T251=6.0D0*T135
      T252=-14.0D0*T87
      T253=16.0D0*T116
      T254=-4.0D0*T130
      T255=16.0D0*T142
      T256=9.0D0*T120
      T257=-4.0D0*T121
      T258=8.0D0*T124
      T259=10.0D0*T125
      T260=2.0D0*T128
      T261=-6.0D0*T129
      T262=4.0D0*T134
      T263=2.0D0*T144
      T264=2.0D0*T136
      T265=-1.3333333333333333333D0*T127
      T266=4.0D0*T144
      T267=3.3333333333333333333D0*T86
      T268=16.0D0*T89
      T269=1.3333333333333333333D0*T30
      T270=18.0D0*T20
      T271=5.3333333333333333333D0*T117
      T272=-1.3333333333333333333D0*T128
      T273=-1.3333333333333333333D0*T129
      T274=-1.3333333333333333333D0*T130
      T275=2.0D0*T141
      T276=-1.3333333333333333333D0*T131
      T277=2.6666666666666666667D0*T134
      T278=1.6666666666666666667D0*T87
      T279=12.666666666666666667D0*T83
      T280=9.0D0*T20
      T281=9.0D0*T21
      T282=Hr6(0,0,0,1,0,0)
      T283=Hr6(0,0,0,0,0,0)
      T284=24.0D0*T91
      T285=-4.0D0*T169
      T286=-4.0D0*T170
      T287=16.0D0*T215
      T288=16.0D0*T216
      T289=-4.0D0*T196
      T290=-4.0D0*T197
      T291=2.0D0*T204
      T292=2.0D0*T205
      T293=6.0D0*T282
      T294=-10.666666666666666667D0*T118
      T295=-10.666666666666666667D0*T119
      T296=8.0D0*T157
      T297=8.0D0*T158
      T298=-2.0D0*T176
      T299=8.0D0*T185
      T300=8.0D0*T186
      T301=18.0D0*T187
      T302=10.0D0*T191
      T303=8.0D0*T192
      T304=2.0D0*T198
      T305=Hr6(0,0,1,0,0,0)
      T306=4.0D0*T206
      T307=6.0D0*T218
      T308=6.0D0*T219
      T309=(-0.09697328777189888301D0*T3+0.17006540270429159318D0*CA*T2-
     &0.068506270821085635901D0*T7*CF)*T221
      T310=(0.025925925925925925926D0*T2-0.013888888888888888889D0*CA*CF
     &)*Nf
      T311=-0.22430555555555555556D0*T5
      T312=0.16944444444444444444D0*T5
      T313=0.0083333333333333333333D0*T5
      T314=0.0044238683127572016461D0*CF*T23
      T315=0.014814814814814814815D0*T5
      T316=0.0018518518518518518519D0*T5
      T317=0.67777777777777777778D0*T21
      T318=-0.56875D0*T22
      T319=-0.81111111111111111111D0*T21
      T320=0.27777777777777777778D0*T22
      T321=0.31944444444444444444D0*T21
      T322=0.061111111111111111111D0*T22
      T323=0.22222222222222222222D0*T20
      T324=0.11111111111111111111D0*T20
      T325=-0.11111111111111111111D0*T22
      T326=3.6666666666666666667D0*T29
      T327=-2.0833333333333333333D0*T30
      T328=-1.1666666666666666667D0*T31
      T329=0.75D0*T38
      T330=-5.5D0*T29
      T331=1.9166666666666666667D0*T30
      T332=0.25D0*T31
      T333=-1.0D0*T38
      T334=0.074074074074074074074D0*T22
      T335=-0.23456790123456790123D0*T5
      T336=-0.33333333333333333333D0*T30
      T337=-0.27777777777777777778D0*T38
      T338=-0.11111111111111111111D0*T29
      T339=-0.11111111111111111111D0*T30
      T340=0.27777777777777777778D0*T31
      T341=-7.0D0*T79
      T342=0.66666666666666666667D0*T89
      T343=-2.75D0*T81
      T344=-2.1666666666666666667D0*T82
      T345=-3.3333333333333333333D0*T90
      T346=2.6666666666666666667D0*T91
      T347=-4.25D0*T83
      T348=55.5D0*T31
      T349=2.5D0*T38
      T350=36.083333333333333333D0*T78
      T351=0.5D0*T80
      T352=1.8333333333333333333D0*T88
      T353=2.3333333333333333333D0*T89
      T354=3.0833333333333333333D0*T81
      T355=5.3333333333333333333D0*T90
      T356=-1.0D0*T83
      T357=-0.83333333333333333333D0*T88
      T358=-0.5D0*T81
      T359=-0.33333333333333333333D0*T82
      T360=-1.6666666666666666667D0*T91
      T361=1.2222222222222222222D0*T38
      T362=-0.7037037037037037037D0*T22
      T363=1.2530864197530864198D0*T5
      T364=2.0D0*T84
      T365=0.66666666666666666667D0*T87
      T366=0.66666666666666666667D0*T88
      T367=1.3333333333333333333D0*T81
      T368=4.6666666666666666667D0*T83
      T369=-6.0D0*T28
      T370=-7.3333333333333333333D0*T38
      T371=0.18518518518518518518D0*T22
      T372=1.3333333333333333333D0*T85
      T373=-1.0D0*T81
      T374=-2.4444444444444444444D0*T38
      T375=14.0D0*T126
      T376=2.0D0*T127
      T377=-1.0D0*T129
      T378=-3.0D0*T141
      T379=-30.0D0*T131
      T380=-10.0D0*T133
      T381=-8.0D0*T136
      T382=15.0D0*T147
      T383=-12.0D0*T124
      T384=-13.0D0*T125
      T385=-13.0D0*T126
      T386=-9.0D0*T127
      T387=-3.0D0*T128
      T388=2.0D0*T130
      T389=27.0D0*T131
      T390=13.0D0*T133
      T391=-9.0D0*T135
      T392=6.0D0*T121
      T393=4.0D0*T124
      T394=5.0D0*T125
      T395=3.0D0*T126
      T396=3.0D0*T127
      T397=3.0D0*T129
      T398=-6.0D0*T131
      T399=-3.0D0*T142
      T400=-4.0D0*T133
      T401=-1.0D0*T136
      T402=9.0D0*T89
      T403=-6.0D0*T82
      T404=0.66666666666666666667D0*T83
      T405=-1.4074074074074074074D0*T38
      T406=1.2530864197530864198D0*T22
      T407=0.66666666666666666667D0*T127
      T408=0.66666666666666666667D0*T141
      T409=6.6666666666666666667D0*T147
      T410=-6.3333333333333333333D0*T83
      T411=11.0D0*T138
      T412=0.66666666666666666667D0*T128
      T413=0.66666666666666666667D0*T129
      T414=0.66666666666666666667D0*T130
      T415=0.66666666666666666667D0*T131
      T416=3.0D0*T86
      T417=-8.0D0*T79
      T418=18.0D0*T175
      T419=-6.0D0*T188
      T420=-2.0D0*T193
      T421=-2.0D0*T194
      T422=-3.0D0*T195
      T423=-7.0D0*T198
      T424=50.0D0*T199
      T425=20.0D0*T200
      T426=20.0D0*T201
      T427=12.0D0*T202
      T428=12.0D0*T203
      T429=8.0D0*T208
      T430=15.0D0*T283
      T431=12.0D0*T136
      T432=Hr6(1,0,0,0,1,0)
      T433=Hr6(1,0,0,0,0,1)
      T434=-12.0D0*T185
      T435=-12.0D0*T186
      T436=-25.0D0*T187
      T437=5.0D0*T188
      T438=-17.0D0*T191
      T439=10.0D0*T192
      T440=7.0D0*T193
      T441=7.0D0*T194
      T442=2.0D0*T196
      T443=2.0D0*T197
      T444=-49.0D0*T199
      T445=-22.0D0*T200
      T446=-22.0D0*T201
      T447=-14.0D0*T202
      T448=-14.0D0*T203
      T449=-1.0D0*T204
      T450=-1.0D0*T205
      T451=6.0D0*T206
      T452=-6.0D0*T208
      T453=18.0D0*T135
      T454=-36.666666666666666667D0*T147
      T455=-18.0D0*T157
      T456=-18.0D0*T158
      T457=6.0D0*T169
      T458=6.0D0*T170
      T459=-3.0D0*T177
      T460=-3.0D0*T178
      T461=4.0D0*T185
      T462=4.0D0*T186
      T463=9.0D0*T187
      T464=-1.0D0*T188
      T465=2.0D0*T215
      T466=2.0D0*T216
      T467=5.0D0*T191
      T468=-3.0D0*T193
      T469=-3.0D0*T194
      T470=-1.0D0*T198
      T471=12.0D0*T199
      T472=6.0D0*T200
      T473=6.0D0*T201
      T474=3.0D0*T202
      T475=4.0D0*T203
      T476=-2.0D0*T305
      T477=-2.0D0*T206
      T478=-3.0D0*T218
      T479=-3.0D0*T219
      T480=75.5D0*T80
      T481=0.11111111111111111111D0*T5
      T482=-15.333333333333333333D0*T27
      T483=1.9166666666666666667D0*T21
      T484=12.75D0*T22
      T485=-11.111111111111111111D0*T26
      T486=11.333333333333333333D0*T27
      T487=-2.3888888888888888889D0*T5
      T488=12.5D0*T76
      T489=3.0D0*T76
      T490=3.6666666666666666667D0*T28
      T491=0.44444444444444444444D0*T25
      T492=2.6666666666666666667D0*T77
      T493=2.6666666666666666667D0*T78
      T494=-1.3333333333333333333D0*T87
      T495=1.3333333333333333333D0*T29
      T496=-1.6666666666666666667D0*T30
      T497=2.6666666666666666667D0*T85
      T498=2.3333333333333333333D0*T31
      T499=1.1666666666666666667D0*T22
      T500=20.0D0*T121
      T501=12.0D0*T30
      T502=9.3333333333333333333D0*T38
      T503=9.75D0*T5
      T504=28.0D0*T114
      T505=8.0D0*T209
      T506=3.0D0*T89
      T507=2.0D0*T90
      T508=-26.583333333333333333D0*T31
      T509=1.3333333333333333333D0*T121
      T510=6.0D0*T86
      T511=6.0D0*T87
      T512=8.0D0*T88
      T513=8.0D0*T89
      T514=25.222222222222222222D0*T25
      T515=37.333333333333333333D0*T110
      T516=26.666666666666666667D0*T111
      T517=26.666666666666666667D0*T112
      T518=13.333333333333333333D0*T114
      T519=12.0D0*T119
      T520=4.0D0*T88
      T521=4.0D0*T89
      T522=1.1666666666666666667D0*T31
      T523=-48.0D0*T164
      T524=16.0D0*T177
      T525=16.0D0*T178
      T526=24.0D0*T138
      T527=32.0D0*T131
      T528=1.3333333333333333333D0*T88
      T529=1.3333333333333333333D0*T89
      T530=3.3333333333333333333D0*T90
      T531=3.3333333333333333333D0*T91
      T532=-20.0D0*T177
      T533=-20.0D0*T178
      T534=140.0D0*T114
      T535=12.0D0*T135
      T536=16.0D0*T157
      T537=16.0D0*T158
      T538=-9.0D0*T161
      T539=-9.0D0*T162
      T540=19.0D0*T172
      T541=2.0D0*T176
      T542=6.0D0*T177
      T543=6.0D0*T178
      T544=8.0D0*T432
      T545=8.0D0*T433
      T546=10.0D0*T133
      T547=-4.0D0*T135
      T548=-4.0D0*T136
      T549=0.27777777777777777778D0*T19
      T550=0.074074074074074074074D0*T19
      T551=-17.0D0*T87
      T552=-1.0D0*T31
      T553=0.75D0*T76
      T554=1.3333333333333333333D0*T79
      T555=0.5D0*T31
      T556=2.0D0*T86
      T557=-0.22222222222222222222D0*T25
      T558=-1.2530864197530864198D0*T4
      T559=4.6666666666666666667D0*T76
      T560=1.3333333333333333333D0*T84
      T561=1.3333333333333333333D0*T77
      T562=1.3333333333333333333D0*T78
      T563=4.6111111111111111111D0*T27
      T564=2.3333333333333333333D0*T28
      T565=-1.0D0*T78
      T566=2.9444444444444444444D0*T25
      T567=1.1666666666666666667D0*T28
      T568=0.16666666666666666667D0*T31
      T569=-4.0D0*T20
      T570=-0.19444444444444444444D0*T21
      T571=15.0D0*T138
      T572=4.0D0*T121
      T573=8.0D0*T122
      T574=2.6666666666666666667D0*T81
      T575=6.0D0*T29
      T576=6.5D0*T30
      T577=12.0D0*T209
      T578=2.6666666666666666667D0*T88
      T579=3.0D0*T82
      T580=-6.5D0*T114
      T581=9.5D0*T116
      T582=-2.5D0*T138
      T583=2.0D0*T119
      T584=0.5D0*T121
      T585=-7.0D0*T123
      T586=-4.0D0*T209
      T587=8.0D0*T86
      T588=0.16666666666666666667D0*T79
      T589=2.6666666666666666667D0*T80
      T590=0.5D0*T90
      T591=-0.083333333333333333333D0*T31
      T592=0.66666666666666666667D0*T76
      T593=8.0D0*T118
      T594=-0.66666666666666666667D0*T121
      T595=-4.0D0*T118
      T596=4.0D0*T119
      T597=4.0D0*T26
      T598=4.0D0*T27
      T599=12.0D0*T164
      T600=28.0D0*T165
      T601=28.0D0*T166
      T602=23.0D0*T184
      T603=-27.333333333333333333D0*T120
      T604=2.6666666666666666667D0*T129
      T605=2.6666666666666666667D0*T130
      T606=81.5D0*T76
      T607=16.666666666666666667D0*T87
      T608=-12.0D0*T432
      T609=-12.0D0*T433
      T610=39.0D0*T138
      T611=-2.6666666666666666667D0*T122
      T612=-2.6666666666666666667D0*T123
      T613=4.3333333333333333333D0*T80
      T614=-5.0D0*T157
      T615=-5.0D0*T158
      T616=1.5D0*T161
      T617=1.5D0*T162
      T618=13.0D0*T165
      T619=13.0D0*T166
      T620=0.5D0*T168
      T621=-1.0D0*T176
      T622=4.0D0*T432
      T623=4.0D0*T433
      T624=T41+(T42+(-T18+T43-16.5D0)*T3+(5.4166666666666666667D0*T4+T44
     &+28.331018518518518518D0)*CA*T2+(-0.58333333333333333333D0*T4+T45-
     &10.263888888888888889D0)*T7*CF)*T35+(T46+((-T66+T47-1.944444444444
     &4444444D0)*T2+(T48-15.388888888888888889D0)*CA*CF)*Nf+(-35.0D0*T19
     &-1.0D0*T20+T36+T49-75.0D0*T4-438.5D0)*T3+(15.5D0*T19+4.5D0*T20+T50
     &+T51+28.083333333333333333D0*T4+8.4444444444444444444D0*T5+450.291
     &66666666666667D0)*CA*T2+(T24-T73+T52+T53+8.5D0*T4-8.08333333333333
     &33333D0*T5-56.314814814814814815D0)*T7*CF)*im*pi+T54+((-3.33333333
     &33333333333D0*T19-0.66666666666666666667D0*T20+T55+T56-16.63888888
     &8888888889D0*T4-5.2222222222222222222D0*T5-6.7098765432098765432D0
     &)*T2+(T24+0.33333333333333333333D0*T20+T58+T59+T223-1.277777777777
     &7777778D0*T5-12.824074074074074074D0)*CA*CF)*Nf+(-25.0D0*T25-13.0D
     &0*T26-21.0D0*T27-11.0D0*T28+T60+T33+T61+T32-133.66666666666666667D
     &0*T19-49.0D0*T20-T70-31.0D0*T22-310.33333333333333333D0*T4-147.0D0
     &*T5-281.875D0)*T3
      T625=(T624+(27.5D0*T25+25.5D0*T26+24.5D0*T27+18.5D0*T28+T62+T63+T6
     &4-T32+161.5D0*T19+76.666666666666666667D0*T20+50.08333333333333333
     &3D0*T21+55.277777777777777778D0*T22+429.05555555555555556D0*T4+189
     &.63888888888888889D0*T5+397.90740740740740741D0)*CA*T2+(-8.0D0*T25
     &-8.5D0*T26-T95+T369+T65+T33-50.166666666666666667D0*T19-25.3333333
     &33333333333D0*T20-22.916666666666666667D0*T21-20.83333333333333333
     &3D0*T22-108.15277777777777778D0*T4-54.013888888888888889D0*T5-36.0
     &76903292181069959D0)*T7*CF)*T13
      T626=(21.666666666666666667D0*T3-26.416666666666666667D0*CA*T2+7.8
     &888888888888888889D0*T7*CF)*im*T34+((2.787037037037037037D0*T2-1.2
     &777777777777777778D0*CA*CF)*Nf+(37.0D0*T4+2.3333333333333333333D0*
     &T5+52.583333333333333333D0)*T3+(-32.333333333333333333D0*T4+3.6666
     &666666666666667D0*T5-81.24537037037037037D0)*CA*T2+(6.166666666666
     &6666667D0*T4+T5+26.444444444444444444D0)*T7*CF)*T35+(0.14814814814
     &814814815D0*CF*T23+((2.6666666666666666667D0*T4-4.1111111111111111
     &111D0*T5+1.9444444444444444444D0)*T2+(1.3333333333333333333D0*T5+9
     &.4444444444444444444D0)*CA*CF)*Nf+(136.0D0*T19-10.0D0*T20-24.0D0*T
     &21-20.0D0*T22+214.0D0*T4+T67+282.5D0)*T3+(-40.0D0*T19+3.0D0*T20+38
     &.0D0*T21+17.0D0*T22-56.666666666666666667D0*T4+3.61111111111111111
     &11D0*T5-135.63888888888888889D0)*CA*T2+(T37-T36-4.0D0*T22-30.5D0*T
     &4+8.6666666666666666667D0*T5-57.287037037037037037D0)*T7*CF)*im*pi
     &+0.42798353909465020576D0*CF*T23+((30.666666666666666667D0*T19+5.3
     &333333333333333333D0*T20+2.6666666666666666667D0*T21-1.44444444444
     &44444444D0*T22+59.555555555555555556D0*T4+14.444444444444444444D0*
     &T5-0.74691358024691358025D0)*T2+(T37-2.6666666666666666667D0*T20-3
     &.3333333333333333333D0*T21+1.3333333333333333333D0*T22-23.55555555
     &5555555556D0*T4-10.444444444444444444D0*T5+23.37037037037037037D0)
     &*CA*CF)*Nf
      T627=(T626+(164.0D0*T25+92.0D0*T26+126.0D0*T27+82.0D0*T28-44.0D0*T
     &29+T40+20.0D0*T31-T39+443.33333333333333333D0*T19+152.0D0*T20+114.
     &0D0*T21+129.0D0*T22+534.5D0*T4-40.5D0*T5+715.5D0)*T3+(-196.0D0*T25
     &-180.0D0*T26-139.0D0*T27-121.0D0*T28+42.0D0*T29+T501-14.0D0*T31+T3
     &9-670.0D0*T19-244.33333333333333333D0*T20-171.66666666666666667D0*
     &T21-197.05555555555555556D0*T22-859.72222222222222222D0*T4+39.9722
     &22222222222222D0*T5-791.73148148148148148D0)*CA*T2+(58.0D0*T25+65.
     &0D0*T26+38.0D0*T27+39.0D0*T28-8.0D0*T29+T40+227.83333333333333333D
     &0*T19+84.666666666666666667D0*T20+68.333333333333333333D0*T21+68.6
     &66666666666666667D0*T22+273.47222222222222222D0*T4+15.277777777777
     &777778D0*T5+66.181584362139917696D0)*T7*CF)*T14
      T628=T41+(T42+(-72.583333333333333333D0*T4+T43-55.6666666666666666
     &67D0)*T3+(73.0D0*T4+T44+77.497685185185185185D0)*CA*T2+(-16.5D0*T4
     &+T45-22.097222222222222222D0)*T7*CF)*T35+(T46+((0.6111111111111111
     &1111D0*T4+T47+1.9444444444444444444D0)*T2+(-0.66666666666666666667
     &D0*T4+T48-3.7222222222222222222D0)*CA*CF)*Nf+(-188.0D0*T19+32.0D0*
     &T20+T36+T49-202.5D0*T4+18.0D0*T5-89.5D0)*T3+(25.5D0*T19-40.0D0*T20
     &+T50+T51-30.861111111111111111D0*T4-32.555555555555555556D0*T5-73.
     &097222222222222222D0)*CA*T2+(T57+T68+T52+T53+66.416666666666666667
     &D0*T4+6.9166666666666666667D0*T5+83.268518518518518518D0)*T7*CF)*i
     &m*pi+T54+((-67.944444444444444444D0*T19-12.666666666666666667D0*T2
     &0+T55+T56-62.361111111111111111D0*T4-13.222222222222222222D0*T5+16
     &.290123456790123457D0)*T2+(T57+T244+T58+T59+29.055555555555555556D
     &0*T4+9.2222222222222222222D0*T5-16.268518518518518518D0)*CA*CF)*Nf
     &+(-359.0D0*T25-190.0D0*T26-257.0D0*T27-198.0D0*T28+T60+T33+T61+T32
     &-490.5D0*T19-133.5D0*T20-123.0D0*T21-165.0D0*T22-738.75D0*T4+171.5
     &D0*T5-721.04166666666666667D0)*T3
      T629=(T628+(444.0D0*T25+385.0D0*T26+284.0D0*T27+273.0D0*T28+T62+T6
     &3+T64-T32+989.94444444444444444D0*T19+241.41666666666666667D0*T20+
     &186.58333333333333333D0*T21+228.27777777777777778D0*T22+1064.44444
     &44444444444D0*T4-196.36111111111111111D0*T5+701.57407407407407407D
     &0)*CA*T2+(-130.0D0*T25-145.0D0*T26-77.0D0*T27-87.0D0*T28+T65+T33-3
     &68.0D0*T19-91.833333333333333333D0*T20-66.916666666666666667D0*T21
     &-74.833333333333333333D0*T22-348.84722222222222222D0*T4+39.1111111
     &11111111111D0*T5-110.59079218106995885D0)*T7*CF)*T16
      T630=((62.166666666666666667D0*T4+19.583333333333333333D0)*T3+(-70
     &.666666666666666667D0*T4-24.583333333333333333D0)*CA*T2+(16.833333
     &333333333333D0*T4+5.9166666666666666667D0)*T7*CF)*T35+(((-3.888888
     &8888888888889D0*T4-1.9444444444444444444D0)*T2+(T66+0.666666666666
     &66666667D0)*CA*CF)*Nf+(108.0D0*T19-30.0D0*T20+63.0D0*T4+T67+8.5D0)
     &*T3+(T71+53.0D0*T20+119.38888888888888889D0*T4+20.5D0*T5+39.444444
     &444444444444D0)*CA*T2+(T69-T270-75.333333333333333333D0*T4-T11-23.
     &416666666666666667D0)*T7*CF)*im*pi+((57.222222222222222222D0*T19+T
     &68+12.611111111111111111D0*T4+4.0D0*T5-10.833333333333333333D0)*T2
     &+(T69-6.0D0*T20-7.8888888888888888889D0*T4+T75+4.22222222222222222
     &22D0)*CA*CF)*Nf+(326.0D0*T25+156.0D0*T26+220.0D0*T27+194.0D0*T28+1
     &85.66666666666666667D0*T19+T149+T70+67.0D0*T22+559.333333333333333
     &33D0*T4-90.5D0*T5+248.75D0)*T3+(-410.0D0*T25-332.0D0*T26-249.0D0*T
     &27-257.0D0*T28-615.88888888888888889D0*T19-56.5D0*T20-58.5D0*T21-8
     &6.5D0*T22-607.44444444444444444D0*T4+103.75D0*T5-215.9166666666666
     &6667D0)*CA*T2
      T631=((-5.0D0*CA*CF*Nf-107.0D0*T3+127.0D0*CA*T2-20.25D0*T7*CF)*im*
     &pi+(-2.0D0*T2+(-2.5D0*T4-2.5D0*T5+1.0D0)*CA*CF)*Nf+(-58.5D0*T4-46.
     &5D0*T5-10.0D0)*T3+(68.0D0*T4+62.0D0*T5+32.916666666666666667D0)*CA
     &*T2+(-11.125D0*T4-11.125D0*T5-13.958333333333333333D0)*T7*CF)*T92+
     &((14.0D0*CA*CF*Nf+344.0D0*T3-408.0D0*CA*T2+74.0D0*T7*CF)*im*pi+(4.
     &0D0*T2+(T18+T17+0.5D0)*CA*CF)*Nf+(181.0D0*T4+153.0D0*T5+48.6666666
     &66666666667D0)*T3+(-213.0D0*T4-199.0D0*T5-124.75D0)*CA*T2+(39.25D0
     &*T4+39.25D0*T5+41.958333333333333333D0)*T7*CF)*T99+T625+T627+T629+
     &(T630+(118.0D0*T25+129.0D0*T26+T72+81.0D0*T28+253.1666666666666666
     &7D0*T19+31.0D0*T20+20.5D0*T21+27.0D0*T22+180.38888888888888889D0*T
     &4-28.5D0*T5+52.486111111111111111D0)*T7*CF)*x+(-19.583333333333333
     &333D0*T4*T3+24.583333333333333333D0*T4*CA*T2-5.9166666666666666667
     &D0*T4*T7*CF)*T35+((1.9444444444444444444D0*T4*T2-0.666666666666666
     &66667D0*T4*CA*CF)*Nf+(-21.0D0*T19+T280+T74)*T3+(-8.0D0*T19-20.5D0*
     &T20-59.944444444444444444D0*T4)*CA*T2+(T71+7.5D0*T20+30.9166666666
     &66666667D0*T4)*T7*CF)*im*pi
      T632=(T631+((-16.611111111111111111D0*T19+T569+6.83333333333333333
     &33D0*T4)*T2+(T71+T250-2.2222222222222222222D0*T4)*CA*CF)*Nf+(-106.
     &0D0*T25-45.0D0*T26-T72-67.0D0*T28-4.8333333333333333333D0*T19+T94+
     &T281-167.25D0*T4)*T3+(134.5D0*T25+101.5D0*T26+79.5D0*T27+86.5D0*T2
     &8+134.44444444444444444D0*T19-17.25D0*T20-6.5D0*T21+118.6666666666
     &6666667D0*T4)*CA*T2+(-38.0D0*T25-40.5D0*T26-22.0D0*T27-27.0D0*T28-
     &62.833333333333333333D0*T19+T73+T21-24.986111111111111111D0*T4)*T7
     &*CF)*zeta3
      T633=(-0.27083333333333333333D0*CA*CF*Nf-5.8208333333333333333D0*T
     &3+7.1416666666666666667D0*CA*T2-1.221875D0*T7*CF)*T93+((0.41666666
     &666666666667D0*T4+0.41666666666666666667D0*T5)*CA*CF*Nf+(5.75D0*T4
     &+11.75D0*T5-11.25D0)*T3+(-9.3333333333333333333D0*T4-12.3333333333
     &33333333D0*T5+14.083333333333333333D0)*CA*T2+(1.854166666666666666
     &7D0*T4+1.8541666666666666667D0*T5-4.2291666666666666667D0)*T7*CF)*
     &im*T34+(((-1.0D0*T4+T5-1.9166666666666666667D0)*T2+(-0.41666666666
     &666666667D0*T19-0.41666666666666666667D0*T20-0.4166666666666666666
     &7D0*T21-T100+T74+T45+0.95833333333333333333D0)*CA*CF)*Nf+(-6.83333
     &33333333333333D0*T19-6.8333333333333333333D0*T20-6.833333333333333
     &3333D0*T21-6.8333333333333333333D0*T22+2.0D0*T4+T75+2.041666666666
     &6666667D0)*T3+(10.208333333333333333D0*T19+10.208333333333333333D0
     &*T20+10.208333333333333333D0*T21+10.208333333333333333D0*T22+4.5D0
     &*T4-4.5D0*T5+10.520833333333333333D0)*CA*T2+(-2.020833333333333333
     &3D0*T19-2.0208333333333333333D0*T20-2.0208333333333333333D0*T21-2.
     &0208333333333333333D0*T22-2.75D0*T4+2.75D0*T5-5.770833333333333333
     &3D0)*T7*CF)*T35
      T634=T633+((-2.5D0*T25+2.5D0*T26-2.5D0*T27+T97-2.5D0*T29+2.5D0*T30
     &-2.5D0*T31+T349)*CA*CF*Nf+(-68.5D0*T25+46.5D0*T26-58.5D0*T27+56.5D
     &0*T28-56.5D0*T29+58.5D0*T30-46.5D0*T31+68.5D0*T38-22.5D0*T19-22.5D
     &0*T20-22.5D0*T21-22.5D0*T22)*T3+(77.0D0*T25-62.0D0*T26+T72-71.0D0*
     &T28+71.0D0*T29-68.0D0*T30+62.0D0*T31-77.0D0*T38+28.166666666666666
     &667D0*T19+28.166666666666666667D0*T20+28.166666666666666667D0*T21+
     &28.166666666666666667D0*T22)*CA*T2+(-13.125D0*T25+11.125D0*T26-11.
     &125D0*T27+13.125D0*T28-13.125D0*T29+11.125D0*T30-11.125D0*T31+13.1
     &25D0*T38-8.4583333333333333333D0*T19-8.4583333333333333333D0*T20-8
     &.4583333333333333333D0*T21-8.4583333333333333333D0*T22)*T7*CF)*im*
     &pi+((-4.0D0*T25-2.0D0*T26-T247+T33+T98+4.0D0*T38-3.833333333333333
     &3333D0*T19-3.8333333333333333333D0*T20-3.8333333333333333333D0*T21
     &-3.8333333333333333333D0*T22)*T2+(T102+2.5D0*T77+2.5D0*T78+2.5D0*T
     &79+2.5D0*T80+2.5D0*T81+2.5D0*T82+2.5D0*T83+T245+T26+T27-1.0D0*T30+
     &T552-T225+1.9166666666666666667D0*T19+1.9166666666666666667D0*T20+
     &T483+1.9166666666666666667D0*T22)*CA*CF)*Nf
      T635=T634+(77.5D0*T76+9.0D0*T84+9.0D0*T85+55.5D0*T77+67.5D0*T78+9.
     &0D0*T86+9.0D0*T87+65.5D0*T79+65.5D0*T80+9.0D0*T88+T402+67.5D0*T81+
     &55.5D0*T82+9.0D0*T90+9.0D0*T91+77.5D0*T83+22.0D0*T25+T597+T598-14.
     &0D0*T28+14.0D0*T29+T40+T96-22.0D0*T38+4.0833333333333333333D0*T19+
     &4.0833333333333333333D0*T20+4.0833333333333333333D0*T21+4.08333333
     &33333333333D0*T22)*T3+(-T606-4.5D0*T84-4.5D0*T85-66.5D0*T77-72.5D0
     &*T78-4.5D0*T86-4.5D0*T87-75.5D0*T79-T480-4.5D0*T88-4.5D0*T89-72.5D
     &0*T81-66.5D0*T82-4.5D0*T90+T109-81.5D0*T83-5.9166666666666666667D0
     &*T25+9.0D0*T26+9.0D0*T27+23.916666666666666667D0*T28-23.9166666666
     &66666667D0*T29-9.0D0*T30-9.0D0*T31+5.9166666666666666667D0*T38+21.
     &041666666666666667D0*T19+21.041666666666666667D0*T20+21.0416666666
     &66666667D0*T21+21.041666666666666667D0*T22)*CA*T2+(13.125D0*T76+11
     &.125D0*T77+11.125D0*T78+13.125D0*T79+13.125D0*T80+11.125D0*T81+11.
     &125D0*T82+13.125D0*T83-2.5416666666666666667D0*T25-5.5D0*T26-5.5D0
     &*T27-8.4583333333333333333D0*T28+8.4583333333333333333D0*T29+5.5D0
     &*T30+5.5D0*T31+2.5416666666666666667D0*T38-11.541666666666666667D0
     &*T19-11.541666666666666667D0*T20-11.541666666666666667D0*T21-11.54
     &1666666666666667D0*T22)*T7*CF
      T636=(0.75833333333333333333D0*CA*CF*Nf+18.363888888888888889D0*T3
     &-22.5D0*CA*T2+4.2756944444444444444D0*T7*CF)*T93+((-1.166666666666
     &6666667D0*T4+T43-0.41666666666666666667D0)*CA*CF*Nf+(-20.833333333
     &333333333D0*T4-34.833333333333333333D0*T5+11.333333333333333333D0)
     &*T3+(30.833333333333333333D0*T4+37.833333333333333333D0*T5-19.7916
     &66666666666667D0)*CA*T2+(-6.5416666666666666667D0*T4-6.54166666666
     &66666667D0*T5+8.4375D0)*T7*CF)*im*T34+(((2.4444444444444444444D0*T
     &4-2.2222222222222222222D0*T5+3.6111111111111111111D0)*T2+(1.166666
     &6666666666667D0*T19+1.1666666666666666667D0*T20+1.1666666666666666
     &667D0*T21+T499-0.80555555555555555555D0*T4+1.5277777777777777778D0
     &*T5-1.8055555555555555556D0)*CA*CF)*Nf+(23.166666666666666667D0*T1
     &9+T94+23.166666666666666667D0*T21+23.5D0*T22+6.9166666666666666667
     &D0*T4+22.916666666666666667D0*T5-3.2083333333333333333D0)*T3+(-32.
     &166666666666666667D0*T19-33.166666666666666667D0*T20-32.1666666666
     &66666667D0*T21-33.166666666666666667D0*T22-26.666666666666666667D0
     &*T4-T503-22.840277777777777778D0)*CA*T2+(6.625D0*T19+7.04166666666
     &66666667D0*T20+6.625D0*T21+7.0416666666666666667D0*T22+10.22916666
     &6666666667D0*T4-2.2291666666666666667D0*T5+12.222222222222222222D0
     &)*T7*CF)*T35
      T637=T636+((7.0D0*T25-7.0D0*T26+T95-7.0D0*T28+7.0D0*T29-7.0D0*T30+
     &T64-7.0D0*T38+2.5D0*T19-2.5D0*T20+2.5D0*T21-2.5D0*T22)*CA*CF*Nf+(1
     &95.0D0*T25-153.0D0*T26+179.0D0*T27-169.0D0*T28+167.0D0*T29-181.0D0
     &*T30+151.0D0*T31-197.0D0*T38+102.66666666666666667D0*T19-12.333333
     &333333333333D0*T20+92.666666666666666667D0*T21-22.3333333333333333
     &33D0*T22+24.5D0*T4+24.5D0*T5)*T3+(-219.0D0*T25+199.0D0*T26-207.0D0
     &*T27+211.0D0*T28-205.0D0*T29+213.0D0*T30-193.0D0*T31+225.0D0*T38-1
     &35.25D0*T19+3.75D0*T20-126.25D0*T21+T484-25.166666666666666667D0*T
     &4-25.166666666666666667D0*T5)*CA*T2+(38.75D0*T25-39.25D0*T26+36.75
     &D0*T27-41.25D0*T28+38.75D0*T29-39.25D0*T30+36.75D0*T31-41.25D0*T38
     &+33.708333333333333333D0*T19+9.4583333333333333333D0*T20+31.708333
     &333333333333D0*T21+T239+6.4583333333333333333D0*T4+6.4583333333333
     &333333D0*T5)*T7*CF)*im*pi+((10.666666666666666667D0*T25+5.33333333
     &33333333333D0*T26+T226+T495+T40+T96-T502+9.2222222222222222222D0*T
     &19+7.2222222222222222222D0*T20+7.2222222222222222222D0*T21+5.22222
     &22222222222222D0*T22+5.8333333333333333333D0*T4+5.8333333333333333
     &333D0*T5)*T2+(-7.0D0*T76-7.0D0*T77-7.0D0*T78+T341-T148-7.0D0*T81-7
     &.0D0*T82-7.0D0*T83-7.8333333333333333333D0*T25-2.66666666666666666
     &67D0*T26-T228-T97-3.1666666666666666667D0*T29+T33+T98+T103-4.61111
     &11111111111111D0*T19-3.6111111111111111111D0*T20-3.611111111111111
     &1111D0*T21-2.6111111111111111111D0*T22-2.9166666666666666667D0*T4-
     &2.9166666666666666667D0*T5)*CA*CF)*Nf
      T638=T637+(-209.0D0*T76-14.0D0*T84-14.0D0*T85-167.0D0*T77-193.0D0*
     &T78-14.0D0*T86+T252-183.0D0*T79-181.0D0*T80-14.0D0*T88-14.0D0*T89-
     &195.0D0*T81-165.0D0*T82-14.0D0*T90-14.0D0*T91-211.0D0*T83-67.16666
     &6666666666667D0*T25+23.5D0*T26+23.5D0*T27-0.83333333333333333333D0
     &*T28-25.166666666666666667D0*T29+55.5D0*T30+T348+21.16666666666666
     &6667D0*T38-26.416666666666666667D0*T19-6.4166666666666666667D0*T20
     &-6.4166666666666666667D0*T21+13.583333333333333333D0*T22+19.916666
     &666666666667D0*T4+19.916666666666666667D0*T5)*T3+(222.0D0*T76+3.0D
     &0*T84+3.0D0*T85+202.0D0*T77+210.0D0*T78+T416+T137+214.0D0*T79+208.
     &0D0*T80+3.0D0*T88+T506+216.0D0*T81+196.0D0*T82+3.0D0*T90+3.0D0*T91
     &+228.0D0*T83+39.5D0*T25-60.416666666666666667D0*T26-60.41666666666
     &6666667D0*T27-21.333333333333333333D0*T28+64.333333333333333333D0*
     &T29-26.583333333333333333D0*T30+T508+21.5D0*T38-33.763888888888888
     &889D0*T19-45.680555555555555556D0*T20-45.680555555555555556D0*T21-
     &57.597222222222222222D0*T22-56.958333333333333333D0*T4-56.95833333
     &3333333333D0*T5)*CA*T2
      T639=(T638+(-36.75D0*T76+T364+2.0D0*T85-37.25D0*T77-34.75D0*T78+T5
     &56+2.0D0*T87-39.25D0*T79-36.75D0*T80+T154+T155-37.25D0*T81-34.75D0
     &*T82+T507+T105-39.25D0*T83+5.2916666666666666667D0*T25+24.33333333
     &3333333333D0*T26+24.333333333333333333D0*T27+19.125D0*T28-17.625D0
     &*T29-0.58333333333333333333D0*T30-0.58333333333333333333D0*T31-7.7
     &916666666666666667D0*T38+23.486111111111111111D0*T19+24.4444444444
     &44444444D0*T20+24.444444444444444444D0*T21+25.402777777777777778D0
     &*T22+23.5D0*T4+23.5D0*T5)*T7*CF)*T99
      T640=((-T107-T324+T101+T100+0.46759259259259259259D0*T4-1.14814814
     &81481481482D0*T5+2.0941358024691358025D0)*T2+(T101+T325+1.11111111
     &11111111111D0*T4+1.3796296296296296296D0*T5+0.25977366255144032922
     &D0)*CA*CF)*Nf+(-0.5D0*T25+1.3333333333333333333D0*T26+0.5D0*T27-T5
     &67+T326+T327+T328+T329-11.263888888888888889D0*T19+12.694444444444
     &444444D0*T20+1.5416666666666666667D0*T21-2.7916666666666666667D0*T
     &22+8.0763888888888888889D0*T4+56.743055555555555556D0*T5-28.111111
     &111111111111D0)*T3+(0.58333333333333333333D0*T25-0.083333333333333
     &333333D0*T26+2.5833333333333333333D0*T27+0.75D0*T28+T330+T331+T332
     &+T333+20.166666666666666667D0*T19-11.333333333333333333D0*T20+6.56
     &94444444444444444D0*T21+0.33333333333333333333D0*T22-8.38425925925
     &92592592D0*T4-55.421296296296296296D0*T5+47.074845679012345679D0)*
     &CA*T2+(-0.083333333333333333333D0*T25-0.41666666666666666667D0*T26
     &-1.4166666666666666667D0*T27-0.16666666666666666667D0*T28+T248-T22
     &7-6.9722222222222222222D0*T19+2.8194444444444444444D0*T20-2.847222
     &2222222222222D0*T21+0.36111111111111111111D0*T22-2.263888888888888
     &8889D0*T4+11.74537037037037037D0*T5-21.31455761316872428D0)*T7*CF
      T641=(T334-0.037037037037037037037D0*T4+T335+0.3343621399176954732
     &5D0)*CF*T23+((-0.88888888888888888889D0*T25+0.66666666666666666667
     &D0*T27-0.55555555555555555555D0*T28+T229+T336+T230+T337-3.43518518
     &51851851852D0*T19+1.462962962962962963D0*T20+2.6296296296296296296
     &D0*T21+T231-7.0925925925925925926D0*T4+2.6759259259259259259D0*T5-
     &2.4850180041152263374D0)*T2+(T491-0.27777777777777777778D0*T26-0.3
     &3333333333333333333D0*T27+0.11111111111111111111D0*T28+T338+T339+T
     &340-T106+0.48148148148148148148D0*T19-1.6018518518518518518D0*T20-
     &2.537037037037037037D0*T21-1.0462962962962962963D0*T22+2.763888888
     &8888888889D0*T4-1.0679012345679012346D0*T5-2.5263774577046181984D0
     &)*CA*CF)*Nf+(-T102-1.6666666666666666667D0*T84-3.83333333333333333
     &33D0*T85-3.3333333333333333333D0*T77-T104-1.5D0*T86+T365-0.6666666
     &6666666666667D0*T79+T234+T238+T342+T343+T344+T345+T346+T347-8.375D
     &0*T25+T485-15.027777777777777778D0*T27+T97-4.8194444444444444444D0
     &*T29+2.75D0*T30-12.5D0*T31+T103-36.152777777777777778D0*T19-38.847
     &222222222222222D0*T20-11.847222222222222222D0*T21-26.2291666666666
     &66667D0*T22-48.621527777777777778D0*T4-46.694444444444444444D0*T5+
     &8.8671875D0)*T3
      T642=T641+(2.75D0*T76+3.6666666666666666667D0*T84+5.25D0*T85+7.75D
     &0*T77+T104+2.75D0*T86+T278+T588+T351+T352+T353+T354+T146+T355+T105
     &+T356+12.930555555555555556D0*T25+7.3611111111111111111D0*T26+6.65
     &27777777777777778D0*T27+5.0138888888888888889D0*T28+2.805555555555
     &5555555D0*T29-4.2916666666666666667D0*T30+11.472222222222222222D0*
     &T31+2.6111111111111111111D0*T38+57.768518518518518519D0*T19+28.391
     &203703703703704D0*T20+4.9328703703703703704D0*T21+34.4120370370370
     &37037D0*T22+97.918981481481481482D0*T4+22.907407407407407407D0*T5+
     &17.580407664609053498D0)*CA*T2+(-T553-1.1666666666666666667D0*T84-
     &1.6666666666666666667D0*T85-2.3333333333333333333D0*T77-0.58333333
     &333333333333D0*T78-0.33333333333333333333D0*T86-1.0D0*T87+T357+T23
     &2+T358+T359-T108+T360-4.2361111111111111111D0*T25+0.16666666666666
     &666667D0*T26+0.22222222222222222222D0*T27-1.9027777777777777778D0*
     &T28-0.375D0*T29+2.9444444444444444444D0*T30-2.8611111111111111111D
     &0*T31+T361-16.099537037037037037D0*T19-2.4398148148148148148D0*T20
     &+4.2592592592592592593D0*T21-10.9375D0*T22-33.699074074074074074D0
     &*T4+4.8823302469135802469D0*T5-5.1812700045724737083D0)*T7*CF
      T643=(T106+T107+T362+0.24074074074074074074D0*T4+T363-1.4122085048
     &010973937D0)*CF*T23+((-0.66666666666666666667D0*T84-T561-T237+T366
     &+T367+T108-T105+T368-1.3333333333333333333D0*T25+0.888888888888888
     &88889D0*T26-0.83333333333333333333D0*T27-T490-0.666666666666666666
     &67D0*T29-4.9444444444444444444D0*T30+4.8888888888888888889D0*T31+T
     &370-2.2222222222222222222D0*T19+8.2222222222222222222D0*T20-7.1388
     &888888888888889D0*T21+3.5185185185185185185D0*T22-9.62037037037037
     &03704D0*T4-5.0563271604938271605D0*T5-7.8533950617283950617D0)*T2+
     &(T241+T373-T108+T91-6.5D0*T25+T246-5.8333333333333333333D0*T27+8.0
     &D0*T28-T575+9.6111111111111111111D0*T30-8.4444444444444444444D0*T3
     &1+4.0555555555555555556D0*T38-5.8055555555555555556D0*T19+2.027777
     &7777777777778D0*T20-1.6944444444444444444D0*T21+14.601851851851851
     &852D0*T22+0.68518518518518518518D0*T4-14.941358024691358025D0*T5+2
     &8.366083676268861454D0)*CA*CF)*Nf+(-32.0D0*T110+14.0D0*T111-17.0D0
     &*T112+27.0D0*T113-18.0D0*T114+13.0D0*T115-5.0D0*T116+18.0D0*T117+T
     &593+T139+T256+T392+T140+T585+T258+T259+T375+T376+T260+T377+T254+T3
     &78+T379-T132+T380+T143+T266+T145+T381+T382-103.5D0*T76+11.5D0*T84-
     &72.666666666666666667D0*T85+54.833333333333333333D0*T77-47.6666666
     &66666666667D0*T78+54.5D0*T86-5.0D0*T87+69.666666666666666667D0*T79
     &+42.833333333333333333D0*T80+26.666666666666666667D0*T88+T268+26.3
     &33333333333333333D0*T81-2.0D0*T82+11.0D0*T90+T109-0.5D0*T83-247.91
     &666666666666667D0*T25+139.66666666666666667D0*T26-236.083333333333
     &33333D0*T27+168.75D0*T28-130.33333333333333333D0*T29+229.75D0*T30-
     &114.0D0*T31+255.58333333333333333D0*T38-228.83333333333333333D0*T1
     &9+52.833333333333333333D0*T20-214.20833333333333333D0*T21+76.33333
     &3333333333333D0*T22-102.29166666666666667D0*T4-50.0729166666666666
     &67D0*T5-2.84375D0)*T3
      T644=T643+(28.0D0*T110-15.0D0*T111+9.5D0*T112-35.5D0*T113+22.0D0*T
     &114-13.5D0*T115+T581-28.0D0*T117+5.0D0*T138+T595+T220-11.5D0*T120+
     &T257-4.5D0*T122+5.5D0*T123+T383+T384+T385+T386+T387+T261+T388+T389
     &+T132+T390+T134+T391+T212+109.0D0*T76+9.9166666666666666667D0*T84+
     &50.166666666666666667D0*T85-49.75D0*T77+49.666666666666666667D0*T7
     &8-35.333333333333333333D0*T86+T137-86.666666666666666667D0*T79-28.
     &333333333333333333D0*T80-24.833333333333333333D0*T88-18.75D0*T89-3
     &2.416666666666666667D0*T81+T235-16.083333333333333333D0*T90+14.75D
     &0*T91-31.666666666666666667D0*T83+289.25D0*T25-160.222222222222222
     &22D0*T26+257.79166666666666667D0*T27-171.20833333333333333D0*T28+1
     &66.125D0*T29-232.09722222222222222D0*T30+98.194444444444444444D0*T
     &31-257.0D0*T38+335.30555555555555556D0*T19-61.513888888888888889D0
     &*T20+335.34722222222222222D0*T21-65.865740740740740741D0*T22+186.3
     &5648148148148148D0*T4+71.47183641975308642D0*T5+77.092978395061728
     &395D0)*CA*T2+(-6.0D0*T110+4.5D0*T111-0.5D0*T112+10.5D0*T113+T580+4
     &.0D0*T115-3.5D0*T116+7.0D0*T117+T582+T139+1.5D0*T120+T584+T140-1.0
     &D0*T123+T393+T394+T395+T396+T128+T397+T150+T398+T399+T400+T143+T15
     &2+T145+T401-28.5D0*T76-5.75D0*T84-7.1666666666666666667D0*T85+14.8
     &33333333333333333D0*T77-13.666666666666666667D0*T78+8.75D0*T86+T15
     &3+24.666666666666666667D0*T79+2.5833333333333333333D0*T80+11.25D0*
     &T88+5.25D0*T89+10.833333333333333333D0*T81+T146+9.0833333333333333
     &333D0*T90-5.75D0*T91+T236-61.041666666666666667D0*T25+17.291666666
     &666666667D0*T26-51.75D0*T27+26.25D0*T28-31.958333333333333333D0*T2
     &9+36.430555555555555556D0*T30-3.2222222222222222222D0*T31+71.55555
     &5555555555556D0*T38-88.805555555555555556D0*T19-2.4444444444444444
     &444D0*T20-98.722222222222222222D0*T21-39.231481481481481481D0*T22-
     &55.884259259259259259D0*T4+22.591049382716049383D0*T5-100.73705418
     &381344307D0)*T7*CF
      T645=(-4.6666666666666666667D0*T110-3.3333333333333333333D0*T111-3
     &.3333333333333333333D0*T112-2.6666666666666666667D0*T113-1.3333333
     &333333333333D0*T114-1.3333333333333333333D0*T117+1.333333333333333
     &3333D0*T138+1.3333333333333333333D0*T118+1.3333333333333333333D0*T
     &119-0.33333333333333333333D0*T120+T407+T272+T273+T274+T408+T276+T1
     &51+T263+T409-14.888888888888888889D0*T76-11.888888888888888889D0*T
     &84-11.888888888888888889D0*T85-6.3333333333333333333D0*T77-2.83333
     &33333333333333D0*T78+T86+T87+T233+T148+T520+T521-0.944444444444444
     &44445D0*T81-4.8888888888888888889D0*T82+T410-27.388888888888888889
     &D0*T25-24.777777777777777778D0*T26-24.777777777777777778D0*T27-10.
     &555555555555555556D0*T28+1.3055555555555555556D0*T29+0.16666666666
     &666666667D0*T30+T568+7.1666666666666666667D0*T38-2.138888888888888
     &8889D0*T19-T149-7.0D0*T21-6.7229938271604938272D0*T22-8.0551697530
     &864197531D0*T4-14.388888888888888889D0*T5+7.4765946502057613169D0)
     &*T2+(2.3333333333333333333D0*T110+1.6666666666666666667D0*T111+1.6
     &666666666666666667D0*T112+T113+0.66666666666666666667D0*T114-0.666
     &66666666666666667D0*T117-0.66666666666666666667D0*T138-0.666666666
     &66666666667D0*T118-0.66666666666666666667D0*T119-0.666666666666666
     &66667D0*T120+T265+T412+T413+T414+T150+T415-T151+T152+14.7777777777
     &77777778D0*T76+5.9444444444444444444D0*T84+5.9444444444444444444D0
     &*T85+11.611111111111111111D0*T77+7.6666666666666666667D0*T78-0.5D0
     &*T86+T153+6.5D0*T79+T589-T154-T155+7.6111111111111111111D0*T81+8.4
     &444444444444444444D0*T82-0.83333333333333333333D0*T83+22.305555555
     &555555556D0*T25+12.388888888888888889D0*T26+12.388888888888888889D
     &0*T27+11.416666666666666667D0*T28+4.6111111111111111111D0*T29-0.08
     &3333333333333333333D0*T30+T591+19.87037037037037037D0*T38+6.712962
     &962962962963D0*T19+3.5D0*T20+3.5D0*T21-14.441358024691358025D0*T22
     &+4.4382716049382716049D0*T4+10.583333333333333333D0*T5+24.10148986
     &4349946654D0)*CA*CF
      T646=38.0D0*T156+6.0D0*T157+6.0D0*T158+20.0D0*T159+28.0D0*T160+11.
     &0D0*T161+11.0D0*T162+38.0D0*T163+T599-6.0D0*T165-6.0D0*T166+7.0D0*
     &T167-1.0D0*T168-T457-T458+12.0D0*T171-10.0D0*T172-10.0D0*T173-10.0
     &D0*T174-2.0D0*T175+T298+T459+T460+6.0D0*T179-12.0D0*T180-6.0D0*T18
     &1-6.0D0*T182-4.0D0*T183+7.0D0*T184+T299+T300+T301+T419+T189+T190+T
     &302+T217+T420+T421+T422+T289+T290+T423+T424+T425+T426+T427+T428+T2
     &91+T292-T306+T207+T429+T430+67.5D0*T110-33.0D0*T111-33.0D0*T112-18
     &.5D0*T113+50.5D0*T114-19.166666666666666667D0*T115-19.166666666666
     &666667D0*T116+35.666666666666666667D0*T117+T526-23.666666666666666
     &667D0*T118-23.666666666666666667D0*T119+30.833333333333333333D0*T1
     &20+9.3333333333333333333D0*T121+4.3333333333333333333D0*T122+4.333
     &3333333333333333D0*T123+74.0D0*T209-100.5D0*T124-57.66666666666666
     &6667D0*T125-57.666666666666666667D0*T126-31.0D0*T127-23.8333333333
     &33333333D0*T128-7.8333333333333333333D0*T129
      T647=(T646-7.8333333333333333333D0*T130+20.0D0*T141-T527-T210-T211
     &-20.0D0*T134+0.5D0*T144+T547+T548+166.0D0*T76-72.16666666666666666
     &7D0*T84-72.166666666666666667D0*T85+78.0D0*T77+173.666666666666666
     &67D0*T78-51.916666666666666667D0*T86-51.916666666666666667D0*T87+1
     &19.83333333333333333D0*T79+145.5D0*T80+9.9166666666666666667D0*T88
     &+9.9166666666666666667D0*T89+239.66666666666666667D0*T81+116.16666
     &666666666667D0*T82+2.1666666666666666667D0*T90+2.16666666666666666
     &67D0*T91+257.75D0*T83+55.583333333333333333D0*T25-163.5D0*T26-163.
     &5D0*T27-97.166666666666666667D0*T28+72.958333333333333333D0*T29-12
     &7.75D0*T30-127.75D0*T31-39.416666666666666667D0*T38+116.0416666666
     &6666667D0*T19+T94+23.5D0*T21-14.572916666666666667D0*T22-34.255208
     &333333333333D0*T4-27.458333333333333333D0*T5-17.609375D0)*T3
      T648=-29.0D0*T156+T213+T214-16.0D0*T159-18.0D0*T160-8.5D0*T161-8.5
     &D0*T162-44.0D0*T163-9.0D0*T164+T618+T619-0.5D0*T167+1.5D0*T168+11.
     &0D0*T169+11.0D0*T170-17.0D0*T171+6.0D0*T172+11.0D0*T173+11.0D0*T17
     &4+7.0D0*T175+T621+1.5D0*T177+1.5D0*T178-10.0D0*T179+13.0D0*T180+9.
     &0D0*T181+9.0D0*T182+4.5D0*T183-5.5D0*T184+T434+T435+T436+T437-T189
     &-T190+T438+T439+T440+T441+T195+T442+T443+T304+T444+T445+T446+T447+
     &T448+T449+T450+T451+T307+T308-T207+T452-35.833333333333333333D0*T1
     &10+64.333333333333333333D0*T111+64.333333333333333333D0*T112+65.41
     &6666666666666667D0*T113-18.416666666666666667D0*T114+22.9166666666
     &66666667D0*T115+22.916666666666666667D0*T116-28.333333333333333333
     &D0*T117-32.833333333333333333D0*T138+16.833333333333333333D0*T118+
     &16.833333333333333333D0*T119-18.5D0*T120+2.1666666666666666667D0*T
     &121+5.1666666666666666667D0*T122+5.1666666666666666667D0*T123-81.5
     &D0*T209+87.5D0*T124+59.166666666666666667D0*T125+59.16666666666666
     &6667D0*T126
      T649=(T648+34.333333333333333333D0*T127+26.166666666666666667D0*T1
     &28+7.4166666666666666667D0*T129+7.4166666666666666667D0*T130-30.16
     &6666666666666667D0*T141+43.333333333333333333D0*T131+T210+T211+14.
     &916666666666666667D0*T134-8.75D0*T144+T251+T212+T454-133.694444444
     &44444444D0*T76+126.47222222222222222D0*T84+126.47222222222222222D0
     &*T85-64.666666666666666667D0*T77-195.33333333333333333D0*T78+31.54
     &1666666666666667D0*T86+31.541666666666666667D0*T87-149.25D0*T79-21
     &6.75D0*T80-34.791666666666666667D0*T88-34.791666666666666667D0*T89
     &-266.88888888888888889D0*T81-91.361111111111111111D0*T82+6.8333333
     &333333333333D0*T90+6.8333333333333333333D0*T91-255.666666666666666
     &67D0*T83-2.4027777777777777778D0*T25+302.56944444444444444D0*T26+3
     &02.56944444444444444D0*T27+201.30555555555555556D0*T28-210.9722222
     &2222222222D0*T29+84.625D0*T30+84.625D0*T31-3.75D0*T38-102.98611111
     &111111111D0*T19+50.625D0*T20+50.625D0*T21+86.013503086419753087D0*
     &T22+98.659915123456790123D0*T4+117.43055555555555556D0*T5+53.80169
     &7530864197531D0)*CA*T2
      T650=5.0D0*T156+T213+T214+3.5D0*T159+2.0D0*T160+T616+T617+12.0D0*T
     &163+1.5D0*T164-5.0D0*T165-5.0D0*T166-1.0D0*T167-T620+T285+T286+3.0
     &D0*T171-0.5D0*T172-3.0D0*T173-3.0D0*T174-3.0D0*T175+T176+1.5D0*T17
     &9-3.5D0*T180-3.0D0*T181-3.0D0*T182-1.0D0*T183+T184+T461+T462+T463+
     &T464+T465+T466+T467+T217+T468+T469+T470+T471+T472+T473+T474+T475+T
     &476+T477+T478+T479+T208+0.91666666666666666667D0*T110-23.916666666
     &666666667D0*T111-23.916666666666666667D0*T112-26.0D0*T113-3.166666
     &6666666666667D0*T114-6.6666666666666666667D0*T115-6.66666666666666
     &66667D0*T116+11.833333333333333333D0*T117+11.166666666666666667D0*
     &T138-2.5D0*T118-T220+6.25D0*T120-3.1666666666666666667D0*T121-3.66
     &66666666666666667D0*T122-3.6666666666666666667D0*T123+21.0D0*T209-
     &17.75D0*T124-15.166666666666666667D0*T125-15.166666666666666667D0*
     &T126-3.9166666666666666667D0*T127-7.0D0*T128-1.75D0*T129-1.75D0*T1
     &30+12.75D0*T141-13.166666666666666667D0*T131
      T651=(T650-7.75D0*T142-7.75D0*T133+T151+3.75D0*T144-2.0D0*T135-T26
     &4+3.0138888888888888889D0*T76-45.194444444444444444D0*T84-45.19444
     &4444444444444D0*T85-15.069444444444444444D0*T77+36.125D0*T78-2.791
     &6666666666666667D0*T86-2.7916666666666666667D0*T87+28.958333333333
     &333333D0*T79+54.208333333333333333D0*T80+14.916666666666666667D0*T
     &88+14.916666666666666667D0*T89+51.347222222222222222D0*T81-0.73611
     &111111111111111D0*T82-3.9583333333333333333D0*T90-3.95833333333333
     &33333D0*T91+81.041666666666666667D0*T83-37.493055555555555556D0*T2
     &5-110.40972222222222222D0*T26-110.40972222222222222D0*T27-96.35416
     &6666666666667D0*T28+71.847222222222222222D0*T29-10.375D0*T30-10.37
     &5D0*T31-56.00462962962962963D0*T38+8.0532407407407407407D0*T19-31.
     &1875D0*T20-31.1875D0*T21+7.9035493827160493827D0*T22-41.6959876543
     &20987654D0*T4-65.125D0*T5-95.684111130163084895D0)*T7*CF
      T652=(T309+(T310+(-0.041666666666666666667D0*T4+T311+0.72291666666
     &666666667D0)*T3+(0.0625D0*T4+T312-1.1405092592592592593D0)*CA*T2+(
     &T313+0.39305555555555555556D0)*T7*CF)*im*T222+(T314+((-0.096296296
     &296296296296D0*T4+T315-0.16170267489711934156D0)*T2+(0.01388888888
     &8888888889D0*T4+T316-0.71286008230452674897D0)*CA*CF)*Nf+(-0.81388
     &888888888888889D0*T19+0.38611111111111111111D0*T20+T317+T318-4.035
     &0694444444444445D0*T4+2.6291666666666666667D0*T5-28.14270833333333
     &3333D0)*T3+(0.59583333333333333333D0*T19-0.28472222222222222222D0*
     &T20+T319+T320+4.7101851851851851852D0*T4-2.268287037037037037D0*T5
     &+32.643531378600823045D0)*CA*T2+(-0.073611111111111111111D0*T19+0.
     &094444444444444444444D0*T20+T321+T322-1.2513888888888888889D0*T4+0
     &.41898148148148148148D0*T5-6.9030992798353909465D0)*T7*CF)*T93+T64
     &0*im*T34+T642*T35+T644*im*pi+(T404-0.33333333333333333333D0*T25+T4
     &05-0.37037037037037037037D0*T19+T406+0.077160493827160493827D0*T4-
     &1.1922629934461210181D0)*CF*T23+T645*Nf+T647+T649+T651)*T13
      T653=(0.19394657554379776602D0*T3-0.34013080540858318636D0*CA*T2+0
     &.1370125416421712718D0*T7*CF)*T221+((-0.051851851851851851852D0*T2
     &+0.027777777777777777778D0*CA*CF)*Nf+(-1.2666666666666666667D0*T4+
     &0.44861111111111111111D0*T5-3.10625D0)*T3+(1.5833333333333333333D0
     &*T4-0.33888888888888888889D0*T5+4.3324074074074074074D0)*CA*T2+(-0
     &.51666666666666666667D0*T4-0.016666666666666666667D0*T5-1.35D0)*T7
     &*CF)*im*T222+(-0.0088477366255144032922D0*CF*T23+((0.3592592592592
     &5925926D0*T4-0.02962962962962962963D0*T5+0.43451646090534979424D0)
     &*T2+(-T223-0.0037037037037037037037D0*T5+0.18775720164609053498D0)
     &*CA*CF)*Nf+(4.1444444444444444445D0*T19-4.8055555555555555556D0*T2
     &0-1.3555555555555555556D0*T21+1.1375D0*T22+14.433333333333333333D0
     &*T4-9.6333333333333333333D0*T5+30.114583333333333333D0)*T3+(-3.166
     &6666666666666667D0*T19+4.9694444444444444444D0*T20+1.6222222222222
     &222222D0*T21-0.55555555555555555555D0*T22-18.159259259259259259D0*
     &T4+9.5379629629629629629D0*T5-35.546090534979423868D0)*CA*T2+(0.50
     &555555555555555555D0*T19-1.3805555555555555556D0*T20-0.63888888888
     &888888889D0*T21-0.12222222222222222222D0*T22+5.3972222222222222222
     &D0*T4-2.1962962962962962963D0*T5+8.9274948559670781893D0)*T7*CF)*T
     &93
      T654=((0.22222222222222222222D0*T19+T323+T224-0.833333333333333333
     &33D0*T22-0.51851851851851851852D0*T4+2.4074074074074074074D0*T5-2.
     &3827160493827160494D0)*T2+(T224+0.22222222222222222222D0*T22-0.777
     &77777777777777778D0*T4-1.037037037037037037D0*T5+0.045267489711934
     &156379D0)*CA*CF)*Nf+(6.0D0*T25-21.666666666666666667D0*T26-5.0D0*T
     &27+10.333333333333333333D0*T28-7.3333333333333333333D0*T29+4.16666
     &66666666666667D0*T30+T498-1.5D0*T38+57.777777777777777778D0*T19-56
     &.722222222222222222D0*T20-12.833333333333333333D0*T21+16.75D0*T22+
     &92.375D0*T4-76.875D0*T5+122.53472222222222222D0)*T3+(-4.6666666666
     &666666667D0*T25+12.666666666666666667D0*T26-14.166666666666666667D
     &0*T27-9.5D0*T28+11.0D0*T29-3.8333333333333333333D0*T30-T555+T225-9
     &0.166666666666666667D0*T19+50.75D0*T20-12.472222222222222222D0*T21
     &-13.416666666666666667D0*T22-136.67592592592592593D0*T4+69.1620370
     &37037037037D0*T5-175.42746913580246914D0)*CA*T2+(0.666666666666666
     &66667D0*T25-0.66666666666666666667D0*T26+8.3333333333333333333D0*T
     &27+T564-T65+T269+30.027777777777777778D0*T19-12.138888888888888889
     &D0*T20+7.6111111111111111111D0*T21+2.7777777777777777778D0*T22+49.
     &361111111111111111D0*T4-16.935185185185185185D0*T5+61.626800411522
     &633745D0)*T7*CF
      T655=(-0.14814814814814814815D0*T22+0.46913580246913580247D0*T5-0.
     &90329218106995884774D0)*CF*T23+((7.1111111111111111111D0*T25-3.333
     &3333333333333333D0*T26-T226+2.4444444444444444444D0*T28-0.44444444
     &444444444444D0*T29+T227+1.1111111111111111111D0*T31+0.555555555555
     &55555555D0*T38+16.259259259259259259D0*T19-9.4814814814814814815D0
     &*T20-10.481481481481481481D0*T21+T371+20.092592592592592593D0*T4-5
     &.6296296296296296296D0*T5+6.0718878600823045267D0)*T2+(-3.55555555
     &55555555555D0*T25+2.2222222222222222222D0*T26+T228-0.8888888888888
     &8888889D0*T28+T229+0.22222222222222222222D0*T30+T230+T240-7.796296
     &2962962962963D0*T19+4.3703703703703703704D0*T20+5.5740740740740740
     &741D0*T21-T231-9.1296296296296296296D0*T4+1.5987654320987654321D0*
     &T5+6.2302240512117055327D0)*CA*CF)*Nf+(8.0D0*T76+17.33333333333333
     &3333D0*T84+32.666666666666666667D0*T85+13.666666666666666667D0*T77
     &+20.666666666666666667D0*T78+T587+T607+3.3333333333333333333D0*T79
     &+2.0D0*T80-3.3333333333333333333D0*T88+T232+5.5D0*T81+4.3333333333
     &333333333D0*T82+6.6666666666666666667D0*T90-5.3333333333333333333D
     &0*T91+8.5D0*T83+36.666666666666666667D0*T25+47.555555555555555556D
     &0*T26+56.888888888888888889D0*T27-17.166666666666666667D0*T28+22.7
     &22222222222222222D0*T29+T496+46.0D0*T31-T225+38.902777777777777778
     &D0*T19+35.916666666666666667D0*T20-28.5D0*T21-16.04166666666666666
     &7D0*T22+72.486111111111111111D0*T4+53.638888888888888889D0*T5-40.6
     &02430555555555556D0)*T3
      T656=T655+(-10.0D0*T76-27.333333333333333333D0*T84-36.0D0*T85-32.0
     &D0*T77-15.666666666666666667D0*T78-10.0D0*T86-23.33333333333333333
     &3D0*T87+T233+T234-3.6666666666666666667D0*T88-4.666666666666666666
     &7D0*T89-6.1666666666666666667D0*T81+T235-10.666666666666666667D0*T
     &90-T243+T236-68.777777777777777778D0*T25-16.555555555555555556D0*T
     &26-8.7222222222222222222D0*T27-9.3611111111111111111D0*T28-15.8611
     &11111111111111D0*T29+6.1666666666666666667D0*T30-45.11111111111111
     &1111D0*T31-6.5555555555555555556D0*T38-110.27314814814814815D0*T19
     &+46.800925925925925926D0*T20+90.175925925925925926D0*T21+6.1342592
     &592592592593D0*T22-183.90509259259259259D0*T4+8.226851851851851851
     &8D0*T5-18.290444958847736626D0)*CA*T2+(T489+8.8333333333333333333D
     &0*T84+9.8333333333333333333D0*T85+11.166666666666666667D0*T77+T493
     &+T237+7.5D0*T87+T238+2.6666666666666666667D0*T89+T81+0.66666666666
     &666666667D0*T82+T242+T531+T514-4.8333333333333333333D0*T26-9.86111
     &11111111111111D0*T27+7.0555555555555555556D0*T28+2.25D0*T29-4.9722
     &222222222222222D0*T30+11.055555555555555556D0*T31+T374+44.58796296
     &2962962963D0*T19-30.231481481481481481D0*T20-38.462962962962962963
     &D0*T21+T239+70.310185185185185185D0*T4-25.415123456790123457D0*T5+
     &6.3640832190214906264D0)*T7*CF
      T657=196.0D0*T110-100.0D0*T111+88.0D0*T112-204.0D0*T113+96.0D0*T11
     &4-140.0D0*T115+T253-204.0D0*T117-12.0D0*T138-70.0D0*T118-10.0D0*T1
     &19-72.0D0*T120-24.0D0*T121-28.0D0*T122+26.0D0*T123-16.0D0*T124-20.
     &0D0*T125-28.0D0*T126-4.0D0*T127-4.0D0*T128+2.0D0*T129+8.0D0*T130+6
     &.0D0*T141+60.0D0*T131+T255+20.0D0*T133+T262-8.0D0*T144-T251+16.0D0
     &*T136-30.0D0*T147+388.0D0*T76+30.0D0*T84+261.33333333333333333D0*T
     &85-168.66666666666666667D0*T77+135.33333333333333333D0*T78-197.0D0
     &*T86+T252-315.33333333333333333D0*T79-152.66666666666666667D0*T80-
     &105.33333333333333333D0*T88-56.0D0*T89-105.66666666666666667D0*T81
     &+12.0D0*T82-30.0D0*T90+T284+9.0D0*T83+357.16666666666666667D0*T25+
     &192.33333333333333333D0*T26+314.5D0*T27+37.666666666666666667D0*T2
     &8-47.5D0*T29-149.5D0*T30-85.5D0*T31-208.5D0*T38+403.66666666666666
     &667D0*T19+126.91666666666666667D0*T20+370.33333333333333333D0*T21+
     &40.833333333333333333D0*T22+194.70833333333333333D0*T4
      T658=-164.0D0*T110+120.0D0*T111-28.0D0*T112+260.0D0*T113-104.0D0*T
     &114+144.0D0*T115-T253+236.0D0*T117-16.0D0*T138+35.0D0*T118+T119+62
     &.0D0*T120+T500+33.0D0*T122-23.0D0*T123+24.0D0*T124+26.0D0*T125+26.
     &0D0*T126+18.0D0*T127+6.0D0*T128+12.0D0*T129+T254-54.0D0*T131-T255-
     &26.0D0*T133+T143+T453-T431-418.0D0*T76-109.33333333333333333D0*T84
     &-196.33333333333333333D0*T85+142.0D0*T77-138.33333333333333333D0*T
     &78+135.66666666666666667D0*T86+41.0D0*T87+350.33333333333333333D0*
     &T79+101.66666666666666667D0*T80+84.666666666666666667D0*T88+52.0D0
     &*T89+99.333333333333333333D0*T81+T403+34.666666666666666667D0*T90-
     &49.0D0*T91+51.333333333333333333D0*T83-431.16666666666666667D0*T25
     &-310.55555555555555556D0*T26-303.0D0*T27-137.0D0*T28+34.25D0*T29+9
     &5.111111111111111111D0*T30+187.86111111111111111D0*T31+165.3333333
     &3333333333D0*T38-605.20833333333333333D0*T19-214.54166666666666667
     &D0*T20-581.69444444444444444D0*T21-100.26851851851851852D0*T22-410
     &.80555555555555556D0*T4
      T659=(-T240+1.4074074074074074074D0*T22+0.22222222222222222222D0*T
     &4-2.5061728395061728395D0*T5+4.0775034293552812071D0)*CF*T23+((T56
     &0+T492+T267+T241-T574-T242+T243-9.3333333333333333333D0*T83+3.5555
     &555555555555555D0*T26+7.3333333333333333333D0*T28+T249-11.77777777
     &7777777778D0*T31+14.666666666666666667D0*T38+6.3333333333333333333
     &D0*T19-T244+20.111111111111111111D0*T21-6.037037037037037037D0*T22
     &+27.805555555555555556D0*T4+3.6682098765432098765D0*T5+28.65046296
     &2962962963D0)*T2+(T578+2.0D0*T81+T242-T105+T245-T246+T247-5.666666
     &6666666666667D0*T28+T248-T249+7.8888888888888888889D0*T31+2.888888
     &8888888888889D0*T38+0.5D0*T19+T250-6.7222222222222222222D0*T21-21.
     &37037037037037037D0*T22-13.083333333333333333D0*T4+30.466049382716
     &049383D0*T5-77.895747599451303155D0)*CA*CF)*Nf+(T657+51.4791666666
     &66666667D0*T5+40.447916666666666667D0)*T3+(T658-42.499228395061728
     &395D0*T5-270.71412037037037037D0)*CA*T2+(33.0D0*T110-36.0D0*T111-8
     &.0D0*T112-78.0D0*T113+T504-38.0D0*T115+4.0D0*T116-62.0D0*T117+T411
     &+T583-T256+T257-10.0D0*T122+5.0D0*T123-T258-T259-6.0D0*T126-6.0D0*
     &T127-T260+T261+T275+12.0D0*T131+6.0D0*T142+8.0D0*T133+T262+T263-T2
     &51+T264+112.0D0*T76+43.5D0*T84+32.833333333333333333D0*T85-38.6666
     &66666666666667D0*T77+35.333333333333333333D0*T78-28.0D0*T86+T551-9
     &6.333333333333333333D0*T79-12.666666666666666667D0*T80-27.0D0*T88-
     &12.0D0*T89-27.666666666666666667D0*T81-17.166666666666666667D0*T90
     &+18.5D0*T91+121.66666666666666667D0*T25+131.08333333333333333D0*T2
     &6+67.75D0*T27+64.583333333333333333D0*T28-10.25D0*T29+4.6388888888
     &888888889D0*T30-79.555555555555555556D0*T31-76.194444444444444444D
     &0*T38+190.33333333333333333D0*T19+74.333333333333333333D0*T20+190.
     &68055555555555556D0*T21+117.75462962962962963D0*T22+151.6111111111
     &1111111D0*T4-72.682098765432098765D0*T5+282.12071330589849108D0)*T
     &7*CF
      T660=(T515+T516+T517+17.333333333333333333D0*T113+10.6666666666666
     &66667D0*T114-T271-10.666666666666666667D0*T138+T294+T295-7.3333333
     &333333333333D0*T120+T265+2.6666666666666666667D0*T128+T604+T605-1.
     &3333333333333333333D0*T141+2.6666666666666666667D0*T131-T277-T266-
     &13.333333333333333333D0*T147+71.111111111111111111D0*T76+55.111111
     &111111111111D0*T84+55.111111111111111111D0*T85+42.6666666666666666
     &67D0*T77+12.666666666666666667D0*T78-T267-3.3333333333333333333D0*
     &T87+T417-22.666666666666666667D0*T80-16.0D0*T88-T268-7.77777777777
     &77777778D0*T81+11.777777777777777778D0*T82+T279+63.666666666666666
     &667D0*T25+70.333333333333333333D0*T26+70.333333333333333333D0*T27+
     &58.0D0*T28-12.777777777777777778D0*T29+T269+1.3333333333333333333D
     &0*T31-5.0D0*T38-12.805555555555555556D0*T19+T270+18.0D0*T21+17.001
     &54320987654321D0*T22-14.333333333333333333D0*T4+11.111111111111111
     &111D0*T5-14.425411522633744856D0)*T2+(-18.666666666666666667D0*T11
     &0-13.333333333333333333D0*T111-13.333333333333333333D0*T112-8.0D0*
     &T113-5.3333333333333333333D0*T114+T271+5.3333333333333333333D0*T13
     &8+5.3333333333333333333D0*T118+5.3333333333333333333D0*T119+5.3333
     &333333333333333D0*T120+2.6666666666666666667D0*T127+T272+T273+T274
     &+T275+T276+T277+T263-37.555555555555555556D0*T76-27.55555555555555
     &5556D0*T84-27.555555555555555556D0*T85-27.555555555555555556D0*T77
     &-8.3333333333333333333D0*T78+T237+T278+2.0D0*T79+9.333333333333333
     &3333D0*T80+T512+T513-0.22222222222222222222D0*T81-7.88888888888888
     &88889D0*T82+T279-35.333333333333333333D0*T25-35.166666666666666667
     &D0*T26-35.166666666666666667D0*T27-30.166666666666666667D0*T28+3.0
     &555555555555555555D0*T29-T227-0.66666666666666666667D0*T31-35.7407
     &40740740740741D0*T38+8.9722222222222222222D0*T19-T280-T281+24.4660
     &49382716049383D0*T22+21.037037037037037037D0*T4-12.416666666666666
     &667D0*T5-48.202979728699893309D0)*CA*CF
      T661=-244.0D0*T156-48.0D0*T157-48.0D0*T158-148.0D0*T159-164.0D0*T1
     &60-76.0D0*T161-76.0D0*T162-280.0D0*T163+T523+48.0D0*T165+48.0D0*T1
     &66-92.0D0*T167-40.0D0*T168-24.0D0*T169-24.0D0*T170-228.0D0*T171+92
     &.0D0*T172+80.0D0*T173+80.0D0*T174+10.0D0*T175+22.0D0*T176+12.0D0*T
     &177+12.0D0*T178-60.0D0*T179+96.0D0*T180+72.0D0*T181+72.0D0*T182+44
     &.0D0*T183-26.0D0*T184-16.0D0*T185-16.0D0*T186-36.0D0*T187+12.0D0*T
     &188-T287-T288-20.0D0*T191+T303+4.0D0*T193+4.0D0*T194+6.0D0*T195+8.
     &0D0*T196+8.0D0*T197+14.0D0*T198-100.0D0*T199-40.0D0*T200-40.0D0*T2
     &01-24.0D0*T202-24.0D0*T203-4.0D0*T204-4.0D0*T205+8.0D0*T206-T293-1
     &6.0D0*T208-30.0D0*T283-204.0D0*T110+160.0D0*T111+160.0D0*T112+166.
     &0D0*T113-128.0D0*T114+109.33333333333333333D0*T115+109.33333333333
     &333333D0*T116-77.333333333333333333D0*T117-50.0D0*T138+85.33333333
     &3333333333D0*T118+85.333333333333333333D0*T119-111.666666666666666
     &67D0*T120
      T662=(T661-60.666666666666666667D0*T121-74.666666666666666667D0*T1
     &22-74.666666666666666667D0*T123-390.0D0*T209+354.0D0*T124+201.3333
     &3333333333333D0*T125+201.33333333333333333D0*T126+96.0D0*T127+86.6
     &66666666666666667D0*T128+30.666666666666666667D0*T129+30.666666666
     &666666667D0*T130-78.0D0*T141+102.0D0*T131+108.0D0*T142+108.0D0*T13
     &3+78.0D0*T134-24.0D0*T144-97.5D0*T76+223.66666666666666667D0*T84+2
     &23.66666666666666667D0*T85+380.0D0*T77-116.16666666666666667D0*T78
     &+162.33333333333333333D0*T86+162.33333333333333333D0*T87+173.0D0*T
     &79+T148-25.5D0*T88-25.5D0*T89-175.0D0*T81+61.5D0*T82-24.0D0*T90-T2
     &84-232.5D0*T83-173.16666666666666667D0*T25+231.25D0*T26+231.25D0*T
     &27+344.66666666666666667D0*T28-252.83333333333333333D0*T29+104.0D0
     &*T30+104.0D0*T31+135.83333333333333333D0*T38-297.375D0*T19-98.1666
     &66666666666667D0*T20-98.166666666666666667D0*T21-57.1875D0*T22-26.
     &0D0*T4-5.2083333333333333333D0*T5+36.46875D0)*T3
      T663=172.0D0*T156+T296+T297+128.0D0*T159+84.0D0*T160+56.0D0*T161+5
     &6.0D0*T162+316.0D0*T163+24.0D0*T164-80.0D0*T165-80.0D0*T166+64.0D0
     &*T167+12.0D0*T168+T285+T286+232.0D0*T171-60.0D0*T172-76.0D0*T173-7
     &6.0D0*T174-41.0D0*T175-7.0D0*T176-T542-T543+56.0D0*T179-104.0D0*T1
     &80-84.0D0*T181-84.0D0*T182-51.0D0*T183+T602+24.0D0*T185+24.0D0*T18
     &6+50.0D0*T187-10.0D0*T188+T287+T288+34.0D0*T191-20.0D0*T192-14.0D0
     &*T193-14.0D0*T194-2.0D0*T195+T289+T290-4.0D0*T198+98.0D0*T199+44.0
     &D0*T200+44.0D0*T201+28.0D0*T202+28.0D0*T203+T291+T292-12.0D0*T206-
     &12.0D0*T218-12.0D0*T219+T293+12.0D0*T208+22.666666666666666667D0*T
     &110-324.66666666666666667D0*T111-324.66666666666666667D0*T112-363.
     &33333333333333333D0*T113+10.333333333333333333D0*T114-115.33333333
     &333333333D0*T115-115.33333333333333333D0*T116+82.66666666666666666
     &7D0*T117+127.66666666666666667D0*T138+T294+T295+125.0D0*T120+1.666
     &6666666666666667D0*T121+42.666666666666666667D0*T122
      T664=T663+42.666666666666666667D0*T123+393.0D0*T209-301.0D0*T124-1
     &99.33333333333333333D0*T125-199.33333333333333333D0*T126-114.66666
     &666666666667D0*T127-85.333333333333333333D0*T128-33.33333333333333
     &3333D0*T129-33.333333333333333333D0*T130+76.333333333333333333D0*T
     &141-118.66666666666666667D0*T131-107.0D0*T142-107.0D0*T133-72.3333
     &33333333333333D0*T134+49.0D0*T144+73.333333333333333333D0*T147-126
     &.44444444444444444D0*T76-451.61111111111111111D0*T84-451.611111111
     &11111111D0*T85-656.16666666666666667D0*T77+140.5D0*T78-56.5D0*T86-
     &56.5D0*T87-114.0D0*T79+162.66666666666666667D0*T80+152.75D0*T88+15
     &2.75D0*T89+247.86111111111111111D0*T81-175.36111111111111111D0*T82
     &+12.5D0*T90+12.5D0*T91+188.83333333333333333D0*T83+32.083333333333
     &333333D0*T25-575.70833333333333333D0*T26-575.70833333333333333D0*T
     &27-750.5D0*T28+525.52777777777777778D0*T29-16.416666666666666667D0
     &*T30-16.416666666666666667D0*T31-115.08333333333333333D0*T38+369.1
     &8055555555555556D0*T19-58.125D0*T20-58.125D0*T21-68.20756172839506
     &1729D0*T22
      T665=-25.0D0*T156+T296+T297-28.0D0*T159-1.0D0*T160+T538+T539-87.0D
     &0*T163+T600+T601-10.0D0*T167+4.0D0*T168+8.0D0*T169+8.0D0*T170-54.0
     &D0*T171+7.0D0*T172+18.0D0*T173+18.0D0*T174+T418+T298-9.0D0*T179+28
     &.0D0*T180+24.0D0*T181+24.0D0*T182+14.0D0*T183-5.0D0*T184-T299-T300
     &-T301+2.0D0*T188-4.0D0*T215-4.0D0*T216-T302+T303+6.0D0*T193+6.0D0*
     &T194+T304-24.0D0*T199-12.0D0*T200-12.0D0*T201-6.0D0*T202-8.0D0*T20
     &3+4.0D0*T305+T306+T307+T308-2.0D0*T208+39.666666666666666667D0*T11
     &0+122.33333333333333333D0*T111+122.33333333333333333D0*T112+136.5D
     &0*T113+26.833333333333333333D0*T114+30.333333333333333333D0*T115+3
     &0.333333333333333333D0*T116-37.666666666666666667D0*T117-51.333333
     &333333333333D0*T138-16.0D0*T118-16.0D0*T119-44.0D0*T120+14.3333333
     &33333333333D0*T121+T611+T612-99.0D0*T209+62.0D0*T124+49.3333333333
     &33333333D0*T125+49.333333333333333333D0*T126+22.333333333333333333
     &D0*T127+21.0D0*T128+9.0D0*T129+9.0D0*T130
      T666=(T665-26.0D0*T141+33.833333333333333333D0*T131+26.5D0*T142+26
     &.5D0*T133+9.3333333333333333333D0*T134-18.5D0*T144+92.222222222222
     &222222D0*T76+169.88888888888888889D0*T84+169.88888888888888889D0*T
     &85+256.97222222222222222D0*T77-T350-12.333333333333333333D0*T86-12
     &.333333333333333333D0*T87+19.25D0*T79-78.083333333333333333D0*T80-
     &70.0D0*T88-70.0D0*T89-65.361111111111111111D0*T81+79.3055555555555
     &55556D0*T82-0.25D0*T90-0.25D0*T91-103.33333333333333333D0*T83+37.8
     &75D0*T25+230.04166666666666667D0*T26+230.04166666666666667D0*T27+2
     &87.875D0*T28-191.97222222222222222D0*T29-17.791666666666666667D0*T
     &30-17.791666666666666667D0*T31+129.25925925925925926D0*T38-111.451
     &38888888888889D0*T19+53.604166666666666667D0*T20+53.60416666666666
     &6667D0*T21-35.577932098765432099D0*T22-67.75462962962962963D0*T4+5
     &9.75D0*T5+191.36822226032616979D0)*T7*CF
      T667=(T653+T654*im*T34+T656*T35+T659*im*pi+(-1.3333333333333333333
     &D0*T83+2.8148148148148148148D0*T38-0.66666666666666666667D0*T19-2.
     &5061728395061728395D0*T22-1.4074074074074074074D0*T4+2.38452598689
     &22420363D0)*CF*T23+T660*Nf+T662+(T664+90.083333333333333333D0*T4-6
     &3.069444444444444444D0*T5-110.58950617283950617D0)*CA*T2+T666)*T14
      T668=((-0.13888888888888888889D0*T19-T323+T101+T100-2.393518518518
     &5185185D0*T4-1.3703703703703703704D0*T5-1.6280864197530864198D0)*T
     &2+(T549+T324+T101+T325+1.7685185185185185185D0*T4+0.51851851851851
     &851852D0*T5+0.96347736625514403292D0)*CA*CF)*Nf+(-13.9166666666666
     &66667D0*T25+58.5D0*T26+14.583333333333333333D0*T27-28.333333333333
     &333333D0*T28+T326+T327+T328+T329-111.0D0*T19+84.25D0*T20+21.625D0*
     &T21-25.125D0*T22-190.95833333333333333D0*T4+63.270833333333333333D
     &0*T5-151.31944444444444444D0)*T3+(9.0D0*T25-38.583333333333333333D
     &0*T26+23.75D0*T27+29.5D0*T28+T330+T331+T332+T333+156.3888888888888
     &8889D0*T19-77.527777777777777778D0*T20+4.6111111111111111111D0*T21
     &+25.833333333333333333D0*T22+287.07407407407407407D0*T4-62.0185185
     &18518518519D0*T5+222.14429012345679012D0)*CA*T2+(-0.83333333333333
     &333333D0*T25+4.6666666666666666667D0*T26+T482-7.666666666666666666
     &7D0*T28+T248-T227-51.277777777777777778D0*T19+18.38888888888888888
     &9D0*T20-6.5138888888888888889D0*T21-6.6388888888888888889D0*T22-98
     &.344907407407407407D0*T4+15.946759259259259259D0*T5-73.84465020576
     &1316872D0)*T7*CF
      T669=(T550+T334+0.34567901234567901235D0*T4+T335+0.803497942386831
     &27572D0)*CF*T23+((-15.722222222222222222D0*T25+10.5555555555555555
     &56D0*T26+12.333333333333333333D0*T27-4.2222222222222222222D0*T28+T
     &229+T336+T230+T337-27.342592592592592593D0*T19+15.9629629629629629
     &63D0*T20+13.462962962962962963D0*T21-1.5925925925925925926D0*T22-2
     &5.953703703703703704D0*T4+6.5648148148148148148D0*T5-9.54983281893
     &00411522D0)*T2+(8.2222222222222222222D0*T25-5.2777777777777777778D
     &0*T26-5.8888888888888888889D0*T27+2.1111111111111111111D0*T28+T338
     &+T339+T340-T106+14.453703703703703704D0*T19-7.5648148148148148148D
     &0*T20-6.3148148148148148148D0*T21+1.3703703703703703704D0*T22+9.57
     &71604938271604938D0*T4-3.2716049382716049383D0*T5-2.45076017375400
     &09145D0)*CA*CF)*Nf+(-13.25D0*T76-39.333333333333333333D0*T84-78.33
     &3333333333333333D0*T85-23.166666666666666667D0*T77-50.75D0*T78-14.
     &333333333333333333D0*T86-52.333333333333333333D0*T87+T341+T234+T23
     &8+T342+T343+T344+T345+T346+T347-56.375D0*T25-71.75D0*T26-72.0D0*T2
     &7+43.0D0*T28-30.125D0*T29-6.25D0*T30-T348-T349-27.1875D0*T19-14.29
     &1666666666666667D0*T20+39.041666666666666667D0*T21+43.104166666666
     &666667D0*T22-32.927083333333333333D0*T4-47.861111111111111111D0*T5
     &+57.228298611111111111D0)*T3
      T670=T669+(T488+62.0D0*T84+81.833333333333333333D0*T85+51.25D0*T77
     &+T350+15.833333333333333333D0*T86+61.833333333333333333D0*T87+3.5D
     &0*T79+T351+T352+T353+T354+T146+T355+T105+T356+127.9722222222222222
     &2D0*T25+0.069444444444444444444D0*T26-23.333333333333333333D0*T27-
     &5.4027777777777777778D0*T28+22.902777777777777778D0*T29+1.20833333
     &33333333333D0*T30+56.305555555555555556D0*T31+5.277777777777777777
     &8D0*T38+148.16203703703703704D0*T19-117.53935185185185185D0*T20-12
     &2.41435185185185185D0*T21-27.00462962962962963D0*T22+160.620370370
     &37037037D0*T4-25.439814814814814815D0*T5+22.596611368312757202D0)*
     &CA*T2+(-4.5D0*T76-21.166666666666666667D0*T84-20.83333333333333333
     &3D0*T85-19.833333333333333333D0*T77-5.0D0*T78-4.333333333333333333
     &3D0*T86-17.333333333333333333D0*T87+T357+T232+T358+T359-T108+T360-
     &51.472222222222222222D0*T25+15.527777777777777778D0*T26+29.0138888
     &88888888889D0*T27-8.6111111111111111111D0*T28-3.388888888888888888
     &9D0*T29+1.1111111111111111111D0*T30-13.527777777777777778D0*T31+T3
     &61-72.104166666666666667D0*T19+61.280092592592592593D0*T20+48.9467
     &59259259259259D0*T21-T53-65.117283950617283951D0*T4+31.78047839506
     &1728395D0*T5-17.21907864654778235D0)*T7*CF
      T671=(T557+T106-1.037037037037037037D0*T19+T362-2.4197530864197530
     &864D0*T4+T363-3.9183813443072702332D0)*CF*T23+((T559-T364+T372+T56
     &2+T365+T366+T367+T108-T105+T368+8.3333333333333333333D0*T25-20.222
     &222222222222222D0*T26+7.1111111111111111111D0*T27+T369+T248-1.6111
     &111111111111111D0*T30+9.2222222222222222222D0*T31+T370-9.981481481
     &4814814815D0*T19-4.6666666666666666667D0*T20-21.75D0*T21+T371-26.7
     &49228395061728395D0*T4+11.721450617283950617D0*T5-33.7407407407407
     &40741D0)*T2+(T84-T372+T565+T494+T241+T373-T108+T91+T566+10.6111111
     &11111111111D0*T26-T563+4.0D0*T28-1.5D0*T29+3.1111111111111111111D0
     &*T30-6.1111111111111111111D0*T31+T374+13.685185185185185185D0*T19-
     &3.0833333333333333333D0*T20+8.3333333333333333333D0*T21+10.4351851
     &85185185185D0*T22+40.469135802469135802D0*T4-20.691358024691358025
     &D0*T5+70.693244170096021948D0)*CA*CF)*Nf+(-411.0D0*T110+224.0D0*T1
     &11-165.0D0*T112+446.0D0*T113-178.0D0*T114+352.0D0*T115-10.0D0*T116
     &+534.0D0*T117+T610+166.0D0*T118+37.0D0*T119+160.0D0*T120+34.0D0*T1
     &21+58.0D0*T122-46.0D0*T123-T505+T258+T259+T375+T376+T260+T377+T254
     &+T378+T379-T132+T380+T143+T266+T145+T381+T382-525.0D0*T76-210.0D0*
     &T84-357.0D0*T85+127.5D0*T77-124.5D0*T78+211.5D0*T86+67.5D0*T87+417
     &.0D0*T79+190.5D0*T80+126.0D0*T88+66.0D0*T89+135.0D0*T81-21.0D0*T82
     &+27.0D0*T90-34.5D0*T91-16.5D0*T83-502.75D0*T25-474.0D0*T26-411.5D0
     &*T27-206.75D0*T28+106.5D0*T29+47.75D0*T30+139.0D0*T31+87.75D0*T38-
     &539.91666666666666667D0*T19-318.41666666666666667D0*T20-361.041666
     &66666666667D0*T21-89.666666666666666667D0*T22-246.5520833333333333
     &3D0*T4-79.739583333333333333D0*T5-72.364583333333333333D0)*T3
      T672=324.0D0*T110-276.0D0*T111+36.0D0*T112-567.0D0*T113+179.0D0*T1
     &14-364.0D0*T115-17.0D0*T116-567.0D0*T117+18.0D0*T138-83.0D0*T118-T
     &519-114.0D0*T120-27.0D0*T121-59.0D0*T122+49.0D0*T123+T577+T383+T38
     &4+T385+T386+T387+T261+T388+T389+T132+T390+T134+T391+T212+574.33333
     &333333333333D0*T76+332.0D0*T84+290.41666666666666667D0*T85-77.25D0
     &*T77+120.16666666666666667D0*T78-147.75D0*T86-124.4166666666666666
     &7D0*T87-438.0D0*T79-130.5D0*T80-92.166666666666666667D0*T88-50.25D
     &0*T89-104.08333333333333333D0*T81+25.5D0*T82-20.083333333333333333
     &D0*T90+53.75D0*T91-7.6666666666666666667D0*T83+587.166666666666666
     &67D0*T25+751.63888888888888889D0*T26+365.43055555555555556D0*T27+3
     &57.125D0*T28-106.875D0*T29-14.180555555555555556D0*T30-219.8888888
     &8888888889D0*T31-68.666666666666666667D0*T38+765.38425925925925926
     &D0*T19+509.79166666666666667D0*T20+535.66666666666666667D0*T21+188
     &.88425925925925926D0*T22+431.41705246913580247D0*T4-13.55594135802
     &4691358D0*T5+310.14930555555555556D0
      T673=(T671+T672*CA*T2+(-63.0D0*T110+82.0D0*T111+24.0D0*T112+172.0D
     &0*T113-43.0D0*T114+94.0D0*T115+12.0D0*T116+150.0D0*T117-17.0D0*T13
     &8-3.0D0*T119+17.0D0*T120+T392+15.0D0*T122-14.0D0*T123+T586+T393+T3
     &94+T395+T396+T128+T397+T150+T398+T399+T400+T143+T152+T145+T401-165
     &.0D0*T76-114.25D0*T84-51.916666666666666667D0*T85+13.5D0*T77-25.25
     &D0*T78+21.75D0*T86+50.833333333333333333D0*T87+118.5D0*T79+20.25D0
     &*T80+20.083333333333333333D0*T88+T402+23.5D0*T81+T403+6.5833333333
     &333333333D0*T90-19.75D0*T91-6.0D0*T83-196.09722222222222222D0*T25-
     &258.19444444444444444D0*T26-76.305555555555555556D0*T27-132.875D0*
     &T28+28.0D0*T29-14.944444444444444444D0*T30+83.819444444444444444D0
     &*T31+37.347222222222222222D0*T38-265.85648148148148148D0*T19-148.4
     &375D0*T20-168.9375D0*T21-105.02314814814814815D0*T22-216.743827160
     &49382716D0*T4+62.028549382716049383D0*T5-262.03026406035665295D0)*
     &T7*CF)*im*pi
      T674=(-90.666666666666666667D0*T110-60.0D0*T111-60.0D0*T112-38.0D0
     &*T113-25.333333333333333333D0*T114+25.333333333333333333D0*T117+23
     &.333333333333333333D0*T138+25.333333333333333333D0*T118+25.3333333
     &33333333333D0*T119+25.333333333333333333D0*T120+T594+T407+T272+T27
     &3+T274+T408+T276+T151+T263+T409-120.33333333333333333D0*T76-94.0D0
     &*T84-94.0D0*T85-94.222222222222222222D0*T77-21.111111111111111111D
     &0*T78+T510+T511+20.0D0*T79+25.0D0*T80+20.0D0*T88+20.0D0*T89+18.388
     &888888888888889D0*T81-9.2222222222222222222D0*T82+T410-68.0D0*T25-
     &85.666666666666666667D0*T26-85.666666666666666667D0*T27-85.0D0*T28
     &+19.25D0*T29+2.8333333333333333333D0*T30+2.8333333333333333333D0*T
     &31+3.1666666666666666667D0*T38+14.304783950617283951D0*T19-24.7222
     &22222222222222D0*T20-24.722222222222222222D0*T21-11.66743827160493
     &8272D0*T22+29.498842592592592593D0*T4+T487+6.4210390946502057613D0
     &)*T2+(42.0D0*T110+30.0D0*T111+30.0D0*T112+19.0D0*T113+T518-12.6666
     &66666666666667D0*T117-T411-12.666666666666666667D0*T118-12.6666666
     &66666666667D0*T119-12.666666666666666667D0*T120+T509+T265+T412+T41
     &3+T414+T150+T415-T151+T152+49.166666666666666667D0*T76+47.0D0*T84+
     &47.0D0*T85+47.611111111111111111D0*T77+11.611111111111111111D0*T78
     &-T416-T137+T417-11.5D0*T80-10.0D0*T88-10.0D0*T89-6.888888888888888
     &8889D0*T81+6.1111111111111111111D0*T82-7.3333333333333333333D0*T83
     &+13.462962962962962963D0*T25+42.833333333333333333D0*T26+42.833333
     &333333333333D0*T27+37.083333333333333333D0*T28-7.08333333333333333
     &33D0*T29-1.4166666666666666667D0*T30-1.4166666666666666667D0*T31+1
     &5.703703703703703704D0*T38-45.580246913580246913D0*T19+12.36111111
     &1111111111D0*T20+12.361111111111111111D0*T21-9.3302469135802469136
     &D0*T22-43.722222222222222222D0*T4+4.75D0*T5+24.101489864349946654D
     &0)*CA*CF
      T675=519.0D0*T156+108.0D0*T157+108.0D0*T158+332.0D0*T159+327.0D0*T
     &160+162.0D0*T161+162.0D0*T162+608.0D0*T163+72.0D0*T164-106.0D0*T16
     &5-106.0D0*T166+246.0D0*T167+138.0D0*T168+128.0D0*T169+128.0D0*T170
     &+662.0D0*T171-223.0D0*T172-184.0D0*T173-184.0D0*T174-T418-57.0D0*T
     &176+T532+T533+140.0D0*T179-206.0D0*T180-172.0D0*T181-172.0D0*T182-
     &114.0D0*T183+54.0D0*T184+T544+T545+T299+T300+T301+T419+T189+T190+T
     &302+T217+T420+T421+T422+T289+T290+T423+T424+T425+T426+T427+T428+T2
     &91+T292-T306+T207+T429+T430+184.5D0*T110-282.0D0*T111-282.0D0*T112
     &-438.0D0*T113+76.5D0*T114-226.5D0*T115-226.5D0*T116-42.0D0*T117+T5
     &71-108.0D0*T118-108.0D0*T119+103.5D0*T120+73.5D0*T121+141.0D0*T122
     &+141.0D0*T123+558.0D0*T209-427.5D0*T124-237.0D0*T125-237.0D0*T126-
     &111.0D0*T127-106.5D0*T128-40.5D0*T129-40.5D0*T130+96.0D0*T141-105.
     &0D0*T131-123.0D0*T142-123.0D0*T133-96.0D0*T134
      T676=(T675+46.5D0*T144+T535+T431+227.0D0*T76-233.0D0*T84-233.0D0*T
     &85-662.0D0*T77+225.25D0*T78-141.25D0*T86-141.25D0*T87-303.0D0*T79-
     &65.0D0*T80+25.75D0*T88+25.75D0*T89+73.5D0*T81-115.5D0*T82+23.5D0*T
     &90+23.5D0*T91+111.25D0*T83+538.91666666666666667D0*T25-10.75D0*T26
     &-10.75D0*T27-317.16666666666666667D0*T28+338.625D0*T29-10.41666666
     &6666666667D0*T30-10.416666666666666667D0*T31-95.583333333333333333
     &D0*T38+397.88541666666666667D0*T19+131.08333333333333333D0*T20+131
     &.08333333333333333D0*T21+54.34375D0*T22+75.098958333333333333D0*T4
     &+13.125D0*T5-20.109375D0)*T3
      T677=-342.0D0*T156+T455+T456-294.0D0*T159-147.0D0*T160-111.0D0*T16
     &1-111.0D0*T162-678.0D0*T163-18.0D0*T164+161.0D0*T165+161.0D0*T166-
     &203.0D0*T167-59.0D0*T168-76.0D0*T169-76.0D0*T170-643.0D0*T171+146.
     &0D0*T172+164.0D0*T173+164.0D0*T174+81.0D0*T175+28.0D0*T176+T524+T5
     &25-98.0D0*T179+217.0D0*T180+190.0D0*T181+190.0D0*T182+131.0D0*T183
     &-61.0D0*T184+T608+T609+T434+T435+T436+T437-T189-T190+T438+T439+T44
     &0+T441+T195+T442+T443+T304+T444+T445+T446+T447+T448+T449+T450+T451
     &+T307+T308-T207+T452+183.66666666666666667D0*T110+588.0D0*T111+588
     &.0D0*T112+761.0D0*T113+91.333333333333333333D0*T114+222.75D0*T115+
     &222.75D0*T116-22.333333333333333333D0*T117-195.83333333333333333D0
     &*T138-80.833333333333333333D0*T118-80.833333333333333333D0*T119-22
     &8.58333333333333333D0*T120+20.916666666666666667D0*T121-103.5D0*T1
     &22-103.5D0*T123-541.5D0*T209+360.0D0*T124+229.5D0*T125+229.5D0*T12
     &6+137.33333333333333333D0*T127
      T678=(T677+97.333333333333333333D0*T128+47.083333333333333333D0*T1
     &29+47.083333333333333333D0*T130-62.166666666666666667D0*T141+103.3
     &3333333333333333D0*T131+120.0D0*T142+120.0D0*T133+99.9166666666666
     &66667D0*T134-71.75D0*T144-T453-18.0D0*T136+T454+167.83333333333333
     &333D0*T76+633.5D0*T84+633.5D0*T85+1252.6388888888888889D0*T77-261.
     &05555555555555556D0*T78-28.125D0*T86-28.125D0*T87+198.75D0*T79-125
     &.25D0*T80-186.875D0*T88-186.875D0*T89-201.05555555555555556D0*T81+
     &211.38888888888888889D0*T82-8.5D0*T90-8.5D0*T91-82.666666666666666
     &667D0*T83-426.66666666666666667D0*T25+384.375D0*T26+384.375D0*T27+
     &858.83333333333333333D0*T28-596.625D0*T29-96.291666666666666667D0*
     &T30-96.291666666666666667D0*T31+91.416666666666666667D0*T38-467.76
     &427469135802469D0*T19+41.888888888888888889D0*T20+41.8888888888888
     &88889D0*T21+18.749614197530864198D0*T22-248.3130787037037037D0*T4+
     &1.6805555555555555556D0*T5+59.773919753086419753D0)*CA*T2
      T679=45.0D0*T156+T455+T456+64.0D0*T159-9.0D0*T160+15.0D0*T161+15.0
     &D0*T162+187.0D0*T163-11.0D0*T164-54.0D0*T165-54.0D0*T166+40.0D0*T1
     &67-6.0D0*T168+T457+T458+156.0D0*T171-T540-36.0D0*T173-36.0D0*T174-
     &36.0D0*T175+T459+T460+14.0D0*T179-58.0D0*T180-52.0D0*T181-52.0D0*T
     &182-37.0D0*T183+18.0D0*T184+T622+T623+T461+T462+T463+T464+T465+T46
     &6+T467+T217+T468+T469+T470+T471+T472+T473+T474+T475+T476+T477+T478
     &+T479+T208-124.5D0*T110-223.5D0*T111-223.5D0*T112-271.75D0*T113-68
     &.833333333333333333D0*T114-54.75D0*T115-54.75D0*T116+28.4166666666
     &66666667D0*T117+89.0D0*T138+67.416666666666666667D0*T118+67.416666
     &666666666667D0*T119+89.166666666666666667D0*T120-34.33333333333333
     &3333D0*T121+16.5D0*T122+16.5D0*T123+135.0D0*T209-75.75D0*T124-55.5
     &D0*T125-55.5D0*T126-35.416666666666666667D0*T127-22.41666666666666
     &6667D0*T128-13.416666666666666667D0*T129-13.416666666666666667D0*T
     &130+13.75D0*T141-26.916666666666666667D0*T131
      T680=(T679-29.25D0*T142-29.25D0*T133-22.666666666666666667D0*T134+
     &25.75D0*T144+T251+T212-103.95833333333333333D0*T76-258.5D0*T84-258
     &.5D0*T85-461.69444444444444444D0*T77+70.680555555555555556D0*T78+4
     &9.375D0*T86+49.375D0*T87-34.0D0*T79+T480+87.0D0*T88+87.0D0*T89+72.
     &055555555555555556D0*T81-85.444444444444444444D0*T82-1.625D0*T90-1
     &.625D0*T91+49.166666666666666667D0*T83+119.4212962962962963D0*T25-
     &189.5D0*T26-189.5D0*T27-323.27083333333333333D0*T28+205.0208333333
     &3333333D0*T29+50.75D0*T30+50.75D0*T31-68.00462962962962963D0*T38+2
     &17.84799382716049383D0*T19-53.715277777777777778D0*T20-53.71527777
     &7777777778D0*T21+13.813271604938271605D0*T22+166.59722222222222222
     &D0*T4-18.125D0*T5-95.684111130163084895D0)*T7*CF
      T681=(T309+(T310+(4.31875D0*T4+T311+4.04375D0)*T3+(-5.405555555555
     &5555556D0*T4+T312-5.243287037037037037D0)*CA*T2+(1.597222222222222
     &2222D0*T4+T313+1.5208333333333333333D0)*T7*CF)*im*T222+(T314+((-0.
     &54074074074074074074D0*T4+T315-0.38392489711934156379D0)*T2+(0.226
     &85185185185185185D0*T4+T316+0.083899176954732510288D0)*CA*CF)*Nf+(
     &-7.01875D0*T19+12.827777777777777778D0*T20+T317+T318-19.2739583333
     &33333333D0*T4+11.379166666666666667D0*T5-21.402430555555555556D0)*
     &T3+(5.575D0*T19-14.086111111111111111D0*T20+T319+T320+27.496990740
     &740740741D0*T4-12.271064814814814815D0*T5+28.571309156378600823D0)
     &*CA*T2+(-1.0583333333333333333D0*T19+3.8361111111111111111D0*T20+T
     &321+T322-8.9997685185185185185D0*T4+3.1356481481481481481D0*T5-8.3
     &960390946502057613D0)*T7*CF)*T93+T668*im*T34+T670*T35+T673+(T592+T
     &404+2.4074074074074074074D0*T25+T405+3.6975308641975308642D0*T19+T
     &406+2.5833333333333333333D0*T4-1.1922629934461210181D0)*CF*T23+T67
     &4*Nf+T676+T678+T680)*T16
      T682=((-4.6708333333333333333D0*T4-1.6604166666666666667D0)*T3+(5.
     &8111111111111111111D0*T4+2.0513888888888888889D0)*CA*T2+(-1.644444
     &4444444444444D0*T4-0.56388888888888888889D0)*T7*CF)*im*T222+(((0.3
     &8888888888888888889D0*T4+0.11111111111111111111D0)*T2+(-0.17592592
     &592592592593D0*T4-0.046296296296296296296D0)*CA*CF)*Nf+(4.85972222
     &22222222222D0*T19-12.783333333333333333D0*T20+11.38819444444444444
     &4D0*T4-4.375D0*T5+6.8875D0)*T3+(-4.0333333333333333333D0*T19+14.40
     &2777777777777778D0*T20-19.356944444444444444D0*T4+5.00138888888888
     &88889D0*T5-10.310416666666666667D0)*CA*T2+(0.89444444444444444444D
     &0*T19-3.9083333333333333333D0*T20+6.8134259259259259259D0*T4-1.358
     &3333333333333333D0*T5+3.3178240740740740741D0)*T7*CF)*T93+(((0.055
     &555555555555555555D0*T19+T323+4.4722222222222222222D0*T4+T481+1.91
     &66666666666666667D0)*T2+(-0.55555555555555555555D0*T19-T323-2.3148
     &148148148148148D0*T4-T481-0.85185185185185185185D0)*CA*CF)*Nf+(11.
     &833333333333333333D0*T25-57.333333333333333333D0*T26-16.1666666666
     &66666667D0*T27+30.333333333333333333D0*T28+93.722222222222222222D0
     &*T19-49.111111111111111111D0*T20-10.916666666666666667D0*T21+11.16
     &6666666666666667D0*T22+142.48611111111111111D0*T4-20.0555555555555
     &55556D0*T5+56.8125D0)*T3+(-6.3333333333333333333D0*T25+39.5D0*T26+
     &T482-33.5D0*T28-122.94444444444444444D0*T19+48.138888888888888889D
     &0*T20+T483-T484-229.0D0*T4+22.777777777777777778D0*T5-88.083333333
     &333333333D0)*CA*T2+(-5.6666666666666666667D0*T26+T486+9.0D0*T28+40
     &.361111111111111111D0*T19-11.638888888888888889D0*T20+1.5833333333
     &333333333D0*T21+3.5D0*T22+79.24537037037037037D0*T4-6.069444444444
     &4444444D0*T5+29.324074074074074074D0)*T7*CF)*im*T34
      T683=(-0.14814814814814814815D0*T19-0.54320987654320987654D0*T4-0.
     &23456790123456790123D0)*CF*T23+((13.666666666666666667D0*T25+T485-
     &T486+3.3333333333333333333D0*T28+19.648148148148148148D0*T19-9.333
     &3333333333333333D0*T20-T52+T22+13.777777777777777778D0*T4+T487+4.2
     &685185185185185185D0)*T2+(-7.5555555555555555556D0*T25+5.0D0*T26+5
     &.1111111111111111111D0*T27-2.0D0*T28-10.777777777777777778D0*T19+5
     &.0925925925925925926D0*T20+2.7222222222222222222D0*T21+T59-1.79320
     &98765432098765D0*T4+1.712962962962962963D0*T5-0.405864197530864197
     &53D0)*CA*CF)*Nf+(T488+33.333333333333333333D0*T84+74.0D0*T85+18.66
     &6666666666666667D0*T77+48.833333333333333333D0*T78+10.666666666666
     &666667D0*T86+52.0D0*T87+6.6666666666666666667D0*T79+36.25D0*T25+45
     &.277777777777777778D0*T26+33.444444444444444444D0*T27-44.5D0*T28+1
     &1.361111111111111111D0*T29+T576+23.0D0*T31+2.3333333333333333333D0
     &*T38+7.4444444444444444444D0*T19+0.72222222222222222222D0*T20-15.0
     &27777777777777778D0*T21-18.5D0*T22-3.7013888888888888889D0*T4+20.0
     &D0*T5-24.326388888888888889D0)*T3
      T684=T683+(-6.0D0*T76-56.666666666666666667D0*T84-76.6666666666666
     &66667D0*T85-37.5D0*T77-34.5D0*T78-12.666666666666666667D0*T86-60.3
     &33333333333333333D0*T87-3.6666666666666666667D0*T79-101.3333333333
     &3333333D0*T25+20.083333333333333333D0*T26+46.222222222222222222D0*
     &T27+18.833333333333333333D0*T28-9.4444444444444444444D0*T29-3.75D0
     &*T30-23.166666666666666667D0*T31-1.3333333333333333333D0*T38-96.99
     &537037037037037D0*T19+84.944444444444444444D0*T20+50.4861111111111
     &11111D0*T21+10.416666666666666667D0*T22-54.868055555555555556D0*T4
     &+8.5555555555555555556D0*T5-9.5671296296296296296D0)*CA*T2+(T489+2
     &0.5D0*T84+18.833333333333333333D0*T85+15.5D0*T77+4.333333333333333
     &3333D0*T78+5.0D0*T86+16.166666666666666667D0*T87+44.22222222222222
     &2222D0*T25-17.222222222222222222D0*T26-29.333333333333333333D0*T27
     &+T490+1.5277777777777777778D0*T29+0.91666666666666666667D0*T30+5.3
     &333333333333333333D0*T31+53.842592592592592593D0*T19-45.1898148148
     &14814815D0*T20-19.958333333333333333D0*T21+0.20833333333333333333D
     &0*T22+19.454475308641975309D0*T4-11.768518518518518519D0*T5+9.5848
     &765432098765432D0)*T7*CF
      T685=(T491+1.6296296296296296296D0*T19+3.2098765432098765432D0*T4+
     &1.2530864197530864198D0)*CF*T23+((-9.3333333333333333333D0*T76+2.6
     &666666666666666667D0*T84-T497-T492-T493-T267+T494-11.3333333333333
     &33333D0*T25+26.222222222222222222D0*T26-10.888888888888888889D0*T2
     &7+4.6666666666666666667D0*T28-T495+T496-2.6666666666666666667D0*T3
     &1+9.8518518518518518518D0*T19-5.2222222222222222222D0*T20+11.72222
     &2222222222222D0*T21+2.3333333333333333333D0*T22+8.5632716049382716
     &049D0*T4-10.333333333333333333D0*T5+12.943672839506172839D0)*T2+(-
     &T364+T497+2.0D0*T78+2.6666666666666666667D0*T87-5.8888888888888888
     &889D0*T25-9.2222222222222222222D0*T26+6.5555555555555555556D0*T27-
     &3.0D0*T28+T29+T498-18.148148148148148148D0*T19+5.55555555555555555
     &56D0*T20-2.2222222222222222222D0*T21-T499-44.429012345679012346D0*
     &T4+5.1666666666666666667D0*T5-21.163580246913580247D0)*CA*CF)*Nf+(
     &362.0D0*T110-204.0D0*T111+134.0D0*T112-388.0D0*T113+T534-336.0D0*T
     &115-8.0D0*T116-528.0D0*T117-42.0D0*T138-154.0D0*T118-40.0D0*T119-1
     &40.0D0*T120-T500-40.0D0*T122+42.0D0*T123+16.0D0*T209+300.0D0*T76+2
     &84.0D0*T84+220.66666666666666667D0*T85+31.666666666666666667D0*T77
     &+33.666666666666666667D0*T78-50.0D0*T86-73.0D0*T87-166.66666666666
     &666667D0*T79-94.333333333333333333D0*T80-42.666666666666666667D0*T
     &88-28.0D0*T89-58.333333333333333333D0*T81+14.0D0*T82-8.0D0*T90+15.
     &0D0*T91+8.0D0*T83+361.66666666666666667D0*T25+300.3333333333333333
     &3D0*T26+279.83333333333333333D0*T27+129.5D0*T28-45.166666666666666
     &667D0*T29-T501-42.5D0*T31-3.3333333333333333333D0*T38+405.83333333
     &333333333D0*T19+248.75D0*T20+155.91666666666666667D0*T21+17.333333
     &333333333333D0*T22+195.64583333333333333D0*T4+53.83333333333333333
     &3D0*T5+34.760416666666666667D0)*T3
      T686=T685+(-268.0D0*T110+252.0D0*T111-26.0D0*T112+496.0D0*T113-134
     &.0D0*T114+350.0D0*T115+44.0D0*T116+538.0D0*T117-8.0D0*T138+77.0D0*
     &T118+11.0D0*T119+88.0D0*T120+10.0D0*T121+37.0D0*T122-51.0D0*T123-2
     &4.0D0*T209-330.66666666666666667D0*T76-375.66666666666666667D0*T84
     &-192.5D0*T85-72.5D0*T77-24.0D0*T78+29.833333333333333333D0*T86+113
     &.83333333333333333D0*T87+171.66666666666666667D0*T79+69.3333333333
     &33333333D0*T80+29.666666666666666667D0*T88+19.5D0*T89+39.833333333
     &333333333D0*T81-19.0D0*T82+T590-19.5D0*T91-12.0D0*T83-404.83333333
     &333333333D0*T25-555.22222222222222222D0*T26-226.02777777777777778D
     &0*T27-245.91666666666666667D0*T28+43.5D0*T29+11.916666666666666667
     &D0*T30+63.916666666666666667D0*T31+T502-549.11574074074074074D0*T1
     &9-321.65277777777777778D0*T20-231.97222222222222222D0*T21-63.66666
     &6666666666667D0*T22-250.01003086419753086D0*T4+T503-116.5281635802
     &4691358D0)*CA*T2+(51.0D0*T110-74.0D0*T111-22.0D0*T112-152.0D0*T113
     &+T504-90.0D0*T115-22.0D0*T116-142.0D0*T117+T411+T596-13.0D0*T120-2
     &.0D0*T121-T573+17.0D0*T123+T505+108.0D0*T76+121.0D0*T84+34.0D0*T85
     &+29.666666666666666667D0*T77-0.83333333333333333333D0*T78+5.5D0*T8
     &6-48.666666666666666667D0*T87-46.666666666666666667D0*T79-12.83333
     &3333333333333D0*T80-4.1666666666666666667D0*T88-T506-7.33333333333
     &33333333D0*T81+5.0D0*T82+T507+7.0D0*T91+4.0D0*T83+156.361111111111
     &11111D0*T25+183.47222222222222222D0*T26+39.361111111111111111D0*T2
     &7+94.5D0*T28-11.416666666666666667D0*T29+0.75D0*T30+T508-3.8333333
     &333333333333D0*T38+198.14351851851851852D0*T19+84.7361111111111111
     &11D0*T20+62.138888888888888889D0*T21+27.5D0*T22+169.89969135802469
     &136D0*T4-18.395833333333333333D0*T5+80.646604938271604938D0)*T7*CF
      T687=(88.0D0*T110+53.333333333333333333D0*T111+53.3333333333333333
     &33D0*T112+34.666666666666666667D0*T113+24.0D0*T114-29.333333333333
     &333333D0*T117-20.0D0*T138-24.0D0*T118-24.0D0*T119+T603+T509+86.888
     &888888888888889D0*T76+70.222222222222222222D0*T84+70.2222222222222
     &22222D0*T85+85.777777777777777778D0*T77+15.555555555555555556D0*T7
     &8-T510-T511-10.666666666666666667D0*T79-10.0D0*T80-T512-T513-9.666
     &6666666666666667D0*T81+2.6666666666666666667D0*T82+T514+44.7777777
     &77777777778D0*T26+44.777777777777777778D0*T27+37.55555555555555555
     &6D0*T28-12.055555555555555556D0*T29-2.3333333333333333333D0*T30-T4
     &98-4.7484567901234567901D0*T19+10.333333333333333333D0*T20+10.3333
     &33333333333333D0*T21-12.943672839506172839D0*T4-0.1666666666666666
     &6667D0*T5+0.52777777777777777778D0)*T2+(-T515-T516-T517-18.0D0*T11
     &3-T518+12.0D0*T117+8.6666666666666666667D0*T138+12.0D0*T118+T519+1
     &2.0D0*T120-2.6666666666666666667D0*T121-24.777777777777777778D0*T7
     &6-35.111111111111111111D0*T84-35.111111111111111111D0*T85-39.0D0*T
     &77-8.8888888888888888889D0*T78+T416+T137+4.0D0*T79+T613+T520+T521+
     &4.0D0*T81-2.3333333333333333333D0*T82+17.018518518518518518D0*T25-
     &22.388888888888888889D0*T26-22.388888888888888889D0*T27-15.8333333
     &33333333333D0*T28+2.3888888888888888889D0*T29+1.166666666666666666
     &7D0*T30+T522+48.947530864197530864D0*T19-5.1666666666666666667D0*T
     &20-5.1666666666666666667D0*T21+21.163580246913580247D0*T4)*CA*CF
      T688=-458.0D0*T156-96.0D0*T157-96.0D0*T158-300.0D0*T159-274.0D0*T1
     &60-140.0D0*T161-140.0D0*T162-528.0D0*T163+T523+92.0D0*T165+92.0D0*
     &T166-244.0D0*T167-152.0D0*T168-160.0D0*T169-160.0D0*T170-688.0D0*T
     &171+210.0D0*T172+168.0D0*T173+168.0D0*T174+14.0D0*T175+56.0D0*T176
     &+T524+T525-124.0D0*T179+172.0D0*T180+152.0D0*T181+152.0D0*T182+112
     &.0D0*T183-58.0D0*T184-16.0D0*T432-16.0D0*T433-27.0D0*T110+216.0D0*
     &T111+216.0D0*T112+452.0D0*T113+29.0D0*T114+201.66666666666666667D0
     &*T115+201.66666666666666667D0*T116+173.33333333333333333D0*T117+T5
     &26+54.666666666666666667D0*T118+54.666666666666666667D0*T119+4.666
     &6666666666666667D0*T120-2.3333333333333333333D0*T121-75.3333333333
     &33333333D0*T122-75.333333333333333333D0*T123-242.0D0*T209+195.0D0*
     &T124+100.66666666666666667D0*T125+100.66666666666666667D0*T126+58.
     &0D0*T127+48.333333333333333333D0*T128+20.333333333333333333D0*T129
     &+20.333333333333333333D0*T130-38.0D0*T141+T527+46.0D0*T142+46.0D0*
     &T133
      T689=284.0D0*T156+T536+T537+268.0D0*T159+114.0D0*T160+88.0D0*T161+
     &88.0D0*T162+584.0D0*T163-134.0D0*T165-134.0D0*T166+216.0D0*T167+76
     &.0D0*T168+120.0D0*T169+120.0D0*T170+658.0D0*T171-136.0D0*T172-144.
     &0D0*T173-144.0D0*T174-67.0D0*T175-31.0D0*T176+T532+T533+68.0D0*T17
     &9-174.0D0*T180-164.0D0*T181-164.0D0*T182-127.0D0*T183+75.0D0*T184+
     &24.0D0*T432+24.0D0*T433-292.0D0*T110-459.33333333333333333D0*T111-
     &459.33333333333333333D0*T112-693.66666666666666667D0*T113-T534-191
     &.16666666666666667D0*T115-191.16666666666666667D0*T116-90.0D0*T117
     &+140.0D0*T138+126.33333333333333333D0*T118+126.33333333333333333D0
     &*T119+156.16666666666666667D0*T120-55.5D0*T121+58.3333333333333333
     &33D0*T122+58.333333333333333333D0*T123+230.0D0*T209-167.0D0*T124-9
     &7.666666666666666667D0*T125-97.666666666666666667D0*T126-68.0D0*T1
     &27-43.333333333333333333D0*T128-23.833333333333333333D0*T129-23.83
     &3333333333333333D0*T130+16.0D0*T141-24.0D0*T131-43.0D0*T142
      T690=(T689-43.0D0*T133-42.5D0*T134+31.5D0*T144+T535+T431-42.555555
     &555555555556D0*T76-406.05555555555555556D0*T84-406.055555555555555
     &56D0*T85-890.61111111111111111D0*T77+266.94444444444444444D0*T78+1
     &11.58333333333333333D0*T86+111.58333333333333333D0*T87-74.0D0*T79+
     &42.5D0*T80+64.666666666666666667D0*T88+64.666666666666666667D0*T89
     &+76.583333333333333333D0*T81-73.25D0*T82-9.3333333333333333333D0*T
     &90-9.3333333333333333333D0*T91+604.69444444444444444D0*T25-3.81944
     &44444444444444D0*T26-3.8194444444444444444D0*T27-312.2222222222222
     &2222D0*T28+282.38888888888888889D0*T29+63.666666666666666667D0*T30
     &+63.666666666666666667D0*T31+282.5007716049382716D0*T19-9.75D0*T20
     &-9.75D0*T21+116.52816358024691358D0*T4+0.91666666666666666667D0*T5
     &-2.9861111111111111111D0)*CA*T2
      T691=-35.0D0*T156+T536+T537-58.0D0*T159+13.0D0*T160+T538+T539-161.
     &0D0*T163+16.0D0*T164+44.0D0*T165+44.0D0*T166-46.0D0*T167+2.0D0*T16
     &8-20.0D0*T169-20.0D0*T170-162.0D0*T171+T540+30.0D0*T173+30.0D0*T17
     &4+30.0D0*T175+T541+T542+T543-7.0D0*T179+46.0D0*T180+44.0D0*T181+44
     &.0D0*T182+36.0D0*T183-25.0D0*T184-T544-T545+126.33333333333333333D
     &0*T110+175.66666666666666667D0*T111+175.66666666666666667D0*T112+2
     &38.0D0*T113+69.833333333333333333D0*T114+45.166666666666666667D0*T
     &115+45.166666666666666667D0*T116+8.8333333333333333333D0*T117-68.6
     &66666666666666667D0*T138-76.833333333333333333D0*T118-76.833333333
     &333333333D0*T119-71.333333333333333333D0*T120+38.33333333333333333
     &3D0*T121-10.333333333333333333D0*T122-10.333333333333333333D0*T123
     &-57.0D0*T209+36.5D0*T124+23.666666666666666667D0*T125+23.666666666
     &666666667D0*T126+19.5D0*T127+9.8333333333333333333D0*T128+6.833333
     &3333333333333D0*T129+6.8333333333333333333D0*T130-0.5D0*T141+5.0D0
     &*T131+10.0D0*T142+T546+12.0D0*T134
      T692=(T691-11.0D0*T144+T547+T548+24.194444444444444444D0*T76+178.1
     &1111111111111111D0*T84+178.11111111111111111D0*T85+332.25D0*T77-70
     &.611111111111111111D0*T78-60.583333333333333333D0*T86-60.583333333
     &333333333D0*T87+11.916666666666666667D0*T79-28.583333333333333333D
     &0*T80-32.666666666666666667D0*T88-32.666666666666666667D0*T89-31.9
     &16666666666666667D0*T81+30.416666666666666667D0*T82+3.833333333333
     &3333333D0*T90+3.8333333333333333333D0*T91-213.7037037037037037D0*T
     &25+41.847222222222222222D0*T26+41.847222222222222222D0*T27+121.083
     &33333333333333D0*T28-84.138888888888888889D0*T29-27.5D0*T30-27.5D0
     &*T31-181.73533950617283951D0*T19+18.395833333333333333D0*T20+18.39
     &5833333333333333D0*T21-80.646604938271604938D0*T4)*T7*CF
      T693=(T682+T684*T35+T686*im*pi+(-1.3333333333333333333D0*T76-3.481
     &4814814814814815D0*T25-3.9135802469135802469D0*T19+T558)*CF*T23+T6
     &87*Nf+(T688+38.0D0*T134-23.0D0*T144-8.0D0*T135+T381-245.5D0*T76+99
     &.666666666666666667D0*T84+99.666666666666666667D0*T85+376.0D0*T77-
     &236.66666666666666667D0*T78+19.166666666666666667D0*T86+19.1666666
     &66666666667D0*T87+127.66666666666666667D0*T79+39.0D0*T80+T528+T529
     &-10.666666666666666667D0*T81+45.833333333333333333D0*T82+T530+T531
     &-576.83333333333333333D0*T25-159.75D0*T26-159.75D0*T27+84.5D0*T28-
     &168.75D0*T29-17.333333333333333333D0*T30-17.333333333333333333D0*T
     &31-260.22916666666666667D0*T19-54.083333333333333333D0*T20-54.0833
     &33333333333333D0*T21-0.25D0*T22-34.760416666666666667D0*T4-0.375D0
     &*T5+1.25D0)*T3+T690+T692)*x
      T694=((T8+T9+(-48.5D0*T4+T10-85.5D0)*T3+(T6+T11+21.347222222222222
     &222D0)*CA*T2+(T6+T12+3.6388888888888888889D0)*T7*CF)*T13+((-68.0D0
     &*T3+56.0D0*CA*T2+14.0D0*T7*CF)*im*pi+(21.555555555555555556D0*T2-6
     &.3333333333333333333D0*CA*CF)*Nf+(298.0D0*T4+22.0D0*T5+314.0D0)*T3
     &+(-66.0D0*T4-15.0D0*T5-108.94444444444444444D0)*CA*T2+(-33.0D0*T4-
     &T17-6.0277777777777777778D0)*T7*CF)*T14+(T8+T9+(-593.5D0*T4+T10-37
     &1.5D0)*T3+(182.5D0*T4+T11+153.84722222222222222D0)*CA*T2+(49.0D0*T
     &4+T12+1.1388888888888888889D0)*T7*CF)*T16+((487.0D0*T4+143.0D0)*T3
     &+(-188.0D0*T4-66.25D0)*CA*T2+(-20.0D0*T4+1.25D0)*T7*CF)*x-143.0D0*
     &T4*T3+66.25D0*T4*CA*T2-1.25D0*T4*T7*CF)*zeta5+(T15*T13+(-4.0D0*T3-
     &22.666666666666666667D0*CA*T2+31.555555555555555556D0*T7*CF)*T14+T
     &15*T16)*zeta3*zeta3+T632+T635*T92+T639+T652+T667+T681+T693+(1.6604
     &166666666666667D0*T4*T3-2.0513888888888888889D0*T4*CA*T2+0.5638888
     &8888888888889D0*T4*T7*CF)*im*T222+((-0.11111111111111111111D0*T4*T
     &2+0.046296296296296296296D0*T4*CA*CF)*Nf+(-1.1715277777777777778D0
     &*T19+4.375D0*T20-2.5125D0*T4)*T3+(1.0291666666666666667D0*T19-5.00
     &13888888888888889D0*T20+5.3090277777777777778D0*T4)*CA*T2+(-0.2680
     &5555555555555556D0*T19+1.3583333333333333333D0*T20-1.9594907407407
     &407407D0*T4)*T7*CF)*T93
      T695=(T550+0.23456790123456790123D0*T4)*CF*T23+((-4.16666666666666
     &66667D0*T25+3.8888888888888888889D0*T26+3.6666666666666666667D0*T2
     &7-1.0D0*T28-5.1296296296296296296D0*T19+1.3888888888888888889D0*T2
     &0+0.38888888888888888889D0*T21-2.2685185185185185185D0*T4)*T2+(2.4
     &444444444444444444D0*T25-1.6666666666666666667D0*T26-1.55555555555
     &55555556D0*T27+0.66666666666666666667D0*T28+2.8888888888888888889D
     &0*T19-1.0462962962962962963D0*T20+T570-1.112654320987654321D0*T4)*
     &CA*CF)*Nf+(-4.75D0*T76-9.6666666666666666667D0*T84-24.5D0*T85-5.83
     &33333333333333333D0*T77-16.416666666666666667D0*T78-2.833333333333
     &3333333D0*T86+T551-2.3333333333333333333D0*T79-8.16666666666666666
     &67D0*T25-9.9722222222222222222D0*T26-3.3055555555555555555D0*T27+1
     &6.166666666666666667D0*T28+0.86111111111111111111D0*T29-T269+T552+
     &0.65972222222222222222D0*T19-0.16666666666666666667D0*T20+3.847222
     &2222222222222D0*T4)*T3+(T553+18.333333333333333333D0*T84+25.583333
     &333333333333D0*T85+10.5D0*T77+11.75D0*T78+4.0833333333333333333D0*
     &T86+20.166666666666666667D0*T87+T554+29.208333333333333333D0*T25-1
     &0.958333333333333333D0*T26-20.819444444444444444D0*T27-9.083333333
     &3333333333D0*T28-0.40277777777777777778D0*T29+T227+T555+23.2962962
     &96296296296D0*T19-19.638888888888888889D0*T20-1.222222222222222222
     &2D0*T21+2.400462962962962963D0*T4)*CA*T2
      T696=(T695+(-T553-7.0D0*T84-6.1666666666666666667D0*T85-4.5D0*T77-
     &1.4166666666666666667D0*T78-T556-5.3333333333333333333D0*T87-13.73
     &6111111111111111D0*T25+6.3611111111111111111D0*T26+9.9583333333333
     &333333D0*T27-0.20833333333333333333D0*T28-0.013888888888888888889D
     &0*T29-14.831018518518518519D0*T19+11.560185185185185185D0*T20+0.61
     &111111111111111111D0*T21+1.5725308641975308642D0*T4)*T7*CF)*T35
      T697=(T557-0.7037037037037037037D0*T19+T558)*CF*T23+((T559-T560+T3
     &72+T561+T562+T237+T365+4.3333333333333333333D0*T25-10.444444444444
     &444444D0*T26+T563-T564+0.33333333333333333333D0*T31-3.981481481481
     &4814815D0*T19+8.0D0*T20-2.9444444444444444444D0*T21+0.000771604938
     &27160493827D0*T4)*T2+(T84-T372+T565+T494+T566+3.111111111111111111
     &1D0*T26-2.6111111111111111111D0*T27+T567-T568+7.268518518518518518
     &5D0*T19+T569+T570+16.358024691358024691D0*T4)*CA*CF)*Nf+(-115.0D0*
     &T110+66.0D0*T111-40.0D0*T112+119.0D0*T113-40.0D0*T114+111.0D0*T115
     &+7.0D0*T116+180.0D0*T117+T571+50.0D0*T118+14.0D0*T119+43.0D0*T120+
     &T572+T573-15.0D0*T123-T505-59.5D0*T76-115.5D0*T84-52.3333333333333
     &33333D0*T85-45.333333333333333333D0*T77+3.1666666666666666667D0*T7
     &8-19.0D0*T86+24.5D0*T87-4.6666666666666666667D0*T79+13.66666666666
     &6666667D0*T80-4.6666666666666666667D0*T88+T155+T574-T579-94.666666
     &666666666667D0*T25-51.833333333333333333D0*T26-67.25D0*T27-16.6666
     &66666666666667D0*T28+T575+T576-1.5D0*T31-120.91666666666666667D0*T
     &19-75.25D0*T20-21.166666666666666667D0*T21-66.010416666666666667D0
     &*T4)*T3
      T698=T697+(80.0D0*T110-81.0D0*T111+8.5D0*T112-153.5D0*T113+37.0D0*
     &T114-116.5D0*T115-20.5D0*T116-179.0D0*T117+T138-25.0D0*T118-T220-2
     &4.5D0*T120+T121-6.5D0*T122+19.5D0*T123+T577+65.333333333333333333D
     &0*T76+143.08333333333333333D0*T84+48.25D0*T85+57.5D0*T77-7.5D0*T78
     &+17.583333333333333333D0*T86-33.416666666666666667D0*T87+2.6666666
     &666666666667D0*T79-12.166666666666666667D0*T80+T578-2.5D0*T89-T574
     &+T579+T90+101.58333333333333333D0*T25+137.36111111111111111D0*T26+
     &44.805555555555555556D0*T27+57.0D0*T28-3.0D0*T29-5.75D0*T30+0.9166
     &6666666666666667D0*T31+160.71759259259259259D0*T19+56.0D0*T20+40.7
     &36111111111111111D0*T21+68.208719135802469136D0*T4)*CA*T2+(-15.0D0
     &*T110+23.5D0*T111+6.5D0*T112+47.5D0*T113+T580+30.0D0*T115+T581+47.
     &0D0*T117+T582-T583+3.5D0*T120-T584+T122+T585+T586-26.5D0*T76-44.5D
     &0*T84-7.75D0*T85-19.333333333333333333D0*T77+4.4166666666666666667
     &D0*T78-T587+15.333333333333333333D0*T87-T588+T589-0.16666666666666
     &666667D0*T88+0.75D0*T89+0.66666666666666666667D0*T81-0.75D0*T82-T5
     &90-46.513888888888888889D0*T25-45.527777777777777778D0*T26-4.68055
     &55555555555556D0*T27-24.333333333333333333D0*T28+1.25D0*T30+T591-5
     &9.064814814814814815D0*T19-9.1875D0*T20-8.4097222222222222222D0*T2
     &1-55.341049382716049383D0*T4)*T7*CF
      T699=(-30.0D0*T110-16.666666666666666667D0*T111-16.666666666666666
     &667D0*T112-11.333333333333333333D0*T113-8.0D0*T114+10.666666666666
     &666667D0*T117+6.0D0*T138+T593+8.0D0*T119+9.6666666666666666667D0*T
     &120+T594-22.777777777777777778D0*T76-19.444444444444444444D0*T84-1
     &9.444444444444444444D0*T85-27.888888888888888889D0*T77-4.277777777
     &7777777778D0*T78+2.3333333333333333333D0*T86+2.3333333333333333333
     &D0*T87+0.66666666666666666667D0*T80+T359-0.16666666666666666667D0*
     &T25-T246-8.0D0*T27+2.9444444444444444444D0*T29-0.00077160493827160
     &493827D0*T19)*T2+(11.666666666666666667D0*T110+8.33333333333333333
     &33D0*T111+8.3333333333333333333D0*T112+6.0D0*T113+4.66666666666666
     &66667D0*T114-4.0D0*T117-2.3333333333333333333D0*T138+T595-T596-4.0
     &D0*T120+T509+2.8888888888888888889D0*T76+9.7222222222222222222D0*T
     &84+9.7222222222222222222D0*T85+11.833333333333333333D0*T77+2.44444
     &44444444444444D0*T78-1.1666666666666666667D0*T86-1.166666666666666
     &6667D0*T87-0.33333333333333333333D0*T80+0.16666666666666666667D0*T
     &82-11.62037037037037037D0*T25+T597+T598+0.19444444444444444444D0*T
     &29-16.358024691358024691D0*T19)*CA*CF
      T700=145.0D0*T156+30.0D0*T157+30.0D0*T158+96.0D0*T159+83.0D0*T160+
     &43.0D0*T161+43.0D0*T162+162.0D0*T163+T599-T600-T601+83.0D0*T167+55
     &.0D0*T168+62.0D0*T169+62.0D0*T170+242.0D0*T171-69.0D0*T172-54.0D0*
     &T173-54.0D0*T174-4.0D0*T175-19.0D0*T176-5.0D0*T177-5.0D0*T178+38.0
     &D0*T179-50.0D0*T180-46.0D0*T181-46.0D0*T182-38.0D0*T183+T602+T544+
     &T545-21.0D0*T110-61.0D0*T111-61.0D0*T112-161.5D0*T113-T504-65.3333
     &33333333333333D0*T115-65.333333333333333333D0*T116-89.666666666666
     &666667D0*T117-13.0D0*T138-8.3333333333333333333D0*T118-8.333333333
     &3333333333D0*T119+T603-19.833333333333333333D0*T121+4.666666666666
     &6666667D0*T122+4.6666666666666666667D0*T123-21.0D0*T124-7.33333333
     &33333333333D0*T125-7.3333333333333333333D0*T126-12.0D0*T127-4.6666
     &666666666666667D0*T128-T604-T605+3.0D0*T131+T606-13.16666666666666
     &6667D0*T84-13.166666666666666667D0*T85-60.5D0*T77+79.4166666666666
     &66667D0*T78+16.666666666666666667D0*T86
      T701=(T700+T607-11.0D0*T80-6.5D0*T88-6.5D0*T89+1.5D0*T82+200.66666
     &666666666667D0*T25+75.25D0*T26+75.25D0*T27+21.166666666666666667D0
     &*T29+66.010416666666666667D0*T19)*T3
      T702=-85.0D0*T156+T614+T615-86.0D0*T159-33.0D0*T160-24.5D0*T161-24
     &.5D0*T162-178.0D0*T163+3.0D0*T164+40.0D0*T165+40.0D0*T166-76.5D0*T
     &167-30.5D0*T168-51.0D0*T169-51.0D0*T170-230.0D0*T171+44.0D0*T172+4
     &5.0D0*T173+45.0D0*T174+20.0D0*T175+11.0D0*T176+8.5D0*T177+8.5D0*T1
     &78-16.0D0*T179+48.0D0*T180+49.0D0*T181+49.0D0*T182+42.5D0*T183-31.
     &5D0*T184+T608+T609+121.5D0*T110+131.66666666666666667D0*T111+131.6
     &6666666666666667D0*T112+230.58333333333333333D0*T113+56.75D0*T114+
     &60.833333333333333333D0*T115+60.833333333333333333D0*T116+58.0D0*T
     &117-T610-51.666666666666666667D0*T118-51.666666666666666667D0*T119
     &-34.083333333333333333D0*T120+30.75D0*T121+T611+T612+20.5D0*T124+8
     &.3333333333333333333D0*T125+8.3333333333333333333D0*T126+11.0D0*T1
     &27+5.1666666666666666667D0*T128+T604+T605-4.0D0*T131-1.0D0*T142-1.
     &0D0*T133-5.6388888888888888889D0*T76+99.194444444444444444D0*T84+9
     &9.194444444444444444D0*T85+223.30555555555555556D0*T77-88.55555555
     &5555555556D0*T78
      T703=(T702-57.0D0*T86-57.0D0*T87+T613+5.75D0*T88+5.75D0*T89-0.9166
     &6666666666666667D0*T82-241.29166666666666667D0*T25-56.0D0*T26-56.0
     &D0*T27-40.736111111111111111D0*T29-68.208719135802469136D0*T19)*CA
     &*T2
      T704=10.0D0*T156+T614+T615+18.5D0*T159-5.0D0*T160+T616+T617+49.0D0
     &*T163-6.5D0*T164-T618-T619+17.0D0*T167+T620+10.0D0*T169+10.0D0*T17
     &0+57.0D0*T171-6.5D0*T172-9.0D0*T173-9.0D0*T174-9.0D0*T175+T621+T45
     &9+T460+0.5D0*T179-12.5D0*T180-13.0D0*T181-13.0D0*T182-12.0D0*T183+
     &11.0D0*T184+T622+T623-42.416666666666666667D0*T110-50.583333333333
     &333333D0*T111-50.583333333333333333D0*T112-76.75D0*T113-24.6666666
     &66666666667D0*T114-14.083333333333333333D0*T115-14.083333333333333
     &333D0*T116-11.416666666666666667D0*T117+19.833333333333333333D0*T1
     &38+27.916666666666666667D0*T118+27.916666666666666667D0*T119+19.91
     &6666666666666667D0*T120-15.166666666666666667D0*T121+0.16666666666
     &666666667D0*T122+0.16666666666666666667D0*T123-5.0D0*T124-2.333333
     &3333333333333D0*T125-2.3333333333333333333D0*T126-2.5D0*T127-1.416
     &6666666666666667D0*T128-T413-T414+1.25D0*T131+0.5D0*T142+0.5D0*T13
     &3+8.1527777777777777778D0*T76-46.305555555555555556D0*T84-46.30555
     &5555555555556D0*T85
      T705=(T704-86.333333333333333333D0*T77+23.513888888888888889D0*T78
     &+24.333333333333333333D0*T86+24.333333333333333333D0*T87+0.5833333
     &3333333333333D0*T80-1.25D0*T88-1.25D0*T89+0.083333333333333333333D
     &0*T82+91.150462962962962963D0*T25+9.1875D0*T26+9.1875D0*T27+8.4097
     &222222222222222D0*T29+55.341049382716049383D0*T19)*T7*CF
      betaHPL_3l_no_nfv=(T694+(((-0.027777777777777777778D0*T19-T324-2.027777777777777
     &7778D0*T4)*T2+(T549+T324+0.96296296296296296296D0*T4)*CA*CF)*Nf+(-
     &3.4166666666666666667D0*T25+19.166666666666666667D0*T26+6.08333333
     &33333333333D0*T27-11.166666666666666667D0*T28-29.23611111111111111
     &1D0*T19+8.8888888888888888889D0*T20+0.58333333333333333333D0*T21-3
     &6.895833333333333333D0*T4)*T3+(1.4166666666666666667D0*T25-13.5D0*
     &T26+3.1666666666666666667D0*T27+12.75D0*T28+36.555555555555555556D
     &0*T19-10.027777777777777778D0*T20-0.625D0*T21+65.48611111111111111
     &1D0*T4)*CA*T2+(0.25D0*T25+2.0833333333333333333D0*T26-2.9166666666
     &666666667D0*T27-3.5D0*T28-12.138888888888888889D0*T19+2.5694444444
     &444444444D0*T20+0.16666666666666666667D0*T21-23.310185185185185185
     &D0*T4)*T7*CF)*im*T34+T696+T698*im*pi+(T592+1.4074074074074074074D0
     &*T25+1.2530864197530864198D0*T19)*CF*T23+T699*Nf+T701+T703+T705)/(
     &T13-2.0D0*T14+T16)

      end function betaHPL_3l_no_nfv

      function gammaHPL_3l_nfv2(x)
        use types
        use constants
        implicit complex(dp) (T)
        include 'nf.f'
        complex(dp) :: gammaHPL_3l_nfv2
        real(dp), intent(in) :: x

      T2=CF*CF
      T3=(56.0D0*T2-21.5D0*CA*CF)*im*pi
      T4=6.6666666666666666667D0*CF*Nf
      T5=x**3
      T6=10.0D0*T2
      T7=x*x
      T8=Hr1(1)
      T9=Hr1(0)
      T10=Hr2(1,1)
      T11=Hr2(1,0)
      T12=Hr2(0,1)
      T13=Hr2(0,0)
      T14=x**4
      T15=pi*pi
      T16=-4.0D0*T9
      T17=5.3333333333333333333D0*T8
      T18=-4.0D0*T13
      T19=8.0D0*T9
      T20=Hr3(0,1,0)
      T21=Hr3(0,0,1)
      T22=(-1.6666666666666666667D0*T2-0.5D0*CA*CF)*im*pi**3
      T23=-2.4444444444444444445D0*CF*Nf
      T24=-20.0D0*T12
      T25=24.0D0*T8
      T26=4.0D0*T9
      T27=0.5D0*T9
      T28=-4.0D0*T12
      T29=1.3333333333333333333D0*T9
      T30=-4.0D0*T20
      T31=-28.0D0*T21
      T32=30.0D0*T10
      T33=10.0D0*T11
      T34=4.0D0*T13
      T35=-4.0D0*Hr3(0,1,1)
      T36=8.0D0*T10
      T37=1.5D0*T13
      T38=10.0D0*T10
      T39=6.0000000000000000001D0*T11
      T40=5.3333333333333333333D0*T10
      T41=pi**3
      T42=-0.33333333333333333333D0*T8
      T43=0.33333333333333333333D0*T9
      T44=4.0D0*T20
      T45=Hr3(1,1,1)
      T46=Hr3(1,1,0)
      T47=Hr3(1,0,1)
      T48=Hr3(1,0,0)
      T49=Hr3(0,1,1)
      T50=Hr3(0,0,0)
      T51=0.66666666666666666667D0*T10
      T52=0.66666666666666666667D0*T11
      T53=0.66666666666666666667D0*T12
      T54=0.66666666666666666667D0*T13
      T55=0.33333333333333333333D0*T10
      T56=0.33333333333333333333D0*T11
      T57=0.33333333333333333333D0*T12
      T58=0.33333333333333333333D0*T13
      T59=Hr4(1,1,1,1)
      T60=Hr4(1,1,0,0)
      T61=Hr4(1,0,1,1)
      T62=Hr4(1,0,0,0)
      T63=Hr4(0,1,1,1)
      T64=Hr4(0,1,0,0)
      T65=Hr4(0,0,1,1)
      T66=Hr4(0,0,0,0)
      T67=x**5
      T68=pi**4
      T69=-0.66666666666666666667D0*T47
      T70=0.66666666666666666667D0*T20
      T71=-0.074074074074074074074D0*T10
      T72=-0.074074074074074074074D0*T12
      T73=0.88888888888888888889D0*T8
      T74=0.88888888888888888889D0*T9
      T75=Hr4(1,1,1,0)
      T76=Hr4(1,1,0,1)
      T77=Hr4(1,0,1,0)
      T78=Hr4(0,1,0,1)
      T79=Hr4(0,0,1,0)
      T80=Hr4(0,0,0,1)
      T81=0.22222222222222222222D0*T8
      T82=0.22222222222222222222D0*T9
      T83=Hr4(1,0,0,1)
      T84=Hr4(0,1,1,0)
      T85=0.66666666666666666667D0*T45
      T86=0.66666666666666666667D0*T48
      T87=-0.66666666666666666667D0*T49
      T88=0.66666666666666666667D0*T50
      T89=18.0D0*T45
      T90=2.0D0*T48
      T91=4.0D0*T50
      T92=-6.6666666666666666667D0*T11
      T93=Hr5(1,1,1,1,0)
      T94=Hr5(1,1,1,0,1)
      T95=Hr5(1,1,0,1,1)
      T96=Hr5(1,1,0,1,0)
      T97=Hr5(1,1,0,0,1)
      T98=Hr5(1,1,0,0,0)
      T99=Hr5(1,0,1,1,1)
      T100=Hr5(1,0,1,1,0)
      T101=Hr5(1,0,1,0,1)
      T102=Hr5(1,0,1,0,0)
      T103=Hr5(0,1,0,1,1)
      T104=Hr5(0,1,0,1,0)
      T105=Hr5(0,1,0,0,1)
      T106=Hr5(0,1,0,0,0)
      T107=Hr5(0,0,1,1,1)
      T108=Hr5(0,0,1,1,0)
      T109=Hr5(0,0,1,0,1)
      T110=Hr5(0,0,1,0,0)
      T111=Hr5(0,0,0,1,0)
      T112=Hr5(0,0,0,0,1)
      T113=pi**5
      T114=1.6666666666666666667D0*T21
      T115=1.6666666666666666667D0*T10
      T116=-1.6666666666666666667D0*T11
      T117=0.44444444444444444445D0*T49
      T118=2.0D0*T21
      T119=1.5D0*T8
      T120=0.66666666666666666667D0*T63
      T121=1.3333333333333333333D0*T78
      T122=1.3333333333333333333D0*T64
      T123=-2.3333333333333333333D0*T20
      T124=-4.0D0*T45
      T125=1.3333333333333333333D0*T45
      T126=4.0D0*T47
      T127=0.11111111111111111111D0*T13
      T128=Hr5(0,1,1,1,0)
      T129=Hr5(0,1,1,0,1)
      T130=-4.0D0*T109
      T131=Hr5(0,0,0,1,1)
      T132=-2.0D0*T59
      T133=2.0D0*T62
      T134=0.83333333333333333334D0*T13
      T135=4.0D0*T105
      T136=6.0000000000000000001D0*T107
      T137=4.0D0*T59
      T138=6.0000000000000000001D0*T75
      T139=4.0D0*T64
      T140=-2.0D0*T50
      T141=1.3333333333333333333D0*T131
      T142=1.3333333333333333333D0*T76
      T143=4.0D0*T61
      T144=-1.3333333333333333333D0*T77
      T145=1.3333333333333333333D0*T46
      T146=Hr5(1,1,1,0,0)
      T147=-4.0D0*T96
      T148=-4.0D0*T97
      T149=Hr5(1,0,0,1,0)
      T150=Hr5(1,0,0,0,1)
      T151=Hr5(0,1,1,0,0)
      T152=-4.0D0*T106
      T153=4.0D0*T108
      T154=4.0D0*T109
      T155=2.0D0*T49
      T156=Hr6(0,1,1,1,0,0)
      T157=Hr6(0,1,1,0,1,1)
      T158=Hr6(0,1,0,1,1,1)
      T159=Hr6(0,1,0,1,1,0)
      T160=Hr6(0,1,0,1,0,1)
      T161=Hr6(0,1,0,1,0,0)
      T162=Hr6(0,1,0,0,1,1)
      T163=Hr6(0,0,1,1,1,1)
      T164=Hr6(0,0,1,1,1,0)
      T165=Hr6(0,0,1,1,0,1)
      T166=Hr6(0,0,1,1,0,0)
      T167=Hr6(0,0,1,0,1,1)
      T168=Hr6(0,0,0,1,1,1)
      T169=4.0D0*T96
      T170=4.0D0*T97
      T171=Hr5(1,0,0,1,1)
      T172=Hr5(0,1,1,1,1)
      T173=-1.0D0*T50
      T174=-0.0080687830687830687831D0*T2
      T175=pi**6
      T176=-0.037037037037037037037D0*CF*Nf
      T177=0.4D0*T12
      T178=0.094444444444444444445D0*T8
      T179=0.17777777777777777778D0*T9
      T180=0.10694444444444444445D0*T9
      T181=0.44444444444444444445D0*T12
      T182=0.11111111111111111111D0*T9
      T183=1.3333333333333333333D0*T49
      T184=2.1666666666666666667D0*T10
      T185=3.8333333333333333334D0*T11
      T186=-0.5D0*T49
      T187=0.083333333333333333333D0*T13
      T188=0.22222222222222222222D0*T11
      T189=-2.3333333333333333333D0*T65
      T190=2.6666666666666666667D0*T79
      T191=2.0D0*T80
      T192=1.6666666666666666667D0*T45
      T193=4.1666666666666666667D0*T46
      T194=1.5D0*T47
      T195=5.6666666666666666667D0*T48
      T196=2.3333333333333333333D0*T20
      T197=0.33333333333333333333D0*T50
      T198=3.8333333333333333334D0*T84
      T199=0.33333333333333333333D0*T78
      T200=4.1666666666666666667D0*T65
      T201=-0.16666666666666666667D0*T80
      T202=4.0D0*T45
      T203=0.83333333333333333334D0*T47
      T204=-2.6666666666666666667D0*T78
      T205=-1.3333333333333333333D0*T80
      T206=1.3333333333333333333D0*T48
      T207=4.0D0*T128
      T208=-4.0D0*T129
      T209=12.0D0*T103
      T210=4.0D0*T104
      T211=12.0D0*T107
      T212=4.0D0*T131
      T213=-4.0D0*T112
      T214=2.0D0*T59
      T215=22.0D0*T76
      T216=14.0D0*T60
      T217=10.0D0*T61
      T218=22.0D0*T77
      T219=6.0000000000000000001D0*T83
      T220=24.0D0*T63
      T221=2.0D0*T84
      T222=6.0000000000000000001D0*T64
      T223=6.0000000000000000001D0*T79
      T224=12.0D0*T80
      T225=1.3333333333333333333D0*T50
      T226=6.0000000000000000001D0*T129
      T227=-4.0D0*T151
      T228=-3.0D0*T103
      T229=-2.0D0*T106
      T230=6.0000000000000000001D0*T109
      T231=-4.0D0*T110
      T232=-2.0D0*T131
      T233=-1.0D0*T111
      T234=8.0D0*T76
      T235=8.0D0*T60
      T236=10.0D0*T83
      T237=3.0D0*T62
      T238=11.5D0*T63
      T239=3.0D0*T50
      T240=2.6666666666666666667D0*T103
      T241=2.6666666666666666667D0*T107
      T242=1.3333333333333333333D0*T75
      T243=1.3333333333333333333D0*T60
      T244=1.3333333333333333333D0*T77
      T245=1.3333333333333333333D0*T83
      T246=5.3333333333333333333D0*T63
      T247=0.66666666666666666667D0*T64
      T248=-2.0D0*T46
      T249=2.0D0*T47
      T250=0.11111111111111111111D0*T20
      T251=0.11111111111111111111D0*T21
      T252=4.0D0*T156
      T253=4.0D0*T157
      T254=-20.0D0*T158
      T255=-8.0D0*T159
      T256=-8.0D0*T160
      T257=-4.0D0*T161
      T258=4.0D0*T162
      T259=-28.0D0*T163
      T260=-16.0D0*T164
      T261=-16.0D0*T165
      T262=-16.0D0*T166
      T263=4.0D0*T167
      T264=-4.0D0*T168
      T265=4.0D0*Hr6(0,0,0,0,1,1)
      T266=2.0D0*T93
      T267=2.0D0*T94
      T268=8.0D0*T146
      T269=26.0D0*T95
      T270=18.0D0*T98
      T271=22.0D0*T99
      T272=12.0D0*T100
      T273=12.0D0*T101
      T274=10.0D0*T102
      T275=8.0D0*T171
      T276=2.0D0*T149
      T277=2.0D0*T150
      T278=56.0D0*T172
      T279=32.0D0*T128
      T280=32.0D0*T129
      T281=30.0D0*T151
      T282=2.0D0*T104
      T283=2.0D0*T105
      T284=4.0D0*T106
      T285=2.0D0*T110
      T286=12.0D0*T131
      T287=0.66666666666666666667D0*T83
      T288=-4.6666666666666666667D0*T78
      T289=0.66666666666666666667D0*T80
      T290=2.0D0*T66
      T291=-2.0D0*T157
      T292=4.0D0*Hr6(0,1,1,0,1,0)
      T293=4.0D0*Hr6(0,1,1,0,0,1)
      T294=7.0000000000000000001D0*T158
      T295=4.0D0*T159
      T296=4.0D0*T160
      T297=-2.0D0*T162
      T298=2.0D0*Hr6(0,1,0,0,1,0)
      T299=2.0D0*Hr6(0,1,0,0,0,1)
      T300=2.0D0*T163
      T301=8.0D0*T164
      T302=8.0D0*T165
      T303=-2.0D0*T167
      T304=4.0D0*Hr6(0,0,1,0,1,0)
      T305=4.0D0*Hr6(0,0,1,0,0,1)
      T306=3.0D0*T168
      T307=Hr6(0,0,0,1,1,0)
      T308=Hr6(0,0,0,1,0,1)
      T309=4.0D0*T93
      T310=4.0D0*T94
      T311=2.0D0*T146
      T312=12.0D0*T95
      T313=12.0D0*T98
      T314=2.0D0*T99
      T315=7.0000000000000000001D0*T171
      T316=3.0D0*T149
      T317=3.0D0*T150
      T318=4.0D0*T172
      T319=15.5D0*T128
      T320=15.5D0*T129
      T321=2.5D0*T106
      T322=-8.0D0*T59
      T323=11.833333333333333333D0*T63
      T324=0.16666666666666666667D0*T10
      T325=0.83333333333333333334D0*T11
      T326=0.22222222222222222222D0*T10
      T327=0.074074074074074074074D0*T10
      T328=0.074074074074074074074D0*T11
      T329=1.3888888888888888889D0*T46
      T330=1.3888888888888888889D0*T47
      T331=2.2222222222222222222D0*T46
      T332=0.66666666666666666667D0*T47
      T333=2.0D0*T20
      T334=0.44444444444444444445D0*T45
      T335=0.44444444444444444445D0*T46
      T336=0.44444444444444444445D0*T47
      T337=0.44444444444444444445D0*T48
      T338=0.66666666666666666667D0*T59
      T339=0.66666666666666666667D0*T75
      T340=4.6666666666666666667D0*T76
      T341=4.6666666666666666667D0*T60
      T342=4.6666666666666666667D0*T61
      T343=4.6666666666666666667D0*T77
      T344=0.66666666666666666667D0*T62
      T345=2.0D0*T46
      T346=-17.0D0*T21
      T347=1.3333333333333333333D0*T59
      T348=2.6666666666666666667D0*T75
      T349=1.3333333333333333333D0*T61
      T350=4.0D0*T49
      T351=0.44444444444444444445D0*T75
      T352=0.44444444444444444445D0*T76
      T353=0.44444444444444444445D0*T60
      T354=0.44444444444444444445D0*T61
      T355=0.44444444444444444445D0*T77
      T356=0.44444444444444444445D0*T83
      T357=4.3333333333333333333D0*T49
      T358=0.66666666666666666667D0*T93
      T359=0.66666666666666666667D0*T94
      T360=6.0000000000000000001D0*T95
      T361=1.3333333333333333333D0*T96
      T362=1.3333333333333333333D0*T97
      T363=6.0000000000000000001D0*T98
      T364=6.0000000000000000001D0*T99
      T365=1.3333333333333333333D0*T100
      T366=1.3333333333333333333D0*T101
      T367=6.0000000000000000001D0*T102
      T368=0.66666666666666666667D0*T149
      T369=0.66666666666666666667D0*T150
      T370=17.0D0*T65
      T371=-14.166666666666666667D0*T46
      T372=-14.166666666666666667D0*T47
      T373=1.3333333333333333333D0*T93
      T374=1.3333333333333333333D0*T94
      T375=1.3333333333333333333D0*T146
      T376=2.6666666666666666667D0*T95
      T377=4.0D0*T98
      T378=4.0D0*T99
      T379=2.6666666666666666667D0*T100
      T380=2.6666666666666666667D0*T101
      T381=-2.0D0*T65
      T382=-1.5D0*T12
      T383=-1.5D0*T11
      T384=2.0D0*T12
      T385=2.0D0*T45
      T386=2.0D0*T61
      T387=((-8.0D0*T2+4.0D0*CA*CF)*im*pi+(-4.0D0*T8+T16-3.0D0)*T2+(2.0D
     &0*T8+2.0D0*T9+0.66666666666666666667D0)*CA*CF)*T67+((-2.3888888888
     &888888889D0*T2+1.1111111111111111111D0*CA*CF)*T15+(0.8888888888888
     &8888889D0*CF*Nf+(T17+5.3333333333333333333D0*T9+25.0D0)*T2+(-6.666
     &6666666666666667D0*T8-2.6666666666666666667D0*T9-29.22222222222222
     &2222D0)*CA*CF)*im*pi+(T73+T74+0.5925925925925925926D0)*CF*Nf+(-T38
     &+T39-10.0D0*T12+6.0000000000000000001D0*T13+12.0D0*T8+T19+11.66666
     &6666666666667D0)*T2+(T40+T92+8.0D0*T12+T18-12.277777777777777778D0
     &*T8-12.277777777777777778D0*T9-8.5370370370370370371D0)*CA*CF)*T14
     &+(T22+(T23-4.0D0*T2+16.111111111111111111D0*CA*CF)*T15+(4.44444444
     &44444444445D0*CF*Nf+(T24+T25+T16+7.6666666666666666667D0)*T2+(28.0
     &D0*T12-40.0D0*T8-T27-75.861111111111111111D0)*CA*CF)*im*pi+(T28+T1
     &7-T29+3.1111111111111111111D0)*CF*Nf+(T30+T31+T32-T33+66.0D0*T12+T
     &18-49.0D0*T8+T19-36.5D0)*T2+(T35+14.0D0*T20+7.0000000000000000001D
     &0*T21-T36-17.0D0*T11+22.0D0*T12+T37-55.916666666666666667D0*T8-48.
     &0D0*T9-5.4722222222222222222D0)*CA*CF)*T5
      T388=(T387+(T22+(T23-12.0D0*T2+13.111111111111111111D0*CA*CF)*T15+
     &(7.1111111111111111112D0*CF*Nf+(T24-T25+T26+0.33333333333333333333
     &D0)*T2+(40.0D0*T12+34.0D0*T8+T27-19.361111111111111111D0)*CA*CF)*i
     &m*pi+(T28-T17+T29-5.1111111111111111111D0)*CF*Nf+(T30+T31-T32+T33-
     &70.0D0*T12+T34+106.0D0*T8+T9+58.5D0)*T2+(T35+26.0D0*T20+T346+T36+1
     &1.0D0*T11+52.0D0*T12-T37+94.416666666666666667D0*T8-12.5D0*T9-5.86
     &11111111111111111D0)*CA*CF)*T7+((-5.3333333333333333333D0*T8*T2+12
     &.666666666666666667D0*T8*CA*CF)*im*pi-0.88888888888888888889D0*T8*
     &CF*Nf+(T38-T39-82.0D0*T8-17.0D0)*T2+(-T40+12.666666666666666667D0*
     &T11-12.0D0*T12-42.722222222222222222D0*T8-14.5D0)*CA*CF)*x+17.0D0*
     &T8*T2+14.5D0*T8*CA*CF)*zeta3
      T389=(-0.43333333333333333334D0*T2+0.21666666666666666667D0*CA*CF)
     &*T68+((0.66666666666666666667D0*T8+0.66666666666666666667D0*T9-0.8
     &3333333333333333334D0)*T2+(T42-T43+0.33333333333333333333D0)*CA*CF
     &)*im*T41+(-0.16666666666666666667D0*CF*Nf+(-T51-T52-T53-T54+T42+T4
     &3-0.25D0)*T2+(T55+T56+T57+T58+1.1666666666666666667D0)*CA*CF)*T15+
     &((T124+4.0D0*T46-T126+4.0D0*T48+T35+T44-4.0D0*T21+T91-T115+T116-1.
     &6666666666666666667D0*T12-1.6666666666666666667D0*T13)*T2+(T385+T2
     &48+T249-T90+T155-T333+T118+T140+T51+T52+T53+T54)*CA*CF)*im*pi+(-T5
     &5-T56-T57-T58)*CF*Nf+(T137+4.0D0*T60+T143+4.0D0*T62+4.0D0*T63+T139
     &+4.0D0*T65+4.0D0*T66+T45-0.66666666666666666667D0*T46+T69-2.333333
     &3333333333333D0*T48+2.3333333333333333333D0*T49+T70+0.666666666666
     &66666667D0*T21+T173-0.5D0*T10-0.5D0*T11-0.5D0*T12-0.5D0*T13)*T2+(T
     &132-2.0D0*T60-T386-T133-2.0D0*T63-2.0D0*T64+T381-T290-T85+T86+T87+
     &T88+2.3333333333333333333D0*T10+2.3333333333333333333D0*T11+2.3333
     &333333333333333D0*T12+2.3333333333333333333D0*T13)*CA*CF
      T390=(0.055555555555555555556D0*T2+0.011111111111111111111D0*CA*CF
     &)*im*T113+(0.0061728395061728395062D0*CF*Nf+(-0.298148148148148148
     &15D0*T8-0.29814814814814814815D0*T9+0.6D0)*T2+(0.15555555555555555
     &556D0*T8+0.3962962962962962963D0*T9-1.3905864197530864198D0)*CA*CF
     &)*T68+((-T81-T82+0.45679012345679012346D0)*CF*Nf+(-T324+T325-0.166
     &66666666666666667D0*T12+T134-T119-3.5D0*T9+5.1388888888888888889D0
     &)*T2+(-T326-0.44444444444444444445D0*T11+0.22222222222222222222D0*
     &T12+1.4907407407407407407D0*T8+1.4907407407407407407D0*T9-1.077160
     &4938271604938D0)*CA*CF)*im*T41+((T71+T328+T72+0.074074074074074074
     &074D0*T13-0.28395061728395061728D0*T8-0.95061728395061728395D0*T9+
     &1.8888888888888888889D0)*CF*Nf+(T45-T329-T330+T48+T49-1.3888888888
     &888888889D0*T20-1.3888888888888888889D0*T21+T50+3.9166666666666666
     &667D0*T10-2.0833333333333333333D0*T11+3.9166666666666666667D0*T12-
     &2.0833333333333333333D0*T13-0.72222222222222222223D0*T8+0.27777777
     &777777777778D0*T9-3.3333333333333333334D0)*T2+(T125-T331+T69-0.222
     &22222222222222222D0*T48-0.22222222222222222222D0*T49-T70-2.2222222
     &222222222222D0*T21+T225+1.6759259259259259259D0*T10-4.509259259259
     &2592593D0*T11+1.6759259259259259259D0*T12-4.5092592592592592593D0*
     &T13-1.2530864197530864198D0*T8+1.4135802469135802469D0*T9-9.055555
     &5555555555556D0)*CA*CF)*T15
      T391=(-T334+T335+T336-T337-T117+0.44444444444444444445D0*T20+0.444
     &44444444444444445D0*T21-0.44444444444444444445D0*T50+T71+1.4074074
     &074074074074D0*T11+T72+1.4074074074074074074D0*T13-T73-T74)*CF*Nf+
     &(T338-T339-T340+T341-T342+T343+T287-T344+T120-0.666666666666666666
     &67D0*T84+T288+4.6666666666666666667D0*T64-4.6666666666666666667D0*
     &T65+4.6666666666666666667D0*T79+T289-0.66666666666666666667D0*T66+
     &T89-9.0000000000000000001D0*T46+15.0D0*T47-6.0000000000000000001D0
     &*T48+14.0D0*T49-13.0D0*T20+11.0D0*T21-10.0D0*T50+15.16666666666666
     &6667D0*T10+7.8333333333333333334D0*T11+15.166666666666666667D0*T12
     &+7.8333333333333333334D0*T13+T42-T43)*T2+(T347-T348-T142+T349+T144
     &+T121-T122+1.3333333333333333333D0*T79+2.6666666666666666667D0*T80
     &-1.3333333333333333333D0*T66-1.6111111111111111111D0*T45-0.8333333
     &3333333333334D0*T46-15.833333333333333333D0*T47+9.3888888888888888
     &889D0*T48-1.6111111111111111111D0*T49-0.83333333333333333334D0*T20
     &-15.833333333333333333D0*T21+9.3888888888888888889D0*T50-6.8703703
     &703703703704D0*T10+0.2037037037037037037D0*T11-6.87037037037037037
     &04D0*T12+0.2037037037037037037D0*T13+T81+T82)*CA*CF
      T392=T390+T391*im*pi+(-T351-T352+T353-T354+T355+T356-0.44444444444
     &444444445D0*T84-0.44444444444444444445D0*T78+0.4444444444444444444
     &5D0*T64-0.44444444444444444445D0*T65+0.44444444444444444445D0*T79+
     &0.44444444444444444445D0*T80+T85-0.074074074074074074074D0*T46-0.0
     &74074074074074074074D0*T47+T86+T87-1.4074074074074074074D0*T20-1.4
     &074074074074074074D0*T21-T88+3.7777777777777777778D0*T10+3.7777777
     &777777777778D0*T11+3.7777777777777777778D0*T12+3.77777777777777777
     &78D0*T13-2.4814814814814814815D0*T8-2.4814814814814814815D0*T9)*CF
     &*Nf+(T358+T359+T360+T361+T362+T363+T364+T365+T366+T367+T368+T369+0
     &.66666666666666666667D0*T128+0.66666666666666666667D0*T129+6.00000
     &00000000000001D0*T103+1.3333333333333333333D0*T104+1.3333333333333
     &333333D0*T105+6.0000000000000000001D0*T106+T136+1.3333333333333333
     &333D0*T108+1.3333333333333333333D0*T109+6.0000000000000000001D0*T1
     &10+0.66666666666666666667D0*T111+0.66666666666666666667D0*T112-19.
     &0D0*T59-1.0D0*T75-1.0D0*T76-9.0000000000000000001D0*T60-21.0D0*T61
     &-5.0D0*T77-5.0D0*T83-11.0D0*T62-15.0D0*T63-1.0D0*T84-1.0D0*T78-13.
     &0D0*T64-T370-5.0D0*T79-5.0D0*T80-15.0D0*T66-T89-4.3333333333333333
     &333D0*T46-4.3333333333333333333D0*T47+T90-16.0D0*T49+T123-2.333333
     &3333333333333D0*T21+T91-8.3333333333333333334D0*T10+T92-6.66666666
     &66666666667D0*T12-5.0D0*T13-5.8333333333333333333D0*T8-5.833333333
     &3333333333D0*T9)*T2
      T393=T373+T374-T375-T376+T147+T148-T377-T378-T379-T380-4.0D0*T102-
     &4.0D0*T103-2.6666666666666666667D0*T104-2.6666666666666666667D0*T1
     &05+T152-4.0D0*T107-T153+T130-2.6666666666666666667D0*T110-T141+1.3
     &333333333333333333D0*T111+1.3333333333333333333D0*T112+6.500000000
     &0000000001D0*T59+4.8888888888888888889D0*T75+4.8888888888888888889
     &D0*T76+1.6111111111111111111D0*T60+13.388888888888888889D0*T61-4.8
     &888888888888888889D0*T77-4.8888888888888888889D0*T83+4.5D0*T62+6.5
     &000000000000000001D0*T63+4.8888888888888888889D0*T84+4.88888888888
     &88888889D0*T78+1.6111111111111111111D0*T64+13.388888888888888889D0
     &*T65-4.8888888888888888889D0*T79-4.8888888888888888889D0*T80+4.5D0
     &*T66+1.1666666666666666667D0*T45-2.037037037037037037D0*T46-2.0370
     &37037037037037D0*T47+1.8333333333333333333D0*T48+6.500000000000000
     &0001D0*T49+3.2962962962962962963D0*T20+3.2962962962962962963D0*T21
     &+7.1666666666666666667D0*T50-13.444444444444444445D0*T10-18.111111
     &111111111111D0*T11-18.111111111111111111D0*T12-22.7777777777777777
     &78D0*T13
      T394=(T174+0.058201058201058201059D0*CA*CF)*T175+(T176-0.316666666
     &66666666667D0*T2-0.11157407407407407407D0*CA*CF)*im*T113+(-0.15432
     &098765432098765D0*CF*Nf+(T177+T178+T179-2.7240740740740740741D0)*T
     &2+(0.17222222222222222222D0*T12-0.56944444444444444445D0*T8-T180+0
     &.88811728395061728395D0)*CA*CF)*T68+((T181-T81-T182-1.111111111111
     &1111111D0)*CF*Nf+(T183+T70+T114-T184-T185-2.6666666666666666667D0*
     &T12-T58-1.1666666666666666667D0*T9-7.2500000000000000001D0)*T2+(T1
     &86+T70+T114+T115+T116-11.111111111111111111D0*T12+T187+15.86111111
     &1111111111D0*T8-4.5694444444444444445D0*T9+22.509259259259259259D0
     &)*CA*CF)*im*T41+((T117+T118-T51-T188-3.0370370370370370371D0*T12-T
     &127+T119-1.0740740740740740741D0*T9-3.7901234567901234568D0)*CF*Nf
     &+(-T120+T121+T122+T189+T190+T191-T192+T193+T194-T195+3.33333333333
     &33333334D0*T49+T123+7.3333333333333333334D0*T21-T197-10.6666666666
     &66666667D0*T10+10.222222222222222222D0*T11-13.111111111111111111D0
     &*T12-8.3333333333333333334D0*T13+2.7222222222222222222D0*T8-T74-2.
     &2222222222222222222D0)*T2+(T198+T199+2.3333333333333333333D0*T64+T
     &200+4.1666666666666666667D0*T79+T201+T124-1.0D0*T46+T203-4.8333333
     &333333333333D0*T48-10.027777777777777778D0*T49+0.33333333333333333
     &333D0*T20+6.0833333333333333334D0*T21+6.0000000000000000001D0*T10+
     &0.98611111111111111112D0*T11+0.27314814814814814815D0*T12-5.333333
     &3333333333333D0*T13+4.0509259259259259259D0*T8-3.03240740740740740
     &74D0*T9+34.530864197530864198D0)*CA*CF)*T15
      T395=(T204+T205+T125-T145+T126+T206-T70-3.1111111111111111111D0*T2
     &1+1.3333333333333333333D0*T10-3.6666666666666666667D0*T11+11.77777
     &7777777777778D0*T12+T127-6.4814814814814814815D0*T8+5.370370370370
     &3703704D0*T9-2.4814814814814814815D0)*CF*Nf+(T207+T208+T209+T210-T
     &135+T211+T130+T212+T213+T132-T138+T215-T216-T217-T218+T219+T133-T2
     &20-T221+18.0D0*T78-T222+10.0D0*T65-T223+T224-14.666666666666666667
     &D0*T45+2.6666666666666666667D0*T46-21.333333333333333333D0*T47+0.3
     &3333333333333333333D0*T48-56.333333333333333334D0*T49+1.3333333333
     &333333333D0*T20-30.666666666666666667D0*T21+13.666666666666666667D
     &0*T50-1.1666666666666666667D0*T10-9.8333333333333333334D0*T11+2.16
     &66666666666666667D0*T12+T134-38.0D0*T8-8.3333333333333333334D0*T9+
     &0.83333333333333333334D0)*T2+(T128+T226+T227+T228+3.0D0*T104+T135+
     &T229+T136+16.0D0*T108+T230+T231+T232+T233-T137+T138-T234+T235+T61-
     &5.5D0*T77-T236+T237-T238-55.5D0*T84+17.166666666666666667D0*T78-T1
     &39+20.0D0*T65-20.5D0*T79+8.8333333333333333334D0*T80+2.58333333333
     &33333333D0*T45+88.75D0*T46-30.916666666666666667D0*T47+3.416666666
     &6666666667D0*T48-14.833333333333333333D0*T49+T44-7.555555555555555
     &5556D0*T21+T140+22.055555555555555556D0*T10+81.138888888888888889D
     &0*T11-63.388888888888888889D0*T12+18.694444444444444444D0*T13+87.0
     &92592592592592593D0*T8+11.018518518518518519D0*T9+13.2592592592592
     &59259D0)*CA*CF
      T396=T394+T395*im*pi+(T240+T241+T141+T242+T142-T243-T143+T144-T245
     &-T246-T247+3.1111111111111111111D0*T65+2.6666666666666666667D0*T45
     &+T145+1.3333333333333333333D0*T47-1.4444444444444444445D0*T48-12.7
     &77777777777777778D0*T49-T250-T251+5.5555555555555555556D0*T10-4.70
     &37037037037037037D0*T11-4.7037037037037037037D0*T12-2.222222222222
     &2222222D0*T13+11.296296296296296296D0*T8+8.8148148148148148148D0*T
     &9)*CF*Nf+(T252+T253+T254+T255+T256+T257+T258+T259+T260+T261+T262+T
     &263+T264+T265-T266-T267-T268-T269+T147+T148-T270+T271+T272+T273-T2
     &74-T275-T276-T277+T278+T279+T280+T281-16.0D0*T103+T282+T283+T152+T
     &153+T154-T285-T286+T322-22.666666666666666667D0*T75-22.66666666666
     &6666667D0*T76-23.0D0*T60+31.0D0*T61+6.6666666666666666667D0*T77+6.
     &6666666666666666667D0*T83+7.0000000000000000001D0*T62+65.0D0*T63+2
     &0.666666666666666667D0*T84+20.666666666666666667D0*T78+22.0D0*T64+
     &29.0D0*T65-1.6666666666666666667D0*T79-1.6666666666666666667D0*T80
     &+12.0D0*T66-7.0000000000000000001D0*T45+T371+T372-22.0D0*T48-T155+
     &2.1666666666666666667D0*T20+2.1666666666666666667D0*T21+T239+45.83
     &3333333333333333D0*T10-T52-T53-15.5D0*T13+27.0D0*T8+27.83333333333
     &3333333D0*T9)*T2
      T397=T156+T291+T292+T293+T294+T295+T296+7.0000000000000000001D0*T1
     &61+T297+T298+T299+T300+T301+T302+24.0D0*T166+T303+T304+T305+T306+T
     &307+T308-T309-T310+T311+T312+T169+T170+T313-T314-1.0D0*T100-1.0D0*
     &T101-6.5000000000000000001D0*T102+T315-T316-T317-T318-T319-T320-71
     &.0D0*T151-10.666666666666666667D0*T103+6.5000000000000000001D0*T10
     &4+6.5000000000000000001D0*T105+T321-13.166666666666666667D0*T107+2
     &1.5D0*T108+21.5D0*T109+T110-8.8333333333333333334D0*T131-3.0D0*T59
     &-0.41666666666666666667D0*T75-0.41666666666666666667D0*T76+95.6666
     &66666666666667D0*T60+20.166666666666666667D0*T61-3.416666666666666
     &6667D0*T77-3.4166666666666666667D0*T83+T323-32.333333333333333333D
     &0*T84-32.333333333333333333D0*T78-28.333333333333333333D0*T64+3.55
     &55555555555555556D0*T65-4.0D0*T79-4.0D0*T80-6.0000000000000000001D
     &0*T66-21.083333333333333333D0*T45+15.638888888888888889D0*T46+15.6
     &38888888888888889D0*T47+91.888888888888888889D0*T48+48.58333333333
     &3333334D0*T49-19.694444444444444444D0*T20
      T398=(T174+0.15661375661375661376D0*CA*CF)*T175+(T176-0.2388888888
     &8888888889D0*T2-0.1587962962962962963D0*CA*CF)*im*T113+(-0.1987654
     &320987654321D0*CF*Nf+(T177-T178-T179-0.97314814814814814815D0)*T2+
     &(-0.027777777777777777778D0*T12+0.66944444444444444445D0*T8+T180+0
     &.92561728395061728396D0)*CA*CF)*T68+((T181+T81+T182+0.888888888888
     &88888889D0)*CF*Nf+(T183+T70+T114+T184+T185+5.3333333333333333333D0
     &*T12+T58-4.0833333333333333333D0*T8+0.58333333333333333334D0*T9-0.
     &58333333333333333334D0)*T2+(T186+2.6666666666666666667D0*T20+5.666
     &6666666666666667D0*T21-T115+T52+0.88888888888888888889D0*T12-T187-
     &30.777777777777777778D0*T8-1.7638888888888888889D0*T9-20.856481481
     &481481482D0)*CA*CF)*im*T41+((T117+T118+T51+T188+4.9629629629629629
     &63D0*T12+T127-6.3888888888888888889D0*T8+1.2962962962962962963D0*T
     &9+0.56790123456790123457D0)*CF*Nf+(-T120+T121+T122+T189+T190+T191+
     &T192-T193-T194+T195-4.6666666666666666667D0*T49+T196+10.0D0*T21+T1
     &97+15.666666666666666667D0*T10-12.972222222222222222D0*T11+25.3611
     &11111111111111D0*T12+T58-26.694444444444444445D0*T8+T27-2.61111111
     &11111111111D0)*T2+(T198+T199+4.3333333333333333333D0*T64+T200+8.16
     &66666666666666667D0*T79+T201+T202+T46-T203+3.8333333333333333334D0
     &*T48+3.1388888888888888889D0*T49+11.0D0*T20-1.75D0*T21-14.41666666
     &6666666667D0*T10-15.152777777777777778D0*T11-0.3101851851851851851
     &9D0*T12-3.6666666666666666667D0*T13+2.1851851851851851852D0*T8-26.
     &884259259259259259D0*T9-19.086419753086419753D0)*CA*CF)*T15
      T399=(T204+T205-T125+T145-T126-T206+T70-8.4444444444444444445D0*T2
     &1-2.0D0*T10+3.0D0*T11-9.7777777777777777778D0*T12-T127+22.81481481
     &4814814815D0*T8-6.4814814814814814815D0*T9+8.8148148148148148148D0
     &)*CF*Nf+(T207+T208+T209+T210-T135+T211-T154+T212+T213+T214+T138-T2
     &15+T216+T217+T218-T219-T133+T220+T221-14.0D0*T78+T222+18.0D0*T65+T
     &223-T224-14.333333333333333333D0*T45+0.33333333333333333333D0*T46+
     &24.333333333333333333D0*T47+T86+55.333333333333333334D0*T49+1.6666
     &666666666666667D0*T20-8.3333333333333333334D0*T21+T225-69.16666666
     &6666666667D0*T10-14.333333333333333333D0*T11+4.6666666666666666667
     &D0*T12-14.833333333333333333D0*T13+75.833333333333333333D0*T8-36.3
     &33333333333333333D0*T9+14.5D0)*T2+(T128+T226+T227+T228+9.000000000
     &0000000001D0*T104+T135+T229+T136+40.0D0*T108+T230+T231+T232+T233+T
     &137-T138+T234-T235-1.0D0*T61+2.5D0*T77+T236-T237+T238+27.5D0*T84+4
     &4.166666666666666667D0*T78-12.0D0*T64+16.0D0*T65-11.5D0*T79+15.833
     &333333333333333D0*T80-12.583333333333333333D0*T45-185.25D0*T46+22.
     &916666666666666667D0*T47-0.91666666666666666667D0*T48+20.333333333
     &333333333D0*T49-45.5D0*T20+42.277777777777777778D0*T21+T239-43.888
     &888888888888889D0*T10-82.638888888888888889D0*T11+7.55555555555555
     &55556D0*T12+9.8055555555555555556D0*T13-194.12037037037037037D0*T8
     &+54.481481481481481482D0*T9-62.50925925925925926D0)*CA*CF
      T400=T398+T399*im*pi+(T240+T241+T141-T242-T142+T243+T143+T244+T245
     &+T246+T247+8.4444444444444444445D0*T65-9.3333333333333333334D0*T45
     &+T248-T249+0.77777777777777777778D0*T48+10.111111111111111111D0*T4
     &9+T250+T251+T225-26.444444444444444445D0*T10+1.2592592592592592593
     &D0*T11+1.2592592592592592593D0*T12-3.2222222222222222222D0*T13-15.
     &148148148148148148D0*T8-6.3333333333333333334D0*T9)*CF*Nf+(T252+T2
     &53+T254+T255+T256+T257+T258+T259+T260+T261+T262+T263+T264+T265+T26
     &6+T267+T268+T269+T169+T170+T270-T271-T272-T273+T274+T275+T276+T277
     &-T278-T279-T280-T281+T209-T282-T283+T284-16.0D0*T107-T153-T154+T28
     &5+T286+70.0D0*T59+55.666666666666666667D0*T75+55.66666666666666666
     &7D0*T76+59.0D0*T60-28.0D0*T61-0.66666666666666666667D0*T77-T287-48
     &.0D0*T63-4.6666666666666666667D0*T84+T288-3.0D0*T64+9.000000000000
     &0000001D0*T65+0.66666666666666666667D0*T79+T289+T290+80.0D0*T45+31
     &.833333333333333333D0*T46+31.833333333333333333D0*T47+17.0D0*T48-T
     &357-0.16666666666666666667D0*T20-0.16666666666666666667D0*T21-12.0
     &D0*T50-77.5D0*T10+9.3333333333333333334D0*T11+9.333333333333333333
     &4D0*T12-22.5D0*T13-36.5D0*T8-22.0D0*T9)*T2
      T401=T156+T291+T292+T293+T294+T295+T296+13.0D0*T161+T297+T298+T299
     &+T300+T301+T302+48.0D0*T166+T303+T304+T305+T306+T307+T308+T309+T31
     &0-T311-T312-T169-T170-T313+T314+T100+T101+3.5D0*T102-T315+T316+T31
     &7+T318+T319+T320+43.0D0*T151-34.666666666666666667D0*T103+9.500000
     &0000000000001D0*T104+9.5000000000000000001D0*T105-T321-20.16666666
     &6666666667D0*T107+10.5D0*T108+10.5D0*T109-1.0D0*T110-15.8333333333
     &33333333D0*T131+T322-20.583333333333333333D0*T75-20.58333333333333
     &3333D0*T76-213.16666666666666667D0*T60-17.166666666666666667D0*T61
     &-1.5833333333333333333D0*T77-1.5833333333333333333D0*T83-2.5D0*T62
     &-T323+37.833333333333333333D0*T84+37.833333333333333333D0*T78-7.66
     &66666666666666667D0*T64-45.277777777777777778D0*T65-3.0D0*T79-3.0D
     &0*T80+52.25D0*T45-42.972222222222222222D0*T46-42.97222222222222222
     &2D0*T47-124.38888888888888889D0*T48-20.083333333333333333D0*T49-11
     &.305555555555555556D0*T20-11.305555555555555556D0*T21-8.8333333333
     &333333334D0*T50
      T402=(0.29814814814814814815D0*T8*T2-0.25555555555555555556D0*T8*C
     &A*CF)*T68+(0.22222222222222222222D0*T8*CF*Nf+(T324-T325+6.16666666
     &66666666667D0*T8+1.25D0)*T2+(T326+1.4444444444444444445D0*T11+T384
     &+18.467592592592592593D0*T8+4.7083333333333333333D0)*CA*CF)*im*T41
     &+((T327-T328+6.6728395061728395062D0*T8+1.5D0)*CF*Nf+(-1.0D0*T45+T
     &329+T330-1.0D0*T48-T38+6.8333333333333333334D0*T11+4.1666666666666
     &666667D0*T12+33.277777777777777778D0*T8+T29+8.4166666666666666667D
     &0)*T2+(-T125+T331+T332+1.2222222222222222222D0*T48+T333+9.57407407
     &40740740741D0*T10+23.759259259259259259D0*T11+0.750000000000000000
     &01D0*T12-7.1219135802469135803D0*T8+5.4166666666666666667D0*T9-7.5
     &555555555555555556D0)*CA*CF)*T15+((T334-T335-T336+T337+0.740740740
     &74074074074D0*T10-0.74074074074074074074D0*T11-4.33333333333333333
     &33D0*T12-20.777777777777777778D0*T8-6.3333333333333333334D0)*CF*Nf
     &+(-T338+T339+T340-T341+T342-T343-T287+T344+20.0D0*T45+T345-17.0D0*
     &T47+T48+7.0000000000000000001D0*T49+T333+T346+75.83333333333333333
     &3D0*T10+6.1666666666666666667D0*T11-18.0D0*T12-50.3333333333333333
     &34D0*T8+1.5D0*T9-15.333333333333333333D0)*T2+(-T347+T348+T142-T349
     &+4.3333333333333333333D0*T77+12.0D0*T84+13.861111111111111111D0*T4
     &5+130.58333333333333333D0*T46+25.833333333333333333D0*T47-11.88888
     &8888888888889D0*T48+T350+0.5D0*T20+T118+35.870370370370370371D0*T1
     &0+0.71296296296296296297D0*T11+44.583333333333333333D0*T12-2.0D0*T
     &13+141.88888888888888889D0*T8-5.9166666666666666667D0*T9+49.25D0)*
     &CA*CF)*im*pi
      T403=T402+(T351+T352-T353+T354-T355-T356+8.0D0*T45+0.7407407407407
     &4074074D0*T46+0.74074074074074074074D0*T47+T357+22.777777777777777
     &778D0*T10+6.3333333333333333334D0*T8)*CF*Nf+(-T358-T359-T360-T361-
     &T362-T363-T364-T365-T366-T367-T368-T369-60.0D0*T59-40.0D0*T75-40.0
     &D0*T76-39.0D0*T60+17.0D0*T61-1.0D0*T77-1.0D0*T83-9.000000000000000
     &0001D0*T63-T221-2.0D0*T78+T370-72.0D0*T45+T371+T372-8.0D0*T48+18.0
     &D0*T49+53.333333333333333334D0*T10+T383+T382+15.333333333333333333
     &D0*T8)*T2+(-T373-T374+T375+T376+T169+T170+T377+T378+T379+T380+7.00
     &00000000000000001D0*T102+12.0D0*T151+9.0000000000000000001D0*T59+2
     &2.861111111111111111D0*T75+22.861111111111111111D0*T76+155.8888888
     &8888888889D0*T60-16.388888888888888889D0*T61+11.888888888888888889
     &D0*T77+11.888888888888888889D0*T83-4.5D0*T63-0.5D0*T84-0.5D0*T78+T
     &381-42.583333333333333333D0*T45+37.287037037037037037D0*T46+37.287
     &037037037037037D0*T47+38.0D0*T48-42.583333333333333333D0*T49+T333+
     &T118-146.97222222222222222D0*T10+5.9166666666666666667D0*T11+5.916
     &6666666666666667D0*T12-49.25D0*T8)*CA*CF
      T404=((9.3333333333333333334D0*T2-14.0D0*CA*CF)*T14+(T3+T4-14.0D0*
     &T2-5.9166666666666666667D0*CA*CF)*T5+(T3+T4+24.0D0*T2-47.416666666
     &666666667D0*CA*CF)*T7)*zeta5+((T6+14.5D0*CA*CF)*T5+(T6+38.5D0*CA*C
     &F)*T7)*zeta3*zeta3+T388+T389*T67+(T392+(T393+17.259259259259259259
     &D0*T8+17.259259259259259259D0*T9)*CA*CF)*T14+(T396+(T397-19.694444
     &444444444444D0*T21+T173-56.916666666666666667D0*T10+50.95370370370
     &3703704D0*T11+50.953703703703703704D0*T12+77.861111111111111111D0*
     &T13-83.768518518518518519D0*T8-70.50925925925925926D0*T9)*CA*CF)*T
     &5+(T400+(T401+179.91666666666666667D0*T10-41.092592592592592593D0*
     &T11-41.092592592592592593D0*T12+2.3888888888888888889D0*T13+115.75
     &925925925925926D0*T8+53.25D0*T9)*CA*CF)*T7+T403*x+(-1.25D0*T8*T2-4
     &.7083333333333333333D0*T8*CA*CF)*im*T41+(-1.5D0*T8*CF*Nf+(1.75D0*T
     &10-1.3333333333333333333D0*T11+T382-8.2500000000000000001D0*T8)*T2
     &+(-3.1666666666666666667D0*T10-5.4166666666666666667D0*T11+2.13888
     &88888888888889D0*T8)*CA*CF)*T15+((T12+T17)*CF*Nf+(-5.0D0*T45+3.0D0
     &*T47-3.0D0*T49+3.0D0*T21-19.0D0*T10+T383-T384+12.83333333333333333
     &3D0*T8)*T2+(-4.25D0*T45-31.25D0*T46-T126+T90-7.8333333333333333334
     &D0*T10+7.9166666666666666667D0*T11-8.2500000000000000001D0*T12-35.
     &083333333333333333D0*T8)*CA*CF)*im*pi
      gammaHPL_3l_nfv2=(T404+(-T385-1.0D0*T49-T40)*CF*Nf+(13.0D0*T59+8.0D0*T75+T234+T
     &235-3.0D0*T61+3.0D0*T63-3.0D0*T65+16.0D0*T45+1.5D0*T46+T194+T155-1
     &2.833333333333333333D0*T10)*T2+(-2.5D0*T59-6.7500000000000000001D0
     &*T75-6.7500000000000000001D0*T76-38.0D0*T60+T386-2.0D0*T77-2.0D0*T
     &83+10.916666666666666667D0*T45-7.9166666666666666667D0*T46-7.91666
     &66666666666667D0*T47+8.2500000000000000001D0*T49+35.08333333333333
     &3333D0*T10)*CA*CF)/(T5-2.0D0*T7+x)


      end function gammaHPL_3l_nfv2

      function gammaHPL_3l_nfv(x)
        use types
        use constants
        implicit complex(dp) (T)
        include 'nf.f'
        complex(dp) :: gammaHPL_3l_nfv
        real(dp), intent(in) :: x

        real(dp) :: dabc2

        dabc2 = (nc**2-4._dp)*(nc**2-1._dp)/nc

      T2=-2.5D0*Hr1(0)+1.6666666666666666667D0
      T3=x**3
      T4=x*x
      T5=Hr1(0)
      T6=Hr1(1)
      T7=Hr2(0,1)
      T8=0.66666666666666666667D0*im*pi**3
      T9=-1.0D0*T5
      T10=pi*pi
      T11=-2.0D0*T7
      T12=2.0D0*Hr2(0,0)
      T13=4.0D0*T6
      T14=4.0D0*T5
      T15=-2.0D0*Hr3(0,1,1)
      T16=-2.0D0*Hr3(0,0,1)
      T17=4.0D0*Hr2(1,1)
      T18=pi**3
      T19=Hr2(1,1)
      T20=Hr2(1,0)
      T21=Hr2(0,0)
      T22=Hr3(0,1,1)
      T23=x**5
      T24=pi**4
      T25=0.083333333333333333333D0*T20
      T26=0.083333333333333333333D0*T21
      T27=Hr3(1,1,1)
      T28=Hr3(1,0,0)
      T29=Hr3(0,0,1)
      T30=Hr3(0,0,0)
      T31=2.5D0*T5
      T32=Hr4(1,1,1,1)
      T33=Hr4(1,1,1,0)
      T34=Hr4(1,1,0,1)
      T35=Hr4(1,1,0,0)
      T36=Hr4(1,0,1,1)
      T37=Hr4(1,0,1,0)
      T38=Hr4(1,0,0,1)
      T39=Hr4(1,0,0,0)
      T40=Hr4(0,1,1,1)
      T41=Hr4(0,1,1,0)
      T42=Hr4(0,1,0,1)
      T43=Hr4(0,1,0,0)
      T44=Hr4(0,0,1,1)
      T45=Hr4(0,0,1,0)
      T46=Hr4(0,0,0,1)
      T47=Hr4(0,0,0,0)
      T48=Hr3(1,1,0)
      T49=Hr3(1,0,1)
      T50=Hr3(0,1,0)
      T51=x**4
      T52=0.5D0*T41
      T53=0.5D0*T45
      T54=0.66666666666666666667D0*T19
      T55=2.0D0*T33
      T56=2.0D0*T41
      T57=1.5D0*T30
      T58=Hr5(0,1,1,0,0)
      T59=Hr5(0,1,0,1,1)
      T60=2.0D0*Hr5(0,1,0,0,1)
      T61=Hr5(0,0,1,1,1)
      T62=Hr5(0,0,0,1,1)
      T63=2.0D0*T34
      T64=0.5D0*T46
      T65=2.0D0*T22
      T66=0.025286596119929453263D0*pi**6
      T67=0.013888888888888888889D0*T5
      T68=pi**5
      T69=-0.061111111111111111111D0*T7
      T70=0.061111111111111111111D0*T21
      T71=-0.33333333333333333333D0*T22
      T72=-0.33333333333333333333D0*T29
      T73=0.66666666666666666667D0*T7
      T74=0.33333333333333333333D0*T42
      T75=0.66666666666666666667D0*T44
      T76=1.3333333333333333333D0*T22
      T77=-1.0D0*Hr5(0,1,1,1,0)
      T78=-1.0D0*T58
      T79=2.0D0*T59
      T80=4.0D0*T61
      T81=-1.0D0*Hr5(0,0,1,1,0)
      T82=4.0D0*Hr5(0,0,1,0,1)
      T83=-1.0D0*Hr5(0,0,1,0,0)
      T84=6.0D0*T62
      T85=6.0D0*Hr5(0,0,0,0,1)
      T86=2.0D0*T35
      T87=4.0D0*T36
      T88=4.0D0*T38
      T89=8.0D0*T40
      T90=8.0D0*T42
      T91=2.0D0*T43
      T92=12.0D0*T44
      T93=12.0D0*T46
      T94=-1.0D0*Hr6(0,1,1,1,0,0)
      T95=Hr6(0,1,1,0,1,1)
      T96=Hr6(0,1,1,0,1,0)
      T97=Hr6(0,1,1,0,0,1)
      T98=-2.0D0*Hr6(0,1,0,1,1,1)
      T99=-2.0D0*Hr6(0,1,0,0,1,1)
      T100=-4.0D0*Hr6(0,0,1,1,1,1)
      T101=-1.0D0*Hr6(0,0,1,1,0,0)
      T102=-3.0D0*Hr6(0,0,1,0,1,1)
      T103=Hr6(0,0,1,0,1,0)
      T104=Hr6(0,0,1,0,0,1)
      T105=-6.0D0*Hr6(0,0,0,1,1,1)
      T106=-6.0D0*Hr6(0,0,0,0,1,1)
      T107=2.0D0*Hr5(1,1,1,0,0)
      T108=2.0D0*Hr5(1,1,0,1,1)
      T109=2.0D0*Hr5(1,1,0,1,0)
      T110=2.0D0*Hr5(1,1,0,0,1)
      T111=4.0D0*Hr5(1,0,1,1,1)
      T112=Hr5(1,0,0,1,1)
      T113=8.0D0*Hr5(0,1,1,1,1)
      T114=2.0D0*T58
      T115=6.0D0*T59
      T116=2.0D0*Hr5(0,1,0,1,0)
      T117=Hr5(0,0,1,1,0)
      T118=Hr5(0,0,1,0,1)
      T119=Hr5(0,0,1,0,0)
      T120=12.0D0*T62
      T121=0.5D0*T37
      T122=0.5D0*T38
      T123=2.0D0*T42
      T124=2.0D0*T45
      T125=2.0D0*T46
      T126=2.0D0*T47
      T127=1.75D0*T29
      T128=2.5D0*T7
      T129=2.25D0*T6
      T130=-1.0D0*T48
      T131=-0.75000000000000000001D0*T28
      T132=2.5D0*T22
      T133=0.125D0*T20
      T134=0.25D0*T19
      T135=0.125D0*T7
      T136=(T2*T3+T2*T4-3.3333333333333333334D0*x)*zeta5+(2.0D0*T3+2.0D0
     &*T4)*zeta3*zeta3+(-2.5D0*T23+(4.5D0*im*pi+1.5D0*T6+1.5D0*T5+8.6250
     &000000000000001D0)*T51+(T8+(T9+2.0D0)*T10+(T11+T12+T13-T14-8.0D0)*
     &im*pi+T15+T16+T17+4.0D0*T7-0.5D0*T6-1.25D0*T5-9.166666666666666666
     &7D0)*T3+(T8+(T9-1.6666666666666666667D0)*T10+(T11+T12-T13+T14-4.0D
     &0)*im*pi+T15+T16-T17+T20-5.0D0*T7+T129+2.0D0*T5+5.0833333333333333
     &333D0)*T4+(im*pi-4.25D0*T6-2.9166666666666666667D0)*x+T6)*zeta3+(0
     &.020833333333333333333D0*T24-1.25D0*im*T18+(0.08333333333333333333
     &3D0*T19+T25+0.083333333333333333333D0*T7+T26+1.625D0)*T10+(-2.5D0*
     &T19-2.5D0*T20-T128-2.5D0*T21)*im*pi+0.5D0*T32+0.5D0*T33+0.5D0*T34+
     &0.5D0*T35+0.5D0*T36+T121+T122+0.5D0*T39+0.5D0*T40+T52+0.5D0*T42+0.
     &5D0*T43+0.5D0*T44+T53+T64+0.5D0*T47+2.5D0*T27-2.5D0*T28+T132-2.5D0
     &*T30+3.25D0*T19+3.25D0*T20+3.25D0*T7+3.25D0*T21)*T23+(0.125D0*T24+
     &(-0.25D0*T6-0.25D0*T5+4.125D0)*im*T18+(0.16666666666666666667D0*T1
     &9-T25+0.16666666666666666667D0*T7-T26+0.41666666666666666667D0*T6+
     &0.22916666666666666667D0*T5-4.15625D0)*T10+(3.0D0*T27-1.5D0*T48+3.
     &0D0*T49-1.5D0*T28+3.0D0*T22-1.5D0*T50+3.0D0*T29-T57+8.250000000000
     &0000001D0*T19+8.2500000000000000001D0*T20+8.2500000000000000001D0*
     &T7+8.2500000000000000001D0*T21+2.5D0*T6+T31)*im*pi-5.0D0*T32-T55-T
     &63-3.5D0*T35-5.0D0*T36-2.0D0*T37-2.0D0*T38-3.5D0*T39-5.0D0*T40-T56
     &-T123-3.5D0*T43-5.0D0*T44-T124-T125-3.5D0*T47-6.125000000000000000
     &1D0*T27+2.125D0*T48+2.125D0*T49+10.375D0*T28-6.5000000000000000001
     &D0*T22+1.75D0*T50+T127+10.0D0*T30-10.8125D0*T19-8.3125000000000000
     &001D0*T20-8.3125000000000000001D0*T7-5.8125D0*T21-0.75000000000000
     &000001D0*T6-0.75000000000000000001D0*T5)*T51
      T137=T66+(T67-0.027777777777777777778D0)*im*T68+(T69+T70+0.1222222
     &2222222222222D0*T6-0.12222222222222222222D0*T5-0.30902777777777777
     &778D0)*T24+(T71+T72+T54+T73+T6-3.1666666666666666667D0)*im*T18+(T5
     &2+T74+T75+T53+T46+T130-0.66666666666666666667D0*T49-T76-1.0D0*T50+
     &T16-T54-0.58333333333333333334D0*T20-0.33333333333333333333D0*T7-0
     &.87500000000000000001D0*T21-2.0416666666666666667D0*T6-0.916666666
     &66666666667D0*T5+2.0D0)*T10+(T77+T78+T79+T60+T80+T81+T82+T83+T84+T
     &85+T55+T86-T87-T88-T89+T56-T90+T91-T92-T93-5.0D0*T27+4.0D0*T48-5.5
     &D0*T49+3.5D0*T28-7.0000000000000000001D0*T22+2.0D0*T50-7.500000000
     &0000000001D0*T29+T57-9.0000000000000000001D0*T19-5.75D0*T20-9.0000
     &000000000000001D0*T7-5.75D0*T21-7.0000000000000000001D0*T6-7.00000
     &00000000000001D0*T5-1.25D0)*im*pi+T94+T95+T96+T97+T98+T99+T100+T10
     &1+T102+T103+T104+T105+T106+T107-T108-T109-T110+T111+4.0D0*T112+T11
     &3+T114+T115-T116
      T138=T66+(T67+0.027777777777777777778D0)*im*T68+(T69+T70-0.1694444
     &4444444444444D0*T6+0.072222222222222222223D0*T5-0.1895833333333333
     &3333D0)*T24+(T71+T72-T54-T73-0.54166666666666666667D0*T6-0.2916666
     &6666666666667D0*T5+0.35416666666666666667D0)*im*T18+(T52+T74+T75+T
     &53+T46+T48+0.83333333333333333334D0*T49-0.33333333333333333333D0*T
     &28+T76+T50+2.3333333333333333333D0*T29-0.16666666666666666667D0*T3
     &0+0.83333333333333333334D0*T19-T133+1.25D0*T7+0.125D0*T21+1.541666
     &6666666666667D0*T6+0.375D0*T5+0.98958333333333333334D0)*T10+(T77+T
     &78+T79+T60+T80+T81+T82+T83+T84+T85-T55-T86+T87+T88+T89-T56+T90-T91
     &+T92+T93+T27-1.75D0*T48+1.5D0*T49-1.25D0*T28+5.5D0*T22-0.5D0*T50+6
     &.0000000000000000001D0*T29+T134+0.375D0*T20+0.5D0*T7+0.625D0*T21+2
     &.8125D0*T6+1.3125D0*T5+3.4375D0)*im*pi+T94+T95+T96+T97+T98+T99+T10
     &0+T101+T102+T103+T104+T105+T106-T107+T108+T109+T110-T111-5.0D0*T11
     &2-1.0D0*Hr5(1,0,0,1,0)-1.0D0*Hr5(1,0,0,0,1)
      T139=T136+(T137-T60+12.0D0*T61+T120+7.0000000000000000001D0*T32+T5
     &5+T63+6.0000000000000000001D0*T35+6.5000000000000000001D0*T36+T37+
     &T38+4.5D0*T39+9.7500000000000000001D0*T40+2.75D0*T41+2.75D0*T42+4.
     &75D0*T43+8.0D0*T44+T53+T64+T126+0.25D0*T27-8.7500000000000000001D0
     &*T48-8.7500000000000000001D0*T49-14.5D0*T28+T65-7.0000000000000000
     &001D0*T50-7.0000000000000000001D0*T29-12.75D0*T30+10.875D0*T19+3.8
     &75D0*T20+3.875D0*T7-3.125D0*T21+3.6875D0*T6+2.4375D0*T5)*T3+(T138-
     &1.0D0*Hr5(1,0,0,0,0)-T113-T114-T115+T116+T60-1
     &1.0D0*T61+T117+T118+T119-T120+T33+T34-0.75000000000000000001D0*T35
     &-2.0D0*T36-T121-T122-1.75D0*T39-3.5D0*T40+T56+T123+1.5D0*T43-4.0D0
     &*T44+T124+T125+T126+7.5000000000000000001D0*T27+7.75D0*T48+7.75D0*
     &T49+8.1250000000000000001D0*T28+4.625D0*T22+5.125D0*T50+5.125D0*T2
     &9+5.75D0*T30-1.75D0*T19+1.0625D0*T20+1.0625D0*T7+2.375D0*T21-5.125
     &D0*T6-1.6875D0*T5)*T4+(0.047916666666666666667D0*T24+(-0.208333333
     &33333333333D0*T6+0.083333333333333333333D0)*im*T18+(-0.75000000000
     &000000001D0*T19+0.87500000000000000001D0*T20+0.2916666666666666666
     &7D0*T7+0.083333333333333333333D0*T6+0.16666666666666666667D0*T5-0.
     &45833333333333333334D0)*T10+(T27-0.75000000000000000001D0*T48+T49+
     &T131+1.75D0*T22+T127+2.75D0*T19+0.25D0*T20+T128+T129-0.125D0*T5-2.
     &1875D0)*im*pi-3.5D0*T32-2.5D0*T33-2.5D0*T34-3.25D0*T35+T37+T38+0.2
     &5D0*T39-1.75D0*T40-1.75D0*T44-3.75D0*T27+T130-1.0D0*T49+T131-T132-
     &2.125D0*T19+T133+T135+2.1875D0*T6)*x
      gammaHPL_3l_nfv=((T139+(0.33333333333333333333D0*T19-0.16666666666666666667D0*
     &T20)*T10+(T134+T133+T135-0.5625D0*T6)*im*pi+T32+T33+T34+T35-0.375D
     &0*T27-0.125D0*T48-0.125D0*T49-0.125D0*T22+0.5625D0*T19)*dabc2
     & )/(Nc*T3-2.0D0*Nc*T4+Nc*x)

      end function gammaHPL_3l_nfv

      function gammaHPL_3l_no_nfv(x)
        use types
        use constants
        implicit complex(dp) (T)
        include 'nf.f'
        complex(dp) :: gammaHPL_3l_no_nfv
        real(dp), intent(in) :: x

      T2=CF*CF
      T3=CF**3
      T4=Hr1(1)
      T5=Hr1(0)
      T6=CA*CA
      T7=9.5D0*T4
      T8=16.0D0*T4
      T9=-18.5D0*T4
      T10=x**4
      T11=x**3
      T12=x*x
      T13=62.0D0*T5
      T14=25.0D0*T5
      T15=Hr2(1,1)
      T16=-4.0D0*Hr2(1,0)
      T17=Hr2(0,1)
      T18=Hr2(0,0)
      T19=Hr2(1,0)
      T20=Nf*Nf
      T21=-3.0D0*T15
      T22=Hr3(1,1,1)
      T23=Hr3(1,1,0)
      T24=Hr3(1,0,1)
      T25=Hr3(1,0,0)
      T26=Hr3(0,1,1)
      T27=Hr3(0,1,0)
      T28=Hr3(0,0,1)
      T29=Hr3(0,0,0)
      T30=-5.0D0*T24
      T31=3.0D0*T29
      T32=2.0D0*T3
      T33=pi**3
      T34=pi*pi
      T35=3.0D0*T15
      T36=8.0D0*T19
      T37=6.0D0*T15
      T38=6.0D0*T18
      T39=-0.66666666666666666667D0*T19
      T40=18.0D0*T27
      T41=2.0D0*T23
      T42=10.0D0*T24
      T43=12.0D0*T27
      T44=-3.5833333333333333333D0*T4
      T45=8.5D0*T4
      T46=-1.5D0*T4
      T47=-0.074074074074074074074D0*CF*T20
      T48=0.61111111111111111111D0*T4
      T49=-0.66666666666666666667D0*T4
      T50=10.0D0*T15
      T51=3.0D0*T5
      T52=-1.5D0*T15
      T53=3.0D0*T19
      T54=2.0D0*T18
      T55=-0.21399176954732510288D0*CF*T20
      T56=4.0555555555555555556D0*T15
      T57=1.3333333333333333333D0*T17
      T58=0.33333333333333333333D0*T19
      T59=-17.0D0*T22
      T60=8.0D0*T23
      T61=-18.0D0*T25
      T62=-10.0D0*T28
      T63=6.0D0*T29
      T64=4.5D0*T18
      T65=21.0D0*T22
      T66=14.0D0*T24
      T67=21.0D0*T25
      T68=6.0D0*T27
      T69=-4.0D0*T22
      T70=-1.0D0*T23
      T71=-6.0D0*T25
      T72=2.0D0*T27
      T73=-0.5D0*T5
      T74=-2.5D0*T28
      T75=Hr4(1,1,1,1)
      T76=Hr4(1,1,0,0)
      T77=Hr4(1,0,1,1)
      T78=Hr4(1,0,0,0)
      T79=Hr4(0,1,1,1)
      T80=Hr4(0,1,0,0)
      T81=Hr4(0,0,1,1)
      T82=Hr4(0,0,0,0)
      T83=4.0D0*T23
      T84=Hr4(1,1,1,0)
      T85=Hr4(1,1,0,1)
      T86=Hr4(1,0,1,0)
      T87=Hr4(1,0,0,1)
      T88=Hr4(0,1,1,0)
      T89=Hr4(0,1,0,1)
      T90=Hr4(0,0,1,0)
      T91=Hr4(0,0,0,1)
      T92=x**6
      T93=pi**4
      T94=1.3333333333333333333D0*T4
      T95=1.3333333333333333333D0*T19
      T96=8.0D0*T22
      T97=2.5D0*T17
      T98=8.0D0*T24
      T99=4.0D0*T24
      T100=-2.5D0*T26
      T101=1.9166666666666666667D0*T18
      T102=4.0D0*T29
      T103=x**5
      T104=1.6666666666666666667D0*T28
      T105=2.5D0*T82
      T106=2.0D0*T84
      T107=2.3333333333333333333D0*T80
      T108=0.22222222222222222222D0*T22
      T109=1.3333333333333333333D0*T85
      T110=-4.5D0*T84
      T111=Hr5(1,1,1,1,0)
      T112=Hr5(1,1,1,0,1)
      T113=Hr5(1,1,0,1,1)
      T114=Hr5(1,1,0,1,0)
      T115=8.0D0*Hr5(1,1,0,0,1)
      T116=Hr5(1,1,0,0,0)
      T117=Hr5(1,0,1,1,0)
      T118=Hr5(1,0,1,0,1)
      T119=Hr5(1,0,1,0,0)
      T120=Hr5(1,0,0,1,1)
      T121=Hr5(1,0,0,1,0)
      T122=Hr5(1,0,0,0,1)
      T123=Hr5(1,0,0,0,0)
      T124=Hr5(0,1,1,1,0)
      T125=Hr5(0,1,1,0,1)
      T126=Hr5(0,1,1,0,0)
      T127=Hr5(0,1,0,1,1)
      T128=Hr5(0,1,0,1,0)
      T129=Hr5(0,1,0,0,1)
      T130=Hr5(0,0,1,1,1)
      T131=Hr5(0,0,1,1,0)
      T132=Hr5(0,0,1,0,1)
      T133=Hr5(0,0,1,0,0)
      T134=Hr5(0,0,0,1,1)
      T135=Hr5(0,0,0,1,0)
      T136=Hr5(0,0,0,0,1)
      T137=Hr5(0,0,0,0,0)
      T138=-3.0D0*T112
      T139=Hr5(1,1,1,0,0)
      T140=2.0D0*T113
      T141=Hr5(1,1,0,0,1)
      T142=Hr5(1,0,1,1,1)
      T143=-2.0D0*T125
      T144=Hr5(0,1,0,0,0)
      T145=1.75D0*T76
      T146=Hr5(1,1,1,1,1)
      T147=1.3333333333333333333D0*T113
      T148=-2.0D0*T86
      T149=-2.0D0*T87
      T150=-0.5D0*T88
      T151=1.2530864197530864198D0*T15
      T152=-7.1666666666666666667D0*T90
      T153=Hr6(1,1,1,1,0,0)
      T154=3.0D0*Hr6(1,1,1,0,1,1)
      T155=Hr6(1,1,1,0,0,0)
      T156=Hr6(1,1,0,1,1,0)
      T157=Hr6(1,1,0,1,0,1)
      T158=Hr6(1,1,0,1,0,0)
      T159=Hr6(1,1,0,0,1,1)
      T160=Hr6(1,1,0,0,1,0)
      T161=Hr6(1,1,0,0,0,1)
      T162=Hr6(1,1,0,0,0,0)
      T163=Hr6(1,0,1,1,1,1)
      T164=Hr6(1,0,1,1,1,0)
      T165=Hr6(1,0,1,1,0,1)
      T166=Hr6(1,0,1,0,1,1)
      T167=Hr6(1,0,1,0,1,0)
      T168=Hr6(1,0,1,0,0,1)
      T169=Hr6(1,0,1,0,0,0)
      T170=Hr6(1,0,0,1,1,1)
      T171=8.0D0*Hr6(1,0,0,1,1,0)
      T172=8.0D0*Hr6(1,0,0,1,0,1)
      T173=Hr6(1,0,0,1,0,0)
      T174=Hr6(1,0,0,0,1,1)
      T175=Hr6(1,0,0,0,1,0)
      T176=Hr6(1,0,0,0,0,1)
      T177=Hr6(0,1,1,1,0,0)
      T178=Hr6(0,1,1,0,1,1)
      T179=Hr6(0,1,1,0,1,0)
      T180=Hr6(0,1,1,0,0,1)
      T181=Hr6(0,1,1,0,0,0)
      T182=Hr6(0,1,0,1,1,1)
      T183=Hr6(0,1,0,1,1,0)
      T184=Hr6(0,1,0,1,0,1)
      T185=Hr6(0,1,0,1,0,0)
      T186=Hr6(0,1,0,0,1,1)
      T187=Hr6(0,1,0,0,1,0)
      T188=Hr6(0,1,0,0,0,1)
      T189=Hr6(0,1,0,0,0,0)
      T190=Hr6(0,0,1,1,1,1)
      T191=Hr6(0,0,1,1,1,0)
      T192=Hr6(0,0,1,1,0,1)
      T193=Hr6(0,0,1,1,0,0)
      T194=Hr6(0,0,1,0,1,1)
      T195=Hr6(0,0,1,0,1,0)
      T196=Hr6(0,0,1,0,0,1)
      T197=Hr6(0,0,1,0,0,0)
      T198=Hr6(0,0,0,1,1,1)
      T199=Hr6(0,0,0,1,1,0)
      T200=Hr6(0,0,0,1,0,1)
      T201=Hr6(0,0,0,1,0,0)
      T202=Hr6(0,0,0,0,1,1)
      T203=Hr6(0,0,0,0,1,0)
      T204=Hr6(0,0,0,0,0,1)
      T205=Hr6(0,0,0,0,0,0)
      T206=-6.0D0*T111
      T207=31.0D0*T114
      T208=31.0D0*T141
      T209=Hr5(0,1,1,1,1)
      T210=Hr6(1,1,1,0,1,0)
      T211=Hr6(1,1,1,0,0,1)
      T212=-4.0D0*T169
      T213=Hr6(1,0,0,1,1,0)
      T214=Hr6(1,0,0,1,0,1)
      T215=-1.0D0*T203
      T216=-1.0D0*T204
      T217=2.5D0*T128
      T218=pi**6
      T219=pi**5
      T220=0.27777777777777777778D0*T15
      T221=0.33333333333333333333D0*T17
      T222=-0.22222222222222222222D0*T19
      T223=-2.1666666666666666667D0*T22
      T224=5.4722222222222222222D0*T15
      T225=3.3333333333333333333D0*T28
      T226=0.55555555555555555555D0*T23
      T227=-0.22222222222222222222D0*T25
      T228=-1.3333333333333333333D0*T86
      T229=-0.66666666666666666667D0*T79
      T230=-2.0D0*T89
      T231=2.0D0*T75
      T232=-3.5D0*T76
      T233=-1.0D0*T78
      T234=0.66666666666666666667D0*T79
      T235=1.6666666666666666667D0*T87
      T236=-0.33333333333333333333D0*T89
      T237=0.55555555555555555555D0*T26
      T238=0.44444444444444444444D0*T22
      T239=4.0D0*T84
      T240=2.6666666666666666667D0*T85
      T241=-1.3333333333333333333D0*T87
      T242=14.666666666666666667D0*T22
      T243=3.3333333333333333333D0*T27
      T244=1.3333333333333333333D0*T27
      T245=4.0D0*T28
      T246=6.0D0*T112
      T247=16.0D0*T141
      T248=4.0D0*T117
      T249=4.0D0*T126
      T250=2.0D0*T111
      T251=-2.0D0*T139
      T252=-4.0D0*T113
      T253=6.0D0*T118
      T254=2.0D0*T119
      T255=10.0D0*T122
      T256=8.0D0*T123
      T257=10.0D0*T135
      T258=-8.0D0*T75
      T259=4.5D0*T84
      T260=9.0D0*T86
      T261=2.0D0*T82
      T262=2.5D0*T29
      T263=1.3333333333333333333D0*T29
      T264=4.0D0*T139
      T265=1.3333333333333333333D0*T120
      T266=0.66666666666666666667D0*T88
      T267=2.0D0*T24
      T268=2.6666666666666666667D0*T113
      T269=1.3333333333333333333D0*T116
      T270=-2.0D0*T142
      T271=1.3333333333333333333D0*T117
      T272=1.3333333333333333333D0*T118
      T273=1.3333333333333333333D0*T119
      T274=-8.0D0*T79
      T275=0.33333333333333333333D0*T89
      T276=Hr6(1,1,1,1,1,1)
      T277=Hr6(1,1,1,0,1,1)
      T278=-2.0D0*T186
      T279=-16.0D0*T111
      T280=2.0D0*T142
      T281=-0.66666666666666666667D0*T117
      T282=-0.66666666666666666667D0*T118
      T283=68.0D0*T24
      T284=6.0D0*T277
      T285=2.0D0*T156
      T286=2.0D0*T157
      T287=-4.0D0*T164
      T288=-4.0D0*T165
      T289=16.0D0*T213
      T290=16.0D0*T214
      T291=48.0D0*T179
      T292=48.0D0*T180
      T293=12.0D0*T197
      T294=6.0D0*T210
      T295=6.0D0*T211
      T296=4.0D0*T155
      T297=Hr6(1,1,0,1,1,1)
      T298=2.0D0*T163
      T299=8.0D0*T169
      T300=10.0D0*T170
      T301=18.0D0*T174
      T302=8.0D0*T175
      T303=8.0D0*T176
      T304=-2.0D0*T185
      T305=-4.0D0*T203
      T306=-4.0D0*T204
      T307=5.0D0*T205
      T308=8.0D0*T111
      T309=2.0D0*T117
      T310=2.0D0*T118
      T311=7.0D0*T119
      T312=3.5D0*T88
      T313=0.26875D0*T4
      T314=-0.28055555555555555556D0*T4
      T315=0.047222222222222222222D0*T4
      T316=0.0044238683127572016461D0*CF*T20
      T317=-0.040740740740740740741D0*T4
      T318=0.014814814814814814815D0*T5
      T319=-0.023148148148148148148D0*T4
      T320=0.0018518518518518518519D0*T5
      T321=0.53125D0*T15
      T322=0.72777777777777777778D0*T19
      T323=-0.35D0*T15
      T324=-0.88611111111111111111D0*T19
      T325=0.016666666666666666667D0*T15
      T326=0.26111111111111111111D0*T19
      T327=-0.13888888888888888889D0*T15
      T328=0.41666666666666666667D0*T18
      T329=0.11111111111111111111D0*T19
      T330=-0.11111111111111111111D0*T17
      T331=0.11111111111111111111D0*T18
      T332=-0.14814814814814814815D0*T5
      T333=1.0833333333333333333D0*T22
      T334=1.5D0*T23
      T335=2.5833333333333333333D0*T24
      T336=-4.3333333333333333333D0*T25
      T337=3.6666666666666666667D0*T26
      T338=-1.5D0*T22
      T339=-1.0833333333333333333D0*T23
      T340=-3.25D0*T24
      T341=5.5D0*T25
      T342=0.66666666666666666667D0*T22
      T343=0.16666666666666666667D0*T23
      T344=1.1666666666666666667D0*T24
      T345=-1.6666666666666666667D0*T25
      T346=2.0D0*T26
      T347=-0.66666666666666666667D0*T27
      T348=0.074074074074074074074D0*T15
      T349=0.23456790123456790123D0*T4
      T350=0.27777777777777777778D0*T22
      T351=0.33333333333333333333D0*T24
      T352=0.22222222222222222222D0*T26
      T353=0.33333333333333333333D0*T27
      T354=1.4074074074074074074D0*T15
      T355=-0.27777777777777777778D0*T23
      T356=0.11111111111111111111D0*T24
      T357=0.11111111111111111111D0*T25
      T358=-0.11111111111111111111D0*T26
      T359=0.27777777777777777778D0*T28
      T360=-4.25D0*T75
      T361=2.6666666666666666667D0*T84
      T362=-3.3333333333333333333D0*T85
      T363=-2.1666666666666666667D0*T76
      T364=-2.75D0*T77
      T365=0.66666666666666666667D0*T86
      T366=1.6666666666666666667D0*T88
      T367=0.66666666666666666667D0*T89
      T368=-1.0D0*T75
      T369=5.3333333333333333333D0*T85
      T370=3.0833333333333333333D0*T77
      T371=2.3333333333333333333D0*T86
      T372=1.8333333333333333333D0*T87
      T373=0.5D0*T78
      T374=2.0D0*T91
      T375=-1.0D0*T82
      T376=-1.6666666666666666667D0*T84
      T377=-0.33333333333333333333D0*T76
      T378=-0.5D0*T77
      T379=-0.83333333333333333333D0*T87
      T380=-0.33333333333333333333D0*T81
      T381=-1.6666666666666666667D0*T91
      T382=-1.2222222222222222222D0*T22
      T383=0.22222222222222222222D0*T29
      T384=-0.7037037037037037037D0*T15
      T385=-0.37037037037037037037D0*T18
      T386=-1.2530864197530864198D0*T4
      T387=4.6666666666666666667D0*T75
      T388=1.3333333333333333333D0*T77
      T389=0.66666666666666666667D0*T87
      T390=1.3333333333333333333D0*T80
      T391=1.3333333333333333333D0*T90
      T392=7.3333333333333333333D0*T22
      T393=2.0D0*T25
      T394=0.33333333333333333333D0*T29
      T395=-1.0D0*T77
      T396=-15.0D0*T146
      T397=10.0D0*T114
      T398=30.0D0*T116
      T399=3.0D0*T142
      T400=-2.0D0*T120
      T401=-14.0D0*T121
      T402=2.0D0*T126
      T403=4.0D0*T129
      T404=2.0D0*T133
      T405=4.0D0*T134
      T406=-4.5D0*T88
      T407=9.0D0*T90
      T408=18.0D0*T82
      T409=9.0D0*T112
      T410=-1.0D0*T113
      T411=-13.0D0*T114
      T412=-27.0D0*T116
      T413=3.0D0*T119
      T414=9.0D0*T120
      T415=13.0D0*T121
      T416=13.0D0*T122
      T417=12.0D0*T123
      T418=-3.0D0*T127
      T419=8.0D0*T131
      T420=13.0D0*T132
      T421=4.0D0*T114
      T422=3.0D0*T141
      T423=6.0D0*T116
      T424=-3.0D0*T118
      T425=-1.0D0*T119
      T426=-3.0D0*T120
      T427=-3.0D0*T121
      T428=-5.0D0*T122
      T429=-4.0D0*T123
      T430=-4.0D0*T132
      T431=-1.0D0*T134
      T432=3.0D0*T135
      T433=5.5D0*T77
      T434=4.5D0*T86
      T435=0.66666666666666666667D0*T75
      T436=1.4074074074074074074D0*T22
      T437=-6.6666666666666666667D0*T146
      T438=-0.66666666666666666667D0*T142
      T439=-0.66666666666666666667D0*T120
      T440=-1.3333333333333333333D0*T128
      T441=-1.3333333333333333333D0*T129
      T442=0.66666666666666666667D0*T144
      T443=1.3333333333333333333D0*T130
      T444=1.3333333333333333333D0*T133
      T445=-6.3333333333333333333D0*T75
      T446=4.0D0*T86
      T447=4.0D0*T87
      T448=2.0D0*T88
      T449=2.0D0*T89
      T450=2.1666666666666666667D0*T22
      T451=2.1666666666666666667D0*T23
      T452=-5.3333333333333333333D0*T29
      T453=2.0D0*T17
      T454=-0.66666666666666666667D0*T116
      T455=-0.66666666666666666667D0*T119
      T456=0.66666666666666666667D0*T127
      T457=0.66666666666666666667D0*T128
      T458=0.66666666666666666667D0*T129
      T459=-1.0D0*T144
      T460=0.66666666666666666667D0*T130
      T461=2.0D0*T86
      T462=2.0D0*T87
      T463=-1.0D0*T88
      T464=-1.0D0*T89
      T465=0.11111111111111111111D0*T80
      T466=15.0D0*T276
      T467=8.0D0*T153
      T468=12.0D0*T158
      T469=12.0D0*T159
      T470=20.0D0*T160
      T471=20.0D0*T161
      T472=50.0D0*T162
      T473=-7.0D0*T163
      T474=-3.0D0*T166
      T475=-2.0D0*T167
      T476=-2.0D0*T168
      T477=-6.0D0*T173
      T478=12.0D0*T111
      T479=-6.0D0*T128
      T480=13.5D0*T132
      T481=1.25D0*T80
      T482=-6.0D0*T153
      T483=6.0D0*T155
      T484=-1.0D0*T156
      T485=-1.0D0*T157
      T486=-14.0D0*T158
      T487=-14.0D0*T159
      T488=-22.0D0*T160
      T489=-22.0D0*T161
      T490=-49.0D0*T162
      T491=2.0D0*T164
      T492=2.0D0*T165
      T493=7.0D0*T167
      T494=7.0D0*T168
      T495=10.0D0*T169
      T496=-17.0D0*T170
      T497=5.0D0*T173
      T498=-25.0D0*T174
      T499=-12.0D0*T175
      T500=-12.0D0*T176
      T501=Hr6(0,1,1,1,1,0)
      T502=Hr6(0,1,1,1,0,1)
      T503=8.0D0*T179
      T504=8.0D0*T180
      T505=10.0D0*T182
      T506=14.0D0*T194
      T507=3.0D0*T201
      T508=36.666666666666666667D0*T146
      T509=-18.0D0*T112
      T510=15.0D0*T114
      T511=15.0D0*T141
      T512=0.66666666666666666667D0*T120
      T513=-3.0D0*T210
      T514=-3.0D0*T211
      T515=-2.0D0*T155
      T516=-2.0D0*T297
      T517=4.0D0*T158
      T518=3.0D0*T159
      T519=6.0D0*T160
      T520=6.0D0*T161
      T521=12.0D0*T162
      T522=-1.0D0*T163
      T523=-3.0D0*T167
      T524=-3.0D0*T168
      T525=5.0D0*T170
      T526=2.0D0*T213
      T527=2.0D0*T214
      T528=-1.0D0*T173
      T529=9.0D0*T174
      T530=4.0D0*T175
      T531=4.0D0*T176
      T532=-1.0D0*T178
      T533=-4.0D0*T182
      T534=-3.0D0*T183
      T535=-3.0D0*T184
      T536=12.0D0*T190
      T537=6.0D0*T191
      T538=6.0D0*T192
      T539=3.0D0*T199
      T540=3.0D0*T200
      T541=6.0D0*T111
      T542=10.333333333333333333D0*T126
      T543=-1.0D0*T17
      T544=1.3333333333333333333D0*T15
      T545=0.66666666666666666667D0*T17
      T546=2.0D0*T28
      T547=3.0D0*T22
      T548=12.5D0*T23
      T549=-2.5D0*T24
      T550=0.66666666666666666667D0*T26
      T551=2.5D0*T22
      T552=7.3333333333333333333D0*T26
      T553=4.0D0*T26
      T554=6.0D0*T84
      T555=2.0D0*T77
      T556=(((36.5D0*T3+2.0D0*CA*T2-23.25D0*T6*CF)*im*pi+(-6.61111111111
     &11111111D0*T2+1.5D0*CA*CF)*Nf+(T7+T14-34.083333333333333333D0)*T3+
     &(T8-15.5D0*T5+60.888888888888888889D0)*CA*T2+(T9+1.5D0*T5-30.90277
     &7777777777778D0)*T6*CF)*T10+((-73.0D0*T3+56.0D0*CA*T2+16.5D0*T6*CF
     &)*im*pi+(-6.7777777777777777778D0*T2+7.0D0*CA*CF)*Nf+(-19.0D0*T4+1
     &00.0D0*T5+33.166666666666666667D0)*T3+(-32.0D0*T4-T13+36.722222222
     &222222222D0)*CA*T2+(37.0D0*T4+6.0D0*T5-12.694444444444444444D0)*T6
     &*CF)*T11+((34.0D0*T3-28.0D0*CA*T2-7.0D0*T6*CF)*im*pi+(-10.77777777
     &7777777778D0*T2+3.1666666666666666667D0*CA*CF)*Nf+(T7-11.0D0*T5+59
     &.0D0)*T3+(T8+7.5D0*T5+62.722222222222222222D0)*CA*T2+(T9+3.5D0*T5-
     &48.361111111111111111D0)*T6*CF)*T12)*zeta5+((21.0D0*T3-3.916666666
     &6666666667D0*CA*T2-11.277777777777777778D0*T6*CF)*T10+(36.0D0*T3-4
     &6.166666666666666667D0*CA*T2+30.055555555555555556D0*T6*CF)*T11+(T
     &32+11.333333333333333333D0*CA*T2-15.777777777777777778D0*T6*CF)*T1
     &2)*zeta3*zeta3
      T557=(-7.0D0*T3+3.5416666666666666667D0*CA*T2+0.055555555555555555
     &555D0*T6*CF)*im*T33+((-2.9490740740740740741D0*T2+1.52777777777777
     &77778D0*CA*CF)*Nf+(T44+T51-17.875D0)*T3+(T45-4.8333333333333333333
     &D0*T5+50.261574074074074074D0)*CA*T2+(T46+1.6666666666666666667D0*
     &T5-21.569444444444444444D0)*T6*CF)*T34+(T47+((T48+11.0555555555555
     &55556D0)*T2+(T49-26.388888888888888889D0)*CA*CF)*Nf+(T50+T16+13.0D
     &0*T17-9.0D0*T18+7.5D0*T4+T14-246.75D0)*T3+(T52+T16-5.5D0*T17+T64-3
     &2.111111111111111111D0*T4-11.25D0*T5+196.79166666666666667D0)*CA*T
     &2+(T21+T53-T97+18.666666666666666667D0*T4-0.25D0*T5+48.93518518518
     &5185185D0)*T6*CF)*im*pi+T55+((T56+T39-2.3333333333333333333D0*T17-
     &4.0D0*T18+15.055555555555555556D0*T4+14.916666666666666667D0*T5-1.
     &3487654320987654321D0)*T2+(T21+T58+T221+T54-19.277777777777777778D
     &0*T4-16.444444444444444444D0*T5-16.601851851851851852D0)*CA*CF)*Nf
     &+(T59+T60+T30+T61+T346+T43-5.0D0*T28-T63-15.5D0*T15-21.33333333333
     &3333333D0*T19+10.5D0*T17-49.5D0*T18-179.25D0*T4-7.9166666666666666
     &667D0*T5-446.5D0)*T3
      T558=(T557+(T65-T41+T66+T67-8.5D0*T26-T40+T74-T31+24.4444444444444
     &44444D0*T15+44.0D0*T19+38.333333333333333333D0*T17+121.25D0*T18+12
     &5.65277777777777778D0*T4-46.041666666666666667D0*T5+521.0462962962
     &962963D0)*CA*T2+(T69+T70+T30+T71+3.0D0*T26+T68+T28+T31-T37-16.9166
     &66666666666667D0*T19-16.083333333333333333D0*T17-48.75D0*T18+31.23
     &6111111111111111D0*T4+55.986111111111111111D0*T5-50.63245884773662
     &5514D0)*T6*CF)*T10
      T559=(-T32-6.0833333333333333333D0*CA*T2+3.3888888888888888889D0*T
     &6*CF)*im*T33+((-0.10185185185185185185D0*T2-0.05555555555555555555
     &5D0*CA*CF)*Nf+(7.1666666666666666667D0*T4+12.0D0*T5+30.25D0)*T3+(-
     &17.0D0*T4-19.333333333333333333D0*T5-38.023148148148148148D0)*CA*T
     &2+(3.0D0*T4+6.6666666666666666667D0*T5+14.638888888888888889D0)*T6
     &*CF)*T34+(0.14814814814814814815D0*CF*T20+((-1.2222222222222222222
     &D0*T4-28.833333333333333333D0)*T2+(T94+27.444444444444444444D0)*CA
     &*CF)*Nf+(-20.0D0*T15+T36+22.0D0*T17-12.0D0*T18-42.0D0*T4-32.0D0*T5
     &+97.5D0)*T3+(T35+T36-7.0D0*T17-T38+87.722222222222222222D0*T4+45.0
     &D0*T5+158.13888888888888889D0)*CA*T2+(T37-6.0D0*T19+T453+T38-43.83
     &3333333333333333D0*T4-16.0D0*T5-169.53703703703703704D0)*T6*CF)*im
     &*pi+0.42798353909465020576D0*CF*T20+((-8.1111111111111111111D0*T15
     &+T95+T545-16.0D0*T18-25.777777777777777778D0*T4+3.4444444444444444
     &444D0*T5-7.4135802469135802469D0)*T2+(T37+T39+T57+8.0D0*T18+20.555
     &555555555555556D0*T4+2.2222222222222222222D0*T5+29.314814814814814
     &815D0)*CA*CF)*Nf
      T560=(T559+(34.0D0*T22-16.0D0*T23+T42+36.0D0*T25+20.0D0*T26+T40-44
     &.0D0*T28-13.0D0*T15+20.666666666666666667D0*T19-11.0D0*T17-14.0D0*
     &T18+62.5D0*T4+94.0D0*T5+350.58333333333333333D0)*T3+(-42.0D0*T22+T
     &83-28.0D0*T24-42.0D0*T25-31.0D0*T26-33.0D0*T27+T62-36.0D0*T29+15.6
     &11111111111111111D0*T15-37.0D0*T19-3.6666666666666666667D0*T17+98.
     &0D0*T18+94.611111111111111111D0*T4-133.11111111111111111D0*T5-290.
     &43981481481481481D0)*CA*T2+(T96+T41+T42+12.0D0*T25+12.0D0*T26+T43+
     &19.0D0*T28+18.0D0*T29-9.0D0*T15+14.333333333333333333D0*T19-9.3333
     &333333333333333D0*T17-43.5D0*T18-102.97222222222222222D0*T4+35.361
     &111111111111111D0*T5-108.55452674897119342D0)*T6*CF)*T11
      T561=(-10.833333333333333333D0*T3+13.208333333333333333D0*CA*T2-3.
     &9444444444444444444D0*T6*CF)*im*T33+((-1.3935185185185185185D0*T2+
     &0.63888888888888888889D0*CA*CF)*Nf+(T44-1.1666666666666666667D0*T5
     &+2.9583333333333333333D0)*T3+(T45-1.8333333333333333333D0*T5+8.122
     &6851851851851852D0)*CA*T2+(T46+T73-6.8472222222222222222D0)*T6*CF)
     &*T34+(T47+((T48+2.0555555555555555555D0*T5-3.8888888888888888889D0
     &)*T2+(T49-0.66666666666666666667D0*T5-2.7222222222222222222D0)*CA*
     &CF)*Nf+(T50+T16+12.0D0*T17+10.0D0*T18+43.5D0*T4-T51+15.75D0)*T3+(T
     &52+T16-19.0D0*T17-8.5D0*T18-61.611111111111111111D0*T4-1.805555555
     &5555555556D0*T5-13.763888888888888889D0)*CA*T2+(-T35+T53+6.0D0*T17
     &+T54+25.916666666666666667D0*T4-2.0833333333333333333D0*T5+30.2685
     &18518518518518D0)*T6*CF)*im*pi+T55+((T56+T39-T57+0.722222222222222
     &22222D0*T18+11.055555555555555556D0*T4-10.138888888888888889D0*T5+
     &13.706790123456790123D0)*T2+(-T35+T58+1.6666666666666666667D0*T17-
     &0.66666666666666666667D0*T18-6.9444444444444444444D0*T4+5.22222222
     &22222222222D0*T5-16.018518518518518518D0)*CA*CF)*Nf
      T562=(T561+(T59+T60+T30+T61+22.0D0*T26+T72+T62+T63+28.5D0*T15+T53-
     &19.5D0*T17-T64-34.75D0*T4-15.75D0*T5-64.5D0)*T3+(T65-T41+T66+T67-2
     &1.0D0*T26-T68+7.0D0*T28-T63-39.055555555555555556D0*T15-8.33333333
     &33333333333D0*T19-0.91666666666666666667D0*T17+10.0277777777777777
     &78D0*T18-57.097222222222222222D0*T4+84.930555555555555556D0*T5-45.
     &509259259259259259D0)*CA*T2+(T69+T70+T30+T71+T553+T72+14.5D0*T15+2
     &.6666666666666666667D0*T19+3.5833333333333333333D0*T17-7.333333333
     &3333333333D0*T18+48.027777777777777778D0*T4-44.388888888888888889D
     &0*T5+108.32587448559670782D0)*T6*CF)*T12
      T563=(-0.27083333333333333333D0*CA*CF*Nf-5.8208333333333333333D0*T
     &3+7.1416666666666666667D0*CA*T2-1.221875D0*T6*CF)*T93+((0.41666666
     &666666666667D0*T4+0.41666666666666666667D0*T5)*CA*CF*Nf+(5.75D0*T4
     &+11.75D0*T5-11.25D0)*T3+(-9.3333333333333333333D0*T4-12.3333333333
     &33333333D0*T5+14.083333333333333333D0)*CA*T2+(1.854166666666666666
     &7D0*T4+1.8541666666666666667D0*T5-4.2291666666666666667D0)*T6*CF)*
     &im*T33+(((-1.0D0*T4+T5-1.9166666666666666667D0)*T2+(-0.41666666666
     &666666667D0*T15-0.41666666666666666667D0*T19-0.4166666666666666666
     &7D0*T17-T328+0.5D0*T4+T73+0.95833333333333333333D0)*CA*CF)*Nf+(-6.
     &8333333333333333333D0*T15-6.8333333333333333333D0*T19-6.8333333333
     &333333333D0*T17-6.8333333333333333333D0*T18+2.0D0*T4-2.0D0*T5+2.04
     &16666666666666667D0)*T3+(10.208333333333333333D0*T15+10.2083333333
     &33333333D0*T19+10.208333333333333333D0*T17+10.208333333333333333D0
     &*T18+4.5D0*T4-4.5D0*T5+10.520833333333333333D0)*CA*T2+(-2.02083333
     &33333333333D0*T15-2.0208333333333333333D0*T19-2.020833333333333333
     &3D0*T17-2.0208333333333333333D0*T18-2.75D0*T4+2.75D0*T5-5.77083333
     &33333333333D0)*T6*CF)*T34
      T564=T563+((-T551+2.5D0*T23+T549+2.5D0*T25+T100+2.5D0*T27+T74+T262
     &)*CA*CF*Nf+(-68.5D0*T22+46.5D0*T23-58.5D0*T24+56.5D0*T25-56.5D0*T2
     &6+58.5D0*T27-46.5D0*T28+68.5D0*T29-22.5D0*T15-22.5D0*T19-22.5D0*T1
     &7-22.5D0*T18)*T3+(77.0D0*T22-62.0D0*T23+T283-71.0D0*T25+71.0D0*T26
     &-68.0D0*T27+62.0D0*T28-77.0D0*T29+28.166666666666666667D0*T15+28.1
     &66666666666666667D0*T19+28.166666666666666667D0*T17+28.16666666666
     &6666667D0*T18)*CA*T2+(-13.125D0*T22+11.125D0*T23-11.125D0*T24+13.1
     &25D0*T25-13.125D0*T26+11.125D0*T27-11.125D0*T28+13.125D0*T29-8.458
     &3333333333333333D0*T15-8.4583333333333333333D0*T19-8.4583333333333
     &333333D0*T17-8.4583333333333333333D0*T18)*T6*CF)*im*pi+((T69-T41-T
     &267+T72+T546+T102-3.8333333333333333333D0*T15-3.833333333333333333
     &3D0*T19-3.8333333333333333333D0*T17-3.8333333333333333333D0*T18)*T
     &2+(2.5D0*T75+2.5D0*T76+2.5D0*T77+2.5D0*T78+2.5D0*T79+2.5D0*T80+2.5
     &D0*T81+T105+2.0D0*T22+T23+T24-1.0D0*T27-1.0D0*T28-2.0D0*T29+1.9166
     &666666666666667D0*T15+1.9166666666666666667D0*T19+1.91666666666666
     &66667D0*T17+T101)*CA*CF)*Nf
      T565=T564+(77.5D0*T75+9.0D0*T84+9.0D0*T85+55.5D0*T76+67.5D0*T77+T2
     &60+9.0D0*T87+65.5D0*T78+65.5D0*T79+9.0D0*T88+9.0D0*T89+67.5D0*T80+
     &55.5D0*T81+T407+9.0D0*T91+77.5D0*T82+22.0D0*T22+T83+T99-14.0D0*T25
     &+14.0D0*T26-4.0D0*T27-T245-22.0D0*T29+4.0833333333333333333D0*T15+
     &4.0833333333333333333D0*T19+4.0833333333333333333D0*T17+4.08333333
     &33333333333D0*T18)*T3+(-81.5D0*T75+T110-4.5D0*T85-66.5D0*T76-72.5D
     &0*T77-T434-4.5D0*T87-75.5D0*T78-75.5D0*T79+T406-4.5D0*T89-72.5D0*T
     &80-66.5D0*T81-4.5D0*T90-4.5D0*T91-81.5D0*T82-5.9166666666666666667
     &D0*T22+9.0D0*T23+9.0D0*T24+23.916666666666666667D0*T25-23.91666666
     &6666666667D0*T26-9.0D0*T27-9.0D0*T28+5.9166666666666666667D0*T29+2
     &1.041666666666666667D0*T15+21.041666666666666667D0*T19+21.04166666
     &6666666667D0*T17+21.041666666666666667D0*T18)*CA*T2+(13.125D0*T75+
     &11.125D0*T76+11.125D0*T77+13.125D0*T78+13.125D0*T79+11.125D0*T80+1
     &1.125D0*T81+13.125D0*T82-2.5416666666666666667D0*T22-5.5D0*T23-5.5
     &D0*T24-8.4583333333333333333D0*T25+8.4583333333333333333D0*T26+5.5
     &D0*T27+5.5D0*T28+2.5416666666666666667D0*T29-11.541666666666666667
     &D0*T15-11.541666666666666667D0*T19-11.541666666666666667D0*T17-11.
     &541666666666666667D0*T18)*T6*CF
      T566=(0.86666666666666666667D0*CA*CF*Nf+16.655555555555555556D0*T3
     &-20.633333333333333333D0*CA*T2+3.1736111111111111111D0*T6*CF)*T93+
     &((-T94-1.3333333333333333333D0*T5-0.41666666666666666667D0)*CA*CF*
     &Nf+(-14.0D0*T4-36.0D0*T5+38.666666666666666667D0)*T3+(26.166666666
     &666666667D0*T4+37.166666666666666667D0*T5-43.041666666666666667D0)
     &*CA*T2+(-5.0D0*T4-5.0D0*T5+13.229166666666666667D0)*T6*CF)*im*T33+
     &(((3.7777777777777777778D0*T4-3.5555555555555555555D0*T5+7.8888888
     &888888888889D0)*T2+(T544+T95+T57+1.3333333333333333333D0*T18-1.472
     &2222222222222222D0*T4+2.1944444444444444444D0*T5-3.944444444444444
     &4444D0)*CA*CF)*Nf+(17.5D0*T15+17.833333333333333333D0*T19+17.5D0*T
     &17+17.833333333333333333D0*T18+10.916666666666666667D0*T4+18.91666
     &6666666666667D0*T5-9.0416666666666666667D0)*T3+(-28.08333333333333
     &3333D0*T15-29.083333333333333333D0*T19-28.083333333333333333D0*T17
     &-29.083333333333333333D0*T18-36.75D0*T4+0.33333333333333333333D0*T
     &5-40.284722222222222222D0)*CA*T2+(5.0833333333333333333D0*T15+5.5D
     &0*T19+5.0833333333333333333D0*T17+5.5D0*T18+14.270833333333333333D
     &0*T4-6.2708333333333333333D0*T5+22.402777777777777778D0)*T6*CF)*T3
     &4
      T567=T566+((T96-T60+T98-8.0D0*T25+8.0D0*T26-8.0D0*T27+8.0D0*T28-8.
     &0D0*T29+2.5D0*T15-2.5D0*T19+T97-2.5D0*T18)*CA*CF*Nf+(214.0D0*T22-1
     &28.0D0*T23+170.0D0*T24-172.0D0*T25+170.0D0*T26-172.0D0*T27+126.0D0
     &*T28-216.0D0*T29+157.33333333333333333D0*T15+42.333333333333333333
     &D0*T19+147.33333333333333333D0*T17+32.333333333333333333D0*T18+24.
     &5D0*T4+24.5D0*T5)*T3+(-237.0D0*T22+179.0D0*T23-195.0D0*T24+221.0D0
     &*T25-215.0D0*T26+201.0D0*T27-173.0D0*T28+243.0D0*T29-181.75D0*T15-
     &42.75D0*T19-172.75D0*T17-33.75D0*T18-25.166666666666666667D0*T4-25
     &.166666666666666667D0*T5)*CA*T2+(37.5D0*T22-30.0D0*T23+27.5D0*T24-
     &40.0D0*T25+37.5D0*T26-30.0D0*T27+27.5D0*T28-40.0D0*T29+43.29166666
     &6666666667D0*T15+19.041666666666666667D0*T19+41.291666666666666667
     &D0*T17+17.041666666666666667D0*T18+6.4583333333333333333D0*T4+6.45
     &83333333333333333D0*T5)*T6*CF)*im*pi+((T242+T60+T98+1.333333333333
     &3333333D0*T25-6.6666666666666666667D0*T27-6.6666666666666666667D0*
     &T28-13.333333333333333333D0*T29+17.777777777777777778D0*T15+15.777
     &777777777777778D0*T19+15.777777777777777778D0*T17+13.7777777777777
     &77778D0*T18+5.8333333333333333333D0*T4+5.8333333333333333333D0*T5)
     &*T2+(T258-8.0D0*T76-8.0D0*T77-8.0D0*T78+T274-8.0D0*T80-8.0D0*T81-8
     &.0D0*T82-9.8333333333333333333D0*T22-T83-T99-3.1666666666666666667
     &D0*T25+T100+T243+T225+4.1666666666666666667D0*T29-8.88888888888888
     &88889D0*T15-7.8888888888888888889D0*T19-7.8888888888888888889D0*T1
     &7-6.8888888888888888889D0*T18-2.9166666666666666667D0*T4-2.9166666
     &666666666667D0*T5)*CA*CF)*Nf
      T568=T567+(-254.0D0*T75-40.0D0*T84-40.0D0*T85-168.0D0*T76-210.0D0*
     &T77-40.0D0*T86-40.0D0*T87-212.0D0*T78-210.0D0*T79-40.0D0*T88-40.0D
     &0*T89-212.0D0*T80-166.0D0*T81-40.0D0*T90-40.0D0*T91-256.0D0*T82-11
     &0.83333333333333333D0*T22+31.5D0*T23+31.5D0*T24+58.833333333333333
     &333D0*T25-84.833333333333333333D0*T26+47.5D0*T27+47.5D0*T28+64.833
     &333333333333333D0*T29-38.083333333333333333D0*T15-18.0833333333333
     &33333D0*T19-18.083333333333333333D0*T17+T101+19.916666666666666667
     &D0*T4+19.916666666666666667D0*T5)*T3+(261.0D0*T75+24.0D0*T84+24.0D
     &0*T85+203.0D0*T76+219.0D0*T77+24.0D0*T86+24.0D0*T87+245.0D0*T78+23
     &9.0D0*T79+24.0D0*T88+24.0D0*T89+225.0D0*T80+197.0D0*T81+24.0D0*T90
     &+24.0D0*T91+267.0D0*T82+57.0D0*T22-80.583333333333333333D0*T23-80.
     &583333333333333333D0*T24-79.166666666666666667D0*T25+122.166666666
     &66666667D0*T26-6.4166666666666666667D0*T27-6.4166666666666666667D0
     &*T28+T102-68.652777777777777778D0*T15-80.569444444444444444D0*T19-
     &80.569444444444444444D0*T17-92.486111111111111111D0*T18-56.9583333
     &33333333333D0*T4-56.958333333333333333D0*T5)*CA*T2
      T569=(T568+(-39.5D0*T75-T106-2.0D0*T85-32.0D0*T76-29.5D0*T77+T148+
     &T149-42.0D0*T78-39.5D0*T79-T448+T230-32.0D0*T80-29.5D0*T81-2.0D0*T
     &90-T374-42.0D0*T82+7.4583333333333333333D0*T22+32.4166666666666666
     &67D0*T23+32.416666666666666667D0*T24+33.125D0*T25-31.625D0*T26-8.6
     &666666666666666667D0*T27-8.6666666666666666667D0*T28-9.95833333333
     &33333333D0*T29+43.847222222222222222D0*T15+44.805555555555555556D0
     &*T19+44.805555555555555556D0*T17+45.763888888888888889D0*T18+23.5D
     &0*T4+23.5D0*T5)*T6*CF)*T103
      T570=((T327+T222+0.16666666666666666667D0*T17-0.490740740740740740
     &74D0*T4-0.18055555555555555556D0*T5-0.46604938271604938272D0)*T2+(
     &T220+T329+2.0462962962962962963D0*T4+1.6111111111111111111D0*T5+1.
     &7227366255144032922D0)*CA*CF)*Nf+(T333+T334+T335+T336+2.6666666666
     &666666667D0*T26+T27+T104+5.6666666666666666667D0*T29-5.375D0*T15-0
     &.23611111111111111111D0*T19+2.7777777777777777778D0*T17-25.2916666
     &66666666667D0*T18-15.701388888888888889D0*T4+58.298611111111111111
     &D0*T5-97.201388888888888889D0)*T3+(T338+T339+T340+T341-2.416666666
     &6666666667D0*T26-3.6666666666666666667D0*T27-3.75D0*T28-4.66666666
     &66666666667D0*T29+T224+7.5833333333333333333D0*T19-4.9444444444444
     &444444D0*T17+26.875D0*T18+9.5393518518518518518D0*T4-62.1180555555
     &55555556D0*T5+117.17669753086419753D0)*CA*T2+(T342+T343+T344+T345+
     &T550+1.5833333333333333333D0*T27+T104+0.91666666666666666667D0*T29
     &-2.1111111111111111111D0*T15-3.8472222222222222222D0*T19+1.2777777
     &777777777778D0*T17-7.2083333333333333333D0*T18-7.94907407407407407
     &41D0*T4+10.958333333333333333D0*T5-40.069187242798353909D0)*T6*CF
      T571=(T348+T349+0.037037037037037037037D0*T5+0.3343621399176954732
     &5D0)*CF*T20+((T350+T226+T351+T227+T237+T347+0.88888888888888888889
     &D0*T29+0.40740740740740740741D0*T15+2.6296296296296296296D0*T19+1.
     &462962962962962963D0*T17-3.4351851851851851852D0*T18-6.56481481481
     &48148148D0*T4+9.8703703703703703704D0*T5-13.179462448559670782D0)*
     &T2+(T108+T355+T356+T357+T358+T353+T359-0.44444444444444444444D0*T2
     &9-1.462962962962962963D0*T15-2.9537037037037037037D0*T19-2.0185185
     &185185185185D0*T17+0.064814814814814814815D0*T18+0.929012345679012
     &34568D0*T4-6.2361111111111111111D0*T5+2.8208447645176040238D0)*CA*
     &CF)*Nf+(T360+T361+T362+T363+T364+T365+T235+T233+T229+T266-1.5D0*T8
     &9-T107-3.3333333333333333333D0*T81-3.8333333333333333333D0*T90+T38
     &1-T105+T223+T548-2.75D0*T24+4.8194444444444444444D0*T25+T100+15.02
     &7777777777777778D0*T27+11.111111111111111111D0*T28+8.375D0*T29-11.
     &229166666666666667D0*T15+1.4861111111111111111D0*T19-23.8472222222
     &22222222D0*T17-22.819444444444444444D0*T18-37.888888888888888889D0
     &*T4-15.961805555555555556D0*T5+23.450520833333333333D0)*T3
      T572=T571+(T368+T106+T369+T145+T370+T371+T372+T373+0.1666666666666
     &6666667D0*T79+T366+2.75D0*T89+T107+7.75D0*T81+5.25D0*T90+3.6666666
     &666666666667D0*T91+2.75D0*T82-2.6111111111111111111D0*T22-11.47222
     &2222222222222D0*T23+4.2916666666666666667D0*T24-2.8055555555555555
     &555D0*T25-5.0138888888888888889D0*T26-6.6527777777777777778D0*T27-
     &7.3611111111111111111D0*T28-12.930555555555555556D0*T29+21.7037037
     &03703703704D0*T15-2.775462962962962963D0*T19+15.68287037037037037D
     &0*T17+50.060185185185185185D0*T18+93.342592592592592593D0*T4-32.08
     &5648148148148148D0*T5+61.191518775720164609D0)*CA*T2+(T376-T109+T3
     &77+T378+T228+T379+T463+T236-0.58333333333333333333D0*T80-2.3333333
     &333333333333D0*T81-1.6666666666666666667D0*T90-1.16666666666666666
     &67D0*T91-0.75D0*T82+T382+2.8611111111111111111D0*T23-2.94444444444
     &44444444D0*T24+0.375D0*T25+1.9027777777777777778D0*T26-0.222222222
     &22222222222D0*T27-0.16666666666666666667D0*T28+4.23611111111111111
     &11D0*T29-6.0416666666666666667D0*T15+7.0717592592592592593D0*T19+2
     &.4560185185185185185D0*T17-13.287037037037037037D0*T18-34.98649691
     &3580246913D0*T4+23.803240740740740741D0*T5-30.632658893461362597D0
     &)*T6*CF
      T573=(-T108+T384+T331+T386-0.24074074074074074074D0*T5-1.412208504
     &8010973937D0)*CF*T20+((T387-T106+T109+T388+T389-1.6666666666666666
     &667D0*T89-1.3333333333333333333D0*T81-0.66666666666666666667D0*T91
     &+T392-4.8888888888888888889D0*T23+4.9444444444444444444D0*T24+0.66
     &666666666666666667D0*T25+T337+0.83333333333333333333D0*T27-0.88888
     &888888888888889D0*T28+T263+3.5185185185185185185D0*T15-7.138888888
     &8888888889D0*T19+8.2222222222222222222D0*T17-2.2222222222222222222
     &D0*T18+5.0563271604938271605D0*T4+9.6203703703703703704D0*T5-7.853
     &3950617283950617D0)*T2+(T84-T109+T395+T241-6.5555555555555555556D0
     &*T22+10.944444444444444444D0*T23-12.111111111111111111D0*T24+8.5D0
     &*T25-10.5D0*T26+8.3333333333333333333D0*T27-10.5D0*T28+9.0D0*T29+2
     &.1018518518518518518D0*T15+10.805555555555555556D0*T19-10.47222222
     &2222222222D0*T17+6.6944444444444444444D0*T18+14.941358024691358025
     &D0*T4-0.68518518518518518518D0*T5+28.366083676268861454D0)*CA*CF)*
     &Nf+(T396+T308+T138-T264+T140+T397+T115+T398+T399+T248+T118-T254+T4
     &00+T401-T255-T256+7.0D0*T124+T143-6.0D0*T126-9.0D0*T127+T128-8.0D0
     &*T129-18.0D0*T130+5.0D0*T131-T420+18.0D0*T133-27.0D0*T134+17.0D0*T
     &135-14.0D0*T136+32.0D0*T137-0.5D0*T75+T110+11.0D0*T85-2.0D0*T76+26
     &.333333333333333333D0*T77+16.0D0*T86+26.666666666666666667D0*T87+4
     &2.833333333333333333D0*T78+69.666666666666666667D0*T79-5.0D0*T88+5
     &4.5D0*T89-47.666666666666666667D0*T80+54.833333333333333333D0*T81-
     &72.666666666666666667D0*T90+11.5D0*T91-103.5D0*T82-298.08333333333
     &333333D0*T22+56.5D0*T23-202.25D0*T24+142.83333333333333333D0*T25-1
     &71.25D0*T26+218.58333333333333333D0*T27-72.166666666666666667D0*T2
     &8+300.41666666666666667D0*T29-372.83333333333333333D0*T15-88.375D0
     &*T19-346.33333333333333333D0*T17-53.0D0*T18-72.427083333333333333D
     &0*T4-20.208333333333333333D0*T5-2.84375D0)*T3
      T574=T573+(T206+T409+T410+T411-T115+T412-T309+T253+T413+T414+T415+
     &T416+T417-5.5D0*T124+4.5D0*T125+T249+11.5D0*T127-T217+T403-5.0D0*T
     &144+28.0D0*T130-9.5D0*T131+T480-22.0D0*T133+35.5D0*T134-9.5D0*T135
     &+15.0D0*T136-28.0D0*T137-31.666666666666666667D0*T75+14.75D0*T84-1
     &6.083333333333333333D0*T85+T232-32.416666666666666667D0*T77-18.75D
     &0*T86-24.833333333333333333D0*T87-28.333333333333333333D0*T78-86.6
     &66666666666666667D0*T79+3.0D0*T88-35.333333333333333333D0*T89+49.6
     &66666666666666667D0*T80-49.75D0*T81+50.166666666666666667D0*T90+9.
     &9166666666666666667D0*T91+109.0D0*T82+287.0D0*T22-63.1944444444444
     &44444D0*T23+187.09722222222222222D0*T24-206.125D0*T25+181.20833333
     &333333333D0*T26-242.79166666666666667D0*T27+95.222222222222222222D
     &0*T28-349.25D0*T29+420.38425925925925926D0*T15+126.597222222222222
     &22D0*T19+379.73611111111111111D0*T17+81.555555555555555556D0*T18+5
     &4.361496913580246913D0*T4-60.523148148148148148D0*T5+77.0929783950
     &61728395D0)*CA*T2+(T111+T138+T139+T140+T421+T422+T423+T142+T424+T4
     &25+T426+T427+T428+T429+T124+T143-0.5D0*T126-1.5D0*T127+T128+2.5D0*
     &T144-7.0D0*T130+3.5D0*T131+T430+6.5D0*T133-10.5D0*T134+0.5D0*T135-
     &4.5D0*T136+6.0D0*T137+T231-5.75D0*T84+9.0833333333333333333D0*T85+
     &T145+10.833333333333333333D0*T77+5.25D0*T86+11.25D0*T87+2.58333333
     &33333333333D0*T78+24.666666666666666667D0*T79+T150+8.75D0*T89-13.6
     &66666666666666667D0*T80+14.833333333333333333D0*T81+T152-5.75D0*T9
     &1-28.5D0*T82-62.180555555555555556D0*T22-13.652777777777777778D0*T
     &23-7.0555555555555555556D0*T24+35.083333333333333333D0*T25-16.875D
     &0*T26+34.875D0*T27+12.083333333333333333D0*T28+64.1666666666666666
     &67D0*T29-128.81481481481481481D0*T15-67.055555555555555556D0*T19-8
     &2.027777777777777778D0*T17-47.138888888888888889D0*T18-54.88271604
     &9382716049D0*T4+23.592592592592592593D0*T5-100.73705418381344307D0
     &)*T6*CF
      T575=(-0.13387804967666078777D0*T3+0.081275720164609053498D0*CA*T2
     &-0.0090045071526553008034D0*T6*CF)*T218+((-0.026851851851851851852
     &D0*T2+0.0083333333333333333333D0*CA*CF)*Nf+(T313+0.202777777777777
     &77778D0*T5-1.1997685185185185185D0)*T3+(T314-0.2208333333333333333
     &3D0*T5+1.605787037037037037D0)*CA*T2+(T315+0.059722222222222222222
     &D0*T5-0.49351851851851851852D0)*T6*CF)*im*T219+(T316+((T317-0.0037
     &037037037037037037D0*T5-0.088554526748971193416D0)*T2+(T319+T320-1
     &.0523662551440329218D0)*CA*CF)*Nf+(T321+T322+0.3D0*T17-1.861111111
     &1111111111D0*T18+1.3388888888888888889D0*T4+6.2003472222222222222D
     &0*T5-19.012152777777777778D0)*T3+(T323+T324+0.14166666666666666667
     &D0*T17+1.4944444444444444444D0*T18-1.7314814814814814815D0*T4-5.71
     &22685185185185185D0*T5+24.593299897119341564D0)*CA*T2+(T325+T326-0
     &.17777777777777777778D0*T17-0.28194444444444444444D0*T18+0.7967592
     &5925925925926D0*T4+1.3453703703703703704D0*T5-3.513477366255144032
     &9D0)*T6*CF)*T93+T570*im*T33+T572*T34+T574*im*pi+(T435+T436+T394+T1
     &51+T385-0.077160493827160493827D0*T5-1.1922629934461210181D0)*CF*T
     &20
      T576=(T437+T251-T147+T269+T438+T271+T272+T273+T439+0.3333333333333
     &3333333D0*T127+T440+T441-1.3333333333333333333D0*T144+T443+T444+2.
     &6666666666666666667D0*T134+3.3333333333333333333D0*T135+3.33333333
     &33333333333D0*T136+4.6666666666666666667D0*T137+T445-4.88888888888
     &88888889D0*T76-0.94444444444444444445D0*T77+T446+T447+7.0D0*T78-1.
     &3333333333333333333D0*T79+T88+T89-2.8333333333333333333D0*T80-6.33
     &33333333333333333D0*T81-11.888888888888888889D0*T90-11.88888888888
     &8888889D0*T91-14.888888888888888889D0*T82-20.5D0*T22-10.1666666666
     &66666667D0*T23-10.166666666666666667D0*T24-7.9722222222222222222D0
     &*T25+10.555555555555555556D0*T26+28.111111111111111111D0*T27+28.11
     &1111111111111111D0*T28+34.055555555555555556D0*T29-38.111882716049
     &382716D0*T15-28.388888888888888889D0*T19-28.388888888888888889D0*T
     &17-13.527777777777777778D0*T18-14.777777777777777778D0*T4-21.11149
     &6913580246914D0*T5+7.4765946502057613169D0)*T2+(T139+T147+T454+T14
     &2+T281+T282+T455+T265+T456+T457+T458+T442+T460-0.66666666666666666
     &667D0*T133+T431-1.6666666666666666667D0*T135-1.6666666666666666667
     &D0*T136-2.3333333333333333333D0*T137+1.6666666666666666667D0*T75+1
     &0.944444444444444444D0*T76+10.111111111111111111D0*T77+T148+T149+5
     &.1666666666666666667D0*T78+9.0D0*T79+T150-0.5D0*T89+10.16666666666
     &6666667D0*T80+14.111111111111111111D0*T81+5.9444444444444444444D0*
     &T90+5.9444444444444444444D0*T91+17.277777777777777778D0*T82-0.7037
     &037037037037037D0*T22+5.0833333333333333333D0*T23+5.08333333333333
     &33333D0*T24+11.222222222222222222D0*T25+1.0833333333333333333D0*T2
     &6-14.055555555555555556D0*T27-14.055555555555555556D0*T28-13.13888
     &8888888888889D0*T29+T151+14.194444444444444444D0*T19+14.1944444444
     &44444444D0*T17+12.407407407407407407D0*T18+4.0D0*T4+10.14506172839
     &5061728D0*T5+24.101489864349946654D0)*CA*CF
      T577=T466+T467+T154-T296+T285+T286+T468+T469+T470+T471+T472+T473+T
     &287+T288+T474+T475+T476+T212+T300+T171+T172+T477+T301+T302+T303+7.
     &0D0*T177-4.0D0*T178-6.0D0*T179-6.0D0*T180-12.0D0*T181+6.0D0*T182+T
     &534+T535+T304+T278-10.0D0*T187-10.0D0*T188-10.0D0*T189+T536-T537-T
     &538-1.0D0*T193+7.0D0*T194-6.0D0*T195-6.0D0*T196+T293+38.0D0*T198+1
     &1.0D0*T199+11.0D0*T200+28.0D0*T201+20.0D0*T202+6.0D0*T203+6.0D0*T2
     &04+38.0D0*T205+4.0D0*T111+4.0D0*T112-0.5D0*T139+20.0D0*T113+T207+T
     &208+32.0D0*T116-20.0D0*T142+7.8333333333333333333D0*T117+7.8333333
     &333333333333D0*T118+23.833333333333333333D0*T119+31.0D0*T120+57.66
     &6666666666666667D0*T121+57.666666666666666667D0*T122+100.5D0*T123-
     &74.0D0*T209-4.3333333333333333333D0*T124-4.3333333333333333333D0*T
     &125-9.3333333333333333333D0*T126-30.833333333333333333D0*T127+23.6
     &66666666666666667D0*T128+23.666666666666666667D0*T129-24.0D0*T144-
     &35.666666666666666667D0*T130+19.166666666666666667D0*T131
      T578=T482-T154+T294+T295+T483+T484+T485+T486+T487+T488+T489+T490+T
     &298+T491+T492+T166+T493+T494+T495+T496-T171-T172+T497+T498+T499+T5
     &00-5.5D0*T177+4.5D0*T178+9.0D0*T179+9.0D0*T180+13.0D0*T181-T505+1.
     &5D0*T183+1.5D0*T184-1.0D0*T185+7.0D0*T186+11.0D0*T187+11.0D0*T188+
     &6.0D0*T189-17.0D0*T190+11.0D0*T191+11.0D0*T192+1.5D0*T193-0.5D0*T1
     &94+13.0D0*T195+13.0D0*T196-9.0D0*T197-44.0D0*T198-8.5D0*T199-8.5D0
     &*T200-18.0D0*T201-16.0D0*T202+T215+T216-29.0D0*T205+T508+T206-T246
     &+8.75D0*T139-14.916666666666666667D0*T113-T207-T208-43.33333333333
     &3333333D0*T116+30.166666666666666667D0*T142-7.4166666666666666667D
     &0*T117-7.4166666666666666667D0*T118-26.166666666666666667D0*T119-3
     &4.333333333333333333D0*T120-59.166666666666666667D0*T121-59.166666
     &666666666667D0*T122-87.5D0*T123+81.5D0*T209-5.1666666666666666667D
     &0*T124-5.1666666666666666667D0*T125-2.1666666666666666667D0*T126+1
     &8.5D0*T127-16.833333333333333333D0*T128
      T579=T578-16.833333333333333333D0*T129+32.833333333333333333D0*T14
     &4+28.333333333333333333D0*T130-22.916666666666666667D0*T131-22.916
     &666666666666667D0*T132+18.416666666666666667D0*T133-65.41666666666
     &6666667D0*T134-64.333333333333333333D0*T135-64.333333333333333333D
     &0*T136+35.833333333333333333D0*T137-338.16666666666666667D0*T75-45
     &.666666666666666667D0*T84-45.666666666666666667D0*T85-108.86111111
     &111111111D0*T76-274.38888888888888889D0*T77-87.291666666666666667D
     &0*T86-87.291666666666666667D0*T87-309.25D0*T78-211.75D0*T79-20.958
     &333333333333333D0*T88-20.958333333333333333D0*T89-232.833333333333
     &33333D0*T80-52.166666666666666667D0*T81+73.972222222222222222D0*T9
     &0+73.972222222222222222D0*T91-246.19444444444444444D0*T82-192.5D0*
     &T22+183.29166666666666667D0*T23+183.29166666666666667D0*T24+248.05
     &555555555555556D0*T25-453.38888888888888889D0*T26-135.486111111111
     &11111D0*T27-135.48611111111111111D0*T28-106.34722222222222222D0*T2
     &9+113.65239197530864198D0*T15+137.84722222222222222D0*T19+137.8472
     &2222222222222D0*T17+43.819444444444444444D0*T18
      T580=T153+T513+T514+T515+T516+T517+T518+T519+T520+T521+T522+T523+T
     &524+T212+T525+T526+T527+T528+T529+T530+T531+T177+T532-3.0D0*T179-3
     &.0D0*T180-3.5D0*T181+1.5D0*T182+T185-3.0D0*T186-3.0D0*T187-3.0D0*T
     &188-0.5D0*T189+3.0D0*T190-4.0D0*T191-4.0D0*T192-0.5D0*T193-1.0D0*T
     &194-5.0D0*T195-5.0D0*T196+1.5D0*T197+12.0D0*T198+1.5D0*T199+1.5D0*
     &T200+2.0D0*T201+3.5D0*T202+T215+T216+T307+T250+2.0D0*T112-3.75D0*T
     &139-T147+7.75D0*T114+7.75D0*T141+13.166666666666666667D0*T116-12.7
     &5D0*T142+1.75D0*T117+1.75D0*T118+T311+3.9166666666666666667D0*T120
     &+15.166666666666666667D0*T121+15.166666666666666667D0*T122+17.75D0
     &*T123-21.0D0*T209+3.6666666666666666667D0*T124+3.66666666666666666
     &67D0*T125+3.1666666666666666667D0*T126-6.25D0*T127+T217+2.5D0*T129
     &-11.166666666666666667D0*T144-11.833333333333333333D0*T130+6.66666
     &66666666666667D0*T131+6.6666666666666666667D0*T132+3.1666666666666
     &666667D0*T133+26.0D0*T134+23.916666666666666667D0*T135
      T581=(T580+23.916666666666666667D0*T136-0.91666666666666666667D0*T
     &137+81.666666666666666667D0*T75+6.0416666666666666667D0*T84+6.0416
     &666666666666667D0*T85-7.6111111111111111111D0*T76+31.9722222222222
     &22222D0*T77+24.916666666666666667D0*T86+24.916666666666666667D0*T8
     &7+67.333333333333333333D0*T78+29.583333333333333333D0*T79+7.208333
     &3333333333333D0*T88+7.2083333333333333333D0*T89+29.25D0*T80-34.444
     &444444444444444D0*T81-35.194444444444444444D0*T90-35.1944444444444
     &44444D0*T91+16.138888888888888889D0*T82+56.837962962962962963D0*T2
     &2-69.208333333333333333D0*T23-69.208333333333333333D0*T24-110.5972
     &2222222222222D0*T25+127.60416666666666667D0*T26+71.243055555555555
     &556D0*T27+71.243055555555555556D0*T28+49.159722222222222222D0*T29-
     &38.207561728395061728D0*T15-82.090277777777777778D0*T19-82.0902777
     &77777777778D0*T17-47.641203703703703704D0*T18-52.375D0*T4-75.80401
     &2345679012346D0*T5-95.684111130163084895D0)*T6*CF
      T582=(T575+T576*Nf+(T577+19.166666666666666667D0*T132-50.5D0*T133+
     &18.5D0*T134+33.0D0*T135+33.0D0*T136-67.5D0*T137+365.25D0*T75+67.16
     &6666666666666667D0*T84+67.166666666666666667D0*T85+123.66666666666
     &666667D0*T76+277.16666666666666667D0*T77+74.916666666666666667D0*T
     &86+74.916666666666666667D0*T87+223.0D0*T78+187.33333333333333333D0
     &*T79+13.083333333333333333D0*T88+13.083333333333333333D0*T89+221.1
     &6666666666666667D0*T80+75.5D0*T81+T152-7.1666666666666666667D0*T91
     &+283.5D0*T82+263.58333333333333333D0*T22-89.75D0*T23-89.75D0*T24-1
     &57.125D0*T25+311.33333333333333333D0*T26-14.0D0*T27-14.0D0*T28-49.
     &75D0*T29+114.59375D0*T15+52.666666666666666667D0*T19+52.6666666666
     &66666667D0*T17+45.208333333333333333D0*T18-72.125D0*T4-65.328125D0
     &*T5-17.609375D0)*T3+(T579+167.36111111111111111D0*T4+186.131751543
     &20987654D0*T5+53.801697530864197531D0)*CA*T2+T581)*T10
      T583=(-0.46531268371546149324D0*T3+0.51668136390358612581D0*CA*T2-
     &0.1383401920438957476D0*T6*CF)*T218+((0.053703703703703703704D0*T2
     &-0.016666666666666666667D0*CA*CF)*Nf+(-0.5375D0*T4+0.9444444444444
     &4444445D0*T5-0.6900462962962962963D0)*T3+(0.56111111111111111111D0
     &*T4-1.0166666666666666667D0*T5+0.73703703703703703704D0)*CA*T2+(-0
     &.094444444444444444444D0*T4+0.27222222222222222222D0*T5-0.26018518
     &518518518518D0)*T6*CF)*im*T219+(-0.0088477366255144032922D0*CF*T20
     &+((0.081481481481481481481D0*T4-T318-0.27844650205761316872D0)*T2+
     &(0.046296296296296296296D0*T4+0.0074074074074074074074D0*T5+0.8121
     &3991769547325103D0)*CA*CF)*Nf+(-1.0625D0*T15-1.4555555555555555556
     &D0*T19-1.65D0*T17-3.3111111111111111111D0*T18-1.622222222222222222
     &2D0*T4-3.2458333333333333333D0*T5+4.6743055555555555556D0)*T3+(0.7
     &D0*T15+1.7722222222222222222D0*T19+1.0083333333333333333D0*T17+1.8
     &444444444444444444D0*T18+2.6185185185185185185D0*T4+2.587037037037
     &037037D0*T5-3.068544238683127572D0)*CA*T2+(-0.03333333333333333333
     &3D0*T15-0.52222222222222222222D0*T19-0.027777777777777777778D0*T17
     &-0.094444444444444444444D0*T18-1.550462962962962963D0*T4-0.6393518
     &5185185185185D0*T5-2.3505915637860082304D0)*T6*CF)*T93
      T584=((T220+0.44444444444444444444D0*T19-T221+1.037037037037037037
     &D0*T4-0.38888888888888888889D0*T5-0.16049382716049382716D0)*T2+(-0
     &.55555555555555555555D0*T15+T222-1.537037037037037037D0*T4-0.55555
     &555555555555555D0*T5-1.3158436213991769547D0)*CA*CF)*Nf+(T223-3.0D
     &0*T23-5.1666666666666666667D0*T24+8.6666666666666666667D0*T25+8.66
     &66666666666666667D0*T26+T27-2.3333333333333333333D0*T28+14.6666666
     &66666666667D0*T29-4.25D0*T15+5.5555555555555555556D0*T19+15.111111
     &111111111111D0*T17+4.8333333333333333333D0*T18+21.5972222222222222
     &22D0*T4+4.5416666666666666667D0*T5+64.034722222222222222D0)*T3+(T5
     &47+T451+6.5D0*T24-11.0D0*T25-9.1666666666666666667D0*T26-5.1666666
     &666666666667D0*T27+T28-10.666666666666666667D0*T29+T224-12.4166666
     &66666666667D0*T19-15.194444444444444444D0*T17-2.833333333333333333
     &3D0*T18-28.537037037037037037D0*T4-11.944444444444444444D0*T5-83.3
     &37191358024691358D0)*CA*T2+(-1.3333333333333333333D0*T22-0.3333333
     &3333333333333D0*T23-2.3333333333333333333D0*T24+3.3333333333333333
     &333D0*T25+2.1666666666666666667D0*T26+2.3333333333333333333D0*T27-
     &0.33333333333333333333D0*T28+1.6666666666666666667D0*T29-T220+5.27
     &77777777777777778D0*T19+5.1944444444444444444D0*T17+0.583333333333
     &33333333D0*T18+13.74537037037037037D0*T4+7.8888888888888888889D0*T
     &5+31.550411522633744856D0)*T6*CF
      T585=(-0.14814814814814814815D0*T15-0.46913580246913580247D0*T4+T3
     &32-0.43415637860082304527D0)*CF*T20+((-0.55555555555555555555D0*T2
     &2-1.1111111111111111111D0*T23-0.66666666666666666667D0*T24+0.44444
     &444444444444444D0*T25+T352-2.6666666666666666667D0*T27-T225+3.5555
     &555555555555555D0*T29-1.8148148148148148148D0*T15-0.03703703703703
     &7037037D0*T19+3.6296296296296296296D0*T17-2.5185185185185185185D0*
     &T18+2.8518518518518518518D0*T4-3.8333333333333333333D0*T5+6.090406
     &3786008230453D0)*T2+(-T238+T226-0.22222222222222222222D0*T24+T227-
     &0.44444444444444444444D0*T26+T244+1.1111111111111111111D0*T28-1.77
     &77777777777777778D0*T29+1.2592592592592592593D0*T15+1.240740740740
     &7407407D0*T19-1.2962962962962962963D0*T17+2.537037037037037037D0*T
     &18+2.6049382716049382716D0*T4+3.8703703703703703704D0*T5+2.7641746
     &6849565615D0)*CA*CF)*Nf+(8.5D0*T75-5.3333333333333333333D0*T84+6.6
     &666666666666666667D0*T85+4.3333333333333333333D0*T76+T433+T228-3.3
     &333333333333333333D0*T87+2.0D0*T78+T229-19.333333333333333333D0*T8
     &8+T230-11.333333333333333333D0*T80+T380-17.333333333333333333D0*T9
     &0-10.666666666666666667D0*T91+T261+6.6666666666666666667D0*T22-T83
     &+9.3333333333333333333D0*T24+3.4444444444444444444D0*T25-7.1666666
     &666666666667D0*T26-3.2222222222222222222D0*T27+3.11111111111111111
     &11D0*T28+3.1666666666666666667D0*T29+22.625D0*T15-19.1111111111111
     &11111D0*T19+21.138888888888888889D0*T17+10.708333333333333333D0*T1
     &8+56.027777777777777778D0*T4-12.833333333333333333D0*T5-3.61631944
     &44444444444D0)*T3
      T586=T585+(T231-T239-10.666666666666666667D0*T85+T232-6.1666666666
     &666666667D0*T77-4.6666666666666666667D0*T86-3.6666666666666666667D
     &0*T87+T233+T234+16.666666666666666667D0*T88+T464+6.333333333333333
     &3333D0*T80+T81+15.0D0*T90+12.666666666666666667D0*T91+T375+3.88888
     &88888888888889D0*T22+0.77777777777777777778D0*T23-11.0D0*T24-4.638
     &8888888888888889D0*T25+10.694444444444444444D0*T26+17.888888888888
     &888889D0*T27+12.888888888888888889D0*T28-17.055555555555555556D0*T
     &29-16.282407407407407407D0*T15+7.5925925925925925926D0*T19-32.8657
     &40740740740741D0*T17-3.3009259259259259259D0*T18-87.64351851851851
     &8518D0*T4+31.104166666666666667D0*T5-34.045074588477366255D0)*CA*T
     &2+(3.3333333333333333333D0*T84+T240+0.66666666666666666667D0*T76+T
     &77+2.6666666666666666667D0*T86+T235-T312+T236-0.333333333333333333
     &33D0*T80-1.8333333333333333333D0*T81-3.1666666666666666667D0*T90-4
     &.1666666666666666667D0*T91+2.4444444444444444444D0*T22-0.388888888
     &88888888889D0*T23+6.8055555555555555556D0*T24+0.75D0*T25-T237-8.97
     &22222222222222222D0*T27-4.1666666666666666667D0*T28+8.277777777777
     &7777778D0*T29+6.2916666666666666667D0*T15-4.4074074074074074074D0*
     &T19+9.9907407407407407407D0*T17-6.0231481481481481481D0*T18+26.822
     &530864197530864D0*T4-17.194444444444444444D0*T5+7.5554412437128486
     &511D0)*T6*CF
      T587=30.0D0*T146+T279+T246+8.0D0*T139+T252-20.0D0*T114-T247-60.0D0
     &*T116-6.0D0*T142-8.0D0*T117-T310+4.0D0*T119+4.0D0*T120+28.0D0*T121
     &+20.0D0*T122+16.0D0*T123-2.0D0*T124-20.0D0*T125-36.0D0*T127-14.0D0
     &*T128-38.0D0*T129-12.0D0*T144-132.0D0*T130-4.0D0*T131-88.0D0*T132+
     &24.0D0*T133-96.0D0*T134+20.0D0*T135-44.0D0*T136+68.0D0*T137-7.0D0*
     &T75-T554-14.0D0*T85-4.0D0*T76+0.33333333333333333333D0*T77-8.0D0*T
     &86+T241-18.666666666666666667D0*T78+36.666666666666666667D0*T79+34
     &.0D0*T88-21.0D0*T89+55.333333333333333333D0*T80-50.666666666666666
     &667D0*T81+29.333333333333333333D0*T90-76.0D0*T91+26.0D0*T82+213.83
     &333333333333333D0*T22+38.5D0*T23+129.5D0*T24-28.833333333333333333
     &D0*T25+152.66666666666666667D0*T26-9.8333333333333333333D0*T27+151
     &.0D0*T28-54.5D0*T29+327.16666666666666667D0*T15+9.8333333333333333
     &333D0*T19+235.08333333333333333D0*T17-65.0D0*T18+96.1875D0*T4+30.5
     &41666666666666667D0*T5-29.072916666666666667D0
      T588=T478+T509+T140+26.0D0*T114+T247+54.0D0*T116+T248-12.0D0*T118-
     &6.0D0*T119-18.0D0*T120-26.0D0*T121-26.0D0*T122-24.0D0*T123-1.0D0*T
     &124+15.0D0*T125+T249+16.0D0*T127+11.0D0*T128+19.0D0*T129+4.0D0*T14
     &4+124.0D0*T130+22.0D0*T131+90.0D0*T132-16.0D0*T133+118.0D0*T134+T2
     &57+60.0D0*T136-52.0D0*T137+75.333333333333333333D0*T75-10.0D0*T84+
     &29.666666666666666667D0*T85+20.0D0*T76+30.333333333333333333D0*T77
     &+23.0D0*T86+14.666666666666666667D0*T87+11.666666666666666667D0*T7
     &8-3.6666666666666666667D0*T79-53.0D0*T88+5.6666666666666666667D0*T
     &89-60.333333333333333333D0*T80+57.0D0*T81-4.3333333333333333333D0*
     &T90+69.666666666666666667D0*T91-T408-152.66666666666666667D0*T22-1
     &09.36111111111111111D0*T23-63.277777777777777778D0*T24+68.75D0*T25
     &-131.83333333333333333D0*T26+18.166666666666666667D0*T27-201.44444
     &444444444444D0*T28+75.833333333333333333D0*T29-327.101851851851851
     &85D0*T15-60.527777777777777778D0*T19-140.23611111111111111D0*T17+5
     &3.152777777777777778D0*T18
      T589=(T588-8.2785493827160493827D0*T4+82.953703703703703704D0*T5-3
     &7.65779320987654321D0)*CA*T2
      T590=(T238+T354-0.44444444444444444444D0*T18+2.5061728395061728395
     &D0*T4+1.1851851851851851852D0*T5+1.5713305898491083676D0)*CF*T20+(
     &(-9.3333333333333333333D0*T75+T239-T240-2.6666666666666666667D0*T7
     &7+T241+3.3333333333333333333D0*T89+2.6666666666666666667D0*T81+1.3
     &333333333333333333D0*T91-T242+7.7777777777777777778D0*T23-11.55555
     &5555555555556D0*T24-2.6666666666666666667D0*T25-T552-T243+7.111111
     &1111111111111D0*T28+T452-8.037037037037037037D0*T15+8.444444444444
     &4444444D0*T19-26.555555555555555556D0*T17+2.5555555555555555555D0*
     &T18-16.557098765432098765D0*T4-10.675925925925925926D0*T5+2.763117
     &283950617284D0)*T2+(-T106+T240+T555+2.6666666666666666667D0*T87-0.
     &88888888888888888889D0*T22-5.8888888888888888889D0*T23+10.22222222
     &2222222222D0*T24-T393+6.3333333333333333333D0*T26-T244+T245-T102-1
     &2.037037037037037037D0*T15-11.5D0*T19+14.888888888888888889D0*T17-
     &2.2777777777777777778D0*T18-29.299382716049382716D0*T4-10.34259259
     &2592592593D0*T5-35.568587105624142661D0)*CA*CF)*Nf+T587*T3+T589+(-
     &T250+T246+T251+T252-8.0D0*T114-6.0D0*T141-12.0D0*T116+T270+T253+T2
     &54+6.0D0*T120+6.0D0*T121+T255+T256+T124+T143-T402+T418-2.0D0*T128+
     &T144-34.0D0*T130-10.0D0*T131-22.0D0*T132+T404-36.0D0*T134-T257-18.
     &0D0*T136+9.0D0*T137+T258+T259-19.166666666666666667D0*T85-7.0D0*T7
     &6-15.666666666666666667D0*T77-T260-18.0D0*T87+2.333333333333333333
     &3D0*T78-2.3333333333333333333D0*T79+19.0D0*T88-7.0D0*T89+19.333333
     &333333333333D0*T80-20.666666666666666667D0*T81-4.16666666666666666
     &67D0*T90-20.5D0*T91+T261+60.027777777777777778D0*T22+52.5555555555
     &55555556D0*T23-19.638888888888888889D0*T24-13.083333333333333333D0
     &*T25+19.583333333333333333D0*T26+5.75D0*T27+30.25D0*T28+T262+133.7
     &5462962962962963D0*T15+56.291666666666666667D0*T19+10.027777777777
     &777778D0*T17-3.0277777777777777778D0*T18+82.265432098765432099D0*T
     &4-7.3425925925925925926D0*T5+120.82750342935528121D0)*T6*CF
      T591=13.333333333333333333D0*T146+T264+T268-2.6666666666666666667D
     &0*T116+1.3333333333333333333D0*T142-2.6666666666666666667D0*T117-2
     &.6666666666666666667D0*T118-2.6666666666666666667D0*T119+T265-8.66
     &66666666666666667D0*T127-5.3333333333333333333D0*T128-5.3333333333
     &333333333D0*T129-5.3333333333333333333D0*T144-10.66666666666666666
     &7D0*T130+5.3333333333333333333D0*T133+6.6666666666666666667D0*T134
     &+13.333333333333333333D0*T135+13.333333333333333333D0*T136+18.6666
     &66666666666667D0*T137+12.666666666666666667D0*T75+7.77777777777777
     &77778D0*T76+11.555555555555555556D0*T77-5.3333333333333333333D0*T7
     &8+13.333333333333333333D0*T79-T266-T367-T390-17.333333333333333333
     &D0*T81-7.5555555555555555556D0*T90-7.5555555555555555556D0*T91-11.
     &555555555555555556D0*T82+10.333333333333333333D0*T22+T41+T267+5.77
     &77777777777777778D0*T25+15.777777777777777778D0*T26-15.44444444444
     &4444444D0*T27-15.444444444444444444D0*T28-19.222222222222222222D0*
     &T29+34.334876543209876543D0*T15+14.444444444444444444D0*T19+14.444
     &444444444444444D0*T17+5.8055555555555555556D0*T18
      T592=((T591+11.888888888888888889D0*T4+11.779320987654320988D0*T5-
     &15.480967078189300412D0)*T2+(T251-T268+T269+T270+T271+T272+T273-2.
     &6666666666666666667D0*T120+2.6666666666666666667D0*T127+2.66666666
     &66666666667D0*T128+2.6666666666666666667D0*T129+2.6666666666666666
     &667D0*T144+2.6666666666666666667D0*T130-2.6666666666666666667D0*T1
     &33-T405-6.6666666666666666667D0*T135-6.6666666666666666667D0*T136-
     &9.3333333333333333333D0*T137+10.666666666666666667D0*T75-5.8888888
     &888888888889D0*T76-10.222222222222222222D0*T77+T274+0.333333333333
     &33333333D0*T88+T275-T107+1.1111111111111111111D0*T81+3.77777777777
     &77777778D0*T90+3.7777777777777777778D0*T91-1.5555555555555555556D0
     &*T82+25.407407407407407407D0*T22+T70-1.0D0*T24-10.1666666666666666
     &67D0*T25-9.5D0*T26+7.7222222222222222222D0*T27+7.72222222222222222
     &22D0*T28+15.555555555555555556D0*T29+21.077160493827160494D0*T15-7
     &.2222222222222222222D0*T19-7.2222222222222222222D0*T17-28.04629629
     &6296296296D0*T18+0.75D0*T4+9.6234567901234567901D0*T5-48.202979728
     &699893309D0)*CA*CF)*Nf
      T593=-30.0D0*T276-16.0D0*T153-T284+8.0D0*T155-4.0D0*T156-4.0D0*T15
     &7-24.0D0*T158-24.0D0*T159-40.0D0*T160-40.0D0*T161-100.0D0*T162+14.
     &0D0*T163+8.0D0*T164+8.0D0*T165+6.0D0*T166+4.0D0*T167+4.0D0*T168+T2
     &99-20.0D0*T170-T289-T290+12.0D0*T173-36.0D0*T174-16.0D0*T175-16.0D
     &0*T176-2.0D0*T177-28.0D0*T178-T291-T292-48.0D0*T181+36.0D0*T182-14
     &.0D0*T185+T278-40.0D0*T187-40.0D0*T188-52.0D0*T189+180.0D0*T190+48
     &.0D0*T191+48.0D0*T192+44.0D0*T193+64.0D0*T194-24.0D0*T195-24.0D0*T
     &196+128.0D0*T198+32.0D0*T199+32.0D0*T200+52.0D0*T201+68.0D0*T202+2
     &4.0D0*T203+24.0D0*T204+92.0D0*T205+T279-16.0D0*T112-22.0D0*T139-T1
     &40-16.0D0*T114-T247-26.0D0*T116+T280+T281+T282-8.66666666666666666
     &67D0*T119-28.0D0*T120-29.333333333333333333D0*T121-29.333333333333
     &333333D0*T122-48.0D0*T123-94.0D0*T209-57.333333333333333333D0*T124
     &-57.333333333333333333D0*T125-23.333333333333333333D0*T126
      T594=(T593+11.666666666666666667D0*T127-9.3333333333333333333D0*T1
     &28-9.3333333333333333333D0*T129+46.0D0*T144+65.333333333333333333D
     &0*T130+32.666666666666666667D0*T131+32.666666666666666667D0*T132+7
     &4.0D0*T133+92.0D0*T134+28.0D0*T135+28.0D0*T136+66.0D0*T137-238.5D0
     &*T75-24.666666666666666667D0*T84-24.666666666666666667D0*T85+13.83
     &3333333333333333D0*T76-183.66666666666666667D0*T77-54.166666666666
     &666667D0*T86-54.166666666666666667D0*T87-89.0D0*T78-132.3333333333
     &3333333D0*T79+5.3333333333333333333D0*T88+5.3333333333333333333D0*
     &T89+1.5D0*T80-132.0D0*T81+25.0D0*T90+25.0D0*T91-26.5D0*T82-250.166
     &66666666666667D0*T22+68.0D0*T23+T283+67.333333333333333333D0*T25-3
     &32.33333333333333333D0*T26-107.75D0*T27-107.75D0*T28-182.5D0*T29-1
     &02.02083333333333333D0*T15-13.333333333333333333D0*T19-13.33333333
     &3333333333D0*T17+15.708333333333333333D0*T18+84.125D0*T4+36.145833
     &333333333333D0*T5+33.96875D0)*T3
      T595=12.0D0*T153+T284-12.0D0*T210-12.0D0*T211-12.0D0*T155+T285+T28
     &6+28.0D0*T158+28.0D0*T159+44.0D0*T160+44.0D0*T161+98.0D0*T162-4.0D
     &0*T163+T287+T288-2.0D0*T166-14.0D0*T167-14.0D0*T168-20.0D0*T169+34
     &.0D0*T170+T289+T290-10.0D0*T173+50.0D0*T174+24.0D0*T175+24.0D0*T17
     &6-1.0D0*T177+33.0D0*T178+T291+T292+52.0D0*T181-16.0D0*T182+11.0D0*
     &T185+13.0D0*T186+32.0D0*T187+32.0D0*T188+36.0D0*T189-164.0D0*T190-
     &40.0D0*T191-40.0D0*T192-18.0D0*T193-62.0D0*T194+28.0D0*T195+28.0D0
     &*T196+T293-140.0D0*T198-22.0D0*T199-22.0D0*T200-12.0D0*T201-64.0D0
     &*T202+T305+T306-56.0D0*T205-73.333333333333333333D0*T146+24.0D0*T1
     &11+24.0D0*T112+14.0D0*T139-12.666666666666666667D0*T113+17.0D0*T11
     &4+17.0D0*T141+54.666666666666666667D0*T116-44.333333333333333333D0
     &*T142-3.6666666666666666667D0*T117-3.6666666666666666667D0*T118+19
     &.333333333333333333D0*T119+22.666666666666666667D0*T120+37.3333333
     &33333333333D0*T121+37.333333333333333333D0*T122
      T596=T595+49.0D0*T123+67.0D0*T209+63.333333333333333333D0*T124+63.
     &333333333333333333D0*T125+T542+51.0D0*T127+56.666666666666666667D0
     &*T128+56.666666666666666667D0*T129-3.6666666666666666667D0*T144-30
     &.666666666666666667D0*T130-23.666666666666666667D0*T131-23.6666666
     &66666666667D0*T132-63.333333333333333333D0*T133-101.66666666666666
     &667D0*T134-67.333333333333333333D0*T135-67.333333333333333333D0*T1
     &36-120.66666666666666667D0*T137+183.83333333333333333D0*T75+20.166
     &666666666666667D0*T84+20.166666666666666667D0*T85-89.1944444444444
     &44444D0*T76+109.69444444444444444D0*T77+46.416666666666666667D0*T8
     &6+46.416666666666666667D0*T87+134.33333333333333333D0*T78+81.0D0*T
     &79-9.6666666666666666667D0*T88-9.6666666666666666667D0*T89-9.16666
     &66666666666667D0*T80+224.83333333333333333D0*T81+5.722222222222222
     &2222D0*T90+5.7222222222222222222D0*T91+71.222222222222222222D0*T82
     &+203.25D0*T22-123.75D0*T23-123.75D0*T24-153.36111111111111111D0*T2
     &5+319.72222222222222222D0*T26+210.40277777777777778D0*T27+210.4027
     &7777777777778D0*T28+299.13888888888888889D0*T29
      T597=-2.0D0*T153+T294+T295+T296+4.0D0*T297-8.0D0*T158-6.0D0*T159-1
     &2.0D0*T160-12.0D0*T161-24.0D0*T162+T298+6.0D0*T167+6.0D0*T168+T299
     &-T300-4.0D0*T213-4.0D0*T214+2.0D0*T173-T301-T302-T303+T177-10.0D0*
     &T178-12.0D0*T179-12.0D0*T180-14.0D0*T181+3.0D0*T182+T304-6.0D0*T18
     &6-6.0D0*T187-6.0D0*T188-5.0D0*T189+42.0D0*T190+8.0D0*T191+8.0D0*T1
     &92-2.0D0*T193+T506-8.0D0*T195-8.0D0*T196-6.0D0*T197+39.0D0*T198+T5
     &39+T540-7.0D0*T201+14.0D0*T202+T305+T306+T307-T308-8.0D0*T112-3.5D
     &0*T139+14.666666666666666667D0*T113-4.5D0*T114-4.5D0*T141-18.83333
     &3333333333333D0*T116+25.0D0*T142+T309+T310-T311+6.6666666666666666
     &667D0*T120-11.333333333333333333D0*T121-11.333333333333333333D0*T1
     &22-9.0D0*T123-15.0D0*T209-17.333333333333333333D0*T124-17.33333333
     &3333333333D0*T125+1.6666666666666666667D0*T126-19.0D0*T127-26.0D0*
     &T128-26.0D0*T129-6.6666666666666666667D0*T144
      T598=(T597+9.6666666666666666667D0*T130+3.6666666666666666667D0*T1
     &31+3.6666666666666666667D0*T132+14.166666666666666667D0*T133+32.5D
     &0*T134+26.666666666666666667D0*T135+26.666666666666666667D0*T136+4
     &3.333333333333333333D0*T137-90.833333333333333333D0*T75-3.91666666
     &66666666667D0*T84-3.9166666666666666667D0*T85+48.63888888888888888
     &9D0*T76+9.9722222222222222222D0*T77-9.6666666666666666667D0*T86-9.
     &6666666666666666667D0*T87-33.75D0*T78-5.0833333333333333333D0*T79+
     &T312+3.5D0*T89+16.583333333333333333D0*T80-46.694444444444444444D0
     &*T81-9.1111111111111111111D0*T90-9.1111111111111111111D0*T91+0.722
     &22222222222222222D0*T82-121.84259259259259259D0*T22+44.875D0*T23+4
     &4.875D0*T24+88.333333333333333333D0*T25-75.458333333333333333D0*T2
     &6-78.263888888888888889D0*T27-78.263888888888888889D0*T28-110.0138
     &8888888888889D0*T29-19.230709876543209877D0*T15+57.534722222222222
     &222D0*T19+57.534722222222222222D0*T17+75.210648148148148148D0*T18+
     &34.25D0*T4+0.46141975308641975309D0*T5+191.36822226032616979D0)*T6
     &*CF
      T599=(T583+T584*im*T33+T586*T34+T590*im*pi+(-1.3333333333333333333
     &D0*T75-2.8148148148148148148D0*T22-T263-2.5061728395061728395D0*T1
     &5+2.1481481481481481482D0*T18-1.0987654320987654321D0*T5+2.3845259
     &868922420363D0)*CF*T20+T592+T594+(T596-120.70756172839506173D0*T15
     &-108.40277777777777778D0*T19-108.40277777777777778D0*T17-40.430555
     &555555555556D0*T18-162.93055555555555556D0*T4-84.86033950617283950
     &6D0*T5-104.61728395061728395D0)*CA*T2+T598)*T11
      T600=(-0.09697328777189888301D0*T3+0.17006540270429159318D0*CA*T2-
     &0.068506270821085635901D0*T6*CF)*T218+((0.025925925925925925926D0*
     &T2-0.013888888888888888889D0*CA*CF)*Nf+(T313-0.2243055555555555555
     &6D0*T5-0.27916666666666666667D0)*T3+(T314+0.16944444444444444444D0
     &*T5-0.1974537037037037037D0)*CA*T2+(T315+0.0083333333333333333333D
     &0*T5+0.20416666666666666667D0)*T6*CF)*im*T219+(T316+((T317+T318+0.
     &0077417695473251028806D0)*T2+(T319+T320-0.12165637860082304527D0)*
     &CA*CF)*Nf+(T321+T322+0.67777777777777777778D0*T17-0.56875D0*T18+0.
     &18333333333333333333D0*T4-1.0447916666666666667D0*T5+0.76979166666
     &666666667D0)*T3+(T323+T324-0.81111111111111111111D0*T17+0.27777777
     &777777777778D0*T18-0.81759259259259259259D0*T4+0.51018518518518518
     &518D0*T5-0.73077417695473251029D0)*CA*T2+(T325+T326+0.319444444444
     &44444444D0*T17+0.061111111111111111111D0*T18+0.7439814814814814814
     &8D0*T4+0.010648148148148148148D0*T5+0.66333590534979423868D0)*T6*C
     &F)*T93+(((T327+T222+T330+T328-0.60185185185185185185D0*T4+0.087962
     &962962962962963D0*T5-1.1975308641975308642D0)*T2+(T220+T329+T330-T
     &331+0.43518518518518518518D0*T4+T332+0.64403292181069958848D0)*CA*
     &CF)*Nf+(T333+T334+T335+T336+T337-2.0833333333333333333D0*T27-1.166
     &6666666666666667D0*T28+0.75D0*T29+9.625D0*T15-3.625D0*T19-1.75D0*T
     &17+1.75D0*T18+4.6875D0*T4+0.875D0*T5+14.763888888888888889D0)*T3+(
     &T338+T339+T340+T341-5.5D0*T26+1.9166666666666666667D0*T27+0.25D0*T
     &28-1.0D0*T29-11.111111111111111111D0*T15+3.3472222222222222222D0*T
     &19+1.6111111111111111111D0*T17-3.2916666666666666667D0*T18+3.01157
     &40740740740741D0*T4-0.83101851851851851852D0*T5-7.4251543209876543
     &21D0)*CA*T2+(T342+T343+T344+T345+T346+T347+2.4722222222222222222D0
     &*T15-1.1111111111111111111D0*T19-0.013888888888888888889D0*T17+0.6
     &1111111111111111111D0*T18-3.7615740740740740741D0*T4+0.80092592592
     &592592592D0*T5-0.35159465020576131687D0)*T6*CF)*im*T33
      T601=(T348+0.074074074074074074074D0*T18+T349-0.123456790123456790
     &12D0*T5+0.099794238683127572017D0)*CF*T20+((T350+T226+T351+T227+T3
     &52-T353-0.55555555555555555555D0*T28-0.27777777777777777778D0*T29+
     &T354-2.2037037037037037037D0*T19-3.7037037037037037037D0*T17+0.824
     &07407407407407408D0*T18+1.4907407407407407407D0*T4-1.2129629629629
     &62963D0*T5+1.1168338477366255144D0)*T2+(T108+T355+T356+T357+T358-0
     &.11111111111111111111D0*T27+T359-T383-0.71296296296296296296D0*T15
     &+0.60185185185185185185D0*T19+1.3518518518518518518D0*T17-0.629629
     &62962962962963D0*T18-2.8950617283950617284D0*T4+1.7839506172839506
     &173D0*T5-2.5989083219021490626D0)*CA*CF)*Nf+(T360+T361+T362+T363+T
     &364+T365+T235+T233-1.0D0*T79+T366+T367-2.75D0*T80-2.16666666666666
     &66667D0*T81-3.3333333333333333333D0*T90+2.6666666666666666667D0*T9
     &1-4.25D0*T82-4.5D0*T22-7.5D0*T23-5.25D0*T24-9.125D0*T25-6.5D0*T26-
     &8.5D0*T27-4.25D0*T28-3.375D0*T29-29.895833333333333333D0*T15+11.62
     &5D0*T19-7.125D0*T17+1.7708333333333333333D0*T18-32.055555555555555
     &556D0*T4+8.03125D0*T5-12.834201388888888889D0)*T3
      T602=T601+(T368+T106+T369+T145+T370+T371+T372+T373+0.5D0*T79+1.833
     &3333333333333333D0*T88+2.3333333333333333333D0*T89+3.0833333333333
     &333333D0*T80+1.75D0*T81+5.3333333333333333333D0*T90+T374+T375-1.27
     &77777777777777778D0*T22+10.194444444444444444D0*T23+6.041666666666
     &6666667D0*T24+7.8472222222222222222D0*T25+3.4027777777777777778D0*
     &T26+9.5833333333333333333D0*T27+5.4305555555555555556D0*T28+0.7777
     &7777777777777778D0*T29+19.328703703703703704D0*T15+9.1689814814814
     &814815D0*T19+14.668981481481481481D0*T17-4.587962962962962963D0*T1
     &8+28.314814814814814815D0*T4+2.7476851851851851852D0*T5+2.61744470
     &1646090535D0)*CA*T2+(T376-T109+T377+T378+T228+T379-0.8333333333333
     &3333333D0*T88-1.3333333333333333333D0*T89-0.5D0*T80+T380-T391+T381
     &+T382-2.4722222222222222222D0*T23-3.8611111111111111111D0*T24-1.11
     &11111111111111111D0*T25-1.1388888888888888889D0*T26-0.763888888888
     &88888889D0*T27-2.0277777777777777778D0*T28+1.2222222222222222222D0
     &*T29-5.1458333333333333333D0*T15-4.9490740740740740741D0*T19-3.949
     &0740740740740741D0*T17+T18-3.7874228395061728395D0*T4-4.6604938271
     &604938272D0*T5+6.4452732053040695016D0)*T6*CF
      T603=(-T108+T383+T384+T385+T386+0.30864197530864197531D0*T5-0.1591
     &2208504801097394D0)*CF*T20+((T387-T106+T109+T388+T389+T266+T390+T3
     &91-T374+4.6666666666666666667D0*T82+T392-3.2222222222222222222D0*T
     &23+6.6111111111111111111D0*T24+T393+6.0D0*T26-2.111111111111111111
     &1D0*T27+4.2222222222222222222D0*T28-T394+3.1851851851851851852D0*T
     &15-4.25D0*T19+25.666666666666666667D0*T17-4.3148148148148148148D0*
     &T18+7.611882716049382716D0*T4+1.0547839506172839506D0*T5+5.0902777
     &777777777778D0)*T2+(T84-T109+T395+T241-1.3333333333333333333D0*T88
     &-1.0D0*T80-T391+T91+1.9444444444444444444D0*T22+0.6111111111111111
     &1111D0*T23-3.6111111111111111111D0*T24-1.0D0*T25+T100+1.1111111111
     &111111111D0*T27-2.1111111111111111111D0*T28-2.4444444444444444444D
     &0*T29+8.9351851851851851852D0*T15+T53-9.9166666666666666667D0*T17+
     &5.3518518518518518519D0*T18+18.941358024691358025D0*T4-5.330246913
     &5802469136D0*T5+7.2025034293552812071D0)*CA*CF)*Nf+(T396+T308+T138
     &-T264+T140+T397+T115+T398+T399+T248+T118-T254+T400+T401-T255-T256+
     &8.0D0*T209+10.0D0*T124+14.0D0*T125+T402+2.0D0*T127-1.0D0*T128-T403
     &-3.0D0*T144-30.0D0*T130-T419-10.0D0*T132-T404+T405+T432-8.0D0*T136
     &+15.0D0*T137+7.5D0*T75+10.5D0*T84+3.0D0*T85+3.0D0*T76-24.0D0*T77-6
     &.0D0*T86-30.0D0*T87-10.5D0*T78-111.0D0*T79+T406-52.5D0*T89-4.5D0*T
     &80-49.5D0*T81-T407-51.0D0*T91+T408-53.25D0*T22-11.0D0*T23-45.25D0*
     &T24-4.5D0*T25-76.25D0*T26-28.0D0*T27-108.5D0*T28-3.75D0*T29-70.0D0
     &*T15+53.875D0*T19-81.5D0*T17-12.75D0*T18-19.260416666666666667D0*T
     &4+31.177083333333333333D0*T5+31.916666666666666667D0)*T3
      T604=T603+(T206+T409+T410+T411-T115+T412-T309+T253+T413+T414+T415+
     &T416+T417-12.0D0*T209-13.0D0*T124-13.0D0*T125-9.0D0*T126+T418+T479
     &+2.0D0*T129+27.0D0*T130+T419+T420+T133-9.0D0*T135+6.0D0*T136-43.66
     &6666666666666667D0*T75-4.75D0*T84-12.583333333333333333D0*T85-13.5
     &D0*T76-0.58333333333333333333D0*T77-6.75D0*T86+12.8333333333333333
     &33D0*T87+4.5D0*T78+93.0D0*T79+16.583333333333333333D0*T88+47.25D0*
     &T89+3.1666666666666666667D0*T80+50.25D0*T81+2.4166666666666666667D
     &0*T90+63.5D0*T91-25.666666666666666667D0*T82+19.666666666666666667
     &D0*T22+67.138888888888888889D0*T23+11.430555555555555556D0*T24-7.6
     &25D0*T25+36.125D0*T26+46.819444444444444444D0*T27+81.3611111111111
     &11111D0*T28+5.8333333333333333333D0*T29+42.884259259259259259D0*T1
     &5-37.333333333333333333D0*T19-42.916666666666666667D0*T17+31.59259
     &2592592592593D0*T18-36.110725308641975309D0*T4-65.4726080246913580
     &24D0*T5-39.435185185185185185D0)*CA*T2+(T111+T138+T139+T140+T421+T
     &422+T423+T142+T424+T425+T426+T427+T428+T429+4.0D0*T209+5.0D0*T124+
     &3.0D0*T125+3.0D0*T126+T127+3.0D0*T128+T459-6.0D0*T130-3.0D0*T131+T
     &430-T404+T431+T432-1.0D0*T136+6.0D0*T75+1.25D0*T84+9.5833333333333
     &333333D0*T85+4.5D0*T76+T433+T434+6.5833333333333333333D0*T87-2.25D
     &0*T78-22.5D0*T79-3.1666666666666666667D0*T88-9.75D0*T89-T481-13.5D
     &0*T81+3.5833333333333333333D0*T90-18.25D0*T91-22.47222222222222222
     &2D0*T22-26.444444444444444444D0*T23+8.0694444444444444444D0*T24+3.
     &875D0*T25-2.75D0*T26-17.069444444444444444D0*T27-13.18055555555555
     &5556D0*T28+6.7222222222222222222D0*T29-39.439814814814814815D0*T15
     &+0.97916666666666666667D0*T19+27.604166666666666667D0*T17-17.48148
     &1481481481481D0*T18-44.111882716049382716D0*T4+32.6327160493827160
     &49D0*T5-20.090449245541838134D0)*T6*CF
      T605=(T437+T251-T147+T269+T438+T271+T272+T273+T439+0.6666666666666
     &6666667D0*T126-1.3333333333333333333D0*T127+T440+T441+T442-T443+T4
     &44+2.0D0*T134+6.6666666666666666667D0*T137+T445-3.2222222222222222
     &222D0*T76-10.611111111111111111D0*T77-T446-T447+T233-12.0D0*T79+T4
     &48+T449-T465-4.2222222222222222222D0*T81+3.6666666666666666667D0*T
     &82+T450+T451+2.1666666666666666667D0*T24-2.0833333333333333333D0*T
     &25-25.666666666666666667D0*T26+T452-6.2785493827160493827D0*T15+2.
     &0D0*T19+T453-2.2229938271604938272D0*T18-2.9444444444444444444D0*T
     &4+3.4988425925925925926D0*T5+8.0043724279835390946D0)*T2+(T139+T14
     &7+T454+T142+T281+T282+T455+T265-1.3333333333333333333D0*T126+T456+
     &T457+T458+T459+T460-T444+T431-6.8333333333333333333D0*T75+0.611111
     &11111111111111D0*T76+5.6111111111111111111D0*T77+T461+T462+4.5D0*T
     &79+T463+T464+T465+2.1111111111111111111D0*T81-7.333333333333333333
     &3D0*T82-19.37037037037037037D0*T22+T339-1.0833333333333333333D0*T2
     &4+1.9166666666666666667D0*T25+9.9166666666666666667D0*T26+7.037037
     &037037037037D0*T29-19.941358024691358025D0*T15-1.0D0*T19+T543+4.25
     &30864197530864197D0*T18-1.8333333333333333333D0*T4-16.851851851851
     &851852D0*T5+24.101489864349946654D0)*CA*CF
      T606=T466+T467+T154-T296+T285+T286+T468+T469+T470+T471+T472+T473+T
     &287+T288+T474+T475+T476+T212+T300+T171+T172+T477+T301+T302+T303+8.
     &0D0*T501+8.0D0*T502+18.0D0*T177-6.0D0*T178+T503+T504+10.0D0*T181+T
     &533-2.0D0*T183-2.0D0*T184-3.0D0*T185-4.0D0*T187-4.0D0*T188-7.0D0*T
     &189+50.0D0*T190+20.0D0*T191+20.0D0*T192+12.0D0*T193+12.0D0*T194+2.
     &0D0*T195+2.0D0*T196-4.0D0*T198+T507+8.0D0*T202+15.0D0*T205+T478+12
     &.0D0*T112+22.5D0*T139-18.0D0*T113-T510-T511-9.0D0*T116+18.0D0*T142
     &-4.5D0*T117-4.5D0*T118-10.5D0*T119+T414-21.0D0*T121-21.0D0*T122-31
     &.5D0*T123+168.0D0*T209+57.0D0*T124+57.0D0*T125+52.5D0*T126+46.5D0*
     &T127+T479-6.0D0*T129-9.0D0*T144+60.0D0*T130+13.5D0*T131+T480+4.5D0
     &*T133+51.0D0*T134+22.5D0*T137+12.75D0*T75-40.5D0*T84-40.5D0*T85-51
     &.5D0*T76+49.0D0*T77+3.75D0*T86+3.75D0*T87+1.5D0*T78+87.5D0*T79+29.
     &25D0*T88+29.25D0*T89+T481
      T607=T482-T154+T294+T295+T483+T484+T485+T486+T487+T488+T489+T490+T
     &298+T491+T492+T166+T493+T494+T495+T496-T171-T172+T497+T498+T499+T5
     &00-12.0D0*T501-12.0D0*T502-25.0D0*T177+5.0D0*T178-T503-T504-17.0D0
     &*T181+T505+7.0D0*T183+7.0D0*T184+T185+2.0D0*T187+2.0D0*T188+2.0D0*
     &T189-49.0D0*T190-22.0D0*T191-22.0D0*T192-14.0D0*T193-T506-1.0D0*T1
     &95-1.0D0*T196+6.0D0*T198+6.0D0*T199+6.0D0*T200-T507-6.0D0*T202+T50
     &8-18.0D0*T111+T509-22.75D0*T139+27.583333333333333333D0*T113+T510+
     &T511-7.3333333333333333333D0*T116+14.166666666666666667D0*T142+8.4
     &166666666666666667D0*T117+8.4166666666666666667D0*T118+1.666666666
     &6666666667D0*T119+T512+13.5D0*T121+13.5D0*T122+18.0D0*T123-148.5D0
     &*T209-55.5D0*T124-55.5D0*T125-38.916666666666666667D0*T126-35.4166
     &66666666666667D0*T127+11.833333333333333333D0*T128+11.833333333333
     &333333D0*T129+9.8333333333333333333D0*T144-55.666666666666666667D0
     &*T130-14.25D0*T131-14.25D0*T132
      T608=(T607-11.833333333333333333D0*T133-63.5D0*T134-36.66666666666
     &6666667D0*T137+7.3333333333333333333D0*T75+32.5D0*T84+32.5D0*T85+9
     &9.638888888888888889D0*T76+13.694444444444444444D0*T77+25.125D0*T8
     &6+25.125D0*T87+9.75D0*T78-31.25D0*T79-45.875D0*T88-45.875D0*T89+0.
     &94444444444444444445D0*T80-81.361111111111111111D0*T81-16.16666666
     &6666666667D0*T82-27.416666666666666667D0*T22+38.625D0*T23+38.625D0
     &*T24+1.2916666666666666667D0*T25+39.416666666666666667D0*T26-3.5D0
     &*T27-3.5D0*T28+38.583333333333333333D0*T29+69.860725308641975309D0
     &*T15+30.083333333333333333D0*T19+30.083333333333333333D0*T17-0.153
     &16358024691358025D0*T18+52.527777777777777778D0*T4-44.313078703703
     &703704D0*T5+50.81558641975308642D0)*CA*T2
      T609=T153+T513+T514+T515+T516+T517+T518+T519+T520+T521+T522+T523+T
     &524+T212+T525+T526+T527+T528+T529+T530+T531+4.0D0*T501+4.0D0*T502+
     &9.0D0*T177+T532+2.0D0*T179+2.0D0*T180+5.0D0*T181+T533+T534+T535-1.
     &0D0*T189+T536+T537+T538+3.0D0*T193+4.0D0*T194-2.0D0*T197-2.0D0*T19
     &8-T539-T540+T202+T541+T246+7.25D0*T139-13.333333333333333333D0*T11
     &3-3.75D0*T114-3.75D0*T141+4.4166666666666666667D0*T116-12.25D0*T14
     &2-3.0833333333333333333D0*T117-3.0833333333333333333D0*T118+1.4166
     &666666666666667D0*T119-8.0833333333333333333D0*T120-1.5D0*T121-1.5
     &D0*T122-3.75D0*T123+36.0D0*T209+13.5D0*T124+13.5D0*T125+T542+5.333
     &3333333333333333D0*T127-4.4166666666666666667D0*T128-4.41666666666
     &66666667D0*T129-2.0D0*T144+13.583333333333333333D0*T130+3.75D0*T13
     &1+3.75D0*T132+7.3333333333333333333D0*T133+18.25D0*T134+29.7916666
     &66666666667D0*T75-6.125D0*T84-6.125D0*T85-32.569444444444444444D0*
     &T76-21.569444444444444444D0*T77-13.5D0*T86-13.5D0*T87
      T610=(T609-4.125D0*T78+1.875D0*T79+15.625D0*T88+15.625D0*T89-1.444
     &4444444444444444D0*T80+13.180555555555555556D0*T81+20.166666666666
     &666667D0*T82+56.046296296296296296D0*T22-11.791666666666666667D0*T
     &23-11.791666666666666667D0*T24-10.8125D0*T25-26.604166666666666667
     &D0*T26+T27+T28-22.87962962962962963D0*T29+35.403549382716049383D0*
     &T15-8.7083333333333333333D0*T19-8.7083333333333333333D0*T17-6.4506
     &17283950617284D0*T18-5.375D0*T4+51.842592592592592593D0*T5-95.6841
     &11130163084895D0)*T6*CF
      T611=(T600+T602*T34+T604*im*pi+(T435+0.66666666666666666667D0*T82+
     &T436-0.40740740740740740741D0*T29+T151-0.52469135802469135802D0*T1
     &8+1.1759259259259259259D0*T5-1.1922629934461210181D0)*CF*T20+T605*
     &Nf+(T606+108.5D0*T81+3.0D0*T82+42.916666666666666667D0*T22-30.0833
     &33333333333333D0*T23-30.083333333333333333D0*T24+23.79166666666666
     &6667D0*T25+84.5D0*T26+3.0D0*T27+3.0D0*T28-11.25D0*T29-7.5729166666
     &666666667D0*T15-25.333333333333333333D0*T19-25.333333333333333333D
     &0*T17-0.90625D0*T18-31.916666666666666667D0*T4+9.265625D0*T5-16.35
     &9375D0)*T3+T608+T610)*T12
      T612=(-1.0D0*T4*T3-1.9583333333333333333D0*T4*CA*T2+1.333333333333
     &3333333D0*T4*T6*CF)*im*T33+((-0.55555555555555555555D0*T4*T2+0.333
     &33333333333333333D0*T4*CA*CF)*Nf+(8.8333333333333333333D0*T15-5.0D
     &0*T19+T543+T4)*T3+(-7.875D0*T15+3.6666666666666666667D0*T19+0.75D0
     &*T17-1.7638888888888888889D0*T4)*CA*T2+(2.0833333333333333333D0*T1
     &5-0.58333333333333333333D0*T19+0.43055555555555555555D0*T4)*T6*CF)
     &*T34+(((T544+T545+3.8888888888888888889D0*T4)*T2+(T52+T543-4.58333
     &33333333333333D0*T4)*CA*CF)*Nf+(-5.0D0*T22+T70-T346+T546-19.166666
     &666666666667D0*T15-16.333333333333333333D0*T19-7.33333333333333333
     &33D0*T17-29.0D0*T4)*T3+(T547-T548+T549-T393+1.5D0*T26-1.5D0*T28+17
     &.416666666666666667D0*T15+26.583333333333333333D0*T19+4.0D0*T17+15
     &.194444444444444444D0*T4)*CA*T2+(T22+6.5D0*T23+T24+T25-0.333333333
     &33333333333D0*T15-9.2083333333333333333D0*T19+2.375D0*T17+10.27083
     &3333333333333D0*T4)*T6*CF)*im*pi+((-2.6666666666666666667D0*T22-T5
     &50-3.8888888888888888889D0*T15)*T2+(T551+T26+4.5833333333333333333
     &D0*T15)*CA*CF)*Nf
      T613=(T612+(34.0D0*T75+29.0D0*T84+29.0D0*T85+28.0D0*T76+2.0D0*T79-
     &2.0D0*T81+32.5D0*T22+16.333333333333333333D0*T23+16.33333333333333
     &3333D0*T24+T552+29.0D0*T15)*T3+(-29.5D0*T75-26.5D0*T84-26.5D0*T85-
     &39.0D0*T76+4.5D0*T77+T461+T462-1.5D0*T79+1.5D0*T81-34.416666666666
     &666667D0*T22-26.583333333333333333D0*T23-26.583333333333333333D0*T
     &24-T553-15.194444444444444444D0*T15)*CA*T2+(5.0D0*T75+T554+6.0D0*T
     &85+12.5D0*T76-T555-1.0D0*T86-1.0D0*T87+4.0416666666666666667D0*T22
     &+9.2083333333333333333D0*T23+9.2083333333333333333D0*T24-2.375D0*T
     &26-10.270833333333333333D0*T15)*T6*CF)*x
      gammaHPL_3l_no_nfv=(T556+(((-5.0D0*CA*CF*Nf-107.0D0*T3+127.0D0*CA*T2-20.25D0*T6*C
     &F)*im*pi+(-2.0D0*T2+(-2.5D0*T4-2.5D0*T5+1.0D0)*CA*CF)*Nf+(-58.5D0*
     &T4-46.5D0*T5-10.0D0)*T3+(68.0D0*T4+T13+32.916666666666666667D0)*CA
     &*T2+(-11.125D0*T4-11.125D0*T5-13.958333333333333333D0)*T6*CF)*T92+
     &((16.0D0*CA*CF*Nf+298.0D0*T3-354.0D0*CA*T2+47.5D0*T6*CF)*im*pi+(8.
     &0D0*T2+(8.0D0*T4+8.0D0*T5-1.5D0)*CA*CF)*Nf+(172.0D0*T4+128.0D0*T5+
     &116.33333333333333333D0)*T3+(-201.0D0*T4-179.0D0*T5-202.75D0)*CA*T
     &2+(30.0D0*T4+30.0D0*T5+64.041666666666666667D0)*T6*CF)*T103+T558+T
     &560+T562+(30.0D0*T4*T3-23.5D0*T4*CA*T2+3.5D0*T4*T6*CF)*x)*zeta3+T5
     &65*T92+T569+T582+T599+T611+T613+(-1.0D0*T15*T3+T15*CA*T2-0.25D0*T1
     &5*T6*CF)*T34+(-3.0D0*T22*T3+3.0D0*T22*CA*T2-0.75D0*T22*T6*CF)*im*p
     &i+3.0D0*T75*T3-3.0D0*T75*CA*T2+0.75D0*T75*T6*CF)/(T10-2.0D0*T11+T1
     &2)

      end function gammaHPL_3l_no_nfv

      end module
