!
!  SPDX-License-Identifier: GPL-3.0-or-later
!  Copyright (C) 2019-2022, respective authors of MCFM.
!
      module nnlo_z1jet_hfun
          use types
          use constants
          implicit none

          public
      
          contains

! Taken from PeTeR helicity_amplitudes_V.cpp, written by Christian Lorentzen
! Thomas Becher, Christian Lorentzen, Matthew D. Schwartz, arXiv:1106.4310
      
!**********************************************************************
!region2a:q+qbar-->V+g*
!*********************************************************************/
      
      function Alpha_2a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=1._dp
      return
      end
      
      
      function Alpha_2a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0._dp
      return
      end
      
      
      function Alpha_2a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-4*(7*omu+v))/(3._dp*omu)+(27*zeta2)/2._dp-3*G2(0,2)-3*G2(2,0)+
     & (8*G2(3,0))/3._dp-(8*G1(3)*H1(0))/3._dp+G1(0)*((17*H1(0))/3._dp+3*H1(1))-
     & (8*H2(0,0))/3._dp-(10*H2(1,0))/3._dp+
     & G1(1)*(H1(0)*(-3+(omv*omv)/(3._dp*(u*u)))+
     & H1(1)*(-3+(omv*omv)/(3._dp*(u*u))))+
     & G2(1,2)*(3-(omv*omv)/(3._dp*(u*u)))+
     & (H1(1)*(omv*omv+u*(u*(27-10*v)-13*(u*u)+3*(-5+4*v+v*v))))/
     & (3._dp*u*(omu*omu))+H1(0)*(-3*H1(1)+
     & (-(omv*v)+(24-9*v)*(u*u)-12*(u*u*u)+u*(-12+10*v+3*(v*v)))/
     & (3._dp*u*(omu*omu)))+G1(2)*
     & (3*H1(1)+H1(0)*(6-(omv*omv)/(3._dp*(u*u)))+
     & (-(omv*omv)+u*(u*(-27+10*v)+13*(u*u)-3*(-5+4*v+v*v)))/
     & (3._dp*u*(omu*omu)))-(3*(G1(0)*G1(0)))/2._dp-(3*(G1(2)*G1(2)))/2._dp-
     & (17*(H1(0)*H1(0)))/6._dp-(3*(H1(1)*H1(1)))/2._dp
      return
      end
      
      
      function Alpha_2a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=-3*pi*G1(0)-3*pi*G1(2)+(8*pi*G1(3))/3._dp+3*pi*H1(0)+3*pi*H1(1)+
     & pi*G1(1)*(3-(omv*omv)/(3._dp*(u*u)))+
     & (pi*(-(omv*omv)+u*(u*(-3+10*v)+u*u-3*(-1+4*v+v*v))))/
     & (3._dp*u*(omu*omu))
      return
      end
      
      
      function Alpha_2a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(5*(8+6*zeta2))/(18._dp*omw)-(16*G4(0,0,1,2))/9._dp+18*G4(0,0,2,2)-
     & 16*G4(0,0,3,0)+18*G4(0,2,0,2)-(92*G4(0,2,1,2))/9._dp+18*G4(0,2,2,0)-
     & 28*G4(0,2,3,0)-8*G4(0,3,0,2)-8*G4(0,3,2,0)+(128*G4(0,3,3,0))/9._dp+
     & 18*G4(2,0,0,2)+18*G4(2,0,2,0)+18*G4(2,2,0,0)-16*G4(2,3,0,0)+
     & 16*G4(2,3,0,2)+16*G4(2,3,2,0)-16*G4(3,0,0,2)+10*G4(3,0,1,2)-
     & 16*G4(3,0,2,0)+(208*G4(3,0,3,0))/9._dp-16*G4(3,2,0,0)+10*G4(3,2,1,2)+
     & (80*G4(3,2,3,0))/9._dp+(128*G4(3,3,0,0))/9._dp-(128*G4(3,3,3,0))/9._dp+
     & 16*G3(0,0,3)*H1(0)+28*G3(0,2,3)*H1(0)+8*G3(0,3,2)*H1(0)-
     & (128*G3(0,3,3)*H1(0))/9._dp-16*G3(2,3,2)*H1(0)-(208*G3(3,0,3)*H1(0))/9._dp-
     & (80*G3(3,2,3)*H1(0))/9._dp+(128*G3(3,3,3)*H1(0))/9._dp+
     & G3(0,0,2)*(63-3*nf-(160*H1(0))/9._dp-18*H1(1))+
     & G3(2,0,0)*(63-3*nf-16*H1(0)-18*H1(1))+G3(3,0,1)*(-10*H1(0)-10*H1(1))+
     & G3(3,2,1)*(-10*H1(0)-10*H1(1))+G3(0,0,1)*((16*H1(0))/9._dp+(16*H1(1))/9._dp)+
     & G3(0,2,1)*((92*H1(0))/9._dp+(92*H1(1))/9._dp)+
     & G3(3,0,0)*((8*(-21+nf))/3._dp+(128*H1(0))/9._dp+16*H1(1))+
     & (64*H4(0,0,0,0))/3._dp+(80*H4(0,0,1,0))/9._dp+(80*H4(0,1,0,0))/9._dp-
     & 10*H4(0,1,0,1)+(10*H4(0,1,1,0))/9._dp+10*H4(1,0,0,0)-(160*H4(1,0,0,1))/9._dp+
     & (110*H4(1,0,1,0))/9._dp-(160*H4(1,1,0,0))/9._dp-40*H4(1,1,0,1)+
     & G2(0,0)*(80-(41*nf)/6._dp-54*zeta2+(8*(-21+nf)*H1(0))/3._dp+
     & 3*(-21+nf)*H1(1)+(128*H2(0,0))/9._dp+16*H2(0,1)+16*H2(1,0)+
     & 18*H2(1,1)+(5*(nf*nf))/36._dp)+
     & G3(0,2,0)*(63-3*nf-18*H1(1)+H1(0)*(-18-(2*(omv*omv))/(u*u)))+
     & G3(2,1,0)*(H1(0)*(9-(omv*omv)/(u*u))+H1(1)*(9-(omv*omv)/(u*u)))+
     & G3(0,1,1)*(H1(0)*(169._dp/9._dp-(omv*omv)/(u*u))+
     & H1(1)*(169._dp/9._dp-(omv*omv)/(u*u)))+
     & G3(1,0,1)*(H1(0)*(-10+(10*(omv*omv))/(9._dp*(u*u)))+
     & H1(1)*(-10+(10*(omv*omv))/(9._dp*(u*u))))+
     & G3(2,1,1)*(H1(0)*(-16+(16*(omv*omv))/(9._dp*(u*u)))+
     & H1(1)*(-16+(16*(omv*omv))/(9._dp*(u*u))))+
     & G3(0,1,0)*(H1(0)*(-18+(2*(omv*omv))/(u*u))+
     & H1(1)*(-18+(2*(omv*omv))/(u*u)))+
     & G3(2,3,3)*H1(0)*(20-(20*(omv*omv))/(9._dp*(u*u)))+
     & G4(0,1,0,2)*(18-(2*(omv*omv))/(u*u))+
     & G4(0,1,2,0)*(18-(2*(omv*omv))/(u*u))+
     & G4(2,1,1,2)*(16-(16*(omv*omv))/(9._dp*(u*u)))+
     & G4(1,0,1,2)*(10-(10*(omv*omv))/(9._dp*(u*u)))+
     & G4(0,1,1,2)*(-169._dp/9._dp+(omv*omv)/(u*u))+
     & G4(0,1,2,2)*(-17+(omv*omv)/(u*u))+
     & G4(2,1,0,2)*(-9+(omv*omv)/(u*u))+
     & G4(2,1,2,0)*(-9+(omv*omv)/(u*u))+
     & G4(2,3,3,0)*(-20+(20*(omv*omv))/(9._dp*(u*u)))+
     & G3(1,2,1)*((H1(0)*(6*omv*u+30*(omv*omv)-244*(u*u)))/(9._dp*(u*u))+
     & (H1(1)*(6*omv*u+30*(omv*omv)-244*(u*u)))/(9._dp*(u*u)))+
     & G3(2,2,1)*((-2*H1(0)*(3*omv*u+11*(omv*omv)-86*(u*u)))/(9._dp*(u*u))-
     & (2*H1(1)*(3*omv*u+11*(omv*omv)-86*(u*u)))/(9._dp*(u*u)))+
     & G3(1,1,0)*((-2*H1(0)*(3*omv*u+2*(omv*omv)-5*(u*u)))/(9._dp*(u*u))-
     & (2*H1(1)*(3*omv*u+2*(omv*omv)-5*(u*u)))/(9._dp*(u*u)))-
     & (G4(1,2,1,2)*(6*omv*u+30*(omv*omv)-244*(u*u)))/(9._dp*(u*u))+
     & (2*G4(2,2,1,2)*(3*omv*u+11*(omv*omv)-86*(u*u)))/(9._dp*(u*u))+
     & (2*G4(1,0,2,2)*(3*omv*u+2*(omv*omv)-77*(u*u)))/(9._dp*(u*u))+
     & (2*G4(1,2,0,2)*(3*omv*u+2*(omv*omv)-77*(u*u)))/(9._dp*(u*u))+
     & (2*G4(1,2,2,0)*(3*omv*u+2*(omv*omv)-77*(u*u)))/(9._dp*(u*u))-
     & (2*G4(1,1,2,2)*(3*omv*u+10*(omv*omv)-77*(u*u)))/(9._dp*(u*u))+
     & (2*G4(1,1,1,2)*(-3*omv*u+6*(omv*omv)-67*(u*u)))/(9._dp*(u*u))+
     & (2*G4(1,1,0,2)*(3*omv*u+2*(omv*omv)-5*(u*u)))/(9._dp*(u*u))+
     & (2*G4(1,1,2,0)*(3*omv*u+2*(omv*omv)-5*(u*u)))/(9._dp*(u*u))-
     & (G4(2,0,1,2)*(6*omv*u+4*(omv*omv)+242*(u*u)))/(9._dp*(u*u))+
     & (G3(2,0,3)*H1(0)*(6*omv*(-1+15*NFV)*u+(17-60*NFV)*(omv*omv)+
     & 3*(27-10*NFV)*(u*u)))/(9._dp*(u*u))-
     & (G4(1,0,3,0)*(omv*(2-30*NFV)*u+(1+20*NFV)*(omv*omv)+
     & (-33+10*NFV)*(u*u)))/(3._dp*(u*u))-
     & (G4(1,2,3,0)*(omv*(2-30*NFV)*u+(1+20*NFV)*(omv*omv)+
     & (-33+10*NFV)*(u*u)))/(3._dp*(u*u))+
     & (G3(1,0,3)*H1(0)*(omv*(2-30*NFV)*u+(1+20*NFV)*(omv*omv)+
     & (-33+10*NFV)*(u*u)))/(3._dp*(u*u))+
     & (G3(1,2,3)*H1(0)*(omv*(2-30*NFV)*u+(1+20*NFV)*(omv*omv)+
     & (-33+10*NFV)*(u*u)))/(3._dp*(u*u))+
     & (G4(2,0,3,0)*(omv*(6-90*NFV)*u+(-17+60*NFV)*(omv*omv)+
     & 3*(-27+10*NFV)*(u*u)))/(9._dp*(u*u))+
     & (G4(2,2,3,0)*(6*omv*(-1+15*NFV)*u+(17-60*NFV)*(omv*omv)-
     & 3*(9+10*NFV)*(u*u)))/(9._dp*(u*u))+
     & (G3(2,2,3)*H1(0)*(omv*(6-90*NFV)*u+(-17+60*NFV)*(omv*omv)+
     & 3*(9+10*NFV)*(u*u)))/(9._dp*(u*u))+
     & G3(1,1,1)*((2*H1(0)*(3*omv*u-6*(omv*omv)+67*(u*u)))/(9._dp*(u*u))+
     & (2*H1(1)*(3*omv*u-6*(omv*omv)+67*(u*u)))/(9._dp*(u*u)))+
     & G3(2,0,1)*((H1(0)*(6*omv*u+4*(omv*omv)+242*(u*u)))/(9._dp*(u*u))+
     & (H1(1)*(6*omv*u+4*(omv*omv)+242*(u*u)))/(9._dp*(u*u)))+
     & (H3(1,0,1)*(-600-40/omw+46*nf-22*v+(144*v)/omu+10/(omw*omw)-
     & 9*(v*v)+(18*(v*v))/omu+(72*(v*v))/(omu*omu)+
     & 10*NFV*u*(2-v+2*(v*v))+(4*(3-8*v+5*(v*v)))/u))/18._dp+
     & (H2(1,1)*((92-12*nf)*(omv*omv)+
     & u*(5*(omu*omu)*(nf*nf)-
     & 4*(-35+139*u-428*v+382*u*v+126*zeta2*((-1+u)*(-1+u))-
     & 81*(u*u)-257*(v*v))-
     & 6*nf*(11+24*v-4*u*(7+5*v)+15*(u*u)+6*(v*v)))))/
     & (36._dp*u*(omu*omu))+G2(1,0)*
     & ((2*H2(1,0)*(-15*omv*NFV*u+u*(1+(-27+5*NFV)*u-v)+
     & (-1+10*NFV)*(omv*omv)))/(3._dp*(u*u))+
     & (2*H2(1,1)*(3*omv*u+2*(omv*omv)-77*(u*u)))/(9._dp*(u*u))-
     & (H2(0,1)*(-6*omv*u+5*(omv*omv)+73*(u*u)))/(9._dp*(u*u))+
     & (H2(0,0)*(omv*(2-30*NFV)*u+5*(-1+4*NFV)*(omv*omv)+
     & (-27+10*NFV)*(u*u)))/(3._dp*(u*u))+
     & (2*zeta2*(-9*omv*(u+10*NFV*u)+(-7+60*NFV)*(omv*omv)+
     & 2*(71+15*NFV)*(u*u)))/(9._dp*(u*u))+
     & (H1(0)*(-63+106*v+omu*
     & (-60*omv*NFV*u+(nf+20*NFV)*(omv*omv)+
     & u*u*(-9*nf+10*NFV*(6+u*(-2+v-2*(v*v)))))+
     & u*u*(289-174*v-75*(v*v))-42*(v*v)+
     & u*(55-70*v+32*(v*v))+u*u*u*(-281+138*v+71*(v*v))))/
     & (18._dp*omu*(u*u))+(H1(1)*
     & (-63+106*v+omu*(-60*omv*NFV*u+(nf+20*NFV)*(omv*omv)+
     & u*u*(-9*nf+10*NFV*(6+u*(-2+v-2*(v*v)))))+
     & u*u*(289-174*v-75*(v*v))-42*(v*v)+
     & u*(55-70*v+32*(v*v))+u*u*u*(-281+138*v+71*(v*v))))/
     & (18._dp*omu*(u*u)))+(H3(0,0,0)*
     & (2*omv*(1+60*NFV)*v+8*(u*u*u*u)*(231-14*nf+20*v+10*(v*v))+
     & u*u*(5544-336*nf+(794+570*NFV)*v+(313-270*NFV)*(v*v))+
     & u*(-1848+112*nf-84*(4+5*NFV)*v+(-113+300*NFV)*(v*v))+
     & u*u*u*(336*nf-2*(2772+5*(62+27*NFV)*v+123*(v*v)))))/
     & (18._dp*u*(omu*omu*omu))+G3(0,1,2)*
     & (-2*H1(0)+H1(1)*(17-(omv*omv)/(u*u))+
     & (nf*(45-(omv*omv)/(u*u))+
     & 2*(-457+20/omw+(82-80/omu)*v-5/(omw*omw)-
     & (8*(omv*omv))/(u*u)+(6+5*v-11*(v*v))/u+
     & (10*(-1-7*u+4*(u*u))*(v*v))/(omu*omu)))/18._dp)+
     & G3(2,0,2)*(H1(0)*(-19-(omv*omv)/(u*u))+
     & (4*(omv*omv)+u*(324+168*v-27*nf*((-1+u)*(-1+u))-
     & 2*(-166+v)*(u*u)+75*(v*v)-2*u*(330+79*v+v*v)))/
     & (9._dp*u*(omu*omu)))+G3(0,2,2)*
     & (H1(0)*(-17+(omv*omv)/(u*u))+
     & (4*(omv*omv)+u*(324+168*v-27*nf*((-1+u)*(-1+u))-
     & 2*(-166+v)*(u*u)+75*(v*v)-2*u*(330+79*v+v*v)))/
     & (9._dp*u*(omu*omu)))+G3(2,2,0)*
     & ((H1(0)*(90*omv*NFV*u+(21-60*NFV)*(omv*omv)-
     & (181+30*NFV)*(u*u)))/(9._dp*(u*u))+
     & (4*(omv*omv)+u*(324+168*v-27*nf*((-1+u)*(-1+u))-
     & 2*(-166+v)*(u*u)+75*(v*v)-2*u*(330+79*v+v*v)))/
     & (9._dp*u*(omu*omu)))+G3(1,2,2)*
     & ((-2*H1(0)*(3*omv*u+10*(omv*omv)-77*(u*u)))/(9._dp*(u*u))+
     & (-3*(-6+nf*(omu*omu))*(omv*omv)+
     & u*(-21+78*v+(-345+2*v)*(u*u*u)+
     & 3*u*(-119-58*v+9*nf*(omu*omu)-9*(v*v))-57*(v*v)+
     & u*u*(705+130*v+2*(v*v))))/(9._dp*(omu*omu)*(u*u)))+
     & G3(2,3,0)*(-2*H1(0)-16*H1(1)-
     & (-28+22*v+(-622+44*nf+4*v)*(u*u*u*u)+
     & u*u*(-1950+132*nf-262*v-73*(v*v))+6*(v*v)+
     & 2*(u*u*u)*(947-66*nf+60*v+2*(v*v))+
     & u*(706-44*nf+116*v+31*(v*v))+
     & 30*NFV*((-32+17*v)*(u*u*u)+8*(u*u*u*u)+4*(2-3*v+v*v)-
     & 2*u*(16-19*v+5*(v*v))+u*u*(48-43*v+9*(v*v))))/
     & (18._dp*u*(omu*omu*omu)))+
     & G2(2,3)*(20*H2(0,0)+16*H2(0,1)+16*H2(1,0)-
     & (4*zeta2*(5*(omv*omv)+63*(u*u)))/(9._dp*(u*u))+
     & (H1(0)*(-28+22*v+(-622+44*nf+4*v)*(u*u*u*u)+
     & u*u*(-1950+132*nf-262*v-73*(v*v))+6*(v*v)+
     & 2*(u*u*u)*(947-66*nf+60*v+2*(v*v))+
     & u*(706-44*nf+116*v+31*(v*v))+
     & 30*NFV*((-32+17*v)*(u*u*u)+8*(u*u*u*u)+4*(2-3*v+v*v)-
     & 2*u*(16-19*v+5*(v*v))+u*u*(48-43*v+9*(v*v)))))/
     & (18._dp*u*(omu*omu*omu)))+
     & G2(0,1)*((20*H2(0,0))/9._dp+2*H2(1,0)+H2(1,1)*(-17+(omv*omv)/(u*u))+
     & H2(0,1)*(-151._dp/9._dp+(omv*omv)/(u*u))+
     & (7*zeta2*(9*(omv*omv)-25*(u*u)))/(9._dp*(u*u))+
     & (H1(0)*(nf*(-45+(omv*omv)/(u*u))+
     & 2*(457-20/omw+(2*(-1+41*u)*v)/omu+5/(omw*omw)+
     & (8*(omv*omv))/(u*u)-
     & (10*(-1-7*u+4*(u*u))*(v*v))/(omu*omu)+
     & (-6-5*v+11*(v*v))/u)))/18._dp+
     & (H1(1)*(nf*(-45+(omv*omv)/(u*u))+
     & 2*(457-20/omw+(2*(-1+41*u)*v)/omu+5/(omw*omw)+
     & (8*(omv*omv))/(u*u)-
     & (10*(-1-7*u+4*(u*u))*(v*v))/(omu*omu)+
     & (-6-5*v+11*(v*v))/u)))/18._dp)+
     & (H2(0,1)*(8160*omu*omw*u*v+
     & 2*(omu*omu)*(u*(-30+
     & omw*(594+90*NFV*v-120*NFV*u*v-1452*zeta2+9*(nf*nf)))+
     & 3*omv*omw*(109-5*(21+4*NFV)*v+20*NFV*(v*v)))-
     & 3*omw*(2*u*(-893+185*u+30*NFV*(1-3*u+2*(u*u)))*(v*v)+
     & nf*(12-34*v-6*(39+35*v)*(u*u)+123*(u*u*u)+22*(v*v)+
     & u*(99+244*v+66*(v*v))))))/(108._dp*omw*u*(omu*omu))+
     & (H3(0,1,0)*(omu*omv*u*(32-73*v)*v-
     & 2*omv*u*v*(67+27*v)*(omu*omu)+
     & 2*omv*u*v*(-16*v+15*NFV*
     & (2*(1+v)-u*(7+3*v)+(5+4*v)*(u*u)))-
     & omu*omu*omu*(120*NFV*v*(omv*omv)+40*NFV*(u*u)+
     & omv*(46*(-1+v)*v-10*NFV*(u*u)*(2-v+2*(v*v))+
     & u*(1014-68*nf+182*v+89*(v*v))))))/
     & (18._dp*omv*u*(omu*omu*omu))+
     & G3(1,1,2)*(H1(0)*(-16+(16*(omv*omv))/(9._dp*(u*u)))+
     & (2*H1(1)*(3*omv*u+10*(omv*omv)-77*(u*u)))/(9._dp*(u*u))-
     & (-14+8*v+4*nf*(omu*omu)*(omv*omv-9*(u*u))+
     & u*(16+38*v-38*(v*v))+7*(v*v)+6*(u*u)*(98-9*v+10*(v*v))+
     & u*u*u*u*(578+302*v+151*(v*v))-
     & 2*(u*u*u)*(584+147*v+154*(v*v))-
     & 10*NFV*(omu*omu*omu)*
     & (2*(omv*omv)+u*(-1-v+2*(v*v)+u*(2-v+2*(v*v)))))/
     & (18._dp*(omu*omu)*(u*u)))+
     & G2(1,1)*(H2(1,0)*(16-(16*(omv*omv))/(9._dp*(u*u)))-
     & (2*H2(1,1)*(3*omv*u+10*(omv*omv)-77*(u*u)))/(9._dp*(u*u))-
     & (2*H2(0,1)*(3*omv*u+2*(omv*omv)-5*(u*u)))/(9._dp*(u*u))-
     & (2*zeta2*(3*omv*u-6*(omv*omv)+67*(u*u)))/(9._dp*(u*u))+
     & (H1(0)*(-14+8*v+4*nf*(omu*omu)*(omv*omv-9*(u*u))+
     & u*(16+38*v-38*(v*v))+7*(v*v)+
     & 6*(u*u)*(98-9*v+10*(v*v))+
     & u*u*u*u*(578+302*v+151*(v*v))-
     & 2*(u*u*u)*(584+147*v+154*(v*v))-
     & 10*NFV*(omu*omu*omu)*
     & (2*(omv*omv)+u*(-1-v+2*(v*v)+u*(2-v+2*(v*v))))))/
     & (18._dp*(omu*omu)*(u*u))+
     & (H1(1)*(-14+8*v+4*nf*(omu*omu)*(omv*omv-9*(u*u))+
     & u*(16+38*v-38*(v*v))+7*(v*v)+
     & 6*(u*u)*(98-9*v+10*(v*v))+
     & u*u*u*u*(578+302*v+151*(v*v))-
     & 2*(u*u*u)*(584+147*v+154*(v*v))-
     & 10*NFV*(omu*omu*omu)*
     & (2*(omv*omv)+u*(-1-v+2*(v*v)+u*(2-v+2*(v*v))))))/
     & (18._dp*(omu*omu)*(u*u)))+
     & (H3(1,1,0)*(omu*u*(32-137*v)*v*(omw*omw)-
     & 2*u*v*(131+35*v)*(omu*omu)*(omw*omw)+
     & omu*omu*omu*(-10*u+40*omw*u+
     & omw*omw*(10+52*v+u*(-649+52*nf-40*v-18*(v*v))-
     & 62*(v*v)))+2*(omw*omw)*
     & (60*NFV*(omv*omv)-u*
     & (16*(v*v)+5*NFV*(u*(-76+95*v-31*(v*v))-
     & 6*(u*u*u)*(4-v+2*(v*v))+
     & 3*(u*u)*(20-15*v+4*(v*v))+
     & u*u*u*u*(4-2*v+4*(v*v))+6*(8-13*v+5*(v*v)))))))/
     & (18._dp*u*(omu*omu*omu)*(omw*omw))+
     & G3(0,3,0)*((64*H1(0))/9._dp+8*H1(1)+
     & (2*omv*(-20+(3+60*NFV)*v)+
     & u*(-8*nf*((-1+u)*(-1+u)*(-1+u))+
     & 10*NFV*(u*u*u*u)*(2-v+2*(v*v))+
     & 2*(-58-21*(11+10*NFV)*v+3*(-17+50*NFV)*(v*v))+
     & u*u*(-668-614*v-217*(v*v)+60*NFV*(1-5*v+v*v))+
     & u*u*u*(236+138*v+71*(v*v)-30*NFV*(2-v+2*(v*v)))+
     & u*(588+892*v+286*(v*v)-10*NFV*(2-58*v+29*(v*v)))))/
     & (18._dp*u*(omu*omu*omu)))+
     & G2(0,3)*((304*zeta2)/9._dp-(128*H2(0,0))/9._dp-8*H2(0,1)-8*H2(1,0)+
     & (H1(0)*(-2*omv*(-20+(3+60*NFV)*v)+
     & u*(116+462*v+420*NFV*v+8*nf*((-1+u)*(-1+u)*(-1+u))+
     & 102*(v*v)-300*NFV*(v*v)-
     & 10*NFV*(u*u*u*u)*(2-v+2*(v*v))+
     & u*u*(668+614*v+217*(v*v)-60*NFV*(1-5*v+v*v))+
     & u*u*u*(-236-138*v-71*(v*v)+30*NFV*(2-v+2*(v*v)))+
     & u*(10*NFV*(2-58*v+29*(v*v))-2*(294+446*v+143*(v*v)))))
     & )/(18._dp*u*(omu*omu*omu)))+
     & G2(2,2)*(-((5+3*u-5*v+10*NFV*(-2+u+2*v))*w*H2(0,0))/(3._dp*(u*u))+
     & (2*H2(0,1)*(3*omv*u+11*(omv*omv)-86*(u*u)))/(9._dp*(u*u))+
     & (H2(1,0)*(90*omv*NFV*u+(12-60*NFV)*(omv*omv)-
     & 2*(14+15*NFV)*(u*u)))/(9._dp*(u*u))+
     & (H1(0)*(-3*(-6+nf*(omu*omu))*(omv*omv)+
     & u*(-21+78*v+(-345+2*v)*(u*u*u)+
     & 3*u*(-119-58*v+9*nf*(omu*omu)-9*(v*v))-57*(v*v)+
     & u*u*(705+130*v+2*(v*v)))))/(9._dp*(omu*omu)*(u*u))+
     & (-24*zeta2*(omu*omu)*(omv*(3-45*NFV)*u+
     & 5*(1+6*NFV)*(omv*omv)+3*(57+5*NFV)*(u*u))+
     & 3*u*((92-12*nf)*(omv*omv)+
     & u*(5*(omu*omu)*(nf*nf)-
     & 6*nf*(11+24*v-4*u*(7+5*v)+15*(u*u)+6*(v*v))+
     & 4*(35+428*v-u*(139+382*v)+81*(u*u)+257*(v*v)))))/
     & (108._dp*(omu*omu)*(u*u)))+
     & G3(1,2,0)*((2*H1(0)*(15*omv*NFV*u+u*(-1+(27-5*NFV)*u+v)+
     & (1-10*NFV)*(omv*omv)))/(3._dp*(u*u))-
     & (2*H1(1)*(3*omv*u+2*(omv*omv)-77*(u*u)))/(9._dp*(u*u))+
     & (63-106*v+u*u*u*(281-138*v-71*(v*v))+
     & u*(-55+70*v-32*(v*v))+42*(v*v)+
     & u*u*(-289+174*v+75*(v*v))+
     & omu*(60*omv*NFV*u-(nf+20*NFV)*(omv*omv)+
     & u*u*(9*nf+10*NFV*(-6+u*(2-v+2*(v*v))))))/(18._dp*omu*(u*u)))
     & +G3(1,0,2)*(-(H1(0)*(6*omv*u+5*(omv*omv)-163*(u*u)))/(9._dp*(u*u))-
     & (2*H1(1)*(3*omv*u+2*(omv*omv)-77*(u*u)))/(9._dp*(u*u))+
     & (63-106*v+u*u*u*(281-138*v-71*(v*v))+
     & u*(-55+70*v-32*(v*v))+42*(v*v)+
     & u*u*(-289+174*v+75*(v*v))+
     & omu*(60*omv*NFV*u-(nf+20*NFV)*(omv*omv)+
     & u*u*(9*nf+10*NFV*(-6+u*(2-v+2*(v*v))))))/(18._dp*omu*(u*u)))
     & +(H3(0,0,1)*(-24+8*nf-22*v+(16*v)/omu-(4*v)/u-9*(v*v)+
     & (2*(v*v))/omu+(4*(v*v))/u+(8*(v*v))/(omu*omu)-
     & (10*NFV*u*(2+3*v-3*(v*v)+2*(v*v*v)))/omv))/18._dp+
     & (H3(1,0,0)*(120*NFV*(-2+v)*(omv*omv)+
     & omv*(u*u*u*(-7924+492*nf-166*v-33*(v*v))+
     & 2*u*(-1358+82*nf-55*v-21*(v*v))-2*(-14+13*v+v*v)+
     & u*u*u*u*(2632-164*nf+18*v+9*(v*v))+
     & u*u*(7980-492*nf+284*v+100*(v*v)))+
     & 10*NFV*u*(u*u*u*(-30+15*v+9*(v*v)-6*(v*v*v))+
     & 6*(16-35*v+24*(v*v)-5*(v*v*v))+
     & 6*(u*u)*(17-23*v+7*(v*v)+v*v*v)+
     & u*u*u*u*(2+3*v-3*(v*v)+2*(v*v*v))+
     & u*(-146+270*v-153*(v*v)+25*(v*v*v)))))/
     & (18._dp*omv*u*(omu*omu*omu))+
     & G3(3,2,0)*((152*H1(0))/9._dp-(10*NFV*(u*u*u)*(2-v+2*(v*v))+
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+20*NFV*(v*v*v))+
     & 2*u*v*(10+5*(-2+3*NFV)*v+(9-45*NFV)*(v*v)+
     & 30*NFV*(v*v*v))+u*u*
     & (24-8*nf+(-22+30*NFV)*v+(9-60*NFV)*(v*v)+60*NFV*(v*v*v))
     & )/(18._dp*(u*u)))+G3(3,0,2)*
     & (18*H1(0)-(10*NFV*(u*u*u)*(2-v+2*(v*v))+
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+20*NFV*(v*v*v))+
     & 2*u*v*(10+5*(-2+3*NFV)*v+(9-45*NFV)*(v*v)+
     & 30*NFV*(v*v*v))+u*u*
     & (24-8*nf+(-22+30*NFV)*v+(9-60*NFV)*(v*v)+60*NFV*(v*v*v))
     & )/(18._dp*(u*u)))+G2(3,2)*
     & ((242*zeta2)/9._dp-(80*H2(0,0))/9._dp+10*H2(0,1)-(10*H2(1,0))/9._dp+
     & (H1(0)*(10*NFV*(u*u*u)*(2-v+2*(v*v))+
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+20*NFV*(v*v*v))+
     & 2*u*v*(10+5*(-2+3*NFV)*v+(9-45*NFV)*(v*v)+
     & 30*NFV*(v*v*v))+
     & u*u*(24-8*nf+(-22+30*NFV)*v+(9-60*NFV)*(v*v)+
     & 60*NFV*(v*v*v))))/(18._dp*(u*u)))+
     & (H2(1,0)*(6*omu*omw*u*w*(-16+(993-56*nf)*v+86*(v*v))+
     & omu*omu*(60*u*w+omw*
     & (-9*(-281+36*nf-20*NFV*v)*w*(u*u*u)+
     & u*w*(-7656+nf*(391-216*v)+3138*v-8004*zeta2+
     & 18*(nf*nf)+1161*(v*v))+
     & 6*(u*u)*(-7+(247+606*v-72*nf*v)*w+
     & 5*NFV*(9+w*(-6-17*v+12*(v*v))))+
     & 6*w*(92-207*v+nf*(-6+13*v-7*(v*v))+
     & 20*NFV*(omv*omv)*(-6+v*v)+107*(v*v)+9*(v*v*v))))-
     & 6*omw*u*w*(v*(-32+v*(-636+nf*(53-43*u+18*(u*u))))-
     & 5*NFV*(2*(6-4*v-10*(v*v)+5*(v*v*v))+
     & u*u*(3-2*v-17*(v*v)+10*(v*v*v))-
     & u*(15+8*v-55*(v*v)+20*(v*v*v))))))/
     & (108._dp*omw*u*w*(omu*omu))+
     & H1(1)*((-4*(-30+nf)*H2(0,0))/9._dp-(5*(-30+nf)*H2(1,0))/9._dp+
     & (omu*u*v*(8*(567-50*nf)*v+6*(32+551*v)*zeta2)*(omw*omw)+
     & 12*zeta2*(omw*omw)*(60*NFV*(omv*omv)-
     & u*(16*(v*v)+5*NFV*
     & (u*(-74+94*v-29*(v*v))+6*(u*u)*(9-7*v+v*v)+
     & u*u*u*u*(2-v+2*(v*v))-3*(u*u*u)*(6-v+2*(v*v))+
     & 6*(8-13*v+5*(v*v)))))-
     & omu*omu*omu*(180*u*zeta2-720*omw*u*zeta2+
     & omw*omw*(-1215+19260*u-1348*nf*u+2430*v-3636*u*v+
     & 432*nf*u*v+1188*u*zeta3+76*nf*(omv*omv)-2529*(u*u)+
     & 324*nf*(u*u)-1215*(v*v)+
     & 6*zeta2*(5-90*v+85*(v*v)+
     & u*(-1022+112*nf+38*v+9*(v*v)))))+
     & omu*omu*(omw*omw)*(-60*NFV*(5+v)*(omv*omv)+
     & u*(v*(32*nf*(-10+v)+36*(112+3*v)+6*(1114-71*v)*zeta2)+
     & 30*NFV*(12-7*v+2*(-4+v)*(u*u)-7*(v*v)+
     & u*(6-13*v+4*(v*v))+2*(v*v*v)))))/
     & (108._dp*u*(omu*omu*omu)*(omw*omw)))+
     & (H2(0,0)*(-6*u*v*w*(32+5*NFV*
     & (34+10*v-u*(56+41*v)+(4+13*v)*(u*u))+
     & 2*v*(-392+3*nf*(13-8*u+3*(u*u))))-
     & 6*omu*u*w*(-16+45*NFV+(-1679+96*nf)*v-225*(v*v))+
     & omu*omu*(-9*(-281+36*nf-20*NFV*v)*w*(u*u*u)+
     & 6*(u*u)*(7+(247+606*v-72*nf*v)*w+
     & 15*NFV*(-3+w*(-2-3*v+4*(v*v))))+
     & 6*v*w*(-194+12*omv*nf+184*v+9*(v*v)+
     & 20*NFV*(5-5*v-v*v+v*v*v))+
     & u*w*(-4*nf*(145+54*v)+36*(nf*nf)+
     & 3*(711+1046*v-2256*zeta2+387*(v*v)+
     & 10*NFV*(33+10*(v*v*v)))))))/(108._dp*u*w*(omu*omu))+
     & (-4320*(-16+45*NFV)*(-1+u)*u*w*zeta3*(v*v)+
     & 360*omu*(-1+u)*w*(2*u*v*
     & (6*(16-138*v+45*NFV*(-1+2*v))*zeta2+
     & 3*(-32+281*v+30*NFV*(3+5*v))*zeta3)+
     & 6*nf*zeta2*(144*(-3+2*v)*(u*u*u*u)+216*(u*u*u*u*u)+
     & u*u*(886+44*v-144*(v*v))-4*(2-3*v+v*v)+
     & 3*(u*u*u)*(-77-144*v+24*(v*v))+u*(-431+88*v+60*(v*v))))
     & -40*(-1+u)*u*w*(omu*omu)*
     & (-2*nf*(7387+1368*v-2502*zeta3+u*(-7387+2502*zeta3))+
     & 9*(36*v*(98+5*NFV*(5+4*v)*zeta3)+
     & 6*zeta2*(16-409*v-300*(v*v)+15*NFV*(-3+2*v+12*(v*v)))))
     & +3*(omu*omu*omu)*(1916370*(-1+u)*u*w*zeta4+
     & 80*w*(18*(40+60*NFV*(-2+v)-51*v)*(-1+v)*zeta3+
     & (-47323+21519*zeta3+2160*NFV*zeta3-36*v*zeta3)*(u*u)-
     & u*(-47323+20799*zeta3+6552*v*zeta3+36*zeta3*(v*v)+
     & 1080*NFV*zeta3*(4-3*v+v*v)))-
     & 720*(-1+u)*zeta2*(6*(281+20*NFV*v)*w*(u*u*u)+
     & u*u*(7+988*w+2424*v*w+
     & 15*NFV*(-3+4*w*(-2-3*v+4*(v*v))))+
     & w*(-305+689*v-421*(v*v)+
     & 20*NFV*(omv*omv)*(7-v+4*(v*v))+36*(v*v*v))+
     & u*w*(-5437+2002*v+774*(v*v)+
     & 5*NFV*(23-12*v-60*(v*v)+40*(v*v*v))))))/
     & (38880._dp*(-1+u)*u*w*(omu*omu*omu))+
     & G2(2,1)*(H2(0,1)*(8-(8*(omv*omv))/(9._dp*(u*u)))+
     & H2(1,0)*(-8+(8*(omv*omv))/(9._dp*(u*u)))-
     & (22*zeta2*(omv*omv-9*(u*u)))/(9._dp*(u*u))+
     & (H1(0)*(16*omu*v*(25+4*v)*(omw*omw)*(u*u)+
     & 200*(omw*omw)*(u*u)*(v*v)+
     & omu*omu*(10*(u*u)-40*omw*(u*u)+
     & omw*omw*(30*omv*NFV*(u*u)+
     & 20*NFV*(omv*omv*omv)*(1+v+v*v)-
     & 10*NFV*(u*u*u)*(2-v+2*(v*v))-
     & (-1+v)*(-1+v)*(38+20*v+9*(v*v))+
     & omv*omv*(nf-30*NFV*u*(1+v+2*(v*v)))-
     & 2*u*(-30+61*v-40*(v*v)+9*(v*v*v))+
     & u*u*(32-9*nf+18*v+(-9+60*NFV)*(v*v)-60*NFV*(v*v*v))
     & ))))/(18._dp*(omu*omu)*(omw*omw)*(u*u))+
     & (H1(1)*(16*omu*v*(25+4*v)*(omw*omw)*(u*u)+
     & 200*(omw*omw)*(u*u)*(v*v)+
     & omu*omu*(10*(u*u)-40*omw*(u*u)+
     & omw*omw*(30*omv*NFV*(u*u)+
     & 20*NFV*(omv*omv*omv)*(1+v+v*v)-
     & 10*NFV*(u*u*u)*(2-v+2*(v*v))-
     & (-1+v)*(-1+v)*(38+20*v+9*(v*v))+
     & omv*omv*(nf-30*NFV*u*(1+v+2*(v*v)))-
     & 2*u*(-30+61*v-40*(v*v)+9*(v*v*v))+
     & u*u*(32-9*nf+18*v+(-9+60*NFV)*(v*v)-60*NFV*(v*v*v))
     & ))))/(18._dp*(omu*omu)*(omw*omw)*(u*u)))+
     & G3(2,1,2)*(H1(0)*(8-(8*(omv*omv))/(9._dp*(u*u)))+
     & (-16*omu*v*(25+4*v)*(omw*omw)*(u*u)-
     & 200*(omw*omw)*(u*u)*(v*v)+
     & omu*omu*(-10*(u*u)+40*omw*(u*u)+
     & omw*omw*(-30*omv*NFV*(u*u)-
     & 20*NFV*(omv*omv*omv)*(1+v+v*v)+
     & 10*NFV*(u*u*u)*(2-v+2*(v*v))+
     & (-1+v)*(-1+v)*(38+20*v+9*(v*v))+
     & omv*omv*(-nf+30*NFV*u*(1+v+2*(v*v)))+
     & 2*u*(-30+61*v-40*(v*v)+9*(v*v*v))+
     & u*u*(-32+9*nf-18*v+(9-60*NFV)*(v*v)+60*NFV*(v*v*v))))
     & )/(18._dp*(omu*omu)*(omw*omw)*(u*u)))+
     & G3(3,3,0)*((-128*H1(0))/9._dp+(10*NFV*(u*u*u)*(2-v+2*(v*v))+
     & u*u*(-532+32*nf+(-22+30*NFV)*v+(9-60*NFV)*(v*v)+
     & 60*NFV*(v*v*v))+v*
     & (40-52*v+(2+20*NFV)*(v*v)+(9-40*NFV)*(v*v*v)+
     & 20*NFV*(v*v*v*v))+
     & 2*u*(20-28*v+5*(-2+3*NFV)*(v*v)+(9-45*NFV)*(v*v*v)+
     & 30*NFV*(v*v*v*v)))/(18._dp*(u*u)))+
     & G2(3,3)*((-256*zeta2)/9._dp+(128*H2(0,0))/9._dp-
     & (H1(0)*(10*NFV*(u*u*u)*(2-v+2*(v*v))+
     & u*u*(-532+32*nf+(-22+30*NFV)*v+(9-60*NFV)*(v*v)+
     & 60*NFV*(v*v*v))+
     & v*(40-52*v+(2+20*NFV)*(v*v)+(9-40*NFV)*(v*v*v)+
     & 20*NFV*(v*v*v*v))+
     & 2*u*(20-28*v+5*(-2+3*NFV)*(v*v)+(9-45*NFV)*(v*v*v)+
     & 30*NFV*(v*v*v*v))))/(18._dp*(u*u)))+
     & ((33-2*nf)/6._dp-(17*H1(0))/2._dp-(9*H1(1))/2._dp)*(G1(0)*G1(0)*G1(0))+
     & (9*(G1(0)*G1(0)*G1(0)*G1(0)))/8._dp+
     & ((-9*H1(1))/2._dp+H1(0)*(-9+(omv*omv)/(2._dp*(u*u)))+
     & (3*(omv*omv)+u*(-12+15*u+36*v-30*u*v-
     & 2*nf*((-1+u)*(-1+u))-6*(u*u)+9*(v*v)))/(6._dp*u*(omu*omu)))
     & *(G1(2)*G1(2)*G1(2))+(9*(G1(2)*G1(2)*G1(2)*G1(2)))/8._dp+
     & ((17*H1(1))/2._dp+(17*(3*omv*v+(-6-4*nf+27*v)*(u*u)+
     & (3+2*nf)*(u*u*u)+u*(3+2*nf-30*v-9*(v*v))))/
     & (54._dp*u*(omu*omu)))*(H1(0)*H1(0)*H1(0))+
     & (289*(H1(0)*H1(0)*H1(0)*H1(0)))/72._dp+
     & G2(3,0)*(H1(0)*(32-(20*nf)/9._dp-8*H1(1))-(64*H2(0,0))/3._dp-8*H2(0,1)-
     & (152*H2(1,0))/9._dp+(H1(1)*(10*NFV*(u*u*u)*(2-v+2*(v*v))+
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+20*NFV*(v*v*v))+
     & 2*u*v*(10+5*(-2+3*NFV)*v+(9-45*NFV)*(v*v)+
     & 30*NFV*(v*v*v))+
     & u*u*(-216+(-22+30*NFV)*v+(9-60*NFV)*(v*v)+60*NFV*(v*v*v))
     & ))/(18._dp*(u*u))+(-6*omu*u*w*(16+103*v+55*(v*v))+
     & omu*omu*(9*(-281+36*nf-20*NFV*v)*w*(u*u*u)+
     & u*w*(3*(2556-1010*v+2448*zeta2-387*(v*v))+
     & 4*nf*(-152+54*v+27*(v*v)))-
     & 6*(u*u)*(7+(247+606*v-72*nf*v)*w+
     & 15*NFV*(-3+w*(-2-3*v+4*(v*v))))-
     & 6*v*w*(-6-4*v+9*(v*v)+20*NFV*(6-5*v-2*(v*v)+v*v*v)))
     & -6*u*w*(-32*v*(1+v)+
     & 5*NFV*(24-32*v-20*(v*v)+
     & u*(-57+52*v+55*(v*v)-20*(v*v*v))+10*(v*v*v)+
     & u*u*(33-2*v-17*(v*v)+10*(v*v*v)))))/
     & (108._dp*u*w*(omu*omu))-(68*(H1(0)*H1(0)))/9._dp-4*(H1(1)*H1(1)))+
     & (4*H2(0,0)+5*H2(1,0)+(4*(-30+nf)*(omv*omv)+
     & u*(-5*(omu*omu)*(nf*nf)+
     & 2*nf*(-4*u*(78+5*v)+157*(u*u)+3*(51+8*v+2*(v*v)))-
     & 3*(270*zeta2*((-1+u)*(-1+u))+
     & 4*(207-444*u+108*v-88*u*v+227*(u*u)+30*(v*v)))))/
     & (72._dp*u*(omu*omu)))*(H1(1)*H1(1))+
     & G1(0)*G1(0)*((9*G2(0,2))/2._dp+(9*G2(2,0))/2._dp-4*G2(3,0)+4*G1(3)*H1(0)+
     & 4*H2(0,0)+5*H2(1,0)+(36*(-119+119*u+4*v)+
     & omu*(366*nf-810*zeta2-5*(nf*nf)))/(72._dp*omu)+
     & G1(1)*(H1(0)*(9._dp/2._dp-(omv*omv)/(2._dp*(u*u)))+
     & H1(1)*(9._dp/2._dp-(omv*omv)/(2._dp*(u*u))))+
     & (G2(1,2)*(-9+(omv*omv)/(u*u)))/2._dp+
     & H1(0)*((27*H1(1))/2._dp+(9*omv*v+(-102-20*nf+81*v)*(u*u)+
     & (51+10*nf)*(u*u*u)+u*(51+10*nf-90*v-27*(v*v)))/
     & (18._dp*u*(omu*omu)))-
     & (H1(1)*(omv*omv+u*(-45+87*u+12*v-10*u*v+
     & nf*((-1+u)*(-1+u))-43*(u*u)+3*(v*v))))/
     & (2._dp*u*(omu*omu))+G1(2)*
     & ((-9*H1(1))/2._dp+H1(0)*(-9+(omv*omv)/(2._dp*(u*u)))+
     & (omv*omv+u*(u*(27-10*v)-13*(u*u)+3*(-5+4*v+v*v)))/
     & (2._dp*u*(omu*omu)))+(9*(G1(2)*G1(2)))/4._dp+(67*(H1(0)*H1(0)))/4._dp+
     & (9*(H1(1)*H1(1)))/4._dp)+G2(0,2)*
     & ((232*H2(0,0))/9._dp+H2(1,0)*(20-(2*(omv*omv))/(u*u))+
     & H2(0,1)*(61._dp/9._dp-(omv*omv)/(u*u))+
     & (H1(1)*(-8*(omv*omv)+u*
     & (45*nf*((-1+u)*(-1+u))+(-394+4*v)*(u*u)+
     & 4*u*(195+79*v+v*v)-6*(63+56*v+25*(v*v)))))/
     & (18._dp*u*(omu*omu))+H1(0)*
     & (9*H1(1)+(40/omw-10/(omw*omw)-
     & (2*omu*v*(162+7*v)*(u*u)+162*(u*u)*(v*v)+
     & omu*omu*((16+nf)*(omv*omv)+
     & u*(12*(-1+v*v)+10*NFV*(u*u)*(2-v+2*(v*v))-
     & u*(-1316+89*nf+26*v+9*(v*v)))))/(omu*omu*(u*u)))/
     & 18._dp)+(omu*omu*(-108*zeta2*(omv*omv)+
     & (-7914*zeta2+3*(nf*nf))*(u*u))-
     & 3*u*(2*nf*(omv*omv)+
     & 4*(298-303*v+18*nf*(-3+2*v))*(u*u*u*u)+
     & 3*(-281+36*nf)*(u*u*u*u*u)+
     & u*u*u*(2760+1414*v-387*(v*v)+nf*(-86-216*v+36*(v*v)))-
     & 2*(27-44*v+8*(v*v)+9*(v*v*v))-
     & 2*(u*u)*(2889-730*v-375*(v*v)+nf*(-195+10*v+36*(v*v))+
     & 9*(v*v*v))+u*(2723-1750*v-647*(v*v)+
     & 6*nf*(-33+16*v+7*(v*v))+36*(v*v*v)))-
     & 30*omu*NFV*u*(4*(1+v)*(omv*omv*omv)+
     & u*(6*v*(u*u*u)+3*(u*u)*(-2-5*v+4*(v*v))+
     & u*(16+5*v-25*(v*v)+10*(v*v*v))+
     & 2*(-7+6*v+8*(v*v)-9*(v*v*v)+2*(v*v*v*v)))))/
     & (108._dp*(omu*omu)*(u*u))+(17*(H1(0)*H1(0)))/2._dp+(9*(H1(1)*H1(1)))/2._dp)+
     & G2(2,0)*(H2(1,0)*(21+(omv*omv)/(u*u))+
     & (H2(0,1)*(-6*omv*u+5*(omv*omv)-71*(u*u)))/(9._dp*(u*u))+
     & (H2(0,0)*(2*omv*(-1+15*NFV)*u+(5-20*NFV)*(omv*omv)+
     & (21-10*NFV)*(u*u)))/(3._dp*(u*u))+
     & (H1(1)*(-8*(omv*omv)+u*
     & (45*nf*((-1+u)*(-1+u))+(-394+4*v)*(u*u)+
     & 4*u*(195+79*v+v*v)-6*(63+56*v+25*(v*v)))))/
     & (18._dp*u*(omu*omu))+(omu*omu*
     & (72*omv*u*zeta2+48*zeta2*(omv*omv)+
     & (-6222*zeta2+3*(nf*nf))*(u*u))-
     & 3*u*(2*nf*(omv*omv)+
     & 4*(298-303*v+18*nf*(-3+2*v))*(u*u*u*u)+
     & 3*(-281+36*nf)*(u*u*u*u*u)+
     & u*u*u*(2760+1414*v-387*(v*v)+nf*(-86-216*v+36*(v*v)))-
     & 2*(27-44*v+8*(v*v)+9*(v*v*v))-
     & 2*(u*u)*(2889-730*v-375*(v*v)+nf*(-195+10*v+36*(v*v))+
     & 9*(v*v*v))+u*(2723-1750*v-647*(v*v)+
     & 6*nf*(-33+16*v+7*(v*v))+36*(v*v*v)))-
     & 30*omu*NFV*u*(4*(1+v)*(omv*omv*omv)+
     & u*(6*v*(u*u*u)+3*(u*u)*(-2-5*v+4*(v*v))+
     & u*(16+5*v-25*(v*v)+10*(v*v*v))+
     & 2*(-7+6*v+8*(v*v)-9*(v*v*v)+2*(v*v*v*v)))))/
     & (108._dp*(omu*omu)*(u*u))+
     & H1(0)*(9*H1(1)-(-63+153*u+106*v-270*u*v+
     & nf*(omu*omu*omu)*(omv*omv-89*(u*u))+1200*(u*u)+
     & 542*v*(u*u)-3888*(u*u*u)-684*v*(u*u*u)+3879*(u*u*u*u)+
     & 324*v*(u*u*u*u)-1281*(u*u*u*u*u)-18*v*(u*u*u*u*u)-
     & 32*(v*v)+102*u*(v*v)-12*(u*u)*(v*v)-56*(u*u*u)*(v*v)-
     & 43*(u*u*u*u)*(v*v)+9*(u*u*u*u*u)*(v*v)-2*(v*v*v)-
     & 12*u*(v*v*v)+48*(u*u)*(v*v*v)-52*(u*u*u)*(v*v*v)+
     & 18*(u*u*u*u)*(v*v*v)-9*(v*v*v*v)+27*u*(v*v*v*v)-
     & 27*(u*u)*(v*v*v*v)+9*(u*u*u)*(v*v*v*v)+
     & 10*NFV*(2*(omv*omv*omv)*(1+v+v*v)+
     & u*(u*u*u*u*u*(2-v+2*(v*v))+
     & 6*(u*u*u*u)*(2+v-2*(v*v)+v*v*v)+
     & 3*(u*u*u)*(-22+11*v+9*(v*v)-9*(v*v*v)+
     & 2*(v*v*v*v))+
     & u*u*(104-97*v-4*(v*v)+47*(v*v*v)-22*(v*v*v*v)+
     & 2*(v*v*v*v*v))-
     & 3*u*(22-27*v+3*(v*v)+13*(v*v*v)-10*(v*v*v*v)+
     & 2*(v*v*v*v*v))+
     & 3*(4-6*v+v*v+5*(v*v*v)-6*(v*v*v*v)+2*(v*v*v*v*v)))
     & ))/(18._dp*(omu*omu*omu)*(u*u)))+(17*(H1(0)*H1(0)))/2._dp+
     & (9*(H1(1)*H1(1)))/2._dp)+G1(2)*G1(2)*
     & ((9*G2(0,2))/2._dp+(9*G2(2,0))/2._dp-4*G2(3,0)+4*G1(3)*H1(0)+4*H2(0,0)+
     & 5*H2(1,0)+(G2(1,2)*(-9+(omv*omv)/(u*u)))/2._dp+
     & (H1(1)*(-3*(omv*omv)+u*
     & (2*nf*((-1+u)*(-1+u))+
     & 3*(4-5*u-12*v+10*u*v+2*(u*u)-3*(v*v)))))/
     & (2._dp*u*(omu*omu))+H1(0)*
     & (H1(1)*(45._dp/2._dp-(omv*omv)/(u*u))+
     & ((-30+nf*(omu*omu))*(omv*omv)+
     & 3*u*(14-25*v+3*(-72+29*v)*(u*u)+105*(u*u*u)+
     & u*(107-82*v+3*nf*(omu*omu)-37*(v*v))+11*(v*v)))/
     & (18._dp*(omu*omu)*(u*u)))+
     & (4*(-30+nf)*(omv*omv)+
     & u*(-5*(omu*omu)*(nf*nf)+
     & 2*nf*(-4*u*(78+5*v)+157*(u*u)+3*(51+8*v+2*(v*v)))-
     & 3*(270*zeta2*((-1+u)*(-1+u))+
     & 4*(207-444*u+108*v-88*u*v+227*(u*u)+30*(v*v)))))/
     & (72._dp*u*(omu*omu))+(71._dp/4._dp-(omv*omv)/(u*u))*(H1(0)*H1(0))+
     & (27*(H1(1)*H1(1)))/4._dp)+H1(0)*H1(0)*
     & ((68*H2(0,0))/9._dp+(85*H2(1,0))/9._dp+
     & (H1(1)*(-17+52*v+(-297-36*nf+332*v)*(u*u)+
     & 2*(70+9*nf)*(u*u*u)+3*u*(58+6*nf-128*v-35*(v*v))-
     & 35*(v*v)))/(18._dp*u*(omu*omu))-
     & (12*omv*(-57+2*nf)*v-
     & 2*(-4*nf*(539+27*v)+3*(6927+890*v+1938*zeta2)+36*(nf*nf))*
     & (u*u)+(20781-2156*nf+5814*zeta2+36*(nf*nf))*(u*u*u)+
     & u*(36*(nf*nf)-4*nf*(539+60*v+18*(v*v))+
     & 3*(6927+2008*v+1938*zeta2+684*(v*v))))/(216._dp*u*(omu*omu))
     & +(35*(H1(1)*H1(1)))/4._dp)+
     & G2(1,2)*(H2(0,1)*(10-(10*(omv*omv))/(9._dp*(u*u)))+
     & (H2(1,0)*(6*omv*(u+15*NFV*u)+(31-60*NFV)*(omv*omv)-
     & 5*(31+6*NFV)*(u*u)))/(9._dp*(u*u))+
     & (H2(0,0)*(6*omv*u+omv*omv-81*(u*u)+
     & 30*NFV*(-3*omv*u+2*(omv*omv)+u*u)))/(9._dp*(u*u))+
     & (H1(1)*((-6+5*nf*(omu*omu))*(omv*omv)-
     & u*(18+36*v+4*(-105+v)*(u*u*u)+
     & 3*u*(-158-96*v+15*nf*(omu*omu)-28*(v*v))-54*(v*v)+
     & u*u*(870+260*v+4*(v*v)))))/(18._dp*(omu*omu)*(u*u))+
     & H1(0)*(H1(1)*(-9+(omv*omv)/(u*u))+
     & ((-36+5*nf*(omu*omu))*(omv*omv)+
     & u*(-3*u*(-166-52*v+15*nf*(omu*omu)+8*(v*v))-
     & 2*(u*u*u)*(-229+82*v+40*(v*v))+
     & 2*(17-70*v+53*(v*v))+2*(u*u)*(-477+38*v+81*(v*v))))/
     & (18._dp*(omu*omu)*(u*u)))+
     & (-4*omw*(nf*(-19+44*u)*(omv*omv)+
     & u*u*(-60*(v*v)+nf*
     & (143+56*v-54*(-3+2*v)*(u*u*u)-81*(u*u*u*u)+
     & u*u*(93+162*v-27*(v*v))-28*(v*v)+
     & 6*u*(-57-10*v+9*(v*v)))))-
     & 6*omu*omw*(30*NFV*v*w*(u*u*u*u)+
     & 15*NFV*w*(u*u*u)*(-2-3*v+2*(v*v))+
     & 10*NFV*w*(-3+2*v-v*v+2*(v*v*v))-
     & 5*NFV*u*w*(6-v-10*(v*v)+8*(v*v*v))+
     & u*u*(90*NFV*w+4*v*(-102+5*NFV*w)-2*(37+35*NFV*w)*(v*v)+
     & 20*NFV*w*(v*v*v)))+
     & omu*omu*(-60*(-1+u)*u+
     & omw*(6*zeta2*(12*u*(-4+4*v+5*NFV*w)+
     & 5*(-1+v)*(-17+17*v+24*NFV*w)+951*(u*u))-
     & 3*(391-802*v+2*(247+606*v)*(u*u*u)+843*(u*u*u*u)+
     & 429*(v*v)+u*u*(-2818+1010*v+387*(v*v))-18*(v*v*v)+
     & 2*u*(-87+80*v+9*(v*v)+9*(v*v*v))))))/
     & (108._dp*omw*(omu*omu)*(u*u))+
     & (17*(-9+(omv*omv)/(u*u))*(H1(0)*H1(0)))/18._dp+
     & ((-9+(omv*omv)/(u*u))*(H1(1)*H1(1)))/2._dp)+
     & G1(3)*((-242*zeta2*H1(1))/9._dp+(16*(-33+2*nf)*H2(0,0))/9._dp+
     & (64*H3(0,0,0))/3._dp+(80*H3(0,1,0))/9._dp+(80*H3(1,0,0))/9._dp-10*H3(1,0,1)+
     & (10*H3(1,1,0))/9._dp-(H2(0,1)*
     & (10*NFV*(u*u*u)*(2-v+2*(v*v))+
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+20*NFV*(v*v*v))+
     & 2*u*v*(10+5*(-2+3*NFV)*v+(9-45*NFV)*(v*v)+
     & 30*NFV*(v*v*v))+
     & u*u*(24-8*nf+(-22+30*NFV)*v+(9-60*NFV)*(v*v)+
     & 60*NFV*(v*v*v))))/(18._dp*(u*u))-
     & (H2(1,0)*(10*NFV*(u*u*u)*(2-v+2*(v*v))+
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+20*NFV*(v*v*v))+
     & 2*u*v*(10+5*(-2+3*NFV)*v+(9-45*NFV)*(v*v)+
     & 30*NFV*(v*v*v))+
     & u*u*(24-8*nf+(-22+30*NFV)*v+(9-60*NFV)*(v*v)+
     & 60*NFV*(v*v*v))))/(18._dp*(u*u))+
     & (-1920*zeta3*(u*u)+6*zeta2*
     & (50*NFV*(u*u*u)*(2-v+2*(v*v))+
     & u*u*(-1196+64*nf+10*(-11+15*NFV)*v+(45-300*NFV)*(v*v)+
     & 300*NFV*(v*v*v))+
     & v*(-40-8*v+10*(1+10*NFV)*(v*v)-5*(-9+40*NFV)*(v*v*v)+
     & 100*NFV*(v*v*v*v))+
     & 2*u*(-20+88*v+25*(-2+3*NFV)*(v*v)-
     & 45*(-1+5*NFV)*(v*v*v)+150*NFV*(v*v*v*v))))/(108._dp*(u*u))+
     & (76._dp/3._dp-(8*nf)/9._dp+8*H1(1))*(H1(0)*H1(0))+(68*(H1(0)*H1(0)*H1(0)))/9._dp+
     & H1(0)*((-4*(-30+nf)*H1(1))/9._dp+
     & (12*(-16+45*NFV)*u*v*(1+v)*w-
     & 6*omu*u*w*(-16-103*v-55*(v*v)+
     & 15*NFV*(3+16*v+7*(v*v)))+
     & omu*omu*(-9*(-281+36*nf-20*NFV*v)*w*(u*u*u)+
     & 6*(u*u)*(7+(247+606*v-72*nf*v)*w+
     & 15*NFV*(-3+w*(-2-3*v+4*(v*v))))+
     & 6*v*w*(-6-4*v+9*(v*v)+
     & 20*NFV*(6-5*v-2*(v*v)+v*v*v))+
     & u*w*(-4*nf*(-152+54*v+27*(v*v))+
     & 3*(-2556+1010*v-2448*zeta2+387*(v*v)+
     & 10*NFV*(33-2*v-17*(v*v)+10*(v*v*v))))))/
     & (108._dp*u*w*(omu*omu))+4*(H1(1)*H1(1))))+
     & G1(0)*(G2(0,2)*((-30+nf)/2._dp-9*H1(0))+G2(2,0)*((-30+nf)/2._dp-9*H1(0))+
     & G2(3,0)*((-4*(-30+nf))/9._dp+8*H1(0))-(64*H3(0,0,0))/3._dp-8*H3(0,0,1)-
     & (152*H3(0,1,0))/9._dp-(232*H3(1,0,0))/9._dp-10*H3(1,0,1)-20*H3(1,1,0)+
     & G2(1,2)*(H1(0)*(9-(omv*omv)/(u*u))+
     & ((-30+nf)*(omv*omv-9*(u*u)))/(18._dp*(u*u)))+
     & (H2(1,1)*(4*(omv*omv)+u*
     & (324+168*v-27*nf*((-1+u)*(-1+u))-2*(-166+v)*(u*u)+
     & 75*(v*v)-2*u*(330+79*v+v*v))))/(9._dp*u*(omu*omu))+
     & (H2(1,0)*(120*NFV*(omv*omv)+6*(1+v-2*(v*v))-
     & 10*NFV*(u*u*u*u*u)*(2-v+2*(v*v))+
     & u*u*u*(4764-306*nf+234*v-25*(v*v)-
     & 60*NFV*(9-7*v+v*v))+
     & u*u*u*u*(-1590+102*nf+26*v+9*(v*v)+
     & 30*NFV*(6-v+2*(v*v)))-
     & 6*u*(-262+17*nf-51*v-23*(v*v)+
     & 10*NFV*(8-13*v+5*(v*v)))+
     & 2*(u*u)*(-2376+153*nf-286*v-71*(v*v)+
     & 5*NFV*(74-94*v+29*(v*v)))))/(18._dp*u*(omu*omu*omu))+
     & (H2(0,1)*(2*omu*u*(82-3*v)*v+82*u*(v*v)+
     & omu*omu*(-10*(-1+v)*v+10*NFV*(u*u)*(2-v+2*(v*v))+
     & u*(915-62*nf+138*v+71*(v*v)))))/(18._dp*u*(omu*omu))+
     & (-384*omu*u*v*(-1+5*v)*zeta2*(omw*omw)+
     & 2*u*(omu*omu)*(omw*omw)*
     & (-2*v*(-69+66*(14+v)*zeta2)+
     & 15*NFV*(2*(-4+v)*(u*u)-2*(-5+4*v+v*v)+
     & u*(-2-3*v+2*(v*v))))+
     & omu*omu*omu*(12*omw*u*(20+9*(-10+nf)*u)-240*(u*u)+
     & omw*omw*(-3*u*(-1775-843*u-369*v+4*nf*(10+27*u+9*v)+
     & 324*zeta3)+12*zeta2*
     & (u*(-485+34*nf-158*v-80*(v*v))-2*(5-12*v+7*(v*v)))
     & ))+24*zeta2*(omw*omw)*
     & (60*NFV*(omv*omv)+
     & u*(-16*(v*v)+15*NFV*
     & ((-16+13*v)*(u*u)+4*(u*u*u)-2*(8-13*v+5*(v*v))+
     & u*(24-31*v+9*(v*v))))))/
     & (108._dp*u*(omu*omu*omu)*(omw*omw))+
     & (H2(0,0)*(-30*(-1+4*NFV)*v*(omv*omv)+
     & omv*u*(1584+4*(58+105*NFV)*v+
     & 96*nf*((-1+u)*(-1+u)*(-1+u))+(173-300*NFV)*(v*v))+
     & u*u*(-4752+(4114-630*NFV)*v+(337+900*NFV)*(v*v)+
     & 20*NFV*v*(u*u*u)*(3-3*v+2*(v*v))+
     & (301-310*NFV)*(v*v*v)+
     & 2*u*(2376+15*(-142+15*NFV)*v-(152+225*NFV)*(v*v)+
     & (-94+60*NFV)*(v*v*v))-
     & 2*(u*u)*(792+(-734+90*NFV)*v-9*(3+10*NFV)*(v*v)+
     & (-31+60*NFV)*(v*v*v)))))/(18._dp*omv*u*(omu*omu*omu))+
     & (H1(1)*(6-32*v+4*(298-303*v+18*nf*(-3+2*v))*(u*u*u*u)+
     & 3*(-281+36*nf)*(u*u*u*u*u)+44*(v*v)+
     & u*u*u*(2880+1414*v+2754*zeta2-387*(v*v)+
     & 12*nf*(-10-18*v+3*(v*v)))-18*(v*v*v)-
     & 2*(u*u)*(2979-430*v+2754*zeta2-375*(v*v)+
     & 12*nf*(-19+3*(v*v))+9*(v*v*v))+
     & u*(2723-1030*v+2754*zeta2-467*(v*v)+
     & 12*nf*(-19+6*v+3*(v*v))+36*(v*v*v))+
     & 10*omu*NFV*(4*(1+v)*(omv*omv*omv)+
     & u*(6*v*(u*u*u)+3*(u*u)*(-2-5*v+4*(v*v))+
     & u*(16+5*v-25*(v*v)+10*(v*v*v))+
     & 2*(-7+6*v+8*(v*v)-9*(v*v*v)+2*(v*v*v*v))))))/
     & (36._dp*u*(omu*omu))+((-17*H1(0))/2._dp-(9*H1(1))/2._dp)*(G1(2)*G1(2))+
     & G1(3)*((4*(-30+nf)*H1(0))/9._dp-8*(H1(0)*H1(0)))+
     & ((-51*H1(1))/2._dp+(-18*omv*v+2*(375+2*nf-81*v)*(u*u)-
     & (375+2*nf)*(u*u*u)+u*(-375-2*nf+180*v+54*(v*v)))/
     & (18._dp*u*(omu*omu)))*(H1(0)*H1(0))+
     & G1(1)*(H1(0)*(H1(1)*(-9+(omv*omv)/(u*u))-
     & ((-30+nf)*(omv*omv-9*(u*u)))/(18._dp*(u*u)))-
     & ((-30+nf)*H1(1)*(omv*omv-9*(u*u)))/(18._dp*(u*u))+
     & (-9+(omv*omv)/(u*u))*(H1(0)*H1(0)))-(289*(H1(0)*H1(0)*H1(0)))/18._dp+
     & H1(0)*(-8*H2(0,0)-10*H2(1,0)+
     & (H1(1)*(18*(omv*omv)+
     & u*(26*nf*((-1+u)*(-1+u))-
     & 3*(341-664*u-72*v+60*u*v+329*(u*u)-18*(v*v)))))/
     & (18._dp*u*(omu*omu))-
     & (9*(-281+36*nf-20*NFV*v)*(u*u*u*u*u*u)+
     & 3*(u*u*u*u*u)*(36*nf*(-9+7*v)+
     & 5*(407-411*v+NFV*(12+38*v-36*(v*v))))+
     & 6*omv*v*(3-13*v+20*NFV*(1+v)*(omv*omv)+9*(v*v))+
     & u*u*u*u*(4*nf*(1-432*v+135*(v*v))+
     & 3*(2927+3818*v+3474*zeta2-1599*(v*v)-
     & 10*NFV*(60+7*v-54*(v*v)+22*(v*v*v))))+
     & u*u*u*(4*nf*(645+v-243*(v*v)+27*(v*v*v))-
     & 3*(12515-3719*v-3474*(-3+v)*zeta2-2533*(v*v)+
     & 405*(v*v*v)+
     & 10*NFV*(-126+94*v+50*(v*v)-63*(v*v*v)+14*(v*v*v*v)))
     & )-u*u*(4*nf*(726-538*v-81*(v*v)+54*(v*v*v))+
     & 3*(-12324+10570*v+3474*(-3+2*v)*zeta2+135*(v*v)-
     & 786*(v*v*v)+18*(v*v*v*v)+
     & 10*NFV*(96-140*v+28*(v*v)+63*(v*v*v)-36*(v*v*v*v)+
     & 4*(v*v*v*v*v))))+
     & u*(4*nf*(242-296*v+27*(v*v)+27*(v*v*v))-
     & 3*(3928-5082*v+3474*omv*zeta2+703*(v*v)+485*(v*v*v)-
     & 36*(v*v*v*v)-
     & 20*NFV*(12-28*v+16*(v*v)+11*(v*v*v)-15*(v*v*v*v)+
     & 4*(v*v*v*v*v)))))/(108._dp*u*w*(omu*omu))-
     & (35*(H1(1)*H1(1)))/2._dp)+((-30+nf)*(H1(1)*H1(1)))/2._dp+
     & G1(2)*((15-nf/2._dp)*H1(1)+
     & H1(0)*(26*H1(1)+((-30+42*u+nf*(omu*omu))*(omv*omv)+
     & u*u*(-17*nf*(omu*omu)+
     & 6*(125-251*u-26*v+30*u*v+124*(u*u)-14*(v*v))))/
     & (18._dp*(omu*omu)*(u*u)))-
     & (u*(omu*omu)*(nf*nf)+
     & 60*(omv*omv+u*(3*v*(4+v)-u*(3+10*v)+2*(u*u)))-
     & 2*nf*(omv*omv+u*
     & (-(u*(33+10*v))+17*(u*u)+3*(5+4*v+v*v))))/
     & (36._dp*u*(omu*omu))+(26-(omv*omv)/(u*u))*(H1(0)*H1(0))+
     & 9*(H1(1)*H1(1)))-(9*(H1(1)*H1(1)*H1(1)))/2._dp)+
     & G1(2)*(G2(0,2)*((-30+nf)/2._dp-9*H1(0)-9*H1(1))+
     & G2(2,0)*((-30+nf)/2._dp-9*H1(0)-9*H1(1))+
     & G2(3,0)*((-4*(-30+nf))/9._dp+8*H1(0)+8*H1(1))+
     & H3(0,0,1)*(18-(2*(omv*omv))/(9._dp*(u*u)))+
     & H3(1,1,0)*(4+(4*(omv*omv))/(u*u))+
     & G2(1,2)*(H1(0)*(9-(omv*omv)/(u*u))+H1(1)*(9-(omv*omv)/(u*u))+
     & ((-30+nf)*(omv*omv-9*(u*u)))/(18._dp*(u*u)))+
     & ((5+3*u-5*v+10*NFV*(-2+u+2*v))*w*H3(1,0,0))/(3._dp*(u*u))-
     & (2*H3(1,0,1)*(3*omv*u+2*(omv*omv)-95*(u*u)))/(9._dp*(u*u))+
     & (H3(0,1,0)*(6*omv*(u+15*NFV*u)+(23-60*NFV)*(omv*omv)+
     & (61-30*NFV)*(u*u)))/(9._dp*(u*u))+
     & (H3(0,0,0)*(6*omv*u+omv*omv+99*(u*u)+
     & 30*NFV*(-3*omv*u+2*(omv*omv)+u*u)))/(9._dp*(u*u))+
     & (H2(1,0)*(omu*v*(-32+137*v)*(omw*omw)*(u*u)+
     & 2*v*(131+35*v)*(omu*omu)*(omw*omw)*(u*u)-
     & 2*u*(omw*omw)*(-16*u*(v*v)+
     & 15*NFV*(7-11*v+4*(-7+4*v)*(u*u*u)+7*(u*u*u*u)+
     & u*(-28+35*v-10*(v*v))+4*(v*v)+
     & u*u*(42-40*v+9*(v*v))))+
     & omu*omu*omu*(10*(u*u)-40*omw*(u*u)+
     & omw*omw*(nf*(omv*omv)+(367-45*nf-4*v)*(u*u)+
     & 41*((-1+v)*(-1+v))+6*u*(12-23*v+11*(v*v))))))/
     & (18._dp*(omu*omu*omu)*(omw*omw)*(u*u))+
     & H1(1)*(-8*H2(0,0)-10*H2(1,0)+
     & (6*zeta2*(omu*omu)*(12*omv*u+80*(omv*omv)+1069*(u*u))+
     & 12*u*((7+2*nf)*(omv*omv)+
     & u*(586-1193*u-104*v+118*u*v+600*(u*u)-167*(v*v)+
     & nf*(-60+114*u+24*v-20*u*v-56*(u*u)+6*(v*v)))))/
     & (108._dp*(omu*omu)*(u*u)))+
     & (H2(0,1)*(-18*omu*v*(8+v)*(omw*omw)*(u*u)-
     & 72*(omw*omw)*(u*u)*(v*v)+
     & omu*omu*(-10*(u*u)+40*omw*(u*u)+
     & omw*omw*(-30*omv*NFV*(u*u)-
     & 20*NFV*(omv*omv*omv)*(1+v+v*v)+
     & 10*NFV*(u*u*u)*(2-v+2*(v*v))+
     & (-1+v)*(-1+v)*(2+20*v+9*(v*v))+
     & 5*(omv*omv)*(nf+6*NFV*u*(1+v+2*(v*v)))+
     & 2*u*(-45+55*v-19*(v*v)+9*(v*v*v))+
     & u*u*(658-45*nf-22*v+(9-60*NFV)*(v*v)+
     & 60*NFV*(v*v*v))))))/(18._dp*(omu*omu)*(omw*omw)*(u*u))+
     & (omu*omu*(omw*omw)*(-720*omv*NFV*w*zeta3+
     & 60*NFV*(-4+v)*w*(u*u*u)-
     & 60*NFV*u*w*(-5-18*zeta3+4*v*(1+3*zeta3)+v*v)+
     & u*u*(-2*v*(2016+45*NFV*w+3342*zeta2)-
     & 60*NFV*w*(1+6*zeta3)+(-108+60*NFV*w+426*zeta2)*(v*v)))
     & +omu*u*(omw*omw)*(76*nf*(omv*omv)+
     & u*(-(v*(4536*v+6*(32+551*v)*zeta2))+
     & 4*nf*(-375+264*v+(-499+108*v)*(u*u)+81*(u*u*u)+
     & 54*(v*v)+u*(774-334*v+27*(v*v)))))-
     & omu*omu*omu*(-180*zeta2*(u*u)+720*omw*zeta2*(u*u)+
     & omw*omw*(3*(3*u*(-1+v)*(-135+135*v+8*zeta3)+
     & 4*(-1605+303*v-232*zeta3)*(u*u)+843*(u*u*u)+
     & 44*zeta3*((-1+v)*(-1+v)))+
     & 6*zeta2*(4*nf*(omv*omv)+
     & u*u*(1062-108*nf-38*v+9*(v*v))+
     & (-1+v)*(-1+v)*(-39+20*v+9*(v*v))+
     & u*(-165+254*v-107*(v*v)+18*(v*v*v)))))+
     & 12*zeta2*(omw*omw)*(-5*NFV*w*(u*u*u*u*u)*(2-v+2*(v*v))-
     & 5*NFV*w*(u*u*u*u)*(-28+3*v-9*(v*v)+4*(v*v*v))+
     & 10*NFV*w*(omv+(-1+v)*(v*v*v))-
     & 5*NFV*w*(u*u*u)*(74-32*v+15*(v*v)-14*(v*v*v)+
     & 2*(v*v*v*v))-5*NFV*u*w*
     & (28-18*v+3*(v*v)-10*(v*v*v)+6*(v*v*v*v))+
     & u*u*(370*NFV*w-185*NFV*v*w+(16+55*NFV*w)*(v*v)-
     & 90*NFV*w*(v*v*v)+30*NFV*w*(v*v*v*v))))/
     & (108._dp*(omu*omu*omu)*(omw*omw)*(u*u))+
     & (H2(0,0)*(omv*omv*omv*(-164-20*v+12*nf*(omu*omu*omu)-
     & 9*(v*v)-20*NFV*(1+v*v*v))-
     & 30*NFV*u*(omv*omv)*(-11+6*v+v*v-4*(v*v*v)+2*(v*v*v*v))+
     & omv*u*(406-844*v+423*(v*v)+
     & u*u*u*u*(-2212-18*v+9*(v*v))-12*(v*v*v)+
     & u*u*u*(6722+112*v-43*(v*v)+18*(v*v*v))+
     & u*(1978+872*v-144*nf*(omu*omu*omu)-441*(v*v)+
     & 48*(v*v*v)-27*(v*v*v*v))+27*(v*v*v*v)+
     & u*u*(-6730-430*v+153*(v*v)-52*(v*v*v)+9*(v*v*v*v)))-
     & 10*NFV*(u*u)*(u*u*u*u*(2+3*v-3*(v*v)+2*(v*v*v))+
     & 3*(u*u*u)*(-11+5*v+6*(v*v)-6*(v*v*v)+2*(v*v*v*v))+
     & 3*(u*u)*(38-48*v+5*(v*v)+18*(v*v*v)-11*(v*v*v*v)+
     & 2*(v*v*v*v*v))+
     & 3*(38-82*v+51*(v*v)+6*(v*v*v)-23*(v*v*v*v)+
     & 12*(v*v*v*v*v)-2*(v*v*v*v*v*v))+
     & u*(-166+294*v-132*(v*v)-47*(v*v*v)+69*(v*v*v*v)-
     & 24*(v*v*v*v*v)+2*(v*v*v*v*v*v)))))/
     & (18._dp*omv*(omu*omu*omu)*(u*u))+
     & G1(3)*(H1(0)*((4*(-30+nf))/9._dp-8*H1(1))-8*(H1(0)*H1(0)))+
     & (H1(1)*(-53._dp/2._dp+(omv*omv)/(u*u))+
     & ((57-2*nf*(omu*omu))*(omv*omv)+
     & u*(-97+176*v+(1323-332*v)*(u*u)-653*(u*u*u)-79*(v*v)+
     & 18*u*(-35+15*v+9*(v*v))))/(18._dp*(omu*omu)*(u*u)))*
     & (H1(0)*H1(0))+(-17+(17*(omv*omv))/(18._dp*(u*u)))*(H1(0)*H1(0)*H1(0))+
     & ((3*(omv*omv)+u*(-12+15*u+36*v-30*u*v-
     & 2*nf*((-1+u)*(-1+u))-6*(u*u)+9*(v*v)))*(H1(1)*H1(1)))/
     & (2._dp*u*(omu*omu))+H1(0)*
     & (-8*H2(0,0)-10*H2(1,0)-
     & (H1(1)*(nf*(omu*omu)*(omv*omv+27*(u*u))+
     & 6*(-5*(omv*omv)+
     & u*(4-5*v+3*(-42+29*v)*(u*u)+60*(u*u*u)+
     & u*(67-92*v-32*(v*v))+v*v))))/(18._dp*(omu*omu)*(u*u))
     & +(9*(-281+36*nf-20*NFV*v)*(u*u*u*u*u*u*u)+
     & 3*(u*u*u*u*u*u)*(1192-2055*v+36*nf*(-6+7*v)+
     & NFV*(60+290*v-180*(v*v)))+
     & omv*v*(-534*(-1+v)*zeta2-3*(391-411*v+18*(v*v))+
     & 4*omv*(19*nf+15*NFV*(3+v-12*zeta2+2*(v*v))))+
     & 3*(u*u*u*u*u)*(5459+2606*v+1128*zeta2-1599*(v*v)+
     & 12*nf*(-29-36*v+15*(v*v))-
     & 20*NFV*(15+19*v+6*zeta2-41*(v*v)+11*(v*v*v)))+
     & u*(-1113+4962*v+534*zeta2-2424*v*zeta2-6159*(v*v)+
     & 2958*zeta2*(v*v)+2466*(v*v*v)-1068*zeta2*(v*v*v)-
     & 4*nf*(-19+76*v-89*(v*v)+32*(v*v*v))-
     & 30*omv*NFV*(-6+10*v-7*(v*v)+
     & 12*zeta2*(2-9*v+4*(v*v))+24*(v*v*v)-12*(v*v*v*v))-
     & 162*(v*v*v*v))+
     & 3*(u*u*u*u)*(-11052+6903*v-2352*zeta2+1224*v*zeta2+
     & 2182*(v*v)-405*(v*v*v)+
     & 12*nf*(76-35*v-24*(v*v)+3*(v*v*v))-
     & 10*NFV*(-48+14*v+12*(-5+4*v)*zeta2+127*(v*v)-
     & 93*(v*v*v)+14*(v*v*v*v)))+
     & u*u*(2436+9477*v-1356*zeta2+6918*v*zeta2-60*(v*v)-
     & 2712*zeta2*(v*v)-4350*(v*v*v)+534*zeta2*(v*v*v)+
     & 4*nf*(-38-253*v+44*(v*v)+64*(v*v*v))+162*(v*v*v*v)-
     & 30*NFV*(6+13*v+21*(v*v)-93*(v*v*v)+
     & 12*zeta2*(-7+20*v-14*(v*v)+2*(v*v*v))+
     & 68*(v*v*v*v)-12*(v*v*v*v*v)))+
     & u*u*u*(14409-35628*v+4494*zeta2-8700*v*zeta2+144*(v*v)+
     & 822*zeta2*(v*v)+2466*(v*v*v)-
     & 4*nf*(323-760*v-10*(v*v)+54*(v*v*v))-54*(v*v*v*v)-
     & 30*NFV*(24-52*v-77*(v*v)+12*zeta2*(9-15*v+5*(v*v))+
     & 139*(v*v*v)-52*(v*v*v*v)+4*(v*v*v*v*v))))/
     & (108._dp*omw*(omu*omu)*(u*u))+
     & (-18+(omv*omv)/(2._dp*(u*u)))*(H1(1)*H1(1)))-(9*(H1(1)*H1(1)*H1(1)))/2._dp
     & )+((-3*(omv*omv)+u*(2*nf*((-1+u)*(-1+u))+
     & 3*(4-5*u-12*v+10*u*v+2*(u*u)-3*(v*v))))*(H1(1)*H1(1)*H1(1)))
     & /(6._dp*u*(omu*omu))+H1(0)*
     & ((76._dp/3._dp-(8*nf)/9._dp)*H2(0,0)-(5*(-57+2*nf)*H2(1,0))/9._dp-
     & (18*omv*(28+3*(-39+40*NFV)*v)*zeta2*(omw*omw)+
     & 192*(-63+5*nf)*u*v*(omu*omu)*(omw*omw)-
     & omu*omu*omu*(-36*omw*u*(20+9*nf*u)+
     & 360*(2+9*u+9*v)*(u*u)+
     & omw*omw*(u*(-69180+9*(843+80*NFV)*v-
     & 4*nf*(-1016+243*v+306*zeta2)+17028*zeta3)+
     & 3*v*(76*nf-15*(81+4*NFV*(-5+v*v)))))+
     & 3*omu*(omw*omw)*(4*nf*(19+54*u+27*(u*u))+
     & 3*(-9*(45+82*u+41*(u*u))+
     & 10*NFV*(8-5*u-5*(u*u)+2*(u*u*u))))*(v*v)+
     & 18*u*zeta2*(omw*omw)*
     & (-466-5*(197+84*NFV)*v+(-914+300*NFV)*(v*v)+
     & 50*NFV*(u*u*u*u)*(2-v+2*(v*v))+
     & u*u*u*(382+530*v+275*(v*v)-150*NFV*(2-v+2*(v*v)))+
     & u*u*(-1174-2303*v-839*(v*v)+60*NFV*(5-7*v+5*(v*v)))+
     & u*(1230+2903*v+1393*(v*v)-10*NFV*(10-62*v+37*(v*v)))))/
     & (324._dp*u*(omu*omu*omu)*(omw*omw))+
     & H1(1)*(8*H2(0,0)+10*H2(1,0)+
     & (-6*u*(omu*omu)*(nf*nf)+
     & nf*(4-10*v-2*(345+29*v)*(u*u)+347*(u*u*u)+6*(v*v)+
     & u*(339+68*v+18*(v*v)))-
     & 3*(38-96*v-2*(945+256*v+342*zeta2)*(u*u)+
     & (964+342*zeta2)*(u*u*u)+58*(v*v)+
     & u*(888+608*v+342*zeta2+174*(v*v))))/(36._dp*u*(omu*omu)))
     & +((-2+5*v+(-12-4*nf+29*v)*(u*u)+(5+2*nf)*(u*u*u)+
     & u*(9+2*nf-34*v-9*(v*v))-3*(v*v))*(H1(1)*H1(1)))/
     & (2._dp*u*(omu*omu))+(9*(H1(1)*H1(1)*H1(1)))/2._dp)+
     & G1(1)*(H3(0,0,1)*(-2+(2*(omv*omv))/(9._dp*(u*u)))+
     & H3(1,0,1)*(-10+(10*(omv*omv))/(9._dp*(u*u)))+
     & (H3(1,1,0)*(-6*omv*(u+15*NFV*u)+(-31+60*NFV)*(omv*omv)+
     & 5*(31+6*NFV)*(u*u)))/(9._dp*(u*u))+
     & (H3(0,1,0)*(-6*omv*(u+15*NFV*u)+(-23+60*NFV)*(omv*omv)+
     & (83+30*NFV)*(u*u)))/(9._dp*(u*u))-
     & (H3(0,0,0)*(6*omv*u+omv*omv-81*(u*u)+
     & 30*NFV*(-3*omv*u+2*(omv*omv)+u*u)))/(9._dp*(u*u))-
     & (H3(1,0,0)*(6*omv*u+omv*omv-81*(u*u)+
     & 30*NFV*(-3*omv*u+2*(omv*omv)+u*u)))/(9._dp*(u*u))+
     & (H2(0,0)*(164-30*omv*NFV*u-308*v-4*(3*nf-5*NFV)*(omv*omv)+
     & 143*(v*v)-2*u*(-29+37*v+v*v)-
     & 10*NFV*(u*u*u)*(2-v+2*(v*v))+
     & u*u*(-1830+108*nf+30*NFV+22*v+9*(v*v))))/(18._dp*(u*u))+
     & (H2(1,1)*(-3*(-6+nf*(omu*omu))*(omv*omv)+
     & u*(-21+78*v+(-345+2*v)*(u*u*u)+
     & 3*u*(-119-58*v+9*nf*(omu*omu)-9*(v*v))-57*(v*v)+
     & u*u*(705+130*v+2*(v*v)))))/(9._dp*(omu*omu)*(u*u))+
     & (H2(1,0)*((93-7*nf*(omu*omu))*(omv*omv)+
     & u*(9*u*(-106-30*v+7*nf*(omu*omu)+9*(v*v))-
     & 4*(37-92*v+55*(v*v))+u*u*u*(-971+164*v+80*(v*v))-
     & 2*(u*u)*(-990+38*v+81*(v*v)))))/(18._dp*(omu*omu)*(u*u))+
     & (H2(0,1)*(107-194*v-11*omu*nf*(omv*omv-9*(u*u))+
     & u*(-57+136*v-96*(v*v))+86*(v*v)+
     & u*u*u*(1549+138*v+71*(v*v))-u*u*(1599+80*v+75*(v*v))+
     & 10*NFV*(omu*omu)*(2*(omv*omv)+
     & u*(-1-v+2*(v*v)+u*(2-v+2*(v*v))))))/(18._dp*omu*(u*u))
     & +(-6*zeta3*(omu*omu)*(-6*omv*(u+15*NFV*u)+
     & (-51+60*NFV)*(omv*omv)+5*(67+6*NFV)*(u*u))+
     & 6*zeta2*(49-178*v+u*(-179+500*v-257*(v*v))+133*(v*v)-
     & 2*(u*u*u*u)*(149+59*v+31*(v*v))+
     & 3*(u*u)*(-29-152*v+42*(v*v))+
     & u*u*u*(515+252*v+124*(v*v))-
     & 2*(omu*omu)*(75*omv*NFV*u+(nf-20*NFV)*(omv*omv)+
     & u*u*(-9*nf+5*NFV*(-15+u*(4-2*v+4*(v*v)))))))/
     & (54._dp*(omu*omu)*(u*u))+
     & (H1(1)*(4*omw*(nf*(-19+44*u)*(omv*omv)+
     & u*u*(-60*(v*v)+nf*
     & (143+56*v-54*(-3+2*v)*(u*u*u)-81*(u*u*u*u)+
     & u*u*(93+162*v-27*(v*v))-28*(v*v)+
     & 6*u*(-57-10*v+9*(v*v)))))+
     & 6*omu*omw*(30*NFV*v*w*(u*u*u*u)+
     & 15*NFV*w*(u*u*u)*(-2-3*v+2*(v*v))+
     & 10*NFV*w*(-3+2*v-v*v+2*(v*v*v))-
     & 5*NFV*u*w*(6-v-10*(v*v)+8*(v*v*v))+
     & u*u*(90*NFV*w+4*v*(-102+5*NFV*w)-
     & 2*(37+35*NFV*w)*(v*v)+20*NFV*w*(v*v*v)))+
     & omu*omu*(60*(-1+u)*u+
     & omw*(-6*zeta2*(12*u*(-4+4*v+5*NFV*w)+
     & 5*(-1+v)*(-17+17*v+24*NFV*w)+951*(u*u))+
     & 3*(391-802*v+2*(247+606*v)*(u*u*u)+843*(u*u*u*u)+
     & 429*(v*v)+u*u*(-2818+1010*v+387*(v*v))-
     & 18*(v*v*v)+2*u*(-87+80*v+9*(v*v)+9*(v*v*v)))))))/
     & (108._dp*omw*(omu*omu)*(u*u))+
     & (H1(0)*(9._dp/2._dp-(omv*omv)/(2._dp*(u*u)))+
     & H1(1)*(9._dp/2._dp-(omv*omv)/(2._dp*(u*u))))*(G1(2)*G1(2))+
     & ((35*H1(1)*(9-(omv*omv)/(u*u)))/18._dp+
     & ((-57+2*nf)*(omv*omv-9*(u*u)))/(18._dp*(u*u)))*(H1(0)*H1(0))+
     & (17*(9-(omv*omv)/(u*u))*(H1(0)*H1(0)*H1(0)))/18._dp+
     & ((-30+nf)*(omv*omv-9*(u*u))*(H1(1)*H1(1)))/(18._dp*(u*u))+
     & H1(0)*(((-29+nf)*H1(1)*(omv*omv-9*(u*u)))/(6._dp*(u*u))+
     & (4*omw*(nf*(-19+44*u)*(omv*omv)+
     & u*u*(-60*(v*v)+
     & nf*(143+56*v-54*(-3+2*v)*(u*u*u)-81*(u*u*u*u)+
     & u*u*(93+162*v-27*(v*v))-28*(v*v)+
     & 6*u*(-57-10*v+9*(v*v)))))+
     & 6*omu*omw*(30*NFV*v*w*(u*u*u*u)+
     & 15*NFV*w*(u*u*u)*(-2-3*v+2*(v*v))+
     & 10*NFV*w*(-3+2*v-v*v+2*(v*v*v))-
     & 5*NFV*u*w*(6-v-10*(v*v)+8*(v*v*v))+
     & u*u*(90*NFV*w+4*v*(-102+5*NFV*w)-
     & 2*(37+35*NFV*w)*(v*v)+20*NFV*w*(v*v*v)))-
     & 3*(omu*omu)*(-20*(-1+u)*u+
     & omw*(-391+802*v+
     & 6*zeta2*(4*u*(-4+4*v+5*NFV*w)+
     & (-1+v)*(-39+39*v+40*NFV*w)+221*(u*u))-
     & 2*(247+606*v)*(u*u*u)-843*(u*u*u*u)+
     & u*u*(2818-1010*v-387*(v*v))-429*(v*v)+18*(v*v*v)-
     & 2*u*(-87+80*v+9*(v*v)+9*(v*v*v)))))/
     & (108._dp*omw*(omu*omu)*(u*u))+
     & (3*(9-(omv*omv)/(u*u))*(H1(1)*H1(1)))/2._dp)+
     & G1(2)*(H1(0)*(H1(1)*(-18+(2*(omv*omv))/(u*u))-
     & ((-30+nf)*(omv*omv-9*(u*u)))/(18._dp*(u*u)))-
     & ((-30+nf)*H1(1)*(omv*omv-9*(u*u)))/(18._dp*(u*u))+
     & (-9+(omv*omv)/(u*u))*(H1(0)*H1(0))+
     & (-9+(omv*omv)/(u*u))*(H1(1)*H1(1)))+
     & (9._dp/2._dp-(omv*omv)/(2._dp*(u*u)))*(H1(1)*H1(1)*H1(1)))+
     & (9*(H1(1)*H1(1)*H1(1)*H1(1)))/8._dp
      return
      end
      
      
      function Alpha_2a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-16*pi*G3(0,0,1))/9._dp+18*pi*G3(0,0,2)-16*pi*G3(0,0,3)+18*pi*G3(0,2,0)-
     & (92*pi*G3(0,2,1))/9._dp+18*pi*G3(0,2,2)-28*pi*G3(0,2,3)-8*pi*G3(0,3,0)-
     & 8*pi*G3(0,3,2)+(128*pi*G3(0,3,3))/9._dp+18*pi*G3(2,0,0)+18*pi*G3(2,0,2)+
     & 18*pi*G3(2,2,0)+16*pi*G3(2,3,2)-16*pi*G3(3,0,0)+10*pi*G3(3,0,1)-
     & 16*pi*G3(3,0,2)+(208*pi*G3(3,0,3))/9._dp-16*pi*G3(3,2,0)+10*pi*G3(3,2,1)+
     & (80*pi*G3(3,2,3))/9._dp+(128*pi*G3(3,3,0))/9._dp-(128*pi*G3(3,3,3))/9._dp+
     & G2(0,0)*(-3*(-21+nf)*pi-16*pi*H1(0)-18*pi*H1(1))-
     & (64*pi*H3(0,0,0))/3._dp-8*pi*H3(0,0,1)-(62*pi*H3(0,1,0))/9._dp-
     & 8*pi*H3(1,0,0)-10*pi*H3(1,0,1)+20*pi*H3(1,1,0)+
     & 2*pi*G3(0,1,0)*(9-(omv*omv)/(u*u))+
     & (10*pi*G3(1,0,1)*(9-(omv*omv)/(u*u)))/9._dp+
     & (16*pi*G3(1,1,2)*(9-(omv*omv)/(u*u)))/9._dp+
     & (16*pi*G3(2,1,1)*(9-(omv*omv)/(u*u)))/9._dp+
     & pi*G3(0,1,1)*(-169._dp/9._dp+(omv*omv)/(u*u))+
     & pi*G3(2,1,0)*(-9+(omv*omv)/(u*u))+
     & pi*G3(2,1,2)*(-9+(omv*omv)/(u*u))+
     & (20*pi*G3(2,3,3)*(-9+(omv*omv)/(u*u)))/9._dp+
     & G3(0,1,2)*(pi-(pi*(omv*omv))/(u*u))+
     & (2*pi*G3(2,2,1)*(3*omv*u+11*(omv*omv)-86*(u*u)))/(9._dp*(u*u))+
     & (2*pi*G3(1,0,2)*(3*omv*u+2*(omv*omv)-77*(u*u)))/(9._dp*(u*u))+
     & (2*pi*G3(1,2,0)*(3*omv*u+2*(omv*omv)-77*(u*u)))/(9._dp*(u*u))+
     & (2*pi*G3(1,2,2)*(3*omv*u+2*(omv*omv)-77*(u*u)))/(9._dp*(u*u))+
     & (2*pi*G3(1,1,0)*(3*omv*u+2*(omv*omv)-5*(u*u)))/(9._dp*(u*u))-
     & (2*pi*G3(1,1,1)*(3*omv*u-6*(omv*omv)+67*(u*u)))/(9._dp*(u*u))-
     & (2*pi*G3(2,0,1)*(3*omv*u+2*(omv*omv)+121*(u*u)))/(9._dp*(u*u))+
     & (2*pi*G3(1,2,1)*(-3*omv*u-15*(omv*omv)+122*(u*u)))/(9._dp*(u*u))-
     & (pi*G3(1,0,3)*(omv*(2-30*NFV)*u+(1+20*NFV)*(omv*omv)+
     & (-33+10*NFV)*(u*u)))/(3._dp*(u*u))-
     & (pi*G3(1,2,3)*(omv*(2-30*NFV)*u+(1+20*NFV)*(omv*omv)+
     & (-33+10*NFV)*(u*u)))/(3._dp*(u*u))+
     & (pi*G3(2,0,3)*(omv*(6-90*NFV)*u+(-17+60*NFV)*(omv*omv)+
     & 3*(-27+10*NFV)*(u*u)))/(9._dp*(u*u))-
     & (pi*G3(2,2,3)*(omv*(6-90*NFV)*u+(-17+60*NFV)*(omv*omv)+
     & 3*(9+10*NFV)*(u*u)))/(9._dp*(u*u))+
     & (pi*H2(1,1)*(4*(omv*omv)+u*
     & (324+168*v-27*nf*((-1+u)*(-1+u))-2*(-166+v)*(u*u)+
     & 75*(v*v)-2*u*(330+79*v+v*v))))/(9._dp*u*(omu*omu))+
     & G2(1,2)*((2*pi*H1(1)*(-3*omv*u-2*(omv*omv)+77*(u*u)))/(9._dp*(u*u))-
     & (pi*H1(0)*(omv*(2-30*NFV)*u+5*(-1+4*NFV)*(omv*omv)+
     & (-27+10*NFV)*(u*u)))/(3._dp*(u*u))-
     & (pi*(3-26*v+omu*omu*
     & (-60*omv*NFV*u+(nf+20*NFV)*(omv*omv)+
     & u*u*(-9*nf+10*NFV*(6+u*(-2+v-2*(v*v)))))+
     & u*u*u*u*(53-142*v-71*(v*v))+24*(v*v)-
     & 4*u*(11-19*v+4*(v*v))+u*u*(132+40*v+49*(v*v))+
     & 2*(u*u*u)*(-72+26*v+71*(v*v))))/(18._dp*(omu*omu)*(u*u)))+
     & (pi*H2(0,1)*(2*omu*u*(82-3*v)*v+82*u*(v*v)+
     & omu*omu*(-10*(-1+v)*v+10*NFV*(u*u)*(2-v+2*(v*v))+
     & u*(915-62*nf+138*v+71*(v*v)))))/(18._dp*u*(omu*omu))-
     & (pi*H2(0,0)*(2*omv*(-17+60*NFV)*v+
     & 10*NFV*(u*u*u*u*u)*(2-v+2*(v*v))+
     & 2*u*(-516+32*nf-(113+210*NFV)*v+2*(-46+75*NFV)*(v*v))+
     & u*u*u*(-3096+192*nf-538*v-217*(v*v)+60*NFV*(1-5*v+v*v))+
     & u*u*u*u*(1032-64*nf+138*v+71*(v*v)-
     & 30*NFV*(2-v+2*(v*v)))+
     & u*u*(-192*nf-10*NFV*(2-58*v+29*(v*v))+
     & 4*(774+165*v+82*(v*v)))))/(18._dp*u*(omu*omu*omu))+
     & G2(0,1)*(pi*H1(0)*(-11._dp/9._dp+(omv*omv)/(u*u))+
     & pi*H1(1)*(-1+(omv*omv)/(u*u))+
     & (pi*(nf*(45-(omv*omv)/(u*u))+
     & 2*(-457+20/omw+(82-80/omu)*v-5/(omw*omw)-
     & (8*(omv*omv))/(u*u)+(6+5*v-11*(v*v))/u+
     & (10*(-1-7*u+4*(u*u))*(v*v))/(omu*omu))))/18._dp)+
     & G2(2,2)*(-(pi*H1(0)*(30*NFV*(-3*omv*u+2*(omv*omv)+u*u)+
     & (omv+3*u)*(omv+3*u)))/(9._dp*(u*u))+
     & (pi*(4*(omv*omv)+u*(324+168*v-27*nf*((-1+u)*(-1+u))-
     & 2*(-166+v)*(u*u)+75*(v*v)-2*u*(330+79*v+v*v))))/
     & (9._dp*u*(omu*omu)))+G2(0,2)*
     & (-18*pi*H1(1)+pi*H1(0)*(-232._dp/9._dp-(omv*omv)/(u*u))+
     & (pi*(4*(omv*omv)+u*(432+168*v-27*nf*((-1+u)*(-1+u))-
     & 2*(-220+v)*(u*u)+75*(v*v)-2*u*(438+79*v+v*v))))/
     & (9._dp*u*(omu*omu)))+G2(2,0)*
     & (-18*pi*H1(1)-(pi*H1(0)*(-6*omv*u+5*(omv*omv)+82*(u*u)))/
     & (9._dp*(u*u))+(pi*(4*(omv*omv)+
     & u*(432+168*v-27*nf*((-1+u)*(-1+u))-2*(-220+v)*(u*u)+
     & 75*(v*v)-2*u*(438+79*v+v*v))))/(9._dp*u*(omu*omu)))+
     & G2(2,3)*(-18*pi*H1(0)-16*pi*H1(1)-
     & (pi*(-28+22*v+(-622+44*nf+4*v)*(u*u*u*u)+
     & u*u*(-1950+132*nf-262*v-73*(v*v))+6*(v*v)+
     & 2*(u*u*u)*(947-66*nf+60*v+2*(v*v))+
     & u*(706-44*nf+116*v+31*(v*v))+
     & 30*NFV*((-32+17*v)*(u*u*u)+8*(u*u*u*u)+4*(2-3*v+v*v)-
     & 2*u*(16-19*v+5*(v*v))+u*u*(48-43*v+9*(v*v)))))/
     & (18._dp*u*(omu*omu*omu)))+
     & G2(1,1)*((16*pi*H1(1)*(-9+(omv*omv)/(u*u)))/9._dp+
     & (pi*(14-8*v-4*nf*(omu*omu)*(omv*omv-9*(u*u))-7*(v*v)-
     & 6*(u*u)*(98-9*v+10*(v*v))+2*u*(-8-19*v+19*(v*v))-
     & u*u*u*u*(578+302*v+151*(v*v))+
     & 2*(u*u*u)*(584+147*v+154*(v*v))+
     & 10*NFV*(omu*omu*omu)*
     & (2*(omv*omv)+u*(-1-v+2*(v*v)+u*(2-v+2*(v*v))))))/
     & (18._dp*(omu*omu)*(u*u)))-
     & (pi*H2(1,0)*(-(omu*u*v*(32+81*v)*(omw*omw))+
     & 2*u*v*(-87+14*v)*(omu*omu)*(omw*omw)+
     & 2*(omu*omu*omu)*(5*u-20*omw*u+
     & omw*omw*(3-19*v+u*(-735+49*nf+2*v)+16*(v*v)))-
     & 2*(omw*omw)*(60*NFV*(omv*omv)+
     & u*(-16*(v*v)+15*NFV*
     & ((-16+13*v)*(u*u)+4*(u*u*u)-2*(8-13*v+5*(v*v))+
     & u*(24-31*v+9*(v*v)))))))/
     & (18._dp*u*(omu*omu*omu)*(omw*omw))+
     & G2(0,3)*((136*pi*H1(0))/9._dp+8*pi*H1(1)+
     & (pi*(2*omv*(-20+(3+60*NFV)*v)+
     & u*(-8*nf*((-1+u)*(-1+u)*(-1+u))+
     & 10*NFV*(u*u*u*u)*(2-v+2*(v*v))+
     & 2*(-58-21*(11+10*NFV)*v+3*(-17+50*NFV)*(v*v))+
     & u*u*(-668-614*v-217*(v*v)+60*NFV*(1-5*v+v*v))+
     & u*u*u*(236+138*v+71*(v*v)-30*NFV*(2-v+2*(v*v)))+
     & u*(588+892*v+286*(v*v)-10*NFV*(2-58*v+29*(v*v))))))/
     & (18._dp*u*(omu*omu*omu)))+
     & (pi*(-12*omu*u*v*(-2*(-567+nf*(46+4*u))*v+6*(-16+57*v)*zeta2)*
     & (omw*omw)+3*(omu*omu)*(omw*omw)*
     & (-(u*v*(-416*nf+12*(397+9*v)+6*(432+7*v)*zeta2))+
     & 30*omv*NFV*(2*(-4+v)*(u*u)-2*(-5+4*v+v*v)+
     & u*(-2-3*v+2*(v*v))))+
     & 36*zeta2*(omw*omw)*(120*NFV*(omv*omv)+
     & u*(-32*(v*v)+5*NFV*
     & (3*(u*u*u)*(6+v-2*(v*v))+u*u*u*u*(2-v+2*(v*v))-
     & 12*(8-13*v+5*(v*v))+u*u*(-90+75*v+6*(v*v))+
     & u*(142-185*v+52*(v*v)))))+
     & omu*omu*omu*(36*omw*u*(20+9*(-10+nf)*u)-720*(u*u)+
     & omw*omw*(-3645+7815*u-664*nf*u+7290*v-7587*u*v+
     & 972*nf*u*v+4716*u*zeta3+228*nf*(omv*omv)-3645*(v*v)+
     & 6*zeta2*(57-66*v+9*(v*v)-3*u*(-53+298*v+151*(v*v))))))
     & )/(324._dp*u*(omu*omu*omu)*(omw*omw))+
     & G2(1,0)*((pi*H1(0)*(-6*omv*u+5*(omv*omv)+73*(u*u)))/(9._dp*(u*u))+
     & (2*pi*H1(1)*(-3*omv*u-2*(omv*omv)+77*(u*u)))/(9._dp*(u*u))+
     & (pi*(63-106*v+u*u*u*(281-138*v-71*(v*v))+
     & u*(-55+70*v-32*(v*v))+42*(v*v)+
     & u*u*(-289+174*v+75*(v*v))+
     & omu*(60*omv*NFV*u-(nf+20*NFV)*(omv*omv)+
     & u*u*(9*nf+10*NFV*(-6+u*(2-v+2*(v*v)))))))/
     & (18._dp*omu*(u*u)))+G2(3,2)*
     & ((62*pi*H1(0))/9._dp-(pi*(10*NFV*(u*u*u)*(2-v+2*(v*v))+
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+20*NFV*(v*v*v))+
     & 2*u*v*(10+5*(-2+3*NFV)*v+(9-45*NFV)*(v*v)+
     & 30*NFV*(v*v*v))+
     & u*u*(24-8*nf+(-22+30*NFV)*v+(9-60*NFV)*(v*v)+
     & 60*NFV*(v*v*v))))/(18._dp*(u*u)))+
     & G2(3,0)*((208*pi*H1(0))/9._dp+16*pi*H1(1)-
     & (pi*(10*NFV*(u*u*u)*(2-v+2*(v*v))+
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+20*NFV*(v*v*v))+
     & 2*u*v*(10+5*(-2+3*NFV)*v+(9-45*NFV)*(v*v)+
     & 30*NFV*(v*v*v))+
     & u*u*(216-8*nf+(-22+30*NFV)*v+(9-60*NFV)*(v*v)+
     & 60*NFV*(v*v*v))))/(18._dp*(u*u)))+
     & G2(2,1)*(pi*H1(1)*(9-(omv*omv)/(u*u))+
     & H1(0)*(pi-(pi*(omv*omv))/(9._dp*(u*u)))-
     & (pi*(16*omu*v*(25+4*v)*(omw*omw)*(u*u)+
     & 200*(omw*omw)*(u*u)*(v*v)+
     & omu*omu*(10*(u*u)-40*omw*(u*u)+
     & omw*omw*(30*omv*NFV*(u*u)+
     & 20*NFV*(omv*omv*omv)*(1+v+v*v)-
     & 10*NFV*(u*u*u)*(2-v+2*(v*v))-
     & (-1+v)*(-1+v)*(38+20*v+9*(v*v))+
     & omv*omv*(nf-30*NFV*u*(1+v+2*(v*v)))-
     & 2*u*(-30+61*v-40*(v*v)+9*(v*v*v))+
     & u*u*(32-9*nf+18*v+(-9+60*NFV)*(v*v)-60*NFV*(v*v*v))
     & ))))/(18._dp*(omu*omu)*(omw*omw)*(u*u)))+
     & G2(3,3)*((-128*pi*H1(0))/9._dp+(pi*
     & (10*NFV*(u*u*u)*(2-v+2*(v*v))+
     & u*u*(-532+32*nf+(-22+30*NFV)*v+(9-60*NFV)*(v*v)+
     & 60*NFV*(v*v*v))+
     & v*(40-52*v+(2+20*NFV)*(v*v)+(9-40*NFV)*(v*v*v)+
     & 20*NFV*(v*v*v*v))+
     & 2*u*(20-28*v+5*(-2+3*NFV)*(v*v)+(9-45*NFV)*(v*v*v)+
     & 30*NFV*(v*v*v*v))))/(18._dp*(u*u)))+
     & (pi*H1(1)*(4*(298-303*v+18*nf*(-3+2*v))*(u*u*u*u)+
     & 3*(-281+36*nf)*(u*u*u*u*u)+
     & u*u*u*(2784+1414*v+2250*zeta2-387*(v*v)+
     & 12*nf*(-10-18*v+3*(v*v)))+
     & 2*(59-128*v+78*(v*v)-9*(v*v*v))-
     & 2*(u*u)*(2827-174*v+2250*zeta2-375*(v*v)+
     & 12*nf*(-19+3*(v*v))+9*(v*v*v))+
     & u*(2403-294*v+2250*zeta2-883*(v*v)+
     & 12*nf*(-19+6*v+3*(v*v))+36*(v*v*v))+
     & 10*omu*NFV*(4*(1+v)*(omv*omv*omv)+
     & u*(6*v*(u*u*u)+3*(u*u)*(-2-5*v+4*(v*v))+
     & u*(16+5*v-25*(v*v)+10*(v*v*v))+
     & 2*(-7+6*v+8*(v*v)-9*(v*v*v)+2*(v*v*v*v))))))/
     & (36._dp*u*(omu*omu))+((9*pi*G1(2))/2._dp-4*pi*G1(3)-(27*pi*H1(0))/2._dp-
     & (9*pi*H1(1))/2._dp+(pi*G1(1)*(omv*omv-9*(u*u)))/(2._dp*(u*u))+
     & (pi*(omv*omv+u*(-33+63*u+12*v-10*u*v+
     & nf*((-1+u)*(-1+u))-31*(u*u)+3*(v*v))))/
     & (2._dp*u*(omu*omu)))*(G1(0)*G1(0))+(9*pi*(G1(0)*G1(0)*G1(0)))/2._dp+
     & (-4*pi*G1(3)-(27*pi*H1(0))/2._dp-(27*pi*H1(1))/2._dp+
     & (pi*(omv*omv+u*(-33+63*u+12*v-10*u*v+
     & nf*((-1+u)*(-1+u))-31*(u*u)+3*(v*v))))/
     & (2._dp*u*(omu*omu)))*(G1(2)*G1(2))+(9*pi*(G1(2)*G1(2)*G1(2)))/2._dp+
     & ((-35*pi*H1(1))/2._dp+(pi*(17-36*v+(945-28*nf-188*v)*(u*u)+
     & 2*(-232+7*nf)*(u*u*u)+19*(v*v)+
     & u*(-498+14*nf+224*v+57*(v*v))))/(18._dp*u*(omu*omu)))*
     & (H1(0)*H1(0))-(17*pi*(H1(0)*H1(0)*H1(0)))/2._dp+
     & (pi*(omv*omv+u*(-33+63*u+12*v-10*u*v+nf*((-1+u)*(-1+u))-
     & 31*(u*u)+3*(v*v)))*(H1(1)*H1(1)))/(2._dp*u*(omu*omu))+
     & H1(0)*((-8*pi*H2(0,0))/9._dp-(10*pi*H2(1,0))/9._dp+
     & (pi*H1(1)*(20*(omv*omv)+
     & u*(-837+1614*u+240*v-200*u*v+26*nf*((-1+u)*(-1+u))-
     & 797*(u*u)+60*(v*v))))/(18._dp*u*(omu*omu))+
     & (pi*(4*omw*u*v*w*(16+5*NFV*(7-17*u+u*u)+
     & v*(-111+nf*(19-20*u+9*(u*u))))-
     & 2*omu*omw*u*w*(16+(377-32*nf)*v+15*NFV*(-3+v*v)+
     & 22*(v*v))+omu*omu*
     & (20*u*w-omw*(3*(281-36*nf+20*NFV*v)*w*(u*u*u)+
     & 2*(u*u)*(7+(247+606*v-72*nf*v)*w+
     & 15*NFV*(-3+w*(-2-3*v+4*(v*v))))+
     & 2*w*(52-25*v-37*(v*v)-
     & 4*(omv*omv)*(nf-5*NFV*(v*v))+9*(v*v*v))+
     & u*w*(-2796+1010*v-8*nf*(-28+9*v)-810*zeta2+
     & 387*(v*v)+10*NFV*(33-17*(v*v)+10*(v*v*v)))))))/
     & (36._dp*omw*u*w*(omu*omu))-(27*pi*(H1(1)*H1(1)))/2._dp)+
     & G1(3)*((-64*pi*H2(0,0))/3._dp-8*pi*H2(0,1)-(62*pi*H2(1,0))/9._dp+
     & (pi*H1(1)*(10*NFV*(u*u*u)*(2-v+2*(v*v))+
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+20*NFV*(v*v*v))+
     & 2*u*v*(10+5*(-2+3*NFV)*v+(9-45*NFV)*(v*v)+
     & 30*NFV*(v*v*v))+
     & u*u*(-216+(-22+30*NFV)*v+(9-60*NFV)*(v*v)+60*NFV*(v*v*v))
     & ))/(18._dp*(u*u))+H1(0)*
     & (-8*pi*H1(1)+(pi*(10*NFV*(u*u*u)*(2-v+2*(v*v))+
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+
     & 20*NFV*(v*v*v))+
     & 2*u*v*(10+5*(-2+3*NFV)*v+(9-45*NFV)*(v*v)+
     & 30*NFV*(v*v*v))+
     & u*u*(-216+(-22+30*NFV)*v+(9-60*NFV)*(v*v)+
     & 60*NFV*(v*v*v))))/(18._dp*(u*u)))+
     & (pi*(-6*omu*u*w*(16+103*v+55*(v*v))+
     & omu*omu*(9*(-281+36*nf-20*NFV*v)*w*(u*u*u)+
     & u*w*(7668-3030*v+2160*zeta2-1161*(v*v)+
     & 4*nf*(-152+54*v+27*(v*v)))-
     & 6*(u*u)*(7+(247+606*v-72*nf*v)*w+
     & 15*NFV*(-3+w*(-2-3*v+4*(v*v))))-
     & 6*v*w*(-6-4*v+9*(v*v)+20*NFV*(6-5*v-2*(v*v)+v*v*v)))
     & -6*u*w*(-32*v*(1+v)+
     & 5*NFV*(24-32*v-20*(v*v)+
     & u*(-57+52*v+55*(v*v)-20*(v*v*v))+10*(v*v*v)+
     & u*u*(33-2*v-17*(v*v)+10*(v*v*v))))))/
     & (108._dp*u*w*(omu*omu))-(76*pi*(H1(0)*H1(0)))/9._dp-4*pi*(H1(1)*H1(1)))+
     & G1(0)*(G1(3)*((-4*(-30+nf)*pi)/9._dp+8*pi*H1(0))+
     & G1(2)*((-30+nf)*pi-18*pi*H1(0)-9*pi*H1(1))+(200*pi*H2(0,0))/9._dp+
     & 16*pi*H2(0,1)+26*pi*H2(1,0)+18*pi*H2(1,1)+
     & G1(1)*(pi*H1(0)*(9-(omv*omv)/(u*u))+
     & ((-30+nf)*pi*(omv*omv-9*(u*u)))/(18._dp*(u*u)))+
     & (pi*H1(1)*(-4*(omv*omv)+
     & u*(18*nf*((-1+u)*(-1+u))+2*(-85+v)*(u*u)+
     & 2*u*(168+79*v+v*v)-3*(54+56*v+25*(v*v)))))/
     & (9._dp*u*(omu*omu))+H1(0)*
     & (19*pi*H1(1)-(pi*(18-26*v+8*(v*v)+
     & 10*NFV*(u*u*u*u)*(2-v+2*(v*v))+
     & u*(300-36*nf+498*v+221*(v*v))+
     & u*u*u*(336-36*nf+138*v+71*(v*v)-
     & 20*NFV*(2-v+2*(v*v)))+
     & 2*(u*u)*(-327+36*nf-305*v-73*(v*v)+
     & 5*NFV*(2-v+2*(v*v)))))/(18._dp*u*(omu*omu)))-
     & (pi*(6-32*v+4*(298-303*v+18*nf*(-3+2*v))*(u*u*u*u)+
     & 3*(-281+36*nf)*(u*u*u*u*u)+44*(v*v)+
     & u*u*u*(2880+1414*v+810*zeta2-387*(v*v)+
     & 12*nf*(-10-18*v+3*(v*v)))-18*(v*v*v)-
     & 2*(u*u)*(2979-430*v+810*zeta2-375*(v*v)+
     & 12*nf*(-19+3*(v*v))+9*(v*v*v))+
     & u*(2723-1030*v+810*zeta2-467*(v*v)+
     & 12*nf*(-19+6*v+3*(v*v))+36*(v*v*v))+
     & 10*omu*NFV*(4*(1+v)*(omv*omv*omv)+
     & u*(6*v*(u*u*u)+3*(u*u)*(-2-5*v+4*(v*v))+
     & u*(16+5*v-25*(v*v)+10*(v*v*v))+
     & 2*(-7+6*v+8*(v*v)-9*(v*v*v)+2*(v*v*v*v))))))/
     & (36._dp*u*(omu*omu))+(9*pi*(G1(2)*G1(2)))/2._dp+
     & (331*pi*(H1(0)*H1(0)))/18._dp+(9*pi*(H1(1)*H1(1)))/2._dp)+
     & G1(2)*(G1(3)*((-4*(-30+nf)*pi)/9._dp+8*pi*H1(0)+8*pi*H1(1))-
     & (pi*H2(1,0)*(-6*omv*u-13*(omv*omv)+u*u))/(9._dp*(u*u))-
     & (pi*H2(0,1)*(6*omv*u-5*(omv*omv)+71*(u*u)))/(9._dp*(u*u))-
     & (pi*H2(0,0)*(omv*(6-90*NFV)*u+(-17+60*NFV)*(omv*omv)+
     & 3*(33+10*NFV)*(u*u)))/(9._dp*(u*u))+
     & (pi*H1(1)*(-13*(omv*omv)+
     & u*(18*nf*((-1+u)*(-1+u))+(-53+2*v)*(u*u)+
     & u*(93+248*v+2*(v*v))-3*(9+92*v+34*(v*v)))))/
     & (9._dp*u*(omu*omu))+H1(0)*
     & (27*pi*H1(1)+(pi*(-(omu*v*(32+161*v)*(omw*omw)*(u*u))+
     & 2*v*(-167+4*v)*(omu*omu)*(omw*omw)*(u*u)+
     & omu*omu*omu*(10*(u*u)-40*omw*(u*u)+
     & omw*omw*((-737+72*nf+4*v)*(u*u)-
     & 41*((-1+v)*(-1+v))+2*u*(53-59*v+6*(v*v))))-
     & 2*u*(omw*omw)*(-16*u*(v*v)+
     & 15*NFV*(7-11*v+4*(-7+4*v)*(u*u*u)+7*(u*u*u*u)+
     & u*(-28+35*v-10*(v*v))+4*(v*v)+
     & u*u*(42-40*v+9*(v*v))))))/
     & (18._dp*(omu*omu*omu)*(omw*omw)*(u*u)))-
     & (pi*(6*zeta2*(omu*omu)*(-24*omv*u-16*(omv*omv)+445*(u*u))+
     & 3*u*(4*(298-303*v+18*nf*(-3+2*v))*(u*u*u*u)+
     & 3*(-281+36*nf)*(u*u*u*u*u)+
     & u*u*u*(2784+1414*v-387*(v*v)+
     & 12*nf*(-10-18*v+3*(v*v)))+
     & u*u*(-5654+348*v+nf*(456-72*(v*v))+750*(v*v)-
     & 18*(v*v*v))+2*(59-128*v+78*(v*v)-9*(v*v*v))+
     & u*(2403-294*v-883*(v*v)+12*nf*(-19+6*v+3*(v*v))+
     & 36*(v*v*v))+10*omu*NFV*
     & (4*(1+v)*(omv*omv*omv)+
     & u*(6*v*(u*u*u)+3*(u*u)*(-2-5*v+4*(v*v))+
     & u*(16+5*v-25*(v*v)+10*(v*v*v))+
     & 2*(-7+6*v+8*(v*v)-9*(v*v*v)+2*(v*v*v*v)))))))/
     & (108._dp*(omu*omu)*(u*u))+
     & pi*(37._dp/2._dp-(omv*omv)/(9._dp*(u*u)))*(H1(0)*H1(0))+
     & (27*pi*(H1(1)*H1(1)))/2._dp)+
     & G1(1)*(G1(2)*(pi*H1(0)*(9-(omv*omv)/(u*u))+
     & pi*H1(1)*(9-(omv*omv)/(u*u))+
     & ((-30+nf)*pi*(omv*omv-9*(u*u)))/(18._dp*(u*u)))+
     & (2*pi*H2(1,1)*(3*omv*u+2*(omv*omv)-77*(u*u)))/(9._dp*(u*u))-
     & (pi*H2(0,1)*(-6*omv*u+5*(omv*omv)+73*(u*u)))/(9._dp*(u*u))+
     & (2*pi*H2(1,0)*(omv*(3-45*NFV)*u+(-8+30*NFV)*(omv*omv)+
     & 3*(-12+5*NFV)*(u*u)))/(9._dp*(u*u))+
     & (pi*H2(0,0)*(omv*(6-90*NFV)*u+(-17+60*NFV)*(omv*omv)+
     & 3*(-21+10*NFV)*(u*u)))/(9._dp*(u*u))+
     & H1(0)*((10*pi*H1(1)*(-9+(omv*omv)/(u*u)))/9._dp+
     & (pi*(-11-30*omv*NFV*u-18*v+u*(-58+94*v)+
     & 4*nf*(omv*omv)+(399-36*nf+30*NFV)*(u*u)+31*(v*v)))/
     & (18._dp*(u*u)))+(pi*H1(1)*
     & (33-86*v+10*NFV*(omu*omu)*
     & (-6*omv*u+2*(omv*omv)+u*u*(6+u*(-2+v-2*(v*v))))+
     & 54*(v*v)-4*u*(26-49*v+19*(v*v))+
     & 2*(u*u*u)*(198+26*v+71*(v*v))-
     & u*u*u*u*(217+142*v+71*(v*v))+u*u*(-108-20*v+79*(v*v))))/
     & (18._dp*(omu*omu)*(u*u))+
     & (pi*(9*(-281+36*nf-20*NFV*v)*(u*u*u*u*u*u*u)+
     & 3*(u*u*u*u*u*u)*(1192-2055*v+36*nf*(-6+7*v)-
     & 30*NFV*(-2-7*v+6*(v*v)))+
     & omv*v*(-3*(391-411*v+18*(v*v))+
     & omv*(76*nf+3*(6*zeta2+20*NFV*(3+v+24*zeta2+2*(v*v))))
     & )+u*u*u*u*u*(8889+7818*v+2046*zeta2-4797*(v*v)+
     & 12*nf*(-31-108*v+45*(v*v))+
     & 60*NFV*(-15-8*v+12*zeta2+31*(v*v)-11*(v*v*v)))+
     & u*u*u*u*(-17928+8385*v-4236*zeta2+2190*v*zeta2+
     & 6066*(v*v)-1215*(v*v*v)+
     & 12*nf*(114-11*v-72*(v*v)+9*(v*v*v))+
     & 30*NFV*(48-38*v+24*(-5+4*v)*zeta2-79*(v*v)+
     & 77*(v*v*v)-14*(v*v*v*v)))+
     & u*(4*nf*(19-44*v)*(omv*omv)+
     & 30*omv*NFV*(6-10*v+3*(v*v)+
     & 24*zeta2*(2-9*v+4*(v*v))-20*(v*v*v)+12*(v*v*v*v))-
     & 3*(371-1758*v+2193*(v*v)-858*(v*v*v)+
     & 6*zeta2*(-1+12*v-13*(v*v)+2*(v*v*v))+54*(v*v*v*v)))
     & +u*u*u*(6417-15084*v+2352*zeta2-4560*v*zeta2-
     & 2160*(v*v)+162*zeta2*(v*v)+1986*(v*v*v)-
     & 4*nf*(143-286*v-88*(v*v)+54*(v*v*v))-54*(v*v*v*v)+
     & 30*NFV*(-24+66*v+31*(v*v)+24*zeta2*(9-15*v+5*(v*v))-
     & 105*(v*v*v)+48*(v*v*v*v)-4*(v*v*v*v*v)))+
     & u*u*(2688+945*v-180*zeta2+2568*v*zeta2+5304*(v*v)-
     & 360*zeta2*(v*v)-1818*(v*v*v)+18*zeta2*(v*v*v)+
     & 4*nf*(-44-55*v-100*(v*v)+28*(v*v*v))+162*(v*v*v*v)+
     & 30*NFV*(-6-17*v+v*v+67*(v*v*v)+
     & 24*zeta2*(-7+20*v-14*(v*v)+2*(v*v*v))-60*(v*v*v*v)+
     & 12*(v*v*v*v*v)))))/(108._dp*omw*(omu*omu)*(u*u))+
     & (pi*(omv*omv-9*(u*u))*(G1(2)*G1(2)))/(2._dp*(u*u))+
     & (19*pi*(omv*omv-9*(u*u))*(H1(0)*H1(0)))/(18._dp*(u*u))+
     & (pi*(omv*omv-9*(u*u))*(H1(1)*H1(1)))/(2._dp*(u*u)))-
     & (9*pi*(H1(1)*H1(1)*H1(1)))/2._dp
      return
      end
      
      
      function Beta_2a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=1._dp
      return
      end
      
      
      function Beta_2a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0._dp
      return
      end
      
      
      function Beta_2a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=-(23*omv+u)/(3._dp*omv)+(27*zeta2)/2._dp-3*G2(0,2)-3*G2(2,0)+
     & (8*G2(3,0))/3._dp-(8*G1(3)*H1(0))/3._dp+
     & ((omv+u*(-14+13*u-3*v))*H1(1))/(3._dp*omu*u)+
     & G1(0)*((17*H1(0))/3._dp+3*H1(1))-(8*H2(0,0))/3._dp-(10*H2(1,0))/3._dp+
     & H1(0)*(-3*H1(1)+(-12-(4*v)/omu-v/u-(u*v)/(omv*omv))/3._dp)+
     & G1(2)*((13-omv/u+(4*v)/omu)/3._dp+3*H1(1)-
     & ((omv+u*(-18*u+v))*H1(0))/(3._dp*(u*u)))+
     & G1(1)*(((omv+u*(-9*u+v))*H1(0))/(3._dp*(u*u))+
     & ((omv+u*(-9*u+v))*H1(1))/(3._dp*(u*u)))-
     & ((omv+u*(-9*u+v))*G2(1,2))/(3._dp*(u*u))-(3*(G1(0)*G1(0)))/2._dp-
     & (3*(G1(2)*G1(2)))/2._dp-(17*(H1(0)*H1(0)))/6._dp-(3*(H1(1)*H1(1)))/2._dp
      return
      end
      
      
      function Beta_2a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=-(pi*(omv+u*(-2+u-3*v)))/(3._dp*omu*u)-3*pi*G1(0)-3*pi*G1(2)+
     & (8*pi*G1(3))/3._dp+3*pi*H1(0)+3*pi*H1(1)-
     & (pi*(omv+u*(-9*u+v))*G1(1))/(3._dp*(u*u))
      return
      end
      
      
      function Beta_2a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-16*G4(0,0,1,2))/9._dp+18*G4(0,0,2,2)-16*G4(0,0,3,0)+18*G4(0,2,0,2)-
     & (92*G4(0,2,1,2))/9._dp+18*G4(0,2,2,0)-28*G4(0,2,3,0)-8*G4(0,3,0,2)-
     & 8*G4(0,3,2,0)+(128*G4(0,3,3,0))/9._dp+18*G4(2,0,0,2)+18*G4(2,0,2,0)+
     & 18*G4(2,2,0,0)-16*G4(2,3,0,0)+16*G4(2,3,0,2)+16*G4(2,3,2,0)-
     & 16*G4(3,0,0,2)+10*G4(3,0,1,2)-16*G4(3,0,2,0)+(208*G4(3,0,3,0))/9._dp-
     & 16*G4(3,2,0,0)+10*G4(3,2,1,2)+(80*G4(3,2,3,0))/9._dp+(128*G4(3,3,0,0))/9._dp-
     & (128*G4(3,3,3,0))/9._dp+16*G3(0,0,3)*H1(0)+28*G3(0,2,3)*H1(0)+
     & 8*G3(0,3,2)*H1(0)-(128*G3(0,3,3)*H1(0))/9._dp-16*G3(2,3,2)*H1(0)-
     & (208*G3(3,0,3)*H1(0))/9._dp-(80*G3(3,2,3)*H1(0))/9._dp+(128*G3(3,3,3)*H1(0))/9._dp+
     & G3(0,0,2)*(63-3*nf-(160*H1(0))/9._dp-18*H1(1))+
     & G3(2,0,0)*(63-3*nf-16*H1(0)-18*H1(1))+G3(3,0,1)*(-10*H1(0)-10*H1(1))+
     & G3(3,2,1)*(-10*H1(0)-10*H1(1))+G3(0,0,1)*((16*H1(0))/9._dp+(16*H1(1))/9._dp)+
     & G3(0,2,1)*((92*H1(0))/9._dp+(92*H1(1))/9._dp)+
     & G3(3,0,0)*((8*(-21+nf))/3._dp+(128*H1(0))/9._dp+16*H1(1))+
     & (64*H4(0,0,0,0))/3._dp+(80*H4(0,0,1,0))/9._dp+(80*H4(0,1,0,0))/9._dp-
     & 10*H4(0,1,0,1)+(10*H4(0,1,1,0))/9._dp+10*H4(1,0,0,0)-(160*H4(1,0,0,1))/9._dp+
     & (110*H4(1,0,1,0))/9._dp-(160*H4(1,1,0,0))/9._dp-40*H4(1,1,0,1)+
     & G2(0,0)*(80-(41*nf)/6._dp-54*zeta2+(8*(-21+nf)*H1(0))/3._dp+
     & 3*(-21+nf)*H1(1)+(128*H2(0,0))/9._dp+16*H2(0,1)+16*H2(1,0)+
     & 18*H2(1,1)+(5*(nf*nf))/36._dp)+
     & G3(0,2,2)*((332-27*nf+(4-4*v)/u-2*v-(77*v)/omu)/9._dp+
     & H1(0)*(-17+v/u+omv/(u*u)))+
     & G4(0,1,1,2)*(-169._dp/9._dp+v/u+omv/(u*u))+
     & G4(0,1,2,2)*(-17+v/u+omv/(u*u))+G4(2,1,0,2)*(-9+v/u+omv/(u*u))+
     & G4(2,1,2,0)*(-9+v/u+omv/(u*u))+
     & G3(0,2,0)*(63-3*nf-18*H1(1)-(2*(omv+u*(9*u+v))*H1(0))/(u*u))+
     & G3(2,0,2)*((332-27*nf+(4-4*v)/u-2*v-(77*v)/omu)/9._dp-
     & ((omv+u*(19*u+v))*H1(0))/(u*u))+
     & G3(1,1,0)*((-2*omu*(2+5*u-2*v)*H1(0))/(9._dp*(u*u))-
     & (2*omu*(2+5*u-2*v)*H1(1))/(9._dp*(u*u)))+
     & G3(2,1,0)*(-(((omv+u*(-9*u+v))*H1(0))/(u*u))-
     & ((omv+u*(-9*u+v))*H1(1))/(u*u))+
     & G3(1,0,1)*((10*(omv+u*(-9*u+v))*H1(0))/(9._dp*(u*u))+
     & (10*(omv+u*(-9*u+v))*H1(1))/(9._dp*(u*u)))+
     & G3(2,1,1)*((16*(omv+u*(-9*u+v))*H1(0))/(9._dp*(u*u))+
     & (16*(omv+u*(-9*u+v))*H1(1))/(9._dp*(u*u)))+
     & G3(0,1,0)*((2*(omv+u*(-9*u+v))*H1(0))/(u*u)+
     & (2*(omv+u*(-9*u+v))*H1(1))/(u*u))+
     & G3(2,2,1)*((-2*(11*omv+u*(3-86*u+11*v))*H1(0))/(9._dp*(u*u))-
     & (2*(11*omv+u*(3-86*u+11*v))*H1(1))/(9._dp*(u*u)))+
     & G3(1,2,1)*((H1(0)*(30+6*u-30*v+30*u*v-244*(u*u)))/(9._dp*(u*u))+
     & (H1(1)*(30+6*u-30*v+30*u*v-244*(u*u)))/(9._dp*(u*u)))+
     & G3(2,2,0)*((332-27*nf+(4-4*v)/u-2*v-(77*v)/omu)/9._dp+
     & (H1(0)*(21-21*v+21*u*v+30*omu*NFV*(-2+u+2*v)-181*(u*u)))/
     & (9._dp*(u*u)))-(2*(omv+u*(-9*u+v))*G4(0,1,0,2))/(u*u)-
     & (2*(omv+u*(-9*u+v))*G4(0,1,2,0))/(u*u)-
     & (10*(omv+u*(-9*u+v))*G4(1,0,1,2))/(9._dp*(u*u))-
     & ((omv+u*(2-33*u+v)+10*NFV*(-1+u)*(-2+u+2*v))*G4(1,0,3,0))/
     & (3._dp*(u*u))+(2*omu*(2+5*u-2*v)*G4(1,1,0,2))/(9._dp*(u*u))+
     & (2*omu*(2+5*u-2*v)*G4(1,1,2,0))/(9._dp*(u*u))-
     & (2*(10*omv+u*(3-77*u+10*v))*G4(1,1,2,2))/(9._dp*(u*u))-
     & (2*(15*omv+u*(3-122*u+15*v))*G4(1,2,1,2))/(9._dp*(u*u))-
     & ((omv+u*(2-33*u+v)+10*NFV*(-1+u)*(-2+u+2*v))*G4(1,2,3,0))/
     & (3._dp*(u*u))+((-17*omv+u*(6-81*u-17*v)-
     & 30*omu*NFV*(-2+u+2*v))*G4(2,0,3,0))/(9._dp*(u*u))-
     & (16*(omv+u*(-9*u+v))*G4(2,1,1,2))/(9._dp*(u*u))+
     & ((17*omv+30*omu*NFV*(-2+u+2*v)+u*(-6-27*u+17*v))*G4(2,2,3,0))/
     & (9._dp*(u*u))+(20*(omv+u*(-9*u+v))*G4(2,3,3,0))/(9._dp*(u*u))+
     & ((omv+u*(2-33*u+v)+10*NFV*(-1+u)*(-2+u+2*v))*G3(1,0,3)*H1(0))/
     & (3._dp*(u*u))+((omv+u*(2-33*u+v)+10*NFV*(-1+u)*(-2+u+2*v))*
     & G3(1,2,3)*H1(0))/(3._dp*(u*u))+
     & ((-17*omv+u*(6+27*u-17*v)-30*omu*NFV*(-2+u+2*v))*G3(2,2,3)*
     & H1(0))/(9._dp*(u*u))-(20*(omv+u*(-9*u+v))*G3(2,3,3)*H1(0))/
     & (9._dp*(u*u))+(G4(2,2,1,2)*(22+6*u-22*v+22*u*v-172*(u*u)))/
     & (9._dp*(u*u))+(G4(1,0,2,2)*(4+6*u-4*v+4*u*v-154*(u*u)))/
     & (9._dp*(u*u))+(G4(1,2,0,2)*(4+6*u-4*v+4*u*v-154*(u*u)))/
     & (9._dp*(u*u))+(G4(1,2,2,0)*(4+6*u-4*v+4*u*v-154*(u*u)))/
     & (9._dp*(u*u))-(2*G4(1,1,1,2)*(-6+3*u+6*v-6*u*v+67*(u*u)))/
     & (9._dp*(u*u))+(G3(2,0,3)*H1(0)*
     & (17-6*u-17*v+17*u*v+30*omu*NFV*(-2+u+2*v)+81*(u*u)))/
     & (9._dp*(u*u))-(G4(2,0,1,2)*(4+6*u-4*v+4*u*v+242*(u*u)))/
     & (9._dp*(u*u))+(H2(1,1)*(92*omv+
     & 4*u*(58-81*u-257*v+126*(-1+u)*zeta2)+5*omu*u*(nf*nf)+
     & 6*nf*(-2-13*u+2*v+6*u*v+15*(u*u))))/(36._dp*omu*u)+
     & G3(1,1,1)*((2*H1(0)*(-6+3*u+6*v-6*u*v+67*(u*u)))/(9._dp*(u*u))+
     & (2*H1(1)*(-6+3*u+6*v-6*u*v+67*(u*u)))/(9._dp*(u*u)))+
     & G3(0,1,1)*((H1(0)*(-9+9*v-9*u*v+169*(u*u)))/(9._dp*(u*u))+
     & (H1(1)*(-9+9*v-9*u*v+169*(u*u)))/(9._dp*(u*u)))+
     & G3(2,0,1)*((H1(0)*(4+6*u-4*v+4*u*v+242*(u*u)))/(9._dp*(u*u))+
     & (H1(1)*(4+6*u-4*v+4*u*v+242*(u*u)))/(9._dp*(u*u)))+
     & G3(1,2,2)*((-2*(10*omv+u*(3-77*u+10*v))*H1(0))/(9._dp*(u*u))+
     & (-3*omv*(-6+omu*nf)+
     & u*(-3+57*v-3*omu*nf*v+9*u*(-40+3*omu*nf+3*v)+
     & (345-2*v)*(u*u)))/(9._dp*omu*(u*u)))+
     & (H3(1,1,0)*(-703+40/omw+52*nf+10/u-(72*u)/omv+26*v+
     & (137*v)/omu+(62*v)/u-18*u*v+54/w-(54*u)/w+
     & (32*v)/(omu*omu)+(72*u)/(omv*omv)-10/(omw*omw)+
     & 10*NFV*((3*(2-5*u)*v)/(omu*omu)+(4*omv*(3+u*u))/u+
     & 4*(-3+v*(u*u)))))/18._dp+
     & G2(2,2)*(-(omu*(5+3*u-5*v+10*NFV*(-2+u+2*v))*H2(0,0))/
     & (3._dp*(u*u))+(H2(0,1)*(22+6*u-22*v+22*u*v-172*(u*u)))/
     & (9._dp*(u*u))+(2*H2(1,0)*(6-6*v+6*u*v+15*omu*NFV*(-2+u+2*v)-
     & 14*(u*u)))/(9._dp*(u*u))+
     & (H1(0)*(-3*omv*(-6+omu*nf)+
     & u*(-3+57*v-3*omu*nf*v+9*u*(-40+3*omu*nf+3*v)+
     & (345-2*v)*(u*u))))/(9._dp*omu*(u*u))-
     & (24*omu*zeta2*(5-5*v-15*omu*NFV*(-2+u+2*v)+u*(3+5*v)+
     & 171*(u*u))+3*u*(-92*omv+4*u*(-58+81*u+257*v)-
     & 5*omu*u*(nf*nf)-6*nf*(-2-13*u+2*v+6*u*v+15*(u*u))))/
     & (108._dp*omu*(u*u)))+G2(0,1)*
     & ((20*H2(0,0))/9._dp+2*H2(1,0)+H2(1,1)*(-17+v/u+omv/(u*u))+
     & H2(0,1)*(-151._dp/9._dp+v/u+omv/(u*u))-
     & (7*zeta2*(-9+9*v-9*u*v+25*(u*u)))/(9._dp*(u*u))+
     & (H1(0)*(851+63/omv-40/omw-45*nf-12/u+82*v-(80*v)/omu-
     & (6*v)/u+(nf*v)/u+10/(omw*omw)+(omv*(16+nf))/(u*u)+
     & (9*(-281+36*nf)*v*(u*u))/(omv*omv*omv)+
     & (9*(281-36*nf)*v*(u*u*u))/(omv*omv*omv*omv)+
     & (u*v*(1031-108*nf+160*v-80*(v*v)))/(omv*omv)))/18._dp+
     & (H1(1)*(851+63/omv-40/omw-45*nf-12/u+82*v-(80*v)/omu-
     & (6*v)/u+(nf*v)/u+10/(omw*omw)+(omv*(16+nf))/(u*u)+
     & (9*(-281+36*nf)*v*(u*u))/(omv*omv*omv)+
     & (9*(281-36*nf)*v*(u*u*u))/(omv*omv*omv*omv)+
     & (u*v*(1031-108*nf+160*v-80*(v*v)))/(omv*omv)))/18._dp)+
     & G3(0,1,2)*(-2*H1(0)-((omv+u*(-17*u+v))*H1(1))/(u*u)+
     & (-851-63/omv+40/omw+45*nf+12/u-82*v+(80*v)/omu+
     & (6*v)/u-(nf*v)/u-10/(omw*omw)-(omv*(16+nf))/(u*u)+
     & (9*(281-36*nf)*v*(u*u))/(omv*omv*omv)+
     & (9*(-281+36*nf)*v*(u*u*u))/(omv*omv*omv*omv)+
     & (u*v*(-1031+108*nf-160*v+80*(v*v)))/(omv*omv))/18._dp)+
     & G3(1,0,2)*(-(H1(0)*(5+6*u-5*v+5*u*v-163*(u*u)))/(9._dp*(u*u))+
     & (2*H1(1)*(-2-3*u+2*v-2*u*v+77*(u*u)))/(9._dp*(u*u))+
     & (63-106*v+(nf*(-1+u)-20*NFV*(omu*omu))*(omv*omv)-
     & 71*v*(u*u*u*u)+u*u*u*
     & (281-9*nf+v*(146-20*NFV*(omu*omu))-71*(v*v))+42*(v*v)+
     & u*u*(-289+nf*(9-8*v)+201*v+75*(v*v)-
     & 20*NFV*(omu*omu)*(1+v*v))-
     & u*(55-70*v+32*(v*v)+20*NFV*(omu*omu)*(-2+v+v*v)))/
     & (18._dp*w*(u*u)))+G3(1,2,0)*
     & ((2*H1(0)*(u*v-5*NFV*(-1+u)*(-2+u+2*v)+w+27*(u*u)))/
     & (3._dp*(u*u))+(2*H1(1)*(-2-3*u+2*v-2*u*v+77*(u*u)))/(9._dp*(u*u))+
     & (63-106*v+(nf*(-1+u)-20*NFV*(omu*omu))*(omv*omv)-
     & 71*v*(u*u*u*u)+u*u*u*
     & (281-9*nf+v*(146-20*NFV*(omu*omu))-71*(v*v))+42*(v*v)+
     & u*u*(-289+nf*(9-8*v)+201*v+75*(v*v)-
     & 20*NFV*(omu*omu)*(1+v*v))-
     & u*(55-70*v+32*(v*v)+20*NFV*(omu*omu)*(-2+v+v*v)))/
     & (18._dp*w*(u*u)))+G2(1,0)*((H2(1,1)*(4+6*u-4*v+4*u*v-154*(u*u)))/
     & (9._dp*(u*u))-(2*zeta2*(7-7*v+30*omu*NFV*(-2+u+2*v)+
     & u*(9+7*v)-142*(u*u)))/(9._dp*(u*u))-
     & (H2(0,0)*(5-5*v+10*omu*NFV*(-2+u+2*v)+u*(-2+5*v)+
     & 27*(u*u)))/(3._dp*(u*u))-
     & (H2(0,1)*(5-6*u-5*v+5*u*v+73*(u*u)))/(9._dp*(u*u))+
     & (H2(1,0)*(10*NFV*(-1+u)*(-2+u+2*v)-2*(u*v+w+27*(u*u))))/
     & (3._dp*(u*u))+(H1(0)*(-63+106*v+
     & (nf-nf*u+20*NFV*(omu*omu))*(omv*omv)+71*v*(u*u*u*u)-
     & 42*(v*v)+u*u*u*(-281+9*nf+2*v*(-73+10*NFV*(omu*omu))+
     & 71*(v*v))+u*u*(289-201*v+nf*(-9+8*v)-75*(v*v)+
     & 20*NFV*(omu*omu)*(1+v*v))+
     & u*(55-70*v+32*(v*v)+20*NFV*(omu*omu)*(-2+v+v*v))))/
     & (18._dp*w*(u*u))+(H1(1)*(-63+106*v+
     & (nf-nf*u+20*NFV*(omu*omu))*(omv*omv)+71*v*(u*u*u*u)-
     & 42*(v*v)+u*u*u*(-281+9*nf+2*v*(-73+10*NFV*(omu*omu))+
     & 71*(v*v))+u*u*(289-201*v+nf*(-9+8*v)-75*(v*v)+
     & 20*NFV*(omu*omu)*(1+v*v))+
     & u*(55-70*v+32*(v*v)+20*NFV*(omu*omu)*(-2+v+v*v))))/
     & (18._dp*w*(u*u)))+G2(1,1)*((-2*omu*(2+5*u-2*v)*H2(0,1))/(9._dp*(u*u))-
     & (16*(omv+u*(-9*u+v))*H2(1,0))/(9._dp*(u*u))-
     & (2*(10*omv+u*(3-77*u+10*v))*H2(1,1))/(9._dp*(u*u))-
     & (2*zeta2*(-6+3*u+6*v-6*u*v+67*(u*u)))/(9._dp*(u*u))+
     & (H1(0)*(-14+8*v-4*omu*nf*
     & ((-1+u)*(omv*omv)+(9-9*u-8*v)*(u*u))+
     & 151*v*(u*u*u*u*u)+u*u*u*(-1168+919*v-308*(v*v))+
     & u*(16+38*v-38*(v*v))+7*(v*v)+
     & u*u*(588-657*v+60*(v*v))+u*u*u*u*(578-459*v+151*(v*v))-
     & 10*NFV*(omu*omu)*(2*(omv*omv)-omv*u*(3+2*(u*u))-
     & u*u*(-3+2*v*(1+u*u)+2*u*(v*v)))))/(18._dp*omu*w*(u*u))+
     & (H1(1)*(-14+8*v-4*omu*nf*
     & ((-1+u)*(omv*omv)+(9-9*u-8*v)*(u*u))+
     & 151*v*(u*u*u*u*u)+u*u*u*(-1168+919*v-308*(v*v))+
     & u*(16+38*v-38*(v*v))+7*(v*v)+
     & u*u*(588-657*v+60*(v*v))+u*u*u*u*(578-459*v+151*(v*v))-
     & 10*NFV*(omu*omu)*(2*(omv*omv)-omv*u*(3+2*(u*u))-
     & u*u*(-3+2*v*(1+u*u)+2*u*(v*v)))))/(18._dp*omu*w*(u*u)))+
     & G3(1,1,2)*((16*(omv+u*(-9*u+v))*H1(0))/(9._dp*(u*u))+
     & (H1(1)*(20+6*u-20*v+20*u*v-154*(u*u)))/(9._dp*(u*u))+
     & (14-8*v+4*omu*nf*((-1+u)*(omv*omv)+(9-9*u-8*v)*(u*u))-
     & 151*v*(u*u*u*u*u)+u*u*u*u*(-578+459*v-151*(v*v))+
     & u*u*(-588+657*v-60*(v*v))-7*(v*v)+
     & 2*u*(-8-19*v+19*(v*v))+u*u*u*(1168-919*v+308*(v*v))+
     & 10*NFV*(omu*omu)*(2*(omv*omv)-omv*u*(3+2*(u*u))-
     & u*u*(-3+2*v*(1+u*u)+2*u*(v*v))))/(18._dp*omu*w*(u*u)))+
     & (H2(0,1)*(18*(nf*nf)+3*nf*(-27-96/omv+(88*v)/omu+
     & (2*(-6+11*v))/u+(330*u*v)/(omv*omv)-
     & (648*v*(u*u))/(omv*omv*omv))+
     & 2*(-1452*zeta2+3*(-482+680/omv-10/omw+
     & (-80-708/omu+30*NFV)*v-20*NFV*(v*v)+
     & (109-5*(21+4*NFV)*v+20*NFV*(v*v))/u+
     & (v*(u*u)*(2529+20*NFV*(9-5*v+2*(v*v))))/
     & (omv*omv*omv)-
     & (u*v*(1293+10*NFV*(16-13*v+6*(v*v))))/(omv*omv)))))/108._dp
     & +(H3(0,0,1)*(15-66/omv+8*nf+11*v-(8*v)/omu-(4*v)/u-9*u*v+
     & 27/w-(27*u)/w+(20*NFV*u*
     & (-1+(-4+3*u)*(v*v*v)+omu*v*(1+3*v+v*v*v)))/
     & (omv*omv*omv)))/18._dp+
     & (1916370*zeta4+80*(-36415+360/omw+20952*zeta3+(567*zeta3)/omv+
     & (18*(40+60*NFV*(-2+v)-51*v)*zeta3)/u-36*v*zeta3-
     & (2529*v*zeta3)/omu-(288*v*zeta3)/(omu*omu)-
     & (27*u*(208+v*(-208-347*zeta3+20*NFV*zeta3)))/(omv*omv)-
     & (22761*v*zeta3*(u*u))/(omv*omv*omv)+
     & (270*NFV*zeta3*(8-2*v+u*(-16+5*v)+8*(u*u)))/(omu*omu)+
     & (22761*v*zeta3*(u*u*u))/(omv*omv*omv*omv))+
     & 360*zeta2*(13071-64/omu-1379/omv+60/omw+1646*v+
     & (552*v)/omu+22/w+u*
     & (4916-11619/omv-1548*v-58/w+2667/(omv*omv))-72*(v*v)+
     & (3372*(u*u*u)*(-2-5*v+v*v))/(omv*omv*omv)+
     & (608-770*v+2/w+72*(v*v))/u+
     & (6*(u*u)*(878+1581*v-4110*(v*v)+808*(v*v*v)))/
     & (omv*omv*omv)+10*NFV*
     & ((36-48*v)*(u*u)-24*(u*u*u)+
     & u*(-6+52*v+15/w+12/(omv*omv)-40*(v*v))+
     & 2*(-13+(9-18*v)/omu-6*v-3/w+22*(v*v)-8*(v*v*v))+
     & (4*(-7+8*v-5*(v*v)+4*(v*v*v)))/u)))/12960._dp+
     & H1(1)*((-4*(-30+nf)*H2(0,0))/9._dp-(5*(-30+nf)*H2(1,0))/9._dp+
     & ((4*nf*(337+73*v+u*(-337+27*v))-v*(4536+3306*zeta2))/omu+
     & (12*(16+NFV*(30-75*u))*v*zeta2)/(omu*omu)+
     & (4*w*(-19*nf+180*NFV*zeta2)*(omv*omv*omv*omv*omv)*
     & (omw*omw)-w*(4935+720*NFV+2529*u+7212*zeta2)*
     & (omv*omv*omv)*(omw*omw)*(u*u)+
     & 54*(281-36*nf*v)*w*zeta2*(omw*omw)*(u*u*u*u)+
     & 9*omv*w*(omw*omw)*(u*u*u)*
     & (-1686*(1+u)*zeta2+36*nf*(-2+v+6*v*zeta2+v*v))+
     & omv*omv*omv*omv*
     & (-180*u*w*zeta2+720*omw*u*w*zeta2+
     & omw*omw*(6*zeta2*
     & (5*(-1+17*v)*w+
     & u*(27-(-995+112*nf+120*NFV+5*v)*w)-
     & 9*(3+v*w)*(u*u)+20*NFV*v*w*(u*u*u))-
     & 3*w*(405*(-1+v)+u*(6494-369*v+396*zeta3)-
     & 1212*(u*u)+
     & 10*NFV*((-5+4*v)*(u*u)-2*(-5+4*v+v*v)+
     & u*(-11-3*v+2*(v*v))))))+
     & w*(omv*omv)*(omw*omw)*(u*u)*
     & (-12*nf*(-19+v*(-17+54*zeta2)+36*(v*v))+
     & 3*u*(2529-20*NFV*(-9-4*v+v*v))+
     & 6*zeta2*(1202+2529*u-
     & 10*NFV*(-2+9*v-6*(v*v)+2*(v*v*v)))))/
     & (u*w*(omv*omv*omv*omv)*(omw*omw)))/108._dp)+
     & G2(0,3)*((304*zeta2)/9._dp-(128*H2(0,0))/9._dp-8*H2(0,1)-8*H2(1,0)+
     & (H1(0)*(146+63/omv-75*v-(71*v)/omu+(150*NFV*v)/omu+
     & (40-6*(1+20*NFV)*v)/u+27/w+(32*v)/(omu*omu)-
     & (90*NFV*v)/(omu*omu)+
     & u*(1111/omv+20*omv*NFV+71*v-27/w-1111/(omv*omv))+
     & (v*(2529+20*NFV*(omv*omv*omv))*(u*u))/(omv*omv*omv)-
     & (2529*v*(u*u*u))/(omv*omv*omv*omv)+
     & (4*nf*(-2+v*(8+27*u-81*(u*u)+81*(u*u*u))+
     & 3*(-4-18*u+27*(u*u))*(v*v)+(8+27*u)*(v*v*v)-
     & 2*(v*v*v*v)))/(omv*omv*omv*omv)))/18._dp)+
     & (H3(1,0,1)*(-627-40/omw+(12-20*v)/u+11*v-(72*v)/omu+27/w+
     & 10/(omw*omw)+(v*(2529+20*NFV*(omv*omv*omv))*(u*u))/
     & (omv*omv*omv)-(2529*v*(u*u*u))/(omv*omv*omv*omv)-
     & (u*(-1130*omv*w+9*(3+v*w)*(omv*omv)+
     & 10*w*(113+NFV*(-2+3*v-6*(v*v)+2*(v*v*v)))))/
     & (w*(omv*omv))+(2*nf*
     & (23+2*v*(-46+27*u-81*(u*u)+81*(u*u*u))+
     & 6*(23-18*u+27*(u*u))*(v*v)+(-92+54*u)*(v*v*v)+
     & 23*(v*v*v*v)))/(omv*omv*omv*omv)))/18._dp+
     & G3(0,3,0)*((64*H1(0))/9._dp+8*H1(1)+
     & (-146-63/omv+75*v+(71*v)/omu-(150*NFV*v)/omu+
     & (2*(-20+(3+60*NFV)*v))/u-27/w-(32*v)/(omu*omu)+
     & (90*NFV*v)/(omu*omu)+
     & u*(-1111/omv-20*omv*NFV-71*v+27/w+1111/(omv*omv))-
     & (v*(2529+20*NFV*(omv*omv*omv))*(u*u))/(omv*omv*omv)+
     & (2529*v*(u*u*u))/(omv*omv*omv*omv)+
     & (4*nf*(2-v*(8+27*u-81*(u*u)+81*(u*u*u))+
     & (12+54*u-81*(u*u))*(v*v)-(8+27*u)*(v*v*v)+2*(v*v*v*v))
     & )/(omv*omv*omv*omv))/18._dp)+
     & (H3(0,0,0)*(-63/omv+(3*omu*(21-50*NFV)*u*v+2*(-16+45*NFV)*u*v+
     & (-1785*u+2*v+120*NFV*v+86*u*v)*(omu*omu))/(u*(omu*omu))
     & -(2529*v*(u*u))/(omv*omv*omv)+
     & (u*v*(1049+160*v-80*(v*v)))/(omv*omv)+
     & (2529*v*(u*u*u)+4*nf*(28-v*(112+27*u-81*(u*u)+81*(u*u*u))+
     & (168+54*u-81*(u*u))*(v*v)-(112+27*u)*(v*v*v)+
     & 28*(v*v*v*v)))/(omv*omv*omv*omv)))/18._dp+
     & G3(3,2,0)*((152*H1(0))/9._dp+(20*NFV*v*(u*u*u*u*u)+
     & u*u*u*u*(9*v+20*NFV*(1-2*v+4*(v*v)))-
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+20*NFV*(v*v*v))+
     & u*u*u*(24-8*nf+2*v+27*(v*v)+
     & 10*NFV*(-2+7*v-14*(v*v)+12*(v*v*v)))+
     & u*v*(-20+(10-30*NFV)*v+2*(-8+55*NFV)*(v*v)+
     & (9-100*NFV)*(v*v*v)+20*NFV*(v*v*v*v))+
     & u*u*(-24+8*omv*nf-6*(-6+5*NFV)*v+5*(-1+24*NFV)*(v*v)-
     & 9*(-3+20*NFV)*(v*v*v)+80*NFV*(v*v*v*v)))/(18._dp*w*(u*u)))+
     & G3(3,0,2)*(18*H1(0)+(20*NFV*v*(u*u*u*u*u)+
     & u*u*u*u*(9*v+20*NFV*(1-2*v+4*(v*v)))-
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+20*NFV*(v*v*v))+
     & u*u*u*(24-8*nf+2*v+27*(v*v)+
     & 10*NFV*(-2+7*v-14*(v*v)+12*(v*v*v)))+
     & u*v*(-20+(10-30*NFV)*v+2*(-8+55*NFV)*(v*v)+
     & (9-100*NFV)*(v*v*v)+20*NFV*(v*v*v*v))+
     & u*u*(-24+8*omv*nf-6*(-6+5*NFV)*v+5*(-1+24*NFV)*(v*v)-
     & 9*(-3+20*NFV)*(v*v*v)+80*NFV*(v*v*v*v)))/(18._dp*w*(u*u)))+
     & G2(3,2)*((242*zeta2)/9._dp-(80*H2(0,0))/9._dp+10*H2(0,1)-(10*H2(1,0))/9._dp-
     & (H1(0)*(20*NFV*v*(u*u*u*u*u)+
     & u*u*u*u*(9*v+20*NFV*(1-2*v+4*(v*v)))-
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+20*NFV*(v*v*v))+
     & u*u*u*(24-8*nf+2*v+27*(v*v)+
     & 10*NFV*(-2+7*v-14*(v*v)+12*(v*v*v)))+
     & u*v*(-20+(10-30*NFV)*v+2*(-8+55*NFV)*(v*v)+
     & (9-100*NFV)*(v*v*v)+20*NFV*(v*v*v*v))+
     & u*u*(-24+8*omv*nf-6*(-6+5*NFV)*v+5*(-1+24*NFV)*(v*v)-
     & 9*(-3+20*NFV)*(v*v*v)+80*NFV*(v*v*v*v))))/(18._dp*w*(u*u)))+
     & (H2(0,0)*(3*omu*(-(omv*w*(u*u)*(40*NFV+9*(313+1124*u+281*(u*u))))+
     & u*w*(omv*omv)*(1379+(5003-20*NFV)*u+2529*(u*u)-
     & 843*(u*u*u))+5058*(1+u)*w*(u*u*u)+
     & omv*omv*omv*(6*(202+5*NFV*(-3+4*v))*w*(u*u*u)+
     & 60*NFV*w*(u*u*u*u)+
     & u*u*(58+(-1184+387*v)*w+
     & 10*NFV*(-15+w*(9-11*v+10*(v*v))))+
     & 2*(-1+w*(1+(-193+100*NFV)*v-9*(v*v)-
     & 20*NFV*(v*v*v)))+
     & u*(-22+w*(-836-433*v-2256*zeta2+12*(nf*nf)+
     & 18*(v*v))+20*NFV*(3+w*(18+v-3*(v*v)+2*(v*v*v))))
     & ))-4*w*(3*(-6*nf*v+u*(45*NFV*(1+v)+8*(-2+49*v)))*
     & (omv*omv*omv)+nf*u*(omv*omv)*(145-145*v+81*(v*v))+
     & nf*(u*u)*(-91+147*v-336*(v*v)+81*(u*u*u)*(-2-5*v+v*v)+
     & 334*(v*v*v)+27*(u*u)*(11+9*v-24*(v*v)+4*(v*v*v))-
     & 54*(v*v*v*v)+9*u*
     & (-21+44*v+61*(v*v)-33*(v*v*v)+3*(v*v*v*v))))))/
     & (108._dp*omu*u*w*(omv*omv*omv))+
     & (H2(1,0)*(-12*omw*u*(16+45*NFV*(-1+v)+(318-23*nf+9*nf*u)*v)*w*
     & (omv*omv*omv)+omu*
     & (3*omw*u*(-19+12*nf+(2109-140*NFV)*u)*w*(omv*omv)-
     & 3*omv*omw*w*(u*u)*(3*
     & (-27+20*NFV+u*(439+1651*v-404*(v*v)))+
     & 2*nf*(36+55*v-108*(v*v)+18*u*(-5-17*v+4*(v*v))+
     & 18*(v*v*v)))+
     & omv*omv*omv*(60*u*w+
     & omw*(180*NFV*w*(u*u*u*u)+
     & 3*(u*u)*(-58+(-1184+387*v)*w+
     & 10*NFV*(15+w*(11-15*v+10*(v*v))))+
     & u*(66+w*(-8175+355*nf-927*v-8004*zeta2+
     & 18*(nf*nf)+54*(v*v))+
     & 60*NFV*(-3+2*v*w-5*w*(v*v)+2*w*(v*v*v)))+
     & 6*(1+w*(91-116*v+nf*(-6+7*v)-9*(v*v)-
     & 20*NFV*(6-6*v-v*v+v*v*v)))))+
     & 3*omw*w*(u*u*u)*(3*(-281+36*nf)*u*(-2-5*v+v*v)-
     & 10*NFV*(9-75*v+83*(v*v)-53*(v*v*v)+12*(v*v*v*v))))))/
     & (108._dp*omu*omw*u*w*(omv*omv*omv))+
     & (H3(0,1,0)*(73*omu*u*v*w*(omv*omv*omv*omv)+
     & 2*(16+NFV*(30-75*u))*u*v*w*(omv*omv*omv*omv)+
     & omu*omu*(-3*u*(1+377*u)*w*(omv*omv*omv)+
     & (1131-40*NFV+2529*u)*w*(omv*omv)*(u*u)-
     & omv*(2529+2529*u+40*NFV*v)*w*(u*u*u)+
     & omv*omv*omv*omv*(2*(23-60*NFV)*v*w+
     & u*(27+(-1038+97*v)*w)-
     & (27+20*NFV*(-1+v)*w+89*v*w)*(u*u)+20*NFV*v*w*(u*u*u))+
     & u*w*(2529*(u*u*u)+4*nf*
     & (17-v*(68+27*u-81*(u*u)+81*(u*u*u))+
     & (102+54*u-81*(u*u))*(v*v)-(68+27*u)*(v*v*v)+
     & 17*(v*v*v*v))))))/(18._dp*u*w*(omu*omu)*(omv*omv*omv*omv))
     & +nf*(4219._dp/486._dp+(100*u)/(27-27*v)-(139*zeta3)/27._dp+
     & (6*u*zeta3)/omv-(6*u*zeta3)/(omv*omv)-
     & (18*zeta3*(u*u))/(omv*omv)+(18*zeta3*(u*u))/(omv*omv*omv)+
     & (18*zeta3*(u*u*u))/(omv*omv*omv)-
     & (18*zeta3*(u*u*u))/(omv*omv*omv*omv)+
     & (zeta2*(4*(-2+v)*(omv*omv*omv)+
     & u*(omv*omv)*(-439+433*v+60*(v*v))+
     & u*u*(591-1417*v+751*(v*v)+216*(u*u*u)*(-2-5*v+v*v)+
     & 219*(v*v*v)+36*(u*u)*(22+45*v-48*(v*v)+8*(v*v*v))-
     & 144*(v*v*v*v)+2*u*
     & (-252-223*v+997*(v*v)-396*(v*v*v)+36*(v*v*v*v)))))/
     & (18._dp*omu*u*(omv*omv*omv)))+
     & (H3(1,0,0)*(63*omu*u*v*w*(omv*omv*omv*omv)+
     & 2*u*(-16+15*NFV*(-2+5*u))*v*w*(omv*omv*omv*omv)+
     & omu*omu*(u*(-3+1111*u)*w*(omv*omv*omv)+
     & omv*(2529+2529*u-40*NFV*v)*w*(u*u*u)+
     & omv*omv*omv*omv*(2*(14+60*NFV*(-2+v)+v)*w+
     & u*(27+(-2656+240*NFV+15*v)*w)-9*(3+v*w)*(u*u)+
     & 20*NFV*v*w*(u*u*u))-
     & w*(omv*omv)*(u*u)*
     & (1111+2529*u+20*NFV*(1-3*(v*v)+v*v*v))+
     & u*w*(-2529*(u*u*u)+
     & 4*nf*(41+v*(-164+27*u-81*(u*u)+81*(u*u*u))+
     & 3*(82-18*u+27*(u*u))*(v*v)+(-164+27*u)*(v*v*v)+
     & 41*(v*v*v*v))))))/(18._dp*u*w*(omu*omu)*(omv*omv*omv*omv))
     & +G3(2,3,0)*(-2*H1(0)-16*H1(1)+
     & (71*omu*u*v*(omv*omv*omv*omv)+
     & 2*u*(-16+15*NFV*(-2+5*u))*v*(omv*omv*omv*omv)+
     & omu*omu*(u*(63+1111*u)*(omv*omv*omv)+
     & (28-685*u+6*v+4*u*v+120*NFV*(-2+2*u+v))*
     & (omv*omv*omv*omv)-(1111+2529*u)*(omv*omv)*(u*u)+
     & 2529*omv*(1+u)*(u*u*u)+
     & u*(-2529*(u*u*u)+4*nf*
     & (11+v*(-44+27*u-81*(u*u)+81*(u*u*u))+
     & (66-54*u+81*(u*u))*(v*v)+(-44+27*u)*(v*v*v)+
     & 11*(v*v*v*v)))))/(18._dp*u*(omu*omu)*(omv*omv*omv*omv)))
     & +G2(2,3)*(20*H2(0,0)+16*H2(0,1)+16*H2(1,0)-
     & (4*zeta2*(5-5*v+5*u*v+63*(u*u)))/(9._dp*(u*u))-
     & (H1(0)*(71*omu*u*v*(omv*omv*omv*omv)+
     & 2*(omv*omv*omv*omv)*
     & (-16*u*v+15*NFV*(u*(24-10*v)+4*(-2+v)+
     & 3*(-8+3*v)*(u*u)+8*(u*u*u)))+
     & omu*omu*(u*(63+1111*u)*(omv*omv*omv)+
     & (28+6*v+u*(-685+4*v))*(omv*omv*omv*omv)-
     & (1111+2529*u)*(omv*omv)*(u*u)+2529*omv*(1+u)*(u*u*u)+
     & u*(-2529*(u*u*u)+
     & 4*nf*(11+v*(-44+27*u-81*(u*u)+81*(u*u*u))+
     & (66-54*u+81*(u*u))*(v*v)+(-44+27*u)*(v*v*v)+
     & 11*(v*v*v*v))))))/
     & (18._dp*u*(omu*omu)*(omv*omv*omv*omv)))+
     & G2(2,1)*((-22*(omv+u*(-9*u+v))*zeta2)/(9._dp*(u*u))-
     & (8*(omv+u*(-9*u+v))*H2(0,1))/(9._dp*(u*u))+
     & (8*(omv+u*(-9*u+v))*H2(1,0))/(9._dp*(u*u))+
     & (H1(0)*(-200*v*(omv*omv*omv*omv)*(omw*omw)*(u*u)+
     & omu*((-63+1111*u)*(omv*omv*omv)*(omw*omw)*(u*u)+
     & omv*omv*omv*omv*omv*(omw*omw)*
     & (nf+10*NFV*(2+3*(u*u)))+
     & (-1111-2529*u+108*nf*v)*(omv*omv)*(omw*omw)*(u*u*u)+
     & 9*omv*(281+281*u-36*nf*v)*(omw*omw)*(u*u*u*u)+
     & 9*(-281+36*nf*v)*(omw*omw)*(u*u*u*u*u)-
     & omv*omv*omv*omv*
     & (-10*(u*u)+40*omw*(u*u)+
     & omw*omw*(20*NFV*v*(u*u*u*u)+
     & u*u*u*(9*v+20*NFV*(1-v+3*(v*v)))-
     & (-1+v)*(38+20*v+9*(v*v)+20*NFV*(v*v*v))+
     & u*u*(-95+9*nf+15*v+(18-60*NFV)*(v*v)+
     & 60*NFV*(v*v*v))+
     & u*(-60-(-100+nf)*v+2*(v*v)+9*(v*v*v)+
     & 10*NFV*(3-2*v+3*(v*v)-6*(v*v*v)+2*(v*v*v*v)))))
     & )))/(18._dp*omu*(omv*omv*omv*omv)*(omw*omw)*(u*u))+
     & (H1(1)*(-200*v*(omv*omv*omv*omv)*(omw*omw)*(u*u)+
     & omu*((-63+1111*u)*(omv*omv*omv)*(omw*omw)*(u*u)+
     & omv*omv*omv*omv*omv*(omw*omw)*
     & (nf+10*NFV*(2+3*(u*u)))+
     & (-1111-2529*u+108*nf*v)*(omv*omv)*(omw*omw)*(u*u*u)+
     & 9*omv*(281+281*u-36*nf*v)*(omw*omw)*(u*u*u*u)+
     & 9*(-281+36*nf*v)*(omw*omw)*(u*u*u*u*u)-
     & omv*omv*omv*omv*
     & (-10*(u*u)+40*omw*(u*u)+
     & omw*omw*(20*NFV*v*(u*u*u*u)+
     & u*u*u*(9*v+20*NFV*(1-v+3*(v*v)))-
     & (-1+v)*(38+20*v+9*(v*v)+20*NFV*(v*v*v))+
     & u*u*(-95+9*nf+15*v+(18-60*NFV)*(v*v)+
     & 60*NFV*(v*v*v))+
     & u*(-60-(-100+nf)*v+2*(v*v)+9*(v*v*v)+
     & 10*NFV*(3-2*v+3*(v*v)-6*(v*v*v)+2*(v*v*v*v)))))
     & )))/(18._dp*omu*(omv*omv*omv*omv)*(omw*omw)*(u*u)))+
     & G3(2,1,2)*((-8*(omv+u*(-9*u+v))*H1(0))/(9._dp*(u*u))+
     & (200*v*(omv*omv*omv*omv)*(omw*omw)*(u*u)+
     & omu*((63-1111*u)*(omv*omv*omv)*(omw*omw)*(u*u)-
     & omv*omv*omv*omv*omv*(omw*omw)*
     & (nf+10*NFV*(2+3*(u*u)))+
     & (1111+2529*u-108*nf*v)*(omv*omv)*(omw*omw)*(u*u*u)-
     & 9*omv*(281+281*u-36*nf*v)*(omw*omw)*(u*u*u*u)+
     & 9*(281-36*nf*v)*(omw*omw)*(u*u*u*u*u)+
     & omv*omv*omv*omv*
     & (-10*(u*u)+40*omw*(u*u)+
     & omw*omw*(20*NFV*v*(u*u*u*u)+
     & u*u*u*(9*v+20*NFV*(1-v+3*(v*v)))-
     & (-1+v)*(38+20*v+9*(v*v)+20*NFV*(v*v*v))+
     & u*u*(-95+9*nf+15*v+(18-60*NFV)*(v*v)+
     & 60*NFV*(v*v*v))+
     & u*(-60-(-100+nf)*v+2*(v*v)+9*(v*v*v)+
     & 10*NFV*(3-2*v+3*(v*v)-6*(v*v*v)+2*(v*v*v*v)))))))
     & /(18._dp*omu*(omv*omv*omv*omv)*(omw*omw)*(u*u)))+
     & G3(3,3,0)*((-128*H1(0))/9._dp-(20*NFV*v*(u*u*u*u*u)+
     & u*u*u*u*(9*v+20*NFV*(1-2*v+4*(v*v)))+
     & u*u*u*(-532+32*nf+2*v+27*(v*v)+
     & 10*NFV*(-2+7*v-14*(v*v)+12*(v*v*v)))-
     & v*(40-52*v+(2+20*NFV)*(v*v)+(9-40*NFV)*(v*v*v)+
     & 20*NFV*(v*v*v*v))+
     & u*u*(572-32*omv*nf-2*(281+15*NFV)*v+5*(-1+24*NFV)*(v*v)-
     & 9*(-3+20*NFV)*(v*v*v)+80*NFV*(v*v*v*v))+
     & u*(-40+96*v-2*(16+15*NFV)*(v*v)+2*(-8+55*NFV)*(v*v*v)+
     & (9-100*NFV)*(v*v*v*v)+20*NFV*(v*v*v*v*v)))/(18._dp*w*(u*u)))+
     & G2(3,3)*((-256*zeta2)/9._dp+(128*H2(0,0))/9._dp+
     & (H1(0)*(20*NFV*v*(u*u*u*u*u)+
     & u*u*u*u*(9*v+20*NFV*(1-2*v+4*(v*v)))+
     & u*u*u*(-532+32*nf+2*v+27*(v*v)+
     & 10*NFV*(-2+7*v-14*(v*v)+12*(v*v*v)))-
     & v*(40-52*v+(2+20*NFV)*(v*v)+(9-40*NFV)*(v*v*v)+
     & 20*NFV*(v*v*v*v))+
     & u*u*(572-32*omv*nf-2*(281+15*NFV)*v+
     & 5*(-1+24*NFV)*(v*v)-9*(-3+20*NFV)*(v*v*v)+
     & 80*NFV*(v*v*v*v))+
     & u*(-40+96*v-2*(16+15*NFV)*(v*v)+2*(-8+55*NFV)*(v*v*v)+
     & (9-100*NFV)*(v*v*v*v)+20*NFV*(v*v*v*v*v))))/(18._dp*w*(u*u)))
     & +((33-2*nf)/6._dp-(17*H1(0))/2._dp-(9*H1(1))/2._dp)*(G1(0)*G1(0)*G1(0))+
     & (9*(G1(0)*G1(0)*G1(0)*G1(0)))/8._dp+
     & ((3*omv+u*(-9+2*nf*(-1+u)+6*u-9*v))/(6._dp*omu*u)-(9*H1(1))/2._dp+
     & ((omv+u*(-18*u+v))*H1(0))/(2._dp*(u*u)))*(G1(2)*G1(2)*G1(2))+
     & (9*(G1(2)*G1(2)*G1(2)*G1(2)))/8._dp+
     & ((17*H1(1))/2._dp-(17*(omv*omv*(-3*v-u*(3+2*nf+9*v)+2*nf*(u*u))+
     & 3*(u*u)*(1+(-3+u)*v+v*v)))/(54._dp*omu*u*(omv*omv)))*
     & (H1(0)*H1(0)*H1(0))+(289*(H1(0)*H1(0)*H1(0)*H1(0)))/72._dp+
     & G2(3,0)*(H1(0)*(32-(20*nf)/9._dp-8*H1(1))-(64*H2(0,0))/3._dp-8*H2(0,1)-
     & (152*H2(1,0))/9._dp-(H1(1)*(20*NFV*v*(u*u*u*u*u)+
     & u*u*u*u*(9*v+20*NFV*(1-2*v+4*(v*v)))-
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+20*NFV*(v*v*v))+
     & u*u*u*(-216+2*v+27*(v*v)+
     & 10*NFV*(-2+7*v-14*(v*v)+12*(v*v*v)))+
     & u*v*(-20+(10-30*NFV)*v+2*(-8+55*NFV)*(v*v)+
     & (9-100*NFV)*(v*v*v)+20*NFV*(v*v*v*v))+
     & u*u*(216-6*(34+5*NFV)*v+5*(-1+24*NFV)*(v*v)-
     & 9*(-3+20*NFV)*(v*v*v)+80*NFV*(v*v*v*v))))/(18._dp*w*(u*u))-
     & (9*(u*u*u*u*u*u)*(20*NFV*(omv*omv*omv)-281*(-2-5*v+v*v)+
     & 36*nf*(-2-5*v+v*v))-
     & 9*omv*(u*u*u*u*u)*(1563+3337*v+10*NFV*(7-6*v)*(omv*omv)-
     & 685*(v*v)+12*nf*(-17-35*v+7*(v*v)))+
     & 6*v*(omv*omv*omv)*(-6-4*v+9*(v*v)+
     & 20*NFV*(6-5*v-2*(v*v)+v*v*v))+
     & 3*(u*u*u*u)*(5322-4558*v-16036*(v*v)+
     & 10*NFV*(omv*omv*omv)*(27-54*v+22*(v*v))+11813*(v*v*v)-
     & 1599*(v*v*v*v)+36*nf*
     & (-18+16*v+54*(v*v)-39*(v*v*v)+5*(v*v*v*v)))-
     & omv*(u*u*u)*(4*nf*(-395+898*v+550*(v*v)-351*(v*v*v)+
     & 27*(v*v*v*v))-3*
     & (-6103+12746*v+5024*(v*v)-4485*(v*v*v)+
     & 2*(omv*omv)*(-1224*zeta2+
     & 5*NFV*(36+58*v-57*(v*v)+14*(v*v*v)))+405*(v*v*v*v)))
     & -u*(omv*omv*omv)*
     & (4*nf*(-152+206*v+27*(v*v))+
     & 3*(2684-3656*v+2448*omv*zeta2-449*(v*v)+36*(v*v*v)+
     & 20*NFV*(-12+40*v-10*(v*v)-13*(v*v*v)+4*(v*v*v*v))))-
     & 2*(omv*omv)*(u*u)*(4*nf*(179-363*v-5*(v*v)+27*(v*v*v))-
     & 3*(3234-6004*v+361*(v*v)+1224*zeta2*(2-3*v+v*v)+
     & 464*(v*v*v)-9*(v*v*v*v)-
     & 5*NFV*(72-124*v-v*v+85*(v*v*v)-36*(v*v*v*v)+
     & 4*(v*v*v*v*v)))))/(108._dp*omu*u*w*(omv*omv*omv))-
     & (68*(H1(0)*H1(0)))/9._dp-4*(H1(1)*H1(1)))+
     & (4*H2(0,0)+5*H2(1,0)-(120*(omv*omv)+12*(-245+3*u+242*v)*(u*u)+
     & omv*(3*u*(928-120*v-270*(-1+u)*zeta2)+5*omu*u*(nf*nf)+
     & 2*nf*(-2-155*u+2*v+6*u*v+157*(u*u))))/(72._dp*omu*omv*u))*
     & (H1(1)*H1(1))+G1(0)*G1(0)*(-62+(61*nf)/12._dp+u/(2._dp*omv)-
     & (45*zeta2)/4._dp+(9*G2(0,2))/2._dp+(9*G2(2,0))/2._dp-4*G2(3,0)+
     & 4*G1(3)*H1(0)+((-omv+u*(44+nf*(-1+u)-43*u+3*v))*H1(1))/
     & (2._dp*omu*u)+4*H2(0,0)+5*H2(1,0)+
     & H1(0)*((27*H1(1))/2._dp+(51+10*nf+9*v*(4/omu+1/u+u/(omv*omv)))/
     & 18._dp)-(5*(nf*nf))/72._dp+
     & G1(2)*((omv+u*(-14+13*u-3*v))/(2._dp*omu*u)-(9*H1(1))/2._dp+
     & ((omv+u*(-18*u+v))*H1(0))/(2._dp*(u*u)))+
     & G1(1)*(-((omv+u*(-9*u+v))*H1(0))/(2._dp*(u*u))-
     & ((omv+u*(-9*u+v))*H1(1))/(2._dp*(u*u)))+
     & ((omv+u*(-9*u+v))*G2(1,2))/(2._dp*(u*u))+(9*(G1(2)*G1(2)))/4._dp+
     & (67*(H1(0)*H1(0)))/4._dp+(9*(H1(1)*H1(1)))/4._dp)+
     & G2(0,2)*((232*H2(0,0))/9._dp-(2*(omv+u*(-10*u+v))*H2(1,0))/(u*u)+
     & (H2(0,1)*(-9+9*v-9*u*v+61*(u*u)))/(9._dp*(u*u))-
     & (H1(1)*(8+u*(386-45*nf-150*v)-8*v+(-394+45*nf+4*v)*(u*u)))/
     & (18._dp*omu*u)+H1(0)*(9*H1(1)+
     & (-1226+40/omw+89*nf+12/u-(63+u)/omv-7*v+
     & (162*v)/omu-(4*v)/u-(nf*v)/u+9*u*v-27/w+(27*u)/w-
     & 10/(omw*omw)-(omv*(16+nf))/(u*u)-20*NFV*v*(u*u)+
     & (u*(1+10*NFV*(-2+3*v-6*(v*v)+2*(v*v*v))))/(omv*omv))/18._dp
     & )+(12*(-75+2*nf)*v*(omv*omv*omv)*(u*u)+
     & omu*(-6*nf*u*(omv*omv*omv*omv)-
     & 120*NFV*u*(1+v)*(omv*omv*omv*omv*omv)-
     & 3*(omv*omv)*(-2224+216*nf+60*NFV-2529*u+843*(u*u))*
     & (u*u*u)+54*(u*u*u*u*u)*(281+6*nf*(-2-5*v+v*v))-
     & 27*omv*(u*u*u*u)*(281*(2+u)+4*nf*(-5-17*v+4*(v*v)))+
     & omv*omv*omv*(-6*zeta2*(18-18*v+18*u*v+1319*(u*u))+
     & 3*u*(54-34*v+6*(202+5*NFV*(-3+4*v))*(u*u*u)+
     & 60*NFV*(u*u*u*u)-18*(v*v)+
     & u*u*(-1220-36*nf*(-4+v)+387*v+
     & 10*NFV*(3-11*v+10*(v*v)))+
     & u*(-2795-327*v+2*nf*(97+18*v)+nf*nf+18*(v*v)+
     & 20*NFV*(5+v-5*(v*v)+2*(v*v*v)))))))/
     & (108._dp*omu*(omv*omv*omv)*(u*u))+(17*(H1(0)*H1(0)))/2._dp+
     & (9*(H1(1)*H1(1)))/2._dp)+G2(2,0)*
     & (H2(1,0)*(21+v/u+omv/(u*u))+
     & (H2(0,1)*(5-6*u-5*v+5*u*v-71*(u*u)))/(9._dp*(u*u))+
     & (H2(0,0)*(5-5*v+10*omu*NFV*(-2+u+2*v)+u*(-2+5*v)+
     & 21*(u*u)))/(3._dp*(u*u))-
     & (H1(1)*(8+u*(386-45*nf-150*v)-8*v+(-394+45*nf+4*v)*(u*u)))/
     & (18._dp*omu*u)+(12*(-75+2*nf)*v*(omv*omv*omv)*(u*u)+
     & omu*(-6*nf*u*(omv*omv*omv*omv)-
     & 120*NFV*u*(1+v)*(omv*omv*omv*omv*omv)-
     & 3*(omv*omv)*(-2224+216*nf+60*NFV-2529*u+843*(u*u))*
     & (u*u*u)+54*(u*u*u*u*u)*(281+6*nf*(-2-5*v+v*v))-
     & 27*omv*(u*u*u*u)*(281*(2+u)+4*nf*(-5-17*v+4*(v*v)))+
     & omv*omv*omv*(6*zeta2*(8-8*v+4*u*(3+2*v)-1037*(u*u))+
     & 3*u*(54-34*v+6*(202+5*NFV*(-3+4*v))*(u*u*u)+
     & 60*NFV*(u*u*u*u)-18*(v*v)+
     & u*u*(-1220-36*nf*(-4+v)+387*v+
     & 10*NFV*(3-11*v+10*(v*v)))+
     & u*(-2795-327*v+2*nf*(97+18*v)+nf*nf+18*(v*v)+
     & 20*NFV*(5+v-5*(v*v)+2*(v*v*v)))))))/
     & (108._dp*omu*(omv*omv*omv)*(u*u))+
     & H1(0)*(9*H1(1)+(153*omu*v*(omv*omv)*(u*u)+
     & 2*v*(omv*omv)*(u*u)*(-16+15*NFV*(-1+3*u+u*u))+
     & omu*omu*(-((nf+20*NFV)*(omv*omv*omv))-
     & omv*(-63+u)*(u*u)+
     & u*u*u*(1+10*NFV*
     & (2-9*v+12*(v*v)-14*(v*v*v)+6*(v*v*v*v)))+
     & omv*omv*(9*v*(u*u*u)+20*NFV*v*(u*u*u*u)-
     & (-1+v)*(63+20*v+9*(v*v)+20*NFV*(v*v*v))+
     & u*u*(89*nf+3*
     & (-448+5*v+6*(v*v)+20*NFV*(3-v*v+v*v*v)))+
     & u*(36+51*v-nf*v+2*(v*v)+9*(v*v*v)+
     & 10*NFV*(-18+10*v+3*(v*v)-6*(v*v*v)+2*(v*v*v*v))))
     & ))/(18._dp*(omu*omu)*(omv*omv)*(u*u)))+
     & (17*(H1(0)*H1(0)))/2._dp+(9*(H1(1)*H1(1)))/2._dp)+
     & G1(2)*G1(2)*((9*G2(0,2))/2._dp+(9*G2(2,0))/2._dp-4*G2(3,0)+4*G1(3)*H1(0)+
     & 4*H2(0,0)+5*H2(1,0)+((omv+u*(-9*u+v))*G2(1,2))/(2._dp*(u*u))+
     & (H1(1)*(3*(-1+v)+u*(9+2*nf+9*v)-2*(3+nf)*(u*u)))/
     & (2._dp*omu*u)-(120*(omv*omv)+12*(-245+3*u+242*v)*(u*u)+
     & omv*(3*u*(928-120*v-270*(-1+u)*zeta2)+5*omu*u*(nf*nf)+
     & 2*nf*(-2-155*u+2*v+6*u*v+157*(u*u))))/(72._dp*omu*omv*u)+
     & H1(0)*((H1(1)*(-2+2*v-2*u*v+45*(u*u)))/(2._dp*(u*u))+
     & (u*(12-33*v+omu*nf*v+3*u*(3*omu*nf+37*(3+v)))*
     & (omv*omv)+(-30+omu*nf)*(omv*omv*omv)-
     & 9*(u*u*u)*(35+(-71+u)*v+35*(v*v)))/
     & (18._dp*omu*(omv*omv)*(u*u)))+
     & ((-4+4*v-4*u*v+71*(u*u))*(H1(0)*H1(0)))/(4._dp*(u*u))+
     & (27*(H1(1)*H1(1)))/4._dp)+H1(0)*H1(0)*
     & ((68*H2(0,0))/9._dp+(85*H2(1,0))/9._dp+
     & (H1(1)*(omv*omv*(-17+35*v+u*(157+18*nf+105*v)-18*nf*(u*u))-
     & 2*(u*u)*(70+(-149+9*u)*v+70*(v*v))))/(18._dp*omu*u*(omv*omv))
     & +(omv*omv*(12*(57-2*nf)*v-
     & u*(21801-2156*nf-2052*v+72*nf*v+5814*zeta2+
     & 36*(nf*nf))+(5814*zeta2+36*(nf*nf))*(u*u))+
     & u*u*(4*nf*(-539+2*(536+3*u)*v-539*(v*v))+
     & 3*(7335-14374*v-4*u*(17+40*v)+7267*(v*v))))/
     & (216._dp*omu*u*(omv*omv))+(35*(H1(1)*H1(1)))/4._dp)+
     & G2(1,2)*(((omv+u*(6-81*u+v)+30*NFV*(-1+u)*(-2+u+2*v))*
     & H2(0,0))/(9._dp*(u*u))-(10*(omv+u*(-9*u+v))*H2(0,1))/(9._dp*(u*u))+
     & (H2(1,0)*(31+6*u-31*v+31*u*v+30*omu*NFV*(-2+u+2*v)-
     & 155*(u*u)))/(9._dp*(u*u))+
     & (H1(1)*(6*(-1+v)-6*u*(4+9*v)+5*omu*nf*(omv+u*(-9*u+v))+
     & (450-84*v)*(u*u)+4*(-105+v)*(u*u*u)))/(18._dp*omu*(u*u))+
     & H1(0)*(H1(1)*(-9+v/u+omv/(u*u))+
     & (omv*(-36+5*omu*nf)+
     & u*(-2-106*v+5*omu*nf*v+u*(496-45*omu*nf+24*v)-
     & 2*(229+81*v)*(u*u)+80*v*(u*u*u)))/(18._dp*omu*(u*u)))+
     & (360*omw*NFV*(-2+u+2*v)*zeta2*(omu*omu)*(omv*omv*omv)-
     & omu*(-7587*omv*omw*(-1+u*u)*(u*u*u)-
     & 3*omw*(omv*omv)*(u*u)*
     & (1379+307*u-2529*(u*u)+843*(u*u*u))+
     & 15174*omw*(-1+u)*(u*u*u*u)+
     & omv*omv*omv*(60*(-1+u)*u+
     & omw*(6*zeta2*(u*(48-85*v)+85*(-1+v)-951*(u*u))+
     & 3*(391-411*v+(-1220+387*v)*(u*u*u)+1212*(u*u*u*u)+
     & u*(-208+357*v-36*(v*v))+18*(v*v)+
     & u*u*(-1567-185*v+18*(v*v)))+
     & 30*NFV*(-6+4*v+3*(-3+4*v)*(u*u*u*u)+6*(u*u*u*u*u)-
     & 2*(v*v)+u*u*u*(3-15*v+10*(v*v))+
     & u*(3+v+10*(v*v)-8*(v*v*v))+4*(v*v*v)+
     & u*u*(3+4*v-18*(v*v)+4*(v*v*v))))))+
     & 4*omw*(u*(-60*u*v+nf*(-25+44*v))*(omv*omv*omv)+
     & 19*nf*(omv*omv*omv*omv)+
     & nf*(omv*omv)*(u*u)*(-168+59*v+28*(v*v))+
     & 3*nf*(u*u*u)*(76-48*v-69*(v*v)+
     & 27*(u*u*u)*(-2-5*v+v*v)+59*(v*v*v)+
     & 9*(u*u)*(11+45*v-24*(v*v)+4*(v*v*v))-18*(v*v*v*v)+
     & 9*u*(-7-40*v+39*(v*v)-11*(v*v*v)+v*v*v*v))))/
     & (108._dp*omu*omw*(omv*omv*omv)*(u*u))+
     & (17*(omv+u*(-9*u+v))*(H1(0)*H1(0)))/(18._dp*(u*u))+
     & ((omv+u*(-9*u+v))*(H1(1)*H1(1)))/(2._dp*(u*u)))+
     & G1(3)*((-242*zeta2*H1(1))/9._dp+(16*(-33+2*nf)*H2(0,0))/9._dp+
     & (64*H3(0,0,0))/3._dp+(80*H3(0,1,0))/9._dp+(80*H3(1,0,0))/9._dp-10*H3(1,0,1)+
     & (10*H3(1,1,0))/9._dp+(H2(0,1)*
     & (20*NFV*v*(u*u*u*u*u)+u*u*u*u*(9*v+20*NFV*(1-2*v+4*(v*v)))-
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+20*NFV*(v*v*v))+
     & u*u*u*(24-8*nf+2*v+27*(v*v)+
     & 10*NFV*(-2+7*v-14*(v*v)+12*(v*v*v)))+
     & u*v*(-20+(10-30*NFV)*v+2*(-8+55*NFV)*(v*v)+
     & (9-100*NFV)*(v*v*v)+20*NFV*(v*v*v*v))+
     & u*u*(-24+8*omv*nf-6*(-6+5*NFV)*v+5*(-1+24*NFV)*(v*v)-
     & 9*(-3+20*NFV)*(v*v*v)+80*NFV*(v*v*v*v))))/(18._dp*w*(u*u))+
     & (H2(1,0)*(20*NFV*v*(u*u*u*u*u)+
     & u*u*u*u*(9*v+20*NFV*(1-2*v+4*(v*v)))-
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+20*NFV*(v*v*v))+
     & u*u*u*(24-8*nf+2*v+27*(v*v)+
     & 10*NFV*(-2+7*v-14*(v*v)+12*(v*v*v)))+
     & u*v*(-20+(10-30*NFV)*v+2*(-8+55*NFV)*(v*v)+
     & (9-100*NFV)*(v*v*v)+20*NFV*(v*v*v*v))+
     & u*u*(-24+8*omv*nf-6*(-6+5*NFV)*v+5*(-1+24*NFV)*(v*v)-
     & 9*(-3+20*NFV)*(v*v*v)+80*NFV*(v*v*v*v))))/(18._dp*w*(u*u))-
     & (1920*w*zeta3*(u*u)+6*zeta2*
     & (100*NFV*v*(u*u*u*u*u)+
     & 5*(u*u*u*u)*(9*v+20*NFV*(1-2*v+4*(v*v)))+
     & u*u*u*(-1196+64*nf+10*v+135*(v*v)+
     & 50*NFV*(-2+7*v-14*(v*v)+12*(v*v*v)))+
     & v*(40+8*v-10*(1+10*NFV)*(v*v)+5*(-9+40*NFV)*(v*v*v)-
     & 100*NFV*(v*v*v*v))+
     & u*u*(1156-64*omv*nf-2*(547+75*NFV)*v+
     & 25*(-1+24*NFV)*(v*v)-45*(-3+20*NFV)*(v*v*v)+
     & 400*NFV*(v*v*v*v))+
     & u*(40-216*v+(92-150*NFV)*(v*v)+10*(-8+55*NFV)*(v*v*v)+
     & (45-500*NFV)*(v*v*v*v)+100*NFV*(v*v*v*v*v))))/
     & (108._dp*w*(u*u))+(76._dp/3._dp-(8*nf)/9._dp+8*H1(1))*(H1(0)*H1(0))+
     & (68*(H1(0)*H1(0)*H1(0)))/9._dp+
     & H1(0)*((-4*(-30+nf)*H1(1))/9._dp+
     & (-12*(-16+45*NFV)*u*(1+v)*w*(omv*omv*omv)+
     & omu*(-7587*omv*w*(u*u)*(1+4*u+u*u)+
     & u*w*(omv*omv)*(4137+14253*u+7587*(u*u)-2529*(u*u*u)-
     & 4*nf*(-152+206*v+27*(v*v)))+
     & 3*(omv*omv*omv)*
     & (6*(202+5*NFV*(-3+4*v))*w*(u*u*u)+60*NFV*w*(u*u*u*u)+
     & u*u*(58+(-1220+387*v)*w+
     & 10*NFV*(-15+w*(3-15*v+10*(v*v))))+
     & 2*(-1+w*(1+5*(-1+24*NFV)*v+(-9+20*NFV)*(v*v)-
     & 20*NFV*(v*v*v)))+
     & u*(-22-w*(4103+469*v+2448*zeta2-18*(v*v))+
     & 20*NFV*(3+w*(18+2*v-5*(v*v)+2*(v*v*v)))))+
     & 54*w*(u*u)*(281*u*(1+u)+
     & 2*nf*(-2+8*v+3*(u*u)*(-2-5*v+v*v)-7*(v*v*v)+
     & u*(5-6*v-21*(v*v)+4*(v*v*v))+v*v*v*v))))/
     & (108._dp*omu*u*w*(omv*omv*omv))+4*(H1(1)*H1(1))))+
     & G1(0)*(G2(0,2)*((-30+nf)/2._dp-9*H1(0))+G2(2,0)*((-30+nf)/2._dp-9*H1(0))+
     & G2(3,0)*((-4*(-30+nf))/9._dp+8*H1(0))+
     & ((332-27*nf+(4-4*v)/u-2*v-(77*v)/omu)*H2(1,1))/9._dp-
     & (64*H3(0,0,0))/3._dp-8*H3(0,0,1)-(152*H3(0,1,0))/9._dp-(232*H3(1,0,0))/9._dp-
     & 10*H3(1,0,1)-20*H3(1,1,0)+
     & G2(1,2)*(((-30+nf)*(omv+u*(-9*u+v)))/(18._dp*(u*u))-
     & ((omv+u*(-9*u+v))*H1(0))/(u*u))+
     & (H2(1,0)*(1563-102*nf+6/u-(18*u)/omv+7*v-(153*v)/omu+
     & (12*v)/u-9*u*v+27/w-(27*u)/w+(32*v)/(omu*omu)+
     & (18*u)/(omv*omv)+
     & 10*NFV*((3*(2-5*u)*v)/(omu*omu)+(2*omv*(6+u*u))/u+
     & 2*(-6+v*(u*u)))))/18._dp+
     & (3*(1195+80/omw-324*zeta3+
     & u*((52*(-17+2*nf))/omv+
     & (-80+4*omw*(-50+9*nf)-9*(-41+4*nf)*(omw*omw))/
     & (omw*omw))-(3*(-281+36*nf)*(2+v)*(u*u))/(omv*omv))
     & +30*NFV*(2*(5+v)-u*(1+2*v)+
     & 6*zeta2*(-8+(8-8*v)/u+(4*v)/(omu*omu)+
     & u*v*(-10/(omu*omu)-1/(omv*omv)))-
     & (2*(u*u)*(-9-4*v+v*v))/(omv*omv))+
     & 6*zeta2*((2*(u*(970-336*v)+5*(-97+56*v)+
     & (-485+88*v)*(u*u)))/(omu*omu)+
     & (4*(-5+17*nf*u+7*v)*(omv*omv*omv*omv)+
     & 9*omv*(-281+36*nf)*v*(u*u*u)+
     & 9*(281-36*nf)*v*(u*u*u*u)-
     & 2*v*(omv*omv)*(u*u)*(-485+54*nf-160*v+80*(v*v)))/
     & (u*(omv*omv*omv*omv))))/108._dp+
     & (H2(0,1)*((1112*u)/omv+(3*(9-9*u+296*w))/w+
     & (2529*v*(u*u))/(omv*omv*omv)+
     & v*(-75-82/omu+10/u+71*u+20*NFV*(u*u))-
     & (2*u*(556+5*NFV*(-2+3*v-6*(v*v)+2*(v*v*v))))/(omv*omv)+
     & (-2529*v*(u*u*u)+2*nf*
     & (-31+2*v*(62+27*u-81*(u*u)+81*(u*u*u))+
     & 6*(-31-18*u+27*(u*u))*(v*v)+2*(62+27*u)*(v*v*v)-
     & 31*(v*v*v*v)))/(omv*omv*omv*omv)))/18._dp+
     & (H1(1)*(40*omu*NFV*(1+v)*(omv*omv*omv*omv*omv)+
     & omv*omv*omv*(6-26*v-2754*zeta2*(u*u)-
     & 60*omu*NFV*(u*u*u*u)+
     & u*(2909+413*v-12*nf*(19+3*v)+2754*zeta2-36*(v*v))+
     & 18*(v*v))+omv*(u*u*u)*
     & (2321+5556*v-3206*(v*v)+387*(v*v*v)-
     & 36*nf*(7+20*v-10*(v*v)+v*v*v))+
     & 3*(u*u*u*u)*(-1001-2617*v+2336*(v*v)-
     & 281*u*(-2-5*v+v*v)-404*(v*v*v)+
     & 12*nf*(11+27*v-24*(v*v)+3*u*(-2-5*v+v*v)+4*(v*v*v)))
     & +u*(omv*omv)*(u*
     & (-3919+981*v+732*(v*v)-12*nf*(-25+v+6*(v*v))-
     & 18*(v*v*v))+10*omu*NFV*
     & (3*(u*u)*(3-7*v+4*(v*v))+
     & u*(3+14*v-21*(v*v)+10*(v*v*v))+
     & 2*(-5+4*v+6*(v*v)-7*(v*v*v)+2*(v*v*v*v))))))/
     & (36._dp*omu*u*(omv*omv*omv))+
     & (H2(0,0)*(-81*omu*u*v*w*(omv*omv*omv*omv)+
     & 2*(16+NFV*(30-75*u))*u*v*w*(omv*omv*omv*omv)+
     & omu*omu*(u*(-3+1111*u)*w*(omv*omv*omv)+
     & omv*omv*omv*omv*
     & (30*(1-4*NFV)*v*w+u*(54+1533*w-64*v*w)+
     & (-54+62*v*w)*(u*u))+
     & omv*v*w*(u*u*u)*(2529-40*NFV*v*(3-3*v+v*v))-
     & w*(omv*omv)*(u*u)*(1111+20*NFV*v*(3-6*v+2*(v*v)))+
     & 3*u*w*(-843*v*(u*u*u)-
     & 4*nf*(8-v*(32+9*u-27*(u*u)+27*(u*u*u))+
     & (48+18*u-27*(u*u))*(v*v)-(32+9*u)*(v*v*v)+
     & 8*(v*v*v*v))))))/
     & (18._dp*u*w*(omu*omu)*(omv*omv*omv*omv))+
     & ((-17*H1(0))/2._dp-(9*H1(1))/2._dp)*(G1(2)*G1(2))+
     & G1(3)*((4*(-30+nf)*H1(0))/9._dp-8*(H1(0)*H1(0)))+
     & ((-51*H1(1))/2._dp+(omv*omv*
     & (-18*v-u*(375+2*nf+54*v)+2*nf*(u*u))+
     & 3*(u*u)*(125+(-256+6*u)*v+125*(v*v)))/
     & (18._dp*omu*u*(omv*omv)))*(H1(0)*H1(0))+
     & G1(1)*(H1(0)*(H1(1)*(-9+v/u+omv/(u*u))-
     & ((-30+nf)*(omv+u*(-9*u+v)))/(18._dp*(u*u)))-
     & ((-30+nf)*(omv+u*(-9*u+v))*H1(1))/(18._dp*(u*u))+
     & (-9+v/u+omv/(u*u))*(H1(0)*H1(0)))-(289*(H1(0)*H1(0)*H1(0)))/18._dp+
     & H1(0)*(((18*omv+u*(-1005-26*nf*(-1+u)+987*u-54*v))*H1(1))/
     & (18._dp*omu*u)-8*H2(0,0)-10*H2(1,0)-
     & (6*v*(omv*omv*omv)*(3-13*v+20*NFV*(1+v)*(omv*omv)+
     & 9*(v*v))+9*(u*u*u*u*u*u)*
     & (20*NFV*(omv*omv*omv)-281*(-2-5*v+v*v)+
     & 36*nf*(-2-5*v+v*v))+
     & 3*(u*u*u*u*u)*(36*nf*(17+36*v-42*(v*v)+7*(v*v*v))-
     & 3*(1563+3460*v-4022*(v*v)+685*(v*v*v))-
     & 10*NFV*(21-45*v+97*(v*v)-67*(v*v*v)+18*(v*v*v*v)))-
     & u*(omv*omv*omv)*(4*nf*(-242+224*v+27*(v*v))+
     & 3*(4268-3880*v+3474*omv*zeta2-431*(v*v)+36*(v*v*v)+
     & 20*NFV*(-12+14*v-9*(v*v*v)+4*(v*v*v*v))))+
     & 3*(u*u*u*u)*(5358+8018*v-23590*(v*v)+11813*(v*v*v)-
     & 1599*(v*v*v*v)+
     & 4*nf*(-162-260*v+728*(v*v)-351*(v*v*v)+45*(v*v*v*v))-
     & 10*NFV*(-27+43*v-143*(v*v)+181*(v*v*v)-100*(v*v*v*v)+
     & 22*(v*v*v*v*v)))-
     & omv*omv*(u*u)*(8*nf*(269-306*v+4*(v*v)+27*(v*v*v))+
     & 3*(-3474*zeta2*(2-3*v+v*v)+
     & 2*(-4836+5861*v-642*(v*v)-384*(v*v*v)+
     & 9*(v*v*v*v))+
     & 10*NFV*(72-118*v+17*(v*v)+51*(v*v*v)-32*(v*v*v*v)+
     & 4*(v*v*v*v*v))))-
     & omv*(u*u*u)*(4*nf*(-485+40*v+772*(v*v)-351*(v*v*v)+
     & 27*(v*v*v*v))+
     & 3*(7759-4700*v+3474*zeta2*(omv*omv)-6976*(v*v)+
     & 4325*(v*v*v)-405*(v*v*v*v)-
     & 10*NFV*(36-96*v-29*(v*v)+114*(v*v*v)-69*(v*v*v*v)+
     & 14*(v*v*v*v*v)))))/(108._dp*omu*u*w*(omv*omv*omv))-
     & (35*(H1(1)*H1(1)))/2._dp)+((-30+nf)*(H1(1)*H1(1)))/2._dp+
     & G1(2)*((15-nf/2._dp)*H1(1)+
     & ((-30+nf)*(2-2*v-u*(-2+nf+6*v)+(-4+nf)*(u*u)))/
     & (36._dp*omu*u)+H1(0)*(26*H1(1)+
     & (omu*nf*(omv+u*(-17*u+v))+
     & 6*(u*(2-7*v)+5*(-1+v)+(127+14*v)*(u*u)-124*(u*u*u))
     & )/(18._dp*omu*(u*u)))-
     & ((omv+u*(-26*u+v))*(H1(0)*H1(0)))/(u*u)+9*(H1(1)*H1(1)))-
     & (9*(H1(1)*H1(1)*H1(1)))/2._dp)+
     & G1(2)*(G2(0,2)*((-30+nf)/2._dp-9*H1(0)-9*H1(1))+
     & G2(2,0)*((-30+nf)/2._dp-9*H1(0)-9*H1(1))+
     & G2(3,0)*((-4*(-30+nf))/9._dp+8*H1(0)+8*H1(1))+
     & (H2(1,0)*(304+63/omv-40/omw+72/u-(19*u)/omv-4*v-
     & (137*v)/omu-(25*v)/u-(32*v)/(omu*omu)+
     & 30*NFV*(7+(-7+4*v)/u+((-2+5*u)*v)/(omu*omu)-
     & (u*v)/(omv*omv))+(19*u)/(omv*omv)+10/(omw*omw)+
     & 41/(u*u)-(41*v)/(u*u)+(nf*(omv+u*(-45*u+v)))/(u*u)))/18._dp
     & +G2(1,2)*(((-30+nf)*(omv+u*(-9*u+v)))/(18._dp*(u*u))-
     & ((omv+u*(-9*u+v))*H1(0))/(u*u)-
     & ((omv+u*(-9*u+v))*H1(1))/(u*u))+
     & ((omv+u*(6+99*u+v)+30*NFV*(-1+u)*(-2+u+2*v))*H3(0,0,0))/
     & (9._dp*(u*u))-(2*(omv+u*(-81*u+v))*H3(0,0,1))/(9._dp*(u*u))+
     & (omu*(5+3*u-5*v+10*NFV*(-2+u+2*v))*H3(1,0,0))/(3._dp*(u*u))+
     & (4*(omv+u*(u+v))*H3(1,1,0))/(u*u)+
     & (H3(0,1,0)*(23+6*u-23*v+23*u*v+30*omu*NFV*(-2+u+2*v)+
     & 61*(u*u)))/(9._dp*(u*u))+
     & (2*H3(1,0,1)*(-2-3*u+2*v-2*u*v+95*(u*u)))/(9._dp*(u*u))+
     & H1(1)*(-8*H2(0,0)-10*H2(1,0)+
     & (6*omu*omv*zeta2*(80+12*u-80*v+80*u*v+1069*(u*u))+
     & 12*u*(2*omv*nf*u*(-29+28*u-3*v)+(7+2*nf)*(omv*omv)+
     & u*(638-654*u-471*v+645*u*v+9*(u*u)-167*(v*v))))/
     & (108._dp*omu*omv*(u*u)))+
     & (H2(0,0)*(-63*omu*v*(omv*omv*omv*omv)*(u*u)+
     & 2*v*(omv*omv*omv*omv)*(u*u)*(16-15*NFV*(-1+3*u+u*u))+
     & omu*omu*(4*(3*nf-5*NFV)*(omv*omv*omv*omv*omv)+
     & (3-1111*u)*(omv*omv*omv)*(u*u)+
     & 9*(281-36*nf*v)*(u*u*u*u*u)+
     & omv*(u*u*u*u)*(-2529-2529*u+324*nf*v+20*NFV*v+
     & 60*NFV*(v*v)-60*NFV*(v*v*v)+20*NFV*(v*v*v*v))+
     & omv*omv*(u*u*u)*
     & (1111+2529*u-108*nf*v-
     & 20*NFV*(-1+6*(v*v)-7*(v*v*v)+3*(v*v*v*v)))-
     & omv*omv*omv*omv*
     & (9*v*(u*u*u)-(-1+v)*
     & (164+20*v+9*(v*v)+20*NFV*(v*v*v))+
     & u*u*(-2209+144*nf+15*v+18*(v*v)+
     & 30*NFV*(9-2*(v*v)+2*(v*v*v)))+
     & u*(86-2*(-85+6*nf)*v+2*(v*v)+9*(v*v*v)+
     & 10*NFV*(-27+14*v+3*(v*v)-6*(v*v*v)+2*(v*v*v*v)))))
     & ))/(18._dp*(omu*omu)*(omv*omv*omv*omv)*(u*u))+
     & (H2(0,1)*(72*v*(omv*omv*omv*omv)*(omw*omw)*(u*u)+
     & omu*((63-1111*u)*(omv*omv*omv)*(omw*omw)*(u*u)+
     & 5*(omv*omv*omv*omv*omv)*(omw*omw)*
     & (nf-2*NFV*(2+3*(u*u)))+
     & (1111+2529*u-108*nf*v)*(omv*omv)*(omw*omw)*(u*u*u)-
     & 9*omv*(281+281*u-36*nf*v)*(omw*omw)*(u*u*u*u)+
     & 9*(281-36*nf*v)*(omw*omw)*(u*u*u*u*u)+
     & omv*omv*omv*omv*
     & (-10*(u*u)+40*omw*(u*u)+
     & omw*omw*(20*NFV*v*(u*u*u*u)+
     & u*u*u*(9*v+20*NFV*(1-v+3*(v*v)))-
     & (-1+v)*(2+20*v+9*(v*v)+20*NFV*(v*v*v))+
     & u*u*(595-45*nf+11*v+(18-60*NFV)*(v*v)+
     & 60*NFV*(v*v*v))+
     & u*(-90+(22+5*nf)*v+2*(v*v)+9*(v*v*v)+
     & 10*NFV*(3-2*v+3*(v*v)-6*(v*v*v)+2*(v*v*v*v)))))
     & )))/(18._dp*omu*(omv*omv*omv*omv)*(omw*omw)*(u*u))+
     & (-4*omu*nf*(100*v*(omv*omv*omv*omv)*(u*u)+
     & omu*(6*zeta2*(omv*omv*omv*omv*omv)+
     & u*(omv*omv*omv*omv)*
     & (-19+u*(337-27*v-162*zeta2)+v*(19+6*zeta2)-
     & 108*(u*u))-27*(9*u+6*v*zeta2)*(omv*omv)*(u*u*u)+
     & 3*(55+27*u)*(omv*omv*omv)*(u*u*u)+
     & 486*omv*v*zeta2*(u*u*u*u)-486*v*zeta2*(u*u*u*u*u)))+
     & (omu*v*(4536+3306*zeta2)*(omv*omv*omv*omv)*(omw*omw)*
     & (u*u)-12*v*zeta2*(omv*omv*omv*omv)*(omw*omw)*(u*u)*
     & (16+15*NFV*(3-7*u+u*u))+
     & omu*omu*(-720*NFV*zeta3*(omv*omv*omv*omv*omv)*
     & (omw*omw)+3*(1645+843*u+2184*zeta2)*(omv*omv*omv)*
     & (omw*omw)*(u*u*u)+
     & 15174*omv*(1+u)*zeta2*(omw*omw)*(u*u*u*u)-
     & 15174*zeta2*(omw*omw)*(u*u*u*u*u)+
     & 120*NFV*zeta2*(omv*omv*omv*omv*omv*omv)*(omw*omw)*
     & (1+v+v*v)-
     & omv*omv*(omw*omw)*(u*u*u)*
     & (3*(2529*u+6*(364+843*u)*zeta2)+
     & 10*NFV*(-57+30*v+39*(v*v)-6*u*(-9-4*v+v*v)-
     & 12*(v*v*v)+
     & 6*zeta2*(2-3*v+12*(v*v)-14*(v*v*v)+6*(v*v*v*v))))
     & -omv*omv*omv*omv*
     & (-180*zeta2*(u*u)+720*omw*zeta2*(u*u)+
     & omw*omw*(3*(-44*(-1+v)*zeta3+1212*(u*u*u)+
     & u*u*(-6494+369*v-928*zeta3+
     & 10*NFV*(11+3*v+12*zeta3-2*(v*v)))+
     & u*(405-405*v-24*zeta3+44*v*zeta3+
     & 20*NFV*(-5+4*v-18*zeta3+12*v*zeta3+v*v)))
     & +6*zeta2*
     & (-39+59*v+9*v*(u*u*u)+20*NFV*v*(u*u*u*u)-
     & 11*(v*v)-9*(v*v*v)+
     & u*u*(1062-5*v+18*(v*v)+
     & 60*NFV*(-4-v*v+v*v*v))+
     & u*(-165+50*v+2*(v*v)+9*(v*v*v)+
     & 10*NFV*
     & (24-14*v+3*(v*v)-6*(v*v*v)+2*(v*v*v*v)))))
     & )))/(omw*omw))/
     & (108._dp*(omu*omu)*(omv*omv*omv*omv)*(u*u))+
     & G1(3)*(H1(0)*((4*(-30+nf))/9._dp-8*H1(1))-8*(H1(0)*H1(0)))+
     & (H1(1)*(-53._dp/2._dp+v/u+omv/(u*u))+
     & ((-2*omu*nf-u*(40+670*u-79*v+162*u*v)+2*nf*v*(omu*omu))*
     & (omv*omv)+57*(omv*omv*omv)+
     & u*u*u*(653+2*(-662+9*u)*v+653*(v*v)))/
     & (18._dp*omu*(omv*omv)*(u*u)))*(H1(0)*H1(0))+
     & (17*(omv+u*(-18*u+v))*(H1(0)*H1(0)*H1(0)))/(18._dp*(u*u))+
     & ((3*omv+u*(-9+2*nf*(-1+u)+6*u-9*v))*(H1(1)*H1(1)))/
     & (2._dp*omu*u)+H1(0)*(-8*H2(0,0)-10*H2(1,0)+
     & (H1(1)*(-(u*(-6-6*v+omu*nf*v+3*u*(132+9*omu*nf+64*v))*
     & (omv*omv))+(30-omu*nf)*(omv*omv*omv)+
     & 18*(u*u*u)*(20+(-41+u)*v+20*(v*v))))/
     & (18._dp*omu*(omv*omv)*(u*u))+
     & (4*v*(omv*omv*omv*omv)*
     & (19*nf+15*NFV*(3+v-12*zeta2+2*(v*v)))+
     & omv*omv*omv*(6*(612-653*v+60*NFV*(-4+3*v))*zeta2*
     & (u*u*u)+72*(-47+5*NFV)*zeta2*(u*u*u*u)+
     & 180*NFV*(u*u*u*u*u*u*u)+
     & v*(-534*(-1+v)*zeta2-3*(391-411*v+18*(v*v)))+
     & u*(-1113+2718*v+534*zeta2-1356*v*zeta2-2250*(v*v)+
     & 1068*zeta2*(v*v)+4*nf*(19-38*v+32*(v*v))+
     & 162*(v*v*v)-
     & 30*NFV*(-6+13*v-9*(v*v)+
     & 12*zeta2*(2-7*v+4*(v*v))+20*(v*v*v)-12*(v*v*v*v))
     & )+30*NFV*(u*u)*
     & (-9+7*v-29*(v*v)+12*zeta2*(5-8*v+2*(v*v))+
     & 48*(v*v*v)-12*(v*v*v*v)))+
     & omv*omv*(u*u)*(4*nf*(-19-291*v+237*(v*v)+64*(v*v*v))+
     & 6*zeta2*(-137+927*v-879*(v*v)+89*(v*v*v))+
     & 3*(475+4311*v-3407*(v*v)-1414*(v*v*v)+54*(v*v*v*v)))+
     & 3*(u*u*u*u)*(-6660+19323*v+690*NFV*v-10039*(v*v)-
     & 2960*NFV*(v*v)-843*(u*u*u)*(-2-5*v+v*v)-6598*(v*v*v)+
     & 4180*NFV*(v*v*v)+4379*(v*v*v*v)-3140*NFV*(v*v*v*v)-
     & u*u*(3*(1001+2055*v-3741*(v*v)+685*(v*v*v))+
     & 10*NFV*(15-99*v+119*(v*v)-77*(v*v*v)+18*(v*v*v*v)))
     & -405*(v*v*v*v*v)+1130*NFV*(v*v*v*v*v)+
     & 4*nf*(132-385*v+113*(v*v)+27*(u*u*u)*(-2-5*v+v*v)+
     & 248*(v*v*v)+9*(u*u)*(11+21*v-39*(v*v)+7*(v*v*v))-
     & 117*(v*v*v*v)+
     & u*(-63-17*v+512*(v*v)-315*(v*v*v)+45*(v*v*v*v))+
     & 9*(v*v*v*v*v))+
     & u*(2355+167*v-16582*(v*v)+10601*(v*v*v)-
     & 1599*(v*v*v*v)-
     & 20*NFV*(-6+73*v-149*(v*v)+135*(v*v*v)-
     & 64*(v*v*v*v)+11*(v*v*v*v*v)))-140*NFV*(v*v*v*v*v*v))
     & +omv*(u*u*u)*(4*nf*
     & (-342+1007*v-775*(v*v)+53*(v*v*v)+54*(v*v*v*v))+
     & 3*(5518-(16371+130*NFV)*v+17*(773+70*NFV)*(v*v)-
     & (1487+2050*NFV)*(v*v*v)+6*(-137+275*NFV)*(v*v*v*v)+
     & (18-520*NFV)*(v*v*v*v*v)+40*NFV*(v*v*v*v*v*v))))/
     & (108._dp*omu*omw*(omv*omv*omv)*(u*u))+
     & ((omv+u*(-36*u+v))*(H1(1)*H1(1)))/(2._dp*(u*u)))-
     & (9*(H1(1)*H1(1)*H1(1)))/2._dp)+
     & ((3*(-1+v)+u*(9+2*nf+9*v)-2*(3+nf)*(u*u))*(H1(1)*H1(1)*H1(1)))/
     & (6._dp*omu*u)+H1(0)*((76._dp/3._dp-(8*nf)/9._dp)*H2(0,0)-
     & (5*(-57+2*nf)*H2(1,0))/9._dp+
     & H1(1)*(8*H2(0,0)+10*H2(1,0)+
     & (omv*omv*(nf*(4+u*(343-18*v)-6*v)-6*omu*u*(nf*nf)+
     & 3*(-38+58*v+u*(-986+174*v-342*zeta2)+342*zeta2*(u*u))
     & )+u*u*(nf*(-347+2*(346+u)*v-347*(v*v))+
     & 12*(259-510*v-u*(3+2*v)+256*(v*v))))/
     & (36._dp*omu*u*(omv*omv)))+
     & (3*(-21320-240/omw+2529*u-(1320*u)/omv+(600*u)/omw+
     & 120*NFV*u-(840*NFV*u)/omv+1107*v-(4536*v)/omu-
     & 30*NFV*v-(1215*v)/u+(300*NFV*v)/u-60*NFV*u*v+5676*zeta3-
     & (1461*u)/(omv*omv)+(720*NFV*u)/(omv*omv)+
     & (240*u)/(omw*omw)-60*NFV*(v*v)+(60*NFV*(v*v))/u+
     & 6*zeta2*(184+63/omv+(-28-3*(-39+40*NFV)*v)/u+135/w+
     & (4*v*(2529+25*NFV*(omv*omv*omv))*(u*u))/
     & (omv*omv*omv)-
     & (v*(680-1001*u+30*NFV*(-2+5*u)+289*(u*u)))/
     & (omu*omu)-(10116*v*(u*u*u))/(omv*omv*omv*omv)+
     & 5*u*(888/omv+(55-20*NFV)*v-27/w+
     & (-888+NFV*(20-22*v+20*(v*v)))/(omv*omv))))+
     & 4*nf*(926+(222*u)/omv-(81*(1+3*omw)*u)/omw-81*v+
     & (300*v)/omu+(57*v)/u+(39*u)/(omv*omv)+
     & (18*zeta2*(-17+4*v*(17+27*u-81*(u*u)+81*(u*u*u))+
     & 6*(-17-36*u+54*(u*u))*(v*v)+4*(17+27*u)*(v*v*v)-
     & 17*(v*v*v*v)))/(omv*omv*omv*omv)))/324._dp+
     & ((omv*omv*(-2+3*v+u*(7+2*nf+9*v)-2*nf*(u*u))-
     & u*u*(5+(-11+u)*v+5*(v*v)))*(H1(1)*H1(1)))/
     & (2._dp*omu*u*(omv*omv))+(9*(H1(1)*H1(1)*H1(1)))/2._dp)+
     & G1(1)*(-((omv+u*(6-81*u+v)+30*NFV*(-1+u)*(-2+u+2*v))*
     & H3(0,0,0))/(9._dp*(u*u))+
     & (2*(omv+u*(-9*u+v))*H3(0,0,1))/(9._dp*(u*u))+
     & ((-23*omv+u*(-6+83*u-23*v)-30*omu*NFV*(-2+u+2*v))*
     & H3(0,1,0))/(9._dp*(u*u))-
     & ((omv+u*(6-81*u+v)+30*NFV*(-1+u)*(-2+u+2*v))*H3(1,0,0))/
     & (9._dp*(u*u))+(10*(omv+u*(-9*u+v))*H3(1,0,1))/(9._dp*(u*u))+
     & ((-31*omv+u*(-6+155*u-31*v)-30*omu*NFV*(-2+u+2*v))*
     & H3(1,1,0))/(9._dp*(u*u))+
     & (H2(1,1)*(-3*omv*(-6+omu*nf)+
     & u*(-3+57*v-3*omu*nf*v+9*u*(-40+3*omu*nf+3*v)+
     & (345-2*v)*(u*u))))/(9._dp*omu*(u*u))+
     & (H2(1,0)*(omv*(93-7*omu*nf)+
     & u*(-55+u*(-1009+63*omu*nf-81*v)+220*v-7*omu*nf*v+
     & (971+162*v)*(u*u)-80*v*(u*u*u))))/(18._dp*omu*(u*u))+
     & (H2(0,1)*(107-194*v+(20*omu*NFV+11*nf*(-1+u))*(omv*omv)-
     & 10*omu*omv*NFV*u*(3+2*(u*u))+
     & (71-20*omu*NFV)*v*(u*u*u*u)+u*(-57+136*v-96*(v*v))+
     & u*u*(-1599+nf*(99-88*v)+10*omu*NFV*(3-2*v)+1501*v-
     & 75*(v*v))+86*(v*v)+
     & u*u*u*(1549-99*nf-146*v+(71-20*omu*NFV)*(v*v))))/
     & (18._dp*w*(u*u))+(H2(0,0)*(164-308*v+
     & 4*(5*omu*NFV+3*nf*(-1+u))*(omv*omv)-
     & 10*omu*omv*NFV*u*(3+2*(u*u))-(9+20*omu*NFV)*v*(u*u*u*u)+
     & u*(-106+234*v-145*(v*v))+143*(v*v)+
     & u*u*(-1888+10*omu*NFV*(3-2*v)+1647*v-12*nf*(-9+8*v)+
     & 11*(v*v))+u*u*u*
     & (1830-108*nf+20*v-(9+20*omu*NFV)*(v*v))))/(18._dp*w*(u*u))
     & +(6*omu*(51*omv+30*omu*NFV*(-2+u+2*v)+
     & u*(6-335*u+51*v))*w*zeta3+
     & 6*zeta2*(49-178*v+2*omu*nf*
     & ((-1+u)*(omv*omv)+(9-9*u-8*v)*(u*u))-
     & 62*v*(u*u*u*u*u)+u*(-179+500*v-257*(v*v))+133*(v*v)-
     & 2*(u*u*u*u)*(149-93*v+31*(v*v))+
     & 3*(u*u)*(-29-68*v+42*(v*v))+
     & u*u*u*(515-242*v+124*(v*v))+
     & 10*NFV*(omu*omu)*(4*(omv*omv)-omv*u*(15+4*(u*u))-
     & u*u*(-15+4*v*(1+u*u)+4*u*(v*v)))))/(54._dp*omu*w*(u*u))+
     & (H1(1)*(-360*omw*NFV*(-2+u+2*v)*zeta2*(omu*omu)*
     & (omv*omv*omv)+omu*
     & (-7587*omv*omw*(-1+u*u)*(u*u*u)-
     & 3*omw*(omv*omv)*(u*u)*
     & (1379+307*u-2529*(u*u)+843*(u*u*u))+
     & 15174*omw*(-1+u)*(u*u*u*u)+
     & omv*omv*omv*(60*(-1+u)*u+
     & omw*(6*zeta2*(u*(48-85*v)+85*(-1+v)-951*(u*u))+
     & 3*(391-411*v+(-1220+387*v)*(u*u*u)+
     & 1212*(u*u*u*u)+u*(-208+357*v-36*(v*v))+
     & 18*(v*v)+u*u*(-1567-185*v+18*(v*v)))+
     & 30*NFV*(-6+4*v+3*(-3+4*v)*(u*u*u*u)+
     & 6*(u*u*u*u*u)-2*(v*v)+
     & u*u*u*(3-15*v+10*(v*v))+
     & u*(3+v+10*(v*v)-8*(v*v*v))+4*(v*v*v)+
     & u*u*(3+4*v-18*(v*v)+4*(v*v*v))))))-
     & 4*omw*(u*(-60*u*v+nf*(-25+44*v))*(omv*omv*omv)+
     & 19*nf*(omv*omv*omv*omv)+
     & nf*(omv*omv)*(u*u)*(-168+59*v+28*(v*v))+
     & 3*nf*(u*u*u)*(76-48*v-69*(v*v)+
     & 27*(u*u*u)*(-2-5*v+v*v)+59*(v*v*v)+
     & 9*(u*u)*(11+45*v-24*(v*v)+4*(v*v*v))-18*(v*v*v*v)+
     & 9*u*(-7-40*v+39*(v*v)-11*(v*v*v)+v*v*v*v)))))/
     & (108._dp*omu*omw*(omv*omv*omv)*(u*u))+
     & (-((omv+u*(-9*u+v))*H1(0))/(2._dp*(u*u))-
     & ((omv+u*(-9*u+v))*H1(1))/(2._dp*(u*u)))*(G1(2)*G1(2))+
     & (((-57+2*nf)*(omv+u*(-9*u+v)))/(18._dp*(u*u))-
     & (35*(omv+u*(-9*u+v))*H1(1))/(18._dp*(u*u)))*(H1(0)*H1(0))-
     & (17*(omv+u*(-9*u+v))*(H1(0)*H1(0)*H1(0)))/(18._dp*(u*u))+
     & ((-30+nf)*(omv+u*(-9*u+v))*(H1(1)*H1(1)))/(18._dp*(u*u))+
     & G1(2)*(H1(0)*(-((-30+nf)*(omv+u*(-9*u+v)))/(18._dp*(u*u))+
     & (2*(omv+u*(-9*u+v))*H1(1))/(u*u))-
     & ((-30+nf)*(omv+u*(-9*u+v))*H1(1))/(18._dp*(u*u))+
     & (-9+v/u+omv/(u*u))*(H1(0)*H1(0))+
     & (-9+v/u+omv/(u*u))*(H1(1)*H1(1)))+
     & H1(0)*(((-29+nf)*(omv+u*(-9*u+v))*H1(1))/(6._dp*(u*u))+
     & (-360*omw*NFV*(-2+u+2*v)*zeta2*(omu*omu)*(omv*omv*omv)+
     & 3*omu*(-2529*omv*omw*(-1+u*u)*(u*u*u)-
     & omw*(omv*omv)*(u*u)*
     & (1379+307*u-2529*(u*u)+843*(u*u*u))+
     & 5058*omw*(-1+u)*(u*u*u*u)+
     & omv*omv*omv*(20*(-1+u)*u+
     & omw*(391-208*u-411*v+357*u*v+
     & 6*zeta2*(-39+16*u+39*v-39*u*v-221*(u*u))-
     & 1567*(u*u)-185*v*(u*u)-1220*(u*u*u)+
     & 387*v*(u*u*u)+1212*(u*u*u*u)+18*(v*v)-36*u*(v*v)+
     & 18*(u*u)*(v*v)+
     & 10*NFV*(-6+4*v+3*(-3+4*v)*(u*u*u*u)+
     & 6*(u*u*u*u*u)-2*(v*v)+
     & u*u*u*(3-15*v+10*(v*v))+
     & u*(3+v+10*(v*v)-8*(v*v*v))+4*(v*v*v)+
     & u*u*(3+4*v-18*(v*v)+4*(v*v*v))))))-
     & 4*omw*(u*(-60*u*v+nf*(-25+44*v))*(omv*omv*omv)+
     & 19*nf*(omv*omv*omv*omv)+
     & nf*(omv*omv)*(u*u)*(-168+59*v+28*(v*v))+
     & 3*nf*(u*u*u)*(76-48*v-69*(v*v)+
     & 27*(u*u*u)*(-2-5*v+v*v)+59*(v*v*v)+
     & 9*(u*u)*(11+45*v-24*(v*v)+4*(v*v*v))-18*(v*v*v*v)+
     & 9*u*(-7-40*v+39*(v*v)-11*(v*v*v)+v*v*v*v))))/
     & (108._dp*omu*omw*(omv*omv*omv)*(u*u))-
     & (3*(omv+u*(-9*u+v))*(H1(1)*H1(1)))/(2._dp*(u*u)))-
     & ((omv+u*(-9*u+v))*(H1(1)*H1(1)*H1(1)))/(2._dp*(u*u)))+
     & (9*(H1(1)*H1(1)*H1(1)*H1(1)))/8._dp
      return
      end
      
      
      function Beta_2a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-16*pi*G3(0,0,1))/9._dp+18*pi*G3(0,0,2)-16*pi*G3(0,0,3)+18*pi*G3(0,2,0)-
     & (92*pi*G3(0,2,1))/9._dp+18*pi*G3(0,2,2)-28*pi*G3(0,2,3)-8*pi*G3(0,3,0)-
     & 8*pi*G3(0,3,2)+(128*pi*G3(0,3,3))/9._dp+18*pi*G3(2,0,0)+18*pi*G3(2,0,2)+
     & 18*pi*G3(2,2,0)+16*pi*G3(2,3,2)-16*pi*G3(3,0,0)+10*pi*G3(3,0,1)-
     & 16*pi*G3(3,0,2)+(208*pi*G3(3,0,3))/9._dp-16*pi*G3(3,2,0)+10*pi*G3(3,2,1)+
     & (80*pi*G3(3,2,3))/9._dp+(128*pi*G3(3,3,0))/9._dp-(128*pi*G3(3,3,3))/9._dp+
     & G2(0,0)*(-3*(-21+nf)*pi-16*pi*H1(0)-18*pi*H1(1))+
     & (pi*(4*omv+u*(328+27*nf*(-1+u)+2*u*(-166+v)-75*v))*H2(1,1))/
     & (9._dp*omu*u)-(64*pi*H3(0,0,0))/3._dp-8*pi*H3(0,0,1)-(62*pi*H3(0,1,0))/9._dp-
     & 8*pi*H3(1,0,0)-10*pi*H3(1,0,1)+20*pi*H3(1,1,0)+
     & G2(2,2)*((pi*(4*omv+u*(328+27*nf*(-1+u)+2*u*(-166+v)-75*v)))/
     & (9._dp*omu*u)-(pi*(omv+u*(6+9*u+v)+
     & 30*NFV*(-1+u)*(-2+u+2*v))*H1(0))/(9._dp*(u*u)))-
     & (2*pi*(omv+u*(-9*u+v))*G3(0,1,0))/(u*u)-
     & (omu*pi*(omv+u)*G3(0,1,2))/(u*u)-
     & (10*pi*(omv+u*(-9*u+v))*G3(1,0,1))/(9._dp*(u*u))-
     & (pi*(omv+u*(2-33*u+v)+10*NFV*(-1+u)*(-2+u+2*v))*G3(1,0,3))/
     & (3._dp*(u*u))+(2*omu*pi*(2+5*u-2*v)*G3(1,1,0))/(9._dp*(u*u))-
     & (16*pi*(omv+u*(-9*u+v))*G3(1,1,2))/(9._dp*(u*u))-
     & (2*pi*(15*omv+u*(3-122*u+15*v))*G3(1,2,1))/(9._dp*(u*u))-
     & (pi*(omv+u*(2-33*u+v)+10*NFV*(-1+u)*(-2+u+2*v))*G3(1,2,3))/
     & (3._dp*(u*u))-(pi*(17*omv+30*omu*NFV*(-2+u+2*v)+
     & u*(-6+81*u+17*v))*G3(2,0,3))/(9._dp*(u*u))+
     & (pi*(omv+u*(-9*u+v))*G3(2,1,0))/(u*u)-
     & (16*pi*(omv+u*(-9*u+v))*G3(2,1,1))/(9._dp*(u*u))+
     & (pi*(omv+u*(-9*u+v))*G3(2,1,2))/(u*u)+
     & (pi*(17*omv+30*omu*NFV*(-2+u+2*v)+u*(-6-27*u+17*v))*
     & G3(2,2,3))/(9._dp*(u*u))+(20*pi*(omv+u*(-9*u+v))*G3(2,3,3))/
     & (9._dp*(u*u))+(pi*G3(0,1,1)*(9-9*v+9*u*v-169*(u*u)))/(9._dp*(u*u))+
     & (2*pi*G3(2,2,1)*(11+3*u-11*v+11*u*v-86*(u*u)))/(9._dp*(u*u))+
     & (2*pi*G3(1,0,2)*(2-2*v+u*(3+2*v)-77*(u*u)))/(9._dp*(u*u))+
     & (2*pi*G3(1,2,0)*(2-2*v+u*(3+2*v)-77*(u*u)))/(9._dp*(u*u))+
     & (2*pi*G3(1,2,2)*(2-2*v+u*(3+2*v)-77*(u*u)))/(9._dp*(u*u))-
     & (2*pi*G3(1,1,1)*(-6+3*u+6*v-6*u*v+67*(u*u)))/(9._dp*(u*u))-
     & (2*pi*G3(2,0,1)*(2-2*v+u*(3+2*v)+121*(u*u)))/(9._dp*(u*u))+
     & G2(2,0)*((pi*(4*omv+u*(436+27*nf*(-1+u)+2*u*(-220+v)-75*v)))/
     & (9._dp*omu*u)-18*pi*H1(1)-
     & (pi*H1(0)*(5-5*v+u*(-6+5*v)+82*(u*u)))/(9._dp*(u*u)))+
     & G2(0,2)*((pi*(4*omv+u*(436+27*nf*(-1+u)+2*u*(-220+v)-75*v)))/
     & (9._dp*omu*u)-18*pi*H1(1)-
     & (pi*H1(0)*(9-9*v+9*u*v+232*(u*u)))/(9._dp*(u*u)))+
     & G2(0,1)*((omu*pi*(omv+u)*H1(1))/(u*u)+
     & (pi*H1(0)*(9-9*v+9*u*v-11*(u*u)))/(9._dp*(u*u))+
     & (pi*(-851-63/omv+40/omw+45*nf+12/u-82*v+(80*v)/omu+
     & (6*v)/u-(nf*v)/u-10/(omw*omw)-(omv*(16+nf))/(u*u)+
     & (9*(281-36*nf)*v*(u*u))/(omv*omv*omv)+
     & (9*(-281+36*nf)*v*(u*u*u))/(omv*omv*omv*omv)+
     & (u*v*(-1031+108*nf-160*v+80*(v*v)))/(omv*omv)))/18._dp)+
     & G2(1,0)*((pi*H1(0)*(5-5*v+u*(-6+5*v)+73*(u*u)))/(9._dp*(u*u))+
     & (2*pi*H1(1)*(-2-3*u+2*v-2*u*v+77*(u*u)))/(9._dp*(u*u))-
     & (pi*(-63+106*v+(nf-nf*u+20*NFV*(omu*omu))*(omv*omv)+
     & 71*v*(u*u*u*u)-42*(v*v)+
     & u*u*u*(-281+9*nf+2*v*(-73+10*NFV*(omu*omu))+71*(v*v))+
     & u*u*(289-201*v+nf*(-9+8*v)-75*(v*v)+
     & 20*NFV*(omu*omu)*(1+v*v))+
     & u*(55-70*v+32*(v*v)+20*NFV*(omu*omu)*(-2+v+v*v))))/
     & (18._dp*w*(u*u)))+G2(1,2)*((pi*H1(0)*
     & (5-5*v+10*omu*NFV*(-2+u+2*v)+u*(-2+5*v)+27*(u*u)))/
     & (3._dp*(u*u))+(2*pi*H1(1)*(-2-3*u+2*v-2*u*v+77*(u*u)))/
     & (9._dp*(u*u))-(pi*(3-26*v+
     & omu*nf*(-((-1+u)*(omv*omv))+(-9+9*u+8*v)*(u*u))-
     & 71*v*(u*u*u*u*u)+u*u*u*u*(53+213*v-71*(v*v))+24*(v*v)-
     & 4*u*(11-19*v+4*(v*v))+u*u*(132-179*v+49*(v*v))+
     & u*u*u*(-144-13*v+142*(v*v))+
     & 20*NFV*(omu*omu*omu)*
     & (omv*omv+u*(-2+u+v+v*(u*u)+v*v+u*(v*v)))))/
     & (18._dp*omu*w*(u*u)))+G2(1,1)*
     & ((16*pi*(omv+u*(-9*u+v))*H1(1))/(9._dp*(u*u))-
     & (pi*(-14+8*v-4*omu*nf*
     & ((-1+u)*(omv*omv)+(9-9*u-8*v)*(u*u))+
     & 151*v*(u*u*u*u*u)+u*u*u*(-1168+919*v-308*(v*v))+
     & u*(16+38*v-38*(v*v))+7*(v*v)+
     & u*u*(588-657*v+60*(v*v))+u*u*u*u*(578-459*v+151*(v*v))-
     & 10*NFV*(omu*omu)*(2*(omv*omv)-omv*u*(3+2*(u*u))-
     & u*u*(-3+2*v*(1+u*u)+2*u*(v*v)))))/(18._dp*omu*w*(u*u)))+
     & (pi*((36*(20+(-50+9*nf)*u))/omw-(720*u)/(omw*omw)+
     & (-24*omu*(-567+50*nf)*u*v*w*(omv*omv*omv*omv)+
     & 18*u*v*w*zeta2*(omv*omv*omv*omv)*
     & (453-550*u-60*NFV*(-2+5*u)+161*(u*u))+
     & omu*omu*(6*w*(38*nf+15*NFV*(10-u+2*v-2*u*v+48*zeta2))*
     & (omv*omv*omv*omv*omv)+
     & 324*omv*(-281+36*nf)*v*w*zeta2*(u*u*u)+
     & omv*omv*omv*omv*
     & (w*(3645*(-1+v)+
     & u*(7041-664*nf-3321*v+324*nf*v+4716*zeta3)+
     & 27*(-281+36*nf)*(u*u))-
     & 18*zeta2*((-19+3*v)*w+u*(27+80*(-1+3*NFV)*w)+
     & (-27+151*v*w)*(u*u)+20*NFV*v*w*(u*u*u)))+
     & 324*(281-36*nf)*v*w*zeta2*(u*u*u*u)-
     & 3*w*(omv*omv*omv)*(u*u)*
     & (-2031+324*nf+13560*zeta2+60*NFV*(-9-4*v+v*v))+
     & 72*w*zeta2*(omv*omv)*(u*u)*
     & (565-54*nf*v+5*NFV*(-1-3*(v*v)+v*v*v))))/
     & (u*w*(omu*omu)*(omv*omv*omv*omv))))/324._dp+
     & (pi*H2(0,1)*((1112*u)/omv+(3*(9-9*u+296*w))/w+
     & (2529*v*(u*u))/(omv*omv*omv)+
     & v*(-75-82/omu+10/u+71*u+20*NFV*(u*u))-
     & (2*u*(556+5*NFV*(-2+3*v-6*(v*v)+2*(v*v*v))))/(omv*omv)+
     & (-2529*v*(u*u*u)+2*nf*
     & (-31+2*v*(62+27*u-81*(u*u)+81*(u*u*u))+
     & 6*(-31-18*u+27*(u*u))*(v*v)+2*(62+27*u)*(v*v*v)-
     & 31*(v*v*v*v)))/(omv*omv*omv*omv)))/18._dp+
     & G2(0,3)*((136*pi*H1(0))/9._dp+8*pi*H1(1)+
     & (pi*(-146-63/omv+75*v+(71*v)/omu-(150*NFV*v)/omu+
     & (2*(-20+(3+60*NFV)*v))/u-27/w-(32*v)/(omu*omu)+
     & (90*NFV*v)/(omu*omu)+
     & u*(-1111/omv-20*omv*NFV-71*v+27/w+1111/(omv*omv))-
     & (v*(2529+20*NFV*(omv*omv*omv))*(u*u))/(omv*omv*omv)+
     & (2529*v*(u*u*u))/(omv*omv*omv*omv)+
     & (4*nf*(2-v*(8+27*u-81*(u*u)+81*(u*u*u))+
     & (12+54*u-81*(u*u))*(v*v)-(8+27*u)*(v*v*v)+
     & 2*(v*v*v*v)))/(omv*omv*omv*omv)))/18._dp)+
     & (pi*H2(0,0)*(20*omv*NFV*u+(63+1111*u)/omv+(3*(9-9*u+314*w))/w-
     & (u*(1111+2529*u))/(omv*omv)+
     & (2529*(1+u)*(u*u))/(omv*omv*omv)+
     & (v*(-73*omu*u+2*(16+NFV*(30-75*u))*u+
     & omu*omu*(34-75*u+71*(u*u)+20*NFV*(-6+u*u*u))))/
     & (u*(omu*omu))+(-2529*(u*u*u)-
     & 4*nf*(16-v*(64+27*u-81*(u*u)+81*(u*u*u))+
     & (96+54*u-81*(u*u))*(v*v)-(64+27*u)*(v*v*v)+
     & 16*(v*v*v*v)))/(omv*omv*omv*omv)))/18._dp+
     & G2(3,0)*((208*pi*H1(0))/9._dp+16*pi*H1(1)+
     & (pi*(20*NFV*v*(u*u*u*u*u)+
     & u*u*u*u*(9*v+20*NFV*(1-2*v+4*(v*v)))-
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+20*NFV*(v*v*v))+
     & u*u*u*(216-8*nf+2*v+27*(v*v)+
     & 10*NFV*(-2+7*v-14*(v*v)+12*(v*v*v)))+
     & u*v*(-20+(10-30*NFV)*v+2*(-8+55*NFV)*(v*v)+
     & (9-100*NFV)*(v*v*v)+20*NFV*(v*v*v*v))+
     & u*u*(-216+8*omv*nf-6*(-38+5*NFV)*v+
     & 5*(-1+24*NFV)*(v*v)-9*(-3+20*NFV)*(v*v*v)+
     & 80*NFV*(v*v*v*v))))/(18._dp*w*(u*u)))+
     & G2(3,2)*((62*pi*H1(0))/9._dp+(pi*
     & (20*NFV*v*(u*u*u*u*u)+u*u*u*u*(9*v+20*NFV*(1-2*v+4*(v*v)))-
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+20*NFV*(v*v*v))+
     & u*u*u*(24-8*nf+2*v+27*(v*v)+
     & 10*NFV*(-2+7*v-14*(v*v)+12*(v*v*v)))+
     & u*v*(-20+(10-30*NFV)*v+2*(-8+55*NFV)*(v*v)+
     & (9-100*NFV)*(v*v*v)+20*NFV*(v*v*v*v))+
     & u*u*(-24+8*omv*nf-6*(-6+5*NFV)*v+5*(-1+24*NFV)*(v*v)-
     & 9*(-3+20*NFV)*(v*v*v)+80*NFV*(v*v*v*v))))/(18._dp*w*(u*u)))+
     & (pi*H1(1)*(u*(omv*omv*omv)*
     & (2701+829*v-12*nf*(19+3*v)+2250*zeta2-36*(v*v))+
     & 3*(-281+36*nf)*(u*u*u*u*u)*(-2-5*v+v*v)+
     & 2*(omv*omv*omv)*(59-69*v+9*(v*v))-
     & omv*omv*(u*u)*(3823-885*v+2250*omv*zeta2-732*(v*v)+
     & 12*nf*(-25+v+6*(v*v))+18*(v*v*v))-
     & omv*(u*u*u)*(-2321-5556*v+3206*(v*v)-387*(v*v*v)+
     & 36*nf*(7+20*v-10*(v*v)+v*v*v))+
     & 3*(u*u*u*u)*(-1001-2617*v+2336*(v*v)-404*(v*v*v)+
     & 12*nf*(11+27*v-24*(v*v)+4*(v*v*v)))+
     & 10*omu*NFV*(omv*omv)*
     & (4*(1+v)*(omv*omv*omv)-6*omv*(u*u*u*u)+
     & u*(3*(u*u)*(3-7*v+4*(v*v))+
     & u*(3+14*v-21*(v*v)+10*(v*v*v))+
     & 2*(-5+4*v+6*(v*v)-7*(v*v*v)+2*(v*v*v*v))))))/
     & (36._dp*omu*u*(omv*omv*omv))-
     & (pi*H2(1,0)*(81*omu*u*v*(omv*omv*omv*omv)*(omw*omw)+
     & 2*u*(-16+15*NFV*(-2+5*u))*v*(omv*omv*omv*omv)*(omw*omw)+
     & omu*omu*(2*(omv*omv*omv*omv)*
     & (5*u-20*omw*u+(3-735*u-16*v+2*u*v+
     & 60*NFV*(-1+u+v))*(omw*omw))+
     & 2*(-574+54*nf+15*NFV)*v*(omv*omv)*(omw*omw)*(u*u)-
     & 9*omv*(-281+36*nf)*v*(omw*omw)*(u*u*u)+
     & u*(omw*omw)*(-2529*v*(u*u*u)+
     & nf*(98+4*v*(-98+81*(u*u*u))+588*(v*v)-392*(v*v*v)+
     & 98*(v*v*v*v))))))/
     & (18._dp*u*(omu*omu)*(omv*omv*omv*omv)*(omw*omw))+
     & G2(2,3)*(-18*pi*H1(0)-16*pi*H1(1)+
     & (pi*(71*omu*u*v*(omv*omv*omv*omv)+
     & 2*u*(-16+15*NFV*(-2+5*u))*v*(omv*omv*omv*omv)+
     & omu*omu*(u*(63+1111*u)*(omv*omv*omv)+
     & (28-685*u+6*v+4*u*v+120*NFV*(-2+2*u+v))*
     & (omv*omv*omv*omv)-(1111+2529*u)*(omv*omv)*(u*u)+
     & 2529*omv*(1+u)*(u*u*u)+
     & u*(-2529*(u*u*u)+
     & 4*nf*(11+v*(-44+27*u-81*(u*u)+81*(u*u*u))+
     & (66-54*u+81*(u*u))*(v*v)+(-44+27*u)*(v*v*v)+
     & 11*(v*v*v*v))))))/
     & (18._dp*u*(omu*omu)*(omv*omv*omv*omv)))+
     & G2(2,1)*(-(pi*(omv+u*(-9*u+v))*H1(0))/(9._dp*(u*u))-
     & (pi*(omv+u*(-9*u+v))*H1(1))/(u*u)-
     & (pi*(-200*v*(omv*omv*omv*omv)*(omw*omw)*(u*u)+
     & omu*((-63+1111*u)*(omv*omv*omv)*(omw*omw)*(u*u)+
     & omv*omv*omv*omv*omv*(omw*omw)*
     & (nf+10*NFV*(2+3*(u*u)))+
     & (-1111-2529*u+108*nf*v)*(omv*omv)*(omw*omw)*(u*u*u)+
     & 9*omv*(281+281*u-36*nf*v)*(omw*omw)*(u*u*u*u)+
     & 9*(-281+36*nf*v)*(omw*omw)*(u*u*u*u*u)-
     & omv*omv*omv*omv*
     & (-10*(u*u)+40*omw*(u*u)+
     & omw*omw*(20*NFV*v*(u*u*u*u)+
     & u*u*u*(9*v+20*NFV*(1-v+3*(v*v)))-
     & (-1+v)*(38+20*v+9*(v*v)+20*NFV*(v*v*v))+
     & u*u*(-95+9*nf+15*v+(18-60*NFV)*(v*v)+
     & 60*NFV*(v*v*v))+
     & u*(-60-(-100+nf)*v+2*(v*v)+9*(v*v*v)+
     & 10*NFV*(3-2*v+3*(v*v)-6*(v*v*v)+2*(v*v*v*v)))))
     & )))/(18._dp*omu*(omv*omv*omv*omv)*(omw*omw)*(u*u)))+
     & G2(3,3)*((-128*pi*H1(0))/9._dp-(pi*
     & (20*NFV*v*(u*u*u*u*u)+u*u*u*u*(9*v+20*NFV*(1-2*v+4*(v*v)))+
     & u*u*u*(-532+32*nf+2*v+27*(v*v)+
     & 10*NFV*(-2+7*v-14*(v*v)+12*(v*v*v)))-
     & v*(40-52*v+(2+20*NFV)*(v*v)+(9-40*NFV)*(v*v*v)+
     & 20*NFV*(v*v*v*v))+
     & u*u*(572-32*omv*nf-2*(281+15*NFV)*v+
     & 5*(-1+24*NFV)*(v*v)-9*(-3+20*NFV)*(v*v*v)+
     & 80*NFV*(v*v*v*v))+
     & u*(-40+96*v-2*(16+15*NFV)*(v*v)+2*(-8+55*NFV)*(v*v*v)+
     & (9-100*NFV)*(v*v*v*v)+20*NFV*(v*v*v*v*v))))/(18._dp*w*(u*u)))
     & +((pi*(omv+u*(-32+nf+31*u-nf*u-3*v)))/(2._dp*omu*u)+
     & (9*pi*G1(2))/2._dp-4*pi*G1(3)-(27*pi*H1(0))/2._dp-(9*pi*H1(1))/2._dp+
     & (pi*(omv+u*(-9*u+v))*G1(1))/(2._dp*(u*u)))*(G1(0)*G1(0))+
     & (9*pi*(G1(0)*G1(0)*G1(0)))/2._dp+
     & ((pi*(omv+u*(-32+nf+31*u-nf*u-3*v)))/(2._dp*omu*u)-4*pi*G1(3)-
     & (27*pi*H1(0))/2._dp-(27*pi*H1(1))/2._dp)*(G1(2)*G1(2))+
     & (9*pi*(G1(2)*G1(2)*G1(2)))/2._dp+
     & ((-35*pi*H1(1))/2._dp-(pi*(omv*omv*
     & (-17+19*v+u*(481-14*nf+57*v)+14*nf*(u*u))-
     & 2*(u*u)*(232+(-465+u)*v+232*(v*v))))/(18._dp*omu*u*(omv*omv))
     & )*(H1(0)*H1(0))-(17*pi*(H1(0)*H1(0)*H1(0)))/2._dp+
     & (pi*(omv+u*(-32+nf+31*u-nf*u-3*v))*(H1(1)*H1(1)))/(2._dp*omu*u)+
     & H1(0)*((pi*(20*omv+u*(-817-26*nf*(-1+u)+797*u-60*v))*H1(1))/
     & (18._dp*omu*u)-(8*pi*H2(0,0))/9._dp-(10*pi*H2(1,0))/9._dp+
     & (pi*(-(omu*(-3*omv*omw*w*(u*u)*(813+40*NFV+3372*u+843*(u*u))+
     & omw*u*w*(omv*omv)*
     & (1379+15*(311+8*NFV)*u+2529*(u*u)-843*(u*u*u))+
     & 5058*omw*(1+u)*w*(u*u*u)+
     & 40*omw*NFV*w*(omv*omv*omv*omv)*(v*v)+
     & omv*omv*omv*(-20*u*w+
     & omw*(-2+106*w+56*v*w+
     & 6*(202+5*NFV*(-3+4*v))*w*(u*u*u)+
     & 60*NFV*w*(u*u*u*u)-18*w*(v*v)+
     & u*u*(58+(-1220+387*v)*w+
     & 10*NFV*(-15+w*(3-15*v+10*(v*v))))+
     & u*(-22-w*(4363+469*v+810*zeta2-18*(v*v))+
     & 20*NFV*(3+w*(18+2*v-5*(v*v)+2*(v*v*v))))))))
     & +4*omw*w*(u*(-16+45*NFV+111*v)*(omv*omv*omv)+
     & 2*nf*(omv*omv*omv*omv)+
     & nf*u*(omv*omv)*(-58+70*v+15*(v*v))+
     & nf*(u*u)*(74-258*v+195*(v*v)+
     & 27*(u*u*u)*(-2-5*v+v*v)+7*(v*v*v)+
     & 9*(u*u)*(11+9*v-24*(v*v)+4*(v*v*v))-18*(v*v*v*v)+
     & 9*u*(-7+14*v+21*(v*v)-11*(v*v*v)+v*v*v*v)))))/
     & (36._dp*omu*omw*u*w*(omv*omv*omv))-(27*pi*(H1(1)*H1(1)))/2._dp)+
     & G1(3)*((-64*pi*H2(0,0))/3._dp-8*pi*H2(0,1)-(62*pi*H2(1,0))/9._dp-
     & (pi*H1(1)*(20*NFV*v*(u*u*u*u*u)+
     & u*u*u*u*(9*v+20*NFV*(1-2*v+4*(v*v)))-
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+20*NFV*(v*v*v))+
     & u*u*u*(-216+2*v+27*(v*v)+
     & 10*NFV*(-2+7*v-14*(v*v)+12*(v*v*v)))+
     & u*v*(-20+(10-30*NFV)*v+2*(-8+55*NFV)*(v*v)+
     & (9-100*NFV)*(v*v*v)+20*NFV*(v*v*v*v))+
     & u*u*(216-6*(34+5*NFV)*v+5*(-1+24*NFV)*(v*v)-
     & 9*(-3+20*NFV)*(v*v*v)+80*NFV*(v*v*v*v))))/(18._dp*w*(u*u))+
     & H1(0)*(-8*pi*H1(1)-(pi*(20*NFV*v*(u*u*u*u*u)+
     & u*u*u*u*(9*v+20*NFV*(1-2*v+4*(v*v)))-
     & v*v*(-10+(2+20*NFV)*v+(9-40*NFV)*(v*v)+
     & 20*NFV*(v*v*v))+
     & u*u*u*(-216+2*v+27*(v*v)+
     & 10*NFV*(-2+7*v-14*(v*v)+12*(v*v*v)))+
     & u*v*(-20+(10-30*NFV)*v+2*(-8+55*NFV)*(v*v)+
     & (9-100*NFV)*(v*v*v)+20*NFV*(v*v*v*v))+
     & u*u*(216-6*(34+5*NFV)*v+5*(-1+24*NFV)*(v*v)-
     & 9*(-3+20*NFV)*(v*v*v)+80*NFV*(v*v*v*v))))/(18._dp*w*(u*u)))
     & +(pi*(2160*u*zeta2*(omv*omv*omv*omv)-
     & 3*(u*u*u*u)*(5322-4558*v-16036*(v*v)-
     & 843*(u*u)*(-2-5*v+v*v)+11813*(v*v*v)-1599*(v*v*v*v)+
     & 36*nf*(-18+16*v+54*(v*v)+3*(u*u)*(-2-5*v+v*v)-
     & 39*(v*v*v)+5*(v*v*v*v)))+
     & omv*(u*u*u)*(4*nf*(-395+898*v+550*(v*v)+
     & 27*(u*u)*(-17-35*v+7*(v*v))-351*(v*v*v)+27*(v*v*v*v))
     & -3*(-6103+12746*v+5024*(v*v)+
     & 3*(u*u)*(-1563-3337*v+685*(v*v))-4485*(v*v*v)+
     & 405*(v*v*v*v)))-
     & omv*omv*omv*(90*NFV*(-7+6*v)*(u*u*u*u*u)+
     & 180*NFV*(u*u*u*u*u*u)+
     & 30*NFV*(u*u*u*u)*(27-54*v+22*(v*v))+
     & 6*v*(-6-4*v+9*(v*v)+20*NFV*(6-5*v-2*(v*v)+v*v*v))+
     & 30*(u*u*u)*(-72*zeta2+
     & NFV*(36+58*v-57*(v*v)+14*(v*v*v)))+
     & u*(-4*nf*(-152+206*v+27*(v*v))-
     & 3*(2684-3656*v-449*(v*v)+36*(v*v*v)+
     & 20*NFV*(-12+40*v-10*(v*v)-13*(v*v*v)+4*(v*v*v*v)))
     & ))+2*(omv*omv)*(u*u)*
     & (4*nf*(179-363*v-5*(v*v)+27*(v*v*v))-
     & 3*(3234-6004*v+361*(v*v)+360*zeta2*(2-3*v+v*v)+
     & 464*(v*v*v)-9*(v*v*v*v)-
     & 5*NFV*(72-124*v-v*v+85*(v*v*v)-36*(v*v*v*v)+
     & 4*(v*v*v*v*v))))))/(108._dp*omu*u*w*(omv*omv*omv))-
     & (76*pi*(H1(0)*H1(0)))/9._dp-4*pi*(H1(1)*H1(1)))+
     & G1(0)*(G1(3)*((-4*(-30+nf)*pi)/9._dp+8*pi*H1(0))+
     & G1(2)*((-30+nf)*pi-18*pi*H1(0)-9*pi*H1(1))+(200*pi*H2(0,0))/9._dp+
     & 16*pi*H2(0,1)+26*pi*H2(1,0)+18*pi*H2(1,1)+
     & G1(1)*(((-30+nf)*pi*(omv+u*(-9*u+v)))/(18._dp*(u*u))-
     & (pi*(omv+u*(-9*u+v))*H1(0))/(u*u))-
     & (pi*H1(1)*(4+u*(166-18*nf-75*v)-4*v+2*(-85+9*nf+v)*(u*u)))/
     & (9._dp*omu*u)+H1(0)*(19*pi*H1(1)+
     & (pi*(-309+75*v+(154*v)/omu+(2*(-9+4*v))/u-27/w-
     & (v*(2529+20*NFV*(omv*omv*omv))*(u*u))/(omv*omv*omv)+
     & (2529*v*(u*u*u))/(omv*omv*omv*omv)+
     & u*(-1112/omv-71*v+27/w+
     & (1112+10*NFV*(-2+3*v-6*(v*v)+2*(v*v*v)))/(omv*omv))
     & +(36*nf*(1-v*(4+3*u-9*(u*u)+9*(u*u*u))+
     & (6+6*u-9*(u*u))*(v*v)-(4+3*u)*(v*v*v)+v*v*v*v))/
     & (omv*omv*omv*omv)))/18._dp)+
     & (pi*(-3*(-281+36*nf)*(u*u*u*u*u)*(-2-5*v+v*v)-
     & 2*(omv*omv*omv)*(3-13*v+9*(v*v))+
     & u*(omv*omv*omv)*(-2909-413*v+12*nf*(19+3*v)-810*zeta2+
     & 36*(v*v))+omv*omv*(u*u)*
     & (3919-981*v+810*omv*zeta2-732*(v*v)+
     & 12*nf*(-25+v+6*(v*v))+18*(v*v*v))+
     & omv*(u*u*u)*(-2321-5556*v+3206*(v*v)-387*(v*v*v)+
     & 36*nf*(7+20*v-10*(v*v)+v*v*v))-
     & 3*(u*u*u*u)*(-1001-2617*v+2336*(v*v)-404*(v*v*v)+
     & 12*nf*(11+27*v-24*(v*v)+4*(v*v*v)))-
     & 10*omu*NFV*(omv*omv)*
     & (4*(1+v)*(omv*omv*omv)-6*omv*(u*u*u*u)+
     & u*(3*(u*u)*(3-7*v+4*(v*v))+
     & u*(3+14*v-21*(v*v)+10*(v*v*v))+
     & 2*(-5+4*v+6*(v*v)-7*(v*v*v)+2*(v*v*v*v))))))/
     & (36._dp*omu*u*(omv*omv*omv))+(9*pi*(G1(2)*G1(2)))/2._dp+
     & (331*pi*(H1(0)*H1(0)))/18._dp+(9*pi*(H1(1)*H1(1)))/2._dp)+
     & G1(2)*(-(pi*(13*omv+u*(40+18*nf*(-1+u)-53*u-102*v+2*u*v))*
     & H1(1))/(9._dp*omu*u)+G1(3)*
     & ((-4*(-30+nf)*pi)/9._dp+8*pi*H1(0)+8*pi*H1(1))+
     & (pi*(17*omv+30*omu*NFV*(-2+u+2*v)+u*(-6-99*u+17*v))*
     & H2(0,0))/(9._dp*(u*u))+(pi*(13*omv+u*(6-u+13*v))*H2(1,0))/
     & (9._dp*(u*u))-(pi*H2(0,1)*(-5+6*u+5*v-5*u*v+71*(u*u)))/
     & (9._dp*(u*u))+(pi*(6*omu*(16*omv+u*(24-445*u+16*v))*zeta2*
     & (omv*omv*omv)+3*u*
     & (-3*(-281+36*nf)*(u*u*u*u*u)*(-2-5*v+v*v)-
     & 2*(omv*omv*omv)*(59-69*v+9*(v*v))+
     & u*(omv*omv*omv)*
     & (-2701-829*v+12*nf*(19+3*v)+36*(v*v))+
     & omv*omv*(u*u)*(3823-885*v-732*(v*v)+
     & 12*nf*(-25+v+6*(v*v))+18*(v*v*v))+
     & omv*(u*u*u)*(-2321-5556*v+3206*(v*v)-387*(v*v*v)+
     & 36*nf*(7+20*v-10*(v*v)+v*v*v))-
     & 3*(u*u*u*u)*(-1001-2617*v+2336*(v*v)-404*(v*v*v)+
     & 12*nf*(11+27*v-24*(v*v)+4*(v*v*v)))-
     & 10*omu*NFV*(omv*omv)*
     & (4*(1+v)*(omv*omv*omv)-6*omv*(u*u*u*u)+
     & u*(3*(u*u)*(3-7*v+4*(v*v))+
     & u*(3+14*v-21*(v*v)+10*(v*v*v))+
     & 2*(-5+4*v+6*(v*v)-7*(v*v*v)+2*(v*v*v*v)))))))/
     & (108._dp*omu*(omv*omv*omv)*(u*u))+
     & H1(0)*(27*pi*H1(1)+(pi*(-40/omw+10/(omw*omw)+
     & (161*omu*v*(omv*omv*omv*omv)*(u*u)+
     & 2*(-16+15*NFV*(-2+5*u))*v*(omv*omv*omv*omv)*(u*u)+
     & omu*omu*(u*(106-737*u-53*v+4*u*v+
     & 30*NFV*(-7+7*u+4*v))*(omv*omv*omv*omv)-
     & 41*(omv*omv*omv*omv*omv)+
     & 6*(18*nf-5*(37+NFV))*v*(omv*omv)*(u*u*u)-
     & 9*omv*(-281+36*nf)*v*(u*u*u*u)+
     & 9*(u*u)*(-281*v*(u*u*u)+
     & 4*nf*(2-8*v+9*v*(u*u*u)+12*(v*v)-8*(v*v*v)+
     & 2*(v*v*v*v)))))/
     & (omu*omu*(omv*omv*omv*omv)*(u*u))))/18._dp)+
     & (pi*(-2+2*v-2*u*v+333*(u*u))*(H1(0)*H1(0)))/(18._dp*(u*u))+
     & (27*pi*(H1(1)*H1(1)))/2._dp)+
     & G1(1)*(G1(2)*(((-30+nf)*pi*(omv+u*(-9*u+v)))/(18._dp*(u*u))-
     & (pi*(omv+u*(-9*u+v))*H1(0))/(u*u)-
     & (pi*(omv+u*(-9*u+v))*H1(1))/(u*u))-
     & (pi*(17*omv+30*omu*NFV*(-2+u+2*v)+u*(-6+63*u+17*v))*
     & H2(0,0))/(9._dp*(u*u))+(2*pi*H2(1,1)*
     & (2-2*v+u*(3+2*v)-77*(u*u)))/(9._dp*(u*u))-
     & (2*pi*H2(1,0)*(8-8*v+15*omu*NFV*(-2+u+2*v)+u*(-3+8*v)+
     & 36*(u*u)))/(9._dp*(u*u))-
     & (pi*H2(0,1)*(5-5*v+u*(-6+5*v)+73*(u*u)))/(9._dp*(u*u))+
     & H1(0)*((10*pi*(omv+u*(-9*u+v))*H1(1))/(9._dp*(u*u))-
     & (pi*(11+30*omv*NFV*u+18*v+4*nf*(-1+u)*(omv*omv)+
     & (-457+30*NFV*(-2+v)+430*v-4*nf*(-9+8*v))*(u*u)+
     & (399-36*nf+30*NFV)*(u*u*u)-31*(v*v)+
     & u*(47-112*v+31*(v*v))))/(18._dp*w*(u*u)))+
     & (pi*H1(1)*(33-86*v-71*v*(u*u*u*u*u)+
     & u*u*u*u*(-217+213*v-71*(v*v))+54*(v*v)-
     & 4*u*(26-49*v+19*(v*v))+u*u*(-108+v+79*(v*v))+
     & u*u*u*(396-253*v+142*(v*v))+
     & 20*NFV*(omu*omu*omu)*
     & (omv*omv+u*(-2+u+v+v*(u*u)+v*v+u*(v*v)))))/
     & (18._dp*omu*w*(u*u))-(pi*
     & (720*omw*NFV*(-2+u+2*v)*zeta2*(omu*omu)*(omv*omv*omv)+
     & omu*(-7587*omv*omw*(-1+u*u)*(u*u*u)-
     & 3*omw*(omv*omv)*(u*u)*
     & (1379+307*u-2529*(u*u)+843*(u*u*u))+
     & 15174*omw*(-1+u)*(u*u*u*u)+
     & omv*omv*omv*(60*(-1+u)*u+
     & omw*(6*zeta2*(-3*u*(-8+v)+3*(-1+v)-341*(u*u))+
     & 3*(391-411*v+(-1220+387*v)*(u*u*u)+
     & 1212*(u*u*u*u)+u*(-208+357*v-36*(v*v))+
     & 18*(v*v)+u*u*(-1567-185*v+18*(v*v)))+
     & 30*NFV*(-6+4*v+3*(-3+4*v)*(u*u*u*u)+
     & 6*(u*u*u*u*u)-2*(v*v)+
     & u*u*u*(3-15*v+10*(v*v))+
     & u*(3+v+10*(v*v)-8*(v*v*v))+4*(v*v*v)+
     & u*u*(3+4*v-18*(v*v)+4*(v*v*v))))))-
     & 4*omw*(u*(-60*u*v+nf*(-25+44*v))*(omv*omv*omv)+
     & 19*nf*(omv*omv*omv*omv)+
     & nf*(omv*omv)*(u*u)*(-168+59*v+28*(v*v))+
     & 3*nf*(u*u*u)*(76-48*v-69*(v*v)+
     & 27*(u*u*u)*(-2-5*v+v*v)+59*(v*v*v)+
     & 9*(u*u)*(11+45*v-24*(v*v)+4*(v*v*v))-18*(v*v*v*v)+
     & 9*u*(-7-40*v+39*(v*v)-11*(v*v*v)+v*v*v*v)))))/
     & (108._dp*omu*omw*(omv*omv*omv)*(u*u))+
     & (pi*(omv+u*(-9*u+v))*(G1(2)*G1(2)))/(2._dp*(u*u))+
     & (19*pi*(omv+u*(-9*u+v))*(H1(0)*H1(0)))/(18._dp*(u*u))+
     & (pi*(omv+u*(-9*u+v))*(H1(1)*H1(1)))/(2._dp*(u*u)))-
     & (9*pi*(H1(1)*H1(1)*H1(1)))/2._dp
      return
      end
      
      
      function Gamma_2a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0._dp
      return
      end
      
      
      function Gamma_2a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0._dp
      return
      end
      
      
      function Gamma_2a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-5+u/omv+(4*v)/omu)/3._dp+((1+3*u)*v*w*H1(1))/(3._dp*u*(omu*omu))+
     & G1(2)*(-((1+3*u)*v*w)/(3._dp*u*(omu*omu))-(v*w*H1(0))/(3._dp*(u*u)))+
     & G1(1)*((v*w*H1(0))/(3._dp*(u*u))+(v*w*H1(1))/(3._dp*(u*u)))-
     & (v*w*G2(1,2))/(3._dp*(u*u))-(v*H1(0)*
     & ((v+u*(-4+3*v))*(omv*omv)+u*u*(3+2*u-8*v-u*u+4*(v*v))))/
     & (3._dp*u*(omu*omu)*(omv*omv))
      return
      end
      
      
      function Gamma_2a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=-(pi*(1+3*u)*v*w)/(3._dp*u*(omu*omu))-(pi*v*w*G1(1))/(3._dp*(u*u))
      return
      end
      
      
      function Gamma_2a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=-((-23-257*u+nf*(3+9*u))*v*w*H2(1,1))/(9._dp*u*(omu*omu))+
     & G3(2,2,1)*((-22*v*w*H1(0))/(9._dp*(u*u))-(22*v*w*H1(1))/(9._dp*(u*u)))+
     & G3(1,1,1)*((-4*v*w*H1(0))/(3._dp*(u*u))-(4*v*w*H1(1))/(3._dp*(u*u)))+
     & G3(0,1,1)*(-((v*w*H1(0))/(u*u))-(v*w*H1(1))/(u*u))+
     & G3(2,1,0)*(-((v*w*H1(0))/(u*u))-(v*w*H1(1))/(u*u))+
     & G3(1,1,0)*((-4*v*w*H1(0))/(9._dp*(u*u))-(4*v*w*H1(1))/(9._dp*(u*u)))+
     & G3(2,0,1)*((4*v*w*H1(0))/(9._dp*(u*u))+(4*v*w*H1(1))/(9._dp*(u*u)))+
     & G3(1,0,1)*((10*v*w*H1(0))/(9._dp*(u*u))+(10*v*w*H1(1))/(9._dp*(u*u)))+
     & G3(2,1,1)*((16*v*w*H1(0))/(9._dp*(u*u))+(16*v*w*H1(1))/(9._dp*(u*u)))+
     & G3(0,1,0)*((2*v*w*H1(0))/(u*u)+(2*v*w*H1(1))/(u*u))+
     & G3(1,2,1)*((10*v*w*H1(0))/(3._dp*(u*u))+(10*v*w*H1(1))/(3._dp*(u*u)))-
     & (2*v*w*G4(0,1,0,2))/(u*u)+(v*w*G4(0,1,1,2))/(u*u)-
     & (2*v*w*G4(0,1,2,0))/(u*u)+(v*w*G4(0,1,2,2))/(u*u)-
     & (10*v*w*G4(1,0,1,2))/(9._dp*(u*u))+(4*v*w*G4(1,0,2,2))/(9._dp*(u*u))-
     & ((1+20*NFV)*v*w*G4(1,0,3,0))/(3._dp*(u*u))+(4*v*w*G4(1,1,0,2))/(9._dp*(u*u))+
     & (4*v*w*G4(1,1,1,2))/(3._dp*(u*u))+(4*v*w*G4(1,1,2,0))/(9._dp*(u*u))-
     & (20*v*w*G4(1,1,2,2))/(9._dp*(u*u))+(4*v*w*G4(1,2,0,2))/(9._dp*(u*u))-
     & (10*v*w*G4(1,2,1,2))/(3._dp*(u*u))+(4*v*w*G4(1,2,2,0))/(9._dp*(u*u))-
     & ((1+20*NFV)*v*w*G4(1,2,3,0))/(3._dp*(u*u))-(4*v*w*G4(2,0,1,2))/(9._dp*(u*u))+
     & ((-17+60*NFV)*v*w*G4(2,0,3,0))/(9._dp*(u*u))+(v*w*G4(2,1,0,2))/(u*u)-
     & (16*v*w*G4(2,1,1,2))/(9._dp*(u*u))+(v*w*G4(2,1,2,0))/(u*u)+
     & (22*v*w*G4(2,2,1,2))/(9._dp*(u*u))-
     & ((-17+60*NFV)*v*w*G4(2,2,3,0))/(9._dp*(u*u))+
     & (20*v*w*G4(2,3,3,0))/(9._dp*(u*u))-(2*v*w*G3(0,2,0)*H1(0))/(u*u)+
     & ((1+20*NFV)*v*w*G3(1,0,3)*H1(0))/(3._dp*(u*u))+
     & ((1+20*NFV)*v*w*G3(1,2,3)*H1(0))/(3._dp*(u*u))-
     & ((-17+60*NFV)*v*w*G3(2,0,3)*H1(0))/(9._dp*(u*u))+
     & ((-17+60*NFV)*v*w*G3(2,2,3)*H1(0))/(9._dp*(u*u))-
     & (20*v*w*G3(2,3,3)*H1(0))/(9._dp*(u*u))+
     & G3(2,0,2)*(-((v*w*H1(0))/(u*u))-
     & (v*w*(-4-75*u+2*(u*u)))/(9._dp*u*(omu*omu)))+
     & G3(0,2,2)*((v*w*H1(0))/(u*u)-
     & (v*w*(-4-75*u+2*(u*u)))/(9._dp*u*(omu*omu)))+
     & G3(2,2,0)*(-((-7+20*NFV)*v*w*H1(0))/(3._dp*(u*u))-
     & (v*w*(-4-75*u+2*(u*u)))/(9._dp*u*(omu*omu)))+
     & G3(1,2,2)*((-20*v*w*H1(0))/(9._dp*(u*u))-
     & (v*w*(-18+57*u+3*nf*(omu*omu)+27*(u*u)-2*(u*u*u)))/
     & (9._dp*(omu*omu)*(u*u)))+
     & G2(2,2)*((5*(-1+4*NFV)*v*w*H2(0,0))/(3._dp*(u*u))+
     & (22*v*w*H2(0,1))/(9._dp*(u*u))-(4*(-1+5*NFV)*v*w*H2(1,0))/(3._dp*(u*u))-
     & (v*w*(3*u*(-23-257*u+nf*(3+9*u))+
     & 30*(1+6*NFV)*zeta2*(omu*omu)))/(27._dp*(omu*omu)*(u*u))-
     & (v*w*H1(0)*(-18+57*u+3*nf*(omu*omu)+27*(u*u)-2*(u*u*u)))/
     & (9._dp*(omu*omu)*(u*u)))+
     & (H3(1,0,1)*(9+(-11+72/omu)*v+(4*(3-5*v)*v)/u-9/w-
     & (5*v*(843-108*nf+4*NFV*(omv*omv*omv))*(u*u))/
     & (omv*omv*omv)-(9*(-281+36*nf)*v*(u*u*u))/
     & (omv*omv*omv*omv)+(9*(-9+u*u)*(v*v))/(omu*omu)+
     & u*(-1712/omv+(9+20*NFV)*v+9/w+
     & (1712-216*nf*v)/(omv*omv)-20*NFV*(v*v))))/18._dp+
     & (H3(1,1,0)*((2*v*(5+60*omv*NFV+31*v))/u+(18*(-1+w))/w+
     & (v*(-195+189*u+30*NFV*(-2+5*u)-26*(u*u)))/(omu*omu)-
     & 40*NFV*v*(u*u)-((-257+331*u-124*(u*u)+
     & 30*NFV*(2-3*u+4*(u*u))+18*(u*u*u))*(v*v))/(omu*omu*omu)
     & +u*(2*(9+20*NFV)*v-40*NFV*(v*v)+
     & (18*(1+(-6+4*u)*v+5*(v*v)))/(w*(omv*omv)))))/18._dp+
     & (H1(1)*(-2*u*v*(8*nf*(31+2*v)+9*(-308+(6+5*NFV*(-5+2*u))*v))*w*
     & (omu*omu)*(omv*omv*omv*omv)+
     & omu*u*v*w*(omv*omv*omv*omv)*
     & (8*(-567+50*nf)*v+6*zeta2*
     & (524-561*u+30*NFV*(-2+5*u)+5*(u*u)))+
     & 6*u*w*zeta2*(omv*omv*omv*omv)*
     & (-439+382*u+98*(u*u)-30*NFV*(2-3*u+4*(u*u))-9*(u*u*u))*
     & (v*v)+omu*omu*omu*
     & (-60*NFV*v*w*(5+v-12*zeta2)*(omv*omv*omv*omv*omv)+
     & w*(9993+2529*u-12*nf*(109+27*u)+270*NFV*(3+v)+
     & 10704*zeta2)*(omv*omv*omv)*(u*u)+
     & 90*omv*(281-36*nf)*v*w*zeta2*(u*u*u)+
     & 54*(-281+36*nf)*v*w*zeta2*(u*u*u*u)+
     & w*(omv*omv)*(u*u)*(48*(-223+27*nf*v)*zeta2+
     & 3*u*(-2529+324*nf+20*NFV*(-9-4*v+v*v)))+
     & omv*omv*omv*omv*(w*
     & ((-1215+76*nf)*(-1+v)*v+
     & 3*u*(74+3*(-527+60*nf)*v-30*NFV*(3+2*v))+
     & 9*(-685+84*nf+20*NFV*v)*(u*u))+
     & 6*zeta2*(9*u*(-1+w)+5*v*(-1+17*v)*w-
     & 20*NFV*v*w*(u*u*u)+
     & u*u*(9+(9+20*NFV)*v*w-20*NFV*w*(v*v)))))))/
     & (108._dp*u*w*(omu*omu*omu)*(omv*omv*omv*omv))+
     & (G2(0,3)*H1(0)*(30-21/omv+75*v+(71*v)/omu+(2*(20-3*v)*v)/u-
     & 9/w-(32*v)/(omu*omu)+
     & u*(-1693/omv-71*v+9/w+(1693-216*nf*v)/(omv*omv))+
     & (15*(-281+36*nf)*v*(u*u))/(omv*omv*omv)-
     & (9*(-281+36*nf)*v*(u*u*u))/(omv*omv*omv*omv)-71*(v*v)-
     & (10*(v*v))/omu-(71*(v*v))/(omu*omu)+
     & (32*(v*v))/(omu*omu*omu)+
     & 10*NFV*v*((-2*(6*v+u*u*u))/u+
     & (3*(-2*(u*u*u)+u*(1-4*v-3*(v*v))+2*(v*v)+
     & u*u*(1+v+4*(v*v))))/(omv*(omu*omu*omu))+
     & (u*(5-6*v+6*(v*v)-2*(v*v*v)))/(omv*omv))))/18._dp-
     & (v*G3(3,0,2)*(omw*omw)*(-10+(2+20*NFV)*v+
     & (9+20*NFV*(-2+3*v))*(u*u)+20*NFV*(u*u*u)+(9-40*NFV)*(v*v)+
     & 2*u*(1+9*v+10*NFV*(1-4*v+3*(v*v)))+20*NFV*(v*v*v)))/
     & (18._dp*w*(u*u))-(v*G3(3,2,0)*(omw*omw)*
     & (-10+(2+20*NFV)*v+(9+20*NFV*(-2+3*v))*(u*u)+
     & 20*NFV*(u*u*u)+(9-40*NFV)*(v*v)+
     & 2*u*(1+9*v+10*NFV*(1-4*v+3*(v*v)))+20*NFV*(v*v*v)))/
     & (18._dp*w*(u*u))+(v*G2(3,2)*H1(0)*(omw*omw)*
     & (-10+(2+20*NFV)*v+(9+20*NFV*(-2+3*v))*(u*u)+
     & 20*NFV*(u*u*u)+(9-40*NFV)*(v*v)+
     & 2*u*(1+9*v+10*NFV*(1-4*v+3*(v*v)))+20*NFV*(v*v*v)))/
     & (18._dp*w*(u*u))+(v*H3(0,0,1)*
     & (-11+8/omu+9*u+(13-22*u-13*v)/(omv*w)+
     & v*(9-2/omu-4/u-8/(omu*omu))-
     & (20*NFV*u*w*(1+3*v-3*(v*v)+v*v*v))/(omv*omv*omv)))/18._dp+
     & (G3(0,3,0)*(-30+21/omv-75*v-(71*v)/omu+(2*v*(-20+3*v))/u+
     & 9/w+(32*v)/(omu*omu)+
     & u*(1693/omv+71*v-9/w+(-1693+216*nf*v)/(omv*omv))-
     & (15*(-281+36*nf)*v*(u*u))/(omv*omv*omv)+
     & (9*(-281+36*nf)*v*(u*u*u))/(omv*omv*omv*omv)+71*(v*v)+
     & (10*(v*v))/omu+(71*(v*v))/(omu*omu)-
     & (32*(v*v))/(omu*omu*omu)+
     & 10*NFV*v*((2*(6*v+u*u*u))/u+
     & (3*(2*(u*u*u)-2*(v*v)+u*(-1+4*v+3*(v*v))-
     & u*u*(1+v+4*(v*v))))/(omv*(omu*omu*omu))+
     & (u*(-5+6*v-6*(v*v)+2*(v*v*v)))/(omv*omv))))/18._dp+
     & G2(2,0)*((-5*(-1+4*NFV)*v*w*H2(0,0))/(3._dp*(u*u))+
     & (5*v*w*H2(0,1))/(9._dp*(u*u))+(v*w*H2(1,0))/(u*u)+
     & (v*w*H1(1)*(-4-75*u+2*(u*u)))/(9._dp*u*(omu*omu))+
     & (H1(0)*(21-21/omv-15*v-(153*v)/omu+(32*v)/(omu*omu)+
     & (v*(13+nf-36*v-27*(v*v)))/u-27*(v*v)-(10*(v*v))/omu+
     & (153*(v*v))/(omu*omu)-(32*(v*v))/(omu*omu*omu)-
     & (u*v*(10-18*v+9*(v*v)))/(omv*omv)+
     & (omv*v*(63-nf+20*v+9*(v*v)))/(u*u)+
     & 10*NFV*v*(u*(2-8*v+3/(omv*omv))-2*(u*u)+
     & 3*(2/omv+(2-5*u)/(omu*omu)+
     & (v*(4-9*u+10*(u*u)-2*(u*u*u)))/(omu*omu*omu)-
     & 4*(v*v))-(2*(omv*omv)*(1+v+v*v))/(u*u)+
     & (2*(-11+6*v+3*(v*v)-4*(v*v*v)))/u)))/18._dp+
     & ((48*omv*v*zeta2)/(u*u)-
     & (2*v*(-81+3*omv*nf+51*v+24*zeta2+
     & 60*NFV*(1+v)*(omv*omv)+27*(v*v)))/u-
     & (9*(u*u*u)*(20*NFV*(omv*omv*omv)-281*(-2-5*v+v*v)+
     & 36*nf*(-2-5*v+v*v)))/(omv*omv*omv)+
     & (9*(u*u)*(720+2775*v+30*NFV*(1-2*v)*(omv*omv)-685*(v*v)+
     & 12*nf*(-8-29*v+7*(v*v))))/(omv*omv)-
     & (3*u*(510+4477*v-1599*(v*v)+36*nf*(-2-15*v+5*(v*v))+
     & 10*omv*NFV*(3-14*v+22*(v*v))))/omv+
     & 3*(180+((-8*(-22+nf)+omu*(307-36*nf+20*NFV))*v)/omu+
     & ((-697+802*u-405*(u*u)+30*NFV*(4-7*u+3*(u*u))+
     & nf*(46-74*u+36*(u*u)))*(v*v))/(omu*omu)-
     & 140*NFV*(v*v*v)))/108._dp)+
     & (H3(1,0,0)*(omu*u*v*(32+63*v)*w*(omv*omv*omv*omv)+
     & u*v*(-63+8*v)*w*(omu*omu)*(omv*omv*omv*omv)-
     & 2*u*v*w*(omv*omv*omv)*
     & (16*omv*v+15*NFV*(2*(u*u*u)+u*(13-4*v-3*(v*v))+
     & 2*(-2+v*v)+u*u*(-11+v+4*(v*v))))+
     & omu*omu*omu*((1-1693*u)*u*w*(omv*omv*omv)+
     & 9*(281-36*nf)*v*w*(u*u*u*u)+
     & omv*omv*omv*omv*(2*v*(14+60*NFV*(-2+v)+v)*w+
     & 9*(1+v*w)*(u*u)+u*(-9+w*(8-15*v+9*(v*v))))+
     & 5*omv*v*w*(u*u*u)*(-843+108*nf+
     & 4*NFV*(1+3*v-3*(v*v)+v*v*v))+
     & w*(omv*omv)*(u*u)*(1693-216*nf*v-
     & 10*NFV*v*(-1+6*v-6*(v*v)+2*(v*v*v))))))/
     & (18._dp*u*w*(omu*omu*omu)*(omv*omv*omv*omv))+
     & (H3(0,1,0)*(omu*u*v*(-32+73*v)*w*(omv*omv*omv*omv)+
     & u*v*(-73+54*v)*w*(omu*omu)*(omv*omv*omv*omv)+
     & 2*u*v*w*(omv*omv*omv)*
     & (16*omv*v-15*NFV*(2*(u*u*u)-2*(v*v)+u*(-1+4*v+3*(v*v))-
     & u*u*(1+v+4*(v*v))))+
     & omu*omu*omu*(u*(1+1713*u)*w*(omv*omv*omv)+
     & 9*(-281+36*nf)*v*w*(u*u*u*u)+
     & omv*omv*omv*omv*((9+89*v*w)*(u*u)+
     & 2*(23-60*NFV)*w*(v*v)+u*(-9+w*(8-97*v+89*(v*v))))+
     & 5*omv*v*w*(u*u*u)*(843-108*nf+
     & 4*NFV*(1+3*v-3*(v*v)+v*v*v))+
     & w*(omv*omv)*(u*u)*(-1713+216*nf*v-
     & 10*NFV*v*(-1+6*v-6*(v*v)+2*(v*v*v))))))/
     & (18._dp*u*w*(omu*omu*omu)*(omv*omv*omv*omv))+
     & G2(0,2)*(-((v*w*H2(0,1))/(u*u))-(2*v*w*H2(1,0))/(u*u)+
     & (v*w*H1(1)*(-4-75*u+2*(u*u)))/(9._dp*u*(omu*omu))+
     & (180+((-8*(-22+nf)+omu*(307-36*nf+20*NFV))*v)/omu-
     & (36*omv*v*zeta2)/(u*u)+
     & ((-697+802*u-405*(u*u)+30*NFV*(4-7*u+3*(u*u))+
     & nf*(46-74*u+36*(u*u)))*(v*v))/(omu*omu)-
     & (2*v*(-27+nf+17*v-nf*v-18*zeta2+
     & 20*NFV*(1+v)*(omv*omv)+9*(v*v)))/u-
     & (3*(u*u*u)*(20*NFV*(omv*omv*omv)-281*(-2-5*v+v*v)+
     & 36*nf*(-2-5*v+v*v)))/(omv*omv*omv)+
     & (3*(u*u)*(720+2775*v+30*NFV*(1-2*v)*(omv*omv)-685*(v*v)+
     & 12*nf*(-8-29*v+7*(v*v))))/(omv*omv)-
     & (u*(510+4477*v-1599*(v*v)+36*nf*(-2-15*v+5*(v*v))+
     & 10*omv*NFV*(3-14*v+22*(v*v))))/omv-140*NFV*(v*v*v))/36._dp
     & +(H1(0)*((21+u-60*NFV*v)/omv-(omv*(16+nf)*v)/(u*u)+
     & (2*omu*u*v*(-81+7*v)*w+
     & omu*omu*(v*(-34+nf+30*NFV+12*v)*w-9*(1+v*w)*(u*u)+
     & 20*NFV*v*w*(u*u*u)+u*(9+w*(-30+7*v-9*(v*v))))+
     & 162*u*w*(v*v))/(u*w*(omu*omu))+
     & (u*(-1+10*NFV*v*(1+6*v-6*(v*v)+2*(v*v*v))))/(omv*omv)))/
     & 18._dp)+(omw*v*G3(3,3,0)*(40-52*v+(9+40*NFV*(-1+2*v))*(u*u*u)+
     & 20*NFV*(u*u*u*u)+(2+20*NFV)*(v*v)+
     & u*u*(2+27*v+20*NFV*(1-6*v+6*(v*v)))+(9-40*NFV)*(v*v*v)+
     & u*(-52+(4+40*NFV)*v-3*(-9+40*NFV)*(v*v)+80*NFV*(v*v*v))+
     & 20*NFV*(v*v*v*v)))/(18._dp*w*(u*u))-
     & (omw*v*G2(3,3)*H1(0)*(40-52*v+(9+40*NFV*(-1+2*v))*(u*u*u)+
     & 20*NFV*(u*u*u*u)+(2+20*NFV)*(v*v)+
     & u*u*(2+27*v+20*NFV*(1-6*v+6*(v*v)))+(9-40*NFV)*(v*v*v)+
     & u*(-52+(4+40*NFV)*v-3*(-9+40*NFV)*(v*v)+80*NFV*(v*v*v))+
     & 20*NFV*(v*v*v*v)))/(18._dp*w*(u*u))+
     & (v*G3(2,3,0)*(2*(14+60*NFV*(-2+v)+3*v)*(omv*omv*omv*omv)+
     & u*(omv*omv*omv)*(-148+158*v-31*(v*v))+
     & u*(omv*omv)*(u*(1962-216*nf-548*v+352*(v*v)-73*(v*v*v))-
     & 30*NFV*(-28+64*v-(1+2*v)*(u*u*u)+u*u*u*u+
     & u*u*(-16+32*v-13*(v*v))-46*(v*v)+
     & u*(36-77*v+49*(v*v)-9*(v*v*v))+10*(v*v*v)))+
     & omv*(u*u*u)*(-9468+5542*v-108*nf*(-11+6*v)-408*(v*v)+
     & 123*(v*v*v)-4*(v*v*v*v))+
     & u*u*u*u*(20278-22882*v-843*(-14+5*v)*(u*u)-2529*(u*u*u)+
     & u*(-21925+16031*v-1693*(v*v))+5166*(v*v)+
     & 108*nf*((-14+5*v)*(u*u)+3*(u*u*u)+u*(26-19*v+2*(v*v))-
     & 3*(8-9*v+2*(v*v)))-37*(v*v*v)+4*(v*v*v*v))))/
     & (18._dp*u*(omu*omu*omu)*(omv*omv*omv*omv))+
     & G2(2,3)*((-20*v*w*zeta2)/(9._dp*(u*u))+
     & (v*H1(0)*(-2*(14+60*NFV*(-2+v)+3*v)*(omv*omv*omv*omv)+
     & u*(omv*omv*omv)*(148-158*v+31*(v*v))+
     & u*(omv*omv)*(30*NFV*
     & (-28+64*v-(1+2*v)*(u*u*u)+u*u*u*u+
     & u*u*(-16+32*v-13*(v*v))-46*(v*v)+
     & u*(36-77*v+49*(v*v)-9*(v*v*v))+10*(v*v*v))+
     & u*(-1962+216*nf+548*v-352*(v*v)+73*(v*v*v)))+
     & u*u*u*u*(-20278+22882*v+843*(-14+5*v)*(u*u)+2529*(u*u*u)-
     & 5166*(v*v)+u*(21925-16031*v+1693*(v*v))-
     & 108*nf*((-14+5*v)*(u*u)+3*(u*u*u)+
     & u*(26-19*v+2*(v*v))-3*(8-9*v+2*(v*v)))+
     & 37*(v*v*v)-4*(v*v*v*v))+
     & omv*(u*u*u)*(9468-5542*v+108*nf*(-11+6*v)+408*(v*v)-
     & 123*(v*v*v)+4*(v*v*v*v))))/
     & (18._dp*u*(omu*omu*omu)*(omv*omv*omv*omv)))+
     & (v*H2(0,1)*(omv*omv*omv*(109-5*(21+4*NFV)*v+nf*(-6+11*v)+
     & 20*NFV*(v*v))+u*(omv*omv)*
     & (100-1245*v+nf*(20+65*v-33*(v*v))+683*(v*v)+
     & 10*NFV*(-3+9*v-13*(v*v)+4*(v*v*v)))+
     & omv*(u*u)*(3152+658*v-911*(v*v)+80*(v*v*v)+
     & 10*NFV*(30-40*v+42*(v*v)-19*(v*v*v)+2*(v*v*v*v)))+
     & u*u*(nf*(-447+475*v+3*(-343+127*v)*(u*u)+324*(u*u*u)+
     & 10*(v*v)+2*u*(569-433*v+26*(v*v))-38*(v*v*v))-
     & u*(8869-6642*v+9*u*(-893+331*v)+2529*(u*u)+222*(v*v)+
     & 80*(v*v*v)+10*NFV*
     & (69-110*v+89*(v*v)+2*(u*u)*(9-5*v+2*(v*v))-
     & 44*(v*v*v)+u*(-60+63*v-37*(v*v)+10*(v*v*v))+
     & 8*(v*v*v*v))))))/(18._dp*u*(omu*omu)*(omv*omv*omv))+
     & ((-108*v*(281*omu*u-12*nf*(-5+3*u+2*v)*w)*zeta3*(u*u))/
     & (omu*(omv*omv*omv*omv))+
     & (32*nf*(44-19*v)*w-3*omu*
     & (12*(404+v*(-404+7*zeta3-20*NFV*zeta3))+
     & 6*zeta2*(-359+20*NFV-6549*u+36*nf*v-5620*(u*u)+
     & 1124*(u*u*u))))/(omu*omv)+
     & (2*u*(-16*nf*w*(25-v*(50+81*zeta3)+25*(v*v))+
     & 9*omu*u*(2810*v*zeta3+
     & 6*zeta2*(281*(1+4*u)+
     & 4*nf*(16+33*v-72*(v*v)+6*u*(-2-5*v+v*v)+
     & 14*(v*v*v))))))/(omu*(omv*omv*omv))-
     & (u*(-36*(208+v*(-208-541*zeta3+10*NFV*zeta3))+
     & 6*zeta2*(6039+38778*u+10116*(u*u)+
     & 4*nf*(72+277*v-720*(v*v)+180*(v*v*v))-
     & 20*NFV*(9-50*v+111*(v*v)-120*(v*v*v)+44*(v*v*v*v)))))/
     & (omv*omv)+(-4*omu*u*v*w*
     & (-3*(588+317*zeta3+60*NFV*zeta3-
     & u*(588+289*zeta3+150*NFV*zeta3)+4*zeta3*(u*u))+
     & 6*zeta2*(5*NFV*(-7+24*v-u*(7+30*v)+(5+24*v)*(u*u))+
     & 2*(8+v*(-69+nf*(13-35*u+18*(u*u))))))+
     & 12*u*w*zeta3*(-419+493*u-106*(u*u)+30*NFV*(2-3*u+4*(u*u)))*
     & (v*v)-12*u*w*zeta2*(omu*omu)*
     & (-16+45*NFV+(-11+16*nf)*v+300*(v*v))+
     & omu*omu*omu*(24*(40+60*NFV*(-2+v)-51*v)*v*w*zeta3+
     & 6*zeta2*(4*(-2887+1599*v)*w*(u*u)+
     & 60*(137+6*NFV*(-1+2*v))*w*(u*u*u)+240*NFV*w*(u*u*u*u)+
     & u*(18+w*(-1939+18*(-89+8*nf)*v+1620*(v*v)+
     & 10*NFV*(15+56*(v*v*v))))+
     & 2*(1+w*(-1-4*(-76+2*nf+35*NFV)*v+
     & (-385+4*nf+160*NFV)*(v*v)-
     & 4*(-9+25*NFV)*(v*v*v)+80*NFV*(v*v*v*v))))))/
     & (u*w*(omu*omu*omu)))/216._dp-
     & (v*H3(0,0,0)*(-2*(v+60*NFV*v)*(omv*omv*omv*omv)+
     & u*(omv*omv*omv)*(96+10*(-23+36*NFV)*v+
     & (113-300*NFV)*(v*v))+
     & omv*omv*(u*u)*(1342-216*nf+1114*v-1058*(v*v)+313*(v*v*v)-
     & 30*NFV*(-2+17*v-(1+2*v)*(u*u)+u*u*u-25*(v*v)+9*(v*v*v))
     & )+3*omv*(u*u*u)*(-2950+1110*v-36*nf*(-11+6*v)+
     & 786*(v*v)-433*(v*v*v)+
     & 10*NFV*(-2+6*v-9*(v*v)+5*(v*v*v))+82*(v*v*v*v))+
     & u*u*u*u*(20002-21590*v-843*(-14+5*v)*(u*u)-2529*(u*u*u)+
     & 2920*(v*v)+108*nf*((-14+5*v)*(u*u)+3*(u*u*u)+
     & u*(26-19*v+2*(v*v))-3*(8-9*v+2*(v*v)))+
     & 1763*(v*v*v)-646*(v*v*v*v)+
     & u*(-21863+15747*v-1231*(v*v)-320*(v*v*v)+80*(v*v*v*v))+
     & 80*(v*v*v*v*v))))/(18._dp*u*(omu*omu*omu)*(omv*omv*omv*omv))+
     & (H2(1,0)*(3*(u*u*u*u*u*u*u)*(20*NFV*(omv*omv*omv)-
     & 281*(-2-5*v+v*v)+36*nf*(-2-5*v+v*v))+
     & 2*v*(omv*omv*omv)*(92-207*v+nf*(-6+13*v-7*(v*v))+
     & 20*NFV*(omv*omv)*(-6+v*v)+107*(v*v)+9*(v*v*v))+
     & 2*(u*u*u*u*u*u)*(36*nf*(13+30*v-30*(v*v)+5*(v*v*v))-
     & 3*(1203+2854*v-2854*(v*v)+483*(v*v*v))-
     & 5*NFV*(27-141*v+173*(v*v)-107*(v*v*v)+24*(v*v*v*v)))-
     & u*(omv*omv*omv)*(-200+(-738+60*nf-660*NFV)*v-
     & 4*(-526+33*nf-275*NFV)*(v*v)+
     & (-1439+78*nf-60*NFV)*(v*v*v)-4*(-9+85*NFV)*(v*v*v*v)+
     & 80*NFV*(v*v*v*v*v))+
     & 2*(u*u*u*u*u)*(6025+14553*v-29157*(v*v)+12935*(v*v*v)-
     & 1827*(v*v*v*v)+nf*(-792-1783*v+3619*(v*v)-1584*(v*v*v)+
     & 216*(v*v*v*v))-5*NFV*
     & (-42+363*v-603*(v*v)+519*(v*v*v)-229*(v*v*v*v)+
     & 40*(v*v*v*v*v)))-
     & 2*(omv*omv)*(u*u)*(530+2996*v-6001*(v*v)+2994*(v*v*v)-
     & 411*(v*v*v*v)+nf*(-36-333*v+589*(v*v)-259*(v*v*v)+
     & 36*(v*v*v*v))+9*(v*v*v*v*v)+
     & 5*NFV*(6-36*v+27*(v*v)-126*(v*v*v)+143*(v*v*v*v)-
     & 48*(v*v*v*v*v)+4*(v*v*v*v*v*v)))-
     & omv*(u*u*u)*(-4192-18193*v+41154*(v*v)-23879*(v*v*v)+
     & 5452*(v*v*v*v)-423*(v*v*v*v*v)+
     & nf*(504+2186*v-4860*(v*v)+2712*(v*v*v)-576*(v*v*v*v)+
     & 36*(v*v*v*v*v))-
     & 10*NFV*(18-288*v+536*(v*v)-583*(v*v*v)+396*(v*v*v*v)-
     & 139*(v*v*v*v*v)+18*(v*v*v*v*v*v)))+
     & 2*(u*u*u*u)*(-4925-13324*v+41601*(v*v)-31357*(v*v*v)+
     & 9007*(v*v*v*v)-1002*(v*v*v*v*v)+
     & nf*(648+1589*v-5084*(v*v)+3783*(v*v*v)-1044*(v*v*v*v)+
     & 108*(v*v*v*v*v))-
     & 5*NFV*(33-456*v+997*(v*v)-1095*(v*v*v)+710*(v*v*v*v)-
     & 249*(v*v*v*v*v)+36*(v*v*v*v*v*v)))))/
     & (36._dp*u*w*(omu*omu)*(omv*omv*omv))+
     & (H2(0,0)*(3*(u*u*u*u*u*u*u)*(20*NFV*(omv*omv*omv)-
     & 281*(-2-5*v+v*v)+36*nf*(-2-5*v+v*v))+
     & 2*(omv*omv*omv)*(v*v)*
     & (-194+12*omv*nf+184*v+9*(v*v)+
     & 20*NFV*(5-5*v-v*v+v*v*v))+
     & 6*(u*u*u*u*u*u)*(-1203-2011*v+5*NFV*(-9+8*v)*(omv*omv*omv)+
     & 2854*(v*v)-483*(v*v*v)+12*nf*(13+21*v-30*(v*v)+5*(v*v*v))
     & )-u*(omv*omv*omv)*
     & (-160+4*(-221+6*nf-60*NFV)*v+
     & (3112-216*nf+840*NFV)*(v*v)+
     & (108*nf-5*(341+48*NFV))*(v*v*v)-4*(-9+65*NFV)*(v*v*v*v)+
     & 80*NFV*(v*v*v*v*v))+
     & 2*(u*u*u*u*u)*(6023+3835*v-23495*(v*v)+12935*(v*v*v)-
     & 1827*(v*v*v*v)+12*nf*
     & (-66-35*v+242*(v*v)-132*(v*v*v)+18*(v*v*v*v))-
     & 5*NFV*(-54+255*v-473*(v*v)+441*(v*v*v)-209*(v*v*v*v)+
     & 40*(v*v*v*v*v)))-
     & 2*(omv*omv)*(u*u)*(520+6*(-127+35*NFV)*v-
     & (6388+425*NFV)*(v*v)+(3607-160*NFV)*(v*v*v)+
     & (-473+545*NFV)*(v*v*v*v)+
     & 12*nf*(-3+14*v+48*(v*v)-25*(v*v*v)+3*(v*v*v*v))+
     & (9-220*NFV)*(v*v*v*v*v)+20*NFV*(v*v*v*v*v*v))-
     & omv*(u*u*u)*(-4268+7873*v+26574*(v*v)-24209*(v*v*v)+
     & 5700*(v*v*v*v)-423*(v*v*v*v*v)+
     & 12*nf*(42-91*v-246*(v*v)+221*(v*v*v)-48*(v*v*v*v)+
     & 3*(v*v*v*v*v))-10*NFV*
     & (18-96*v+244*(v*v)-387*(v*v*v)+326*(v*v*v*v)-
     & 127*(v*v*v*v*v)+18*(v*v*v*v*v*v)))+
     & 2*(u*u*u*u)*(-4951+4157*v+23165*(v*v)-27909*(v*v*v)+
     & 9069*(v*v*v*v)-1002*(v*v*v*v*v)+
     & 12*nf*(54-51*v-230*(v*v)+278*(v*v*v)-87*(v*v*v*v)+
     & 9*(v*v*v*v*v))-5*NFV*
     & (51-300*v+693*(v*v)-855*(v*v*v)+604*(v*v*v*v)-
     & 229*(v*v*v*v*v)+36*(v*v*v*v*v*v)))))/
     & (36._dp*u*w*(omu*omu)*(omv*omv*omv))+
     & G2(3,0)*((v*H1(1)*(omw*omw)*(-10+(2+20*NFV)*v+
     & (9+20*NFV*(-2+3*v))*(u*u)+20*NFV*(u*u*u)+
     & (9-40*NFV)*(v*v)+2*u*(1+9*v+10*NFV*(1-4*v+3*(v*v)))+
     & 20*NFV*(v*v*v)))/(18._dp*w*(u*u))+
     & (omv*omv*omv*(-30*NFV*(-9+8*v)*(u*u*u*u*u*u)-
     & 60*NFV*(u*u*u*u*u*u*u)+
     & 2*(v*v)*(6+4*v-9*(v*v)-
     & 20*NFV*(6-5*v-2*(v*v)+v*v*v))+
     & u*(-160-12*(-61+6*nf+20*NFV)*v+
     & (724-72*nf+860*NFV)*(v*v)+
     & (-485+36*nf-60*NFV)*(v*v*v)-4*(-9+85*NFV)*(v*v*v*v)+
     & 80*NFV*(v*v*v*v*v)))+
     & u*u*u*u*(9902-7342*v-47806*(v*v)+
     & 843*(u*u*u)*(-2-5*v+v*v)+56430*(v*v*v)+
     & 6*(u*u)*(1203+2011*v-2854*(v*v)+483*(v*v*v))-
     & 18246*(v*v*v*v)+2*u*
     & (-6023-3997*v+23675*(v*v)-12953*(v*v*v)+1827*(v*v*v*v))
     & +2004*(v*v*v*v*v)-
     & 36*nf*(3*(u*u*u)*(-2-5*v+v*v)+
     & 2*(u*u)*(13+21*v-30*(v*v)+5*(v*v*v))+
     & 2*u*(-22-12*v+81*(v*v)-44*(v*v*v)+6*(v*v*v*v))+
     & 2*(18-16*v-78*(v*v)+93*(v*v*v)-29*(v*v*v*v)+
     & 3*(v*v*v*v*v))))+
     & 2*(omv*omv)*(u*u)*(520-2216*v-3498*(v*v)+2297*(v*v*v)-
     & 491*(v*v*v*v)+36*nf*
     & (-1+7*v+11*(v*v)-6*(v*v*v)+v*v*v*v)+9*(v*v*v*v*v)+
     & 5*NFV*(u*u*u*(-54+141*v-133*(v*v)+40*(v*v*v))+
     & u*u*(51-180*v+268*(v*v)-169*(v*v*v)+36*(v*v*v*v))+
     & v*(48-71*v-82*(v*v)+143*(v*v*v)-48*(v*v*v*v)+
     & 4*(v*v*v*v*v))))+
     & omv*(u*u*u)*(-4268+8517*v+24350*(v*v)-23025*(v*v*v)+
     & 5808*(v*v*v*v)-423*(v*v*v*v*v)+
     & 36*nf*(14-31*v-78*(v*v)+71*(v*v*v)-16*(v*v*v*v)+
     & v*v*v*v*v)-10*NFV*
     & (18-78*v+252*(v*v)-451*(v*v*v)+380*(v*v*v*v)-
     & 139*(v*v*v*v*v)+18*(v*v*v*v*v*v))))/
     & (36._dp*u*w*(omu*omu)*(omv*omv*omv)))+
     & G1(3)*(-(v*H2(0,1)*(omw*omw)*
     & (-10+(2+20*NFV)*v+(9+20*NFV*(-2+3*v))*(u*u)+
     & 20*NFV*(u*u*u)+(9-40*NFV)*(v*v)+
     & 2*u*(1+9*v+10*NFV*(1-4*v+3*(v*v)))+20*NFV*(v*v*v)))/
     & (18._dp*w*(u*u))-(v*H2(1,0)*(omw*omw)*
     & (-10+(2+20*NFV)*v+(9+20*NFV*(-2+3*v))*(u*u)+
     & 20*NFV*(u*u*u)+(9-40*NFV)*(v*v)+
     & 2*u*(1+9*v+10*NFV*(1-4*v+3*(v*v)))+20*NFV*(v*v*v)))/
     & (18._dp*w*(u*u))+(omw*v*zeta2*
     & (-40-8*v+5*(9+40*NFV*(-1+2*v))*(u*u*u)+100*NFV*(u*u*u*u)+
     & 10*(1+10*NFV)*(v*v)+
     & 5*(u*u)*(2+27*v+20*NFV*(1-6*v+6*(v*v)))-
     & 5*(-9+40*NFV)*(v*v*v)+
     & u*(-8+20*(1+10*NFV)*v-15*(-9+40*NFV)*(v*v)+
     & 400*NFV*(v*v*v))+100*NFV*(v*v*v*v)))/(18._dp*w*(u*u))+
     & (H1(0)*(omv*omv*omv*(30*NFV*(-9+8*v)*(u*u*u*u*u*u)+
     & 60*NFV*(u*u*u*u*u*u*u)+
     & 2*(v*v)*(-6-4*v+9*(v*v)+
     & 20*NFV*(6-5*v-2*(v*v)+v*v*v))+
     & u*(160+12*(-61+6*nf+20*NFV)*v+
     & (-724+72*nf-860*NFV)*(v*v)+
     & (485-36*nf+60*NFV)*(v*v*v)+4*(-9+85*NFV)*(v*v*v*v)-
     & 80*NFV*(v*v*v*v*v)))+
     & u*u*u*u*(-9902+7342*v+47806*(v*v)-
     & 843*(u*u*u)*(-2-5*v+v*v)-56430*(v*v*v)-
     & 6*(u*u)*(1203+2011*v-2854*(v*v)+483*(v*v*v))+
     & u*(12046+7994*v-47350*(v*v)+25906*(v*v*v)-
     & 3654*(v*v*v*v))+18246*(v*v*v*v)-2004*(v*v*v*v*v)+
     & 36*nf*(3*(u*u*u)*(-2-5*v+v*v)+
     & 2*(u*u)*(13+21*v-30*(v*v)+5*(v*v*v))+
     & 2*u*(-22-12*v+81*(v*v)-44*(v*v*v)+6*(v*v*v*v))+
     & 2*(18-16*v-78*(v*v)+93*(v*v*v)-29*(v*v*v*v)+
     & 3*(v*v*v*v*v))))-
     & 2*(omv*omv)*(u*u)*(520-2216*v-3498*(v*v)+2297*(v*v*v)-
     & 491*(v*v*v*v)+36*nf*
     & (-1+7*v+11*(v*v)-6*(v*v*v)+v*v*v*v)+9*(v*v*v*v*v)+
     & 5*NFV*(u*u*u*(-54+141*v-133*(v*v)+40*(v*v*v))+
     & u*u*(51-180*v+268*(v*v)-169*(v*v*v)+36*(v*v*v*v))+
     & v*(48-71*v-82*(v*v)+143*(v*v*v)-48*(v*v*v*v)+
     & 4*(v*v*v*v*v))))+
     & omv*(u*u*u)*(4268-8517*v-24350*(v*v)+23025*(v*v*v)-
     & 5808*(v*v*v*v)+423*(v*v*v*v*v)-
     & 36*nf*(14-31*v-78*(v*v)+71*(v*v*v)-16*(v*v*v*v)+
     & v*v*v*v*v)+10*NFV*
     & (18-78*v+252*(v*v)-451*(v*v*v)+380*(v*v*v*v)-
     & 139*(v*v*v*v*v)+18*(v*v*v*v*v*v)))))/
     & (36._dp*u*w*(omu*omu)*(omv*omv*omv)))+
     & G2(0,1)*((7*v*w*zeta2)/(u*u)+(v*w*H2(0,1))/(u*u)+(v*w*H2(1,1))/(u*u)+
     & (H1(0)*(21-21/omv-82*v+(80*v)/omu-(v*(-34+nf+22*v))/u+
     & (omv*(16+nf)*v)/(u*u)-
     & (15*(-281+36*nf)*v*(u*u))/(omv*omv*omv)+
     & (9*(-281+36*nf)*v*(u*u*u))/(omv*omv*omv*omv)+80*(v*v)-
     & (20*(v*v))/omu-(80*(v*v))/(omu*omu)+
     & (u*v*(-1613+216*nf-160*v+80*(v*v)))/(omv*omv)-
     & (30*NFV*v*(w*w))/(u*(omv*omv))))/18._dp+
     & (H1(1)*(21-21/omv-82*v+(80*v)/omu-(v*(-34+nf+22*v))/u+
     & (omv*(16+nf)*v)/(u*u)-
     & (15*(-281+36*nf)*v*(u*u))/(omv*omv*omv)+
     & (9*(-281+36*nf)*v*(u*u*u))/(omv*omv*omv*omv)+80*(v*v)-
     & (20*(v*v))/omu-(80*(v*v))/(omu*omu)+
     & (u*v*(-1613+216*nf-160*v+80*(v*v)))/(omv*omv)-
     & (30*NFV*v*(w*w))/(u*(omv*omv))))/18._dp)+
     & G2(1,0)*((2*(-7+60*NFV)*v*w*zeta2)/(9._dp*(u*u))+
     & (5*(-1+4*NFV)*v*w*H2(0,0))/(3._dp*(u*u))-(5*v*w*H2(0,1))/(9._dp*(u*u))+
     & (2*(-1+10*NFV)*v*w*H2(1,0))/(3._dp*(u*u))+(4*v*w*H2(1,1))/(9._dp*(u*u))+
     & (v*H1(0)*(-63+141*u+106*v-152*u*v-158*(u*u)+196*v*(u*u)+
     & 226*(u*u*u)-292*v*(u*u*u)-217*(u*u*u*u)+142*v*(u*u*u*u)+
     & 71*(u*u*u*u*u)-42*(v*v)+32*u*(v*v)-75*(u*u)*(v*v)+
     & 71*(u*u*u)*(v*v)+omu*nf*(w*w)+
     & 20*NFV*(omu*omu)*(1+u+u*u)*(w*w)))/(18._dp*omu*w*(u*u))+
     & (v*H1(1)*(-63+141*u+106*v-152*u*v-158*(u*u)+196*v*(u*u)+
     & 226*(u*u*u)-292*v*(u*u*u)-217*(u*u*u*u)+142*v*(u*u*u*u)+
     & 71*(u*u*u*u*u)-42*(v*v)+32*u*(v*v)-75*(u*u)*(v*v)+
     & 71*(u*u*u)*(v*v)+omu*nf*(w*w)+
     & 20*NFV*(omu*omu)*(1+u+u*u)*(w*w)))/(18._dp*omu*w*(u*u)))+
     & G3(2,1,2)*((-8*v*w*H1(0))/(9._dp*(u*u))+
     & (21-21/omv-15*v-(200*v)/omu-
     & (15*(-281+36*nf)*v*(u*u))/(omv*omv*omv)+
     & (9*(-281+36*nf)*v*(u*u*u))/(omv*omv*omv*omv)+
     & (v*(-50+nf+38*v-27*(v*v)))/u-27*(v*v)+(64*(v*v))/omu+
     & (200*(v*v))/(omu*omu)-
     & (u*v*(1702-216*nf-18*v+9*(v*v)))/(omv*omv)+
     & (omv*v*(38-nf+20*v+9*(v*v)))/(u*u)-
     & (10*NFV*v*(2*(omv*omv)*(1+v+u*u+v*v)+
     & u*(5-6*(v*v)+4*(v*v*v)))*(w*w))/(omv*omv*(u*u)))/18._dp)+
     & G2(2,1)*((-22*v*w*zeta2)/(9._dp*(u*u))-(8*v*w*H2(0,1))/(9._dp*(u*u))+
     & (8*v*w*H2(1,0))/(9._dp*(u*u))+
     & (H1(0)*(-21+21/omv+15*v+(200*v)/omu+
     & (15*(-281+36*nf)*v*(u*u))/(omv*omv*omv)-
     & (9*(-281+36*nf)*v*(u*u*u))/(omv*omv*omv*omv)+27*(v*v)-
     & (64*(v*v))/omu-(200*(v*v))/(omu*omu)+
     & (u*v*(1702-216*nf-18*v+9*(v*v)))/(omv*omv)-
     & (omv*v*(38-nf+20*v+9*(v*v)))/(u*u)+
     & (v*(50-nf-38*v+27*(v*v)))/u+
     & (10*NFV*v*(2*(omv*omv)*(1+v+u*u+v*v)+
     & u*(5-6*(v*v)+4*(v*v*v)))*(w*w))/(omv*omv*(u*u))))/18._dp+
     & (H1(1)*(-21+21/omv+15*v+(200*v)/omu+
     & (15*(-281+36*nf)*v*(u*u))/(omv*omv*omv)-
     & (9*(-281+36*nf)*v*(u*u*u))/(omv*omv*omv*omv)+27*(v*v)-
     & (64*(v*v))/omu-(200*(v*v))/(omu*omu)+
     & (u*v*(1702-216*nf-18*v+9*(v*v)))/(omv*omv)-
     & (omv*v*(38-nf+20*v+9*(v*v)))/(u*u)+
     & (v*(50-nf-38*v+27*(v*v)))/u+
     & (10*NFV*v*(2*(omv*omv)*(1+v+u*u+v*v)+
     & u*(5-6*(v*v)+4*(v*v*v)))*(w*w))/(omv*omv*(u*u))))/18._dp)
     & +G3(0,1,2)*(-((v*w*H1(1))/(u*u))+
     & (21/omv-(omv*(16+nf)*v)/(u*u)+
     & (15*(-281+36*nf)*v*(u*u))/(omv*omv*omv)+
     & (9*(281-36*nf)*v*(u*u*u))/(omv*omv*omv*omv)+
     & (20*omu*u*(-4+v)*v+
     & omu*omu*(v*(-34+nf+22*v)+u*(-21+82*v-80*(v*v)))+
     & 80*u*(v*v))/(u*(omu*omu))+
     & (v*(u*u*(1613-216*nf+160*v-80*(v*v))+30*NFV*(w*w)))/
     & (u*(omv*omv)))/18._dp)+
     & G3(1,0,2)*((-5*v*w*H1(0))/(9._dp*(u*u))-(4*v*w*H1(1))/(9._dp*(u*u))-
     & (v*(-63+106*v+(-217+142*v)*(u*u*u*u)+71*(u*u*u*u*u)-
     & 42*(v*v)+u*u*u*(226-292*v+71*(v*v))+omu*nf*(w*w)+
     & 20*NFV*(omu*omu)*(w*w)+
     & u*u*(-158+196*v-75*(v*v)+20*NFV*(omu*omu)*(w*w))+
     & u*(141-152*v+32*(v*v)+20*NFV*(omu*omu)*(w*w))))/
     & (18._dp*omu*w*(u*u)))+G3(1,2,0)*
     & ((-2*(-1+10*NFV)*v*w*H1(0))/(3._dp*(u*u))-(4*v*w*H1(1))/(9._dp*(u*u))-
     & (v*(-63+106*v+(-217+142*v)*(u*u*u*u)+71*(u*u*u*u*u)-
     & 42*(v*v)+u*u*u*(226-292*v+71*(v*v))+omu*nf*(w*w)+
     & 20*NFV*(omu*omu)*(w*w)+
     & u*u*(-158+196*v-75*(v*v)+20*NFV*(omu*omu)*(w*w))+
     & u*(141-152*v+32*(v*v)+20*NFV*(omu*omu)*(w*w))))/
     & (18._dp*omu*w*(u*u)))+G3(1,1,2)*
     & ((16*v*w*H1(0))/(9._dp*(u*u))+(20*v*w*H1(1))/(9._dp*(u*u))-
     & (v*(-14+8*v+(-610+302*v)*(u*u*u*u*u)+151*(u*u*u*u*u*u)+
     & u*u*u*(-372+718*v-308*(v*v))+7*(v*v)+
     & u*u*u*u*(800-918*v+151*(v*v))+4*nf*(omu*omu)*(w*w)-
     & 20*NFV*(omu*omu*omu)*(w*w)+
     & u*(30+28*v-38*(v*v)-20*NFV*(omu*omu*omu)*(w*w))+
     & u*u*(15-138*v+60*(v*v)-20*NFV*(omu*omu*omu)*(w*w))))/
     & (18._dp*w*(omu*omu)*(u*u)))+
     & G2(1,1)*((4*v*w*zeta2)/(3._dp*(u*u))-(4*v*w*H2(0,1))/(9._dp*(u*u))-
     & (16*v*w*H2(1,0))/(9._dp*(u*u))-(20*v*w*H2(1,1))/(9._dp*(u*u))+
     & (v*H1(0)*(-14+8*v+(-610+302*v)*(u*u*u*u*u)+151*(u*u*u*u*u*u)+
     & u*u*u*(-372+718*v-308*(v*v))+7*(v*v)+
     & u*u*u*u*(800-918*v+151*(v*v))+4*nf*(omu*omu)*(w*w)-
     & 20*NFV*(omu*omu*omu)*(w*w)+
     & u*(30+28*v-38*(v*v)-20*NFV*(omu*omu*omu)*(w*w))+
     & u*u*(15-138*v+60*(v*v)-20*NFV*(omu*omu*omu)*(w*w))))/
     & (18._dp*w*(omu*omu)*(u*u))+
     & (v*H1(1)*(-14+8*v+(-610+302*v)*(u*u*u*u*u)+151*(u*u*u*u*u*u)+
     & u*u*u*(-372+718*v-308*(v*v))+7*(v*v)+
     & u*u*u*u*(800-918*v+151*(v*v))+4*nf*(omu*omu)*(w*w)-
     & 20*NFV*(omu*omu*omu)*(w*w)+
     & u*(30+28*v-38*(v*v)-20*NFV*(omu*omu*omu)*(w*w))+
     & u*u*(15-138*v+60*(v*v)-20*NFV*(omu*omu*omu)*(w*w))))/
     & (18._dp*w*(omu*omu)*(u*u)))+
     & ((5-u/omv-(4*v)/omu)/2._dp-((1+3*u)*v*w*H1(1))/(2._dp*u*(omu*omu))+
     & G1(2)*(((1+3*u)*v*w)/(2._dp*u*(omu*omu))+(v*w*H1(0))/(2._dp*(u*u)))+
     & G1(1)*(-(v*w*H1(0))/(2._dp*(u*u))-(v*w*H1(1))/(2._dp*(u*u)))+
     & (v*w*G2(1,2))/(2._dp*(u*u))+
     & (v*H1(0)*((v+u*(-4+3*v))*(omv*omv)+
     & u*u*(3+2*u-8*v-u*u+4*(v*v))))/
     & (2._dp*u*(omu*omu)*(omv*omv)))*(G1(0)*G1(0))+
     & (((1+3*u)*v*w)/(2._dp*u*(omu*omu))+(v*w*H1(0))/(2._dp*(u*u)))*
     & (G1(2)*G1(2)*G1(2))+((v*H1(1)*
     & ((-17+35*v+u*(-106+105*v))*(omv*omv)+
     & 3*(u*u)*(35+12*u-82*v-6*(u*u)+41*(v*v))))/
     & (18._dp*u*(omu*omu)*(omv*omv))-
     & (w*((-17+2*(-20+nf)*v)*(u*u*u)+omv*(-57+2*nf)*(v*v)+
     & u*u*(102-(113+2*nf)*v+(125-2*nf)*(v*v))+
     & u*(-85+(381-8*nf)*v+4*(-131+4*nf)*(v*v)+
     & (171-6*nf)*(v*v*v))))/(18._dp*u*(omu*omu)*(omv*omv)))*
     & (H1(0)*H1(0))+G1(2)*G1(2)*((-3*(1+3*u)*v*w*H1(1))/(2._dp*u*(omu*omu))+
     & (v*w*G2(1,2))/(2._dp*(u*u))+
     & (w*(omv*(-30+nf)*v+18*(-3+2*v)*(u*u)+9*(u*u*u)+
     & u*(45+3*(-42+nf)*v-3*(-30+nf)*(v*v))))/
     & (18._dp*omv*u*(omu*omu))+
     & H1(0)*(-((v*w*H1(1))/(u*u))+
     & (v*w*((-30+nf*(omu*omu))*(omv*omv)+
     & 3*u*(14-25*v-3*(1+v)*(u*u)+3*(u*u*u)+
     & u*(-40+80*v-37*(v*v))+11*(v*v))))/
     & (18._dp*(omu*omu)*(omv*omv)*(u*u)))-(v*w*(H1(0)*H1(0)))/(u*u))+
     & G1(0)*(G2(1,2)*(((-30+nf)*v*w)/(18._dp*(u*u))-(v*w*H1(0))/(u*u))-
     & (v*w*H2(1,1)*(-4-75*u+2*(u*u)))/(9._dp*u*(omu*omu))+
     & (H2(0,1)*(9+(75+82/omu)*v-9/w-
     & (5*v*(843-108*nf+4*NFV*(omv*omv*omv))*(u*u))/
     & (omv*omv*omv)-(9*(-281+36*nf)*v*(u*u*u))/
     & (omv*omv*omv*omv)+(10*(v*v))/u-
     & ((147-136*u+71*(u*u))*(v*v))/(omu*omu)+
     & u*(-1694/omv+(-71+20*NFV)*v+9/w+
     & (1694-216*nf*v)/(omv*omv)-20*NFV*(v*v))))/18._dp+
     & (H2(1,0)*(9+(6*v*(1+20*omv*NFV+2*v))/u-9/w+
     & (v*(114-139*u+30*NFV*(-2+5*u)-7*(u*u)))/(omu*omu)-
     & 20*NFV*v*(u*u)-((102-106*u-37*(u*u)+
     & 30*NFV*(2-3*u+4*(u*u))+9*(u*u*u))*(v*v))/
     & (omu*omu*omu)+u*
     & ((9+20*NFV)*v-20*NFV*(v*v)+
     & (9*(1+2*(-2+u)*v+3*(v*v)))/(w*(omv*omv)))))/18._dp+
     & (H2(0,0)*(-(omu*u*v*(32+81*v)*w*(omv*omv*omv*omv))+
     & u*v*(81+28*v)*w*(omu*omu)*(omv*omv*omv*omv)+
     & 2*u*v*w*(omv*omv*omv)*
     & (16*omv*v-15*NFV*
     & (2*(u*u*u)-2*(v*v)+u*(-1+4*v+3*(v*v))-
     & u*u*(1+v+4*(v*v))))+
     & omu*omu*omu*((1-1693*u)*u*w*(omv*omv*omv)+
     & 9*(281-36*nf)*v*w*(u*u*u*u)+
     & omv*omv*omv*omv*
     & ((18-62*v*w)*(u*u)+u*(-18+w*(17+64*v-62*(v*v)))+
     & 30*(1-4*NFV)*w*(v*v))+
     & 5*omv*v*w*(u*u*u)*(-843+108*nf+8*NFV*v*(3-3*v+v*v))+
     & w*(omv*omv)*(u*u)*
     & (1693-216*nf*v-10*NFV*v*(-3+12*v-12*(v*v)+4*(v*v*v)))
     & )))/(18._dp*u*w*(omu*omu*omu)*(omv*omv*omv*omv))+
     & (24*omu*u*v*(-16+80*v+15*NFV*(3+5*v))*zeta2*
     & (omv*omv*omv*omv)-
     & 2*u*v*(138+6*(160-22*v)*zeta2+
     & 15*NFV*(-9+12*(5+4*v)*zeta2))*(omu*omu)*
     & (omv*omv*omv*omv)+
     & 24*(16-45*NFV)*u*zeta2*(omv*omv*omv*omv)*(v*v)+
     & omu*omu*omu*(12*nf*u*(-10+9*v)*w*(omv*omv*omv)+
     & 1440*NFV*v*zeta2*(omv*omv*omv*omv*omv)-
     & 54*v*(281*u+36*nf*w)*zeta2*(u*u*u)+
     & omv*omv*omv*omv*
     & (24*v*(-5+7*v)*zeta2+
     & u*(1260+v*(-1107+30*NFV-1056*zeta2)+
     & 20*(3*NFV+48*zeta2)*(v*v)))+
     & 2*(omv*omv)*(u*u)*
     & (2037-v*(219+4656*zeta2)-2*(909+480*zeta2)*(v*v)+
     & 480*zeta2*(v*v*v)+15*NFV*(9-2*v-11*(v*v)+4*(v*v*v)))+
     & 3*omv*(u*u)*(36*nf*w*(-6+v*(3+12*zeta2)+3*(v*v))+
     & u*(281*(-6+3*v+30*v*zeta2+3*(v*v))-
     & 20*NFV*(9-5*v-5*(v*v)+v*v*v)))))/
     & (108._dp*u*(omu*omu*omu)*(omv*omv*omv*omv))+
     & (H1(1)*(40*NFV*v*(1+v)*(omv*omv*omv*omv*omv)+
     & omv*(u*u*u)*(-3360-16206*v+15046*(v*v)-4315*(v*v*v)+
     & 405*(v*v*v*v)-36*nf*
     & (-12-56*v+50*(v*v)-13*(v*v*v)+v*v*v*v))+
     & u*u*u*u*(6516+20002*v-31646*(v*v)-
     & 843*(u*u)*(-2-5*v+v*v)+11785*(v*v*v)-
     & 3*u*(1844+4865*v-4022*(v*v)+685*(v*v*v))-
     & 1599*(v*v*v*v)+36*nf*
     & (-24-68*v+108*(v*v)+3*(u*u)*(-2-5*v+v*v)-
     & 39*(v*v*v)+u*(20+51*v-42*(v*v)+7*(v*v*v))+
     & 5*(v*v*v*v)))+
     & omv*omv*omv*(30*NFV*(-7+6*v)*(u*u*u*u*u)+
     & 60*NFV*(u*u*u*u*u*u)+2*v*(3-13*v+9*(v*v))+
     & 10*NFV*(u*u*u*u)*(27-50*v+22*(v*v))+
     & 10*NFV*(u*u*u)*(-15+44*v-53*(v*v)+14*(v*v*v))-
     & u*(180+(255-36*nf+100*NFV)*v+
     & (-449+36*nf+40*NFV)*(v*v)-4*(-9+55*NFV)*(v*v*v)+
     & 80*NFV*(v*v*v*v)))+
     & omv*omv*(u*u)*(870+4673*v-2923*(v*v)+786*(v*v*v)-
     & 36*nf*(2+17*v-9*(v*v)+2*(v*v*v))-18*(v*v*v*v)-
     & 10*NFV*(-3+9*v-45*(v*v)+71*(v*v*v)-36*(v*v*v*v)+
     & 4*(v*v*v*v*v)))))/(36._dp*u*(omu*omu)*(omv*omv*omv))+
     & H1(0)*(((1+3*u)*v*w*H1(1))/(u*(omu*omu))-
     & (2*(omv*omv*omv)*(v*v)*
     & (3-13*v+20*NFV*(1+v)*(omv*omv)+9*(v*v))+
     & 3*(u*u*u*u*u*u*u)*(20*NFV*(omv*omv*omv)-
     & 281*(-2-5*v+v*v)+36*nf*(-2-5*v+v*v))+
     & 2*(u*u*u*u*u*u)*(36*nf*(13+30*v-30*(v*v)+5*(v*v*v))-
     & 3*(1203+2854*v-2854*(v*v)+483*(v*v*v))-
     & 5*NFV*(27-69*v+133*(v*v)-91*(v*v*v)+24*(v*v*v*v)))-
     & u*(omv*omv*omv)*(-340+16*(14+2*nf-15*NFV)*v+
     & (564-72*nf+240*NFV)*(v*v)+
     & (-467+36*nf+60*NFV)*(v*v*v)-4*(-9+65*NFV)*(v*v*v*v)+
     & 80*NFV*(v*v*v*v*v))+
     & 2*(u*u*u*u*u)*(6041+14500*v-29138*(v*v)+12953*(v*v*v)-
     & 1827*(v*v*v*v)+
     & 4*nf*(-198-445*v+904*(v*v)-396*(v*v*v)+54*(v*v*v*v))-
     & 5*NFV*(-54+105*v-299*(v*v)+345*(v*v*v)-185*(v*v*v*v)+
     & 40*(v*v*v*v*v)))-
     & 2*(omv*omv)*(u*u)*
     & (808+10*(130+51*NFV)*v-(3677+485*NFV)*(v*v)+
     & (1968-160*NFV)*(v*v*v)+(-411+485*NFV)*(v*v*v*v)+
     & 4*nf*(-9-70*v+125*(v*v)-54*(v*v*v)+9*(v*v*v*v))+
     & (9-220*NFV)*(v*v*v*v*v)+20*NFV*(v*v*v*v*v*v))+
     & omv*(u*u*u)*(4916+14877*v-37298*(v*v)+22573*(v*v*v)-
     & 5488*(v*v*v*v)+423*(v*v*v*v*v)-
     & 4*nf*(126+525*v-1182*(v*v)+665*(v*v*v)-144*(v*v*v*v)+
     & 9*(v*v*v*v*v))+
     & 10*NFV*(18+102*v-36*(v*v)-195*(v*v*v)+254*(v*v*v*v)-
     & 119*(v*v*v*v*v)+18*(v*v*v*v*v*v)))+
     & 2*(u*u*u*u)*(-5095-12720*v+41017*(v*v)-31243*(v*v*v)+
     & 9043*(v*v*v*v)-1002*(v*v*v*v*v)+
     & 4*nf*(162+394*v-1266*(v*v)+944*(v*v*v)-261*(v*v*v*v)+
     & 27*(v*v*v*v*v))-
     & 5*NFV*(51-48*v+237*(v*v)-507*(v*v*v)+460*(v*v*v*v)-
     & 205*(v*v*v*v*v)+36*(v*v*v*v*v*v))))/
     & (36._dp*u*w*(omu*omu)*(omv*omv*omv)))-
     & (v*((v+u*(-4+3*v))*(omv*omv)+
     & u*u*(3+2*u-8*v-u*u+4*(v*v)))*(H1(0)*H1(0)))/
     & (u*(omu*omu)*(omv*omv))+
     & G1(2)*(((-30+nf)*(1+3*u)*v*w)/(18._dp*u*(omu*omu))+
     & (v*w*H1(0)*(-30+42*u+nf*(omu*omu)-84*(u*u)))/
     & (18._dp*(omu*omu)*(u*u))-(v*w*(H1(0)*H1(0)))/(u*u))+
     & G1(1)*(H1(0)*(-((-30+nf)*v*w)/(18._dp*(u*u))+(v*w*H1(1))/(u*u))-
     & ((-30+nf)*v*w*H1(1))/(18._dp*(u*u))+(v*w*(H1(0)*H1(0)))/(u*u)))+
     & (17*v*((v+u*(-4+3*v))*(omv*omv)+
     & u*u*(3+2*u-8*v-u*u+4*(v*v)))*(H1(0)*H1(0)*H1(0)))/
     & (18._dp*u*(omu*omu)*(omv*omv))+
     & (w*(omv*(-30+nf)*v+18*(-3+2*v)*(u*u)+9*(u*u*u)+
     & u*(45+3*(-42+nf)*v-3*(-30+nf)*(v*v)))*(H1(1)*H1(1)))/
     & (18._dp*omv*u*(omu*omu))+G2(1,2)*
     & (((1+60*NFV)*v*w*H2(0,0))/(9._dp*(u*u))-(10*v*w*H2(0,1))/(9._dp*(u*u))-
     & ((-31+60*NFV)*v*w*H2(1,0))/(9._dp*(u*u))+
     & (v*w*H1(1)*(-6+54*u+5*nf*(omu*omu)+84*(u*u)-4*(u*u*u)))/
     & (18._dp*(omu*omu)*(u*u))+
     & H1(0)*((v*w*H1(1))/(u*u)+(v*w*
     & (-36+106*u+5*nf*(omu*omu)-24*(u*u)+162*(u*u*u)-
     & 80*(u*u*u*u)))/(18._dp*(omu*omu)*(u*u)))+
     & (omu*omu*(-2529*omv*(-7+4*u+3*(u*u))*(u*u*u)-
     & 3*(omv*omv)*(u*u)*(857+2515*u-4215*(u*u)+843*(u*u*u))+
     & 15174*(-1+u)*(u*u*u*u)+
     & omv*omv*omv*((-8634+4797*v)*(u*u*u)+6165*(u*u*u*u)+
     & 9*(u*u)*(243-55*v+135*(v*v))+
     & v*(-510*(-1+v)*zeta2-3*(391-411*v+18*(v*v)))+
     & u*(-102+v*(1167-510*zeta2)+54*(v*v)+54*(v*v*v))))+
     & 6*omu*(omv*omv)*(2*omv*
     & (60*NFV*u*v*w*zeta2+(48-37*v)*v*(u*u)+
     & 15*NFV*w*(u*u*u*u*u)+5*NFV*v*w*(3+v-12*zeta2+2*(v*v)))
     & -5*NFV*u*w*(-9-3*(v*v)+3*(u*u*u)*(3-7*v+4*(v*v))+
     & 14*(v*v*v)+u*u*(-9+18*v-25*(v*v)+10*(v*v*v))-
     & 8*(v*v*v*v)+u*(15-3*v+18*(v*v)-22*(v*v*v)+
     & 4*(v*v*v*v))))+
     & 4*(-60*(omv*omv*omv)*(u*u)*(v*v)+
     & nf*(u*v*(-63+44*v)*(omv*omv*omv)+
     & 19*v*(omv*omv*omv*omv)+
     & 81*(u*u*u*u*u*u*u)*(-2-5*v+v*v)+
     & v*(omv*omv)*(u*u)*(153-100*v+28*(v*v))+
     & 27*(u*u*u*u*u*u)*(20+69*v-42*(v*v)+7*(v*v*v))+
     & 27*(u*u*u*u*u)*(-24-125*v+129*(v*v)-39*(v*v*v)+
     & 5*(v*v*v*v))+
     & 27*(u*u*u*u)*(12+107*v-154*(v*v)+66*(v*v*v)-
     & 14*(v*v*v*v)+v*v*v*v*v)-
     & u*u*u*(54+1081*v-2001*(v*v)+1164*(v*v*v)-
     & 352*(v*v*v*v)+54*(v*v*v*v*v)))))/
     & (108._dp*(omu*omu)*(omv*omv*omv)*(u*u))+
     & (17*v*w*(H1(0)*H1(0)))/(18._dp*(u*u))+(v*w*(H1(1)*H1(1)))/(2._dp*(u*u)))+
     & H1(0)*((omu*u*v*w*(8*(-567+nf*(46+4*u))*v-6*(32+423*v)*zeta2)*
     & (omv*omv*omv*omv)+
     & u*v*w*(-496*nf+18*(308-6*v+15*NFV*v)+6*(423+103*v)*zeta2)*
     & (omu*omu)*(omv*omv*omv*omv)+
     & 12*u*v*w*zeta2*(omv*omv*omv)*
     & (16*omv*v-15*NFV*(2*(u*u*u)-2*(v*v)+u*(-1+4*v+3*(v*v))-
     & u*u*(1+v+4*(v*v))))-
     & omu*omu*omu*(3*u*w*(82+8*nf-120*NFV-440*u+42*zeta2+
     & 13536*u*zeta2)*(omv*omv*omv)+
     & 72*omv*(1405+843*u-180*nf*v)*w*zeta2*(u*u*u)+
     & 216*(-281+36*nf*v)*w*zeta2*(u*u*u*u)+
     & w*(omv*omv)*(u*u)*
     & (4*nf*(6+8*v*(11+162*zeta2)-81*(v*v))-
     & 3*(487+12*(1128+2810*u+5*NFV*v)*zeta2+
     & 20*NFV*v*(-9-4*v+v*v)))+
     & omv*omv*omv*omv*
     & (w*(2529*(u*u)-(76*nf+15*(-81+4*NFV*(5+v)))*(v*v)-
     & 6*u*(-169-606*v+24*nf*(1+3*v)+
     & 5*NFV*(-12-3*v+4*(v*v))))+
     & 6*zeta2*(v*(28+3*(-39+40*NFV)*v)*w+100*NFV*v*w*(u*u*u)+
     & 5*(u*u)*(-9+5*(11-4*NFV)*v*w+20*NFV*w*(v*v))+
     & u*(45+w*(-66-289*v+275*(v*v)))))))/
     & (108._dp*u*w*(omu*omu*omu)*(omv*omv*omv*omv))-
     & (w*H1(1)*(omv*v*(57-87*v+nf*(-2+3*v))+
     & (-18+(-12+nf)*v)*(u*u*u)-
     & u*u*(-108+(150+nf)*v+(-102+nf)*(v*v))+
     & u*(-90+(453-10*nf)*v+(-654+20*nf)*(v*v)-
     & 9*(-29+nf)*(v*v*v))))/(18._dp*u*(omu*omu)*(omv*omv))+
     & (v*((-2+3*v+u*(-8+9*v))*(omv*omv)+
     & u*u*(9+2*u-20*v-u*u+10*(v*v)))*(H1(1)*H1(1)))/
     & (2._dp*u*(omu*omu)*(omv*omv)))+
     & G1(2)*(G2(1,2)*(((-30+nf)*v*w)/(18._dp*(u*u))-(v*w*H1(0))/(u*u)-
     & (v*w*H1(1))/(u*u))+((1+60*NFV)*v*w*H3(0,0,0))/(9._dp*(u*u))-
     & (2*v*w*H3(0,0,1))/(9._dp*(u*u))-
     & ((-23+60*NFV)*v*w*H3(0,1,0))/(9._dp*(u*u))-
     & (5*(-1+4*NFV)*v*w*H3(1,0,0))/(3._dp*(u*u))-
     & (4*v*w*H3(1,0,1))/(9._dp*(u*u))+(4*v*w*H3(1,1,0))/(u*u)-
     & (w*H1(1)*(-120*omv*v*zeta2+omv*u*v*(-21-6*nf+240*zeta2)+
     & 54*(-3+2*v)*(u*u*u)+27*(u*u*u*u)+
     & u*u*(135+v*(393-18*nf-120*zeta2)+
     & (-501+18*nf+120*zeta2)*(v*v))))/(27._dp*omv*(omu*omu)*(u*u))
     & +(v*H2(1,0)*((u*(-38+120*NFV*(-2+v)+57*v)+
     & nf*w*(omu*omu*omu))*(omv*omv)+41*(omv*omv*omv)+
     & omv*(u*u)*(20-205*v+164*(v*v)+
     & 60*NFV*(14-18*v+5*(v*v)))+
     & u*u*u*(-60+265*v+19*(u*u*u)-344*(v*v)-
     & u*u*(40+13*v+4*(v*v))-
     & 30*NFV*(36-77*v-(1+2*v)*(u*u)+u*u*u+
     & u*(-16+32*v-13*(v*v))+49*(v*v)-9*(v*v*v))+
     & u*(58-69*v+72*(v*v)-4*(v*v*v))+120*(v*v*v))))/
     & (18._dp*(omu*omu*omu)*(omv*omv)*(u*u))+
     & (H2(0,0)*(1-1/omv+15*v+(63*v)/omu-(32*v)/(omu*omu)-
     & (15*(-281+36*nf)*v*(u*u))/(omv*omv*omv)+
     & (9*(-281+36*nf)*v*(u*u*u))/(omv*omv*omv*omv)+27*(v*v)-
     & (8*(v*v))/omu-(63*(v*v))/(omu*omu)+
     & (32*(v*v))/(omu*omu*omu)+
     & (u*v*(-1684+216*nf-18*v+9*(v*v)))/(omv*omv)-
     & (omv*v*(164-12*nf+20*v+9*(v*v)))/(u*u)+
     & (v*(136-12*nf+18*v+27*(v*v)))/u+
     & 10*NFV*v*(u*(-2+8*v+1/(omv*omv))+
     & (2-4/(omv*omv*omv))*(u*u)+
     & 3*(-2/omv+(-2+5*u)/(omu*omu)+
     & (v*(-4+9*u-10*(u*u)+2*(u*u*u)))/(omu*omu*omu)+
     & 4*(v*v))+(26-12*v-6*(v*v)+8*(v*v*v))/u-
     & (2*(omv+v*v*v-v*v*v*v))/(u*u))))/18._dp+
     & (-222+4743*v-(5544*v)/omu-540*nf*v+(496*nf*v)/omu-
     & 30*v*zeta2-(3306*v*zeta2)/omu+(192*v*zeta2)/(omu*omu)-
     & (54*(-281+36*nf)*v*zeta2*(u*u*u))/(omv*omv*omv*omv)+
     & (108*(v*v))/omu+(32*nf*(v*v))/omu+162*zeta2*(v*v)-
     & (426*zeta2*(v*v))/omu+(4536*(v*v))/(omu*omu)-
     & (400*nf*(v*v))/(omu*omu)+(3306*zeta2*(v*v))/(omu*omu)-
     & (192*zeta2*(v*v))/(omu*omu*omu)+
     & (3*(-281+36*nf)*(u*u)*(-6+v*(3+30*zeta2)+3*(v*v)))/
     & (omv*omv*omv)-(omv*v*
     & (132*zeta3+6*zeta2*(-39+4*nf+20*v+9*(v*v))))/(u*u)+
     & (v*(4*nf*(19-19*v+6*zeta2)+3*(-405+405*v+44*zeta3)+
     & 6*zeta2*(60-65*v+27*(v*v))))/u+
     & (3*u*(-1276+v*(-779+3366*zeta2)+(2055-36*zeta2)*(v*v)-
     & 4*nf*(-46+v*(-17+108*zeta2)+63*(v*v))+18*zeta2*(v*v*v)))
     & /(omv*omv)+10*NFV*
     & (u*u*(12*v*zeta2-(6*(-9-4*v+v*v))/(omv*omv))-
     & (2*omv*v*(36*zeta3+6*zeta2*(-1+v*v*v)))/(u*u)+
     & (v*(-6*(-5+4*v-12*zeta3+v*v)+
     & 6*zeta2*(-23+12*v-6*(v*v)+8*(v*v*v))))/u+
     & (3*(6*omu*(2-5*u)*v*zeta2+
     & (-15+3*u*(12+6*zeta2)-(27+12*zeta2)*(u*u)+
     & 2*(3+6*zeta2)*(u*u*u))*(v*v)+
     & omu*omu*omu*(9+6*v+24*zeta2*(v*v*v))))/
     & (omu*omu*omu)+
     & (u*(-81+4*v*(9+6*zeta2)+3*(21+24*zeta2)*(v*v)-
     & 18*(1+6*zeta2)*(v*v*v)+48*zeta2*(v*v*v*v)))/(omv*omv)))
     & /108._dp+(H2(0,1)*(21-21/omv-11*v-(72*v)/omu-
     & (15*(-281+36*nf)*v*(u*u))/(omv*omv*omv)+
     & (9*(-281+36*nf)*v*(u*u*u))/(omv*omv*omv*omv)-27*(v*v)+
     & (18*(v*v))/omu+(72*(v*v))/(omu*omu)-
     & (u*v*(1702-216*nf-18*v+9*(v*v)))/(omv*omv)+
     & (omv*v*(2+5*nf+20*v+9*(v*v)))/(u*u)-
     & (v*(-28+5*nf+4*v+27*(v*v)))/u-
     & (10*NFV*v*(2*(omv*omv)*(1+v+u*u+v*v)+
     & u*(5-6*(v*v)+4*(v*v*v)))*(w*w))/(omv*omv*(u*u))))/18._dp+
     & ((v*w*H1(1))/(u*u)+(v*w*((57-2*nf*(omu*omu))*(omv*omv)+
     & u*(-97+176*v+18*(1+v)*(u*u)-18*(u*u*u)-79*(v*v)+
     & 18*u*(10-20*v+9*(v*v)))))/
     & (18._dp*(omu*omu)*(omv*omv)*(u*u)))*(H1(0)*H1(0))+
     & (17*v*w*(H1(0)*H1(0)*H1(0)))/(18._dp*(u*u))+
     & (3*(1+3*u)*v*w*(H1(1)*H1(1)))/(2._dp*u*(omu*omu))+
     & H1(0)*((v*w*H1(1)*((30-nf*(omu*omu))*(omv*omv)-
     & 6*u*(4-5*v-3*(1+v)*(u*u)+3*(u*u*u)+
     & u*(-35+70*v-32*(v*v))+v*v)))/
     & (18._dp*(omu*omu)*(omv*omv)*(u*u))+
     & (9*(u*u*u*u*u*u*u)*(20*NFV*(omv*omv*omv)-281*(-2-5*v+v*v)+
     & 36*nf*(-2-5*v+v*v))-
     & v*(omv*omv*omv)*(534*(-1+v)*zeta2+
     & 3*(391-411*v+18*(v*v))-
     & 4*omv*(19*nf+15*NFV*(3+v-12*zeta2+2*(v*v))))+
     & omv*omv*(u*u)*(-720+3*v*(-1347+148*nf+534*zeta2)+
     & (9150-688*nf-2136*zeta2)*(v*v)+
     & (-4350+256*nf+534*zeta2)*(v*v*v)+162*(v*v*v*v)-
     & 30*omv*NFV*(33+6*v*(-1+12*zeta2)+
     & (43-24*zeta2)*(v*v)-56*(v*v*v)+12*(v*v*v*v)))+
     & u*(omv*omv*omv)*(-102-3*v*(-1067+76*nf+534*zeta2)+
     & 2*(-1152+64*nf+534*zeta2)*(v*v)+162*(v*v*v)+
     & 30*NFV*(9+12*v*(-1+6*zeta2)+(9-48*zeta2)*(v*v)-
     & 24*(v*v*v)+12*(v*v*v*v)))+
     & 3*(u*u*u*u*u*u)*(36*nf*(20+51*v-42*(v*v)+7*(v*v*v))-
     & 3*(1844+4865*v-4022*(v*v)+685*(v*v*v))-
     & 10*NFV*(21-117*v+137*(v*v)-83*(v*v*v)+18*(v*v*v*v)))+
     & 3*(u*u*u*u*u)*(6550+19937*v-31615*(v*v)+11785*(v*v*v)-
     & 1599*(v*v*v*v)+
     & 4*nf*(-216-611*v+971*(v*v)-351*(v*v*v)+45*(v*v*v*v))-
     & 10*NFV*(-33+267*v-397*(v*v)+329*(v*v*v)-
     & 140*(v*v*v*v)+22*(v*v*v*v*v)))+
     & omv*(u*u*u)*(3378+11823*v-534*v*zeta2-26310*(v*v)+
     & 1068*zeta2*(v*v)+13584*(v*v*v)-534*zeta2*(v*v*v)-
     & 2520*(v*v*v*v)+
     & 4*nf*(-54-451*v+794*(v*v)-340*(v*v*v)+54*(v*v*v*v))+
     & 54*(v*v*v*v*v)+
     & 30*NFV*(48+4*v*(-39+6*zeta2)+(213-48*zeta2)*(v*v)+
     & 4*(-70+6*zeta2)*(v*v*v)+197*(v*v*v*v)-
     & 56*(v*v*v*v*v)+4*(v*v*v*v*v*v)))+
     & 3*(u*u*u*u)*(-3556-12283*v+30726*(v*v)-19220*(v*v*v)+
     & 4738*(v*v*v*v)-405*(v*v*v*v*v)+
     & 4*nf*(108+395*v-954*(v*v)+568*(v*v*v)-126*(v*v*v*v)+
     & 9*(v*v*v*v*v))-
     & 10*NFV*(42-300*v+537*(v*v)-573*(v*v*v)+379*(v*v*v*v)-
     & 123*(v*v*v*v*v)+14*(v*v*v*v*v*v))))/
     & (108._dp*(omu*omu)*(omv*omv*omv)*(u*u))+
     & (v*w*(H1(1)*H1(1)))/(2._dp*(u*u))))-
     & ((1+3*u)*v*w*(H1(1)*H1(1)*H1(1)))/(2._dp*u*(omu*omu))+
     & G1(1)*(-((1+60*NFV)*v*w*H3(0,0,0))/(9._dp*(u*u))+
     & (2*v*w*H3(0,0,1))/(9._dp*(u*u))+
     & ((-23+60*NFV)*v*w*H3(0,1,0))/(9._dp*(u*u))-
     & ((1+60*NFV)*v*w*H3(1,0,0))/(9._dp*(u*u))+(10*v*w*H3(1,0,1))/(9._dp*(u*u))+
     & ((-31+60*NFV)*v*w*H3(1,1,0))/(9._dp*(u*u))-
     & (v*w*H2(1,1)*(-18+57*u+3*nf*(omu*omu)+27*(u*u)-2*(u*u*u)))/
     & (9._dp*(omu*omu)*(u*u))-
     & (v*w*H2(1,0)*(-93+220*u+7*nf*(omu*omu)-81*(u*u)+162*(u*u*u)-
     & 80*(u*u*u*u)))/(18._dp*(omu*omu)*(u*u))+
     & (H1(1)*(omu*omu*(2529*omv*(-7+4*u+3*(u*u))*(u*u*u)+
     & 3*(omv*omv)*(u*u)*(857+2515*u-4215*(u*u)+843*(u*u*u))-
     & 15174*(-1+u)*(u*u*u*u)-
     & omv*omv*omv*((-8634+4797*v)*(u*u*u)+6165*(u*u*u*u)+
     & 9*(u*u)*(243-55*v+135*(v*v))+
     & v*(-510*(-1+v)*zeta2-3*(391-411*v+18*(v*v)))+
     & u*(-102+v*(1167-510*zeta2)+54*(v*v)+54*(v*v*v))))-
     & 6*omu*(omv*omv)*(2*omv*
     & (60*NFV*u*v*w*zeta2+(48-37*v)*v*(u*u)+
     & 15*NFV*w*(u*u*u*u*u)+5*NFV*v*w*(3+v-12*zeta2+2*(v*v))
     & )-5*NFV*u*w*(-9-3*(v*v)+
     & 3*(u*u*u)*(3-7*v+4*(v*v))+14*(v*v*v)+
     & u*u*(-9+18*v-25*(v*v)+10*(v*v*v))-8*(v*v*v*v)+
     & u*(15-3*v+18*(v*v)-22*(v*v*v)+4*(v*v*v*v))))-
     & 4*(-60*(omv*omv*omv)*(u*u)*(v*v)+
     & nf*(u*v*(-63+44*v)*(omv*omv*omv)+
     & 19*v*(omv*omv*omv*omv)+
     & 81*(u*u*u*u*u*u*u)*(-2-5*v+v*v)+
     & v*(omv*omv)*(u*u)*(153-100*v+28*(v*v))+
     & 27*(u*u*u*u*u*u)*(20+69*v-42*(v*v)+7*(v*v*v))+
     & 27*(u*u*u*u*u)*
     & (-24-125*v+129*(v*v)-39*(v*v*v)+5*(v*v*v*v))+
     & 27*(u*u*u*u)*(12+107*v-154*(v*v)+66*(v*v*v)-
     & 14*(v*v*v*v)+v*v*v*v*v)-
     & u*u*u*(54+1081*v-2001*(v*v)+1164*(v*v*v)-
     & 352*(v*v*v*v)+54*(v*v*v*v*v))))))/
     & (108._dp*(omu*omu)*(omv*omv*omv)*(u*u))+
     & (v*H2(0,1)*(107-329*u-194*v+404*u*v+270*(u*u)-60*v*(u*u)+
     & 98*(u*u*u)-292*v*(u*u*u)-217*(u*u*u*u)+142*v*(u*u*u*u)+
     & 71*(u*u*u*u*u)+86*(v*v)-96*u*(v*v)-75*(u*u)*(v*v)+
     & 71*(u*u*u)*(v*v)-11*omu*nf*(w*w)+
     & 20*NFV*(omu*omu)*(1+u+u*u)*(w*w)))/(18._dp*omu*w*(u*u))+
     & (v*H2(0,0)*(-308*v+9*(u*u*u*u)+143*(v*v)-2*u*(164-154*v+v*v)+
     & u*u*(183-22*v+9*(v*v))+4*(41-3*nf*(w*w)+5*NFV*(w*w))+
     & 2*(u*u*u)*(9*v-10*(1+NFV*(w*w)))))/(18._dp*w*(u*u))+
     & (v*(18*(17-20*NFV)*zeta3*(omu*omu)*(w*w)+
     & 6*zeta2*(49-178*v-124*(-2+v)*(u*u*u*u*u)-
     & 62*(u*u*u*u*u*u)+133*(v*v)-
     & 2*(u*u*u*u)*(65-186*v+31*(v*v))+
     & u*u*u*(-441+40*v+124*(v*v))-2*nf*(omu*omu)*(w*w)+
     & 40*NFV*(omu*omu*omu)*(w*w)+
     & u*(-303+640*v-257*(v*v)+40*NFV*(omu*omu*omu)*(w*w))+
     & u*u*(639-750*v+126*(v*v)+40*NFV*(omu*omu*omu)*(w*w)))))
     & /(54._dp*w*(omu*omu)*(u*u))+
     & (-(v*w*H1(0))/(2._dp*(u*u))-(v*w*H1(1))/(2._dp*(u*u)))*(G1(2)*G1(2))+
     & (((-57+2*nf)*v*w)/(18._dp*(u*u))-(35*v*w*H1(1))/(18._dp*(u*u)))*
     & (H1(0)*H1(0))-(17*v*w*(H1(0)*H1(0)*H1(0)))/(18._dp*(u*u))+
     & ((-30+nf)*v*w*(H1(1)*H1(1)))/(18._dp*(u*u))+
     & H1(0)*(((-29+nf)*v*w*H1(1))/(6._dp*(u*u))-
     & (3*(omu*omu)*(-843*omv*(-7+4*u+3*(u*u))*(u*u*u)-
     & omv*omv*(u*u)*(857+2515*u-4215*(u*u)+843*(u*u*u))+
     & 5058*(-1+u)*(u*u*u*u)+
     & omv*omv*omv*((-2878+1599*v)*(u*u*u)+2055*(u*u*u*u)+
     & v*(-391+411*v-234*(-1+v)*zeta2-18*(v*v))+
     & 3*(u*u)*(243-55*v+135*(v*v))+
     & u*(-34+v*(389-234*zeta2)+18*(v*v)+18*(v*v*v))))+
     & 6*omu*(omv*omv)*(2*omv*
     & (60*NFV*u*v*w*zeta2+(48-37*v)*v*(u*u)+
     & 15*NFV*w*(u*u*u*u*u)+
     & 5*NFV*v*w*(3+v-12*zeta2+2*(v*v)))-
     & 5*NFV*u*w*(-9-3*(v*v)+3*(u*u*u)*(3-7*v+4*(v*v))+
     & 14*(v*v*v)+u*u*(-9+18*v-25*(v*v)+10*(v*v*v))-
     & 8*(v*v*v*v)+
     & u*(15-3*v+18*(v*v)-22*(v*v*v)+4*(v*v*v*v))))+
     & 4*(-60*(omv*omv*omv)*(u*u)*(v*v)+
     & nf*(u*v*(-63+44*v)*(omv*omv*omv)+
     & 19*v*(omv*omv*omv*omv)+
     & 81*(u*u*u*u*u*u*u)*(-2-5*v+v*v)+
     & v*(omv*omv)*(u*u)*(153-100*v+28*(v*v))+
     & 27*(u*u*u*u*u*u)*(20+69*v-42*(v*v)+7*(v*v*v))+
     & 27*(u*u*u*u*u)*
     & (-24-125*v+129*(v*v)-39*(v*v*v)+5*(v*v*v*v))+
     & 27*(u*u*u*u)*(12+107*v-154*(v*v)+66*(v*v*v)-
     & 14*(v*v*v*v)+v*v*v*v*v)-
     & u*u*u*(54+1081*v-2001*(v*v)+1164*(v*v*v)-
     & 352*(v*v*v*v)+54*(v*v*v*v*v)))))/
     & (108._dp*(omu*omu)*(omv*omv*omv)*(u*u))-
     & (3*v*w*(H1(1)*H1(1)))/(2._dp*(u*u)))+
     & G1(2)*(H1(0)*(-((-30+nf)*v*w)/(18._dp*(u*u))+(2*v*w*H1(1))/(u*u))-
     & ((-30+nf)*v*w*H1(1))/(18._dp*(u*u))+(v*w*(H1(0)*H1(0)))/(u*u)+
     & (v*w*(H1(1)*H1(1)))/(u*u))-(v*w*(H1(1)*H1(1)*H1(1)))/(2._dp*(u*u)))
      return
      end
      
      
      function Gamma_2a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-2*pi*v*w*G3(0,1,0))/(u*u)+(pi*v*w*G3(0,1,1))/(u*u)-
     & (pi*v*w*G3(0,1,2))/(u*u)-(10*pi*v*w*G3(1,0,1))/(9._dp*(u*u))+
     & (4*pi*v*w*G3(1,0,2))/(9._dp*(u*u))-
     & ((1+20*NFV)*pi*v*w*G3(1,0,3))/(3._dp*(u*u))+
     & (4*pi*v*w*G3(1,1,0))/(9._dp*(u*u))+(4*pi*v*w*G3(1,1,1))/(3._dp*(u*u))-
     & (16*pi*v*w*G3(1,1,2))/(9._dp*(u*u))+(4*pi*v*w*G3(1,2,0))/(9._dp*(u*u))-
     & (10*pi*v*w*G3(1,2,1))/(3._dp*(u*u))+(4*pi*v*w*G3(1,2,2))/(9._dp*(u*u))-
     & ((1+20*NFV)*pi*v*w*G3(1,2,3))/(3._dp*(u*u))-
     & (4*pi*v*w*G3(2,0,1))/(9._dp*(u*u))+
     & ((-17+60*NFV)*pi*v*w*G3(2,0,3))/(9._dp*(u*u))+(pi*v*w*G3(2,1,0))/(u*u)-
     & (16*pi*v*w*G3(2,1,1))/(9._dp*(u*u))+(pi*v*w*G3(2,1,2))/(u*u)+
     & (22*pi*v*w*G3(2,2,1))/(9._dp*(u*u))-
     & ((-17+60*NFV)*pi*v*w*G3(2,2,3))/(9._dp*(u*u))+
     & (20*pi*v*w*G3(2,3,3))/(9._dp*(u*u))-
     & (pi*v*w*H2(1,1)*(-4-75*u+2*(u*u)))/(9._dp*u*(omu*omu))+
     & G2(0,2)*(-((pi*v*w*H1(0))/(u*u))-
     & (pi*v*w*(-4-75*u+2*(u*u)))/(9._dp*u*(omu*omu)))+
     & G2(2,0)*((-5*pi*v*w*H1(0))/(9._dp*(u*u))-
     & (pi*v*w*(-4-75*u+2*(u*u)))/(9._dp*u*(omu*omu)))+
     & G2(2,2)*(-((1+60*NFV)*pi*v*w*H1(0))/(9._dp*(u*u))-
     & (pi*v*w*(-4-75*u+2*(u*u)))/(9._dp*u*(omu*omu)))+
     & (pi*H2(0,1)*(9+(75+82/omu)*v-9/w-
     & (5*v*(843-108*nf+4*NFV*(omv*omv*omv))*(u*u))/
     & (omv*omv*omv)-(9*(-281+36*nf)*v*(u*u*u))/
     & (omv*omv*omv*omv)+(10*(v*v))/u-
     & ((147-136*u+71*(u*u))*(v*v))/(omu*omu)+
     & u*(-1694/omv+(-71+20*NFV)*v+9/w+
     & (1694-216*nf*v)/(omv*omv)-20*NFV*(v*v))))/18._dp+
     & (pi*H2(0,0)*(30-21/omv+75*v+(73*v)/omu-9/w-(32*v)/(omu*omu)+
     & u*(-1693/omv-71*v+9/w+(1693-216*nf*v)/(omv*omv))+
     & (15*(-281+36*nf)*v*(u*u))/(omv*omv*omv)-
     & (9*(-281+36*nf)*v*(u*u*u))/(omv*omv*omv*omv)-71*(v*v)+
     & (30*(v*v))/omu+(34*(v*v))/u-(73*(v*v))/(omu*omu)+
     & (32*(v*v))/(omu*omu*omu)+
     & 10*NFV*v*((-2*(6*v+u*u*u))/u+
     & (3*(-2*(u*u*u)+u*(1-4*v-3*(v*v))+2*(v*v)+
     & u*u*(1+v+4*(v*v))))/(omv*(omu*omu*omu))+
     & (u*(5-6*v+6*(v*v)-2*(v*v*v)))/(omv*omv))))/18._dp-
     & (pi*v*G2(3,0)*(omw*omw)*(-10+(2+20*NFV)*v+
     & (9+20*NFV*(-2+3*v))*(u*u)+20*NFV*(u*u*u)+(9-40*NFV)*(v*v)+
     & 2*u*(1+9*v+10*NFV*(1-4*v+3*(v*v)))+20*NFV*(v*v*v)))/
     & (18._dp*w*(u*u))-(pi*v*G2(3,2)*(omw*omw)*
     & (-10+(2+20*NFV)*v+(9+20*NFV*(-2+3*v))*(u*u)+
     & 20*NFV*(u*u*u)+(9-40*NFV)*(v*v)+
     & 2*u*(1+9*v+10*NFV*(1-4*v+3*(v*v)))+20*NFV*(v*v*v)))/
     & (18._dp*w*(u*u))+(pi*G2(0,3)*(-30+21/omv-75*v-(71*v)/omu+
     & (2*v*(-20+3*v))/u+9/w+(32*v)/(omu*omu)+
     & u*(1693/omv+71*v-9/w+(-1693+216*nf*v)/(omv*omv))-
     & (15*(-281+36*nf)*v*(u*u))/(omv*omv*omv)+
     & (9*(-281+36*nf)*v*(u*u*u))/(omv*omv*omv*omv)+71*(v*v)+
     & (10*(v*v))/omu+(71*(v*v))/(omu*omu)-
     & (32*(v*v))/(omu*omu*omu)+
     & 10*NFV*v*((2*(6*v+u*u*u))/u+
     & (3*(2*(u*u*u)-2*(v*v)+u*(-1+4*v+3*(v*v))-
     & u*u*(1+v+4*(v*v))))/(omv*(omu*omu*omu))+
     & (u*(-5+6*v-6*(v*v)+2*(v*v*v)))/(omv*omv))))/18._dp+
     & (pi*(6*zeta2*(((19-3*v)*v)/u+(9*omw)/w+
     & u*(3424/omv+151*v-9/w+(16*(-214+27*nf*v))/(omv*omv))-
     & (30*(-281+36*nf)*v*(u*u))/(omv*omv*omv)-
     & (v*(453-550*u+161*(u*u)))/(omu*omu)+
     & (18*(-281+36*nf)*v*(u*u*u))/(omv*omv*omv*omv)-
     & ((-450+695*u-460*(u*u)+151*(u*u*u))*(v*v))/(omu*omu*omu))
     & +(w*(omv*(-1215+76*nf)*v-
     & 3*(166+3*(-281+36*nf)*v)*(u*u*u)+
     & u*u*(720+(-5889+756*nf)*v-27*(-41+4*nf)*(v*v))-
     & 3*u*(74+(-13+36*nf)*v+18*(-41+4*nf)*(v*v))))/
     & (omv*u*(omu*omu))+
     & (10*NFV*w*(6*omv*v*(5+v+24*zeta2)+
     & 12*omv*v*zeta2*(u*u*u*u*u)+
     & 6*(u*u*u*u)*(-9-v*(4+6*zeta2)+(1+6*zeta2)*(v*v))+
     & 3*(u*u*u)*(45+2*v*(16+6*zeta2)-(7+12*zeta2)*(v*v)+
     & 2*(v*v*v))+3*u*
     & (9-4*v*(1+30*zeta2)+(13+120*zeta2)*(v*v)+6*(v*v*v))-
     & 2*(u*u)*(54+v*(45-156*zeta2)+156*zeta2*(v*v)+9*(v*v*v))))
     & /(omv*u*(omu*omu*omu))))/108._dp+
     & (omw*pi*v*G2(3,3)*(40-52*v+(9+40*NFV*(-1+2*v))*(u*u*u)+
     & 20*NFV*(u*u*u*u)+(2+20*NFV)*(v*v)+
     & u*u*(2+27*v+20*NFV*(1-6*v+6*(v*v)))+(9-40*NFV)*(v*v*v)+
     & u*(-52+(4+40*NFV)*v-3*(-9+40*NFV)*(v*v)+80*NFV*(v*v*v))+
     & 20*NFV*(v*v*v*v)))/(18._dp*w*(u*u))+
     & (pi*v*H2(1,0)*(omv*omv*omv*omv*
     & (-6+32*v+120*NFV*w+u*(71-117*v-300*NFV*w)+
     & 270*NFV*w*(u*u))+omv*omv*(u*u)*
     & (-1890+216*nf+441*v-402*(v*v)+121*(v*v*v))+
     & u*u*u*u*(-20368+23041*v+843*(-14+5*v)*(u*u)+2529*(u*u*u)-
     & 5214*(v*v)+u*(21962-16105*v+1730*(v*v))-
     & 108*nf*((-14+5*v)*(u*u)+3*(u*u*u)+u*(26-19*v+2*(v*v))-
     & 3*(8-9*v+2*(v*v)))+16*(v*v*v)-4*(v*v*v*v))+
     & omv*(u*u*u)*(9504-5491*v+108*nf*(-11+6*v)+309*(v*v)-
     & 111*(v*v*v)+4*(v*v*v*v))))/
     & (18._dp*u*(omu*omu*omu)*(omv*omv*omv*omv))+
     & (pi*v*G2(2,3)*(2*(14+60*NFV*(-2+v)+3*v)*(omv*omv*omv*omv)+
     & u*(omv*omv*omv)*(-148+158*v-31*(v*v))+
     & u*(omv*omv)*(u*(1962-216*nf-548*v+352*(v*v)-73*(v*v*v))-
     & 30*NFV*(-28+64*v-(1+2*v)*(u*u*u)+u*u*u*u+
     & u*u*(-16+32*v-13*(v*v))-46*(v*v)+
     & u*(36-77*v+49*(v*v)-9*(v*v*v))+10*(v*v*v)))+
     & omv*(u*u*u)*(-9468+5542*v-108*nf*(-11+6*v)-408*(v*v)+
     & 123*(v*v*v)-4*(v*v*v*v))+
     & u*u*u*u*(20278-22882*v-843*(-14+5*v)*(u*u)-2529*(u*u*u)+
     & u*(-21925+16031*v-1693*(v*v))+5166*(v*v)+
     & 108*nf*((-14+5*v)*(u*u)+3*(u*u*u)+u*(26-19*v+2*(v*v))-
     & 3*(8-9*v+2*(v*v)))-37*(v*v*v)+4*(v*v*v*v))))/
     & (18._dp*u*(omu*omu*omu)*(omv*omv*omv*omv))+
     & (pi*H1(1)*(40*NFV*v*(1+v)*(omv*omv*omv*omv*omv)+
     & omv*(u*u*u)*(-3360-16206*v+15046*(v*v)-4315*(v*v*v)+
     & 405*(v*v*v*v)-36*nf*
     & (-12-56*v+50*(v*v)-13*(v*v*v)+v*v*v*v))+
     & u*u*u*u*(6516+20002*v-31646*(v*v)-843*(u*u)*(-2-5*v+v*v)+
     & 11785*(v*v*v)-3*u*(1844+4865*v-4022*(v*v)+685*(v*v*v))-
     & 1599*(v*v*v*v)+36*nf*
     & (-24-68*v+108*(v*v)+3*(u*u)*(-2-5*v+v*v)-39*(v*v*v)+
     & u*(20+51*v-42*(v*v)+7*(v*v*v))+5*(v*v*v*v)))+
     & omv*omv*omv*(30*NFV*(-7+6*v)*(u*u*u*u*u)+
     & 60*NFV*(u*u*u*u*u*u)+2*v*(59-69*v+9*(v*v))+
     & 10*NFV*(u*u*u*u)*(27-50*v+22*(v*v))+
     & 10*NFV*(u*u*u)*(-15+44*v-53*(v*v)+14*(v*v*v))-
     & u*(180+(783-36*nf+100*NFV)*v+
     & (-865+36*nf+40*NFV)*(v*v)-4*(-9+55*NFV)*(v*v*v)+
     & 80*NFV*(v*v*v*v)))+
     & omv*omv*(u*u)*(870+5089*v-3339*(v*v)+786*(v*v*v)-
     & 36*nf*(2+17*v-9*(v*v)+2*(v*v*v))-18*(v*v*v*v)-
     & 10*NFV*(-3+9*v-45*(v*v)+71*(v*v*v)-36*(v*v*v*v)+
     & 4*(v*v*v*v*v)))))/(36._dp*u*(omu*omu)*(omv*omv*omv))+
     & G1(0)*(G1(1)*(((-30+nf)*pi*v*w)/(18._dp*(u*u))-(pi*v*w*H1(0))/(u*u))+
     & (pi*v*w*H1(1)*(-4-75*u+2*(u*u)))/(9._dp*u*(omu*omu))+
     & (pi*H1(0)*((1694*u)/omv+(9*(omw-u))/w+
     & (2*u*(-847+108*nf*v))/(omv*omv)+
     & (15*(281-36*nf)*v*(u*u))/(omv*omv*omv)+
     & v*(-75-154/omu-18/u+71*u-20*NFV*u+20*NFV*(u*u))+
     & (9*(-281+36*nf)*v*(u*u*u))/(omv*omv*omv*omv)+
     & (8/u+20*NFV*u+(237-154*u+71*(u*u))/(omu*omu))*(v*v)))/18._dp
     & -(pi*(40*NFV*v*(1+v)*(omv*omv*omv*omv*omv)+
     & omv*(u*u*u)*(-3360-16206*v+15046*(v*v)-4315*(v*v*v)+
     & 405*(v*v*v*v)-36*nf*
     & (-12-56*v+50*(v*v)-13*(v*v*v)+v*v*v*v))+
     & u*u*u*u*(6516+20002*v-31646*(v*v)-
     & 843*(u*u)*(-2-5*v+v*v)+11785*(v*v*v)-
     & 3*u*(1844+4865*v-4022*(v*v)+685*(v*v*v))-
     & 1599*(v*v*v*v)+36*nf*
     & (-24-68*v+108*(v*v)+3*(u*u)*(-2-5*v+v*v)-
     & 39*(v*v*v)+u*(20+51*v-42*(v*v)+7*(v*v*v))+
     & 5*(v*v*v*v)))+
     & omv*omv*omv*(30*NFV*(-7+6*v)*(u*u*u*u*u)+
     & 60*NFV*(u*u*u*u*u*u)+2*v*(3-13*v+9*(v*v))+
     & 10*NFV*(u*u*u*u)*(27-50*v+22*(v*v))+
     & 10*NFV*(u*u*u)*(-15+44*v-53*(v*v)+14*(v*v*v))-
     & u*(180+(255-36*nf+100*NFV)*v+
     & (-449+36*nf+40*NFV)*(v*v)-4*(-9+55*NFV)*(v*v*v)+
     & 80*NFV*(v*v*v*v)))+
     & omv*omv*(u*u)*(870+4673*v-2923*(v*v)+786*(v*v*v)-
     & 36*nf*(2+17*v-9*(v*v)+2*(v*v*v))-18*(v*v*v*v)-
     & 10*NFV*(-3+9*v-45*(v*v)+71*(v*v*v)-36*(v*v*v*v)+
     & 4*(v*v*v*v*v)))))/(36._dp*u*(omu*omu)*(omv*omv*omv)))+
     & G1(3)*((pi*v*H1(0)*(omw*omw)*
     & (-10+(2+20*NFV)*v+(9+20*NFV*(-2+3*v))*(u*u)+
     & 20*NFV*(u*u*u)+(9-40*NFV)*(v*v)+
     & 2*u*(1+9*v+10*NFV*(1-4*v+3*(v*v)))+20*NFV*(v*v*v)))/
     & (18._dp*w*(u*u))+(pi*v*H1(1)*(omw*omw)*
     & (-10+(2+20*NFV)*v+(9+20*NFV*(-2+3*v))*(u*u)+
     & 20*NFV*(u*u*u)+(9-40*NFV)*(v*v)+
     & 2*u*(1+9*v+10*NFV*(1-4*v+3*(v*v)))+20*NFV*(v*v*v)))/
     & (18._dp*w*(u*u))-(pi*(omv*omv*omv*
     & (30*NFV*(-9+8*v)*(u*u*u*u*u*u)+60*NFV*(u*u*u*u*u*u*u)+
     & 2*(v*v)*(-6-4*v+9*(v*v)+
     & 20*NFV*(6-5*v-2*(v*v)+v*v*v))+
     & u*(160+12*(-61+6*nf+20*NFV)*v+
     & (-724+72*nf-860*NFV)*(v*v)+
     & (485-36*nf+60*NFV)*(v*v*v)+4*(-9+85*NFV)*(v*v*v*v)-
     & 80*NFV*(v*v*v*v*v)))+
     & u*u*u*u*(-9902+7342*v+47806*(v*v)-
     & 843*(u*u*u)*(-2-5*v+v*v)-56430*(v*v*v)-
     & 6*(u*u)*(1203+2011*v-2854*(v*v)+483*(v*v*v))+
     & u*(12046+7994*v-47350*(v*v)+25906*(v*v*v)-
     & 3654*(v*v*v*v))+18246*(v*v*v*v)-2004*(v*v*v*v*v)+
     & 36*nf*(3*(u*u*u)*(-2-5*v+v*v)+
     & 2*(u*u)*(13+21*v-30*(v*v)+5*(v*v*v))+
     & 2*u*(-22-12*v+81*(v*v)-44*(v*v*v)+6*(v*v*v*v))+
     & 2*(18-16*v-78*(v*v)+93*(v*v*v)-29*(v*v*v*v)+
     & 3*(v*v*v*v*v))))-
     & 2*(omv*omv)*(u*u)*(520-2216*v-3498*(v*v)+2297*(v*v*v)-
     & 491*(v*v*v*v)+36*nf*
     & (-1+7*v+11*(v*v)-6*(v*v*v)+v*v*v*v)+9*(v*v*v*v*v)+
     & 5*NFV*(u*u*u*(-54+141*v-133*(v*v)+40*(v*v*v))+
     & u*u*(51-180*v+268*(v*v)-169*(v*v*v)+36*(v*v*v*v))+
     & v*(48-71*v-82*(v*v)+143*(v*v*v)-48*(v*v*v*v)+
     & 4*(v*v*v*v*v))))+
     & omv*(u*u*u)*(4268-8517*v-24350*(v*v)+23025*(v*v*v)-
     & 5808*(v*v*v*v)+423*(v*v*v*v*v)-
     & 36*nf*(14-31*v-78*(v*v)+71*(v*v*v)-16*(v*v*v*v)+
     & v*v*v*v*v)+10*NFV*
     & (18-78*v+252*(v*v)-451*(v*v*v)+380*(v*v*v*v)-
     & 139*(v*v*v*v*v)+18*(v*v*v*v*v*v)))))/
     & (36._dp*u*w*(omu*omu)*(omv*omv*omv)))+
     & H1(0)*((10*pi*(1+3*u)*v*w*H1(1))/(9._dp*u*(omu*omu))+
     & (pi*(-3*(u*u*u*u*u*u*u)*(20*NFV*(omv*omv*omv)-
     & 281*(-2-5*v+v*v)+36*nf*(-2-5*v+v*v))-
     & 6*(u*u*u*u*u*u)*(-1203-2011*v+
     & 5*NFV*(-9+8*v)*(omv*omv*omv)+2854*(v*v)-483*(v*v*v)+
     & 12*nf*(13+21*v-30*(v*v)+5*(v*v*v)))-
     & 2*v*(omv*omv*omv)*
     & (52-25*v-4*nf*(omv*omv)+(-37+20*NFV)*(v*v)+
     & (9-40*NFV)*(v*v*v)+20*NFV*(v*v*v*v))+
     & u*(omv*omv*omv)*(-180-4*(-161+16*nf+45*NFV)*v-
     & 4*(-413+26*nf-5*NFV)*(v*v)+
     & 3*(-353+20*nf+120*NFV)*(v*v*v)-
     & 4*(-9+85*NFV)*(v*v*v*v)+80*NFV*(v*v*v*v*v))-
     & 2*(u*u*u*u*u)*(6025+4038*v-23718*(v*v)+12953*(v*v*v)-
     & 1827*(v*v*v*v)+36*nf*
     & (-22-12*v+81*(v*v)-44*(v*v*v)+6*(v*v*v*v))-
     & 5*NFV*(-54+261*v-481*(v*v)+447*(v*v*v)-213*(v*v*v*v)+
     & 40*(v*v*v*v*v)))+
     & 2*(omv*omv)*(u*u)*(552+(-1997+180*NFV)*v+
     & 5*(-845+NFV)*(v*v)+(2780-710*NFV)*(v*v*v)+
     & (-491+715*NFV)*(v*v*v*v)+
     & 4*nf*(-9+58*v+110*(v*v)-60*(v*v*v)+9*(v*v*v*v))+
     & (9-240*NFV)*(v*v*v*v*v)+20*NFV*(v*v*v*v*v*v))-
     & omv*(u*u*u)*(4340-8243*v-24902*(v*v)+23321*(v*v*v)-
     & 5808*(v*v*v*v)+423*(v*v*v*v*v)-
     & 12*nf*(42-91*v-238*(v*v)+215*(v*v*v)-48*(v*v*v*v)+
     & 3*(v*v*v*v*v))+
     & 10*NFV*(18-96*v+264*(v*v)-457*(v*v*v)+380*(v*v*v*v)-
     & 139*(v*v*v*v*v)+18*(v*v*v*v*v*v)))-
     & 2*(u*u*u*u)*(-4967+3618*v+23967*(v*v)-28210*(v*v*v)+
     & 9123*(v*v*v*v)-1002*(v*v*v*v*v)+
     & 36*nf*(18-16*v-78*(v*v)+93*(v*v*v)-29*(v*v*v*v)+
     & 3*(v*v*v*v*v))-
     & 5*NFV*(51-306*v+703*(v*v)-885*(v*v*v)+642*(v*v*v*v)-
     & 241*(v*v*v*v*v)+36*(v*v*v*v*v*v)))))/
     & (36._dp*u*w*(omu*omu)*(omv*omv*omv)))+
     & G2(2,1)*(-(pi*v*w*H1(0))/(9._dp*(u*u))-(pi*v*w*H1(1))/(u*u)+
     & (pi*(21-21/omv-15*v-(200*v)/omu-
     & (15*(-281+36*nf)*v*(u*u))/(omv*omv*omv)+
     & (9*(-281+36*nf)*v*(u*u*u))/(omv*omv*omv*omv)+
     & (v*(-50+nf+38*v-27*(v*v)))/u-27*(v*v)+(64*(v*v))/omu+
     & (200*(v*v))/(omu*omu)-
     & (u*v*(1702-216*nf-18*v+9*(v*v)))/(omv*omv)+
     & (omv*v*(38-nf+20*v+9*(v*v)))/(u*u)-
     & (10*NFV*v*(2*(omv*omv)*(1+v+u*u+v*v)+
     & u*(5-6*(v*v)+4*(v*v*v)))*(w*w))/(omv*omv*(u*u))))/18._dp)
     & +G2(0,1)*((pi*v*w*H1(0))/(u*u)+(pi*v*w*H1(1))/(u*u)+
     & (pi*(21/omv-(omv*(16+nf)*v)/(u*u)+
     & (15*(-281+36*nf)*v*(u*u))/(omv*omv*omv)+
     & (9*(281-36*nf)*v*(u*u*u))/(omv*omv*omv*omv)+
     & (20*omu*u*(-4+v)*v+
     & omu*omu*(v*(-34+nf+22*v)+u*(-21+82*v-80*(v*v)))+
     & 80*u*(v*v))/(u*(omu*omu))+
     & (v*(u*u*(1613-216*nf+160*v-80*(v*v))+30*NFV*(w*w)))/
     & (u*(omv*omv))))/18._dp)+
     & G2(1,0)*((5*pi*v*w*H1(0))/(9._dp*(u*u))-(4*pi*v*w*H1(1))/(9._dp*(u*u))-
     & (pi*v*(-63+106*v+(-217+142*v)*(u*u*u*u)+71*(u*u*u*u*u)-
     & 42*(v*v)+u*u*u*(226-292*v+71*(v*v))+omu*nf*(w*w)+
     & 20*NFV*(omu*omu)*(w*w)+
     & u*u*(-158+196*v-75*(v*v)+20*NFV*(omu*omu)*(w*w))+
     & u*(141-152*v+32*(v*v)+20*NFV*(omu*omu)*(w*w))))/
     & (18._dp*omu*w*(u*u)))+G2(1,1)*
     & ((16*pi*v*w*H1(1))/(9._dp*(u*u))-
     & (pi*v*(-14+8*v+(-610+302*v)*(u*u*u*u*u)+151*(u*u*u*u*u*u)+
     & u*u*u*(-372+718*v-308*(v*v))+7*(v*v)+
     & u*u*u*u*(800-918*v+151*(v*v))+4*nf*(omu*omu)*(w*w)-
     & 20*NFV*(omu*omu*omu)*(w*w)+
     & u*(30+28*v-38*(v*v)-20*NFV*(omu*omu*omu)*(w*w))+
     & u*u*(15-138*v+60*(v*v)-20*NFV*(omu*omu*omu)*(w*w))))/
     & (18._dp*w*(omu*omu)*(u*u)))+
     & G2(1,2)*((-5*(-1+4*NFV)*pi*v*w*H1(0))/(3._dp*(u*u))-
     & (4*pi*v*w*H1(1))/(9._dp*(u*u))-
     & (pi*v*(3-26*v-142*(-2+v)*(u*u*u*u*u)-71*(u*u*u*u*u*u)+
     & u*u*u*u*(-279+426*v-71*(v*v))+24*(v*v)+
     & 2*(u*u*u)*(-11-84*v+71*(v*v))+nf*(omu*omu)*(w*w)+
     & 20*NFV*(omu*omu*omu)*(w*w)-
     & 2*u*(9-27*v+8*(v*v)-10*NFV*(omu*omu*omu)*(w*w))+
     & u*u*(103-144*v+49*(v*v)+20*NFV*(omu*omu*omu)*(w*w))))/
     & (18._dp*w*(omu*omu)*(u*u)))+
     & ((pi*(1+3*u)*v*w)/(2._dp*u*(omu*omu))+(pi*v*w*G1(1))/(2._dp*(u*u)))*
     & (G1(0)*G1(0))+(pi*(1+3*u)*v*w*(G1(2)*G1(2)))/(2._dp*u*(omu*omu))-
     & (pi*v*((-17+19*v+u*(-42+57*v))*(omv*omv)+
     & u*u*(57+4*u-118*v-2*(u*u)+59*(v*v)))*(H1(0)*H1(0)))/
     & (18._dp*u*(omu*omu)*(omv*omv))+
     & G1(2)*(-((-17+60*NFV)*pi*v*w*H2(0,0))/(9._dp*(u*u))+
     & (5*pi*v*w*H2(0,1))/(9._dp*(u*u))+(13*pi*v*w*H2(1,0))/(9._dp*(u*u))+
     & (pi*v*w*H1(1)*(-13-102*u+2*(u*u)))/(9._dp*u*(omu*omu))-
     & (pi*v*H1(0)*(u*(-190+457*u+135*v-280*u*v+
     & 30*NFV*(7-4*v+u*(-23+10*v)))*(omv*omv*omv*omv)+
     & 41*(omv*omv*omv*omv*omv)+
     & 2*(omv*omv)*(u*u*u)*
     & (-1118+108*nf+655*v-494*(v*v)+
     & 15*NFV*(26-65*v-6*(u*u)+2*(u*u*u)+46*(v*v)-
     & 9*(v*v*v))+111*(v*v*v))+
     & u*u*u*u*u*(-20254+22813*v+843*(-14+5*v)*(u*u)+
     & 2529*(u*u*u)-5100*(v*v)+9*u*(2436-1781*v+188*(v*v))-
     & 108*nf*((-14+5*v)*(u*u)+3*(u*u*u)+
     & u*(26-19*v+2*(v*v))-3*(8-9*v+2*(v*v)))+
     & 16*(v*v*v)-4*(v*v*v*v))+
     & omv*(u*u*u*u)*(9531-5800*v+108*nf*(-11+6*v)+732*(v*v)-
     & 252*(v*v*v)+180*NFV*(-1+5*v-6*(v*v)+2*(v*v*v))+
     & 4*(v*v*v*v))))/
     & (18._dp*(omu*omu*omu)*(omv*omv*omv*omv)*(u*u))+
     & (pi*(96*v*zeta2*(omv*omv*omv*omv)-
     & 120*NFV*u*v*(1+v)*(omv*omv*omv*omv*omv)+
     & 3*omv*(u*u*u*u)*(3360+16206*v-15046*(v*v)+4315*(v*v*v)-
     & 405*(v*v*v*v)+36*nf*
     & (-12-56*v+50*(v*v)-13*(v*v*v)+v*v*v*v))-
     & 3*(u*u*u*u*u)*(6516+20002*v-31646*(v*v)-
     & 843*(u*u)*(-2-5*v+v*v)+11785*(v*v*v)-
     & 3*u*(1844+4865*v-4022*(v*v)+685*(v*v*v))-
     & 1599*(v*v*v*v)+36*nf*
     & (-24-68*v+108*(v*v)+3*(u*u)*(-2-5*v+v*v)-
     & 39*(v*v*v)+u*(20+51*v-42*(v*v)+7*(v*v*v))+
     & 5*(v*v*v*v)))+
     & u*(omv*omv*omv)*(-90*NFV*(-7+6*v)*(u*u*u*u*u)-
     & 180*NFV*(u*u*u*u*u*u)-
     & 30*NFV*(u*u*u*u)*(27-50*v+22*(v*v))-
     & 2*v*(-48*(-3+2*v)*zeta2+3*(59-69*v+9*(v*v)))-
     & 30*NFV*(u*u*u)*(-15+44*v-53*(v*v)+14*(v*v*v))+
     & u*(540+3*v*(783-36*nf+100*NFV+96*zeta2)+
     & (-2595+108*nf+120*NFV-96*zeta2)*(v*v)-
     & 12*(-9+55*NFV)*(v*v*v)+240*NFV*(v*v*v*v)))+
     & omv*omv*(u*u*u)*(-2610-15267*v-96*v*zeta2+10017*(v*v)+
     & 96*zeta2*(v*v)-2358*(v*v*v)+
     & 108*nf*(2+17*v-9*(v*v)+2*(v*v*v))+54*(v*v*v*v)+
     & 30*NFV*(-3+9*v-45*(v*v)+71*(v*v*v)-36*(v*v*v*v)+
     & 4*(v*v*v*v*v)))))/(108._dp*(omu*omu)*(omv*omv*omv)*(u*u))
     & -(pi*v*w*(H1(0)*H1(0)))/(9._dp*(u*u)))+
     & (pi*(1+3*u)*v*w*(H1(1)*H1(1)))/(2._dp*u*(omu*omu))+
     & G1(1)*(G1(2)*(((-30+nf)*pi*v*w)/(18._dp*(u*u))-(pi*v*w*H1(0))/(u*u)-
     & (pi*v*w*H1(1))/(u*u))+((-17+60*NFV)*pi*v*w*H2(0,0))/(9._dp*(u*u))-
     & (5*pi*v*w*H2(0,1))/(9._dp*(u*u))+
     & (4*(-4+15*NFV)*pi*v*w*H2(1,0))/(9._dp*(u*u))+
     & (4*pi*v*w*H2(1,1))/(9._dp*(u*u))-
     & (pi*(-3*(omu*omu)*(-843*omv*(-7+4*u+3*(u*u))*(u*u*u)-
     & omv*omv*(u*u)*(857+2515*u-4215*(u*u)+843*(u*u*u))+
     & 5058*(-1+u)*(u*u*u*u)+
     & omv*omv*omv*((-2878+1599*v)*(u*u*u)+2055*(u*u*u*u)+
     & v*(-391+411*v+6*zeta2-6*v*zeta2-18*(v*v))+
     & 3*(u*u)*(243-55*v+135*(v*v))+
     & u*(-34+389*v-6*v*zeta2+18*(v*v)+18*(v*v*v))))-
     & 6*omu*(omv*omv)*(2*omv*
     & (-120*NFV*u*v*w*zeta2+(48-37*v)*v*(u*u)+
     & 15*NFV*w*(u*u*u*u*u)+5*NFV*v*w*(3+v+24*zeta2+2*(v*v))
     & )-5*NFV*u*w*(-9-3*(v*v)+
     & 3*(u*u*u)*(3-7*v+4*(v*v))+14*(v*v*v)+
     & u*u*(-9+18*v-25*(v*v)+10*(v*v*v))-8*(v*v*v*v)+
     & u*(15-3*v+18*(v*v)-22*(v*v*v)+4*(v*v*v*v))))-
     & 4*(-60*(omv*omv*omv)*(u*u)*(v*v)+
     & nf*(u*v*(-63+44*v)*(omv*omv*omv)+
     & 19*v*(omv*omv*omv*omv)+
     & 81*(u*u*u*u*u*u*u)*(-2-5*v+v*v)+
     & v*(omv*omv)*(u*u)*(153-100*v+28*(v*v))+
     & 27*(u*u*u*u*u*u)*(20+69*v-42*(v*v)+7*(v*v*v))+
     & 27*(u*u*u*u*u)*
     & (-24-125*v+129*(v*v)-39*(v*v*v)+5*(v*v*v*v))+
     & 27*(u*u*u*u)*(12+107*v-154*(v*v)+66*(v*v*v)-
     & 14*(v*v*v*v)+v*v*v*v*v)-
     & u*u*u*(54+1081*v-2001*(v*v)+1164*(v*v*v)-
     & 352*(v*v*v*v)+54*(v*v*v*v*v))))))/
     & (108._dp*(omu*omu)*(omv*omv*omv)*(u*u))-
     & (pi*v*H1(1)*(-33+86*v+142*(-2+v)*(u*u*u*u*u)+71*(u*u*u*u*u*u)+
     & u*u*(-283+324*v-79*(v*v))-54*(v*v)+
     & 2*u*(69-117*v+38*(v*v))+u*u*u*u*(249-426*v+71*(v*v))-
     & 2*(u*u*u)*(-71-54*v+71*(v*v))-
     & 20*NFV*(omu*omu*omu)*(1+u+u*u)*(w*w)))/
     & (18._dp*w*(omu*omu)*(u*u))+
     & H1(0)*((10*pi*v*w*H1(1))/(9._dp*(u*u))+
     & (pi*v*(-11-18*v+2*u*(-9+31*v)+31*(u*u)+31*(v*v)+
     & 4*nf*(w*w)))/(18._dp*w*(u*u)))+(pi*v*w*(G1(2)*G1(2)))/(2._dp*(u*u))+
     & (19*pi*v*w*(H1(0)*H1(0)))/(18._dp*(u*u))+(pi*v*w*(H1(1)*H1(1)))/(2._dp*(u*u)))
      return
      end
      
      
!**********************************************************************
!region3a:qbar+g-->V+qbar*
!*********************************************************************/
      
      function Alpha_3a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=1
      return
      end
      
      
      function Alpha_3a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function Alpha_3a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(u*H1(1))/3._dp+(G2(0,2)*(omu*omu))/3._dp+(G2(2,0)*(omu*omu))/3._dp+
     & H1(0)*(H1(1)/3._dp+(u*(u*(-12+v)+(-16+v)*v))/(3._dp*(omw*omw)))+
     & (48*(-8+u/omw)+6*zeta2*(81+16*u-8*(u*u)))/36._dp+
     & G1(2)*(-u/3._dp-H1(1)/3._dp+(H1(0)*(-3+2*u-u*u))/3._dp)+
     & (G2(1,2)*(-2+2*u-u*u))/3._dp+(G2(3,0)*(8+2*u-u*u))/3._dp+
     & (G1(3)*H1(0)*(-8-2*u+u*u))/3._dp+(H2(0,0)*(-8-2*u+u*u))/3._dp+
     & (H2(1,0)*(10-2*u+u*u))/3._dp+
     & G1(1)*((H1(0)*(2-2*u+u*u))/3._dp+(H1(1)*(2-2*u+u*u))/3._dp)+
     & G1(0)*(-(H1(1)*(omu*omu))/3._dp+(H1(0)*(17+2*u-u*u))/3._dp+
     & (u*(16*v+u*(12+v)+u*u))/(3._dp*(omw*omw)))-(3*(G1(0)*G1(0)))/2._dp+
     & (G1(2)*G1(2))/6._dp-(17*(H1(0)*H1(0)))/6._dp+(H1(1)*H1(1))/6._dp
      return
      end
      
      
      function Alpha_3a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(pi*G1(0)*(omu*omu))/3._dp+(pi*G1(2)*(omu*omu))/3._dp-
     & (pi*H1(0)*(omu*omu))/3._dp-(pi*H1(1)*(omu*omu))/3._dp-
     & (pi*G1(3)*(-8-2*u+u*u))/3._dp-(pi*G1(1)*(2-2*u+u*u))/3._dp-
     & (pi*v*(12*v+u*(8+v)+u*u))/(3._dp*(omw*omw))
      return
      end
      
      
      function Alpha_3a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-20*G4(1,1,0,2))/9._dp-(20*G4(1,1,2,0))/9._dp+
     & G3(1,1,0)*((20*H1(0))/9._dp+(20*H1(1))/9._dp)-2*(-2+u)*u*H4(0,0,0,1)+
     & (16*G4(1,0,2,2)*(omu*omu))/9._dp+(16*G4(1,2,0,2)*(omu*omu))/9._dp+
     & (16*G4(1,2,2,0)*(omu*omu))/9._dp+(16*G4(3,0,0,2)*(omu*omu))/9._dp+
     & (16*G4(3,0,2,0)*(omu*omu))/9._dp+(16*G4(3,2,0,0)*(omu*omu))/9._dp-
     & (10*H4(1,0,0,1)*(omu*omu))/9._dp+
     & G3(2,1,0)*(H1(0)*(-2+2*u-u*u)+H1(1)*(-2+2*u-u*u))+
     & (H4(1,1,0,0)*(70+44*u-25*(u*u)))/9._dp+
     & (2*G4(3,0,3,0)*(104+33*u-15*(u*u)))/9._dp+
     & (G4(0,0,2,2)*(2+14*u-7*(u*u)))/9._dp+
     & (G4(0,2,0,2)*(2+14*u-7*(u*u)))/9._dp+
     & (G4(0,2,2,0)*(2+14*u-7*(u*u)))/9._dp+
     & (G4(2,0,0,2)*(2+14*u-7*(u*u)))/9._dp+
     & (G4(2,0,2,0)*(2+14*u-7*(u*u)))/9._dp+
     & (G4(2,2,0,0)*(2+14*u-7*(u*u)))/9._dp+
     & (2*G3(3,3,3)*H1(0)*(64+9*u-6*(u*u)))/9._dp+
     & (G4(2,3,3,0)*(160+4*u-5*(u*u)))/9._dp+
     & (2*H4(1,0,1,0)*(130+3*u-3*(u*u)))/9._dp+
     & H4(1,0,0,0)*(-10+6*u-3*(u*u))+
     & G3(0,0,3)*H1(0)*(16+(50*u)/9._dp-(22*(u*u))/9._dp)+
     & G4(2,2,3,0)*(-4+(32*u)/9._dp-(13*(u*u))/9._dp)+
     & (G4(0,3,0,2)*(8+2*u-u*u))/9._dp+(G4(0,3,2,0)*(8+2*u-u*u))/9._dp-
     & (16*G4(0,3,3,0)*(-8-2*u+u*u))/9._dp+
     & (G3(0,3,2)*H1(0)*(-8-2*u+u*u))/9._dp+
     & (16*G3(0,3,3)*H1(0)*(-8-2*u+u*u))/9._dp-
     & (8*H4(0,0,0,0)*(-8-2*u+u*u))/3._dp-2*G4(0,1,0,2)*(2-2*u+u*u)-
     & 2*G4(0,1,2,0)*(2-2*u+u*u)+G4(2,1,0,2)*(2-2*u+u*u)-
     & 4*G4(2,1,1,2)*(2-2*u+u*u)+G4(2,1,2,0)*(2-2*u+u*u)+
     & G3(2,2,3)*H1(0)*(4-(32*u)/9._dp+(13*(u*u))/9._dp)-
     & (2*G4(1,0,3,0)*(80-7*u+2*(u*u)))/9._dp-
     & (2*G4(1,2,3,0)*(80-7*u+2*(u*u)))/9._dp+
     & (2*G3(1,0,3)*H1(0)*(80-7*u+2*(u*u)))/9._dp+
     & (2*G3(1,2,3)*H1(0)*(80-7*u+2*(u*u)))/9._dp-
     & (2*H4(0,0,1,0)*(40-u+2*(u*u)))/9._dp-
     & (2*H4(0,1,0,0)*(40-u+2*(u*u)))/9._dp-
     & (2*G4(2,3,0,2)*(13-8*u+4*(u*u)))/9._dp-
     & (2*G4(2,3,2,0)*(13-8*u+4*(u*u)))/9._dp+
     & (2*G3(2,3,2)*H1(0)*(13-8*u+4*(u*u)))/9._dp+
     & (G3(2,3,3)*H1(0)*(-160-4*u+5*(u*u)))/9._dp+
     & (2*G4(2,0,3,0)*(49-15*u+6*(u*u)))/9._dp-
     & (2*G3(2,0,3)*H1(0)*(49-15*u+6*(u*u)))/9._dp+
     & (2*G4(0,2,3,0)*(94-15*u+6*(u*u)))/9._dp-
     & (2*G3(0,2,3)*H1(0)*(94-15*u+6*(u*u)))/9._dp+
     & (2*G4(3,3,3,0)*(-64-9*u+6*(u*u)))/9._dp+
     & (2*G4(3,2,3,0)*(-40-17*u+7*(u*u)))/9._dp-
     & (2*G3(3,2,3)*H1(0)*(-40-17*u+7*(u*u)))/9._dp-
     & (2*G4(3,3,0,0)*(-64-23*u+10*(u*u)))/9._dp+
     & (G4(2,3,0,0)*(16-28*u+11*(u*u)))/9._dp+
     & (2*G4(0,0,3,0)*(-72-25*u+11*(u*u)))/9._dp+
     & (2*G3(3,0,3)*H1(0)*(-104-33*u+15*(u*u)))/9._dp+
     & (G4(3,0,1,2)*(10+(-22+30*NFV)*u+(11-60*NFV)*(u*u)))/9._dp+
     & (G4(3,2,1,2)*(10+(-22+30*NFV)*u+(11-60*NFV)*(u*u)))/9._dp+
     & (G4(1,1,1,2)*(112+6*(-19+5*NFV)*u+(57-60*NFV)*(u*u)))/9._dp+
     & (G4(1,1,2,2)*(-32+30*(1+NFV)*u-15*(1+4*NFV)*(u*u)))/9._dp+
     & (G4(1,2,1,2)*(-32+30*(1+NFV)*u-15*(1+4*NFV)*(u*u)))/9._dp-
     & (2*G4(0,0,1,2)*(8-3*(-4+5*NFV)*u+6*(-1+5*NFV)*(u*u)))/9._dp+
     & (10*G4(0,1,1,2)*(-2-3*(-2+NFV)*u+(-3+6*NFV)*(u*u)))/9._dp+
     & (G4(1,0,1,2)*(-20-6*(-7+5*NFV)*u+3*(-7+20*NFV)*(u*u)))/9._dp+
     & (H4(0,1,1,0)*(30+2*(2+5*NFV)*u-(1+20*NFV)*(u*u)))/3._dp+
     & (2*G4(0,1,2,2)*(8-3*(2+5*NFV)*u+(3+30*NFV)*(u*u)))/9._dp+
     & (H4(0,1,0,1)*(-10+(22-30*NFV)*u+(-11+60*NFV)*(u*u)))/9._dp+
     & (H4(1,1,0,1)*(-20+(6-30*NFV)*u+(-3+60*NFV)*(u*u)))/9._dp+
     & (G4(2,2,1,2)*(-4+(6-30*NFV)*u+(-3+60*NFV)*(u*u)))/9._dp+
     & (G4(0,2,1,2)*(8-2*(7+15*NFV)*u+(7+60*NFV)*(u*u)))/9._dp+
     & (G4(2,0,1,2)*(18-2*(7+15*NFV)*u+(7+60*NFV)*(u*u)))/9._dp+
     & (H4(1,1,1,0)*(-110+(52+30*NFV)*u-(23+60*NFV)*(u*u)))/9._dp+
     & G3(0,1,0)*(2*H1(0)*(2-2*u+u*u)+2*H1(1)*(2-2*u+u*u))+
     & G3(2,1,1)*(4*H1(0)*(2-2*u+u*u)+4*H1(1)*(2-2*u+u*u))+
     & G3(2,2,1)*((H1(0)*(4+6*(-1+5*NFV)*u+(3-60*NFV)*(u*u)))/9._dp+
     & (H1(1)*(4+6*(-1+5*NFV)*u+(3-60*NFV)*(u*u)))/9._dp)+
     & G3(1,0,1)*((H1(0)*(20+6*(-7+5*NFV)*u+(21-60*NFV)*(u*u)))/9._dp+
     & (H1(1)*(20+6*(-7+5*NFV)*u+(21-60*NFV)*(u*u)))/9._dp)+
     & G3(1,2,1)*((H1(0)*(32-30*(1+NFV)*u+15*(1+4*NFV)*(u*u)))/9._dp+
     & (H1(1)*(32-30*(1+NFV)*u+15*(1+4*NFV)*(u*u)))/9._dp)+
     & G3(0,0,1)*((2*H1(0)*(8-3*(-4+5*NFV)*u+6*(-1+5*NFV)*(u*u)))/9._dp+
     & (2*H1(1)*(8-3*(-4+5*NFV)*u+6*(-1+5*NFV)*(u*u)))/9._dp)+
     & G3(0,1,1)*((-10*H1(0)*(-2-3*(-2+NFV)*u+(-3+6*NFV)*(u*u)))/9._dp-
     & (10*H1(1)*(-2-3*(-2+NFV)*u+(-3+6*NFV)*(u*u)))/9._dp)+
     & G3(1,1,1)*((H1(0)*(-112-6*(-19+5*NFV)*u+(-57+60*NFV)*(u*u)))/9._dp+
     & (H1(1)*(-112-6*(-19+5*NFV)*u+(-57+60*NFV)*(u*u)))/9._dp)+
     & G3(3,0,1)*((H1(0)*(-10+(22-30*NFV)*u+(-11+60*NFV)*(u*u)))/9._dp+
     & (H1(1)*(-10+(22-30*NFV)*u+(-11+60*NFV)*(u*u)))/9._dp)+
     & G3(3,2,1)*((H1(0)*(-10+(22-30*NFV)*u+(-11+60*NFV)*(u*u)))/9._dp+
     & (H1(1)*(-10+(22-30*NFV)*u+(-11+60*NFV)*(u*u)))/9._dp)+
     & G3(2,0,1)*((H1(0)*(-18+2*(7+15*NFV)*u-(7+60*NFV)*(u*u)))/9._dp+
     & (H1(1)*(-18+2*(7+15*NFV)*u-(7+60*NFV)*(u*u)))/9._dp)+
     & G3(0,2,1)*((H1(0)*(-8+2*(7+15*NFV)*u-(7+60*NFV)*(u*u)))/9._dp+
     & (H1(1)*(-8+2*(7+15*NFV)*u-(7+60*NFV)*(u*u)))/9._dp)+
     & (H2(1,1)*(-567+nf*(36-24*u)+606*u-480*NFV*u-
     & 12*zeta2*(96+(16-90*NFV)*u+(-11+180*NFV)*(u*u))))/108._dp+
     & G2(2,3)*(H2(0,0)*(-20-(4*u)/9._dp+(5*(u*u))/9._dp)-
     & (2*H2(0,1)*(13-8*u+4*(u*u)))/9._dp-
     & (2*H2(1,0)*(13-8*u+4*(u*u)))/9._dp+
     & (4*zeta2*(-13-4*u+5*(u*u)))/9._dp+
     & (H1(0)*(-912+36*nf+4*u*(28-5/omv+80/omw-1/w)+
     & (-73+34/omw+4/w-10/(omv*omv)-80/(omw*omw))*(u*u)))/18._dp
     & )+G3(2,3,0)*((-2*H1(0)*(-41-8*u+4*(u*u)))/9._dp+
     & (2*H1(1)*(13-8*u+4*(u*u)))/9._dp+
     & (912-36*nf+4*u*(-28+5/omv-80/omw+1/w)+
     & (73-34/omw-4/w+10/(omv*omv)+80/(omw*omw))*(u*u))/18._dp)+
     & G3(0,0,2)*((-2*u*(28-15*NFV-14*u+30*NFV*u)*H1(0))/9._dp+
     & (H1(1)*(-2-14*u+7*(u*u)))/9._dp+
     & (-63+(72+52/omw)*u+3*nf*(omu*omu)-
     & ((13+17*omw+18*(omw*omw))*(u*u))/(omw*omw))/9._dp)+
     & G3(2,0,0)*((H1(1)*(-2-14*u+7*(u*u)))/9._dp+
     & (H1(0)*(16-28*u+11*(u*u)))/9._dp+
     & (-63+(72+52/omw)*u+3*nf*(omu*omu)-
     & ((13+17*omw+18*(omw*omw))*(u*u))/(omw*omw))/9._dp)+
     & G3(0,2,0)*((H1(1)*(-2-14*u+7*(u*u)))/9._dp+
     & (H1(0)*(160-28*u+11*(u*u)))/9._dp+
     & (-63+(72+52/omw)*u+3*nf*(omu*omu)-
     & ((13+17*omw+18*(omw*omw))*(u*u))/(omw*omw))/9._dp)+
     & G3(0,1,2)*(-2*H1(0)*(omu*omu)-
     & (2*H1(1)*(8-3*(2+5*NFV)*u+(3+30*NFV)*(u*u)))/9._dp+
     & (141+u*(20/omv+(32-90*NFV-4*omw*(14+15*NFV)+
     & (-232+90*NFV)*(omw*omw))/(omw*omw))+
     & nf*(-6+10*u-5*(u*u))+
     & (10/(omv*omv)+(-32+90*NFV+omw*(41+60*NFV)+
     & 24*(-1+5*NFV)*(omw*omw)+77*(omw*omw*omw))/
     & (omw*omw*omw))*(u*u))/18._dp)+
     & G3(3,0,0)*((-16*H1(1)*(omu*omu))/9._dp-
     & (2*H1(0)*(-64-23*u+10*(u*u)))/9._dp+
     & (-504+u*(-72+256/omw-2/w)+
     & (23/omw+2*(9+1/w)-64/(omw*omw))*(u*u)-
     & 3*nf*(-8-2*u+u*u))/9._dp)+
     & G3(1,2,2)*((H1(0)*(-32+30*(1+NFV)*u-15*(1+4*NFV)*(u*u)))/9._dp+
     & (123-140*u+81*(u*u)-4*nf*(2-2*u+u*u)+
     & 10*NFV*(4-11*u+4*(u*u)))/18._dp)+
     & G2(2,2)*(((4-5*u)*u*H2(0,0))/9._dp+
     & (H2(1,0)*(-12+2*(2+5*NFV)*u-(1+20*NFV)*(u*u)))/3._dp+
     & (H2(0,1)*(-4+(6-30*NFV)*u+(-3+60*NFV)*(u*u)))/9._dp+
     & (H1(0)*(123-140*u+81*(u*u)-4*nf*(2-2*u+u*u)+
     & 10*NFV*(4-11*u+4*(u*u))))/18._dp+
     & (-567+nf*(36-24*u)+606*u-480*NFV*u-
     & 12*zeta2*(38+(56-90*NFV)*u+(-31+180*NFV)*(u*u)))/108._dp)+
     & G3(2,2,0)*((H1(0)*(-20+3*(u*u)))/9._dp+
     & (-60+(202+30*NFV)*u+4*nf*(omu*omu*omu)-
     & (223+30*NFV)*(u*u)+(81+40*NFV)*(u*u*u))/(18._dp*omu))+
     & G3(2,0,2)*((2*H1(0)*(-5-3*(2+5*NFV)*u+(3+30*NFV)*(u*u)))/9._dp+
     & (-60+(202+30*NFV)*u+4*nf*(omu*omu*omu)-
     & (223+30*NFV)*(u*u)+(81+40*NFV)*(u*u*u))/(18._dp*omu))+
     & G3(0,2,2)*((2*H1(0)*(8-3*(2+5*NFV)*u+(3+30*NFV)*(u*u)))/9._dp+
     & (-60+(202+30*NFV)*u+4*nf*(omu*omu*omu)-
     & (223+30*NFV)*(u*u)+(81+40*NFV)*(u*u*u))/(18._dp*omu))+
     & G2(0,0)*((-16*H2(0,1)*(omu*omu))/9._dp+
     & (H2(1,0)*(-16+28*u-11*(u*u)))/9._dp+
     & (H2(1,1)*(2+14*u-7*(u*u)))/9._dp-
     & (2*H2(0,0)*(-64-23*u+10*(u*u)))/9._dp+
     & (H1(1)*(63+(-72-52/omw)*u-3*nf*(omu*omu)+
     & ((13+17*omw+18*(omw*omw))*(u*u))/(omw*omw)))/9._dp+
     & (H1(0)*(-504+u*(-72+256/omw-2/w)+
     & (23/omw+2*(9+1/w)-64/(omw*omw))*(u*u)-
     & 3*nf*(-8-2*u+u*u)))/9._dp+
     & (omw*omw*(15*(nf*nf)+12*zeta2*(-6-50*u+31*(u*u)))+
     & 6*(-2*(-544+75*nf)*u*v+(208-51*nf-46*v+6*nf*v)*(u*u)+
     & (-46+6*nf)*(u*u*u)+3*(480-41*nf)*(v*v)))/(108._dp*(omw*omw)))
     & +G3(2,1,2)*(-2*H1(0)*(2-2*u+u*u)+
     & (-20*omv*u*(omw*omw*omw)-10*(omw*omw*omw)*(u*u)+
     & omv*omv*(omw*u*(32+41*u+30*NFV*(-3+2*u))+
     & 4*u*(-14-15*NFV-6*u+30*NFV*u)*(omw*omw)+
     & 2*(-16+45*NFV)*(u*u)-
     & omw*omw*omw*(-153+150*u+2*v-20*u*v-87*(u*u)+
     & 4*nf*(2-2*u+u*u)-10*NFV*(2-13*u+2*(u*u))+v*v)))/
     & (18._dp*(omv*omv)*(omw*omw*omw)))+
     & G2(2,1)*(2*zeta2*(2-2*u+u*u)-2*H2(0,1)*(2-2*u+u*u)+
     & 2*H2(1,0)*(2-2*u+u*u)+
     & (H1(0)*(20*omv*u*(omw*omw*omw)+10*(omw*omw*omw)*(u*u)+
     & omv*omv*(-(omw*u*(32+41*u+30*NFV*(-3+2*u)))+
     & 4*u*(14+NFV*(15-30*u)+6*u)*(omw*omw)+
     & 2*(16-45*NFV)*(u*u)+
     & omw*omw*omw*(-153+150*u+2*v-20*u*v-87*(u*u)+
     & 4*nf*(2-2*u+u*u)-10*NFV*(2-13*u+2*(u*u))+v*v))))/
     & (18._dp*(omv*omv)*(omw*omw*omw))+
     & (H1(1)*(20*omv*u*(omw*omw*omw)+10*(omw*omw*omw)*(u*u)+
     & omv*omv*(-(omw*u*(32+41*u+30*NFV*(-3+2*u)))+
     & 4*u*(14+NFV*(15-30*u)+6*u)*(omw*omw)+
     & 2*(16-45*NFV)*(u*u)+
     & omw*omw*omw*(-153+150*u+2*v-20*u*v-87*(u*u)+
     & 4*nf*(2-2*u+u*u)-10*NFV*(2-13*u+2*(u*u))+v*v))))/
     & (18._dp*(omv*omv)*(omw*omw*omw)))+
     & G2(0,1)*(2*H2(1,0)*(omu*omu)+
     & (2*zeta2*(104+75*(-2+NFV)*u-75*(-1+2*NFV)*(u*u)))/9._dp+
     & (2*H2(0,1)*(-1-3*(-4+5*NFV)*u+6*(-1+5*NFV)*(u*u)))/9._dp+
     & (2*H2(1,1)*(8-3*(2+5*NFV)*u+(3+30*NFV)*(u*u)))/9._dp+
     & (H1(0)*(-141+u*(-20/omv+
     & (-32+90*NFV+omw*(56+60*NFV)+
     & (232-90*NFV)*(omw*omw))/(omw*omw))+
     & nf*(6-10*u+5*(u*u))+
     & u*u*(-77+24/omw-10/(omv*omv)-41/(omw*omw)+
     & 32/(omw*omw*omw)-
     & (30*NFV*(3+2*v+u*(2+8*v)+4*(u*u)+4*(v*v)))/
     & (omw*omw*omw))))/18._dp+
     & (H1(1)*(-141+u*(-20/omv+
     & (-32+90*NFV+omw*(56+60*NFV)+
     & (232-90*NFV)*(omw*omw))/(omw*omw))+
     & nf*(6-10*u+5*(u*u))+
     & u*u*(-77+24/omw-10/(omv*omv)-41/(omw*omw)+
     & 32/(omw*omw*omw)-
     & (30*NFV*(3+2*v+u*(2+8*v)+4*(u*u)+4*(v*v)))/
     & (omw*omw*omw))))/18._dp)+
     & (H3(0,0,0)*(-2178-326/w-14*nf*(-8-2*u+u*u)+
     & u*(-574-20/omv+768/omw+328/w-160/(w*w))+
     & u*u*(259+48/omw-2/w-10/(omv*omv)-192/(omw*omw)+
     & 80/(w*w))+80/(w*w)))/18._dp+
     & (H2(0,1)*(nf*(54-36*u)+6*zeta2*
     & (316+4*(-64+45*NFV)*u+(134-360*NFV)*(u*u))+
     & (3*(-20*u*(omw*omw)*(w*w)+
     & omv*(omw*omw*(9*w*(-4*u-41*w+38*u*w+4*(u*u))+
     & 20*NFV*u*(2-4*u-3*w+3*u*w+2*(u*u)-8*(w*w)))+
     & 4*omw*(55+45*NFV*(-1+u)-22*u)*u*(w*w)+
     & 4*(-37+45*NFV)*(u*u)*(w*w))))/(omv*(omw*omw)*(w*w))))/
     & 108._dp+H1(1)*((H2(0,0)*(-8-2*u+u*u))/6._dp+
     & (H2(1,0)*(10-2*u+u*u))/6._dp+
     & (4*nf*(-28-57*u+18*zeta2*(1-4*u+2*(u*u)))+
     & 3*(586+1215*u-(276*u)/omw+5892*zeta3-1680*u*zeta3+
     & 768*zeta3*(u*u)+6*zeta2*
     & (-336+u*(291-20/omv+464/omw-46/w+96/(omw*omw))+
     & u*u*(-61-140/omw+84/w-10/(omv*omv)-35/(omw*omw)-
     & 96/(omw*omw*omw)-9/(w*w))+
     & (2*(-10+19*v)*(u*u*u))/(w*w)+(29*(u*u*u*u))/(w*w))+
     & 10*NFV*(27+3*u*(-12+9/omw+2/w+
     & 6*zeta2*(-2+8/omu-6/omw-1/w-9/(omw*omw)))+
     & 3*(u*u)*(-2/w+6*zeta2*
     & (-8+12/omw+2/w+6/(omw*omw)+9/(omw*omw*omw)-
     & 1/(w*w)))-(6*(-1+6*v)*zeta2*(u*u*u*u))/(w*w*w)-
     & (12*zeta2*(u*u*u*u*u))/(w*w*w)-
     & (6*zeta2*(u*u*u)*(-1-3*v+6*(v*v)))/(w*w*w))))/324._dp)+
     & G2(0,3)*((H2(0,1)*(8+2*u-u*u))/9._dp+(H2(1,0)*(8+2*u-u*u))/9._dp+
     & (22*zeta2*(-8-2*u+u*u))/9._dp+(16*H2(0,0)*(-8-2*u+u*u))/9._dp+
     & (H1(0)*(624+u*(20/omv-800/omw-(2*(19+15*NFV-59*w))/w)+
     & nf*(-8-2*u+u*u)+
     & u*u*(-64/omw+10/(omv*omv)+200/(omw*omw)+
     & (-9+30*NFV-60*NFV*v+76*w-67*(w*w))/(w*w))-
     & ((-49+67*v+10*NFV*(-7+6*v))*(u*u*u*u))/(w*w*w)-
     & ((29+20*NFV)*(u*u*u*u*u))/(w*w*w)-
     & (2*(u*u*u)*(10-29*v+19*(v*v)+5*NFV*(5-9*v+6*(v*v))))/
     & (w*w*w)))/18._dp)+G3(0,3,0)*
     & ((-8*H1(0)*(-8-2*u+u*u))/9._dp+(H1(1)*(-8-2*u+u*u))/9._dp+
     & (-624+u*(-20/omv+800/omw+(38+30*NFV-118*w)/w)+
     & nf*(8+2*u-u*u)+u*u*
     & (64/omw-10/(omv*omv)-200/(omw*omw)+
     & (9-30*NFV+60*NFV*v-76*w+67*(w*w))/(w*w))+
     & ((-49+67*v+10*NFV*(-7+6*v))*(u*u*u*u))/(w*w*w)+
     & ((29+20*NFV)*(u*u*u*u*u))/(w*w*w)+
     & (2*(u*u*u)*(10-29*v+19*(v*v)+5*NFV*(5-9*v+6*(v*v))))/
     & (w*w*w))/18._dp)+(H3(0,1,0)*
     & (1500-326/w-6*nf*(10-2*u+u*u)+80/(w*w)+
     & u*(-20/omv-360/omw-
     & (2*(80+5*(-29+3*NFV*v)*w+67*(w*w)))/(w*w))+
     & u*u*(4/omw-10/(omv*omv)+90/(omw*omw)+
     & (71-30*NFV*v+74*w+74*(w*w))/(w*w))-
     & ((-49+67*v+20*NFV*(-2+3*v))*(u*u*u*u))/(w*w*w)-
     & ((29+20*NFV)*(u*u*u*u*u))/(w*w*w)-
     & (2*(u*u*u)*(10-29*v+19*(v*v)+10*NFV*(1-3*v+3*(v*v))))/
     & (w*w*w)))/18._dp+(H3(1,0,0)*
     & (2892+u*(20/omv-360/omw-(38+30*NFV+454*w)/w)-
     & 12*nf*(13-2*u+u*u)+
     & u*u*(4/omw+10/(omv*omv)+90/(omw*omw)+
     & (-9+30*NFV-60*NFV*v+76*w+208*(w*w))/(w*w))-
     & ((-49+67*v+10*NFV*(-7+6*v))*(u*u*u*u))/(w*w*w)-
     & ((29+20*NFV)*(u*u*u*u*u))/(w*w*w)-
     & (2*(u*u*u)*(10-29*v+19*(v*v)+5*NFV*(5-9*v+6*(v*v))))/(w*w*w)
     & ))/18._dp+(H3(1,0,1)*(-30+
     & u*(52+20/omv+56/omw-42/w+
     & 30*NFV*(1+2/omw-1/w+3/(omw*omw))-32/(omw*omw))+
     & u*u*(-35+24/omw+80/w+10/(omv*omv)-41/(omw*omw)+
     & 32/(omw*omw*omw)+
     & 30*NFV*(-4/omw-2/(omw*omw)-3/(omw*omw*omw)+
     & (-1+2*w)/(w*w))-9/(w*w))-
     & ((-49+67*v+10*NFV*(-1+6*v))*(u*u*u*u))/(w*w*w)-
     & ((29+20*NFV)*(u*u*u*u*u))/(w*w*w)-
     & (2*(u*u*u)*(10-29*v+19*(v*v)+5*NFV*(-1-3*v+6*(v*v))))/
     & (w*w*w)))/18._dp+G3(3,2,0)*
     & ((H1(0)*(-88-3*(u*u)))/9._dp+
     & (-6-2*(2+15*NFV)*v+(-30+NFV*(40-60*v)+38*v)*(u*u*u*u)+
     & (6-20*NFV)*(u*u*u*u*u)+(34+30*NFV)*(v*v)+
     & u*u*u*(60-132*v+77*(v*v)-20*NFV*(1-3*v+3*(v*v)))-
     & 4*(6+5*NFV)*(v*v*v)+
     & u*u*(-60+146*v-173*(v*v)+(63-20*NFV)*(v*v*v))+v*v*v*v+
     & u*(30+6*(-8+5*NFV)*v+62*(v*v)-10*(7+2*NFV)*(v*v*v)+
     & 17*(v*v*v*v))-v*v*v*v*v)/(18._dp*(w*w*w)))+
     & G3(3,0,2)*((H1(0)*(2+6*(2+5*NFV)*u-6*(1+10*NFV)*(u*u)))/9._dp+
     & (-6-2*(2+15*NFV)*v+(-30+NFV*(40-60*v)+38*v)*(u*u*u*u)+
     & (6-20*NFV)*(u*u*u*u*u)+(34+30*NFV)*(v*v)+
     & u*u*u*(60-132*v+77*(v*v)-20*NFV*(1-3*v+3*(v*v)))-
     & 4*(6+5*NFV)*(v*v*v)+
     & u*u*(-60+146*v-173*(v*v)+(63-20*NFV)*(v*v*v))+v*v*v*v+
     & u*(30+6*(-8+5*NFV)*v+62*(v*v)-10*(7+2*NFV)*(v*v*v)+
     & 17*(v*v*v*v))-v*v*v*v*v)/(18._dp*(w*w*w)))+
     & G2(3,2)*((2*H2(0,0)*(40-u+2*(u*u)))/9._dp+
     & (H2(0,1)*(10+(-22+30*NFV)*u+(11-60*NFV)*(u*u)))/9._dp+
     & (H2(1,0)*(-30-2*(2+5*NFV)*u+(1+20*NFV)*(u*u)))/3._dp+
     & (zeta2*(-158+(128-90*NFV)*u+(-67+180*NFV)*(u*u)))/9._dp+
     & (H1(0)*(6+(4+30*NFV)*v+(30-38*v+20*NFV*(-2+3*v))*(u*u*u*u)+
     & (-6+20*NFV)*(u*u*u*u*u)-2*(17+15*NFV)*(v*v)+
     & u*u*u*(-60+132*v-77*(v*v)+20*NFV*(1-3*v+3*(v*v)))+
     & 4*(6+5*NFV)*(v*v*v)+
     & u*u*(60-146*v+173*(v*v)+(-63+20*NFV)*(v*v*v))+
     & u*(-30-6*(-8+5*NFV)*v-62*(v*v)+10*(7+2*NFV)*(v*v*v)-
     & 17*(v*v*v*v))-v*v*v*v+v*v*v*v*v))/(18._dp*(w*w*w)))-
     & (H3(0,0,1)*(10*(3-5*v+2*NFV*(-2+5*v))*(u*u*u*u*u*u)+
     & (-6+20*NFV)*(u*u*u*u*u*u*u)-24*(omv*omv*omv)*(v*v)+
     & 2*u*v*(omv*omv)*(-4+53*v+(8+15*NFV)*(v*v))+
     & u*u*u*u*u*(-60+176*v-149*(v*v)+20*NFV*(1-7*v+10*(v*v)))+
     & u*u*u*u*(60+2*(-99+5*NFV)*v+(373-130*NFV)*(v*v)+
     & 9*(-23+20*NFV)*(v*v*v))+
     & omv*(u*u)*(6+26*v+2*(-53+30*NFV)*(v*v)-
     & 18*(6+5*NFV)*(v*v*v)+35*(v*v*v*v))+
     & u*u*u*(-30+30*(2+NFV)*v-10*(19+10*NFV)*(v*v)+
     & 6*(59+5*NFV)*(v*v*v)+(-137+60*NFV)*(v*v*v*v))))/
     & (18._dp*(omw*omw)*(w*w*w))+
     & (H3(1,1,0)*(omw*omw*omw*(10*NFV*u*
     & (6*u*w*(-1+2*w)+4*(-1+3*v)*(u*u*u)+8*(u*u*u*u)+
     & 3*(-2+w)*(w*w)-2*(u*u)*(2-9*w+6*(w*w)))+
     & w*(-2*u*w*(44+25*w)+(36-76*w)*(u*u*u)-18*(u*u*u*u)-
     & 4*(45+nf*(omu*omu))*(w*w)+u*u*(-18+164*w+23*(w*w))))
     & +omw*u*(32-39*u+30*NFV*(-3+2*u))*(w*w*w)+
     & 2*u*(132+5*u+30*NFV*(-1+2*u))*(omw*omw)*(w*w*w)+
     & 2*(-16+45*NFV)*(u*u)*(w*w*w)))/(18._dp*(omw*omw*omw)*(w*w*w))+
     & G3(1,1,2)*(4*H1(0)*(2-2*u+u*u)+
     & (H1(1)*(32-30*(1+NFV)*u+15*(1+4*NFV)*(u*u)))/9._dp+
     & (-159+453*v+(-355+97*v)*(u*u*u*u)+39*(u*u*u*u*u)-420*(v*v)+
     & u*u*u*(990-864*v+68*(v*v))+126*(v*v*v)+
     & 10*NFV*(-2+3*v+(2+6*v)*(u*u*u*u)+2*(u*u*u*u*u)-3*(v*v)+
     & 2*(u*u*u)*(-8+6*v+3*(v*v))+u*u*(16-39*v+21*(v*v))+
     & u*(-2+18*v-24*(v*v)+9*(v*v*v)))+8*nf*(w*w*w)+
     & u*(715-1576*v+1046*(v*v)-194*(v*v*v)-8*nf*(w*w*w))+
     & 2*(u*u)*(-615+945*v-347*(v*v)+5*(v*v*v)+2*nf*(w*w*w)))/
     & (18._dp*(w*w*w)))+G2(1,0)*((16*H2(1,1)*(omu*omu))/9._dp+
     & (2*H2(0,0)*(80-5*u+u*u))/9._dp+(H2(0,1)*(-2-14*u+7*(u*u)))/9._dp-
     & (4*zeta2*(-78-17*u+10*(u*u)))/9._dp+
     & (H2(1,0)*(178-28*u+11*(u*u)))/9._dp+
     & (H1(0)*(-2*(34+13*v)*(u*u*u*u)-2*(u*u*u*u*u)+
     & u*u*u*(234-130*v-55*(v*v))+
     & 10*NFV*(2-3*v+(-7+6*v)*(u*u*u*u)+2*(u*u*u*u*u)+
     & u*u*(-7+15*v-6*(v*v))+3*(v*v)+u*(-1-3*v+3*(v*v))+
     & u*u*u*(11-15*v+6*(v*v)))+
     & u*(128-338*v+253*(v*v)-34*(v*v*v)-10*nf*(w*w*w))+
     & 3*(-6+26*v-37*(v*v)+17*(v*v*v)+2*nf*(w*w*w))+
     & u*u*(-274+416*v-87*(v*v)-31*(v*v*v)+5*nf*(w*w*w))))/
     & (18._dp*(w*w*w))+(H1(1)*(-2*(34+13*v)*(u*u*u*u)-2*(u*u*u*u*u)+
     & u*u*u*(234-130*v-55*(v*v))+
     & 10*NFV*(2-3*v+(-7+6*v)*(u*u*u*u)+2*(u*u*u*u*u)+
     & u*u*(-7+15*v-6*(v*v))+3*(v*v)+u*(-1-3*v+3*(v*v))+
     & u*u*u*(11-15*v+6*(v*v)))+
     & u*(128-338*v+253*(v*v)-34*(v*v*v)-10*nf*(w*w*w))+
     & 3*(-6+26*v-37*(v*v)+17*(v*v*v)+2*nf*(w*w*w))+
     & u*u*(-274+416*v-87*(v*v)-31*(v*v*v)+5*nf*(w*w*w))))/
     & (18._dp*(w*w*w)))+G2(1,1)*(-4*H2(1,0)*(2-2*u+u*u)+
     & (H2(0,1)*(40+6*(-7+5*NFV)*u+(21-60*NFV)*(u*u)))/9._dp+
     & (H2(1,1)*(-32+30*(1+NFV)*u-15*(1+4*NFV)*(u*u)))/9._dp+
     & (zeta2*(-8-6*(-23+25*NFV)*u+(-69+300*NFV)*(u*u)))/9._dp+
     & (H1(0)*(159-453*v+(355-97*v)*(u*u*u*u)-39*(u*u*u*u*u)+
     & u*u*u*(-990+864*v-68*(v*v))+420*(v*v)-126*(v*v*v)-
     & 10*NFV*(-2+3*v+(2+6*v)*(u*u*u*u)+2*(u*u*u*u*u)-3*(v*v)+
     & 2*(u*u*u)*(-8+6*v+3*(v*v))+u*u*(16-39*v+21*(v*v))+
     & u*(-2+18*v-24*(v*v)+9*(v*v*v)))-8*nf*(w*w*w)-
     & 2*(u*u)*(-615+945*v-347*(v*v)+5*(v*v*v)+2*nf*(w*w*w))+
     & u*(-715+1576*v-1046*(v*v)+194*(v*v*v)+8*nf*(w*w*w))))/
     & (18._dp*(w*w*w))+(H1(1)*(159-453*v+(355-97*v)*(u*u*u*u)-
     & 39*(u*u*u*u*u)+u*u*u*(-990+864*v-68*(v*v))+420*(v*v)-
     & 126*(v*v*v)-10*NFV*
     & (-2+3*v+(2+6*v)*(u*u*u*u)+2*(u*u*u*u*u)-3*(v*v)+
     & 2*(u*u*u)*(-8+6*v+3*(v*v))+u*u*(16-39*v+21*(v*v))+
     & u*(-2+18*v-24*(v*v)+9*(v*v*v)))-8*nf*(w*w*w)-
     & 2*(u*u)*(-615+945*v-347*(v*v)+5*(v*v*v)+2*nf*(w*w*w))+
     & u*(-715+1576*v-1046*(v*v)+194*(v*v*v)+8*nf*(w*w*w))))/
     & (18._dp*(w*w*w)))+G3(1,2,0)*
     & ((-16*H1(1)*(omu*omu))/9._dp+(H1(0)*(-178+28*u-11*(u*u)))/9._dp+
     & ((68+26*v)*(u*u*u*u)+2*(u*u*u*u*u)+
     & u*u*u*(-234+130*v+55*(v*v))-
     & 10*NFV*(2-3*v+(-7+6*v)*(u*u*u*u)+2*(u*u*u*u*u)+
     & u*u*(-7+15*v-6*(v*v))+3*(v*v)+u*(-1-3*v+3*(v*v))+
     & u*u*u*(11-15*v+6*(v*v)))+
     & u*u*(274-416*v+87*(v*v)+31*(v*v*v)-5*nf*(w*w*w))-
     & 3*(-6+26*v-37*(v*v)+17*(v*v*v)+2*nf*(w*w*w))+
     & u*(-128+338*v-253*(v*v)+34*(v*v*v)+10*nf*(w*w*w)))/
     & (18._dp*(w*w*w)))+G3(1,0,2)*
     & ((-16*H1(1)*(omu*omu))/9._dp+
     & (2*H1(0)*(-9+(28-15*NFV)*u+2*(-7+15*NFV)*(u*u)))/9._dp+
     & ((68+26*v)*(u*u*u*u)+2*(u*u*u*u*u)+
     & u*u*u*(-234+130*v+55*(v*v))-
     & 10*NFV*(2-3*v+(-7+6*v)*(u*u*u*u)+2*(u*u*u*u*u)+
     & u*u*(-7+15*v-6*(v*v))+3*(v*v)+u*(-1-3*v+3*(v*v))+
     & u*u*u*(11-15*v+6*(v*v)))+
     & u*u*(274-416*v+87*(v*v)+31*(v*v*v)-5*nf*(w*w*w))-
     & 3*(-6+26*v-37*(v*v)+17*(v*v*v)+2*nf*(w*w*w))+
     & u*(-128+338*v-253*(v*v)+34*(v*v*v)+10*nf*(w*w*w)))/
     & (18._dp*(w*w*w)))+(40*nf*(17510-(2736*u)/omw-5004*zeta3-
     & 432*u*zeta3+216*zeta3*(u*u)+
     & 18*zeta2*(-1023+4*u*(-73+96/omw+54/w+27/(w*w))+
     & u*u*(152+24/omw-96/(omw*omw)+54/(w*w))))+
     & 3*(90*zeta4*(21293+(8584-2640*NFV)*u+32*(-157+165*NFV)*(u*u))+
     & 80*(-52255+30123*zeta3-(5868*zeta3)/w+
     & 9*zeta3*(u*u)*(-93-64/omw-12/w-10/(omv*omv)+
     & 144/(omw*omw)+160/(w*w))+(1440*zeta3)/(w*w)+
     & 18*u*((294-288*zeta3)/omw-(10*(2+zeta3))/omv+
     & (zeta3*(-160+332*w+(77+15*NFV)*(w*w)))/(w*w)))+
     & (360*zeta2*(20*u*(1+4*u)*(omw*omw)*(w*w*w)+
     & omv*(omw*omw*(w*
     & (15*w*(32+471*w)+18*(-1+4*w)*(u*u*u)+
     & u*u*(-333+186*w-1492*(w*w))+
     & 4*u*(-123-611*w+2*(246+v)*(w*w)))-
     & 10*NFV*u*(-4+37*w+4*(-3+4*v)*(u*u*u)+
     & 16*(u*u*u*u)+u*u*(-6+24*w-40*(w*w))-22*(w*w)+
     & u*(6-39*w+11*(w*w)-48*(w*w*w))-36*(w*w*w)))-
     & 6*omw*u*(312-99*u+NFV*(-90+65*u))*(w*w*w)+
     & 4*(142-135*NFV)*(u*u)*(w*w*w))))/(omv*(omw*omw)*(w*w*w))
     & ))/38880._dp+G2(3,3)*((2*H2(0,0)*(64+9*u-6*(u*u)))/9._dp+
     & (8*zeta2*(-32-15*u+6*(u*u)))/9._dp+
     & (H1(0)*(2*omw*u*(256+9*u)*(w*w*w)-128*(u*u)*(w*w*w)+
     & omw*omw*(w*(151-610*w+(627-32*nf+2*v+v*v)*(w*w)+
     & u*u*(151-6*w+2*(-7+2*nf)*(w*w))-
     & 2*u*(151-308*w+2*(-4+2*nf+5*v)*(w*w)))-
     & 10*NFV*(-2+3*w-3*(w*w)+2*(u*u)*(-1+w*w*w)+2*(w*w*w)-
     & u*(-4+3*w+w*w*w)))))/(18._dp*(omw*omw)*(w*w*w)))+
     & G3(3,3,0)*((16*H1(0)*(-8-2*u+u*u))/9._dp+
     & (-2*omw*u*(256+9*u)*(w*w*w)+128*(u*u)*(w*w*w)+
     & omw*omw*(-(w*(151-610*w+(627-32*nf+2*v+v*v)*(w*w)+
     & u*u*(151-6*w+2*(-7+2*nf)*(w*w))-
     & 2*u*(151-308*w+2*(-4+2*nf+5*v)*(w*w))))+
     & 10*NFV*(-2+3*w-3*(w*w)+2*(u*u)*(-1+w*w*w)+2*(w*w*w)-
     & u*(-4+3*w+w*w*w))))/(18._dp*(omw*omw)*(w*w*w)))+
     & (H2(0,0)*(54*v*(omv*omv)*(omw*omw)*(u*u*u)-
     & 60*u*(1+u)*(omw*omw)*(w*w*w)+
     & omv*(120*NFV*(omw*omw*omw)*(u*u*u*u)+
     & 48*omw*u*(nf*(16+u)-4*(85+3*u))*(w*w*w)+
     & 48*(97-4*nf)*(u*u)*(w*w*w)-
     & omw*omw*(54*(-1+2*v)*(u*u*u*u)+54*(u*u*u*u*u)+
     & (-480+w*(-17193+1444*nf+6768*zeta2-36*(nf*nf)))*(w*w)+
     & 2*u*w*(-738-1881*w+nf*(108+216*w-40*(w*w))+
     & (153+3*v+2562*zeta2)*(w*w))-
     & w*(u*u)*(999+930*w+(1119+2382*zeta2)*(w*w)-
     & 4*nf*(27+19*(w*w)))+
     & 30*NFV*u*(4-9*w+20*v*(u*u*u)+10*(u*u*u*u)+
     & 2*(u*u)*(-2+5*(v*v))+16*(w*w)-23*(w*w*w)+
     & u*(-6+19*w-23*(w*w)+12*(w*w*w)))))))/
     & (108._dp*omv*(omw*omw)*(w*w*w))+
     & (H2(1,0)*(-60*(omw*omw)*(u*u)*(w*w*w)+
     & omv*(120*NFV*(omw*omw*omw)*(u*u*u*u)-
     & 12*omw*u*(7*(-9+u)+15*NFV*(-3+4*u))*(w*w*w)-
     & 60*(1+9*NFV)*(u*u)*(w*w*w)+
     & omw*omw*(w*(w*(480+w*(2613-346*nf+9516*zeta2))-
     & 54*(1+w)*(u*u*u)+
     & u*u*(nf*(108-76*(w*w))+
     & 3*(-333+118*w+(373+282*zeta2)*(w*w)))+
     & 2*u*(2*nf*(54+108*w+23*(w*w))-
     & 3*(246+983*w+(119+v+342*zeta2)*(w*w))))-
     & 30*NFV*u*(-4+5*w+8*(u*u*u)-2*(w*w)+
     & 2*(u*u)*(-3+5*(w*w))-23*(w*w*w)+
     & u*(6-3*w-11*(w*w)+12*(w*w*w)))))))/
     & (108._dp*omv*(omw*omw)*(w*w*w))+
     & (-nf/3._dp+(H1(1)*(omu*omu))/2._dp+(H1(0)*(-17-2*u+u*u))/2._dp-
     & (-6*u*v+(1+v)*(u*u)+u*u*u-11*(v*v))/(2._dp*(omw*omw)))*
     & (G1(0)*G1(0)*G1(0))+(9*(G1(0)*G1(0)*G1(0)*G1(0)))/8._dp+
     & ((2*nf-3*(11+u))/54._dp-H1(1)/18._dp+(H1(0)*(-3+2*u-u*u))/18._dp)*
     & (G1(2)*G1(2)*G1(2))+(G1(2)*G1(2)*G1(2)*G1(2))/72._dp+
     & ((-17*H1(1))/18._dp+(17*(u*v*(4*nf-3*(6+v))+(3+2*nf-3*v)*(u*u)+
     & (-33+2*nf)*(v*v)))/(54._dp*(omw*omw)))*(H1(0)*H1(0)*H1(0))+
     & (289*(H1(0)*H1(0)*H1(0)*H1(0)))/72._dp+
     & H1(0)*H1(0)*((-17*H2(0,0)*(-8-2*u+u*u))/18._dp-
     & (17*H2(1,0)*(10-2*u+u*u))/18._dp+
     & (H1(1)*(u*(34-4*nf-15*v)*v+(9-2*nf-32*v)*(u*u)-17*(u*u*u)+
     & (33-2*nf)*(v*v)))/(18._dp*(omw*omw))-
     & (-2*u*v*(4*nf*(563+3*v)-3*(7703+114*v)+6*(-9+80*v)*zeta2-
     & 36*(nf*nf))-960*omv*zeta2*(u*u*u)+480*zeta2*(u*u*u*u)+
     & (28173-2444*nf+54*zeta2+36*(nf*nf))*(v*v)+
     & u*u*(-4*nf*(539+6*v)+9*(2309+76*v)+36*(nf*nf)+
     & 6*zeta2*(9-320*v+80*(v*v))))/(216._dp*(omw*omw))-
     & (5*(H1(1)*H1(1)))/12._dp)+G1(0)*G1(0)*
     & (-(G2(0,2)*(omu*omu))/2._dp-(G2(2,0)*(omu*omu))/2._dp+
     & G1(1)*(H1(0)*(-omu-(u*u)/2._dp)+H1(1)*(-omu-(u*u)/2._dp))+
     & (H1(1)*(-30+51*u+nf*(omu*omu)-30*(u*u)))/18._dp+
     & H2(1,0)*(-5+u-(u*u)/2._dp)+G1(3)*H1(0)*(4+u-(u*u)/2._dp)+
     & H2(0,0)*(4+u-(u*u)/2._dp)+(G2(3,0)*(-8-2*u+u*u))/2._dp+
     & (G2(1,2)*(2-2*u+u*u))/2._dp+
     & G1(2)*(u/2._dp+H1(1)/2._dp+(H1(0)*(3-2*u+u*u))/2._dp)-
     & (2*u*v*(-334*nf+3*(1084+6*(45+8*v)*zeta2)+5*(nf*nf))+
     & 4*(-30+nf+72*omv*zeta2)*(u*u*u)-144*zeta2*(u*u*u*u)+
     & u*u*(nf*(-318+4*v)+5*(nf*nf)+
     & 3*(948-40*v+6*zeta2*(45+32*v-8*(v*v))))+
     & (-366*nf+45*(92+18*zeta2)+5*(nf*nf))*(v*v))/(72._dp*(omw*omw))
     & +H1(0)*(H1(1)*(-3._dp/2._dp+2*u-u*u)+
     & (u*v*(318-2*nf*(-10+v)+51*v)-
     & 2*(-39+omv*nf+30*v)*(u*u*u)+(-30+nf)*(u*u*u*u)+
     & (-57+10*nf)*(v*v)+
     & u*u*(267+129*v-30*(v*v)+nf*(10-4*v+v*v)))/
     & (18._dp*(omw*omw)))-(G1(2)*G1(2))/4._dp+
     & (67._dp/4._dp+2*u-u*u)*(H1(0)*H1(0))-(H1(1)*H1(1))/4._dp)+
     & G1(2)*G1(2)*(((33-2*nf+3*u)*H1(1))/18._dp+(G2(0,2)*(omu*omu))/18._dp+
     & (G2(2,0)*(omu*omu))/18._dp+
     & (987-76*nf+36*u+(48*u)/omw+6*zeta2*(45+16*u-8*(u*u)))/
     & 216._dp+(G2(1,2)*(-2+2*u-u*u))/18._dp+(G2(3,0)*(8+2*u-u*u))/18._dp+
     & (G1(3)*H1(0)*(-8-2*u+u*u))/18._dp+(H2(0,0)*(-8-2*u+u*u))/18._dp+
     & (H2(1,0)*(10-2*u+u*u))/18._dp+
     & H1(0)*((H1(1)*(7-4*u+2*(u*u)))/18._dp+
     & (u*(62-4*nf-3*v)*v+(-4+6*v)*(u*u*u)+3*(u*u*u*u)+
     & (39-2*nf)*(v*v)+u*u*(27-2*nf-7*v+3*(v*v)))/
     & (18._dp*(omw*omw)))+((-7-8*u+4*(u*u))*(H1(0)*H1(0)))/36._dp+
     & (H1(1)*H1(1))/12._dp)+((987-76*nf+36*u+(48*u)/omw+
     & 6*zeta2*(45+16*u-8*(u*u)))/216._dp+
     & (H2(0,0)*(-8-2*u+u*u))/18._dp+(H2(1,0)*(10-2*u+u*u))/18._dp)*
     & (H1(1)*H1(1))+G2(2,0)*((H2(1,0)*(-52-3*(u*u)))/9._dp+
     & (H2(0,1)*(28-2*u+u*u))/9._dp-(2*H2(0,0)*(39-13*u+5*(u*u)))/9._dp-
     & (H1(1)*(-63+(211+30*NFV)*u+4*nf*(omu*omu*omu)-
     & 2*(116+15*NFV)*(u*u)+(84+40*NFV)*(u*u*u)))/(18._dp*omu)+
     & H1(0)*((H1(1)*(omu*omu))/9._dp+
     & (1026-286*u-(392*u)/omw-2*v+20*u*v+(4*u)/w+
     & nf*(-46+18*u-9*(u*u))+123*(u*u)+(2*(u*u))/omw-
     & (4*(u*u))/w+(98*(u*u))/(omw*omw)+
     & 10*NFV*(2-4/omu-u+2*(u*u))-v*v)/18._dp)+
     & (nf*(-31+122*u-76*(u*u))+6*zeta2*(-147-166*u+77*(u*u))+
     & 3*(50+u*u*(373-20/omv-120*NFV-(6*(23+5*NFV))/omw+
     & 36/w+(40*NFV)/w-84/(omw*omw)-(40*NFV)/(w*w))+
     & (2*(-9-30*NFV+9*v+50*NFV*v)*(u*u*u))/(w*w)+
     & (6*(3+10*NFV)*(u*u*u*u))/(w*w)+
     & 2*u*(86/omw+(-(w*(9+(242+v)*w))+
     & 5*NFV*(4-2*w+15*(w*w)))/(w*w))))/108._dp-
     & (17*(omu*omu)*(H1(0)*H1(0)))/18._dp+(omu*omu*(H1(1)*H1(1)))/18._dp)+
     & G2(0,2)*(H2(1,0)*(18+(40*u)/9._dp-(23*(u*u))/9._dp)+
     & (H2(0,1)*(-8-2*u+u*u))/9._dp-(2*H2(0,0)*(94-33*u+15*(u*u)))/9._dp-
     & (H1(1)*(-63+(211+30*NFV)*u+4*nf*(omu*omu*omu)-
     & 2*(116+15*NFV)*(u*u)+(84+40*NFV)*(u*u*u)))/(18._dp*omu)+
     & (nf*(-31+122*u-76*(u*u))+6*zeta2*(353-86*u+37*(u*u))+
     & 3*(50+u*u*(373-20/omv-120*NFV-(6*(23+5*NFV))/omw+
     & 36/w+(40*NFV)/w-84/(omw*omw)-(40*NFV)/(w*w))+
     & (2*(-9-30*NFV+9*v+50*NFV*v)*(u*u*u))/(w*w)+
     & (6*(3+10*NFV)*(u*u*u*u))/(w*w)+
     & 2*u*(86/omw+(-(w*(9+(242+v)*w))+
     & 5*NFV*(4-2*w+15*(w*w)))/(w*w))))/108._dp+
     & H1(0)*((H1(1)*(omu*omu))/9._dp+
     & (231+u*(-362-(4*(32+15*NFV))/omw+
     & NFV*(100-40/omu+30/w)+42/w+(32-90*NFV)/(omw*omw))
     & +nf*(-10+18*u-9*(u*u))+
     & u*u*((12+120*NFV)/omw+(59+60*NFV)/(omw*omw)+
     & (-32+90*NFV)/(omw*omw*omw)+
     & (9-80*w+152*(w*w)+10*NFV*(3-6*w+4*(w*w)))/(w*w))+
     & ((-49+67*v+10*NFV*(-1+6*v))*(u*u*u*u))/(w*w*w)+
     & ((29+20*NFV)*(u*u*u*u*u))/(w*w*w)+
     & (2*(u*u*u)*(10-29*v+19*(v*v)+5*NFV*(-1-3*v+6*(v*v))))/
     & (w*w*w))/18._dp)-(17*(omu*omu)*(H1(0)*H1(0)))/18._dp+
     & (omu*omu*(H1(1)*H1(1)))/18._dp)+
     & G2(1,2)*((2*H2(0,0)*(98-25*u+11*(u*u)))/9._dp+
     & (2*H2(1,0)*(-64-29*u+16*(u*u)))/9._dp+
     & (H1(1)*(-129+146*u-84*(u*u)+4*nf*(2-2*u+u*u)-
     & 10*NFV*(4-11*u+4*(u*u))))/18._dp+
     & H1(0)*((H1(1)*(-2+2*u-u*u))/9._dp+
     & (-228+268*u-85*(u*u)+8*nf*(2-2*u+u*u)-
     & 20*NFV*(2-u+2*(u*u)))/18._dp)+
     & (60*(-1+u)*(omw*omw)*(u*u)*(w*w)-
     & omv*(-6*omw*(-1+u)*u*(16+25*u+15*NFV*(-3+7*u))*(w*w)-
     & 12*(-16+45*NFV)*(-1+u)*(u*u)*(w*w)+
     & omw*omw*(w*(-42*w+
     & u*(54+w*(819-116*nf+6*v+3132*zeta2))-
     & 2*(27+w*(828-64*nf+3*v+906*zeta2))*(u*u)+
     & (54+w*(1119-76*nf+870*zeta2))*(u*u*u)-54*(u*u*u*u))
     & +30*NFV*(6*(1-2*w)*w*(u*u*u)+2*(-3+5*v)*(u*u*u*u)+
     & 6*(u*u*u*u*u)+9*(w*w)+u*(-4+4*w+6*(w*w))+
     & u*u*(4+15*(w*w))))))/(108._dp*omv*u*(omw*omw)*(w*w))+
     & (17*(2-2*u+u*u)*(H1(0)*H1(0)))/18._dp+
     & ((-2+2*u-u*u)*(H1(1)*H1(1)))/18._dp)+
     & G2(3,0)*((H2(1,0)*(88+3*(u*u)))/9._dp+
     & (2*H2(0,0)*(-96-31*u+14*(u*u)))/9._dp+
     & (H2(0,1)*(8-34*u+17*(u*u)))/9._dp+
     & H1(0)*((H1(1)*(8+2*u-u*u))/9._dp+
     & (5*nf*(-8-2*u+u*u)+
     & 2*(261+163/w+u*u*
     & (-18-16/omw+1/w+64/(omw*omw)-40/(w*w))+
     & u*(72-256/omw-164/w+80/(w*w))-40/(w*w)))/18._dp)+
     & (H1(1)*(30+(-68+30*NFV)*v+
     & (15-29*v+20*NFV*(-2+3*v))*(u*u*u*u)+
     & (-3+20*NFV)*(u*u*u*u*u)+(38-30*NFV)*(v*v)+
     & u*u*u*(-57+96*v-68*(v*v)+20*NFV*(1-3*v+3*(v*v)))+
     & 20*NFV*(v*v*v)+u*u*
     & (111-173*v+146*(v*v)+20*(-3+NFV)*(v*v*v))+
     & u*(-96-6*(-29+5*NFV)*v-116*(v*v)+4*(16+5*NFV)*(v*v*v)-
     & 17*(v*v*v*v))-v*v*v*v+v*v*v*v*v))/(18._dp*(w*w*w))+
     & (60*u*(1+u)*(omw*omw)*(w*w*w)+
     & omv*(6*(10*NFV*(-2+3*u+5*v)-9*w)*(omw*omw*omw)*(u*u*u)+
     & 12*omw*u*(-244-(37+15*NFV)*u+2*nf*(16+u))*(w*w*w)+
     & 48*(5-2*nf)*(u*u)*(w*w*w)+
     & omw*omw*(w*(8*w*(-60+w*(870-76*nf+198*zeta2))+
     & 2*u*(-738-1935*w+nf*(108+216*w-76*(w*w))+
     & (717+3*v+1578*zeta2)*(w*w))-
     & u*u*(999+822*w+(1119+1542*zeta2)*(w*w)-
     & 4*nf*(27+19*(w*w))))+
     & 30*NFV*u*(4-5*w+10*(w*w)-15*(w*w*w)+
     & u*(-6+11*w-17*(w*w)+12*(w*w*w))))))/
     & (108._dp*omv*(omw*omw)*(w*w*w))+
     & (17*(-8-2*u+u*u)*(H1(0)*H1(0)))/18._dp+
     & ((8+2*u-u*u)*(H1(1)*H1(1)))/18._dp)+
     & G1(3)*(-2*(-2+u)*u*H3(0,0,1)-(8*H3(0,0,0)*(-8-2*u+u*u))/3._dp-
     & (2*H3(0,1,0)*(40-u+2*(u*u)))/9._dp-
     & (2*H3(1,0,0)*(40-u+2*(u*u)))/9._dp+
     & (zeta2*H1(1)*(158+2*(-64+45*NFV)*u+(67-180*NFV)*(u*u)))/9._dp+
     & (H3(1,1,0)*(30+2*(2+5*NFV)*u-(1+20*NFV)*(u*u)))/3._dp+
     & (H3(1,0,1)*(-10+(22-30*NFV)*u+(-11+60*NFV)*(u*u)))/9._dp+
     & (H2(0,0)*(-474-326/w-4*nf*(-8-2*u+u*u)+
     & u*(-186+256/omw+330/w-160/(w*w))+
     & u*u*(75+9/omw-4/w-64/(omw*omw)+80/(w*w))+80/(w*w)))/9._dp
     & +(H2(0,1)*(-6-2*(2+15*NFV)*v+
     & (-30+NFV*(40-60*v)+38*v)*(u*u*u*u)+
     & (6-20*NFV)*(u*u*u*u*u)+(34+30*NFV)*(v*v)+
     & u*u*u*(60-132*v+77*(v*v)-20*NFV*(1-3*v+3*(v*v)))-
     & 4*(6+5*NFV)*(v*v*v)+
     & u*u*(-60+146*v-173*(v*v)+(63-20*NFV)*(v*v*v))+v*v*v*v+
     & u*(30+6*(-8+5*NFV)*v+62*(v*v)-10*(7+2*NFV)*(v*v*v)+
     & 17*(v*v*v*v))-v*v*v*v*v))/(18._dp*(w*w*w))+
     & (H2(1,0)*(-6-2*(2+15*NFV)*v+
     & (-30+NFV*(40-60*v)+38*v)*(u*u*u*u)+
     & (6-20*NFV)*(u*u*u*u*u)+(34+30*NFV)*(v*v)+
     & u*u*u*(60-132*v+77*(v*v)-20*NFV*(1-3*v+3*(v*v)))-
     & 4*(6+5*NFV)*(v*v*v)+
     & u*u*(-60+146*v-173*(v*v)+(63-20*NFV)*(v*v*v))+v*v*v*v+
     & u*(30+6*(-8+5*NFV)*v+62*(v*v)-10*(7+2*NFV)*(v*v*v)+
     & 17*(v*v*v*v))-v*v*v*v*v))/(18._dp*(w*w*w))+
     & (-48*omw*u*(128+15*u)*zeta2*(w*w*w)+1536*zeta2*(u*u)*(w*w*w)+
     & omw*omw*(w*(48*zeta3*(-40-17*u+7*(u*u))*(w*w)+
     & 6*zeta2*(97-358*w+(177+64*nf-10*v-5*(v*v))*(w*w)+
     & u*u*(97-6*w-8*(-41+nf)*(w*w))+
     & 2*u*(-97+182*w+(-220+8*nf+50*v)*(w*w))))-
     & 60*NFV*zeta2*(14*(u*u)*(-1+w*w*w)+
     & 7*(-2+3*w-3*(w*w)+2*(w*w*w))+u*(28-21*w+11*(w*w*w))
     & )))/(108._dp*(omw*omw)*(w*w*w))+
     & (((-57+2*nf)*(-8-2*u+u*u))/18._dp+(H1(1)*(-8-2*u+u*u))/9._dp)*
     & (H1(0)*H1(0))-(17*(-8-2*u+u*u)*(H1(0)*H1(0)*H1(0)))/18._dp+
     & H1(0)*((H1(1)*(-8-2*u+u*u))/6._dp+
     & (-60*u*(1+u)*(omw*omw)*(w*w*w)+
     & omv*(-6*(10*NFV*(-2+3*u+5*v)-9*w)*(omw*omw*omw)*
     & (u*u*u)+12*omw*u*(244+(37+15*NFV)*u-2*nf*(16+u))*
     & (w*w*w)+48*(-5+2*nf)*(u*u)*(w*w*w)+
     & omw*omw*(w*(8*w*(60+w*(-870+76*nf-198*zeta2))+
     & u*u*(999+822*w+(1119+1734*zeta2)*(w*w)-
     & 4*nf*(27+19*(w*w)))+
     & 2*u*(738+1935*w-(717+3*v+1914*zeta2)*(w*w)+
     & 4*nf*(-27-54*w+19*(w*w))))-
     & 30*NFV*u*(4-5*w+10*(w*w)-15*(w*w*w)+
     & u*(-6+11*w-17*(w*w)+12*(w*w*w))))))/
     & (108._dp*omv*(omw*omw)*(w*w*w))+((-8-2*u+u*u)*(H1(1)*H1(1)))/18._dp
     & ))+G1(2)*((10*H3(0,0,1)*(omu*omu))/9._dp+
     & G2(0,2)*(-(omu*omu)/6._dp-(H1(0)*(omu*omu))/9._dp-
     & (H1(1)*(omu*omu))/9._dp)+
     & G2(2,0)*(-(omu*omu)/6._dp-(H1(0)*(omu*omu))/9._dp-
     & (H1(1)*(omu*omu))/9._dp)+H3(0,0,0)*(16._dp/9._dp-6*u+3*(u*u))+
     & (2*H3(0,1,0)*(-95-3*u+3*(u*u)))/9._dp+
     & (H3(1,0,0)*(20-44*u+25*(u*u)))/9._dp+
     & (H3(1,0,1)*(10+2*(-1+5*NFV)*u+(1-20*NFV)*(u*u)))/3._dp+
     & (H3(1,1,0)*(-98-2*(26+15*NFV)*u+(23+60*NFV)*(u*u)))/9._dp+
     & G2(3,0)*((-8-2*u+u*u)/6._dp+(H1(0)*(-8-2*u+u*u))/9._dp+
     & (H1(1)*(-8-2*u+u*u))/9._dp)+
     & G2(1,2)*((2-2*u+u*u)/6._dp+(H1(0)*(2-2*u+u*u))/9._dp+
     & (H1(1)*(2-2*u+u*u))/9._dp)+
     & (H2(1,0)*(omw*u*(-32+NFV*(90-60*u)+39*u)*w-
     & 2*u*(132+5*u+30*NFV*(-1+2*u))*w*(omw*omw)+
     & 2*(16-45*NFV)*w*(u*u)+
     & 2*(omw*omw*omw)*((321-14*nf-20*NFV)*w+
     & u*(2+(90-4*nf-5*NFV)*w)+
     & 2*(-1+(-21+nf-10*NFV)*w)*(u*u))))/(18._dp*w*(omw*omw*omw))
     & +H1(1)*((H2(1,0)*(-10+2*u-u*u))/9._dp+(H2(0,0)*(8+2*u-u*u))/9._dp+
     & (8*nf*(5+3*u)+6*(-70+(-107-8/omw+80*NFV)*u)+
     & 6*zeta2*(-289-4*(-4+45*NFV)*u+2*(-7+180*NFV)*(u*u)))/108._dp)
     & +(H2(0,0)*(-1320+542*u-(20*u)/omv+(360*u)/omw+2*v-
     & 20*u*v-(4*u)/w-240*(u*u)-(4*(u*u))/omw+(4*(u*u))/w-
     & (10*(u*u))/(omv*omv)-(90*(u*u))/(omw*omw)+
     & 12*nf*(5-2*u+u*u)-10*NFV*(2-u+2*(u*u))+v*v))/18._dp+
     & (-4*nf*(-28-57*u+18*zeta2*(9-4*u+2*(u*u)))+
     & 3*(-586-1215*u+(276*u)/omw-2556*zeta3+1680*u*zeta3-
     & 768*zeta3*(u*u)+10*NFV*
     & (-27+u*(36-27/omw-6/w)+(6*(u*u))/w+
     & 6*zeta2*(22-24/omu+
     & (u*(27+18*omw+7*(omw*omw)))/(omw*omw)+
     & ((-27-18*omw-36*(omw*omw)+22*(omw*omw*omw))*
     & (u*u))/(omw*omw*omw)))+
     & 6*zeta2*(828+2*v+
     & u*(-209+20/omv-464/omw-20*v+4/w-96/(omw*omw))+
     & (32+140/omw-4/w+10/(omv*omv)+35/(omw*omw)+
     & 96/(omw*omw*omw))*(u*u)+v*v)))/324._dp-
     & (H2(0,1)*(20*omv*u*(omw*omw*omw)+10*(omw*omw*omw)*(u*u)+
     & omv*omv*(-(omw*u*(32+41*u+30*NFV*(-3+2*u)))+
     & 4*u*(14+NFV*(15-30*u)+6*u)*(omw*omw)+
     & 2*(16-45*NFV)*(u*u)+
     & omw*omw*omw*(-30+10*u+2*v-20*u*v-6*(u*u)+
     & 20*NFV*(1+u+u*u)+v*v))))/
     & (18._dp*(omv*omv)*(omw*omw*omw))+
     & ((H1(1)*(11+4*u-2*(u*u)))/18._dp+
     & (u*(194-4*omv*nf-99*v)*v+(-97+4*omv*nf+114*v)*(u*u*u)+
     & (57-2*nf)*(u*u*u*u)+(81-2*nf)*(v*v)+
     & u*u*(105-196*v+57*(v*v)-2*nf*(1-4*v+v*v)))/
     & (18._dp*(omw*omw)))*(H1(0)*H1(0))+
     & G1(3)*(H1(0)*((8+2*u-u*u)/6._dp+(H1(1)*(8+2*u-u*u))/9._dp)+
     & ((8+2*u-u*u)*(H1(0)*H1(0)))/9._dp)+
     & (17*(3-2*u+u*u)*(H1(0)*H1(0)*H1(0)))/18._dp+
     & ((2*nf-3*(11+u))*(H1(1)*H1(1)))/18._dp+
     & H1(0)*((H2(1,0)*(-10+2*u-u*u))/9._dp+(H2(0,0)*(8+2*u-u*u))/9._dp+
     & (H1(1)*(8*(-14+nf)*u*v+(2-6*v)*(u*u*u)-3*(u*u*u*u)+
     & u*u*(-48+4*nf+2*v-3*(v*v))+4*(-18+nf)*(v*v)))/
     & (18._dp*(omw*omw))+(60*(omw*omw)*(u*u*u)*(w*w)+
     & omv*(12*(16-45*NFV+15*u)*(u*u)*(w*w)+
     & 6*omw*u*(-16-79*u+72*(u*u)+15*NFV*(3-4*u+u*u))*
     & (w*w)+omw*omw*
     & (w*(42*w-u*(54+w*(1239-156*nf+6*v+3810*zeta2))+
     & 2*(81+w*(477-52*nf+3*v+330*zeta2))*(u*u)+
     & (-162+w*(-1119+76*nf-150*zeta2))*(u*u*u)+
     & 54*(u*u*u*u))-
     & 30*NFV*(2*(-1+5*v)*(u*u*u*u)+6*(u*u*u*u*u)+9*(w*w)-
     & u*u*(-8+6*w+w*w)-4*(u*u*u)*(2-3*w+3*(w*w))+
     & u*(-4+4*w+6*(w*w))))))/
     & (108._dp*omv*u*(omw*omw)*(w*w))+((-5+2*u-u*u)*(H1(1)*H1(1)))/18._dp
     & )-(H1(1)*H1(1)*H1(1))/18._dp)+
     & H1(0)*(((-57+2*nf)*H2(0,0)*(-8-2*u+u*u))/18._dp+
     & ((-57+2*nf)*H2(1,0)*(10-2*u+u*u))/18._dp+
     & H1(1)*((H2(0,0)*(-8-2*u+u*u))/9._dp+(H2(1,0)*(10-2*u+u*u))/9._dp+
     & (2*u*v*(1407-162*v+nf*(-94+6*v)+6*(-63+116*v)*zeta2)+
     & 2*(-171+6*nf+696*omv*zeta2)*(u*u*u)-696*zeta2*(u*u*u*u)+
     & u*u*(1359-666*v+nf*(-94+24*v)+
     & 6*zeta2*(-63+464*v-116*(v*v)))+
     & (1527-94*nf-378*zeta2)*(v*v))/(108._dp*(omw*omw)))+
     & (-90*omw*NFV*v*(9*omu*v+6*u*(-1+6*v)*zeta2)*
     & (omv*omv*omv*omv)+
     & 120*omu*u*(-6+6*u*zeta2)*(omw*omw)*(w*w*w)+
     & 12*omu*omv*(-27*nf+10*(33+2*u*(3+6*zeta2)))*(omw*omw)*
     & (w*w*w)+omv*omv*(omu*w*(omw*omw)*
     & (w*(-4*nf*(-162+81*u-314*w+57*u*w+81*(u*u))+
     & 3*(u*(315+1215*w-816*w*zeta3)+
     & 2*(-711-6655*w+2838*w*zeta3)+
     & 3*(369+112*w*zeta3)*(u*u)))-
     & 18*zeta2*(-240+978*w+10*(-9+19*w)*(u*u*u)+
     & 45*(u*u*u*u)+4*(-429+17*nf)*(w*w)-
     & u*u*(195+374*w+(-25+8*nf)*(w*w))+
     & u*(480-794*w+(-219+16*nf)*(w*w))))+
     & 24*omu*(567-50*nf+60*zeta2)*(u*u)*(w*w*w)-
     & 3*omw*u*(10*NFV*(-3*omu*v*
     & ((-38+32*v)*(u*u)+10*(u*u*u)+
     & u*(46-87*v+34*(v*v))+
     & 2*(-9+32*v-29*(v*v)+6*(v*v*v)))+
     & 6*u*zeta2*(-3+36*v+(-45+56*v)*(u*u*u*u)+
     & 14*(u*u*u*u*u)-96*(v*v)+
     & 6*(u*u*u)*(9-23*v+14*(v*v))+81*(v*v*v)+
     & u*u*(-32+138*v-159*(v*v)+66*(v*v*v))-
     & 18*(v*v*v*v)+
     & u*(12-89*v+147*(v*v)-84*(v*v*v)+24*(v*v*v*v))))
     & +omu*(32*nf*(-35+u)+36*(364+3*u)+
     & 6*(320+333*u)*zeta2)*(w*w*w))))/
     & (324._dp*omu*(omv*omv)*(omw*omw)*(w*w*w))+
     & ((u*v*(50-4*nf+3*v)+(21-2*nf+5*v)*(u*u)+2*(u*u*u)+
     & (33-2*nf)*(v*v))*(H1(1)*H1(1)))/(18._dp*(omw*omw))+
     & (H1(1)*H1(1)*H1(1))/18._dp)+((33-2*nf+3*u)*(H1(1)*H1(1)*H1(1)))/54._dp+
     & G1(0)*(G2(0,2)*(-((-30+nf)*(omu*omu))/18._dp+H1(0)*(omu*omu))+
     & G2(2,0)*(-((-30+nf)*(omu*omu))/18._dp+H1(0)*(omu*omu))+
     & (H3(1,0,1)*(-10+2*u-u*u))/9._dp+(H3(1,1,0)*(-20+3*(u*u)))/9._dp+
     & (H3(0,1,0)*(88+3*(u*u)))/9._dp+(2*H3(1,0,0)*(84-13*u+5*(u*u)))/9._dp+
     & (2*H3(0,0,0)*(-96-31*u+14*(u*u)))/9._dp+
     & (H3(0,0,1)*(8-34*u+17*(u*u)))/9._dp+
     & G2(3,0)*(H1(0)*(8+2*u-u*u)+((-30+nf)*(-8-2*u+u*u))/18._dp)+
     & G2(1,2)*(H1(0)*(-2+2*u-u*u)+((-30+nf)*(2-2*u+u*u))/18._dp)+
     & (H2(1,1)*(-60+(202+30*NFV)*u+4*nf*(omu*omu*omu)-
     & (223+30*NFV)*(u*u)+(81+40*NFV)*(u*u*u)))/(18._dp*omu)+
     & (H1(1)*(6*zeta2*(-33+166*u-77*(u*u))+4*nf*(10-32*u+19*(u*u))+
     & 3*(-140+u*u*(-373+20/omv+120*NFV+(6*(24+5*NFV))/omw-
     & 36/w-(40*NFV)/w+60/(omw*omw)+(40*NFV)/(w*w))-
     & (2*(-9-30*NFV+9*v+50*NFV*v)*(u*u*u))/(w*w)-
     & (6*(3+10*NFV)*(u*u*u*u))/(w*w)+
     & 2*u*(272-38/omw+v+9/w-
     & (5*NFV*(4-2*w+15*(w*w)))/(w*w)))))/108._dp-
     & (720*NFV*zeta2*(omv*omv)*(omw*omw)*(u*u*u)*(w*w)+
     & omu*(270*omw*NFV*u*v*(omv*omv*omv)+
     & 120*omv*u*zeta2*(omw*omw)*(w*w)+
     & 60*zeta2*(omw*omw)*(u*u)*(w*w)+
     & omv*omv*(w*(omw*omw)*
     & (2*w*(-3474+nf*(114-204*zeta2)+2934*zeta2+
     & 486*zeta3)-
     & 2*u*(-711+6*(-2+87*w-15*NFV*w)*zeta2+
     & 4*nf*(27+6*w*zeta2)+24*w*zeta3)+
     & (1107+6*(-4+229*w)*zeta2+4*nf*(-27+6*w*zeta2)+
     & 60*w*zeta3)*(u*u))+
     & 8*(567-50*nf+60*zeta2)*(u*u)*(w*w)+
     & omw*u*(30*NFV*u*
     & (18-37*v+u*(-28+22*v)+10*(u*u)+12*(v*v))-
     & (32*nf*(-35+u)+36*(364+3*u)+6*(320+277*u)*zeta2)*
     & (w*w)))))/(108._dp*omu*(omv*omv)*(omw*omw)*(w*w))+
     & (H2(0,0)*(984+242*u+(10*u)/omv-(384*u)/omw-(40*u)/w-
     & 115*(u*u)-(31*(u*u))/omw+(78*(u*u))/w+
     & (5*(u*u))/(omv*omv)+(96*(u*u))/(omw*omw)+
     & 6*nf*(-8-2*u+u*u)-(38*(u*u*u))/w-(9*(u*u))/(w*w)+
     & (18*(u*u*u))/(w*w)-(9*(u*u*u*u))/(w*w)-
     & (10*NFV*u*v*(3*(omv*omv)+
     & u*(-3*(2-3*v+v*v)+u*(3-3*v+2*(v*v)))))/
     & (omu*(w*w*w))))/9._dp+
     & (H2(0,1)*(6*nf*(omu*omu)+
     & (20*omv*u*(omw*omw)*(w*w)+10*(omw*omw)*(u*u)*(w*w)-
     & omv*omv*(36*omw*(-2+u)*u*(w*w)+18*(u*u)*(w*w)+
     & omw*omw*(2*u*(21-122*w)*w+2*(-9+19*w)*(u*u*u)+
     & 9*(u*u*u*u)+147*(w*w)+u*u*(9-80*w+132*(w*w)))))/
     & (omv*omv*(omw*omw)*(w*w))-
     & (10*NFV*u*(3*v*(omv*omv)+6*(omv*omv)*(u*u*u)-
     & 6*omv*(u*u*u*u)+2*(u*u*u*u*u)-3*u*v*(2-3*v+v*v)+
     & u*u*(-2+9*v-9*(v*v)+4*(v*v*v))))/(omu*(w*w*w))))/18._dp-
     & (H2(1,0)*(30*NFV*u*(-1+2*v)*(omv*omv)*(omw*omw)+
     & omw*omw*(10*NFV*(u*u)*
     & (12-33*v+(-9+6*v)*(u*u*u)+2*(u*u*u*u)+27*(v*v)+
     & 3*(u*u)*(6-7*v+2*(v*v))-6*(v*v*v)+
     & 2*u*(-10+18*v-9*(v*v)+2*(v*v*v)))+
     & omu*w*(2*u*w*(23+(-191+10*nf)*w)+2*(-9+19*w)*(u*u*u)+
     & 9*(u*u*u*u)+(807-46*nf)*(w*w)+
     & u*u*(9-84*w+(179-10*nf)*(w*w))))+
     & 2*omu*omw*(-196+u)*u*(w*w*w)+98*omu*(u*u)*(w*w*w)))/
     & (18._dp*omu*(omw*omw)*(w*w*w))+
     & (-(H1(1)*(omu*omu))/18._dp+(H1(0)*(17+2*u-u*u))/18._dp+
     & (u*(16*v+u*(12+v)+u*u))/(18._dp*(omw*omw)))*(G1(2)*G1(2))+
     & ((H1(1)*(17-10*u+5*(u*u)))/6._dp+
     & (-2*u*v*(439+2*omv*nf+48*v)+
     & (-131+4*omv*nf+114*v)*(u*u*u)+(57-2*nf)*(u*u*u*u)-
     & (159+2*nf)*(v*v)-
     & u*u*(579+227*v-57*(v*v)+2*nf*(1-4*v+v*v)))/
     & (18._dp*(omw*omw)))*(H1(0)*H1(0))+
     & G1(3)*(-((-30+nf)*H1(0)*(-8-2*u+u*u))/18._dp+
     & (-8-2*u+u*u)*(H1(0)*H1(0)))+
     & G1(1)*(-((-30+nf)*H1(1)*(2-2*u+u*u))/18._dp+
     & H1(0)*(-((-30+nf)*(2-2*u+u*u))/18._dp+H1(1)*(2-2*u+u*u))+
     & (2-2*u+u*u)*(H1(0)*H1(0)))+
     & (17*(-17-2*u+u*u)*(H1(0)*H1(0)*H1(0)))/18._dp+
     & ((omw*u*(16+u)-3*(omw*omw)*((-1+u)*(-1+u))-4*(u*u))*
     & (H1(1)*H1(1)))/(18._dp*(omw*omw))+
     & G1(2)*((H1(1)*(-2*omw*u*(16+u)+3*(omw*omw)*((-1+u)*(-1+u))+
     & 8*(u*u)))/(18._dp*(omw*omw))+
     & (2*u*(42+nf*(-3+v)-30*v)*v+
     & (18-126*v+nf*(-3+4*v))*(u*u)+2*(-33+nf)*(u*u*u)-
     & 3*(-30+nf)*(v*v))/(36._dp*(omw*omw))+
     & H1(0)*((2*H1(1)*(-8-2*u+u*u))/9._dp+
     & (-2*u*(100+nf*(-2+v)-18*v)*v-
     & 2*(-17+omv*nf+27*v)*(u*u*u)+(-27+nf)*(u*u*u*u)+
     & 2*(-42+nf)*(v*v)+
     & u*u*(-108+70*v-27*(v*v)+nf*(2-4*v+v*v)))/
     & (18._dp*(omw*omw)))+((-35+16*u-8*(u*u))*(H1(0)*H1(0)))/9._dp+
     & (omu*omu*(H1(1)*H1(1)))/9._dp)+
     & H1(0)*(H2(0,0)*(-8-2*u+u*u)+H2(1,0)*(10-2*u+u*u)+
     & (H1(1)*(-2*u*v*(-97+2*omv*nf+45*v)+
     & 4*(-22+omv*nf+27*v)*(u*u*u)-2*(-27+nf)*(u*u*u*u)+
     & (81-2*nf)*(v*v)+
     & u*u*(105-178*v+54*(v*v)-2*nf*(1-4*v+v*v))))/
     & (18._dp*(omw*omw))+(60*(omw*omw)*(u*u)*(w*w*w)+
     & omv*(-120*NFV*(omw*omw*omw)*(u*u*u*u)-
     & 24*omw*u*(-242-6*u+nf*(16+u))*(w*w*w)+
     & 96*(-20+nf)*(u*u)*(w*w*w)+
     & omw*omw*(w*(54*(-1+w)*(u*u*u)+
     & (7140-680*nf+4662*zeta2)*(w*w)+
     & 2*u*(-738-2415*w+nf*(108+216*w-64*(w*w))+
     & (819+3*v+1362*zeta2)*(w*w))-
     & u*u*(999+342*w+(1119+1326*zeta2)*(w*w)-
     & 4*nf*(27+19*(w*w))))+
     & 30*NFV*u*(4-9*w+8*(w*w)+2*(u*u)*(3-6*w+5*(w*w))-
     & 15*(w*w*w)+u*(-6+11*w-7*(w*w)+12*(w*w*w))))))/
     & (108._dp*omv*(omw*omw)*(w*w*w))+((5+2*u-u*u)*(H1(1)*H1(1)))/6._dp)
     & -(omu*omu*(H1(1)*H1(1)*H1(1)))/18._dp)+
     & G1(1)*(-2*H3(0,0,1)*(2-2*u+u*u)-
     & (2*H3(0,1,0)*(-82-11*u+7*(u*u)))/9._dp-
     & (2*H3(0,0,0)*(98-25*u+11*(u*u)))/9._dp-
     & (2*H3(1,0,0)*(98-25*u+11*(u*u)))/9._dp-
     & (2*H3(1,1,0)*(-64-29*u+16*(u*u)))/9._dp+
     & (H2(1,0)*(171-191*u+71*(u*u)-5*nf*(2-2*u+u*u)+
     & 10*NFV*(2-u+2*(u*u))))/9._dp+
     & (H2(1,1)*(123-140*u+81*(u*u)-4*nf*(2-2*u+u*u)+
     & 10*NFV*(4-11*u+4*(u*u))))/18._dp+
     & (H1(1)*(-60*(-1+u)*(omw*omw)*(u*u)*(w*w)+
     & omv*(-6*omw*(-1+u)*u*(16+25*u+15*NFV*(-3+7*u))*(w*w)-
     & 12*(-16+45*NFV)*(-1+u)*(u*u)*(w*w)+
     & omw*omw*(w*(-42*w+
     & u*(54+w*(819-116*nf+6*v+3132*zeta2))-
     & 2*(27+w*(828-64*nf+3*v+906*zeta2))*(u*u)+
     & (54+w*(1119-76*nf+870*zeta2))*(u*u*u)-54*(u*u*u*u)
     & )+30*NFV*(6*(1-2*w)*w*(u*u*u)+
     & 2*(-3+5*v)*(u*u*u*u)+6*(u*u*u*u*u)+9*(w*w)+
     & u*(-4+4*w+6*(w*w))+u*u*(4+15*(w*w)))))))/
     & (108._dp*omv*u*(omw*omw)*(w*w))+
     & (H2(0,1)*((453-329*v)*(u*u*u*u)-103*(u*u*u*u*u)-
     & 2*(u*u*u)*(462-567*v+179*(v*v))-
     & 10*NFV*(-2+9*v+2*(-8+3*v)*(u*u*u*u)+2*(u*u*u*u*u)-
     & 9*(v*v)+u*u*u*(38-42*v+6*(v*v))+
     & u*(16-48*v+42*(v*v)-11*(v*v*v))+4*(v*v*v)+
     & u*u*(-38+75*v-39*(v*v)+4*(v*v*v)))-
     & 2*(u*u)*(-542+927*v-439*(v*v)+66*(v*v*v)+3*nf*(w*w*w))-
     & 3*(-61+191*v-202*(v*v)+72*(v*v*v)+4*nf*(w*w*w))+
     & u*(-693+1622*v-1126*(v*v)+188*(v*v*v)+12*nf*(w*w*w))))/
     & (18._dp*(w*w*w))+(H2(0,0)*((878-512*v)*(u*u*u*u)-164*(u*u*u*u*u)+
     & u*u*u*(-2052+2226*v-541*(v*v))-
     & 10*NFV*(-2+9*v+(-7+6*v)*(u*u*u*u)+2*(u*u*u*u*u)-
     & 9*(v*v)+u*u*u*(11-15*v+6*(v*v))+
     & u*(7-21*v+15*(v*v)-2*(v*v*v))+4*(v*v*v)+
     & u*u*(-11+21*v-12*(v*v)+4*(v*v*v)))+
     & u*u*(2528-4146*v+1787*(v*v)-193*(v*v*v)-12*nf*(w*w*w))-
     & 3*(-134+410*v-421*(v*v)+145*(v*v*v)+8*nf*(w*w*w))+
     & u*(-1592+3662*v-2509*(v*v)+430*(v*v*v)+24*nf*(w*w*w))))/
     & (18._dp*(w*w*w))+(24*zeta3*(-64-29*u+16*(u*u))*(w*w*w)+
     & 6*zeta2*(669-1815*v+(1223-863*v)*(u*u*u*u)-261*(u*u*u*u*u)+
     & u*u*u*(-2772+3198*v-979*(v*v))+1551*(v*v)-405*(v*v*v)+
     & 10*NFV*(5*(-23+12*v)*(u*u*u*u)+20*(u*u*u*u*u)+
     & 5*(u*u*u)*(49-57*v+12*(v*v))+
     & u*(67-225*v+201*(v*v)-51*(v*v*v))+
     & 4*(1+3*v-3*(v*v)+3*(v*v*v))+
     & u*u*(-221+438*v-213*(v*v)+12*(v*v*v)))-16*nf*(w*w*w)+
     & u*(-2447+5102*v-3175*(v*v)+592*(v*v*v)-8*nf*(w*w*w))+
     & u*u*(3588-5622*v+2603*(v*v)-377*(v*v*v)+4*nf*(w*w*w))))/
     & (108._dp*(w*w*w))+((H1(0)*(2-2*u+u*u))/18._dp+
     & (H1(1)*(2-2*u+u*u))/18._dp)*(G1(2)*G1(2))+
     & (((-57+2*nf)*(2-2*u+u*u))/18._dp-(5*H1(1)*(2-2*u+u*u))/6._dp)*
     & (H1(0)*H1(0))-(17*(2-2*u+u*u)*(H1(0)*H1(0)*H1(0)))/18._dp+
     & ((2-2*u+u*u)*(H1(1)*H1(1)))/6._dp+
     & G1(2)*((H1(1)*(-2+2*u-u*u))/6._dp+
     & H1(0)*((-2+2*u-u*u)/6._dp-(2*H1(1)*(2-2*u+u*u))/9._dp)+
     & ((-2+2*u-u*u)*(H1(0)*H1(0)))/9._dp+((-2+2*u-u*u)*(H1(1)*H1(1)))/9._dp
     & )+H1(0)*(((-27+nf)*H1(1)*(2-2*u+u*u))/9._dp+
     & (-60*(-1+u)*(omw*omw)*(u*u)*(w*w)+
     & omv*(-6*omw*(-1+u)*u*(16+25*u+15*NFV*(-3+7*u))*(w*w)-
     & 12*(-16+45*NFV)*(-1+u)*(u*u)*(w*w)+
     & omw*omw*(w*(-42*w+
     & u*(54+w*(819-116*nf+6*v+2268*zeta2))-
     & 2*(27+w*(828-64*nf+3*v+474*zeta2))*(u*u)+
     & (54+w*(1119-76*nf+438*zeta2))*(u*u*u)-
     & 54*(u*u*u*u))+
     & 30*NFV*(6*(1-2*w)*w*(u*u*u)+2*(-3+5*v)*(u*u*u*u)+
     & 6*(u*u*u*u*u)+9*(w*w)+u*(-4+4*w+6*(w*w))+
     & u*u*(4+15*(w*w))))))/(108._dp*omv*u*(omw*omw)*(w*w))+
     & ((2-2*u+u*u)*(H1(1)*H1(1)))/6._dp)+
     & ((2-2*u+u*u)*(H1(1)*H1(1)*H1(1)))/18._dp)+(H1(1)*H1(1)*H1(1)*H1(1))/72._dp
      return
      end
      
      
      function Alpha_3a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-20*pi*G3(1,1,0))/9._dp-(2*pi*u*(omu+5*NFV*(-1+2*u))*H3(1,1,0))/3._dp+
     & (16*pi*G3(1,0,2)*(omu*omu))/9._dp+(16*pi*G3(1,2,0)*(omu*omu))/9._dp+
     & (16*pi*G3(1,2,2)*(omu*omu))/9._dp+(16*pi*G3(3,0,0)*(omu*omu))/9._dp+
     & (16*pi*G3(3,0,2)*(omu*omu))/9._dp+(16*pi*G3(3,2,0)*(omu*omu))/9._dp+
     & (2*pi*G3(3,0,3)*(104+33*u-15*(u*u)))/9._dp+
     & (pi*G3(0,0,2)*(2+14*u-7*(u*u)))/9._dp+
     & (pi*G3(0,2,0)*(2+14*u-7*(u*u)))/9._dp+
     & (pi*G3(0,2,2)*(2+14*u-7*(u*u)))/9._dp+
     & (pi*G3(2,0,0)*(2+14*u-7*(u*u)))/9._dp+
     & (pi*G3(2,0,2)*(2+14*u-7*(u*u)))/9._dp+
     & (pi*G3(2,2,0)*(2+14*u-7*(u*u)))/9._dp+
     & (pi*G3(2,3,3)*(160+4*u-5*(u*u)))/9._dp-
     & (pi*G3(0,3,0)*(-8-2*u+u*u))/9._dp-(pi*G3(0,3,2)*(-8-2*u+u*u))/9._dp-
     & (16*pi*G3(0,3,3)*(-8-2*u+u*u))/9._dp-2*pi*G3(0,1,0)*(2-2*u+u*u)+
     & pi*G3(2,1,0)*(2-2*u+u*u)-4*pi*G3(2,1,1)*(2-2*u+u*u)+
     & pi*G3(2,1,2)*(2-2*u+u*u)-(pi*H3(1,0,1)*(10-2*u+u*u))/9._dp-
     & (2*pi*G3(1,0,3)*(80-7*u+2*(u*u)))/9._dp-
     & (2*pi*G3(1,2,3)*(80-7*u+2*(u*u)))/9._dp+
     & (pi*G3(2,3,0)*(-10-12*u+3*(u*u)))/9._dp-
     & (2*pi*G3(2,3,2)*(13-8*u+4*(u*u)))/9._dp+
     & (2*pi*G3(2,0,3)*(49-15*u+6*(u*u)))/9._dp+
     & (2*pi*G3(0,2,3)*(94-15*u+6*(u*u)))/9._dp+
     & (2*pi*G3(3,3,3)*(-64-9*u+6*(u*u)))/9._dp+
     & (2*pi*G3(3,2,3)*(-40-17*u+7*(u*u)))/9._dp-
     & (2*pi*G3(3,3,0)*(-64-23*u+10*(u*u)))/9._dp+
     & (2*pi*H3(1,0,0)*(89-23*u+10*(u*u)))/9._dp+
     & (2*pi*G3(0,0,3)*(-72-25*u+11*(u*u)))/9._dp-
     & (pi*G3(2,2,3)*(36-32*u+13*(u*u)))/9._dp+
     & (pi*H3(0,0,1)*(8-34*u+17*(u*u)))/9._dp+
     & (2*pi*H3(0,0,0)*(-96-49*u+23*(u*u)))/9._dp+
     & (pi*G3(3,0,1)*(10+(-22+30*NFV)*u+(11-60*NFV)*(u*u)))/9._dp+
     & (pi*G3(3,2,1)*(10+(-22+30*NFV)*u+(11-60*NFV)*(u*u)))/9._dp+
     & (pi*G3(1,1,1)*(112+6*(-19+5*NFV)*u+(57-60*NFV)*(u*u)))/9._dp+
     & (pi*G3(1,1,2)*(-52+30*(1+NFV)*u-15*(1+4*NFV)*(u*u)))/9._dp+
     & (pi*G3(1,2,1)*(-32+30*(1+NFV)*u-15*(1+4*NFV)*(u*u)))/9._dp+
     & (2*pi*G3(0,1,2)*(-10-3*(-4+5*NFV)*u+6*(-1+5*NFV)*(u*u)))/9._dp-
     & (2*pi*G3(0,0,1)*(8-3*(-4+5*NFV)*u+6*(-1+5*NFV)*(u*u)))/9._dp+
     & (10*pi*G3(0,1,1)*(-2-3*(-2+NFV)*u+(-3+6*NFV)*(u*u)))/9._dp+
     & (pi*G3(1,0,1)*(-20-6*(-7+5*NFV)*u+3*(-7+20*NFV)*(u*u)))/9._dp-
     & (2*pi*H3(0,1,0)*(-49+(11-15*NFV)*u+(-7+30*NFV)*(u*u)))/9._dp+
     & (pi*G3(2,2,1)*(-4+(6-30*NFV)*u+(-3+60*NFV)*(u*u)))/9._dp+
     & (pi*G3(0,2,1)*(8-2*(7+15*NFV)*u+(7+60*NFV)*(u*u)))/9._dp+
     & (pi*G3(2,0,1)*(18-2*(7+15*NFV)*u+(7+60*NFV)*(u*u)))/9._dp+
     & (pi*H2(1,0)*(-387+252*u-(20*u)/omv+(336*u)/omw-(4*u)/w+
     & 6*nf*(omu*omu)+(32*u)/(omw*omw)+
     & 10*NFV*u*(4/omu+(u*(9+6*omw+12*(omw*omw)-
     & 4*(omw*omw*omw)))/(omw*omw*omw)-
     & ((3+omw)*(3+omw))/(omw*omw))-105*(u*u)-
     & (26*(u*u))/omw+(4*(u*u))/w-(10*(u*u))/(omv*omv)-
     & (57*(u*u))/(omw*omw)-(32*(u*u))/(omw*omw*omw)))/18._dp+
     & G2(2,3)*(12*pi*H1(0)+(2*pi*H1(1)*(13-8*u+4*(u*u)))/9._dp+
     & (pi*(912-36*nf+4*u*(-28+5/omv-80/omw+1/w)+
     & (73-34/omw-4/w+10/(omv*omv)+80/(omw*omw))*(u*u)))/18._dp)
     & +G2(0,0)*((16*pi*H1(0)*(omu*omu))/9._dp+
     & (pi*H1(1)*(-2-14*u+7*(u*u)))/9._dp+
     & (pi*(-63+(72+52/omw)*u+3*nf*(omu*omu)-
     & ((13+17*omw+18*(omw*omw))*(u*u))/(omw*omw)))/9._dp)+
     & G2(0,1)*((-2*pi*H1(0)*(-19-15*(-2+NFV)*u+15*(-1+2*NFV)*(u*u)))/9._dp-
     & (2*pi*H1(1)*(-10-3*(-4+5*NFV)*u+6*(-1+5*NFV)*(u*u)))/9._dp+
     & (pi*(141+u*(20/omv+(32-90*NFV-4*omw*(14+15*NFV)+
     & (-232+90*NFV)*(omw*omw))/(omw*omw))+
     & nf*(-6+10*u-5*(u*u))+
     & (10/(omv*omv)+(-32+90*NFV+omw*(41+60*NFV)+
     & 24*(-1+5*NFV)*(omw*omw)+77*(omw*omw*omw))/
     & (omw*omw*omw))*(u*u)))/18._dp)+
     & G2(2,0)*((pi*H1(0)*(-21-8*u+u*u))/9._dp+
     & (pi*H1(1)*(-2-14*u+7*(u*u)))/9._dp+
     & (pi*(5*nf*(omu*omu)+(2*omw*(52-17*u)*u+
     & omw*omw*(-117+148*u-48*(u*u))-26*(u*u))/(omw*omw)+
     & (10*NFV*u*(3-3*u+4*(u*u)))/omu))/18._dp)+
     & G2(0,2)*((pi*H1(0)*(159-8*u+u*u))/9._dp+
     & (pi*H1(1)*(-2-14*u+7*(u*u)))/9._dp+
     & (pi*(5*nf*(omu*omu)+(2*omw*(52-17*u)*u+
     & omw*omw*(-117+148*u-48*(u*u))-26*(u*u))/(omw*omw)+
     & (10*NFV*u*(3-3*u+4*(u*u)))/omu))/18._dp)+
     & (pi*H2(1,1)*(-60+(202+30*NFV)*u+4*nf*(omu*omu*omu)-
     & (223+30*NFV)*(u*u)+(81+40*NFV)*(u*u*u)))/(18._dp*omu)+
     & G2(2,2)*((-2*pi*H1(0)*(8+(3-15*NFV)*u+(-3+30*NFV)*(u*u)))/9._dp+
     & (pi*(-60+(202+30*NFV)*u+4*nf*(omu*omu*omu)-
     & (223+30*NFV)*(u*u)+(81+40*NFV)*(u*u*u)))/(18._dp*omu))+
     & G2(2,1)*(pi*H1(0)*(2-2*u+u*u)-pi*H1(1)*(2-2*u+u*u)-
     & (pi*(20*omv*u*(omw*omw*omw)+10*(omw*omw*omw)*(u*u)+
     & omv*omv*(-(omw*u*(32+41*u+30*NFV*(-3+2*u)))+
     & 4*u*(14+NFV*(15-30*u)+6*u)*(omw*omw)+
     & 2*(16-45*NFV)*(u*u)+
     & omw*omw*omw*(-153+150*u+2*v-20*u*v-87*(u*u)+
     & 4*nf*(2-2*u+u*u)-10*NFV*(2-13*u+2*(u*u))+v*v))))/
     & (18._dp*(omv*omv)*(omw*omw*omw)))+
     & (pi*H1(1)*(6*zeta2*(-5+110*u-49*(u*u))+4*nf*(10-32*u+19*(u*u))+
     & 3*(-140+u*u*(-373+20/omv+120*NFV+(6*(24+5*NFV))/omw-
     & 36/w-(40*NFV)/w+60/(omw*omw)+(40*NFV)/(w*w))-
     & (2*(-9-30*NFV+9*v+50*NFV*v)*(u*u*u))/(w*w)-
     & (6*(3+10*NFV)*(u*u*u*u))/(w*w)+
     & 2*u*(207-38/omw+v+9/w+(5*NFV*(-4+2*w+w*w))/(w*w)))))/
     & 108._dp+G2(0,3)*((-7*pi*H1(0)*(-8-2*u+u*u))/9._dp+
     & (pi*H1(1)*(-8-2*u+u*u))/9._dp+
     & (pi*(-624+u*(-20/omv+800/omw+(38+30*NFV-118*w)/w)+
     & nf*(8+2*u-u*u)+
     & u*u*(64/omw-10/(omv*omv)-200/(omw*omw)+
     & (9-30*NFV+60*NFV*v-76*w+67*(w*w))/(w*w))+
     & ((-49+67*v+10*NFV*(-7+6*v))*(u*u*u*u))/(w*w*w)+
     & ((29+20*NFV)*(u*u*u*u*u))/(w*w*w)+
     & (2*(u*u*u)*(10-29*v+19*(v*v)+5*NFV*(5-9*v+6*(v*v))))/
     & (w*w*w)))/18._dp)+(pi*H2(0,0)*
     & (1632+390*u+(20*u)/omv-(728*u)/omw-(38*u)/w-156*(u*u)-
     & (100*(u*u))/omw+(76*(u*u))/w+(10*(u*u))/(omv*omv)+
     & (182*(u*u))/(omw*omw)+8*nf*(-8-2*u+u*u)-(38*(u*u*u))/w-
     & (9*(u*u))/(w*w)+(18*(u*u*u))/(w*w)-(9*(u*u*u*u))/(w*w)-
     & (10*NFV*u*(3*v*(omv*omv)+6*(omv*omv)*(u*u*u)-
     & 6*omv*(u*u*u*u)+2*(u*u*u*u*u)-3*u*v*(2-3*v+v*v)+
     & u*u*(-2+9*v-9*(v*v)+4*(v*v*v))))/(omu*(w*w*w))))/18._dp+
     & (pi*H2(0,1)*(6*nf*(omu*omu)+
     & (20*omv*u*(omw*omw)*(w*w)+10*(omw*omw)*(u*u)*(w*w)-
     & omv*omv*(36*omw*(-2+u)*u*(w*w)+18*(u*u)*(w*w)+
     & omw*omw*(2*u*(21-122*w)*w+2*(-9+19*w)*(u*u*u)+
     & 9*(u*u*u*u)+147*(w*w)+u*u*(9-80*w+132*(w*w)))))/
     & (omv*omv*(omw*omw)*(w*w))-
     & (10*NFV*u*(3*v*(omv*omv)+6*(omv*omv)*(u*u*u)-
     & 6*omv*(u*u*u*u)+2*(u*u*u*u*u)-3*u*v*(2-3*v+v*v)+
     & u*u*(-2+9*v-9*(v*v)+4*(v*v*v))))/(omu*(w*w*w))))/18._dp+
     & G2(3,2)*((2*pi*H1(0)*(-49+(11-15*NFV)*u+(-7+30*NFV)*(u*u)))/9._dp-
     & (pi*(6+(4+30*NFV)*v+(30-38*v+20*NFV*(-2+3*v))*(u*u*u*u)+
     & (-6+20*NFV)*(u*u*u*u*u)-2*(17+15*NFV)*(v*v)+
     & u*u*u*(-60+132*v-77*(v*v)+20*NFV*(1-3*v+3*(v*v)))+
     & 4*(6+5*NFV)*(v*v*v)+
     & u*u*(60-146*v+173*(v*v)+(-63+20*NFV)*(v*v*v))+
     & u*(-30-6*(-8+5*NFV)*v-62*(v*v)+10*(7+2*NFV)*(v*v*v)-
     & 17*(v*v*v*v))-v*v*v*v+v*v*v*v*v))/(18._dp*(w*w*w)))+
     & (pi*(180*NFV*u*(-1+2*v)*zeta2*(omv*omv*omv*omv)*(omw*omw)+
     & 1152*omu*nf*(omv*omv*omv)*(v*v)*(w*w)-
     & 240*omu*omv*u*zeta2*(omw*omw)*(w*w*w)-
     & 120*omu*zeta2*(omw*omw)*(u*u)*(w*w*w)+
     & omv*omv*(-2*omu*u*(u*w*(2268+474*zeta2)+
     & 2*nf*((3+19*v)*(u*u)+v*(-272+487*v+19*(v*v))+
     & u*(-84+202*v+38*(v*v))))*(w*w)+
     & omw*omw*(-(omu*w*(3*w*
     & (5542*w-2324*w*zeta3+u*(474+405*w-136*w*zeta3)+
     & (369+80*w*zeta3)*(u*u))+
     & 6*zeta2*(-(u*w*(38+125*w))+(18-38*w)*(u*u*u)-
     & 9*(u*u*u*u)+648*(w*w)+u*u*(-9+76*w+117*(w*w)))))
     & +10*NFV*(6*zeta2*(u*u)*
     & (12-33*v+(-33+30*v)*(u*u*u)+10*(u*u*u*u)+
     & 27*(v*v)+u*u*(42-69*v+30*(v*v))-6*(v*v*v)+
     & 2*u*(-14+30*v-21*(v*v)+6*(v*v*v)))+
     & 3*omu*w*(u*w*(-11+12*w-24*w*zeta3)+2*(u*u*u)-
     & 9*(w*w)+u*u*(-2+5*w+48*zeta3*(w*w)))))+
     & omu*omw*u*(-270*NFV*(1+u)+12*(1199+9*u)+
     & 6*(632+175*u)*zeta2)*(w*w*w))))/
     & (108._dp*omu*(omv*omv)*(omw*omw)*(w*w*w))+
     & G2(1,2)*((-16*pi*H1(1)*(omu*omu))/9._dp-(2*pi*H1(0)*(80-5*u+u*u))/9._dp-
     & (pi*(-3+33*v+2*(-53+5*v)*(u*u*u*u)+10*(u*u*u*u*u)+
     & u*u*u*(261-208*v-19*(v*v))-66*(v*v)+
     & u*(85-254*v+214*(v*v)-36*(v*v*v))+36*(v*v*v)-
     & u*u*(247-419*v+129*(v*v)+19*(v*v*v))+
     & 10*NFV*(-2+9*v+6*(-5+3*v)*(u*u*u*u)+6*(u*u*u*u*u)-
     & 9*(v*v)+6*(u*u*u)*(10-12*v+3*(v*v))+
     & u*(22-60*v+48*(v*v)-11*(v*v*v))+4*(v*v*v)+
     & u*u*(-56+105*v-51*(v*v)+4*(v*v*v)))+
     & 4*nf*(omu*omu)*(w*w*w)))/(18._dp*(w*w*w)))+
     & G2(3,0)*((-16*pi*H1(1)*(omu*omu))/9._dp+
     & (2*pi*H1(0)*(24+31*u-14*(u*u)))/9._dp-
     & (pi*(omw*omw*(10*NFV*((-4+6*v)*(u*u*u*u)+2*(u*u*u*u*u)+
     & u*v*(-3+2*(v*v))+v*(3-3*v+2*(v*v))+
     & u*u*u*(2-6*v+6*(v*v))+2*(u*u)*(v*v*v))+
     & w*(-9+42*w-(-429+8*nf+2*v+v*v)*(w*w)+
     & u*u*(-9-2*w+(39+nf)*(w*w))-
     & 2*u*(-9+20*w+(2+nf-10*v)*(w*w))))-
     & 2*omw*u*(256+23*u)*(w*w*w)+128*(u*u)*(w*w*w)))/
     & (18._dp*(omw*omw)*(w*w*w)))+
     & G2(1,1)*((pi*H1(1)*(52-30*(1+NFV)*u+15*(1+4*NFV)*(u*u)))/9._dp+
     & (pi*H1(0)*(-20-6*(-7+5*NFV)*u+3*(-7+20*NFV)*(u*u)))/9._dp+
     & (pi*(-159+453*v+(-355+97*v)*(u*u*u*u)+39*(u*u*u*u*u)-
     & 420*(v*v)+u*u*u*(990-864*v+68*(v*v))+126*(v*v*v)+
     & 10*NFV*(-2+3*v+(2+6*v)*(u*u*u*u)+2*(u*u*u*u*u)-3*(v*v)+
     & 2*(u*u*u)*(-8+6*v+3*(v*v))+u*u*(16-39*v+21*(v*v))+
     & u*(-2+18*v-24*(v*v)+9*(v*v*v)))+8*nf*(w*w*w)+
     & u*(715-1576*v+1046*(v*v)-194*(v*v*v)-8*nf*(w*w*w))+
     & 2*(u*u)*(-615+945*v-347*(v*v)+5*(v*v*v)+2*nf*(w*w*w))))/
     & (18._dp*(w*w*w)))+G2(1,0)*((-16*pi*H1(1)*(omu*omu))/9._dp+
     & (pi*H1(0)*(2+14*u-7*(u*u)))/9._dp-
     & (pi*(-2*(34+13*v)*(u*u*u*u)-2*(u*u*u*u*u)+
     & u*u*u*(234-130*v-55*(v*v))+
     & 10*NFV*(2-3*v+(-7+6*v)*(u*u*u*u)+2*(u*u*u*u*u)+
     & u*u*(-7+15*v-6*(v*v))+3*(v*v)+u*(-1-3*v+3*(v*v))+
     & u*u*u*(11-15*v+6*(v*v)))+
     & u*(128-338*v+253*(v*v)-34*(v*v*v)-10*nf*(w*w*w))+
     & 3*(-6+26*v-37*(v*v)+17*(v*v*v)+2*nf*(w*w*w))+
     & u*u*(-274+416*v-87*(v*v)-31*(v*v*v)+5*nf*(w*w*w))))/
     & (18._dp*(w*w*w)))+G2(3,3)*((16*pi*H1(0)*(-8-2*u+u*u))/9._dp-
     & (pi*(2*omw*u*(256+9*u)*(w*w*w)-128*(u*u)*(w*w*w)+
     & omw*omw*(w*(151-610*w+(627-32*nf+2*v+v*v)*(w*w)+
     & u*u*(151-6*w+2*(-7+2*nf)*(w*w))-
     & 2*u*(151-308*w+2*(-4+2*nf+5*v)*(w*w)))-
     & 10*NFV*(-2+3*w-3*(w*w)+2*(u*u)*(-1+w*w*w)+2*(w*w*w)-
     & u*(-4+3*w+w*w*w)))))/(18._dp*(omw*omw)*(w*w*w)))+
     & (-(pi*G1(2)*(omu*omu))/2._dp+(3*pi*H1(0)*(omu*omu))/2._dp+
     & (pi*H1(1)*(omu*omu))/2._dp+(pi*G1(3)*(-8-2*u+u*u))/2._dp+
     & (pi*G1(1)*(2-2*u+u*u))/2._dp-
     & (pi*(u*v*(-132+2*omv*nf+51*v)-2*omv*(-30+nf)*(u*u*u)+
     & (-30+nf)*(u*u*u*u)+(-138+nf)*(v*v)+
     & u*u*(-30+111*v-30*(v*v)+nf*(1-4*v+v*v))))/
     & (18._dp*(omw*omw)))*(G1(0)*G1(0))-
     & (pi*(omu*omu)*(G1(0)*G1(0)*G1(0)))/2._dp+
     & (-(pi*H1(0)*(omu*omu))/6._dp-(pi*H1(1)*(omu*omu))/6._dp-
     & (pi*G1(3)*(-8-2*u+u*u))/18._dp-
     & (pi*(u*(14-5*v)*v-6*omv*(u*u*u)+3*(u*u*u*u)+15*(v*v)+
     & u*u*(3-11*v+3*(v*v))))/(18._dp*(omw*omw)))*(G1(2)*G1(2))+
     & (pi*(omu*omu)*(G1(2)*G1(2)*G1(2)))/18._dp+
     & ((5*pi*H1(1)*(omu*omu))/6._dp-
     & (pi*(u*v*(1082+115*v-4*nf*(17+v))+2*omv*(57-2*nf)*(u*u*u)+
     & (-57+2*nf)*(u*u*u*u)+(549-34*nf)*(v*v)+
     & u*u*(537+229*v-57*(v*v)+2*nf*(-17-4*v+v*v))))/
     & (18._dp*(omw*omw)))*(H1(0)*H1(0))+
     & (17*pi*(omu*omu)*(H1(0)*H1(0)*H1(0)))/18._dp-
     & (pi*(u*(14-5*v)*v-6*omv*(u*u*u)+3*(u*u*u*u)+15*(v*v)+
     & u*u*(3-11*v+3*(v*v)))*(H1(1)*H1(1)))/(18._dp*(omw*omw))+
     & H1(0)*(pi*H2(0,0)*(8+2*u-u*u)-pi*H2(1,0)*(10-2*u+u*u)-
     & (pi*H1(1)*(2*u*v*(-73+2*omv*nf+64*v)-
     & 2*(-63+2*omv*nf+54*v)*(u*u*u)+2*(-27+nf)*(u*u*u*u)+
     & (-57+2*nf)*(v*v)+
     & u*u*(-81+254*v-54*(v*v)+2*nf*(1-4*v+v*v))))/
     & (18._dp*(omw*omw))-(pi*(-60*u*(1+u)*(omw*omw)*(w*w*w)+
     & omv*(-6*(10*NFV*(-2+3*u+5*v)-9*w)*(omw*omw*omw)*
     & (u*u*u)+6*omw*(-90+90*NFV*(-1+u)-29*u)*u*(w*w*w)+
     & 540*(1+NFV)*(u*u)*(w*w*w)+
     & omw*omw*(w*(5*(84-8*nf+54*zeta2)*(w*w)+
     & 2*u*(738+2361*w-(594+3*v+570*zeta2)*(w*w)+
     & 4*nf*(-27-54*w+16*(w*w)))+
     & u*u*(999+450*w+(1119+534*zeta2)*(w*w)-
     & 4*nf*(27+19*(w*w))))-
     & 30*NFV*u*(4-13*w+14*(w*w)+w*w*w+
     & u*(-6+19*w-13*(w*w)+12*(w*w*w)))))))/
     & (108._dp*omv*(omw*omw)*(w*w*w))-(pi*(omu*omu)*(H1(1)*H1(1)))/6._dp)+
     & G1(0)*((-16*pi*H2(0,1)*(omu*omu))/9._dp+
     & G1(2)*(-((-27+nf)*pi*(omu*omu))/18._dp+(8*pi*H1(0)*(omu*omu))/9._dp-
     & (pi*H1(1)*(omu*omu))/9._dp)+(pi*H2(0,0)*(120+80*u-37*(u*u)))/9._dp+
     & (pi*H2(1,1)*(2+14*u-7*(u*u)))/9._dp-
     & (2*pi*H2(1,0)*(3-13*u+5*(u*u)))/9._dp+
     & G1(3)*(pi*H1(0)*(8+2*u-u*u)+((-30+nf)*pi*(-8-2*u+u*u))/18._dp)+
     & G1(1)*(((-30+nf)*pi*(2-2*u+u*u))/18._dp-pi*H1(0)*(2-2*u+u*u))+
     & (pi*H1(1)*(-4*nf*(omu*omu)-(10*NFV*u*(3-3*u+4*(u*u)))/omu+
     & (2*omw*u*(-52+17*u)+26*(u*u)+
     & omw*omw*(90-94*u+21*(u*u)))/(omw*omw)))/18._dp+
     & (pi*(-4*nf*(10-32*u+19*(u*u))+6*zeta2*(45-106*u+65*(u*u))+
     & 3*(140+u*u*(373-20/omv-32/omw+36/w+244/(omw*omw)+
     & 10*NFV*(-3/omw+(4*(-1+w-3*(w*w)))/(w*w)))+
     & (2*(-9-30*NFV+9*v+50*NFV*v)*(u*u*u))/(w*w)+
     & (6*(3+10*NFV)*(u*u*u*u))/(w*w)+
     & u*(-244/omw+(2*
     & (-(w*(9+(272+v)*w))+5*NFV*(4-2*w+15*(w*w))))/
     & (w*w)))))/108._dp+
     & H1(0)*((pi*H1(1)*(omu*omu))/9._dp+
     & (pi*(-342+u*(-94-20/omv+440/omw+38/w)-4*nf*(omu*omu)+
     & u*u*(12+82/omw-76/w-10/(omv*omv)-110/(omw*omw)+
     & 9/(w*w))+(2*(-9+19*w)*(u*u*u))/(w*w)+
     & (9*(u*u*u*u))/(w*w)+
     & (10*NFV*u*(3*v*(omv*omv)+6*(omv*omv)*(u*u*u)-
     & 6*omv*(u*u*u*u)+2*(u*u*u*u*u)-3*u*v*(2-3*v+v*v)+
     & u*u*(-2+9*v-9*(v*v)+4*(v*v*v))))/(omu*(w*w*w))))/18._dp
     & )+(pi*(omu*omu)*(G1(2)*G1(2)))/18._dp-
     & (pi*(179-34*u+17*(u*u))*(H1(0)*H1(0)))/18._dp+
     & (pi*(omu*omu)*(H1(1)*H1(1)))/18._dp)+
     & G1(2)*((pi*H2(0,1)*(28-2*u+u*u))/9._dp-
     & (2*pi*H2(0,0)*(44-23*u+10*(u*u)))/9._dp+
     & (2*pi*H2(1,0)*(-41+(3-15*NFV)*u+(-3+30*NFV)*(u*u)))/9._dp+
     & G1(3)*((pi*(-8-2*u+u*u))/6._dp+(pi*H1(0)*(-8-2*u+u*u))/9._dp+
     & (pi*H1(1)*(-8-2*u+u*u))/9._dp)+
     & H1(0)*((pi*H1(1)*(omu*omu))/3._dp+
     & (pi*(858-124*u+(20*u)/omv-(368*u)/omw+(4*u)/w-
     & (32*u)/(omw*omw)+
     & 10*NFV*u*(-4/omu+
     & (u*(-9-6*omw-12*(omw*omw)+4*(omw*omw*omw)))/
     & (omw*omw*omw)+((3+omw)*(3+omw))/(omw*omw))+
     & 51*(u*u)+(24*(u*u))/omw-(4*(u*u))/w+
     & (10*(u*u))/(omv*omv)+(65*(u*u))/(omw*omw)+
     & (32*(u*u))/(omw*omw*omw)-4*nf*(9-2*u+u*u)))/18._dp)+
     & (pi*H1(1)*(-4*nf*(omu*omu)-(10*NFV*u*(3-3*u+4*(u*u)))/omu+
     & (-2*omw*u*(16+u)+8*(u*u)+omw*omw*(90-152*u+87*(u*u)))/
     & (omw*omw)))/18._dp+
     & (pi*(-4*nf*(10-32*u+19*(u*u))+6*zeta2*(-195-110*u+49*(u*u))+
     & 3*(140+u*u*(373-20/omv-120*NFV-(6*(24+5*NFV))/omw+
     & 36/w+(40*NFV)/w-60/(omw*omw)-(40*NFV)/(w*w))+
     & (2*(-9-30*NFV+9*v+50*NFV*v)*(u*u*u))/(w*w)+
     & (6*(3+10*NFV)*(u*u*u*u))/(w*w)-
     & (2*u*(-38*(w*w)+
     & omw*(w*(9+(207+v)*w)+5*NFV*(-4+2*w+w*w))))/
     & (omw*(w*w)))))/108._dp+(pi*(7-2*u+u*u)*(H1(0)*H1(0)))/6._dp+
     & (pi*(omu*omu)*(H1(1)*H1(1)))/6._dp)+
     & G1(3)*((pi*H2(0,1)*(8-34*u+17*(u*u)))/9._dp+
     & (2*pi*H2(0,0)*(-96-49*u+23*(u*u)))/9._dp-
     & (2*pi*H2(1,0)*(-49+(11-15*NFV)*u+(-7+30*NFV)*(u*u)))/9._dp+
     & H1(0)*(-(pi*H1(1)*(-8-2*u+u*u))/9._dp+
     & (pi*((157-215*v+20*NFV*(-2+5*v))*(u*u*u*u*u*u)+
     & (-39+20*NFV)*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(-75+694*v-489*(v*v)+
     & 20*NFV*(1-7*v+10*(v*v)))+
     & u*u*u*u*(-327+(-49+40*NFV)*v-5*(-241+32*NFV)*(v*v)+
     & 5*(-117+40*NFV)*(v*v*v))+
     & u*u*u*(446-2*(596+15*NFV)*v+(357+20*NFV)*(v*v)-
     & 20*(-51+2*NFV)*(v*v*v)+5*(-77+20*NFV)*(v*v*v*v))+
     & u*v*(-68+(-94+60*NFV)*v-2*(67+45*NFV)*(v*v)+
     & (338+40*NFV)*(v*v*v)+(62+20*NFV)*(v*v*v*v)-
     & 15*(v*v*v*v*v))+
     & v*v*(222+(-478+30*NFV)*v+(202-30*NFV)*(v*v)+
     & (54+20*NFV)*(v*v*v)-v*v*v*v+v*v*v*v*v)+
     & u*u*(-162+10*(83+3*NFV)*v-(1201+90*NFV)*(v*v)+
     & 5*(123+4*NFV)*(v*v*v)+5*(83+8*NFV)*(v*v*v*v)+
     & (-129+20*NFV)*(v*v*v*v*v))))/(18._dp*(omw*omw)*(w*w*w)))+
     & (pi*H1(1)*(30+(-68+30*NFV)*v+
     & (15-29*v+20*NFV*(-2+3*v))*(u*u*u*u)+
     & (-3+20*NFV)*(u*u*u*u*u)+(38-30*NFV)*(v*v)+
     & u*u*u*(-57+96*v-68*(v*v)+20*NFV*(1-3*v+3*(v*v)))+
     & 20*NFV*(v*v*v)+u*u*
     & (111-173*v+146*(v*v)+20*(-3+NFV)*(v*v*v))+
     & u*(-96-6*(-29+5*NFV)*v-116*(v*v)+4*(16+5*NFV)*(v*v*v)-
     & 17*(v*v*v*v))-v*v*v*v+v*v*v*v*v))/(18._dp*(w*w*w))-
     & (pi*(-60*u*(1+u)*(omw*omw)*(w*w*w)+
     & omv*(-6*(10*NFV*(-2+3*u+5*v)-9*w)*(omw*omw*omw)*
     & (u*u*u)+12*omw*u*(244+(37+15*NFV)*u-2*nf*(16+u))*
     & (w*w*w)+48*(-5+2*nf)*(u*u)*(w*w*w)+
     & omw*omw*(w*(8*w*(60+w*(-870+76*nf-270*zeta2))+
     & u*u*(999+822*w+(1119+966*zeta2)*(w*w)-
     & 4*nf*(27+19*(w*w)))+
     & 2*u*(738+1935*w-(717+3*v+1002*zeta2)*(w*w)+
     & 4*nf*(-27-54*w+19*(w*w))))-
     & 30*NFV*u*(4-5*w+10*(w*w)-15*(w*w*w)+
     & u*(-6+11*w-17*(w*w)+12*(w*w*w)))))))/
     & (108._dp*omv*(omw*omw)*(w*w*w))-
     & (pi*(-8-2*u+u*u)*(H1(0)*H1(0)))/18._dp-
     & (pi*(-8-2*u+u*u)*(H1(1)*H1(1)))/18._dp)+
     & G1(1)*((16*pi*H2(1,1)*(omu*omu))/9._dp+
     & (pi*H2(0,1)*(-2-14*u+7*(u*u)))/9._dp+
     & (2*pi*H2(0,0)*(98-23*u+10*(u*u)))/9._dp+
     & (pi*H2(1,0)*(178-28*u+11*(u*u)))/9._dp+
     & G1(2)*((pi*(2-2*u+u*u))/6._dp+(pi*H1(0)*(2-2*u+u*u))/9._dp+
     & (pi*H1(1)*(2-2*u+u*u))/9._dp)-
     & (pi*(-60*(-1+u)*(omw*omw)*(u*u)*(w*w)+
     & omv*(-6*omw*(-1+u)*u*(16+25*u+15*NFV*(-3+7*u))*(w*w)-
     & 12*(-16+45*NFV)*(-1+u)*(u*u)*(w*w)+
     & omw*omw*(-(w*(42*w+
     & u*(-54+w*(-819+116*nf-6*v+3300*zeta2))+
     & 2*(27+w*(828-64*nf+3*v+510*zeta2))*(u*u)+
     & (-54+w*(-1119+76*nf-582*zeta2))*(u*u*u)+
     & 54*(u*u*u*u)))+
     & 30*NFV*(6*(1-2*w)*w*(u*u*u)+2*(-3+5*v)*(u*u*u*u)+
     & 6*(u*u*u*u*u)+9*(w*w)+u*(-4+4*w+6*(w*w))+
     & u*u*(4+15*(w*w)))))))/(108._dp*omv*u*(omw*omw)*(w*w))+
     & (pi*H1(1)*(-9+51*v+(-121+19*v)*(u*u*u*u)+13*(u*u*u*u*u)-
     & 84*(v*v)-2*(u*u*u)*(-147+122*v+5*(v*v))+
     & u*(109-308*v+250*(v*v)-42*(v*v*v))+42*(v*v*v)-
     & 2*(u*u)*(143-241*v+78*(v*v)+8*(v*v*v))+
     & 10*NFV*(-2+9*v+6*(-5+3*v)*(u*u*u*u)+6*(u*u*u*u*u)-
     & 9*(v*v)+6*(u*u*u)*(10-12*v+3*(v*v))+
     & u*(22-60*v+48*(v*v)-11*(v*v*v))+4*(v*v*v)+
     & u*u*(-56+105*v-51*(v*v)+4*(v*v*v)))+
     & 4*nf*(omu*omu)*(w*w*w)))/(18._dp*(w*w*w))+
     & H1(0)*((-10*pi*H1(1)*(2-2*u+u*u))/9._dp+
     & (pi*(51+15*(-7+4*NFV)*v+
     & (109-75*v+10*NFV*(-23+12*v))*(u*u*u*u)+
     & 5*(-5+8*NFV)*(u*u*u*u*u)+(39-60*NFV)*(v*v)+
     & u*u*u*(-228+248*v-75*(v*v)+
     & 10*NFV*(49-57*v+12*(v*v)))+5*(3+8*NFV)*(v*v*v)+
     & u*u*(280-376*v+169*(v*v)-25*(v*v*v)+
     & 10*NFV*(-45+90*v-45*(v*v)+4*(v*v*v)))+
     & u*(-187+308*v-133*(v*v)+30*(v*v*v)-
     & 10*NFV*(-15+51*v-45*(v*v)+11*(v*v*v)))+
     & 4*nf*(omu*omu)*(w*w*w)))/(18._dp*(w*w*w)))-
     & (pi*(2-2*u+u*u)*(G1(2)*G1(2)))/18._dp-
     & (pi*(2-2*u+u*u)*(H1(0)*H1(0)))/18._dp-
     & (pi*(2-2*u+u*u)*(H1(1)*H1(1)))/18._dp)-
     & (pi*(omu*omu)*(H1(1)*H1(1)*H1(1)))/18._dp
      return
      end
      
      
      function Beta_3a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=1
      return
      end
      
      
      function Beta_3a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function Beta_3a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=H1(0)*(H1(1)/3._dp+(-16+(4*u)/omw+v+v/(omu*omu))/3._dp)+
     & (H1(1)*(omw+v/((-1+u)*(-1+u))))/3._dp+
     & (-288+276*u+6*omu*zeta2*(81-8*u*(-2+v)+16*v-8*(u*u)))/
     & (36._dp*omu)+G1(2)*(-H1(1)/3._dp-
     & (v+(u+v)*(omu*omu))/(3._dp*(omu*omu))+
     & (H1(0)*(-3-u*(-2+v)+2*v-u*u))/3._dp)+
     & G1(0)*((16+u-(4*u)/omw)/3._dp+(H1(1)*(-1-u*(-2+v)+2*v-u*u))/3._dp+
     & (H1(0)*(17-u*(-2+v)+2*v-u*u))/3._dp)+
     & (G2(1,2)*(-2+2*u+2*v-u*v-u*u))/3._dp+
     & (G2(3,0)*(8+2*u+2*v-u*v-u*u))/3._dp+
     & (H2(1,0)*(-2*(-5+v)+u*(-2+v)+u*u))/3._dp+
     & (G2(0,2)*(1+u*(-2+v)-2*v+u*u))/3._dp+
     & (G2(2,0)*(1+u*(-2+v)-2*v+u*u))/3._dp+
     & (G1(3)*H1(0)*(u*(-2+v)-2*(4+v)+u*u))/3._dp+
     & (H2(0,0)*(u*(-2+v)-2*(4+v)+u*u))/3._dp+
     & G1(1)*((H1(0)*(2+u*(-2+v)-2*v+u*u))/3._dp+
     & (H1(1)*(2+u*(-2+v)-2*v+u*u))/3._dp)-(3*(G1(0)*G1(0)))/2._dp+
     & (G1(2)*G1(2))/6._dp-(17*(H1(0)*H1(0)))/6._dp+(H1(1)*H1(1))/6._dp
      return
      end
      
      
      function Beta_3a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(pi*v*(-1+4/omw-1/(omu*omu)))/3._dp+
     & (pi*G1(0)*(1+u*(-2+v)-2*v+u*u))/3._dp+
     & (pi*G1(2)*(1+u*(-2+v)-2*v+u*u))/3._dp-
     & (pi*H1(0)*(1+u*(-2+v)-2*v+u*u))/3._dp-
     & (pi*H1(1)*(1+u*(-2+v)-2*v+u*u))/3._dp-
     & (pi*G1(1)*(2+u*(-2+v)-2*v+u*u))/3._dp-
     & (pi*G1(3)*(u*(-2+v)-2*(4+v)+u*u))/3._dp
      return
      end
      
      
      function Beta_3a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-20*G4(1,1,0,2))/9._dp-(20*G4(1,1,2,0))/9._dp+
     & G3(1,1,0)*((20*H1(0))/9._dp+(20*H1(1))/9._dp)-2*omw*(-2+u)*H4(0,0,0,1)+
     & G3(2,1,0)*(H1(0)*(-2+2*u+2*v-u*v-u*u)+
     & H1(1)*(-2+2*u+2*v-u*v-u*u))+
     & (H4(1,1,0,0)*(70+u*(44-25*v)+44*v-25*(u*u)))/9._dp+
     & (G4(2,2,3,0)*(-36+32*u+32*v-13*u*v-13*(u*u)))/9._dp+
     & (G4(0,0,2,2)*(2-7*u*(-2+v)+14*v-7*(u*u)))/9._dp+
     & (G4(0,2,0,2)*(2-7*u*(-2+v)+14*v-7*(u*u)))/9._dp+
     & (G4(0,2,2,0)*(2-7*u*(-2+v)+14*v-7*(u*u)))/9._dp+
     & (G4(2,0,0,2)*(2-7*u*(-2+v)+14*v-7*(u*u)))/9._dp+
     & (G4(2,0,2,0)*(2-7*u*(-2+v)+14*v-7*(u*u)))/9._dp+
     & (G4(2,2,0,0)*(2-7*u*(-2+v)+14*v-7*(u*u)))/9._dp+
     & (G4(2,3,3,0)*(u*(4-5*v)+4*(40+v)-5*(u*u)))/9._dp+
     & (2*H4(1,0,1,0)*(130+3*omv*u+3*v-3*(u*u)))/9._dp+
     & H4(1,0,0,0)*(-10-3*u*(-2+v)+6*v-3*(u*u))+
     & (G4(0,3,0,2)*(8+2*u+2*v-u*v-u*u))/9._dp+
     & (G4(0,3,2,0)*(8+2*u+2*v-u*v-u*u))/9._dp+
     & (16*G4(1,0,2,2)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (16*G4(1,2,0,2)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (16*G4(1,2,2,0)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (16*G4(3,0,0,2)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (16*G4(3,0,2,0)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (16*G4(3,2,0,0)*(1+u*(-2+v)-2*v+u*u))/9._dp-
     & (10*H4(1,0,0,1)*(1+u*(-2+v)-2*v+u*u))/9._dp-
     & 2*G4(0,1,0,2)*(2+u*(-2+v)-2*v+u*u)-
     & 2*G4(0,1,2,0)*(2+u*(-2+v)-2*v+u*u)+
     & G4(2,1,0,2)*(2+u*(-2+v)-2*v+u*u)-
     & 4*G4(2,1,1,2)*(2+u*(-2+v)-2*v+u*u)+
     & G4(2,1,2,0)*(2+u*(-2+v)-2*v+u*u)-
     & (16*G4(0,3,3,0)*(u*(-2+v)-2*(4+v)+u*u))/9._dp+
     & (G3(0,3,2)*H1(0)*(u*(-2+v)-2*(4+v)+u*u))/9._dp+
     & (16*G3(0,3,3)*H1(0)*(u*(-2+v)-2*(4+v)+u*u))/9._dp-
     & (8*H4(0,0,0,0)*(u*(-2+v)-2*(4+v)+u*u))/3._dp-
     & (2*G4(1,0,3,0)*(80-7*v+u*(-7+2*v)+2*(u*u)))/9._dp-
     & (2*G4(1,2,3,0)*(80-7*v+u*(-7+2*v)+2*(u*u)))/9._dp+
     & (2*G3(1,0,3)*H1(0)*(80-7*v+u*(-7+2*v)+2*(u*u)))/9._dp+
     & (2*G3(1,2,3)*H1(0)*(80-7*v+u*(-7+2*v)+2*(u*u)))/9._dp-
     & (2*H4(0,0,1,0)*(40-v+u*(-1+2*v)+2*(u*u)))/9._dp-
     & (2*H4(0,1,0,0)*(40-v+u*(-1+2*v)+2*(u*u)))/9._dp-
     & (2*G4(2,3,0,2)*(13+4*u*(-2+v)-8*v+4*(u*u)))/9._dp-
     & (2*G4(2,3,2,0)*(13+4*u*(-2+v)-8*v+4*(u*u)))/9._dp+
     & (2*G3(2,3,2)*H1(0)*(13+4*u*(-2+v)-8*v+4*(u*u)))/9._dp+
     & (G3(2,3,3)*H1(0)*(-4*(40+v)+u*(-4+5*v)+5*(u*u)))/9._dp+
     & (2*G4(2,0,3,0)*(49-15*v+3*u*(-5+2*v)+6*(u*u)))/9._dp-
     & (2*G3(2,0,3)*H1(0)*(49-15*v+3*u*(-5+2*v)+6*(u*u)))/9._dp+
     & (2*G4(0,2,3,0)*(94-15*v+3*u*(-5+2*v)+6*(u*u)))/9._dp-
     & (2*G3(0,2,3)*H1(0)*(94-15*v+3*u*(-5+2*v)+6*(u*u)))/9._dp+
     & (2*G4(3,3,3,0)*(-64-9*v+u*(-9+6*v)+6*(u*u)))/9._dp-
     & (2*G3(3,3,3)*H1(0)*(-64-9*v+u*(-9+6*v)+6*(u*u)))/9._dp+
     & (2*G4(3,2,3,0)*(-40-17*v+u*(-17+7*v)+7*(u*u)))/9._dp-
     & (2*G3(3,2,3)*H1(0)*(-40-17*v+u*(-17+7*v)+7*(u*u)))/9._dp-
     & (2*G4(3,3,0,0)*(-64-23*v+u*(-23+10*v)+10*(u*u)))/9._dp+
     & (G4(2,3,0,0)*(16-28*v+u*(-28+11*v)+11*(u*u)))/9._dp+
     & (2*G4(0,0,3,0)*(-72-25*v+u*(-25+11*v)+11*(u*u)))/9._dp-
     & (2*G3(0,0,3)*H1(0)*(-72-25*v+u*(-25+11*v)+11*(u*u)))/9._dp+
     & (G3(2,2,3)*H1(0)*(36-32*v+u*(-32+13*v)+13*(u*u)))/9._dp-
     & (2*G4(3,0,3,0)*(-104-33*v+3*u*(-11+5*v)+15*(u*u)))/9._dp+
     & (2*G3(3,0,3)*H1(0)*(-104-33*v+3*u*(-11+5*v)+15*(u*u)))/9._dp+
     & (G4(3,0,1,2)*(10+u*(NFV*(30-60*v)+11*(-2+v))+(-22+30*NFV)*v+
     & (11-60*NFV)*(u*u)))/9._dp+
     & (G4(3,2,1,2)*(10+u*(NFV*(30-60*v)+11*(-2+v))+(-22+30*NFV)*v+
     & (11-60*NFV)*(u*u)))/9._dp+
     & (G4(1,1,1,2)*(u*(NFV*(30-60*v)+57*(-2+v))+
     & 2*(56+3*(-19+5*NFV)*v)+(57-60*NFV)*(u*u)))/9._dp+
     & (G4(1,1,2,2)*(-32+30*(1+NFV)*v-15*u*(-2+v+NFV*(-2+4*v))-
     & 15*(1+4*NFV)*(u*u)))/9._dp+
     & (G4(1,2,1,2)*(-32+30*(1+NFV)*v-15*u*(-2+v+NFV*(-2+4*v))-
     & 15*(1+4*NFV)*(u*u)))/9._dp-
     & (2*G4(0,0,1,2)*(8-3*(-4+5*NFV)*v+3*u*(4-2*v+5*NFV*(-1+2*v))+
     & 6*(-1+5*NFV)*(u*u)))/9._dp+
     & (10*G4(0,1,1,2)*(-2-3*(-2+NFV)*v+u*(6-3*v+NFV*(-3+6*v))+
     & (-3+6*NFV)*(u*u)))/9._dp+
     & (G4(1,0,1,2)*(-20-6*(-7+5*NFV)*v+3*u*(14-10*NFV-7*v+20*NFV*v)+
     & 3*(-7+20*NFV)*(u*u)))/9._dp+
     & (H4(0,1,1,0)*(30+2*(2+5*NFV)*v-u*(-4+v+10*NFV*(-1+2*v))-
     & (1+20*NFV)*(u*u)))/3._dp+
     & (2*G4(0,1,2,2)*(8-3*(2+5*NFV)*v+3*u*(-2+v+5*NFV*(-1+2*v))+
     & (3+30*NFV)*(u*u)))/9._dp+
     & (H4(0,1,0,1)*(-10+(22-30*NFV)*v+u*(22-30*NFV-11*v+60*NFV*v)+
     & (-11+60*NFV)*(u*u)))/9._dp+
     & (H4(1,1,0,1)*(-20+(6-30*NFV)*v+u*(6-3*v+30*NFV*(-1+2*v))+
     & (-3+60*NFV)*(u*u)))/9._dp+
     & (G4(2,2,1,2)*(-4+(6-30*NFV)*v+u*(6-3*v+30*NFV*(-1+2*v))+
     & (-3+60*NFV)*(u*u)))/9._dp+
     & (G4(0,2,1,2)*(8-2*(7+15*NFV)*v+u*(7*(-2+v)+30*NFV*(-1+2*v))+
     & (7+60*NFV)*(u*u)))/9._dp+
     & (G4(2,0,1,2)*(-2*(-9+(7+15*NFV)*v)+
     & u*(7*(-2+v)+30*NFV*(-1+2*v))+(7+60*NFV)*(u*u)))/9._dp+
     & (H4(1,1,1,0)*(-110+u*(52+NFV*(30-60*v)-23*v)+(52+30*NFV)*v-
     & (23+60*NFV)*(u*u)))/9._dp+
     & G3(0,1,0)*(2*H1(0)*(2+u*(-2+v)-2*v+u*u)+
     & 2*H1(1)*(2+u*(-2+v)-2*v+u*u))+
     & G3(2,1,1)*(4*H1(0)*(2+u*(-2+v)-2*v+u*u)+
     & 4*H1(1)*(2+u*(-2+v)-2*v+u*u))+
     & G3(2,2,1)*((H1(0)*(4+6*(-1+5*NFV)*v+u*(-6+30*NFV+3*v-60*NFV*v)+
     & (3-60*NFV)*(u*u)))/9._dp+
     & (H1(1)*(4+6*(-1+5*NFV)*v+u*(-6+30*NFV+3*v-60*NFV*v)+
     & (3-60*NFV)*(u*u)))/9._dp)+
     & G3(1,0,1)*((H1(0)*(20+u*(NFV*(30-60*v)+21*(-2+v))+
     & 6*(-7+5*NFV)*v+(21-60*NFV)*(u*u)))/9._dp+
     & (H1(1)*(20+u*(NFV*(30-60*v)+21*(-2+v))+6*(-7+5*NFV)*v+
     & (21-60*NFV)*(u*u)))/9._dp)+
     & G3(1,2,1)*((H1(0)*(32-30*(1+NFV)*v+15*u*(-2+v+NFV*(-2+4*v))+
     & 15*(1+4*NFV)*(u*u)))/9._dp+
     & (H1(1)*(32-30*(1+NFV)*v+15*u*(-2+v+NFV*(-2+4*v))+
     & 15*(1+4*NFV)*(u*u)))/9._dp)+
     & G3(0,0,1)*((2*H1(0)*(8-3*(-4+5*NFV)*v+
     & 3*u*(4-2*v+5*NFV*(-1+2*v))+6*(-1+5*NFV)*(u*u)))/9._dp+
     & (2*H1(1)*(8-3*(-4+5*NFV)*v+3*u*(4-2*v+5*NFV*(-1+2*v))+
     & 6*(-1+5*NFV)*(u*u)))/9._dp)+
     & G3(0,1,1)*((-10*H1(0)*(-2-3*(-2+NFV)*v+u*(6-3*v+NFV*(-3+6*v))+
     & (-3+6*NFV)*(u*u)))/9._dp-
     & (10*H1(1)*(-2-3*(-2+NFV)*v+u*(6-3*v+NFV*(-3+6*v))+
     & (-3+6*NFV)*(u*u)))/9._dp)+
     & G3(1,1,1)*((H1(0)*(3*u*(38-10*NFV-19*v+20*NFV*v)-
     & 2*(56+3*(-19+5*NFV)*v)+(-57+60*NFV)*(u*u)))/9._dp+
     & (H1(1)*(3*u*(38-10*NFV-19*v+20*NFV*v)-
     & 2*(56+3*(-19+5*NFV)*v)+(-57+60*NFV)*(u*u)))/9._dp)+
     & G3(3,0,1)*((H1(0)*(-10+(22-30*NFV)*v+
     & u*(22-30*NFV-11*v+60*NFV*v)+(-11+60*NFV)*(u*u)))/9._dp+
     & (H1(1)*(-10+(22-30*NFV)*v+u*(22-30*NFV-11*v+60*NFV*v)+
     & (-11+60*NFV)*(u*u)))/9._dp)+
     & G3(3,2,1)*((H1(0)*(-10+(22-30*NFV)*v+
     & u*(22-30*NFV-11*v+60*NFV*v)+(-11+60*NFV)*(u*u)))/9._dp+
     & (H1(1)*(-10+(22-30*NFV)*v+u*(22-30*NFV-11*v+60*NFV*v)+
     & (-11+60*NFV)*(u*u)))/9._dp)+
     & G3(0,2,1)*((H1(0)*(-8+2*(7+15*NFV)*v+
     & u*(14+30*NFV-7*v-60*NFV*v)-(7+60*NFV)*(u*u)))/9._dp+
     & (H1(1)*(-8+2*(7+15*NFV)*v+u*(14+30*NFV-7*v-60*NFV*v)-
     & (7+60*NFV)*(u*u)))/9._dp)+
     & G3(2,0,1)*((H1(0)*(u*(14+30*NFV-7*v-60*NFV*v)+
     & 2*(-9+(7+15*NFV)*v)-(7+60*NFV)*(u*u)))/9._dp+
     & (H1(1)*(u*(14+30*NFV-7*v-60*NFV*v)+2*(-9+(7+15*NFV)*v)-
     & (7+60*NFV)*(u*u)))/9._dp)+
     & G3(0,1,2)*(-2*H1(0)*(1+u*(-2+v)-2*v+u*u)-
     & (2*H1(1)*(8-3*(2+5*NFV)*v+3*u*(-2+v+5*NFV*(-1+2*v))+
     & (3+30*NFV)*(u*u)))/9._dp+
     & (85+32/omw-256*u+(20*u)/omv+(41*u)/omw-232*v-
     & (63*v)/omu+77*u*v-(19*v)/(omu*omu)+
     & 30*NFV*(-2+7*u+(-3+2*u)/omw+3*v+v/(omu*omu)+
     & (3*u)/(omw*omw))-(32*u)/(omw*omw)+
     & nf*(-6-5*u*(-2+v)+10*v-5*(u*u))+77*(u*u)+
     & (10*(u*u))/(omv*omv))/18._dp)+
     & G3(0,0,2)*((-2*omw*(28-15*NFV-14*u+30*NFV*u)*H1(0))/9._dp+
     & (H1(1)*(7*u*(-2+v)-2*(1+7*v)+7*(u*u)))/9._dp+
     & (-11+u*(55-13/omw-18*v)+72*v-18*(u*u)+
     & 3*nf*(1+u*(-2+v)-2*v+u*u))/9._dp)+
     & G3(2,0,0)*((H1(1)*(7*u*(-2+v)-2*(1+7*v)+7*(u*u)))/9._dp+
     & (H1(0)*(16-28*v+u*(-28+11*v)+11*(u*u)))/9._dp+
     & (-11+u*(55-13/omw-18*v)+72*v-18*(u*u)+
     & 3*nf*(1+u*(-2+v)-2*v+u*u))/9._dp)+
     & G3(0,2,0)*((H1(1)*(7*u*(-2+v)-2*(1+7*v)+7*(u*u)))/9._dp+
     & (H1(0)*(160-28*v+u*(-28+11*v)+11*(u*u)))/9._dp+
     & (-11+u*(55-13/omw-18*v)+72*v-18*(u*u)+
     & 3*nf*(1+u*(-2+v)-2*v+u*u))/9._dp)+
     & G3(3,0,0)*((-16*H1(1)*(1+u*(-2+v)-2*v+u*u))/9._dp-
     & (2*H1(0)*(-64-23*v+u*(-23+10*v)+10*(u*u)))/9._dp+
     & (-246-72*v+u*(-51-64/omw+18*v+2/w)-2/w+18*(u*u)-
     & 3*nf*(u*(-2+v)-2*(4+v)+u*u))/9._dp)+
     & G3(1,2,2)*((H1(0)*(-32+30*(1+NFV)*v-15*u*(-2+v+NFV*(-2+4*v))-
     & 15*(1+4*NFV)*(u*u)))/9._dp+
     & (123-140*u-140*v+81*u*v-(34*v)/(omu*omu)+81*(u*u)-
     & 4*nf*(2+u*(-2+v)-2*v+u*u)+
     & 10*NFV*(4+u*(-11+4*v)+v*(-11-6/(omu*omu))+4*(u*u)))/18._dp)
     & +G2(0,1)*(2*H2(1,0)*(1+u*(-2+v)-2*v+u*u)+
     & (2*zeta2*(104+75*(-2+NFV)*v-75*u*(2-v+NFV*(-1+2*v))-
     & 75*(-1+2*NFV)*(u*u)))/9._dp+
     & (2*H2(0,1)*(-1-3*(-4+5*NFV)*v+3*u*(4-2*v+5*NFV*(-1+2*v))+
     & 6*(-1+5*NFV)*(u*u)))/9._dp+
     & (2*H2(1,1)*(8-3*(2+5*NFV)*v+3*u*(-2+v+5*NFV*(-1+2*v))+
     & (3+30*NFV)*(u*u)))/9._dp+
     & (H1(0)*(-85+60*NFV+256*u-(20*u)/omv-210*NFV*u-
     & (32-90*NFV+41*u+60*NFV*u)/omw+232*v+(63*v)/omu-
     & 90*NFV*v-77*u*v+(19*v)/(omu*omu)-(30*NFV*v)/(omu*omu)+
     & ((32-90*NFV)*u)/(omw*omw)-77*(u*u)-
     & (10*(u*u))/(omv*omv)+nf*(6+5*u*(-2+v)-10*v+5*(u*u))))/
     & 18._dp+(H1(1)*(-85+60*NFV+256*u-(20*u)/omv-210*NFV*u-
     & (32-90*NFV+41*u+60*NFV*u)/omw+232*v+(63*v)/omu-
     & 90*NFV*v-77*u*v+(19*v)/(omu*omu)-(30*NFV*v)/(omu*omu)+
     & ((32-90*NFV)*u)/(omw*omw)-77*(u*u)-
     & (10*(u*u))/(omv*omv)+nf*(6+5*u*(-2+v)-10*v+5*(u*u))))/
     & 18._dp)+G2(2,3)*((-2*H2(0,1)*(13+4*u*(-2+v)-8*v+4*(u*u)))/9._dp-
     & (2*H2(1,0)*(13+4*u*(-2+v)-8*v+4*(u*u)))/9._dp+
     & (4*zeta2*(-13-4*v+u*(-4+5*v)+5*(u*u)))/9._dp+
     & (H2(0,0)*(-4*(45+v)+u*(-4+5*v)+5*(u*u)))/9._dp+
     & (H1(0)*(-588+112*v+(63*v)/omu+
     & u*(142-20/omv-80/omw-73*v+4/w)-4/w+
     & (1111*v)/(omu*omu)-(2529*v)/(omu*omu*omu)+
     & (2529*v)/(omu*omu*omu*omu)+(-73-10/(omv*omv))*(u*u)+
     & 36*nf*(1-(3*v*(1+u+u*u))/(omu*omu*omu*omu))))/18._dp)+
     & G3(2,3,0)*((-2*H1(0)*(-41+4*u*(-2+v)-8*v+4*(u*u)))/9._dp+
     & (2*H1(1)*(13+4*u*(-2+v)-8*v+4*(u*u)))/9._dp+
     & (588-112*v-(63*v)/omu+
     & u*(-142+20/omv+80/omw+73*v-4/w)+4/w-
     & (1111*v)/(omu*omu)+(2529*v)/(omu*omu*omu)-
     & (2529*v)/(omu*omu*omu*omu)+(73+10/(omv*omv))*(u*u)+
     & 36*nf*(-1+(3*v*(1+u+u*u))/(omu*omu*omu*omu)))/18._dp)+
     & G2(0,0)*((H2(1,0)*(-16+28*u+28*v-11*u*v-11*(u*u)))/9._dp+
     & (H2(1,1)*(2-7*u*(-2+v)+14*v-7*(u*u)))/9._dp-
     & (16*H2(0,1)*(1+u*(-2+v)-2*v+u*u))/9._dp-
     & (2*H2(0,0)*(-64-23*v+u*(-23+10*v)+10*(u*u)))/9._dp+
     & (H1(1)*(11-72*v+u*(-55+13/omw+18*v)+18*(u*u)-
     & 3*nf*(1+u*(-2+v)-2*v+u*u)))/9._dp+
     & (H1(0)*(-246-72*v+u*(-51-64/omw+18*v+2/w)-2/w+18*(u*u)-
     & 3*nf*(u*(-2+v)-2*(4+v)+u*u)))/9._dp+
     & (18*nf*(-9+(2*(-4+omw)*u)/omw)+15*(nf*nf)+
     & 2*(6*(-176+(-23+280/omw)*u)+
     & 6*zeta2*(-6-50*v+u*(-50+31*v)+31*(u*u))))/108._dp)-
     & (H2(1,1)*(12*zeta2*(omu*omu)*
     & (96+(16-90*NFV)*v+u*(16-11*v+90*NFV*(-1+2*v))+
     & (-11+180*NFV)*(u*u))+
     & 12*nf*(-3-4*u*(-2+v)+4*v+(-7+2*v)*(u*u)+2*(u*u*u))+
     & 3*(189+80*(-5+3*NFV)*v-4*u*(145-101*v+40*NFV*(-1+2*v))+
     & (593+160*NFV*(-2+v)-202*v)*(u*u)+2*(-101+80*NFV)*(u*u*u))
     & ))/(108._dp*(omu*omu))+
     & G2(2,2)*(-(omw*(-4+5*u)*H2(0,0))/9._dp+
     & (H2(1,0)*(2*(-6+(2+5*NFV)*v)-u*(-4+v+10*NFV*(-1+2*v))-
     & (1+20*NFV)*(u*u)))/3._dp+
     & (H2(0,1)*(-4+(6-30*NFV)*v+u*(6-3*v+30*NFV*(-1+2*v))+
     & (-3+60*NFV)*(u*u)))/9._dp+
     & (H1(0)*(123-140*u-140*v+81*u*v-(34*v)/(omu*omu)+81*(u*u)-
     & 4*nf*(2+u*(-2+v)-2*v+u*u)+
     & 10*NFV*(4+u*(-11+4*v)+v*(-11-6/(omu*omu))+4*(u*u))))/
     & 18._dp-(12*zeta2*(omu*omu)*
     & (38+(56-90*NFV)*v+u*(56-31*v+90*NFV*(-1+2*v))+
     & (-31+180*NFV)*(u*u))+
     & 12*nf*(-3-4*u*(-2+v)+4*v+(-7+2*v)*(u*u)+2*(u*u*u))+
     & 3*(189+80*(-5+3*NFV)*v-4*u*(145-101*v+40*NFV*(-1+2*v))+
     & (593+160*NFV*(-2+v)-202*v)*(u*u)+2*(-101+80*NFV)*(u*u*u)
     & ))/(108._dp*(omu*omu)))+
     & G3(2,2,0)*((H1(0)*(-20+3*u*v+3*(u*u)))/9._dp+
     & (-60+156*v+u*(322+30*NFV-518*v-60*NFV*v)+
     & (-687-90*omv*NFV+666*v)*(u*u)+
     & 4*nf*(omu*omu*omu)*(1+u*(-2+v)-2*v+u*u)+
     & (729+130*NFV-385*v-110*NFV*v)*(u*u*u)+
     & (-385-110*NFV+81*v+40*NFV*v)*(u*u*u*u)+
     & (81+40*NFV)*(u*u*u*u*u))/(18._dp*(omu*omu*omu)))+
     & G3(2,0,2)*((2*H1(0)*(-5-3*(2+5*NFV)*v+
     & 3*u*(-2+v+5*NFV*(-1+2*v))+(3+30*NFV)*(u*u)))/9._dp+
     & (-60+156*v+u*(322+30*NFV-518*v-60*NFV*v)+
     & (-687-90*omv*NFV+666*v)*(u*u)+
     & 4*nf*(omu*omu*omu)*(1+u*(-2+v)-2*v+u*u)+
     & (729+130*NFV-385*v-110*NFV*v)*(u*u*u)+
     & (-385-110*NFV+81*v+40*NFV*v)*(u*u*u*u)+
     & (81+40*NFV)*(u*u*u*u*u))/(18._dp*(omu*omu*omu)))+
     & G3(0,2,2)*((2*H1(0)*(8-3*(2+5*NFV)*v+3*u*(-2+v+5*NFV*(-1+2*v))+
     & (3+30*NFV)*(u*u)))/9._dp+
     & (-60+156*v+u*(322+30*NFV-518*v-60*NFV*v)+
     & (-687-90*omv*NFV+666*v)*(u*u)+
     & 4*nf*(omu*omu*omu)*(1+u*(-2+v)-2*v+u*u)+
     & (729+130*NFV-385*v-110*NFV*v)*(u*u*u)+
     & (-385-110*NFV+81*v+40*NFV*v)*(u*u*u*u)+
     & (81+40*NFV)*(u*u*u*u*u))/(18._dp*(omu*omu*omu)))+
     & G3(2,1,2)*(-2*H1(0)*(2+u*(-2+v)-2*v+u*u)+
     & (97+32/omw-174*u-(20*u)/omv+(41*u)/omw-152*v+(3*v)/u+
     & 107*u*v-(52*v)/(omu*omu)-(32*u)/(omw*omw)+87*(u*u)-
     & (10*(u*u))/(omv*omv)-4*nf*(2+u*(-2+v)-2*v+u*u)+
     & 10*NFV*(-4+(-9+6*u)/omw-13*v+u*(-1+2*v)-
     & (6*v)/(omu*omu)+(9*u)/(omw*omw)+2*(u*u))+19*(v*v)-
     & (2*(v*v))/u-(v*v*v)/u)/18._dp)+
     & G2(2,1)*(2*zeta2*(2+u*(-2+v)-2*v+u*u)-
     & 2*H2(0,1)*(2+u*(-2+v)-2*v+u*u)+
     & 2*H2(1,0)*(2+u*(-2+v)-2*v+u*u)+
     & (H1(0)*(-97-32/omw+174*u+(20*u)/omv-(41*u)/omw+152*v-
     & (3*v)/u-107*u*v+(52*v)/(omu*omu)+(32*u)/(omw*omw)+
     & 10*NFV*(4+(9-6*u)/omw+u+13*v-2*u*v+
     & (6*v)/(omu*omu)-(9*u)/(omw*omw)-2*(u*u))-87*(u*u)+
     & (10*(u*u))/(omv*omv)+4*nf*(2+u*(-2+v)-2*v+u*u)-
     & 19*(v*v)+(2*(v*v))/u+(v*v*v)/u))/18._dp+
     & (H1(1)*(-97-32/omw+174*u+(20*u)/omv-(41*u)/omw+152*v-
     & (3*v)/u-107*u*v+(52*v)/(omu*omu)+(32*u)/(omw*omw)+
     & 10*NFV*(4+(9-6*u)/omw+u+13*v-2*u*v+
     & (6*v)/(omu*omu)-(9*u)/(omw*omw)-2*(u*u))-87*(u*u)+
     & (10*(u*u))/(omv*omv)+4*nf*(2+u*(-2+v)-2*v+u*u)-
     & 19*(v*v)+(2*(v*v))/u+(v*v*v)/u))/18._dp)+
     & (H2(0,1)*(-339-108/omu+654*u-(60*u)/omv-(444*u)/omw+1026*v+
     & (108*u)/w+1896*zeta2-1536*u*zeta2-1536*v*zeta2+
     & 804*u*v*zeta2+18*nf*(3-2*u+v*(-2-2/(omu*omu)))+
     & (522*v)/(omu*omu)+804*zeta2*(u*u)+
     & 60*NFV*(-6-3/omu-8*v+18*v*zeta2-(4*v)/(omu*omu)+
     & u*(-2+9/omw+7/w+6*(3-6*v)*zeta2-2/(w*w))+
     & 2*(u*u)*(-18*zeta2+omw/(w*w)))))/108._dp+
     & G2(3,3)*((8*zeta2*(-32-15*v+3*u*(-5+2*v)+6*(u*u)))/9._dp-
     & (2*H2(0,0)*(-64-9*v+u*(-9+6*v)+6*(u*u)))/9._dp+
     & (H1(0)*(523+40*u-(128*u)/omw+18*v+(24*v)/u-34*u*v+308/w-
     & (157*u)/w-14*(u*u)+4*nf*(u*(-2+v)-2*(4+v)+u*u)-
     & 19*(v*v)+(2*(v*v))/u+(v*v*v)/u+
     & 10*NFV*(-2+omw-2*(u*u)+4/((-1+u+v)*(-1+u+v))+
     & u*(-2*v-(2*(u+v))/
     & ((-1+u+v)*(-1+u+v)*(-1+u+v)))+
     & 2/((-1+u+v)*(-1+u+v)*(-1+u+v)))-151/(w*w)+
     & (151*u)/(w*w)))/18._dp)+
     & (H3(0,0,0)*(-1738-574*v-(63*v)/omu+162/w-(1129*v)/(omu*omu)+
     & (2529*v)/(omu*omu*omu)-(2529*v)/(omu*omu*omu*omu)+
     & (259-10/(omv*omv))*(u*u)-
     & (2*nf*(7*(omu*omu*omu*omu)*(-8+u*(-2+v)+u*u)+
     & 2*v*(-34+u-69*(u*u)+28*(u*u*u)-7*(u*u*u*u))))/
     & (omu*omu*omu*omu)+
     & u*(-524-20/omv-192/omw+259*v-82/w+80/(w*w))-80/(w*w)))/
     & 18._dp+nf*(4219._dp/486._dp+100/(27-27*u)-(139*zeta3)/27._dp+
     & (2*u*(-2+v)*zeta3)/9._dp-(4*v*zeta3)/9._dp+(6*v*zeta3)/(omu*omu)-
     & (18*v*zeta3)/(omu*omu*omu)+(18*v*zeta3)/(omu*omu*omu*omu)+
     & (2*zeta3*(u*u))/9._dp+(zeta2*
     & (-486*omu*omw*w*(1+2*w)+
     & 4*omw*(243+v*(-376+90*u-273*(u*u)+73*(u*u*u)))*(w*w)+
     & 162*omw*(omu*omu)*(-1+3*w+2*(w*w))+
     & omu*omu*omu*(-96*u*(w*w)+
     & omw*(152*(u*u)*(w*w)-9*(-18+12*w+95*(w*w))+
     & 2*u*(27-27*w+2*(-67+38*v)*(w*w))))))/
     & (54._dp*omw*(omu*omu*omu)*(w*w)))+
     & H1(1)*((H2(1,0)*(-2*(-5+v)+u*(-2+v)+u*u))/6._dp+
     & (H2(0,0)*(u*(-2+v)-2*(4+v)+u*u))/6._dp+
     & (4*nf*(-28-57*u-57*v-(39*v)/(omu*omu)+
     & 18*zeta2*(1+2*u*(-2+v)+2*(u*u)-
     & (v*(31+11*u+51*(u*u)-16*(u*u*u)+4*(u*u*u*u)))/
     & (omu*omu*omu*omu)))+
     & 3*(532-498/omu+1215*u+1215*v+5892*zeta3-1680*u*zeta3-
     & 1680*v*zeta3+768*u*v*zeta3+(1461*v)/(omu*omu)+
     & 768*zeta3*(u*u)+6*zeta2*
     & (132+96/omw-4/w+
     & u*(67-20/omv-35/omw-61*v+42/w-96/(omw*omw))+
     & (3*v*(442-187*u+831*(u*u)-340*(u*u*u)+97*(u*u*u*u)))/
     & (omu*omu*omu*omu)+
     & u*u*(-23-56/w-10/(omv*omv)+9/(w*w))-
     & (9*v*(u*u*u))/(w*w)-(9*(u*u*u*u))/(w*w))+
     & 10*NFV*(21+60/omu-36*v-(6*u*(1+5*w))/w-
     & (72*v)/(omu*omu)+
     & 6*zeta2*(3*(8/omu-9/omw-2*(7+v)+(7*v)/(omu*omu)-
     & (8*v)/(omu*omu*omu))+
     & 3*u*(8+6/omw-8*v+1/w+9/(omw*omw))-
     & (3*(u*u)*(-2+5*w+6*(w*w)))/(w*w)+
     & (2*(-1+5*v)*(u*u*u*u))/(w*w*w)+
     & (4*(u*u*u*u*u))/(w*w*w)+
     & (2*(u*u*u)*(-1-v+3*(v*v)))/(w*w*w)))))/324._dp)+
     & G3(3,3,0)*((16*H1(0)*(u*(-2+v)-2*(4+v)+u*u))/9._dp+
     & (-523-40*u+(128*u)/omw-18*v-(24*v)/u+34*u*v-308/w+
     & (157*u)/w+14*(u*u)-4*nf*(u*(-2+v)-2*(4+v)+u*u)+
     & 19*(v*v)-(2*(v*v))/u-(v*v*v)/u+151/(w*w)-(151*u)/(w*w)+
     & 10*NFV*(2-v+2*(u*u)-4/(w*w)+
     & u*(-1+2*v+2/(w*w)-2/(w*w*w))+2/(w*w*w)))/18._dp)+
     & (H3(1,0,1)*(26-32/omw+52*v+(18*v)/(omu*omu)+
     & u*(-4+20/omv-41/omw-35*v+38/w+32/(omw*omw))+
     & u*u*(3-56/w+10/(omv*omv)+9/(w*w))-(9*omw*(u*u*u))/(w*w)+
     & 10*NFV*(6+9/omw+3*v+3*u*(-5-2/omw+1/w-3/(omw*omw))+
     & (3*(u*u)*(2-5*w+2*(w*w)))/(w*w)-(2*omw*(u*u*u*u))/(w*w*w)+
     & (2*(u*u*u)*(-1-v+u*(-1+6*v)+3*(u*u)+3*(v*v)))/(w*w*w))))/
     & 18._dp+G3(0,3,0)*((-8*H1(0)*(u*(-2+v)-2*(4+v)+u*u))/9._dp+
     & (H1(1)*(u*(-2+v)-2*(4+v)+u*u))/9._dp+
     & (180-118*v-(63*v)/omu+
     & u*(22-20/omv-200/omw+60*NFV+67*v-34/w-(30*NFV)/w)-
     & 4/w-(1111*v)/(omu*omu)+(2529*v)/(omu*omu*omu)-
     & (2529*v)/(omu*omu*omu*omu)+
     & nf*(-(u*(-2+v))-u*u+
     & 2*(4+(v*(55+50*u+60*(u*u)-4*(u*u*u)+u*u*u*u))/
     & (omu*omu*omu*omu)))+
     & u*u*(-10/(omv*omv)+
     & (-9+56*w+29*(w*w)-30*NFV*(2-5*w+2*(w*w)))/(w*w))+
     & (20*omw*NFV*(u*u*u*u))/(w*w*w)-
     & (u*u*u*(9*(-1+w)*w+20*NFV*(1-5*w+3*(w*w))))/(w*w*w))/18._dp)+
     & G2(0,3)*((H2(0,1)*(8+2*u+2*v-u*v-u*u))/9._dp+
     & (H2(1,0)*(8+2*u+2*v-u*v-u*u))/9._dp+
     & (22*zeta2*(u*(-2+v)-2*(4+v)+u*u))/9._dp+
     & (16*H2(0,0)*(u*(-2+v)-2*(4+v)+u*u))/9._dp+
     & (H1(0)*(-180+118*v+(63*v)/omu+
     & u*(-22+20/omv+200/omw-67*v+30*NFV*(-2+1/w)+34/w)+
     & 4/w+(1111*v)/(omu*omu)-(2529*v)/(omu*omu*omu)+
     & (2529*v)/(omu*omu*omu*omu)+
     & nf*(-8+u*(-2+v)+u*u-
     & (2*v*(55+50*u+60*(u*u)-4*(u*u*u)+u*u*u*u))/
     & (omu*omu*omu*omu))+
     & u*u*(10/(omv*omv)+
     & (9-56*w-29*(w*w)+30*NFV*(2-5*w+2*(w*w)))/(w*w))-
     & (20*omw*NFV*(u*u*u*u))/(w*w*w)+
     & (u*u*u*(9*(-1+w)*w+20*NFV*(1-5*w+3*(w*w))))/(w*w*w)))/18._dp)
     & +(H3(0,0,1)*(u*(10/omw+(38+30*NFV*v-26*w-35*v*w)/w)+
     & 2*(-8+v*(8+15*NFV-9/(omu*omu)))+
     & (u*u*(9-56*w+60*NFV*(omw*omw)+3*(w*w)))/(w*w)+
     & ((-9+18*v+20*NFV*(-1+5*v))*(u*u*u*u))/(w*w*w)+
     & ((9+40*NFV)*(u*u*u*u*u))/(w*w*w)+
     & (u*u*u*(-9*omv*v+20*NFV*(-1-v+3*(v*v))))/(w*w*w)))/18._dp+
     & G2(1,0)*((2*H2(0,0)*(80+u*(-5+v)-5*v+u*u))/9._dp+
     & (16*H2(1,1)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (H2(0,1)*(7*u*(-2+v)-2*(1+7*v)+7*(u*u)))/9._dp-
     & (4*zeta2*(-78-17*v+u*(-17+10*v)+10*(u*u)))/9._dp+
     & (H2(1,0)*(178-28*v+u*(-28+11*v)+11*(u*u)))/9._dp-
     & (H1(0)*(-27*v*(omv*omv*omv)+
     & (64+NFV*(110-80*v)+28*v+5*nf*(-7+4*v))*(u*u*u*u*u*u*u)+
     & (2+5*nf-20*NFV)*(u*u*u*u*u*u*u*u)+
     & u*u*u*u*u*u*(-368+142*v+81*(v*v)-
     & 10*NFV*(27-38*v+12*(v*v))+nf*(106-125*v+30*(v*v)))+
     & u*u*u*u*u*(810-844*v+55*(v*v)+86*(v*v*v)-
     & 10*NFV*(-36+76*v-45*(v*v)+6*(v*v*v))+
     & nf*(-180+328*v-165*(v*v)+20*(v*v*v)))+
     & u*u*u*(548-913*v+408*(v*v)+5*(v*v*v)-
     & 10*NFV*(-3+31*v-42*(v*v)+18*(v*v*v))+
     & nf*(-115+368*v-399*(v*v)+166*(v*v*v)-20*(v*v*v*v))-
     & 28*(v*v*v*v))+u*u*
     & (-164+133*v+212*(v*v)+
     & nf*(omv*omv)*(40-77*v+25*(v*v))-225*(v*v*v)-
     & 10*NFV*(-5+7*v-3*(v*v)+3*(v*v*v))+53*(v*v*v*v))+
     & u*(18+93*v+2*nf*(-3+5*v)*(omv*omv*omv)-313*(v*v)+
     & 266*(v*v*v)-64*(v*v*v*v)-
     & 10*NFV*(2-7*v+9*(v*v)-9*(v*v*v)+3*(v*v*v*v)))+
     & u*u*u*u*(-910+1388*v-524*(v*v)-51*(v*v*v)+
     & 10*NFV*(-24+77*v-69*(v*v)+18*(v*v*v))+31*(v*v*v*v)+
     & nf*(185-462*v+363*(v*v)-95*(v*v*v)+5*(v*v*v*v)))))/
     & (18._dp*u*(omu*omu)*(w*w*w))-
     & (H1(1)*(-27*v*(omv*omv*omv)+
     & (64+NFV*(110-80*v)+28*v+5*nf*(-7+4*v))*(u*u*u*u*u*u*u)+
     & (2+5*nf-20*NFV)*(u*u*u*u*u*u*u*u)+
     & u*u*u*u*u*u*(-368+142*v+81*(v*v)-
     & 10*NFV*(27-38*v+12*(v*v))+nf*(106-125*v+30*(v*v)))+
     & u*u*u*u*u*(810-844*v+55*(v*v)+86*(v*v*v)-
     & 10*NFV*(-36+76*v-45*(v*v)+6*(v*v*v))+
     & nf*(-180+328*v-165*(v*v)+20*(v*v*v)))+
     & u*u*u*(548-913*v+408*(v*v)+5*(v*v*v)-
     & 10*NFV*(-3+31*v-42*(v*v)+18*(v*v*v))+
     & nf*(-115+368*v-399*(v*v)+166*(v*v*v)-20*(v*v*v*v))-
     & 28*(v*v*v*v))+u*u*
     & (-164+133*v+212*(v*v)+
     & nf*(omv*omv)*(40-77*v+25*(v*v))-225*(v*v*v)-
     & 10*NFV*(-5+7*v-3*(v*v)+3*(v*v*v))+53*(v*v*v*v))+
     & u*(18+93*v+2*nf*(-3+5*v)*(omv*omv*omv)-313*(v*v)+
     & 266*(v*v*v)-64*(v*v*v*v)-
     & 10*NFV*(2-7*v+9*(v*v)-9*(v*v*v)+3*(v*v*v*v)))+
     & u*u*u*u*(-910+1388*v-524*(v*v)-51*(v*v*v)+
     & 10*NFV*(-24+77*v-69*(v*v)+18*(v*v*v))+31*(v*v*v*v)+
     & nf*(185-462*v+363*(v*v)-95*(v*v*v)+5*(v*v*v*v)))))/
     & (18._dp*u*(omu*omu)*(w*w*w)))+
     & G2(3,2)*((2*H2(0,0)*(40-v+u*(-1+2*v)+2*(u*u)))/9._dp+
     & (H2(0,1)*(10+u*(NFV*(30-60*v)+11*(-2+v))+(-22+30*NFV)*v+
     & (11-60*NFV)*(u*u)))/9._dp+
     & (H2(1,0)*(-2*(15+(2+5*NFV)*v)+u*(-4+v+10*NFV*(-1+2*v))+
     & (1+20*NFV)*(u*u)))/3._dp+
     & (zeta2*(-2*(79+(-64+45*NFV)*v)+
     & u*(128-67*v+90*NFV*(-1+2*v))+(-67+180*NFV)*(u*u)))/9._dp+
     & (H1(0)*((30-44*v+40*NFV*(-1+2*v))*(u*u*u*u*u)+
     & (-6+20*NFV)*(u*u*u*u*u*u)-
     & v*(omv*omv*omv)*(-30+2*v+v*v)+
     & u*u*u*u*(-60+162*v-115*(v*v)+20*NFV*(1-5*v+6*(v*v)))+
     & u*u*u*(60-263*v+(305-60*NFV)*(v*v)+
     & 20*(-7+4*NFV)*(v*v*v))+
     & u*u*(-30+237*v-(379+60*NFV)*(v*v)+(243+20*NFV)*(v*v*v)+
     & 20*(-4+NFV)*(v*v*v*v))+
     & u*(6-122*v+272*(v*v)-(209+40*NFV)*(v*v*v)+
     & (69+20*NFV)*(v*v*v*v)-16*(v*v*v*v*v))))/(18._dp*u*(w*w*w)))+
     & G3(3,2,0)*((H1(0)*(-88-3*u*v-3*(u*u)))/9._dp+
     & ((-30+NFV*(40-80*v)+44*v)*(u*u*u*u*u)+
     & (6-20*NFV)*(u*u*u*u*u*u)+v*(omv*omv*omv)*(-30+2*v+v*v)+
     & u*u*u*u*(60-162*v+115*(v*v)-20*NFV*(1-5*v+6*(v*v)))+
     & u*u*u*(-60+263*v+5*(-61+12*NFV)*(v*v)-
     & 20*(-7+4*NFV)*(v*v*v))+
     & u*u*(30-237*v+(379+60*NFV)*(v*v)-(243+20*NFV)*(v*v*v)-
     & 20*(-4+NFV)*(v*v*v*v))+
     & u*(-6+122*v-272*(v*v)+(209+40*NFV)*(v*v*v)-
     & (69+20*NFV)*(v*v*v*v)+16*(v*v*v*v*v)))/(18._dp*u*(w*w*w)))+
     & G3(3,0,2)*((-2*H1(0)*(-1-3*(2+5*NFV)*v+
     & 3*u*(-2+v+5*NFV*(-1+2*v))+(3+30*NFV)*(u*u)))/9._dp+
     & ((-30+NFV*(40-80*v)+44*v)*(u*u*u*u*u)+
     & (6-20*NFV)*(u*u*u*u*u*u)+v*(omv*omv*omv)*(-30+2*v+v*v)+
     & u*u*u*u*(60-162*v+115*(v*v)-20*NFV*(1-5*v+6*(v*v)))+
     & u*u*u*(-60+263*v+5*(-61+12*NFV)*(v*v)-
     & 20*(-7+4*NFV)*(v*v*v))+
     & u*u*(30-237*v+(379+60*NFV)*(v*v)-(243+20*NFV)*(v*v*v)-
     & 20*(-4+NFV)*(v*v*v*v))+
     & u*(-6+122*v-272*(v*v)+(209+40*NFV)*(v*v*v)-
     & (69+20*NFV)*(v*v*v*v)+16*(v*v*v*v*v)))/(18._dp*u*(w*w*w)))+
     & (H3(0,1,0)*(808-134*v-(63*v)/omu+30*NFV*v+166/w+
     & (1113*v)/(omu*omu)-(60*NFV*v)/(omu*omu)-
     & (2529*v)/(omu*omu*omu)+(2529*v)/(omu*omu*omu*omu)-
     & (6*nf*(omu*omu*omu*omu*(10+u*(-2+v)+u*u)+
     & 2*v*(8+13*u+3*(u*u)+4*(u*u*u)-u*u*u*u)))/
     & (omu*omu*omu*omu)+
     & u*(-204-20/omv+90/omw+74*v-48/w+(30*omw*NFV)/w+
     & 80/(w*w))-80/(w*w)+
     & u*u*(-10/(omv*omv)+(9-56*w+112*(w*w)+
     & 30*NFV*(2-5*w+2*(w*w)))/(w*w))-
     & (20*omw*NFV*(u*u*u*u))/(w*w*w)+
     & (u*u*u*(9*(-1+w)*w+20*NFV*(1-5*w+3*(w*w))))/(w*w*w)))/18._dp+
     & (H3(1,0,0)*(2528-454*v-(63*v)/omu+
     & u*(-526+20/omv+90/omw+208*v+30*NFV*(-2+1/w)+34/w)+
     & 4/w-(1129*v)/(omu*omu)+(2529*v)/(omu*omu*omu)-
     & (2529*v)/(omu*omu*omu*omu)+
     & 12*nf*(-13-u*(-2+v)-u*u+
     & (v*(11+u+21*(u*u)-8*(u*u*u)+2*(u*u*u*u)))/
     & (omu*omu*omu*omu))+
     & u*u*(10/(omv*omv)+(9-56*w+246*(w*w)+
     & 30*NFV*(2-5*w+2*(w*w)))/(w*w))-
     & (20*omw*NFV*(u*u*u*u))/(w*w*w)+
     & (u*u*u*(9*(-1+w)*w+20*NFV*(1-5*w+3*(w*w))))/(w*w*w)))/18._dp+
     & (H3(1,1,0)*((32-90*NFV-39*u+60*NFV*u)/omw+
     & (2*(-16+45*NFV)*u)/(omw*omw)-(40*omw*NFV*(u*u*u*u))/(w*w*w)+
     & (-2529*omu*v*(w*w*w)+3*(359-20*NFV)*v*(omu*omu)*(w*w*w)-
     & 63*v*(omu*omu*omu)*(w*w*w)+
     & v*(2529+4*nf*(-25-35*u-15*(u*u)-8*(u*u*u)+2*(u*u*u*u)))*
     & (w*w*w)+omu*omu*omu*omu*
     & (w*(-2*w*(2+(-44+2*nf+25*v)*w)+
     & u*w*(80+(-204+8*nf+23*v-4*nf*v)*w)+
     & 18*(-1+w)*(u*u*u)+u*u*(18-112*w+(99-4*nf)*(w*w)))
     & +10*NFV*(4*(-1+6*v)*(u*u*u*u)+12*(u*u*u*u*u)+
     & 4*(u*u*u)*(-1-v+3*(v*v))+3*u*(2+w)*(w*w)+
     & 6*w*(u*u)*(2-5*w+2*(w*w))+3*(-2+v)*(w*w*w))))/
     & (omu*omu*omu*omu*(w*w*w))))/18._dp+
     & G2(1,1)*(-4*H2(1,0)*(2+u*(-2+v)-2*v+u*u)+
     & (H2(0,1)*(40+u*(NFV*(30-60*v)+21*(-2+v))+6*(-7+5*NFV)*v+
     & (21-60*NFV)*(u*u)))/9._dp+
     & (H2(1,1)*(-32+30*(1+NFV)*v-15*u*(-2+v+NFV*(-2+4*v))-
     & 15*(1+4*NFV)*(u*u)))/9._dp+
     & (zeta2*(-2*(4+(-69+75*NFV)*v)+
     & 3*u*(-23*(-2+v)+50*NFV*(-1+2*v))+(-69+300*NFV)*(u*u)))/9._dp
     & +(H1(0)*(10*(-11+6*NFV)*u*v*(w*w*w)+
     & omu*omu*(w*(9*(-1+w)*(u*u*u*u)+27*v*(w*w)+
     & u*u*u*(9-56*w-4*(-12+nf)*(w*w))+
     & u*(9-20*w+2*(85+4*nf*(-1+v)-97*v)*(w*w))+
     & u*u*(-9+49*w+(-276+8*nf+10*v-4*nf*v)*(w*w)))-
     & 10*NFV*u*(2-4*w+2*omw*(u*u*u*u)-
     & 3*w*(u*u)*(2-5*w+2*(w*w))-
     & 2*(u*u*u)*(1-5*w+3*(w*w))-9*v*(w*w*w)-
     & u*(2-2*w+3*(w*w)+3*(w*w*w))))))/
     & (18._dp*u*(omu*omu)*(w*w*w))+
     & (H1(1)*(10*(-11+6*NFV)*u*v*(w*w*w)+
     & omu*omu*(w*(9*(-1+w)*(u*u*u*u)+27*v*(w*w)+
     & u*u*u*(9-56*w-4*(-12+nf)*(w*w))+
     & u*(9-20*w+2*(85+4*nf*(-1+v)-97*v)*(w*w))+
     & u*u*(-9+49*w+(-276+8*nf+10*v-4*nf*v)*(w*w)))-
     & 10*NFV*u*(2-4*w+2*omw*(u*u*u*u)-
     & 3*w*(u*u)*(2-5*w+2*(w*w))-
     & 2*(u*u*u)*(1-5*w+3*(w*w))-9*v*(w*w*w)-
     & u*(2-2*w+3*(w*w)+3*(w*w*w))))))/
     & (18._dp*u*(omu*omu)*(w*w*w)))+
     & G3(1,1,2)*(4*H1(0)*(2+u*(-2+v)-2*v+u*u)+
     & (H1(1)*(32-30*(1+NFV)*v+15*u*(-2+v+NFV*(-2+4*v))+
     & 15*(1+4*NFV)*(u*u)))/9._dp+
     & (10*(11-6*NFV)*u*v*(w*w*w)+
     & omu*omu*(w*(-9*(-1+w)*(u*u*u*u)-27*v*(w*w)+
     & u*u*u*(-9+56*w+4*(-12+nf)*(w*w))+
     & u*(-9+20*w-2*(85+4*nf*(-1+v)-97*v)*(w*w))+
     & u*u*(9-49*w+2*(138+2*nf*(-2+v)-5*v)*(w*w)))+
     & 10*NFV*u*(2-4*w+2*omw*(u*u*u*u)-
     & 3*w*(u*u)*(2-5*w+2*(w*w))-
     & 2*(u*u*u)*(1-5*w+3*(w*w))-9*v*(w*w*w)-
     & u*(2-2*w+3*(w*w)+3*(w*w*w)))))/
     & (18._dp*u*(omu*omu)*(w*w*w)))+
     & G3(1,2,0)*((H1(0)*(-178+u*(28-11*v)+28*v-11*(u*u)))/9._dp-
     & (16*H1(1)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (63*omu*u*v*(w*w*w)+(-19+30*NFV)*u*v*(w*w*w)+
     & omu*omu*(w*(9*(-1+w)*(u*u*u*u)-27*v*(w*w)+
     & u*u*u*(9-56*w+(7-5*nf)*(w*w))+
     & u*u*(9+27*w-(112+5*nf*(-2+v)+31*v)*(w*w))+
     & u*(-9+20*w+(7-6*nf-34*v+10*nf*v)*(w*w)))-
     & 10*NFV*u*(-2+4*w+2*omw*(u*u*u*u)-
     & 3*w*(u*u)*(2-5*w+2*(w*w))-
     & 2*(u*u*u)*(1-5*w+3*(w*w))+
     & u*(2-2*w-3*(w*w)+6*(w*w*w)))))/
     & (18._dp*u*(omu*omu)*(w*w*w)))+
     & G3(1,0,2)*((-16*H1(1)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (2*H1(0)*(-9+(28-15*NFV)*v+u*(28-15*NFV-14*v+30*NFV*v)+
     & 2*(-7+15*NFV)*(u*u)))/9._dp+
     & (63*omu*u*v*(w*w*w)+(-19+30*NFV)*u*v*(w*w*w)+
     & omu*omu*(w*(9*(-1+w)*(u*u*u*u)-27*v*(w*w)+
     & u*u*u*(9-56*w+(7-5*nf)*(w*w))+
     & u*u*(9+27*w-(112+5*nf*(-2+v)+31*v)*(w*w))+
     & u*(-9+20*w+(7-6*nf-34*v+10*nf*v)*(w*w)))-
     & 10*NFV*u*(-2+4*w+2*omw*(u*u*u*u)-
     & 3*w*(u*u)*(2-5*w+2*(w*w))-
     & 2*(u*u*u)*(1-5*w+3*(w*w))+
     & u*(2-2*w-3*(w*w)+6*(w*w*w)))))/
     & (18._dp*u*(omu*omu)*(w*w*w)))+
     & (H2(0,0)*(3*omu*omv*omw*(-2529+(-5058+(-2241+40*NFV)*v)*w)*(w*w)-
     & 2*omv*omw*w*(2*nf*(169-70*v+(-61+20*v)*(u*u*u*u*u)-
     & 421*(v*v)+u*u*u*u*(413-141*v+40*(v*v))+322*(v*v*v)+
     & u*u*u*(-880+859*v-59*(v*v)+20*(v*v*v))+
     & u*u*(934-716*v+549*(v*v)+21*(v*v*v))+
     & u*(-575+48*v+377*(v*v)+123*(v*v*v)))-7587*(1+v)*(w*w))
     & +3*omv*omw*w*(omu*omu)*
     & (-843+2529*w+(2186-60*NFV+1379*v)*(w*w))+
     & omu*omu*omu*(-60*omw*u*(1+u)*(w*w*w)+
     & omv*(120*NFV*(omw*omw*omw)*(u*u*u)+
     & 48*(97-4*nf)*u*(w*w*w)-
     & omw*(w*(-2529+2160*w+
     & (3429+264*v+12*(564+427*v)*zeta2-36*(nf*nf)+
     & 6*(v*v))*(w*w)+
     & u*u*(-54+108*w+(-1173+76*nf-2382*zeta2)*(w*w))+
     & u*(-1053+123*w+
     & (1710-1113*v+6*(854-397*v)*zeta2)*(w*w)+
     & 4*nf*(27-27*w+(-32+19*v)*(w*w))))+
     & 30*NFV*(-2*(3+7*v)*(w*w*w)+
     & 2*(u*u)*(1-11*w+15*(w*w)+w*w*w)+
     & u*(-4+11*w-34*(w*w)+3*(3+4*v)*(w*w*w))))))))/
     & (108._dp*omv*omw*(omu*omu*omu)*(w*w*w))+
     & (H2(1,0)*(3*omu*omv*omw*(2529+(5058+(-2217+40*NFV)*v)*w)*(w*w)+
     & 2*omv*omw*w*(nf*(-173-394*v+2*(-118+57*v)*(u*u*u*u*u*u)+
     & 38*(u*u*u*u*u*u*u)+1199*(v*v)+
     & u*u*u*u*u*(945-632*v+114*(v*v))+
     & u*(1451-2032*v+889*(v*v)-308*(v*v*v))+
     & u*u*u*(3838-3914*v+1141*(v*v)-160*(v*v*v))-
     & 632*(v*v*v)+u*u*u*u*
     & (-2407+1996*v-556*(v*v)+38*(v*v*v))+
     & u*u*(-3456+4862*v-1815*(v*v)+90*(v*v*v)))+
     & 7587*(-1+v)*(w*w))+
     & 3*omv*omw*w*(omu*omu)*
     & (843-2529*w+(-2226+120*NFV+1379*v)*(w*w))+
     & omu*omu*omu*(-60*omw*(u*u)*(w*w*w)+
     & omv*(-60*(1+9*NFV)*u*(w*w*w)+
     & omw*(w*(-2529+1200*w+
     & (9807-672*v+6*(1586-342*v)*zeta2-6*(v*v))*(w*w)+
     & 3*(u*u)*(-18+(391+282*zeta2)*(w*w))+
     & 3*u*(-351+433*w+
     & (7*(-50+53*v)+6*(-114+47*v)*zeta2)*(w*w)))+
     & 30*NFV*(4*(u*u*u)*(-1+w*w)+2*(3+7*v)*(w*w*w)-
     & 2*(u*u)*(-1+5*w+5*(w*w)+w*w*w)-
     & u*(4-7*w+3*(7+4*v)*(w*w*w))))))))/
     & (108._dp*omv*omw*(omu*omu*omu)*(w*w*w))+
     & (90*zeta4*(21293+(8584-2640*NFV)*v+
     & 8*u*(1073-628*v+330*NFV*(-1+2*v))+32*(-157+165*NFV)*(u*u))
     & +80*(-36055-5616/omu+18963*zeta3+(2988*zeta3)/w+
     & 9*zeta3*(-93-10/(omv*omv))*(u*u)+
     & (9*v*zeta3*(-824-1252*u+72*(u*u)+
     & 30*NFV*(omu*omu)*(3-2*u+u*u)-679*(u*u*u)+154*(u*u*u*u)
     & ))/(omu*omu*omu*omu)+
     & 9*u*(30*NFV*zeta3-93*v*zeta3+
     & 2*((-10*(2+zeta3))/omv+
     & zeta3*(51+72/omw-86/w+80/(w*w))))-(1440*zeta3)/(w*w))
     & +(360*zeta2*(-3*omu*omv*omw*
     & (-843+2*(-843+815*v+20*NFV*(2+5*v))*w)*(w*w)+
     & omv*omw*w*(omu*omu)*
     & (843-3*(843+80*NFV)*w+(-2286+60*NFV+2644*v)*(w*w))+
     & 18*omv*omw*(-281+(562+80*NFV)*v)*(w*w*w)+
     & omu*omu*omu*(20*omw*u*(1+4*u)*(w*w*w)+
     & omv*(4*(142-135*NFV)*u*(w*w*w)+
     & omw*(w*(-843+80*w+(8527+1912*v+8*(v*v))*(w*w)-
     & 2*(u*u)*(9-45*w+782*(w*w))+
     & u*(-351+591*w-28*(-80+53*v)*(w*w)))-
     & 10*NFV*(4*v*(-3+4*v)*(u*u*u)+4*(-3+8*v)*(u*u*u*u)+
     & 16*(u*u*u*u*u)-24*(1+3*(1+v)*w)*(w*w)-
     & 2*(u*u)*(1-7*w+32*(w*w)+4*(w*w*w))+
     & u*(4-39*w+18*(w*w)-4*(11+12*v)*(w*w*w))))))))/
     & (omv*omw*(omu*omu*omu)*(w*w*w)))/12960._dp+
     & (-5._dp/2._dp-nf/3._dp+(-1._dp/2._dp+2/omw)*u+
     & (H1(0)*(-17+u*(-2+v)-2*v+u*u))/2._dp+
     & (H1(1)*(1+u*(-2+v)-2*v+u*u))/2._dp)*(G1(0)*G1(0)*G1(0))+
     & (9*(G1(0)*G1(0)*G1(0)*G1(0)))/8._dp+
     & (-H1(1)/18._dp+(2*nf-(3*(v+(11+u+v)*(omu*omu)))/(omu*omu))/54._dp+
     & (H1(0)*(-3-u*(-2+v)+2*v-u*u))/18._dp)*(G1(2)*G1(2)*G1(2))+
     & (G1(2)*G1(2)*G1(2)*G1(2))/72._dp+((-17*H1(1))/18._dp+
     & (17*((15+2*nf-6*v)*v+(3+2*nf-3*v)*(u*u*u)+
     & u*(3+nf*(2-4*v)-36*v+6*(v*v))+
     & u*u*(2*nf*(-2+v)-3*(2-7*v+v*v))))/(54._dp*omw*(omu*omu)))*
     & (H1(0)*H1(0)*H1(0))+(289*(H1(0)*H1(0)*H1(0)*H1(0)))/72._dp+
     & H1(0)*H1(0)*((-17*H2(1,0)*(-2*(-5+v)+u*(-2+v)+u*u))/18._dp-
     & (17*H2(0,0)*(u*(-2+v)-2*(4+v)+u*u))/18._dp+
     & (204*omu*omw+12*omw*(-57+2*nf)*v-
     & omu*omu*(48*(57-2*nf)*u+
     & omw*(19065+684*v-4*nf*(515+6*v)+36*(nf*nf)+
     & 6*zeta2*(9+80*u*(-2+v)-160*v+80*(u*u)))))/
     & (216._dp*omw*(omu*omu))+
     & (H1(1)*((1-2*nf-30*v)*v+(43-2*nf-32*v)*(u*u*u)-
     & 17*(u*u*u*u)+u*u*(-35+4*nf+65*v-2*nf*v-15*(v*v))+
     & u*(9-49*v+nf*(-2+4*v)+30*(v*v))))/(18._dp*omw*(omu*omu))-
     & (5*(H1(1)*H1(1)))/12._dp)+G1(0)*G1(0)*
     & (G1(1)*((H1(0)*(-(u*v)-2*w-u*u))/2._dp+(H1(1)*(-(u*v)-2*w-u*u))/2._dp)+
     & (G2(0,2)*(-1+2*omw-u*v-u*u))/2._dp+
     & (G2(2,0)*(-1+2*omw-u*v-u*u))/2._dp+
     & H2(1,0)*(-5+omw-(u*v)/2._dp-(u*u)/2._dp)+
     & G1(3)*H1(0)*(4+omw-(u*v)/2._dp-(u*u)/2._dp)+
     & H2(0,0)*(4+omw-(u*v)/2._dp-(u*u)/2._dp)+
     & (G2(1,2)*(2+u*(-2+v)-2*v+u*u))/2._dp+
     & (G2(3,0)*(u*(-2+v)-2*(4+v)+u*u))/2._dp+
     & G1(2)*(H1(1)/2._dp+(omw+v/((-1+u)*(-1+u)))/2._dp+
     & (H1(0)*(3+u*(-2+v)-2*v+u*u))/2._dp)+
     & H1(0)*(H1(1)*(-3._dp/2._dp-u*(-2+v)+2*v-u*u)+
     & (375+u*(78-108/omw-30*v)+51*v-(9*v)/(omu*omu)-
     & 30*(u*u)+nf*(-2*(-5+v)+u*(-2+v)+u*u))/18._dp)+
     & (nf*(302-(4*(-4+omw)*u)/omw)-5*(nf*nf)+
     & 3*(-848+12/omu+(40*(-4+omw)*u)/omw+
     & 6*zeta2*(-45+8*u*(-2+v)-16*v+8*(u*u))))/72._dp+
     & (H1(1)*(nf*(omu*omu)*(1+u*(-2+v)-2*v+u*u)-
     & 3*(10-14*v+u*(-37+44*v)+(54-37*v)*(u*u)+
     & (-37+10*v)*(u*u*u)+10*(u*u*u*u))))/(18._dp*(omu*omu))-
     & (G1(2)*G1(2))/4._dp+(67._dp/4._dp-u*(-2+v)+2*v-u*u)*(H1(0)*H1(0))-
     & (H1(1)*H1(1))/4._dp)+G1(2)*G1(2)*
     & ((H1(1)*(33+3*omw-2*nf+(3*v)/((-1+u)*(-1+u))))/18._dp+
     & (G2(1,2)*(-2+2*u+2*v-u*v-u*u))/18._dp+
     & (G2(3,0)*(8+2*u+2*v-u*v-u*u))/18._dp+
     & (H2(1,0)*(-2*(-5+v)+u*(-2+v)+u*u))/18._dp+
     & (G2(0,2)*(1+u*(-2+v)-2*v+u*u))/18._dp+
     & (G2(2,0)*(1+u*(-2+v)-2*v+u*u))/18._dp+
     & (G1(3)*H1(0)*(u*(-2+v)-2*(4+v)+u*u))/18._dp+
     & (H2(0,0)*(u*(-2+v)-2*(4+v)+u*u))/18._dp+
     & H1(0)*((H1(1)*(7+2*u*(-2+v)-4*v+2*(u*u)))/18._dp+
     & (23-2*nf-3*v+u*(-4+4/omw+3*v)+(3*v)/(omu*omu)+
     & 3*(u*u))/18._dp)+(omu*omu*
     & (-76*nf+6*zeta2*(45-8*u*(-2+v)+16*v-8*(u*u)))+
     & 3*(361+24*v-6*u*(119+4*v)+(341+12*v)*(u*u)+12*(u*u*u)))/
     & (216._dp*(omu*omu))+((-7+4*u*(-2+v)-8*v+4*(u*u))*(H1(0)*H1(0)))/
     & 36._dp+(H1(1)*H1(1))/12._dp)+
     & ((H2(1,0)*(-2*(-5+v)+u*(-2+v)+u*u))/18._dp+
     & (H2(0,0)*(u*(-2+v)-2*(4+v)+u*u))/18._dp+
     & (omu*omu*(-76*nf+6*zeta2*(45-8*u*(-2+v)+16*v-8*(u*u)))+
     & 3*(361+24*v-6*u*(119+4*v)+(341+12*v)*(u*u)+12*(u*u*u)))/
     & (216._dp*(omu*omu)))*(H1(1)*H1(1))+
     & G2(1,2)*((2*H2(0,0)*(98-25*v+u*(-25+11*v)+11*(u*u)))/9._dp+
     & (2*H2(1,0)*(-64-29*v+u*(-29+16*v)+16*(u*u)))/9._dp+
     & (H1(1)*(-129+146*u+146*v-84*u*v+(34*v)/(omu*omu)+
     & 10*NFV*(-4+u*(11-4*v)+v*(11+6/(omu*omu))-4*(u*u))-
     & 84*(u*u)+4*nf*(2+u*(-2+v)-2*v+u*u)))/18._dp+
     & H1(0)*((H1(1)*(-2+2*u+2*v-u*v-u*u))/9._dp+
     & (u*(724-621*v)+4*(-57+85*v)+(-849+438*v)*(u*u)+
     & 4*(omu*omu)*(2*nf*(2+u*(-2+v)-2*v+u*u)+
     & 5*NFV*(omw-2*(1+u*v+u*u)))+(438-85*v)*(u*u*u)-
     & 85*(u*u*u*u))/(18._dp*(omu*omu)))+
     & ((-12*(8+45*NFV))/omu+(6*(-69+4*nf-60*NFV)*v)/(omu*omu)+
     & (60*omw*(-1+u)*(u*u)*(w*w)+
     & omv*(60*NFV*(2*u-w)*(omw*omw)*(u*u*u)+
     & 12*(-16+45*NFV)*(-1+u)*u*(w*w)+
     & omw*(w*(w*(1758-1113*v+4*nf*(-32+19*v)+
     & 6*(302-145*v)*zeta2)*(u*u)+
     & (54+w*(-1173+76*nf-870*zeta2))*(u*u*u)-
     & 6*w*(-17-13*v+v*v)+
     & u*(54+w*(-987+1608*v-4*nf*(-29+32*v)+
     & 6*(-522+302*v)*zeta2+6*(v*v))))+
     & 30*NFV*(w*(2+3*(7+4*v)*w)*(u*u)+16*w*(u*u*u)-
     & 3*(3+2*v)*(w*w)-2*u*(2-3*w+(13+3*v)*(w*w))))))/
     & (omv*omw*u*(w*w)))/108._dp+
     & (17*(2+u*(-2+v)-2*v+u*u)*(H1(0)*H1(0)))/18._dp+
     & ((-2+2*u+2*v-u*v-u*u)*(H1(1)*H1(1)))/18._dp)+
     & G2(3,0)*((H2(1,0)*(88+3*u*v+3*(u*u)))/9._dp+
     & (2*H2(0,0)*(-96-31*v+u*(-31+14*v)+14*(u*u)))/9._dp+
     & (H2(0,1)*(8+17*u*(-2+v)-34*v+17*(u*u)))/9._dp+
     & H1(0)*((H1(1)*(8+2*u+2*v-u*v-u*u))/9._dp+
     & (5*nf*(u*(-2+v)-2*(4+v)+u*u)+
     & 2*(169+72*v-81/w-18*(u*u)+
     & u*(55+64/omw-18*v+41/w-40/(w*w))+40/(w*w)))/18._dp)-
     & (omw*H1(1)*(omv*omv*omv*(-30+2*v+v*v)+
     & u*(96-231*v+(-15+NFV*(40-60*v)+29*v)*(u*u*u)+
     & (3-20*NFV)*(u*u*u*u)+2*(91+20*NFV)*(v*v)+
     & u*u*(57-96*v+68*(v*v)-20*NFV*(1-3*v+3*(v*v)))-
     & 4*(16+5*NFV)*(v*v*v)+
     & u*(-111+(206+20*NFV)*v-146*(v*v)-
     & 20*(-3+NFV)*(v*v*v))+17*(v*v*v*v))))/(18._dp*u*(w*w*w))+
     & (-27*omu*omv*omw*(36*nf*(1+2*w)-281*(1+(2+v)*w))*(w*w)+
     & 3*omv*omw*w*(omu*omu)*
     & (843-2529*w-(2222+1379*v)*(w*w)+108*nf*(-1+3*w+2*(w*w)))
     & +2*omv*omw*(-7587*(1+v)+
     & nf*(972+v*(572+390*u-66*(u*u)+76*(u*u*u))))*(w*w*w)+
     & omu*omu*omu*(60*omw*u*(1+u)*(w*w*w)+
     & omv*(-120*NFV*(omw*omw*omw)*(u*u*u)+
     & 48*(5-2*nf)*u*(w*w*w)+
     & omw*(w*(-2529+2160*w+8442*(w*w)+1392*v*(w*w)+
     & 1584*zeta2*(w*w)+3156*v*zeta2*(w*w)+6*(v*v)*(w*w)-
     & u*u*(54-108*w+(1173+1542*zeta2)*(w*w))+
     & u*(-1053+231*w+
     & (1710-1113*v+6*(526-257*v)*zeta2)*(w*w))+
     & 4*nf*(81-54*w-164*(w*w)+19*(u*u)*(w*w)+
     & u*(27-27*w+(-32+19*v)*(w*w))))+
     & 30*NFV*(-6*v*(w*w*w)+
     & 2*(u*u)*(1-9*w+13*(w*w)+w*w*w)+
     & u*(-4+7*w-20*(w*w)+(5+12*v)*(w*w*w)))))))/
     & (108._dp*omv*omw*(omu*omu*omu)*(w*w*w))+
     & (17*(u*(-2+v)-2*(4+v)+u*u)*(H1(0)*H1(0)))/18._dp+
     & ((8+2*u+2*v-u*v-u*u)*(H1(1)*H1(1)))/18._dp)+
     & G2(2,0)*((H2(1,0)*(-52-3*u*v-3*(u*u)))/9._dp+
     & (H2(0,1)*(28+u*(-2+v)-2*v+u*u))/9._dp-
     & (2*H2(0,0)*(39-13*u-13*v+5*u*v+5*(u*u)))/9._dp-
     & (H1(1)*(9*(-7+18*v)+u*(337+30*NFV-539*v-60*NFV*v)+
     & (-717-90*omv*NFV+693*v)*(u*u)+
     & 4*nf*(omu*omu*omu)*(1+u*(-2+v)-2*v+u*u)+
     & (759+130*NFV-400*v-110*NFV*v)*(u*u*u)+
     & 2*(-200-55*NFV+42*v+20*NFV*v)*(u*u*u*u)+
     & (84+40*NFV)*(u*u*u*u*u)))/(18._dp*(omu*omu*omu))+
     & H1(0)*((H1(1)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (630-280*u+(98*u)/omw-288*v+(3*v)/omu+(3*v)/u+
     & 143*u*v+4/w-(4*u)/w-(1129*v)/(omu*omu)+
     & (2529*v)/(omu*omu*omu)-(2529*v)/(omu*omu*omu*omu)+
     & 123*(u*u)+10*NFV*(2-4/omu+u*(-1+2*v)+2*(u*u)+
     & (v*(1+5*u-3*(u*u)+u*u*u))/(omu*omu*omu))+
     & nf*(-46-9*u*(-2+v)-9*(u*u)+
     & (18*v*(7+2*u+12*(u*u)-4*(u*u*u)+u*u*u*u))/
     & (omu*omu*omu*omu))+19*(v*v)-(2*(v*v))/u-(v*v*v)/u)/
     & 18._dp)+(3*omv*omw*w*((-2+19*v)*w+40*NFV*(1+w))*
     & (omu*omu)-720*omv*omw*NFV*v*(w*w)+
     & omu*omv*omw*(60*NFV*(2+9*v)+
     & v*(171+2*nf*(34-104*u+61*(u*u))))*(w*w)-
     & omu*omu*omu*(60*omw*(u*u)*(w*w)+
     & omv*(120*NFV*(omw*omw)*(u*u*u)+252*u*(w*w)+
     & omw*(w*(u*(-54+
     & w*(1872-1113*v+2*nf*(-61+38*v)+
     & 6*(166-77*v)*zeta2))+
     & (54+w*(-1173+76*nf-462*zeta2))*(u*u)+
     & w*(31*nf+6*(147+166*v)*zeta2+
     & 6*(-120+235*v+v*v)))+
     & 30*NFV*(2*w*(2+(4-3*v)*w)+2*w*(7+w)*(u*u)+
     & u*(4+(1+12*v)*(w*w)))))))/
     & (108._dp*omv*omw*(omu*omu*omu)*(w*w))-
     & (17*(1+u*(-2+v)-2*v+u*u)*(H1(0)*H1(0)))/18._dp+
     & ((1+u*(-2+v)-2*v+u*u)*(H1(1)*H1(1)))/18._dp)+
     & G2(0,2)*((H2(1,0)*(162+u*(40-23*v)+40*v-23*(u*u)))/9._dp+
     & (H2(0,1)*(u*(-2+v)-2*(4+v)+u*u))/9._dp-
     & (2*H2(0,0)*(94-33*v+3*u*(-11+5*v)+15*(u*u)))/9._dp-
     & (H1(1)*(9*(-7+18*v)+u*(337+30*NFV-539*v-60*NFV*v)+
     & (-717-90*omv*NFV+693*v)*(u*u)+
     & 4*nf*(omu*omu*omu)*(1+u*(-2+v)-2*v+u*u)+
     & (759+130*NFV-400*v-110*NFV*v)*(u*u*u)+
     & 2*(-200-55*NFV+42*v+20*NFV*v)*(u*u*u*u)+
     & (84+40*NFV)*(u*u*u*u*u)))/(18._dp*(omu*omu*omu))+
     & (3*omv*omw*w*((-2+19*v)*w+40*NFV*(1+w))*(omu*omu)-
     & 720*omv*omw*NFV*v*(w*w)+
     & omu*omv*omw*(60*NFV*(2+9*v)+
     & v*(171+2*nf*(34-104*u+61*(u*u))))*(w*w)-
     & omu*omu*omu*(60*omw*(u*u)*(w*w)+
     & omv*(120*NFV*(omw*omw)*(u*u*u)+252*u*(w*w)+
     & omw*(w*(u*(-54+
     & w*(1872-1113*v+2*nf*(-61+38*v)+
     & 6*(86-37*v)*zeta2))+
     & (54+w*(-1173+76*nf-222*zeta2))*(u*u)+
     & w*(31*nf+6*(-353+86*v)*zeta2+
     & 6*(-120+235*v+v*v)))+
     & 30*NFV*(2*w*(2+(4-3*v)*w)+2*w*(7+w)*(u*u)+
     & u*(4+(1+12*v)*(w*w)))))))/
     & (108._dp*omv*omw*(omu*omu*omu)*(w*w))+
     & H1(0)*((H1(1)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & ((32-90*NFV+59*u+60*NFV*u)/omw+
     & (2*(-16+45*NFV)*u)/(omw*omw)+
     & (20*omw*NFV*(u*u*u*u))/(w*w*w)-
     & (-40*NFV*v*(w*w*w)+2*omu*(9+10*NFV)*v*(w*w*w)+
     & 20*(2*NFV+3*v)*(omu*omu)*(w*w*w)+
     & omu*omu*omu*(w*
     & (u*w*(38+(270+9*nf*(-2+v)-152*v)*w)+
     & 9*(-1+w)*(u*u*u)+
     & (-103+362*v-2*nf*(-5+9*v))*(w*w)+
     & u*u*(9-56*w+3*(-38+3*nf)*(w*w)))+
     & 10*NFV*(u*(3-4*(7+v)*w)*(w*w)+
     & w*(u*u)*(6-15*w+2*(w*w))+
     & 2*(u*u*u)*(1-5*w+3*(w*w))+2*(1-5*v)*(w*w*w))))/
     & (omu*omu*omu*(w*w*w)))/18._dp)-
     & (17*(1+u*(-2+v)-2*v+u*u)*(H1(0)*H1(0)))/18._dp+
     & ((1+u*(-2+v)-2*v+u*u)*(H1(1)*H1(1)))/18._dp)+
     & G1(3)*(-2*omw*(-2+u)*H3(0,0,1)-
     & (8*H3(0,0,0)*(u*(-2+v)-2*(4+v)+u*u))/3._dp-
     & (2*H3(0,1,0)*(40-v+u*(-1+2*v)+2*(u*u)))/9._dp-
     & (2*H3(1,0,0)*(40-v+u*(-1+2*v)+2*(u*u)))/9._dp+
     & (zeta2*H1(1)*(2*(79+(-64+45*NFV)*v)+
     & u*(-128+67*v-90*NFV*(-1+2*v))+(67-180*NFV)*(u*u)))/9._dp+
     & (H3(1,1,0)*(30+2*(2+5*NFV)*v-u*(-4+v+10*NFV*(-1+2*v))-
     & (1+20*NFV)*(u*u)))/3._dp+
     & (H3(1,0,1)*(-10+(22-30*NFV)*v+u*(22-30*NFV-11*v+60*NFV*v)+
     & (-11+60*NFV)*(u*u)))/9._dp+
     & (H2(0,0)*(-548-186*v+164/w+75*(u*u)-
     & 4*nf*(u*(-2+v)-2*(4+v)+u*u)+
     & u*(-173-64/omw+75*v-84/w+80/(w*w))-80/(w*w)))/9._dp+
     & (H2(0,1)*((-30+NFV*(40-80*v)+44*v)*(u*u*u*u*u)+
     & (6-20*NFV)*(u*u*u*u*u*u)+
     & v*(omv*omv*omv)*(-30+2*v+v*v)+
     & u*u*u*u*(60-162*v+115*(v*v)-20*NFV*(1-5*v+6*(v*v)))+
     & u*u*u*(-60+263*v+5*(-61+12*NFV)*(v*v)-
     & 20*(-7+4*NFV)*(v*v*v))+
     & u*u*(30-237*v+(379+60*NFV)*(v*v)-(243+20*NFV)*(v*v*v)-
     & 20*(-4+NFV)*(v*v*v*v))+
     & u*(-6+122*v-272*(v*v)+(209+40*NFV)*(v*v*v)-
     & (69+20*NFV)*(v*v*v*v)+16*(v*v*v*v*v))))/(18._dp*u*(w*w*w))+
     & (H2(1,0)*((-30+NFV*(40-80*v)+44*v)*(u*u*u*u*u)+
     & (6-20*NFV)*(u*u*u*u*u*u)+
     & v*(omv*omv*omv)*(-30+2*v+v*v)+
     & u*u*u*u*(60-162*v+115*(v*v)-20*NFV*(1-5*v+6*(v*v)))+
     & u*u*u*(-60+263*v+5*(-61+12*NFV)*(v*v)-
     & 20*(-7+4*NFV)*(v*v*v))+
     & u*u*(30-237*v+(379+60*NFV)*(v*v)-(243+20*NFV)*(v*v*v)-
     & 20*(-4+NFV)*(v*v*v*v))+
     & u*(-6+122*v-272*(v*v)+(209+40*NFV)*(v*v*v)-
     & (69+20*NFV)*(v*v*v*v)+16*(v*v*v*v*v))))/(18._dp*u*(w*w*w))+
     & (48*zeta3*(-40-17*v+u*(-17+7*v)+7*(u*u))+
     & 6*zeta2*(-1211-554*u+(256*u)/omw-450*v+(204*v)/u+
     & 428*u*v+200/w-(103*u)/w+328*(u*u)-
     & 8*nf*(u*(-2+v)-2*(4+v)+u*u)+95*(v*v)-(10*(v*v))/u-
     & (5*(v*v*v))/u-97/(w*w)+(97*u)/(w*w)-
     & (10*NFV*(14-28*w+(14+11*v)*(w*w*w)+14*(u*u)*(w*w*w)+
     & u*(-14+14*w+(11+14*v)*(w*w*w))))/(w*w*w)))/108._dp+
     & (((-57+2*nf)*(u*(-2+v)-2*(4+v)+u*u))/18._dp+
     & (H1(1)*(u*(-2+v)-2*(4+v)+u*u))/9._dp)*(H1(0)*H1(0))-
     & (17*(u*(-2+v)-2*(4+v)+u*u)*(H1(0)*H1(0)*H1(0)))/18._dp+
     & H1(0)*((H1(1)*(u*(-2+v)-2*(4+v)+u*u))/6._dp+
     & (-7587*omu*omv*omw*(1+(2+v)*w)*(w*w)-
     & 2*omv*omw*w*(2*nf*
     & (-56+344*v+2*(82+19*v)*(u*u*u*u*u)-574*(v*v)+
     & u*u*u*u*(-712+219*v+76*(v*v))+
     & u*u*(-1316+1750*v+126*(v*v)-33*(v*v*v))+
     & 286*(v*v*v)+
     & u*u*u*(1370-743*v+22*(v*v)+38*(v*v*v))+
     & u*(550-1608*v+836*(v*v)+195*(v*v*v)))-
     & 7587*(1+v)*(w*w))+
     & 3*omv*omw*w*(omu*omu)*
     & (-843+2529*w+(2222+1379*v)*(w*w))+
     & omu*omu*omu*(-60*omw*u*(1+u)*(w*w*w)+
     & omv*(120*NFV*(omw*omw*omw)*(u*u*u)+
     & 48*(-5+2*nf)*u*(w*w*w)-
     & omw*(w*(-2529+2160*w+
     & 2*(66*(12+29*v)*zeta2+3*(1407+232*v+v*v))*
     & (w*w)+
     & u*u*(-54+108*w+
     & (76*nf-17*(69+102*zeta2))*(w*w))+
     & u*(-1053+231*w+
     & (1710-1113*v+6*(638-289*v)*zeta2)*(w*w)+
     & 4*nf*(27-27*w+(-32+19*v)*(w*w))))+
     & 30*NFV*(-6*v*(w*w*w)+
     & 2*(u*u)*(1-9*w+13*(w*w)+w*w*w)+
     & u*(-4+7*w-20*(w*w)+(5+12*v)*(w*w*w)))))))/
     & (108._dp*omv*omw*(omu*omu*omu)*(w*w*w))+
     & ((u*(-2+v)-2*(4+v)+u*u)*(H1(1)*H1(1)))/18._dp))+
     & G1(2)*(G2(0,2)*((-1-u*(-2+v)+2*v-u*u)/6._dp+
     & (H1(0)*(-1-u*(-2+v)+2*v-u*u))/9._dp+
     & (H1(1)*(-1-u*(-2+v)+2*v-u*u))/9._dp)+
     & G2(2,0)*((-1-u*(-2+v)+2*v-u*u)/6._dp+
     & (H1(0)*(-1-u*(-2+v)+2*v-u*u))/9._dp+
     & (H1(1)*(-1-u*(-2+v)+2*v-u*u))/9._dp)-
     & (2*H3(0,1,0)*(95+3*omv*u+3*v-3*(u*u)))/9._dp+
     & (10*H3(0,0,1)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & H3(0,0,0)*(16._dp/9._dp+3*u*(-2+v)-6*v+3*(u*u))+
     & (H3(1,0,0)*(20-44*v+u*(-44+25*v)+25*(u*u)))/9._dp+
     & (H3(1,0,1)*(u*(-2+NFV*(10-20*v)+v)+2*(5+(-1+5*NFV)*v)+
     & (1-20*NFV)*(u*u)))/3._dp+
     & (H3(1,1,0)*(-2*(49+(26+15*NFV)*v)+
     & u*(-52+23*v+30*NFV*(-1+2*v))+(23+60*NFV)*(u*u)))/9._dp+
     & G2(1,2)*((2+u*(-2+v)-2*v+u*u)/6._dp+
     & (H1(0)*(2+u*(-2+v)-2*v+u*u))/9._dp+
     & (H1(1)*(2+u*(-2+v)-2*v+u*u))/9._dp)+
     & G2(3,0)*((u*(-2+v)-2*(4+v)+u*u)/6._dp+
     & (H1(0)*(u*(-2+v)-2*(4+v)+u*u))/9._dp+
     & (H1(1)*(u*(-2+v)-2*(4+v)+u*u))/9._dp)+
     & H1(1)*((H2(1,0)*(2*(-5+v)-u*(-2+v)-u*u))/9._dp+
     & (H2(0,0)*(8+2*u+2*v-u*v-u*u))/9._dp+
     & (6*zeta2*(omu*omu)*(-289-4*(-4+45*NFV)*v+
     & 2*u*(8-7*v+90*NFV*(-1+2*v))+2*(-7+180*NFV)*(u*u))+
     & 8*nf*(5+6*v-u*(7+6*v)+(-1+3*v)*(u*u)+3*(u*u*u))+
     & 6*(-86+4*(-53+30*NFV)*v+
     & u*(67+214*v-80*NFV*(-1+2*v))+
     & (126+80*NFV*(-2+v)-107*v)*(u*u)+(-107+80*NFV)*(u*u*u)
     & ))/(108._dp*(omu*omu)))+
     & (H2(1,0)*((-32+NFV*(90-60*u)+39*u)/omw+
     & ((32-90*NFV)*u)/(omw*omw)+
     & (2529*omu*v*w+3*(-359+20*NFV)*v*w*(omu*omu)+
     & 63*v*w*(omu*omu*omu)+
     & 2*(omu*omu*omu*omu)*
     & (2+(187-14*nf-5*NFV*(-2+v)+90*v)*w+
     & u*(-2+(87+2*nf*(-2+v)-42*v-5*NFV*(13+4*v))*w)+
     & 2*(-21+nf-10*NFV)*w*(u*u))+
     & v*w*(-2529+4*nf*
     & (25+35*u+15*(u*u)+8*(u*u*u)-2*(u*u*u*u))))/
     & (w*(omu*omu*omu*omu))))/18._dp+
     & (H2(0,1)*(-26+32/omw-12*v-(18*v)/(omu*omu)+
     & u*(-34-20/omv+41/omw+26*v-32/(omw*omw))+
     & (6-10/(omv*omv))*(u*u)-
     & (10*NFV*(omw*(9-6*u)-9*u+
     & 2*(omw*omw)*(4+u*(-5+v)+v+u*u)))/(omw*omw)+
     & 19*(v*v)-(v*(-3+2*v+v*v))/u))/18._dp+
     & (4*nf*(28+57*u+57*v+(39*v)/(omu*omu)+
     & 18*zeta2*(-9-2*u*(-2+v)-2*(u*u)+
     & (v*(31+11*u+51*(u*u)-16*(u*u*u)+4*(u*u*u*u)))/
     & (omu*omu*omu*omu)))+
     & 3*(-532+498/omu-1215*u-1215*v-2556*zeta3+1680*u*zeta3+
     & 1680*v*zeta3-768*u*v*zeta3-(1461*v)/(omu*omu)-
     & 768*zeta3*(u*u)+10*NFV*
     & (-21-60/omu+36*v+6*u*(5+1/w)+(72*v)/(omu*omu)+
     & 6*zeta2*(40-24/omu+(27-18*u)/omw-29*u+22*u*v-
     & (27*u)/(omw*omw)+22*(u*u)+
     & (v*(10+21*(u*u)-7*(u*u*u)))/(omu*omu*omu)))+
     & 6*zeta2*(360-96/omw+4/w+
     & u*(-65+20/omv+35/omw+12*v-4/w+96/(omw*omw))+
     & (32+10/(omv*omv))*(u*u)-
     & (9*v*(138-25*u+221*(u*u)-76*(u*u*u)+23*(u*u*u*u)))/
     & (omu*omu*omu*omu)-19*(v*v)+(v*(-3+2*v+v*v))/u)))/
     & 324._dp+(H2(0,0)*(-956+534*u-(20*u)/omv-(90*u)/omw+544*v+
     & (63*v)/omu-(3*v)/u-260*u*v-4/w+(4*u)/w+
     & (1129*v)/(omu*omu)-(2529*v)/(omu*omu*omu)+
     & (2529*v)/(omu*omu*omu*omu)-240*(u*u)-
     & (10*(u*u))/(omv*omv)-10*NFV*(2-v+u*(-1+2*v)+2*(u*u))+
     & 12*nf*(5+u*(-2+v)+u*u-
     & (v*(11+u+21*(u*u)-8*(u*u*u)+2*(u*u*u*u)))/
     & (omu*omu*omu*omu))-19*(v*v)+(2*(v*v))/u+(v*v*v)/u))/
     & 18._dp+((H1(1)*(11-2*u*(-2+v)+4*v-2*(u*u)))/18._dp+
     & (v*(113-84*v+nf*(-2+4*v))+
     & (-211-4*nf*(-2+v)+114*v)*(u*u*u*u)+
     & (57-2*nf)*(u*u*u*u*u)+
     & u*u*u*(356-424*v+57*(v*v)-2*nf*(6-8*v+v*v))+
     & u*(105-407*v+255*(v*v)-2*nf*(1-6*v+5*(v*v)))+
     & u*u*(-307+619*v-213*(v*v)+nf*(8-22*v+8*(v*v))))/
     & (18._dp*omw*(omu*omu)))*(H1(0)*H1(0))+
     & G1(3)*(H1(0)*((8+2*u+2*v-u*v-u*u)/6._dp+
     & (H1(1)*(8+2*u+2*v-u*v-u*u))/9._dp)+
     & ((8+2*u+2*v-u*v-u*u)*(H1(0)*H1(0)))/9._dp)+
     & (17*(3+u*(-2+v)-2*v+u*u)*(H1(0)*H1(0)*H1(0)))/18._dp+
     & ((2*nf-(3*(v+(11+u+v)*(omu*omu)))/(omu*omu))*(H1(1)*H1(1)))/
     & 18._dp+H1(0)*((H1(1)*(-40+4*nf+u*(2-8/omw-3*v)-
     & (6*v)/(omu*omu)-3*(u*u)))/18._dp+
     & (H2(1,0)*(2*(-5+v)-u*(-2+v)-u*u))/9._dp+
     & (H2(0,0)*(8+2*u+2*v-u*v-u*u))/9._dp+
     & ((24-360*NFV)/omu+(12*(-51+4*nf-10*NFV)*v)/(omu*omu)+
     & (60*omw*(u*u*u)*(w*w)+
     & omv*(120*NFV*(omw*omw)*(u*u*u*u)+
     & 12*u*(16-45*NFV+15*u)*(w*w)+
     & omw*(w*((-108+
     & w*(1446-1113*v+4*nf*(-26+19*v)-
     & 30*(-22+5*v)*zeta2))*(u*u)+
     & (54+w*(-1173+76*nf-150*zeta2))*(u*u*u)-
     & 6*w*(-17-13*v+v*v)+
     & u*(54+
     & w*(-1995+906*v-52*nf*(-3+2*v)+
     & 30*(-127+22*v)*zeta2+6*(v*v))))+
     & 30*NFV*(-3*(3+2*v)*(w*w)+
     & 2*(u*u*u)*(-2+9*w+w*w)+
     & 2*u*(-2+3*w+(-7+5*v)*(w*w))+
     & u*u*(4-12*w+(25+12*v)*(w*w))))))/
     & (omv*omw*u*(w*w)))/108._dp+
     & ((-5-u*(-2+v)+2*v-u*u)*(H1(1)*H1(1)))/18._dp)-
     & (H1(1)*H1(1)*H1(1))/18._dp)+H1(0)*
     & (((-57+2*nf)*H2(1,0)*(-2*(-5+v)+u*(-2+v)+u*u))/18._dp+
     & ((-57+2*nf)*H2(0,0)*(u*(-2+v)-2*(4+v)+u*u))/18._dp+
     & H1(1)*((H2(1,0)*(-2*(-5+v)+u*(-2+v)+u*u))/9._dp+
     & (H2(0,0)*(u*(-2+v)-2*(4+v)+u*u))/9._dp+
     & (v*(1335-648*v+nf*(-94+24*v)+6*(-63+232*v)*zeta2)+
     & 2*(-171+6*nf-696*(-2+v)*zeta2)*(u*u*u*u)-
     & 696*zeta2*(u*u*u*u*u)+
     & u*u*u*(2103-666*v+2*nf*(-59+12*v)+
     & 6*zeta2*(-643+928*v-116*(v*v)))+
     & u*(1407-3672*v+6*zeta2*(-63+590*v-580*(v*v))+
     & nf*(-94+224*v-24*(v*v))+648*(v*v))+
     & u*u*(-3168+2679*v-324*(v*v)+2*nf*(100-71*v+6*(v*v))+
     & 6*zeta2*(358-1223*v+464*(v*v))))/(108._dp*omw*(omu*omu)))
     & +((9*(-927+116*nf-288*v*zeta2))/omu+
     & (54*zeta2*(-843*v+10*NFV*
     & (-(u*(-8+v))-3*v-2*(8+3*v)*(u*u)+2*(4+v)*(u*u*u))))
     & /(omu*omu*omu)-(18*v*zeta2*
     & (-2529+4*nf*(31+11*u+51*(u*u)-16*(u*u*u)+4*(u*u*u*u))))/
     & (omu*omu*omu*omu)-
     & (3*v*(30*NFV*((-39+22*v)*(u*u*u)+10*(u*u*u*u)+
     & u*(-73+104*v-24*(v*v))+3*(u*u)*(25-21*v+4*(v*v))+
     & 9*(3-7*v+4*(v*v)))+(52*nf-3*(487+2214*zeta2))*(w*w)
     & ))/(omu*omu*(w*w))+
     & (120*omw*u*(-6+6*u*zeta2)*(w*w*w)+
     & 12*omv*omw*(-27*nf+10*(21+2*u*(3+6*zeta2)))*(w*w*w)+
     & omv*omv*(-1800*NFV*zeta2*(omw*omw)*(u*u*u*u)+
     & omw*(180*NFV*u*zeta2*
     & (10*(-1+6*v)*(u*u*u)+30*(u*u*u*u)+
     & 10*(u*u)*(-1-v+3*(v*v))-
     & 3*(-5+4*(3+2*v)*w)*(w*w)+
     & 3*u*w*(10-25*w+2*(w*w)))+
     & w*(w*(-4*nf*(162+81*u+(-1145+57*v)*w)+
     & 3*(1422+
     & w*(-24989+1215*v+5676*zeta3-816*v*zeta3)+
     & 3*u*(369+16*(-17+7*v)*w*zeta3)+
     & 336*w*zeta3*(u*u)))+
     & 18*zeta2*(-240+506*w+45*(-1+w)*(u*u*u)+
     & (392-68*nf+219*v)*(w*w)+
     & u*u*(45-280*w+(165+8*nf)*(w*w))+
     & u*(240-76*w+(-488+8*nf*(-2+v)-25*v)*(w*w))))
     & )+24*u*(567-50*nf+60*zeta2)*(w*w*w)))/
     & (omw*(omv*omv)*(w*w*w)))/324._dp+
     & ((17-2*nf+(2*(2+omw)*u)/omw+3*v+(3*v)/(omu*omu))*
     & (H1(1)*H1(1)))/18._dp+(H1(1)*H1(1)*H1(1))/18._dp)+
     & ((33+3*omw-2*nf+(3*v)/((-1+u)*(-1+u)))*(H1(1)*H1(1)*H1(1)))/54._dp+
     & G1(0)*((H3(1,0,1)*(2*(-5+v)-u*(-2+v)-u*u))/9._dp+
     & (H3(1,1,0)*(-20+3*u*v+3*(u*u)))/9._dp+
     & (H3(0,1,0)*(88+3*u*v+3*(u*u)))/9._dp+
     & (2*H3(1,0,0)*(84-13*v+u*(-13+5*v)+5*(u*u)))/9._dp+
     & (2*H3(0,0,0)*(-96-31*v+u*(-31+14*v)+14*(u*u)))/9._dp+
     & (H3(0,0,1)*(8+17*u*(-2+v)-34*v+17*(u*u)))/9._dp+
     & G2(0,2)*(-((-30+nf)*(1+u*(-2+v)-2*v+u*u))/18._dp+
     & H1(0)*(1+u*(-2+v)-2*v+u*u))+
     & G2(2,0)*(-((-30+nf)*(1+u*(-2+v)-2*v+u*u))/18._dp+
     & H1(0)*(1+u*(-2+v)-2*v+u*u))+
     & G2(1,2)*(H1(0)*(-2+2*u+2*v-u*v-u*u)+
     & ((-30+nf)*(2+u*(-2+v)-2*v+u*u))/18._dp)+
     & G2(3,0)*(H1(0)*(8+2*u+2*v-u*v-u*u)+
     & ((-30+nf)*(u*(-2+v)-2*(4+v)+u*u))/18._dp)+
     & (H2(1,1)*(-60+156*v+u*(322+30*NFV-518*v-60*NFV*v)+
     & (-687-90*omv*NFV+666*v)*(u*u)+
     & 4*nf*(omu*omu*omu)*(1+u*(-2+v)-2*v+u*u)+
     & (729+130*NFV-385*v-110*NFV*v)*(u*u*u)+
     & (-385-110*NFV+81*v+40*NFV*v)*(u*u*u*u)+
     & (81+40*NFV)*(u*u*u*u*u)))/(18._dp*(omu*omu*omu))+
     & (6*zeta2*(-654+447*u-(20*u)/omv-(80*u)/omw+174*v+
     & (9*v)/omu-229*u*v-4/w+(4*u)/w+(2209*v)/(omu*omu)-
     & (5058*v)/(omu*omu*omu)+(5058*v)/(omu*omu*omu*omu)-
     & 229*(u*u)-(10*(u*u))/(omv*omv)-
     & (30*NFV*u*(1+u*(-2+v)-2*v+u*u)*(omu+4*(u*u)))/
     & (omu*omu*omu))+
     & 4*nf*(-391-19*u+(100*u)/omw+81/w+(27*u)/w-
     & (3*(27+55*w))/(omu*w)+243/(omu*omu)-
     & 6*zeta2*(-17+u*(-2+v)+u*u)+
     & (12*v*zeta2*(-26-31*u-21*(u*u)-4*(u*u*u)+u*u*u*u))/
     & (omu*omu*omu*omu))-
     & (3*(omw*(w*(2529*w-omu*(843+1645*w)+
     & omu*omu*(843-4*w*(1808-81*zeta3+4*v*zeta3)+
     & u*(369+w*(-405-16*zeta3+20*v*zeta3))+
     & 20*w*zeta3*(u*u)))+
     & 10*NFV*u*((-39+22*v)*(u*u*u)+10*(u*u*u*u)+
     & u*(-73+104*v-24*(v*v))+
     & 3*(u*u)*(25-21*v+4*(v*v))+9*(3-7*v+4*(v*v))))+
     & 1512*u*(omu*omu)*(w*w)))/(omw*(omu*omu)*(w*w)))/108._dp+
     & (H1(1)*(-702+6/omu+2070*u+(180*u)/omw+1590*v-(57*v)/omu-
     & 1113*u*v-(54*u)/w-198*zeta2+996*u*zeta2+996*v*zeta2-
     & 462*u*v*zeta2+(9*v)/(omu*omu)-1173*(u*u)+(60*(u*u))/omv+
     & (54*(u*u))/w-462*zeta2*(u*u)+
     & (4*nf*(1+u*(-2+v)-2*v+u*u)*(10-32*u+19*(u*u)))/
     & (omu*omu)+6*(v*v)+
     & (30*NFV*(-4*w*(1+w)*(omu*omu)+24*v*(w*w)-
     & 2*omu*(2+9*v)*(w*w)+
     & omu*omu*omu*(2*w*(2+4*w-3*v*w)-
     & 2*(-8+omw)*w*(u*u)+4*omw*(u*u*u)+
     & u*(4+w*w+12*v*(w*w)))))/(omu*omu*omu*(w*w))))/108._dp
     & +(H2(0,0)*(1196+266*u+(20*u)/omv+(192*u)/omw+484*v-
     & (3*v)/omu-230*u*v+4/w+(72*u)/w+(1111*v)/(omu*omu)-
     & (2529*v)/(omu*omu*omu)+(2529*v)/(omu*omu*omu*omu)-
     & 154*(u*u)-(112*(u*u))/w+(10*(u*u))/(omv*omv)+
     & (18*(u*u*u))/w+12*nf*
     & (-8+u*(-2+v)+u*u-
     & (v*(11+u+21*(u*u)-8*(u*u*u)+2*(u*u*u*u)))/
     & (omu*omu*omu*omu))+(18*(u*u))/(w*w)-
     & (18*(u*u*u))/(w*w)-
     & (20*NFV*u*v*(3*(1-2*v+u*(-4+3*v))*(omv*omv)+
     & u*u*(18-33*v+23*(v*v)+u*u*(3-3*v+2*(v*v))-
     & 7*(v*v*v)+u*(-12+15*v-10*(v*v)+2*(v*v*v)))))/
     & (omu*omu*omu*(w*w*w))))/18._dp+
     & (H2(1,0)*(-411+296*u-(98*u)/omw+382*v-(3*v)/omu-179*u*v-
     & 4/w+(42*u)/w+(1129*v)/(omu*omu)-
     & (2529*v)/(omu*omu*omu)+(2529*v)/(omu*omu*omu*omu)-
     & 141*(u*u)-(56*(u*u))/w+(9*(u*u*u))/w+
     & 2*nf*(23+5*u*(-2+v)+5*(u*u)-
     & (2*v*(32+7*u+57*(u*u)-20*(u*u*u)+5*(u*u*u*u)))/
     & (omu*omu*omu*omu))+(9*(u*u))/(w*w)-(9*(u*u*u))/(w*w)+
     & (10*NFV*u*(omu*omu*omu*
     & (-2*(1+omw-6*v)*(u*u*u)+6*(u*u*u*u)+
     & u*u*(-2-2*v+6*(v*v))+(3-4*(1+v)*w)*(w*w)+
     & u*w*(6-15*w+2*(w*w)))-
     & 2*(-2+u*(4-3*v)+4*v+(-2+v)*(u*u))*(w*w*w)))/
     & (omu*omu*omu*(w*w*w))))/18._dp+
     & (H2(0,1)*(-75+128*u+(20*u)/omv-(18*u)/omw+244*v-(3*v)/omu-
     & 132*u*v+(38*u)/w-v/(omu*omu)-94*(u*u)-(56*(u*u))/w+
     & (10*(u*u))/(omv*omv)+6*nf*(1+u*(-2+v)-2*v+u*u)+
     & (9*(u*u*u))/w+(9*(u*u))/(w*w)-(9*(u*u*u))/(w*w)+
     & (10*NFV*u*(omu*omu*omu*
     & (-2*(1+omw-6*v)*(u*u*u)+6*(u*u*u*u)+
     & u*u*(-2-2*v+6*(v*v))+(3-7*w-4*v*w)*(w*w)+
     & u*w*(6-15*w+2*(w*w)))+
     & (4-2*v-u*(8+3*v)+(4+v)*(u*u))*(w*w*w)))/
     & (omu*omu*omu*(w*w*w))))/18._dp+
     & ((16+u-(4*u)/omw)/18._dp+(H1(1)*(-1-u*(-2+v)+2*v-u*u))/18._dp+
     & (H1(0)*(17-u*(-2+v)+2*v-u*u))/18._dp)*(G1(2)*G1(2))+
     & ((H1(1)*(17+5*u*(-2+v)-10*v+5*(u*u)))/6._dp+
     & (v*(-719-78*v+nf*(-2+4*v))+
     & (-245-4*nf*(-2+v)+114*v)*(u*u*u*u)+
     & (57-2*nf)*(u*u*u*u*u)-
     & u*u*u*(260+455*v-57*(v*v)+2*nf*(6-8*v+v*v))+
     & u*(-579+1229*v+249*(v*v)-2*nf*(1-6*v+5*(v*v)))+
     & u*u*(1027-151*v-210*(v*v)+nf*(8-22*v+8*(v*v))))/
     & (18._dp*omw*(omu*omu)))*(H1(0)*H1(0))+
     & G1(1)*(-((-30+nf)*H1(1)*(2+u*(-2+v)-2*v+u*u))/18._dp+
     & H1(0)*(-((-30+nf)*(2+u*(-2+v)-2*v+u*u))/18._dp+
     & H1(1)*(2+u*(-2+v)-2*v+u*u))+
     & (2+u*(-2+v)-2*v+u*u)*(H1(0)*H1(0)))+
     & G1(3)*(-((-30+nf)*H1(0)*(u*(-2+v)-2*(4+v)+u*u))/18._dp+
     & (u*(-2+v)-2*(4+v)+u*u)*(H1(0)*H1(0)))+
     & (17*(-17+u*(-2+v)-2*v+u*u)*(H1(0)*H1(0)*H1(0)))/18._dp+
     & ((13+u*(7-4/omw-3*v)+6*v-3*(u*u))*(H1(1)*H1(1)))/18._dp+
     & H1(0)*(H2(1,0)*(-2*(-5+v)+u*(-2+v)+u*u)+
     & H2(0,0)*(u*(-2+v)-2*(4+v)+u*u)+
     & (H1(1)*(v*(113-72*v+nf*(-2+4*v))-
     & 4*(49+nf*(-2+v)-27*v)*(u*u*u*u)-
     & 2*(-27+nf)*(u*u*u*u*u)+
     & u*u*u*(335-394*v+54*(v*v)-2*nf*(6-8*v+v*v))+
     & u*(105-386*v+234*(v*v)-2*nf*(1-6*v+5*(v*v)))+
     & u*u*(-298+577*v-198*(v*v)+nf*(8-22*v+8*(v*v)))))/
     & (18._dp*omw*(omu*omu))+
     & (-(omu*omv*omw*(3*(-2529+
     & (-3*(1686+v)+20*NFV*(2+9*v))*w)+
     & 4*nf*(243+w*(486+v*(20-55*u+32*(u*u)))))*(w*w))+
     & 3*omv*omw*w*(omu*omu)*
     & (843-(2529+40*NFV)*w+(-2258+20*NFV-19*v)*(w*w)+
     & 108*nf*(-1+3*w+2*(w*w)))+
     & 18*omv*omw*(-843+108*nf+40*NFV*v)*(w*w*w)+
     & omu*omu*omu*(60*omw*(u*u)*(w*w*w)+
     & omv*(-120*NFV*(omw*omw*omw)*(u*u*u)+
     & 96*(-20+nf)*u*(w*w*w)+
     & omw*(w*(-2529+1680*w+18858*(w*w)+1596*v*(w*w)+
     & 4662*zeta2*(w*w)+2724*v*zeta2*(w*w)+
     & 6*(v*v)*(w*w)-
     & u*u*(54-108*w+17*(69+78*zeta2)*(w*w))+
     & u*(-1053+711*w+
     & (2022-1113*v+6*(454-221*v)*zeta2)*(w*w))+
     & 4*nf*(81-54*w-374*(w*w)+19*(u*u)*(w*w)+
     & u*(27-27*w+19*(-2+v)*(w*w))))+
     & 30*NFV*(2*(2+w-3*v*w)*(w*w)+
     & 2*(u*u)*(1-7*w+11*(w*w)+w*w*w)+
     & u*(-4+11*w-10*(w*w)+(1+12*v)*(w*w*w)))))))/
     & (108._dp*omv*omw*(omu*omu*omu)*(w*w*w))+
     & ((5-u*(-2+v)+2*v-u*u)*(H1(1)*H1(1)))/6._dp)+
     & G1(2)*((-6+(-66+24/omw)*u-60*v+
     & nf*(-3+2*u+2*v+(2*v)/(omu*omu))-(60*v)/(omu*omu))/36._dp
     & +(H1(1)*(-(v*(29+6*v))+(-8+6*v)*(u*u)+3*(u*u*u)+
     & u*(-21-14*v+3*(v*v))))/(18._dp*omw)+
     & H1(0)*((2*H1(1)*(u*(-2+v)-2*(4+v)+u*u))/9._dp+
     & (2*v*(-58+nf+9*v-nf*v)+
     & 2*(44+nf*(-2+v)-27*v)*(u*u*u*u)+
     & (-27+nf)*(u*u*u*u*u)+
     & u*u*u*(-203+178*v-27*(v*v)+nf*(7-8*v+v*v))+
     & u*(-108+284*v-99*(v*v)+nf*(2-8*v+5*(v*v)))+
     & u*u*(-2*nf*(3-6*v+2*(v*v))+10*(25-31*v+9*(v*v))))/
     & (18._dp*omw*(omu*omu)))+
     & ((-35-8*u*(-2+v)+16*v-8*(u*u))*(H1(0)*H1(0)))/9._dp+
     & ((1+u*(-2+v)-2*v+u*u)*(H1(1)*H1(1)))/9._dp)+
     & ((-1-u*(-2+v)+2*v-u*u)*(H1(1)*H1(1)*H1(1)))/18._dp)+
     & G1(1)*(-2*H3(0,0,1)*(2+u*(-2+v)-2*v+u*u)-
     & (2*H3(0,1,0)*(-82-11*v+u*(-11+7*v)+7*(u*u)))/9._dp-
     & (2*H3(0,0,0)*(98-25*v+u*(-25+11*v)+11*(u*u)))/9._dp-
     & (2*H3(1,0,0)*(98-25*v+u*(-25+11*v)+11*(u*u)))/9._dp-
     & (2*H3(1,1,0)*(-64-29*v+u*(-29+16*v)+16*(u*u)))/9._dp+
     & (H2(1,1)*(123-140*u-140*v+81*u*v-(34*v)/(omu*omu)+81*(u*u)-
     & 4*nf*(2+u*(-2+v)-2*v+u*u)+
     & 10*NFV*(4+u*(-11+4*v)+v*(-11-6/(omu*omu))+4*(u*u))))/
     & 18._dp+(H2(1,0)*(171-191*u-191*v+71*u*v-(36*v)/(omu*omu)+
     & 71*(u*u)-5*nf*(2+u*(-2+v)-2*v+u*u)-
     & 10*NFV*(omw-2*(1+u*v+u*u))))/9._dp+
     & (H1(1)*((96+540*NFV)/omu+(6*(69-4*nf+60*NFV)*v)/(omu*omu)-
     & (60*omw*(-1+u)*(u*u)*(w*w)+
     & omv*(120*NFV*(omw*omw)*(u*u*u*u)+
     & 12*(-16+45*NFV)*(-1+u)*u*(w*w)+
     & omw*(w*(w*(1758-1113*v+4*nf*(-32+19*v)+
     & 6*(302-145*v)*zeta2)*(u*u)+
     & (54+w*(-1173+76*nf-870*zeta2))*(u*u*u)-
     & 6*w*(-17-13*v+v*v)+
     & u*(54+w*
     & (-987+1608*v-4*nf*(-29+32*v)+
     & 6*(-522+302*v)*zeta2+6*(v*v))))+
     & 30*NFV*(w*(2+3*(7+4*v)*w)*(u*u)+
     & 2*w*(7+w)*(u*u*u)-3*(3+2*v)*(w*w)-
     & 2*u*(2-3*w+(13+3*v)*(w*w))))))/
     & (omv*omw*u*(w*w))))/108._dp-
     & (H2(0,0)*(27*v*(omv*omv*omv)+
     & (-878+nf*(60-48*v)+676*v+10*NFV*(-7+8*v))*(u*u*u*u*u)-
     & 4*(-41+3*nf-5*NFV)*(u*u*u*u*u*u)+
     & u*u*u*u*(2052-3104*v+1053*(v*v)-
     & 12*nf*(11-17*v+6*(v*v))+10*NFV*(11-22*v+12*(v*v)))+
     & u*u*u*(-2528+5769*v+12*nf*(13-4*v)*(omv*omv)-
     & 4013*(v*v)+734*(v*v*v)+
     & 10*NFV*(-11+30*v-27*(v*v)+10*(v*v*v)))+
     & u*(-402+1553*v+24*nf*(omv*omv*omv*omv)-2321*(v*v)+
     & 1600*(v*v*v)-430*(v*v*v*v)-
     & 20*NFV*(1-5*v+9*(v*v)-5*(v*v*v)+v*v*v*v))+
     & u*u*(1592-4921*v+12*nf*(-8+v)*(omv*omv*omv)+
     & 5344*(v*v)-2217*(v*v*v)+193*(v*v*v*v)+
     & 10*NFV*(7-26*v+27*(v*v)-14*(v*v*v)+4*(v*v*v*v)))))/
     & (18._dp*u*(w*w*w))+(6*zeta2*
     & (216*v*(omv*omv*omv)+
     & (1745-1124*v-4*nf*(-7+4*v)+50*NFV*(-31+16*v))*
     & (u*u*u*u*u*u*u)+(-261-4*nf+200*NFV)*(u*u*u*u*u*u*u*u)+
     & u*u*u*u*u*u*(-5479+6669*v-1842*(v*v)-
     & 4*nf*(16-25*v+6*(v*v))+50*NFV*(99-112*v+24*(v*v)))+
     & u*u*u*u*u*(10355-18681*v+9485*(v*v)-1356*(v*v*v)-
     & 4*nf*(-10+50*v-33*(v*v)+4*(v*v*v))+
     & 10*NFV*(-826+1543*v-738*(v*v)+72*(v*v*v)))+
     & u*u*u*u*(-12395+30698*v-23108*(v*v)+5907*(v*v*v)-
     & 377*(v*v*v*v)-4*nf*
     & (-15-30*v+57*(v*v)-19*(v*v*v)+v*v*v*v)+
     & 10*NFV*(754-2160*v+1707*(v*v)-408*(v*v*v)+12*(v*v*v*v))
     & )+u*(669-4077*v-8*nf*(2+v)*(omv*omv*omv)+
     & 7625*(v*v)-5623*(v*v*v)+1406*(v*v*v*v)-
     & 10*NFV*(-4-67*v+225*(v*v)-201*(v*v*v)+63*(v*v*v*v)))+
     & u*u*u*(9151-30054*v+31458*(v*v)-12061*(v*v*v)+
     & 1346*(v*v*v*v)+
     & 4*nf*(-29+20*v+33*(v*v)-28*(v*v*v)+4*(v*v*v*v))-
     & 10*NFV*(351-1598*v+1905*(v*v)-765*(v*v*v)+75*(v*v*v*v))
     & )+u*u*(-3785+16353*v+
     & 4*nf*(omv*omv)*(18+5*v-5*(v*v))-22970*(v*v)+
     & 12485*(v*v*v)-2155*(v*v*v*v)+
     & 10*NFV*(59-568*v+1041*(v*v)-630*(v*v*v)+114*(v*v*v*v)))
     & )+24*u*zeta3*(omu*omu)*
     & (-64-29*v+u*(-29+16*v)+16*(u*u))*(w*w*w))/
     & (108._dp*u*(omu*omu)*(w*w*w))+
     & (H2(0,1)*(2*(19-30*NFV)*u*v*(w*w*w)+
     & omu*omu*(w*(-9*(-1+w)*(u*u*u*u)-27*v*(w*w)+
     & u*u*u*(-9+56*w+(94-6*nf)*(w*w))+
     & u*u*(9-49*w-2*(53+3*nf*(-2+v)-66*v)*(w*w))+
     & u*(-9+20*w+4*(43+3*nf*(-1+v)-47*v)*(w*w)))+
     & 10*NFV*u*(2-4*w+2*omw*(u*u*u*u)+
     & w*(u*u)*(-6+15*w-2*(w*w))-
     & 2*(u*u*u)*(1-5*w+3*(w*w))+(4-11*v)*(w*w*w)+
     & u*(-2+2*w-3*(w*w)+(-5+4*v)*(w*w*w))))))/
     & (18._dp*u*(omu*omu)*(w*w*w))+
     & ((H1(0)*(2+u*(-2+v)-2*v+u*u))/18._dp+
     & (H1(1)*(2+u*(-2+v)-2*v+u*u))/18._dp)*(G1(2)*G1(2))+
     & (((-57+2*nf)*(2+u*(-2+v)-2*v+u*u))/18._dp-
     & (5*H1(1)*(2+u*(-2+v)-2*v+u*u))/6._dp)*(H1(0)*H1(0))-
     & (17*(2+u*(-2+v)-2*v+u*u)*(H1(0)*H1(0)*H1(0)))/18._dp+
     & ((2+u*(-2+v)-2*v+u*u)*(H1(1)*H1(1)))/6._dp+
     & G1(2)*((H1(1)*(-2+2*u+2*v-u*v-u*u))/6._dp+
     & H1(0)*((-2+2*u+2*v-u*v-u*u)/6._dp-
     & (2*H1(1)*(2+u*(-2+v)-2*v+u*u))/9._dp)+
     & ((-2+2*u+2*v-u*v-u*u)*(H1(0)*H1(0)))/9._dp+
     & ((-2+2*u+2*v-u*v-u*u)*(H1(1)*H1(1)))/9._dp)+
     & H1(0)*(((-27+nf)*H1(1)*(2+u*(-2+v)-2*v+u*u))/9._dp+
     & ((96+540*NFV)/omu+(6*(69-4*nf+60*NFV)*v)/(omu*omu)-
     & (60*omw*(-1+u)*(u*u)*(w*w)+
     & omv*(120*NFV*(omw*omw)*(u*u*u*u)+
     & 12*(-16+45*NFV)*(-1+u)*u*(w*w)+
     & omw*(w*(w*(1758-1113*v+4*nf*(-32+19*v)+
     & 6*(158-73*v)*zeta2)*(u*u)+
     & (54+w*(-1173+76*nf-438*zeta2))*(u*u*u)-
     & 6*w*(-17-13*v+v*v)+
     & u*(54+
     & w*(-987+1608*v-4*nf*(-29+32*v)+
     & 12*(-189+79*v)*zeta2+6*(v*v))))+
     & 30*NFV*(w*(2+3*(7+4*v)*w)*(u*u)+
     & 2*w*(7+w)*(u*u*u)-3*(3+2*v)*(w*w)-
     & 2*u*(2-3*w+(13+3*v)*(w*w))))))/
     & (omv*omw*u*(w*w)))/108._dp+
     & ((2+u*(-2+v)-2*v+u*u)*(H1(1)*H1(1)))/6._dp)+
     & ((2+u*(-2+v)-2*v+u*u)*(H1(1)*H1(1)*H1(1)))/18._dp)+
     & (H1(1)*H1(1)*H1(1)*H1(1))/72._dp
      return
      end
      
      
      function Beta_3a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-20*pi*G3(1,1,0))/9._dp-(2*omw*pi*(omu+5*NFV*(-1+2*u))*H3(1,1,0))/3._dp+
     & (pi*G3(0,0,2)*(2-7*u*(-2+v)+14*v-7*(u*u)))/9._dp+
     & (pi*G3(0,2,0)*(2-7*u*(-2+v)+14*v-7*(u*u)))/9._dp+
     & (pi*G3(0,2,2)*(2-7*u*(-2+v)+14*v-7*(u*u)))/9._dp+
     & (pi*G3(2,0,0)*(2-7*u*(-2+v)+14*v-7*(u*u)))/9._dp+
     & (pi*G3(2,0,2)*(2-7*u*(-2+v)+14*v-7*(u*u)))/9._dp+
     & (pi*G3(2,2,0)*(2-7*u*(-2+v)+14*v-7*(u*u)))/9._dp+
     & (pi*G3(2,3,3)*(u*(4-5*v)+4*(40+v)-5*(u*u)))/9._dp-
     & (pi*H3(1,0,1)*(-2*(-5+v)+u*(-2+v)+u*u))/9._dp+
     & (16*pi*G3(1,0,2)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (16*pi*G3(1,2,0)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (16*pi*G3(1,2,2)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (16*pi*G3(3,0,0)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (16*pi*G3(3,0,2)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (16*pi*G3(3,2,0)*(1+u*(-2+v)-2*v+u*u))/9._dp-
     & 2*pi*G3(0,1,0)*(2+u*(-2+v)-2*v+u*u)+
     & pi*G3(2,1,0)*(2+u*(-2+v)-2*v+u*u)-
     & 4*pi*G3(2,1,1)*(2+u*(-2+v)-2*v+u*u)+
     & pi*G3(2,1,2)*(2+u*(-2+v)-2*v+u*u)-
     & (pi*G3(0,3,0)*(u*(-2+v)-2*(4+v)+u*u))/9._dp-
     & (pi*G3(0,3,2)*(u*(-2+v)-2*(4+v)+u*u))/9._dp-
     & (16*pi*G3(0,3,3)*(u*(-2+v)-2*(4+v)+u*u))/9._dp-
     & (2*pi*G3(1,0,3)*(80-7*v+u*(-7+2*v)+2*(u*u)))/9._dp-
     & (2*pi*G3(1,2,3)*(80-7*v+u*(-7+2*v)+2*(u*u)))/9._dp+
     & (pi*G3(2,3,0)*(3*u*(-4+v)-2*(5+6*v)+3*(u*u)))/9._dp-
     & (2*pi*G3(2,3,2)*(13+4*u*(-2+v)-8*v+4*(u*u)))/9._dp+
     & (2*pi*G3(2,0,3)*(49-15*v+3*u*(-5+2*v)+6*(u*u)))/9._dp+
     & (2*pi*G3(0,2,3)*(94-15*v+3*u*(-5+2*v)+6*(u*u)))/9._dp+
     & (2*pi*G3(3,3,3)*(-64-9*v+u*(-9+6*v)+6*(u*u)))/9._dp+
     & (2*pi*G3(3,2,3)*(-40-17*v+u*(-17+7*v)+7*(u*u)))/9._dp-
     & (2*pi*G3(3,3,0)*(-64-23*v+u*(-23+10*v)+10*(u*u)))/9._dp+
     & (2*pi*H3(1,0,0)*(89-23*v+u*(-23+10*v)+10*(u*u)))/9._dp+
     & (2*pi*G3(0,0,3)*(-72-25*v+u*(-25+11*v)+11*(u*u)))/9._dp-
     & (pi*G3(2,2,3)*(36-32*v+u*(-32+13*v)+13*(u*u)))/9._dp-
     & (2*pi*G3(3,0,3)*(-104-33*v+3*u*(-11+5*v)+15*(u*u)))/9._dp+
     & (pi*H3(0,0,1)*(8+17*u*(-2+v)-34*v+17*(u*u)))/9._dp+
     & (2*pi*H3(0,0,0)*(-96-49*v+u*(-49+23*v)+23*(u*u)))/9._dp+
     & (pi*G3(3,0,1)*(10+u*(NFV*(30-60*v)+11*(-2+v))+(-22+30*NFV)*v+
     & (11-60*NFV)*(u*u)))/9._dp+
     & (pi*G3(3,2,1)*(10+u*(NFV*(30-60*v)+11*(-2+v))+(-22+30*NFV)*v+
     & (11-60*NFV)*(u*u)))/9._dp+
     & (pi*G3(1,1,1)*(u*(NFV*(30-60*v)+57*(-2+v))+
     & 2*(56+3*(-19+5*NFV)*v)+(57-60*NFV)*(u*u)))/9._dp-
     & (pi*G3(1,2,1)*(32-30*(1+NFV)*v+15*u*(-2+v+NFV*(-2+4*v))+
     & 15*(1+4*NFV)*(u*u)))/9._dp-
     & (pi*G3(1,1,2)*(52-30*(1+NFV)*v+15*u*(-2+v+NFV*(-2+4*v))+
     & 15*(1+4*NFV)*(u*u)))/9._dp+
     & (2*pi*G3(0,1,2)*(-10-3*(-4+5*NFV)*v+
     & 3*u*(4-2*v+5*NFV*(-1+2*v))+6*(-1+5*NFV)*(u*u)))/9._dp-
     & (2*pi*G3(0,0,1)*(8-3*(-4+5*NFV)*v+3*u*(4-2*v+5*NFV*(-1+2*v))+
     & 6*(-1+5*NFV)*(u*u)))/9._dp+
     & (10*pi*G3(0,1,1)*(-2-3*(-2+NFV)*v+u*(6-3*v+NFV*(-3+6*v))+
     & (-3+6*NFV)*(u*u)))/9._dp+
     & (pi*G3(1,0,1)*(-20-6*(-7+5*NFV)*v+
     & 3*u*(14-10*NFV-7*v+20*NFV*v)+3*(-7+20*NFV)*(u*u)))/9._dp-
     & (2*pi*H3(0,1,0)*(-49+(11-15*NFV)*v+u*(11-7*v+15*NFV*(-1+2*v))+
     & (-7+30*NFV)*(u*u)))/9._dp+
     & (pi*G3(2,2,1)*(-4+(6-30*NFV)*v+u*(6-3*v+30*NFV*(-1+2*v))+
     & (-3+60*NFV)*(u*u)))/9._dp+
     & (pi*G3(0,2,1)*(8-2*(7+15*NFV)*v+u*(7*(-2+v)+30*NFV*(-1+2*v))+
     & (7+60*NFV)*(u*u)))/9._dp+
     & (pi*G3(2,0,1)*(-2*(-9+(7+15*NFV)*v)+
     & u*(7*(-2+v)+30*NFV*(-1+2*v))+(7+60*NFV)*(u*u)))/9._dp+
     & G2(0,1)*((-2*pi*H1(0)*(-19-15*(-2+NFV)*v+
     & 15*u*(2-v+NFV*(-1+2*v))+15*(-1+2*NFV)*(u*u)))/9._dp-
     & (2*pi*H1(1)*(-10-3*(-4+5*NFV)*v+3*u*(4-2*v+5*NFV*(-1+2*v))+
     & 6*(-1+5*NFV)*(u*u)))/9._dp+
     & (pi*(85+32/omw-256*u+(20*u)/omv+(41*u)/omw-232*v-
     & (63*v)/omu+77*u*v-(19*v)/(omu*omu)+
     & 30*NFV*(-2+7*u+(-3+2*u)/omw+3*v+v/(omu*omu)+
     & (3*u)/(omw*omw))-(32*u)/(omw*omw)+
     & nf*(-6-5*u*(-2+v)+10*v-5*(u*u))+77*(u*u)+
     & (10*(u*u))/(omv*omv)))/18._dp)+
     & G2(0,0)*((16*pi*H1(0)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (pi*H1(1)*(7*u*(-2+v)-2*(1+7*v)+7*(u*u)))/9._dp+
     & (pi*(-11+u*(55-13/omw-18*v)+72*v-18*(u*u)+
     & 3*nf*(1+u*(-2+v)-2*v+u*u)))/9._dp)+
     & G2(2,3)*(12*pi*H1(0)+(2*pi*H1(1)*(13+4*u*(-2+v)-8*v+4*(u*u)))/9._dp+
     & (pi*(588-112*v-(63*v)/omu+
     & u*(-142+20/omv+80/omw+73*v-4/w)+4/w-
     & (1111*v)/(omu*omu)+(2529*v)/(omu*omu*omu)-
     & (2529*v)/(omu*omu*omu*omu)+(73+10/(omv*omv))*(u*u)+
     & 36*nf*(-1+(3*v*(1+u+u*u))/(omu*omu*omu*omu))))/18._dp)+
     & (pi*H2(1,0)*(-47+32/omw+222*u-(20*u)/omv-(57*u)/omw+252*v-
     & (3*v)/omu-105*u*v-4/w+(4*u)/w+(1111*v)/(omu*omu)-
     & (2529*v)/(omu*omu*omu)+(2529*v)/(omu*omu*omu*omu)-
     & (32*u)/(omw*omw)+10*NFV*
     & (-10+4/omu-9/omw-
     & (v*(4-2*omu+omu*omu*omu))/(omu*omu*omu)+
     & u*(11+6/omw-4*v+9/(omw*omw))-4*(u*u))-105*(u*u)-
     & (10*(u*u))/(omv*omv)+
     & 6*nf*(1+u*(-2+v)+u*u-
     & (2*v*(10+5*u+15*(u*u)-4*(u*u*u)+u*u*u*u))/
     & (omu*omu*omu*omu))))/18._dp+
     & (pi*H2(1,1)*(-60+156*v+u*(322+30*NFV-518*v-60*NFV*v)+
     & (-687-90*omv*NFV+666*v)*(u*u)+
     & 4*nf*(omu*omu*omu)*(1+u*(-2+v)-2*v+u*u)+
     & (729+130*NFV-385*v-110*NFV*v)*(u*u*u)+
     & (-385-110*NFV+81*v+40*NFV*v)*(u*u*u*u)+
     & (81+40*NFV)*(u*u*u*u*u)))/(18._dp*(omu*omu*omu))+
     & G2(2,2)*((-2*pi*H1(0)*(8+3*omv*u+3*v+15*NFV*(-1+2*u)*(u+v)-
     & 3*(u*u)))/9._dp+(pi*(-60+156*v+
     & u*(322+30*NFV-518*v-60*NFV*v)+
     & (-687-90*omv*NFV+666*v)*(u*u)+
     & 4*nf*(omu*omu*omu)*(1+u*(-2+v)-2*v+u*u)+
     & (729+130*NFV-385*v-110*NFV*v)*(u*u*u)+
     & (-385-110*NFV+81*v+40*NFV*v)*(u*u*u*u)+
     & (81+40*NFV)*(u*u*u*u*u)))/(18._dp*(omu*omu*omu)))+
     & G2(2,0)*((pi*H1(0)*(-21+u*(-8+v)-8*v+u*u))/9._dp+
     & (pi*H1(1)*(7*u*(-2+v)-2*(1+7*v)+7*(u*u)))/9._dp-
     & (pi*(v*(13-162*v+5*nf*(-1+2*v))+
     & (258+NFV*(110-80*v)-96*v+5*nf*(-5+2*v))*(u*u*u*u*u)+
     & (5*nf-8*(6+5*NFV))*(u*u*u*u*u*u)+
     & u*u*u*u*(-525+550*v-48*(v*v)+5*nf*(10-10*v+v*v)-
     & 10*NFV*(13-22*v+4*(v*v)))+
     & u*(39-15*(21+2*NFV)*v+(503+60*NFV)*(v*v)-
     & 5*nf*(1-7*v+7*(v*v)))+
     & u*u*(-231+932*v-585*(v*v)-30*NFV*(1-5*v+3*(v*v))+
     & 5*nf*(5-17*v+9*(v*v)))+
     & u*u*u*(507-1084*v+292*(v*v)-5*nf*(10-19*v+5*(v*v))+
     & 10*NFV*(9-22*v+11*(v*v)))))/(18._dp*omw*(omu*omu*omu)))+
     & G2(0,2)*((pi*H1(0)*(159+u*(-8+v)-8*v+u*u))/9._dp+
     & (pi*H1(1)*(7*u*(-2+v)-2*(1+7*v)+7*(u*u)))/9._dp-
     & (pi*(v*(13-162*v+5*nf*(-1+2*v))+
     & (258+NFV*(110-80*v)-96*v+5*nf*(-5+2*v))*(u*u*u*u*u)+
     & (5*nf-8*(6+5*NFV))*(u*u*u*u*u*u)+
     & u*u*u*u*(-525+550*v-48*(v*v)+5*nf*(10-10*v+v*v)-
     & 10*NFV*(13-22*v+4*(v*v)))+
     & u*(39-15*(21+2*NFV)*v+(503+60*NFV)*(v*v)-
     & 5*nf*(1-7*v+7*(v*v)))+
     & u*u*(-231+932*v-585*(v*v)-30*NFV*(1-5*v+3*(v*v))+
     & 5*nf*(5-17*v+9*(v*v)))+
     & u*u*u*(507-1084*v+292*(v*v)-5*nf*(10-19*v+5*(v*v))+
     & 10*NFV*(9-22*v+11*(v*v)))))/(18._dp*omw*(omu*omu*omu)))+
     & G2(2,1)*(pi*H1(0)*(2+u*(-2+v)-2*v+u*u)-
     & pi*H1(1)*(2+u*(-2+v)-2*v+u*u)+
     & (pi*(97+32/omw-174*u-(20*u)/omv+(41*u)/omw-152*v+
     & (3*v)/u+107*u*v-(52*v)/(omu*omu)-(32*u)/(omw*omw)+
     & 87*(u*u)-(10*(u*u))/(omv*omv)-
     & 4*nf*(2+u*(-2+v)-2*v+u*u)+
     & 10*NFV*(-4+(-9+6*u)/omw-13*v+u*(-1+2*v)-
     & (6*v)/(omu*omu)+(9*u)/(omw*omw)+2*(u*u))+19*(v*v)-
     & (2*(v*v))/u-(v*v*v)/u))/18._dp)+
     & (pi*H1(1)*(-702+6/omu+1680*u+(180*u)/omw+1200*v-(57*v)/omu-
     & 1113*u*v-(54*u)/w-30*zeta2+660*u*zeta2+660*v*zeta2-
     & 294*u*v*zeta2-(369*v)/(omu*omu)-1173*(u*u)+(60*(u*u))/omv+
     & (54*(u*u))/w-294*zeta2*(u*u)+
     & (4*nf*(1+u*(-2+v)-2*v+u*u)*(10-32*u+19*(u*u)))/
     & (omu*omu)+6*(v*v)+
     & (30*NFV*(-4*w*(1+w)*(omu*omu)+24*v*(w*w)-
     & 2*omu*(2+5*v)*(w*w)+
     & omu*omu*omu*(2*w*(2+(4+5*v)*w)-2*(-8+omw)*w*(u*u)+
     & 4*omw*(u*u*u)+u*(4+(17+12*v)*(w*w)))))/
     & (omu*omu*omu*(w*w))))/108._dp+
     & G2(3,3)*((16*pi*H1(0)*(u*(-2+v)-2*(4+v)+u*u))/9._dp+
     & (pi*(-523-40*u+(128*u)/omw-18*v-(24*v)/u+34*u*v-308/w+
     & (157*u)/w+14*(u*u)-4*nf*(u*(-2+v)-2*(4+v)+u*u)+
     & 19*(v*v)-(2*(v*v))/u-(v*v*v)/u+151/(w*w)-(151*u)/(w*w)+
     & 10*NFV*(2-v+2*(u*u)-4/(w*w)+
     & u*(-1+2*v+2/(w*w)-2/(w*w*w))+2/(w*w*w))))/18._dp)+
     & G2(3,0)*((-16*pi*H1(1)*(1+u*(-2+v)-2*v+u*u))/9._dp-
     & (2*pi*H1(0)*(-24-31*v+u*(-31+14*v)+14*(u*u)))/9._dp+
     & (pi*(43+48*u-(128*u)/omw+6*v-(30*v)/u-59*u*v+16/w-
     & (7*u)/w-39*(u*u)-nf*(u*(-2+v)-2*(4+v)+u*u)-
     & 19*(v*v)+(2*(v*v))/u+(v*v*v)/u-9/(w*w)+(9*u)/(w*w)-
     & (20*NFV*(omw*omw)*((-2+v)*v-2*omv*(u*u)+u*u*u+
     & u*(omv+v*v)))/(w*w*w)))/18._dp)+
     & G2(0,3)*((-7*pi*H1(0)*(u*(-2+v)-2*(4+v)+u*u))/9._dp+
     & (pi*H1(1)*(u*(-2+v)-2*(4+v)+u*u))/9._dp+
     & (pi*(180-118*v-(63*v)/omu+
     & u*(22-20/omv-200/omw+60*NFV+67*v-34/w-(30*NFV)/w)-
     & 4/w-(1111*v)/(omu*omu)+(2529*v)/(omu*omu*omu)-
     & (2529*v)/(omu*omu*omu*omu)+
     & nf*(-(u*(-2+v))-u*u+
     & 2*(4+(v*(55+50*u+60*(u*u)-4*(u*u*u)+u*u*u*u))/
     & (omu*omu*omu*omu)))+
     & u*u*(-10/(omv*omv)+
     & (-9+56*w+29*(w*w)-30*NFV*(2-5*w+2*(w*w)))/(w*w))+
     & (20*omw*NFV*(u*u*u*u))/(w*w*w)-
     & (u*u*u*(9*(-1+w)*w+20*NFV*(1-5*w+3*(w*w))))/(w*w*w)))/18._dp)
     & +G2(1,0)*((pi*H1(0)*(2-7*u*(-2+v)+14*v-7*(u*u)))/9._dp-
     & (16*pi*H1(1)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (pi*(-27*v*(omv*omv*omv)+
     & (64+NFV*(110-80*v)+28*v+5*nf*(-7+4*v))*(u*u*u*u*u*u*u)+
     & (2+5*nf-20*NFV)*(u*u*u*u*u*u*u*u)+
     & u*u*u*u*u*u*(-368+142*v+81*(v*v)-
     & 10*NFV*(27-38*v+12*(v*v))+nf*(106-125*v+30*(v*v)))+
     & u*u*u*u*u*(810-844*v+55*(v*v)+86*(v*v*v)-
     & 10*NFV*(-36+76*v-45*(v*v)+6*(v*v*v))+
     & nf*(-180+328*v-165*(v*v)+20*(v*v*v)))+
     & u*u*u*(548-913*v+408*(v*v)+5*(v*v*v)-
     & 10*NFV*(-3+31*v-42*(v*v)+18*(v*v*v))+
     & nf*(-115+368*v-399*(v*v)+166*(v*v*v)-20*(v*v*v*v))-
     & 28*(v*v*v*v))+u*u*
     & (-164+133*v+212*(v*v)+
     & nf*(omv*omv)*(40-77*v+25*(v*v))-225*(v*v*v)-
     & 10*NFV*(-5+7*v-3*(v*v)+3*(v*v*v))+53*(v*v*v*v))+
     & u*(18+93*v+2*nf*(-3+5*v)*(omv*omv*omv)-313*(v*v)+
     & 266*(v*v*v)-64*(v*v*v*v)-
     & 10*NFV*(2-7*v+9*(v*v)-9*(v*v*v)+3*(v*v*v*v)))+
     & u*u*u*u*(-910+1388*v-524*(v*v)-51*(v*v*v)+
     & 10*NFV*(-24+77*v-69*(v*v)+18*(v*v*v))+31*(v*v*v*v)+
     & nf*(185-462*v+363*(v*v)-95*(v*v*v)+5*(v*v*v*v)))))/
     & (18._dp*u*(omu*omu)*(w*w*w)))+
     & G2(1,1)*((pi*H1(1)*(52-30*(1+NFV)*v+15*u*(-2+v+NFV*(-2+4*v))+
     & 15*(1+4*NFV)*(u*u)))/9._dp+
     & (pi*H1(0)*(-20-6*(-7+5*NFV)*v+3*u*(14-10*NFV-7*v+20*NFV*v)+
     & 3*(-7+20*NFV)*(u*u)))/9._dp-
     & (pi*(27*v*(omv*omv*omv)+
     & (433+NFV*(20-80*v)-136*v+4*nf*(-7+4*v))*
     & (u*u*u*u*u*u*u)+(-39+4*nf-20*NFV)*(u*u*u*u*u*u*u*u)+
     & u*u*u*u*u*u*(-1739+1491*v-165*(v*v)+
     & 4*nf*(22-25*v+6*(v*v))-20*NFV*(-9-v+6*(v*v)))+
     & u*u*u*u*u*(3565-5322*v+1888*(v*v)-78*(v*v*v)+
     & 4*nf*(-40+68*v-33*(v*v)+4*(v*v*v))-
     & 10*NFV*(50-77*v+9*(v*v)+6*(v*v*v)))+
     & u*u*u*u*(-4165+9217*v-5845*(v*v)+1044*(v*v*v)-
     & 20*NFV*(-25+87*v-60*(v*v)+9*(v*v*v))-10*(v*v*v*v)+
     & 4*nf*(45-102*v+75*(v*v)-19*(v*v*v)+v*v*v*v))+
     & u*u*u*(2819-8616*v+8250*(v*v)-
     & 4*nf*(omv*omv)*(31-26*v+4*(v*v))-2687*(v*v*v)+
     & 214*(v*v*v*v)-10*NFV*
     & (18-160*v+210*(v*v)-81*(v*v*v)+9*(v*v*v*v)))+
     & u*u*(-1033+4257*v+4*nf*(12-5*v)*(omv*omv*omv)-
     & 5773*(v*v)+2965*(v*v*v)-425*(v*v*v*v)+
     & 20*NFV*(-1-34*v+78*(v*v)-51*(v*v*v)+9*(v*v*v*v)))+
     & u*(159-918*v-8*nf*(omv*omv*omv*omv)+1726*(v*v)-
     & 1325*(v*v*v)+358*(v*v*v*v)-
     & 10*NFV*(-2-11*v+45*(v*v)-45*(v*v*v)+15*(v*v*v*v)))))/
     & (18._dp*u*(omu*omu)*(w*w*w)))+
     & G2(1,2)*((-2*pi*H1(0)*(80+u*(-5+v)-5*v+u*u))/9._dp-
     & (16*pi*H1(1)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (pi*(-27*v*(omv*omv*omv)+
     & 2*(63-10*v+2*nf*(-7+4*v)-30*NFV*(-7+4*v))*
     & (u*u*u*u*u*u*u)+2*(-5+2*nf-30*NFV)*(u*u*u*u*u*u*u*u)+
     & u*u*u*u*u*u*(-483+354*v+9*(v*v)+4*nf*(21-25*v+6*(v*v))-
     & 60*NFV*(21-25*v+6*(v*v)))+
     & u*u*u*u*u*(875-1169*v+319*(v*v)+38*(v*v*v)+
     & 4*nf*(-35+65*v-33*(v*v)+4*(v*v*v))-
     & 10*NFV*(-206+387*v-195*(v*v)+22*(v*v*v)))+
     & u*u*u*u*(-840+1562*v-845*(v*v)+89*(v*v*v)+19*(v*v*v*v)+
     & 4*nf*(35-90*v+72*(v*v)-19*(v*v*v)+v*v*v*v)-
     & 10*NFV*(194-527*v+420*(v*v)-106*(v*v*v)+4*(v*v*v*v)))+
     & u*(3+102*v+4*nf*(-1+2*v)*(omv*omv*omv)-250*(v*v)+
     & 173*(v*v*v)-28*(v*v*v*v)+
     & 20*NFV*(1-11*v+27*(v*v)-23*(v*v*v)+7*(v*v*v*v)))+
     & u*u*u*(420-885*v+609*(v*v)-122*(v*v*v)-2*(v*v*v*v)-
     & 4*nf*(21-70*v+78*(v*v)-33*(v*v*v)+4*(v*v*v*v))+
     & 10*NFV*(102-395*v+453*(v*v)-183*(v*v*v)+19*(v*v*v*v)))
     & +u*u*(-91+83*v+77*(v*v)+
     & 4*nf*(omv*omv)*(7-15*v+5*(v*v))-97*(v*v*v)+
     & 37*(v*v*v*v)-10*NFV*
     & (26-151*v+246*(v*v)-145*(v*v*v)+26*(v*v*v*v)))))/
     & (18._dp*u*(omu*omu)*(w*w*w)))+
     & G2(3,2)*((2*pi*H1(0)*(-49+(11-15*NFV)*v+
     & u*(11-7*v+15*NFV*(-1+2*v))+(-7+30*NFV)*(u*u)))/9._dp-
     & (pi*((30-44*v+40*NFV*(-1+2*v))*(u*u*u*u*u)+
     & (-6+20*NFV)*(u*u*u*u*u*u)-
     & v*(omv*omv*omv)*(-30+2*v+v*v)+
     & u*u*u*u*(-60+162*v-115*(v*v)+20*NFV*(1-5*v+6*(v*v)))+
     & u*u*u*(60-263*v+(305-60*NFV)*(v*v)+
     & 20*(-7+4*NFV)*(v*v*v))+
     & u*u*(-30+237*v-(379+60*NFV)*(v*v)+(243+20*NFV)*(v*v*v)+
     & 20*(-4+NFV)*(v*v*v*v))+
     & u*(6-122*v+272*(v*v)-(209+40*NFV)*(v*v*v)+
     & (69+20*NFV)*(v*v*v*v)-16*(v*v*v*v*v))))/(18._dp*u*(w*w*w)))+
     & (pi*H2(0,1)*(-75+128*u+(20*u)/omv-(18*u)/omw+244*v-(3*v)/omu-
     & 132*u*v+(38*u)/w-v/(omu*omu)-94*(u*u)-(56*(u*u))/w+
     & (10*(u*u))/(omv*omv)+6*nf*(1+u*(-2+v)-2*v+u*u)+
     & (9*(u*u*u))/w+(9*(u*u))/(w*w)-(9*(u*u*u))/(w*w)+
     & (10*NFV*u*(omu*omu*omu*
     & (-2*(1+omw-6*v)*(u*u*u)+6*(u*u*u*u)+
     & u*u*(-2-2*v+6*(v*v))+(3-7*w-4*v*w)*(w*w)+
     & u*w*(6-15*w+2*(w*w)))+
     & (4-2*v-u*(8+3*v)+(4+v)*(u*u))*(w*w*w)))/
     & (omu*omu*omu*(w*w*w))))/18._dp+
     & (pi*H2(0,0)*(900+214*u+(20*u)/omv+(182*u)/omw+390*v-
     & (3*v)/omu-156*u*v+4/w+(34*u)/w+(1129*v)/(omu*omu)-
     & (2529*v)/(omu*omu*omu)+(2529*v)/(omu*omu*omu*omu)-
     & 118*(u*u)-(56*(u*u))/w+(10*(u*u))/(omv*omv)+(9*(u*u*u))/w+
     & 4*nf*(-16+2*u*(-2+v)+2*(u*u)-
     & (v*(31+11*u+51*(u*u)-16*(u*u*u)+4*(u*u*u*u)))/
     & (omu*omu*omu*omu))+(9*(u*u))/(w*w)-(9*(u*u*u))/(w*w)+
     & 10*NFV*(4/omu+(2*v)/(omu*omu)-(4*v)/(omu*omu*omu)+
     & (-2*(1+omw-6*v)*(u*u*u*u)+6*(u*u*u*u*u)+
     & u*u*u*(-2-2*v+6*(v*v))+u*(3-7*w-4*v*w)*(w*w)+
     & w*(u*u)*(6-15*w+2*(w*w))-(4+v)*(w*w*w))/(w*w*w))))/18._dp
     & +(pi*((3*(843+1727*w+40*NFV*(1+w)-4*nf*(27+53*w)+
     & 144*v*w*zeta2))/(omu*w)-
     & (108*v*zeta2*(-281+12*nf*(1+u+u*u)))/(omu*omu*omu*omu)-
     & (36*zeta2*(843*v+10*NFV*u*(2-u*(4+3*v)+(2+v)*(u*u))))/
     & (omu*omu*omu)+(-7587+720*NFV*(-1+v)-1461*v+
     & 13038*v*zeta2+4*nf*(243+v*(32-38*u+19*(u*u))))/(omu*omu)
     & +(-240*omv*omw*u*zeta2*(w*w*w)-120*omw*zeta2*(u*u)*(w*w*w)+
     & omv*omv*(20*NFV*(omw*omw)*(u*u)*(3*w+6*zeta2*(u*u))-
     & omw*(10*NFV*(-3*(-4+w*(-7+12*v-24*v*zeta3)+
     & u*(7+w*(-2-24*zeta3+48*v*zeta3))+
     & 48*w*zeta3*(u*u))*(w*w)+
     & 6*u*zeta2*(-3*(-1+4*(1+v)*w)*(w*w)-
     & 3*u*w*(-2+5*w+2*(w*w))+
     & 2*(u*u)*(1-5*w+3*(w*w))))+
     & w*(w*(-4*nf*(81+27*u-100*w)+
     & 3*(843-w*(148-405*v+2324*zeta3+136*v*zeta3)+
     & u*(369+8*(-17+10*v)*w*zeta3)+80*w*zeta3*(u*u))
     & )+6*zeta2*
     & (w*(4+(12-125*v)*w)+u*w*(34+(-376+117*v)*w)+
     & 9*(-1+w)*(u*u*u)+u*u*(9-56*w+155*(w*w)))))+
     & 2*u*(-2268+200*nf-474*zeta2)*(w*w*w)))/
     & (omw*(omv*omv)*(w*w*w))))/108._dp+
     & (-(pi*G1(2)*(1+u*(-2+v)-2*v+u*u))/2._dp+
     & (3*pi*H1(0)*(1+u*(-2+v)-2*v+u*u))/2._dp+
     & (pi*H1(1)*(1+u*(-2+v)-2*v+u*u))/2._dp+
     & (pi*G1(1)*(2+u*(-2+v)-2*v+u*u))/2._dp+
     & (pi*G1(3)*(u*(-2+v)-2*(4+v)+u*u))/2._dp+
     & (pi*(-6+(6*u*(6+5*omw*(-2+v)))/omw-51*v+(9*v)/(omu*omu)+
     & 30*(u*u)-nf*(1+u*(-2+v)-2*v+u*u)))/18._dp)*(G1(0)*G1(0))-
     & (pi*(1+u*(-2+v)-2*v+u*u)*(G1(0)*G1(0)*G1(0)))/2._dp+
     & ((pi*(1+u*(6-4/omw-3*v)+v*(5-1/(omu*omu))-3*(u*u)))/18._dp-
     & (pi*H1(0)*(1+u*(-2+v)-2*v+u*u))/6._dp-
     & (pi*H1(1)*(1+u*(-2+v)-2*v+u*u))/6._dp-
     & (pi*G1(3)*(u*(-2+v)-2*(4+v)+u*u))/18._dp)*(G1(2)*G1(2))+
     & (pi*(1+u*(-2+v)-2*v+u*u)*(G1(2)*G1(2)*G1(2)))/18._dp+
     & ((5*pi*H1(1)*(1+u*(-2+v)-2*v+u*u))/6._dp+
     & (pi*(-533+(u*(-4+57*omw*(-2+v)))/omw-115*v-v/(omu*omu)+
     & 57*(u*u)-2*nf*(-17+u*(-2+v)-2*v+u*u)))/18._dp)*(H1(0)*H1(0))
     & +(17*pi*(1+u*(-2+v)-2*v+u*u)*(H1(0)*H1(0)*H1(0)))/18._dp+
     & (pi*(1+u*(6-4/omw-3*v)+v*(5-1/(omu*omu))-3*(u*u))*
     & (H1(1)*H1(1)))/18._dp+H1(0)*(-(pi*H2(1,0)*
     & (-2*(-5+v)+u*(-2+v)+u*u))-
     & pi*H2(0,0)*(u*(-2+v)-2*(4+v)+u*u)-
     & (pi*H1(1)*(v*(-89+nf*(2-4*v)+148*v)+
     & 2*(117+2*nf*(-2+v)-54*v)*(u*u*u*u)+
     & 2*(-27+nf)*(u*u*u*u*u)+
     & u*u*(288-705*v+nf*(-8+22*v-8*(v*v))+236*(v*v))+
     & u*u*u*(-387+470*v-54*(v*v)+2*nf*(6-8*v+v*v))+
     & u*(-81+452*v-310*(v*v)+2*nf*(1-6*v+5*(v*v)))))/
     & (18._dp*omw*(omu*omu))-
     & (pi*(-3*omu*omv*omw*(180*omv*NFV+3*(843+(1686+7*v)*w)-
     & 20*NFV*(u*(29-18*v)+5*v*w+(-27+7*v)*(u*u)+7*(u*u*u)))*
     & (w*w)+3*omv*omw*w*(omu*omu)*
     & (-843+2529*w+(2150+19*v)*(w*w))-
     & 2*omv*omw*w*(-2*nf*
     & ((-127+57*v)*(u*u*u*u*u*u)+19*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(279-343*v+57*(v*v))+
     & 2*(-5+47*v-52*(v*v)+10*(v*v*v))+
     & u*u*u*u*(-209+656*v-305*(v*v)+19*(v*v*v))-
     & 2*u*(94-239*v+98*(v*v)+47*(v*v*v))-
     & u*u*u*(214+574*v-521*(v*v)+89*(v*v*v))+
     & u*u*(450-368*v-459*(v*v)+144*(v*v*v)))+
     & 9*(-843+40*NFV*v)*(w*w))+
     & omu*omu*omu*(-60*omw*u*(1+u)*(w*w*w)+
     & omv*(120*NFV*(omw*omw*omw)*(u*u*u)+
     & 540*(1+NFV)*u*(w*w*w)-
     & omw*(w*(-2529+1680*w+
     & (30*(-9+38*v)*zeta2+6*(807+191*v+v*v))*(w*w)-
     & u*u*(54-108*w+(1173+534*zeta2)*(w*w))+
     & u*(-1053+603*w+
     & (1710-1113*v+6*(190-89*v)*zeta2)*(w*w)))+
     & 30*NFV*(10*v*(w*w*w)+
     & 2*(u*u)*(1-9*w+13*(w*w)+w*w*w)+
     & u*(-4+15*w-24*(w*w)+(5+12*v)*(w*w*w))))))))/
     & (108._dp*omv*omw*(omu*omu*omu)*(w*w*w))-
     & (pi*(1+u*(-2+v)-2*v+u*u)*(H1(1)*H1(1)))/6._dp)+
     & G1(0)*((pi*H2(0,0)*(120+80*u+80*v-37*u*v-37*(u*u)))/9._dp+
     & (pi*H2(1,1)*(2-7*u*(-2+v)+14*v-7*(u*u)))/9._dp-
     & (16*pi*H2(0,1)*(1+u*(-2+v)-2*v+u*u))/9._dp-
     & (2*pi*H2(1,0)*(3-13*v+u*(-13+5*v)+5*(u*u)))/9._dp+
     & G1(2)*(-((-27+nf)*pi*(1+u*(-2+v)-2*v+u*u))/18._dp+
     & (8*pi*H1(0)*(1+u*(-2+v)-2*v+u*u))/9._dp-
     & (pi*H1(1)*(1+u*(-2+v)-2*v+u*u))/9._dp)+
     & G1(1)*(((-30+nf)*pi*(2+u*(-2+v)-2*v+u*u))/18._dp-
     & pi*H1(0)*(2+u*(-2+v)-2*v+u*u))+
     & G1(3)*(((-30+nf)*pi*(u*(-2+v)-2*(4+v)+u*u))/18._dp-
     & pi*H1(0)*(u*(-2+v)-2*(4+v)+u*u))+
     & (pi*H1(1)*(-14-60*u+(26*u)/omw-94*v+(3*v)/omu+21*u*v-
     & (17*v)/(omu*omu)+21*(u*u)-
     & 4*nf*(1+u*(-2+v)-2*v+u*u)-
     & (10*NFV*u*(1+u*(-2+v)-2*v+u*u)*(3-3*u+4*(u*u)))/
     & (omu*omu*omu)))/18._dp+
     & (pi*((-6+57*v+(120*NFV*(1+w))/w)/omu-
     & (720*NFV*v)/(omu*omu*omu)+
     & (-9*v+60*NFV*(2+9*v)-
     & 4*nf*(1+u*(-2+v)-2*v+u*u)*(10-32*u+19*(u*u)))/
     & (omu*omu)-(60*omw*(u*u)*(w*w)+
     & omv*(120*NFV*(omw*omw)*(u*u*u)-732*u*(w*w)+
     & omw*(w*(u*(-54+
     & w*(1734-1113*v+6*(106-65*v)*zeta2))+
     & (54-w*(1173+390*zeta2))*(u*u)+
     & w*(6*(-45+106*v)*zeta2+6*(43+265*v+v*v)))+
     & 30*NFV*(2*w*(2+4*w-3*v*w)+2*w*(7+w)*(u*u)+
     & u*(4+w*w+12*v*(w*w))))))/(omv*omw*(w*w))))/108._dp
     & +H1(0)*((pi*H1(1)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (pi*(102+64*u-(20*u)/omv-(110*u)/omw-94*v+(3*v)/omu+
     & 12*u*v-4/w-(34*u)/w-(17*v)/(omu*omu)-26*(u*u)+
     & (56*(u*u))/w-(10*(u*u))/(omv*omv)-
     & 4*nf*(1+u*(-2+v)-2*v+u*u)-(9*(u*u*u))/w-
     & (9*(u*u))/(w*w)+(9*(u*u*u))/(w*w)+
     & 10*NFV*u*((2*(-2+v)+u*(8+3*v)-(4+v)*(u*u))/
     & (omu*omu*omu)+
     & (2*omw*(u*u*u)+u*w*(-6+15*w-2*(w*w))+
     & (-3+7*w+4*v*w)*(w*w)-2*(u*u)*(1-5*w+3*(w*w)))/
     & (w*w*w))))/18._dp)+
     & (pi*(1+u*(-2+v)-2*v+u*u)*(G1(2)*G1(2)))/18._dp-
     & (pi*(179+17*u*(-2+v)-34*v+17*(u*u))*(H1(0)*H1(0)))/18._dp+
     & (pi*(1+u*(-2+v)-2*v+u*u)*(H1(1)*H1(1)))/18._dp)+
     & G1(2)*((2*pi*H2(1,0)*(-41+3*omv*u+3*v+15*NFV*(-1+2*u)*(u+v)-
     & 3*(u*u)))/9._dp+(pi*H2(0,1)*(28+u*(-2+v)-2*v+u*u))/9._dp-
     & (2*pi*H2(0,0)*(44-23*v+u*(-23+10*v)+10*(u*u)))/9._dp+
     & G1(3)*((pi*(u*(-2+v)-2*(4+v)+u*u))/6._dp+
     & (pi*H1(0)*(u*(-2+v)-2*(4+v)+u*u))/9._dp+
     & (pi*H1(1)*(u*(-2+v)-2*(4+v)+u*u))/9._dp)+
     & (pi*H1(1)*(58-154*u+(8*u)/omw-152*v+(3*v)/omu+87*u*v-
     & (15*v)/(omu*omu)+87*(u*u)-
     & 4*nf*(1+u*(-2+v)-2*v+u*u)-
     & (10*NFV*u*(1+u*(-2+v)-2*v+u*u)*(3-3*u+4*(u*u)))/
     & (omu*omu*omu)))/18._dp+
     & H1(0)*((pi*H1(1)*(1+u*(-2+v)-2*v+u*u))/3._dp+
     & (pi*(486-32/omw-96*u+(20*u)/omv+(65*u)/omw-124*v+
     & (3*v)/omu+51*u*v+4/w-(4*u)/w-(1091*v)/(omu*omu)+
     & (2529*v)/(omu*omu*omu)-(2529*v)/(omu*omu*omu*omu)+
     & (32*u)/(omw*omw)+51*(u*u)+(10*(u*u))/(omv*omv)+
     & 10*NFV*(10-4/omu+(9-6*u)/omw-11*u+4*u*v-
     & (9*u)/(omw*omw)+4*(u*u)-
     & (v*(-3+u-3*(u*u)+u*u*u))/(omu*omu*omu))+
     & 4*nf*(-9-u*(-2+v)-u*u+
     & (v*(29+19*u+39*(u*u)-8*(u*u*u)+2*(u*u*u*u)))/
     & (omu*omu*omu*omu))))/18._dp)+
     & (pi*((-6+57*v)/omu-(720*NFV*v)/(omu*omu*omu)+
     & (w*(369*v-4*nf*(1+u*(-2+v)-2*v+u*u)*
     & (10-32*u+19*(u*u)))+
     & 60*NFV*(u*(8-6*v)+5*v*w+4*(-3+v)*(u*u)+4*(u*u*u)))/
     & (w*(omu*omu))-(60*omw*(u*u)*(w*w)+
     & omv*(180*u*(w*w)+
     & omw*(w*(u*(-54+
     & w*(21*(80-53*v)+6*(110-49*v)*zeta2))+
     & (54-w*(1173+294*zeta2))*(u*u)+
     & w*(30*(39+22*v)*zeta2+6*(-117+200*v+v*v)))+
     & 30*NFV*(2*w*(7+w)*(u*u)+4*v*(u*u*u)+4*(u*u*u*u)+
     & 10*v*(w*w)+u*(4+(17+12*v)*(w*w))))))/
     & (omv*omw*(w*w))))/108._dp+
     & (pi*(7+u*(-2+v)-2*v+u*u)*(H1(0)*H1(0)))/6._dp+
     & (pi*(1+u*(-2+v)-2*v+u*u)*(H1(1)*H1(1)))/6._dp)+
     & G1(1)*((16*pi*H2(1,1)*(1+u*(-2+v)-2*v+u*u))/9._dp+
     & (pi*H2(0,1)*(7*u*(-2+v)-2*(1+7*v)+7*(u*u)))/9._dp+
     & (2*pi*H2(0,0)*(98-23*v+u*(-23+10*v)+10*(u*u)))/9._dp+
     & (pi*H2(1,0)*(178-28*v+u*(-28+11*v)+11*(u*u)))/9._dp+
     & G1(2)*((pi*(2+u*(-2+v)-2*v+u*u))/6._dp+
     & (pi*H1(0)*(2+u*(-2+v)-2*v+u*u))/9._dp+
     & (pi*H1(1)*(2+u*(-2+v)-2*v+u*u))/9._dp)+
     & (pi*((-12*(8+45*NFV))/omu+
     & (6*(-69+4*nf-60*NFV)*v)/(omu*omu)+
     & (60*omw*(-1+u)*(u*u)*(w*w)+
     & omv*(60*NFV*(2*u-w)*(omw*omw)*(u*u*u)+
     & 12*(-16+45*NFV)*(-1+u)*u*(w*w)+
     & omw*(w*(w*(1758-1113*v+4*nf*(-32+19*v)+
     & 6*(170-97*v)*zeta2)*(u*u)+
     & (54+w*(-1173+76*nf-582*zeta2))*(u*u*u)-
     & 6*w*(-17-13*v+v*v)+
     & u*(54+w*
     & (-987+1608*v-4*nf*(-29+32*v)+
     & 60*(55+17*v)*zeta2+6*(v*v))))+
     & 30*NFV*(w*(2+3*(7+4*v)*w)*(u*u)+16*w*(u*u*u)-
     & 3*(3+2*v)*(w*w)-2*u*(2-3*w+(13+3*v)*(w*w)))))
     & )/(omv*omw*u*(w*w))))/108._dp+
     & H1(0)*((-10*pi*H1(1)*(2+u*(-2+v)-2*v+u*u))/9._dp+
     & (pi*(73-34*u-30*v-(63*v)/omu+(54*v)/u+25*u*v-40/w+
     & (22*u)/w-(19*v)/(omu*omu)+25*(u*u)+
     & 4*nf*(1+u*(-2+v)-2*v+u*u)+18/(w*w)-(18*u)/(w*w)+
     & 10*NFV*(-4+11*v+(3*v)/(omu*omu)-4*(u*u)+8/(w*w)+
     & u*(11-4*v-4/(w*w)+4/(w*w*w))-4/(w*w*w))))/18._dp)-
     & (pi*H1(1)*(-27*v*(omv*omv*omv)+
     & (147-32*v+4*nf*(-7+4*v)-60*NFV*(-7+4*v))*
     & (u*u*u*u*u*u*u)+(-13+4*nf-60*NFV)*(u*u*u*u*u*u*u*u)+
     & u*u*u*u*u*u*(4*nf*(21-25*v+6*(v*v))-
     & 3*(183-143*v+3*(v*v)+20*NFV*(21-25*v+6*(v*v))))+
     & u*u*u*u*u*(995-1373*v+418*(v*v)+26*(v*v*v)+
     & 4*nf*(-35+65*v-33*(v*v)+4*(v*v*v))-
     & 10*NFV*(-206+387*v-195*(v*v)+22*(v*v*v)))+
     & u*u*u*u*(-975+1868*v-1070*(v*v)+146*(v*v*v)+16*(v*v*v*v)+
     & 4*nf*(35-90*v+72*(v*v)-19*(v*v*v)+v*v*v*v)-
     & 10*NFV*(194-527*v+420*(v*v)-106*(v*v*v)+4*(v*v*v*v)))+
     & u*(9+78*v+4*nf*(-1+2*v)*(omv*omv*omv)-214*(v*v)+
     & 149*(v*v*v)-22*(v*v*v*v)+
     & 20*NFV*(1-11*v+27*(v*v)-23*(v*v*v)+7*(v*v*v*v)))+
     & u*u*u*(513-1149*v+870*(v*v)-224*(v*v*v)+10*(v*v*v*v)-
     & 4*nf*(21-70*v+78*(v*v)-33*(v*v*v)+4*(v*v*v*v))+
     & 10*NFV*(102-395*v+453*(v*v)-183*(v*v*v)+19*(v*v*v*v)))
     & +u*u*(-127+206*v-76*(v*v)+
     & 4*nf*(omv*omv)*(7-15*v+5*(v*v))-16*(v*v*v)+
     & 22*(v*v*v*v)-10*NFV*
     & (26-151*v+246*(v*v)-145*(v*v*v)+26*(v*v*v*v)))))/
     & (18._dp*u*(omu*omu)*(w*w*w))-
     & (pi*(2+u*(-2+v)-2*v+u*u)*(G1(2)*G1(2)))/18._dp-
     & (pi*(2+u*(-2+v)-2*v+u*u)*(H1(0)*H1(0)))/18._dp-
     & (pi*(2+u*(-2+v)-2*v+u*u)*(H1(1)*H1(1)))/18._dp)+
     & G1(3)*((pi*H2(0,1)*(8+17*u*(-2+v)-34*v+17*(u*u)))/9._dp+
     & (2*pi*H2(0,0)*(-96-49*v+u*(-49+23*v)+23*(u*u)))/9._dp-
     & (2*pi*H2(1,0)*(-49+(11-15*NFV)*v+
     & u*(11-7*v+15*NFV*(-1+2*v))+(-7+30*NFV)*(u*u)))/9._dp+
     & H1(0)*(-(pi*H1(1)*(u*(-2+v)-2*(4+v)+u*u))/9._dp-
     & (pi*((-157+215*v-20*NFV*(-2+5*v))*(u*u*u*u*u*u)+
     & (39-20*NFV)*(u*u*u*u*u*u*u)+
     & omv*omv*omv*(v*v)*(-30+2*v+v*v)+
     & u*u*u*u*u*(75-694*v+489*(v*v)-
     & 20*NFV*(1-7*v+10*(v*v)))+
     & u*u*u*u*(327-4*(-34+5*NFV)*v+5*(-241+32*NFV)*(v*v)+
     & (585-200*NFV)*(v*v*v))+
     & u*u*u*(-446+1299*v+(-9+60*NFV)*(v*v)+
     & 20*(-51+2*NFV)*(v*v*v)-5*(-77+20*NFV)*(v*v*v*v))+
     & u*v*(260-678*v+455*(v*v)+10*(1+4*NFV)*(v*v*v)-
     & 2*(31+10*NFV)*(v*v*v*v)+15*(v*v*v*v*v))+
     & u*u*(162-1216*v+1522*(v*v)+(-93+100*NFV)*(v*v*v)-
     & 5*(83+8*NFV)*(v*v*v*v)+(129-20*NFV)*(v*v*v*v*v))))/
     & (18._dp*omw*u*(w*w*w)))-
     & (omw*pi*H1(1)*(omv*omv*omv*(-30+2*v+v*v)+
     & u*(96-231*v+(-15+NFV*(40-60*v)+29*v)*(u*u*u)+
     & (3-20*NFV)*(u*u*u*u)+2*(91+20*NFV)*(v*v)+
     & u*u*(57-96*v+68*(v*v)-20*NFV*(1-3*v+3*(v*v)))-
     & 4*(16+5*NFV)*(v*v*v)+
     & u*(-111+(206+20*NFV)*v-146*(v*v)-
     & 20*(-3+NFV)*(v*v*v))+17*(v*v*v*v))))/(18._dp*u*(w*w*w))-
     & (pi*(27*omu*omv*omw*(36*nf*(1+2*w)-281*(1+(2+v)*w))*(w*w)+
     & 3*omv*omw*w*(omu*omu)*
     & (-843+2529*w+(2222+1379*v)*(w*w)-
     & 108*nf*(-1+3*w+2*(w*w)))-
     & 2*omv*omw*(-7587*(1+v)+
     & nf*(972+v*(572+390*u-66*(u*u)+76*(u*u*u))))*(w*w*w)+
     & omu*omu*omu*(-60*omw*u*(1+u)*(w*w*w)+
     & omv*(120*NFV*(omw*omw*omw)*(u*u*u)+
     & 48*(-5+2*nf)*u*(w*w*w)-
     & omw*(w*(-2529+2160*w+8442*(w*w)+1392*v*(w*w)+
     & 2160*zeta2*(w*w)+2004*v*zeta2*(w*w)+
     & 6*(v*v)*(w*w)-
     & u*u*(54-108*w+23*(51+42*zeta2)*(w*w))+
     & u*(-1053+231*w+
     & (1710-1113*v+6*(334-161*v)*zeta2)*(w*w))+
     & 4*nf*(81-54*w-164*(w*w)+19*(u*u)*(w*w)+
     & u*(27-27*w+(-32+19*v)*(w*w))))+
     & 30*NFV*(-6*v*(w*w*w)+
     & 2*(u*u)*(1-9*w+13*(w*w)+w*w*w)+
     & u*(-4+7*w-20*(w*w)+(5+12*v)*(w*w*w))))))))/
     & (108._dp*omv*omw*(omu*omu*omu)*(w*w*w))-
     & (pi*(u*(-2+v)-2*(4+v)+u*u)*(H1(0)*H1(0)))/18._dp-
     & (pi*(u*(-2+v)-2*(4+v)+u*u)*(H1(1)*H1(1)))/18._dp)-
     & (pi*(1+u*(-2+v)-2*v+u*u)*(H1(1)*H1(1)*H1(1)))/18._dp
      return
      end
      
      
      function Gamma_3a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function Gamma_3a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function Gamma_3a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-3+3/omu-(12*u)/omw-12*u*v*zeta2)/9._dp+(u*v*G2(0,2))/3._dp-
     & (u*v*G2(1,2))/3._dp+(u*v*G2(2,0))/3._dp-(u*v*G2(3,0))/3._dp+(u*v*G1(3)*H1(0))/3._dp+
     & G1(1)*((u*v*H1(0))/3._dp+(u*v*H1(1))/3._dp)+(u*v*H2(0,0))/3._dp+(u*v*H2(1,0))/3._dp+
     & G1(2)*(-(u*v*H1(0))/3._dp-((-2+u)*u*v)/(3._dp*(omu*omu)))+
     & ((-2+u)*u*v*H1(1))/(3._dp*(omu*omu))+
     & G1(0)*(-(u*v*H1(0))/3._dp-(u*v*H1(1))/3._dp+
     & ((-4+omw)*u*v)/(3._dp*(omw*omw)))+
     & (u*v*H1(0)*(4-v+(4+v)*(u*u)-2*(v*v)+u*(-9-2*v+v*v)))/
     & (3._dp*(omu*omu)*(omw*omw))
      return
      end
      
      
      function Gamma_3a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(pi*u*v*G1(0))/3._dp-(pi*u*v*G1(1))/3._dp+(pi*u*v*G1(2))/3._dp-(pi*u*v*G1(3))/3._dp-
     & (pi*u*v*H1(0))/3._dp-(pi*u*v*H1(1))/3._dp+
     & (pi*u*v*(-4+v-(4+v)*(u*u)+u*(9+2*v-v*v)+2*(v*v)))/
     & (3._dp*(omu*omu)*(omw*omw))
      return
      end
      
      
      function Gamma_3a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(4*(1-5*NFV)*u*v*G4(0,0,1,2))/3._dp-(7*u*v*G4(0,0,2,2))/9._dp+
     & (22*u*v*G4(0,0,3,0))/9._dp-2*u*v*G4(0,1,0,2)+
     & (10*(-1+2*NFV)*u*v*G4(0,1,1,2))/3._dp-2*u*v*G4(0,1,2,0)+
     & (2*(1+10*NFV)*u*v*G4(0,1,2,2))/3._dp-(7*u*v*G4(0,2,0,2))/9._dp+
     & ((7+60*NFV)*u*v*G4(0,2,1,2))/9._dp-(7*u*v*G4(0,2,2,0))/9._dp+
     & (4*u*v*G4(0,2,3,0))/3._dp-(u*v*G4(0,3,0,2))/9._dp-(u*v*G4(0,3,2,0))/9._dp-
     & (16*u*v*G4(0,3,3,0))/9._dp+((-7+20*NFV)*u*v*G4(1,0,1,2))/3._dp+
     & (16*u*v*G4(1,0,2,2))/9._dp-(4*u*v*G4(1,0,3,0))/9._dp+
     & ((19-20*NFV)*u*v*G4(1,1,1,2))/3._dp-(5*(1+4*NFV)*u*v*G4(1,1,2,2))/3._dp+
     & (16*u*v*G4(1,2,0,2))/9._dp-(5*(1+4*NFV)*u*v*G4(1,2,1,2))/3._dp+
     & (16*u*v*G4(1,2,2,0))/9._dp-(4*u*v*G4(1,2,3,0))/9._dp-(7*u*v*G4(2,0,0,2))/9._dp+
     & ((7+60*NFV)*u*v*G4(2,0,1,2))/9._dp-(7*u*v*G4(2,0,2,0))/9._dp+
     & (4*u*v*G4(2,0,3,0))/3._dp+u*v*G4(2,1,0,2)-4*u*v*G4(2,1,1,2)+
     & u*v*G4(2,1,2,0)-(7*u*v*G4(2,2,0,0))/9._dp+
     & ((-1+20*NFV)*u*v*G4(2,2,1,2))/3._dp-(13*u*v*G4(2,2,3,0))/9._dp+
     & (11*u*v*G4(2,3,0,0))/9._dp-(8*u*v*G4(2,3,0,2))/9._dp-(8*u*v*G4(2,3,2,0))/9._dp-
     & (5*u*v*G4(2,3,3,0))/9._dp+(16*u*v*G4(3,0,0,2))/9._dp+
     & ((11-60*NFV)*u*v*G4(3,0,1,2))/9._dp+(16*u*v*G4(3,0,2,0))/9._dp-
     & (10*u*v*G4(3,0,3,0))/3._dp+(16*u*v*G4(3,2,0,0))/9._dp+
     & ((11-60*NFV)*u*v*G4(3,2,1,2))/9._dp+(14*u*v*G4(3,2,3,0))/9._dp-
     & (20*u*v*G4(3,3,0,0))/9._dp+(4*u*v*G4(3,3,3,0))/3._dp-
     & (22*u*v*G3(0,0,3)*H1(0))/9._dp-(4*u*v*G3(0,2,3)*H1(0))/3._dp+
     & (u*v*G3(0,3,2)*H1(0))/9._dp+(16*u*v*G3(0,3,3)*H1(0))/9._dp+
     & (4*u*v*G3(1,0,3)*H1(0))/9._dp+(4*u*v*G3(1,2,3)*H1(0))/9._dp-
     & (4*u*v*G3(2,0,3)*H1(0))/3._dp+(13*u*v*G3(2,2,3)*H1(0))/9._dp+
     & (8*u*v*G3(2,3,2)*H1(0))/9._dp+(5*u*v*G3(2,3,3)*H1(0))/9._dp+
     & (10*u*v*G3(3,0,3)*H1(0))/3._dp-(14*u*v*G3(3,2,3)*H1(0))/9._dp-
     & (4*u*v*G3(3,3,3)*H1(0))/3._dp+G3(2,1,0)*(-(u*v*H1(0))-u*v*H1(1))+
     & G3(0,1,0)*(2*u*v*H1(0)+2*u*v*H1(1))+G3(2,1,1)*(4*u*v*H1(0)+4*u*v*H1(1))+
     & G3(2,2,1)*(((1-20*NFV)*u*v*H1(0))/3._dp+((1-20*NFV)*u*v*H1(1))/3._dp)+
     & G3(1,0,1)*(((7-20*NFV)*u*v*H1(0))/3._dp+((7-20*NFV)*u*v*H1(1))/3._dp)+
     & G3(0,1,1)*((10*(1-2*NFV)*u*v*H1(0))/3._dp+(10*(1-2*NFV)*u*v*H1(1))/3._dp)+
     & G3(1,2,1)*((5*(1+4*NFV)*u*v*H1(0))/3._dp+(5*(1+4*NFV)*u*v*H1(1))/3._dp)+
     & G3(0,0,1)*((4*(-1+5*NFV)*u*v*H1(0))/3._dp+(4*(-1+5*NFV)*u*v*H1(1))/3._dp)+
     & G3(1,1,1)*(((-19+20*NFV)*u*v*H1(0))/3._dp+((-19+20*NFV)*u*v*H1(1))/3._dp)+
     & G3(3,0,1)*(((-11+60*NFV)*u*v*H1(0))/9._dp+((-11+60*NFV)*u*v*H1(1))/9._dp)+
     & G3(3,2,1)*(((-11+60*NFV)*u*v*H1(0))/9._dp+((-11+60*NFV)*u*v*H1(1))/9._dp)+
     & G3(0,2,1)*(-((7+60*NFV)*u*v*H1(0))/9._dp-((7+60*NFV)*u*v*H1(1))/9._dp)+
     & G3(2,0,1)*(-((7+60*NFV)*u*v*H1(0))/9._dp-((7+60*NFV)*u*v*H1(1))/9._dp)-
     & (8*u*v*H4(0,0,0,0))/3._dp-2*u*v*H4(0,0,0,1)-(4*u*v*H4(0,0,1,0))/9._dp-
     & (4*u*v*H4(0,1,0,0))/9._dp+((-11+60*NFV)*u*v*H4(0,1,0,1))/9._dp-
     & ((1+20*NFV)*u*v*H4(0,1,1,0))/3._dp-3*u*v*H4(1,0,0,0)-
     & (10*u*v*H4(1,0,0,1))/9._dp-(2*u*v*H4(1,0,1,0))/3._dp-(25*u*v*H4(1,1,0,0))/9._dp+
     & ((-1+20*NFV)*u*v*H4(1,1,0,1))/3._dp-((23+60*NFV)*u*v*H4(1,1,1,0))/9._dp+
     & G3(1,2,2)*((-5*(1+4*NFV)*u*v*H1(0))/3._dp+
     & (v*(-14+81*u-4*nf*u+10*NFV*(-9+4*u)+
     & (34+60*NFV)/(omu*omu)))/18._dp)+
     & (u*v*H2(1,1)*(-600-12*nf*(-2+u)+303*u+66*zeta2-132*u*zeta2-
     & 60*NFV*(-6+4*u+18*zeta2*(omu*omu))+66*zeta2*(u*u)))/
     & (54._dp*(omu*omu))+G2(2,2)*((-5*u*v*H2(0,0))/9._dp+
     & ((-1+20*NFV)*u*v*H2(0,1))/3._dp-((1+20*NFV)*u*v*H2(1,0))/3._dp+
     & (v*H1(0)*(-14+81*u-4*nf*u+10*NFV*(-9+4*u)+
     & (34+60*NFV)/(omu*omu)))/18._dp+
     & (u*v*(-600-12*nf*(-2+u)+303*u+186*zeta2-372*u*zeta2-
     & 60*NFV*(-6+4*u+18*zeta2*(omu*omu))+186*zeta2*(u*u)))/
     & (54._dp*(omu*omu)))+G3(2,1,2)*
     & (-2*u*v*H1(0)+(2*v*(-26-45*NFV+10*v+(26+30*NFV)/(omu*omu))-
     & (v*(omv*omv))/u+(u*
     & (-32+90*NFV+omw*(41+60*NFV)+
     & (-24+87*v-4*nf*v+20*NFV*(6+v))*(omw*omw)))/(omw*omw)
     & -((-32+90*NFV+omw*(41+60*NFV)+
     & 24*(-1+5*NFV)*(omw*omw))*(u*u))/(omw*omw*omw))/18._dp)+
     & G2(2,1)*(2*u*v*zeta2-2*u*v*H2(0,1)+2*u*v*H2(1,0)+
     & (H1(0)*(2*v*(26+45*NFV-10*v-(26+30*NFV)/(omu*omu))+
     & (v*(omv*omv))/u-(u*
     & (-32+90*NFV+omw*(41+60*NFV)+
     & (-24+87*v-4*nf*v+20*NFV*(6+v))*(omw*omw)))/
     & (omw*omw)+((-32+90*NFV+omw*(41+60*NFV)+
     & 24*(-1+5*NFV)*(omw*omw))*(u*u))/(omw*omw*omw)))/18._dp+
     & (H1(1)*(2*v*(26+45*NFV-10*v-(26+30*NFV)/(omu*omu))+
     & (v*(omv*omv))/u-(u*
     & (-32+90*NFV+omw*(41+60*NFV)+
     & (-24+87*v-4*nf*v+20*NFV*(6+v))*(omw*omw)))/
     & (omw*omw)+((-32+90*NFV+omw*(41+60*NFV)+
     & 24*(-1+5*NFV)*(omw*omw))*(u*u))/(omw*omw*omw)))/18._dp)+
     & G2(0,1)*((50*(1-2*NFV)*u*v*zeta2)/3._dp+(4*(-1+5*NFV)*u*v*H2(0,1))/3._dp+
     & 2*u*v*H2(1,0)+(2*(1+10*NFV)*u*v*H2(1,1))/3._dp+
     & (H1(0)*(u*(omu*omu)*(2*(-16+45*NFV)*u+
     & omw*(32+41*u+30*NFV*(-3+2*u))+
     & (-41-24*u+60*NFV*(-1+2*u))*(omw*omw)+
     & (24-120*NFV-77*v+5*nf*v)*(omw*omw*omw))+
     & v*(omw*omw*omw)*(20-99*u+60*(u*u)-
     & 30*NFV*(1-6*u+4*(u*u)))))/(18._dp*(omu*omu)*(omw*omw*omw))
     & +(H1(1)*(u*(omu*omu)*(2*(-16+45*NFV)*u+
     & omw*(32+41*u+30*NFV*(-3+2*u))+
     & (-41-24*u+60*NFV*(-1+2*u))*(omw*omw)+
     & (24-120*NFV-77*v+5*nf*v)*(omw*omw*omw))+
     & v*(omw*omw*omw)*(20-99*u+60*(u*u)-
     & 30*NFV*(1-6*u+4*(u*u)))))/(18._dp*(omu*omu)*(omw*omw*omw))
     & )+G3(0,1,2)*(-2*u*v*H1(0)-(2*(1+10*NFV)*u*v*H1(1))/3._dp+
     & (u*(omu*omu)*(32*u-90*NFV*u-
     & omw*(32-90*NFV+41*u+60*NFV*u)+
     & (41+60*NFV+24*u-120*NFV*u)*(omw*omw)+
     & (-24+120*NFV+77*v-5*nf*v)*(omw*omw*omw))+
     & v*(omw*omw*omw)*(-20+99*u-60*(u*u)+
     & 30*NFV*(1-6*u+4*(u*u))))/(18._dp*(omu*omu)*(omw*omw*omw)))
     & +G3(2,2,0)*((u*v*H1(0))/3._dp+
     & (u*v*(-114+(292+90*NFV)*u+4*nf*(omu*omu*omu)-
     & (259+90*NFV)*(u*u)+(81+40*NFV)*(u*u*u)))/
     & (18._dp*(omu*omu*omu)))+
     & G3(0,2,2)*((2*(1+10*NFV)*u*v*H1(0))/3._dp+
     & (u*v*(-114+(292+90*NFV)*u+4*nf*(omu*omu*omu)-
     & (259+90*NFV)*(u*u)+(81+40*NFV)*(u*u*u)))/
     & (18._dp*(omu*omu*omu)))+
     & G3(2,0,2)*((2*(1+10*NFV)*u*v*H1(0))/3._dp+
     & (u*v*(-114+(292+90*NFV)*u+4*nf*(omu*omu*omu)-
     & (259+90*NFV)*(u*u)+(81+40*NFV)*(u*u*u)))/
     & (18._dp*(omu*omu*omu)))+
     & G2(2,3)*((20*u*v*zeta2)/9._dp+(5*u*v*H2(0,0))/9._dp-(8*u*v*H2(0,1))/9._dp-
     & (8*u*v*H2(1,0))/9._dp+(u*H1(0)*
     & (30-80/omw-73*v+4/w+u*(-34/omw-4/w+80/(omw*omw))+
     & (v*(-878-2*(794+15*NFV)*u+108*nf*(1+2*u)+
     & (-91+60*NFV)*(u*u)+(28-30*NFV)*(u*u*u)))/
     & (omu*omu*omu*omu)))/18._dp)+
     & G3(2,3,0)*((-8*u*v*H1(0))/9._dp+(8*u*v*H1(1))/9._dp+
     & (u*(-30+80/omw+73*v-4/w+u*(34/omw+4/w-80/(omw*omw))+
     & (v*(878+2*(794+15*NFV)*u-108*nf*(1+2*u)+
     & (91-60*NFV)*(u*u)+(-28+30*NFV)*(u*u*u)))/
     & (omu*omu*omu*omu)))/18._dp)+
     & G3(0,2,0)*((11*u*v*H1(0))/9._dp+(7*u*v*H1(1))/9._dp+
     & (u*v*(-13-17*v+u*(-17+6*(-6+nf)*v)+3*(-6+nf)*(u*u)+
     & 3*(-6+nf)*(v*v)))/(9._dp*(omw*omw)))+
     & G3(2,0,0)*((11*u*v*H1(0))/9._dp+(7*u*v*H1(1))/9._dp+
     & (u*v*(-13-17*v+u*(-17+6*(-6+nf)*v)+3*(-6+nf)*(u*u)+
     & 3*(-6+nf)*(v*v)))/(9._dp*(omw*omw)))+
     & G3(0,0,2)*((4*(7-15*NFV)*u*v*H1(0))/9._dp+(7*u*v*H1(1))/9._dp+
     & (u*v*(-13-17*v+u*(-17+6*(-6+nf)*v)+3*(-6+nf)*(u*u)+
     & 3*(-6+nf)*(v*v)))/(9._dp*(omw*omw)))+
     & G3(3,0,0)*((-20*u*v*H1(0))/9._dp-(16*u*v*H1(1))/9._dp+
     & (u*v*(-64+87*v+(-3-54*v+nf*(-3+9*v))*(u*u)+
     & 3*(-6+nf)*(u*u*u)-3*(1+nf)*(v*v)+
     & u*(87-6*(1+nf)*v+9*(-6+nf)*(v*v))+3*(-6+nf)*(v*v*v)))/
     & (9._dp*w*(omw*omw)))+G2(0,0)*
     & ((-20*u*v*H2(0,0))/9._dp-(16*u*v*H2(0,1))/9._dp-(11*u*v*H2(1,0))/9._dp-
     & (7*u*v*H2(1,1))/9._dp+(u*v*H1(1)*
     & (13+17*v+u*(17-6*(-6+nf)*v)-3*(-6+nf)*(u*u)-
     & 3*(-6+nf)*(v*v)))/(9._dp*(omw*omw))+
     & (u*v*(1680+18*(-4+omw)*nf-138*v+2*u*(-69+186*v*zeta2)+
     & 186*zeta2*(u*u)+186*zeta2*(v*v)))/(54._dp*(omw*omw))+
     & (u*v*H1(0)*(-64+87*v+(-3-54*v+nf*(-3+9*v))*(u*u)+
     & 3*(-6+nf)*(u*u*u)-3*(1+nf)*(v*v)+
     & u*(87-6*(1+nf)*v+9*(-6+nf)*(v*v))+3*(-6+nf)*(v*v*v)))/
     & (9._dp*w*(omw*omw)))+(u*H3(0,0,0)*
     & (50-192/omw-7*(-37+2*nf)*v-82/w+
     & (v*(914+2*(749+15*NFV)*u-108*nf*(1+2*u)+
     & (163-60*NFV)*(u*u)+(-46+30*NFV)*(u*u*u)))/
     & (omu*omu*omu*omu)+
     & 2*u*(-24/omw+96/(omw*omw)+(-40+w)/(w*w))+80/(w*w)))/18._dp+
     & (H2(0,1)*((-3*(10*omv+u*(-38+268*v-12*nf*v+
     & u*(28-171*v+6*nf*v)+10*NFV*(-3+3*u-12*v+8*u*v))))/
     & (omu*omu)-(u*(6*omw*(32+45*NFV*(-1+u)-22*u)*(w*w)+
     & 6*(-37+45*NFV)*u*(w*w)+
     & omw*omw*(w*(54*u+w*(186-402*v*zeta2))+
     & 30*NFV*(u*(-2+5*w)+2*(u*u)+6*(-1+6*v*zeta2)*(w*w)))))/
     & (omw*omw*(w*w))))/54._dp+
     & G2(3,3)*((16*u*v*zeta2)/3._dp-(4*u*v*H2(0,0))/3._dp+
     & (H1(0)*(2*(1-10*v)*v+(v*(-8-2*v+v*v))/u+
     & u*u*(-18/omw+128/(omw*omw)+(-151+6*w)/(w*w))+
     & u*(24-128/omw-157/w+151/(w*w)+
     & v*(-14+4*nf+20*NFV*(-1+1/(w*w*w))))))/18._dp)+
     & G3(3,3,0)*((16*u*v*H1(0))/9._dp+
     & (2*v*(-1+10*v)+(v*(8+2*v-v*v))/u+
     & u*u*(18/omw-128/(omw*omw)+(151-6*w)/(w*w))+
     & u*(-24+128/omw+157/w-151/(w*w)+
     & v*(14-4*nf+20*NFV-(20*NFV)/(w*w*w))))/18._dp)+
     & (H3(0,0,1)*(u*(38+10/omw-35*v)+18*v*(-1+1/(omu*omu))-
     & (2*(5*w+19*omw*w+(19+30*NFV*v-19*w)*(omw*omw))*(u*u))/
     & (w*(omw*omw))-((-38+(47+60*NFV)*v)*(u*u*u))/(w*w)-
     & (2*(19+(-19+10*NFV)*v)*(u*u*u*u))/(w*w*w)+
     & (38*(u*u*u*u*u))/(w*w*w)))/18._dp+
     & G3(3,2,0)*(-(u*v*H1(0))/3._dp-(v*
     & ((58-60*omv*NFV-38*v)*(u*u*u*u)+(-6+20*NFV)*(u*u*u*u*u)+
     & u*u*u*(-117+212*v+60*NFV*(omv*omv)-77*(v*v))-
     & omv*omv*omv*(10-2*v+v*v)-
     & u*(omv*omv)*(8-52*v+17*(v*v))+
     & u*u*(83+5*(-53+12*NFV)*v+(245-60*NFV)*(v*v)+
     & (-63+20*NFV)*(v*v*v))))/(18._dp*u*(w*w*w)))+
     & G3(3,0,2)*((-2*(1+10*NFV)*u*v*H1(0))/3._dp-
     & (v*((58-60*omv*NFV-38*v)*(u*u*u*u)+(-6+20*NFV)*(u*u*u*u*u)+
     & u*u*u*(-117+212*v+60*NFV*(omv*omv)-77*(v*v))-
     & omv*omv*omv*(10-2*v+v*v)-
     & u*(omv*omv)*(8-52*v+17*(v*v))+
     & u*u*(83+5*(-53+12*NFV)*v+(245-60*NFV)*(v*v)+
     & (-63+20*NFV)*(v*v*v))))/(18._dp*u*(w*w*w)))+
     & G2(3,2)*(((-67+180*NFV)*u*v*zeta2)/9._dp+(4*u*v*H2(0,0))/9._dp+
     & ((11-60*NFV)*u*v*H2(0,1))/9._dp+((1+20*NFV)*u*v*H2(1,0))/3._dp+
     & (v*H1(0)*((58-60*omv*NFV-38*v)*(u*u*u*u)+
     & (-6+20*NFV)*(u*u*u*u*u)+
     & u*u*u*(-117+212*v+60*NFV*(omv*omv)-77*(v*v))-
     & omv*omv*omv*(10-2*v+v*v)-
     & u*(omv*omv)*(8-52*v+17*(v*v))+
     & u*u*(83+5*(-53+12*NFV)*v+(245-60*NFV)*(v*v)+
     & (-63+20*NFV)*(v*v*v))))/(18._dp*u*(w*w*w)))+
     & G3(1,2,0)*((-11*u*v*H1(0))/9._dp-(16*u*v*H1(1))/9._dp+
     & (v*(-27*u*(omv*omv)+9*(omv*omv*omv)+
     & (NFV*(130-60*v)+26*(-2+v)+5*nf*(-5+3*v))*(u*u*u*u*u*u)+
     & (2+5*nf-20*NFV)*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(178-226*v+55*(v*v)+5*nf*(10-12*v+3*(v*v))-
     & 10*NFV*(29-33*v+6*(v*v)))-
     & u*u*(7+20*NFV-91*v+5*nf*(omv*omv*omv)+125*(v*v)-
     & 41*(v*v*v))+u*u*u*u*
     & (-242+492*v-265*(v*v)+10*NFV*(25-48*v+21*(v*v))+
     & 31*(v*v*v)+5*nf*(-10+18*v-9*(v*v)+v*v*v))+
     & u*u*u*(139-410*v-5*nf*(-5+2*v)*(omv*omv)+335*(v*v)-
     & 82*(v*v*v)+10*NFV*(-5+21*v-15*(v*v)+3*(v*v*v)))))/
     & (18._dp*u*(omu*omu)*(w*w*w)))+
     & G3(1,0,2)*((4*(-7+15*NFV)*u*v*H1(0))/9._dp-(16*u*v*H1(1))/9._dp+
     & (v*(-27*u*(omv*omv)+9*(omv*omv*omv)+
     & (NFV*(130-60*v)+26*(-2+v)+5*nf*(-5+3*v))*(u*u*u*u*u*u)+
     & (2+5*nf-20*NFV)*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(178-226*v+55*(v*v)+5*nf*(10-12*v+3*(v*v))-
     & 10*NFV*(29-33*v+6*(v*v)))-
     & u*u*(7+20*NFV-91*v+5*nf*(omv*omv*omv)+125*(v*v)-
     & 41*(v*v*v))+u*u*u*u*
     & (-242+492*v-265*(v*v)+10*NFV*(25-48*v+21*(v*v))+
     & 31*(v*v*v)+5*nf*(-10+18*v-9*(v*v)+v*v*v))+
     & u*u*u*(139-410*v-5*nf*(-5+2*v)*(omv*omv)+335*(v*v)-
     & 82*(v*v*v)+10*NFV*(-5+21*v-15*(v*v)+3*(v*v*v)))))/
     & (18._dp*u*(omu*omu)*(w*w*w)))+
     & G2(1,0)*((-40*u*v*zeta2)/9._dp+(2*u*v*H2(0,0))/9._dp+(7*u*v*H2(0,1))/9._dp+
     & (11*u*v*H2(1,0))/9._dp+(16*u*v*H2(1,1))/9._dp-
     & (v*H1(0)*(-27*u*(omv*omv)+9*(omv*omv*omv)+
     & (NFV*(130-60*v)+26*(-2+v)+5*nf*(-5+3*v))*(u*u*u*u*u*u)+
     & (2+5*nf-20*NFV)*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(178-226*v+55*(v*v)+5*nf*(10-12*v+3*(v*v))-
     & 10*NFV*(29-33*v+6*(v*v)))-
     & u*u*(7+20*NFV-91*v+5*nf*(omv*omv*omv)+125*(v*v)-
     & 41*(v*v*v))+u*u*u*u*
     & (-242+492*v-265*(v*v)+10*NFV*(25-48*v+21*(v*v))+
     & 31*(v*v*v)+5*nf*(-10+18*v-9*(v*v)+v*v*v))+
     & u*u*u*(139-410*v-5*nf*(-5+2*v)*(omv*omv)+335*(v*v)-
     & 82*(v*v*v)+10*NFV*(-5+21*v-15*(v*v)+3*(v*v*v)))))/
     & (18._dp*u*(omu*omu)*(w*w*w))-
     & (v*H1(1)*(-27*u*(omv*omv)+9*(omv*omv*omv)+
     & (NFV*(130-60*v)+26*(-2+v)+5*nf*(-5+3*v))*(u*u*u*u*u*u)+
     & (2+5*nf-20*NFV)*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(178-226*v+55*(v*v)+5*nf*(10-12*v+3*(v*v))-
     & 10*NFV*(29-33*v+6*(v*v)))-
     & u*u*(7+20*NFV-91*v+5*nf*(omv*omv*omv)+125*(v*v)-
     & 41*(v*v*v))+u*u*u*u*
     & (-242+492*v-265*(v*v)+10*NFV*(25-48*v+21*(v*v))+
     & 31*(v*v*v)+5*nf*(-10+18*v-9*(v*v)+v*v*v))+
     & u*u*u*(139-410*v-5*nf*(-5+2*v)*(omv*omv)+335*(v*v)-
     & 82*(v*v*v)+10*NFV*(-5+21*v-15*(v*v)+3*(v*v*v)))))/
     & (18._dp*u*(omu*omu)*(w*w*w)))+
     & G3(1,1,2)*(4*u*v*H1(0)+(5*(1+4*NFV)*u*v*H1(1))/3._dp+
     & (v*(omv*omv*omv*(9-30*NFV*u+4*nf*(u*u))+
     & u*(omv*omv)*(-43+16*v+4*nf*(-5+2*v)*(u*u))+
     & u*u*(-143+547*v+(-289+97*v-4*nf*(-5+3*v))*(u*u*u*u)+
     & (39-4*nf)*(u*u*u*u*u)-629*(v*v)+
     & u*u*u*(851-670*v+68*(v*v)-4*nf*(10-12*v+3*(v*v)))+
     & u*(769-1768*v+1113*(v*v)-132*(v*v*v))+225*(v*v*v)+
     & u*u*(-1193+1719*v-504*(v*v)+10*(v*v*v)-
     & 4*nf*(-10+18*v-9*(v*v)+v*v*v))+
     & 10*NFV*(25-72*v+(-1+6*v)*(u*u*u*u)+2*(u*u*u*u*u)+
     & 63*(v*v)+u*u*u*(-25+3*v+6*(v*v))+
     & u*u*(64-78*v+15*(v*v))-18*(v*v*v)+
     & u*(-62+132*v-75*(v*v)+9*(v*v*v))))))/
     & (18._dp*u*(omu*omu)*(w*w*w)))+
     & G2(1,1)*(((-23+100*NFV)*u*v*zeta2)/3._dp+((7-20*NFV)*u*v*H2(0,1))/3._dp-
     & 4*u*v*H2(1,0)-(5*(1+4*NFV)*u*v*H2(1,1))/3._dp-
     & (v*H1(0)*(omv*omv*omv*(9-30*NFV*u+4*nf*(u*u))+
     & u*(omv*omv)*(-43+16*v+4*nf*(-5+2*v)*(u*u))+
     & u*u*(-143+547*v+(-289+97*v-4*nf*(-5+3*v))*(u*u*u*u)+
     & (39-4*nf)*(u*u*u*u*u)-629*(v*v)+
     & u*u*u*(851-670*v+68*(v*v)-4*nf*(10-12*v+3*(v*v)))+
     & u*(769-1768*v+1113*(v*v)-132*(v*v*v))+225*(v*v*v)+
     & u*u*(-1193+1719*v-504*(v*v)+10*(v*v*v)-
     & 4*nf*(-10+18*v-9*(v*v)+v*v*v))+
     & 10*NFV*(25-72*v+(-1+6*v)*(u*u*u*u)+2*(u*u*u*u*u)+
     & 63*(v*v)+u*u*u*(-25+3*v+6*(v*v))+
     & u*u*(64-78*v+15*(v*v))-18*(v*v*v)+
     & u*(-62+132*v-75*(v*v)+9*(v*v*v))))))/
     & (18._dp*u*(omu*omu)*(w*w*w))-
     & (v*H1(1)*(omv*omv*omv*(9-30*NFV*u+4*nf*(u*u))+
     & u*(omv*omv)*(-43+16*v+4*nf*(-5+2*v)*(u*u))+
     & u*u*(-143+547*v+(-289+97*v-4*nf*(-5+3*v))*(u*u*u*u)+
     & (39-4*nf)*(u*u*u*u*u)-629*(v*v)+
     & u*u*u*(851-670*v+68*(v*v)-4*nf*(10-12*v+3*(v*v)))+
     & u*(769-1768*v+1113*(v*v)-132*(v*v*v))+225*(v*v*v)+
     & u*u*(-1193+1719*v-504*(v*v)+10*(v*v*v)-
     & 4*nf*(-10+18*v-9*(v*v)+v*v*v))+
     & 10*NFV*(25-72*v+(-1+6*v)*(u*u*u*u)+2*(u*u*u*u*u)+
     & 63*(v*v)+u*u*u*(-25+3*v+6*(v*v))+
     & u*u*(64-78*v+15*(v*v))-18*(v*v*v)+
     & u*(-62+132*v-75*(v*v)+9*(v*v*v))))))/
     & (18._dp*u*(omu*omu)*(w*w*w)))-
     & (u*H3(0,1,0)*(-3*v*(omw*omw)*
     & (-280+2*(-284+5*NFV)*u+36*nf*(1+2*u)+(9-20*NFV)*(u*u)+
     & 2*(-2+5*NFV)*(u*u*u))*(w*w*w)+
     & omu*omu*omu*omu*(60*NFV*u*(omw*omw*omw)*(w*w)+
     & omw*omw*(20*NFV*(u*u)*
     & (3*(-1+2*v)*w+u*(-3+4*v+6*w)+3*(u*u))+
     & w*(-80+86*w+(-38+47*v)*(u*u)+38*(u*u*u)+
     & u*(80+32*w-38*(w*w))+2*(-5-37*v+3*nf*v)*(w*w)))+
     & 90*u*(w*w*w)+2*omw*(-45+2*u)*(w*w*w))))/
     & (18._dp*(omu*omu*omu*omu)*(omw*omw)*(w*w*w))+
     & (u*H3(1,1,0)*(-((39+10*u+60*NFV*(-1+2*u))/omw)+
     & (-32+NFV*(90-60*u)+39*u)/(omw*omw)+
     & ((32-90*NFV)*u)/(omw*omw*omw)+
     & (omu*omu*omu*omu*(w*
     & (4*u*w*(-20+19*w)+w*(4+(6+23*v-4*nf*v)*w)+
     & (76-94*v)*(u*u)-76*(u*u*u))-
     & 40*NFV*(3*(-1+2*v)*w*(u*u)+(-3+4*v+6*w)*(u*u*u)+
     & 3*(u*u*u*u)-3*u*(-1+w)*(w*w)-3*(w*w*w)))+
     & 3*v*(-256+2*(-314+5*NFV)*u+36*nf*(1+2*u)+
     & (57-20*NFV)*(u*u)+2*(-8+5*NFV)*(u*u*u))*(w*w*w))/
     & (omu*omu*omu*omu*(w*w*w))))/18._dp+
     & (u*H3(1,0,0)*((90-4*u)/omw-(60*omw*NFV*u)/w-(90*u)/(omw*omw)+
     & (-(omu*omu*omu*omu*(20*NFV*(u*u)*
     & (3*(-1+2*v)*w+u*(-3+4*v+6*w)+3*(u*u))+
     & w*(2*u*(17-19*w)*w+4*w*(1+(-2-52*v+3*nf*v)*w)+
     & (-38+47*v)*(u*u)+38*(u*u*u))))+
     & v*(914+2*(749+15*NFV)*u-108*nf*(1+2*u)+
     & (163-60*NFV)*(u*u)+(-46+30*NFV)*(u*u*u))*(w*w*w))/
     & (omu*omu*omu*omu*(w*w*w))))/18._dp+
     & G2(0,3)*((22*u*v*zeta2)/9._dp+(16*u*v*H2(0,0))/9._dp-(u*v*H2(0,1))/9._dp-
     & (u*v*H2(1,0))/9._dp+(u*H1(0)*
     & ((8*(25+8*u))/omw-(60*omw*NFV*u)/w-(200*u)/(omw*omw)+
     & (-(omu*omu*omu*omu*
     & (20*NFV*(u*u)*
     & (3*(-1+2*v)*w+u*(-3+4*v+6*w)+3*(u*u))+
     & w*(2*u*(17-19*w)*w+w*(4+(60+67*v-nf*v)*w)+
     & (-38+47*v)*(u*u)+38*(u*u*u))))+
     & v*(-878-2*(794+15*NFV)*u+108*nf*(1+2*u)+
     & (-91+60*NFV)*(u*u)+(28-30*NFV)*(u*u*u))*(w*w*w))/
     & (omu*omu*omu*omu*(w*w*w))))/18._dp)+
     & G3(0,3,0)*((-8*u*v*H1(0))/9._dp+(u*v*H1(1))/9._dp+
     & (u*((-8*(25+8*u))/omw+(60*omw*NFV*u)/w+(200*u)/(omw*omw)+
     & (omu*omu*omu*omu*
     & (20*NFV*(u*u)*(3*(-1+2*v)*w+u*(-3+4*v+6*w)+
     & 3*(u*u))+w*
     & (2*u*(17-19*w)*w+w*(4+(60+67*v-nf*v)*w)+
     & (-38+47*v)*(u*u)+38*(u*u*u)))+
     & v*(878+2*(794+15*NFV)*u-108*nf*(1+2*u)+
     & (91-60*NFV)*(u*u)+(-28+30*NFV)*(u*u*u))*(w*w*w))/
     & (omu*omu*omu*omu*(w*w*w))))/18._dp)-
     & (u*H3(1,0,1)*(-18*(-2+u)*v*(omw*omw*omw)*(w*w*w)+
     & omu*omu*(2*(16-45*NFV)*u*(w*w*w)-
     & omw*(32+41*u+30*NFV*(-3+2*u))*(w*w*w)+
     & (41+24*u-60*NFV*(-1+2*u))*(omw*omw)*(w*w*w)+
     & omw*omw*omw*(w*(-38*u*(-1+w)*w+(-38+47*v)*(u*u)+
     & 38*(u*u*u)+(-24+35*v)*(w*w))+
     & 20*NFV*(3*(-1+2*v)*w*(u*u)+(-3+4*v+6*w)*(u*u*u)+
     & 3*(u*u*u*u)-3*u*(-1+w)*(w*w)+6*(w*w*w))))))/
     & (18._dp*(omu*omu)*(omw*omw*omw)*(w*w*w))+
     & (H2(0,0)*(3*w*(omw*omw)*((22+24*nf*v)*(u*u*u*u*u)+
     & 2*(u*u*u*u)*(-887+22*v+6*nf*(18-v+4*(v*v)))+20*v*(w*w)+
     & 2*(u*u*u)*(1752+11*(v*v)+
     & 6*nf*(-36+99*v+2*(v*v)+2*(v*v*v))-
     & v*(2595+(197+70*NFV)*(w*w)))+
     & u*(22-64*(26+15*nf)*(v*v)+588*nf*(v*v*v)+
     & v*(2485+264*nf-(4937+180*NFV)*(w*w)))+
     & u*u*(-1774-3416*(v*v)+
     & 12*nf*(18-122*v+128*(v*v)+3*(v*v*v))+
     & v*(2661+(253+320*NFV)*(w*w))))+
     & omu*omu*omu*(-120*NFV*(omw*omw*omw)*(u*u*u*u)-
     & 48*omw*u*(nf*(4+u)-3*(35+4*u))*(w*w*w)+
     & 48*(-97+4*nf)*(u*u)*(w*w*w)+
     & omw*omw*(w*(54*(-1+w)*(u*u*u)-6*(v*v)*(w*w)+
     & 3*(u*u)*(-351+36*nf-310*w+18*(w*w))+
     & u*(-2529-453*w+(-1500+1119*v+2382*v*zeta2)*(w*w)+
     & nf*(324+108*w+(48-76*v)*(w*w))))+
     & 30*NFV*u*(2*(u*u)*(1-10*w+7*(w*w))-12*v*(w*w*w)+
     & u*(-4+7*w-31*(w*w)+10*(w*w*w)))))))/
     & (108._dp*(omu*omu*omu)*(omw*omw)*(w*w*w))+
     & (H2(1,0)*(3*omu*(-2529+(-8430+5589*v+
     & 20*NFV*u*(-3+3*u-9*v+7*u*v))*w)*(omw*omw)*(w*w)+
     & 2*w*(omw*omw)*(2*nf*u*
     & (-108+686*v+2*v*(-40+19*v)*(u*u*u*u)+19*v*(u*u*u*u*u)+
     & v*(u*u)*(-217+270*v-42*(v*v))-994*(v*v)+416*(v*v*v)+
     & u*u*u*(-54+169*v-122*(v*v)+19*(v*v*v))+
     & u*(162-577*v+322*(v*v)+93*(v*v*v)))-7587*(-1+v)*(w*w))
     & -3*w*(omu*omu)*(omw*omw)*
     & (843-4215*w+5*(-678+157*v)*(w*w))-
     & 3*(omu*omu*omu)*(2*(20*NFV*u+9*w)*(omw*omw*omw)*(u*u*u)-
     & 4*omw*u*(-40+7*u+15*NFV*(-3+4*u))*(w*w*w)-
     & 20*(1+9*NFV)*(u*u)*(w*w*w)-
     & omw*omw*(w*(843-1686*w+36*(u*u*u)-
     & 2*(19-129*v+v*v)*(w*w)+u*u*(351-82*w+18*(w*w))+
     & u*(843+471*w+(-144+373*v+282*v*zeta2)*(w*w)))+
     & 10*NFV*u*(8*(u*u*u)+2*(u*u)*(-1+4*w+7*(w*w))-
     & 12*(1+v)*(w*w*w)+u*(4-3*w+w*w+10*(w*w*w)))))))/
     & (108._dp*(omu*omu*omu)*(omw*omw)*(w*w*w))+
     & (720*(-157+165*NFV)*u*v*zeta4+
     & 180*u*((3*(8*(26+29*v*zeta3)-2*u*(312+(-344+5*NFV)*v*zeta3)+
     & (624+5*(-21+4*NFV)*v*zeta3)*(u*u)-
     & 2*(104+(-14+5*NFV)*v*zeta3)*(u*u*u)))/(omu*omu*omu*omu)
     & +(-144*u*zeta3*(w*w)+
     & 4*omw*(-147+36*zeta3+16*u*zeta3)*(w*w)+
     & zeta3*(omw*omw)*(160-172*w+4*u*(-40+3*w)-
     & (52+93*v)*(w*w)))/(omw*omw*(w*w)))+
     & 20*nf*(12*u*(38/omw+3*v*zeta3-
     & (50+81*v*zeta3+6*u*(-25+27*v*zeta3)+150*(u*u)-
     & 50*(u*u*u))/(omu*omu*omu*omu))+
     & 6*u*zeta2*((-12*(4+u))/omw+(48*u)/(omw*omw)+
     & (-3*(2*v*(u*u*u*u)+v*(-353+598*v-272*(v*v))+
     & u*u*u*(54-58*v+4*(v*v))+
     & u*(54+409*v-328*(v*v)-54*(v*v*v))+
     & 2*(u*u)*(-54-56*(v*v)+v*v*v))+
     & omu*omu*omu*(-81-27*u-27*w+4*(3+19*v)*(w*w)))/
     & (omu*omu*omu*(w*w))))+
     & (90*zeta2*(3*omu*(-843+(-2810+3878*v)*w)*(omw*omw)*(w*w)+
     & 6*(omw*omw)*(843*(1-2*v)*w+
     & 20*NFV*u*(8-25*v+(-4+6*v)*(u*u*u)+
     & u*(-20+42*v-13*(v*v))+19*(v*v)+
     & u*u*(16-23*v+6*(v*v))))*(w*w)+
     & w*(omu*omu)*(omw*omw)*
     & (-843+4215*w+(3450-1664*v)*(w*w))+
     & omu*omu*omu*(6*omw*u*(-36-90*NFV-99*u+65*NFV*u)*
     & (w*w*w)+4*(-142+135*NFV)*(u*u)*(w*w*w)+
     & omw*omw*(w*(843-1686*w+(18-72*w)*(u*u*u)+
     & 2*(29-8*v+4*(v*v))*(w*w)-
     & 3*(u*u)*(-117+80*w+24*(w*w))+
     & u*(843+791*w+(400-1492*v)*(w*w)))+
     & 10*NFV*u*(4*(-3+4*v)*(u*u*u)+16*(u*u*u*u)+
     & 24*(-1+(-5+2*v)*w)*(w*w)-
     & 2*(u*u)*(1-6*w+28*(w*w))-
     & u*(-4+35*w+13*(w*w)+40*(w*w*w)))))))/
     & (omu*omu*omu*(omw*omw)*(w*w*w)))/3240._dp+
     & H1(1)*((u*v*H2(0,0))/6._dp+(u*v*H2(1,0))/6._dp+
     & ((6*(83+(41+4*nf)*v))/omu+((-1461+52*nf)*v)/(omu*omu)+
     & (120*NFV*u*(-5+v*(9+12*zeta2)-u*(-10+v*(12+6*zeta2))+
     & (-5+v*(3+6*zeta2))*(u*u)))/(omu*omu*omu)+
     & (18*u*v*zeta2*(-315-485*u+36*nf*(1+2*u)-52*(u*u)+
     & 9*(u*u*u)))/(omu*omu*omu*omu)+
     & (6*omw*u*(-96+35*u-90*NFV*(-3+2*u))*zeta2*(w*w*w)+
     & u*(276+210*(-1+4*u)*zeta2-90*NFV*(3+6*(-2+4*u)*zeta2))*
     & (omw*omw)*(w*w*w)+36*(16-45*NFV)*zeta2*(u*u)*(w*w*w)+
     & omw*omw*omw*(w*(6*u*zeta2*
     & (2*u*w*(-21+19*w)+w*(4+(-144-61*v+8*nf*v)*w)+
     & (38-47*v)*(u*u)-38*(u*u*u))+
     & (-498+v*(1215-76*nf+768*u*zeta3))*(w*w))-
     & 20*NFV*u*(-3*(u+w)*(w*w)+
     & 6*zeta2*(3*(-1+2*v)*w*(u*u)+(-3+4*v+6*w)*(u*u*u)+
     & 3*(u*u*u*u)-3*u*(-1+w)*(w*w)+6*(-3+2*v)*(w*w*w)))
     & ))/(omw*omw*omw*(w*w*w)))/108._dp)+
     & ((u*v*H1(0))/2._dp+(u*v*H1(1))/2._dp-((-4+omw)*u*v)/(2._dp*(omw*omw)))*
     & (G1(0)*G1(0)*G1(0))+(-(u*v*H1(0))/18._dp-((-2+u)*u*v)/(18._dp*(omu*omu)))*
     & (G1(2)*G1(2)*G1(2))+((-17*u*v*H2(0,0))/18._dp-(17*u*v*H2(1,0))/18._dp+
     & (u*v*H1(1)*(8-2*v+(42-32*v)*(u*u)-17*(u*u*u)+
     & u*(-18+64*v-15*(v*v))+30*(v*v)))/
     & (18._dp*(omu*omu)*(omw*omw))-
     & (u*(-5*(51+48*omv*v*zeta2)*(u*u*u)+120*v*zeta2*(u*u*u*u)+
     & 2*v*(240-60*v+(-171+60*zeta2)*(v*v)+
     & 3*nf*(-4+v+2*(v*v)))+
     & u*(-204+3*(-343+18*nf)*v+(-393+12*nf+240*zeta2)*(v*v)+
     & (171-6*nf-240*zeta2)*(v*v*v))+
     & u*u*(459+v*(378-24*nf+120*zeta2)+
     & (171-6*nf-480*zeta2)*(v*v)+120*zeta2*(v*v*v))))/
     & (54._dp*(omu*omu)*(omw*omw)))*(H1(0)*H1(0))+
     & G1(0)*G1(0)*(-(u*v*G2(0,2))/2._dp+(u*v*G2(1,2))/2._dp-(u*v*G2(2,0))/2._dp+
     & (u*v*G2(3,0))/2._dp-(u*v*G1(3)*H1(0))/2._dp+
     & G1(1)*(-(u*v*H1(0))/2._dp-(u*v*H1(1))/2._dp)-(u*v*H2(0,0))/2._dp-
     & (u*v*H2(1,0))/2._dp+G1(2)*((u*v*H1(0))/2._dp+
     & ((-2+u)*u*v)/(2._dp*(omu*omu)))+
     & (u*v*H1(1)*(-12+51*u+nf*(omu*omu)-30*(u*u)))/(18._dp*(omu*omu))+
     & H1(0)*(-(u*v*H1(1))+(u*(27-108/omw+(-30+nf)*v)+
     & 9*v*(-1+1/(omu*omu))-(27*(-4+omw)*(u*u))/(omw*omw))/18._dp
     & )-(u*(36*v*zeta2*(u*u*u)+
     & v*(84+nf*(-4+v)-21*v-36*zeta2*(v*v))+
     & u*u*(45-v*(-30+nf+36*zeta2)+72*zeta2*(v*v))+
     & u*(-36+(-96+5*nf)*v-(-30+nf+72*zeta2)*(v*v)+
     & 36*zeta2*(v*v*v))))/(18._dp*omu*(omw*omw))-u*v*(H1(0)*H1(0)))+
     & G1(2)*G1(2)*((u*v*G2(0,2))/18._dp-(u*v*G2(1,2))/18._dp+(u*v*G2(2,0))/18._dp-
     & (u*v*G2(3,0))/18._dp+(u*v*G1(3)*H1(0))/18._dp+(u*v*H2(0,0))/18._dp+
     & (u*v*H2(1,0))/18._dp+((3*u*(-4+5*u+v))/(omu*omw)+
     & v*(9-12*u*zeta2-9/(omu*omu)))/54._dp+
     & ((-2+u)*u*v*H1(1))/(6._dp*(omu*omu))+
     & H1(0)*((u*v*H1(1))/9._dp+(u*v*
     & (4-v+(-4+6*v)*(u*u*u)+3*(u*u*u*u)-3*(v*v)+
     & u*u*(3-7*v+3*(v*v))-u*(9+4*v+3*(v*v))))/
     & (18._dp*(omu*omu)*(omw*omw)))+(u*v*(H1(0)*H1(0)))/9._dp)-
     & (17*u*v*(4-v+(4+v)*(u*u)-2*(v*v)+u*(-9-2*v+v*v))*
     & (H1(0)*H1(0)*H1(0)))/(18._dp*(omu*omu)*(omw*omw))+
     & ((u*v*H2(0,0))/18._dp+(u*v*H2(1,0))/18._dp+
     & ((3*u*(-4+5*u+v))/(omu*omw)+v*(9-12*u*zeta2-9/(omu*omu)))/
     & 54._dp)*(H1(1)*H1(1))+G2(1,2)*
     & ((22*u*v*H2(0,0))/9._dp+(32*u*v*H2(1,0))/9._dp+
     & H1(0)*(-(u*v*H1(1))/9._dp+(v*
     & (54+(-85+8*nf-40*NFV)*u-72/(omu*omu)))/18._dp)+
     & (v*H1(1)*(7+2*(-21+nf)*u+5*NFV*(9-4*u-6/(omu*omu))-
     & 17/(omu*omu)))/9._dp+
     & ((96+540*NFV)/omu+(6*v*
     & (-13+(54-8*nf)*u-30*NFV*(-3+u*u)+4*(7+nf)*(u*u)))/
     & (omu*omu)-(6*omw*(-1+u)*u*(16+25*u+15*NFV*(-3+7*u))*
     & (w*w)+12*(-16+45*NFV)*(-1+u)*(u*u)*(w*w)+
     & omw*omw*(w*((54+w*(-198+v*(1119-76*nf+870*zeta2)))*
     & (u*u)+54*w*(u*u*u)+54*(u*u*u*u)-6*u*w*(-40+v*v)+
     & 6*w*(10-11*v+v*v))+
     & 60*NFV*u*(w*(1+5*w)*(u*u)+7*w*(u*u*u)+2*(u*u*u*u)+
     & 12*(w*w)+u*(-2+w-2*(1+3*v)*(w*w)))))/
     & (u*(omw*omw)*(w*w)))/108._dp+(17*u*v*(H1(0)*H1(0)))/18._dp-
     & (u*v*(H1(1)*H1(1)))/18._dp)+G2(3,0)*
     & ((28*u*v*H2(0,0))/9._dp+(17*u*v*H2(0,1))/9._dp+(u*v*H2(1,0))/3._dp+
     & H1(0)*(-(u*v*H1(1))/9._dp+(u*v*
     & (-36+5*nf-(2*(-64+144*v+(-57+51*v)*(u*u)+17*(u*u*u)-
     & 57*(v*v)+3*u*(48-38*v+17*(v*v))+17*(v*v*v)))/
     & (omw*omw*(w*w))))/18._dp)+
     & (v*H1(1)*((49-60*omv*NFV-29*v)*(u*u*u*u)+
     & (-3+20*NFV)*(u*u*u*u*u)+
     & 2*(u*u*u)*(-54+97*v+30*NFV*(omv*omv)-34*(v*v))-
     & omv*omv*omv*(10-2*v+v*v)-
     & u*(omv*omv)*(8-52*v+17*(v*v))+
     & 4*(u*u)*(20+(-64+15*NFV)*v+(59-15*NFV)*(v*v)+
     & 5*(-3+NFV)*(v*v*v))))/(18._dp*u*(w*w*w))-
     & (2529*omu*(3+(10+7*v)*w)*(omw*omw)*(w*w)+
     & 54*w*(omw*omw)*(6*nf*u*
     & ((2+v)*(u*u*u)+u*u*(-4+9*v+2*(v*v))+
     & v*(2-8*v+5*(v*v))+u*(2-12*v+12*(v*v)+v*v*v))-
     & 281*(1+v)*(w*w))-
     & 3*omw*(omu*omu)*(10*NFV*u*
     & (6*v*(-1+3*v)*(omv*omv*omv)+(-9+6*v)*(u*u*u*u*u)+
     & u*u*u*u*(30-75*v+26*(v*v))+
     & u*u*u*(-39+183*v-197*(v*v)+46*(v*v*v))+
     & u*v*(60-201*v+243*(v*v)-110*(v*v*v)+12*(v*v*v*v))+
     & u*u*(18-168*v+336*(v*v)-223*(v*v*v)+38*(v*v*v*v)))+
     & omw*w*(-843+4215*w+(3386+857*v)*(w*w)))+
     & w*(omu*omu*omu)*(12*omw*u*(8-37*u+2*nf*(4+u))*(w*w)+
     & 48*(5-2*nf)*(u*u)*(w*w)+
     & omw*omw*(-2529+5058*w+54*(-1+w)*(u*u*u)-
     & 6*(-15-9*v+v*v)*(w*w)+
     & 3*(u*u)*(-351+36*nf-274*w+18*(w*w))+
     & u*(-2529-453*w+(-372+1119*v+1542*v*zeta2)*(w*w)-
     & 4*nf*(-81-27*w+(6+19*v)*(w*w))))))/
     & (108._dp*(omu*omu*omu)*(omw*omw)*(w*w*w))+
     & (17*u*v*(H1(0)*H1(0)))/18._dp-(u*v*(H1(1)*H1(1)))/18._dp)+
     & G2(2,0)*((-10*u*v*H2(0,0))/9._dp+(u*v*H2(0,1))/9._dp-(u*v*H2(1,0))/3._dp-
     & (u*v*H1(1)*(-117+(301+90*NFV)*u+4*nf*(omu*omu*omu)-
     & 2*(134+45*NFV)*(u*u)+(84+40*NFV)*(u*u*u)))/
     & (18._dp*(omu*omu*omu))+
     & H1(0)*((u*v*H1(1))/9._dp+(u*
     & (6+98/omw+(123-9*nf+20*NFV)*v-4/w)-
     & (v*(omv*omv))/u+(-2/omw+4/w-98/(omw*omw))*(u*u)+
     & (v*(-20*omv-4*u*(-243+27*nf+10*NFV+20*v)+
     & (1444-216*nf+70*NFV+120*v)*(u*u)+
     & (177-60*NFV-80*v)*(u*u*u)+
     & (-44+30*NFV+20*v)*(u*u*u*u)))/(omu*omu*omu*omu))/18._dp)
     & +(u*(-516-240/omw+v*(1119-76*nf+462*zeta2))+
     & 18*(3+23/omw-3/w+14/(omw*omw))*(u*u)+(54*(u*u*u))/w-
     & (3*(2*(10-11*v+v*v)+2*(u*u)*(11+(-6+5*nf)*v+v*v)-
     & u*(42+(-91+16*nf)*v+4*(v*v))))/(omu*omu)+
     & (30*NFV*(u*u)*(3-12*v+(-9+6*v)*(u*u*u*u*u)+77*(v*v)+
     & u*u*u*u*(39-68*v+20*(v*v))-118*(v*v*v)+
     & u*u*u*(-66+216*v-153*(v*v)+26*(v*v*v))+
     & u*(-21+122*v-319*(v*v)+252*(v*v*v)-42*(v*v*v*v))+
     & 54*(v*v*v*v)+u*u*
     & (54-264*v+375*(v*v)-136*(v*v*v)+12*(v*v*v*v))))/
     & (omw*(omu*omu*omu)*(w*w)))/108._dp-(17*u*v*(H1(0)*H1(0)))/18._dp+
     & (u*v*(H1(1)*H1(1)))/18._dp)+G2(0,2)*
     & ((-10*u*v*H2(0,0))/3._dp+(u*v*H2(0,1))/9._dp-(23*u*v*H2(1,0))/9._dp-
     & (u*v*H1(1)*(-117+(301+90*NFV)*u+4*nf*(omu*omu*omu)-
     & 2*(134+45*NFV)*(u*u)+(84+40*NFV)*(u*u*u)))/
     & (18._dp*(omu*omu*omu))+
     & (u*(-516-240/omw+v*(1119-76*nf+222*zeta2))+
     & 18*(3+23/omw-3/w+14/(omw*omw))*(u*u)+(54*(u*u*u))/w-
     & (3*(2*(10-11*v+v*v)+2*(u*u)*(11+(-6+5*nf)*v+v*v)-
     & u*(42+(-91+16*nf)*v+4*(v*v))))/(omu*omu)+
     & (30*NFV*(u*u)*(3-12*v+(-9+6*v)*(u*u*u*u*u)+77*(v*v)+
     & u*u*u*u*(39-68*v+20*(v*v))-118*(v*v*v)+
     & u*u*u*(-66+216*v-153*(v*v)+26*(v*v*v))+
     & u*(-21+122*v-319*(v*v)+252*(v*v*v)-42*(v*v*v*v))+
     & 54*(v*v*v*v)+u*u*
     & (54-264*v+375*(v*v)-136*(v*v*v)+12*(v*v*v*v))))/
     & (omw*(omu*omu*omu)*(w*w)))/108._dp+
     & H1(0)*((u*v*H1(1))/9._dp+((u*
     & (-32+90*NFV+omw*(59+60*NFV)+
     & (12+152*v-9*nf*v+40*NFV*(3+v))*(omw*omw)))/
     & (omw*omw)-((2*(-16+45*NFV)*w+omw*(59+60*NFV)*w+
     & 12*(w+10*NFV*w)*(omw*omw)+
     & 2*(19+30*NFV)*(-1+w)*(omw*omw*omw))*(u*u))/
     & (w*(omw*omw*omw))-
     & (2*v*(10-58*u+77*(u*u)-29*(u*u*u)+
     & 5*NFV*(-3+25*u-33*(u*u)+15*(u*u*u))))/(omu*omu*omu)
     & +((-38+47*u+47*v+60*NFV*(-1+2*u+2*v))*(u*u*u))/
     & (w*w)+((20*NFV*(-3+4*u+4*v)-9*w)*(u*u*u*u))/(w*w*w)-
     & (20*NFV*(u*u*u*u*u))/(w*w*w))/18._dp)-(17*u*v*(H1(0)*H1(0)))/18._dp+
     & (u*v*(H1(1)*H1(1)))/18._dp)+H1(0)*
     & (((-57+2*nf)*u*v*H2(0,0))/18._dp+((-57+2*nf)*u*v*H2(1,0))/18._dp+
     & H1(1)*((u*v*H2(0,0))/9._dp+(u*v*H2(1,0))/9._dp-
     & (u*(348*v*zeta2*(u*u*u*u)+
     & u*u*u*(30+v*(171-6*nf-696*zeta2)+696*zeta2*(v*v))+
     & v*(-12+3*v+2*(-162+6*nf+174*zeta2)*(v*v))+
     & u*u*(-54+2*v*(-171+6*nf+174*zeta2)+
     & (333-12*nf-1392*zeta2)*(v*v)+348*zeta2*(v*v*v))+
     & u*(24+21*v+4*(-165+6*nf+174*zeta2)*(v*v)-
     & 2*(-81+3*nf+348*zeta2)*(v*v*v))))/
     & (54._dp*(omu*omu)*(omw*omw)))+
     & (-3*v*(487+3378*zeta2)*(omu*omu)*(omw*omw)*(w*w*w)+
     & 3*(927+2*v*(41+48*zeta2))*(omu*omu*omu)*(omw*omw)*(w*w*w)+
     & 15*omu*(omw*omw)*(1686*v*zeta2+
     & 8*NFV*u*(1+9*v+12*v*zeta2-u*(2+v*(12+6*zeta2))+
     & (1+v*(3+6*zeta2))*(u*u)))*(w*w*w)+
     & omw*omw*(-15174*v*zeta2-
     & 4*nf*u*(87-32*v-162*v*zeta2+u*(-261+83*v-324*v*zeta2)+
     & (261-70*v)*(u*u)+(-87+19*v)*(u*u*u)))*(w*w*w)-
     & omu*omu*omu*omu*(1800*NFV*zeta2*(omw*omw*omw)*(u*u)*(w*w)-
     & omw*u*(16*nf*(-19+2*u)+18*(196+6*u-15*NFV*u)+
     & 6*(80+333*u)*zeta2)*(w*w*w)+
     & 8*(567-50*nf+60*zeta2)*(u*u)*(w*w*w)+
     & omw*omw*(w*(w*(27*(103-45*v)*w+
     & u*(1422+1215*w-4*nf*(54+19*w)-336*v*w*zeta3)-
     & 27*(-41+4*nf)*(u*u))+
     & 6*zeta2*(-5*(-9+47*w)*(u*u*u)-45*(u*u*u*u)+
     & u*u*(240+164*w-190*(w*w))+45*v*(w*w)+
     & u*(-240+266*w+(307+25*v-8*nf*v)*(w*w))))+
     & 10*NFV*u*(-3*w*(u*(-2+7*w)+w*(-9+14*w)+2*(u*u))+
     & 6*zeta2*(30*(-1+2*v)*w*(u*u)+
     & 10*(-3+4*v+6*w)*(u*u*u)+30*(u*u*u*u)+24*v*(w*w*w)
     & )))))/(108._dp*(omu*omu*omu*omu)*(omw*omw)*(w*w*w))+
     & (u*v*(4-v+5*v*(u*u)+2*(u*u*u)-6*(v*v)+
     & u*(-9-10*v+3*(v*v)))*(H1(1)*H1(1)))/(18._dp*(omu*omu)*(omw*omw))
     & )+G1(2)*(G2(0,2)*(-(u*v)/6._dp-(u*v*H1(0))/9._dp-(u*v*H1(1))/9._dp)+
     & G2(2,0)*(-(u*v)/6._dp-(u*v*H1(0))/9._dp-(u*v*H1(1))/9._dp)+
     & G2(1,2)*((u*v)/6._dp+(u*v*H1(0))/9._dp+(u*v*H1(1))/9._dp)+
     & G2(3,0)*((u*v)/6._dp+(u*v*H1(0))/9._dp+(u*v*H1(1))/9._dp)+3*u*v*H3(0,0,0)+
     & (10*u*v*H3(0,0,1))/9._dp+(2*u*v*H3(0,1,0))/3._dp+(25*u*v*H3(1,0,0))/9._dp+
     & ((1-20*NFV)*u*v*H3(1,0,1))/3._dp+((23+60*NFV)*u*v*H3(1,1,0))/9._dp+
     & (H2(0,1)*(2*v*(-19+10*v+9/(omu*omu))-(v*(omv*omv))/u+
     & (u*(-32+90*NFV+omw*(41+60*NFV)+
     & (-20*NFV*(-6+v)+6*(-4+v))*(omw*omw)))/(omw*omw)-
     & ((-32+90*NFV+omw*(41+60*NFV)+24*(-1+5*NFV)*(omw*omw))*
     & (u*u))/(omw*omw*omw)))/18._dp+
     & H1(1)*(-(u*v*H2(0,0))/9._dp-(u*v*H2(1,0))/9._dp+
     & (6-6/omu+(24*u)/omw+
     & (u*v*(636+12*nf*(-2+u)-321*u-42*zeta2+84*u*zeta2+
     & 60*NFV*(-6+4*u+18*zeta2*(omu*omu))-42*zeta2*(u*u)))/
     & (omu*omu))/54._dp)-
     & (u*H2(1,0)*(omu*omu*omu*omu*
     & (2*(16-45*NFV)*u*w+omw*(-32+NFV*(90-60*u)+39*u)*w-
     & (39+10*u+60*NFV*(-1+2*u))*w*(omw*omw)+
     & (4-4*u+2*(3+42*v-2*nf*v+20*NFV*(3+v))*w)*
     & (omw*omw*omw))+
     & 3*v*w*(omw*omw*omw)*
     & (-256+2*(-314+5*NFV)*u+36*nf*(1+2*u)+
     & (57-20*NFV)*(u*u)+2*(-8+5*NFV)*(u*u*u))))/
     & (18._dp*w*(omu*omu*omu*omu)*(omw*omw*omw))+
     & (H2(0,0)*(u*(-90/omw+4*(-2+(-60+3*nf-5*NFV)*v+1/w))+
     & (v*(omv*omv))/u+(4/omw-4/w+90/(omw*omw))*(u*u)-
     & (v*(-20*omv+u*(994-108*nf*(1+2*u)-80*v+
     & 2*u*(689+15*NFV+60*v)+(243-60*NFV-80*v)*(u*u)+
     & (-66+30*NFV+20*v)*(u*u*u))))/(omu*omu*omu*omu)))/
     & 18._dp+((-6*(83+(41+4*nf)*v))/omu+
     & ((1461-52*nf)*v)/(omu*omu)-
     & (120*NFV*u*(-5+v*(9+12*zeta2)-u*(-10+v*(12+6*zeta2))+
     & (-5+v*(3+6*zeta2))*(u*u)))/(omu*omu*omu)-
     & (6*v*zeta2*(-20*omv+
     & u*(-865+108*nf*(1+2*u)-80*v+15*u*(-105+8*v)-
     & 4*(19+20*v)*(u*u)+(7+20*v)*(u*u*u))))/
     & (omu*omu*omu*omu)+
     & (6*omw*(96-35*u+90*NFV*(-3+2*u))*w*zeta2*(u*u)+
     & w*(-276-210*(-1+4*u)*zeta2+90*NFV*(3+6*(-2+4*u)*zeta2))*
     & (omw*omw)*(u*u)+36*(-16+45*NFV)*w*zeta2*(u*u*u)+
     & omw*omw*omw*(-(u*
     & (60*NFV*u*(u+w)+
     & w*(-498+v*(1215-76*nf+768*u*zeta3))))+
     & 6*zeta2*(v*w*(omv*omv)+
     & 4*(-1+(36-90*NFV+8*v-2*nf*v+55*NFV*v)*w)*(u*u)+
     & 4*(u*u*u))))/(u*w*(omw*omw*omw)))/108._dp+
     & (-(u*v*H1(1))/9._dp+(u*(2-(2*(4+u))/omw+
     & v*(57-2*nf+(15*(-2+u))/(omu*omu))+(8*u)/(omw*omw)))/
     & 18._dp)*(H1(0)*H1(0))+G1(3)*
     & (H1(0)*(-(u*v)/6._dp-(u*v*H1(1))/9._dp)-(u*v*(H1(0)*H1(0)))/9._dp)+
     & (17*u*v*(H1(0)*H1(0)*H1(0)))/18._dp-
     & ((-2+u)*u*v*(H1(1)*H1(1)))/(6._dp*(omu*omu))+
     & H1(0)*(-(u*v*H2(0,0))/9._dp-(u*v*H2(1,0))/9._dp+
     & (u*v*H1(1)*(-8+2*v+2*u*(9+7*v)+(2-6*v)*(u*u*u)-
     & 3*(u*u*u*u)+u*u*(-3+2*v-3*(v*v))+9*(v*v)))/
     & (18._dp*(omu*omu)*(omw*omw))+
     & ((24*(-1+15*NFV))/omu+
     & (6*v*(-23+(214-16*nf)*u+(-89+8*nf)*(u*u)+
     & 10*NFV*(9-12*u+5*(u*u))))/(omu*omu)+
     & (12*(-16+45*NFV-15*u)*(u*u)*(w*w)-
     & 6*omw*u*(-16-69*u+72*(u*u)+15*NFV*(3-4*u+u*u))*
     & (w*w)+omw*omw*
     & (-(w*((54+w*(-588+v*(1119-76*nf+150*zeta2)))*(u*u)+
     & 54*(-2+w)*(u*u*u)+54*(u*u*u*u)-
     & 6*u*w*(-10+v*v)+6*w*(10-11*v+v*v)))+
     & 60*NFV*u*((-5+7*v)*(u*u*u)+5*(u*u*u*u)+
     & u*u*(-2+4*w-5*(w*w))-9*(w*w)+
     & u*(2-w+(-4+6*v)*(w*w)))))/(u*(omw*omw)*(w*w)))/
     & 108._dp-(u*v*(H1(1)*H1(1)))/18._dp))+
     & G1(3)*(((67-180*NFV)*u*v*zeta2*H1(1))/9._dp-(8*u*v*H3(0,0,0))/3._dp-
     & 2*u*v*H3(0,0,1)-(4*u*v*H3(0,1,0))/9._dp-(4*u*v*H3(1,0,0))/9._dp+
     & ((-11+60*NFV)*u*v*H3(1,0,1))/9._dp-((1+20*NFV)*u*v*H3(1,1,0))/3._dp+
     & (u*v*H2(0,0)*(75-4*nf+(-64+137*v+(-6+39*v)*(u*u)+
     & 13*(u*u*u)-6*(v*v)+u*(137-12*v+39*(v*v))+13*(v*v*v))/
     & (omw*omw*(w*w))))/9._dp+
     & (336*u*v*zeta3+6*zeta2*(2*v*(-113+50*v)+
     & (v*(-68+10*v-5*(v*v)))/u+
     & u*u*(120/omw-256/(omw*omw)+(-97+6*w)/(w*w))+
     & u*(-114+256/omw-103/w+97/(w*w)+
     & 4*v*(82-2*nf+35*NFV*(-1+1/(w*w*w))))))/108._dp-
     & (v*H2(0,1)*((58-60*omv*NFV-38*v)*(u*u*u*u)+
     & (-6+20*NFV)*(u*u*u*u*u)+
     & u*u*u*(-117+212*v+60*NFV*(omv*omv)-77*(v*v))-
     & omv*omv*omv*(10-2*v+v*v)-
     & u*(omv*omv)*(8-52*v+17*(v*v))+
     & u*u*(83+5*(-53+12*NFV)*v+(245-60*NFV)*(v*v)+
     & (-63+20*NFV)*(v*v*v))))/(18._dp*u*(w*w*w))-
     & (v*H2(1,0)*((58-60*omv*NFV-38*v)*(u*u*u*u)+
     & (-6+20*NFV)*(u*u*u*u*u)+
     & u*u*u*(-117+212*v+60*NFV*(omv*omv)-77*(v*v))-
     & omv*omv*omv*(10-2*v+v*v)-
     & u*(omv*omv)*(8-52*v+17*(v*v))+
     & u*u*(83+5*(-53+12*NFV)*v+(245-60*NFV)*(v*v)+
     & (-63+20*NFV)*(v*v*v))))/(18._dp*u*(w*w*w))+
     & (((-57+2*nf)*u*v)/18._dp+(u*v*H1(1))/9._dp)*(H1(0)*H1(0))-
     & (17*u*v*(H1(0)*H1(0)*H1(0)))/18._dp+
     & H1(0)*((u*v*H1(1))/6._dp+(2529*omu*(3+(10+7*v)*w)*(omw*omw)*(w*w)+
     & 54*w*(omw*omw)*(6*nf*u*
     & ((2+v)*(u*u*u)+u*u*(-4+9*v+2*(v*v))+
     & v*(2-8*v+5*(v*v))+u*(2-12*v+12*(v*v)+v*v*v))-
     & 281*(1+v)*(w*w))-
     & 3*omw*(omu*omu)*(10*NFV*u*
     & (6*v*(-1+3*v)*(omv*omv*omv)+(-9+6*v)*(u*u*u*u*u)+
     & u*u*u*u*(30-75*v+26*(v*v))+
     & u*u*u*(-39+183*v-197*(v*v)+46*(v*v*v))+
     & u*v*(60-201*v+243*(v*v)-110*(v*v*v)+12*(v*v*v*v))+
     & u*u*(18-168*v+336*(v*v)-223*(v*v*v)+38*(v*v*v*v)))
     & +omw*w*(-843+4215*w+(3386+857*v)*(w*w)))+
     & w*(omu*omu*omu)*(12*omw*u*(8-37*u+2*nf*(4+u))*(w*w)+
     & 48*(5-2*nf)*(u*u)*(w*w)+
     & omw*omw*(-2529+5058*w+54*(-1+w)*(u*u*u)-
     & 6*(-15-9*v+v*v)*(w*w)+
     & 3*(u*u)*(-351+36*nf-274*w+18*(w*w))+
     & u*(-2529-453*w+(-372+1119*v+1734*v*zeta2)*(w*w)-
     & 4*nf*(-81-27*w+(6+19*v)*(w*w))))))/
     & (108._dp*(omu*omu*omu)*(omw*omw)*(w*w*w))+(u*v*(H1(1)*H1(1)))/18._dp)
     & )+((-2+u)*u*v*(H1(1)*H1(1)*H1(1)))/(18._dp*(omu*omu))+
     & G1(0)*(G2(1,2)*(((-30+nf)*u*v)/18._dp-u*v*H1(0))+
     & G2(3,0)*(((-30+nf)*u*v)/18._dp-u*v*H1(0))+
     & G2(0,2)*(-((-30+nf)*u*v)/18._dp+u*v*H1(0))+
     & G2(2,0)*(-((-30+nf)*u*v)/18._dp+u*v*H1(0))+(28*u*v*H3(0,0,0))/9._dp+
     & (17*u*v*H3(0,0,1))/9._dp+(u*v*H3(0,1,0))/3._dp+(10*u*v*H3(1,0,0))/9._dp-
     & (u*v*H3(1,0,1))/9._dp+(u*v*H3(1,1,0))/3._dp+
     & (u*v*H2(1,1)*(-114+(292+90*NFV)*u+4*nf*(omu*omu*omu)-
     & (259+90*NFV)*(u*u)+(81+40*NFV)*(u*u*u)))/
     & (18._dp*(omu*omu*omu))+
     & (H1(1)*(u*(534+168/omw+v*(-1119+76*nf-462*zeta2))+
     & 18*(-3-24/omw+3/w-10/(omw*omw))*(u*u)-(54*(u*u*u))/w+
     & (3*(2*(10-11*v+v*v)+2*(u*u)*(11+4*(6+nf)*v+v*v)-
     & u*(42+(29+12*nf)*v+4*(v*v))))/(omu*omu)-
     & (30*NFV*(u*u)*(3-12*v+(-9+6*v)*(u*u*u*u*u)+77*(v*v)+
     & u*u*u*u*(39-68*v+20*(v*v))-118*(v*v*v)+
     & u*u*u*(-66+216*v-153*(v*v)+26*(v*v*v))+
     & u*(-21+122*v-319*(v*v)+252*(v*v*v)-42*(v*v*v*v))+
     & 54*(v*v*v*v)+u*u*
     & (54-264*v+375*(v*v)-136*(v*v*v)+12*(v*v*v*v))))/
     & (omw*(omu*omu*omu)*(w*w))))/108._dp+
     & (-3*w*(843+w*(3331+6*v*zeta2))*(omu*omu*omu)*(omw*omw)+
     & w*(omu*omu)*(omw*omw)*
     & (w*(7587-20238*v*zeta2)+
     & 12*nf*u*(-26+53*v+u*(-2+28*v)+28*(u*u)))+
     & 30*omu*w*(omw*omw)*(1686*v*w*zeta2+
     & NFV*u*(u*(-4+6*(7-3*v)*v*zeta2)+3*(4+6*v*zeta2)*(u*u*u)+
     & 4*(2-v*(3+6*zeta2)+6*zeta2*(v*v))+
     & u*u*(-16-6*v*(-2+6*zeta2)+18*zeta2*(v*v))))+
     & 108*(-281+12*nf*u*(1+2*u))*v*zeta2*(omw*omw)*(w*w)-
     & omu*omu*omu*omu*(omw*omw*
     & (30*NFV*u*(5*u*w-4*w*(1+w*(-3+6*v*zeta2))+2*(u*u))+
     & w*(-2529-2406*w-24*v*w*zeta2+
     & u*(6*(-4+(-273+229*v)*w)*zeta2+
     & 4*nf*(81+w*(19+6*v*zeta2))+
     & 3*(-843+5*w*(-81+4*v*zeta3)))+
     & (-1107+108*nf+24*zeta2)*(u*u)))+
     & omw*u*(16*nf*(-19+2*u)+18*(196+6*u-15*NFV*u)+
     & 6*(80+277*u)*zeta2)*(w*w)+
     & 8*(-567+50*nf-60*zeta2)*(u*u)*(w*w)))/
     & (108._dp*(omu*omu*omu*omu)*(omw*omw)*(w*w))+
     & (u*H2(1,0)*(-6-98/omw+(-179+10*nf-40*NFV)*v+4/w+
     & u*(38+2/omw-42/w-(60*omw*NFV)/w+98/(omw*omw))+
     & (v*(-892-1564*u+108*nf*(1+2*u)-97*(u*u)+24*(u*u*u)-
     & 10*NFV*(-4+7*u-6*(u*u)+3*(u*u*u))))/
     & (omu*omu*omu*omu)-
     & ((-38+47*u+47*v+60*NFV*(-1+2*u+2*v))*(u*u))/(w*w)-
     & ((20*NFV*(-3+4*u+4*v)-9*w)*(u*u*u))/(w*w*w)+
     & (20*NFV*(u*u*u*u))/(w*w*w)))/18._dp+
     & (u*H2(0,0)*((192+62*u)/omw-(120*omw*NFV*u)/w-
     & (192*u)/(omw*omw)+
     & (v*(-856-1654*u+108*nf*(1+2*u)-25*(u*u)+6*(u*u*u)-
     & 10*NFV*(-4+7*u-6*(u*u)+3*(u*u*u)))*(w*w*w)-
     & 2*(omu*omu*omu*omu)*
     & (w*(2*u*(18-19*w)*w+w*(2+(29+115*v-6*nf*v)*w)+
     & (-38+47*v)*(u*u)+38*(u*u*u))+
     & 20*NFV*(3*(-1+2*v)*w*(u*u)+(-3+4*v+6*w)*(u*u*u)+
     & 3*(u*u*u*u)+v*(w*w*w))))/(omu*omu*omu*omu*(w*w*w))))
     & /18._dp+(u*H2(0,1)*((18*(-1+2*u))/omw-(60*omw*NFV*u)/w+
     & (18*u)/(omw*omw)+
     & (v*(3-5*u+2*(u*u)+10*NFV*(4-3*u+3*(u*u)))*(w*w*w)-
     & omu*omu*omu*(w*
     & (-38*u*(-1+w)*w+(-38+47*v)*(u*u)+38*(u*u*u)+
     & 6*(6+22*v-nf*v)*(w*w))+
     & 20*NFV*(3*(-1+2*v)*w*(u*u)+(-3+4*v+6*w)*(u*u*u)+
     & 3*(u*u*u*u)+2*v*(w*w*w))))/(omu*omu*omu*(w*w*w))))/
     & 18._dp+(-(u*v*H1(0))/18._dp-(u*v*H1(1))/18._dp+
     & ((-4+omw)*u*v)/(18._dp*(omw*omw)))*(G1(2)*G1(2))+
     & ((5*u*v*H1(1))/6._dp-(u*v*(-140+35*v+
     & (131-4*omv*nf-114*v)*(u*u*u)+(-57+2*nf)*(u*u*u*u)+
     & (-21+2*nf)*(v*v)+
     & u*(315+4*(-28+nf)*v-4*(-24+nf)*(v*v))+
     & u*u*(-231+227*v-57*(v*v)+2*nf*(1-4*v+v*v))))/
     & (18._dp*(omu*omu)*(omw*omw)))*(H1(0)*H1(0))+
     & G1(3)*(-((-30+nf)*u*v*H1(0))/18._dp+u*v*(H1(0)*H1(0)))+
     & G1(1)*(-((-30+nf)*u*v*H1(1))/18._dp+
     & H1(0)*(-((-30+nf)*u*v)/18._dp+u*v*H1(1))+u*v*(H1(0)*H1(0)))+
     & (17*u*v*(H1(0)*H1(0)*H1(0)))/18._dp+
     & (u*v*(-4+omw-6*u*v-3*(u*u)-3*(v*v))*(H1(1)*H1(1)))/
     & (18._dp*(omw*omw))+H1(0)*
     & (u*v*H2(0,0)+u*v*H2(1,0)-
     & (u*v*H1(1)*(4-v-2*(-22+omv*nf+27*v)*(u*u*u)+
     & (-27+nf)*(u*u*u*u)+(-9+nf)*(v*v)+
     & u*(-9+2*(-8+nf)*v+(45-2*nf)*(v*v))+
     & u*u*(-3+89*v-27*(v*v)+nf*(1-4*v+v*v))))/
     & (9._dp*(omu*omu)*(omw*omw))+
     & (omw*omw*(30*NFV*(u*u)*
     & (18-99*v+(-9+6*v)*(u*u*u*u*u)+203*(v*v)+
     & u*u*u*u*(48-78*v+20*(v*v))-172*(v*v*v)+
     & u*u*u*(-108+297*v-175*(v*v)+26*(v*v*v))+
     & u*(-75+357*v-541*(v*v)+294*(v*v*v)-42*(v*v*v*v))+
     & 54*(v*v*v*v)+
     & u*u*(126-483*v+493*(v*v)-148*(v*v*v)+12*(v*v*v*v)))
     & -w*(9*omu*w*(843+(2810+v)*w)+
     & 6*(2*nf*u*(2*v*(u*u*u*u)+3*v*(-26+16*v+v*v)+
     & u*u*u*(54-9*v+4*(v*v))+
     & u*(54-92*v+124*(v*v)-5*(v*v*v))+
     & u*u*(-108+177*v-14*(v*v)+2*(v*v*v)))-
     & 2529*(w*w))+
     & 3*(omu*omu)*(843-4215*w+(-3422+67*v)*(w*w))+
     & omu*omu*omu*
     & (54*(-1+w)*(u*u*u)+
     & 9*(u*u)*(-117+12*nf-38*w+6*(w*w))-
     & 3*(843-1686*w+2*(-25+25*v+v*v)*(w*w))+
     & u*(-2529-933*w+
     & (-480+1119*v+1326*v*zeta2)*(w*w)+
     & 4*nf*(81+27*w+(6-19*v)*(w*w))))))+
     & 24*omw*u*(nf*(4+u)-6*(19+u))*(omu*omu*omu)*(w*w*w)-
     & 96*(-20+nf)*(omu*omu*omu)*(u*u)*(w*w*w))/
     & (108._dp*(omu*omu*omu)*(omw*omw)*(w*w*w))-(u*v*(H1(1)*H1(1)))/6._dp)
     & +G1(2)*(H1(0)*((2*u*v*H1(1))/9._dp+
     & (u*(-2+8/omw+(-27+nf)*v)+18*v*(-1+1/(omu*omu))+
     & (2*(-4+omw)*(u*u))/(omw*omw))/18._dp)+
     & (u*v*H1(1)*(8-2*v+u*(-2+6*v)+3*(u*u)+3*(v*v)))/
     & (18._dp*(omw*omw))+(u*v*
     & (12-3*v+(78-2*omv*nf-63*v)*(u*u)+(-33+nf)*(u*u*u)-
     & 2*(-30+nf)*(v*v)+u*(-27+(126-4*nf)*v+(-30+nf)*(v*v))
     & ))/(18._dp*(omu*omu)*(omw*omw))-(8*u*v*(H1(0)*H1(0)))/9._dp+
     & (u*v*(H1(1)*H1(1)))/9._dp)-(u*v*(H1(1)*H1(1)*H1(1)))/18._dp)+
     & G1(1)*((-22*u*v*H3(0,0,0))/9._dp-2*u*v*H3(0,0,1)-(14*u*v*H3(0,1,0))/9._dp-
     & (22*u*v*H3(1,0,0))/9._dp-(32*u*v*H3(1,1,0))/9._dp+
     & (v*H2(1,0)*(-27+(71-5*nf+20*NFV)*u+36/(omu*omu)))/9._dp+
     & (v*H2(1,1)*(-14+81*u-4*nf*u+10*NFV*(-9+4*u)+
     & (34+60*NFV)/(omu*omu)))/18._dp+
     & (H1(1)*((-12*(8+45*NFV))/omu+
     & (6*v*(13+(-54+8*nf)*u+30*NFV*(-3+u*u)-4*(7+nf)*(u*u)))/
     & (omu*omu)+(6*omw*(-1+u)*u*(16+25*u+15*NFV*(-3+7*u))*
     & (w*w)+12*(-16+45*NFV)*(-1+u)*(u*u)*(w*w)+
     & omw*omw*(w*((54+w*(-198+v*(1119-76*nf+870*zeta2)))*
     & (u*u)+54*w*(u*u*u)+54*(u*u*u*u)-
     & 6*u*w*(-40+v*v)+6*w*(10-11*v+v*v))+
     & 60*NFV*u*(w*(1+5*w)*(u*u)+7*w*(u*u*u)+2*(u*u*u*u)+
     & 12*(w*w)+u*(-2+w-2*(1+3*v)*(w*w)))))/
     & (u*(omw*omw)*(w*w))))/108._dp-
     & (v*H2(0,0)*(9*u*(1+2*v)*(omv*omv)-9*(omv*omv*omv)+
     & 4*(-123+9*omv*nf-15*omv*NFV+128*v)*(u*u*u*u)-
     & 4*(-41+3*nf-5*NFV)*(u*u*u*u*u)+
     & u*u*u*(501-1024*v+(-36*nf+60*NFV)*(omv*omv)+541*(v*v))+
     & u*u*(-173+503*v+12*nf*(omv*omv*omv)-523*(v*v)+
     & 193*(v*v*v)+20*NFV*(-1+6*v-6*(v*v)+2*(v*v*v)))))/
     & (18._dp*u*(w*w*w))-(v*H2(0,1)*
     & (omv*omv*omv*(-9+30*NFV*u+6*nf*(u*u))+
     & u*(omv*omv)*(25+2*v+6*nf*(-5+2*v)*(u*u))+
     & u*u*(-53+95*v+(-475+329*v-6*nf*(-5+3*v))*(u*u*u*u)+
     & (103-6*nf)*(u*u*u*u*u)-67*(v*v)+
     & u*u*u*(839-1196*v+358*(v*v)-6*nf*(10-12*v+3*(v*v)))+
     & u*(265-692*v+651*(v*v)-206*(v*v*v))+25*(v*v*v)+
     & u*u*(-695+1485*v-936*(v*v)+132*(v*v*v)-
     & 6*nf*(-10+18*v-9*(v*v)+v*v*v))+
     & 10*NFV*(-29+84*v+(-19+6*v)*(u*u*u*u)+2*(u*u*u*u*u)-
     & 75*(v*v)+u*u*u*(65-51*v+6*(v*v))+
     & u*(82-180*v+111*(v*v)-17*(v*v*v))+22*(v*v*v)+
     & u*u*(-104+150*v-51*(v*v)+4*(v*v*v))))))/
     & (18._dp*u*(omu*omu)*(w*w*w))+
     & (v*(12*zeta2*(omv*omv*omv)*(-36+75*NFV*u+2*nf*(u*u))+
     & 24*u*zeta2*(omv*omv)*(47+7*v+nf*(-5+2*v)*(u*u))+
     & u*u*(6*zeta2*(595-2261*v+
     & (1703-863*v-4*nf*(-5+3*v))*(u*u*u*u)-
     & (261+4*nf)*(u*u*u*u*u)+2449*(v*v)-
     & u*u*u*(4270-4622*v+979*(v*v)+
     & 4*nf*(10-12*v+3*(v*v)))-783*(v*v*v)+
     & u*(-3113+7598*v-5397*(v*v)+1056*(v*v*v))+
     & u*u*(5230-8964*v+4011*(v*v)-377*(v*v*v)-
     & 4*nf*(-10+18*v-9*(v*v)+v*v*v))+
     & 10*NFV*(-131+396*v+(-163+60*v)*(u*u*u*u)+
     & 20*(u*u*u*u*u)-351*(v*v)+
     & u*u*u*(479-429*v+60*(v*v))+
     & u*(430-966*v+591*(v*v)-87*(v*v*v))+102*(v*v*v)+
     & u*u*(-650+984*v-345*(v*v)+12*(v*v*v))))+
     & 384*zeta3*(omu*omu)*(w*w*w))))/(108._dp*u*(omu*omu)*(w*w*w))+
     & ((u*v*H1(0))/18._dp+(u*v*H1(1))/18._dp)*(G1(2)*G1(2))+
     & (((-57+2*nf)*u*v)/18._dp-(5*u*v*H1(1))/6._dp)*(H1(0)*H1(0))-
     & (17*u*v*(H1(0)*H1(0)*H1(0)))/18._dp+(u*v*(H1(1)*H1(1)))/6._dp+
     & G1(2)*(-(u*v*H1(1))/6._dp+H1(0)*(-(u*v)/6._dp-(2*u*v*H1(1))/9._dp)-
     & (u*v*(H1(0)*H1(0)))/9._dp-(u*v*(H1(1)*H1(1)))/9._dp)+
     & H1(0)*(((-27+nf)*u*v*H1(1))/9._dp+
     & ((-12*(8+45*NFV))/omu+
     & (6*v*(13+(-54+8*nf)*u+30*NFV*(-3+u*u)-
     & 4*(7+nf)*(u*u)))/(omu*omu)+
     & (6*omw*(-1+u)*u*(16+25*u+15*NFV*(-3+7*u))*(w*w)+
     & 12*(-16+45*NFV)*(-1+u)*(u*u)*(w*w)+
     & omw*omw*(w*((54+w*(-198+v*(1119-76*nf+438*zeta2)))*
     & (u*u)+54*w*(u*u*u)+54*(u*u*u*u)-
     & 6*u*w*(-40+v*v)+6*w*(10-11*v+v*v))+
     & 60*NFV*u*(w*(1+5*w)*(u*u)+7*w*(u*u*u)+2*(u*u*u*u)+
     & 12*(w*w)+u*(-2+w-2*(1+3*v)*(w*w)))))/
     & (u*(omw*omw)*(w*w)))/108._dp+(u*v*(H1(1)*H1(1)))/6._dp)+
     & (u*v*(H1(1)*H1(1)*H1(1)))/18._dp)
      return
      end
      
      
      function Gamma_3a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(4*(1-5*NFV)*pi*u*v*G3(0,0,1))/3._dp-(7*pi*u*v*G3(0,0,2))/9._dp+
     & (22*pi*u*v*G3(0,0,3))/9._dp-2*pi*u*v*G3(0,1,0)+
     & (10*(-1+2*NFV)*pi*u*v*G3(0,1,1))/3._dp+
     & (4*(-1+5*NFV)*pi*u*v*G3(0,1,2))/3._dp-(7*pi*u*v*G3(0,2,0))/9._dp+
     & ((7+60*NFV)*pi*u*v*G3(0,2,1))/9._dp-(7*pi*u*v*G3(0,2,2))/9._dp+
     & (4*pi*u*v*G3(0,2,3))/3._dp-(pi*u*v*G3(0,3,0))/9._dp-(pi*u*v*G3(0,3,2))/9._dp-
     & (16*pi*u*v*G3(0,3,3))/9._dp+((-7+20*NFV)*pi*u*v*G3(1,0,1))/3._dp+
     & (16*pi*u*v*G3(1,0,2))/9._dp-(4*pi*u*v*G3(1,0,3))/9._dp+
     & ((19-20*NFV)*pi*u*v*G3(1,1,1))/3._dp-(5*(1+4*NFV)*pi*u*v*G3(1,1,2))/3._dp+
     & (16*pi*u*v*G3(1,2,0))/9._dp-(5*(1+4*NFV)*pi*u*v*G3(1,2,1))/3._dp+
     & (16*pi*u*v*G3(1,2,2))/9._dp-(4*pi*u*v*G3(1,2,3))/9._dp-
     & (7*pi*u*v*G3(2,0,0))/9._dp+((7+60*NFV)*pi*u*v*G3(2,0,1))/9._dp-
     & (7*pi*u*v*G3(2,0,2))/9._dp+(4*pi*u*v*G3(2,0,3))/3._dp+pi*u*v*G3(2,1,0)-
     & 4*pi*u*v*G3(2,1,1)+pi*u*v*G3(2,1,2)-(7*pi*u*v*G3(2,2,0))/9._dp+
     & ((-1+20*NFV)*pi*u*v*G3(2,2,1))/3._dp-(13*pi*u*v*G3(2,2,3))/9._dp+
     & (pi*u*v*G3(2,3,0))/3._dp-(8*pi*u*v*G3(2,3,2))/9._dp-(5*pi*u*v*G3(2,3,3))/9._dp+
     & (16*pi*u*v*G3(3,0,0))/9._dp+((11-60*NFV)*pi*u*v*G3(3,0,1))/9._dp+
     & (16*pi*u*v*G3(3,0,2))/9._dp-(10*pi*u*v*G3(3,0,3))/3._dp+
     & (16*pi*u*v*G3(3,2,0))/9._dp+((11-60*NFV)*pi*u*v*G3(3,2,1))/9._dp+
     & (14*pi*u*v*G3(3,2,3))/9._dp-(20*pi*u*v*G3(3,3,0))/9._dp+
     & (4*pi*u*v*G3(3,3,3))/3._dp+(46*pi*u*v*H3(0,0,0))/9._dp+
     & (17*pi*u*v*H3(0,0,1))/9._dp+(2*(7-30*NFV)*pi*u*v*H3(0,1,0))/9._dp+
     & (20*pi*u*v*H3(1,0,0))/9._dp-(pi*u*v*H3(1,0,1))/9._dp+
     & (2*(1-10*NFV)*pi*u*v*H3(1,1,0))/3._dp+
     & G2(2,1)*(pi*u*v*H1(0)-pi*u*v*H1(1)+
     & (pi*(2*v*(-26-45*NFV+10*v+(26+30*NFV)/(omu*omu))-
     & (v*(omv*omv))/u+(u*
     & (-32+90*NFV+omw*(41+60*NFV)+
     & (-24+87*v-4*nf*v+20*NFV*(6+v))*(omw*omw)))/
     & (omw*omw)-((-32+90*NFV+omw*(41+60*NFV)+
     & 24*(-1+5*NFV)*(omw*omw))*(u*u))/(omw*omw*omw)))/18._dp)+
     & G2(0,2)*((pi*u*v*H1(0))/9._dp+(7*pi*u*v*H1(1))/9._dp+
     & (pi*u*(-34-26/omw+(5*nf-8*(6+5*NFV))*v+
     & (2*u*(13+17*u+17*v))/(omw*omw)+
     & (v*(-33+49*u-16*(u*u)+10*NFV*(4-3*u+3*(u*u))))/
     & (omu*omu*omu)))/18._dp)+
     & G2(2,0)*((pi*u*v*H1(0))/9._dp+(7*pi*u*v*H1(1))/9._dp+
     & (pi*u*(-34-26/omw+(5*nf-8*(6+5*NFV))*v+
     & (2*u*(13+17*u+17*v))/(omw*omw)+
     & (v*(-33+49*u-16*(u*u)+10*NFV*(4-3*u+3*(u*u))))/
     & (omu*omu*omu)))/18._dp)+
     & G2(0,1)*((10*(1-2*NFV)*pi*u*v*H1(0))/3._dp+(4*(1-5*NFV)*pi*u*v*H1(1))/3._dp+
     & (pi*(u*(omu*omu)*(32*u-90*NFV*u-
     & omw*(32-90*NFV+41*u+60*NFV*u)+
     & (41+60*NFV+24*u-120*NFV*u)*(omw*omw)+
     & (-24+120*NFV+77*v-5*nf*v)*(omw*omw*omw))+
     & v*(omw*omw*omw)*(-20+99*u-60*(u*u)+
     & 30*NFV*(1-6*u+4*(u*u)))))/(18._dp*(omu*omu)*(omw*omw*omw))
     & )+(pi*u*v*H2(1,1)*(-114+(292+90*NFV)*u+4*nf*(omu*omu*omu)-
     & (259+90*NFV)*(u*u)+(81+40*NFV)*(u*u*u)))/(18._dp*(omu*omu*omu))
     & +G2(2,2)*((2*(1-10*NFV)*pi*u*v*H1(0))/3._dp+
     & (pi*u*v*(-114+(292+90*NFV)*u+4*nf*(omu*omu*omu)-
     & (259+90*NFV)*(u*u)+(81+40*NFV)*(u*u*u)))/
     & (18._dp*(omu*omu*omu)))+
     & G2(2,3)*((8*pi*u*v*H1(1))/9._dp+
     & (pi*u*(-30+80/omw+73*v-4/w+
     & u*(34/omw+4/w-80/(omw*omw))+
     & (v*(878+2*(794+15*NFV)*u-108*nf*(1+2*u)+
     & (91-60*NFV)*(u*u)+(-28+30*NFV)*(u*u*u)))/
     & (omu*omu*omu*omu)))/18._dp)+
     & G2(0,0)*((16*pi*u*v*H1(0))/9._dp+(7*pi*u*v*H1(1))/9._dp+
     & (pi*u*v*(-13-17*v+u*(-17+6*(-6+nf)*v)+3*(-6+nf)*(u*u)+
     & 3*(-6+nf)*(v*v)))/(9._dp*(omw*omw)))-
     & (pi*u*H2(1,0)*(v*w*(omw*omw*omw)*
     & (856+1654*u-108*nf*(1+2*u)+25*(u*u)-6*(u*u*u)+
     & 10*NFV*(-4+7*u-6*(u*u)+3*(u*u*u)))+
     & omu*omu*omu*omu*(omw*(32-90*NFV-57*u)*w+
     & (57-60*NFV-26*u)*w*(omw*omw)+
     & (-4+4*u+(30+40*NFV*(-3+v)+105*v-6*nf*v)*w)*
     & (omw*omw*omw)+2*u*w*
     & (-16+15*NFV*(3+2*v+u*(2+8*v)+4*(u*u)+4*(v*v))))))/
     & (18._dp*w*(omu*omu*omu*omu)*(omw*omw*omw))+
     & (pi*H1(1)*(u*(534+168/omw+v*(-1119+76*nf-294*zeta2))+
     & 18*(-3-24/omw+3/w-10/(omw*omw))*(u*u)-(54*(u*u*u))/w+
     & (3*(2*(10-11*v+v*v)+2*(u*u)*(11+(-41+4*nf)*v+v*v)-
     & u*(42+(-227+12*nf)*v+4*(v*v))))/(omu*omu)-
     & (30*NFV*u*((-9+6*v)*(u*u*u*u*u*u)+24*(omv*omv)*(v*v)+
     & u*u*u*u*u*(39-52*v+20*(v*v))+
     & u*u*u*u*(-66+144*v-105*(v*v)+26*(v*v*v))+
     & u*u*(-21+34*v-71*(v*v)+100*(v*v*v)-26*(v*v*v*v))+
     & u*u*u*(54-144*v+191*(v*v)-88*(v*v*v)+12*(v*v*v*v))+
     & u*(3+12*v-59*(v*v)+34*(v*v*v)+14*(v*v*v*v))))/
     & (omw*(omu*omu*omu)*(w*w))))/108._dp+
     & G2(3,3)*((16*pi*u*v*H1(0))/9._dp+
     & (pi*(2*v*(-1+10*v)+(v*(8+2*v-v*v))/u+
     & u*u*(18/omw-128/(omw*omw)+(151-6*w)/(w*w))+
     & u*(-24+128/omw+157/w-151/(w*w)+
     & v*(14-4*nf+20*NFV-(20*NFV)/(w*w*w)))))/18._dp)+
     & G2(3,0)*((-28*pi*u*v*H1(0))/9._dp-(16*pi*u*v*H1(1))/9._dp+
     & (pi*(2*(19-10*v)*v+(v*(10-2*v+v*v))/u+
     & u*u*(-46/omw+128/(omw*omw)-(9+2*w)/(w*w))+
     & u*(44-128/omw-7/w+9/(w*w)+
     & v*(-39-nf+20*NFV-(20*NFV)/(w*w*w)))))/18._dp)+
     & G2(3,2)*((2*(-7+30*NFV)*pi*u*v*H1(0))/9._dp-
     & (pi*v*((58-60*omv*NFV-38*v)*(u*u*u*u)+
     & (-6+20*NFV)*(u*u*u*u*u)+
     & u*u*u*(-117+212*v+60*NFV*(omv*omv)-77*(v*v))-
     & omv*omv*omv*(10-2*v+v*v)-
     & u*(omv*omv)*(8-52*v+17*(v*v))+
     & u*u*(83+5*(-53+12*NFV)*v+(245-60*NFV)*(v*v)+
     & (-63+20*NFV)*(v*v*v))))/(18._dp*u*(w*w*w)))+
     & G2(1,0)*((-7*pi*u*v*H1(0))/9._dp-(16*pi*u*v*H1(1))/9._dp+
     & (pi*v*(-27*u*(omv*omv)+9*(omv*omv*omv)+
     & (NFV*(130-60*v)+26*(-2+v)+5*nf*(-5+3*v))*(u*u*u*u*u*u)+
     & (2+5*nf-20*NFV)*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(178-226*v+55*(v*v)+5*nf*(10-12*v+3*(v*v))-
     & 10*NFV*(29-33*v+6*(v*v)))-
     & u*u*(7+20*NFV-91*v+5*nf*(omv*omv*omv)+125*(v*v)-
     & 41*(v*v*v))+u*u*u*u*
     & (-242+492*v-265*(v*v)+10*NFV*(25-48*v+21*(v*v))+
     & 31*(v*v*v)+5*nf*(-10+18*v-9*(v*v)+v*v*v))+
     & u*u*u*(139-410*v-5*nf*(-5+2*v)*(omv*omv)+335*(v*v)-
     & 82*(v*v*v)+10*NFV*(-5+21*v-15*(v*v)+3*(v*v*v)))))/
     & (18._dp*u*(omu*omu)*(w*w*w)))+
     & G2(1,2)*((-2*pi*u*v*H1(0))/9._dp-(16*pi*u*v*H1(1))/9._dp-
     & (pi*v*(omv*omv*omv*(-9+30*NFV*u+4*nf*(u*u))+
     & u*(omv*omv)*(7+20*v+4*nf*(-5+2*v)*(u*u))+
     & u*u*(27-91*v-2*(11-5*v+2*nf*(-5+3*v))*(u*u*u*u)+
     & (10-4*nf)*(u*u*u*u*u)+65*(v*v)+
     & u*u*u*(12+40*v-19*(v*v)-4*nf*(10-12*v+3*(v*v)))-
     & v*v*v+u*(-41+116*v-101*(v*v)+44*(v*v*v))+
     & u*u*(16-108*v+115*(v*v)-19*(v*v*v)-
     & 4*nf*(-10+18*v-9*(v*v)+v*v*v))+
     & 10*NFV*(-29+84*v+6*(-7+3*v)*(u*u*u*u)+6*(u*u*u*u*u)-
     & 75*(v*v)+6*(u*u*u)*(19-18*v+3*(v*v))+
     & u*(97-213*v+132*(v*v)-20*(v*v*v))+22*(v*v*v)+
     & u*u*(-149+228*v-84*(v*v)+4*(v*v*v))))))/
     & (18._dp*u*(omu*omu)*(w*w*w)))+
     & G2(1,1)*(((-7+20*NFV)*pi*u*v*H1(0))/3._dp+(5*(1+4*NFV)*pi*u*v*H1(1))/3._dp+
     & (pi*v*(omv*omv*omv*(9-30*NFV*u+4*nf*(u*u))+
     & u*(omv*omv)*(-43+16*v+4*nf*(-5+2*v)*(u*u))+
     & u*u*(-143+547*v+(-289+97*v-4*nf*(-5+3*v))*(u*u*u*u)+
     & (39-4*nf)*(u*u*u*u*u)-629*(v*v)+
     & u*u*u*(851-670*v+68*(v*v)-4*nf*(10-12*v+3*(v*v)))+
     & u*(769-1768*v+1113*(v*v)-132*(v*v*v))+225*(v*v*v)+
     & u*u*(-1193+1719*v-504*(v*v)+10*(v*v*v)-
     & 4*nf*(-10+18*v-9*(v*v)+v*v*v))+
     & 10*NFV*(25-72*v+(-1+6*v)*(u*u*u*u)+2*(u*u*u*u*u)+
     & 63*(v*v)+u*u*u*(-25+3*v+6*(v*v))+
     & u*u*(64-78*v+15*(v*v))-18*(v*v*v)+
     & u*(-62+132*v-75*(v*v)+9*(v*v*v))))))/
     & (18._dp*u*(omu*omu)*(w*w*w)))+
     & G2(0,3)*((-7*pi*u*v*H1(0))/9._dp+(pi*u*v*H1(1))/9._dp+
     & (pi*u*((-8*(25+8*u))/omw+(60*omw*NFV*u)/w+(200*u)/(omw*omw)+
     & (omu*omu*omu*omu*
     & (20*NFV*(u*u)*(3*(-1+2*v)*w+u*(-3+4*v+6*w)+
     & 3*(u*u))+w*
     & (2*u*(17-19*w)*w+w*(4+(60+67*v-nf*v)*w)+
     & (-38+47*v)*(u*u)+38*(u*u*u)))+
     & v*(878+2*(794+15*NFV)*u-108*nf*(1+2*u)+
     & (91-60*NFV)*(u*u)+(-28+30*NFV)*(u*u*u))*(w*w*w))/
     & (omu*omu*omu*omu*(w*w*w))))/18._dp)+
     & (pi*u*H2(0,0)*((2*(91+50*u))/omw-(60*omw*NFV*u)/w-
     & (182*u)/(omw*omw)+(v*
     & (-892-1564*u+108*nf*(1+2*u)-97*(u*u)+24*(u*u*u)-
     & 10*NFV*(-4+7*u-6*(u*u)+3*(u*u*u)))*(w*w*w)-
     & omu*omu*omu*omu*(w*
     & (2*u*(17-19*w)*w+4*w*(1+(24+39*v-2*nf*v)*w)+
     & (-38+47*v)*(u*u)+38*(u*u*u))+
     & 20*NFV*(3*(-1+2*v)*w*(u*u)+(-3+4*v+6*w)*(u*u*u)+
     & 3*(u*u*u*u)+2*v*(w*w*w))))/(omu*omu*omu*omu*(w*w*w))))/
     & 18._dp+(pi*u*H2(0,1)*((18*(-1+2*u))/omw-(60*omw*NFV*u)/w+
     & (18*u)/(omw*omw)+(v*
     & (3-5*u+2*(u*u)+10*NFV*(4-3*u+3*(u*u)))*(w*w*w)-
     & omu*omu*omu*(w*(-38*u*(-1+w)*w+(-38+47*v)*(u*u)+
     & 38*(u*u*u)+6*(6+22*v-nf*v)*(w*w))+
     & 20*NFV*(3*(-1+2*v)*w*(u*u)+(-3+4*v+6*w)*(u*u*u)+
     & 3*(u*u*u*u)+2*v*(w*w*w))))/(omu*omu*omu*(w*w*w))))/18._dp
     & +(pi*(-3*(843+w*(3413+v*(82+48*zeta2)))*(omu*omu*omu)*
     & (omw*omw)*(w*w)+omu*omu*(omw*omw)*
     & (w*(7587+v*(1461-20022*zeta2))+
     & 4*nf*u*(-84+165*v-32*v*w+u*(6+78*v+19*v*w)+78*(u*u)))*
     & (w*w)+30*omu*(omw*omw)*
     & (1686*v*w*zeta2+2*NFV*u*
     & ((-4+v*(6+6*zeta2))*(u*u*u)+
     & u*u*(22-2*v*(17+6*zeta2)+(6+6*zeta2)*(v*v))+
     & 2*(7-v*(17+6*zeta2)+(9+6*zeta2)*(v*v))-
     & u*(32-v*(62+18*zeta2)+(24+6*zeta2)*(v*v))))*(w*w)+
     & 108*(-281+12*nf*u*(1+2*u))*v*zeta2*(omw*omw)*(w*w*w)+
     & omu*omu*omu*omu*(360*NFV*zeta2*(omw*omw*omw)*(u*u)*(w*w)+
     & omw*u*(-4812+nf*(400-32*u)-108*u+270*NFV*(1+u)-
     & 948*zeta2-1050*u*zeta2)*(w*w*w)+
     & 2*(2268-200*nf+474*zeta2)*(u*u)*(w*w*w)+
     & omw*omw*(w*(w*(3*(843+(884-405*v)*w)+
     & u*(-4*nf*(81+19*w)+3*(843+405*w-80*v*w*zeta3))-
     & 27*(-41+4*nf)*(u*u))+
     & 6*zeta2*(u*w*(4+171*w-117*v*w)+2*(17-19*w)*w*(u*u)+
     & (9-47*w)*(u*u*u)-9*(u*u*u*u)-11*v*(w*w)))+
     & 10*NFV*u*(3*w*(-7*u*w+2*w*(2-7*w+24*v*w*zeta3)-
     & 2*(u*u))+12*zeta2*
     & (3*(-1+2*v)*w*(u*u)+(-3+4*v+6*w)*(u*u*u)+
     & 3*(u*u*u*u)+6*v*(w*w*w)))))))/
     & (108._dp*(omu*omu*omu*omu)*(omw*omw)*(w*w*w))+
     & ((pi*u*v*G1(1))/2._dp-(pi*u*v*G1(2))/2._dp+(pi*u*v*G1(3))/2._dp+
     & (3*pi*u*v*H1(0))/2._dp+(pi*u*v*H1(1))/2._dp+
     & (pi*u*(-9+36/omw-(-30+nf)*v+(9*(-2+u)*v)/(omu*omu)+
     & (9*(-4+omw)*u)/(omw*omw)))/18._dp)*(G1(0)*G1(0))-
     & (pi*u*v*(G1(0)*G1(0)*G1(0)))/2._dp+
     & (-(pi*u*v*G1(3))/18._dp-(pi*u*v*H1(0))/6._dp-(pi*u*v*H1(1))/6._dp-
     & (pi*u*v*(4-v-6*omv*(u*u*u)+3*(u*u*u*u)+
     & u*(-9+4*v-5*(v*v))+v*v+u*u*(7-11*v+3*(v*v))))/
     & (18._dp*(omu*omu)*(omw*omw)))*(G1(2)*G1(2))+
     & (pi*u*v*(G1(2)*G1(2)*G1(2)))/18._dp+
     & ((5*pi*u*v*H1(1))/6._dp+(pi*(u*(1-4/omw+57*v-2*nf*v)+
     & v*(-1+1/(omu*omu))-((-4+omw)*(u*u))/(omw*omw)))/18._dp)*
     & (H1(0)*H1(0))+(17*pi*u*v*(H1(0)*H1(0)*H1(0)))/18._dp-
     & (pi*u*v*(4-v-6*omv*(u*u*u)+3*(u*u*u*u)+u*(-9+4*v-5*(v*v))+
     & v*v+u*u*(7-11*v+3*(v*v)))*(H1(1)*H1(1)))/
     & (18._dp*(omu*omu)*(omw*omw))+
     & H1(0)*(-(pi*u*v*H2(0,0))-pi*u*v*H2(1,0)-
     & (pi*u*v*H1(1)*(4-v+(63-2*omv*nf-54*v)*(u*u*u)+
     & (-27+nf)*(u*u*u*u)+(-47+nf)*(v*v)+
     & u*(-9+2*(-46+nf)*v-2*(-32+nf)*(v*v))+
     & u*u*(-41+127*v-27*(v*v)+nf*(1-4*v+v*v))))/
     & (9._dp*(omu*omu)*(omw*omw))+
     & (pi*((102+90*NFV*(-3+2*v)+v*(-1173+76*nf-534*zeta2))*
     & (u*u*u*u*u*u*u*u*u)+
     & 6*(omv*omv*omv)*(v*v)*(10-20*v+v*v)+
     & u*u*u*u*u*u*u*u*(-558+6*v*(1276-72*nf+534*zeta2)+
     & 5*(-1161+76*nf-534*zeta2)*(v*v)+
     & 480*NFV*(3-6*v+2*(v*v)))+
     & u*u*u*u*u*u*u*(-210-22257*v-8010*v*zeta2+32571*(v*v)+
     & 14418*zeta2*(v*v)-11484*(v*v*v)-5340*zeta2*(v*v*v)+
     & 30*NFV*(-108+387*v-320*(v*v)+72*(v*v*v))+
     & 4*nf*(54+279*v-483*(v*v)+190*(v*v*v)))+
     & u*v*(omv*omv)*(-432-6*(-623+72*nf)*v+
     & (-5352+688*nf-534*zeta2)*(v*v)+
     & (594-40*nf+534*zeta2)*(v*v*v)+
     & 180*NFV*(-3-v*v+4*(v*v*v))+18*(v*v*v*v))+
     & 2*(u*u*u*u*u*u)*(-315+8727*v+5340*v*zeta2-39843*(v*v)-
     & 16020*zeta2*(v*v)+27900*(v*v*v)+12816*zeta2*(v*v*v)-
     & 5673*(v*v*v*v)-2670*zeta2*(v*v*v*v)+
     & 15*NFV*(126-657*v+1027*(v*v)-500*(v*v*v)+
     & 84*(v*v*v*v))+
     & 4*nf*(-27+116*v+537*(v*v)-426*(v*v*v)+95*(v*v*v*v)))+
     & u*u*u*u*u*(7152-14274*v-8010*v*zeta2+49440*(v*v)+
     & 37380*zeta2*(v*v)-113706*(v*v*v)-49662*zeta2*(v*v*v)+
     & 46986*(v*v*v*v)+22428*zeta2*(v*v*v*v)-5589*(v*v*v*v*v)-
     & 2670*zeta2*(v*v*v*v*v)+
     & 30*NFV*(-75+483*v-1363*(v*v)+1253*(v*v*v)-
     & 405*(v*v*v*v)+50*(v*v*v*v*v))+
     & 4*nf*(-162-468*v+596*(v*v)+1590*(v*v*v)-738*(v*v*v*v)+
     & 95*(v*v*v*v*v)))+
     & u*u*u*u*(-9348+43074*v+3204*v*zeta2-32484*(v*v)-
     & 24030*zeta2*(v*v)+46788*(v*v*v)+49128*zeta2*(v*v*v)-
     & 77184*(v*v*v*v)-36846*zeta2*(v*v*v*v)+19170*(v*v*v*v*v)+
     & 9612*zeta2*(v*v*v*v*v)-1089*(v*v*v*v*v*v)-
     & 534*zeta2*(v*v*v*v*v*v)+
     & 30*NFV*(18-45*v+699*(v*v)-1215*(v*v*v)+747*(v*v*v*v)-
     & 164*(v*v*v*v*v)+12*(v*v*v*v*v*v))+
     & 4*nf*(270-666*v-912*(v*v)+880*(v*v*v)+1116*(v*v*v*v)-
     & 312*(v*v*v*v*v)+19*(v*v*v*v*v*v)))+
     & u*u*(48+(9966-864*nf+2700*NFV)*v-
     & 2*(21426-2092*nf+2070*NFV+534*zeta2)*(v*v)+
     & 6*(9041-844*nf+255*NFV+1068*zeta2)*(v*v*v)-
     & 3*(7026-328*nf+330*NFV+3738*zeta2)*(v*v*v*v)+
     & 2*(1122+296*nf+300*NFV+3738*zeta2)*(v*v*v*v*v)+
     & 3*(-852+56*nf+140*NFV-534*zeta2)*(v*v*v*v*v*v)-
     & 18*(v*v*v*v*v*v*v))+
     & u*u*u*(3444-v*(40014+4500*NFV+534*zeta2)+
     & 3*(24718+270*NFV+2670*zeta2)*(v*v)+
     & 6*(-5543+1815*NFV-4272*zeta2)*(v*v*v)+
     & (16692-12150*NFV+29370*zeta2)*(v*v*v*v)+
     & 3*(-8013+1960*NFV-4272*zeta2)*(v*v*v*v*v)+
     & 3*(983-260*NFV+534*zeta2)*(v*v*v*v*v*v)-
     & 4*nf*(108-928*v+1308*(v*v)+444*(v*v*v)-620*(v*v*v*v)-
     & 363*(v*v*v*v*v)+51*(v*v*v*v*v*v))+6*(v*v*v*v*v*v*v))))/
     & (108._dp*(omu*omu*omu)*(omw*omw)*(w*w*w))-(pi*u*v*(H1(1)*H1(1)))/6._dp)
     & +G1(3)*((46*pi*u*v*H2(0,0))/9._dp+(17*pi*u*v*H2(0,1))/9._dp+
     & (2*(7-30*NFV)*pi*u*v*H2(1,0))/9._dp+
     & H1(0)*(-(pi*u*v*H1(1))/9._dp+
     & (pi*(2*v*(-19+10*v)-(v*(10-2*v+v*v))/u+
     & u*u*(32/omw-128/(omw*omw)+(89-4*w)/(w*w))+
     & u*(-36+128/omw+93/w-89/(w*w)+
     & v*(39+20*NFV*(-1+1/(w*w*w))))))/18._dp)+
     & (pi*v*H1(1)*((49-60*omv*NFV-29*v)*(u*u*u*u)+
     & (-3+20*NFV)*(u*u*u*u*u)+
     & 2*(u*u*u)*(-54+97*v+30*NFV*(omv*omv)-34*(v*v))-
     & omv*omv*omv*(10-2*v+v*v)-
     & u*(omv*omv)*(8-52*v+17*(v*v))+
     & 4*(u*u)*(20+(-64+15*NFV)*v+(59-15*NFV)*(v*v)+
     & 5*(-3+NFV)*(v*v*v))))/(18._dp*u*(w*w*w))-
     & (pi*(2529*omu*(3+(10+7*v)*w)*(omw*omw)*(w*w)+
     & 54*w*(omw*omw)*(6*nf*u*
     & ((2+v)*(u*u*u)+u*u*(-4+9*v+2*(v*v))+
     & v*(2-8*v+5*(v*v))+u*(2-12*v+12*(v*v)+v*v*v))-
     & 281*(1+v)*(w*w))-
     & 3*omw*(omu*omu)*(10*NFV*u*
     & (6*v*(-1+3*v)*(omv*omv*omv)+(-9+6*v)*(u*u*u*u*u)+
     & u*u*u*u*(30-75*v+26*(v*v))+
     & u*u*u*(-39+183*v-197*(v*v)+46*(v*v*v))+
     & u*v*(60-201*v+243*(v*v)-110*(v*v*v)+12*(v*v*v*v))+
     & u*u*(18-168*v+336*(v*v)-223*(v*v*v)+38*(v*v*v*v)))+
     & omw*w*(-843+4215*w+(3386+857*v)*(w*w)))+
     & w*(omu*omu*omu)*(12*omw*u*(8-37*u+2*nf*(4+u))*(w*w)+
     & 48*(5-2*nf)*(u*u)*(w*w)+
     & omw*omw*(-2529+5058*w+54*(-1+w)*(u*u*u)-
     & 6*(-15-9*v+v*v)*(w*w)+
     & 3*(u*u)*(-351+36*nf-274*w+18*(w*w))+
     & u*(-2529-453*w+(-372+1119*v+966*v*zeta2)*(w*w)-
     & 4*nf*(-81-27*w+(6+19*v)*(w*w)))))))/
     & (108._dp*(omu*omu*omu)*(omw*omw)*(w*w*w))-
     & (pi*u*v*(H1(0)*H1(0)))/18._dp-(pi*u*v*(H1(1)*H1(1)))/18._dp)+
     & G1(1)*(G1(2)*((pi*u*v)/6._dp+(pi*u*v*H1(0))/9._dp+(pi*u*v*H1(1))/9._dp)+
     & (20*pi*u*v*H2(0,0))/9._dp+(7*pi*u*v*H2(0,1))/9._dp+(11*pi*u*v*H2(1,0))/9._dp+
     & (16*pi*u*v*H2(1,1))/9._dp+(pi*
     & ((96+540*NFV)/omu+
     & (6*v*(-13+(54-8*nf)*u-30*NFV*(-3+u*u)+4*(7+nf)*(u*u)))/
     & (omu*omu)-(6*omw*(-1+u)*u*(16+25*u+15*NFV*(-3+7*u))*
     & (w*w)+12*(-16+45*NFV)*(-1+u)*(u*u)*(w*w)+
     & omw*omw*(w*((54+w*(-198+v*(1119-76*nf+582*zeta2)))*
     & (u*u)+54*w*(u*u*u)+54*(u*u*u*u)-
     & 6*u*w*(-40+v*v)+6*w*(10-11*v+v*v))+
     & 60*NFV*u*(w*(1+5*w)*(u*u)+7*w*(u*u*u)+2*(u*u*u*u)+
     & 12*(w*w)+u*(-2+w-2*(1+3*v)*(w*w)))))/
     & (u*(omw*omw)*(w*w))))/108._dp+
     & H1(0)*((-10*pi*u*v*H1(1))/9._dp+
     & (pi*((-18*v)/u+(v*(-38+135*u-78*(u*u)+
     & 30*NFV*(1-6*u+4*(u*u))))/(omu*omu)+
     & u*((25+4*nf-40*NFV)*v+
     & (2*omw*(-7+20*NFV+5*v+u*(5+4*v)+2*(u*u)+
     & 2*(v*v)))/(w*w*w))-
     & (2*(20*NFV+w*(-9+2*w))*(u*u))/(w*w*w)))/18._dp)+
     & (pi*v*H1(1)*(omv*omv*omv*(-9+30*NFV*u+4*nf*(u*u))+
     & u*(omv*omv)*(7+20*v+4*nf*(-5+2*v)*(u*u))+
     & u*u*((-37+19*v-4*nf*(-5+3*v))*(u*u*u*u)+
     & (13-4*nf)*(u*u*u*u*u)-
     & 2*(u*u*u)*(-21-2*v+5*(v*v)+nf*(20-24*v+6*(v*v)))+
     & 2*(12-41*v+28*(v*v)+v*v*v)+
     & u*(-26+80*v-74*(v*v)+38*(v*v*v))-
     & 2*(u*u)*(7+27*v-44*(v*v)+8*(v*v*v)+
     & 2*nf*(-10+18*v-9*(v*v)+v*v*v))+
     & 10*NFV*(-29+84*v+6*(-7+3*v)*(u*u*u*u)+6*(u*u*u*u*u)-
     & 75*(v*v)+6*(u*u*u)*(19-18*v+3*(v*v))+
     & u*(97-213*v+132*(v*v)-20*(v*v*v))+22*(v*v*v)+
     & u*u*(-149+228*v-84*(v*v)+4*(v*v*v))))))/
     & (18._dp*u*(omu*omu)*(w*w*w))-(pi*u*v*(G1(2)*G1(2)))/18._dp-
     & (pi*u*v*(H1(0)*H1(0)))/18._dp-(pi*u*v*(H1(1)*H1(1)))/18._dp)+
     & G1(0)*(G1(1)*(((-30+nf)*pi*u*v)/18._dp-pi*u*v*H1(0))+
     & G1(3)*(((-30+nf)*pi*u*v)/18._dp-pi*u*v*H1(0))+
     & G1(2)*(-((-27+nf)*pi*u*v)/18._dp+(8*pi*u*v*H1(0))/9._dp-
     & (pi*u*v*H1(1))/9._dp)-(37*pi*u*v*H2(0,0))/9._dp-(16*pi*u*v*H2(0,1))/9._dp-
     & (10*pi*u*v*H2(1,0))/9._dp-(7*pi*u*v*H2(1,1))/9._dp+
     & (pi*u*H1(1)*(34+26/omw+(21-4*nf+40*NFV)*v-
     & (2*u*(13+17*u+17*v))/(omw*omw)-
     & (v*(-33+49*u-16*(u*u)+10*NFV*(4-3*u+3*(u*u))))/
     & (omu*omu*omu)))/18._dp+
     & (pi*(u*(-198+744/omw+v*(1119-76*nf+390*zeta2))+
     & (54+96/omw-54/w-732/(omw*omw))*(u*u)+(54*(u*u*u))/w-
     & (3*(2*(10-11*v+v*v)+2*(u*u)*(11+4*(6+nf)*v+v*v)-
     & u*(42+(29+12*nf)*v+4*(v*v))))/(omu*omu)+
     & (30*NFV*(u*u)*(3-12*v+(-9+6*v)*(u*u*u*u*u)+77*(v*v)+
     & u*u*u*u*(39-68*v+20*(v*v))-118*(v*v*v)+
     & u*u*u*(-66+216*v-153*(v*v)+26*(v*v*v))+
     & u*(-21+122*v-319*(v*v)+252*(v*v*v)-42*(v*v*v*v))+
     & 54*(v*v*v*v)+u*u*
     & (54-264*v+375*(v*v)-136*(v*v*v)+12*(v*v*v*v))))/
     & (omw*(omu*omu*omu)*(w*w))))/108._dp+
     & H1(0)*((pi*u*v*H1(1))/9._dp+(pi*u*
     & ((-2*(55+41*u))/omw+(60*omw*NFV*u)/w+
     & (110*u)/(omw*omw)+
     & (v*(33-49*u+16*(u*u)-10*NFV*(4-3*u+3*(u*u)))*
     & (w*w*w)+omu*omu*omu*
     & (w*(2*u*(17-19*w)*w+2*w*(2+(39+6*v-2*nf*v)*w)+
     & (-38+47*v)*(u*u)+38*(u*u*u))+
     & 20*NFV*(3*(-1+2*v)*w*(u*u)+(-3+4*v+6*w)*(u*u*u)+
     & 3*(u*u*u*u)+2*v*(w*w*w))))/(omu*omu*omu*(w*w*w))))
     & /18._dp)+(pi*u*v*(G1(2)*G1(2)))/18._dp-(17*pi*u*v*(H1(0)*H1(0)))/18._dp+
     & (pi*u*v*(H1(1)*H1(1)))/18._dp)+
     & G1(2)*(G1(3)*((pi*u*v)/6._dp+(pi*u*v*H1(0))/9._dp+(pi*u*v*H1(1))/9._dp)-
     & (20*pi*u*v*H2(0,0))/9._dp+(pi*u*v*H2(0,1))/9._dp+
     & (2*(-1+10*NFV)*pi*u*v*H2(1,0))/3._dp+
     & (pi*u*H1(1)*(-2+8/omw+(87-4*nf+40*NFV)*v+
     & (2*(-4+omw)*u)/(omw*omw)-
     & (v*(-29+43*u-14*(u*u)+10*NFV*(4-3*u+3*(u*u))))/
     & (omu*omu*omu)))/18._dp+
     & H1(0)*((pi*u*v*H1(1))/3._dp+(pi*u*
     & (omu*omu*omu*omu*
     & (2*(-16+45*NFV)*u*w+
     & omw*(32-65*u+30*NFV*(-3+2*u))*w+
     & (65-24*u+60*NFV*(-1+2*u))*w*(omw*omw)+
     & (-4+4*u+(28+40*NFV*(-3+v)+51*v-4*nf*v)*w)*
     & (omw*omw*omw))+
     & v*w*(omw*omw*omw)*
     & (816+1754*u-108*nf*(1+2*u)-55*(u*u)+14*(u*u*u)+
     & 10*NFV*(-4+7*u-6*(u*u)+3*(u*u*u)))))/
     & (18._dp*w*(omu*omu*omu*omu)*(omw*omw*omw)))+
     & (pi*(u*(-534-168/omw+v*(1119-76*nf+294*zeta2))+
     & 18*(3+24/omw-3/w+10/(omw*omw))*(u*u)+(54*(u*u*u))/w-
     & (3*(2*(10-11*v+v*v)+2*(u*u)*(11+(-41+4*nf)*v+v*v)-
     & u*(42+(-227+12*nf)*v+4*(v*v))))/(omu*omu)+
     & (30*NFV*u*((-9+6*v)*(u*u*u*u*u*u)+24*(omv*omv)*(v*v)+
     & u*u*u*u*u*(39-52*v+20*(v*v))+
     & u*u*u*u*(-66+144*v-105*(v*v)+26*(v*v*v))+
     & u*u*(-21+34*v-71*(v*v)+100*(v*v*v)-26*(v*v*v*v))+
     & u*u*u*(54-144*v+191*(v*v)-88*(v*v*v)+12*(v*v*v*v))+
     & u*(3+12*v-59*(v*v)+34*(v*v*v)+14*(v*v*v*v))))/
     & (omw*(omu*omu*omu)*(w*w))))/108._dp+(pi*u*v*(H1(0)*H1(0)))/6._dp+
     & (pi*u*v*(H1(1)*H1(1)))/6._dp)-(pi*u*v*(H1(1)*H1(1)*H1(1)))/18._dp
      return
      end
      
      
!**********************************************************************
!region4a:q+g-->V+q*
!*********************************************************************/
      
      function Alpha_4a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=1
      return
      end
      
      
      function Alpha_4a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function Alpha_4a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=-32._dp/3._dp+4/(3-3*v)+(27*zeta2)/2._dp+G2(0,2)/3._dp+G2(2,0)/3._dp+
     & (8*G2(3,0))/3._dp-(8*G1(3)*H1(0))/3._dp+G1(0)*((17*H1(0))/3._dp-H1(1)/3._dp)+
     & H1(1)/(3._dp*u)-(8*H2(0,0))/3._dp+(10*H2(1,0))/3._dp+
     & H1(0)*(H1(1)/3._dp-(12*u+omv*v-16*u*v)/(3._dp*u*(omv*omv)))-
     & (G2(1,2)*(1-2*u+2*(u*u)))/(3._dp*(u*u))+
     & G1(1)*((H1(0)*(1-2*u+2*(u*u)))/(3._dp*(u*u))+
     & (H1(1)*(1-2*u+2*(u*u)))/(3._dp*(u*u)))+
     & G1(2)*(-1/(3._dp*u)-H1(1)/3._dp-(H1(0)*(1-2*u+3*(u*u)))/(3._dp*(u*u)))-
     & (3*(G1(0)*G1(0)))/2._dp+(G1(2)*G1(2))/6._dp-(17*(H1(0)*H1(0)))/6._dp+(H1(1)*H1(1))/6._dp
      return
      end
      
      
      function Alpha_4a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=pi*(-4-1/(3._dp*u))+(pi*G1(0))/3._dp+(pi*G1(2))/3._dp+(8*pi*G1(3))/3._dp-
     & (pi*H1(0))/3._dp-(pi*H1(1))/3._dp-(pi*G1(1)*(1-2*u+2*(u*u)))/(3._dp*(u*u))
      return
      end
      
      
      function Alpha_4a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(5*(-8+6*zeta2))/(18._dp*omw)-(16*G4(0,0,1,2))/9._dp+(2*G4(0,0,2,2))/9._dp-
     & 16*G4(0,0,3,0)+(2*G4(0,2,0,2))/9._dp+(8*G4(0,2,1,2))/9._dp+
     & (2*G4(0,2,2,0))/9._dp+(188*G4(0,2,3,0))/9._dp+(8*G4(0,3,0,2))/9._dp+
     & (8*G4(0,3,2,0))/9._dp+(128*G4(0,3,3,0))/9._dp-(20*G4(1,0,1,2))/9._dp+
     & (2*G4(2,0,0,2))/9._dp+(2*G4(2,0,2,0))/9._dp+(2*G4(2,2,0,0))/9._dp+
     & (16*G4(2,3,0,0))/9._dp-(26*G4(2,3,0,2))/9._dp-(26*G4(2,3,2,0))/9._dp+
     & (16*G4(3,0,0,2))/9._dp+(10*G4(3,0,1,2))/9._dp+(16*G4(3,0,2,0))/9._dp+
     & (208*G4(3,0,3,0))/9._dp+(16*G4(3,2,0,0))/9._dp+(10*G4(3,2,1,2))/9._dp-
     & (80*G4(3,2,3,0))/9._dp+(128*G4(3,3,0,0))/9._dp-(128*G4(3,3,3,0))/9._dp+
     & 16*G3(0,0,3)*H1(0)-(188*G3(0,2,3)*H1(0))/9._dp-(8*G3(0,3,2)*H1(0))/9._dp-
     & (128*G3(0,3,3)*H1(0))/9._dp+(26*G3(2,3,2)*H1(0))/9._dp-(208*G3(3,0,3)*H1(0))/9._dp+
     & (80*G3(3,2,3)*H1(0))/9._dp+(128*G3(3,3,3)*H1(0))/9._dp+
     & G3(3,0,0)*((8*(-21+nf))/3._dp+(128*H1(0))/9._dp-(16*H1(1))/9._dp)+
     & G3(3,0,1)*((-10*H1(0))/9._dp-(10*H1(1))/9._dp)+
     & G3(3,2,1)*((-10*H1(0))/9._dp-(10*H1(1))/9._dp)+
     & G3(0,2,1)*((-8*H1(0))/9._dp-(8*H1(1))/9._dp)+
     & G3(0,0,2)*(-7+nf/3._dp-(2*H1(1))/9._dp)+
     & G3(2,0,0)*(-7+nf/3._dp+(16*H1(0))/9._dp-(2*H1(1))/9._dp)+
     & G3(0,0,1)*((16*H1(0))/9._dp+(16*H1(1))/9._dp)+
     & G3(1,0,1)*((20*H1(0))/9._dp+(20*H1(1))/9._dp)+
     & ((202-8*nf-160*NFV-189*u+12*nf*u-384*u*zeta2)*H2(1,1))/(36._dp*u)+
     & (64*H4(0,0,0,0))/3._dp-(80*H4(0,0,1,0))/9._dp-(80*H4(0,1,0,0))/9._dp-
     & (10*H4(0,1,0,1))/9._dp+10*H4(0,1,1,0)-10*H4(1,0,0,0)-(10*H4(1,0,0,1))/9._dp+
     & (260*H4(1,0,1,0))/9._dp+(70*H4(1,1,0,0))/9._dp-(20*H4(1,1,0,1))/9._dp-
     & (110*H4(1,1,1,0))/9._dp+G2(0,0)*
     & (80-(41*nf)/6._dp-(2*zeta2)/3._dp+(8*(-21+nf)*H1(0))/3._dp+
     & (7-nf/3._dp)*H1(1)+(128*H2(0,0))/9._dp-(16*H2(0,1))/9._dp-(16*H2(1,0))/9._dp+
     & (2*H2(1,1))/9._dp+(5*(nf*nf))/36._dp)+
     & G3(0,2,0)*(-7+nf/3._dp-(2*H1(1))/9._dp+H1(0)*(160._dp/9._dp+4/u-2/(u*u)))+
     & G3(2,1,0)*(H1(0)*(-2+2/u-1/(u*u))+H1(1)*(-2+2/u-1/(u*u)))+
     & G3(0,1,1)*(H1(0)*(20._dp/9._dp+2/u-1/(u*u))+H1(1)*(20._dp/9._dp+2/u-1/(u*u)))+
     & G3(0,1,0)*(H1(0)*(4-4/u+2/(u*u))+H1(1)*(4-4/u+2/(u*u)))+
     & G3(2,1,1)*(H1(0)*(8-8/u+4/(u*u))+H1(1)*(8-8/u+4/(u*u)))+
     & G3(0,1,2)*(-2*H1(0)+H1(1)*(-16._dp/9._dp+2/u-1/(u*u))+
     & (78-20/omw-6*nf+(56+60*NFV)/omv-10/u+(2*nf)/u+
     & 24/(u-u*v)-(120*NFV)/(u-u*v)-(41+60*NFV)/(omv*omv)+
     & (2*(16-45*NFV)*u*v)/(omv*omv*omv)-10/(omw*omw)-16/(u*u)-
     & nf/(u*u))/18._dp)+G4(2,1,1,2)*(-8+8/u-4/(u*u))+
     & G4(0,1,0,2)*(-4+4/u-2/(u*u))+G4(0,1,2,0)*(-4+4/u-2/(u*u))+
     & G4(0,1,1,2)*(-20._dp/9._dp-2/u+1/(u*u))+
     & G4(0,1,2,2)*(16._dp/9._dp-2/u+1/(u*u))+G4(2,1,0,2)*(2-2/u+1/(u*u))+
     & G4(2,1,2,0)*(2-2/u+1/(u*u))+
     & (G4(1,0,3,0)*(17+30*NFV*(-2+u)-28*u-160*(u*u)))/(9._dp*(u*u))+
     & (G4(1,2,3,0)*(17+30*NFV*(-2+u)-28*u-160*(u*u)))/(9._dp*(u*u))+
     & (G3(2,0,3)*H1(0)*(-3+30*NFV*(-2+u)+12*u-98*(u*u)))/(9._dp*(u*u))+
     & (G4(1,1,2,2)*(-15+30*NFV*(-2+u)+30*u-32*(u*u)))/(9._dp*(u*u))+
     & (G4(1,2,1,2)*(-15+30*NFV*(-2+u)+30*u-32*(u*u)))/(9._dp*(u*u))+
     & (G4(2,2,3,0)*(-1+10*NFV*(-2+u)+4*u-12*(u*u)))/(3._dp*(u*u))-
     & (G4(2,2,1,2)*(3+30*NFV*(-2+u)-6*u+4*(u*u)))/(9._dp*(u*u))+
     & (20*G4(2,3,3,0)*(-1+2*u+8*(u*u)))/(9._dp*(u*u))-
     & (20*G3(2,3,3)*H1(0)*(-1+2*u+8*(u*u)))/(9._dp*(u*u))+
     & (G3(2,2,3)*H1(0)*(1-10*NFV*(-2+u)-4*u+12*(u*u)))/(3._dp*(u*u))+
     & (G4(1,0,2,2)*(-1-30*NFV*(-2+u)+2*u+16*(u*u)))/(9._dp*(u*u))+
     & (G4(1,2,0,2)*(-1-30*NFV*(-2+u)+2*u+16*(u*u)))/(9._dp*(u*u))+
     & (G4(1,2,2,0)*(-1-30*NFV*(-2+u)+2*u+16*(u*u)))/(9._dp*(u*u))+
     & (G4(2,0,1,2)*(11+30*NFV*(-2+u)-22*u+18*(u*u)))/(9._dp*(u*u))-
     & (G4(1,1,0,2)*(21+30*NFV*(-2+u)-42*u+20*(u*u)))/(9._dp*(u*u))-
     & (G4(1,1,2,0)*(21+30*NFV*(-2+u)-42*u+20*(u*u)))/(9._dp*(u*u))+
     & (G4(2,0,3,0)*(3-30*NFV*(-2+u)-12*u+98*(u*u)))/(9._dp*(u*u))+
     & (G4(1,1,1,2)*(57+30*NFV*(-2+u)-114*u+112*(u*u)))/(9._dp*(u*u))+
     & (G3(1,0,3)*H1(0)*(-17-30*NFV*(-2+u)+28*u+160*(u*u)))/(9._dp*(u*u))+
     & (G3(1,2,3)*H1(0)*(-17-30*NFV*(-2+u)+28*u+160*(u*u)))/(9._dp*(u*u))+
     & G3(2,2,1)*((H1(0)*(3+30*NFV*(-2+u)-6*u+4*(u*u)))/(9._dp*(u*u))+
     & (H1(1)*(3+30*NFV*(-2+u)-6*u+4*(u*u)))/(9._dp*(u*u)))+
     & G3(2,0,1)*(-(H1(0)*(11+30*NFV*(-2+u)-22*u+18*(u*u)))/(9._dp*(u*u))-
     & (H1(1)*(11+30*NFV*(-2+u)-22*u+18*(u*u)))/(9._dp*(u*u)))+
     & G3(1,1,0)*((H1(0)*(21+30*NFV*(-2+u)-42*u+20*(u*u)))/(9._dp*(u*u))+
     & (H1(1)*(21+30*NFV*(-2+u)-42*u+20*(u*u)))/(9._dp*(u*u)))+
     & G3(1,2,1)*((H1(0)*(15-30*NFV*(-2+u)-30*u+32*(u*u)))/(9._dp*(u*u))+
     & (H1(1)*(15-30*NFV*(-2+u)-30*u+32*(u*u)))/(9._dp*(u*u)))+
     & G3(1,1,1)*(-(H1(0)*(57+30*NFV*(-2+u)-114*u+112*(u*u)))/(9._dp*(u*u))-
     & (H1(1)*(57+30*NFV*(-2+u)-114*u+112*(u*u)))/(9._dp*(u*u)))+
     & G2(0,1)*(2*H2(1,0)+H2(0,1)*(-2._dp/9._dp-2/u+1/(u*u))+
     & H2(1,1)*(16._dp/9._dp-2/u+1/(u*u))+
     & (H1(0)*(-78+20/omw+6*nf-(56+60*NFV)/omv+10/u-(2*nf)/u-
     & 24/(u-u*v)+(120*NFV)/(u-u*v)+(41+60*NFV)/(omv*omv)+
     & (2*(-16+45*NFV)*u*v)/(omv*omv*omv)+10/(omw*omw)+
     & 16/(u*u)+nf/(u*u)))/18._dp+
     & (H1(1)*(-78+20/omw+6*nf-(56+60*NFV)/omv+10/u-(2*nf)/u-
     & 24/(u-u*v)+(120*NFV)/(u-u*v)+(41+60*NFV)/(omv*omv)+
     & (2*(-16+45*NFV)*u*v)/(omv*omv*omv)+10/(omw*omw)+
     & 16/(u*u)+nf/(u*u)))/18._dp+
     & (zeta2*(63-126*u+208*(u*u)))/(9._dp*(u*u)))+
     & G3(0,2,2)*(H1(0)*(16._dp/9._dp-2/u+1/(u*u))+
     & (omu*(-2+(-63+4*nf)*u)+40*NFV*(3-3*u+u*u))/(18._dp*omu*u))+
     & G3(2,0,2)*((H1(0)*(1-2*u-10*(u*u)))/(9._dp*(u*u))+
     & (omu*(-2+(-63+4*nf)*u)+40*NFV*(3-3*u+u*u))/(18._dp*omu*u))+
     & G3(2,2,0)*((-2*H1(0)*(2-7*u+10*(u*u)))/(9._dp*(u*u))+
     & (omu*(-2+(-63+4*nf)*u)+40*NFV*(3-3*u+u*u))/(18._dp*omu*u))+
     & G3(1,2,2)*((H1(0)*(-15+30*NFV*(-2+u)+30*u-32*(u*u)))/(9._dp*(u*u))+
     & (81-140*u+nf*(-4+8*u-8*(u*u))+123*(u*u)+
     & 10*NFV*(4-11*u+4*(u*u)))/(18._dp*(u*u)))+
     & G2(2,2)*(H2(1,0)*(-4+32/(9._dp*u)-13/(9._dp*(u*u)))+
     & ((-5-10*NFV*(-2+u)+8*u)*H2(0,0))/(3._dp*(u*u))+
     & (3*u*(202-160*NFV-189*u+4*nf*(-2+3*u))+
     & 24*zeta2*(-5+15*NFV*(-2+u)+13*u-19*(u*u)))/(108._dp*(u*u))-
     & (H2(0,1)*(3+30*NFV*(-2+u)-6*u+4*(u*u)))/(9._dp*(u*u))+
     & (H1(0)*(81-140*u+nf*(-4+8*u-8*(u*u))+123*(u*u)+
     & 10*NFV*(4-11*u+4*(u*u))))/(18._dp*(u*u)))+
     & G2(2,3)*(-20*H2(0,0)-(26*H2(0,1))/9._dp-(26*H2(1,0))/9._dp-
     & (4*zeta2*(-5+10*u+13*(u*u)))/(9._dp*(u*u))+
     & (H1(0)*(36*nf*u*(omv*omv*omv*omv)-
     & 2*(omv*omv*omv)*(14+60*NFV*(-2+v)-17*v+18*nf*(u*u))+
     & 2*(16-45*NFV)*v*(u*u*u)+
     & u*(omv*omv)*(-542+1288*v+60*NFV*(-4+3*v)-795*(v*v))+
     & omv*(u*u)*(570+2*(-679+15*NFV)*v+795*(v*v))))/
     & (18._dp*u*w*(omv*omv*omv)))+
     & G3(2,3,0)*((82*H1(0))/9._dp+(26*H1(1))/9._dp+
     & (-36*nf*u*(omv*omv*omv*omv)+
     & 2*(omv*omv*omv)*(14+60*NFV*(-2+v)-17*v+18*nf*(u*u))+
     & 2*(-16+45*NFV)*v*(u*u*u)-
     & omv*(u*u)*(570+2*(-679+15*NFV)*v+795*(v*v))+
     & u*(omv*omv)*(542-1288*v-60*NFV*(-4+3*v)+795*(v*v)))/
     & (18._dp*u*w*(omv*omv*omv)))+
     & G3(2,1,2)*(H1(0)*(-4+4/u-2/(u*u))+
     & (153-20/omw-8*nf+(8*nf)/u-24/(u-u*v)+41/(omv*omv)-
     & 10/(omw*omw)+87/(u*u)-(4*nf)/(u*u)+(20*NFV)/(u*u)-
     & (20*v)/(u*u)-(v*v)/(u*u)-
     & (2*(75+28*u+NFV*(5-65*v)-76*v+v*v))/(omv*u)+
     & (-32*u*v+10*NFV*(2+9*u*v-2*(v*v*v)))/(omv*omv*omv))/18._dp)+
     & G2(2,1)*(H2(0,1)*(-4+4/u-2/(u*u))+H2(1,0)*(4-4/u+2/(u*u))+
     & (2*zeta2*(1-2*u+2*(u*u)))/(u*u)+
     & (H1(0)*(-153+56/omv+20/omw+24/(u-u*v)-41/(omv*omv)-
     & (20*NFV)/(omv*omv*omv)-
     & (2*(-16+45*NFV)*u*v)/(omv*omv*omv)+10/(omw*omw)+
     & nf*(8-8/u+4/(u*u))+
     & (2*(75+NFV*(5-65*v)-76*v+v*v))/(omv*u)+
     & (-87-20*NFV+20*v+v*v)/(u*u)+
     & (20*NFV*(v*v*v))/(omv*omv*omv)))/18._dp+
     & (H1(1)*(-153+56/omv+20/omw+24/(u-u*v)-41/(omv*omv)-
     & (20*NFV)/(omv*omv*omv)-
     & (2*(-16+45*NFV)*u*v)/(omv*omv*omv)+10/(omw*omw)+
     & nf*(8-8/u+4/(u*u))+
     & (2*(75+NFV*(5-65*v)-76*v+v*v))/(omv*u)+
     & (-87-20*NFV+20*v+v*v)/(u*u)+
     & (20*NFV*(v*v*v))/(omv*omv*omv)))/18._dp)+
     & H2(0,1)*(-41._dp/4._dp-5/(9._dp*omw)+nf/2._dp-(5*NFV)/omv+19/(2._dp*u)-
     & nf/(3._dp*u)-(40*NFV)/(9._dp*u)+55/(9-9*v)-22/(9*u-9*u*v)+
     & (5*NFV)/(u-u*v)+1/w+(5*NFV)/(3._dp*w)-1/(u*w)-(5*NFV)/(3._dp*u*w)+
     & (158*zeta2)/9._dp+(-37._dp/9._dp+5*NFV)/(omv*omv)-(20*NFV)/(9._dp*(w*w))+
     & (10*NFV)/(9._dp*u*(w*w))+(10*NFV*u)/(9._dp*(w*w)))+
     & (H3(0,0,0)*(112*nf*u*(omv*omv*omv*omv*omv)+
     & 2*(omv*omv*omv*omv)*(v+60*NFV*v-112*nf*(u*u))+
     & 2*(-16+45*NFV)*v*(u*u*u*u)+
     & u*(omv*omv*omv)*(-1848+4022*v-180*NFV*v+112*nf*(u*u)-
     & 2295*(v*v))-3*omv*(u*u*u)*
     & (616+10*(-131+4*NFV)*v+683*(v*v))+
     & 2*(omv*omv)*(u*u)*(1848+(-3961+45*NFV)*v+2132*(v*v))))/
     & (18._dp*u*(omv*omv*omv)*(w*w))+
     & H1(1)*((-4*H2(0,0))/3._dp+(5*H2(1,0))/3._dp+
     & ((4*nf*(-57+u*(-28+18*zeta2)))/u+
     & 3*(586-276/omv+1215/u+5892*zeta3+
     & 6*zeta2*(-453+776/omv-60/omw+28/(u-u*v)+
     & (25+2/w)/u-612/w-167/(omv*omv)-30/(omw*omw)+
     & u*(32/(omv*omv)-32/(omv*omv*omv)+
     & (-302+610*w)/(w*w))+151/(w*w)+(151*(u*u))/(w*w))+
     & 10*NFV*(27+27/omv+(6*(-6+1/w))/u-6/w+
     & 6*zeta2*(48/u+6*
     & (-4+4/omu-1/omv+2/(u-u*v)+1/(omv*omv))+
     & u*(-9/(omv*omv)+9/(omv*omv*omv)+
     & (-2+3*w-3*(w*w))/(w*w*w))+
     & ((1+3*v)*(u*u))/(w*w*w)+(u*u*u)/(w*w*w)))))/324._dp)+
     & (H3(1,1,0)*(-189+832/omv-20/omw-4*nf-(2*(11+60*NFV))/u+
     & 52/(u-u*v)-568/w-208/(omv*omv)-10/(omw*omw)+142/(w*w)-
     & (2*(71*(5-7*v)+10*NFV*(-4+3*v))*(u*u))/(w*w*w)-
     & (2*(-213+20*NFV)*(u*u*u))/(w*w*w)-
     & (4*u*(-71*(1-3*v+2*(v*v))+5*NFV*(2-3*v+3*(v*v))))/(w*w*w)))/
     & 18._dp+G2(0,3)*((-176*zeta2)/9._dp-(128*H2(0,0))/9._dp+(8*H2(0,1))/9._dp+
     & (8*H2(1,0))/9._dp+(H1(0)*(-8*nf*u*(omv*omv*omv*omv*omv*omv)+
     & 2*(omv*omv*omv*omv*omv)*
     & (-20+17*v-60*NFV*v+12*nf*(u*u))+
     & 2*(-16+45*NFV)*v*(u*u*u*u*u)-
     & 6*u*(omv*omv*omv*omv)*
     & (-66+163*v-50*NFV*v+4*nf*(u*u)-119*(v*v))-
     & omv*(u*u*u*u)*(276-834*v+501*(v*v)+
     & 10*NFV*(2+17*v+2*(v*v)))+
     & omv*omv*(u*u*u)*(868-2482*v+1645*(v*v)+
     & 10*NFV*(4+14*v+3*(v*v)))+
     & 2*(omv*omv*omv)*(u*u)*
     & (-474+1312*v+4*nf*(u*u)-929*(v*v)-
     & 5*NFV*(2+24*v+3*(v*v)))))/(18._dp*u*(omv*omv*omv)*(w*w*w)))
     & +G3(0,3,0)*((64*H1(0))/9._dp-(8*H1(1))/9._dp+
     & (8*nf*u*(omv*omv*omv*omv*omv*omv)+
     & omv*omv*omv*omv*omv*(40-34*v+120*NFV*v-24*nf*(u*u))+
     & 2*(16-45*NFV)*v*(u*u*u*u*u)+
     & 6*u*(omv*omv*omv*omv)*
     & (-66+163*v-50*NFV*v+4*nf*(u*u)-119*(v*v))+
     & omv*(u*u*u*u)*(276-834*v+501*(v*v)+
     & 10*NFV*(2+17*v+2*(v*v)))-
     & omv*omv*(u*u*u)*(868-2482*v+1645*(v*v)+
     & 10*NFV*(4+14*v+3*(v*v)))+
     & 2*(omv*omv*omv)*(u*u)*
     & (474-1312*v-4*nf*(u*u)+929*(v*v)+5*NFV*(2+24*v+3*(v*v)))
     & )/(18._dp*u*(omv*omv*omv)*(w*w*w)))+
     & G3(3,2,0)*((-88*H1(0))/9._dp-(10*NFV*
     & ((-4+6*v)*(u*u*u*u)+2*(u*u*u*u*u)-u*(-3+v)*(v*v)+
     & v*(u*u)*(3-6*v+2*(v*v))+u*u*u*(2-9*v+6*(v*v))+
     & 2*(v*v*v))+v*(-((16+5*v)*(u*u*u))-2*(u*u*u*u)+
     & u*u*(38-15*v-3*(v*v))+v*(10-28*v+17*(v*v)+v*v*v)+
     & u*(-20+10*v+18*(v*v)+v*v*v)))/(18._dp*(u*u)*(w*w*w)))+
     & G3(3,0,2)*((2*H1(0))/9._dp-(10*NFV*
     & ((-4+6*v)*(u*u*u*u)+2*(u*u*u*u*u)-u*(-3+v)*(v*v)+
     & v*(u*u)*(3-6*v+2*(v*v))+u*u*u*(2-9*v+6*(v*v))+
     & 2*(v*v*v))+v*(-((16+5*v)*(u*u*u))-2*(u*u*u*u)+
     & u*u*(38-15*v-3*(v*v))+v*(10-28*v+17*(v*v)+v*v*v)+
     & u*(-20+10*v+18*(v*v)+v*v*v)))/(18._dp*(u*u)*(w*w*w)))+
     & G2(3,2)*((-158*zeta2)/9._dp+(80*H2(0,0))/9._dp+(10*H2(0,1))/9._dp-10*H2(1,0)+
     & (H1(0)*(10*NFV*((-4+6*v)*(u*u*u*u)+2*(u*u*u*u*u)-
     & u*(-3+v)*(v*v)+v*(u*u)*(3-6*v+2*(v*v))+
     & u*u*u*(2-9*v+6*(v*v))+2*(v*v*v))+
     & v*(-((16+5*v)*(u*u*u))-2*(u*u*u*u)+
     & u*u*(38-15*v-3*(v*v))+v*(10-28*v+17*(v*v)+v*v*v)+
     & u*(-20+10*v+18*(v*v)+v*v*v))))/(18._dp*(u*u)*(w*w*w)))+
     & G3(3,3,0)*((-128*H1(0))/9._dp+(2*
     & (-718+48*omv*nf+NFV*(20-30*v)+737*v)*(u*u*u*u)-
     & 4*(-123+8*nf+5*NFV)*(u*u*u*u*u)+
     & u*u*u*(1356-2764*v-96*nf*(omv*omv)+1471*(v*v)-
     & 10*NFV*(2-9*v+6*(v*v)))+
     & u*u*(-372+(1066-30*NFV)*v+32*nf*(omv*omv*omv)+
     & 3*(-401+20*NFV)*(v*v)+(489-20*NFV)*(v*v*v))+
     & v*(-40+108*v-2*(43+10*NFV)*(v*v)+17*(v*v*v)+v*v*v*v)+
     & u*(-40+264*v-2*(188+15*NFV)*(v*v)+2*(71+5*NFV)*(v*v*v)+
     & v*v*v*v))/(18._dp*(u*u)*(w*w*w)))+
     & G2(3,3)*((-256*zeta2)/9._dp+(128*H2(0,0))/9._dp-
     & (H1(0)*(2*(-718+48*omv*nf+NFV*(20-30*v)+737*v)*(u*u*u*u)-
     & 4*(-123+8*nf+5*NFV)*(u*u*u*u*u)+
     & u*u*u*(1356-2764*v-96*nf*(omv*omv)+1471*(v*v)-
     & 10*NFV*(2-9*v+6*(v*v)))+
     & u*u*(-372+(1066-30*NFV)*v+32*nf*(omv*omv*omv)+
     & 3*(-401+20*NFV)*(v*v)+(489-20*NFV)*(v*v*v))+
     & v*(-40+108*v-2*(43+10*NFV)*(v*v)+17*(v*v*v)+v*v*v*v)+
     & u*(-40+264*v-2*(188+15*NFV)*(v*v)+2*(71+5*NFV)*(v*v*v)+
     & v*v*v*v)))/(18._dp*(u*u)*(w*w*w)))-
     & (H3(0,0,1)*(u*v*(106+3*v)*(omv*omv*omv)-36*v*(omv*omv*omv*omv)+
     & 2*v*(u*u*u)*(37-70*v+5*u*(-2+3*v)+33*(v*v))+
     & omv*omv*(u*u)*(v*(-124+33*v)+
     & 10*NFV*(2-4*u-3*v+3*u*v+2*(u*u)+3*(v*v)))))/
     & (18._dp*u*(omv*omv)*(w*w*w))+
     & (H3(0,1,0)*(-60*nf*u*(omv*omv*omv*omv*omv*omv)+
     & 2*(omv*omv*omv*omv*omv)*((-7+60*NFV)*v+90*nf*(u*u))+
     & 2*(16-45*NFV)*v*(u*u*u*u*u)-
     & 6*u*(omv*omv*omv*omv)*
     & (-165+369*v+50*NFV*v+30*nf*(u*u)-244*(v*v))+
     & 2*(omv*omv*omv)*(u*u)*
     & (-1485+3278*v+30*nf*(u*u)-2054*(v*v)-
     & 5*NFV*(2-30*v+3*(v*v)))+
     & omv*omv*(u*u*u)*(2970-6382*v+3895*(v*v)+
     & 10*NFV*(4-28*v+3*(v*v)))-
     & omv*(u*u*u*u)*(10*NFV*(2-25*v+2*(v*v))+
     & 3*(330-674*v+417*(v*v)))))/(18._dp*u*(omv*omv*omv)*(w*w*w))+
     & (H3(1,0,0)*(12*(10*NFV*(-2+v)-13*nf*u*w)*(omv*omv*omv*omv*omv)+
     & omv*omv*omv*omv*(-60*NFV*u*(-12+7*v)+
     & 2*w*(14+v+156*nf*(u*u)))+
     & 15*omv*w*(u*u*u)*(172-362*v+189*(v*v))-
     & 2*u*(omv*omv*omv)*(w*(-1262+2672*v+78*nf*(u*u)-1401*(v*v))+
     & 5*NFV*u*(74-48*v+3*(v*v)))+
     & 2*(omv*omv)*(u*u)*(w*(-2566+5370*v-2823*(v*v))+
     & 5*NFV*u*(28-10*v+3*(v*v)))+
     & 2*(u*u*u*u)*(16*v*w+5*NFV*
     & (-2-15*v+9*u*v+15*(v*v)+2*(v*v*v)))))/
     & (18._dp*u*(omv*omv*omv)*(w*w*w))+
     & (H2(0,0)*(12*u*(omv*omv*omv*omv*omv)*(-564*zeta2+3*(nf*nf))-
     & 36*nf*(omv*omv*omv*omv)*(-2*v+3*nf*(u*u))+
     & 108*(omv*omv*omv)*(nf*nf)*(u*u*u)-
     & 6*omv*(u*u*u)*(-2099+9999*v+3384*zeta2-6768*v*zeta2-
     & 10498*(v*v)+3384*zeta2*(v*v)+4*nf*(86-268*v+221*(v*v))+
     & 21*(v*v*v)-5*NFV*(6+38*v-32*(v*v)+27*(v*v*v)))+
     & 2*(u*u*u*u)*(4*nf*(113-298*v+221*(v*v))-
     & 3*(1109-3790*v+3486*(v*v)-u*(23+2*v+7*(v*v))-
     & 21*(v*v*v)+15*NFV*
     & (-12+25*v-16*(v*v)+3*u*(2-v+v*v)+9*(v*v*v))))+
     & omv*omv*(12*(564*zeta2-3*(nf*nf))*(u*u*u*u)+
     & 6*v*(-194+398*v-205*(v*v)+20*NFV*(5-10*v+4*(v*v))+
     & v*v*v)+3*(u*u)*(-2769+17510*v+6768*zeta2-
     & 13536*v*zeta2-21114*(v*v)+6768*zeta2*(v*v)+
     & 4*nf*(145-488*v+442*(v*v))+14*(v*v*v)-
     & 10*NFV*(48-58*v+4*(v*v)+9*(v*v*v)))+
     & u*(4*nf*(-145+579*v-876*(v*v)+442*(v*v*v))-
     & 3*(-743+5405*v-11722*(v*v)+7078*(v*v*v)+
     & 20*NFV*(-12+49*v-37*(v*v)+3*(v*v*v)))))))/
     & (108._dp*u*(omv*omv)*(w*w*w))+
     & (H3(1,0,1)*(-57+56/omv+20/omw+24/(u-u*v)+44/w-2/(u*w)-
     & 41/(omv*omv)-(60*NFV*(2+(-2+u)*v))/(u*(omv*omv))+
     & 10/(omw*omw)-9/(w*w)+
     & ((10*NFV*(1+3*u+3*v)-9*w)*(u*u))/(w*w*w)-
     & (20*NFV*(u*u*u))/(w*w*w)-
     & (2*u*(omv*omv*omv*(3*w*(-3+7*w)+5*NFV*(2-3*w+3*(w*w)))+
     & omv*(16-45*NFV)*(w*w*w)+(-16+45*NFV)*(w*w*w)))/
     & (omv*omv*omv*(w*w*w))))/18._dp+
     & (H2(1,0)*((6*(-369+64*nf+16*u))/omv-
     & (6*(5*NFV*(u*u)*(-18+22*v+28*(v*v)+9*(u*u)*(2-v+v*v)-
     & 59*(v*v*v)+3*u*(-12+19*v-16*(v*v)+9*(v*v*v))+
     & 27*(v*v*v*v))+2*(1+8*nf+16*u)*(w*w*w)))/
     & (omv*omv*(w*w*w))+(omw*
     & (-30*NFV*(-1+v)*(24-64*v+52*(v*v)-16*(v*v*v)+
     & 2*u*(-42+75*v-37*(v*v)+3*(v*v*v))+
     & u*u*(96-90*v+4*(v*v)+9*(v*v*v)))+
     & w*(6*(-1+v)*(u*u)*(-246+36*nf+71*w-7*(w*w))+
     & u*(-1+v)*(-999+4032*w+2*(3336+4758*zeta2)*(w*w)-
     & 2*nf*(-54+216*w+335*(w*w)))-
     & 6*(-9+110*w-3*(3+2*nf)*(w*w)+v*v*(w*w)+
     & v*(9-110*w+5*(-27+2*nf)*(w*w)))))+
     & 60*u*(-1+v)*(w*w*w))/(omw*u*(-1+v)*(w*w*w))))/108._dp+
     & G3(1,2,0)*((H1(0)*(6+30*NFV*(-2+u)-6*u-178*(u*u)))/(9._dp*(u*u))+
     & (H1(1)*(1+30*NFV*(-2+u)-2*u-16*(u*u)))/(9._dp*(u*u))-
     & (-63+209*v+(-771+579*v)*(u*u*u*u)+201*(u*u*u*u*u)-238*(v*v)+
     & 2*(u*u*u)*(585-851*v+273*(v*v))+92*(v*v*v)+
     & 10*NFV*(2-6*v+(8+3*v)*(u*u*u*u)+2*(u*u*u*u*u)+6*(v*v)+
     & u*u*u*(-34+30*v+3*(v*v))+u*u*(38-69*v+30*(v*v))+
     & u*(-16+42*v-33*(v*v)+9*(v*v*v)))+nf*(w*w*w)+
     & u*(357-962*v+822*(v*v)-208*(v*v*v)-2*nf*(w*w*w))+
     & 2*(u*u)*(-447+938*v-565*(v*v)+84*(v*v*v)+3*nf*(w*w*w)))/
     & (18._dp*(u*u)*(w*w*w)))+G3(1,0,2)*
     & ((H1(1)*(1+30*NFV*(-2+u)-2*u-16*(u*u)))/(9._dp*(u*u))+
     & (2*H1(0)*(5+15*NFV*(-2+u)-10*u-9*(u*u)))/(9._dp*(u*u))-
     & (-63+209*v+(-771+579*v)*(u*u*u*u)+201*(u*u*u*u*u)-238*(v*v)+
     & 2*(u*u*u)*(585-851*v+273*(v*v))+92*(v*v*v)+
     & 10*NFV*(2-6*v+(8+3*v)*(u*u*u*u)+2*(u*u*u*u*u)+6*(v*v)+
     & u*u*u*(-34+30*v+3*(v*v))+u*u*(38-69*v+30*(v*v))+
     & u*(-16+42*v-33*(v*v)+9*(v*v*v)))+nf*(w*w*w)+
     & u*(357-962*v+822*(v*v)-208*(v*v*v)-2*nf*(w*w*w))+
     & 2*(u*u)*(-447+938*v-565*(v*v)+84*(v*v*v)+3*nf*(w*w*w)))/
     & (18._dp*(u*u)*(w*w*w)))+G2(1,0)*
     & ((-2*H2(0,1)*(5+15*NFV*(-2+u)-10*u+u*u))/(9._dp*(u*u))+
     & (H2(1,1)*(-1-30*NFV*(-2+u)+2*u+16*(u*u)))/(9._dp*(u*u))+
     & (2*H2(1,0)*(-3-15*NFV*(-2+u)+3*u+89*(u*u)))/(9._dp*(u*u))+
     & (H2(0,0)*(-15-30*NFV*(-2+u)+24*u+160*(u*u)))/(9._dp*(u*u))+
     & (zeta2*(11+90*NFV*(-2+u)-34*u+312*(u*u)))/(9._dp*(u*u))+
     & (H1(0)*(-63+209*v+(-771+579*v)*(u*u*u*u)+201*(u*u*u*u*u)-
     & 238*(v*v)+2*(u*u*u)*(585-851*v+273*(v*v))+92*(v*v*v)+
     & 10*NFV*(2-6*v+(8+3*v)*(u*u*u*u)+2*(u*u*u*u*u)+6*(v*v)+
     & u*u*u*(-34+30*v+3*(v*v))+u*u*(38-69*v+30*(v*v))+
     & u*(-16+42*v-33*(v*v)+9*(v*v*v)))+nf*(w*w*w)+
     & u*(357-962*v+822*(v*v)-208*(v*v*v)-2*nf*(w*w*w))+
     & 2*(u*u)*(-447+938*v-565*(v*v)+84*(v*v*v)+3*nf*(w*w*w))))/
     & (18._dp*(u*u)*(w*w*w))+(H1(1)*
     & (-63+209*v+(-771+579*v)*(u*u*u*u)+201*(u*u*u*u*u)-
     & 238*(v*v)+2*(u*u*u)*(585-851*v+273*(v*v))+92*(v*v*v)+
     & 10*NFV*(2-6*v+(8+3*v)*(u*u*u*u)+2*(u*u*u*u*u)+6*(v*v)+
     & u*u*u*(-34+30*v+3*(v*v))+u*u*(38-69*v+30*(v*v))+
     & u*(-16+42*v-33*(v*v)+9*(v*v*v)))+nf*(w*w*w)+
     & u*(357-962*v+822*(v*v)-208*(v*v*v)-2*nf*(w*w*w))+
     & 2*(u*u)*(-447+938*v-565*(v*v)+84*(v*v*v)+3*nf*(w*w*w))))/
     & (18._dp*(u*u)*(w*w*w)))+G3(1,1,2)*
     & (H1(0)*(8-8/u+4/(u*u))+
     & (H1(1)*(15-30*NFV*(-2+u)-30*u+32*(u*u)))/(9._dp*(u*u))+
     & (-39+97*v+(-715+453*v)*(u*u*u*u)+159*(u*u*u*u*u)-68*(v*v)+
     & 2*(u*u*u)*(615-788*v+210*(v*v))+10*(v*v*v)+
     & 10*NFV*(-2+6*v+(2+3*v)*(u*u*u*u)+2*(u*u*u*u*u)-6*(v*v)+
     & u*u*u*(-16+18*v+3*(v*v))+u*u*(16-39*v+24*(v*v))+
     & u*(-2+12*v-21*(v*v)+9*(v*v*v)))+4*nf*(w*w*w)+
     & u*(355-864*v+694*(v*v)-194*(v*v*v)-8*nf*(w*w*w))+
     & 2*(u*u)*(-495+945*v-523*(v*v)+63*(v*v*v)+4*nf*(w*w*w)))/
     & (18._dp*(u*u)*(w*w*w)))+G2(1,1)*
     & (H2(1,0)*(-8+8/u-4/(u*u))+
     & (H2(1,1)*(-15+30*NFV*(-2+u)+30*u-32*(u*u)))/(9._dp*(u*u))+
     & (zeta2*(57+30*NFV*(-2+u)-114*u-8*(u*u)))/(9._dp*(u*u))+
     & (H2(0,1)*(21+30*NFV*(-2+u)-42*u+40*(u*u)))/(9._dp*(u*u))+
     & (H1(0)*(39-97*v+(715-453*v)*(u*u*u*u)-159*(u*u*u*u*u)+
     & 68*(v*v)-2*(u*u*u)*(615-788*v+210*(v*v))-10*(v*v*v)-
     & 10*NFV*(-2+6*v+(2+3*v)*(u*u*u*u)+2*(u*u*u*u*u)-6*(v*v)+
     & u*u*u*(-16+18*v+3*(v*v))+u*u*(16-39*v+24*(v*v))+
     & u*(-2+12*v-21*(v*v)+9*(v*v*v)))-4*nf*(w*w*w)-
     & 2*(u*u)*(-495+945*v-523*(v*v)+63*(v*v*v)+4*nf*(w*w*w))+
     & u*(-355+864*v-694*(v*v)+194*(v*v*v)+8*nf*(w*w*w))))/
     & (18._dp*(u*u)*(w*w*w))+(H1(1)*
     & (39-97*v+(715-453*v)*(u*u*u*u)-159*(u*u*u*u*u)+68*(v*v)-
     & 2*(u*u*u)*(615-788*v+210*(v*v))-10*(v*v*v)-
     & 10*NFV*(-2+6*v+(2+3*v)*(u*u*u*u)+2*(u*u*u*u*u)-6*(v*v)+
     & u*u*u*(-16+18*v+3*(v*v))+u*u*(16-39*v+24*(v*v))+
     & u*(-2+12*v-21*(v*v)+9*(v*v*v)))-4*nf*(w*w*w)-
     & 2*(u*u)*(-495+945*v-523*(v*v)+63*(v*v*v)+4*nf*(w*w*w))+
     & u*(-355+864*v-694*(v*v)+194*(v*v*v)+8*nf*(w*w*w))))/
     & (18._dp*(u*u)*(w*w*w)))+(4320*(-16+45*NFV)*(-1+v)*zeta3*(u*u)*
     & (w*w*w)-360*(-1+v)*(omv*omv)*
     & (8*u*(38*nf+3*(286+15*NFV)*zeta3)+
     & 6*zeta2*(99+360*u+64*(u*u)))*(w*w*w)+
     & 360*omv*u*(-1+v)*(2*(6*(36-8*nf+64*u)*zeta2+
     & 3*(313+32*u)*zeta3)*(w*w*w)+
     & 5*NFV*(6*zeta2*(u*u)*(-58+69*v+117*(v*v)+
     & 36*(u*u)*(2-v+v*v)-236*(v*v*v)+
     & 6*u*(-24+41*v-32*(v*v)+18*(v*v*v))+108*(v*v*v*v))+
     & 36*(2-3*u)*zeta3*(w*w*w)))+
     & omv*omv*omv*(w*(-80*w*(-54*(-2+42*w+v*(2+11*w))*zeta3+
     & u*(108*(-1+v)*zeta3+
     & w*(-140889+156765*v+89154*zeta3-89154*v*zeta3+
     & nf*(-1+v)*(-8755+2502*zeta3))))+
     & 5749110*u*(-1+v)*zeta4*(w*w)+
     & 1080*zeta2*(4*(-1+v)*(u*u)*(-123+18*nf+58*w+14*(w*w))+
     & 2*(9-191*w+2*(89+8*nf)*(w*w)+4*(v*v)*(w*w)-
     & v*(9-191*w+2*(91+10*nf)*(w*w)))+
     & u*(3*(-1+v)*(-111+364*w+3271*(w*w))-
     & 2*nf*(18-72*w-349*(w*w)+v*(-18+72*w+413*(w*w))))))
     & +1800*NFV*(6*(-1+v)*zeta2*
     & (52-184*v+184*(v*v)-48*(v*v*v)+
     & u*(-234+528*v-284*(v*v)+24*(v*v*v))+
     & u*u*(312-357*v+16*(v*v)+36*(v*v*v)))+
     & 144*(-2+v)*zeta3*(w*w*w))))/
     & (38880._dp*u*(-1+v)*(omv*omv*omv)*(w*w*w))+
     & ((33-2*nf)/6._dp-(17*H1(0))/2._dp+H1(1)/2._dp)*(G1(0)*G1(0)*G1(0))+
     & (9*(G1(0)*G1(0)*G1(0)*G1(0)))/8._dp+
     & (-(3+33*u-2*nf*u)/(54._dp*u)-H1(1)/18._dp-
     & (H1(0)*(1-2*u+3*(u*u)))/(18._dp*(u*u)))*(G1(2)*G1(2)*G1(2))+
     & (G1(2)*G1(2)*G1(2)*G1(2))/72._dp+((-17*H1(1))/18._dp+
     & (17*(3*omv*v+u*(3+18*v+2*nf*(omv*omv)-33*(v*v))))/
     & (54._dp*u*(omv*omv)))*(H1(0)*H1(0)*H1(0))+
     & (289*(H1(0)*H1(0)*H1(0)*H1(0)))/72._dp+
     & G1(3)*((158*zeta2*H1(1))/9._dp+(16*(-33+2*nf)*H2(0,0))/9._dp+
     & (64*H3(0,0,0))/3._dp-(80*H3(0,1,0))/9._dp-(80*H3(1,0,0))/9._dp-
     & (10*H3(1,0,1))/9._dp+10*H3(1,1,0)-
     & (H2(0,1)*(10*NFV*((-4+6*v)*(u*u*u*u)+2*(u*u*u*u*u)-
     & u*(-3+v)*(v*v)+v*(u*u)*(3-6*v+2*(v*v))+
     & u*u*u*(2-9*v+6*(v*v))+2*(v*v*v))+
     & v*(-((16+5*v)*(u*u*u))-2*(u*u*u*u)+
     & u*u*(38-15*v-3*(v*v))+v*(10-28*v+17*(v*v)+v*v*v)+
     & u*(-20+10*v+18*(v*v)+v*v*v))))/(18._dp*(u*u)*(w*w*w))-
     & (H2(1,0)*(10*NFV*((-4+6*v)*(u*u*u*u)+2*(u*u*u*u*u)-
     & u*(-3+v)*(v*v)+v*(u*u)*(3-6*v+2*(v*v))+
     & u*u*u*(2-9*v+6*(v*v))+2*(v*v*v))+
     & v*(-((16+5*v)*(u*u*u))-2*(u*u*u*u)+
     & u*u*(38-15*v-3*(v*v))+v*(10-28*v+17*(v*v)+v*v*v)+
     & u*(-20+10*v+18*(v*v)+v*v*v))))/(18._dp*(u*u)*(w*w*w))+
     & (6*zeta2*(2*(286+96*omv*nf-311*v+70*NFV*(-2+3*v))*(u*u*u*u)-
     & 4*(51+16*nf-35*NFV)*(u*u*u*u*u)+
     & u*u*u*(-492+940*v-192*nf*(omv*omv)-637*(v*v)+
     & 70*NFV*(2-9*v+6*(v*v)))+
     & u*u*(84+(26+210*NFV)*v+64*nf*(omv*omv*omv)+
     & (249-420*NFV)*(v*v)+(-219+140*NFV)*(v*v*v))+
     & v*(40-48*v+2*(-41+70*NFV)*(v*v)+85*(v*v*v)+
     & 5*(v*v*v*v))+u*
     & (40-384*v+(436+210*NFV)*(v*v)-2*(17+35*NFV)*(v*v*v)+
     & 5*(v*v*v*v)))-1920*zeta3*(u*u)*(w*w*w))/(108._dp*(u*u)*(w*w*w))
     & +(76._dp/3._dp-(8*nf)/9._dp-(8*H1(1))/9._dp)*(H1(0)*H1(0))+
     & (68*(H1(0)*H1(0)*H1(0)))/9._dp+
     & H1(0)*((-4*H1(1))/3._dp+(-1584*u*zeta2*(omv*omv*omv*omv*omv)-
     & 3*(u*u*u*u)*(-1049+2106*v-1079*(v*v)-
     & 2*u*(23+2*v+7*(v*v))-42*(v*v*v)+
     & 30*NFV*(-12+25*v-16*(v*v)+3*u*(2-v+v*v)+9*(v*v*v)))
     & +3*omv*(u*u*u)*(-5603+8720*v-1584*zeta2+3168*v*zeta2-
     & 3157*(v*v)-1584*zeta2*(v*v)+
     & 4*nf*(125-196*v+71*(v*v))-42*(v*v*v)+
     & 10*NFV*(6+32*v-32*(v*v)+27*(v*v*v)))+
     & omv*omv*(4*(-71*nf+396*zeta2)*(u*u*u*u)+
     & 6*v*(-6+22*v-17*(v*v)+20*NFV*(6-13*v+6*(v*v))+
     & v*v*v)+3*(u*u)*
     & (7032-10492*v+1584*zeta2-3168*v*zeta2+3039*(v*v)+
     & 1584*zeta2*(v*v)-4*nf*(152-232*v+71*(v*v))+
     & 14*(v*v*v)-10*NFV*(48-74*v+4*(v*v)+9*(v*v*v)))+
     & u*(-4*nf*(-152+402*v-321*(v*v)+71*(v*v*v))-
     & 3*(2524-6456*v+4923*(v*v)-973*(v*v*v)+
     & 20*NFV*(-12+56*v-46*(v*v)+3*(v*v*v))))))/
     & (108._dp*u*(omv*omv)*(w*w*w))-(4*(H1(1)*H1(1)))/9._dp))+
     & H1(0)*H1(0)*((68*H2(0,0))/9._dp-(85*H2(1,0))/9._dp+
     & (H1(1)*(-17+32*v-15*(v*v)+
     & u*(9-34*v-2*nf*(omv*omv)+33*(v*v))))/(18._dp*u*(omv*omv))+
     & (12*omv*(57-2*nf)*v-u*
     & (36*(omv*omv)*(nf*nf)-4*nf*(539-1126*v+611*(v*v))+
     & 3*(6927-15406*v+18*zeta2*(omv*omv)+9391*(v*v))))/
     & (216._dp*u*(omv*omv))-(5*(H1(1)*H1(1)))/12._dp)+
     & G1(0)*G1(0)*(-G2(0,2)/2._dp-G2(2,0)/2._dp-4*G2(3,0)+4*G1(3)*H1(0)+
     & ((-9+(-30+nf)*u)*H1(1))/(18._dp*u)+4*H2(0,0)-5*H2(1,0)-
     & (4284-366*omv*nf-4140*v+810*omv*zeta2+5*omv*(nf*nf))/
     & (72._dp*omv)+G1(1)*(H1(0)*(-1+1/u-1/(2._dp*(u*u)))+
     & H1(1)*(-1+1/u-1/(2._dp*(u*u))))+
     & G1(2)*(1/(2._dp*u)+H1(1)/2._dp+(H1(0)*(3-2/u+1/(u*u)))/2._dp)+
     & G2(1,2)*(1-1/u+1/(2._dp*(u*u)))+
     & H1(0)*((-3*H1(1))/2._dp+(9*omv*v+
     & u*(51-30*v+10*nf*(omv*omv)-57*(v*v)))/(18._dp*u*(omv*omv)))
     & -(G1(2)*G1(2))/4._dp+(67*(H1(0)*H1(0)))/4._dp-(H1(1)*H1(1))/4._dp)+
     & G2(2,0)*(H2(1,0)*(-52._dp/9._dp-2/u+1/(u*u))+
     & (H2(0,0)*(5+10*NFV*(-2+u)-8*u-26*(u*u)))/(3._dp*(u*u))+
     & (H2(0,1)*(10+30*NFV*(-2+u)-20*u+28*(u*u)))/(9._dp*(u*u))+
     & (H1(1)*(omu*(1+(33-2*nf)*u)-20*NFV*(3-3*u+u*u)))/
     & (9._dp*omu*u)+H1(0)*(H1(1)/9._dp+
     & (-288/omv+72/(omv*omv)-
     & (20*NFV*(-1+v)*w*(1+6*u-6*(u*u)+u*u*u)+
     & omu*((-1+v)*(4+(-1029+46*nf)*w)*(u*u)+
     & (-1+v)*w*(-63+nf+20*v+v*v)-
     & 2*u*(-2+72*w-nf*w+v*(2+(-57+nf)*w)+w*(v*v))))/
     & (omu*(-1+v)*w*(u*u)))/18._dp)+
     & (6*(u*u*u)*(15*NFV*u*(3*u*(2-v+v*v)+
     & 2*(-3+5*v-5*(v*v)+3*(v*v*v)))-32*(w*w))+
     & 6*omv*(33+16*u)*(u*u)*(w*w)+
     & omv*omv*(-(w*(u*(54+w*(-216+6*nf+6*v-24*zeta2))+
     & 12*w*zeta2+(-108+w*(-129+67*nf+882*zeta2))*(u*u)+
     & 6*(9+7*w)*(u*u*u)))+
     & 30*NFV*(u*u*(26-28*v+6*(v*v))+u*u*u*(-22+7*v+9*(v*v))+
     & 24*zeta2*(w*w)+u*(-4+8*v-12*zeta2*(w*w)))))/
     & (108._dp*(omv*omv)*(u*u)*(w*w))-(17*(H1(0)*H1(0)))/18._dp+(H1(1)*H1(1))/18._dp
     & )+G2(0,2)*((-188*H2(0,0))/9._dp+H2(1,0)*(18+4/u-2/(u*u))+
     & H2(0,1)*(-8._dp/9._dp+2/u-1/(u*u))+
     & (H1(1)*(omu*(1+(33-2*nf)*u)-20*NFV*(3-3*u+u*u)))/
     & (9._dp*omu*u)+(omv*omv*
     & (-(w*(6*u*(9+w*(-36+nf+v-36*zeta2))+108*w*zeta2+
     & (-108+w*(-129+67*nf-2118*zeta2))*(u*u)+
     & 6*(9+7*w)*(u*u*u)))+
     & 30*NFV*u*(-4+8*v+u*(26-28*v+6*(v*v))+
     & u*u*(-22+7*v+9*(v*v))))+
     & 6*(u*u*u)*(15*NFV*u*(3*u*(2-v+v*v)+
     & 2*(-3+5*v-5*(v*v)+3*(v*v*v)))-32*(w*w))+
     & 6*omv*(33+16*u)*(u*u)*(w*w))/(108._dp*(omv*omv)*(u*u)*(w*w))+
     & H1(0)*(H1(1)/9._dp+(144+32/omv-20/omw+2/(u-u*v)+
     & (2*(5-60*NFV+1/w))/u-44/w-8/(omv*omv)-
     & 10/(omw*omw)+nf*(-10+2/u-1/(u*u))-16/(u*u)+9/(w*w)+
     & ((10*NFV*(-4+3*v)+9*w)*(u*u))/(w*w*w)+
     & (20*NFV*(u*u*u))/(w*w*w)-
     & (2*u*(omu*(3*(-4+7*u+7*v)*w-5*NFV*(2-3*v+3*(v*v)))+
     & 20*NFV*(w*w*w)))/(omu*(w*w*w)))/18._dp)-
     & (17*(H1(0)*H1(0)))/18._dp+(H1(1)*H1(1))/18._dp)+
     & G1(2)*G1(2)*((987+48/omv-76*nf+36/u+270*zeta2)/216._dp+G2(0,2)/18._dp+
     & G2(2,0)/18._dp+(4*G2(3,0))/9._dp-(4*G1(3)*H1(0))/9._dp+
     & ((33-2*nf+3/u)*H1(1))/18._dp-(4*H2(0,0))/9._dp+(5*H2(1,0))/9._dp-
     & (G2(1,2)*(1-2*u+2*(u*u)))/(18._dp*(u*u))+
     & H1(0)*((H1(1)*(2-4*u+7*(u*u)))/(18._dp*(u*u))+
     & (omv*omv*(3-2*nf*(u*u))+
     & u*(-4+7*v-3*(v*v)+u*(27-62*v+39*(v*v))))/
     & (18._dp*(omv*omv)*(u*u)))+
     & ((4-8*u-7*(u*u))*(H1(0)*H1(0)))/(36._dp*(u*u))+(H1(1)*H1(1))/12._dp)+
     & G2(3,0)*(H1(0)*(32-(20*nf)/9._dp+(8*H1(1))/9._dp)-(64*H2(0,0))/3._dp+
     & (8*H2(0,1))/9._dp+(88*H2(1,0))/9._dp+
     & (H1(1)*((72-74*v+20*NFV*(-2+3*v))*(u*u*u*u)+
     & 4*(-6+5*NFV)*(u*u*u*u*u)+
     & u*u*u*(-72+128*v-77*(v*v)+10*NFV*(2-9*v+6*(v*v)))+
     & v*v*(10+4*(-7+5*NFV)*v+17*(v*v)+v*v*v)+
     & u*v*(-20+10*(1+3*NFV)*v-2*(-9+5*NFV)*(v*v)+v*v*v)+
     & u*u*(24+(-34+30*NFV)*v+(57-60*NFV)*(v*v)+
     & (-27+20*NFV)*(v*v*v))))/(18._dp*(u*u)*(w*w*w))+
     & (1584*u*zeta2*(omv*omv*omv*omv*omv)+
     & 3*(u*u*u*u)*(-1049+2106*v-1079*(v*v)-
     & 2*u*(23+2*v+7*(v*v))-42*(v*v*v)+
     & 30*NFV*(-12+25*v-16*(v*v)+3*u*(2-v+v*v)+9*(v*v*v)))-
     & 3*omv*(u*u*u)*(-5603+8720*v-1584*zeta2+3168*v*zeta2-
     & 3157*(v*v)-1584*zeta2*(v*v)+4*nf*(125-196*v+71*(v*v))-
     & 42*(v*v*v)+10*NFV*(6+32*v-32*(v*v)+27*(v*v*v)))+
     & omv*omv*(4*(71*nf-396*zeta2)*(u*u*u*u)-
     & 6*v*(-6+22*v-17*(v*v)+20*NFV*(6-13*v+6*(v*v))+
     & v*v*v)+3*(u*u)*
     & (-7032+10492*v-1584*zeta2+3168*v*zeta2-3039*(v*v)-
     & 1584*zeta2*(v*v)+4*nf*(152-232*v+71*(v*v))-
     & 14*(v*v*v)+10*NFV*(48-74*v+4*(v*v)+9*(v*v*v)))+
     & u*(4*nf*(-152+402*v-321*(v*v)+71*(v*v*v))+
     & 3*(2524-6456*v+4923*(v*v)-973*(v*v*v)+
     & 20*NFV*(-12+56*v-46*(v*v)+3*(v*v*v))))))/
     & (108._dp*u*(omv*omv)*(w*w*w))-(68*(H1(0)*H1(0)))/9._dp+(4*(H1(1)*H1(1)))/9._dp
     & )+((987+48/omv-76*nf+36/u+270*zeta2)/216._dp-(4*H2(0,0))/9._dp+
     & (5*H2(1,0))/9._dp)*(H1(1)*H1(1))+
     & G2(1,2)*(-(H2(1,0)*(-11+30*NFV*(-2+u)+16*u+128*(u*u)))/(9._dp*(u*u))+
     & (H2(0,0)*(1-30*NFV*(-2+u)-8*u+196*(u*u)))/(9._dp*(u*u))+
     & (H1(1)*(-84+146*u-129*(u*u)-10*NFV*(4-11*u+4*(u*u))+
     & nf*(4-8*u+8*(u*u))))/(18._dp*(u*u))+
     & H1(0)*(-(H1(1)*(1-2*u+2*(u*u)))/(9._dp*(u*u))+
     & (-85+268*u-228*(u*u)+8*nf*(1-2*u+2*(u*u))-
     & 20*NFV*(2-u+2*(u*u)))/(18._dp*(u*u)))+
     & (-819-54/omv-60/omw+1656/u+60/(omw*u)+42*u-(96*u)/omv+
     & (6*v)/u+150/(u-u*v)-54/w+54/(u*w)+(54*u)/w-3132*zeta2+
     & (372*zeta2)/u-192/(omv*omv)+(192*u)/(omv*omv)-1119/(u*u)-
     & (6*v)/(u*u)-54/(w*(u*u))-(150*zeta2)/(u*u)+
     & (4*nf*(19-32*u+29*(u*u)))/(u*u)+
     & (30*NFV*(48*(-2+u)*zeta2+
     & (omv*omv*(-6*(u*u)*(4-2*v+v*v)-
     & 3*u*(2-9*v+5*(v*v))+2*(3-7*v+6*(v*v)))+
     & u*u*u*(24-70*v+44*(v*v)-9*(u*u)*(2-v+v*v)+
     & 11*(v*v*v)-6*u*(-3+2*v-5*(v*v)+3*(v*v*v))-
     & 9*(v*v*v*v)))/(omv*omv*(w*w))))/(u*u))/108._dp+
     & (17*(1-2*u+2*(u*u))*(H1(0)*H1(0)))/(18._dp*(u*u))-
     & ((1-2*u+2*(u*u))*(H1(1)*H1(1)))/(18._dp*(u*u)))+
     & G1(0)*(G2(0,2)*(5._dp/3._dp-nf/18._dp+H1(0))+G2(2,0)*(5._dp/3._dp-nf/18._dp+H1(0))+
     & G2(3,0)*((-4*(-30+nf))/9._dp+8*H1(0))-(64*H3(0,0,0))/3._dp+
     & (8*H3(0,0,1))/9._dp+(88*H3(0,1,0))/9._dp+(56*H3(1,0,0))/3._dp-
     & (10*H3(1,0,1))/9._dp-(20*H3(1,1,0))/9._dp+
     & (H2(1,1)*(omu*(-2+(-63+4*nf)*u)+40*NFV*(3-3*u+u*u)))/
     & (18._dp*omu*u)+G2(1,2)*(H1(0)*(-2+2/u-1/(u*u))+
     & ((-30+nf)*(1-2*u+2*(u*u)))/(18._dp*(u*u)))+
     & (H1(1)*(u*w*(76*nf-198*zeta2)*(omv*omv*omv)+
     & 3*omv*w*(u*u)*(263-147*v+14*(v*v))+
     & omv*omv*(w*(u*(-669+393*v)+(-76*nf+198*zeta2)*(u*u)-
     & 6*(-3-7*v+v*v))-
     & 30*NFV*(-4+8*v+u*(26-28*v+6*(v*v))+
     & u*u*(-22+7*v+9*(v*v))))-
     & 6*(u*u*u)*(w*(23+2*v+7*(v*v))+
     & 15*NFV*(3*u*(2-v+v*v)+2*(-3+5*v-5*(v*v)+3*(v*v*v))))
     & ))/(108._dp*u*(omv*omv)*(w*w))+
     & (4*nf*(-57+27/w+(27*(1+omw)*u)/(omw*w)+102*zeta2)+
     & 6*zeta2*(30*NFV*(4-4/omu-2/omv-12/u+4/(u-u*v)+
     & 2/(omv*omv)+(3*u*v)/(omv*omv*omv))+
     & ((20-22*v)*(omv*omv*omv*omv)+
     & u*(omv*omv*omv)*(-1030+2046*v-969*(v*v))+
     & 2*(omv*omv)*(u*u)*(1000-2111*v+1132*(v*v))+
     & u*u*u*(-990+3220*v-32*u*v-3445*(v*v)+1215*(v*v*v)))/
     & (u*(omv*omv*omv)*(w*w)))+
     & 3*((-92+90*NFV)/omv+
     & (omw*omw*(-3*w*(123+158*u-772*w+108*w*zeta3)+
     & 10*NFV*(-2-7*u+9*u*v+3*w+9*(u*u)))+80*u*(w*w)-
     & 40*omw*(2+11*u)*(w*w))/(omw*omw*(w*w))))/108._dp+
     & (H2(0,1)*(-123+24/omv+6*nf-20/u+22/(u-u*v)+44/w-2/(u*w)-
     & (42*u)/w-33/(omv*omv)-(32*u)/(omv*omv)+
     & (32*u)/(omv*omv*omv)-9/(w*w)+(18*u)/(w*w)-
     & (9*(u*u))/(w*w)+10*NFV*
     & (-4+4/omu+6/omv+12/u-12/(u-u*v)-6/(omv*omv)+
     & u*(9/(omv*omv)-9/(omv*omv*omv)+
     & (-2+3*w-3*(w*w))/(w*w*w))+((1+3*v)*(u*u))/(w*w*w)+
     & (u*u*u)/(w*w*w))))/18._dp+
     & (H2(0,0)*(2019-304/omv-96*nf-30/u+28/(u-u*v)-244/w+
     & 2/(u*w)+(242*u)/w+49/(omv*omv)-(32*u)/(omv*omv)+
     & (32*u)/(omv*omv*omv)+62/(w*w)-(124*u)/(w*w)+
     & (62*(u*u))/(w*w)-(10*NFV*v*
     & (6*u*(-7+2*v)*(omv*omv*omv*omv)+
     & 12*(omv*omv*omv*omv*omv)-
     & 3*(omv*omv)*(u*u*u)*(20-15*v+2*(v*v))+
     & omv*omv*omv*(u*u)*(63-36*v+4*(v*v))+
     & 3*(u*u*u*u)*(16-27*v+u*(-10+7*v)+3*(u*u)+13*(v*v)-
     & 2*(v*v*v))))/(omu*u*(omv*omv*omv)*(w*w*w))))/18._dp+
     & (H2(1,0)*(-783+288/omv+46*nf+(12*(-1+10*NFV))/u+
     & 18/(u-u*v)-284/w-72/(omv*omv)+71/(w*w)-
     & ((10*NFV*(-4+3*v)-71*w)*(u*u))/(w*w*w)-
     & (20*NFV*(u*u*u))/(w*w*w)-
     & (2*u*(omu*(71*(-1+2*u+2*v)*w+5*NFV*(2-3*v+3*(v*v)))-
     & 20*NFV*(w*w*w)))/(omu*(w*w*w))))/18._dp+
     & ((17*H1(0))/18._dp-H1(1)/18._dp)*(G1(2)*G1(2))+
     & G1(3)*((4*(-30+nf)*H1(0))/9._dp-8*(H1(0)*H1(0)))+
     & ((17*H1(1))/6._dp-(18*omv*v+
     & u*(375-606*v+2*nf*(omv*omv)+159*(v*v)))/
     & (18._dp*u*(omv*omv)))*(H1(0)*H1(0))+
     & G1(1)*(-((-30+nf)*H1(1)*(1-2*u+2*(u*u)))/(18._dp*(u*u))+
     & H1(0)*(H1(1)*(2-2/u+1/(u*u))-
     & ((-30+nf)*(1-2*u+2*(u*u)))/(18._dp*(u*u)))+
     & (2-2/u+1/(u*u))*(H1(0)*H1(0)))-(289*(H1(0)*H1(0)*H1(0)))/18._dp+
     & G1(2)*(-((-30+nf)*(-2+3*u))/(36._dp*u)+H1(1)/6._dp+
     & H1(0)*((-16*H1(1))/9._dp+(-30+nf+42*u-2*nf*u-84*(u*u)+
     & 2*nf*(u*u))/(18._dp*(u*u)))+
     & (-35._dp/9._dp+2/u-1/(u*u))*(H1(0)*H1(0))+(H1(1)*H1(1))/9._dp)+
     & H1(0)*((9._dp/2._dp-nf/9._dp+1/u)*H1(1)-8*H2(0,0)+10*H2(1,0)+
     & (4662*u*zeta2*(omv*omv*omv*omv*omv)+
     & 3*(u*u*u*u)*(-2453+4894*v-2339*(v*v)-
     & 2*u*(23+2*v+7*(v*v))-42*(v*v*v)+
     & 30*NFV*(-12+19*v-16*(v*v)+3*u*(2-v+v*v)+9*(v*v*v)))
     & +3*omv*(u*u*u)*(9815-17066*v+4662*zeta2-9324*v*zeta2+
     & 7097*(v*v)+4662*zeta2*(v*v)-
     & 4*nf*(215-376*v+161*(v*v))+42*(v*v*v)-
     & 10*NFV*(6-4*v-32*(v*v)+27*(v*v*v)))+
     & omv*omv*(7*(92*nf-666*zeta2)*(u*u*u*u)-
     & 6*v*(3+NFV*(20-40*v)+4*v-8*(v*v)+v*v*v)+
     & 3*(u*u)*(-11244+18802*v-4662*zeta2+9324*v*zeta2-
     & 7139*(v*v)-4662*zeta2*(v*v)+
     & 4*nf*(242-412*v+161*(v*v))-14*(v*v*v)+
     & 10*NFV*(48-64*v+4*(v*v)+9*(v*v*v)))+
     & u*(4*nf*(-242+672*v-591*(v*v)+161*(v*v*v))+
     & 3*(3928-10594*v+9077*(v*v)-2393*(v*v*v)+
     & 20*NFV*(-12+32*v-20*(v*v)+3*(v*v*v))))))/
     & (108._dp*u*(omv*omv)*(w*w*w))+(5*(H1(1)*H1(1)))/6._dp)-
     & (H1(1)*H1(1))/6._dp-(H1(1)*H1(1)*H1(1))/18._dp)+
     & G1(2)*(G2(3,0)*(-4._dp/3._dp-(8*H1(0))/9._dp-(8*H1(1))/9._dp)+
     & G2(0,2)*(-1._dp/6._dp-H1(0)/9._dp-H1(1)/9._dp)+
     & G2(2,0)*(-1._dp/6._dp-H1(0)/9._dp-H1(1)/9._dp)+
     & H3(0,0,1)*(10._dp/9._dp-4/u+2/(u*u))+
     & H1(1)*((8*H2(0,0))/9._dp-(10*H2(1,0))/9._dp+
     & (6*u*(107*(-1+v)+u*(-78+70*v))+
     & omv*(8*u*(60*NFV+nf*(3+5*u))+
     & 6*zeta2*(30-60*NFV*(-2+u)-60*u-289*(u*u))))/
     & (108._dp*omv*(u*u)))+(H3(0,0,0)*
     & (-30*NFV*(-2+u)+(1-4*u)*(1-4*u)))/(9._dp*(u*u))+
     & (H3(1,1,0)*(26-52*u-98*(u*u)))/(9._dp*(u*u))+
     & (H3(1,0,0)*(15+30*NFV*(-2+u)-24*u+20*(u*u)))/(9._dp*(u*u))+
     & (H3(1,0,1)*(11+30*NFV*(-2+u)-22*u+30*(u*u)))/(9._dp*(u*u))-
     & (H3(0,1,0)*(7+30*NFV*(-2+u)-20*u+190*(u*u)))/(9._dp*(u*u))+
     & G2(1,2)*((1-2*u+2*(u*u))/(6._dp*(u*u))+
     & (H1(0)*(1-2*u+2*(u*u)))/(9._dp*(u*u))+
     & (H1(1)*(1-2*u+2*(u*u)))/(9._dp*(u*u)))+
     & (H2(1,0)*(759-320/omv+20/omw+28/u-34/(u-u*v)-4/w+
     & 4/(u*w)+80/(omv*omv)+10/(omw*omw)-8/(u*u)-
     & (10*NFV*(4-11*u+4*(u*u)))/(u*u)-
     & (4*nf*(-1+2*u+7*(u*u)))/(u*u)))/18._dp+
     & ((-4*nf*(-(u*(57+28*u))+18*zeta2*(-1+2*u+9*(u*u))))/(u*u)+
     & 3*(-586+276/omv-1215/u-2556*zeta3+(1296*zeta3)/u+
     & 10*NFV*(-27-27/omv+6/w+(6*(6-1/w-6*zeta3))/u+
     & 6*zeta2*(22-24/omu+6/omv-47/u-12/(u-u*v)-
     & 6/(omv*omv)-(9*u*v)/(omv*omv*omv)-2/(u*u))+
     & (72*zeta3)/(u*u))-(612*zeta3)/(u*u)+
     & 6*zeta2*(1080-264/omv+60/omw-10/(u-u*v)-
     & (9+2*v-4/w)/u-4/w+39/(omv*omv)+
     & (32*u*v)/(omv*omv*omv)+30/(omw*omw)+
     & (-14+20*v+v*v)/(u*u))))/324._dp+
     & (H2(0,1)*(30-56/omv-20/omw+(2*(-5+v))/u-24/(u-u*v)+
     & 41/(omv*omv)-(32*u*v)/(omv*omv*omv)-10/(omw*omw)-
     & (-6+20*v+v*v)/(u*u)+
     & (10*NFV*(2*u*(5+v)*(omv*omv)-2*(omv*omv*omv)+
     & 9*v*(u*u*u)+2*(u*u)*(-1+6*v-6*(v*v)+v*v*v)))/
     & (omv*omv*omv*(u*u))))/18._dp+
     & (H2(0,0)*((250*NFV+304*u-130*NFV*v)/(omv*u)+
     & (-49+32*u)/(omv*omv)+
     & ((-1+v)*(4+3*(-401+20*nf)*w)*(u*u)+
     & (-1+v)*w*(-164+12*nf-20*NFV+20*v+v*v)-
     & 2*u*(-2+181*w-12*nf*w+2*v*(1-98*w+6*nf*w)+w*(v*v)))/
     & ((-1+v)*w*(u*u))+
     & (2*(-16*u+5*NFV*(-2+3*(4+3*u)*v-12*(v*v)+2*(v*v*v))))/
     & (omv*omv*omv)))/18._dp+
     & G1(3)*(H1(0)*(4._dp/3._dp+(8*H1(1))/9._dp)+(8*(H1(0)*H1(0)))/9._dp)+
     & ((H1(1)*(-2+4*u+11*(u*u)))/(18._dp*(u*u))+
     & ((57-2*nf*(omu*omu))*(omv*omv)+
     & u*(-97+196*v-99*(v*v)+u*(105-194*v+81*(v*v))))/
     & (18._dp*(omv*omv)*(u*u)))*(H1(0)*H1(0))+
     & (17*(1-2*u+3*(u*u))*(H1(0)*H1(0)*H1(0)))/(18._dp*(u*u))+
     & ((-33+2*nf-3/u)*(H1(1)*H1(1)))/18._dp+
     & H1(0)*((8*H2(0,0))/9._dp-(10*H2(1,0))/9._dp+
     & (H1(1)*(2*omv*u+omv*omv*(-3+4*nf*(u*u))-
     & 8*(u*u)*(6-14*v+9*(v*v))))/(18._dp*(omv*omv)*(u*u))+
     & (-1239-474/omv+954/u+60/(omw*u)+42*u-(96*u)/omv+
     & (6*v)/u+432/(u-u*v)-162/w+162/(u*w)+(54*u)/w-
     & 3810*zeta2+(324*zeta2)/u+180/(omv*omv)+
     & (192*u)/(omv*omv)-1119/(u*u)-(6*v)/(u*u)-54/(w*(u*u))-
     & (126*zeta2)/(u*u)+(4*nf*(19-26*u+39*(u*u)))/(u*u)+
     & 30*NFV*(-6+(3*(-4+3*u))/omv+3/(u-u*v)-6/w-
     & (18*u)/(omv*omv)+u*(-9+4/w-8/(w*w))+
     & (1+12/w+48*zeta2-8/(w*w))/u+
     & (2*(6-5/w-48*zeta2+2/(w*w)))/(u*u)+8/(w*w)+
     & (4*(u*u))/(w*w)))/108._dp-
     & ((1-2*u+5*(u*u))*(H1(1)*H1(1)))/(18._dp*(u*u)))-(H1(1)*H1(1)*H1(1))/18._dp
     & )+H1(0)*((76._dp/3._dp-(8*nf)/9._dp)*H2(0,0)+(5*(-57+2*nf)*H2(1,0))/9._dp+
     & H1(1)*((-8*H2(0,0))/9._dp+(10*H2(1,0))/9._dp-
     & ((2*nf*(-6+47*u)+378*u*zeta2)*(omv*omv)+
     & 3*(u*(-453+938*v-509*(v*v))+6*(19-37*v+18*(v*v))))/
     & (108._dp*u*(omv*omv)))+
     & (4*nf*(314+840/omv-24/(u-u*v)+(-57+81/w)/u+81/w-
     & (81*(1+omw)*u)/(omw*w)-306*zeta2-300/(omv*omv))+
     & 3*(-13310-13104/omv+240/omw+1215/u+(1320*u)/omw-
     & 108/(u-u*v)-315/w-1107/(u*w)+(1422*u)/w+10242*zeta2-
     & (8160*zeta2)/omv-(282*zeta2)/u-(102*zeta2)/(u-u*v)-
     & (672*zeta2)/w-(24*zeta2)/(u*w)+(696*u*zeta2)/w+5676*zeta3+
     & 4536/(omv*omv)+(1392*zeta2)/(omv*omv)-
     & (768*u*zeta2)/(omv*omv)+(768*u*zeta2)/(omv*omv*omv)-
     & (240*u)/(omw*omw)+(210*zeta2)/(w*w)-(420*u*zeta2)/(w*w)+
     & (210*zeta2*(u*u))/(w*w)+
     & 10*NFV*((-3*v*(9*(-2+v)*(u*u)-2*(5-11*v+6*(v*v))+
     & u*(28-37*v+9*(v*v))))/(omv*u*(w*w))+
     & 6*zeta2*(48/u+24*
     & (-1+1/omu+1/omv-2/(u-u*v)-1/(omv*omv))+
     & u*(36/(omv*omv)-36/(omv*omv*omv)-
     & (5*(2-3*w+3*(w*w)))/(w*w*w))+
     & (5*(1+3*v)*(u*u))/(w*w*w)+(5*(u*u*u))/(w*w*w)))))/324._dp
     & +((2-5*v+3*(v*v)+u*(21-50*v-2*nf*(omv*omv)+33*(v*v)))*
     & (H1(1)*H1(1)))/(18._dp*u*(omv*omv))+(H1(1)*H1(1)*H1(1))/18._dp)+
     & ((3+33*u-2*nf*u)*(H1(1)*H1(1)*H1(1)))/(54._dp*u)+
     & G1(1)*(H3(0,0,1)*(-4+4/u-2/(u*u))+
     & (H3(0,0,0)*(-1+30*NFV*(-2+u)+8*u-196*(u*u)))/(9._dp*(u*u))+
     & (H3(1,0,0)*(-1+30*NFV*(-2+u)+8*u-196*(u*u)))/(9._dp*(u*u))+
     & (H3(1,1,0)*(-11+30*NFV*(-2+u)+16*u+128*(u*u)))/(9._dp*(u*u))+
     & (H3(0,1,0)*(7+30*NFV*(-2+u)-20*u+164*(u*u)))/(9._dp*(u*u))+
     & (H2(1,0)*(71-191*u+171*(u*u)-5*nf*(1-2*u+2*(u*u))+
     & 10*NFV*(2-u+2*(u*u))))/(9._dp*(u*u))+
     & (H2(1,1)*(81-140*u+nf*(-4+8*u-8*(u*u))+123*(u*u)+
     & 10*NFV*(4-11*u+4*(u*u))))/(18._dp*(u*u))+
     & (H1(1)*(819+54/omv+60/omw-1656/u-60/(omw*u)-42*u+
     & (96*u)/omv-(6*v)/u-150/(u-u*v)+54/w-54/(u*w)-
     & (54*u)/w+3132*zeta2-(372*zeta2)/u+192/(omv*omv)-
     & (192*u)/(omv*omv)+1119/(u*u)+(6*v)/(u*u)+54/(w*(u*u))+
     & (150*zeta2)/(u*u)-(4*nf*(19-32*u+29*(u*u)))/(u*u)-
     & (30*NFV*(3*omv*u*(7+3*(u*u))*(w*w)-
     & 6*(u*u)*(3+3*u-7*v+v*v)*(w*w)+
     & omv*omv*(4*(u*u*u*u)+u*u*u*(-4+4*w-9*(w*w))+
     & 2*(2-5*w+(6-48*zeta2)*(w*w))+
     & u*(-4+6*w+(-15+48*zeta2)*(w*w)))))/
     & (omv*omv*(u*u)*(w*w))))/108._dp+
     & (H2(0,1)*(103-329*v+(693-573*v)*(u*u*u*u)-183*(u*u*u*u*u)+
     & 358*(v*v)-2*(u*u*u)*(542-811*v+303*(v*v))-132*(v*v*v)-
     & 10*NFV*(-2+6*v+(-16+9*v)*(u*u*u*u)+2*(u*u*u*u*u)-
     & 6*(v*v)+u*u*u*(38-48*v+9*(v*v))+
     & u*(16-42*v+39*(v*v)-11*(v*v*v))+4*(v*v*v)+
     & u*u*(-38+75*v-42*(v*v)+4*(v*v*v)))-6*nf*(w*w*w)-
     & 2*(u*u)*(-462+927*v-563*(v*v)+108*(v*v*v)+6*nf*(w*w*w))+
     & u*(-453+1134*v-878*(v*v)+188*(v*v*v)+12*nf*(w*w*w))))/
     & (18._dp*(u*u)*(w*w*w))+(H2(0,0)*
     & (164-512*v-2*(-796+615*v)*(u*u*u*u)-402*(u*u*u*u*u)+
     & u*u*u*(-2528+3662*v-1263*(v*v))+541*(v*v)-193*(v*v*v)-
     & 10*NFV*(-2+6*v+(-7+9*v)*(u*u*u*u)+2*(u*u*u*u*u)-
     & 6*(v*v)+u*u*u*(11-21*v+9*(v*v))+
     & u*(7-15*v+12*(v*v)-2*(v*v*v))+4*(v*v*v)+
     & u*u*(-11+21*v-15*(v*v)+4*(v*v*v)))-12*nf*(w*w*w)+
     & u*u*(2052-4146*v+2509*(v*v)-435*(v*v*v)-24*nf*(w*w*w))+
     & u*(-878+2226*v-1787*(v*v)+430*(v*v*v)+24*nf*(w*w*w))))/
     & (18._dp*(u*u)*(w*w*w))+(-12*zeta3*
     & (-11+30*NFV*(-2+u)+16*u+128*(u*u))*(w*w*w)+
     & 6*zeta2*(-129+547*v+(-1411+1191*v)*(u*u*u*u)+
     & 429*(u*u*u*u*u)-779*(v*v)+
     & u*u*u*(1788-3082*v+1059*(v*v))+361*(v*v*v)+
     & 10*NFV*((-25+48*v)*(u*u*u*u)+20*(u*u*u*u*u)+
     & u*u*u*(-25-27*v+48*(v*v))+
     & u*(-23+57*v-21*(v*v)+3*(v*v*v))+
     & 4*(1-3*v+3*(v*v)+3*(v*v*v))+
     & u*u*(49-66*v-3*(v*v)+12*(v*v*v)))-20*nf*(w*w*w)+
     & u*u*(-1188+3138*v-2087*(v*v)+297*(v*v*v)-16*nf*(w*w*w))+
     & u*(511-1794*v+1807*(v*v)-452*(v*v*v)+40*nf*(w*w*w))))/
     & (108._dp*(u*u)*(w*w*w))+((H1(0)*(1-2*u+2*(u*u)))/(18._dp*(u*u))+
     & (H1(1)*(1-2*u+2*(u*u)))/(18._dp*(u*u)))*(G1(2)*G1(2))+
     & (((-57+2*nf)*(1-2*u+2*(u*u)))/(18._dp*(u*u))-
     & (5*H1(1)*(1-2*u+2*(u*u)))/(6._dp*(u*u)))*(H1(0)*H1(0))-
     & (17*(1-2*u+2*(u*u))*(H1(0)*H1(0)*H1(0)))/(18._dp*(u*u))+
     & ((1-2*u+2*(u*u))*(H1(1)*H1(1)))/(6._dp*(u*u))+
     & G1(2)*(-(H1(1)*(1-2*u+2*(u*u)))/(6._dp*(u*u))+
     & H1(0)*(-(1-2*u+2*(u*u))/(6._dp*(u*u))-
     & (H1(1)*(2-4*u+4*(u*u)))/(9._dp*(u*u)))-
     & ((1-2*u+2*(u*u))*(H1(0)*H1(0)))/(9._dp*(u*u))-
     & ((1-2*u+2*(u*u))*(H1(1)*H1(1)))/(9._dp*(u*u)))+
     & H1(0)*(((-27+nf)*H1(1)*(1-2*u+2*(u*u)))/(9._dp*(u*u))+
     & (819+54/omv+60/omw-1656/u-60/(omw*u)-42*u+
     & (96*u)/omv-(6*v)/u-150/(u-u*v)+54/w-54/(u*w)-
     & (54*u)/w+2268*zeta2+(492*zeta2)/u+192/(omv*omv)-
     & (192*u)/(omv*omv)+1119/(u*u)+(6*v)/(u*u)+54/(w*(u*u))-
     & (282*zeta2)/(u*u)-(4*nf*(19-32*u+29*(u*u)))/(u*u)-
     & (30*NFV*(3*omv*u*(7+3*(u*u))*(w*w)-
     & 6*(u*u)*(3+3*u-7*v+v*v)*(w*w)+
     & omv*omv*(4*(u*u*u*u)+u*u*u*(-4+4*w-9*(w*w))+
     & 2*(2-5*w+(6-48*zeta2)*(w*w))+
     & u*(-4+6*w+(-15+48*zeta2)*(w*w)))))/
     & (omv*omv*(u*u)*(w*w)))/108._dp+
     & ((1-2*u+2*(u*u))*(H1(1)*H1(1)))/(6._dp*(u*u)))+
     & ((1-2*u+2*(u*u))*(H1(1)*H1(1)*H1(1)))/(18._dp*(u*u)))+
     & (H1(1)*H1(1)*H1(1)*H1(1))/72._dp
      return
      end
      
      
      function Alpha_4a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-16*pi*G3(0,0,1))/9._dp+(2*pi*G3(0,0,2))/9._dp-16*pi*G3(0,0,3)+
     & (2*pi*G3(0,2,0))/9._dp+(8*pi*G3(0,2,1))/9._dp+(2*pi*G3(0,2,2))/9._dp+
     & (188*pi*G3(0,2,3))/9._dp+(8*pi*G3(0,3,0))/9._dp+(8*pi*G3(0,3,2))/9._dp+
     & (128*pi*G3(0,3,3))/9._dp-(20*pi*G3(1,0,1))/9._dp+(2*pi*G3(2,0,0))/9._dp+
     & (2*pi*G3(2,0,2))/9._dp+(2*pi*G3(2,2,0))/9._dp-(10*pi*G3(2,3,0))/9._dp-
     & (26*pi*G3(2,3,2))/9._dp+(16*pi*G3(3,0,0))/9._dp+(10*pi*G3(3,0,1))/9._dp+
     & (16*pi*G3(3,0,2))/9._dp+(208*pi*G3(3,0,3))/9._dp+(16*pi*G3(3,2,0))/9._dp+
     & (10*pi*G3(3,2,1))/9._dp-(80*pi*G3(3,2,3))/9._dp+(128*pi*G3(3,3,0))/9._dp-
     & (128*pi*G3(3,3,3))/9._dp+G2(0,0)*
     & (((-21+nf)*pi)/3._dp+(16*pi*H1(0))/9._dp-(2*pi*H1(1))/9._dp)-
     & (64*pi*H3(0,0,0))/3._dp+(8*pi*H3(0,0,1))/9._dp+(98*pi*H3(0,1,0))/9._dp+
     & (178*pi*H3(1,0,0))/9._dp-(10*pi*H3(1,0,1))/9._dp+
     & G2(0,1)*(pi*H1(1)*(20._dp/9._dp-2/u+1/(u*u))+
     & pi*H1(0)*(38._dp/9._dp-2/u+1/(u*u))+
     & (pi*(78-20/omw-6*nf+(56+60*NFV)/omv-10/u+(2*nf)/u+
     & 24/(u-u*v)-(120*NFV)/(u-u*v)-(41+60*NFV)/(omv*omv)+
     & (2*(16-45*NFV)*u*v)/(omv*omv*omv)-10/(omw*omw)-
     & 16/(u*u)-nf/(u*u)))/18._dp)+
     & pi*G3(0,1,1)*(-20._dp/9._dp-2/u+1/(u*u))+
     & (pi*G3(1,0,3)*(17+30*NFV*(-2+u)-28*u-160*(u*u)))/(9._dp*(u*u))+
     & (pi*G3(1,2,3)*(17+30*NFV*(-2+u)-28*u-160*(u*u)))/(9._dp*(u*u))+
     & (pi*G3(1,2,1)*(-15+30*NFV*(-2+u)+30*u-32*(u*u)))/(9._dp*(u*u))+
     & (pi*G3(2,2,3)*(-1+10*NFV*(-2+u)+4*u-12*(u*u)))/(3._dp*(u*u))-
     & (2*pi*G3(0,1,0)*(1-2*u+2*(u*u)))/(u*u)+
     & (pi*G3(2,1,0)*(1-2*u+2*(u*u)))/(u*u)-
     & (4*pi*G3(2,1,1)*(1-2*u+2*(u*u)))/(u*u)+
     & (pi*G3(2,1,2)*(1-2*u+2*(u*u)))/(u*u)-
     & (pi*G3(2,2,1)*(3+30*NFV*(-2+u)-6*u+4*(u*u)))/(9._dp*(u*u))+
     & (20*pi*G3(2,3,3)*(-1+2*u+8*(u*u)))/(9._dp*(u*u))-
     & (4*pi*G3(1,1,2)*(9-18*u+13*(u*u)))/(9._dp*(u*u))+
     & (pi*G3(1,0,2)*(-1-30*NFV*(-2+u)+2*u+16*(u*u)))/(9._dp*(u*u))+
     & (pi*G3(1,2,0)*(-1-30*NFV*(-2+u)+2*u+16*(u*u)))/(9._dp*(u*u))+
     & (pi*G3(1,2,2)*(-1-30*NFV*(-2+u)+2*u+16*(u*u)))/(9._dp*(u*u))+
     & (pi*G3(2,0,1)*(11+30*NFV*(-2+u)-22*u+18*(u*u)))/(9._dp*(u*u))-
     & (pi*G3(1,1,0)*(21+30*NFV*(-2+u)-42*u+20*(u*u)))/(9._dp*(u*u))-
     & (pi*G3(0,1,2)*(9-18*u+20*(u*u)))/(9._dp*(u*u))+
     & (pi*G3(2,0,3)*(3-30*NFV*(-2+u)-12*u+98*(u*u)))/(9._dp*(u*u))+
     & (pi*G3(1,1,1)*(57+30*NFV*(-2+u)-114*u+112*(u*u)))/(9._dp*(u*u))+
     & (pi*H2(1,1)*(omu*(-2+(-63+4*nf)*u)+40*NFV*(3-3*u+u*u)))/
     & (18._dp*omu*u)+G2(2,0)*((-2*pi*H1(1))/9._dp-
     & (pi*H1(0)*(10+30*NFV*(-2+u)-20*u+21*(u*u)))/(9._dp*(u*u))+
     & (pi*(omu*(-2+5*(-24+nf)*u)+40*NFV*(3-3*u+u*u)))/(18._dp*omu*u))
     & +G2(0,2)*((-2*pi*H1(1))/9._dp+(pi*H1(0)*(-3+6*u+53*(u*u)))/(3._dp*(u*u))+
     & (pi*(omu*(-2+5*(-24+nf)*u)+40*NFV*(3-3*u+u*u)))/(18._dp*omu*u))
     & +G2(2,2)*((pi*H1(0)*(30*NFV*(-2+u)-(1-4*u)*(1-4*u)))/(9._dp*(u*u))+
     & (pi*(omu*(-2+(-63+4*nf)*u)+40*NFV*(3-3*u+u*u)))/(18._dp*omu*u))
     & +G2(2,3)*(12*pi*H1(0)+(26*pi*H1(1))/9._dp-
     & (pi*(36*nf*u*(omv*omv*omv*omv)-
     & 2*(omv*omv*omv)*(14+60*NFV*(-2+v)-17*v+18*nf*(u*u))+
     & 2*(16-45*NFV)*v*(u*u*u)+
     & u*(omv*omv)*(-542+1288*v+60*NFV*(-4+3*v)-795*(v*v))+
     & omv*(u*u)*(570+2*(-679+15*NFV)*v+795*(v*v))))/
     & (18._dp*u*w*(omv*omv*omv)))+
     & G2(2,1)*((pi*H1(0)*(1-2*u+2*(u*u)))/(u*u)-
     & (pi*H1(1)*(1-2*u+2*(u*u)))/(u*u)+
     & (pi*(153-20/omw-8*nf+(8*nf)/u-24/(u-u*v)+41/(omv*omv)-
     & 10/(omw*omw)+87/(u*u)-(4*nf)/(u*u)+(20*NFV)/(u*u)-
     & (20*v)/(u*u)-(v*v)/(u*u)-
     & (2*(75+28*u+NFV*(5-65*v)-76*v+v*v))/(omv*u)+
     & (-32*u*v+10*NFV*(2+9*u*v-2*(v*v*v)))/(omv*omv*omv)))/18._dp)
     & +(pi*H2(1,0)*((232-60*NFV)/omv+(-31+60*NFV+32*u)/(omv*omv)+
     & (2*u*(-16+45*NFV*v))/(omv*omv*omv)+
     & 2*(-168-10/omw+3*nf-20*NFV+(20*NFV)/omu-6/u+
     & (60*NFV)/u-3/(u-u*v)+(60*NFV)/(u-u*v)-164/w+1/(u*w)+
     & (163*u)/w-5/(omw*omw)+40/(w*w)-(80*u)/(w*w)+
     & (40*(u*u))/(w*w))))/18._dp-
     & (pi*H1(1)*(u*w*(-76*nf+30*zeta2)*(omv*omv*omv)-
     & 3*omv*w*(u*u)*(263-147*v+14*(v*v))+
     & omv*omv*(w*(u*(279-393*v)+(76*nf-30*zeta2)*(u*u)+
     & 6*(62-72*v+v*v))+
     & 30*NFV*(-4*(5-10*v+4*(v*v))+u*(58-60*v+6*(v*v))+
     & u*u*(-38+7*v+9*(v*v))))+
     & 6*(u*u*u)*(w*(23+2*v+7*(v*v))+
     & 15*NFV*(3*u*(2-v+v*v)+2*(-3+5*v-5*(v*v)+3*(v*v*v))))))/
     & (108._dp*u*(omv*omv)*(w*w))+
     & (pi*(-16626+1008/omv-240/omw-1215/u-(1320*u)/omw+
     & 4*nf*(288-24/omv+19/u+27/w+(27*(1+omw)*u)/(omw*w))-
     & 1107/w-(1422*u)/w-2970*zeta2+(1248*zeta2)/omv-
     & (222*zeta2)/u-(168*zeta2)/(u-u*v)-(2232*zeta2)/w+
     & (24*zeta2)/(u*w)+(2208*u*zeta2)/w+6972*zeta3+
     & (12*zeta2)/(omv*omv)+(384*u*zeta2)/(omv*omv)-
     & (384*u*zeta2)/(omv*omv*omv)+(240*u)/(omw*omw)+
     & (534*zeta2)/(w*w)-(1068*u*zeta2)/(w*w)+(534*zeta2*(u*u))/(w*w)+
     & 10*NFV*((-3*(9*(-2+v)*(u*u)-2*(5-11*v+6*(v*v))+
     & u*(28-37*v+9*(v*v))))/(u*(w*w))+
     & 6*zeta2*(-24/u+12*(1-1/omu-1/omv+2/(u-u*v)+
     & 1/(omv*omv))+
     & u*(-18/(omv*omv)+18/(omv*omv*omv)+
     & (2-3*w+3*(w*w))/(w*w*w))-((1+3*v)*(u*u))/(w*w*w)-
     & (u*u*u)/(w*w*w)))))/108._dp+
     & (pi*H2(0,1)*(-123+24/omv+6*nf-20/u+22/(u-u*v)+44/w-2/(u*w)-
     & (42*u)/w-33/(omv*omv)-(32*u)/(omv*omv)+
     & (32*u)/(omv*omv*omv)-9/(w*w)+(18*u)/(w*w)-(9*(u*u))/(w*w)+
     & 10*NFV*(-4+4/omu+6/omv+12/u-12/(u-u*v)-6/(omv*omv)+
     & u*(9/(omv*omv)-9/(omv*omv*omv)+
     & (-2+3*w-3*(w*w))/(w*w*w))+((1+3*v)*(u*u))/(w*w*w)+
     & (u*u*u)/(w*w*w))))/18._dp+
     & (pi*H2(0,0)*(1710-264/omv-64*nf+6/u-10/(u-u*v)-288/w+
     & 4/(u*w)+(284*u)/w+39/(omv*omv)-(32*u)/(omv*omv)+
     & (32*u)/(omv*omv*omv)+71/(w*w)-(142*u)/(w*w)+
     & (71*(u*u))/(w*w)+10*NFV*
     & (-4+4/omu+6/omv+12/u-12/(u-u*v)-6/(omv*omv)+
     & u*(9/(omv*omv)-9/(omv*omv*omv)+
     & (-2+3*w-3*(w*w))/(w*w*w))+((1+3*v)*(u*u))/(w*w*w)+
     & (u*u*u)/(w*w*w))))/18._dp+
     & G2(0,3)*((56*pi*H1(0))/9._dp-(8*pi*H1(1))/9._dp+
     & (pi*(8*nf*u*(omv*omv*omv*omv*omv*omv)+
     & omv*omv*omv*omv*omv*(40-34*v+120*NFV*v-24*nf*(u*u))+
     & 2*(16-45*NFV)*v*(u*u*u*u*u)+
     & 6*u*(omv*omv*omv*omv)*
     & (-66+163*v-50*NFV*v+4*nf*(u*u)-119*(v*v))+
     & omv*(u*u*u*u)*(276-834*v+501*(v*v)+
     & 10*NFV*(2+17*v+2*(v*v)))-
     & omv*omv*(u*u*u)*(868-2482*v+1645*(v*v)+
     & 10*NFV*(4+14*v+3*(v*v)))+
     & 2*(omv*omv*omv)*(u*u)*
     & (474-1312*v-4*nf*(u*u)+929*(v*v)+
     & 5*NFV*(2+24*v+3*(v*v)))))/(18._dp*u*(omv*omv*omv)*(w*w*w)))
     & +G2(3,0)*((16*pi*H1(0))/3._dp-(16*pi*H1(1))/9._dp-
     & (pi*(-2*(-684+12*omv*nf+NFV*(20-30*v)+685*v)*(u*u*u*u)+
     & 4*(-114+2*nf+5*NFV)*(u*u*u*u*u)+
     & u*u*u*(-1368+2720*v+24*nf*(omv*omv)-1373*(v*v)+
     & 10*NFV*(2-9*v+6*(v*v)))+
     & v*v*(10+4*(-7+5*NFV)*v+17*(v*v)+v*v*v)+
     & u*v*(-20+10*(1+3*NFV)*v-2*(-9+5*NFV)*(v*v)+v*v*v)+
     & u*u*(456+10*(-133+3*NFV)*v-8*nf*(omv*omv*omv)+
     & (1353-60*NFV)*(v*v)+(-459+20*NFV)*(v*v*v))))/
     & (18._dp*(u*u)*(w*w*w)))+G2(3,2)*
     & ((-98*pi*H1(0))/9._dp-(pi*(10*NFV*
     & ((-4+6*v)*(u*u*u*u)+2*(u*u*u*u*u)-u*(-3+v)*(v*v)+
     & v*(u*u)*(3-6*v+2*(v*v))+u*u*u*(2-9*v+6*(v*v))+
     & 2*(v*v*v))+v*(-((16+5*v)*(u*u*u))-2*(u*u*u*u)+
     & u*u*(38-15*v-3*(v*v))+v*(10-28*v+17*(v*v)+v*v*v)+
     & u*(-20+10*v+18*(v*v)+v*v*v))))/(18._dp*(u*u)*(w*w*w)))+
     & G2(1,2)*((pi*H1(0)*(15+30*NFV*(-2+u)-24*u-160*(u*u)))/(9._dp*(u*u))+
     & (pi*H1(1)*(1+30*NFV*(-2+u)-2*u-16*(u*u)))/(9._dp*(u*u))-
     & (pi*(w*(-75+170*v+(542-8*omv*nf-348*v)*(u*u*u)+
     & 2*(-93+2*nf)*(u*u*u*u)+
     & u*u*(-601+728*v+4*nf*(omv*omv)-153*(v*v))-104*(v*v)+
     & 10*u*(32-55*v+21*(v*v)))+
     & 10*NFV*(-2+6*v-15*omv*(u*u*u*u)+6*(u*u*u*u*u)-6*(v*v)+
     & 3*(u*u*u)*(5-9*v+5*(v*v))+
     & u*(7-15*v+12*(v*v)-2*(v*v*v))+4*(v*v*v)+
     & u*u*(-11+21*v-15*(v*v)+4*(v*v*v)))))/(18._dp*(u*u)*(w*w*w)))
     & +G2(3,3)*((-128*pi*H1(0))/9._dp-
     & (pi*(-2*(-718+48*omv*nf+NFV*(20-30*v)+737*v)*(u*u*u*u)+
     & 4*(-123+8*nf+5*NFV)*(u*u*u*u*u)+
     & u*u*u*(-1356+2764*v+96*nf*(omv*omv)-1471*(v*v)+
     & 10*NFV*(2-9*v+6*(v*v)))+
     & u*u*(372+2*(-533+15*NFV)*v-32*nf*(omv*omv*omv)+
     & (1203-60*NFV)*(v*v)+(-489+20*NFV)*(v*v*v))-
     & v*(-40+108*v-2*(43+10*NFV)*(v*v)+17*(v*v*v)+v*v*v*v)-
     & u*(-40+264*v-2*(188+15*NFV)*(v*v)+2*(71+5*NFV)*(v*v*v)+
     & v*v*v*v)))/(18._dp*(u*u)*(w*w*w)))+
     & G2(1,0)*((pi*H1(1)*(1+30*NFV*(-2+u)-2*u-16*(u*u)))/(9._dp*(u*u))+
     & (2*pi*H1(0)*(5+15*NFV*(-2+u)-10*u+u*u))/(9._dp*(u*u))-
     & (pi*(-63+209*v+(-771+579*v)*(u*u*u*u)+201*(u*u*u*u*u)-
     & 238*(v*v)+2*(u*u*u)*(585-851*v+273*(v*v))+92*(v*v*v)+
     & 10*NFV*(2-6*v+(8+3*v)*(u*u*u*u)+2*(u*u*u*u*u)+6*(v*v)+
     & u*u*u*(-34+30*v+3*(v*v))+u*u*(38-69*v+30*(v*v))+
     & u*(-16+42*v-33*(v*v)+9*(v*v*v)))+nf*(w*w*w)+
     & u*(357-962*v+822*(v*v)-208*(v*v*v)-2*nf*(w*w*w))+
     & 2*(u*u)*(-447+938*v-565*(v*v)+84*(v*v*v)+3*nf*(w*w*w))))/
     & (18._dp*(u*u)*(w*w*w)))+G2(1,1)*
     & ((-20*pi*H1(0))/9._dp+(4*pi*H1(1)*(9-18*u+13*(u*u)))/(9._dp*(u*u))+
     & (pi*(-39+97*v+(-715+453*v)*(u*u*u*u)+159*(u*u*u*u*u)-
     & 68*(v*v)+2*(u*u*u)*(615-788*v+210*(v*v))+10*(v*v*v)+
     & 10*NFV*(-2+6*v+(2+3*v)*(u*u*u*u)+2*(u*u*u*u*u)-6*(v*v)+
     & u*u*u*(-16+18*v+3*(v*v))+u*u*(16-39*v+24*(v*v))+
     & u*(-2+12*v-21*(v*v)+9*(v*v*v)))+4*nf*(w*w*w)+
     & u*(355-864*v+694*(v*v)-194*(v*v*v)-8*nf*(w*w*w))+
     & 2*(u*u)*(-495+945*v-523*(v*v)+63*(v*v*v)+4*nf*(w*w*w))))/
     & (18._dp*(u*u)*(w*w*w)))+((pi*(9-(-138+nf)*u))/(18._dp*u)-
     & (pi*G1(2))/2._dp-4*pi*G1(3)+(3*pi*H1(0))/2._dp+(pi*H1(1))/2._dp+
     & (pi*G1(1)*(2-2/u+1/(u*u)))/2._dp)*(G1(0)*G1(0))-
     & (pi*(G1(0)*G1(0)*G1(0)))/2._dp+(-(pi+15*pi*u)/(18._dp*u)+(4*pi*G1(3))/9._dp-
     & (pi*H1(0))/6._dp-(pi*H1(1))/6._dp)*(G1(2)*G1(2))+(pi*(G1(2)*G1(2)*G1(2)))/18._dp+
     & ((5*pi*H1(1))/6._dp+(pi*(17-16*v-v*v+
     & u*(34*nf*(omv*omv)-9*(37-90*v+61*(v*v)))))/
     & (18._dp*u*(omv*omv)))*(H1(0)*H1(0))+(17*pi*(H1(0)*H1(0)*H1(0)))/18._dp-
     & ((pi+15*pi*u)*(H1(1)*H1(1)))/(18._dp*u)+
     & H1(0)*(-(pi*(20+(-57+2*nf)*u)*H1(1))/(18._dp*u)+8*pi*H2(0,0)-
     & 10*pi*H2(1,0)+(pi*(3*(omv*omv*omv*omv)*
     & (8*nf*v+90*(-1+4*v)*zeta2*(u*u))+
     & 3*omv*(u*u*u)*(-48-2749*v+270*zeta2-1080*v*zeta2+
     & 4897*(v*v)+1350*zeta2*(v*v)-898*(v*v*v)-
     & 540*zeta2*(v*v*v)+4*nf*(-4+83*v-143*(v*v)+38*(v*v*v))+
     & 56*(v*v*v*v)-10*NFV*
     & (-48+120*v-38*(v*v)-37*(v*v*v)+36*(v*v*v*v)))+
     & u*u*u*u*(-4*nf*(-105+436*v-431*(v*v)+
     & u*(91-134*v+19*(v*v))+76*(v*v*v))+
     & 3*(-1549+6140*v-5593*(v*v)-2*(u*u)*(23+2*v+7*(v*v))+
     & u*(1363-1948*v+177*(v*v)-56*(v*v*v))+686*(v*v*v)-
     & 84*(v*v*v*v)+
     & 10*NFV*(-6-62*v+139*(v*v)+9*(u*u)*(2-v+v*v)-
     & 107*(v*v*v)+u*(-36+93*v-57*(v*v)+36*(v*v*v))+
     & 54*(v*v*v*v))))+
     & omv*omv*(270*(-3+4*v)*zeta2*(u*u*u*u)+
     & 270*zeta2*(u*u*u*u*u)-
     & 6*v*(52+8*(-22+15*NFV)*v+(206-220*NFV)*(v*v)+
     & (-83+80*NFV)*(v*v*v)+v*v*v*v)+
     & u*(-4*nf*(-6+20*v+12*(v*v)-45*(v*v*v)+19*(v*v*v*v))+
     & 3*(-84+v*(716-480*NFV-90*zeta2)+
     & 5*(-212+296*NFV+54*zeta2)*(v*v)+
     & (321-920*NFV-270*zeta2)*(v*v*v)+
     & 5*(25+12*NFV+18*zeta2)*(v*v*v*v)))-
     & u*u*(4*nf*(8+30*v-177*(v*v)+76*(v*v*v))-
     & 3*(364-696*v-1093*(v*v)+548*(v*v*v)-14*(v*v*v*v)+
     & 10*NFV*(-24+152*v-142*(v*v)+10*(v*v*v)+
     & 9*(v*v*v*v)))))))/(108._dp*omw*u*(omv*omv)*(w*w*w))-
     & (pi*(H1(1)*H1(1)))/6._dp)+G1(0)*
     & (G1(3)*((-4*(-30+nf)*pi)/9._dp+8*pi*H1(0))+
     & G1(2)*(-((-27+nf)*pi)/18._dp+(8*pi*H1(0))/9._dp-(pi*H1(1))/9._dp)+
     & (40*pi*H2(0,0))/3._dp-(16*pi*H2(0,1))/9._dp-(2*pi*H2(1,0))/3._dp+
     & (2*pi*H2(1,1))/9._dp-(pi*H1(1)*
     & (omu*(-2+(-93+4*nf)*u)+40*NFV*(3-3*u+u*u)))/(18._dp*omu*u)
     & +G1(1)*(((-30+nf)*pi*(1-2*u+2*(u*u)))/(18._dp*(u*u))-
     & (pi*H1(0)*(1-2*u+2*(u*u)))/(u*u))+
     & (pi*(u*w*(-76*nf+270*zeta2)*(omv*omv*omv)-
     & 3*omv*w*(u*u)*(263-147*v+14*(v*v))+
     & omv*omv*(w*(u*(669-393*v)+(76*nf-270*zeta2)*(u*u)+
     & 6*(-3-7*v+v*v))+
     & 30*NFV*(-4+8*v+u*(26-28*v+6*(v*v))+
     & u*u*(-22+7*v+9*(v*v))))+
     & 6*(u*u*u)*(w*(23+2*v+7*(v*v))+
     & 15*NFV*(3*u*(2-v+v*v)+2*(-3+5*v-5*(v*v)+3*(v*v*v))))
     & ))/(108._dp*u*(omv*omv)*(w*w))+
     & H1(0)*((pi*H1(1))/9._dp+(pi*(-366-24/omv-4*nf+2/u-22/(u-u*v)-
     & 44/w+2/(u*w)+(42*u)/w+33/(omv*omv)+
     & (32*u)/(omv*omv)-(32*u)/(omv*omv*omv)+9/(w*w)-
     & (18*u)/(w*w)+(9*(u*u))/(w*w)+
     & 10*NFV*(4-4/omu-6/omv-12/u+12/(u-u*v)+
     & 6/(omv*omv)+
     & u*(-9/(omv*omv)+9/(omv*omv*omv)+
     & (2-3*w+3*(w*w))/(w*w*w))-
     & ((1+3*v)*(u*u))/(w*w*w)-(u*u*u)/(w*w*w))))/18._dp)+
     & (pi*(G1(2)*G1(2)))/18._dp-(179*pi*(H1(0)*H1(0)))/18._dp+(pi*(H1(1)*H1(1)))/18._dp)
     & +G1(2)*(G1(3)*((-4*pi)/3._dp-(8*pi*H1(0))/9._dp-(8*pi*H1(1))/9._dp)+
     & H1(0)*((pi*H1(1))/3._dp+(pi*(861+20/omw-36*nf+40*NFV-
     & (40*NFV)/omu+(4*(-58+15*NFV))/omv+56/u-(8*nf)/u-
     & (120*NFV)/u-8/(u-u*v)-(120*NFV)/(u-u*v)-4/w+
     & 4/(u*w)+(31-60*NFV-32*u)/(omv*omv)+
     & (u*(32-90*NFV*v))/(omv*omv*omv)+10/(omw*omw)-
     & 12/(u*u)+(4*nf)/(u*u)))/18._dp)+
     & (pi*H2(0,0)*(-3+30*NFV*(-2+u)+12*u-88*(u*u)))/(9._dp*(u*u))+
     & (2*pi*H2(0,1)*(5+15*NFV*(-2+u)-10*u+14*(u*u)))/(9._dp*(u*u))-
     & (2*pi*H2(1,0)*(1+15*NFV*(-2+u)-2*u+41*(u*u)))/(9._dp*(u*u))-
     & (pi*H1(1)*(omu*(-4+(-93+4*nf)*u)+40*NFV*(3-3*u+u*u)))/
     & (18._dp*omu*u)-(pi*(-6*(u*u*u)*
     & (15*NFV*u*(3*u*(2-v+v*v)+
     & 2*(-3+5*v-5*(v*v)+3*(v*v*v)))-32*(w*w))-
     & 6*omv*(33+16*u)*(u*u)*(w*w)+
     & omv*omv*(w*(144*w*zeta2-6*u*(-9+w*(71-v+48*zeta2))+
     & (-108+w*(-399+76*nf+1170*zeta2))*(u*u)+
     & 6*(9+7*w)*(u*u*u))-
     & 30*NFV*(u*u*(58-60*v+6*(v*v))+
     & u*u*u*(-38+7*v+9*(v*v))+48*zeta2*(w*w)-
     & 4*u*(5-10*v+4*(v*v)+6*zeta2*(w*w))))))/
     & (108._dp*(omv*omv)*(u*u)*(w*w))+
     & pi*(7._dp/6._dp-2/u+1/(u*u))*(H1(0)*H1(0))+(pi*(H1(1)*H1(1)))/6._dp)+
     & G1(3)*((-64*pi*H2(0,0))/3._dp+(8*pi*H2(0,1))/9._dp+(98*pi*H2(1,0))/9._dp+
     & H1(0)*((8*pi*H1(1))/9._dp+(pi*
     & ((72-74*v+20*NFV*(-2+3*v))*(u*u*u*u)+
     & 4*(-6+5*NFV)*(u*u*u*u*u)+
     & u*u*u*(-72+128*v-77*(v*v)+10*NFV*(2-9*v+6*(v*v)))+
     & v*v*(10+4*(-7+5*NFV)*v+17*(v*v)+v*v*v)+
     & u*v*(-20+10*(1+3*NFV)*v-2*(-9+5*NFV)*(v*v)+v*v*v)+
     & u*u*(24+(-34+30*NFV)*v+(57-60*NFV)*(v*v)+
     & (-27+20*NFV)*(v*v*v))))/(18._dp*(u*u)*(w*w*w)))+
     & (pi*H1(1)*((72-74*v+20*NFV*(-2+3*v))*(u*u*u*u)+
     & 4*(-6+5*NFV)*(u*u*u*u*u)+
     & u*u*u*(-72+128*v-77*(v*v)+10*NFV*(2-9*v+6*(v*v)))+
     & v*v*(10+4*(-7+5*NFV)*v+17*(v*v)+v*v*v)+
     & u*v*(-20+10*(1+3*NFV)*v-2*(-9+5*NFV)*(v*v)+v*v*v)+
     & u*u*(24+(-34+30*NFV)*v+(57-60*NFV)*(v*v)+
     & (-27+20*NFV)*(v*v*v))))/(18._dp*(u*u)*(w*w*w))+
     & (pi*(2160*u*zeta2*(omv*omv*omv*omv*omv)+
     & 3*(u*u*u*u)*(-1049+2106*v-1079*(v*v)-
     & 2*u*(23+2*v+7*(v*v))-42*(v*v*v)+
     & 30*NFV*(-12+25*v-16*(v*v)+3*u*(2-v+v*v)+9*(v*v*v)))
     & -3*omv*(u*u*u)*(-5603+8720*v-2160*zeta2+4320*v*zeta2-
     & 3157*(v*v)-2160*zeta2*(v*v)+4*nf*(125-196*v+71*(v*v))-
     & 42*(v*v*v)+10*NFV*(6+32*v-32*(v*v)+27*(v*v*v)))+
     & omv*omv*(4*(71*nf-540*zeta2)*(u*u*u*u)-
     & 6*v*(-6+22*v-17*(v*v)+20*NFV*(6-13*v+6*(v*v))+
     & v*v*v)+3*(u*u)*
     & (-7032+10492*v-2160*zeta2+4320*v*zeta2-3039*(v*v)-
     & 2160*zeta2*(v*v)+4*nf*(152-232*v+71*(v*v))-
     & 14*(v*v*v)+10*NFV*(48-74*v+4*(v*v)+9*(v*v*v)))+
     & u*(4*nf*(-152+402*v-321*(v*v)+71*(v*v*v))+
     & 3*(2524-6456*v+4923*(v*v)-973*(v*v*v)+
     & 20*NFV*(-12+56*v-46*(v*v)+3*(v*v*v)))))))/
     & (108._dp*u*(omv*omv)*(w*w*w))+(4*pi*(H1(0)*H1(0)))/9._dp+
     & (4*pi*(H1(1)*H1(1)))/9._dp)+G1(1)*
     & ((-2*pi*H2(0,1)*(5+15*NFV*(-2+u)-10*u+u*u))/(9._dp*(u*u))+
     & (pi*H2(1,1)*(-1-30*NFV*(-2+u)+2*u+16*(u*u)))/(9._dp*(u*u))+
     & (2*pi*H2(1,0)*(-3-15*NFV*(-2+u)+3*u+89*(u*u)))/(9._dp*(u*u))+
     & (pi*H2(0,0)*(3-30*NFV*(-2+u)-12*u+196*(u*u)))/(9._dp*(u*u))+
     & G1(2)*((pi*(1-2*u+2*(u*u)))/(6._dp*(u*u))+
     & (pi*H1(0)*(1-2*u+2*(u*u)))/(9._dp*(u*u))+
     & (pi*H1(1)*(1-2*u+2*(u*u)))/(9._dp*(u*u)))+
     & (pi*(-819-54/omv-60/omw+1656/u+60/(omw*u)+42*u-
     & (96*u)/omv+(6*v)/u+150/(u-u*v)-54/w+54/(u*w)+
     & (54*u)/w+3300*zeta2+(300*zeta2)/u-192/(omv*omv)+
     & (192*u)/(omv*omv)-1119/(u*u)-(6*v)/(u*u)-54/(w*(u*u))-
     & (222*zeta2)/(u*u)+(4*nf*(19-32*u+29*(u*u)))/(u*u)+
     & (30*NFV*(24*(-2+u)*zeta2+
     & (omv*omv*(-6*(u*u)*(4-2*v+v*v)-
     & 3*u*(2-9*v+5*(v*v))+2*(3-7*v+6*(v*v)))+
     & u*u*u*(24-70*v+44*(v*v)-9*(u*u)*(2-v+v*v)+
     & 11*(v*v*v)-6*u*(-3+2*v-5*(v*v)+3*(v*v*v))-
     & 9*(v*v*v*v)))/(omv*omv*(w*w))))/(u*u)))/108._dp+
     & H1(0)*((-10*pi*H1(1)*(1-2*u+2*(u*u)))/(9._dp*(u*u))-
     & (pi*(20-80*v-2*(-114+3*omv*nf+99*v+10*NFV*(-2+3*v))*
     & (u*u*u*u)+2*(-33+nf-10*NFV)*(u*u*u*u*u)+109*(v*v)+
     & 2*(u*u*u)*(3*nf*(omv*omv)-5*NFV*(2-9*v+6*(v*v))-
     & 11*(14-24*v+9*(v*v)))-(49+20*NFV)*(v*v*v)+
     & u*(-90+292*v-(283+30*NFV)*(v*v)+
     & 2*(36+5*NFV)*(v*v*v))-
     & 2*(u*u)*(-108+(271+15*NFV)*v+nf*(omv*omv*omv)-
     & 6*(31+5*NFV)*(v*v)+(33+10*NFV)*(v*v*v))))/
     & (9._dp*(u*u)*(w*w*w)))+
     & (pi*H1(1)*(w*(-78+176*v-8*(-70+omv*nf+45*v)*(u*u*u)+
     & 4*(-48+nf)*(u*u*u*u)+
     & u*u*(-622+752*v+4*nf*(omv*omv)-159*(v*v))-107*(v*v)+
     & 4*u*(83-142*v+54*(v*v)))+
     & 10*NFV*(-2+6*v-15*omv*(u*u*u*u)+6*(u*u*u*u*u)-6*(v*v)+
     & 3*(u*u*u)*(5-9*v+5*(v*v))+
     & u*(7-15*v+12*(v*v)-2*(v*v*v))+4*(v*v*v)+
     & u*u*(-11+21*v-15*(v*v)+4*(v*v*v)))))/(18._dp*(u*u)*(w*w*w))
     & -(pi*(1-2*u+2*(u*u))*(G1(2)*G1(2)))/(18._dp*(u*u))-
     & (pi*(1-2*u+2*(u*u))*(H1(0)*H1(0)))/(18._dp*(u*u))-
     & (pi*(1-2*u+2*(u*u))*(H1(1)*H1(1)))/(18._dp*(u*u)))-
     & (pi*(H1(1)*H1(1)*H1(1)))/18._dp
      return
      end
      
      
      function Beta_4a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=1
      return
      end
      
      
      function Beta_4a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function Beta_4a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=-8+1/(3-3*u)+(27*zeta2)/2._dp+G2(0,2)/3._dp+G2(2,0)/3._dp+(8*G2(3,0))/3._dp-
     & (8*G1(3)*H1(0))/3._dp+G1(0)*((17*H1(0))/3._dp-H1(1)/3._dp)-(8*H2(0,0))/3._dp+
     & (10*H2(1,0))/3._dp+((omv+u*(-2+u+2*v-2*u*v))*H1(1))/
     & (3._dp*u*(omu*omu))+G1(2)*(-H1(1)/3._dp-
     & (omv+u*(-2+u+2*v-2*u*v))/(3._dp*u*(omu*omu))-
     & ((omv+u*(-2+3*u+2*v))*H1(0))/(3._dp*(u*u)))+
     & G1(1)*(((omv+2*u*(-1+u+v))*H1(0))/(3._dp*(u*u))+
     & ((omv+2*u*(-1+u+v))*H1(1))/(3._dp*(u*u)))-
     & ((omv+2*u*(-1+u+v))*G2(1,2))/(3._dp*(u*u))+
     & H1(0)*(H1(1)/3._dp+(-16+4/omv-(v*(1-2*u+2*(u*u)))/(u*(omu*omu)))/
     & 3._dp)-(3*(G1(0)*G1(0)))/2._dp+(G1(2)*G1(2))/6._dp-(17*(H1(0)*H1(0)))/6._dp+
     & (H1(1)*H1(1))/6._dp
      return
      end
      
      
      function Beta_4a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(pi*G1(0))/3._dp+(pi*G1(2))/3._dp+(8*pi*G1(3))/3._dp-(pi*H1(0))/3._dp-(pi*H1(1))/3._dp-
     & (pi*(omv+2*u*(-1+u+v))*G1(1))/(3._dp*(u*u))-
     & (pi*(omv+u*(2*(5+v)-u*(23+2*v)+12*(u*u))))/(3._dp*u*(omu*omu))
      return
      end
      
      
      function Beta_4a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=-41671._dp/162._dp-20/(9._dp*omw)+104/(3-3*u)+(359*zeta3)/3._dp-
     & (10*NFV*zeta3)/3._dp-(40*zeta3)/(9._dp*u)+(40*NFV*zeta3)/(3._dp*u)-
     & (5*NFV*u*zeta3)/omv+(313*zeta3)/(18-18*v)+(16*u*zeta3)/(9-9*v)+
     & (10*NFV*zeta3)/(3-3*v)-(184*v*zeta3)/(3._dp*omu)+
     & (10*NFV*v*zeta3)/(3-3*u)-(11*v*zeta3)/(9._dp*u)-
     & (20*NFV*v*zeta3)/(3._dp*u)+(2*zeta3)/(9._dp*w)-(2*u*zeta3)/(9._dp*w)+
     & (21293*zeta4)/144._dp-(16*G4(0,0,1,2))/9._dp+(2*G4(0,0,2,2))/9._dp-
     & 16*G4(0,0,3,0)+(2*G4(0,2,0,2))/9._dp+(8*G4(0,2,1,2))/9._dp+
     & (2*G4(0,2,2,0))/9._dp+(188*G4(0,2,3,0))/9._dp+(8*G4(0,3,0,2))/9._dp+
     & (8*G4(0,3,2,0))/9._dp+(128*G4(0,3,3,0))/9._dp-(20*G4(1,0,1,2))/9._dp+
     & (2*G4(2,0,0,2))/9._dp+(2*G4(2,0,2,0))/9._dp+(2*G4(2,2,0,0))/9._dp+
     & (16*G4(2,3,0,0))/9._dp-(26*G4(2,3,0,2))/9._dp-(26*G4(2,3,2,0))/9._dp+
     & (16*G4(3,0,0,2))/9._dp+(10*G4(3,0,1,2))/9._dp+(16*G4(3,0,2,0))/9._dp+
     & (208*G4(3,0,3,0))/9._dp+(16*G4(3,2,0,0))/9._dp+(10*G4(3,2,1,2))/9._dp-
     & (80*G4(3,2,3,0))/9._dp+(128*G4(3,3,0,0))/9._dp-(128*G4(3,3,3,0))/9._dp+
     & 16*G3(0,0,3)*H1(0)-(188*G3(0,2,3)*H1(0))/9._dp-(8*G3(0,3,2)*H1(0))/9._dp-
     & (128*G3(0,3,3)*H1(0))/9._dp+(26*G3(2,3,2)*H1(0))/9._dp-(208*G3(3,0,3)*H1(0))/9._dp+
     & (80*G3(3,2,3)*H1(0))/9._dp+(128*G3(3,3,3)*H1(0))/9._dp+
     & G3(3,0,0)*((8*(-21+nf))/3._dp+(128*H1(0))/9._dp-(16*H1(1))/9._dp)+
     & G3(3,0,1)*((-10*H1(0))/9._dp-(10*H1(1))/9._dp)+
     & G3(3,2,1)*((-10*H1(0))/9._dp-(10*H1(1))/9._dp)+
     & G3(0,2,1)*((-8*H1(0))/9._dp-(8*H1(1))/9._dp)+
     & G3(0,0,2)*(-7+nf/3._dp-(2*H1(1))/9._dp)+
     & G3(2,0,0)*(-7+nf/3._dp+(16*H1(0))/9._dp-(2*H1(1))/9._dp)+
     & G3(0,0,1)*((16*H1(0))/9._dp+(16*H1(1))/9._dp)+
     & G3(1,0,1)*((20*H1(0))/9._dp+(20*H1(1))/9._dp)+(64*H4(0,0,0,0))/3._dp-
     & (80*H4(0,0,1,0))/9._dp-(80*H4(0,1,0,0))/9._dp-(10*H4(0,1,0,1))/9._dp+
     & 10*H4(0,1,1,0)-10*H4(1,0,0,0)-(10*H4(1,0,0,1))/9._dp+(260*H4(1,0,1,0))/9._dp+
     & (70*H4(1,1,0,0))/9._dp-(20*H4(1,1,0,1))/9._dp-(110*H4(1,1,1,0))/9._dp+
     & (595*v*zeta3)/(3._dp*(omu*omu))-(10*NFV*v*zeta3)/(3._dp*(omu*omu))-
     & (281*v*zeta3)/(omu*omu*omu)+
     & (281*v*zeta3)/(2._dp*(omu*omu*omu*omu))-(16*u*zeta3)/(9._dp*(omv*omv))+
     & (5*NFV*u*zeta3)/(omv*omv)+
     & G2(0,0)*(80-(41*nf)/6._dp-(2*zeta2)/3._dp+(8*(-21+nf)*H1(0))/3._dp+
     & (7-nf/3._dp)*H1(1)+(128*H2(0,0))/9._dp-(16*H2(0,1))/9._dp-(16*H2(1,0))/9._dp+
     & (2*H2(1,1))/9._dp+(5*(nf*nf))/36._dp)+
     & G3(0,1,1)*(H1(0)*(20._dp/9._dp+(omv*(-1+2*u))/(u*u))+
     & H1(1)*(20._dp/9._dp+(omv*(-1+2*u))/(u*u)))+
     & G3(0,2,0)*(-7+nf/3._dp-(2*H1(1))/9._dp+
     & H1(0)*(160._dp/9._dp+(2*omv*(-1+2*u))/(u*u)))+
     & G4(0,1,2,2)*(16._dp/9._dp-(2*omv)/u+(1-v)/(u*u))+
     & G3(2,1,0)*(-(((omv+2*u*(-1+u+v))*H1(0))/(u*u))-
     & ((omv+2*u*(-1+u+v))*H1(1))/(u*u))+
     & G3(1,1,1)*((H1(0)*(omv*(-30*NFV*(-2+u)+57*(-1+2*u))-112*(u*u)))/
     & (9._dp*(u*u))+(H1(1)*(omv*(-30*NFV*(-2+u)+57*(-1+2*u))-
     & 112*(u*u)))/(9._dp*(u*u)))+
     & G3(2,0,1)*((H1(0)*(omv*(-11-30*NFV*(-2+u)+22*u)-18*(u*u)))/
     & (9._dp*(u*u))+(H1(1)*(omv*(-11-30*NFV*(-2+u)+22*u)-18*(u*u)))/
     & (9._dp*(u*u)))-((omv-2*(u*(1+8*u-v)+15*NFV*(-2+u)*(-1+v)))*
     & G4(1,0,2,2))/(9._dp*(u*u))-((omv-
     & 2*(u*(1+8*u-v)+15*NFV*(-2+u)*(-1+v)))*G4(1,2,0,2))/(9._dp*(u*u))
     & -((omv-2*(u*(1+8*u-v)+15*NFV*(-2+u)*(-1+v)))*G4(1,2,2,0))/
     & (9._dp*(u*u))+((omv+2*u*(-1+u+v))*G4(2,1,0,2))/(u*u)+
     & ((omv+2*u*(-1+u+v))*G4(2,1,2,0))/(u*u)-
     & ((omv+2*(5*NFV*(-2+u)*(-1+v)+2*u*(-1+3*u+v)))*G4(2,2,3,0))/
     & (3._dp*(u*u))-(20*(omv+2*u*(-1-4*u+v))*G4(2,3,3,0))/(9._dp*(u*u))+
     & ((omv+2*(5*NFV*(-2+u)*(-1+v)+2*u*(-1+3*u+v)))*G3(2,2,3)*H1(0))/
     & (3._dp*(u*u))+(20*(omv+2*u*(-1-4*u+v))*G3(2,3,3)*H1(0))/(9._dp*(u*u))+
     & (G4(1,0,3,0)*(omv*(17+30*NFV*(-2+u)-28*u)-160*(u*u)))/(9._dp*(u*u))+
     & (G4(1,2,3,0)*(omv*(17+30*NFV*(-2+u)-28*u)-160*(u*u)))/(9._dp*(u*u))+
     & (G3(2,0,3)*H1(0)*(-3+6*omv*(5*NFV*(-2+u)+2*u)+3*v-98*(u*u)))/
     & (9._dp*(u*u))+(G4(1,1,2,2)*(15*omv*(-1+2*NFV*(-2+u)+2*u)-
     & 32*(u*u)))/(9._dp*(u*u))+
     & (G4(1,2,1,2)*(15*omv*(-1+2*NFV*(-2+u)+2*u)-32*(u*u)))/(9._dp*(u*u))+
     & (G4(1,1,0,2)*(omv*(-21-30*NFV*(-2+u)+42*u)-20*(u*u)))/(9._dp*(u*u))+
     & (G4(1,1,2,0)*(omv*(-21-30*NFV*(-2+u)+42*u)-20*(u*u)))/(9._dp*(u*u))+
     & (G4(2,1,1,2)*(-4+8*u+4*v-8*u*v-8*(u*u)))/(u*u)+
     & (G4(0,1,0,2)*(-2+4*u+2*v-4*u*v-4*(u*u)))/(u*u)+
     & (G4(0,1,2,0)*(-2+4*u+2*v-4*u*v-4*(u*u)))/(u*u)-
     & (G4(2,2,1,2)*(3+30*omv*NFV*(-2+u)+6*u*(-1+v)-3*v+4*(u*u)))/
     & (9._dp*(u*u))+(G4(2,0,1,2)*(omv*(11+30*NFV*(-2+u)-22*u)+18*(u*u)))/
     & (9._dp*(u*u))-(G4(0,1,1,2)*(-9+18*omv*u+9*v+20*(u*u)))/(9._dp*(u*u))+
     & (G4(2,0,3,0)*(3-6*omv*(5*NFV*(-2+u)+2*u)-3*v+98*(u*u)))/
     & (9._dp*(u*u))+(G4(1,1,1,2)*(3*omv*(19+10*NFV*(-2+u)-38*u)+
     & 112*(u*u)))/(9._dp*(u*u))+
     & (G3(1,0,3)*H1(0)*(omv*(-17-30*NFV*(-2+u)+28*u)+160*(u*u)))/
     & (9._dp*(u*u))+(G3(1,2,3)*H1(0)*
     & (omv*(-17-30*NFV*(-2+u)+28*u)+160*(u*u)))/(9._dp*(u*u))+
     & G3(2,2,1)*((H1(0)*(3+30*omv*NFV*(-2+u)-6*omv*u-3*v+4*(u*u)))/
     & (9._dp*(u*u))+(H1(1)*(3+30*omv*NFV*(-2+u)-6*omv*u-3*v+
     & 4*(u*u)))/(9._dp*(u*u)))+
     & G3(0,1,0)*((H1(0)*(2-4*omv*u-2*v+4*(u*u)))/(u*u)+
     & (H1(1)*(2-4*omv*u-2*v+4*(u*u)))/(u*u))+
     & G3(2,1,1)*((H1(0)*(4-8*omv*u-4*v+8*(u*u)))/(u*u)+
     & (H1(1)*(4-8*omv*u-4*v+8*(u*u)))/(u*u))+
     & G3(0,1,2)*(-2*H1(0)+(134-41/omv-20/omw+14/u-(32*u)/omv+
     & (44*v)/omu+(10*v)/u+(19*v)/(omu*omu)+(32*u)/(omv*omv)-
     & 10/(omw*omw)+30*NFV*
     & (-4/u-(3*u*v)/(omv*omv)+
     & (v*(-2+u*(3+v)-2*(u*u)))/(omv*(omu*omu)))-16/(u*u)+
     & (16*v)/(u*u)-(nf*(omv+2*u*(-1+3*u+v)))/(u*u))/18._dp-
     & (H1(1)*(9-18*u-9*v+18*u*v+16*(u*u)))/(9._dp*(u*u)))+
     & G3(1,1,0)*((H1(0)*(3*omv*(7+10*NFV*(-2+u)-14*u)+20*(u*u)))/
     & (9._dp*(u*u))+(H1(1)*(3*omv*(7+10*NFV*(-2+u)-14*u)+20*(u*u)))/
     & (9._dp*(u*u)))+G3(1,2,1)*((H1(0)*
     & (-15*omv*(-1+2*NFV*(-2+u)+2*u)+32*(u*u)))/(9._dp*(u*u))+
     & (H1(1)*(-15*omv*(-1+2*NFV*(-2+u)+2*u)+32*(u*u)))/(9._dp*(u*u)))+
     & G2(0,1)*(2*H2(1,0)+H2(1,1)*(16._dp/9._dp-(2*omv)/u+(1-v)/(u*u))+
     & (H1(0)*(-134+41/omv+20/omw-14/u+(32*u)/omv-(44*v)/omu-
     & (10*v)/u-(19*v)/(omu*omu)-(32*u)/(omv*omv)+
     & 10/(omw*omw)+30*NFV*
     & (4/u+(3*u*v)/(omv*omv)-
     & (v*(-2+u*(3+v)-2*(u*u)))/(omv*(omu*omu)))+16/(u*u)-
     & (16*v)/(u*u)+(nf*(omv+2*u*(-1+3*u+v)))/(u*u)))/18._dp+
     & (H1(1)*(-134+41/omv+20/omw-14/u+(32*u)/omv-(44*v)/omu-
     & (10*v)/u-(19*v)/(omu*omu)-(32*u)/(omv*omv)+
     & 10/(omw*omw)+30*NFV*
     & (4/u+(3*u*v)/(omv*omv)-
     & (v*(-2+u*(3+v)-2*(u*u)))/(omv*(omu*omu)))+16/(u*u)-
     & (16*v)/(u*u)+(nf*(omv+2*u*(-1+3*u+v)))/(u*u)))/18._dp-
     & (H2(0,1)*(-9+18*omv*u+9*v+2*(u*u)))/(9._dp*(u*u))+
     & (zeta2*(-63*omv*(-1+2*u)+208*(u*u)))/(9._dp*(u*u)))+
     & G3(0,2,2)*(H1(0)*(16._dp/9._dp-(2*omv)/u+(1-v)/(u*u))-
     & (10*NFV*(12*omv*(-1+3*u)+
     & (-40+u*(20-17*v)+37*v-4*(u*u))*(u*u))+
     & omu*(2*omv+u*(59+v+2*u*(-62+9*v)-
     & 4*nf*((-1+u)*(-1+u))+63*(u*u))))/(18._dp*u*(omu*omu*omu))
     & )+G3(2,0,2)*(((omv-2*u*(1+5*u-v))*H1(0))/(9._dp*(u*u))-
     & (10*NFV*(12*omv*(-1+3*u)+
     & (-40+u*(20-17*v)+37*v-4*(u*u))*(u*u))+
     & omu*(2*omv+u*(59+v+2*u*(-62+9*v)-
     & 4*nf*((-1+u)*(-1+u))+63*(u*u))))/(18._dp*u*(omu*omu*omu))
     & )+G3(2,2,0)*((2*H1(0)*(7*omv*u+2*(-1+v-5*(u*u))))/(9._dp*(u*u))-
     & (10*NFV*(12*omv*(-1+3*u)+
     & (-40+u*(20-17*v)+37*v-4*(u*u))*(u*u))+
     & omu*(2*omv+u*(59+v+2*u*(-62+9*v)-
     & 4*nf*((-1+u)*(-1+u))+63*(u*u))))/(18._dp*u*(omu*omu*omu))
     & )+(H2(1,1)*(-8*omv*(nf+20*NFV*(1-2*u))-202*(-1+v)+
     & u*(-593+404*v-4*nf*(-7+4*v)-384*zeta2)+
     & 4*(145-40*NFV+4*nf*(-2+v)-100*v+60*NFV*v+192*zeta2)*
     & (u*u)+(-189+12*nf-384*zeta2)*(u*u*u)))/(36._dp*u*(omu*omu))+
     & G3(1,2,2)*((H1(0)*(15*omv*(-1+2*NFV*(-2+u)+2*u)-32*(u*u)))/
     & (9._dp*(u*u))+(81-302*u-81*v+302*u*v-
     & 4*nf*(omv+2*u*(-1+u+v))*(omu*omu)+484*(u*u)-
     & 361*v*(u*u)-386*(u*u*u)+174*v*(u*u*u)-
     & 10*NFV*(19*omv*u+4*(-1+v)+(-30+26*v)*(u*u)+
     & (19-17*v)*(u*u*u)-4*(u*u*u*u))+123*(u*u*u*u))/
     & (18._dp*(omu*omu)*(u*u)))+
     & G2(2,2)*(H2(1,0)*(-4+(omv*(-13+32*u))/(9._dp*(u*u)))-
     & (omv*(5+10*NFV*(-2+u)-8*u)*H2(0,0))/(3._dp*(u*u))-
     & (H2(0,1)*(3+30*omv*NFV*(-2+u)+6*u*(-1+v)-3*v+4*(u*u)))/
     & (9._dp*(u*u))+(24*zeta2*(omu*omu)*
     & (-5+omv*(15*NFV*(-2+u)+13*u)+5*v-19*(u*u))-
     & 3*u*(8*omv*(nf+20*NFV*(1-2*u))+202*(-1+v)+
     & u*(593-404*v+4*nf*(-7+4*v))-
     & 4*(4*nf*(-2+v)+5*(29-8*NFV-20*v+12*NFV*v))*(u*u)-
     & 3*(-63+4*nf)*(u*u*u)))/(108._dp*(omu*omu)*(u*u))+
     & (H1(0)*(81-302*u-81*v+302*u*v-
     & 4*nf*(omv+2*u*(-1+u+v))*(omu*omu)+484*(u*u)-
     & 361*v*(u*u)-386*(u*u*u)+174*v*(u*u*u)-
     & 10*NFV*(19*omv*u+4*(-1+v)+(-30+26*v)*(u*u)+
     & (19-17*v)*(u*u*u)-4*(u*u*u*u))+123*(u*u*u*u)))/
     & (18._dp*(omu*omu)*(u*u)))+
     & G3(2,3,0)*((82*H1(0))/9._dp+(26*H1(1))/9._dp+
     & ((39-32*u)/omv+(60*NFV*(-1+v)*v+u*(32-90*NFV*v))/
     & (omv*omv)+(-5058*omu*u*v*w+3640*u*v*w*(omu*omu)-
     & 1048*u*v*w*(omu*omu*omu)+
     & omu*omu*omu*omu*
     & (2*(14+60*NFV*(-2+v)-17*v)*w+u*(4+527*w)-4*(u*u))-
     & 9*u*w*(-281*v+4*nf*
     & (1+u*(-4+3*v)+3*(2+v)*(u*u)+(-4+3*v)*(u*u*u)+
     & u*u*u*u)))/(u*w*(omu*omu*omu*omu)))/18._dp)+
     & G2(2,3)*(-20*H2(0,0)-(26*H2(0,1))/9._dp-(26*H2(1,0))/9._dp-
     & (4*zeta2*(5*omv*(-1+2*u)+13*(u*u)))/(9._dp*(u*u))+
     & (H1(0)*((-39+32*u)/omv+
     & (-60*NFV*(-1+v)*v+u*(-32+90*NFV*v))/(omv*omv)+
     & (5058*omu*u*v*w-3640*u*v*w*(omu*omu)+
     & 1048*u*v*w*(omu*omu*omu)+
     & omu*omu*omu*omu*
     & (2*(-14-60*NFV*(-2+v)+17*v)*w-u*(4+527*w)+4*(u*u))
     & +9*u*w*(-281*v+
     & 4*nf*(1+u*(-4+3*v)+3*(2+v)*(u*u)+
     & (-4+3*v)*(u*u*u)+u*u*u*u)))/
     & (u*w*(omu*omu*omu*omu))))/18._dp)+
     & G2(2,1)*((2*(omv+2*u*(-1+u+v))*zeta2)/(u*u)+
     & (H2(0,1)*(-2+4*u+2*v-4*u*v-4*(u*u)))/(u*u)+
     & (H2(1,0)*(2-4*omv*u-2*v+4*(u*u)))/(u*u)+
     & (H1(0)*(-((41+60*NFV+32*u)/omv)+(u*(32-90*NFV*v))/(omv*omv)+
     & (4*omv*(nf-5*NFV))/(u*u)+
     & (52*omu*v*(omw*omw)*(u*u)-
     & 4*(13+15*NFV*u)*v*(omw*omw)*(u*u)+
     & omu*omu*(10*(u*u)+20*omw*(u*u)+
     & omw*omw*(-87+107*v+(-97+8*nf+40*NFV+3*v)*(u*u)-
     & 19*(v*v)+2*u*
     & (87+5*NFV+4*nf*(-1+v)-76*v-65*NFV*v+v*v)-
     & v*v*v)))/(omu*omu*(omw*omw)*(u*u))))/18._dp+
     & (H1(1)*(-((41+60*NFV+32*u)/omv)+(u*(32-90*NFV*v))/(omv*omv)+
     & (4*omv*(nf-5*NFV))/(u*u)+
     & (52*omu*v*(omw*omw)*(u*u)-
     & 4*(13+15*NFV*u)*v*(omw*omw)*(u*u)+
     & omu*omu*(10*(u*u)+20*omw*(u*u)+
     & omw*omw*(-87+107*v+(-97+8*nf+40*NFV+3*v)*(u*u)-
     & 19*(v*v)+2*u*
     & (87+5*NFV+4*nf*(-1+v)-76*v-65*NFV*v+v*v)-
     & v*v*v)))/(omu*omu*(omw*omw)*(u*u))))/18._dp)+
     & G3(2,1,2)*((H1(0)*(-2+4*u+2*v-4*u*v-4*(u*u)))/(u*u)+
     & (97+41/omv-20/omw-174/u+(32*u)/omv-3*v-(52*v)/omu+
     & (152*v)/u+(52*v)/(omu*omu)-(32*u)/(omv*omv)-
     & 10/(omw*omw)+10*NFV*
     & (-4+6/omv+(-1+13*v)/u+(6*u*v)/(omu*omu)+
     & (9*u*v)/(omv*omv)+(2-2*v)/(u*u))+87/(u*u)-
     & (107*v)/(u*u)-(4*nf*(omv+2*u*(-1+u+v)))/(u*u)-
     & (2*(v*v))/u+(19*(v*v))/(u*u)+(v*v*v)/(u*u))/18._dp)+
     & (H2(0,1)*(-447+108/omu-444/omv-60/omw+654/u+(522*v)/omu-
     & (1026*v)/u-108/w+1896*zeta2+
     & 18*nf*(3-(2*omv)/u+(2*u*v)/(omu*omu))-(522*v)/(omu*omu)+
     & 60*NFV*(-9+3/omu+9/omv+(-2+8*v+2/w)/u-7/w+
     & (4*u*v)/(omu*omu)+2/(w*w)-(2*u)/(w*w))))/108._dp+
     & (H3(0,0,0)*(-1731-31/omv+(32*u)/omv-(1066*v)/omu+(2*v)/u-
     & 86/w+(166*u)/w+(3658*v)/(omu*omu)-(5058*v)/(omu*omu*omu)+
     & (2529*v)/(omu*omu*omu*omu)-(32*u)/(omv*omv)+
     & (30*NFV*v*(2*omv*u+4*(omv*omv)+3*(u*u)))/(u*(omv*omv))+
     & 4*nf*(28-(27*u*v*(1+u+u*u))/(omu*omu*omu*omu))-
     & (80*u)/(w*w)+(80*(u*u))/(w*w)))/18._dp+
     & nf*(6019._dp/486._dp-100/(27._dp*omu)-(139*zeta3)/27._dp+(6*v*zeta3)/omu-
     & (24*v*zeta3)/(omu*omu)+(36*v*zeta3)/(omu*omu*omu)-
     & (18*v*zeta3)/(omu*omu*omu*omu)+
     & (zeta2*(4*(-4+5*v)*(omv*omv*omv)-
     & 3*u*(omv*omv)*(113-115*v+4*(v*v))-
     & u*u*(-1899+4645*v+(-365+349*v)*(u*u*u*u)-3381*(v*v)+
     & u*u*u*(1989-2971*v+1030*(v*v))+387*(v*v*v)+
     & u*u*(-4056+7776*v-4817*(v*v)+1049*(v*v*v))+
     & 248*(v*v*v*v)+u*
     & (3976-8708*v+6343*(v*v)-1963*(v*v*v)+368*(v*v*v*v)))))/
     & (18._dp*omv*u*(omu*omu*omu)*(w*w)))+
     & H1(1)*((-4*H2(0,0))/3._dp+(5*H2(1,0))/3._dp+
     & (4*nf*(-28-(57*omv)/u-(39*v)/omu+(39*v)/(omu*omu)+
     & (18*zeta2*(1+u*(-4+27*v)+3*(2+9*v)*(u*u)+
     & (-4+27*v)*(u*u*u)+u*u*u*u))/(omu*omu*omu*omu))+
     & 3*(34+498/omu+1215/u+(1461*v)/omu-(1215*v)/u+
     & 5892*zeta3-(1461*v)/(omu*omu)+
     & 6*zeta2*(-289-167/omv-60/omw+(55-25*v)/u+153/w-
     & 30/(omw*omw)-
     & (9*v*(-13+167*u-11*(u*u)+135*(u*u*u)+3*(u*u*u*u)))/
     & (omu*omu*omu*omu)+
     & u*(32/omv-32/(omv*omv)+(151-304*w)/(w*w))-
     & (151*(u*u))/(w*w))+
     & 10*NFV*(81-60/omu-(72*v)/omu+(6*(-5+6*v))/u+6/w+
     & (72*v)/(omu*omu)+
     & 6*zeta2*(-30+24/omu+6/omv+(60-48*v)/u+
     & u*(-9/omv+(3*v)/(omu*omu*omu)+9/(omv*omv)-
     & 2/(w*w))+
     & u*u*((21*v)/(omu*omu*omu)+(2-4*v)/(w*w*w))-
     & (2*(u*u*u))/(w*w*w)))))/324._dp)+
     & G3(0,3,0)*((64*H1(0))/9._dp-(8*H1(1))/9._dp+
     & (-162-39/omv+27*v-(1048*v)/omu+(60*NFV*v)/omv+
     & (40-34*v+120*NFV*v)/u-75/w+(3640*v)/(omu*omu)-
     & (5058*v)/(omu*omu*omu)+(2529*v)/(omu*omu*omu*omu)+
     & (4*nf*(2-u*(8+27*v)-3*(-4+9*v)*(u*u)-(8+27*v)*(u*u*u)+
     & 2*(u*u*u*u)))/(omu*omu*omu*omu)+
     & (u*((-71+20*NFV+146*w)*(omv*omv)+
     & 2*omv*(16-45*NFV)*(w*w)+2*(-16+45*NFV)*(w*w)))/
     & (omv*omv*(w*w))+((20*NFV*(-1+2*v)+71*w)*(u*u))/(w*w*w)+
     & (20*NFV*(u*u*u))/(w*w*w))/18._dp)+
     & (H3(1,0,1)*(43-41/omv+20/omw+(22-120*NFV)/u-11/w+
     & 10/(omw*omw)+3*v*((-20*NFV)/omv-
     & (3*(3-4*u+3*(u*u)))/(omu*omu))+
     & (u*((-9-20*NFV+20*w)*(omv*omv)+2*(16-45*NFV)*(w*w)+
     & 2*omv*(-16+45*NFV)*(w*w)))/(omv*omv*(w*w))-
     & ((20*NFV*(-1+2*v)-9*w)*(u*u))/(w*w*w)-(20*NFV*(u*u*u))/(w*w*w)))
     & /18._dp+(H3(1,1,0)*(75-208/omv-20/omw-54*v+(1140*v)/omu-
     & (60*NFV*v)/omu+(30-120*omv*NFV+22*v)/u+142/w-
     & (3606*v)/(omu*omu)+(60*NFV*v)/(omu*omu)+
     & (5058*v)/(omu*omu*omu)-(2529*v)/(omu*omu*omu*omu)-
     & 10/(omw*omw)-(4*nf*(1-u*(4+27*v)+(6-27*v)*(u*u)-
     & (4+27*v)*(u*u*u)+u*u*u*u))/(omu*omu*omu*omu)-
     & (2*u*(71+20*NFV-142*v))/(w*w)+
     & (2*(71*omv+20*NFV*(1-2*v))*(u*u))/(w*w*w)-
     & (2*(71+20*NFV)*(u*u*u))/(w*w*w)))/18._dp+
     & (H3(0,0,1)*(9*v*(4/u+(-3+8*u-3*(u*u))/(omu*omu))-
     & (20*omv*NFV*u*(omv+u*(-2+u+2*v))+
     & v*(17*u*(omv*omv)+omv*omv*omv-28*omv*(u*u)+10*(u*u*u)))
     & /(omv*(w*w*w))))/18._dp+
     & G2(3,3)*((-256*zeta2)/9._dp+(128*H2(0,0))/9._dp-
     & (H1(0)*((492-32*nf-20*NFV+3*v)*(u*u*u*u*u)+
     & u*u*u*u*(-1436+96*omv*nf+NFV*(40-70*v)+1383*v+
     & 11*(v*v))+u*u*u*
     & (1356-2560*v-96*nf*(omv*omv)+1181*(v*v)-
     & 10*NFV*(2-8*v+9*(v*v))+14*(v*v*v))+
     & omv*v*(-40+108*v-2*(43+10*NFV)*(v*v)+17*(v*v*v)+
     & v*v*v*v)+omv*u*
     & (-40+264*v-2*(188+15*NFV)*(v*v)+2*(71+5*NFV)*(v*v*v)+
     & v*v*v*v)+u*u*(-372+(910-30*NFV)*v+
     & 32*nf*(omv*omv*omv)+(-709+60*NFV)*(v*v)+
     & (165-50*NFV)*(v*v*v)+6*(v*v*v*v))))/(18._dp*(u*u)*(w*w*w)))+
     & G3(3,2,0)*((-88*H1(0))/9._dp-(10*NFV*
     & ((-4+7*v)*(u*u*u*u)+2*(u*u*u*u*u)+u*(v*v)*(3-4*v+v*v)+
     & v*(u*u)*(3-6*v+5*(v*v))+u*u*u*(2-8*v+9*(v*v))+
     & 2*omv*(v*v*v))+v*
     & ((31+11*v)*(u*u*u*u)+3*(u*u*u*u*u)-
     & v*(omv*omv)*(-10+18*v+v*v)+
     & u*u*u*(-82+77*v+14*(v*v))+
     & u*u*(68-119*v+45*(v*v)+6*(v*v*v))-
     & u*(20-30*v-8*(v*v)+17*(v*v*v)+v*v*v*v)))/
     & (18._dp*(u*u)*(w*w*w)))+G3(3,0,2)*
     & ((2*H1(0))/9._dp-(10*NFV*((-4+7*v)*(u*u*u*u)+2*(u*u*u*u*u)+
     & u*(v*v)*(3-4*v+v*v)+v*(u*u)*(3-6*v+5*(v*v))+
     & u*u*u*(2-8*v+9*(v*v))+2*omv*(v*v*v))+
     & v*((31+11*v)*(u*u*u*u)+3*(u*u*u*u*u)-
     & v*(omv*omv)*(-10+18*v+v*v)+
     & u*u*u*(-82+77*v+14*(v*v))+
     & u*u*(68-119*v+45*(v*v)+6*(v*v*v))-
     & u*(20-30*v-8*(v*v)+17*(v*v*v)+v*v*v*v)))/
     & (18._dp*(u*u)*(w*w*w)))+G2(3,2)*
     & ((-158*zeta2)/9._dp+(80*H2(0,0))/9._dp+(10*H2(0,1))/9._dp-10*H2(1,0)+
     & (H1(0)*(10*NFV*((-4+7*v)*(u*u*u*u)+2*(u*u*u*u*u)+
     & u*(v*v)*(3-4*v+v*v)+v*(u*u)*(3-6*v+5*(v*v))+
     & u*u*u*(2-8*v+9*(v*v))+2*omv*(v*v*v))+
     & v*((31+11*v)*(u*u*u*u)+3*(u*u*u*u*u)-
     & v*(omv*omv)*(-10+18*v+v*v)+
     & u*u*u*(-82+77*v+14*(v*v))+
     & u*u*(68-119*v+45*(v*v)+6*(v*v*v))-
     & u*(20-30*v-8*(v*v)+17*(v*v*v)+v*v*v*v))))/
     & (18._dp*(u*u)*(w*w*w)))+G3(3,3,0)*
     & ((-128*H1(0))/9._dp+(32*nf*(omv*omv*omv)*(u*u)-
     & 96*nf*(omv*omv)*(u*u*u)+
     & u*u*(-372+(910-30*NFV)*v+
     & (492-32*nf-20*NFV+3*v)*(u*u*u)+(-709+60*NFV)*(v*v)+
     & u*u*(-1436+NFV*(40-70*v)+1383*v+11*(v*v))+
     & (165-50*NFV)*(v*v*v)+
     & u*(1356-2560*v+1181*(v*v)-10*NFV*(2-8*v+9*(v*v))+
     & 14*(v*v*v))+6*(v*v*v*v))+
     & omv*(96*nf*(u*u*u*u)+
     & v*(-40+108*v-2*(43+10*NFV)*(v*v)+17*(v*v*v)+v*v*v*v)+
     & u*(-40+264*v-2*(188+15*NFV)*(v*v)+
     & 2*(71+5*NFV)*(v*v*v)+v*v*v*v)))/(18._dp*(u*u)*(w*w*w)))+
     & (H3(0,1,0)*(792+123/omv+(32*u)/omv-27*v+(1176*v)/omu-
     & (14*v)/u+75/w-(146*u)/w-(3642*v)/(omu*omu)+
     & (5058*v)/(omu*omu*omu)-(2529*v)/(omu*omu*omu*omu)-
     & (32*u)/(omv*omv)-(12*nf*
     & (5-u*(20+9*v)+(30-9*v)*(u*u)-(20+9*v)*(u*u*u)+
     & 5*(u*u*u*u)))/(omu*omu*omu*omu)+(71*u)/(w*w)-
     & (71*(u*u))/(w*w)+10*NFV*
     & ((6*omu*v+6*u*(u-v)*v-9*u*(omu*omu))/(omv*(omu*omu))+
     & (9*u)/(omv*omv)-(2*
     & (w*(u*u)+(-1+2*v)*(u*u*u)+u*u*u*u-6*v*(w*w*w)))/
     & (u*(w*w*w)))))/18._dp+
     & G3(1,0,2)*(((omv-2*(u*(1+8*u-v)+15*NFV*(-2+u)*(-1+v)))*H1(1))/
     & (9._dp*(u*u))+(2*H1(0)*(5*omv*(1+3*NFV*(-2+u)-2*u)-9*(u*u)))/
     & (9._dp*(u*u))+(63-272*v+3*(-67+9*v)*(u*u*u*u*u*u*u)+447*(v*v)+
     & u*u*u*u*u*u*(1173-1030*v+81*(v*v))-330*(v*v*v)+
     & u*u*u*u*u*(-2913+4606*v-1810*(v*v)+81*(v*v*v))+
     & u*u*u*(-3315+9396*v-9126*(v*v)+3380*(v*v*v)-344*(v*v*v*v))+
     & 92*(v*v*v*v)+u*u*u*u*
     & (4005-8955*v+6224*(v*v)-1325*(v*v*v)+27*(v*v*v*v))-
     & 10*NFV*(2-8*v+(4-8*v)*(u*u*u*u*u*u)+2*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(-48+83*v-36*(v*v))+12*(v*v)+
     & u*u*u*u*(114-247*v+168*(v*v)-36*(v*v*v))-6*(v*v*v)+
     & u*u*u*(-126+335*v-303*(v*v)+105*(v*v*v)-12*(v*v*v*v))+
     & u*(-20+74*v-99*(v*v)+54*(v*v*v)-9*(v*v*v*v))+
     & u*u*(72-229*v+258*(v*v)-117*(v*v*v)+18*(v*v*v*v)))-
     & omv*nf*(omu*omu)*(w*w*w)+
     & u*u*(1671-5635*v+6862*(v*v)-3496*(v*v*v)+598*(v*v*v*v)-
     & 6*nf*(omu*omu)*(w*w*w))-
     & u*(-1+v)*(-483+1380*v-1298*(v*v)+392*(v*v*v)+
     & 2*nf*(omu*omu)*(w*w*w)))/(18._dp*(omu*omu)*(u*u)*(w*w*w)))+
     & G3(1,2,0)*(((omv-2*(u*(1+8*u-v)+15*NFV*(-2+u)*(-1+v)))*H1(1))/
     & (9._dp*(u*u))-(2*H1(0)*(-3+3*omv*(-5*NFV*(-2+u)+u)+3*v+
     & 89*(u*u)))/(9._dp*(u*u))+
     & (63-272*v+3*(-67+9*v)*(u*u*u*u*u*u*u)+447*(v*v)+
     & u*u*u*u*u*u*(1173-1030*v+81*(v*v))-330*(v*v*v)+
     & u*u*u*u*u*(-2913+4606*v-1810*(v*v)+81*(v*v*v))+
     & u*u*u*(-3315+9396*v-9126*(v*v)+3380*(v*v*v)-344*(v*v*v*v))+
     & 92*(v*v*v*v)+u*u*u*u*
     & (4005-8955*v+6224*(v*v)-1325*(v*v*v)+27*(v*v*v*v))-
     & 10*NFV*(2-8*v+(4-8*v)*(u*u*u*u*u*u)+2*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(-48+83*v-36*(v*v))+12*(v*v)+
     & u*u*u*u*(114-247*v+168*(v*v)-36*(v*v*v))-6*(v*v*v)+
     & u*u*u*(-126+335*v-303*(v*v)+105*(v*v*v)-12*(v*v*v*v))+
     & u*(-20+74*v-99*(v*v)+54*(v*v*v)-9*(v*v*v*v))+
     & u*u*(72-229*v+258*(v*v)-117*(v*v*v)+18*(v*v*v*v)))-
     & omv*nf*(omu*omu)*(w*w*w)+
     & u*u*(1671-5635*v+6862*(v*v)-3496*(v*v*v)+598*(v*v*v*v)-
     & 6*nf*(omu*omu)*(w*w*w))-
     & u*(-1+v)*(-483+1380*v-1298*(v*v)+392*(v*v*v)+
     & 2*nf*(omu*omu)*(w*w*w)))/(18._dp*(omu*omu)*(u*u)*(w*w*w)))+
     & G2(1,0)*(-((omv-2*(u*(1+8*u-v)+15*NFV*(-2+u)*(-1+v)))*H2(1,1))/
     & (9._dp*(u*u))-(2*H2(0,1)*(5+5*omv*(3*NFV*(-2+u)-2*u)-5*v+
     & u*u))/(9._dp*(u*u))+(H2(0,0)*
     & (-3*omv*(5+10*NFV*(-2+u)-8*u)+160*(u*u)))/(9._dp*(u*u))+
     & (H2(1,0)*(-6-30*omv*NFV*(-2+u)+6*u+6*v-6*u*v+178*(u*u)))/
     & (9._dp*(u*u))+(zeta2*(omv*(11+90*NFV*(-2+u)-34*u)+312*(u*u)))/
     & (9._dp*(u*u))+(H1(0)*(-63+272*v-3*(-67+9*v)*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*u*(-1173+1030*v-81*(v*v))-447*(v*v)+
     & u*u*u*u*u*(2913-4606*v+1810*(v*v)-81*(v*v*v))+
     & 330*(v*v*v)+u*u*u*u*
     & (-4005+8955*v-6224*(v*v)+1325*(v*v*v)-27*(v*v*v*v))-
     & 92*(v*v*v*v)+u*u*u*
     & (3315-9396*v+9126*(v*v)-3380*(v*v*v)+344*(v*v*v*v))+
     & 10*NFV*(2-8*v+(4-8*v)*(u*u*u*u*u*u)+2*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(-48+83*v-36*(v*v))+12*(v*v)+
     & u*u*u*u*(114-247*v+168*(v*v)-36*(v*v*v))-6*(v*v*v)+
     & u*u*u*(-126+335*v-303*(v*v)+105*(v*v*v)-12*(v*v*v*v))+
     & u*(-20+74*v-99*(v*v)+54*(v*v*v)-9*(v*v*v*v))+
     & u*u*(72-229*v+258*(v*v)-117*(v*v*v)+18*(v*v*v*v)))+
     & omv*nf*(omu*omu)*(w*w*w)+
     & u*(-1+v)*(-483+1380*v-1298*(v*v)+392*(v*v*v)+
     & 2*nf*(omu*omu)*(w*w*w))+
     & u*u*(-1671+5635*v-6862*(v*v)+3496*(v*v*v)-598*(v*v*v*v)+
     & 6*nf*(omu*omu)*(w*w*w))))/(18._dp*(omu*omu)*(u*u)*(w*w*w))+
     & (H1(1)*(-63+272*v-3*(-67+9*v)*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*u*(-1173+1030*v-81*(v*v))-447*(v*v)+
     & u*u*u*u*u*(2913-4606*v+1810*(v*v)-81*(v*v*v))+
     & 330*(v*v*v)+u*u*u*u*
     & (-4005+8955*v-6224*(v*v)+1325*(v*v*v)-27*(v*v*v*v))-
     & 92*(v*v*v*v)+u*u*u*
     & (3315-9396*v+9126*(v*v)-3380*(v*v*v)+344*(v*v*v*v))+
     & 10*NFV*(2-8*v+(4-8*v)*(u*u*u*u*u*u)+2*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(-48+83*v-36*(v*v))+12*(v*v)+
     & u*u*u*u*(114-247*v+168*(v*v)-36*(v*v*v))-6*(v*v*v)+
     & u*u*u*(-126+335*v-303*(v*v)+105*(v*v*v)-12*(v*v*v*v))+
     & u*(-20+74*v-99*(v*v)+54*(v*v*v)-9*(v*v*v*v))+
     & u*u*(72-229*v+258*(v*v)-117*(v*v*v)+18*(v*v*v*v)))+
     & omv*nf*(omu*omu)*(w*w*w)+
     & u*(-1+v)*(-483+1380*v-1298*(v*v)+392*(v*v*v)+
     & 2*nf*(omu*omu)*(w*w*w))+
     & u*u*(-1671+5635*v-6862*(v*v)+3496*(v*v*v)-598*(v*v*v*v)+
     & 6*nf*(omu*omu)*(w*w*w))))/(18._dp*(omu*omu)*(u*u)*(w*w*w)))+
     & G2(1,1)*((H2(1,1)*(15*omv*(-1+2*NFV*(-2+u)+2*u)-32*(u*u)))/
     & (9._dp*(u*u))+(zeta2*(3*omv*(19+10*NFV*(-2+u)-38*u)-8*(u*u)))/
     & (9._dp*(u*u))+(H2(1,0)*(-4+8*u+4*v-8*u*v-8*(u*u)))/(u*u)+
     & (H2(0,1)*(3*omv*(7+10*NFV*(-2+u)-14*u)+40*(u*u)))/(9._dp*(u*u))+
     & (H1(0)*(39-136*v+3*(-53+9*v)*(u*u*u*u*u*u*u)+165*(v*v)+
     & u*u*u*u*u*u*(1033-918*v+81*(v*v))-78*(v*v*v)+
     & u*u*u*u*u*(-2819+4257*v-1726*(v*v)+81*(v*v*v))+
     & u*u*u*(-3565+9217*v-8250*(v*v)+2965*(v*v*v)-
     & 358*(v*v*v*v))+10*(v*v*v*v)+
     & u*u*u*u*(4165-8616*v+5773*(v*v)-1325*(v*v*v)+
     & 27*(v*v*v*v))-10*NFV*
     & (-2+8*v-(2+11*v)*(u*u*u*u*u*u)+2*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(-18+68*v-45*(v*v))-12*(v*v)+
     & u*u*u*u*(50-160*v+156*(v*v)-45*(v*v*v))+6*(v*v*v)+
     & u*u*u*(-50+174*v-210*(v*v)+102*(v*v*v)-15*(v*v*v*v))+
     & u*(2-2*v-9*(v*v)+18*(v*v*v)-9*(v*v*v*v))+
     & u*u*(18-77*v+120*(v*v)-81*(v*v*v)+18*(v*v*v*v)))-
     & 4*omv*nf*(omu*omu)*(w*w*w)+
     & u*u*(1739-5322*v+5845*(v*v)-2687*(v*v*v)+425*(v*v*v*v)-
     & 8*nf*(omu*omu)*(w*w*w))-
     & u*(-1+v)*(-433+1058*v-830*(v*v)+214*(v*v*v)+
     & 8*nf*(omu*omu)*(w*w*w))))/(18._dp*(omu*omu)*(u*u)*(w*w*w))+
     & (H1(1)*(39-136*v+3*(-53+9*v)*(u*u*u*u*u*u*u)+165*(v*v)+
     & u*u*u*u*u*u*(1033-918*v+81*(v*v))-78*(v*v*v)+
     & u*u*u*u*u*(-2819+4257*v-1726*(v*v)+81*(v*v*v))+
     & u*u*u*(-3565+9217*v-8250*(v*v)+2965*(v*v*v)-
     & 358*(v*v*v*v))+10*(v*v*v*v)+
     & u*u*u*u*(4165-8616*v+5773*(v*v)-1325*(v*v*v)+
     & 27*(v*v*v*v))-10*NFV*
     & (-2+8*v-(2+11*v)*(u*u*u*u*u*u)+2*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(-18+68*v-45*(v*v))-12*(v*v)+
     & u*u*u*u*(50-160*v+156*(v*v)-45*(v*v*v))+6*(v*v*v)+
     & u*u*u*(-50+174*v-210*(v*v)+102*(v*v*v)-15*(v*v*v*v))+
     & u*(2-2*v-9*(v*v)+18*(v*v*v)-9*(v*v*v*v))+
     & u*u*(18-77*v+120*(v*v)-81*(v*v*v)+18*(v*v*v*v)))-
     & 4*omv*nf*(omu*omu)*(w*w*w)+
     & u*u*(1739-5322*v+5845*(v*v)-2687*(v*v*v)+425*(v*v*v*v)-
     & 8*nf*(omu*omu)*(w*w*w))-
     & u*(-1+v)*(-433+1058*v-830*(v*v)+214*(v*v*v)+
     & 8*nf*(omu*omu)*(w*w*w))))/(18._dp*(omu*omu)*(u*u)*(w*w*w)))+
     & G3(1,1,2)*((H1(0)*(4-8*omv*u-4*v+8*(u*u)))/(u*u)+
     & (H1(1)*(-15*omv*(-1+2*NFV*(-2+u)+2*u)+32*(u*u)))/(9._dp*(u*u))+
     & (-39+136*v-3*(-53+9*v)*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*u*(-1033+918*v-81*(v*v))-165*(v*v)+
     & u*u*u*u*u*(2819-4257*v+1726*(v*v)-81*(v*v*v))+78*(v*v*v)+
     & u*u*u*u*(-4165+8616*v-5773*(v*v)+1325*(v*v*v)-
     & 27*(v*v*v*v))-10*(v*v*v*v)+
     & u*u*u*(3565-9217*v+8250*(v*v)-2965*(v*v*v)+358*(v*v*v*v))+
     & 10*NFV*(-2+8*v-(2+11*v)*(u*u*u*u*u*u)+2*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(-18+68*v-45*(v*v))-12*(v*v)+
     & u*u*u*u*(50-160*v+156*(v*v)-45*(v*v*v))+6*(v*v*v)+
     & u*u*u*(-50+174*v-210*(v*v)+102*(v*v*v)-15*(v*v*v*v))+
     & u*(2-2*v-9*(v*v)+18*(v*v*v)-9*(v*v*v*v))+
     & u*u*(18-77*v+120*(v*v)-81*(v*v*v)+18*(v*v*v*v)))+
     & 4*omv*nf*(omu*omu)*(w*w*w)+
     & u*(-1+v)*(-433+1058*v-830*(v*v)+214*(v*v*v)+
     & 8*nf*(omu*omu)*(w*w*w))+
     & u*u*(-1739+5322*v-5845*(v*v)+2687*(v*v*v)-425*(v*v*v*v)+
     & 8*nf*(omu*omu)*(w*w*w)))/(18._dp*(omu*omu)*(u*u)*(w*w*w)))+
     & (H3(1,0,0)*((49-32*u+90*NFV*u-60*NFV*v)/omv+
     & ((32-90*NFV)*u)/(omv*omv)+
     & (-5058*omu*u*v*(w*w*w)+3658*u*v*(omu*omu)*(w*w*w)-
     & 1066*u*v*(omu*omu*omu)*(w*w*w)-
     & 3*u*(-843*v+4*nf*(13+u*(-52+9*v)+(78+9*v)*(u*u)+
     & (-52+9*v)*(u*u*u)+13*(u*u*u*u)))*(w*w*w)+
     & omu*omu*omu*omu*(w*
     & (u*w*(-7-27*(-94+v)*w)+(-9+16*w)*(u*u)+9*(u*u*u)+
     & 2*(14+v)*(w*w))-
     & 20*NFV*(w*(u*u)+(-1+2*v)*(u*u*u)+u*u*u*u-
     & 6*(-2+v)*(w*w*w))))/(u*(omu*omu*omu*omu)*(w*w*w))))/18._dp
     & +G2(0,3)*((-176*zeta2)/9._dp-(128*H2(0,0))/9._dp+(8*H2(0,1))/9._dp+
     & (8*H2(1,0))/9._dp+(H1(0)*((39-32*u+90*NFV*u-60*NFV*v)/omv+
     & ((32-90*NFV)*u)/(omv*omv)+
     & (5058*omu*u*v*(w*w*w)-3640*u*v*(omu*omu)*(w*w*w)+
     & 1048*u*v*(omu*omu*omu)*(w*w*w)+
     & u*(-2529*v-4*nf*
     & (2-u*(8+27*v)-3*(-4+9*v)*(u*u)-
     & (8+27*v)*(u*u*u)+2*(u*u*u*u)))*(w*w*w)-
     & omu*omu*omu*omu*
     & (w*(3*u*w*(-25+9*(-6+v)*w)+(-71+146*w)*(u*u)+
     & 71*(u*u*u)+2*(20-17*v)*(w*w))+
     & 20*NFV*(w*(u*u)+(-1+2*v)*(u*u*u)+u*u*u*u+6*v*(w*w*w))
     & ))/(u*(omu*omu*omu*omu)*(w*w*w))))/18._dp)+
     & (H2(0,0)*(3*omu*omv*u*(2529+(10116+7299*v-40*NFV*v)*w)*(w*w)-
     & 2*w*(-36*nf*v*(omv*omv*omv)+
     & 2*nf*u*(omv*omv)*(145-37*v+27*(v*v))+
     & 2*nf*(u*u)*(-779+1597*v+2*(-113+77*v)*(u*u*u*u)-
     & 866*(v*v)+2*(u*u*u)*(430-647*v+325*(v*v))-105*(v*v*v)+
     & u*u*(-1531+2286*v-1755*(v*v)+784*(v*v*v))+
     & 153*(v*v*v*v)+u*
     & (1531-2398*v+1185*(v*v)-534*(v*v*v)+288*(v*v*v*v)))+
     & 7587*omv*u*(1+v)*(w*w))+
     & 3*omv*u*w*(omu*omu)*(843-5058*w+
     & 4*(-1811-905*v+5*NFV*(3+2*v))*(w*w))-
     & 3*(omu*omu*omu)*(-4*u*(392-16*u+45*NFV*(1+u))*(w*w*w)+
     & omv*(w*(-2*w*(9+w*(-9-203*v+v*v))+
     & u*u*(492-418*w+34*(w*w))-
     & u*(-825+2890*w+
     & 2*(402+13*v-1128*zeta2+6*(nf*nf))*(w*w)))+
     & 10*NFV*(4*omw*(u*u*u)+4*w*(1-3*w+(2-4*v)*(w*w))-
     & u*u*(2-15*w+6*(w*w)+9*(w*w*w))+
     & 2*u*(2-9*w+13*(w*w)+(-4+3*v)*(w*w*w)))))))/
     & (108._dp*omv*u*(omu*omu*omu)*(w*w*w))+
     & (zeta2*(3*omu*omv*omw*u*(-843+(-3372+80*NFV+5002*v)*w)*(w*w)+
     & omv*omw*u*w*(omu*omu)*
     & (-843+6*(843+40*NFV)*w+(7344-300*NFV-7534*v)*(w*w))-
     & 6*omv*omw*u*(-843+2*v*
     & (843+10*NFV*(-2+13*u-u*u+2*(u*u*u))))*(w*w*w)+
     & omu*omu*omu*(4*omw*u*(36+64*u-45*NFV*(1+4*u))*(w*w*w)+
     & omv*(60*u*(w*w*w)+
     & omw*(w*(-2*w*(9+2*w*(43-91*v+2*(v*v)))+
     & 4*(u*u)*(123-198*w+34*(w*w))+
     & u*(825-2480*w+(8709-104*v)*(w*w)))+
     & 10*NFV*(4*(-3+4*v)*(u*u*u)+16*(u*u*u*u)+
     & 4*w*(1-10*w+(22-12*v)*(w*w))+
     & u*(4-46*w+82*(w*w)-86*(w*w*w))-
     & u*u*(2-7*w+24*(w*w)+36*(w*w*w))))))))/
     & (36._dp*omv*omw*u*(omu*omu*omu)*(w*w*w))+
     & (H2(1,0)*(-3*omu*omv*omw*u*(2529+(10116-7275*v+40*NFV*v)*w)*
     & (w*w)+3*omv*omw*u*w*(omu*omu)*
     & (-843+5058*w+4*(1821+10*NFV*(-3+v)-899*v)*(w*w))-
     & 2*omw*w*(-6*nf*(-3+5*v)*(omv*omv*omv)+
     & nf*u*(omv*omv)*(263-317*v+72*(v*v))+
     & nf*(u*u)*(-1477+3407*v+(-191+143*v)*(u*u*u*u)-2209*(v*v)+
     & u*u*u*(1477-2099*v+766*(v*v))-3*(v*v*v)+
     & u*u*(-3278+6696*v-4773*(v*v)+1211*(v*v*v))+
     & 282*(v*v*v*v)+u*
     & (3188-7220*v+6075*(v*v)-2583*(v*v*v)+588*(v*v*v*v)))+
     & 7587*omv*u*(-1+v)*(w*w))+
     & omu*omu*omu*(12*omw*u*(-1+(-16+45*NFV)*u)*(w*w*w)+
     & omv*(-120*NFV*(omw*omw)*(u*u*u)+60*u*(w*w*w)+
     & omw*(w*(6*w*(-9+w*(101-135*v+v*v))-
     & 6*(u*u)*(-246+351*w+17*(w*w))+
     & u*(2475-7926*w+2*(1206+39*v+4758*zeta2)*(w*w)))+
     & 30*NFV*(8*(u*u*u)+4*w*(1-2*w+(-5+4*v)*(w*w))+
     & u*u*(-2+3*w+6*(w*w)+9*(w*w*w))+
     & u*(4-14*w+8*(w*w)+(26-6*v)*(w*w*w))))))))/
     & (108._dp*omv*omw*u*(omu*omu*omu)*(w*w*w))+
     & ((33-2*nf)/6._dp-(17*H1(0))/2._dp+H1(1)/2._dp)*(G1(0)*G1(0)*G1(0))+
     & (9*(G1(0)*G1(0)*G1(0)*G1(0)))/8._dp+
     & (-H1(1)/18._dp+(-3*omv+u*(-27+63*u-6*v+6*u*v+
     & 2*nf*((-1+u)*(-1+u))-33*(u*u)))/(54._dp*u*(omu*omu))-
     & ((omv+u*(-2+3*u+2*v))*H1(0))/(18._dp*(u*u)))*(G1(2)*G1(2)*G1(2))+
     & (G1(2)*G1(2)*G1(2)*G1(2))/72._dp+((-17*H1(1))/18._dp+
     & (17*(3*omv*v+(3+2*omv*nf-15*v)*(u*u*u)+
     & u*(3+2*omv*nf-21*v+6*(v*v))-
     & 2*(u*u)*(2*omv*nf+3*(1-6*v+v*v))))/(54._dp*omv*u*(omu*omu)))
     & *(H1(0)*H1(0)*H1(0))+(289*(H1(0)*H1(0)*H1(0)*H1(0)))/72._dp+
     & G1(3)*((158*zeta2*H1(1))/9._dp+(16*(-33+2*nf)*H2(0,0))/9._dp+
     & (64*H3(0,0,0))/3._dp-(80*H3(0,1,0))/9._dp-(80*H3(1,0,0))/9._dp-
     & (10*H3(1,0,1))/9._dp+10*H3(1,1,0)-
     & (H2(0,1)*(10*NFV*((-4+7*v)*(u*u*u*u)+2*(u*u*u*u*u)+
     & u*(v*v)*(3-4*v+v*v)+v*(u*u)*(3-6*v+5*(v*v))+
     & u*u*u*(2-8*v+9*(v*v))+2*omv*(v*v*v))+
     & v*((31+11*v)*(u*u*u*u)+3*(u*u*u*u*u)-
     & v*(omv*omv)*(-10+18*v+v*v)+
     & u*u*u*(-82+77*v+14*(v*v))+
     & u*u*(68-119*v+45*(v*v)+6*(v*v*v))-
     & u*(20-30*v-8*(v*v)+17*(v*v*v)+v*v*v*v))))/
     & (18._dp*(u*u)*(w*w*w))-(H2(1,0)*
     & (10*NFV*((-4+7*v)*(u*u*u*u)+2*(u*u*u*u*u)+
     & u*(v*v)*(3-4*v+v*v)+v*(u*u)*(3-6*v+5*(v*v))+
     & u*u*u*(2-8*v+9*(v*v))+2*omv*(v*v*v))+
     & v*((31+11*v)*(u*u*u*u)+3*(u*u*u*u*u)-
     & v*(omv*omv)*(-10+18*v+v*v)+
     & u*u*u*(-82+77*v+14*(v*v))+
     & u*u*(68-119*v+45*(v*v)+6*(v*v*v))-
     & u*(20-30*v-8*(v*v)+17*(v*v*v)+v*v*v*v))))/
     & (18._dp*(u*u)*(w*w*w))+(384*nf*zeta2*(omv*omv*omv)*(u*u)-
     & 1152*nf*zeta2*(omv*omv)*(u*u*u)+
     & 6*omv*zeta2*(192*nf*(u*u*u*u)+
     & v*(40-48*v+2*(-41+70*NFV)*(v*v)+85*(v*v*v)+
     & 5*(v*v*v*v))+u*
     & (40-384*v+(436+210*NFV)*(v*v)-2*(17+35*NFV)*(v*v*v)+
     & 5*(v*v*v*v)))+u*u*
     & (6*zeta2*(84+(362+210*NFV)*v+
     & (-204-64*nf+140*NFV+15*v)*(u*u*u)-
     & (869+420*NFV)*(v*v)+
     & u*u*(572-333*v+70*NFV*(-4+7*v)+55*(v*v))+
     & (393+350*NFV)*(v*v*v)+
     & u*(-492+340*v+145*(v*v)+70*NFV*(2-8*v+9*(v*v))+
     & 70*(v*v*v))+30*(v*v*v*v))-1920*zeta3*(w*w*w)))/
     & (108._dp*(u*u)*(w*w*w))+(76._dp/3._dp-(8*nf)/9._dp-(8*H1(1))/9._dp)*
     & (H1(0)*H1(0))+(68*(H1(0)*H1(0)*H1(0)))/9._dp+
     & H1(0)*((-4*H1(1))/3._dp+(7587*omu*omv*u*(1+(4+3*v)*w)*(w*w)+
     & 2*omv*u*w*(2*nf*(152-250*v+(625+236*v)*(u*u*u*u)-
     & 71*(u*u*u*u*u)+17*(v*v)+
     & u*u*u*(-1439+757*v+631*(v*v))+81*(v*v*v)+
     & u*(-706+1000*v-375*(v*v)+81*(v*v*v))+
     & u*u*(1439-1743*v+213*(v*v)+324*(v*v*v)))-
     & 7587*(1+v)*(w*w))-
     & 3*omv*u*w*(omu*omu)*(-843+5058*w+4*(1820+977*v)*(w*w))-
     & 3*(omu*omu*omu)*(20*NFV*(omv*omv)*
     & (-2*v*(6-13*v+6*(v*v))+
     & u*(-12+56*v-46*(v*v)+3*(v*v*v)))+
     & omv*w*(-2*w*(9+w*(-9-15*v+v*v))+
     & u*u*(492-418*w+34*(w*w))+
     & u*(825-2926*w+(831-26*v+528*zeta2)*(w*w)))+
     & 2*u*(-5*NFV*u*(-48+164*v+9*(-3+v)*(u*u*u)-197*(v*v)+
     & 3*(u*u)*(19-32*v+7*(v*v))+90*(v*v*v)+
     & u*(-12+62*v-59*(v*v)+9*(v*v*v))-9*(v*v*v*v))+
     & 32*(1+u)*(w*w*w))))/(108._dp*omv*u*(omu*omu*omu)*(w*w*w))
     & -(4*(H1(1)*H1(1)))/9._dp))+
     & H1(0)*H1(0)*((68*H2(0,0))/9._dp-(85*H2(1,0))/9._dp-
     & (H1(1)*(17-32*v+(-9+2*omv*nf+v)*(u*u*u)+
     & u*(-43+2*omv*nf+65*v-30*(v*v))+15*(v*v)+
     & u*u*(35-4*omv*nf-49*v+30*(v*v))))/(18._dp*omv*u*(omu*omu))
     & -(12*omv*(-57+2*nf)*v+
     & (4*nf*(-539+515*v)+3*(7199-6287*v+18*omv*zeta2)+
     & 36*omv*(nf*nf))*(u*u*u)-
     & 2*(u*u)*(36*omv*(nf*nf)+
     & 9*(2411-2031*v+6*omv*zeta2-76*(v*v))+
     & 4*nf*(-539+509*v+6*(v*v)))+
     & u*(36*omv*(nf*nf)+3*(7267-5899*v+18*omv*zeta2-456*(v*v))+
     & 4*nf*(-539+503*v+12*(v*v))))/(216._dp*omv*u*(omu*omu))-
     & (5*(H1(1)*H1(1)))/12._dp)+G1(0)*G1(0)*
     & (-G2(0,2)/2._dp-G2(2,0)/2._dp-4*G2(3,0)+4*G1(3)*H1(0)+4*H2(0,0)-5*H2(1,0)-
     & (4464-366*omu*nf-4428*u+810*omu*zeta2+5*omu*(nf*nf))/
     & (72._dp*omu)+G1(2)*(H1(1)/2._dp+
     & (omv+u*(-2+u+2*v-2*u*v))/(2._dp*u*(omu*omu))+
     & ((omv+u*(-2+3*u+2*v))*H1(0))/(2._dp*(u*u)))+
     & G1(1)*(-((omv+2*u*(-1+u+v))*H1(0))/(2._dp*(u*u))-
     & ((omv+2*u*(-1+u+v))*H1(1))/(2._dp*(u*u)))+
     & ((omv+2*u*(-1+u+v))*G2(1,2))/(2._dp*(u*u))+
     & H1(0)*((-3*H1(1))/2._dp+(87-36/omv+10*nf+
     & (9*v*(1-2*u+2*(u*u)))/(u*(omu*omu)))/18._dp)+
     & (H1(1)*(-9*omv+u*(nf*((-1+u)*(-1+u))-
     & 3*(4-17*u+6*v-6*u*v+10*(u*u)))))/(18._dp*u*(omu*omu))-
     & (G1(2)*G1(2))/4._dp+(67*(H1(0)*H1(0)))/4._dp-(H1(1)*H1(1))/4._dp)+
     & G2(2,0)*((H2(0,0)*(5+10*omv*NFV*(-2+u)-8*omv*u-5*v-26*(u*u)))/
     & (3._dp*(u*u))+(2*H2(0,1)*(5+5*omv*(3*NFV*(-2+u)-2*u)-5*v+
     & 14*(u*u)))/(9._dp*(u*u))-
     & (H2(1,0)*(-9+18*omv*u+9*v+52*(u*u)))/(9._dp*(u*u))+
     & (H1(1)*(10*NFV*(12*omv*(-1+3*u)+
     & (-40+u*(20-17*v)+37*v-4*(u*u))*(u*u))+
     & omu*(2*omv+u*(62+v+2*u*(-65+9*v)-
     & 4*nf*((-1+u)*(-1+u))+66*(u*u)))))/
     & (18._dp*u*(omu*omu*omu))+
     & H1(0)*(H1(1)/9._dp+(3658*omv*v*w*(omu*omu)*(u*u)-
     & 1132*omv*v*w*(omu*omu*omu)*(u*u)-
     & 2*omu*omv*w*(10*omv*NFV+
     & u*(2529*u*v+10*NFV*
     & (4-4*v+u*(-17+18*v)-19*(-1+v)*(u*u)+
     & 8*(-1+v)*(u*u*u)+u*u*u*u)))-
     & omv*w*(omv*nf+u*
     & (-2529*u*v+nf*
     & (u*(60-14*v)+6*(-1+v)+4*(-50+31*v)*(u*u)+
     & 3*(95+33*v)*(u*u*u)+2*(-93+55*v)*(u*u*u*u)+
     & 46*(u*u*u*u*u))))+
     & omu*omu*omu*omu*
     & (72*w*(u*u)+omv*
     & ((4+737*w-3*v*w)*(u*u)-4*(u*u*u)-
     & 2*u*w*(70-57*v+v*v)+w*(63-83*v+19*(v*v)+v*v*v)))
     & )/(18._dp*omv*w*(omu*omu*omu*omu)*(u*u)))+
     & (6*omv*w*(w+19*v*w+10*NFV*(-2+9*v*w))*(omu*omu)*(u*u)+
     & 720*omv*NFV*v*(u*u)*(w*w)-
     & 3*omu*omv*(3*(19+2*nf*(-2+u))*v+20*NFV*(2+21*v))*(u*u)*
     & (w*w)+omu*omu*omu*
     & (-6*(nf*u+60*NFV*(-2+u)*zeta2)*(omv*omv)*(w*w)+
     & 12*(-16+45*NFV)*(u*u*u)*(w*w)+
     & omv*(w*(12*(-1+v)*w*zeta2+
     & (-54+w*(489-67*nf+78*v-882*zeta2))*(u*u)-
     & 6*(-9+17*w)*(u*u*u)+
     & 2*u*w*(-12*(-1+v)*zeta2+3*(27-37*v+v*v)))+
     & 30*NFV*u*(4*(1-2*w)*w+3*w*(2+3*w)*(u*u)+4*(u*u*u)+
     & u*(4-8*w+(26-6*v)*(w*w))))))/
     & (108._dp*omv*(omu*omu*omu)*(u*u)*(w*w))-(17*(H1(0)*H1(0)))/18._dp+
     & (H1(1)*H1(1))/18._dp)+G2(0,2)*
     & ((-188*H2(0,0))/9._dp-(2*(omv+u*(-2-9*u+2*v))*H2(1,0))/(u*u)-
     & (H2(0,1)*(9-18*u-9*v+18*u*v+8*(u*u)))/(9._dp*(u*u))+
     & (H1(1)*(10*NFV*(12*omv*(-1+3*u)+
     & (-40+u*(20-17*v)+37*v-4*(u*u))*(u*u))+
     & omu*(2*omv+u*(62+v+2*u*(-65+9*v)-
     & 4*nf*((-1+u)*(-1+u))+66*(u*u)))))/
     & (18._dp*u*(omu*omu*omu))+
     & (6*omv*w*(-20*NFV+w+19*v*w)*(omu*omu)*(u*u)-
     & 3*omu*omv*(40*NFV+3*(19+2*nf*(-2+u))*v)*(u*u)*(w*w)+
     & 180*omv*NFV*v*(u*u)*(-1+4*u+u*u*u)*(w*w)+
     & omu*omu*omu*(-6*nf*u*(omv*omv)*(w*w)+
     & 12*(-16+45*NFV)*(u*u*u)*(w*w)+
     & omv*(w*(108*(-1+v)*w*zeta2+
     & (-54+w*(489-67*nf+78*v+2118*zeta2))*(u*u)-
     & 6*(-9+17*w)*(u*u*u)-
     & 6*u*w*(-27+37*v+36*(-1+v)*zeta2-v*v))+
     & 30*NFV*u*(4*(1-2*w)*w+3*w*(2+3*w)*(u*u)+4*(u*u*u)+
     & u*(4-8*w+26*(w*w))))))/
     & (108._dp*omv*(omu*omu*omu)*(u*u)*(w*w))+
     & H1(0)*(H1(1)/9._dp+(132-8/omv-20/omw+14/u+27*v+(42*v)/omu-
     & (10*v)/u+11/w-(20*u)/w+(18*v)/(omu*omu)-
     & 10/(omw*omw)-16/(u*u)+(16*v)/(u*u)-
     & (nf*(omv+2*u*(-1+5*u+v)))/(u*u)+(9*u)/(w*w)-
     & (9*(u*u))/(w*w)+20*NFV*
     & ((-6*omv)/u+(u*(omu*omu*omu*(u*(-1+2*v)+w+u*u)-
     & (2+u*(-4+v)+v+2*(u*u))*(w*w*w)))/
     & (omu*omu*omu*(w*w*w))))/18._dp)-(17*(H1(0)*H1(0)))/18._dp+
     & (H1(1)*H1(1))/18._dp)+G1(2)*G1(2)*
     & (G2(0,2)/18._dp+G2(2,0)/18._dp+(4*G2(3,0))/9._dp-(4*G1(3)*H1(0))/9._dp-
     & (4*H2(0,0))/9._dp+(5*H2(1,0))/9._dp-
     & ((omv+2*u*(-1+u+v))*G2(1,2))/(18._dp*(u*u))+
     & (36*omv+u*(-76*nf*((-1+u)*(-1+u))+
     & 3*(341+24*v-6*u*(119+4*v)+90*zeta2*((-1+u)*(-1+u))+
     & 361*(u*u))))/(216._dp*u*(omu*omu))+
     & (H1(1)*(3-3*v+u*(27-2*nf+6*v)+(-63+4*nf-6*v)*(u*u)+
     & (33-2*nf)*(u*u*u)))/(18._dp*u*(omu*omu))+
     & H1(0)*((H1(1)*(2-4*omv*u-2*v+7*(u*u)))/(18._dp*(u*u))+
     & (3*(omv*omv)-2*omv*nf*((-1+u)*(-1+u))*(u*u)+
     & u*(-10+19*v+(-58+50*v)*(u*u)+(27-23*v)*(u*u*u)-
     & 9*(v*v)+u*(38-43*v+9*(v*v))))/
     & (18._dp*omv*(omu*omu)*(u*u)))-
     & ((-4+8*omv*u+4*v+7*(u*u))*(H1(0)*H1(0)))/(36._dp*(u*u))+
     & (H1(1)*H1(1))/12._dp)+G2(3,0)*
     & (H1(0)*(32-(20*nf)/9._dp+(8*H1(1))/9._dp)-(64*H2(0,0))/3._dp+(8*H2(0,1))/9._dp+
     & (88*H2(1,0))/9._dp+(H1(1)*((20*NFV+3*(-8+v))*(u*u*u*u*u)+
     & u*u*u*u*(72-41*v+10*NFV*(-4+7*v)+11*(v*v))+
     & omv*(v*v)*(10+4*(-7+5*NFV)*v+17*(v*v)+v*v*v)+
     & omv*u*v*(-20+10*(1+3*NFV)*v-2*(-9+5*NFV)*(v*v)+v*v*v)+
     & u*u*u*(-72+62*v+5*(v*v)+10*NFV*(2-8*v+9*(v*v))+
     & 14*(v*v*v))+u*u*
     & (24+(-4+30*NFV)*v-(47+60*NFV)*(v*v)+
     & (21+50*NFV)*(v*v*v)+6*(v*v*v*v))))/(18._dp*(u*u)*(w*w*w))+
     & (-7587*omu*omv*u*(1+(4+3*v)*w)*(w*w)+
     & 2*omv*u*w*(2*nf*(-152+250*v-(625+236*v)*(u*u*u*u)+
     & 71*(u*u*u*u*u)+u*u*u*(1439-757*v-631*(v*v))-17*(v*v)+
     & u*(706-1000*v+375*(v*v)-81*(v*v*v))-81*(v*v*v)-
     & u*u*(1439-1743*v+213*(v*v)+324*(v*v*v)))+
     & 7587*(1+v)*(w*w))+
     & 3*omv*u*w*(omu*omu)*(-843+5058*w+4*(1820+977*v)*(w*w))+
     & 3*(omu*omu*omu)*(20*NFV*(omv*omv)*
     & (-2*v*(6-13*v+6*(v*v))+
     & u*(-12+56*v-46*(v*v)+3*(v*v*v)))+
     & omv*w*(-2*w*(9+w*(-9-15*v+v*v))+
     & u*u*(492-418*w+34*(w*w))+
     & u*(825-2926*w+(831-26*v+528*zeta2)*(w*w)))+
     & 2*u*(-5*NFV*u*(-48+164*v+9*(-3+v)*(u*u*u)-197*(v*v)+
     & 3*(u*u)*(19-32*v+7*(v*v))+90*(v*v*v)+
     & u*(-12+62*v-59*(v*v)+9*(v*v*v))-9*(v*v*v*v))+
     & 32*(1+u)*(w*w*w))))/(108._dp*omv*u*(omu*omu*omu)*(w*w*w))-
     & (68*(H1(0)*H1(0)))/9._dp+(4*(H1(1)*H1(1)))/9._dp)+
     & ((-4*H2(0,0))/9._dp+(5*H2(1,0))/9._dp+
     & (36*omv+u*(-76*nf*((-1+u)*(-1+u))+
     & 3*(341+24*v-6*u*(119+4*v)+90*zeta2*((-1+u)*(-1+u))+
     & 361*(u*u))))/(216._dp*u*(omu*omu)))*(H1(1)*H1(1))+
     & G2(1,2)*(((omv+30*NFV*(-2+u)*(-1+v)+4*u*(-2+49*u+2*v))*H2(0,0))/
     & (9._dp*(u*u))+(H2(1,0)*(omv*(11-30*NFV*(-2+u)-16*u)-128*(u*u)))/
     & (9._dp*(u*u))+H1(0)*(-((omv+2*u*(-1+u+v))*H1(1))/(9._dp*(u*u))+
     & (85*(-1+v)-438*u*(-1+v)+(-849+621*v)*(u*u)+
     & 4*(omu*omu)*(omv*(2*nf+5*NFV*u)+
     & 2*(2*nf*u*(-1+u+v)-5*NFV*(1-v+u*u)))+
     & (724-340*v)*(u*u*u)-228*(u*u*u*u))/(18._dp*(omu*omu)*(u*u)))+
     & (H1(1)*(-84+314*u+84*v-314*u*v+
     & 4*nf*(omv+2*u*(-1+u+v))*(omu*omu)-505*(u*u)+
     & 376*v*(u*u)+404*(u*u*u)-180*v*(u*u*u)+
     & 10*NFV*(19*omv*u+4*(-1+v)+(-30+26*v)*(u*u)+
     & (19-17*v)*(u*u*u)-4*(u*u*u*u))-129*(u*u*u*u)))/
     & (18._dp*(omu*omu)*(u*u))+
     & (6*omu*omv*omw*(16+90*NFV-69*v)*(u*u)*(w*w)+
     & 2*omv*omw*(2*omv*nf*(19-70*u)+
     & u*u*(2*nf*(112-83*v+u*(-90+26*v)+29*(u*u))+
     & 9*v*(23+10*NFV*(1+u*u))))*(w*w)+
     & omu*omu*(1440*omw*NFV*(-2+u)*zeta2*(omv*omv)*(w*w)-
     & 12*omw*(-16+45*NFV)*(-1+u)*(u*u)*(w*w)-
     & omv*(60*(-1+u)*u*(w*w)+
     & omw*(w*(3*w*(361+26*v+1044*zeta2)*(u*u)-
     & 6*(-9+17*w)*(u*u*u)+
     & w*(-150*(-1+v)*zeta2-3*(-391+371*v+2*(v*v)))+
     & 2*u*(27+
     & w*(186*(-1+v)*zeta2+3*(-293+268*v+v*v))))+
     & 30*NFV*(2*w*(-2+(-1+6*v)*w)+2*w*(1+22*w)*(u*u)+
     & 3*w*(2+3*w)*(u*u*u)+4*(u*u*u*u)-
     & u*(4-14*w+3*(7+2*v)*(w*w)))))))/
     & (108._dp*omv*omw*(omu*omu)*(u*u)*(w*w))+
     & (17*(omv+2*u*(-1+u+v))*(H1(0)*H1(0)))/(18._dp*(u*u))-
     & ((omv+2*u*(-1+u+v))*(H1(1)*H1(1)))/(18._dp*(u*u)))+
     & G1(0)*(G2(0,2)*(5._dp/3._dp-nf/18._dp+H1(0))+G2(2,0)*(5._dp/3._dp-nf/18._dp+H1(0))+
     & G2(3,0)*((-4*(-30+nf))/9._dp+8*H1(0))-(64*H3(0,0,0))/3._dp+
     & (8*H3(0,0,1))/9._dp+(88*H3(0,1,0))/9._dp+(56*H3(1,0,0))/3._dp-
     & (10*H3(1,0,1))/9._dp-(20*H3(1,1,0))/9._dp+
     & G2(1,2)*(((-30+nf)*(omv+2*u*(-1+u+v)))/(18._dp*(u*u))-
     & ((omv+2*u*(-1+u+v))*H1(0))/(u*u))-
     & (H2(1,1)*(10*NFV*(12*omv*(-1+3*u)+
     & (-40+u*(20-17*v)+37*v-4*(u*u))*(u*u))+
     & omu*(2*omv+u*(59+v+2*u*(-62+9*v)-
     & 4*nf*((-1+u)*(-1+u))+63*(u*u)))))/
     & (18._dp*u*(omu*omu*omu))+
     & (H1(1)*((-6+66*v+(120*NFV)/w)/omu+
     & (3*(40*NFV+(-3-12*nf+8*nf*u)*v))/(omu*omu)-
     & (180*NFV*v*(-1+4*u+u*u*u))/(omu*omu*omu)+
     & (12*(16-45*NFV)*(u*u)*(w*w)-
     & omv*(w*(u*(-54+w*(759-76*nf+78*v+198*zeta2))-
     & 6*(-9+17*w)*(u*u)+6*w*(-3-7*v+v*v))+
     & 30*NFV*(4*(1-2*w)*w+3*w*(2+3*w)*(u*u)+4*(u*u*u)+
     & u*(4-8*w+26*(w*w)))))/(omv*u*(w*w))))/108._dp+
     & ((-2529+2*w*(-6261+6600*v*zeta2)-120*NFV*(1+w*(12+6*zeta2)))/
     & (omu*w)+(36*(1686-5*NFV*u*(-3+7*u))*v*zeta2)/
     & (omu*omu*omu)+(12*zeta2*
     & (-2529*v+2*nf*(17+u*(-68+54*v)+6*(17+9*v)*(u*u)+
     & (-68+54*v)*(u*u*u)+17*(u*u*u*u))))/(omu*omu*omu*omu)
     & +(omw*w*(7587+720*NFV-43602*v*zeta2)+
     & 12*nf*u*(9-26*v+(7+83*v)*(u*u)+28*(u*u*u)+26*(v*v)+
     & u*(-44+24*v+55*(v*v))))/(omw*w*(omu*omu))+
     & (6*omv*u*(-39+60*NFV+32*u)*zeta2*(omw*omw)*(w*w)+
     & 12*(-16+45*NFV*v)*zeta2*(omw*omw)*(u*u)*(w*w)+
     & omv*omv*(-120*omw*u*(2+7*u)*(w*w)+240*(u*u)*(w*w)+
     & omw*omw*(3*u*w*(843+474*u+2920*w-324*w*zeta3)+
     & 6*zeta2*(u*(82-1033*w)*w+(80-162*w)*(u*u)-
     & 80*(u*u*u)+2*(10-11*v)*(w*w))+
     & 30*NFV*(2*u*w*(1+w*(18+6*zeta2))+(-2+9*w)*(u*u)+
     & 24*(-2+3*v)*zeta2*(w*w)))))/
     & (u*(omv*omv)*(omw*omw)*(w*w)))/108._dp+
     & (H2(0,0)*(1471+49/omv-(32*u)/omv-54*v+(1114*v)/omu+
     & (30*v)/u+64/w-(126*u)/w-(3640*v)/(omu*omu)+
     & (5058*v)/(omu*omu*omu)-(2529*v)/(omu*omu*omu*omu)+
     & (32*u)/(omv*omv)-
     & (12*nf*(8-u*(32+9*v)+(48-9*v)*(u*u)-(32+9*v)*(u*u*u)+
     & 8*(u*u*u*u)))/(omu*omu*omu*omu)+(62*u)/(w*w)-
     & (62*(u*u))/(w*w)+10*NFV*
     & ((-9*u)/(omv*omv)+
     & (-6*v+2*(2+v)*(u*u*u)+
     & u*(4+16*v+9*(omu*omu*omu)-2*(v*v))-
     & 2*(u*u)*((2+v)*(2+v)))/(omv*(omu*omu*omu))-
     & (4*(w*(u*u)+(-1+2*v)*(u*u*u)+u*u*u*u+3*v*(w*w*w)))/
     & (u*(w*w*w)))))/18._dp+
     & (H2(0,1)*(-55-33/omv+6*nf-(32*u)/omv-27*v+(2*v)/omu+
     & (20*v)/u-11/w+(20*u)/w+v/(omu*omu)+(32*u)/(omv*omv)-
     & (9*u)/(w*w)+(9*(u*u))/(w*w)+
     & 10*NFV*(4/omu+(-6+9*u)/omv+
     & (u*(-1+5*u)*v)/(omu*omu*omu)-(9*u)/(omv*omv)-
     & (2*(w*(u*u)+(-1+2*v)*(u*u*u)+u*u*u*u-u*(w*w*w)+
     & 6*v*(w*w*w)))/(u*(w*w*w)))))/18._dp+
     & (H2(1,0)*(-72/omv+(-3658*u*v*(omu*omu)*(w*w*w)+
     & 1132*u*v*(omu*omu*omu)*(w*w*w)+
     & 2*omu*u*(2529*v+10*NFV*u*(2+u*(-4+v)+v+2*(u*u)))*
     & (w*w*w)+u*(-2529*v+
     & 2*nf*(23+u*(-92+54*v)+6*(23+9*v)*(u*u)+
     & (-92+54*v)*(u*u*u)+23*(u*u*u*u)))*(w*w*w)-
     & omu*omu*omu*omu*
     & (w*(u*w*(-71+779*w+27*v*w)+71*(-1+2*w)*(u*u)+
     & 71*(u*u*u)-6*(1+2*v)*(w*w))+
     & 20*NFV*(w*(u*u)+(-1+2*v)*(u*u*u)+u*u*u*u+
     & 6*(-1+v)*(w*w*w))))/(u*(omu*omu*omu*omu)*(w*w*w))))/
     & 18._dp+((17*H1(0))/18._dp-H1(1)/18._dp)*(G1(2)*G1(2))+
     & G1(3)*((4*(-30+nf)*H1(0))/9._dp-8*(H1(0)*H1(0)))+
     & ((17*H1(1))/6._dp-(18*omv*v+(375+2*omv*nf-447*v)*(u*u*u)-
     & 2*(u*u)*(375+2*omv*nf-465*v+18*(v*v))+
     & u*(375+2*omv*nf-483*v+36*(v*v)))/(18._dp*omv*u*(omu*omu)))*
     & (H1(0)*H1(0))+G1(1)*(H1(0)*
     & (-((-30+nf)*(omv+2*u*(-1+u+v)))/(18._dp*(u*u))+
     & ((omv+2*u*(-1+u+v))*H1(1))/(u*u))-
     & ((-30+nf)*(omv+2*u*(-1+u+v))*H1(1))/(18._dp*(u*u))+
     & ((omv+2*u*(-1+u+v))*(H1(0)*H1(0)))/(u*u))-
     & (289*(H1(0)*H1(0)*H1(0)))/18._dp+
     & G1(2)*(H1(1)/6._dp+((-30+nf)*
     & (2*omv+u*(-7-4*u*(-2+v)+4*v-3*(u*u))))/
     & (36._dp*u*(omu*omu))+
     & H1(0)*((-16*H1(1))/9._dp+(nf*(omv+2*u*(-1+u+v))*(omu*omu)+
     & 6*(17*omv*u+5*(-1+v)+(-33+19*v)*(u*u)+
     & (35-4*v)*(u*u*u)-14*(u*u*u*u)))/(18._dp*(omu*omu)*(u*u)))
     & -((9-18*u-9*v+18*u*v+35*(u*u))*(H1(0)*H1(0)))/(9._dp*(u*u))+
     & (H1(1)*H1(1))/9._dp)+H1(0)*
     & (-8*H2(0,0)+10*H2(1,0)+
     & (H1(1)*(18*omv+u*(-2*nf*((-1+u)*(-1+u))+
     & 9*(5+4*v-4*u*(4+v)+9*(u*u)))))/(18._dp*u*(omu*omu))+
     & (4662*u*zeta2*(omv*omv*omv*omv)-
     & omv*omv*(6*v*(3+NFV*(20-40*v)+4*v-8*(v*v)+v*v*v)+
     & 3*(u*u)*(24472-36470*v+11762*(v*v)+
     & 4662*zeta2*(2-3*v+v*v)-103*(v*v*v)+6*(v*v*v*v))+
     & u*(4*nf*(242-502*v+251*(v*v)+9*(v*v*v))-
     & 3*(4268-8876*v+4681*(v*v)-61*(v*v*v)+
     & 20*NFV*(-12+38*v-32*(v*v)+3*(v*v*v))+6*(v*v*v*v))
     & ))+omv*(u*u)*
     & (-4*nf*(161*(u*u*u*u*u)+
     & u*(3495-7128*v+4545*(v*v)-906*(v*v*v)-
     & 6*(v*v*v*v))+
     & 3*(-466+1142*v-872*(v*v)+191*(v*v*v)+5*(v*v*v*v)))
     & +3*(10*NFV*(120-394*v+445*(v*v)-191*(v*v*v)+
     & u*(-204+584*v-518*(v*v)+137*(v*v*v))+24*(v*v*v*v)
     & )+u*(59559-121032*v+75997*(v*v)-14708*(v*v*v)-
     & 4662*zeta2*(-5+10*v-6*(v*v)+v*v*v)+
     & 92*(v*v*v*v)-2*(v*v*v*v*v))))+
     & u*u*u*u*(4*nf*(-1+v)*
     & (-4678+7377*v+3*(-412+141*v)*(u*u)-3039*(v*v)+
     & u*(3414-3354*v+357*(v*v))+89*(v*v*v))-
     & 3*(77092-54524*u-196777*v+108858*u*v+19158*(u*u)-
     & 27159*v*(u*u)-2273*(u*u*u)+2449*v*(u*u*u)-
     & 98*(u*u*u*u)+34*v*(u*u*u*u)+168474*(v*v)-
     & 61597*u*(v*v)+7607*(u*u)*(v*v)+76*(u*u*u)*(v*v)-
     & 51402*(v*v*v)+7557*u*(v*v*v)+22*(u*u)*(v*v*v)+
     & 1554*(-1+v)*zeta2*
     & (-20+30*v+3*(-2+v)*(u*u)+u*u*u-12*(v*v)+
     & 3*u*(5-5*v+v*v)+v*v*v)+2585*(v*v*v*v)-
     & 50*u*(v*v*v*v)-
     & 10*NFV*(75-382*v+9*(-3+v)*(u*u*u*u)+499*(v*v)+
     & 3*(u*u*u)*(46-35*v+7*(v*v))-209*(v*v*v)+
     & u*u*(-264+311*v-110*(v*v)+9*(v*v*v))+
     & u*(186-227*v+77*(v*v)-9*(v*v*v)-9*(v*v*v*v))+
     & 23*(v*v*v*v)-6*(v*v*v*v*v))-32*(v*v*v*v*v))))/
     & (108._dp*omv*u*(omu*omu*omu)*(w*w*w))+(5*(H1(1)*H1(1)))/6._dp)-
     & (H1(1)*H1(1))/6._dp-(H1(1)*H1(1)*H1(1))/18._dp)+
     & G1(2)*(G2(3,0)*(-4._dp/3._dp-(8*H1(0))/9._dp-(8*H1(1))/9._dp)+
     & G2(0,2)*(-1._dp/6._dp-H1(0)/9._dp-H1(1)/9._dp)+
     & G2(2,0)*(-1._dp/6._dp-H1(0)/9._dp-H1(1)/9._dp)+
     & G2(1,2)*((omv+2*u*(-1+u+v))/(6._dp*(u*u))+
     & ((omv+2*u*(-1+u+v))*H1(0))/(9._dp*(u*u))+
     & ((omv+2*u*(-1+u+v))*H1(1))/(9._dp*(u*u)))+
     & ((omv+30*NFV*(-2+u)*(-1+v)+8*u*(-1+2*u+v))*H3(0,0,0))/
     & (9._dp*(u*u))+(H3(0,1,0)*(omv*(-7-30*NFV*(-2+u)+20*u)-
     & 190*(u*u)))/(9._dp*(u*u))+
     & (H3(1,1,0)*(26*omv-52*omv*u-98*(u*u)))/(9._dp*(u*u))+
     & (2*H3(0,0,1)*(9-18*omv*u-9*v+5*(u*u)))/(9._dp*(u*u))+
     & (H3(1,0,0)*(3*omv*(5+10*NFV*(-2+u)-8*u)+20*(u*u)))/(9._dp*(u*u))+
     & (H3(1,0,1)*(omv*(11+30*NFV*(-2+u)-22*u)+30*(u*u)))/(9._dp*(u*u))+
     & H1(1)*((8*H2(0,0))/9._dp-(10*H2(1,0))/9._dp+
     & (6*zeta2*(omu*omu)*(-30*omv*(-1+2*NFV*(-2+u)+2*u)-
     & 289*(u*u))+2*u*
     & (-3*omv*(107+80*NFV*(-1+2*u))+
     & 3*u*(126-214*v+u*(67+80*NFV+212*v-120*NFV*v)-
     & 86*(u*u))+4*nf*
     & (3-3*v+u*(-1+6*v)-(7+6*v)*(u*u)+5*(u*u*u))))/
     & (108._dp*(omu*omu)*(u*u)))+
     & (H2(1,0)*(435+80/omv+20/omw-2/u-(1140*v)/omu-(28*v)/u+
     & 4/w-(4*u)/w+(3606*v)/(omu*omu)-(5058*v)/(omu*omu*omu)+
     & (2529*v)/(omu*omu*omu*omu)+10/(omw*omw)-8/(u*u)+
     & (8*v)/(u*u)+10*NFV*
     & ((omv*(-4+11*u))/(u*u)-
     & (4-8*u+6*u*v+4*(u*u))/(omu*omu))+
     & (4*nf*(omv-u*(6-6*v+7*u*(-1+2*v)+(-12+11*v)*(u*u)+
     & (33+36*v)*(u*u*u)+(-26+25*v)*(u*u*u*u)+
     & 7*(u*u*u*u*u))))/(omu*omu*omu*omu*(u*u))))/18._dp+
     & (H2(0,1)*(-26+41/omv-20/omw-3*v-(18*v)/omu+
     & (20*NFV*(-1+4*v))/omv+(18*v)/(omu*omu)+
     & (2*(-16+45*NFV)*u*v)/(omv*omv)-10/(omw*omw)+
     & (2*(-17+6*v+10*NFV*(5+v)-v*v))/u-
     & (omv*(-6+20*NFV+20*v+v*v))/(u*u)))/18._dp+
     & (H2(0,0)*((-49+32*u)/omv+
     & (-32*u+10*NFV*(-2+(10+9*u)*v-8*(v*v)))/(omv*omv)+
     & (omv*(12*nf+omu*omu*omu*omu*(-164-20*NFV+20*v+v*v)))/
     & (omu*omu*omu*omu*(u*u))+
     & (5058*omu*u*v*w-3658*u*v*w*(omu*omu)+
     & 1066*u*v*w*(omu*omu*omu)+
     & 3*w*(-843*u*v+4*nf*
     & (u*(19-14*v)+6*(-1+v)+(-36+25*v)*(u*u)+
     & 39*(u*u*u)+11*(-2+v)*(u*u*u*u)+5*(u*u*u*u*u)))+
     & omu*omu*omu*omu*
     & (u*(-4-895*w+3*v*w)+4*(u*u)+
     & 2*w*(179+125*NFV-196*v-65*NFV*v+v*v)))/
     & (u*w*(omu*omu*omu*omu))))/18._dp+
     & ((4*nf*(18*omv*zeta2+u*
     & (omu*omu*(57-86*u-57*v+114*u*v+u*u-96*v*(u*u)+
     & 28*(u*u*u))-
     & 18*zeta2*(6-6*v+u*(-5+14*v)+(-20+11*v)*(u*u)+
     & 9*(5+4*v)*(u*u*u)+(-34+25*v)*(u*u*u*u)+
     & 9*(u*u*u*u*u)))))/(omu*omu*omu*omu*(u*u))+
     & 3*(-34-498/omu-1215/u-(1461*v)/omu+(1215*v)/u-
     & 2556*zeta3+(1296*zeta3)/u-(1296*v*zeta3)/u+
     & (1461*v)/(omu*omu)+
     & 10*NFV*(-81+60/omu+(72*v)/omu-6/w+
     & (6*(5+6*v*(-1+zeta3)-6*zeta3))/u-(72*v)/(omu*omu)+
     & 6*zeta2*(-24/omu-6/omv+(-59+28*u+47*v)/u-
     & (3*u*(1+7*u)*v)/(omu*omu*omu)-(9*u*v)/(omv*omv)-
     & (2*omv)/(u*u))+(72*omv*zeta3)/(u*u))-
     & (612*zeta3)/(u*u)+(612*v*zeta3)/(u*u)+
     & 6*zeta2*(812+39/omv+60/omw+4/w-
     & (4*u*(-8*w+8*omv*w+omv*omv))/(w*(omv*omv))+
     & 30/(omw*omw)+
     & (3*v*(-47+533*u-81*(u*u)+437*(u*u*u)+u*u*u*u))/
     & (omu*omu*omu*omu)+(-15+7*v+2*(v*v))/u-
     & (14-34*v+19*(v*v)+v*v*v)/(u*u))))/324._dp+
     & G1(3)*(H1(0)*(4._dp/3._dp+(8*H1(1))/9._dp)+(8*(H1(0)*H1(0)))/9._dp)+
     & ((H1(1)*(-2+4*u+2*v-4*u*v+11*(u*u)))/(18._dp*(u*u))-
     & (2*omv*nf*(omv+u*(-2+u+2*v))*(omu*omu)-57*(omv*omv)+
     & (-105+113*v)*(u*u*u*u)+u*u*(-356+619*v-255*(v*v))+
     & u*u*u*(307-407*v+84*(v*v))+u*(211-424*v+213*(v*v)))/
     & (18._dp*omv*(omu*omu)*(u*u)))*(H1(0)*H1(0))+
     & (17*(omv+u*(-2+3*u+2*v))*(H1(0)*H1(0)*H1(0)))/(18._dp*(u*u))+
     & ((-3*omv+u*(-27+63*u-6*v+6*u*v+2*nf*((-1+u)*(-1+u))-
     & 33*(u*u)))*(H1(1)*H1(1)))/(18._dp*u*(omu*omu))+
     & H1(0)*((8*H2(0,0))/9._dp-(10*H2(1,0))/9._dp+
     & (H1(1)*(-3*(omv*omv)+4*omv*nf*((-1+u)*(-1+u))*(u*u)+
     & u*(8-14*v+8*(-6+5*v)*(u*u*u)+
     & u*u*(98-76*v-6*(v*v))+u*(-55+50*v-3*(v*v))+
     & 6*(v*v))))/(18._dp*omv*(omu*omu)*(u*u))+
     & (12*omu*omv*omw*(-2+30*NFV-51*v)*(u*u)*(w*w)+
     & 4*omv*omw*(omv*nf*(19-64*u)+
     & u*u*(nf*(110-71*v+2*u*(-52+7*v)+39*(u*u))+
     & 3*v*(51+5*NFV*(3-4*u+3*(u*u)))))*(w*w)+
     & omu*omu*(1440*omw*NFV*(-2+u)*zeta2*(omv*omv)*(w*w)+
     & 12*omw*(15+(16-45*NFV)*u)*(u*u)*(w*w)-
     & omv*(-60*u*(w*w)+
     & omw*(w*((-108+w*(1971+78*v+3810*zeta2))*(u*u)-
     & 6*(-9+17*w)*(u*u*u)-
     & 3*w*(-391+371*v+42*(-1+v)*zeta2+2*(v*v))+
     & 6*u*(9+w*(-241+151*v+54*(-1+v)*zeta2+v*v)))
     & +30*NFV*(2*w*(-2+(-1+6*v)*w)+4*(u*u*u*u)+
     & u*u*u*(-4+6*w+9*(w*w))+
     & 2*(u*u)*(2-6*w+13*(w*w))+
     & u*(-4+18*w+5*(-5+2*v)*(w*w)))))))/
     & (108._dp*omv*omw*(omu*omu)*(u*u)*(w*w))-
     & ((omv+u*(-2+5*u+2*v))*(H1(1)*H1(1)))/(18._dp*(u*u)))-
     & (H1(1)*H1(1)*H1(1))/18._dp)+H1(0)*
     & ((76._dp/3._dp-(8*nf)/9._dp)*H2(0,0)+(5*(-57+2*nf)*H2(1,0))/9._dp+
     & H1(1)*((-8*H2(0,0))/9._dp+(10*H2(1,0))/9._dp+
     & (12*nf*(omv*omv)-omv*u*
     & (378*zeta2*((-1+u)*(-1+u))+94*nf*(u*u))-
     & 3*(-469+445*v)*(u*u*u)-18*(19-37*v+18*(v*v))+
     & 8*(u*u)*(nf*(25-28*v+3*(v*v))-9*(44-51*v+9*(v*v)))+
     & u*(-2*nf*(59-71*v+12*(v*v))+3*(701-893*v+216*(v*v))))/
     & (108._dp*omv*u*(omu*omu)))+
     & ((-12*nf*(87+13*v)+9*(927+v*(487+2502*zeta2)))/omu+
     & (162*v*zeta2*(-281+12*nf*u*(1+u+u*u)))/(omu*omu*omu*omu)+
     & (108*zeta2*(843*omv*v+
     & 5*NFV*(-8*v+8*(u*u*u)+u*(8+17*v-v*v)-
     & u*u*(16+v+7*(v*v)))))/(omv*(omu*omu*omu))+
     & (3*v*(30*NFV*((-73+63*v)*(u*u*u)+27*(u*u*u*u)+
     & 2*(5-11*v+6*(v*v))-3*u*(13-21*v+8*(v*v))+
     & u*u*(75-104*v+36*(v*v)))+
     & u*(-1461+52*nf-21816*zeta2)*(w*w)))/(u*(omu*omu)*(w*w))+
     & (24*omv*u*(567-50*nf+6*(29-16*u+45*NFV*u)*zeta2)*
     & (omw*omw)*(w*w*w)+
     & 144*(16-45*NFV)*zeta2*(omw*omw)*(u*u)*(w*w*w)-
     & omv*omv*(36*omw*u*(-20+(-70+9*nf)*u)*(w*w*w)+
     & 720*(u*u)*(w*w*w)+
     & omw*omw*(w*(w*(3*(1215-76*nf)*v*w+
     & u*(3321+83310*w-4*nf*(81+1406*w)-
     & 17028*w*zeta3)-54*(-79+12*nf)*(u*u))+
     & 18*zeta2*(u*w*(-31+(-235+68*nf+135*v)*w)+
     & (-35+66*w)*(u*u)+35*(u*u*u)+(68-47*v)*(w*w)))
     & +360*NFV*zeta2*
     & (5*w*(u*u)+5*(-1+2*v)*(u*u*u)+5*(u*u*u*u)+
     & 24*v*(w*w*w)))))/(u*(omv*omv)*(omw*omw)*(w*w*w)))/
     & 324._dp+((17+4/omv-2*nf+(2-3*v)/u+(3*v)/omu-
     & (3*v)/(omu*omu))*(H1(1)*H1(1)))/18._dp+(H1(1)*H1(1)*H1(1))/18._dp)+
     & ((3-3*v+u*(27-2*nf+6*v)+(-63+4*nf-6*v)*(u*u)+
     & (33-2*nf)*(u*u*u))*(H1(1)*H1(1)*H1(1)))/(54._dp*u*(omu*omu))+
     & G1(1)*(-((omv+30*NFV*(-2+u)*(-1+v)+4*u*(-2+49*u+2*v))*
     & H3(0,0,0))/(9._dp*(u*u))-
     & ((omv+30*NFV*(-2+u)*(-1+v)+4*u*(-2+49*u+2*v))*H3(1,0,0))/
     & (9._dp*(u*u))+(H3(0,0,1)*(-2+4*u+2*v-4*u*v-4*(u*u)))/(u*u)+
     & (H3(1,1,0)*(omv*(-11+30*NFV*(-2+u)+16*u)+128*(u*u)))/
     & (9._dp*(u*u))+(H3(0,1,0)*(7+30*omv*NFV*(-2+u)-20*omv*u-7*v+
     & 164*(u*u)))/(9._dp*(u*u))+
     & (H2(1,1)*(81-302*u-81*v+302*u*v-
     & 4*nf*(omv+2*u*(-1+u+v))*(omu*omu)+484*(u*u)-
     & 361*v*(u*u)-386*(u*u*u)+174*v*(u*u*u)-
     & 10*NFV*(19*omv*u+4*(-1+v)+(-30+26*v)*(u*u)+
     & (19-17*v)*(u*u*u)-4*(u*u*u*u))+123*(u*u*u*u)))/
     & (18._dp*(omu*omu)*(u*u))+
     & (H2(1,0)*(-71*(-1+v)+333*u*(-1+v)+(624-453*v)*(u*u)-
     & 5*(omu*omu)*(omv*(nf+2*NFV*u)+
     & 2*(nf*u*(-1+u+v)-2*NFV*(1-v+u*u)))+
     & (-533+227*v)*(u*u*u)+171*(u*u*u*u)))/(9._dp*(omu*omu)*(u*u))+
     & (H1(1)*(-6*omu*omv*omw*(16+90*NFV-69*v)*(u*u)*(w*w)-
     & 6*omv*omw*v*(u*u)*(69-4*nf*u+30*NFV*(1+u*u))*(w*w)+
     & omu*omu*(4*omw*(nf*(-19+32*u)-360*NFV*(-2+u)*zeta2)*
     & (omv*omv)*(w*w)+
     & 12*omw*(-16+45*NFV)*(-1+u)*(u*u)*(w*w)+
     & omv*(60*(-1+u)*u*(w*w)+
     & omw*(w*(w*(1083-116*nf+78*v+3132*zeta2)*(u*u)-
     & 6*(-9+17*w)*(u*u*u)+
     & w*(-150*(-1+v)*zeta2-3*(-391+371*v+2*(v*v)))+
     & 2*u*(27+
     & w*(186*(-1+v)*zeta2+3*(-293+268*v+v*v))))+
     & 30*NFV*(2*w*(-2+(-1+6*v)*w)+2*w*(1+22*w)*(u*u)+
     & 3*w*(2+3*w)*(u*u*u)+4*(u*u*u*u)-
     & u*(4-14*w+3*(7+2*v)*(w*w))))))))/
     & (108._dp*omv*omw*(omu*omu)*(u*u)*(w*w))-
     & (H2(0,0)*(-164+676*v+3*(134+9*v)*(u*u*u*u*u)-1053*(v*v)+
     & u*u*u*u*(-1592+1553*v+81*(v*v))+734*(v*v*v)+
     & u*u*u*(2528-4921*v+2321*(v*v)+81*(v*v*v))-193*(v*v*v*v)+
     & 10*NFV*((-7+10*v)*(u*u*u*u)+2*(u*u*u*u*u)+
     & u*u*u*(11-26*v+18*(v*v))+
     & u*u*(-11+30*v-27*(v*v)+10*(v*v*v))+
     & u*(7-22*v+27*(v*v)-14*(v*v*v)+2*(v*v*v*v))-
     & 2*(1-4*v+6*(v*v)-5*(v*v*v)+2*(v*v*v*v)))+
     & 12*omv*nf*(w*w*w)+
     & u*(-1+v)*(-878+2226*v-1787*(v*v)+430*(v*v*v)+
     & 24*nf*(w*w*w))+u*u*
     & (-2052+5769*v-5344*(v*v)+1600*(v*v*v)+27*(v*v*v*v)+
     & 24*nf*(w*w*w))))/(18._dp*(u*u)*(w*w*w))+
     & (H2(0,1)*(103-432*v-3*(61+9*v)*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*u*(1059-562*v-81*(v*v))+687*(v*v)-490*(v*v*v)-
     & u*u*u*u*u*(2653-3261*v+662*(v*v)+81*(v*v*v))+
     & u*u*u*(-3385+8779*v-7488*(v*v)+2181*(v*v*v)-
     & 96*(v*v*v*v))+u*u*u*u*
     & (3785-7296*v+3893*(v*v)-379*(v*v*v)-27*(v*v*v*v))+
     & 132*(v*v*v*v)-10*NFV*
     & (5*(-4+5*v)*(u*u*u*u*u*u)+2*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(72-136*v+63*(v*v))+
     & u*u*u*u*(-130+308*v-234*(v*v)+55*(v*v*v))+
     & u*u*(-72+235*v-282*(v*v)+147*(v*v*v)-26*(v*v*v*v))-
     & 2*(1-4*v+6*(v*v)-5*(v*v*v)+2*(v*v*v*v))+
     & u*u*u*(130-366*v+360*(v*v)-142*(v*v*v)+17*(v*v*v*v))+
     & u*(20-74*v+105*(v*v)-70*(v*v*v)+19*(v*v*v*v)))-
     & 6*omv*nf*(omu*omu)*(w*w*w)+
     & u*u*(1933-6174*v+7037*(v*v)-3277*(v*v*v)+481*(v*v*v*v)-
     & 12*nf*(omu*omu)*(w*w*w))-
     & u*(-1+v)*(-659+1792*v-1594*(v*v)+452*(v*v*v)+
     & 12*nf*(omu*omu)*(w*w*w))))/(18._dp*(omu*omu)*(u*u)*(w*w*w))+
     & (-12*zeta3*(omu*omu)*(omv*(-11+30*NFV*(-2+u)+16*u)+
     & 128*(u*u))*(w*w*w)+
     & 6*zeta2*((429-108*v)*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*u*(-2269+2661*v-324*(v*v))+
     & u*u*u*u*u*(5039-10485*v+5113*(v*v)-324*(v*v*v))+
     & u*u*u*u*(-6175+18198*v-15646*(v*v)+3923*(v*v*v)-
     & 108*(v*v*v*v))+
     & u*u*u*(4675-17350*v+20850*(v*v)-9145*(v*v*v)+
     & 1042*(v*v*v*v))+
     & 10*NFV*((-65+61*v)*(u*u*u*u*u*u)+20*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(45-112*v+63*(v*v))+
     & u*u*u*u*(74-70*v-51*(v*v)+39*(v*v*v))+
     & u*u*(99-287*v+255*(v*v)-45*(v*v*v)-6*(v*v*v*v))+
     & 4*(1-4*v+6*(v*v)-3*(v*v*v*v))+
     & u*u*u*(-146+312*v-165*(v*v)-18*(v*v*v)+9*(v*v*v*v))+
     & u*(-31+112*v-126*(v*v)+24*(v*v*v)+21*(v*v*v*v)))-
     & (-1+v)*(-129+547*v-779*(v*v)+361*(v*v*v)-
     & 20*nf*(omu*omu)*(w*w*w))-
     & u*u*(2339-10065*v+14920*(v*v)-8945*(v*v*v)+
     & 1751*(v*v*v*v)+16*nf*(omu*omu)*(w*w*w))+
     & u*(769-3657*v+6253*(v*v)-4539*(v*v*v)+1174*(v*v*v*v)+
     & 40*omv*nf*(omu*omu)*(w*w*w))))/
     & (108._dp*(omu*omu)*(u*u)*(w*w*w))+
     & (((omv+2*u*(-1+u+v))*H1(0))/(18._dp*(u*u))+
     & ((omv+2*u*(-1+u+v))*H1(1))/(18._dp*(u*u)))*(G1(2)*G1(2))+
     & (((-57+2*nf)*(omv+2*u*(-1+u+v)))/(18._dp*(u*u))-
     & (5*(omv+2*u*(-1+u+v))*H1(1))/(6._dp*(u*u)))*(H1(0)*H1(0))-
     & (17*(omv+2*u*(-1+u+v))*(H1(0)*H1(0)*H1(0)))/(18._dp*(u*u))+
     & ((omv+2*u*(-1+u+v))*(H1(1)*H1(1)))/(6._dp*(u*u))+
     & G1(2)*(H1(0)*(-(omv+2*u*(-1+u+v))/(6._dp*(u*u))-
     & (2*(omv+2*u*(-1+u+v))*H1(1))/(9._dp*(u*u)))-
     & ((omv+2*u*(-1+u+v))*H1(1))/(6._dp*(u*u))-
     & ((omv+2*u*(-1+u+v))*(H1(0)*H1(0)))/(9._dp*(u*u))-
     & ((omv+2*u*(-1+u+v))*(H1(1)*H1(1)))/(9._dp*(u*u)))+
     & H1(0)*(((-27+nf)*(omv+2*u*(-1+u+v))*H1(1))/(9._dp*(u*u))+
     & (-6*omu*omv*omw*(16+90*NFV-69*v)*(u*u)*(w*w)-
     & 6*omv*omw*v*(u*u)*(69-4*nf*u+30*NFV*(1+u*u))*(w*w)+
     & omu*omu*(4*omw*(nf*(-19+32*u)-360*NFV*(-2+u)*zeta2)*
     & (omv*omv)*(w*w)+
     & 12*omw*(-16+45*NFV)*(-1+u)*(u*u)*(w*w)+
     & omv*(60*(-1+u)*u*(w*w)+
     & omw*(w*(w*(1083-116*nf+78*v+2268*zeta2)*(u*u)-
     & 6*(-9+17*w)*(u*u*u)+
     & w*(282*(-1+v)*zeta2-3*(-391+371*v+2*(v*v)))+
     & u*(54+
     & w*(-492*(-1+v)*zeta2+6*(-293+268*v+v*v))))
     & +30*NFV*(2*w*(-2+(-1+6*v)*w)+
     & 2*w*(1+22*w)*(u*u)+3*w*(2+3*w)*(u*u*u)+
     & 4*(u*u*u*u)-u*(4-14*w+3*(7+2*v)*(w*w)))))))/
     & (108._dp*omv*omw*(omu*omu)*(u*u)*(w*w))+
     & ((omv+2*u*(-1+u+v))*(H1(1)*H1(1)))/(6._dp*(u*u)))+
     & ((omv+2*u*(-1+u+v))*(H1(1)*H1(1)*H1(1)))/(18._dp*(u*u)))+
     & (H1(1)*H1(1)*H1(1)*H1(1))/72._dp
      return
      end
      
      
      function Beta_4a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-16*pi*G3(0,0,1))/9._dp+(2*pi*G3(0,0,2))/9._dp-16*pi*G3(0,0,3)+
     & (2*pi*G3(0,2,0))/9._dp+(8*pi*G3(0,2,1))/9._dp+(2*pi*G3(0,2,2))/9._dp+
     & (188*pi*G3(0,2,3))/9._dp+(8*pi*G3(0,3,0))/9._dp+(8*pi*G3(0,3,2))/9._dp+
     & (128*pi*G3(0,3,3))/9._dp-(20*pi*G3(1,0,1))/9._dp+(2*pi*G3(2,0,0))/9._dp+
     & (2*pi*G3(2,0,2))/9._dp+(2*pi*G3(2,2,0))/9._dp-(10*pi*G3(2,3,0))/9._dp-
     & (26*pi*G3(2,3,2))/9._dp+(16*pi*G3(3,0,0))/9._dp+(10*pi*G3(3,0,1))/9._dp+
     & (16*pi*G3(3,0,2))/9._dp+(208*pi*G3(3,0,3))/9._dp+(16*pi*G3(3,2,0))/9._dp+
     & (10*pi*G3(3,2,1))/9._dp-(80*pi*G3(3,2,3))/9._dp+(128*pi*G3(3,3,0))/9._dp-
     & (128*pi*G3(3,3,3))/9._dp+G2(0,0)*
     & (((-21+nf)*pi)/3._dp+(16*pi*H1(0))/9._dp-(2*pi*H1(1))/9._dp)-
     & (64*pi*H3(0,0,0))/3._dp+(8*pi*H3(0,0,1))/9._dp+(98*pi*H3(0,1,0))/9._dp+
     & (178*pi*H3(1,0,0))/9._dp-(10*pi*H3(1,0,1))/9._dp-
     & (2*pi*(omv+2*u*(-1+u+v))*G3(0,1,0))/(u*u)-
     & (pi*(omv-2*(u*(1+8*u-v)+15*NFV*(-2+u)*(-1+v)))*G3(1,0,2))/
     & (9._dp*(u*u))-(pi*(omv-2*(u*(1+8*u-v)+15*NFV*(-2+u)*(-1+v)))*
     & G3(1,2,0))/(9._dp*(u*u))-(pi*
     & (omv-2*(u*(1+8*u-v)+15*NFV*(-2+u)*(-1+v)))*G3(1,2,2))/
     & (9._dp*(u*u))+(pi*(omv+2*u*(-1+u+v))*G3(2,1,0))/(u*u)-
     & (4*pi*(omv+2*u*(-1+u+v))*G3(2,1,1))/(u*u)+
     & (pi*(omv+2*u*(-1+u+v))*G3(2,1,2))/(u*u)-
     & (pi*(omv+2*(5*NFV*(-2+u)*(-1+v)+2*u*(-1+3*u+v)))*G3(2,2,3))/
     & (3._dp*(u*u))-(20*pi*(omv+2*u*(-1-4*u+v))*G3(2,3,3))/(9._dp*(u*u))+
     & (pi*G3(1,0,3)*(omv*(17+30*NFV*(-2+u)-28*u)-160*(u*u)))/
     & (9._dp*(u*u))+(pi*G3(1,2,3)*(omv*(17+30*NFV*(-2+u)-28*u)-
     & 160*(u*u)))/(9._dp*(u*u))+
     & (pi*G3(1,2,1)*(15*omv*(-1+2*NFV*(-2+u)+2*u)-32*(u*u)))/
     & (9._dp*(u*u))+(pi*G3(0,1,2)*(-9+18*omv*u+9*v-20*(u*u)))/(9._dp*(u*u))+
     & (4*pi*G3(1,1,2)*(-9+18*omv*u+9*v-13*(u*u)))/(9._dp*(u*u))-
     & (pi*G3(2,2,1)*(3+30*omv*NFV*(-2+u)+6*u*(-1+v)-3*v+4*(u*u)))/
     & (9._dp*(u*u))+(pi*G3(2,0,1)*(omv*(11+30*NFV*(-2+u)-22*u)+
     & 18*(u*u)))/(9._dp*(u*u))-
     & (pi*G3(1,1,0)*(3*omv*(7+10*NFV*(-2+u)-14*u)+20*(u*u)))/
     & (9._dp*(u*u))-(pi*G3(0,1,1)*(-9+18*omv*u+9*v+20*(u*u)))/(9._dp*(u*u))+
     & (pi*G3(2,0,3)*(3-6*omv*(5*NFV*(-2+u)+2*u)-3*v+98*(u*u)))/
     & (9._dp*(u*u))+(pi*G3(1,1,1)*(3*omv*(19+10*NFV*(-2+u)-38*u)+
     & 112*(u*u)))/(9._dp*(u*u))+
     & G2(0,1)*((pi*(134-41/omv-20/omw+14/u-(32*u)/omv+(44*v)/omu+
     & (10*v)/u+(19*v)/(omu*omu)+(32*u)/(omv*omv)-
     & 10/(omw*omw)+30*NFV*
     & (-4/u-(3*u*v)/(omv*omv)+
     & (v*(-2+u*(3+v)-2*(u*u)))/(omv*(omu*omu)))-16/(u*u)+
     & (16*v)/(u*u)-(nf*(omv+2*u*(-1+3*u+v)))/(u*u)))/18._dp+
     & (pi*H1(1)*(9-18*omv*u-9*v+20*(u*u)))/(9._dp*(u*u))+
     & (pi*H1(0)*(9-18*omv*u-9*v+38*(u*u)))/(9._dp*(u*u)))-
     & (pi*H2(1,1)*(10*NFV*(12*omv*(-1+3*u)+
     & (-40+u*(20-17*v)+37*v-4*(u*u))*(u*u))+
     & omu*(2*omv+u*(59+v+2*u*(-62+9*v)-
     & 4*nf*((-1+u)*(-1+u))+63*(u*u)))))/(18._dp*u*(omu*omu*omu))
     & +G2(2,2)*(-(pi*(omv+30*NFV*(-2+u)*(-1+v)+8*u*(-1+2*u+v))*
     & H1(0))/(9._dp*(u*u))-(pi*
     & (10*NFV*(12*omv*(-1+3*u)+
     & (-40+u*(20-17*v)+37*v-4*(u*u))*(u*u))+
     & omu*(2*omv+u*(59+v+2*u*(-62+9*v)-
     & 4*nf*((-1+u)*(-1+u))+63*(u*u)))))/
     & (18._dp*u*(omu*omu*omu)))+
     & G2(2,0)*((-2*pi*H1(1))/9._dp-(pi*H1(0)*
     & (10*omv*(1+3*NFV*(-2+u)-2*u)+21*(u*u)))/(9._dp*(u*u))-
     & (pi*(10*NFV*(12*omv*(-1+3*u)+
     & (-40+20*u+37*v-17*u*v-4*(u*u))*(u*u))+
     & omu*(2*omv+u*(116-238*u+v+18*u*v-
     & 5*nf*((-1+u)*(-1+u))+120*(u*u)))))/
     & (18._dp*u*(omu*omu*omu)))+
     & G2(0,2)*((-2*pi*H1(1))/9._dp+(pi*H1(0)*(-3+6*u+3*v-6*u*v+53*(u*u)))/
     & (3._dp*(u*u))-(pi*(10*NFV*
     & (12*omv*(-1+3*u)+
     & (-40+20*u+37*v-17*u*v-4*(u*u))*(u*u))+
     & omu*(2*omv+u*(116-238*u+v+18*u*v-
     & 5*nf*((-1+u)*(-1+u))+120*(u*u)))))/
     & (18._dp*u*(omu*omu*omu)))+
     & G2(2,3)*(12*pi*H1(0)+(26*pi*H1(1))/9._dp+
     & (pi*((39-32*u)/omv+(60*NFV*(-1+v)*v+u*(32-90*NFV*v))/
     & (omv*omv)+(-5058*omu*u*v*w+3640*u*v*w*(omu*omu)-
     & 1048*u*v*w*(omu*omu*omu)+
     & omu*omu*omu*omu*
     & (2*(14+60*NFV*(-2+v)-17*v)*w+u*(4+527*w)-4*(u*u))
     & -9*u*w*(-281*v+
     & 4*nf*(1+u*(-4+3*v)+3*(2+v)*(u*u)+
     & (-4+3*v)*(u*u*u)+u*u*u*u)))/
     & (u*w*(omu*omu*omu*omu))))/18._dp)+
     & G2(2,1)*((pi*(omv+2*u*(-1+u+v))*H1(0))/(u*u)-
     & (pi*(omv+2*u*(-1+u+v))*H1(1))/(u*u)+
     & (pi*(97+41/omv-20/omw-174/u+(32*u)/omv-3*v-(52*v)/omu+
     & (152*v)/u+(52*v)/(omu*omu)-(32*u)/(omv*omv)-
     & 10/(omw*omw)+10*NFV*
     & (-4+6/omv+(-1+13*v)/u+(6*u*v)/(omu*omu)+
     & (9*u*v)/(omv*omv)+(2-2*v)/(u*u))+87/(u*u)-
     & (107*v)/(u*u)-(4*nf*(omv+2*u*(-1+u+v)))/(u*u)-
     & (2*(v*v))/u+(19*(v*v))/(u*u)+(v*v*v)/(u*u)))/18._dp)+
     & (pi*H2(1,0)*(-432-31/omv-20/omw-16/u+(32*u)/omv+
     & (1114*v)/omu+(12*v)/u+82/w-(162*u)/w-(3640*v)/(omu*omu)+
     & (5058*v)/(omu*omu*omu)-(2529*v)/(omu*omu*omu*omu)+
     & 10*NFV*(-10+4/omu+6/omv-(12*(-2+v))/u+
     & (2*u*(1+u)*v)/(omu*omu*omu)+(9*u*v)/(omv*omv))-
     & (32*u)/(omv*omv)-10/(omw*omw)+
     & (6*nf*(1+2*u*(-2+9*v)+6*(1+3*v)*(u*u)+
     & 2*(-2+9*v)*(u*u*u)+u*u*u*u))/(omu*omu*omu*omu)+
     & (80*u)/(w*w)-(80*(u*u))/(w*w)))/18._dp-
     & (pi*H1(1)*(6*omv*u*w*(-20*NFV+w+52*v*w)*(omu*omu)-
     & 3*omu*omv*u*(40*NFV+(123-12*nf+8*nf*u)*v)*(w*w)+
     & 60*omv*NFV*u*v*(-3+16*u-4*(u*u)+3*(u*u*u))*(w*w)+
     & omu*omu*omu*(12*(-16+45*NFV)*(u*u)*(w*w)+
     & omv*(w*(u*(-54+w*(759-76*nf+78*v+30*zeta2))-
     & 6*(-9+17*w)*(u*u)+6*w*(62-72*v+v*v))+
     & 30*NFV*(4*w*(1-6*w+4*v*w)+3*w*(2+3*w)*(u*u)+
     & 4*(u*u*u)+u*(4-8*w+26*(w*w)))))))/
     & (108._dp*omv*u*(omu*omu*omu)*(w*w))+
     & G2(0,3)*((56*pi*H1(0))/9._dp-(8*pi*H1(1))/9._dp+
     & (pi*(-162-39/omv+27*v-(1048*v)/omu+(60*NFV*v)/omv+
     & (40-34*v+120*NFV*v)/u-75/w+(3640*v)/(omu*omu)-
     & (5058*v)/(omu*omu*omu)+(2529*v)/(omu*omu*omu*omu)+
     & (4*nf*(2-u*(8+27*v)-3*(-4+9*v)*(u*u)-
     & (8+27*v)*(u*u*u)+2*(u*u*u*u)))/(omu*omu*omu*omu)+
     & (u*((-71+20*NFV+146*w)*(omv*omv)+
     & 2*omv*(16-45*NFV)*(w*w)+2*(-16+45*NFV)*(w*w)))/
     & (omv*omv*(w*w))+((20*NFV*(-1+2*v)+71*w)*(u*u))/(w*w*w)+
     & (20*NFV*(u*u*u))/(w*w*w)))/18._dp)+
     & G2(3,0)*((16*pi*H1(0))/3._dp-(16*pi*H1(1))/9._dp-
     & (pi*((-456+8*nf+20*NFV+3*v)*(u*u*u*u*u)+
     & u*u*u*u*(1368-24*omv*nf-1337*v+10*NFV*(-4+7*v)+
     & 11*(v*v))+omv*(v*v)*
     & (10+4*(-7+5*NFV)*v+17*(v*v)+v*v*v)+
     & omv*u*v*(-20+10*(1+3*NFV)*v-2*(-9+5*NFV)*(v*v)+v*v*v)+
     & u*u*u*(-1368+2654*v+24*nf*(omv*omv)-1291*(v*v)+
     & 10*NFV*(2-8*v+9*(v*v))+14*(v*v*v))+
     & u*u*(456+10*(-130+3*NFV)*v-8*nf*(omv*omv*omv)+
     & (1249-60*NFV)*(v*v)+(-411+50*NFV)*(v*v*v)+6*(v*v*v*v))))
     & /(18._dp*(u*u)*(w*w*w)))+
     & G2(3,2)*((-98*pi*H1(0))/9._dp-(pi*
     & (10*NFV*((-4+7*v)*(u*u*u*u)+2*(u*u*u*u*u)+
     & u*(v*v)*(3-4*v+v*v)+v*(u*u)*(3-6*v+5*(v*v))+
     & u*u*u*(2-8*v+9*(v*v))+2*omv*(v*v*v))+
     & v*((31+11*v)*(u*u*u*u)+3*(u*u*u*u*u)-
     & v*(omv*omv)*(-10+18*v+v*v)+
     & u*u*u*(-82+77*v+14*(v*v))+
     & u*u*(68-119*v+45*(v*v)+6*(v*v*v))-
     & u*(20-30*v-8*(v*v)+17*(v*v*v)+v*v*v*v))))/
     & (18._dp*(u*u)*(w*w*w)))+G2(3,3)*
     & ((-128*pi*H1(0))/9._dp+(pi*(32*nf*(omv*omv*omv)*(u*u)-
     & 96*nf*(omv*omv)*(u*u*u)+
     & u*u*(-372+(910-30*NFV)*v+
     & (492-32*nf-20*NFV+3*v)*(u*u*u)+(-709+60*NFV)*(v*v)+
     & u*u*(-1436+NFV*(40-70*v)+1383*v+11*(v*v))+
     & (165-50*NFV)*(v*v*v)+
     & u*(1356-2560*v+1181*(v*v)-10*NFV*(2-8*v+9*(v*v))+
     & 14*(v*v*v))+6*(v*v*v*v))+
     & omv*(96*nf*(u*u*u*u)+
     & v*(-40+108*v-2*(43+10*NFV)*(v*v)+17*(v*v*v)+
     & v*v*v*v)+u*(-40+264*v-2*(188+15*NFV)*(v*v)+
     & 2*(71+5*NFV)*(v*v*v)+v*v*v*v))))/(18._dp*(u*u)*(w*w*w)))+
     & G2(1,2)*((pi*(omv-2*(u*(1+8*u-v)+15*NFV*(-2+u)*(-1+v)))*H1(1))/
     & (9._dp*(u*u))+(pi*H1(0)*(3*omv*(5+10*NFV*(-2+u)-8*u)-
     & 160*(u*u)))/(9._dp*(u*u))-
     & (pi*(w*(-75+245*v+(-186+4*nf+27*v)*(u*u*u*u*u*u)-274*(v*v)+
     & 2*(u*u*u*u*u)*(457+4*nf*(-2+v)-404*v+27*(v*v))+
     & 104*(v*v*v)+u*u*u*u*
     & (-1871+2834*v-1011*(v*v)+4*nf*(6-6*v+v*v)+
     & 27*(v*v*v))-2*(u*u*u)*
     & (-1032+2111*v-1257*(v*v)+4*nf*(2-3*v+v*v)+
     & 190*(v*v*v))-
     & 2*u*(-235+680*v-654*(v*v)+209*(v*v*v))+
     & u*u*(-1316+3284*v+4*nf*(omv*omv)-2591*(v*v)+
     & 614*(v*v*v)))+
     & 10*NFV*(3*(-9+7*v)*(u*u*u*u*u*u)+6*(u*u*u*u*u*u*u)+
     & 3*(u*u*u*u*u)*(17-25*v+9*(v*v))+
     & u*u*u*u*(-56+117*v-81*(v*v)+19*(v*v*v))+
     & u*u*(-27+82*v-93*(v*v)+48*(v*v*v)-8*(v*v*v*v))-
     & 2*(1-4*v+6*(v*v)-5*(v*v*v)+2*(v*v*v*v))+
     & u*u*u*(44-115*v+108*(v*v)-43*(v*v*v)+5*(v*v*v*v))+
     & u*(11-38*v+51*(v*v)-34*(v*v*v)+10*(v*v*v*v)))))/
     & (18._dp*(omu*omu)*(u*u)*(w*w*w)))+
     & (pi*H2(0,0)*(1158+39/omv-(32*u)/omv-27*v+(1132*v)/omu-
     & (6*v)/u+75/w-(146*u)/w-(3658*v)/(omu*omu)+
     & (5058*v)/(omu*omu*omu)-(2529*v)/(omu*omu*omu*omu)+
     & (32*u)/(omv*omv)+nf*
     & (-64+(108*u*v*(1+u+u*u))/(omu*omu*omu*omu))+
     & (71*u)/(w*w)-(71*(u*u))/(w*w)+
     & 10*NFV*((-9*u)/(omv*omv)+
     & (-6*v+2*(2+v)*(u*u*u)+
     & u*(4+16*v+9*(omu*omu*omu)-2*(v*v))-
     & 2*(u*u)*((2+v)*(2+v)))/(omv*(omu*omu*omu))-
     & (2*(w*(u*u)+(-1+2*v)*(u*u*u)+u*u*u*u+6*v*(w*w*w)))/
     & (u*(w*w*w)))))/18._dp+
     & (pi*H2(0,1)*(-55-33/omv+6*nf-(32*u)/omv-27*v+(2*v)/omu+
     & (20*v)/u-11/w+(20*u)/w+v/(omu*omu)+(32*u)/(omv*omv)-
     & (9*u)/(w*w)+(9*(u*u))/(w*w)+
     & 10*NFV*(4/omu+(-6+9*u)/omv+
     & (u*(-1+5*u)*v)/(omu*omu*omu)-(9*u)/(omv*omv)-
     & (2*(w*(u*u)+(-1+2*v)*(u*u*u)+u*u*u*u-u*(w*w*w)+
     & 6*v*(w*w*w)))/(u*(w*w*w)))))/18._dp+
     & G2(1,0)*((pi*(omv-2*(u*(1+8*u-v)+15*NFV*(-2+u)*(-1+v)))*H1(1))/
     & (9._dp*(u*u))+(2*pi*H1(0)*
     & (5+5*omv*(3*NFV*(-2+u)-2*u)-5*v+u*u))/(9._dp*(u*u))-
     & (pi*(-63+272*v-3*(-67+9*v)*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*u*(-1173+1030*v-81*(v*v))-447*(v*v)+
     & u*u*u*u*u*(2913-4606*v+1810*(v*v)-81*(v*v*v))+
     & 330*(v*v*v)+u*u*u*u*
     & (-4005+8955*v-6224*(v*v)+1325*(v*v*v)-27*(v*v*v*v))-
     & 92*(v*v*v*v)+u*u*u*
     & (3315-9396*v+9126*(v*v)-3380*(v*v*v)+344*(v*v*v*v))+
     & 10*NFV*(2-8*v+(4-8*v)*(u*u*u*u*u*u)+2*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(-48+83*v-36*(v*v))+12*(v*v)+
     & u*u*u*u*(114-247*v+168*(v*v)-36*(v*v*v))-6*(v*v*v)+
     & u*u*u*(-126+335*v-303*(v*v)+105*(v*v*v)-12*(v*v*v*v))+
     & u*(-20+74*v-99*(v*v)+54*(v*v*v)-9*(v*v*v*v))+
     & u*u*(72-229*v+258*(v*v)-117*(v*v*v)+18*(v*v*v*v)))+
     & omv*nf*(omu*omu)*(w*w*w)+
     & u*(-1+v)*(-483+1380*v-1298*(v*v)+392*(v*v*v)+
     & 2*nf*(omu*omu)*(w*w*w))+
     & u*u*(-1671+5635*v-6862*(v*v)+3496*(v*v*v)-598*(v*v*v*v)+
     & 6*nf*(omu*omu)*(w*w*w))))/(18._dp*(omu*omu)*(u*u)*(w*w*w)))+
     & G2(1,1)*((-20*pi*H1(0))/9._dp+(4*pi*H1(1)*(9-18*omv*u-9*v+13*(u*u)))/
     & (9._dp*(u*u))+(pi*(-39+136*v-3*(-53+9*v)*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*u*(-1033+918*v-81*(v*v))-165*(v*v)+
     & u*u*u*u*u*(2819-4257*v+1726*(v*v)-81*(v*v*v))+78*(v*v*v)+
     & u*u*u*u*(-4165+8616*v-5773*(v*v)+1325*(v*v*v)-
     & 27*(v*v*v*v))-10*(v*v*v*v)+
     & u*u*u*(3565-9217*v+8250*(v*v)-2965*(v*v*v)+
     & 358*(v*v*v*v))+10*NFV*
     & (-2+8*v-(2+11*v)*(u*u*u*u*u*u)+2*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(-18+68*v-45*(v*v))-12*(v*v)+
     & u*u*u*u*(50-160*v+156*(v*v)-45*(v*v*v))+6*(v*v*v)+
     & u*u*u*(-50+174*v-210*(v*v)+102*(v*v*v)-15*(v*v*v*v))+
     & u*(2-2*v-9*(v*v)+18*(v*v*v)-9*(v*v*v*v))+
     & u*u*(18-77*v+120*(v*v)-81*(v*v*v)+18*(v*v*v*v)))+
     & 4*omv*nf*(omu*omu)*(w*w*w)+
     & u*(-1+v)*(-433+1058*v-830*(v*v)+214*(v*v*v)+
     & 8*nf*(omu*omu)*(w*w*w))+
     & u*u*(-1739+5322*v-5845*(v*v)+2687*(v*v*v)-425*(v*v*v*v)+
     & 8*nf*(omu*omu)*(w*w*w))))/(18._dp*(omu*omu)*(u*u)*(w*w*w)))+
     & (pi*(-((12768+2529/w+v*(1461-12606*zeta2)+720*NFV*zeta2)/omu)+
     & (72*(843+5*NFV*(1-3*u)*u)*v*zeta2)/(omu*omu*omu)+
     & (108*v*zeta2*(-281+12*nf*u*(1+u+u*u)))/(omu*omu*omu*omu)+
     & (1164*omv*nf-3504*nf*u+2612*nf*u*v+7587*w+1461*v*w-
     & 43386*v*w*zeta2+2868*nf*(u*u)-476*nf*v*(u*u)-
     & 528*nf*(u*u*u)-30*NFV*
     & (-1+5*v+(1-15*v)*(u*u)+9*(u*u*u)+
     & u*(-9+34*v-24*(v*v)))+52*nf*u*(v*v))/(w*(omu*omu))+
     & (12*omv*u*(1+NFV*(60-90*u)+32*u)*zeta2*(omw*omw)*(w*w*w)+
     & 76*nf*(omv*omv*omv)*(omw*omw)*(w*w*w)+
     & 24*(-16+45*NFV)*zeta2*(omw*omw)*(u*u)*(w*w*w)+
     & omv*omv*(12*omw*u*(-20+(-70+9*nf)*u)*(w*w*w)+
     & 240*(u*u)*(w*w*w)+
     & omw*omw*(w*(3*w*
     & (405*(-1+v)*w+u*(843-4082*w+2324*w*zeta3)+
     & (474-72*nf)*(u*u))+
     & 6*zeta2*(u*w*(93-659*w+27*v*w)+(89-182*w)*(u*u)-
     & 89*(u*u*u)+(-61+37*v)*(w*w)))+
     & 10*NFV*(-3*w*((-7+9*v)*(u*u)+9*(u*u*u)+
     & 2*(-5+6*v)*(w*w))+
     & 12*zeta2*(w*(u*u)+(-1+2*v)*(u*u*u)+u*u*u*u+
     & 12*v*(w*w*w))))))/(u*(omv*omv)*(omw*omw)*(w*w*w))))
     & /108._dp+(-(pi*G1(2))/2._dp-4*pi*G1(3)+(3*pi*H1(0))/2._dp+(pi*H1(1))/2._dp+
     & (pi*(omv+2*u*(-1+u+v))*G1(1))/(2._dp*(u*u))+
     & (pi*(9*omv-u*(nf*((-1+u)*(-1+u))-
     & 3*(40-89*u+6*v-6*u*v+46*(u*u)))))/(18._dp*u*(omu*omu)))*
     & (G1(0)*G1(0))-(pi*(G1(0)*G1(0)*G1(0)))/2._dp+
     & ((4*pi*G1(3))/9._dp-(pi*H1(0))/6._dp-(pi*H1(1))/6._dp-
     & (pi*(omv+u*(13+2*v-u*(29+2*v)+15*(u*u))))/(18._dp*u*(omu*omu))
     & )*(G1(2)*G1(2))+(pi*(G1(2)*G1(2)*G1(2)))/18._dp+
     & ((5*pi*H1(1))/6._dp-(pi*(-17+16*v+(333-34*omv*nf-261*v)*(u*u*u)+
     & v*v-u*(-367+34*omv*nf+293*v+2*(v*v))+
     & u*u*(-683+68*omv*nf+537*v+2*(v*v))))/
     & (18._dp*omv*u*(omu*omu)))*(H1(0)*H1(0))+(17*pi*(H1(0)*H1(0)*H1(0)))/18._dp-
     & (pi*(omv+u*(13+2*v-u*(29+2*v)+15*(u*u)))*(H1(1)*H1(1)))/
     & (18._dp*u*(omu*omu))+H1(0)*(8*pi*H2(0,0)-10*pi*H2(1,0)-
     & (pi*H1(1)*(20*omv+u*(-97+134*u+40*v-40*u*v+
     & 2*nf*((-1+u)*(-1+u))-57*(u*u))))/(18._dp*u*(omu*omu))-
     & (pi*(3*omu*omv*omw*u*(2529+(10116-40*NFV+21*v)*w)*(w*w)+
     & 3*omv*omw*u*w*(omu*omu)*
     & (843-2*(2529+20*NFV)*w+8*(-901+15*NFV-5*v)*(w*w))-
     & 2*omw*w*(12*nf*(omv*omv*omv)+
     & 2*nf*u*(omv*omv)*(-20+2*v+9*(v*v))+
     & 2*nf*(u*u)*(-44+340*v+(-91+115*v)*(u*u*u*u)-509*(v*v)+
     & u*u*u*(179-541*v+290*(v*v))+228*(v*v*v)+
     & u*u*(-151+378*v-336*(v*v)+181*(v*v*v))-
     & 15*(v*v*v*v)+
     & u*(121-316*v+66*(v*v)+99*(v*v*v)+6*(v*v*v*v)))+
     & 3*omv*u*(2529-10*NFV*v*(-3+16*u-4*(u*u)+3*(u*u*u)))*
     & (w*w))-3*(omu*omu*omu)*
     & (4*omw*u*(-45*NFV*(1+u)+4*(25+4*u))*(w*w*w)+
     & omv*(40*NFV*(omw*omw)*(u*u*u)+20*u*(w*w*w)-
     & omw*(w*(2*w*(9+w*(43-81*v+v*v))+
     & u*u*(-492+578*w-34*(w*w))+
     & u*(-825+2730*w+(2221+26*v+90*zeta2)*(w*w)))+
     & 10*NFV*(4*w*(-1+4*w+(-3+4*v)*(w*w))+
     & u*(-4+22*w-32*(w*w)+8*(w*w*w))+
     & u*u*(2-11*w+6*(w*w)+9*(w*w*w))))))))/
     & (108._dp*omv*omw*u*(omu*omu*omu)*(w*w*w))-(pi*(H1(1)*H1(1)))/6._dp)+
     & G1(0)*(G1(3)*((-4*(-30+nf)*pi)/9._dp+8*pi*H1(0))+
     & G1(2)*(-((-27+nf)*pi)/18._dp+(8*pi*H1(0))/9._dp-(pi*H1(1))/9._dp)+
     & (40*pi*H2(0,0))/3._dp-(16*pi*H2(0,1))/9._dp-(2*pi*H2(1,0))/3._dp+
     & (2*pi*H2(1,1))/9._dp+G1(1)*(((-30+nf)*pi*(omv+2*u*(-1+u+v)))/
     & (18._dp*(u*u))-(pi*(omv+2*u*(-1+u+v))*H1(0))/(u*u))+
     & (pi*H1(1)*(10*NFV*(12*omv*(-1+3*u)+
     & (-40+u*(20-17*v)+37*v-4*(u*u))*(u*u))+
     & omu*(2*omv+u*(89+v+2*u*(-92+9*v)-
     & 4*nf*((-1+u)*(-1+u))+93*(u*u)))))/
     & (18._dp*u*(omu*omu*omu))+
     & (pi*((6-66*v-(120*NFV)/w)/omu+
     & (-120*NFV+3*(3+12*nf-8*nf*u)*v)/(omu*omu)+
     & (180*NFV*v*(-1+4*u+u*u*u))/(omu*omu*omu)+
     & (12*(-16+45*NFV)*(u*u)*(w*w)+
     & omv*(w*(u*(-54+w*(759-76*nf+78*v+270*zeta2))-
     & 6*(-9+17*w)*(u*u)+6*w*(-3-7*v+v*v))+
     & 30*NFV*(4*(1-2*w)*w+3*w*(2+3*w)*(u*u)+4*(u*u*u)+
     & u*(4-8*w+26*(w*w)))))/(omv*u*(w*w))))/108._dp+
     & H1(0)*((pi*H1(1))/9._dp+(pi*((33+60*NFV+32*u-90*NFV*u)/omv+
     & (2*(-16+45*NFV)*u)/(omv*omv)+
     & (17*omu*u*v*(w*w*w)-20*u*(2*NFV+v)*(omu*omu)*(w*w*w)+
     & 10*NFV*(1-5*u)*v*(u*u)*(w*w*w)+
     & omu*omu*omu*
     & (-(w*(u*w*(-11+(434+4*nf-27*v)*w)+
     & (-9+20*w)*(u*u)+9*(u*u*u)+2*(9+v)*(w*w)))+
     & 20*NFV*(w*(u*u)+(-1+2*v)*(u*u*u)+u*u*u*u-
     & u*(w*w*w)+6*v*(w*w*w))))/(u*(omu*omu*omu)*(w*w*w))
     & ))/18._dp)+(pi*(G1(2)*G1(2)))/18._dp-(179*pi*(H1(0)*H1(0)))/18._dp+
     & (pi*(H1(1)*H1(1)))/18._dp)+G1(2)*
     & (G1(3)*((-4*pi)/3._dp-(8*pi*H1(0))/9._dp-(8*pi*H1(1))/9._dp)-
     & (2*pi*(omv-15*NFV*(-2+u)*(-1+v)+u*(-2+41*u+2*v))*H2(1,0))/
     & (9._dp*(u*u))+(pi*H2(0,0)*(-3+6*omv*(5*NFV*(-2+u)+2*u)+3*v-
     & 88*(u*u)))/(9._dp*(u*u))+
     & (2*pi*H2(0,1)*(5+5*omv*(3*NFV*(-2+u)-2*u)-5*v+14*(u*u)))/
     & (9._dp*(u*u))+(pi*H1(1)*(10*NFV*
     & (12*omv*(-1+3*u)+
     & (-40+u*(20-17*v)+37*v-4*(u*u))*(u*u))+
     & omu*(4*omv+u*(14*u*(-13+v)+5*(17+v)-
     & 4*nf*((-1+u)*(-1+u))+93*(u*u)))))/
     & (18._dp*u*(omu*omu*omu))+
     & H1(0)*((pi*H1(1))/3._dp+(pi*(625+31/omv+20/omw+52/u-
     & (32*u)/omv-(1094*v)/omu-(56*v)/u+4/w-(4*u)/w+
     & (3620*v)/(omu*omu)-(5058*v)/(omu*omu*omu)+
     & (2529*v)/(omu*omu*omu*omu)+
     & 10*NFV*(10-4/omu-6/omv+(12*(-2+v))/u-
     & (2*u*(1+u)*v)/(omu*omu*omu)-(9*u*v)/(omv*omv))+
     & (32*u)/(omv*omv)+10/(omw*omw)-12/(u*u)+(12*v)/(u*u)+
     & (4*nf*(omv-u*(6-6*v+u*(-5+14*v)+(-20+11*v)*(u*u)+
     & 9*(5+4*v)*(u*u*u)+(-34+25*v)*(u*u*u*u)+
     & 9*(u*u*u*u*u))))/(omu*omu*omu*omu*(u*u))))/18._dp)+
     & (pi*((6*(w+52*v*w+10*NFV*(-2+5*v*w)))/(omu*w)-
     & (3*((123-12*nf+8*nf*u)*v+20*NFV*(2+17*v)))/(omu*omu)+
     & (720*NFV*v)/(omu*omu*omu)+
     & (-144*(5*NFV*(-2+u)-2*u)*zeta2*(omv*omv)*(w*w)+
     & 12*(-16+45*NFV)*(u*u*u)*(w*w)+
     & omv*(w*(144*(-1+v)*w*zeta2+
     & (-54+w*(759-76*nf+78*v-1170*zeta2))*(u*u)-
     & 6*(-9+17*w)*(u*u*u)+6*u*w*(62-72*v+v*v))+
     & 30*NFV*u*(4*w*(1+(-6+4*v)*w)+3*w*(2+3*w)*(u*u)+
     & 4*(u*u*u)+u*(4-8*w+(26-6*v)*(w*w)))))/
     & (omv*(u*u)*(w*w))))/108._dp+
     & (pi*(6-12*omv*u-6*v+7*(u*u))*(H1(0)*H1(0)))/(6._dp*(u*u))+
     & (pi*(H1(1)*H1(1)))/6._dp)+G1(3)*
     & ((-64*pi*H2(0,0))/3._dp+(8*pi*H2(0,1))/9._dp+(98*pi*H2(1,0))/9._dp+
     & H1(0)*((8*pi*H1(1))/9._dp+(pi*
     & ((20*NFV+3*(-8+v))*(u*u*u*u*u)+
     & u*u*u*u*(72-41*v+10*NFV*(-4+7*v)+11*(v*v))+
     & omv*(v*v)*(10+4*(-7+5*NFV)*v+17*(v*v)+v*v*v)+
     & omv*u*v*(-20+10*(1+3*NFV)*v-2*(-9+5*NFV)*(v*v)+
     & v*v*v)+u*u*u*
     & (-72+62*v+5*(v*v)+10*NFV*(2-8*v+9*(v*v))+
     & 14*(v*v*v))+u*u*
     & (24+(-4+30*NFV)*v-(47+60*NFV)*(v*v)+
     & (21+50*NFV)*(v*v*v)+6*(v*v*v*v))))/(18._dp*(u*u)*(w*w*w)))
     & +(pi*H1(1)*((20*NFV+3*(-8+v))*(u*u*u*u*u)+
     & u*u*u*u*(72-41*v+10*NFV*(-4+7*v)+11*(v*v))+
     & omv*(v*v)*(10+4*(-7+5*NFV)*v+17*(v*v)+v*v*v)+
     & omv*u*v*(-20+10*(1+3*NFV)*v-2*(-9+5*NFV)*(v*v)+v*v*v)+
     & u*u*u*(-72+62*v+5*(v*v)+10*NFV*(2-8*v+9*(v*v))+
     & 14*(v*v*v))+u*u*
     & (24+(-4+30*NFV)*v-(47+60*NFV)*(v*v)+
     & (21+50*NFV)*(v*v*v)+6*(v*v*v*v))))/(18._dp*(u*u)*(w*w*w))+
     & (pi*(-7587*omu*omv*u*(1+(4+3*v)*w)*(w*w)+
     & 2*omv*u*w*(2*nf*(-152+250*v-(625+236*v)*(u*u*u*u)+
     & 71*(u*u*u*u*u)+u*u*u*(1439-757*v-631*(v*v))-
     & 17*(v*v)+u*(706-1000*v+375*(v*v)-81*(v*v*v))-
     & 81*(v*v*v)-u*u*(1439-1743*v+213*(v*v)+324*(v*v*v)))
     & +7587*(1+v)*(w*w))+
     & 3*omv*u*w*(omu*omu)*(-843+5058*w+4*(1820+977*v)*(w*w))+
     & 3*(omu*omu*omu)*(20*NFV*(omv*omv)*
     & (-2*v*(6-13*v+6*(v*v))+
     & u*(-12+56*v-46*(v*v)+3*(v*v*v)))+
     & omv*w*(-2*w*(9+w*(-9-15*v+v*v))+
     & u*u*(492-418*w+34*(w*w))+
     & u*(825-2926*w+(831-26*v+720*zeta2)*(w*w)))+
     & 2*u*(-5*NFV*u*(-48+164*v+9*(-3+v)*(u*u*u)-197*(v*v)+
     & 3*(u*u)*(19-32*v+7*(v*v))+90*(v*v*v)+
     & u*(-12+62*v-59*(v*v)+9*(v*v*v))-9*(v*v*v*v))+
     & 32*(1+u)*(w*w*w)))))/(108._dp*omv*u*(omu*omu*omu)*(w*w*w))
     & +(4*pi*(H1(0)*H1(0)))/9._dp+(4*pi*(H1(1)*H1(1)))/9._dp)+
     & G1(1)*(G1(2)*((pi*(omv+2*u*(-1+u+v)))/(6._dp*(u*u))+
     & (pi*(omv+2*u*(-1+u+v))*H1(0))/(9._dp*(u*u))+
     & (pi*(omv+2*u*(-1+u+v))*H1(1))/(9._dp*(u*u)))-
     & (pi*(omv-2*(u*(1+8*u-v)+15*NFV*(-2+u)*(-1+v)))*H2(1,1))/
     & (9._dp*(u*u))-(2*pi*H2(0,1)*
     & (5+5*omv*(3*NFV*(-2+u)-2*u)-5*v+u*u))/(9._dp*(u*u))+
     & (2*pi*H2(1,0)*(-3-15*omv*NFV*(-2+u)+3*u+3*v-3*u*v+
     & 89*(u*u)))/(9._dp*(u*u))+
     & (pi*H2(0,0)*(3-6*omv*(5*NFV*(-2+u)+2*u)-3*v+196*(u*u)))/
     & (9._dp*(u*u))+(pi*(6*omu*omv*omw*(16+90*NFV-69*v)*(u*u)*(w*w)+
     & 2*omv*omw*(2*omv*nf*(19-70*u)+
     & u*u*(2*nf*(112-83*v+u*(-90+26*v)+29*(u*u))+
     & 9*v*(23+10*NFV*(1+u*u))))*(w*w)+
     & omu*omu*(720*omw*NFV*(-2+u)*zeta2*(omv*omv)*(w*w)-
     & 12*omw*(-16+45*NFV)*(-1+u)*(u*u)*(w*w)-
     & omv*(60*(-1+u)*u*(w*w)+
     & omw*(w*(w*(1083+78*v-3300*zeta2)*(u*u)-
     & 6*(-9+17*w)*(u*u*u)+
     & w*(-222*(-1+v)*zeta2-3*(-391+371*v+2*(v*v)))+
     & 2*u*(27+
     & w*(150*(-1+v)*zeta2+3*(-293+268*v+v*v))))+
     & 30*NFV*(2*w*(-2+(-1+6*v)*w)+2*w*(1+22*w)*(u*u)+
     & 3*w*(2+3*w)*(u*u*u)+4*(u*u*u*u)-
     & u*(4-14*w+3*(7+2*v)*(w*w))))))))/
     & (108._dp*omv*omw*(omu*omu)*(u*u)*(w*w))+
     & H1(0)*((-10*pi*(omv+2*u*(-1+u+v))*H1(1))/(9._dp*(u*u))+
     & (pi*(4*nf*(omv*omv*omv)*(u*u)+
     & 4*nf*(-5+2*v)*(omv*omv)*(u*u*u)-
     & 2*omv*(20-80*v+109*(v*v)-(49+20*NFV)*(v*v*v)+
     & u*(-130+452*v-3*(167+10*NFV)*(v*v)+
     & 10*(17+5*NFV)*(v*v*v)))+
     & u*u*(-832+(3241+60*NFV)*v+
     & 2*(-360-80*NFV+nf*(10-6*v)+292*v+85*NFV*v)*
     & (u*u*u*u)-4*(-33+nf-10*NFV)*(u*u*u*u*u)-
     & 3*(1483+80*NFV)*(v*v)+
     & u*u*u*(1660-2701*v+960*(v*v)-
     & 4*nf*(10-12*v+3*(v*v))+10*NFV*(24-53*v+27*(v*v))
     & )+(2489+300*NFV)*(v*v*v)+
     & u*u*(-2120+5163*v-3691*(v*v)+696*(v*v*v)-
     & 4*nf*(-10+18*v-9*(v*v)+v*v*v)+
     & 10*NFV*(-16+61*v-66*(v*v)+19*(v*v*v)))-
     & (449+80*NFV)*(v*v*v*v)+
     & u*(1660-5323*v+5652*(v*v)-2159*(v*v*v)+
     & 188*(v*v*v*v)+
     & 10*NFV*(4-31*v+57*(v*v)-37*(v*v*v)+5*(v*v*v*v)))))
     & )/(18._dp*(omu*omu)*(u*u)*(w*w*w)))+
     & (pi*H1(1)*(w*(-78+254*v+(-192+4*nf+27*v)*(u*u*u*u*u*u)-
     & 283*(v*v)+2*(u*u*u*u*u)*
     & (472+4*nf*(-2+v)-413*v+27*(v*v))+107*(v*v*v)+
     & u*u*u*u*(-1934+2909*v-1029*(v*v)+4*nf*(6-6*v+v*v)+
     & 27*(v*v*v))-2*(u*u*u)*
     & (-1068+2174*v-1287*(v*v)+4*nf*(2-3*v+v*v)+
     & 193*(v*v*v))-
     & 2*u*(-244+704*v-675*(v*v)+215*(v*v*v))+
     & u*u*(-1364+3392*v+4*nf*(omv*omv)-2666*(v*v)+
     & 629*(v*v*v)))+
     & 10*NFV*(3*(-9+7*v)*(u*u*u*u*u*u)+6*(u*u*u*u*u*u*u)+
     & 3*(u*u*u*u*u)*(17-25*v+9*(v*v))+
     & u*u*u*u*(-56+117*v-81*(v*v)+19*(v*v*v))+
     & u*u*(-27+82*v-93*(v*v)+48*(v*v*v)-8*(v*v*v*v))-
     & 2*(1-4*v+6*(v*v)-5*(v*v*v)+2*(v*v*v*v))+
     & u*u*u*(44-115*v+108*(v*v)-43*(v*v*v)+5*(v*v*v*v))+
     & u*(11-38*v+51*(v*v)-34*(v*v*v)+10*(v*v*v*v)))))/
     & (18._dp*(omu*omu)*(u*u)*(w*w*w))-
     & (pi*(omv+2*u*(-1+u+v))*(G1(2)*G1(2)))/(18._dp*(u*u))-
     & (pi*(omv+2*u*(-1+u+v))*(H1(0)*H1(0)))/(18._dp*(u*u))-
     & (pi*(omv+2*u*(-1+u+v))*(H1(1)*H1(1)))/(18._dp*(u*u)))-
     & (pi*(H1(1)*H1(1)*H1(1)))/18._dp
      return
      end
      
      
      function Gamma_4a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function Gamma_4a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function Gamma_4a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-5+4*u+v)/(3._dp*omu*omv)+((-1+2*u)*v*H1(1))/(3._dp*u*(omu*omu))+
     & G1(2)*((v-2*u*v)/(3._dp*u*(omu*omu))+(v*H1(0))/(3._dp*(u*u)))+
     & G1(1)*(-(v*H1(0))/(3._dp*(u*u))-(v*H1(1))/(3._dp*(u*u)))+
     & (v*G2(1,2))/(3._dp*(u*u))-(v*H1(0)*
     & (-(omv*v)+(-9+v)*(u*u)+4*(u*u*u)+u*(4+2*v-2*(v*v))))/
     & (3._dp*u*(omu*omu)*(omv*omv))
      return
      end
      
      
      function Gamma_4a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(pi*(1-2*u)*v)/(3._dp*u*(omu*omu))+(pi*v*G1(1))/(3._dp*(u*u))
      return
      end
      
      
      function Gamma_4a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=((-101+nf*(4-8*u)+200*u-40*NFV*(-2+3*u))*v*H2(1,1))/
     & (18._dp*u*(omu*omu))+G3(2,1,1)*((-4*v*H1(0))/(u*u)-(4*v*H1(1))/(u*u))+
     & G3(0,1,0)*((-2*v*H1(0))/(u*u)-(2*v*H1(1))/(u*u))+
     & G3(0,1,1)*((v*H1(0))/(u*u)+(v*H1(1))/(u*u))+
     & G3(2,1,0)*((v*H1(0))/(u*u)+(v*H1(1))/(u*u))+
     & G3(2,0,1)*(((11-60*NFV)*v*H1(0))/(9._dp*(u*u))+
     & ((11-60*NFV)*v*H1(1))/(9._dp*(u*u)))+
     & G3(1,1,1)*(((19-20*NFV)*v*H1(0))/(3._dp*(u*u))+
     & ((19-20*NFV)*v*H1(1))/(3._dp*(u*u)))+
     & G3(1,1,0)*(((-7+20*NFV)*v*H1(0))/(3._dp*(u*u))+
     & ((-7+20*NFV)*v*H1(1))/(3._dp*(u*u)))+
     & G3(2,2,1)*(((-1+20*NFV)*v*H1(0))/(3._dp*(u*u))+
     & ((-1+20*NFV)*v*H1(1))/(3._dp*(u*u)))+
     & G3(1,2,1)*((-5*(v+4*NFV*v)*H1(0))/(3._dp*(u*u))-
     & (5*(v+4*NFV*v)*H1(1))/(3._dp*(u*u)))+(2*v*G4(0,1,0,2))/(u*u)-
     & (v*G4(0,1,1,2))/(u*u)+(2*v*G4(0,1,2,0))/(u*u)-(v*G4(0,1,2,2))/(u*u)+
     & ((v-60*NFV*v)*G4(1,0,2,2))/(9._dp*(u*u))+
     & ((-17+60*NFV)*v*G4(1,0,3,0))/(9._dp*(u*u))+
     & ((7-20*NFV)*v*G4(1,1,0,2))/(3._dp*(u*u))+
     & ((-19+20*NFV)*v*G4(1,1,1,2))/(3._dp*(u*u))+
     & ((7-20*NFV)*v*G4(1,1,2,0))/(3._dp*(u*u))+
     & (5*(v+4*NFV*v)*G4(1,1,2,2))/(3._dp*(u*u))+
     & ((v-60*NFV*v)*G4(1,2,0,2))/(9._dp*(u*u))+
     & (5*(v+4*NFV*v)*G4(1,2,1,2))/(3._dp*(u*u))+
     & ((v-60*NFV*v)*G4(1,2,2,0))/(9._dp*(u*u))+
     & ((-17+60*NFV)*v*G4(1,2,3,0))/(9._dp*(u*u))+
     & ((-11+60*NFV)*v*G4(2,0,1,2))/(9._dp*(u*u))-
     & ((v+20*NFV*v)*G4(2,0,3,0))/(3._dp*(u*u))-(v*G4(2,1,0,2))/(u*u)+
     & (4*v*G4(2,1,1,2))/(u*u)-(v*G4(2,1,2,0))/(u*u)+
     & ((v-20*NFV*v)*G4(2,2,1,2))/(3._dp*(u*u))+
     & ((v+20*NFV*v)*G4(2,2,3,0))/(3._dp*(u*u))+(20*v*G4(2,3,3,0))/(9._dp*(u*u))+
     & (2*v*G3(0,2,0)*H1(0))/(u*u)+((17-60*NFV)*v*G3(1,0,3)*H1(0))/(9._dp*(u*u))+
     & ((17-60*NFV)*v*G3(1,2,3)*H1(0))/(9._dp*(u*u))+
     & ((v+20*NFV*v)*G3(2,0,3)*H1(0))/(3._dp*(u*u))-
     & ((v+20*NFV*v)*G3(2,2,3)*H1(0))/(3._dp*(u*u))-
     & (20*v*G3(2,3,3)*H1(0))/(9._dp*(u*u))+
     & G2(2,3)*((-20*v*zeta2)/(9._dp*(u*u))+
     & (H1(0)*(-39/omv-10/(u-u*v)+
     & (2*(3+17*v+(60*NFV*(-2+v)*v)/omv+2/w))/u-4/w+
     & 39/(omv*omv)+u*((90*NFV*v)/(omu*omu)+
     & ((32-90*NFV)*v)/(omv*omv*omv))+
     & v*((-60*NFV)/(omu*omu)-(60*NFV)/(omv*omv)+
     & (21+(1630-216*nf)*u+(892-108*nf)*(u*u)-14*(u*u*u))/
     & (omu*omu*omu*omu))))/18._dp)+
     & G3(0,2,2)*(-((v*H1(0))/(u*u))+
     & (v*(2-7*u+25*(u*u)-20*(u*u*u)+
     & 10*NFV*(-12+30*u-25*(u*u)+3*(u*u*u))))/
     & (18._dp*u*(omu*omu*omu)))+
     & G3(2,0,2)*(-(v*H1(0))/(9._dp*(u*u))+
     & (v*(2-7*u+25*(u*u)-20*(u*u*u)+
     & 10*NFV*(-12+30*u-25*(u*u)+3*(u*u*u))))/
     & (18._dp*u*(omu*omu*omu)))+
     & G3(2,2,0)*((4*v*H1(0))/(9._dp*(u*u))+
     & (v*(2-7*u+25*(u*u)-20*(u*u*u)+
     & 10*NFV*(-12+30*u-25*(u*u)+3*(u*u*u))))/
     & (18._dp*u*(omu*omu*omu)))+
     & G3(1,2,2)*((5*(v+4*NFV*v)*H1(0))/(3._dp*(u*u))+
     & (v*(-81+176*u+4*nf*(omu*omu)-109*(u*u)-20*(u*u*u)+
     & 10*NFV*(-4+17*u-22*(u*u)+3*(u*u*u))))/(18._dp*(omu*omu)*(u*u))
     & )+G2(2,2)*((5*(1-4*NFV)*v*H2(0,0))/(3._dp*(u*u))+
     & ((v-20*NFV*v)*H2(0,1))/(3._dp*(u*u))+(13*v*H2(1,0))/(9._dp*(u*u))+
     & (v*(3*u*(-101+80*NFV+nf*(4-8*u)+200*u-120*NFV*u)+
     & 60*(1+6*NFV)*zeta2*(omu*omu)))/(54._dp*(omu*omu)*(u*u))+
     & (v*H1(0)*(-81+176*u+4*nf*(omu*omu)-109*(u*u)-20*(u*u*u)+
     & 10*NFV*(-4+17*u-22*(u*u)+3*(u*u*u))))/(18._dp*(omu*omu)*(u*u))
     & )+(G3(2,3,0)*(39/omv+10/(u-u*v)+
     & (2*(-3-17*v-(60*NFV*(-2+v)*v)/omv-2/w))/u+4/w-
     & 39/(omv*omv)+u*((-90*NFV*v)/(omu*omu)+
     & (2*(-16+45*NFV)*v)/(omv*omv*omv))+
     & v*(60*NFV*(1/(omu*omu)+1/(omv*omv))+
     & (-21+2*(-815+108*nf)*u+4*(-223+27*nf)*(u*u)+14*(u*u*u))/
     & (omu*omu*omu*omu))))/18._dp+
     & G3(2,1,2)*((2*v*H1(0))/(u*u)+
     & (v*(omv*omv*omv*(u*u)*(1-6*(9+10*NFV)*u+u*u)+
     & omu*omu*(-(omv*(41+60*NFV)*(u*u))+2*(16-45*NFV)*(u*u*u)+
     & omv*omv*omv*(-87+4*nf-20*NFV+20*v+v*v)-
     & 2*u*(omv*omv)*(-38+15*NFV+25*v+45*NFV*v+v*v))))/
     & (18._dp*(omu*omu)*(omv*omv*omv)*(u*u)))+
     & G3(0,1,2)*((v*H1(1))/(u*u)+(v*
     & (2*u*(-38-15*NFV*(-5+v)+26*v)*(omv*omv)+
     & (16+nf*(omu*omu))*(omv*omv*omv)+
     & omv*(u*u)*(124-118*v+60*NFV*(-3+4*v)+35*(v*v))+
     & u*u*u*(-156+105*u+184*v-41*u*v-32*(u*u)-78*(v*v)+
     & 30*NFV*(3-4*v-2*u*(2+v)+3*(u*u)+4*(v*v))+18*(v*v*v))
     & ))/(18._dp*(omu*omu)*(omv*omv*omv)*(u*u)))+
     & G2(0,1)*((-7*v*zeta2)/(u*u)-(v*H2(0,1))/(u*u)-(v*H2(1,1))/(u*u)-
     & (v*H1(0)*(2*u*(-38-15*NFV*(-5+v)+26*v)*(omv*omv)+
     & (16+nf*(omu*omu))*(omv*omv*omv)+
     & omv*(u*u)*(124-118*v+60*NFV*(-3+4*v)+35*(v*v))+
     & u*u*u*(-156+105*u+184*v-41*u*v-32*(u*u)-78*(v*v)+
     & 30*NFV*(3-4*v-2*u*(2+v)+3*(u*u)+4*(v*v))+18*(v*v*v))
     & ))/(18._dp*(omu*omu)*(omv*omv*omv)*(u*u))-
     & (v*H1(1)*(2*u*(-38-15*NFV*(-5+v)+26*v)*(omv*omv)+
     & (16+nf*(omu*omu))*(omv*omv*omv)+
     & omv*(u*u)*(124-118*v+60*NFV*(-3+4*v)+35*(v*v))+
     & u*u*u*(-156+105*u+184*v-41*u*v-32*(u*u)-78*(v*v)+
     & 30*NFV*(3-4*v-2*u*(2+v)+3*(u*u)+4*(v*v))+18*(v*v*v))
     & ))/(18._dp*(omu*omu)*(omv*omv*omv)*(u*u)))+
     & G2(2,1)*((-2*v*zeta2)/(u*u)+(2*v*H2(0,1))/(u*u)-(2*v*H2(1,0))/(u*u)+
     & (v*H1(0)*(87-250*u-281*v+764*u*v-
     & 4*nf*(omu*omu)*(omv*omv*omv)+279*(u*u)-723*v*(u*u)-
     & 136*(u*u*u)+122*v*(u*u*u)+104*(u*u*u*u)-38*v*(u*u*u*u)-
     & 32*(u*u*u*u*u)+320*(v*v)-814*u*(v*v)+665*(u*u)*(v*v)-
     & 12*(u*u*u)*(v*v)-3*(u*u*u*u)*(v*v)-144*(v*v*v)+
     & 334*u*(v*v*v)-235*(u*u)*(v*v*v)-8*(u*u*u)*(v*v*v)+
     & u*u*u*u*(v*v*v)+10*NFV*
     & (u*(-1+13*v)*(omv*omv)+2*(omv*omv*omv)+
     & u*u*(2-18*v-6*(2+v)*(u*u)+9*(u*u*u)+36*(v*v)-
     & 20*(v*v*v)+3*u*(2-v+v*v+v*v*v)))+17*(v*v*v*v)-
     & 32*u*(v*v*v*v)+13*(u*u)*(v*v*v*v)+2*(u*u*u)*(v*v*v*v)+
     & v*v*v*v*v-2*u*(v*v*v*v*v)+u*u*(v*v*v*v*v)))/
     & (18._dp*(omu*omu)*(omv*omv*omv)*(u*u))+
     & (v*H1(1)*(87-250*u-281*v+764*u*v-
     & 4*nf*(omu*omu)*(omv*omv*omv)+279*(u*u)-723*v*(u*u)-
     & 136*(u*u*u)+122*v*(u*u*u)+104*(u*u*u*u)-38*v*(u*u*u*u)-
     & 32*(u*u*u*u*u)+320*(v*v)-814*u*(v*v)+665*(u*u)*(v*v)-
     & 12*(u*u*u)*(v*v)-3*(u*u*u*u)*(v*v)-144*(v*v*v)+
     & 334*u*(v*v*v)-235*(u*u)*(v*v*v)-8*(u*u*u)*(v*v*v)+
     & u*u*u*u*(v*v*v)+10*NFV*
     & (u*(-1+13*v)*(omv*omv)+2*(omv*omv*omv)+
     & u*u*(2-18*v-6*(2+v)*(u*u)+9*(u*u*u)+36*(v*v)-
     & 20*(v*v*v)+3*u*(2-v+v*v+v*v*v)))+17*(v*v*v*v)-
     & 32*u*(v*v*v*v)+13*(u*u)*(v*v*v*v)+2*(u*u*u)*(v*v*v*v)+
     & v*v*v*v*v-2*u*(v*v*v*v*v)+u*u*(v*v*v*v*v)))/
     & (18._dp*(omu*omu)*(omv*omv*omv)*(u*u)))+
     & G2(2,0)*((5*(-1+4*NFV)*v*H2(0,0))/(3._dp*(u*u))+
     & (10*(-1+6*NFV)*v*H2(0,1))/(9._dp*(u*u))-(v*H2(1,0))/(u*u)-
     & (v*H1(1)*(2-7*u+25*(u*u)-20*(u*u*u)+
     & 10*NFV*(-12+30*u-25*(u*u)+3*(u*u*u))))/
     & (18._dp*u*(omu*omu*omu))+
     & (H1(0)*(72/omv+32/(u-u*v)+4/w-72/(omv*omv)+
     & (v*(2+2*(-859+108*nf)*u+4*(-196+27*nf)*(u*u)-
     & 30*(u*u*u)-10*NFV*(-6+17*u-20*(u*u)+9*(u*u*u))+
     & u*u*u*u))/(omu*omu*omu*omu)+
     & (v*(-63+nf+20*NFV+20*v+v*v))/(u*u)+
     & (2*(-14+(13+60*NFV)*v-2/w+v*v))/u))/18._dp+
     & ((6*(9*omw+v*(-36+nf+v)*w))/(u*w)+(12*v*zeta2)/(u*u)+
     & 3*(48-2/omu-66/omv-18/w+
     & (v*(31+4*u-2*nf*(2+u)+22*(u*u)))/(omu*omu))+
     & (12*u*(13-2*v+5*(v*v)))/(omv*omv)+
     & 30*NFV*(2*(3-6/omv+(-2+2/w)/omu-1/w+2/(omu*omu)+
     & (3*v*(-2+u-3*(u*u)))/(omu*omu*omu))-
     & (24*v*zeta2)/(u*u)+u*(9/omv-18/(omv*omv)-4/(w*w))-
     & (4*(2-3/w+1/(w*w)))/u))/108._dp)+
     & (H3(0,0,0)*(-31/omv-8/(u-u*v)-86/w+31/(omv*omv)+
     & v*(60*NFV*(1/(omu*omu)-1/(omv*omv))+
     & (-21+8*(-206+27*nf)*u+4*(-214+27*nf)*(u*u)-4*(u*u*u))/
     & (omu*omu*omu*omu))+
     & (2*(-(omv*(-4+(1+60*NFV)*(v*v)))+u*(-1+(1+60*NFV)*(v*v))))/
     & (omv*u*w)+u*((-90*NFV*v)/(omu*omu)-32/(omv*omv)+
     & (32-90*NFV*v)/(omv*omv*omv)-80/(w*w))+80/(w*w)))/18._dp-
     & (v*H2(0,1)*(omv*(u*u)*(771-1199*v+542*(v*v)-
     & 30*NFV*(5-7*v+5*(v*v))-10*(v*v*v))+
     & 2*(u*u*u)*(-308+448*v+(-42+5*v)*(u*u)-192*(v*v)+
     & 15*NFV*(10+3*u*(-3+v)-8*v+3*(u*u)+v*v)+
     & u*(151-92*v+15*(v*v))+15*(v*v*v))+
     & omv*omv*(109+20*(-14+5*NFV)*v+(171-80*NFV)*(v*v)-
     & 6*nf*(w*w)+2*u*(-241+391*v-134*(v*v)+
     & 5*NFV*(3-25*v+12*(v*v))+6*nf*(w*w)))))/
     & (18._dp*u*(omu*omu)*(omv*omv)*(w*w))+
     & (4*(4*nf*(50/omu+38/omv+
     & (81*u*(2+u)*v*zeta3)/(omu*omu*omu*omu))+
     & 3*(-624/omu-588/omv+(313*zeta3)/omv+(106*zeta3)/(u-u*v)+
     & (2*(-51-11*v+(60*NFV*(-2+v)*v)/omv-2/w)*zeta3)/u+
     & (4*zeta3)/w-(313*zeta3)/(omv*omv)+
     & 2*u*v*zeta3*((45*NFV)/(omu*omu)+
     & (16-45*NFV)/(omv*omv*omv))+
     & 3*v*zeta3*((-20*NFV)/(omu*omu)-(20*NFV)/(omv*omv)+
     & (7-562*u-302*(u*u)+14*(u*u*u))/(omu*omu*omu*omu))))
     & +6*zeta2*((omv*omv*(2*v*
     & (95-268*v+186*(v*v)+4*nf*(4-9*v+5*(v*v))-
     & 4*(v*v*v))+u*
     & (-810+(2049-356*nf)*v+2*(-999+340*nf)*(v*v)+
     & (892-312*nf)*(v*v*v)+24*(v*v*v*v)))+
     & omv*(u*u)*(3198+5065*v-26221*(v*v)+27618*(v*v*v)-
     & 9198*(v*v*v*v)+
     & 4*nf*(18-303*v+737*(v*v)-726*(v*v*v)+256*(v*v*v*v))+
     & 24*(v*v*v*v*v))-
     & 2*(u*u*u)*(3322-3*(-541+392*nf)*v+
     & (-20693+3132*nf)*(v*v)-8*(u*u*u*u)*(13-2*v+5*(v*v))+
     & 4*(u*u*u)*(50+(5+4*nf)*v+54*(v*v)-31*(v*v*v))+
     & (24347-2892*nf)*(v*v*v)+
     & u*u*(1251-2422*v+554*(v*v)-12*nf*(6-5*v+3*(v*v))+
     & 581*(v*v*v)-132*(v*v*v*v))+(-8744+920*nf)*(v*v*v*v)+
     & u*(-3475+3626*v+2938*(v*v)-3715*(v*v*v)+
     & 4*nf*(27+74*v-178*(v*v)+89*(v*v*v))+590*(v*v*v*v)-
     & 52*(v*v*v*v*v))+221*(v*v*v*v*v)-4*(v*v*v*v*v*v)))/
     & (u*(omu*omu*omu)*(omv*omv)*(w*w))+
     & 10*NFV*(-24+48/omu+30/omv+27/(u-u*v)+8/w-24/(omu*w)-
     & 24/(omu*omu)+(36*v)/(omu*omu*omu)+18/(omv*omv)+
     & (24*v*(u*u))/(omu*omu*omu)-3/(w*w)+
     & (40-48*v-107/w+44/(w*w)-4/(w*w*w))/u+
     & 4*u*(-9/omv+(21*v)/(omu*omu*omu)+18/(omv*omv)+
     & (-1+4*w)/(w*w*w))+2/(w*w*w))))/216._dp+
     & G2(0,2)*((v*H2(0,1))/(u*u)+(2*v*H2(1,0))/(u*u)-
     & (v*H1(1)*(2-7*u+25*(u*u)-20*(u*u*u)+
     & 10*NFV*(-12+30*u-25*(u*u)+3*(u*u*u))))/
     & (18._dp*u*(omu*omu*omu))+
     & ((-2+2*(-24+nf)*v-(40*NFV*(-1+w))/w)/omu+
     & (40*NFV+57*v-6*nf*v)/(omu*omu)-
     & (60*NFV*v*(2-u+3*(u*u)))/(omu*omu*omu)+
     & (2*(3*omv*(-11+5*NFV*(-4+3*u))*(u*u)*(w*w)+
     & 2*(u*u*u)*(13-45*NFV-2*v+5*(v*v))*(w*w)+
     & omv*omv*(w*(u*v*(9+(-36+nf+v)*w)+18*v*w*zeta2+
     & (24+11*v)*w*(u*u))-
     & 10*NFV*u*(2-6*w+2*(u*u)+u*(w-3*(w*w))+4*(w*w)))))/
     & (omv*omv*(u*u)*(w*w)))/36._dp+
     & (H1(0)*(-8/omv-2/(u-u*v)+(2*(2+5*(-4+15*NFV)*v-1/w))/u+
     & 11/w+8/(omv*omv)+((16+nf)*v)/(u*u)+
     & (v*(-29+49*u-29*(u*u)+40*NFV*(3-5*u+3*(u*u))+
     & 9*(u*u*u)))/(omu*omu*omu)-9/(w*w)-
     & (u*(-9+(9+20*NFV)*u+(9+20*NFV)*v))/(w*w*w)+
     & (20*NFV*(u*u))/(w*w*w)))/18._dp)+
     & (H1(1)*(-498/omu+276/omv-(1215*v)/omu+(76*nf*v)/omu-
     & (1215*v)/u+(76*nf*v)/u+(1461*v)/(omu*omu)-
     & (52*nf*v)/(omu*omu)+
     & 6*zeta2*(-167/omv-28/(u-u*v)+(30-25*v-2/w)/u+153/w+
     & 167/(omv*omv)+(3*v*
     & (-13+(623-72*nf)*u+(201-36*nf)*(u*u)+29*(u*u*u)+
     & 3*(u*u*u*u)))/(omu*omu*omu*omu)+
     & u*(-32/(omv*omv)+32/(omv*omv*omv)+151/(w*w))-151/(w*w))
     & +10*NFV*((6*(1+6*v-1/w+6*(2-8*v)*zeta2))/u+
     & 3*((4*(5+3*v))/omu-(24*v)/(omu*omu)+
     & (-9*omv*u-12*(2+(-2+u)*v)*zeta2)/(u*(omv*omv))-
     & (24*v*zeta2*(2-3*u+3*(u*u)))/(omu*omu*omu))+
     & 6*u*zeta2*(9/(omv*omv)-9/(omv*omv*omv)+
     & (2-2*w)/(w*w*w))-(12*zeta2*(u*u))/(w*w*w))))/108._dp+
     & (H3(1,0,1)*((-41+(120*NFV*v)/u)/omv-24/(u-u*v)+(22+2/w)/u-
     & 11/w+(9*v)/(omu*omu)+
     & (41+32*u-90*NFV*u+60*NFV*v)/(omv*omv)+
     & (2*(-16+45*NFV)*u)/(omv*omv*omv)+9/(w*w)+
     & u*u*((9*v)/(omu*omu)-(20*NFV)/(w*w*w))-
     & (u*(20*NFV*(-1+w)+9*w))/(w*w*w)))/18._dp+
     & (H3(1,1,0)*(-208/omv+(52+22*v+120*NFV*v)/u-52/(u-u*v)+142/w+
     & 208/(omv*omv)-(3*v*(1+(-550+72*nf)*u+
     & 10*NFV*(-2+3*u)*(omu*omu)+(-314+36*nf)*(u*u)+
     & 26*(u*u*u)-6*(u*u*u*u)))/(omu*omu*omu*omu)-142/(w*w)+
     & (2*u*(71+(-71+20*NFV)*u+(-71+20*NFV)*v))/(w*w*w)-
     & (40*NFV*(u*u))/(w*w*w)))/18._dp+
     & (H3(1,0,0)*(49/omv-28/(u-u*v)+
     & (2*(15+v-(60*NFV*(-2+v)*v)/omv-1/w))/u-7/w-
     & 49/(omv*omv)+v*(60*NFV*(1/(omu*omu)+1/(omv*omv))+
     & (-12+4*(-421+54*nf)*u+2*(-401+54*nf)*(u*u)-40*(u*u*u)+
     & 9*(u*u*u*u))/(omu*omu*omu*omu))+9/(w*w)+
     & u*((-90*NFV*v)/(omu*omu)+(32-90*NFV)/(omv*omv)+
     & (-32+90*NFV)/(omv*omv*omv)+(-20*NFV*(-1+w)-9*w)/(w*w*w))
     & -(20*NFV*(u*u))/(w*w*w)))/18._dp+
     & (H3(0,1,0)*(123/omv+18/(u-u*v)+75/w-123/(omv*omv)+
     & v*(60*NFV*(1/(omu*omu)-1/(omv*omv))+
     & (3*(2+u)*(-2-36*(-8+nf)*u-8*(u*u)+3*(u*u*u)))/
     & (omu*omu*omu*omu))+
     & (2*(omv*(-9+(7-60*NFV)*(v*v))+u*(7+(-7+60*NFV)*(v*v))))/
     & (omv*u*w)-71/(w*w)+
     & u*((-90*NFV*v)/(omu*omu)+(-32+90*NFV)/(omv*omv)+
     & (32-90*NFV)/(omv*omv*omv)+(-20*NFV*(-1+w)+71*w)/(w*w*w))
     & -(20*NFV*(u*u))/(w*w*w)))/18._dp+
     & (G2(0,3)*H1(0)*(39/omv+10/(u-u*v)+75/w-39/(omv*omv)+
     & v*(60*NFV*(-(1/(omu*omu))+1/(omv*omv))+
     & (30+(1594-216*nf)*u+(946-108*nf)*(u*u)-50*(u*u*u)+
     & 9*(u*u*u*u))/(omu*omu*omu*omu))+
     & (-6+34*v-4/w+(120*NFV*(v*v))/omv)/u-71/(w*w)+
     & u*((90*NFV*v)/(omu*omu)+(32-90*NFV)/(omv*omv)+
     & (-32+90*NFV)/(omv*omv*omv)+(-20*NFV*(-1+w)+71*w)/(w*w*w)
     & )-(20*NFV*(u*u))/(w*w*w)))/18._dp+
     & (H3(0,0,1)*(10/omv-38/(u-u*v)+(36+36*v+2/w)/u-11/w-
     & 10/(omv*omv)+(9*v*(1-4*u+u*u))/(omu*omu)+9/(w*w)+
     & (u*(-9+(9+20*NFV)*u+(9+20*NFV)*v))/(w*w*w)-
     & (20*NFV*(u*u))/(w*w*w)))/18._dp+
     & (G3(0,3,0)*(-39/omv-10/(u-u*v)-75/w+39/(omv*omv)+
     & v*(60*NFV*(1/(omu*omu)-1/(omv*omv))+
     & (-30+2*(-797+108*nf)*u+2*(-473+54*nf)*(u*u)+50*(u*u*u)-
     & 9*(u*u*u*u))/(omu*omu*omu*omu))+
     & (6-34*v+4/w-(120*NFV*(v*v))/omv)/u+71/(w*w)+
     & u*((-90*NFV*v)/(omu*omu)+(-32+90*NFV)/(omv*omv)+
     & (32-90*NFV)/(omv*omv*omv)+(20*NFV*(-1+w)-71*w)/(w*w*w))
     & +(20*NFV*(u*u))/(w*w*w)))/18._dp+
     & G2(3,0)*(-(v*H1(1)*(omw*omw)*
     & (10+4*(-7+5*NFV)*v+(17+3*v)*(u*u)+u*u*u+17*(v*v)+
     & u*(-28+20*NFV+34*v+3*(v*v))+v*v*v))/(18._dp*(u*u)*(w*w*w))+
     & (u*u*u*u*u*u*u*u*(-52+8*v+90*NFV*(1+v)-20*(v*v))+
     & 2*(u*u*u*u*u*u*u)*(93-58*v+70*(v*v)+
     & 15*NFV*(-16-3*v+7*(v*v))-41*(v*v*v))+
     & 2*(omv*omv)*(v*v)*(-6+22*v-17*(v*v)+
     & 20*NFV*(6-13*v+6*(v*v))+v*v*v)+
     & 2*omv*(u*u*u*u*u*u)*(611-72*omv*nf-672*v-181*(v*v)+
     & 15*NFV*(36+v+3*(v*v))+64*(v*v*v))+
     & u*u*u*u*u*(-4182+16919*v-36*nf*(-10+29*v)*(omv*omv)-
     & 20888*(v*v)+7399*(v*v*v)+716*(v*v*v*v)-
     & 90*NFV*(14-39*v+39*(v*v)-25*(v*v*v)+7*(v*v*v*v))-
     & 92*(v*v*v*v*v))-u*(omv*omv)*
     & (160-4*(-23+18*nf+60*NFV)*v+
     & 4*(-514+72*nf+445*NFV)*(v*v)+
     & (2803-324*nf-2220*NFV)*(v*v*v)+
     & 3*(-329+36*nf+220*NFV)*(v*v*v*v)+6*(v*v*v*v*v))+
     & omv*omv*(u*u)*(400-16*(-49+90*NFV)*v+
     & 10*(50+453*NFV)*(v*v)-40*(152+87*NFV)*(v*v*v)+
     & (4001+600*NFV)*(v*v*v*v)-
     & 36*nf*(-2+10*v-24*(v*v*v)+15*(v*v*v*v))+6*(v*v*v*v*v))+
     & u*u*u*u*(4294-25276*v+54471*(v*v)-
     & 36*nf*(omv*omv)*(6-52*v+61*(v*v))-49534*(v*v*v)+
     & 15649*(v*v*v*v)+488*(v*v*v*v*v)-
     & 10*NFV*(-75+492*v-961*(v*v)+917*(v*v*v)-415*(v*v*v*v)+
     & 60*(v*v*v*v*v))-28*(v*v*v*v*v*v))+
     & omv*(u*u*u)*(-1708+9111*v-26974*(v*v)+33046*(v*v*v)-
     & 13425*(v*v*v*v)+36*nf*
     & (-2-13*v+84*(v*v)-120*(v*v*v)+51*(v*v*v*v))-
     & 134*(v*v*v*v*v)+10*NFV*
     & (-18+348*v-828*(v*v)+769*(v*v*v)-256*(v*v*v*v)+
     & 18*(v*v*v*v*v))+2*(v*v*v*v*v*v)))/
     & (36._dp*u*(omu*omu*omu)*(omv*omv)*(w*w*w)))+
     & G1(3)*((v*H2(0,1)*(omw*omw)*(10+4*(-7+5*NFV)*v+(17+3*v)*(u*u)+
     & u*u*u+17*(v*v)+u*(-28+20*NFV+34*v+3*(v*v))+v*v*v))/
     & (18._dp*(u*u)*(w*w*w))+(v*H2(1,0)*(omw*omw)*
     & (10+4*(-7+5*NFV)*v+(17+3*v)*(u*u)+u*u*u+17*(v*v)+
     & u*(-28+20*NFV+34*v+3*(v*v))+v*v*v))/(18._dp*(u*u)*(w*w*w))-
     & (omw*v*zeta2*(40-48*v+5*(17+4*v)*(u*u*u)+5*(u*u*u*u)+
     & 2*(-41+70*NFV)*(v*v)+u*u*(-82+140*NFV+255*v+30*(v*v))+
     & 85*(v*v*v)+u*(-48+4*(-41+70*NFV)*v+255*(v*v)+
     & 20*(v*v*v))+5*(v*v*v*v)))/(18._dp*(u*u)*(w*w*w))+
     & (H1(0)*(u*u*u*u*u*u*u*u*(52-8*v-90*NFV*(1+v)+20*(v*v))-
     & 2*(u*u*u*u*u*u*u)*(93-58*v+70*(v*v)+
     & 15*NFV*(-16-3*v+7*(v*v))-41*(v*v*v))-
     & 2*(omv*omv)*(v*v)*(-6+22*v-17*(v*v)+
     & 20*NFV*(6-13*v+6*(v*v))+v*v*v)-
     & 2*omv*(u*u*u*u*u*u)*
     & (611-72*omv*nf-672*v-181*(v*v)+
     & 15*NFV*(36+v+3*(v*v))+64*(v*v*v))+
     & u*(omv*omv)*(160-4*(-23+18*nf+60*NFV)*v+
     & 4*(-514+72*nf+445*NFV)*(v*v)+
     & (2803-324*nf-2220*NFV)*(v*v*v)+
     & 3*(-329+36*nf+220*NFV)*(v*v*v*v)+6*(v*v*v*v*v))-
     & omv*omv*(u*u)*(400-16*(-49+90*NFV)*v+
     & 10*(50+453*NFV)*(v*v)-40*(152+87*NFV)*(v*v*v)+
     & (4001+600*NFV)*(v*v*v*v)-
     & 36*nf*(-2+10*v-24*(v*v*v)+15*(v*v*v*v))+6*(v*v*v*v*v))
     & +u*u*u*u*u*(4182-16919*v+36*nf*(-10+29*v)*(omv*omv)+
     & 20888*(v*v)-7399*(v*v*v)-716*(v*v*v*v)+
     & 90*NFV*(14-39*v+39*(v*v)-25*(v*v*v)+7*(v*v*v*v))+
     & 92*(v*v*v*v*v))-
     & omv*(u*u*u)*(-1708+9111*v-26974*(v*v)+33046*(v*v*v)-
     & 13425*(v*v*v*v)+
     & 36*nf*(-2-13*v+84*(v*v)-120*(v*v*v)+51*(v*v*v*v))-
     & 134*(v*v*v*v*v)+
     & 10*NFV*(-18+348*v-828*(v*v)+769*(v*v*v)-256*(v*v*v*v)+
     & 18*(v*v*v*v*v))+2*(v*v*v*v*v*v))+
     & u*u*u*u*(-4294+25276*v-54471*(v*v)+
     & 36*nf*(omv*omv)*(6-52*v+61*(v*v))+49534*(v*v*v)-
     & 15649*(v*v*v*v)-488*(v*v*v*v*v)+
     & 10*NFV*(-75+492*v-961*(v*v)+917*(v*v*v)-415*(v*v*v*v)+
     & 60*(v*v*v*v*v))+28*(v*v*v*v*v*v))))/
     & (36._dp*u*(omu*omu*omu)*(omv*omv)*(w*w*w)))+
     & (v*G3(3,0,2)*(omw*omw)*(10+4*(-7+5*NFV)*v+(17+3*v)*(u*u)+
     & u*u*u+17*(v*v)+u*(-28+20*NFV+34*v+3*(v*v))+v*v*v))/
     & (18._dp*(u*u)*(w*w*w))+(v*G3(3,2,0)*(omw*omw)*
     & (10+4*(-7+5*NFV)*v+(17+3*v)*(u*u)+u*u*u+17*(v*v)+
     & u*(-28+20*NFV+34*v+3*(v*v))+v*v*v))/(18._dp*(u*u)*(w*w*w))-
     & (v*G2(3,2)*H1(0)*(omw*omw)*(10+4*(-7+5*NFV)*v+(17+3*v)*(u*u)+
     & u*u*u+17*(v*v)+u*(-28+20*NFV+34*v+3*(v*v))+v*v*v))/
     & (18._dp*(u*u)*(w*w*w))-(omw*v*G3(3,3,0)*
     & (-40+108*v+(17+4*v)*(u*u*u)+u*u*u*u-2*(43+10*NFV)*(v*v)+
     & u*u*(-86-20*NFV+51*v+6*(v*v))+17*(v*v*v)+
     & u*(108-4*(43+10*NFV)*v+51*(v*v)+4*(v*v*v))+v*v*v*v))/
     & (18._dp*(u*u)*(w*w*w))+(omw*v*G2(3,3)*H1(0)*
     & (-40+108*v+(17+4*v)*(u*u*u)+u*u*u*u-2*(43+10*NFV)*(v*v)+
     & u*u*(-86-20*NFV+51*v+6*(v*v))+17*(v*v*v)+
     & u*(108-4*(43+10*NFV)*v+51*(v*v)+4*(v*v*v))+v*v*v*v))/
     & (18._dp*(u*u)*(w*w*w))+(H2(1,0)*
     & (u*u*u*u*u*u*u*u*(52-8*v-90*NFV*(1+v)+20*(v*v))+
     & u*u*u*u*u*u*u*(-474+4*(95+8*nf)*v-176*(v*v)-
     & 30*NFV*(-16-9*v+7*(v*v))+82*(v*v*v))-
     & 2*v*(omv*omv)*(92-328*v+2*nf*(-3+5*v)*(omv*omv)+
     & 372*(v*v)-137*(v*v*v)+
     & 20*NFV*(-6+16*v-13*(v*v)+4*(v*v*v))+v*v*v*v)+
     & 2*(u*u*u*u*u*u)*(1501-2349*v+1173*(v*v)-
     & 4*nf*(18-11*v+5*(v*v))-299*(v*v*v)+
     & 15*NFV*(-34+5*v+8*(v*v)+3*(v*v*v))+64*(v*v*v*v))+
     & omv*omv*(u*u)*(740+1824*v-7478*(v*v)+8680*(v*v*v)-
     & 3449*(v*v*v*v)-30*NFV*
     & (2-128*v+209*(v*v)-108*(v*v*v)+16*(v*v*v*v))+
     & 4*nf*(18-206*v+405*(v*v)-363*(v*v*v)+125*(v*v*v*v))-
     & 6*(v*v*v*v*v))+u*(omv*omv)*
     & (-200-2*(-1325+92*nf+750*NFV)*v+
     & (-6602+556*nf+3160*NFV)*(v*v)-
     & 3*(-1969+180*nf+700*NFV)*(v*v*v)+
     & (-1779+168*nf+460*NFV)*(v*v*v*v)+6*(v*v*v*v*v))+
     & u*u*u*u*u*(-6822+12777*v-7784*(v*v)+2509*(v*v*v)+
     & 4*nf*(90-49*v-46*(v*v)+29*(v*v*v))-824*(v*v*v*v)+
     & 30*NFV*(38-69*v+93*(v*v)-77*(v*v*v)+21*(v*v*v*v))+
     & 92*(v*v*v*v*v))-omv*(u*u*u)*
     & (2912+7525*v-29272*(v*v)+27518*(v*v*v)-8277*(v*v*v*v)+
     & 4*nf*(18-611*v+1345*(v*v)-1101*(v*v*v)+323*(v*v*v*v))-
     & 134*(v*v*v*v*v)+10*NFV*
     & (-30+540*v-1126*(v*v)+851*(v*v*v)-244*(v*v*v*v)+
     & 18*(v*v*v*v*v))+2*(v*v*v*v*v*v))+
     & u*u*u*u*(6614-7048*v-9739*(v*v)+14874*(v*v*v)-
     & 4209*(v*v*v*v)+4*nf*
     & (-54-325*v+963*(v*v)-837*(v*v*v)+245*(v*v*v*v))-
     & 524*(v*v*v*v*v)+10*NFV*
     & (-75+474*v-1051*(v*v)+1025*(v*v*v)-433*(v*v*v*v)+
     & 60*(v*v*v*v*v))+28*(v*v*v*v*v*v))))/
     & (36._dp*u*(omu*omu*omu)*(omv*omv)*(w*w*w))+
     & (H2(0,0)*(u*u*u*u*u*u*u*u*(52-8*v-90*NFV*(1+v)+20*(v*v))-
     & 2*(u*u*u*u*u*u*u)*(93+(750-48*nf)*v+78*(v*v)+
     & 15*NFV*(-16-3*v+7*(v*v))-41*(v*v*v))-
     & 2*(omv*omv)*(v*v)*(-194+398*v+12*nf*(omv*omv)-205*(v*v)+
     & 20*NFV*(5-10*v+4*(v*v))+v*v*v)+
     & 2*(u*u*u*u*u*u)*(-611+6293*v-3021*(v*v)+
     & 12*nf*(6-37*v+19*(v*v))-277*(v*v*v)+
     & 15*NFV*(-36+37*v-4*(v*v)+3*(v*v*v))+64*(v*v*v*v))-
     & omv*omv*(u*u)*(400-4*(2309+375*NFV)*v+
     & 2*(508+1935*NFV)*(v*v)-8*(229+345*NFV)*(v*v*v)+
     & 12*nf*(6+20*v+48*(v*v*v)-41*(v*v*v*v))+
     & 5*(661+96*NFV)*(v*v*v*v)+6*(v*v*v*v*v))+
     & u*(omv*omv)*(160+12*(-127+2*nf-20*NFV)*v+
     & 4*(-651+84*nf+370*NFV)*(v*v)+
     & (6023-540*nf-1680*NFV)*(v*v*v)+
     & (-2043+180*nf+460*NFV)*(v*v*v*v)+6*(v*v*v*v*v))+
     & u*u*u*u*u*(4182-42455*v+46412*(v*v)-12187*(v*v*v)+
     & 12*nf*(-30+275*v-334*(v*v)+113*(v*v*v))-812*(v*v*v*v)+
     & 30*NFV*(42-125*v+127*(v*v)-77*(v*v*v)+21*(v*v*v*v))+
     & 92*(v*v*v*v*v))-omv*(u*u*u)*
     & (-1708+34647*v-48350*(v*v)+32278*(v*v*v)-11565*(v*v*v*v)+
     & 12*nf*(-6-167*v+358*(v*v)-354*(v*v*v)+143*(v*v*v*v))-
     & 150*(v*v*v*v*v)+10*NFV*
     & (-18+372*v-788*(v*v)+681*(v*v*v)-226*(v*v*v*v)+
     & 18*(v*v*v*v*v))+2*(v*v*v*v*v*v))+
     & u*u*u*u*(-4294+59540*v-104335*(v*v)+66994*(v*v*v)-
     & 15813*(v*v*v*v)+12*nf*
     & (18-364*v+765*(v*v)-612*(v*v*v)+185*(v*v*v*v))-
     & 552*(v*v*v*v*v)+10*NFV*
     & (-75+528*v-997*(v*v)+899*(v*v*v)-397*(v*v*v*v)+
     & 60*(v*v*v*v*v))+28*(v*v*v*v*v*v))))/
     & (36._dp*u*(omu*omu*omu)*(omv*omv)*(w*w*w))+
     & G3(1,0,2)*((10*(-1+6*NFV)*v*H1(0))/(9._dp*(u*u))+
     & ((-1+60*NFV)*v*H1(1))/(9._dp*(u*u))+
     & (v*(-63+209*v+(25-27*v)*(u*u*u*u*u*u)-9*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(154+48*v-27*(v*v))-238*(v*v)+
     & u*(397-1078*v+934*(v*v)-244*(v*v*v))+
     & u*u*u*u*(-730+526*v+21*(v*v)-9*(v*v*v))+92*(v*v*v)-
     & 2*(u*u*u)*(-607+874*v-277*(v*v)+v*v*v)+
     & 2*(u*u)*(-494+1035*v-622*(v*v)+91*(v*v*v))+
     & 10*NFV*(2-6*v+(-25+9*v)*(u*u*u*u*u)+3*(u*u*u*u*u*u)+
     & 6*(v*v)+u*u*u*u*(77-72*v+9*(v*v))+
     & u*u*u*(-109+165*v-63*(v*v)+3*(v*v*v))+
     & 2*u*(-11+30*v-24*(v*v)+6*(v*v*v))-
     & 2*(u*u)*(-37+78*v-48*(v*v)+9*(v*v*v)))+
     & nf*(omu*omu)*(w*w*w)))/(18._dp*(omu*omu)*(u*u)*(w*w*w)))+
     & G3(1,2,0)*((2*(-1+10*NFV)*v*H1(0))/(3._dp*(u*u))+
     & ((-1+60*NFV)*v*H1(1))/(9._dp*(u*u))+
     & (v*(-63+209*v+(25-27*v)*(u*u*u*u*u*u)-9*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(154+48*v-27*(v*v))-238*(v*v)+
     & u*(397-1078*v+934*(v*v)-244*(v*v*v))+
     & u*u*u*u*(-730+526*v+21*(v*v)-9*(v*v*v))+92*(v*v*v)-
     & 2*(u*u*u)*(-607+874*v-277*(v*v)+v*v*v)+
     & 2*(u*u)*(-494+1035*v-622*(v*v)+91*(v*v*v))+
     & 10*NFV*(2-6*v+(-25+9*v)*(u*u*u*u*u)+3*(u*u*u*u*u*u)+
     & 6*(v*v)+u*u*u*u*(77-72*v+9*(v*v))+
     & u*u*u*(-109+165*v-63*(v*v)+3*(v*v*v))+
     & 2*u*(-11+30*v-24*(v*v)+6*(v*v*v))-
     & 2*(u*u)*(-37+78*v-48*(v*v)+9*(v*v*v)))+
     & nf*(omu*omu)*(w*w*w)))/(18._dp*(omu*omu)*(u*u)*(w*w*w)))+
     & G2(1,0)*(((-11+180*NFV)*v*zeta2)/(9._dp*(u*u))+
     & (5*(1-4*NFV)*v*H2(0,0))/(3._dp*(u*u))-
     & (10*(-1+6*NFV)*v*H2(0,1))/(9._dp*(u*u))+
     & ((2-20*NFV)*v*H2(1,0))/(3._dp*(u*u))+
     & ((v-60*NFV*v)*H2(1,1))/(9._dp*(u*u))-
     & (v*H1(0)*(-63+209*v+(25-27*v)*(u*u*u*u*u*u)-9*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(154+48*v-27*(v*v))-238*(v*v)+
     & u*(397-1078*v+934*(v*v)-244*(v*v*v))+
     & u*u*u*u*(-730+526*v+21*(v*v)-9*(v*v*v))+92*(v*v*v)-
     & 2*(u*u*u)*(-607+874*v-277*(v*v)+v*v*v)+
     & 2*(u*u)*(-494+1035*v-622*(v*v)+91*(v*v*v))+
     & 10*NFV*(2-6*v+(-25+9*v)*(u*u*u*u*u)+3*(u*u*u*u*u*u)+
     & 6*(v*v)+u*u*u*u*(77-72*v+9*(v*v))+
     & u*u*u*(-109+165*v-63*(v*v)+3*(v*v*v))+
     & 2*u*(-11+30*v-24*(v*v)+6*(v*v*v))-
     & 2*(u*u)*(-37+78*v-48*(v*v)+9*(v*v*v)))+
     & nf*(omu*omu)*(w*w*w)))/(18._dp*(omu*omu)*(u*u)*(w*w*w))-
     & (v*H1(1)*(-63+209*v+(25-27*v)*(u*u*u*u*u*u)-9*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(154+48*v-27*(v*v))-238*(v*v)+
     & u*(397-1078*v+934*(v*v)-244*(v*v*v))+
     & u*u*u*u*(-730+526*v+21*(v*v)-9*(v*v*v))+92*(v*v*v)-
     & 2*(u*u*u)*(-607+874*v-277*(v*v)+v*v*v)+
     & 2*(u*u)*(-494+1035*v-622*(v*v)+91*(v*v*v))+
     & 10*NFV*(2-6*v+(-25+9*v)*(u*u*u*u*u)+3*(u*u*u*u*u*u)+
     & 6*(v*v)+u*u*u*u*(77-72*v+9*(v*v))+
     & u*u*u*(-109+165*v-63*(v*v)+3*(v*v*v))+
     & 2*u*(-11+30*v-24*(v*v)+6*(v*v*v))-
     & 2*(u*u)*(-37+78*v-48*(v*v)+9*(v*v*v)))+
     & nf*(omu*omu)*(w*w*w)))/(18._dp*(omu*omu)*(u*u)*(w*w*w)))+
     & G3(1,1,2)*((-4*v*H1(0))/(u*u)-(5*(v+4*NFV*v)*H1(1))/(3._dp*(u*u))-
     & (v*(-39+97*v+(43-27*v)*(u*u*u*u*u*u)-9*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(143+102*v-27*(v*v))-68*(v*v)+
     & u*(289-670*v+504*(v*v)-132*(v*v*v))+
     & u*u*u*u*(-769+547*v+75*(v*v)-9*(v*v*v))+10*(v*v*v)+
     & u*u*u*(1193-1768*v+629*(v*v)+16*(v*v*v))+
     & u*u*(-851+1719*v-1113*(v*v)+225*(v*v*v))+
     & 10*NFV*(-2+6*v+(-25+9*v)*(u*u*u*u*u)+3*(u*u*u*u*u*u)-
     & 6*(v*v)+u*u*u*u*(62-72*v+9*(v*v))+
     & u*u*(25-78*v+75*(v*v)-18*(v*v*v))+
     & u*u*u*(-64+132*v-63*(v*v)+3*(v*v*v))+
     & u*(1+3*v-15*(v*v)+9*(v*v*v)))+4*nf*(omu*omu)*(w*w*w)))
     & /(18._dp*(omu*omu)*(u*u)*(w*w*w)))+
     & G2(1,1)*(((-19+20*NFV)*v*zeta2)/(3._dp*(u*u))+
     & ((-7+20*NFV)*v*H2(0,1))/(3._dp*(u*u))+(4*v*H2(1,0))/(u*u)+
     & (5*(v+4*NFV*v)*H2(1,1))/(3._dp*(u*u))+
     & (v*H1(0)*(-39+97*v+(43-27*v)*(u*u*u*u*u*u)-9*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(143+102*v-27*(v*v))-68*(v*v)+
     & u*(289-670*v+504*(v*v)-132*(v*v*v))+
     & u*u*u*u*(-769+547*v+75*(v*v)-9*(v*v*v))+10*(v*v*v)+
     & u*u*u*(1193-1768*v+629*(v*v)+16*(v*v*v))+
     & u*u*(-851+1719*v-1113*(v*v)+225*(v*v*v))+
     & 10*NFV*(-2+6*v+(-25+9*v)*(u*u*u*u*u)+3*(u*u*u*u*u*u)-
     & 6*(v*v)+u*u*u*u*(62-72*v+9*(v*v))+
     & u*u*(25-78*v+75*(v*v)-18*(v*v*v))+
     & u*u*u*(-64+132*v-63*(v*v)+3*(v*v*v))+
     & u*(1+3*v-15*(v*v)+9*(v*v*v)))+4*nf*(omu*omu)*(w*w*w)))
     & /(18._dp*(omu*omu)*(u*u)*(w*w*w))+
     & (v*H1(1)*(-39+97*v+(43-27*v)*(u*u*u*u*u*u)-9*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(143+102*v-27*(v*v))-68*(v*v)+
     & u*(289-670*v+504*(v*v)-132*(v*v*v))+
     & u*u*u*u*(-769+547*v+75*(v*v)-9*(v*v*v))+10*(v*v*v)+
     & u*u*u*(1193-1768*v+629*(v*v)+16*(v*v*v))+
     & u*u*(-851+1719*v-1113*(v*v)+225*(v*v*v))+
     & 10*NFV*(-2+6*v+(-25+9*v)*(u*u*u*u*u)+3*(u*u*u*u*u*u)-
     & 6*(v*v)+u*u*u*u*(62-72*v+9*(v*v))+
     & u*u*(25-78*v+75*(v*v)-18*(v*v*v))+
     & u*u*u*(-64+132*v-63*(v*v)+3*(v*v*v))+
     & u*(1+3*v-15*(v*v)+9*(v*v*v)))+4*nf*(omu*omu)*(w*w*w)))
     & /(18._dp*(omu*omu)*(u*u)*(w*w*w)))+
     & (2/omv+1/(2-2*u)+((v-2*u*v)*H1(1))/(2._dp*u*(omu*omu))+
     & G1(2)*(((-1+2*u)*v)/(2._dp*u*(omu*omu))-(v*H1(0))/(2._dp*(u*u)))+
     & G1(1)*((v*H1(0))/(2._dp*(u*u))+(v*H1(1))/(2._dp*(u*u)))-
     & (v*G2(1,2))/(2._dp*(u*u))+(v*H1(0)*
     & (-(omv*v)+(-9+v)*(u*u)+4*(u*u*u)+u*(4+2*v-2*(v*v))))/
     & (2._dp*u*(omu*omu)*(omv*omv)))*(G1(0)*G1(0))+
     & ((v-2*u*v)/(18._dp*u*(omu*omu))+(v*H1(0))/(18._dp*(u*u)))*
     & (G1(2)*G1(2)*G1(2))+(-(v*H1(1)*
     & (-17+32*v+2*(-9+v)*(u*u)+8*(u*u*u)-15*(v*v)+
     & u*(42-64*v+30*(v*v))))/(18._dp*u*(omu*omu)*(omv*omv))+
     & (-((-9+v)*(-17+2*(-20+nf)*v)*(u*u))+
     & (68-8*(-20+nf)*v)*(u*u*u)+omv*(-57+2*nf)*(v*v)+
     & u*(85+(126-8*nf)*v+(131-4*nf)*(v*v)+2*(-57+2*nf)*(v*v*v))
     & )/(18._dp*u*(omu*omu)*(omv*omv)))*(H1(0)*H1(0))+
     & G1(2)*G1(2)*(((-1+2*u)*v*H1(1))/(6._dp*u*(omu*omu))+
     & (v*G2(1,2))/(18._dp*(u*u))+H1(0)*
     & (-(v*H1(1))/(9._dp*(u*u))-(v*
     & (3*(omv*omv)+u*(-4+7*v+(-9+v)*(u*u)+4*(u*u*u)+
     & u*(3+4*v-3*(v*v))-3*(v*v))))/
     & (18._dp*(omu*omu)*(omv*omv)*(u*u)))-
     & (3*omv*v+(-9+v)*(u*u)+4*(u*u*u)+u*(5-7*v+6*(v*v)))/
     & (18._dp*omv*u*(omu*omu))-(v*(H1(0)*H1(0)))/(9._dp*(u*u)))+
     & G1(0)*(G2(1,2)*(-((-30+nf)*v)/(18._dp*(u*u))+(v*H1(0))/(u*u))+
     & (v*H2(1,1)*(2-7*u+25*(u*u)-20*(u*u*u)+
     & 10*NFV*(-12+30*u-25*(u*u)+3*(u*u*u))))/
     & (18._dp*u*(omu*omu*omu))+
     & ((-3*(-420+379*v+(-2290+2106*v)*(u*u)+
     & 4*omv*nf*(10-v+u*(-74+82*v)+64*(u*u))+92*(u*u*u)+
     & 41*(v*v)+u*(2618-5014*v+2488*(v*v))))/(omv*w*(omu*omu))
     & +6*zeta2*(-39/omv+4/(u-u*v)+82/w-
     & (2*(1+w+11*v*w))/(u*w)+39/(omv*omv)-
     & (216*nf*v*(u*u))/(omu*omu*omu*omu)+
     & (v*(3+3364*u+1693*(u*u)-2*(u*u*u)))/(omu*omu*omu*omu)+
     & 16*u*((-27*nf*v)/(omu*omu*omu*omu)-2/(omv*omv)+
     & 2/(omv*omv*omv)+5/(w*w))-80/(w*w))+
     & 30*NFV*(36/omu-9/omv-9/w+4/(omu*w)-24/(omu*omu)+
     & 6*v*zeta2*((8-12*v)/(u-u*v)-2/(omv*omv)-
     & (3*u)/(omv*omv*omv)+(6-11*u+9*(u*u))/(omu*omu*omu))
     & +2/(w*w)))/108._dp+
     & (H1(1)*(omv*omv*(-2*v*(3+NFV*(20-40*v)+4*v-8*(v*v)+v*v*v)+
     & u*(20+(7+4*nf+420*NFV)*v+(68-8*nf-540*NFV)*(v*v)+
     & (-83+4*nf+120*NFV)*(v*v*v)+6*(v*v*v*v))-
     & u*u*(150+(5+12*nf)*v+(226-16*nf)*(v*v)+
     & (-147+4*nf)*(v*v*v)+
     & 10*NFV*(-3+87*v-74*(v*v)+6*(v*v*v))+6*(v*v*v*v)))+
     & omv*(u*u*u)*(452+(-287+12*nf)*v-5*(-75+4*nf)*(v*v)+
     & (-452+8*nf)*(v*v*v)+108*(v*v*v*v)-
     & 10*NFV*(21-82*v+155*(v*v)-88*(v*v*v)+18*(v*v*v*v))-
     & 2*(v*v*v*v*v))+u*u*u*u*
     & (-708+875*v-4*nf*v-614*(v*v)+8*nf*(v*v)+
     & 4*(u*u*u)*(13-2*v+5*(v*v))+655*(v*v*v)-4*nf*(v*v*v)+
     & 2*(u*u)*(-139+68*v-56*(v*v)+31*(v*v*v))-
     & 30*NFV*(-18+22*v+3*(1+v)*(u*u*u)-43*(v*v)+
     & u*u*(-13-9*v+4*(v*v))+u*(22+3*(v*v)-7*(v*v*v))+
     & 47*(v*v*v)-14*(v*v*v*v))-298*(v*v*v*v)+
     & u*(612-520*v+332*(v*v)-298*(v*v*v)+66*(v*v*v*v))+
     & 26*(v*v*v*v*v))))/(36._dp*u*(omu*omu*omu)*(omv*omv)*(w*w))+
     & (H2(0,0)*(49/omv-28/(u-u*v)+64/w-49/(omv*omv)+
     & v*(60*NFV*(-(1/(omu*omu*omu))+1/(omv*omv))+
     & (17-8*(-203+27*nf)*u+(934-108*nf)*(u*u)-64*(u*u*u)+
     & 18*(u*u*u*u))/(omu*omu*omu*omu))+
     & (2*(-15*u*(1+(-1+4*NFV)*(v*v))+
     & omv*(14+15*(-1+4*NFV)*(v*v))))/(omv*u*w)-62/(w*w)+
     & NFV*(u*u)*((-90*v)/(omu*omu*omu)-40/(w*w*w))+
     & u*((110*NFV*v)/(omu*omu*omu)+(32-90*NFV)/(omv*omv)+
     & (-32+90*NFV)/(omv*omv*omv)+
     & (-40*NFV*(-1+w)+62*w)/(w*w*w))))/18._dp+
     & (H2(0,1)*(-33/omv+9*v-22/(u-u*v)-11/w+
     & (2-20*(-1+6*NFV)*(1+v)*w)/(u*w)-v/(omu*omu)+
     & 33/(omv*omv)+10*NFV*
     & ((6*(2+(-2+u)*v))/(u*(omv*omv))+
     & (v*(-6+11*u-9*(u*u)))/(omu*omu*omu))+9/(w*w)+
     & u*((32-90*NFV)/(omv*omv)+(-32+90*NFV)/(omv*omv*omv)+
     & (-20*NFV*(-1+w)-9*w)/(w*w*w))-(20*NFV*(u*u))/(w*w*w)))/18._dp
     & +(H2(1,0)*(-72/omv+(6*(3+(2-20*NFV)*v))/u-18/(u-u*v)+
     & 71/w+72/(omv*omv)+
     & (v*(8+(1678-216*nf)*u+(844-108*nf)*(u*u)-10*(u*u*u)+
     & 10*NFV*(-6+17*u-20*(u*u)+9*(u*u*u))+9*(u*u*u*u)))/
     & (omu*omu*omu*omu)-71/(w*w)+
     & (u*(71+(-71+20*NFV)*u+(-71+20*NFV)*v))/(w*w*w)-
     & (20*NFV*(u*u))/(w*w*w)))/18._dp+
     & H1(0)*(((-1+2*u)*v*H1(1))/(u*(omu*omu))+
     & (omv*omv*(20*nf*(18-25*v)*(u*u*u*u*u)-144*nf*(u*u*u*u*u*u)-
     & 4*nf*(u*u*u*u)*(54-86*v+141*(v*v))+
     & 2*(v*v)*(3+NFV*(20-40*v)+4*v-8*(v*v)+v*v*v)-
     & u*(340+16*(-71+2*nf-15*NFV)*v+
     & (852-24*nf+800*NFV)*(v*v)+
     & (41-12*nf-660*NFV)*(v*v*v)+
     & (-85+4*nf+120*NFV)*(v*v*v*v)+6*(v*v*v*v*v))+
     & u*u*(1444+(656-900*NFV)*v+2*(-2969+955*NFV)*(v*v)+
     & (3598-920*NFV)*(v*v*v)+3*(-51+20*NFV)*(v*v*v*v)+
     & 4*nf*(18-148*v+252*(v*v)-114*(v*v*v)+v*v*v*v)+
     & 6*(v*v*v*v*v)))+
     & u*u*u*u*(7534-23138*v+27809*(v*v)-
     & 4*(u*u*u*u)*(13-2*v+5*(v*v))+
     & u*u*u*(42+172*v+120*(v*v)-82*(v*v*v))-16494*(v*v*v)+
     & u*u*(2122-4640*v+1864*(v*v)+410*(v*v*v)-
     & 128*(v*v*v*v))+3849*(v*v*v*v)+
     & u*(-6522+18865*v-16926*(v*v)+4323*(v*v*v)+
     & 596*(v*v*v*v)-92*(v*v*v*v*v))+
     & 10*NFV*(75-204*v+9*(1+v)*(u*u*u*u)+403*(v*v)+
     & 3*(u*u*u)*(-16-9*v+7*(v*v))-503*(v*v*v)-
     & 3*(u*u)*(-36+5*v+10*(v*v)+3*(v*v*v))+
     & 289*(v*v*v*v)-
     & 9*u*(14-15*v+15*(v*v)-19*(v*v*v)+7*(v*v*v*v))-
     & 60*(v*v*v*v*v))+408*(v*v*v*v*v)-28*(v*v*v*v*v*v))+
     & omv*(u*u*u)*(-4228+4921*v+1338*(v*v)-686*(v*v*v)-
     & 1121*(v*v*v*v)+
     & 12*nf*(-6+71*v-104*(v*v)+22*(v*v*v)+17*(v*v*v*v))-
     & 114*(v*v*v*v*v)+
     & 10*NFV*(-18+150*v-356*(v*v)+315*(v*v*v)-
     & 112*(v*v*v*v)+18*(v*v*v*v*v))+2*(v*v*v*v*v*v)))/
     & (36._dp*u*(omu*omu*omu)*(omv*omv)*(w*w*w)))+
     & (v*(omv*v-(-9+v)*(u*u)-4*(u*u*u)+2*u*(-2-v+v*v))*
     & (H1(0)*H1(0)))/(u*(omu*omu)*(omv*omv))+
     & G1(1)*(H1(0)*(((-30+nf)*v)/(18._dp*(u*u))-(v*H1(1))/(u*u))+
     & ((-30+nf)*v*H1(1))/(18._dp*(u*u))-(v*(H1(0)*H1(0)))/(u*u))+
     & G1(2)*(((-30+nf)*(-1+2*u)*v)/(18._dp*u*(omu*omu))-
     & (v*H1(0)*(nf*(omu*omu)+6*(-5+7*u+u*u)))/
     & (18._dp*(omu*omu)*(u*u))+(v*(H1(0)*H1(0)))/(u*u)))+
     & (17*v*(-(omv*v)+(-9+v)*(u*u)+4*(u*u*u)+u*(4+2*v-2*(v*v)))*
     & (H1(0)*H1(0)*H1(0)))/(18._dp*u*(omu*omu)*(omv*omv))-
     & ((3*omv*v+(-9+v)*(u*u)+4*(u*u*u)+u*(5-7*v+6*(v*v)))*
     & (H1(1)*H1(1)))/(18._dp*omv*u*(omu*omu))+
     & G2(1,2)*(-((v+60*NFV*v)*H2(0,0))/(9._dp*(u*u))-
     & ((11+60*NFV)*v*H2(1,0))/(9._dp*(u*u))+
     & (v*H1(1)*(42-91*u-2*nf*(omu*omu)+56*(u*u)+10*(u*u*u)-
     & 5*NFV*(-4+17*u-22*(u*u)+3*(u*u*u))))/(9._dp*(omu*omu)*(u*u))
     & +H1(0)*((v*H1(1))/(9._dp*(u*u))+
     & (v*(85-224*u-8*(nf-5*NFV)*(omu*omu)+193*(u*u)+
     & 18*(u*u*u)))/(18._dp*(omu*omu)*(u*u)))+
     & (6*(-24-16/omu+9/omv-25/(u-u*v)+9/w+32/(omv*omv)+
     & (v*(-121+(63+4*nf)*u-11*(u*u)))/(omu*omu))+
     & (54*omw+v*w*(1119-76*nf+6*v+150*zeta2))/(w*(u*u))-
     & (6*(-33+4*(7+nf)*v+v*v))/u-
     & (12*u*(13-2*v+5*(v*v)))/(omv*omv)+
     & 30*NFV*(-6-18/omu+30/omv-21/(u-u*v)+(4+6*v+2/w)/u+
     & 2/w-(12*(-1+2*u)*v)/(omu*omu)-18/(omv*omv)+
     & (2*(-5+7/w+v*(-6+48*zeta2)-2/(w*w)))/(u*u)+
     & u*(-9/omv+18/(omv*omv)+4/(w*w))))/108._dp-
     & (17*v*(H1(0)*H1(0)))/(18._dp*(u*u))+(v*(H1(1)*H1(1)))/(18._dp*(u*u)))+
     & H1(0)*((H1(1)*(omv*(57-2*omv*nf-54*v)*v-4*(2+v)*(u*u*u)+
     & u*u*(18+7*v-v*v)+
     & 2*u*(-5+(-57+2*nf)*v+(110-4*nf)*(v*v)+
     & 2*(-27+nf)*(v*v*v))))/(18._dp*u*(omu*omu)*(omv*omv))+
     & ((4*nf*(87+19*v)-3*(927+405*v-40*NFV*(-1+3*v)+102*v*zeta2))/
     & omu+(v*(1461-52*nf-720*NFV+5076*zeta2))/(omu*omu)-
     & (54*(-281+12*nf*u*(2+u))*v*zeta2)/(omu*omu*omu*omu)-
     & (72*v*zeta2*(281+10*NFV*(2-3*u+3*(u*u))))/(omu*omu*omu)+
     & (8*u*(-1+v)*(441-38*nf+174*zeta2)*(omv*omv)*(w*w*w)+
     & 48*(-16+45*NFV)*(-1+v)*zeta2*(u*u)*(w*w*w)+
     & 8*omv*(-1+v)*(u*(-567+50*nf+6*(-29+16*u)*zeta2)-
     & 90*NFV*zeta2*(-4+4*v-2*u*v+3*(u*u)))*(w*w*w)+
     & omv*omv*omv*(w*(w*
     & (-4*nf*(-27+54*u*(-1+v)+27*v+27*w-19*w*(v*v))+
     & 9*(158*u*(-1+v)-3*(41-41*v-41*w+45*w*(v*v))))
     & +6*zeta2*(35*(-1+v)*(u*u)+
     & w*(-4+4*v+4*w-68*v*w+47*w*(v*v))+
     & u*(-1+v)*(-35+31*w+45*v*(w*w))))-
     & 10*NFV*(-3*w*(-2+7*w+u*(-1+v)*(-2+9*w)-5*(w*w)+
     & 12*(v*v)*(w*w)+v*(2-7*w+2*(w*w)))+
     & 12*zeta2*(5*(-1+v)*(-1+w)*(u*u)+5*(-1+v)*(u*u*u)+
     & 24*(-1+v*v)*(w*w*w)))))/
     & (u*(-1+v)*(omv*omv*omv)*(w*w*w)))/108._dp-
     & (v*(2-5*v+2*u*(5-3*v)*v+(-9+v)*(u*u)+4*(u*u*u)+3*(v*v))*
     & (H1(1)*H1(1)))/(18._dp*u*(omu*omu)*(omv*omv)))+
     & G1(2)*(G2(1,2)*(-v/(6._dp*(u*u))-(v*H1(0))/(9._dp*(u*u))-(v*H1(1))/(9._dp*(u*u)))-
     & ((v+60*NFV*v)*H3(0,0,0))/(9._dp*(u*u))-(2*v*H3(0,0,1))/(u*u)+
     & ((7-60*NFV)*v*H3(0,1,0))/(9._dp*(u*u))+
     & (5*(-1+4*NFV)*v*H3(1,0,0))/(3._dp*(u*u))+
     & ((-11+60*NFV)*v*H3(1,0,1))/(9._dp*(u*u))-(26*v*H3(1,1,0))/(9._dp*(u*u))+
     & (H2(1,0)*(80/omv+34/(u-u*v)-(30+2*(-7+60*NFV)*v+4/w)/u+
     & 4/w-80/(omv*omv)-(4*(-2+nf-10*NFV)*v)/(u*u)+
     & (3*v*(7+(-574+72*nf)*u+10*NFV*(-2+3*u)*(omu*omu)+
     & (-278+36*nf)*(u*u)+2*(u*u*u)))/(omu*omu*omu*omu)))/18._dp
     & +(H2(0,0)*(-49/omv+28/(u-u*v)-4/w+49/(omv*omv)+
     & u*((90*NFV*v)/(omu*omu)+((32-90*NFV)*v)/(omv*omv*omv))+
     & v*((-60*NFV)/(omu*omu)-(60*NFV)/(omv*omv)+
     & (20-4*(-413+54*nf)*u+(850-108*nf)*(u*u)+8*(u*u*u)-
     & u*u*u*u)/(omu*omu*omu*omu))+
     & (2*(-16-2*v+(60*NFV*(-2+v)*v)/omv+2/w-v*v))/u-
     & (v*(-164+12*nf-20*NFV+20*v+v*v))/(u*u)))/18._dp+
     & (H1(1)*(-30*omv*(1+4*NFV)*v*zeta2+
     & omv*u*v*(107-4*nf+60*zeta2+40*NFV*(-2+6*zeta2))+
     & 2*(-9+v)*(u*u*u)+8*(u*u*u*u)+
     & u*u*(10+v*(-214+8*nf-30*zeta2-20*NFV*(-6+6*zeta2))+
     & (212-8*nf+30*zeta2+20*NFV*(-6+6*zeta2))*(v*v))))/
     & (18._dp*omv*(omu*omu)*(u*u))+
     & (498/omu-276/omv+(1215*v)/omu-(76*nf*v)/omu+(1215*v)/u-
     & (76*nf*v)/u-(1461*v)/(omu*omu)+(52*nf*v)/(omu*omu)+
     & (612*v*zeta3)/(u*u)+10*NFV*
     & ((6*(-1-6*v+1/w+6*(-2+8*v)*zeta2))/u+
     & (54*u*v*zeta2)/(omv*omv*omv)+
     & (2*v*(6*zeta2-36*zeta3))/(u*u)+
     & 3*((-4*(5+3*v))/omu+(24*v)/(omu*omu)+
     & (9*omv*u+12*(2+(-2+u)*v)*zeta2)/(u*(omv*omv))+
     & (24*v*zeta2*(2-3*u+3*(u*u)))/(omu*omu*omu)))+
     & 6*zeta2*(39/omv+10/(u-u*v)+4/w-39/(omv*omv)-
     & (32*u*v)/(omv*omv*omv)-
     & (v*(-47+(1901-216*nf)*u+(555-108*nf)*(u*u)+
     & 119*(u*u*u)+u*u*u*u))/(omu*omu*omu*omu)+
     & (-6+23*v-4/w-2*(v*v))/u-
     & (v*(-14+4*nf+20*v+v*v))/(u*u)))/108._dp-
     & (v*H2(0,1)*(10*NFV*(omu*omu)*
     & (12*u*(omv*omv)-2*(omv*omv*omv)+6*omv*(u*u)+9*(u*u*u))
     & -32*(u*u*u*u*u)-omv*omv*omv*(-6+20*v+v*v)+
     & u*u*u*u*(104-38*v-3*(v*v)+v*v*v)-
     & 2*u*(omv*omv)*(37-44*v+18*(v*v)+v*v*v)+
     & 2*(u*u*u)*(-78+91*v-36*(v*v)+6*(v*v*v)+v*v*v*v)+
     & u*u*(170-396*v+338*(v*v)-126*(v*v*v)+13*(v*v*v*v)+
     & v*v*v*v*v)))/(18._dp*(omu*omu)*(omv*omv*omv)*(u*u))+
     & ((v*H1(1))/(9._dp*(u*u))+(v*
     & ((-57+2*nf*(omu*omu))*(omv*omv)+
     & u*(97-196*v+2*(-9+v)*(u*u)+8*(u*u*u)+
     & u*(-15+50*v-27*(v*v))+99*(v*v))))/
     & (18._dp*(omu*omu)*(omv*omv)*(u*u)))*(H1(0)*H1(0))-
     & (17*v*(H1(0)*H1(0)*H1(0)))/(18._dp*(u*u))+
     & ((v-2*u*v)*(H1(1)*H1(1)))/(6._dp*u*(omu*omu))+
     & H1(0)*((v*H1(1)*(-2*omv*u+3*(omv*omv)+
     & u*u*(3+2*u*(-9+v)+14*v+8*(u*u)-9*(v*v))))/
     & (18._dp*(omu*omu)*(omv*omv)*(u*u))+
     & (v*(-76*nf+18*(7+160*NFV)*zeta2)*(omv*omv*omv*omv)-
     & 3*v*(omv*omv*omv)*(-391+371*v+2*(v*v))+
     & omv*omv*(-60*NFV*v*(3-7*v+6*(v*v))-
     & 2*u*(-51+v*(2397-128*nf+252*zeta2)+
     & 3*(-1052+60*nf-126*zeta2)*(v*v)+
     & (828-52*nf+126*zeta2)*(v*v*v)+
     & 15*NFV*(9+v*(-33+384*zeta2)+(40-576*zeta2)*(v*v)+
     & 2*(-7+96*zeta2)*(v*v*v))+9*(v*v*v*v))+
     & u*u*(-234+6*v*(1111-44*nf+126*zeta2)+
     & 6*(-696+28*nf-126*zeta2)*(v*v)+
     & (-303+20*nf+126*zeta2)*(v*v*v)+
     & 60*NFV*(15+3*v*(-5+96*zeta2)-4*(1+72*zeta2)*(v*v)+
     & (6+48*zeta2)*(v*v*v))+18*(v*v*v*v)))+
     & u*u*u*u*(1188+v*(20*nf+
     & 3*(-545+42*zeta2+20*NFV*(21+48*zeta2)))+
     & 6*(u*u*u)*(-26+4*v+45*NFV*(1+v)-10*(v*v))-
     & 2*(20*nf+3*(-141+42*zeta2+80*NFV*(9+12*zeta2)))*
     & (v*v)+6*(u*u)*
     & (129-120*v+56*(v*v)+30*NFV*(-5-3*v+2*(v*v))-
     & 31*(v*v*v))+
     & (20*nf+3*(-465+42*zeta2+160*NFV*(9+6*zeta2)))*
     & (v*v*v)+(894-1260*NFV)*(v*v*v*v)-
     & 6*u*(243-363*v+208*(v*v)-149*(v*v*v)+
     & 15*NFV*(-11+5*v-7*(v*v)+7*(v*v*v))+33*(v*v*v*v))-
     & 78*(v*v*v*v*v))+
     & 2*omv*(u*u*u)*(-108+2*v*(-843+16*nf-126*zeta2)+
     & 3*(521-4*nf+126*zeta2)*(v*v)+
     & (354-20*nf-126*zeta2)*(v*v*v)-162*(v*v*v*v)+
     & 15*NFV*(-33-2*v*(3+192*zeta2)+16*(7+36*zeta2)*(v*v)-
     & 2*(47+96*zeta2)*(v*v*v)+18*(v*v*v*v))+3*(v*v*v*v*v)))/
     & (108._dp*(omu*omu)*(omv*omv)*(u*u)*(w*w))+
     & (v*(H1(1)*H1(1)))/(18._dp*(u*u))))+
     & ((-1+2*u)*v*(H1(1)*H1(1)*H1(1)))/(18._dp*u*(omu*omu))+
     & G1(1)*(((v+60*NFV*v)*H3(0,0,0))/(9._dp*(u*u))+(2*v*H3(0,0,1))/(u*u)+
     & ((-7+60*NFV)*v*H3(0,1,0))/(9._dp*(u*u))+
     & ((v+60*NFV*v)*H3(1,0,0))/(9._dp*(u*u))+
     & ((11+60*NFV)*v*H3(1,1,0))/(9._dp*(u*u))+
     & (v*H2(1,0)*(-71+169*u+5*(nf-4*NFV)*(omu*omu)-125*(u*u)-
     & 9*(u*u*u)))/(9._dp*(omu*omu)*(u*u))+
     & (v*H2(1,1)*(-81+176*u+4*nf*(omu*omu)-109*(u*u)-20*(u*u*u)+
     & 10*NFV*(-4+17*u-22*(u*u)+3*(u*u*u))))/(18._dp*(omu*omu)*(u*u))
     & +(H1(1)*(6*(24+16/omu-9/omv+25/(u-u*v)-9/w-
     & 32/(omv*omv)+(v*(121-(63+4*nf)*u+11*(u*u)))/
     & (omu*omu))+((54*(-1+w))/w+
     & v*(-1119+76*nf-150*zeta2)-6*(v*v))/(u*u)+
     & (6*(-33+4*(7+nf)*v+v*v))/u+
     & (12*u*(13-2*v+5*(v*v)))/(omv*omv)+
     & 30*NFV*(6+18/omu-30/omv+21/(u-u*v)-(4+6*v+2/w)/u-
     & 2/w+(12*(-1+2*u)*v)/(omu*omu)+18/(omv*omv)+
     & u*(9/omv-18/(omv*omv)-4/(w*w))+
     & (2*(5-7/w+v*(6-48*zeta2)+2/(w*w)))/(u*u))))/108._dp+
     & (v*H2(0,0)*(-164+512*v+9*(-1+3*v)*(u*u*u*u)+9*(u*u*u*u*u)-
     & 541*(v*v)+u*u*u*(173+27*(v*v))+193*(v*v*v)+
     & 20*NFV*(-1+3*v+(-3+6*v)*(u*u)+u*u*u-3*(v*v)+
     & u*(3-6*v+6*(v*v))+2*(v*v*v))+
     & u*u*(-501+503*v+27*(v*v)+9*(v*v*v))+
     & u*(492-1024*v+523*(v*v)+18*(v*v*v))+12*nf*(w*w*w)))/
     & (18._dp*(u*u)*(w*w*w))+(v*H2(0,1)*
     & (-103+329*v+(-25+27*v)*(u*u*u*u*u*u)+9*(u*u*u*u*u*u*u)-
     & 358*(v*v)+u*u*u*u*u*(53-48*v+27*(v*v))+
     & u*(475-1196*v+936*(v*v)-206*(v*v*v))+132*(v*v*v)+
     & u*u*u*(695-692*v+67*(v*v)+2*(v*v*v))+
     & u*u*u*u*(-265+95*v-21*(v*v)+9*(v*v*v))+
     & u*u*(-839+1485*v-651*(v*v)+25*(v*v*v))-
     & 10*NFV*(2-6*v+(-29+9*v)*(u*u*u*u*u)+3*(u*u*u*u*u*u)+
     & 6*(v*v)+u*u*u*u*(82-84*v+9*(v*v))+
     & u*u*(65-150*v+111*(v*v)-22*(v*v*v))-4*(v*v*v)+
     & u*u*u*(-104+180*v-75*(v*v)+3*(v*v*v))+
     & u*(-19+51*v-51*(v*v)+17*(v*v*v)))+
     & 6*nf*(omu*omu)*(w*w*w)))/(18._dp*(omu*omu)*(u*u)*(w*w*w))-
     & (v*(12*(11+60*NFV)*zeta3*(omu*omu)*(w*w*w)+
     & 6*zeta2*(-129+547*v-4*(-31+27*v)*(u*u*u*u*u*u)-
     & 36*(u*u*u*u*u*u*u)+u*u*u*u*u*(371+264*v-108*(v*v))-
     & 779*(v*v)+u*(991-3202*v+3183*(v*v)-900*(v*v*v))+
     & u*u*u*u*(-2101+1441*v+156*(v*v)-36*(v*v*v))+
     & 361*(v*v*v)+u*u*u*(3506-5350*v+1625*(v*v)+16*(v*v*v))+
     & u*u*(-2726+6408*v-4077*(v*v)+555*(v*v*v))+
     & 10*NFV*((-7+9*v)*(u*u*u*u*u)+3*(u*u*u*u*u*u)+
     & u*u*u*u*(62-36*v+9*(v*v))+
     & u*u*(139-240*v+75*(v*v)-6*(v*v*v))+
     & u*(-47+129*v-69*(v*v)+3*(v*v*v))+
     & u*u*u*(-154+150*v-27*(v*v)+3*(v*v*v))+
     & 4*(1-3*v+3*(v*v)+3*(v*v*v)))-
     & 20*nf*(omu*omu)*(w*w*w))))/(108._dp*(omu*omu)*(u*u)*(w*w*w))+
     & (-(v*H1(0))/(18._dp*(u*u))-(v*H1(1))/(18._dp*(u*u)))*(G1(2)*G1(2))+
     & (((57-2*nf)*v)/(18._dp*(u*u))+(5*v*H1(1))/(6._dp*(u*u)))*(H1(0)*H1(0))+
     & (17*v*(H1(0)*H1(0)*H1(0)))/(18._dp*(u*u))-(v*(H1(1)*H1(1)))/(6._dp*(u*u))+
     & H1(0)*(-((-27+nf)*v*H1(1))/(9._dp*(u*u))+
     & (6*(24+16/omu-9/omv+25/(u-u*v)-9/w-32/(omv*omv)+
     & (v*(121-(63+4*nf)*u+11*(u*u)))/(omu*omu))+
     & ((54*(-1+w))/w+v*(-1119+76*nf+282*zeta2)-6*(v*v))/
     & (u*u)+(6*(-33+4*(7+nf)*v+v*v))/u+
     & (12*u*(13-2*v+5*(v*v)))/(omv*omv)+
     & 30*NFV*(6+18/omu-30/omv+21/(u-u*v)-
     & (4+6*v+2/w)/u-2/w+(12*(-1+2*u)*v)/(omu*omu)+
     & 18/(omv*omv)+u*(9/omv-18/(omv*omv)-4/(w*w))+
     & (2*(5-7/w+v*(6-48*zeta2)+2/(w*w)))/(u*u)))/108._dp-
     & (v*(H1(1)*H1(1)))/(6._dp*(u*u)))+
     & G1(2)*(H1(0)*(v/(6._dp*(u*u))+(2*v*H1(1))/(9._dp*(u*u)))+
     & (v*H1(1))/(6._dp*(u*u))+(v*(H1(0)*H1(0)))/(9._dp*(u*u))+
     & (v*(H1(1)*H1(1)))/(9._dp*(u*u)))-(v*(H1(1)*H1(1)*H1(1)))/(18._dp*(u*u)))
      return
      end
      
      
      function Gamma_4a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4,nf,NFV)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw,NFV,nf
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(2*pi*v*G3(0,1,0))/(u*u)-(pi*v*G3(0,1,1))/(u*u)+(pi*v*G3(0,1,2))/(u*u)+
     & ((1-60*NFV)*pi*v*G3(1,0,2))/(9._dp*(u*u))+
     & ((-17+60*NFV)*pi*v*G3(1,0,3))/(9._dp*(u*u))+
     & ((7-20*NFV)*pi*v*G3(1,1,0))/(3._dp*(u*u))+
     & ((-19+20*NFV)*pi*v*G3(1,1,1))/(3._dp*(u*u))+(4*pi*v*G3(1,1,2))/(u*u)+
     & ((1-60*NFV)*pi*v*G3(1,2,0))/(9._dp*(u*u))+
     & (5*(1+4*NFV)*pi*v*G3(1,2,1))/(3._dp*(u*u))+
     & ((1-60*NFV)*pi*v*G3(1,2,2))/(9._dp*(u*u))+
     & ((-17+60*NFV)*pi*v*G3(1,2,3))/(9._dp*(u*u))+
     & ((-11+60*NFV)*pi*v*G3(2,0,1))/(9._dp*(u*u))-
     & ((1+20*NFV)*pi*v*G3(2,0,3))/(3._dp*(u*u))-(pi*v*G3(2,1,0))/(u*u)+
     & (4*pi*v*G3(2,1,1))/(u*u)-(pi*v*G3(2,1,2))/(u*u)+
     & ((1-20*NFV)*pi*v*G3(2,2,1))/(3._dp*(u*u))+
     & ((1+20*NFV)*pi*v*G3(2,2,3))/(3._dp*(u*u))+(20*pi*v*G3(2,3,3))/(9._dp*(u*u))+
     & (pi*v*H2(1,1)*(2-7*u+25*(u*u)-20*(u*u*u)+
     & 10*NFV*(-12+30*u-25*(u*u)+3*(u*u*u))))/(18._dp*u*(omu*omu*omu))
     & +G2(0,2)*((pi*v*H1(0))/(u*u)+
     & (pi*v*(2-7*u+25*(u*u)-20*(u*u*u)+
     & 10*NFV*(-12+30*u-25*(u*u)+3*(u*u*u))))/
     & (18._dp*u*(omu*omu*omu)))+
     & G2(2,0)*((-10*(-1+6*NFV)*pi*v*H1(0))/(9._dp*(u*u))+
     & (pi*v*(2-7*u+25*(u*u)-20*(u*u*u)+
     & 10*NFV*(-12+30*u-25*(u*u)+3*(u*u*u))))/
     & (18._dp*u*(omu*omu*omu)))+
     & G2(2,2)*(((1+60*NFV)*pi*v*H1(0))/(9._dp*(u*u))+
     & (pi*v*(2-7*u+25*(u*u)-20*(u*u*u)+
     & 10*NFV*(-12+30*u-25*(u*u)+3*(u*u*u))))/
     & (18._dp*u*(omu*omu*omu)))+
     & (pi*G2(2,3)*(39/omv+10/(u-u*v)+
     & (2*(-3-17*v-(60*NFV*(-2+v)*v)/omv-2/w))/u+4/w-
     & 39/(omv*omv)+u*((-90*NFV*v)/(omu*omu)+
     & (2*(-16+45*NFV)*v)/(omv*omv*omv))+
     & v*(60*NFV*(1/(omu*omu)+1/(omv*omv))+
     & (-21+2*(-815+108*nf)*u+4*(-223+27*nf)*(u*u)+14*(u*u*u))/
     & (omu*omu*omu*omu))))/18._dp+
     & G2(0,1)*(-((pi*v*H1(0))/(u*u))-(pi*v*H1(1))/(u*u)+
     & (pi*v*(2*u*(-38-15*NFV*(-5+v)+26*v)*(omv*omv)+
     & (16+nf*(omu*omu))*(omv*omv*omv)+
     & omv*(u*u)*(124-118*v+60*NFV*(-3+4*v)+35*(v*v))+
     & u*u*u*(-156+105*u+184*v-41*u*v-32*(u*u)-78*(v*v)+
     & 30*NFV*(3-4*v-2*u*(2+v)+3*(u*u)+4*(v*v))+18*(v*v*v))
     & ))/(18._dp*(omu*omu)*(omv*omv*omv)*(u*u)))+
     & G2(2,1)*(-((pi*v*H1(0))/(u*u))+(pi*v*H1(1))/(u*u)-
     & (pi*v*(87-250*u-281*v+764*u*v-
     & 4*nf*(omu*omu)*(omv*omv*omv)+279*(u*u)-723*v*(u*u)-
     & 136*(u*u*u)+122*v*(u*u*u)+104*(u*u*u*u)-38*v*(u*u*u*u)-
     & 32*(u*u*u*u*u)+320*(v*v)-814*u*(v*v)+665*(u*u)*(v*v)-
     & 12*(u*u*u)*(v*v)-3*(u*u*u*u)*(v*v)-144*(v*v*v)+
     & 334*u*(v*v*v)-235*(u*u)*(v*v*v)-8*(u*u*u)*(v*v*v)+
     & u*u*u*u*(v*v*v)+10*NFV*
     & (u*(-1+13*v)*(omv*omv)+2*(omv*omv*omv)+
     & u*u*(2-18*v-6*(2+v)*(u*u)+9*(u*u*u)+36*(v*v)-
     & 20*(v*v*v)+3*u*(2-v+v*v+v*v*v)))+17*(v*v*v*v)-
     & 32*u*(v*v*v*v)+13*(u*u)*(v*v*v*v)+2*(u*u*u)*(v*v*v*v)+
     & v*v*v*v*v-2*u*(v*v*v*v*v)+u*u*(v*v*v*v*v)))/
     & (18._dp*(omu*omu)*(omv*omv*omv)*(u*u)))+
     & (pi*H2(1,0)*(-31/omv+6/(u-u*v)+
     & (2*(-2+6*v+(60*NFV*(-2+v)*v)/omv-1/w))/u+82/w+
     & 31/(omv*omv)-(90*NFV*v*(u*u))/(omu*omu*omu)+
     & v*((-60*NFV)/(omu*omu*omu)-(60*NFV)/(omv*omv)+
     & (-1-8*(-212+27*nf)*u+(826-108*nf)*(u*u)+8*(u*u*u))/
     & (omu*omu*omu*omu))+
     & u*((110*NFV*v)/(omu*omu*omu)-32/(omv*omv)+
     & (32-90*NFV*v)/(omv*omv*omv)+80/(w*w))-80/(w*w)))/18._dp-
     & (pi*H1(1)*(omv*omv*(2*v*(-62+134*v-73*(v*v)+
     & 20*NFV*(5-10*v+4*(v*v))+v*v*v)-
     & u*(20+(-639+4*nf+1140*NFV)*v-
     & 4*(-275+2*nf+415*NFV)*(v*v)+
     & (-469+4*nf+520*NFV)*(v*v*v)+6*(v*v*v*v))+
     & u*u*(150+(-1153+12*nf)*v-2*(-755+8*nf)*(v*v)+
     & (-403+4*nf)*(v*v*v)+
     & 10*NFV*(-3+207*v-202*(v*v)+30*(v*v*v))+6*(v*v*v*v)))+
     & u*u*u*u*(708-1131*v+4*nf*v+1126*(v*v)-8*nf*(v*v)-
     & 4*(u*u*u)*(13-2*v+5*(v*v))-911*(v*v*v)+4*nf*(v*v*v)-
     & 2*(u*u)*(-139+68*v-56*(v*v)+31*(v*v*v))+
     & u*(-612+520*v-332*(v*v)+298*(v*v*v)-66*(v*v*v*v))+
     & 30*NFV*(-18+30*v+3*(1+v)*(u*u*u)-59*(v*v)+
     & u*u*(-13-9*v+4*(v*v))+u*(22+3*(v*v)-7*(v*v*v))+
     & 55*(v*v*v)-14*(v*v*v*v))+298*(v*v*v*v)-26*(v*v*v*v*v))+
     & omv*(u*u*u)*(-452-3*(-395+4*nf)*v+5*(-357+4*nf)*(v*v)+
     & (964-8*nf)*(v*v*v)-108*(v*v*v*v)+
     & 10*NFV*(21-170*v+291*(v*v)-136*(v*v*v)+18*(v*v*v*v))+
     & 2*(v*v*v*v*v))))/(36._dp*u*(omu*omu*omu)*(omv*omv)*(w*w))+
     & G1(0)*(G1(1)*(-((-30+nf)*pi*v)/(18._dp*(u*u))+(pi*v*H1(0))/(u*u))-
     & (pi*v*H1(1)*(2-7*u+25*(u*u)-20*(u*u*u)+
     & 10*NFV*(-12+30*u-25*(u*u)+3*(u*u*u))))/
     & (18._dp*u*(omu*omu*omu))+
     & (pi*(omv*omv*(2*v*(3+NFV*(20-40*v)+4*v-8*(v*v)+v*v*v)-
     & u*(20+(7+4*nf+420*NFV)*v+(68-8*nf-540*NFV)*(v*v)+
     & (-83+4*nf+120*NFV)*(v*v*v)+6*(v*v*v*v))+
     & u*u*(150+(5+12*nf)*v+(226-16*nf)*(v*v)+
     & (-147+4*nf)*(v*v*v)+
     & 10*NFV*(-3+87*v-74*(v*v)+6*(v*v*v))+6*(v*v*v*v)))+
     & u*u*u*u*(708-875*v+4*nf*v+614*(v*v)-8*nf*(v*v)-
     & 4*(u*u*u)*(13-2*v+5*(v*v))-655*(v*v*v)+4*nf*(v*v*v)-
     & 2*(u*u)*(-139+68*v-56*(v*v)+31*(v*v*v))+
     & u*(-612+520*v-332*(v*v)+298*(v*v*v)-66*(v*v*v*v))+
     & 30*NFV*(-18+22*v+3*(1+v)*(u*u*u)-43*(v*v)+
     & u*u*(-13-9*v+4*(v*v))+u*(22+3*(v*v)-7*(v*v*v))+
     & 47*(v*v*v)-14*(v*v*v*v))+298*(v*v*v*v)-26*(v*v*v*v*v))
     & +omv*(u*u*u)*(-452+(287-12*nf)*v+5*(-75+4*nf)*(v*v)+
     & (452-8*nf)*(v*v*v)-108*(v*v*v*v)+
     & 10*NFV*(21-82*v+155*(v*v)-88*(v*v*v)+18*(v*v*v*v))+
     & 2*(v*v*v*v*v))))/(36._dp*u*(omu*omu*omu)*(omv*omv)*(w*w))+
     & (pi*H1(0)*(22/(u-u*v)+11/w-(2*(1+(10+v)*w))/(u*w)-
     & (8*v)/(omu*omu)+(60*NFV*v)/(omu*omu*omu)-
     & (33+32*u-90*NFV*u+60*NFV*v)/(omv*omv)+
     & ((32-90*NFV)*u)/(omv*omv*omv)+
     & (33-(120*NFV*(v*v))/u)/omv-9/(w*w)+
     & u*u*((-9*v)/(omu*omu)+(90*NFV*v)/(omu*omu*omu)+
     & (20*NFV)/(w*w*w))+
     & u*((-110*NFV*v)/(omu*omu*omu)+(20*NFV*(-1+w)+9*w)/(w*w*w))
     & ))/18._dp)+(pi*H2(0,0)*
     & (39/omv+10/(u-u*v)+75/w-39/(omv*omv)+
     & v*(60*NFV*(-(1/(omu*omu*omu))+1/(omv*omv))+
     & (8+(1678-216*nf)*u+(844-108*nf)*(u*u)-10*(u*u*u)+
     & 9*(u*u*u*u))/(omu*omu*omu*omu))+
     & (2*(-3*u*(-1+(1+20*NFV)*(v*v))+omv*(-5+(3+60*NFV)*(v*v))))/
     & (omv*u*w)-71/(w*w)+
     & NFV*(u*u)*((-90*v)/(omu*omu*omu)-20/(w*w*w))+
     & u*((110*NFV*v)/(omu*omu*omu)+(32-90*NFV)/(omv*omv)+
     & (-32+90*NFV)/(omv*omv*omv)+(-20*NFV*(-1+w)+71*w)/(w*w*w)
     & )))/18._dp+(pi*H2(0,1)*
     & (-33/omv+9*v-22/(u-u*v)-11/w+
     & (2-20*(-1+6*NFV)*(1+v)*w)/(u*w)-v/(omu*omu)+
     & 33/(omv*omv)+10*NFV*
     & ((6*(2+(-2+u)*v))/(u*(omv*omv))+
     & (v*(-6+11*u-9*(u*u)))/(omu*omu*omu))+9/(w*w)+
     & u*((32-90*NFV)/(omv*omv)+(-32+90*NFV)/(omv*omv*omv)+
     & (-20*NFV*(-1+w)-9*w)/(w*w*w))-(20*NFV*(u*u))/(w*w*w)))/18._dp+
     & (pi*G2(0,3)*(-39/omv-10/(u-u*v)-75/w+39/(omv*omv)+
     & v*(60*NFV*(1/(omu*omu)-1/(omv*omv))+
     & (-30+2*(-797+108*nf)*u+2*(-473+54*nf)*(u*u)+50*(u*u*u)-
     & 9*(u*u*u*u))/(omu*omu*omu*omu))+
     & (6-34*v+4/w-(120*NFV*(v*v))/omv)/u+71/(w*w)+
     & u*((-90*NFV*v)/(omu*omu)+(-32+90*NFV)/(omv*omv)+
     & (32-90*NFV)/(omv*omv*omv)+(20*NFV*(-1+w)-71*w)/(w*w*w))
     & +(20*NFV*(u*u))/(w*w*w)))/18._dp+
     & (pi*((-60*NFV)/u+(1215*v)/u-(76*nf*v)/u-(360*NFV*v)/u-1422/w+
     & (216*nf)/w-(270*NFV)/w+(60*NFV)/(u*w)-(144*zeta2)/u+
     & (1440*NFV*zeta2)/u-54*v*zeta2+(222*v*zeta2)/u+
     & (1440*NFV*v*zeta2)/u+(168*zeta2)/(u-u*v)+(558*zeta2)/w-
     & (24*zeta2)/(u*w)+(2*(-504+48*nf+6*zeta2))/omv+
     & (2529+7710*w+1215*v*w-120*NFV*(-1-4*w+3*v*w)-
     & 4*nf*(81+(240+19*v)*w)+354*v*w*zeta2)/(omu*w)+
     & (-7587+720*NFV*(-1+v)-1461*v+nf*(972+52*v)+9906*v*zeta2)/
     & (omu*omu)-(108*(-281+12*nf*u*(2+u))*v*zeta2)/
     & (omu*omu*omu*omu)+
     & (24*(16-45*NFV)*u*zeta2)/(omv*omv*omv)+
     & (72*v*zeta2*(-562+5*NFV*(2-3*u+3*(u*u))))/(omu*omu*omu)+
     & (12*zeta2*(-(u*(1+32*u))+30*NFV*(-4+4*v-2*u*v+3*(u*u))))/
     & (u*(omv*omv))+(60*NFV)/(w*w)-(534*zeta2)/(w*w)+
     & (534*u*zeta2)/(w*w)+(120*NFV*u*zeta2)/(w*w)-
     & (120*NFV*u*zeta2)/(w*w*w)+(120*NFV*zeta2*(u*u))/(w*w*w)))/108._dp+
     & G2(1,2)*((5*(-1+4*NFV)*pi*v*H1(0))/(3._dp*(u*u))+
     & ((-1+60*NFV)*pi*v*H1(1))/(9._dp*(u*u))+
     & (pi*v*(-75+245*v-9*(-5+3*v)*(u*u*u*u*u*u)-9*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(134+108*v-27*(v*v))-274*(v*v)+
     & u*(471-1264*v+1084*(v*v)-282*(v*v*v))+
     & u*u*u*u*(-828+526*v+81*(v*v)-9*(v*v*v))+104*(v*v*v)+
     & 2*(u*u*u)*(720-1021*v+307*(v*v)+9*(v*v*v))+
     & 2*(u*u)*(-589+1227*v-739*(v*v)+111*(v*v*v))+
     & 10*NFV*(-2+6*v+3*(-5+4*v)*(u*u*u*u)+6*(u*u*u*u*u)-
     & 6*(v*v)+3*(u*u*u)*(5-9*v+4*(v*v))+
     & u*(7-15*v+15*(v*v)-5*(v*v*v))+4*(v*v*v)+
     & u*u*(-11+24*v-21*(v*v)+4*(v*v*v)))))/
     & (18._dp*(omu*omu)*(u*u)*(w*w*w)))+
     & G1(3)*(-(pi*v*H1(0)*(omw*omw)*
     & (10+4*(-7+5*NFV)*v+(17+3*v)*(u*u)+u*u*u+17*(v*v)+
     & u*(-28+20*NFV+34*v+3*(v*v))+v*v*v))/(18._dp*(u*u)*(w*w*w))-
     & (pi*v*H1(1)*(omw*omw)*(10+4*(-7+5*NFV)*v+(17+3*v)*(u*u)+
     & u*u*u+17*(v*v)+u*(-28+20*NFV+34*v+3*(v*v))+v*v*v))/
     & (18._dp*(u*u)*(w*w*w))+(pi*
     & (u*u*u*u*u*u*u*u*(-52+8*v+90*NFV*(1+v)-20*(v*v))+
     & 2*(u*u*u*u*u*u*u)*(93-58*v+70*(v*v)+
     & 15*NFV*(-16-3*v+7*(v*v))-41*(v*v*v))+
     & 2*(omv*omv)*(v*v)*(-6+22*v-17*(v*v)+
     & 20*NFV*(6-13*v+6*(v*v))+v*v*v)+
     & 2*omv*(u*u*u*u*u*u)*
     & (611-72*omv*nf-672*v-181*(v*v)+
     & 15*NFV*(36+v+3*(v*v))+64*(v*v*v))+
     & u*u*u*u*u*(-4182+16919*v-36*nf*(-10+29*v)*(omv*omv)-
     & 20888*(v*v)+7399*(v*v*v)+716*(v*v*v*v)-
     & 90*NFV*(14-39*v+39*(v*v)-25*(v*v*v)+7*(v*v*v*v))-
     & 92*(v*v*v*v*v))-
     & u*(omv*omv)*(160-4*(-23+18*nf+60*NFV)*v+
     & 4*(-514+72*nf+445*NFV)*(v*v)+
     & (2803-324*nf-2220*NFV)*(v*v*v)+
     & 3*(-329+36*nf+220*NFV)*(v*v*v*v)+6*(v*v*v*v*v))+
     & omv*omv*(u*u)*(400-16*(-49+90*NFV)*v+
     & 10*(50+453*NFV)*(v*v)-40*(152+87*NFV)*(v*v*v)+
     & (4001+600*NFV)*(v*v*v*v)-
     & 36*nf*(-2+10*v-24*(v*v*v)+15*(v*v*v*v))+6*(v*v*v*v*v))
     & +u*u*u*u*(4294-25276*v+54471*(v*v)-
     & 36*nf*(omv*omv)*(6-52*v+61*(v*v))-49534*(v*v*v)+
     & 15649*(v*v*v*v)+488*(v*v*v*v*v)-
     & 10*NFV*(-75+492*v-961*(v*v)+917*(v*v*v)-415*(v*v*v*v)+
     & 60*(v*v*v*v*v))-28*(v*v*v*v*v*v))+
     & omv*(u*u*u)*(-1708+9111*v-26974*(v*v)+33046*(v*v*v)-
     & 13425*(v*v*v*v)+
     & 36*nf*(-2-13*v+84*(v*v)-120*(v*v*v)+51*(v*v*v*v))-
     & 134*(v*v*v*v*v)+
     & 10*NFV*(-18+348*v-828*(v*v)+769*(v*v*v)-256*(v*v*v*v)+
     & 18*(v*v*v*v*v))+2*(v*v*v*v*v*v))))/
     & (36._dp*u*(omu*omu*omu)*(omv*omv)*(w*w*w)))+
     & H1(0)*((10*pi*(1-2*u)*v*H1(1))/(9._dp*u*(omu*omu))+
     & (pi*(u*u*u*u*u*u*u*u*(-52+8*v+90*NFV*(1+v)-20*(v*v))+
     & 2*(u*u*u*u*u*u*u)*(165+2*(-149+8*nf)*v+70*(v*v)+
     & 15*NFV*(-16-3*v+7*(v*v))-41*(v*v*v))+
     & 2*v*(omv*omv)*(52+8*(-22+15*NFV)*v-4*nf*(omv*omv)+
     & (206-220*NFV)*(v*v)+(-83+80*NFV)*(v*v*v)+v*v*v*v)-
     & 2*(u*u*u*u*u*u)*(-161-493*v+331*(v*v)+
     & 4*nf*(18-11*v+5*(v*v))-245*(v*v*v)+
     & 15*NFV*(-36+35*v-2*(v*v)+3*(v*v*v))+64*(v*v*v*v))+
     & u*u*u*u*u*(-1842+1917*v-3016*(v*v)+1181*(v*v*v)-
     & 4*nf*(-90+173*v-202*(v*v)+95*(v*v*v))+716*(v*v*v*v)-
     & 30*NFV*(42-107*v+97*(v*v)-65*(v*v*v)+21*(v*v*v*v))-
     & 92*(v*v*v*v*v))-
     & u*(omv*omv)*(-20-12*(-57+4*nf+25*NFV)*v+
     & 4*(-413+20*nf+440*NFV)*(v*v)+
     & (1387-36*nf-1920*NFV)*(v*v*v)+
     & (-387+4*nf+520*NFV)*(v*v*v*v)+6*(v*v*v*v*v))+
     & omv*omv*(u*u)*(-644+(8154-1320*NFV)*v+
     & 2*(-5711+1795*NFV)*(v*v)+(5158-2360*NFV)*(v*v*v)+
     & 25*(-13+12*NFV)*(v*v*v*v)+
     & 4*nf*(18-226*v+306*(v*v)-120*(v*v*v)+v*v*v*v)+
     & 6*(v*v*v*v*v))+u*u*u*u*
     & (1054+3858*v-3771*(v*v)-3100*(v*v*v)+1899*(v*v*v*v)-
     & 4*nf*(54-4*v+67*(v*v)-236*(v*v*v)+127*(v*v*v*v))+
     & 488*(v*v*v*v*v)-
     & 10*NFV*(-75+408*v-703*(v*v)+653*(v*v*v)-325*(v*v*v*v)+
     & 60*(v*v*v*v*v))-28*(v*v*v*v*v*v))+
     & omv*(u*u*u)*(812-14183*v+19040*(v*v)-6392*(v*v*v)-
     & 403*(v*v*v*v)+4*nf*
     & (-18+373*v-492*(v*v)+114*(v*v*v)+49*(v*v*v*v))-
     & 134*(v*v*v*v*v)+
     & 10*NFV*(-18+276*v-574*(v*v)+497*(v*v*v)-166*(v*v*v*v)+
     & 18*(v*v*v*v*v))+2*(v*v*v*v*v*v))))/
     & (36._dp*u*(omu*omu*omu)*(omv*omv)*(w*w*w)))+
     & (pi*v*G2(3,0)*(omw*omw)*(10+4*(-7+5*NFV)*v+(17+3*v)*(u*u)+
     & u*u*u+17*(v*v)+u*(-28+20*NFV+34*v+3*(v*v))+v*v*v))/
     & (18._dp*(u*u)*(w*w*w))+(pi*v*G2(3,2)*(omw*omw)*
     & (10+4*(-7+5*NFV)*v+(17+3*v)*(u*u)+u*u*u+17*(v*v)+
     & u*(-28+20*NFV+34*v+3*(v*v))+v*v*v))/(18._dp*(u*u)*(w*w*w))-
     & (omw*pi*v*G2(3,3)*(-40+108*v+(17+4*v)*(u*u*u)+u*u*u*u-
     & 2*(43+10*NFV)*(v*v)+u*u*(-86-20*NFV+51*v+6*(v*v))+
     & 17*(v*v*v)+u*(108-4*(43+10*NFV)*v+51*(v*v)+4*(v*v*v))+
     & v*v*v*v))/(18._dp*(u*u)*(w*w*w))+
     & G2(1,0)*((10*(-1+6*NFV)*pi*v*H1(0))/(9._dp*(u*u))+
     & ((-1+60*NFV)*pi*v*H1(1))/(9._dp*(u*u))+
     & (pi*v*(-63+209*v+(25-27*v)*(u*u*u*u*u*u)-9*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(154+48*v-27*(v*v))-238*(v*v)+
     & u*(397-1078*v+934*(v*v)-244*(v*v*v))+
     & u*u*u*u*(-730+526*v+21*(v*v)-9*(v*v*v))+92*(v*v*v)-
     & 2*(u*u*u)*(-607+874*v-277*(v*v)+v*v*v)+
     & 2*(u*u)*(-494+1035*v-622*(v*v)+91*(v*v*v))+
     & 10*NFV*(2-6*v+(-25+9*v)*(u*u*u*u*u)+3*(u*u*u*u*u*u)+
     & 6*(v*v)+u*u*u*u*(77-72*v+9*(v*v))+
     & u*u*u*(-109+165*v-63*(v*v)+3*(v*v*v))+
     & 2*u*(-11+30*v-24*(v*v)+6*(v*v*v))-
     & 2*(u*u)*(-37+78*v-48*(v*v)+9*(v*v*v)))+
     & nf*(omu*omu)*(w*w*w)))/(18._dp*(omu*omu)*(u*u)*(w*w*w)))+
     & G2(1,1)*((-4*pi*v*H1(1))/(u*u)-
     & (pi*v*(-39+97*v+(43-27*v)*(u*u*u*u*u*u)-9*(u*u*u*u*u*u*u)+
     & u*u*u*u*u*(143+102*v-27*(v*v))-68*(v*v)+
     & u*(289-670*v+504*(v*v)-132*(v*v*v))+
     & u*u*u*u*(-769+547*v+75*(v*v)-9*(v*v*v))+10*(v*v*v)+
     & u*u*u*(1193-1768*v+629*(v*v)+16*(v*v*v))+
     & u*u*(-851+1719*v-1113*(v*v)+225*(v*v*v))+
     & 10*NFV*(-2+6*v+(-25+9*v)*(u*u*u*u*u)+3*(u*u*u*u*u*u)-
     & 6*(v*v)+u*u*u*u*(62-72*v+9*(v*v))+
     & u*u*(25-78*v+75*(v*v)-18*(v*v*v))+
     & u*u*u*(-64+132*v-63*(v*v)+3*(v*v*v))+
     & u*(1+3*v-15*(v*v)+9*(v*v*v)))+4*nf*(omu*omu)*(w*w*w)))
     & /(18._dp*(omu*omu)*(u*u)*(w*w*w)))+
     & ((pi*(-1+2*u)*v)/(2._dp*u*(omu*omu))-(pi*v*G1(1))/(2._dp*(u*u)))*
     & (G1(0)*G1(0))+(pi*(1-2*u)*v*(G1(2)*G1(2)))/(18._dp*u*(omu*omu))+
     & (pi*v*(-17+16*v+18*(-9+v)*(u*u)+72*(u*u*u)+v*v-
     & 2*u*(-53+16*v+v*v))*(H1(0)*H1(0)))/(18._dp*u*(omu*omu)*(omv*omv))
     & +G1(2)*(((1+20*NFV)*pi*v*H2(0,0))/(3._dp*(u*u))+
     & (10*(-1+6*NFV)*pi*v*H2(0,1))/(9._dp*(u*u))-
     & (2*(-1+30*NFV)*pi*v*H2(1,0))/(9._dp*(u*u))-
     & (pi*v*H1(1)*(4-13*u+29*(u*u)-20*(u*u*u)+
     & 10*NFV*(-12+30*u-25*(u*u)+3*(u*u*u))))/
     & (18._dp*u*(omu*omu*omu))+
     & (pi*H1(0)*(31/omv+8/(u-u*v)+
     & (4*(-1-8*v-(30*NFV*(-2+v)*v)/omv-1/w))/u+4/w-
     & 31/(omv*omv)+2*u*v*
     & ((-55*NFV)/(omu*omu*omu)+(-16+45*NFV)/(omv*omv*omv))-
     & (4*(-3+nf)*v)/(u*u)+(90*NFV*v*(u*u))/(omu*omu*omu)+
     & (v*(omv*omv*(1+4*(-419+54*nf)*u+2*(-433+54*nf)*(u*u)+
     & 12*(u*u*u))-
     & 60*omu*NFV*(-2+3*u+2*v-3*(u*u)+u*u*u-v*v)))/
     & (omu*omu*omu*omu*(omv*omv))))/18._dp+
     & (pi*((-2*(20*NFV*(-1+w)+w+59*v*w))/(omu*w)+
     & (40*NFV+123*v-4*nf*v)/(omu*omu)-
     & (20*NFV*v*(2+9*u+u*u))/(omu*omu*omu)-
     & (2*(3*omv*(11-5*NFV*(-4+3*u))*(u*u)*(w*w)+
     & 2*(u*u*u)*(-13+45*NFV+2*v-5*(v*v))*(w*w)+
     & omv*omv*(-(w*(24*v*w*zeta2+(-9+(24+11*v)*w)*(u*u)+
     & u*(9+w*(-9-71*v+v*v))))+
     & 10*NFV*(2*(u*u*u)+u*u*(w-3*(w*w))+24*v*zeta2*(w*w)+
     & u*(2-6*w+(4-8*v)*(w*w))))))/
     & (omv*omv*(u*u)*(w*w))))/36._dp-(pi*v*(H1(0)*H1(0)))/(u*u))+
     & (pi*(1-2*u)*v*(H1(1)*H1(1)))/(18._dp*u*(omu*omu))+
     & G1(1)*(G1(2)*(-(pi*v)/(6._dp*(u*u))-(pi*v*H1(0))/(9._dp*(u*u))-
     & (pi*v*H1(1))/(9._dp*(u*u)))-((1+20*NFV)*pi*v*H2(0,0))/(3._dp*(u*u))-
     & (10*(-1+6*NFV)*pi*v*H2(0,1))/(9._dp*(u*u))+
     & (2*(1-10*NFV)*pi*v*H2(1,0))/(3._dp*(u*u))+
     & ((1-60*NFV)*pi*v*H2(1,1))/(9._dp*(u*u))+
     & (pi*(6*(-24-16/omu+9/omv-25/(u-u*v)+9/w+32/(omv*omv)+
     & (v*(-121+(63+4*nf)*u-11*(u*u)))/(omu*omu))+
     & (54*omw+v*w*(1119-76*nf+6*v+222*zeta2))/(w*(u*u))-
     & (6*(-33+4*(7+nf)*v+v*v))/u-
     & (12*u*(13-2*v+5*(v*v)))/(omv*omv)+
     & 30*NFV*(-6-18/omu+30/omv-21/(u-u*v)+
     & (4+6*v+2/w)/u+2/w-(12*(-1+2*u)*v)/(omu*omu)-
     & 18/(omv*omv)+(2*(-5+7/w+v*(-6+24*zeta2)-2/(w*w)))/
     & (u*u)+u*(-9/omv+18/(omv*omv)+4/(w*w)))))/108._dp+
     & H1(0)*((10*pi*v*H1(1))/(9._dp*(u*u))-
     & (pi*v*(-40+160*v+(-365+243*v+10*NFV*(-5+12*v))*(u*u*u*u)+
     & (81+40*NFV)*(u*u*u*u*u)-218*(v*v)+
     & u*u*u*(649-928*v+243*(v*v)+
     & 10*NFV*(-5-15*v+12*(v*v)))+(98+40*NFV)*(v*v*v)+
     & u*u*(-567+1287*v-761*(v*v)+81*(v*v*v)+
     & 10*NFV*(9-6*v-15*(v*v)+4*(v*v*v)))+
     & u*(242-762*v+736*(v*v)-198*(v*v*v)-
     & 10*NFV*(3-9*v-3*(v*v)+5*(v*v*v)))))/
     & (18._dp*(omu*omu)*(u*u)*(w*w*w)))-
     & (pi*v*H1(1)*(-78+254*v-9*(-5+3*v)*(u*u*u*u*u*u)-
     & 9*(u*u*u*u*u*u*u)+u*u*u*u*u*(137+108*v-27*(v*v))-
     & 283*(v*v)+u*(486-1300*v+1111*(v*v)-288*(v*v*v))+
     & u*u*u*u*(-843+535*v+81*(v*v)-9*(v*v*v))+107*(v*v*v)+
     & u*u*u*(1470-2078*v+623*(v*v)+18*(v*v*v))+
     & u*u*(-1208+2508*v-1505*(v*v)+225*(v*v*v))+
     & 10*NFV*(-2+6*v+3*(-5+4*v)*(u*u*u*u)+6*(u*u*u*u*u)-
     & 6*(v*v)+3*(u*u*u)*(5-9*v+4*(v*v))+
     & u*(7-15*v+15*(v*v)-5*(v*v*v))+4*(v*v*v)+
     & u*u*(-11+24*v-21*(v*v)+4*(v*v*v)))))/
     & (18._dp*(omu*omu)*(u*u)*(w*w*w))+(pi*v*(G1(2)*G1(2)))/(18._dp*(u*u))+
     & (pi*v*(H1(0)*H1(0)))/(18._dp*(u*u))+(pi*v*(H1(1)*H1(1)))/(18._dp*(u*u)))
      return
      end
      
      
!**********************************************************************
!region2a:g+g-->V+g*
!*********************************************************************/
      
      function GGAlpha1_2a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGAlpha1_2a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGAlpha1_2a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=G1(0)*((-4*(1+omw)*u)/(omw*w)-(4*(omv+u*(u+v))*G1(3))/(w*w)-
     & (4*(omv+u*(u+v))*H1(0))/(w*w))+
     & H1(0)*((-4*v*(omv+u*(u+v)))/(omv*omw*w)-
     & (4*(omv+u*(u+v))*H1(1))/(w*w))-
     & (4*(omv+u*(u+v))*zeta2)/(w*w)+(4*(omv+u*(u+v))*G2(0,3))/(w*w)+
     & (4*(omv+u*(u+v))*G1(3)*H1(0))/(w*w)+
     & (4*(omv+u*(u+v))*H2(0,1))/(w*w)+
     & (2*(omv+u*(u+v))*(H1(0)*H1(0)))/(w*w)
      return
      end
      
      
      function GGAlpha1_2a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-4*(1+omw)*pi*u)/(omw*w)-(4*pi*(omv+u*(u+v))*G1(3))/(w*w)-
     & (4*pi*(omv+u*(u+v))*H1(0))/(w*w)
      return
      end
      
      
      function GGAlpha1_2a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGAlpha1_2a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGAlpha2_2a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGAlpha2_2a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGAlpha2_2a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=G1(0)*((-4*u*(u+2*omv*v-u*v+u*u))/(omw*omw*(w*w))-
     & (4*G1(3)*(omv*omv+u*u))/(w*w*w)-(4*H1(0)*(omv*omv+u*u))/(w*w*w))
     & +H1(0)*((-4*v*(v+u*v+2*(u*u)-v*v))/(omw*omw*(w*w))-
     & (4*H1(1)*(omv*omv+u*u))/(w*w*w))+
     & (4*G2(0,3)*(omv*omv+u*u))/(w*w*w)+
     & (4*G1(3)*H1(0)*(omv*omv+u*u))/(w*w*w)+
     & (4*H2(0,1)*(omv*omv+u*u))/(w*w*w)-
     & (2*((6*v*zeta2+u*(6+6*zeta2))*(omv*omv)+
     & (-12+u*(6+6*zeta2)+v*(12+6*zeta2))*(u*u)))/(3._dp*omw*(w*w*w))
     & +(2*(omv*omv+u*u)*(H1(0)*H1(0)))/(w*w*w)
      return
      end
      
      
      function GGAlpha2_2a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-4*pi*u*(u+2*omv*v-u*v+u*u))/(omw*omw*(w*w))-
     & (4*pi*G1(3)*(omv*omv+u*u))/(w*w*w)-
     & (4*pi*H1(0)*(omv*omv+u*u))/(w*w*w)
      return
      end
      
      
      function GGAlpha2_2a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGAlpha2_2a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGAlpha3_2a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGAlpha3_2a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGAlpha3_2a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=G1(0)*((-4*(1+omw)*u)/(w*w)-(4*u*G1(3)*(1+2*u*v+u*u+v*v))/(w*w*w)-
     & (4*u*H1(0)*(1+2*u*v+u*u+v*v))/(w*w*w))+
     & H1(0)*((4*u*v*(-2+v+u*v+v*v))/(omv*omv*(w*w))-
     & (4*u*H1(1)*(1+2*u*v+u*u+v*v))/(w*w*w))+
     & (4*u*G2(0,3)*(1+2*u*v+u*u+v*v))/(w*w*w)+
     & (4*u*G1(3)*H1(0)*(1+2*u*v+u*u+v*v))/(w*w*w)+
     & (4*u*H2(0,1)*(1+2*u*v+u*u+v*v))/(w*w*w)-
     & (2*u*(6*omv*zeta2*(1+2*u*v+u*u+v*v)+6*(w*w)))/(3._dp*omv*(w*w*w))+
     & (2*u*(1+2*u*v+u*u+v*v)*(H1(0)*H1(0)))/(w*w*w)
      return
      end
      
      
      function GGAlpha3_2a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-4*(1+omw)*pi*u)/(w*w)-(4*pi*u*G1(3)*(1+2*u*v+u*u+v*v))/(w*w*w)-
     & (4*pi*u*H1(0)*(1+2*u*v+u*u+v*v))/(w*w*w)
      return
      end
      
      
      function GGAlpha3_2a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGAlpha3_2a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGBeta1_2a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGBeta1_2a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGBeta1_2a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-4*omw)/u
      return
      end
      
      
      function GGBeta1_2a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGBeta1_2a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGBeta1_2a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGBeta2_2a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGBeta2_2a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGBeta2_2a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-4*omu)/w
      return
      end
      
      
      function GGBeta2_2a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGBeta2_2a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGBeta2_2a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGBeta3_2a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGBeta3_2a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGBeta3_2a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=-8
      return
      end
      
      
      function GGBeta3_2a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGBeta3_2a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGBeta3_2a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
!**********************************************************************
!region4a:g+g-->V+g*
!*********************************************************************/
      
      function GGAlpha1_4a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGAlpha1_4a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGAlpha1_4a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=-16*(omv+2*u*(-1+u+v))*zeta2+(4*(-1+2*u)*w*H1(1))/omu+
     & G1(2)*(((4-8*u)*w)/omu+H1(0)*(-4+8*u+4*v-8*u*v-8*(u*u)))+
     & G2(1,2)*(-4+8*u+4*v-8*u*v-8*(u*u))+
     & H2(0,1)*(-4+8*u+4*v-8*u*v-8*(u*u))+
     & G2(0,3)*(4-8*omv*u-4*v+8*(u*u))+
     & G2(1,1)*(4-8*omv*u-4*v+8*(u*u))+
     & G1(3)*H1(0)*(4-8*omv*u-4*v+8*(u*u))+
     & G1(0)*((8-4/omw)*u+G1(3)*(-4+8*u+4*v-8*u*v-8*(u*u))+
     & H1(0)*(-4+8*u+4*v-8*u*v-8*(u*u))+
     & H1(1)*(-4+8*u+4*v-8*u*v-8*(u*u))+
     & G1(2)*(4-8*omv*u-4*v+8*(u*u)))+
     & H1(0)*(4*(2-1/omu-1/omw)*v+H1(1)*(4-8*omv*u-4*v+8*(u*u)))+
     & G1(1)*(H1(0)*(4-8*omv*u-4*v+8*(u*u))+
     & H1(1)*(4-8*omv*u-4*v+8*(u*u)))-
     & 2*(omv+2*u*(-1+u+v))*(G1(1)*G1(1))+
     & (2-4*omv*u-2*v+4*(u*u))*(H1(0)*H1(0))
      return
      end
      
      
      function GGAlpha1_4a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=4*(-2+1/omu+1/omw)*pi*v+4*pi*(omv+2*u*(-1+u+v))*G1(0)-
     & 4*pi*(omv+2*u*(-1+u+v))*G1(1)+
     & 4*pi*(omv+2*u*(-1+u+v))*G1(2)-
     & 4*pi*(omv+2*u*(-1+u+v))*G1(3)-
     & 4*pi*(omv+2*u*(-1+u+v))*H1(0)-4*pi*(omv+2*u*(-1+u+v))*H1(1)
      return
      end
      
      
      function GGAlpha1_4a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGAlpha1_4a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGAlpha2_4a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGAlpha2_4a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGAlpha2_4a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=4*(-1+2*u)*w*H1(1)+4*w*G2(0,3)*(1-2*u+2*(u*u))+
     & 4*w*G2(1,1)*(1-2*u+2*(u*u))-4*w*G2(1,2)*(1-2*u+2*(u*u))+
     & 4*w*G1(3)*H1(0)*(1-2*u+2*(u*u))-4*w*H2(0,1)*(1-2*u+2*(u*u))+
     & G1(2)*((4-8*u)*w-4*w*H1(0)*(1-2*u+2*(u*u)))+
     & H1(0)*(4*v*(2-2*u-((1+omw)*v)/(omw*omw))+
     & 4*w*H1(1)*(1-2*u+2*(u*u)))+
     & G1(1)*(4*w*H1(0)*(1-2*u+2*(u*u))+4*w*H1(1)*(1-2*u+2*(u*u)))-
     & (4*w*(12*v*zeta2+u*(3+6*(2-4*v)*zeta2)-24*omv*zeta2*(u*u)+
     & 24*zeta2*(u*u*u)))/(3._dp*omw)+
     & G1(0)*(4*w*G1(2)*(1-2*u+2*(u*u))-4*w*G1(3)*(1-2*u+2*(u*u))-
     & 4*w*H1(0)*(1-2*u+2*(u*u))-4*w*H1(1)*(1-2*u+2*(u*u))+
     & 4*u*(3-2*u-(u+2*v+u*v+v*v)/(omw*omw)))-
     & 2*w*(1-2*u+2*(u*u))*(G1(1)*G1(1))+2*w*(1-2*u+2*(u*u))*(H1(0)*H1(0))
      return
      end
      
      
      function GGAlpha2_4a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=4*pi*v*(-2+2*u+((1+omw)*v)/(omw*omw))+
     & 4*pi*w*G1(0)*(1-2*u+2*(u*u))-4*pi*w*G1(1)*(1-2*u+2*(u*u))+
     & 4*pi*w*G1(2)*(1-2*u+2*(u*u))-4*pi*w*G1(3)*(1-2*u+2*(u*u))-
     & 4*pi*w*H1(0)*(1-2*u+2*(u*u))-4*pi*w*H1(1)*(1-2*u+2*(u*u))
      return
      end
      
      
      function GGAlpha2_4a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGAlpha2_4a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGAlpha3_4a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGAlpha3_4a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGAlpha3_4a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=-4*u*G2(0,3)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))-
     & 4*u*G2(1,1)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))+
     & 4*u*G2(1,2)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))-
     & 4*u*G1(3)*H1(0)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))+
     & 4*u*H2(0,1)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))-
     & (4*u*H1(1)*(-1+4*v+(-5+4*v)*(u*u)+2*(u*u*u)-3*(v*v)+
     & 2*u*(2-4*v+v*v)))/(omu*omu)+
     & (4*u*(3*w+12*omu*zeta2*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))))/
     & (3._dp*omu)+H1(0)*((-4*u*v*(2+2*u*(-2+v)-3*v+2*(u*u)))/
     & (omu*omu)-4*u*H1(1)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v)))+
     & G1(1)*(-4*u*H1(0)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))-
     & 4*u*H1(1)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v)))+
     & G1(0)*(-4*u*(-1+2*u+2*v)-
     & 4*u*G1(2)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))+
     & 4*u*G1(3)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))+
     & 4*u*H1(0)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))+
     & 4*u*H1(1)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v)))+
     & G1(2)*(4*u*H1(0)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))+
     & (4*u*(-1+4*v+(-5+4*v)*(u*u)+2*(u*u*u)-3*(v*v)+
     & 2*u*(2-4*v+v*v)))/(omu*omu))+
     & 2*u*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))*(G1(1)*G1(1))-
     & 2*u*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))*(H1(0)*H1(0))
      return
      end
      
      
      function GGAlpha3_4a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(4*pi*u*v*(2+2*u*(-2+v)-3*v+2*(u*u)))/(omu*omu)-
     & 4*pi*u*G1(0)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))+
     & 4*pi*u*G1(1)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))-
     & 4*pi*u*G1(2)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))+
     & 4*pi*u*G1(3)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))+
     & 4*pi*u*H1(0)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))+
     & 4*pi*u*H1(1)*(1-2*v+u*(-2+4*v)+2*(u*u)+2*(v*v))
      return
      end
      
      
      function GGAlpha3_4a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGAlpha3_4a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGBeta1_4a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGBeta1_4a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGBeta1_4a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=(-4*omw)/u
      return
      end
      
      
      function GGBeta1_4a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGBeta1_4a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGBeta1_4a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGBeta2_4a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGBeta2_4a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGBeta2_4a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=-4*omv
      return
      end
      
      
      function GGBeta2_4a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGBeta2_4a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGBeta2_4a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGBeta3_4a0re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGBeta3_4a0im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      fxn=0
      return
      end
      
      
      function GGBeta3_4a1re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=-8
      return
      end
      
      
      function GGBeta3_4a1im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGBeta3_4a2re(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end
      
      
      function GGBeta3_4a2im(u,v,
     & H1,H2,H3,H4,G1,G2,G3,G4)
     & result(fxn)
      implicit  none
      include 'types.f'
      include 'zeta.f'
      include 'hpls.f'
      real(dp)::fxn
      real(dp)::u,v,w,omu,omv,omw
      w=1-u-v
      omu=1-u
      omv=1-v
      omw=1-w
      fxn=0
      return
      end

      end module nnlo_z1jet_hfun
